--
--  Copyright (C) 2014-2022, AdaCore
--  SPDX-License-Identifier: Apache-2.0
--

with Ada.Containers;                  use Ada.Containers;
with Ada.Containers.Hashed_Maps;
with Ada.Containers.Vectors;
with Ada.Directories;
with Ada.Exceptions;
with Ada.Finalization;
with Ada.Strings.Unbounded;           use Ada.Strings.Unbounded;
with Ada.Strings.Wide_Wide_Unbounded; use Ada.Strings.Wide_Wide_Unbounded;
pragma Warnings (Off, "internal");
with Ada.Strings.Wide_Wide_Unbounded.Aux;
use Ada.Strings.Wide_Wide_Unbounded.Aux;
pragma Warnings (On, "internal");
with Ada.Text_IO;                     use Ada.Text_IO;
with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;
with System;
with GNAT.Task_Lock;
with GNAT.Traceback.Symbolic;
with GNATCOLL.Traces;
with Langkit_Support.Adalog.Debug;
with Langkit_Support.Generic_API.Analysis;
with Langkit_Support.Generic_API.Introspection;
with Langkit_Support.Hashes; use Langkit_Support.Hashes;
with Langkit_Support.Images; use Langkit_Support.Images;
with Langkit_Support.Names;  use Langkit_Support.Names;
with Langkit_Support.Relative_Get;
with Libadalang.Private_Converters;
use Libadalang.Private_Converters;
pragma Warnings (Off, "referenced");
          with Ada.Containers.Hashed_Sets;
          with Libadalang.Env_Hooks;
            use Libadalang.Env_Hooks;
          with Libadalang.Implementation.Extensions;
            use Libadalang.Implementation.Extensions;
          with Libadalang.Internal_Default_Provider;
          with Libadalang.Sources;
pragma Warnings (On, "referenced");
package body Libadalang.Implementation is
   use Precomputed_Symbols;
   pragma Warnings (Off, "has no effect");
   use Solver;
   pragma Warnings (On, "has no effect");
   package Context_Vectors is new Ada.Containers.Vectors
     (Index_Type   => Positive,
      Element_Type => Internal_Context);
   type Contexts_Destructor is limited
      new Ada.Finalization.Limited_Controlled with null record;
   overriding procedure Finalize (CD : in out Contexts_Destructor);
   --  Helper to destroy all contexts when terminating the process
   package Context_Pool is
      procedure Acquire (Context : out Internal_Context)
         with Post => Context /= null;
      --  If a context is free for reuse, increment its serial number and
      --  return it. Otherwise, allocate a new one. In any case, this does not
      --  initialize it, except for the Serial_Number field.
      procedure Release (Context : in out Internal_Context)
         with Pre  => Context /= null,
              Post => Context = null;
      --  Tag Context as free for reuse and set it to null
      procedure Free;
      --  Free all contexts in this pool. Intended to be called only when the
      --  process is terminating, to avoid reported memory leaks.
   private
      Available : Context_Vectors.Vector;
      --  List of allocated contexts that can be re-used right now
      CD : Contexts_Destructor with Unreferenced;
      --  Singleton whose only purpose is to free all contexts in Available
      --  when finalized.
   end Context_Pool;
   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure);
   --  Common underlying implementation for Register_Destroyable_Gen
   pragma Warnings (Off, "referenced");
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access;
   pragma Warnings (On, "referenced");
   procedure Destroy (Env : in out Lexical_Env_Access);
   function Snaps_At_Start (Self : Bare_Ada_Node) return Boolean;
   function Snaps_At_End (Self : Bare_Ada_Node) return Boolean;
   --  Those maps are used to give unique ids to lexical envs while pretty
   --  printing them.
   package Address_To_Id_Maps is new Ada.Containers.Hashed_Maps
     (Lexical_Env, Integer, Hash, "=");
   type Dump_Lexical_Env_State is record
      Env_Ids : Address_To_Id_Maps.Map;
      --  Mapping: Lexical_Env -> Integer, used to remember which unique Ids we
      --  assigned to the lexical environments we found.
      Next_Id : Positive := 1;
      --  Id to assign to the next unknown lexical environment
      Root_Env : Lexical_Env;
      --  Lexical environment we consider a root (this is the Root_Scope from
      --  the current analysis context), or null if unknown.
   end record;
   --  Holder for the state of lexical environment dumpers
   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String;
   --  If E is known, return its unique Id from State. Otherwise, assign it a
   --  new unique Id and return it.
   procedure Print
     (Node        : Langkit_Support.Generic_API.Analysis.Lk_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "");
   --  Helper for the public overload, but working on the generic API node type
   ------------------------
   -- Precomputed_Symbol --
   ------------------------
   pragma Warnings (Off, "referenced");
   function Precomputed_Symbol
     (Index : Precomputed_Symbol_Index) return Text_Type is
   pragma Warnings (On, "referenced");
   begin
         declare
            Raw_Text : constant Text_Type := (case Index is
               when Precomputed_Sym => """&""",
               when Precomputed_Sym_1 => """&""""+""",
               when Precomputed_Sym_10 => """=""",
               when Precomputed_Sym_11 => """>""",
               when Precomputed_Sym_12 => """>=""",
               when Precomputed_Sym_2 => """*""",
               when Precomputed_Sym_20 => "<<>>",
               when Precomputed_Sym_3 => """**""",
               when Precomputed_Sym_4 => """+""",
               when Precomputed_Sym_5 => """-""",
               when Precomputed_Sym_6 => """/""",
               when Precomputed_Sym_7 => """/=""",
               when Precomputed_Sym_8 => """<""",
               when Precomputed_Sym_9 => """<=""",
               when Precomputed_Sym_Abort => "abort",
               when Precomputed_Sym_Abort_Signal => "Abort_Signal",
               when Precomputed_Sym_Abort_Signal_286 => "abort_signal_",
               when Precomputed_Sym_Abs => """abs""",
               when Precomputed_Sym_Abs_287 => "abs",
               when Precomputed_Sym_Abstract => "abstract",
               when Precomputed_Sym_Abstract_State => "Abstract_State",
               when Precomputed_Sym_Accept => "accept",
               when Precomputed_Sym_Access => "Access",
               when Precomputed_Sym_Access_290 => "access",
               when Precomputed_Sym_Ada => "Ada",
               when Precomputed_Sym_Ada_291 => "ada",
               when Precomputed_Sym_Address => "Address",
               when Precomputed_Sym_Address_Size => "Address_Size",
               when Precomputed_Sym_Adjacent => "Adjacent",
               when Precomputed_Sym_Adjacent_292 => "adjacent",
               when Precomputed_Sym_Aft => "Aft",
               when Precomputed_Sym_Aliased => "aliased",
               when Precomputed_Sym_Alignment => "Alignment",
               when Precomputed_Sym_All => "all",
               when Precomputed_Sym_And => """and""",
               when Precomputed_Sym_And_295 => "and",
               when Precomputed_Sym_Annotate => "Annotate",
               when Precomputed_Sym_Array => "array",
               when Precomputed_Sym_Asm_Input => "Asm_Input",
               when Precomputed_Sym_Asm_Input_297 => "asm_input",
               when Precomputed_Sym_Asm_Input_Operand => "Asm_Input_Operand",
               when Precomputed_Sym_Asm_Output => "Asm_Output",
               when Precomputed_Sym_Asm_Output_298 => "asm_output",
               when Precomputed_Sym_Asm_Output_Operand => "Asm_Output_Operand",
               when Precomputed_Sym_Assert => "Assert",
               when Precomputed_Sym_Assert_And_Cut => "Assert_And_Cut",
               when Precomputed_Sym_Assume => "Assume",
               when Precomputed_Sym_Asynchronous => "Asynchronous",
               when Precomputed_Sym_At => "at",
               when Precomputed_Sym_Atomic => "Atomic",
               when Precomputed_Sym_Atomic_Components => "Atomic_Components",
               when Precomputed_Sym_Attach_Handler => "Attach_Handler",
               when Precomputed_Sym_Aux_Dec => "Aux_DEC",
               when Precomputed_Sym_Base => "Base",
               when Precomputed_Sym_Begin => "begin",
               when Precomputed_Sym_Bit_Order => "Bit_Order",
               when Precomputed_Sym_Body => "body",
               when Precomputed_Sym_Boolean => "Boolean",
               when Precomputed_Sym_Buffer => "Buffer",
               when Precomputed_Sym_Callable => "Callable",
               when Precomputed_Sym_Caller => "Caller",
               when Precomputed_Sym_Case => "case",
               when Precomputed_Sym_Ceiling => "Ceiling",
               when Precomputed_Sym_Ceiling_303 => "ceiling",
               when Precomputed_Sym_Character => "Character",
               when Precomputed_Sym_Class => "Class",
               when Precomputed_Sym_Compile_Time_Error => "Compile_Time_Error",
               when Precomputed_Sym_Compile_Time_Warning => "Compile_Time_Warning",
               when Precomputed_Sym_Component_Size => "Component_Size",
               when Precomputed_Sym_Compose => "Compose",
               when Precomputed_Sym_Compose_304 => "compose",
               when Precomputed_Sym_Constant => "constant",
               when Precomputed_Sym_Constant_Indexing => "Constant_Indexing",
               when Precomputed_Sym_Constrained => "Constrained",
               when Precomputed_Sym_Contract_Cases => "Contract_Cases",
               when Precomputed_Sym_Convention => "Convention",
               when Precomputed_Sym_Copy_Sign => "Copy_Sign",
               when Precomputed_Sym_Copy_Sign_306 => "copy_sign",
               when Precomputed_Sym_Count => "Count",
               when Precomputed_Sym_Cursor => "Cursor",
               when Precomputed_Sym_Debug => "Debug",
               when Precomputed_Sym_Declare => "declare",
               when Precomputed_Sym_Default_Bit_Order => "Default_Bit_Order",
               when Precomputed_Sym_Default_Initial_Condition => "Default_Initial_Condition",
               when Precomputed_Sym_Default_Scalar_Storage_Order => "Default_Scalar_Storage_Order",
               when Precomputed_Sym_Definite => "Definite",
               when Precomputed_Sym_Delay => "delay",
               when Precomputed_Sym_Delta => "Delta",
               when Precomputed_Sym_Delta_309 => "delta",
               when Precomputed_Sym_Denorm => "Denorm",
               when Precomputed_Sym_Depends => "Depends",
               when Precomputed_Sym_Descriptor_Size => "Descriptor_Size",
               when Precomputed_Sym_Digits => "Digits",
               when Precomputed_Sym_Digits_310 => "digits",
               when Precomputed_Sym_Discard_Names => "Discard_Names",
               when Precomputed_Sym_Do => "do",
               when Precomputed_Sym_Dummy => "__dummy",
               when Precomputed_Sym_Duration => "Duration",
               when Precomputed_Sym_Dynamic_Predicate => "Dynamic_Predicate",
               when Precomputed_Sym_Elaborate_Body => "Elaborate_Body",
               when Precomputed_Sym_Element => "Element",
               when Precomputed_Sym_Else => "else",
               when Precomputed_Sym_Elsif => "elsif",
               when Precomputed_Sym_Enabled => "Enabled",
               when Precomputed_Sym_End => "end",
               when Precomputed_Sym_Ensures => "Ensures",
               when Precomputed_Sym_Entry => "entry",
               when Precomputed_Sym_Enum_Rep => "Enum_Rep",
               when Precomputed_Sym_Enum_Rep_316 => "enum_rep",
               when Precomputed_Sym_Enum_Val => "Enum_Val",
               when Precomputed_Sym_Enum_Val_317 => "enum_val",
               when Precomputed_Sym_Epsilon => "Epsilon",
               when Precomputed_Sym_Exception => "exception",
               when Precomputed_Sym_Exception_Id => "Exception_Id",
               when Precomputed_Sym_Exception_Occurrence => "Exception_Occurrence",
               when Precomputed_Sym_Exceptions => "Exceptions",
               when Precomputed_Sym_Exit => "exit",
               when Precomputed_Sym_Exponent => "Exponent",
               when Precomputed_Sym_Exponent_320 => "exponent",
               when Precomputed_Sym_Export => "Export",
               when Precomputed_Sym_External_Tag => "External_Tag",
               when Precomputed_Sym_Finalization_Size => "Finalization_Size",
               when Precomputed_Sym_First => "First",
               when Precomputed_Sym_First_Bit => "First_Bit",
               when Precomputed_Sym_First_Valid => "First_Valid",
               when Precomputed_Sym_Fixed_Value => "Fixed_Value",
               when Precomputed_Sym_Fixed_Value_321 => "fixed_value",
               when Precomputed_Sym_Floor => "Floor",
               when Precomputed_Sym_Floor_322 => "floor",
               when Precomputed_Sym_For => "for",
               when Precomputed_Sym_Fore => "Fore",
               when Precomputed_Sym_Fraction => "Fraction",
               when Precomputed_Sym_Fraction_324 => "fraction",
               when Precomputed_Sym_Function => "function",
               when Precomputed_Sym_Generic => "generic",
               when Precomputed_Sym_Ghost => "Ghost",
               when Precomputed_Sym_Global => "Global",
               when Precomputed_Sym_Gnatprove => "GNATprove",
               when Precomputed_Sym_Goto => "goto",
               when Precomputed_Sym_Has_Access_Values => "Has_Access_Values",
               when Precomputed_Sym_Has_Discriminants => "Has_Discriminants",
               when Precomputed_Sym_Has_Same_Storage => "Has_Same_Storage",
               when Precomputed_Sym_Has_Tagged_Values => "Has_Tagged_Values",
               when Precomputed_Sym_Identity => "Identity",
               when Precomputed_Sym_If => "if",
               when Precomputed_Sym_Image => "Image",
               when Precomputed_Sym_Image_329 => "image",
               when Precomputed_Sym_Img => "Img",
               when Precomputed_Sym_Implicit_Dereference => "Implicit_Dereference",
               when Precomputed_Sym_Import => "Import",
               when Precomputed_Sym_Import_Function => "Import_Function",
               when Precomputed_Sym_Import_Procedure => "Import_Procedure",
               when Precomputed_Sym_In => "in",
               when Precomputed_Sym_Independent => "Independent",
               when Precomputed_Sym_Independent_Components => "Independent_Components",
               when Precomputed_Sym_Index => "Index",
               when Precomputed_Sym_Initial_Condition => "Initial_Condition",
               when Precomputed_Sym_Initialized => "Initialized",
               when Precomputed_Sym_Inline => "Inline",
               when Precomputed_Sym_Input => "Input",
               when Precomputed_Sym_Input_331 => "input",
               when Precomputed_Sym_Integer => "Integer",
               when Precomputed_Sym_Integer_Literal => "Integer_Literal",
               when Precomputed_Sym_Integer_Value => "Integer_Value",
               when Precomputed_Sym_Integer_Value_332 => "integer_value",
               when Precomputed_Sym_Interface => "Interface",
               when Precomputed_Sym_Interface_333 => "interface",
               when Precomputed_Sym_Interrupt_Handler => "Interrupt_Handler",
               when Precomputed_Sym_Invalid_Value => "Invalid_Value",
               when Precomputed_Sym_Invariant => "Invariant",
               when Precomputed_Sym_Invariant_Class => "Invariant'Class",
               when Precomputed_Sym_Is => "is",
               when Precomputed_Sym_Iterable => "Iterable",
               when Precomputed_Sym_Iterator_Element => "Iterator_Element",
               when Precomputed_Sym_Iterator_Interfaces => "Iterator_Interfaces",
               when Precomputed_Sym_Large => "Large",
               when Precomputed_Sym_Last => "Last",
               when Precomputed_Sym_Last_Bit => "Last_Bit",
               when Precomputed_Sym_Last_Valid => "Last_Valid",
               when Precomputed_Sym_Leading_Part => "Leading_Part",
               when Precomputed_Sym_Leading_Part_335 => "leading_part",
               when Precomputed_Sym_Left => "left",
               when Precomputed_Sym_Length => "Length",
               when Precomputed_Sym_Library_Level => "Library_Level",
               when Precomputed_Sym_Limited => "limited",
               when Precomputed_Sym_Loop => "loop",
               when Precomputed_Sym_Loop_Entry => "Loop_Entry",
               when Precomputed_Sym_Loop_Invariant => "Loop_Invariant",
               when Precomputed_Sym_Machine => "Machine",
               when Precomputed_Sym_Machine_339 => "machine",
               when Precomputed_Sym_Machine_Code => "Machine_Code",
               when Precomputed_Sym_Machine_Emax => "Machine_Emax",
               when Precomputed_Sym_Machine_Emin => "Machine_Emin",
               when Precomputed_Sym_Machine_Mantissa => "Machine_Mantissa",
               when Precomputed_Sym_Machine_Overflows => "Machine_Overflows",
               when Precomputed_Sym_Machine_Radix => "Machine_Radix",
               when Precomputed_Sym_Machine_Rounding => "Machine_Rounding",
               when Precomputed_Sym_Machine_Rounding_340 => "machine_rounding",
               when Precomputed_Sym_Machine_Rounds => "Machine_Rounds",
               when Precomputed_Sym_Mantissa => "Mantissa",
               when Precomputed_Sym_Max => "Max",
               when Precomputed_Sym_Max_341 => "max",
               when Precomputed_Sym_Max_Integer_Size => "Max_Integer_Size",
               when Precomputed_Sym_Max_Size_In_Storage_Elements => "Max_Size_In_Storage_Elements",
               when Precomputed_Sym_Maximum_Alignment => "Maximum_Alignment",
               when Precomputed_Sym_Min => "Min",
               when Precomputed_Sym_Min_342 => "min",
               when Precomputed_Sym_Mod => """mod""",
               when Precomputed_Sym_Mod_153 => "Mod",
               when Precomputed_Sym_Mod_343 => "mod",
               when Precomputed_Sym_Model => "Model",
               when Precomputed_Sym_Model_344 => "model",
               when Precomputed_Sym_Model_Emin => "Model_Emin",
               when Precomputed_Sym_Model_Epsilon => "Model_Epsilon",
               when Precomputed_Sym_Model_Mantissa => "Model_Mantissa",
               when Precomputed_Sym_Model_Small => "Model_Small",
               when Precomputed_Sym_Modulus => "Modulus",
               when Precomputed_Sym_New => "new",
               when Precomputed_Sym_Nextpart => "__nextpart",
               when Precomputed_Sym_No_Elaboration_Code => "No_Elaboration_Code",
               when Precomputed_Sym_No_Return => "No_Return",
               when Precomputed_Sym_Not => """not""",
               when Precomputed_Sym_Not_346 => "not",
               when Precomputed_Sym_Null => "null",
               when Precomputed_Sym_Object_Size => "Object_Size",
               when Precomputed_Sym_Obsolescent => "Obsolescent",
               when Precomputed_Sym_Of => "of",
               when Precomputed_Sym_Old => "Old",
               when Precomputed_Sym_On => "On",
               when Precomputed_Sym_Or => """or""",
               when Precomputed_Sym_Or_349 => "or",
               when Precomputed_Sym_Others => "others",
               when Precomputed_Sym_Out => "out",
               when Precomputed_Sym_Output => "Output",
               when Precomputed_Sym_Output_352 => "output",
               when Precomputed_Sym_Overlaps_Storage => "Overlaps_Storage",
               when Precomputed_Sym_Overriding => "overriding",
               when Precomputed_Sym_Pack => "Pack",
               when Precomputed_Sym_Package => "package",
               when Precomputed_Sym_Pos => "Pos",
               when Precomputed_Sym_Pos_355 => "pos",
               when Precomputed_Sym_Position => "Position",
               when Precomputed_Sym_Post => "Post",
               when Precomputed_Sym_Post_Class => "Post'Class",
               when Precomputed_Sym_Postcondition => "Postcondition",
               when Precomputed_Sym_Postcondition_Class => "Postcondition'Class",
               when Precomputed_Sym_Pragma => "pragma",
               when Precomputed_Sym_Pre => "Pre",
               when Precomputed_Sym_Pre_Class => "Pre'Class",
               when Precomputed_Sym_Precondition => "Precondition",
               when Precomputed_Sym_Precondition_Class => "Precondition'Class",
               when Precomputed_Sym_Pred => "Pred",
               when Precomputed_Sym_Pred_357 => "pred",
               when Precomputed_Sym_Predicate => "Predicate",
               when Precomputed_Sym_Predicate_Failure => "Predicate_Failure",
               when Precomputed_Sym_Preelaborate => "Preelaborate",
               when Precomputed_Sym_Private => "private",
               when Precomputed_Sym_Privatepart => "__privatepart",
               when Precomputed_Sym_Procedure => "procedure",
               when Precomputed_Sym_Protected => "protected",
               when Precomputed_Sym_Pure => "Pure",
               when Precomputed_Sym_Put_Image => "Put_Image",
               when Precomputed_Sym_Put_Image_361 => "put_image",
               when Precomputed_Sym_Raise => "raise",
               when Precomputed_Sym_Range => "Range",
               when Precomputed_Sym_Range_363 => "range",
               when Precomputed_Sym_Range_Length => "Range_Length",
               when Precomputed_Sym_Read => "Read",
               when Precomputed_Sym_Read_364 => "read",
               when Precomputed_Sym_Real_Literal => "Real_Literal",
               when Precomputed_Sym_Record => "record",
               when Precomputed_Sym_Reduce => "Reduce",
               when Precomputed_Sym_Refined_Depends => "Refined_Depends",
               when Precomputed_Sym_Refined_Global => "Refined_Global",
               when Precomputed_Sym_Refined_Post => "Refined_Post",
               when Precomputed_Sym_Refined_State => "Refined_State",
               when Precomputed_Sym_Rem => """rem""",
               when Precomputed_Sym_Rem_366 => "rem",
               when Precomputed_Sym_Remainder => "Remainder",
               when Precomputed_Sym_Remainder_367 => "remainder",
               when Precomputed_Sym_Remote_Call_Interface => "Remote_Call_Interface",
               when Precomputed_Sym_Remote_Types => "Remote_Types",
               when Precomputed_Sym_Renames => "renames",
               when Precomputed_Sym_Requeue => "requeue",
               when Precomputed_Sym_Requires => "Requires",
               when Precomputed_Sym_Restrictions => "Restrictions",
               when Precomputed_Sym_Result => "Result",
               when Precomputed_Sym_Return => "return",
               when Precomputed_Sym_Reverse => "reverse",
               when Precomputed_Sym_Right => "right",
               when Precomputed_Sym_Root_Buffer_Type => "Root_Buffer_Type",
               when Precomputed_Sym_Root_Integer => "root_integer",
               when Precomputed_Sym_Root_Real => "root_real",
               when Precomputed_Sym_Root_Storage_Pool => "Root_Storage_Pool",
               when Precomputed_Sym_Root_Stream_Type => "Root_Stream_Type",
               when Precomputed_Sym_Root_Types => "root_types_",
               when Precomputed_Sym_Rounding => "Rounding",
               when Precomputed_Sym_Rounding_376 => "rounding",
               when Precomputed_Sym_S => "S",
               when Precomputed_Sym_Safe_First => "Safe_First",
               when Precomputed_Sym_Safe_Large => "Safe_Large",
               when Precomputed_Sym_Safe_Last => "Safe_Last",
               when Precomputed_Sym_Safe_Small => "Safe_Small",
               when Precomputed_Sym_Scalar_Storage_Order => "Scalar_Storage_Order",
               when Precomputed_Sym_Scaling => "Scaling",
               when Precomputed_Sym_Scaling_377 => "scaling",
               when Precomputed_Sym_Select => "select",
               when Precomputed_Sym_Separate => "separate",
               when Precomputed_Sym_Shared_Passive => "Shared_Passive",
               when Precomputed_Sym_Signed_Zeros => "Signed_Zeros",
               when Precomputed_Sym_Size => "Size",
               when Precomputed_Sym_Skip_Flow_And_Proof => "Skip_Flow_And_Proof",
               when Precomputed_Sym_Skip_Proof => "Skip_Proof",
               when Precomputed_Sym_Small => "Small",
               when Precomputed_Sym_Small_Denominator => "Small_Denominator",
               when Precomputed_Sym_Small_Numerator => "Small_Numerator",
               when Precomputed_Sym_Some => "some",
               when Precomputed_Sym_Spark_Mode => "SPARK_Mode",
               when Precomputed_Sym_Stable_Properties => "Stable_Properties",
               when Precomputed_Sym_Standard => "Standard",
               when Precomputed_Sym_Standard_381 => "standard",
               when Precomputed_Sym_Static_Predicate => "Static_Predicate",
               when Precomputed_Sym_Storage_Elements => "Storage_Elements",
               when Precomputed_Sym_Storage_Pool => "Storage_Pool",
               when Precomputed_Sym_Storage_Pools => "Storage_Pools",
               when Precomputed_Sym_Storage_Size => "Storage_Size",
               when Precomputed_Sym_Storage_Unit => "Storage_Unit",
               when Precomputed_Sym_Stream_Size => "Stream_Size",
               when Precomputed_Sym_Streams => "Streams",
               when Precomputed_Sym_String => "String",
               when Precomputed_Sym_String_Literal => "String_Literal",
               when Precomputed_Sym_Strings => "Strings",
               when Precomputed_Sym_Style_Checks => "Style_Checks",
               when Precomputed_Sym_Subtype => "subtype",
               when Precomputed_Sym_Succ => "Succ",
               when Precomputed_Sym_Succ_383 => "succ",
               when Precomputed_Sym_Synchronized => "synchronized",
               when Precomputed_Sym_System => "System",
               when Precomputed_Sym_System_Allocator_Alignment => "System_Allocator_Alignment",
               when Precomputed_Sym_Tag => "Tag",
               when Precomputed_Sym_Tagged => "tagged",
               when Precomputed_Sym_Tags => "Tags",
               when Precomputed_Sym_Target_Name => "Target_Name",
               when Precomputed_Sym_Task => "task",
               when Precomputed_Sym_Task_Id => "Task_Id",
               when Precomputed_Sym_Task_Identification => "Task_Identification",
               when Precomputed_Sym_Terminate => "terminate",
               when Precomputed_Sym_Terminated => "Terminated",
               when Precomputed_Sym_Test_Case => "Test_Case",
               when Precomputed_Sym_Text_Buffers => "Text_Buffers",
               when Precomputed_Sym_Text_Io => "text_io",
               when Precomputed_Sym_Then => "then",
               when Precomputed_Sym_To_Address => "To_Address",
               when Precomputed_Sym_Truncation => "Truncation",
               when Precomputed_Sym_Truncation_390 => "truncation",
               when Precomputed_Sym_Type => "type",
               when Precomputed_Sym_Type_Class => "Type_Class",
               when Precomputed_Sym_Type_Invariant => "Type_Invariant",
               when Precomputed_Sym_Type_Invariant_Class => "Type_Invariant'Class",
               when Precomputed_Sym_Type_Key => "Type_Key",
               when Precomputed_Sym_Unbiased_Rounding => "Unbiased_Rounding",
               when Precomputed_Sym_Unbiased_Rounding_392 => "unbiased_rounding",
               when Precomputed_Sym_Unchecked_Access => "Unchecked_Access",
               when Precomputed_Sym_Unchecked_Union => "Unchecked_Union",
               when Precomputed_Sym_Unconstrained_Array => "Unconstrained_Array",
               when Precomputed_Sym_Universal_Int_Type => "Universal_Int_Type_",
               when Precomputed_Sym_Universal_Real_Type => "Universal_Real_Type_",
               when Precomputed_Sym_Unreferenced => "Unreferenced",
               when Precomputed_Sym_Unrestricted_Access => "Unrestricted_Access",
               when Precomputed_Sym_Until => "until",
               when Precomputed_Sym_Update => "Update",
               when Precomputed_Sym_Use => "use",
               when Precomputed_Sym_Vads_Size => "VADS_Size",
               when Precomputed_Sym_Val => "Val",
               when Precomputed_Sym_Val_395 => "val",
               when Precomputed_Sym_Valid => "Valid",
               when Precomputed_Sym_Valid_Scalars => "Valid_Scalars",
               when Precomputed_Sym_Value => "Value",
               when Precomputed_Sym_Value_396 => "value",
               when Precomputed_Sym_Value_Size => "Value_Size",
               when Precomputed_Sym_Variable_Indexing => "Variable_Indexing",
               when Precomputed_Sym_Volatile => "Volatile",
               when Precomputed_Sym_Volatile_Components => "Volatile_Components",
               when Precomputed_Sym_Warnings => "Warnings",
               when Precomputed_Sym_Wchar_T_Size => "Wchar_T_Size",
               when Precomputed_Sym_When => "when",
               when Precomputed_Sym_While => "while",
               when Precomputed_Sym_Wide_Character => "Wide_Character",
               when Precomputed_Sym_Wide_Image => "Wide_Image",
               when Precomputed_Sym_Wide_Image_399 => "wide_image",
               when Precomputed_Sym_Wide_String => "Wide_String",
               when Precomputed_Sym_Wide_Text_Io => "wide_text_io",
               when Precomputed_Sym_Wide_Value => "Wide_Value",
               when Precomputed_Sym_Wide_Value_401 => "wide_value",
               when Precomputed_Sym_Wide_Wide_Character => "Wide_Wide_Character",
               when Precomputed_Sym_Wide_Wide_Image => "Wide_Wide_Image",
               when Precomputed_Sym_Wide_Wide_Image_402 => "wide_wide_image",
               when Precomputed_Sym_Wide_Wide_String => "Wide_Wide_String",
               when Precomputed_Sym_Wide_Wide_Text_Io => "wide_wide_text_io",
               when Precomputed_Sym_Wide_Wide_Value => "Wide_Wide_Value",
               when Precomputed_Sym_Wide_Wide_Value_404 => "wide_wide_value",
               when Precomputed_Sym_Width => "Width",
               when Precomputed_Sym_With => "with",
               when Precomputed_Sym_Word_Size => "Word_Size",
               when Precomputed_Sym_Write => "Write",
               when Precomputed_Sym_Write_406 => "write",
               when Precomputed_Sym_Xor => """xor""",
               when Precomputed_Sym_Xor_407 => "xor"
            );
            Symbol : constant Symbolization_Result :=
                  Libadalang.Sources.Canonicalize (Raw_Text)
            ;
         begin
            if Symbol.Success then
               return Symbol.Symbol;
            else
               raise Program_Error with
                 "Cannot canonicalize symbol literal: " & Image (Raw_Text);
            end if;
         end;
   end Precomputed_Symbol;
   ----------------------------
   -- Construct_Entity_Array --
   ----------------------------
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access
   is
      Ret : Internal_Entity_Array_Access :=
        Create_Internal_Entity_Array (V.Length);
   begin
      for J in V.First_Index .. V.Last_Index loop
         Ret.Items (J) := V.Get (J);
      end loop;
      declare
         Tmp : AST_Envs.Entity_Vectors.Vector := V;
      begin
         Tmp.Destroy;
      end;
      return Ret;
   end Construct_Entity_Array;
   -----------
   -- Image --
   -----------
   function Image (Self : Symbol_Type) return String_Type is
   begin
      return Create_String (Image (Self));
   end Image;
   ------------------
   -- Context_Pool --
   ------------------
   package body Context_Pool is
      -------------
      -- Acquire --
      -------------
      procedure Acquire (Context : out Internal_Context) is
      begin
         GNAT.Task_Lock.Lock;
         if Available.Is_Empty then
            Context := new Analysis_Context_Type;
            Context.Serial_Number := 1;
         else
            Context := Available.Last_Element;
            Available.Delete_Last;
         end if;
         GNAT.Task_Lock.Unlock;
         Context.Initialized := False;
         Context.Ref_Count := 1;
      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Acquire;
      -------------
      -- Release --
      -------------
      procedure Release (Context : in out Internal_Context) is
      begin
         GNAT.Task_Lock.Lock;
         Available.Append (Context);
         Context.Serial_Number := Context.Serial_Number + 1;
         Context := null;
         GNAT.Task_Lock.Unlock;
      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Release;
      ----------
      -- Free --
      ----------
      procedure Free is
      begin
         GNAT.Task_Lock.Lock;
         for C of Available loop
            Free (C);
         end loop;
         GNAT.Task_Lock.Unlock;
      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Free;
   end Context_Pool;
   --------------
   -- Finalize --
   --------------
   overriding procedure Finalize (CD : in out Contexts_Destructor) is
      pragma Unreferenced (CD);
   begin
      Context_Pool.Free;
   end Finalize;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (File_Reader : in out Internal_File_Reader_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_File_Reader'Class, Internal_File_Reader_Access);
   begin
      if File_Reader /= null and then File_Reader.all.Dec_Ref then
         Destroy (File_Reader);
      end if;
   end Dec_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Self : in out Internal_Event_Handler_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_Event_Handler'Class, Internal_Event_Handler_Access);
   begin
      if Self /= null and then Self.all.Dec_Ref then
         Destroy (Self);
      end if;
   end Dec_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Provider : in out Internal_Unit_Provider_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_Unit_Provider'Class, Internal_Unit_Provider_Access);
   begin
      if Provider /= null and then Provider.all.Dec_Ref then
         Destroy (Provider);
      end if;
   end Dec_Ref;
   ----------------
   -- Get_Env_Id --
   ----------------
   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String
   is
      C        : Address_To_Id_Maps.Cursor;
      Inserted : Boolean;
   begin
      if E = Null_Lexical_Env then
         return "$null";
      elsif E = State.Root_Env then
         --  Insert root env with a special Id so that we only print it once
         State.Env_Ids.Insert (E, -1, C, Inserted);
         return "$root";
      end if;
      State.Env_Ids.Insert (E, State.Next_Id, C, Inserted);
      if Inserted then
         State.Next_Id := State.Next_Id + 1;
      end if;
      return '@' & Stripped_Image (Address_To_Id_Maps.Element (C));
   end Get_Env_Id;
   pragma Warnings (Off, "referenced");
   function To_Lookup_Kind_Type (K : Lookup_Kind) return Lookup_Kind_Type
   is
     (Lookup_Kind_Type'Val (Lookup_Kind'Pos (K)));
   pragma Warnings (On, "referenced");
   ----------------------
   -- Allocate_Context --
   ----------------------
   function Allocate_Context return Internal_Context is
   begin
      return Context : Internal_Context do
         Context_Pool.Acquire (Context);
      end return;
   end Allocate_Context;
   ------------------------
   -- Initialize_Context --
   ------------------------
   procedure Initialize_Context
     (Context        : Internal_Context;
      Charset        : String;
      File_Reader    : Internal_File_Reader_Access;
      Unit_Provider  : Internal_Unit_Provider_Access;
      Event_Handler  : Internal_Event_Handler_Access;
      With_Trivia    : Boolean;
      Tab_Stop       : Positive)
   is
      Actual_Charset : constant String :=
        (if Charset = "" then Default_Charset else Charset);
      Symbols        : constant Precomputed_Symbol_Table
        := Create_Symbol_Table;
   begin
      Context.Initialized := True;
      Context.Symbols := Symbol_Table (Symbols);
      Context.Charset := To_Unbounded_String (Actual_Charset);
      Context.Tab_Stop := Tab_Stop;
      Context.With_Trivia := With_Trivia;
      Context.Root_Scope := Create_Static_Lexical_Env
        (Parent => Null_Lexical_Env,
         Node   => null);
      --  Create a new ownership share for Event_Handler so that it lives at
      --  least as long as this analysis context.
      Context.Event_Handler := Event_Handler;
      if Context.Event_Handler /= null then
         Context.Event_Handler.Inc_Ref;
      end if;
      --  Create a new ownership share for File_Reader so that it lives at
      --  least as long as this analysis context.
      Context.File_Reader := File_Reader;
      if Context.File_Reader /= null then
         Context.File_Reader.Inc_Ref;
      end if;
      --  Create a new ownership share for Unit_Provider so that it lives at
      --  least as long as this analysis context.
      Context.Unit_Provider := Unit_Provider;
      if Context.Unit_Provider /= null then
         Context.Unit_Provider.Inc_Ref;
      end if;
         if Context.Unit_Provider = null then
            Context.Unit_Provider := Libadalang.Internal_Default_Provider.Create;
         end if;
      Initialize (Context.Parser);
      Context.Discard_Errors_In_Populate_Lexical_Env := True;
      Context.Logic_Resolution_Timeout :=
        Langkit_Support.Adalog.Default_Timeout_Ticks_Number;
      Context.In_Populate_Lexical_Env := False;
      Context.Cache_Version := 0;
      Context.Reparse_Cache_Version := 0;
      Context.Rewriting_Handle := No_Rewriting_Handle_Pointer;
      Context.Templates_Unit := No_Analysis_Unit;
      Context.Available_Rebindings := Env_Rebindings_Vectors.Empty_Vector;
Fetch_Standard (Context);
   end Initialize_Context;
   -----------------
   -- Create_Unit --
   -----------------
   function Create_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Unit : Internal_Unit;
   begin
      Unit := Create_Special_Unit
        (Context, Normalized_Filename, Charset, Rule);
      Context.Units.Insert (Normalized_Filename, Unit);
      return Unit;
   end Create_Unit;
   --------------
   -- Get_Unit --
   --------------
   function Get_Unit
     (Context           : Internal_Context;
      Filename, Charset : String;
      Reparse           : Boolean;
      Input             : Internal_Lexer_Input;
      Rule              : Grammar_Rule;
      Is_Internal       : Boolean := False) return Internal_Unit
   is
      use Units_Maps;
      Normalized_Filename : constant GNATCOLL.VFS.Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur     : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
      Created : constant Boolean := Cur = No_Element;
      Unit    : Internal_Unit;
      Actual_Charset : Unbounded_String;
      Refined_Input  : Internal_Lexer_Input := Input;
   begin
      --  Determine which encoding to use. Use the Charset parameter (if
      --  provided), otherwise use the context-wide default.
      Actual_Charset := (if Charset'Length /= 0
                         then To_Unbounded_String (Charset)
                         else Context.Charset);
      if Refined_Input.Kind = File then
         Refined_Input.Filename := Normalized_Filename;
      end if;
      if Refined_Input.Kind in File | Bytes_Buffer then
         Refined_Input.Charset := Actual_Charset;
         --  Unless the caller requested a specific charset for this unit,
         --  allow the lexer to automatically discover the source file encoding
         --  before defaulting to the context-specific one. We do this trying
         --  to match a byte order mark.
         Refined_Input.Read_BOM := Charset'Length = 0;
      end if;
      --  Create the Internal_Unit if needed
      Unit :=
        (if Created
         then Create_Unit (Context, Normalized_Filename,
                           To_String (Actual_Charset), Rule)
         else Element (Cur));
      --  If an internal unit is requested, set the corresponding flag.
      --  Otherwise, make sure that the unit we return isn't internal.
      if Is_Internal then
         Unit.Is_Internal := True;
      end if;
      --  (Re)parse it if needed
      if Created or else Reparse then
         --  It is illegal to reparse an internal unit for public API users.
         --  Since public APIs do not allow to pass True to Is_Internal, we can
         --  check here that only the implementation can ask to reparse an
         --  internal unit.
         if Unit.Is_Internal and then not Is_Internal then
            raise Precondition_Failure with "cannot reparse an internal unit";
         end if;
         declare
            Reparsed : Reparsed_Unit;
         begin
            Do_Parsing (Unit, Refined_Input, Reparsed);
            Update_After_Reparse (Unit, Reparsed);
         end;
         --  Now that we have removed reparsed the unit, update its current
         --  charset.
         Unit.Charset := Actual_Charset;
      end if;
      if Context.Event_Handler /= null then
         Context.Event_Handler.Unit_Parsed_Callback
           (Context  => Context,
            Unit     => Unit,
            Reparsed => not Created and then Reparse);
      end if;
      return Unit;
   end Get_Unit;
   --------------
   -- Has_Unit --
   --------------
   function Has_Unit
     (Context : Internal_Context; Unit_Filename : String) return Boolean is
   begin
      return Context.Units.Contains
        (Normalized_Unit_Filename (Context, Unit_Filename));
   end Has_Unit;
   -------------------
   -- Get_From_File --
   -------------------
   function Get_From_File
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Reparse  : Boolean;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind     => File,
         Charset  => <>,
         Read_BOM => False,
         Filename => <>);
   begin
      if Reparse and then Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot reparse during tree rewriting";
      end if;
      return Get_Unit (Context, Filename, Charset, Reparse, Input, Rule);
   end Get_From_File;
   ---------------------
   -- Get_From_Buffer --
   ---------------------
   function Get_From_Buffer
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Buffer   : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind        => Bytes_Buffer,
         Charset     => <>,
         Read_BOM    => False,
         Bytes       => Buffer'Address,
         Bytes_Count => Buffer'Length);
   begin
      if Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot parse from buffer during tree rewriting";
      elsif Context.File_Reader /= null then
         raise Precondition_Failure with
            "cannot parse from buffer with a file reader";
      end if;
      return Get_Unit (Context, Filename, Charset, True, Input, Rule);
   end Get_From_Buffer;
   --------------------
   -- Get_With_Error --
   --------------------
   function Get_With_Error
     (Context  : Internal_Context;
      Filename : String;
      Error    : Text_Type;
      Charset  : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Normalized_Filename : constant Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur                 : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
   begin
      if Cur = No_Element then
         declare
            Unit : constant Internal_Unit := Create_Unit
              (Context, Normalized_Filename, Charset, Rule);
         begin
            Append (Unit.Diagnostics, No_Source_Location_Range, Error);
            return Unit;
         end;
      else
         return Element (Cur);
      end if;
   end Get_With_Error;
   -----------------------
   -- Get_From_Provider --
   -----------------------
   function Get_From_Provider
     (Context : Internal_Context;
      Name    : Text_Type;
      Kind    : Analysis_Unit_Kind;
      Charset : String;
      Reparse : Boolean) return Internal_Unit is
   begin
      if Reparse and then Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot reparse during tree rewriting";
      end if;
      declare
         Result      : Internal_Unit;
         Dummy_Index : Positive;
      begin
         Context.Unit_Provider.Get_Unit_And_PLE_Root
           (Context, Name, Kind, Charset, Reparse, Result, Dummy_Index);
         return Result;
      exception
         when Precondition_Failure | Property_Error =>
            raise Invalid_Unit_Name_Error with
               "Invalid unit name: " & Image (Name, With_Quotes => True)
               & " (" & Analysis_Unit_Kind'Image (Kind) & ")";
      end;
   end Get_From_Provider;
   -------------------
   -- Unit_Provider --
   -------------------
   function Unit_Provider
     (Context : Internal_Context) return Internal_Unit_Provider_Access
   is (Context.Unit_Provider);
   ------------------
   -- Resolve_Unit --
   ------------------
   procedure Resolve_Unit
     (Context : Internal_Context;
      Name    : Text_Type;
      Kind    : Analysis_Unit_Kind;
      Unit    : out Resolved_Unit)
   is
      --  Look for the cache entry corresponding to Unit; create one if needed
      Dummy    : Resolved_Unit_Array;
      Key      : constant Symbol_Type := Find (Context.Symbols, Name);
      Pos      : Unit_Provider_Cache_Maps.Cursor;
      Inserted : Boolean;
   begin
      Context.Unit_Provider_Cache.Insert (Key, Dummy, Pos, Inserted);
      declare
         Units : Resolved_Unit_Array renames
           Context.Unit_Provider_Cache.Reference (Pos);
         U     : Resolved_Unit renames Units (Kind);
      begin
         --  If the cache entry is not populated for the requested kind, run
         --  the query and save the result for later requests.
         if U.Filename = null then
            declare
               Provider : Internal_Unit_Provider'Class renames
                 Context.Unit_Provider.all;
               Filename : Unbounded_String;
            begin
               Provider.Get_Unit_Location
                 (Name           => Name,
                  Kind           => Kind,
                  Filename       => Filename,
                  PLE_Root_Index => U.PLE_Root_Index);
               Provider.Get_Unit_And_PLE_Root
                 (Context        => Context,
                  Name           => Name,
                  Kind           => Kind,
                  Unit           => U.Unit,
                  PLE_Root_Index => U.PLE_Root_Index);
               U.Filename := new String'(To_String (Filename));
            end;
         end if;
         Unit := U;
      end;
   end Resolve_Unit;
   -----------------------
   -- Get_Unit_Location --
   -----------------------
   procedure Get_Unit_Location
     (Context        : Internal_Context;
      Name           : Text_Type;
      Kind           : Analysis_Unit_Kind;
      Filename       : out String_Access;
      PLE_Root_Index : out Positive)
   is
      U : Resolved_Unit;
   begin
      Resolve_Unit (Context, Name, Kind, U);
      Filename := U.Filename;
      PLE_Root_Index := U.PLE_Root_Index;
   end Get_Unit_Location;
   ---------------------------
   -- Get_Unit_And_PLE_Root --
   ---------------------------
   procedure Get_Unit_And_PLE_Root
     (Context        : Internal_Context;
      Name           : Text_Type;
      Kind           : Analysis_Unit_Kind;
      Unit           : out Internal_Unit;
      PLE_Root_Index : out Positive)
   is
      U : Resolved_Unit;
   begin
      Resolve_Unit (Context, Name, Kind, U);
      Unit := U.Unit;
      PLE_Root_Index := U.PLE_Root_Index;
   end Get_Unit_And_PLE_Root;
   ----------
   -- Hash --
   ----------
   function Hash (Context : Internal_Context) return Hash_Type is
      function H is new Hash_Access (Analysis_Context_Type, Internal_Context);
   begin
      return H (Context);
   end Hash;
   ---------------------
   -- Has_With_Trivia --
   ---------------------
   function Has_With_Trivia (Context : Internal_Context) return Boolean is
   begin
      return Context.With_Trivia;
   end Has_With_Trivia;
   --------------------------------------------
   -- Discard_Errors_In_Populate_Lexical_Env --
   --------------------------------------------
   procedure Discard_Errors_In_Populate_Lexical_Env
     (Context : Internal_Context; Discard : Boolean) is
   begin
      Context.Discard_Errors_In_Populate_Lexical_Env := Discard;
   end Discard_Errors_In_Populate_Lexical_Env;
   ----------------------------------
   -- Set_Logic_Resolution_Timeout --
   ----------------------------------
   procedure Set_Logic_Resolution_Timeout
     (Context : Internal_Context; Timeout : Natural) is
   begin
      Context.Logic_Resolution_Timeout := Timeout;
   end Set_Logic_Resolution_Timeout;
   --------------------------
   -- Has_Rewriting_Handle --
   --------------------------
   function Has_Rewriting_Handle (Context : Internal_Context) return Boolean is
   begin
      return Context.Rewriting_Handle /= No_Rewriting_Handle_Pointer;
   end Has_Rewriting_Handle;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Context : Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Context : in out Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count - 1;
         if Context.Ref_Count = 0 then
            --  If this context was not completely initialized, just release
            --  the allocated object. Do the full destruction otherwise.
            if Context.Initialized then
               Destroy (Context);
            end if;
            Context_Pool.Release (Context);
         end if;
      end if;
   end Dec_Ref;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Context : Internal_Context) is
   begin
      --  Destroy all named environment data structures
      for Desc of Context.Named_Envs loop
         for V of Desc.Foreign_Nodes loop
            V.Destroy;
         end loop;
         Destroy (Desc);
      end loop;
      Context.Named_Envs.Clear;
      --  If we are asked to free this context, it means that no one else have
      --  references to its analysis units, so it's safe to destroy these.
      for Unit of Context.Units loop
         Destroy (Unit);
      end loop;
      Context.Units := Units_Maps.Empty_Map;
      Context.Filenames := Virtual_File_Maps.Empty_Map;
      declare
         procedure Destroy is new Ada.Unchecked_Deallocation
           (Env_Rebindings_Type, Env_Rebindings);
         AR : Env_Rebindings_Vectors.Vector renames
            Context.Available_Rebindings;
         R  : Env_Rebindings;
      begin
         for I in AR.First_Index .. AR.Last_Index loop
            R := AR.Get (I);
            Destroy (R);
         end loop;
         AR.Destroy;
      end;
      for Pos in Context.Unit_Provider_Cache.Iterate loop
         declare
            Units : Resolved_Unit_Array renames
              Context.Unit_Provider_Cache.Reference (Pos);
         begin
            for U of Units loop
               Free (U.Filename);
            end loop;
         end;
      end loop;
      Context.Unit_Provider_Cache.Clear;
      Destroy (Context.Templates_Unit);
      AST_Envs.Destroy (Context.Root_Scope);
      Destroy (Context.Symbols);
      Destroy (Context.Parser);
      Dec_Ref (Context.File_Reader);
      Dec_Ref (Context.Unit_Provider);
      Dec_Ref (Context.Event_Handler);
   end Destroy;
   -------------
   -- Context --
   -------------
   function Context (Unit : Internal_Unit) return Internal_Context is
   begin
      return Unit.Context;
   end Context;
   ----------
   -- Hash --
   ----------
   function Hash (Unit : Internal_Unit) return Hash_Type is
      function H is new Hash_Access (Analysis_Unit_Type, Internal_Unit);
   begin
      return H (Unit);
   end Hash;
   -------------
   -- Reparse --
   -------------
   procedure Reparse (Unit : Internal_Unit; Charset : String) is
      Dummy : constant Internal_Unit := Get_From_File
        (Unit.Context, +Unit.Filename.Full_Name, Charset,
         Reparse => True,
         Rule    => Unit.Rule);
   begin
      null;
   end Reparse;
   -------------
   -- Reparse --
   -------------
   procedure Reparse (Unit : Internal_Unit; Charset : String; Buffer : String)
   is
      Dummy : constant Internal_Unit := Get_From_Buffer
        (Unit.Context, +Unit.Filename.Full_Name, Charset, Buffer, Unit.Rule);
   begin
      null;
   end Reparse;
   --------------------------
   -- Populate_Lexical_Env --
   --------------------------
   procedure Populate_Lexical_Env
     (Unit           : Internal_Unit;
      PLE_Root_Index : Positive
   ) is
      Context : constant Internal_Context := Unit.Context;
      Saved_In_Populate_Lexical_Env : constant Boolean :=
        Context.In_Populate_Lexical_Env;
      Has_Errors : Boolean := False;
      --  Whether at least one Property_Error occurred during this PLE pass
      procedure Reset_Envs_Caches (Unit : Internal_Unit);
      --  Reset the env caches of all lexical environments created for ``Unit``
      -----------------------
      -- Reset_Envs_Caches --
      -----------------------
      procedure Reset_Envs_Caches (Unit : Internal_Unit) is
         procedure Internal (Node : Bare_Ada_Node);
         --  Reset env caches in ``Node`` and then in its children recursively
         --------------
         -- Internal --
         --------------
         procedure Internal (Node : Bare_Ada_Node) is
         begin
            if Node = null then
               return;
            end if;
            Reset_Caches (Node.Self_Env);
            for I in 1 .. Children_Count (Node) loop
               Internal (Child (Node, I));
            end loop;
         end Internal;
      begin
         Internal (Unit.Ast_Root);
      end Reset_Envs_Caches;
   begin
      --  TODO??? Handle env invalidation when reparsing a unit and when a
      --  previous call raised a Property_Error.
      --  If we have already run PLE on this root, there is nothing to do.
      --  Otherwise, keep track of the fact that PLE was requested for it,
      --  possibly extending the vector if needed.
      if Unit.Env_Populated_Roots.Last_Index >= PLE_Root_Index
         and then Unit.Env_Populated_Roots.Get (PLE_Root_Index)
      then
         return;
      end if;
      for Dummy in Unit.Env_Populated_Roots.Last_Index + 1 .. PLE_Root_Index
      loop
         Unit.Env_Populated_Roots.Append (False);
      end loop;
      Unit.Env_Populated_Roots.Set (PLE_Root_Index, True);
      --  Create context for the PLE run: all exit points must call the Cleanup
      --  procedure above first to clean this context.
      Context.In_Populate_Lexical_Env := True;
      if Main_Trace.Active then
         Main_Trace.Trace
           ("Populating lexical envs for"
            & " root" & PLE_Root_Index'Image & " of"
            & " unit: " & Basename (Unit));
         Main_Trace.Increase_Indent;
      end if;
      --  Fetch the node on which to run PLE: it's the unit root node, or one
      --  of its children if PLE roots are enabled and the unit has a list of
      --  PLE roots. Then run PLE itself.
      declare
         PLE_Root : Bare_Ada_Node := Unit.Ast_Root;
      begin
            if Unit.Ast_Root /= null
               and then Unit.Ast_Root.Kind
                        = Ada_Compilation_Unit_List
            then
               PLE_Root := Child (Unit.Ast_Root, PLE_Root_Index);
            end if;
         if PLE_Root /= null then
            Has_Errors := Populate_Lexical_Env (PLE_Root);
         end if;
      end;
      --  Restore the context for PLE run (undo what was done above)
      Context.In_Populate_Lexical_Env := Saved_In_Populate_Lexical_Env;
      if Main_Trace.Active then
         Main_Trace.Decrease_Indent;
         Main_Trace.Trace
           ("Finished populating lexical envs for"
            & " root" & PLE_Root_Index'Image & " of"
            & " unit: " & Basename (Unit));
      end if;
      Reset_Envs_Caches (Unit);
      if Has_Errors and then not Context.Discard_Errors_In_Populate_Lexical_Env
      then
         raise Property_Error with
            "errors occurred in Populate_Lexical_Env";
      end if;
   end Populate_Lexical_Env;
   -----------------------------------
   -- Populate_Lexical_Env_For_Unit --
   -----------------------------------
   procedure Populate_Lexical_Env_For_Unit (Node : Bare_Ada_Node) is
      Root  : Bare_Ada_Node;
      Index : Natural;
   begin
      Lookup_PLE_Root (Node, Root, Index);
      if Index = 0 then
         Index := 1;
      end if;
      Populate_Lexical_Env (Node.Unit, Index);
   end Populate_Lexical_Env_For_Unit;
   ------------------
   -- Get_Filename --
   ------------------
   function Get_Filename (Unit : Internal_Unit) return String is
     (+Unit.Filename.Full_Name);
   -----------------
   -- Get_Charset --
   -----------------
   function Get_Charset (Unit : Internal_Unit) return String is
   begin
      return To_String (Unit.Charset);
   end Get_Charset;
   ---------------------
   -- Has_Diagnostics --
   ---------------------
   function Has_Diagnostics (Unit : Internal_Unit) return Boolean is
   begin
      return not Unit.Diagnostics.Is_Empty;
   end Has_Diagnostics;
   -----------------
   -- Diagnostics --
   -----------------
   function Diagnostics (Unit : Internal_Unit) return Diagnostics_Array is
      Result : Diagnostics_Array (1 .. Natural (Unit.Diagnostics.Length));
      I      : Natural := 1;
   begin
      for D of Unit.Diagnostics loop
         Result (I) := D;
         I := I + 1;
      end loop;
      return Result;
   end Diagnostics;
   ---------------------------
   -- Format_GNU_Diagnostic --
   ---------------------------
   function Format_GNU_Diagnostic
     (Unit : Internal_Unit; D : Diagnostic) return String
   is
      Filename : constant String := Basename (Unit);
      Sloc     : constant Source_Location := Start_Sloc (D.Sloc_Range);
      Msg      : constant String :=
         Image
           (Ada.Strings.Wide_Wide_Unbounded.To_Wide_Wide_String (D.Message));
   begin
      return (Filename
              & (if Sloc = No_Source_Location then "" else ":" & Image (Sloc))
              & ": " & Msg);
   end Format_GNU_Diagnostic;
   ----------
   -- Root --
   ----------
   function Root (Unit : Internal_Unit) return Bare_Ada_Node is
     (Unit.Ast_Root);
   -----------------
   -- First_Token --
   -----------------
   function First_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.Context,
                            Unit.TDH'Access,
                            First_Token_Or_Trivia (Unit.TDH)));
   ----------------
   -- Last_Token --
   ----------------
   function Last_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.Context,
                            Unit.TDH'Access,
                            Last_Token_Or_Trivia (Unit.TDH)));
   -----------------
   -- Token_Count --
   -----------------
   function Token_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Tokens.Length);
   ------------------
   -- Trivia_Count --
   ------------------
   function Trivia_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Trivias.Length);
   ----------
   -- Text --
   ----------
   function Text (Unit : Internal_Unit) return Text_Type is
   begin
      return Text (First_Token (Unit), Last_Token (Unit));
   end Text;
   ------------------
   -- Lookup_Token --
   ------------------
   function Lookup_Token
     (Unit : Internal_Unit; Sloc : Source_Location) return Token_Reference
   is
      Result : constant Token_Or_Trivia_Index := Lookup_Token (Unit.TDH, Sloc);
   begin
      return Wrap_Token_Reference (Unit.Context, Unit.TDH'Access, Result);
   end Lookup_Token;
   ---------------------
   -- Lookup_PLE_Root --
   ---------------------
   procedure Lookup_PLE_Root
     (Node  : Bare_Ada_Node;
      Root  : out Bare_Ada_Node;
      Index : out Natural)
   is
      Unit : constant Internal_Unit := Node.Unit;
   begin
      --  If this unit does not contain a list of PLE roots, just return the
      --  unit root node.
      if Unit.PLE_Roots_Starting_Token.Is_Empty then
         Root := Unit.Ast_Root;
         Index := 0;
         return;
      end if;
      --  Otherwise, look for the last PLE root whose first token (in
      --  Unit.PLE_Roots_Starting_Token) appears before Node's (T). This vector
      --  is sorted by construction, so we can perform a binary search.
      declare
         T      : constant Token_Index := Node.Token_Start_Index;
         Tokens : Token_Index_Vectors.Vector renames
           Unit.PLE_Roots_Starting_Token;
         First : Positive := Tokens.First_Index;
         Last  : Positive := Tokens.Last_Index;
         I     : Positive;
      begin
         while First < Last loop
            --  Because we look for the "floor" (last element that is <= T), we
            --  need to look at the value in Last when there are only two
            --  elements left to look at. If we did not do that, then we would
            --  go into an infinite loop when Tokens[First] < T.
            I := (if First + 1 = Last
                  then Last
                  else (First + Last) / 2);
            declare
               I_T : constant Token_Index := Tokens.Get (I);
            begin
               if I_T <= T then
                  First := I;
               else
                  Last := I - 1;
               end if;
            end;
         end loop;
         Root := Child (Unit.Ast_Root, First);
         Index := First;
      end;
   end Lookup_PLE_Root;
   --------------
   -- Ple_Root --
   --------------
   function Ple_Root
     (Node : Bare_Ada_Node) return Bare_Ada_Node
   is
      Root        : Bare_Ada_Node;
      Dummy_Index : Natural;
   begin
      if Node = null then
         raise Property_Error with "null node dereference";
      end if;
      Lookup_PLE_Root (Node, Root, Dummy_Index);
      return Root;
   end Ple_Root;
   ----------------------
   -- Dump_Lexical_Env --
   ----------------------
   procedure Dump_Lexical_Env (Unit : Internal_Unit) is
      Node     : constant Bare_Ada_Node := Unit.Ast_Root;
      Root_Env : constant Lexical_Env := Unit.Context.Root_Scope;
      State    : Dump_Lexical_Env_State := (Root_Env => Root_Env, others => <>);
      function Get_Parent (Env : Lexical_Env) return Lexical_Env
      is (Unwrap (Env).Parent);
      --------------------------
      -- Explore_Parent_Chain --
      --------------------------
      procedure Explore_Parent_Chain (Env : Lexical_Env) is
         P : Lexical_Env;
      begin
         if Env /= Null_Lexical_Env then
            P := Get_Parent (Env);
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (P, State));
            Explore_Parent_Chain (P);
         end if;
      end Explore_Parent_Chain;
      --------------
      -- Internal --
      --------------
      procedure Internal (Current : Bare_Ada_Node) is
         Explore_Parent : Boolean := False;
         Env, Parent    : Lexical_Env;
      begin
         if Current = null then
            return;
         end if;
         --  We only dump environments that we haven't dumped before. This way
         --  we'll only dump environments at the site of their creation, and
         --  not in any subsequent link. We use the Env_Ids map to check which
         --  envs we have already seen or not.
         if not State.Env_Ids.Contains (Current.Self_Env) then
            Env := Current.Self_Env;
            Parent := Get_Parent (Env);
            Explore_Parent := not State.Env_Ids.Contains (Parent);
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (Parent, State));
            if Explore_Parent then
               Explore_Parent_Chain (Parent);
            end if;
         end if;
         for Child of Internal_Bare_Ada_Node_Array'(Children (Current))
         loop
            Internal (Child);
         end loop;
      end Internal;
      --  This procedure implements the main recursive logic of dumping the
      --  environments.
   begin
      Internal (Bare_Ada_Node (Node));
   end Dump_Lexical_Env;
   --------------
   -- Get_Line --
   --------------
   function Get_Line
     (Unit : Internal_Unit; Line_Number : Positive) return Text_Type
   is
   begin
      return Get_Line (Unit.TDH, Line_Number);
   end Get_Line;
   -----------
   -- Print --
   -----------
   procedure Print (Unit : Internal_Unit; Show_Slocs : Boolean) is
   begin
      if Unit.Ast_Root = null then
         Put_Line ("<empty analysis unit>");
      else
         Print (Unit.Ast_Root, Show_Slocs);
      end if;
   end Print;
   ---------------
   -- PP_Trivia --
   ---------------
   procedure PP_Trivia (Unit : Internal_Unit) is
      procedure Process (Trivia : Token_Index) is
         Data : constant Stored_Token_Data :=
            Unit.TDH.Trivias.Get (Natural (Trivia)).T;
      begin
         Put_Line (Image (Text (Unit.TDH, Data)));
      end Process;
      Last_Token : constant Token_Index :=
         Token_Index (Token_Vectors.Last_Index (Unit.TDH.Tokens) - 1);
      --  Index for the last token in Unit excluding the Termination token
      --  (hence the -1).
   begin
      for Tok of Get_Leading_Trivias (Unit.TDH) loop
         Process (Tok);
      end loop;
      PP_Trivia (Unit.Ast_Root);
      for Tok of Get_Trivias (Unit.TDH, Last_Token) loop
         Process (Tok);
      end loop;
   end PP_Trivia;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Unit : in out Internal_Unit) is
   begin
      if Unit = No_Analysis_Unit then
         return;
      end if;
      Unit.PLE_Roots_Starting_Token.Destroy;
      Unit.Env_Populated_Roots.Destroy;
      Unit.Exiled_Entries.Destroy;
      Unit.Foreign_Nodes.Destroy;
      Unit.Exiled_Entries_In_NED.Destroy;
      Unit.Exiled_Envs.Destroy;
      Unit.Named_Envs.Destroy;
      Analysis_Unit_Sets.Destroy (Unit.Referenced_Units);
         Destroy (Unit.Memoization_Map);
      Destroy_Rebindings (Unit.Rebindings'Access);
      Unit.Rebindings.Destroy;
      if Unit.Ast_Root /= null then
         Destroy (Unit.Ast_Root);
      end if;
      Free (Unit.TDH);
      Free (Unit.Ast_Mem_Pool);
      Destroy_Unit_Destroyables (Unit);
      Destroyable_Vectors.Destroy (Unit.Destroyables);
for Cur in Unit.Nodes_Nameres.Iterate loop
   declare
      V : Resolution_Val renames Unit.Nodes_Nameres.Reference (Cur);
   begin
      Free_Memoized_Error (V.Exc_Id, V.Exc_Msg);
   end;
end loop;
      Free (Unit);
   end Destroy;
   -------------------
   -- Is_Token_Node --
   -------------------
   function Is_Token_Node (Node : Bare_Ada_Node) return Boolean is
   begin
      return Is_Token_Node (Node.Kind);
   end Is_Token_Node;
   ------------------
   -- Is_Synthetic --
   ------------------
   function Is_Synthetic (Node : Bare_Ada_Node) return Boolean is
   begin
      return Node.Kind in Synthetic_Nodes;
   end Is_Synthetic;
   ------------------------------
   -- Raise_Property_Exception --
   ------------------------------
   procedure Raise_Property_Exception
     (Node    : Bare_Ada_Node;
      Exc     : Ada.Exceptions.Exception_Id;
      Message : String)
   is
      Sloc_Prefix : constant String :=
        (if Node = null
         then ""
         else Ada.Directories.Simple_Name (Get_Filename (Unit (Node)))
              & ":" & Image (Sloc_Range (Node)) & ": ");
   begin
      Ada.Exceptions.Raise_Exception (Exc, Sloc_Prefix & Message);
   end Raise_Property_Exception;
   ------------------------------
   -- Register_Destroyable_Gen --
   ------------------------------
   procedure Register_Destroyable_Gen
     (Unit : Internal_Unit; Object : T_Access)
   is
      function Convert is new Ada.Unchecked_Conversion
        (System.Address, Destroy_Procedure);
      procedure Destroy_Procedure (Object : in out T_Access) renames Destroy;
   begin
      Register_Destroyable_Helper
        (Unit,
         Object.all'Address,
         Convert (Destroy_Procedure'Address));
   end Register_Destroyable_Gen;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Ada_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Ada_Node
   is
      function Absolute_Get
        (T : Bare_Ada_Node_Array_Access; Index : Integer)
         return Bare_Ada_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Ada_Node,
         Sequence_Type => Bare_Ada_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Ada_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Ada_Node_Array_Access) return Bare_Ada_Node_Array_Access is
      Ret : Bare_Ada_Node_Array_Access := Create_Bare_Ada_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Ada_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Ada_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Ada_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Ada_Node_Array (Items_Count : Natural) return Bare_Ada_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Ada_Node_Array_Type
       else new Bare_Ada_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Ada_Node_Array
     (Items : Internal_Bare_Ada_Node_Array) return Bare_Ada_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Ada_Node_Array_Type;
      end if;
      return new Bare_Ada_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Ada_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Ada_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Anonymous_Expr_Decl_Vectors is new Langkit_Support.Vectors (Bare_Anonymous_Expr_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Anonymous_Expr_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Anonymous_Expr_Decl
   is
      function Absolute_Get
        (T : Bare_Anonymous_Expr_Decl_Array_Access; Index : Integer)
         return Bare_Anonymous_Expr_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Anonymous_Expr_Decl,
         Sequence_Type => Bare_Anonymous_Expr_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Anonymous_Expr_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Anonymous_Expr_Decl_Array_Access) return Bare_Anonymous_Expr_Decl_Array_Access is
      Ret : Bare_Anonymous_Expr_Decl_Array_Access := Create_Bare_Anonymous_Expr_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Anonymous_Expr_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Anonymous_Expr_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Anonymous_Expr_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Anonymous_Expr_Decl_Array (Items_Count : Natural) return Bare_Anonymous_Expr_Decl_Array_Access
   is (if Items_Count = 0
       then No_Bare_Anonymous_Expr_Decl_Array_Type
       else new Bare_Anonymous_Expr_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Anonymous_Expr_Decl_Array
     (Items : Internal_Bare_Anonymous_Expr_Decl_Array) return Bare_Anonymous_Expr_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Anonymous_Expr_Decl_Array_Type;
      end if;
      return new Bare_Anonymous_Expr_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Anonymous_Expr_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Anonymous_Expr_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Compilation_Unit_Vectors is new Langkit_Support.Vectors (Bare_Compilation_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Compilation_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Compilation_Unit
   is
      function Absolute_Get
        (T : Bare_Compilation_Unit_Array_Access; Index : Integer)
         return Bare_Compilation_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Compilation_Unit,
         Sequence_Type => Bare_Compilation_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Compilation_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Compilation_Unit_Array_Access) return Bare_Compilation_Unit_Array_Access is
      Ret : Bare_Compilation_Unit_Array_Access := Create_Bare_Compilation_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Compilation_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Compilation_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Compilation_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Compilation_Unit_Array (Items_Count : Natural) return Bare_Compilation_Unit_Array_Access
   is (if Items_Count = 0
       then No_Bare_Compilation_Unit_Array_Type
       else new Bare_Compilation_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Compilation_Unit_Array
     (Items : Internal_Bare_Compilation_Unit_Array) return Bare_Compilation_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Compilation_Unit_Array_Type;
      end if;
      return new Bare_Compilation_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Compilation_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Compilation_Unit_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Expr_Vectors is new Langkit_Support.Vectors (Bare_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Expr
   is
      function Absolute_Get
        (T : Bare_Expr_Array_Access; Index : Integer)
         return Bare_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Expr,
         Sequence_Type => Bare_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Expr_Array_Access) return Bare_Expr_Array_Access is
      Ret : Bare_Expr_Array_Access := Create_Bare_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Expr_Array (Items_Count : Natural) return Bare_Expr_Array_Access
   is (if Items_Count = 0
       then No_Bare_Expr_Array_Type
       else new Bare_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Expr_Array
     (Items : Internal_Bare_Expr_Array) return Bare_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Expr_Array_Type;
      end if;
      return new Bare_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Expr_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Name_Vectors is new Langkit_Support.Vectors (Bare_Name);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Name
   is
      function Absolute_Get
        (T : Bare_Name_Array_Access; Index : Integer)
         return Bare_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Name,
         Sequence_Type => Bare_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Name_Array_Access) return Bare_Name_Array_Access is
      Ret : Bare_Name_Array_Access := Create_Bare_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Name_Array (Items_Count : Natural) return Bare_Name_Array_Access
   is (if Items_Count = 0
       then No_Bare_Name_Array_Type
       else new Bare_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Name_Array
     (Items : Internal_Bare_Name_Array) return Bare_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Name_Array_Type;
      end if;
      return new Bare_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Name_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Pragma_Node_Vectors is new Langkit_Support.Vectors (Bare_Pragma_Node);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Pragma_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Pragma_Node
   is
      function Absolute_Get
        (T : Bare_Pragma_Node_Array_Access; Index : Integer)
         return Bare_Pragma_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Pragma_Node,
         Sequence_Type => Bare_Pragma_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Pragma_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Pragma_Node_Array_Access) return Bare_Pragma_Node_Array_Access is
      Ret : Bare_Pragma_Node_Array_Access := Create_Bare_Pragma_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Pragma_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Pragma_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Pragma_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Pragma_Node_Array (Items_Count : Natural) return Bare_Pragma_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Pragma_Node_Array_Type
       else new Bare_Pragma_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Pragma_Node_Array
     (Items : Internal_Bare_Pragma_Node_Array) return Bare_Pragma_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Pragma_Node_Array_Type;
      end if;
      return new Bare_Pragma_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Pragma_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Pragma_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Single_Tok_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Single_Tok_Node
   is
      function Absolute_Get
        (T : Bare_Single_Tok_Node_Array_Access; Index : Integer)
         return Bare_Single_Tok_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Single_Tok_Node,
         Sequence_Type => Bare_Single_Tok_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Single_Tok_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Single_Tok_Node_Array_Access) return Bare_Single_Tok_Node_Array_Access is
      Ret : Bare_Single_Tok_Node_Array_Access := Create_Bare_Single_Tok_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Single_Tok_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Single_Tok_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Single_Tok_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Single_Tok_Node_Array (Items_Count : Natural) return Bare_Single_Tok_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Single_Tok_Node_Array_Type
       else new Bare_Single_Tok_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Single_Tok_Node_Array
     (Items : Internal_Bare_Single_Tok_Node_Array) return Bare_Single_Tok_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Single_Tok_Node_Array_Type;
      end if;
      return new Bare_Single_Tok_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Single_Tok_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Single_Tok_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Use_Clause_Vectors is new Langkit_Support.Vectors (Bare_Use_Clause);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Use_Clause_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Use_Clause
   is
      function Absolute_Get
        (T : Bare_Use_Clause_Array_Access; Index : Integer)
         return Bare_Use_Clause
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Use_Clause,
         Sequence_Type => Bare_Use_Clause_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Use_Clause;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Use_Clause_Array_Access) return Bare_Use_Clause_Array_Access is
      Ret : Bare_Use_Clause_Array_Access := Create_Bare_Use_Clause_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Use_Clause_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Use_Clause_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Use_Clause_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Use_Clause_Array (Items_Count : Natural) return Bare_Use_Clause_Array_Access
   is (if Items_Count = 0
       then No_Bare_Use_Clause_Array_Type
       else new Bare_Use_Clause_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Use_Clause_Array
     (Items : Internal_Bare_Use_Clause_Array) return Bare_Use_Clause_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Use_Clause_Array_Type;
      end if;
      return new Bare_Use_Clause_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Use_Clause_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Use_Clause_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Aspect_Vectors is new Langkit_Support.Vectors (Internal_Aspect);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Aspect_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Aspect
   is
      function Absolute_Get
        (T : Internal_Aspect_Array_Access; Index : Integer)
         return Internal_Aspect
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Aspect,
         Sequence_Type => Internal_Aspect_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Aspect;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Aspect;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Aspect_Array_Access) return Internal_Aspect_Array_Access is
      Ret : Internal_Aspect_Array_Access := Create_Internal_Aspect_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Aspect_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Aspect_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Aspect_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Aspect_Array (Items_Count : Natural) return Internal_Aspect_Array_Access
   is (if Items_Count = 0
       then No_Internal_Aspect_Array_Type
       else new Internal_Aspect_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Aspect_Array
     (Items : Internal_Internal_Aspect_Array) return Internal_Aspect_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Aspect_Array_Type;
      end if;
      return new Internal_Aspect_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Aspect_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Aspect_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Completion_Item_Vectors is new Langkit_Support.Vectors (Internal_Completion_Item);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Completion_Item_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Completion_Item
   is
      function Absolute_Get
        (T : Internal_Completion_Item_Array_Access; Index : Integer)
         return Internal_Completion_Item
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Completion_Item,
         Sequence_Type => Internal_Completion_Item_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Completion_Item;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Completion_Item;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Completion_Item_Array_Access) return Internal_Completion_Item_Array_Access is
      Ret : Internal_Completion_Item_Array_Access := Create_Internal_Completion_Item_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Completion_Item_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Completion_Item_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Completion_Item_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Completion_Item_Array (Items_Count : Natural) return Internal_Completion_Item_Array_Access
   is (if Items_Count = 0
       then No_Internal_Completion_Item_Array_Type
       else new Internal_Completion_Item_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Completion_Item_Array
     (Items : Internal_Internal_Completion_Item_Array) return Internal_Completion_Item_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Completion_Item_Array_Type;
      end if;
      return new Internal_Completion_Item_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Completion_Item_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- To_Iterator --
      -----------------
      function To_Iterator
        (Self    : Internal_Completion_Item_Array_Access;
         Context : Internal_Context) return Internal_Completion_Item_Iterator_Access
      is
      begin
         Inc_Ref (Self);
         return new Internal_Internal_Completion_Item_Iterator'
           (Ref_Count  => 1,
            Safety_Net => Create_Safety_Net (Context),
            Elements   => Self,
            Index      => 1);
      end To_Iterator;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Completion_Item_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Discriminant_Values_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Discriminant_Values
   is
      function Absolute_Get
        (T : Internal_Discriminant_Values_Array_Access; Index : Integer)
         return Internal_Discriminant_Values
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Discriminant_Values,
         Sequence_Type => Internal_Discriminant_Values_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Discriminant_Values;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Discriminant_Values;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Discriminant_Values_Array_Access) return Internal_Discriminant_Values_Array_Access is
      Ret : Internal_Discriminant_Values_Array_Access := Create_Internal_Discriminant_Values_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Discriminant_Values_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Discriminant_Values_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Discriminant_Values_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Discriminant_Values_Array (Items_Count : Natural) return Internal_Discriminant_Values_Array_Access
   is (if Items_Count = 0
       then No_Internal_Discriminant_Values_Array_Type
       else new Internal_Discriminant_Values_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Discriminant_Values_Array
     (Items : Internal_Internal_Discriminant_Values_Array) return Internal_Discriminant_Values_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Discriminant_Values_Array_Type;
      end if;
      return new Internal_Discriminant_Values_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Discriminant_Values_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Discriminant_Values_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Doc_Annotation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Doc_Annotation
   is
      function Absolute_Get
        (T : Internal_Doc_Annotation_Array_Access; Index : Integer)
         return Internal_Doc_Annotation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Doc_Annotation,
         Sequence_Type => Internal_Doc_Annotation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Doc_Annotation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Doc_Annotation;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Doc_Annotation_Array_Access) return Internal_Doc_Annotation_Array_Access is
      Ret : Internal_Doc_Annotation_Array_Access := Create_Internal_Doc_Annotation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Doc_Annotation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Doc_Annotation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Doc_Annotation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Doc_Annotation_Array (Items_Count : Natural) return Internal_Doc_Annotation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Doc_Annotation_Array_Type
       else new Internal_Doc_Annotation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Doc_Annotation_Array
     (Items : Internal_Internal_Doc_Annotation_Array) return Internal_Doc_Annotation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Doc_Annotation_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Doc_Annotation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Doc_Annotation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Doc_Annotation_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Accept_Stmt_Vectors is new Langkit_Support.Vectors (Internal_Entity_Accept_Stmt);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Accept_Stmt_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Accept_Stmt
   is
      function Absolute_Get
        (T : Internal_Entity_Accept_Stmt_Array_Access; Index : Integer)
         return Internal_Entity_Accept_Stmt
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Accept_Stmt,
         Sequence_Type => Internal_Entity_Accept_Stmt_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Accept_Stmt;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Accept_Stmt;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Accept_Stmt_Array_Access) return Internal_Entity_Accept_Stmt_Array_Access is
      Ret : Internal_Entity_Accept_Stmt_Array_Access := Create_Internal_Entity_Accept_Stmt_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Accept_Stmt_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Accept_Stmt_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Accept_Stmt_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Accept_Stmt_Array (Items_Count : Natural) return Internal_Entity_Accept_Stmt_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Accept_Stmt_Array_Type
       else new Internal_Entity_Accept_Stmt_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Accept_Stmt_Array
     (Items : Internal_Internal_Entity_Accept_Stmt_Array) return Internal_Entity_Accept_Stmt_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Accept_Stmt_Array_Type;
      end if;
      return new Internal_Entity_Accept_Stmt_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Accept_Stmt_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Accept_Stmt_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Vectors is new Langkit_Support.Vectors (Internal_Entity);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity
   is
      function Absolute_Get
        (T : Internal_Entity_Array_Access; Index : Integer)
         return Internal_Entity
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity,
         Sequence_Type => Internal_Entity_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Array_Access) return Internal_Entity_Array_Access is
      Ret : Internal_Entity_Array_Access := Create_Internal_Entity_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Array (Items_Count : Natural) return Internal_Entity_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Array
     (Items : AST_Envs.Entity_Array) return Internal_Entity_Array_Access
   is (if Items'Length = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'
         (N         => Items'Length,
          Items     => Implementation.Internal_Internal_Entity_Array (Items),
          Ref_Count => 1));
   function Create_Internal_Entity_Array
     (Items : Internal_Internal_Entity_Array) return Internal_Entity_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Array_Type;
      end if;
      return new Internal_Entity_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Base_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Assoc
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Assoc_Array_Access; Index : Integer)
         return Internal_Entity_Base_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Assoc,
         Sequence_Type => Internal_Entity_Base_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Assoc_Array_Access) return Internal_Entity_Base_Assoc_Array_Access is
      Ret : Internal_Entity_Base_Assoc_Array_Access := Create_Internal_Entity_Base_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Assoc_Array (Items_Count : Natural) return Internal_Entity_Base_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Assoc_Array_Type
       else new Internal_Entity_Base_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Assoc_Array
     (Items : Internal_Internal_Entity_Base_Assoc_Array) return Internal_Entity_Base_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Assoc_Array_Type;
      end if;
      return new Internal_Entity_Base_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Formal_Param_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Formal_Param_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Formal_Param_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Base_Formal_Param_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Formal_Param_Decl,
         Sequence_Type => Internal_Entity_Base_Formal_Param_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Formal_Param_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Formal_Param_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Internal_Entity_Base_Formal_Param_Decl_Array_Access is
      Ret : Internal_Entity_Base_Formal_Param_Decl_Array_Access := Create_Internal_Entity_Base_Formal_Param_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Formal_Param_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Formal_Param_Decl_Array (Items_Count : Natural) return Internal_Entity_Base_Formal_Param_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Formal_Param_Decl_Array_Type
       else new Internal_Entity_Base_Formal_Param_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Formal_Param_Decl_Array
     (Items : Internal_Internal_Entity_Base_Formal_Param_Decl_Array) return Internal_Entity_Base_Formal_Param_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
      end if;
      return new Internal_Entity_Base_Formal_Param_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Type_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Base_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Base_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Type_Decl,
         Sequence_Type => Internal_Entity_Base_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Type_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Type_Decl_Array_Access) return Internal_Entity_Base_Type_Decl_Array_Access is
      Ret : Internal_Entity_Base_Type_Decl_Array_Access := Create_Internal_Entity_Base_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Type_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Base_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Type_Decl_Array_Type
       else new Internal_Entity_Base_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Type_Decl_Array
     (Items : Internal_Internal_Entity_Base_Type_Decl_Array) return Internal_Entity_Base_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Type_Decl_Array_Type;
      end if;
      return new Internal_Entity_Base_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Type_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Base_Type_Decl_Array_Access) return Internal_Entity_Base_Type_Decl_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Base_Type_Decl,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Base_Type_Decl renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Base_Type_Decl_Array_Access :=
               Create_Internal_Entity_Base_Type_Decl_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Base_Type_Decl renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      ----------
      -- Hash --
      ----------
      function Hash (R : Internal_Entity_Base_Type_Decl_Array_Access) return Hash_Type is
         Result : Hash_Type := Initial_Hash;
      begin
         for I in R.Items'Range loop
            Result := Combine (Result, Hash (R.Items (I)));
         end loop;
         return Result;
      end Hash;
      package Internal_Entity_Basic_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Basic_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Basic_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Basic_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Basic_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Basic_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Basic_Decl,
         Sequence_Type => Internal_Entity_Basic_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Basic_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Basic_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Basic_Decl_Array_Access) return Internal_Entity_Basic_Decl_Array_Access is
      Ret : Internal_Entity_Basic_Decl_Array_Access := Create_Internal_Entity_Basic_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Basic_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Basic_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Basic_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Basic_Decl_Array (Items_Count : Natural) return Internal_Entity_Basic_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Basic_Decl_Array_Type
       else new Internal_Entity_Basic_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Basic_Decl_Array
     (Items : Internal_Internal_Entity_Basic_Decl_Array) return Internal_Entity_Basic_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Basic_Decl_Array_Type;
      end if;
      return new Internal_Entity_Basic_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Basic_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Basic_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Basic_Decl_Array_Access) return Internal_Entity_Basic_Decl_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Basic_Decl,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Basic_Decl renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Basic_Decl_Array_Access :=
               Create_Internal_Entity_Basic_Decl_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Basic_Decl renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Internal_Entity_Compilation_Unit_Vectors is new Langkit_Support.Vectors (Internal_Entity_Compilation_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Compilation_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Compilation_Unit
   is
      function Absolute_Get
        (T : Internal_Entity_Compilation_Unit_Array_Access; Index : Integer)
         return Internal_Entity_Compilation_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Compilation_Unit,
         Sequence_Type => Internal_Entity_Compilation_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Compilation_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Compilation_Unit;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Compilation_Unit_Array_Access) return Internal_Entity_Compilation_Unit_Array_Access is
      Ret : Internal_Entity_Compilation_Unit_Array_Access := Create_Internal_Entity_Compilation_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Compilation_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Compilation_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Compilation_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Compilation_Unit_Array (Items_Count : Natural) return Internal_Entity_Compilation_Unit_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Compilation_Unit_Array_Type
       else new Internal_Entity_Compilation_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Compilation_Unit_Array
     (Items : Internal_Internal_Entity_Compilation_Unit_Array) return Internal_Entity_Compilation_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Compilation_Unit_Array_Type;
      end if;
      return new Internal_Entity_Compilation_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Compilation_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Compilation_Unit_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Compilation_Unit_Array_Access) return Internal_Entity_Compilation_Unit_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Compilation_Unit,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Compilation_Unit renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Compilation_Unit_Array_Access :=
               Create_Internal_Entity_Compilation_Unit_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Compilation_Unit renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Declarative_Part_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Declarative_Part
   is
      function Absolute_Get
        (T : Internal_Entity_Declarative_Part_Array_Access; Index : Integer)
         return Internal_Entity_Declarative_Part
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Declarative_Part,
         Sequence_Type => Internal_Entity_Declarative_Part_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Declarative_Part;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Declarative_Part;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Declarative_Part_Array_Access) return Internal_Entity_Declarative_Part_Array_Access is
      Ret : Internal_Entity_Declarative_Part_Array_Access := Create_Internal_Entity_Declarative_Part_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Declarative_Part_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Declarative_Part_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Declarative_Part_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Declarative_Part_Array (Items_Count : Natural) return Internal_Entity_Declarative_Part_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Declarative_Part_Array_Type
       else new Internal_Entity_Declarative_Part_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Declarative_Part_Array
     (Items : Internal_Internal_Entity_Declarative_Part_Array) return Internal_Entity_Declarative_Part_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Declarative_Part_Array_Type;
      end if;
      return new Internal_Entity_Declarative_Part_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Declarative_Part_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Declarative_Part_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Defining_Name_Vectors is new Langkit_Support.Vectors (Internal_Entity_Defining_Name);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Defining_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Defining_Name
   is
      function Absolute_Get
        (T : Internal_Entity_Defining_Name_Array_Access; Index : Integer)
         return Internal_Entity_Defining_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Defining_Name,
         Sequence_Type => Internal_Entity_Defining_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Defining_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Defining_Name;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Defining_Name_Array_Access) return Internal_Entity_Defining_Name_Array_Access is
      Ret : Internal_Entity_Defining_Name_Array_Access := Create_Internal_Entity_Defining_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Defining_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Defining_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Defining_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Defining_Name_Array (Items_Count : Natural) return Internal_Entity_Defining_Name_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Defining_Name_Array_Type
       else new Internal_Entity_Defining_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Defining_Name_Array
     (Items : Internal_Internal_Entity_Defining_Name_Array) return Internal_Entity_Defining_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Defining_Name_Array_Type;
      end if;
      return new Internal_Entity_Defining_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Defining_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Defining_Name_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Expr,
         Sequence_Type => Internal_Entity_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Expr;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Expr_Array_Access) return Internal_Entity_Expr_Array_Access is
      Ret : Internal_Entity_Expr_Array_Access := Create_Internal_Entity_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Expr_Array (Items_Count : Natural) return Internal_Entity_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Expr_Array_Type
       else new Internal_Entity_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Expr_Array
     (Items : Internal_Internal_Entity_Expr_Array) return Internal_Entity_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Expr_Array_Type;
      end if;
      return new Internal_Entity_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Expr_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Generic_Instantiation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Generic_Instantiation
   is
      function Absolute_Get
        (T : Internal_Entity_Generic_Instantiation_Array_Access; Index : Integer)
         return Internal_Entity_Generic_Instantiation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Generic_Instantiation,
         Sequence_Type => Internal_Entity_Generic_Instantiation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Generic_Instantiation;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Generic_Instantiation;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Generic_Instantiation_Array_Access) return Internal_Entity_Generic_Instantiation_Array_Access is
      Ret : Internal_Entity_Generic_Instantiation_Array_Access := Create_Internal_Entity_Generic_Instantiation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Generic_Instantiation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Generic_Instantiation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Generic_Instantiation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Generic_Instantiation_Array (Items_Count : Natural) return Internal_Entity_Generic_Instantiation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Generic_Instantiation_Array_Type
       else new Internal_Entity_Generic_Instantiation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Generic_Instantiation_Array
     (Items : Internal_Internal_Entity_Generic_Instantiation_Array) return Internal_Entity_Generic_Instantiation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Generic_Instantiation_Array_Type;
      end if;
      return new Internal_Entity_Generic_Instantiation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Generic_Instantiation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Generic_Instantiation_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Identifier_Vectors is new Langkit_Support.Vectors (Internal_Entity_Identifier);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Identifier_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Identifier
   is
      function Absolute_Get
        (T : Internal_Entity_Identifier_Array_Access; Index : Integer)
         return Internal_Entity_Identifier
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Identifier,
         Sequence_Type => Internal_Entity_Identifier_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Identifier;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Identifier;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Identifier_Array_Access) return Internal_Entity_Identifier_Array_Access is
      Ret : Internal_Entity_Identifier_Array_Access := Create_Internal_Entity_Identifier_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Identifier_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Identifier_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Identifier_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Identifier_Array (Items_Count : Natural) return Internal_Entity_Identifier_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Identifier_Array_Type
       else new Internal_Entity_Identifier_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Identifier_Array
     (Items : Internal_Internal_Entity_Identifier_Array) return Internal_Entity_Identifier_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Identifier_Array_Type;
      end if;
      return new Internal_Entity_Identifier_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Identifier_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Identifier_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Mode_Vectors is new Langkit_Support.Vectors (Internal_Entity_Mode);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Mode_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Mode
   is
      function Absolute_Get
        (T : Internal_Entity_Mode_Array_Access; Index : Integer)
         return Internal_Entity_Mode
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Mode,
         Sequence_Type => Internal_Entity_Mode_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Mode;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Mode;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Mode_Array_Access) return Internal_Entity_Mode_Array_Access is
      Ret : Internal_Entity_Mode_Array_Access := Create_Internal_Entity_Mode_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Mode_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Mode_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Mode_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Mode_Array (Items_Count : Natural) return Internal_Entity_Mode_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Mode_Array_Type
       else new Internal_Entity_Mode_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Mode_Array
     (Items : Internal_Internal_Entity_Mode_Array) return Internal_Entity_Mode_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Mode_Array_Type;
      end if;
      return new Internal_Entity_Mode_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Mode_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Mode_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Name_Vectors is new Langkit_Support.Vectors (Internal_Entity_Name);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Name
   is
      function Absolute_Get
        (T : Internal_Entity_Name_Array_Access; Index : Integer)
         return Internal_Entity_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Name,
         Sequence_Type => Internal_Entity_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Name;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Name_Array_Access) return Internal_Entity_Name_Array_Access is
      Ret : Internal_Entity_Name_Array_Access := Create_Internal_Entity_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Name_Array (Items_Count : Natural) return Internal_Entity_Name_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Name_Array_Type
       else new Internal_Entity_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Name_Array
     (Items : Internal_Internal_Entity_Name_Array) return Internal_Entity_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Name_Array_Type;
      end if;
      return new Internal_Entity_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Name_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Param_Spec_Vectors is new Langkit_Support.Vectors (Internal_Entity_Param_Spec);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Param_Spec_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Param_Spec
   is
      function Absolute_Get
        (T : Internal_Entity_Param_Spec_Array_Access; Index : Integer)
         return Internal_Entity_Param_Spec
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Param_Spec,
         Sequence_Type => Internal_Entity_Param_Spec_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Param_Spec;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Param_Spec;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Param_Spec_Array_Access) return Internal_Entity_Param_Spec_Array_Access is
      Ret : Internal_Entity_Param_Spec_Array_Access := Create_Internal_Entity_Param_Spec_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Param_Spec_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Param_Spec_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Param_Spec_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Param_Spec_Array (Items_Count : Natural) return Internal_Entity_Param_Spec_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Param_Spec_Array_Type
       else new Internal_Entity_Param_Spec_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Param_Spec_Array
     (Items : Internal_Internal_Entity_Param_Spec_Array) return Internal_Entity_Param_Spec_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Param_Spec_Array_Type;
      end if;
      return new Internal_Entity_Param_Spec_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Param_Spec_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Param_Spec_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Pragma_Node_Vectors is new Langkit_Support.Vectors (Internal_Entity_Pragma_Node);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Pragma_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Pragma_Node
   is
      function Absolute_Get
        (T : Internal_Entity_Pragma_Node_Array_Access; Index : Integer)
         return Internal_Entity_Pragma_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Pragma_Node,
         Sequence_Type => Internal_Entity_Pragma_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Pragma_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Pragma_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Pragma_Node_Array_Access) return Internal_Entity_Pragma_Node_Array_Access is
      Ret : Internal_Entity_Pragma_Node_Array_Access := Create_Internal_Entity_Pragma_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Pragma_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Pragma_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Pragma_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Pragma_Node_Array (Items_Count : Natural) return Internal_Entity_Pragma_Node_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Pragma_Node_Array_Type
       else new Internal_Entity_Pragma_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Pragma_Node_Array
     (Items : Internal_Internal_Entity_Pragma_Node_Array) return Internal_Entity_Pragma_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Pragma_Node_Array_Type;
      end if;
      return new Internal_Entity_Pragma_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Pragma_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Pragma_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Decl,
         Sequence_Type => Internal_Entity_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Type_Decl_Array_Access) return Internal_Entity_Type_Decl_Array_Access is
      Ret : Internal_Entity_Type_Decl_Array_Access := Create_Internal_Entity_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Type_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Decl_Array_Type
       else new Internal_Entity_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Type_Decl_Array
     (Items : Internal_Internal_Entity_Type_Decl_Array) return Internal_Entity_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Decl_Array_Type;
      end if;
      return new Internal_Entity_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Type_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Type_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Type_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Type_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Expr,
         Sequence_Type => Internal_Entity_Type_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Type_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Expr;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Type_Expr_Array_Access) return Internal_Entity_Type_Expr_Array_Access is
      Ret : Internal_Entity_Type_Expr_Array_Access := Create_Internal_Entity_Type_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Type_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Type_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Type_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Type_Expr_Array (Items_Count : Natural) return Internal_Entity_Type_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Expr_Array_Type
       else new Internal_Entity_Type_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Type_Expr_Array
     (Items : Internal_Internal_Entity_Type_Expr_Array) return Internal_Entity_Type_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Expr_Array_Type;
      end if;
      return new Internal_Entity_Type_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Type_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Type_Expr_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Env_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Env_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Env_Assoc,
         Sequence_Type => Internal_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Env_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Env_Assoc_Array_Access) return Internal_Env_Assoc_Array_Access is
      Ret : Internal_Env_Assoc_Array_Access := Create_Internal_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Env_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Env_Assoc_Array (Items_Count : Natural) return Internal_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Env_Assoc_Array_Type
       else new Internal_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Env_Assoc_Array
     (Items : Internal_Internal_Env_Assoc_Array) return Internal_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Env_Assoc_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Env_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Expected_Type_For_Expr_Vectors is new Langkit_Support.Vectors (Internal_Expected_Type_For_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Expected_Type_For_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Expected_Type_For_Expr
   is
      function Absolute_Get
        (T : Internal_Expected_Type_For_Expr_Array_Access; Index : Integer)
         return Internal_Expected_Type_For_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Expected_Type_For_Expr,
         Sequence_Type => Internal_Expected_Type_For_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Expected_Type_For_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Expected_Type_For_Expr;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Expected_Type_For_Expr_Array_Access) return Internal_Expected_Type_For_Expr_Array_Access is
      Ret : Internal_Expected_Type_For_Expr_Array_Access := Create_Internal_Expected_Type_For_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Expected_Type_For_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Expected_Type_For_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Expected_Type_For_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Expected_Type_For_Expr_Array (Items_Count : Natural) return Internal_Expected_Type_For_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Expected_Type_For_Expr_Array_Type
       else new Internal_Expected_Type_For_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Expected_Type_For_Expr_Array
     (Items : Internal_Internal_Expected_Type_For_Expr_Array) return Internal_Expected_Type_For_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Expected_Type_For_Expr_Array_Type;
      end if;
      return new Internal_Expected_Type_For_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Expected_Type_For_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Expected_Type_For_Expr_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Inner_Env_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Inner_Env_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Inner_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Inner_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Inner_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Inner_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Inner_Env_Assoc,
         Sequence_Type => Internal_Inner_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Inner_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Inner_Env_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Inner_Env_Assoc_Array_Access) return Internal_Inner_Env_Assoc_Array_Access is
      Ret : Internal_Inner_Env_Assoc_Array_Access := Create_Internal_Inner_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Inner_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Inner_Env_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Inner_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Inner_Env_Assoc_Array (Items_Count : Natural) return Internal_Inner_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Inner_Env_Assoc_Array_Type
       else new Internal_Inner_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Inner_Env_Assoc_Array
     (Items : Internal_Internal_Inner_Env_Assoc_Array) return Internal_Inner_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Inner_Env_Assoc_Array_Type;
      end if;
      return new Internal_Inner_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Inner_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Inner_Env_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Param_Actual_Vectors is new Langkit_Support.Vectors (Internal_Param_Actual);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Param_Actual_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Actual
   is
      function Absolute_Get
        (T : Internal_Param_Actual_Array_Access; Index : Integer)
         return Internal_Param_Actual
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Actual,
         Sequence_Type => Internal_Param_Actual_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Param_Actual;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Actual;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Param_Actual_Array_Access) return Internal_Param_Actual_Array_Access is
      Ret : Internal_Param_Actual_Array_Access := Create_Internal_Param_Actual_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Param_Actual_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Param_Actual_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Param_Actual_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Param_Actual_Array (Items_Count : Natural) return Internal_Param_Actual_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Actual_Array_Type
       else new Internal_Param_Actual_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Param_Actual_Array
     (Items : Internal_Internal_Param_Actual_Array) return Internal_Param_Actual_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Actual_Array_Type;
      end if;
      return new Internal_Param_Actual_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Param_Actual_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Param_Actual_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Param_Match_Vectors is new Langkit_Support.Vectors (Internal_Param_Match);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Param_Match_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Match
   is
      function Absolute_Get
        (T : Internal_Param_Match_Array_Access; Index : Integer)
         return Internal_Param_Match
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Match,
         Sequence_Type => Internal_Param_Match_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Param_Match;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Match;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Param_Match_Array_Access) return Internal_Param_Match_Array_Access is
      Ret : Internal_Param_Match_Array_Access := Create_Internal_Param_Match_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Param_Match_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Param_Match_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Param_Match_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Param_Match_Array (Items_Count : Natural) return Internal_Param_Match_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Match_Array_Type
       else new Internal_Param_Match_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Param_Match_Array
     (Items : Internal_Internal_Param_Match_Array) return Internal_Param_Match_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Match_Array_Type;
      end if;
      return new Internal_Param_Match_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Param_Match_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Param_Match_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Ref_Result_Vectors is new Langkit_Support.Vectors (Internal_Ref_Result);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Ref_Result_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Ref_Result
   is
      function Absolute_Get
        (T : Internal_Ref_Result_Array_Access; Index : Integer)
         return Internal_Ref_Result
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Ref_Result,
         Sequence_Type => Internal_Ref_Result_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Ref_Result;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Ref_Result;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Ref_Result_Array_Access) return Internal_Ref_Result_Array_Access is
      Ret : Internal_Ref_Result_Array_Access := Create_Internal_Ref_Result_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Ref_Result_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Ref_Result_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Ref_Result_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Ref_Result_Array (Items_Count : Natural) return Internal_Ref_Result_Array_Access
   is (if Items_Count = 0
       then No_Internal_Ref_Result_Array_Type
       else new Internal_Ref_Result_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Ref_Result_Array
     (Items : Internal_Internal_Ref_Result_Array) return Internal_Ref_Result_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Ref_Result_Array_Type;
      end if;
      return new Internal_Ref_Result_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Ref_Result_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Ref_Result_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Shape_Vectors is new Langkit_Support.Vectors (Internal_Shape);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Shape_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Shape
   is
      function Absolute_Get
        (T : Internal_Shape_Array_Access; Index : Integer)
         return Internal_Shape
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Shape,
         Sequence_Type => Internal_Shape_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Shape;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Shape;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Shape_Array_Access) return Internal_Shape_Array_Access is
      Ret : Internal_Shape_Array_Access := Create_Internal_Shape_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Shape_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Shape_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Shape_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Shape_Array (Items_Count : Natural) return Internal_Shape_Array_Access
   is (if Items_Count = 0
       then No_Internal_Shape_Array_Type
       else new Internal_Shape_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Shape_Array
     (Items : Internal_Internal_Shape_Array) return Internal_Shape_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Shape_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Shape_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Shape_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Shape_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Single_Actual_Vectors is new Langkit_Support.Vectors (Internal_Single_Actual);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Single_Actual_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Single_Actual
   is
      function Absolute_Get
        (T : Internal_Single_Actual_Array_Access; Index : Integer)
         return Internal_Single_Actual
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Single_Actual,
         Sequence_Type => Internal_Single_Actual_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Single_Actual;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Single_Actual;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Single_Actual_Array_Access) return Internal_Single_Actual_Array_Access is
      Ret : Internal_Single_Actual_Array_Access := Create_Internal_Single_Actual_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Single_Actual_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Single_Actual_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Single_Actual_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Single_Actual_Array (Items_Count : Natural) return Internal_Single_Actual_Array_Access
   is (if Items_Count = 0
       then No_Internal_Single_Actual_Array_Type
       else new Internal_Single_Actual_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Single_Actual_Array
     (Items : Internal_Internal_Single_Actual_Array) return Internal_Single_Actual_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Single_Actual_Array_Type;
      end if;
      return new Internal_Single_Actual_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Single_Actual_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Single_Actual_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Substitution_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Substitution
   is
      function Absolute_Get
        (T : Internal_Substitution_Array_Access; Index : Integer)
         return Internal_Substitution
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Substitution,
         Sequence_Type => Internal_Substitution_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Substitution;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Substitution;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Substitution_Array_Access) return Internal_Substitution_Array_Access is
      Ret : Internal_Substitution_Array_Access := Create_Internal_Substitution_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Substitution_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Substitution_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Substitution_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Substitution_Array (Items_Count : Natural) return Internal_Substitution_Array_Access
   is (if Items_Count = 0
       then No_Internal_Substitution_Array_Type
       else new Internal_Substitution_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Substitution_Array
     (Items : Internal_Internal_Substitution_Array) return Internal_Substitution_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Substitution_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Substitution_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Substitution_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Substitution_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Unit_Vectors is new Langkit_Support.Vectors (Internal_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Unit
   is
      function Absolute_Get
        (T : Internal_Unit_Array_Access; Index : Integer)
         return Internal_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Unit,
         Sequence_Type => Internal_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Unit_Array_Access) return Internal_Unit_Array_Access is
      Ret : Internal_Unit_Array_Access := Create_Internal_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Unit_Array (Items_Count : Natural) return Internal_Unit_Array_Access
   is (if Items_Count = 0
       then No_Internal_Unit_Array_Type
       else new Internal_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Unit_Array
     (Items : Internal_Internal_Unit_Array) return Internal_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Unit_Array_Type;
      end if;
      return new Internal_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Unit_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Unit_Array_Access) return Internal_Unit_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Unit,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Unit renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Unit_Array_Access :=
               Create_Internal_Unit_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Unit renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Lexical_Env_Vectors is new Langkit_Support.Vectors (Lexical_Env);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Lexical_Env_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Lexical_Env
   is
      function Absolute_Get
        (T : Lexical_Env_Array_Access; Index : Integer)
         return Lexical_Env
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Lexical_Env,
         Sequence_Type => Lexical_Env_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Lexical_Env;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Empty_Env;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Lexical_Env_Array_Access) return Lexical_Env_Array_Access is
      Ret : Lexical_Env_Array_Access := Create_Lexical_Env_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Lexical_Env_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Lexical_Env_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Lexical_Env_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Lexical_Env_Array (Items_Count : Natural) return Lexical_Env_Array_Access
   is (if Items_Count = 0
       then No_Lexical_Env_Array_Type
       else new Lexical_Env_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Lexical_Env_Array
     (Items : Internal_Lexical_Env_Array) return Lexical_Env_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Lexical_Env_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Lexical_Env_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Lexical_Env_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Lexical_Env_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Logic_Equation_Vectors is new Langkit_Support.Vectors (Logic_Equation);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Logic_Equation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Logic_Equation
   is
      function Absolute_Get
        (T : Logic_Equation_Array_Access; Index : Integer)
         return Logic_Equation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Logic_Equation,
         Sequence_Type => Logic_Equation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Logic_Equation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Null_Logic_Equation;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Logic_Equation_Array_Access) return Logic_Equation_Array_Access is
      Ret : Logic_Equation_Array_Access := Create_Logic_Equation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Logic_Equation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Logic_Equation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Logic_Equation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Logic_Equation_Array (Items_Count : Natural) return Logic_Equation_Array_Access
   is (if Items_Count = 0
       then No_Logic_Equation_Array_Type
       else new Logic_Equation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Logic_Equation_Array
     (Items : Internal_Logic_Equation_Array) return Logic_Equation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Logic_Equation_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Logic_Equation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Logic_Equation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Logic_Equation_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package String_Type_Vectors is new Langkit_Support.Vectors (String_Type);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : String_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return String_Type
   is
      function Absolute_Get
        (T : String_Type_Array_Access; Index : Integer)
         return String_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => String_Type,
         Sequence_Type => String_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : String_Type;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Empty_String;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : String_Type_Array_Access) return String_Type_Array_Access is
      Ret : String_Type_Array_Access := Create_String_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
      ------------------
      -- Join_Strings --
      ------------------
      function Join_Strings
        (Separator : String_Type;
         Strings   : String_Type_Array_Access) return String_Type
      is
         Separator_Length : constant Natural := Separator.Length;
         Length           : Natural := 0;
         First            : Boolean;
      begin
         --  First, compute the length of the result: the sum of all string
         --  lengths in Strings.
         First := True;
         for S of Strings.Items loop
            if First then
               First := False;
            else
               Length := Length + Separator_Length;
            end if;
            Length := Length + S.Length;
         end loop;
         --  Create the result string with the correct length. Do not use our
         --  constructor and initialize the content in-place, to avoid extra
         --  copies.
         return Result : constant String_Type :=
            new String_Record (Length)
         do
            Result.Ref_Count := 1;
            --  Now copy the content of all strings into the result
            declare
               Last : Natural := 0;
            begin
               First := True;
               for S of Strings.Items loop
                  if First then
                     First := False;
                  else
                     Result.Content (Last + 1 .. Last + Separator_Length) :=
                        Separator.Content;
                     Last := Last + Separator_Length;
                  end if;
                  Result.Content (Last + 1 .. Last + S.Length) := S.Content;
                  Last := Last + S.Length;
               end loop;
            end;
         end return;
      end Join_Strings;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : String_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : String_Type_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out String_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_String_Type_Array (Items_Count : Natural) return String_Type_Array_Access
   is (if Items_Count = 0
       then No_String_Type_Array_Type
       else new String_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_String_Type_Array
     (Items : Internal_String_Type_Array) return String_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_String_Type_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new String_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : String_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : String_Type_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Symbol_Type_Vectors is new Langkit_Support.Vectors (Symbol_Type);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Symbol_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Symbol_Type
   is
      function Absolute_Get
        (T : Symbol_Type_Array_Access; Index : Integer)
         return Symbol_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Symbol_Type,
         Sequence_Type => Symbol_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Symbol_Type;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Symbol_Type_Array_Access) return Symbol_Type_Array_Access is
      Ret : Symbol_Type_Array_Access := Create_Symbol_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Symbol_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Symbol_Type_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Symbol_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Symbol_Type_Array (Items_Count : Natural) return Symbol_Type_Array_Access
   is (if Items_Count = 0
       then No_Symbol_Type_Array_Type
       else new Symbol_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Symbol_Type_Array
     (Items : Internal_Symbol_Type_Array) return Symbol_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Symbol_Type_Array_Type;
      end if;
      return new Symbol_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Symbol_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Symbol_Type_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Bare_Ada_Node_Iterator_Access;
      Element : out Bare_Ada_Node) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Ada_Node_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Ada_Node_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Ada_Node_Iterator_Access) return String is
      begin
         return "<Iterator of AdaNode, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Completion_Item_Iterator_Access;
      Element : out Internal_Completion_Item) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Completion_Item_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Completion_Item_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Completion_Item_Iterator_Access) return String is
      begin
         return "<Iterator of CompletionItem, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Entity_Iterator_Access;
      Element : out Internal_Entity) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Iterator_Access) return String is
      begin
         return "<Iterator of AdaNode.entity, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Inner_Env_Assoc_Iterator_Access;
      Element : out Internal_Inner_Env_Assoc) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Inner_Env_Assoc_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Inner_Env_Assoc_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Inner_Env_Assoc_Iterator_Access) return String is
      begin
         return "<Iterator of InnerEnvAssoc, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Internal_Unit) return Boolean is
   begin
      return Left.Filename < Right.Filename;
   end "<";
function Hash (Key : Mmz_Key_Item) return Hash_Type;
function Equivalent (L, R : Mmz_Key_Item) return Boolean;
procedure Destroy (Key : in out Mmz_Key_Array_Access);
----------------
-- Equivalent --
----------------
function Equivalent (L, R : Mmz_Key_Item) return Boolean is
begin
   if L.Kind /= R.Kind then
      return False;
   end if;
   case L.Kind is
         when Mmz_Bare_Access_Def =>
               return L.As_Bare_Access_Def = R.As_Bare_Access_Def;
         when Mmz_Bare_Ada_Node =>
               return L.As_Bare_Ada_Node = R.As_Bare_Ada_Node;
         when Mmz_Bare_Anonymous_Expr_Decl =>
               return L.As_Bare_Anonymous_Expr_Decl = R.As_Bare_Anonymous_Expr_Decl;
         when Mmz_Bare_Array_Type_Def =>
               return L.As_Bare_Array_Type_Def = R.As_Bare_Array_Type_Def;
         when Mmz_Bare_Assoc_List =>
               return L.As_Bare_Assoc_List = R.As_Bare_Assoc_List;
         when Mmz_Bare_Base_Aggregate =>
               return L.As_Bare_Base_Aggregate = R.As_Bare_Base_Aggregate;
         when Mmz_Bare_Base_Id =>
               return L.As_Bare_Base_Id = R.As_Bare_Base_Id;
         when Mmz_Bare_Base_Subp_Body =>
               return L.As_Bare_Base_Subp_Body = R.As_Bare_Base_Subp_Body;
         when Mmz_Bare_Base_Subp_Spec =>
               return L.As_Bare_Base_Subp_Spec = R.As_Bare_Base_Subp_Spec;
         when Mmz_Bare_Base_Type_Decl =>
               return L.As_Bare_Base_Type_Decl = R.As_Bare_Base_Type_Decl;
         when Mmz_Bare_Basic_Decl =>
               return L.As_Bare_Basic_Decl = R.As_Bare_Basic_Decl;
         when Mmz_Bare_Body_Node =>
               return L.As_Bare_Body_Node = R.As_Bare_Body_Node;
         when Mmz_Bare_Classwide_Type_Decl =>
               return L.As_Bare_Classwide_Type_Decl = R.As_Bare_Classwide_Type_Decl;
         when Mmz_Bare_Compilation_Unit =>
               return L.As_Bare_Compilation_Unit = R.As_Bare_Compilation_Unit;
         when Mmz_Bare_Decimal_Fixed_Point_Def =>
               return L.As_Bare_Decimal_Fixed_Point_Def = R.As_Bare_Decimal_Fixed_Point_Def;
         when Mmz_Bare_Defining_Name =>
               return L.As_Bare_Defining_Name = R.As_Bare_Defining_Name;
         when Mmz_Bare_End_Name =>
               return L.As_Bare_End_Name = R.As_Bare_End_Name;
         when Mmz_Bare_Enum_Literal_Decl =>
               return L.As_Bare_Enum_Literal_Decl = R.As_Bare_Enum_Literal_Decl;
         when Mmz_Bare_Enum_Type_Def =>
               return L.As_Bare_Enum_Type_Def = R.As_Bare_Enum_Type_Def;
         when Mmz_Bare_For_Loop_Spec =>
               return L.As_Bare_For_Loop_Spec = R.As_Bare_For_Loop_Spec;
         when Mmz_Bare_For_Loop_Var_Decl =>
               return L.As_Bare_For_Loop_Var_Decl = R.As_Bare_For_Loop_Var_Decl;
         when Mmz_Bare_Formal_Discrete_Type_Def =>
               return L.As_Bare_Formal_Discrete_Type_Def = R.As_Bare_Formal_Discrete_Type_Def;
         when Mmz_Bare_Generic_Instantiation =>
               return L.As_Bare_Generic_Instantiation = R.As_Bare_Generic_Instantiation;
         when Mmz_Bare_Mod_Int_Type_Def =>
               return L.As_Bare_Mod_Int_Type_Def = R.As_Bare_Mod_Int_Type_Def;
         when Mmz_Bare_Name =>
               return L.As_Bare_Name = R.As_Bare_Name;
         when Mmz_Bare_Ordinary_Fixed_Point_Def =>
               return L.As_Bare_Ordinary_Fixed_Point_Def = R.As_Bare_Ordinary_Fixed_Point_Def;
         when Mmz_Bare_Private_Type_Def =>
               return L.As_Bare_Private_Type_Def = R.As_Bare_Private_Type_Def;
         when Mmz_Bare_Real_Type_Def =>
               return L.As_Bare_Real_Type_Def = R.As_Bare_Real_Type_Def;
         when Mmz_Bare_Record_Type_Def =>
               return L.As_Bare_Record_Type_Def = R.As_Bare_Record_Type_Def;
         when Mmz_Bare_Signed_Int_Type_Def =>
               return L.As_Bare_Signed_Int_Type_Def = R.As_Bare_Signed_Int_Type_Def;
         when Mmz_Bare_Subunit =>
               return L.As_Bare_Subunit = R.As_Bare_Subunit;
         when Mmz_Bare_Type_Decl =>
               return L.As_Bare_Type_Decl = R.As_Bare_Type_Decl;
         when Mmz_Bare_Type_Expr =>
               return L.As_Bare_Type_Expr = R.As_Bare_Type_Expr;
         when Mmz_Boolean =>
               return L.As_Boolean = R.As_Boolean;
         when Mmz_Env_Rebindings =>
               return L.As_Env_Rebindings = R.As_Env_Rebindings;
         when Mmz_Integer =>
               return L.As_Integer = R.As_Integer;
         when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
               return Equivalent (L.As_Internal_Entity_Base_Type_Decl_Array_Access, R.As_Internal_Entity_Base_Type_Decl_Array_Access);
         when Mmz_Internal_Entity_Info =>
               return L.As_Internal_Entity_Info = R.As_Internal_Entity_Info;
         when Mmz_Internal_Metadata =>
               return L.As_Internal_Metadata = R.As_Internal_Metadata;
         when Mmz_Lexical_Env =>
               return Equivalent (L.As_Lexical_Env, R.As_Lexical_Env);
         when Mmz_Symbol_Type =>
               return L.As_Symbol_Type = R.As_Symbol_Type;
   end case;
end Equivalent;
----------
-- Hash --
----------
function Hash (Key : Mmz_Key_Item) return Hash_Type is
begin
   case Key.Kind is
         when Mmz_Bare_Access_Def =>
            return Hash (Key.As_Bare_Access_Def);
         when Mmz_Bare_Ada_Node =>
            return Hash (Key.As_Bare_Ada_Node);
         when Mmz_Bare_Anonymous_Expr_Decl =>
            return Hash (Key.As_Bare_Anonymous_Expr_Decl);
         when Mmz_Bare_Array_Type_Def =>
            return Hash (Key.As_Bare_Array_Type_Def);
         when Mmz_Bare_Assoc_List =>
            return Hash (Key.As_Bare_Assoc_List);
         when Mmz_Bare_Base_Aggregate =>
            return Hash (Key.As_Bare_Base_Aggregate);
         when Mmz_Bare_Base_Id =>
            return Hash (Key.As_Bare_Base_Id);
         when Mmz_Bare_Base_Subp_Body =>
            return Hash (Key.As_Bare_Base_Subp_Body);
         when Mmz_Bare_Base_Subp_Spec =>
            return Hash (Key.As_Bare_Base_Subp_Spec);
         when Mmz_Bare_Base_Type_Decl =>
            return Hash (Key.As_Bare_Base_Type_Decl);
         when Mmz_Bare_Basic_Decl =>
            return Hash (Key.As_Bare_Basic_Decl);
         when Mmz_Bare_Body_Node =>
            return Hash (Key.As_Bare_Body_Node);
         when Mmz_Bare_Classwide_Type_Decl =>
            return Hash (Key.As_Bare_Classwide_Type_Decl);
         when Mmz_Bare_Compilation_Unit =>
            return Hash (Key.As_Bare_Compilation_Unit);
         when Mmz_Bare_Decimal_Fixed_Point_Def =>
            return Hash (Key.As_Bare_Decimal_Fixed_Point_Def);
         when Mmz_Bare_Defining_Name =>
            return Hash (Key.As_Bare_Defining_Name);
         when Mmz_Bare_End_Name =>
            return Hash (Key.As_Bare_End_Name);
         when Mmz_Bare_Enum_Literal_Decl =>
            return Hash (Key.As_Bare_Enum_Literal_Decl);
         when Mmz_Bare_Enum_Type_Def =>
            return Hash (Key.As_Bare_Enum_Type_Def);
         when Mmz_Bare_For_Loop_Spec =>
            return Hash (Key.As_Bare_For_Loop_Spec);
         when Mmz_Bare_For_Loop_Var_Decl =>
            return Hash (Key.As_Bare_For_Loop_Var_Decl);
         when Mmz_Bare_Formal_Discrete_Type_Def =>
            return Hash (Key.As_Bare_Formal_Discrete_Type_Def);
         when Mmz_Bare_Generic_Instantiation =>
            return Hash (Key.As_Bare_Generic_Instantiation);
         when Mmz_Bare_Mod_Int_Type_Def =>
            return Hash (Key.As_Bare_Mod_Int_Type_Def);
         when Mmz_Bare_Name =>
            return Hash (Key.As_Bare_Name);
         when Mmz_Bare_Ordinary_Fixed_Point_Def =>
            return Hash (Key.As_Bare_Ordinary_Fixed_Point_Def);
         when Mmz_Bare_Private_Type_Def =>
            return Hash (Key.As_Bare_Private_Type_Def);
         when Mmz_Bare_Real_Type_Def =>
            return Hash (Key.As_Bare_Real_Type_Def);
         when Mmz_Bare_Record_Type_Def =>
            return Hash (Key.As_Bare_Record_Type_Def);
         when Mmz_Bare_Signed_Int_Type_Def =>
            return Hash (Key.As_Bare_Signed_Int_Type_Def);
         when Mmz_Bare_Subunit =>
            return Hash (Key.As_Bare_Subunit);
         when Mmz_Bare_Type_Decl =>
            return Hash (Key.As_Bare_Type_Decl);
         when Mmz_Bare_Type_Expr =>
            return Hash (Key.As_Bare_Type_Expr);
         when Mmz_Boolean =>
            return Hash (Key.As_Boolean);
         when Mmz_Env_Rebindings =>
            return Hash (Key.As_Env_Rebindings);
         when Mmz_Integer =>
            return Hash (Key.As_Integer);
         when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
            return Hash (Key.As_Internal_Entity_Base_Type_Decl_Array_Access);
         when Mmz_Internal_Entity_Info =>
            return Hash (Key.As_Internal_Entity_Info);
         when Mmz_Internal_Metadata =>
            return Hash (Key.As_Internal_Metadata);
         when Mmz_Lexical_Env =>
            return Hash (Key.As_Lexical_Env);
         when Mmz_Symbol_Type =>
            return Hash (Key.As_Symbol_Type);
   end case;
end Hash;
----------
-- Hash --
----------
function Hash (Key : Mmz_Key) return Hash_Type is
   Result : Hash_Type := Mmz_Property'Pos (Key.Property);
begin
   for K of Key.Items.all loop
      Result := Combine (Result, Hash (K));
   end loop;
   return Result;
end Hash;
----------------
-- Equivalent --
----------------
function Equivalent (L, R : Mmz_Key) return Boolean is
   L_Items : Mmz_Key_Array renames L.Items.all;
   R_Items : Mmz_Key_Array renames R.Items.all;
begin
   if L.Property /= R.Property or else L_Items'Length /= R_Items'Length then
      return False;
   end if;
   for I in L_Items'Range loop
      if not Equivalent (L_Items (I), R_Items (I)) then
         return False;
      end if;
   end loop;
   return True;
end Equivalent;
-------------
-- Destroy --
-------------
procedure Destroy (Map : in out Memoization_Maps.Map) is
   use Memoization_Maps;
   --  We need keys and values to be valid when clearing the memoization map,
   --  but on the other hand we need to free keys and values as well. To
   --  achieve both goals, we first copy key and values into arrays, then we
   --  clear the map, and then we free keys/values in the arrays. Allocate both
   --  arrays on the heap to avoid stack overflow, as they can be quite big.
   Length : constant Natural := Natural (Map.Length);
   type Key_Array is array (1 .. Length) of Mmz_Key_Array_Access;
   type Key_Array_Access is access Key_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Key_Array, Key_Array_Access);
   type Value_Array is array (1 .. Length) of Mmz_Value;
   type Value_Array_Access is access Value_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Value_Array, Value_Array_Access);
   Keys   : Key_Array_Access := new Key_Array;
   Values : Value_Array_Access := new Value_Array;
   I      : Positive := 1;
begin
   for Cur in Map.Iterate loop
      Keys (I) := Key (Cur).Items;
      Values (I) := Element (Cur);
      I := I + 1;
   end loop;
   Map.Clear;
   for K_Array of Keys.all loop
      Destroy (K_Array);
   end loop;
   for V of Values.all loop
      case V.Kind is
         when Mmz_Error =>
            Free_Memoized_Error (V.Exc_Id, V.Exc_Msg);
            when Mmz_Internal_Entity_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Array_Access);
            when Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access);
            when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Base_Type_Decl_Array_Access);
            when Mmz_Internal_Entity_Basic_Decl_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Basic_Decl_Array_Access);
            when Mmz_Internal_Entity_Compilation_Unit_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Compilation_Unit_Array_Access);
            when Mmz_Internal_Entity_Expr_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Expr_Array_Access);
            when Mmz_Internal_Env_Assoc =>
               Dec_Ref (V.As_Internal_Env_Assoc);
            when Mmz_Internal_Env_Assoc_Array_Access =>
               Dec_Ref (V.As_Internal_Env_Assoc_Array_Access);
            when Mmz_Internal_Inner_Env_Assoc_Array_Access =>
               Dec_Ref (V.As_Internal_Inner_Env_Assoc_Array_Access);
            when Mmz_Internal_Param_Match_Array_Access =>
               Dec_Ref (V.As_Internal_Param_Match_Array_Access);
            when Mmz_Internal_Single_Actual_Array_Access =>
               Dec_Ref (V.As_Internal_Single_Actual_Array_Access);
            when Mmz_Internal_Unit_Array_Access =>
               Dec_Ref (V.As_Internal_Unit_Array_Access);
            when Mmz_Lexical_Env =>
               Dec_Ref (V.As_Lexical_Env);
         when others => null;
      end case;
   end loop;
   Free (Keys);
   Free (Values);
end Destroy;
-------------
-- Destroy --
-------------
procedure Destroy (Key : in out Mmz_Key_Array_Access) is
   procedure Free is new Ada.Unchecked_Deallocation
     (Mmz_Key_Array, Mmz_Key_Array_Access);
begin
      for K of Key.all loop
         case K.Kind is
               when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
                  Dec_Ref (K.As_Internal_Entity_Base_Type_Decl_Array_Access);
               when Mmz_Lexical_Env =>
                  Dec_Ref (K.As_Lexical_Env);
            when others => null;
         end case;
      end loop;
   Free (Key);
end Destroy;
-------------------------
-- Find_Memoized_Value --
-------------------------
function Find_Memoized_Value
  (Unit       : Internal_Unit;
   Handle     : out Memoization_Handle;
   Value      : out Mmz_Value;
   Create_Key : access function return Mmz_Key) return Boolean
is
   Inserted : Boolean;
begin
   --  Make sure that we don't lookup stale caches
   Reset_Caches (Unit);
   --  Initialize handle: create the key and create a cursor pointing to an
   --  existing entry.
   Handle.Key := Create_Key.all;
   Handle.Cache_Version := Unit.Cache_Version;
   Value := (Kind => Mmz_Evaluating);
   Unit.Memoization_Map.Insert (Handle.Key, Value, Handle.Cur, Inserted);
   --  No existing entry yet? The above just created one. Otherwise, destroy
   --  our key and reuse the existing entry's.
   if not Inserted then
      Destroy (Handle.Key.Items);
      Handle.Key := Memoization_Maps.Key (Handle.Cur);
      Value := Memoization_Maps.Element (Handle.Cur);
   end if;
   return not Inserted;
end Find_Memoized_Value;
------------------------
-- Add_Memoized_Value --
------------------------
procedure Add_Memoized_Value
  (Unit   : Internal_Unit;
   Handle : in out Memoization_Handle;
   Value  : Mmz_Value;
   Stored : out Boolean) is
begin
   --  If Handle was created using a memoization map that has been since then
   --  reset, do nothing: the result can be partly stale due to the event that
   --  triggered the memoization tables reset.
   Stored := Unit.Cache_Version <= Handle.Cache_Version;
   if Stored then
      Unit.Memoization_Map.Replace_Element (Handle.Cur, Value);
   end if;
end Add_Memoized_Value;
------------------------
-- Add_Memoized_Error --
------------------------
procedure Add_Memoized_Error
  (Unit   : Internal_Unit;
   Handle : in out Memoization_Handle;
   Exc    : Ada.Exceptions.Exception_Occurrence;
   Stored : out Boolean)
is
   Value : Mmz_Value (Kind => Mmz_Error);
begin
   Store_Memoized_Error (Exc, Value.Exc_Id, Value.Exc_Msg);
   Add_Memoized_Value (Unit, Handle, Value, Stored);
   if not Stored then
      Free_Memoized_Error (Value.Exc_Id, Value.Exc_Msg);
   end if;
end Add_Memoized_Error;
----------------------------
-- Reraise_Memoized_Error --
----------------------------
procedure Reraise_Memoized_Error (Value : Mmz_Value) is
begin
   Reraise_Memoized_Error (Value.Exc_Id, Value.Exc_Msg);
end Reraise_Memoized_Error;
--------------------------
-- Store_Memoized_Error --
--------------------------
procedure Store_Memoized_Error
  (Exc     : Ada.Exceptions.Exception_Occurrence;
   Exc_Id  : out Ada.Exceptions.Exception_Id;
   Exc_Msg : out String_Access) is
begin
   Exc_Id := Ada.Exceptions.Exception_Identity (Exc);
   Exc_Msg := new String'(Ada.Exceptions.Exception_Message (Exc));
end Store_Memoized_Error;
-------------------------
-- Free_Memoized_Error --
-------------------------
procedure Free_Memoized_Error
  (Exc_Id  : in out Ada.Exceptions.Exception_Id;
   Exc_Msg : in out String_Access)
is
   pragma Unreferenced (Exc_Id);
begin
   Free (Exc_Msg);
end Free_Memoized_Error;
----------------------------
-- Reraise_Memoized_Error --
----------------------------
procedure Reraise_Memoized_Error
  (Exc_Id  : Ada.Exceptions.Exception_Id;
   Exc_Msg : String_Access) is
begin
   Ada.Exceptions.Raise_Exception (Exc_Id, Exc_Msg.all & " (memoized)");
end Reraise_Memoized_Error;
   -------------------
   -- Solve_Wrapper --
   -------------------
   function Solve_Wrapper
     (R            : Solver.Relation;
      Context_Node : Bare_Ada_Node) return Boolean is
   begin
      if Context_Node /= null and then Langkit_Support.Adalog.Debug.Debug then
         Assign_Names_To_Logic_Vars (Context_Node);
      end if;
      begin
         return Solver.Solve_First
           (R, Timeout => Context_Node.Unit.Context.Logic_Resolution_Timeout);
      exception
         when Langkit_Support.Adalog.Early_Binding_Error =>
            Raise_Property_Exception
              (Context_Node,
               Property_Error'Identity,
               "invalid equation for logic resolution");
         when Langkit_Support.Adalog.Timeout_Error =>
            Raise_Property_Exception
              (Context_Node,
               Property_Error'Identity,
               "logic resolution timed out");
      end;
   end Solve_Wrapper;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Env : in out Lexical_Env_Access) is
      Mutable_Env : Lexical_Env :=
        (Wrap (Env), 0, Env.Kind, No_Generic_Unit, 0);
   begin
      Destroy (Mutable_Env);
      Env := null;
   end Destroy;
   ----------------
   -- Initialize --
   ----------------
   procedure Initialize
     (Self              : Bare_Ada_Node;
      Kind              : Ada_Node_Kind_Type;
      Unit              : Internal_Unit;
      Token_Start_Index : Token_Index;
      Token_End_Index   : Token_Index;
      Parent            : Bare_Ada_Node := null;
      Self_Env          : Lexical_Env := AST_Envs.Empty_Env) is
   begin
      pragma Unreferenced (Kind);
      Self.Parent := Parent;
      Self.Unit := Unit;
      Self.Token_Start_Index := Token_Start_Index;
      Self.Token_End_Index := Token_End_Index;
      Self.Self_Env := Self_Env;
      Self.Last_Attempted_Child := -1;
   end Initialize;
   --------------------
   -- Use_Direct_Env --
   --------------------
   procedure Use_Direct_Env (State : in out PLE_Node_State; Env : Lexical_Env)
   is
   begin
      State.Current_Env := Env;
      State.Current_NED := null;
   end Use_Direct_Env;
   -------------------
   -- Use_Named_Env --
   -------------------
   procedure Use_Named_Env
     (State   : in out PLE_Node_State;
      Context : Internal_Context;
      Name    : Symbol_Type) is
   begin
      State.Current_NED := Get_Named_Env_Descriptor (Context, Name);
      State.Current_Env := State.Current_NED.Env_With_Precedence;
   end Use_Named_Env;
   ---------------------
   -- Set_Initial_Env --
   ---------------------
   procedure Set_Initial_Env
     (Self         : Bare_Ada_Node;
      State        : in out PLE_Node_State;
      Env          : Internal_Designated_Env;
      DSL_Location : String) is
   begin
      case Env.Kind is
         when None =>
            Use_Direct_Env (State, Empty_Env);
         when Current_Env =>
            null;
         when Named_Env =>
            Use_Named_Env (State, Self.Unit.Context, Env.Env_Name);
         when Direct_Env =>
            --  Sanitize this environment value: make sure it's a non-foreign
            --  and primary environment.
            if Env.Direct_Env.Kind /= Static_Primary then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "Cannot set an env that is not static-primary as the"
                  & " initial env");
            elsif Is_Foreign_Strict (Env.Direct_Env, Self) then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "unsound foreign environment in SetInitialEnv ("
                  & DSL_Location & ")");
            end if;
            Use_Direct_Env (State, Env.Direct_Env);
      end case;
   end Set_Initial_Env;
   ----------------
   -- Add_To_Env --
   ----------------
   procedure Add_To_Env
     (Self         : Bare_Ada_Node;
      State        : PLE_Node_State;
      Key          : Symbol_Type;
      Value        : Bare_Ada_Node;
      Md           : Internal_Metadata;
      Resolver     : Entity_Resolver;
      Dest_Env     : Internal_Designated_Env;
      DSL_Location : String)
   is
      Context    : constant Internal_Context := Self.Unit.Context;
      Root_Scope : Lexical_Env renames Context.Root_Scope;
      --  Shortcuts
      Actual_Dest_Env : Lexical_Env;
      Dest_NED        : Named_Env_Descriptor_Access;
      --  Description for the destination environment
   begin
      --  Skip the env addition if explicitly requested
      if Key = null
         or else Value = null
         or else (case Dest_Env.Kind is
                  when None        => True,
                  when Current_Env => False,
                  when Named_Env   => Dest_Env.Env_Name = null,
                  when Direct_Env  => Dest_Env.Direct_Env = Empty_Env)
      then
         return;
      end if;
      if Value.Unit /= Self.Unit then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add_to_env an AST node that comes from another analysis"
            & " unit");
      end if;
         --  Make sure metadata does not contain any foreign node
         if (Md.Primitive /= null and then Md.Primitive.Unit /= Self.Unit) or else (Md.Primitive_Real_Type /= null and then Md.Primitive_Real_Type.Unit /= Self.Unit)
         then
            Raise_Property_Exception
              (Self,
               Property_Error'Identity,
               "Cannot add metadata that contains foreign nodes");
         end if;
      --  Then determine the destination environment
      case Dest_Env.Kind is
         when None =>
            raise Program_Error with "unreachable code";
         when Current_Env =>
            --  Just use the current environment
            Dest_NED := State.Current_NED;
            Actual_Dest_Env := State.Current_Env;
         when Named_Env =>
            --  There is an environment name: just lookup the corresponding
            --  NED/env.
            Dest_NED := Get_Named_Env_Descriptor (Context, Dest_Env.Env_Name);
            Actual_Dest_Env := Dest_NED.Env_With_Precedence;
         when Direct_Env =>
            --  There is an explicit destination environment
            Dest_NED := null;
            Actual_Dest_Env := Dest_Env.Direct_Env;
      end case;
      --  Sanitize it
      if Actual_Dest_Env.Kind /= Static_Primary then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add elements to a lexical env that is not static-primary");
      elsif
         --  Since lexical envs need to sort the foreign nodes they contain,
         --  and that the total order on nodes is not defined for synthetic
         --  nodes, it is not possible to add a synthetic node to a foreign
         --  lexical environment.
         --
         --  This reasoning applies to environments that belong to foreign
         --  units, but also to the root environment.
         Is_Foreign (Actual_Dest_Env, Self) and then Is_Synthetic (Value)
      then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add a synthetic node to a lexical env from another"
            & " analysis unit");
      elsif
         --  Reject direct references to foreign destination environments.
         --
         --  This is an attempt at identifying uses of the unsound relocation
         --  mechanism (as opposed to named environments), so this applies to
         --  all foreign environments (root scope included).
         DSL_Location'Length > 0
         and then Dest_Env.Kind = Direct_Env
         and then Is_Foreign_Strict (Actual_Dest_Env, Self)
      then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "unsound foreign environment in AddToEnv (" & DSL_Location & ")");
      end if;
      --  Now that everything is sanitized, we can proceed with the actual
      --  key/value pair addition. Note that this does nothing if
      --  Actual_Dest_Env ended up empty.
      Add (Actual_Dest_Env, Key, Value, Md, Resolver);
      --  If we're adding the element to an environment by env name, we must
      --  register this association in two places: in the target named env
      --  entry, and in Value's unit.
      if Dest_NED /= null then
         declare
            use NED_Assoc_Maps;
            FN    : Map renames Dest_NED.Foreign_Nodes;
            Dummy : Boolean;
            Cur   : Cursor;
         begin
            FN.Insert (Key      => Key,
                       New_Item => Internal_Map_Node_Vectors.Empty_Vector,
                       Position => Cur,
                       Inserted => Dummy);
            declare
               V : Internal_Map_Node_Vectors.Vector renames
                  FN.Reference (Cur);
            begin
               V.Append ((Value, Md, Resolver));
            end;
         end;
         Value.Unit.Exiled_Entries_In_NED.Append ((Dest_NED, Key, Value));
      --  Otherwise, if we're adding the element to an environment that belongs
      --  to a different unit, or to the root scope, then...
      elsif Is_Foreign_Not_Empty (Actual_Dest_Env, Self) then
         --  Add the Key/Value association to the list of entries contained in
         --  other units, so we can remove them when reparsing Value's unit.
         Value.Unit.Exiled_Entries.Append ((Actual_Dest_Env, Key, Value));
         if Actual_Dest_Env /= Root_Scope then
            --  Add Val to the list of foreign nodes that Actual_Dest_Env's
            --  unit contains, so that when that unit is reparsed, we can call
            --  Add_To_Env again on those nodes.
            Convert_Unit (Actual_Dest_Env.Owner).Foreign_Nodes.Append
              ((Value, Self.Unit));
         end if;
      end if;
   end Add_To_Env;
   -------------
   -- Ref_Env --
   -------------
   procedure Ref_Env
     (Self                : Bare_Ada_Node;
      Dest_Env            : Lexical_Env;
      Ref_Env_Nodes       : in out Bare_Ada_Node_Array_Access;
      Resolver            : Lexical_Env_Resolver;
      Kind                : Ref_Kind;
      Cats                : Ref_Categories;
      Shed_Rebindings     : Boolean) is
   begin
      for N of Ref_Env_Nodes.Items loop
         if N /= null then
            if N.Unit /= Self.Unit then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "attempt to add a referenced environment to a foreign unit");
            end if;
            Reference (Dest_Env, N, Resolver, Kind, Cats, Shed_Rebindings);
         end if;
      end loop;
      Dec_Ref (Ref_Env_Nodes);
   end Ref_Env;
   -------------
   -- Add_Env --
   -------------
   procedure Add_Env
     (Self              : Bare_Ada_Node;
      State             : in out PLE_Node_State;
      No_Parent         : Boolean;
      Transitive_Parent : Boolean;
      Names             : in out Symbol_Type_Array_Access)
   is
      Parent_From_Name : constant Boolean := State.Current_NED /= null;
      --  Does the parent environment comes from a named environment lookup?
      --  Determine the parent of this new environment:
      --
      --  (1) no parent if requested;
      --  (2) the current environment as the static parent if it comes from a
      --      named env lookup or if it is not foreign (or is the empty/root
      --      environment).
      Parent : constant Lexical_Env :=
        (if No_Parent
         then Null_Lexical_Env
         else State.Current_Env);
   begin
      --  Create the environment itself
      Self.Self_Env := Create_Static_Lexical_Env
        (Parent            => Parent,
         Node              => Self,
         Transitive_Parent => Transitive_Parent);
      --  If the parent of this new environment comes from a named environment
      --  lookup, register this new environment so that its parent is updated
      --  when the precence for this named environment changes.
      if Parent_From_Name then
         declare
            NED : constant Named_Env_Descriptor_Access := State.Current_NED;
         begin
            Self.Unit.Exiled_Envs.Append ((NED, Self.Self_Env));
            NED.Foreign_Envs.Insert (Self, Self.Self_Env);
         end;
      end if;
      --  From now on, the current environment is Self.Self_Env, with a direct
      --  access to it. It does not go through the env naming scheme, since
      --  only this node and its children (i.e. non-foreign nodes) will access
      --  it as a "current" environment during PLE.
      Use_Direct_Env (State, Self.Self_Env);
      --  Register the environment we just created on all the requested names
      if Names /= null then
         declare
            Context   : constant Internal_Context := Self.Unit.Context;
            Env       : constant Lexical_Env := Self.Self_Env;
            NENU      : NED_Maps.Map renames
               State.Unit_State.Named_Envs_Needing_Update;
         begin
            for N of Names.Items loop
               Register_Named_Env (Context, N, Env, NENU);
            end loop;
            Dec_Ref (Names);
         end;
      end if;
   end Add_Env;
   ---------------------
   -- Pre_Env_Actions --
   ---------------------
   procedure Pre_Env_Actions
     (Self            : Bare_Ada_Node;
      State           : in out PLE_Node_State;
      Add_To_Env_Only : Boolean := False) is
   begin
   case Self.Kind is
            when Ada_With_Clause =>
            With_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Use_Type_Clause =>
            Use_Type_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Use_Package_Clause =>
            Use_Package_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Access_To_Subp_Def =>
            Access_To_Subp_Def_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Named_Stmt =>
            Named_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Extended_Return_Stmt =>
            Extended_Return_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Begin_Block .. Ada_Decl_Block =>
            Block_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_For_Loop_Stmt =>
            For_Loop_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
            Accept_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Pragma_Node =>
            Pragma_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Decl_Expr =>
            Decl_Expr_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Private_Part =>
            Private_Part_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Task_Decl =>
            Single_Task_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Protected_Decl =>
            Single_Protected_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Renaming_Decl =>
            Package_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
            Object_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Number_Decl =>
            Number_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Label_Decl =>
            Label_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Renaming_Decl =>
            Generic_Subp_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Renaming_Decl =>
            Generic_Package_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
            Generic_Instantiation_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Decl =>
            Generic_Subp_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Decl =>
            Generic_Package_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_For_Loop_Var_Decl =>
            For_Loop_Var_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Exception_Handler =>
            Exception_Handler_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Exception_Decl =>
            Exception_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Index_Spec =>
            Entry_Index_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Body =>
            Task_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Body =>
            Protected_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Body =>
            Package_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Body =>
            Entry_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Body_Stub =>
            Task_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Subp_Body_Stub =>
            Subp_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Body_Stub =>
            Protected_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Body_Stub =>
            Package_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
            Base_Subp_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Internal =>
            Generic_Subp_Internal_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
            Enum_Literal_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Decl =>
            Entry_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl | Ada_Synthetic_Subp_Decl =>
            Basic_Subp_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
            null;
            when Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
            Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Task_Type_Decl =>
            Single_Task_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Type_Decl =>
            Task_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Type_Decl =>
            Protected_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
            Incomplete_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Classwide_Type_Decl =>
            null;
            when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
            Base_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Decl =>
            Package_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Internal =>
            Generic_Package_Internal_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Param_Spec =>
            Param_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Discriminant_Spec =>
            Discriminant_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Component_Decl =>
            Component_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Abstract_State_Decl =>
            Abstract_State_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      when others =>  null; 
   end case;
   end Pre_Env_Actions;
   ----------------------
   -- Post_Env_Actions --
   ----------------------
   procedure Post_Env_Actions
     (Self : Bare_Ada_Node; State : in out PLE_Node_State) is
   begin
   case Self.Kind is
            when Ada_With_Clause =>
            null;
            when Ada_Use_Type_Clause =>
            Use_Type_Clause_Post_Env_Actions (Self, State);
            when Ada_Use_Package_Clause =>
            Use_Package_Clause_Post_Env_Actions (Self, State);
            when Ada_Access_To_Subp_Def =>
            null;
            when Ada_Named_Stmt =>
            null;
            when Ada_Extended_Return_Stmt =>
            null;
            when Ada_Begin_Block .. Ada_Decl_Block =>
            null;
            when Ada_For_Loop_Stmt =>
            null;
            when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
            null;
            when Ada_Pragma_Node =>
            null;
            when Ada_Decl_Expr =>
            null;
            when Ada_Private_Part =>
            null;
            when Ada_Single_Task_Decl =>
            null;
            when Ada_Single_Protected_Decl =>
            null;
            when Ada_Package_Renaming_Decl =>
            null;
            when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
            null;
            when Ada_Number_Decl =>
            null;
            when Ada_Label_Decl =>
            null;
            when Ada_Generic_Subp_Renaming_Decl =>
            null;
            when Ada_Generic_Package_Renaming_Decl =>
            null;
            when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
            null;
            when Ada_Generic_Subp_Decl =>
            Generic_Subp_Decl_Post_Env_Actions (Self, State);
            when Ada_Generic_Package_Decl =>
            null;
            when Ada_For_Loop_Var_Decl =>
            null;
            when Ada_Exception_Handler =>
            null;
            when Ada_Exception_Decl =>
            null;
            when Ada_Entry_Index_Spec =>
            null;
            when Ada_Task_Body =>
            null;
            when Ada_Protected_Body =>
            null;
            when Ada_Package_Body =>
            null;
            when Ada_Entry_Body =>
            null;
            when Ada_Task_Body_Stub =>
            null;
            when Ada_Subp_Body_Stub =>
            null;
            when Ada_Protected_Body_Stub =>
            null;
            when Ada_Package_Body_Stub =>
            null;
            when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
            null;
            when Ada_Generic_Subp_Internal =>
            null;
            when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
            null;
            when Ada_Entry_Decl =>
            null;
            when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl | Ada_Synthetic_Subp_Decl =>
            Basic_Subp_Decl_Post_Env_Actions (Self, State);
            when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
            null;
            when Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
            Type_Decl_Post_Env_Actions (Self, State);
            when Ada_Single_Task_Type_Decl =>
            null;
            when Ada_Task_Type_Decl =>
            null;
            when Ada_Protected_Type_Decl =>
            null;
            when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
            null;
            when Ada_Classwide_Type_Decl =>
            null;
            when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
            null;
            when Ada_Package_Decl =>
            null;
            when Ada_Generic_Package_Internal =>
            null;
            when Ada_Param_Spec =>
            null;
            when Ada_Discriminant_Spec =>
            null;
            when Ada_Component_Decl =>
            null;
            when Ada_Abstract_State_Decl =>
            null;
      when others =>  null; 
   end case;
   end Post_Env_Actions;
   ----------------
   -- Get_Symbol --
   ----------------
   function Get_Symbol
     (Node : Bare_Ada_Node) return Symbol_Type is
   begin
      if Node = null then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "cannot get the symbol of a null node");
      end if;
      return Get_Symbol (Token (Node, Node.Token_Start_Index));
   end Get_Symbol;
   ----------
   -- Text --
   ----------
   function Text
     (Node : Bare_Ada_Node) return Text_Type
   is
   begin
      if Node = null then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "cannot get the text of a null node");
      end if;
      declare
         Start_T : constant Token_Reference :=
            Token (Node, Node.Token_Start_Index);
         End_T   : constant Token_Reference :=
            Token (Node, Node.Token_End_Index);
      begin
         --  No text is associated to synthetic and ghost nodes
         if Is_Synthetic (Node) then
            return "";
         end if;
         if Is_Ghost (Node) then
            return "";
         end if;
         return Text (Start_T, End_T);
      end;
   end Text;
   ---------------------
   -- Is_Visible_From --
   ---------------------
   function Is_Visible_From
     (Self                     : Bare_Ada_Node;
      Referenced_Env, Base_Env : Lexical_Env) return Boolean
   is
      Referenced_Unit : constant Internal_Unit :=
         Convert_Unit (Referenced_Env.Owner);
      Base_Unit       : constant Internal_Unit :=
         Convert_Unit (Base_Env.Owner);
   begin
      if Referenced_Unit = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "referenced environment does not belong to any analysis unit");
      elsif Base_Unit = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "base environment does not belong to any analysis unit");
      end if;
      return Is_Referenced_From (Referenced_Unit, Base_Unit);
   end Is_Visible_From;
   ----------
   -- Unit --
   ----------
   function Unit (Node : Bare_Ada_Node) return Internal_Unit is
   begin
      return Node.Unit;
   end Unit;
   function Lookup_Internal
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node;
   procedure Lookup_Relative
     (Node       : Bare_Ada_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Ada_Node);
   --  Implementation helpers for the looking up process
   -----------------
   -- Set_Parents --
   -----------------
   procedure Set_Parents
     (Node, Parent : Bare_Ada_Node)
   is
   begin
      if Node = null then
         return;
      end if;
      Node.Parent := Bare_Ada_Node (Parent);
      for I in 1 .. Children_Count (Node) loop
         Set_Parents (Child (Node, I), Node);
      end loop;
   end Set_Parents;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Node : Bare_Ada_Node) is
   begin
      if Node = null then
         return;
      end if;
      Free_User_Fields (Node);
      for I in 1 .. Children_Count (Node) loop
         Destroy (Child (Node, I));
      end loop;
   end Destroy;
   -----------
   -- Child --
   -----------
   function Child (Node  : Bare_Ada_Node;
                   Index : Positive) return Bare_Ada_Node
   is
      Result          : Bare_Ada_Node;
      Index_In_Bounds : Boolean;
   begin
      Get_Child (Node, Index, Index_In_Bounds, Result);
      return Result;
   end Child;
   --------------
   -- Traverse --
   --------------
   function Traverse
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node)
              return Visit_Status)
     return Visit_Status
   is
      Status : Visit_Status := Into;
   begin
      if Node /= null then
         Status := Visit (Node);
         --  Skip processing the child nodes if the returned status is Over
         --  or Stop. In the former case the previous call to Visit has taken
         --  care of processing the needed childs, and in the latter case we
         --  must immediately stop processing the tree.
         if Status = Into then
            for I in 1 .. Children_Count (Node) loop
               declare
                  Cur_Child : constant Bare_Ada_Node :=
                     Child (Node, I);
               begin
                  if Cur_Child /= null then
                     Status := Traverse (Cur_Child, Visit);
                     exit when Status /= Into;
                  end if;
               end;
            end loop;
         end if;
      end if;
      if Status = Stop then
         return Stop;
      --  At this stage the Over status has no sense and we just continue
      --  processing the tree.
      else
         return Into;
      end if;
   end Traverse;
   --------------
   -- Traverse --
   --------------
   procedure Traverse
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node)
                               return Visit_Status)
   is
      Result_Status : Visit_Status;
      pragma Unreferenced (Result_Status);
   begin
      Result_Status := Traverse (Node, Visit);
   end Traverse;
   ------------------------
   -- Traverse_With_Data --
   ------------------------
   function Traverse_With_Data
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node;
                               Data : in out Data_Type)
                               return Visit_Status;
      Data  : in out Data_Type)
      return Visit_Status
   is
      function Helper (Node : Bare_Ada_Node) return Visit_Status;
      ------------
      -- Helper --
      ------------
      function Helper (Node : Bare_Ada_Node) return Visit_Status is
      begin
         return Visit (Node, Data);
      end Helper;
      Saved_Data : Data_Type;
      Result     : Visit_Status;
   begin
      if Reset_After_Traversal then
         Saved_Data := Data;
      end if;
      Result := Traverse (Node, Helper'Access);
      if Reset_After_Traversal then
         Data := Saved_Data;
      end if;
      return Result;
   end Traverse_With_Data;
   ----------------
   -- Sloc_Range --
   ----------------
   function Sloc_Range
     (Node : Bare_Ada_Node) return Source_Location_Range
   is
      type Token_Anchor is (T_Start, T_End);
      type Token_Pos is record
         Pos    : Token_Index;
         Anchor : Token_Anchor;
      end record;
      TDH                    : Token_Data_Handler renames Node.Unit.TDH;
      Token_Start, Token_End : Token_Pos;
      function Get (Index : Token_Index) return Stored_Token_Data is
        (Get_Token (TDH, Index));
      function Sloc (T : Token_Pos) return Source_Location is
        (if T.Anchor = T_Start
         then Sloc_Start (TDH, Get (T.Pos))
         else Sloc_End (TDH, Get (T.Pos)));
   begin
      if Is_Synthetic (Node) then
         return Sloc_Range (Node.Parent);
      end if;
      if Is_Ghost (Node) then
         Token_Start := (if Node.Token_Start_Index = 1
                         then (1, T_Start)
                         else (Node.Token_Start_Index - 1, T_End));
         Token_End := Token_Start;
      else
         Token_Start := (Node.Token_Start_Index, T_Start);
         Token_End := (Node.Token_End_Index, T_End);
      end if;
      if Snaps_At_Start (Node)
         and then not Is_Ghost (Node)
         and then Token_Start.Pos /= 1
      then
         Token_Start := (Token_Start.Pos - 1, T_End);
      end if;
      if Snaps_At_End (Node) and then Token_End.Pos /= Last_Token (TDH) then
         Token_End := (Token_End.Pos + 1, T_Start);
      end if;
      return Make_Range (Sloc (Token_Start), Sloc (Token_End));
   end Sloc_Range;
   ------------
   -- Lookup --
   ------------
   function Lookup
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node
   is
      Position : Relative_Position;
      Result   : Bare_Ada_Node;
   begin
      if Sloc = No_Source_Location then
         return null;
      end if;
      Lookup_Relative
        (Bare_Ada_Node (Node), Sloc, Position, Result);
      return Result;
   end Lookup;
   ---------------------
   -- Lookup_Internal --
   ---------------------
   function Lookup_Internal
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node
   is
      --  For this implementation helper (i.e. internal primitive), we can
      --  assume that all lookups fall into this node's sloc range.
      pragma Assert (Compare (Sloc_Range (Node), Sloc) = Inside);
      Children : constant Internal_Bare_Ada_Node_Array :=
         Implementation.Children (Node);
      Pos      : Relative_Position;
      Result   : Bare_Ada_Node;
   begin
      --  Look for a child node that contains Sloc (i.e. return the most
      --  precise result).
      for Child of Children loop
         --  Note that we assume here that child nodes are ordered so that the
         --  first one has a sloc range that is before the sloc range of the
         --  second child node, etc.
         if Child /= null then
            Lookup_Relative (Child, Sloc, Pos, Result);
            case Pos is
               when Before =>
                   --  If this is the first node, Sloc is before it, so we can
                   --  stop here.  Otherwise, Sloc is between the previous
                   --  child node and the next one...  so we can stop here,
                   --  too.
                   return Node;
               when Inside =>
                   return Result;
               when After =>
                   --  Sloc is after the current child node, so see with the
                   --  next one.
                   null;
            end case;
         end if;
      end loop;
      --  If we reach this point, we found no children that covers Sloc, but
      --  Node still covers it (see the assertion).
      return Node;
   end Lookup_Internal;
   -------------
   -- Compare --
   -------------
   function Compare
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Relative_Position is
   begin
      return Compare (Sloc_Range (Node), Sloc);
   end Compare;
   ---------------------
   -- Lookup_Relative --
   ---------------------
   procedure Lookup_Relative
     (Node       : Bare_Ada_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Ada_Node)
   is
      Result : constant Relative_Position :=
        Compare (Node, Sloc);
   begin
      Position := Result;
      Node_Found := (if Result = Inside
                     then Lookup_Internal (Node, Sloc)
                     else null);
   end Lookup_Relative;
   -------------
   -- Compare --
   -------------
   function Compare
     (Self, Left, Right : Bare_Ada_Node;
      Relation          : Comparison_Relation) return Boolean
   is
      LS, RS : Source_Location;
   begin
      if Left = null or else Right = null or else Left.Unit /= Right.Unit then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "invalid node comparison");
      end if;
      LS := Start_Sloc (Sloc_Range (Left));
      RS := Start_Sloc (Sloc_Range (Right));
      return (case Relation is
              when Langkit_Support.Types.Less_Than        => LS < RS,
              when Langkit_Support.Types.Less_Or_Equal    => LS <= RS,
              when Langkit_Support.Types.Greater_Than     => LS > RS,
              when Langkit_Support.Types.Greater_Or_Equal => LS >= RS);
   end Compare;
   --------------
   -- Children --
   --------------
   function Children
     (Node : Bare_Ada_Node) return Internal_Bare_Ada_Node_Array
   is
      First : constant Integer := Bare_Ada_Node_Vectors.Index_Type'First;
      Last  : constant Integer := First + Children_Count (Node) - 1;
   begin
      return A : Internal_Bare_Ada_Node_Array (First .. Last)
      do
         for I in First .. Last loop
            A (I) := Child (Node, I);
         end loop;
      end return;
   end Children;
   function Children
     (Node : Bare_Ada_Node) return Bare_Ada_Node_Array_Access
   is
      C : Internal_Bare_Ada_Node_Array := Children (Node);
   begin
      return Ret : Bare_Ada_Node_Array_Access :=
         Create_Bare_Ada_Node_Array (C'Length)
      do
         Ret.Items := C;
      end return;
   end Children;
   ---------
   -- Get --
   ---------
   function Get
     (Self    : Bare_Ada_Node;
      Node    : Bare_Ada_List;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Ada_Node
   is
      function Length (Node : Bare_Ada_List) return Natural
      is (Node.Count);
      --  Wrapper around the Length primitive to get the compiler happy for the
      --  the package instantiation below.
      function Absolute_Get
        (L     : Bare_Ada_List;
         Index : Integer) return Bare_Ada_Node
      is (L.Nodes.all (Index + 1));
      --  L.Nodes is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Ada_Node,
         Sequence_Type => Bare_Ada_List,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Ada_Node;
   begin
      if Node = null and then Or_Null then
         return null;
      elsif Relative_Get (Node, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out-of-bounds AST list access");
      end if;
   end Get;
   ---------------
   -- PP_Trivia --
   ---------------
   procedure PP_Trivia
     (Node        : Bare_Ada_Node;
      Line_Prefix : String := "")
   is
      Children_Prefix : constant String := Line_Prefix & "|  ";
   begin
      if Node = null then
         Put_Line (Line_Prefix & "None");
         return;
      end if;
      Put_Line (Line_Prefix & Kind_Name (Node));
      for C of Children_And_Trivia (Node) loop
         case C.Kind is
            when Trivia =>
               Put_Line (Children_Prefix & Image (Text (C.Trivia)));
            when Child =>
               PP_Trivia (C.Node, Children_Prefix);
         end case;
      end loop;
   end PP_Trivia;
   --------------------------
   -- Populate_Lexical_Env --
   --------------------------
   function Populate_Lexical_Env (Node : Bare_Ada_Node) return Boolean is
      Context    : constant Internal_Context := Node.Unit.Context;
      Unit_State : aliased PLE_Unit_State := (Named_Envs_Needing_Update => <>);
      Root_State : constant PLE_Node_State :=
        (Unit_State  => Unit_State'Unchecked_Access,
         Current_Env => Context.Root_Scope,
         Current_NED => null);
      function Populate_Internal
        (Node         : Bare_Ada_Node;
         Parent_State : PLE_Node_State) return Boolean;
      --  Do the lexical env population on Node and recurse on its children
      procedure Register_Foreign_Env
        (Node : Bare_Ada_Node; State : PLE_Node_State);
      --  Given a node and its PLE state, register Node.Self_Env as being
      --  initialized through the named environment mechanism, if that's indeed
      --  the case. Do nothing otherwise.
      -----------------------
      -- Populate_Internal --
      -----------------------
      function Populate_Internal
        (Node         : Bare_Ada_Node;
         Parent_State : PLE_Node_State) return Boolean
      is
         Result : Boolean := False;
         State  : PLE_Node_State := Parent_State;
      begin
         if Node = null then
            return Result;
         end if;
         --  By default (i.e. unless env actions add a new env), the
         --  environment we store in Node is the current one.
         Node.Self_Env := State.Current_Env;
         --  Run pre/post actions, and run PLE on children in between. Make
         --  sure we register the potential foreign Node.Self_Env environment
         --  at the end, even when an exception interrupts PLE to keep the
         --  state consistent.
         begin
            Pre_Env_Actions (Node, State);
            if State.Current_Env /= Null_Lexical_Env then
               Node.Self_Env := State.Current_Env;
               Register_Foreign_Env (Node, State);
            end if;
            --  Call recursively on children
            for I in First_Child_Index (Node) .. Last_Child_Index (Node) loop
               Result := Populate_Internal
                 (Child (Node, I), State) or else Result;
            end loop;
            Post_Env_Actions (Node, State);
         exception
            when Exc : Precondition_Failure | Property_Error =>
               if PLE_Errors_Trace.Is_Active then
                   GNATCOLL.Traces.Trace
                     (PLE_Errors_Trace,
                      "Exception raised during PLE "
                      & Ada.Exceptions.Exception_Name (Exc) & " : "
                      & Ada.Exceptions.Exception_Message (Exc));
                   GNATCOLL.Traces.Trace
                     (PLE_Errors_Trace,
                      GNAT.Traceback.Symbolic.Symbolic_Traceback (Exc));
               end if;
               Register_Foreign_Env (Node, State);
               return True;
         end;
         return Result;
      end Populate_Internal;
      --------------------------
      -- Register_Foreign_Env --
      --------------------------
      procedure Register_Foreign_Env
        (Node : Bare_Ada_Node; State : PLE_Node_State) is
      begin
         if State.Current_NED /= null then
            State.Current_NED.Nodes_With_Foreign_Env.Insert (Node);
            Node.Unit.Nodes_With_Foreign_Env.Insert (Node, State.Current_NED);
         end if;
      end Register_Foreign_Env;
   begin
      --  This is intended to be called on the root node only (when there is no
      --  PLE root) or on a PLE root (child of the root node with a specific
      --  kind).
      if
         Node.Parent /= null
            and then Node.Kind /= Ada_Compilation_Unit
      then
         raise Program_Error;
      end if;
      return Result : constant Boolean :=
         Populate_Internal (Node, Root_State)
      do
         Update_Named_Envs (Context, Unit_State.Named_Envs_Needing_Update);
      end return;
   end Populate_Lexical_Env;
   ------------------------------
   -- AST_Envs_Node_Text_Image --
   ------------------------------
   function AST_Envs_Node_Text_Image
     (Node  : Bare_Ada_Node;
      Short : Boolean := True) return Text_Type is
   begin
      if Short then
         if Node = null then
            return "null";
         end if;
         return To_Text (Basename (Node.Unit))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node))));
      else
         return Short_Text_Image (Node);
      end if;
   end AST_Envs_Node_Text_Image;
   -------------------
   -- Is_Rebindable --
   -------------------
   function Is_Rebindable (Node : Bare_Ada_Node) return Boolean is
   begin
         return Node.Kind in Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl;
   end Is_Rebindable;
   -----------------------
   -- Acquire_Rebinding --
   -----------------------
   function Acquire_Rebinding
     (Node             : Bare_Ada_Node;
      Parent           : Env_Rebindings;
      Old_Env, New_Env : Lexical_Env) return Env_Rebindings
   is
      Result    : Env_Rebindings;
      Available : Env_Rebindings_Vectors.Vector renames
         Node.Unit.Context.Available_Rebindings;
   begin
      --  Use an existing and available Env_Rebindings_Type record for Node's
      --  Context, otherwise allocate a new rebinding.
      Result := (if Available.Is_Empty
                 then new Env_Rebindings_Type'(Version => 0, others => <>)
                 else Available.Pop);
      Result.Parent := Parent;
      Result.Old_Env := Old_Env;
      Result.New_Env := New_Env;
      Result.Children := Env_Rebindings_Vectors.Empty_Vector;
      return Result;
   end Acquire_Rebinding;
   -----------------------
   -- Release_Rebinding --
   -----------------------
   procedure Release_Rebinding (Self : in out Env_Rebindings) is
      Available : Env_Rebindings_Vectors.Vector renames
         Unwrap (Self.Old_Env).Node.Unit.Context.Available_Rebindings;
   begin
      --  Bumping the version number, to invalidate existing references to
      --  Self.
      Self.Version := Self.Version + 1;
      Self.Children.Destroy;
      Available.Append (Self);
      Self := null;
   end Release_Rebinding;
   ------------------------
   -- Register_Rebinding --
   ------------------------
   procedure Register_Rebinding
     (Node : Bare_Ada_Node; Rebinding : Env_Rebindings) is
   begin
      Node.Unit.Rebindings.Append (Rebinding);
   end Register_Rebinding;
   --------------------
   -- Element_Parent --
   --------------------
   function Element_Parent
     (Node : Bare_Ada_Node) return Bare_Ada_Node
   is (Node.Parent);
   ---------------
   -- Node_Unit --
   ---------------
   function Node_Unit (Node : Bare_Ada_Node) return Generic_Unit_Ptr is
   begin
      return Convert_Unit (Node.Unit);
   end Node_Unit;
   ----------
   -- Hash --
   ----------
   function Hash (Node : Bare_Ada_Node) return Hash_Type
   is
      function H is new Hash_Access
        (Root_Node_Record, Bare_Ada_Node);
   begin
      return H (Node);
   end Hash;
      function Hash (B : Boolean) return Hash_Type is (Boolean'Pos (B));
      function Hash (I : Integer) return Hash_Type is (Hash_Type'Mod (I));
   ------------------------
   -- Named environments --
   ------------------------
   ---------
   -- Add --
   ---------
   procedure Add
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : AST_Envs.Internal_Map_Node)
   is
      use NED_Assoc_Maps;
      Pos   : Cursor;
      Dummy : Boolean;
   begin
      --  Make sure there is a vector entry for Key
      Self.Insert (Key, Internal_Map_Node_Vectors.Empty_Vector, Pos, Dummy);
      --  Append Node to that vector
      declare
         V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Pos);
      begin
         V.Append (Node);
      end;
   end Add;
   ------------
   -- Remove --
   ------------
   procedure Remove
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : Bare_Ada_Node)
   is
      use NED_Assoc_Maps;
      V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Key);
   begin
      --  Remove the (assumed unique) entry in V whose node is Node. The order
      --  of items in V is not significant, so we can use Pop for efficient
      --  removal. Do the traversal in reverse order for correctness.
      for I in reverse 1 .. V.Length loop
         if V.Get_Access (I).Node = Node then
            V.Pop (I);
            exit;
         end if;
      end loop;
   end Remove;
   ------------------------------
   -- Get_Named_Env_Descriptor --
   ------------------------------
   function Get_Named_Env_Descriptor
     (Context : Internal_Context;
      Name    : Symbol_Type) return Named_Env_Descriptor_Access
   is
      use NED_Maps;
      --  Look for an existing entry for Name
      Pos : constant Cursor := Context.Named_Envs.Find (Name);
   begin
      if Has_Element (Pos) then
         return Element (Pos);
      end if;
      --  There is no such entry: create one
      return Result : constant Named_Env_Descriptor_Access :=
         new Named_Env_Descriptor'
           (Name                   => Name,
            Envs                   => <>,
            Env_With_Precedence    => Empty_Env,
            Foreign_Nodes          => <>,
            Foreign_Envs           => <>,
            Nodes_With_Foreign_Env => <>)
      do
         Context.Named_Envs.Insert (Name, Result);
      end return;
   end Get_Named_Env_Descriptor;
   ------------------------
   -- Register_Named_Env --
   ------------------------
   procedure Register_Named_Env
     (Context                   : Internal_Context;
      Name                      : Symbol_Type;
      Env                       : Lexical_Env;
      Named_Envs_Needing_Update : in out NED_Maps.Map)
   is
      NED_Access : constant Named_Env_Descriptor_Access :=
         Get_Named_Env_Descriptor (Context, Name);
      NED        : Named_Env_Descriptor renames NED_Access.all;
      Node       : constant Bare_Ada_Node := Env_Node (Env);
   begin
      NED.Envs.Insert (Node, Env);
      Node.Unit.Named_Envs.Append ((Name, Env));
      --  If that insertion must change the env that has precedence, signal
      --  that NED requires an update.
      if NED.Envs.First_Element /= NED.Env_With_Precedence then
         Named_Envs_Needing_Update.Include (Name, NED_Access);
      end if;
   end Register_Named_Env;
   ----------------------
   -- Update_Named_Env --
   ----------------------
   procedure Update_Named_Envs
     (Context : Internal_Context; Named_Envs : NED_Maps.Map)
   is
      Require_Cache_Reset : Boolean := False;
   begin
      for Cur in Named_Envs.Iterate loop
         declare
            NE      : Named_Env_Descriptor renames NED_Maps.Element (Cur).all;
            New_Env : constant Lexical_Env :=
              (if NE.Envs.Is_Empty
               then Empty_Env
               else NE.Envs.First_Element);
         begin
            --  If there was an environment with precedence, remove its foreign
            --  nodes.
            if NE.Env_With_Precedence /= Empty_Env then
               for Cur in NE.Foreign_Nodes.Iterate loop
                  declare
                     Key   : constant Symbol_Type :=
                        NED_Assoc_Maps.Key (Cur);
                     Nodes : Internal_Map_Node_Vectors.Vector renames
                        NE.Foreign_Nodes.Reference (Cur);
                  begin
                     for N of Nodes loop
                        Remove (NE.Env_With_Precedence, Key, N.Node);
                     end loop;
                  end;
               end loop;
            end if;
            --  Now, set the new environment that has precedence
            NE.Env_With_Precedence := New_Env;
            --  Add the foreign nodes to the new environment with precedence,
            --  if any.
            for Cur in NE.Foreign_Nodes.Iterate loop
               declare
                  Key   : constant Symbol_Type :=
                     NED_Assoc_Maps.Key (Cur);
                  Nodes : Internal_Map_Node_Vectors.Vector renames
                     NE.Foreign_Nodes.Reference (Cur);
               begin
                  for N of Nodes loop
                     Add (New_Env, Key, N.Node, N.Md, N.Resolver);
                  end loop;
               end;
            end loop;
            --  Set the parent environment of all foreign environments
            for Cur in NE.Foreign_Envs.Iterate loop
               declare
                  Env : Lexical_Env_Record renames
                     Unwrap (Sorted_Env_Maps.Element (Cur)).all;
               begin
                  Env.Parent := New_Env;
                  --  We have updated the lexical env hierarchy (e.g. an env
                  --  which had no parent may have one now), so the cached
                  --  entries for queries that traveresed the old env hierarchy
                  --  need to be invalidated.
                  Require_Cache_Reset := True;
               end;
            end loop;
            --  Update nodes whose environment was the old env with precedence
            for N of NE.Nodes_With_Foreign_Env loop
               N.Self_Env := New_Env;
            end loop;
         end;
      end loop;
      if Require_Cache_Reset then
         Invalidate_Caches (Context, Invalidate_Envs => True);
      end if;
   end Update_Named_Envs;
   --------------------------
   -- Big integers wrapper --
   --------------------------
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer
     (Image : String; Base : Integer := 10) return Big_Integer_Type
   is
      use GNATCOLL.GMP;
      use GNATCOLL.GMP.Integers;
   begin
      return new Big_Integer_Record'(Value     => Make (Image, Int (Base)),
                                     Ref_Count => 1);
   end Create_Big_Integer;
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer
     (Big_Int : GNATCOLL.GMP.Integers.Big_Integer) return Big_Integer_Type
   is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (Big_Int);
      return Result;
   end Create_Big_Integer;
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer (Int : Integer) return Big_Integer_Type is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (GNATCOLL.GMP.Long (Int));
      return Result;
   end Create_Big_Integer;
   -------------------------------
   -- Create_Public_Big_Integer --
   -------------------------------
   function Create_Public_Big_Integer
     (Big_Int : Big_Integer_Type) return GNATCOLL.GMP.Integers.Big_Integer is
   begin
      return Result : GNATCOLL.GMP.Integers.Big_Integer do
         Result.Set (Big_Int.Value);
      end return;
   end Create_Public_Big_Integer;
   -----------------
   -- Trace_Image --
   -----------------
   function Trace_Image (I : Big_Integer_Type) return String is
   begin
      return GNATCOLL.GMP.Integers.Image (I.Value);
   end Trace_Image;
   ----------------
   -- To_Integer --
   ----------------
   function To_Integer
     (Self    : Bare_Ada_Node;
      Big_Int : Big_Integer_Type) return Integer
   is
      Image : constant String := Big_Int.Value.Image;
   begin
      return Integer'Value (Image);
   exception
      when Constraint_Error =>
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out of range big integer");
   end To_Integer;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Big_Int : Big_Integer_Type) is
   begin
      if Big_Int.Ref_Count /= -1 then
         Big_Int.Ref_Count := Big_Int.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Big_Int : in out Big_Integer_Type) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Big_Integer_Record, Big_Integer_Type);
   begin
      if Big_Int = null or else Big_Int.Ref_Count = -1 then
         return;
      end if;
      Big_Int.Ref_Count := Big_Int.Ref_Count - 1;
      if Big_Int.Ref_Count = 0 then
         Destroy (Big_Int);
      end if;
   end Dec_Ref;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value = Right.Value;
   end Equivalent;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value < Right.Value;
   end "<";
   ----------
   -- "<=" --
   ----------
   function "<=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value <= Right.Value;
   end "<=";
   ---------
   -- ">" --
   ---------
   function ">" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value > Right.Value;
   end ">";
   ----------
   -- ">=" --
   ----------
   function ">=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value >= Right.Value;
   end ">=";
   ---------
   -- "+" --
   ---------
   function "+" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value + Right.Value);
   end "+";
   ---------
   -- "-" --
   ---------
   function "-" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value - Right.Value);
   end "-";
   ------------------
   -- Unit_Version --
   ------------------
   function Unit_Version (Unit : Generic_Unit_Ptr) return Version_Number is
   begin
      return Convert_Unit (Unit).Unit_Version;
   end Unit_Version;
   -------------------------
   -- Get_Context_Version --
   -------------------------
   function Get_Context_Version
     (Node : Bare_Ada_Node) return Version_Number is
   begin
      return Node.Unit.Context.Cache_Version;
   end Get_Context_Version;
   --------------------------
   -- Properties_May_Raise --
   --------------------------
   function Properties_May_Raise
     (Exc : Ada.Exceptions.Exception_Occurrence) return Boolean is
   begin
      return Ada.Exceptions.Exception_Identity (Exc) in
            Precondition_Failure'Identity
               |
            Property_Error'Identity
      ;
   end Properties_May_Raise;
   ----------------------
   -- Short_Text_Image --
   ----------------------
   function Short_Text_Image (Self : Bare_Ada_Node) return Text_Type
   is
   begin
      if Self = null then
         return "None";
      end if;
   case Self.Kind is
            when Ada_Char_Literal .. Ada_String_Literal =>
         return Base_Id_Short_Image (Self);
            when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
         return Defining_Name_Short_Image (Self);
            when Ada_Abstract_State_Decl .. Ada_Single_Task_Decl =>
         return Basic_Decl_Short_Image (Self);
      when others => 
         return "<" & To_Text (Kind_Name (Self))
                & " "
                & To_Text
                  (Ada.Directories.Simple_Name
                     (Get_Filename (Unit (Self))))
                & ":" & To_Text (Image (Sloc_Range (Self))) & ">";
   end case;
   end Short_Text_Image;
   --------------------
   -- Snaps_At_Start --
   --------------------
   function Snaps_At_Start (Self : Bare_Ada_Node) return Boolean is
   begin
   case Self.Kind is
            when Ada_Handled_Stmts =>
         return True;
            when Ada_Declarative_Part .. Ada_Public_Part =>
         return True;
      when others => 
         return False;
   end case;
   end Snaps_At_Start;
   ------------------
   -- Snaps_At_End --
   ------------------
   function Snaps_At_End (Self : Bare_Ada_Node) return Boolean is
   begin
   case Self.Kind is
            when Ada_Handled_Stmts =>
         return True;
            when Ada_Declarative_Part .. Ada_Public_Part =>
         return True;
      when others => 
         return Is_Incomplete (Self);
   end case;
   end Snaps_At_End;
   -------------
   -- Parents --
   -------------
   function Parents
     (Node      : Bare_Ada_Node;
      With_Self : Boolean := True)
      return Bare_Ada_Node_Array_Access
   is
      Count : Natural := 0;
      Start : Bare_Ada_Node :=
        (if With_Self then Node else Node.Parent);
      Cur   : Bare_Ada_Node := Start;
   begin
      while Cur /= null loop
         Count := Count + 1;
         Cur := Cur.Parent;
      end loop;
      declare
         Result : constant Bare_Ada_Node_Array_Access :=
            Create_Bare_Ada_Node_Array (Count);
      begin
         Cur := Start;
         for I in Result.Items'Range loop
            Result.Items (I) := Cur;
            Cur := Cur.Parent;
         end loop;
         return Result;
      end;
   end Parents;
   -----------------------
   -- First_Child_Index --
   -----------------------
   function First_Child_Index (Node : Bare_Ada_Node) return Natural
   is (1);
   ----------------------
   -- Last_Child_Index --
   ----------------------
   function Last_Child_Index (Node : Bare_Ada_Node) return Natural
   is (Children_Count (Node));
   ---------------
   -- Get_Child --
   ---------------
   procedure Get_Child
     (Node            : Bare_Ada_Node;
      Index           : Positive;
      Index_In_Bounds : out Boolean;
      Result          : out Bare_Ada_Node)
   is
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      Index_In_Bounds := True;
      Result := null;
      case Ada_Ada_Node (K) is
when Ada_Ada_List =>
declare
N_Bare_Ada_List : constant Bare_Ada_List := Node;
begin
                    if Index > N_Bare_Ada_List.Count then
                        Index_In_Bounds := False;
                    else
                        Result := N_Bare_Ada_List.Nodes (Index);
                    end if;
                    return;
end;
when Ada_Constrained_Array_Indices_Range =>
declare
N_Bare_Constrained_Array_Indices : constant Bare_Constrained_Array_Indices := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Constrained_Array_Indices.Constrained_Array_Indices_F_List;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Unconstrained_Array_Indices_Range =>
declare
N_Bare_Unconstrained_Array_Indices : constant Bare_Unconstrained_Array_Indices := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Unconstrained_Array_Indices.Unconstrained_Array_Indices_F_Types;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aspect_Assoc_Range =>
declare
N_Bare_Aspect_Assoc : constant Bare_Aspect_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aspect_Assoc.Aspect_Assoc_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Aspect_Assoc.Aspect_Assoc_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_At_Clause_Range =>
declare
N_Bare_At_Clause : constant Bare_At_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_At_Clause.At_Clause_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_At_Clause.At_Clause_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Attribute_Def_Clause_Range =>
declare
N_Bare_Attribute_Def_Clause : constant Bare_Attribute_Def_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Attribute_Def_Clause.Attribute_Def_Clause_F_Attribute_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Attribute_Def_Clause.Attribute_Def_Clause_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Rep_Clause_Range =>
declare
N_Bare_Enum_Rep_Clause : constant Bare_Enum_Rep_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Rep_Clause.Enum_Rep_Clause_F_Type_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Enum_Rep_Clause.Enum_Rep_Clause_F_Aggregate;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Record_Rep_Clause_Range =>
declare
N_Bare_Record_Rep_Clause : constant Bare_Record_Rep_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_At_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aspect_Spec_Range =>
declare
N_Bare_Aspect_Spec : constant Bare_Aspect_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aspect_Spec.Aspect_Spec_F_Aspect_Assocs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Contract_Case_Assoc_Range =>
declare
N_Bare_Contract_Case_Assoc : constant Bare_Contract_Case_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Contract_Case_Assoc.Contract_Case_Assoc_F_Guard;
                            return;
                        when 2 =>
                            Result := N_Bare_Contract_Case_Assoc.Contract_Case_Assoc_F_Consequence;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pragma_Argument_Assoc_Range =>
declare
N_Bare_Pragma_Argument_Assoc : constant Bare_Pragma_Argument_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pragma_Argument_Assoc.Pragma_Argument_Assoc_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Pragma_Argument_Assoc.Pragma_Argument_Assoc_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Spec_Range =>
declare
N_Bare_Entry_Spec : constant Bare_Entry_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Entry_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Family_Type;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Entry_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Spec_Range =>
declare
N_Bare_Subp_Spec : constant Bare_Subp_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Returns;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Binary_Spec_Range =>
declare
N_Bare_Synthetic_Binary_Spec : constant Bare_Synthetic_Binary_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Binary_Spec.Synthetic_Binary_Spec_F_Left_Param;
                            return;
                        when 2 =>
                            Result := N_Bare_Synthetic_Binary_Spec.Synthetic_Binary_Spec_F_Right_Param;
                            return;
                        when 3 =>
                            Result := N_Bare_Synthetic_Binary_Spec.Synthetic_Binary_Spec_F_Return_Type_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Unary_Spec_Range =>
declare
N_Bare_Synthetic_Unary_Spec : constant Bare_Synthetic_Unary_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Unary_Spec.Synthetic_Unary_Spec_F_Right_Param;
                            return;
                        when 2 =>
                            Result := N_Bare_Synthetic_Unary_Spec.Synthetic_Unary_Spec_F_Return_Type_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_List_Range =>
declare
N_Bare_Component_List : constant Bare_Component_List := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_List.Component_List_F_Components;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_List.Component_List_F_Variant_Part;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Known_Discriminant_Part_Range =>
declare
N_Bare_Known_Discriminant_Part : constant Bare_Known_Discriminant_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Known_Discriminant_Part.Known_Discriminant_Part_F_Discr_Specs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Completion_Formal_Params_Range =>
declare
N_Bare_Entry_Completion_Formal_Params : constant Bare_Entry_Completion_Formal_Params := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Completion_Formal_Params.Entry_Completion_Formal_Params_F_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Formal_Part_Range =>
declare
N_Bare_Generic_Formal_Part : constant Bare_Generic_Formal_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Formal_Part.Generic_Formal_Part_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Record_Def =>
declare
N_Bare_Base_Record_Def : constant Bare_Base_Record_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Record_Def.Base_Record_Def_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aggregate_Assoc_Range =>
declare
N_Bare_Aggregate_Assoc : constant Bare_Aggregate_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aggregate_Assoc.Aggregate_Assoc_F_Designators;
                            return;
                        when 2 =>
                            Result := N_Bare_Aggregate_Assoc.Aggregate_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Composite_Constraint_Assoc_Range =>
declare
N_Bare_Composite_Constraint_Assoc : constant Bare_Composite_Constraint_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Composite_Constraint_Assoc.Composite_Constraint_Assoc_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Composite_Constraint_Assoc.Composite_Constraint_Assoc_F_Constraint_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Iterated_Assoc_Range =>
declare
N_Bare_Iterated_Assoc : constant Bare_Iterated_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Param_Assoc_Range =>
declare
N_Bare_Param_Assoc : constant Bare_Param_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Param_Assoc.Param_Assoc_F_Designator;
                            return;
                        when 2 =>
                            Result := N_Bare_Param_Assoc.Param_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abstract_State_Decl_Range =>
declare
N_Bare_Abstract_State_Decl : constant Bare_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abstract_State_Decl.Abstract_State_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Abstract_State_Decl.Abstract_State_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Expr_Decl_Range =>
declare
N_Bare_Anonymous_Expr_Decl : constant Bare_Anonymous_Expr_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Anonymous_Expr_Decl.Anonymous_Expr_Decl_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Decl_Range =>
declare
N_Bare_Component_Decl : constant Bare_Component_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Component_Def;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discriminant_Spec_Range =>
declare
N_Bare_Discriminant_Spec : constant Bare_Discriminant_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Type_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Formal =>
declare
N_Bare_Generic_Formal : constant Bare_Generic_Formal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Formal.Generic_Formal_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Param_Spec_Range =>
declare
N_Bare_Param_Spec : constant Bare_Param_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Has_Aliased;
                            return;
                        when 3 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Mode;
                            return;
                        when 4 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Type_Expr;
                            return;
                        when 5 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Default_Expr;
                            return;
                        when 6 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Formal_Param_Decl_Range =>
declare
N_Bare_Synthetic_Formal_Param_Decl : constant Bare_Synthetic_Formal_Param_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Formal_Param_Decl.Synthetic_Formal_Param_Decl_F_Param_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Package_Decl =>
declare
N_Bare_Base_Package_Decl : constant Bare_Base_Package_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Package_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Public_Part;
                            return;
                        when 4 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Private_Part;
                            return;
                        when 5 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Type_Decl =>
declare
N_Bare_Base_Type_Decl : constant Bare_Base_Type_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Type_Decl.Base_Type_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
case Ada_Base_Type_Decl (K) is
when Ada_Subtype_Decl_Range =>
declare
N_Bare_Subtype_Decl : constant Bare_Subtype_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Subtype_Decl.Subtype_Decl_F_Subtype;
                            return;
                        when 3 =>
                            Result := N_Bare_Subtype_Decl.Subtype_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Incomplete_Type_Decl_Range =>
declare
N_Bare_Incomplete_Type_Decl : constant Bare_Incomplete_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Incomplete_Type_Decl.Incomplete_Type_Decl_F_Discriminants;
                            return;
                        when others => null;
                    end case;
case Ada_Incomplete_Type_Decl_Range (K) is
when Ada_Incomplete_Formal_Type_Decl_Range =>
declare
N_Bare_Incomplete_Formal_Type_Decl : constant Bare_Incomplete_Formal_Type_Decl := N_Bare_Incomplete_Type_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Incomplete_Formal_Type_Decl.Incomplete_Formal_Type_Decl_F_Is_Tagged;
                            return;
                        when 4 =>
                            Result := N_Bare_Incomplete_Formal_Type_Decl.Incomplete_Formal_Type_Decl_F_Default_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Incomplete_Tagged_Type_Decl_Range =>
declare
N_Bare_Incomplete_Tagged_Type_Decl : constant Bare_Incomplete_Tagged_Type_Decl := N_Bare_Incomplete_Type_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Incomplete_Tagged_Type_Decl.Incomplete_Tagged_Type_Decl_F_Has_Abstract;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Protected_Type_Decl_Range =>
declare
N_Bare_Protected_Type_Decl : constant Bare_Protected_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Interfaces;
                            return;
                        when 5 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Type_Decl_Range =>
declare
N_Bare_Task_Type_Decl : constant Bare_Task_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Type_Decl =>
declare
N_Bare_Type_Decl : constant Bare_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Type_Def;
                            return;
                        when others => null;
                    end case;
case Ada_Type_Decl (K) is
when Ada_Concrete_Type_Decl_Range =>
declare
N_Bare_Concrete_Type_Decl : constant Bare_Concrete_Type_Decl := N_Bare_Type_Decl;
begin
case Index is
                        when 4 =>
                            Result := N_Bare_Concrete_Type_Decl.Concrete_Type_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Formal_Type_Decl_Range =>
declare
N_Bare_Formal_Type_Decl : constant Bare_Formal_Type_Decl := N_Bare_Type_Decl;
begin
case Index is
                        when 4 =>
                            Result := N_Bare_Formal_Type_Decl.Formal_Type_Decl_F_Default_Type;
                            return;
                        when 5 =>
                            Result := N_Bare_Formal_Type_Decl.Formal_Type_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when others => null;
end case;
end;
when Ada_Classic_Subp_Decl =>
declare
N_Bare_Classic_Subp_Decl : constant Bare_Classic_Subp_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Classic_Subp_Decl.Classic_Subp_Decl_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Classic_Subp_Decl.Classic_Subp_Decl_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
case Ada_Classic_Subp_Decl (K) is
when Ada_Abstract_Subp_Decl_Range =>
declare
N_Bare_Abstract_Subp_Decl : constant Bare_Abstract_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Abstract_Subp_Decl.Abstract_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Formal_Subp_Decl =>
declare
N_Bare_Formal_Subp_Decl : constant Bare_Formal_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Formal_Subp_Decl.Formal_Subp_Decl_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Formal_Subp_Decl.Formal_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Decl_Range =>
declare
N_Bare_Subp_Decl : constant Bare_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Decl.Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Entry_Decl_Range =>
declare
N_Bare_Entry_Decl : constant Bare_Entry_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Literal_Decl_Range =>
declare
N_Bare_Enum_Literal_Decl : constant Bare_Enum_Literal_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Literal_Decl.Enum_Literal_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Internal_Range =>
declare
N_Bare_Generic_Subp_Internal : constant Bare_Generic_Subp_Internal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Internal.Generic_Subp_Internal_F_Subp_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Internal.Generic_Subp_Internal_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Subp_Decl_Range =>
declare
N_Bare_Synthetic_Subp_Decl : constant Bare_Synthetic_Subp_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Subp_Decl.Synthetic_Subp_Decl_F_Spec;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Accept_Stmt_Body_Range =>
declare
N_Bare_Accept_Stmt_Body : constant Bare_Accept_Stmt_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Accept_Stmt_Body.Accept_Stmt_Body_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Subp_Body =>
declare
N_Bare_Base_Subp_Body : constant Bare_Base_Subp_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Subp_Body.Base_Subp_Body_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Subp_Body.Base_Subp_Body_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
case Ada_Base_Subp_Body (K) is
when Ada_Expr_Function_Range =>
declare
N_Bare_Expr_Function : constant Bare_Expr_Function := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Expr_Function.Expr_Function_F_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Expr_Function.Expr_Function_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Null_Subp_Decl_Range =>
declare
N_Bare_Null_Subp_Decl : constant Bare_Null_Subp_Decl := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Null_Subp_Decl.Null_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Body_Range =>
declare
N_Bare_Subp_Body : constant Bare_Subp_Body := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Decls;
                            return;
                        when 5 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Stmts;
                            return;
                        when 6 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Renaming_Decl_Range =>
declare
N_Bare_Subp_Renaming_Decl : constant Bare_Subp_Renaming_Decl := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Renaming_Decl.Subp_Renaming_Decl_F_Renames;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Renaming_Decl.Subp_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Package_Body_Stub_Range =>
declare
N_Bare_Package_Body_Stub : constant Bare_Package_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Body_Stub.Package_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Body_Stub.Package_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Body_Stub_Range =>
declare
N_Bare_Protected_Body_Stub : constant Bare_Protected_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Body_Stub.Protected_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Body_Stub.Protected_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Body_Stub_Range =>
declare
N_Bare_Subp_Body_Stub : constant Bare_Subp_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Subp_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Body_Stub_Range =>
declare
N_Bare_Task_Body_Stub : constant Bare_Task_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Body_Stub.Task_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Body_Stub.Task_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Body_Range =>
declare
N_Bare_Entry_Body : constant Bare_Entry_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Entry_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Index_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Aspects;
                            return;
                        when 5 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Barrier;
                            return;
                        when 6 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Decls;
                            return;
                        when 7 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Stmts;
                            return;
                        when 8 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Package_Body_Range =>
declare
N_Bare_Package_Body : constant Bare_Package_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Package_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Package_Body.Package_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Body_Range =>
declare
N_Bare_Protected_Body : constant Bare_Protected_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Body_Range =>
declare
N_Bare_Task_Body : constant Bare_Task_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Task_Body.Task_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Index_Spec_Range =>
declare
N_Bare_Entry_Index_Spec : constant Bare_Entry_Index_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Subtype;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exception_Decl_Range =>
declare
N_Bare_Exception_Decl : constant Bare_Exception_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exception_Handler_Range =>
declare
N_Bare_Exception_Handler : constant Bare_Exception_Handler := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Handled_Exceptions;
                            return;
                        when 3 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_For_Loop_Var_Decl_Range =>
declare
N_Bare_For_Loop_Var_Decl : constant Bare_For_Loop_Var_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Var_Decl.For_Loop_Var_Decl_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_For_Loop_Var_Decl.For_Loop_Var_Decl_F_Id_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Decl =>
declare
N_Bare_Generic_Decl : constant Bare_Generic_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Decl.Generic_Decl_F_Formal_Part;
                            return;
                        when others => null;
                    end case;
case Ada_Generic_Decl (K) is
when Ada_Generic_Package_Decl_Range =>
declare
N_Bare_Generic_Package_Decl : constant Bare_Generic_Package_Decl := N_Bare_Generic_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Generic_Package_Decl.Generic_Package_Decl_F_Package_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Decl_Range =>
declare
N_Bare_Generic_Subp_Decl : constant Bare_Generic_Subp_Decl := N_Bare_Generic_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Decl.Generic_Subp_Decl_F_Subp_Decl;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Generic_Package_Instantiation_Range =>
declare
N_Bare_Generic_Package_Instantiation : constant Bare_Generic_Package_Instantiation := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Generic_Pkg_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Instantiation_Range =>
declare
N_Bare_Generic_Subp_Instantiation : constant Bare_Generic_Subp_Instantiation := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Kind;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Subp_Name;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Generic_Subp_Name;
                            return;
                        when 5 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Params;
                            return;
                        when 6 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Package_Renaming_Decl_Range =>
declare
N_Bare_Generic_Package_Renaming_Decl : constant Bare_Generic_Package_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Renaming_Decl_Range =>
declare
N_Bare_Generic_Subp_Renaming_Decl : constant Bare_Generic_Subp_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Renames;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Label_Decl_Range =>
declare
N_Bare_Label_Decl : constant Bare_Label_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Label_Decl.Label_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Named_Stmt_Decl_Range =>
declare
N_Bare_Named_Stmt_Decl : constant Bare_Named_Stmt_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Named_Stmt_Decl.Named_Stmt_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Number_Decl_Range =>
declare
N_Bare_Number_Decl : constant Bare_Number_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Number_Decl.Number_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Number_Decl.Number_Decl_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Object_Decl_Range =>
declare
N_Bare_Object_Decl : constant Bare_Object_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Has_Aliased;
                            return;
                        when 3 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Has_Constant;
                            return;
                        when 4 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Mode;
                            return;
                        when 5 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Type_Expr;
                            return;
                        when 6 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Default_Expr;
                            return;
                        when 7 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Renaming_Clause;
                            return;
                        when 8 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Package_Renaming_Decl_Range =>
declare
N_Bare_Package_Renaming_Decl : constant Bare_Package_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Single_Protected_Decl_Range =>
declare
N_Bare_Single_Protected_Decl : constant Bare_Single_Protected_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Interfaces;
                            return;
                        when 4 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Single_Task_Decl_Range =>
declare
N_Bare_Single_Task_Decl : constant Bare_Single_Task_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Single_Task_Decl.Single_Task_Decl_F_Task_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Stmt_Alternative_Range =>
declare
N_Bare_Case_Stmt_Alternative : constant Bare_Case_Stmt_Alternative := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Stmt_Alternative.Case_Stmt_Alternative_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Stmt_Alternative.Case_Stmt_Alternative_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Compilation_Unit_Range =>
declare
N_Bare_Compilation_Unit : constant Bare_Compilation_Unit := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Prelude;
                            return;
                        when 2 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Body;
                            return;
                        when 3 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Pragmas;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Clause_Range =>
declare
N_Bare_Component_Clause : constant Bare_Component_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Position;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Def_Range =>
declare
N_Bare_Component_Def : constant Bare_Component_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Has_Aliased;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Has_Constant;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Type_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Composite_Constraint_Range =>
declare
N_Bare_Composite_Constraint : constant Bare_Composite_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Composite_Constraint.Composite_Constraint_F_Constraints;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Delta_Constraint_Range =>
declare
N_Bare_Delta_Constraint : constant Bare_Delta_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Delta_Constraint.Delta_Constraint_F_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Delta_Constraint.Delta_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Digits_Constraint_Range =>
declare
N_Bare_Digits_Constraint : constant Bare_Digits_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Digits_Constraint.Digits_Constraint_F_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Digits_Constraint.Digits_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Range_Constraint_Range =>
declare
N_Bare_Range_Constraint : constant Bare_Range_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Range_Constraint.Range_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Declarative_Part_Range =>
declare
N_Bare_Declarative_Part : constant Bare_Declarative_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Declarative_Part.Declarative_Part_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Elsif_Expr_Part_Range =>
declare
N_Bare_Elsif_Expr_Part : constant Bare_Elsif_Expr_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Elsif_Expr_Part.Elsif_Expr_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Elsif_Expr_Part.Elsif_Expr_Part_F_Then_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Elsif_Stmt_Part_Range =>
declare
N_Bare_Elsif_Stmt_Part : constant Bare_Elsif_Stmt_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Elsif_Stmt_Part.Elsif_Stmt_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Elsif_Stmt_Part.Elsif_Stmt_Part_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abstract_State_Decl_Expr_Range =>
declare
N_Bare_Abstract_State_Decl_Expr : constant Bare_Abstract_State_Decl_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abstract_State_Decl_Expr.Abstract_State_Decl_Expr_F_State_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Allocator_Range =>
declare
N_Bare_Allocator : constant Bare_Allocator := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Allocator.Allocator_F_Subpool;
                            return;
                        when 2 =>
                            Result := N_Bare_Allocator.Allocator_F_Type_Or_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Aggregate =>
declare
N_Bare_Base_Aggregate : constant Bare_Base_Aggregate := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Aggregate.Base_Aggregate_F_Ancestor_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Aggregate.Base_Aggregate_F_Assocs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Bin_Op_Range =>
declare
N_Bare_Bin_Op : constant Bare_Bin_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Left;
                            return;
                        when 2 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Op;
                            return;
                        when 3 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Right;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Expr_Alternative_Range =>
declare
N_Bare_Case_Expr_Alternative : constant Bare_Case_Expr_Alternative := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Expr_Alternative.Case_Expr_Alternative_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Expr_Alternative.Case_Expr_Alternative_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Concat_Op_Range =>
declare
N_Bare_Concat_Op : constant Bare_Concat_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Concat_Op.Concat_Op_F_First_Operand;
                            return;
                        when 2 =>
                            Result := N_Bare_Concat_Op.Concat_Op_F_Other_Operands;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Concat_Operand_Range =>
declare
N_Bare_Concat_Operand : constant Bare_Concat_Operand := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Concat_Operand.Concat_Operand_F_Operator;
                            return;
                        when 2 =>
                            Result := N_Bare_Concat_Operand.Concat_Operand_F_Operand;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Expr_Range =>
declare
N_Bare_Case_Expr : constant Bare_Case_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Expr.Case_Expr_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Expr.Case_Expr_F_Cases;
                            return;
                        when others => null;
                    end case;
end;
when Ada_If_Expr_Range =>
declare
N_Bare_If_Expr : constant Bare_If_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Then_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Alternatives;
                            return;
                        when 4 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Else_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Contract_Cases_Range =>
declare
N_Bare_Contract_Cases : constant Bare_Contract_Cases := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Contract_Cases.Contract_Cases_F_Contract_Cases;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decl_Expr_Range =>
declare
N_Bare_Decl_Expr : constant Bare_Decl_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decl_Expr.Decl_Expr_F_Decls;
                            return;
                        when 2 =>
                            Result := N_Bare_Decl_Expr.Decl_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Membership_Expr_Range =>
declare
N_Bare_Membership_Expr : constant Bare_Membership_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Op;
                            return;
                        when 3 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Membership_Exprs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Attribute;
                            return;
                        when 3 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Defining_Name_Range =>
declare
N_Bare_Defining_Name : constant Bare_Defining_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Defining_Name.Defining_Name_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discrete_Subtype_Name_Range =>
declare
N_Bare_Discrete_Subtype_Name : constant Bare_Discrete_Subtype_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discrete_Subtype_Name.Discrete_Subtype_Name_F_Subtype;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Dotted_Name_Range =>
declare
N_Bare_Dotted_Name : constant Bare_Dotted_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Dotted_Name.Dotted_Name_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Dotted_Name.Dotted_Name_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_End_Name_Range =>
declare
N_Bare_End_Name : constant Bare_End_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_End_Name.End_Name_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Explicit_Deref.Explicit_Deref_F_Prefix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Qual_Expr_Range =>
declare
N_Bare_Qual_Expr : constant Bare_Qual_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Qual_Expr.Qual_Expr_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Qual_Expr.Qual_Expr_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Reduce_Attribute_Ref_Range =>
declare
N_Bare_Reduce_Attribute_Ref : constant Bare_Reduce_Attribute_Ref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_F_Attribute;
                            return;
                        when 3 =>
                            Result := N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Update_Attribute_Ref_Range =>
declare
N_Bare_Update_Attribute_Ref : constant Bare_Update_Attribute_Ref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_F_Attribute;
                            return;
                        when 3 =>
                            Result := N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_F_Values;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Paren_Expr_Range =>
declare
N_Bare_Paren_Expr : constant Bare_Paren_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Paren_Expr.Paren_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Quantified_Expr_Range =>
declare
N_Bare_Quantified_Expr : constant Bare_Quantified_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Quantifier;
                            return;
                        when 2 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Loop_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Raise_Expr_Range =>
declare
N_Bare_Raise_Expr : constant Bare_Raise_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Error_Message;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Un_Op_Range =>
declare
N_Bare_Un_Op : constant Bare_Un_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Op;
                            return;
                        when 2 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Handled_Stmts_Range =>
declare
N_Bare_Handled_Stmts : constant Bare_Handled_Stmts := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Handled_Stmts.Handled_Stmts_F_Stmts;
                            return;
                        when 2 =>
                            Result := N_Bare_Handled_Stmts.Handled_Stmts_F_Exceptions;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Library_Item_Range =>
declare
N_Bare_Library_Item : constant Bare_Library_Item := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Library_Item.Library_Item_F_Has_Private;
                            return;
                        when 2 =>
                            Result := N_Bare_Library_Item.Library_Item_F_Item;
                            return;
                        when others => null;
                    end case;
end;
when Ada_For_Loop_Spec_Range =>
declare
N_Bare_For_Loop_Spec : constant Bare_For_Loop_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Var_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Loop_Type;
                            return;
                        when 3 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Has_Reverse;
                            return;
                        when 4 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Iter_Expr;
                            return;
                        when 5 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Iter_Filter;
                            return;
                        when others => null;
                    end case;
end;
when Ada_While_Loop_Spec_Range =>
declare
N_Bare_While_Loop_Spec : constant Bare_While_Loop_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_While_Loop_Spec.While_Loop_Spec_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Multi_Abstract_State_Decl_Range =>
declare
N_Bare_Multi_Abstract_State_Decl : constant Bare_Multi_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Multi_Abstract_State_Decl.Multi_Abstract_State_Decl_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Params_Range =>
declare
N_Bare_Params : constant Bare_Params := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Params.Params_F_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Paren_Abstract_State_Decl_Range =>
declare
N_Bare_Paren_Abstract_State_Decl : constant Bare_Paren_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Paren_Abstract_State_Decl.Paren_Abstract_State_Decl_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pp_Elsif_Directive_Range =>
declare
N_Bare_Pp_Elsif_Directive : constant Bare_Pp_Elsif_Directive := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pp_Elsif_Directive.Pp_Elsif_Directive_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Pp_Elsif_Directive.Pp_Elsif_Directive_F_Then_Kw;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pp_If_Directive_Range =>
declare
N_Bare_Pp_If_Directive : constant Bare_Pp_If_Directive := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pp_If_Directive.Pp_If_Directive_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Pp_If_Directive.Pp_If_Directive_F_Then_Kw;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pragma_Node_Range =>
declare
N_Bare_Pragma_Node : constant Bare_Pragma_Node := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pragma_Node.Pragma_Node_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Pragma_Node.Pragma_Node_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Def_Range =>
declare
N_Bare_Protected_Def : constant Bare_Protected_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_Public_Part;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_Private_Part;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Range_Spec_Range =>
declare
N_Bare_Range_Spec : constant Bare_Range_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Range_Spec.Range_Spec_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Renaming_Clause_Range =>
declare
N_Bare_Renaming_Clause : constant Bare_Renaming_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Renaming_Clause.Renaming_Clause_F_Renamed_Object;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Select_When_Part_Range =>
declare
N_Bare_Select_When_Part : constant Bare_Select_When_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Select_When_Part.Select_When_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Select_When_Part.Select_When_Part_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Accept_Stmt_Range =>
declare
N_Bare_Accept_Stmt : constant Bare_Accept_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Body_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Entry_Index_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Params;
                            return;
                        when others => null;
                    end case;
case Ada_Accept_Stmt_Range (K) is
when Ada_Accept_Stmt_With_Stmts_Range =>
declare
N_Bare_Accept_Stmt_With_Stmts : constant Bare_Accept_Stmt_With_Stmts := N_Bare_Accept_Stmt;
begin
case Index is
                        when 4 =>
                            Result := N_Bare_Accept_Stmt_With_Stmts.Accept_Stmt_With_Stmts_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Accept_Stmt_With_Stmts.Accept_Stmt_With_Stmts_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Base_Loop_Stmt =>
declare
N_Bare_Base_Loop_Stmt : constant Bare_Base_Loop_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Begin_Block_Range =>
declare
N_Bare_Begin_Block : constant Bare_Begin_Block := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Begin_Block.Begin_Block_F_Stmts;
                            return;
                        when 2 =>
                            Result := N_Bare_Begin_Block.Begin_Block_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decl_Block_Range =>
declare
N_Bare_Decl_Block : constant Bare_Decl_Block := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_Decls;
                            return;
                        when 2 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Stmt_Range =>
declare
N_Bare_Case_Stmt : constant Bare_Case_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Pragmas;
                            return;
                        when 3 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Alternatives;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Extended_Return_Stmt_Range =>
declare
N_Bare_Extended_Return_Stmt : constant Bare_Extended_Return_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Extended_Return_Stmt.Extended_Return_Stmt_F_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Extended_Return_Stmt.Extended_Return_Stmt_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_If_Stmt_Range =>
declare
N_Bare_If_Stmt : constant Bare_If_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Then_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Alternatives;
                            return;
                        when 4 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Else_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Named_Stmt_Range =>
declare
N_Bare_Named_Stmt : constant Bare_Named_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Named_Stmt.Named_Stmt_F_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Named_Stmt.Named_Stmt_F_Stmt;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Select_Stmt_Range =>
declare
N_Bare_Select_Stmt : constant Bare_Select_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Guards;
                            return;
                        when 2 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Else_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Abort_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abort_Stmt_Range =>
declare
N_Bare_Abort_Stmt : constant Bare_Abort_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abort_Stmt.Abort_Stmt_F_Names;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Assign_Stmt_Range =>
declare
N_Bare_Assign_Stmt : constant Bare_Assign_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Assign_Stmt.Assign_Stmt_F_Dest;
                            return;
                        when 2 =>
                            Result := N_Bare_Assign_Stmt.Assign_Stmt_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Call_Stmt_Range =>
declare
N_Bare_Call_Stmt : constant Bare_Call_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Call_Stmt.Call_Stmt_F_Call;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Delay_Stmt_Range =>
declare
N_Bare_Delay_Stmt : constant Bare_Delay_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Delay_Stmt.Delay_Stmt_F_Has_Until;
                            return;
                        when 2 =>
                            Result := N_Bare_Delay_Stmt.Delay_Stmt_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exit_Stmt_Range =>
declare
N_Bare_Exit_Stmt : constant Bare_Exit_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exit_Stmt.Exit_Stmt_F_Loop_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Exit_Stmt.Exit_Stmt_F_Cond_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Goto_Stmt_Range =>
declare
N_Bare_Goto_Stmt : constant Bare_Goto_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Goto_Stmt.Goto_Stmt_F_Label_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Label_Range =>
declare
N_Bare_Label : constant Bare_Label := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Label.Label_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Raise_Stmt_Range =>
declare
N_Bare_Raise_Stmt : constant Bare_Raise_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Raise_Stmt.Raise_Stmt_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Raise_Stmt.Raise_Stmt_F_Error_Message;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Requeue_Stmt_Range =>
declare
N_Bare_Requeue_Stmt : constant Bare_Requeue_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Requeue_Stmt.Requeue_Stmt_F_Call_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Requeue_Stmt.Requeue_Stmt_F_Has_Abort;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Return_Stmt_Range =>
declare
N_Bare_Return_Stmt : constant Bare_Return_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Return_Stmt.Return_Stmt_F_Return_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subunit_Range =>
declare
N_Bare_Subunit : constant Bare_Subunit := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subunit.Subunit_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Subunit.Subunit_F_Body;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Def_Range =>
declare
N_Bare_Task_Def : constant Bare_Task_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Interfaces;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Public_Part;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Private_Part;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Def.Task_Def_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Access_Def =>
declare
N_Bare_Access_Def : constant Bare_Access_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Access_Def.Access_Def_F_Has_Not_Null;
                            return;
                        when others => null;
                    end case;
case Ada_Access_Def (K) is
when Ada_Access_To_Subp_Def_Range =>
declare
N_Bare_Access_To_Subp_Def : constant Bare_Access_To_Subp_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Access_To_Subp_Def.Access_To_Subp_Def_F_Has_Protected;
                            return;
                        when 3 =>
                            Result := N_Bare_Access_To_Subp_Def.Access_To_Subp_Def_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Type_Access_Def_Range =>
declare
N_Bare_Anonymous_Type_Access_Def : constant Bare_Anonymous_Type_Access_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Anonymous_Type_Access_Def.Anonymous_Type_Access_Def_F_Type_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Type_Access_Def_Range =>
declare
N_Bare_Type_Access_Def : constant Bare_Type_Access_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Has_All;
                            return;
                        when 3 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Has_Constant;
                            return;
                        when 4 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Subtype_Indication;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Array_Type_Def_Range =>
declare
N_Bare_Array_Type_Def : constant Bare_Array_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Array_Type_Def.Array_Type_Def_F_Indices;
                            return;
                        when 2 =>
                            Result := N_Bare_Array_Type_Def.Array_Type_Def_F_Component_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Derived_Type_Def_Range =>
declare
N_Bare_Derived_Type_Def : constant Bare_Derived_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Limited;
                            return;
                        when 3 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Synchronized;
                            return;
                        when 4 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Subtype_Indication;
                            return;
                        when 5 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Interfaces;
                            return;
                        when 6 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Record_Extension;
                            return;
                        when 7 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_With_Private;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Type_Def_Range =>
declare
N_Bare_Enum_Type_Def : constant Bare_Enum_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Type_Def.Enum_Type_Def_F_Enum_Literals;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Interface_Type_Def_Range =>
declare
N_Bare_Interface_Type_Def : constant Bare_Interface_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Interface_Type_Def.Interface_Type_Def_F_Interface_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Interface_Type_Def.Interface_Type_Def_F_Interfaces;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Mod_Int_Type_Def_Range =>
declare
N_Bare_Mod_Int_Type_Def : constant Bare_Mod_Int_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Mod_Int_Type_Def.Mod_Int_Type_Def_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Private_Type_Def_Range =>
declare
N_Bare_Private_Type_Def : constant Bare_Private_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Tagged;
                            return;
                        when 3 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Limited;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decimal_Fixed_Point_Def_Range =>
declare
N_Bare_Decimal_Fixed_Point_Def : constant Bare_Decimal_Fixed_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Digits;
                            return;
                        when 3 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Floating_Point_Def_Range =>
declare
N_Bare_Floating_Point_Def : constant Bare_Floating_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Floating_Point_Def.Floating_Point_Def_F_Num_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Floating_Point_Def.Floating_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Ordinary_Fixed_Point_Def_Range =>
declare
N_Bare_Ordinary_Fixed_Point_Def : constant Bare_Ordinary_Fixed_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Ordinary_Fixed_Point_Def.Ordinary_Fixed_Point_Def_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Ordinary_Fixed_Point_Def.Ordinary_Fixed_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Record_Type_Def_Range =>
declare
N_Bare_Record_Type_Def : constant Bare_Record_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Tagged;
                            return;
                        when 3 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Limited;
                            return;
                        when 4 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Record_Def;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Signed_Int_Type_Def_Range =>
declare
N_Bare_Signed_Int_Type_Def : constant Bare_Signed_Int_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Signed_Int_Type_Def.Signed_Int_Type_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Type_Range =>
declare
N_Bare_Anonymous_Type : constant Bare_Anonymous_Type := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Anonymous_Type.Anonymous_Type_F_Type_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subtype_Indication_Range =>
declare
N_Bare_Subtype_Indication : constant Bare_Subtype_Indication := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Has_Not_Null;
                            return;
                        when 2 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Constraint;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Type_Expr_Range =>
declare
N_Bare_Synthetic_Type_Expr : constant Bare_Synthetic_Type_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Type_Expr.Synthetic_Type_Expr_F_Target_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Unconstrained_Array_Index_Range =>
declare
N_Bare_Unconstrained_Array_Index : constant Bare_Unconstrained_Array_Index := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Unconstrained_Array_Index.Unconstrained_Array_Index_F_Subtype_Indication;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Use_Package_Clause_Range =>
declare
N_Bare_Use_Package_Clause : constant Bare_Use_Package_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Use_Package_Clause.Use_Package_Clause_F_Packages;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Use_Type_Clause_Range =>
declare
N_Bare_Use_Type_Clause : constant Bare_Use_Type_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Use_Type_Clause.Use_Type_Clause_F_Has_All;
                            return;
                        when 2 =>
                            Result := N_Bare_Use_Type_Clause.Use_Type_Clause_F_Types;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Value_Sequence_Range =>
declare
N_Bare_Value_Sequence : constant Bare_Value_Sequence := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Value_Sequence.Value_Sequence_F_Iter_Assoc;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Variant_Range =>
declare
N_Bare_Variant : constant Bare_Variant := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Variant.Variant_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Variant.Variant_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Variant_Part_Range =>
declare
N_Bare_Variant_Part : constant Bare_Variant_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Variant_Part.Variant_Part_F_Discr_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Variant_Part.Variant_Part_F_Variant;
                            return;
                        when others => null;
                    end case;
end;
when Ada_With_Clause_Range =>
declare
N_Bare_With_Clause : constant Bare_With_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Has_Limited;
                            return;
                        when 2 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Has_Private;
                            return;
                        when 3 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Packages;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
      --  Execution should reach this point iff nothing matched this index, so
      --  we must be out of bounds.
      Index_In_Bounds := False;
   end Get_Child;
   -----------
   -- Print --
   -----------
   procedure Print
     (Node        : Langkit_Support.Generic_API.Analysis.Lk_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "")
   is
      use Langkit_Support.Generic_API.Analysis;
      use Langkit_Support.Generic_API.Introspection;
      T : Type_Ref;
   begin
      if Node.Is_Null then
         Put_Line ("None");
         return;
      end if;
      T := Type_Of (Node);
      Put (Line_Prefix & Image (Node_Type_Repr_Name (T)));
      if Show_Slocs then
         Put ("[" & Image (Node.Sloc_Range) & "]");
      end if;
      if Node.Is_Incomplete then
         Put (" <<INCOMPLETE>>");
      end if;
      if Node.Is_Token_Node then
         Put_Line (": " & Image (Node.Text));
      elsif Is_List_Node (Node) then
         --  List nodes are displayed in a special way (they have no field)
         declare
            Count : constant Natural := Node.Children_Count;
            Child : Lk_Node;
         begin
            if Count = 0 then
               Put_Line (": <empty list>");
               return;
            end if;
            New_Line;
            for I in 1 .. Count loop
               Child := Node.Child (I);
               if not Child.Is_Null then
                  Print (Child, Show_Slocs, Line_Prefix & "|  ");
               end if;
            end loop;
         end;
      else
         --  This is for regular nodes: display each syntax field (i.e.
         --  non-property member).
         declare
            Attr_Prefix     : constant String := Line_Prefix & "|";
            Children_Prefix : constant String := Line_Prefix & "|  ";
            M_List          : constant Struct_Member_Ref_Array := Members (T);
            Child           : Lk_Node;
         begin
            New_Line;
            for M of M_List loop
               if not Is_Property (M) and then not Is_Null_For (M, T) then
                  Child := As_Node (Eval_Node_Member (Node, M));
                  Put (Attr_Prefix
                       & Image (Format_Name (Member_Name (M), Lower)) & ":");
                  if Child.Is_Null then
                     Put_Line (" <null>");
                  else
                     New_Line;
                     Print (Child, Show_Slocs, Children_Prefix);
                  end if;
               end if;
            end loop;
         end;
      end if;
   end Print;
   -----------
   -- Print --
   -----------
   procedure Print
     (Node        : Bare_Ada_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "")
   is
      Entity : constant Internal_Entity := (Node, No_Entity_Info);
   begin
      Print (To_Generic_Node (Entity), Show_Slocs, Line_Prefix);
   end Print;
   ------------
   -- Parent --
   ------------
   function Parent (Node : Bare_Ada_Node) return Bare_Ada_Node is
   begin
      return Node.Parent;
   end Parent;
   ------------------
   -- Stored_Token --
   ------------------
   function Stored_Token
     (Node  : Bare_Ada_Node;
      Token : Token_Reference) return Token_Index
   is
      Index : constant Token_Or_Trivia_Index := Get_Token_Index (Token);
   begin
      if Node.Unit.TDH'Access /= Get_Token_TDH (Token) then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "Cannot associate a token and a node from different analysis"
            & " units");
      elsif Index.Trivia /= No_Token_Index then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "A node cannot hold trivia");
      end if;
      return Index.Token;
   end Stored_Token;
   -------------------------
   -- Children_And_Trivia --
   -------------------------
   function Children_And_Trivia
     (Node : Bare_Ada_Node) return Bare_Children_Vector
   is
      Ret_Vec : Bare_Children_Vector;
      Ctx     : Internal_Context renames Node.Unit.Context;
      TDH     : Token_Data_Handler renames Node.Unit.TDH;
      procedure Append_Trivias (First, Last : Token_Index);
      --  Append all the trivias of tokens between indices First and Last to
      --  the returned vector.
      function Filter_Children
        (Parent : Bare_Ada_Node)
         return Internal_Bare_Ada_Node_Array;
      --  Return an array for all children in Parent that are not null
      --------------------
      -- Append_Trivias --
      --------------------
      procedure Append_Trivias (First, Last : Token_Index) is
      begin
         for I in First .. Last loop
            for D of Get_Trivias (TDH, I) loop
               Ret_Vec.Append
                 (Bare_Child_Record'
                    (Kind   => Trivia,
                     Trivia => Wrap_Token_Reference
                                 (Ctx, TDH'Access, (I, D))));
            end loop;
         end loop;
      end Append_Trivias;
      ---------------------
      -- Filter_Children --
      ---------------------
      function Filter_Children
        (Parent : Bare_Ada_Node)
         return Internal_Bare_Ada_Node_Array
      is
         Children : constant Internal_Bare_Ada_Node_Array :=
            Implementation.Children (Parent);
         Result   : Internal_Bare_Ada_Node_Array (Children'Range);
         Next     : Integer := Result'First;
      begin
         for I in Children'Range loop
            if Children (I) /= null then
               Result (Next) := Children (I);
               Next := Next + 1;
            end if;
         end loop;
         return Result (Result'First .. Next - 1);
      end Filter_Children;
      First_Child : constant Positive := 1;
      N_Children  : constant Internal_Bare_Ada_Node_Array :=
         Filter_Children (Node);
   begin
      if N_Children'Length > 0
        and then (Node.Token_Start_Index
                    /= N_Children (First_Child).Token_Start_Index)
      then
         Append_Trivias (Node.Token_Start_Index,
                         N_Children (First_Child).Token_Start_Index - 1);
      end if;
      --  Append each node to Ret_Vec, and append trivia that follow after each
      --  non-ghost nodes.
      for I in N_Children'Range loop
         Ret_Vec.Append (Bare_Child_Record'(Child, N_Children (I)));
         if not Is_Ghost (N_Children (I)) then
            Append_Trivias (N_Children (I).Token_End_Index,
                            (if I = N_Children'Last
                             then Node.Token_End_Index - 1
                             else N_Children (I + 1).Token_Start_Index - 1));
         end if;
      end loop;
      return Ret_Vec;
   end Children_And_Trivia;
   --------------
   -- Is_Ghost --
   --------------
   function Is_Ghost (Node : Bare_Ada_Node) return Boolean
   is (Node.Token_End_Index = No_Token_Index);
   -------------------
   -- Is_Incomplete --
   -------------------
   function Is_Incomplete (Node : Bare_Ada_Node) return Boolean
   is
      LGC : Bare_Ada_Node;
   begin
     if Is_List_Node (Node.Kind) then
        LGC := (if Last_Child_Index (Node) /= 0
                then Child (Node, Last_Child_Index (Node))
                else null);
        return LGC /= null and then Is_Incomplete (LGC);
      else
         return Node.Last_Attempted_Child > -1;
      end if;
   end;
   -----------------
   -- Token_Start --
   -----------------
   function Token_Start (Node : Bare_Ada_Node) return Token_Reference
   is (Token (Node, Node.Token_Start_Index));
   ---------------
   -- Token_End --
   ---------------
   function Token_End (Node : Bare_Ada_Node) return Token_Reference
   is
     (if Node.Token_End_Index = No_Token_Index
      then Token_Start (Node)
      else Token (Node, Node.Token_End_Index));
   -----------
   -- Token --
   -----------
   function Token
     (Node  : Bare_Ada_Node;
      Index : Token_Index) return Token_Reference
   is
      Unit    : constant Internal_Unit := Node.Unit;
      Context : constant Internal_Context := Unit.Context;
   begin
      return Wrap_Token_Reference
        (Context, Token_Data (Unit), (Index, No_Token_Index));
   end Token;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Bare_Ada_Node) return Boolean is
   begin
      --  Reject invalid inputs
      if Left /= null and Is_Synthetic (Left) then
         raise Property_Error with "left node is synthetic";
      elsif Right /= null and Is_Synthetic (Right) then
         raise Property_Error with "right node is synthetic";
      end if;
      --  Null nodes come first
      if Left = null then
         return Right /= null;
      elsif Right = null then
         return False;
      end if;
      --  So we have two non-null nodes. Sort by unit filename
      if Left.Unit < Right.Unit then
         return True;
      elsif Left.Unit /= Right.Unit then
         return False;
      end if;
      --  Both nodes come from the same unit: compare their token indexes
      if Left.Token_Start_Index < Right.Token_Start_Index then
         return True;
      elsif Left.Token_Start_Index > Right.Token_Start_Index then
         return False;
      else
         return Left.Token_End_Index < Right.Token_End_Index;
      end if;
   end "<";
   -------------
   -- Is_Null --
   -------------
   function Is_Null (Node : Bare_Ada_Node) return Boolean
   is (Node = null);
   ----------
   -- Kind --
   ----------
   function Kind (Node : Bare_Ada_Node) return Ada_Node_Kind_Type
   is (Node.Kind);
   -----------------
   -- Child_Index --
   -----------------
   function Child_Index (Node : Bare_Ada_Node) return Integer
   is
      N : Bare_Ada_Node := null;
   begin
      if Node.Parent = null then
         raise Property_Error with
            "Trying to get the child index of a root node";
      end if;
      for I in First_Child_Index (Node.Parent)
            .. Last_Child_Index (Node.Parent)
      loop
         N := Child (Node.Parent, I);
         if N = Node then
            return I - 1;
         end if;
      end loop;
      --  If we reach this point, then Node isn't a Child of Node.Parent. This
      --  is not supposed to happen.
      raise Program_Error;
   end Child_Index;
   -------------------
   -- Fetch_Sibling --
   -------------------
   function Fetch_Sibling
     (Node   : Bare_Ada_Node;
      Offset : Integer) return Bare_Ada_Node is
   begin
      --  Root nodes have no sibling: handle them now to avoid invalid requests
      --  in the code below.
      if Node.Parent = null then
         return null;
      end if;
      declare
         Node_Index : constant Positive := Child_Index (Node) + 1;
         --  Child_Index is 0-based, but the Child primitive expects a 1-based
         --  index.
         Sibling_Index : constant Integer := Node_Index + Offset;
      begin
         --  Child returns null for out-of-bound indexes
         return (if Sibling_Index >= 1
                 then Child (Node.Parent, Sibling_Index)
                 else null);
      end;
   end Fetch_Sibling;
   -------------------
   -- Fetch_Sibling --
   -------------------
   function Fetch_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info;
      Offset : Integer) return Internal_Entity
   is
      Sibling : constant Bare_Ada_Node := Fetch_Sibling (Node, Offset);
   begin
      --  Don't forget to clear entity info if the result is nul
      return (if Sibling = null
              then No_Entity
              else (Sibling, E_Info));
   end Fetch_Sibling;
   ----------------------
   -- Previous_Sibling --
   ----------------------
   function Previous_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, -1);
   end Previous_Sibling;
   ------------------
   -- Next_Sibling --
   ------------------
   function Next_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, 1);
   end Next_Sibling;
   ----------------------
   -- Compare_Metadata --
   ----------------------
   --  Deactivate "not referenced" warnings because if the metadata struct has
   --  no fields, formals and temporaries won't be referenced in the two
   --  following functions.
   pragma Warnings (Off, "referenced");
   function Compare_Metadata (L, R : Internal_Metadata) return Boolean is
   begin
            if L.Primitive /= R.Primitive then
               return False;
            end if;
            if L.Primitive_Real_Type /= R.Primitive_Real_Type then
               return False;
            end if;
      return True;
   end Compare_Metadata;
   ----------
   -- Hash --
   ----------
   function Hash (Self : Internal_Metadata) return Hash_Type is
      Ret : Hash_Type := Langkit_Support.Hashes.Initial_Hash;
   begin
               Ret := Combine (Ret, Hash (Self.Primitive));
               Ret := Combine (Ret, Hash (Self.Primitive_Real_Type));
      return Ret;
   end Hash;
   pragma Warnings (On, "referenced");
   -------------
   -- Combine --
   -------------
   function Combine
     (L, R : Internal_Metadata) return Internal_Metadata
   is
      Ret : Internal_Metadata := No_Metadata;
   begin
         Ret.Dottable_Subp := L.Dottable_Subp or R.Dottable_Subp;
         if L.Primitive = No_Bare_Ada_Node then
            Ret.Primitive := R.Primitive;
         elsif R.Primitive = No_Bare_Ada_Node then
            Ret.Primitive := L.Primitive;
         else
            raise Property_Error with "Wrong combine for metadata field";
         end if;
         if L.Primitive_Real_Type = No_Bare_Ada_Node then
            Ret.Primitive_Real_Type := R.Primitive_Real_Type;
         elsif R.Primitive_Real_Type = No_Bare_Ada_Node then
            Ret.Primitive_Real_Type := L.Primitive_Real_Type;
         else
            raise Property_Error with "Wrong combine for metadata field";
         end if;
      return Ret;
   end Combine;
   -------------------------------
   -- Create_Static_Lexical_Env --
   -------------------------------
   function Create_Static_Lexical_Env
     (Parent            : Lexical_Env;
      Node              : Bare_Ada_Node;
      Transitive_Parent : Boolean := False) return Lexical_Env
   is
      Unit : constant Internal_Unit :=
        (if Node = null then null else Node.Unit);
   begin
      return Result : Lexical_Env := Create_Lexical_Env
        (Parent, Node, Transitive_Parent, Convert_Unit (Unit))
      do
         if Unit /= null then
            Register_Destroyable (Unit, Unwrap (Result.Env));
         end if;
      end return;
   end Create_Static_Lexical_Env;
   ---------
   -- Get --
   ---------
   function Get
     (Self  : Bare_Ada_Node;
      A     : AST_Envs.Entity_Array;
      Index : Integer) return Internal_Entity
   is
      function Length (A : AST_Envs.Entity_Array) return Natural
      is (A'Length);
      function Get
        (A     : AST_Envs.Entity_Array;
         Index : Integer) return Internal_Entity
      is (A (Index + 1)); --  A is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Entity,
         Sequence_Type => AST_Envs.Entity_Array,
         Length        => Length,
         Get           => Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (A, Index, Result) then
         return Result;
      else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out-of-bounds array access");
      end if;
   end Get;
   -----------
   -- Group --
   -----------
   function Group
     (Envs   : Lexical_Env_Array_Access;
      Env_Md : Internal_Metadata := No_Metadata) return Lexical_Env
   is (Group (Lexical_Env_Array (Envs.Items), Env_Md));
      --
      --  Untyped wrappers for Bare_Ada_Node
      --
function Ada_Node_P_Resolve_Generic_Actual
  (E : Entity
  ) return Internal_Entity;
function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (E : Entity
  ) return Lexical_Env;
function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env;
function Ada_Node_P_Nested_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Base_Type_Decl
      --
function Base_Type_Decl_P_Dottable_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
      --
      --  Untyped wrappers for Bare_Type_Decl
      --
function Type_Decl_F_Direct_Primitive_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
function Type_Decl_P_Refined_Parent_Primitives_Env
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Body_Node
      --
function Body_Node_P_Subunit_Decl_Env
  (E : Entity
  ) return Lexical_Env;
function Body_Node_P_Body_Decl_Scope
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Package_Body
      --
function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Task_Body
      --
function Task_Body_P_Task_Type_Decl_Scope
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Generic_Instantiation
      --
function Generic_Instantiation_P_Instantiation_Bindings
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
function Generic_Instantiation_P_Parent_Instantiation_Env
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Name
      --
function Name_P_Name_Designated_Type_Env
  (E : Entity
  ) return Lexical_Env;
function Name_P_Use_Package_Name_Designated_Env
  (E : Entity
  ) return Lexical_Env;
   ------------------
   -- Children_Env --
   ------------------
   function Children_Env
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is (Rebind_Env (Node.Self_Env, E_Info));
   --------------
   -- Node_Env --
   --------------
   function Node_Env
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is
      function Get_Base_Env return Lexical_Env;
      --  Return the environment that we need to rebind before returning
      ------------------
      -- Get_Base_Env --
      ------------------
      function Get_Base_Env return Lexical_Env is
         pragma Warnings (Off, "referenced");
         function Get_Parent_Env return Lexical_Env;
         pragma Warnings (On, "referenced");
         --------------------
         -- Get_Parent_Env --
         --------------------
         function Get_Parent_Env return Lexical_Env is
            Parent : constant Lexical_Env := AST_Envs.Parent (Node.Self_Env);
         begin
            --  If Node is the root scope or the empty environment, Parent can
            --  be a wrapper around the null node. Turn this into the
            --  Empty_Env, as null envs are erroneous values in properties.
            return (if Unwrap (Parent) = null
                    then Empty_Env
                    else Parent);
         end Get_Parent_Env;
      begin
         return
           (if Node.Kind in Ada_Generic_Package_Internal | Ada_Package_Decl | Ada_Incomplete_Type_Decl | Ada_Incomplete_Formal_Type_Decl | Ada_Incomplete_Tagged_Type_Decl | Ada_Protected_Type_Decl | Ada_Task_Type_Decl | Ada_Single_Task_Type_Decl | Ada_Concrete_Type_Decl | Ada_Formal_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Abstract_Formal_Subp_Decl | Ada_Concrete_Formal_Subp_Decl | Ada_Subp_Decl | Ada_Entry_Decl | Ada_Enum_Literal_Decl | Ada_Synthetic_Char_Enum_Lit | Ada_Generic_Subp_Internal | Ada_Synthetic_Subp_Decl | Ada_Expr_Function | Ada_Null_Subp_Decl | Ada_Subp_Body | Ada_Subp_Renaming_Decl | Ada_Package_Body_Stub | Ada_Protected_Body_Stub | Ada_Subp_Body_Stub | Ada_Task_Body_Stub | Ada_Entry_Body | Ada_Package_Body | Ada_Protected_Body | Ada_Task_Body | Ada_Exception_Handler | Ada_Generic_Package_Decl | Ada_Generic_Subp_Decl | Ada_Generic_Package_Instantiation | Ada_Generic_Subp_Instantiation | Ada_Generic_Package_Renaming_Decl | Ada_Generic_Subp_Renaming_Decl | Ada_Package_Renaming_Decl | Ada_Single_Protected_Decl | Ada_Single_Task_Decl | Ada_Private_Part | Ada_Decl_Expr | Ada_Accept_Stmt | Ada_Accept_Stmt_With_Stmts | Ada_For_Loop_Stmt | Ada_Begin_Block | Ada_Decl_Block | Ada_Extended_Return_Stmt | Ada_Named_Stmt | Ada_Access_To_Subp_Def
            then Get_Parent_Env
            else Node.Self_Env);
      end Get_Base_Env;
      Base_Env : Lexical_Env := Get_Base_Env;
      Result   : constant Lexical_Env := Rebind_Env (Base_Env, E_Info);
   begin
      Dec_Ref (Base_Env);
      return Result;
   end Node_Env;
   ------------
   -- Parent --
   ------------
   function Parent
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      --  TODO: shed entity information as appropriate
      return (Node.Parent, E_Info);
   end Parent;
   -------------
   -- Parents --
   -------------
   function Parents
     (Node      : Bare_Ada_Node;
      With_Self : Boolean := True;
      E_Info    : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Parents : Bare_Ada_Node_Array_Access := Parents (Node, With_Self);
      Result       : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Parents.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Parents.Items'Range loop
         Result.Items (I) := (Bare_Parents.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Parents);
      return Result;
   end Parents;
   --------------
   -- Children --
   --------------
   function Children
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Children : Bare_Ada_Node_Array_Access := Children (Node);
      Result        : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Children.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Children.Items'Range loop
         Result.Items (I) := (Bare_Children.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Children);
      return Result;
   end Children;
   ---------------------
   -- New_Unit_String --
   ---------------------
   function New_Unit_String
     (Unit : Internal_Unit; Str : String) return String_Access
   is
      procedure Register_Destroyable_String is new Register_Destroyable_Gen
        (String, String_Access, Free);
   begin
      return Ret : String_Access := new String'(Str) do
         Register_Destroyable_String (Unit, Ret);
      end return;
   end New_Unit_String;
   --------------------------------
   -- Assign_Names_To_Logic_Vars --
   --------------------------------
   procedure Assign_Names_To_Logic_Vars (Node : Bare_Ada_Node) is
      pragma Warnings (Off, "referenced");
      procedure Assign
        (Node  : Bare_Ada_Node;
         LV    : in out Logic_Var_Record;
         Field : String);
      --  Assign a name to the LV logic variable. Node must be the node that
      --  owns LV, and Field must be the name of the field in Node that holds
      --  LV.
      ------------
      -- Assign --
      ------------
      procedure Assign
        (Node  : Bare_Ada_Node;
         LV    : in out Logic_Var_Record;
         Field : String) is
      begin
         LV.Dbg_Name :=
           new String'(Image (Short_Text_Image (Node)) & "." & Field);
      end Assign;
      K : constant Ada_Node_Kind_Type := Node.Kind;
      pragma Warnings (On, "referenced");
   begin
      case Ada_Ada_Node (K) is
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
Assign (N_Bare_Attribute_Ref,        N_Bare_Attribute_Ref.Attribute_Ref_R_Ref_Var,        "r_ref_var");
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
Assign (N_Bare_Call_Expr,        N_Bare_Call_Expr.Call_Expr_R_Called_Spec,        "r_called_spec");
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
Assign (N_Bare_Explicit_Deref,        N_Bare_Explicit_Deref.Explicit_Deref_R_Called_Spec,        "r_called_spec");
end;
when Ada_Reduce_Attribute_Ref_Range =>
declare
N_Bare_Reduce_Attribute_Ref : constant Bare_Reduce_Attribute_Ref := Node;
begin
Assign (N_Bare_Reduce_Attribute_Ref,        N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_R_Ref_Var,        "r_ref_var");
end;
when Ada_Target_Name_Range =>
declare
N_Bare_Target_Name : constant Bare_Target_Name := Node;
begin
Assign (N_Bare_Target_Name,        N_Bare_Target_Name.Target_Name_R_Ref_Var,        "r_ref_var");
end;
when Ada_Update_Attribute_Ref_Range =>
declare
N_Bare_Update_Attribute_Ref : constant Bare_Update_Attribute_Ref := Node;
begin
Assign (N_Bare_Update_Attribute_Ref,        N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_R_Ref_Var,        "r_ref_var");
end;
when others => null;
end case;
      for Child of Internal_Bare_Ada_Node_Array'(Children (Node)) loop
         if Child /= null then
            Assign_Names_To_Logic_Vars (Child);
         end if;
      end loop;
   end Assign_Names_To_Logic_Vars;
   ----------------
   -- Text_Image --
   ----------------
   function Text_Image (Ent : Internal_Entity) return Text_Type is
   begin
      if Ent.Node /= null then
         declare
            Node_Image : constant Text_Type := Short_Text_Image (Ent.Node);
         begin
            return
            (if Ent.Info.Rebindings /= null
             then "<| "
             & Node_Image (Node_Image'First + 1 .. Node_Image'Last - 1) & " "
             & AST_Envs.Text_Image (Ent.Info.Rebindings) & " |>"
             else Node_Image);
         end;
      else
         return "None";
      end if;
   end Text_Image;
   ---------------------
   -- Full_Sloc_Image --
   ---------------------
   function Full_Sloc_Image (Node : Bare_Ada_Node) return String_Type
   is
      Res      : constant Text_Type :=
        To_Text
          (Ada.Directories.Simple_Name
             (Get_Filename (Unit (Node))))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node)))) & ": ";
   begin
      return Create_String (Res);
   end Full_Sloc_Image;
   -----------
   -- Image --
   -----------
   function Image (Ent : Internal_Entity) return String is
      Result : constant Text_Type := Text_Image (Ent);
   begin
      return Image (Result);
   end Image;
   ---------------
   -- Can_Reach --
   ---------------
   function Can_Reach (El, From : Bare_Ada_Node) return Boolean is
   begin
      return Ada_Node_P_Can_Reach (El, From);
   end Can_Reach;
   -----------------
   -- Hash_Entity --
   -----------------
   function Hash_Entity (Self : Internal_Entity) return Hash_Type is
   begin
      return Combine
        ((Hash (Self.Node), Hash (Self.Info.Rebindings), Hash (Self.Info.Md)));
   end Hash_Entity;
   --------------------
   -- Compare_Entity --
   --------------------
   function Compare_Entity (Left, Right : Internal_Entity) return Boolean
   is
   begin
      return Left.Node = Right.Node
             and then Left.Info.Rebindings = Right.Info.Rebindings
             and then Compare_Metadata (Left.Info.Md, Right.Info.Md);
   end Compare_Entity;
   --------------------------------
   -- Create_Dynamic_Lexical_Env --
   --------------------------------
   function Create_Dynamic_Lexical_Env
     (Self              : Bare_Ada_Node;
      Assocs_Getter     : Inner_Env_Assocs_Resolver;
      Assoc_Resolver    : Entity_Resolver;
      Transitive_Parent : Boolean) return Lexical_Env
   is
      Unit : constant Internal_Unit := Self.Unit;
   begin
      --  This restriction is necessary to avoid relocation issues when
      --  Self.Self_Env is terminated.
      if Is_Foreign_Strict (Self.Self_Env, Self) then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "cannot create a dynamic lexical env when Self.Self_Env is"
            & " foreign");
      end if;
      return Result : constant Lexical_Env := Create_Dynamic_Lexical_Env
        (Parent            => Null_Lexical_Env,
         Node              => Self,
         Transitive_Parent => Transitive_Parent,
         Owner             => Convert_Unit (Unit),
         Assocs_Getter     => Assocs_Getter,
         Assoc_Resolver    => Assoc_Resolver)
      do
         --  Since dynamic lexical environments can only be created in lazy
         --  field initializers, it is fine to tie Result's lifetime to the
         --  its owning unit's lifetime.
         Register_Destroyable (Unit, Unwrap (Result));
      end return;
   end Create_Dynamic_Lexical_Env;
   procedure Destroy_Synthetic_Node (Node : in out Bare_Ada_Node);
   --  Helper for the Register_Destroyable above
   ------------
   -- Length --
   ------------
   function Length (Node : Bare_Ada_List) return Natural
   is (if Node = null then 0 else Children_Count (Node));
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (B : Boolean) return String is
      begin
         return (if B then "True" else "False");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (I : Integer) return String is
      begin
         return Integer'Image (I);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (S : Symbol_Type) return String is
      begin
         return (if S = null
                 then "None"
                 else Image (S.all, With_Quotes => True));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (C : Character_Type) return String is
         C_Str : constant Text_Type := (1 => C);
      begin
         return "'" & Image (C_Str) & "'";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (S : String_Type) return String is
      begin
         return Image (S.Content, With_Quotes => True);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Env : Lexical_Env) return String is
      begin
         case Env.Kind is
         when Static_Primary =>
            return "<LexicalEnv static-primary for "
                   & Trace_Image (Env_Node (Env)) & ">";
         when others =>
            return "<LexicalEnv synthetic>";
         end case;
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Env_Rebindings) return String is
      begin
         return Image (Text_Image (R));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Unit : Internal_Unit) return String is
      begin
         return "Internal_Unit (""" & Basename (Unit) & """)";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Eq : Logic_Equation) return String is
         pragma Unreferenced (Eq);
      begin
         return "<LogicEquation>";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Var : Logic_Var) return String is
         pragma Unreferenced (Var);
      begin
         return "<LogicVariable>";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (K : Analysis_Unit_Kind) return String is
      begin
         return Analysis_Unit_Kind'Image (K);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Self : Ref_Categories) return String is
         Result : Unbounded_String;
         First  : Boolean := True;
      begin
         Append (Result, "RefCategories(");
         for C in Ref_Category loop
            if Self (C) then
               if First then
                  First := False;
               else
                  Append (Result, ", ");
               end if;
               Append (Result, C'Image);
            end if;
         end loop;
         Append (Result, ")");
         return To_String (Result);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Metadata) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Dottable_Subp => " & Trace_Image (R.Dottable_Subp)
                        & ", "
                     & "Primitive => " & Trace_Image (R.Primitive)
                        & ", "
                     & "Primitive_Real_Type => " & Trace_Image (R.Primitive_Real_Type)
               & ")");
      end Trace_Image;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Info) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine ((Hash (R.Md), Hash (R.Rebindings), Hash (R.From_Rebound)));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Info) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Md => " & Trace_Image (R.Md)
                        & ", "
                     & "Rebindings => " & Trace_Image (R.Rebindings)
                        & ", "
                     & "From_Rebound => " & Trace_Image (R.From_Rebound)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity
        (Node : Bare_Ada_Node; Info : Internal_Entity_Info)
         return Internal_Entity is
      begin
         if Node = null then
            return No_Entity;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr
        (Node : Bare_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Expr is
      begin
         if Node = null then
            return No_Entity_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Aspect) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Exists => " & Trace_Image (R.Exists)
                        & ", "
                     & "Node => " & Trace_Image (R.Node)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Inherited => " & Trace_Image (R.Inherited)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Basic_Decl
        (Node : Bare_Basic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Basic_Decl) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Completion_Item) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Decl => " & Trace_Image (R.Decl)
                        & ", "
                     & "Is_Dot_Call => " & Trace_Image (R.Is_Dot_Call)
                        & ", "
                     & "Is_Visible => " & Trace_Image (R.Is_Visible)
                        & ", "
                     & "Weight => " & Trace_Image (R.Weight)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Designated_Env) is
      begin
               Inc_Ref (R.Direct_Env);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Designated_Env) is
      begin
               Dec_Ref (R.Direct_Env);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Designated_Env) return Boolean is
      begin
         return L.Kind = R.Kind and then L.Env_Name = R.Env_Name and then Equivalent (L.Direct_Env, R.Direct_Env);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Designated_Env) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Kind => " & Trace_Image (R.Kind)
                        & ", "
                     & "Env_Name => " & Trace_Image (R.Env_Name)
                        & ", "
                     & "Direct_Env => " & Trace_Image (R.Direct_Env)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Discrete_Range) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Low_Bound => " & Trace_Image (R.Low_Bound)
                        & ", "
                     & "High_Bound => " & Trace_Image (R.High_Bound)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Ada_List
        (Node : Bare_Ada_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ada_List is
      begin
         if Node = null then
            return No_Entity_Ada_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ada_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Ada_Node_List
        (Node : Bare_Ada_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ada_Node_List is
      begin
         if Node = null then
            return No_Entity_Ada_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ada_Node_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Alternatives_List
        (Node : Bare_Alternatives_List; Info : Internal_Entity_Info)
         return Internal_Entity_Alternatives_List is
      begin
         if Node = null then
            return No_Entity_Alternatives_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Alternatives_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Name
        (Node : Bare_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Name is
      begin
         if Node = null then
            return No_Entity_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Tok_Node
        (Node : Bare_Single_Tok_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Tok_Node is
      begin
         if Node = null then
            return No_Entity_Single_Tok_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Tok_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Id
        (Node : Bare_Base_Id; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Id is
      begin
         if Node = null then
            return No_Entity_Base_Id;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Id) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Identifier
        (Node : Bare_Identifier; Info : Internal_Entity_Info)
         return Internal_Entity_Identifier is
      begin
         if Node = null then
            return No_Entity_Identifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Identifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Discriminant_Values) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Discriminant => " & Trace_Image (R.Discriminant)
                        & ", "
                     & "Values => " & Trace_Image (R.Values)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Doc_Annotation) is
      begin
               Inc_Ref (R.Key);
               Inc_Ref (R.Value);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Doc_Annotation) is
      begin
               Dec_Ref (R.Key);
               Dec_Ref (R.Value);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Doc_Annotation) return Boolean is
      begin
         return Equivalent (L.Key, R.Key) and then Equivalent (L.Value, R.Value);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Doc_Annotation) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Abort_Node
        (Node : Bare_Abort_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Node is
      begin
         if Node = null then
            return No_Entity_Abort_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Absent
        (Node : Bare_Abort_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Absent is
      begin
         if Node = null then
            return No_Entity_Abort_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Present
        (Node : Bare_Abort_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Present is
      begin
         if Node = null then
            return No_Entity_Abort_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Stmt
        (Node : Bare_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Stmt is
      begin
         if Node = null then
            return No_Entity_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Simple_Stmt
        (Node : Bare_Simple_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Simple_Stmt is
      begin
         if Node = null then
            return No_Entity_Simple_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Simple_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Stmt
        (Node : Bare_Abort_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Stmt is
      begin
         if Node = null then
            return No_Entity_Abort_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Node
        (Node : Bare_Abstract_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Node is
      begin
         if Node = null then
            return No_Entity_Abstract_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Absent
        (Node : Bare_Abstract_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Absent is
      begin
         if Node = null then
            return No_Entity_Abstract_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Subp_Decl
        (Node : Bare_Basic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Classic_Subp_Decl
        (Node : Bare_Classic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Classic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Classic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Classic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Subp_Decl
        (Node : Bare_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Formal_Subp_Decl
        (Node : Bare_Abstract_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Present
        (Node : Bare_Abstract_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Present is
      begin
         if Node = null then
            return No_Entity_Abstract_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl
        (Node : Bare_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl_Expr
        (Node : Bare_Abstract_State_Decl_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl_Expr is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl_List
        (Node : Bare_Abstract_State_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl_List is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Subp_Decl
        (Node : Bare_Abstract_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Composite_Stmt
        (Node : Bare_Composite_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Composite_Stmt is
      begin
         if Node = null then
            return No_Entity_Composite_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Composite_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt
        (Node : Bare_Accept_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Accept_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Body_Node
        (Node : Bare_Body_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Body_Node is
      begin
         if Node = null then
            return No_Entity_Body_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Body_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt_Body
        (Node : Bare_Accept_Stmt_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt_Body is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Accept_Stmt_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node : Bare_Accept_Stmt_With_Stmts; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt_With_Stmts is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt_With_Stmts;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Accept_Stmt_With_Stmts) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Def
        (Node : Bare_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Def is
      begin
         if Node = null then
            return No_Entity_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Access_Def
        (Node : Bare_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Access_Def is
      begin
         if Node = null then
            return No_Entity_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Access_To_Subp_Def
        (Node : Bare_Access_To_Subp_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Access_To_Subp_Def is
      begin
         if Node = null then
            return No_Entity_Access_To_Subp_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Access_To_Subp_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Aggregate
        (Node : Bare_Base_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Aggregate is
      begin
         if Node = null then
            return No_Entity_Base_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aggregate
        (Node : Bare_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Aggregate is
      begin
         if Node = null then
            return No_Entity_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Assoc
        (Node : Bare_Basic_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Assoc is
      begin
         if Node = null then
            return No_Entity_Basic_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aggregate_Assoc
        (Node : Bare_Aggregate_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Aggregate_Assoc is
      begin
         if Node = null then
            return No_Entity_Aggregate_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aggregate_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Node
        (Node : Bare_Aliased_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Node is
      begin
         if Node = null then
            return No_Entity_Aliased_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Absent
        (Node : Bare_Aliased_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Absent is
      begin
         if Node = null then
            return No_Entity_Aliased_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Present
        (Node : Bare_Aliased_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Present is
      begin
         if Node = null then
            return No_Entity_Aliased_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Node
        (Node : Bare_All_Node; Info : Internal_Entity_Info)
         return Internal_Entity_All_Node is
      begin
         if Node = null then
            return No_Entity_All_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Absent
        (Node : Bare_All_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_All_Absent is
      begin
         if Node = null then
            return No_Entity_All_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Present
        (Node : Bare_All_Present; Info : Internal_Entity_Info)
         return Internal_Entity_All_Present is
      begin
         if Node = null then
            return No_Entity_All_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Allocator
        (Node : Bare_Allocator; Info : Internal_Entity_Info)
         return Internal_Entity_Allocator is
      begin
         if Node = null then
            return No_Entity_Allocator;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Allocator) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Expr_Decl
        (Node : Bare_Anonymous_Expr_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Expr_Decl is
      begin
         if Node = null then
            return No_Entity_Anonymous_Expr_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Expr_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Expr
        (Node : Bare_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type
        (Node : Bare_Anonymous_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Type_Access_Def
        (Node : Bare_Base_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Base_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type_Access_Def
        (Node : Bare_Anonymous_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Type_Decl
        (Node : Bare_Base_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Base_Type_Decl) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Decl
        (Node : Bare_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type_Decl
        (Node : Bare_Anonymous_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Indices
        (Node : Bare_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Type_Def
        (Node : Bare_Array_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Type_Def is
      begin
         if Node = null then
            return No_Entity_Array_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Array_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Assoc
        (Node : Bare_Aspect_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Assoc is
      begin
         if Node = null then
            return No_Entity_Aspect_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Assoc_List
        (Node : Bare_Aspect_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Aspect_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Clause
        (Node : Bare_Aspect_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Clause is
      begin
         if Node = null then
            return No_Entity_Aspect_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Spec
        (Node : Bare_Aspect_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Spec is
      begin
         if Node = null then
            return No_Entity_Aspect_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Assign_Stmt
        (Node : Bare_Assign_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Assign_Stmt is
      begin
         if Node = null then
            return No_Entity_Assign_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Assign_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Assoc_List
        (Node : Bare_Basic_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Basic_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Assoc_List
        (Node : Bare_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_At_Clause
        (Node : Bare_At_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_At_Clause is
      begin
         if Node = null then
            return No_Entity_At_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_At_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Attribute_Def_Clause
        (Node : Bare_Attribute_Def_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Attribute_Def_Clause is
      begin
         if Node = null then
            return No_Entity_Attribute_Def_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Attribute_Def_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Attribute_Ref
        (Node : Bare_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Attribute_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Assoc
        (Node : Bare_Base_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Assoc is
      begin
         if Node = null then
            return No_Entity_Base_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Assoc_List
        (Node : Bare_Base_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Base_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Formal_Param_Decl
        (Node : Bare_Base_Formal_Param_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Formal_Param_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Formal_Param_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Formal_Param_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Formal_Param_Holder
        (Node : Bare_Base_Formal_Param_Holder; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Formal_Param_Holder is
      begin
         if Node = null then
            return No_Entity_Base_Formal_Param_Holder;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Formal_Param_Holder) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Loop_Stmt
        (Node : Bare_Base_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_Base_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Package_Decl
        (Node : Bare_Base_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Record_Def
        (Node : Bare_Base_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Record_Def is
      begin
         if Node = null then
            return No_Entity_Base_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subp_Body
        (Node : Bare_Base_Subp_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subp_Body is
      begin
         if Node = null then
            return No_Entity_Base_Subp_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subp_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subp_Spec
        (Node : Bare_Base_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Base_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subtype_Decl
        (Node : Bare_Base_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Decl_List
        (Node : Bare_Basic_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Decl_List is
      begin
         if Node = null then
            return No_Entity_Basic_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Block_Stmt
        (Node : Bare_Block_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Block_Stmt is
      begin
         if Node = null then
            return No_Entity_Block_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Block_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Begin_Block
        (Node : Bare_Begin_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Begin_Block is
      begin
         if Node = null then
            return No_Entity_Begin_Block;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Begin_Block) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bin_Op
        (Node : Bare_Bin_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Bin_Op is
      begin
         if Node = null then
            return No_Entity_Bin_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bin_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Body_Stub
        (Node : Bare_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Box_Expr
        (Node : Bare_Box_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Box_Expr is
      begin
         if Node = null then
            return No_Entity_Box_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Box_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bracket_Aggregate
        (Node : Bare_Bracket_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Bracket_Aggregate is
      begin
         if Node = null then
            return No_Entity_Bracket_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bracket_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delta_Aggregate
        (Node : Bare_Delta_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Delta_Aggregate is
      begin
         if Node = null then
            return No_Entity_Delta_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delta_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bracket_Delta_Aggregate
        (Node : Bare_Bracket_Delta_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Bracket_Delta_Aggregate is
      begin
         if Node = null then
            return No_Entity_Bracket_Delta_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bracket_Delta_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Call_Expr
        (Node : Bare_Call_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Expr is
      begin
         if Node = null then
            return No_Entity_Call_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Call_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Call_Stmt
        (Node : Bare_Call_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Stmt is
      begin
         if Node = null then
            return No_Entity_Call_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Call_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Cond_Expr
        (Node : Bare_Cond_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Cond_Expr is
      begin
         if Node = null then
            return No_Entity_Cond_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Cond_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr
        (Node : Bare_Case_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr is
      begin
         if Node = null then
            return No_Entity_Case_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr_Alternative
        (Node : Bare_Case_Expr_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr_Alternative is
      begin
         if Node = null then
            return No_Entity_Case_Expr_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr_Alternative_List
        (Node : Bare_Case_Expr_Alternative_List; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr_Alternative_List is
      begin
         if Node = null then
            return No_Entity_Case_Expr_Alternative_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr_Alternative_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt
        (Node : Bare_Case_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt is
      begin
         if Node = null then
            return No_Entity_Case_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt_Alternative
        (Node : Bare_Case_Stmt_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt_Alternative is
      begin
         if Node = null then
            return No_Entity_Case_Stmt_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt_Alternative_List
        (Node : Bare_Case_Stmt_Alternative_List; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt_Alternative_List is
      begin
         if Node = null then
            return No_Entity_Case_Stmt_Alternative_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt_Alternative_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Char_Literal
        (Node : Bare_Char_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Char_Literal is
      begin
         if Node = null then
            return No_Entity_Char_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Char_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Classwide_Type_Decl
        (Node : Bare_Classwide_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Classwide_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Classwide_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Classwide_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Compilation_Unit
        (Node : Bare_Compilation_Unit; Info : Internal_Entity_Info)
         return Internal_Entity_Compilation_Unit is
      begin
         if Node = null then
            return No_Entity_Compilation_Unit;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Compilation_Unit) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Compilation_Unit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Compilation_Unit_List
        (Node : Bare_Compilation_Unit_List; Info : Internal_Entity_Info)
         return Internal_Entity_Compilation_Unit_List is
      begin
         if Node = null then
            return No_Entity_Compilation_Unit_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Compilation_Unit_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Clause
        (Node : Bare_Component_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Clause is
      begin
         if Node = null then
            return No_Entity_Component_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Decl
        (Node : Bare_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Def
        (Node : Bare_Component_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Def is
      begin
         if Node = null then
            return No_Entity_Component_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_List
        (Node : Bare_Component_List; Info : Internal_Entity_Info)
         return Internal_Entity_Component_List is
      begin
         if Node = null then
            return No_Entity_Component_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constraint
        (Node : Bare_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Constraint is
      begin
         if Node = null then
            return No_Entity_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Composite_Constraint
        (Node : Bare_Composite_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Composite_Constraint is
      begin
         if Node = null then
            return No_Entity_Composite_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Composite_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Composite_Constraint_Assoc
        (Node : Bare_Composite_Constraint_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Composite_Constraint_Assoc is
      begin
         if Node = null then
            return No_Entity_Composite_Constraint_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Composite_Constraint_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concat_Op
        (Node : Bare_Concat_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Concat_Op is
      begin
         if Node = null then
            return No_Entity_Concat_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concat_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concat_Operand
        (Node : Bare_Concat_Operand; Info : Internal_Entity_Info)
         return Internal_Entity_Concat_Operand is
      begin
         if Node = null then
            return No_Entity_Concat_Operand;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concat_Operand) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concat_Operand_List
        (Node : Bare_Concat_Operand_List; Info : Internal_Entity_Info)
         return Internal_Entity_Concat_Operand_List is
      begin
         if Node = null then
            return No_Entity_Concat_Operand_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concat_Operand_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concrete_Formal_Subp_Decl
        (Node : Bare_Concrete_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Concrete_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Concrete_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concrete_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concrete_Type_Decl
        (Node : Bare_Concrete_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Concrete_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Concrete_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concrete_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Node
        (Node : Bare_Constant_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Node is
      begin
         if Node = null then
            return No_Entity_Constant_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Absent
        (Node : Bare_Constant_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Absent is
      begin
         if Node = null then
            return No_Entity_Constant_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Present
        (Node : Bare_Constant_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Present is
      begin
         if Node = null then
            return No_Entity_Constant_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constrained_Array_Indices
        (Node : Bare_Constrained_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Constrained_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Constrained_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constrained_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subtype_Indication
        (Node : Bare_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constrained_Subtype_Indication
        (Node : Bare_Constrained_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Constrained_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Constrained_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constrained_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constraint_List
        (Node : Bare_Constraint_List; Info : Internal_Entity_Info)
         return Internal_Entity_Constraint_List is
      begin
         if Node = null then
            return No_Entity_Constraint_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constraint_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Case_Assoc
        (Node : Bare_Contract_Case_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Case_Assoc is
      begin
         if Node = null then
            return No_Entity_Contract_Case_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Case_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Case_Assoc_List
        (Node : Bare_Contract_Case_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Case_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Contract_Case_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Case_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Cases
        (Node : Bare_Contract_Cases; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Cases is
      begin
         if Node = null then
            return No_Entity_Contract_Cases;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Cases) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Real_Type_Def
        (Node : Bare_Real_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Real_Type_Def is
      begin
         if Node = null then
            return No_Entity_Real_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Real_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decimal_Fixed_Point_Def
        (Node : Bare_Decimal_Fixed_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Decimal_Fixed_Point_Def is
      begin
         if Node = null then
            return No_Entity_Decimal_Fixed_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decimal_Fixed_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_Block
        (Node : Bare_Decl_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Block is
      begin
         if Node = null then
            return No_Entity_Decl_Block;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Block) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_Expr
        (Node : Bare_Decl_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Expr is
      begin
         if Node = null then
            return No_Entity_Decl_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_List
        (Node : Bare_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_List is
      begin
         if Node = null then
            return No_Entity_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Declarative_Part
        (Node : Bare_Declarative_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Declarative_Part is
      begin
         if Node = null then
            return No_Entity_Declarative_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Declarative_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Defining_Name
        (Node : Bare_Defining_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Defining_Name is
      begin
         if Node = null then
            return No_Entity_Defining_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Defining_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Defining_Name_List
        (Node : Bare_Defining_Name_List; Info : Internal_Entity_Info)
         return Internal_Entity_Defining_Name_List is
      begin
         if Node = null then
            return No_Entity_Defining_Name_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Defining_Name_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delay_Stmt
        (Node : Bare_Delay_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Delay_Stmt is
      begin
         if Node = null then
            return No_Entity_Delay_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delay_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delta_Constraint
        (Node : Bare_Delta_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Delta_Constraint is
      begin
         if Node = null then
            return No_Entity_Delta_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delta_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Derived_Type_Def
        (Node : Bare_Derived_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Derived_Type_Def is
      begin
         if Node = null then
            return No_Entity_Derived_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Derived_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Digits_Constraint
        (Node : Bare_Digits_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Digits_Constraint is
      begin
         if Node = null then
            return No_Entity_Digits_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Digits_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Base_Subtype_Decl
        (Node : Bare_Discrete_Base_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Base_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Discrete_Base_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Base_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Subtype_Indication
        (Node : Bare_Discrete_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Discrete_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Subtype_Name
        (Node : Bare_Discrete_Subtype_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Subtype_Name is
      begin
         if Node = null then
            return No_Entity_Discrete_Subtype_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Subtype_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Identifier_List
        (Node : Bare_Identifier_List; Info : Internal_Entity_Info)
         return Internal_Entity_Identifier_List is
      begin
         if Node = null then
            return No_Entity_Identifier_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Identifier_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Choice_List
        (Node : Bare_Discriminant_Choice_List; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Choice_List is
      begin
         if Node = null then
            return No_Entity_Discriminant_Choice_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Choice_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Part
        (Node : Bare_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Spec
        (Node : Bare_Discriminant_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Spec is
      begin
         if Node = null then
            return No_Entity_Discriminant_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Spec_List
        (Node : Bare_Discriminant_Spec_List; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Spec_List is
      begin
         if Node = null then
            return No_Entity_Discriminant_Spec_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Spec_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Dotted_Name
        (Node : Bare_Dotted_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Dotted_Name is
      begin
         if Node = null then
            return No_Entity_Dotted_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Dotted_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Expr_Part
        (Node : Bare_Elsif_Expr_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Expr_Part is
      begin
         if Node = null then
            return No_Entity_Elsif_Expr_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Expr_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Expr_Part_List
        (Node : Bare_Elsif_Expr_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Expr_Part_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Expr_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Expr_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Stmt_Part
        (Node : Bare_Elsif_Stmt_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Stmt_Part is
      begin
         if Node = null then
            return No_Entity_Elsif_Stmt_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Stmt_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Stmt_Part_List
        (Node : Bare_Elsif_Stmt_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Stmt_Part_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Stmt_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Stmt_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_End_Name
        (Node : Bare_End_Name; Info : Internal_Entity_Info)
         return Internal_Entity_End_Name is
      begin
         if Node = null then
            return No_Entity_End_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_End_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Body
        (Node : Bare_Entry_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Body is
      begin
         if Node = null then
            return No_Entity_Entry_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Completion_Formal_Params
        (Node : Bare_Entry_Completion_Formal_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Completion_Formal_Params is
      begin
         if Node = null then
            return No_Entity_Entry_Completion_Formal_Params;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Completion_Formal_Params) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Decl
        (Node : Bare_Entry_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Decl is
      begin
         if Node = null then
            return No_Entity_Entry_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Index_Spec
        (Node : Bare_Entry_Index_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Index_Spec is
      begin
         if Node = null then
            return No_Entity_Entry_Index_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Index_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Spec
        (Node : Bare_Entry_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Spec is
      begin
         if Node = null then
            return No_Entity_Entry_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Lit_Synth_Type_Expr
        (Node : Bare_Enum_Lit_Synth_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Lit_Synth_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Enum_Lit_Synth_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Lit_Synth_Type_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Literal_Decl
        (Node : Bare_Enum_Literal_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Literal_Decl is
      begin
         if Node = null then
            return No_Entity_Enum_Literal_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Literal_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Literal_Decl_List
        (Node : Bare_Enum_Literal_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Literal_Decl_List is
      begin
         if Node = null then
            return No_Entity_Enum_Literal_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Literal_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Rep_Clause
        (Node : Bare_Enum_Rep_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Rep_Clause is
      begin
         if Node = null then
            return No_Entity_Enum_Rep_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Rep_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Subp_Spec
        (Node : Bare_Enum_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Enum_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Type_Def
        (Node : Bare_Enum_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Type_Def is
      begin
         if Node = null then
            return No_Entity_Enum_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Error_Decl
        (Node : Bare_Error_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Error_Decl is
      begin
         if Node = null then
            return No_Entity_Error_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Error_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Error_Stmt
        (Node : Bare_Error_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Error_Stmt is
      begin
         if Node = null then
            return No_Entity_Error_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Error_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exception_Decl
        (Node : Bare_Exception_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Exception_Decl is
      begin
         if Node = null then
            return No_Entity_Exception_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exception_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exception_Handler
        (Node : Bare_Exception_Handler; Info : Internal_Entity_Info)
         return Internal_Entity_Exception_Handler is
      begin
         if Node = null then
            return No_Entity_Exception_Handler;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exception_Handler) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exit_Stmt
        (Node : Bare_Exit_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Exit_Stmt is
      begin
         if Node = null then
            return No_Entity_Exit_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exit_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Explicit_Deref
        (Node : Bare_Explicit_Deref; Info : Internal_Entity_Info)
         return Internal_Entity_Explicit_Deref is
      begin
         if Node = null then
            return No_Entity_Explicit_Deref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Explicit_Deref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_List
        (Node : Bare_Expr_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_List is
      begin
         if Node = null then
            return No_Entity_Expr_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_Alternatives_List
        (Node : Bare_Expr_Alternatives_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_Alternatives_List is
      begin
         if Node = null then
            return No_Entity_Expr_Alternatives_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_Alternatives_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_Function
        (Node : Bare_Expr_Function; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_Function is
      begin
         if Node = null then
            return No_Entity_Expr_Function;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_Function) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Extended_Return_Stmt
        (Node : Bare_Extended_Return_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Extended_Return_Stmt is
      begin
         if Node = null then
            return No_Entity_Extended_Return_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Extended_Return_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Object_Decl
        (Node : Bare_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Object_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Extended_Return_Stmt_Object_Decl
        (Node : Bare_Extended_Return_Stmt_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Extended_Return_Stmt_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Extended_Return_Stmt_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Extended_Return_Stmt_Object_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Floating_Point_Def
        (Node : Bare_Floating_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Floating_Point_Def is
      begin
         if Node = null then
            return No_Entity_Floating_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Floating_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Loop_Spec
        (Node : Bare_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Spec
        (Node : Bare_For_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_For_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Stmt
        (Node : Bare_For_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_For_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Var_Decl
        (Node : Bare_For_Loop_Var_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Var_Decl is
      begin
         if Node = null then
            return No_Entity_For_Loop_Var_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Var_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Discrete_Type_Def
        (Node : Bare_Formal_Discrete_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Discrete_Type_Def is
      begin
         if Node = null then
            return No_Entity_Formal_Discrete_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Formal_Discrete_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Type_Decl
        (Node : Bare_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Formal_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Decl
        (Node : Bare_Generic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal
        (Node : Bare_Generic_Formal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal is
      begin
         if Node = null then
            return No_Entity_Generic_Formal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Obj_Decl
        (Node : Bare_Generic_Formal_Obj_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Obj_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Obj_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Obj_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Package
        (Node : Bare_Generic_Formal_Package; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Package is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Package;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Package) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Part
        (Node : Bare_Generic_Formal_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Part is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node : Bare_Generic_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node : Bare_Generic_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Instantiation
        (Node : Bare_Generic_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Decl
        (Node : Bare_Generic_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Instantiation
        (Node : Bare_Generic_Package_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Internal
        (Node : Bare_Generic_Package_Internal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Internal is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Internal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Internal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Renaming_Decl
        (Node : Bare_Generic_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Renaming_Decl
        (Node : Bare_Generic_Package_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Decl
        (Node : Bare_Generic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Instantiation
        (Node : Bare_Generic_Subp_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Internal
        (Node : Bare_Generic_Subp_Internal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Internal is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Internal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Internal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Renaming_Decl
        (Node : Bare_Generic_Subp_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Goto_Stmt
        (Node : Bare_Goto_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Goto_Stmt is
      begin
         if Node = null then
            return No_Entity_Goto_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Goto_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Handled_Stmts
        (Node : Bare_Handled_Stmts; Info : Internal_Entity_Info)
         return Internal_Entity_Handled_Stmts is
      begin
         if Node = null then
            return No_Entity_Handled_Stmts;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Handled_Stmts) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_If_Expr
        (Node : Bare_If_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_If_Expr is
      begin
         if Node = null then
            return No_Entity_If_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_If_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_If_Stmt
        (Node : Bare_If_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_If_Stmt is
      begin
         if Node = null then
            return No_Entity_If_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_If_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Type_Decl
        (Node : Bare_Incomplete_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Incomplete_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Formal_Type_Decl
        (Node : Bare_Incomplete_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Incomplete_Formal_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Tagged_Type_Decl
        (Node : Bare_Incomplete_Tagged_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Tagged_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Tagged_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Incomplete_Tagged_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Num_Literal
        (Node : Bare_Num_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Num_Literal is
      begin
         if Node = null then
            return No_Entity_Num_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Num_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Int_Literal
        (Node : Bare_Int_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Int_Literal is
      begin
         if Node = null then
            return No_Entity_Int_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Int_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind
        (Node : Bare_Interface_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind is
      begin
         if Node = null then
            return No_Entity_Interface_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Limited
        (Node : Bare_Interface_Kind_Limited; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Limited is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Limited;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Limited) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Protected
        (Node : Bare_Interface_Kind_Protected; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Protected is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Protected;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Protected) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Synchronized
        (Node : Bare_Interface_Kind_Synchronized; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Synchronized is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Synchronized;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Synchronized) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Task
        (Node : Bare_Interface_Kind_Task; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Task is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Task;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Task) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Type_Def
        (Node : Bare_Interface_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Type_Def is
      begin
         if Node = null then
            return No_Entity_Interface_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type
        (Node : Bare_Iter_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type is
      begin
         if Node = null then
            return No_Entity_Iter_Type;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type_In
        (Node : Bare_Iter_Type_In; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type_In is
      begin
         if Node = null then
            return No_Entity_Iter_Type_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type_Of
        (Node : Bare_Iter_Type_Of; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type_Of is
      begin
         if Node = null then
            return No_Entity_Iter_Type_Of;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type_Of) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iterated_Assoc
        (Node : Bare_Iterated_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Iterated_Assoc is
      begin
         if Node = null then
            return No_Entity_Iterated_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iterated_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Known_Discriminant_Part
        (Node : Bare_Known_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Known_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Known_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Known_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Label
        (Node : Bare_Label; Info : Internal_Entity_Info)
         return Internal_Entity_Label is
      begin
         if Node = null then
            return No_Entity_Label;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Label) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Label_Decl
        (Node : Bare_Label_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Label_Decl is
      begin
         if Node = null then
            return No_Entity_Label_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Label_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Library_Item
        (Node : Bare_Library_Item; Info : Internal_Entity_Info)
         return Internal_Entity_Library_Item is
      begin
         if Node = null then
            return No_Entity_Library_Item;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Library_Item) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Node
        (Node : Bare_Limited_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Node is
      begin
         if Node = null then
            return No_Entity_Limited_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Absent
        (Node : Bare_Limited_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Absent is
      begin
         if Node = null then
            return No_Entity_Limited_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Present
        (Node : Bare_Limited_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Present is
      begin
         if Node = null then
            return No_Entity_Limited_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Loop_Stmt
        (Node : Bare_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Membership_Expr
        (Node : Bare_Membership_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Membership_Expr is
      begin
         if Node = null then
            return No_Entity_Membership_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Membership_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mod_Int_Type_Def
        (Node : Bare_Mod_Int_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Mod_Int_Type_Def is
      begin
         if Node = null then
            return No_Entity_Mod_Int_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mod_Int_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode
        (Node : Bare_Mode; Info : Internal_Entity_Info)
         return Internal_Entity_Mode is
      begin
         if Node = null then
            return No_Entity_Mode;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_Default
        (Node : Bare_Mode_Default; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_Default is
      begin
         if Node = null then
            return No_Entity_Mode_Default;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_Default) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_In
        (Node : Bare_Mode_In; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_In is
      begin
         if Node = null then
            return No_Entity_Mode_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_In_Out
        (Node : Bare_Mode_In_Out; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_In_Out is
      begin
         if Node = null then
            return No_Entity_Mode_In_Out;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_In_Out) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_Out
        (Node : Bare_Mode_Out; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_Out is
      begin
         if Node = null then
            return No_Entity_Mode_Out;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_Out) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Multi_Abstract_State_Decl
        (Node : Bare_Multi_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Multi_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Multi_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Multi_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Multi_Dim_Array_Assoc
        (Node : Bare_Multi_Dim_Array_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Multi_Dim_Array_Assoc is
      begin
         if Node = null then
            return No_Entity_Multi_Dim_Array_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Multi_Dim_Array_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Name_List
        (Node : Bare_Name_List; Info : Internal_Entity_Info)
         return Internal_Entity_Name_List is
      begin
         if Node = null then
            return No_Entity_Name_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Name_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Named_Stmt
        (Node : Bare_Named_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Stmt is
      begin
         if Node = null then
            return No_Entity_Named_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Named_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Named_Stmt_Decl
        (Node : Bare_Named_Stmt_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Stmt_Decl is
      begin
         if Node = null then
            return No_Entity_Named_Stmt_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Named_Stmt_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_No_Type_Object_Renaming_Decl
        (Node : Bare_No_Type_Object_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_No_Type_Object_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_No_Type_Object_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_No_Type_Object_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null
        (Node : Bare_Not_Null; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null is
      begin
         if Node = null then
            return No_Entity_Not_Null;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null_Absent
        (Node : Bare_Not_Null_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null_Absent is
      begin
         if Node = null then
            return No_Entity_Not_Null_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null_Present
        (Node : Bare_Not_Null_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null_Present is
      begin
         if Node = null then
            return No_Entity_Not_Null_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Component_Decl
        (Node : Bare_Null_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Null_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Component_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Literal
        (Node : Bare_Null_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Literal is
      begin
         if Node = null then
            return No_Entity_Null_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Record_Aggregate
        (Node : Bare_Null_Record_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Record_Aggregate is
      begin
         if Node = null then
            return No_Entity_Null_Record_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Record_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Record_Def
        (Node : Bare_Null_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Record_Def is
      begin
         if Node = null then
            return No_Entity_Null_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Stmt
        (Node : Bare_Null_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Stmt is
      begin
         if Node = null then
            return No_Entity_Null_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Subp_Decl
        (Node : Bare_Null_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Null_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Number_Decl
        (Node : Bare_Number_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Number_Decl is
      begin
         if Node = null then
            return No_Entity_Number_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Number_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op
        (Node : Bare_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Op is
      begin
         if Node = null then
            return No_Entity_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Abs
        (Node : Bare_Op_Abs; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Abs is
      begin
         if Node = null then
            return No_Entity_Op_Abs;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Abs) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_And
        (Node : Bare_Op_And; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And is
      begin
         if Node = null then
            return No_Entity_Op_And;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_And) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_And_Then
        (Node : Bare_Op_And_Then; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And_Then is
      begin
         if Node = null then
            return No_Entity_Op_And_Then;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_And_Then) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Concat
        (Node : Bare_Op_Concat; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Concat is
      begin
         if Node = null then
            return No_Entity_Op_Concat;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Concat) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Div
        (Node : Bare_Op_Div; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Div is
      begin
         if Node = null then
            return No_Entity_Op_Div;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Div) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Double_Dot
        (Node : Bare_Op_Double_Dot; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Double_Dot is
      begin
         if Node = null then
            return No_Entity_Op_Double_Dot;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Double_Dot) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Eq
        (Node : Bare_Op_Eq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Eq is
      begin
         if Node = null then
            return No_Entity_Op_Eq;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Eq) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Gt
        (Node : Bare_Op_Gt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gt is
      begin
         if Node = null then
            return No_Entity_Op_Gt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Gt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Gte
        (Node : Bare_Op_Gte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gte is
      begin
         if Node = null then
            return No_Entity_Op_Gte;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Gte) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_In
        (Node : Bare_Op_In; Info : Internal_Entity_Info)
         return Internal_Entity_Op_In is
      begin
         if Node = null then
            return No_Entity_Op_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Lt
        (Node : Bare_Op_Lt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lt is
      begin
         if Node = null then
            return No_Entity_Op_Lt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Lt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Lte
        (Node : Bare_Op_Lte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lte is
      begin
         if Node = null then
            return No_Entity_Op_Lte;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Lte) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Minus
        (Node : Bare_Op_Minus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Minus is
      begin
         if Node = null then
            return No_Entity_Op_Minus;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Minus) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Mod
        (Node : Bare_Op_Mod; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mod is
      begin
         if Node = null then
            return No_Entity_Op_Mod;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Mod) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Mult
        (Node : Bare_Op_Mult; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mult is
      begin
         if Node = null then
            return No_Entity_Op_Mult;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Mult) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Neq
        (Node : Bare_Op_Neq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Neq is
      begin
         if Node = null then
            return No_Entity_Op_Neq;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Neq) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Not
        (Node : Bare_Op_Not; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Not is
      begin
         if Node = null then
            return No_Entity_Op_Not;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Not) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Not_In
        (Node : Bare_Op_Not_In; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Not_In is
      begin
         if Node = null then
            return No_Entity_Op_Not_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Not_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Or
        (Node : Bare_Op_Or; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or is
      begin
         if Node = null then
            return No_Entity_Op_Or;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Or) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Or_Else
        (Node : Bare_Op_Or_Else; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or_Else is
      begin
         if Node = null then
            return No_Entity_Op_Or_Else;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Or_Else) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Plus
        (Node : Bare_Op_Plus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Plus is
      begin
         if Node = null then
            return No_Entity_Op_Plus;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Plus) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Pow
        (Node : Bare_Op_Pow; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Pow is
      begin
         if Node = null then
            return No_Entity_Op_Pow;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Pow) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Rem
        (Node : Bare_Op_Rem; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Rem is
      begin
         if Node = null then
            return No_Entity_Op_Rem;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Rem) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Xor
        (Node : Bare_Op_Xor; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Xor is
      begin
         if Node = null then
            return No_Entity_Op_Xor;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Xor) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Ordinary_Fixed_Point_Def
        (Node : Bare_Ordinary_Fixed_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Ordinary_Fixed_Point_Def is
      begin
         if Node = null then
            return No_Entity_Ordinary_Fixed_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ordinary_Fixed_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Others_Designator
        (Node : Bare_Others_Designator; Info : Internal_Entity_Info)
         return Internal_Entity_Others_Designator is
      begin
         if Node = null then
            return No_Entity_Others_Designator;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Others_Designator) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Node
        (Node : Bare_Overriding_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Node is
      begin
         if Node = null then
            return No_Entity_Overriding_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Not_Overriding
        (Node : Bare_Overriding_Not_Overriding; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Not_Overriding is
      begin
         if Node = null then
            return No_Entity_Overriding_Not_Overriding;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Not_Overriding) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Overriding
        (Node : Bare_Overriding_Overriding; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Overriding is
      begin
         if Node = null then
            return No_Entity_Overriding_Overriding;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Overriding) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Unspecified
        (Node : Bare_Overriding_Unspecified; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Unspecified is
      begin
         if Node = null then
            return No_Entity_Overriding_Unspecified;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Unspecified) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Body
        (Node : Bare_Package_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Body is
      begin
         if Node = null then
            return No_Entity_Package_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Body_Stub
        (Node : Bare_Package_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Package_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Decl
        (Node : Bare_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Renaming_Decl
        (Node : Bare_Package_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Package_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Assoc
        (Node : Bare_Param_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Assoc is
      begin
         if Node = null then
            return No_Entity_Param_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Spec
        (Node : Bare_Param_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Spec is
      begin
         if Node = null then
            return No_Entity_Param_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Spec_List
        (Node : Bare_Param_Spec_List; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Spec_List is
      begin
         if Node = null then
            return No_Entity_Param_Spec_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Spec_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Params
        (Node : Bare_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Params is
      begin
         if Node = null then
            return No_Entity_Params;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Params) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Paren_Abstract_State_Decl
        (Node : Bare_Paren_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Paren_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Paren_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Paren_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Paren_Expr
        (Node : Bare_Paren_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Paren_Expr is
      begin
         if Node = null then
            return No_Entity_Paren_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Paren_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Parent_List
        (Node : Bare_Parent_List; Info : Internal_Entity_Info)
         return Internal_Entity_Parent_List is
      begin
         if Node = null then
            return No_Entity_Parent_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Parent_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_Directive
        (Node : Bare_Pp_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_Else_Directive
        (Node : Bare_Pp_Else_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_Else_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_Else_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_Else_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_Elsif_Directive
        (Node : Bare_Pp_Elsif_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_Elsif_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_Elsif_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_Elsif_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_End_If_Directive
        (Node : Bare_Pp_End_If_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_End_If_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_End_If_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_End_If_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_If_Directive
        (Node : Bare_Pp_If_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_If_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_If_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_If_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_Then_Kw
        (Node : Bare_Pp_Then_Kw; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_Then_Kw is
      begin
         if Node = null then
            return No_Entity_Pp_Then_Kw;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_Then_Kw) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Argument_Assoc
        (Node : Bare_Pragma_Argument_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Argument_Assoc is
      begin
         if Node = null then
            return No_Entity_Pragma_Argument_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Argument_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Node
        (Node : Bare_Pragma_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Node is
      begin
         if Node = null then
            return No_Entity_Pragma_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Node_List
        (Node : Bare_Pragma_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Node_List is
      begin
         if Node = null then
            return No_Entity_Pragma_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Node_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Node
        (Node : Bare_Private_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Node is
      begin
         if Node = null then
            return No_Entity_Private_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Absent
        (Node : Bare_Private_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Absent is
      begin
         if Node = null then
            return No_Entity_Private_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Part
        (Node : Bare_Private_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Part is
      begin
         if Node = null then
            return No_Entity_Private_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Present
        (Node : Bare_Private_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Present is
      begin
         if Node = null then
            return No_Entity_Private_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Type_Def
        (Node : Bare_Private_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Type_Def is
      begin
         if Node = null then
            return No_Entity_Private_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Node
        (Node : Bare_Protected_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Node is
      begin
         if Node = null then
            return No_Entity_Protected_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Absent
        (Node : Bare_Protected_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Absent is
      begin
         if Node = null then
            return No_Entity_Protected_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Body
        (Node : Bare_Protected_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Body is
      begin
         if Node = null then
            return No_Entity_Protected_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Body_Stub
        (Node : Bare_Protected_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Protected_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Def
        (Node : Bare_Protected_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Def is
      begin
         if Node = null then
            return No_Entity_Protected_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Present
        (Node : Bare_Protected_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Present is
      begin
         if Node = null then
            return No_Entity_Protected_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Type_Decl
        (Node : Bare_Protected_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Protected_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Public_Part
        (Node : Bare_Public_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Public_Part is
      begin
         if Node = null then
            return No_Entity_Public_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Public_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Qual_Expr
        (Node : Bare_Qual_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Qual_Expr is
      begin
         if Node = null then
            return No_Entity_Qual_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Qual_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantified_Expr
        (Node : Bare_Quantified_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Quantified_Expr is
      begin
         if Node = null then
            return No_Entity_Quantified_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantified_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier
        (Node : Bare_Quantifier; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier is
      begin
         if Node = null then
            return No_Entity_Quantifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier_All
        (Node : Bare_Quantifier_All; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier_All is
      begin
         if Node = null then
            return No_Entity_Quantifier_All;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier_All) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier_Some
        (Node : Bare_Quantifier_Some; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier_Some is
      begin
         if Node = null then
            return No_Entity_Quantifier_Some;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier_Some) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Raise_Expr
        (Node : Bare_Raise_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Expr is
      begin
         if Node = null then
            return No_Entity_Raise_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Raise_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Raise_Stmt
        (Node : Bare_Raise_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Stmt is
      begin
         if Node = null then
            return No_Entity_Raise_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Raise_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Range_Constraint
        (Node : Bare_Range_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Range_Constraint is
      begin
         if Node = null then
            return No_Entity_Range_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Range_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Range_Spec
        (Node : Bare_Range_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Range_Spec is
      begin
         if Node = null then
            return No_Entity_Range_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Range_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Real_Literal
        (Node : Bare_Real_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Real_Literal is
      begin
         if Node = null then
            return No_Entity_Real_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Real_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Def
        (Node : Bare_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Def is
      begin
         if Node = null then
            return No_Entity_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Rep_Clause
        (Node : Bare_Record_Rep_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Rep_Clause is
      begin
         if Node = null then
            return No_Entity_Record_Rep_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Rep_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Type_Def
        (Node : Bare_Record_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Type_Def is
      begin
         if Node = null then
            return No_Entity_Record_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reduce_Attribute_Ref
        (Node : Bare_Reduce_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Reduce_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Reduce_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reduce_Attribute_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Relation_Op
        (Node : Bare_Relation_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Relation_Op is
      begin
         if Node = null then
            return No_Entity_Relation_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Relation_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Renaming_Clause
        (Node : Bare_Renaming_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Renaming_Clause is
      begin
         if Node = null then
            return No_Entity_Renaming_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Renaming_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Requeue_Stmt
        (Node : Bare_Requeue_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Requeue_Stmt is
      begin
         if Node = null then
            return No_Entity_Requeue_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Requeue_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Return_Stmt
        (Node : Bare_Return_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Return_Stmt is
      begin
         if Node = null then
            return No_Entity_Return_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Return_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Node
        (Node : Bare_Reverse_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Node is
      begin
         if Node = null then
            return No_Entity_Reverse_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Absent
        (Node : Bare_Reverse_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Absent is
      begin
         if Node = null then
            return No_Entity_Reverse_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Present
        (Node : Bare_Reverse_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Present is
      begin
         if Node = null then
            return No_Entity_Reverse_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_Stmt
        (Node : Bare_Select_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Select_Stmt is
      begin
         if Node = null then
            return No_Entity_Select_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_When_Part
        (Node : Bare_Select_When_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Select_When_Part is
      begin
         if Node = null then
            return No_Entity_Select_When_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_When_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_When_Part_List
        (Node : Bare_Select_When_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Select_When_Part_List is
      begin
         if Node = null then
            return No_Entity_Select_When_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_When_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Signed_Int_Type_Def
        (Node : Bare_Signed_Int_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Signed_Int_Type_Def is
      begin
         if Node = null then
            return No_Entity_Signed_Int_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Signed_Int_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Protected_Decl
        (Node : Bare_Single_Protected_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Protected_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Protected_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Protected_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Task_Decl
        (Node : Bare_Single_Task_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Task_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Task_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Task_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Type_Decl
        (Node : Bare_Task_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Task_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Task_Type_Decl
        (Node : Bare_Single_Task_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Task_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Task_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Task_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Stmt_List
        (Node : Bare_Stmt_List; Info : Internal_Entity_Info)
         return Internal_Entity_Stmt_List is
      begin
         if Node = null then
            return No_Entity_Stmt_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Stmt_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_String_Literal
        (Node : Bare_String_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_String_Literal is
      begin
         if Node = null then
            return No_Entity_String_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_String_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Body
        (Node : Bare_Subp_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Body is
      begin
         if Node = null then
            return No_Entity_Subp_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Body_Stub
        (Node : Bare_Subp_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Subp_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Decl
        (Node : Bare_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind
        (Node : Bare_Subp_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind is
      begin
         if Node = null then
            return No_Entity_Subp_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind_Function
        (Node : Bare_Subp_Kind_Function; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind_Function is
      begin
         if Node = null then
            return No_Entity_Subp_Kind_Function;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind_Function) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind_Procedure
        (Node : Bare_Subp_Kind_Procedure; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind_Procedure is
      begin
         if Node = null then
            return No_Entity_Subp_Kind_Procedure;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind_Procedure) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Renaming_Decl
        (Node : Bare_Subp_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Subp_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Spec
        (Node : Bare_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subtype_Decl
        (Node : Bare_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subunit
        (Node : Bare_Subunit; Info : Internal_Entity_Info)
         return Internal_Entity_Subunit is
      begin
         if Node = null then
            return No_Entity_Subunit;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subunit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Node
        (Node : Bare_Synchronized_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Node is
      begin
         if Node = null then
            return No_Entity_Synchronized_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Absent
        (Node : Bare_Synchronized_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Absent is
      begin
         if Node = null then
            return No_Entity_Synchronized_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Present
        (Node : Bare_Synchronized_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Present is
      begin
         if Node = null then
            return No_Entity_Synchronized_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synth_Anonymous_Type_Decl
        (Node : Bare_Synth_Anonymous_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synth_Anonymous_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Synth_Anonymous_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synth_Anonymous_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Binary_Spec
        (Node : Bare_Synthetic_Binary_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Binary_Spec is
      begin
         if Node = null then
            return No_Entity_Synthetic_Binary_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Binary_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Char_Enum_Lit
        (Node : Bare_Synthetic_Char_Enum_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Char_Enum_Lit is
      begin
         if Node = null then
            return No_Entity_Synthetic_Char_Enum_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Char_Enum_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Defining_Name
        (Node : Bare_Synthetic_Defining_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Defining_Name is
      begin
         if Node = null then
            return No_Entity_Synthetic_Defining_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Defining_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Formal_Param_Decl
        (Node : Bare_Synthetic_Formal_Param_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Formal_Param_Decl is
      begin
         if Node = null then
            return No_Entity_Synthetic_Formal_Param_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Formal_Param_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Identifier
        (Node : Bare_Synthetic_Identifier; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Identifier is
      begin
         if Node = null then
            return No_Entity_Synthetic_Identifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Identifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Renaming_Clause
        (Node : Bare_Synthetic_Renaming_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Renaming_Clause is
      begin
         if Node = null then
            return No_Entity_Synthetic_Renaming_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Renaming_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Subp_Decl
        (Node : Bare_Synthetic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Synthetic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Type_Expr
        (Node : Bare_Synthetic_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Synthetic_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Type_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Unary_Spec
        (Node : Bare_Synthetic_Unary_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Unary_Spec is
      begin
         if Node = null then
            return No_Entity_Synthetic_Unary_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Unary_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Node
        (Node : Bare_Tagged_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Node is
      begin
         if Node = null then
            return No_Entity_Tagged_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Absent
        (Node : Bare_Tagged_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Absent is
      begin
         if Node = null then
            return No_Entity_Tagged_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Present
        (Node : Bare_Tagged_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Present is
      begin
         if Node = null then
            return No_Entity_Tagged_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Target_Name
        (Node : Bare_Target_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Target_Name is
      begin
         if Node = null then
            return No_Entity_Target_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Target_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Body
        (Node : Bare_Task_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Body is
      begin
         if Node = null then
            return No_Entity_Task_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Body_Stub
        (Node : Bare_Task_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Task_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Def
        (Node : Bare_Task_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Def is
      begin
         if Node = null then
            return No_Entity_Task_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Terminate_Alternative
        (Node : Bare_Terminate_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Terminate_Alternative is
      begin
         if Node = null then
            return No_Entity_Terminate_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Terminate_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Access_Def
        (Node : Bare_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Attributes_Repository
        (Node : Bare_Type_Attributes_Repository; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Attributes_Repository is
      begin
         if Node = null then
            return No_Entity_Type_Attributes_Repository;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Attributes_Repository) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Un_Op
        (Node : Bare_Un_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Un_Op is
      begin
         if Node = null then
            return No_Entity_Un_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Un_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Index
        (Node : Bare_Unconstrained_Array_Index; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Index is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Index;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Index) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Index_List
        (Node : Bare_Unconstrained_Array_Index_List; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Index_List is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Index_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Index_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Indices
        (Node : Bare_Unconstrained_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unknown_Discriminant_Part
        (Node : Bare_Unknown_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Unknown_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Unknown_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unknown_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Node
        (Node : Bare_Until_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Node is
      begin
         if Node = null then
            return No_Entity_Until_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Absent
        (Node : Bare_Until_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Absent is
      begin
         if Node = null then
            return No_Entity_Until_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Present
        (Node : Bare_Until_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Present is
      begin
         if Node = null then
            return No_Entity_Until_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Update_Attribute_Ref
        (Node : Bare_Update_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Update_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Update_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Update_Attribute_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Clause
        (Node : Bare_Use_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Package_Clause
        (Node : Bare_Use_Package_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Package_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Package_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Package_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Type_Clause
        (Node : Bare_Use_Type_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Type_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Type_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Type_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Value_Sequence
        (Node : Bare_Value_Sequence; Info : Internal_Entity_Info)
         return Internal_Entity_Value_Sequence is
      begin
         if Node = null then
            return No_Entity_Value_Sequence;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Value_Sequence) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant
        (Node : Bare_Variant; Info : Internal_Entity_Info)
         return Internal_Entity_Variant is
      begin
         if Node = null then
            return No_Entity_Variant;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant_List
        (Node : Bare_Variant_List; Info : Internal_Entity_Info)
         return Internal_Entity_Variant_List is
      begin
         if Node = null then
            return No_Entity_Variant_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant_Part
        (Node : Bare_Variant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Variant_Part is
      begin
         if Node = null then
            return No_Entity_Variant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_While_Loop_Spec
        (Node : Bare_While_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_While_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_While_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_While_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_While_Loop_Stmt
        (Node : Bare_While_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_While_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_While_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_While_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Clause
        (Node : Bare_With_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_With_Clause is
      begin
         if Node = null then
            return No_Entity_With_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private
        (Node : Bare_With_Private; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private is
      begin
         if Node = null then
            return No_Entity_With_Private;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private_Absent
        (Node : Bare_With_Private_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private_Absent is
      begin
         if Node = null then
            return No_Entity_With_Private_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private_Present
        (Node : Bare_With_Private_Present; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private_Present is
      begin
         if Node = null then
            return No_Entity_With_Private_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Env_Assoc) is
      begin
               Inc_Ref (R.Dest_Env);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Env_Assoc) is
      begin
               Dec_Ref (R.Dest_Env);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Env_Assoc) return Boolean is
      begin
         return L.Key = R.Key and then L.Value = R.Value and then Equivalent (L.Dest_Env, R.Dest_Env) and then L.Metadata = R.Metadata;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Env_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Dest_Env => " & Trace_Image (R.Dest_Env)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Eval_Discrete_Range) is
      begin
               Inc_Ref (R.Low_Bound);
               Inc_Ref (R.High_Bound);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Eval_Discrete_Range) is
      begin
               Dec_Ref (R.Low_Bound);
               Dec_Ref (R.High_Bound);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Eval_Discrete_Range) return Boolean is
      begin
         return Equivalent (L.Low_Bound, R.Low_Bound) and then Equivalent (L.High_Bound, R.High_Bound);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Eval_Discrete_Range) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Low_Bound => " & Trace_Image (R.Low_Bound)
                        & ", "
                     & "High_Bound => " & Trace_Image (R.High_Bound)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Expected_Type_For_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Expected_Type => " & Trace_Image (R.Expected_Type)
                        & ", "
                     & "Expr => " & Trace_Image (R.Expr)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Inner_Env_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Logic_Val_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Success => " & Trace_Image (R.Success)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Multidim_Aggregate_Info) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Agg => " & Trace_Image (R.Agg)
                        & ", "
                     & "Typ => " & Trace_Image (R.Typ)
                        & ", "
                     & "Rank => " & Trace_Image (R.Rank)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Param_Actual) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Param => " & Trace_Image (R.Param)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Single_Actual) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Name => " & Trace_Image (R.Name)
                        & ", "
                     & "Assoc => " & Trace_Image (R.Assoc)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Param_Match) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Has_Matched => " & Trace_Image (R.Has_Matched)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
                        & ", "
                     & "Formal => " & Trace_Image (R.Formal)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Ref_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Ref => " & Trace_Image (R.Ref)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Refd_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Decl => " & Trace_Image (R.Decl)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Refd_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Def_Name => " & Trace_Image (R.Def_Name)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Shape) is
      begin
               Inc_Ref (R.Components);
               Inc_Ref (R.Discriminants_Values);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Shape) is
      begin
               Dec_Ref (R.Components);
               Dec_Ref (R.Discriminants_Values);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Shape) return Boolean is
      begin
         return Equivalent (L.Components, R.Components) and then Equivalent (L.Discriminants_Values, R.Discriminants_Values);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Shape) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Components => " & Trace_Image (R.Components)
                        & ", "
                     & "Discriminants_Values => " & Trace_Image (R.Discriminants_Values)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Substitution) is
      begin
               Inc_Ref (R.To_Value);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Substitution) is
      begin
               Dec_Ref (R.To_Value);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Substitution) return Boolean is
      begin
         return L.From_Decl = R.From_Decl and then Equivalent (L.To_Value, R.To_Value) and then L.Value_Type = R.Value_Type;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Substitution) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "From_Decl => " & Trace_Image (R.From_Decl)
                        & ", "
                     & "To_Value => " & Trace_Image (R.To_Value)
                        & ", "
                     & "Value_Type => " & Trace_Image (R.Value_Type)
               & ")");
      end Trace_Image;
   type Ada_Node_P_Is_Not_Null_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Ada_Node_P_Is_Not_Null_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Ada_Node_P_Is_Not_Null_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Ada_Node_P_Is_Not_Null_0_Predicate);
   function Create_Ada_Node_P_Is_Not_Null_0_Predicate
      return Ada_Node_P_Is_Not_Null_0_Predicate is
   begin
      return Ada_Node_P_Is_Not_Null_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Ada_Node_P_Is_Not_Null_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : constant Bare_Ada_Node := Entity.Node;
   begin
      return Ada_Node_P_Is_Not_Null (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Ada_Node_P_Is_Not_Null_0_Predicate) return String is
   begin
      return "AdaNode.is_not_null";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Ada_Node_P_Is_Not_Null_0_Predicate) is
      begin
         null;
      end Destroy;
   type Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Basic_Decl;
         Field_1 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate);
   function Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate
   (
         Field_0 : Internal_Entity_Basic_Decl;
         Field_1 : Bare_Ada_Node
   )
      return Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate is
   begin
      return Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Field_1 => Field_1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Basic_Decl := Entity.Node;
   begin
      return Basic_Decl_P_Subp_Decl_Match_Signature (Node, Self.Field_0, Self.Field_1, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate) return String is
   begin
      return "BasicDecl.subp_decl_match_signature";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate) is
      begin
         null;
      end Destroy;
   type Basic_Decl_P_Is_Subprogram_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Is_Subprogram_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Is_Subprogram_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Basic_Decl_P_Is_Subprogram_0_Predicate);
   function Create_Basic_Decl_P_Is_Subprogram_0_Predicate
      return Basic_Decl_P_Is_Subprogram_0_Predicate is
   begin
      return Basic_Decl_P_Is_Subprogram_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Is_Subprogram_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : constant Bare_Basic_Decl := Entity.Node;
   begin
      return Basic_Decl_P_Is_Subprogram (Node);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Is_Subprogram_0_Predicate) return String is
   begin
      return "BasicDecl.is_subprogram";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Basic_Decl_P_Is_Subprogram_0_Predicate) is
      begin
         null;
      end Destroy;
   type Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Boolean;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate);
   function Create_Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate
   (
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Boolean
   )
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate is
   begin
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Field_1 => Field_1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Basic_Decl := Entity.Node;
   begin
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type (Node, Self.Field_0, Self.Field_1, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate) return String is
   begin
      return "BasicDecl.is_stream_subprogram_for_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Base_Type_Decl;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate);
   function Create_Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate
   (
         Field_0 : Internal_Entity_Base_Type_Decl
   )
      return Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate is
   begin
      return Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Basic_Decl := Entity.Node;
   begin
      return Basic_Decl_P_Is_Put_Image_Subprogram_For_Type (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate) return String is
   begin
      return "BasicDecl.is_put_image_subprogram_for_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Array_Or_Rec (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_array_or_rec";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Task_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Task_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Task_Type_0_Predicate
      return Base_Type_Decl_P_Is_Task_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Task_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Dispatcher_Base_Type_Decl_P_Is_Task_Type (Node);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_task_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Task_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_One_Non_Universal_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_One_Non_Universal_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_One_Non_Universal_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_One_Non_Universal_0_Predicate);
   function Create_Base_Type_Decl_P_One_Non_Universal_0_Predicate
      return Base_Type_Decl_P_One_Non_Universal_0_Predicate is
   begin
      return Base_Type_Decl_P_One_Non_Universal_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_One_Non_Universal_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         pragma Unreferenced (Self);
         Entity : Internal_Entity := Entities (1);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_One_Non_Universal (Node, (Node => Entities (2).Node, Info => Entities (2).Info), E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_One_Non_Universal_0_Predicate) return String is
   begin
      return "BaseTypeDecl.one_non_universal";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_One_Non_Universal_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Non_Null_Char_Type (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_non_null_char_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Universal_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Universal_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate
      return Base_Type_Decl_P_Is_Universal_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Universal_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Universal_Type (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_universal_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Universal_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate
      return Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Not_Universal_Type (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_not_universal_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Access_Type_Predicate (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_access_type_predicate";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Iterator_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Iterator_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Iterator_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Iterator_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Iterator_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Iterator_Type (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_iterator_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Iterator_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Discrete_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Discrete_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Discrete_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Discrete_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_discrete_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Discrete_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate
      return Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Not_Root_Int_Type (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_not_root_int_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Allows_String_Literal_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Allows_String_Literal_0_Predicate);
   function Create_Base_Type_Decl_P_Allows_String_Literal_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Allows_String_Literal_0_Predicate is
   begin
      return Base_Type_Decl_P_Allows_String_Literal_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Allows_String_Literal (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate) return String is
   begin
      return "BaseTypeDecl.allows_string_literal";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Allows_String_Literal_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Subp_Access_Of (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_subp_access_of";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Tagged_Type_With_Deref (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_tagged_type_with_deref";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_array_def_with_deref";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate);
   function Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null (Node, Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_array_def_with_deref_or_null";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Matching_Prefix_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_prefix_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Matching_Formal_Prim_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_formal_prim_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Matching_Formal_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Formal_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Matching_Formal_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Formal_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Formal_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Matching_Formal_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_formal_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Formal_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Matching_Assign_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Assign_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Matching_Assign_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Assign_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Assign_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Matching_Assign_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_assign_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Assign_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Matching_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Matching_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Matching_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate) return String;
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate);
   function Create_Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate
   (
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Field_1 => Field_1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : constant Bare_Base_Type_Decl := Entity.Node;
   begin
      return Base_Type_Decl_P_Matching_Allocator_Type (Node, Self.Field_0, Self.Field_1, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_allocator_type";
   end Image;
      -------------
      -- Destroy --
      -------------
      overriding procedure Destroy (Self : in out Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate) is
      begin
         null;
      end Destroy;
   type Logic_Functor_669 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_669; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_669) return String;
   overriding function Convert (Self : Logic_Functor_669; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Accessed_Type_No_Call (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_669) return String is
   begin
      return ("BaseTypeDecl.accessed_type_no_call");
   end Image;
   type Logic_Functor_616 is new Solver_Ifc.Converter_Type with
      null record;
   overriding function Convert (Self : Logic_Functor_616; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_616) return String;
   overriding function Convert (Self : Logic_Functor_616; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Anonymous_Access_Type_Or_Null (Node => From.Node, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_616) return String is
   begin
      return ("BaseTypeDecl.anonymous_access_type_or_null");
   end Image;
   type Logic_Functor_646 is new Solver_Ifc.Combiner_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Combine (Self : Logic_Functor_646; Vals : Entity_Vars.Value_Array) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_646) return String;
   overriding function Combine (Self : Logic_Functor_646; Vals : Entity_Vars.Value_Array) return Internal_Entity is
         From : constant Internal_Entity := Vals (1);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
            if Vals (2).Node /= null
               and then Vals (2).Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
      Ret := Base_Type_Decl_P_Array_Concat_Result_Type (Node => From.Node, Other => (Node => Vals (2).Node,  Info => Vals (2).Info), Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Combine;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_646) return String is
   begin
      return ("BaseTypeDecl.array_concat_result_type");
   end Image;
   type Logic_Functor_614 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_614; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_614) return String;
   overriding function Convert (Self : Logic_Functor_614; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Base_Subtype (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_614) return String is
   begin
      return ("BaseTypeDecl.base_subtype");
   end Image;
   type Logic_Functor_685 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_685; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_685) return String;
   overriding function Convert (Self : Logic_Functor_685; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Comp_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_685) return String is
   begin
      return ("BaseTypeDecl.comp_type");
   end Image;
   type Logic_Functor_710 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_710; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_710) return String;
   overriding function Convert (Self : Logic_Functor_710; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Basic_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Corresponding_Char_Literal (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_710) return String is
   begin
      return ("BaseTypeDecl.corresponding_char_literal");
   end Image;
   type Logic_Functor_643 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_643; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_643) return String;
   overriding function Convert (Self : Logic_Functor_643; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Derefed_Base_Subtype (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_643) return String is
   begin
      return ("BaseTypeDecl.derefed_base_subtype");
   end Image;
   type Logic_Functor_642 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_642; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_642) return String;
   overriding function Convert (Self : Logic_Functor_642; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Derefed_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_642) return String is
   begin
      return ("BaseTypeDecl.derefed_type");
   end Image;
   type Logic_Functor_647 is new Solver_Ifc.Combiner_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Combine (Self : Logic_Functor_647; Vals : Entity_Vars.Value_Array) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_647) return String;
   overriding function Combine (Self : Logic_Functor_647; Vals : Entity_Vars.Value_Array) return Internal_Entity is
         From : constant Internal_Entity := Vals (1);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
            if Vals (2).Node /= null
               and then Vals (2).Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
      Ret := Base_Type_Decl_P_Expected_Array_Concat_Operand_Type (Node => From.Node, Operand_Type => (Node => Vals (2).Node,  Info => Vals (2).Info), Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Combine;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_647) return String is
   begin
      return ("BaseTypeDecl.expected_array_concat_operand_type");
   end Image;
   type Logic_Functor_645 is new Solver_Ifc.Combiner_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Combine (Self : Logic_Functor_645; Vals : Entity_Vars.Value_Array) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_645) return String;
   overriding function Combine (Self : Logic_Functor_645; Vals : Entity_Vars.Value_Array) return Internal_Entity is
         From : constant Internal_Entity := Vals (1);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
            if Vals (2).Node /= null
               and then Vals (2).Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
      Ret := Base_Type_Decl_P_Non_Universal_Base_Subtype (Node => From.Node, Second => (Node => Vals (2).Node,  Info => Vals (2).Info), Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Combine;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_645) return String is
   begin
      return ("BaseTypeDecl.non_universal_base_subtype");
   end Image;
   type Logic_Functor_652 is new Solver_Ifc.Converter_Type with
      null record;
   overriding function Convert (Self : Logic_Functor_652; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_652) return String;
   overriding function Convert (Self : Logic_Functor_652; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Discrete_Base_Subtype_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Scalar_Base_Type (Node => From.Node, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_652) return String is
   begin
      return ("BaseTypeDecl.scalar_base_type");
   end Image;
   type Logic_Functor_218 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_218; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_218) return String;
   overriding function Convert (Self : Logic_Functor_218; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Basic_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Basic_Decl_P_Identity_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_218) return String is
   begin
      return ("BasicDecl.identity_type");
   end Image;
   type Logic_Functor_223 is new Solver_Ifc.Converter_Type with
      null record;
   overriding function Convert (Self : Logic_Functor_223; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_223) return String;
   overriding function Convert (Self : Logic_Functor_223; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Subp_Spec;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Basic_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Basic_Decl_P_Subp_Spec_Or_Null (Node => From.Node, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_223) return String is
   begin
      return ("BasicDecl.subp_spec_or_null");
   end Image;
   type Logic_Functor_651 is new Solver_Ifc.Converter_Type with
      null record;
   overriding function Convert (Self : Logic_Functor_651; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_651) return String;
   overriding function Convert (Self : Logic_Functor_651; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Classwide_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => From.Node, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_651) return String is
   begin
      return ("BaseTypeDecl.classwide_type");
   end Image;
   type Logic_Functor_221 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_221; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_221) return String;
   overriding function Convert (Self : Logic_Functor_221; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in Ada_Basic_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Dispatcher_Basic_Decl_P_Expr_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_221) return String is
   begin
      return ("BasicDecl.expr_type");
   end Image;
   --
   --  Primitives for Bare_Ada_Node
   --
--# property-start AdaNode.declarative_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:144
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Declarative_Scope
  (Node : Bare_Ada_Node
  )
   return Bare_Declarative_Part
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Declarative_Part;
            procedure Finalizer_Scope_3 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Declarative_Part;
            procedure Finalizer_Scope_3 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_3;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4 '<Cast to DeclarativePart at ast.py:147>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:147
--# expr-start 3 '<Find at ast.py:145>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:145
   --# expr-start 1 '<FieldAccess for parents at ast.py:145>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:145
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 1
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 2 '<IsA DeclarativePart at ast.py:146>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:146
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Declarative_Part_Range; 
--# expr-done 2
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 3
   if Find_Result = null
      or else Find_Result.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_3;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Declarative_Scope;
--# end
--# property-start AdaNode.owning_unit_kind /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:153
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Owning_Unit_Kind
  (Node : Bare_Ada_Node
  )
   return Analysis_Unit_Kind
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Analysis_Unit_Kind;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Analysis_Unit_Kind;
Let_Result : Analysis_Unit_Kind;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8 '<FieldAccess for unit_kind at ast.py:158>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:158
--# expr-start 7 '<Cast to CompilationUnit at ast.py:158>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:158
--# expr-start 6 '<FieldAccess for root at ast.py:158>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:158
--# expr-start 5 '<FieldAccess for unit at ast.py:158>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:158
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 5
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 6
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.unit_kind
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Unit_Kind (Node => Cast_Result);
--# end
--# expr-done 8
Let_Result := Fld_2; 
--# end
--# expr-done 9
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Owning_Unit_Kind;
--# end
--# property-start AdaNode.withed_unit_helper /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:160
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Withed_Unit_Helper
  (Node : Bare_Ada_Node
      ; Unit_Name : Bare_Name
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind unit_name Unit_Name
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_5 with Inline_Always;
      Block_Var : Symbol_Type_Array_Access;
Block_Var_1 : Bare_Compilation_Unit;
Block_Var_2 : Bare_Compilation_Unit;
Block_Var_3 : Bare_Compilation_Unit;
Var_Expr : Bare_Compilation_Unit;
Var_Expr_1 : Bare_Compilation_Unit;
Fld : Symbol_Type_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Fld_2 : Bare_Compilation_Unit;
Is_Null : Boolean;
If_Result : Bare_Compilation_Unit;
Fld_3 : Bare_Compilation_Unit;
Fld_4 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Result_Var : Bare_Basic_Decl;
Is_A : Boolean;
Is_Null_1 : Boolean;
If_Result_1 : Boolean;
Is_Null_2 : Boolean;
Is_Null_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Bare_Compilation_Unit;
Scope_Result_1 : Bare_Compilation_Unit;
Result_Var_1 : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_5 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_5;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 22 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10 '<FieldAccess for as_symbol_array at ast.py:172>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:172
      if Unit_Name = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Unit_Name);
--# end
--# expr-done 10
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind unit_name_array Block_Var
--# expr-start 11 '<FieldAccess for designated_compilation_unit at ast.py:173>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:173
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld_1 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Block_Var, Kind => Unit_Specification, Not_Found_Is_Error => False);
--# end
--# expr-done 11
Block_Var_1 := Fld_1; 
--# bind spec Block_Var_1
--# expr-start 14 '<If at ast.py:178>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:178
--# expr-start 12 '<IsNull at ast.py:179>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:179
Is_Null := Block_Var_1 = null; 
--# expr-done 12
if Is_Null then
   --# expr-start 13 '<FieldAccess for designated_compilation_unit at ast.py:180>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:180
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Block_Var, Kind => Unit_Body, Not_Found_Is_Error => False);
--# end
--# expr-done 13
   If_Result := Fld_2;
else
   If_Result := No_Bare_Ada_Node;
end if;
--# expr-done 14
Block_Var_2 := If_Result; 
--# bind body Block_Var_2
--# expr-start 21 '<If at ast.py:192>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:192
--# expr-start 16 '<IsA PackageBody at ast.py:193>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:193
Var_Expr := Block_Var_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 15 '<FieldAccess for decl at ast.py:193>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:193
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr);
--# end
--# expr-done 15
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_A := Result_Var /= null 
and then Result_Var.Kind in Ada_Package_Body_Range; 
--# expr-done 16
if Is_A then
   --# expr-start 17 '<IsNull at ast.py:193>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:193
Is_Null_1 := Block_Var_1 = null; 
--# expr-done 17
   If_Result_1 := Is_Null_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 18 '<IsNull at ast.py:194>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:194
Is_Null_2 := Block_Var_1 = null; 
--# expr-done 18
if Is_Null_2 then
   --# expr-start 19 '<IsNull at ast.py:194>' Is_Null_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:194
Is_Null_3 := Block_Var_2 = null; 
--# expr-done 19
   If_Result_2 := Is_Null_3;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   --# expr-start 20 '<FieldAccess for designated_compilation_unit at ast.py:196>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:196
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld_3 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Block_Var, Kind => Unit_Specification, Not_Found_Is_Error => True);
--# end
--# expr-done 20
   If_Result_4 := Fld_3;
else
   If_Result_4 := No_Bare_Ada_Node;
end if;
--# expr-done 21
Block_Var_3 := If_Result_4; 
Var_Expr_1 := Block_Var_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var_2;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 22
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_5;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_5;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Withed_Unit_Helper;
--# end
--# property-start '[dispatcher]AdaNode.custom_id_text' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Custom_Id_Text
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : String_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Signed_Int_Type_Def | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.custom_id_text
                  Property_Result := Ada_Node_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.custom_id_text
                  Property_Result := Basic_Decl_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.custom_id_text
                  Property_Result := Anonymous_Type_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Lit_Synth_Type_Expr =>
                  --# property-call-start EnumLitSynthTypeExpr.custom_id_text
                  Property_Result := Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.custom_id_text
                  Property_Result := Subtype_Indication_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Custom_Id_Text;
--# end
--# property-start AdaNode.is_contract_aspect /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:216
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Contract_Aspect
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
      Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
Sym_21 : Symbol_Type;
Is_Equal_21 : Boolean;
If_Result_20 : Boolean;
Sym_22 : Symbol_Type;
Is_Equal_22 : Boolean;
If_Result_21 : Boolean;
Sym_23 : Symbol_Type;
Is_Equal_23 : Boolean;
If_Result_22 : Boolean;
Sym_24 : Symbol_Type;
Is_Equal_24 : Boolean;
If_Result_23 : Boolean;
Sym_25 : Symbol_Type;
Is_Equal_25 : Boolean;
If_Result_24 : Boolean;
Sym_26 : Symbol_Type;
Is_Equal_26 : Boolean;
If_Result_25 : Boolean;
Sym_27 : Symbol_Type;
Is_Equal_27 : Boolean;
If_Result_26 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 79 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 24 '<Eq at ast.py:218>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 23 '<Symbol Pre at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 23
Is_Equal := Name = Sym; 
--# expr-done 24
if Is_Equal then
   If_Result := True;
else
   --# expr-start 26 '<Eq at ast.py:218>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 25 '<Symbol Pre'"'"'Class at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre_Class); 
--# expr-done 25
Is_Equal_1 := Name = Sym_1; 
--# expr-done 26
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 28 '<Eq at ast.py:218>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 27 '<Symbol Post at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 27
Is_Equal_2 := Name = Sym_2; 
--# expr-done 28
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 30 '<Eq at ast.py:218>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 29 '<Symbol Post'"'"'Class at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post_Class); 
--# expr-done 29
Is_Equal_3 := Name = Sym_3; 
--# expr-done 30
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 32 '<Eq at ast.py:218>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 31 '<Symbol Refined_Post at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Post); 
--# expr-done 31
Is_Equal_4 := Name = Sym_4; 
--# expr-done 32
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 34 '<Eq at ast.py:218>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 33 '<Symbol Precondition at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition); 
--# expr-done 33
Is_Equal_5 := Name = Sym_5; 
--# expr-done 34
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 36 '<Eq at ast.py:218>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 35 '<Symbol Postcondition at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition); 
--# expr-done 35
Is_Equal_6 := Name = Sym_6; 
--# expr-done 36
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 38 '<Eq at ast.py:218>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 37 '<Symbol Precondition'"'"'Class at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition_Class); 
--# expr-done 37
Is_Equal_7 := Name = Sym_7; 
--# expr-done 38
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 40 '<Eq at ast.py:218>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 39 '<Symbol Postcondition'"'"'Class at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition_Class); 
--# expr-done 39
Is_Equal_8 := Name = Sym_8; 
--# expr-done 40
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 42 '<Eq at ast.py:218>' Is_Equal_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 41 '<Symbol Invariant at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invariant); 
--# expr-done 41
Is_Equal_9 := Name = Sym_9; 
--# expr-done 42
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 44 '<Eq at ast.py:218>' Is_Equal_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 43 '<Symbol Invariant'"'"'Class at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invariant_Class); 
--# expr-done 43
Is_Equal_10 := Name = Sym_10; 
--# expr-done 44
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 46 '<Eq at ast.py:218>' Is_Equal_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 45 '<Symbol Type_Invariant at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Invariant); 
--# expr-done 45
Is_Equal_11 := Name = Sym_11; 
--# expr-done 46
   If_Result_10 := Is_Equal_11;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 48 '<Eq at ast.py:218>' Is_Equal_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 47 '<Symbol Type_Invariant'"'"'Class at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Invariant_Class); 
--# expr-done 47
Is_Equal_12 := Name = Sym_12; 
--# expr-done 48
   If_Result_11 := Is_Equal_12;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 50 '<Eq at ast.py:218>' Is_Equal_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 49 '<Symbol Predicate at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate); 
--# expr-done 49
Is_Equal_13 := Name = Sym_13; 
--# expr-done 50
   If_Result_12 := Is_Equal_13;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 52 '<Eq at ast.py:218>' Is_Equal_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 51 '<Symbol Static_Predicate at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Static_Predicate); 
--# expr-done 51
Is_Equal_14 := Name = Sym_14; 
--# expr-done 52
   If_Result_13 := Is_Equal_14;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 54 '<Eq at ast.py:218>' Is_Equal_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 53 '<Symbol Dynamic_Predicate at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Dynamic_Predicate); 
--# expr-done 53
Is_Equal_15 := Name = Sym_15; 
--# expr-done 54
   If_Result_14 := Is_Equal_15;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 56 '<Eq at ast.py:218>' Is_Equal_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 55 '<Symbol Default_Initial_Condition at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Default_Initial_Condition); 
--# expr-done 55
Is_Equal_16 := Name = Sym_16; 
--# expr-done 56
   If_Result_15 := Is_Equal_16;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 58 '<Eq at ast.py:218>' Is_Equal_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 57 '<Symbol Initial_Condition at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Initial_Condition); 
--# expr-done 57
Is_Equal_17 := Name = Sym_17; 
--# expr-done 58
   If_Result_16 := Is_Equal_17;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 60 '<Eq at ast.py:218>' Is_Equal_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 59 '<Symbol Contract_Cases at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 59
Is_Equal_18 := Name = Sym_18; 
--# expr-done 60
   If_Result_17 := Is_Equal_18;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 62 '<Eq at ast.py:218>' Is_Equal_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 61 '<Symbol Test_Case at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 61
Is_Equal_19 := Name = Sym_19; 
--# expr-done 62
   If_Result_18 := Is_Equal_19;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 64 '<Eq at ast.py:218>' Is_Equal_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 63 '<Symbol Global at ???>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Global); 
--# expr-done 63
Is_Equal_20 := Name = Sym_20; 
--# expr-done 64
   If_Result_19 := Is_Equal_20;
end if;
if If_Result_19 then
   If_Result_20 := True;
else
   --# expr-start 66 '<Eq at ast.py:218>' Is_Equal_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 65 '<Symbol Refined_Global at ???>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Global); 
--# expr-done 65
Is_Equal_21 := Name = Sym_21; 
--# expr-done 66
   If_Result_20 := Is_Equal_21;
end if;
if If_Result_20 then
   If_Result_21 := True;
else
   --# expr-start 68 '<Eq at ast.py:218>' Is_Equal_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 67 '<Symbol Refined_State at ???>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_State); 
--# expr-done 67
Is_Equal_22 := Name = Sym_22; 
--# expr-done 68
   If_Result_21 := Is_Equal_22;
end if;
if If_Result_21 then
   If_Result_22 := True;
else
   --# expr-start 70 '<Eq at ast.py:218>' Is_Equal_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 69 '<Symbol Stable_Properties at ???>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Stable_Properties); 
--# expr-done 69
Is_Equal_23 := Name = Sym_23; 
--# expr-done 70
   If_Result_22 := Is_Equal_23;
end if;
if If_Result_22 then
   If_Result_23 := True;
else
   --# expr-start 72 '<Eq at ast.py:218>' Is_Equal_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 71 '<Symbol Depends at ???>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Depends); 
--# expr-done 71
Is_Equal_24 := Name = Sym_24; 
--# expr-done 72
   If_Result_23 := Is_Equal_24;
end if;
if If_Result_23 then
   If_Result_24 := True;
else
   --# expr-start 74 '<Eq at ast.py:218>' Is_Equal_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 73 '<Symbol Refined_Depends at ???>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Depends); 
--# expr-done 73
Is_Equal_25 := Name = Sym_25; 
--# expr-done 74
   If_Result_24 := Is_Equal_25;
end if;
if If_Result_24 then
   If_Result_25 := True;
else
   --# expr-start 76 '<Eq at ast.py:218>' Is_Equal_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 75 '<Symbol Predicate_Failure at ???>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate_Failure); 
--# expr-done 75
Is_Equal_26 := Name = Sym_26; 
--# expr-done 76
   If_Result_25 := Is_Equal_26;
end if;
if If_Result_25 then
   If_Result_26 := True;
else
   --# expr-start 78 '<Eq at ast.py:218>' Is_Equal_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:218
--# expr-start 77 '<Symbol SPARK_Mode at ???>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 77
Is_Equal_27 := Name = Sym_27; 
--# expr-done 78
   If_Result_26 := Is_Equal_27;
end if;
Let_Result := If_Result_26; 
--# end
--# expr-done 79
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Contract_Aspect;
--# end
--# property-start AdaNode.in_contract /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:232
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_In_Contract
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_8 with Inline_Always;
      Var_Expr : Boolean;
Var_Expr_1 : Bare_Aspect_Assoc;
Var_Expr_2 : Bare_Pragma_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Pragma_Node;
Fld_3 : Bare_Identifier;
As_Entity_1 : Internal_Entity_Identifier;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Identifier;
Fld_4 : Symbol_Type;
Fld_5 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_1 : Boolean;
Result_Var_2 : Boolean;
Find_Result : Bare_Ada_Node;
Is_Null_2 : Boolean;
Not_Val : Boolean;
            procedure Finalizer_Scope_8 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_8;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 93 '<Not at ast.py:232>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:232
--# expr-start 92 '<IsNull at ast.py:238>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:238
--# expr-start 91 '<Find at ast.py:232>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:232
   --# expr-start 80 '<FieldAccess for parents at ast.py:232>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:232
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 80
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
--# expr-start 81 '<Cast to AspectAssoc at ast.py:233>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:233
Cast_Expr := Item_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 81
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind a Var_Expr_1
--# expr-start 85 '<FieldAccess for is_contract_aspect at ast.py:234>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:234
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 84 '<FieldAccess for name_symbol at ast.py:234>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:234
--# expr-start 83 '<as_bare_entity at ast.py:234>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:234
--# expr-start 82 '<FieldAccess for id at ast.py:234>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:234
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Var_Expr_1.Aspect_Assoc_F_Id;
--# expr-done 82
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 83
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result.Node);
--# end
--# expr-done 84
--# property-call-start AdaNode.is_contract_aspect
Fld_2 := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Fld_1);
--# end
--# expr-done 85
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Var_Expr := Result_Var;
if Var_Expr /= False then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 86 '<Cast to Pragma at ast.py:235>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:235
Cast_Expr_1 := Item_1; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 86
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_2
--# expr-start 90 '<FieldAccess for is_contract_aspect at ast.py:236>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:236
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 89 '<FieldAccess for name_symbol at ast.py:236>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:236
--# expr-start 88 '<as_bare_entity at ast.py:236>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:236
--# expr-start 87 '<FieldAccess for id at ast.py:236>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:236
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_2.Pragma_Node_F_Id;
--# expr-done 87
Is_Null_1 := Fld_3 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Identifier;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 88
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result_1.Node);
--# end
--# expr-done 89
--# property-call-start AdaNode.is_contract_aspect
Fld_5 := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Fld_4);
--# end
--# expr-done 90
Scope_Result_2 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := False;
end if;
   Result_Var_2 := Result_Var_1;
end if;
      if Result_Var_2 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Result_Var_2;
         end loop;
      end;
--# expr-done 91
Is_Null_2 := Find_Result = null; 
--# expr-done 92
Not_Val := not (Is_Null_2); 
--# expr-done 93
         Property_Result := Not_Val;
   --# end
      Finalizer_Scope_8;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_8;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_In_Contract;
--# end
--# property-start AdaNode.in_aspect /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:240
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_In_Aspect
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
            procedure Finalizer_Scope_9 with Inline_Always;
      Var_Expr : Bare_Aspect_Assoc;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld : Bare_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_9 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_9;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 99 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 98 '<AnyQuantifier at ast.py:245>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:245
   --# expr-start 94 '<FieldAccess for parents at ast.py:245>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:245
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 94
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_2
--# expr-start 95 '<Cast to AspectAssoc at ast.py:246>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:246
Cast_Expr := Item_2; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 95
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 97 '<FieldAccess for name_is at ast.py:246>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:246
--# expr-start 96 '<FieldAccess for id at ast.py:246>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:246
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 96
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld, Sym => Name);
--# end
--# expr-done 97
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      Quantifier_Result := Result_Var;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 98
Let_Result := Quantifier_Result; 
--# end
--# expr-done 99
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_9;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_9;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_In_Aspect;
--# end
--# property-start AdaNode.empty_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:249
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Empty_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_10 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_10 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld);
            end Finalizer_Scope_10;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 104 '<FieldAccess for get_empty_env at ast.py:251>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:251
--# expr-start 103 '<Cast to CompilationUnit at ast.py:251>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:251
--# expr-start 102 '<Find at ast.py:250>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:250
   --# expr-start 100 '<FieldAccess for parents at ast.py:250>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:250
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 100
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_3
      --# expr-start 101 '<IsA CompilationUnit at ast.py:250>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:250
Is_A := Item_3 /= null 
and then Item_3.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 101
      if Is_A then
         Find_Result := Item_3; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 102
   if Find_Result = null
      or else Find_Result.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 103
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Compilation_Unit_P_Get_Empty_Env (Node => Cast_Result);
--# expr-done 104
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_10;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_10;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Empty_Env;
--# end
--# property-start AdaNode.is_not_null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:254
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Not_Null
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 107 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 106 '<Not at ast.py:263>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:263
--# expr-start 105 '<IsNull at ast.py:263>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:263
Is_Null := Ent.Node = null; 
--# expr-done 105
Not_Val := not (Is_Null); 
--# expr-done 106
Let_Result := Not_Val; 
--# end
--# expr-done 107
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Not_Null;
--# end
--# property-start AdaNode.eval_discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:265
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Eval_Discrete_Range
  (Node : Bare_Ada_Node
      ; Dr : Internal_Discrete_Range
  )
   return Internal_Eval_Discrete_Range
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind dr Dr
   Property_Result : Internal_Eval_Discrete_Range;
            procedure Finalizer_Scope_12 with Inline_Always;
            procedure Finalizer_Scope_1795 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Scope_Result : Big_Integer_Type;
Big_Int : Big_Integer_Type;
Result_Var : Big_Integer_Type;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Big_Integer_Type;
New_Struct : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Is_Equal : Boolean;
If_Result : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Let_Result : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
            procedure Finalizer_Scope_12 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Big_Int);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (New_Struct);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_12;
            procedure Finalizer_Scope_1795 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1795;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 116 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 115 '<If at ast.py:270>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:270
--# expr-start 108 '<Eq at ast.py:271>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:271
Is_Equal := Dr = No_Discrete_Range; 
--# expr-done 108
if Is_Equal then
   If_Result := raise Property_Error with "Attempting to evaluate a null discrete range";
else
   --# expr-start 114 '<New EvalDiscreteRange at ast.py:276>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:276
--# expr-start 110 '<FieldAccess for eval_as_int at ast.py:281>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:281
--# expr-start 109 '<FieldAccess for high_bound at ast.py:281>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:281
Fld_2 := Dr.High_Bound;
--# expr-done 109
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_3 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 110
--# expr-start 111 '<FieldAccess for low_bound at ast.py:277>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:277
Fld := Dr.Low_Bound;
--# expr-done 111
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind lb Var_Expr
--# expr-start 112 '<FieldAccess for eval_as_int at ast.py:278>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:278
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 112
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1795;
   Result_Var := Scope_Result;
else
   --# expr-start 113 '<BigInteger 0 at ast.py:279>' Big_Int /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:279
Big_Int := Create_Big_Integer ("0"); 
--# expr-done 113
   Result_Var := Big_Int;
end if;
      Inc_Ref (Result_Var);
Inc_Ref (Fld_3);
Inc_Ref (Result_Var);
New_Struct := (High_Bound => Fld_3, Low_Bound => Result_Var); 
--# expr-done 114
   If_Result := New_Struct;
end if;
      Inc_Ref (If_Result);
--# expr-done 115
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 116
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_12;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_12;
                     Finalizer_Scope_1795;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Eval_Discrete_Range;
--# end
--# property-start AdaNode.sym_join /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:285
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Sym_Join
  (Node : Bare_Ada_Node
      ; Syms : Symbol_Type_Array_Access
      ; Sep : String_Type
  )
   return String_Type
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind syms Syms
   --# bind sep Sep
   Property_Result : String_Type;
            procedure Finalizer_Scope_13 with Inline_Always;
            procedure Finalizer_Scope_1796 with Inline_Always;
      Fld : String_Type;
Map_Result : String_Type_Array_Access;
Join_Result : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_13 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Join_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_13;
            procedure Finalizer_Scope_1796 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1796;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 120 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 119 '<Join at ast.py:290>' Join_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:290
--# expr-start 118 '<Map at ast.py:290>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:290
      declare
         Map_Result_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Symbol_Type_Array_Access := Syms;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_4
      --# expr-start 117 '<FieldAccess for image at ast.py:290>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:290
      if Item_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Image (Self => Item_4);
--# expr-done 117
         declare
            Item_To_Append : constant String_Type := Fld;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1796;
         end loop;
      end;
         Map_Result := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := String_Type_Vectors.Get
              (Map_Result_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 118
Join_Result := Join_Strings (Sep, Map_Result); 
--# expr-done 119
Let_Result := Join_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 120
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_13;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_13;
                     Finalizer_Scope_1796;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Sym_Join;
--# end
--# property-start AdaNode.enclosing_compilation_unit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:292
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Enclosing_Compilation_Unit
  (Node : Bare_Ada_Node
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_14 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_14 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_14;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 125 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 124 '<Cast to CompilationUnit at ast.py:304>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:304
--# expr-start 123 '<Find at ast.py:302>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:302
   --# expr-start 121 '<FieldAccess for parents at ast.py:302>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:302
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 121
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_5
      --# expr-start 122 '<IsA CompilationUnit at ast.py:303>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:303
Is_A := Item_5 /= null 
and then Item_5.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 122
      if Is_A then
         Find_Result := Item_5; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 123
   if Find_Result = null
      or else Find_Result.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Find_Result;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 124
Let_Result := Cast_Result; 
--# end
--# expr-done 125
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_14;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_14;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Enclosing_Compilation_Unit;
--# end
--# property-start AdaNode.is_children_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:306
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Children_Env
  (Node : Bare_Ada_Node
      ; Parent : Lexical_Env
      ; Current_Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind parent Parent
   --# bind current_env Current_Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_15 with Inline_Always;
      Is_Equal : Boolean;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
Env_Parent : Lexical_Env := Empty_Env;
Fld : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_15 is
            begin
                     Dec_Ref (Env_Parent);
            end Finalizer_Scope_15;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 132 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 131 '<Cond at ast.py:312>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:312
--# expr-start 126 '<Eq at ast.py:313>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:313
Is_Equal := Equivalent (Parent, Empty_Env); 
--# expr-done 126
if Is_Equal then
   If_Result_2 := False;
else
--# expr-start 127 '<Eq at ast.py:314>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:314
Is_Equal_1 := Equivalent (Current_Env, Parent); 
--# expr-done 127
if Is_Equal_1 then
   If_Result_1 := True;
else
--# expr-start 128 '<IsNull at ast.py:315>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:315
Is_Equal_2 := Equivalent (Current_Env, Empty_Env); 
--# expr-done 128
if Is_Equal_2 then
   If_Result := False;
else
   --# expr-start 130 '<FieldAccess for is_children_env at ast.py:316>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:316
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 129 '<env_parent at ast.py:316>' Env_Parent /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:316
Env_Parent := AST_Envs.Parent (Current_Env); 
--# expr-done 129
--# property-call-start AdaNode.is_children_env
Fld := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Parent, Current_Env => Env_Parent);
--# end
--# expr-done 130
   If_Result := Fld;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 131
Let_Result := If_Result_2; 
--# end
--# expr-done 132
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_15;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_15;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Children_Env;
--# end
--# property-start AdaNode.without_md /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:319
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Without_Md
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity;
      Fld : Bare_Ada_Node;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Info;
Fld_4 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 140 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 139 '<New AdaNode.entity at ???>' New_Struct_1 None
--# expr-start 137 '<New EntityInfo at ast.py:325>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:325
--# expr-start 134 '<FieldAccess for from_rebound at ast.py:328>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:328
--# expr-start 133 '<FieldAccess for info at ast.py:328>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:328
Fld_3 := Ent.Info;
--# expr-done 133
Fld_4 := Fld_3.From_Rebound;
--# expr-done 134
--# expr-start 136 '<FieldAccess for rebindings at ast.py:326>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:326
--# expr-start 135 '<FieldAccess for info at ast.py:326>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:326
Fld_1 := Ent.Info;
--# expr-done 135
Fld_2 := Fld_1.Rebindings;
--# expr-done 136
New_Struct := (From_Rebound => Fld_4, Md => No_Metadata, Rebindings => Fld_2); 
--# expr-done 137
--# expr-start 138 '<FieldAccess for node at ast.py:325>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:325
Fld := Ent.Node;
--# expr-done 138
New_Struct_1 := (Info => New_Struct, Node => Fld); 
--# expr-done 139
Let_Result := New_Struct_1; 
--# end
--# expr-done 140
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Without_Md;
--# end
--# property-start AdaNode.get_uninstantiated_node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:332
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Get_Uninstantiated_Node
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity;
      As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 142 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 141 '<as_bare_entity at ast.py:339>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:339
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 141
Let_Result := If_Result; 
--# end
--# expr-done 142
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Get_Uninstantiated_Node;
--# end
--# property-start AdaNode.complete /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:341
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Iterator_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Completion_Item_Iterator_Access;
            procedure Finalizer_Scope_18 with Inline_Always;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Completion_Item_Array_Access;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Synthetic_Subp_Decl;
Is_Null : Boolean;
Map_Result : Internal_Completion_Item_Array_Access;
Iter : Internal_Completion_Item_Iterator_Access;
Dyn_Var_Bind_Result : Internal_Completion_Item_Iterator_Access;
Let_Result : Internal_Completion_Item_Iterator_Access;
            procedure Finalizer_Scope_18 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Iter);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_18;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 151 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 150 '<bind at ast.py:347>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:347
--# expr-start 143 '<FieldAccess for origin_node at ast.py:348>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:348
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 143
Bound_Origin := Fld; 
--# expr-start 149 '<FieldAccess for to_iterator at ast.py:354>' Iter /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:354
--# expr-start 148 '<Filter at ast.py:349>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:349
   --# expr-start 144 '<FieldAccess for complete_items at ast.py:349>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:349
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.complete_items'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Complete_Items (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 144
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Completion_Item_Array_Access := Fld_1;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_6
         --# expr-start 147 '<IsNull at ast.py:353>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:353
--# expr-start 146 '<Cast to SyntheticSubpDecl at ast.py:353>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:353
--# expr-start 145 '<FieldAccess for decl at ast.py:353>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:353
Fld_2 := Item_6.Decl;
--# expr-done 145
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Synthetic_Subp_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Synthetic_Subp_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Synthetic_Subp_Decl;
   end if;
--# expr-done 146
Is_Null := Cast_Result.Node = null; 
--# expr-done 147
         if Is_Null then
         declare
            Item_To_Append : constant Internal_Completion_Item := Item_6;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 148
Iter := To_Iterator (Map_Result, Self.Unit.Context); 
--# expr-done 149
Dyn_Var_Bind_Result := Iter; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 150
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 151
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_18;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_18;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Complete;
--# end
--# property-start '[dispatcher]AdaNode.complete_items' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Complete_Items
  (Node : Bare_Ada_Node
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Abstract_State_Decl_List | Ada_Constraint_List .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.complete_items
                  Property_Result := Ada_Node_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Alternatives_List =>
                  --# property-call-start AlternativesList.complete_items
                  Property_Result := Alternatives_List_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.complete_items
                  Property_Result := Dotted_Name_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Identifier =>
                  --# property-call-start Identifier.complete_items
                  Property_Result := Identifier_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.complete_items
                  Property_Result := Subtype_Indication_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Complete_Items;
--# end
--# property-start '[dispatcher]AdaNode.complete_item_weight' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Complete_Item_Weight
  (Node : Bare_Ada_Node
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Constant_Present | Ada_Delta_Constraint .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Abort_Stmt | Ada_Call_Stmt .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.complete_item_weight
                  Property_Result := Ada_Node_P_Complete_Item_Weight
                    (Self
                        , Item
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Composite_Constraint =>
                  --# property-call-start CompositeConstraint.complete_item_weight
                  Property_Result := Composite_Constraint_P_Complete_Item_Weight
                    (Self
                        , Item
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.complete_item_weight
                  Property_Result := Dotted_Name_P_Complete_Item_Weight
                    (Self
                        , Item
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Assign_Stmt =>
                  --# property-call-start AssignStmt.complete_item_weight
                  Property_Result := Assign_Stmt_P_Complete_Item_Weight
                    (Self
                        , Item
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Complete_Item_Weight;
--# end
--# property-start AdaNode.complete_item_weight_matching_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:391
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete_Item_Weight_Matching_Type
  (Node : Bare_Ada_Node
      ; Item : Internal_Entity_Basic_Decl
      ; Name : Internal_Entity_Name
      ; Origin : Bare_Ada_Node
  )
   return Integer
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind item Item
   --# bind name Name
   --# bind origin Origin
   Property_Result : Integer;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Type_Expr;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Type_Expr;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Type_Expr;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Type_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Scope_Result_1 : Internal_Entity_Type_Expr;
Result_Var_1 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result : Boolean;
Bound_Imprecise_Fallback : Boolean;
Fld_8 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Type_Expr;
Scope_Result_3 : Internal_Entity_Type_Expr;
Result_Var_3 : Internal_Entity_Type_Expr;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result_2 : Integer;
If_Result_3 : Integer;
Scope_Result_5 : Integer;
Result_Var_5 : Integer;
Let_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 168 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 154 '<Not at ast.py:400>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:400
--# expr-start 153 '<IsNull at ast.py:400>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:400
--# expr-start 152 '<FieldAccess for type_expression at ast.py:400>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:400
      if Item.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Item.Node, E_Info => Item.Info);
--# end
--# expr-done 152
Is_Null := Fld.Node = null; 
--# expr-done 153
Not_Val := not (Is_Null); 
--# expr-done 154
Block_Var := Not_Val; 
--# bind te_not_null Block_Var
--# expr-start 155 '<FieldAccess for type_expression at ast.py:401>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:401
      if Item.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Item.Node, E_Info => Item.Info);
--# end
--# expr-done 155
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 156 '<FieldAccess for designated_type_decl at ast.py:401>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:401
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_2 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 156
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind td Block_Var_1
--# expr-start 157 '<FieldAccess for expr_type at ast.py:404>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:404
      if Item.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Item.Node, Origin => Origin, E_Info => Item.Info);
--# end
--# expr-done 157
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 167 '<Cond at ast.py:405>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:405
--# expr-start 158 '<FieldAccess for referenced_decl at ast.py:407>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:407
      if Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_4 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Name.Node, Imprecise_Fallback => False, E_Info => Name.Info);
--# end
--# expr-done 158
Var_Expr_3 := Fld_4;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 159 '<FieldAccess for type_expression at ast.py:407>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:407
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 159
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Type_Expr;
end if;
Var_Expr_2 := Result_Var_1;
if Var_Expr_2 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 161 '<FieldAccess for matching_assign_type at ast.py:408>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:408
--# expr-start 160 '<FieldAccess for designated_type_decl at ast.py:408>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:408
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_6 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 160
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_6.Node, Expected_Type => Block_Var_1, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 161
Scope_Result_2 := Fld_7;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   If_Result := Block_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_3 := 100;
else
--# expr-start 166 '<bind at ast.py:413>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:413
Bound_Imprecise_Fallback := True; 
--# expr-start 162 '<FieldAccess for referenced_decl at ast.py:415>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:415
      if Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Name.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Name.Info);
--# end
--# expr-done 162
Var_Expr_5 := Fld_8;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 163 '<FieldAccess for type_expression at ast.py:415>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:415
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 163
Scope_Result_3 := Fld_9;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Type_Expr;
end if;
Var_Expr_4 := Result_Var_3;
if Var_Expr_4 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 165 '<FieldAccess for matching_assign_type at ast.py:416>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:416
--# expr-start 164 '<FieldAccess for designated_type_decl at ast.py:416>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:416
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_10 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 164
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_10.Node, Expected_Type => Block_Var_1, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 165
Scope_Result_4 := Fld_11;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
   If_Result_1 := Block_Var;
else
   If_Result_1 := False;
end if;
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 166
if Dyn_Var_Bind_Result then
   If_Result_2 := 70;
else
   If_Result_2 := 50;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 167
Scope_Result_5 := If_Result_3;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := 0;
end if;
Let_Result := Result_Var_5; 
--# end
--# expr-done 168
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Complete_Item_Weight_Matching_Type;
--# end
--# property-start AdaNode.valid_keywords /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:427
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Valid_Keywords
  (Node : Bare_Ada_Node
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_24 with Inline_Always;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Sym_28 : Symbol_Type;
Sym_29 : Symbol_Type;
Sym_30 : Symbol_Type;
Sym_31 : Symbol_Type;
Sym_32 : Symbol_Type;
Sym_33 : Symbol_Type;
Sym_34 : Symbol_Type;
Sym_35 : Symbol_Type;
Sym_36 : Symbol_Type;
Sym_37 : Symbol_Type;
Sym_38 : Symbol_Type;
Sym_39 : Symbol_Type;
Sym_40 : Symbol_Type;
Sym_41 : Symbol_Type;
Sym_42 : Symbol_Type;
Sym_43 : Symbol_Type;
Sym_44 : Symbol_Type;
Sym_45 : Symbol_Type;
Sym_46 : Symbol_Type;
Sym_47 : Symbol_Type;
Sym_48 : Symbol_Type;
Sym_49 : Symbol_Type;
Sym_50 : Symbol_Type;
Sym_51 : Symbol_Type;
Sym_52 : Symbol_Type;
Sym_53 : Symbol_Type;
Sym_54 : Symbol_Type;
Sym_55 : Symbol_Type;
Sym_56 : Symbol_Type;
Sym_57 : Symbol_Type;
Sym_58 : Symbol_Type;
Sym_59 : Symbol_Type;
Sym_60 : Symbol_Type;
Sym_61 : Symbol_Type;
Sym_62 : Symbol_Type;
Sym_63 : Symbol_Type;
Sym_64 : Symbol_Type;
Sym_65 : Symbol_Type;
Sym_66 : Symbol_Type;
Sym_67 : Symbol_Type;
Sym_68 : Symbol_Type;
Sym_69 : Symbol_Type;
Sym_70 : Symbol_Type;
Sym_71 : Symbol_Type;
Sym_72 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_24 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_24;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 243 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 242 '<ArrayLiteral at ast.py:436>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:436
--# expr-start 169 '<Symbol abort at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abort); 
--# expr-done 169
--# expr-start 170 '<Symbol abs at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs_287); 
--# expr-done 170
--# expr-start 171 '<Symbol abstract at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abstract); 
--# expr-done 171
--# expr-start 172 '<Symbol accept at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Accept); 
--# expr-done 172
--# expr-start 173 '<Symbol access at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Access_290); 
--# expr-done 173
--# expr-start 174 '<Symbol aliased at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aliased); 
--# expr-done 174
--# expr-start 175 '<Symbol all at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_All); 
--# expr-done 175
--# expr-start 176 '<Symbol and at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And_295); 
--# expr-done 176
--# expr-start 177 '<Symbol array at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Array); 
--# expr-done 177
--# expr-start 178 '<Symbol at at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_At); 
--# expr-done 178
--# expr-start 179 '<Symbol begin at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Begin); 
--# expr-done 179
--# expr-start 180 '<Symbol body at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Body); 
--# expr-done 180
--# expr-start 181 '<Symbol case at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Case); 
--# expr-done 181
--# expr-start 182 '<Symbol constant at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant); 
--# expr-done 182
--# expr-start 183 '<Symbol declare at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Declare); 
--# expr-done 183
--# expr-start 184 '<Symbol delay at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Delay); 
--# expr-done 184
--# expr-start 185 '<Symbol delta at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Delta_309); 
--# expr-done 185
--# expr-start 186 '<Symbol digits at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Digits_310); 
--# expr-done 186
--# expr-start 187 '<Symbol do at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Do); 
--# expr-done 187
--# expr-start 188 '<Symbol else at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Else); 
--# expr-done 188
--# expr-start 189 '<Symbol elsif at ???>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Elsif); 
--# expr-done 189
--# expr-start 190 '<Symbol end at ???>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_End); 
--# expr-done 190
--# expr-start 191 '<Symbol entry at ???>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Entry); 
--# expr-done 191
--# expr-start 192 '<Symbol exception at ???>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exception); 
--# expr-done 192
--# expr-start 193 '<Symbol exit at ???>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exit); 
--# expr-done 193
--# expr-start 194 '<Symbol for at ???>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_For); 
--# expr-done 194
--# expr-start 195 '<Symbol function at ???>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Function); 
--# expr-done 195
--# expr-start 196 '<Symbol generic at ???>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Generic); 
--# expr-done 196
--# expr-start 197 '<Symbol goto at ???>' Sym_28 None
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Goto); 
--# expr-done 197
--# expr-start 198 '<Symbol if at ???>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_If); 
--# expr-done 198
--# expr-start 199 '<Symbol in at ???>' Sym_30 None
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_In); 
--# expr-done 199
--# expr-start 200 '<Symbol interface at ???>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface_333); 
--# expr-done 200
--# expr-start 201 '<Symbol is at ???>' Sym_32 None
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Is); 
--# expr-done 201
--# expr-start 202 '<Symbol limited at ???>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Limited); 
--# expr-done 202
--# expr-start 203 '<Symbol loop at ???>' Sym_34 None
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Loop); 
--# expr-done 203
--# expr-start 204 '<Symbol mod at ???>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod_343); 
--# expr-done 204
--# expr-start 205 '<Symbol new at ???>' Sym_36 None
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_New); 
--# expr-done 205
--# expr-start 206 '<Symbol not at ???>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not_346); 
--# expr-done 206
--# expr-start 207 '<Symbol null at ???>' Sym_38 None
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Null); 
--# expr-done 207
--# expr-start 208 '<Symbol others at ???>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Others); 
--# expr-done 208
--# expr-start 209 '<Symbol out at ???>' Sym_40 None
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Out); 
--# expr-done 209
--# expr-start 210 '<Symbol of at ???>' Sym_41 None
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Of); 
--# expr-done 210
--# expr-start 211 '<Symbol or at ???>' Sym_42 None
Sym_42 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or_349); 
--# expr-done 211
--# expr-start 212 '<Symbol overriding at ???>' Sym_43 None
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Overriding); 
--# expr-done 212
--# expr-start 213 '<Symbol package at ???>' Sym_44 None
Sym_44 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Package); 
--# expr-done 213
--# expr-start 214 '<Symbol pragma at ???>' Sym_45 None
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pragma); 
--# expr-done 214
--# expr-start 215 '<Symbol private at ???>' Sym_46 None
Sym_46 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Private); 
--# expr-done 215
--# expr-start 216 '<Symbol procedure at ???>' Sym_47 None
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Procedure); 
--# expr-done 216
--# expr-start 217 '<Symbol protected at ???>' Sym_48 None
Sym_48 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Protected); 
--# expr-done 217
--# expr-start 218 '<Symbol raise at ???>' Sym_49 None
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Raise); 
--# expr-done 218
--# expr-start 219 '<Symbol range at ???>' Sym_50 None
Sym_50 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range_363); 
--# expr-done 219
--# expr-start 220 '<Symbol record at ???>' Sym_51 None
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Record); 
--# expr-done 220
--# expr-start 221 '<Symbol rem at ???>' Sym_52 None
Sym_52 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem_366); 
--# expr-done 221
--# expr-start 222 '<Symbol renames at ???>' Sym_53 None
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Renames); 
--# expr-done 222
--# expr-start 223 '<Symbol requeue at ???>' Sym_54 None
Sym_54 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Requeue); 
--# expr-done 223
--# expr-start 224 '<Symbol return at ???>' Sym_55 None
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Return); 
--# expr-done 224
--# expr-start 225 '<Symbol reverse at ???>' Sym_56 None
Sym_56 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Reverse); 
--# expr-done 225
--# expr-start 226 '<Symbol select at ???>' Sym_57 None
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Select); 
--# expr-done 226
--# expr-start 227 '<Symbol separate at ???>' Sym_58 None
Sym_58 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Separate); 
--# expr-done 227
--# expr-start 228 '<Symbol some at ???>' Sym_59 None
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Some); 
--# expr-done 228
--# expr-start 229 '<Symbol subtype at ???>' Sym_60 None
Sym_60 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Subtype); 
--# expr-done 229
--# expr-start 230 '<Symbol synchronized at ???>' Sym_61 None
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Synchronized); 
--# expr-done 230
--# expr-start 231 '<Symbol tagged at ???>' Sym_62 None
Sym_62 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tagged); 
--# expr-done 231
--# expr-start 232 '<Symbol task at ???>' Sym_63 None
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Task); 
--# expr-done 232
--# expr-start 233 '<Symbol terminate at ???>' Sym_64 None
Sym_64 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Terminate); 
--# expr-done 233
--# expr-start 234 '<Symbol then at ???>' Sym_65 None
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Then); 
--# expr-done 234
--# expr-start 235 '<Symbol type at ???>' Sym_66 None
Sym_66 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type); 
--# expr-done 235
--# expr-start 236 '<Symbol until at ???>' Sym_67 None
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Until); 
--# expr-done 236
--# expr-start 237 '<Symbol use at ???>' Sym_68 None
Sym_68 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Use); 
--# expr-done 237
--# expr-start 238 '<Symbol when at ???>' Sym_69 None
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_When); 
--# expr-done 238
--# expr-start 239 '<Symbol while at ???>' Sym_70 None
Sym_70 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_While); 
--# expr-done 239
--# expr-start 240 '<Symbol with at ???>' Sym_71 None
Sym_71 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_With); 
--# expr-done 240
--# expr-start 241 '<Symbol xor at ???>' Sym_72 None
Sym_72 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor_407); 
--# expr-done 241
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1, 3 => Sym_2, 4 => Sym_3, 5 => Sym_4, 6 => Sym_5, 7 => Sym_6, 8 => Sym_7, 9 => Sym_8, 10 => Sym_9, 11 => Sym_10, 12 => Sym_11, 13 => Sym_12, 14 => Sym_13, 15 => Sym_14, 16 => Sym_15, 17 => Sym_16, 18 => Sym_17, 19 => Sym_18, 20 => Sym_19, 21 => Sym_20, 22 => Sym_21, 23 => Sym_22, 24 => Sym_23, 25 => Sym_24, 26 => Sym_25, 27 => Sym_26, 28 => Sym_27, 29 => Sym_28, 30 => Sym_29, 31 => Sym_30, 32 => Sym_31, 33 => Sym_32, 34 => Sym_33, 35 => Sym_34, 36 => Sym_35, 37 => Sym_36, 38 => Sym_37, 39 => Sym_38, 40 => Sym_39, 41 => Sym_40, 42 => Sym_41, 43 => Sym_42, 44 => Sym_43, 45 => Sym_44, 46 => Sym_45, 47 => Sym_46, 48 => Sym_47, 49 => Sym_48, 50 => Sym_49, 51 => Sym_50, 52 => Sym_51, 53 => Sym_52, 54 => Sym_53, 55 => Sym_54, 56 => Sym_55, 57 => Sym_56, 58 => Sym_57, 59 => Sym_58, 60 => Sym_59, 61 => Sym_60, 62 => Sym_61, 63 => Sym_62, 64 => Sym_63, 65 => Sym_64, 66 => Sym_65, 67 => Sym_66, 68 => Sym_67, 69 => Sym_68, 70 => Sym_69, 71 => Sym_70, 72 => Sym_71, 73 => Sym_72)); 
--# expr-done 242
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 243
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_24;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_24;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Valid_Keywords;
--# end
--# property-start AdaNode.generic_instantiations /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:450
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Generic_Instantiations
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Instantiation_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_25 with Inline_Always;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_25 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_25;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 247 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 246 '<FieldAccess for generic_instantiations_internal at ast.py:457>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:457
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 245 '<FieldAccess for rebindings at ast.py:457>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:457
--# expr-start 244 '<FieldAccess for info at ast.py:457>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:457
Fld := Ent.Info;
--# expr-done 244
Fld_1 := Fld.Rebindings;
--# expr-done 245
--# property-call-start AdaNode.generic_instantiations_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations_Internal (Node => Self, R => Fld_1);
--# end
--# expr-done 246
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 247
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_25;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_25;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Generic_Instantiations;
--# end
--# property-start AdaNode.generic_instantiations_internal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:459
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Generic_Instantiations_Internal
  (Node : Bare_Ada_Node
      ; R : Env_Rebindings
  )
   return Internal_Entity_Generic_Instantiation_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind r R
   Property_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_26 with Inline_Always;
      Head : Internal_Entity_Generic_Instantiation;
Tail : Internal_Entity_Generic_Instantiation_Array_Access;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Generic_Instantiation_Array_Access;
Singleton : Internal_Entity_Generic_Instantiation_Array_Access;
Concat_Result : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result : Internal_Entity_Generic_Instantiation_Array_Access;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result_1 : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_26 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Tail);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_26;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 258 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 257 '<If at ast.py:461>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:461
--# expr-start 248 '<Eq at ast.py:462>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:462
Is_Equal := R = null; 
--# expr-done 248
if Is_Equal then
   If_Result_1 := No_Internal_Entity_Generic_Instantiation_Array_Type;
else
   --# expr-start 256 '<Let at ast.py:465>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:465
--# scope-start
--# expr-start 252 '<as_bare_entity at ast.py:467>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:467
--# expr-start 251 '<Cast to GenericInstantiation at ast.py:467>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:467
--# expr-start 250 '<env_node at ast.py:466>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:466
--# expr-start 249 '<FieldAccess for new_env at ast.py:466>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:466
      if R = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := R.New_Env;
Inc_Ref (Fld);
--# expr-done 249
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 250
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Env_Node;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 251
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 252
Head := If_Result; 
--# bind head Head
--# expr-start 254 '<FieldAccess for generic_instantiations_internal at ast.py:468>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:468
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 253 '<FieldAccess for get_parent at ast.py:468>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:468
      if R = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := R.Parent;
--# expr-done 253
--# property-call-start AdaNode.generic_instantiations_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations_Internal (Node => Self, R => Fld_1);
--# end
--# expr-done 254
Tail := Fld_2; Inc_Ref (Tail);
--# bind tail Tail
--# expr-start 255 '<Concat at ast.py:469>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:469
                Singleton := Create_Internal_Entity_Generic_Instantiation_Array (Items_Count => 1);
                Singleton.Items (1) := Head;
Concat_Result := Concat (Singleton, Tail); 
--# expr-done 255
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 256
   If_Result_1 := Let_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 257
Let_Result_1 := If_Result_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 258
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_26;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_26;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Generic_Instantiations_Internal;
--# end
--# property-start AdaNode.remove_rebindings /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:472
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Remove_Rebindings
  (Node : Bare_Ada_Node
      ; Base : Env_Rebindings
      ; Suffix : Env_Rebindings
  )
   return Env_Rebindings
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind base Base
   --# bind suffix Suffix
   Property_Result : Env_Rebindings;
            procedure Finalizer_Scope_27 with Inline_Always;
      Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Is_Equal_2 : Boolean;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Fld_4 : Env_Rebindings;
Fld_5 : Env_Rebindings;
Fld_6 : Env_Rebindings;
If_Result_2 : Env_Rebindings;
If_Result_3 : Env_Rebindings;
Let_Result : Env_Rebindings;
            procedure Finalizer_Scope_27 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_27;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 271 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 270 '<Cond at ast.py:478>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:478
--# expr-start 259 '<IsNull at ast.py:479>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:479
Is_Equal := Base = null; 
--# expr-done 259
if Is_Equal then
   If_Result := True;
else
   --# expr-start 260 '<IsNull at ast.py:479>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:479
Is_Equal_1 := Suffix = null; 
--# expr-done 260
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_3 := Base;
else
--# expr-start 263 '<Eq at ast.py:482>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:482
--# expr-start 261 '<FieldAccess for old_env at ast.py:482>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:482
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Base.Old_Env;
Inc_Ref (Fld);
--# expr-done 261
--# expr-start 262 '<FieldAccess for old_env at ast.py:482>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:482
      if Suffix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Suffix.Old_Env;
Inc_Ref (Fld_1);
--# expr-done 262
Is_Equal_2 := Equivalent (Fld, Fld_1); 
--# expr-done 263
if Is_Equal_2 then
   --# expr-start 266 '<Eq at ast.py:483>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:483
--# expr-start 264 '<FieldAccess for new_env at ast.py:483>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:483
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Base.New_Env;
Inc_Ref (Fld_2);
--# expr-done 264
--# expr-start 265 '<FieldAccess for new_env at ast.py:483>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:483
      if Suffix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Suffix.New_Env;
Inc_Ref (Fld_3);
--# expr-done 265
Is_Equal_3 := Equivalent (Fld_2, Fld_3); 
--# expr-done 266
   If_Result_1 := Is_Equal_3;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 269 '<FieldAccess for remove_rebindings at ast.py:484>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:484
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 267 '<FieldAccess for get_parent at ast.py:484>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:484
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Base.Parent;
--# expr-done 267
--# expr-start 268 '<FieldAccess for get_parent at ast.py:484>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:484
      if Suffix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Suffix.Parent;
--# expr-done 268
--# property-call-start AdaNode.remove_rebindings
Fld_6 := Libadalang.Implementation.Ada_Node_P_Remove_Rebindings (Node => Self, Base => Fld_4, Suffix => Fld_5);
--# end
--# expr-done 269
   If_Result_2 := Fld_6;
else
   If_Result_2 := Base;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 270
Let_Result := If_Result_3; 
--# end
--# expr-done 271
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_27;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_27;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Remove_Rebindings;
--# end
--# property-start AdaNode.logic_val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:493
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Logic_Val
  (Node : Bare_Ada_Node
      ; From_Node : Internal_Entity
      ; Lvar : Logic_Var
  )
   return Internal_Logic_Val_Result
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind from_node From_Node
   --# bind lvar Lvar
   Property_Result : Internal_Logic_Val_Result;
      Block_Var : Boolean;
Fld : Boolean;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
If_Result_1 : Internal_Entity;
New_Struct : Internal_Logic_Val_Result;
Let_Result : Internal_Logic_Val_Result;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 276 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 272 '<FieldAccess for resolve_names_from_closest_entry_point at ast.py:495>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:495
      if From_Node.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => From_Node.Node, E_Info => From_Node.Info);
--# end
--# expr-done 272
Block_Var := Fld; 
--# bind success Block_Var
--# expr-start 275 '<New LogicValResult at ast.py:497>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:497
--# expr-start 274 '<If at ast.py:497>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:497
if Block_Var then
   --# expr-start 273 '<get_value at ast.py:498>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:498
Is_Logic_Var_Defined := Entity_Vars.Is_Defined (Lvar); 
if Is_Logic_Var_Defined then
Eq_Solution := Entity_Vars.Get_Value (Lvar); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 273
   If_Result_1 := If_Result;
else
   If_Result_1 := No_Entity;
end if;
--# expr-done 274
New_Struct := (Success => Block_Var, Value => If_Result_1); 
--# expr-done 275
Let_Result := New_Struct; 
--# end
--# expr-done 276
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Logic_Val;
--# end
--# property-start AdaNode.semantic_parent_helper /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:501
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parent_Helper
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1817 with Inline_Always;
      Var_Expr : Lexical_Env := Empty_Env;
Var_Expr_1 : Internal_Entity;
Env_Node : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Env_Parent : Lexical_Env := Empty_Env;
Fld : Internal_Entity;
Result_Var : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_1817 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Env_Parent);
            end Finalizer_Scope_1817;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 281 '<Block at ???>' Let_Result None
--# scope-start
Var_Expr := Env;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind env Var_Expr
--# expr-start 278 '<as_entity at ast.py:503>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:503
--# expr-start 277 '<env_node at ast.py:503>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:503
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 277
Is_Null := Env_Node = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Env_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 278
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 280 '<FieldAccess for semantic_parent_helper at ast.py:504>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:504
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 279 '<env_parent at ast.py:504>' Env_Parent /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:504
Env_Parent := AST_Envs.Parent (Var_Expr); 
--# expr-done 279
--# property-call-start AdaNode.semantic_parent_helper
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent_Helper (Node => Ent.Node, Env => Env_Parent, E_Info => Ent.Info);
--# end
--# expr-done 280
   Result_Var := Fld;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_1817;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 281
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1817;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Semantic_Parent_Helper;
--# end
--# property-start AdaNode.semantic_parent /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:507
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parent
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_30 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_30 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_30;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 284 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 283 '<FieldAccess for semantic_parent_helper at ast.py:517>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:517
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 282 '<FieldAccess for node_env at ast.py:517>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:517
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 282
--# property-call-start AdaNode.semantic_parent_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent_Helper (Node => Ent.Node, Env => Fld, E_Info => Ent.Info);
--# end
--# expr-done 283
Let_Result := Fld_1; 
--# end
--# expr-done 284
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_30;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_30;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Semantic_Parent;
--# end
--# property-start AdaNode.is_in_top_level_public_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:519
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_In_Top_Level_Public_Part
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Block_Var : Internal_Entity;
Var_Expr : Internal_Entity_Base_Package_Decl;
Fld : Internal_Entity;
Is_Null : Boolean;
Is_A : Boolean;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Package_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Is_A_1 : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Package_Body;
Fld_3 : Boolean;
If_Result_1 : Boolean;
Is_A_2 : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Package_Decl;
Fld_4 : Boolean;
If_Result_2 : Boolean;
Fld_5 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 299 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 285 '<FieldAccess for semantic_parent at ast.py:525>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:525
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 285
Block_Var := Fld; 
--# bind node Block_Var
--# expr-start 298 '<Cond at ast.py:526>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:526
--# expr-start 286 '<IsNull at ast.py:527>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:527
Is_Null := Block_Var.Node = null; 
--# expr-done 286
if Is_Null then
   If_Result_6 := False;
else
--# expr-start 287 '<IsA PrivatePart at ast.py:530>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:530
Is_A := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Private_Part_Range; 
--# expr-done 287
if Is_A then
--# expr-start 289 '<Cast to BasePackageDecl at ast.py:531>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:531
--# expr-start 288 '<parent at ast.py:531>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:531
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 288
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 289
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Package_Decl then
   --# scope-start
--# expr-start 290 '<FieldAccess for is_compilation_unit_root at ast.py:531>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:531
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Var_Expr.Node);
--# end
--# expr-done 290
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_5 := False;
else
--# expr-start 291 '<IsA PackageBody at ast.py:534>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:534
Is_A_1 := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Package_Body_Range; 
--# expr-done 291
if Is_A_1 then
   --# expr-start 293 '<FieldAccess for is_compilation_unit_root at ast.py:535>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:535
--# expr-start 292 '<Cast to PackageBody at ast.py:535>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:535
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Body_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Package_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Package_Body;
   end if;
--# expr-done 292
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Cast_Result_1.Node);
--# end
--# expr-done 293
   If_Result_1 := Fld_3;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_4 := False;
else
--# expr-start 294 '<IsA BasePackageDecl at ast.py:538>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:538
Is_A_2 := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Base_Package_Decl; 
--# expr-done 294
if Is_A_2 then
   --# expr-start 296 '<FieldAccess for is_compilation_unit_root at ast.py:539>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:539
--# expr-start 295 '<Cast to BasePackageDecl at ast.py:539>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:539
Cast_Expr_1 := Block_Var; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Package_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 295
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Cast_Result_2.Node);
--# end
--# expr-done 296
   If_Result_2 := Fld_4;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 297 '<FieldAccess for is_in_top_level_public_part at ast.py:542>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:542
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_in_top_level_public_part
Fld_5 := Libadalang.Implementation.Ada_Node_P_Is_In_Top_Level_Public_Part (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 297
   If_Result_3 := Fld_5;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
--# expr-done 298
Let_Result := If_Result_6; 
--# end
--# expr-done 299
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_In_Top_Level_Public_Part;
--# end
--# property-start AdaNode.parent_basic_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:545
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Basic_Decl
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Generic_Instantiation;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity_Generic_Decl;
Var_Expr_4 : Internal_Entity_Generic_Instantiation;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Generic_Instantiation;
Scope_Result : Internal_Entity_Generic_Instantiation;
Result_Var : Internal_Entity_Generic_Instantiation;
Scope_Result_1 : Internal_Entity_Generic_Instantiation;
Fld_4 : Internal_Entity;
Is_A : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Generic_Decl;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Generic_Instantiation;
Scope_Result_2 : Internal_Entity_Generic_Instantiation;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity_Generic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Result_Var_2 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Cast_Expr_3 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Result_Var_3 : Internal_Entity_Basic_Decl;
If_Result : Internal_Entity_Basic_Decl;
If_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_5 : Internal_Entity_Basic_Decl;
Result_Var_4 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Result_Var_5 : Internal_Entity_Basic_Decl;
Is_A_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 317 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 316 '<If at ast.py:561>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:561
--# expr-start 300 '<IsA ClasswideTypeDecl, DiscreteBaseSubtypeDecl, SynthAnonymousTypeDecl at ast.py:566>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:566
Is_A_2 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Classwide_Type_Decl_Range | Ada_Discrete_Base_Subtype_Decl_Range | Ada_Synth_Anonymous_Type_Decl_Range; 
--# expr-done 300
if Is_A_2 then
   --# expr-start 302 '<FieldAccess for parent_basic_decl at ast.py:568>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:568
--# expr-start 301 '<FieldAccess for semantic_parent at ast.py:568>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:568
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 301
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 302
   If_Result_2 := Fld_1;
else
--# expr-start 303 '<Cast to GenericDecl at ast.py:569>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:569
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Decl;
   end if;
--# expr-done 303
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_1
--# expr-start 305 '<FieldAccess for get_instantiation at ast.py:570>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:570
--# expr-start 304 '<FieldAccess for decl at ast.py:570>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:570
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_2 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 304
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_instantiation
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Get_Instantiation (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 305
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Instantiation;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Scope_Result_1.Node,
         Info => Scope_Result_1.Info);
   Result_Var_5 := Cast_Result_5;
else
--# expr-start 306 '<FieldAccess for semantic_parent at ast.py:571>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:571
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_4 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 306
Var_Expr_2 := Fld_4;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind sp Var_Expr_2
--# expr-start 315 '<Cond at ast.py:572>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:572
--# expr-start 307 '<IsA GenericDecl at ast.py:573>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:573
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Generic_Decl; 
--# expr-done 307
if Is_A then
--# expr-start 308 '<Cast to GenericDecl at ast.py:574>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:574
Cast_Expr_1 := Var_Expr_2; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Decl;
   end if;
--# expr-done 308
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_3
--# expr-start 310 '<FieldAccess for get_instantiation at ast.py:575>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:575
--# expr-start 309 '<FieldAccess for decl at ast.py:575>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:575
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_5 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 309
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_instantiation
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Get_Instantiation (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 310
Var_Expr_4 := Fld_6;
if Var_Expr_4 /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind e Var_Expr_4
Scope_Result_2 := Var_Expr_4;
   --# end
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Scope_Result_2.Node,
         Info => Scope_Result_2.Info);
   Result_Var_1 := Cast_Result_2;
else
Cast_Expr_2 := Var_Expr_3; 
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   Result_Var_1 := Cast_Result_3;
end if;
Scope_Result_3 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_3;
else
   Result_Var_2 := No_Entity_Basic_Decl;
end if;
   If_Result_1 := Result_Var_2;
else
--# expr-start 311 '<IsA GenericSubpInternal, GenericPackageInternal at ast.py:578>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:578
Is_A_1 := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Generic_Subp_Internal_Range | Ada_Generic_Package_Internal_Range; 
--# expr-done 311
if Is_A_1 then
   --# expr-start 312 '<FieldAccess for parent_basic_decl at ast.py:579>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:579
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_7 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 312
   If_Result := Fld_7;
else
--# expr-start 313 '<Cast to BasicDecl at ast.py:581>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:581
Cast_Expr_3 := Var_Expr_2; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_4 := No_Entity_Basic_Decl;
   end if;
--# expr-done 313
Var_Expr_5 := Cast_Result_4;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_5
Scope_Result_4 := Var_Expr_5;
   --# end
   Result_Var_3 := Scope_Result_4;
else
   --# expr-start 314 '<FieldAccess for parent_basic_decl at ast.py:581>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:581
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_8 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 314
   Result_Var_3 := Fld_8;
end if;
   If_Result := Result_Var_3;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 315
Scope_Result_5 := If_Result_1;
   --# end
   Result_Var_4 := Scope_Result_5;
else
   Result_Var_4 := No_Entity_Basic_Decl;
end if;
   Result_Var_5 := Result_Var_4;
end if;
   If_Result_2 := Result_Var_5;
end if;
--# expr-done 316
Let_Result := If_Result_2; 
--# end
--# expr-done 317
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Parent_Basic_Decl;
--# end
--# property-start '[dispatcher]AdaNode.immediate_declarative_region' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Immediate_Declarative_Region
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Declarative_Part | Ada_Public_Part .. Ada_Begin_Block | Ada_Case_Stmt .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.immediate_declarative_region
                  Property_Result := Ada_Node_P_Immediate_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.immediate_declarative_region
                  Property_Result := Basic_Decl_P_Immediate_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Private_Part =>
                  --# property-call-start PrivatePart.immediate_declarative_region
                  Property_Result := Private_Part_P_Immediate_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Decl_Block =>
                  --# property-call-start DeclBlock.immediate_declarative_region
                  Property_Result := Decl_Block_P_Immediate_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Immediate_Declarative_Region;
--# end
--# property-start AdaNode.designated_compilation_unit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:629
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Designated_Compilation_Unit
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type_Array_Access
      ; Kind : Analysis_Unit_Kind
      ; Load_If_Needed : Boolean
         := True
      ; Not_Found_Is_Error : Boolean
         := True
      ; Process_Parents : Boolean
         := True
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   --# bind kind Kind
   --# bind load_if_needed Load_If_Needed
   --# bind not_found_is_error Not_Found_Is_Error
   --# bind process_parents Process_Parents
   Property_Result : Bare_Compilation_Unit;
      Block_Var : Internal_Unit;
Fld : Internal_Unit;
Fld_1 : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 320 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 318 '<FieldAccess for get_unit at ast.py:640>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:640
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Name, Kind => Kind, Load_If_Needed => Load_If_Needed, Not_Found_Is_Error => Not_Found_Is_Error, Process_Parents => Process_Parents);
--# expr-done 318
Block_Var := Fld; 
--# bind designated_analysis_unit Block_Var
--# expr-start 319 '<FieldAccess for compilation_unit_with_name at ast.py:644>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:644
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.compilation_unit_with_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Compilation_Unit_With_Name (Node => Self, Unit => Block_Var, Name => Name);
--# end
--# expr-done 319
Let_Result := Fld_1; 
--# end
--# expr-done 320
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Designated_Compilation_Unit;
--# end
--# property-start AdaNode.compilation_unit_with_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:646
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Compilation_Unit_With_Name
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
      ; Name : Symbol_Type_Array_Access
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind unit Unit
   --# bind name Name
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1835 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Match_0 : Bare_Compilation_Unit;
Match_1 : Bare_Compilation_Unit_List;
Match_2 : Bare_Pragma_Node_List;
Match_3 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Symbol_Type_Array_Access;
Is_Equal : Boolean;
Item_8 : Bare_Compilation_Unit;
Find_Result : Bare_Compilation_Unit;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
Scope_Result : Bare_Compilation_Unit;
Cast_Result_1 : Bare_Compilation_Unit_List;
Let_Result_1 : Bare_Compilation_Unit;
Scope_Result_1 : Bare_Compilation_Unit;
Cast_Result_2 : Bare_Pragma_Node_List;
Let_Result_2 : Bare_Compilation_Unit;
Scope_Result_2 : Bare_Compilation_Unit;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_3 : Bare_Compilation_Unit;
Scope_Result_3 : Bare_Compilation_Unit;
Match_Result : Bare_Compilation_Unit;
Scope_Result_4 : Bare_Compilation_Unit;
Result_Var : Bare_Compilation_Unit;
Let_Result_4 : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1835 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1835;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 326 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 321 '<FieldAccess for root at ast.py:653>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:653
      if Unit = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Unit.Ast_Root;
--# expr-done 321
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 325 '<Match at ast.py:653>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:653
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Compilation_Unit =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind single Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Compilation_Unit_List =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind multi Match_1
--# expr-start 324 '<Find at ast.py:660>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:660
      Find_Result := No_Bare_Ada_Node;
      if Match_1 = null then
      Find_Result := No_Bare_Ada_Node;
      else
      declare
         Collection : constant Bare_Compilation_Unit_List := Match_1;
      begin
         for Untyped_Item_8 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_8 := Untyped_Item_8; 
   --# scope-start
               --# bind c Item_8
      --# expr-start 323 '<Eq at ast.py:661>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:661
--# expr-start 322 '<FieldAccess for syntactic_fully_qualified_name at ast.py:661>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:661
      if Item_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Item_8);
--# end
--# expr-done 322
Is_Equal := Equivalent (Fld_1, Name); 
--# expr-done 323
      if Is_Equal then
         Find_Result := Item_8; 
      end if;
   --# end
      Finalizer_Scope_1835;
               exit when Is_Equal;
         end loop;
      end;
      end if;
--# expr-done 324
Let_Result_1 := Find_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Pragma_Node_List =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Case_Stmt_Alternative_List | Ada_Concat_Operand_List .. Ada_Param_Spec_List | Ada_Select_When_Part_List .. Ada_Case_Stmt_Alternative | Ada_Component_Clause .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Unexpected analysis unit root"; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 325
Scope_Result_4 := Match_Result;
   --# end
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Let_Result_4 := Result_Var; 
--# end
--# expr-done 326
         Property_Result := Let_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1835;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Compilation_Unit_With_Name;
--# end
--# property-start AdaNode.get_unit_root_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:672
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Get_Unit_Root_Decl
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type_Array_Access
      ; Kind : Analysis_Unit_Kind
      ; Load_If_Needed : Boolean
         := True
      ; Not_Found_Is_Error : Boolean
         := True
      ; Process_Parents : Boolean
         := True
  )
   return Bare_Basic_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   --# bind kind Kind
   --# bind load_if_needed Load_If_Needed
   --# bind not_found_is_error Not_Found_Is_Error
   --# bind process_parents Process_Parents
   Property_Result : Bare_Basic_Decl;
      Block_Var : Bare_Compilation_Unit;
Var_Expr : Bare_Compilation_Unit;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Result_Var : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 329 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 327 '<FieldAccess for designated_compilation_unit at ast.py:683>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:683
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Name, Kind => Kind, Load_If_Needed => Load_If_Needed, Not_Found_Is_Error => Not_Found_Is_Error, Process_Parents => Process_Parents);
--# end
--# expr-done 327
Block_Var := Fld; 
--# bind cu Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 328 '<FieldAccess for decl at ast.py:686>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:686
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr);
--# end
--# expr-done 328
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 329
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Get_Unit_Root_Decl;
--# end
--# property-start '[dispatcher]AdaNode.xref_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_All_Present | Ada_Aspect_Spec .. Ada_Pragma_Argument_Assoc | Ada_Component_List .. Ada_Record_Def | Ada_Composite_Constraint_Assoc | Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Synthetic_Formal_Param_Decl .. Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Accept_Stmt_Body | Ada_Null_Subp_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Task_Body | Ada_Error_Decl | Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl | Ada_Label_Decl .. Ada_Named_Stmt_Decl | Ada_Single_Task_Decl | Ada_Compilation_Unit | Ada_Constant_Absent .. Ada_Constant_Present | Ada_Declarative_Part .. Ada_Elsif_Expr_Part | Ada_Case_Expr_Alternative | Ada_Concat_Operand | Ada_Contract_Cases | Ada_Discrete_Subtype_Name | Ada_Synthetic_Identifier | Ada_Handled_Stmts .. Ada_Limited_Present | Ada_Mode_Default .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Pp_Then_Kw | Ada_Private_Absent .. Ada_Quantifier_Some | Ada_Renaming_Clause .. Ada_Reverse_Present | Ada_Error_Stmt | Ada_Subp_Kind_Function .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_Tagged_Present | Ada_Type_Attributes_Repository | Ada_Anonymous_Type_Access_Def | Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_Until_Present | Ada_Variant | Ada_With_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_equation
                  Property_Result := Ada_Node_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.xref_equation
                  Property_Result := Constrained_Array_Indices_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.xref_equation
                  Property_Result := Unconstrained_Array_Indices_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aspect_Assoc =>
                  --# property-call-start AspectAssoc.xref_equation
                  Property_Result := Aspect_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_At_Clause =>
                  --# property-call-start AtClause.xref_equation
                  Property_Result := At_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Def_Clause =>
                  --# property-call-start AttributeDefClause.xref_equation
                  Property_Result := Attribute_Def_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Rep_Clause =>
                  --# property-call-start EnumRepClause.xref_equation
                  Property_Result := Enum_Rep_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Rep_Clause =>
                  --# property-call-start RecordRepClause.xref_equation
                  Property_Result := Record_Rep_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec .. Ada_Synthetic_Unary_Spec =>
                  --# property-call-start BaseSubpSpec.xref_equation
                  Property_Result := Base_Subp_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.xref_equation
                  Property_Result := Entry_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.xref_equation
                  Property_Result := Aggregate_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.xref_equation
                  Property_Result := Iterated_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.xref_equation
                  Property_Result := Param_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.xref_equation
                  Property_Result := Component_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.xref_equation
                  Property_Result := Discriminant_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.xref_equation
                  Property_Result := Generic_Formal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.xref_equation
                  Property_Result := Param_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.xref_equation
                  Property_Result := Subtype_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.xref_equation
                  Property_Result := Incomplete_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Formal_Type_Decl =>
                  --# property-call-start IncompleteFormalTypeDecl.xref_equation
                  Property_Result := Incomplete_Formal_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.xref_equation
                  Property_Result := Protected_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.xref_equation
                  Property_Result := Task_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Concrete_Type_Decl =>
                  --# property-call-start TypeDecl.xref_equation
                  Property_Result := Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Type_Decl =>
                  --# property-call-start FormalTypeDecl.xref_equation
                  Property_Result := Formal_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.xref_equation
                  Property_Result := Formal_Subp_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function =>
                  --# property-call-start ExprFunction.xref_equation
                  Property_Result := Expr_Function_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Renaming_Decl =>
                  --# property-call-start SubpRenamingDecl.xref_equation
                  Property_Result := Subp_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.xref_equation
                  Property_Result := Entry_Body_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.xref_equation
                  Property_Result := Entry_Index_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.xref_equation
                  Property_Result := Exception_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.xref_equation
                  Property_Result := Exception_Handler_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.xref_equation
                  Property_Result := For_Loop_Var_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericInstantiation.xref_equation
                  Property_Result := Generic_Instantiation_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericRenamingDecl.xref_equation
                  Property_Result := Generic_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.xref_equation
                  Property_Result := Number_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.xref_equation
                  Property_Result := Object_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start NoTypeObjectRenamingDecl.xref_equation
                  Property_Result := No_Type_Object_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.xref_equation
                  Property_Result := Package_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.xref_equation
                  Property_Result := Single_Protected_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Stmt_Alternative =>
                  --# property-call-start CaseStmtAlternative.xref_equation
                  Property_Result := Case_Stmt_Alternative_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Clause =>
                  --# property-call-start ComponentClause.xref_equation
                  Property_Result := Component_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Def =>
                  --# property-call-start ComponentDef.xref_equation
                  Property_Result := Component_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Composite_Constraint =>
                  --# property-call-start CompositeConstraint.xref_equation
                  Property_Result := Composite_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Delta_Constraint =>
                  --# property-call-start DeltaConstraint.xref_equation
                  Property_Result := Delta_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Digits_Constraint =>
                  --# property-call-start DigitsConstraint.xref_equation
                  Property_Result := Digits_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Range_Constraint =>
                  --# property-call-start RangeConstraint.xref_equation
                  Property_Result := Range_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Elsif_Stmt_Part =>
                  --# property-call-start ElsifStmtPart.xref_equation
                  Property_Result := Elsif_Stmt_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl_Expr =>
                  --# property-call-start AbstractStateDeclExpr.xref_equation
                  Property_Result := Abstract_State_Decl_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Allocator =>
                  --# property-call-start Allocator.xref_equation
                  Property_Result := Allocator_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
                  --# property-call-start BaseAggregate.xref_equation
                  Property_Result := Base_Aggregate_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.xref_equation
                  Property_Result := Bin_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Box_Expr =>
                  --# property-call-start BoxExpr.xref_equation
                  Property_Result := Box_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Concat_Op =>
                  --# property-call-start ConcatOp.xref_equation
                  Property_Result := Concat_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.xref_equation
                  Property_Result := Case_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.xref_equation
                  Property_Result := If_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decl_Expr =>
                  --# property-call-start DeclExpr.xref_equation
                  Property_Result := Decl_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Membership_Expr =>
                  --# property-call-start MembershipExpr.xref_equation
                  Property_Result := Membership_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.xref_equation
                  Property_Result := Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.xref_equation
                  Property_Result := Call_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.xref_equation
                  Property_Result := Defining_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.xref_equation
                  Property_Result := Dotted_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.xref_equation
                  Property_Result := End_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.xref_equation
                  Property_Result := Explicit_Deref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.xref_equation
                  Property_Result := Qual_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Reduce_Attribute_Ref =>
                  --# property-call-start ReduceAttributeRef.xref_equation
                  Property_Result := Reduce_Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Identifier =>
                  --# property-call-start BaseId.xref_equation
                  Property_Result := Base_Id_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal =>
                  --# property-call-start CharLiteral.xref_equation
                  Property_Result := Char_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.xref_equation
                  Property_Result := Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_String_Literal =>
                  --# property-call-start StringLiteral.xref_equation
                  Property_Result := String_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Null_Literal =>
                  --# property-call-start NullLiteral.xref_equation
                  Property_Result := Null_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Int_Literal =>
                  --# property-call-start IntLiteral.xref_equation
                  Property_Result := Int_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Real_Literal =>
                  --# property-call-start RealLiteral.xref_equation
                  Property_Result := Real_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.xref_equation
                  Property_Result := Target_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Update_Attribute_Ref =>
                  --# property-call-start UpdateAttributeRef.xref_equation
                  Property_Result := Update_Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Paren_Expr =>
                  --# property-call-start ParenExpr.xref_equation
                  Property_Result := Paren_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Quantified_Expr =>
                  --# property-call-start QuantifiedExpr.xref_equation
                  Property_Result := Quantified_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Raise_Expr =>
                  --# property-call-start RaiseExpr.xref_equation
                  Property_Result := Raise_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.xref_equation
                  Property_Result := Un_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Spec =>
                  --# property-call-start ForLoopSpec.xref_equation
                  Property_Result := For_Loop_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_While_Loop_Spec =>
                  --# property-call-start WhileLoopSpec.xref_equation
                  Property_Result := While_Loop_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Others_Designator =>
                  --# property-call-start OthersDesignator.xref_equation
                  Property_Result := Others_Designator_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_equation
                  Property_Result := Pragma_Node_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Range_Spec =>
                  --# property-call-start RangeSpec.xref_equation
                  Property_Result := Range_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Select_When_Part =>
                  --# property-call-start SelectWhenPart.xref_equation
                  Property_Result := Select_When_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
                  --# property-call-start AcceptStmt.xref_equation
                  Property_Result := Accept_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Stmt .. Ada_While_Loop_Stmt =>
                  --# property-call-start BaseLoopStmt.xref_equation
                  Property_Result := Base_Loop_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Begin_Block .. Ada_Decl_Block =>
                  --# property-call-start BlockStmt.xref_equation
                  Property_Result := Block_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Stmt =>
                  --# property-call-start CaseStmt.xref_equation
                  Property_Result := Case_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Extended_Return_Stmt =>
                  --# property-call-start ExtendedReturnStmt.xref_equation
                  Property_Result := Extended_Return_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_If_Stmt =>
                  --# property-call-start IfStmt.xref_equation
                  Property_Result := If_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt =>
                  --# property-call-start NamedStmt.xref_equation
                  Property_Result := Named_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Select_Stmt =>
                  --# property-call-start SelectStmt.xref_equation
                  Property_Result := Select_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abort_Stmt =>
                  --# property-call-start AbortStmt.xref_equation
                  Property_Result := Abort_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Assign_Stmt =>
                  --# property-call-start AssignStmt.xref_equation
                  Property_Result := Assign_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Stmt =>
                  --# property-call-start CallStmt.xref_equation
                  Property_Result := Call_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Delay_Stmt =>
                  --# property-call-start DelayStmt.xref_equation
                  Property_Result := Delay_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exit_Stmt =>
                  --# property-call-start ExitStmt.xref_equation
                  Property_Result := Exit_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Goto_Stmt =>
                  --# property-call-start GotoStmt.xref_equation
                  Property_Result := Goto_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Label =>
                  --# property-call-start Label.xref_equation
                  Property_Result := Label_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Null_Stmt =>
                  --# property-call-start NullStmt.xref_equation
                  Property_Result := Null_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Raise_Stmt =>
                  --# property-call-start RaiseStmt.xref_equation
                  Property_Result := Raise_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Requeue_Stmt =>
                  --# property-call-start RequeueStmt.xref_equation
                  Property_Result := Requeue_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Return_Stmt =>
                  --# property-call-start ReturnStmt.xref_equation
                  Property_Result := Return_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Terminate_Alternative =>
                  --# property-call-start TerminateAlternative.xref_equation
                  Property_Result := Terminate_Alternative_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subunit =>
                  --# property-call-start Subunit.xref_equation
                  Property_Result := Subunit_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Def =>
                  --# property-call-start TaskDef.xref_equation
                  Property_Result := Task_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Access_To_Subp_Def =>
                  --# property-call-start AccessToSubpDef.xref_equation
                  Property_Result := Access_To_Subp_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Access_Def =>
                  --# property-call-start TypeAccessDef.xref_equation
                  Property_Result := Type_Access_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.xref_equation
                  Property_Result := Array_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.xref_equation
                  Property_Result := Derived_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.xref_equation
                  Property_Result := Enum_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.xref_equation
                  Property_Result := Formal_Discrete_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.xref_equation
                  Property_Result := Interface_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.xref_equation
                  Property_Result := Mod_Int_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.xref_equation
                  Property_Result := Private_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def =>
                  --# property-call-start DecimalFixedPointDef.xref_equation
                  Property_Result := Decimal_Fixed_Point_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Floating_Point_Def =>
                  --# property-call-start FloatingPointDef.xref_equation
                  Property_Result := Floating_Point_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start OrdinaryFixedPointDef.xref_equation
                  Property_Result := Ordinary_Fixed_Point_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.xref_equation
                  Property_Result := Record_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.xref_equation
                  Property_Result := Signed_Int_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.xref_equation
                  Property_Result := Anonymous_Type_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.xref_equation
                  Property_Result := Subtype_Indication_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Use_Package_Clause =>
                  --# property-call-start UsePackageClause.xref_equation
                  Property_Result := Use_Package_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Use_Type_Clause =>
                  --# property-call-start UseTypeClause.xref_equation
                  Property_Result := Use_Type_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Value_Sequence =>
                  --# property-call-start ValueSequence.xref_equation
                  Property_Result := Value_Sequence_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Variant_Part =>
                  --# property-call-start VariantPart.xref_equation
                  Property_Result := Variant_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_With_Clause =>
                  --# property-call-start WithClause.xref_equation
                  Property_Result := With_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Xref_Equation;
--# end
--# property-start '[dispatcher]AdaNode.xref_stop_resolution' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Stop_Resolution
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Record_Def | Ada_Composite_Constraint_Assoc | Ada_Param_Assoc .. Ada_Classwide_Type_Decl | Ada_Protected_Type_Decl .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Quantifier_Some | Ada_Renaming_Clause .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_stop_resolution
                  Property_Result := Ada_Node_P_Xref_Stop_Resolution
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.xref_stop_resolution
                  Property_Result := Aggregate_Assoc_P_Xref_Stop_Resolution
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.xref_stop_resolution
                  Property_Result := Iterated_Assoc_P_Xref_Stop_Resolution
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.xref_stop_resolution
                  Property_Result := Incomplete_Type_Decl_P_Xref_Stop_Resolution
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Formal_Type_Decl =>
                  --# property-call-start IncompleteFormalTypeDecl.xref_stop_resolution
                  Property_Result := Incomplete_Formal_Type_Decl_P_Xref_Stop_Resolution
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl_Expr .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Un_Op =>
                  --# property-call-start Expr.xref_stop_resolution
                  Property_Result := Expr_P_Xref_Stop_Resolution
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.xref_stop_resolution
                  Property_Result := Call_Expr_P_Xref_Stop_Resolution
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Range_Spec =>
                  --# property-call-start RangeSpec.xref_stop_resolution
                  Property_Result := Range_Spec_P_Xref_Stop_Resolution
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Xref_Stop_Resolution;
--# end
--# property-start '[dispatcher]AdaNode.stop_resolution_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Stop_Resolution_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.stop_resolution_equation
                  Property_Result := Ada_Node_P_Stop_Resolution_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.stop_resolution_equation
                  Property_Result := Call_Expr_P_Stop_Resolution_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Stop_Resolution_Equation;
--# end
--# property-start '[dispatcher]AdaNode.xref_initial_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Initial_Env
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Pp_Then_Kw | Ada_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_initial_env
                  Property_Result := Ada_Node_P_Xref_Initial_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_initial_env
                  Property_Result := Pragma_Node_P_Xref_Initial_Env
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Xref_Initial_Env;
--# end
--# property-start AdaNode.sub_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:730
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Sub_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_43 with Inline_Always;
      Fld : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_43 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_43;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 334 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 333 '<If at ast.py:739>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:739
--# expr-start 330 '<FieldAccess for xref_stop_resolution at ast.py:739>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:739
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 330
if Fld_2 then
   --# expr-start 331 '<FieldAccess for stop_resolution_equation at ast.py:740>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:740
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.stop_resolution_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Stop_Resolution_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 331
   If_Result := Fld;
else
   --# expr-start 332 '<FieldAccess for xref_equation at ast.py:741>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:741
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 332
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 333
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 334
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_43;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_43;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Sub_Equation;
--# end
--# property-start AdaNode.resolve_children_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:753
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Children_Names
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_45 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Node_Children : Bare_Ada_Node_Array_Access;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld : Boolean;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_1 : Boolean;
If_Result_2 : Boolean;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity;
Fld_2 : Boolean;
If_Result_4 : Boolean;
Fld_3 : Boolean;
If_Result_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_45 is
            begin
                     Dec_Ref (Node_Children);
            end Finalizer_Scope_45;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 346 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 345 '<AllQuantifier at ast.py:759>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:759
   --# expr-start 335 '<children at ast.py:759>' Node_Children /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:759
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Children := Children (Self); 
--# expr-done 335
      Quantifier_Result := True;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Children;
      begin
         for Item_9 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_9
Var_Expr := Item_9;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 344 '<If at ast.py:762>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:762
--# expr-start 336 '<FieldAccess for xref_entry_point at ast.py:763>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:763
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Var_Expr);
--# end
--# expr-done 336
if Fld_3 then
   If_Result_5 := True;
else
--# expr-start 341 '<If at ast.py:765>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:765
--# expr-start 338 '<FieldAccess for xref_stop_resolution at ast.py:766>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:766
--# expr-start 337 '<as_entity at ast.py:766>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:766
Is_Null_1 := Var_Expr = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
As_Entity_1 := (Info => E_Info, Node => Var_Expr); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 337
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 338
if Fld_1 then
   --# expr-start 340 '<FieldAccess for resolve_own_names at ast.py:767>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:767
--# expr-start 339 '<as_entity at ast.py:767>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:767
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 339
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => If_Result.Node, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# expr-done 340
   If_Result_2 := Fld;
else
   If_Result_2 := True;
end if;
--# expr-done 341
if If_Result_2 then
   --# expr-start 343 '<FieldAccess for resolve_children_names at ast.py:769>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:769
--# expr-start 342 '<as_entity at ast.py:769>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:769
Is_Null_2 := Var_Expr = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity;
else
As_Entity_2 := (Info => E_Info, Node => Var_Expr); 
   If_Result_3 := As_Entity_2;
end if;
--# expr-done 342
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => If_Result_3.Node, Env => Env, Origin => Origin, E_Info => If_Result_3.Info);
--# end
--# expr-done 343
   If_Result_4 := Fld_2;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
--# expr-done 344
Scope_Result := If_Result_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
      Quantifier_Result := Result_Var;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
--# expr-done 345
Let_Result := Quantifier_Result; 
--# end
--# expr-done 346
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_45;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_45;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Children_Names;
--# end
--# property-start AdaNode.resolve_names_internal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:774
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_Internal
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 349 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 347 '<FieldAccess for resolve_own_names at ast.py:779>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:779
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# expr-done 347
if Fld then
   --# expr-start 348 '<FieldAccess for resolve_children_names at ast.py:779>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:779
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 348
   If_Result := Fld_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 349
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Names_Internal;
--# end
--# property-start AdaNode.resolve_names_internal_with_eq /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:781
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_Internal_With_Eq
  (Node : Bare_Ada_Node
      ; Additional_Equation : Logic_Equation
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind additional_equation Additional_Equation
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_47 with Inline_Always;
      Fld : Logic_Equation;
And_Pred : Logic_Equation;
Solve_Success : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_47 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_47;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 354 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 352 '<solve at ast.py:787>' Solve_Success /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:787
--# expr-start 351 '<And at ast.py:787>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:787
--# expr-start 350 '<FieldAccess for xref_equation at ast.py:787>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:787
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 350
And_Pred := Create_And (Fld, Additional_Equation, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:787") else null)); 
--# expr-done 351
Solve_Success := Solve_Wrapper (And_Pred, Self); 
--# expr-done 352
if Solve_Success then
   --# expr-start 353 '<FieldAccess for resolve_children_names at ast.py:788>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:788
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 353
   If_Result := Fld_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 354
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_47;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_47;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Names_Internal_With_Eq;
--# end
--# property-start '[dispatcher]AdaNode.xref_entry_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Entry_Point
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Unconstrained_Array_Indices | Ada_Aspect_Spec .. Ada_Pragma_Argument_Assoc | Ada_Component_List .. Ada_Iterated_Assoc | Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Synthetic_Formal_Param_Decl .. Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Accept_Stmt_Body | Ada_Null_Subp_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Task_Body | Ada_Error_Decl | Ada_For_Loop_Var_Decl .. Ada_Generic_Subp_Decl | Ada_Label_Decl .. Ada_Named_Stmt_Decl | Ada_Single_Task_Decl | Ada_Compilation_Unit | Ada_Component_Def .. Ada_Elsif_Expr_Part | Ada_Abstract_State_Decl_Expr .. Ada_Call_Expr | Ada_Discrete_Subtype_Name .. Ada_Dotted_Name | Ada_Explicit_Deref .. Ada_Limited_Present | Ada_While_Loop_Spec .. Ada_Pp_Then_Kw | Ada_Private_Absent .. Ada_Select_When_Part | Ada_Subp_Kind_Function .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_Type_Access_Def | Ada_Derived_Type_Def .. Ada_Until_Present | Ada_Value_Sequence .. Ada_Variant | Ada_With_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_entry_point
                  Property_Result := Ada_Node_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Aspect_Assoc =>
                  --# property-call-start AspectAssoc.xref_entry_point
                  Property_Result := Aspect_Assoc_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_At_Clause .. Ada_Record_Rep_Clause =>
                  --# property-call-start AspectClause.xref_entry_point
                  Property_Result := Aspect_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Synthetic_Unary_Spec =>
                  --# property-call-start BaseSubpSpec.xref_entry_point
                  Property_Result := Base_Subp_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.xref_entry_point
                  Property_Result := Param_Assoc_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.xref_entry_point
                  Property_Result := Component_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.xref_entry_point
                  Property_Result := Discriminant_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.xref_entry_point
                  Property_Result := Generic_Formal_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.xref_entry_point
                  Property_Result := Param_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.xref_entry_point
                  Property_Result := Subtype_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.xref_entry_point
                  Property_Result := Protected_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.xref_entry_point
                  Property_Result := Task_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.xref_entry_point
                  Property_Result := Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start AnonymousTypeDecl.xref_entry_point
                  Property_Result := Anonymous_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Expr_Function =>
                  --# property-call-start ExprFunction.xref_entry_point
                  Property_Result := Expr_Function_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subp_Renaming_Decl =>
                  --# property-call-start SubpRenamingDecl.xref_entry_point
                  Property_Result := Subp_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.xref_entry_point
                  Property_Result := Entry_Body_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.xref_entry_point
                  Property_Result := Entry_Index_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.xref_entry_point
                  Property_Result := Exception_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.xref_entry_point
                  Property_Result := Exception_Handler_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericInstantiation.xref_entry_point
                  Property_Result := Generic_Instantiation_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericRenamingDecl.xref_entry_point
                  Property_Result := Generic_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.xref_entry_point
                  Property_Result := Number_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.xref_entry_point
                  Property_Result := Object_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.xref_entry_point
                  Property_Result := Package_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.xref_entry_point
                  Property_Result := Single_Protected_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Case_Stmt_Alternative =>
                  --# property-call-start CaseStmtAlternative.xref_entry_point
                  Property_Result := Case_Stmt_Alternative_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Component_Clause =>
                  --# property-call-start ComponentClause.xref_entry_point
                  Property_Result := Component_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Elsif_Stmt_Part =>
                  --# property-call-start ElsifStmtPart.xref_entry_point
                  Property_Result := Elsif_Stmt_Part_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.xref_entry_point
                  Property_Result := Defining_Name_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.xref_entry_point
                  Property_Result := End_Name_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_For_Loop_Spec =>
                  --# property-call-start ForLoopSpec.xref_entry_point
                  Property_Result := For_Loop_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_entry_point
                  Property_Result := Pragma_Node_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Accept_Stmt .. Ada_Terminate_Alternative =>
                  --# property-call-start Stmt.xref_entry_point
                  Property_Result := Stmt_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subunit =>
                  --# property-call-start Subunit.xref_entry_point
                  Property_Result := Subunit_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.xref_entry_point
                  Property_Result := Array_Type_Def_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Use_Package_Clause .. Ada_Use_Type_Clause =>
                  --# property-call-start UseClause.xref_entry_point
                  Property_Result := Use_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Variant_Part =>
                  --# property-call-start VariantPart.xref_entry_point
                  Property_Result := Variant_Part_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_With_Clause =>
                  --# property-call-start WithClause.xref_entry_point
                  Property_Result := With_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Xref_Entry_Point;
--# end
--# property-start AdaNode.resolve_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:805
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_49 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_49 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_49;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Resolve_Names,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 360 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 359 '<bind at ast.py:813>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:813
--# expr-start 355 '<FieldAccess for xref_initial_env at ast.py:814>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:814
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_initial_env'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Initial_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 355
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 358 '<bind at ast.py:815>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:815
--# expr-start 356 '<FieldAccess for origin_node at ast.py:815>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:815
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 356
Bound_Origin := Fld_1; 
--# expr-start 357 '<FieldAccess for resolve_names_internal at ast.py:815>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:815
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 357
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 358
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 359
Let_Result := Dyn_Var_Bind_Result_1; 
--# end
--# expr-done 360
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_49;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_49;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Resolve_Names;
--# end
--# property-start AdaNode.resolve_names_from_closest_entry_point_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:818
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_From_Closest_Entry_Point_Impl
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_50 with Inline_Always;
            procedure Finalizer_Scope_1845 with Inline_Always;
      Res : Lexical_Env := Empty_Env;
Var_Expr : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Boolean;
If_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Fld_5 : Boolean;
Bound_Env_1 : Lexical_Env := Empty_Env;
Fld_6 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_7 : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_3 : Lexical_Env := Empty_Env;
If_Result_2 : Lexical_Env := Empty_Env;
If_Result_3 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Fld_8 : Boolean;
If_Result_4 : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_50 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (If_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Res);
                     Dec_Ref (Bound_Env_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
                     Dec_Ref (Dyn_Var_Bind_Result_3);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_50;
            procedure Finalizer_Scope_1845 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1845;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 380 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 379 '<If at ast.py:829>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:829
--# expr-start 361 '<FieldAccess for xref_entry_point at ast.py:833>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:833
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_8 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Ent.Node);
--# end
--# expr-done 361
if Fld_8 then
   --# expr-start 367 '<bind at ast.py:834>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:834
--# expr-start 362 '<FieldAccess for xref_initial_env at ast.py:835>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:835
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_initial_env'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Initial_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 362
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 366 '<bind at ast.py:836>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:836
--# expr-start 363 '<FieldAccess for origin_node at ast.py:837>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:837
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 363
Bound_Origin := Fld_1; 
--# expr-start 365 '<If at ast.py:838>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:838
--# expr-start 364 '<FieldAccess for resolve_own_names at ast.py:838>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:838
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# expr-done 364
if Fld_2 then
   If_Result := Bound_Env;
else
   If_Result := Empty_Env;
end if;
      Inc_Ref (If_Result);
--# expr-done 365
Dyn_Var_Bind_Result := If_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 366
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 367
   If_Result_4 := Dyn_Var_Bind_Result_1;
else
   --# expr-start 378 '<Let at ast.py:842>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:842
--# scope-start
--# expr-start 368 '<parent at ast.py:847>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:847
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 368
Var_Expr := Fld_3;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 369 '<FieldAccess for resolve_names_from_closest_entry_point_impl at ast.py:848>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:848
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point_impl
Fld_4 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point_Impl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 369
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1845;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Res := Result_Var; Inc_Ref (Res);
--# bind res Res
--# expr-start 377 '<Cond at ast.py:850>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:850
--# expr-start 370 '<Eq at ast.py:852>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:852
Is_Equal := Equivalent (Res, Empty_Env); 
--# expr-done 370
if Is_Equal then
   If_Result_3 := Res;
else
--# expr-start 371 '<FieldAccess for xref_stop_resolution at ast.py:858>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:858
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 371
if Fld_5 then
   --# expr-start 376 '<bind at ast.py:859>' Dyn_Var_Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:859
Bound_Env_1 := Res; Inc_Ref (Bound_Env_1);
--# expr-start 375 '<bind at ast.py:861>' Dyn_Var_Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:861
--# expr-start 372 '<FieldAccess for origin_node at ast.py:862>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:862
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 372
Bound_Origin_1 := Fld_6; 
--# expr-start 374 '<If at ast.py:863>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:863
--# expr-start 373 '<FieldAccess for resolve_own_names at ast.py:863>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:863
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Bound_Env_1, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# expr-done 373
if Fld_7 then
   If_Result_1 := Res;
else
   If_Result_1 := Empty_Env;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 374
Dyn_Var_Bind_Result_2 := If_Result_1; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 375
Dyn_Var_Bind_Result_3 := Dyn_Var_Bind_Result_2; Inc_Ref (Dyn_Var_Bind_Result_3);
--# expr-done 376
   If_Result_2 := Dyn_Var_Bind_Result_3;
else
   If_Result_2 := Res;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 377
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 378
   If_Result_4 := Let_Result;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 379
Let_Result_1 := If_Result_4; Inc_Ref (Let_Result_1);
--# end
--# expr-done 380
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_50;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_50;
                     Finalizer_Scope_1845;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Names_From_Closest_Entry_Point_Impl;
--# end
--# property-start AdaNode.resolve_names_from_closest_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:875
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_From_Closest_Entry_Point
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_51 with Inline_Always;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_51 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_51;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 384 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 381 '<FieldAccess for resolve_names_from_closest_entry_point_impl at ast.py:898>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:898
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point_impl
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point_Impl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 381
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind result Block_Var
--# expr-start 383 '<Not at ast.py:899>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:899
--# expr-start 382 '<Eq at ast.py:899>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:899
Is_Equal := Equivalent (Block_Var, Empty_Env); 
--# expr-done 382
Not_Val := not (Is_Equal); 
--# expr-done 383
Let_Result := Not_Val; 
--# end
--# expr-done 384
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_51;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_51;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Names_From_Closest_Entry_Point;
--# end
--# property-start AdaNode.parent_unit_env_helper /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:901
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Unit_Env_Helper
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind unit Unit
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_52 with Inline_Always;
            procedure Finalizer_Scope_1846 with Inline_Always;
            procedure Finalizer_Scope_1847 with Inline_Always;
      Var_Expr : Lexical_Env := Empty_Env;
Var_Expr_1 : Bare_Ada_Node;
Env_Parent : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Unit;
Is_Equal : Boolean;
If_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_52 is
            begin
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_52;
            procedure Finalizer_Scope_1846 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1846;
            procedure Finalizer_Scope_1847 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1847;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 391 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 385 '<env_parent at ast.py:903>' Env_Parent /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:903
Env_Parent := AST_Envs.Parent (Env); 
--# expr-done 385
Var_Expr := Env_Parent;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind parent_env Var_Expr
--# expr-start 386 '<env_node at ast.py:903>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:903
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 386
Var_Expr_1 := Env_Node;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind parent_node Var_Expr_1
--# expr-start 390 '<If at ast.py:904>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:904
--# expr-start 388 '<Eq at ast.py:905>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:905
--# expr-start 387 '<FieldAccess for unit at ast.py:905>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:905
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Var_Expr_1);
--# expr-done 387
Is_Equal := Fld_1 = Unit; 
--# expr-done 388
if Is_Equal then
   --# expr-start 389 '<FieldAccess for parent_unit_env_helper at ast.py:906>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:906
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_unit_env_helper
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env_Helper (Node => Self, Unit => Unit, Env => Var_Expr);
--# end
--# expr-done 389
   If_Result := Fld;
else
   If_Result := Var_Expr;
end if;
      Inc_Ref (If_Result);
--# expr-done 390
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1847;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1846;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 391
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_52;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_52;
                     Finalizer_Scope_1846;
                     Finalizer_Scope_1847;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Parent_Unit_Env_Helper;
--# end
--# property-start AdaNode.parent_unit_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:911
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Unit_Env
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_53 with Inline_Always;
            procedure Finalizer_Scope_1848 with Inline_Always;
      Var_Expr : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld : Internal_Unit;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_53 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_53;
            procedure Finalizer_Scope_1848 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1848;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 395 '<Block at ???>' Let_Result None
--# scope-start
Var_Expr := Env;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind env Var_Expr
--# expr-start 394 '<FieldAccess for parent_unit_env_helper at ast.py:918>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:918
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 393 '<FieldAccess for unit at ast.py:918>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:918
--# expr-start 392 '<env_node at ast.py:918>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:918
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 392
      if Env_Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Env_Node);
--# expr-done 393
--# property-call-start AdaNode.parent_unit_env_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env_Helper (Node => Self, Unit => Fld, Env => Var_Expr);
--# end
--# expr-done 394
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1848;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 395
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_53;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_53;
                     Finalizer_Scope_1848;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Parent_Unit_Env;
--# end
--# property-start AdaNode.std /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:930
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Fld_3 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 402 '<as_bare_entity at ast.py:932>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:932
--# expr-start 401 '<FieldAccess for item at ast.py:932>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:932
--# expr-start 400 '<Cast to LibraryItem at ast.py:932>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:932
--# expr-start 399 '<FieldAccess for body at ast.py:932>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:932
--# expr-start 398 '<Cast to CompilationUnit at ast.py:931>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:931
--# expr-start 397 '<FieldAccess for root at ast.py:931>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:931
--# expr-start 396 '<FieldAccess for standard_unit at ast.py:931>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:931
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Standard_Unit (Node => Self);
--# expr-done 396
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 397
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 398
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Body;
--# expr-done 399
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Library_Item_Range
   then
      Cast_Result_1 := Fld_2;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 400
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Cast_Result_1.Library_Item_F_Item;
--# expr-done 401
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 402
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std;
--# end
--# property-start AdaNode.std_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:939
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_56 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_56 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_56;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 404 '<FieldAccess for children_env at ast.py:940>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:940
--# expr-start 403 '<FieldAccess for std at ast.py:940>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:940
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std
Fld := Libadalang.Implementation.Ada_Node_P_Std (Node => Self);
--# end
--# expr-done 403
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 404
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_56;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_56;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Env;
--# end
--# property-start AdaNode.std_entity /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:944
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Entity
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 408 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 407 '<FieldAccess for std_entity_implem at ast.py:945>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:945
--# expr-start 406 '<FieldAccess for root at ast.py:945>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:945
--# expr-start 405 '<FieldAccess for unit at ast.py:945>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:945
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 405
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 406
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_entity_implem
Fld_2 := Libadalang.Implementation.Ada_Node_P_Std_Entity_Implem (Node => Fld_1, Sym => Sym);
--# end
--# expr-done 407
Let_Result := Fld_2; 
--# end
--# expr-done 408
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Entity;
--# end
--# property-start AdaNode.std_entity_implem /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:951
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Entity_Implem
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_58 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_58 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_58;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Std_Entity_Implem,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Sym);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 411 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 410 '<EnvGet at ast.py:952>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:952
--# expr-start 409 '<FieldAccess for std_env at ast.py:952>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:952
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_env
Fld := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Self);
--# end
--# expr-done 409
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 410
Let_Result := Env_Get_Result; 
--# end
--# expr-done 411
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_58;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_58;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity,
                        As_Internal_Entity => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Std_Entity_Implem;
--# end
--# property-start AdaNode.bool_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:956
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Bool_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 414 '<Cast to BaseTypeDecl at ast.py:957>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:957
--# expr-start 413 '<FieldAccess for std_entity at ast.py:957>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:957
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 412 '<Symbol Boolean at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Boolean); 
--# expr-done 412
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 413
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 414
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Bool_Type;
--# end
--# property-start AdaNode.int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:961
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 417 '<Cast to BaseTypeDecl at ast.py:962>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:962
--# expr-start 416 '<FieldAccess for std_entity at ast.py:962>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:962
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 415 '<Symbol Integer at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer); 
--# expr-done 415
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 416
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 417
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Int_Type;
--# end
--# property-start AdaNode.universal_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:966
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 419 '<FieldAccess for std_entity at ast.py:967>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:967
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 418 '<Symbol Universal_Int_Type_ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Universal_Int_Type); 
--# expr-done 418
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 419
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Int_Type;
--# end
--# property-start AdaNode.universal_real_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:971
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Real_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 421 '<FieldAccess for std_entity at ast.py:972>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:972
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 420 '<Symbol Universal_Real_Type_ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Universal_Real_Type); 
--# expr-done 420
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 421
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Real_Type;
--# end
--# property-start AdaNode.std_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:976
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Char_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 424 '<Cast to BaseTypeDecl at ast.py:977>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:977
--# expr-start 423 '<FieldAccess for std_entity at ast.py:977>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:977
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 422 '<Symbol Character at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Character); 
--# expr-done 422
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 423
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 424
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Char_Type;
--# end
--# property-start AdaNode.std_wide_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:981
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Wide_Char_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 427 '<Cast to BaseTypeDecl at ast.py:982>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:982
--# expr-start 426 '<FieldAccess for std_entity at ast.py:982>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:982
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 425 '<Symbol Wide_Character at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Character); 
--# expr-done 425
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 426
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 427
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Wide_Char_Type;
--# end
--# property-start AdaNode.std_wide_wide_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:987
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Wide_Wide_Char_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 430 '<Cast to BaseTypeDecl at ast.py:988>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:988
--# expr-start 429 '<FieldAccess for std_entity at ast.py:988>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:988
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 428 '<Symbol Wide_Wide_Character at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Character); 
--# expr-done 428
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 429
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 430
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Wide_Wide_Char_Type;
--# end
--# property-start AdaNode.std_root_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:994
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Root_Types
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_66 with Inline_Always;
            procedure Finalizer_Scope_1849 with Inline_Always;
      Var_Expr : Internal_Entity_Package_Decl;
Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Package_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_66 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_66;
            procedure Finalizer_Scope_1849 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1849;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 433 '<Cast to PackageDecl at ast.py:995>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:995
--# expr-start 432 '<FieldAccess for std_entity at ast.py:995>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:995
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 431 '<Symbol root_types_ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Types); 
--# expr-done 431
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 432
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Package_Decl;
   end if;
--# expr-done 433
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Package_Decl then
   --# scope-start
--# expr-start 434 '<FieldAccess for children_env at ast.py:995>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:995
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 434
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1849;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_66;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_66;
                     Finalizer_Scope_1849;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Root_Types;
--# end
--# property-start AdaNode.root_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1001
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_67 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_67 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_67;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 438 '<Cast to BaseTypeDecl at ast.py:1004>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1004
--# expr-start 437 '<EnvGet at ast.py:1002>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1002
--# expr-start 435 '<FieldAccess for std_root_types at ast.py:1002>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1002
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_root_types
Fld := Libadalang.Implementation.Ada_Node_P_Std_Root_Types (Node => Self);
--# end
--# expr-done 435
--# expr-start 436 '<Symbol root_integer at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Integer); 
--# expr-done 436
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 437
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 438
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_67;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_67;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Int_Type;
--# end
--# property-start AdaNode.root_real_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1007
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Real_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_68 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_68 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_68;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 442 '<Cast to BaseTypeDecl at ast.py:1010>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1010
--# expr-start 441 '<EnvGet at ast.py:1008>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1008
--# expr-start 439 '<FieldAccess for std_root_types at ast.py:1008>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1008
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_root_types
Fld := Libadalang.Implementation.Ada_Node_P_Std_Root_Types (Node => Self);
--# end
--# expr-done 439
--# expr-start 440 '<Symbol root_real at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Real); 
--# expr-done 440
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 441
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 442
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_68;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_68;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Real_Type;
--# end
--# property-start AdaNode.system_address_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1014
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_System_Address_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_69 with Inline_Always;
            procedure Finalizer_Scope_1850 with Inline_Always;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_69 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_69;
            procedure Finalizer_Scope_1850 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1850;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 449 '<Cast to BaseTypeDecl at ast.py:1017>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1017
--# expr-start 448 '<EnvGet at ast.py:1016>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1016
--# expr-start 445 '<FieldAccess for get_unit_root_decl at ast.py:1015>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1015
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 444 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 443 '<Symbol System at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 443
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 444
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 445
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 446 '<FieldAccess for children_env at ast.py:1016>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1016
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 446
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1850;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 447 '<Symbol Address at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address); 
--# expr-done 447
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_1, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 448
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 449
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_69;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_69;
                     Finalizer_Scope_1850;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_System_Address_Type;
--# end
--# property-start AdaNode.root_type_ops_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1021
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Type_Ops_Impl
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_70 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_70 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_70;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Root_Type_Ops_Impl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Sym);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
      end if;
      begin
   --# scope-start
         --# expr-start 456 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 455 '<FilterMap at ast.py:1029>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1029
   --# expr-start 451 '<EnvGet at ast.py:1027>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1027
--# expr-start 450 '<FieldAccess for std_root_types at ast.py:1027>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1027
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_root_types
Fld := Libadalang.Implementation.Ada_Node_P_Std_Root_Types (Node => Self);
--# end
--# expr-done 450
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 451
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_10 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_10
         --# expr-start 453 '<FieldAccess for is_subprogram at ast.py:1031>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1031
--# expr-start 452 '<Cast to BasicDecl at ast.py:1031>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1031
Cast_Expr_1 := Item_10; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 452
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Cast_Result_1.Node);
--# end
--# expr-done 453
         if Fld_1 then
      --# expr-start 454 '<Cast to BasicDecl at ast.py:1030>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1030
Cast_Expr := Item_10; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 454
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 455
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 456
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_70;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_70;
               if Self /= null then
                     Add_Memoized_Error
                       (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
               end if;
            raise;
      end;
      if Self /= null then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl_Array_Access,
                        As_Internal_Entity_Basic_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
      end if;
   return Property_Result;
end Ada_Node_P_Root_Type_Ops_Impl;
--# end
--# property-start AdaNode.root_type_ops /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1034
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Type_Ops
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_71 with Inline_Always;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_71 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_71;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 460 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 459 '<FieldAccess for root_type_ops_impl at ast.py:1041>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1041
--# expr-start 458 '<FieldAccess for root at ast.py:1041>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1041
--# expr-start 457 '<FieldAccess for unit at ast.py:1041>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1041
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 457
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 458
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_type_ops_impl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Root_Type_Ops_Impl (Node => Fld_1, Sym => Sym);
--# end
--# expr-done 459
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 460
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_71;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_71;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Type_Ops;
--# end
--# property-start AdaNode.exc_id_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1043
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Exc_Id_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_72 with Inline_Always;
            procedure Finalizer_Scope_1852 with Inline_Always;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_72 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_72;
            procedure Finalizer_Scope_1852 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1852;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 468 '<Cast to BaseTypeDecl at ast.py:1047>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1047
--# expr-start 467 '<EnvGet at ast.py:1046>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1046
--# expr-start 464 '<FieldAccess for get_unit_root_decl at ast.py:1044>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1044
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 463 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 461 '<Symbol Ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 461
--# expr-start 462 '<Symbol Exceptions at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exceptions); 
--# expr-done 462
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 463
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 464
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 465 '<FieldAccess for children_env at ast.py:1046>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1046
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 465
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1852;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 466 '<Symbol Exception_Id at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exception_Id); 
--# expr-done 466
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 467
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 468
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_72;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_72;
                     Finalizer_Scope_1852;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Exc_Id_Type;
--# end
--# property-start AdaNode.task_id_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1053
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Task_Id_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_73 with Inline_Always;
            procedure Finalizer_Scope_1853 with Inline_Always;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_73 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_73;
            procedure Finalizer_Scope_1853 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1853;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 476 '<Cast to BaseTypeDecl at ast.py:1057>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1057
--# expr-start 475 '<EnvGet at ast.py:1056>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1056
--# expr-start 472 '<FieldAccess for get_unit_root_decl at ast.py:1054>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1054
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 471 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 469 '<Symbol Ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 469
--# expr-start 470 '<Symbol Task_Identification at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Task_Identification); 
--# expr-done 470
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 471
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 472
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 473 '<FieldAccess for children_env at ast.py:1056>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1056
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 473
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1853;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 474 '<Symbol Task_Id at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Task_Id); 
--# expr-done 474
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 475
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 476
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_73;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_73;
                     Finalizer_Scope_1853;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Task_Id_Type;
--# end
--# property-start AdaNode.root_buffer_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1062
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Buffer_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_74 with Inline_Always;
            procedure Finalizer_Scope_1854 with Inline_Always;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_3 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_74 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_74;
            procedure Finalizer_Scope_1854 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1854;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 485 '<Cast to BaseTypeDecl at ast.py:1066>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1066
--# expr-start 484 '<EnvGet at ast.py:1065>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1065
--# expr-start 481 '<FieldAccess for get_unit_root_decl at ast.py:1063>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1063
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 480 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 477 '<Symbol Ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 477
--# expr-start 478 '<Symbol Strings at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Strings); 
--# expr-done 478
--# expr-start 479 '<Symbol Text_Buffers at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Text_Buffers); 
--# expr-done 479
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1, 3 => Sym_2)); 
--# expr-done 480
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 481
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 482 '<FieldAccess for children_env at ast.py:1065>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1065
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 482
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1854;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 483 '<Symbol Root_Buffer_Type at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Buffer_Type); 
--# expr-done 483
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_3, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 484
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 485
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_74;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_74;
                     Finalizer_Scope_1854;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Buffer_Type;
--# end
--# property-start AdaNode.root_stream_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1071
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Stream_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_75 with Inline_Always;
            procedure Finalizer_Scope_1855 with Inline_Always;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_75 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_75;
            procedure Finalizer_Scope_1855 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1855;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 495 '<Cast to BaseTypeDecl at ast.py:1074>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1074
--# expr-start 494 '<FieldAccess for classwide_type at ast.py:1074>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1074
--# expr-start 493 '<Cast to BaseTypeDecl at ast.py:1074>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1074
--# expr-start 492 '<EnvGet at ast.py:1073>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1073
--# expr-start 489 '<FieldAccess for get_unit_root_decl at ast.py:1072>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1072
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 488 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 486 '<Symbol Ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 486
--# expr-start 487 '<Symbol Streams at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Streams); 
--# expr-done 487
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 488
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 489
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 490 '<FieldAccess for children_env at ast.py:1073>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1073
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 490
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1855;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 491 '<Symbol Root_Stream_Type at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Stream_Type); 
--# expr-done 491
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 492
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 493
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 494
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 495
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_75;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_75;
                     Finalizer_Scope_1855;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Stream_Type;
--# end
--# property-start AdaNode.has_private_part_parent /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1079
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Private_Part_Parent
  (Node : Bare_Ada_Node
      ; Barrier : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind barrier Barrier
   Property_Result : Boolean;
            procedure Finalizer_Scope_76 with Inline_Always;
      Block_Var : Bare_Ada_Node;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Is_Null : Boolean;
Is_A : Boolean;
If_Result : Boolean;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_76 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_76;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 503 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 497 '<env_node at ast.py:1090>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1090
--# expr-start 496 '<FieldAccess for node_env at ast.py:1090>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1090
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 496
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 497
Block_Var := Env_Node; 
--# bind parent Block_Var
--# expr-start 498 '<IsNull at ast.py:1092>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1092
Is_Null := Block_Var = null; 
--# expr-done 498
if Is_Null then
   If_Result := True;
else
   --# expr-start 499 '<IsA PrivatePart at ast.py:1093>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1093
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Private_Part_Range; 
--# expr-done 499
   If_Result := Is_A;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 501 '<Not at ast.py:1094>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1094
--# expr-start 500 '<Eq at ast.py:1094>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1094
Cast_Expr := Block_Var; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Barrier; 
--# expr-done 500
Not_Val := not (Is_Equal); 
--# expr-done 501
if Not_Val then
   --# expr-start 502 '<FieldAccess for has_private_part_parent at ast.py:1095>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1095
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_private_part_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Private_Part_Parent (Node => Block_Var, Barrier => Barrier);
--# end
--# expr-done 502
   If_Result_1 := Fld_1;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 503
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_76;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_76;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_Private_Part_Parent;
--# end
--# property-start AdaNode.has_private_with_visibility /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1098
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Private_With_Visibility
  (Node : Bare_Ada_Node
      ; Self_Cu : Bare_Compilation_Unit
      ; Refd_Unit : Internal_Unit
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind self_cu Self_Cu
   --# bind refd_unit Refd_Unit
   Property_Result : Boolean;
            procedure Finalizer_Scope_77 with Inline_Always;
      Var_Expr : Bare_Library_Item;
Decl : Bare_Basic_Decl;
Fld : Internal_Unit;
Is_Equal : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Fld_2 : Bare_Private_Node;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_4 : Bare_Basic_Decl;
Fld_5 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_6 : Boolean;
Is_A : Boolean;
Fld_7 : Internal_Unit_Array_Access;
Is_Equal_1 : Boolean;
Quantifier_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
If_Result_3 : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_77 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_77;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 520 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 505 '<Eq at ast.py:1108>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1108
--# expr-start 504 '<FieldAccess for unit at ast.py:1108>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1108
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 504
Is_Equal := Fld = Refd_Unit; 
--# expr-done 505
if Is_Equal then
   If_Result := True;
else
--# expr-start 507 '<Cast to LibraryItem at ast.py:1111>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1111
--# expr-start 506 '<FieldAccess for body at ast.py:1111>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1111
      if Self_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self_Cu.Compilation_Unit_F_Body;
--# expr-done 506
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Library_Item_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 507
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 509 '<FieldAccess for as_bool at ast.py:1111>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1111
--# expr-start 508 '<FieldAccess for has_private at ast.py:1111>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1111
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr.Library_Item_F_Has_Private;
--# expr-done 508
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Private.as_bool'
Fld_3 := Libadalang.Implementation.Dispatcher_Private_Node_P_As_Bool (Node => Fld_2);
--# end
--# expr-done 509
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_3 := True;
else
   --# expr-start 519 '<Let at ast.py:1113>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1113
--# scope-start
--# expr-start 510 '<FieldAccess for decl at ast.py:1113>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1113
      if Self_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self_Cu);
--# end
--# expr-done 510
Decl := Fld_4; 
--# bind decl Decl
--# expr-start 518 '<If at ast.py:1113>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1113
--# expr-start 511 '<IsA BasePackageDecl, GenericPackageDecl at ast.py:1116>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1116
Is_A := Decl /= null 
and then Decl.Kind in Ada_Base_Package_Decl | Ada_Generic_Package_Decl_Range; 
--# expr-done 511
if Is_A then
   --# expr-start 514 '<Contains at ast.py:1119>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1119
   --# expr-start 512 '<FieldAccess for privately_imported_units at ast.py:1119>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1119
      if Self_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.privately_imported_units
Fld_7 := Libadalang.Implementation.Compilation_Unit_P_Privately_Imported_Units (Node => Self_Cu);
--# end
--# expr-done 512
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Unit_Array_Access := Fld_7;
      begin
         for Item_11 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_11
      --# expr-start 513 '<Eq at ???>' Is_Equal_1 None
Is_Equal_1 := Item_11 = Refd_Unit; 
--# expr-done 513
      Quantifier_Result := Is_Equal_1;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 514
   If_Result_1 := Quantifier_Result;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 517 '<FieldAccess for has_private_part_parent at ast.py:1121>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1121
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 516 '<env_node at ast.py:1121>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1121
--# expr-start 515 '<FieldAccess for children_env at ast.py:1121>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1121
      if Decl = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Decl);
--# expr-done 515
Env_Node := AST_Envs.Env_Node (Fld_5); 
--# expr-done 516
--# property-call-start AdaNode.has_private_part_parent
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_Private_Part_Parent (Node => Self, Barrier => Env_Node);
--# end
--# expr-done 517
   If_Result_2 := Fld_6;
else
   If_Result_2 := True;
end if;
--# expr-done 518
Let_Result := If_Result_2; 
--# end
--# expr-done 519
   If_Result_3 := Let_Result;
end if;
Let_Result_1 := If_Result_3; 
--# end
--# expr-done 520
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_77;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_77;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_Private_With_Visibility;
--# end
--# property-start AdaNode.has_with_visibility /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1126
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_With_Visibility
  (Node : Bare_Ada_Node
      ; Refd_Unit : Internal_Unit
      ; Omit_Privacy_Check : Boolean
         := False
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind refd_unit Refd_Unit
   --# bind omit_privacy_check Omit_Privacy_Check
   Property_Result : Boolean;
            procedure Finalizer_Scope_78 with Inline_Always;
      Block_Var : Bare_Compilation_Unit;
Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Internal_Entity_Base_Subp_Body;
Var_Expr_2 : Bare_Compilation_Unit;
Var_Expr_3 : Bare_Compilation_Unit;
Var_Expr_4 : Bare_Compilation_Unit;
Fld : Bare_Compilation_Unit;
Fld_1 : Internal_Unit;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_4 : Bare_Basic_Decl;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_2 : Boolean;
Fld_8 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Bare_Compilation_Unit;
Fld_11 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Cast_Expr : Bare_Compilation_Unit;
Cast_Result_1 : Bare_Compilation_Unit;
Is_Equal : Boolean;
If_Result_4 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_12 : Boolean;
If_Result_5 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_6 : Boolean;
Fld_13 : Bare_Ada_Node;
Cast_Result_2 : Bare_Compilation_Unit;
Fld_14 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_78 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_78;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 543 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 521 '<FieldAccess for enclosing_compilation_unit at ast.py:1135>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1135
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 521
Block_Var := Fld; 
--# bind cu Block_Var
--# expr-start 523 '<FieldAccess for is_referenced_from at ast.py:1137>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1137
      if Refd_Unit = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 522 '<FieldAccess for unit at ast.py:1137>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1137
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 522
Fld_2 := Libadalang.Implementation.Is_Referenced_From (Self => Refd_Unit, Unit => Fld_1);
--# expr-done 523
if Fld_2 then
if Omit_Privacy_Check then
   If_Result := True;
else
   --# expr-start 524 '<FieldAccess for has_private_with_visibility at ast.py:1139>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1139
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_private_with_visibility
Fld_3 := Libadalang.Implementation.Ada_Node_P_Has_Private_With_Visibility (Node => Self, Self_Cu => Block_Var, Refd_Unit => Refd_Unit);
--# end
--# expr-done 524
   If_Result := Fld_3;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 528 '<env_node at ast.py:1147>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1147
--# expr-start 527 '<FieldAccess for parent_unit_env at ast.py:1141>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1141
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 526 '<FieldAccess for children_env at ast.py:1146>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1146
--# expr-start 525 '<FieldAccess for decl at ast.py:1146>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1146
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Block_Var);
--# end
--# expr-done 525
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Fld_4);
--# expr-done 526
--# property-call-start AdaNode.parent_unit_env
Fld_6 := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env (Node => Self, Env => Fld_5);
--# end
--# expr-done 527
Env_Node := AST_Envs.Env_Node (Fld_6); 
--# expr-done 528
Var_Expr := Env_Node;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 529 '<FieldAccess for has_with_visibility at ast.py:1147>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1147
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_with_visibility
Fld_7 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Var_Expr, Refd_Unit => Refd_Unit, Omit_Privacy_Check => True);
--# end
--# expr-done 529
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_2 := Result_Var;
end if;
if If_Result_2 then
   If_Result_6 := True;
else
--# expr-start 532 '<Cast to BaseSubpBody at ast.py:1159>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1159
--# expr-start 531 '<as_bare_entity at ast.py:1159>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1159
--# expr-start 530 '<FieldAccess for decl at ast.py:1159>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1159
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_8 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Block_Var);
--# end
--# expr-done 530
Is_Null := Fld_8 = null; 
if Is_Null then
   If_Result_3 := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_8); 
   If_Result_3 := As_Entity;
end if;
--# expr-done 531
   if If_Result_3.Node = null
      or else If_Result_3.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => If_Result_3.Node,
         Info => If_Result_3.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 532
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Body then
   --# scope-start
   --# bind b Var_Expr_1
--# expr-start 539 '<If at ast.py:1160>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1160
--# expr-start 533 '<FieldAccess for is_library_item at ast.py:1161>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1161
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Var_Expr_1.Node);
--# end
--# expr-done 533
if Fld_12 then
--# expr-start 535 '<FieldAccess for referenced_unit at ast.py:1162>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1162
--# expr-start 534 '<FieldAccess for defining_name at ast.py:1162>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1162
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 534
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_unit
Fld_10 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Fld_9.Node, Kind => Unit_Specification, Not_Found_Is_Error => False);
--# end
--# expr-done 535
Var_Expr_2 := Fld_10;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind u Var_Expr_2
--# expr-start 538 '<If at ast.py:1164>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1164
--# expr-start 536 '<Eq at ast.py:1170>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1170
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Is_Equal := Cast_Result_1 = Var_Expr_2; 
--# expr-done 536
if Is_Equal then
   If_Result_4 := False;
else
Var_Expr_3 := Var_Expr_2;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 537 '<FieldAccess for has_with_visibility at ast.py:1172>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1172
      if Var_Expr_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_with_visibility
Fld_11 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Var_Expr_3, Refd_Unit => Refd_Unit, Omit_Privacy_Check => True);
--# end
--# expr-done 537
Scope_Result_1 := Fld_11;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_4 := Result_Var_1;
end if;
--# expr-done 538
Scope_Result_2 := If_Result_4;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_5 := Result_Var_2;
else
   If_Result_5 := False;
end if;
--# expr-done 539
Scope_Result_3 := If_Result_5;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_6 := Result_Var_3;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
--# expr-start 541 '<Cast to CompilationUnit at ast.py:1188>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1188
--# expr-start 540 '<FieldAccess for root at ast.py:1188>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1188
      if Refd_Unit = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Refd_Unit.Ast_Root;
--# expr-done 540
   if Fld_13 = null
      or else Fld_13.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_2 := Fld_13;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 541
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 542 '<FieldAccess for is_text_io_child at ast.py:1188>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1188
      if Var_Expr_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_text_io_child
Fld_14 := Libadalang.Implementation.Compilation_Unit_P_Is_Text_Io_Child (Node => Var_Expr_4);
--# end
--# expr-done 542
Scope_Result_4 := Fld_14;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
   If_Result_7 := Result_Var_4;
end if;
Let_Result := If_Result_7; 
--# end
--# expr-done 543
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_78;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_78;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_With_Visibility;
--# end
--# property-start AdaNode.has_visibility /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1191
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Visibility
  (Node : Bare_Ada_Node
      ; Other_Entity : Internal_Entity
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind other_entity Other_Entity
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Generic_Package_Instantiation;
Var_Expr_1 : Internal_Entity_Package_Renaming_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Instantiation;
Fld : Internal_Entity_Info;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Package_Renaming_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Fld_6 : Internal_Unit;
Fld_7 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 556 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 544 '<Cast to GenericPackageInstantiation at ast.py:1196>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1196
Cast_Expr := Other_Entity; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 544
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Generic_Package_Instantiation then
   --# scope-start
--# expr-start 546 '<FieldAccess for from_rebound at ast.py:1196>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1196
--# expr-start 545 '<FieldAccess for info at ast.py:1196>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1196
Fld := Var_Expr.Info;
--# expr-done 545
Fld_1 := Fld.From_Rebound;
--# expr-done 546
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result := True;
else
--# expr-start 547 '<Cast to PackageRenamingDecl at ast.py:1198>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1198
Cast_Expr_1 := Other_Entity; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 547
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Package_Renaming_Decl then
   --# scope-start
--# expr-start 549 '<FieldAccess for from_rebound at ast.py:1198>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1198
--# expr-start 548 '<FieldAccess for info at ast.py:1198>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1198
Fld_2 := Var_Expr_1.Info;
--# expr-done 548
Fld_3 := Fld_2.From_Rebound;
--# expr-done 549
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 552 '<Not at ast.py:1201>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1201
--# expr-start 551 '<FieldAccess for is_compilation_unit_root at ast.py:1201>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1201
--# expr-start 550 '<Cast to BasicDecl at ast.py:1201>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1201
Cast_Expr_2 := Other_Entity; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 550
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Cast_Result_2.Node);
--# end
--# expr-done 551
Not_Val := not (Fld_4); 
--# expr-done 552
   If_Result_1 := Not_Val;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 555 '<FieldAccess for has_with_visibility at ast.py:1204>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1204
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 554 '<FieldAccess for unit at ast.py:1204>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1204
--# expr-start 553 '<FieldAccess for node at ast.py:1204>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1204
Fld_5 := Other_Entity.Node;
--# expr-done 553
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Unit (Node => Fld_5);
--# expr-done 554
--# property-call-start AdaNode.has_with_visibility
Fld_7 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_6);
--# end
--# expr-done 555
   If_Result_2 := Fld_7;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 556
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_Visibility;
--# end
--# property-start AdaNode.resolve_generic_actual /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1207
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Generic_Actual
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1866 with Inline_Always;
      Match_0 : Internal_Entity_Anonymous_Expr_Decl;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Attribute_Ref;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Result_Var_2 : Internal_Entity;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Anonymous_Expr_Decl;
Cast_Expr_1 : Internal_Entity_Anonymous_Expr_Decl;
Cast_Result_4 : Internal_Entity;
Let_Result : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_1 : Internal_Entity;
Scope_Result_4 : Internal_Entity;
Cast_Result_6 : Internal_Entity;
Let_Result_2 : Internal_Entity;
Scope_Result_5 : Internal_Entity;
Match_Result : Internal_Entity;
Let_Result_3 : Internal_Entity;
            procedure Finalizer_Scope_1866 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1866;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 564 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 563 '<Match at ast.py:1212>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1212
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Expr_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Anonymous_Expr_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind aod Match_0
Cast_Expr_1 := Match_0; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind n Match_1
--# expr-start 558 '<Cast to AdaNode.entity at ast.py:1222>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1222
--# expr-start 557 '<FieldAccess for name_designated_type at ast.py:1222>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1222
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 557
      Cast_Result := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 558
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 559 '<Cast to AttributeRef at ast.py:1224>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1224
Cast_Expr := Match_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Attribute_Ref;
   end if;
--# expr-done 559
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Attribute_Ref then
   --# scope-start
--# expr-start 560 '<FieldAccess for attribute_subprogram at ast.py:1224>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1224
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_1 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 560
Scope_Result_1 := Fld_1;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
      Cast_Result_2 := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   Result_Var_1 := Cast_Result_2;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_2 := Var_Expr;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 562 '<at at ast.py:1226>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1226
--# expr-start 561 '<FieldAccess for all_env_elements at ast.py:1226>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1226
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 561
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 562
   Result_Var_2 := Get_Result;
end if;
Let_Result_1 := Result_Var_2; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
      Finalizer_Scope_1866;
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Abstract_State_Decl | Ada_Component_Decl .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind _ Match_2
Let_Result_2 := No_Entity; 
--# end
Scope_Result_5 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 563
Let_Result_3 := Match_Result; 
--# end
--# expr-done 564
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1866;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Generic_Actual;
--# end
--# property-start AdaNode.top_level_use_package_clauses /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1231
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Package_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_81 with Inline_Always;
            procedure Finalizer_Scope_1872 with Inline_Always;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_13 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Package_Clause;
Fld_3 : Bare_Name_List;
Cast_Expr_1 : Bare_Name;
Cast_Result_2 : Bare_Ada_Node;
Item_14 : Bare_Name;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Map_Result_2 : Bare_Ada_Node_Array_Access;
Let_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_81 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_81;
            procedure Finalizer_Scope_1872 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1872;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 576 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 575 '<Mapcat at ast.py:1242>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1242
   --# expr-start 570 '<Filter at ast.py:1241>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1241
   --# expr-start 568 '<FieldAccess for prelude at ast.py:1240>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1240
--# expr-start 567 '<Cast to CompilationUnit at ast.py:1239>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1239
--# expr-start 566 '<parent at ast.py:1239>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1239
--# expr-start 565 '<parent at ast.py:1239>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1239
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 565
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 566
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 567
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Prelude;
--# expr-done 568
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_13 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_13 := Untyped_Item_13; 
   --# scope-start
               --# bind p Item_13
         --# expr-start 569 '<IsA UsePackageClause at ast.py:1241>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1241
Is_A := Item_13 /= null 
and then Item_13.Kind in Ada_Use_Package_Clause_Range; 
--# expr-done 569
         if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node := Item_13;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 570
      declare
         Map_Result_2_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Map_Result;
      begin
         for Item_12 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_12
      --# expr-start 574 '<Map at ast.py:1243>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1243
   --# expr-start 572 '<FieldAccess for packages at ast.py:1243>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1243
--# expr-start 571 '<Cast to UsePackageClause at ast.py:1243>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1243
Cast_Expr := Item_12; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 571
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Cast_Result_1.Use_Package_Clause_F_Packages;
--# expr-done 572
      declare
         Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld_3 = null then
      Map_Result_1 := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_3;
      begin
         for Untyped_Item_14 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_14 := Untyped_Item_14; 
   --# scope-start
               --# bind n Item_14
      --# expr-start 573 '<Cast to AdaNode at ast.py:1244>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1244
Cast_Expr_1 := Item_14; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-done 573
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result_2;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 574
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Bare_Ada_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1872;
         end loop;
      end;
         Map_Result_2 := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_2_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 575
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 576
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_81;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_81;
                     Finalizer_Scope_1872;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_Use_Package_Clauses;
--# end
--# property-start AdaNode.top_level_use_type_clauses /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1249
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Type_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_82 with Inline_Always;
            procedure Finalizer_Scope_1875 with Inline_Always;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_16 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Type_Clause;
Fld_3 : Bare_Name_List;
Cast_Expr_1 : Bare_Name;
Cast_Result_2 : Bare_Ada_Node;
Item_17 : Bare_Name;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Map_Result_2 : Bare_Ada_Node_Array_Access;
Let_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_82 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_82;
            procedure Finalizer_Scope_1875 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1875;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 588 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 587 '<Mapcat at ast.py:1260>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1260
   --# expr-start 582 '<Filter at ast.py:1259>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1259
   --# expr-start 580 '<FieldAccess for prelude at ast.py:1258>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1258
--# expr-start 579 '<Cast to CompilationUnit at ast.py:1257>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1257
--# expr-start 578 '<parent at ast.py:1257>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1257
--# expr-start 577 '<parent at ast.py:1257>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1257
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 577
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 578
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 579
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Prelude;
--# expr-done 580
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_16 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_16 := Untyped_Item_16; 
   --# scope-start
               --# bind p Item_16
         --# expr-start 581 '<IsA UseTypeClause at ast.py:1259>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1259
Is_A := Item_16 /= null 
and then Item_16.Kind in Ada_Use_Type_Clause_Range; 
--# expr-done 581
         if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node := Item_16;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 582
      declare
         Map_Result_2_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Map_Result;
      begin
         for Item_15 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_15
      --# expr-start 586 '<Map at ast.py:1261>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1261
   --# expr-start 584 '<FieldAccess for types at ast.py:1261>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1261
--# expr-start 583 '<Cast to UseTypeClause at ast.py:1261>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1261
Cast_Expr := Item_15; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Type_Clause_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 583
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Cast_Result_1.Use_Type_Clause_F_Types;
--# expr-done 584
      declare
         Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld_3 = null then
      Map_Result_1 := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_3;
      begin
         for Untyped_Item_17 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_17 := Untyped_Item_17; 
   --# scope-start
               --# bind n Item_17
      --# expr-start 585 '<Cast to AdaNode at ast.py:1262>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1262
Cast_Expr_1 := Item_17; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-done 585
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result_2;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 586
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Bare_Ada_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1875;
         end loop;
      end;
         Map_Result_2 := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_2_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 587
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 588
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_82;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_82;
                     Finalizer_Scope_1875;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_Use_Type_Clauses;
--# end
--# property-start AdaNode.top_level_use_clauses /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1267
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Use_Clause_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Use_Clause_Array_Access;
            procedure Finalizer_Scope_83 with Inline_Always;
      Block_Var : Bare_Compilation_Unit;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Clause;
Item_18 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Use_Clause_Array_Access;
Let_Result : Bare_Use_Clause_Array_Access;
            procedure Finalizer_Scope_83 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_83;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 596 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 591 '<Cast to CompilationUnit at ast.py:1273>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1273
--# expr-start 590 '<parent at ast.py:1273>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1273
--# expr-start 589 '<parent at ast.py:1273>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1273
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 589
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 590
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 591
Block_Var := Cast_Result; 
--# bind cu Block_Var
--# expr-start 595 '<FilterMap at ast.py:1274>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1274
   --# expr-start 592 '<FieldAccess for prelude at ast.py:1274>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1274
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Block_Var.Compilation_Unit_F_Prelude;
--# expr-done 592
      declare
         Map_Result_Vec : Bare_Use_Clause_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Bare_Use_Clause_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_18 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_18 := Untyped_Item_18; 
   --# scope-start
               --# bind p Item_18
         --# expr-start 593 '<IsA UseClause at ast.py:1276>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1276
Is_A := Item_18 /= null 
and then Item_18.Kind in Ada_Use_Clause; 
--# expr-done 593
         if Is_A then
      --# expr-start 594 '<Cast to UseClause at ast.py:1275>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1275
Cast_Expr := Item_18; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Clause
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 594
         declare
            Item_To_Append : constant Bare_Use_Clause := Cast_Result_1;
         begin
            Bare_Use_Clause_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Use_Clause_Array
           (Items_Count => Natural (Bare_Use_Clause_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Use_Clause_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Use_Clause_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Use_Clause_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 595
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 596
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_83;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_83;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_Use_Clauses;
--# end
--# property-start AdaNode.top_level_with_package_clauses /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1279
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_With_Package_Clauses
  (Node : Bare_Ada_Node
      ; Include_Privates : Boolean
         := True
  )
   return Bare_Name_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind include_privates Include_Privates
   Property_Result : Bare_Name_Array_Access;
            procedure Finalizer_Scope_84 with Inline_Always;
            procedure Finalizer_Scope_1878 with Inline_Always;
            procedure Finalizer_Scope_1879 with Inline_Always;
      Var_Expr : Bare_With_Clause;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Ada_Node_List;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_With_Clause;
Fld_2 : Bare_Name_List;
Item_386 : Bare_Name;
Map_Result : Bare_Name_Array_Access;
Fld_3 : Bare_Private_Node;
Fld_4 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Bare_Name_Array_Access;
Scope_Result : Bare_Name_Array_Access;
Result_Var : Bare_Name_Array_Access;
Item_19 : Bare_Ada_Node;
Map_Result_1 : Bare_Name_Array_Access;
Let_Result : Bare_Name_Array_Access;
            procedure Finalizer_Scope_84 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_84;
            procedure Finalizer_Scope_1878 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1878;
            procedure Finalizer_Scope_1879 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_1879;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 607 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 606 '<Mapcat at ast.py:1286>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1286
   --# expr-start 598 '<FieldAccess for prelude at ast.py:1286>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1286
--# expr-start 597 '<FieldAccess for enclosing_compilation_unit at ast.py:1286>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1286
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 597
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Compilation_Unit_F_Prelude;
--# expr-done 598
      declare
         Map_Result_1_Vec : Bare_Name_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result_1 := Create_Bare_Name_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_19 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_19 := Untyped_Item_19; 
   --# scope-start
               --# bind clause Item_19
--# expr-start 599 '<Cast to WithClause at ast.py:1287>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1287
Cast_Expr := Item_19; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_With_Clause_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 599
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind with_clause Var_Expr
--# expr-start 605 '<If at ast.py:1288>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1288
--# expr-start 601 '<FieldAccess for as_bool at ast.py:1289>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1289
--# expr-start 600 '<FieldAccess for has_private at ast.py:1289>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1289
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr.With_Clause_F_Has_Private;
--# expr-done 600
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Private.as_bool'
Fld_4 := Libadalang.Implementation.Dispatcher_Private_Node_P_As_Bool (Node => Fld_3);
--# end
--# expr-done 601
if Fld_4 then
   --# expr-start 602 '<Not at ast.py:1289>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1289
Not_Val := not (Include_Privates); 
--# expr-done 602
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := No_Bare_Name_Array_Type;
else
   --# expr-start 604 '<TakeWhile at ???>' Map_Result None
   --# expr-start 603 '<FieldAccess for packages at ast.py:1291>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1291
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr.With_Clause_F_Packages;
--# expr-done 603
      declare
         Map_Result_Vec : Bare_Name_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Bare_Name_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_2;
      begin
         for Untyped_Item_386 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_386 := Untyped_Item_386; 
   --# scope-start
         declare
            Item_To_Append : constant Bare_Name := Item_386;
         begin
            Bare_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Name_Array
           (Items_Count => Natural (Bare_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Name_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 604
   If_Result_1 := Map_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 605
Scope_Result := If_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1879;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
            Bare_Name_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1878;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Bare_Name_Array
           (Items_Count => Natural (Bare_Name_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Name_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Name_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Name_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 606
Let_Result := Map_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 607
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_84;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_84;
                     Finalizer_Scope_1878;
                     Finalizer_Scope_1879;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_With_Package_Clauses;
--# end
--# property-start AdaNode.use_clauses_in_spec_of_subp_body /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1296
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_85 with Inline_Always;
            procedure Finalizer_Scope_1882 with Inline_Always;
            procedure Finalizer_Scope_1883 with Inline_Always;
      Block_Var : Symbol_Type_Array_Access;
Block_Var_1 : Bare_Compilation_Unit;
Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Bare_Compilation_Unit;
Fld : Bare_Compilation_Unit;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Compilation_Unit;
Fld_3 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Result_Var : Bare_Basic_Decl;
Fld_4 : Bare_Use_Clause_Array_Access;
Scope_Result_1 : Bare_Use_Clause_Array_Access;
Result_Var_1 : Bare_Use_Clause_Array_Access;
As_Entity : Internal_Entity_Use_Clause;
Is_Null : Boolean;
If_Result : Internal_Entity_Use_Clause;
Fld_5 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_85 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_85;
            procedure Finalizer_Scope_1882 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1882;
            procedure Finalizer_Scope_1883 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1883;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 617 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 609 '<FieldAccess for syntactic_fully_qualified_name at ast.py:1303>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1303
--# expr-start 608 '<FieldAccess for enclosing_compilation_unit at ast.py:1303>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1303
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 608
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Fld);
--# end
--# expr-done 609
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind fqn Block_Var
--# expr-start 610 '<FieldAccess for designated_compilation_unit at ast.py:1305>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1305
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Block_Var, Kind => Unit_Specification, Not_Found_Is_Error => False);
--# end
--# expr-done 610
Block_Var_1 := Fld_2; 
--# bind spec Block_Var_1
--# expr-start 616 '<env_group at ast.py:1312>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1312
--# expr-start 615 '<Map at ast.py:1310>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1310
Var_Expr_1 := Block_Var_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 611 '<FieldAccess for decl at ast.py:1310>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1310
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_3 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr_1);
--# end
--# expr-done 611
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 612 '<FieldAccess for top_level_use_clauses at ast.py:1310>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1310
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_clauses
Fld_4 := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Clauses (Node => Var_Expr);
--# end
--# expr-done 612
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1882;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Use_Clause_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Use_Clause_Array_Access := Result_Var_1;
      begin
         for Item_20 of
               Collection.Items
         loop
   --# scope-start
               --# bind clause Item_20
      --# expr-start 614 '<FieldAccess for used_envs at ast.py:1311>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1311
--# expr-start 613 '<as_bare_entity at ast.py:1311>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1311
Is_Null := Item_20 = null; 
if Is_Null then
   If_Result := No_Entity_Use_Clause;
else
As_Entity := (Info => No_Entity_Info, Node => Item_20); 
   If_Result := As_Entity;
end if;
--# expr-done 613
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.used_envs
Fld_5 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 614
         declare
            Item_To_Append : constant Lexical_Env := Fld_5;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1883;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 615
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 616
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 617
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_85;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_85;
                     Finalizer_Scope_1882;
                     Finalizer_Scope_1883;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body;
--# end
--# property-start AdaNode.use_clauses_in_generic_formal_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1314
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_86 with Inline_Always;
            procedure Finalizer_Scope_1885 with Inline_Always;
      Block_Var : Internal_Entity_Generic_Decl;
Var_Expr : Internal_Entity_Body_Node;
Var_Expr_1 : Internal_Entity_Generic_Decl;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Generic_Formal_Part;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_86 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_86;
            procedure Finalizer_Scope_1885 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1885;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 624 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 619 '<Cast to Body at ast.py:1323>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1323
--# expr-start 618 '<as_bare_entity at ast.py:1323>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1323
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 618
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 619
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 620 '<FieldAccess for safe_generic_decl_part at ast.py:1323>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1323
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.safe_generic_decl_part
Fld := Libadalang.Implementation.Body_Node_P_Safe_Generic_Decl_Part (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 620
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
Block_Var := Result_Var; 
--# bind gen_decl Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_1
--# expr-start 622 '<FieldAccess for use_clauses_envs at ast.py:1325>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1325
--# expr-start 621 '<FieldAccess for formal_part at ast.py:1325>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1325
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_1.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_1.Info);
--# expr-done 621
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericFormalPart.use_clauses_envs
Fld_2 := Libadalang.Implementation.Generic_Formal_Part_P_Use_Clauses_Envs (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 622
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1885;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 623 '<FieldAccess for empty_env at ast.py:1326>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1326
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 623
   Result_Var_1 := Fld_3;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 624
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_86;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_86;
                     Finalizer_Scope_1885;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Use_Clauses_In_Generic_Formal_Part;
--# end
--# property-start AdaNode.nested_generic_formal_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1329
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Nested_Generic_Formal_Part
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_87 with Inline_Always;
            procedure Finalizer_Scope_1887 with Inline_Always;
      Block_Var : Internal_Entity_Generic_Decl;
Var_Expr : Internal_Entity_Body_Node;
Var_Expr_1 : Internal_Entity_Generic_Decl;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_87 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_87;
            procedure Finalizer_Scope_1887 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1887;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 631 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 626 '<Cast to Body at ast.py:1344>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1344
--# expr-start 625 '<as_bare_entity at ast.py:1344>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1344
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 625
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 626
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 627 '<FieldAccess for safe_generic_decl_part at ast.py:1344>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1344
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.safe_generic_decl_part
Fld := Libadalang.Implementation.Body_Node_P_Safe_Generic_Decl_Part (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 627
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
Block_Var := Result_Var; 
--# bind gen_decl Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_1
--# expr-start 629 '<FieldAccess for children_env at ast.py:1346>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1346
--# expr-start 628 '<FieldAccess for node at ast.py:1346>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1346
Fld_1 := Var_Expr_1.Node;
--# expr-done 628
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 629
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1887;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 630 '<FieldAccess for empty_env at ast.py:1347>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1347
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 630
   Result_Var_1 := Fld_3;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 631
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_87;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_87;
                     Finalizer_Scope_1887;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Nested_Generic_Formal_Part;
--# end
--# property-start AdaNode.is_package /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1350
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Package
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 633 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 632 '<IsA PackageDecl, PackageBody, GenericPackageInstantiation, PackageRenamingDecl, GenericPackageDecl at ast.py:1355>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1355
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Decl_Range | Ada_Package_Body_Range | Ada_Generic_Package_Instantiation_Range | Ada_Package_Renaming_Decl_Range | Ada_Generic_Package_Decl_Range; 
--# expr-done 632
Let_Result := Is_A; 
--# end
--# expr-done 633
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Package;
--# end
--# property-start AdaNode.default_initial_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1358
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Default_Initial_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_89 with Inline_Always;
            procedure Finalizer_Scope_1888 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_89 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_89;
            procedure Finalizer_Scope_1888 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1888;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 637 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 634 '<parent at ast.py:1364>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1364
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 634
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 635 '<FieldAccess for children_env at ast.py:1364>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1364
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 635
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1888;
   Result_Var := Scope_Result;
else
   --# expr-start 636 '<FieldAccess for children_env at ast.py:1365>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1365
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 636
   Result_Var := Fld_2;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 637
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_89;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_89;
                     Finalizer_Scope_1888;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Default_Initial_Env;
--# end
--# property-start AdaNode.top_level_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1367
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Decl
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
  )
   return Bare_Basic_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind unit Unit
   Property_Result : Bare_Basic_Decl;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Internal_Unit;
Fld : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Result_Var_1 : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 641 '<Block at ???>' Let_Result None
--# scope-start
Var_Expr_1 := Unit;
if Var_Expr_1 /= null then
   --# scope-start
--# expr-start 638 '<FieldAccess for root at ast.py:1373>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1373
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Var_Expr_1.Ast_Root;
--# expr-done 638
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 640 '<FieldAccess for decl at ast.py:1373>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1373
--# expr-start 639 '<Cast to CompilationUnit at ast.py:1373>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1373
Cast_Expr := Var_Expr; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Cast_Expr;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 639
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Cast_Result);
--# end
--# expr-done 640
Scope_Result_1 := Fld_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 641
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_Decl;
--# end
--# property-start AdaNode.unpack_formals /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1375
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Unpack_Formals
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind formal_params Formal_Params
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_91 with Inline_Always;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_91 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_91;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 645 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 644 '<FieldAccess for unpack_formals_impl at ast.py:1380>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1380
--# expr-start 643 '<FieldAccess for root at ast.py:1380>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1380
--# expr-start 642 '<FieldAccess for unit at ast.py:1380>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1380
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 642
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 643
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.unpack_formals_impl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals_Impl (Node => Fld_1, Formal_Params => Formal_Params);
--# end
--# expr-done 644
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 645
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_91;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_91;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Unpack_Formals;
--# end
--# property-start AdaNode.unpack_formals_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1382
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Unpack_Formals_Impl
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind formal_params Formal_Params
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_92 with Inline_Always;
            procedure Finalizer_Scope_1891 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_92 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_92;
            procedure Finalizer_Scope_1891 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1891;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 648 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 647 '<Mapcat at ast.py:1384>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1384
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Formal_Params;
      begin
         for Item_21 of
               Collection.Items
         loop
   --# scope-start
               --# bind spec Item_21
      --# expr-start 646 '<FieldAccess for defining_names at ast.py:1384>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1384
      if Item_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Item_21.Node, E_Info => Item_21.Info);
--# end
--# expr-done 646
         for Item_To_Append of
               Fld.Items
         loop
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1891;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 647
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 648
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_92;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_92;
                     Finalizer_Scope_1891;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Unpack_Formals_Impl;
--# end
--# property-start AdaNode.match_formals /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1386
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Match_Formals
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind formal_params Formal_Params
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_93 with Inline_Always;
            procedure Finalizer_Scope_1892 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Var_Expr : Internal_Entity_Assoc_List;
Idx : Integer;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Bare_Base_Id;
Var_Expr_3 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Single_Actual_Array_Access;
Arith_Result : Integer;
If_Result : Integer;
Get_Result : Internal_Entity_Defining_Name;
New_Struct : Internal_Param_Match;
Scope_Result : Internal_Param_Match;
Result_Var : Internal_Param_Match;
Let_Result : Internal_Param_Match;
Fld_2 : Bare_Base_Id;
Fld_3 : Internal_Entity_Name;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Name;
Fld_4 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
New_Struct_1 : Internal_Param_Match;
Scope_Result_1 : Internal_Param_Match;
Result_Var_1 : Internal_Param_Match;
Scope_Result_2 : Internal_Param_Match;
Result_Var_2 : Internal_Param_Match;
Fld_5 : Bare_Base_Id;
Is_Null : Boolean;
If_Result_1 : Internal_Param_Match;
Index_23 : Integer;
Map_Result : Internal_Param_Match_Array_Access;
Scope_Result_3 : Internal_Param_Match_Array_Access;
Result_Var_3 : Internal_Param_Match_Array_Access;
Let_Result_1 : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_93 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_93;
            procedure Finalizer_Scope_1892 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1892;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 665 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 649 '<FieldAccess for unpack_formals at ast.py:1395>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1395
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.unpack_formals
Fld := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Formal_Params);
--# end
--# expr-done 649
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind unpacked_formals Block_Var
Var_Expr := Params;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 664 '<Map at ast.py:1397>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1397
   --# expr-start 650 '<FieldAccess for unpacked_params at ast.py:1397>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1397
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 650
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
         Index_23 := 0;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_1;
      begin
         for Item_22 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_22
               --# bind i Index_23
      --# expr-start 663 '<If at ast.py:1397>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1397
--# expr-start 652 '<IsNull at ast.py:1398>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1398
--# expr-start 651 '<FieldAccess for name at ast.py:1398>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1398
Fld_5 := Item_22.Name;
--# expr-done 651
Is_Null := Fld_5 = null; 
--# expr-done 652
if Is_Null then
   --# expr-start 657 '<Let at ast.py:1400>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1400
--# scope-start
--# expr-start 654 '<If at ast.py:1401>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1401
if Is_Dottable_Subp then
   --# expr-start 653 '<Arithmetic + at ast.py:1401>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1401
Arith_Result := (Index_23 + 1); 
--# expr-done 653
   If_Result := Arith_Result;
else
   If_Result := Index_23;
end if;
--# expr-done 654
Idx := If_Result; 
--# bind idx Idx
--# expr-start 655 '<at at ast.py:1404>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1404
Get_Result := Get (Self, Block_Var, Idx, True); 
--# expr-done 655
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind sp Var_Expr_1
--# expr-start 656 '<New ParamMatch at ast.py:1405>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1405
New_Struct := (Actual => Item_22, Formal => Var_Expr_1, Has_Matched => True); 
--# expr-done 656
Scope_Result := New_Struct;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Param_Match;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 657
   If_Result_1 := Let_Result;
else
--# expr-start 658 '<FieldAccess for name at ast.py:1415>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1415
Fld_2 := Item_22.Name;
--# expr-done 658
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind id Var_Expr_2
--# expr-start 661 '<Find at ast.py:1416>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1416
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Block_Var;
      begin
         for Item_24 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_24
      --# expr-start 660 '<FieldAccess for matches at ast.py:1416>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1416
--# expr-start 659 '<FieldAccess for name at ast.py:1416>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1416
      if Item_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Item_24.Node.Defining_Name_F_Name, Info => Item_24.Info);
--# expr-done 659
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Var_Expr_2; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.matches
Fld_4 := Libadalang.Implementation.Name_P_Matches (Node => Fld_3.Node, N => Cast_Result);
--# end
--# expr-done 660
      if Fld_4 then
         Find_Result := Item_24; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 661
Var_Expr_3 := Find_Result;
if Var_Expr_3 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind sp Var_Expr_3
--# expr-start 662 '<New ParamMatch at ast.py:1417>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1417
New_Struct_1 := (Actual => Item_22, Formal => Var_Expr_3, Has_Matched => True); 
--# expr-done 662
Scope_Result_1 := New_Struct_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Param_Match;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Param_Match;
end if;
   If_Result_1 := Result_Var_2;
end if;
--# expr-done 663
         declare
            Item_To_Append : constant Internal_Param_Match := If_Result_1;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
               Index_23 := Index_23 + 1;
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 664
Scope_Result_3 := Map_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1892;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_1 := Result_Var_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 665
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_93;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_93;
                     Finalizer_Scope_1892;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Match_Formals;
--# end
--# property-start AdaNode.choice_match /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1425
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Choice_Match
  (Node : Bare_Ada_Node
      ; Value : Big_Integer_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind value Value
   Property_Result : Boolean;
            procedure Finalizer_Scope_1901 with Inline_Always;
            procedure Finalizer_Scope_1902 with Inline_Always;
            procedure Finalizer_Scope_1904 with Inline_Always;
            procedure Finalizer_Scope_1907 with Inline_Always;
            procedure Finalizer_Scope_1908 with Inline_Always;
      Match_0 : Internal_Entity_Bin_Op;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Subtype_Indication;
Match_3 : Internal_Entity_Expr;
Match_4 : Internal_Entity_Others_Designator;
Match_5 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Discrete_Range;
Edr : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Var_Expr_2 : Internal_Discrete_Range;
Edr_1 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Comp_Result : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Big_Integer_Type;
Comp_Result_1 : Boolean;
If_Result : Boolean;
Fld_4 : Big_Integer_Type;
Is_Equal : Boolean;
Fld_5 : Internal_Entity_Op;
Is_A : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Discrete_Range;
Fld_8 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld_9 : Big_Integer_Type;
Comp_Result_2 : Boolean;
Fld_10 : Big_Integer_Type;
Comp_Result_3 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Bound_Origin : Bare_Ada_Node;
Bound_Imprecise_Fallback : Boolean;
Fld_11 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
If_Result_3 : Boolean;
Scope_Result_1 : Boolean;
Fld_12 : Big_Integer_Type;
Is_Equal_1 : Boolean;
Result_Var_1 : Boolean;
Fld_13 : Internal_Discrete_Range;
Fld_14 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld_15 : Big_Integer_Type;
Comp_Result_4 : Boolean;
Fld_16 : Big_Integer_Type;
Comp_Result_5 : Boolean;
If_Result_4 : Boolean;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Bound_Origin_1 : Bare_Ada_Node;
Bound_Imprecise_Fallback_1 : Boolean;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
Dyn_Var_Bind_Result_3 : Boolean;
If_Result_5 : Boolean;
Fld_19 : Big_Integer_Type;
Is_Equal_2 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Bin_Op;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_1 : Internal_Entity_Name;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_2 : Internal_Entity_Subtype_Indication;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_3 : Internal_Entity_Expr;
Let_Result_5 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_6 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_5 : Internal_Entity;
Let_Result_7 : Boolean;
Scope_Result_8 : Boolean;
Match_Result : Boolean;
Let_Result_8 : Boolean;
            procedure Finalizer_Scope_1901 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1901;
            procedure Finalizer_Scope_1902 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1902;
            procedure Finalizer_Scope_1904 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Edr);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1904;
            procedure Finalizer_Scope_1907 is
            begin
                     Dec_Ref (Fld_14);
                     Dec_Ref (Edr_1);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_1907;
            procedure Finalizer_Scope_1908 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_1908;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 698 '<Block at ???>' Let_Result_8 None
--# scope-start
--# expr-start 697 '<Match at ast.py:1437>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1437
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bo Match_0
--# expr-start 674 '<If at ast.py:1441>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1441
--# expr-start 667 '<IsA Op.DoubleDot at ast.py:1443>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1443
--# expr-start 666 '<FieldAccess for op at ast.py:1443>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1443
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Op (Node => Match_0.Node.Bin_Op_F_Op, Info => Match_0.Info);
--# expr-done 666
Is_A := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Op_Double_Dot_Range; 
--# expr-done 667
if Is_A then
--# expr-start 669 '<FieldAccess for eval_as_int at ast.py:1445>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1445
--# expr-start 668 '<FieldAccess for left at ast.py:1445>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1445
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Match_0.Node.Bin_Op_F_Left, Info => Match_0.Info);
--# expr-done 668
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 669
Comp_Result := Value >= Fld_1; 
if Comp_Result then
--# expr-start 671 '<FieldAccess for eval_as_int at ast.py:1446>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1446
--# expr-start 670 '<FieldAccess for right at ast.py:1446>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1446
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Match_0.Node.Bin_Op_F_Right, Info => Match_0.Info);
--# expr-done 670
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_3 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 671
Comp_Result_1 := Value <= Fld_3; 
   If_Result := Comp_Result_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
else
   --# expr-start 673 '<Eq at ast.py:1448>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1448
--# expr-start 672 '<FieldAccess for eval_as_int at ast.py:1448>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1448
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_4 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 672
Is_Equal := Equivalent (Value, Fld_4); 
--# expr-done 673
   If_Result_1 := Is_Equal;
end if;
--# expr-done 674
Let_Result_2 := If_Result_1; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
      Finalizer_Scope_1901;
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 675 '<FieldAccess for name_designated_type at ast.py:1453>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1453
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 675
Var_Expr := Fld_6;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr
--# expr-start 676 '<FieldAccess for discrete_range at ast.py:1454>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1454
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 676
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Discrete_Range then
   --# scope-start
   --# bind dr Var_Expr_1
--# expr-start 680 '<Let at ast.py:1455>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1455
--# scope-start
--# expr-start 677 '<FieldAccess for eval_discrete_range at ast.py:1456>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1456
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.eval_discrete_range
Fld_8 := Libadalang.Implementation.Ada_Node_P_Eval_Discrete_Range (Node => Self, Dr => Var_Expr_1);
--# end
--# expr-done 677
Edr := Fld_8; Inc_Ref (Edr);
--# bind edr Edr
--# expr-start 678 '<FieldAccess for low_bound at ast.py:1457>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1457
Fld_9 := Edr.Low_Bound;
Inc_Ref (Fld_9);
--# expr-done 678
Comp_Result_2 := Value >= Fld_9; 
if Comp_Result_2 then
--# expr-start 679 '<FieldAccess for high_bound at ast.py:1457>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1457
Fld_10 := Edr.High_Bound;
Inc_Ref (Fld_10);
--# expr-done 679
Comp_Result_3 := Value <= Fld_10; 
   If_Result_2 := Comp_Result_3;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 680
Scope_Result := Let_Result;
   --# end
      Finalizer_Scope_1904;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
if Result_Var then
   --# expr-start 683 '<bind at ast.py:1461>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1461
Bound_Origin := Self; 
--# expr-start 682 '<bind at ast.py:1462>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1462
Bound_Imprecise_Fallback := False; 
--# expr-start 681 '<FieldAccess for satisfies_type_predicates at ast.py:1463>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1463
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Var_Expr.Node, Value => Value, Imprecise_Fallback => Bound_Imprecise_Fallback, Origin => Bound_Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 681
Dyn_Var_Bind_Result := Fld_11; 
--# expr-done 682
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 683
   If_Result_3 := Dyn_Var_Bind_Result_1;
else
   If_Result_3 := False;
end if;
Scope_Result_1 := If_Result_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 685 '<Eq at ast.py:1466>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1466
--# expr-start 684 '<FieldAccess for eval_as_int at ast.py:1466>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1466
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_12 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 684
Is_Equal_1 := Equivalent (Value, Fld_12); 
--# expr-done 685
   Result_Var_1 := Is_Equal_1;
end if;
Let_Result_3 := Result_Var_1; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
      Finalizer_Scope_1902;
            Match_Result := Scope_Result_4; 
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind st Match_2
--# expr-start 686 '<FieldAccess for discrete_range at ast.py:1470>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1470
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_13 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 686
Var_Expr_2 := Fld_13;
if Var_Expr_2 /= No_Discrete_Range then
   --# scope-start
   --# bind dr Var_Expr_2
--# expr-start 690 '<Let at ast.py:1471>' Let_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1471
--# scope-start
--# expr-start 687 '<FieldAccess for eval_discrete_range at ast.py:1472>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1472
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.eval_discrete_range
Fld_14 := Libadalang.Implementation.Ada_Node_P_Eval_Discrete_Range (Node => Self, Dr => Var_Expr_2);
--# end
--# expr-done 687
Edr_1 := Fld_14; Inc_Ref (Edr_1);
--# bind edr Edr_1
--# expr-start 688 '<FieldAccess for low_bound at ast.py:1473>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1473
Fld_15 := Edr_1.Low_Bound;
Inc_Ref (Fld_15);
--# expr-done 688
Comp_Result_4 := Value >= Fld_15; 
if Comp_Result_4 then
--# expr-start 689 '<FieldAccess for high_bound at ast.py:1473>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1473
Fld_16 := Edr_1.High_Bound;
Inc_Ref (Fld_16);
--# expr-done 689
Comp_Result_5 := Value <= Fld_16; 
   If_Result_4 := Comp_Result_5;
else
   If_Result_4 := False;
end if;
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 690
Scope_Result_2 := Let_Result_1;
   --# end
      Finalizer_Scope_1907;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
if Result_Var_2 then
   --# expr-start 694 '<bind at ast.py:1477>' Dyn_Var_Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1477
Bound_Origin_1 := Self; 
--# expr-start 693 '<bind at ast.py:1478>' Dyn_Var_Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1478
Bound_Imprecise_Fallback_1 := False; 
--# expr-start 692 '<FieldAccess for satisfies_type_predicates at ast.py:1479>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1479
--# expr-start 691 '<FieldAccess for designated_type at ast.py:1479>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1479
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_2.Node, Origin => Bound_Origin_1, E_Info => Match_2.Info);
--# end
--# expr-done 691
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_18 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Fld_17.Node, Value => Value, Imprecise_Fallback => Bound_Imprecise_Fallback_1, Origin => Bound_Origin_1, E_Info => Fld_17.Info);
--# end
--# expr-done 692
Dyn_Var_Bind_Result_2 := Fld_18; 
--# expr-done 693
Dyn_Var_Bind_Result_3 := Dyn_Var_Bind_Result_2; 
--# expr-done 694
   If_Result_5 := Dyn_Var_Bind_Result_3;
else
   If_Result_5 := False;
end if;
Let_Result_4 := If_Result_5; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind e Match_3
--# expr-start 696 '<Eq at ast.py:1484>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1484
--# expr-start 695 '<FieldAccess for eval_as_int at ast.py:1484>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1484
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_19 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 695
Is_Equal_2 := Equivalent (Value, Fld_19); 
--# expr-done 696
Let_Result_5 := Is_Equal_2; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
      Finalizer_Scope_1908;
            Match_Result := Scope_Result_6; 
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_6 := True; 
--# end
Scope_Result_7 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_7 := False; 
--# end
Scope_Result_8 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 697
Let_Result_8 := Match_Result; 
--# end
--# expr-done 698
         Property_Result := Let_Result_8;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1901;
                     Finalizer_Scope_1902;
                     Finalizer_Scope_1904;
                     Finalizer_Scope_1907;
                     Finalizer_Scope_1908;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Choice_Match;
--# end
--# property-start AdaNode.gnat_xref /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1492
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Gnat_Xref
  (Node : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1923 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Boolean;
Var_Expr_8 : Internal_Entity_Defining_Name;
Var_Expr_9 : Internal_Entity_Base_Type_Decl;
Var_Expr_10 : Internal_Entity_Defining_Name;
Var_Expr_11 : Internal_Entity_Name;
Dbd : Internal_Entity_Basic_Decl;
Var_Expr_12 : Internal_Entity_Generic_Instantiation;
Var_Expr_13 : Internal_Entity_Defining_Name;
Var_Expr_14 : Internal_Entity_Basic_Decl;
Var_Expr_15 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Bound_Origin : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_2 : Internal_Entity;
Is_A_1 : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Defining_Name;
Is_A_2 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_5 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Defining_Name;
Is_A_3 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_7 : Internal_Entity;
Is_A_4 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Defining_Name;
Is_A_5 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Abstract_Subp_Decl;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Internal_Entity_Defining_Name;
Is_A_6 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Basic_Subp_Decl;
Fld_13 : Internal_Entity_Base_Subp_Spec;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
Fld_16 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity_Defining_Name;
Scope_Result_6 : Internal_Entity_Defining_Name;
Result_Var_6 : Internal_Entity_Defining_Name;
Scope_Result_7 : Internal_Entity_Defining_Name;
Fld_18 : Internal_Entity_Defining_Name;
Result_Var_7 : Internal_Entity_Defining_Name;
Scope_Result_8 : Internal_Entity_Defining_Name;
Result_Var_8 : Internal_Entity_Defining_Name;
Scope_Result_9 : Internal_Entity_Defining_Name;
Result_Var_9 : Internal_Entity_Defining_Name;
Cast_Expr_3 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Name;
Fld_19 : Internal_Entity_Defining_Name;
Scope_Result_10 : Internal_Entity_Defining_Name;
Result_Var_10 : Internal_Entity_Defining_Name;
Fld_20 : Internal_Entity_Basic_Decl;
Is_A_7 : Boolean;
Cast_Expr_4 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Param_Spec;
Fld_21 : Internal_Entity_Defining_Name;
Is_A_8 : Boolean;
Fld_22 : Internal_Entity_Generic_Instantiation_Array_Access;
Get_Result : Internal_Entity_Generic_Instantiation;
Cast_Expr_5 : Internal_Entity_Generic_Instantiation;
Cast_Result_8 : Internal_Entity_Basic_Decl;
Fld_23 : Internal_Entity_Defining_Name;
Scope_Result_11 : Internal_Entity_Defining_Name;
Result_Var_11 : Internal_Entity_Defining_Name;
Is_A_9 : Boolean;
Cast_Expr_6 : Internal_Entity_Basic_Decl;
Cast_Result_9 : Internal_Entity_Object_Decl;
Fld_24 : Internal_Entity_Basic_Decl;
Fld_25 : Internal_Entity_Defining_Name;
Scope_Result_12 : Internal_Entity_Defining_Name;
Result_Var_12 : Internal_Entity_Defining_Name;
Scope_Result_13 : Internal_Entity_Defining_Name;
Result_Var_13 : Internal_Entity_Defining_Name;
Is_A_10 : Boolean;
Cast_Expr_7 : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity_Base_Subp_Body;
Fld_26 : Internal_Entity_Basic_Decl;
Scope_Result_14 : Internal_Entity_Basic_Decl;
Result_Var_14 : Internal_Entity_Basic_Decl;
Fld_27 : Internal_Entity_Defining_Name;
If_Result_2 : Internal_Entity_Defining_Name;
If_Result_3 : Internal_Entity_Defining_Name;
If_Result_4 : Internal_Entity_Defining_Name;
If_Result_5 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
Scope_Result_15 : Internal_Entity_Defining_Name;
Result_Var_15 : Internal_Entity_Defining_Name;
If_Result_6 : Internal_Entity_Defining_Name;
If_Result_7 : Internal_Entity_Defining_Name;
If_Result_8 : Internal_Entity_Defining_Name;
If_Result_9 : Internal_Entity_Defining_Name;
If_Result_10 : Internal_Entity_Defining_Name;
Dyn_Var_Bind_Result : Internal_Entity_Defining_Name;
Let_Result_1 : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1923 is
            begin
                     Dec_Ref (Fld_22);
            end Finalizer_Scope_1923;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 754 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 700 '<FieldAccess for enclosing_defining_name at ast.py:1499>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1499
--# expr-start 699 '<Cast to Name at ast.py:1499>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1499
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 699
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 700
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 701 '<FieldAccess for basic_decl at ast.py:1500>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1500
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 701
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Block_Var := Result_Var; 
--# bind bd Block_Var
--# expr-start 753 '<bind at ast.py:1502>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1502
Bound_Origin := Self; 
--# expr-start 752 '<Cond at ast.py:1502>' If_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1502
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 702 '<IsA ParamSpec at ast.py:1503>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1503
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 702
Scope_Result_1 := Is_A;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   --# expr-start 704 '<IsA SubpDecl, ExprFunction, GenericSubpInternal, BaseTypeDecl at ast.py:1504>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1504
--# expr-start 703 '<FieldAccess for semantic_parent at ast.py:1504>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1504
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 703
Is_A_1 := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Subp_Decl_Range | Ada_Expr_Function_Range | Ada_Generic_Subp_Internal_Range | Ada_Base_Type_Decl; 
--# expr-done 704
   If_Result := Is_A_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 707 '<FieldAccess for defining_name at ast.py:1507>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1507
--# expr-start 706 '<Cast to BasicDecl at ast.py:1507>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1507
--# expr-start 705 '<FieldAccess for semantic_parent at ast.py:1507>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1507
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_3 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 705
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 706
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 707
   If_Result_10 := Fld_4;
else
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 708 '<IsA DiscriminantSpec at ast.py:1509>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1509
Is_A_2 := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Discriminant_Spec_Range; 
--# expr-done 708
Scope_Result_2 := Is_A_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 711 '<FieldAccess for defining_name at ast.py:1510>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1510
--# expr-start 710 '<Cast to BasicDecl at ast.py:1510>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1510
--# expr-start 709 '<FieldAccess for semantic_parent at ast.py:1510>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1510
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_5 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 709
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 710
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 711
   If_Result_9 := Fld_6;
else
Var_Expr_3 := Block_Var;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 712 '<IsA ParamSpec at ast.py:1512>' Is_A_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1512
Is_A_3 := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 712
Scope_Result_3 := Is_A_3;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
   --# expr-start 714 '<IsA AbstractSubpDecl, FormalSubpDecl, NullSubpDecl at ast.py:1513>' Is_A_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1513
--# expr-start 713 '<FieldAccess for semantic_parent at ast.py:1513>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1513
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_7 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 713
Is_A_4 := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Abstract_Subp_Decl_Range | Ada_Formal_Subp_Decl | Ada_Null_Subp_Decl_Range; 
--# expr-done 714
   If_Result_1 := Is_A_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 717 '<FieldAccess for defining_name at ast.py:1515>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1515
--# expr-start 716 '<Cast to BasicDecl at ast.py:1515>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1515
--# expr-start 715 '<FieldAccess for semantic_parent at ast.py:1515>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1515
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_8 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 715
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 716
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 717
   If_Result_8 := Fld_9;
else
Var_Expr_4 := Block_Var;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_4
--# expr-start 718 '<IsA AbstractSubpDecl at ast.py:1517>' Is_A_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1517
Is_A_5 := Var_Expr_4.Node /= null 
and then Var_Expr_4.Node.Kind in Ada_Abstract_Subp_Decl_Range; 
--# expr-done 718
Scope_Result_4 := Is_A_5;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
   --# expr-start 722 '<FieldAccess for defining_name at ast.py:1519>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1519
--# expr-start 721 '<FieldAccess for primitive_subp_first_type at ast.py:1519>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1519
--# expr-start 720 '<FieldAccess for subp_decl_spec at ast.py:1518>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1518
--# expr-start 719 '<Cast to AbstractSubpDecl at ast.py:1518>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1518
Cast_Expr_1 := Block_Var; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Abstract_Subp_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Abstract_Subp_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_4 := No_Entity_Abstract_Subp_Decl;
   end if;
--# expr-done 719
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 720
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_subp_first_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Subp_First_Type (Node => Fld_10.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_10.Info);
--# end
--# expr-done 721
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 722
   If_Result_7 := Fld_12;
else
Var_Expr_5 := Block_Var;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_5
--# expr-start 723 '<IsA BasicSubpDecl at ast.py:1521>' Is_A_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1521
Is_A_6 := Var_Expr_5.Node /= null 
and then Var_Expr_5.Node.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 723
Scope_Result_5 := Is_A_6;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
--# expr-start 726 '<FieldAccess for primitive_subp_first_type at ast.py:1523>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1523
--# expr-start 725 '<FieldAccess for subp_decl_spec at ast.py:1522>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1522
--# expr-start 724 '<Cast to BasicSubpDecl at ast.py:1522>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1522
Cast_Expr_2 := Block_Var; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Subp_Decl;
   end if;
--# expr-done 724
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_5.Node, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 725
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_subp_first_type
Fld_14 := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Subp_First_Type (Node => Fld_13.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_13.Info);
--# end
--# expr-done 726
Var_Expr_6 := Fld_14;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind prim_typ Var_Expr_6
--# expr-start 727 '<FieldAccess for is_tagged_type at ast.py:1525>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1525
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Var_Expr_6.Node, Origin => Bound_Origin, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 727
Var_Expr_7 := Fld_15;
if Var_Expr_7 /= False then
   --# scope-start
   --# bind _ Var_Expr_7
--# expr-start 728 '<FieldAccess for private_completion at ast.py:1526>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1526
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_16 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 728
Var_Expr_9 := Fld_16;
if Var_Expr_9 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pc Var_Expr_9
--# expr-start 729 '<FieldAccess for defining_name at ast.py:1527>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1527
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_9.Node, E_Info => Var_Expr_9.Info);
--# end
--# expr-done 729
Scope_Result_6 := Fld_17;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Defining_Name;
end if;
Var_Expr_8 := Result_Var_6;
if Var_Expr_8 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_8
Scope_Result_7 := Var_Expr_8;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   --# expr-start 730 '<FieldAccess for defining_name at ast.py:1528>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1528
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 730
   Result_Var_7 := Fld_18;
end if;
Scope_Result_8 := Result_Var_7;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := No_Entity_Defining_Name;
end if;
Scope_Result_9 := Result_Var_8;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := No_Entity_Defining_Name;
end if;
   If_Result_6 := Result_Var_9;
else
--# expr-start 731 '<Cast to Name at ast.py:1532>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1532
Cast_Expr_3 := Ent; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_6 := No_Entity_Name;
   end if;
--# expr-done 731
Var_Expr_11 := Cast_Result_6;
if Var_Expr_11 /= No_Entity_Name then
   --# scope-start
--# expr-start 732 '<FieldAccess for gnat_xref_decl at ast.py:1532>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1532
      if Var_Expr_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.gnat_xref_decl
Fld_19 := Libadalang.Implementation.Name_P_Gnat_Xref_Decl (Node => Var_Expr_11.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 732
Scope_Result_10 := Fld_19;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := No_Entity_Defining_Name;
end if;
Var_Expr_10 := Result_Var_10;
if Var_Expr_10 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind ret Var_Expr_10
--# expr-start 751 '<Let at ast.py:1534>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1534
--# scope-start
--# expr-start 733 '<FieldAccess for basic_decl at ast.py:1534>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1534
      if Var_Expr_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_20 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_10.Node, E_Info => Var_Expr_10.Info);
--# end
--# expr-done 733
Dbd := Fld_20; 
--# bind dbd Dbd
--# expr-start 750 '<Cond at ast.py:1534>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1534
--# expr-start 734 '<IsA ParamSpec at ast.py:1535>' Is_A_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1535
Is_A_7 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 734
if Is_A_7 then
   --# expr-start 736 '<FieldAccess for decl_param at ast.py:1536>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1536
--# expr-start 735 '<Cast to ParamSpec at ast.py:1536>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1536
Cast_Expr_4 := Dbd; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_7 := No_Entity_Param_Spec;
   end if;
--# expr-done 735
      if Cast_Result_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.decl_param
Fld_21 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Decl_Param (Node => Cast_Result_7.Node, Param => Var_Expr_10, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_7.Info);
--# end
--# expr-done 736
   If_Result_5 := Fld_21;
else
--# expr-start 737 '<IsA GenericSubpInternal, GenericPackageInternal at ast.py:1538>' Is_A_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1538
Is_A_8 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Generic_Subp_Internal_Range | Ada_Generic_Package_Internal_Range; 
--# expr-done 737
if Is_A_8 then
--# expr-start 739 '<at at ast.py:1539>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1539
--# expr-start 738 '<FieldAccess for generic_instantiations at ast.py:1539>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1539
      if Dbd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.generic_instantiations
Fld_22 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations (Node => Dbd.Node, E_Info => Dbd.Info);
--# end
--# expr-done 738
Get_Result := Get (Self, Fld_22, 0, True); 
--# expr-done 739
Var_Expr_12 := Get_Result;
if Var_Expr_12 /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind gi Var_Expr_12
--# expr-start 741 '<FieldAccess for defining_name at ast.py:1540>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1540
--# expr-start 740 '<Cast to BasicDecl at ast.py:1540>' Cast_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1540
Cast_Expr_5 := Var_Expr_12; 
      Cast_Result_8 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
--# expr-done 740
      if Cast_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_23 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_8.Node, E_Info => Cast_Result_8.Info);
--# end
--# expr-done 741
Scope_Result_11 := Fld_23;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   Result_Var_11 := Var_Expr_10;
end if;
   If_Result_4 := Result_Var_11;
else
--# expr-start 742 '<IsA ObjectDecl at ast.py:1544>' Is_A_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1544
Is_A_9 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 742
if Is_A_9 then
--# expr-start 744 '<FieldAccess for public_part_decl at ast.py:1545>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1545
--# expr-start 743 '<Cast to ObjectDecl at ast.py:1545>' Cast_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1545
Cast_Expr_6 := Dbd; 
   if Cast_Expr_6.Node = null
      or else Cast_Expr_6.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Object_Decl
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
   else
         Cast_Result_9 := No_Entity_Object_Decl;
   end if;
--# expr-done 743
      if Cast_Result_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ObjectDecl.public_part_decl
Fld_24 := Libadalang.Implementation.Object_Decl_P_Public_Part_Decl (Node => Cast_Result_9.Node, E_Info => Cast_Result_9.Info);
--# end
--# expr-done 744
Var_Expr_14 := Fld_24;
if Var_Expr_14 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ppd Var_Expr_14
--# expr-start 745 '<FieldAccess for defining_name at ast.py:1546>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1546
      if Var_Expr_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_25 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_14.Node, E_Info => Var_Expr_14.Info);
--# end
--# expr-done 745
Scope_Result_12 := Fld_25;
   --# end
   Result_Var_12 := Scope_Result_12;
else
   Result_Var_12 := No_Entity_Defining_Name;
end if;
Var_Expr_13 := Result_Var_12;
if Var_Expr_13 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_13
Scope_Result_13 := Var_Expr_13;
   --# end
   Result_Var_13 := Scope_Result_13;
else
   Result_Var_13 := Var_Expr_10;
end if;
   If_Result_3 := Result_Var_13;
else
--# expr-start 746 '<IsA BaseSubpBody at ast.py:1550>' Is_A_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1550
Is_A_10 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Base_Subp_Body; 
--# expr-done 746
if Is_A_10 then
   --# expr-start 749 '<FieldAccess for defining_name at ast.py:1552>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1552
--# expr-start 748 '<FieldAccess for decl_part at ast.py:1552>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1552
--# expr-start 747 '<Cast to BaseSubpBody at ast.py:1551>' Cast_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1551
Cast_Expr_7 := Dbd; 
   if Cast_Expr_7.Node = null
      or else Cast_Expr_7.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_10 := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr_7.Node,
         Info => Cast_Expr_7.Info);
   else
         Cast_Result_10 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 747
      if Cast_Result_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_26 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result_10.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_10.Info);
--# end
--# expr-done 748
Var_Expr_15 := Fld_26;
if Var_Expr_15 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_15
Scope_Result_14 := Var_Expr_15;
   --# end
   Result_Var_14 := Scope_Result_14;
else
   Result_Var_14 := Dbd;
end if;
      if Result_Var_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_27 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Result_Var_14.Node, E_Info => Result_Var_14.Info);
--# end
--# expr-done 749
   If_Result_2 := Fld_27;
else
   If_Result_2 := Var_Expr_10;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
--# expr-done 750
Let_Result := If_Result_5; 
--# end
--# expr-done 751
Scope_Result_15 := Let_Result;
   --# end
      Finalizer_Scope_1923;
   Result_Var_15 := Scope_Result_15;
else
   Result_Var_15 := No_Entity_Defining_Name;
end if;
   If_Result_6 := Result_Var_15;
end if;
   If_Result_7 := If_Result_6;
end if;
   If_Result_8 := If_Result_7;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
end if;
--# expr-done 752
Dyn_Var_Bind_Result := If_Result_10; 
--# expr-done 753
Let_Result_1 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 754
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1923;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Gnat_Xref;
--# end
--# property-start AdaNode.env_get_real_from_node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1559
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get_Real_From_Node
  (Node : Bare_Ada_Node
      ; From_Node : Bare_Ada_Node
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind from_node From_Node
   Property_Result : Bare_Ada_Node;
            procedure Finalizer_Scope_96 with Inline_Always;
      C : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Is_Null : Boolean;
Is_A_1 : Boolean;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null_1 : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Bare_Defining_Name;
Is_A_2 : Boolean;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Subp_Spec;
As_Entity_1 : Internal_Entity_Base_Subp_Spec;
Is_Null_2 : Boolean;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Bare_Defining_Name;
Cast_Result_2 : Bare_Ada_Node;
If_Result_2 : Bare_Ada_Node;
Cast_Result_3 : Bare_Ada_Node;
If_Result_3 : Bare_Ada_Node;
If_Result_4 : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
Is_Null_3 : Boolean;
If_Result_5 : Bare_Ada_Node;
Let_Result_1 : Bare_Ada_Node;
            procedure Finalizer_Scope_96 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_96;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 773 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 772 '<If at ast.py:1567>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1567
--# expr-start 755 '<IsNull at ast.py:1567>' Is_Null_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1567
Is_Null_3 := From_Node = null; 
--# expr-done 755
if Is_Null_3 then
   If_Result_5 := From_Node;
else
   --# expr-start 771 '<Let at ast.py:1567>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1567
--# scope-start
--# expr-start 758 '<Find at ast.py:1568>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1568
   --# expr-start 756 '<FieldAccess for parents at ast.py:1568>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1568
      if From_Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (From_Node, True); 
--# expr-done 756
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_25 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_25
      --# expr-start 757 '<IsA GenericInstantiation, BaseSubpSpec, ComponentDecl at ast.py:1569>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1569
Is_A := Item_25 /= null 
and then Item_25.Kind in Ada_Generic_Instantiation | Ada_Base_Subp_Spec | Ada_Component_Decl_Range; 
--# expr-done 757
      if Is_A then
         Find_Result := Item_25; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 758
C := Find_Result; 
--# bind c C
--# expr-start 770 '<Cond at ast.py:1571>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1571
--# expr-start 759 '<IsNull at ast.py:1572>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1572
Is_Null := C = null; 
--# expr-done 759
if Is_Null then
   If_Result_4 := From_Node;
else
--# expr-start 760 '<IsA GenericInstantiation at ast.py:1579>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1579
Is_A_1 := C /= null 
and then C.Kind in Ada_Generic_Instantiation; 
--# expr-done 760
if Is_A_1 then
--# expr-start 764 '<FieldAccess for node at ast.py:1580>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1580
--# expr-start 763 '<FieldAccess for defining_name at ast.py:1580>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1580
--# expr-start 762 '<as_bare_entity at ast.py:1580>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1580
--# expr-start 761 '<Cast to GenericInstantiation at ast.py:1580>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1580
Cast_Expr := C; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 761
Is_Null_1 := Cast_Result = null; 
if Is_Null_1 then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 762
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 763
Fld_1 := Fld.Node;
--# expr-done 764
      Cast_Result_3 := Fld_1;
   If_Result_3 := Cast_Result_3;
else
--# expr-start 765 '<IsA BaseSubpSpec at ast.py:1582>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1582
Is_A_2 := C /= null 
and then C.Kind in Ada_Base_Subp_Spec; 
--# expr-done 765
if Is_A_2 then
--# expr-start 769 '<FieldAccess for node at ast.py:1583>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1583
--# expr-start 768 '<FieldAccess for name at ast.py:1583>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1583
--# expr-start 767 '<as_bare_entity at ast.py:1583>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1583
--# expr-start 766 '<Cast to BaseSubpSpec at ast.py:1583>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1583
Cast_Expr_1 := C; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 766
Is_Null_2 := Cast_Result_1 = null; 
if Is_Null_2 then
   If_Result_1 := No_Entity_Base_Subp_Spec;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 767
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 768
Fld_3 := Fld_2.Node;
--# expr-done 769
      Cast_Result_2 := Fld_3;
   If_Result_2 := Cast_Result_2;
else
   If_Result_2 := C;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 770
Let_Result := If_Result_4; 
--# end
--# expr-done 771
   If_Result_5 := Let_Result;
end if;
--# expr-done 772
Let_Result_1 := If_Result_5; 
--# end
--# expr-done 773
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_96;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_96;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Get_Real_From_Node;
--# end
--# property-start AdaNode.entity_no_md /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1589
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Entity_No_Md
  (Node : Bare_Ada_Node
      ; N : Bare_Ada_Node
      ; Rebindings : Env_Rebindings
      ; From_Rebound : Boolean
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind n N
   --# bind rebindings Rebindings
   --# bind from_rebound From_Rebound
   Property_Result : Internal_Entity;
      New_Struct : Internal_Entity_Info;
Is_Null : Boolean;
If_Result : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 778 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 777 '<New AdaNode.entity at ast.py:1596>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1596
--# expr-start 776 '<If at ast.py:1598>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1598
--# expr-start 774 '<IsNull at ast.py:1598>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1598
Is_Null := N = null; 
--# expr-done 774
if Is_Null then
   If_Result := No_Entity_Info;
else
   --# expr-start 775 '<New EntityInfo at ast.py:1598>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1598
New_Struct := (From_Rebound => From_Rebound, Md => No_Metadata, Rebindings => Rebindings); 
--# expr-done 775
   If_Result := New_Struct;
end if;
--# expr-done 776
New_Struct_1 := (Info => If_Result, Node => N); 
--# expr-done 777
Let_Result := New_Struct_1; 
--# end
--# expr-done 778
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Entity_No_Md;
--# end
--# property-start AdaNode.env_mappings /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1605
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Mappings
  (Node : Bare_Ada_Node
      ; Defining_Names : Bare_Defining_Name_List
      ; Value : Bare_Ada_Node
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind defining_names Defining_Names
   --# bind value Value
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_98 with Inline_Always;
            procedure Finalizer_Scope_1932 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Item_26 : Bare_Defining_Name;
Map_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_98 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_98;
            procedure Finalizer_Scope_1932 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1932;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 783 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 782 '<Map at ast.py:1611>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1611
      declare
         Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
      begin
      if Defining_Names = null then
      Map_Result := Create_Internal_Env_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Defining_Names;
      begin
         for Untyped_Item_26 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_26 := Untyped_Item_26; 
   --# scope-start
               --# bind n Item_26
      --# expr-start 781 '<New EnvAssoc at ast.py:1612>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1612
--# expr-start 779 '<New DesignatedEnv at ast.py:1612>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1612
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 779
--# expr-start 780 '<FieldAccess for name_symbol at ast.py:1612>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1612
      if Item_26 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_26);
--# end
--# expr-done 780
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Value); 
--# expr-done 781
         declare
            Item_To_Append : constant Internal_Env_Assoc := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1932;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Env_Assoc_Array
           (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 782
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 783
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_98;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_98;
                     Finalizer_Scope_1932;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Mappings;
--# end
--# property-start AdaNode.comp_bind /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1615
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Comp_Bind
  (Node : Bare_Ada_Node
      ; Left : Logic_Var
      ; Right : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind left Left
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_99 with Inline_Always;
      Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_99 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_99;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 785 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 784 '<Bind at ast.py:1617>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1617
Left.Value := No_Entity;
Entity_Vars.Reset (Left);
Right.Value := No_Entity;
Entity_Vars.Reset (Right);
Bind_Result := Solver.Create_Propagate (Left, Right, Logic_Functor_685'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:1617") else null)); 
--# expr-done 784
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 785
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_99;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_99;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Comp_Bind;
--# end
--# property-start AdaNode.universal_int_bind /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1619
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Int_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_100 with Inline_Always;
      Fld : Internal_Entity;
Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_100 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_100;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 788 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 787 '<Bind at ast.py:1626>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1626
Type_Var.Value := No_Entity;
Entity_Vars.Reset (Type_Var);
--# expr-start 786 '<FieldAccess for universal_int_type at ast.py:1626>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1626
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 786
Bind_Result := Solver.Create_Assign (Type_Var, Fld, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:1626") else null)); 
--# expr-done 787
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 788
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_100;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_100;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Int_Bind;
--# end
--# property-start AdaNode.universal_real_bind /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1628
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Real_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_101 with Inline_Always;
      Fld : Internal_Entity;
Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_101 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_101;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 791 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 790 '<Bind at ast.py:1636>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1636
Type_Var.Value := No_Entity;
Entity_Vars.Reset (Type_Var);
--# expr-start 789 '<FieldAccess for universal_real_type at ast.py:1636>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1636
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 789
Bind_Result := Solver.Create_Assign (Type_Var, Fld, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:1636") else null)); 
--# expr-done 790
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 791
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_101;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_101;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Real_Bind;
--# end
--# property-start AdaNode.origin_node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1638
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Origin_Node
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node;
      Fld : Boolean;
If_Result : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 794 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 793 '<If at ast.py:1644>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1644
--# expr-start 792 '<FieldAccess for in_contract at ast.py:1644>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1644
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.in_contract
Fld := Libadalang.Implementation.Ada_Node_P_In_Contract (Node => Self);
--# end
--# expr-done 792
if Fld then
   If_Result := No_Bare_Ada_Node;
else
   If_Result := Self;
end if;
--# expr-done 793
Let_Result := If_Result; 
--# end
--# expr-done 794
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Origin_Node;
--# end
--# property-start AdaNode.env_hook /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1646
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Hook
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Match_0_1 : Bare_Body_Node;
Match_1_1 : Bare_Basic_Decl;
Match_2_1 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Boolean;
Fld_2 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Body_Node;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Fld_3 : Boolean;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_3 : Bare_Library_Item;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Bare_Subunit;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Bare_Ada_Node;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Match_Result_1 : Boolean;
Let_Result_6 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 801 '<Block at ???>' Let_Result_6 None
--# scope-start
--# expr-start 800 '<Match at ast.py:1654>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1654
--# expr-start 795 '<parent at ast.py:1654>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1654
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 795
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix_1;
Match_0 := Cast_Result_3; 
--# bind _ Match_0
--# expr-start 798 '<Match at ast.py:1655>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1655
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0_1 := Cast_Result; 
--# bind b Match_0_1
--# expr-start 796 '<FieldAccess for env_hook_body at ast.py:1656>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1656
      if Match_0_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.env_hook_body
Fld_1 := Libadalang.Implementation.Body_Node_P_Env_Hook_Body (Node => Match_0_1);
--# end
--# expr-done 796
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1_1 := Cast_Result_1; 
--# bind bd Match_1_1
--# expr-start 797 '<FieldAccess for env_hook_basic_decl at ast.py:1657>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1657
      if Match_1_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.env_hook_basic_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Env_Hook_Basic_Decl (Node => Match_1_1);
--# end
--# expr-done 797
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2_1 := Cast_Result_2; 
--# bind _ Match_2_1
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 798
Let_Result_3 := Match_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_1 := Cast_Result_4; 
--# bind su Match_1
--# expr-start 799 '<FieldAccess for env_hook_subunit at ast.py:1660>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1660
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.env_hook_subunit
Fld_3 := Libadalang.Implementation.Subunit_P_Env_Hook_Subunit (Node => Match_1);
--# end
--# expr-done 799
Let_Result_4 := Fld_3; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix_1;
Match_2 := Cast_Result_5; 
--# bind _ Match_2
Let_Result_5 := False; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
end case;
--# expr-done 800
Let_Result_6 := Match_Result_1; 
--# end
--# expr-done 801
         Property_Result := Let_Result_6;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Hook;
--# end
--# property-start AdaNode.env_get /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1664
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Symbol : Symbol_Type
      ; Lookup : Lookup_Kind
         := Recursive
      ; From_Node : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind symbol Symbol
   --# bind lookup Lookup
   --# bind from_node From_Node
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_104 with Inline_Always;
            procedure Finalizer_Scope_1805 with Inline_Always;
      Block_Var : Bare_Ada_Node;
Block_Var_1 : Internal_Entity_Array_Access;
Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Bare_Ada_Node;
Match_0 : Bare_Basic_Decl;
Match_1 : Bare_Defining_Name;
Match_2 : Bare_Ada_Node;
Var_Expr_2 : Internal_Entity_Defining_Name;
Fld : Bare_Ada_Node;
Env_Get_Result : Internal_Entity_Array_Access;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Defining_Name;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Bare_Basic_Decl;
Fld_5 : Boolean;
If_Result_3 : Bare_Basic_Decl;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Defining_Name;
Let_Result_1 : Bare_Basic_Decl;
Scope_Result_2 : Bare_Basic_Decl;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Bare_Basic_Decl;
Scope_Result_3 : Bare_Basic_Decl;
Match_Result : Bare_Basic_Decl;
Scope_Result_4 : Bare_Basic_Decl;
Result_Var_1 : Bare_Basic_Decl;
Fld_6 : Bare_Ada_Node;
Cast_Expr : Bare_Basic_Decl;
Cast_Result_3 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Scope_Result_5 : Internal_Entity_Array_Access;
Result_Var_2 : Internal_Entity_Array_Access;
Let_Result_3 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_104 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_104;
            procedure Finalizer_Scope_1805 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1805;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 818 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 802 '<FieldAccess for env_get_real_from_node at ast.py:1699>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1699
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_get_real_from_node
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get_Real_From_Node (Node => Self, From_Node => From_Node);
--# end
--# expr-done 802
Block_Var := Fld; 
--# bind real_from_node Block_Var
--# expr-start 803 '<EnvGet at ast.py:1700>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1700
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Symbol, Lookup_Kind => To_Lookup_Kind_Type (Lookup), Categories => Categories, From => Block_Var)); 
--# expr-done 803
Block_Var_1 := Env_Get_Result; Inc_Ref (Block_Var_1);
--# bind results Block_Var_1
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 813 '<Match at ast.py:1705>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1705
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind bd Match_0
--# expr-start 807 '<If at ast.py:1706>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1706
--# expr-start 805 '<FieldAccess for defining_name at ast.py:1707>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1707
--# expr-start 804 '<as_bare_entity at ast.py:1707>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1707
Is_Null := Match_0 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Match_0); 
   If_Result := As_Entity;
end if;
--# expr-done 804
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 805
Var_Expr_2 := Fld_1;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 806 '<FieldAccess for name_is at ast.py:1707>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1707
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_2 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_2.Node, Sym => Symbol);
--# end
--# expr-done 806
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result_1 := Match_0;
else
   If_Result_1 := No_Bare_Ada_Node;
end if;
--# expr-done 807
Let_Result := If_Result_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind dn Match_1
--# expr-start 812 '<If at ast.py:1711>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1711
--# expr-start 808 '<FieldAccess for name_is at ast.py:1712>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1712
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Match_1, Sym => Symbol);
--# end
--# expr-done 808
if Fld_5 then
   --# expr-start 811 '<FieldAccess for node at ast.py:1713>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1713
--# expr-start 810 '<FieldAccess for basic_decl at ast.py:1713>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1713
--# expr-start 809 '<as_bare_entity at ast.py:1713>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1713
Is_Null_1 := Match_1 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Defining_Name;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Match_1); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 809
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 810
Fld_4 := Fld_3.Node;
--# expr-done 811
   If_Result_3 := Fld_4;
else
   If_Result_3 := No_Bare_Ada_Node;
end if;
--# expr-done 812
Let_Result_1 := If_Result_3; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Call_Expr | Ada_Discrete_Subtype_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 813
Scope_Result_4 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind enclosing_bd Var_Expr
--# expr-start 817 '<Filter at ast.py:1722>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1722
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_27 of
               Collection.Items
         loop
   --# scope-start
               --# bind r Item_27
         --# expr-start 816 '<Not at ast.py:1722>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1722
--# expr-start 815 '<Eq at ast.py:1722>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1722
--# expr-start 814 '<FieldAccess for node at ast.py:1722>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1722
Fld_6 := Item_27.Node;
--# expr-done 814
Cast_Expr := Var_Expr; 
      Cast_Result_3 := Cast_Expr;
Is_Equal := Fld_6 = Cast_Result_3; 
--# expr-done 815
Not_Val := not (Is_Equal); 
--# expr-done 816
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_27;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 817
Scope_Result_5 := Map_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_1805;
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := Block_Var_1;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_3 := Result_Var_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 818
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_104;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_104;
                     Finalizer_Scope_1805;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Get;
--# end
--# property-start AdaNode.env_get_public /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1726
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get_Public
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Symbol : Symbol_Type
      ; Lookup : Lookup_Kind
         := Recursive
      ; From_Node : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind symbol Symbol
   --# bind lookup Lookup
   --# bind from_node From_Node
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_105 with Inline_Always;
      Var_Expr : Internal_Entity_Package_Decl;
Fld : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Package_Decl;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_105 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_105;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 826 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 825 '<Filter at ast.py:1739>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1739
   --# expr-start 819 '<FieldAccess for env_get at ast.py:1739>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1739
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_get
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Symbol, Lookup => Lookup, From_Node => From_Node, Categories => Categories);
--# end
--# expr-done 819
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_28 of
               Collection.Items
         loop
   --# scope-start
               --# bind x Item_28
--# expr-start 820 '<Cast to PackageDecl at ast.py:1740>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1740
Cast_Expr := Item_28; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Decl;
   end if;
--# expr-done 820
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Package_Decl then
   --# scope-start
   --# bind pkg Var_Expr
--# expr-start 824 '<Not at ast.py:1741>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1741
--# expr-start 823 '<Eq at ast.py:1741>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1741
--# expr-start 821 '<FieldAccess for name_symbol at ast.py:1741>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1741
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr.Node);
--# end
--# expr-done 821
--# expr-start 822 '<Symbol root_types_ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Types); 
--# expr-done 822
Is_Equal := Fld_1 = Sym; 
--# expr-done 823
Not_Val := not (Is_Equal); 
--# expr-done 824
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
         if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity := Item_28;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 825
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 826
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_105;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_105;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Get_Public;
--# end
--# property-start AdaNode.synthesize_defining_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1746
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Synthesize_Defining_Name
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Bare_Defining_Name
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Bare_Defining_Name;
      New_Node : Bare_Synthetic_Identifier;
Cast_Result : Bare_Name;
New_Node_1 : Bare_Synthetic_Defining_Name;
Let_Result : Bare_Synthetic_Defining_Name;
Cast_Result_1 : Bare_Defining_Name;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Synthesize_Defining_Name,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Sym);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Bare_Defining_Name;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 829 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 828 '<New SyntheticDefiningName at ast.py:1753>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1753
--# expr-start 827 '<New SyntheticIdentifier at ast.py:1754>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1754
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Identifier);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Identifier,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
      New_Node.Expr_Logic_Vars :=
         System.Null_Address;
      New_Node.Synthetic_Identifier_Sym :=
         Sym;
--# expr-done 827
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "DefiningName.name cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Defining_Name);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Defining_Name,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Defining_Name
     (Self => New_Node_1,
      Defining_Name_F_Name => Cast_Result);
      New_Node_1.Expr_Logic_Vars :=
         System.Null_Address;
--# expr-done 828
Let_Result := New_Node_1; 
--# end
--# expr-done 829
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Bare_Defining_Name,
                        As_Bare_Defining_Name => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Synthesize_Defining_Name;
--# end
--# property-start AdaNode.create_unop_assoc /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1761
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Unop_Assoc
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Rhs : Bare_Base_Type_Decl
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_107 with Inline_Always;
      Sym : Symbol_Type;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
New_Node_2 : Bare_Synthetic_Type_Expr;
New_Node_3 : Bare_Synthetic_Unary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_4 : Bare_Synthetic_Subp_Decl;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result_2 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_107 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_107;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Unop_Assoc,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Rhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 838 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 837 '<New EnvAssoc at ast.py:1768>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1768
--# expr-start 830 '<New DesignatedEnv at ast.py:1768>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1768
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 830
--# expr-start 836 '<New SyntheticSubpDecl at ast.py:1770>' New_Node_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1770
--# expr-start 835 '<New SyntheticUnarySpec at ast.py:1770>' New_Node_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1770
--# expr-start 831 '<New SyntheticTypeExpr at ast.py:1776>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1776
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_2,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 831
--# expr-start 834 '<New SyntheticFormalParamDecl at ast.py:1772>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1772
--# expr-start 832 '<New SyntheticTypeExpr at ast.py:1774>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1774
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Rhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Rhs);
--# expr-done 832
      Cast_Result := New_Node;
--# expr-start 833 '<Symbol right at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Right); 
--# expr-done 833
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym;
--# expr-done 834
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node_3,
      Synthetic_Unary_Spec_F_Return_Type_Expr => New_Node_2, Synthetic_Unary_Spec_F_Right_Param => New_Node_1);
      New_Node_3.Synthetic_Unary_Spec_Subp_Symbol :=
         Op;
--# expr-done 835
      Cast_Result_1 := New_Node_3;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_4 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_4,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_4);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_4,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 836
      Cast_Result_2 := New_Node_4;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Op, Metadata => No_Metadata, Value => Cast_Result_2); 
--# expr-done 837
Let_Result := New_Struct_1; Inc_Ref (Let_Result);
--# end
--# expr-done 838
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_107;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_107;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Unop_Assoc;
--# end
--# property-start AdaNode.create_binop_assoc_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1780
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc_Impl
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Type_Expr
      ; Rhs : Bare_Type_Expr
      ; Ret : Bare_Type_Expr
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_108 with Inline_Always;
      Sym : Symbol_Type;
New_Node : Bare_Synthetic_Formal_Param_Decl;
Sym_1 : Symbol_Type;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
New_Node_2 : Bare_Synthetic_Binary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_3 : Bare_Synthetic_Subp_Decl;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result_1 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_108 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_108;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc_Impl,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 847 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 846 '<New EnvAssoc at ast.py:1788>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1788
--# expr-start 839 '<New DesignatedEnv at ast.py:1788>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1788
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 839
--# expr-start 845 '<New SyntheticSubpDecl at ast.py:1790>' New_Node_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1790
--# expr-start 844 '<New SyntheticBinarySpec at ast.py:1790>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1790
--# expr-start 841 '<New SyntheticFormalParamDecl at ast.py:1792>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1792
--# expr-start 840 '<Symbol left at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Left); 
--# expr-done 840
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Lhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node,
      Synthetic_Formal_Param_Decl_F_Param_Type => Lhs);
      New_Node.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym;
--# expr-done 841
--# expr-start 843 '<New SyntheticFormalParamDecl at ast.py:1796>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1796
--# expr-start 842 '<Symbol right at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Right); 
--# expr-done 842
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Rhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Rhs);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym_1;
--# expr-done 843
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node_2,
      Synthetic_Binary_Spec_F_Left_Param => New_Node, Synthetic_Binary_Spec_F_Return_Type_Expr => Ret, Synthetic_Binary_Spec_F_Right_Param => New_Node_1);
      New_Node_2.Synthetic_Binary_Spec_Subp_Symbol :=
         Op;
--# expr-done 844
      Cast_Result := New_Node_2;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_3,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 845
      Cast_Result_1 := New_Node_3;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Op, Metadata => No_Metadata, Value => Cast_Result_1); 
--# expr-done 846
Let_Result := New_Struct_1; Inc_Ref (Let_Result);
--# end
--# expr-done 847
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_108;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_108;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc_Impl;
--# end
--# property-start AdaNode.create_binop_assoc /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1804
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Base_Type_Decl
      ; Rhs : Bare_Base_Type_Decl
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_109 with Inline_Always;
      New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
New_Node_2 : Bare_Synthetic_Type_Expr;
Cast_Result_2 : Bare_Type_Expr;
Fld : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_109 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_109;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 852 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 851 '<FieldAccess for create_binop_assoc_impl at ast.py:1813>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1813
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 848 '<New SyntheticTypeExpr at ast.py:1815>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1815
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Lhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Lhs);
--# expr-done 848
      Cast_Result := New_Node;
--# expr-start 849 '<New SyntheticTypeExpr at ast.py:1816>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1816
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Rhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_1,
      Synthetic_Type_Expr_F_Target_Type => Rhs);
--# expr-done 849
      Cast_Result_1 := New_Node_1;
--# expr-start 850 '<New SyntheticTypeExpr at ast.py:1817>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1817
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_2,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 850
      Cast_Result_2 := New_Node_2;
--# property-call-start AdaNode.create_binop_assoc_impl
Fld := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_Impl (Node => Self, Op => Op, Lhs => Cast_Result, Rhs => Cast_Result_1, Ret => Cast_Result_2);
--# end
--# expr-done 851
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 852
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_109;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_109;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc;
--# end
--# property-start AdaNode.create_binop_assoc_l_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1820
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc_L_Expr
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Type_Expr
      ; Rhs : Bare_Base_Type_Decl
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_110 with Inline_Always;
      New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
Fld : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_110 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_110;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc_L_Expr,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 856 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 855 '<FieldAccess for create_binop_assoc_impl at ast.py:1828>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1828
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 853 '<New SyntheticTypeExpr at ast.py:1831>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1831
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Rhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Rhs);
--# expr-done 853
      Cast_Result := New_Node;
--# expr-start 854 '<New SyntheticTypeExpr at ast.py:1832>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1832
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_1,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 854
      Cast_Result_1 := New_Node_1;
--# property-call-start AdaNode.create_binop_assoc_impl
Fld := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_Impl (Node => Self, Op => Op, Lhs => Lhs, Rhs => Cast_Result, Ret => Cast_Result_1);
--# end
--# expr-done 855
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 856
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_110;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_110;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc_L_Expr;
--# end
--# property-start AdaNode.create_binop_assoc_r_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1835
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc_R_Expr
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Base_Type_Decl
      ; Rhs : Bare_Type_Expr
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_111 with Inline_Always;
      New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
Fld : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_111 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_111;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc_R_Expr,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 860 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 859 '<FieldAccess for create_binop_assoc_impl at ast.py:1843>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1843
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 857 '<New SyntheticTypeExpr at ast.py:1845>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1845
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Lhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Lhs);
--# expr-done 857
      Cast_Result := New_Node;
--# expr-start 858 '<New SyntheticTypeExpr at ast.py:1847>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1847
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_1,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 858
      Cast_Result_1 := New_Node_1;
--# property-call-start AdaNode.create_binop_assoc_impl
Fld := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_Impl (Node => Self, Op => Op, Lhs => Cast_Result, Rhs => Rhs, Ret => Cast_Result_1);
--# end
--# expr-done 859
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 860
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_111;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_111;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc_R_Expr;
--# end
--# property-start AdaNode.create_binop_assoc_l_r_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1850
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc_L_R_Expr
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Type_Expr
      ; Rhs : Bare_Type_Expr
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_112 with Inline_Always;
      New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
Fld : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_112 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_112;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc_L_R_Expr,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 863 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 862 '<FieldAccess for create_binop_assoc_impl at ast.py:1858>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1858
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 861 '<New SyntheticTypeExpr at ast.py:1859>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1859
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 861
      Cast_Result := New_Node;
--# property-call-start AdaNode.create_binop_assoc_impl
Fld := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_Impl (Node => Self, Op => Op, Lhs => Lhs, Rhs => Rhs, Ret => Cast_Result);
--# end
--# expr-done 862
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 863
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_112;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_112;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc_L_R_Expr;
--# end
--# property-start AdaNode.custom_id_text /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:207
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Custom_Id_Text
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : String_Type;
            procedure Finalizer_Scope_6 with Inline_Always;
      Str : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_6 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_6;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 865 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 864 '<String at ast.py:214>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:214
Str := Create_String (""); 
--# expr-done 864
Let_Result := Str; Inc_Ref (Let_Result);
--# end
--# expr-done 865
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_6;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_6;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Custom_Id_Text;
--# end
--# property-start AdaNode.complete_items /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:357
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete_Items
  (Node : Bare_Ada_Node
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_20 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Internal_Metadata;
Fld_4 : Boolean;
Fld_5 : Internal_Unit;
Fld_6 : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_7 : Integer;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
Let_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_20 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_20;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 878 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 877 '<Map at ast.py:368>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:368
   --# expr-start 867 '<FieldAccess for env_get_public at ast.py:365>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:365
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 866 '<FieldAccess for children_env at ast.py:366>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:366
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 866
--# property-call-start AdaNode.env_get_public
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Get_Public (Node => Self, Env => Fld, Symbol => null);
--# end
--# expr-done 867
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_7 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_7
      --# expr-start 876 '<New CompletionItem at ast.py:369>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:369
--# expr-start 868 '<Cast to BasicDecl at ast.py:370>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:370
Cast_Expr := Item_7; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 868
--# expr-start 871 '<FieldAccess for dottable_subp at ast.py:371>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:371
--# expr-start 870 '<FieldAccess for md at ast.py:371>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:371
--# expr-start 869 '<FieldAccess for info at ast.py:371>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:371
Fld_2 := Item_7.Info;
--# expr-done 869
Fld_3 := Fld_2.Md;
--# expr-done 870
Fld_4 := Fld_3.Dottable_Subp;
--# expr-done 871
--# expr-start 873 '<FieldAccess for has_with_visibility at ast.py:372>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:372
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 872 '<FieldAccess for unit at ast.py:372>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:372
      if Item_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Unit (Node => Item_7.Node);
--# expr-done 872
--# property-call-start AdaNode.has_with_visibility
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_5);
--# end
--# expr-done 873
--# expr-start 875 '<FieldAccess for complete_item_weight at ast.py:373>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:373
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 874 '<Cast to BasicDecl at ast.py:373>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:373
Cast_Expr_1 := Item_7; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 874
--# property-call-start '[dispatcher]AdaNode.complete_item_weight'
Fld_7 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Complete_Item_Weight (Node => Ent.Node, Item => Cast_Result_1, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 875
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_4, Is_Visible => Fld_6, Weight => Fld_7); 
--# expr-done 876
         declare
            Item_To_Append : constant Internal_Completion_Item := New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 877
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 878
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_20;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_20;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Complete_Items;
--# end
--# property-start AdaNode.complete_item_weight /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:377
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete_Item_Weight
  (Node : Bare_Ada_Node
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
      Let_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 879 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := 0; 
--# end
--# expr-done 879
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Complete_Item_Weight;
--# end
--# property-start AdaNode.immediate_declarative_region /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:586
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Immediate_Declarative_Region
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_33 with Inline_Always;
      Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_33 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_33;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 880 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := Empty_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 880
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_33;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_33;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Immediate_Declarative_Region;
--# end
 function Ada_Node_P_Xref_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is (raise Property_Error
    with "Property AdaNode.xref_equation not implemented on type "
    & Kind_Name (Node));
--# property-start AdaNode.xref_stop_resolution /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:715
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Stop_Resolution
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Xref_Stop_Resolution;
--# end
--# property-start AdaNode.stop_resolution_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:716
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Stop_Resolution_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_41 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_41 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_41;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:717") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_41;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_41;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Stop_Resolution_Equation;
--# end
--# property-start AdaNode.xref_initial_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:721
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Initial_Env
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_42 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_42 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_42;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 882 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 881 '<FieldAccess for children_env at ast.py:728>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:728
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 881
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 882
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_42;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_42;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Xref_Initial_Env;
--# end
--# property-start AdaNode.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:790
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Entry_Point
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Xref_Entry_Point;
--# end
function Ada_Node_P_Resolve_Generic_Actual
  (E : Entity
  ) return Internal_Entity
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   declare
      Result : constant Internal_Entity :=
         Ada_Node_P_Resolve_Generic_Actual (E.Node, E_Info);
   begin
         return (Node => Result.Node, Info => Result.Info);
   end;
end;
function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (E : Entity
  ) return Lexical_Env
is
begin
   declare
      Result : constant Lexical_Env :=
         Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body (E.Node);
   begin
         return Result;
   end;
end;
function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env
is
begin
   declare
      Result : constant Lexical_Env :=
         Ada_Node_P_Use_Clauses_In_Generic_Formal_Part (E.Node);
   begin
         return Result;
   end;
end;
function Ada_Node_P_Nested_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env
is
begin
   declare
      Result : constant Lexical_Env :=
         Ada_Node_P_Nested_Generic_Formal_Part (E.Node);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Abort_Node
   --
--# property-start '[dispatcher]Abort.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Abort_Node_P_As_Bool
  (Node : Bare_Abort_Node
  )
   return Boolean
is
   Self : Bare_Abort_Node  := Bare_Abort_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Abort_Node (Self.Kind) is
               when Ada_Abort_Absent =>
                  --# property-call-start Abort.Absent.as_bool
                  Property_Result := Abort_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Abort_Present =>
                  --# property-call-start Abort.Present.as_bool
                  Property_Result := Abort_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Abort_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abort_Absent
   --
--# property-start Abort.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5316
pragma Warnings (Off, "is not referenced");
 function Abort_Absent_P_As_Bool
  (Node : Bare_Abort_Absent
  )
   return Boolean
is
   Self : Bare_Abort_Absent  := Bare_Abort_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abort_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abort_Present
   --
--# property-start Abort.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5316
pragma Warnings (Off, "is not referenced");
 function Abort_Present_P_As_Bool
  (Node : Bare_Abort_Present
  )
   return Boolean
is
   Self : Bare_Abort_Present  := Bare_Abort_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abort_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Node
   --
--# property-start '[dispatcher]Abstract.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Abstract_Node_P_As_Bool
  (Node : Bare_Abstract_Node
  )
   return Boolean
is
   Self : Bare_Abstract_Node  := Bare_Abstract_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Abstract_Node (Self.Kind) is
               when Ada_Abstract_Absent =>
                  --# property-call-start Abstract.Absent.as_bool
                  Property_Result := Abstract_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Abstract_Present =>
                  --# property-call-start Abstract.Present.as_bool
                  Property_Result := Abstract_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Abstract_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Absent
   --
--# property-start Abstract.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5260
pragma Warnings (Off, "is not referenced");
 function Abstract_Absent_P_As_Bool
  (Node : Bare_Abstract_Absent
  )
   return Boolean
is
   Self : Bare_Abstract_Absent  := Bare_Abstract_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abstract_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Present
   --
--# property-start Abstract.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5260
pragma Warnings (Off, "is not referenced");
 function Abstract_Present_P_As_Bool
  (Node : Bare_Abstract_Present
  )
   return Boolean
is
   Self : Bare_Abstract_Present  := Bare_Abstract_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abstract_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Ada_List
   --
   --
   --  Primitives for Bare_Ada_Node_List
   --
   --
   --  Primitives for Bare_Abstract_State_Decl_List
   --
   --
   --  Primitives for Bare_Alternatives_List
   --
--# property-start AlternativesList.enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22292
pragma Warnings (Off, "is not referenced");
 function Alternatives_List_P_Enum_Type
  (Node : Bare_Alternatives_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Alternatives_List  := Bare_Alternatives_List (Node);
     Ent : Internal_Entity_Alternatives_List :=
       Internal_Entity_Alternatives_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Case_Stmt;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Case_Stmt;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 889 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 886 '<Cast to CaseStmt at ast.py:22298>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22298
--# expr-start 885 '<parent at ast.py:22298>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22298
--# expr-start 884 '<parent at ast.py:22298>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22298
--# expr-start 883 '<parent at ast.py:22298>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22298
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 883
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 884
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 885
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Case_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Case_Stmt
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Case_Stmt;
   end if;
--# expr-done 886
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Case_Stmt then
   --# scope-start
   --# bind cs Var_Expr
--# expr-start 888 '<FieldAccess for expression_type at ast.py:22299>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22299
--# expr-start 887 '<FieldAccess for expr at ast.py:22299>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22299
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Case_Stmt_F_Expr, Info => Var_Expr.Info);
--# expr-done 887
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_4 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 888
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 889
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Alternatives_List_P_Enum_Type;
--# end
--# property-start AlternativesList.complete_items /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22302
pragma Warnings (Off, "is not referenced");
 function Alternatives_List_P_Complete_Items
  (Node : Bare_Alternatives_List
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Alternatives_List  := Bare_Alternatives_List (Node);
     Ent : Internal_Entity_Alternatives_List :=
       Internal_Entity_Alternatives_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1521 with Inline_Always;
      Match_0 : Internal_Entity_Enum_Literal_Decl;
Match_1 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Internal_Unit;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Integer;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Enum_Literal_Decl;
Let_Result : Integer;
Scope_Result : Integer;
Cast_Result_4 : Internal_Entity;
Let_Result_1 : Integer;
Scope_Result_1 : Integer;
Match_Result : Integer;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
Let_Result_2 : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1521 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1521;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 905 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 904 '<Map at ast.py:22307>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22307
   --# expr-start 891 '<EnvGet at ast.py:22307>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22307
--# expr-start 890 '<FieldAccess for children_env at ast.py:22307>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22307
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 890
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 891
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_29 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_29
      --# expr-start 903 '<New CompletionItem at ast.py:22308>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22308
--# expr-start 892 '<Cast to BasicDecl at ast.py:22309>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22309
Cast_Expr := Item_29; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 892
--# expr-start 895 '<FieldAccess for dottable_subp at ast.py:22310>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22310
--# expr-start 894 '<FieldAccess for md at ast.py:22310>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22310
--# expr-start 893 '<FieldAccess for info at ast.py:22310>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22310
Fld_1 := Item_29.Info;
--# expr-done 893
Fld_2 := Fld_1.Md;
--# expr-done 894
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 895
--# expr-start 897 '<FieldAccess for has_with_visibility at ast.py:22311>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22311
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 896 '<FieldAccess for unit at ast.py:22311>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22311
      if Item_29.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Unit (Node => Item_29.Node);
--# expr-done 896
--# property-call-start AdaNode.has_with_visibility
Fld_5 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_4);
--# end
--# expr-done 897
--# expr-start 902 '<Match at ast.py:22312>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22312
      if Item_29.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_29; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind eld Match_0
--# expr-start 901 '<If at ast.py:22313>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22313
--# expr-start 900 '<Eq at ast.py:22314>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22314
--# expr-start 898 '<FieldAccess for enum_type at ast.py:22314>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22314
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AlternativesList.enum_type
Fld_6 := Libadalang.Implementation.Alternatives_List_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 898
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
--# expr-start 899 '<FieldAccess for enum_type at ast.py:22314>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22314
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 899
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 900
if Is_Equiv then
   If_Result := 100;
else
   If_Result := 0;
end if;
--# expr-done 901
Let_Result := If_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := 0; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 902
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_3, Is_Visible => Fld_5, Weight => Match_Result); 
--# expr-done 903
         declare
            Item_To_Append : constant Internal_Completion_Item := New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 904
Let_Result_2 := Map_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 905
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1521;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1521;
            raise;
      end;
   return Property_Result;
end Alternatives_List_P_Complete_Items;
--# end
   --
   --  Primitives for Bare_Constraint_List
   --
   --
   --  Primitives for Bare_Decl_List
   --
   --
   --  Primitives for Bare_Stmt_List
   --
   --
   --  Primitives for Bare_Aspect_Assoc_List
   --
   --
   --  Primitives for Bare_Base_Assoc_List
   --
   --
   --  Primitives for Bare_Basic_Assoc_List
   --
   --
   --  Primitives for Bare_Assoc_List
   --
--# property-start AssocList.actual_for_param_at /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15883
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Actual_For_Param_At
  (Node : Bare_Assoc_List
      ; Param : Internal_Entity_Defining_Name
      ; Pos : Integer
      ; Default_Expr : Internal_Entity_Expr
         := No_Entity_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Assoc_List  := Bare_Assoc_List (Node);
     Ent : Internal_Entity_Assoc_List :=
       Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind param Param
   --# bind pos Pos
   --# bind default_expr Default_Expr
   Property_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_1121 with Inline_Always;
      Block_Var : Internal_Single_Actual_Array_Access;
Var_Expr : Internal_Single_Actual;
Var_Expr_1 : Bare_Base_Id;
Fld : Internal_Single_Actual_Array_Access;
Fld_1 : Bare_Base_Id;
Fld_2 : Internal_Entity_Name;
Fld_3 : Bare_Name;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Single_Actual;
Fld_5 : Internal_Entity_Basic_Assoc;
Fld_6 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Get_Result : Internal_Single_Actual;
Fld_7 : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Get_Result_1 : Internal_Single_Actual;
Is_Equal : Boolean;
Get_Result_2 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Bare_Ada_Node_Array_Access;
Is_Equal_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_1121 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1121;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 925 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 906 '<FieldAccess for unpacked_params at ast.py:15891>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15891
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 906
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind up Block_Var
--# expr-start 911 '<Find at ast.py:15893>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15893
      Find_Result := No_Single_Actual;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Block_Var;
      begin
         for Item_30 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_30
--# expr-start 907 '<FieldAccess for name at ast.py:15895>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15895
Fld_1 := Item_30.Name;
--# expr-done 907
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 910 '<FieldAccess for matches at ast.py:15895>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15895
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 909 '<FieldAccess for node at ast.py:15895>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15895
--# expr-start 908 '<FieldAccess for name at ast.py:15895>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15895
      if Param.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Param.Node.Defining_Name_F_Name, Info => Param.Info);
--# expr-done 908
Fld_3 := Fld_2.Node;
--# expr-done 909
--# property-call-start Name.matches
Fld_4 := Libadalang.Implementation.Name_P_Matches (Node => Var_Expr_1, N => Fld_3);
--# end
--# expr-done 910
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_30; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 911
Var_Expr := Find_Result;
if Var_Expr /= No_Single_Actual then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 913 '<FieldAccess for expr at ast.py:15897>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15897
--# expr-start 912 '<FieldAccess for assoc at ast.py:15897>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15897
Fld_5 := Var_Expr.Assoc;
--# expr-done 912
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 913
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 924 '<If at ast.py:15899>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15899
--# expr-start 915 '<IsNull at ast.py:15901>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15901
--# expr-start 914 '<at at ast.py:15901>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15901
Get_Result_1 := Get (Self, Block_Var, Pos, True); 
--# expr-done 914
Is_Equal := Get_Result_1 = No_Single_Actual; 
--# expr-done 915
if Is_Equal then
   If_Result := True;
else
   --# expr-start 920 '<Not at ast.py:15902>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15902
--# expr-start 919 '<IsNull at ast.py:15902>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15902
--# expr-start 918 '<FieldAccess for names at ast.py:15902>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15902
--# expr-start 917 '<FieldAccess for assoc at ast.py:15902>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15902
--# expr-start 916 '<at at ast.py:15902>' Get_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15902
Get_Result_2 := Get (Self, Block_Var, Pos, True); 
--# expr-done 916
Fld_9 := Get_Result_2.Assoc;
--# expr-done 917
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Fld_9.Node);
--# end
--# expr-done 918
Is_Equal_1 := Equivalent (Fld_10, No_Bare_Ada_Node_Array_Type); 
--# expr-done 919
Not_Val := not (Is_Equal_1); 
--# expr-done 920
   If_Result := Not_Val;
end if;
if If_Result then
   If_Result_1 := Default_Expr;
else
   --# expr-start 923 '<FieldAccess for expr at ast.py:15908>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15908
--# expr-start 922 '<FieldAccess for assoc at ast.py:15908>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15908
--# expr-start 921 '<at at ast.py:15908>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15908
Get_Result := Get (Self, Block_Var, Pos, True); 
--# expr-done 921
Fld_7 := Get_Result.Assoc;
--# expr-done 922
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 923
   If_Result_1 := Fld_8;
end if;
--# expr-done 924
   Result_Var_1 := If_Result_1;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 925
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1121;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1121;
            raise;
      end;
   return Property_Result;
end Assoc_List_P_Actual_For_Param_At;
--# end
--# property-start AssocList.unpacked_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15912
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Unpacked_Params
  (Node : Bare_Assoc_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Single_Actual_Array_Access
is
   Self : Bare_Assoc_List  := Bare_Assoc_List (Node);
     Ent : Internal_Entity_Assoc_List :=
       Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Single_Actual_Array_Access;
            procedure Finalizer_Scope_1122 with Inline_Always;
            procedure Finalizer_Scope_1893 with Inline_Always;
      Names : Bare_Ada_Node_Array_Access;
Saved : Internal_Entity_Assoc_List;
Fld : Bare_Assoc_List;
Fld_1 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_2 : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Identifier;
Cast_Result : Bare_Base_Id;
New_Struct : Internal_Single_Actual;
Singleton : Internal_Single_Actual_Array_Access;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Id;
New_Struct_1 : Internal_Single_Actual;
Is_A : Boolean;
Map_Result : Internal_Single_Actual_Array_Access;
Len : Integer;
Is_Equal : Boolean;
If_Result_1 : Internal_Single_Actual_Array_Access;
Let_Result : Internal_Single_Actual_Array_Access;
Item_31 : Internal_Entity_Basic_Assoc;
Bare_Item_31 : Bare_Basic_Assoc;
Map_Result_1 : Internal_Single_Actual_Array_Access;
Let_Result_1 : Internal_Single_Actual_Array_Access;
            procedure Finalizer_Scope_1122 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1122;
            procedure Finalizer_Scope_1893 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Names);
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1893;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Assoc_List_Assoc_List_P_Unpacked_Params,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Assoc_List,
                                As_Bare_Assoc_List => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Single_Actual_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 937 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 936 '<Mapcat at ast.py:15919>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15919
Saved := Ent; 
Fld := Saved.Node;
      declare
         Map_Result_1_Vec : Internal_Single_Actual_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result_1 := Create_Internal_Single_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld;
      begin
         for Untyped_Item_31 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_31 := Untyped_Item_31; 
Is_Null := Bare_Item_31 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_1 := Saved.Info;
As_Entity := (Info => Fld_1, Node => Bare_Item_31); 
   If_Result := As_Entity;
end if;
                  Item_31 := If_Result; 
   --# scope-start
               --# bind pa Item_31
      --# expr-start 935 '<Let at ast.py:15919>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15919
--# scope-start
--# expr-start 926 '<FieldAccess for names at ast.py:15919>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15919
      if Item_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Item_31.Node);
--# end
--# expr-done 926
Names := Fld_2; Inc_Ref (Names);
--# bind names Names
--# expr-start 934 '<If at ast.py:15919>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15919
--# expr-start 928 '<Eq at ast.py:15920>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15920
--# expr-start 927 '<length at ast.py:15920>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15920
Len := Length (Names); 
--# expr-done 927
Is_Equal := Len = 0; 
--# expr-done 928
if Is_Equal then
   --# expr-start 929 '<New SingleActual at ast.py:15921>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15921
Cast_Expr := No_Bare_Ada_Node; 
      Cast_Result := Cast_Expr;
New_Struct := (Assoc => Item_31, Name => Cast_Result); 
--# expr-done 929
                Singleton := Create_Internal_Single_Actual_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
   If_Result_1 := Singleton;
else
   --# expr-start 933 '<FilterMap at ast.py:15922>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15922
      declare
         Map_Result_Vec : Internal_Single_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Names;
      begin
         for Item_32 of
               Collection.Items
         loop
   --# scope-start
               --# bind i Item_32
         --# expr-start 930 '<IsA BaseId at ast.py:15924>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15924
Is_A := Item_32 /= null 
and then Item_32.Kind in Ada_Base_Id; 
--# expr-done 930
         if Is_A then
      --# expr-start 932 '<New SingleActual at ast.py:15923>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15923
--# expr-start 931 '<Cast to BaseId at ast.py:15923>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15923
Cast_Expr_1 := Item_32; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 931
New_Struct_1 := (Assoc => Item_31, Name => Cast_Result_1); 
--# expr-done 932
         declare
            Item_To_Append : constant Internal_Single_Actual := New_Struct_1;
         begin
            Internal_Single_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Single_Actual_Array
           (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Single_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Single_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 933
   If_Result_1 := Map_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 934
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 935
         for Item_To_Append of
               Let_Result.Items
         loop
            Internal_Single_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1893;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Internal_Single_Actual_Array
           (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Single_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Single_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 936
Let_Result_1 := Map_Result_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 937
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1122;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1122;
                     Finalizer_Scope_1893;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Single_Actual_Array_Access,
                        As_Internal_Single_Actual_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Assoc_List_P_Unpacked_Params;
--# end
--# property-start AssocList.zip_with_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15928
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Zip_With_Params
  (Node : Bare_Assoc_List
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Assoc_List  := Bare_Assoc_List (Node);
     Ent : Internal_Entity_Assoc_List :=
       Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1123 with Inline_Always;
            procedure Finalizer_Scope_1951 with Inline_Always;
            procedure Finalizer_Scope_1952 with Inline_Always;
            procedure Finalizer_Scope_1953 with Inline_Always;
            procedure Finalizer_Scope_1954 with Inline_Always;
            procedure Finalizer_Scope_1955 with Inline_Always;
            procedure Finalizer_Scope_1956 with Inline_Always;
            procedure Finalizer_Scope_1957 with Inline_Always;
            procedure Finalizer_Scope_1958 with Inline_Always;
            procedure Finalizer_Scope_1959 with Inline_Always;
            procedure Finalizer_Scope_1961 with Inline_Always;
            procedure Finalizer_Scope_1963 with Inline_Always;
            procedure Finalizer_Scope_1965 with Inline_Always;
            procedure Finalizer_Scope_1971 with Inline_Always;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_2 : Internal_Entity_Basic_Assoc;
Block_Var_3 : Internal_Param_Actual_Array_Access;
Block_Var_4 : Internal_Param_Actual_Array_Access;
Block_Var_5 : Internal_Param_Actual_Array_Access;
Block_Var_6 : Internal_Param_Actual_Array_Access;
Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Generic_Instantiation;
Match_2 : Internal_Entity_Composite_Constraint;
Match_3 : Internal_Entity_Base_Aggregate;
Match_4 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_3 : Internal_Entity_Generic_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Record_Def;
Var_Expr_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_8 : Internal_Entity_Generic_Formal_Subp_Decl;
Var_Expr_9 : Internal_Entity_Formal_Subp_Decl;
Decl : Internal_Entity_Generic_Formal_Subp_Decl;
Subp : Internal_Entity_Formal_Subp_Decl;
Var_Expr_10 : Internal_Entity_Basic_Decl;
Var_Expr_11 : Internal_Entity_Basic_Assoc;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Decl;
Fld_7 : Internal_Entity_Generic_Formal_Part;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Expr : Bare_Assoc_List;
Cast_Result_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_11 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Internal_Entity_Base_Record_Def;
Fld_14 : Internal_Entity_Component_List;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_4 : Internal_Entity_Generic_Instantiation;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_5 : Internal_Entity_Composite_Constraint;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_6 : Internal_Entity_Base_Aggregate;
Let_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_7 : Internal_Entity;
Let_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Match_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Saved : Internal_Entity_Assoc_List;
Fld_17 : Bare_Assoc_List;
Fld_18 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_19 : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
Item_33 : Internal_Entity_Basic_Assoc;
Bare_Item_33 : Bare_Basic_Assoc;
Find_Result : Internal_Entity_Basic_Assoc;
Fld_20 : Internal_Param_Match_Array_Access;
Fld_21 : Internal_Entity_Defining_Name;
Fld_22 : Internal_Single_Actual;
Fld_23 : Internal_Entity_Basic_Assoc;
Fld_24 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
Map_Result : Internal_Param_Actual_Array_Access;
Scope_Result_11 : Internal_Param_Actual_Array_Access;
Result_Var_6 : Internal_Param_Actual_Array_Access;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_8 : Internal_Entity_Generic_Formal_Subp_Decl;
Cast_Expr_2 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_9 : Internal_Entity_Generic_Formal_Subp_Decl;
Fld_25 : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity_Formal_Subp_Decl;
Fld_26 : Internal_Entity_Defining_Name;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Internal_Entity;
Cast_Result_11 : Internal_Entity_Generic_Instantiation;
Fld_29 : Internal_Entity_Basic_Decl;
Fld_30 : Internal_Entity_Defining_Name;
Scope_Result_12 : Internal_Entity_Defining_Name;
Result_Var_7 : Internal_Entity_Defining_Name;
Cast_Result_12 : Internal_Entity_Expr;
Fld_31 : Internal_Entity_Expr;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Expr;
New_Struct_1 : Internal_Param_Actual;
Let_Result_5 : Internal_Param_Actual;
Cast_Expr_3 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_13 : Internal_Entity_Generic_Formal_Subp_Decl;
Fld_32 : Internal_Entity_Basic_Decl;
Cast_Result_14 : Internal_Entity_Formal_Subp_Decl;
Fld_33 : Internal_Entity_Expr;
Is_A_2 : Boolean;
Fld_34 : Internal_Entity_Defining_Name;
Fld_35 : Internal_Entity_Defining_Name;
Cast_Result_15 : Internal_Entity;
Cast_Result_16 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Scope_Result_13 : Boolean;
Result_Var_8 : Boolean;
Scope_Result_14 : Boolean;
Result_Var_9 : Boolean;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Scope_Result_15 : Internal_Param_Actual_Array_Access;
Result_Var_10 : Internal_Param_Actual_Array_Access;
Concat_Result : Internal_Param_Actual_Array_Access;
Fld_36 : Internal_Entity_Defining_Name_Array_Access;
Fld_37 : Internal_Entity_Expr;
New_Struct_2 : Internal_Param_Actual;
Fld_38 : Internal_Entity_Defining_Name;
Cast_Result_17 : Internal_Entity;
Cast_Expr_4 : Internal_Entity_Defining_Name;
Cast_Result_18 : Internal_Entity;
Is_Equiv_1 : Boolean;
Quantifier_Result_2 : Boolean;
Not_Val_1 : Boolean;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_16 : Internal_Param_Actual_Array_Access;
Result_Var_11 : Internal_Param_Actual_Array_Access;
Concat_Result_1 : Internal_Param_Actual_Array_Access;
Let_Result_6 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1123 is
            begin
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Scope_Result_15);
                     Dec_Ref (Result_Var_10);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Scope_Result_16);
                     Dec_Ref (Result_Var_11);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_1123;
            procedure Finalizer_Scope_1951 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1951;
            procedure Finalizer_Scope_1952 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_1952;
            procedure Finalizer_Scope_1953 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1953;
            procedure Finalizer_Scope_1954 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_1954;
            procedure Finalizer_Scope_1955 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_1955;
            procedure Finalizer_Scope_1956 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_1956;
            procedure Finalizer_Scope_1957 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1957;
            procedure Finalizer_Scope_1958 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
            end Finalizer_Scope_1958;
            procedure Finalizer_Scope_1959 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_1959;
            procedure Finalizer_Scope_1961 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_1961;
            procedure Finalizer_Scope_1963 is
            begin
                     Dec_Ref (Var_Expr_6);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1963;
            procedure Finalizer_Scope_1965 is
            begin
                     Dec_Ref (Var_Expr_7);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1965;
            procedure Finalizer_Scope_1971 is
            begin
                     Dec_Ref (Fld_36);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1971;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1008 '<Block at ???>' Let_Result_6 None
--# scope-start
--# expr-start 941 '<bind at ast.py:15938>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15938
Bound_Imprecise_Fallback := False; 
--# expr-start 939 '<Cast to Name at ast.py:15939>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15939
--# expr-start 938 '<parent at ast.py:15939>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15939
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 938
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 939
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 940 '<FieldAccess for is_dot_call at ast.py:15939>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15939
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_dot_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Var_Expr.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 940
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 941
Block_Var := Dyn_Var_Bind_Result; 
--# bind is_dottable_subp Block_Var
--# expr-start 942 '<parent at ast.py:15942>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15942
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 942
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity then
   --# scope-start
--# expr-start 960 '<Match at ast.py:15942>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15942
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind e Match_0
--# expr-start 943 '<FieldAccess for called_subp_spec at ast.py:15943>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15943
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_subp_spec'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Called_Subp_Spec (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 943
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 944 '<FieldAccess for abstract_formal_params at ast.py:15943>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15943
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 944
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1953;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
Scope_Result_5 := Let_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_1952;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
         when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Generic_Instantiation
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind i Match_1
--# expr-start 947 '<Cast to GenericDecl at ast.py:15946>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15946
--# expr-start 946 '<FieldAccess for referenced_decl at ast.py:15946>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15946
--# expr-start 945 '<FieldAccess for generic_entity_name at ast.py:15946>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15946
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld_5 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 945
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_6 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 946
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Decl;
   end if;
--# expr-done 947
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 949 '<FieldAccess for abstract_formal_params at ast.py:15947>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15947
--# expr-start 948 '<FieldAccess for formal_part at ast.py:15947>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15947
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_3.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_3.Info);
--# expr-done 948
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 949
Scope_Result_2 := Fld_8;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1955;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_6 := Let_Result_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_1954;
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Composite_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Composite_Constraint
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind c Match_2
--# expr-start 950 '<FieldAccess for subtype at ast.py:15950>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15950
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_9 := Libadalang.Implementation.Constraint_P_Subtype (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 950
Var_Expr_4 := Fld_9;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 951 '<FieldAccess for discriminants_list at ast.py:15950>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15950
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_4.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 951
Scope_Result_3 := Fld_10;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1957;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_2 := Result_Var_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_7 := Let_Result_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_1956;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Aggregate
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind a Match_3
--# expr-start 959 '<bind at ast.py:15952>' Dyn_Var_Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15952
Cast_Expr := Self; 
      Cast_Result_2 := Cast_Expr;
Bound_Origin := Cast_Result_2; 
--# expr-start 958 '<bind at ast.py:15952>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15952
--# expr-start 952 '<FieldAccess for node_env at ast.py:15953>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15953
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 952
Bound_Env := Fld_11; Inc_Ref (Bound_Env);
--# expr-start 954 '<FieldAccess for record_def at ast.py:15954>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15954
--# expr-start 953 '<FieldAccess for expression_type at ast.py:15954>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15954
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_12 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 953
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_13 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld_12.Node, Origin => Bound_Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 954
Var_Expr_5 := Fld_13;
if Var_Expr_5 /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 957 '<FieldAccess for abstract_formal_params_for_assocs at ast.py:15955>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15955
--# expr-start 955 '<FieldAccess for components at ast.py:15955>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15955
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Component_List (Node => Var_Expr_5.Node.Base_Record_Def_F_Components, Info => Var_Expr_5.Info);
--# expr-done 955
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 956 '<FieldAccess for ancestor_expr_type at ast.py:15959>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15959
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.ancestor_expr_type
Fld_15 := Libadalang.Implementation.Base_Aggregate_P_Ancestor_Expr_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 956
--# property-call-start ComponentList.abstract_formal_params_for_assocs
Fld_16 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Assocs (Node => Fld_14.Node, Assocs => Ent, Stop_Recurse_At => Fld_15, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 957
Scope_Result_4 := Fld_16;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1959;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Dyn_Var_Bind_Result_1 := Result_Var_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 958
Dyn_Var_Bind_Result_2 := Dyn_Var_Bind_Result_1; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 959
Let_Result_3 := Dyn_Var_Bind_Result_2; Inc_Ref (Let_Result_3);
--# end
Scope_Result_8 := Let_Result_3;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_1958;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Generic_Subp_Decl | Ada_Generic_Package_Renaming_Decl .. Ada_Constant_Present | Ada_Delta_Constraint .. Ada_Allocator | Ada_Bin_Op .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind _ Match_4
Let_Result_4 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type; Inc_Ref (Let_Result_4);
--# end
Scope_Result_9 := Let_Result_4;
   Inc_Ref (Scope_Result_9);
   --# end
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
end case;
--# expr-done 960
Scope_Result_10 := Match_Result;
   Inc_Ref (Scope_Result_10);
   --# end
      Finalizer_Scope_1951;
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_5);
Block_Var_1 := Result_Var_5; Inc_Ref (Block_Var_1);
--# bind params Block_Var_1
--# expr-start 964 '<Find at ast.py:15966>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15966
Saved := Ent; 
Fld_17 := Saved.Node;
      Find_Result := No_Entity_Basic_Assoc;
      if Fld_17 = null then
      Find_Result := No_Entity_Basic_Assoc;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_17;
      begin
         for Untyped_Item_33 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_33 := Untyped_Item_33; 
Is_Null := Bare_Item_33 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_18 := Saved.Info;
As_Entity := (Info => Fld_18, Node => Bare_Item_33); 
   If_Result := As_Entity;
end if;
                  Item_33 := If_Result; 
   --# scope-start
               --# bind assoc Item_33
      --# expr-start 963 '<AnyQuantifier at ast.py:15967>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15967
   --# expr-start 961 '<FieldAccess for names at ast.py:15967>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15967
      if Item_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_19 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Item_33.Node);
--# end
--# expr-done 961
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_19;
      begin
         for Item_34 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_34
      --# expr-start 962 '<IsA OthersDesignator at ast.py:15968>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15968
Is_A := Item_34 /= null 
and then Item_34.Kind in Ada_Others_Designator_Range; 
--# expr-done 962
      Quantifier_Result := Is_A;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 963
      if Quantifier_Result then
         Find_Result := Item_33; 
      end if;
   --# end
      Finalizer_Scope_1961;
               exit when Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 964
Block_Var_2 := Find_Result; 
--# bind others_assoc Block_Var_2
Var_Expr_6 := Block_Var_1;
      Inc_Ref (Var_Expr_6);
if Var_Expr_6 /= No_Internal_Entity_Base_Formal_Param_Decl_Array_Type then
   --# scope-start
   --# bind _ Var_Expr_6
--# expr-start 971 '<Map at ast.py:15973>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15973
   --# expr-start 965 '<FieldAccess for match_formals at ast.py:15973>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15973
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.match_formals
Fld_20 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Block_Var_1, Params => Ent, Is_Dottable_Subp => Block_Var);
--# end
--# expr-done 965
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_20;
      begin
         for Item_35 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_35
      --# expr-start 970 '<New ParamActual at ast.py:15974>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15974
--# expr-start 968 '<FieldAccess for expr at ast.py:15976>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15976
--# expr-start 967 '<FieldAccess for assoc at ast.py:15976>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15976
--# expr-start 966 '<FieldAccess for actual at ast.py:15976>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15976
Fld_22 := Item_35.Actual;
--# expr-done 966
Fld_23 := Fld_22.Assoc;
--# expr-done 967
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_24 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_23.Node, E_Info => Fld_23.Info);
--# end
--# expr-done 968
--# expr-start 969 '<FieldAccess for formal at ast.py:15975>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15975
Fld_21 := Item_35.Formal;
--# expr-done 969
New_Struct := (Actual => Fld_24, Param => Fld_21); 
--# expr-done 970
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 971
Scope_Result_11 := Map_Result;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_1963;
   Result_Var_6 := Scope_Result_11;
else
   Result_Var_6 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Block_Var_3 := Result_Var_6; Inc_Ref (Block_Var_3);
--# bind explicit_matches Block_Var_3
Var_Expr_7 := Block_Var_1;
      Inc_Ref (Var_Expr_7);
if Var_Expr_7 /= No_Internal_Entity_Base_Formal_Param_Decl_Array_Type then
   --# scope-start
   --# bind _ Var_Expr_7
--# expr-start 997 '<FilterMap at ast.py:15981>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15981
      declare
         Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Block_Var_1;
      begin
         for Item_36 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_36
--# expr-start 972 '<Cast to GenericFormalSubpDecl at ast.py:16000>' Cast_Result_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16000
Cast_Expr_3 := Item_36; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_13 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_13 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 972
Var_Expr_8 := Cast_Result_13;
if Var_Expr_8 /= No_Entity_Generic_Formal_Subp_Decl then
   --# scope-start
   --# bind fd Var_Expr_8
--# expr-start 974 '<Cast to FormalSubpDecl at ast.py:16001>' Cast_Result_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16001
--# expr-start 973 '<FieldAccess for decl at ast.py:16001>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16001
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Create_Internal_Entity_Basic_Decl (Node => Var_Expr_8.Node.Generic_Formal_F_Decl, Info => Var_Expr_8.Info);
--# expr-done 973
   if Fld_32.Node = null
      or else Fld_32.Node.Kind in Ada_Formal_Subp_Decl
   then
      Cast_Result_14 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Fld_32.Node,
         Info => Fld_32.Info);
   else
         Cast_Result_14 := No_Entity_Formal_Subp_Decl;
   end if;
--# expr-done 974
Var_Expr_9 := Cast_Result_14;
if Var_Expr_9 /= No_Entity_Formal_Subp_Decl then
   --# scope-start
   --# bind subp Var_Expr_9
--# expr-start 976 '<IsA BoxExpr, Name at ast.py:16005>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16005
--# expr-start 975 '<FieldAccess for default_expr at ast.py:16005>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16005
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Create_Internal_Entity_Expr (Node => Var_Expr_9.Node.Formal_Subp_Decl_F_Default_Expr, Info => Var_Expr_9.Info);
--# expr-done 975
Is_A_2 := Fld_33.Node /= null 
and then Fld_33.Node.Kind in Ada_Box_Expr_Range | Ada_Name; 
--# expr-done 976
if Is_A_2 then
   --# expr-start 981 '<Not at ast.py:16008>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16008
--# expr-start 980 '<AnyQuantifier at ast.py:16008>' Quantifier_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16008
      Quantifier_Result_1 := False;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Block_Var_3;
      begin
         for Item_37 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_37
      --# expr-start 979 '<Eq at ast.py:16009>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16009
--# expr-start 977 '<FieldAccess for param at ast.py:16009>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16009
Fld_34 := Item_37.Param;
--# expr-done 977
      Cast_Result_15 := Create_Internal_Entity
        (Node => Fld_34.Node,
         Info => Fld_34.Info);
--# expr-start 978 '<FieldAccess for defining_name at ast.py:16009>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16009
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_35 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_9.Node, E_Info => Var_Expr_9.Info);
--# end
--# expr-done 978
      Cast_Result_16 := Create_Internal_Entity
        (Node => Fld_35.Node,
         Info => Fld_35.Info);
Is_Equiv := Equivalent (Cast_Result_15, Cast_Result_16); 
--# expr-done 979
      Quantifier_Result_1 := Is_Equiv;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 980
Not_Val := not (Quantifier_Result_1); 
--# expr-done 981
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
Scope_Result_13 := If_Result_2;
   --# end
   Result_Var_8 := Scope_Result_13;
else
   Result_Var_8 := False;
end if;
Scope_Result_14 := Result_Var_8;
   --# end
   Result_Var_9 := Scope_Result_14;
else
   Result_Var_9 := False;
end if;
         if Result_Var_9 then
      --# expr-start 996 '<Let at ast.py:15984>' Let_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15984
--# scope-start
--# expr-start 982 '<Cast to GenericFormalSubpDecl at ast.py:15986>' Cast_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15986
Cast_Expr_1 := Item_36; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_8 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_8 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 982
Decl := Cast_Result_8; 
--# bind decl Decl
--# expr-start 985 '<Cast to FormalSubpDecl at ast.py:15987>' Cast_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15987
--# expr-start 984 '<FieldAccess for decl at ast.py:15987>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15987
--# expr-start 983 '<Cast to GenericFormalSubpDecl at ast.py:15987>' Cast_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15987
Cast_Expr_2 := Item_36; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_9 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 983
      if Cast_Result_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Basic_Decl (Node => Cast_Result_9.Node.Generic_Formal_F_Decl, Info => Cast_Result_9.Info);
--# expr-done 984
   if Fld_25.Node = null
      or else Fld_25.Node.Kind in Ada_Formal_Subp_Decl
   then
      Cast_Result_10 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Fld_25.Node,
         Info => Fld_25.Info);
   else
         Cast_Result_10 := No_Entity_Formal_Subp_Decl;
   end if;
--# expr-done 985
Subp := Cast_Result_10; 
--# bind subp Subp
--# expr-start 995 '<New ParamActual at ast.py:15989>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15989
--# expr-start 993 '<If at ast.py:15991>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15991
--# expr-start 987 '<IsA Name at ast.py:15992>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15992
--# expr-start 986 '<FieldAccess for default_expr at ast.py:15992>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15992
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Expr (Node => Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Subp.Info);
--# expr-done 986
Is_A_1 := Fld_31.Node /= null 
and then Fld_31.Node.Kind in Ada_Name; 
--# expr-done 987
if Is_A_1 then
   --# expr-start 988 '<FieldAccess for default_expr at ast.py:15993>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15993
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Create_Internal_Entity_Expr (Node => Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Subp.Info);
--# expr-done 988
   If_Result_1 := Fld_27;
else
--# expr-start 991 '<FieldAccess for designated_subprogram_from at ast.py:15994>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15994
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 990 '<Cast to GenericInstantiation at ast.py:15995>' Cast_Result_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15995
--# expr-start 989 '<parent at ast.py:15995>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15995
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 989
   if Fld_28.Node = null
      or else Fld_28.Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result_11 := Create_Internal_Entity_Generic_Instantiation
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
   else
         Cast_Result_11 := No_Entity_Generic_Instantiation;
   end if;
--# expr-done 990
--# property-call-start FormalSubpDecl.designated_subprogram_from
Fld_29 := Libadalang.Implementation.Formal_Subp_Decl_P_Designated_Subprogram_From (Node => Subp.Node, Inst => Cast_Result_11, E_Info => Subp.Info);
--# end
--# expr-done 991
Var_Expr_10 := Fld_29;
if Var_Expr_10 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 992 '<FieldAccess for defining_name at ast.py:15996>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15996
      if Var_Expr_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_30 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_10.Node, E_Info => Var_Expr_10.Info);
--# end
--# expr-done 992
Scope_Result_12 := Fld_30;
   --# end
   Result_Var_7 := Scope_Result_12;
else
   Result_Var_7 := No_Entity_Defining_Name;
end if;
      Cast_Result_12 := Create_Internal_Entity_Expr
        (Node => Result_Var_7.Node,
         Info => Result_Var_7.Info);
   If_Result_1 := Cast_Result_12;
end if;
--# expr-done 993
--# expr-start 994 '<FieldAccess for defining_name at ast.py:15990>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15990
      if Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_26 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Decl.Node, E_Info => Decl.Info);
--# end
--# expr-done 994
New_Struct_1 := (Actual => If_Result_1, Param => Fld_26); 
--# expr-done 995
Let_Result_5 := New_Struct_1; 
--# end
--# expr-done 996
         declare
            Item_To_Append : constant Internal_Param_Actual := Let_Result_5;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 997
Scope_Result_15 := Map_Result_1;
   Inc_Ref (Scope_Result_15);
   --# end
      Finalizer_Scope_1965;
   Result_Var_10 := Scope_Result_15;
else
   Result_Var_10 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_10);
Block_Var_4 := Result_Var_10; Inc_Ref (Block_Var_4);
--# bind default_subp_matches Block_Var_4
--# expr-start 998 '<Concat at ast.py:16016>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16016
Concat_Result := Concat (Block_Var_3, Block_Var_4); 
--# expr-done 998
Block_Var_5 := Concat_Result; Inc_Ref (Block_Var_5);
--# bind given_matches Block_Var_5
Var_Expr_11 := Block_Var_2;
if Var_Expr_11 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind oa Var_Expr_11
--# expr-start 1006 '<FilterMap at ast.py:16019>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16019
   --# expr-start 999 '<FieldAccess for unpack_formals at ast.py:16019>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16019
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.unpack_formals
Fld_36 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Block_Var_1);
--# end
--# expr-done 999
      declare
         Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_36;
      begin
         for Item_38 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_38
         --# expr-start 1003 '<Not at ast.py:16024>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16024
--# expr-start 1002 '<AnyQuantifier at ast.py:16024>' Quantifier_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16024
      Quantifier_Result_2 := False;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Block_Var_5;
      begin
         for Item_39 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_39
      --# expr-start 1001 '<Eq at ast.py:16024>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16024
--# expr-start 1000 '<FieldAccess for param at ast.py:16024>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16024
Fld_38 := Item_39.Param;
--# expr-done 1000
      Cast_Result_17 := Create_Internal_Entity
        (Node => Fld_38.Node,
         Info => Fld_38.Info);
Cast_Expr_4 := Item_38; 
      Cast_Result_18 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
Is_Equiv_1 := Equivalent (Cast_Result_17, Cast_Result_18); 
--# expr-done 1001
      Quantifier_Result_2 := Is_Equiv_1;
   --# end
               exit when Quantifier_Result_2;
         end loop;
      end;
--# expr-done 1002
Not_Val_1 := not (Quantifier_Result_2); 
--# expr-done 1003
         if Not_Val_1 then
      --# expr-start 1005 '<New ParamActual at ast.py:16020>' New_Struct_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16020
--# expr-start 1004 '<FieldAccess for expr at ast.py:16022>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16022
      if Var_Expr_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_37 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_11.Node, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 1004
New_Struct_2 := (Actual => Fld_37, Param => Item_38); 
--# expr-done 1005
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_2;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 1006
Scope_Result_16 := Map_Result_2;
   Inc_Ref (Scope_Result_16);
   --# end
      Finalizer_Scope_1971;
   Result_Var_11 := Scope_Result_16;
else
   Result_Var_11 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_11);
Block_Var_6 := Result_Var_11; Inc_Ref (Block_Var_6);
--# bind others_matches Block_Var_6
--# expr-start 1007 '<Concat at ast.py:16028>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16028
Concat_Result_1 := Concat (Block_Var_5, Block_Var_6); 
--# expr-done 1007
Let_Result_6 := Concat_Result_1; Inc_Ref (Let_Result_6);
--# end
--# expr-done 1008
         Property_Result := Let_Result_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1123;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1123;
                     Finalizer_Scope_1951;
                     Finalizer_Scope_1952;
                     Finalizer_Scope_1953;
                     Finalizer_Scope_1954;
                     Finalizer_Scope_1955;
                     Finalizer_Scope_1956;
                     Finalizer_Scope_1957;
                     Finalizer_Scope_1958;
                     Finalizer_Scope_1959;
                     Finalizer_Scope_1961;
                     Finalizer_Scope_1963;
                     Finalizer_Scope_1965;
                     Finalizer_Scope_1971;
            raise;
      end;
   return Property_Result;
end Assoc_List_P_Zip_With_Params;
--# end
   --
   --  Primitives for Bare_Basic_Decl_List
   --
   --
   --  Primitives for Bare_Case_Expr_Alternative_List
   --
   --
   --  Primitives for Bare_Case_Stmt_Alternative_List
   --
   --
   --  Primitives for Bare_Compilation_Unit_List
   --
   --
   --  Primitives for Bare_Concat_Operand_List
   --
   --
   --  Primitives for Bare_Contract_Case_Assoc_List
   --
   --
   --  Primitives for Bare_Defining_Name_List
   --
   --
   --  Primitives for Bare_Discriminant_Spec_List
   --
   --
   --  Primitives for Bare_Elsif_Expr_Part_List
   --
   --
   --  Primitives for Bare_Elsif_Stmt_Part_List
   --
   --
   --  Primitives for Bare_Enum_Literal_Decl_List
   --
   --
   --  Primitives for Bare_Expr_List
   --
   --
   --  Primitives for Bare_Expr_Alternatives_List
   --
   --
   --  Primitives for Bare_Identifier_List
   --
   --
   --  Primitives for Bare_Discriminant_Choice_List
   --
   --
   --  Primitives for Bare_Name_List
   --
   --
   --  Primitives for Bare_Parent_List
   --
   --
   --  Primitives for Bare_Param_Spec_List
   --
   --
   --  Primitives for Bare_Pragma_Node_List
   --
   --
   --  Primitives for Bare_Select_When_Part_List
   --
   --
   --  Primitives for Bare_Unconstrained_Array_Index_List
   --
   --
   --  Primitives for Bare_Variant_List
   --
   --
   --  Primitives for Bare_Aliased_Node
   --
--# property-start '[dispatcher]Aliased.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Aliased_Node_P_As_Bool
  (Node : Bare_Aliased_Node
  )
   return Boolean
is
   Self : Bare_Aliased_Node  := Bare_Aliased_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Aliased_Node (Self.Kind) is
               when Ada_Aliased_Absent =>
                  --# property-call-start Aliased.Absent.as_bool
                  Property_Result := Aliased_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Aliased_Present =>
                  --# property-call-start Aliased.Present.as_bool
                  Property_Result := Aliased_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Aliased_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Aliased_Absent
   --
--# property-start Aliased.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5284
pragma Warnings (Off, "is not referenced");
 function Aliased_Absent_P_As_Bool
  (Node : Bare_Aliased_Absent
  )
   return Boolean
is
   Self : Bare_Aliased_Absent  := Bare_Aliased_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aliased_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Aliased_Present
   --
--# property-start Aliased.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5284
pragma Warnings (Off, "is not referenced");
 function Aliased_Present_P_As_Bool
  (Node : Bare_Aliased_Present
  )
   return Boolean
is
   Self : Bare_Aliased_Present  := Bare_Aliased_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aliased_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Node
   --
--# property-start '[dispatcher]All.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_All_Node_P_As_Bool
  (Node : Bare_All_Node
  )
   return Boolean
is
   Self : Bare_All_Node  := Bare_All_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_All_Node (Self.Kind) is
               when Ada_All_Absent =>
                  --# property-call-start All.Absent.as_bool
                  Property_Result := All_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_All_Present =>
                  --# property-call-start All.Present.as_bool
                  Property_Result := All_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_All_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Absent
   --
--# property-start All.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5308
pragma Warnings (Off, "is not referenced");
 function All_Absent_P_As_Bool
  (Node : Bare_All_Absent
  )
   return Boolean
is
   Self : Bare_All_Absent  := Bare_All_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end All_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Present
   --
--# property-start All.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5308
pragma Warnings (Off, "is not referenced");
 function All_Present_P_As_Bool
  (Node : Bare_All_Present
  )
   return Boolean
is
   Self : Bare_All_Present  := Bare_All_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end All_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Array_Indices
   --
--# property-start '[dispatcher]ArrayIndices.ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Ndims
  (Node : Bare_Array_Indices
  )
   return Integer
is
   Self : Bare_Array_Indices  := Bare_Array_Indices (Node);
      --# bind self Self
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.ndims
                  Property_Result := Constrained_Array_Indices_P_Ndims
                    (Self
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.ndims
                  Property_Result := Unconstrained_Array_Indices_P_Ndims
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Array_Indices_P_Ndims;
--# end
--# property-start '[dispatcher]ArrayIndices.constrain_index_expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Indices  := Bare_Array_Indices (Node);
      --# bind self Self
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.constrain_index_expr
                  Property_Result := Constrained_Array_Indices_P_Constrain_Index_Expr
                    (Self
                        , Index_Expr
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.constrain_index_expr
                  Property_Result := Unconstrained_Array_Indices_P_Constrain_Index_Expr
                    (Self
                        , Index_Expr
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start '[dispatcher]ArrayIndices.index_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Index_Type
  (Node : Bare_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Indices  := Bare_Array_Indices (Node);
      --# bind self Self
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.index_type
                  Property_Result := Constrained_Array_Indices_P_Index_Type
                    (Self
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.index_type
                  Property_Result := Unconstrained_Array_Indices_P_Index_Type
                    (Self
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Array_Indices_P_Index_Type;
--# end
--# property-start '[dispatcher]ArrayIndices.is_static' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Is_Static
  (Node : Bare_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Indices  := Bare_Array_Indices (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.is_static
                  Property_Result := Constrained_Array_Indices_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.is_static
                  Property_Result := Unconstrained_Array_Indices_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Constrained_Array_Indices
   --
      procedure Initialize_Fields_For_Constrained_Array_Indices
        (Self : Bare_Constrained_Array_Indices
         ; Constrained_Array_Indices_F_List : Bare_Constraint_List
        ) is
      begin
            Self.Constrained_Array_Indices_F_List := Constrained_Array_Indices_F_List;
      end Initialize_Fields_For_Constrained_Array_Indices;
   function Constrained_Array_Indices_F_List
     (Node : Bare_Constrained_Array_Indices) return Bare_Constraint_List
   is
   begin
         return Node.Constrained_Array_Indices_F_List;
   end;
--# property-start ConstrainedArrayIndices.ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8752
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Ndims
  (Node : Bare_Constrained_Array_Indices
  )
   return Integer
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
      --# bind self Self
   Property_Result : Integer;
      Fld : Bare_Constraint_List;
Len : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1010 '<length at ast.py:8752>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8752
--# expr-start 1009 '<FieldAccess for list at ast.py:8752>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8752
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Constrained_Array_Indices_F_List;
--# expr-done 1009
Len := Length (Fld); 
--# expr-done 1010
         Property_Result := Len;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Ndims;
--# end
--# property-start ConstrainedArrayIndices.constrain_index_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8754
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Constrained_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
     Ent : Internal_Entity_Constrained_Array_Indices :=
       Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_659 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_659 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_659;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1016 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1015 '<And at ast.py:8756>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8756
--# expr-start 1013 '<Bind at ast.py:8757>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8757
--# expr-start 1011 '<FieldAccess for expected_type_var at ast.py:8757>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8757
      if Index_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Index_Expr.Node);
--# expr-done 1011
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 1012 '<FieldAccess for index_type at ast.py:8757>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8757
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Ent.Node, Dim => Dim, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1012
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8757") else null)); 
--# expr-done 1013
--# expr-start 1014 '<FieldAccess for matches_expected_type at ast.py:8758>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8758
      if Index_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_2 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Index_Expr.Node, Origin => Origin);
--# end
--# expr-done 1014
And_Pred := Create_And (Bind_Result, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8756") else null)); 
--# expr-done 1015
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1016
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_659;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_659;
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start ConstrainedArrayIndices.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8761
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Xref_Equation
  (Node : Bare_Constrained_Array_Indices
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
     Ent : Internal_Entity_Constrained_Array_Indices :=
       Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_660 with Inline_Always;
            procedure Finalizer_Scope_1975 with Inline_Always;
            procedure Finalizer_Scope_1976 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Constraint_List;
Fld_1 : Bare_Constraint_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
Fld_5 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Var;
Pred_2 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Item_40 : Internal_Entity;
Bare_Item_40 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_660 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_660;
            procedure Finalizer_Scope_1975 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1975;
            procedure Finalizer_Scope_1976 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred_2);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1976;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1031 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1030 '<LogicAll at ast.py:8763>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8763
--# expr-start 1029 '<Map at ast.py:8763>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8763
   --# expr-start 1017 '<FieldAccess for list at ast.py:8763>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8763
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 1017
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Constraint_List := Fld_1;
      begin
         for Untyped_Item_40 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_40 := Untyped_Item_40; 
Is_Null := Bare_Item_40 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_40); 
   If_Result := As_Entity;
end if;
                  Item_40 := If_Result; 
   --# scope-start
               --# bind index Item_40
      --# expr-start 1028 '<And at ast.py:8765>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8765
--# expr-start 1018 '<FieldAccess for sub_equation at ast.py:8765>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8765
      if Item_40.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_40.Node, Env => Env, Origin => Origin, E_Info => Item_40.Info);
--# end
--# expr-done 1018
--# expr-start 1019 '<Cast to Expr at ast.py:8766>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8766
Cast_Expr := Item_40; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Expr;
   end if;
--# expr-done 1019
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr
--# expr-start 1027 '<And at ast.py:8768>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8768
--# expr-start 1024 '<And at ast.py:8768>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8768
--# expr-start 1021 '<Predicate on AdaNode.is_not_null at ast.py:8768>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8768
--# expr-start 1020 '<FieldAccess for type_var at ast.py:8768>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8768
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 1020
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_Predicate (Fld_4, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 1021
--# expr-start 1023 '<Predicate on BaseTypeDecl.is_discrete_type at ast.py:8769>' Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8769
--# expr-start 1022 '<FieldAccess for type_var at ast.py:8769>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8769
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 1022
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Pred_1 := Solver.Create_Predicate (Fld_5, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin)); 
--# expr-done 1023
And_Pred := Create_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8768") else null)); 
--# expr-done 1024
--# expr-start 1026 '<Predicate on BaseTypeDecl.is_not_root_int_type at ast.py:8770>' Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8770
--# expr-start 1025 '<FieldAccess for type_var at ast.py:8770>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8770
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 1025
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Pred_2 := Solver.Create_Predicate (Fld_6, Create_Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate); 
--# expr-done 1026
And_Pred_1 := Create_And (And_Pred, Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8768") else null)); 
--# expr-done 1027
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1976;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8771") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8765") else null)); 
--# expr-done 1028
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1975;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1029
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8763") else null)); 
--# expr-done 1030
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 1031
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_660;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_660;
                     Finalizer_Scope_1975;
                     Finalizer_Scope_1976;
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Xref_Equation;
--# end
--# property-start ConstrainedArrayIndices.index_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8775
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Index_Type
  (Node : Bare_Constrained_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
     Ent : Internal_Entity_Constrained_Array_Indices :=
       Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_661 with Inline_Always;
      Block_Var : Boolean;
Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Expr;
Match_2 : Internal_Entity;
Node_Parents : Bare_Ada_Node_Array_Access;
Fld : Boolean;
Find_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Constraint_List;
Fld_3 : Bare_Constraint_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_661 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_661;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1042 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 1036 '<FieldAccess for resolve_names at ast.py:8779>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8779
--# expr-start 1035 '<as_entity at ast.py:8779>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8779
--# expr-start 1034 '<Find at ast.py:8778>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8778
   --# expr-start 1032 '<FieldAccess for parents at ast.py:8778>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8778
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 1032
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_41 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_41
      --# expr-start 1033 '<FieldAccess for xref_entry_point at ast.py:8779>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8779
      if Item_41 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Item_41);
--# end
--# expr-done 1033
      if Fld then
         Find_Result := Item_41; 
      end if;
   --# end
               exit when Fld;
         end loop;
      end;
--# expr-done 1034
Is_Null := Find_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Find_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1035
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1036
Block_Var := Fld_1; 
--# expr-start 1037 '<FieldAccess for list at ast.py:8782>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8782
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 1037
Fld_3 := Fld_2.Node;
Get_Result := Get (Self, Fld_3, Dim, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_4 := Fld_2.Info;
As_Entity_1 := (Info => Fld_4, Node => Get_Result); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr := If_Result_1;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 1041 '<Match at ast.py:8782>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8782
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind st Match_0
--# expr-start 1038 '<FieldAccess for designated_type at ast.py:8783>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8783
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 1038
Let_Result := Fld_5; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 1040 '<Cast to BaseTypeDecl.entity at ast.py:8784>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8784
--# expr-start 1039 '<FieldAccess for type_val at ast.py:8784>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8784
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_6 := Libadalang.Implementation.Expr_P_Type_Val (Node => Match_1.Node);
--# end
--# expr-done 1039
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1040
Let_Result_1 := Cast_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 1041
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 1042
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_661;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_661;
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Index_Type;
--# end
--# property-start ConstrainedArrayIndices.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8788
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Is_Static
  (Node : Bare_Constrained_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
     Ent : Internal_Entity_Constrained_Array_Indices :=
       Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Bin_Op;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Constraint_List;
Fld_1 : Bare_Constraint_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Bin_Op;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Item_42 : Internal_Entity;
Bare_Item_42 : Bare_Ada_Node;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1050 '<AllQuantifier at ast.py:8788>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8788
   --# expr-start 1043 '<FieldAccess for list at ast.py:8788>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8788
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 1043
Fld_1 := Fld.Node;
      Quantifier_Result := True;
      if Fld_1 = null then
      null;
      else
      declare
         Collection : constant Bare_Constraint_List := Fld_1;
      begin
         for Untyped_Item_42 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_42 := Untyped_Item_42; 
Is_Null := Bare_Item_42 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_42); 
   If_Result := As_Entity;
end if;
                  Item_42 := If_Result; 
   --# scope-start
               --# bind t Item_42
      --# expr-start 1049 '<Match at ast.py:8789>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8789
      if Item_42.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_42; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 1044 '<FieldAccess for is_static_subtype at ast.py:8790>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8790
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_3 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 1044
Let_Result := Fld_3; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind e Match_1
--# expr-start 1046 '<FieldAccess for is_static_expr at ast.py:8791>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8791
--# expr-start 1045 '<FieldAccess for left at ast.py:8791>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8791
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 1045
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_5 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 1046
if Fld_5 then
   --# expr-start 1048 '<FieldAccess for is_static_expr at ast.py:8791>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8791
--# expr-start 1047 '<FieldAccess for right at ast.py:8791>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8791
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 1047
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_7 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_6.Info);
--# end
--# expr-done 1048
   If_Result_1 := Fld_7;
else
   If_Result_1 := False;
end if;
Let_Result_1 := If_Result_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 1049
      Quantifier_Result := Match_Result;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 1050
         Property_Result := Quantifier_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Unconstrained_Array_Indices
   --
      procedure Initialize_Fields_For_Unconstrained_Array_Indices
        (Self : Bare_Unconstrained_Array_Indices
         ; Unconstrained_Array_Indices_F_Types : Bare_Unconstrained_Array_Index_List
        ) is
      begin
            Self.Unconstrained_Array_Indices_F_Types := Unconstrained_Array_Indices_F_Types;
      end Initialize_Fields_For_Unconstrained_Array_Indices;
   function Unconstrained_Array_Indices_F_Types
     (Node : Bare_Unconstrained_Array_Indices) return Bare_Unconstrained_Array_Index_List
   is
   begin
         return Node.Unconstrained_Array_Indices_F_Types;
   end;
--# property-start UnconstrainedArrayIndices.ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8722
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Ndims
  (Node : Bare_Unconstrained_Array_Indices
  )
   return Integer
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
      --# bind self Self
   Property_Result : Integer;
      Fld : Bare_Unconstrained_Array_Index_List;
Len : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1052 '<length at ast.py:8722>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8722
--# expr-start 1051 '<FieldAccess for types at ast.py:8722>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8722
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Unconstrained_Array_Indices_F_Types;
--# expr-done 1051
Len := Length (Fld); 
--# expr-done 1052
         Property_Result := Len;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Ndims;
--# end
--# property-start UnconstrainedArrayIndices.constrain_index_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8724
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Unconstrained_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
     Ent : Internal_Entity_Unconstrained_Array_Indices :=
       Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_654 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_654 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_654;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1058 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1057 '<And at ast.py:8726>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8726
--# expr-start 1055 '<Bind at ast.py:8727>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8727
--# expr-start 1053 '<FieldAccess for expected_type_var at ast.py:8727>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8727
      if Index_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Index_Expr.Node);
--# expr-done 1053
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 1054 '<FieldAccess for index_type at ast.py:8727>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8727
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Ent.Node, Dim => Dim, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1054
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8727") else null)); 
--# expr-done 1055
--# expr-start 1056 '<FieldAccess for matches_expected_type at ast.py:8728>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8728
      if Index_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_2 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Index_Expr.Node, Origin => Origin);
--# end
--# expr-done 1056
And_Pred := Create_And (Bind_Result, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8726") else null)); 
--# expr-done 1057
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1058
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_654;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_654;
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start UnconstrainedArrayIndices.index_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8731
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Index_Type
  (Node : Bare_Unconstrained_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
     Ent : Internal_Entity_Unconstrained_Array_Indices :=
       Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Unconstrained_Array_Index;
Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Unconstrained_Array_Index;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1061 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1059 '<FieldAccess for types at ast.py:8733>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8733
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 1059
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, Dim, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Unconstrained_Array_Index then
   --# scope-start
--# expr-start 1060 '<FieldAccess for designated_type at ast.py:8733>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8733
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UnconstrainedArrayIndex.designated_type
Fld_3 := Libadalang.Implementation.Unconstrained_Array_Index_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1060
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1061
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Index_Type;
--# end
--# property-start UnconstrainedArrayIndices.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8735
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Xref_Equation
  (Node : Bare_Unconstrained_Array_Indices
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
     Ent : Internal_Entity_Unconstrained_Array_Indices :=
       Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_656 with Inline_Always;
            procedure Finalizer_Scope_1987 with Inline_Always;
      Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Subtype_Indication;
Fld_4 : Logic_Equation;
Item_43 : Internal_Entity_Unconstrained_Array_Index;
Bare_Item_43 : Bare_Unconstrained_Array_Index;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_656 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_656;
            procedure Finalizer_Scope_1987 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1987;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1067 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1066 '<LogicAll at ast.py:8737>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8737
--# expr-start 1065 '<Map at ast.py:8737>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8737
   --# expr-start 1062 '<FieldAccess for types at ast.py:8737>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8737
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 1062
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Unconstrained_Array_Index_List := Fld_1;
      begin
         for Untyped_Item_43 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_43 := Untyped_Item_43; 
Is_Null := Bare_Item_43 = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_43); 
   If_Result := As_Entity;
end if;
                  Item_43 := If_Result; 
   --# scope-start
               --# bind typ Item_43
      --# expr-start 1064 '<FieldAccess for sub_equation at ast.py:8738>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8738
--# expr-start 1063 '<FieldAccess for subtype_indication at ast.py:8738>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8738
      if Item_43.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subtype_Indication (Node => Item_43.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Item_43.Info);
--# expr-done 1063
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 1064
         declare
            Item_To_Append : constant Logic_Equation := Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1987;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1065
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8737") else null)); 
--# expr-done 1066
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 1067
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_656;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_656;
                     Finalizer_Scope_1987;
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Xref_Equation;
--# end
--# property-start UnconstrainedArrayIndices.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8741
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Is_Static
  (Node : Bare_Unconstrained_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
     Ent : Internal_Entity_Unconstrained_Array_Indices :=
       Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Subtype_Indication;
Fld_4 : Boolean;
Item_44 : Internal_Entity_Unconstrained_Array_Index;
Bare_Item_44 : Bare_Unconstrained_Array_Index;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1071 '<AllQuantifier at ast.py:8741>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8741
   --# expr-start 1068 '<FieldAccess for types at ast.py:8741>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8741
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 1068
Fld_1 := Fld.Node;
      Quantifier_Result := True;
      if Fld_1 = null then
      null;
      else
      declare
         Collection : constant Bare_Unconstrained_Array_Index_List := Fld_1;
      begin
         for Untyped_Item_44 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_44 := Untyped_Item_44; 
Is_Null := Bare_Item_44 = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_44); 
   If_Result := As_Entity;
end if;
                  Item_44 := If_Result; 
   --# scope-start
               --# bind t Item_44
      --# expr-start 1070 '<FieldAccess for is_static_subtype at ast.py:8742>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8742
--# expr-start 1069 '<FieldAccess for subtype_indication at ast.py:8742>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8742
      if Item_44.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subtype_Indication (Node => Item_44.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Item_44.Info);
--# expr-done 1069
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_4 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_3.Info);
--# end
--# expr-done 1070
      Quantifier_Result := Fld_4;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 1071
         Property_Result := Quantifier_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Aspect_Assoc
   --
      procedure Initialize_Fields_For_Aspect_Assoc
        (Self : Bare_Aspect_Assoc
         ; Aspect_Assoc_F_Id : Bare_Name
         ; Aspect_Assoc_F_Expr : Bare_Expr
        ) is
      begin
            Self.Aspect_Assoc_F_Id := Aspect_Assoc_F_Id;
            Self.Aspect_Assoc_F_Expr := Aspect_Assoc_F_Expr;
      end Initialize_Fields_For_Aspect_Assoc;
   function Aspect_Assoc_F_Id
     (Node : Bare_Aspect_Assoc) return Bare_Name
   is
   begin
         return Node.Aspect_Assoc_F_Id;
   end;
   function Aspect_Assoc_F_Expr
     (Node : Bare_Aspect_Assoc) return Bare_Expr
   is
   begin
         return Node.Aspect_Assoc_F_Expr;
   end;
--# property-start AspectAssoc.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10533
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Xref_Entry_Point
  (Node : Bare_Aspect_Assoc
  )
   return Boolean
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Xref_Entry_Point;
--# end
--# property-start AspectAssoc.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10535
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Xref_Equation
  (Node : Bare_Aspect_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
     Ent : Internal_Entity_Aspect_Assoc :=
       Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_822 with Inline_Always;
            procedure Finalizer_Scope_1989 with Inline_Always;
            procedure Finalizer_Scope_1991 with Inline_Always;
      Block_Var : Bare_Ada_Node;
Expr_Equation_Env : Lexical_Env := Empty_Env;
Var_Expr : Bare_Type_Decl;
Var_Expr_1 : Internal_Entity_Expr;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Name;
Sym : Symbol_Type;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Internal_Single_Actual_Array_Access;
Fld_8 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_10 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_11 : Internal_Entity_Name;
Fld_12 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Fld_13 : Internal_Entity_Name;
Fld_14 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_15 : Internal_Entity_Name;
Fld_16 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_17 : Internal_Entity_Name;
Fld_18 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_19 : Internal_Entity_Name;
Fld_20 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Fld_21 : Internal_Entity_Name;
Fld_22 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Fld_23 : Internal_Entity_Name;
Fld_24 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Fld_25 : Internal_Entity_Name;
Fld_26 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Fld_27 : Internal_Entity_Name;
Fld_28 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Cast_Expr : Bare_Ada_Node;
Cast_Result_2 : Bare_Type_Decl;
Fld_29 : Bare_Type_Def;
Fld_30 : Lexical_Env := Empty_Env;
Fld_31 : Lexical_Env := Empty_Env;
Fld_32 : Internal_Entity_Name;
Fld_33 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_9 : Boolean;
Fld_34 : Internal_Entity_Name;
Fld_35 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_8 : Boolean;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_3 : Bare_Type_Decl;
Fld_36 : Bare_Type_Def;
Scope_Result : Bare_Type_Def;
Result_Var : Bare_Type_Def;
Is_A : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Lexical_Env := Empty_Env;
Fld_37 : Bare_Expr;
Fld_38 : Logic_Var;
Fld_39 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Bound_Env : Lexical_Env := Empty_Env;
Fld_40 : Internal_Entity_Expr;
Fld_41 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_42 : Bare_Expr;
Fld_43 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
Fld_44 : Internal_Entity_Name;
Sym_12 : Symbol_Type;
Fld_45 : Boolean;
Fld_46 : Internal_Entity_Expr;
Fld_47 : Logic_Equation;
Fld_48 : Internal_Entity_Name;
Sym_13 : Symbol_Type;
Fld_49 : Boolean;
Fld_50 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Name;
Fld_51 : Logic_Equation;
Fld_52 : Internal_Entity_Expr;
Cast_Result_6 : Internal_Entity_Name;
Fld_53 : Logic_Var;
Cast_Expr_2 : Bare_Ada_Node;
Cast_Result_7 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result_11 : Internal_Entity_Base_Type_Decl;
Pred : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_54 : Internal_Entity_Name;
Sym_14 : Symbol_Type;
Fld_55 : Boolean;
Fld_56 : Internal_Entity_Name;
Sym_15 : Symbol_Type;
Fld_57 : Boolean;
If_Result_12 : Boolean;
True_Rel : Logic_Equation;
Fld_58 : Internal_Entity_Expr;
Fld_59 : Logic_Equation;
Fld_60 : Internal_Entity_Expr;
Is_A_1 : Boolean;
If_Result_13 : Logic_Equation;
Fld_61 : Internal_Entity_Name;
Sym_16 : Symbol_Type;
Fld_62 : Boolean;
True_Rel_1 : Logic_Equation;
Fld_63 : Internal_Entity_Name;
Sym_17 : Symbol_Type;
Fld_64 : Boolean;
Fld_65 : Logic_Equation;
Fld_66 : Internal_Entity_Name;
Sym_18 : Symbol_Type;
Fld_67 : Boolean;
Fld_68 : Internal_Entity_Expr;
Cast_Result_8 : Internal_Entity_Identifier;
Fld_69 : Logic_Var;
Cast_Expr_3 : Bare_Ada_Node;
Cast_Result_9 : Bare_Type_Decl;
As_Entity_1 : Internal_Entity_Type_Decl;
Is_Null_1 : Boolean;
If_Result_14 : Internal_Entity_Type_Decl;
Fld_70 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_71 : Internal_Entity_Name;
Sym_19 : Symbol_Type;
Fld_72 : Boolean;
Fld_73 : Internal_Entity_Expr;
Cast_Result_11 : Internal_Entity_Identifier;
Fld_74 : Logic_Var;
Cast_Expr_4 : Bare_Ada_Node;
Cast_Result_12 : Bare_Type_Decl;
As_Entity_2 : Internal_Entity_Type_Decl;
Is_Null_2 : Boolean;
If_Result_15 : Internal_Entity_Type_Decl;
Fld_75 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_13 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_76 : Internal_Entity_Expr;
Fld_77 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_2 : Logic_Equation;
Result_Var_1 : Logic_Equation;
True_Rel_3 : Logic_Equation;
Or_Pred : Logic_Equation;
If_Result_16 : Logic_Equation;
If_Result_17 : Logic_Equation;
If_Result_18 : Logic_Equation;
If_Result_19 : Logic_Equation;
If_Result_20 : Logic_Equation;
If_Result_21 : Logic_Equation;
If_Result_22 : Logic_Equation;
If_Result_23 : Logic_Equation;
If_Result_24 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_822 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_30);
                     Dec_Ref (Fld_31);
                     Dec_Ref (If_Result_10);
                     Dec_Ref (Expr_Equation_Env);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_41);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_43);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Fld_47);
                     Dec_Ref (Fld_51);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_59);
                     Dec_Ref (If_Result_13);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Fld_65);
                     Dec_Ref (Fld_70);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_75);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (True_Rel_3);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result_16);
                     Dec_Ref (If_Result_17);
                     Dec_Ref (If_Result_18);
                     Dec_Ref (If_Result_19);
                     Dec_Ref (If_Result_20);
                     Dec_Ref (If_Result_21);
                     Dec_Ref (If_Result_22);
                     Dec_Ref (If_Result_23);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_822;
            procedure Finalizer_Scope_1989 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1989;
            procedure Finalizer_Scope_1991 is
            begin
                     Dec_Ref (Fld_77);
            end Finalizer_Scope_1991;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1214 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 1074 '<parent at ast.py:10537>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10537
--# expr-start 1073 '<parent at ast.py:10537>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10537
--# expr-start 1072 '<parent at ast.py:10537>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10537
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 1072
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1073
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 1074
Block_Var := Fld_2; 
--# bind target Block_Var
--# expr-start 1213 '<Cond at ast.py:10538>' If_Result_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10538
--# expr-start 1077 '<FieldAccess for name_is at ast.py:10540>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10540
--# expr-start 1075 '<FieldAccess for id at ast.py:10540>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10540
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1075
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1076 '<Symbol Iterable at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 1076
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Sym);
--# end
--# expr-done 1077
if Fld_4 then
   --# expr-start 1087 '<LogicAll at ast.py:10541>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10541
--# expr-start 1086 '<Map at ast.py:10541>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10541
   --# expr-start 1081 '<FieldAccess for unpacked_params at ast.py:10541>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10541
--# expr-start 1080 '<FieldAccess for assocs at ast.py:10541>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10541
--# expr-start 1079 '<Cast to Aggregate at ast.py:10541>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10541
--# expr-start 1078 '<FieldAccess for expr at ast.py:10541>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10541
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1078
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 1079
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Base_Aggregate_F_Assocs, Info => Cast_Result.Info);
--# expr-done 1080
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_7 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1081
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_7;
      begin
         for Item_45 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_45
      --# expr-start 1085 '<FieldAccess for xref_no_overloading at ast.py:10544>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10544
--# expr-start 1084 '<Cast to Name at ast.py:10544>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10544
--# expr-start 1083 '<FieldAccess for expr at ast.py:10543>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10543
--# expr-start 1082 '<FieldAccess for assoc at ast.py:10543>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10543
Fld_8 := Item_45.Assoc;
--# expr-done 1082
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 1083
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1084
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_10 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Sequential => False, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 1085
         declare
            Item_To_Append : constant Logic_Equation := Fld_10;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1989;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1086
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10541") else null)); 
--# expr-done 1087
   If_Result_24 := Logic_Boolean_Op;
else
--# expr-start 1091 '<Eq at ast.py:10548>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1089 '<FieldAccess for name_symbol at ast.py:10548>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1088 '<FieldAccess for id at ast.py:10548>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1088
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_11.Node);
--# end
--# expr-done 1089
--# expr-start 1090 '<Symbol Pre at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 1090
Is_Equal := Fld_12 = Sym_1; 
--# expr-done 1091
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1095 '<Eq at ast.py:10548>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1093 '<FieldAccess for name_symbol at ast.py:10548>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1092 '<FieldAccess for id at ast.py:10548>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1092
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_13.Node);
--# end
--# expr-done 1093
--# expr-start 1094 '<Symbol Post at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 1094
Is_Equal_1 := Fld_14 = Sym_2; 
--# expr-done 1095
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 1099 '<Eq at ast.py:10548>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1097 '<FieldAccess for name_symbol at ast.py:10548>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1096 '<FieldAccess for id at ast.py:10548>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1096
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_15.Node);
--# end
--# expr-done 1097
--# expr-start 1098 '<Symbol Refined_Post at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Post); 
--# expr-done 1098
Is_Equal_2 := Fld_16 = Sym_3; 
--# expr-done 1099
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 1103 '<Eq at ast.py:10548>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1101 '<FieldAccess for name_symbol at ast.py:10548>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1100 '<FieldAccess for id at ast.py:10548>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1100
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_17.Node);
--# end
--# expr-done 1101
--# expr-start 1102 '<Symbol Type_Invariant at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Invariant); 
--# expr-done 1102
Is_Equal_3 := Fld_18 = Sym_4; 
--# expr-done 1103
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 1107 '<Eq at ast.py:10548>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1105 '<FieldAccess for name_symbol at ast.py:10548>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1104 '<FieldAccess for id at ast.py:10548>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1104
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_19.Node);
--# end
--# expr-done 1105
--# expr-start 1106 '<Symbol Invariant at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invariant); 
--# expr-done 1106
Is_Equal_4 := Fld_20 = Sym_5; 
--# expr-done 1107
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 1111 '<Eq at ast.py:10548>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1109 '<FieldAccess for name_symbol at ast.py:10548>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1108 '<FieldAccess for id at ast.py:10548>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1108
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_22 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_21.Node);
--# end
--# expr-done 1109
--# expr-start 1110 '<Symbol Predicate at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate); 
--# expr-done 1110
Is_Equal_5 := Fld_22 = Sym_6; 
--# expr-done 1111
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 1115 '<Eq at ast.py:10548>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1113 '<FieldAccess for name_symbol at ast.py:10548>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1112 '<FieldAccess for id at ast.py:10548>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1112
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_24 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_23.Node);
--# end
--# expr-done 1113
--# expr-start 1114 '<Symbol Static_Predicate at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Static_Predicate); 
--# expr-done 1114
Is_Equal_6 := Fld_24 = Sym_7; 
--# expr-done 1115
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 1119 '<Eq at ast.py:10548>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1117 '<FieldAccess for name_symbol at ast.py:10548>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1116 '<FieldAccess for id at ast.py:10548>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1116
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_26 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_25.Node);
--# end
--# expr-done 1117
--# expr-start 1118 '<Symbol Dynamic_Predicate at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Dynamic_Predicate); 
--# expr-done 1118
Is_Equal_7 := Fld_26 = Sym_8; 
--# expr-done 1119
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 1123 '<Eq at ast.py:10548>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1121 '<FieldAccess for name_symbol at ast.py:10548>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
--# expr-start 1120 '<FieldAccess for id at ast.py:10548>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1120
      if Fld_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_28 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_27.Node);
--# end
--# expr-done 1121
--# expr-start 1122 '<Symbol Initial_Condition at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Initial_Condition); 
--# expr-done 1122
Is_Equal_8 := Fld_28 = Sym_9; 
--# expr-done 1123
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   --# expr-start 1151 '<Let at ast.py:10553>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10553
--# scope-start
--# expr-start 1139 '<If at ast.py:10554>' If_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10554
--# expr-start 1127 '<Eq at ast.py:10559>' Is_Equal_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10559
--# expr-start 1125 '<FieldAccess for name_symbol at ast.py:10559>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10559
--# expr-start 1124 '<FieldAccess for id at ast.py:10559>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10559
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1124
      if Fld_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_33 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_32.Node);
--# end
--# expr-done 1125
--# expr-start 1126 '<Symbol Pre at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 1126
Is_Equal_9 := Fld_33 = Sym_10; 
--# expr-done 1127
if Is_Equal_9 then
   If_Result_8 := True;
else
   --# expr-start 1131 '<Eq at ast.py:10559>' Is_Equal_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10559
--# expr-start 1129 '<FieldAccess for name_symbol at ast.py:10559>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10559
--# expr-start 1128 '<FieldAccess for id at ast.py:10559>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10559
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1128
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_35 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_34.Node);
--# end
--# expr-done 1129
--# expr-start 1130 '<Symbol Post at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 1130
Is_Equal_10 := Fld_35 = Sym_11; 
--# expr-done 1131
   If_Result_8 := Is_Equal_10;
end if;
if If_Result_8 then
   --# expr-start 1134 '<IsA AccessToSubpDef at ast.py:10560>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10560
--# expr-start 1132 '<Cast to TypeDecl at ast.py:10560>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10560
Cast_Expr_1 := Block_Var; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Type_Decl
   then
      Cast_Result_3 := Cast_Expr_1;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 1132
Var_Expr := Cast_Result_3;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1133 '<FieldAccess for type_def at ast.py:10560>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10560
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Var_Expr.Type_Decl_F_Type_Def;
--# expr-done 1133
Scope_Result := Fld_36;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_A := Result_Var /= null 
and then Result_Var.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 1134
   If_Result_9 := Is_A;
else
   If_Result_9 := False;
end if;
if If_Result_9 then
   --# expr-start 1137 '<FieldAccess for children_env at ast.py:10561>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10561
--# expr-start 1136 '<FieldAccess for type_def at ast.py:10561>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10561
--# expr-start 1135 '<Cast to TypeDecl at ast.py:10561>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10561
Cast_Expr := Block_Var; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Type_Decl
   then
      Cast_Result_2 := Cast_Expr;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 1135
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Cast_Result_2.Type_Decl_F_Type_Def;
--# expr-done 1136
      if Fld_29 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Libadalang.Implementation.Children_Env (Node => Fld_29);
--# expr-done 1137
   If_Result_10 := Fld_30;
else
   --# expr-start 1138 '<FieldAccess for node_env at ast.py:10562>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10562
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1138
   If_Result_10 := Fld_31;
end if;
      Inc_Ref (If_Result_10);
--# expr-done 1139
Expr_Equation_Env := If_Result_10; Inc_Ref (Expr_Equation_Env);
--# bind expr_equation_env Expr_Equation_Env
--# expr-start 1150 '<And at ast.py:10564>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10564
--# expr-start 1147 '<And at ast.py:10564>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10564
--# expr-start 1143 '<Bind at ast.py:10564>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10564
--# expr-start 1141 '<FieldAccess for expected_type_var at ast.py:10564>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10564
--# expr-start 1140 '<FieldAccess for expr at ast.py:10564>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10564
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Self.Aspect_Assoc_F_Expr;
--# expr-done 1140
      if Fld_37 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_37);
--# expr-done 1141
Fld_38.Value := No_Entity;
Entity_Vars.Reset (Fld_38);
--# expr-start 1142 '<FieldAccess for bool_type at ast.py:10564>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10564
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_39 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 1142
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_39.Node,
         Info => Fld_39.Info);
Bind_Result := Solver.Create_Assign (Fld_38, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10564") else null)); 
--# expr-done 1143
--# expr-start 1146 '<bind at ast.py:10565>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10565
Bound_Env := Expr_Equation_Env; Inc_Ref (Bound_Env);
--# expr-start 1145 '<FieldAccess for sub_equation at ast.py:10566>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10566
--# expr-start 1144 '<FieldAccess for expr at ast.py:10566>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10566
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1144
      if Fld_40.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_41 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_40.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_40.Info);
--# end
--# expr-done 1145
Dyn_Var_Bind_Result := Fld_41; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 1146
And_Pred := Create_And (Bind_Result, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10564") else null)); 
--# expr-done 1147
--# expr-start 1149 '<FieldAccess for matches_expected_formal_prim_type at ast.py:10567>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10567
--# expr-start 1148 '<FieldAccess for expr at ast.py:10567>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10567
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Self.Aspect_Assoc_F_Expr;
--# expr-done 1148
      if Fld_42 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_43 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_42, Origin => Origin);
--# end
--# expr-done 1149
And_Pred_1 := Create_And (And_Pred, Fld_43, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10564") else null)); 
--# expr-done 1150
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1151
   If_Result_23 := Let_Result;
else
--# expr-start 1154 '<FieldAccess for name_is at ast.py:10570>' Fld_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10570
--# expr-start 1152 '<FieldAccess for id at ast.py:10570>' Fld_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10570
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_44 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1152
      if Fld_44.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1153 '<Symbol Contract_Cases at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 1153
--# property-call-start Name.name_is
Fld_45 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_44.Node, Sym => Sym_12);
--# end
--# expr-done 1154
if Fld_45 then
   --# expr-start 1156 '<FieldAccess for sub_equation at ast.py:10571>' Fld_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10571
--# expr-start 1155 '<FieldAccess for expr at ast.py:10571>' Fld_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10571
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_46 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1155
      if Fld_46.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_47 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_46.Node, Env => Env, Origin => Origin, E_Info => Fld_46.Info);
--# end
--# expr-done 1156
   If_Result_22 := Fld_47;
else
--# expr-start 1159 '<FieldAccess for name_is at ast.py:10574>' Fld_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10574
--# expr-start 1157 '<FieldAccess for id at ast.py:10574>' Fld_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10574
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1157
      if Fld_48.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1158 '<Symbol Put_Image at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image); 
--# expr-done 1158
--# property-call-start Name.name_is
Fld_49 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_48.Node, Sym => Sym_13);
--# end
--# expr-done 1159
if Fld_49 then
   --# expr-start 1169 '<And at ast.py:10575>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10575
--# expr-start 1162 '<FieldAccess for xref_no_overloading at ast.py:10575>' Fld_51 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10575
--# expr-start 1161 '<Cast to Name at ast.py:10575>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10575
--# expr-start 1160 '<FieldAccess for expr at ast.py:10575>' Fld_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10575
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1160
   if Fld_50.Node = null
      or else Fld_50.Node.Kind in Ada_Name
   then
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Fld_50.Node,
         Info => Fld_50.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1161
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_51 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_5.Node, Sequential => False, All_Els => True, Env => Env, Origin => Origin, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 1162
--# expr-start 1168 '<Predicate on BasicDecl.is_put_image_subprogram_for_type at ast.py:10577>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10577
--# expr-start 1165 '<FieldAccess for ref_var at ast.py:10579>' Fld_53 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10579
--# expr-start 1164 '<Cast to Name at ast.py:10579>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10579
--# expr-start 1163 '<FieldAccess for expr at ast.py:10579>' Fld_52 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10579
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_52 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1163
   if Fld_52.Node = null
      or else Fld_52.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Fld_52.Node,
         Info => Fld_52.Info);
   else
         Cast_Result_6 := No_Entity_Name;
   end if;
--# expr-done 1164
      if Cast_Result_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_53 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_6.Node);
--# end
--# expr-done 1165
Fld_53.Value := No_Entity;
Entity_Vars.Reset (Fld_53);
--# expr-start 1167 '<as_entity at ast.py:10580>' If_Result_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10580
--# expr-start 1166 '<Cast to BaseTypeDecl at ast.py:10580>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10580
Cast_Expr_2 := Block_Var; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_7 := Cast_Expr_2;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1166
Is_Null := Cast_Result_7 = null; 
if Is_Null then
   If_Result_11 := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_7); 
   If_Result_11 := As_Entity;
end if;
--# expr-done 1167
Pred := Solver.Create_Predicate (Fld_53, Create_Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate (If_Result_11)); 
--# expr-done 1168
And_Pred_2 := Create_And (Fld_51, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10575") else null)); 
--# expr-done 1169
   If_Result_21 := And_Pred_2;
else
--# expr-start 1172 '<FieldAccess for name_is at ast.py:10585>' Fld_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10585
--# expr-start 1170 '<FieldAccess for id at ast.py:10585>' Fld_54 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10585
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_54 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1170
      if Fld_54.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1171 '<Symbol Global at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Global); 
--# expr-done 1171
--# property-call-start Name.name_is
Fld_55 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_54.Node, Sym => Sym_14);
--# end
--# expr-done 1172
if Fld_55 then
   If_Result_12 := True;
else
   --# expr-start 1175 '<FieldAccess for name_is at ast.py:10585>' Fld_57 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10585
--# expr-start 1173 '<FieldAccess for id at ast.py:10585>' Fld_56 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10585
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_56 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1173
      if Fld_56.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1174 '<Symbol Refined_Global at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Global); 
--# expr-done 1174
--# property-call-start Name.name_is
Fld_57 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_56.Node, Sym => Sym_15);
--# end
--# expr-done 1175
   If_Result_12 := Fld_57;
end if;
if If_Result_12 then
   --# expr-start 1180 '<If at ast.py:10586>' If_Result_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10586
--# expr-start 1177 '<IsA NullLiteral at ast.py:10587>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10587
--# expr-start 1176 '<FieldAccess for expr at ast.py:10587>' Fld_60 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10587
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_60 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1176
Is_A_1 := Fld_60.Node /= null 
and then Fld_60.Node.Kind in Ada_Null_Literal_Range; 
--# expr-done 1177
if Is_A_1 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10588") else null)); 
   If_Result_13 := True_Rel;
else
   --# expr-start 1179 '<FieldAccess for sub_equation at ast.py:10589>' Fld_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10589
--# expr-start 1178 '<FieldAccess for expr at ast.py:10589>' Fld_58 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10589
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_58 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1178
      if Fld_58.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_59 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_58.Node, Env => Env, Origin => Origin, E_Info => Fld_58.Info);
--# end
--# expr-done 1179
   If_Result_13 := Fld_59;
end if;
      Inc_Ref (If_Result_13);
--# expr-done 1180
   If_Result_20 := If_Result_13;
else
--# expr-start 1183 '<FieldAccess for name_is at ast.py:10595>' Fld_62 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10595
--# expr-start 1181 '<FieldAccess for id at ast.py:10595>' Fld_61 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10595
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_61 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1181
      if Fld_61.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1182 '<Symbol Convention at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Convention); 
--# expr-done 1182
--# property-call-start Name.name_is
Fld_62 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_61.Node, Sym => Sym_16);
--# end
--# expr-done 1183
if Fld_62 then
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10595") else null)); 
   If_Result_19 := True_Rel_1;
else
--# expr-start 1186 '<FieldAccess for name_is at ast.py:10597>' Fld_64 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10597
--# expr-start 1184 '<FieldAccess for id at ast.py:10597>' Fld_63 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10597
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_63 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1184
      if Fld_63.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1185 '<Symbol Stable_Properties at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Stable_Properties); 
--# expr-done 1185
--# property-call-start Name.name_is
Fld_64 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_63.Node, Sym => Sym_17);
--# end
--# expr-done 1186
if Fld_64 then
   --# expr-start 1187 '<FieldAccess for stable_properties_assoc_equation at ast.py:10598>' Fld_65 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10598
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AspectAssoc.stable_properties_assoc_equation
Fld_65 := Libadalang.Implementation.Aspect_Assoc_P_Stable_Properties_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1187
   If_Result_18 := Fld_65;
else
--# expr-start 1190 '<FieldAccess for name_is at ast.py:10604>' Fld_67 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10604
--# expr-start 1188 '<FieldAccess for id at ast.py:10604>' Fld_66 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10604
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_66 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1188
      if Fld_66.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1189 '<Symbol Constant_Indexing at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant_Indexing); 
--# expr-done 1189
--# property-call-start Name.name_is
Fld_67 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_66.Node, Sym => Sym_18);
--# end
--# expr-done 1190
if Fld_67 then
   --# expr-start 1198 '<Bind at ast.py:10605>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10605
--# expr-start 1193 '<FieldAccess for ref_var at ast.py:10605>' Fld_69 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10605
--# expr-start 1192 '<Cast to Identifier at ast.py:10605>' Cast_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10605
--# expr-start 1191 '<FieldAccess for expr at ast.py:10605>' Fld_68 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10605
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_68 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1191
   if Fld_68.Node = null
      or else Fld_68.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result_8 := Create_Internal_Entity_Identifier
        (Node => Fld_68.Node,
         Info => Fld_68.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1192
      if Cast_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_69 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_8.Node);
--# end
--# expr-done 1193
Fld_69.Value := No_Entity;
Entity_Vars.Reset (Fld_69);
--# expr-start 1197 '<at at ast.py:10606>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10606
--# expr-start 1196 '<FieldAccess for constant_indexing_fns at ast.py:10606>' Fld_70 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10606
--# expr-start 1195 '<as_entity at ast.py:10606>' If_Result_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10606
--# expr-start 1194 '<Cast to TypeDecl at ast.py:10606>' Cast_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10606
Cast_Expr_3 := Block_Var; 
   if Cast_Expr_3 = null
      or else Cast_Expr_3.Kind in Ada_Type_Decl
   then
      Cast_Result_9 := Cast_Expr_3;
   else
         Cast_Result_9 := No_Bare_Ada_Node;
   end if;
--# expr-done 1194
Is_Null_1 := Cast_Result_9 = null; 
if Is_Null_1 then
   If_Result_14 := No_Entity_Type_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_9); 
   If_Result_14 := As_Entity_1;
end if;
--# expr-done 1195
      if If_Result_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_70 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => If_Result_14.Node, E_Info => If_Result_14.Info);
--# end
--# expr-done 1196
Get_Result := Get (Self, Fld_70, 0, True); 
--# expr-done 1197
      Cast_Result_10 := Create_Internal_Entity
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_69, Cast_Result_10, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10605") else null)); 
--# expr-done 1198
   If_Result_17 := Bind_Result_1;
else
--# expr-start 1201 '<FieldAccess for name_is at ast.py:10608>' Fld_72 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10608
--# expr-start 1199 '<FieldAccess for id at ast.py:10608>' Fld_71 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10608
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_71 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1199
      if Fld_71.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1200 '<Symbol Variable_Indexing at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Variable_Indexing); 
--# expr-done 1200
--# property-call-start Name.name_is
Fld_72 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_71.Node, Sym => Sym_19);
--# end
--# expr-done 1201
if Fld_72 then
   --# expr-start 1209 '<Bind at ast.py:10609>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10609
--# expr-start 1204 '<FieldAccess for ref_var at ast.py:10609>' Fld_74 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10609
--# expr-start 1203 '<Cast to Identifier at ast.py:10609>' Cast_Result_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10609
--# expr-start 1202 '<FieldAccess for expr at ast.py:10609>' Fld_73 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10609
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_73 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1202
   if Fld_73.Node = null
      or else Fld_73.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result_11 := Create_Internal_Entity_Identifier
        (Node => Fld_73.Node,
         Info => Fld_73.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1203
      if Cast_Result_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_74 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_11.Node);
--# end
--# expr-done 1204
Fld_74.Value := No_Entity;
Entity_Vars.Reset (Fld_74);
--# expr-start 1208 '<at at ast.py:10610>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10610
--# expr-start 1207 '<FieldAccess for variable_indexing_fns at ast.py:10610>' Fld_75 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10610
--# expr-start 1206 '<as_entity at ast.py:10610>' If_Result_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10610
--# expr-start 1205 '<Cast to TypeDecl at ast.py:10610>' Cast_Result_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10610
Cast_Expr_4 := Block_Var; 
   if Cast_Expr_4 = null
      or else Cast_Expr_4.Kind in Ada_Type_Decl
   then
      Cast_Result_12 := Cast_Expr_4;
   else
         Cast_Result_12 := No_Bare_Ada_Node;
   end if;
--# expr-done 1205
Is_Null_2 := Cast_Result_12 = null; 
if Is_Null_2 then
   If_Result_15 := No_Entity_Type_Decl;
else
As_Entity_2 := (Info => E_Info, Node => Cast_Result_12); 
   If_Result_15 := As_Entity_2;
end if;
--# expr-done 1206
      if If_Result_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_75 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => If_Result_15.Node, E_Info => If_Result_15.Info);
--# end
--# expr-done 1207
Get_Result_1 := Get (Self, Fld_75, 0, True); 
--# expr-done 1208
      Cast_Result_13 := Create_Internal_Entity
        (Node => Get_Result_1.Node,
         Info => Get_Result_1.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_74, Cast_Result_13, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10609") else null)); 
--# expr-done 1209
   If_Result_16 := Bind_Result_2;
else
   --# expr-start 1212 '<Or at ast.py:10615>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10615
--# expr-start 1210 '<FieldAccess for expr at ast.py:10615>' Fld_76 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10615
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_76 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1210
Var_Expr_1 := Fld_76;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr_1
--# expr-start 1211 '<FieldAccess for sub_equation at ast.py:10615>' Fld_77 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10615
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_77 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1211
Scope_Result_1 := Fld_77;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1991;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10615") else null)); 
   Result_Var_1 := True_Rel_2;
end if;
      Inc_Ref (Result_Var_1);
True_Rel_3 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10616") else null)); 
Or_Pred := Create_Or (Result_Var_1, True_Rel_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10615") else null)); 
--# expr-done 1212
   If_Result_16 := Or_Pred;
end if;
      Inc_Ref (If_Result_16);
   If_Result_17 := If_Result_16;
end if;
      Inc_Ref (If_Result_17);
   If_Result_18 := If_Result_17;
end if;
      Inc_Ref (If_Result_18);
   If_Result_19 := If_Result_18;
end if;
      Inc_Ref (If_Result_19);
   If_Result_20 := If_Result_19;
end if;
      Inc_Ref (If_Result_20);
   If_Result_21 := If_Result_20;
end if;
      Inc_Ref (If_Result_21);
   If_Result_22 := If_Result_21;
end if;
      Inc_Ref (If_Result_22);
   If_Result_23 := If_Result_22;
end if;
      Inc_Ref (If_Result_23);
   If_Result_24 := If_Result_23;
end if;
      Inc_Ref (If_Result_24);
--# expr-done 1213
Let_Result_1 := If_Result_24; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1214
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_822;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_822;
                     Finalizer_Scope_1989;
                     Finalizer_Scope_1991;
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Xref_Equation;
--# end
--# property-start AspectAssoc.stable_properties_assoc_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10619
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Stable_Properties_Assoc_Equation
  (Node : Bare_Aspect_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
     Ent : Internal_Entity_Aspect_Assoc :=
       Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_823 with Inline_Always;
            procedure Finalizer_Scope_1992 with Inline_Always;
            procedure Finalizer_Scope_1994 with Inline_Always;
            procedure Finalizer_Scope_1995 with Inline_Always;
            procedure Finalizer_Scope_2000 with Inline_Always;
            procedure Finalizer_Scope_2002 with Inline_Always;
      Block_Var : Internal_Entity_Identifier_Array_Access;
Match_0 : Internal_Entity_Aggregate;
Match_1 : Internal_Entity_Paren_Expr;
Match_2 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Un_Op;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_4 : Internal_Entity_Expr;
Item_48 : Internal_Entity_Basic_Assoc;
Bare_Item_48 : Bare_Basic_Assoc;
Map_Result : Internal_Entity_Expr_Array_Access;
Fld_5 : Internal_Entity_Expr;
Singleton : Internal_Entity_Expr_Array_Access;
Singleton_1 : Internal_Entity_Expr_Array_Access;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Let_Result : Internal_Entity_Expr_Array_Access;
Scope_Result : Internal_Entity_Expr_Array_Access;
Cast_Result_1 : Internal_Entity_Paren_Expr;
Let_Result_1 : Internal_Entity_Expr_Array_Access;
Scope_Result_1 : Internal_Entity_Expr_Array_Access;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_2 : Internal_Entity_Expr_Array_Access;
Scope_Result_2 : Internal_Entity_Expr_Array_Access;
Match_Result : Internal_Entity_Expr_Array_Access;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Un_Op;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Internal_Entity_Op;
Is_A : Boolean;
If_Result_1 : Internal_Entity_Expr;
Scope_Result_3 : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Map_Result_1 : Internal_Entity_Expr_Array_Access;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Identifier;
Map_Result_2 : Internal_Entity_Identifier_Array_Access;
Fld_8 : Symbol_Type;
Fld_9 : Bare_Ada_Node;
Fld_10 : Internal_Entity_Array_Access;
Is_A_1 : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Boolean;
If_Result_2 : Boolean;
Map_Result_3 : Internal_Entity_Array_Access;
Fld_13 : Logic_Var;
Bind_Result : Logic_Equation;
Map_Result_4 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Map_Result_5 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_823 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result_5);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_823;
            procedure Finalizer_Scope_1992 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1992;
            procedure Finalizer_Scope_1994 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1994;
            procedure Finalizer_Scope_1995 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_1995;
            procedure Finalizer_Scope_2000 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_2000;
            procedure Finalizer_Scope_2002 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_2002;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1243 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 1228 '<Map at ast.py:10640>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10640
   --# expr-start 1226 '<Map at ast.py:10633>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10633
   --# expr-start 1220 '<Match at ast.py:10627>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10627
--# expr-start 1215 '<FieldAccess for expr at ast.py:10627>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10627
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1215
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Aggregate .. Ada_Bracket_Aggregate =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind a Match_0
--# expr-start 1218 '<Map at ast.py:10629>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10629
   --# expr-start 1216 '<FieldAccess for assocs at ast.py:10629>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10629
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Match_0.Node.Base_Aggregate_F_Assocs, Info => Match_0.Info);
--# expr-done 1216
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_2;
      begin
         for Untyped_Item_48 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_48 := Untyped_Item_48; 
Is_Null := Bare_Item_48 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_48); 
   If_Result := As_Entity;
end if;
                  Item_48 := If_Result; 
   --# scope-start
               --# bind i Item_48
      --# expr-start 1217 '<FieldAccess for expr at ast.py:10629>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10629
      if Item_48.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_48.Node, E_Info => Item_48.Info);
--# end
--# expr-done 1217
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1218
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1992;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind pe Match_1
--# expr-start 1219 '<FieldAccess for expr at ast.py:10631>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10631
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Match_1.Node.Paren_Expr_F_Expr, Info => Match_1.Info);
--# expr-done 1219
                Singleton := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_5;
Let_Result_1 := Singleton; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1994;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl_Expr .. Ada_Allocator | Ada_Delta_Aggregate .. Ada_Update_Attribute_Ref | Ada_Quantified_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
                Singleton_1 := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton_1.Items (1) := No_Entity_Expr;
Let_Result_2 := Singleton_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1995;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 1220
      declare
         Map_Result_1_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Match_Result;
      begin
         for Item_47 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_47
--# expr-start 1221 '<Cast to UnOp at ast.py:10636>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10636
Cast_Expr := Item_47; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Un_Op_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Un_Op
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Un_Op;
   end if;
--# expr-done 1221
Var_Expr := Cast_Result_3;
if Var_Expr /= No_Entity_Un_Op then
   --# scope-start
   --# bind uo Var_Expr
--# expr-start 1225 '<If at ast.py:10637>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10637
--# expr-start 1223 '<IsA Op.Not at ast.py:10637>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10637
--# expr-start 1222 '<FieldAccess for op at ast.py:10637>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10637
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Op (Node => Var_Expr.Node.Un_Op_F_Op, Info => Var_Expr.Info);
--# expr-done 1222
Is_A := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Op_Not_Range; 
--# expr-done 1223
if Is_A then
   --# expr-start 1224 '<FieldAccess for expr at ast.py:10637>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10637
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Un_Op_F_Expr, Info => Var_Expr.Info);
--# expr-done 1224
   If_Result_1 := Fld_6;
else
   If_Result_1 := Item_47;
end if;
--# expr-done 1225
Scope_Result_3 := If_Result_1;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := Item_47;
end if;
         declare
            Item_To_Append : constant Internal_Entity_Expr := Result_Var;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1226
      declare
         Map_Result_2_Vec : Internal_Entity_Identifier_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Map_Result_1;
      begin
         for Item_46 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_46
      --# expr-start 1227 '<Cast to Identifier at ast.py:10642>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10642
Cast_Expr_1 := Item_46; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Identifier
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1227
         declare
            Item_To_Append : constant Internal_Entity_Identifier := Cast_Result_4;
         begin
            Internal_Entity_Identifier_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Identifier_Array
           (Items_Count => Natural (Internal_Entity_Identifier_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Identifier_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Identifier_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Identifier_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 1228
Block_Var := Map_Result_2; Inc_Ref (Block_Var);
--# bind identifiers Block_Var
--# expr-start 1242 '<LogicAll at ast.py:10646>' Logic_Boolean_Op_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10646
--# expr-start 1241 '<Map at ast.py:10646>' Map_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10646
      declare
         Map_Result_5_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Identifier_Array_Access := Block_Var;
      begin
         for Item_49 of
               Collection.Items
         loop
   --# scope-start
               --# bind i Item_49
      --# expr-start 1240 '<LogicAny at ast.py:10659>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10659
--# expr-start 1239 '<Map at ast.py:10659>' Map_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10659
   --# expr-start 1236 '<Filter at ast.py:10653>' Map_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10653
   --# expr-start 1231 '<FieldAccess for env_get at ast.py:10647>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10647
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1229 '<FieldAccess for sym at ast.py:10649>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10649
      if Item_49.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_8 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Item_49.Node);
--# end
--# expr-done 1229
--# expr-start 1230 '<FieldAccess for origin_node at ast.py:10651>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10651
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_9 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1230
--# property-call-start AdaNode.env_get
Fld_10 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld_8, Lookup => Recursive, From_Node => Fld_9, Categories => (Inherited_Primitives => True, Nocat => True));
--# end
--# expr-done 1231
      declare
         Map_Result_3_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_10;
      begin
         for Item_51 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_51
--# expr-start 1232 '<IsA SubpDecl, ExprFunction at ast.py:10655>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10655
Is_A_1 := Item_51.Node /= null 
and then Item_51.Node.Kind in Ada_Subp_Decl_Range | Ada_Expr_Function_Range; 
--# expr-done 1232
if Is_A_1 then
   --# expr-start 1235 '<FieldAccess for denotes_the_property_function at ast.py:10656>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10656
      if Item_49.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1234 '<FieldAccess for subp_spec_or_null at ast.py:10657>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10657
--# expr-start 1233 '<Cast to BasicDecl at ast.py:10657>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10657
Cast_Expr_2 := Item_51; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Decl;
   end if;
--# expr-done 1233
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_5.Node, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 1234
--# property-call-start BaseId.denotes_the_property_function
Fld_12 := Libadalang.Implementation.Base_Id_P_Denotes_The_Property_Function (Node => Item_49.Node, Subp_Spec => Fld_11, E_Info => Item_49.Info);
--# end
--# expr-done 1235
   If_Result_2 := Fld_12;
else
   If_Result_2 := False;
end if;
         if If_Result_2 then
         declare
            Item_To_Append : constant Internal_Entity := Item_51;
         begin
            Internal_Entity_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 1236
      declare
         Map_Result_4_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result_3;
      begin
         for Item_50 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_50
      --# expr-start 1238 '<Bind at ast.py:10660>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10660
--# expr-start 1237 '<FieldAccess for ref_var at ast.py:10660>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10660
      if Item_49.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Item_49.Node);
--# end
--# expr-done 1237
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
Bind_Result := Solver.Create_Assign (Fld_13, Item_50, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10660") else null)); 
--# expr-done 1238
         declare
            Item_To_Append : constant Logic_Equation := Bind_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2002;
         end loop;
      end;
         Map_Result_4 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_4_Vec)));
         for I in Map_Result_4.Items'Range loop
            Map_Result_4.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_4_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_4.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_4_Vec);
      end;
--# expr-done 1239
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result_4.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10659") else null)); 
--# expr-done 1240
         declare
            Item_To_Append : constant Logic_Equation := Logic_Boolean_Op;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_5_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2000;
         end loop;
      end;
         Map_Result_5 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_5_Vec)));
         for I in Map_Result_5.Items'Range loop
            Map_Result_5.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_5_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_5.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_5_Vec);
      end;
--# expr-done 1241
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_5.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10646") else null)); 
--# expr-done 1242
Let_Result_3 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 1243
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_823;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_823;
                     Finalizer_Scope_1992;
                     Finalizer_Scope_1994;
                     Finalizer_Scope_1995;
                     Finalizer_Scope_2000;
                     Finalizer_Scope_2002;
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Stable_Properties_Assoc_Equation;
--# end
--# property-start AspectAssoc.aspect_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10664
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Aspect_Name
  (Node : Bare_Aspect_Assoc
      ; N : Internal_Entity_Name
  )
   return String_Type
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
      --# bind self Self
   --# bind n N
   Property_Result : String_Type;
            procedure Finalizer_Scope_824 with Inline_Always;
            procedure Finalizer_Scope_2003 with Inline_Always;
            procedure Finalizer_Scope_2004 with Inline_Always;
      Match_0 : Internal_Entity_Base_Id;
Match_1 : Internal_Entity_Attribute_Ref;
Match_2 : Internal_Entity_Name;
Fld : Symbol_Type;
Fld_1 : String_Type;
Fld_2 : Internal_Entity_Name;
Fld_3 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Fld_6 : String_Type;
Concat_Result_1 : String_Type;
Match_Prefix : Internal_Entity_Name;
Cast_Result : Internal_Entity_Base_Id;
Let_Result : String_Type;
Scope_Result : String_Type;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Let_Result_1 : String_Type;
Scope_Result_1 : String_Type;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_2 : String_Type;
Scope_Result_2 : String_Type;
Match_Result : String_Type;
Let_Result_3 : String_Type;
            procedure Finalizer_Scope_824 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_824;
            procedure Finalizer_Scope_2003 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2003;
            procedure Finalizer_Scope_2004 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_2004;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1255 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 1254 '<Match at ast.py:10672>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10672
      if N.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := N; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bid Match_0
--# expr-start 1245 '<FieldAccess for image at ast.py:10673>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10673
--# expr-start 1244 '<FieldAccess for sym at ast.py:10673>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10673
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_0.Node);
--# end
--# expr-done 1244
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Image (Self => Fld);
--# expr-done 1245
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2003;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ar Match_1
--# expr-start 1253 '<Concat at ast.py:10676>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10676
--# expr-start 1249 '<Concat at ast.py:10675>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10675
--# expr-start 1247 '<FieldAccess for aspect_name at ast.py:10674>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10674
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1246 '<FieldAccess for prefix at ast.py:10674>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10674
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Attribute_Ref_F_Prefix, Info => Match_1.Info);
--# expr-done 1246
--# property-call-start AspectAssoc.aspect_name
Fld_3 := Libadalang.Implementation.Aspect_Assoc_P_Aspect_Name (Node => Self, N => Fld_2);
--# end
--# expr-done 1247
--# expr-start 1248 '<String at ast.py:10675>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10675
Str := Create_String ("'"); 
--# expr-done 1248
Concat_Result := Concat_String (Fld_3, Str); 
--# expr-done 1249
--# expr-start 1252 '<FieldAccess for image at ast.py:10676>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10676
--# expr-start 1251 '<FieldAccess for sym at ast.py:10676>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10676
--# expr-start 1250 '<FieldAccess for attribute at ast.py:10676>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10676
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Match_1.Node.Attribute_Ref_F_Attribute, Info => Match_1.Info);
--# expr-done 1250
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_5 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_4.Node);
--# end
--# expr-done 1251
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Image (Self => Fld_5);
--# expr-done 1252
Concat_Result_1 := Concat_String (Concat_Result, Fld_6); 
--# expr-done 1253
Let_Result_1 := Concat_Result_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2004;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "aspect_name called on an invalid aspect name"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 1254
Let_Result_3 := Match_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 1255
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_824;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_824;
                     Finalizer_Scope_2003;
                     Finalizer_Scope_2004;
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Aspect_Name;
--# end
--# property-start AspectAssoc.is_ghost_code /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10683
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Is_Ghost_Code
  (Node : Bare_Aspect_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
     Ent : Internal_Entity_Aspect_Assoc :=
       Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Name;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1268 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1259 '<Eq at ast.py:10688>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
--# expr-start 1257 '<FieldAccess for name_symbol at ast.py:10688>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
--# expr-start 1256 '<FieldAccess for id at ast.py:10688>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1256
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 1257
--# expr-start 1258 '<Symbol Pre at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 1258
Is_Equal := Fld_1 = Sym; 
--# expr-done 1259
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1263 '<Eq at ast.py:10688>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
--# expr-start 1261 '<FieldAccess for name_symbol at ast.py:10688>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
--# expr-start 1260 '<FieldAccess for id at ast.py:10688>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1260
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 1261
--# expr-start 1262 '<Symbol Post at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 1262
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 1263
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 1267 '<Eq at ast.py:10688>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
--# expr-start 1265 '<FieldAccess for name_symbol at ast.py:10688>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
--# expr-start 1264 '<FieldAccess for id at ast.py:10688>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10688
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1264
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4.Node);
--# end
--# expr-done 1265
--# expr-start 1266 '<Symbol Contract_Cases at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 1266
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 1267
   If_Result_1 := Is_Equal_2;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 1268
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Is_Ghost_Code;
--# end
   --
   --  Primitives for Bare_Aspect_Clause
   --
--# property-start AspectClause.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10313
pragma Warnings (Off, "is not referenced");
 function Aspect_Clause_P_Xref_Entry_Point
  (Node : Bare_Aspect_Clause
  )
   return Boolean
is
   Self : Bare_Aspect_Clause  := Bare_Aspect_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aspect_Clause_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_At_Clause
   --
      procedure Initialize_Fields_For_At_Clause
        (Self : Bare_At_Clause
         ; At_Clause_F_Name : Bare_Base_Id
         ; At_Clause_F_Expr : Bare_Expr
        ) is
      begin
            Self.At_Clause_F_Name := At_Clause_F_Name;
            Self.At_Clause_F_Expr := At_Clause_F_Expr;
      end Initialize_Fields_For_At_Clause;
   function At_Clause_F_Name
     (Node : Bare_At_Clause) return Bare_Base_Id
   is
   begin
         return Node.At_Clause_F_Name;
   end;
   function At_Clause_F_Expr
     (Node : Bare_At_Clause) return Bare_Expr
   is
   begin
         return Node.At_Clause_F_Expr;
   end;
--# property-start AtClause.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10450
pragma Warnings (Off, "is not referenced");
 function At_Clause_P_Xref_Equation
  (Node : Bare_At_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_At_Clause  := Bare_At_Clause (Node);
     Ent : Internal_Entity_At_Clause :=
       Internal_Entity_At_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_810 with Inline_Always;
      Fld : Internal_Entity_Base_Id;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_810 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_810;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1274 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1273 '<And at ast.py:10452>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10452
--# expr-start 1270 '<FieldAccess for sub_equation at ast.py:10452>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10452
--# expr-start 1269 '<FieldAccess for name at ast.py:10452>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10452
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Id (Node => Ent.Node.At_Clause_F_Name, Info => Ent.Info);
--# expr-done 1269
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1270
--# expr-start 1272 '<FieldAccess for sub_equation at ast.py:10452>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10452
--# expr-start 1271 '<FieldAccess for expr at ast.py:10452>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10452
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.At_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1271
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 1272
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10452") else null)); 
--# expr-done 1273
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1274
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_810;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_810;
            raise;
      end;
   return Property_Result;
end At_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Attribute_Def_Clause
   --
      procedure Initialize_Fields_For_Attribute_Def_Clause
        (Self : Bare_Attribute_Def_Clause
         ; Attribute_Def_Clause_F_Attribute_Expr : Bare_Name
         ; Attribute_Def_Clause_F_Expr : Bare_Expr
        ) is
      begin
            Self.Attribute_Def_Clause_F_Attribute_Expr := Attribute_Def_Clause_F_Attribute_Expr;
            Self.Attribute_Def_Clause_F_Expr := Attribute_Def_Clause_F_Expr;
      end Initialize_Fields_For_Attribute_Def_Clause;
   function Attribute_Def_Clause_F_Attribute_Expr
     (Node : Bare_Attribute_Def_Clause) return Bare_Name
   is
   begin
         return Node.Attribute_Def_Clause_F_Attribute_Expr;
   end;
   function Attribute_Def_Clause_F_Expr
     (Node : Bare_Attribute_Def_Clause) return Bare_Expr
   is
   begin
         return Node.Attribute_Def_Clause_F_Expr;
   end;
--# property-start AttributeDefClause.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10357
pragma Warnings (Off, "is not referenced");
 function Attribute_Def_Clause_P_Xref_Equation
  (Node : Bare_Attribute_Def_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Def_Clause  := Bare_Attribute_Def_Clause (Node);
     Ent : Internal_Entity_Attribute_Def_Clause :=
       Internal_Entity_Attribute_Def_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_806 with Inline_Always;
      Block_Var : Internal_Entity_Attribute_Ref;
Block_Var_1 : Symbol_Type;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Attribute_Ref;
Fld_1 : Internal_Entity_Identifier;
Fld_2 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_3 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_6 : Logic_Var;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Internal_Entity_Name;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
Fld_11 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Fld_12 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Name;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Name;
Fld_16 : Internal_Entity_Base_Type_Decl;
Pred_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_17 : Internal_Entity_Name;
Fld_18 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
Fld_21 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
Fld_22 : Bare_Expr;
Fld_23 : Logic_Var;
Sym_7 : Symbol_Type;
Fld_24 : Internal_Entity;
Bind_Result : Logic_Equation;
Sym_8 : Symbol_Type;
Is_Equal_7 : Boolean;
Fld_25 : Bare_Expr;
Fld_26 : Logic_Var;
Fld_27 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
If_Result_3 : Logic_Equation;
If_Result_4 : Logic_Equation;
And_Pred_5 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_806 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_806;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1337 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1276 '<Cast to AttributeRef at ast.py:10359>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10359
--# expr-start 1275 '<FieldAccess for attribute_expr at ast.py:10359>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10359
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Def_Clause_F_Attribute_Expr, Info => Ent.Info);
--# expr-done 1275
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Create_Internal_Entity_Attribute_Ref
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1276
Block_Var := Cast_Result; 
--# bind attr Block_Var
--# expr-start 1278 '<FieldAccess for name_symbol at ast.py:10360>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10360
--# expr-start 1277 '<FieldAccess for attribute at ast.py:10360>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10360
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Identifier (Node => Block_Var.Node.Attribute_Ref_F_Attribute, Info => Block_Var.Info);
--# expr-done 1277
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 1278
Block_Var_1 := Fld_2; 
--# bind rel_name Block_Var_1
--# expr-start 1336 '<Cond at ast.py:10362>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10362
--# expr-start 1280 '<Eq at ast.py:10363>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10363
--# expr-start 1279 '<Symbol Read at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read); 
--# expr-done 1279
Is_Equal := Block_Var_1 = Sym; 
--# expr-done 1280
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1282 '<Eq at ast.py:10363>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10363
--# expr-start 1281 '<Symbol Write at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write); 
--# expr-done 1281
Is_Equal_1 := Block_Var_1 = Sym_1; 
--# expr-done 1282
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 1284 '<Eq at ast.py:10363>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10363
--# expr-start 1283 '<Symbol Input at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input); 
--# expr-done 1283
Is_Equal_2 := Block_Var_1 = Sym_2; 
--# expr-done 1284
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 1286 '<Eq at ast.py:10363>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10363
--# expr-start 1285 '<Symbol Output at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output); 
--# expr-done 1285
Is_Equal_3 := Block_Var_1 = Sym_3; 
--# expr-done 1286
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   --# expr-start 1301 '<And at ast.py:10364>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10364
--# expr-start 1298 '<And at ast.py:10364>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10364
--# expr-start 1289 '<FieldAccess for xref_no_overloading at ast.py:10364>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10364
--# expr-start 1288 '<Cast to Name at ast.py:10364>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10364
--# expr-start 1287 '<FieldAccess for expr at ast.py:10364>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10364
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1287
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1288
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_4 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 1289
--# expr-start 1297 '<Predicate on BasicDecl.is_stream_subprogram_for_type at ast.py:10365>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10365
--# expr-start 1292 '<FieldAccess for ref_var at ast.py:10367>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10367
--# expr-start 1291 '<Cast to Name at ast.py:10367>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10367
--# expr-start 1290 '<FieldAccess for expr at ast.py:10367>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10367
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1290
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 1291
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_2.Node);
--# end
--# expr-done 1292
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 1294 '<FieldAccess for name_designated_type at ast.py:10368>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10368
--# expr-start 1293 '<FieldAccess for prefix at ast.py:10368>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10368
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Block_Var.Node.Attribute_Ref_F_Prefix, Info => Block_Var.Info);
--# expr-done 1293
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 1294
--# expr-start 1296 '<Eq at ast.py:10369>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10369
--# expr-start 1295 '<Symbol Input at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input); 
--# expr-done 1295
Is_Equal_4 := Block_Var_1 = Sym_4; 
--# expr-done 1296
Pred := Solver.Create_Predicate (Fld_6, Create_Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate (Fld_8, Is_Equal_4)); 
--# expr-done 1297
And_Pred := Create_And (Fld_4, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10364") else null)); 
--# expr-done 1298
--# expr-start 1300 '<FieldAccess for sub_equation at ast.py:10371>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10371
--# expr-start 1299 '<FieldAccess for prefix at ast.py:10371>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10371
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Block_Var.Node.Attribute_Ref_F_Prefix, Info => Block_Var.Info);
--# expr-done 1299
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 1300
And_Pred_1 := Create_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10364") else null)); 
--# expr-done 1301
   If_Result_6 := And_Pred_1;
else
--# expr-start 1303 '<Eq at ast.py:10373>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10373
--# expr-start 1302 '<Symbol Put_Image at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image); 
--# expr-done 1302
Is_Equal_5 := Block_Var_1 = Sym_5; 
--# expr-done 1303
if Is_Equal_5 then
   --# expr-start 1316 '<And at ast.py:10374>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10374
--# expr-start 1313 '<And at ast.py:10374>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10374
--# expr-start 1306 '<FieldAccess for xref_no_overloading at ast.py:10374>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10374
--# expr-start 1305 '<Cast to Name at ast.py:10374>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10374
--# expr-start 1304 '<FieldAccess for expr at ast.py:10374>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10374
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1304
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1305
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_12 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_3.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 1306
--# expr-start 1312 '<Predicate on BasicDecl.is_put_image_subprogram_for_type at ast.py:10375>' Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10375
--# expr-start 1309 '<FieldAccess for ref_var at ast.py:10377>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10377
--# expr-start 1308 '<Cast to Name at ast.py:10377>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10377
--# expr-start 1307 '<FieldAccess for expr at ast.py:10377>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10377
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1307
   if Fld_13.Node = null
      or else Fld_13.Node.Kind in Ada_Name
   then
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   else
         Cast_Result_4 := No_Entity_Name;
   end if;
--# expr-done 1308
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_4.Node);
--# end
--# expr-done 1309
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 1311 '<FieldAccess for name_designated_type at ast.py:10378>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10378
--# expr-start 1310 '<FieldAccess for prefix at ast.py:10378>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10378
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Name (Node => Block_Var.Node.Attribute_Ref_F_Prefix, Info => Block_Var.Info);
--# expr-done 1310
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_16 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_15.Node, E_Info => Fld_15.Info);
--# end
--# expr-done 1311
Pred_1 := Solver.Create_Predicate (Fld_14, Create_Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate (Fld_16)); 
--# expr-done 1312
And_Pred_2 := Create_And (Fld_12, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10374") else null)); 
--# expr-done 1313
--# expr-start 1315 '<FieldAccess for sub_equation at ast.py:10380>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10380
--# expr-start 1314 '<FieldAccess for prefix at ast.py:10380>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10380
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Block_Var.Node.Attribute_Ref_F_Prefix, Info => Block_Var.Info);
--# expr-done 1314
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_17.Node, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 1315
And_Pred_3 := Create_And (And_Pred_2, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10374") else null)); 
--# expr-done 1316
   If_Result_5 := And_Pred_3;
else
   --# expr-start 1335 '<And at ast.py:10382>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10382
--# expr-start 1320 '<And at ast.py:10382>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10382
--# expr-start 1318 '<FieldAccess for sub_equation at ast.py:10382>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10382
--# expr-start 1317 '<FieldAccess for expr at ast.py:10382>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10382
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1317
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_19.Node, Env => Env, Origin => Origin, E_Info => Fld_19.Info);
--# end
--# expr-done 1318
--# expr-start 1319 '<FieldAccess for sub_equation at ast.py:10383>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10383
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_21 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var.Node, Env => Env, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1319
And_Pred_4 := Create_And (Fld_20, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10382") else null)); 
--# expr-done 1320
--# expr-start 1334 '<Cond at ast.py:10384>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10384
--# expr-start 1322 '<Eq at ast.py:10385>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10385
--# expr-start 1321 '<Symbol External_Tag at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_External_Tag); 
--# expr-done 1321
Is_Equal_6 := Block_Var_1 = Sym_6; 
--# expr-done 1322
if Is_Equal_6 then
   --# expr-start 1327 '<Bind at ast.py:10386>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10386
--# expr-start 1324 '<FieldAccess for expected_type_var at ast.py:10386>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10386
--# expr-start 1323 '<FieldAccess for expr at ast.py:10386>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10386
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Self.Attribute_Def_Clause_F_Expr;
--# expr-done 1323
      if Fld_22 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_22);
--# expr-done 1324
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
--# expr-start 1326 '<FieldAccess for std_entity at ast.py:10386>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10386
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1325 '<Symbol String at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 1325
--# property-call-start AdaNode.std_entity
Fld_24 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_7);
--# end
--# expr-done 1326
Bind_Result := Solver.Create_Assign (Fld_23, Fld_24, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10386") else null)); 
--# expr-done 1327
   If_Result_4 := Bind_Result;
else
--# expr-start 1329 '<Eq at ast.py:10388>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10388
--# expr-start 1328 '<Symbol Address at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address); 
--# expr-done 1328
Is_Equal_7 := Block_Var_1 = Sym_8; 
--# expr-done 1329
if Is_Equal_7 then
   --# expr-start 1333 '<Bind at ast.py:10389>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10389
--# expr-start 1331 '<FieldAccess for expected_type_var at ast.py:10389>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10389
--# expr-start 1330 '<FieldAccess for expr at ast.py:10389>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10389
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Self.Attribute_Def_Clause_F_Expr;
--# expr-done 1330
      if Fld_25 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_25);
--# expr-done 1331
Fld_26.Value := No_Entity;
Entity_Vars.Reset (Fld_26);
--# expr-start 1332 '<FieldAccess for system_address_type at ast.py:10389>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10389
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.system_address_type
Fld_27 := Libadalang.Implementation.Ada_Node_P_System_Address_Type (Node => Ent.Node);
--# end
--# expr-done 1332
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_27.Node,
         Info => Fld_27.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_26, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10389") else null)); 
--# expr-done 1333
   If_Result_3 := Bind_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10391") else null)); 
   If_Result_3 := True_Rel;
end if;
      Inc_Ref (If_Result_3);
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 1334
And_Pred_5 := Create_And (And_Pred_4, If_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10382") else null)); 
--# expr-done 1335
   If_Result_5 := And_Pred_5;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 1336
Let_Result := If_Result_6; Inc_Ref (Let_Result);
--# end
--# expr-done 1337
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_806;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_806;
            raise;
      end;
   return Property_Result;
end Attribute_Def_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Enum_Rep_Clause
   --
      procedure Initialize_Fields_For_Enum_Rep_Clause
        (Self : Bare_Enum_Rep_Clause
         ; Enum_Rep_Clause_F_Type_Name : Bare_Name
         ; Enum_Rep_Clause_F_Aggregate : Bare_Base_Aggregate
        ) is
      begin
            Self.Enum_Rep_Clause_F_Type_Name := Enum_Rep_Clause_F_Type_Name;
            Self.Enum_Rep_Clause_F_Aggregate := Enum_Rep_Clause_F_Aggregate;
      end Initialize_Fields_For_Enum_Rep_Clause;
   function Enum_Rep_Clause_F_Type_Name
     (Node : Bare_Enum_Rep_Clause) return Bare_Name
   is
   begin
         return Node.Enum_Rep_Clause_F_Type_Name;
   end;
   function Enum_Rep_Clause_F_Aggregate
     (Node : Bare_Enum_Rep_Clause) return Bare_Base_Aggregate
   is
   begin
         return Node.Enum_Rep_Clause_F_Aggregate;
   end;
--# property-start EnumRepClause.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10323
pragma Warnings (Off, "is not referenced");
 function Enum_Rep_Clause_P_Xref_Equation
  (Node : Bare_Enum_Rep_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Enum_Rep_Clause  := Bare_Enum_Rep_Clause (Node);
     Ent : Internal_Entity_Enum_Rep_Clause :=
       Internal_Entity_Enum_Rep_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_804 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_804 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_804;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1340 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1339 '<FieldAccess for xref_no_overloading at ast.py:10326>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10326
--# expr-start 1338 '<FieldAccess for type_name at ast.py:10326>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10326
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Enum_Rep_Clause_F_Type_Name, Info => Ent.Info);
--# expr-done 1338
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1339
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1340
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_804;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_804;
            raise;
      end;
   return Property_Result;
end Enum_Rep_Clause_P_Xref_Equation;
--# end
--# property-start EnumRepClause.params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10328
pragma Warnings (Off, "is not referenced");
 function Enum_Rep_Clause_P_Params
  (Node : Bare_Enum_Rep_Clause
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Enum_Rep_Clause  := Bare_Enum_Rep_Clause (Node);
     Ent : Internal_Entity_Enum_Rep_Clause :=
       Internal_Entity_Enum_Rep_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_805 with Inline_Always;
      Block_Var : Internal_Entity_Enum_Literal_Decl_List;
Block_Var_1 : Internal_Entity_Assoc_List;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Decl;
Fld_3 : Internal_Entity_Type_Def;
Cast_Result_2 : Internal_Entity_Enum_Type_Def;
Fld_4 : Internal_Entity_Enum_Literal_Decl_List;
Fld_5 : Internal_Entity_Base_Aggregate;
Fld_6 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Enum_Literal_Decl_List;
Fld_7 : Bare_Enum_Literal_Decl_List;
Fld_8 : Internal_Entity_Info;
As_Entity : Internal_Entity_Enum_Literal_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Literal_Decl;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
Item_52 : Internal_Entity_Enum_Literal_Decl;
Bare_Item_52 : Bare_Enum_Literal_Decl;
Index_53 : Integer;
Map_Result : Internal_Param_Actual_Array_Access;
Let_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_805 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_805;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1356 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1348 '<FieldAccess for enum_literals at ast.py:10337>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10337
--# expr-start 1347 '<Cast to EnumTypeDef at ast.py:10336>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10336
--# expr-start 1346 '<FieldAccess for type_def at ast.py:10336>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10336
--# expr-start 1345 '<Cast to TypeDecl at ast.py:10336>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10336
--# expr-start 1344 '<FieldAccess for root_type at ast.py:10336>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10336
--# expr-start 1343 '<Cast to BaseTypeDecl at ast.py:10335>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10335
--# expr-start 1342 '<FieldAccess for referenced_decl at ast.py:10335>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10335
--# expr-start 1341 '<FieldAccess for type_name at ast.py:10335>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10335
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Enum_Rep_Clause_F_Type_Name, Info => Ent.Info);
--# expr-done 1341
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld.Node, Imprecise_Fallback => False, E_Info => Fld.Info);
--# end
--# expr-done 1342
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1343
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Cast_Result.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1344
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 1345
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Cast_Result_1.Node.Type_Decl_F_Type_Def, Info => Cast_Result_1.Info);
--# expr-done 1346
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Enum_Type_Def_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Enum_Type_Def
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_2 := No_Entity_Enum_Type_Def;
   end if;
--# expr-done 1347
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Enum_Literal_Decl_List (Node => Cast_Result_2.Node.Enum_Type_Def_F_Enum_Literals, Info => Cast_Result_2.Info);
--# expr-done 1348
Block_Var := Fld_4; 
--# bind el Block_Var
--# expr-start 1350 '<FieldAccess for assocs at ast.py:10339>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10339
--# expr-start 1349 '<FieldAccess for aggregate at ast.py:10339>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10339
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Base_Aggregate (Node => Ent.Node.Enum_Rep_Clause_F_Aggregate, Info => Ent.Info);
--# expr-done 1349
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Fld_5.Node.Base_Aggregate_F_Assocs, Info => Fld_5.Info);
--# expr-done 1350
Block_Var_1 := Fld_6; 
--# bind ra Block_Var_1
--# expr-start 1355 '<Map at ast.py:10341>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10341
Saved := Block_Var; 
Fld_7 := Saved.Node;
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      if Fld_7 = null then
      Map_Result := Create_Internal_Param_Actual_Array (0);
      else
         Index_53 := 0;
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld_7;
      begin
         for Untyped_Item_52 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_52 := Untyped_Item_52; 
Is_Null := Bare_Item_52 = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Literal_Decl;
else
Fld_8 := Saved.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_52); 
   If_Result := As_Entity;
end if;
                  Item_52 := If_Result; 
   --# scope-start
               --# bind l Item_52
               --# bind i Index_53
      --# expr-start 1354 '<New ParamActual at ast.py:10342>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10342
--# expr-start 1352 '<FieldAccess for actual_for_param_at at ast.py:10344>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10344
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1351 '<FieldAccess for name at ast.py:10344>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10344
      if Item_52.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Defining_Name (Node => Item_52.Node.Enum_Literal_Decl_F_Name, Info => Item_52.Info);
--# expr-done 1351
--# property-call-start AssocList.actual_for_param_at
Fld_11 := Libadalang.Implementation.Assoc_List_P_Actual_For_Param_At (Node => Block_Var_1.Node, Param => Fld_10, Pos => Index_53, E_Info => Block_Var_1.Info);
--# end
--# expr-done 1352
--# expr-start 1353 '<FieldAccess for name at ast.py:10343>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10343
      if Item_52.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Defining_Name (Node => Item_52.Node.Enum_Literal_Decl_F_Name, Info => Item_52.Info);
--# expr-done 1353
New_Struct := (Actual => Fld_11, Param => Fld_9); 
--# expr-done 1354
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
               Index_53 := Index_53 + 1;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1355
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1356
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_805;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_805;
            raise;
      end;
   return Property_Result;
end Enum_Rep_Clause_P_Params;
--# end
   --
   --  Primitives for Bare_Record_Rep_Clause
   --
      procedure Initialize_Fields_For_Record_Rep_Clause
        (Self : Bare_Record_Rep_Clause
         ; Record_Rep_Clause_F_Name : Bare_Name
         ; Record_Rep_Clause_F_At_Expr : Bare_Expr
         ; Record_Rep_Clause_F_Components : Bare_Ada_Node_List
        ) is
      begin
            Self.Record_Rep_Clause_F_Name := Record_Rep_Clause_F_Name;
            Self.Record_Rep_Clause_F_At_Expr := Record_Rep_Clause_F_At_Expr;
            Self.Record_Rep_Clause_F_Components := Record_Rep_Clause_F_Components;
      end Initialize_Fields_For_Record_Rep_Clause;
   function Record_Rep_Clause_F_Name
     (Node : Bare_Record_Rep_Clause) return Bare_Name
   is
   begin
         return Node.Record_Rep_Clause_F_Name;
   end;
   function Record_Rep_Clause_F_At_Expr
     (Node : Bare_Record_Rep_Clause) return Bare_Expr
   is
   begin
         return Node.Record_Rep_Clause_F_At_Expr;
   end;
   function Record_Rep_Clause_F_Components
     (Node : Bare_Record_Rep_Clause) return Bare_Ada_Node_List
   is
   begin
         return Node.Record_Rep_Clause_F_Components;
   end;
--# property-start RecordRepClause.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10432
pragma Warnings (Off, "is not referenced");
 function Record_Rep_Clause_P_Xref_Equation
  (Node : Bare_Record_Rep_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Record_Rep_Clause  := Bare_Record_Rep_Clause (Node);
     Ent : Internal_Entity_Record_Rep_Clause :=
       Internal_Entity_Record_Rep_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_809 with Inline_Always;
            procedure Finalizer_Scope_2007 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_809 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_809;
            procedure Finalizer_Scope_2007 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2007;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1362 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1361 '<And at ast.py:10434>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10434
--# expr-start 1358 '<FieldAccess for xref_no_overloading at ast.py:10435>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10435
--# expr-start 1357 '<FieldAccess for name at ast.py:10435>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10435
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Record_Rep_Clause_F_Name, Info => Ent.Info);
--# expr-done 1357
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1358
--# expr-start 1359 '<FieldAccess for at_expr at ast.py:10436>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10436
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Record_Rep_Clause_F_At_Expr, Info => Ent.Info);
--# expr-done 1359
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 1360 '<FieldAccess for sub_equation at ast.py:10437>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10437
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1360
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2007;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10438") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10434") else null)); 
--# expr-done 1361
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1362
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_809;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_809;
                     Finalizer_Scope_2007;
            raise;
      end;
   return Property_Result;
end Record_Rep_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Aspect_Spec
   --
      procedure Initialize_Fields_For_Aspect_Spec
        (Self : Bare_Aspect_Spec
         ; Aspect_Spec_F_Aspect_Assocs : Bare_Aspect_Assoc_List
        ) is
      begin
            Self.Aspect_Spec_F_Aspect_Assocs := Aspect_Spec_F_Aspect_Assocs;
      end Initialize_Fields_For_Aspect_Spec;
   function Aspect_Spec_F_Aspect_Assocs
     (Node : Bare_Aspect_Spec) return Bare_Aspect_Assoc_List
   is
   begin
         return Node.Aspect_Spec_F_Aspect_Assocs;
   end;
   --
   --  Primitives for Bare_Base_Assoc
   --
--# property-start '[dispatcher]BaseAssoc.assoc_expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Assoc_P_Assoc_Expr
  (Node : Bare_Base_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Assoc  := Bare_Base_Assoc (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Assoc (Self.Kind) is
               when Ada_Contract_Case_Assoc =>
                  --# property-call-start ContractCaseAssoc.assoc_expr
                  Property_Result := Contract_Case_Assoc_P_Assoc_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Argument_Assoc =>
                  --# property-call-start PragmaArgumentAssoc.assoc_expr
                  Property_Result := Pragma_Argument_Assoc_P_Assoc_Expr
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Contract_Case_Assoc
   --
      procedure Initialize_Fields_For_Contract_Case_Assoc
        (Self : Bare_Contract_Case_Assoc
         ; Contract_Case_Assoc_F_Guard : Bare_Ada_Node
         ; Contract_Case_Assoc_F_Consequence : Bare_Expr
        ) is
      begin
            Self.Contract_Case_Assoc_F_Guard := Contract_Case_Assoc_F_Guard;
            Self.Contract_Case_Assoc_F_Consequence := Contract_Case_Assoc_F_Consequence;
      end Initialize_Fields_For_Contract_Case_Assoc;
   function Contract_Case_Assoc_F_Guard
     (Node : Bare_Contract_Case_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Contract_Case_Assoc_F_Guard;
   end;
   function Contract_Case_Assoc_F_Consequence
     (Node : Bare_Contract_Case_Assoc) return Bare_Expr
   is
   begin
         return Node.Contract_Case_Assoc_F_Consequence;
   end;
--# property-start ContractCaseAssoc.assoc_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12750
pragma Warnings (Off, "is not referenced");
 function Contract_Case_Assoc_P_Assoc_Expr
  (Node : Bare_Contract_Case_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Contract_Case_Assoc  := Bare_Contract_Case_Assoc (Node);
     Ent : Internal_Entity_Contract_Case_Assoc :=
       Internal_Entity_Contract_Case_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1363 '<FieldAccess for consequence at ast.py:12750>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12750
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Contract_Case_Assoc_F_Consequence, Info => Ent.Info);
--# expr-done 1363
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Contract_Case_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Pragma_Argument_Assoc
   --
      procedure Initialize_Fields_For_Pragma_Argument_Assoc
        (Self : Bare_Pragma_Argument_Assoc
         ; Pragma_Argument_Assoc_F_Name : Bare_Name
         ; Pragma_Argument_Assoc_F_Expr : Bare_Expr
        ) is
      begin
            Self.Pragma_Argument_Assoc_F_Name := Pragma_Argument_Assoc_F_Name;
            Self.Pragma_Argument_Assoc_F_Expr := Pragma_Argument_Assoc_F_Expr;
      end Initialize_Fields_For_Pragma_Argument_Assoc;
   function Pragma_Argument_Assoc_F_Name
     (Node : Bare_Pragma_Argument_Assoc) return Bare_Name
   is
   begin
         return Node.Pragma_Argument_Assoc_F_Name;
   end;
   function Pragma_Argument_Assoc_F_Expr
     (Node : Bare_Pragma_Argument_Assoc) return Bare_Expr
   is
   begin
         return Node.Pragma_Argument_Assoc_F_Expr;
   end;
--# property-start PragmaArgumentAssoc.assoc_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10305
pragma Warnings (Off, "is not referenced");
 function Pragma_Argument_Assoc_P_Assoc_Expr
  (Node : Bare_Pragma_Argument_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Pragma_Argument_Assoc  := Bare_Pragma_Argument_Assoc (Node);
     Ent : Internal_Entity_Pragma_Argument_Assoc :=
       Internal_Entity_Pragma_Argument_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1364 '<FieldAccess for expr at ast.py:10305>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10305
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Pragma_Argument_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1364
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Argument_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Base_Formal_Param_Holder
   --
--# property-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.abstract_formal_params
                  Property_Result := Base_Subp_Spec_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Binary_Spec =>
                  --# property-call-start SyntheticBinarySpec.abstract_formal_params
                  Property_Result := Synthetic_Binary_Spec_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Unary_Spec =>
                  --# property-call-start SyntheticUnarySpec.abstract_formal_params
                  Property_Result := Synthetic_Unary_Spec_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_List =>
                  --# property-call-start ComponentList.abstract_formal_params
                  Property_Result := Component_List_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Unknown_Discriminant_Part =>
                  --# property-call-start DiscriminantPart.abstract_formal_params
                  Property_Result := Discriminant_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Known_Discriminant_Part =>
                  --# property-call-start KnownDiscriminantPart.abstract_formal_params
                  Property_Result := Known_Discriminant_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Completion_Formal_Params =>
                  --# property-call-start EntryCompletionFormalParams.abstract_formal_params
                  Property_Result := Entry_Completion_Formal_Params_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Part =>
                  --# property-call-start GenericFormalPart.abstract_formal_params
                  Property_Result := Generic_Formal_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.unpacked_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4304
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Unpacked_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_266 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_266 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_266;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1366 '<FieldAccess for unpack_formals at ast.py:4305>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4305
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1365 '<FieldAccess for abstract_formal_params at ast.py:4305>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4305
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1365
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 1366
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_266;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_266;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Unpacked_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4311
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_267 with Inline_Always;
            procedure Finalizer_Scope_2008 with Inline_Always;
            procedure Finalizer_Scope_2010 with Inline_Always;
      Match_0 : Internal_Entity_Component_List;
Match_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_54 : Internal_Entity;
Bare_Item_54 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Fld_4 : Internal_Entity_Defining_Name_Array_Access;
Match_Prefix : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_1 : Internal_Entity_Component_List;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Match_Result : Internal_Entity_Defining_Name_Array_Access;
Let_Result_2 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_267 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_267;
            procedure Finalizer_Scope_2008 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2008;
            procedure Finalizer_Scope_2010 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2010;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1374 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 1373 '<Match at ast.py:4318>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4318
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Formal_Param_Holder (Match_Prefix.Node.Kind) is
         when Ada_Component_List =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Component_List
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind r Match_0
--# expr-start 1371 '<FieldAccess for unpack_formals at ast.py:4320>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4320
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1370 '<FilterMap at ast.py:4320>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4320
   --# expr-start 1367 '<FieldAccess for components at ast.py:4320>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4320
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Match_0.Node.Component_List_F_Components, Info => Match_0.Info);
--# expr-done 1367
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_54 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_54 := Untyped_Item_54; 
Is_Null := Bare_Item_54 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_54); 
   If_Result := As_Entity;
end if;
                  Item_54 := If_Result; 
   --# scope-start
               --# bind e Item_54
         --# expr-start 1368 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_54.Node /= null 
and then Item_54.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1368
         if Is_A then
      --# expr-start 1369 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_54; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1369
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1370
--# property-call-start AdaNode.unpack_formals
Fld_3 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Map_Result);
--# end
--# expr-done 1371
Let_Result := Fld_3; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2008;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Entry_Spec .. Ada_Synthetic_Unary_Spec | Ada_Known_Discriminant_Part .. Ada_Generic_Formal_Part =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 1372 '<FieldAccess for unpacked_formal_params at ast.py:4321>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4321
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1372
Let_Result_1 := Fld_4; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2010;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 1373
Let_Result_2 := Match_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 1374
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_267;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_267;
                     Finalizer_Scope_2008;
                     Finalizer_Scope_2010;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.match_param_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4324
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Param_List
  (Node : Bare_Base_Formal_Param_Holder
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   --# bind env Env
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_268 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_268 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_268;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1377 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1376 '<FieldAccess for match_formals at ast.py:4327>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4327
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1375 '<FieldAccess for abstract_formal_params at ast.py:4328>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4328
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1375
--# property-call-start AdaNode.match_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Params, Is_Dottable_Subp => Is_Dottable_Subp);
--# end
--# expr-done 1376
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1377
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_268;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_268;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Match_Param_List;
--# end
--# property-start BaseFormalParamHolder.nb_min_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4331
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Nb_Min_Params
  (Node : Bare_Base_Formal_Param_Holder
  )
   return Integer
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   Property_Result : Integer;
            procedure Finalizer_Scope_269 with Inline_Always;
      As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Len : Integer;
            procedure Finalizer_Scope_269 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_269;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1383 '<length at ast.py:4334>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4334
--# expr-start 1382 '<Filter at ast.py:4332>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4332
   --# expr-start 1379 '<FieldAccess for unpacked_formal_params at ast.py:4332>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4332
--# expr-start 1378 '<as_bare_entity at ast.py:4332>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4332
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1378
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1379
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_55 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_55
         --# expr-start 1381 '<FieldAccess for is_mandatory at ast.py:4333>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4333
--# expr-start 1380 '<FieldAccess for formal_decl at ast.py:4333>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4333
      if Item_55.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_55.Node, E_Info => Item_55.Info);
--# end
--# expr-done 1380
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_1.Node);
--# end
--# expr-done 1381
         if Fld_2 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_55;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1382
Len := Length (Map_Result); 
--# expr-done 1383
         Property_Result := Len;
   --# end
      Finalizer_Scope_269;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_269;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Nb_Min_Params;
--# end
--# property-start BaseFormalParamHolder.nb_max_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4341
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Nb_Max_Params
  (Node : Bare_Base_Formal_Param_Holder
  )
   return Integer
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   Property_Result : Integer;
            procedure Finalizer_Scope_270 with Inline_Always;
      As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Defining_Name_Array_Access;
Len : Integer;
            procedure Finalizer_Scope_270 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_270;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1386 '<length at ast.py:4342>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4342
--# expr-start 1385 '<FieldAccess for unpacked_formal_params at ast.py:4342>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4342
--# expr-start 1384 '<as_bare_entity at ast.py:4342>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4342
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1384
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1385
Len := Length (Fld); 
--# expr-done 1386
         Property_Result := Len;
   --# end
      Finalizer_Scope_270;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_270;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Nb_Max_Params;
--# end
--# property-start BaseFormalParamHolder.paramless /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4349
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Paramless
  (Node : Bare_Base_Formal_Param_Holder
      ; Dottable_Subp : Boolean
      ; Can_Be : Boolean
         := True
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   --# bind dottable_subp Dottable_Subp
   --# bind can_be Can_Be
   Property_Result : Boolean;
      Block_Var : Integer;
Fld : Integer;
Fld_1 : Integer;
If_Result : Integer;
Is_Equal : Boolean;
If_Result_1 : Boolean;
Is_Equal_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1392 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1389 '<If at ast.py:4356>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4356
if Can_Be then
   --# expr-start 1387 '<FieldAccess for nb_min_params at ast.py:4356>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4356
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Self);
--# end
--# expr-done 1387
   If_Result := Fld;
else
   --# expr-start 1388 '<FieldAccess for nb_max_params at ast.py:4356>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4356
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Self);
--# end
--# expr-done 1388
   If_Result := Fld_1;
end if;
--# expr-done 1389
Block_Var := If_Result; 
--# bind nb_params Block_Var
if Dottable_Subp then
   --# expr-start 1390 '<Eq at ast.py:4358>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4358
Is_Equal := Block_Var = 1; 
--# expr-done 1390
   If_Result_1 := Is_Equal;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 1391 '<Eq at ast.py:4359>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4359
Is_Equal_1 := Block_Var = 0; 
--# expr-done 1391
   If_Result_2 := Is_Equal_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 1392
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Paramless;
--# end
--# property-start BaseFormalParamHolder.is_matching_param_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4362
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Is_Matching_Param_List
  (Node : Bare_Base_Formal_Param_Holder
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
      ; Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   --# bind env Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_272 with Inline_Always;
      Block_Var : Internal_Entity_Base_Formal_Param_Holder;
Block_Var_1 : Internal_Param_Match_Array_Access;
Block_Var_2 : Integer;
Block_Var_3 : Integer;
As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Param_Match_Array_Access;
Fld_1 : Integer;
Arith_Result : Integer;
Fld_2 : Integer;
If_Result_1 : Integer;
Fld_3 : Integer;
Arith_Result_1 : Integer;
Fld_4 : Integer;
As_Entity_1 : Internal_Entity_Base_Formal_Param_Holder;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl;
Fld_7 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Integer;
Fld_8 : Bare_Assoc_List;
Len : Integer;
Comp_Result : Boolean;
Fld_9 : Boolean;
Quantifier_Result : Boolean;
If_Result_5 : Boolean;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Base_Formal_Param_Decl;
Fld_12 : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Len_1 : Integer;
Is_Equal : Boolean;
If_Result_6 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_272 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_272;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1417 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1393 '<as_bare_entity at ast.py:4369>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4369
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1393
Block_Var := If_Result; 
--# bind bare Block_Var
--# expr-start 1394 '<FieldAccess for match_param_list at ast.py:4370>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4370
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.match_param_list
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Block_Var.Node, Params => Params, Is_Dottable_Subp => Is_Dottable_Subp, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 1394
Block_Var_1 := Fld; Inc_Ref (Block_Var_1);
--# bind match_list Block_Var_1
--# expr-start 1398 '<If at ast.py:4379>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4379
if Is_Dottable_Subp then
   --# expr-start 1396 '<Arithmetic - at ast.py:4379>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4379
--# expr-start 1395 '<FieldAccess for nb_max_params at ast.py:4379>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4379
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Block_Var.Node);
--# end
--# expr-done 1395
Arith_Result := (Fld_1 - 1); 
--# expr-done 1396
   If_Result_1 := Arith_Result;
else
   --# expr-start 1397 '<FieldAccess for nb_max_params at ast.py:4379>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4379
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Block_Var.Node);
--# end
--# expr-done 1397
   If_Result_1 := Fld_2;
end if;
--# expr-done 1398
Block_Var_2 := If_Result_1; 
--# bind nb_max_params Block_Var_2
--# expr-start 1407 '<If at ast.py:4382>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4382
if Is_Dottable_Subp then
   --# expr-start 1403 '<FieldAccess for is_mandatory at ast.py:4390>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4390
--# expr-start 1402 '<FieldAccess for formal_decl at ast.py:4390>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4390
--# expr-start 1401 '<at at ast.py:4389>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4389
--# expr-start 1400 '<FieldAccess for unpacked_formal_params at ast.py:4389>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4389
--# expr-start 1399 '<as_bare_entity at ast.py:4389>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4389
Is_Null_1 := Self = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Base_Formal_Param_Holder;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Self); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 1399
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 1400
Get_Result := Get (Self, Fld_5, 0, True); 
--# expr-done 1401
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 1402
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_6.Node);
--# end
--# expr-done 1403
   If_Result_3 := Fld_7;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 1405 '<Arithmetic - at ast.py:4392>' Arith_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4392
--# expr-start 1404 '<FieldAccess for nb_min_params at ast.py:4392>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4392
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Block_Var.Node);
--# end
--# expr-done 1404
Arith_Result_1 := (Fld_3 - 1); 
--# expr-done 1405
   If_Result_4 := Arith_Result_1;
else
   --# expr-start 1406 '<FieldAccess for nb_min_params at ast.py:4393>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4393
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Block_Var.Node);
--# end
--# expr-done 1406
   If_Result_4 := Fld_4;
end if;
--# expr-done 1407
Block_Var_3 := If_Result_4; 
--# bind nb_min_params Block_Var_3
--# expr-start 1408 '<length at ast.py:4398>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4398
Fld_8 := Params.Node; 
Len := Length (Fld_8); 
--# expr-done 1408
Comp_Result := Len <= Block_Var_2; 
if Comp_Result then
   --# expr-start 1410 '<AllQuantifier at ast.py:4399>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4399
      Quantifier_Result := True;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Block_Var_1;
      begin
         for Item_56 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_56
      --# expr-start 1409 '<FieldAccess for has_matched at ast.py:4399>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4399
Fld_9 := Item_56.Has_Matched;
--# expr-done 1409
      Quantifier_Result := Fld_9;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
--# expr-done 1410
   If_Result_5 := Quantifier_Result;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   --# expr-start 1416 '<Eq at ast.py:4400>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4400
--# expr-start 1415 '<length at ast.py:4402>' Len_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4402
--# expr-start 1414 '<Filter at ast.py:4400>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4400
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Block_Var_1;
      begin
         for Item_57 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_57
         --# expr-start 1413 '<FieldAccess for is_mandatory at ast.py:4401>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4401
--# expr-start 1412 '<FieldAccess for formal_decl at ast.py:4401>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4401
--# expr-start 1411 '<FieldAccess for formal at ast.py:4401>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4401
Fld_10 := Item_57.Formal;
--# expr-done 1411
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_11 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 1412
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_11.Node);
--# end
--# expr-done 1413
         if Fld_12 then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_57;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1414
Len_1 := Length (Map_Result); 
--# expr-done 1415
Is_Equal := Len_1 = Block_Var_3; 
--# expr-done 1416
   If_Result_6 := Is_Equal;
else
   If_Result_6 := False;
end if;
Let_Result := If_Result_6; 
--# end
--# expr-done 1417
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_272;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_272;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Is_Matching_Param_List;
--# end
--# property-start BaseFormalParamHolder.param_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4405
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Param_Types
  (Node : Bare_Base_Formal_Param_Holder
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_273 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_273 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_273;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1423 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1422 '<Map at ast.py:4411>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4411
   --# expr-start 1418 '<FieldAccess for unpacked_formal_params at ast.py:4411>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4411
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1418
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_58 of
               Collection.Items
         loop
   --# scope-start
               --# bind fp Item_58
      --# expr-start 1421 '<FieldAccess for real_designated_type at ast.py:4413>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4413
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1420 '<FieldAccess for type_expression at ast.py:4413>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4413
--# expr-start 1419 '<FieldAccess for formal_decl at ast.py:4413>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4413
      if Item_58.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_58.Node, E_Info => Item_58.Info);
--# end
--# expr-done 1419
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 1420
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Fld_2, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1421
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1422
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1423
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_273;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_273;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Param_Types;
--# end
--# property-start BaseFormalParamHolder.param_modes /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4416
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Param_Modes
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Mode_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Mode_Array_Access;
            procedure Finalizer_Scope_274 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result : Internal_Entity_Param_Spec;
Fld_2 : Internal_Entity_Mode;
Map_Result : Internal_Entity_Mode_Array_Access;
Let_Result : Internal_Entity_Mode_Array_Access;
            procedure Finalizer_Scope_274 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_274;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1429 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1428 '<Map at ast.py:4421>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4421
   --# expr-start 1424 '<FieldAccess for unpacked_formal_params at ast.py:4421>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4421
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1424
      declare
         Map_Result_Vec : Internal_Entity_Mode_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_59 of
               Collection.Items
         loop
   --# scope-start
               --# bind fp Item_59
      --# expr-start 1427 '<FieldAccess for mode at ast.py:4422>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4422
--# expr-start 1426 '<Cast to ParamSpec at ast.py:4422>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4422
--# expr-start 1425 '<FieldAccess for formal_decl at ast.py:4422>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4422
      if Item_59.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_59.Node, E_Info => Item_59.Info);
--# end
--# expr-done 1425
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Param_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Param_Spec;
   end if;
--# expr-done 1426
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Mode (Node => Cast_Result.Node.Param_Spec_F_Mode, Info => Cast_Result.Info);
--# expr-done 1427
         declare
            Item_To_Append : constant Internal_Entity_Mode := Fld_2;
         begin
            Internal_Entity_Mode_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Mode_Array
           (Items_Count => Natural (Internal_Entity_Mode_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Mode_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Mode_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Mode_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1428
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1429
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_274;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_274;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Param_Modes;
--# end
--# property-start BaseFormalParamHolder.shed_subp_rebindings /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4425
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Shed_Subp_Rebindings
  (Node : Bare_Base_Formal_Param_Holder
      ; R : Env_Rebindings
  )
   return Env_Rebindings
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   --# bind r R
   Property_Result : Env_Rebindings;
            procedure Finalizer_Scope_275 with Inline_Always;
            procedure Finalizer_Scope_2016 with Inline_Always;
            procedure Finalizer_Scope_2017 with Inline_Always;
      Var_Expr : Lexical_Env := Empty_Env;
Var_Expr_1 : Env_Rebindings;
Fld : Env_Rebindings;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Scope_Result_1 : Bare_Ada_Node;
Result_Var_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Env_Rebindings;
Let_Result : Env_Rebindings;
            procedure Finalizer_Scope_275 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_275;
            procedure Finalizer_Scope_2016 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2016;
            procedure Finalizer_Scope_2017 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_2017;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1436 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1435 '<If at ast.py:4431>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4431
--# expr-start 1432 '<IsA GenericSubpInstantiation at ast.py:4432>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4432
Var_Expr_1 := R;
if Var_Expr_1 /= null then
   --# scope-start
--# expr-start 1430 '<FieldAccess for new_env at ast.py:4432>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4432
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_1.New_Env;
Inc_Ref (Fld_2);
--# expr-done 1430
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2016;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Var_Expr := Result_Var;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
--# expr-start 1431 '<env_node at ast.py:4432>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4432
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 1431
Scope_Result_1 := Env_Node;
   --# end
      Finalizer_Scope_2017;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Is_A := Result_Var_1 /= null 
and then Result_Var_1.Kind in Ada_Generic_Subp_Instantiation_Range; 
--# expr-done 1432
if Is_A then
   --# expr-start 1434 '<FieldAccess for shed_subp_rebindings at ast.py:4433>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4433
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1433 '<FieldAccess for get_parent at ast.py:4433>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4433
      if R = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := R.Parent;
--# expr-done 1433
--# property-call-start BaseFormalParamHolder.shed_subp_rebindings
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Shed_Subp_Rebindings (Node => Self, R => Fld);
--# end
--# expr-done 1434
   If_Result := Fld_1;
else
   If_Result := R;
end if;
--# expr-done 1435
Let_Result := If_Result; 
--# end
--# expr-done 1436
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_275;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_275;
                     Finalizer_Scope_2016;
                     Finalizer_Scope_2017;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Shed_Subp_Rebindings;
--# end
--# property-start BaseFormalParamHolder.real_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4437
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Real_Type
  (Node : Bare_Base_Formal_Param_Holder
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Bare_Ada_Node;
Var_Expr_3 : Internal_Entity_Anonymous_Type_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Internal_Entity_Info;
Fld_6 : Boolean;
Fld_7 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_8 : Bare_Base_Type_Decl;
Is_Null : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Bare_Base_Type_Decl;
Scope_Result : Bare_Base_Type_Decl;
Result_Var : Bare_Base_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Bare_Base_Type_Decl;
Scope_Result_1 : Bare_Base_Type_Decl;
Result_Var_1 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Is_Equal : Boolean;
Fld_13 : Internal_Entity_Info;
Fld_14 : Internal_Metadata;
Fld_15 : Bare_Ada_Node;
Scope_Result_2 : Bare_Ada_Node;
Fld_16 : Bare_Base_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Result_Var_2 : Bare_Ada_Node;
Fld_17 : Internal_Entity_Info;
Fld_18 : Env_Rebindings;
Fld_19 : Env_Rebindings;
Fld_20 : Internal_Entity_Info;
Fld_21 : Boolean;
Fld_22 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Anonymous_Type_Decl;
Fld_23 : Internal_Entity_Type_Def;
Is_A : Boolean;
Not_Val : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_24 : Internal_Entity_Base_Type_Decl;
Fld_25 : Internal_Entity_Base_Type_Decl;
Fld_26 : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_5 : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1474 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1445 '<Cast to BaseTypeDecl at ast.py:4450>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4450
--# expr-start 1444 '<FieldAccess for entity_no_md at ast.py:4446>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4446
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1439 '<FieldAccess for primitive at ast.py:4447>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4447
--# expr-start 1438 '<FieldAccess for md at ast.py:4447>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4447
--# expr-start 1437 '<FieldAccess for info at ast.py:4447>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4447
Fld := Ent.Info;
--# expr-done 1437
Fld_1 := Fld.Md;
--# expr-done 1438
Fld_2 := Fld_1.Primitive;
--# expr-done 1439
--# expr-start 1441 '<FieldAccess for rebindings at ast.py:4448>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4448
--# expr-start 1440 '<FieldAccess for info at ast.py:4448>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4448
Fld_3 := Ent.Info;
--# expr-done 1440
Fld_4 := Fld_3.Rebindings;
--# expr-done 1441
--# expr-start 1443 '<FieldAccess for from_rebound at ast.py:4449>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4449
--# expr-start 1442 '<FieldAccess for info at ast.py:4449>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4449
Fld_5 := Ent.Info;
--# expr-done 1442
Fld_6 := Fld_5.From_Rebound;
--# expr-done 1443
--# property-call-start AdaNode.entity_no_md
Fld_7 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Fld_2, Rebindings => Fld_4, From_Rebound => Fld_6);
--# end
--# expr-done 1444
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1445
Block_Var := Cast_Result; 
--# bind prim_type Block_Var
--# expr-start 1473 '<Cond at ast.py:4452>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4452
--# expr-start 1447 '<IsNull at ast.py:4453>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4453
--# expr-start 1446 '<FieldAccess for node at ast.py:4453>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4453
Fld_8 := Block_Var.Node;
--# expr-done 1446
Is_Null := Fld_8 = null; 
--# expr-done 1447
if Is_Null then
   If_Result_3 := Typ;
else
--# expr-start 1452 '<Eq at ast.py:4456>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4456
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1449 '<FieldAccess for node at ast.py:4456>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4456
--# expr-start 1448 '<FieldAccess for canonical_type at ast.py:4456>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4456
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1448
Fld_10 := Fld_9.Node;
--# expr-done 1449
Scope_Result := Fld_10;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
      Cast_Result_1 := Result_Var;
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1451 '<FieldAccess for node at ast.py:4456>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4456
--# expr-start 1450 '<FieldAccess for canonical_type at ast.py:4456>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4456
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1450
Fld_12 := Fld_11.Node;
--# expr-done 1451
Scope_Result_1 := Fld_12;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Is_Equal := Cast_Result_1 = Result_Var_1; 
--# expr-done 1452
if Is_Equal then
   --# expr-start 1463 '<Cast to BaseTypeDecl at ast.py:4467>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4467
--# expr-start 1462 '<FieldAccess for entity_no_md at ast.py:4457>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4457
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1455 '<FieldAccess for primitive_real_type at ast.py:4458>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4458
--# expr-start 1454 '<FieldAccess for md at ast.py:4458>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4458
--# expr-start 1453 '<FieldAccess for info at ast.py:4458>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4458
Fld_13 := Ent.Info;
--# expr-done 1453
Fld_14 := Fld_13.Md;
--# expr-done 1454
Fld_15 := Fld_14.Primitive_Real_Type;
--# expr-done 1455
Var_Expr_2 := Fld_15;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_2 := Var_Expr_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
--# expr-start 1456 '<FieldAccess for node at ast.py:4458>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4458
Fld_16 := Typ.Node;
--# expr-done 1456
      Cast_Result_2 := Fld_16;
   Result_Var_2 := Cast_Result_2;
end if;
--# expr-start 1459 '<FieldAccess for shed_subp_rebindings at ast.py:4465>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4465
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1458 '<FieldAccess for rebindings at ast.py:4465>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4465
--# expr-start 1457 '<FieldAccess for info at ast.py:4465>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4465
Fld_17 := Ent.Info;
--# expr-done 1457
Fld_18 := Fld_17.Rebindings;
--# expr-done 1458
--# property-call-start BaseFormalParamHolder.shed_subp_rebindings
Fld_19 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Shed_Subp_Rebindings (Node => Self, R => Fld_18);
--# end
--# expr-done 1459
--# expr-start 1461 '<FieldAccess for from_rebound at ast.py:4466>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4466
--# expr-start 1460 '<FieldAccess for info at ast.py:4466>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4466
Fld_20 := Ent.Info;
--# expr-done 1460
Fld_21 := Fld_20.From_Rebound;
--# expr-done 1461
--# property-call-start AdaNode.entity_no_md
Fld_22 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Result_Var_2, Rebindings => Fld_19, From_Rebound => Fld_21);
--# end
--# expr-done 1462
   if Fld_22.Node = null
      or else Fld_22.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_22.Node,
         Info => Fld_22.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1463
   If_Result_2 := Cast_Result_3;
else
--# expr-start 1464 '<Cast to AnonymousTypeDecl at ast.py:4472>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4472
Cast_Expr := Typ; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Anonymous_Type_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_4 := No_Entity_Anonymous_Type_Decl;
   end if;
--# expr-done 1464
Var_Expr_3 := Cast_Result_4;
if Var_Expr_3 /= No_Entity_Anonymous_Type_Decl then
   --# scope-start
   --# bind td Var_Expr_3
--# expr-start 1467 '<Not at ast.py:4473>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4473
--# expr-start 1466 '<IsA AccessToSubpDef at ast.py:4473>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4473
--# expr-start 1465 '<FieldAccess for type_def at ast.py:4473>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4473
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Type_Def (Node => Var_Expr_3.Node.Type_Decl_F_Type_Def, Info => Var_Expr_3.Info);
--# expr-done 1465
Is_A := Fld_23.Node /= null 
and then Fld_23.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 1466
Not_Val := not (Is_A); 
--# expr-done 1467
Scope_Result_3 := Not_Val;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
--# expr-start 1468 '<FieldAccess for accessed_type at ast.py:4475>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4475
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_24 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 1468
Var_Expr_4 := Fld_24;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr_4
--# expr-start 1469 '<FieldAccess for real_type at ast.py:4476>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4476
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_25 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Var_Expr_4, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1469
Var_Expr_5 := Fld_25;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind rat Var_Expr_5
--# expr-start 1472 '<If at ast.py:4477>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4477
--# expr-start 1470 '<Eq at ast.py:4478>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4478
Cast_Expr_1 := Var_Expr_4; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr_5; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_5, Cast_Result_6); 
--# expr-done 1470
if Is_Equiv then
   If_Result := Typ;
else
   --# expr-start 1471 '<FieldAccess for anonymous_access_type at ast.py:4480>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4480
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_26 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 1471
   If_Result := Fld_26;
end if;
--# expr-done 1472
Scope_Result_4 := If_Result;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_5 := Result_Var_4;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := Result_Var_5;
else
   If_Result_1 := Typ;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 1473
Let_Result := If_Result_3; 
--# end
--# expr-done 1474
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Real_Type;
--# end
--# property-start BaseFormalParamHolder.real_designated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4488
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Real_Designated_Type
  (Node : Bare_Base_Formal_Param_Holder
      ; Typ : Internal_Entity_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Internal_Metadata;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Rt : Internal_Entity_Base_Type_Decl;
Base_Rebindings : Env_Rebindings;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_6 : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_7 : Env_Rebindings;
Scope_Result_1 : Env_Rebindings;
Result_Var_1 : Env_Rebindings;
Fld_8 : Bare_Type_Expr;
Fld_9 : Internal_Entity_Info;
Fld_10 : Env_Rebindings;
Rebinding : Env_Rebindings;
Fld_11 : Internal_Entity_Info;
Fld_12 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Type_Expr;
Fld_13 : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Fld_14 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1503 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 1476 '<FieldAccess for md at ast.py:4551>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4551
--# expr-start 1475 '<FieldAccess for info at ast.py:4551>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4551
Fld := Ent.Info;
--# expr-done 1475
Fld_1 := Fld.Md;
--# expr-done 1476
Block_Var := Fld_1; 
--# bind md Block_Var
--# expr-start 1479 '<Cast to BaseTypeDecl at ast.py:4553>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4553
--# expr-start 1477 '<FieldAccess for designated_type at ast.py:4553>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4553
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 1477
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1478 '<FieldAccess for without_md at ast.py:4553>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4553
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.without_md
Fld_3 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1478
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1479
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 1502 '<If at ast.py:4554>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4554
--# expr-start 1480 '<FieldAccess for dottable_subp at ast.py:4563>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4563
Fld_14 := Block_Var.Dottable_Subp;
--# expr-done 1480
if Fld_14 then
   If_Result_3 := Var_Expr;
else
   --# expr-start 1501 '<Let at ast.py:4565>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4565
--# scope-start
--# expr-start 1481 '<FieldAccess for real_type at ast.py:4567>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4567
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Var_Expr, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1481
Rt := Fld_4; 
--# bind rt Rt
--# expr-start 1484 '<as_bare_entity at ast.py:4569>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4569
--# expr-start 1483 '<Cast to BaseTypeDecl at ast.py:4568>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4568
--# expr-start 1482 '<FieldAccess for primitive_real_type at ast.py:4568>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4568
Fld_5 := Block_Var.Primitive_Real_Type;
--# expr-done 1482
   if Fld_5 = null
      or else Fld_5.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Fld_5;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1483
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 1484
Var_Expr_2 := If_Result;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1487 '<FieldAccess for find_base_type_rebindings at ast.py:4569>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4569
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1486 '<Cast to BaseTypeDecl at ast.py:4570>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4570
--# expr-start 1485 '<FieldAccess for primitive at ast.py:4570>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4570
Fld_6 := Block_Var.Primitive;
--# expr-done 1485
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Fld_6;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 1486
--# property-call-start BaseTypeDecl.find_base_type_rebindings
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings (Node => Var_Expr_2.Node, Target => Cast_Result_2, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1487
Scope_Result_1 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := null;
end if;
Base_Rebindings := Result_Var_1; 
--# bind base_rebindings Base_Rebindings
--# expr-start 1500 '<If at ast.py:4571>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4571
--# expr-start 1488 '<Eq at ast.py:4572>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4572
Cast_Expr := Var_Expr; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Rt; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_3, Cast_Result_4); 
--# expr-done 1488
if Is_Equiv then
   --# expr-start 1499 '<If at ast.py:4573>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4573
--# expr-start 1489 '<IsNull at ast.py:4574>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4574
Is_Equal := Base_Rebindings = null; 
--# expr-done 1489
if Is_Equal then
   If_Result_1 := Var_Expr;
else
   --# expr-start 1498 '<FieldAccess for designated_type at ???>' Fld_13 None
--# expr-start 1497 '<New TypeExpr.entity at ???>' New_Struct_1 None
--# expr-start 1495 '<New EntityInfo at ast.py:4578>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4578
--# expr-start 1491 '<FieldAccess for from_rebound at ast.py:4582>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4582
--# expr-start 1490 '<FieldAccess for info at ast.py:4582>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4582
Fld_11 := Typ.Info;
--# expr-done 1490
Fld_12 := Fld_11.From_Rebound;
--# expr-done 1491
--# expr-start 1494 '<concat_rebindings at ast.py:4581>' Rebinding /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4581
--# expr-start 1493 '<FieldAccess for rebindings at ast.py:4580>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4580
--# expr-start 1492 '<FieldAccess for info at ast.py:4580>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4580
Fld_9 := Typ.Info;
--# expr-done 1492
Fld_10 := Fld_9.Rebindings;
--# expr-done 1493
Rebinding := AST_Envs.Combine (Fld_10, Base_Rebindings); 
--# expr-done 1494
New_Struct := (From_Rebound => Fld_12, Md => No_Metadata, Rebindings => Rebinding); 
--# expr-done 1495
--# expr-start 1496 '<FieldAccess for node at ast.py:4577>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4577
Fld_8 := Typ.Node;
--# expr-done 1496
New_Struct_1 := (Info => New_Struct, Node => Fld_8); 
--# expr-done 1497
      if New_Struct_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => New_Struct_1.Node, Origin => Origin, E_Info => New_Struct_1.Info);
--# end
--# expr-done 1498
   If_Result_1 := Fld_13;
end if;
--# expr-done 1499
   If_Result_2 := If_Result_1;
else
   If_Result_2 := Rt;
end if;
--# expr-done 1500
Let_Result := If_Result_2; 
--# end
--# expr-done 1501
   If_Result_3 := Let_Result;
end if;
--# expr-done 1502
Scope_Result_2 := If_Result_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Let_Result_1 := Result_Var_2; 
--# end
--# expr-done 1503
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Real_Designated_Type;
--# end
--# property-start BaseFormalParamHolder.call_argument_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4592
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Call_Argument_Equation
  (Node : Bare_Base_Formal_Param_Holder
      ; Param : Internal_Entity_Base_Formal_Param_Decl
      ; Arg : Internal_Entity_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind param Param
   --# bind arg Arg
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_278 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_278 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_278;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1510 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1505 '<FieldAccess for real_designated_type at ast.py:4601>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4601
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1504 '<FieldAccess for type_expression at ast.py:4601>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4601
      if Param.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Param.Node, E_Info => Param.Info);
--# end
--# expr-done 1504
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Fld, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1505
Block_Var := Fld_1; 
--# bind param_type Block_Var
--# expr-start 1509 '<And at ast.py:4602>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4602
--# expr-start 1507 '<Bind at ast.py:4603>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4603
--# expr-start 1506 '<FieldAccess for expected_type_var at ast.py:4603>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4603
      if Arg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Arg.Node);
--# expr-done 1506
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4603") else null)); 
--# expr-done 1507
--# expr-start 1508 '<FieldAccess for matches_expected_formal_type at ast.py:4604>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4604
      if Arg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_3 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Arg.Node, Origin => Origin);
--# end
--# expr-done 1508
And_Pred := Create_And (Bind_Result, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4602") else null)); 
--# expr-done 1509
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1510
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_278;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_278;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Call_Argument_Equation;
--# end
--# property-start BaseFormalParamHolder.match_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4607
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
      ; Ignore_First_Param : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   --# bind ignore_first_param Ignore_First_Param
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_279 with Inline_Always;
            procedure Finalizer_Scope_2027 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Block_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Block_Var_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Block_Var_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Var_Expr : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Defining_Name_Array_Access;
Index_61 : Integer;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
If_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Len : Integer;
Len_1 : Integer;
Is_Equal_1 : Boolean;
Not_Val_1 : Boolean;
Fld_4 : Internal_Entity_Name;
Get_Result : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Name;
Fld_6 : Bare_Name;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Get_Result_1 : Internal_Entity_Base_Type_Decl;
Get_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_2 : Boolean;
Index_63 : Integer;
Quantifier_Result : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_279 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Block_Var_3);
            end Finalizer_Scope_279;
            procedure Finalizer_Scope_2027 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2027;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1532 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1511 '<FieldAccess for unpacked_formal_params at ast.py:4616>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4616
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1511
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Defining_Name_Array_Type then
   --# scope-start
   --# bind params Var_Expr
--# expr-start 1515 '<If at ast.py:4617>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4617
if Ignore_First_Param then
   --# expr-start 1514 '<Filter at ast.py:4619>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4619
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
         Index_61 := 0;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Var_Expr;
      begin
         for Item_60 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_60
               --# bind i Index_61
         --# expr-start 1513 '<Not at ast.py:4619>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4619
--# expr-start 1512 '<Eq at ast.py:4619>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4619
Is_Equal := Index_61 = 0; 
--# expr-done 1512
Not_Val := not (Is_Equal); 
--# expr-done 1513
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_60;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
               Index_61 := Index_61 + 1;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1514
   If_Result := Map_Result;
else
   If_Result := Var_Expr;
end if;
      Inc_Ref (If_Result);
--# expr-done 1515
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2027;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
Block_Var := Result_Var; Inc_Ref (Block_Var);
--# bind self_params Block_Var
--# expr-start 1516 '<FieldAccess for unpacked_formal_params at ast.py:4623>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4623
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1516
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind other_params Block_Var_1
--# expr-start 1517 '<FieldAccess for param_types at ast.py:4625>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4625
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1517
Block_Var_2 := Fld_2; Inc_Ref (Block_Var_2);
--# bind self_types Block_Var_2
--# expr-start 1518 '<FieldAccess for param_types at ast.py:4626>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4626
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 1518
Block_Var_3 := Fld_3; Inc_Ref (Block_Var_3);
--# bind other_types Block_Var_3
--# expr-start 1521 '<Eq at ast.py:4628>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4628
--# expr-start 1519 '<length at ast.py:4628>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4628
Len := Length (Block_Var); 
--# expr-done 1519
--# expr-start 1520 '<length at ast.py:4628>' Len_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4628
Len_1 := Length (Block_Var_1); 
--# expr-done 1520
Is_Equal_1 := Len = Len_1; 
--# expr-done 1521
if Is_Equal_1 then
   --# expr-start 1531 '<AllQuantifier at ast.py:4629>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4629
      Quantifier_Result := True;
         Index_63 := 0;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Block_Var;
      begin
         for Item_62 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_62
               --# bind i Index_63
--# expr-start 1522 '<Not at ast.py:4630>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4630
Not_Val_1 := not (Match_Names); 
--# expr-done 1522
if Not_Val_1 then
   If_Result_1 := True;
else
   --# expr-start 1527 '<FieldAccess for matches at ast.py:4631>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4631
--# expr-start 1523 '<FieldAccess for name at ast.py:4631>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4631
      if Item_62.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Item_62.Node.Defining_Name_F_Name, Info => Item_62.Info);
--# expr-done 1523
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1526 '<FieldAccess for node at ast.py:4631>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4631
--# expr-start 1525 '<FieldAccess for name at ast.py:4631>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4631
--# expr-start 1524 '<at at ast.py:4631>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4631
Get_Result := Get (Self, Block_Var_1, Index_63, True); 
--# expr-done 1524
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Get_Result.Node.Defining_Name_F_Name, Info => Get_Result.Info);
--# expr-done 1525
Fld_6 := Fld_5.Node;
--# expr-done 1526
--# property-call-start Name.matches
Fld_7 := Libadalang.Implementation.Name_P_Matches (Node => Fld_4.Node, N => Fld_6);
--# end
--# expr-done 1527
   If_Result_1 := Fld_7;
end if;
if If_Result_1 then
--# expr-start 1528 '<at at ast.py:4632>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4632
Get_Result_1 := Get (Self, Block_Var_2, Index_63, True); 
--# expr-done 1528
Var_Expr_1 := Get_Result_1;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1530 '<FieldAccess for matching_type at ast.py:4632>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4632
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1529 '<at at ast.py:4632>' Get_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4632
Get_Result_2 := Get (Self, Block_Var_3, Index_63, True); 
--# expr-done 1529
--# property-call-start BaseTypeDecl.matching_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr_1.Node, Expected_Type => Get_Result_2, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1530
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_2 := Result_Var_1;
else
   If_Result_2 := False;
end if;
      Quantifier_Result := If_Result_2;
   --# end
               exit when not Quantifier_Result;
               Index_63 := Index_63 + 1;
         end loop;
      end;
--# expr-done 1531
   If_Result_3 := Quantifier_Result;
else
   If_Result_3 := False;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 1532
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_279;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_279;
                     Finalizer_Scope_2027;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Match_Formal_Params;
--# end
--# property-start '[dispatcher]BaseFormalParamHolder.match_other' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Match_Other
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Component_List .. Ada_Generic_Formal_Part =>
                  --# property-call-start BaseFormalParamHolder.match_other
                  Property_Result := Base_Formal_Param_Holder_P_Match_Other
                    (Self
                        , Other
                        , Match_Names
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Synthetic_Unary_Spec =>
                  --# property-call-start BaseSubpSpec.match_other
                  Property_Result := Base_Subp_Spec_P_Match_Other
                    (Self
                        , Other
                        , Match_Names
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Formal_Param_Holder_P_Match_Other;
--# end
--# property-start BaseFormalParamHolder.match_other /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4636
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Other
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1534 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1533 '<FieldAccess for match_formal_params at ast.py:4645>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4645
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Ent.Node, Other => Other, Match_Names => Match_Names, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1533
Let_Result := Fld; 
--# end
--# expr-done 1534
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Match_Other;
--# end
   --
   --  Primitives for Bare_Base_Subp_Spec
   --
--# property-start '[dispatcher]BaseSubpSpec.name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Name
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.name
                  Property_Result := Entry_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.name
                  Property_Result := Enum_Subp_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.name
                  Property_Result := Subp_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Binary_Spec =>
                  --# property-call-start SyntheticBinarySpec.name
                  Property_Result := Synthetic_Binary_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Unary_Spec =>
                  --# property-call-start SyntheticUnarySpec.name
                  Property_Result := Synthetic_Unary_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Subp_Spec_P_Name;
--# end
--# property-start '[dispatcher]BaseSubpSpec.returns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Returns
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.returns
                  Property_Result := Entry_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.returns
                  Property_Result := Enum_Subp_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.returns
                  Property_Result := Subp_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Binary_Spec =>
                  --# property-call-start SyntheticBinarySpec.returns
                  Property_Result := Synthetic_Binary_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Unary_Spec =>
                  --# property-call-start SyntheticUnarySpec.returns
                  Property_Result := Synthetic_Unary_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Subp_Spec_P_Returns;
--# end
--# property-start '[dispatcher]BaseSubpSpec.params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Synthetic_Binary_Spec .. Ada_Synthetic_Unary_Spec =>
                  --# property-call-start BaseSubpSpec.params
                  Property_Result := Base_Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.params
                  Property_Result := Entry_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.params
                  Property_Result := Enum_Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.params
                  Property_Result := Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Subp_Spec_P_Params;
--# end
--# property-start BaseSubpSpec.abstract_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18068
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Abstract_Formal_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1254 with Inline_Always;
      Fld : Internal_Entity_Param_Spec_Array_Access;
Cast_Expr : Internal_Entity_Param_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1254 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1254;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1537 '<Map at ast.py:18069>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18069
   --# expr-start 1535 '<FieldAccess for params at ast.py:18069>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18069
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1535
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Param_Spec_Array_Access := Fld;
      begin
         for Item_64 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_64
      --# expr-start 1536 '<Cast to BaseFormalParamDecl at ast.py:18069>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18069
Cast_Expr := Item_64; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 1536
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1537
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1254;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1254;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Abstract_Formal_Params;
--# end
--# property-start BaseSubpSpec.match_return_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18072
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Return_Type
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
Fld_3 : Internal_Entity_Type_Expr;
Is_Null_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Type_Expr;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Fld_5 : Internal_Entity_Type_Expr;
Is_Null_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1553 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1538 '<FieldAccess for return_type at ast.py:18079>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18079
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1538
Block_Var := Fld; 
--# bind self_ret Block_Var
--# expr-start 1539 '<FieldAccess for return_type at ast.py:18080>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18080
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 1539
Block_Var_1 := Fld_1; 
--# bind other_ret Block_Var_1
--# expr-start 1541 '<IsNull at ast.py:18082>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18082
--# expr-start 1540 '<FieldAccess for returns at ast.py:18082>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18082
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1540
Is_Null := Fld_2.Node = null; 
--# expr-done 1541
if Is_Null then
   --# expr-start 1543 '<IsNull at ast.py:18082>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18082
--# expr-start 1542 '<FieldAccess for returns at ast.py:18082>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18082
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1542
Is_Null_1 := Fld_3.Node = null; 
--# expr-done 1543
   If_Result := Is_Null_1;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_3 := True;
else
--# expr-start 1546 '<Not at ast.py:18084>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18084
--# expr-start 1545 '<IsNull at ast.py:18084>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18084
--# expr-start 1544 '<FieldAccess for returns at ast.py:18084>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18084
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1544
Is_Null_2 := Fld_4.Node = null; 
--# expr-done 1545
Not_Val := not (Is_Null_2); 
--# expr-done 1546
if Not_Val then
   --# expr-start 1549 '<Not at ast.py:18084>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18084
--# expr-start 1548 '<IsNull at ast.py:18084>' Is_Null_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18084
--# expr-start 1547 '<FieldAccess for returns at ast.py:18084>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18084
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1547
Is_Null_3 := Fld_5.Node = null; 
--# expr-done 1548
Not_Val_1 := not (Is_Null_3); 
--# expr-done 1549
   If_Result_1 := Not_Val_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 1552 '<bind at ast.py:18085>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18085
--# expr-start 1550 '<FieldAccess for origin_node at ast.py:18085>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18085
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1550
Bound_Origin := Fld_6; 
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1551 '<FieldAccess for matching_type at ast.py:18086>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18086
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr.Node, Expected_Type => Block_Var_1, Origin => Bound_Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1551
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 1552
   If_Result_2 := Dyn_Var_Bind_Result;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 1553
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Match_Return_Type;
--# end
--# property-start BaseSubpSpec.match_signature /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18090
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Signature
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Subp_Spec
      ; Match_Name : Boolean
      ; Use_Entity_Info : Boolean
         := True
      ; Ignore_First_Param : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_name Match_Name
   --# bind use_entity_info Use_Entity_Info
   --# bind ignore_first_param Ignore_First_Param
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Subp_Spec;
As_Entity : Internal_Entity_Base_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Spec;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Not_Val : Boolean;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Bare_Defining_Name;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Bare_Defining_Name;
Cast_Result : Bare_Name;
Fld_4 : Boolean;
If_Result_2 : Boolean;
Fld_5 : Boolean;
If_Result_3 : Boolean;
Cast_Expr : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_6 : Boolean;
If_Result_4 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1564 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1555 '<If at ast.py:18114>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18114
if Use_Entity_Info then
   If_Result_1 := Ent;
else
   --# expr-start 1554 '<as_bare_entity at ast.py:18114>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18114
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Spec;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1554
   If_Result_1 := If_Result;
end if;
--# expr-done 1555
Block_Var := If_Result_1; 
--# bind ent Block_Var
--# expr-start 1556 '<Not at ast.py:18117>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18117
Not_Val := not (Match_Name); 
--# expr-done 1556
if Not_Val then
   If_Result_2 := True;
else
   --# expr-start 1561 '<FieldAccess for matches at ast.py:18117>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18117
--# expr-start 1558 '<FieldAccess for node at ast.py:18117>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18117
--# expr-start 1557 '<FieldAccess for name at ast.py:18117>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18117
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1557
Fld_1 := Fld.Node;
--# expr-done 1558
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1560 '<FieldAccess for node at ast.py:18117>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18117
--# expr-start 1559 '<FieldAccess for name at ast.py:18117>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18117
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1559
Fld_3 := Fld_2.Node;
--# expr-done 1560
      Cast_Result := Fld_3;
--# property-call-start Name.matches
Fld_4 := Libadalang.Implementation.Name_P_Matches (Node => Fld_1, N => Cast_Result);
--# end
--# expr-done 1561
   If_Result_2 := Fld_4;
end if;
if If_Result_2 then
   --# expr-start 1562 '<FieldAccess for match_return_type at ast.py:18118>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18118
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_return_type
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Return_Type (Node => Block_Var.Node, Other => Other, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1562
   If_Result_3 := Fld_5;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 1563 '<FieldAccess for match_formal_params at ast.py:18119>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18119
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Other; 
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Block_Var.Node, Other => Cast_Result_1, Match_Names => Match_Name, Ignore_First_Param => Ignore_First_Param, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1563
   If_Result_4 := Fld_6;
else
   If_Result_4 := False;
end if;
Let_Result := If_Result_4; 
--# end
--# expr-done 1564
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Match_Signature;
--# end
--# property-start BaseSubpSpec.match_other /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18122
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Other
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   --# bind origin Origin
   Property_Result : Boolean;
      Cast_Expr : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1567 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1566 '<FieldAccess for match_signature at ast.py:18125>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18125
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1565 '<Cast to BaseSubpSpec at ast.py:18126>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18126
Cast_Expr := Other; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1565
--# property-call-start BaseSubpSpec.match_signature
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Ent.Node, Other => Cast_Result, Match_Name => Match_Names, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1566
Let_Result := Fld; 
--# end
--# expr-done 1567
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Match_Other;
--# end
--# property-start BaseSubpSpec.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18129
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Defining_Env
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1258 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1258 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1258;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1573 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1572 '<If at ast.py:18134>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18134
--# expr-start 1569 '<IsNull at ast.py:18134>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18134
--# expr-start 1568 '<FieldAccess for returns at ast.py:18134>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18134
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1568
Is_Null := Fld_2.Node = null; 
--# expr-done 1569
if Is_Null then
   If_Result := Empty_Env;
else
   --# expr-start 1571 '<FieldAccess for defining_env at ast.py:18135>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18135
--# expr-start 1570 '<FieldAccess for return_type at ast.py:18135>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18135
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1570
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 1571
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 1572
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1573
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1258;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1258;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Defining_Env;
--# end
--# property-start BaseSubpSpec.potential_dottable_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18137
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Potential_Dottable_Type
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1259 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Formal_Param_Decl;
Var_Expr_1 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1259 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1259;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1578 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1575 '<at at ast.py:18143>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18143
--# expr-start 1574 '<FieldAccess for abstract_formal_params at ast.py:18143>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18143
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1574
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 1575
Var_Expr := Get_Result;
if Var_Expr /= No_Entity_Base_Formal_Param_Decl then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1576 '<FieldAccess for type_expression at ast.py:18144>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18144
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1576
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 1577 '<FieldAccess for element_type at ast.py:18144>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18144
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.element_type
Fld_2 := Libadalang.Implementation.Type_Expr_P_Element_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1577
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 1578
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1259;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1259;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Potential_Dottable_Type;
--# end
--# property-start BaseSubpSpec.get_candidate_type_for_primitive /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18147
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Get_Candidate_Type_For_Primitive
  (Node : Bare_Base_Subp_Spec
      ; Type_Expr : Internal_Entity_Type_Expr
      ; Canonicalize : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind type_expr Type_Expr
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1260 with Inline_Always;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Block_Var_4 : Bare_Declarative_Part;
Var_Expr : Internal_Entity;
Var_Expr_1 : Internal_Entity_Generic_Instantiation;
Match_0 : Internal_Entity_Anonymous_Type;
Match_1 : Internal_Entity_Type_Expr;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Boolean;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Incomplete_Type_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Bare_Declarative_Part;
Var_Expr_8 : Bare_Declarative_Part;
Var_Expr_9 : Internal_Entity_Type_Decl;
Var_Expr_10 : Internal_Entity_Base_Type_Decl;
Var_Expr_11 : Bare_Declarative_Part;
Var_Expr_12 : Bare_Base_Package_Decl;
Var_Expr_13 : Bare_Declarative_Part;
Fld : Internal_Entity_Generic_Instantiation_Array_Access;
Get_Result : Internal_Entity_Generic_Instantiation;
Fld_1 : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Fld_2 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Fld_3 : Bare_Ada_Node;
Fld_4 : Bare_Ada_Node;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result : Internal_Entity_Declarative_Part;
Fld_7 : Bare_Declarative_Part;
Fld_8 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
Not_Val : Boolean;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Expr;
Cast_Result_1 : Internal_Entity_Anonymous_Type;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Fld_12 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_13 : Internal_Entity_Base_Type_Decl;
Scope_Result_6 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Incomplete_Type_Decl;
Fld_14 : Internal_Entity_Base_Type_Decl;
Scope_Result_7 : Internal_Entity_Base_Type_Decl;
Result_Var_5 : Internal_Entity_Base_Type_Decl;
Fld_15 : Bare_Base_Type_Decl;
Fld_16 : Bare_Ada_Node;
Fld_17 : Bare_Ada_Node;
Cast_Result_4 : Bare_Declarative_Part;
Scope_Result_8 : Bare_Declarative_Part;
Result_Var_6 : Bare_Declarative_Part;
Fld_18 : Bare_Ada_Node;
Scope_Result_9 : Bare_Ada_Node;
Result_Var_7 : Bare_Ada_Node;
Is_A_1 : Boolean;
Fld_19 : Bare_Ada_Node;
Scope_Result_10 : Bare_Ada_Node;
Result_Var_8 : Bare_Ada_Node;
Is_A_2 : Boolean;
If_Result_3 : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity_Type_Decl;
Fld_20 : Boolean;
Scope_Result_11 : Boolean;
Result_Var_9 : Boolean;
If_Result_4 : Boolean;
Fld_21 : Boolean;
Scope_Result_12 : Boolean;
Result_Var_10 : Boolean;
Not_Val_1 : Boolean;
If_Result_5 : Boolean;
Cast_Expr_2 : Bare_Declarative_Part;
Cast_Result_6 : Bare_Declarative_Part;
Is_Equal : Boolean;
Fld_22 : Bare_Ada_Node;
Scope_Result_13 : Bare_Ada_Node;
Result_Var_11 : Bare_Ada_Node;
Cast_Result_7 : Bare_Base_Package_Decl;
Fld_23 : Bare_Public_Part;
Scope_Result_14 : Bare_Public_Part;
Result_Var_12 : Bare_Public_Part;
Cast_Result_8 : Bare_Declarative_Part;
Is_Equal_1 : Boolean;
If_Result_6 : Boolean;
Scope_Result_15 : Boolean;
Result_Var_13 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1260 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1260;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1623 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 1591 '<FieldAccess for node at ast.py:18166>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18166
--# expr-start 1590 '<Cast to DeclarativePart at ast.py:18166>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18166
--# expr-start 1589 '<If at ast.py:18160>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18160
--# expr-start 1580 '<IsA GenericSubpInternal at ast.py:18161>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18161
--# expr-start 1579 '<parent at ast.py:18161>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18161
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Parent; 
--# expr-done 1579
Is_A := Fld_6 /= null 
and then Fld_6.Kind in Ada_Generic_Subp_Internal_Range; 
--# expr-done 1580
if Is_A then
--# expr-start 1582 '<at at ast.py:18164>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18164
--# expr-start 1581 '<FieldAccess for generic_instantiations at ast.py:18164>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18164
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.generic_instantiations
Fld := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1581
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 1582
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity_Generic_Instantiation then
   --# scope-start
--# expr-start 1583 '<parent at ast.py:18164>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18164
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 1583
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 1584 '<parent at ast.py:18164>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18164
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 1584
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   If_Result_1 := Result_Var_1;
else
   --# expr-start 1588 '<as_entity at ast.py:18165>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18165
--# expr-start 1587 '<parent at ast.py:18165>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18165
--# expr-start 1586 '<parent at ast.py:18165>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18165
--# expr-start 1585 '<parent at ast.py:18165>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18165
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 1585
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 1586
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Fld_4.Parent; 
--# expr-done 1587
Is_Null := Fld_5 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Fld_5); 
   If_Result := As_Entity;
end if;
--# expr-done 1588
   If_Result_1 := If_Result;
end if;
--# expr-done 1589
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result := No_Entity_Declarative_Part;
   end if;
--# expr-done 1590
Fld_7 := Cast_Result.Node;
--# expr-done 1591
Block_Var := Fld_7; 
--# bind decl_scope Block_Var
--# expr-start 1598 '<bind at ast.py:18169>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18169
--# expr-start 1592 '<FieldAccess for origin_node at ast.py:18169>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18169
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_8 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1592
Bound_Origin := Fld_8; 
--# expr-start 1597 '<Match at ast.py:18169>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18169
      if Type_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Type_Expr; 
case Ada_Type_Expr (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Anonymous_Type
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind at Match_0
--# expr-start 1593 '<FieldAccess for element_type at ast.py:18170>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18170
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.element_type
Fld_9 := Libadalang.Implementation.Type_Expr_P_Element_Type (Node => Match_0.Node, Origin => Bound_Origin, E_Info => Match_0.Info);
--# end
--# expr-done 1593
Var_Expr_2 := Fld_9;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind et Var_Expr_2
--# expr-start 1595 '<Not at ast.py:18173>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18173
--# expr-start 1594 '<FieldAccess for is_classwide at ast.py:18173>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18173
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1594
Not_Val := not (Fld_10); 
--# expr-done 1595
Var_Expr_3 := Not_Val;
if Var_Expr_3 /= False then
   --# scope-start
   --# bind _ Var_Expr_3
Scope_Result_2 := Var_Expr_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_3 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_3; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Enum_Lit_Synth_Type_Expr .. Ada_Synthetic_Type_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind other Match_1
--# expr-start 1596 '<FieldAccess for designated_type at ast.py:18177>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18177
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_11 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 1596
Let_Result_1 := Fld_11; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 1597
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 1598
Block_Var_1 := Dyn_Var_Bind_Result; 
--# bind typ Block_Var_1
--# expr-start 1602 '<If at ast.py:18181>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18181
if Canonicalize then
   --# expr-start 1601 '<bind at ast.py:18183>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18183
--# expr-start 1599 '<FieldAccess for origin_node at ast.py:18183>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18183
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_12 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1599
Bound_Origin_1 := Fld_12; 
Var_Expr_4 := Block_Var_1;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1600 '<FieldAccess for canonical_type at ast.py:18183>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18183
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_4.Node, Origin => Bound_Origin_1, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 1600
Scope_Result_6 := Fld_13;
   --# end
   Result_Var_4 := Scope_Result_6;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
Dyn_Var_Bind_Result_1 := Result_Var_4; 
--# expr-done 1601
   If_Result_2 := Dyn_Var_Bind_Result_1;
else
   If_Result_2 := Block_Var_1;
end if;
--# expr-done 1602
Block_Var_2 := If_Result_2; 
--# bind canon_type Block_Var_2
--# expr-start 1603 '<Cast to IncompleteTypeDecl at ast.py:18187>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18187
Cast_Expr := Block_Var_2; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Incomplete_Type_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Incomplete_Type_Decl;
   end if;
--# expr-done 1603
Var_Expr_5 := Cast_Result_3;
if Var_Expr_5 /= No_Entity_Incomplete_Type_Decl then
   --# scope-start
   --# bind i Var_Expr_5
--# expr-start 1604 '<FieldAccess for next_part at ast.py:18188>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18188
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_14 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 1604
Scope_Result_7 := Fld_14;
   --# end
   Result_Var_5 := Scope_Result_7;
else
   Result_Var_5 := Block_Var_2;
end if;
Block_Var_3 := Result_Var_5; 
--# bind final_type Block_Var_3
Var_Expr_6 := Block_Var_3;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_6
--# expr-start 1608 '<Cast to DeclarativePart at ast.py:18193>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18193
--# expr-start 1607 '<parent at ast.py:18193>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18193
--# expr-start 1606 '<parent at ast.py:18193>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18193
--# expr-start 1605 '<FieldAccess for node at ast.py:18193>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18193
Fld_15 := Var_Expr_6.Node;
--# expr-done 1605
      if Fld_15 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Fld_15.Parent; 
--# expr-done 1606
      if Fld_16 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Fld_16.Parent; 
--# expr-done 1607
   if Fld_17 = null
      or else Fld_17.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result_4 := Fld_17;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 1608
Scope_Result_8 := Cast_Result_4;
   --# end
   Result_Var_6 := Scope_Result_8;
else
   Result_Var_6 := No_Bare_Ada_Node;
end if;
Block_Var_4 := Result_Var_6; 
--# bind type_scope Block_Var_4
--# expr-start 1622 '<If at ast.py:18196>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18196
--# expr-start 1610 '<IsA BasePackageDecl at ast.py:18202>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18202
Var_Expr_7 := Block_Var_4;
if Var_Expr_7 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1609 '<parent at ast.py:18202>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18202
      if Var_Expr_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Var_Expr_7.Parent; 
--# expr-done 1609
Scope_Result_9 := Fld_18;
   --# end
   Result_Var_7 := Scope_Result_9;
else
   Result_Var_7 := No_Bare_Ada_Node;
end if;
Is_A_1 := Result_Var_7 /= null 
and then Result_Var_7.Kind in Ada_Base_Package_Decl; 
--# expr-done 1610
if Is_A_1 then
   --# expr-start 1612 '<IsA BasePackageDecl at ast.py:18203>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18203
Var_Expr_8 := Block_Var;
if Var_Expr_8 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1611 '<parent at ast.py:18203>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18203
      if Var_Expr_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Var_Expr_8.Parent; 
--# expr-done 1611
Scope_Result_10 := Fld_19;
   --# end
   Result_Var_8 := Scope_Result_10;
else
   Result_Var_8 := No_Bare_Ada_Node;
end if;
Is_A_2 := Result_Var_8 /= null 
and then Result_Var_8.Kind in Ada_Base_Package_Decl; 
--# expr-done 1612
   If_Result_3 := Is_A_2;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
--# expr-start 1613 '<Cast to TypeDecl at ast.py:18218>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18218
Cast_Expr_1 := Block_Var_3; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_5 := No_Entity_Type_Decl;
   end if;
--# expr-done 1613
Var_Expr_9 := Cast_Result_5;
if Var_Expr_9 /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 1614 '<FieldAccess for is_derived_tagged_type at ast.py:18218>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18218
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.is_derived_tagged_type
Fld_20 := Libadalang.Implementation.Type_Decl_P_Is_Derived_Tagged_Type (Node => Var_Expr_9.Node, E_Info => Var_Expr_9.Info);
--# end
--# expr-done 1614
Scope_Result_11 := Fld_20;
   --# end
   Result_Var_9 := Scope_Result_11;
else
   Result_Var_9 := False;
end if;
   If_Result_4 := Result_Var_9;
end if;
if If_Result_4 then
   --# expr-start 1616 '<Not at ast.py:18222>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18222
Var_Expr_10 := Block_Var_3;
if Var_Expr_10 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1615 '<FieldAccess for is_classwide at ast.py:18222>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18222
      if Var_Expr_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_21 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_10.Node, E_Info => Var_Expr_10.Info);
--# end
--# expr-done 1615
Scope_Result_12 := Fld_21;
   --# end
   Result_Var_10 := Scope_Result_12;
else
   Result_Var_10 := False;
end if;
Not_Val_1 := not (Result_Var_10); 
--# expr-done 1616
   If_Result_5 := Not_Val_1;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
Var_Expr_11 := Block_Var_4;
if Var_Expr_11 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ds Var_Expr_11
--# expr-start 1617 '<Eq at ast.py:18227>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18227
Cast_Expr_2 := Block_Var; 
      Cast_Result_6 := Cast_Expr_2;
Is_Equal := Cast_Result_6 = Var_Expr_11; 
--# expr-done 1617
if Is_Equal then
   If_Result_6 := True;
else
   --# expr-start 1621 '<Eq at ast.py:18227>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18227
--# expr-start 1619 '<Cast to BasePackageDecl at ast.py:18229>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18229
Var_Expr_13 := Block_Var;
if Var_Expr_13 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1618 '<parent at ast.py:18229>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18229
      if Var_Expr_13 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Var_Expr_13.Parent; 
--# expr-done 1618
Scope_Result_13 := Fld_22;
   --# end
   Result_Var_11 := Scope_Result_13;
else
   Result_Var_11 := No_Bare_Ada_Node;
end if;
   if Result_Var_11 = null
      or else Result_Var_11.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_7 := Result_Var_11;
   else
         Cast_Result_7 := No_Bare_Ada_Node;
   end if;
--# expr-done 1619
Var_Expr_12 := Cast_Result_7;
if Var_Expr_12 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1620 '<FieldAccess for public_part at ast.py:18229>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18229
      if Var_Expr_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Var_Expr_12.Base_Package_Decl_F_Public_Part;
--# expr-done 1620
Scope_Result_14 := Fld_23;
   --# end
   Result_Var_12 := Scope_Result_14;
else
   Result_Var_12 := No_Bare_Ada_Node;
end if;
      Cast_Result_8 := Result_Var_12;
Is_Equal_1 := Var_Expr_11 = Cast_Result_8; 
--# expr-done 1621
   If_Result_6 := Is_Equal_1;
end if;
Scope_Result_15 := If_Result_6;
   --# end
   Result_Var_13 := Scope_Result_15;
else
   Result_Var_13 := False;
end if;
   If_Result_7 := Result_Var_13;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
   If_Result_8 := Block_Var_3;
else
   If_Result_8 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 1622
Let_Result_2 := If_Result_8; 
--# end
--# expr-done 1623
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_1260;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1260;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Get_Candidate_Type_For_Primitive;
--# end
--# property-start BaseSubpSpec.candidate_primitive_subp_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18236
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Candidate_Primitive_Subp_Types
  (Node : Bare_Base_Subp_Spec
      ; Canonicalize : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1261 with Inline_Always;
            procedure Finalizer_Scope_2053 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Block_Var_1 : Internal_Entity_Type_Expr_Array_Access;
Var_Expr : Internal_Entity_Type_Expr;
Var_Expr_1 : Internal_Entity_Incomplete_Type_Decl;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Map_Result : Internal_Entity_Type_Expr_Array_Access;
Fld_3 : Internal_Entity_Type_Expr;
Singleton : Internal_Entity_Type_Expr_Array_Access;
Scope_Result : Internal_Entity_Type_Expr_Array_Access;
Result_Var : Internal_Entity_Type_Expr_Array_Access;
Concat_Result : Internal_Entity_Type_Expr_Array_Access;
Fld_4 : Internal_Entity_Base_Type_Decl;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Incomplete_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Map_Result_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Unique_Array : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1261 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1261;
            procedure Finalizer_Scope_2053 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2053;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Candidate_Primitive_Subp_Types,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Canonicalize);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 1639 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1624 '<FieldAccess for unpacked_formal_params at ast.py:18246>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18246
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1624
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind params Block_Var
--# expr-start 1629 '<Concat at ast.py:18247>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18247
--# expr-start 1627 '<Map at ast.py:18247>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18247
      declare
         Map_Result_Vec : Internal_Entity_Type_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Block_Var;
      begin
         for Item_65 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_65
      --# expr-start 1626 '<FieldAccess for type_expression at ast.py:18247>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18247
--# expr-start 1625 '<FieldAccess for formal_decl at ast.py:18247>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18247
      if Item_65.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_65.Node, E_Info => Item_65.Info);
--# end
--# expr-done 1625
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 1626
         declare
            Item_To_Append : constant Internal_Entity_Type_Expr := Fld_2;
         begin
            Internal_Entity_Type_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Type_Expr_Array
           (Items_Count => Natural (Internal_Entity_Type_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Type_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Type_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Type_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1627
--# expr-start 1628 '<FieldAccess for returns at ast.py:18248>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18248
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1628
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
                Singleton := Create_Internal_Entity_Type_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2053;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Map_Result, Result_Var); 
--# expr-done 1629
Block_Var_1 := Concat_Result; Inc_Ref (Block_Var_1);
--# bind types Block_Var_1
--# expr-start 1638 '<unique at ast.py:18262>' Unique_Array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18262
--# expr-start 1637 '<Map at ast.py:18257>' Map_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18257
   --# expr-start 1634 '<Filter at ast.py:18255>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18255
   --# expr-start 1631 '<Map at ast.py:18251>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18251
      declare
         Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Type_Expr_Array_Access := Block_Var_1;
      begin
         for Item_68 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_68
      --# expr-start 1630 '<FieldAccess for get_candidate_type_for_primitive at ast.py:18252>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18252
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.get_candidate_type_for_primitive
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Candidate_Type_For_Primitive (Node => Ent.Node, Type_Expr => Item_68, Canonicalize => Canonicalize, E_Info => Ent.Info);
--# end
--# expr-done 1630
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1631
      declare
         Map_Result_2_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_1;
      begin
         for Item_67 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_67
         --# expr-start 1633 '<Not at ast.py:18256>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18256
--# expr-start 1632 '<IsNull at ast.py:18256>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18256
Is_Null := Item_67.Node = null; 
--# expr-done 1632
Not_Val := not (Is_Null); 
--# expr-done 1633
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_67;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 1634
      declare
         Map_Result_3_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_2;
      begin
         for Item_66 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_66
--# expr-start 1635 '<Cast to IncompleteTypeDecl at ast.py:18258>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18258
Cast_Expr := Item_66; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Incomplete_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Incomplete_Type_Decl;
   end if;
--# expr-done 1635
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Incomplete_Type_Decl then
   --# scope-start
   --# bind i Var_Expr_1
--# expr-start 1636 '<FieldAccess for next_part at ast.py:18259>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18259
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1636
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Item_66;
end if;
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Result_Var_1;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 1637
Unique_Array := Make_Unique (Map_Result_3); 
--# expr-done 1638
Let_Result := Unique_Array; Inc_Ref (Let_Result);
--# end
--# expr-done 1639
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1261;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1261;
                     Finalizer_Scope_2053;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl_Array_Access,
                        As_Internal_Entity_Base_Type_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Subp_Spec_P_Candidate_Primitive_Subp_Types;
--# end
--# property-start BaseSubpSpec.candidate_primitive_subp_first_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18264
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Candidate_Primitive_Subp_First_Type
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1262 with Inline_Always;
            procedure Finalizer_Scope_2058 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1262 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1262;
            procedure Finalizer_Scope_2058 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_2058;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1642 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1640 '<FieldAccess for candidate_primitive_subp_types at ast.py:18270>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18270
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Types (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1640
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Base_Type_Decl_Array_Type then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1641 '<at at ast.py:18270>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18270
Get_Result := Get (Self, Var_Expr, 0, True); 
--# expr-done 1641
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_2058;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1642
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1262;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1262;
                     Finalizer_Scope_2058;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Candidate_Primitive_Subp_First_Type;
--# end
--# property-start BaseSubpSpec.candidate_primitive_subp_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18272
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type
  (Node : Bare_Base_Subp_Spec
      ; Canonicalize : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1263 with Inline_Always;
      Cast_Expr : Bare_Base_Subp_Spec;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Find_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1263 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1263;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Canonicalize);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 1648 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1647 '<bind at ast.py:18279>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18279
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 1646 '<Find at ast.py:18280>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18280
   --# expr-start 1643 '<FieldAccess for candidate_primitive_subp_types at ast.py:18280>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18280
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Types (Node => Ent.Node, Canonicalize => Canonicalize, E_Info => Ent.Info);
--# end
--# expr-done 1643
      Find_Result := No_Entity_Base_Type_Decl;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld;
      begin
         for Item_69 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_69
      --# expr-start 1645 '<FieldAccess for is_tagged_type at ast.py:18281>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18281
--# expr-start 1644 '<FieldAccess for full_view at ast.py:18281>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18281
      if Item_69.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Item_69.Node, E_Info => Item_69.Info);
--# end
--# expr-done 1644
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 1645
      if Fld_2 then
         Find_Result := Item_69; 
      end if;
   --# end
               exit when Fld_2;
         end loop;
      end;
--# expr-done 1646
Dyn_Var_Bind_Result := Find_Result; 
--# expr-done 1647
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 1648
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1263;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1263;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type;
--# end
--# property-start BaseSubpSpec.decl_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18285
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Decl_Spec
  (Node : Bare_Base_Subp_Spec
      ; Follow_Generic : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind follow_generic Follow_Generic
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Not_Val : Boolean;
Is_A : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Scope_Result_2 : Internal_Entity_Base_Subp_Spec;
Result_Var_2 : Internal_Entity_Base_Subp_Spec;
Let_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1656 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1649 '<FieldAccess for name at ast.py:18299>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18299
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1649
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 1650 '<FieldAccess for basic_decl at ast.py:18299>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18299
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1650
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Block_Var := Result_Var; 
--# bind bd Block_Var
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1651 '<FieldAccess for canonical_part at ast.py:18300>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18300
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1651
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Var_Expr_1 := Result_Var_1;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr_1
--# expr-start 1655 '<If at ast.py:18301>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18301
--# expr-start 1652 '<Not at ast.py:18302>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18302
Not_Val := not (Follow_Generic); 
--# expr-done 1652
if Not_Val then
   --# expr-start 1653 '<IsA GenericSubpInternal at ast.py:18302>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18302
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Generic_Subp_Internal_Range; 
--# expr-done 1653
   If_Result := Is_A;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := No_Entity_Base_Subp_Spec;
else
   --# expr-start 1654 '<FieldAccess for subp_spec_or_null at ast.py:18304>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18304
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_1.Node, Follow_Generic => Follow_Generic, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1654
   If_Result_1 := Fld_3;
end if;
--# expr-done 1655
Scope_Result_2 := If_Result_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Ent;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 1656
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Decl_Spec;
--# end
--# property-start BaseSubpSpec.primitive_decl_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18309
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Decl_Spec
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Base_Subp_Spec;
Let_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1658 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1657 '<FieldAccess for decl_spec at ast.py:18317>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18317
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Decl_Spec (Node => Ent.Node, Follow_Generic => False, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1657
Let_Result := Fld; 
--# end
--# expr-done 1658
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Primitive_Decl_Spec;
--# end
--# property-start BaseSubpSpec.as_primitive_subp_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18319
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_As_Primitive_Subp_Type
  (Node : Bare_Base_Subp_Spec
      ; Typ : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2063 with Inline_Always;
            procedure Finalizer_Scope_2064 with Inline_Always;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Boolean;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Fld_3 : Bare_Ada_Node;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Bare_Basic_Decl;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Quantifier_Result : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2063 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_2063;
            procedure Finalizer_Scope_2064 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2064;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1669 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1659 '<FieldAccess for name at ast.py:18329>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18329
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1659
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 1667 '<AnyQuantifier at ast.py:18333>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18333
   --# expr-start 1662 '<EnvGet at ast.py:18333>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18333
Var_Expr_2 := Typ;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1660 '<FieldAccess for primitives_env at ast.py:18333>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18333
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1660
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2064;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 1661 '<FieldAccess for name_symbol at ast.py:18333>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18333
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr.Node);
--# end
--# expr-done 1661
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Result_Var, Key => Fld_2, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1662
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_70 of
               Collection.Items
         loop
   --# scope-start
               --# bind b Item_70
      --# expr-start 1666 '<Eq at ast.py:18334>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18334
--# expr-start 1663 '<FieldAccess for node at ast.py:18334>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18334
Fld_3 := Item_70.Node;
--# expr-done 1663
--# expr-start 1665 '<FieldAccess for node at ast.py:18334>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18334
--# expr-start 1664 '<FieldAccess for basic_decl at ast.py:18334>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18334
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1664
Fld_5 := Fld_4.Node;
--# expr-done 1665
      Cast_Result := Fld_5;
Is_Equal := Fld_3 = Cast_Result; 
--# expr-done 1666
      Quantifier_Result := Is_Equal;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 1667
Var_Expr_1 := Quantifier_Result;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 1668 '<FieldAccess for real_type at ast.py:18339>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18339
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Typ, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 1668
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
      Finalizer_Scope_2063;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 1669
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2063;
                     Finalizer_Scope_2064;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_As_Primitive_Subp_Type;
--# end
--# property-start BaseSubpSpec.primitive_subp_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18343
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_Types
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1267 with Inline_Always;
            procedure Finalizer_Scope_2067 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1267 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1267;
            procedure Finalizer_Scope_2067 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2067;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1676 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1670 '<FieldAccess for primitive_decl_spec at ast.py:18349>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18349
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1670
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 1675 '<Filter at ast.py:18350>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18350
   --# expr-start 1671 '<FieldAccess for candidate_primitive_subp_types at ast.py:18350>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18350
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_types
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Types (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1671
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_1;
      begin
         for Item_71 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_71
         --# expr-start 1674 '<Not at ast.py:18351>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18351
--# expr-start 1673 '<IsNull at ast.py:18351>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18351
--# expr-start 1672 '<FieldAccess for as_primitive_subp_type at ast.py:18351>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18351
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.as_primitive_subp_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_As_Primitive_Subp_Type (Node => Var_Expr.Node, Typ => Item_71, E_Info => Var_Expr.Info);
--# end
--# expr-done 1672
Is_Null := Fld_2.Node = null; 
--# expr-done 1673
Not_Val := not (Is_Null); 
--# expr-done 1674
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_71;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1675
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2067;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 1676
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1267;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1267;
                     Finalizer_Scope_2067;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Primitive_Subp_Types;
--# end
--# property-start BaseSubpSpec.primitive_subp_first_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18355
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_First_Type
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1680 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1677 '<FieldAccess for primitive_decl_spec at ast.py:18361>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18361
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1677
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 1679 '<FieldAccess for as_primitive_subp_type at ast.py:18362>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18362
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1678 '<FieldAccess for candidate_primitive_subp_first_type at ast.py:18363>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18363
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_first_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_First_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1678
--# property-call-start BaseSubpSpec.as_primitive_subp_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_As_Primitive_Subp_Type (Node => Var_Expr.Node, Typ => Fld_1, E_Info => Var_Expr.Info);
--# end
--# expr-done 1679
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1680
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Primitive_Subp_First_Type;
--# end
--# property-start BaseSubpSpec.primitive_subp_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18367
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_Tagged_Type
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1684 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1681 '<FieldAccess for primitive_decl_spec at ast.py:18374>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18374
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1681
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 1683 '<FieldAccess for as_primitive_subp_type at ast.py:18375>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18375
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1682 '<FieldAccess for candidate_primitive_subp_tagged_type at ast.py:18376>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18376
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_tagged_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1682
--# property-call-start BaseSubpSpec.as_primitive_subp_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_As_Primitive_Subp_Type (Node => Var_Expr.Node, Typ => Fld_1, E_Info => Var_Expr.Info);
--# end
--# expr-done 1683
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1684
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Primitive_Subp_Tagged_Type;
--# end
--# property-start BaseSubpSpec.has_controlling_result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18380
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Has_Controlling_Result
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1690 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1685 '<FieldAccess for candidate_primitive_subp_tagged_type at ast.py:18388>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18388
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_tagged_type
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1685
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 1687 '<Not at ast.py:18389>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18389
--# expr-start 1686 '<IsNull at ast.py:18389>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18389
Is_Null := Block_Var.Node = null; 
--# expr-done 1686
Not_Val := not (Is_Null); 
--# expr-done 1687
if Not_Val then
   --# expr-start 1689 '<Eq at ast.py:18389>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18389
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 1688 '<FieldAccess for return_type at ast.py:18389>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18389
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 1688
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1689
   If_Result := Is_Equiv;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 1690
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Has_Controlling_Result;
--# end
--# property-start BaseSubpSpec.dottable_subp_of /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18391
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Dottable_Subp_Of
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Classwide_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_6 : Integer;
Comp_Result : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Dottable_Subp_Of,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 1703 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1702 '<bind at ast.py:18397>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18397
--# expr-start 1692 '<FieldAccess for origin_node at ast.py:18397>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18397
--# expr-start 1691 '<FieldAccess for name at ast.py:18397>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18397
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1691
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld.Node);
--# end
--# expr-done 1692
Bound_Origin := Fld_1; 
--# expr-start 1701 '<If at ast.py:18397>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18397
--# expr-start 1693 '<FieldAccess for nb_max_params at ast.py:18398>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18398
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Ent.Node);
--# end
--# expr-done 1693
Comp_Result := Fld_6 > 0; 
if Comp_Result then
--# expr-start 1694 '<FieldAccess for potential_dottable_type at ast.py:18399>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18399
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.potential_dottable_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Potential_Dottable_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 1694
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 1700 '<Cond at ast.py:18399>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18399
--# expr-start 1695 '<IsA ClasswideTypeDecl at ast.py:18400>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18400
Is_A := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range; 
--# expr-done 1695
if Is_A then
   --# expr-start 1697 '<FieldAccess for type_decl at ast.py:18401>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18401
--# expr-start 1696 '<Cast to ClasswideTypeDecl at ast.py:18401>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18401
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 1696
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld_3 := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1697
   If_Result_1 := Fld_3;
else
--# expr-start 1699 '<FieldAccess for is_tagged_type at ast.py:18408>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18408
--# expr-start 1698 '<FieldAccess for full_view at ast.py:18408>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18408
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1698
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_4.Node, Origin => Bound_Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 1699
if Fld_5 then
   If_Result := Var_Expr;
else
   If_Result := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 1700
Scope_Result := If_Result_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
   If_Result_2 := Result_Var;
else
   If_Result_2 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 1701
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 1702
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 1703
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Subp_Spec_P_Dottable_Subp_Of;
--# end
--# property-start BaseSubpSpec.return_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18416
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Return_Type
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1706 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1704 '<FieldAccess for returns at ast.py:18423>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18423
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1704
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind rt Var_Expr
--# expr-start 1705 '<FieldAccess for real_designated_type at ast.py:18424>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18424
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Var_Expr, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1705
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1706
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Return_Type;
--# end
--# property-start BaseSubpSpec.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18427
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Xref_Entry_Point
  (Node : Bare_Base_Subp_Spec
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Xref_Entry_Point;
--# end
--# property-start BaseSubpSpec.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18428
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Xref_Equation
  (Node : Bare_Base_Subp_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1274 with Inline_Always;
            procedure Finalizer_Scope_2073 with Inline_Always;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_1274 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1274;
            procedure Finalizer_Scope_2073 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2073;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1707 '<FieldAccess for returns at ast.py:18428>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18428
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1707
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 1708 '<FieldAccess for sub_equation at ast.py:18428>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18428
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1708
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2073;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18429") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1274;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1274;
                     Finalizer_Scope_2073;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Xref_Equation;
--# end
 function Base_Subp_Spec_P_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is (raise Property_Error
    with "Property BaseSubpSpec.params not implemented on type "
    & Kind_Name (Node));
   --
   --  Primitives for Bare_Entry_Spec
   --
      procedure Initialize_Fields_For_Entry_Spec
        (Self : Bare_Entry_Spec
         ; Entry_Spec_F_Entry_Name : Bare_Defining_Name
         ; Entry_Spec_F_Family_Type : Bare_Ada_Node
         ; Entry_Spec_F_Entry_Params : Bare_Params
        ) is
      begin
            Self.Entry_Spec_F_Entry_Name := Entry_Spec_F_Entry_Name;
            Self.Entry_Spec_F_Family_Type := Entry_Spec_F_Family_Type;
            Self.Entry_Spec_F_Entry_Params := Entry_Spec_F_Entry_Params;
      end Initialize_Fields_For_Entry_Spec;
   function Entry_Spec_F_Entry_Name
     (Node : Bare_Entry_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Spec_F_Entry_Name;
   end;
   function Entry_Spec_F_Family_Type
     (Node : Bare_Entry_Spec) return Bare_Ada_Node
   is
   begin
         return Node.Entry_Spec_F_Family_Type;
   end;
   function Entry_Spec_F_Entry_Params
     (Node : Bare_Entry_Spec) return Bare_Params
   is
   begin
         return Node.Entry_Spec_F_Entry_Params;
   end;
--# property-start EntrySpec.name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18715
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Name
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Entry_Spec  := Bare_Entry_Spec (Node);
     Ent : Internal_Entity_Entry_Spec :=
       Internal_Entity_Entry_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1709 '<FieldAccess for entry_name at ast.py:18715>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18715
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Spec_F_Entry_Name, Info => Ent.Info);
--# expr-done 1709
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Spec_P_Name;
--# end
--# property-start EntrySpec.params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18716
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Params
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Entry_Spec  := Bare_Entry_Spec (Node);
     Ent : Internal_Entity_Entry_Spec :=
       Internal_Entity_Entry_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_1308 with Inline_Always;
            procedure Finalizer_Scope_2074 with Inline_Always;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Item_72 : Internal_Entity_Param_Spec;
Bare_Item_72 : Bare_Param_Spec;
Map_Result : Internal_Entity_Param_Spec_Array_Access;
Scope_Result : Internal_Entity_Param_Spec_Array_Access;
Result_Var : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_1308 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1308;
            procedure Finalizer_Scope_2074 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2074;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 1710 '<FieldAccess for entry_params at ast.py:18717>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18717
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Entry_Spec_F_Entry_Params, Info => Ent.Info);
--# expr-done 1710
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1712 '<Map at ast.py:18718>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18718
   --# expr-start 1711 '<FieldAccess for params at ast.py:18718>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18718
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1711
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Param_Spec_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Param_Spec_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_72 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_72 := Untyped_Item_72; 
Is_Null := Bare_Item_72 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_72); 
   If_Result := As_Entity;
end if;
                  Item_72 := If_Result; 
   --# scope-start
               --# bind p Item_72
         declare
            Item_To_Append : constant Internal_Entity_Param_Spec := Item_72;
         begin
            Internal_Entity_Param_Spec_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Param_Spec_Array
           (Items_Count => Natural (Internal_Entity_Param_Spec_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Param_Spec_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Param_Spec_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Param_Spec_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1712
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2074;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Param_Spec_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1308;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1308;
                     Finalizer_Scope_2074;
            raise;
      end;
   return Property_Result;
end Entry_Spec_P_Params;
--# end
--# property-start EntrySpec.returns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18722
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Returns
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Entry_Spec  := Bare_Entry_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Type_Expr;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Spec_P_Returns;
--# end
--# property-start EntrySpec.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18724
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Xref_Equation
  (Node : Bare_Entry_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Spec  := Bare_Entry_Spec (Node);
     Ent : Internal_Entity_Entry_Spec :=
       Internal_Entity_Entry_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1310 with Inline_Always;
            procedure Finalizer_Scope_2076 with Inline_Always;
      Var_Expr : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_1310 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1310;
            procedure Finalizer_Scope_2076 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2076;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1713 '<FieldAccess for family_type at ast.py:18724>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18724
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Spec_F_Family_Type, Info => Ent.Info);
--# expr-done 1713
Var_Expr := Fld;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 1714 '<FieldAccess for sub_equation at ast.py:18725>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18725
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1714
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2076;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18726") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1310;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1310;
                     Finalizer_Scope_2076;
            raise;
      end;
   return Property_Result;
end Entry_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Enum_Subp_Spec
   --
--# property-start EnumSubpSpec.enum_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18440
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Enum_Decl
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Literal_Decl
is
   Self : Bare_Enum_Subp_Spec  := Bare_Enum_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Enum_Literal_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Enum_Literal_Decl;
As_Entity : Internal_Entity_Enum_Literal_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Literal_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1717 '<as_entity at ast.py:18440>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18440
--# expr-start 1716 '<Cast to EnumLiteralDecl at ast.py:18440>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18440
--# expr-start 1715 '<parent at ast.py:18440>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18440
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 1715
   if Fld = null
      or else Fld.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1716
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Literal_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1717
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Subp_Spec_P_Enum_Decl;
--# end
--# property-start EnumSubpSpec.name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18442
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Name
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Enum_Subp_Spec  := Bare_Enum_Subp_Spec (Node);
     Ent : Internal_Entity_Enum_Subp_Spec :=
       Internal_Entity_Enum_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1719 '<FieldAccess for enum_decl_name at ast.py:18442>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18442
--# expr-start 1718 '<FieldAccess for enum_decl at ast.py:18442>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18442
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EnumSubpSpec.enum_decl
Fld := Libadalang.Implementation.Enum_Subp_Spec_P_Enum_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1718
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_decl_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Decl_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1719
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Subp_Spec_P_Name;
--# end
--# property-start EnumSubpSpec.returns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18443
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Returns
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Enum_Subp_Spec  := Bare_Enum_Subp_Spec (Node);
     Ent : Internal_Entity_Enum_Subp_Spec :=
       Internal_Entity_Enum_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Cast_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1721 '<FieldAccess for synth_type_expr at ast.py:18443>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18443
--# expr-start 1720 '<FieldAccess for enum_decl at ast.py:18443>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18443
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EnumSubpSpec.enum_decl
Fld := Libadalang.Implementation.Enum_Subp_Spec_P_Enum_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1720
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EnumLiteralDecl.synth_type_expr
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Synth_Type_Expr (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1721
      Cast_Result := Create_Internal_Entity_Type_Expr
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Subp_Spec_P_Returns;
--# end
--# property-start EnumSubpSpec.params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18444
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Params
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Enum_Subp_Spec  := Bare_Enum_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Param_Spec_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Subp_Spec_P_Params;
--# end
   --
   --  Primitives for Bare_Subp_Spec
   --
      procedure Initialize_Fields_For_Subp_Spec
        (Self : Bare_Subp_Spec
         ; Subp_Spec_F_Subp_Kind : Bare_Subp_Kind
         ; Subp_Spec_F_Subp_Name : Bare_Defining_Name
         ; Subp_Spec_F_Subp_Params : Bare_Params
         ; Subp_Spec_F_Subp_Returns : Bare_Type_Expr
        ) is
      begin
            Self.Subp_Spec_F_Subp_Kind := Subp_Spec_F_Subp_Kind;
            Self.Subp_Spec_F_Subp_Name := Subp_Spec_F_Subp_Name;
            Self.Subp_Spec_F_Subp_Params := Subp_Spec_F_Subp_Params;
            Self.Subp_Spec_F_Subp_Returns := Subp_Spec_F_Subp_Returns;
      end Initialize_Fields_For_Subp_Spec;
   function Subp_Spec_F_Subp_Kind
     (Node : Bare_Subp_Spec) return Bare_Subp_Kind
   is
   begin
         return Node.Subp_Spec_F_Subp_Kind;
   end;
   function Subp_Spec_F_Subp_Name
     (Node : Bare_Subp_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Subp_Spec_F_Subp_Name;
   end;
   function Subp_Spec_F_Subp_Params
     (Node : Bare_Subp_Spec) return Bare_Params
   is
   begin
         return Node.Subp_Spec_F_Subp_Params;
   end;
   function Subp_Spec_F_Subp_Returns
     (Node : Bare_Subp_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Subp_Spec_F_Subp_Returns;
   end;
--# property-start SubpSpec.name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18601
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Name
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Subp_Spec  := Bare_Subp_Spec (Node);
     Ent : Internal_Entity_Subp_Spec :=
       Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1722 '<FieldAccess for subp_name at ast.py:18601>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18601
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Subp_Spec_F_Subp_Name, Info => Ent.Info);
--# expr-done 1722
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Spec_P_Name;
--# end
--# property-start SubpSpec.params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18602
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Params
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Subp_Spec  := Bare_Subp_Spec (Node);
     Ent : Internal_Entity_Subp_Spec :=
       Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_1297 with Inline_Always;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Scope_Result : Internal_Entity_Param_Spec_List;
Result_Var : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Item_73 : Internal_Entity_Param_Spec;
Bare_Item_73 : Bare_Param_Spec;
Map_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_1297 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1297;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1725 '<Map at ast.py:18602>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18602
--# expr-start 1723 '<FieldAccess for subp_params at ast.py:18602>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18602
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Subp_Spec_F_Subp_Params, Info => Ent.Info);
--# expr-done 1723
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
--# expr-start 1724 '<FieldAccess for params at ast.py:18602>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18602
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1724
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Param_Spec_List;
end if;
Fld_2 := Result_Var.Node;
      declare
         Map_Result_Vec : Internal_Entity_Param_Spec_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Param_Spec_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_73 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_73 := Untyped_Item_73; 
Is_Null := Bare_Item_73 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_73); 
   If_Result := As_Entity;
end if;
                  Item_73 := If_Result; 
   --# scope-start
               --# bind p Item_73
         declare
            Item_To_Append : constant Internal_Entity_Param_Spec := Item_73;
         begin
            Internal_Entity_Param_Spec_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Param_Spec_Array
           (Items_Count => Natural (Internal_Entity_Param_Spec_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Param_Spec_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Param_Spec_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Param_Spec_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1725
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1297;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1297;
            raise;
      end;
   return Property_Result;
end Subp_Spec_P_Params;
--# end
--# property-start SubpSpec.returns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18604
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Returns
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Subp_Spec  := Bare_Subp_Spec (Node);
     Ent : Internal_Entity_Subp_Spec :=
       Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1726 '<FieldAccess for subp_returns at ast.py:18604>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18604
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Subp_Spec_F_Subp_Returns, Info => Ent.Info);
--# expr-done 1726
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Spec_P_Returns;
--# end
   --
   --  Primitives for Bare_Synthetic_Binary_Spec
   --
      procedure Initialize_Fields_For_Synthetic_Binary_Spec
        (Self : Bare_Synthetic_Binary_Spec
         ; Synthetic_Binary_Spec_F_Left_Param : Bare_Synthetic_Formal_Param_Decl
         ; Synthetic_Binary_Spec_F_Right_Param : Bare_Synthetic_Formal_Param_Decl
         ; Synthetic_Binary_Spec_F_Return_Type_Expr : Bare_Type_Expr
        ) is
      begin
            Self.Synthetic_Binary_Spec_F_Left_Param := Synthetic_Binary_Spec_F_Left_Param;
            Self.Synthetic_Binary_Spec_F_Right_Param := Synthetic_Binary_Spec_F_Right_Param;
            Self.Synthetic_Binary_Spec_F_Return_Type_Expr := Synthetic_Binary_Spec_F_Return_Type_Expr;
      Self.Synthetic_Binary_Spec_Subp_Symbol := null;
      end Initialize_Fields_For_Synthetic_Binary_Spec;
   function Synthetic_Binary_Spec_F_Left_Param
     (Node : Bare_Synthetic_Binary_Spec) return Bare_Synthetic_Formal_Param_Decl
   is
   begin
         return Node.Synthetic_Binary_Spec_F_Left_Param;
   end;
   function Synthetic_Binary_Spec_F_Right_Param
     (Node : Bare_Synthetic_Binary_Spec) return Bare_Synthetic_Formal_Param_Decl
   is
   begin
         return Node.Synthetic_Binary_Spec_F_Right_Param;
   end;
   function Synthetic_Binary_Spec_F_Return_Type_Expr
     (Node : Bare_Synthetic_Binary_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Synthetic_Binary_Spec_F_Return_Type_Expr;
   end;
--# property-start SyntheticBinarySpec.name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18559
pragma Warnings (Off, "is not referenced");
 function Synthetic_Binary_Spec_P_Name
  (Node : Bare_Synthetic_Binary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Synthetic_Binary_Spec  := Bare_Synthetic_Binary_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1729 '<as_entity at ast.py:18559>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18559
--# expr-start 1728 '<FieldAccess for synthesize_defining_name at ast.py:18559>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18559
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1727 '<FieldAccess for subp_symbol at ast.py:18559>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18559
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Binary_Spec_Subp_Symbol;
--# expr-done 1727
--# property-call-start AdaNode.synthesize_defining_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Synthesize_Defining_Name (Node => Self, Sym => Fld);
--# end
--# expr-done 1728
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 1729
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Binary_Spec_P_Name;
--# end
--# property-start SyntheticBinarySpec.returns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18560
pragma Warnings (Off, "is not referenced");
 function Synthetic_Binary_Spec_P_Returns
  (Node : Bare_Synthetic_Binary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Synthetic_Binary_Spec  := Bare_Synthetic_Binary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Binary_Spec :=
       Internal_Entity_Synthetic_Binary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1730 '<FieldAccess for return_type_expr at ast.py:18560>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18560
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Synthetic_Binary_Spec_F_Return_Type_Expr, Info => Ent.Info);
--# expr-done 1730
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Binary_Spec_P_Returns;
--# end
--# property-start SyntheticBinarySpec.abstract_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18562
pragma Warnings (Off, "is not referenced");
 function Synthetic_Binary_Spec_P_Abstract_Formal_Params
  (Node : Bare_Synthetic_Binary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Synthetic_Binary_Spec  := Bare_Synthetic_Binary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Binary_Spec :=
       Internal_Entity_Synthetic_Binary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1292 with Inline_Always;
      Fld : Internal_Entity_Synthetic_Formal_Param_Decl;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Fld_1 : Internal_Entity_Synthetic_Formal_Param_Decl;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Decl;
Array_Lit : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1292 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1292;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1736 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1735 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 1732 '<Cast to BaseFormalParamDecl at ast.py:18565>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18565
--# expr-start 1731 '<FieldAccess for left_param at ast.py:18565>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18565
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Synthetic_Formal_Param_Decl (Node => Ent.Node.Synthetic_Binary_Spec_F_Left_Param, Info => Ent.Info);
--# expr-done 1731
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 1732
--# expr-start 1734 '<Cast to BaseFormalParamDecl at ast.py:18566>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18566
--# expr-start 1733 '<FieldAccess for right_param at ast.py:18566>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18566
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Synthetic_Formal_Param_Decl (Node => Ent.Node.Synthetic_Binary_Spec_F_Right_Param, Info => Ent.Info);
--# expr-done 1733
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 1734
Array_Lit := Create_Internal_Entity_Base_Formal_Param_Decl_Array (Internal_Internal_Entity_Base_Formal_Param_Decl_Array'(1 => Cast_Result, 2 => Cast_Result_1)); 
--# expr-done 1735
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 1736
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1292;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1292;
            raise;
      end;
   return Property_Result;
end Synthetic_Binary_Spec_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Synthetic_Unary_Spec
   --
      procedure Initialize_Fields_For_Synthetic_Unary_Spec
        (Self : Bare_Synthetic_Unary_Spec
         ; Synthetic_Unary_Spec_F_Right_Param : Bare_Synthetic_Formal_Param_Decl
         ; Synthetic_Unary_Spec_F_Return_Type_Expr : Bare_Synthetic_Type_Expr
        ) is
      begin
            Self.Synthetic_Unary_Spec_F_Right_Param := Synthetic_Unary_Spec_F_Right_Param;
            Self.Synthetic_Unary_Spec_F_Return_Type_Expr := Synthetic_Unary_Spec_F_Return_Type_Expr;
      Self.Synthetic_Unary_Spec_Subp_Symbol := null;
      end Initialize_Fields_For_Synthetic_Unary_Spec;
   function Synthetic_Unary_Spec_F_Right_Param
     (Node : Bare_Synthetic_Unary_Spec) return Bare_Synthetic_Formal_Param_Decl
   is
   begin
         return Node.Synthetic_Unary_Spec_F_Right_Param;
   end;
   function Synthetic_Unary_Spec_F_Return_Type_Expr
     (Node : Bare_Synthetic_Unary_Spec) return Bare_Synthetic_Type_Expr
   is
   begin
         return Node.Synthetic_Unary_Spec_F_Return_Type_Expr;
   end;
--# property-start SyntheticUnarySpec.name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18541
pragma Warnings (Off, "is not referenced");
 function Synthetic_Unary_Spec_P_Name
  (Node : Bare_Synthetic_Unary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Synthetic_Unary_Spec  := Bare_Synthetic_Unary_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1739 '<as_entity at ast.py:18541>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18541
--# expr-start 1738 '<FieldAccess for synthesize_defining_name at ast.py:18541>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18541
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1737 '<FieldAccess for subp_symbol at ast.py:18541>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18541
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Unary_Spec_Subp_Symbol;
--# expr-done 1737
--# property-call-start AdaNode.synthesize_defining_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Synthesize_Defining_Name (Node => Self, Sym => Fld);
--# end
--# expr-done 1738
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 1739
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Unary_Spec_P_Name;
--# end
--# property-start SyntheticUnarySpec.returns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18542
pragma Warnings (Off, "is not referenced");
 function Synthetic_Unary_Spec_P_Returns
  (Node : Bare_Synthetic_Unary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Synthetic_Unary_Spec  := Bare_Synthetic_Unary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Unary_Spec :=
       Internal_Entity_Synthetic_Unary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Synthetic_Type_Expr;
Cast_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 1740 '<FieldAccess for return_type_expr at ast.py:18542>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18542
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Synthetic_Type_Expr (Node => Ent.Node.Synthetic_Unary_Spec_F_Return_Type_Expr, Info => Ent.Info);
--# expr-done 1740
      Cast_Result := Create_Internal_Entity_Type_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Unary_Spec_P_Returns;
--# end
--# property-start SyntheticUnarySpec.abstract_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18544
pragma Warnings (Off, "is not referenced");
 function Synthetic_Unary_Spec_P_Abstract_Formal_Params
  (Node : Bare_Synthetic_Unary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Synthetic_Unary_Spec  := Bare_Synthetic_Unary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Unary_Spec :=
       Internal_Entity_Synthetic_Unary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1289 with Inline_Always;
      Fld : Internal_Entity_Synthetic_Formal_Param_Decl;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Array_Lit : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1289 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1289;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1744 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1743 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 1742 '<Cast to BaseFormalParamDecl at ast.py:18546>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18546
--# expr-start 1741 '<FieldAccess for right_param at ast.py:18546>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18546
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Synthetic_Formal_Param_Decl (Node => Ent.Node.Synthetic_Unary_Spec_F_Right_Param, Info => Ent.Info);
--# expr-done 1741
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 1742
Array_Lit := Create_Internal_Entity_Base_Formal_Param_Decl_Array (Internal_Internal_Entity_Base_Formal_Param_Decl_Array'(1 => Cast_Result)); 
--# expr-done 1743
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 1744
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1289;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1289;
            raise;
      end;
   return Property_Result;
end Synthetic_Unary_Spec_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Component_List
   --
      procedure Initialize_Fields_For_Component_List
        (Self : Bare_Component_List
         ; Component_List_F_Components : Bare_Ada_Node_List
         ; Component_List_F_Variant_Part : Bare_Variant_Part
        ) is
      begin
            Self.Component_List_F_Components := Component_List_F_Components;
            Self.Component_List_F_Variant_Part := Component_List_F_Variant_Part;
      end Initialize_Fields_For_Component_List;
   function Component_List_F_Components
     (Node : Bare_Component_List) return Bare_Ada_Node_List
   is
   begin
         return Node.Component_List_F_Components;
   end;
   function Component_List_F_Variant_Part
     (Node : Bare_Component_List) return Bare_Variant_Part
   is
   begin
         return Node.Component_List_F_Variant_Part;
   end;
--# property-start ComponentList.type_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5068
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Type_Def
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Def
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Def;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Type_Def;
As_Entity : Internal_Entity_Type_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1748 '<as_entity at ast.py:5068>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5068
--# expr-start 1747 '<Cast to TypeDef at ast.py:5068>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5068
--# expr-start 1746 '<parent at ast.py:5068>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5068
--# expr-start 1745 '<parent at ast.py:5068>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5068
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 1745
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1746
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Type_Def
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1747
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Type_Def;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1748
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_List_P_Type_Def;
--# end
--# property-start ComponentList.type_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5069
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Type_Decl
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1751 '<Cast to TypeDecl at ast.py:5069>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5069
--# expr-start 1750 '<parent at ast.py:5069>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5069
--# expr-start 1749 '<FieldAccess for type_def at ast.py:5069>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5069
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_def
Fld := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1749
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1750
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 1751
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_List_P_Type_Decl;
--# end
--# property-start ComponentList.parent_component_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5071
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Parent_Component_List
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Component_List
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Component_List;
      Var_Expr : Internal_Entity_Base_Record_Def;
Var_Expr_1 : Internal_Entity_Derived_Type_Def;
Cast_Expr : Bare_Component_List;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Record_Def;
Scope_Result : Internal_Entity_Base_Record_Def;
Result_Var : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Scope_Result_1 : Internal_Entity_Component_List;
Result_Var_1 : Internal_Entity_Component_List;
Dyn_Var_Bind_Result : Internal_Entity_Component_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1757 '<bind at ast.py:5071>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5071
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 1753 '<Cast to DerivedTypeDef at ast.py:5073>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5073
--# expr-start 1752 '<FieldAccess for type_def at ast.py:5073>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5073
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_def
Fld := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1752
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result_1 := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 1753
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Derived_Type_Def then
   --# scope-start
--# expr-start 1755 '<FieldAccess for record_def at ast.py:5073>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5073
--# expr-start 1754 '<FieldAccess for base_type at ast.py:5073>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5073
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1754
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 1755
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Record_Def;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 1756 '<FieldAccess for comps at ast.py:5073>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5073
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1756
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Component_List;
end if;
Dyn_Var_Bind_Result := Result_Var_1; 
--# expr-done 1757
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_List_P_Parent_Component_List;
--# end
--# property-start ComponentList.abstract_formal_params_for_assocs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5076
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_For_Assocs
  (Node : Bare_Component_List
      ; Assocs : Internal_Entity_Assoc_List
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind assocs Assocs
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_323 with Inline_Always;
      Block_Var : Internal_Entity_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_2 : Internal_Param_Match_Array_Access;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_2 : Internal_Param_Match_Array_Access;
Fld_3 : Internal_Entity_Defining_Name;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Expr : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Find_Result : Internal_Entity_Base_Formal_Param_Decl;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_6 : Internal_Entity_Base_Record_Def;
Fld_7 : Internal_Entity_Component_List;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_323 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_323;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1774 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1758 '<FieldAccess for type_decl at ast.py:5083>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5083
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1758
Block_Var := Fld; 
--# bind td Block_Var
--# expr-start 1759 '<FieldAccess for discriminants_list at ast.py:5084>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5084
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1759
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind discriminants Block_Var_1
--# expr-start 1770 '<Filter at ast.py:5089>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5089
   --# expr-start 1760 '<FieldAccess for match_formals at ast.py:5087>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5087
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Block_Var_1, Params => Assocs, Is_Dottable_Subp => False);
--# end
--# expr-done 1760
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_2;
      begin
         for Item_74 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_74
--# expr-start 1763 '<Not at ast.py:5091>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5091
--# expr-start 1762 '<IsNull at ast.py:5091>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5091
--# expr-start 1761 '<FieldAccess for formal at ast.py:5091>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5091
Fld_3 := Item_74.Formal;
--# expr-done 1761
Is_Null := Fld_3.Node = null; 
--# expr-done 1762
Not_Val := not (Is_Null); 
--# expr-done 1763
if Not_Val then
   --# expr-start 1769 '<Not at ast.py:5092>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5092
--# expr-start 1768 '<IsNull at ast.py:5093>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5093
--# expr-start 1767 '<Find at ast.py:5092>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5092
      Find_Result := No_Entity_Base_Formal_Param_Decl;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Block_Var_1;
      begin
         for Item_75 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_75
      --# expr-start 1766 '<Eq at ast.py:5093>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5093
Cast_Expr := Item_75; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 1765 '<FieldAccess for formal_decl at ast.py:5093>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5093
--# expr-start 1764 '<FieldAccess for formal at ast.py:5093>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5093
Fld_4 := Item_74.Formal;
--# expr-done 1764
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_5 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1765
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1766
      if Is_Equiv then
         Find_Result := Item_75; 
      end if;
   --# end
               exit when Is_Equiv;
         end loop;
      end;
--# expr-done 1767
Is_Null_1 := Find_Result.Node = null; 
--# expr-done 1768
Not_Val_1 := not (Is_Null_1); 
--# expr-done 1769
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
         if If_Result then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_74;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1770
Block_Var_2 := Map_Result; Inc_Ref (Block_Var_2);
--# bind discriminants_matches Block_Var_2
--# expr-start 1773 '<FieldAccess for abstract_formal_params_impl at ast.py:5101>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5101
--# expr-start 1772 '<FieldAccess for comps at ast.py:5101>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5101
--# expr-start 1771 '<FieldAccess for record_def at ast.py:5101>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5101
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1771
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_7 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1772
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_8 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Fld_7.Node, Discriminants => Block_Var_2, Stop_Recurse_At => Stop_Recurse_At, Assocs => Assocs, E_Info => Fld_7.Info);
--# end
--# expr-done 1773
Let_Result := Fld_8; Inc_Ref (Let_Result);
--# end
--# expr-done 1774
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_323;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_323;
            raise;
      end;
   return Property_Result;
end Component_List_P_Abstract_Formal_Params_For_Assocs;
--# end
--# property-start ComponentList.abstract_formal_params_for_delta_assocs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5107
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_For_Delta_Assocs
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_324 with Inline_Always;
            procedure Finalizer_Scope_2084 with Inline_Always;
            procedure Finalizer_Scope_2086 with Inline_Always;
      Block_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Variant_Part;
Var_Expr_1 : Internal_Entity_Component_List;
Fld : Internal_Entity_Variant_Part;
Fld_1 : Internal_Entity_Variant_List;
Scope_Result : Internal_Entity_Variant_List;
Result_Var : Internal_Entity_Variant_List;
Fld_2 : Bare_Variant_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Fld_4 : Internal_Entity_Component_List;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Item_76 : Internal_Entity_Variant;
Bare_Item_76 : Bare_Variant;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_6 : Internal_Entity_Ada_Node_List;
Fld_7 : Bare_Ada_Node_List;
Fld_8 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_77 : Internal_Entity;
Bare_Item_77 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Entity_Component_List;
Fld_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_324 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_324;
            procedure Finalizer_Scope_2084 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2084;
            procedure Finalizer_Scope_2086 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2086;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1788 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1779 '<Mapcat at ast.py:5115>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5115
--# expr-start 1775 '<FieldAccess for variant_part at ast.py:5115>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5115
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1775
Var_Expr := Fld;
if Var_Expr /= No_Entity_Variant_Part then
   --# scope-start
--# expr-start 1776 '<FieldAccess for variant at ast.py:5115>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5115
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Variant_List (Node => Var_Expr.Node.Variant_Part_F_Variant, Info => Var_Expr.Info);
--# expr-done 1776
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Variant_List;
end if;
Fld_2 := Result_Var.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_2;
      begin
         for Untyped_Item_76 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_76 := Untyped_Item_76; 
Is_Null := Bare_Item_76 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_76); 
   If_Result := As_Entity;
end if;
                  Item_76 := If_Result; 
   --# scope-start
               --# bind v Item_76
      --# expr-start 1778 '<FieldAccess for abstract_formal_params_for_delta_assocs at ast.py:5116>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5116
--# expr-start 1777 '<FieldAccess for components at ast.py:5116>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5116
      if Item_76.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Component_List (Node => Item_76.Node.Variant_F_Components, Info => Item_76.Info);
--# expr-done 1777
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_for_delta_assocs
Fld_5 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Delta_Assocs (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1778
         for Item_To_Append of
               Fld_5.Items
         loop
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2084;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1779
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind variant_part_components Block_Var
--# expr-start 1783 '<FilterMap at ast.py:5118>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5118
   --# expr-start 1780 '<FieldAccess for components at ast.py:5118>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5118
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1780
Fld_7 := Fld_6.Node;
      declare
         Map_Result_1_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_7 = null then
      Map_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_7;
      begin
         for Untyped_Item_77 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_77 := Untyped_Item_77; 
Is_Null_1 := Bare_Item_77 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_8 := Fld_6.Info;
As_Entity_1 := (Info => Fld_8, Node => Bare_Item_77); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_77 := If_Result_1; 
   --# scope-start
               --# bind e Item_77
         --# expr-start 1781 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_77.Node /= null 
and then Item_77.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1781
         if Is_A then
      --# expr-start 1782 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_77; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1782
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1783
Block_Var_1 := Map_Result_1; Inc_Ref (Block_Var_1);
--# bind self_components Block_Var_1
--# expr-start 1784 '<FieldAccess for parent_component_list at ast.py:5119>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5119
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.parent_component_list
Fld_9 := Libadalang.Implementation.Component_List_P_Parent_Component_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1784
Var_Expr_1 := Fld_9;
if Var_Expr_1 /= No_Entity_Component_List then
   --# scope-start
--# expr-start 1785 '<FieldAccess for abstract_formal_params_for_delta_assocs at ast.py:5120>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5120
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_for_delta_assocs
Fld_10 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Delta_Assocs (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1785
Scope_Result_1 := Fld_10;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2086;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_2 := Result_Var_1; Inc_Ref (Block_Var_2);
--# bind parent_components Block_Var_2
--# expr-start 1787 '<Concat at ast.py:5122>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5122
--# expr-start 1786 '<Concat at ast.py:5121>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5121
Concat_Result := Concat (Block_Var, Block_Var_1); 
--# expr-done 1786
Concat_Result_1 := Concat (Concat_Result, Block_Var_2); 
--# expr-done 1787
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1788
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_324;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_324;
                     Finalizer_Scope_2084;
                     Finalizer_Scope_2086;
            raise;
      end;
   return Property_Result;
end Component_List_P_Abstract_Formal_Params_For_Delta_Assocs;
--# end
--# property-start ComponentList.abstract_formal_params_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5124
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_Impl
  (Node : Bare_Component_List
      ; Discriminants : Internal_Param_Match_Array_Access
      ; Include_Discriminants : Boolean
         := True
      ; Recurse : Boolean
         := True
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Assocs : Internal_Entity_Assoc_List
         := No_Entity_Assoc_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind discriminants Discriminants
   --# bind include_discriminants Include_Discriminants
   --# bind recurse Recurse
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind assocs Assocs
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_325 with Inline_Always;
            procedure Finalizer_Scope_2088 with Inline_Always;
            procedure Finalizer_Scope_2089 with Inline_Always;
            procedure Finalizer_Scope_2091 with Inline_Always;
      Block_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Variant_Part;
Var_Expr_1 : Internal_Entity_Component_List;
Var_Expr_2 : Internal_Entity_Composite_Constraint;
Var_Expr_3 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_78 : Internal_Entity;
Bare_Item_78 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Variant_Part;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Component_List;
Fld_6 : Internal_Entity_Type_Decl;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Fld_9 : Internal_Entity_Subtype_Indication;
Fld_10 : Internal_Entity_Constraint;
Cast_Result_2 : Internal_Entity_Composite_Constraint;
Fld_11 : Internal_Entity_Assoc_List;
Scope_Result_1 : Internal_Entity_Assoc_List;
Result_Var_1 : Internal_Entity_Assoc_List;
Fld_12 : Internal_Param_Match_Array_Access;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_14 : Internal_Entity_Type_Decl;
Fld_15 : Boolean;
If_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_16 : Internal_Entity_Type_Decl;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_325 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_325;
            procedure Finalizer_Scope_2088 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2088;
            procedure Finalizer_Scope_2089 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_2089;
            procedure Finalizer_Scope_2091 is
            begin
                     Dec_Ref (Fld_17);
            end Finalizer_Scope_2091;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1816 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1795 '<Concat at ast.py:5154>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5154
--# expr-start 1792 '<FilterMap at ast.py:5154>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5154
   --# expr-start 1789 '<FieldAccess for components at ast.py:5154>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5154
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1789
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_78 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_78 := Untyped_Item_78; 
Is_Null := Bare_Item_78 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_78); 
   If_Result := As_Entity;
end if;
                  Item_78 := If_Result; 
   --# scope-start
               --# bind e Item_78
         --# expr-start 1790 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_78.Node /= null 
and then Item_78.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1790
         if Is_A then
      --# expr-start 1791 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_78; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1791
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1792
--# expr-start 1793 '<FieldAccess for variant_part at ast.py:5155>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5155
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1793
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Variant_Part then
   --# scope-start
--# expr-start 1794 '<FieldAccess for get_components at ast.py:5155>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5155
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start VariantPart.get_components
Fld_4 := Libadalang.Implementation.Variant_Part_P_Get_Components (Node => Var_Expr.Node, Discriminants => Discriminants, E_Info => Var_Expr.Info);
--# end
--# expr-done 1794
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2088;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Map_Result, Result_Var); 
--# expr-done 1795
Block_Var := Concat_Result; Inc_Ref (Block_Var);
--# bind self_comps Block_Var
--# expr-start 1811 '<If at ast.py:5162>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5162
if Recurse then
--# expr-start 1796 '<FieldAccess for parent_component_list at ast.py:5164>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5164
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.parent_component_list
Fld_5 := Libadalang.Implementation.Component_List_P_Parent_Component_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1796
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Entity_Component_List then
   --# scope-start
   --# bind pcl Var_Expr_1
--# expr-start 1810 '<If at ast.py:5165>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5165
--# expr-start 1798 '<FieldAccess for matching_type at ast.py:5166>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5166
--# expr-start 1797 '<FieldAccess for type_decl at ast.py:5166>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5166
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_14 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1797
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_14.Node, Expected_Type => Stop_Recurse_At, Origin => No_Bare_Ada_Node, E_Info => Fld_14.Info);
--# end
--# expr-done 1798
if Fld_15 then
   If_Result_1 := Block_Var;
else
   --# expr-start 1809 '<Concat at ast.py:5185>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5185
--# expr-start 1808 '<FieldAccess for abstract_formal_params_impl at ast.py:5168>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5168
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1807 '<FieldAccess for match_formals at ast.py:5169>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5169
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1800 '<FieldAccess for discriminants_list at ast.py:5170>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5170
--# expr-start 1799 '<FieldAccess for type_decl at ast.py:5170>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5170
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_6 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1799
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_6.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => No_Bare_Ada_Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1800
--# expr-start 1805 '<Cast to CompositeConstraint at ast.py:5173>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5173
--# expr-start 1804 '<FieldAccess for constraint at ast.py:5172>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5172
--# expr-start 1803 '<FieldAccess for subtype_indication at ast.py:5172>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5172
--# expr-start 1802 '<Cast to DerivedTypeDef at ast.py:5171>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5171
--# expr-start 1801 '<FieldAccess for type_def at ast.py:5171>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5171
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_def
Fld_8 := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1801
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_1 := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 1802
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Subtype_Indication (Node => Cast_Result_1.Node.Derived_Type_Def_F_Subtype_Indication, Info => Cast_Result_1.Info);
--# expr-done 1803
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Constraint (Node => Fld_9.Node.Subtype_Indication_F_Constraint, Info => Fld_9.Info);
--# expr-done 1804
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Composite_Constraint_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Composite_Constraint
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_2 := No_Entity_Composite_Constraint;
   end if;
--# expr-done 1805
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Entity_Composite_Constraint then
   --# scope-start
   --# bind cc Var_Expr_2
--# expr-start 1806 '<FieldAccess for constraints at ast.py:5174>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5174
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Assoc_List (Node => Var_Expr_2.Node.Composite_Constraint_F_Constraints, Info => Var_Expr_2.Info);
--# expr-done 1806
Scope_Result_1 := Fld_11;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Assocs;
end if;
--# property-call-start AdaNode.match_formals
Fld_12 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Var_Expr_1.Node, Formal_Params => Fld_7, Params => Result_Var_1, Is_Dottable_Subp => False);
--# end
--# expr-done 1807
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_13 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Var_Expr_1.Node, Discriminants => Fld_12, Include_Discriminants => False, Stop_Recurse_At => Stop_Recurse_At, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1808
Concat_Result_1 := Concat (Fld_13, Block_Var); 
--# expr-done 1809
   If_Result_1 := Concat_Result_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1810
Scope_Result_2 := If_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2089;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := Block_Var;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1811
Block_Var_1 := If_Result_2; Inc_Ref (Block_Var_1);
--# bind ret Block_Var_1
--# expr-start 1815 '<If at ast.py:5192>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5192
if Include_Discriminants then
   --# expr-start 1814 '<Concat at ast.py:5194>' Concat_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5194
--# expr-start 1812 '<FieldAccess for type_decl at ast.py:5194>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5194
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_16 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1812
Var_Expr_3 := Fld_16;
if Var_Expr_3 /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 1813 '<FieldAccess for discriminants_list at ast.py:5194>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5194
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_3.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 1813
Scope_Result_3 := Fld_17;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2091;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Concat_Result_2 := Concat (Result_Var_3, Block_Var_1); 
--# expr-done 1814
   If_Result_3 := Concat_Result_2;
else
   If_Result_3 := Block_Var_1;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 1815
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1816
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_325;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_325;
                     Finalizer_Scope_2088;
                     Finalizer_Scope_2089;
                     Finalizer_Scope_2091;
            raise;
      end;
   return Property_Result;
end Component_List_P_Abstract_Formal_Params_Impl;
--# end
--# property-start ComponentList.abstract_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5198
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_326 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_326 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_326;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1818 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1817 '<FieldAccess for abstract_formal_params_impl at ast.py:5200>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5200
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Ent.Node, Discriminants => No_Internal_Param_Match_Array_Type, E_Info => Ent.Info);
--# end
--# expr-done 1817
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 1818
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_326;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_326;
            raise;
      end;
   return Property_Result;
end Component_List_P_Abstract_Formal_Params;
--# end
--# property-start ComponentList.shapes /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5202
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Shapes
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Shape_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_327 with Inline_Always;
            procedure Finalizer_Scope_2093 with Inline_Always;
            procedure Finalizer_Scope_2094 with Inline_Always;
            procedure Finalizer_Scope_2095 with Inline_Always;
      Block_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Variant_Part;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_79 : Internal_Entity;
Bare_Item_79 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Variant_Part;
Fld_4 : Internal_Entity_Variant_List;
Fld_5 : Bare_Variant_List;
Fld_6 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Variant;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Variant;
Fld_7 : Internal_Entity_Component_List;
Fld_8 : Internal_Shape_Array_Access;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_10 : Internal_Entity_Identifier;
Fld_11 : Internal_Entity_Alternatives_List;
New_Struct : Internal_Discriminant_Values;
Singleton : Internal_Discriminant_Values_Array_Access;
Fld_12 : Internal_Discriminant_Values_Array_Access;
Concat_Result_1 : Internal_Discriminant_Values_Array_Access;
New_Struct_1 : Internal_Shape := No_Shape;
Map_Result_1 : Internal_Shape_Array_Access;
Item_80 : Internal_Entity_Variant;
Bare_Item_80 : Bare_Variant;
Map_Result_2 : Internal_Shape_Array_Access;
Scope_Result : Internal_Shape_Array_Access;
New_Struct_2 : Internal_Shape := No_Shape;
Singleton_1 : Internal_Shape_Array_Access;
Result_Var : Internal_Shape_Array_Access;
Let_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_327 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_327;
            procedure Finalizer_Scope_2093 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2093;
            procedure Finalizer_Scope_2094 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2094;
            procedure Finalizer_Scope_2095 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2095;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1838 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1822 '<FilterMap at ast.py:5207>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5207
   --# expr-start 1819 '<FieldAccess for components at ast.py:5207>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5207
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1819
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_79 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_79 := Untyped_Item_79; 
Is_Null := Bare_Item_79 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_79); 
   If_Result := As_Entity;
end if;
                  Item_79 := If_Result; 
   --# scope-start
               --# bind e Item_79
         --# expr-start 1820 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_79.Node /= null 
and then Item_79.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1820
         if Is_A then
      --# expr-start 1821 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_79; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1821
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1822
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind self_comps Block_Var
--# expr-start 1823 '<FieldAccess for variant_part at ast.py:5208>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5208
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1823
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Variant_Part then
   --# scope-start
   --# bind vpart Var_Expr
--# expr-start 1836 '<Mapcat at ast.py:5209>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5209
   --# expr-start 1824 '<FieldAccess for variant at ast.py:5209>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5209
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Variant_List (Node => Var_Expr.Node.Variant_Part_F_Variant, Info => Var_Expr.Info);
--# expr-done 1824
Fld_5 := Fld_4.Node;
      declare
         Map_Result_2_Vec : Internal_Shape_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result_2 := Create_Internal_Shape_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_5;
      begin
         for Untyped_Item_80 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_80 := Untyped_Item_80; 
Is_Null_1 := Bare_Item_80 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Variant;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Bare_Item_80); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_80 := If_Result_1; 
   --# scope-start
               --# bind v Item_80
      --# expr-start 1835 '<Map at ast.py:5210>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5210
   --# expr-start 1826 '<FieldAccess for shapes at ast.py:5210>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5210
--# expr-start 1825 '<FieldAccess for components at ast.py:5210>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5210
      if Item_80.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Component_List (Node => Item_80.Node.Variant_F_Components, Info => Item_80.Info);
--# expr-done 1825
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.shapes
Fld_8 := Libadalang.Implementation.Component_List_P_Shapes (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 1826
      declare
         Map_Result_1_Vec : Internal_Shape_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Shape_Array_Access := Fld_8;
      begin
         for Item_81 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_81
      --# expr-start 1834 '<New Shape at ast.py:5211>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5211
--# expr-start 1828 '<Concat at ast.py:5212>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5212
--# expr-start 1827 '<FieldAccess for components at ast.py:5212>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5212
Fld_9 := Item_81.Components;
Inc_Ref (Fld_9);
--# expr-done 1827
Concat_Result := Concat (Block_Var, Fld_9); 
--# expr-done 1828
--# expr-start 1833 '<Concat at ast.py:5216>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5216
--# expr-start 1831 '<New DiscriminantValues at ast.py:5213>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5213
--# expr-start 1829 '<FieldAccess for discr_name at ast.py:5214>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5214
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Identifier (Node => Var_Expr.Node.Variant_Part_F_Discr_Name, Info => Var_Expr.Info);
--# expr-done 1829
--# expr-start 1830 '<FieldAccess for choices at ast.py:5215>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5215
      if Item_80.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Alternatives_List (Node => Item_80.Node.Variant_F_Choices, Info => Item_80.Info);
--# expr-done 1830
New_Struct := (Discriminant => Fld_10, Values => Fld_11); 
--# expr-done 1831
                Singleton := Create_Internal_Discriminant_Values_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
--# expr-start 1832 '<FieldAccess for discriminants_values at ast.py:5216>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5216
Fld_12 := Item_81.Discriminants_Values;
Inc_Ref (Fld_12);
--# expr-done 1832
Concat_Result_1 := Concat (Singleton, Fld_12); 
--# expr-done 1833
Inc_Ref (Concat_Result);
Inc_Ref (Concat_Result_1);
New_Struct_1 := (Components => Concat_Result, Discriminants_Values => Concat_Result_1); 
--# expr-done 1834
         declare
            Item_To_Append : constant Internal_Shape := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2095;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1835
         for Item_To_Append of
               Map_Result_1.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2094;
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 1836
Scope_Result := Map_Result_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2093;
   Result_Var := Scope_Result;
else
   --# expr-start 1837 '<New Shape at ast.py:5220>' New_Struct_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5220
Inc_Ref (Block_Var);
Inc_Ref (No_Internal_Discriminant_Values_Array_Type);
New_Struct_2 := (Components => Block_Var, Discriminants_Values => No_Internal_Discriminant_Values_Array_Type); 
--# expr-done 1837
                Singleton_1 := Create_Internal_Shape_Array (Items_Count => 1);
                Singleton_1.Items (1) := New_Struct_2;
                Inc_Ref (Singleton_1.Items (1));
   Result_Var := Singleton_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 1838
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_327;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_327;
                     Finalizer_Scope_2093;
                     Finalizer_Scope_2094;
                     Finalizer_Scope_2095;
            raise;
      end;
   return Property_Result;
end Component_List_P_Shapes;
--# end
   --
   --  Primitives for Bare_Discriminant_Part
   --
--# property-start DiscriminantPart.abstract_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4654
pragma Warnings (Off, "is not referenced");
 function Discriminant_Part_P_Abstract_Formal_Params
  (Node : Bare_Discriminant_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Discriminant_Part  := Bare_Discriminant_Part (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_281 with Inline_Always;
      Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_281 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_281;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1839 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type; Inc_Ref (Let_Result);
--# end
--# expr-done 1839
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_281;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_281;
            raise;
      end;
   return Property_Result;
end Discriminant_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Known_Discriminant_Part
   --
      procedure Initialize_Fields_For_Known_Discriminant_Part
        (Self : Bare_Known_Discriminant_Part
         ; Known_Discriminant_Part_F_Discr_Specs : Bare_Discriminant_Spec_List
        ) is
      begin
            Self.Known_Discriminant_Part_F_Discr_Specs := Known_Discriminant_Part_F_Discr_Specs;
      end Initialize_Fields_For_Known_Discriminant_Part;
   function Known_Discriminant_Part_F_Discr_Specs
     (Node : Bare_Known_Discriminant_Part) return Bare_Discriminant_Spec_List
   is
   begin
         return Node.Known_Discriminant_Part_F_Discr_Specs;
   end;
--# property-start KnownDiscriminantPart.abstract_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4666
pragma Warnings (Off, "is not referenced");
 function Known_Discriminant_Part_P_Abstract_Formal_Params
  (Node : Bare_Known_Discriminant_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Known_Discriminant_Part  := Bare_Known_Discriminant_Part (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_282 with Inline_Always;
      Fld : Bare_Discriminant_Spec_List;
Cast_Expr : Bare_Discriminant_Spec;
Cast_Result : Bare_Base_Formal_Param_Decl;
As_Entity : Internal_Entity_Base_Formal_Param_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_82 : Bare_Discriminant_Spec;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_282 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_282;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1844 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1843 '<Map at ast.py:4668>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4668
   --# expr-start 1840 '<FieldAccess for discr_specs at ast.py:4668>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4668
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Known_Discriminant_Part_F_Discr_Specs;
--# expr-done 1840
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Discriminant_Spec_List := Fld;
      begin
         for Untyped_Item_82 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_82 := Untyped_Item_82; 
   --# scope-start
               --# bind e Item_82
      --# expr-start 1842 '<as_entity at ast.py:4669>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4669
--# expr-start 1841 '<Cast to BaseFormalParamDecl at ast.py:4669>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4669
Cast_Expr := Item_82; 
      Cast_Result := Cast_Expr;
--# expr-done 1841
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1842
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := If_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1843
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1844
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_282;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_282;
            raise;
      end;
   return Property_Result;
end Known_Discriminant_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Unknown_Discriminant_Part
   --
   --
   --  Primitives for Bare_Entry_Completion_Formal_Params
   --
      procedure Initialize_Fields_For_Entry_Completion_Formal_Params
        (Self : Bare_Entry_Completion_Formal_Params
         ; Entry_Completion_Formal_Params_F_Params : Bare_Params
        ) is
      begin
            Self.Entry_Completion_Formal_Params_F_Params := Entry_Completion_Formal_Params_F_Params;
      end Initialize_Fields_For_Entry_Completion_Formal_Params;
   function Entry_Completion_Formal_Params_F_Params
     (Node : Bare_Entry_Completion_Formal_Params) return Bare_Params
   is
   begin
         return Node.Entry_Completion_Formal_Params_F_Params;
   end;
--# property-start EntryCompletionFormalParams.abstract_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21517
pragma Warnings (Off, "is not referenced");
 function Entry_Completion_Formal_Params_P_Abstract_Formal_Params
  (Node : Bare_Entry_Completion_Formal_Params
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Entry_Completion_Formal_Params  := Bare_Entry_Completion_Formal_Params (Node);
     Ent : Internal_Entity_Entry_Completion_Formal_Params :=
       Internal_Entity_Entry_Completion_Formal_Params'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1468 with Inline_Always;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Scope_Result : Internal_Entity_Param_Spec_List;
Result_Var : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Cast_Expr : Internal_Entity_Param_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_83 : Internal_Entity_Param_Spec;
Bare_Item_83 : Bare_Param_Spec;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1468 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1468;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1848 '<Map at ast.py:21518>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21518
--# expr-start 1845 '<FieldAccess for params at ast.py:21518>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21518
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Entry_Completion_Formal_Params_F_Params, Info => Ent.Info);
--# expr-done 1845
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
--# expr-start 1846 '<FieldAccess for params at ast.py:21518>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21518
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1846
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Param_Spec_List;
end if;
Fld_2 := Result_Var.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_83 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_83 := Untyped_Item_83; 
Is_Null := Bare_Item_83 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_83); 
   If_Result := As_Entity;
end if;
                  Item_83 := If_Result; 
   --# scope-start
               --# bind p Item_83
      --# expr-start 1847 '<Cast to BaseFormalParamDecl at ast.py:21518>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21518
Cast_Expr := Item_83; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 1847
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1848
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1468;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1468;
            raise;
      end;
   return Property_Result;
end Entry_Completion_Formal_Params_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Part
   --
      procedure Initialize_Fields_For_Generic_Formal_Part
        (Self : Bare_Generic_Formal_Part
         ; Generic_Formal_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Self.Generic_Formal_Part_F_Decls := Generic_Formal_Part_F_Decls;
      end Initialize_Fields_For_Generic_Formal_Part;
   function Generic_Formal_Part_F_Decls
     (Node : Bare_Generic_Formal_Part) return Bare_Ada_Node_List
   is
   begin
         return Node.Generic_Formal_Part_F_Decls;
   end;
--# property-start GenericFormalPart.abstract_formal_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11952
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Part_P_Abstract_Formal_Params
  (Node : Bare_Generic_Formal_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Generic_Formal_Part  := Bare_Generic_Formal_Part (Node);
     Ent : Internal_Entity_Generic_Formal_Part :=
       Internal_Entity_Generic_Formal_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_915 with Inline_Always;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_84 : Internal_Entity;
Bare_Item_84 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_915 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_915;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1852 '<FilterMap at ast.py:11952>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11952
   --# expr-start 1849 '<FieldAccess for decls at ast.py:11952>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11952
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Generic_Formal_Part_F_Decls, Info => Ent.Info);
--# expr-done 1849
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_84 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_84 := Untyped_Item_84; 
Is_Null := Bare_Item_84 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_84); 
   If_Result := As_Entity;
end if;
                  Item_84 := If_Result; 
   --# scope-start
               --# bind e Item_84
         --# expr-start 1850 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_84.Node /= null 
and then Item_84.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1850
         if Is_A then
      --# expr-start 1851 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_84; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1851
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1852
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_915;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_915;
            raise;
      end;
   return Property_Result;
end Generic_Formal_Part_P_Abstract_Formal_Params;
--# end
--# property-start GenericFormalPart.use_clauses_envs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11954
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Part_P_Use_Clauses_Envs
  (Node : Bare_Generic_Formal_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Formal_Part  := Bare_Generic_Formal_Part (Node);
     Ent : Internal_Entity_Generic_Formal_Part :=
       Internal_Entity_Generic_Formal_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_916 with Inline_Always;
            procedure Finalizer_Scope_2100 with Inline_Always;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Use_Clause;
Fld_3 : Lexical_Env := Empty_Env;
Item_85 : Internal_Entity;
Bare_Item_85 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_916 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_916;
            procedure Finalizer_Scope_2100 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2100;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1859 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1858 '<env_group at ast.py:11963>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11963
--# expr-start 1857 '<FilterMap at ast.py:11960>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11960
   --# expr-start 1853 '<FieldAccess for decls at ast.py:11960>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11960
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Generic_Formal_Part_F_Decls, Info => Ent.Info);
--# expr-done 1853
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_85 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_85 := Untyped_Item_85; 
Is_Null := Bare_Item_85 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_85); 
   If_Result := As_Entity;
end if;
                  Item_85 := If_Result; 
   --# scope-start
               --# bind u Item_85
         --# expr-start 1854 '<IsA UseClause at ast.py:11962>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11962
Is_A := Item_85.Node /= null 
and then Item_85.Node.Kind in Ada_Use_Clause; 
--# expr-done 1854
         if Is_A then
      --# expr-start 1856 '<FieldAccess for used_envs at ast.py:11961>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11961
--# expr-start 1855 '<Cast to UseClause at ast.py:11961>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11961
Cast_Expr := Item_85; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Use_Clause
   then
      Cast_Result := Create_Internal_Entity_Use_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Use_Clause;
   end if;
--# expr-done 1855
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.used_envs
Fld_3 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1856
         declare
            Item_To_Append : constant Lexical_Env := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
      Finalizer_Scope_2100;
         end loop;
      end;
      end if;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1857
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 1858
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 1859
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_916;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_916;
                     Finalizer_Scope_2100;
            raise;
      end;
   return Property_Result;
end Generic_Formal_Part_P_Use_Clauses_Envs;
--# end
   --
   --  Primitives for Bare_Base_Record_Def
   --
      procedure Initialize_Fields_For_Base_Record_Def
        (Self : Bare_Base_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Self.Base_Record_Def_F_Components := Base_Record_Def_F_Components;
      end Initialize_Fields_For_Base_Record_Def;
   function Base_Record_Def_F_Components
     (Node : Bare_Base_Record_Def) return Bare_Component_List
   is
   begin
         return Node.Base_Record_Def_F_Components;
   end;
--# property-start BaseRecordDef.comps /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5235
pragma Warnings (Off, "is not referenced");
 function Base_Record_Def_P_Comps
  (Node : Bare_Base_Record_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Component_List
is
   Self : Bare_Base_Record_Def  := Bare_Base_Record_Def (Node);
     Ent : Internal_Entity_Base_Record_Def :=
       Internal_Entity_Base_Record_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Component_List;
      Fld : Internal_Entity_Component_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1860 '<FieldAccess for components at ast.py:5235>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5235
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Component_List (Node => Ent.Node.Base_Record_Def_F_Components, Info => Ent.Info);
--# expr-done 1860
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Record_Def_P_Comps;
--# end
   --
   --  Primitives for Bare_Null_Record_Def
   --
      procedure Initialize_Fields_For_Null_Record_Def
        (Self : Bare_Null_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Initialize_Fields_For_Base_Record_Def
              (Self, Base_Record_Def_F_Components);
      end Initialize_Fields_For_Null_Record_Def;
   --
   --  Primitives for Bare_Record_Def
   --
      procedure Initialize_Fields_For_Record_Def
        (Self : Bare_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Initialize_Fields_For_Base_Record_Def
              (Self, Base_Record_Def_F_Components);
      end Initialize_Fields_For_Record_Def;
   --
   --  Primitives for Bare_Basic_Assoc
   --
--# property-start '[dispatcher]BasicAssoc.expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Assoc_P_Expr
  (Node : Bare_Basic_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Basic_Assoc  := Bare_Basic_Assoc (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Assoc (Self.Kind) is
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.expr
                  Property_Result := Aggregate_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Composite_Constraint_Assoc =>
                  --# property-call-start CompositeConstraintAssoc.expr
                  Property_Result := Composite_Constraint_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.expr
                  Property_Result := Iterated_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.expr
                  Property_Result := Param_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Assoc_P_Expr;
--# end
--# property-start '[dispatcher]BasicAssoc.names' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Assoc_P_Names
  (Node : Bare_Basic_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Assoc  := Bare_Basic_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Assoc (Self.Kind) is
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.names
                  Property_Result := Aggregate_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Composite_Constraint_Assoc =>
                  --# property-call-start CompositeConstraintAssoc.names
                  Property_Result := Composite_Constraint_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.names
                  Property_Result := Iterated_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.names
                  Property_Result := Param_Assoc_P_Names
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Assoc_P_Names;
--# end
--# property-start BasicAssoc.get_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8485
pragma Warnings (Off, "is not referenced");
 function Basic_Assoc_P_Get_Params
  (Node : Bare_Basic_Assoc
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Assoc  := Bare_Basic_Assoc (Node);
     Ent : Internal_Entity_Basic_Assoc :=
       Internal_Entity_Basic_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_619 with Inline_Always;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_1 : Internal_Param_Actual_Array_Access;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_619 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_619;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1869 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1868 '<FilterMap at ast.py:8493>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8493
   --# expr-start 1863 '<FieldAccess for zip_with_params at ast.py:8493>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8493
--# expr-start 1862 '<Cast to AssocList at ast.py:8493>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8493
--# expr-start 1861 '<parent at ast.py:8493>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8493
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1861
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1862
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.zip_with_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Cast_Result.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 1863
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Fld_1;
      begin
         for Item_86 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_86
         --# expr-start 1866 '<Eq at ast.py:8495>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8495
--# expr-start 1864 '<FieldAccess for actual at ast.py:8495>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8495
Fld_3 := Item_86.Actual;
--# expr-done 1864
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 1865 '<FieldAccess for expr at ast.py:8495>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8495
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1865
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1866
         if Is_Equiv then
      --# expr-start 1867 '<FieldAccess for param at ast.py:8494>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8494
Fld_2 := Item_86.Param;
--# expr-done 1867
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Fld_2;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1868
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1869
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_619;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_619;
            raise;
      end;
   return Property_Result;
end Basic_Assoc_P_Get_Params;
--# end
   --
   --  Primitives for Bare_Aggregate_Assoc
   --
      procedure Initialize_Fields_For_Aggregate_Assoc
        (Self : Bare_Aggregate_Assoc
         ; Aggregate_Assoc_F_Designators : Bare_Alternatives_List
         ; Aggregate_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Aggregate_Assoc_F_Designators := Aggregate_Assoc_F_Designators;
            Self.Aggregate_Assoc_F_R_Expr := Aggregate_Assoc_F_R_Expr;
      end Initialize_Fields_For_Aggregate_Assoc;
   function Aggregate_Assoc_F_Designators
     (Node : Bare_Aggregate_Assoc) return Bare_Alternatives_List
   is
   begin
         return Node.Aggregate_Assoc_F_Designators;
   end;
   function Aggregate_Assoc_F_R_Expr
     (Node : Bare_Aggregate_Assoc) return Bare_Expr
   is
   begin
         return Node.Aggregate_Assoc_F_R_Expr;
   end;
--# property-start AggregateAssoc.expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15525
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Expr
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1870 '<FieldAccess for r_expr at ast.py:15525>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15525
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Aggregate_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 1870
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Expr;
--# end
--# property-start AggregateAssoc.names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15526
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Names
  (Node : Bare_Aggregate_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1104 with Inline_Always;
      Fld : Bare_Alternatives_List;
Item_87 : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1104 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1104;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1872 '<Map at ast.py:15526>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15526
   --# expr-start 1871 '<FieldAccess for designators at ast.py:15526>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15526
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Aggregate_Assoc_F_Designators;
--# expr-done 1871
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld;
      begin
         for Untyped_Item_87 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_87 := Untyped_Item_87; 
   --# scope-start
               --# bind d Item_87
         declare
            Item_To_Append : constant Bare_Ada_Node := Item_87;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1872
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1104;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1104;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Names;
--# end
--# property-start AggregateAssoc.xref_stop_resolution /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15528
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Xref_Stop_Resolution
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Xref_Stop_Resolution;
--# end
--# property-start AggregateAssoc.base_aggregate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15530
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Base_Aggregate
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Aggregate
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Aggregate;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Aggregate;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1875 '<Cast to BaseAggregate at ast.py:15531>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15531
--# expr-start 1874 '<parent at ast.py:15531>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15531
--# expr-start 1873 '<parent at ast.py:15531>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15531
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1873
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1874
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1875
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Base_Aggregate;
--# end
--# property-start AggregateAssoc.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15534
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Xref_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1107 with Inline_Always;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Multidim_Aggregate_Info;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Aggregate;
Fld_4 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Sym : Symbol_Type;
Fld_6 : Boolean;
Sym_1 : Symbol_Type;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Logic_Equation;
Sym_2 : Symbol_Type;
Fld_9 : Boolean;
Sym_3 : Symbol_Type;
Fld_10 : Boolean;
If_Result_2 : Boolean;
Fld_11 : Logic_Equation;
Sym_4 : Symbol_Type;
Fld_12 : Boolean;
Fld_13 : Logic_Equation;
Sym_5 : Symbol_Type;
Fld_14 : Boolean;
Fld_15 : Logic_Equation;
Sym_6 : Symbol_Type;
Fld_16 : Boolean;
Fld_17 : Logic_Equation;
Fld_18 : Internal_Entity;
Is_A : Boolean;
True_Rel : Logic_Equation;
Is_Null : Boolean;
Fld_19 : Logic_Equation;
Fld_20 : Logic_Equation;
If_Result_3 : Logic_Equation;
If_Result_4 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
If_Result_7 : Logic_Equation;
If_Result_8 : Logic_Equation;
If_Result_9 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1107 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_17);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1107;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1911 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1876 '<FieldAccess for base_aggregate at ast.py:15536>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15536
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1876
Block_Var := Fld; 
--# bind agg Block_Var
--# expr-start 1877 '<FieldAccess for multidim_root_aggregate at ast.py:15538>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15538
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1877
Block_Var_1 := Fld_1; 
--# bind mra Block_Var_1
--# expr-start 1884 '<If at ast.py:15543>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15543
--# expr-start 1879 '<Not at ast.py:15544>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15544
--# expr-start 1878 '<IsNull at ast.py:15544>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15544
Is_Equal := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 1878
Not_Val := not (Is_Equal); 
--# expr-done 1879
if Not_Val then
   --# expr-start 1880 '<FieldAccess for typ at ast.py:15545>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15545
Fld_2 := Block_Var_1.Typ;
--# expr-done 1880
   If_Result := Fld_2;
else
   --# expr-start 1883 '<Cast to BaseTypeDecl at ast.py:15546>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15546
--# expr-start 1882 '<FieldAccess for type_val at ast.py:15546>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15546
--# expr-start 1881 '<FieldAccess for base_aggregate at ast.py:15546>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15546
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld_3 := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1881
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_4 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_3.Node);
--# end
--# expr-done 1882
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1883
   If_Result := Cast_Result;
end if;
--# expr-done 1884
Block_Var_2 := If_Result; 
--# bind td Block_Var_2
Var_Expr := Block_Var_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1885 '<FieldAccess for array_def at ast.py:15549>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15549
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1885
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var_3 := Result_Var; 
--# bind atd Block_Var_3
--# expr-start 1910 '<Cond at ast.py:15551>' If_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15551
--# expr-start 1887 '<FieldAccess for in_aspect at ast.py:15552>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15552
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1886 '<Symbol Global at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Global); 
--# expr-done 1886
--# property-call-start AdaNode.in_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym);
--# end
--# expr-done 1887
if Fld_6 then
   If_Result_1 := True;
else
   --# expr-start 1889 '<FieldAccess for in_aspect at ast.py:15552>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15552
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1888 '<Symbol Refined_Global at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Global); 
--# expr-done 1888
--# property-call-start AdaNode.in_aspect
Fld_7 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_1);
--# end
--# expr-done 1889
   If_Result_1 := Fld_7;
end if;
if If_Result_1 then
   --# expr-start 1890 '<FieldAccess for globals_assoc_equation at ast.py:15553>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15553
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.globals_assoc_equation
Fld_8 := Libadalang.Implementation.Aggregate_Assoc_P_Globals_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1890
   If_Result_9 := Fld_8;
else
--# expr-start 1892 '<FieldAccess for in_aspect at ast.py:15555>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15555
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1891 '<Symbol Depends at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Depends); 
--# expr-done 1891
--# property-call-start AdaNode.in_aspect
Fld_9 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_2);
--# end
--# expr-done 1892
if Fld_9 then
   If_Result_2 := True;
else
   --# expr-start 1894 '<FieldAccess for in_aspect at ast.py:15555>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15555
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1893 '<Symbol Refined_Depends at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Depends); 
--# expr-done 1893
--# property-call-start AdaNode.in_aspect
Fld_10 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_3);
--# end
--# expr-done 1894
   If_Result_2 := Fld_10;
end if;
if If_Result_2 then
   --# expr-start 1895 '<FieldAccess for depends_assoc_equation at ast.py:15556>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15556
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.depends_assoc_equation
Fld_11 := Libadalang.Implementation.Aggregate_Assoc_P_Depends_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1895
   If_Result_8 := Fld_11;
else
--# expr-start 1897 '<FieldAccess for in_aspect at ast.py:15558>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15558
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1896 '<Symbol Test_Case at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 1896
--# property-call-start AdaNode.in_aspect
Fld_12 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_4);
--# end
--# expr-done 1897
if Fld_12 then
   --# expr-start 1898 '<FieldAccess for test_case_assoc_equation at ast.py:15559>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15559
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.test_case_assoc_equation
Fld_13 := Libadalang.Implementation.Aggregate_Assoc_P_Test_Case_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1898
   If_Result_7 := Fld_13;
else
--# expr-start 1900 '<FieldAccess for in_aspect at ast.py:15561>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15561
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1899 '<Symbol Refined_State at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_State); 
--# expr-done 1899
--# property-call-start AdaNode.in_aspect
Fld_14 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_5);
--# end
--# expr-done 1900
if Fld_14 then
   --# expr-start 1901 '<FieldAccess for exprs_assoc_equation at ast.py:15564>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15564
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.exprs_assoc_equation
Fld_15 := Libadalang.Implementation.Aggregate_Assoc_P_Exprs_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1901
   If_Result_6 := Fld_15;
else
--# expr-start 1903 '<FieldAccess for in_aspect at ast.py:15566>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15566
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1902 '<Symbol Contract_Cases at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 1902
--# property-call-start AdaNode.in_aspect
Fld_16 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_6);
--# end
--# expr-done 1903
if Fld_16 then
   --# expr-start 1904 '<FieldAccess for contract_cases_assoc_equation at ast.py:15567>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15567
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.contract_cases_assoc_equation
Fld_17 := Libadalang.Implementation.Aggregate_Assoc_P_Contract_Cases_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1904
   If_Result_5 := Fld_17;
else
--# expr-start 1906 '<IsA AspectClause, AspectAssoc, PragmaArgumentAssoc at ast.py:15569>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15569
--# expr-start 1905 '<parent at ast.py:15569>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15569
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 1905
Is_A := Fld_18.Node /= null 
and then Fld_18.Node.Kind in Ada_Aspect_Clause | Ada_Aspect_Assoc_Range | Ada_Pragma_Argument_Assoc_Range; 
--# expr-done 1906
if Is_A then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15570") else null)); 
   If_Result_4 := True_Rel;
else
--# expr-start 1907 '<IsNull at ast.py:15572>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15572
Is_Null := Block_Var_3.Node = null; 
--# expr-done 1907
if Is_Null then
   --# expr-start 1908 '<FieldAccess for record_assoc_equation at ast.py:15573>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15573
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.record_assoc_equation
Fld_19 := Libadalang.Implementation.Aggregate_Assoc_P_Record_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1908
   If_Result_3 := Fld_19;
else
   --# expr-start 1909 '<FieldAccess for array_assoc_equation at ast.py:15574>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15574
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.array_assoc_equation
Fld_20 := Libadalang.Implementation.Aggregate_Assoc_P_Array_Assoc_Equation (Node => Ent.Node, Atd => Block_Var_3, Mra => Block_Var_1, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1909
   If_Result_3 := Fld_20;
end if;
      Inc_Ref (If_Result_3);
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
   If_Result_9 := If_Result_8;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 1910
Let_Result := If_Result_9; Inc_Ref (Let_Result);
--# end
--# expr-done 1911
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1107;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1107;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Xref_Equation;
--# end
--# property-start AggregateAssoc.record_assoc_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15577
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Record_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1108 with Inline_Always;
            procedure Finalizer_Scope_2107 with Inline_Always;
            procedure Finalizer_Scope_2108 with Inline_Always;
            procedure Finalizer_Scope_2109 with Inline_Always;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Param_Match_Array_Access;
Block_Var_2 : Internal_Param_Match_Array_Access;
Block_Var_3 : Boolean;
Var_Expr : Bare_Base_Id;
Var_Expr_1 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Param_Match_Array_Access;
Fld_2 : Internal_Single_Actual;
Fld_3 : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Aggregate_Assoc;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Single_Actual;
Fld_6 : Internal_Entity_Basic_Assoc;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Aggregate_Assoc;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Map_Result_1 : Internal_Param_Match_Array_Access;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Param_Match_Array_Access;
Fld_7 : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
Fld_8 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Var;
Fld_12 : Internal_Entity_Defining_Name;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl;
Fld_14 : Internal_Entity_Type_Expr;
Fld_15 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_16 : Internal_Single_Actual;
Fld_17 : Internal_Entity_Basic_Assoc;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_20 : Internal_Single_Actual;
Fld_21 : Internal_Entity_Basic_Assoc;
Fld_22 : Internal_Entity_Expr;
Fld_23 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_24 : Internal_Single_Actual;
Fld_25 : Bare_Base_Id;
Fld_26 : Logic_Var;
Fld_27 : Internal_Entity_Defining_Name;
Fld_28 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_29 : Internal_Entity_Defining_Name;
Fld_30 : Internal_Entity_Expr;
Fld_31 : Logic_Var;
Fld_32 : Internal_Entity_Base_Formal_Param_Decl;
Fld_33 : Internal_Entity_Type_Expr;
Fld_34 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_35 : Internal_Entity_Expr;
Fld_36 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_37 : Internal_Entity_Expr;
Fld_38 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Not_Val_1 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1108 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1108;
            procedure Finalizer_Scope_2107 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2107;
            procedure Finalizer_Scope_2108 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2108;
            procedure Finalizer_Scope_2109 is
            begin
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_36);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_38);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2109;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1972 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 1912 '<FieldAccess for base_aggregate at ast.py:15583>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15583
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1912
Block_Var := Fld; 
--# bind agg Block_Var
--# expr-start 1917 '<Filter at ast.py:15586>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15586
   --# expr-start 1913 '<FieldAccess for matched_discriminants at ast.py:15586>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15586
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.matched_discriminants
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Matched_Discriminants (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1913
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_1;
      begin
         for Item_88 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_88
         --# expr-start 1916 '<Eq at ast.py:15587>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15587
--# expr-start 1915 '<FieldAccess for assoc at ast.py:15587>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15587
--# expr-start 1914 '<FieldAccess for actual at ast.py:15587>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15587
Fld_2 := Item_88.Actual;
--# expr-done 1914
Fld_3 := Fld_2.Assoc;
--# expr-done 1915
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1916
         if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_88;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1917
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind discr_matches Block_Var_1
--# expr-start 1925 '<If at ast.py:15599>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15599
--# expr-start 1919 '<Not at ast.py:15600>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15600
--# expr-start 1918 '<IsNull at ast.py:15600>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15600
Is_Equal := Equivalent (Block_Var_1, No_Internal_Param_Match_Array_Type); 
--# expr-done 1918
Not_Val := not (Is_Equal); 
--# expr-done 1919
if Not_Val then
   If_Result := Block_Var_1;
else
   --# expr-start 1924 '<Filter at ast.py:15602>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15602
   --# expr-start 1920 '<FieldAccess for matched_components at ast.py:15602>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15602
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.matched_components
Fld_4 := Libadalang.Implementation.Base_Aggregate_P_Matched_Components (Node => Block_Var.Node, Origin => Origin, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 1920
      declare
         Map_Result_1_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_4;
      begin
         for Item_89 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_89
         --# expr-start 1923 '<Eq at ast.py:15603>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15603
--# expr-start 1922 '<FieldAccess for assoc at ast.py:15603>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15603
--# expr-start 1921 '<FieldAccess for actual at ast.py:15603>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15603
Fld_5 := Item_89.Actual;
--# expr-done 1921
Fld_6 := Fld_5.Assoc;
--# expr-done 1922
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 1923
         if Is_Equiv_1 then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_89;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1924
   If_Result := Map_Result_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 1925
Block_Var_2 := If_Result; Inc_Ref (Block_Var_2);
--# bind matches Block_Var_2
--# expr-start 1928 '<AnyQuantifier at ast.py:15608>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15608
   --# expr-start 1926 '<FieldAccess for names at ast.py:15608>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15608
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1926
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_7;
      begin
         for Item_90 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_90
      --# expr-start 1927 '<IsA OthersDesignator at ast.py:15609>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15609
Is_A := Item_90 /= null 
and then Item_90.Kind in Ada_Others_Designator_Range; 
--# expr-done 1927
      Quantifier_Result := Is_A;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 1928
Block_Var_3 := Quantifier_Result; 
--# bind is_others_assoc Block_Var_3
--# expr-start 1971 '<If at ast.py:15612>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15612
--# expr-start 1929 '<Not at ast.py:15613>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15613
Not_Val_1 := not (Block_Var_3); 
--# expr-done 1929
if Not_Val_1 then
   --# expr-start 1957 '<LogicAll at ast.py:15615>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15615
--# expr-start 1956 '<Map at ast.py:15615>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15615
      declare
         Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Block_Var_2;
      begin
         for Item_91 of
               Collection.Items
         loop
   --# scope-start
               --# bind match Item_91
      --# expr-start 1955 '<And at ast.py:15615>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15615
--# expr-start 1948 '<And at ast.py:15615>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15615
--# expr-start 1943 '<And at ast.py:15615>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15615
--# expr-start 1938 '<Bind at ast.py:15616>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15616
--# expr-start 1933 '<FieldAccess for expected_type_var at ast.py:15617>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15617
--# expr-start 1932 '<FieldAccess for expr at ast.py:15617>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15617
--# expr-start 1931 '<FieldAccess for assoc at ast.py:15617>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15617
--# expr-start 1930 '<FieldAccess for actual at ast.py:15617>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15617
Fld_8 := Item_91.Actual;
--# expr-done 1930
Fld_9 := Fld_8.Assoc;
--# expr-done 1931
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 1932
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_10.Node);
--# expr-done 1933
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 1937 '<FieldAccess for designated_type at ast.py:15618>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15618
--# expr-start 1936 '<FieldAccess for type_expression at ast.py:15618>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15618
--# expr-start 1935 '<FieldAccess for formal_decl at ast.py:15618>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15618
--# expr-start 1934 '<FieldAccess for formal at ast.py:15618>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15618
Fld_12 := Item_91.Formal;
--# expr-done 1934
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_13 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_12.Node, E_Info => Fld_12.Info);
--# end
--# expr-done 1935
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 1936
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_14.Node, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 1937
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
Bind_Result := Solver.Create_Assign (Fld_11, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15616") else null)); 
--# expr-done 1938
--# expr-start 1942 '<FieldAccess for sub_equation at ast.py:15620>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15620
--# expr-start 1941 '<FieldAccess for expr at ast.py:15620>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15620
--# expr-start 1940 '<FieldAccess for assoc at ast.py:15620>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15620
--# expr-start 1939 '<FieldAccess for actual at ast.py:15620>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15620
Fld_16 := Item_91.Actual;
--# expr-done 1939
Fld_17 := Fld_16.Assoc;
--# expr-done 1940
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_17.Node, E_Info => Fld_17.Info);
--# end
--# expr-done 1941
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_19 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_18.Node, Env => Env, Origin => Origin, E_Info => Fld_18.Info);
--# end
--# expr-done 1942
And_Pred := Create_And (Bind_Result, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15615") else null)); 
--# expr-done 1943
--# expr-start 1947 '<FieldAccess for matches_expected_assign_type at ast.py:15621>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15621
--# expr-start 1946 '<FieldAccess for expr at ast.py:15621>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15621
--# expr-start 1945 '<FieldAccess for assoc at ast.py:15621>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15621
--# expr-start 1944 '<FieldAccess for actual at ast.py:15621>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15621
Fld_20 := Item_91.Actual;
--# expr-done 1944
Fld_21 := Fld_20.Assoc;
--# expr-done 1945
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_22 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_21.Node, E_Info => Fld_21.Info);
--# end
--# expr-done 1946
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_23 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_22.Node, Origin => Origin);
--# end
--# expr-done 1947
And_Pred_1 := Create_And (And_Pred, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15615") else null)); 
--# expr-done 1948
--# expr-start 1950 '<FieldAccess for name at ast.py:15622>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15622
--# expr-start 1949 '<FieldAccess for actual at ast.py:15622>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15622
Fld_24 := Item_91.Actual;
--# expr-done 1949
Fld_25 := Fld_24.Name;
--# expr-done 1950
Var_Expr := Fld_25;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 1954 '<Bind at ast.py:15623>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15623
--# expr-start 1951 '<FieldAccess for ref_var at ast.py:15623>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15623
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_26 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr);
--# end
--# expr-done 1951
Fld_26.Value := No_Entity;
Entity_Vars.Reset (Fld_26);
--# expr-start 1953 '<FieldAccess for formal_decl at ast.py:15623>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15623
--# expr-start 1952 '<FieldAccess for formal at ast.py:15623>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15623
Fld_27 := Item_91.Formal;
--# expr-done 1952
      if Fld_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_28 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_27.Node, E_Info => Fld_27.Info);
--# end
--# expr-done 1953
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_26, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15623") else null)); 
--# expr-done 1954
Scope_Result := Bind_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2108;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15624") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (And_Pred_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15615") else null)); 
--# expr-done 1955
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2107;
         end loop;
      end;
         Map_Result_2 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_2_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 1956
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15615") else null)); 
--# expr-done 1957
   If_Result_1 := Logic_Boolean_Op;
else
--# expr-start 1958 '<FieldAccess for first_unmatched_formal at ast.py:15632>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15632
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.first_unmatched_formal
Fld_29 := Libadalang.Implementation.Base_Aggregate_P_First_Unmatched_Formal (Node => Block_Var.Node, Origin => Origin, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 1958
Var_Expr_1 := Fld_29;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind unmatched_formal Var_Expr_1
--# expr-start 1970 '<And at ast.py:15634>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15634
--# expr-start 1967 '<And at ast.py:15634>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15634
--# expr-start 1964 '<Bind at ast.py:15634>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15634
--# expr-start 1960 '<FieldAccess for expected_type_var at ast.py:15635>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15635
--# expr-start 1959 '<FieldAccess for expr at ast.py:15635>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15635
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1959
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_30.Node);
--# expr-done 1960
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
--# expr-start 1963 '<FieldAccess for designated_type at ast.py:15637>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15637
--# expr-start 1962 '<FieldAccess for type_expression at ast.py:15637>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15637
--# expr-start 1961 '<FieldAccess for formal_decl at ast.py:15636>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15636
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_32 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1961
      if Fld_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_33 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_32.Node, E_Info => Fld_32.Info);
--# end
--# expr-done 1962
      if Fld_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_34 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_33.Node, Origin => Origin, E_Info => Fld_33.Info);
--# end
--# expr-done 1963
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_34.Node,
         Info => Fld_34.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_31, Cast_Result_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15634") else null)); 
--# expr-done 1964
--# expr-start 1966 '<FieldAccess for sub_equation at ast.py:15639>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15639
--# expr-start 1965 '<FieldAccess for expr at ast.py:15639>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15639
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_35 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1965
      if Fld_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_36 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_35.Node, Env => Env, Origin => Origin, E_Info => Fld_35.Info);
--# end
--# expr-done 1966
And_Pred_3 := Create_And (Bind_Result_2, Fld_36, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15634") else null)); 
--# expr-done 1967
--# expr-start 1969 '<FieldAccess for matches_expected_type at ast.py:15640>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15640
--# expr-start 1968 '<FieldAccess for expr at ast.py:15640>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15640
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_37 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1968
      if Fld_37.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_38 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_37.Node, Origin => Origin);
--# end
--# expr-done 1969
And_Pred_4 := Create_And (And_Pred_3, Fld_38, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15634") else null)); 
--# expr-done 1970
Scope_Result_1 := And_Pred_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2109;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15641") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_1 := Result_Var_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1971
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1972
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1108;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1108;
                     Finalizer_Scope_2107;
                     Finalizer_Scope_2108;
                     Finalizer_Scope_2109;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Record_Assoc_Equation;
--# end
--# property-start AggregateAssoc.array_assoc_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15645
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Array_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Atd : Internal_Entity_Array_Type_Def
      ; Mra : Internal_Multidim_Aggregate_Info
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind atd Atd
   --# bind mra Mra
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1109 with Inline_Always;
            procedure Finalizer_Scope_2110 with Inline_Always;
            procedure Finalizer_Scope_2111 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel : Logic_Equation;
Is_Equal : Boolean;
Fld_7 : Integer;
Fld_8 : Integer;
Arith_Result : Integer;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Fld_9 : Internal_Entity_Alternatives_List;
Fld_10 : Bare_Alternatives_List;
Fld_11 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_2 : Internal_Entity;
Fld_12 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Name;
Fld_13 : Logic_Equation;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Fld_14 : Logic_Var;
Fld_15 : Integer;
Fld_16 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_17 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var : Logic_Equation;
Cast_Expr_2 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Name;
Fld_18 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result_3 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Item_92 : Internal_Entity;
Bare_Item_92 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1109 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1109;
            procedure Finalizer_Scope_2110 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2110;
            procedure Finalizer_Scope_2111 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2111;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2009 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2008 '<And at ast.py:15652>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15652
--# expr-start 1988 '<If at ast.py:15653>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15653
--# expr-start 1973 '<IsNull at ast.py:15656>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15656
Is_Equal := Mra = No_Multidim_Aggregate_Info; 
--# expr-done 1973
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1977 '<Eq at ast.py:15656>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15656
--# expr-start 1974 '<FieldAccess for rank at ast.py:15656>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15656
Fld_7 := Mra.Rank;
--# expr-done 1974
--# expr-start 1976 '<Arithmetic - at ast.py:15656>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15656
--# expr-start 1975 '<FieldAccess for array_ndims at ast.py:15656>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15656
      if Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_8 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Atd.Node, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 1975
Arith_Result := (Fld_8 - 1); 
--# expr-done 1976
Is_Equal_1 := Fld_7 = Arith_Result; 
--# expr-done 1977
   If_Result := Is_Equal_1;
end if;
if If_Result then
   --# expr-start 1987 '<And at ast.py:15659>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15659
--# expr-start 1984 '<And at ast.py:15659>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15659
--# expr-start 1981 '<Bind at ast.py:15659>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15659
--# expr-start 1979 '<FieldAccess for expected_type_var at ast.py:15659>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15659
--# expr-start 1978 '<FieldAccess for expr at ast.py:15659>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15659
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1978
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 1979
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 1980 '<FieldAccess for comp_type at ast.py:15659>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15659
      if Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_2 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Atd.Node, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 1980
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15659") else null)); 
--# expr-done 1981
--# expr-start 1983 '<FieldAccess for sub_equation at ast.py:15660>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15660
--# expr-start 1982 '<FieldAccess for expr at ast.py:15660>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15660
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1982
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 1983
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15659") else null)); 
--# expr-done 1984
--# expr-start 1986 '<FieldAccess for matches_expected_type at ast.py:15661>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15661
--# expr-start 1985 '<FieldAccess for expr at ast.py:15661>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15661
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1985
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 1986
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15659") else null)); 
--# expr-done 1987
   If_Result_1 := And_Pred_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15665") else null)); 
   If_Result_1 := True_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1988
--# expr-start 2007 '<LogicAll at ast.py:15668>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15668
--# expr-start 2006 '<Map at ast.py:15668>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15668
   --# expr-start 1989 '<FieldAccess for designators at ast.py:15668>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15668
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Aggregate_Assoc_F_Designators, Info => Ent.Info);
--# expr-done 1989
Fld_10 := Fld_9.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_10 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_10;
      begin
         for Untyped_Item_92 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_92 := Untyped_Item_92; 
Is_Null := Bare_Item_92 = null; 
if Is_Null then
   If_Result_2 := No_Entity;
else
Fld_11 := Fld_9.Info;
As_Entity := (Info => Fld_11, Node => Bare_Item_92); 
   If_Result_2 := As_Entity;
end if;
                  Item_92 := If_Result_2; 
   --# scope-start
               --# bind n Item_92
      --# expr-start 2005 '<And at ast.py:15669>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15669
--# expr-start 1990 '<FieldAccess for sub_equation at ast.py:15669>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15669
      if Item_92.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_92.Node, Env => Env, Origin => Origin, E_Info => Item_92.Info);
--# end
--# expr-done 1990
--# expr-start 2004 '<If at ast.py:15669>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15669
--# expr-start 1994 '<Not at ast.py:15670>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15670
--# expr-start 1993 '<IsNull at ast.py:15670>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15670
--# expr-start 1991 '<Cast to Name at ast.py:15670>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15670
Cast_Expr_2 := Item_92; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Name
   then
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_4 := No_Entity_Name;
   end if;
--# expr-done 1991
Var_Expr := Cast_Result_4;
if Var_Expr /= No_Entity_Name then
   --# scope-start
--# expr-start 1992 '<FieldAccess for name_designated_type at ast.py:15670>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15670
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_18 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1992
Scope_Result_1 := Fld_18;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Is_Null_1 := Result_Var_1.Node = null; 
--# expr-done 1993
Not_Val := not (Is_Null_1); 
--# expr-done 1994
if Not_Val then
   --# expr-start 1996 '<FieldAccess for xref_no_overloading at ast.py:15672>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15672
--# expr-start 1995 '<Cast to Name at ast.py:15672>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15672
Cast_Expr := Item_92; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 1995
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_13 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 1996
   If_Result_3 := Fld_13;
else
--# expr-start 1997 '<Cast to Expr at ast.py:15674>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15674
Cast_Expr_1 := Item_92; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Expr
   then
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Expr;
   end if;
--# expr-done 1997
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind n Var_Expr_1
--# expr-start 2003 '<And at ast.py:15676>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15676
--# expr-start 2001 '<Bind at ast.py:15676>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15676
--# expr-start 1998 '<FieldAccess for expected_type_var at ast.py:15676>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15676
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 1998
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 2000 '<FieldAccess for index_type at ast.py:15676>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15676
      if Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1999 '<FieldAccess for rank at ast.py:15676>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15676
Fld_15 := Mra.Rank;
--# expr-done 1999
--# property-call-start ArrayTypeDef.index_type
Fld_16 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Atd.Node, Dim => Fld_15, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 2000
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_14, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15676") else null)); 
--# expr-done 2001
--# expr-start 2002 '<FieldAccess for matches_expected_type at ast.py:15677>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15677
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_17 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr_1.Node, Origin => Origin);
--# end
--# expr-done 2002
And_Pred_2 := Create_And (Bind_Result_1, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15676") else null)); 
--# expr-done 2003
Scope_Result := And_Pred_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2111;
   Result_Var := Scope_Result;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15678") else null)); 
   Result_Var := True_Rel_1;
end if;
      Inc_Ref (Result_Var);
   If_Result_3 := Result_Var;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 2004
And_Pred_3 := Create_And (Fld_12, If_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15669") else null)); 
--# expr-done 2005
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2110;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2006
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15668") else null)); 
--# expr-done 2007
And_Pred_4 := Create_And (If_Result_1, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15652") else null)); 
--# expr-done 2008
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 2009
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1109;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1109;
                     Finalizer_Scope_2110;
                     Finalizer_Scope_2111;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Array_Assoc_Equation;
--# end
--# property-start AggregateAssoc.globals_assoc_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15684
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Globals_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1110 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1110 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1110;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2012 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2011 '<FieldAccess for sub_equation at ast.py:15693>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15693
--# expr-start 2010 '<FieldAccess for expr at ast.py:15693>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15693
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2010
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2011
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2012
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1110;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1110;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Globals_Assoc_Equation;
--# end
--# property-start AggregateAssoc.depends_assoc_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15695
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Depends_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1111 with Inline_Always;
            procedure Finalizer_Scope_2113 with Inline_Always;
      Var_Expr : Internal_Entity_Identifier;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Identifier;
As_Entity : Internal_Entity_Identifier;
Is_Null : Boolean;
If_Result : Internal_Entity_Identifier;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1111 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1111;
            procedure Finalizer_Scope_2113 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2113;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2021 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2020 '<And at ast.py:15701>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15701
--# expr-start 2014 '<FieldAccess for sub_equation at ast.py:15706>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15706
--# expr-start 2013 '<FieldAccess for expr at ast.py:15706>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15706
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2013
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2014
--# expr-start 2018 '<as_entity at ast.py:15714>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15714
--# expr-start 2017 '<Cast to Identifier at ast.py:15714>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15714
--# expr-start 2016 '<at at ast.py:15714>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15714
--# expr-start 2015 '<FieldAccess for names at ast.py:15714>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15714
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2015
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 2016
   if Get_Result = null
      or else Get_Result.Kind in Ada_Identifier_Range
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2017
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Identifier;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2018
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Identifier then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 2019 '<FieldAccess for sub_equation at ast.py:15715>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15715
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2019
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2113;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15715") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15701") else null)); 
--# expr-done 2020
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 2021
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1111;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1111;
                     Finalizer_Scope_2113;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Depends_Assoc_Equation;
--# end
--# property-start AggregateAssoc.test_case_assoc_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15719
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Test_Case_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1112 with Inline_Always;
      Var_Expr : Bare_Base_Id;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_2 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Base_Id;
Fld_3 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_4 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1112 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1112;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2034 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2033 '<If at ast.py:15725>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15725
--# expr-start 2024 '<Cast to BaseId at ast.py:15728>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15728
--# expr-start 2023 '<at at ast.py:15728>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15728
--# expr-start 2022 '<FieldAccess for names at ast.py:15728>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15728
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2022
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 2023
   if Get_Result = null
      or else Get_Result.Kind in Ada_Base_Id
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2024
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 2027 '<Eq at ast.py:15729>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15729
--# expr-start 2025 '<FieldAccess for name_symbol at ast.py:15729>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15729
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr);
--# end
--# expr-done 2025
--# expr-start 2026 '<Symbol Requires at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Requires); 
--# expr-done 2026
Is_Equal := Fld_3 = Sym; 
--# expr-done 2027
if Is_Equal then
   If_Result := True;
else
   --# expr-start 2030 '<Eq at ast.py:15729>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15729
--# expr-start 2028 '<FieldAccess for name_symbol at ast.py:15729>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15729
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr);
--# end
--# expr-done 2028
--# expr-start 2029 '<Symbol Ensures at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ensures); 
--# expr-done 2029
Is_Equal_1 := Fld_4 = Sym_1; 
--# expr-done 2030
   If_Result := Is_Equal_1;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 2032 '<FieldAccess for sub_equation at ast.py:15733>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15733
--# expr-start 2031 '<FieldAccess for expr at ast.py:15733>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15733
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2031
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2032
   If_Result_1 := Fld_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15734") else null)); 
   If_Result_1 := True_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2033
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2034
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1112;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1112;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Test_Case_Assoc_Equation;
--# end
--# property-start AggregateAssoc.contract_cases_assoc_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15737
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Contract_Cases_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1113 with Inline_Always;
            procedure Finalizer_Scope_2115 with Inline_Always;
            procedure Finalizer_Scope_2116 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Alternatives_List;
Fld_1 : Bare_Alternatives_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Item_93 : Internal_Entity;
Bare_Item_93 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1113 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1113;
            procedure Finalizer_Scope_2115 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2115;
            procedure Finalizer_Scope_2116 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2116;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2057 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2056 '<And at ast.py:15744>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15744
--# expr-start 2053 '<And at ast.py:15744>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15744
--# expr-start 2050 '<And at ast.py:15744>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15744
--# expr-start 2045 '<LogicAll at ast.py:15745>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15745
--# expr-start 2044 '<Map at ast.py:15745>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15745
   --# expr-start 2035 '<FieldAccess for designators at ast.py:15745>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15745
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Aggregate_Assoc_F_Designators, Info => Ent.Info);
--# expr-done 2035
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_1;
      begin
         for Untyped_Item_93 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_93 := Untyped_Item_93; 
Is_Null := Bare_Item_93 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_93); 
   If_Result := As_Entity;
end if;
                  Item_93 := If_Result; 
   --# scope-start
               --# bind d Item_93
--# expr-start 2036 '<Cast to Expr at ast.py:15746>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15746
Cast_Expr := Item_93; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Expr;
   end if;
--# expr-done 2036
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 2043 '<And at ast.py:15748>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15748
--# expr-start 2041 '<And at ast.py:15748>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15748
--# expr-start 2039 '<Bind at ast.py:15748>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15748
--# expr-start 2037 '<FieldAccess for expected_type_var at ast.py:15748>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15748
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 2037
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 2038 '<FieldAccess for bool_type at ast.py:15748>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15748
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 2038
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15748") else null)); 
--# expr-done 2039
--# expr-start 2040 '<FieldAccess for sub_equation at ast.py:15749>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15749
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2040
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15748") else null)); 
--# expr-done 2041
--# expr-start 2042 '<FieldAccess for matches_expected_formal_prim_type at ast.py:15750>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15750
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 2042
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15748") else null)); 
--# expr-done 2043
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2116;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15753") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2115;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2044
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15745") else null)); 
--# expr-done 2045
--# expr-start 2049 '<Bind at ast.py:15756>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15756
--# expr-start 2047 '<FieldAccess for expected_type_var at ast.py:15756>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15756
--# expr-start 2046 '<FieldAccess for expr at ast.py:15756>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15756
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2046
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_7.Node);
--# expr-done 2047
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 2048 '<FieldAccess for bool_type at ast.py:15756>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15756
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_9 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 2048
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_8, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15756") else null)); 
--# expr-done 2049
And_Pred_2 := Create_And (Logic_Boolean_Op, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15744") else null)); 
--# expr-done 2050
--# expr-start 2052 '<FieldAccess for sub_equation at ast.py:15757>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15757
--# expr-start 2051 '<FieldAccess for expr at ast.py:15757>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15757
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2051
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 2052
And_Pred_3 := Create_And (And_Pred_2, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15744") else null)); 
--# expr-done 2053
--# expr-start 2055 '<FieldAccess for matches_expected_formal_prim_type at ast.py:15758>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15758
--# expr-start 2054 '<FieldAccess for expr at ast.py:15758>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15758
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2054
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_12.Node, Origin => Origin);
--# end
--# expr-done 2055
And_Pred_4 := Create_And (And_Pred_3, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15744") else null)); 
--# expr-done 2056
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 2057
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1113;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1113;
                     Finalizer_Scope_2115;
                     Finalizer_Scope_2116;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Contract_Cases_Assoc_Equation;
--# end
--# property-start AggregateAssoc.exprs_assoc_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15761
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Exprs_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1114 with Inline_Always;
            procedure Finalizer_Scope_2117 with Inline_Always;
            procedure Finalizer_Scope_2118 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Fld : Internal_Entity_Alternatives_List;
Fld_1 : Bare_Alternatives_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Logic_Equation;
Item_94 : Internal_Entity;
Bare_Item_94 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1114 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1114;
            procedure Finalizer_Scope_2117 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2117;
            procedure Finalizer_Scope_2118 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2118;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2066 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2065 '<And at ast.py:15768>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15768
--# expr-start 2061 '<LogicAll at ast.py:15769>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15769
--# expr-start 2060 '<Map at ast.py:15769>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15769
   --# expr-start 2058 '<FieldAccess for designators at ast.py:15769>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15769
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Aggregate_Assoc_F_Designators, Info => Ent.Info);
--# expr-done 2058
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_1;
      begin
         for Untyped_Item_94 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_94 := Untyped_Item_94; 
Is_Null := Bare_Item_94 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_94); 
   If_Result := As_Entity;
end if;
                  Item_94 := If_Result; 
   --# scope-start
               --# bind d Item_94
      --# expr-start 2059 '<FieldAccess for sub_equation at ast.py:15769>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15769
      if Item_94.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_94.Node, Env => Env, Origin => Origin, E_Info => Item_94.Info);
--# end
--# expr-done 2059
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2117;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2060
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15769") else null)); 
--# expr-done 2061
--# expr-start 2063 '<Cast to Name at ast.py:15770>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15770
--# expr-start 2062 '<FieldAccess for expr at ast.py:15770>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15770
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2062
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 2063
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 2064 '<FieldAccess for sub_equation at ast.py:15771>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15771
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2064
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2118;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15772") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Logic_Boolean_Op, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15768") else null)); 
--# expr-done 2065
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 2066
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1114;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1114;
                     Finalizer_Scope_2117;
                     Finalizer_Scope_2118;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Exprs_Assoc_Equation;
--# end
   --
   --  Primitives for Bare_Multi_Dim_Array_Assoc
   --
      procedure Initialize_Fields_For_Multi_Dim_Array_Assoc
        (Self : Bare_Multi_Dim_Array_Assoc
         ; Aggregate_Assoc_F_Designators : Bare_Alternatives_List
         ; Aggregate_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Aggregate_Assoc
              (Self, Aggregate_Assoc_F_Designators, Aggregate_Assoc_F_R_Expr);
      end Initialize_Fields_For_Multi_Dim_Array_Assoc;
   --
   --  Primitives for Bare_Composite_Constraint_Assoc
   --
      procedure Initialize_Fields_For_Composite_Constraint_Assoc
        (Self : Bare_Composite_Constraint_Assoc
         ; Composite_Constraint_Assoc_F_Ids : Bare_Discriminant_Choice_List
         ; Composite_Constraint_Assoc_F_Constraint_Expr : Bare_Ada_Node
        ) is
      begin
            Self.Composite_Constraint_Assoc_F_Ids := Composite_Constraint_Assoc_F_Ids;
            Self.Composite_Constraint_Assoc_F_Constraint_Expr := Composite_Constraint_Assoc_F_Constraint_Expr;
      end Initialize_Fields_For_Composite_Constraint_Assoc;
   function Composite_Constraint_Assoc_F_Ids
     (Node : Bare_Composite_Constraint_Assoc) return Bare_Discriminant_Choice_List
   is
   begin
         return Node.Composite_Constraint_Assoc_F_Ids;
   end;
   function Composite_Constraint_Assoc_F_Constraint_Expr
     (Node : Bare_Composite_Constraint_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Composite_Constraint_Assoc_F_Constraint_Expr;
   end;
--# property-start CompositeConstraintAssoc.expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8507
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_Assoc_P_Expr
  (Node : Bare_Composite_Constraint_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Composite_Constraint_Assoc  := Bare_Composite_Constraint_Assoc (Node);
     Ent : Internal_Entity_Composite_Constraint_Assoc :=
       Internal_Entity_Composite_Constraint_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2068 '<Cast to Expr at ast.py:8507>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8507
--# expr-start 2067 '<FieldAccess for constraint_expr at ast.py:8507>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8507
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Composite_Constraint_Assoc_F_Constraint_Expr, Info => Ent.Info);
--# expr-done 2067
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2068
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Composite_Constraint_Assoc_P_Expr;
--# end
--# property-start CompositeConstraintAssoc.names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8508
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_Assoc_P_Names
  (Node : Bare_Composite_Constraint_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Composite_Constraint_Assoc  := Bare_Composite_Constraint_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_621 with Inline_Always;
      Fld : Bare_Discriminant_Choice_List;
Cast_Expr : Bare_Identifier;
Cast_Result : Bare_Ada_Node;
Item_95 : Bare_Identifier;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_621 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_621;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2071 '<Map at ast.py:8508>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8508
   --# expr-start 2069 '<FieldAccess for ids at ast.py:8508>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8508
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Composite_Constraint_Assoc_F_Ids;
--# expr-done 2069
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Discriminant_Choice_List := Fld;
      begin
         for Untyped_Item_95 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_95 := Untyped_Item_95; 
   --# scope-start
               --# bind i Item_95
      --# expr-start 2070 '<Cast to AdaNode at ast.py:8508>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8508
Cast_Expr := Item_95; 
      Cast_Result := Cast_Expr;
--# expr-done 2070
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2071
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_621;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_621;
            raise;
      end;
   return Property_Result;
end Composite_Constraint_Assoc_P_Names;
--# end
   --
   --  Primitives for Bare_Iterated_Assoc
   --
      procedure Initialize_Fields_For_Iterated_Assoc
        (Self : Bare_Iterated_Assoc
         ; Iterated_Assoc_F_Spec : Bare_For_Loop_Spec
         ; Iterated_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Iterated_Assoc_F_Spec := Iterated_Assoc_F_Spec;
            Self.Iterated_Assoc_F_R_Expr := Iterated_Assoc_F_R_Expr;
      end Initialize_Fields_For_Iterated_Assoc;
   function Iterated_Assoc_F_Spec
     (Node : Bare_Iterated_Assoc) return Bare_For_Loop_Spec
   is
   begin
         return Node.Iterated_Assoc_F_Spec;
   end;
   function Iterated_Assoc_F_R_Expr
     (Node : Bare_Iterated_Assoc) return Bare_Expr
   is
   begin
         return Node.Iterated_Assoc_F_R_Expr;
   end;
--# property-start IteratedAssoc.expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15784
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Expr
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2072 '<FieldAccess for r_expr at ast.py:15784>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15784
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Iterated_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 2072
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Expr;
--# end
--# property-start IteratedAssoc.names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15785
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Names
  (Node : Bare_Iterated_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Bare_Ada_Node_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Names;
--# end
--# property-start IteratedAssoc.base_aggregate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15787
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Base_Aggregate
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Aggregate
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Aggregate;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Aggregate;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2075 '<Cast to BaseAggregate at ast.py:15788>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15788
--# expr-start 2074 '<parent at ast.py:15788>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15788
--# expr-start 2073 '<parent at ast.py:15788>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15788
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2073
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 2074
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2075
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Base_Aggregate;
--# end
--# property-start IteratedAssoc.xref_stop_resolution /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15791
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Stop_Resolution
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2078 '<IsA BaseAggregate at ast.py:15791>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15791
--# expr-start 2077 '<parent at ast.py:15791>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15791
--# expr-start 2076 '<parent at ast.py:15791>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15791
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2076
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 2077
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Base_Aggregate; 
--# expr-done 2078
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Xref_Stop_Resolution;
--# end
--# property-start IteratedAssoc.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15793
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Equation
  (Node : Bare_Iterated_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1119 with Inline_Always;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Multidim_Aggregate_Info;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Array_Type_Def;
Block_Var_4 : Boolean;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_5 : Internal_Entity_For_Loop_Spec;
Fld_6 : Internal_Entity_For_Loop_Spec;
Fld_7 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Expr;
Fld_8 : Logic_Var;
Fld_9 : Integer;
Fld_10 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_11 : Internal_Entity_For_Loop_Spec;
Fld_12 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel : Logic_Equation;
Fld_14 : Bare_For_Loop_Spec;
Fld_15 : Bare_Iter_Type;
Is_A : Boolean;
If_Result_1 : Logic_Equation;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Var;
Fld_21 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_22 : Internal_Entity_Expr;
Fld_23 : Logic_Equation;
And_Pred_2 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Is_Equal_1 : Boolean;
Fld_24 : Integer;
Fld_25 : Integer;
Arith_Result : Integer;
Is_Equal_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Logic_Equation;
False_Rel : Logic_Equation;
If_Result_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1119 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1119;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2123 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2079 '<FieldAccess for base_aggregate at ast.py:15795>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15795
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IteratedAssoc.base_aggregate
Fld := Libadalang.Implementation.Iterated_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2079
Block_Var := Fld; 
--# bind aggregate Block_Var
--# expr-start 2080 '<FieldAccess for multidim_root_aggregate at ast.py:15797>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15797
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2080
Block_Var_1 := Fld_1; 
--# bind root_agg Block_Var_1
--# expr-start 2086 '<If at ast.py:15802>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15802
--# expr-start 2082 '<Not at ast.py:15803>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15803
--# expr-start 2081 '<IsNull at ast.py:15803>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15803
Is_Equal := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 2081
Not_Val := not (Is_Equal); 
--# expr-done 2082
if Not_Val then
   --# expr-start 2083 '<FieldAccess for typ at ast.py:15804>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15804
Fld_2 := Block_Var_1.Typ;
--# expr-done 2083
   If_Result := Fld_2;
else
   --# expr-start 2085 '<Cast to BaseTypeDecl at ast.py:15805>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15805
--# expr-start 2084 '<FieldAccess for type_val at ast.py:15805>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15805
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Block_Var.Node);
--# end
--# expr-done 2084
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2085
   If_Result := Cast_Result;
end if;
--# expr-done 2086
Block_Var_2 := If_Result; 
--# bind type_decl Block_Var_2
Var_Expr := Block_Var_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2087 '<FieldAccess for array_def at ast.py:15808>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15808
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2087
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var_3 := Result_Var; 
--# bind array_type_def Block_Var_3
--# expr-start 2105 '<FieldAccess for resolve_names_internal_with_eq at ast.py:15816>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15816
--# expr-start 2088 '<FieldAccess for spec at ast.py:15816>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15816
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2088
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2104 '<If at ast.py:15817>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15817
--# expr-start 2091 '<IsA IterType.In at ast.py:15817>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15817
--# expr-start 2090 '<FieldAccess for loop_type at ast.py:15817>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15817
--# expr-start 2089 '<FieldAccess for spec at ast.py:15817>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15817
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Self.Iterated_Assoc_F_Spec;
--# expr-done 2089
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Fld_14.For_Loop_Spec_F_Loop_Type;
--# expr-done 2090
Is_A := Fld_15 /= null 
and then Fld_15.Kind in Ada_Iter_Type_In_Range; 
--# expr-done 2091
if Is_A then
   --# expr-start 2103 '<And at ast.py:15818>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15818
--# expr-start 2098 '<Bind at ast.py:15818>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15818
--# expr-start 2095 '<FieldAccess for expected_type_var at ast.py:15818>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15818
--# expr-start 2094 '<Cast to Expr at ast.py:15818>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15818
--# expr-start 2093 '<FieldAccess for iter_expr at ast.py:15818>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15818
--# expr-start 2092 '<FieldAccess for spec at ast.py:15818>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15818
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2092
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity (Node => Fld_6.Node.For_Loop_Spec_F_Iter_Expr, Info => Fld_6.Info);
--# expr-done 2093
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Expr
   then
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_1 := No_Entity_Expr;
   end if;
--# expr-done 2094
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Cast_Result_1.Node);
--# expr-done 2095
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 2097 '<FieldAccess for index_type at ast.py:15819>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15819
      if Block_Var_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2096 '<FieldAccess for rank at ast.py:15819>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15819
Fld_9 := Block_Var_1.Rank;
--# expr-done 2096
--# property-call-start ArrayTypeDef.index_type
Fld_10 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Block_Var_3.Node, Dim => Fld_9, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 2097
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
Bind_Result := Solver.Create_Assign (Fld_8, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15818") else null)); 
--# expr-done 2098
--# expr-start 2102 '<FieldAccess for matches_expected_type at ast.py:15820>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15820
--# expr-start 2101 '<Cast to Expr at ast.py:15820>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15820
--# expr-start 2100 '<FieldAccess for iter_expr at ast.py:15820>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15820
--# expr-start 2099 '<FieldAccess for spec at ast.py:15820>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15820
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2099
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity (Node => Fld_11.Node.For_Loop_Spec_F_Iter_Expr, Info => Fld_11.Info);
--# expr-done 2100
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Expr
   then
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Cast_Result_3 := No_Entity_Expr;
   end if;
--# expr-done 2101
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Cast_Result_3.Node, Origin => Origin);
--# end
--# expr-done 2102
And_Pred := Create_And (Bind_Result, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15818") else null)); 
--# expr-done 2103
   If_Result_1 := And_Pred;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15821") else null)); 
   If_Result_1 := True_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2104
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_16 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_5.Node, Additional_Equation => If_Result_1, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 2105
Block_Var_4 := Fld_16; 
--# bind spec_success Block_Var_4
--# expr-start 2122 '<If at ast.py:15824>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15824
if Block_Var_4 then
   --# expr-start 2121 '<If at ast.py:15826>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15826
--# expr-start 2106 '<IsNull at ast.py:15829>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15829
Is_Equal_1 := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 2106
if Is_Equal_1 then
   If_Result_2 := True;
else
   --# expr-start 2110 '<Eq at ast.py:15830>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15830
--# expr-start 2107 '<FieldAccess for rank at ast.py:15830>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15830
Fld_24 := Block_Var_1.Rank;
--# expr-done 2107
--# expr-start 2109 '<Arithmetic - at ast.py:15830>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15830
--# expr-start 2108 '<FieldAccess for array_ndims at ast.py:15830>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15830
      if Block_Var_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_25 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var_3.Node, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 2108
Arith_Result := (Fld_25 - 1); 
--# expr-done 2109
Is_Equal_2 := Fld_24 = Arith_Result; 
--# expr-done 2110
   If_Result_2 := Is_Equal_2;
end if;
if If_Result_2 then
   --# expr-start 2120 '<And at ast.py:15833>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15833
--# expr-start 2117 '<And at ast.py:15833>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15833
--# expr-start 2112 '<FieldAccess for sub_equation at ast.py:15833>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15833
--# expr-start 2111 '<FieldAccess for expr at ast.py:15833>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15833
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2111
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_17.Node, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 2112
--# expr-start 2116 '<Bind at ast.py:15834>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15834
--# expr-start 2114 '<FieldAccess for expected_type_var at ast.py:15834>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15834
--# expr-start 2113 '<FieldAccess for expr at ast.py:15834>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15834
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_19 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2113
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_19.Node);
--# expr-done 2114
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
--# expr-start 2115 '<FieldAccess for comp_type at ast.py:15834>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15834
      if Block_Var_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_21 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Block_Var_3.Node, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 2115
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_21.Node,
         Info => Fld_21.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_20, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15834") else null)); 
--# expr-done 2116
And_Pred_1 := Create_And (Fld_18, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15833") else null)); 
--# expr-done 2117
--# expr-start 2119 '<FieldAccess for matches_expected_type at ast.py:15835>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15835
--# expr-start 2118 '<FieldAccess for expr at ast.py:15835>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15835
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_22 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2118
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_23 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_22.Node, Origin => Origin);
--# end
--# expr-done 2119
And_Pred_2 := Create_And (And_Pred_1, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15833") else null)); 
--# expr-done 2120
   If_Result_3 := And_Pred_2;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15839") else null)); 
   If_Result_3 := True_Rel_1;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 2121
   If_Result_4 := If_Result_3;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15841") else null)); 
   If_Result_4 := False_Rel;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 2122
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 2123
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1119;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1119;
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Xref_Equation;
--# end
--# property-start IteratedAssoc.xref_equation_for_reduce /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15844
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Equation_For_Reduce
  (Node : Bare_Iterated_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1120 with Inline_Always;
      Block_Var : Logic_Var;
Block_Var_1 : Boolean;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Value_Sequence;
Fld_1 : Internal_Entity_Iterated_Assoc;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_For_Loop_Spec;
Fld_5 : Internal_Entity_For_Loop_Spec;
Fld_6 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Expr;
Fld_7 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_8 : Internal_Entity_For_Loop_Spec;
Fld_9 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel : Logic_Equation;
Fld_11 : Bare_For_Loop_Spec;
Fld_12 : Bare_Iter_Type;
Is_A : Boolean;
If_Result : Logic_Equation;
Fld_13 : Boolean;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Logic_Equation;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Equation;
And_Pred_2 : Logic_Equation;
False_Rel : Logic_Equation;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1120 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1120;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2155 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2128 '<FieldAccess for expected_type_var at ast.py:15853>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15853
--# expr-start 2127 '<FieldAccess for expr at ast.py:15852>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15852
--# expr-start 2126 '<FieldAccess for iter_assoc at ast.py:15852>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15852
--# expr-start 2125 '<Cast to ValueSequence at ast.py:15852>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15852
--# expr-start 2124 '<parent at ast.py:15852>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15852
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2124
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Value_Sequence_Range
   then
      Cast_Result := Create_Internal_Entity_Value_Sequence
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Value_Sequence;
   end if;
--# expr-done 2125
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Iterated_Assoc (Node => Cast_Result.Node.Value_Sequence_F_Iter_Assoc, Info => Cast_Result.Info);
--# expr-done 2126
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2127
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2.Node);
--# expr-done 2128
Block_Var := Fld_3; 
--# bind expected_typ Block_Var
--# expr-start 2144 '<FieldAccess for resolve_names_internal_with_eq at ast.py:15856>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15856
--# expr-start 2129 '<FieldAccess for spec at ast.py:15856>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15856
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2129
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2143 '<If at ast.py:15857>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15857
--# expr-start 2132 '<IsA IterType.In at ast.py:15857>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15857
--# expr-start 2131 '<FieldAccess for loop_type at ast.py:15857>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15857
--# expr-start 2130 '<FieldAccess for spec at ast.py:15857>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15857
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Iterated_Assoc_F_Spec;
--# expr-done 2130
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Fld_11.For_Loop_Spec_F_Loop_Type;
--# expr-done 2131
Is_A := Fld_12 /= null 
and then Fld_12.Kind in Ada_Iter_Type_In_Range; 
--# expr-done 2132
if Is_A then
   --# expr-start 2142 '<And at ast.py:15858>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15858
--# expr-start 2137 '<Bind at ast.py:15858>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15858
--# expr-start 2136 '<FieldAccess for expected_type_var at ast.py:15858>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15858
--# expr-start 2135 '<Cast to Expr at ast.py:15858>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15858
--# expr-start 2134 '<FieldAccess for iter_expr at ast.py:15858>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15858
--# expr-start 2133 '<FieldAccess for spec at ast.py:15858>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15858
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2133
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity (Node => Fld_5.Node.For_Loop_Spec_F_Iter_Expr, Info => Fld_5.Info);
--# expr-done 2134
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Expr
   then
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result_1 := No_Entity_Expr;
   end if;
--# expr-done 2135
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Cast_Result_1.Node);
--# expr-done 2136
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Block_Var.Value := No_Entity;
Entity_Vars.Reset (Block_Var);
Bind_Result := Solver.Create_Unify (Fld_7, Block_Var, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15858") else null)); 
--# expr-done 2137
--# expr-start 2141 '<FieldAccess for matches_expected_type at ast.py:15860>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15860
--# expr-start 2140 '<Cast to Expr at ast.py:15860>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15860
--# expr-start 2139 '<FieldAccess for iter_expr at ast.py:15860>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15860
--# expr-start 2138 '<FieldAccess for spec at ast.py:15860>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15860
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2138
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity (Node => Fld_8.Node.For_Loop_Spec_F_Iter_Expr, Info => Fld_8.Info);
--# expr-done 2139
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Expr
   then
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_2 := No_Entity_Expr;
   end if;
--# expr-done 2140
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_10 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Cast_Result_2.Node, Origin => Origin);
--# end
--# expr-done 2141
And_Pred := Create_And (Bind_Result, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15858") else null)); 
--# expr-done 2142
   If_Result := And_Pred;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15861") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 2143
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_13 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_4.Node, Additional_Equation => If_Result, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 2144
Block_Var_1 := Fld_13; 
--# bind spec_success Block_Var_1
--# expr-start 2154 '<If at ast.py:15864>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15864
if Block_Var_1 then
   --# expr-start 2153 '<And at ast.py:15865>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15865
--# expr-start 2150 '<And at ast.py:15865>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15865
--# expr-start 2146 '<FieldAccess for sub_equation at ast.py:15865>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15865
--# expr-start 2145 '<FieldAccess for expr at ast.py:15865>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15865
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2145
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_14.Node, Env => Env, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 2146
--# expr-start 2149 '<Bind at ast.py:15866>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15866
--# expr-start 2148 '<FieldAccess for expected_type_var at ast.py:15866>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15866
--# expr-start 2147 '<FieldAccess for expr at ast.py:15866>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15866
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2147
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_16.Node);
--# expr-done 2148
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
Block_Var.Value := No_Entity;
Entity_Vars.Reset (Block_Var);
Bind_Result_1 := Solver.Create_Unify (Fld_17, Block_Var, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15866") else null)); 
--# expr-done 2149
And_Pred_1 := Create_And (Fld_15, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15865") else null)); 
--# expr-done 2150
--# expr-start 2152 '<FieldAccess for matches_expected_type at ast.py:15867>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15867
--# expr-start 2151 '<FieldAccess for expr at ast.py:15867>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15867
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2151
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_19 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_18.Node, Origin => Origin);
--# end
--# expr-done 2152
And_Pred_2 := Create_And (And_Pred_1, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15865") else null)); 
--# expr-done 2153
   If_Result_1 := And_Pred_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15868") else null)); 
   If_Result_1 := False_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2154
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2155
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1120;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1120;
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Xref_Equation_For_Reduce;
--# end
   --
   --  Primitives for Bare_Param_Assoc
   --
      procedure Initialize_Fields_For_Param_Assoc
        (Self : Bare_Param_Assoc
         ; Param_Assoc_F_Designator : Bare_Ada_Node
         ; Param_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Param_Assoc_F_Designator := Param_Assoc_F_Designator;
            Self.Param_Assoc_F_R_Expr := Param_Assoc_F_R_Expr;
      end Initialize_Fields_For_Param_Assoc;
   function Param_Assoc_F_Designator
     (Node : Bare_Param_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Param_Assoc_F_Designator;
   end;
   function Param_Assoc_F_R_Expr
     (Node : Bare_Param_Assoc) return Bare_Expr
   is
   begin
         return Node.Param_Assoc_F_R_Expr;
   end;
--# property-start ParamAssoc.expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15497
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Expr
  (Node : Bare_Param_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
     Ent : Internal_Entity_Param_Assoc :=
       Internal_Entity_Param_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2156 '<FieldAccess for r_expr at ast.py:15497>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15497
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Param_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 2156
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Expr;
--# end
--# property-start ParamAssoc.names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15498
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Names
  (Node : Bare_Param_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1099 with Inline_Always;
      Fld : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Fld_1 : Bare_Ada_Node;
Is_Null : Boolean;
If_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1099 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1099;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2160 '<If at ast.py:15498>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15498
--# expr-start 2158 '<IsNull at ast.py:15498>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15498
--# expr-start 2157 '<FieldAccess for designator at ast.py:15498>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15498
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Param_Assoc_F_Designator;
--# expr-done 2157
Is_Null := Fld_1 = null; 
--# expr-done 2158
if Is_Null then
   If_Result := No_Bare_Ada_Node_Array_Type;
else
   --# expr-start 2159 '<FieldAccess for designator at ast.py:15499>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15499
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Assoc_F_Designator;
--# expr-done 2159
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
   If_Result := Singleton;
end if;
      Inc_Ref (If_Result);
--# expr-done 2160
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1099;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1099;
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Names;
--# end
--# property-start ParamAssoc.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15501
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Xref_Entry_Point
  (Node : Bare_Param_Assoc
  )
   return Boolean
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2161 '<FieldAccess for is_static_attribute_assoc at ast.py:15501>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15501
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ParamAssoc.is_static_attribute_assoc
Fld := Libadalang.Implementation.Param_Assoc_P_Is_Static_Attribute_Assoc (Node => Self);
--# end
--# expr-done 2161
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Xref_Entry_Point;
--# end
--# property-start ParamAssoc.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15502
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Xref_Equation
  (Node : Bare_Param_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
     Ent : Internal_Entity_Param_Assoc :=
       Internal_Entity_Param_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1101 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_1101 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1101;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2165 '<If at ast.py:15502>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15502
--# expr-start 2162 '<FieldAccess for xref_entry_point at ast.py:15503>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15503
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Self);
--# end
--# expr-done 2162
if Fld_2 then
   --# expr-start 2164 '<FieldAccess for sub_equation at ast.py:15504>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15504
--# expr-start 2163 '<FieldAccess for expr at ast.py:15504>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15504
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2163
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2164
   If_Result := Fld_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15505") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 2165
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1101;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1101;
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Xref_Equation;
--# end
--# property-start ParamAssoc.is_static_attribute_assoc /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15508
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Is_Static_Attribute_Assoc
  (Node : Bare_Param_Assoc
  )
   return Boolean
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Attribute_Ref;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Attribute_Ref;
Fld_2 : Bare_Identifier;
Fld_3 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_4 : Bare_Identifier;
Fld_5 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_6 : Bare_Identifier;
Fld_7 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Bare_Identifier;
Fld_9 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_10 : Bare_Identifier;
Fld_11 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Fld_12 : Bare_Identifier;
Fld_13 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2193 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2168 '<Cast to AttributeRef at ast.py:15510>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15510
--# expr-start 2167 '<parent at ast.py:15510>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15510
--# expr-start 2166 '<parent at ast.py:15510>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15510
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2166
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 2167
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2168
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ar Var_Expr
--# expr-start 2172 '<Eq at ast.py:15511>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2170 '<FieldAccess for name_symbol at ast.py:15511>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2169 '<FieldAccess for attribute at ast.py:15511>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 2169
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2);
--# end
--# expr-done 2170
--# expr-start 2171 '<Symbol First at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 2171
Is_Equal := Fld_3 = Sym; 
--# expr-done 2172
if Is_Equal then
   If_Result := True;
else
   --# expr-start 2176 '<Eq at ast.py:15511>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2174 '<FieldAccess for name_symbol at ast.py:15511>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2173 '<FieldAccess for attribute at ast.py:15511>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 2173
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4);
--# end
--# expr-done 2174
--# expr-start 2175 '<Symbol Last at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 2175
Is_Equal_1 := Fld_5 = Sym_1; 
--# expr-done 2176
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2180 '<Eq at ast.py:15511>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2178 '<FieldAccess for name_symbol at ast.py:15511>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2177 '<FieldAccess for attribute at ast.py:15511>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 2177
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6);
--# end
--# expr-done 2178
--# expr-start 2179 '<Symbol Range at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 2179
Is_Equal_2 := Fld_7 = Sym_2; 
--# expr-done 2180
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 2184 '<Eq at ast.py:15511>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2182 '<FieldAccess for name_symbol at ast.py:15511>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2181 '<FieldAccess for attribute at ast.py:15511>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 2181
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_8);
--# end
--# expr-done 2182
--# expr-start 2183 '<Symbol Length at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 2183
Is_Equal_3 := Fld_9 = Sym_3; 
--# expr-done 2184
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 2188 '<Eq at ast.py:15511>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2186 '<FieldAccess for name_symbol at ast.py:15511>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2185 '<FieldAccess for attribute at ast.py:15511>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 2185
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_10);
--# end
--# expr-done 2186
--# expr-start 2187 '<Symbol Has_Same_Storage at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Same_Storage); 
--# expr-done 2187
Is_Equal_4 := Fld_11 = Sym_4; 
--# expr-done 2188
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 2192 '<Eq at ast.py:15511>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2190 '<FieldAccess for name_symbol at ast.py:15511>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
--# expr-start 2189 '<FieldAccess for attribute at ast.py:15511>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15511
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 2189
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_12);
--# end
--# expr-done 2190
--# expr-start 2191 '<Symbol Overlaps_Storage at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Overlaps_Storage); 
--# expr-done 2191
Is_Equal_5 := Fld_13 = Sym_5; 
--# expr-done 2192
   If_Result_4 := Is_Equal_5;
end if;
Scope_Result := If_Result_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2193
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Is_Static_Attribute_Assoc;
--# end
   --
   --  Primitives for Bare_Basic_Decl
   --
   function Basic_Decl_F_Aspects
     (Node : Bare_Basic_Decl) return Bare_Aspect_Spec
   is
         Kind : constant Ada_Basic_Decl := Node.Kind;
   begin
         case Kind is
               when Ada_Error_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Discriminant_Spec =>
         return Node.Discriminant_Spec_F_Aspects;
               when Ada_Component_Decl =>
         return Node.Component_Decl_F_Aspects;
               when Ada_Classwide_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Concrete_Type_Decl =>
         return Node.Concrete_Type_Decl_F_Aspects;
               when Ada_Formal_Type_Decl =>
         return Node.Formal_Type_Decl_F_Aspects;
               when Ada_Anonymous_Type_Decl | Ada_Synth_Anonymous_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Subtype_Decl =>
         return Node.Subtype_Decl_F_Aspects;
               when Ada_Discrete_Base_Subtype_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Task_Type_Decl | Ada_Single_Task_Type_Decl =>
         return Node.Task_Type_Decl_F_Aspects;
               when Ada_Protected_Type_Decl =>
         return Node.Protected_Type_Decl_F_Aspects;
               when Ada_Param_Spec =>
         return Node.Param_Spec_F_Aspects;
               when Ada_Subp_Decl =>
         return Node.Subp_Decl_F_Aspects;
               when Ada_Abstract_Subp_Decl =>
         return Node.Abstract_Subp_Decl_F_Aspects;
               when Ada_Single_Task_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Single_Protected_Decl =>
         return Node.Single_Protected_Decl_F_Aspects;
               when Ada_Number_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Object_Decl | Ada_Extended_Return_Stmt_Object_Decl | Ada_No_Type_Object_Renaming_Decl =>
         return Node.Object_Decl_F_Aspects;
               when Ada_Anonymous_Expr_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Package_Internal | Ada_Package_Decl =>
         return Node.Base_Package_Decl_F_Aspects;
               when Ada_Exception_Decl =>
         return Node.Exception_Decl_F_Aspects;
               when Ada_Generic_Subp_Instantiation =>
         return Node.Generic_Subp_Instantiation_F_Aspects;
               when Ada_Generic_Package_Instantiation =>
         return Node.Generic_Package_Instantiation_F_Aspects;
               when Ada_Package_Renaming_Decl =>
         return Node.Package_Renaming_Decl_F_Aspects;
               when Ada_Generic_Package_Renaming_Decl =>
         return Node.Generic_Package_Renaming_Decl_F_Aspects;
               when Ada_Generic_Subp_Renaming_Decl =>
         return Node.Generic_Subp_Renaming_Decl_F_Aspects;
               when Ada_Abstract_Formal_Subp_Decl | Ada_Concrete_Formal_Subp_Decl =>
         return Node.Formal_Subp_Decl_F_Aspects;
               when Ada_Generic_Formal_Obj_Decl | Ada_Generic_Formal_Package | Ada_Generic_Formal_Subp_Decl | Ada_Generic_Formal_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Subp_Internal =>
         return Node.Generic_Subp_Internal_F_Aspects;
               when Ada_Generic_Subp_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Package_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Enum_Literal_Decl | Ada_Synthetic_Char_Enum_Lit =>
                     return No_Bare_Ada_Node;
               when Ada_Synthetic_Formal_Param_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Synthetic_Subp_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Entry_Decl =>
         return Node.Entry_Decl_F_Aspects;
               when Ada_For_Loop_Var_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Expr_Function =>
         return Node.Expr_Function_F_Aspects;
               when Ada_Null_Subp_Decl =>
         return Node.Null_Subp_Decl_F_Aspects;
               when Ada_Subp_Renaming_Decl =>
         return Node.Subp_Renaming_Decl_F_Aspects;
               when Ada_Subp_Body =>
         return Node.Subp_Body_F_Aspects;
               when Ada_Exception_Handler =>
                     return No_Bare_Ada_Node;
               when Ada_Label_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Named_Stmt_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Accept_Stmt_Body =>
                     return No_Bare_Ada_Node;
               when Ada_Package_Body =>
         return Node.Package_Body_F_Aspects;
               when Ada_Task_Body =>
         return Node.Task_Body_F_Aspects;
               when Ada_Protected_Body =>
         return Node.Protected_Body_F_Aspects;
               when Ada_Entry_Body =>
         return Node.Entry_Body_F_Aspects;
               when Ada_Entry_Index_Spec =>
         return Node.Entry_Index_Spec_F_Aspects;
               when Ada_Protected_Body_Stub =>
         return Node.Protected_Body_Stub_F_Aspects;
               when Ada_Subp_Body_Stub =>
         return Node.Subp_Body_Stub_F_Aspects;
               when Ada_Package_Body_Stub =>
         return Node.Package_Body_Stub_F_Aspects;
               when Ada_Task_Body_Stub =>
         return Node.Task_Body_Stub_F_Aspects;
               when Ada_Incomplete_Type_Decl | Ada_Incomplete_Formal_Type_Decl | Ada_Incomplete_Tagged_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Abstract_State_Decl =>
         return Node.Abstract_State_Decl_F_Aspects;
         end case;
   end;
--# property-start BasicDecl.is_spark_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1892
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Spark_Impl
  (Node : Bare_Basic_Decl
      ; Include_Skip_Proof_Annotations : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_skip_proof_annotations Include_Skip_Proof_Annotations
   Property_Result : Boolean;
            procedure Finalizer_Scope_129 with Inline_Always;
            procedure Finalizer_Scope_2122 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Subp_Body;
Var_Expr_1 : Internal_Entity_Expr;
Var_Expr_2 : Internal_Entity_Declarative_Part;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Pragma_Node;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Expr_Array_Access;
Scope_Result : Internal_Entity_Expr_Array_Access;
Result_Var : Internal_Entity_Expr_Array_Access;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Cast_Expr_2 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_2 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Find_Result : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Sym_2 : Symbol_Type;
Fld_3 : Boolean;
Sym_3 : Symbol_Type;
Fld_4 : Internal_Aspect;
Fld_5 : Internal_Entity_Expr;
Cast_Expr_3 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Sym_4 : Symbol_Type;
Fld_6 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Fld_9 : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null_1 : Boolean;
If_Result_3 : Internal_Entity_Declarative_Part;
Fld_10 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_11 : Bare_Compilation_Unit;
Fld_12 : Internal_Entity_Pragma_Node;
Fld_13 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_129 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_129;
            procedure Finalizer_Scope_2122 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_2122;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Is_Spark_Impl,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Skip_Proof_Annotations);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 2226 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2225 '<Cond at ast.py:1905>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1905
if Include_Skip_Proof_Annotations then
   --# expr-start 2206 '<Not at ast.py:1908>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1908
--# expr-start 2205 '<IsNull at ast.py:1913>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1913
--# expr-start 2204 '<Find at ast.py:1909>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1909
--# expr-start 2194 '<Cast to BaseSubpBody at ast.py:1909>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1909
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 2194
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 2195 '<FieldAccess for gnatprove_annotations at ast.py:1909>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1909
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpBody.gnatprove_annotations
Fld := Libadalang.Implementation.Base_Subp_Body_P_Gnatprove_Annotations (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2195
Scope_Result := Fld;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2122;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Find_Result := No_Entity_Expr;
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Result_Var;
      begin
         for Item_96 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_96
--# expr-start 2199 '<Eq at ast.py:1910>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1910
--# expr-start 2197 '<FieldAccess for name_symbol at ast.py:1910>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1910
--# expr-start 2196 '<Cast to Name at ast.py:1910>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1910
Cast_Expr_1 := Item_96; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 2196
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result_1.Node);
--# end
--# expr-done 2197
--# expr-start 2198 '<Symbol Skip_Proof at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Skip_Proof); 
--# expr-done 2198
Is_Equal := Fld_1 = Sym; 
--# expr-done 2199
if Is_Equal then
   If_Result := True;
else
   --# expr-start 2203 '<Eq at ast.py:1910>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1910
--# expr-start 2201 '<FieldAccess for name_symbol at ast.py:1910>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1910
--# expr-start 2200 '<Cast to Name at ast.py:1910>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1910
Cast_Expr_2 := Item_96; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 2200
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result_2.Node);
--# end
--# expr-done 2201
--# expr-start 2202 '<Symbol Skip_Flow_And_Proof at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Skip_Flow_And_Proof); 
--# expr-done 2202
Is_Equal_1 := Fld_2 = Sym_1; 
--# expr-done 2203
   If_Result := Is_Equal_1;
end if;
      if If_Result then
         Find_Result := Item_96; 
      end if;
   --# end
               exit when If_Result;
         end loop;
      end;
--# expr-done 2204
Is_Null := Find_Result.Node = null; 
--# expr-done 2205
Not_Val := not (Is_Null); 
--# expr-done 2206
   If_Result_1 := Not_Val;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_6 := False;
else
--# expr-start 2208 '<FieldAccess for has_aspect at ast.py:1918>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1918
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2207 '<Symbol SPARK_Mode at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 2207
--# property-call-start BasicDecl.has_aspect
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_2, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2208
if Fld_3 then
--# expr-start 2211 '<FieldAccess for value at ast.py:1919>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1919
--# expr-start 2210 '<FieldAccess for get_aspect at ast.py:1919>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1919
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2209 '<Symbol SPARK_Mode at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 2209
--# property-call-start BasicDecl.get_aspect
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym_3, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2210
Fld_5 := Fld_4.Value;
--# expr-done 2211
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind mode Var_Expr_1
--# expr-start 2214 '<FieldAccess for name_is at ast.py:1920>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1920
--# expr-start 2212 '<Cast to Name at ast.py:1920>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1920
Cast_Expr_3 := Var_Expr_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Name;
   end if;
--# expr-done 2212
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2213 '<Symbol On at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_On); 
--# expr-done 2213
--# property-call-start Name.name_is
Fld_6 := Libadalang.Implementation.Name_P_Name_Is (Node => Cast_Result_3.Node, Sym => Sym_4);
--# end
--# expr-done 2214
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := True;
end if;
   If_Result_5 := Result_Var_1;
else
--# expr-start 2220 '<as_entity at ast.py:1931>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1931
--# expr-start 2219 '<FieldAccess for declarative_scope at ast.py:1931>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1931
--# expr-start 2218 '<If at ast.py:1927>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1927
--# expr-start 2215 '<FieldAccess for previous_part_for_decl at ast.py:1928>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1928
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2215
Var_Expr_3 := Fld_8;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2216 '<IsA BodyStub at ast.py:1928>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1928
Is_A := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_Body_Stub; 
--# expr-done 2216
Scope_Result_2 := Is_A;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 2217 '<FieldAccess for previous_part_for_decl at ast.py:1929>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1929
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2217
   If_Result_2 := Fld_7;
else
   If_Result_2 := Ent;
end if;
--# expr-done 2218
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_9 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => If_Result_2.Node);
--# end
--# expr-done 2219
Is_Null_1 := Fld_9 = null; 
if Is_Null_1 then
   If_Result_3 := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Fld_9); 
   If_Result_3 := As_Entity;
end if;
--# expr-done 2220
Var_Expr_2 := If_Result_3;
if Var_Expr_2 /= No_Entity_Declarative_Part then
   --# scope-start
--# expr-start 2221 '<FieldAccess for is_spark at ast.py:1931>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1931
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]DeclarativePart.is_spark'
Fld_10 := Libadalang.Implementation.Dispatcher_Declarative_Part_P_Is_Spark (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2221
Scope_Result_3 := Fld_10;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
   If_Result_4 := True;
else
--# expr-start 2223 '<FieldAccess for spark_config_pragma at ast.py:1936>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1936
--# expr-start 2222 '<FieldAccess for enclosing_compilation_unit at ast.py:1936>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1936
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_11 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Ent.Node);
--# end
--# expr-done 2222
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.spark_config_pragma
Fld_12 := Libadalang.Implementation.Compilation_Unit_P_Spark_Config_Pragma (Node => Fld_11);
--# end
--# expr-done 2223
Var_Expr_4 := Fld_12;
if Var_Expr_4 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr_4
--# expr-start 2224 '<FieldAccess for spark_mode_is_on at ast.py:1937>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1937
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.spark_mode_is_on
Fld_13 := Libadalang.Implementation.Pragma_Node_P_Spark_Mode_Is_On (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 2224
Scope_Result_4 := Fld_13;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
   If_Result_4 := Result_Var_4;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
--# expr-done 2225
Let_Result := If_Result_6; 
--# end
--# expr-done 2226
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_129;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_129;
                     Finalizer_Scope_2122;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Basic_Decl_P_Is_Spark_Impl;
--# end
--# property-start BasicDecl.subp_constrain_prefix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1943
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_130 with Inline_Always;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel : Logic_Equation;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_130 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_130;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2240 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2239 '<If at ast.py:1950>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1950
--# expr-start 2229 '<FieldAccess for dottable_subp at ast.py:1954>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1954
--# expr-start 2228 '<FieldAccess for md at ast.py:1954>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1954
--# expr-start 2227 '<FieldAccess for info at ast.py:1954>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1954
Fld_6 := Ent.Info;
--# expr-done 2227
Fld_7 := Fld_6.Md;
--# expr-done 2228
Fld_8 := Fld_7.Dottable_Subp;
--# expr-done 2229
if Fld_8 then
   --# expr-start 2238 '<And at ast.py:1955>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1955
--# expr-start 2236 '<Bind at ast.py:1955>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1955
--# expr-start 2230 '<FieldAccess for expected_type_var at ast.py:1955>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1955
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Prefix);
--# expr-done 2230
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 2233 '<at at ast.py:1957>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1957
--# expr-start 2232 '<FieldAccess for unpacked_formal_params at ast.py:1957>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1957
--# expr-start 2231 '<FieldAccess for subp_spec_or_null at ast.py:1956>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1956
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2231
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2232
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 2233
Var_Expr := Get_Result;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2235 '<FieldAccess for formal_type at ast.py:1957>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1957
--# expr-start 2234 '<FieldAccess for formal_decl at ast.py:1957>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1957
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2234
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 2235
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:1955") else null)); 
--# expr-done 2236
--# expr-start 2237 '<FieldAccess for matches_expected_prefix_type at ast.py:1958>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1958
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_prefix_type
Fld_5 := Libadalang.Implementation.Expr_P_Matches_Expected_Prefix_Type (Node => Prefix, Origin => Origin);
--# end
--# expr-done 2237
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:1955") else null)); 
--# expr-done 2238
   If_Result := And_Pred;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:1959") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 2239
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2240
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_130;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_130;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Subp_Constrain_Prefix;
--# end
--# property-start BasicDecl.env_hook_basic_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1962
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Hook_Basic_Decl
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1936 with Inline_Always;
      Var_Expr : Internal_Entity_Dotted_Name;
Var_Expr_1 : Internal_Unit;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Symbol_Type_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Unit;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1936 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1936;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2252 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2251 '<If at ast.py:1967>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1967
--# expr-start 2241 '<IsA PackageDecl, BasicSubpDecl, PackageRenamingDecl, GenericPackageDecl, GenericPackageInstantiation, GenericSubpInstantiation, GenericSubpDecl, SubpBody at ast.py:1970>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1970
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Package_Decl_Range | Ada_Basic_Subp_Decl | Ada_Package_Renaming_Decl_Range | Ada_Generic_Package_Decl_Range | Ada_Generic_Package_Instantiation_Range | Ada_Generic_Subp_Instantiation_Range | Ada_Generic_Subp_Decl_Range | Ada_Subp_Body_Range; 
--# expr-done 2241
if Is_A_1 then
--# expr-start 2245 '<Cast to DottedName at ast.py:1975>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1975
--# expr-start 2244 '<FieldAccess for name at ast.py:1975>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1975
--# expr-start 2243 '<FieldAccess for defining_name at ast.py:1975>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1975
--# expr-start 2242 '<as_bare_entity at ast.py:1975>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1975
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2242
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 2243
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 2244
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 2245
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 2250 '<FieldAccess for get_unit at ast.py:1977>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1977
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2247 '<FieldAccess for as_symbol_array at ast.py:1977>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1977
--# expr-start 2246 '<FieldAccess for prefix at ast.py:1977>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1977
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 2246
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_2.Node);
--# end
--# expr-done 2247
--# expr-start 2249 '<Not at ast.py:1980>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1980
--# expr-start 2248 '<IsA SubpBody at ast.py:1980>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1980
Is_A := Self /= null 
and then Self.Kind in Ada_Subp_Body_Range; 
--# expr-done 2248
Not_Val := not (Is_A); 
--# expr-done 2249
Fld_4 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_3, Kind => Unit_Specification, Load_If_Needed => True, Not_Found_Is_Error => Not_Val);
--# expr-done 2250
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= null then
   --# scope-start
   --# bind _ Var_Expr_1
Scope_Result := False;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_1936;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
--# expr-done 2251
Let_Result := If_Result_1; 
--# end
--# expr-done 2252
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1936;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Env_Hook_Basic_Decl;
--# end
--# property-start BasicDecl.populate_body_unit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1986
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Populate_Body_Unit
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_132 with Inline_Always;
      Var_Expr : Internal_Unit;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Fld_2 : Symbol_Type_Array_Access;
Fld_3 : Internal_Unit;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_132 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_132;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2260 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2259 '<If at ast.py:1993>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1993
--# expr-start 2253 '<FieldAccess for is_library_item at ast.py:1994>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1994
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 2253
if Fld_4 then
--# expr-start 2258 '<FieldAccess for get_unit at ast.py:1995>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1995
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2257 '<FieldAccess for as_symbol_array at ast.py:1996>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1996
--# expr-start 2256 '<FieldAccess for name at ast.py:1996>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1996
--# expr-start 2255 '<FieldAccess for defining_name at ast.py:1996>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1996
--# expr-start 2254 '<as_bare_entity at ast.py:1996>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:1996
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2254
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 2255
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 2256
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_1.Node);
--# end
--# expr-done 2257
Fld_3 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_2, Kind => Unit_Body, Load_If_Needed => True, Not_Found_Is_Error => False);
--# expr-done 2258
Var_Expr := Fld_3;
if Var_Expr /= null then
   --# scope-start
   --# bind _ Var_Expr
Scope_Result := False;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_1 := Result_Var;
else
   If_Result_1 := False;
end if;
--# expr-done 2259
Let_Result := If_Result_1; 
--# end
--# expr-done 2260
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_132;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_132;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Populate_Body_Unit;
--# end
--# property-start BasicDecl.has_top_level_env_name_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2004
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Top_Level_Env_Name_Impl
  (Node : Bare_Basic_Decl
      ; Allow_Bodies : Boolean
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind allow_bodies Allow_Bodies
   Property_Result : Boolean;
            procedure Finalizer_Scope_133 with Inline_Always;
      Block_Var : Boolean;
Block_Var_1 : Boolean;
Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_2 : Bare_Basic_Decl;
Is_A : Boolean;
Is_A_1 : Boolean;
Fld : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_1 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Is_A_2 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Is_A_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_3 : Boolean;
Fld_2 : Boolean;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Basic_Decl;
Is_Equal : Boolean;
If_Result_4 : Boolean;
Scope_Result : Boolean;
Is_A_4 : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
If_Result_5 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_133 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_133;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2278 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2261 '<IsA BasePackageDecl, BasicSubpDecl, GenericDecl, TaskTypeDecl, ProtectedTypeDecl, SingleTaskDecl, SingleProtectedDecl at ast.py:2010>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2010
Is_A := Self /= null 
and then Self.Kind in Ada_Base_Package_Decl | Ada_Basic_Subp_Decl | Ada_Generic_Decl | Ada_Task_Type_Decl_Range | Ada_Protected_Type_Decl_Range | Ada_Single_Task_Decl_Range | Ada_Single_Protected_Decl_Range; 
--# expr-done 2261
Block_Var := Is_A; 
--# bind is_decl Block_Var
--# expr-start 2262 '<IsA Body at ast.py:2016>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2016
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Body_Node; 
--# expr-done 2262
Block_Var_1 := Is_A_1; 
--# bind is_body Block_Var_1
--# expr-start 2263 '<FieldAccess for is_compilation_unit_root at ast.py:2018>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2018
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2263
if Fld then
   If_Result_7 := True;
else
if Block_Var then
   If_Result_1 := True;
else
if Allow_Bodies then
   If_Result := Block_Var_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
--# expr-start 2265 '<env_node at ast.py:2020>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2020
--# expr-start 2264 '<FieldAccess for node_env at ast.py:2020>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2020
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 2264
Env_Node := AST_Envs.Env_Node (Fld_1); 
--# expr-done 2265
Var_Expr := Env_Node;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind node Var_Expr
--# expr-start 2266 '<Cast to BasicDecl at ast.py:2021>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2021
Cast_Expr := Var_Expr; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Basic_Decl
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2266
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 2273 '<If at ast.py:2022>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2022
--# expr-start 2267 '<Eq at ast.py:2023>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2023
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Cast_Result_1 = Var_Expr_1; 
--# expr-done 2267
if Is_Equal then
   If_Result_4 := True;
else
   --# expr-start 2272 '<FieldAccess for has_top_level_env_name_impl at ast.py:2025>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2025
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
if Allow_Bodies then
   --# expr-start 2269 '<Not at ast.py:2028>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2028
--# expr-start 2268 '<IsA BaseSubpBody at ast.py:2028>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2028
Is_A_2 := Self /= null 
and then Self.Kind in Ada_Base_Subp_Body; 
--# expr-done 2268
Not_Val := not (Is_A_2); 
--# expr-done 2269
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 2271 '<Not at ast.py:2029>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2029
--# expr-start 2270 '<IsA BaseSubpBody at ast.py:2029>' Is_A_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2029
Is_A_3 := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Base_Subp_Body; 
--# expr-done 2270
Not_Val_1 := not (Is_A_3); 
--# expr-done 2271
   If_Result_3 := Not_Val_1;
else
   If_Result_3 := False;
end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_1, Allow_Bodies => If_Result_3);
--# end
--# expr-done 2272
   If_Result_4 := Fld_2;
end if;
--# expr-done 2273
Scope_Result := If_Result_4;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 2274 '<IsA PrivatePart at ast.py:2034>' Is_A_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2034
Is_A_4 := Var_Expr /= null 
and then Var_Expr.Kind in Ada_Private_Part_Range; 
--# expr-done 2274
if Is_A_4 then
--# expr-start 2276 '<Cast to BasicDecl at ast.py:2035>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2035
--# expr-start 2275 '<parent at ast.py:2035>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2035
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr.Parent; 
--# expr-done 2275
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 2276
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 2277 '<FieldAccess for has_top_level_env_name_impl at ast.py:2037>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2037
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_2, Allow_Bodies => Allow_Bodies);
--# end
--# expr-done 2277
Scope_Result_1 := Fld_4;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   If_Result_5 := Result_Var;
else
   If_Result_5 := False;
end if;
   Result_Var_1 := If_Result_5;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
   If_Result_6 := Result_Var_2;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
Let_Result := If_Result_7; 
--# end
--# expr-done 2278
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_133;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_133;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Has_Top_Level_Env_Name_Impl;
--# end
--# property-start BasicDecl.has_top_level_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2045
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Top_Level_Env_Name
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_134 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_2 : Bare_Basic_Decl;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Is_A : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Basic_Decl;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_134 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_134;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2287 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2280 '<env_node at ast.py:2114>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2114
--# expr-start 2279 '<FieldAccess for children_env at ast.py:2114>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2114
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 2279
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 2280
Var_Expr := Env_Node;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind node Var_Expr
--# expr-start 2281 '<Cast to BasicDecl at ast.py:2115>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2115
Cast_Expr := Var_Expr; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Basic_Decl
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2281
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 2282 '<FieldAccess for has_top_level_env_name_impl at ast.py:2116>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2116
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_1, Allow_Bodies => True);
--# end
--# expr-done 2282
Scope_Result := Fld_1;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 2283 '<IsA PrivatePart at ast.py:2120>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2120
Is_A := Var_Expr /= null 
and then Var_Expr.Kind in Ada_Private_Part_Range; 
--# expr-done 2283
if Is_A then
--# expr-start 2285 '<Cast to BasicDecl at ast.py:2121>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2121
--# expr-start 2284 '<parent at ast.py:2121>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2121
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr.Parent; 
--# expr-done 2284
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Fld_2;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2285
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 2286 '<FieldAccess for has_top_level_env_name_impl at ast.py:2122>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2122
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_2, Allow_Bodies => True);
--# end
--# expr-done 2286
Scope_Result_1 := Fld_3;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
   Result_Var_1 := If_Result;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 2287
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_134;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_134;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Has_Top_Level_Env_Name;
--# end
--# property-start BasicDecl.env_spec_fully_qualified_name_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2131
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl
  (Node : Bare_Basic_Decl
      ; Self_Env : Lexical_Env
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind self_env Self_Env
   Property_Result : String_Type;
            procedure Finalizer_Scope_135 with Inline_Always;
            procedure Finalizer_Scope_2141 with Inline_Always;
      Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_2 : Bare_Private_Part;
Fld : Boolean;
Fld_1 : Bare_Compilation_Unit;
Fld_2 : Symbol_Type_Array_Access;
Str : String_Type;
Fld_3 : String_Type;
Is_A : Boolean;
Fld_4 : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Fld_5 : Bare_Ada_Node;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : String_Type;
Env_Node : Bare_Ada_Node;
Cast_Result_1 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Env_Node_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Private_Part;
Fld_8 : Bare_Ada_Node;
Scope_Result_1 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result_3 : Bare_Basic_Decl;
Result_Var_1 : Bare_Basic_Decl;
Fld_9 : Lexical_Env := Empty_Env;
Fld_10 : String_Type;
Str_1 : String_Type;
Concat_Result : String_Type;
Fld_11 : Symbol_Type;
Fld_12 : String_Type;
Concat_Result_1 : String_Type;
Scope_Result_2 : String_Type;
Result_Var_2 : String_Type;
If_Result : String_Type;
If_Result_1 : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_135 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_135;
            procedure Finalizer_Scope_2141 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_2141;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2313 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2312 '<Cond at ast.py:2136>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2136
--# expr-start 2288 '<FieldAccess for is_compilation_unit_root at ast.py:2139>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2139
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2288
if Fld then
   --# expr-start 2292 '<FieldAccess for sym_join at ast.py:2140>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2140
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2290 '<FieldAccess for syntactic_fully_qualified_name at ast.py:2141>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2141
--# expr-start 2289 '<FieldAccess for enclosing_compilation_unit at ast.py:2141>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2141
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_1 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 2289
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Fld_1);
--# end
--# expr-done 2290
--# expr-start 2291 '<String at ast.py:2142>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2142
Str := Create_String ("."); 
--# expr-done 2291
--# property-call-start AdaNode.sym_join
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Self, Syms => Fld_2, Sep => Str);
--# end
--# expr-done 2292
   If_Result_1 := Fld_3;
else
--# expr-start 2293 '<IsA GenericPackageInternal, GenericSubpInternal at ast.py:2147>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2147
Is_A := Self /= null 
and then Self.Kind in Ada_Generic_Package_Internal_Range | Ada_Generic_Subp_Internal_Range; 
--# expr-done 2293
if Is_A then
   --# expr-start 2298 '<FieldAccess for env_spec_fully_qualified_name_impl at ast.py:2148>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2148
--# expr-start 2295 '<Cast to BasicDecl at ast.py:2148>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2148
--# expr-start 2294 '<parent at ast.py:2148>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2148
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Parent; 
--# expr-done 2294
   if Fld_4 = null
      or else Fld_4.Kind in Ada_Basic_Decl
   then
      Cast_Result := Fld_4;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2295
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2297 '<FieldAccess for node_env at ast.py:2149>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2149
--# expr-start 2296 '<parent at ast.py:2149>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2149
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Parent; 
--# expr-done 2296
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Node_Env (Node => Fld_5);
--# expr-done 2297
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Cast_Result, Self_Env => Fld_6);
--# end
--# expr-done 2298
   If_Result := Fld_7;
else
--# expr-start 2300 '<Cast to BasicDecl at ast.py:2153>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2153
--# expr-start 2299 '<env_node at ast.py:2153>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2153
Env_Node := AST_Envs.Env_Node (Self_Env); 
--# expr-done 2299
   if Env_Node = null
      or else Env_Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Env_Node;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2300
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result;
else
   --# expr-start 2304 '<Cast to BasicDecl at ast.py:2154>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2154
--# expr-start 2302 '<Cast to PrivatePart at ast.py:2154>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2154
--# expr-start 2301 '<env_node at ast.py:2154>' Env_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2154
Env_Node_1 := AST_Envs.Env_Node (Self_Env); 
--# expr-done 2301
   if Env_Node_1 = null
      or else Env_Node_1.Kind in Ada_Private_Part_Range
   then
      Cast_Result_2 := Env_Node_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 2302
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2303 '<parent at ast.py:2154>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2154
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Var_Expr_2.Parent; 
--# expr-done 2303
Scope_Result_1 := Fld_8;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Result_Var;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 2304
   Result_Var_1 := Cast_Result_3;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr
--# expr-start 2311 '<Concat at ast.py:2160>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2160
--# expr-start 2308 '<Concat at ast.py:2159>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2159
--# expr-start 2306 '<FieldAccess for env_spec_fully_qualified_name_impl at ast.py:2158>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2158
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2305 '<FieldAccess for node_env at ast.py:2158>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2158
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Node_Env (Node => Var_Expr);
--# expr-done 2305
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Var_Expr, Self_Env => Fld_9);
--# end
--# expr-done 2306
--# expr-start 2307 '<String at ast.py:2159>' Str_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2159
Str_1 := Create_String ("."); 
--# expr-done 2307
Concat_Result := Concat_String (Fld_10, Str_1); 
--# expr-done 2308
--# expr-start 2310 '<FieldAccess for image at ast.py:2160>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2160
--# expr-start 2309 '<FieldAccess for name_symbol at ast.py:2160>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2160
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 2309
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Image (Self => Fld_11);
--# expr-done 2310
Concat_Result_1 := Concat_String (Concat_Result, Fld_12); 
--# expr-done 2311
Scope_Result_2 := Concat_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2141;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_String;
end if;
      Inc_Ref (Result_Var_2);
   If_Result := Result_Var_2;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2312
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2313
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_135;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_135;
                     Finalizer_Scope_2141;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl;
--# end
--# property-start BasicDecl.env_spec_fully_qualified_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2164
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Spec_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : String_Type;
            procedure Finalizer_Scope_136 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_136 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_136;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2316 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2315 '<FieldAccess for env_spec_fully_qualified_name_impl at ast.py:2174>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2174
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2314 '<FieldAccess for children_env at ast.py:2174>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2174
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 2314
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Self, Self_Env => Fld);
--# end
--# expr-done 2315
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2316
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_136;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_136;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Env_Spec_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.top_level_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2176
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Top_Level_Env_Name
  (Node : Bare_Basic_Decl
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : String_Type;
            procedure Finalizer_Scope_137 with Inline_Always;
      Fld : String_Type;
Fld_1 : Boolean;
If_Result : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_137 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_137;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2320 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2319 '<If at ast.py:2183>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2183
--# expr-start 2317 '<FieldAccess for has_top_level_env_name at ast.py:2184>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2184
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 2317
if Fld_1 then
   --# expr-start 2318 '<FieldAccess for env_spec_fully_qualified_name at ast.py:2185>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2185
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.env_spec_fully_qualified_name
Fld := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 2318
   If_Result := Fld;
else
   If_Result := Empty_String;
end if;
      Inc_Ref (If_Result);
--# expr-done 2319
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2320
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_137;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_137;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Top_Level_Env_Name;
--# end
--# property-start '[dispatcher]BasicDecl.child_decl_initial_env_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Basic_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Entry_Index_Spec | Ada_Exception_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.child_decl_initial_env_name
                  Property_Result := Basic_Decl_P_Child_Decl_Initial_Env_Name
                    (Self
                        , Private_Part
                    );
                  --# end
               when Ada_Error_Decl =>
                  --# property-call-start ErrorDecl.child_decl_initial_env_name
                  Property_Result := Error_Decl_P_Child_Decl_Initial_Env_Name
                    (Self
                        , Private_Part
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name;
--# end
--# property-start BasicDecl.child_decl_initial_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2223
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Child_Decl_Initial_Env
  (Node : Bare_Basic_Decl
      ; Private_Part : Boolean
         := False
  )
   return Internal_Designated_Env
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_139 with Inline_Always;
            procedure Finalizer_Scope_2143 with Inline_Always;
      Var_Expr : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Lexical_Env := Empty_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Let_Result : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_139 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_139;
            procedure Finalizer_Scope_2143 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2143;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2325 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2321 '<FieldAccess for child_decl_initial_env_name at ast.py:2233>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2233
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => Private_Part);
--# end
--# expr-done 2321
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 2322 '<New DesignatedEnv at ast.py:2234>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2234
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr, Kind => Named_Env); 
--# expr-done 2322
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2143;
   Result_Var := Scope_Result;
else
   --# expr-start 2324 '<New DesignatedEnv at ast.py:2235>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2235
--# expr-start 2323 '<FieldAccess for default_initial_env at ast.py:2235>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2235
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 2323
Inc_Ref (Fld_1);
New_Struct_1 := (Direct_Env => Fld_1, Env_Name => null, Kind => Direct_Env); 
--# expr-done 2324
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2325
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_139;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_139;
                     Finalizer_Scope_2143;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Child_Decl_Initial_Env;
--# end
--# property-start BasicDecl.basic_decl_env_assocs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2238
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Basic_Decl_Env_Assocs
  (Node : Bare_Basic_Decl
      ; Dest_Env : Internal_Designated_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dest_env Dest_Env
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_141 with Inline_Always;
      Block_Var : Symbol_Type;
Block_Var_1 : Internal_Env_Assoc_Array_Access;
Block_Var_2 : Internal_Env_Assoc_Array_Access;
Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Cast_Expr : Bare_Basic_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Singleton : Internal_Env_Assoc_Array_Access;
Sym : Symbol_Type;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Singleton_1 : Internal_Env_Assoc_Array_Access;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Concat_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_141 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Singleton);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_141;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2335 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2326 '<FieldAccess for name_symbol at ast.py:2248>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2248
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2326
Block_Var := Fld; 
--# bind name Block_Var
--# expr-start 2328 '<New EnvAssoc at ast.py:2251>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2251
--# expr-start 2327 '<FieldAccess for name_symbol at ast.py:2252>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2252
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2327
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Dest_Env);
New_Struct := (Dest_Env => Dest_Env, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 2328
                Singleton := Create_Internal_Env_Assoc_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
                Inc_Ref (Singleton.Items (1));
Block_Var_1 := Singleton; Inc_Ref (Block_Var_1);
--# bind base_assoc Block_Var_1
--# expr-start 2333 '<If at ast.py:2258>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2258
--# expr-start 2330 '<Eq at ast.py:2259>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2259
--# expr-start 2329 '<Symbol "=" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 2329
Is_Equal := Block_Var = Sym_1; 
--# expr-done 2330
if Is_Equal then
   --# expr-start 2332 '<New EnvAssoc at ast.py:2260>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2260
--# expr-start 2331 '<Symbol "/=" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 2331
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Inc_Ref (Dest_Env);
New_Struct_1 := (Dest_Env => Dest_Env, Key => Sym, Metadata => No_Metadata, Value => Cast_Result_1); 
--# expr-done 2332
                Singleton_1 := Create_Internal_Env_Assoc_Array (Items_Count => 1);
                Singleton_1.Items (1) := New_Struct_1;
                Inc_Ref (Singleton_1.Items (1));
   If_Result := Singleton_1;
else
   If_Result := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2333
Block_Var_2 := If_Result; Inc_Ref (Block_Var_2);
--# bind implicit_neq_assoc Block_Var_2
--# expr-start 2334 '<Concat at ast.py:2268>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2268
Concat_Result := Concat (Block_Var_1, Block_Var_2); 
--# expr-done 2334
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2335
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_141;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_141;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Basic_Decl_Env_Assocs;
--# end
--# property-start BasicDecl.child_decl_env_assocs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2270
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Child_Decl_Env_Assocs
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_142 with Inline_Always;
            procedure Finalizer_Scope_2144 with Inline_Always;
      Block_Var : Internal_Designated_Env := No_Designated_Env;
Var_Expr : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_142 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_142;
            procedure Finalizer_Scope_2144 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2144;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2340 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2336 '<FieldAccess for child_decl_initial_env_name at ast.py:2282>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2282
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => False);
--# end
--# expr-done 2336
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind non_null_name Var_Expr
--# expr-start 2337 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr, Kind => Named_Env); 
--# expr-done 2337
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2144;
   Result_Var := Scope_Result;
else
   --# expr-start 2338 '<New DesignatedEnv at ast.py:2281>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2281
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 2338
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
Block_Var := Result_Var; Inc_Ref (Block_Var);
--# bind dest_env Block_Var
--# expr-start 2339 '<FieldAccess for basic_decl_env_assocs at ast.py:2285>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2285
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.basic_decl_env_assocs
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Env_Assocs (Node => Ent.Node, Dest_Env => Block_Var, E_Info => Ent.Info);
--# end
--# expr-done 2339
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2340
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_142;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_142;
                     Finalizer_Scope_2144;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Child_Decl_Env_Assocs;
--# end
--# property-start BasicDecl.is_formal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2287
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Formal
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2342 '<IsA GenericFormal at ast.py:2288>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2288
--# expr-start 2341 '<parent at ast.py:2288>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2288
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2341
Is_A := Fld /= null 
and then Fld.Kind in Ada_Generic_Formal; 
--# expr-done 2342
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Formal;
--# end
--# property-start '[dispatcher]BasicDecl.previous_part_for_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.previous_part_for_decl
                  Property_Result := Basic_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start BaseFormalParamDecl.previous_part_for_decl
                  Property_Result := Base_Formal_Param_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.previous_part_for_decl
                  Property_Result := Object_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BasicDecl.canonical_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2343
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Part
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Canonical_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 2345 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2343 '<FieldAccess for previous_part_for_decl at ast.py:2351>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2351
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2343
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 2344 '<FieldAccess for canonical_part at ast.py:2352>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2352
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 2344
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2345
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Basic_Decl_P_Canonical_Part;
--# end
--# property-start BasicDecl.all_previous_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2355
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Previous_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_148 with Inline_Always;
            procedure Finalizer_Scope_2146 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_148 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_148;
            procedure Finalizer_Scope_2146 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2146;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2351 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2346 '<FieldAccess for previous_part_for_decl at ast.py:2362>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2362
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2346
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 2350 '<If at ast.py:2363>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2363
--# expr-start 2347 '<Eq at ast.py:2364>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2364
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2347
if Is_Equiv then
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
else
   --# expr-start 2349 '<Concat at ast.py:2366>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2366
--# expr-start 2348 '<FieldAccess for all_previous_parts at ast.py:2366>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2366
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_previous_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Previous_Parts (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 2348
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Concat_Result := Concat (Fld_1, Singleton); 
--# expr-done 2349
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 2350
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2146;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2351
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_148;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_148;
                     Finalizer_Scope_2146;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_All_Previous_Parts;
--# end
--# property-start BasicDecl.all_next_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2370
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Next_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_149 with Inline_Always;
            procedure Finalizer_Scope_2151 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_149 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_149;
            procedure Finalizer_Scope_2151 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2151;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2357 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2352 '<FieldAccess for next_part_for_decl at ast.py:2377>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2377
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2352
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 2356 '<If at ast.py:2378>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2378
--# expr-start 2353 '<Eq at ast.py:2379>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2379
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2353
if Is_Equiv then
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
else
   --# expr-start 2355 '<Concat at ast.py:2381>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2381
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
--# expr-start 2354 '<FieldAccess for all_next_parts at ast.py:2381>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2381
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_next_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Next_Parts (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 2354
Concat_Result := Concat (Singleton, Fld_1); 
--# expr-done 2355
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 2356
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2151;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2357
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_149;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_149;
                     Finalizer_Scope_2151;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_All_Next_Parts;
--# end
--# property-start BasicDecl.all_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2385
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_150 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_150 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_150;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2362 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2358 '<FieldAccess for all_previous_parts at ast.py:2392>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2392
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_previous_parts
Fld := Libadalang.Implementation.Basic_Decl_P_All_Previous_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2358
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind prevs Block_Var
--# expr-start 2359 '<FieldAccess for all_next_parts at ast.py:2393>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2393
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_next_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Next_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2359
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind nexts Block_Var_1
--# expr-start 2361 '<Concat at ast.py:2394>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2394
--# expr-start 2360 '<Concat at ast.py:2394>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2394
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
Concat_Result := Concat (Block_Var, Singleton); 
--# expr-done 2360
Concat_Result_1 := Concat (Concat_Result, Block_Var_1); 
--# expr-done 2361
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2362
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_150;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_150;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_All_Parts;
--# end
--# property-start '[dispatcher]BasicDecl.is_static_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Static_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl | Ada_Component_Decl .. Ada_Package_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Entry_Decl | Ada_Generic_Subp_Internal | Ada_Accept_Stmt_Body .. Ada_Named_Stmt_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_static_decl
                  Property_Result := Basic_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.is_static_decl
                  Property_Result := Anonymous_Expr_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl =>
                  --# property-call-start DiscreteBaseSubtypeDecl.is_static_decl
                  Property_Result := Discrete_Base_Subtype_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.is_static_decl
                  Property_Result := Subtype_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_static_decl
                  Property_Result := Type_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start EnumLiteralDecl.is_static_decl
                  Property_Result := Enum_Literal_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Subp_Decl =>
                  --# property-call-start SyntheticSubpDecl.is_static_decl
                  Property_Result := Synthetic_Subp_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.is_static_decl
                  Property_Result := Number_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.is_static_decl
                  Property_Result := Object_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_Static_Decl;
--# end
--# property-start BasicDecl.unshed_rebindings /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2403
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unshed_Rebindings
  (Node : Bare_Basic_Decl
      ; Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind rebindings Rebindings
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_152 with Inline_Always;
      Is_Equal : Boolean;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Basic_Decl;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Basic_Decl;
If_Result : Internal_Entity_Basic_Decl;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_152 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_152;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2371 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2370 '<Cond at ast.py:2411>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2411
--# expr-start 2363 '<Eq at ast.py:2412>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2412
Is_Equal := Rebindings = null; 
--# expr-done 2363
if Is_Equal then
   If_Result_1 := Ent;
else
--# expr-start 2366 '<Eq at ast.py:2415>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2415
--# expr-start 2365 '<env_node at ast.py:2415>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2415
--# expr-start 2364 '<FieldAccess for old_env at ast.py:2415>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2415
      if Rebindings = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Rebindings.Old_Env;
Inc_Ref (Fld);
--# expr-done 2364
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 2365
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Env_Node = Cast_Result; 
--# expr-done 2366
if Is_Equal_1 then
   --# expr-start 2367 '<FieldAccess for unshed_rebindings_helper at ast.py:2416>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2416
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.unshed_rebindings_helper
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings_Helper (Node => Ent.Node, Rebindings => Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 2367
   If_Result := Fld_1;
else
   --# expr-start 2369 '<FieldAccess for unshed_rebindings at ast.py:2418>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2418
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2368 '<FieldAccess for get_parent at ast.py:2418>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2418
      if Rebindings = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Rebindings.Parent;
--# expr-done 2368
--# property-call-start BasicDecl.unshed_rebindings
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Ent.Node, Rebindings => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 2369
   If_Result := Fld_3;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 2370
Let_Result := If_Result_1; 
--# end
--# expr-done 2371
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_152;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_152;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Unshed_Rebindings;
--# end
--# property-start BasicDecl.unshed_rebindings_helper /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2421
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unshed_Rebindings_Helper
  (Node : Bare_Basic_Decl
      ; Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind rebindings Rebindings
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Info;
Fld_3 : Env_Rebindings;
Is_Equal_1 : Boolean;
Fld_4 : Env_Rebindings;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Is_Equal_2 : Boolean;
If_Result : Boolean;
Fld_7 : Internal_Entity_Info;
Fld_8 : Internal_Metadata;
Fld_9 : Internal_Entity_Info;
Fld_10 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Basic_Decl;
If_Result_1 : Internal_Entity_Basic_Decl;
If_Result_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2389 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2388 '<Cond at ast.py:2428>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2428
--# expr-start 2374 '<Eq at ast.py:2431>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2431
--# expr-start 2373 '<FieldAccess for rebindings at ast.py:2431>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2431
--# expr-start 2372 '<FieldAccess for info at ast.py:2431>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2431
Fld := Ent.Info;
--# expr-done 2372
Fld_1 := Fld.Rebindings;
--# expr-done 2373
Is_Equal := Fld_1 = Rebindings; 
--# expr-done 2374
if Is_Equal then
   If_Result_2 := Ent;
else
--# expr-start 2377 '<Eq at ast.py:2435>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2435
--# expr-start 2376 '<FieldAccess for rebindings at ast.py:2435>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2435
--# expr-start 2375 '<FieldAccess for info at ast.py:2435>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2435
Fld_2 := Ent.Info;
--# expr-done 2375
Fld_3 := Fld_2.Rebindings;
--# expr-done 2376
Is_Equal_1 := Fld_3 = null; 
--# expr-done 2377
if Is_Equal_1 then
   If_Result := True;
else
   --# expr-start 2381 '<Eq at ast.py:2436>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2436
--# expr-start 2378 '<FieldAccess for get_parent at ast.py:2436>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2436
      if Rebindings = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Rebindings.Parent;
--# expr-done 2378
--# expr-start 2380 '<FieldAccess for rebindings at ast.py:2436>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2436
--# expr-start 2379 '<FieldAccess for info at ast.py:2436>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2436
Fld_5 := Ent.Info;
--# expr-done 2379
Fld_6 := Fld_5.Rebindings;
--# expr-done 2380
Is_Equal_2 := Fld_4 = Fld_6; 
--# expr-done 2381
   If_Result := Is_Equal_2;
end if;
if If_Result then
   --# expr-start 2387 '<New BasicDecl.entity at ???>' New_Struct_1 None
--# expr-start 2386 '<New EntityInfo at ast.py:2441>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2441
--# expr-start 2383 '<FieldAccess for from_rebound at ast.py:2444>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2444
--# expr-start 2382 '<FieldAccess for info at ast.py:2444>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2444
Fld_9 := Ent.Info;
--# expr-done 2382
Fld_10 := Fld_9.From_Rebound;
--# expr-done 2383
--# expr-start 2385 '<FieldAccess for md at ast.py:2443>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2443
--# expr-start 2384 '<FieldAccess for info at ast.py:2443>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2443
Fld_7 := Ent.Info;
--# expr-done 2384
Fld_8 := Fld_7.Md;
--# expr-done 2385
New_Struct := (From_Rebound => Fld_10, Md => Fld_8, Rebindings => Rebindings); 
--# expr-done 2386
New_Struct_1 := (Info => New_Struct, Node => Self); 
--# expr-done 2387
   If_Result_1 := New_Struct_1;
else
   If_Result_1 := raise Property_Error with "Incorrect rebindings";
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 2388
Let_Result := If_Result_2; 
--# end
--# expr-done 2389
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Unshed_Rebindings_Helper;
--# end
--# property-start '[dispatcher]BasicDecl.is_library_item' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Library_Item
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Synthetic_Formal_Param_Decl | Ada_Package_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_library_item
                  Property_Result := Basic_Decl_P_Is_Library_Item
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Internal =>
                  --# property-call-start GenericPackageInternal.is_library_item
                  Property_Result := Generic_Package_Internal_P_Is_Library_Item
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_Library_Item;
--# end
--# property-start BasicDecl.decl_private_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2458
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Decl_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Private_Part
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Private_Part;
      Match_0 : Internal_Entity_Base_Package_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Single_Task_Decl;
Match_3 : Internal_Entity_Protected_Type_Decl;
Match_4 : Internal_Entity_Single_Protected_Decl;
Match_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Private_Part;
Fld_1 : Internal_Entity_Task_Def;
Fld_2 : Internal_Entity_Private_Part;
Fld_3 : Internal_Entity_Single_Task_Type_Decl;
Fld_4 : Internal_Entity_Task_Def;
Fld_5 : Internal_Entity_Private_Part;
Fld_6 : Internal_Entity_Protected_Def;
Fld_7 : Internal_Entity_Private_Part;
Fld_8 : Internal_Entity_Protected_Def;
Fld_9 : Internal_Entity_Private_Part;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Private_Part;
Scope_Result : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Private_Part;
Scope_Result_1 : Internal_Entity_Private_Part;
Cast_Result_2 : Internal_Entity_Single_Task_Decl;
Let_Result_2 : Internal_Entity_Private_Part;
Scope_Result_2 : Internal_Entity_Private_Part;
Cast_Result_3 : Internal_Entity_Protected_Type_Decl;
Let_Result_3 : Internal_Entity_Private_Part;
Scope_Result_3 : Internal_Entity_Private_Part;
Cast_Result_4 : Internal_Entity_Single_Protected_Decl;
Let_Result_4 : Internal_Entity_Private_Part;
Scope_Result_4 : Internal_Entity_Private_Part;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Private_Part;
Scope_Result_5 : Internal_Entity_Private_Part;
Match_Result : Internal_Entity_Private_Part;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2400 '<Match at ast.py:2458>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2458
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bpd Match_0
--# expr-start 2390 '<FieldAccess for private_part at ast.py:2459>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2459
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Private_Part (Node => Match_0.Node.Base_Package_Decl_F_Private_Part, Info => Match_0.Info);
--# expr-done 2390
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ttd Match_1
--# expr-start 2392 '<FieldAccess for private_part at ast.py:2460>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2460
--# expr-start 2391 '<FieldAccess for definition at ast.py:2460>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2460
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Task_Def (Node => Match_1.Node.Task_Type_Decl_F_Definition, Info => Match_1.Info);
--# expr-done 2391
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Fld_1.Node.Task_Def_F_Private_Part, Info => Fld_1.Info);
--# expr-done 2392
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind td Match_2
--# expr-start 2395 '<FieldAccess for private_part at ast.py:2461>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2461
--# expr-start 2394 '<FieldAccess for definition at ast.py:2461>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2461
--# expr-start 2393 '<FieldAccess for task_type at ast.py:2461>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2461
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Single_Task_Type_Decl (Node => Match_2.Node.Single_Task_Decl_F_Task_Type, Info => Match_2.Info);
--# expr-done 2393
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Task_Def (Node => Fld_3.Node.Task_Type_Decl_F_Definition, Info => Fld_3.Info);
--# expr-done 2394
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Private_Part (Node => Fld_4.Node.Task_Def_F_Private_Part, Info => Fld_4.Info);
--# expr-done 2395
Let_Result_2 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind ptd Match_3
--# expr-start 2397 '<FieldAccess for private_part at ast.py:2462>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2462
--# expr-start 2396 '<FieldAccess for definition at ast.py:2462>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2462
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Protected_Def (Node => Match_3.Node.Protected_Type_Decl_F_Definition, Info => Match_3.Info);
--# expr-done 2396
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Private_Part (Node => Fld_6.Node.Protected_Def_F_Private_Part, Info => Fld_6.Info);
--# expr-done 2397
Let_Result_3 := Fld_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Single_Protected_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind spd Match_4
--# expr-start 2399 '<FieldAccess for private_part at ast.py:2463>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2463
--# expr-start 2398 '<FieldAccess for definition at ast.py:2463>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2463
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Protected_Def (Node => Match_4.Node.Single_Protected_Decl_F_Definition, Info => Match_4.Info);
--# expr-done 2398
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Private_Part (Node => Fld_8.Node.Protected_Def_F_Private_Part, Info => Fld_8.Info);
--# expr-done 2399
Let_Result_4 := Fld_9; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Formal_Param_Decl | Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Package_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_5 := No_Entity_Private_Part; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 2400
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Decl_Private_Part;
--# end
--# property-start '[dispatcher]BasicDecl.declarative_parts' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Declarative_Parts
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Synthetic_Formal_Param_Decl | Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Task_Body_Stub | Ada_Entry_Index_Spec .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Package_Renaming_Decl | Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.declarative_parts
                  Property_Result := Basic_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.declarative_parts
                  Property_Result := Base_Package_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.declarative_parts
                  Property_Result := Protected_Type_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.declarative_parts
                  Property_Result := Task_Type_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body =>
                  --# property-call-start SubpBody.declarative_parts
                  Property_Result := Subp_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.declarative_parts
                  Property_Result := Entry_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.declarative_parts
                  Property_Result := Package_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body =>
                  --# property-call-start ProtectedBody.declarative_parts
                  Property_Result := Protected_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body =>
                  --# property-call-start TaskBody.declarative_parts
                  Property_Result := Task_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.declarative_parts
                  Property_Result := Generic_Package_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.declarative_parts
                  Property_Result := Single_Protected_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Declarative_Parts;
--# end
--# property-start BasicDecl.immediate_declarative_region /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2475
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Immediate_Declarative_Region
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_157 with Inline_Always;
            procedure Finalizer_Scope_2158 with Inline_Always;
            procedure Finalizer_Scope_2159 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl_Array_Access;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Declarative_Part_Array_Access;
Fld_2 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Map_Result_1 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_157 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_157;
            procedure Finalizer_Scope_2158 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2158;
            procedure Finalizer_Scope_2159 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2159;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Immediate_Declarative_Region,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 2408 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2407 '<env_group at ast.py:2481>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2481
--# expr-start 2406 '<Mapcat at ast.py:2477>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2477
   --# expr-start 2402 '<Concat at ast.py:2477>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2477
--# expr-start 2401 '<FieldAccess for all_previous_parts at ast.py:2477>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2477
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_previous_parts
Fld := Libadalang.Implementation.Basic_Decl_P_All_Previous_Parts (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2401
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
Concat_Result := Concat (Fld, Singleton); 
--# expr-done 2402
      declare
         Map_Result_1_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_97 of
               Collection.Items
         loop
   --# scope-start
               --# bind part Item_97
      --# expr-start 2405 '<Map at ast.py:2478>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2478
   --# expr-start 2403 '<FieldAccess for declarative_parts at ast.py:2478>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2478
      if Item_97.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.declarative_parts'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Declarative_Parts (Node => Item_97.Node, E_Info => Item_97.Info);
--# end
--# expr-done 2403
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Declarative_Part_Array_Access := Fld_1;
      begin
         for Item_98 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_98
      --# expr-start 2404 '<FieldAccess for children_env at ast.py:2479>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2479
      if Item_98.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Item_98.Node, E_Info => Item_98.Info);
--# expr-done 2404
         declare
            Item_To_Append : constant Lexical_Env := Fld_2;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2159;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2405
         for Item_To_Append of
               Map_Result.Items
         loop
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2158;
         end loop;
      end;
         Map_Result_1 := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_1_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2406
Group_Env := Group (Map_Result_1, No_Metadata); 
--# expr-done 2407
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 2408
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_157;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_157;
                     Finalizer_Scope_2158;
                     Finalizer_Scope_2159;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Basic_Decl_P_Immediate_Declarative_Region;
--# end
--# property-start '[dispatcher]BasicDecl.get_aspect_assoc' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.get_aspect_assoc
                  Property_Result := Basic_Decl_P_Get_Aspect_Assoc
                    (Self
                        , Name
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.get_aspect_assoc
                  Property_Result := Classwide_Type_Decl_P_Get_Aspect_Assoc
                    (Self
                        , Name
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericDecl.get_aspect_assoc
                  Property_Result := Generic_Decl_P_Get_Aspect_Assoc
                    (Self
                        , Name
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start BasicDecl.get_aspect_spec_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2496
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Spec_Expr
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Expr;
      Var_Expr : Internal_Entity_Aspect_Assoc;
Fld : Internal_Entity_Aspect_Assoc;
Fld_1 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2411 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2409 '<FieldAccess for get_aspect_assoc at ast.py:2502>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2502
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 2409
Var_Expr := Fld;
if Var_Expr /= No_Entity_Aspect_Assoc then
   --# scope-start
--# expr-start 2410 '<FieldAccess for expr at ast.py:2502>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2502
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Aspect_Assoc_F_Expr, Info => Var_Expr.Info);
--# expr-done 2410
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2411
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Aspect_Spec_Expr;
--# end
--# property-start BasicDecl.library_item_pragmas /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2504
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Library_Item_Pragmas
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node_List
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Pragma_Node_List;
      Fld : Internal_Entity;
Is_A : Boolean;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_3 : Internal_Entity_Pragma_Node_List;
Fld_4 : Internal_Entity;
Fld_5 : Internal_Entity;
Is_A_1 : Boolean;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity;
Fld_8 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Compilation_Unit;
Fld_9 : Internal_Entity_Pragma_Node_List;
If_Result : Internal_Entity_Pragma_Node_List;
If_Result_1 : Internal_Entity_Pragma_Node_List;
Let_Result : Internal_Entity_Pragma_Node_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2427 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2426 '<Cond at ast.py:2509>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2509
--# expr-start 2413 '<IsA LibraryItem at ast.py:2510>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2510
--# expr-start 2412 '<parent at ast.py:2510>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2510
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2412
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 2413
if Is_A then
   --# expr-start 2417 '<FieldAccess for pragmas at ast.py:2511>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2511
--# expr-start 2416 '<Cast to CompilationUnit at ast.py:2511>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2511
--# expr-start 2415 '<parent at ast.py:2511>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2511
--# expr-start 2414 '<parent at ast.py:2511>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2511
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2414
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 2415
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Compilation_Unit;
   end if;
--# expr-done 2416
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Pragma_Node_List (Node => Cast_Result.Node.Compilation_Unit_F_Pragmas, Info => Cast_Result.Info);
--# expr-done 2417
   If_Result_1 := Fld_3;
else
--# expr-start 2420 '<IsA LibraryItem at ast.py:2513>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2513
--# expr-start 2419 '<parent at ast.py:2513>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2513
--# expr-start 2418 '<parent at ast.py:2513>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2513
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2418
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 2419
Is_A_1 := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 2420
if Is_A_1 then
   --# expr-start 2425 '<FieldAccess for pragmas at ast.py:2514>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2514
--# expr-start 2424 '<Cast to CompilationUnit at ast.py:2514>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2514
--# expr-start 2423 '<parent at ast.py:2514>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2514
--# expr-start 2422 '<parent at ast.py:2514>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2514
--# expr-start 2421 '<parent at ast.py:2514>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2514
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2421
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# expr-done 2422
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Parent (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# expr-done 2423
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_1 := No_Entity_Compilation_Unit;
   end if;
--# expr-done 2424
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Pragma_Node_List (Node => Cast_Result_1.Node.Compilation_Unit_F_Pragmas, Info => Cast_Result_1.Info);
--# expr-done 2425
   If_Result := Fld_9;
else
   If_Result := No_Entity_Pragma_Node_List;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 2426
Let_Result := If_Result_1; 
--# end
--# expr-done 2427
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Library_Item_Pragmas;
--# end
--# property-start BasicDecl.get_aspect /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2519
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Aspect;
Scope_Result : Internal_Aspect;
Result_Var : Internal_Aspect;
Let_Result : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2430 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2428 '<FieldAccess for defining_name_or_raise at ast.py:2534>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2534
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2428
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2429 '<FieldAccess for get_aspect at ast.py:2534>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2534
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Aspect (Node => Var_Expr.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 2429
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Aspect;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2430
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Aspect;
--# end
--# property-start BasicDecl.has_aspect /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2536
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Aspect
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2433 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2431 '<FieldAccess for defining_name_or_raise at ast.py:2548>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2431
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2432 '<FieldAccess for has_aspect at ast.py:2548>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2548
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.has_aspect
Fld_1 := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Var_Expr.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 2432
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2433
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Has_Aspect;
--# end
--# property-start BasicDecl.get_pragma /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2550
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Pragma
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Pragma_Node;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Pragma_Node;
Scope_Result : Internal_Entity_Pragma_Node;
Result_Var : Internal_Entity_Pragma_Node;
Let_Result : Internal_Entity_Pragma_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2436 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2434 '<FieldAccess for defining_name_or_raise at ast.py:2559>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2559
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2434
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2435 '<FieldAccess for get_pragma at ast.py:2559>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2559
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_pragma
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Pragma (Node => Var_Expr.Node, Name => Name, E_Info => Var_Expr.Info);
--# end
--# expr-done 2435
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Pragma_Node;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2436
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Pragma;
--# end
--# property-start BasicDecl.get_representation_clause /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2561
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Representation_Clause
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Attribute_Def_Clause
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Attribute_Def_Clause;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Attribute_Def_Clause;
Scope_Result : Internal_Entity_Attribute_Def_Clause;
Result_Var : Internal_Entity_Attribute_Def_Clause;
Let_Result : Internal_Entity_Attribute_Def_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2439 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2437 '<FieldAccess for defining_name_or_raise at ast.py:2568>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2568
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2437
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2438 '<FieldAccess for get_representation_clause at ast.py:2568>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2568
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_representation_clause
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Representation_Clause (Node => Var_Expr.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 2438
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Attribute_Def_Clause;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2439
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Representation_Clause;
--# end
--# property-start BasicDecl.get_at_clause /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2570
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_At_Clause
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_At_Clause
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_At_Clause;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_At_Clause;
Scope_Result : Internal_Entity_At_Clause;
Result_Var : Internal_Entity_At_Clause;
Let_Result : Internal_Entity_At_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2442 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2440 '<FieldAccess for defining_name_or_raise at ast.py:2576>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2576
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2440
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2441 '<FieldAccess for get_at_clause at ast.py:2576>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2576
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_at_clause
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_At_Clause (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 2441
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_At_Clause;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2442
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_At_Clause;
--# end
--# property-start '[dispatcher]BasicDecl.is_imported' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Imported
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_imported
                  Property_Result := Basic_Decl_P_Is_Imported
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.is_imported
                  Property_Result := Generic_Subp_Decl_P_Is_Imported
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_Imported;
--# end
--# property-start BasicDecl.is_ghost_code /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2585
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Ghost_Code
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2445 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2443 '<FieldAccess for defining_name_or_raise at ast.py:2590>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2590
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2443
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2444 '<FieldAccess for is_ghost_code at ast.py:2590>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2590
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_ghost_code
Fld_1 := Libadalang.Implementation.Defining_Name_P_Is_Ghost_Code (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2444
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2445
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Ghost_Code;
--# end
--# property-start BasicDecl.get_instantiation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2592
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Instantiation
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Instantiation
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Instantiation;
            procedure Finalizer_Scope_2170 with Inline_Always;
      Block_Var : Bare_Generic_Instantiation;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Env_Rebindings;
Var_Expr_1 : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
Scope_Result : Bare_Generic_Instantiation;
Result_Var : Bare_Generic_Instantiation;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Info;
Fld_5 : Env_Rebindings;
Fld_6 : Internal_Entity_Info;
Fld_7 : Env_Rebindings;
Fld_8 : Env_Rebindings;
Fld_9 : Internal_Entity_Info;
Fld_10 : Boolean;
New_Struct : Internal_Metadata;
New_Struct_1 : Internal_Entity_Info;
New_Struct_2 : Internal_Entity_Generic_Instantiation;
Fld_11 : Bare_Basic_Decl;
Fld_12 : Bare_Ada_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Generic_Instantiation;
Let_Result : Internal_Entity_Generic_Instantiation;
            procedure Finalizer_Scope_2170 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2170;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2467 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2447 '<FieldAccess for rebindings at ast.py:2605>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2605
--# expr-start 2446 '<FieldAccess for info at ast.py:2605>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2605
Fld := Ent.Info;
--# expr-done 2446
Fld_1 := Fld.Rebindings;
--# expr-done 2447
Var_Expr := Fld_1;
if Var_Expr /= null then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 2450 '<Cast to GenericInstantiation at ast.py:2606>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2606
--# expr-start 2449 '<env_node at ast.py:2606>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2606
--# expr-start 2448 '<FieldAccess for new_env at ast.py:2606>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2606
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr.New_Env;
Inc_Ref (Fld_2);
--# expr-done 2448
Env_Node := AST_Envs.Env_Node (Fld_2); 
--# expr-done 2449
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Env_Node;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2450
Scope_Result := Cast_Result;
   --# end
      Finalizer_Scope_2170;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Block_Var := Result_Var; 
--# bind inst_node Block_Var
--# expr-start 2451 '<as_bare_entity at ast.py:2609>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2609
Is_Null := Block_Var = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Block_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 2451
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity_Generic_Instantiation then
   --# scope-start
--# expr-start 2452 '<FieldAccess for designated_generic_decl at ast.py:2609>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2609
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_3 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2452
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind designated_decl Block_Var_1
--# expr-start 2466 '<If at ast.py:2611>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2611
--# expr-start 2455 '<Eq at ast.py:2612>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2612
--# expr-start 2453 '<FieldAccess for node at ast.py:2612>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2612
Fld_11 := Block_Var_1.Node;
--# expr-done 2453
      Cast_Result_1 := Fld_11;
--# expr-start 2454 '<parent at ast.py:2612>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2612
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Parent; 
--# expr-done 2454
Is_Equal := Cast_Result_1 = Fld_12; 
--# expr-done 2455
if Is_Equal then
   --# expr-start 2465 '<New GenericInstantiation.entity at ???>' New_Struct_2 None
--# expr-start 2464 '<New EntityInfo at ast.py:2615>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2615
--# expr-start 2457 '<FieldAccess for from_rebound at ast.py:2622>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2622
--# expr-start 2456 '<FieldAccess for info at ast.py:2622>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2622
Fld_9 := Ent.Info;
--# expr-done 2456
Fld_10 := Fld_9.From_Rebound;
--# expr-done 2457
--# expr-start 2458 '<New Metadata at ast.py:2623>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2623
New_Struct := (Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 2458
--# expr-start 2463 '<FieldAccess for remove_rebindings at ast.py:2618>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2618
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2460 '<FieldAccess for rebindings at ast.py:2619>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2619
--# expr-start 2459 '<FieldAccess for info at ast.py:2619>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2619
Fld_4 := Ent.Info;
--# expr-done 2459
Fld_5 := Fld_4.Rebindings;
--# expr-done 2460
--# expr-start 2462 '<FieldAccess for rebindings at ast.py:2620>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2620
--# expr-start 2461 '<FieldAccess for info at ast.py:2620>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2620
Fld_6 := Block_Var_1.Info;
--# expr-done 2461
Fld_7 := Fld_6.Rebindings;
--# expr-done 2462
--# property-call-start AdaNode.remove_rebindings
Fld_8 := Libadalang.Implementation.Ada_Node_P_Remove_Rebindings (Node => Self, Base => Fld_5, Suffix => Fld_7);
--# end
--# expr-done 2463
New_Struct_1 := (From_Rebound => Fld_10, Md => New_Struct, Rebindings => Fld_8); 
--# expr-done 2464
New_Struct_2 := (Info => New_Struct_1, Node => Block_Var); 
--# expr-done 2465
   If_Result_1 := New_Struct_2;
else
   If_Result_1 := No_Entity_Generic_Instantiation;
end if;
--# expr-done 2466
Let_Result := If_Result_1; 
--# end
--# expr-done 2467
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2170;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Instantiation;
--# end
--# property-start BasicDecl.is_compilation_unit_root /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2629
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Compilation_Unit_Root
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Ada_Node;
Match_0 : Bare_Library_Item;
Match_1 : Bare_Generic_Package_Decl;
Match_2 : Bare_Subunit;
Match_3 : Bare_Ada_Node;
Var_Expr_1 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Bare_Generic_Package_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Bare_Subunit;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Match_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_1 : Boolean;
Let_Result_4 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2472 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 2468 '<parent at ast.py:2634>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2634
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2468
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 2471 '<Match at ast.py:2634>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2634
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind gen_pkg_decl Match_1
--# expr-start 2469 '<parent at ast.py:2637>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2637
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_1.Parent; 
--# expr-done 2469
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 2470 '<IsA LibraryItem at ast.py:2637>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2637
Is_A := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Library_Item_Range; 
--# expr-done 2470
Scope_Result := Is_A;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := True; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := False; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 2471
Scope_Result_5 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := False;
end if;
Let_Result_4 := Result_Var_1; 
--# end
--# expr-done 2472
         Property_Result := Let_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Compilation_Unit_Root;
--# end
--# property-start BasicDecl.populate_dependent_units /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2642
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Populate_Dependent_Units
  (Node : Bare_Basic_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_170 with Inline_Always;
      Fld : Bare_Name_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Map_Result : Bare_Compilation_Unit_Array_Access;
Fld_2 : Boolean;
If_Result : Bare_Compilation_Unit_Array_Access;
Let_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_170 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_170;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2478 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2477 '<If at ast.py:2644>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2644
--# expr-start 2473 '<FieldAccess for is_compilation_unit_root at ast.py:2645>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2645
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2473
if Fld_2 then
   --# expr-start 2476 '<Map at ast.py:2646>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2646
   --# expr-start 2474 '<FieldAccess for top_level_with_package_clauses at ast.py:2646>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2646
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_with_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_With_Package_Clauses (Node => Self);
--# end
--# expr-done 2474
      declare
         Map_Result_Vec : Bare_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Name_Array_Access := Fld;
      begin
         for Item_100 of
               Collection.Items
         loop
   --# scope-start
               --# bind package_name Item_100
      --# expr-start 2475 '<FieldAccess for withed_unit_helper at ast.py:2648>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2648
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.withed_unit_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Withed_Unit_Helper (Node => Self, Unit_Name => Item_100);
--# end
--# expr-done 2475
         declare
            Item_To_Append : constant Bare_Compilation_Unit := Fld_1;
         begin
            Bare_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Bare_Compilation_Unit_Array
           (Items_Count => Natural (Bare_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Compilation_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2476
   If_Result := Map_Result;
else
   If_Result := No_Bare_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2477
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2478
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_170;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_170;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Populate_Dependent_Units;
--# end
--# property-start BasicDecl.should_ref_generic_formals /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2653
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Should_Ref_Generic_Formals
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2484 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2480 '<Not at ast.py:2664>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2664
--# expr-start 2479 '<FieldAccess for is_compilation_unit_root at ast.py:2664>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2664
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2479
Not_Val := not (Fld); 
--# expr-done 2480
if Not_Val then
   If_Result := True;
else
   --# expr-start 2482 '<IsA Subunit at ast.py:2669>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2669
--# expr-start 2481 '<parent at ast.py:2669>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2669
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 2481
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Subunit_Range; 
--# expr-done 2482
   If_Result := Is_A;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2483 '<IsA BaseSubpBody at ast.py:2675>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2675
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Base_Subp_Body; 
--# expr-done 2483
   If_Result_1 := Is_A_1;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2484
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Should_Ref_Generic_Formals;
--# end
--# property-start BasicDecl.is_in_public_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2678
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_In_Public_Part
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2487 '<IsA PublicPart at ast.py:2678>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2678
--# expr-start 2486 '<parent at ast.py:2678>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2678
--# expr-start 2485 '<parent at ast.py:2678>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2678
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2485
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 2486
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Public_Part_Range; 
--# expr-done 2487
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_In_Public_Part;
--# end
--# property-start '[dispatcher]BasicDecl.is_in_private_part' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_In_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_in_private_part
                  Property_Result := Basic_Decl_P_Is_In_Private_Part
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_in_private_part
                  Property_Result := Classwide_Type_Decl_P_Is_In_Private_Part
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_In_Private_Part;
--# end
--# property-start BasicDecl.is_visible /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2681
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Visible
  (Node : Bare_Basic_Decl
      ; From_Node : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind from_node From_Node
   Property_Result : Boolean;
            procedure Finalizer_Scope_174 with Inline_Always;
      Var_Expr : Internal_Entity_Body_Node;
Fld : Boolean;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Fld_3 : Internal_Entity;
Fld_4 : Internal_Entity;
Fld_5 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Package_Decl;
Fld_6 : Boolean;
Fld_7 : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Fld_8 : Boolean;
Fld_9 : Internal_Entity_Basic_Decl;
Fld_10 : Boolean;
Fld_11 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
Fld_12 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Not_Val_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_174 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_174;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2509 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2508 '<Cond at ast.py:2692>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2692
--# expr-start 2488 '<FieldAccess for is_in_private_part at ast.py:2695>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2695
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2488
if Fld then
--# expr-start 2493 '<Contains at ast.py:2696>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2696
   --# expr-start 2491 '<EnvGet at ast.py:2696>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2696
--# expr-start 2489 '<FieldAccess for node_env at ast.py:2696>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2696
      if From_Node.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => From_Node.Node, E_Info => From_Node.Info);
--# expr-done 2489
--# expr-start 2490 '<FieldAccess for name_symbol at ast.py:2696>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2696
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2490
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Fld_2, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 2491
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_101 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_101
      --# expr-start 2492 '<Eq at ???>' Is_Equiv None
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Item_101, Cast_Result); 
--# expr-done 2492
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 2493
if Quantifier_Result then
   --# expr-start 2500 '<Not at ast.py:2703>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2703
--# expr-start 2498 '<FieldAccess for is_compilation_unit_root at ast.py:2705>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2705
--# expr-start 2497 '<Cast to PackageDecl at ast.py:2704>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2704
--# expr-start 2496 '<parent at ast.py:2704>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2704
--# expr-start 2495 '<parent at ast.py:2704>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2704
--# expr-start 2494 '<parent at ast.py:2704>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2704
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2494
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# expr-done 2495
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 2496
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Package_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Package_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Package_Decl;
   end if;
--# expr-done 2497
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Cast_Result_1.Node);
--# end
--# expr-done 2498
if Fld_6 then
   --# expr-start 2499 '<FieldAccess for is_in_top_level_public_part at ast.py:2706>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2706
      if From_Node.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_in_top_level_public_part
Fld_7 := Libadalang.Implementation.Ada_Node_P_Is_In_Top_Level_Public_Part (Node => From_Node.Node, E_Info => From_Node.Info);
--# end
--# expr-done 2499
   If_Result := Fld_7;
else
   If_Result := False;
end if;
Not_Val := not (If_Result); 
--# expr-done 2500
   If_Result_1 := Not_Val;
else
   If_Result_1 := False;
end if;
   If_Result_5 := If_Result_1;
else
--# expr-start 2501 '<FieldAccess for is_in_public_part at ast.py:2712>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2712
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_in_public_part
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Is_In_Public_Part (Node => Ent.Node);
--# end
--# expr-done 2501
if Fld_8 then
   --# expr-start 2503 '<FieldAccess for is_visible at ast.py:2713>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2713
--# expr-start 2502 '<FieldAccess for parent_basic_decl at ast.py:2713>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2713
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_9 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2502
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_visible
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Is_Visible (Node => Fld_9.Node, From_Node => From_Node, E_Info => Fld_9.Info);
--# end
--# expr-done 2503
   If_Result_4 := Fld_10;
else
--# expr-start 2504 '<FieldAccess for is_compilation_unit_root at ast.py:2718>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2718
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Ent.Node);
--# end
--# expr-done 2504
if Fld_11 then
   --# expr-start 2507 '<Not at ast.py:2719>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2719
--# expr-start 2505 '<Cast to Body at ast.py:2719>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2719
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 2505
Var_Expr := Cast_Result_2;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 2506 '<FieldAccess for is_subunit at ast.py:2719>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2719
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld_12 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Var_Expr.Node);
--# end
--# expr-done 2506
Scope_Result := Fld_12;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Not_Val_1 := not (Result_Var); 
--# expr-done 2507
   If_Result_2 := Not_Val_1;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   If_Result_3 := raise Property_Error with "Only package-level declaration support visibility checks for now.";
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
--# expr-done 2508
Let_Result := If_Result_5; 
--# end
--# expr-done 2509
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_174;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_174;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Visible;
--# end
--# property-start BasicDecl.subp_decl_match_signature /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2728
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Decl_Match_Signature
  (Node : Bare_Basic_Decl
      ; Other : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2514 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2513 '<FieldAccess for match_signature at ast.py:2730>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2730
--# expr-start 2510 '<FieldAccess for subp_spec_or_null at ast.py:2730>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2730
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2510
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2512 '<Cast to SubpSpec at ast.py:2731>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2731
--# expr-start 2511 '<FieldAccess for subp_spec_or_null at ast.py:2731>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2731
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 2511
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Subp_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2512
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result.Node,
         Info => Cast_Result.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld.Node, Other => Cast_Result_1, Match_Name => False, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2513
Let_Result := Fld_2; 
--# end
--# expr-done 2514
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Subp_Decl_Match_Signature;
--# end
--# property-start BasicDecl.base_subp_declarations_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2735
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Base_Subp_Declarations_Impl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_176 with Inline_Always;
            procedure Finalizer_Scope_2176 with Inline_Always;
            procedure Finalizer_Scope_2177 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Base_Subp_Spec;
Block_Var_3 : Lexical_Env := Empty_Env;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Fld_7 : Lexical_Env := Empty_Env;
Singleton : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
Fld_11 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_12 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Base_Subp_Spec;
Scope_Result_1 : Internal_Entity_Base_Subp_Spec;
Result_Var_1 : Internal_Entity_Base_Subp_Spec;
Fld_14 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_15 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Unique_Array : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_176 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_176;
            procedure Finalizer_Scope_2176 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2176;
            procedure Finalizer_Scope_2177 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2177;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2544 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2516 '<FieldAccess for parent_basic_decl at ast.py:2742>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2742
--# expr-start 2515 '<FieldAccess for canonical_part at ast.py:2742>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2742
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2515
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2516
Block_Var := Fld_1; 
--# bind parent Block_Var
--# expr-start 2517 '<IsA ProtectedTypeDecl, TaskTypeDecl at ast.py:2743>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2743
Is_A := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Protected_Type_Decl_Range | Ada_Task_Type_Decl_Range; 
--# expr-done 2517
Block_Var_1 := Is_A; 
--# bind task_or_protected Block_Var_1
--# expr-start 2520 '<FieldAccess for subp_spec_or_null at ast.py:2748>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2748
--# expr-start 2519 '<Cast to BasicDecl at ast.py:2748>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2748
--# expr-start 2518 '<FieldAccess for without_md at ast.py:2748>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2748
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.without_md
Fld_2 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2518
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 2519
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 2520
Block_Var_2 := Fld_3; 
--# bind spec Block_Var_2
--# expr-start 2532 '<If at ast.py:2752>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2752
if Block_Var_1 then
   --# expr-start 2528 '<env_group at ast.py:2759>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2759
--# expr-start 2527 '<Concat at ast.py:2759>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2759
--# expr-start 2525 '<Map at ast.py:2757>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2757
   --# expr-start 2522 '<FieldAccess for base_types at ast.py:2757>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2757
--# expr-start 2521 '<Cast to BaseTypeDecl at ast.py:2757>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2757
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2521
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Cast_Result_1.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 2522
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_102 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_102
      --# expr-start 2524 '<FieldAccess for primitives_env at ast.py:2758>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2758
--# expr-start 2523 '<FieldAccess for full_view at ast.py:2758>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2758
      if Item_102.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Item_102.Node, E_Info => Item_102.Info);
--# end
--# expr-done 2523
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 2524
         declare
            Item_To_Append : constant Lexical_Env := Fld_6;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2176;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2525
--# expr-start 2526 '<FieldAccess for children_env at ast.py:2759>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2759
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 2526
                Singleton := Create_Lexical_Env_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_7;
                Inc_Ref (Singleton.Items (1));
Concat_Result := Concat (Map_Result, Singleton); 
--# expr-done 2527
Group_Env := Group (Concat_Result, No_Metadata); 
--# expr-done 2528
   If_Result := Group_Env;
else
--# expr-start 2529 '<FieldAccess for candidate_primitive_subp_tagged_type at ast.py:2762>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2762
      if Block_Var_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_tagged_type
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type (Node => Block_Var_2.Node, Canonicalize => False, E_Info => Block_Var_2.Info);
--# end
--# expr-done 2529
Var_Expr := Fld_8;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 2531 '<FieldAccess for primitives_env at ast.py:2763>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2763
--# expr-start 2530 '<FieldAccess for full_view at ast.py:2763>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2763
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2530
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 2531
Scope_Result := Fld_10;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2177;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
   If_Result := Result_Var;
end if;
      Inc_Ref (If_Result);
--# expr-done 2532
Block_Var_3 := If_Result; Inc_Ref (Block_Var_3);
--# bind prims_env Block_Var_3
--# expr-start 2543 '<unique at ast.py:2795>' Unique_Array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2795
--# expr-start 2542 '<FilterMap at ast.py:2779>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2779
   --# expr-start 2534 '<EnvGet at ast.py:2779>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2779
--# expr-start 2533 '<FieldAccess for name_symbol at ast.py:2779>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2779
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2533
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_3, Key => Fld_11, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 2534
      declare
         Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_103 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_103
--# expr-start 2535 '<Cast to BasicDecl at ast.py:2781>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2781
Cast_Expr_2 := Item_103; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2535
Var_Expr_2 := Cast_Result_3;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2536 '<FieldAccess for subp_spec_or_null at ast.py:2781>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2781
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_13 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2536
Scope_Result_1 := Fld_13;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Subp_Spec;
end if;
Var_Expr_1 := Result_Var_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind s Var_Expr_1
--# expr-start 2539 '<bind at ast.py:2788>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2788
--# expr-start 2537 '<FieldAccess for origin_node at ast.py:2788>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2788
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_14 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Var_Expr_1.Node);
--# end
--# expr-done 2537
Bound_Origin := Fld_14; 
--# expr-start 2538 '<FieldAccess for match_signature at ast.py:2788>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2788
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_15 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Var_Expr_1.Node, Other => Block_Var_2, Match_Name => False, Use_Entity_Info => True, Ignore_First_Param => Block_Var_1, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2538
Dyn_Var_Bind_Result := Fld_15; 
--# expr-done 2539
Scope_Result_2 := Dyn_Var_Bind_Result;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
         if Result_Var_2 then
      --# expr-start 2541 '<FieldAccess for canonical_part at ast.py:2780>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2780
--# expr-start 2540 '<Cast to BasicDecl at ast.py:2780>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2780
Cast_Expr_1 := Item_103; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2540
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Cast_Result_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 2541
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Fld_12;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2542
Unique_Array := Make_Unique (Map_Result_1); 
--# expr-done 2543
Let_Result := Unique_Array; Inc_Ref (Let_Result);
--# end
--# expr-done 2544
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_176;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_176;
                     Finalizer_Scope_2176;
                     Finalizer_Scope_2177;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Base_Subp_Declarations_Impl;
--# end
--# property-start BasicDecl.base_subp_declarations /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2797
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Base_Subp_Declarations
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_177 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_177 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_177;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Base_Subp_Declarations,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 2548 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2547 '<If at ast.py:2808>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2808
--# expr-start 2545 '<FieldAccess for is_subprogram at ast.py:2809>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2809
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 2545
if Fld_1 then
   --# expr-start 2546 '<FieldAccess for base_subp_declarations_impl at ast.py:2810>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2810
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.base_subp_declarations_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations_Impl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2546
   If_Result := Fld;
else
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2547
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2548
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_177;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_177;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl_Array_Access,
                        As_Internal_Entity_Basic_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Basic_Decl_P_Base_Subp_Declarations;
--# end
--# property-start BasicDecl.root_subp_declarations_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2814
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Root_Subp_Declarations_Impl
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_178 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Basic_Decl;
Block_Var_2 : Boolean;
Block_Var_3 : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_3 : Bare_Basic_Decl;
Cast_Result : Bare_Basic_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Len : Integer;
Comp_Result : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_4 : Internal_Entity_Info;
Fld_5 : Internal_Metadata;
Fld_6 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Unique_Array : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val_1 : Boolean;
Fld_7 : Boolean;
If_Result_3 : Boolean;
Quantifier_Result : Boolean;
Not_Val_2 : Boolean;
Map_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Bare_Ada_Node;
Fld_11 : Bare_Base_Type_Decl;
Cast_Result_4 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Find_Result : Internal_Entity_Basic_Decl;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_178 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_178;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2579 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2549 '<FieldAccess for base_subp_declarations at ast.py:2825>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2825
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.base_subp_declarations
Fld := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2549
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind raw_base_decls Block_Var
--# expr-start 2551 '<FieldAccess for parent_basic_decl at ast.py:2834>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2834
--# expr-start 2550 '<FieldAccess for canonical_part at ast.py:2834>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2834
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2550
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2551
Block_Var_1 := Fld_2; 
--# bind parent Block_Var_1
--# expr-start 2552 '<IsA ProtectedTypeDecl, TaskTypeDecl at ast.py:2835>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2835
Is_A := Block_Var_1.Node /= null 
and then Block_Var_1.Node.Kind in Ada_Protected_Type_Decl_Range | Ada_Task_Type_Decl_Range; 
--# expr-done 2552
Block_Var_2 := Is_A; 
--# bind task_or_protected Block_Var_2
--# expr-start 2558 '<If at ast.py:2836>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2836
if Block_Var_2 then
   --# expr-start 2553 '<length at ast.py:2837>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2837
Len := Length (Block_Var); 
--# expr-done 2553
Comp_Result := Len > 1; 
   If_Result := Comp_Result;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 2557 '<Filter at ast.py:2838>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2838
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var;
      begin
         for Item_104 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_104
         --# expr-start 2556 '<Not at ast.py:2838>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2838
--# expr-start 2555 '<Eq at ast.py:2838>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2838
--# expr-start 2554 '<FieldAccess for node at ast.py:2838>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2838
Fld_3 := Item_104.Node;
--# expr-done 2554
      Cast_Result := Fld_3;
Is_Equal := Cast_Result = Self; 
--# expr-done 2555
Not_Val := not (Is_Equal); 
--# expr-done 2556
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Item_104;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2557
   If_Result_1 := Map_Result;
else
   If_Result_1 := Block_Var;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2558
Block_Var_3 := If_Result_1; Inc_Ref (Block_Var_3);
--# bind base_decls Block_Var_3
--# expr-start 2565 '<unique at ast.py:2845>' Unique_Array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2845
--# expr-start 2564 '<Map at ast.py:2843>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2843
      declare
         Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_3;
      begin
         for Item_105 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_105
      --# expr-start 2563 '<as_bare_entity at ast.py:2844>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2844
--# expr-start 2562 '<Cast to BaseTypeDecl at ast.py:2844>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2844
--# expr-start 2561 '<FieldAccess for primitive at ast.py:2844>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2844
--# expr-start 2560 '<FieldAccess for md at ast.py:2844>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2844
--# expr-start 2559 '<FieldAccess for info at ast.py:2844>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2844
Fld_4 := Item_105.Info;
--# expr-done 2559
Fld_5 := Fld_4.Md;
--# expr-done 2560
Fld_6 := Fld_5.Primitive;
--# expr-done 2561
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Fld_6;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2562
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result_2 := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result_2 := As_Entity;
end if;
--# expr-done 2563
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := If_Result_2;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2564
Unique_Array := Make_Unique (Map_Result_1); 
--# expr-done 2565
Block_Var_4 := Unique_Array; Inc_Ref (Block_Var_4);
--# bind base_types Block_Var_4
--# expr-start 2578 '<Map at ast.py:2857>' Map_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2857
   --# expr-start 2571 '<Filter at ast.py:2850>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2850
      declare
         Map_Result_2_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var_4;
      begin
         for Item_107 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_107
         --# expr-start 2570 '<Not at ast.py:2851>' Not_Val_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2851
--# expr-start 2569 '<AnyQuantifier at ast.py:2851>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2851
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var_4;
      begin
         for Item_108 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_108
--# expr-start 2567 '<Not at ast.py:2853>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2853
--# expr-start 2566 '<Eq at ast.py:2853>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2853
Cast_Expr := Item_107; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_108; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 2566
Not_Val_1 := not (Is_Equiv); 
--# expr-done 2567
if Not_Val_1 then
   --# expr-start 2568 '<FieldAccess for is_derived_type at ast.py:2854>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2854
      if Item_107.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Item_107.Node, Other_Type => Item_108, Origin => Origin, E_Info => Item_107.Info);
--# end
--# expr-done 2568
   If_Result_3 := Fld_7;
else
   If_Result_3 := False;
end if;
      Quantifier_Result := If_Result_3;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 2569
Not_Val_2 := not (Quantifier_Result); 
--# expr-done 2570
         if Not_Val_2 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_107;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 2571
      declare
         Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_2;
      begin
         for Item_106 of
               Collection.Items
         loop
   --# scope-start
               --# bind root_type Item_106
      --# expr-start 2577 '<Find at ast.py:2859>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2859
      Find_Result := No_Entity_Basic_Decl;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_3;
      begin
         for Item_109 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_109
      --# expr-start 2576 '<Eq at ast.py:2860>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2860
--# expr-start 2574 '<FieldAccess for primitive at ast.py:2860>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2860
--# expr-start 2573 '<FieldAccess for md at ast.py:2860>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2860
--# expr-start 2572 '<FieldAccess for info at ast.py:2860>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2860
Fld_8 := Item_109.Info;
--# expr-done 2572
Fld_9 := Fld_8.Md;
--# expr-done 2573
Fld_10 := Fld_9.Primitive;
--# expr-done 2574
--# expr-start 2575 '<FieldAccess for node at ast.py:2860>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2860
Fld_11 := Item_106.Node;
--# expr-done 2575
      Cast_Result_4 := Fld_11;
Is_Equal_1 := Fld_10 = Cast_Result_4; 
--# expr-done 2576
      if Is_Equal_1 then
         Find_Result := Item_109; 
      end if;
   --# end
               exit when Is_Equal_1;
         end loop;
      end;
--# expr-done 2577
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Find_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 2578
Let_Result := Map_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 2579
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_178;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_178;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Root_Subp_Declarations_Impl;
--# end
--# property-start BasicDecl.root_subp_declarations /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2864
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Root_Subp_Declarations
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_179 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_179 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_179;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2583 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2582 '<If at ast.py:2881>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2881
--# expr-start 2580 '<FieldAccess for is_subprogram at ast.py:2882>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2882
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 2580
if Fld_1 then
   --# expr-start 2581 '<FieldAccess for root_subp_declarations_impl at ast.py:2883>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2883
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.root_subp_declarations_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Root_Subp_Declarations_Impl (Node => Ent.Node, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2581
   If_Result := Fld;
else
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2582
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2583
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_179;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_179;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Root_Subp_Declarations;
--# end
--# property-start BasicDecl.find_all_overrides /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2887
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Find_All_Overrides
  (Node : Bare_Basic_Decl
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_181 with Inline_Always;
            procedure Finalizer_Scope_2188 with Inline_Always;
            procedure Finalizer_Scope_2189 with Inline_Always;
      Block_Var : Internal_Entity_Base_Subp_Spec;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Type_Decl_Array_Access;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Prims : Internal_Entity_Basic_Decl_Array_Access;
Base_P : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_5 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Basic_Decl;
Is_Equal : Boolean;
Find_Result : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Bare_Ada_Node;
Fld_9 : Bare_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Fld_10 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Internal_Entity_Base_Subp_Spec;
Fld_13 : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_14 : Internal_Entity_Basic_Decl;
Fld_15 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Map_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Unique_Array : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_181 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_181;
            procedure Finalizer_Scope_2188 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2188;
            procedure Finalizer_Scope_2189 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Prims);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_2189;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2614 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 2584 '<FieldAccess for subp_spec_or_null at ast.py:2895>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2895
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2584
Block_Var := Fld; 
--# bind spec Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 2585 '<FieldAccess for candidate_primitive_subp_tagged_type at ast.py:2899>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2899
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_tagged_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2585
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind prim_type Block_Var_1
Var_Expr_1 := Block_Var_1;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2586 '<FieldAccess for find_all_derived_types at ast.py:2901>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2901
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.find_all_derived_types
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Find_All_Derived_Types (Node => Var_Expr_1.Node, Units => Units, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2586
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2188;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_2 := Result_Var_1; Inc_Ref (Block_Var_2);
--# bind derivations Block_Var_2
--# expr-start 2613 '<unique at ast.py:2936>' Unique_Array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2936
--# expr-start 2612 '<Mapcat at ast.py:2903>' Map_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2903
      declare
         Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Block_Var_2;
      begin
         for Item_110 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_110
      --# expr-start 2611 '<Let at ast.py:2904>' Let_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2904
--# scope-start
--# expr-start 2591 '<Map at ast.py:2906>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2906
   --# expr-start 2589 '<EnvGet at ast.py:2906>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2906
--# expr-start 2587 '<FieldAccess for primitives_env at ast.py:2906>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2906
      if Item_110.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Item_110.Node, E_Info => Item_110.Info);
--# end
--# expr-done 2587
--# expr-start 2588 '<FieldAccess for name_symbol at ast.py:2906>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2906
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2588
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_3, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 2589
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_111 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_111
      --# expr-start 2590 '<Cast to BasicDecl at ast.py:2907>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2907
Cast_Expr := Item_111; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2590
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2591
Prims := Map_Result; Inc_Ref (Prims);
--# bind prims Prims
--# expr-start 2610 '<Let at ast.py:2908>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2908
--# scope-start
--# expr-start 2594 '<Find at ast.py:2911>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2911
      Find_Result := No_Entity_Basic_Decl;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Prims;
      begin
         for Item_112 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_112
      --# expr-start 2593 '<Eq at ast.py:2911>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2911
--# expr-start 2592 '<FieldAccess for node at ast.py:2911>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2911
Fld_5 := Item_112.Node;
--# expr-done 2592
      Cast_Result_1 := Fld_5;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 2593
      if Is_Equal then
         Find_Result := Item_112; 
      end if;
   --# end
               exit when Is_Equal;
         end loop;
      end;
--# expr-done 2594
Base_P := Find_Result; 
--# bind base_p Base_P
--# expr-start 2609 '<Map at ast.py:2930>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2930
   --# expr-start 2605 '<Filter at ast.py:2913>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2913
      declare
         Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Prims;
      begin
         for Item_114 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_114
--# expr-start 2599 '<Eq at ast.py:2919>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2919
--# expr-start 2597 '<FieldAccess for primitive at ast.py:2919>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2919
--# expr-start 2596 '<FieldAccess for md at ast.py:2919>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2919
--# expr-start 2595 '<FieldAccess for info at ast.py:2919>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2919
Fld_6 := Item_114.Info;
--# expr-done 2595
Fld_7 := Fld_6.Md;
--# expr-done 2596
Fld_8 := Fld_7.Primitive;
--# expr-done 2597
--# expr-start 2598 '<FieldAccess for node at ast.py:2919>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2919
Fld_9 := Item_110.Node;
--# expr-done 2598
      Cast_Result_2 := Fld_9;
Is_Equal_1 := Fld_8 = Cast_Result_2; 
--# expr-done 2599
if Is_Equal_1 then
   --# expr-start 2604 '<bind at ast.py:2922>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2922
--# expr-start 2600 '<FieldAccess for origin_node at ast.py:2923>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2923
      if Base_P.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_10 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Base_P.Node);
--# end
--# expr-done 2600
Bound_Origin := Fld_10; 
--# expr-start 2603 '<FieldAccess for match_signature at ast.py:2924>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2924
--# expr-start 2601 '<FieldAccess for subp_spec_or_null at ast.py:2924>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2924
      if Base_P.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Base_P.Node, E_Info => Base_P.Info);
--# end
--# expr-done 2601
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2602 '<FieldAccess for subp_spec_or_null at ast.py:2925>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2925
      if Item_114.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_114.Node, E_Info => Item_114.Info);
--# end
--# expr-done 2602
--# property-call-start BaseSubpSpec.match_signature
Fld_13 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_11.Node, Other => Fld_12, Match_Name => False, Use_Entity_Info => True, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 2603
Dyn_Var_Bind_Result := Fld_13; 
--# expr-done 2604
   If_Result := Dyn_Var_Bind_Result;
else
   If_Result := False;
end if;
         if If_Result then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Item_114;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2605
      declare
         Map_Result_2_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result_1;
      begin
         for Item_113 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_113
      --# expr-start 2608 '<Cast to BasicDecl at ast.py:2932>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2932
--# expr-start 2607 '<FieldAccess for without_md at ast.py:2932>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2932
--# expr-start 2606 '<FieldAccess for canonical_part at ast.py:2932>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2932
      if Item_113.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Item_113.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_113.Info);
--# end
--# expr-done 2606
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.without_md
Fld_15 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 2607
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2608
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result_3;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 2609
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 2610
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2611
         for Item_To_Append of
               Let_Result_1.Items
         loop
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2189;
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 2612
Unique_Array := Make_Unique (Map_Result_3); 
--# expr-done 2613
Let_Result_2 := Unique_Array; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2614
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_181;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_181;
                     Finalizer_Scope_2188;
                     Finalizer_Scope_2189;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Find_All_Overrides;
--# end
--# property-start '[dispatcher]BasicDecl.defining_names' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Defining_Names
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl =>
                  --# property-call-start AbstractStateDecl.defining_names
                  Property_Result := Abstract_State_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.defining_names
                  Property_Result := Anonymous_Expr_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.defining_names
                  Property_Result := Component_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.defining_names
                  Property_Result := Discriminant_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.defining_names
                  Property_Result := Generic_Formal_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.defining_names
                  Property_Result := Param_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start SyntheticFormalParamDecl.defining_names
                  Property_Result := Synthetic_Formal_Param_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.defining_names
                  Property_Result := Base_Package_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.defining_names
                  Property_Result := Base_Type_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl | Ada_Subp_Decl | Ada_Generic_Subp_Internal .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.defining_names
                  Property_Result := Basic_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.defining_names
                  Property_Result := Formal_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.defining_names
                  Property_Result := Entry_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.defining_names
                  Property_Result := Enum_Literal_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start SyntheticCharEnumLit.defining_names
                  Property_Result := Synthetic_Char_Enum_Lit_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt_Body =>
                  --# property-call-start AcceptStmtBody.defining_names
                  Property_Result := Accept_Stmt_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.defining_names
                  Property_Result := Base_Subp_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub =>
                  --# property-call-start PackageBodyStub.defining_names
                  Property_Result := Package_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body_Stub =>
                  --# property-call-start ProtectedBodyStub.defining_names
                  Property_Result := Protected_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body_Stub =>
                  --# property-call-start SubpBodyStub.defining_names
                  Property_Result := Subp_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body_Stub =>
                  --# property-call-start TaskBodyStub.defining_names
                  Property_Result := Task_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.defining_names
                  Property_Result := Entry_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.defining_names
                  Property_Result := Package_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body =>
                  --# property-call-start ProtectedBody.defining_names
                  Property_Result := Protected_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body =>
                  --# property-call-start TaskBody.defining_names
                  Property_Result := Task_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.defining_names
                  Property_Result := Entry_Index_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Error_Decl =>
                  --# property-call-start ErrorDecl.defining_names
                  Property_Result := Error_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.defining_names
                  Property_Result := Exception_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.defining_names
                  Property_Result := Exception_Handler_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.defining_names
                  Property_Result := For_Loop_Var_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.defining_names
                  Property_Result := Generic_Package_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.defining_names
                  Property_Result := Generic_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.defining_names
                  Property_Result := Generic_Package_Instantiation_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.defining_names
                  Property_Result := Generic_Subp_Instantiation_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.defining_names
                  Property_Result := Generic_Package_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericSubpRenamingDecl.defining_names
                  Property_Result := Generic_Subp_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Label_Decl =>
                  --# property-call-start LabelDecl.defining_names
                  Property_Result := Label_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt_Decl =>
                  --# property-call-start NamedStmtDecl.defining_names
                  Property_Result := Named_Stmt_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.defining_names
                  Property_Result := Number_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.defining_names
                  Property_Result := Object_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.defining_names
                  Property_Result := Package_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.defining_names
                  Property_Result := Single_Protected_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.defining_names
                  Property_Result := Single_Task_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Defining_Names;
--# end
--# property-start BasicDecl.defining_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2947
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_183 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_183 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_183;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2616 '<at at ast.py:2948>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2948
--# expr-start 2615 '<FieldAccess for defining_names at ast.py:2948>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2948
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2615
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 2616
         Property_Result := Get_Result;
   --# end
      Finalizer_Scope_183;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_183;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Defining_Name;
--# end
--# property-start '[dispatcher]BasicDecl.defining_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Defining_Env
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl | Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl | Ada_Synthetic_Formal_Param_Decl | Ada_Package_Body_Stub .. Ada_Entry_Body | Ada_Protected_Body | Ada_Error_Decl .. Ada_Exception_Handler | Ada_Generic_Subp_Decl | Ada_Generic_Subp_Renaming_Decl .. Ada_Label_Decl | Ada_Number_Decl =>
                  --# property-call-start BasicDecl.defining_env
                  Property_Result := Basic_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.defining_env
                  Property_Result := Anonymous_Expr_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.defining_env
                  Property_Result := Component_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.defining_env
                  Property_Result := Discriminant_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.defining_env
                  Property_Result := Param_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.defining_env
                  Property_Result := Base_Package_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.defining_env
                  Property_Result := Base_Subtype_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.defining_env
                  Property_Result := Classwide_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.defining_env
                  Property_Result := Incomplete_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.defining_env
                  Property_Result := Protected_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.defining_env
                  Property_Result := Task_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.defining_env
                  Property_Result := Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.defining_env
                  Property_Result := Basic_Subp_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt_Body =>
                  --# property-call-start AcceptStmtBody.defining_env
                  Property_Result := Accept_Stmt_Body_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.defining_env
                  Property_Result := Base_Subp_Body_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.defining_env
                  Property_Result := Package_Body_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body =>
                  --# property-call-start TaskBody.defining_env
                  Property_Result := Task_Body_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.defining_env
                  Property_Result := Entry_Index_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.defining_env
                  Property_Result := For_Loop_Var_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.defining_env
                  Property_Result := Generic_Package_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.defining_env
                  Property_Result := Generic_Package_Instantiation_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.defining_env
                  Property_Result := Generic_Subp_Instantiation_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.defining_env
                  Property_Result := Generic_Package_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt_Decl =>
                  --# property-call-start NamedStmtDecl.defining_env
                  Property_Result := Named_Stmt_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.defining_env
                  Property_Result := Object_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.defining_env
                  Property_Result := Package_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.defining_env
                  Property_Result := Single_Protected_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.defining_env
                  Property_Result := Single_Task_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Defining_Env;
--# end
--# property-start BasicDecl.defining_name_or_raise /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2965
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Name_Or_Raise
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_185 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Len : Integer;
Comp_Result : Boolean;
If_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_185 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_185;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2621 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2617 '<FieldAccess for defining_names at ast.py:2971>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2971
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2617
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind dns Block_Var
--# expr-start 2620 '<If at ast.py:2972>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2972
--# expr-start 2618 '<length at ast.py:2973>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2973
Len := Length (Block_Var); 
--# expr-done 2618
Comp_Result := Len > 1; 
if Comp_Result then
   If_Result := raise Property_Error with "BasicDecl with multiple defining names";
else
   --# expr-start 2619 '<at at ast.py:2976>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2976
Get_Result := Get (Self, Block_Var, 0, True); 
--# expr-done 2619
   If_Result := Get_Result;
end if;
--# expr-done 2620
Let_Result := If_Result; 
--# end
--# expr-done 2621
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_185;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_185;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Defining_Name_Or_Raise;
--# end
--# property-start BasicDecl.identity_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2979
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Identity_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Exception_Decl;
Match_1 : Internal_Entity_Single_Task_Decl;
Match_2 : Internal_Entity_Object_Decl;
Match_3 : Internal_Entity_For_Loop_Var_Decl;
Match_4 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Exception_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Object_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_For_Loop_Var_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_4 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_5 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2627 '<Block at ???>' Let_Result_5 None
--# scope-start
--# expr-start 2626 '<Match at ast.py:2981>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2981
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Exception_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Exception_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 2622 '<FieldAccess for exc_id_type at ast.py:2982>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2982
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.exc_id_type
Fld := Libadalang.Implementation.Ada_Node_P_Exc_Id_Type (Node => Self);
--# end
--# expr-done 2622
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 2623 '<FieldAccess for task_id_type at ast.py:2983>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2983
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 2623
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Object_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 2624 '<FieldAccess for task_id_type at ast.py:2987>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2987
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 2624
Let_Result_2 := Fld_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_For_Loop_Var_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_For_Loop_Var_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 2625 '<FieldAccess for task_id_type at ast.py:2989>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2989
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 2625
Let_Result_3 := Fld_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abstract_State_Decl .. Ada_Error_Decl | Ada_Exception_Handler | Ada_Generic_Package_Decl .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_4 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 2626
Let_Result_5 := Match_Result; 
--# end
--# expr-done 2627
         Property_Result := Let_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Identity_Type;
--# end
--# property-start '[dispatcher]BasicDecl.array_ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Array_Ndims
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Discriminant_Spec .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.array_ndims
                  Property_Result := Basic_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.array_ndims
                  Property_Result := Component_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.array_ndims
                  Property_Result := Base_Type_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.array_ndims
                  Property_Result := Base_Subtype_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.array_ndims
                  Property_Result := Type_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Array_Ndims;
--# end
--# property-start BasicDecl.is_array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2997
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Array
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Integer;
Comp_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2628 '<FieldAccess for array_ndims at ast.py:2997>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2997
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2628
Comp_Result := Fld > 0; 
         Property_Result := Comp_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Array;
--# end
--# property-start '[dispatcher]BasicDecl.expr_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Expr_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Task_Body | Ada_Error_Decl .. Ada_Exception_Decl | Ada_Generic_Package_Decl .. Ada_Generic_Package_Instantiation | Ada_Generic_Package_Renaming_Decl .. Ada_Named_Stmt_Decl | Ada_Object_Decl .. Ada_Single_Protected_Decl =>
                  --# property-call-start BasicDecl.expr_type
                  Property_Result := Basic_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.expr_type
                  Property_Result := Base_Type_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.expr_type
                  Property_Result := Basic_Subp_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.expr_type
                  Property_Result := Base_Subp_Body_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.expr_type
                  Property_Result := Entry_Index_Spec_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.expr_type
                  Property_Result := Exception_Handler_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.expr_type
                  Property_Result := For_Loop_Var_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.expr_type
                  Property_Result := Generic_Subp_Instantiation_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.expr_type
                  Property_Result := Number_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.expr_type
                  Property_Result := Single_Task_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Expr_Type;
--# end
--# property-start '[dispatcher]BasicDecl.type_expression' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Type_Expression
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Generic_Package_Internal .. Ada_Formal_Type_Decl | Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.type_expression
                  Property_Result := Basic_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl =>
                  --# property-call-start AbstractStateDecl.type_expression
                  Property_Result := Abstract_State_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.type_expression
                  Property_Result := Anonymous_Expr_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.type_expression
                  Property_Result := Component_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.type_expression
                  Property_Result := Discriminant_Spec_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.type_expression
                  Property_Result := Generic_Formal_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.type_expression
                  Property_Result := Param_Spec_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start SyntheticFormalParamDecl.type_expression
                  Property_Result := Synthetic_Formal_Param_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.type_expression
                  Property_Result := Basic_Subp_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.type_expression
                  Property_Result := Base_Subp_Body_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body_Stub =>
                  --# property-call-start SubpBodyStub.type_expression
                  Property_Result := Subp_Body_Stub_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.type_expression
                  Property_Result := Object_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Type_Expression;
--# end
--# property-start BasicDecl.subp_spec_or_null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3033
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Spec_Or_Null
  (Node : Bare_Basic_Decl
      ; Follow_Generic : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind follow_generic Follow_Generic
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Match_0 : Internal_Entity_Basic_Subp_Decl;
Match_1 : Internal_Entity_Base_Subp_Body;
Match_2 : Internal_Entity_Subp_Body_Stub;
Match_3 : Internal_Entity_Generic_Subp_Decl;
Match_4 : Internal_Entity_Generic_Subp_Instantiation;
Match_5 : Internal_Entity_Generic_Subp_Renaming_Decl;
Match_6 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Subp_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Generic_Subp_Internal;
Fld_4 : Internal_Entity_Subp_Spec;
If_Result : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Basic_Subp_Decl;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Result_Var : Internal_Entity_Base_Subp_Spec;
Cast_Expr : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Internal_Entity_Generic_Decl;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Scope_Result_1 : Internal_Entity_Base_Subp_Spec;
Result_Var_1 : Internal_Entity_Base_Subp_Spec;
Cast_Expr_1 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Subp_Spec;
If_Result_2 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Basic_Subp_Decl;
Let_Result : Internal_Entity_Base_Subp_Spec;
Scope_Result_2 : Internal_Entity_Base_Subp_Spec;
Cast_Result_3 : Internal_Entity_Base_Subp_Body;
Cast_Result_4 : Internal_Entity_Base_Subp_Spec;
Let_Result_1 : Internal_Entity_Base_Subp_Spec;
Scope_Result_3 : Internal_Entity_Base_Subp_Spec;
Cast_Result_5 : Internal_Entity_Subp_Body_Stub;
Cast_Result_6 : Internal_Entity_Base_Subp_Spec;
Let_Result_2 : Internal_Entity_Base_Subp_Spec;
Scope_Result_4 : Internal_Entity_Base_Subp_Spec;
Cast_Result_7 : Internal_Entity_Generic_Subp_Decl;
Cast_Result_8 : Internal_Entity_Base_Subp_Spec;
Let_Result_3 : Internal_Entity_Base_Subp_Spec;
Scope_Result_5 : Internal_Entity_Base_Subp_Spec;
Cast_Result_9 : Internal_Entity_Generic_Subp_Instantiation;
Let_Result_4 : Internal_Entity_Base_Subp_Spec;
Scope_Result_6 : Internal_Entity_Base_Subp_Spec;
Cast_Result_10 : Internal_Entity_Generic_Subp_Renaming_Decl;
Let_Result_5 : Internal_Entity_Base_Subp_Spec;
Scope_Result_7 : Internal_Entity_Base_Subp_Spec;
Cast_Result_11 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity_Subp_Spec;
Cast_Result_12 : Internal_Entity_Base_Subp_Spec;
Let_Result_6 : Internal_Entity_Base_Subp_Spec;
Scope_Result_8 : Internal_Entity_Base_Subp_Spec;
Match_Result : Internal_Entity_Base_Subp_Spec;
Let_Result_7 : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2642 '<Block at ???>' Let_Result_7 None
--# scope-start
--# expr-start 2641 '<Match at ast.py:3041>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3041
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind subp Match_0
--# expr-start 2629 '<FieldAccess for subp_decl_spec at ast.py:3042>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3042
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 2629
Let_Result := Fld; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind subp Match_1
--# expr-start 2630 '<FieldAccess for subp_spec at ast.py:3043>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3043
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Match_1.Node.Base_Subp_Body_F_Subp_Spec, Info => Match_1.Info);
--# expr-done 2630
      Cast_Result_4 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind subp Match_2
--# expr-start 2631 '<FieldAccess for subp_spec at ast.py:3044>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3044
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Match_2.Node.Subp_Body_Stub_F_Subp_Spec, Info => Match_2.Info);
--# expr-done 2631
      Cast_Result_6 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_6; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_7; 
--# bind gsp Match_3
--# expr-start 2634 '<If at ast.py:3046>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3046
if Follow_Generic then
   --# expr-start 2633 '<FieldAccess for subp_spec at ast.py:3046>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3046
--# expr-start 2632 '<FieldAccess for subp_decl at ast.py:3046>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3046
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Generic_Subp_Internal (Node => Match_3.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Match_3.Info);
--# expr-done 2632
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Fld_3.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Fld_3.Info);
--# expr-done 2633
   If_Result := Fld_4;
else
   If_Result := No_Entity_Subp_Spec;
end if;
--# expr-done 2634
      Cast_Result_8 := Create_Internal_Entity_Base_Subp_Spec
        (Node => If_Result.Node,
         Info => If_Result.Info);
Let_Result_3 := Cast_Result_8; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Generic_Subp_Instantiation
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_9; 
--# bind gsi Match_4
--# expr-start 2637 '<If at ast.py:3048>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3048
if Follow_Generic then
--# expr-start 2635 '<FieldAccess for designated_subp at ast.py:3049>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3049
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericSubpInstantiation.designated_subp
Fld_5 := Libadalang.Implementation.Generic_Subp_Instantiation_P_Designated_Subp (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 2635
Var_Expr := Fld_5;
if Var_Expr /= No_Entity_Basic_Subp_Decl then
   --# scope-start
--# expr-start 2636 '<FieldAccess for subp_spec_or_null at ast.py:3049>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3049
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2636
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Subp_Spec;
end if;
   If_Result_1 := Result_Var;
else
Cast_Expr := No_Entity_Subp_Spec; 
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_1 := Cast_Result;
end if;
--# expr-done 2637
Let_Result_4 := If_Result_1; 
--# end
Scope_Result_6 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Generic_Subp_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_10; 
--# bind gsr Match_5
--# expr-start 2640 '<If at ast.py:3052>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3052
if Follow_Generic then
--# expr-start 2638 '<FieldAccess for resolve at ast.py:3053>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3053
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_7 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_5.Node, E_Info => Match_5.Info);
--# end
--# expr-done 2638
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 2639 '<FieldAccess for subp_spec_or_null at ast.py:3053>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3053
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2639
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Subp_Spec;
end if;
   If_Result_2 := Result_Var_1;
else
Cast_Expr_1 := No_Entity_Subp_Spec; 
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   If_Result_2 := Cast_Result_1;
end if;
--# expr-done 2640
Let_Result_5 := If_Result_2; 
--# end
Scope_Result_7 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Abstract_State_Decl .. Ada_Formal_Type_Decl | Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation | Ada_Generic_Package_Renaming_Decl | Ada_Label_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_11; 
--# bind _ Match_6
Cast_Expr_2 := No_Entity_Subp_Spec; 
      Cast_Result_12 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Let_Result_6 := Cast_Result_12; 
--# end
Scope_Result_8 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 2641
Let_Result_7 := Match_Result; 
--# end
--# expr-done 2642
         Property_Result := Let_Result_7;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Subp_Spec_Or_Null;
--# end
--# property-start BasicDecl.formal_param_holder_or_null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3058
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Formal_Param_Holder_Or_Null
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Entry_Body;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Entry_Completion_Formal_Params;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity_Entry_Body;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Match_Result : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2647 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 2646 '<Match at ast.py:3060>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3060
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
--# expr-start 2643 '<FieldAccess for discriminants at ast.py:3061>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3061
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Match_0.Node.Type_Decl_F_Discriminants, Info => Match_0.Info);
--# expr-done 2643
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld.Node,
         Info => Fld.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 2644 '<FieldAccess for params at ast.py:3062>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3062
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_1.Node.Entry_Body_F_Params, Info => Match_1.Info);
--# expr-done 2644
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
--# expr-start 2645 '<FieldAccess for subp_spec_or_null at ast.py:3063>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3063
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2645
      Cast_Result_5 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2646
Let_Result_3 := Match_Result; 
--# end
--# expr-done 2647
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Formal_Param_Holder_Or_Null;
--# end
--# property-start BasicDecl.is_subprogram /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3066
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Subprogram
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2649 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2648 '<IsA BasicSubpDecl, BaseSubpBody, SubpBodyStub, EntryDecl, GenericSubpDecl, GenericSubpInstantiation, GenericSubpRenamingDecl at ast.py:3077>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3077
Is_A := Self /= null 
and then Self.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body | Ada_Subp_Body_Stub_Range | Ada_Entry_Decl_Range | Ada_Generic_Subp_Decl_Range | Ada_Generic_Subp_Instantiation_Range | Ada_Generic_Subp_Renaming_Decl_Range; 
--# expr-done 2648
Let_Result := Is_A; 
--# end
--# expr-done 2649
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Subprogram;
--# end
--# property-start BasicDecl.is_valid_reducer_candidate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3081
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Valid_Reducer_Candidate
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_2222 with Inline_Always;
      Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Param_Types : Internal_Entity_Base_Type_Decl_Array_Access;
Return_Type : Internal_Entity_Base_Type_Decl;
Param_Modes : Internal_Entity_Mode_Array_Access;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl;
Len : Integer;
Is_Equal : Boolean;
Fld_4 : Internal_Entity_Mode_Array_Access;
Get_Result : Internal_Entity_Mode;
Is_A : Boolean;
Get_Result_1 : Internal_Entity_Mode;
Is_A_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
Get_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Is_Null : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Let_Result_2 : Boolean;
            procedure Finalizer_Scope_2222 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Param_Types);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Param_Modes);
            end Finalizer_Scope_2222;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2667 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 2650 '<FieldAccess for is_subprogram at ast.py:3089>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3089
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 2650
Var_Expr := Fld;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 2651 '<FieldAccess for subp_spec_or_null at ast.py:3090>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3090
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2651
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_1
--# expr-start 2666 '<Let at ast.py:3091>' Let_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3091
--# scope-start
--# expr-start 2652 '<FieldAccess for param_types at ast.py:3092>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3092
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Var_Expr_1.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2652
Param_Types := Fld_2; Inc_Ref (Param_Types);
--# bind param_types Param_Types
--# expr-start 2653 '<FieldAccess for return_type at ast.py:3093>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3093
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2653
Return_Type := Fld_3; 
--# bind return_type Return_Type
--# expr-start 2655 '<Eq at ast.py:3097>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3097
--# expr-start 2654 '<length at ast.py:3097>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3097
Len := Length (Param_Types); 
--# expr-done 2654
Is_Equal := Len = 2; 
--# expr-done 2655
if Is_Equal then
   --# expr-start 2665 '<If at ast.py:3099>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3099
--# expr-start 2656 '<IsNull at ast.py:3100>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3100
Is_Null := Return_Type.Node = null; 
--# expr-done 2656
if Is_Null then
   --# expr-start 2662 '<Let at ast.py:3103>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3103
--# scope-start
--# expr-start 2657 '<FieldAccess for param_modes at ast.py:3104>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3104
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_modes
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Modes (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2657
Param_Modes := Fld_4; Inc_Ref (Param_Modes);
--# bind param_modes Param_Modes
--# expr-start 2659 '<IsA Mode.InOut at ast.py:3105>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3105
--# expr-start 2658 '<at at ast.py:3105>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3105
Get_Result := Get (Self, Param_Modes, 0, True); 
--# expr-done 2658
Is_A := Get_Result.Node /= null 
and then Get_Result.Node.Kind in Ada_Mode_In_Out_Range; 
--# expr-done 2659
if Is_A then
   --# expr-start 2661 '<IsA Mode.In, Mode.Default at ast.py:3106>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3106
--# expr-start 2660 '<at at ast.py:3106>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3106
Get_Result_1 := Get (Self, Param_Modes, 1, True); 
--# expr-done 2660
Is_A_1 := Get_Result_1.Node /= null 
and then Get_Result_1.Node.Kind in Ada_Mode_In_Range | Ada_Mode_Default_Range; 
--# expr-done 2661
   If_Result := Is_A_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2662
   If_Result_1 := Let_Result;
else
   --# expr-start 2664 '<Eq at ast.py:3111>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3111
Cast_Expr := Return_Type; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 2663 '<at at ast.py:3111>' Get_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3111
Get_Result_2 := Get (Self, Param_Types, 0, True); 
--# expr-done 2663
      Cast_Result_1 := Create_Internal_Entity
        (Node => Get_Result_2.Node,
         Info => Get_Result_2.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2664
   If_Result_1 := Is_Equiv;
end if;
--# expr-done 2665
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result_1 := If_Result_2; 
--# end
--# expr-done 2666
Scope_Result := Let_Result_1;
   --# end
      Finalizer_Scope_2222;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Let_Result_2 := Result_Var_1; 
--# end
--# expr-done 2667
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2222;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Valid_Reducer_Candidate;
--# end
--# property-start BasicDecl.is_stream_subprogram_for_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3118
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Stream_Subprogram_For_Type
  (Node : Bare_Basic_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Return_Obj : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind return_obj Return_Obj
   Property_Result : Boolean;
            procedure Finalizer_Scope_195 with Inline_Always;
            procedure Finalizer_Scope_2223 with Inline_Always;
            procedure Finalizer_Scope_2224 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_5 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_6 : Boolean;
Get_Result : Internal_Entity_Defining_Name;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result : Boolean;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_195 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_1);
            end Finalizer_Scope_195;
            procedure Finalizer_Scope_2223 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2223;
            procedure Finalizer_Scope_2224 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2224;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2695 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2677 '<Cast to BaseTypeDecl at ast.py:3126>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3126
--# expr-start 2676 '<FieldAccess for classwide_type at ast.py:3126>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3126
--# expr-start 2675 '<Cast to BaseTypeDecl at ast.py:3126>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3126
--# expr-start 2674 '<EnvGet at ast.py:3125>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3125
--# expr-start 2671 '<FieldAccess for get_unit_root_decl at ast.py:3122>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3122
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2670 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 2668 '<Symbol Ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 2668
--# expr-start 2669 '<Symbol Streams at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Streams); 
--# expr-done 2669
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 2670
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 2671
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2672 '<FieldAccess for children_env at ast.py:3124>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3124
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 2672
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2223;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 2673 '<Symbol Root_Stream_Type at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Stream_Type); 
--# expr-done 2673
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 2674
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2675
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 2676
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 2677
Block_Var := Cast_Result_1; 
--# bind root_stream_type Block_Var
--# expr-start 2678 '<FieldAccess for subp_spec_or_null at ast.py:3128>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3128
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2678
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 2679 '<FieldAccess for unpacked_formal_params at ast.py:3128>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3128
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2679
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2224;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_1 := Result_Var_1; Inc_Ref (Block_Var_1);
--# bind params Block_Var_1
--# expr-start 2694 '<bind at ast.py:3130>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3130
--# expr-start 2680 '<FieldAccess for origin_node at ast.py:3131>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3131
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 2680
Bound_Origin := Fld_5; 
--# expr-start 2681 '<FieldAccess for is_subprogram at ast.py:3132>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3132
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 2681
if Fld_6 then
   --# expr-start 2685 '<FieldAccess for is_access_to at ast.py:3134>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3134
--# expr-start 2684 '<FieldAccess for formal_type at ast.py:3133>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3133
--# expr-start 2683 '<FieldAccess for formal_decl at ast.py:3133>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3133
--# expr-start 2682 '<at at ast.py:3133>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3133
Get_Result := Get (Self, Block_Var_1, 0, True); 
--# expr-done 2682
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 2683
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_7.Node, Origin => Bound_Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 2684
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_access_to
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Access_To (Node => Fld_8.Node, Typ => Block_Var, Origin => Bound_Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 2685
   If_Result := Fld_9;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 2693 '<If at ast.py:3135>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3135
if Return_Obj then
   --# expr-start 2688 '<FieldAccess for matching_formal_type at ast.py:3137>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3137
--# expr-start 2687 '<FieldAccess for return_type at ast.py:3137>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3137
--# expr-start 2686 '<FieldAccess for subp_spec_or_null at ast.py:3137>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3137
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2686
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_10.Node, Origin => Bound_Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 2687
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_11.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 2688
   If_Result_1 := Fld_12;
else
   --# expr-start 2692 '<FieldAccess for matching_formal_type at ast.py:3138>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3138
--# expr-start 2691 '<FieldAccess for formal_type at ast.py:3138>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3138
--# expr-start 2690 '<FieldAccess for formal_decl at ast.py:3138>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3138
--# expr-start 2689 '<at at ast.py:3138>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3138
Get_Result_1 := Get (Self, Block_Var_1, 1, True); 
--# expr-done 2689
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_13 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 2690
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_14 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_13.Node, Origin => Bound_Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 2691
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_14.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 2692
   If_Result_1 := Fld_15;
end if;
--# expr-done 2693
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 2694
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2695
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_195;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_195;
                     Finalizer_Scope_2223;
                     Finalizer_Scope_2224;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Stream_Subprogram_For_Type;
--# end
--# property-start BasicDecl.is_put_image_subprogram_for_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3142
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Put_Image_Subprogram_For_Type
  (Node : Bare_Basic_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   Property_Result : Boolean;
            procedure Finalizer_Scope_196 with Inline_Always;
            procedure Finalizer_Scope_2225 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Classwide_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_4 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_5 : Boolean;
Get_Result : Internal_Entity_Defining_Name;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
If_Result : Boolean;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Boolean;
If_Result_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_196 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var_1);
            end Finalizer_Scope_196;
            procedure Finalizer_Scope_2225 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2225;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2712 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2698 '<Cast to BaseTypeDecl at ast.py:3149>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3149
--# expr-start 2697 '<FieldAccess for classwide_type at ast.py:3149>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3149
--# expr-start 2696 '<FieldAccess for root_buffer_type at ast.py:3149>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3149
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_buffer_type
Fld := Libadalang.Implementation.Ada_Node_P_Root_Buffer_Type (Node => Self);
--# end
--# expr-done 2696
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2697
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 2698
Block_Var := Cast_Result; 
--# bind root_buffer_type Block_Var
--# expr-start 2699 '<FieldAccess for subp_spec_or_null at ast.py:3151>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3151
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2699
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 2700 '<FieldAccess for unpacked_formal_params at ast.py:3151>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3151
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2700
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2225;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
Block_Var_1 := Result_Var; Inc_Ref (Block_Var_1);
--# bind params Block_Var_1
--# expr-start 2711 '<bind at ast.py:3153>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3153
--# expr-start 2701 '<FieldAccess for origin_node at ast.py:3154>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3154
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_4 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 2701
Bound_Origin := Fld_4; 
--# expr-start 2702 '<FieldAccess for is_subprogram at ast.py:3155>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3155
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 2702
if Fld_5 then
   --# expr-start 2706 '<FieldAccess for matching_formal_type at ast.py:3158>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3158
--# expr-start 2705 '<FieldAccess for formal_type at ast.py:3157>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3157
--# expr-start 2704 '<FieldAccess for formal_decl at ast.py:3157>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3157
--# expr-start 2703 '<at at ast.py:3157>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3157
Get_Result := Get (Self, Block_Var_1, 0, True); 
--# expr-done 2703
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 2704
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_7 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_6.Node, Origin => Bound_Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 2705
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_7.Node, Formal_Type => Block_Var, Origin => Bound_Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 2706
   If_Result := Fld_8;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 2710 '<FieldAccess for matching_formal_type at ast.py:3161>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3161
--# expr-start 2709 '<FieldAccess for formal_type at ast.py:3160>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3160
--# expr-start 2708 '<FieldAccess for formal_decl at ast.py:3160>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3160
--# expr-start 2707 '<at at ast.py:3160>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3160
Get_Result_1 := Get (Self, Block_Var_1, 1, True); 
--# expr-done 2707
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_9 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 2708
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_10 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_9.Node, Origin => Bound_Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 2709
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_10.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 2710
   If_Result_1 := Fld_11;
else
   If_Result_1 := False;
end if;
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 2711
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2712
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_196;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_196;
                     Finalizer_Scope_2225;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Put_Image_Subprogram_For_Type;
--# end
--# property-start BasicDecl.can_be_paramless /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3164
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Can_Be_Paramless
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2718 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2713 '<FieldAccess for subp_spec_or_null at ast.py:3170>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3170
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2713
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 2717 '<FieldAccess for paramless at ast.py:3171>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3171
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2716 '<FieldAccess for dottable_subp at ast.py:3172>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3172
--# expr-start 2715 '<FieldAccess for md at ast.py:3172>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3172
--# expr-start 2714 '<FieldAccess for info at ast.py:3172>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3172
Fld_1 := Ent.Info;
--# expr-done 2714
Fld_2 := Fld_1.Md;
--# expr-done 2715
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 2716
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_3, Can_Be => True);
--# end
--# expr-done 2717
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2718
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Can_Be_Paramless;
--# end
--# property-start BasicDecl.is_paramless /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3177
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Paramless
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2724 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2719 '<FieldAccess for subp_spec_or_null at ast.py:3183>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3183
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2719
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 2723 '<FieldAccess for paramless at ast.py:3184>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3184
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2722 '<FieldAccess for dottable_subp at ast.py:3185>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3185
--# expr-start 2721 '<FieldAccess for md at ast.py:3185>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3185
--# expr-start 2720 '<FieldAccess for info at ast.py:3185>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3185
Fld_1 := Ent.Info;
--# expr-done 2720
Fld_2 := Fld_1.Md;
--# expr-done 2721
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 2722
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_3, Can_Be => False);
--# end
--# expr-done 2723
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2724
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Paramless;
--# end
--# property-start '[dispatcher]BasicDecl.constrain_prefix' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Discriminant_Spec .. Ada_Generic_Formal_Type_Decl | Ada_Synthetic_Formal_Param_Decl .. Ada_Formal_Type_Decl | Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.constrain_prefix
                  Property_Result := Basic_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.constrain_prefix
                  Property_Result := Component_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.constrain_prefix
                  Property_Result := Param_Spec_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.constrain_prefix
                  Property_Result := Basic_Subp_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.constrain_prefix
                  Property_Result := Base_Subp_Body_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicDecl.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3217
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Relative_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2725 '<FieldAccess for defining_name at ast.py:3218>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3218
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2725
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2726 '<FieldAccess for relative_name at ast.py:3218>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3218
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2726
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Relative_Name;
--# end
--# property-start BasicDecl.relative_name_text /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3224
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Relative_Name_Text
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Symbol_Type;
      Var_Expr : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Scope_Result : Symbol_Type;
Result_Var : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2727 '<FieldAccess for relative_name at ast.py:3225>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3225
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.relative_name
Fld := Libadalang.Implementation.Basic_Decl_P_Relative_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2727
Var_Expr := Fld;
if Var_Expr /= No_Entity_Name then
   --# scope-start
--# expr-start 2728 '<FieldAccess for name_symbol at ast.py:3225>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3225
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr.Node);
--# end
--# expr-done 2728
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := null;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Relative_Name_Text;
--# end
--# property-start BasicDecl.name_symbol /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3230
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Name_Symbol
  (Node : Bare_Basic_Decl
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2731 '<FieldAccess for name_symbol at ast.py:3230>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3230
--# expr-start 2730 '<FieldAccess for relative_name at ast.py:3230>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3230
--# expr-start 2729 '<as_bare_entity at ast.py:3230>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3230
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2729
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.relative_name
Fld := Libadalang.Implementation.Basic_Decl_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 2730
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 2731
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Name_Symbol;
--# end
--# property-start BasicDecl.basic_decl_next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3232
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Basic_Decl_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_203 with Inline_Always;
      Block_Var : Bare_Compilation_Unit;
Match_0 : Bare_Body_Node;
Match_1 : Bare_Basic_Decl;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Is_A : Boolean;
Is_A_1 : Boolean;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Sym : Symbol_Type;
Fld_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Not_Val : Boolean;
Fld_4 : Bare_Compilation_Unit;
Scope_Result_1 : Bare_Compilation_Unit;
Result_Var_1 : Bare_Compilation_Unit;
Match_Prefix : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Let_Result : Bare_Compilation_Unit;
Scope_Result_2 : Bare_Compilation_Unit;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Bare_Compilation_Unit;
Scope_Result_3 : Bare_Compilation_Unit;
Match_Result : Bare_Compilation_Unit;
Fld_5 : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_203 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_203;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2748 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 2743 '<Match at ast.py:3241>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3241
--# expr-start 2733 '<FieldAccess for decl at ast.py:3241>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3241
--# expr-start 2732 '<FieldAccess for enclosing_compilation_unit at ast.py:3241>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3241
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 2732
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Fld);
--# end
--# expr-done 2733
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Basic_Decl (Match_Prefix.Kind) is
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind b Match_1
--# expr-start 2734 '<as_bare_entity at ast.py:3244>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3244
Is_Null := Match_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Match_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2734
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2735 '<FieldAccess for defining_name at ast.py:3244>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3244
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2735
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2742 '<FieldAccess for referenced_unit at ast.py:3244>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3244
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2741 '<Not at ast.py:3246>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3246
--# expr-start 2736 '<IsA BasePackageDecl, GenericPackageDecl, GenericInstantiation, PackageRenamingDecl, SubpRenamingDecl, GenericRenamingDecl at ast.py:3250>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3250
Is_A := Match_1 /= null 
and then Match_1.Kind in Ada_Base_Package_Decl | Ada_Generic_Package_Decl_Range | Ada_Generic_Instantiation | Ada_Package_Renaming_Decl_Range | Ada_Subp_Renaming_Decl_Range | Ada_Generic_Renaming_Decl; 
--# expr-done 2736
if Is_A then
   If_Result_3 := True;
else
--# expr-start 2737 '<IsA BasicSubpDecl, GenericSubpDecl at ast.py:3261>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3261
Is_A_1 := Match_1 /= null 
and then Match_1.Kind in Ada_Basic_Subp_Decl | Ada_Generic_Subp_Decl_Range; 
--# expr-done 2737
if Is_A_1 then
   --# expr-start 2740 '<FieldAccess for has_aspect at ast.py:3262>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3262
--# expr-start 2738 '<as_entity at ast.py:3262>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3262
Is_Null_1 := Match_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Match_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 2738
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2739 '<Symbol Import at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 2739
--# property-call-start BasicDecl.has_aspect
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => If_Result_1.Node, Name => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => If_Result_1.Info);
--# end
--# expr-done 2740
   If_Result_2 := Fld_3;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Not_Val := not (If_Result_3); 
--# expr-done 2741
--# property-call-start Name.referenced_unit
Fld_4 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Var_Expr.Node, Kind => Unit_Body, Not_Found_Is_Error => Not_Val);
--# end
--# expr-done 2742
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Let_Result_1 := Result_Var_1; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 2743
Block_Var := Match_Result; 
--# expr-start 2747 '<Cast to BasicDecl at ast.py:3272>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3272
--# expr-start 2746 '<EnvGet at ast.py:3268>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3268
--# expr-start 2744 '<FieldAccess for children_env at ast.py:3268>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3268
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2744
--# expr-start 2745 '<Symbol __nextpart at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 2745
Env_Get_Result := AST_Envs.Get_First (Self => Fld_5, Key => Sym_1, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 2746
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2747
Let_Result_2 := Cast_Result_2; 
--# end
--# expr-done 2748
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_203;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_203;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Basic_Decl_Next_Part_For_Decl;
--# end
--# property-start '[dispatcher]BasicDecl.next_part_for_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Package_Decl | Ada_Entry_Index_Spec .. Ada_Error_Decl | Ada_Exception_Handler .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.next_part_for_decl
                  Property_Result := Basic_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start BaseFormalParamDecl.next_part_for_decl
                  Property_Result := Base_Formal_Param_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Task_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.next_part_for_decl
                  Property_Result := Base_Type_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.next_part_for_decl
                  Property_Result := Protected_Type_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl | Ada_Enum_Literal_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.next_part_for_decl
                  Property_Result := Basic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.next_part_for_decl
                  Property_Result := Entry_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Subp_Decl =>
                  --# property-call-start SyntheticSubpDecl.next_part_for_decl
                  Property_Result := Synthetic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt_Body .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Task_Body =>
                  --# property-call-start Body.next_part_for_decl
                  Property_Result := Body_Node_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
                  --# property-call-start BodyStub.next_part_for_decl
                  Property_Result := Body_Stub_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.next_part_for_decl
                  Property_Result := Exception_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericDecl.next_part_for_decl
                  Property_Result := Generic_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.next_part_for_decl
                  Property_Result := Generic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.next_part_for_decl
                  Property_Result := Object_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicDecl.body_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3286
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Body_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
      Var_Expr : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Body_Stub;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Stub;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
Let_Result_2 : Internal_Entity_Body_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2753 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 2752 '<Cast to Body at ast.py:3301>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3301
--# expr-start 2749 '<FieldAccess for next_part_for_decl at ast.py:3296>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3296
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2749
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 2751 '<Match at ast.py:3297>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3297
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind stub Match_0
--# expr-start 2750 '<FieldAccess for next_part_for_decl at ast.py:3298>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3298
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 2750
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind other Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2751
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 2752
Let_Result_2 := Cast_Result_2; 
--# end
--# expr-done 2753
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Body_Part_For_Decl;
--# end
--# property-start '[dispatcher]BasicDecl.next_part_for_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Next_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.next_part_for_name
                  Property_Result := Basic_Decl_P_Next_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start BaseFormalParamDecl.next_part_for_name
                  Property_Result := Base_Formal_Param_Decl_P_Next_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.next_part_for_name
                  Property_Result := Object_Decl_P_Next_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Next_Part_For_Name;
--# end
--# property-start '[dispatcher]BasicDecl.previous_part_for_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Previous_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.previous_part_for_name
                  Property_Result := Basic_Decl_P_Previous_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start BaseFormalParamDecl.previous_part_for_name
                  Property_Result := Base_Formal_Param_Decl_P_Previous_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.previous_part_for_name
                  Property_Result := Object_Decl_P_Previous_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Previous_Part_For_Name;
--# end
--# property-start BasicDecl.canonical_part_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3351
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2756 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2754 '<FieldAccess for previous_part_for_name at ast.py:3358>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3358
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Ent.Node, Sym => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2754
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 2755 '<FieldAccess for canonical_part_for_name at ast.py:3359>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3359
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part_for_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part_For_Name (Node => Var_Expr.Node, Sym => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 2755
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2756
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Canonical_Part_For_Name;
--# end
--# property-start BasicDecl.most_visible_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3362
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Most_Visible_Part
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2760 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2758 '<FieldAccess for most_visible_part at ast.py:3370>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3370
--# expr-start 2757 '<FieldAccess for defining_name_or_raise at ast.py:3370>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3370
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2757
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.most_visible_part
Fld_1 := Libadalang.Implementation.Defining_Name_P_Most_Visible_Part (Node => Fld.Node, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 2758
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2759 '<FieldAccess for basic_decl at ast.py:3370>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3370
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2759
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2760
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Most_Visible_Part;
--# end
--# property-start BasicDecl.most_visible_part_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3372
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Most_Visible_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_210 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Is_Null_1 : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
As_Entity : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
If_Result_3 : Internal_Entity_Basic_Decl;
If_Result_4 : Internal_Entity_Basic_Decl;
If_Result_5 : Internal_Entity_Basic_Decl;
If_Result_6 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_210 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_210;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2778 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2761 '<FieldAccess for next_part_for_name at ast.py:3379>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3379
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Name (Node => Ent.Node, Sym => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2761
Block_Var := Fld; 
--# bind np Block_Var
--# expr-start 2777 '<Cond at ast.py:3380>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3380
--# expr-start 2762 '<IsNull at ast.py:3382>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3382
Is_Null := Block_Var.Node = null; 
--# expr-done 2762
if Is_Null then
   If_Result_6 := Ent;
else
--# expr-start 2763 '<IsNull at ast.py:3386>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3386
Is_Null_1 := Origin = null; 
--# expr-done 2763
if Is_Null_1 then
   --# expr-start 2764 '<FieldAccess for most_visible_part_for_name at ast.py:3387>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3387
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_part_for_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Part_For_Name (Node => Block_Var.Node, Sym => Sym, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 2764
   If_Result_5 := Fld_1;
else
--# expr-start 2767 '<Not at ast.py:3391>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3391
--# expr-start 2765 '<FieldAccess for is_in_private_part at ast.py:3391>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3391
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2765
if Fld_2 then
   If_Result := True;
else
   --# expr-start 2766 '<FieldAccess for is_in_public_part at ast.py:3391>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3391
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_in_public_part
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_In_Public_Part (Node => Block_Var.Node);
--# end
--# expr-done 2766
   If_Result := Fld_3;
end if;
Not_Val := not (If_Result); 
--# expr-done 2767
if Not_Val then
   --# expr-start 2773 '<If at ast.py:3392>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3392
--# expr-start 2771 '<Contains at ast.py:3393>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3393
   --# expr-start 2769 '<EnvGet at ast.py:3393>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3393
--# expr-start 2768 '<FieldAccess for node_env at ast.py:3393>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3393
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Node_Env (Node => Origin);
--# expr-done 2768
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_5, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 2769
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_115 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_115
      --# expr-start 2770 '<Eq at ???>' Is_Equiv None
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Item_115, Cast_Result); 
--# expr-done 2770
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 2771
if Quantifier_Result then
   --# expr-start 2772 '<FieldAccess for most_visible_part_for_name at ast.py:3394>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3394
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_part_for_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Part_For_Name (Node => Block_Var.Node, Sym => Sym, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 2772
   If_Result_1 := Fld_4;
else
   If_Result_1 := Ent;
end if;
--# expr-done 2773
   If_Result_4 := If_Result_1;
else
--# expr-start 2775 '<FieldAccess for is_visible at ast.py:3400>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3400
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2774 '<as_bare_entity at ast.py:3400>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3400
Is_Null_2 := Origin = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Origin); 
   If_Result_2 := As_Entity;
end if;
--# expr-done 2774
--# property-call-start BasicDecl.is_visible
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Visible (Node => Block_Var.Node, From_Node => If_Result_2, E_Info => Block_Var.Info);
--# end
--# expr-done 2775
if Fld_6 then
   --# expr-start 2776 '<FieldAccess for most_visible_part_for_name at ast.py:3401>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3401
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_part_for_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Part_For_Name (Node => Block_Var.Node, Sym => Sym, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 2776
   If_Result_3 := Fld_7;
else
   If_Result_3 := Ent;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
--# expr-done 2777
Let_Result := If_Result_6; 
--# end
--# expr-done 2778
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_210;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_210;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Most_Visible_Part_For_Name;
--# end
--# property-start BasicDecl.fqn_extra_suffix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3407
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fqn_Extra_Suffix
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_211 with Inline_Always;
      Is_A : Boolean;
Str : String_Type;
Is_A_1 : Boolean;
Str_1 : String_Type;
Is_A_2 : Boolean;
Str_2 : String_Type;
Str_3 : String_Type;
If_Result : String_Type;
If_Result_1 : String_Type;
If_Result_2 : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_211 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Str_1);
                     Dec_Ref (Str_2);
                     Dec_Ref (Str_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_211;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2787 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2786 '<Cond at ast.py:3414>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3414
--# expr-start 2779 '<IsA ClasswideTypeDecl at ast.py:3415>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3415
Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Classwide_Type_Decl_Range; 
--# expr-done 2779
if Is_A then
   --# expr-start 2780 '<String at ast.py:3416>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3416
Str := Create_String ("'Class"); 
--# expr-done 2780
   If_Result_2 := Str;
else
--# expr-start 2781 '<IsA DiscreteBaseSubtypeDecl at ast.py:3418>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3418
Is_A_1 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Discrete_Base_Subtype_Decl_Range; 
--# expr-done 2781
if Is_A_1 then
   --# expr-start 2782 '<String at ast.py:3419>' Str_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3419
Str_1 := Create_String ("'Base"); 
--# expr-done 2782
   If_Result_1 := Str_1;
else
--# expr-start 2783 '<IsA SynthAnonymousTypeDecl at ast.py:3427>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3427
Is_A_2 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Synth_Anonymous_Type_Decl_Range; 
--# expr-done 2783
if Is_A_2 then
   --# expr-start 2784 '<String at ast.py:3428>' Str_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3428
Str_2 := Create_String ("'Anonymous_Access"); 
--# expr-done 2784
   If_Result := Str_2;
else
   --# expr-start 2785 '<String at ast.py:3430>' Str_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3430
Str_3 := Create_String (""); 
--# expr-done 2785
   If_Result := Str_3;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 2786
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 2787
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_211;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_211;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Fqn_Extra_Suffix;
--# end
--# property-start BasicDecl.fully_qualified_name_string_array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3433
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name_String_Array
  (Node : Bare_Basic_Decl
      ; Include_Profile : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_profile Include_Profile
   Property_Result : String_Type_Array_Access;
            procedure Finalizer_Scope_213 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : String_Type;
Fld_2 : String_Type_Array_Access;
Let_Result : String_Type_Array_Access;
            procedure Finalizer_Scope_213 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_213;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2791 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2790 '<FieldAccess for fully_qualified_name_impl at ast.py:3439>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3439
--# expr-start 2788 '<FieldAccess for defining_name_or_raise at ast.py:3439>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3439
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2788
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2789 '<FieldAccess for fqn_extra_suffix at ast.py:3441>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3441
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fqn_extra_suffix
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Fqn_Extra_Suffix (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2789
--# property-call-start DefiningName.fully_qualified_name_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Fully_Qualified_Name_Impl (Node => Fld.Node, Include_Profile => Include_Profile, Suffix => Fld_1, E_Info => Fld.Info);
--# end
--# expr-done 2790
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 2791
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_213;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_213;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Fully_Qualified_Name_String_Array;
--# end
--# property-start BasicDecl.fully_qualified_name_array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3444
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name_Array
  (Node : Bare_Basic_Decl
      ; Include_Profile : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_profile Include_Profile
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_214 with Inline_Always;
      Fld : String_Type_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_214 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_214;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2795 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2794 '<Map at ast.py:3452>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3452
   --# expr-start 2792 '<FieldAccess for fully_qualified_name_string_array at ast.py:3450>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3450
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fully_qualified_name_string_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_String_Array (Node => Ent.Node, Include_Profile => Include_Profile, E_Info => Ent.Info);
--# end
--# expr-done 2792
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant String_Type_Array_Access := Fld;
      begin
         for Item_116 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_116
      --# expr-start 2793 '<to_symbol at ast.py:3452>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3452
Sym := String_To_Symbol (Self, Self.Unit.Context, Item_116); 
--# expr-done 2793
         declare
            Item_To_Append : constant Symbol_Type := Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2794
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2795
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_214;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_214;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Fully_Qualified_Name_Array;
--# end
--# property-start BasicDecl.fully_qualified_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3454
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_215 with Inline_Always;
      Str : String_Type;
Fld : String_Type_Array_Access;
Join_Result : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_215 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Join_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_215;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2799 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2798 '<Join at ast.py:3459>' Join_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3459
--# expr-start 2796 '<String at ast.py:3459>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3459
Str := Create_String ("."); 
--# expr-done 2796
--# expr-start 2797 '<FieldAccess for fully_qualified_name_string_array at ast.py:3459>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3459
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fully_qualified_name_string_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_String_Array (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2797
Join_Result := Join_Strings (Str, Fld); 
--# expr-done 2798
Let_Result := Join_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2799
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_215;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_215;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.canonical_fully_qualified_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3461
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_216 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : String_Type;
Fld_2 : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_216 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_216;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2803 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2802 '<FieldAccess for canonical_fully_qualified_name_impl at ast.py:3469>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3469
--# expr-start 2800 '<FieldAccess for defining_name_or_raise at ast.py:3468>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3468
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2800
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2801 '<FieldAccess for fqn_extra_suffix at ast.py:3471>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3471
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fqn_extra_suffix
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Fqn_Extra_Suffix (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2801
--# property-call-start DefiningName.canonical_fully_qualified_name_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Canonical_Fully_Qualified_Name_Impl (Node => Fld.Node, Include_Profile => False, Suffix => Fld_1, E_Info => Fld.Info);
--# end
--# expr-done 2802
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 2803
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_216;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_216;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Canonical_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.unique_identifying_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3475
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unique_Identifying_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_217 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : String_Type;
Fld_2 : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_217 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_217;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2807 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2806 '<FieldAccess for unique_identifying_name_impl at ast.py:3487>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3487
--# expr-start 2804 '<FieldAccess for defining_name_or_raise at ast.py:3487>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3487
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2804
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2805 '<FieldAccess for fqn_extra_suffix at ast.py:3488>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3488
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fqn_extra_suffix
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Fqn_Extra_Suffix (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2805
--# property-call-start DefiningName.unique_identifying_name_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Unique_Identifying_Name_Impl (Node => Fld.Node, Suffix => Fld_1, E_Info => Fld.Info);
--# end
--# expr-done 2806
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 2807
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_217;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_217;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Unique_Identifying_Name;
--# end
--# property-start BasicDecl.custom_id_text /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3491
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Custom_Id_Text
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_218 with Inline_Always;
            procedure Finalizer_Scope_2236 with Inline_Always;
            procedure Finalizer_Scope_2237 with Inline_Always;
            procedure Finalizer_Scope_2238 with Inline_Always;
            procedure Finalizer_Scope_2239 with Inline_Always;
            procedure Finalizer_Scope_2240 with Inline_Always;
            procedure Finalizer_Scope_2241 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Type_Expr;
Var_Expr_2 : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_3 : Internal_Entity_Type_Expr;
Var_Expr_4 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Base_Subp_Spec;
Str : String_Type;
Fld_1 : Internal_Entity_Type_Expr;
Str_1 : String_Type;
Scope_Result : String_Type;
Result_Var : String_Type;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Str_2 : String_Type;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl;
Fld_4 : Internal_Entity_Type_Expr;
Fld_5 : String_Type;
Map_Result : String_Type_Array_Access;
Join_Result : String_Type;
Scope_Result_1 : String_Type;
Result_Var_1 : String_Type;
Fld_6 : Internal_Entity_Type_Expr;
Str_3 : String_Type;
Scope_Result_2 : String_Type;
Result_Var_2 : String_Type;
Concat_Result : String_Type;
Concat_Result_1 : String_Type;
Concat_Result_2 : String_Type;
Fld_7 : Internal_Entity_Type_Expr;
Str_4 : String_Type;
Fld_8 : String_Type;
Concat_Result_3 : String_Type;
Scope_Result_3 : String_Type;
Result_Var_3 : String_Type;
Concat_Result_4 : String_Type;
Str_5 : String_Type;
Concat_Result_5 : String_Type;
Scope_Result_4 : String_Type;
Str_6 : String_Type;
Result_Var_4 : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_218 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Str_6);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_218;
            procedure Finalizer_Scope_2236 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Concat_Result_4);
                     Dec_Ref (Str_5);
                     Dec_Ref (Concat_Result_5);
            end Finalizer_Scope_2236;
            procedure Finalizer_Scope_2237 is
            begin
                     Dec_Ref (Str_1);
            end Finalizer_Scope_2237;
            procedure Finalizer_Scope_2238 is
            begin
                     Dec_Ref (Var_Expr_2);
                     Dec_Ref (Str_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Join_Result);
            end Finalizer_Scope_2238;
            procedure Finalizer_Scope_2239 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2239;
            procedure Finalizer_Scope_2240 is
            begin
                     Dec_Ref (Str_3);
            end Finalizer_Scope_2240;
            procedure Finalizer_Scope_2241 is
            begin
                     Dec_Ref (Str_4);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result_3);
            end Finalizer_Scope_2241;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2832 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2808 '<FieldAccess for subp_spec_or_null at ast.py:3493>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3493
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2808
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 2830 '<Concat at ast.py:3509>' Concat_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3509
--# expr-start 2828 '<Concat at ast.py:3507>' Concat_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3507
--# expr-start 2823 '<Concat at ast.py:3496>' Concat_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3496
--# expr-start 2809 '<String at ast.py:3496>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3496
Str := Create_String ("("); 
--# expr-done 2809
--# expr-start 2822 '<Concat at ast.py:3497>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3497
--# expr-start 2810 '<FieldAccess for returns at ast.py:3497>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3497
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2810
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 2811 '<String at ast.py:3497>' Str_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3497
Str_1 := Create_String ("("); 
--# expr-done 2811
Scope_Result := Str_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2237;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_String;
end if;
      Inc_Ref (Result_Var);
--# expr-start 2821 '<Concat at ast.py:3505>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3505
--# expr-start 2812 '<FieldAccess for unpacked_formal_params at ast.py:3498>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3498
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2812
Var_Expr_2 := Fld_2;
      Inc_Ref (Var_Expr_2);
if Var_Expr_2 /= No_Internal_Entity_Defining_Name_Array_Type then
   --# scope-start
   --# bind ufp Var_Expr_2
--# expr-start 2818 '<Join at ast.py:3499>' Join_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3499
--# expr-start 2813 '<String at ast.py:3499>' Str_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3499
Str_2 := Create_String (", "); 
--# expr-done 2813
--# expr-start 2817 '<Map at ast.py:3500>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3500
      declare
         Map_Result_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Var_Expr_2;
      begin
         for Item_117 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_117
      --# expr-start 2816 '<FieldAccess for custom_id_text at ast.py:3502>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3502
--# expr-start 2815 '<FieldAccess for type_expression at ast.py:3502>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3502
--# expr-start 2814 '<FieldAccess for formal_decl at ast.py:3502>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3502
      if Item_117.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_117.Node, E_Info => Item_117.Info);
--# end
--# expr-done 2814
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 2815
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 2816
         declare
            Item_To_Append : constant String_Type := Fld_5;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2239;
         end loop;
      end;
         Map_Result := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := String_Type_Vectors.Get
              (Map_Result_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2817
Join_Result := Join_Strings (Str_2, Map_Result); 
--# expr-done 2818
Scope_Result_1 := Join_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2238;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_String;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 2819 '<FieldAccess for returns at ast.py:3505>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3505
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2819
Var_Expr_3 := Fld_6;
if Var_Expr_3 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind _ Var_Expr_3
--# expr-start 2820 '<String at ast.py:3505>' Str_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3505
Str_3 := Create_String (")"); 
--# expr-done 2820
Scope_Result_2 := Str_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2240;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_String;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat_String (Result_Var_1, Result_Var_2); 
--# expr-done 2821
Concat_Result_1 := Concat_String (Result_Var, Concat_Result); 
--# expr-done 2822
Concat_Result_2 := Concat_String (Str, Concat_Result_1); 
--# expr-done 2823
--# expr-start 2824 '<FieldAccess for returns at ast.py:3507>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3507
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2824
Var_Expr_4 := Fld_7;
if Var_Expr_4 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind r Var_Expr_4
--# expr-start 2827 '<Concat at ast.py:3508>' Concat_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3508
--# expr-start 2825 '<String at ast.py:3508>' Str_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3508
Str_4 := Create_String (" -> "); 
--# expr-done 2825
--# expr-start 2826 '<FieldAccess for custom_id_text at ast.py:3508>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3508
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_8 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 2826
Concat_Result_3 := Concat_String (Str_4, Fld_8); 
--# expr-done 2827
Scope_Result_3 := Concat_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2241;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Empty_String;
end if;
      Inc_Ref (Result_Var_3);
Concat_Result_4 := Concat_String (Concat_Result_2, Result_Var_3); 
--# expr-done 2828
--# expr-start 2829 '<String at ast.py:3509>' Str_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3509
Str_5 := Create_String (")"); 
--# expr-done 2829
Concat_Result_5 := Concat_String (Concat_Result_4, Str_5); 
--# expr-done 2830
Scope_Result_4 := Concat_Result_5;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2236;
   Result_Var_4 := Scope_Result_4;
else
   --# expr-start 2831 '<String at ast.py:3510>' Str_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3510
Str_6 := Create_String (""); 
--# expr-done 2831
   Result_Var_4 := Str_6;
end if;
      Inc_Ref (Result_Var_4);
Let_Result := Result_Var_4; Inc_Ref (Let_Result);
--# end
--# expr-done 2832
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_218;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_218;
                     Finalizer_Scope_2236;
                     Finalizer_Scope_2237;
                     Finalizer_Scope_2238;
                     Finalizer_Scope_2239;
                     Finalizer_Scope_2240;
                     Finalizer_Scope_2241;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Custom_Id_Text;
--# end
--# property-start BasicDecl.does_aspects_make_preelaborable /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3513
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Does_Aspects_Make_Preelaborable
  (Node : Bare_Basic_Decl
      ; From_Body : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind from_body From_Body
   Property_Result : Boolean;
      Bound_Imprecise_Fallback : Boolean;
Sym : Symbol_Type;
Fld : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Not_Val : Boolean;
Sym_3 : Symbol_Type;
Fld_3 : Boolean;
Sym_4 : Symbol_Type;
Fld_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2845 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2844 '<bind at ast.py:3523>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3523
Bound_Imprecise_Fallback := False; 
--# expr-start 2834 '<FieldAccess for has_aspect at ast.py:3525>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3525
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2833 '<Symbol Pure at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pure); 
--# expr-done 2833
--# property-call-start BasicDecl.has_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2834
if Fld then
   If_Result := True;
else
   --# expr-start 2836 '<FieldAccess for has_aspect at ast.py:3526>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3526
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2835 '<Symbol Preelaborate at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Preelaborate); 
--# expr-done 2835
--# property-call-start BasicDecl.has_aspect
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2836
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2838 '<FieldAccess for has_aspect at ast.py:3527>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3527
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2837 '<Symbol Shared_Passive at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Shared_Passive); 
--# expr-done 2837
--# property-call-start BasicDecl.has_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_2, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2838
   If_Result_1 := Fld_2;
end if;
if If_Result_1 then
   If_Result_4 := True;
else
--# expr-start 2839 '<Not at ast.py:3528>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3528
Not_Val := not (From_Body); 
--# expr-done 2839
if Not_Val then
--# expr-start 2841 '<FieldAccess for has_aspect at ast.py:3530>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3530
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2840 '<Symbol Remote_Types at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remote_Types); 
--# expr-done 2840
--# property-call-start BasicDecl.has_aspect
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_3, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2841
if Fld_3 then
   If_Result_2 := True;
else
   --# expr-start 2843 '<FieldAccess for has_aspect at ast.py:3531>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3531
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2842 '<Symbol Remote_Call_Interface at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remote_Call_Interface); 
--# expr-done 2842
--# property-call-start BasicDecl.has_aspect
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_4, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2843
   If_Result_2 := Fld_4;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
Dyn_Var_Bind_Result := If_Result_4; 
--# expr-done 2844
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2845
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Does_Aspects_Make_Preelaborable;
--# end
--# property-start BasicDecl.wrap_public_reference /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3535
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Wrap_Public_Reference
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Generic_Instantiation;
Scope_Result : Internal_Entity_Generic_Instantiation;
Cast_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2849 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2848 '<If at ast.py:3558>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3558
--# expr-start 2846 '<IsA GenericSubpInternal, GenericPackageInternal at ast.py:3559>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3559
Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Generic_Subp_Internal_Range | Ada_Generic_Package_Internal_Range; 
--# expr-done 2846
if Is_A then
--# expr-start 2847 '<FieldAccess for get_instantiation at ast.py:3560>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3560
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_instantiation
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Instantiation (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2847
Var_Expr := Fld;
if Var_Expr /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Scope_Result.Node,
         Info => Scope_Result.Info);
   Result_Var := Cast_Result;
else
   Result_Var := Ent;
end if;
   If_Result := Result_Var;
else
   If_Result := Ent;
end if;
--# expr-done 2848
Let_Result := If_Result; 
--# end
--# expr-done 2849
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Wrap_Public_Reference;
--# end
--# property-start '[dispatcher]BasicDecl.is_constant_object' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Constant_Object
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl | Ada_Synthetic_Formal_Param_Decl .. Ada_Concrete_Formal_Subp_Decl | Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_Exception_Decl | Ada_Generic_Package_Decl .. Ada_Named_Stmt_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_constant_object
                  Property_Result := Basic_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.is_constant_object
                  Property_Result := Component_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.is_constant_object
                  Property_Result := Discriminant_Spec_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.is_constant_object
                  Property_Result := Param_Spec_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Decl =>
                  --# property-call-start SubpDecl.is_constant_object
                  Property_Result := Subp_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start EnumLiteralDecl.is_constant_object
                  Property_Result := Enum_Literal_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.is_constant_object
                  Property_Result := Exception_Handler_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.is_constant_object
                  Property_Result := For_Loop_Var_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.is_constant_object
                  Property_Result := Number_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.is_constant_object
                  Property_Result := Object_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_Constant_Object;
--# end
--# property-start BasicDecl.child_decl_initial_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2189
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Basic_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_138 with Inline_Always;
            procedure Finalizer_Scope_2142 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name;
Block_Var_1 : Internal_Entity_Dotted_Name;
Var_Expr : Internal_Entity_Dotted_Name;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : String_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Name;
Fld_5 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym_2 : Symbol_Type;
Fld_6 : Internal_Entity_Name;
Fld_7 : String_Type;
Sym_3 : Symbol_Type;
If_Result_1 : Symbol_Type;
Scope_Result : Symbol_Type;
Sym_4 : Symbol_Type;
Result_Var : Symbol_Type;
If_Result_2 : Symbol_Type;
If_Result_3 : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_138 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_138;
            procedure Finalizer_Scope_2142 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2142;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2870 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2851 '<FieldAccess for defining_name at ast.py:2198>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2198
--# expr-start 2850 '<as_bare_entity at ast.py:2198>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2198
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2850
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 2851
Block_Var := Fld; 
--# bind defining_name Block_Var
--# expr-start 2853 '<Cast to DottedName at ast.py:2199>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2199
--# expr-start 2852 '<FieldAccess for name at ast.py:2199>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2199
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Block_Var.Node.Defining_Name_F_Name, Info => Block_Var.Info);
--# expr-done 2852
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 2853
Block_Var_1 := Cast_Result; 
--# bind child_name Block_Var_1
--# expr-start 2869 '<Cond at ast.py:2200>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2200
--# expr-start 2857 '<Eq at ast.py:2203>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2203
--# expr-start 2855 '<to_symbol at ast.py:2203>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2203
--# expr-start 2854 '<FieldAccess for text at ast.py:2203>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2203
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Text (Node => Block_Var.Node);
--# expr-done 2854
Sym := String_To_Symbol (Self, Self.Unit.Context, Fld_2); 
--# expr-done 2855
--# expr-start 2856 '<Symbol standard at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard_381); 
--# expr-done 2856
Is_Equal := Sym = Sym_1; 
--# expr-done 2857
if Is_Equal then
   If_Result_3 := null;
else
--# expr-start 2858 '<FieldAccess for is_library_item at ast.py:2206>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2206
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 2858
if Fld_3 then
Var_Expr := Block_Var_1;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 2867 '<If at ast.py:2210>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2210
if Private_Part then
   --# expr-start 2863 '<to_symbol at ast.py:2212>' Sym_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2212
--# expr-start 2862 '<Concat at ast.py:2212>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2212
--# expr-start 2860 '<FieldAccess for text at ast.py:2212>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2212
--# expr-start 2859 '<FieldAccess for prefix at ast.py:2212>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2212
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 2859
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Text (Node => Fld_4.Node);
--# expr-done 2860
--# expr-start 2861 '<String at ast.py:2212>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2212
Str := Create_String (".__privatepart"); 
--# expr-done 2861
Concat_Result := Concat_String (Fld_5, Str); 
--# expr-done 2862
Sym_2 := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 2863
   If_Result_1 := Sym_2;
else
   --# expr-start 2866 '<to_symbol at ast.py:2213>' Sym_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2213
--# expr-start 2865 '<FieldAccess for text at ast.py:2213>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2213
--# expr-start 2864 '<FieldAccess for prefix at ast.py:2213>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2213
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 2864
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Text (Node => Fld_6.Node);
--# expr-done 2865
Sym_3 := String_To_Symbol (Self, Self.Unit.Context, Fld_7); 
--# expr-done 2866
   If_Result_1 := Sym_3;
end if;
--# expr-done 2867
Scope_Result := If_Result_1;
   --# end
      Finalizer_Scope_2142;
   Result_Var := Scope_Result;
else
   --# expr-start 2868 '<Symbol standard at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard_381); 
--# expr-done 2868
   Result_Var := Sym_4;
end if;
   If_Result_2 := Result_Var;
else
   If_Result_2 := null;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 2869
Let_Result := If_Result_3; 
--# end
--# expr-done 2870
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_138;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_138;
                     Finalizer_Scope_2142;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Child_Decl_Initial_Env_Name;
--# end
--# property-start BasicDecl.previous_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2326
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Body_Node;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2875 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 2874 '<Match at ast.py:2336>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2336
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind btd Match_0
--# expr-start 2872 '<Cast to BasicDecl at ast.py:2338>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2338
--# expr-start 2871 '<FieldAccess for previous_part at ast.py:2338>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2338
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Match_0.Node, Go_To_Incomplete => True, E_Info => Match_0.Info);
--# end
--# expr-done 2871
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 2872
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind bd Match_1
--# expr-start 2873 '<FieldAccess for previous_part_internal at ast.py:2339>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2339
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part_internal
Fld_1 := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 2873
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2874
Let_Result_3 := Match_Result; 
--# end
--# expr-done 2875
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BasicDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2396
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Static_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2876 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 2876
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Static_Decl;
--# end
--# property-start BasicDecl.is_library_item /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2451
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Library_Item
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2879 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2878 '<IsA LibraryItem at ast.py:2456>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2456
--# expr-start 2877 '<parent at ast.py:2456>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2456
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2877
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range; 
--# expr-done 2878
Let_Result := Is_A; 
--# end
--# expr-done 2879
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Library_Item;
--# end
--# property-start BasicDecl.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2467
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Declarative_Parts
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_156 with Inline_Always;
      Let_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_156 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_156;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2880 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Internal_Entity_Declarative_Part_Array_Type; Inc_Ref (Let_Result);
--# end
--# expr-done 2880
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_156;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_156;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Declarative_Parts;
--# end
--# property-start BasicDecl.get_aspect_assoc /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2487
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
            procedure Finalizer_Scope_2161 with Inline_Always;
      Var_Expr : Internal_Entity_Aspect_Spec;
Fld : Internal_Entity_Aspect_Spec;
Fld_1 : Internal_Entity_Aspect_Assoc_List;
Scope_Result : Internal_Entity_Aspect_Assoc_List;
Result_Var : Internal_Entity_Aspect_Assoc_List;
Fld_2 : Bare_Aspect_Assoc_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Aspect_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Aspect_Assoc;
Fld_4 : Internal_Entity_Name;
Fld_5 : String_Type;
Fld_6 : String_Type;
Is_Equal : Boolean;
Item_99 : Internal_Entity_Aspect_Assoc;
Bare_Item_99 : Bare_Aspect_Assoc;
Find_Result : Internal_Entity_Aspect_Assoc;
Let_Result : Internal_Entity_Aspect_Assoc;
            procedure Finalizer_Scope_2161 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2161;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2888 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2887 '<Find at ast.py:2492>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2492
--# expr-start 2881 '<FieldAccess for aspects at ast.py:2492>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2492
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Aspect_Spec (Node => Implementation.Basic_Decl_F_Aspects (Ent.Node), Info => Ent.Info);
--# expr-done 2881
Var_Expr := Fld;
if Var_Expr /= No_Entity_Aspect_Spec then
   --# scope-start
--# expr-start 2882 '<FieldAccess for aspect_assocs at ast.py:2492>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2492
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Aspect_Assoc_List (Node => Var_Expr.Node.Aspect_Spec_F_Aspect_Assocs, Info => Var_Expr.Info);
--# expr-done 2882
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Aspect_Assoc_List;
end if;
Fld_2 := Result_Var.Node;
      Find_Result := No_Entity_Aspect_Assoc;
      if Fld_2 = null then
      Find_Result := No_Entity_Aspect_Assoc;
      else
      declare
         Collection : constant Bare_Aspect_Assoc_List := Fld_2;
      begin
         for Untyped_Item_99 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_99 := Untyped_Item_99; 
Is_Null := Bare_Item_99 = null; 
if Is_Null then
   If_Result := No_Entity_Aspect_Assoc;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_99); 
   If_Result := As_Entity;
end if;
                  Item_99 := If_Result; 
   --# scope-start
               --# bind asp Item_99
      --# expr-start 2886 '<Eq at ast.py:2493>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2493
--# expr-start 2884 '<FieldAccess for aspect_name at ast.py:2493>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2493
      if Item_99.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2883 '<FieldAccess for id at ast.py:2493>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2493
      if Item_99.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Item_99.Node.Aspect_Assoc_F_Id, Info => Item_99.Info);
--# expr-done 2883
--# property-call-start AspectAssoc.aspect_name
Fld_5 := Libadalang.Implementation.Aspect_Assoc_P_Aspect_Name (Node => Item_99.Node, N => Fld_4);
--# end
--# expr-done 2884
--# expr-start 2885 '<FieldAccess for image at ast.py:2493>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2493
      if Name = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Image (Self => Name);
--# expr-done 2885
Is_Equal := Equivalent (Fld_5, Fld_6); 
--# expr-done 2886
      if Is_Equal then
         Find_Result := Item_99; 
      end if;
   --# end
      Finalizer_Scope_2161;
               exit when Is_Equal;
         end loop;
      end;
      end if;
--# expr-done 2887
Let_Result := Find_Result; 
--# end
--# expr-done 2888
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2161;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start BasicDecl.is_imported /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2578
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Imported
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2891 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2889 '<FieldAccess for defining_name_or_raise at ast.py:2583>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2583
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2889
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2890 '<FieldAccess for is_imported at ast.py:2583>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2583
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_imported
Fld_1 := Libadalang.Implementation.Defining_Name_P_Is_Imported (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2890
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2891
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Imported;
--# end
--# property-start BasicDecl.is_in_private_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2679
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_In_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2894 '<IsA PrivatePart at ast.py:2679>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2679
--# expr-start 2893 '<parent at ast.py:2679>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2679
--# expr-start 2892 '<parent at ast.py:2679>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2679
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2892
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 2893
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Private_Part_Range; 
--# expr-done 2894
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_In_Private_Part;
--# end
--# property-start BasicDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2955
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Env
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Defining_Env;
--# end
--# property-start BasicDecl.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2993
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Array_Ndims
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
Let_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2897 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2896 '<FieldAccess for array_ndims at ast.py:2995>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2995
--# expr-start 2895 '<FieldAccess for expr_type at ast.py:2995>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2995
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2895
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2896
Let_Result := Fld_1; 
--# end
--# expr-done 2897
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Array_Ndims;
--# end
--# property-start BasicDecl.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:2999
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Expr_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2900 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2898 '<FieldAccess for type_expression at ast.py:3021>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3021
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2898
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind te Var_Expr
--# expr-start 2899 '<FieldAccess for designated_type at ast.py:3021>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3021
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2899
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2900
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Expr_Type;
--# end
--# property-start BasicDecl.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3023
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Type_Expression
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Node_For_Entity : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2902 '<as_entity at ast.py:3024>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3024
--# expr-start 2901 '<No TypeExpr at ast.py:3024>' Node_For_Entity /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3024
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 2901
Is_Null := Node_For_Entity = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Node_For_Entity); 
   If_Result := As_Entity;
end if;
--# expr-done 2902
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Type_Expression;
--# end
--# property-start BasicDecl.constrain_prefix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3190
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_199 with Inline_Always;
      True_Rel : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_199 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_199;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2903 '<Block at ???>' Let_Result None
--# scope-start
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:3215") else null)); 
Let_Result := True_Rel; Inc_Ref (Let_Result);
--# end
--# expr-done 2903
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_199;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_199;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3274
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2905 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2904 '<FieldAccess for basic_decl_next_part_for_decl at ast.py:3284>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3284
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2904
Let_Result := Fld; 
--# end
--# expr-done 2905
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicDecl.next_part_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3303
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Next_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2907 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2906 '<FieldAccess for next_part_for_decl at ast.py:3325>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3325
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2906
Let_Result := Fld; 
--# end
--# expr-done 2907
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Next_Part_For_Name;
--# end
--# property-start BasicDecl.previous_part_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3327
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Previous_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2909 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2908 '<FieldAccess for previous_part_for_decl at ast.py:3349>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3349
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2908
Let_Result := Fld; 
--# end
--# expr-done 2909
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Previous_Part_For_Name;
--# end
 function Basic_Decl_P_Is_Constant_Object
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is (raise Property_Error
    with "Property BasicDecl.is_constant_object not implemented on type "
    & Kind_Name (Node));
   --
   --  Primitives for Bare_Abstract_State_Decl
   --
      procedure Abstract_State_Decl_Pre_Env_Actions
        (Self            : Bare_Abstract_State_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_227 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:22371");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Abstract_State_Decl
        (Self : Bare_Abstract_State_Decl
         ; Abstract_State_Decl_F_Name : Bare_Defining_Name
         ; Abstract_State_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Abstract_State_Decl_F_Name := Abstract_State_Decl_F_Name;
            Self.Abstract_State_Decl_F_Aspects := Abstract_State_Decl_F_Aspects;
      end Initialize_Fields_For_Abstract_State_Decl;
   function Abstract_State_Decl_F_Name
     (Node : Bare_Abstract_State_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Abstract_State_Decl_F_Name;
   end;
--# property-start AbstractStateDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22367
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_P_Defining_Names
  (Node : Bare_Abstract_State_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Abstract_State_Decl  := Bare_Abstract_State_Decl (Node);
     Ent : Internal_Entity_Abstract_State_Decl :=
       Internal_Entity_Abstract_State_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1523 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1523 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1523;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2910 '<FieldAccess for name at ast.py:22367>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22367
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Abstract_State_Decl_F_Name, Info => Ent.Info);
--# expr-done 2910
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1523;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1523;
            raise;
      end;
   return Property_Result;
end Abstract_State_Decl_P_Defining_Names;
--# end
--# property-start AbstractStateDecl.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22368
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_P_Type_Expression
  (Node : Bare_Abstract_State_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Abstract_State_Decl  := Bare_Abstract_State_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Type_Expr;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abstract_State_Decl_P_Type_Expression;
--# end
--# property-start AbstractStateDecl._env_mappings_227 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22371
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_227
  (Node : Bare_Abstract_State_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Abstract_State_Decl  := Bare_Abstract_State_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1754 with Inline_Always;
      Fld : Bare_Defining_Name;
Fld_1 : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Abstract_State_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1754 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1754;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2914 '<New EnvAssoc at ast.py:22371>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22371
--# expr-start 2911 '<New DesignatedEnv at ast.py:22371>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22371
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 2911
--# expr-start 2913 '<FieldAccess for name_symbol at ast.py:22371>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22371
--# expr-start 2912 '<FieldAccess for name at ast.py:22371>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22371
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Abstract_State_Decl_F_Name;
--# expr-done 2912
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 2913
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 2914
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1754;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1754;
            raise;
      end;
   return Property_Result;
end Env_Mappings_227;
--# end
   --
   --  Primitives for Bare_Anonymous_Expr_Decl
   --
      procedure Initialize_Fields_For_Anonymous_Expr_Decl
        (Self : Bare_Anonymous_Expr_Decl
         ; Anonymous_Expr_Decl_F_Expr : Bare_Expr
        ) is
      begin
            Self.Anonymous_Expr_Decl_F_Expr := Anonymous_Expr_Decl_F_Expr;
      end Initialize_Fields_For_Anonymous_Expr_Decl;
   function Anonymous_Expr_Decl_F_Expr
     (Node : Bare_Anonymous_Expr_Decl) return Bare_Expr
   is
   begin
         return Node.Anonymous_Expr_Decl_F_Expr;
   end;
--# property-start AnonymousExprDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10869
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Defining_Names
  (Node : Bare_Anonymous_Expr_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Defining_Name_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Expr_Decl_P_Defining_Names;
--# end
--# property-start AnonymousExprDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10870
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Defining_Env
  (Node : Bare_Anonymous_Expr_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
     Ent : Internal_Entity_Anonymous_Expr_Decl :=
       Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_846 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_846 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_846;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2916 '<FieldAccess for defining_env at ast.py:10870>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10870
--# expr-start 2915 '<FieldAccess for type_expression at ast.py:10870>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10870
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2915
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 2916
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_846;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_846;
            raise;
      end;
   return Property_Result;
end Anonymous_Expr_Decl_P_Defining_Env;
--# end
--# property-start AnonymousExprDecl.get_formal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10872
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Get_Formal
  (Node : Bare_Anonymous_Expr_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
     Ent : Internal_Entity_Anonymous_Expr_Decl :=
       Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_847 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Assoc;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_847 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_847;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Anonymous_Expr_Decl_Anonymous_Expr_Decl_P_Get_Formal,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Anonymous_Expr_Decl,
                                As_Bare_Anonymous_Expr_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Defining_Name;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 2922 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2919 '<Cast to BasicAssoc at ast.py:10880>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10880
--# expr-start 2918 '<parent at ast.py:10880>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10880
--# expr-start 2917 '<FieldAccess for expr at ast.py:10880>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10880
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Anonymous_Expr_Decl_F_Expr, Info => Ent.Info);
--# expr-done 2917
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 2918
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Assoc
   then
      Cast_Result := Create_Internal_Entity_Basic_Assoc
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Assoc;
   end if;
--# expr-done 2919
Block_Var := Cast_Result; 
--# bind assoc Block_Var
--# expr-start 2921 '<at at ast.py:10881>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10881
--# expr-start 2920 '<FieldAccess for get_params at ast.py:10881>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10881
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicAssoc.get_params
Fld_2 := Libadalang.Implementation.Basic_Assoc_P_Get_Params (Node => Block_Var.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 2920
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 2921
Let_Result := Get_Result; 
--# end
--# expr-done 2922
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_847;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_847;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Defining_Name,
                        As_Internal_Entity_Defining_Name => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Anonymous_Expr_Decl_P_Get_Formal;
--# end
--# property-start AnonymousExprDecl.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10883
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Type_Expression
  (Node : Bare_Anonymous_Expr_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
     Ent : Internal_Entity_Anonymous_Expr_Decl :=
       Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Let_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2926 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2925 '<FieldAccess for type_expression at ast.py:10890>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10890
--# expr-start 2924 '<FieldAccess for basic_decl at ast.py:10890>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10890
--# expr-start 2923 '<FieldAccess for get_formal at ast.py:10890>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10890
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AnonymousExprDecl.get_formal
Fld := Libadalang.Implementation.Anonymous_Expr_Decl_P_Get_Formal (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2923
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2924
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2925
Let_Result := Fld_2; 
--# end
--# expr-done 2926
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Expr_Decl_P_Type_Expression;
--# end
--# property-start AnonymousExprDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10892
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Is_Static_Decl
  (Node : Bare_Anonymous_Expr_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
     Ent : Internal_Entity_Anonymous_Expr_Decl :=
       Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2929 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2928 '<FieldAccess for is_static_expr at ast.py:10894>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10894
--# expr-start 2927 '<FieldAccess for expr at ast.py:10894>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10894
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Anonymous_Expr_Decl_F_Expr, Info => Ent.Info);
--# expr-done 2927
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_1 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 2928
Let_Result := Fld_1; 
--# end
--# expr-done 2929
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Expr_Decl_P_Is_Static_Decl;
--# end
   --
   --  Primitives for Bare_Base_Formal_Param_Decl
   --
--# property-start '[dispatcher]BaseFormalParamDecl.is_mandatory' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Base_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Formal_Param_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start BaseFormalParamDecl.is_mandatory
                  Property_Result := Base_Formal_Param_Decl_P_Is_Mandatory
                    (Self
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.is_mandatory
                  Property_Result := Param_Spec_P_Is_Mandatory
                    (Self
                    );
                  --# end
               when Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start SyntheticFormalParamDecl.is_mandatory
                  Property_Result := Synthetic_Formal_Param_Decl_P_Is_Mandatory
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory;
--# end
--# property-start BaseFormalParamDecl.formal_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4192
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Formal_Type
  (Node : Bare_Base_Formal_Param_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2932 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2930 '<FieldAccess for type_expression at ast.py:4198>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4198
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2930
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 2931 '<FieldAccess for designated_type at ast.py:4198>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4198
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2931
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2932
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Formal_Type;
--# end
--# property-start BaseFormalParamDecl.parent_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4200
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Parent_Decl
  (Node : Bare_Base_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2935 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2934 '<Cast to BasicDecl at ast.py:4202>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4202
--# expr-start 2933 '<FieldAccess for semantic_parent at ast.py:4202>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4202
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2933
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 2934
Let_Result := Cast_Result; 
--# end
--# expr-done 2935
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Parent_Decl;
--# end
--# property-start BaseFormalParamDecl.get_param /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4204
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Get_Param
  (Node : Bare_Base_Formal_Param_Decl
      ; Part : Internal_Entity_Basic_Decl
      ; Param : Symbol_Type
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
      --# bind self Self
   --# bind part Part
   --# bind param Param
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2244 with Inline_Always;
            procedure Finalizer_Scope_2245 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Base_Formal_Param_Holder;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_2 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2244 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2244;
            procedure Finalizer_Scope_2245 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2245;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2940 '<Block at ???>' Let_Result None
--# scope-start
Var_Expr := Part;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 2939 '<Find at ast.py:4209>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4209
--# expr-start 2936 '<FieldAccess for formal_param_holder_or_null at ast.py:4208>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4208
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2936
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 2937 '<FieldAccess for unpacked_formal_params at ast.py:4208>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4208
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2937
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2245;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Result_Var;
      begin
         for Item_118 of
               Collection.Items
         loop
   --# scope-start
               --# bind sf Item_118
      --# expr-start 2938 '<FieldAccess for name_is at ast.py:4209>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4209
      if Item_118.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_2 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_118.Node, Sym => Param);
--# end
--# expr-done 2938
      if Fld_2 then
         Find_Result := Item_118; 
      end if;
   --# end
               exit when Fld_2;
         end loop;
      end;
--# expr-done 2939
Scope_Result_1 := Find_Result;
   --# end
      Finalizer_Scope_2244;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Defining_Name;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2940
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2244;
                     Finalizer_Scope_2245;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Get_Param;
--# end
--# property-start BaseFormalParamDecl.decl_param /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4212
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Decl_Param
  (Node : Bare_Base_Formal_Param_Decl
      ; Param : Internal_Entity_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind param Param
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Symbol_Type;
Fld_3 : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2946 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2945 '<FieldAccess for get_param at ast.py:4219>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4219
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2942 '<Cast to BaseSubpBody at ast.py:4220>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4220
--# expr-start 2941 '<FieldAccess for parent_decl at ast.py:4220>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4220
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2941
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 2942
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 2943 '<FieldAccess for decl_part at ast.py:4220>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4220
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2943
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# expr-start 2944 '<FieldAccess for name_symbol at ast.py:4221>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4221
      if Param.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Param.Node);
--# end
--# expr-done 2944
--# property-call-start BaseFormalParamDecl.get_param
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, Param => Fld_2);
--# end
--# expr-done 2945
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Param;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2946
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Decl_Param;
--# end
--# property-start BaseFormalParamDecl.next_part_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4224
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Next_Part_For_Name
  (Node : Bare_Base_Formal_Param_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2951 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2949 '<FieldAccess for get_param at ast.py:4226>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4226
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2947 '<FieldAccess for parent_decl at ast.py:4227>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4227
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2947
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2948 '<FieldAccess for next_part_for_decl at ast.py:4227>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4227
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2948
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, Param => Sym);
--# end
--# expr-done 2949
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2950 '<FieldAccess for basic_decl at ast.py:4229>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4229
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2950
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2951
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Next_Part_For_Name;
--# end
--# property-start BaseFormalParamDecl.previous_part_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4231
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Previous_Part_For_Name
  (Node : Bare_Base_Formal_Param_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2956 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2954 '<FieldAccess for get_param at ast.py:4233>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4233
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2952 '<FieldAccess for parent_decl at ast.py:4234>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4234
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2952
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2953 '<FieldAccess for previous_part_for_decl at ast.py:4234>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4234
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2953
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, Param => Sym);
--# end
--# expr-done 2954
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2955 '<FieldAccess for basic_decl at ast.py:4236>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4236
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2955
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2956
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Previous_Part_For_Name;
--# end
--# property-start BaseFormalParamDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4238
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Next_Part_For_Decl
  (Node : Bare_Base_Formal_Param_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2959 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2958 '<FieldAccess for next_part_for_name at ast.py:4240>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4240
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2957 '<FieldAccess for name_symbol at ast.py:4240>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4240
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2957
--# property-call-start '[dispatcher]BasicDecl.next_part_for_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Name (Node => Ent.Node, Sym => Fld, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2958
Let_Result := Fld_1; 
--# end
--# expr-done 2959
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BaseFormalParamDecl.previous_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4242
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Base_Formal_Param_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2962 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2961 '<FieldAccess for previous_part_for_name at ast.py:4244>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4244
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2960 '<FieldAccess for name_symbol at ast.py:4244>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4244
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2960
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Ent.Node, Sym => Fld, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2961
Let_Result := Fld_1; 
--# end
--# expr-done 2962
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BaseFormalParamDecl.is_mandatory /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4190
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Base_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Is_Mandatory;
--# end
   --
   --  Primitives for Bare_Component_Decl
   --
      procedure Component_Decl_Pre_Env_Actions
        (Self            : Bare_Component_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_1 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:4981");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Component_Decl
        (Self : Bare_Component_Decl
         ; Component_Decl_F_Ids : Bare_Defining_Name_List
         ; Component_Decl_F_Component_Def : Bare_Component_Def
         ; Component_Decl_F_Default_Expr : Bare_Expr
         ; Component_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Component_Decl_F_Ids := Component_Decl_F_Ids;
            Self.Component_Decl_F_Component_Def := Component_Decl_F_Component_Def;
            Self.Component_Decl_F_Default_Expr := Component_Decl_F_Default_Expr;
            Self.Component_Decl_F_Aspects := Component_Decl_F_Aspects;
      end Initialize_Fields_For_Component_Decl;
   function Component_Decl_F_Ids
     (Node : Bare_Component_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Component_Decl_F_Ids;
   end;
   function Component_Decl_F_Component_Def
     (Node : Bare_Component_Decl) return Bare_Component_Def
   is
   begin
         return Node.Component_Decl_F_Component_Def;
   end;
   function Component_Decl_F_Default_Expr
     (Node : Bare_Component_Decl) return Bare_Expr
   is
   begin
         return Node.Component_Decl_F_Default_Expr;
   end;
--# property-start ComponentDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4983
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Defining_Env
  (Node : Bare_Component_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_310 with Inline_Always;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_310 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_310;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2965 '<FieldAccess for defining_env at ast.py:4984>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4984
--# expr-start 2964 '<FieldAccess for type_expr at ast.py:4984>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4984
--# expr-start 2963 '<FieldAccess for component_def at ast.py:4984>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4984
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2963
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 2964
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_2 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld_1.Info);
--# end
--# expr-done 2965
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_310;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_310;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Defining_Env;
--# end
--# property-start ComponentDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4988
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Defining_Names
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_311 with Inline_Always;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_119 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_311 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_311;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2968 '<Map at ast.py:4988>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4988
   --# expr-start 2966 '<FieldAccess for ids at ast.py:4988>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4988
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Component_Decl_F_Ids;
--# expr-done 2966
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_119 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_119 := Untyped_Item_119; 
   --# scope-start
               --# bind i Item_119
      --# expr-start 2967 '<as_entity at ast.py:4988>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4988
Is_Null := Item_119 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_119); 
   If_Result := As_Entity;
end if;
--# expr-done 2967
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2968
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_311;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_311;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Defining_Names;
--# end
--# property-start ComponentDecl.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4990
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Array_Ndims
  (Node : Bare_Component_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2971 '<FieldAccess for array_ndims at ast.py:4990>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4990
--# expr-start 2970 '<FieldAccess for type_expr at ast.py:4990>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4990
--# expr-start 2969 '<FieldAccess for component_def at ast.py:4990>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4990
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2969
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 2970
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.array_ndims
Fld_2 := Libadalang.Implementation.Type_Expr_P_Array_Ndims (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2971
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Array_Ndims;
--# end
--# property-start ComponentDecl.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4992
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Type_Expression
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Bare_Component_Def;
Fld_1 : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2974 '<as_entity at ast.py:4992>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4992
--# expr-start 2973 '<FieldAccess for type_expr at ast.py:4992>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4992
--# expr-start 2972 '<FieldAccess for component_def at ast.py:4992>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4992
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Component_Decl_F_Component_Def;
--# expr-done 2972
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Component_Def_F_Type_Expr;
--# expr-done 2973
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2974
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Type_Expression;
--# end
--# property-start ComponentDecl.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4994
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Is_Constant_Object
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Is_Constant_Object;
--# end
--# property-start ComponentDecl.constrain_prefix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4996
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Constrain_Prefix
  (Node : Bare_Component_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_315 with Inline_Always;
      Var_Expr : Bare_Attribute_Ref;
True_Rel : Logic_Equation;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
Cast_Expr : Bare_Expr;
Cast_Result_1 : Bare_Attribute_Ref;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_315 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_315;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2986 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2985 '<If at ast.py:4998>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4998
--# expr-start 2975 '<Cast to AttributeRef at ast.py:5002>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5002
Cast_Expr := Prefix; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2975
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2976 '<FieldAccess for is_access_attr at ast.py:5002>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5002
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Var_Expr);
--# end
--# expr-done 2976
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result := True;
else
   --# expr-start 2979 '<AnyQuantifier at ast.py:5006>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5006
   --# expr-start 2977 '<FieldAccess for parents at ast.py:5006>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5006
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 2977
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_120 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_120
      --# expr-start 2978 '<IsA ProtectedDef at ast.py:5006>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5006
Is_A := Item_120 /= null 
and then Item_120.Kind in Ada_Protected_Def_Range; 
--# expr-done 2978
      Quantifier_Result := Is_A;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 2979
   If_Result := Quantifier_Result;
end if;
if If_Result then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5008") else null)); 
   If_Result_1 := True_Rel;
else
   --# expr-start 2984 '<And at ast.py:5012>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5012
--# expr-start 2982 '<Bind at ast.py:5013>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5013
--# expr-start 2980 '<FieldAccess for expected_type_var at ast.py:5013>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5013
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Prefix);
--# expr-done 2980
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 2981 '<FieldAccess for container_type at ast.py:5013>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5013
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentDecl.container_type
Fld_1 := Libadalang.Implementation.Component_Decl_P_Container_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2981
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5013") else null)); 
--# expr-done 2982
--# expr-start 2983 '<FieldAccess for matches_expected_prefix_type at ast.py:5014>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5014
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_prefix_type
Fld_2 := Libadalang.Implementation.Expr_P_Matches_Expected_Prefix_Type (Node => Prefix, Origin => Origin);
--# end
--# expr-done 2983
And_Pred := Create_And (Bind_Result, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5012") else null)); 
--# expr-done 2984
   If_Result_1 := And_Pred;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2985
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2986
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_315;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_315;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Constrain_Prefix;
--# end
--# property-start ComponentDecl.container_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5018
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Container_Type
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_316 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_316 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_316;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2992 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2991 '<as_entity at ast.py:5025>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5025
--# expr-start 2990 '<Cast to BaseTypeDecl at ast.py:5025>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5025
--# expr-start 2989 '<Find at ast.py:5023>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5023
   --# expr-start 2987 '<FieldAccess for parents at ast.py:5023>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5023
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 2987
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_121 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_121
      --# expr-start 2988 '<IsA BaseTypeDecl at ast.py:5024>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5024
Is_A := Item_121 /= null 
and then Item_121.Kind in Ada_Base_Type_Decl; 
--# expr-done 2988
      if Is_A then
         Find_Result := Item_121; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 2989
   if Find_Result = null
      or else Find_Result.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2990
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2991
Let_Result := If_Result; 
--# end
--# expr-done 2992
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_316;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_316;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Container_Type;
--# end
--# property-start ComponentDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5027
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Xref_Equation
  (Node : Bare_Component_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_317 with Inline_Always;
            procedure Finalizer_Scope_2257 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Component_Def;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_317 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_317;
            procedure Finalizer_Scope_2257 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2257;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3005 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 2993 '<FieldAccess for expr_type at ast.py:5029>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5029
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2993
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 3004 '<And at ast.py:5031>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5031
--# expr-start 2996 '<FieldAccess for sub_equation at ast.py:5031>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5031
--# expr-start 2995 '<FieldAccess for type_expr at ast.py:5031>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5031
--# expr-start 2994 '<FieldAccess for component_def at ast.py:5031>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5031
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2994
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Type_Expr (Node => Fld_1.Node.Component_Def_F_Type_Expr, Info => Fld_1.Info);
--# expr-done 2995
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 2996
--# expr-start 2997 '<FieldAccess for default_expr at ast.py:5032>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5032
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Component_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 2997
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 3003 '<And at ast.py:5034>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5034
--# expr-start 3001 '<And at ast.py:5034>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5034
--# expr-start 2999 '<Bind at ast.py:5034>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5034
--# expr-start 2998 '<FieldAccess for expected_type_var at ast.py:5034>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5034
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 2998
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5034") else null)); 
--# expr-done 2999
--# expr-start 3000 '<FieldAccess for sub_equation at ast.py:5035>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5035
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3000
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5034") else null)); 
--# expr-done 3001
--# expr-start 3002 '<FieldAccess for matches_expected_assign_type at ast.py:5036>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5036
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_7 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 3002
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5034") else null)); 
--# expr-done 3003
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2257;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5037") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5031") else null)); 
--# expr-done 3004
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 3005
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_317;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_317;
                     Finalizer_Scope_2257;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Xref_Equation;
--# end
--# property-start ComponentDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5041
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Xref_Entry_Point
  (Node : Bare_Component_Decl
  )
   return Boolean
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Xref_Entry_Point;
--# end
--# property-start ComponentDecl._env_mappings_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4981
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_1
  (Node : Bare_Component_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1527 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Component_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1527 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1527;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3007 '<FieldAccess for env_mappings at ast.py:4981>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4981
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3006 '<FieldAccess for ids at ast.py:4981>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4981
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Component_Decl_F_Ids;
--# expr-done 3006
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3007
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1527;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1527;
            raise;
      end;
   return Property_Result;
end Env_Mappings_1;
--# end
   --
   --  Primitives for Bare_Discriminant_Spec
   --
      procedure Discriminant_Spec_Pre_Env_Actions
        (Self            : Bare_Discriminant_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_0 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:4256");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Discriminant_Spec
        (Self : Bare_Discriminant_Spec
         ; Discriminant_Spec_F_Ids : Bare_Defining_Name_List
         ; Discriminant_Spec_F_Type_Expr : Bare_Type_Expr
         ; Discriminant_Spec_F_Default_Expr : Bare_Expr
         ; Discriminant_Spec_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Discriminant_Spec_F_Ids := Discriminant_Spec_F_Ids;
            Self.Discriminant_Spec_F_Type_Expr := Discriminant_Spec_F_Type_Expr;
            Self.Discriminant_Spec_F_Default_Expr := Discriminant_Spec_F_Default_Expr;
            Self.Discriminant_Spec_F_Aspects := Discriminant_Spec_F_Aspects;
      end Initialize_Fields_For_Discriminant_Spec;
   function Discriminant_Spec_F_Ids
     (Node : Bare_Discriminant_Spec) return Bare_Defining_Name_List
   is
   begin
         return Node.Discriminant_Spec_F_Ids;
   end;
   function Discriminant_Spec_F_Type_Expr
     (Node : Bare_Discriminant_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Discriminant_Spec_F_Type_Expr;
   end;
   function Discriminant_Spec_F_Default_Expr
     (Node : Bare_Discriminant_Spec) return Bare_Expr
   is
   begin
         return Node.Discriminant_Spec_F_Default_Expr;
   end;
--# property-start DiscriminantSpec.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4258
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Defining_Names
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_259 with Inline_Always;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_122 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_259 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_259;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3010 '<Map at ast.py:4258>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4258
   --# expr-start 3008 '<FieldAccess for ids at ast.py:4258>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4258
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Discriminant_Spec_F_Ids;
--# expr-done 3008
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_122 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_122 := Untyped_Item_122; 
   --# scope-start
               --# bind id Item_122
      --# expr-start 3009 '<as_entity at ast.py:4258>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4258
Is_Null := Item_122 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_122); 
   If_Result := As_Entity;
end if;
--# expr-done 3009
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3010
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_259;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_259;
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Defining_Names;
--# end
--# property-start DiscriminantSpec.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4259
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Defining_Env
  (Node : Bare_Discriminant_Spec
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
     Ent : Internal_Entity_Discriminant_Spec :=
       Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_260 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_260 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_260;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3012 '<FieldAccess for defining_env at ast.py:4259>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4259
--# expr-start 3011 '<FieldAccess for type_expr at ast.py:4259>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4259
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3011
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 3012
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_260;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_260;
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Defining_Env;
--# end
--# property-start DiscriminantSpec.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4261
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Type_Expression
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
     Ent : Internal_Entity_Discriminant_Spec :=
       Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3013 '<FieldAccess for type_expr at ast.py:4261>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4261
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3013
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Type_Expression;
--# end
--# property-start DiscriminantSpec.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4263
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Xref_Entry_Point
  (Node : Bare_Discriminant_Spec
  )
   return Boolean
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Xref_Entry_Point;
--# end
--# property-start DiscriminantSpec.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4265
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Is_Constant_Object
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Is_Constant_Object;
--# end
--# property-start DiscriminantSpec.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4267
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Xref_Equation
  (Node : Bare_Discriminant_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
     Ent : Internal_Entity_Discriminant_Spec :=
       Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_264 with Inline_Always;
            procedure Finalizer_Scope_2259 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_264 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_264;
            procedure Finalizer_Scope_2259 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2259;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3025 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3024 '<And at ast.py:4269>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4269
--# expr-start 3015 '<FieldAccess for sub_equation at ast.py:4270>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4270
--# expr-start 3014 '<FieldAccess for type_expr at ast.py:4270>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4270
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3014
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3015
--# expr-start 3016 '<FieldAccess for default_expr at ast.py:4271>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4271
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Discriminant_Spec_F_Default_Expr, Info => Ent.Info);
--# expr-done 3016
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 3023 '<And at ast.py:4273>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4273
--# expr-start 3021 '<And at ast.py:4273>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4273
--# expr-start 3019 '<Bind at ast.py:4273>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4273
--# expr-start 3017 '<FieldAccess for expected_type_var at ast.py:4273>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4273
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 3017
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 3018 '<FieldAccess for expr_type at ast.py:4273>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4273
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3018
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4273") else null)); 
--# expr-done 3019
--# expr-start 3020 '<FieldAccess for sub_equation at ast.py:4274>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4274
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3020
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4273") else null)); 
--# expr-done 3021
--# expr-start 3022 '<FieldAccess for matches_expected_assign_type at ast.py:4275>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4275
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 3022
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4273") else null)); 
--# expr-done 3023
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2259;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4276") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4269") else null)); 
--# expr-done 3024
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 3025
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_264;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_264;
                     Finalizer_Scope_2259;
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Xref_Equation;
--# end
--# property-start DiscriminantSpec._env_mappings_0 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4256
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_0
  (Node : Bare_Discriminant_Spec
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1526 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Discriminant_Spec;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1526 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1526;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3027 '<FieldAccess for env_mappings at ast.py:4256>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4256
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3026 '<FieldAccess for ids at ast.py:4256>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4256
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Discriminant_Spec_F_Ids;
--# expr-done 3026
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3027
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1526;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1526;
            raise;
      end;
   return Property_Result;
end Env_Mappings_0;
--# end
   --
   --  Primitives for Bare_Generic_Formal
   --
      procedure Initialize_Fields_For_Generic_Formal
        (Self : Bare_Generic_Formal
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Self.Generic_Formal_F_Decl := Generic_Formal_F_Decl;
      end Initialize_Fields_For_Generic_Formal;
   function Generic_Formal_F_Decl
     (Node : Bare_Generic_Formal) return Bare_Basic_Decl
   is
   begin
         return Node.Generic_Formal_F_Decl;
   end;
--# property-start GenericFormal.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11975
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Defining_Names
  (Node : Bare_Generic_Formal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Formal  := Bare_Generic_Formal (Node);
     Ent : Internal_Entity_Generic_Formal :=
       Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_917 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_917 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_917;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3029 '<FieldAccess for defining_names at ast.py:11975>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11975
--# expr-start 3028 '<FieldAccess for decl at ast.py:11975>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11975
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 3028
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3029
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_917;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_917;
            raise;
      end;
   return Property_Result;
end Generic_Formal_P_Defining_Names;
--# end
--# property-start GenericFormal.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11976
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Type_Expression
  (Node : Bare_Generic_Formal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Generic_Formal  := Bare_Generic_Formal (Node);
     Ent : Internal_Entity_Generic_Formal :=
       Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3031 '<FieldAccess for type_expression at ast.py:11976>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11976
--# expr-start 3030 '<FieldAccess for decl at ast.py:11976>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11976
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 3030
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3031
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Formal_P_Type_Expression;
--# end
--# property-start GenericFormal.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11978
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Xref_Entry_Point
  (Node : Bare_Generic_Formal
  )
   return Boolean
is
   Self : Bare_Generic_Formal  := Bare_Generic_Formal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Formal_P_Xref_Entry_Point;
--# end
--# property-start GenericFormal.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11979
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Xref_Equation
  (Node : Bare_Generic_Formal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Formal  := Bare_Generic_Formal (Node);
     Ent : Internal_Entity_Generic_Formal :=
       Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_920 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_920 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_920;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3033 '<FieldAccess for xref_equation at ast.py:11979>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11979
--# expr-start 3032 '<FieldAccess for decl at ast.py:11979>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11979
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 3032
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3033
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_920;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_920;
            raise;
      end;
   return Property_Result;
end Generic_Formal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Obj_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Obj_Decl
        (Self : Bare_Generic_Formal_Obj_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Obj_Decl;
   --
   --  Primitives for Bare_Generic_Formal_Package
   --
      procedure Initialize_Fields_For_Generic_Formal_Package
        (Self : Bare_Generic_Formal_Package
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Package;
   --
   --  Primitives for Bare_Generic_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Subp_Decl
        (Self : Bare_Generic_Formal_Subp_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Generic_Formal_Type_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Type_Decl
        (Self : Bare_Generic_Formal_Type_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Type_Decl;
--# property-start GenericFormalTypeDecl.default_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11993
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Type_Decl_P_Default_Type
  (Node : Bare_Generic_Formal_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Generic_Formal_Type_Decl  := Bare_Generic_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Generic_Formal_Type_Decl :=
       Internal_Entity_Generic_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Name;
Match_0 : Internal_Entity_Formal_Type_Decl;
Match_1 : Internal_Entity_Incomplete_Formal_Type_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Formal_Type_Decl;
Let_Result : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Incomplete_Formal_Type_Decl;
Let_Result_1 : Internal_Entity_Name;
Scope_Result_1 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Name;
Scope_Result_2 : Internal_Entity_Name;
Match_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3039 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 3037 '<Match at ast.py:11995>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11995
--# expr-start 3034 '<FieldAccess for decl at ast.py:11995>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11995
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 3034
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ft Match_0
--# expr-start 3035 '<FieldAccess for default_type at ast.py:11996>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11996
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Formal_Type_Decl_F_Default_Type, Info => Match_0.Info);
--# expr-done 3035
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Incomplete_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Incomplete_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ift Match_1
--# expr-start 3036 '<FieldAccess for default_type at ast.py:11997>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11997
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Incomplete_Formal_Type_Decl_F_Default_Type, Info => Match_1.Info);
--# expr-done 3036
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl .. Ada_Concrete_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Name; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3037
Var_Expr := Match_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
--# expr-start 3038 '<FieldAccess for name_designated_type at ast.py:11999>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11999
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3038
Scope_Result_3 := Fld_3;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 3039
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Formal_Type_Decl_P_Default_Type;
--# end
   --
   --  Primitives for Bare_Param_Spec
   --
      procedure Param_Spec_Pre_Env_Actions
        (Self            : Bare_Param_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_24 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:9696");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Param_Spec
        (Self : Bare_Param_Spec
         ; Param_Spec_F_Ids : Bare_Defining_Name_List
         ; Param_Spec_F_Has_Aliased : Bare_Aliased_Node
         ; Param_Spec_F_Mode : Bare_Mode
         ; Param_Spec_F_Type_Expr : Bare_Type_Expr
         ; Param_Spec_F_Default_Expr : Bare_Expr
         ; Param_Spec_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Param_Spec_F_Ids := Param_Spec_F_Ids;
            Self.Param_Spec_F_Has_Aliased := Param_Spec_F_Has_Aliased;
            Self.Param_Spec_F_Mode := Param_Spec_F_Mode;
            Self.Param_Spec_F_Type_Expr := Param_Spec_F_Type_Expr;
            Self.Param_Spec_F_Default_Expr := Param_Spec_F_Default_Expr;
            Self.Param_Spec_F_Aspects := Param_Spec_F_Aspects;
      end Initialize_Fields_For_Param_Spec;
   function Param_Spec_F_Ids
     (Node : Bare_Param_Spec) return Bare_Defining_Name_List
   is
   begin
         return Node.Param_Spec_F_Ids;
   end;
   function Param_Spec_F_Has_Aliased
     (Node : Bare_Param_Spec) return Bare_Aliased_Node
   is
   begin
         return Node.Param_Spec_F_Has_Aliased;
   end;
   function Param_Spec_F_Mode
     (Node : Bare_Param_Spec) return Bare_Mode
   is
   begin
         return Node.Param_Spec_F_Mode;
   end;
   function Param_Spec_F_Type_Expr
     (Node : Bare_Param_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Param_Spec_F_Type_Expr;
   end;
   function Param_Spec_F_Default_Expr
     (Node : Bare_Param_Spec) return Bare_Expr
   is
   begin
         return Node.Param_Spec_F_Default_Expr;
   end;
--# property-start ParamSpec.is_mandatory /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9692
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Is_Mandatory
  (Node : Bare_Param_Spec
  )
   return Boolean
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Expr;
Is_Null : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3041 '<IsNull at ast.py:9692>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9692
--# expr-start 3040 '<FieldAccess for default_expr at ast.py:9692>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9692
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Spec_F_Default_Expr;
--# expr-done 3040
Is_Null := Fld = null; 
--# expr-done 3041
         Property_Result := Is_Null;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Is_Mandatory;
--# end
--# property-start ParamSpec.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9693
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Defining_Names
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_772 with Inline_Always;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_123 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_772 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_772;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3044 '<Map at ast.py:9693>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9693
   --# expr-start 3042 '<FieldAccess for ids at ast.py:9693>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9693
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Spec_F_Ids;
--# expr-done 3042
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_123 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_123 := Untyped_Item_123; 
   --# scope-start
               --# bind id Item_123
      --# expr-start 3043 '<as_entity at ast.py:9693>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9693
Is_Null := Item_123 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_123); 
   If_Result := As_Entity;
end if;
--# expr-done 3043
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3044
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_772;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_772;
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Defining_Names;
--# end
--# property-start ParamSpec.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9699
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Type_Expression
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
     Ent : Internal_Entity_Param_Spec :=
       Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3045 '<FieldAccess for type_expr at ast.py:9699>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9699
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3045
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Type_Expression;
--# end
--# property-start ParamSpec.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9701
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Is_Constant_Object
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Mode;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3048 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3047 '<IsA Mode.In, Mode.Default at ast.py:9703>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9703
--# expr-start 3046 '<FieldAccess for mode at ast.py:9703>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9703
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Spec_F_Mode;
--# expr-done 3046
Is_A := Fld /= null 
and then Fld.Kind in Ada_Mode_In_Range | Ada_Mode_Default_Range; 
--# expr-done 3047
Let_Result := Is_A; 
--# end
--# expr-done 3048
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Is_Constant_Object;
--# end
--# property-start ParamSpec.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9705
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Defining_Env
  (Node : Bare_Param_Spec
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
     Ent : Internal_Entity_Param_Spec :=
       Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_775 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_775 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_775;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3051 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3050 '<FieldAccess for defining_env at ast.py:9707>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9707
--# expr-start 3049 '<FieldAccess for type_expr at ast.py:9707>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9707
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3049
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 3050
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3051
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_775;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_775;
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Defining_Env;
--# end
--# property-start ParamSpec.constrain_prefix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9709
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Constrain_Prefix
  (Node : Bare_Param_Spec
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_776 with Inline_Always;
            procedure Finalizer_Scope_2265 with Inline_Always;
      Var_Expr : Bare_Name;
Fld : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Cast_Expr_1 : Bare_Expr;
Cast_Result_1 : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Scope_Result : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_776 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_776;
            procedure Finalizer_Scope_2265 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2265;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3058 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3057 '<And at ast.py:9716>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9716
--# expr-start 3053 '<Bind at ast.py:9717>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9717
--# expr-start 3052 '<FieldAccess for type_var at ast.py:9717>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9717
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Prefix);
--# expr-done 3052
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9717") else null)); 
--# expr-done 3053
--# expr-start 3054 '<Cast to Name at ast.py:9718>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9718
Cast_Expr_1 := Prefix; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Name
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 3054
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 3056 '<Bind at ast.py:9719>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9719
--# expr-start 3055 '<FieldAccess for subp_spec_var at ast.py:9719>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9719
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Var_Expr);
--# end
--# expr-done 3055
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Cast_Expr_2 := No_Entity_Base_Formal_Param_Holder; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_1, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9719") else null)); 
--# expr-done 3056
Scope_Result := Bind_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2265;
   Result_Var := Scope_Result;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9721") else null)); 
   Result_Var := False_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Bind_Result, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9716") else null)); 
--# expr-done 3057
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 3058
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_776;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_776;
                     Finalizer_Scope_2265;
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Constrain_Prefix;
--# end
--# property-start ParamSpec.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9725
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Xref_Equation
  (Node : Bare_Param_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
     Ent : Internal_Entity_Param_Spec :=
       Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_777 with Inline_Always;
            procedure Finalizer_Scope_2266 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_777 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_777;
            procedure Finalizer_Scope_2266 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2266;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3070 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3059 '<FieldAccess for expr_type at ast.py:9727>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9727
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3059
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 3069 '<And at ast.py:9729>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9729
--# expr-start 3061 '<FieldAccess for sub_equation at ast.py:9729>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9729
--# expr-start 3060 '<FieldAccess for type_expr at ast.py:9729>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9729
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3060
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3061
--# expr-start 3062 '<FieldAccess for default_expr at ast.py:9731>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9731
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Param_Spec_F_Default_Expr, Info => Ent.Info);
--# expr-done 3062
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 3068 '<And at ast.py:9733>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9733
--# expr-start 3066 '<And at ast.py:9733>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9733
--# expr-start 3064 '<Bind at ast.py:9733>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9733
--# expr-start 3063 '<FieldAccess for expected_type_var at ast.py:9733>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9733
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 3063
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9733") else null)); 
--# expr-done 3064
--# expr-start 3065 '<FieldAccess for sub_equation at ast.py:9734>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9734
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3065
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9733") else null)); 
--# expr-done 3066
--# expr-start 3067 '<FieldAccess for matches_expected_assign_type at ast.py:9735>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9735
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 3067
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9733") else null)); 
--# expr-done 3068
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2266;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9736") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9729") else null)); 
--# expr-done 3069
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 3070
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_777;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_777;
                     Finalizer_Scope_2266;
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Xref_Equation;
--# end
--# property-start ParamSpec.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9740
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Xref_Entry_Point
  (Node : Bare_Param_Spec
  )
   return Boolean
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Xref_Entry_Point;
--# end
--# property-start ParamSpec._env_mappings_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9696
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_24
  (Node : Bare_Param_Spec
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1551 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Param_Spec;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1551 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1551;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3072 '<FieldAccess for env_mappings at ast.py:9696>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9696
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3071 '<FieldAccess for ids at ast.py:9696>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9696
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Spec_F_Ids;
--# expr-done 3071
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3072
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1551;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1551;
            raise;
      end;
   return Property_Result;
end Env_Mappings_24;
--# end
   --
   --  Primitives for Bare_Synthetic_Formal_Param_Decl
   --
      procedure Initialize_Fields_For_Synthetic_Formal_Param_Decl
        (Self : Bare_Synthetic_Formal_Param_Decl
         ; Synthetic_Formal_Param_Decl_F_Param_Type : Bare_Type_Expr
        ) is
      begin
            Self.Synthetic_Formal_Param_Decl_F_Param_Type := Synthetic_Formal_Param_Decl_F_Param_Type;
      Self.Synthetic_Formal_Param_Decl_Param_Name := null;
      end Initialize_Fields_For_Synthetic_Formal_Param_Decl;
   function Synthetic_Formal_Param_Decl_F_Param_Type
     (Node : Bare_Synthetic_Formal_Param_Decl) return Bare_Type_Expr
   is
   begin
         return Node.Synthetic_Formal_Param_Decl_F_Param_Type;
   end;
--# property-start SyntheticFormalParamDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18524
pragma Warnings (Off, "is not referenced");
 function Synthetic_Formal_Param_Decl_P_Defining_Names
  (Node : Bare_Synthetic_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Synthetic_Formal_Param_Decl  := Bare_Synthetic_Formal_Param_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1284 with Inline_Always;
      Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1284 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1284;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3076 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 3075 '<as_entity at ast.py:18525>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18525
--# expr-start 3074 '<FieldAccess for synthesize_defining_name at ast.py:18525>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18525
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3073 '<FieldAccess for param_name at ast.py:18525>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18525
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Formal_Param_Decl_Param_Name;
--# expr-done 3073
--# property-call-start AdaNode.synthesize_defining_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Synthesize_Defining_Name (Node => Self, Sym => Fld);
--# end
--# expr-done 3074
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3075
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => If_Result)); 
--# expr-done 3076
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1284;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1284;
            raise;
      end;
   return Property_Result;
end Synthetic_Formal_Param_Decl_P_Defining_Names;
--# end
--# property-start SyntheticFormalParamDecl.is_mandatory /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18528
pragma Warnings (Off, "is not referenced");
 function Synthetic_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Synthetic_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Synthetic_Formal_Param_Decl  := Bare_Synthetic_Formal_Param_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Formal_Param_Decl_P_Is_Mandatory;
--# end
--# property-start SyntheticFormalParamDecl.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18529
pragma Warnings (Off, "is not referenced");
 function Synthetic_Formal_Param_Decl_P_Type_Expression
  (Node : Bare_Synthetic_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Synthetic_Formal_Param_Decl  := Bare_Synthetic_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Synthetic_Formal_Param_Decl :=
       Internal_Entity_Synthetic_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3077 '<FieldAccess for param_type at ast.py:18529>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18529
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Synthetic_Formal_Param_Decl_F_Param_Type, Info => Ent.Info);
--# expr-done 3077
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Formal_Param_Decl_P_Type_Expression;
--# end
   --
   --  Primitives for Bare_Base_Package_Decl
   --
      procedure Initialize_Fields_For_Base_Package_Decl
        (Self : Bare_Base_Package_Decl
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Base_Package_Decl_F_Package_Name := Base_Package_Decl_F_Package_Name;
            Self.Base_Package_Decl_F_Aspects := Base_Package_Decl_F_Aspects;
            Self.Base_Package_Decl_F_Public_Part := Base_Package_Decl_F_Public_Part;
            Self.Base_Package_Decl_F_Private_Part := Base_Package_Decl_F_Private_Part;
            Self.Base_Package_Decl_F_End_Name := Base_Package_Decl_F_End_Name;
      end Initialize_Fields_For_Base_Package_Decl;
   function Base_Package_Decl_F_Package_Name
     (Node : Bare_Base_Package_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Base_Package_Decl_F_Package_Name;
   end;
   function Base_Package_Decl_F_Public_Part
     (Node : Bare_Base_Package_Decl) return Bare_Public_Part
   is
   begin
         return Node.Base_Package_Decl_F_Public_Part;
   end;
   function Base_Package_Decl_F_Private_Part
     (Node : Bare_Base_Package_Decl) return Bare_Private_Part
   is
   begin
         return Node.Base_Package_Decl_F_Private_Part;
   end;
   function Base_Package_Decl_F_End_Name
     (Node : Bare_Base_Package_Decl) return Bare_End_Name
   is
   begin
         return Node.Base_Package_Decl_F_End_Name;
   end;
--# property-start BasePackageDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11059
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Defining_Names
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
     Ent : Internal_Entity_Base_Package_Decl :=
       Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_858 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_858 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_858;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3078 '<FieldAccess for package_name at ast.py:11059>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11059
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Base_Package_Decl_F_Package_Name, Info => Ent.Info);
--# expr-done 3078
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_858;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_858;
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Defining_Names;
--# end
--# property-start BasePackageDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11060
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Defining_Env
  (Node : Bare_Base_Package_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
     Ent : Internal_Entity_Base_Package_Decl :=
       Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_859 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_859 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_859;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3079 '<FieldAccess for children_env at ast.py:11060>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11060
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3079
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_859;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_859;
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Defining_Env;
--# end
--# property-start BasePackageDecl.body_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11062
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Body_Part
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Package_Body
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
     Ent : Internal_Entity_Base_Package_Decl :=
       Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Package_Body;
      Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Package_Body;
Dyn_Var_Bind_Result : Internal_Entity_Package_Body;
Let_Result : Internal_Entity_Package_Body;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3083 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3082 '<bind at ast.py:11067>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11067
Bound_Imprecise_Fallback := False; 
--# expr-start 3081 '<Cast to PackageBody at ast.py:11070>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11070
--# expr-start 3080 '<FieldAccess for body_part_for_decl at ast.py:11069>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11069
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3080
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Package_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Package_Body;
   end if;
--# expr-done 3081
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 3082
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3083
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Body_Part;
--# end
--# property-start BasePackageDecl.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11073
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Declarative_Parts
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
     Ent : Internal_Entity_Base_Package_Decl :=
       Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_861 with Inline_Always;
            procedure Finalizer_Scope_2267 with Inline_Always;
      Var_Expr : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
Fld_1 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Declarative_Part;
Singleton_1 : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result : Internal_Entity_Declarative_Part_Array_Access;
Result_Var : Internal_Entity_Declarative_Part_Array_Access;
Concat_Result : Internal_Entity_Declarative_Part_Array_Access;
Let_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_861 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_861;
            procedure Finalizer_Scope_2267 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_2267;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3089 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3088 '<Concat at ast.py:11075>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11075
--# expr-start 3085 '<Cast to DeclarativePart at ast.py:11075>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11075
--# expr-start 3084 '<FieldAccess for public_part at ast.py:11075>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11075
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Public_Part (Node => Ent.Node.Base_Package_Decl_F_Public_Part, Info => Ent.Info);
--# expr-done 3084
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 3085
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
--# expr-start 3087 '<Cast to DeclarativePart at ast.py:11076>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11076
--# expr-start 3086 '<FieldAccess for private_part at ast.py:11076>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11076
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Private_Part (Node => Ent.Node.Base_Package_Decl_F_Private_Part, Info => Ent.Info);
--# expr-done 3086
      Cast_Result_1 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 3087
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Declarative_Part then
   --# scope-start
                Singleton_1 := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr;
Scope_Result := Singleton_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2267;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Singleton, Result_Var); 
--# expr-done 3088
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3089
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_861;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_861;
                     Finalizer_Scope_2267;
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Declarative_Parts;
--# end
--# property-start BasePackageDecl.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11079
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Env_Names
  (Node : Bare_Base_Package_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_862 with Inline_Always;
            procedure Finalizer_Scope_2268 with Inline_Always;
      Block_Var : String_Type;
Var_Expr : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Sym_1 : Symbol_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym_2 : Symbol_Type;
Array_Lit_1 : Symbol_Type_Array_Access;
Fld_1 : Bare_Private_Part;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_862 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_862;
            procedure Finalizer_Scope_2268 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2268;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3102 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3090 '<FieldAccess for top_level_env_name at ast.py:11087>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11087
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3090
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind fqn Block_Var
Var_Expr := Block_Var;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 3101 '<If at ast.py:11089>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11089
--# expr-start 3093 '<Not at ast.py:11090>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11090
--# expr-start 3092 '<IsNull at ast.py:11090>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11090
--# expr-start 3091 '<FieldAccess for private_part at ast.py:11090>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11090
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Base_Package_Decl_F_Private_Part;
--# expr-done 3091
Is_Null := Fld_1 = null; 
--# expr-done 3092
Not_Val := not (Is_Null); 
--# expr-done 3093
if Not_Val then
   --# expr-start 3095 '<ArrayLiteral at ast.py:11091>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11091
--# expr-start 3094 '<to_symbol at ast.py:11092>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11092
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr); 
--# expr-done 3094
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 3095
   If_Result := Array_Lit;
else
   --# expr-start 3100 '<ArrayLiteral at ast.py:11094>' Array_Lit_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11094
--# expr-start 3096 '<to_symbol at ast.py:11095>' Sym_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11095
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Var_Expr); 
--# expr-done 3096
--# expr-start 3099 '<to_symbol at ast.py:11096>' Sym_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11096
--# expr-start 3098 '<Concat at ast.py:11096>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11096
--# expr-start 3097 '<String at ast.py:11096>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11096
Str := Create_String (".__privatepart"); 
--# expr-done 3097
Concat_Result := Concat_String (Var_Expr, Str); 
--# expr-done 3098
Sym_2 := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 3099
Array_Lit_1 := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym_1, 2 => Sym_2)); 
--# expr-done 3100
   If_Result := Array_Lit_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 3101
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2268;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3102
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_862;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_862;
                     Finalizer_Scope_2268;
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Env_Names;
--# end
   --
   --  Primitives for Bare_Generic_Package_Internal
   --
      procedure Generic_Package_Internal_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Internal;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_110 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_111 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Package_Internal
        (Self : Bare_Generic_Package_Internal
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Package_Decl
              (Self, Base_Package_Decl_F_Package_Name, Base_Package_Decl_F_Aspects, Base_Package_Decl_F_Public_Part, Base_Package_Decl_F_Private_Part, Base_Package_Decl_F_End_Name);
      end Initialize_Fields_For_Generic_Package_Internal;
--# property-start GenericPackageInternal.is_library_item /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12118
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Internal_P_Is_Library_Item
  (Node : Bare_Generic_Package_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Package_Internal  := Bare_Generic_Package_Internal (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3106 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3105 '<IsA LibraryItem at ast.py:12123>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12123
--# expr-start 3104 '<parent at ast.py:12123>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12123
--# expr-start 3103 '<parent at ast.py:12123>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12123
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3103
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 3104
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Library_Item_Range; 
--# expr-done 3105
Let_Result := Is_A; 
--# end
--# expr-done 3106
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Internal_P_Is_Library_Item;
--# end
--# property-start GenericPackageInternal._env_trans_parent_110 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12125
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_110
  (Node : Bare_Generic_Package_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Package_Internal  := Bare_Generic_Package_Internal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_110;
--# end
--# property-start GenericPackageInternal._env_names_111 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12125
pragma Warnings (Off, "is not referenced");
 function Env_Names_111
  (Node : Bare_Generic_Package_Internal
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Package_Internal  := Bare_Generic_Package_Internal (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1638 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1638 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1638;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3107 '<FieldAccess for env_names at ast.py:12125>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12125
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.env_names
Fld := Libadalang.Implementation.Base_Package_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 3107
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1638;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1638;
            raise;
      end;
   return Property_Result;
end Env_Names_111;
--# end
   --
   --  Primitives for Bare_Package_Decl
   --
      procedure Package_Decl_Pre_Env_Actions
        (Self            : Bare_Package_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_48 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_49 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:11109");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_50 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:11116");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_51 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_52 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_53 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_55 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_54 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_57 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_56 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Decl
        (Self : Bare_Package_Decl
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Package_Decl
              (Self, Base_Package_Decl_F_Package_Name, Base_Package_Decl_F_Aspects, Base_Package_Decl_F_Public_Part, Base_Package_Decl_F_Private_Part, Base_Package_Decl_F_End_Name);
      end Initialize_Fields_For_Package_Decl;
--# property-start PackageDecl._env_do_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11107
pragma Warnings (Off, "is not referenced");
 function Env_Do_48
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3108 '<FieldAccess for env_hook at ast.py:11107>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11107
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3108
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_48;
--# end
--# property-start PackageDecl._initial_env_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11113
pragma Warnings (Off, "is not referenced");
 function Initial_Env_49
  (Node : Bare_Package_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1576 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1576 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1576;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3109 '<FieldAccess for child_decl_initial_env at ast.py:11113>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11113
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self, Private_Part => True);
--# end
--# expr-done 3109
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1576;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1576;
            raise;
      end;
   return Property_Result;
end Initial_Env_49;
--# end
--# property-start PackageDecl._env_mappings_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11116
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_50
  (Node : Bare_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
     Ent : Internal_Entity_Package_Decl :=
       Internal_Entity_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1577 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1577 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1577;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3110 '<FieldAccess for child_decl_env_assocs at ast.py:11116>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11116
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_env_assocs
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Env_Assocs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3110
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1577;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1577;
            raise;
      end;
   return Property_Result;
end Env_Mappings_50;
--# end
--# property-start PackageDecl._env_trans_parent_51 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11106
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_51
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_51;
--# end
--# property-start PackageDecl._env_names_52 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11118
pragma Warnings (Off, "is not referenced");
 function Env_Names_52
  (Node : Bare_Package_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1579 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1579 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1579;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3111 '<FieldAccess for env_names at ast.py:11118>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11118
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.env_names
Fld := Libadalang.Implementation.Base_Package_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 3111
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1579;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1579;
            raise;
      end;
   return Property_Result;
end Env_Names_52;
--# end
--# property-start PackageDecl._env_do_53 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11120
pragma Warnings (Off, "is not referenced");
 function Env_Do_53
  (Node : Bare_Package_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1580 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1580 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1580;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3112 '<FieldAccess for populate_dependent_units at ast.py:11120>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11120
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3112
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1580;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1580;
            raise;
      end;
   return Property_Result;
end Env_Do_53;
--# end
--# property-start PackageDecl._ref_env_nodes_54 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11123
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_54
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1581 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1581 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1581;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3113 '<FieldAccess for top_level_use_package_clauses at ast.py:11123>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11123
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3113
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1581;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1581;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_54;
--# end
--# property-start PackageDecl._ref_cond_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11125
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_55
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3115 '<IsA LibraryItem, Subunit at ast.py:11125>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11125
--# expr-start 3114 '<parent at ast.py:11125>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11125
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3114
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3115
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_55;
--# end
--# property-start PackageDecl._ref_env_nodes_56 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11129
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_56
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1583 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1583 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1583;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3116 '<FieldAccess for top_level_use_type_clauses at ast.py:11129>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11129
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3116
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1583;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1583;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_56;
--# end
--# property-start PackageDecl._ref_cond_57 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11131
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_57
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3118 '<IsA LibraryItem, Subunit at ast.py:11131>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11131
--# expr-start 3117 '<parent at ast.py:11131>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11131
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3117
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3118
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_57;
--# end
   --
   --  Primitives for Bare_Base_Type_Decl
   --
      procedure Base_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Base_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_2 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:5924");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Base_Type_Decl
        (Self : Bare_Base_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Base_Type_Decl_F_Name := Base_Type_Decl_F_Name;
      Self.Lf_Present_Attributes_Repo := False;
      Self.Lf_Stg_Attributes_Repo := No_Bare_Ada_Node;
      Self.Lf_Present_Dottable_Subps_Env := False;
      Self.Lf_Stg_Dottable_Subps_Env := Empty_Env;
      end Initialize_Fields_For_Base_Type_Decl;
   function Base_Type_Decl_F_Name
     (Node : Bare_Base_Type_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Base_Type_Decl_F_Name;
   end;
--# property-start BaseTypeDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5926
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Defining_Names
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_422 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_422 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_422;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3119 '<FieldAccess for name at ast.py:5926>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5926
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Base_Type_Decl_F_Name, Info => Ent.Info);
--# expr-done 3119
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_422;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_422;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Defining_Names;
--# end
--# property-start BaseTypeDecl.base_subtype /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5928
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Discrete_Base_Subtype_Decl;
Match_1 : Internal_Entity_Base_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
Cast_Expr : Internal_Entity_Discrete_Base_Subtype_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Subtype_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Base_Subtype,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3123 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 3122 '<Match at ast.py:5936>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5936
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Discrete_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind db Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind st Match_1
--# expr-start 3121 '<FieldAccess for base_subtype at ast.py:5938>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5938
--# expr-start 3120 '<FieldAccess for get_type at ast.py:5938>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5938
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 3120
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3121
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Classwide_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := Ent; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3122
Let_Result_3 := Match_Result; 
--# end
--# expr-done 3123
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Base_Subtype;
--# end
--# property-start BaseTypeDecl.anonymous_access_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5942
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Anonymous_Access_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Anonymous_Type_Access_Def;
Cast_Result : Bare_Type_Def;
New_Node_1 : Bare_Synth_Anonymous_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Anonymous_Access_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3129 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3128 '<as_entity at ???>' If_Result None
--# expr-start 3127 '<Cast to BaseTypeDecl at ???>' Cast_Result_1 None
--# expr-start 3126 '<New SynthAnonymousTypeDecl at ???>' New_Node_1 None
--# expr-start 3124 '<FieldAccess for name at ast.py:5945>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5945
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 3124
--# expr-start 3125 '<New AnonymousTypeAccessDef at ???>' New_Node None
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Self = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "AnonymousTypeAccessDef.type_decl cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Anonymous_Type_Access_Def);
Initialize
  (Self => New_Node,
   Kind => Ada_Anonymous_Type_Access_Def,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Anonymous_Type_Access_Def
     (Self => New_Node,
      Access_Def_F_Has_Not_Null => No_Bare_Ada_Node, Anonymous_Type_Access_Def_F_Type_Decl => Self);
--# expr-done 3125
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "TypeDecl.type_def cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synth_Anonymous_Type_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synth_Anonymous_Type_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synth_Anonymous_Type_Decl
     (Self => New_Node_1,
      Base_Type_Decl_F_Name => Fld, Type_Decl_F_Discriminants => No_Bare_Ada_Node, Type_Decl_F_Type_Def => Cast_Result);
--# expr-done 3126
      Cast_Result_1 := New_Node_1;
--# expr-done 3127
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3128
Let_Result := If_Result; 
--# end
--# expr-done 3129
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Anonymous_Access_Type;
--# end
--# property-start BaseTypeDecl.anonymous_access_type_or_null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5953
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Anonymous_Access_Type_Or_Null
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3131 '<Block at ???>' Let_Result None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3130 '<FieldAccess for anonymous_access_type at ast.py:5955>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5955
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3130
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3131
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Anonymous_Access_Type_Or_Null;
--# end
--# property-start BaseTypeDecl.attributes_repo /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5957
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_F_Attributes_Repo
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Type_Attributes_Repository
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Type_Attributes_Repository;
      New_Node : Bare_Type_Attributes_Repository;
Let_Result : Bare_Type_Attributes_Repository;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Attributes_Repo then
         Property_Result := Self.Lf_Stg_Attributes_Repo;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3133 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3132 '<New TypeAttributesRepository at ast.py:5960>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5960
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Type_Attributes_Repository);
Initialize
  (Self => New_Node,
   Kind => Ada_Type_Attributes_Repository,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
      New_Node.Type_Attributes_Repository_Base_Type :=
         Self;
--# expr-done 3132
Let_Result := New_Node; 
--# end
--# expr-done 3133
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Attributes_Repo := True;
      Self.Lf_Stg_Attributes_Repo := Property_Result;
   return Property_Result;
end Base_Type_Decl_F_Attributes_Repo;
--# end
--# property-start BaseTypeDecl.private_completion /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5962
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Private_Completion
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Bare_Ada_Node_List;
Var_Expr_1 : Bare_Private_Part;
Var_Expr_2 : Bare_Base_Package_Decl;
Var_Expr_3 : Bare_Public_Part;
Var_Expr_4 : Bare_Base_Type_Decl;
Fld : Bare_Declarative_Part;
Cast_Result : Bare_Public_Part;
Fld_1 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Package_Decl;
Fld_2 : Bare_Private_Part;
Scope_Result_1 : Bare_Private_Part;
Result_Var_1 : Bare_Private_Part;
Fld_3 : Bare_Ada_Node_List;
Scope_Result_2 : Bare_Ada_Node_List;
Result_Var_2 : Bare_Ada_Node_List;
Cast_Expr : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_4 : Symbol_Type;
Fld_5 : Symbol_Type;
Is_Equal : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Item_124 : Bare_Ada_Node;
Find_Result : Bare_Ada_Node;
Scope_Result_4 : Bare_Ada_Node;
Result_Var_4 : Bare_Ada_Node;
Cast_Result_3 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Private_Completion,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3147 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3146 '<as_entity at ast.py:5977>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5977
--# expr-start 3145 '<Cast to BaseTypeDecl at ast.py:5977>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5977
--# expr-start 3137 '<Cast to BasePackageDecl at ast.py:5971>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5971
--# expr-start 3135 '<Cast to PublicPart at ast.py:5970>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5970
--# expr-start 3134 '<FieldAccess for declarative_scope at ast.py:5970>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5970
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3134
   if Fld = null
      or else Fld.Kind in Ada_Public_Part_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3135
Var_Expr_3 := Cast_Result;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3136 '<parent at ast.py:5971>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5971
      if Var_Expr_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_3.Parent; 
--# expr-done 3136
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_1 := Result_Var;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 3137
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3138 '<FieldAccess for private_part at ast.py:5972>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5972
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_2.Base_Package_Decl_F_Private_Part;
--# expr-done 3138
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Var_Expr_1 := Result_Var_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3139 '<FieldAccess for decls at ast.py:5972>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5972
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_1.Declarative_Part_F_Decls;
--# expr-done 3139
Scope_Result_2 := Fld_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3144 '<Find at ast.py:5972>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5972
      Find_Result := No_Bare_Ada_Node;
      if Var_Expr = null then
      Find_Result := No_Bare_Ada_Node;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Var_Expr;
      begin
         for Untyped_Item_124 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_124 := Untyped_Item_124; 
   --# scope-start
               --# bind d Item_124
--# expr-start 3140 '<Cast to BaseTypeDecl at ast.py:5973>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5973
Cast_Expr := Item_124; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Cast_Expr;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 3140
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pp Var_Expr_4
--# expr-start 3143 '<Eq at ast.py:5975>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5975
--# expr-start 3141 '<FieldAccess for name_symbol at ast.py:5975>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5975
      if Var_Expr_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr_4);
--# end
--# expr-done 3141
--# expr-start 3142 '<FieldAccess for name_symbol at ast.py:5975>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5975
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3142
Is_Equal := Fld_4 = Fld_5; 
--# expr-done 3143
Scope_Result_3 := Is_Equal;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
      if Result_Var_3 then
         Find_Result := Item_124; 
      end if;
   --# end
               exit when Result_Var_3;
         end loop;
      end;
      end if;
--# expr-done 3144
Scope_Result_4 := Find_Result;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Bare_Ada_Node;
end if;
   if Result_Var_4 = null
      or else Result_Var_4.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Result_Var_4;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 3145
Is_Null := Cast_Result_3 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_3); 
   If_Result := As_Entity;
end if;
--# expr-done 3146
Let_Result := If_Result; 
--# end
--# expr-done 3147
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Private_Completion;
--# end
--# property-start BaseTypeDecl.is_view_of_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5980
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_View_Of_Type
  (Node : Bare_Base_Type_Decl
      ; Comp_View : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind comp_view Comp_View
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3153 '<Block at ???>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind typ Block_Var
--# expr-start 3152 '<Cond at ast.py:5987>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5987
--# expr-start 3148 '<IsNull at ast.py:5988>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5988
Is_Null := Comp_View.Node = null; 
--# expr-done 3148
if Is_Null then
   If_Result_1 := False;
else
--# expr-start 3149 '<Eq at ast.py:5989>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5989
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Comp_View; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 3149
if Is_Equiv then
   If_Result := True;
else
   --# expr-start 3151 '<FieldAccess for is_view_of_type at ast.py:5990>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5990
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3150 '<FieldAccess for previous_part at ast.py:5990>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5990
      if Comp_View.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Comp_View.Node, Go_To_Incomplete => True, E_Info => Comp_View.Info);
--# end
--# expr-done 3150
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Block_Var.Node, Comp_View => Fld, E_Info => Block_Var.Info);
--# end
--# expr-done 3151
   If_Result := Fld_1;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 3152
Let_Result := If_Result_1; 
--# end
--# expr-done 3153
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_View_Of_Type;
--# end
--# property-start BaseTypeDecl.is_array_or_rec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5993
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Or_Rec
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3158 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3155 '<Not at ast.py:5995>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5995
--# expr-start 3154 '<IsNull at ast.py:5995>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5995
Is_Null := Self = null; 
--# expr-done 3154
Not_Val := not (Is_Null); 
--# expr-done 3155
if Not_Val then
--# expr-start 3156 '<FieldAccess for is_array at ast.py:5995>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5995
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3156
if Fld then
   If_Result := True;
else
   --# expr-start 3157 '<FieldAccess for is_record_type at ast.py:5995>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5995
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3157
   If_Result := Fld_1;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 3158
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Array_Or_Rec;
--# end
--# property-start BaseTypeDecl.is_inherited_primitive /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5997
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Inherited_Primitive
  (Node : Bare_Base_Type_Decl
      ; P : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind p P
   Property_Result : Boolean;
      Fld : Bare_Base_Type_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3165 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3164 '<Not at ast.py:6003>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6003
--# expr-start 3163 '<Eq at ast.py:6003>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6003
--# expr-start 3159 '<FieldAccess for node at ast.py:6003>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6003
Fld := Ent.Node;
--# expr-done 3159
      Cast_Result := Fld;
--# expr-start 3162 '<FieldAccess for primitive at ast.py:6003>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6003
--# expr-start 3161 '<FieldAccess for md at ast.py:6003>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6003
--# expr-start 3160 '<FieldAccess for info at ast.py:6003>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6003
Fld_1 := P.Info;
--# expr-done 3160
Fld_2 := Fld_1.Md;
--# expr-done 3161
Fld_3 := Fld_2.Primitive;
--# expr-done 3162
Is_Equal := Cast_Result = Fld_3; 
--# expr-done 3163
Not_Val := not (Is_Equal); 
--# expr-done 3164
Let_Result := Not_Val; 
--# end
--# expr-done 3165
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Inherited_Primitive;
--# end
--# property-start BaseTypeDecl.get_record_representation_clause /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6005
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Record_Representation_Clause
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Record_Rep_Clause
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Record_Rep_Clause;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Record_Rep_Clause;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Record_Rep_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Item_125 : Internal_Entity;
Bare_Item_125 : Bare_Ada_Node;
Find_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Record_Rep_Clause;
Let_Result : Internal_Entity_Record_Rep_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3175 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3174 '<Cast to RecordRepClause.entity at ast.py:6016>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6016
--# expr-start 3173 '<Find at ast.py:6012>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6012
   --# expr-start 3168 '<as_entity at ast.py:6012>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6012
--# expr-start 3166 '<FieldAccess for declarative_scope at ast.py:6012>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6012
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3166
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3167 '<FieldAccess for decls at ast.py:6012>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6012
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 3167
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 3168
Fld_2 := If_Result.Node;
      Find_Result := No_Entity;
      if Fld_2 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_125 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_125 := Untyped_Item_125; 
Is_Null_1 := Bare_Item_125 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_125); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_125 := If_Result_1; 
   --# scope-start
               --# bind d Item_125
--# expr-start 3169 '<Cast to RecordRepClause at ast.py:6013>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6013
Cast_Expr := Item_125; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Record_Rep_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Record_Rep_Clause;
   end if;
--# expr-done 3169
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Record_Rep_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 3172 '<Eq at ast.py:6014>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6014
--# expr-start 3171 '<FieldAccess for referenced_decl at ast.py:6014>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6014
--# expr-start 3170 '<FieldAccess for name at ast.py:6014>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6014
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Record_Rep_Clause_F_Name, Info => Var_Expr_1.Info);
--# expr-done 3170
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 3171
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 3172
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
      if Result_Var_1 then
         Find_Result := Item_125; 
      end if;
   --# end
               exit when Result_Var_1;
         end loop;
      end;
      end if;
--# expr-done 3173
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Record_Rep_Clause
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_3 := No_Entity_Record_Rep_Clause;
   end if;
--# expr-done 3174
Let_Result := Cast_Result_3; 
--# end
--# expr-done 3175
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Get_Record_Representation_Clause;
--# end
--# property-start BaseTypeDecl.get_enum_representation_clause /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6018
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Enum_Representation_Clause
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Rep_Clause
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Enum_Rep_Clause;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Enum_Rep_Clause;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Enum_Rep_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Item_126 : Internal_Entity;
Bare_Item_126 : Bare_Ada_Node;
Find_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Enum_Rep_Clause;
Let_Result : Internal_Entity_Enum_Rep_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3185 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3184 '<Cast to EnumRepClause.entity at ast.py:6029>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6029
--# expr-start 3183 '<Find at ast.py:6025>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6025
   --# expr-start 3178 '<as_entity at ast.py:6025>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6025
--# expr-start 3176 '<FieldAccess for declarative_scope at ast.py:6025>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6025
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3176
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3177 '<FieldAccess for decls at ast.py:6025>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6025
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 3177
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 3178
Fld_2 := If_Result.Node;
      Find_Result := No_Entity;
      if Fld_2 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_126 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_126 := Untyped_Item_126; 
Is_Null_1 := Bare_Item_126 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_126); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_126 := If_Result_1; 
   --# scope-start
               --# bind d Item_126
--# expr-start 3179 '<Cast to EnumRepClause at ast.py:6026>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6026
Cast_Expr := Item_126; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Enum_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Rep_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Enum_Rep_Clause;
   end if;
--# expr-done 3179
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Enum_Rep_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 3182 '<Eq at ast.py:6027>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6027
--# expr-start 3181 '<FieldAccess for referenced_decl at ast.py:6027>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6027
--# expr-start 3180 '<FieldAccess for type_name at ast.py:6027>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6027
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Enum_Rep_Clause_F_Type_Name, Info => Var_Expr_1.Info);
--# expr-done 3180
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 3181
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 3182
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
      if Result_Var_1 then
         Find_Result := Item_126; 
      end if;
   --# end
               exit when Result_Var_1;
         end loop;
      end;
      end if;
--# expr-done 3183
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Enum_Rep_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Enum_Rep_Clause
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_3 := No_Entity_Enum_Rep_Clause;
   end if;
--# expr-done 3184
Let_Result := Cast_Result_3; 
--# end
--# expr-done 3185
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Get_Enum_Representation_Clause;
--# end
--# property-start '[dispatcher]BaseTypeDecl.primitives_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.primitives_env
                  Property_Result := Base_Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.primitives_env
                  Property_Result := Base_Subtype_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.primitives_env
                  Property_Result := Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Primitives_Env;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_record_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Record_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_record_type
                  Property_Result := Base_Type_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_record_type
                  Property_Result := Base_Subtype_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Record_Type;
--# end
--# property-start BaseTypeDecl.is_array_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6045
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3187 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3186 '<FieldAccess for is_array at ast.py:6050>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6050
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3186
Let_Result := Fld; 
--# end
--# expr-done 3187
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Array_Type;
--# end
--# property-start BaseTypeDecl.find_derived_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6052
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Derived_Types
  (Node : Bare_Base_Type_Decl
      ; Root : Internal_Entity
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_436 with Inline_Always;
            procedure Finalizer_Scope_2285 with Inline_Always;
            procedure Finalizer_Scope_2287 with Inline_Always;
            procedure Finalizer_Scope_2288 with Inline_Always;
            procedure Finalizer_Scope_2289 with Inline_Always;
      Var_Expr : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Type_Decl;
Var_Expr_2 : Boolean;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Map_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Type_Decl;
Fld_3 : Boolean;
Singleton : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_436 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_436;
            procedure Finalizer_Scope_2285 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2285;
            procedure Finalizer_Scope_2287 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2287;
            procedure Finalizer_Scope_2288 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2288;
            procedure Finalizer_Scope_2289 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2289;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3199 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3198 '<Concat at ast.py:6062>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6062
--# expr-start 3188 '<children at ast.py:6059>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6059
      if Root.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 3188
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 3195 '<Mapcat at ast.py:6061>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6061
   --# expr-start 3193 '<Filter at ast.py:6060>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6060
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr;
      begin
         for Item_128 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_128
         --# expr-start 3192 '<Not at ast.py:6060>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6060
--# expr-start 3189 '<IsNull at ast.py:6060>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6060
Is_Null := Item_128.Node = null; 
--# expr-done 3189
if Is_Null then
   If_Result := True;
else
   --# expr-start 3191 '<Eq at ast.py:6060>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6060
--# expr-start 3190 '<FieldAccess for node at ast.py:6060>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6060
Fld_1 := Item_128.Node;
--# expr-done 3190
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Origin; 
--# expr-done 3191
   If_Result := Is_Equal;
end if;
Not_Val := not (If_Result); 
--# expr-done 3192
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_128;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3193
      declare
         Map_Result_1_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_127 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_127
      --# expr-start 3194 '<FieldAccess for find_derived_types at ast.py:6061>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6061
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.find_derived_types
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Find_Derived_Types (Node => Ent.Node, Root => Item_127, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3194
         for Item_To_Append of
               Fld_2.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2287;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 3195
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2285;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 3196 '<Cast to TypeDecl at ast.py:6062>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6062
Cast_Expr := Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 3196
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Type_Decl then
   --# scope-start
   --# bind type_decl Var_Expr_1
--# expr-start 3197 '<FieldAccess for is_derived_type at ast.py:6063>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6063
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr_1.Node, Other_Type => Ent, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3197
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
                Singleton := Create_Internal_Entity_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr_1;
Scope_Result_1 := Singleton;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2289;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2288;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var, Result_Var_2); 
--# expr-done 3198
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3199
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_436;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_436;
                     Finalizer_Scope_2285;
                     Finalizer_Scope_2287;
                     Finalizer_Scope_2288;
                     Finalizer_Scope_2289;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Find_Derived_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_task_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Task_Type
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Protected_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_task_type
                  Property_Result := Base_Type_Decl_P_Is_Task_Type
                    (Self
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.is_task_type
                  Property_Result := Task_Type_Decl_P_Is_Task_Type
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Task_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_real_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Real_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_real_type
                  Property_Result := Base_Type_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_real_type
                  Property_Result := Base_Subtype_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_real_type
                  Property_Result := Type_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Real_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_float_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Float_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_float_type
                  Property_Result := Base_Type_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_float_type
                  Property_Result := Base_Subtype_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_float_type
                  Property_Result := Type_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Float_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_fixed_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_fixed_point
                  Property_Result := Base_Type_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_fixed_point
                  Property_Result := Base_Subtype_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_fixed_point
                  Property_Result := Type_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseTypeDecl.is_scalar_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6082
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Scalar_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Boolean;
If_Result_2 : Boolean;
Fld_4 : Boolean;
If_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 3200 '<FieldAccess for is_int_type at ast.py:6083>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6083
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3200
if Fld then
   If_Result := True;
else
   --# expr-start 3201 '<FieldAccess for is_real_type at ast.py:6083>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6083
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3201
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 3202 '<FieldAccess for is_float_type at ast.py:6083>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6083
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3202
   If_Result_1 := Fld_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 3203 '<FieldAccess for is_fixed_point at ast.py:6084>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6084
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3203
   If_Result_2 := Fld_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 3204 '<FieldAccess for is_enum_type at ast.py:6084>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6084
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3204
   If_Result_3 := Fld_4;
end if;
         Property_Result := If_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Scalar_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_enum_type
                  Property_Result := Base_Type_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_enum_type
                  Property_Result := Base_Subtype_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_enum_type
                  Property_Result := Type_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Enum_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_classwide' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Classwide
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_classwide
                  Property_Result := Base_Type_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_classwide
                  Property_Result := Base_Subtype_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_classwide
                  Property_Result := Classwide_Type_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Classwide;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_access_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_access_type
                  Property_Result := Base_Type_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_access_type
                  Property_Result := Base_Subtype_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_access_type
                  Property_Result := Type_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Access_Type;
--# end
--# property-start BaseTypeDecl.is_implicit_deref /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6103
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Implicit_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Access_Def;
Fld : Internal_Entity_Access_Def;
Is_A : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3205 '<FieldAccess for access_def at ast.py:6104>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6104
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3205
Var_Expr := Fld;
if Var_Expr /= No_Entity_Access_Def then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 3207 '<Not at ast.py:6104>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6104
--# expr-start 3206 '<IsA AccessToSubpDef at ast.py:6104>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6104
Is_A := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 3206
Not_Val := not (Is_A); 
--# expr-done 3207
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result := True;
else
   --# expr-start 3210 '<Not at ast.py:6105>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6105
--# expr-start 3209 '<IsNull at ast.py:6105>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6105
--# expr-start 3208 '<FieldAccess for get_imp_deref at ast.py:6105>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6105
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3208
Is_Null := Fld_1.Node = null; 
--# expr-done 3209
Not_Val_1 := not (Is_Null); 
--# expr-done 3210
   If_Result := Not_Val_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Implicit_Deref;
--# end
--# property-start '[dispatcher]BaseTypeDecl.has_ud_indexing' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.has_ud_indexing
                  Property_Result := Base_Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.has_ud_indexing
                  Property_Result := Base_Subtype_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.has_ud_indexing
                  Property_Result := Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start '[dispatcher]BaseTypeDecl.constant_indexing_fns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.constant_indexing_fns
                  Property_Result := Base_Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.constant_indexing_fns
                  Property_Result := Base_Subtype_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.constant_indexing_fns
                  Property_Result := Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start '[dispatcher]BaseTypeDecl.variable_indexing_fns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.variable_indexing_fns
                  Property_Result := Base_Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.variable_indexing_fns
                  Property_Result := Base_Subtype_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.variable_indexing_fns
                  Property_Result := Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start '[dispatcher]BaseTypeDecl.get_imp_deref' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.get_imp_deref
                  Property_Result := Base_Type_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.get_imp_deref
                  Property_Result := Type_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start BaseTypeDecl.specific_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6134
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Specific_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Classwide_Type_Decl;
Match_1 : Internal_Entity_Base_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Bt : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Classwide_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Subtype_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_4 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3219 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 3218 '<Match at ast.py:6146>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6146
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Classwide_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind cw Match_0
--# expr-start 3212 '<FieldAccess for specific_type at ast.py:6149>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6149
--# expr-start 3211 '<FieldAccess for type_decl at ast.py:6149>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6149
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 3211
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3212
Let_Result_1 := Fld_1; 
--# end
Scope_Result := Let_Result_1;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind bt Match_1
--# expr-start 3217 '<Let at ast.py:6150>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6150
--# scope-start
--# expr-start 3213 '<FieldAccess for base_subtype at ast.py:6151>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6151
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Match_1.Node, Origin => No_Bare_Ada_Node, E_Info => Match_1.Info);
--# end
--# expr-done 3213
Bt := Fld_2; 
--# bind bt Bt
--# expr-start 3216 '<If at ast.py:6154>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6154
--# expr-start 3214 '<Eq at ast.py:6154>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6154
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Bt; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 3214
if Is_Equiv then
   If_Result := Ent;
else
   --# expr-start 3215 '<FieldAccess for specific_type at ast.py:6154>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6154
      if Bt.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Bt.Node, E_Info => Bt.Info);
--# end
--# expr-done 3215
   If_Result := Fld_3;
end if;
--# expr-done 3216
Let_Result := If_Result; 
--# end
--# expr-done 3217
Let_Result_2 := Let_Result; 
--# end
Scope_Result_1 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_3 := Ent; 
--# end
Scope_Result_2 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3218
Let_Result_4 := Match_Result; 
--# end
--# expr-done 3219
         Property_Result := Let_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Specific_Type;
--# end
--# property-start BaseTypeDecl.derefed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6159
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Derefed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Fld_1 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3225 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3224 '<If at ast.py:6166>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6166
--# expr-start 3220 '<IsNull at ast.py:6167>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6167
Is_Null := Ent.Node = null; 
--# expr-done 3220
if Is_Null then
   If_Result := True;
else
   --# expr-start 3222 '<IsNull at ast.py:6167>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6167
--# expr-start 3221 '<FieldAccess for get_imp_deref at ast.py:6167>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6167
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3221
Is_Null_1 := Fld_1.Node = null; 
--# expr-done 3222
   If_Result := Is_Null_1;
end if;
if If_Result then
   If_Result_1 := Ent;
else
   --# expr-start 3223 '<FieldAccess for accessed_type at ast.py:6169>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6169
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3223
   If_Result_1 := Fld;
end if;
--# expr-done 3224
Let_Result := If_Result_1; 
--# end
--# expr-done 3225
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Derefed_Type;
--# end
--# property-start BaseTypeDecl.derefed_base_subtype /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6172
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Derefed_Base_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3234 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3233 '<If at ast.py:6180>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6180
--# expr-start 3226 '<IsNull at ast.py:6181>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6181
Is_Null_1 := Ent.Node = null; 
--# expr-done 3226
if Is_Null_1 then
   If_Result_1 := Ent;
else
   --# expr-start 3232 '<FieldAccess for base_subtype at ast.py:6187>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6187
--# expr-start 3231 '<If at ast.py:6183>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6183
--# expr-start 3229 '<Not at ast.py:6184>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6184
--# expr-start 3228 '<IsNull at ast.py:6184>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6184
--# expr-start 3227 '<FieldAccess for get_imp_deref at ast.py:6184>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6184
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3227
Is_Null := Fld_1.Node = null; 
--# expr-done 3228
Not_Val := not (Is_Null); 
--# expr-done 3229
if Not_Val then
   --# expr-start 3230 '<FieldAccess for accessed_type at ast.py:6185>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6185
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3230
   If_Result := Fld;
else
   If_Result := Ent;
end if;
--# expr-done 3231
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 3232
   If_Result_1 := Fld_2;
end if;
--# expr-done 3233
Let_Result := If_Result_1; 
--# end
--# expr-done 3234
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Derefed_Base_Subtype;
--# end
--# property-start BaseTypeDecl.one_non_universal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6190
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_One_Non_Universal
  (Node : Bare_Base_Type_Decl
      ; Second : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind second Second
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
If_Result : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3241 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3236 '<Not at ast.py:6193>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6193
--# expr-start 3235 '<IsNull at ast.py:6193>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6193
Is_Null := Ent.Node = null; 
--# expr-done 3235
Not_Val := not (Is_Null); 
--# expr-done 3236
if Not_Val then
   --# expr-start 3237 '<FieldAccess for is_not_universal_type at ast.py:6193>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6193
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_not_universal_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Is_Not_Universal_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3237
   If_Result := Fld;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 3239 '<Not at ast.py:6194>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6194
--# expr-start 3238 '<IsNull at ast.py:6194>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6194
Is_Null_1 := Second.Node = null; 
--# expr-done 3238
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3239
if Not_Val_1 then
   --# expr-start 3240 '<FieldAccess for is_not_universal_type at ast.py:6194>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6194
      if Second.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_not_universal_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Not_Universal_Type (Node => Second.Node, E_Info => Second.Info);
--# end
--# expr-done 3240
   If_Result_1 := Fld_1;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 3241
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_One_Non_Universal;
--# end
--# property-start BaseTypeDecl.non_universal_base_subtype /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6197
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Non_Universal_Base_Subtype
  (Node : Bare_Base_Type_Decl
      ; Second : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind second Second
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Boolean;
Block_Var_1 : Boolean;
First_St : Internal_Entity_Base_Type_Decl;
Second_St : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
If_Result : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
If_Result_4 : Internal_Entity_Base_Type_Decl;
If_Result_5 : Internal_Entity_Base_Type_Decl;
If_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3256 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 3243 '<Not at ast.py:6200>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6200
--# expr-start 3242 '<IsNull at ast.py:6200>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6200
Is_Null := Ent.Node = null; 
--# expr-done 3242
Not_Val := not (Is_Null); 
--# expr-done 3243
if Not_Val then
   --# expr-start 3244 '<FieldAccess for is_not_universal_type at ast.py:6200>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6200
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_not_universal_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Is_Not_Universal_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3244
   If_Result := Fld;
else
   If_Result := False;
end if;
Block_Var := If_Result; 
--# bind first_ok Block_Var
--# expr-start 3246 '<Not at ast.py:6201>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6201
--# expr-start 3245 '<IsNull at ast.py:6201>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6201
Is_Null_1 := Second.Node = null; 
--# expr-done 3245
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3246
if Not_Val_1 then
   --# expr-start 3247 '<FieldAccess for is_not_universal_type at ast.py:6201>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6201
      if Second.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_not_universal_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Not_Universal_Type (Node => Second.Node, E_Info => Second.Info);
--# end
--# expr-done 3247
   If_Result_1 := Fld_1;
else
   If_Result_1 := False;
end if;
Block_Var_1 := If_Result_1; 
--# bind second_ok Block_Var_1
--# expr-start 3255 '<Cond at ast.py:6202>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6202
if Block_Var then
   If_Result_2 := Block_Var_1;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 3252 '<Let at ast.py:6204>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6204
--# scope-start
--# expr-start 3248 '<FieldAccess for derefed_base_subtype at ast.py:6206>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6206
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3248
First_St := Fld_2; 
--# bind first_st First_St
--# expr-start 3249 '<FieldAccess for derefed_base_subtype at ast.py:6207>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6207
      if Second.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Second.Node, Origin => Origin, E_Info => Second.Info);
--# end
--# expr-done 3249
Second_St := Fld_3; 
--# bind second_st Second_St
--# expr-start 3251 '<If at ast.py:6208>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6208
--# expr-start 3250 '<FieldAccess for matching_formal_type at ast.py:6209>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6209
      if Second_St.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Second_St.Node, Formal_Type => First_St, Origin => Origin, E_Info => Second_St.Info);
--# end
--# expr-done 3250
if Fld_4 then
   If_Result_3 := First_St;
else
   If_Result_3 := Second_St;
end if;
--# expr-done 3251
Let_Result := If_Result_3; 
--# end
--# expr-done 3252
   If_Result_6 := Let_Result;
else
if Block_Var then
   --# expr-start 3253 '<FieldAccess for derefed_base_subtype at ast.py:6216>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6216
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3253
   If_Result_5 := Fld_5;
else
if Block_Var_1 then
   --# expr-start 3254 '<FieldAccess for derefed_base_subtype at ast.py:6219>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6219
      if Second.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Second.Node, Origin => Origin, E_Info => Second.Info);
--# end
--# expr-done 3254
   If_Result_4 := Fld_6;
else
   If_Result_4 := No_Entity_Base_Type_Decl;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
--# expr-done 3255
Let_Result_1 := If_Result_6; 
--# end
--# expr-done 3256
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Non_Universal_Base_Subtype;
--# end
--# property-start BaseTypeDecl.array_concat_result_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6224
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Concat_Result_Type
  (Node : Bare_Base_Type_Decl
      ; Other : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Is_Null : Boolean;
Is_Null_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result_2 : Boolean;
Fld_7 : Internal_Entity_Base_Type_Decl;
If_Result_3 : Internal_Entity_Base_Type_Decl;
If_Result_4 : Internal_Entity_Base_Type_Decl;
If_Result_5 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3268 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3267 '<Cond at ast.py:6232>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6232
--# expr-start 3257 '<IsNull at ast.py:6233>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6233
Is_Null := Ent.Node = null; 
--# expr-done 3257
if Is_Null then
   If_Result := True;
else
   --# expr-start 3258 '<IsNull at ast.py:6233>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6233
Is_Null_1 := Other.Node = null; 
--# expr-done 3258
   If_Result := Is_Null_1;
end if;
if If_Result then
   If_Result_5 := No_Entity_Base_Type_Decl;
else
--# expr-start 3259 '<FieldAccess for matching_formal_type at ast.py:6236>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6236
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Other.Node, Formal_Type => Ent, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3259
if Fld then
   If_Result_1 := True;
else
   --# expr-start 3261 '<FieldAccess for matching_formal_type at ast.py:6237>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6237
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3260 '<FieldAccess for comp_type at ast.py:6237>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6237
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3260
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Other.Node, Formal_Type => Fld_1, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3261
   If_Result_1 := Fld_2;
end if;
if If_Result_1 then
   --# expr-start 3262 '<FieldAccess for derefed_base_subtype at ast.py:6238>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6238
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3262
   If_Result_4 := Fld_3;
else
--# expr-start 3263 '<FieldAccess for matching_formal_type at ast.py:6240>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6240
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Ent.Node, Formal_Type => Other, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3263
if Fld_4 then
   If_Result_2 := True;
else
   --# expr-start 3265 '<FieldAccess for matching_formal_type at ast.py:6241>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6241
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3264 '<FieldAccess for comp_type at ast.py:6241>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6241
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3264
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Ent.Node, Formal_Type => Fld_5, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3265
   If_Result_2 := Fld_6;
end if;
if If_Result_2 then
   --# expr-start 3266 '<FieldAccess for derefed_base_subtype at ast.py:6242>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6242
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3266
   If_Result_3 := Fld_7;
else
   If_Result_3 := No_Entity_Base_Type_Decl;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
--# expr-done 3267
Let_Result := If_Result_5; 
--# end
--# expr-done 3268
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Array_Concat_Result_Type;
--# end
--# property-start BaseTypeDecl.expected_array_concat_operand_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6247
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Expected_Array_Concat_Operand_Type
  (Node : Bare_Base_Type_Decl
      ; Operand_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind operand_type Operand_Type
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Is_Null : Boolean;
Is_Null_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3276 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3275 '<Cond at ast.py:6258>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6258
--# expr-start 3269 '<IsNull at ast.py:6259>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6259
Is_Null := Ent.Node = null; 
--# expr-done 3269
if Is_Null then
   If_Result := True;
else
   --# expr-start 3270 '<IsNull at ast.py:6259>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6259
Is_Null_1 := Operand_Type.Node = null; 
--# expr-done 3270
   If_Result := Is_Null_1;
end if;
if If_Result then
   If_Result_3 := No_Entity_Base_Type_Decl;
else
--# expr-start 3271 '<FieldAccess for matching_formal_type at ast.py:6262>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6262
      if Operand_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Operand_Type.Node, Formal_Type => Ent, Origin => Origin, E_Info => Operand_Type.Info);
--# end
--# expr-done 3271
if Fld then
   If_Result_1 := True;
else
   --# expr-start 3273 '<FieldAccess for matching_formal_type at ast.py:6263>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6263
      if Operand_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3272 '<FieldAccess for comp_type at ast.py:6263>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6263
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3272
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Operand_Type.Node, Formal_Type => Fld_1, Origin => Origin, E_Info => Operand_Type.Info);
--# end
--# expr-done 3273
   If_Result_1 := Fld_2;
end if;
if If_Result_1 then
   --# expr-start 3274 '<FieldAccess for derefed_base_subtype at ast.py:6264>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6264
      if Operand_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Operand_Type.Node, Origin => Origin, E_Info => Operand_Type.Info);
--# end
--# expr-done 3274
   If_Result_2 := Fld_3;
else
   If_Result_2 := No_Entity_Base_Type_Decl;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 3275
Let_Result := If_Result_3; 
--# end
--# expr-done 3276
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Expected_Array_Concat_Operand_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.access_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Access_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.access_def
                  Property_Result := Base_Type_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.access_def
                  Property_Result := Base_Subtype_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.access_def
                  Property_Result := Type_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Access_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_char_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_char_type
                  Property_Result := Base_Type_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_char_type
                  Property_Result := Base_Subtype_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_char_type
                  Property_Result := Type_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Char_Type;
--# end
--# property-start BaseTypeDecl.is_non_null_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6278
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Non_Null_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3278 '<Not at ast.py:6279>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6279
--# expr-start 3277 '<IsNull at ast.py:6279>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6279
Is_Null := Self = null; 
--# expr-done 3277
Not_Val := not (Is_Null); 
--# expr-done 3278
if Not_Val then
   --# expr-start 3279 '<FieldAccess for is_char_type at ast.py:6279>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6279
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3279
   If_Result := Fld;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Non_Null_Char_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.classwide_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Classwide_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.classwide_type
                  Property_Result := Base_Type_Decl_P_Classwide_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.classwide_type
                  Property_Result := Classwide_Type_Decl_P_Classwide_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Classwide_Type;
--# end
--# property-start BaseTypeDecl.scalar_base_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6294
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
      Fld : Bare_Discrete_Base_Subtype_Decl;
As_Entity : Internal_Entity_Discrete_Base_Subtype_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3281 '<as_entity at ast.py:6294>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6294
--# expr-start 3280 '<FieldAccess for scalar_base_subtype_node at ast.py:6294>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6294
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype_Node (Node => Self);
--# end
--# expr-done 3280
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Discrete_Base_Subtype_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 3281
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Scalar_Base_Type;
--# end
--# property-start BaseTypeDecl.is_universal_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6296
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Universal_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3288 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3283 '<Not at ast.py:6305>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6305
--# expr-start 3282 '<IsNull at ast.py:6305>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6305
Is_Null := Ent.Node = null; 
--# expr-done 3282
Not_Val := not (Is_Null); 
--# expr-done 3283
if Not_Val then
--# expr-start 3285 '<Eq at ast.py:6306>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6306
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 3284 '<FieldAccess for universal_int_type at ast.py:6306>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6306
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 3284
Is_Equiv := Equivalent (Cast_Result, Fld); 
--# expr-done 3285
if Is_Equiv then
   If_Result := True;
else
   --# expr-start 3287 '<Eq at ast.py:6307>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6307
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 3286 '<FieldAccess for universal_real_type at ast.py:6307>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6307
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 3286
Is_Equiv_1 := Equivalent (Cast_Result_1, Fld_1); 
--# expr-done 3287
   If_Result := Is_Equiv_1;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 3288
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Universal_Type;
--# end
--# property-start BaseTypeDecl.is_not_universal_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6310
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Not_Universal_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3293 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3290 '<Not at ast.py:6319>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6319
--# expr-start 3289 '<IsNull at ast.py:6319>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6319
Is_Null := Ent.Node = null; 
--# expr-done 3289
Not_Val := not (Is_Null); 
--# expr-done 3290
if Not_Val then
   --# expr-start 3292 '<Not at ast.py:6319>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6319
--# expr-start 3291 '<FieldAccess for is_universal_type at ast.py:6319>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6319
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_universal_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Is_Universal_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3291
Not_Val_1 := not (Fld); 
--# expr-done 3292
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3293
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Not_Universal_Type;
--# end
--# property-start BaseTypeDecl.is_access_type_predicate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6321
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_Type_Predicate
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3295 '<Block at ???>' Let_Result None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3294 '<FieldAccess for is_access_type at ast.py:6328>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6328
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3294
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3295
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Access_Type_Predicate;
--# end
--# property-start BaseTypeDecl.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6330
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Ndims
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
      Let_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3296 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := 0; 
--# end
--# expr-done 3296
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Array_Ndims;
--# end
--# property-start '[dispatcher]BaseTypeDecl.discrete_range' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Discrete_Range
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.discrete_range
                  Property_Result := Base_Type_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.discrete_range
                  Property_Result := Subtype_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.discrete_range
                  Property_Result := Type_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Discrete_Range;
--# end
--# property-start BaseTypeDecl.static_predicate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6341
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Static_Predicate
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Expr;
      Var_Expr : Internal_Entity_Expr;
Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Aspect;
Fld_3 : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3303 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3299 '<FieldAccess for value at ast.py:6348>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6348
--# expr-start 3298 '<FieldAccess for get_aspect at ast.py:6348>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6348
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3297 '<Symbol Static_Predicate at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Static_Predicate); 
--# expr-done 3297
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3298
Fld_1 := Fld.Value;
--# expr-done 3299
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 3302 '<FieldAccess for value at ast.py:6349>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6349
--# expr-start 3301 '<FieldAccess for get_aspect at ast.py:6349>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6349
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3300 '<Symbol Predicate at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate); 
--# expr-done 3300
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3301
Fld_3 := Fld_2.Value;
--# expr-done 3302
   Result_Var := Fld_3;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3303
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Static_Predicate;
--# end
--# property-start BaseTypeDecl.satisfies_type_predicates /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6352
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Satisfies_Type_Predicates
  (Node : Bare_Base_Type_Decl
      ; Value : Big_Integer_Type
      ; Imprecise_Fallback : Boolean
         := False
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind value Value
   --# bind imprecise_fallback Imprecise_Fallback
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_468 with Inline_Always;
            procedure Finalizer_Scope_2296 with Inline_Always;
      Block_Var : Big_Integer_Type;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subtype_Decl;
Big_Int : Big_Integer_Type;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
New_Struct : Internal_Substitution := No_Substitution;
Singleton : Internal_Substitution_Array_Access;
Fld_1 : Big_Integer_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Subtype_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_4 : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_5 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_468 is
            begin
                     Dec_Ref (Big_Int);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_468;
            procedure Finalizer_Scope_2296 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2296;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3319 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3304 '<BigInteger 1 at ast.py:6362>' Big_Int /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6362
Big_Int := Create_Big_Integer ("1"); 
--# expr-done 3304
Block_Var := Big_Int; Inc_Ref (Block_Var);
--# bind true_val Block_Var
--# expr-start 3305 '<FieldAccess for static_predicate at ast.py:6365>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6365
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.static_predicate
Fld := Libadalang.Implementation.Base_Type_Decl_P_Static_Predicate (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3305
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind pred Var_Expr
--# expr-start 3308 '<Eq at ast.py:6366>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6366
--# expr-start 3307 '<FieldAccess for eval_as_int_in_env at ast.py:6366>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6366
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3306 '<New Substitution at ast.py:6367>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6367
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Inc_Ref (Value);
New_Struct := (From_Decl => Cast_Result, To_Value => Value, Value_Type => Ent); 
--# expr-done 3306
                Singleton := Create_Internal_Substitution_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
                Inc_Ref (Singleton.Items (1));
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Eval_As_Int_In_Env (Node => Var_Expr.Node, Env => Singleton, E_Info => Var_Expr.Info);
--# expr-done 3307
Is_Equal := Equivalent (Block_Var, Fld_1); 
--# expr-done 3308
Scope_Result := Is_Equal;
   --# end
      Finalizer_Scope_2296;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Block_Var_1 := Result_Var; 
--# bind satisfies_own_predicate Block_Var_1
--# expr-start 3309 '<Cast to BaseSubtypeDecl at ast.py:6377>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6377
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Subtype_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Subtype_Decl;
   end if;
--# expr-done 3309
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Base_Subtype_Decl then
   --# scope-start
--# expr-start 3310 '<FieldAccess for get_type at ast.py:6377>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6377
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3310
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_2 := Result_Var_1; 
--# bind from_type Block_Var_2
--# expr-start 3311 '<FieldAccess for base_type at ast.py:6378>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6378
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3311
Block_Var_3 := Fld_3; 
--# bind base_type Block_Var_3
if Block_Var_1 then
   --# expr-start 3318 '<Cond at ast.py:6380>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6380
--# expr-start 3313 '<Not at ast.py:6381>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6381
--# expr-start 3312 '<IsNull at ast.py:6381>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6381
Is_Null := Block_Var_2.Node = null; 
--# expr-done 3312
Not_Val := not (Is_Null); 
--# expr-done 3313
if Not_Val then
   --# expr-start 3314 '<FieldAccess for satisfies_type_predicates at ast.py:6382>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6382
      if Block_Var_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Block_Var_2.Node, Value => Value, Imprecise_Fallback => Imprecise_Fallback, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 3314
   If_Result_1 := Fld_4;
else
--# expr-start 3316 '<Not at ast.py:6384>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6384
--# expr-start 3315 '<IsNull at ast.py:6384>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6384
Is_Null_1 := Block_Var_3.Node = null; 
--# expr-done 3315
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3316
if Not_Val_1 then
   --# expr-start 3317 '<FieldAccess for satisfies_type_predicates at ast.py:6385>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6385
      if Block_Var_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Block_Var_3.Node, Value => Value, Imprecise_Fallback => Imprecise_Fallback, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 3317
   If_Result := Fld_5;
else
   If_Result := True;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 3318
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 3319
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_468;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_468;
                     Finalizer_Scope_2296;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Satisfies_Type_Predicates;
--# end
--# property-start BaseTypeDecl.is_iterator_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6390
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Iterator_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_469 with Inline_Always;
      Block_Var : Bare_Basic_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Classwide_Type_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Classwide_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Fld_5 : Internal_Entity_Basic_Decl;
Sym_2 : Symbol_Type;
Fld_6 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_469 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_469;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Is_Iterator_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3333 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3323 '<FieldAccess for get_unit_root_decl at ast.py:6392>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6392
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3322 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 3320 '<Symbol Ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 3320
--# expr-start 3321 '<Symbol Iterator_Interfaces at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator_Interfaces); 
--# expr-done 3321
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 3322
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 3323
Block_Var := Fld; 
--# bind iifcs Block_Var
--# expr-start 3324 '<Cast to ClasswideTypeDecl at ast.py:6395>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6395
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 3324
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Classwide_Type_Decl then
   --# scope-start
   --# bind cw Var_Expr
--# expr-start 3325 '<FieldAccess for type_decl at ast.py:6396>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6396
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld_1 := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3325
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Block_Var_1 := Result_Var; 
--# bind typ Block_Var_1
--# expr-start 3329 '<Eq at ast.py:6399>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6399
--# expr-start 3328 '<FieldAccess for node at ast.py:6399>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6399
--# expr-start 3327 '<FieldAccess for semantic_parent at ast.py:6399>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6399
--# expr-start 3326 '<FieldAccess for semantic_parent at ast.py:6399>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6399
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3326
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_3 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 3327
Fld_4 := Fld_3.Node;
--# expr-done 3328
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Fld_4 = Cast_Result_1; 
--# expr-done 3329
if Is_Equal then
   If_Result := True;
else
   --# expr-start 3332 '<FieldAccess for has_aspect at ast.py:6400>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6400
--# expr-start 3330 '<FieldAccess for canonical_part at ast.py:6400>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6400
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 3330
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3331 '<Symbol Iterable at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 3331
--# property-call-start BasicDecl.has_aspect
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Fld_5.Node, Name => Sym_2, Imprecise_Fallback => False, E_Info => Fld_5.Info);
--# end
--# expr-done 3332
   If_Result := Fld_6;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3333
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_469;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_469;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Is_Iterator_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_discrete_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_discrete_type
                  Property_Result := Base_Type_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_discrete_type
                  Property_Result := Base_Subtype_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_discrete_type
                  Property_Result := Type_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseTypeDecl.is_not_root_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6410
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Not_Root_Int_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3339 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3335 '<Not at ast.py:6412>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6412
--# expr-start 3334 '<IsNull at ast.py:6412>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6412
Is_Null := Self = null; 
--# expr-done 3334
Not_Val := not (Is_Null); 
--# expr-done 3335
if Not_Val then
   --# expr-start 3338 '<Not at ast.py:6412>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6412
--# expr-start 3337 '<Eq at ast.py:6412>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6412
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 3336 '<FieldAccess for root_int_type at ast.py:6412>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6412
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Root_Int_Type (Node => Self);
--# end
--# expr-done 3336
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 3337
Not_Val_1 := not (Is_Equiv); 
--# expr-done 3338
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3339
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Not_Root_Int_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_int_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Int_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_int_type
                  Property_Result := Base_Type_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_int_type
                  Property_Result := Base_Subtype_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_int_type
                  Property_Result := Type_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Int_Type;
--# end
--# property-start BaseTypeDecl.allows_universal_int /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6419
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Allows_Universal_Int
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld_2 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3346 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3340 '<FieldAccess for is_int_type at ast.py:6426>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6426
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3340
if Fld then
   If_Result := True;
else
--# expr-start 3341 '<FieldAccess for base_subtype at ast.py:6426>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6426
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3341
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 3345 '<Not at ast.py:6429>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6429
--# expr-start 3344 '<IsNull at ast.py:6429>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6429
--# expr-start 3343 '<FieldAccess for get_aspect_spec_expr at ast.py:6429>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6429
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3342 '<Symbol Integer_Literal at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Literal); 
--# expr-done 3342
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Var_Expr.Node, Name => Sym, E_Info => Var_Expr.Info);
--# end
--# expr-done 3343
Is_Null := Fld_2.Node = null; 
--# expr-done 3344
Not_Val := not (Is_Null); 
--# expr-done 3345
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3346
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Allows_Universal_Int;
--# end
--# property-start BaseTypeDecl.allows_universal_real /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6433
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Allows_Universal_Real
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld_2 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3353 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3347 '<FieldAccess for is_real_type at ast.py:6440>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6440
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3347
if Fld then
   If_Result := True;
else
--# expr-start 3348 '<FieldAccess for base_subtype at ast.py:6440>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6440
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3348
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 3352 '<Not at ast.py:6443>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6443
--# expr-start 3351 '<IsNull at ast.py:6443>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6443
--# expr-start 3350 '<FieldAccess for get_aspect_spec_expr at ast.py:6443>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6443
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3349 '<Symbol Real_Literal at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Real_Literal); 
--# expr-done 3349
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Var_Expr.Node, Name => Sym, E_Info => Var_Expr.Info);
--# end
--# expr-done 3350
Is_Null := Fld_2.Node = null; 
--# expr-done 3351
Not_Val := not (Is_Null); 
--# expr-done 3352
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3353
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Allows_Universal_Real;
--# end
--# property-start BaseTypeDecl.allows_string_literal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6447
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Allows_String_Literal
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld_2 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3362 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3355 '<Not at ast.py:6459>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6459
--# expr-start 3354 '<IsNull at ast.py:6459>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6459
Is_Null := Self = null; 
--# expr-done 3354
Not_Val := not (Is_Null); 
--# expr-done 3355
if Not_Val then
--# expr-start 3356 '<FieldAccess for is_str_type at ast.py:6460>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6460
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_str_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Is_Str_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3356
if Fld then
   If_Result := True;
else
--# expr-start 3357 '<FieldAccess for base_subtype at ast.py:6460>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6460
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3357
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 3361 '<Not at ast.py:6464>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6464
--# expr-start 3360 '<IsNull at ast.py:6464>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6464
--# expr-start 3359 '<FieldAccess for get_aspect_spec_expr at ast.py:6464>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6464
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3358 '<Symbol String_Literal at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String_Literal); 
--# expr-done 3358
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Var_Expr.Node, Name => Sym, E_Info => Var_Expr.Info);
--# end
--# expr-done 3359
Is_Null_1 := Fld_2.Node = null; 
--# expr-done 3360
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3361
Scope_Result := Not_Val_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 3362
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Allows_String_Literal;
--# end
--# property-start BaseTypeDecl.is_str_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6469
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Str_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Integer;
Is_Equal : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3367 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3364 '<Eq at ast.py:6474>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6474
--# expr-start 3363 '<FieldAccess for array_ndims at ast.py:6474>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6474
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3363
Is_Equal := Fld = 1; 
--# expr-done 3364
if Is_Equal then
--# expr-start 3365 '<FieldAccess for comp_type at ast.py:6474>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6474
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3365
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3366 '<FieldAccess for is_char_type at ast.py:6474>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6474
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3366
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3367
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Str_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.accessed_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.accessed_type
                  Property_Result := Base_Type_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.accessed_type
                  Property_Result := Base_Subtype_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.accessed_type
                  Property_Result := Type_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Accessed_Type;
--# end
--# property-start BaseTypeDecl.accessed_type_no_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6484
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Accessed_Type_No_Call
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Fld_1 : Internal_Entity_Access_Def;
Is_A : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3373 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3372 '<If at ast.py:6490>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6490
--# expr-start 3368 '<IsNull at ast.py:6491>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6491
Is_Null := Ent.Node = null; 
--# expr-done 3368
if Is_Null then
   If_Result := True;
else
   --# expr-start 3370 '<IsA AccessToSubpDef at ast.py:6491>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6491
--# expr-start 3369 '<FieldAccess for access_def at ast.py:6491>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6491
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3369
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 3370
   If_Result := Is_A;
end if;
if If_Result then
   If_Result_1 := No_Entity_Base_Type_Decl;
else
   --# expr-start 3371 '<FieldAccess for accessed_type at ast.py:6493>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6493
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3371
   If_Result_1 := Fld;
end if;
--# expr-done 3372
Let_Result := If_Result_1; 
--# end
--# expr-done 3373
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Accessed_Type_No_Call;
--# end
--# property-start BaseTypeDecl.final_accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6496
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Final_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; First_Call : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind first_call First_Call
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3377 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3374 '<FieldAccess for accessed_type at ast.py:6508>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6508
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3374
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr
--# expr-start 3375 '<FieldAccess for final_accessed_type at ast.py:6509>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6509
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.final_accessed_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Final_Accessed_Type (Node => Var_Expr.Node, First_Call => False, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3375
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 3376 '<If at ast.py:6510>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6510
if First_Call then
   If_Result := No_Entity_Base_Type_Decl;
else
   If_Result := Ent;
end if;
--# expr-done 3376
   Result_Var := If_Result;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3377
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Final_Accessed_Type;
--# end
--# property-start BaseTypeDecl.is_access_to /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6513
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_To
  (Node : Bare_Base_Type_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3380 '<Block at ???>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind access_type Block_Var
--# expr-start 3379 '<FieldAccess for matching_formal_type at ast.py:6516>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6516
--# expr-start 3378 '<FieldAccess for accessed_type at ast.py:6516>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6516
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3378
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld.Node, Formal_Type => Typ, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3379
Let_Result := Fld_1; 
--# end
--# expr-done 3380
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Access_To;
--# end
--# property-start BaseTypeDecl.is_subp_access_of /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6518
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Subp_Access_Of
  (Node : Bare_Base_Type_Decl
      ; Entity : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind entity Entity
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Access_To_Subp_Def;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3386 '<Block at ???>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind access_type Block_Var
--# expr-start 3382 '<Cast to AccessToSubpDef at ast.py:6525>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6525
--# expr-start 3381 '<FieldAccess for access_def at ast.py:6525>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6525
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3381
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 3382
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr
--# expr-start 3383 '<FieldAccess for subp_spec_or_null at ast.py:6526>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6526
      if Entity.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Entity.Node, E_Info => Entity.Info);
--# end
--# expr-done 3383
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind se Var_Expr_1
--# expr-start 3385 '<FieldAccess for match_signature at ast.py:6527>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6527
--# expr-start 3384 '<FieldAccess for subp_spec at ast.py:6527>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6527
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr.Info);
--# expr-done 3384
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_2.Node, Other => Var_Expr_1, Match_Name => False, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 3385
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 3386
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Subp_Access_Of;
--# end
--# property-start BaseTypeDecl.is_generic_formal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6531
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Generic_Formal
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3392 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3388 '<IsA GenericFormalTypeDecl at ast.py:6537>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6537
--# expr-start 3387 '<parent at ast.py:6537>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6537
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3387
Is_A := Fld /= null 
and then Fld.Kind in Ada_Generic_Formal_Type_Decl_Range; 
--# expr-done 3388
if Is_A then
   If_Result := True;
else
--# expr-start 3390 '<Cast to BaseTypeDecl at ast.py:6538>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6538
--# expr-start 3389 '<parent at ast.py:6538>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6538
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 3389
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3390
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3391 '<FieldAccess for is_generic_formal at ast.py:6538>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6538
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Var_Expr);
--# end
--# expr-done 3391
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3392
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Generic_Formal;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_tagged_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl | Ada_Protected_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_tagged_type
                  Property_Result := Base_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_tagged_type
                  Property_Result := Base_Subtype_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_tagged_type
                  Property_Result := Classwide_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Formal_Type_Decl =>
                  --# property-call-start IncompleteFormalTypeDecl.is_tagged_type
                  Property_Result := Incomplete_Formal_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTaggedTypeDecl.is_tagged_type
                  Property_Result := Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_tagged_type
                  Property_Result := Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseTypeDecl.is_tagged_type_with_deref /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6546
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Tagged_Type_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3399 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3398 '<FieldAccess for is_tagged_type at ast.py:6556>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6556
--# expr-start 3397 '<If at ast.py:6552>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6552
--# expr-start 3395 '<Not at ast.py:6553>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6553
--# expr-start 3394 '<IsNull at ast.py:6553>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6553
--# expr-start 3393 '<FieldAccess for get_imp_deref at ast.py:6553>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6553
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3393
Is_Null := Fld_1.Node = null; 
--# expr-done 3394
Not_Val := not (Is_Null); 
--# expr-done 3395
if Not_Val then
   --# expr-start 3396 '<FieldAccess for accessed_type at ast.py:6554>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6554
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3396
   If_Result := Fld;
else
   If_Result := Ent;
end if;
--# expr-done 3397
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 3398
Let_Result := Fld_2; 
--# end
--# expr-done 3399
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Tagged_Type_With_Deref;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_type
                  Property_Result := Base_Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_type
                  Property_Result := Base_Subtype_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.base_type
                  Property_Result := Classwide_Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.base_type
                  Property_Result := Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Base_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_types' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Types
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_types
                  Property_Result := Base_Type_Decl_P_Base_Types
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_types
                  Property_Result := Base_Subtype_Decl_P_Base_Types
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Base_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_interfaces' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Interfaces
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_interfaces
                  Property_Result := Base_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_interfaces
                  Property_Result := Base_Subtype_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.base_interfaces
                  Property_Result := Classwide_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.base_interfaces
                  Property_Result := Protected_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.base_interfaces
                  Property_Result := Task_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.base_interfaces
                  Property_Result := Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Base_Interfaces;
--# end
--# property-start BaseTypeDecl.find_all_derived_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6576
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_All_Derived_Types
  (Node : Bare_Base_Type_Decl
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_488 with Inline_Always;
            procedure Finalizer_Scope_2307 with Inline_Always;
            procedure Finalizer_Scope_2308 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Unit_Array_Access;
Fld_2 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Map_Result : Internal_Entity_Type_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_488 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_488;
            procedure Finalizer_Scope_2307 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2307;
            procedure Finalizer_Scope_2308 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2308;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3407 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3406 '<bind at ast.py:6583>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6583
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 3405 '<Mapcat at ast.py:6586>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6586
   --# expr-start 3401 '<FieldAccess for filter_is_imported_by at ast.py:6585>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6585
--# expr-start 3400 '<FieldAccess for canonical_type at ast.py:6585>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6585
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 3400
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Ada_Node_P_Filter_Is_Imported_By (Node => Fld.Node, Units => Units, Transitive => True);
--# expr-done 3401
      declare
         Map_Result_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Unit_Array_Access := Fld_1;
      begin
         for Item_129 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_129
--# expr-start 3402 '<FieldAccess for root at ast.py:6586>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6586
      if Item_129 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Item_129.Ast_Root;
--# expr-done 3402
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 3404 '<FieldAccess for find_derived_types at ast.py:6587>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6587
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3403 '<as_bare_entity at ast.py:6587>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6587
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 3403
--# property-call-start BaseTypeDecl.find_derived_types
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Find_Derived_Types (Node => Ent.Node, Root => If_Result, Origin => Bound_Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3404
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2308;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2307;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3405
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3406
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3407
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_488;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_488;
                     Finalizer_Scope_2307;
                     Finalizer_Scope_2308;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Find_All_Derived_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.record_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Record_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.record_def
                  Property_Result := Base_Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.record_def
                  Property_Result := Base_Subtype_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.record_def
                  Property_Result := Classwide_Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.record_def
                  Property_Result := Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Record_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.array_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Array_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.array_def
                  Property_Result := Base_Type_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.array_def
                  Property_Result := Base_Subtype_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.array_def
                  Property_Result := Type_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Array_Def;
--# end
--# property-start BaseTypeDecl.array_def_with_deref /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6594
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Def_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Array_Type_Def;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
If_Result : Internal_Entity_Array_Type_Def;
If_Result_1 : Internal_Entity_Array_Type_Def;
Let_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3414 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3413 '<Cond at ast.py:6601>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6601
--# expr-start 3408 '<FieldAccess for is_array at ast.py:6602>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6602
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3408
if Fld then
   --# expr-start 3409 '<FieldAccess for array_def at ast.py:6602>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6602
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3409
   If_Result_1 := Fld_1;
else
--# expr-start 3410 '<FieldAccess for is_implicit_deref at ast.py:6604>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6604
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3410
if Fld_2 then
--# expr-start 3411 '<FieldAccess for accessed_type at ast.py:6605>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6605
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3411
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 3412 '<FieldAccess for array_def at ast.py:6605>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6605
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3412
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
   If_Result := Result_Var;
else
   If_Result := No_Entity_Array_Type_Def;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 3413
Let_Result := If_Result_1; 
--# end
--# expr-done 3414
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Array_Def_With_Deref;
--# end
--# property-start BaseTypeDecl.is_array_def_with_deref /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6610
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Def_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity_Array_Type_Def;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3416 '<Not at ast.py:6611>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6611
--# expr-start 3415 '<IsNull at ast.py:6611>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6611
Is_Null := Self = null; 
--# expr-done 3415
Not_Val := not (Is_Null); 
--# expr-done 3416
if Not_Val then
   --# expr-start 3419 '<Not at ast.py:6611>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6611
--# expr-start 3418 '<IsNull at ast.py:6611>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6611
--# expr-start 3417 '<FieldAccess for array_def_with_deref at ast.py:6611>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6611
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3417
Is_Null_1 := Fld.Node = null; 
--# expr-done 3418
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3419
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Array_Def_With_Deref;
--# end
--# property-start BaseTypeDecl.is_array_def_with_deref_or_null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6615
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Fld : Internal_Entity_Array_Type_Def;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3420 '<IsNull at ast.py:6616>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6616
Is_Null := Self = null; 
--# expr-done 3420
if Is_Null then
   If_Result := True;
else
   --# expr-start 3423 '<Not at ast.py:6616>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6616
--# expr-start 3422 '<IsNull at ast.py:6616>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6616
--# expr-start 3421 '<FieldAccess for array_def_with_deref at ast.py:6616>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6616
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3421
Is_Null_1 := Fld.Node = null; 
--# expr-done 3422
Not_Val := not (Is_Null_1); 
--# expr-done 3423
   If_Result := Not_Val;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null;
--# end
--# property-start BaseTypeDecl.comp_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6620
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Is_Subscript : Boolean
         := False
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind is_subscript Is_Subscript
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Ad : Internal_Entity_Array_Type_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Array_Type_Def;
Var_Expr_3 : Internal_Entity_Access_Def;
Match_0 : Internal_Entity_Access_To_Subp_Def;
Match_1 : Internal_Entity_Base_Type_Access_Def;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Array_Type_Def;
If_Result : Internal_Entity_Array_Type_Def;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Access_Def;
Fld_4 : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Access_Def;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3434 '<Block at ???>' Let_Result_3 None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 3433 '<Let at ast.py:6632>' Let_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6632
--# scope-start
--# expr-start 3426 '<If at ast.py:6633>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6633
if Is_Subscript then
   --# expr-start 3424 '<FieldAccess for array_def_with_deref at ast.py:6634>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3424
   If_Result := Fld;
else
   --# expr-start 3425 '<FieldAccess for array_def at ast.py:6635>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6635
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3425
   If_Result := Fld_1;
end if;
--# expr-done 3426
Ad := If_Result; 
--# bind ad Ad
Var_Expr_2 := Ad;
if Var_Expr_2 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind ad Var_Expr_2
--# expr-start 3427 '<FieldAccess for comp_type at ast.py:6636>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6636
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_2 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 3427
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 3428 '<FieldAccess for access_def at ast.py:6637>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6637
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3428
Var_Expr_3 := Fld_3;
if Var_Expr_3 /= No_Entity_Access_Def then
   --# scope-start
--# expr-start 3432 '<Match at ast.py:6637>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6637
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_3; 
case Ada_Access_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind asd Match_0
--# expr-start 3430 '<FieldAccess for return_type at ast.py:6639>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6639
--# expr-start 3429 '<FieldAccess for subp_spec at ast.py:6639>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6639
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Match_0.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Match_0.Info);
--# expr-done 3429
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 3430
Let_Result := Fld_5; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Anonymous_Type_Access_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tad Match_1
--# expr-start 3431 '<FieldAccess for accessed_type at ast.py:6640>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6640
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 3431
Let_Result_1 := Fld_6; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 3432
Scope_Result_4 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_2 := Result_Var_1;
end if;
Let_Result_2 := Result_Var_2; 
--# end
--# expr-done 3433
Scope_Result_5 := Let_Result_2;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Let_Result_3 := Result_Var_3; 
--# end
--# expr-done 3434
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Comp_Type;
--# end
--# property-start BaseTypeDecl.index_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6646
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Index_Type
  (Node : Bare_Base_Type_Decl
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Array_Type_Def;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3437 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3435 '<FieldAccess for array_def_with_deref at ast.py:6655>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6655
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3435
Var_Expr := Fld;
if Var_Expr /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind ad Var_Expr
--# expr-start 3436 '<FieldAccess for index_type at ast.py:6655>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6655
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.index_type
Fld_1 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Var_Expr.Node, Dim => Dim, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3436
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3437
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Index_Type;
--# end
--# property-start BaseTypeDecl.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6659
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Expr_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Ent;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Expr_Type;
--# end
--# property-start BaseTypeDecl.is_derived_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6661
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Derived_Type
  (Node : Bare_Base_Type_Decl
      ; Other_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other_type Other_Type
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_497 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Boolean;
Quantifier_Result : Boolean;
If_Result_2 : Boolean;
Fld_6 : Boolean;
Fld_7 : Boolean;
If_Result_3 : Boolean;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Classwide_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_10 : Boolean;
Quantifier_Result_1 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_497 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_497;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3454 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3438 '<FieldAccess for canonical_type at ast.py:6667>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6667
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3438
Block_Var := Fld; 
--# bind entity_can Block_Var
--# expr-start 3439 '<FieldAccess for canonical_type at ast.py:6668>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6668
      if Other_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Other_Type.Node, Origin => Origin, E_Info => Other_Type.Info);
--# end
--# expr-done 3439
Block_Var_1 := Fld_1; 
--# bind other_can Block_Var_1
--# expr-start 3440 '<Eq at ast.py:6672>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6672
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 3440
if Is_Equiv then
   If_Result_1 := True;
else
--# expr-start 3441 '<FieldAccess for is_classwide at ast.py:6681>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6681
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3441
if Fld_2 then
   --# expr-start 3443 '<Eq at ast.py:6682>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6682
--# expr-start 3442 '<FieldAccess for classwide_type at ast.py:6682>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6682
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 3442
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr_2 := Block_Var_1; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 3443
   If_Result := Is_Equiv_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 3446 '<AnyQuantifier at ast.py:6685>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6685
   --# expr-start 3444 '<FieldAccess for base_types at ast.py:6685>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6685
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3444
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_130 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_130
      --# expr-start 3445 '<FieldAccess for is_derived_type at ast.py:6685>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6685
      if Item_130.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Item_130.Node, Other_Type => Other_Type, Origin => Origin, E_Info => Item_130.Info);
--# end
--# expr-done 3445
      Quantifier_Result := Fld_5;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 3446
   If_Result_2 := Quantifier_Result;
end if;
if If_Result_2 then
   If_Result_5 := True;
else
--# expr-start 3447 '<FieldAccess for is_classwide at ast.py:6692>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6692
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 3447
if Fld_6 then
   --# expr-start 3448 '<FieldAccess for is_classwide at ast.py:6693>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6693
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3448
   If_Result_3 := Fld_7;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 3453 '<AnyQuantifier at ast.py:6694>' Quantifier_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6694
   --# expr-start 3451 '<FieldAccess for base_types at ast.py:6694>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6694
--# expr-start 3450 '<FieldAccess for type_decl at ast.py:6694>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6694
--# expr-start 3449 '<Cast to ClasswideTypeDecl at ast.py:6694>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6694
Cast_Expr_3 := Block_Var; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_4 := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 3449
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld_8 := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 3450
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Fld_8.Node, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 3451
      Quantifier_Result_1 := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_9;
      begin
         for Item_131 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_131
      --# expr-start 3452 '<FieldAccess for is_derived_type at ast.py:6695>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6695
      if Item_131.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Item_131.Node, Other_Type => Other_Type, Origin => Origin, E_Info => Item_131.Info);
--# end
--# expr-done 3452
      Quantifier_Result_1 := Fld_10;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 3453
   If_Result_4 := Quantifier_Result_1;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 3454
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_497;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_497;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Derived_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_iterable_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_iterable_type
                  Property_Result := Base_Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_iterable_type
                  Property_Result := Base_Subtype_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_iterable_type
                  Property_Result := Classwide_Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_iterable_type
                  Property_Result := Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_interface_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Interface_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_interface_type
                  Property_Result := Base_Type_Decl_P_Is_Interface_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_interface_type
                  Property_Result := Classwide_Type_Decl_P_Is_Interface_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Interface_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.iterable_comp_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.iterable_comp_type
                  Property_Result := Base_Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.iterable_comp_type
                  Property_Result := Base_Subtype_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.iterable_comp_type
                  Property_Result := Classwide_Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.iterable_comp_type
                  Property_Result := Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseTypeDecl.matching_prefix_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6724
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Prefix_Type
  (Node : Bare_Base_Type_Decl
      ; Container_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind container_type Container_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3464 '<Block at ???>' Let_Result None
--# scope-start
Block_Var := Container_Type; 
--# bind cont_type Block_Var
--# expr-start 3456 '<Not at ast.py:6732>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6732
--# expr-start 3455 '<IsNull at ast.py:6732>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6732
Is_Null := Self = null; 
--# expr-done 3455
Not_Val := not (Is_Null); 
--# expr-done 3456
if Not_Val then
   --# expr-start 3458 '<Not at ast.py:6732>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6732
--# expr-start 3457 '<IsNull at ast.py:6732>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6732
Is_Null_1 := Container_Type.Node = null; 
--# expr-done 3457
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3458
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 3459 '<FieldAccess for matching_formal_prim_type at ast.py:6734>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6734
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Ent.Node, Formal_Type => Block_Var, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3459
if Fld then
   If_Result_1 := True;
else
--# expr-start 3460 '<FieldAccess for final_accessed_type at ast.py:6737>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6737
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.final_accessed_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Final_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3460
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3461 '<FieldAccess for matching_formal_prim_type at ast.py:6737>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6737
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Var_Expr.Node, Formal_Type => Block_Var, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3461
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_1 := Result_Var;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 3462 '<FieldAccess for accessed_type at ast.py:6741>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6741
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3462
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr_1
--# expr-start 3463 '<FieldAccess for matching_formal_prim_type at ast.py:6742>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6742
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Ent.Node, Formal_Type => Var_Expr_1, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3463
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_2 := Result_Var_1;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 3464
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Prefix_Type;
--# end
--# property-start BaseTypeDecl.matching_access_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6746
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; For_Assignment : Boolean
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind for_assignment For_Assignment
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Anonymous_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Match_0_1 : Internal_Entity_Anonymous_Type_Decl;
Match_1_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
Match_Prefix_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Anonymous_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Match_Result_1 : Boolean;
Let_Result_4 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3469 '<Block at ???>' Let_Result_4 None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 3468 '<Match at ast.py:6753>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6753
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Expected_Type; 
case Ada_Base_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_2; 
--# bind atd Match_0
--# expr-start 3465 '<FieldAccess for access_def_matches at ast.py:6755>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6755
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AnonymousTypeDecl.access_def_matches
Fld := Libadalang.Implementation.Anonymous_Type_Decl_P_Access_Def_Matches (Node => Match_0.Node, Other => Block_Var, For_Assignment => For_Assignment, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 3465
Let_Result_2 := Fld; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_2; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl | Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_3; 
--# bind _ Match_1
--# expr-start 3467 '<Match at ast.py:6756>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6756
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Block_Var; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind atd2 Match_0_1
--# expr-start 3466 '<FieldAccess for access_def_matches at ast.py:6758>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6758
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AnonymousTypeDecl.access_def_matches
Fld_1 := Libadalang.Implementation.Anonymous_Type_Decl_P_Access_Def_Matches (Node => Match_0_1.Node, Other => Expected_Type, For_Assignment => For_Assignment, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 3466
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl | Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind _ Match_1_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3467
Let_Result_3 := Match_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
end case;
--# expr-done 3468
Let_Result_4 := Match_Result_1; 
--# end
--# expr-done 3469
         Property_Result := Let_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Access_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_prim_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6763
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Prim_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3475 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3471 '<Not at ast.py:6766>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6766
--# expr-start 3470 '<IsNull at ast.py:6766>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6766
Is_Null := Formal_Type.Node = null; 
--# expr-done 3470
Not_Val := not (Is_Null); 
--# expr-done 3471
if Not_Val then
   --# expr-start 3473 '<Not at ast.py:6767>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6767
--# expr-start 3472 '<IsNull at ast.py:6767>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6767
Is_Null_1 := Self = null; 
--# expr-done 3472
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3473
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3474 '<FieldAccess for matching_formal_type_impl at ast.py:6768>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6768
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Accept_Derived => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3474
   If_Result_1 := Fld;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 3475
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Formal_Prim_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6771
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3481 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3477 '<Not at ast.py:6774>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6774
--# expr-start 3476 '<IsNull at ast.py:6774>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6774
Is_Null := Formal_Type.Node = null; 
--# expr-done 3476
Not_Val := not (Is_Null); 
--# expr-done 3477
if Not_Val then
   --# expr-start 3479 '<Not at ast.py:6775>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6775
--# expr-start 3478 '<IsNull at ast.py:6775>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6775
Is_Null_1 := Self = null; 
--# expr-done 3478
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3479
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3480 '<FieldAccess for matching_formal_type_impl at ast.py:6776>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6776
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3480
   If_Result_1 := Fld;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 3481
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Formal_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_type_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6779
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type_Impl
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Accept_Derived : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind accept_derived Accept_Derived
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
If_Result : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
If_Result_4 : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
If_Result_5 : Boolean;
Fld_11 : Boolean;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_8 : Boolean;
Fld_14 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
Fld_17 : Boolean;
If_Result_11 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3502 '<Block at ???>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 3482 '<FieldAccess for is_classwide at ast.py:6784>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6784
      if Formal_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Formal_Type.Node, E_Info => Formal_Type.Info);
--# end
--# expr-done 3482
if Fld then
   If_Result := True;
else
   If_Result := Accept_Derived;
end if;
if If_Result then
   --# expr-start 3484 '<FieldAccess for is_derived_type at ast.py:6785>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6785
--# expr-start 3483 '<FieldAccess for specific_type at ast.py:6785>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6785
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 3483
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Fld_1.Node, Other_Type => Formal_Type, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3484
   If_Result_1 := Fld_2;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 3485 '<FieldAccess for is_classwide at ast.py:6787>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6787
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 3485
if Fld_3 then
   --# expr-start 3487 '<FieldAccess for matching_type at ast.py:6788>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6788
--# expr-start 3486 '<FieldAccess for specific_type at ast.py:6788>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6788
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 3486
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_4.Node, Expected_Type => Formal_Type, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 3487
   If_Result_2 := Fld_5;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_8 := True;
else
--# expr-start 3488 '<FieldAccess for accessed_type at ast.py:6791>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6791
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3488
Var_Expr := Fld_6;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind actual_accessed_type Var_Expr
--# expr-start 3489 '<FieldAccess for accessed_type at ast.py:6793>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6793
      if Formal_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Formal_Type.Node, Origin => Origin, E_Info => Formal_Type.Info);
--# end
--# expr-done 3489
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind formal_accessed_type Var_Expr_1
--# expr-start 3490 '<FieldAccess for is_classwide at ast.py:6796>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6796
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3490
if Fld_8 then
   If_Result_4 := True;
else
   If_Result_4 := Accept_Derived;
end if;
if If_Result_4 then
   --# expr-start 3492 '<FieldAccess for is_derived_type at ast.py:6798>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6798
--# expr-start 3491 '<FieldAccess for specific_type at ast.py:6797>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6797
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3491
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Fld_9.Node, Other_Type => Var_Expr_1, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 3492
   If_Result_5 := Fld_10;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   If_Result_7 := True;
else
--# expr-start 3493 '<FieldAccess for is_classwide at ast.py:6800>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6800
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3493
if Fld_11 then
   --# expr-start 3495 '<FieldAccess for matching_type at ast.py:6801>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6801
--# expr-start 3494 '<FieldAccess for specific_type at ast.py:6801>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6801
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3494
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_12.Node, Expected_Type => Var_Expr_1, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 3495
   If_Result_6 := Fld_13;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
Scope_Result := If_Result_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_8 := Result_Var_1;
end if;
if If_Result_8 then
   If_Result_10 := True;
else
--# expr-start 3498 '<Not at ast.py:6807>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6807
--# expr-start 3497 '<IsNull at ast.py:6807>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6807
--# expr-start 3496 '<FieldAccess for get_imp_deref at ast.py:6807>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6807
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_14 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 3496
Is_Null := Fld_14.Node = null; 
--# expr-done 3497
Not_Val := not (Is_Null); 
--# expr-done 3498
if Not_Val then
   --# expr-start 3500 '<FieldAccess for matching_formal_type at ast.py:6809>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6809
--# expr-start 3499 '<FieldAccess for accessed_type at ast.py:6809>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6809
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3499
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_16 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_15.Node, Formal_Type => Formal_Type, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 3500
   If_Result_9 := Fld_16;
else
   If_Result_9 := False;
end if;
   If_Result_10 := If_Result_9;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 3501 '<FieldAccess for matching_type at ast.py:6811>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6811
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_17 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Block_Var.Node, Expected_Type => Formal_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3501
   If_Result_11 := Fld_17;
end if;
Let_Result := If_Result_11; 
--# end
--# expr-done 3502
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Formal_Type_Impl;
--# end
--# property-start BaseTypeDecl.matching_assign_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6814
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Assign_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_3 : Internal_Entity_Expr;
Is_Null_2 : Boolean;
Not_Val_2 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Fld_6 : Internal_Entity_Expr;
Is_Null_3 : Boolean;
Not_Val_3 : Boolean;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Fld_9 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3521 '<Block at ???>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 3504 '<Not at ast.py:6817>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6817
--# expr-start 3503 '<IsNull at ast.py:6817>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6817
Is_Null := Self = null; 
--# expr-done 3503
Not_Val := not (Is_Null); 
--# expr-done 3504
if Not_Val then
   --# expr-start 3506 '<Not at ast.py:6817>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6817
--# expr-start 3505 '<IsNull at ast.py:6817>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6817
Is_Null_1 := Expected_Type.Node = null; 
--# expr-done 3505
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3506
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 3507 '<FieldAccess for matching_type at ast.py:6818>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6818
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Ent.Node, Expected_Type => Expected_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3507
if Fld then
   If_Result_2 := True;
else
--# expr-start 3508 '<FieldAccess for is_classwide at ast.py:6821>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6821
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Expected_Type.Node, E_Info => Expected_Type.Info);
--# end
--# expr-done 3508
if Fld_1 then
   --# expr-start 3509 '<FieldAccess for matching_formal_prim_type at ast.py:6822>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6822
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Block_Var.Node, Formal_Type => Expected_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3509
   If_Result_1 := Fld_2;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
   If_Result_4 := True;
else
--# expr-start 3512 '<Not at ast.py:6826>' Not_Val_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6826
--# expr-start 3511 '<IsNull at ast.py:6826>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6826
--# expr-start 3510 '<FieldAccess for get_imp_deref at ast.py:6826>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6826
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 3510
Is_Null_2 := Fld_3.Node = null; 
--# expr-done 3511
Not_Val_2 := not (Is_Null_2); 
--# expr-done 3512
if Not_Val_2 then
   --# expr-start 3514 '<FieldAccess for matching_assign_type at ast.py:6828>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6828
--# expr-start 3513 '<FieldAccess for accessed_type at ast.py:6828>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6828
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3513
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_4.Node, Expected_Type => Expected_Type, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 3514
   If_Result_3 := Fld_5;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
if If_Result_4 then
   If_Result_6 := True;
else
--# expr-start 3517 '<Not at ast.py:6832>' Not_Val_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6832
--# expr-start 3516 '<IsNull at ast.py:6832>' Is_Null_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6832
--# expr-start 3515 '<FieldAccess for get_imp_deref at ast.py:6832>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6832
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Expected_Type.Node, E_Info => Expected_Type.Info);
--# end
--# expr-done 3515
Is_Null_3 := Fld_6.Node = null; 
--# expr-done 3516
Not_Val_3 := not (Is_Null_3); 
--# expr-done 3517
if Not_Val_3 then
   --# expr-start 3519 '<FieldAccess for matching_assign_type at ast.py:6834>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6834
--# expr-start 3518 '<FieldAccess for accessed_type at ast.py:6834>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6834
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 3518
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_7.Node, Expected_Type => Block_Var, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 3519
   If_Result_5 := Fld_8;
else
   If_Result_5 := False;
end if;
   If_Result_6 := If_Result_5;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 3520 '<FieldAccess for matching_access_type at ast.py:6837>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6837
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_access_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Access_Type (Node => Ent.Node, Expected_Type => Expected_Type, For_Assignment => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3520
   If_Result_7 := Fld_9;
end if;
   If_Result_8 := If_Result_7;
else
   If_Result_8 := False;
end if;
Let_Result := If_Result_8; 
--# end
--# expr-done 3521
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Assign_Type;
--# end
--# property-start BaseTypeDecl.matching_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6840
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
Fld_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
Fld_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_4 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv_2 : Boolean;
Fld_5 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Fld_6 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_3 : Boolean;
Fld_7 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Cast_Result_5 : Internal_Entity;
Is_Equiv_4 : Boolean;
If_Result_8 : Boolean;
Fld_10 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3542 '<Block at ???>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 3523 '<Not at ast.py:6848>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6848
--# expr-start 3522 '<IsNull at ast.py:6848>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6848
Is_Null := Expected_Type.Node = null; 
--# expr-done 3522
Not_Val := not (Is_Null); 
--# expr-done 3523
if Not_Val then
   --# expr-start 3525 '<Not at ast.py:6849>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6849
--# expr-start 3524 '<IsNull at ast.py:6849>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6849
Is_Null_1 := Block_Var.Node = null; 
--# expr-done 3524
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3525
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 3527 '<Eq at ast.py:6851>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6851
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 3526 '<FieldAccess for universal_int_type at ast.py:6851>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6851
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 3526
Is_Equiv := Equivalent (Cast_Result, Fld); 
--# expr-done 3527
if Is_Equiv then
   --# expr-start 3528 '<FieldAccess for allows_universal_int at ast.py:6852>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6852
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.allows_universal_int
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Allows_Universal_Int (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 3528
   If_Result_1 := Fld_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 3530 '<Eq at ast.py:6854>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6854
Cast_Expr_1 := Expected_Type; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 3529 '<FieldAccess for universal_int_type at ast.py:6854>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6854
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 3529
Is_Equiv_1 := Equivalent (Cast_Result_1, Fld_2); 
--# expr-done 3530
if Is_Equiv_1 then
   --# expr-start 3531 '<FieldAccess for is_int_type at ast.py:6855>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6855
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3531
   If_Result_2 := Fld_3;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_5 := True;
else
--# expr-start 3533 '<Eq at ast.py:6857>' Is_Equiv_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6857
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 3532 '<FieldAccess for universal_real_type at ast.py:6857>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6857
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 3532
Is_Equiv_2 := Equivalent (Cast_Result_2, Fld_4); 
--# expr-done 3533
if Is_Equiv_2 then
   --# expr-start 3534 '<FieldAccess for allows_universal_real at ast.py:6858>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6858
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.allows_universal_real
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Allows_Universal_Real (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 3534
   If_Result_4 := Fld_5;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
if If_Result_5 then
   If_Result_7 := True;
else
--# expr-start 3536 '<Eq at ast.py:6860>' Is_Equiv_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6860
Cast_Expr_3 := Expected_Type; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-start 3535 '<FieldAccess for universal_real_type at ast.py:6860>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6860
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_6 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 3535
Is_Equiv_3 := Equivalent (Cast_Result_3, Fld_6); 
--# expr-done 3536
if Is_Equiv_3 then
   --# expr-start 3537 '<FieldAccess for is_real_type at ast.py:6861>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6861
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3537
   If_Result_6 := Fld_7;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 3540 '<Eq at ast.py:6863>' Is_Equiv_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6863
--# expr-start 3538 '<FieldAccess for canonical_type at ast.py:6863>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6863
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3538
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
--# expr-start 3539 '<FieldAccess for canonical_type at ast.py:6863>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6863
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 3539
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Is_Equiv_4 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 3540
   If_Result_8 := Is_Equiv_4;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 3541 '<FieldAccess for matching_access_type at ast.py:6865>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6865
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_access_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Access_Type (Node => Block_Var.Node, Expected_Type => Expected_Type, For_Assignment => False, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 3541
   If_Result_9 := Fld_10;
end if;
   If_Result_10 := If_Result_9;
else
   If_Result_10 := False;
end if;
Let_Result := If_Result_10; 
--# end
--# expr-done 3542
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Type;
--# end
--# property-start BaseTypeDecl.matching_allocator_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6869
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Allocator_Type
  (Node : Bare_Base_Type_Decl
      ; Allocated_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind allocated_type Allocated_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3546 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3543 '<FieldAccess for is_access_type at ast.py:6872>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6872
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3543
if Fld then
   --# expr-start 3545 '<FieldAccess for matching_formal_type at ast.py:6873>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6873
      if Allocated_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3544 '<FieldAccess for accessed_type at ast.py:6873>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6873
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3544
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Allocated_Type.Node, Formal_Type => Fld_1, Origin => Origin, E_Info => Allocated_Type.Info);
--# end
--# expr-done 3545
   If_Result := Fld_2;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3546
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Allocator_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.canonical_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Canonical_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.canonical_type
                  Property_Result := Base_Type_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.canonical_type
                  Property_Result := Base_Subtype_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.canonical_type
                  Property_Result := Classwide_Type_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Canonical_Type;
--# end
--# property-start BaseTypeDecl.classwide_type_node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6887
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Classwide_Type_Node
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Classwide_Type_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Classwide_Type_Decl;
Let_Result : Bare_Classwide_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Classwide_Type_Node,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Bare_Classwide_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3549 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3548 '<New ClasswideTypeDecl at ???>' New_Node None
--# expr-start 3547 '<FieldAccess for name at ast.py:6890>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6890
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 3547
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Classwide_Type_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Classwide_Type_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Classwide_Type_Decl
     (Self => New_Node,
      Base_Type_Decl_F_Name => Fld);
--# expr-done 3548
Let_Result := New_Node; 
--# end
--# expr-done 3549
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     Add_Memoized_Error
                       (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
               end if;
            raise;
      end;
      if Self /= null then
            Mmz_Val := (Kind => Mmz_Bare_Classwide_Type_Decl,
                        As_Bare_Classwide_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   return Property_Result;
end Base_Type_Decl_P_Classwide_Type_Node;
--# end
--# property-start BaseTypeDecl.scalar_base_subtype_node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6892
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Subtype_Node
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Discrete_Base_Subtype_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Discrete_Base_Subtype_Decl;
Let_Result : Bare_Discrete_Base_Subtype_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Scalar_Base_Subtype_Node,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Bare_Discrete_Base_Subtype_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3552 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3551 '<New DiscreteBaseSubtypeDecl at ast.py:6900>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6900
--# expr-start 3550 '<FieldAccess for name at ast.py:6900>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6900
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 3550
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Discrete_Base_Subtype_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Discrete_Base_Subtype_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Discrete_Base_Subtype_Decl
     (Self => New_Node,
      Base_Type_Decl_F_Name => Fld);
--# expr-done 3551
Let_Result := New_Node; 
--# end
--# expr-done 3552
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     Add_Memoized_Error
                       (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
               end if;
            raise;
      end;
      if Self /= null then
            Mmz_Val := (Kind => Mmz_Bare_Discrete_Base_Subtype_Decl,
                        As_Bare_Discrete_Base_Subtype_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   return Property_Result;
end Base_Type_Decl_P_Scalar_Base_Subtype_Node;
--# end
--# property-start BaseTypeDecl.scalar_base_subtype /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6902
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Subtype
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
      Fld : Bare_Discrete_Base_Subtype_Decl;
As_Entity : Internal_Entity_Discrete_Base_Subtype_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
Let_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3555 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3554 '<as_entity at ast.py:6908>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6908
--# expr-start 3553 '<FieldAccess for scalar_base_subtype_node at ast.py:6908>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6908
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype_Node (Node => Self);
--# end
--# expr-done 3553
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Discrete_Base_Subtype_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 3554
Let_Result := If_Result; 
--# end
--# expr-done 3555
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Scalar_Base_Subtype;
--# end
--# property-start '[dispatcher]BaseTypeDecl.previous_part' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Previous_Part
  (Node : Bare_Base_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.previous_part
                  Property_Result := Base_Type_Decl_P_Previous_Part
                    (Self
                        , Go_To_Incomplete
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.previous_part
                  Property_Result := Classwide_Type_Decl_P_Previous_Part
                    (Self
                        , Go_To_Incomplete
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Previous_Part;
--# end
--# property-start BaseTypeDecl.next_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6941
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Next_Part
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Incomplete_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Bare_Declarative_Part;
Var_Expr_2 : Boolean;
Var_Expr_3 : Internal_Entity_Package_Body;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Bare_Declarative_Part;
Fld_4 : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
As_Entity_1 : Internal_Entity_Base_Package_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Package_Decl;
Fld_5 : Internal_Entity_Package_Body;
Fld_6 : Internal_Entity_Declarative_Part;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
Fld_10 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Incomplete_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Next_Part,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3573 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 3572 '<Match at ast.py:6953>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6953
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind itd Match_0
--# expr-start 3556 '<FieldAccess for declarative_scope at ast.py:6957>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6957
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Match_0.Node);
--# end
--# expr-done 3556
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind s Var_Expr_1
--# expr-start 3558 '<FieldAccess for find_next_part_in at ast.py:6958>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6958
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3557 '<as_entity at ast.py:6958>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6958
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3557
--# property-call-start IncompleteTypeDecl.find_next_part_in
Fld_1 := Libadalang.Implementation.Incomplete_Type_Decl_P_Find_Next_Part_In (Node => Match_0.Node, Decl_Part => If_Result, E_Info => Match_0.Info);
--# end
--# expr-done 3558
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_1;
else
--# expr-start 3559 '<FieldAccess for is_in_private_part at ast.py:6964>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6964
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3559
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 3564 '<FieldAccess for body_part at ast.py:6967>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6967
--# expr-start 3563 '<as_entity at ast.py:6967>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6967
--# expr-start 3562 '<Cast to BasePackageDecl at ast.py:6967>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6967
--# expr-start 3561 '<parent at ast.py:6966>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6966
--# expr-start 3560 '<FieldAccess for declarative_scope at ast.py:6966>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6966
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_3 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3560
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 3561
   if Fld_4 = null
      or else Fld_4.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld_4;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 3562
Is_Null_1 := Cast_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Package_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 3563
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_5 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 3564
Var_Expr_3 := Fld_5;
if Var_Expr_3 /= No_Entity_Package_Body then
   --# scope-start
   --# bind p Var_Expr_3
--# expr-start 3566 '<FieldAccess for find_next_part_in at ast.py:6968>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6968
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3565 '<FieldAccess for decls at ast.py:6968>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6968
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_3.Node.Package_Body_F_Decls, Info => Var_Expr_3.Info);
--# expr-done 3565
--# property-call-start IncompleteTypeDecl.find_next_part_in
Fld_7 := Libadalang.Implementation.Incomplete_Type_Decl_P_Find_Next_Part_In (Node => Match_0.Node, Decl_Part => Fld_6, E_Info => Match_0.Info);
--# end
--# expr-done 3566
Scope_Result_2 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_3 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_3;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Let_Result := Result_Var_3; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Classwide_Type_Decl | Ada_Protected_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 3571 '<If at ast.py:6972>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6972
--# expr-start 3567 '<FieldAccess for is_private at ast.py:6973>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6973
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3567
if Fld_9 then
   --# expr-start 3569 '<Not at ast.py:6974>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6974
--# expr-start 3568 '<FieldAccess for is_generic_formal at ast.py:6974>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6974
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Ent.Node);
--# end
--# expr-done 3568
Not_Val := not (Fld_10); 
--# expr-done 3569
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 3570 '<FieldAccess for private_completion at ast.py:6975>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6975
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3570
   If_Result_3 := Fld_8;
else
   If_Result_3 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 3571
Let_Result_1 := If_Result_3; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 3572
Let_Result_2 := Match_Result; 
--# end
--# expr-done 3573
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Next_Part;
--# end
--# property-start BaseTypeDecl.full_view /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6981
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Full_View
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3576 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3574 '<FieldAccess for next_part at ast.py:6986>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6986
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3574
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 3575 '<FieldAccess for full_view at ast.py:6987>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6987
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3575
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3576
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Full_View;
--# end
--# property-start BaseTypeDecl.is_definite_subtype /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6991
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Definite_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Incomplete_Type_Decl;
Match_1 : Internal_Entity_Type_Decl;
Match_2 : Internal_Entity_Subtype_Decl;
Match_3 : Internal_Entity_Classwide_Type_Decl;
Match_4 : Internal_Entity_Task_Type_Decl;
Match_5 : Internal_Entity_Protected_Type_Decl;
Match_6 : Internal_Entity_Base_Type_Decl;
Match_0_1 : Internal_Entity_Derived_Type_Def;
Match_1_1 : Internal_Entity_Array_Type_Def;
Match_2_1 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Discriminant_Part;
Is_Null : Boolean;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Internal_Entity_Subtype_Indication;
Fld_3 : Internal_Entity_Constraint;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Array_Indices;
Is_A : Boolean;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Array_Type_Def;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Fld_7 : Internal_Entity_Subtype_Indication;
Fld_8 : Internal_Entity_Constraint;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
If_Result_2 : Boolean;
Fld_11 : Internal_Entity_Discriminant_Part;
Is_Null_3 : Boolean;
Fld_12 : Internal_Entity_Discriminant_Part;
Is_Null_4 : Boolean;
Match_Prefix_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Incomplete_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Internal_Entity_Type_Decl;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Internal_Entity_Subtype_Decl;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_6 : Internal_Entity_Classwide_Type_Decl;
Let_Result_6 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_7 : Internal_Entity_Task_Type_Decl;
Let_Result_7 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_8 : Internal_Entity_Protected_Type_Decl;
Let_Result_8 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_9 : Internal_Entity_Base_Type_Decl;
Let_Result_9 : Boolean;
Scope_Result_9 : Boolean;
Match_Result_1 : Boolean;
Let_Result_10 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3600 '<Block at ???>' Let_Result_10 None
--# scope-start
--# expr-start 3599 '<Match at ast.py:7000>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7000
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Ent; 
case Ada_Base_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_3; 
--# bind _ Match_0
Let_Result_3 := False; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_4; 
--# bind td Match_1
--# expr-start 3578 '<IsNull at ast.py:7002>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7002
--# expr-start 3577 '<FieldAccess for discriminants at ast.py:7002>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7002
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Match_1.Node.Type_Decl_F_Discriminants, Info => Match_1.Info);
--# expr-done 3577
Is_Null := Fld.Node = null; 
--# expr-done 3578
if Is_Null then
   --# expr-start 3588 '<Match at ast.py:7002>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7002
--# expr-start 3579 '<FieldAccess for type_def at ast.py:7002>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7002
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_1.Node.Type_Decl_F_Type_Def, Info => Match_1.Info);
--# expr-done 3579
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind dtd Match_0_1
--# expr-start 3583 '<Not at ast.py:7004>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7004
--# expr-start 3582 '<IsNull at ast.py:7004>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7004
--# expr-start 3581 '<FieldAccess for constraint at ast.py:7004>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7004
--# expr-start 3580 '<FieldAccess for subtype_indication at ast.py:7004>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7004
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subtype_Indication (Node => Match_0_1.Node.Derived_Type_Def_F_Subtype_Indication, Info => Match_0_1.Info);
--# expr-done 3580
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Constraint (Node => Fld_2.Node.Subtype_Indication_F_Constraint, Info => Fld_2.Info);
--# expr-done 3581
Is_Null_1 := Fld_3.Node = null; 
--# expr-done 3582
Not_Val := not (Is_Null_1); 
--# expr-done 3583
if Not_Val then
   If_Result := True;
else
   --# expr-start 3585 '<FieldAccess for is_definite_subtype at ast.py:7005>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7005
--# expr-start 3584 '<FieldAccess for base_type at ast.py:7005>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7005
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0_1.Node, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 3584
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 3585
   If_Result := Fld_5;
end if;
Let_Result := If_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind atd Match_1_1
--# expr-start 3587 '<IsA ConstrainedArrayIndices at ast.py:7008>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7008
--# expr-start 3586 '<FieldAccess for indices at ast.py:7008>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7008
      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Array_Indices (Node => Match_1_1.Node.Array_Type_Def_F_Indices, Info => Match_1_1.Info);
--# expr-done 3586
Is_A := Fld_6.Node /= null 
and then Fld_6.Node.Kind in Ada_Constrained_Array_Indices_Range; 
--# expr-done 3587
Let_Result_1 := Is_A; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_2; 
--# bind _ Match_2_1
Let_Result_2 := True; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3588
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
Let_Result_4 := If_Result_1; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_5; 
--# bind st Match_2
--# expr-start 3592 '<Not at ast.py:7012>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7012
--# expr-start 3591 '<IsNull at ast.py:7012>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7012
--# expr-start 3590 '<FieldAccess for constraint at ast.py:7012>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7012
--# expr-start 3589 '<FieldAccess for subtype at ast.py:7012>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7012
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Subtype_Indication (Node => Match_2.Node.Subtype_Decl_F_Subtype, Info => Match_2.Info);
--# expr-done 3589
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Constraint (Node => Fld_7.Node.Subtype_Indication_F_Constraint, Info => Fld_7.Info);
--# expr-done 3590
Is_Null_2 := Fld_8.Node = null; 
--# expr-done 3591
Not_Val_1 := not (Is_Null_2); 
--# expr-done 3592
if Not_Val_1 then
   If_Result_2 := True;
else
   --# expr-start 3594 '<FieldAccess for is_definite_subtype at ast.py:7013>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7013
--# expr-start 3593 '<FieldAccess for get_type at ast.py:7013>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7013
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_2.Node, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 3593
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 3594
   If_Result_2 := Fld_10;
end if;
Let_Result_5 := If_Result_2; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
         when Ada_Classwide_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_6; 
--# bind _ Match_3
Let_Result_6 := False; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result_1 := Scope_Result_6; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_4 := Cast_Result_7; 
--# bind ttd Match_4
--# expr-start 3596 '<IsNull at ast.py:7015>' Is_Null_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7015
--# expr-start 3595 '<FieldAccess for discriminants at ast.py:7015>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7015
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Discriminant_Part (Node => Match_4.Node.Task_Type_Decl_F_Discriminants, Info => Match_4.Info);
--# expr-done 3595
Is_Null_3 := Fld_11.Node = null; 
--# expr-done 3596
Let_Result_7 := Is_Null_3; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result_1 := Scope_Result_7; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_5 := Cast_Result_8; 
--# bind ptd Match_5
--# expr-start 3598 '<IsNull at ast.py:7016>' Is_Null_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7016
--# expr-start 3597 '<FieldAccess for discriminants at ast.py:7016>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7016
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Discriminant_Part (Node => Match_5.Node.Protected_Type_Decl_F_Discriminants, Info => Match_5.Info);
--# expr-done 3597
Is_Null_4 := Fld_12.Node = null; 
--# expr-done 3598
Let_Result_8 := Is_Null_4; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result_1 := Scope_Result_8; 
         when Ada_Discrete_Base_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_6 := Cast_Result_9; 
--# bind _ Match_6
Let_Result_9 := True; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result_1 := Scope_Result_9; 
end case;
--# expr-done 3599
Let_Result_10 := Match_Result_1; 
--# end
--# expr-done 3600
         Property_Result := Let_Result_10;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Definite_Subtype;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_private' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Private
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_private
                  Property_Result := Base_Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_private
                  Property_Result := Base_Subtype_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_private
                  Property_Result := Classwide_Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_private
                  Property_Result := Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Private;
--# end
--# property-start '[dispatcher]BaseTypeDecl.discriminants_list' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Discriminants_List
  (Node : Bare_Base_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.discriminants_list
                  Property_Result := Base_Subtype_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.discriminants_list
                  Property_Result := Classwide_Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.discriminants_list
                  Property_Result := Incomplete_Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.discriminants_list
                  Property_Result := Protected_Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.discriminants_list
                  Property_Result := Task_Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.discriminants_list
                  Property_Result := Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Discriminants_List;
--# end
--# property-start BaseTypeDecl.corresponding_char_literal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7048
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Corresponding_Char_Literal
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Enum_Type_Def;
Block_Var_2 : Symbol_Type;
Block_Var_3 : Internal_Entity_Enum_Literal_Decl;
Var_Expr : Internal_Entity_Type_Decl;
Var_Expr_1 : Bare_Char_Literal;
Var_Expr_2 : Internal_Entity_Enum_Type_Def;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Type_Def;
Scope_Result : Internal_Entity_Type_Def;
Result_Var : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Enum_Type_Def;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Char_Literal;
Sym : Symbol_Type;
Scope_Result_1 : Symbol_Type;
Result_Var_1 : Symbol_Type;
Fld_2 : Internal_Entity_Enum_Literal_Decl_List;
Scope_Result_2 : Internal_Entity_Enum_Literal_Decl_List;
Result_Var_2 : Internal_Entity_Enum_Literal_Decl_List;
Fld_3 : Bare_Enum_Literal_Decl_List;
Fld_4 : Internal_Entity_Info;
As_Entity : Internal_Entity_Enum_Literal_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Literal_Decl;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Boolean;
Item_133 : Internal_Entity_Enum_Literal_Decl;
Bare_Item_133 : Bare_Enum_Literal_Decl;
Find_Result : Internal_Entity_Enum_Literal_Decl;
Fld_7 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Type_Decl;
New_Node : Bare_Synthetic_Char_Enum_Lit;
As_Entity_1 : Internal_Entity_Synthetic_Char_Enum_Lit;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Synthetic_Char_Enum_Lit;
Cast_Result_4 : Internal_Entity_Enum_Literal_Decl;
Is_Null_2 : Boolean;
Fld_8 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Enum_Literal_Decl;
Let_Result : Internal_Entity_Enum_Literal_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Corresponding_Char_Literal,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 3618 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3601 '<FieldAccess for root_type at ast.py:7060>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7060
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3601
Block_Var := Fld; 
--# bind root Block_Var
--# expr-start 3604 '<Cast to EnumTypeDef at ast.py:7061>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7061
--# expr-start 3602 '<Cast to TypeDecl at ast.py:7061>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7061
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 3602
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 3603 '<FieldAccess for type_def at ast.py:7061>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7061
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Var_Expr.Node.Type_Decl_F_Type_Def, Info => Var_Expr.Info);
--# expr-done 3603
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Enum_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Enum_Type_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Enum_Type_Def;
   end if;
--# expr-done 3604
Block_Var_1 := Cast_Result_1; 
--# bind enum_type Block_Var_1
--# expr-start 3605 '<Cast to CharLiteral at ast.py:7062>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7062
Cast_Expr_1 := Origin; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Char_Literal_Range
   then
      Cast_Result_2 := Cast_Expr_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 3605
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind l Var_Expr_1
--# expr-start 3606 '<GetSymbol at ast.py:7062>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7062
Sym := Get_Symbol (Var_Expr_1); 
--# expr-done 3606
Scope_Result_1 := Sym;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := null;
end if;
Block_Var_2 := Result_Var_1; 
--# bind sym Block_Var_2
--# expr-start 3610 '<Find at ast.py:7064>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7064
Var_Expr_2 := Block_Var_1;
if Var_Expr_2 /= No_Entity_Enum_Type_Def then
   --# scope-start
--# expr-start 3607 '<FieldAccess for enum_literals at ast.py:7064>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7064
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Enum_Literal_Decl_List (Node => Var_Expr_2.Node.Enum_Type_Def_F_Enum_Literals, Info => Var_Expr_2.Info);
--# expr-done 3607
Scope_Result_2 := Fld_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Enum_Literal_Decl_List;
end if;
Fld_3 := Result_Var_2.Node;
      Find_Result := No_Entity_Enum_Literal_Decl;
      if Fld_3 = null then
      Find_Result := No_Entity_Enum_Literal_Decl;
      else
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld_3;
      begin
         for Untyped_Item_133 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_133 := Untyped_Item_133; 
Is_Null := Bare_Item_133 = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Literal_Decl;
else
Fld_4 := Result_Var_2.Info;
As_Entity := (Info => Fld_4, Node => Bare_Item_133); 
   If_Result := As_Entity;
end if;
                  Item_133 := If_Result; 
   --# scope-start
               --# bind lit_decl Item_133
      --# expr-start 3609 '<FieldAccess for name_is at ast.py:7065>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7065
--# expr-start 3608 '<FieldAccess for name at ast.py:7065>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7065
      if Item_133.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Defining_Name (Node => Item_133.Node.Enum_Literal_Decl_F_Name, Info => Item_133.Info);
--# expr-done 3608
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_6 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_5.Node, Sym => Block_Var_2);
--# end
--# expr-done 3609
      if Fld_6 then
         Find_Result := Item_133; 
      end if;
   --# end
               exit when Fld_6;
         end loop;
      end;
      end if;
--# expr-done 3610
Block_Var_3 := Find_Result; 
--# bind char_lit Block_Var_3
--# expr-start 3617 '<If at ast.py:7068>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7068
--# expr-start 3611 '<IsNull at ast.py:7072>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7072
Is_Null_2 := Block_Var_3.Node = null; 
--# expr-done 3611
if Is_Null_2 then
   --# expr-start 3612 '<FieldAccess for is_std_char_type at ast.py:7072>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7072
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EnumTypeDef.is_std_char_type
Fld_8 := Libadalang.Implementation.Enum_Type_Def_P_Is_Std_Char_Type (Node => Block_Var_1.Node);
--# end
--# expr-done 3612
   If_Result_2 := Fld_8;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
--# expr-start 3616 '<as_entity at ast.py:7085>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7085
--# expr-start 3615 '<New SyntheticCharEnumLit at ast.py:7074>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7074
--# expr-start 3614 '<Cast to TypeDecl at ast.py:7084>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7084
--# expr-start 3613 '<parent at ast.py:7084>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7084
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# expr-done 3613
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Type_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_3 := No_Entity_Type_Decl;
   end if;
--# expr-done 3614
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Char_Enum_Lit);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Char_Enum_Lit,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Char_Enum_Lit
     (Self => New_Node,
      Enum_Literal_Decl_F_Name => No_Bare_Ada_Node);
      New_Node.Synthetic_Char_Enum_Lit_Char_Symbol :=
         Block_Var_2;
      New_Node.Synthetic_Char_Enum_Lit_Enum_Type_Decl :=
         Cast_Result_3;
--# expr-done 3615
Is_Null_1 := New_Node = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Synthetic_Char_Enum_Lit;
else
As_Entity_1 := (Info => E_Info, Node => New_Node); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 3616
      Cast_Result_4 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   If_Result_3 := Cast_Result_4;
else
   If_Result_3 := Block_Var_3;
end if;
--# expr-done 3617
Let_Result := If_Result_3; 
--# end
--# expr-done 3618
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);
         Property_Result := Cast_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Corresponding_Char_Literal;
--# end
--# property-start '[dispatcher]BaseTypeDecl.root_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Root_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.root_type
                  Property_Result := Base_Type_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.root_type
                  Property_Result := Base_Subtype_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.root_type
                  Property_Result := Type_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Root_Type;
--# end
--# property-start BaseTypeDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7100
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Next_Part_For_Decl
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Match_0 : Internal_Entity_Single_Task_Type_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3625 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 3624 '<Match at ast.py:7102>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7102
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind sttd Match_0
--# expr-start 3620 '<FieldAccess for basic_decl_next_part_for_decl at ast.py:7106>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7106
--# expr-start 3619 '<FieldAccess for parent_basic_decl at ast.py:7106>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7106
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 3619
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 3620
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind ttd Match_1
--# expr-start 3621 '<FieldAccess for basic_decl_next_part_for_decl at ast.py:7107>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7107
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 3621
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Protected_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
--# expr-start 3623 '<Cast to BasicDecl.entity at ast.py:7108>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7108
--# expr-start 3622 '<FieldAccess for next_part at ast.py:7108>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7108
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3622
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-done 3623
Let_Result_2 := Cast_Result; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3624
Let_Result_3 := Match_Result; 
--# end
--# expr-done 3625
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BaseTypeDecl.shapes /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7111
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Shapes
  (Node : Bare_Base_Type_Decl
      ; Include_Discriminants : Boolean
         := True
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Shape_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_discriminants Include_Discriminants
   --# bind origin Origin
   Property_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_522 with Inline_Always;
            procedure Finalizer_Scope_2361 with Inline_Always;
            procedure Finalizer_Scope_2362 with Inline_Always;
            procedure Finalizer_Scope_2363 with Inline_Always;
            procedure Finalizer_Scope_2364 with Inline_Always;
      Block_Var : Internal_Entity_Base_Record_Def;
Block_Var_1 : Internal_Entity_Component_List;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Record_Def;
Block_Var_4 : Internal_Shape_Array_Access;
Block_Var_5 : Internal_Shape_Array_Access;
Block_Var_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Derived_Type_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Record_Def;
Fld : Internal_Entity_Base_Record_Def;
Fld_1 : Internal_Entity_Component_List;
Fld_2 : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Record_Def;
Scope_Result_1 : Internal_Entity_Base_Record_Def;
Result_Var_1 : Internal_Entity_Base_Record_Def;
Fld_5 : Internal_Shape_Array_Access;
Fld_6 : Internal_Shape_Array_Access;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Discriminant_Values_Array_Access;
Fld_10 : Internal_Discriminant_Values_Array_Access;
Concat_Result_1 : Internal_Discriminant_Values_Array_Access;
New_Struct : Internal_Shape := No_Shape;
Map_Result : Internal_Shape_Array_Access;
Map_Result_1 : Internal_Shape_Array_Access;
Scope_Result_2 : Internal_Shape_Array_Access;
Result_Var_2 : Internal_Shape_Array_Access;
Fld_11 : Internal_Entity_Type_Decl;
Fld_12 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_14 : Internal_Discriminant_Values_Array_Access;
New_Struct_1 : Internal_Shape := No_Shape;
Map_Result_2 : Internal_Shape_Array_Access;
If_Result : Internal_Shape_Array_Access;
Let_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_522 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_522;
            procedure Finalizer_Scope_2361 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2361;
            procedure Finalizer_Scope_2362 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2362;
            procedure Finalizer_Scope_2363 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2363;
            procedure Finalizer_Scope_2364 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2364;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3651 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3626 '<FieldAccess for record_def at ast.py:7167>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7167
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3626
Block_Var := Fld; 
--# bind rdef Block_Var
--# expr-start 3627 '<FieldAccess for components at ast.py:7168>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7168
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Component_List (Node => Block_Var.Node.Base_Record_Def_F_Components, Info => Block_Var.Info);
--# expr-done 3627
Block_Var_1 := Fld_1; 
--# bind comps Block_Var_1
--# expr-start 3629 '<Cast to DerivedTypeDef at ast.py:7170>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7170
--# expr-start 3628 '<FieldAccess for type_def at ast.py:7170>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7170
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_def
Fld_2 := Libadalang.Implementation.Component_List_P_Type_Def (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3628
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 3629
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Derived_Type_Def then
   --# scope-start
--# expr-start 3630 '<FieldAccess for base_type at ast.py:7170>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7170
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3630
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_2 := Result_Var; 
--# bind parent_type Block_Var_2
Var_Expr_1 := Block_Var_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3631 '<FieldAccess for record_def at ast.py:7172>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7172
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3631
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Record_Def;
end if;
Block_Var_3 := Result_Var_1; 
--# bind parent_record Block_Var_3
--# expr-start 3632 '<FieldAccess for shapes at ast.py:7174>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7174
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.shapes
Fld_5 := Libadalang.Implementation.Component_List_P_Shapes (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3632
Block_Var_4 := Fld_5; Inc_Ref (Block_Var_4);
--# bind own_shapes Block_Var_4
Var_Expr_2 := Block_Var_3;
if Var_Expr_2 /= No_Entity_Base_Record_Def then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 3642 '<Mapcat at ast.py:7179>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7179
   --# expr-start 3633 '<FieldAccess for shapes at ast.py:7179>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7179
      if Block_Var_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.shapes
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Shapes (Node => Block_Var_2.Node, Include_Discriminants => False, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 3633
      declare
         Map_Result_1_Vec : Internal_Shape_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Shape_Array_Access := Fld_6;
      begin
         for Item_134 of
               Collection.Items
         loop
   --# scope-start
               --# bind parent_shape Item_134
      --# expr-start 3641 '<Map at ast.py:7180>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7180
      declare
         Map_Result_Vec : Internal_Shape_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Shape_Array_Access := Block_Var_4;
      begin
         for Item_135 of
               Collection.Items
         loop
   --# scope-start
               --# bind own_shape Item_135
      --# expr-start 3640 '<New Shape at ast.py:7181>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7181
--# expr-start 3636 '<Concat at ast.py:7182>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7182
--# expr-start 3634 '<FieldAccess for components at ast.py:7182>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7182
Fld_7 := Item_134.Components;
Inc_Ref (Fld_7);
--# expr-done 3634
--# expr-start 3635 '<FieldAccess for components at ast.py:7183>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7183
Fld_8 := Item_135.Components;
Inc_Ref (Fld_8);
--# expr-done 3635
Concat_Result := Concat (Fld_7, Fld_8); 
--# expr-done 3636
--# expr-start 3639 '<Concat at ast.py:7186>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7186
--# expr-start 3637 '<FieldAccess for discriminants_values at ast.py:7185>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7185
Fld_9 := Item_134.Discriminants_Values;
Inc_Ref (Fld_9);
--# expr-done 3637
--# expr-start 3638 '<FieldAccess for discriminants_values at ast.py:7187>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7187
Fld_10 := Item_135.Discriminants_Values;
Inc_Ref (Fld_10);
--# expr-done 3638
Concat_Result_1 := Concat (Fld_9, Fld_10); 
--# expr-done 3639
Inc_Ref (Concat_Result);
Inc_Ref (Concat_Result_1);
New_Struct := (Components => Concat_Result, Discriminants_Values => Concat_Result_1); 
--# expr-done 3640
         declare
            Item_To_Append : constant Internal_Shape := New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2363;
         end loop;
      end;
         Map_Result := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3641
         for Item_To_Append of
               Map_Result.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2362;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 3642
Scope_Result_2 := Map_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2361;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var_4;
end if;
      Inc_Ref (Result_Var_2);
Block_Var_5 := Result_Var_2; Inc_Ref (Block_Var_5);
--# bind all_shapes Block_Var_5
--# expr-start 3644 '<FieldAccess for discriminants_list at ast.py:7195>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7195
--# expr-start 3643 '<FieldAccess for type_decl at ast.py:7195>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7195
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_11 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3643
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_11.Node, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 3644
Block_Var_6 := Fld_12; Inc_Ref (Block_Var_6);
--# bind discrs Block_Var_6
--# expr-start 3650 '<If at ast.py:7196>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7196
if Include_Discriminants then
   --# expr-start 3649 '<Map at ast.py:7198>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7198
      declare
         Map_Result_2_Vec : Internal_Shape_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Shape_Array_Access := Block_Var_5;
      begin
         for Item_136 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_136
      --# expr-start 3648 '<New Shape at ast.py:7199>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7199
--# expr-start 3646 '<Concat at ast.py:7200>' Concat_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7200
--# expr-start 3645 '<FieldAccess for components at ast.py:7200>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7200
Fld_13 := Item_136.Components;
Inc_Ref (Fld_13);
--# expr-done 3645
Concat_Result_2 := Concat (Block_Var_6, Fld_13); 
--# expr-done 3646
--# expr-start 3647 '<FieldAccess for discriminants_values at ast.py:7201>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7201
Fld_14 := Item_136.Discriminants_Values;
Inc_Ref (Fld_14);
--# expr-done 3647
Inc_Ref (Concat_Result_2);
Inc_Ref (Fld_14);
New_Struct_1 := (Components => Concat_Result_2, Discriminants_Values => Fld_14); 
--# expr-done 3648
         declare
            Item_To_Append : constant Internal_Shape := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2364;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 3649
   If_Result := Map_Result_2;
else
   If_Result := Block_Var_5;
end if;
      Inc_Ref (If_Result);
--# expr-done 3650
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3651
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_522;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_522;
                     Finalizer_Scope_2361;
                     Finalizer_Scope_2362;
                     Finalizer_Scope_2363;
                     Finalizer_Scope_2364;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Shapes;
--# end
--# property-start BaseTypeDecl.dottable_subps_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7207
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_F_Dottable_Subps_Env
  (Node : Bare_Base_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_523 with Inline_Always;
      Dyn_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_523 is
            begin
                     Dec_Ref (Dyn_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_523;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Dottable_Subps_Env then
         Property_Result := Self.Lf_Stg_Dottable_Subps_Env;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3653 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3652 '<DynamicLexicalEnv at ast.py:7213>' Dyn_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7213
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Base_Type_Decl_P_Dottable_Subps'Access, null, False); 
--# expr-done 3652
Let_Result := Dyn_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 3653
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_523;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_523;
            raise;
      end;
      Self.Lf_Present_Dottable_Subps_Env := True;
      Self.Lf_Stg_Dottable_Subps_Env := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Base_Type_Decl_F_Dottable_Subps_Env;
--# end
--# property-start BaseTypeDecl.dottable_subps /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7218
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Dottable_Subps
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_525 with Inline_Always;
            procedure Finalizer_Scope_2370 with Inline_Always;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Internal_Entity_Base_Package_Decl;
Var_Expr : Bare_Package_Body;
Var_Expr_1 : Bare_Declarative_Part;
Var_Expr_2 : Bare_Declarative_Part;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_4 : Internal_Entity_Package_Body;
Var_Expr_5 : Internal_Entity_Declarative_Part;
Var_Expr_6 : Internal_Entity_Base_Subp_Spec;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Bd : Internal_Entity_Basic_Decl;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result : Bare_Package_Body;
Bound_Imprecise_Fallback : Boolean;
As_Entity : Internal_Entity_Package_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Package_Body;
Fld_2 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Package_Decl;
Scope_Result_1 : Internal_Entity_Base_Package_Decl;
Fld_3 : Bare_Ada_Node;
Scope_Result_2 : Bare_Ada_Node;
Result_Var_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Package_Decl;
As_Entity_1 : Internal_Entity_Base_Package_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Package_Decl;
Result_Var_2 : Internal_Entity_Base_Package_Decl;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_2 : Boolean;
Not_Val : Boolean;
Is_Null_2 : Boolean;
Fld_7 : Internal_Entity_Public_Part;
Cast_Result_3 : Internal_Entity_Declarative_Part;
Fld_8 : Internal_Entity_Private_Part;
Cast_Result_4 : Internal_Entity_Declarative_Part;
Fld_9 : Internal_Entity_Package_Body;
Fld_10 : Internal_Entity_Declarative_Part;
Scope_Result_4 : Internal_Entity_Declarative_Part;
Result_Var_4 : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
Fld_11 : Internal_Entity_Ada_Node_List;
Scope_Result_5 : Internal_Entity_Ada_Node_List;
Result_Var_5 : Internal_Entity_Ada_Node_List;
Fld_12 : Bare_Ada_Node_List;
Fld_13 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity;
Item_387 : Internal_Entity;
Bare_Item_387 : Bare_Ada_Node;
Map_Result : Internal_Entity_Array_Access;
Map_Result_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Fld_14 : Internal_Entity_Defining_Name;
Fld_15 : Symbol_Type;
Fld_16 : Bare_Basic_Decl;
New_Struct : Internal_Metadata;
Cast_Result_6 : Bare_Ada_Node;
New_Struct_1 : Internal_Inner_Env_Assoc;
Let_Result : Internal_Inner_Env_Assoc;
Cast_Expr_1 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Basic_Decl;
Fld_17 : Internal_Entity_Base_Subp_Spec;
Scope_Result_6 : Internal_Entity_Base_Subp_Spec;
Result_Var_6 : Internal_Entity_Base_Subp_Spec;
Fld_18 : Internal_Entity_Base_Type_Decl;
Scope_Result_7 : Internal_Entity_Base_Type_Decl;
Result_Var_7 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_9 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result_2 : Internal_Inner_Env_Assoc_Array_Access;
If_Result_4 : Internal_Inner_Env_Assoc_Array_Access;
If_Result_5 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_525 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_525;
            procedure Finalizer_Scope_2370 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2370;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Dottable_Subps,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Inner_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3692 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 3654 '<FieldAccess for declarative_scope at ast.py:7225>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7225
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3654
Block_Var := Fld; 
--# bind scope Block_Var
--# expr-start 3656 '<Cast to PackageBody at ast.py:7227>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7227
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3655 '<parent at ast.py:7227>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7227
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_1.Parent; 
--# expr-done 3655
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Package_Body_Range
   then
      Cast_Result := Result_Var;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3656
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind body Var_Expr
--# expr-start 3660 '<Cast to BasePackageDecl at ast.py:7231>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7231
--# expr-start 3659 '<bind at ast.py:7228>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7228
Bound_Imprecise_Fallback := False; 
--# expr-start 3658 '<FieldAccess for previous_part at ast.py:7230>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7230
--# expr-start 3657 '<as_entity at ast.py:7230>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7230
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity_Package_Body;
else
As_Entity := (Info => E_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 3657
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Previous_Part (Node => If_Result.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => If_Result.Info);
--# end
--# expr-done 3658
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 3659
   if Dyn_Var_Bind_Result.Node = null
      or else Dyn_Var_Bind_Result.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Package_Decl
        (Node => Dyn_Var_Bind_Result.Node,
         Info => Dyn_Var_Bind_Result.Info);
   else
         Cast_Result_1 := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 3660
Scope_Result_1 := Cast_Result_1;
   --# end
   Result_Var_2 := Scope_Result_1;
else
   --# expr-start 3663 '<as_entity at ast.py:7232>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7232
--# expr-start 3662 '<Cast to BasePackageDecl at ast.py:7232>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7232
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3661 '<parent at ast.py:7232>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7232
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_2.Parent; 
--# expr-done 3661
Scope_Result_2 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
   if Result_Var_1 = null
      or else Result_Var_1.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_2 := Result_Var_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 3662
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Package_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 3663
   Result_Var_2 := If_Result_1;
end if;
Block_Var_1 := Result_Var_2; 
--# bind pkg Block_Var_1
--# expr-start 3691 '<Cond at ast.py:7235>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7235
--# expr-start 3667 '<Not at ast.py:7236>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7236
--# expr-start 3664 '<FieldAccess for is_tagged_type at ast.py:7236>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7236
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 3664
if Fld_4 then
   If_Result_2 := True;
else
--# expr-start 3665 '<FieldAccess for private_completion at ast.py:7238>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7238
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3665
Var_Expr_3 := Fld_5;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3666 '<FieldAccess for is_tagged_type at ast.py:7238>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7238
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Var_Expr_3.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 3666
Scope_Result_3 := Fld_6;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_2 := Result_Var_3;
end if;
Not_Val := not (If_Result_2); 
--# expr-done 3667
if Not_Val then
   If_Result_5 := No_Internal_Inner_Env_Assoc_Array_Type;
else
--# expr-start 3668 '<IsNull at ast.py:7241>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7241
Is_Null_2 := Block_Var_1.Node = null; 
--# expr-done 3668
if Is_Null_2 then
   If_Result_4 := No_Internal_Inner_Env_Assoc_Array_Type;
else
   --# expr-start 3690 '<FilterMap at ast.py:7250>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7250
   --# expr-start 3678 '<Mapcat at ast.py:7248>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7248
   --# expr-start 3675 '<ArrayLiteral at ast.py:7244>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7244
--# expr-start 3670 '<Cast to DeclarativePart at ast.py:7245>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7245
--# expr-start 3669 '<FieldAccess for public_part at ast.py:7245>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7245
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Public_Part (Node => Block_Var_1.Node.Base_Package_Decl_F_Public_Part, Info => Block_Var_1.Info);
--# expr-done 3669
      Cast_Result_3 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
--# expr-done 3670
--# expr-start 3672 '<Cast to DeclarativePart at ast.py:7246>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7246
--# expr-start 3671 '<FieldAccess for private_part at ast.py:7246>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7246
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Private_Part (Node => Block_Var_1.Node.Base_Package_Decl_F_Private_Part, Info => Block_Var_1.Info);
--# expr-done 3671
      Cast_Result_4 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
--# expr-done 3672
--# expr-start 3673 '<FieldAccess for body_part at ast.py:7247>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7247
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_9 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3673
Var_Expr_4 := Fld_9;
if Var_Expr_4 /= No_Entity_Package_Body then
   --# scope-start
--# expr-start 3674 '<FieldAccess for decls at ast.py:7247>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7247
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_4.Node.Package_Body_F_Decls, Info => Var_Expr_4.Info);
--# expr-done 3674
Scope_Result_4 := Fld_10;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Declarative_Part;
end if;
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Cast_Result_3, 2 => Cast_Result_4, 3 => Result_Var_4)); 
--# expr-done 3675
      declare
         Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Declarative_Part_Array_Access := Array_Lit;
      begin
         for Item_138 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_138
      --# expr-start 3677 '<TakeWhile at ???>' Map_Result None
Var_Expr_5 := Item_138;
if Var_Expr_5 /= No_Entity_Declarative_Part then
   --# scope-start
--# expr-start 3676 '<FieldAccess for decls at ast.py:7249>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7249
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_5.Node.Declarative_Part_F_Decls, Info => Var_Expr_5.Info);
--# expr-done 3676
Scope_Result_5 := Fld_11;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Ada_Node_List;
end if;
Fld_12 := Result_Var_5.Node;
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      if Fld_12 = null then
      Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_12;
      begin
         for Untyped_Item_387 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_387 := Untyped_Item_387; 
Is_Null_3 := Bare_Item_387 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity;
else
Fld_13 := Result_Var_5.Info;
As_Entity_2 := (Info => Fld_13, Node => Bare_Item_387); 
   If_Result_3 := As_Entity_2;
end if;
                  Item_387 := If_Result_3; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity := Item_387;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3677
         for Item_To_Append of
               Map_Result.Items
         loop
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2370;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 3678
      declare
         Map_Result_2_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result_1;
      begin
         for Item_137 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_137
         --# expr-start 3682 '<Eq at ast.py:7260>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7260
--# expr-start 3679 '<Cast to BasicDecl at ast.py:7260>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7260
Cast_Expr_1 := Item_137; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_7 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3679
Var_Expr_7 := Cast_Result_7;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3680 '<FieldAccess for subp_spec_or_null at ast.py:7261>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7261
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_7.Node, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 3680
Scope_Result_6 := Fld_17;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Base_Subp_Spec;
end if;
Var_Expr_6 := Result_Var_6;
if Var_Expr_6 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3681 '<FieldAccess for dottable_subp_of at ast.py:7261>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7261
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.dottable_subp_of
Fld_18 := Libadalang.Implementation.Base_Subp_Spec_P_Dottable_Subp_Of (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 3681
Scope_Result_7 := Fld_18;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := No_Entity_Base_Type_Decl;
end if;
      Cast_Result_8 := Create_Internal_Entity
        (Node => Result_Var_7.Node,
         Info => Result_Var_7.Info);
Cast_Expr_2 := Ent; 
      Cast_Result_9 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_8, Cast_Result_9); 
--# expr-done 3682
         if Is_Equiv then
      --# expr-start 3689 '<Let at ast.py:7251>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7251
--# scope-start
--# expr-start 3683 '<Cast to BasicDecl at ast.py:7252>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7252
Cast_Expr := Item_137; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3683
Bd := Cast_Result_5; 
--# bind bd Bd
--# expr-start 3688 '<New InnerEnvAssoc at ast.py:7252>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7252
--# expr-start 3685 '<FieldAccess for name_symbol at ast.py:7253>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7253
--# expr-start 3684 '<FieldAccess for defining_name at ast.py:7253>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7253
      if Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Bd.Node, E_Info => Bd.Info);
--# end
--# expr-done 3684
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_14.Node);
--# end
--# expr-done 3685
--# expr-start 3686 '<New Metadata at ast.py:7255>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7255
New_Struct := (Dottable_Subp => True, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3686
--# expr-start 3687 '<FieldAccess for node at ast.py:7254>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7254
Fld_16 := Bd.Node;
--# expr-done 3687
      Cast_Result_6 := Fld_16;
New_Struct_1 := (Key => Fld_15, Metadata => New_Struct, Value => Cast_Result_6); 
--# expr-done 3688
Let_Result := New_Struct_1; 
--# end
--# expr-done 3689
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := Let_Result;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 3690
   If_Result_4 := Map_Result_2;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 3691
Let_Result_1 := If_Result_5; Inc_Ref (Let_Result_1);
--# end
--# expr-done 3692
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_525;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_525;
                     Finalizer_Scope_2370;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Inner_Env_Assoc_Array_Access,
                        As_Internal_Inner_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Dottable_Subps;
--# end
--# property-start BaseTypeDecl.find_base_type_rebindings_among /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7265
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Base_Type_Rebindings_Among
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
      ; Base_Types : Internal_Entity_Base_Type_Decl_Array_Access
      ; Index : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Env_Rebindings
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   --# bind base_types Base_Types
   --# bind index Index
   --# bind origin Origin
   Property_Result : Env_Rebindings;
      Var_Expr : Env_Rebindings;
Get_Result : Internal_Entity_Base_Type_Decl;
Fld : Env_Rebindings;
Scope_Result : Env_Rebindings;
Arith_Result : Integer;
Fld_1 : Env_Rebindings;
Result_Var : Env_Rebindings;
Len : Integer;
Comp_Result : Boolean;
If_Result : Env_Rebindings;
Let_Result : Env_Rebindings;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3699 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3698 '<If at ast.py:7274>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7274
--# expr-start 3693 '<length at ast.py:7275>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7275
Len := Length (Base_Types); 
--# expr-done 3693
Comp_Result := Index >= Len; 
if Comp_Result then
   If_Result := null;
else
--# expr-start 3695 '<FieldAccess for find_base_type_rebindings at ast.py:7277>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7277
--# expr-start 3694 '<at at ast.py:7277>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7277
Get_Result := Get (Self, Base_Types, Index, True); 
--# expr-done 3694
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.find_base_type_rebindings
Fld := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings (Node => Get_Result.Node, Target => Target, Origin => Origin, E_Info => Get_Result.Info);
--# end
--# expr-done 3695
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 3697 '<FieldAccess for find_base_type_rebindings_among at ast.py:7278>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7278
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3696 '<Arithmetic + at ast.py:7279>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7279
Arith_Result := (Index + 1); 
--# expr-done 3696
--# property-call-start BaseTypeDecl.find_base_type_rebindings_among
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings_Among (Node => Ent.Node, Target => Target, Base_Types => Base_Types, Index => Arith_Result, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3697
   Result_Var := Fld_1;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 3698
Let_Result := If_Result; 
--# end
--# expr-done 3699
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Find_Base_Type_Rebindings_Among;
--# end
--# property-start BaseTypeDecl.find_base_type_rebindings /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7284
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Base_Type_Rebindings
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Env_Rebindings
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   --# bind origin Origin
   Property_Result : Env_Rebindings;
            procedure Finalizer_Scope_527 with Inline_Always;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Env_Rebindings;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Base_Type_Decl;
Is_Equal : Boolean;
If_Result : Env_Rebindings;
Let_Result : Env_Rebindings;
            procedure Finalizer_Scope_527 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_527;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Find_Base_Type_Rebindings,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Target);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Env_Rebindings;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3706 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3705 '<If at ast.py:7303>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7303
--# expr-start 3700 '<Eq at ast.py:7304>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7304
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Target; 
--# expr-done 3700
if Is_Equal then
   --# expr-start 3702 '<FieldAccess for rebindings at ast.py:7305>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7305
--# expr-start 3701 '<FieldAccess for info at ast.py:7305>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7305
Fld := Ent.Info;
--# expr-done 3701
Fld_1 := Fld.Rebindings;
--# expr-done 3702
   If_Result := Fld_1;
else
   --# expr-start 3704 '<FieldAccess for find_base_type_rebindings_among at ast.py:7306>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7306
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3703 '<FieldAccess for base_types at ast.py:7308>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7308
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3703
--# property-call-start BaseTypeDecl.find_base_type_rebindings_among
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings_Among (Node => Ent.Node, Target => Target, Base_Types => Fld_2, Index => 0, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3704
   If_Result := Fld_3;
end if;
--# expr-done 3705
Let_Result := If_Result; 
--# end
--# expr-done 3706
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_527;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_527;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Env_Rebindings,
                        As_Env_Rebindings => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Find_Base_Type_Rebindings;
--# end
--# property-start BaseTypeDecl.has_base_type_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7313
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Has_Base_Type_Impl
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   Property_Result : Boolean;
            procedure Finalizer_Scope_528 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
Cast_Result : Bare_Base_Type_Decl;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Boolean;
Quantifier_Result : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_528 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_528;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3713 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3709 '<Eq at ast.py:7320>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7320
--# expr-start 3708 '<FieldAccess for node at ast.py:7320>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7320
--# expr-start 3707 '<FieldAccess for canonical_type at ast.py:7320>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7320
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 3707
Fld_1 := Fld.Node;
--# expr-done 3708
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Target; 
--# expr-done 3709
if Is_Equal then
   If_Result := True;
else
   --# expr-start 3712 '<AnyQuantifier at ast.py:7321>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7321
   --# expr-start 3710 '<FieldAccess for base_types at ast.py:7321>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7321
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 3710
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_2;
      begin
         for Item_139 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_139
      --# expr-start 3711 '<FieldAccess for has_base_type_impl at ast.py:7322>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7322
      if Item_139.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.has_base_type_impl
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type_Impl (Node => Item_139.Node, Target => Target, E_Info => Item_139.Info);
--# end
--# expr-done 3711
      Quantifier_Result := Fld_3;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 3712
   If_Result := Quantifier_Result;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3713
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_528;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_528;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Has_Base_Type_Impl;
--# end
--# property-start BaseTypeDecl.has_base_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7326
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Has_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3719 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3718 '<FieldAccess for has_base_type_impl at ast.py:7336>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7336
--# expr-start 3714 '<FieldAccess for full_view at ast.py:7336>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7336
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3714
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3717 '<FieldAccess for node at ast.py:7337>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7337
--# expr-start 3716 '<FieldAccess for canonical_type at ast.py:7337>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7337
--# expr-start 3715 '<as_bare_entity at ast.py:7337>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7337
Is_Null := Target = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Target); 
   If_Result := As_Entity;
end if;
--# expr-done 3715
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => If_Result.Node, Origin => No_Bare_Ada_Node, E_Info => If_Result.Info);
--# end
--# expr-done 3716
Fld_2 := Fld_1.Node;
--# expr-done 3717
--# property-call-start BaseTypeDecl.has_base_type_impl
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type_Impl (Node => Fld.Node, Target => Fld_2, E_Info => Fld.Info);
--# end
--# expr-done 3718
Let_Result := Fld_3; 
--# end
--# expr-done 3719
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Has_Base_Type;
--# end
--# property-start BaseTypeDecl._env_mappings_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5924
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_2
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1528 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1528 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1528;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3722 '<New EnvAssoc at ast.py:5924>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5924
--# expr-start 3720 '<New DesignatedEnv at ast.py:5924>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5924
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 3720
--# expr-start 3721 '<FieldAccess for name_symbol at ast.py:5924>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5924
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3721
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 3722
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1528;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1528;
            raise;
      end;
   return Property_Result;
end Env_Mappings_2;
--# end
--# property-start BaseTypeDecl.primitives_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6031
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_433 with Inline_Always;
      Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_433 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_433;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3723 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := Empty_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 3723
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_433;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_433;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Primitives_Env;
--# end
--# property-start BaseTypeDecl.is_record_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6035
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Record_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Record_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3727 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3726 '<Not at ast.py:6043>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6043
--# expr-start 3725 '<IsNull at ast.py:6043>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6043
--# expr-start 3724 '<FieldAccess for record_def at ast.py:6043>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3724
Is_Null := Fld.Node = null; 
--# expr-done 3725
Not_Val := not (Is_Null); 
--# expr-done 3726
Let_Result := Not_Val; 
--# end
--# expr-done 3727
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Record_Type;
--# end
--# property-start BaseTypeDecl.is_task_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6068
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Task_Type
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Task_Type;
--# end
--# property-start BaseTypeDecl.is_real_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6070
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Real_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Real_Type;
--# end
--# property-start BaseTypeDecl.is_float_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6074
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Float_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Float_Type;
--# end
--# property-start BaseTypeDecl.is_fixed_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6078
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseTypeDecl.is_enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6090
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Enum_Type;
--# end
--# property-start BaseTypeDecl.is_classwide /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6095
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Classwide
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Classwide;
--# end
--# property-start BaseTypeDecl.is_access_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6097
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Access_Type;
--# end
--# property-start BaseTypeDecl.has_ud_indexing /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6110
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start BaseTypeDecl.constant_indexing_fns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6114
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Basic_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start BaseTypeDecl.variable_indexing_fns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6121
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Basic_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start BaseTypeDecl.get_imp_deref /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6129
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Expr;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start BaseTypeDecl.access_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6269
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Access_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Access_Def;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Access_Def;
--# end
--# property-start BaseTypeDecl.is_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6271
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Char_Type;
--# end
--# property-start BaseTypeDecl.classwide_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6284
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Classwide_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
      Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Bare_Classwide_Type_Decl;
As_Entity : Internal_Entity_Classwide_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Classwide_Type_Decl;
Fld_1 : Boolean;
If_Result_1 : Internal_Entity_Classwide_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3732 '<bind at ast.py:6285>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6285
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 3731 '<If at ast.py:6285>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6285
--# expr-start 3728 '<FieldAccess for is_tagged_type at ast.py:6286>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6286
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 3728
if Fld_1 then
   --# expr-start 3730 '<as_entity at ast.py:6287>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6287
--# expr-start 3729 '<FieldAccess for classwide_type_node at ast.py:6287>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6287
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.classwide_type_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Classwide_Type_Node (Node => Self);
--# end
--# expr-done 3729
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Classwide_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 3730
   If_Result_1 := If_Result;
else
   If_Result_1 := No_Entity_Classwide_Type_Decl;
end if;
--# expr-done 3731
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 3732
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Classwide_Type;
--# end
--# property-start BaseTypeDecl.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6334
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Discrete_Range
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Discrete_Range;
      Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3733 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Discrete_Range; 
--# end
--# expr-done 3733
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Discrete_Range;
--# end
--# property-start BaseTypeDecl.is_discrete_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6403
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3737 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3734 '<FieldAccess for is_int_type at ast.py:6408>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6408
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3734
if Fld then
   If_Result := True;
else
   --# expr-start 3735 '<FieldAccess for is_enum_type at ast.py:6408>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6408
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3735
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 3736 '<FieldAccess for is_char_type at ast.py:6408>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6408
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3736
   If_Result_1 := Fld_2;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 3737
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseTypeDecl.is_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6414
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Int_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3738 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 3738
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Int_Type;
--# end
--# property-start BaseTypeDecl.accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6476
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3739 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 3739
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Accessed_Type;
--# end
--# property-start BaseTypeDecl.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6541
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseTypeDecl.base_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6558
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Base_Type;
--# end
--# property-start BaseTypeDecl.base_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6564
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Types
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_486 with Inline_Always;
            procedure Finalizer_Scope_2175 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Singleton : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_486 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_486;
            procedure Finalizer_Scope_2175 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2175;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3742 '<Concat at ast.py:6566>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6566
--# expr-start 3740 '<FieldAccess for base_type at ast.py:6565>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6565
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3740
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
                Singleton := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2175;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 3741 '<FieldAccess for base_interfaces at ast.py:6566>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6566
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Interfaces (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3741
Concat_Result := Concat (Result_Var, Fld_1); 
--# expr-done 3742
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_486;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_486;
                     Finalizer_Scope_2175;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Base_Types;
--# end
--# property-start BaseTypeDecl.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6572
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Interfaces
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Base_Interfaces;
--# end
--# property-start BaseTypeDecl.record_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6591
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Record_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Record_Def;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Record_Def;
--# end
--# property-start BaseTypeDecl.array_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6592
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Array_Type_Def;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Array_Def;
--# end
--# property-start BaseTypeDecl.is_iterable_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6700
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseTypeDecl.is_interface_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6708
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Interface_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Def;
Is_A : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Subtype_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Scope_Result_3 : Boolean;
Result_Var : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3749 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 3743 '<FieldAccess for full_view at ast.py:6714>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6714
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3743
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3748 '<Match at ast.py:6714>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6714
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind td Match_0
--# expr-start 3745 '<IsA InterfaceTypeDef at ast.py:6715>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6715
--# expr-start 3744 '<FieldAccess for type_def at ast.py:6715>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6715
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_0.Node.Type_Decl_F_Type_Def, Info => Match_0.Info);
--# expr-done 3744
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Interface_Type_Def_Range; 
--# expr-done 3745
Let_Result := Is_A; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind sb Match_1
--# expr-start 3747 '<FieldAccess for is_interface_type at ast.py:6716>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6716
--# expr-start 3746 '<FieldAccess for get_type at ast.py:6716>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6716
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 3746
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_interface_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Interface_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 3747
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3748
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := False;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 3749
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Interface_Type;
--# end
--# property-start BaseTypeDecl.iterable_comp_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6720
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3750 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 3750
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseTypeDecl.canonical_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6876
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Canonical_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3754 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3753 '<bind at ast.py:6883>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6883
Bound_Imprecise_Fallback := False; 
--# expr-start 3752 '<Cast to BaseTypeDecl at ast.py:6884>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6884
--# expr-start 3751 '<FieldAccess for canonical_part at ast.py:6884>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6884
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3751
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3752
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 3753
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3754
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Canonical_Type;
--# end
--# property-start BaseTypeDecl.previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6910
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Previous_Part
  (Node : Bare_Base_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2331 with Inline_Always;
            procedure Finalizer_Scope_2332 with Inline_Always;
      Var_Expr : Bare_Defining_Name;
Var_Expr_1 : Internal_Entity_Array_Access;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Fld : Bare_Defining_Name;
Fld_1 : Internal_Entity;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Symbol_Type;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Env_Get_Result : Internal_Entity_Array_Access;
Fld_4 : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Find_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2331 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_2331;
            procedure Finalizer_Scope_2332 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_2332;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3768 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3767 '<If at ast.py:6917>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6917
--# expr-start 3755 '<FieldAccess for is_generic_formal at ast.py:6918>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6918
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Self);
--# end
--# expr-done 3755
if Fld_6 then
   If_Result_3 := No_Entity_Base_Type_Decl;
else
--# expr-start 3756 '<FieldAccess for name at ast.py:6925>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6925
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 3756
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind type_name Var_Expr
--# expr-start 3766 '<Cast to BaseTypeDecl at ast.py:6937>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6937
--# expr-start 3760 '<EnvGet at ast.py:6928>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6928
--# expr-start 3758 '<FieldAccess for immediate_declarative_region at ast.py:6928>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6928
--# expr-start 3757 '<FieldAccess for semantic_parent at ast.py:6928>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6928
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3757
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.immediate_declarative_region'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Immediate_Declarative_Region (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 3758
--# expr-start 3759 '<FieldAccess for name_symbol at ast.py:6929>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6929
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr);
--# end
--# expr-done 3759
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_2, Key => Fld_3, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True), From => Cast_Result)); 
--# expr-done 3760
Var_Expr_1 := Env_Get_Result;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 3765 '<Find at ast.py:6932>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6932
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_1;
      begin
         for Item_132 of
               Collection.Items
         loop
   --# scope-start
               --# bind pp Item_132
--# expr-start 3761 '<FieldAccess for is_in_private_part at ast.py:6933>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6933
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3761
if Fld_4 then
--# expr-start 3762 '<Cast to BaseTypeDecl at ast.py:6934>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6934
Cast_Expr_1 := Item_132; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3762
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3763 '<FieldAccess for is_private at ast.py:6934>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6934
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 3763
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := True;
else
if Go_To_Incomplete then
   --# expr-start 3764 '<IsA IncompleteTypeDecl at ast.py:6936>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:6936
Is_A := Item_132.Node /= null 
and then Item_132.Node.Kind in Ada_Incomplete_Type_Decl_Range; 
--# expr-done 3764
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
      if If_Result_2 then
         Find_Result := Item_132; 
      end if;
   --# end
               exit when If_Result_2;
         end loop;
      end;
--# expr-done 3765
Scope_Result_1 := Find_Result;
   --# end
      Finalizer_Scope_2332;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3766
Scope_Result_2 := Cast_Result_2;
   --# end
      Finalizer_Scope_2331;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   If_Result_3 := Result_Var_2;
end if;
--# expr-done 3767
Let_Result := If_Result_3; 
--# end
--# expr-done 3768
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2331;
                     Finalizer_Scope_2332;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Previous_Part;
--# end
--# property-start BaseTypeDecl.is_private /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7020
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Private
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Private;
--# end
--# property-start BaseTypeDecl.root_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7090
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Root_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Ent;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Root_Type;
--# end
function Base_Type_Decl_P_Dottable_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Base_Type_Decl
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Internal_Inner_Env_Assoc_Array_Access :=
         Base_Type_Decl_P_Dottable_Subps (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Base_Subtype_Decl
   --
      procedure Initialize_Fields_For_Base_Subtype_Decl
        (Self : Bare_Base_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Base_Subtype_Decl;
--# property-start BaseSubtypeDecl.from_type_bound /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8913
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_From_Type_Bound
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3772 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3771 '<bind at ast.py:8919>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8919
--# expr-start 3769 '<FieldAccess for origin_node at ast.py:8919>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8919
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3769
Bound_Origin := Fld; 
--# expr-start 3770 '<FieldAccess for get_type at ast.py:8919>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8919
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 3770
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 3771
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3772
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_From_Type_Bound;
--# end
--# property-start '[dispatcher]BaseSubtypeDecl.get_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subtype_Decl_P_Get_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Subtype_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl =>
                  --# property-call-start DiscreteBaseSubtypeDecl.get_type
                  Property_Result := Discrete_Base_Subtype_Decl_P_Get_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.get_type
                  Property_Result := Subtype_Decl_P_Get_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Subtype_Decl_P_Get_Type;
--# end
--# property-start BaseSubtypeDecl.primitives_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8931
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Primitives_Env
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_676 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_676 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_676;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3774 '<FieldAccess for primitives_env at ast.py:8931>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8931
--# expr-start 3773 '<FieldAccess for from_type_bound at ast.py:8931>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8931
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3773
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3774
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_676;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_676;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Primitives_Env;
--# end
--# property-start BaseSubtypeDecl.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8933
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Array_Ndims
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3776 '<FieldAccess for array_ndims at ast.py:8933>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8933
--# expr-start 3775 '<FieldAccess for get_type at ast.py:8933>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8933
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3775
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3776
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Array_Ndims;
--# end
--# property-start BaseSubtypeDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8934
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Defining_Env
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_678 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_678 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_678;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3778 '<FieldAccess for defining_env at ast.py:8934>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8934
--# expr-start 3777 '<FieldAccess for get_type at ast.py:8934>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8934
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3777
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 3778
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_678;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_678;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Defining_Env;
--# end
--# property-start BaseSubtypeDecl.canonical_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8936
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Canonical_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3780 '<FieldAccess for canonical_type at ast.py:8936>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8936
--# expr-start 3779 '<FieldAccess for get_type at ast.py:8936>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8936
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3779
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3780
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Canonical_Type;
--# end
--# property-start BaseSubtypeDecl.record_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8937
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Record_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3782 '<FieldAccess for record_def at ast.py:8937>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8937
--# expr-start 3781 '<FieldAccess for get_type at ast.py:8937>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8937
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3781
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3782
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Record_Def;
--# end
--# property-start BaseSubtypeDecl.accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8938
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Accessed_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3784 '<FieldAccess for accessed_type at ast.py:8938>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8938
--# expr-start 3783 '<FieldAccess for get_type at ast.py:8938>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8938
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3783
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3784
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Accessed_Type;
--# end
--# property-start BaseSubtypeDecl.is_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8939
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Int_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3786 '<FieldAccess for is_int_type at ast.py:8939>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8939
--# expr-start 3785 '<FieldAccess for get_type at ast.py:8939>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8939
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3785
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3786
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Int_Type;
--# end
--# property-start BaseSubtypeDecl.is_discrete_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8940
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3788 '<FieldAccess for is_discrete_type at ast.py:8940>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8940
--# expr-start 3787 '<FieldAccess for get_type at ast.py:8940>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8940
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3787
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3788
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseSubtypeDecl.is_real_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8942
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Real_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3790 '<FieldAccess for is_real_type at ast.py:8942>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8942
--# expr-start 3789 '<FieldAccess for get_type at ast.py:8942>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8942
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3789
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3790
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Real_Type;
--# end
--# property-start BaseSubtypeDecl.is_float_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8943
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Float_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3792 '<FieldAccess for is_float_type at ast.py:8943>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8943
--# expr-start 3791 '<FieldAccess for get_type at ast.py:8943>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8943
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3791
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3792
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Float_Type;
--# end
--# property-start BaseSubtypeDecl.is_fixed_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8944
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3794 '<FieldAccess for is_fixed_point at ast.py:8944>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8944
--# expr-start 3793 '<FieldAccess for get_type at ast.py:8944>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8944
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3793
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3794
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseSubtypeDecl.is_enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8945
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3796 '<FieldAccess for is_enum_type at ast.py:8945>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8945
--# expr-start 3795 '<FieldAccess for get_type at ast.py:8945>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8945
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3795
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3796
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Enum_Type;
--# end
--# property-start BaseSubtypeDecl.is_access_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8946
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Access_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3798 '<FieldAccess for is_access_type at ast.py:8946>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8946
--# expr-start 3797 '<FieldAccess for get_type at ast.py:8946>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8946
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3797
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3798
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Access_Type;
--# end
--# property-start BaseSubtypeDecl.access_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8947
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Access_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Access_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3800 '<FieldAccess for access_def at ast.py:8947>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8947
--# expr-start 3799 '<FieldAccess for get_type at ast.py:8947>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8947
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3799
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3800
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Access_Def;
--# end
--# property-start BaseSubtypeDecl.is_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8948
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Char_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3802 '<FieldAccess for is_char_type at ast.py:8948>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8948
--# expr-start 3801 '<FieldAccess for get_type at ast.py:8948>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8948
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3801
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3802
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Char_Type;
--# end
--# property-start BaseSubtypeDecl.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8949
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3804 '<FieldAccess for is_tagged_type at ast.py:8949>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8949
--# expr-start 3803 '<FieldAccess for get_type at ast.py:8949>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8949
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3803
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3804
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseSubtypeDecl.base_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8950
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3806 '<FieldAccess for base_type at ast.py:8950>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8950
--# expr-start 3805 '<FieldAccess for get_type at ast.py:8950>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8950
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3805
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3806
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Base_Type;
--# end
--# property-start BaseSubtypeDecl.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8951
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Interfaces
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_693 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_693 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_693;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3808 '<FieldAccess for base_interfaces at ast.py:8951>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8951
--# expr-start 3807 '<FieldAccess for get_type at ast.py:8951>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8951
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3807
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Interfaces (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3808
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_693;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_693;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Base_Interfaces;
--# end
--# property-start BaseSubtypeDecl.base_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8952
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Types
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_694 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_694 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_694;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3810 '<FieldAccess for base_types at ast.py:8952>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8952
--# expr-start 3809 '<FieldAccess for get_type at ast.py:8952>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8952
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3809
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3810
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_694;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_694;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Base_Types;
--# end
--# property-start BaseSubtypeDecl.array_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8953
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Array_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3812 '<FieldAccess for array_def at ast.py:8953>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8953
--# expr-start 3811 '<FieldAccess for get_type at ast.py:8953>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8953
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3811
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3812
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Array_Def;
--# end
--# property-start BaseSubtypeDecl.is_classwide /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8954
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Classwide
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3814 '<FieldAccess for is_classwide at ast.py:8954>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8954
--# expr-start 3813 '<FieldAccess for from_type_bound at ast.py:8954>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8954
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3813
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3814
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Classwide;
--# end
--# property-start BaseSubtypeDecl.is_iterable_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8955
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3816 '<FieldAccess for is_iterable_type at ast.py:8955>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8955
--# expr-start 3815 '<FieldAccess for get_type at ast.py:8955>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8955
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3815
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3816
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseSubtypeDecl.iterable_comp_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8956
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3818 '<FieldAccess for iterable_comp_type at ast.py:8956>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8956
--# expr-start 3817 '<FieldAccess for get_type at ast.py:8956>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8956
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3817
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3818
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseSubtypeDecl.is_record_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8957
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Record_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3820 '<FieldAccess for is_record_type at ast.py:8957>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8957
--# expr-start 3819 '<FieldAccess for get_type at ast.py:8957>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8957
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3819
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3820
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Record_Type;
--# end
--# property-start BaseSubtypeDecl.is_private /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8958
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Private
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3822 '<FieldAccess for is_private at ast.py:8958>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8958
--# expr-start 3821 '<FieldAccess for from_type_bound at ast.py:8958>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8958
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3821
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3822
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Private;
--# end
--# property-start BaseSubtypeDecl.root_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8959
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Root_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3824 '<FieldAccess for root_type at ast.py:8959>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8959
--# expr-start 3823 '<FieldAccess for get_type at ast.py:8959>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8959
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3823
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3824
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Root_Type;
--# end
--# property-start BaseSubtypeDecl.has_ud_indexing /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8961
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3826 '<FieldAccess for has_ud_indexing at ast.py:8962>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8962
--# expr-start 3825 '<FieldAccess for from_type_bound at ast.py:8962>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8962
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3825
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3826
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Has_Ud_Indexing;
--# end
--# property-start BaseSubtypeDecl.constant_indexing_fns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8964
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_703 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_703 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_703;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3828 '<FieldAccess for constant_indexing_fns at ast.py:8965>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8965
--# expr-start 3827 '<FieldAccess for from_type_bound at ast.py:8965>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8965
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3827
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3828
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_703;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_703;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start BaseSubtypeDecl.variable_indexing_fns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8967
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_704 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_704 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_704;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3830 '<FieldAccess for variable_indexing_fns at ast.py:8968>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8968
--# expr-start 3829 '<FieldAccess for from_type_bound at ast.py:8968>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8968
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3829
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3830
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_704;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_704;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start BaseSubtypeDecl.discriminants_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8971
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Discriminants_List
  (Node : Bare_Base_Subtype_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_705 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_705 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_705;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3833 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3832 '<FieldAccess for discriminants_list at ast.py:8975>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8975
--# expr-start 3831 '<FieldAccess for get_type at ast.py:8975>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8975
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3831
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3832
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3833
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_705;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_705;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Discriminants_List;
--# end
   --
   --  Primitives for Bare_Discrete_Base_Subtype_Decl
   --
      procedure Initialize_Fields_For_Discrete_Base_Subtype_Decl
        (Self : Bare_Discrete_Base_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Subtype_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Discrete_Base_Subtype_Decl;
--# property-start DiscreteBaseSubtypeDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9012
pragma Warnings (Off, "is not referenced");
 function Discrete_Base_Subtype_Decl_P_Is_Static_Decl
  (Node : Bare_Discrete_Base_Subtype_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Discrete_Base_Subtype_Decl  := Bare_Discrete_Base_Subtype_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discrete_Base_Subtype_Decl_P_Is_Static_Decl;
--# end
--# property-start DiscreteBaseSubtypeDecl.get_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9016
pragma Warnings (Off, "is not referenced");
 function Discrete_Base_Subtype_Decl_P_Get_Type
  (Node : Bare_Discrete_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Discrete_Base_Subtype_Decl  := Bare_Discrete_Base_Subtype_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3836 '<as_entity at ast.py:9017>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9017
--# expr-start 3835 '<Cast to BaseTypeDecl at ast.py:9017>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9017
--# expr-start 3834 '<parent at ast.py:9017>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9017
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3834
   if Fld = null
      or else Fld.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 3835
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 3836
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discrete_Base_Subtype_Decl_P_Get_Type;
--# end
   --
   --  Primitives for Bare_Subtype_Decl
   --
      procedure Initialize_Fields_For_Subtype_Decl
        (Self : Bare_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Subtype_Decl_F_Subtype : Bare_Subtype_Indication
         ; Subtype_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subtype_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Subtype_Decl_F_Subtype := Subtype_Decl_F_Subtype;
            Self.Subtype_Decl_F_Aspects := Subtype_Decl_F_Aspects;
      end Initialize_Fields_For_Subtype_Decl;
   function Subtype_Decl_F_Subtype
     (Node : Bare_Subtype_Decl) return Bare_Subtype_Indication
   is
   begin
         return Node.Subtype_Decl_F_Subtype;
   end;
--# property-start SubtypeDecl.get_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8985
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Get_Type
  (Node : Bare_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Subtype_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3841 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 3840 '<Match at ast.py:8987>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8987
--# expr-start 3838 '<FieldAccess for designated_type at ast.py:8987>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8987
--# expr-start 3837 '<FieldAccess for subtype at ast.py:8987>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8987
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 3837
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3838
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 3839 '<FieldAccess for get_type at ast.py:8988>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8988
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 3839
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3840
Let_Result_2 := Match_Result; 
--# end
--# expr-done 3841
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Get_Type;
--# end
--# property-start SubtypeDecl.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8992
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Discrete_Range
  (Node : Bare_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3844 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3843 '<FieldAccess for discrete_range at ast.py:8994>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8994
--# expr-start 3842 '<FieldAccess for subtype at ast.py:8994>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8994
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 3842
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3843
Let_Result := Fld_1; 
--# end
--# expr-done 3844
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Discrete_Range;
--# end
--# property-start SubtypeDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8996
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Xref_Equation
  (Node : Bare_Subtype_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_708 with Inline_Always;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_708 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_708;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3847 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3846 '<FieldAccess for sub_equation at ast.py:8998>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8998
--# expr-start 3845 '<FieldAccess for subtype at ast.py:8998>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8998
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 3845
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3846
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3847
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_708;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_708;
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Xref_Equation;
--# end
--# property-start SubtypeDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9000
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Is_Static_Decl
  (Node : Bare_Subtype_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3849 '<FieldAccess for is_static_subtype at ast.py:9000>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9000
--# expr-start 3848 '<FieldAccess for subtype at ast.py:9000>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9000
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 3848
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 3849
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Is_Static_Decl;
--# end
--# property-start SubtypeDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9001
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Xref_Entry_Point
  (Node : Bare_Subtype_Decl
  )
   return Boolean
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Classwide_Type_Decl
   --
      procedure Initialize_Fields_For_Classwide_Type_Decl
        (Self : Bare_Classwide_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Classwide_Type_Decl;
--# property-start ClasswideTypeDecl.type_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7352
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Type_Decl
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3851 '<Cast to BaseTypeDecl at ast.py:7352>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7352
--# expr-start 3850 '<parent at ast.py:7352>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7352
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3850
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3851
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Type_Decl;
--# end
--# property-start ClasswideTypeDecl.is_classwide /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7354
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Classwide
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Classwide;
--# end
--# property-start ClasswideTypeDecl.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7356
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start ClasswideTypeDecl.base_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7357
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Base_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Base_Type;
--# end
--# property-start ClasswideTypeDecl.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7358
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Base_Interfaces
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Base_Interfaces;
--# end
--# property-start ClasswideTypeDecl.record_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7359
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Record_Def
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3853 '<FieldAccess for record_def at ast.py:7359>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7359
--# expr-start 3852 '<FieldAccess for type_decl at ast.py:7359>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7359
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3852
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3853
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Record_Def;
--# end
--# property-start ClasswideTypeDecl.classwide_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7360
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Classwide_Type
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Ent;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Classwide_Type;
--# end
--# property-start ClasswideTypeDecl.is_iterable_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7361
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3855 '<FieldAccess for is_iterable_type at ast.py:7361>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7361
--# expr-start 3854 '<FieldAccess for type_decl at ast.py:7361>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7361
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3854
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3855
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start ClasswideTypeDecl.iterable_comp_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7362
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3857 '<FieldAccess for iterable_comp_type at ast.py:7362>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7362
--# expr-start 3856 '<FieldAccess for type_decl at ast.py:7362>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7362
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3856
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3857
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start ClasswideTypeDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7363
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Defining_Env
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_539 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_539 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_539;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3859 '<FieldAccess for defining_env at ast.py:7363>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7363
--# expr-start 3858 '<FieldAccess for type_decl at ast.py:7363>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7363
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3858
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 3859
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_539;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_539;
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Defining_Env;
--# end
--# property-start ClasswideTypeDecl.is_private /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7364
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Private
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3861 '<FieldAccess for is_private at ast.py:7364>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7364
--# expr-start 3860 '<FieldAccess for type_decl at ast.py:7364>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7364
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3860
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3861
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Private;
--# end
--# property-start ClasswideTypeDecl.is_in_private_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7365
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_In_Private_Part
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3863 '<FieldAccess for is_in_private_part at ast.py:7365>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7365
--# expr-start 3862 '<FieldAccess for type_decl at ast.py:7365>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7365
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3862
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3863
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_In_Private_Part;
--# end
--# property-start ClasswideTypeDecl.get_aspect_assoc /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7367
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Classwide_Type_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Aspect_Assoc;
Let_Result : Internal_Entity_Aspect_Assoc;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3866 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3865 '<FieldAccess for get_aspect_assoc at ast.py:7369>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7369
--# expr-start 3864 '<FieldAccess for type_decl at ast.py:7369>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7369
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3864
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Fld.Node, Name => Name, E_Info => Fld.Info);
--# end
--# expr-done 3865
Let_Result := Fld_1; 
--# end
--# expr-done 3866
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start ClasswideTypeDecl.is_interface_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7371
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Interface_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3868 '<FieldAccess for is_interface_type at ast.py:7371>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7371
--# expr-start 3867 '<FieldAccess for type_decl at ast.py:7371>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7371
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3867
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_interface_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Interface_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3868
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Interface_Type;
--# end
--# property-start ClasswideTypeDecl.discriminants_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7373
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Discriminants_List
  (Node : Bare_Classwide_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_544 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_544 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_544;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3871 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3870 '<FieldAccess for discriminants_list at ast.py:7377>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7377
--# expr-start 3869 '<FieldAccess for type_decl at ast.py:7377>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7377
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3869
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3870
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3871
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_544;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_544;
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Discriminants_List;
--# end
--# property-start ClasswideTypeDecl.previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7379
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Previous_Part
  (Node : Bare_Classwide_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Result_Var : Internal_Entity_Classwide_Type_Decl;
Let_Result : Internal_Entity_Classwide_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Classwide_Type_Decl_Classwide_Type_Decl_P_Previous_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Classwide_Type_Decl,
                                As_Bare_Classwide_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Go_To_Incomplete);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 3875 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3873 '<FieldAccess for previous_part at ast.py:7382>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7382
--# expr-start 3872 '<FieldAccess for type_decl at ast.py:7382>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7382
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3872
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Fld.Node, Go_To_Incomplete => Go_To_Incomplete, E_Info => Fld.Info);
--# end
--# expr-done 3873
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 3874 '<FieldAccess for classwide_type at ast.py:7383>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7383
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3874
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Classwide_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3875
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Classwide_Type_Decl_P_Previous_Part;
--# end
--# property-start ClasswideTypeDecl.canonical_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7386
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Canonical_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Classwide_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3877 '<FieldAccess for canonical_type at ast.py:7386>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7386
--# expr-start 3876 '<FieldAccess for type_decl at ast.py:7386>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7386
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3876
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3877
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 3878 '<FieldAccess for classwide_type at ast.py:7390>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7390
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3878
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Classwide_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Scope_Result.Node,
         Info => Scope_Result.Info);
   Result_Var := Cast_Result;
else
   Result_Var := Var_Expr;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Canonical_Type;
--# end
   --
   --  Primitives for Bare_Incomplete_Type_Decl
   --
      procedure Incomplete_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Incomplete_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_225 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:22216");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_226 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Incomplete_Type_Decl
        (Self : Bare_Incomplete_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Incomplete_Type_Decl_F_Discriminants := Incomplete_Type_Decl_F_Discriminants;
      end Initialize_Fields_For_Incomplete_Type_Decl;
   function Incomplete_Type_Decl_F_Discriminants
     (Node : Bare_Incomplete_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Incomplete_Type_Decl_F_Discriminants;
   end;
--# property-start IncompleteTypeDecl.find_next_part_in /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22201
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Find_Next_Part_In
  (Node : Bare_Incomplete_Type_Decl
      ; Decl_Part : Internal_Entity_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
     Ent : Internal_Entity_Incomplete_Type_Decl :=
       Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind decl_part Decl_Part
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type;
Fld_5 : Boolean;
Cast_Expr_1 : Internal_Entity_Incomplete_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Item_140 : Internal_Entity;
Bare_Item_140 : Bare_Ada_Node;
Find_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3888 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3887 '<Cast to BaseTypeDecl at ast.py:22213>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22213
--# expr-start 3886 '<Find at ast.py:22208>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22208
   --# expr-start 3879 '<FieldAccess for decls at ast.py:22208>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22208
      if Decl_Part.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Decl_Part.Node.Declarative_Part_F_Decls, Info => Decl_Part.Info);
--# expr-done 3879
Fld_1 := Fld.Node;
      Find_Result := No_Entity;
      if Fld_1 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_140 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_140 := Untyped_Item_140; 
Is_Null := Bare_Item_140 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_140); 
   If_Result := As_Entity;
end if;
                  Item_140 := If_Result; 
   --# scope-start
               --# bind t Item_140
--# expr-start 3880 '<Cast to BaseTypeDecl at ast.py:22209>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22209
Cast_Expr := Item_140; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3880
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind btd Var_Expr
--# expr-start 3883 '<FieldAccess for name_is at ast.py:22211>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22211
--# expr-start 3881 '<FieldAccess for name at ast.py:22211>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22211
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Defining_Name (Node => Var_Expr.Node.Base_Type_Decl_F_Name, Info => Var_Expr.Info);
--# expr-done 3881
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3882 '<FieldAccess for name_symbol at ast.py:22211>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22211
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 3882
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Fld_4);
--# end
--# expr-done 3883
if Fld_5 then
   --# expr-start 3885 '<Not at ast.py:22211>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22211
--# expr-start 3884 '<Eq at ast.py:22211>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22211
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 3884
Not_Val := not (Is_Equiv); 
--# expr-done 3885
   If_Result_1 := Not_Val;
else
   If_Result_1 := False;
end if;
Scope_Result := If_Result_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_140; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
      end if;
--# expr-done 3886
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3887
Let_Result := Cast_Result_3; 
--# end
--# expr-done 3888
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Incomplete_Type_Decl_P_Find_Next_Part_In;
--# end
--# property-start IncompleteTypeDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22220
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Defining_Env
  (Node : Bare_Incomplete_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1512 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1512 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_1512;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3892 '<env_group at ast.py:22221>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22221
--# expr-start 3891 '<ArrayLiteral at ast.py:22221>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22221
--# expr-start 3889 '<FieldAccess for children_env at ast.py:22221>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22221
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 3889
--# expr-start 3890 '<FieldAccess for dottable_subps_env at ast.py:22221>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22221
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.dottable_subps_env
Fld_1 := Libadalang.Implementation.Base_Type_Decl_F_Dottable_Subps_Env (Node => Self);
--# end
--# expr-done 3890
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_1)); 
--# expr-done 3891
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3892
         Property_Result := Group_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1512;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1512;
            raise;
      end;
   return Property_Result;
end Incomplete_Type_Decl_P_Defining_Env;
--# end
--# property-start IncompleteTypeDecl.discriminants_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22225
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Discriminants_List
  (Node : Bare_Incomplete_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
     Ent : Internal_Entity_Incomplete_Type_Decl :=
       Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1513 with Inline_Always;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1513 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1513;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3895 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3894 '<FieldAccess for abstract_formal_params at ast.py:22230>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22230
--# expr-start 3893 '<FieldAccess for discriminants at ast.py:22230>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22230
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Incomplete_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 3893
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3894
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3895
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1513;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1513;
            raise;
      end;
   return Property_Result;
end Incomplete_Type_Decl_P_Discriminants_List;
--# end
--# property-start IncompleteTypeDecl.xref_stop_resolution /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22232
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Xref_Stop_Resolution
  (Node : Bare_Incomplete_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Incomplete_Type_Decl_P_Xref_Stop_Resolution;
--# end
--# property-start IncompleteTypeDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22233
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Xref_Equation
  (Node : Bare_Incomplete_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1515 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1515 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1515;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:22233") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1515;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1515;
            raise;
      end;
   return Property_Result;
end Incomplete_Type_Decl_P_Xref_Equation;
--# end
--# property-start IncompleteTypeDecl._env_mappings_225 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22216
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_225
  (Node : Bare_Incomplete_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
     Ent : Internal_Entity_Incomplete_Type_Decl :=
       Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1752 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Incomplete_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1752 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1752;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3898 '<New EnvAssoc at ast.py:22216>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22216
--# expr-start 3896 '<New DesignatedEnv at ast.py:22216>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22216
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 3896
--# expr-start 3897 '<FieldAccess for name_symbol at ast.py:22216>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22216
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3897
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 3898
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1752;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1752;
            raise;
      end;
   return Property_Result;
end Env_Mappings_225;
--# end
--# property-start IncompleteTypeDecl._env_trans_parent_226 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22215
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_226
  (Node : Bare_Incomplete_Type_Decl
  )
   return Boolean
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_226;
--# end
   --
   --  Primitives for Bare_Incomplete_Formal_Type_Decl
   --
      procedure Initialize_Fields_For_Incomplete_Formal_Type_Decl
        (Self : Bare_Incomplete_Formal_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Incomplete_Formal_Type_Decl_F_Is_Tagged : Bare_Tagged_Node
         ; Incomplete_Formal_Type_Decl_F_Default_Type : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Incomplete_Type_Decl
              (Self, Base_Type_Decl_F_Name, Incomplete_Type_Decl_F_Discriminants);
            Self.Incomplete_Formal_Type_Decl_F_Is_Tagged := Incomplete_Formal_Type_Decl_F_Is_Tagged;
            Self.Incomplete_Formal_Type_Decl_F_Default_Type := Incomplete_Formal_Type_Decl_F_Default_Type;
      end Initialize_Fields_For_Incomplete_Formal_Type_Decl;
   function Incomplete_Formal_Type_Decl_F_Is_Tagged
     (Node : Bare_Incomplete_Formal_Type_Decl) return Bare_Tagged_Node
   is
   begin
         return Node.Incomplete_Formal_Type_Decl_F_Is_Tagged;
   end;
   function Incomplete_Formal_Type_Decl_F_Default_Type
     (Node : Bare_Incomplete_Formal_Type_Decl) return Bare_Name
   is
   begin
         return Node.Incomplete_Formal_Type_Decl_F_Default_Type;
   end;
--# property-start IncompleteFormalTypeDecl.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22253
pragma Warnings (Off, "is not referenced");
 function Incomplete_Formal_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Incomplete_Formal_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Incomplete_Formal_Type_Decl  := Bare_Incomplete_Formal_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Is_Null : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3901 '<Not at ast.py:22253>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22253
--# expr-start 3900 '<IsNull at ast.py:22253>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22253
--# expr-start 3899 '<FieldAccess for is_tagged at ast.py:22253>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22253
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Incomplete_Formal_Type_Decl_F_Is_Tagged;
--# expr-done 3899
Is_Null := Fld = null; 
--# expr-done 3900
Not_Val := not (Is_Null); 
--# expr-done 3901
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Incomplete_Formal_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start IncompleteFormalTypeDecl.xref_stop_resolution /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22255
pragma Warnings (Off, "is not referenced");
 function Incomplete_Formal_Type_Decl_P_Xref_Stop_Resolution
  (Node : Bare_Incomplete_Formal_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Incomplete_Formal_Type_Decl  := Bare_Incomplete_Formal_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Name;
Is_Null : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3903 '<IsNull at ast.py:22255>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22255
--# expr-start 3902 '<FieldAccess for default_type at ast.py:22255>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22255
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Incomplete_Formal_Type_Decl_F_Default_Type;
--# expr-done 3902
Is_Null := Fld = null; 
--# expr-done 3903
         Property_Result := Is_Null;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Incomplete_Formal_Type_Decl_P_Xref_Stop_Resolution;
--# end
--# property-start IncompleteFormalTypeDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22256
pragma Warnings (Off, "is not referenced");
 function Incomplete_Formal_Type_Decl_P_Xref_Equation
  (Node : Bare_Incomplete_Formal_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Incomplete_Formal_Type_Decl  := Bare_Incomplete_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Incomplete_Formal_Type_Decl :=
       Internal_Entity_Incomplete_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1519 with Inline_Always;
      Fld : Logic_Equation;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Fld_3 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_1519 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1519;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3908 '<If at ast.py:22256>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22256
--# expr-start 3904 '<FieldAccess for xref_stop_resolution at ast.py:22257>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22257
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3904
if Fld_3 then
   --# expr-start 3905 '<Super at ast.py:22258>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22258
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IncompleteTypeDecl.xref_equation
Fld := Libadalang.Implementation.Incomplete_Type_Decl_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3905
   If_Result := Fld;
else
   --# expr-start 3907 '<FieldAccess for sub_equation at ast.py:22259>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22259
--# expr-start 3906 '<FieldAccess for default_type at ast.py:22259>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22259
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Incomplete_Formal_Type_Decl_F_Default_Type, Info => Ent.Info);
--# expr-done 3906
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3907
   If_Result := Fld_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 3908
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1519;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1519;
            raise;
      end;
   return Property_Result;
end Incomplete_Formal_Type_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Incomplete_Tagged_Type_Decl
   --
      procedure Initialize_Fields_For_Incomplete_Tagged_Type_Decl
        (Self : Bare_Incomplete_Tagged_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Incomplete_Tagged_Type_Decl_F_Has_Abstract : Bare_Abstract_Node
        ) is
      begin
            Initialize_Fields_For_Incomplete_Type_Decl
              (Self, Base_Type_Decl_F_Name, Incomplete_Type_Decl_F_Discriminants);
            Self.Incomplete_Tagged_Type_Decl_F_Has_Abstract := Incomplete_Tagged_Type_Decl_F_Has_Abstract;
      end Initialize_Fields_For_Incomplete_Tagged_Type_Decl;
   function Incomplete_Tagged_Type_Decl_F_Has_Abstract
     (Node : Bare_Incomplete_Tagged_Type_Decl) return Bare_Abstract_Node
   is
   begin
         return Node.Incomplete_Tagged_Type_Decl_F_Has_Abstract;
   end;
--# property-start IncompleteTaggedTypeDecl.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22243
pragma Warnings (Off, "is not referenced");
 function Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Incomplete_Tagged_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Incomplete_Tagged_Type_Decl  := Bare_Incomplete_Tagged_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type;
--# end
   --
   --  Primitives for Bare_Protected_Type_Decl
   --
      procedure Protected_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Protected_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_13 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:9163");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_14 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_15 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Protected_Type_Decl
        (Self : Bare_Protected_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Protected_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Protected_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Protected_Type_Decl_F_Interfaces : Bare_Parent_List
         ; Protected_Type_Decl_F_Definition : Bare_Protected_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Protected_Type_Decl_F_Discriminants := Protected_Type_Decl_F_Discriminants;
            Self.Protected_Type_Decl_F_Aspects := Protected_Type_Decl_F_Aspects;
            Self.Protected_Type_Decl_F_Interfaces := Protected_Type_Decl_F_Interfaces;
            Self.Protected_Type_Decl_F_Definition := Protected_Type_Decl_F_Definition;
      end Initialize_Fields_For_Protected_Type_Decl;
   function Protected_Type_Decl_F_Discriminants
     (Node : Bare_Protected_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Protected_Type_Decl_F_Discriminants;
   end;
   function Protected_Type_Decl_F_Interfaces
     (Node : Bare_Protected_Type_Decl) return Bare_Parent_List
   is
   begin
         return Node.Protected_Type_Decl_F_Interfaces;
   end;
   function Protected_Type_Decl_F_Definition
     (Node : Bare_Protected_Type_Decl) return Bare_Protected_Def
   is
   begin
         return Node.Protected_Type_Decl_F_Definition;
   end;
--# property-start ProtectedTypeDecl.discriminants_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9117
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Discriminants_List
  (Node : Bare_Protected_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_722 with Inline_Always;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_722 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_722;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3911 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3910 '<FieldAccess for abstract_formal_params at ast.py:9122>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9122
--# expr-start 3909 '<FieldAccess for discriminants at ast.py:9122>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9122
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Protected_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 3909
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3910
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3911
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_722;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_722;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Discriminants_List;
--# end
--# property-start ProtectedTypeDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9124
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Defining_Env
  (Node : Bare_Protected_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_723 with Inline_Always;
            procedure Finalizer_Scope_2385 with Inline_Always;
      Var_Expr : Internal_Entity_Private_Part;
Fld : Internal_Entity_Protected_Def;
Fld_1 : Internal_Entity_Private_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_723 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_723;
            procedure Finalizer_Scope_2385 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2385;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3913 '<FieldAccess for private_part at ast.py:9125>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9125
--# expr-start 3912 '<FieldAccess for definition at ast.py:9125>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9125
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Protected_Def (Node => Ent.Node.Protected_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 3912
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Private_Part (Node => Fld.Node.Protected_Def_F_Private_Part, Info => Fld.Info);
--# expr-done 3913
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Private_Part then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 3917 '<env_group at ast.py:9130>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9130
--# expr-start 3916 '<ArrayLiteral at ast.py:9130>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9130
--# expr-start 3914 '<FieldAccess for children_env at ast.py:9130>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9130
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3914
--# expr-start 3915 '<FieldAccess for children_env at ast.py:9130>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9130
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 3915
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Fld_3)); 
--# expr-done 3916
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3917
Scope_Result := Group_Env;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2385;
   Result_Var := Scope_Result;
else
   --# expr-start 3918 '<FieldAccess for children_env at ast.py:9131>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9131
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3918
   Result_Var := Fld_4;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_723;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_723;
                     Finalizer_Scope_2385;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Defining_Env;
--# end
--# property-start ProtectedTypeDecl.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9135
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Base_Interfaces
  (Node : Bare_Protected_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_724 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Item_141 : Internal_Entity_Name;
Bare_Item_141 : Bare_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_724 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_724;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3921 '<Map at ast.py:9136>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9136
   --# expr-start 3919 '<FieldAccess for interfaces at ast.py:9136>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9136
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Protected_Type_Decl_F_Interfaces, Info => Ent.Info);
--# expr-done 3919
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_141 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_141 := Untyped_Item_141; 
Is_Null := Bare_Item_141 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_141); 
   If_Result := As_Entity;
end if;
                  Item_141 := If_Result; 
   --# scope-start
               --# bind i Item_141
      --# expr-start 3920 '<FieldAccess for name_designated_type at ast.py:9136>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9136
      if Item_141.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_141.Node, E_Info => Item_141.Info);
--# end
--# expr-done 3920
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3921
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_724;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_724;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Base_Interfaces;
--# end
--# property-start ProtectedTypeDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9139
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Next_Part_For_Decl
  (Node : Bare_Protected_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3923 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3922 '<FieldAccess for basic_decl_next_part_for_decl at ast.py:9141>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9141
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3922
Let_Result := Fld; 
--# end
--# expr-done 3923
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ProtectedTypeDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9143
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Protected_Type_Decl
  )
   return Boolean
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start ProtectedTypeDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9145
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Xref_Equation
  (Node : Bare_Protected_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_727 with Inline_Always;
            procedure Finalizer_Scope_2387 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_142 : Internal_Entity_Name;
Bare_Item_142 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_727 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_727;
            procedure Finalizer_Scope_2387 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2387;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3928 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3927 '<LogicAll at ast.py:9147>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9147
--# expr-start 3926 '<Map at ast.py:9147>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9147
   --# expr-start 3924 '<FieldAccess for interfaces at ast.py:9147>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9147
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Protected_Type_Decl_F_Interfaces, Info => Ent.Info);
--# expr-done 3924
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_142 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_142 := Untyped_Item_142; 
Is_Null := Bare_Item_142 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_142); 
   If_Result := As_Entity;
end if;
                  Item_142 := If_Result; 
   --# scope-start
               --# bind ifc Item_142
      --# expr-start 3925 '<FieldAccess for xref_equation at ast.py:9147>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9147
      if Item_142.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_142.Node, Env => Env, Origin => Origin, E_Info => Item_142.Info);
--# end
--# expr-done 3925
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2387;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3926
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9147") else null)); 
--# expr-done 3927
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 3928
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_727;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_727;
                     Finalizer_Scope_2387;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Xref_Equation;
--# end
--# property-start ProtectedTypeDecl.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9149
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Declarative_Parts
  (Node : Bare_Protected_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_728 with Inline_Always;
            procedure Finalizer_Scope_2388 with Inline_Always;
      Block_Var : Internal_Entity_Protected_Def;
Var_Expr : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Protected_Def;
Fld_1 : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
Fld_2 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Declarative_Part;
Singleton_1 : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result : Internal_Entity_Declarative_Part_Array_Access;
Result_Var : Internal_Entity_Declarative_Part_Array_Access;
Concat_Result : Internal_Entity_Declarative_Part_Array_Access;
Let_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_728 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_728;
            procedure Finalizer_Scope_2388 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_2388;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3935 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3929 '<FieldAccess for definition at ast.py:9151>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9151
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Protected_Def (Node => Ent.Node.Protected_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 3929
Block_Var := Fld; 
--# bind pdef Block_Var
--# expr-start 3934 '<Concat at ast.py:9152>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9152
--# expr-start 3931 '<Cast to DeclarativePart at ast.py:9152>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9152
--# expr-start 3930 '<FieldAccess for public_part at ast.py:9152>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9152
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Block_Var.Node.Protected_Def_F_Public_Part, Info => Block_Var.Info);
--# expr-done 3930
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 3931
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
--# expr-start 3933 '<Cast to DeclarativePart at ast.py:9153>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9153
--# expr-start 3932 '<FieldAccess for private_part at ast.py:9153>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9153
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Block_Var.Node.Protected_Def_F_Private_Part, Info => Block_Var.Info);
--# expr-done 3932
      Cast_Result_1 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 3933
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Declarative_Part then
   --# scope-start
                Singleton_1 := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr;
Scope_Result := Singleton_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2388;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Singleton, Result_Var); 
--# expr-done 3934
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3935
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_728;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_728;
                     Finalizer_Scope_2388;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Declarative_Parts;
--# end
--# property-start ProtectedTypeDecl.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9156
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Env_Names
  (Node : Bare_Protected_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_729 with Inline_Always;
            procedure Finalizer_Scope_2389 with Inline_Always;
      Var_Expr : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_729 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_729;
            procedure Finalizer_Scope_2389 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2389;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3938 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3936 '<FieldAccess for top_level_env_name at ast.py:9158>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9158
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3936
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 3937 '<to_symbol at ast.py:9159>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9159
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr); 
--# expr-done 3937
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2389;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3938
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_729;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_729;
                     Finalizer_Scope_2389;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Env_Names;
--# end
--# property-start ProtectedTypeDecl._env_mappings_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9163
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_13
  (Node : Bare_Protected_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1540 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Protected_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1540 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1540;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3941 '<New EnvAssoc at ast.py:9163>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9163
--# expr-start 3939 '<New DesignatedEnv at ast.py:9163>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9163
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 3939
--# expr-start 3940 '<FieldAccess for name_symbol at ast.py:9163>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9163
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3940
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 3941
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1540;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1540;
            raise;
      end;
   return Property_Result;
end Env_Mappings_13;
--# end
--# property-start ProtectedTypeDecl._env_trans_parent_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9162
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_14
  (Node : Bare_Protected_Type_Decl
  )
   return Boolean
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_14;
--# end
--# property-start ProtectedTypeDecl._env_names_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9164
pragma Warnings (Off, "is not referenced");
 function Env_Names_15
  (Node : Bare_Protected_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1542 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1542 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1542;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3942 '<FieldAccess for env_names at ast.py:9164>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9164
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ProtectedTypeDecl.env_names
Fld := Libadalang.Implementation.Protected_Type_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 3942
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1542;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1542;
            raise;
      end;
   return Property_Result;
end Env_Names_15;
--# end
   --
   --  Primitives for Bare_Task_Type_Decl
   --
      procedure Task_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Task_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_9 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:9064");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_10 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_11 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Task_Type_Decl
        (Self : Bare_Task_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Task_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Task_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Task_Type_Decl_F_Definition : Bare_Task_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Task_Type_Decl_F_Discriminants := Task_Type_Decl_F_Discriminants;
            Self.Task_Type_Decl_F_Aspects := Task_Type_Decl_F_Aspects;
            Self.Task_Type_Decl_F_Definition := Task_Type_Decl_F_Definition;
      end Initialize_Fields_For_Task_Type_Decl;
   function Task_Type_Decl_F_Discriminants
     (Node : Bare_Task_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Task_Type_Decl_F_Discriminants;
   end;
   function Task_Type_Decl_F_Definition
     (Node : Bare_Task_Type_Decl) return Bare_Task_Def
   is
   begin
         return Node.Task_Type_Decl_F_Definition;
   end;
--# property-start TaskTypeDecl.is_task_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9051
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Is_Task_Type
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Is_Task_Type;
--# end
--# property-start TaskTypeDecl.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9053
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Base_Interfaces
  (Node : Bare_Task_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_715 with Inline_Always;
      Fld : Internal_Entity_Task_Def;
Fld_1 : Internal_Entity_Parent_List;
Fld_2 : Bare_Parent_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
Item_143 : Internal_Entity_Name;
Bare_Item_143 : Bare_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_715 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_715;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3946 '<Map at ast.py:9054>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9054
   --# expr-start 3944 '<FieldAccess for interfaces at ast.py:9054>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9054
--# expr-start 3943 '<FieldAccess for definition at ast.py:9054>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9054
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Task_Def (Node => Ent.Node.Task_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 3943
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Parent_List (Node => Fld.Node.Task_Def_F_Interfaces, Info => Fld.Info);
--# expr-done 3944
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_2;
      begin
         for Untyped_Item_143 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_143 := Untyped_Item_143; 
Is_Null := Bare_Item_143 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_143); 
   If_Result := As_Entity;
end if;
                  Item_143 := If_Result; 
   --# scope-start
               --# bind i Item_143
      --# expr-start 3945 '<FieldAccess for name_designated_type at ast.py:9054>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9054
      if Item_143.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_4 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_143.Node, E_Info => Item_143.Info);
--# end
--# expr-done 3945
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3946
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_715;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_715;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Base_Interfaces;
--# end
--# property-start TaskTypeDecl.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9057
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Env_Names
  (Node : Bare_Task_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_716 with Inline_Always;
            procedure Finalizer_Scope_2391 with Inline_Always;
      Var_Expr : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_716 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_716;
            procedure Finalizer_Scope_2391 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2391;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3949 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3947 '<FieldAccess for top_level_env_name at ast.py:9059>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9059
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3947
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 3948 '<to_symbol at ast.py:9060>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9060
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr); 
--# expr-done 3948
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2391;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3949
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_716;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_716;
                     Finalizer_Scope_2391;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Env_Names;
--# end
--# property-start TaskTypeDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9068
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Defining_Env
  (Node : Bare_Task_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_717 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_717 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_717;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3950 '<FieldAccess for children_env at ast.py:9068>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9068
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3950
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_717;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_717;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Defining_Env;
--# end
--# property-start TaskTypeDecl.discriminants_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9070
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Discriminants_List
  (Node : Bare_Task_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_718 with Inline_Always;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_718 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_718;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3953 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3952 '<FieldAccess for abstract_formal_params at ast.py:9075>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9075
--# expr-start 3951 '<FieldAccess for discriminants at ast.py:9075>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9075
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Task_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 3951
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3952
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3953
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_718;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_718;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Discriminants_List;
--# end
--# property-start TaskTypeDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9077
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start TaskTypeDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9079
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Xref_Equation
  (Node : Bare_Task_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_720 with Inline_Always;
            procedure Finalizer_Scope_2392 with Inline_Always;
      Var_Expr : Internal_Entity_Task_Def;
Fld : Internal_Entity_Task_Def;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_720 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_720;
            procedure Finalizer_Scope_2392 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2392;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3956 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3954 '<FieldAccess for definition at ast.py:9081>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9081
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Task_Def (Node => Ent.Node.Task_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 3954
Var_Expr := Fld;
if Var_Expr /= No_Entity_Task_Def then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 3955 '<FieldAccess for xref_equation at ast.py:9082>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9082
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3955
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2392;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9083") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3956
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_720;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_720;
                     Finalizer_Scope_2392;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Xref_Equation;
--# end
--# property-start TaskTypeDecl.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9086
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Declarative_Parts
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_721 with Inline_Always;
            procedure Finalizer_Scope_2393 with Inline_Always;
            procedure Finalizer_Scope_2394 with Inline_Always;
      Var_Expr : Internal_Entity_Task_Def;
Var_Expr_1 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Task_Def;
Fld_1 : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
Fld_2 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Declarative_Part;
Singleton_1 : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result : Internal_Entity_Declarative_Part_Array_Access;
Result_Var : Internal_Entity_Declarative_Part_Array_Access;
Concat_Result : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result_1 : Internal_Entity_Declarative_Part_Array_Access;
Result_Var_1 : Internal_Entity_Declarative_Part_Array_Access;
Let_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_721 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_721;
            procedure Finalizer_Scope_2393 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2393;
            procedure Finalizer_Scope_2394 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_2394;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3963 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 3957 '<FieldAccess for definition at ast.py:9088>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9088
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Task_Def (Node => Ent.Node.Task_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 3957
Var_Expr := Fld;
if Var_Expr /= No_Entity_Task_Def then
   --# scope-start
   --# bind tdef Var_Expr
--# expr-start 3962 '<Concat at ast.py:9090>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9090
--# expr-start 3959 '<Cast to DeclarativePart at ast.py:9090>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9090
--# expr-start 3958 '<FieldAccess for public_part at ast.py:9090>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9090
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Var_Expr.Node.Task_Def_F_Public_Part, Info => Var_Expr.Info);
--# expr-done 3958
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 3959
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
--# expr-start 3961 '<Cast to DeclarativePart at ast.py:9091>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9091
--# expr-start 3960 '<FieldAccess for private_part at ast.py:9091>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9091
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Var_Expr.Node.Task_Def_F_Private_Part, Info => Var_Expr.Info);
--# expr-done 3960
      Cast_Result_1 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 3961
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Declarative_Part then
   --# scope-start
                Singleton_1 := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr_1;
Scope_Result := Singleton_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2394;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Singleton, Result_Var); 
--# expr-done 3962
Scope_Result_1 := Concat_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2393;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3963
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_721;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_721;
                     Finalizer_Scope_2393;
                     Finalizer_Scope_2394;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Declarative_Parts;
--# end
--# property-start TaskTypeDecl._env_mappings_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9064
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_9
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1536 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Task_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1536 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1536;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3966 '<New EnvAssoc at ast.py:9064>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9064
--# expr-start 3964 '<New DesignatedEnv at ast.py:9064>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9064
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 3964
--# expr-start 3965 '<FieldAccess for name_symbol at ast.py:9064>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9064
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3965
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 3966
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1536;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1536;
            raise;
      end;
   return Property_Result;
end Env_Mappings_9;
--# end
--# property-start TaskTypeDecl._env_trans_parent_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9063
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_10
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_10;
--# end
--# property-start TaskTypeDecl._env_names_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9065
pragma Warnings (Off, "is not referenced");
 function Env_Names_11
  (Node : Bare_Task_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1538 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1538 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1538;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3967 '<FieldAccess for env_names at ast.py:9065>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9065
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskTypeDecl.env_names
Fld := Libadalang.Implementation.Task_Type_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 3967
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1538;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1538;
            raise;
      end;
   return Property_Result;
end Env_Names_11;
--# end
   --
   --  Primitives for Bare_Single_Task_Type_Decl
   --
      procedure Single_Task_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Task_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_12 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Task_Type_Decl
        (Self : Bare_Single_Task_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Task_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Task_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Task_Type_Decl_F_Definition : Bare_Task_Def
        ) is
      begin
            Initialize_Fields_For_Task_Type_Decl
              (Self, Base_Type_Decl_F_Name, Task_Type_Decl_F_Discriminants, Task_Type_Decl_F_Aspects, Task_Type_Decl_F_Definition);
      end Initialize_Fields_For_Single_Task_Type_Decl;
--# property-start SingleTaskTypeDecl._env_trans_parent_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9100
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_12
  (Node : Bare_Single_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Single_Task_Type_Decl  := Bare_Single_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_12;
--# end
   --
   --  Primitives for Bare_Type_Decl
   --
      procedure Type_Decl_Pre_Env_Actions
        (Self            : Bare_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_3 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7701");
         Dec_Ref (Mapping.Dest_Env);
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_4 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7702");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_5 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Type_Decl_Post_Env_Actions
        (Self : Bare_Type_Decl; State : in out PLE_Node_State) is
      begin
      if Libadalang.Implementation.Ref_Cond_8 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_6 (Node => 
Self);
            Env : Lexical_Env :=
              Libadalang.Implementation.Env_Dest_7 (Node => 
Self);
         begin
               if Is_Foreign_Strict (Env, Self) then
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "unsound foreign environment in RefEnvs ("
                     & "ast.py:7705)");
               end if;
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Type_Decl_P_Refined_Parent_Primitives_Env'Access,
               Transitive,
               (Inherited_Primitives => True, others => False),
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Type_Decl
        (Self : Bare_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Type_Decl_F_Discriminants := Type_Decl_F_Discriminants;
            Self.Type_Decl_F_Type_Def := Type_Decl_F_Type_Def;
      Self.Lf_Present_Direct_Primitives_Env := False;
      Self.Lf_Stg_Direct_Primitives_Env := Empty_Env;
      Self.Lf_Present_Direct_Primitive_Subps := False;
      Self.Lf_Stg_Direct_Primitive_Subps := No_Internal_Inner_Env_Assoc_Array_Type;
      end Initialize_Fields_For_Type_Decl;
   function Type_Decl_F_Discriminants
     (Node : Bare_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Type_Decl_F_Discriminants;
   end;
   function Type_Decl_F_Type_Def
     (Node : Bare_Type_Decl) return Bare_Type_Def
   is
   begin
         return Node.Type_Decl_F_Type_Def;
   end;
--# property-start TypeDecl.is_iterable_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7403
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Sym : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Type_Def;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Internal_Entity_Type_Def;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result_2 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3968 '<FieldAccess for is_array at ast.py:7405>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7405
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3968
if Fld then
   If_Result := True;
else
   --# expr-start 3972 '<Not at ast.py:7406>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7406
--# expr-start 3971 '<IsNull at ast.py:7406>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7406
--# expr-start 3970 '<FieldAccess for get_aspect_spec_expr at ast.py:7406>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7406
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3969 '<Symbol Iterator_Element at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator_Element); 
--# expr-done 3969
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 3970
Is_Null := Fld_1.Node = null; 
--# expr-done 3971
Not_Val := not (Is_Null); 
--# expr-done 3972
   If_Result := Not_Val;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 3976 '<Not at ast.py:7410>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7410
--# expr-start 3975 '<IsNull at ast.py:7410>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7410
--# expr-start 3974 '<FieldAccess for get_aspect_spec_expr at ast.py:7410>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7410
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3973 '<Symbol Iterable at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 3973
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 3974
Is_Null_1 := Fld_2.Node = null; 
--# expr-done 3975
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3976
   If_Result_1 := Not_Val_1;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 3980 '<Match at ast.py:7411>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7411
--# expr-start 3977 '<FieldAccess for type_def at ast.py:7411>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7411
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3977
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_3; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind dtd Match_0
--# expr-start 3978 '<FieldAccess for base_type at ast.py:7413>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7413
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 3978
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 3979 '<FieldAccess for is_iterable_type at ast.py:7413>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7413
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3979
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3980
   If_Result_2 := Match_Result;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
--# expr-start 3981 '<FieldAccess for previous_part at ast.py:7416>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7416
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 3981
Var_Expr_1 := Fld_6;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 3982 '<FieldAccess for is_iterable_type at ast.py:7416>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7416
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3982
Scope_Result_3 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_3;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
end if;
         Property_Result := If_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start TypeDecl.iterable_comp_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7424
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_548 with Inline_Always;
            procedure Finalizer_Scope_2399 with Inline_Always;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Name;
Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Bound_Imprecise_Fallback : Boolean;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_4 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_5 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Internal_Single_Actual_Array_Access;
Fld_8 : Bare_Base_Id;
Sym_2 : Symbol_Type;
Fld_9 : Boolean;
Find_Result : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_12 : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Base_Type_Decl;
Fld_14 : Internal_Entity_Type_Def;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result_3 : Internal_Entity_Derived_Type_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Type_Def;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_548 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_548;
            procedure Finalizer_Scope_2399 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_2399;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4017 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 3984 '<FieldAccess for get_aspect_spec_expr at ast.py:7426>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7426
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3983 '<Symbol Iterator_Element at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator_Element); 
--# expr-done 3983
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 3984
Block_Var := Fld; 
--# bind ie Block_Var
--# expr-start 3986 '<FieldAccess for get_aspect_spec_expr at ast.py:7427>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7427
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3985 '<Symbol Iterable at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 3985
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 3986
Block_Var_1 := Fld_1; 
--# bind it Block_Var_1
--# expr-start 4016 '<bind at ast.py:7429>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7429
Bound_Imprecise_Fallback := False; 
--# expr-start 4013 '<Cond at ast.py:7429>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7429
--# expr-start 3987 '<FieldAccess for is_array at ast.py:7430>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7430
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3987
if Fld_2 then
   --# expr-start 3988 '<FieldAccess for comp_type at ast.py:7430>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7430
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3988
   If_Result_2 := Fld_3;
else
--# expr-start 3990 '<Not at ast.py:7432>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7432
--# expr-start 3989 '<IsNull at ast.py:7432>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7432
Is_Null := Block_Var.Node = null; 
--# expr-done 3989
Not_Val := not (Is_Null); 
--# expr-done 3990
if Not_Val then
--# expr-start 3991 '<Cast to Name at ast.py:7432>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7432
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 3991
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 3994 '<bind at ast.py:7433>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7433
--# expr-start 3992 '<FieldAccess for node_env at ast.py:7433>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7433
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Node_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 3992
Bound_Env := Fld_4; Inc_Ref (Bound_Env);
--# expr-start 3993 '<FieldAccess for designated_type_impl at ast.py:7433>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7433
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Var_Expr_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3993
Dyn_Var_Bind_Result := Fld_5; 
--# expr-done 3994
Scope_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_2399;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := Result_Var;
else
--# expr-start 3996 '<Not at ast.py:7436>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7436
--# expr-start 3995 '<IsNull at ast.py:7436>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7436
Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 3995
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3996
if Not_Val_1 then
   --# expr-start 4008 '<FieldAccess for expr_type at ast.py:7440>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7440
--# expr-start 4007 '<FieldAccess for referenced_decl at ast.py:7440>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7440
--# expr-start 4006 '<Cast to Name at ast.py:7439>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7439
--# expr-start 4005 '<FieldAccess for expr at ast.py:7439>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7439
--# expr-start 4004 '<FieldAccess for assoc at ast.py:7439>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7439
--# expr-start 4003 '<Find at ast.py:7437>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7437
   --# expr-start 3999 '<FieldAccess for unpacked_params at ast.py:7437>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7437
--# expr-start 3998 '<FieldAccess for assocs at ast.py:7437>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7437
--# expr-start 3997 '<Cast to Aggregate at ast.py:7437>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7437
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 3997
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_1.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_1.Info);
--# expr-done 3998
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_7 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 3999
      Find_Result := No_Single_Actual;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_7;
      begin
         for Item_144 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_144
      --# expr-start 4002 '<FieldAccess for name_is at ast.py:7438>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7438
--# expr-start 4000 '<FieldAccess for name at ast.py:7438>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7438
Fld_8 := Item_144.Name;
--# expr-done 4000
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4001 '<Symbol Element at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Element); 
--# expr-done 4001
--# property-call-start Name.name_is
Fld_9 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_8, Sym => Sym_2);
--# end
--# expr-done 4002
      if Fld_9 then
         Find_Result := Item_144; 
      end if;
   --# end
               exit when Fld_9;
         end loop;
      end;
--# expr-done 4003
Fld_10 := Find_Result.Assoc;
--# expr-done 4004
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 4005
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4006
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_12 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_2.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 4007
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Fld_12.Node, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 4008
   If_Result := Fld_13;
else
   --# expr-start 4012 '<Match at ast.py:7442>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7442
--# expr-start 4009 '<FieldAccess for type_def at ast.py:7442>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7442
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4009
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_14; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind dtd Match_0
--# expr-start 4010 '<FieldAccess for base_type at ast.py:7444>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7444
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4010
Var_Expr_2 := Fld_15;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_2
--# expr-start 4011 '<FieldAccess for iterable_comp_type at ast.py:7444>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7444
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4011
Scope_Result_1 := Fld_16;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 4012
   If_Result := Match_Result;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 4013
Var_Expr := If_Result_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_4 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_4;
else
--# expr-start 4014 '<FieldAccess for previous_part at ast.py:7447>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7447
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 4014
Var_Expr_3 := Fld_17;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr_3
--# expr-start 4015 '<FieldAccess for iterable_comp_type at ast.py:7448>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7448
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_18 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_3.Node, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 4015
Scope_Result_5 := Fld_18;
   --# end
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Dyn_Var_Bind_Result_1 := Result_Var_3; 
--# expr-done 4016
Let_Result_2 := Dyn_Var_Bind_Result_1; 
--# end
--# expr-done 4017
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_548;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_548;
                     Finalizer_Scope_2399;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start TypeDecl.iterable_cursor_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7450
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Iterable_Cursor_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2406 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Single_Actual_Array_Access;
Fld_3 : Bare_Base_Id;
Sym_1 : Symbol_Type;
Fld_4 : Boolean;
Find_Result : Internal_Single_Actual;
Fld_5 : Internal_Entity_Basic_Assoc;
Fld_6 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2406 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2406;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4032 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4019 '<FieldAccess for get_aspect_spec_expr at ast.py:7456>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7456
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4018 '<Symbol Iterable at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 4018
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 4019
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind it Var_Expr
--# expr-start 4031 '<FieldAccess for expr_type at ast.py:7459>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7459
--# expr-start 4030 '<FieldAccess for referenced_decl at ast.py:7459>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7459
--# expr-start 4029 '<Cast to Name at ast.py:7459>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7459
--# expr-start 4028 '<FieldAccess for expr at ast.py:7459>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7459
--# expr-start 4027 '<FieldAccess for assoc at ast.py:7459>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7459
--# expr-start 4026 '<Find at ast.py:7457>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7457
   --# expr-start 4022 '<FieldAccess for unpacked_params at ast.py:7457>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7457
--# expr-start 4021 '<FieldAccess for assocs at ast.py:7457>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7457
--# expr-start 4020 '<Cast to Aggregate at ast.py:7457>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7457
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 4020
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Base_Aggregate_F_Assocs, Info => Cast_Result.Info);
--# expr-done 4021
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_2 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4022
      Find_Result := No_Single_Actual;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_2;
      begin
         for Item_145 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_145
      --# expr-start 4025 '<FieldAccess for name_is at ast.py:7458>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7458
--# expr-start 4023 '<FieldAccess for name at ast.py:7458>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7458
Fld_3 := Item_145.Name;
--# expr-done 4023
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4024 '<Symbol First at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 4024
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3, Sym => Sym_1);
--# end
--# expr-done 4025
      if Fld_4 then
         Find_Result := Item_145; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 4026
Fld_5 := Find_Result.Assoc;
--# expr-done 4027
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 4028
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4029
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_7 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_1.Node, Imprecise_Fallback => False, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4030
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Fld_7.Node, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 4031
Scope_Result := Fld_8;
   --# end
      Finalizer_Scope_2406;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4032
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2406;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Iterable_Cursor_Type;
--# end
--# property-start TypeDecl.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7461
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Discrete_Range
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4035 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4034 '<FieldAccess for discrete_range at ast.py:7463>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7463
--# expr-start 4033 '<FieldAccess for type_def at ast.py:7463>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7463
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4033
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.discrete_range'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4034
Let_Result := Fld_1; 
--# end
--# expr-done 4035
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Discrete_Range;
--# end
--# property-start TypeDecl.discriminants_list /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7465
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Discriminants_List
  (Node : Bare_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_551 with Inline_Always;
            procedure Finalizer_Scope_2408 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Discriminant_Part;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Discriminant_Part;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Len : Integer;
Comp_Result : Boolean;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_6 : Boolean;
If_Result : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_551 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_551;
            procedure Finalizer_Scope_2408 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2408;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4051 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4036 '<FieldAccess for base_type at ast.py:7470>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7470
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4036
Block_Var := Fld; 
--# bind base_type Block_Var
--# expr-start 4037 '<FieldAccess for discriminants at ast.py:7471>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7471
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 4037
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Discriminant_Part then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 4038 '<FieldAccess for abstract_formal_params at ast.py:7472>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7472
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4038
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2408;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Block_Var_1 := Result_Var; Inc_Ref (Block_Var_1);
--# bind self_discs Block_Var_1
--# expr-start 4050 '<Cond at ast.py:7475>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7475
--# expr-start 4039 '<FieldAccess for is_access_type at ast.py:7476>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7476
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4039
if Fld_3 then
   --# expr-start 4041 '<FieldAccess for discriminants_list at ast.py:7477>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7477
--# expr-start 4040 '<FieldAccess for accessed_type at ast.py:7477>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7477
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4040
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_4.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4041
   If_Result_4 := Fld_5;
else
--# expr-start 4042 '<length at ast.py:7479>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7479
Len := Length (Block_Var_1); 
--# expr-done 4042
Comp_Result := Len > 0; 
if Comp_Result then
   If_Result_3 := Block_Var_1;
else
--# expr-start 4044 '<Not at ast.py:7481>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7481
--# expr-start 4043 '<IsNull at ast.py:7481>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7481
Is_Null := Block_Var.Node = null; 
--# expr-done 4043
Not_Val := not (Is_Null); 
--# expr-done 4044
if Not_Val then
   --# expr-start 4045 '<FieldAccess for matching_type at ast.py:7481>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7481
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Block_Var.Node, Expected_Type => Stop_Recurse_At, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4045
   If_Result := Fld_6;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := Block_Var_1;
else
--# expr-start 4047 '<Not at ast.py:7484>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7484
--# expr-start 4046 '<IsNull at ast.py:7484>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7484
Is_Null_1 := Block_Var.Node = null; 
--# expr-done 4046
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4047
if Not_Val_1 then
   --# expr-start 4049 '<FieldAccess for discriminants_list at ast.py:7485>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7485
--# expr-start 4048 '<FieldAccess for base_type at ast.py:7485>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7485
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4048
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_7.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 4049
   If_Result_1 := Fld_8;
else
   If_Result_1 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 4050
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 4051
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_551;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_551;
                     Finalizer_Scope_2408;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Discriminants_List;
--# end
--# property-start TypeDecl.direct_primitives_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7490
pragma Warnings (Off, "is not referenced");
 function Type_Decl_F_Direct_Primitives_Env
  (Node : Bare_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_552 with Inline_Always;
      Dyn_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_552 is
            begin
                     Dec_Ref (Dyn_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_552;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Direct_Primitives_Env then
         Property_Result := Self.Lf_Stg_Direct_Primitives_Env;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4053 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4052 '<DynamicLexicalEnv at ast.py:7496>' Dyn_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7496
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Type_Decl_F_Direct_Primitive_Subps'Access, null, False); 
--# expr-done 4052
Let_Result := Dyn_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 4053
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_552;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_552;
            raise;
      end;
      Self.Lf_Present_Direct_Primitives_Env := True;
      Self.Lf_Stg_Direct_Primitives_Env := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Decl_F_Direct_Primitives_Env;
--# end
--# property-start TypeDecl.direct_primitive_subps /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7501
pragma Warnings (Off, "is not referenced");
 function Type_Decl_F_Direct_Primitive_Subps
  (Node : Bare_Type_Decl
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_553 with Inline_Always;
            procedure Finalizer_Scope_2410 with Inline_Always;
            procedure Finalizer_Scope_2411 with Inline_Always;
            procedure Finalizer_Scope_2415 with Inline_Always;
            procedure Finalizer_Scope_2416 with Inline_Always;
            procedure Finalizer_Scope_2418 with Inline_Always;
            procedure Finalizer_Scope_2422 with Inline_Always;
            procedure Finalizer_Scope_2425 with Inline_Always;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Array_Access;
Block_Var_3 : Internal_Inner_Env_Assoc_Array_Access;
Block_Var_4 : Internal_Inner_Env_Assoc_Array_Access;
Block_Var_5 : Internal_Inner_Env_Assoc_Array_Access;
Var_Expr : Internal_Entity;
Var_Expr_1 : Bare_Declarative_Part;
Match_0 : Internal_Entity_Base_Package_Decl;
Match_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Private_Part;
Var_Expr_3 : Bare_Declarative_Part;
Var_Expr_4 : Bare_Enum_Type_Def;
Var_Expr_5 : Internal_Entity_Base_Subp_Spec;
Var_Expr_6 : Internal_Entity_Basic_Decl;
Bd : Internal_Entity_Basic_Decl;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld_1 : Boolean;
Fld_2 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_3 : Internal_Entity_Public_Part;
Fld_4 : Internal_Entity_Ada_Node_List;
Fld_5 : Bare_Ada_Node_List;
Fld_6 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Item_388 : Internal_Entity;
Bare_Item_388 : Bare_Ada_Node;
Map_Result : Internal_Entity_Array_Access;
Fld_7 : Internal_Entity_Private_Part;
Fld_8 : Internal_Entity_Ada_Node_List;
Scope_Result_1 : Internal_Entity_Ada_Node_List;
Result_Var_1 : Internal_Entity_Ada_Node_List;
Fld_9 : Bare_Ada_Node_List;
Fld_10 : Internal_Entity_Info;
As_Entity_3 : Internal_Entity;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity;
Item_389 : Internal_Entity;
Bare_Item_389 : Bare_Ada_Node;
Map_Result_1 : Internal_Entity_Array_Access;
Concat_Result : Internal_Entity_Array_Access;
Fld_11 : Bare_Ada_Node;
Fld_12 : Bare_Ada_Node;
Cast_Result : Bare_Declarative_Part;
As_Entity_4 : Internal_Entity_Declarative_Part;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity_Declarative_Part;
Fld_13 : Internal_Entity_Ada_Node_List;
Fld_14 : Bare_Ada_Node_List;
Fld_15 : Internal_Entity_Info;
As_Entity_5 : Internal_Entity;
Is_Null_5 : Boolean;
If_Result_5 : Internal_Entity;
Item_390 : Internal_Entity;
Bare_Item_390 : Bare_Ada_Node;
Map_Result_2 : Internal_Entity_Array_Access;
Scope_Result_2 : Internal_Entity_Array_Access;
Result_Var_2 : Internal_Entity_Array_Access;
If_Result_6 : Internal_Entity_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Array_Access;
Scope_Result_3 : Internal_Entity_Array_Access;
Cast_Result_2 : Internal_Entity;
Let_Result_1 : Internal_Entity_Array_Access;
Scope_Result_4 : Internal_Entity_Array_Access;
Match_Result : Internal_Entity_Array_Access;
Scope_Result_5 : Internal_Entity_Array_Access;
Result_Var_3 : Internal_Entity_Array_Access;
Fld_16 : Bare_Type_Def;
Cast_Result_3 : Bare_Enum_Type_Def;
Fld_17 : Bare_Enum_Literal_Decl_List;
Fld_18 : Bare_Defining_Name;
Fld_19 : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_4 : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Cast_Expr_1 : Bare_Enum_Literal_Decl;
Cast_Result_5 : Bare_Ada_Node;
New_Struct_1 : Internal_Inner_Env_Assoc;
Item_146 : Bare_Enum_Literal_Decl;
Map_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
Scope_Result_6 : Internal_Inner_Env_Assoc_Array_Access;
Result_Var_4 : Internal_Inner_Env_Assoc_Array_Access;
Cast_Expr_2 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Fld_20 : Internal_Entity_Base_Subp_Spec;
Scope_Result_7 : Internal_Entity_Base_Subp_Spec;
Result_Var_5 : Internal_Entity_Base_Subp_Spec;
Fld_21 : Internal_Entity_Base_Type_Decl_Array_Access;
As_Entity_6 : Internal_Entity_Type_Decl;
Is_Null_6 : Boolean;
If_Result_7 : Internal_Entity_Type_Decl;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Cast_Result_8 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Fld_22 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
As_Entity_7 : Internal_Entity_Type_Decl;
Is_Null_7 : Boolean;
If_Result_8 : Internal_Entity_Type_Decl;
Fld_23 : Lexical_Env := Empty_Env;
Fld_24 : Internal_Entity_Defining_Name;
Fld_25 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr_4 : Internal_Entity;
Cast_Result_9 : Internal_Entity_Basic_Decl;
Fld_26 : Internal_Entity_Base_Subp_Spec;
Fld_27 : Boolean;
Quantifier_Result_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_28 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_6 : Boolean;
Map_Result_4 : Internal_Entity_Array_Access;
Cast_Expr_5 : Internal_Entity;
Cast_Result_10 : Internal_Entity_Basic_Decl;
Fld_29 : Internal_Entity_Defining_Name;
Fld_30 : Symbol_Type;
Fld_31 : Bare_Basic_Decl;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_11 : Bare_Ada_Node;
New_Struct_2 : Internal_Metadata;
Cast_Result_12 : Bare_Ada_Node;
New_Struct_3 : Internal_Inner_Env_Assoc;
Singleton : Internal_Inner_Env_Assoc_Array_Access;
Sym : Symbol_Type;
Fld_32 : Bare_Basic_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_13 : Bare_Ada_Node;
New_Struct_4 : Internal_Metadata;
Cast_Result_14 : Bare_Ada_Node;
New_Struct_5 : Internal_Inner_Env_Assoc;
Singleton_1 : Internal_Inner_Env_Assoc_Array_Access;
Fld_33 : Internal_Entity_Defining_Name;
Sym_1 : Symbol_Type;
Fld_34 : Boolean;
If_Result_11 : Internal_Inner_Env_Assoc_Array_Access;
Concat_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Inner_Env_Assoc_Array_Access;
Map_Result_5 : Internal_Inner_Env_Assoc_Array_Access;
Fld_35 : Internal_Env_Assoc_Array_Access;
Fld_36 : Symbol_Type;
Fld_37 : Bare_Ada_Node;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_15 : Bare_Ada_Node;
New_Struct_6 : Internal_Metadata;
New_Struct_7 : Internal_Inner_Env_Assoc;
Map_Result_6 : Internal_Inner_Env_Assoc_Array_Access;
Concat_Result_2 : Internal_Inner_Env_Assoc_Array_Access;
Concat_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_553 is
            begin
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Map_Result_5);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Fld_35);
                     Dec_Ref (Map_Result_6);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Concat_Result_3);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_553;
            procedure Finalizer_Scope_2410 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2410;
            procedure Finalizer_Scope_2411 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2411;
            procedure Finalizer_Scope_2415 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_6);
            end Finalizer_Scope_2415;
            procedure Finalizer_Scope_2416 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2416;
            procedure Finalizer_Scope_2418 is
            begin
                     Dec_Ref (Map_Result_3);
            end Finalizer_Scope_2418;
            procedure Finalizer_Scope_2422 is
            begin
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_2422;
            procedure Finalizer_Scope_2425 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_2425;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Direct_Primitive_Subps then
         Property_Result := Self.Lf_Stg_Direct_Primitive_Subps;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4129 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 4054 '<FieldAccess for declarative_scope at ast.py:7508>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7508
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 4054
Block_Var := Fld; 
--# bind scope Block_Var
--# expr-start 4056 '<FieldAccess for is_derived_tagged_type at ast.py:7509>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7509
--# expr-start 4055 '<as_bare_entity at ast.py:7509>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7509
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4055
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.is_derived_tagged_type
Fld_1 := Libadalang.Implementation.Type_Decl_P_Is_Derived_Tagged_Type (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 4056
Block_Var_1 := Fld_1; 
--# bind is_derived_tagged Block_Var_1
--# expr-start 4058 '<as_bare_entity at ast.py:7510>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7510
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4057 '<parent at ast.py:7510>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7510
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_1.Parent; 
--# expr-done 4057
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null_1 := Result_Var = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Result_Var); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 4058
Var_Expr := If_Result_1;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 4073 '<Match at ast.py:7510>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7510
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind pkg_decl Match_0
--# expr-start 4065 '<Concat at ast.py:7514>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7514
--# expr-start 4061 '<TakeWhile at ???>' Map_Result None
   --# expr-start 4060 '<FieldAccess for decls at ast.py:7514>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7514
--# expr-start 4059 '<FieldAccess for public_part at ast.py:7514>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7514
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Public_Part (Node => Match_0.Node.Base_Package_Decl_F_Public_Part, Info => Match_0.Info);
--# expr-done 4059
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Ada_Node_List (Node => Fld_3.Node.Declarative_Part_F_Decls, Info => Fld_3.Info);
--# expr-done 4060
Fld_5 := Fld_4.Node;
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_5;
      begin
         for Untyped_Item_388 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_388 := Untyped_Item_388; 
Is_Null_2 := Bare_Item_388 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
Fld_6 := Fld_4.Info;
As_Entity_2 := (Info => Fld_6, Node => Bare_Item_388); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_388 := If_Result_2; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity := Item_388;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4061
--# expr-start 4064 '<TakeWhile at ???>' Map_Result_1 None
--# expr-start 4062 '<FieldAccess for private_part at ast.py:7515>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7515
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Private_Part (Node => Match_0.Node.Base_Package_Decl_F_Private_Part, Info => Match_0.Info);
--# expr-done 4062
Var_Expr_2 := Fld_7;
if Var_Expr_2 /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 4063 '<FieldAccess for decls at ast.py:7515>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7515
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_2.Node.Declarative_Part_F_Decls, Info => Var_Expr_2.Info);
--# expr-done 4063
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Ada_Node_List;
end if;
Fld_9 := Result_Var_1.Node;
      declare
         Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
      begin
      if Fld_9 = null then
      Map_Result_1 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_9;
      begin
         for Untyped_Item_389 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_389 := Untyped_Item_389; 
Is_Null_3 := Bare_Item_389 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity;
else
Fld_10 := Result_Var_1.Info;
As_Entity_3 := (Info => Fld_10, Node => Bare_Item_389); 
   If_Result_3 := As_Entity_3;
end if;
                  Item_389 := If_Result_3; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity := Item_389;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 4064
Concat_Result := Concat (Map_Result, Map_Result_1); 
--# expr-done 4065
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2411;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Synthetic_Formal_Param_Decl | Ada_Discrete_Base_Subtype_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 4072 '<If at ast.py:7521>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7521
if Block_Var_1 then
--# expr-start 4068 '<Cast to DeclarativePart at ast.py:7523>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7523
--# expr-start 4067 '<parent at ast.py:7523>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7523
--# expr-start 4066 '<parent at ast.py:7523>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7523
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Parent; 
--# expr-done 4066
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Fld_11.Parent; 
--# expr-done 4067
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Fld_12;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4068
Var_Expr_3 := Cast_Result;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind dp Var_Expr_3
--# expr-start 4071 '<TakeWhile at ???>' Map_Result_2 None
   --# expr-start 4070 '<FieldAccess for decls at ast.py:7524>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7524
--# expr-start 4069 '<as_bare_entity at ast.py:7524>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7524
Is_Null_4 := Var_Expr_3 = null; 
if Is_Null_4 then
   If_Result_4 := No_Entity_Declarative_Part;
else
As_Entity_4 := (Info => No_Entity_Info, Node => Var_Expr_3); 
   If_Result_4 := As_Entity_4;
end if;
--# expr-done 4069
      if If_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Ada_Node_List (Node => If_Result_4.Node.Declarative_Part_F_Decls, Info => If_Result_4.Info);
--# expr-done 4070
Fld_14 := Fld_13.Node;
      declare
         Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
      begin
      if Fld_14 = null then
      Map_Result_2 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_14;
      begin
         for Untyped_Item_390 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_390 := Untyped_Item_390; 
Is_Null_5 := Bare_Item_390 = null; 
if Is_Null_5 then
   If_Result_5 := No_Entity;
else
Fld_15 := Fld_13.Info;
As_Entity_5 := (Info => Fld_15, Node => Bare_Item_390); 
   If_Result_5 := As_Entity_5;
end if;
                  Item_390 := If_Result_5; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity := Item_390;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 4071
Scope_Result_2 := Map_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2416;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_6 := Result_Var_2;
else
   If_Result_6 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 4072
Let_Result_1 := If_Result_6; Inc_Ref (Let_Result_1);
--# end
Scope_Result_4 := Let_Result_1;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2415;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;
--# expr-done 4073
Scope_Result_5 := Match_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2410;
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Block_Var_2 := Result_Var_3; Inc_Ref (Block_Var_2);
--# bind decl_parts Block_Var_2
--# expr-start 4075 '<Cast to EnumTypeDef at ast.py:7529>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7529
--# expr-start 4074 '<FieldAccess for type_def at ast.py:7529>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7529
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.Type_Decl_F_Type_Def;
--# expr-done 4074
   if Fld_16 = null
      or else Fld_16.Kind in Ada_Enum_Type_Def_Range
   then
      Cast_Result_3 := Fld_16;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 4075
Var_Expr_4 := Cast_Result_3;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind etf Var_Expr_4
--# expr-start 4081 '<Map at ast.py:7530>' Map_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7530
   --# expr-start 4076 '<FieldAccess for enum_literals at ast.py:7530>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7530
      if Var_Expr_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Var_Expr_4.Enum_Type_Def_F_Enum_Literals;
--# expr-done 4076
      declare
         Map_Result_3_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      if Fld_17 = null then
      Map_Result_3 := Create_Internal_Inner_Env_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld_17;
      begin
         for Untyped_Item_146 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_146 := Untyped_Item_146; 
   --# scope-start
               --# bind lit Item_146
      --# expr-start 4080 '<New InnerEnvAssoc at ast.py:7531>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7531
--# expr-start 4078 '<FieldAccess for name_symbol at ast.py:7532>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7532
--# expr-start 4077 '<FieldAccess for name at ast.py:7532>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7532
      if Item_146 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Item_146.Enum_Literal_Decl_F_Name;
--# expr-done 4077
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_18);
--# end
--# expr-done 4078
--# expr-start 4079 '<New Metadata at ast.py:7534>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7534
Cast_Expr := Self; 
      Cast_Result_4 := Cast_Expr;
New_Struct := (Dottable_Subp => False, Primitive => Cast_Result_4, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 4079
Cast_Expr_1 := Item_146; 
      Cast_Result_5 := Cast_Expr_1;
New_Struct_1 := (Key => Fld_19, Metadata => New_Struct, Value => Cast_Result_5); 
--# expr-done 4080
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := New_Struct_1;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_3 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 4081
Scope_Result_6 := Map_Result_3;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2418;
   Result_Var_4 := Scope_Result_6;
else
   Result_Var_4 := No_Internal_Inner_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Block_Var_3 := Result_Var_4; Inc_Ref (Block_Var_3);
--# bind enum_lits Block_Var_3
--# expr-start 4120 '<Mapcat at ast.py:7564>' Map_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7564
   --# expr-start 4103 '<Filter at ast.py:7538>' Map_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7538
      declare
         Map_Result_4_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_2;
      begin
         for Item_148 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_148
--# expr-start 4082 '<Cast to BasicDecl at ast.py:7539>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7539
Cast_Expr_2 := Item_148; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_6 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4082
Var_Expr_6 := Cast_Result_6;
if Var_Expr_6 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4083 '<FieldAccess for subp_spec_or_null at ast.py:7539>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7539
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_20 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 4083
Scope_Result_7 := Fld_20;
   --# end
   Result_Var_5 := Scope_Result_7;
else
   Result_Var_5 := No_Entity_Base_Subp_Spec;
end if;
Var_Expr_5 := Result_Var_5;
if Var_Expr_5 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_5
--# expr-start 4087 '<Contains at ast.py:7540>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7540
   --# expr-start 4084 '<FieldAccess for candidate_primitive_subp_types at ast.py:7540>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7540
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_types
Fld_21 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Types (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 4084
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_21;
      begin
         for Item_149 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_149
      --# expr-start 4086 '<Eq at ???>' Is_Equiv None
Cast_Expr_3 := Item_149; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-start 4085 '<as_bare_entity at ast.py:7541>' If_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7541
Is_Null_6 := Self = null; 
if Is_Null_6 then
   If_Result_7 := No_Entity_Type_Decl;
else
As_Entity_6 := (Info => No_Entity_Info, Node => Self); 
   If_Result_7 := As_Entity_6;
end if;
--# expr-done 4085
      Cast_Result_8 := Create_Internal_Entity
        (Node => If_Result_7.Node,
         Info => If_Result_7.Info);
Is_Equiv := Equivalent (Cast_Result_7, Cast_Result_8); 
--# expr-done 4086
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 4087
if Quantifier_Result then
   --# expr-start 4102 '<If at ast.py:7542>' If_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7542
--# expr-start 4090 '<Not at ast.py:7549>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7549
--# expr-start 4089 '<IsA BasePackageDecl at ast.py:7549>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7549
--# expr-start 4088 '<parent at ast.py:7549>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7549
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Block_Var.Parent; 
--# expr-done 4088
Is_A := Fld_28 /= null 
and then Fld_28.Kind in Ada_Base_Package_Decl; 
--# expr-done 4089
Not_Val := not (Is_A); 
--# expr-done 4090
if Not_Val then
   --# expr-start 4101 '<bind at ast.py:7550>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7550
--# expr-start 4091 '<FieldAccess for origin_node at ast.py:7551>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7551
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_22 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Var_Expr_5.Node);
--# end
--# expr-done 4091
Bound_Origin := Fld_22; 
--# expr-start 4100 '<AnyQuantifier at ast.py:7554>' Quantifier_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7554
   --# expr-start 4096 '<EnvGet at ast.py:7552>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7552
--# expr-start 4093 '<FieldAccess for parent_primitives_env at ast.py:7552>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7552
--# expr-start 4092 '<as_bare_entity at ast.py:7552>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7552
Is_Null_7 := Self = null; 
if Is_Null_7 then
   If_Result_8 := No_Entity_Type_Decl;
else
As_Entity_7 := (Info => No_Entity_Info, Node => Self); 
   If_Result_8 := As_Entity_7;
end if;
--# expr-done 4092
      if If_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.parent_primitives_env
Fld_23 := Libadalang.Implementation.Type_Decl_P_Parent_Primitives_Env (Node => If_Result_8.Node, E_Info => If_Result_8.Info);
--# end
--# expr-done 4093
--# expr-start 4095 '<FieldAccess for name_symbol at ast.py:7553>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7553
--# expr-start 4094 '<FieldAccess for name at ast.py:7553>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7553
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_24 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 4094
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 4095
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_23, Key => Fld_25, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 4096
      Quantifier_Result_1 := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_150 of
               Collection.Items
         loop
   --# scope-start
               --# bind x Item_150
      --# expr-start 4099 '<FieldAccess for match_signature at ast.py:7556>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7556
--# expr-start 4098 '<FieldAccess for subp_spec_or_null at ast.py:7555>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7555
--# expr-start 4097 '<Cast to BasicDecl at ast.py:7555>' Cast_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7555
Cast_Expr_4 := Item_150; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_9 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_9 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4097
      if Cast_Result_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_26 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_9.Node, E_Info => Cast_Result_9.Info);
--# end
--# expr-done 4098
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_27 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_26.Node, Other => Var_Expr_5, Match_Name => False, Use_Entity_Info => True, Origin => Bound_Origin, E_Info => Fld_26.Info);
--# end
--# expr-done 4099
      Quantifier_Result_1 := Fld_27;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 4100
Dyn_Var_Bind_Result := Quantifier_Result_1; 
--# expr-done 4101
   If_Result_9 := Dyn_Var_Bind_Result;
else
   If_Result_9 := True;
end if;
--# expr-done 4102
   If_Result_10 := If_Result_9;
else
   If_Result_10 := False;
end if;
Scope_Result_8 := If_Result_10;
   --# end
      Finalizer_Scope_2422;
   Result_Var_6 := Scope_Result_8;
else
   Result_Var_6 := False;
end if;
         if Result_Var_6 then
         declare
            Item_To_Append : constant Internal_Entity := Item_148;
         begin
            Internal_Entity_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_4 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_4_Vec)));
         for I in Map_Result_4.Items'Range loop
            Map_Result_4.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_4_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_4.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_4_Vec);
      end;
--# expr-done 4103
      declare
         Map_Result_5_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result_4;
      begin
         for Item_147 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_147
      --# expr-start 4119 '<Let at ast.py:7565>' Let_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7565
--# scope-start
--# expr-start 4104 '<Cast to BasicDecl at ast.py:7566>' Cast_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7566
Cast_Expr_5 := Item_147; 
   if Cast_Expr_5.Node = null
      or else Cast_Expr_5.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_10 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
   else
         Cast_Result_10 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4104
Bd := Cast_Result_10; 
--# bind bd Bd
--# expr-start 4118 '<Concat at ast.py:7570>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7570
--# expr-start 4109 '<New InnerEnvAssoc at ast.py:7566>' New_Struct_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7566
--# expr-start 4106 '<FieldAccess for name_symbol at ast.py:7567>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7567
--# expr-start 4105 '<FieldAccess for defining_name at ast.py:7567>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7567
      if Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_29 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Bd.Node, E_Info => Bd.Info);
--# end
--# expr-done 4105
      if Fld_29.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_30 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_29.Node);
--# end
--# expr-done 4106
--# expr-start 4107 '<New Metadata at ast.py:7569>' New_Struct_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7569
Cast_Expr_6 := Self; 
      Cast_Result_11 := Cast_Expr_6;
New_Struct_2 := (Dottable_Subp => False, Primitive => Cast_Result_11, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 4107
--# expr-start 4108 '<FieldAccess for node at ast.py:7568>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7568
Fld_31 := Bd.Node;
--# expr-done 4108
      Cast_Result_12 := Fld_31;
New_Struct_3 := (Key => Fld_30, Metadata => New_Struct_2, Value => Cast_Result_12); 
--# expr-done 4109
                Singleton := Create_Internal_Inner_Env_Assoc_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct_3;
--# expr-start 4117 '<If at ast.py:7570>' If_Result_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7570
--# expr-start 4112 '<FieldAccess for name_is at ast.py:7571>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7571
--# expr-start 4110 '<FieldAccess for defining_name at ast.py:7571>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7571
      if Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_33 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Bd.Node, E_Info => Bd.Info);
--# end
--# expr-done 4110
      if Fld_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4111 '<Symbol "=" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 4111
--# property-call-start Name.name_is
Fld_34 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_33.Node, Sym => Sym_1);
--# end
--# expr-done 4112
if Fld_34 then
   --# expr-start 4116 '<New InnerEnvAssoc at ast.py:7572>' New_Struct_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7572
--# expr-start 4113 '<Symbol "/=" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 4113
--# expr-start 4114 '<New Metadata at ast.py:7575>' New_Struct_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7575
Cast_Expr_7 := Self; 
      Cast_Result_13 := Cast_Expr_7;
New_Struct_4 := (Dottable_Subp => False, Primitive => Cast_Result_13, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 4114
--# expr-start 4115 '<FieldAccess for node at ast.py:7574>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7574
Fld_32 := Bd.Node;
--# expr-done 4115
      Cast_Result_14 := Fld_32;
New_Struct_5 := (Key => Sym, Metadata => New_Struct_4, Value => Cast_Result_14); 
--# expr-done 4116
                Singleton_1 := Create_Internal_Inner_Env_Assoc_Array (Items_Count => 1);
                Singleton_1.Items (1) := New_Struct_5;
   If_Result_11 := Singleton_1;
else
   If_Result_11 := No_Internal_Inner_Env_Assoc_Array_Type;
end if;
      Inc_Ref (If_Result_11);
--# expr-done 4117
Concat_Result_1 := Concat (Singleton, If_Result_11); 
--# expr-done 4118
Let_Result_2 := Concat_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 4119
         for Item_To_Append of
               Let_Result_2.Items
         loop
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_5_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2425;
         end loop;
      end;
         Map_Result_5 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_5_Vec)));
         for I in Map_Result_5.Items'Range loop
            Map_Result_5.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_5_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_5.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_5_Vec);
      end;
--# expr-done 4120
Block_Var_4 := Map_Result_5; Inc_Ref (Block_Var_4);
--# bind prim_subps Block_Var_4
--# expr-start 4126 '<Map at ast.py:7583>' Map_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7583
   --# expr-start 4121 '<FieldAccess for predefined_operators at ast.py:7583>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7583
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.predefined_operators
Fld_35 := Libadalang.Implementation.Type_Decl_P_Predefined_Operators (Node => Self);
--# end
--# expr-done 4121
      declare
         Map_Result_6_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Env_Assoc_Array_Access := Fld_35;
      begin
         for Item_151 of
               Collection.Items
         loop
   --# scope-start
               --# bind assoc Item_151
      --# expr-start 4125 '<New InnerEnvAssoc at ast.py:7584>' New_Struct_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7584
--# expr-start 4122 '<FieldAccess for key at ast.py:7585>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7585
Fld_36 := Item_151.Key;
--# expr-done 4122
--# expr-start 4123 '<New Metadata at ast.py:7587>' New_Struct_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7587
Cast_Expr_8 := Self; 
      Cast_Result_15 := Cast_Expr_8;
New_Struct_6 := (Dottable_Subp => False, Primitive => Cast_Result_15, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 4123
--# expr-start 4124 '<FieldAccess for value at ast.py:7586>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7586
Fld_37 := Item_151.Value;
--# expr-done 4124
New_Struct_7 := (Key => Fld_36, Metadata => New_Struct_6, Value => Fld_37); 
--# expr-done 4125
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := New_Struct_7;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_6_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_6 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_6_Vec)));
         for I in Map_Result_6.Items'Range loop
            Map_Result_6.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_6_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_6.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_6_Vec);
      end;
--# expr-done 4126
Block_Var_5 := Map_Result_6; Inc_Ref (Block_Var_5);
--# bind predefined_ops Block_Var_5
--# expr-start 4128 '<Concat at ast.py:7591>' Concat_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7591
--# expr-start 4127 '<Concat at ast.py:7591>' Concat_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7591
Concat_Result_2 := Concat (Block_Var_3, Block_Var_4); 
--# expr-done 4127
Concat_Result_3 := Concat (Concat_Result_2, Block_Var_5); 
--# expr-done 4128
Let_Result_3 := Concat_Result_3; Inc_Ref (Let_Result_3);
--# end
--# expr-done 4129
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_553;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_553;
                     Finalizer_Scope_2410;
                     Finalizer_Scope_2411;
                     Finalizer_Scope_2415;
                     Finalizer_Scope_2416;
                     Finalizer_Scope_2418;
                     Finalizer_Scope_2422;
                     Finalizer_Scope_2425;
            raise;
      end;
      Self.Lf_Present_Direct_Primitive_Subps := True;
      Self.Lf_Stg_Direct_Primitive_Subps := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Decl_F_Direct_Primitive_Subps;
--# end
--# property-start TypeDecl.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7593
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Ndims
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4131 '<FieldAccess for array_ndims at ast.py:7593>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7593
--# expr-start 4130 '<FieldAccess for type_def at ast.py:7593>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7593
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4130
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4131
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Array_Ndims;
--# end
--# property-start TypeDecl.is_real_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7595
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Real_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4133 '<FieldAccess for is_real_type at ast.py:7595>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7595
--# expr-start 4132 '<FieldAccess for type_def at ast.py:7595>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7595
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4132
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.is_real_type
Fld_1 := Libadalang.Implementation.Type_Def_P_Is_Real_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4133
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Real_Type;
--# end
--# property-start TypeDecl.is_float_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7596
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Float_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4135 '<FieldAccess for is_float_type at ast.py:7596>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7596
--# expr-start 4134 '<FieldAccess for type_def at ast.py:7596>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7596
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4134
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4135
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Float_Type;
--# end
--# property-start TypeDecl.is_fixed_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7597
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4137 '<FieldAccess for is_fixed_point at ast.py:7597>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7597
--# expr-start 4136 '<FieldAccess for type_def at ast.py:7597>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7597
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4136
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4137
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start TypeDecl.is_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7598
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Int_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4139 '<FieldAccess for is_int_type at ast.py:7598>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7598
--# expr-start 4138 '<FieldAccess for type_def at ast.py:7598>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7598
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4138
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4139
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Int_Type;
--# end
--# property-start TypeDecl.is_access_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7599
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Access_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4142 '<FieldAccess for is_access_type at ast.py:7599>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7599
--# expr-start 4141 '<FieldAccess for type_def at ast.py:7599>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7599
--# expr-start 4140 '<as_bare_entity at ast.py:7599>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7599
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4140
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => If_Result.Node.Type_Decl_F_Type_Def, Info => If_Result.Info);
--# expr-done 4141
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Access_Type (Node => Fld.Node, Origin => Origin);
--# end
--# expr-done 4142
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Access_Type;
--# end
--# property-start TypeDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7600
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Static_Decl
  (Node : Bare_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4145 '<FieldAccess for is_static at ast.py:7600>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7600
--# expr-start 4144 '<FieldAccess for type_def at ast.py:7600>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7600
--# expr-start 4143 '<as_bare_entity at ast.py:7600>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7600
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4143
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => If_Result.Node.Type_Decl_F_Type_Def, Info => If_Result.Info);
--# expr-done 4144
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_static'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Static (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 4145
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Static_Decl;
--# end
--# property-start TypeDecl.accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7602
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Accessed_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_561 with Inline_Always;
      Block_Var : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Internal_Entity_Base_Type_Decl;
Bound_Include_Ud_Indexing : Boolean;
Fld_3 : Internal_Entity_Type_Def;
Fld_4 : Lexical_Env := Empty_Env;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Dyn_Var_Bind_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_561 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_561;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4160 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4146 '<FieldAccess for get_imp_deref at ast.py:7604>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7604
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4146
Block_Var := Fld; 
--# bind imp_deref Block_Var
--# expr-start 4159 '<If at ast.py:7606>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7606
--# expr-start 4147 '<IsNull at ast.py:7607>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7607
Is_Null := Block_Var.Node = null; 
--# expr-done 4147
if Is_Null then
   --# expr-start 4149 '<FieldAccess for accessed_type at ast.py:7608>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7608
--# expr-start 4148 '<FieldAccess for type_def at ast.py:7608>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7608
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4148
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4149
   If_Result := Fld_2;
else
   --# expr-start 4158 '<FieldAccess for accessed_type at ast.py:7622>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7622
--# expr-start 4157 '<FieldAccess for formal_type at ast.py:7622>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7622
--# expr-start 4156 '<Cast to BaseFormalParamDecl at ast.py:7622>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7622
--# expr-start 4155 '<bind at ast.py:7612>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7612
Bound_Include_Ud_Indexing := False; 
--# expr-start 4154 '<EnvGet at ast.py:7614>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7614
--# expr-start 4151 '<FieldAccess for defining_env at ast.py:7614>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7614
--# expr-start 4150 '<FieldAccess for type_def at ast.py:7614>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7614
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4150
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.defining_env
Fld_4 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, Include_Ud_Indexing => Bound_Include_Ud_Indexing, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld_3.Info);
--# end
--# expr-done 4151
--# expr-start 4153 '<FieldAccess for name_symbol at ast.py:7615>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7615
--# expr-start 4152 '<Cast to Name at ast.py:7615>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7615
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 4152
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result.Node);
--# end
--# expr-done 4153
Env_Get_Result := AST_Envs.Get_First (Self => Fld_4, Key => Fld_5, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 4154
Dyn_Var_Bind_Result := Env_Get_Result; 
--# expr-done 4155
   if Dyn_Var_Bind_Result.Node = null
      or else Dyn_Var_Bind_Result.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Dyn_Var_Bind_Result.Node,
         Info => Dyn_Var_Bind_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4156
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Cast_Result_1.Node, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4157
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 4158
   If_Result := Fld_7;
end if;
--# expr-done 4159
Let_Result := If_Result; 
--# end
--# expr-done 4160
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_561;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_561;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Accessed_Type;
--# end
--# property-start TypeDecl.access_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7625
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Access_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
      Match_0 : Internal_Entity_Access_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Access_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Access_Def;
Let_Result : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Access_Def;
Scope_Result_1 : Internal_Entity_Access_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Access_Def;
Scope_Result_2 : Internal_Entity_Access_Def;
Match_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4164 '<Match at ast.py:7625>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7625
--# expr-start 4161 '<FieldAccess for type_def at ast.py:7625>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7625
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4161
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ad Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind dtd Match_1
--# expr-start 4163 '<FieldAccess for access_def at ast.py:7627>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7627
--# expr-start 4162 '<FieldAccess for base_type at ast.py:7627>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7627
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4162
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4163
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Access_Def; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4164
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Access_Def;
--# end
--# property-start TypeDecl.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7631
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4166 '<FieldAccess for is_tagged_type at ast.py:7631>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7631
--# expr-start 4165 '<FieldAccess for type_def at ast.py:7631>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7631
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4165
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Tagged_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4166
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start TypeDecl.base_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7632
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4168 '<FieldAccess for base_type at ast.py:7632>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7632
--# expr-start 4167 '<FieldAccess for type_def at ast.py:7632>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7632
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4167
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4168
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Base_Type;
--# end
--# property-start TypeDecl.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7633
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Interfaces
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_565 with Inline_Always;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_565 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_565;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4170 '<FieldAccess for base_interfaces at ast.py:7633>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7633
--# expr-start 4169 '<FieldAccess for type_def at ast.py:7633>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7633
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4169
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Interfaces (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4170
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_565;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_565;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Base_Interfaces;
--# end
--# property-start TypeDecl.is_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7634
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Char_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4172 '<FieldAccess for is_char_type at ast.py:7634>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7634
--# expr-start 4171 '<FieldAccess for type_def at ast.py:7634>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4171
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4172
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Char_Type;
--# end
--# property-start TypeDecl.is_enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7635
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Enum_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4174 '<FieldAccess for is_enum_type at ast.py:7635>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7635
--# expr-start 4173 '<FieldAccess for type_def at ast.py:7635>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7635
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4173
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4174
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Enum_Type;
--# end
--# property-start TypeDecl.is_private /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7636
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Private
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Derived_Type_Def;
Fld : Bare_Type_Def;
Is_A : Boolean;
Fld_1 : Bare_Type_Def;
Cast_Result : Bare_Derived_Type_Def;
Fld_2 : Bare_With_Private;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4176 '<IsA PrivateTypeDef at ast.py:7637>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7637
--# expr-start 4175 '<FieldAccess for type_def at ast.py:7637>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7637
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 4175
Is_A := Fld /= null 
and then Fld.Kind in Ada_Private_Type_Def_Range; 
--# expr-done 4176
if Is_A then
   If_Result := True;
else
--# expr-start 4178 '<Cast to DerivedTypeDef at ast.py:7638>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7638
--# expr-start 4177 '<FieldAccess for type_def at ast.py:7638>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7638
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Type_Decl_F_Type_Def;
--# expr-done 4177
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4178
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind dtd Var_Expr
--# expr-start 4180 '<FieldAccess for as_bool at ast.py:7639>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7639
--# expr-start 4179 '<FieldAccess for has_with_private at ast.py:7639>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7639
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr.Derived_Type_Def_F_Has_With_Private;
--# expr-done 4179
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]WithPrivate.as_bool'
Fld_3 := Libadalang.Implementation.Dispatcher_With_Private_P_As_Bool (Node => Fld_2);
--# end
--# expr-done 4180
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Private;
--# end
--# property-start TypeDecl.is_derived_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7643
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Derived_Tagged_Type
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Type_Def;
Is_A : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4182 '<FieldAccess for is_tagged_type at ast.py:7644>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7644
--# expr-start 4181 '<FieldAccess for type_def at ast.py:7644>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7644
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4181
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Tagged_Type (Node => Fld.Node, Origin => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 4182
if Fld_1 then
   --# expr-start 4184 '<IsA DerivedTypeDef, InterfaceTypeDef at ast.py:7645>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7645
--# expr-start 4183 '<FieldAccess for type_def at ast.py:7645>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7645
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4183
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 4184
   If_Result := Is_A;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Derived_Tagged_Type;
--# end
--# property-start TypeDecl.array_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7648
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Match_0 : Internal_Entity_Array_Type_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Array_Type_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Array_Type_Def;
Let_Result : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Array_Type_Def;
Scope_Result_1 : Internal_Entity_Array_Type_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Array_Type_Def;
Scope_Result_2 : Internal_Entity_Array_Type_Def;
Match_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4188 '<Match at ast.py:7648>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7648
--# expr-start 4185 '<FieldAccess for type_def at ast.py:7648>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7648
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4185
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind atd Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind dtd Match_1
--# expr-start 4187 '<FieldAccess for array_def at ast.py:7650>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7650
--# expr-start 4186 '<FieldAccess for base_type at ast.py:7650>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7650
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4186
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4187
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Array_Type_Def; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4188
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Array_Def;
--# end
--# property-start TypeDecl.root_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7654
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Root_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Def;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4192 '<Match at ast.py:7654>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7654
--# expr-start 4189 '<FieldAccess for type_def at ast.py:7654>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7654
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4189
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind dtd Match_0
--# expr-start 4191 '<FieldAccess for root_type at ast.py:7655>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7655
--# expr-start 4190 '<FieldAccess for base_type at ast.py:7655>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7655
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4190
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4191
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Cast_Expr := Ent; 
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4192
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Root_Type;
--# end
--# property-start TypeDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7659
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Defining_Env
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_572 with Inline_Always;
            procedure Finalizer_Scope_2446 with Inline_Always;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
If_Result : Boolean;
Bound_Include_Ud_Indexing : Boolean;
Fld_6 : Internal_Entity_Basic_Decl_Array_Access;
Fld_7 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_8 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Array_Lit_1 : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_572 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_572;
            procedure Finalizer_Scope_2446 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2446;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4213 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4193 '<FieldAccess for get_imp_deref at ast.py:7661>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7661
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4193
Block_Var := Fld; 
--# bind imp_deref Block_Var
--# expr-start 4195 '<FieldAccess for defining_env at ast.py:7665>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7665
--# expr-start 4194 '<FieldAccess for type_def at ast.py:7665>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7665
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4194
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.defining_env
Fld_2 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld_1.Info);
--# end
--# expr-done 4195
Block_Var_1 := Fld_2; Inc_Ref (Block_Var_1);
--# bind self_env Block_Var_1
--# expr-start 4212 '<Cond at ast.py:7667>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7667
--# expr-start 4197 '<Not at ast.py:7668>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7668
--# expr-start 4196 '<IsNull at ast.py:7668>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7668
Is_Null := Block_Var.Node = null; 
--# expr-done 4196
Not_Val := not (Is_Null); 
--# expr-done 4197
if Not_Val then
   --# expr-start 4201 '<env_group at ast.py:7669>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7669
--# expr-start 4200 '<ArrayLiteral at ast.py:7669>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7669
--# expr-start 4199 '<FieldAccess for defining_env at ast.py:7669>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7669
--# expr-start 4198 '<FieldAccess for accessed_type at ast.py:7669>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7669
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4198
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld_3.Info);
--# end
--# expr-done 4199
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_1, 2 => Fld_4)); 
--# expr-done 4200
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 4201
   If_Result_2 := Group_Env;
else
if Include_Ud_Indexing then
   --# expr-start 4202 '<FieldAccess for has_ud_indexing at ast.py:7671>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7671
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4202
   If_Result := Fld_5;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4211 '<bind at ast.py:7672>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7672
Bound_Include_Ud_Indexing := False; 
--# expr-start 4210 '<env_group at ast.py:7677>' Group_Env_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7677
--# expr-start 4209 '<Concat at ast.py:7677>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7677
--# expr-start 4207 '<Map at ast.py:7676>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7676
   --# expr-start 4205 '<Concat at ast.py:7675>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7675
--# expr-start 4203 '<FieldAccess for constant_indexing_fns at ast.py:7674>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7674
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4203
--# expr-start 4204 '<FieldAccess for variable_indexing_fns at ast.py:7675>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7675
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4204
Concat_Result := Concat (Fld_6, Fld_7); 
--# expr-done 4205
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_152 of
               Collection.Items
         loop
   --# scope-start
               --# bind fn Item_152
      --# expr-start 4206 '<FieldAccess for defining_env at ast.py:7676>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7676
      if Item_152.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Item_152.Node, Origin => Origin, Include_Ud_Indexing => Bound_Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Item_152.Info);
--# end
--# expr-done 4206
         declare
            Item_To_Append : constant Lexical_Env := Fld_8;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2446;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4207
--# expr-start 4208 '<ArrayLiteral at ???>' Array_Lit_1 None
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_1)); 
--# expr-done 4208
Concat_Result_1 := Concat (Map_Result, Array_Lit_1); 
--# expr-done 4209
Group_Env_1 := Group (Concat_Result_1, No_Metadata); 
--# expr-done 4210
Dyn_Var_Bind_Result := Group_Env_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4211
   If_Result_1 := Dyn_Var_Bind_Result;
else
   If_Result_1 := Block_Var_1;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4212
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4213
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_572;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_572;
                     Finalizer_Scope_2446;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Defining_Env;
--# end
--# property-start TypeDecl.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7683
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Predefined_Operators
  (Node : Bare_Type_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_573 with Inline_Always;
      Fld : Bare_Type_Def;
Fld_1 : Internal_Env_Assoc_Array_Access;
Fld_2 : Internal_Entity;
Fld_3 : Bare_Ada_Node;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Fld_4 : Internal_Entity;
Fld_5 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_573 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_573;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4223 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4222 '<If at ast.py:7693>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7693
--# expr-start 4216 '<Eq at ast.py:7694>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7694
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-start 4215 '<FieldAccess for node at ast.py:7694>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7694
--# expr-start 4214 '<FieldAccess for universal_int_type at ast.py:7694>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7694
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 4214
Fld_3 := Fld_2.Node;
--# expr-done 4215
Is_Equal := Cast_Result = Fld_3; 
--# expr-done 4216
if Is_Equal then
   If_Result := True;
else
   --# expr-start 4219 '<Eq at ast.py:7694>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7694
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
--# expr-start 4218 '<FieldAccess for node at ast.py:7695>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7695
--# expr-start 4217 '<FieldAccess for universal_real_type at ast.py:7695>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7695
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 4217
Fld_5 := Fld_4.Node;
--# expr-done 4218
Is_Equal_1 := Cast_Result_1 = Fld_5; 
--# expr-done 4219
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := No_Internal_Env_Assoc_Array_Type;
else
   --# expr-start 4221 '<FieldAccess for predefined_operators at ast.py:7697>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7697
--# expr-start 4220 '<FieldAccess for type_def at ast.py:7697>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7697
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 4220
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.predefined_operators'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Predefined_Operators (Node => Fld);
--# end
--# expr-done 4221
   If_Result_1 := Fld_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4222
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4223
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_573;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_573;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Type_Decl_P_Predefined_Operators;
--# end
--# property-start TypeDecl.record_def /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7715
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Record_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Match_0 : Internal_Entity_Record_Type_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Record_Def;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Record_Def;
Scope_Result : Internal_Entity_Base_Record_Def;
Result_Var : Internal_Entity_Base_Record_Def;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Record_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Record_Type_Def;
Let_Result : Internal_Entity_Base_Record_Def;
Scope_Result_1 : Internal_Entity_Base_Record_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Base_Record_Def;
Scope_Result_2 : Internal_Entity_Base_Record_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Base_Record_Def;
Scope_Result_3 : Internal_Entity_Base_Record_Def;
Match_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4231 '<Match at ast.py:7716>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7716
--# expr-start 4224 '<FieldAccess for type_def at ast.py:7716>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7716
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4224
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Record_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Record_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind r Match_0
--# expr-start 4225 '<FieldAccess for record_def at ast.py:7717>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7717
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Base_Record_Def (Node => Match_0.Node.Record_Type_Def_F_Record_Def, Info => Match_0.Info);
--# expr-done 4225
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind d Match_1
--# expr-start 4230 '<If at ast.py:7720>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7720
--# expr-start 4226 '<FieldAccess for is_tagged_type at ast.py:7721>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7721
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4226
if Fld_5 then
   --# expr-start 4227 '<FieldAccess for record_extension at ast.py:7722>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7722
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Record_Def (Node => Match_1.Node.Derived_Type_Def_F_Record_Extension, Info => Match_1.Info);
--# expr-done 4227
   If_Result := Fld_2;
else
--# expr-start 4228 '<FieldAccess for base_type at ast.py:7723>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7723
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4228
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4229 '<FieldAccess for record_def at ast.py:7723>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7723
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4229
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Record_Def;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 4230
Let_Result_1 := If_Result; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Record_Def; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 4231
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Record_Def;
--# end
--# property-start TypeDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7729
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start TypeDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7731
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Xref_Equation
  (Node : Bare_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_576 with Inline_Always;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_576 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_576;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4234 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4233 '<FieldAccess for sub_equation at ast.py:7734>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7734
--# expr-start 4232 '<FieldAccess for type_def at ast.py:7734>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7734
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4232
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4233
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4234
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_576;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_576;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Xref_Equation;
--# end
--# property-start TypeDecl.is_discrete_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7736
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4236 '<FieldAccess for is_discrete_type at ast.py:7736>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7736
--# expr-start 4235 '<FieldAccess for type_def at ast.py:7736>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7736
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4235
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Discrete_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4236
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start TypeDecl.own_primitives_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7738
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Own_Primitives_Env
  (Node : Bare_Type_Decl
      ; With_Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_578 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Rebound_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_578 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Rebound_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_578;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4239 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4238 '<rebind_env at ast.py:7744>' Rebound_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7744
--# expr-start 4237 '<FieldAccess for direct_primitives_env at ast.py:7744>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7744
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.direct_primitives_env
Fld := Libadalang.Implementation.Type_Decl_F_Direct_Primitives_Env (Node => Ent.Node);
--# end
--# expr-done 4237
Rebound_Env := Rebind_Env (Fld, With_Rebindings); 
--# expr-done 4238
Let_Result := Rebound_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 4239
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_578;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_578;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Own_Primitives_Env;
--# end
--# property-start TypeDecl.own_primitives_envs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7746
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Own_Primitives_Envs
  (Node : Bare_Type_Decl
      ; With_Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_579 with Inline_Always;
            procedure Finalizer_Scope_2452 with Inline_Always;
      Var_Expr : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Scope_Result : Lexical_Env_Array_Access;
Fld_3 : Lexical_Env := Empty_Env;
Singleton : Lexical_Env_Array_Access;
Result_Var : Lexical_Env_Array_Access;
Let_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_579 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Singleton);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_579;
            procedure Finalizer_Scope_2452 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2452;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4246 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4241 '<Cast to TypeDecl at ast.py:7754>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7754
--# expr-start 4240 '<FieldAccess for previous_part at ast.py:7754>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7754
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 4240
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 4241
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 4244 '<ArrayLiteral at ast.py:7755>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7755
--# expr-start 4242 '<FieldAccess for own_primitives_env at ast.py:7756>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7756
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_1 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 4242
--# expr-start 4243 '<FieldAccess for own_primitives_env at ast.py:7757>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7757
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_2 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Var_Expr.Node, With_Rebindings => With_Rebindings, E_Info => Var_Expr.Info);
--# end
--# expr-done 4243
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_1, 2 => Fld_2)); 
--# expr-done 4244
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2452;
   Result_Var := Scope_Result;
else
   --# expr-start 4245 '<FieldAccess for own_primitives_env at ast.py:7759>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7759
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_3 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 4245
                Singleton := Create_Lexical_Env_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_3;
                Inc_Ref (Singleton.Items (1));
   Result_Var := Singleton;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 4246
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_579;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_579;
                     Finalizer_Scope_2452;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Own_Primitives_Envs;
--# end
--# property-start TypeDecl.primitives_envs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7762
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Primitives_Envs
  (Node : Bare_Type_Decl
      ; With_Rebindings : Env_Rebindings
      ; Stop_At : Internal_Entity_Base_Type_Decl_Array_Access
      ; Include_Self : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   --# bind stop_at Stop_At
   --# bind include_self Include_Self
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_580 with Inline_Always;
            procedure Finalizer_Scope_2453 with Inline_Always;
            procedure Finalizer_Scope_2457 with Inline_Always;
      Var_Expr : Internal_Entity_Type_Decl;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Base_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Bare_Base_Subtype_Decl;
Fld_2 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Subtype_Decl;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;
Scope_Result_2 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;
Fld_4 : Lexical_Env_Array_Access;
Fld_5 : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Type_Decl;
Cast_Result_6 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
If_Result : Lexical_Env_Array_Access;
Scope_Result_3 : Lexical_Env_Array_Access;
Result_Var : Lexical_Env_Array_Access;
Map_Result : Lexical_Env_Array_Access;
Fld_6 : Lexical_Env_Array_Access;
If_Result_1 : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Dyn_Var_Bind_Result_1 : Lexical_Env_Array_Access;
Let_Result_3 : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_580 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_580;
            procedure Finalizer_Scope_2453 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2453;
            procedure Finalizer_Scope_2457 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2457;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4265 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 4264 '<bind at ast.py:7775>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7775
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 4263 '<Concat at ast.py:7788>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7788
--# expr-start 4260 '<Mapcat at ast.py:7775>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7775
   --# expr-start 4247 '<FieldAccess for base_types at ast.py:7775>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7775
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 4247
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld;
      begin
         for Item_153 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_153
--# expr-start 4253 '<Match at ast.py:7775>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7775
      if Item_153.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_153; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind td Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind std Match_1
--# expr-start 4252 '<bind at ast.py:7777>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7777
--# expr-start 4249 '<FieldAccess for origin_node at ast.py:7778>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7778
--# expr-start 4248 '<FieldAccess for node at ast.py:7778>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7778
Fld_1 := Match_1.Node;
--# expr-done 4248
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_1);
--# end
--# expr-done 4249
Bound_Origin_1 := Fld_2; 
--# expr-start 4251 '<Cast to TypeDecl at ast.py:7778>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7778
--# expr-start 4250 '<FieldAccess for get_type at ast.py:7778>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7778
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_1.Node, Origin => Bound_Origin_1, E_Info => Match_1.Info);
--# end
--# expr-done 4250
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 4251
Dyn_Var_Bind_Result := Cast_Result_1; 
--# expr-done 4252
Let_Result_1 := Dyn_Var_Bind_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4253
Var_Expr := Match_Result;
if Var_Expr /= No_Entity_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 4259 '<If at ast.py:7782>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7782
--# expr-start 4255 '<Contains at ast.py:7783>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7783
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Stop_At;
      begin
         for Item_154 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_154
      --# expr-start 4254 '<Eq at ???>' Is_Equiv None
Cast_Expr_1 := Item_154; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_5, Cast_Result_6); 
--# expr-done 4254
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 4255
if Quantifier_Result then
   If_Result := No_Lexical_Env_Array_Type;
else
   --# expr-start 4258 '<Concat at ast.py:7786>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7786
--# expr-start 4256 '<FieldAccess for own_primitives_envs at ast.py:7785>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7785
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.own_primitives_envs
Fld_4 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Envs (Node => Var_Expr.Node, With_Rebindings => With_Rebindings, E_Info => Var_Expr.Info);
--# end
--# expr-done 4256
--# expr-start 4257 '<FieldAccess for primitives_envs at ast.py:7786>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7786
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.primitives_envs
Fld_5 := Libadalang.Implementation.Type_Decl_P_Primitives_Envs (Node => Var_Expr.Node, With_Rebindings => With_Rebindings, Stop_At => Stop_At, Include_Self => False, E_Info => Var_Expr.Info);
--# end
--# expr-done 4257
Concat_Result := Concat (Fld_4, Fld_5); 
--# expr-done 4258
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 4259
Scope_Result_3 := If_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2457;
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Lexical_Env_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2453;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4260
--# expr-start 4262 '<If at ast.py:7789>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7789
if Include_Self then
   --# expr-start 4261 '<FieldAccess for own_primitives_envs at ast.py:7790>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7790
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.own_primitives_envs
Fld_6 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Envs (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 4261
   If_Result_1 := Fld_6;
else
   If_Result_1 := No_Lexical_Env_Array_Type;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4262
Concat_Result_1 := Concat (Map_Result, If_Result_1); 
--# expr-done 4263
Dyn_Var_Bind_Result_1 := Concat_Result_1; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 4264
Let_Result_3 := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 4265
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_580;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_580;
                     Finalizer_Scope_2453;
                     Finalizer_Scope_2457;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Primitives_Envs;
--# end
--# property-start TypeDecl.compute_primitives_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7794
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Compute_Primitives_Env
  (Node : Bare_Type_Decl
      ; Include_Self : Boolean
         := True
      ; Stop_At : Internal_Entity_Base_Type_Decl_Array_Access
         := No_Internal_Entity_Base_Type_Decl_Array_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_self Include_Self
   --# bind stop_at Stop_At
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_581 with Inline_Always;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env_Array_Access;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_581 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_581;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Compute_Primitives_Env,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Self);
               Mmz_K.Items (3) := (Kind => Mmz_Internal_Entity_Base_Type_Decl_Array_Access,
                                      As_Internal_Entity_Base_Type_Decl_Array_Access => Stop_At);
                  Inc_Ref (Mmz_K.Items (3).As_Internal_Entity_Base_Type_Decl_Array_Access);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4271 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4270 '<env_group at ast.py:7807>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7807
--# expr-start 4268 '<FieldAccess for primitives_envs at ast.py:7803>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7803
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4267 '<FieldAccess for rebindings at ast.py:7804>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7804
--# expr-start 4266 '<FieldAccess for info at ast.py:7804>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7804
Fld := Ent.Info;
--# expr-done 4266
Fld_1 := Fld.Rebindings;
--# expr-done 4267
--# property-call-start TypeDecl.primitives_envs
Fld_2 := Libadalang.Implementation.Type_Decl_P_Primitives_Envs (Node => Ent.Node, With_Rebindings => Fld_1, Stop_At => Stop_At, Include_Self => Include_Self, E_Info => Ent.Info);
--# end
--# expr-done 4268
--# expr-start 4269 '<New Metadata at ast.py:7808>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7808
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
New_Struct := (Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => Cast_Result); 
--# expr-done 4269
Group_Env := Group (Fld_2, New_Struct); 
--# expr-done 4270
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 4271
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_581;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_581;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Type_Decl_P_Compute_Primitives_Env;
--# end
--# property-start TypeDecl.parent_primitives_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7811
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Parent_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_582 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Bare_Type_Def;
Is_A : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_582 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_582;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4277 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4276 '<If at ast.py:7813>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7813
--# expr-start 4273 '<IsA DerivedTypeDef, InterfaceTypeDef at ast.py:7814>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7814
--# expr-start 4272 '<FieldAccess for type_def at ast.py:7814>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7814
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Decl_F_Type_Def;
--# expr-done 4272
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 4273
if Is_A then
   --# expr-start 4274 '<FieldAccess for compute_primitives_env at ast.py:7815>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7815
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => False, E_Info => Ent.Info);
--# end
--# expr-done 4274
   If_Result := Fld;
else
   --# expr-start 4275 '<FieldAccess for empty_env at ast.py:7816>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7816
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 4275
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 4276
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4277
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_582;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_582;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Parent_Primitives_Env;
--# end
--# property-start TypeDecl.refined_parent_primitives_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7819
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Refined_Parent_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_583 with Inline_Always;
            procedure Finalizer_Scope_2459 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_583 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_583;
            procedure Finalizer_Scope_2459 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2459;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4281 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4280 '<FieldAccess for compute_primitives_env at ast.py:7841>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7841
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4278 '<FieldAccess for previous_part at ast.py:7843>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7843
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4278
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4279 '<FieldAccess for base_types at ast.py:7843>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7843
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Var_Expr.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4279
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2459;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# property-call-start TypeDecl.compute_primitives_env
Fld_2 := Libadalang.Implementation.Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => False, Stop_At => Result_Var, E_Info => Ent.Info);
--# end
--# expr-done 4280
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4281
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_583;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_583;
                     Finalizer_Scope_2459;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Refined_Parent_Primitives_Env;
--# end
--# property-start TypeDecl.primitives_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7846
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_584 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_584 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_584;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4283 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4282 '<FieldAccess for compute_primitives_env at ast.py:7848>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7848
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => True, E_Info => Ent.Info);
--# end
--# expr-done 4282
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4283
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_584;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_584;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Primitives_Env;
--# end
--# property-start TypeDecl.get_primitives /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7850
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Get_Primitives
  (Node : Bare_Type_Decl
      ; Only_Inherited : Boolean
         := False
      ; Include_Predefined_Operators : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind only_inherited Only_Inherited
   --# bind include_predefined_operators Include_Predefined_Operators
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_585 with Inline_Always;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
A_Spec : Internal_Entity_Base_Subp_Spec;
A_Prim : Internal_Entity_Base_Type_Decl;
B_Prim : Bare_Base_Type_Decl;
B_Prim_Ent : Internal_Entity_Base_Type_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
If_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Index_158 : Integer;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Bare_Defining_Name;
Fld_11 : Internal_Entity_Defining_Name;
Fld_12 : Bare_Defining_Name;
Cast_Result_3 : Bare_Name;
Fld_13 : Boolean;
Not_Val_1 : Boolean;
Fld_14 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_15 : Internal_Entity_Base_Subp_Spec;
Fld_16 : Boolean;
Not_Val_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result_3 : Boolean;
As_Entity_1 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity_Base_Type_Decl;
Comp_Result : Boolean;
Fld_17 : Bare_Base_Type_Decl;
Fld_18 : Boolean;
Not_Val_3 : Boolean;
If_Result_5 : Boolean;
Comp_Result_1 : Boolean;
Fld_19 : Bare_Base_Type_Decl;
Fld_20 : Boolean;
Not_Val_4 : Boolean;
If_Result_6 : Boolean;
Fld_21 : Bare_Base_Type_Decl;
Fld_22 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
If_Result_8 : Boolean;
Let_Result_1 : Boolean;
Index_160 : Integer;
Quantifier_Result : Boolean;
Let_Result_2 : Boolean;
Map_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_585 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_585;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4330 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 4286 '<If at ast.py:7860>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7860
if Only_Inherited then
   --# expr-start 4284 '<FieldAccess for parent_primitives_env at ast.py:7862>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7862
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.parent_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Parent_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4284
   If_Result := Fld;
else
   --# expr-start 4285 '<FieldAccess for primitives_env at ast.py:7863>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7863
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4285
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 4286
Block_Var := If_Result; Inc_Ref (Block_Var);
--# bind prim_env Block_Var
--# expr-start 4289 '<Map at ast.py:7866>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7866
   --# expr-start 4287 '<EnvGet at ast.py:7866>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7866
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 4287
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_155 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_155
      --# expr-start 4288 '<Cast to BasicDecl at ast.py:7867>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7867
Cast_Expr := Item_155; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4288
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4289
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind all_prims Block_Var_1
--# expr-start 4293 '<If at ast.py:7870>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7870
if Include_Predefined_Operators then
   If_Result_1 := Block_Var_1;
else
   --# expr-start 4292 '<Filter at ast.py:7873>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7873
      declare
         Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
      begin
         for Item_156 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_156
         --# expr-start 4291 '<Not at ast.py:7873>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7873
--# expr-start 4290 '<IsA SyntheticSubpDecl at ast.py:7873>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7873
Is_A := Item_156.Node /= null 
and then Item_156.Node.Kind in Ada_Synthetic_Subp_Decl_Range; 
--# expr-done 4290
Not_Val := not (Is_A); 
--# expr-done 4291
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Item_156;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 4292
   If_Result_1 := Map_Result_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4293
Block_Var_2 := If_Result_1; Inc_Ref (Block_Var_2);
--# bind bds Block_Var_2
--# expr-start 4329 '<Filter at ast.py:7878>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7878
      declare
         Map_Result_2_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
         Index_158 := 0;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_2;
      begin
         for Item_157 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_157
               --# bind i Index_158
         --# expr-start 4328 '<Let at ast.py:7878>' Let_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7878
--# scope-start
--# expr-start 4294 '<FieldAccess for subp_spec_or_null at ast.py:7880>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7880
      if Item_157.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_157.Node, E_Info => Item_157.Info);
--# end
--# expr-done 4294
A_Spec := Fld_2; 
--# bind a_spec A_Spec
--# expr-start 4299 '<Cast to BaseTypeDecl at ast.py:7881>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7881
--# expr-start 4298 '<as_bare_entity at ast.py:7881>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7881
--# expr-start 4297 '<FieldAccess for primitive at ast.py:7881>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7881
--# expr-start 4296 '<FieldAccess for md at ast.py:7881>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7881
--# expr-start 4295 '<FieldAccess for info at ast.py:7881>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7881
Fld_3 := Item_157.Info;
--# expr-done 4295
Fld_4 := Fld_3.Md;
--# expr-done 4296
Fld_5 := Fld_4.Primitive;
--# expr-done 4297
Is_Null := Fld_5 = null; 
if Is_Null then
   If_Result_2 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_5); 
   If_Result_2 := As_Entity;
end if;
--# expr-done 4298
   if If_Result_2.Node = null
      or else If_Result_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4299
A_Prim := Cast_Result_1; 
--# bind a_prim A_Prim
--# expr-start 4327 '<AllQuantifier at ast.py:7883>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7883
      Quantifier_Result := True;
         Index_160 := 0;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_2;
      begin
         for Item_159 of
               Collection.Items
         loop
   --# scope-start
               --# bind b Item_159
               --# bind j Index_160
      --# expr-start 4326 '<Let at ast.py:7883>' Let_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7883
--# scope-start
--# expr-start 4303 '<Cast to BaseTypeDecl at ast.py:7884>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7884
--# expr-start 4302 '<FieldAccess for primitive at ast.py:7884>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7884
--# expr-start 4301 '<FieldAccess for md at ast.py:7884>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7884
--# expr-start 4300 '<FieldAccess for info at ast.py:7884>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7884
Fld_6 := Item_159.Info;
--# expr-done 4300
Fld_7 := Fld_6.Md;
--# expr-done 4301
Fld_8 := Fld_7.Primitive;
--# expr-done 4302
   if Fld_8 = null
      or else Fld_8.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Fld_8;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 4303
B_Prim := Cast_Result_2; 
--# bind b_prim B_Prim
--# expr-start 4309 '<Not at ast.py:7891>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7891
--# expr-start 4308 '<FieldAccess for matches at ast.py:7891>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7891
--# expr-start 4305 '<FieldAccess for node at ast.py:7891>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7891
--# expr-start 4304 '<FieldAccess for defining_name at ast.py:7891>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7891
      if Item_157.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_157.Node, E_Info => Item_157.Info);
--# end
--# expr-done 4304
Fld_10 := Fld_9.Node;
--# expr-done 4305
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4307 '<FieldAccess for node at ast.py:7891>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7891
--# expr-start 4306 '<FieldAccess for defining_name at ast.py:7891>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7891
      if Item_159.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_159.Node, E_Info => Item_159.Info);
--# end
--# expr-done 4306
Fld_12 := Fld_11.Node;
--# expr-done 4307
      Cast_Result_3 := Fld_12;
--# property-call-start Name.matches
Fld_13 := Libadalang.Implementation.Name_P_Matches (Node => Fld_10, N => Cast_Result_3);
--# end
--# expr-done 4308
Not_Val_1 := not (Fld_13); 
--# expr-done 4309
if Not_Val_1 then
   If_Result_3 := True;
else
   --# expr-start 4314 '<bind at ast.py:7894>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7894
--# expr-start 4310 '<FieldAccess for origin_node at ast.py:7894>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7894
      if Item_159.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_14 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Item_159.Node);
--# end
--# expr-done 4310
Bound_Origin := Fld_14; 
--# expr-start 4313 '<Not at ast.py:7894>' Not_Val_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7894
--# expr-start 4312 '<FieldAccess for match_signature at ast.py:7894>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7894
      if A_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4311 '<FieldAccess for subp_spec_or_null at ast.py:7895>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7895
      if Item_159.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_159.Node, E_Info => Item_159.Info);
--# end
--# expr-done 4311
--# property-call-start BaseSubpSpec.match_signature
Fld_16 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => A_Spec.Node, Other => Fld_15, Match_Name => False, Use_Entity_Info => True, Origin => Bound_Origin, E_Info => A_Spec.Info);
--# end
--# expr-done 4312
Not_Val_2 := not (Fld_16); 
--# expr-done 4313
Dyn_Var_Bind_Result := Not_Val_2; 
--# expr-done 4314
   If_Result_3 := Dyn_Var_Bind_Result;
end if;
if If_Result_3 then
   If_Result_8 := True;
else
   --# expr-start 4325 '<Let at ast.py:7899>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7899
--# scope-start
--# expr-start 4315 '<as_bare_entity at ast.py:7899>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7899
Is_Null_1 := B_Prim = null; 
if Is_Null_1 then
   If_Result_4 := No_Entity_Base_Type_Decl;
else
As_Entity_1 := (Info => No_Entity_Info, Node => B_Prim); 
   If_Result_4 := As_Entity_1;
end if;
--# expr-done 4315
B_Prim_Ent := If_Result_4; 
--# bind b_prim_ent B_Prim_Ent
--# expr-start 4324 '<If at ast.py:7899>' If_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7899
--# expr-start 4317 '<FieldAccess for has_base_type at ast.py:7902>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7902
      if A_Prim.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4316 '<FieldAccess for node at ast.py:7902>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7902
Fld_21 := B_Prim_Ent.Node;
--# expr-done 4316
--# property-call-start BaseTypeDecl.has_base_type
Fld_22 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type (Node => A_Prim.Node, Target => Fld_21, E_Info => A_Prim.Info);
--# end
--# expr-done 4317
if Fld_22 then
Comp_Result := Index_158 >= Index_160; 
if Comp_Result then
   If_Result_5 := True;
else
   --# expr-start 4320 '<Not at ast.py:7914>' Not_Val_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7914
--# expr-start 4319 '<FieldAccess for has_base_type at ast.py:7914>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7914
      if B_Prim_Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4318 '<FieldAccess for node at ast.py:7914>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7914
Fld_17 := A_Prim.Node;
--# expr-done 4318
--# property-call-start BaseTypeDecl.has_base_type
Fld_18 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type (Node => B_Prim_Ent.Node, Target => Fld_17, E_Info => B_Prim_Ent.Info);
--# end
--# expr-done 4319
Not_Val_3 := not (Fld_18); 
--# expr-done 4320
   If_Result_5 := Not_Val_3;
end if;
   If_Result_7 := If_Result_5;
else
Comp_Result_1 := Index_158 <= Index_160; 
if Comp_Result_1 then
   --# expr-start 4323 '<Not at ast.py:7926>' Not_Val_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7926
--# expr-start 4322 '<FieldAccess for has_base_type at ast.py:7926>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7926
      if B_Prim_Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4321 '<FieldAccess for node at ast.py:7926>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7926
Fld_19 := A_Prim.Node;
--# expr-done 4321
--# property-call-start BaseTypeDecl.has_base_type
Fld_20 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type (Node => B_Prim_Ent.Node, Target => Fld_19, E_Info => B_Prim_Ent.Info);
--# end
--# expr-done 4322
Not_Val_4 := not (Fld_20); 
--# expr-done 4323
   If_Result_6 := Not_Val_4;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
--# expr-done 4324
Let_Result := If_Result_7; 
--# end
--# expr-done 4325
   If_Result_8 := Let_Result;
end if;
Let_Result_1 := If_Result_8; 
--# end
--# expr-done 4326
      Quantifier_Result := Let_Result_1;
   --# end
               exit when not Quantifier_Result;
               Index_160 := Index_160 + 1;
         end loop;
      end;
--# expr-done 4327
Let_Result_2 := Quantifier_Result; 
--# end
--# expr-done 4328
         if Let_Result_2 then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Item_157;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
               Index_158 := Index_158 + 1;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 4329
Let_Result_3 := Map_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 4330
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_585;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_585;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Get_Primitives;
--# end
--# property-start TypeDecl.get_imp_deref /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7932
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4332 '<FieldAccess for get_aspect_spec_expr at ast.py:7933>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7933
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4331 '<Symbol Implicit_Dereference at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Implicit_Dereference); 
--# expr-done 4331
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 4332
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start TypeDecl.has_ud_indexing /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7936
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4336 '<Not at ast.py:7937>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7937
--# expr-start 4335 '<IsNull at ast.py:7937>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7937
--# expr-start 4334 '<FieldAccess for get_aspect_spec_expr at ast.py:7937>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7937
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4333 '<Symbol Constant_Indexing at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant_Indexing); 
--# expr-done 4333
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 4334
Is_Null := Fld.Node = null; 
--# expr-done 4335
Not_Val := not (Is_Null); 
--# expr-done 4336
if Not_Val then
   If_Result := True;
else
   --# expr-start 4340 '<Not at ast.py:7938>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7938
--# expr-start 4339 '<IsNull at ast.py:7938>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7938
--# expr-start 4338 '<FieldAccess for get_aspect_spec_expr at ast.py:7938>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7938
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4337 '<Symbol Variable_Indexing at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Variable_Indexing); 
--# expr-done 4337
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 4338
Is_Null_1 := Fld_1.Node = null; 
--# expr-done 4339
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4340
   If_Result := Not_Val_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start TypeDecl.constant_indexing_fns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7941
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_588 with Inline_Always;
            procedure Finalizer_Scope_2464 with Inline_Always;
            procedure Finalizer_Scope_2466 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Entity_Defining_Name;
Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr_3 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_588 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_588;
            procedure Finalizer_Scope_2464 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2464;
            procedure Finalizer_Scope_2466 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2466;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4356 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4355 '<FilterMap at ast.py:7946>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7946
--# expr-start 4342 '<FieldAccess for get_aspect_spec_expr at ast.py:7944>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7944
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4341 '<Symbol Constant_Indexing at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant_Indexing); 
--# expr-done 4341
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 4342
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
--# expr-start 4344 '<FieldAccess for all_env_elements_internal at ast.py:7945>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7945
--# expr-start 4343 '<Cast to Name at ast.py:7945>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7945
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4343
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Cast_Result.Node, Seq => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 4344
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2464;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_161 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_161
--# expr-start 4346 '<FieldAccess for subp_spec_or_null at ast.py:7949>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7949
--# expr-start 4345 '<Cast to BasicDecl at ast.py:7949>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7949
Cast_Expr_2 := Item_161; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4345
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 4346
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_1
--# expr-start 4353 '<bind at ast.py:7951>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7951
--# expr-start 4347 '<FieldAccess for origin_node at ast.py:7952>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7952
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4347
Bound_Origin := Fld_3; 
--# expr-start 4352 '<FieldAccess for matching_formal_type at ast.py:7954>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7954
--# expr-start 4349 '<at at ast.py:7953>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7953
--# expr-start 4348 '<FieldAccess for unpacked_formal_params at ast.py:7953>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7953
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4348
Get_Result := Get (Self, Fld_4, 0, True); 
--# expr-done 4349
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 4351 '<FieldAccess for formal_type at ast.py:7954>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7954
--# expr-start 4350 '<FieldAccess for formal_decl at ast.py:7954>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7954
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_5 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4350
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_5.Node, Origin => Bound_Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 4351
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
      if Result_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_3 := Ent; 
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Result_Var_1.Node, Formal_Type => Cast_Result_3, Origin => Bound_Origin, E_Info => Result_Var_1.Info);
--# end
--# expr-done 4352
Dyn_Var_Bind_Result := Fld_7; 
--# expr-done 4353
Scope_Result_2 := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_2466;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
         if Result_Var_2 then
      --# expr-start 4354 '<Cast to BasicDecl at ast.py:7947>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7947
Cast_Expr_1 := Item_161; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4354
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result_1;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4355
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4356
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_588;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_588;
                     Finalizer_Scope_2464;
                     Finalizer_Scope_2466;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start TypeDecl.variable_indexing_fns /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7960
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_589 with Inline_Always;
            procedure Finalizer_Scope_2468 with Inline_Always;
            procedure Finalizer_Scope_2470 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Entity_Defining_Name;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Sym : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Expr_3 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_589 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_589;
            procedure Finalizer_Scope_2468 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2468;
            procedure Finalizer_Scope_2470 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2470;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4374 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4373 '<bind at ast.py:7962>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7962
--# expr-start 4357 '<FieldAccess for origin_node at ast.py:7963>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7963
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4357
Bound_Origin := Fld; 
--# expr-start 4359 '<FieldAccess for get_aspect_spec_expr at ast.py:7964>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7964
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4358 '<Symbol Variable_Indexing at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Variable_Indexing); 
--# expr-done 4358
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 4359
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 4372 '<FilterMap at ast.py:7966>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7966
   --# expr-start 4361 '<FieldAccess for all_env_elements_internal at ast.py:7966>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7966
--# expr-start 4360 '<Cast to Name at ast.py:7965>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7965
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4360
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Cast_Result.Node, Seq => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 4361
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_162 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_162
--# expr-start 4363 '<FieldAccess for subp_spec_or_null at ast.py:7969>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7969
--# expr-start 4362 '<Cast to BasicDecl at ast.py:7969>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7969
Cast_Expr_2 := Item_162; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4362
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 4363
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_1
--# expr-start 4368 '<FieldAccess for matching_formal_type at ast.py:7972>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7972
--# expr-start 4365 '<at at ast.py:7971>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7971
--# expr-start 4364 '<FieldAccess for unpacked_formal_params at ast.py:7971>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7971
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4364
Get_Result := Get (Self, Fld_4, 0, True); 
--# expr-done 4365
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 4367 '<FieldAccess for formal_type at ast.py:7972>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7972
--# expr-start 4366 '<FieldAccess for formal_decl at ast.py:7972>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7972
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_5 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4366
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_5.Node, Origin => Bound_Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 4367
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      if Result_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_3 := Ent; 
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Result_Var.Node, Formal_Type => Cast_Result_3, Origin => Bound_Origin, E_Info => Result_Var.Info);
--# end
--# expr-done 4368
if Fld_7 then
   --# expr-start 4370 '<FieldAccess for is_implicit_deref at ast.py:7973>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7973
--# expr-start 4369 '<FieldAccess for return_type at ast.py:7973>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7973
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4369
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Fld_8.Node, Origin => Bound_Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 4370
   If_Result := Fld_9;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
      Finalizer_Scope_2470;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
         if Result_Var_1 then
      --# expr-start 4371 '<Cast to BasicDecl at ast.py:7967>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7967
Cast_Expr_1 := Item_162; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4371
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result_1;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4372
Scope_Result_2 := Map_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2468;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Dyn_Var_Bind_Result := Result_Var_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4373
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4374
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_589;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_589;
                     Finalizer_Scope_2468;
                     Finalizer_Scope_2470;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start TypeDecl._env_mappings_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7701
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_3
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1529 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1529 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1529;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4377 '<New EnvAssoc at ast.py:7701>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7701
--# expr-start 4375 '<New DesignatedEnv at ast.py:7701>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7701
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 4375
--# expr-start 4376 '<FieldAccess for name_symbol at ast.py:7701>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7701
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4376
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4377
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1529;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1529;
            raise;
      end;
   return Property_Result;
end Env_Mappings_3;
--# end
--# property-start TypeDecl._env_mappings_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7702
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_4
  (Node : Bare_Type_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1530 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1530 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1530;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4378 '<FieldAccess for predefined_operators at ast.py:7702>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7702
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.predefined_operators
Fld := Libadalang.Implementation.Type_Decl_P_Predefined_Operators (Node => Self);
--# end
--# expr-done 4378
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1530;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1530;
            raise;
      end;
   return Property_Result;
end Env_Mappings_4;
--# end
--# property-start TypeDecl._env_trans_parent_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7700
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_5
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_5;
--# end
--# property-start TypeDecl._ref_env_nodes_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7706
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_6
  (Node : Bare_Type_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1533 with Inline_Always;
      Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1533 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1533;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4379 '<Cast to AdaNode at ast.py:7706>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7706
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 4379
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1533;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1533;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_6;
--# end
--# property-start TypeDecl._env_dest_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7709
pragma Warnings (Off, "is not referenced");
 function Env_Dest_7
  (Node : Bare_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1534 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1534 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1534;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4380 '<FieldAccess for node_env at ast.py:7709>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7709
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 4380
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1534;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1534;
            raise;
      end;
   return Property_Result;
end Env_Dest_7;
--# end
--# property-start TypeDecl._ref_cond_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7710
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_8
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Type_Def;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4382 '<IsA DerivedTypeDef, InterfaceTypeDef at ast.py:7710>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7710
--# expr-start 4381 '<FieldAccess for type_def at ast.py:7710>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7710
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 4381
Is_A := Fld /= null 
and then Fld.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 4382
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_8;
--# end
function Type_Decl_F_Direct_Primitive_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Type_Decl
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Internal_Inner_Env_Assoc_Array_Access :=
         Type_Decl_F_Direct_Primitive_Subps (E.Node);
   begin
         return Result;
   end;
end;
function Type_Decl_P_Refined_Parent_Primitives_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Type_Decl
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Type_Decl_P_Refined_Parent_Primitives_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Anonymous_Type_Decl
   --
      procedure Initialize_Fields_For_Anonymous_Type_Decl
        (Self : Bare_Anonymous_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
        ) is
      begin
            Initialize_Fields_For_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def);
      end Initialize_Fields_For_Anonymous_Type_Decl;
--# property-start AnonymousTypeDecl.access_def_matches /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8010
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Decl_P_Access_Def_Matches
  (Node : Bare_Anonymous_Type_Decl
      ; Other : Internal_Entity_Base_Type_Decl
      ; For_Assignment : Boolean
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Anonymous_Type_Decl  := Bare_Anonymous_Type_Decl (Node);
     Ent : Internal_Entity_Anonymous_Type_Decl :=
       Internal_Entity_Anonymous_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind for_assignment For_Assignment
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Access_To_Subp_Def;
Block_Var_1 : Internal_Entity_Access_To_Subp_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Access_Def;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Exp : Internal_Entity_Base_Type_Decl;
Act : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Access_To_Subp_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Boolean;
Is_Null_2 : Boolean;
Is_Null_3 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Internal_Entity_Type_Def;
Cast_Result_3 : Internal_Entity_Access_Def;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
Fld_11 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Let_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4407 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 4384 '<Cast to AccessToSubpDef at ast.py:8021>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8021
--# expr-start 4383 '<FieldAccess for type_def at ast.py:8021>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8021
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4383
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 4384
Block_Var := Cast_Result; 
--# bind self_subp_access Block_Var
--# expr-start 4386 '<Cast to AccessToSubpDef at ast.py:8022>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8022
--# expr-start 4385 '<FieldAccess for access_def at ast.py:8022>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8022
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 4385
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result_1 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 4386
Block_Var_1 := Cast_Result_1; 
--# bind other_subp_access Block_Var_1
--# expr-start 4406 '<Cond at ast.py:8026>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8026
--# expr-start 4388 '<Not at ast.py:8027>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8027
--# expr-start 4387 '<IsNull at ast.py:8027>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8027
Is_Null := Block_Var.Node = null; 
--# expr-done 4387
Not_Val := not (Is_Null); 
--# expr-done 4388
if Not_Val then
   --# expr-start 4390 '<Not at ast.py:8027>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8027
--# expr-start 4389 '<IsNull at ast.py:8027>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8027
Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 4389
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4390
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4393 '<FieldAccess for match_signature at ast.py:8028>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8028
--# expr-start 4391 '<FieldAccess for subp_spec at ast.py:8028>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8028
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Block_Var_1.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Block_Var_1.Info);
--# expr-done 4391
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4392 '<FieldAccess for subp_spec at ast.py:8029>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8029
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Block_Var.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Block_Var.Info);
--# expr-done 4392
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_2.Node, Other => Cast_Result_2, Match_Name => False, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4393
   If_Result_4 := Fld_4;
else
--# expr-start 4394 '<IsNull at ast.py:8032>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8032
Is_Null_2 := Block_Var.Node = null; 
--# expr-done 4394
if Is_Null_2 then
   --# expr-start 4395 '<IsNull at ast.py:8032>' Is_Null_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8032
Is_Null_3 := Block_Var_1.Node = null; 
--# expr-done 4395
   If_Result_1 := Is_Null_3;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
--# expr-start 4397 '<Cast to AccessDef at ast.py:8033>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8033
--# expr-start 4396 '<FieldAccess for type_def at ast.py:8033>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8033
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4396
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Access_Def
   then
      Cast_Result_3 := Create_Internal_Entity_Access_Def
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_3 := No_Entity_Access_Def;
   end if;
--# expr-done 4397
Var_Expr_1 := Cast_Result_3;
if Var_Expr_1 /= No_Entity_Access_Def then
   --# scope-start
--# expr-start 4398 '<FieldAccess for accessed_type at ast.py:8033>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8033
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4398
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind ast Var_Expr
--# expr-start 4399 '<FieldAccess for accessed_type at ast.py:8034>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8034
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 4399
Var_Expr_2 := Fld_7;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind oat Var_Expr_2
--# expr-start 4405 '<Let at ast.py:8035>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8035
--# scope-start
--# expr-start 4400 '<FieldAccess for specific_type at ast.py:8040>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8040
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4400
Exp := Fld_8; 
--# bind exp Exp
--# expr-start 4401 '<FieldAccess for specific_type at ast.py:8040>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8040
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4401
Act := Fld_9; 
--# bind act Act
--# expr-start 4404 '<If at ast.py:8041>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8041
if For_Assignment then
   --# expr-start 4402 '<FieldAccess for matching_assign_type at ast.py:8047>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8047
      if Act.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Act.Node, Expected_Type => Var_Expr, Origin => Origin, E_Info => Act.Info);
--# end
--# expr-done 4402
   If_Result_2 := Fld_10;
else
   --# expr-start 4403 '<FieldAccess for matching_type at ast.py:8049>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8049
      if Act.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Act.Node, Expected_Type => Exp, Origin => Origin, E_Info => Act.Info);
--# end
--# expr-done 4403
   If_Result_2 := Fld_11;
end if;
--# expr-done 4404
Let_Result := If_Result_2; 
--# end
--# expr-done 4405
Scope_Result_1 := Let_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_3 := Result_Var_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 4406
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 4407
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Type_Decl_P_Access_Def_Matches;
--# end
--# property-start AnonymousTypeDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8058
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Anonymous_Type_Decl
  )
   return Boolean
is
   Self : Bare_Anonymous_Type_Decl  := Bare_Anonymous_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Type_Decl_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Synth_Anonymous_Type_Decl
   --
      procedure Initialize_Fields_For_Synth_Anonymous_Type_Decl
        (Self : Bare_Synth_Anonymous_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
        ) is
      begin
            Initialize_Fields_For_Anonymous_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def);
      end Initialize_Fields_For_Synth_Anonymous_Type_Decl;
   --
   --  Primitives for Bare_Concrete_Type_Decl
   --
      procedure Initialize_Fields_For_Concrete_Type_Decl
        (Self : Bare_Concrete_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Concrete_Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def);
            Self.Concrete_Type_Decl_F_Aspects := Concrete_Type_Decl_F_Aspects;
      end Initialize_Fields_For_Concrete_Type_Decl;
   --
   --  Primitives for Bare_Formal_Type_Decl
   --
      procedure Initialize_Fields_For_Formal_Type_Decl
        (Self : Bare_Formal_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Formal_Type_Decl_F_Default_Type : Bare_Name
         ; Formal_Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def);
            Self.Formal_Type_Decl_F_Default_Type := Formal_Type_Decl_F_Default_Type;
            Self.Formal_Type_Decl_F_Aspects := Formal_Type_Decl_F_Aspects;
      end Initialize_Fields_For_Formal_Type_Decl;
   function Formal_Type_Decl_F_Default_Type
     (Node : Bare_Formal_Type_Decl) return Bare_Name
   is
   begin
         return Node.Formal_Type_Decl_F_Default_Type;
   end;
--# property-start FormalTypeDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7994
pragma Warnings (Off, "is not referenced");
 function Formal_Type_Decl_P_Xref_Equation
  (Node : Bare_Formal_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Type_Decl  := Bare_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Formal_Type_Decl :=
       Internal_Entity_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_590 with Inline_Always;
            procedure Finalizer_Scope_2475 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Fld : Logic_Equation;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_590 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_590;
            procedure Finalizer_Scope_2475 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2475;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4412 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4411 '<And at ast.py:7996>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7996
--# expr-start 4408 '<Super at ast.py:7996>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7996
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.xref_equation
Fld := Libadalang.Implementation.Type_Decl_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4408
--# expr-start 4409 '<FieldAccess for default_type at ast.py:7996>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7996
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Formal_Type_Decl_F_Default_Type, Info => Ent.Info);
--# expr-done 4409
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind dt Var_Expr
--# expr-start 4410 '<FieldAccess for sub_equation at ast.py:7997>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:7997
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4410
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2475;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:7998") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:7996") else null)); 
--# expr-done 4411
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 4412
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_590;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_590;
                     Finalizer_Scope_2475;
            raise;
      end;
   return Property_Result;
end Formal_Type_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Basic_Subp_Decl
   --
      procedure Basic_Subp_Decl_Pre_Env_Actions
        (Self            : Bare_Basic_Subp_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_25 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_26 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:9873");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_27 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:9880");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_28 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_29 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_30 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_32 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_31 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_34 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_33 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Basic_Subp_Decl_Post_Env_Actions
        (Self : Bare_Basic_Subp_Decl; State : in out PLE_Node_State) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_35 (Node => 
Self);
      end;
      end;
--# property-start BasicSubpDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9764
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Defining_Names
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_779 with Inline_Always;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_779 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_779;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4414 '<FieldAccess for name at ast.py:9764>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9764
--# expr-start 4413 '<FieldAccess for subp_decl_spec at ast.py:9764>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9764
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4413
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4414
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_1;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_779;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_779;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Defining_Names;
--# end
--# property-start BasicSubpDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9766
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Defining_Env
  (Node : Bare_Basic_Subp_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_780 with Inline_Always;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_780 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_780;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4416 '<FieldAccess for defining_env at ast.py:9766>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9766
--# expr-start 4415 '<FieldAccess for subp_decl_spec at ast.py:9766>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9766
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4415
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4416
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_780;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_780;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Defining_Env;
--# end
--# property-start BasicSubpDecl.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9768
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Type_Expression
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4418 '<FieldAccess for returns at ast.py:9769>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9769
--# expr-start 4417 '<FieldAccess for subp_decl_spec at ast.py:9769>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9769
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4417
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4418
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Type_Expression;
--# end
--# property-start BasicSubpDecl.has_spark_mode_on /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9775
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Has_Spark_Mode_On
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4420 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4419 '<FieldAccess for is_spark_impl at ast.py:9784>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9784
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_spark_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Spark_Impl (Node => Ent.Node, Include_Skip_Proof_Annotations => False, E_Info => Ent.Info);
--# end
--# expr-done 4419
Let_Result := Fld; 
--# end
--# expr-done 4420
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Has_Spark_Mode_On;
--# end
--# property-start BasicSubpDecl.get_body_in_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9786
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Get_Body_In_Env
  (Node : Bare_Basic_Subp_Decl
      ; Env : Lexical_Env
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
            procedure Finalizer_Scope_783 with Inline_Always;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Internal_Entity_Base_Subp_Spec;
Block_Var_2 : Internal_Entity_Body_Node;
Block_Var_3 : Internal_Entity_Body_Node;
Var_Expr : Internal_Entity_Body_Node;
Var_Expr_1 : Internal_Entity_Body_Node;
Fld : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Bare_Base_Subp_Spec;
As_Entity : Internal_Entity_Base_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Spec;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld_5 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Expr : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Body_Node;
Dyn_Var_Bind_Result : Internal_Entity_Body_Node;
Is_A : Boolean;
Find_Result_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Body_Node;
Is_Null_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Body_Node;
Fld_7 : Bare_Body_Node;
As_Entity_2 : Internal_Entity_Body_Node;
Is_Null_3 : Boolean;
If_Result_4 : Internal_Entity_Body_Node;
Fld_8 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Body_Node;
            procedure Finalizer_Scope_783 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_783;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4444 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4422 '<EnvGet at ast.py:9789>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9789
--# expr-start 4421 '<FieldAccess for name_symbol at ast.py:9789>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9789
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4421
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Fld, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 4422
Block_Var := Env_Get_Result; Inc_Ref (Block_Var);
--# bind elements Block_Var
--# expr-start 4425 '<as_bare_entity at ast.py:9791>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9791
--# expr-start 4424 '<FieldAccess for node at ast.py:9791>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9791
--# expr-start 4423 '<FieldAccess for subp_decl_spec at ast.py:9791>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9791
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4423
Fld_2 := Fld_1.Node;
--# expr-done 4424
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Spec;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 4425
Block_Var_1 := If_Result; 
--# bind self_spec Block_Var_1
--# expr-start 4434 '<bind at ast.py:9792>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9792
--# expr-start 4426 '<FieldAccess for origin_node at ast.py:9792>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9792
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Block_Var_1.Node);
--# end
--# expr-done 4426
Bound_Origin := Fld_3; 
--# expr-start 4433 '<Cast to Body at ast.py:9798>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9798
--# expr-start 4432 '<Find at ast.py:9792>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9792
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_163 of
               Collection.Items
         loop
   --# scope-start
               --# bind ent Item_163
--# expr-start 4429 '<Cast to Body at ast.py:9796>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9796
--# expr-start 4428 '<as_bare_entity at ast.py:9796>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9796
--# expr-start 4427 '<FieldAccess for node at ast.py:9796>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9796
Fld_4 := Item_163.Node;
--# expr-done 4427
Is_Null_1 := Fld_4 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_4); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 4428
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 4429
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 4431 '<FieldAccess for match_other at ast.py:9797>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9797
--# expr-start 4430 '<FieldAccess for formal_param_holder_or_null at ast.py:9797>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9797
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4430
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start '[dispatcher]BaseFormalParamHolder.match_other'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Match_Other (Node => Fld_5.Node, Other => Cast_Result_1, Match_Names => True, Origin => Bound_Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 4431
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_163; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 4432
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 4433
Dyn_Var_Bind_Result := Cast_Result_2; 
--# expr-done 4434
Block_Var_2 := Dyn_Var_Bind_Result; 
--# bind precise Block_Var_2
--# expr-start 4439 '<If at ast.py:9800>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9800
--# expr-start 4435 '<IsNull at ast.py:9801>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9801
Is_Null_2 := Block_Var_2.Node = null; 
--# expr-done 4435
if Is_Null_2 then
   If_Result_2 := Imprecise_Fallback;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 4438 '<Cast to Body at ast.py:9802>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9802
--# expr-start 4437 '<Find at ast.py:9802>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9802
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_164 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_164
      --# expr-start 4436 '<IsA Body at ast.py:9802>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9802
Is_A := Item_164.Node /= null 
and then Item_164.Node.Kind in Ada_Body_Node; 
--# expr-done 4436
      if Is_A then
         Find_Result_1 := Item_164; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 4437
   if Find_Result_1.Node = null
      or else Find_Result_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_3 := Create_Internal_Entity_Body_Node
        (Node => Find_Result_1.Node,
         Info => Find_Result_1.Info);
   else
         Cast_Result_3 := No_Entity_Body_Node;
   end if;
--# expr-done 4438
   If_Result_3 := Cast_Result_3;
else
   If_Result_3 := Block_Var_2;
end if;
--# expr-done 4439
Block_Var_3 := If_Result_3; 
--# bind result Block_Var_3
--# expr-start 4443 '<Cast to Body at ast.py:9807>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9807
--# expr-start 4441 '<as_entity at ast.py:9807>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9807
--# expr-start 4440 '<FieldAccess for node at ast.py:9807>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9807
Fld_7 := Block_Var_3.Node;
--# expr-done 4440
Is_Null_3 := Fld_7 = null; 
if Is_Null_3 then
   If_Result_4 := No_Entity_Body_Node;
else
As_Entity_2 := (Info => E_Info, Node => Fld_7); 
   If_Result_4 := As_Entity_2;
end if;
--# expr-done 4441
Var_Expr_1 := If_Result_4;
if Var_Expr_1 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 4442 '<FieldAccess for without_md at ast.py:9807>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9807
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.without_md
Fld_8 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4442
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_4 := Create_Internal_Entity_Body_Node
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_4 := No_Entity_Body_Node;
   end if;
--# expr-done 4443
Let_Result := Cast_Result_4; 
--# end
--# expr-done 4444
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_783;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_783;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Get_Body_In_Env;
--# end
--# property-start BasicSubpDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9809
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_784 with Inline_Always;
            procedure Finalizer_Scope_2481 with Inline_Always;
            procedure Finalizer_Scope_2482 with Inline_Always;
            procedure Finalizer_Scope_2484 with Inline_Always;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Internal_Entity_Basic_Decl;
Block_Var_2 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Declarative_Part;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Body_Node;
Var_Expr_3 : Internal_Entity_Private_Part;
Var_Expr_4 : Internal_Entity_Body_Node;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Is_A : Boolean;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Internal_Entity_Body_Node;
Scope_Result_1 : Internal_Entity_Body_Node;
Result_Var_1 : Internal_Entity_Body_Node;
Is_A_1 : Boolean;
Fld_6 : Internal_Entity_Private_Part;
Fld_7 : Lexical_Env := Empty_Env;
Fld_8 : Internal_Entity_Body_Node;
Scope_Result_2 : Internal_Entity_Body_Node;
Result_Var_2 : Internal_Entity_Body_Node;
Scope_Result_3 : Internal_Entity_Body_Node;
Fld_9 : Internal_Entity_Body_Node;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Internal_Entity_Body_Node;
Scope_Result_4 : Internal_Entity_Body_Node;
Result_Var_3 : Internal_Entity_Body_Node;
Result_Var_4 : Internal_Entity_Body_Node;
Is_Null_2 : Boolean;
Fld_12 : Lexical_Env := Empty_Env;
Fld_13 : Internal_Entity_Body_Node;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Body_Node;
Cast_Result_2 : Internal_Entity_Basic_Decl;
If_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
If_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
If_Result_3 : Internal_Entity_Basic_Decl;
If_Result_4 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_784 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_784;
            procedure Finalizer_Scope_2481 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2481;
            procedure Finalizer_Scope_2482 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2482;
            procedure Finalizer_Scope_2484 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2484;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4467 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4445 '<FieldAccess for declarative_scope at ast.py:9811>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9811
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 4445
Block_Var := Fld; 
--# bind decl_scope Block_Var
--# expr-start 4446 '<as_entity at ast.py:9812>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9812
Is_Null := Block_Var = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Block_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 4446
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind ds Var_Expr
--# expr-start 4448 '<Cast to BasicDecl at ast.py:9813>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9813
--# expr-start 4447 '<FieldAccess for semantic_parent at ast.py:9813>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9813
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4447
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4448
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind parent_decl Block_Var_1
--# expr-start 4449 '<FieldAccess for basic_decl_next_part_for_decl at ast.py:9816>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9816
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4449
Block_Var_2 := Fld_2; 
--# bind default_next_part Block_Var_2
--# expr-start 4466 '<Cond at ast.py:9818>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9818
--# expr-start 4451 '<Not at ast.py:9821>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9821
--# expr-start 4450 '<IsNull at ast.py:9821>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9821
Is_Null_1 := Block_Var_2.Node = null; 
--# expr-done 4450
Not_Val := not (Is_Null_1); 
--# expr-done 4451
if Not_Val then
   If_Result_4 := Block_Var_2;
else
--# expr-start 4452 '<IsA PrivatePart at ast.py:9825>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9825
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Private_Part_Range; 
--# expr-done 4452
if Is_A then
--# expr-start 4453 '<FieldAccess for next_part_for_decl at ast.py:9826>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9826
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4453
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr_1
--# expr-start 4455 '<FieldAccess for get_body_in_env at ast.py:9827>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9827
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4454 '<FieldAccess for children_env at ast.py:9827>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9827
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 4454
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_5 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_4, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4455
Scope_Result_1 := Fld_5;
   --# end
      Finalizer_Scope_2481;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Body_Node;
end if;
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   If_Result_3 := Cast_Result_4;
else
--# expr-start 4456 '<IsA PublicPart at ast.py:9831>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9831
Is_A_1 := Block_Var /= null 
and then Block_Var.Kind in Ada_Public_Part_Range; 
--# expr-done 4456
if Is_A_1 then
--# expr-start 4457 '<FieldAccess for decl_private_part at ast.py:9834>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9834
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.decl_private_part
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Decl_Private_Part (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4457
Var_Expr_3 := Fld_6;
if Var_Expr_3 /= No_Entity_Private_Part then
   --# scope-start
   --# bind dpp Var_Expr_3
--# expr-start 4459 '<FieldAccess for get_body_in_env at ast.py:9835>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9835
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4458 '<FieldAccess for children_env at ast.py:9835>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9835
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# expr-done 4458
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_8 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_7, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4459
Scope_Result_2 := Fld_8;
   --# end
      Finalizer_Scope_2482;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Body_Node;
end if;
Var_Expr_2 := Result_Var_2;
if Var_Expr_2 /= No_Entity_Body_Node then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_3 := Var_Expr_2;
   --# end
   Result_Var_4 := Scope_Result_3;
else
--# expr-start 4460 '<FieldAccess for body_part_for_decl at ast.py:9838>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9838
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4460
Var_Expr_4 := Fld_9;
if Var_Expr_4 /= No_Entity_Body_Node then
   --# scope-start
   --# bind np Var_Expr_4
--# expr-start 4462 '<FieldAccess for get_body_in_env at ast.py:9839>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9839
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4461 '<FieldAccess for children_env at ast.py:9839>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9839
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# expr-done 4461
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_11 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_10, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4462
Scope_Result_4 := Fld_11;
   --# end
      Finalizer_Scope_2484;
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Entity_Body_Node;
end if;
   Result_Var_4 := Result_Var_3;
end if;
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var_4.Node,
         Info => Result_Var_4.Info);
   If_Result_2 := Cast_Result_3;
else
--# expr-start 4463 '<IsNull at ast.py:9843>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9843
Is_Null_2 := Block_Var = null; 
--# expr-done 4463
if Is_Null_2 then
Cast_Expr := No_Entity_Body_Node; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_1 := Cast_Result_2;
else
--# expr-start 4465 '<FieldAccess for get_body_in_env at ast.py:9847>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9847
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4464 '<FieldAccess for children_env at ast.py:9847>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9847
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Children_Env (Node => Block_Var);
--# expr-done 4464
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_13 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_12, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4465
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   If_Result_1 := Cast_Result_1;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 4466
Let_Result := If_Result_4; 
--# end
--# expr-done 4467
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_784;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_784;
                     Finalizer_Scope_2481;
                     Finalizer_Scope_2482;
                     Finalizer_Scope_2484;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicSubpDecl.constrain_prefix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9850
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Subp_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_785 with Inline_Always;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_785 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_785;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4469 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4468 '<FieldAccess for subp_constrain_prefix at ast.py:9852>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9852
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_constrain_prefix
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Constrain_Prefix (Node => Ent.Node, Prefix => Prefix, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4468
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4469
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_785;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_785;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicSubpDecl.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9854
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Expr_Type
  (Node : Bare_Basic_Subp_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4472 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4470 '<FieldAccess for subp_spec_or_null at ast.py:9856>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9856
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4470
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 4471 '<FieldAccess for return_type at ast.py:9856>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9856
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4471
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4472
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Expr_Type;
--# end
--# property-start BasicSubpDecl.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9858
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Env_Names
  (Node : Bare_Basic_Subp_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_787 with Inline_Always;
            procedure Finalizer_Scope_2486 with Inline_Always;
      Var_Expr : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_787 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_787;
            procedure Finalizer_Scope_2486 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2486;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4475 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4473 '<FieldAccess for top_level_env_name at ast.py:9860>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9860
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4473
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 4474 '<to_symbol at ast.py:9861>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9861
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr); 
--# expr-done 4474
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2486;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 4475
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_787;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_787;
                     Finalizer_Scope_2486;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Env_Names;
--# end
--# property-start '[dispatcher]BasicSubpDecl.subp_decl_spec' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Subp_Decl (Self.Kind) is
               when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl =>
                  --# property-call-start ClassicSubpDecl.subp_decl_spec
                  Property_Result := Classic_Subp_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.subp_decl_spec
                  Property_Result := Entry_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start EnumLiteralDecl.subp_decl_spec
                  Property_Result := Enum_Literal_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Internal =>
                  --# property-call-start GenericSubpInternal.subp_decl_spec
                  Property_Result := Generic_Subp_Internal_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Subp_Decl =>
                  --# property-call-start SyntheticSubpDecl.subp_decl_spec
                  Property_Result := Synthetic_Subp_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start BasicSubpDecl._env_do_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9871
pragma Warnings (Off, "is not referenced");
 function Env_Do_25
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4476 '<FieldAccess for env_hook at ast.py:9871>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9871
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4476
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_25;
--# end
--# property-start BasicSubpDecl._initial_env_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9877
pragma Warnings (Off, "is not referenced");
 function Initial_Env_26
  (Node : Bare_Basic_Subp_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1553 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1553 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1553;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4477 '<FieldAccess for child_decl_initial_env at ast.py:9877>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9877
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self, Private_Part => True);
--# end
--# expr-done 4477
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1553;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1553;
            raise;
      end;
   return Property_Result;
end Initial_Env_26;
--# end
--# property-start BasicSubpDecl._env_mappings_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9880
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_27
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1554 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1554 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1554;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4478 '<FieldAccess for child_decl_env_assocs at ast.py:9880>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9880
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_env_assocs
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Env_Assocs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4478
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1554;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1554;
            raise;
      end;
   return Property_Result;
end Env_Mappings_27;
--# end
--# property-start BasicSubpDecl._env_trans_parent_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9869
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_28
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_28;
--# end
--# property-start BasicSubpDecl._env_names_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9882
pragma Warnings (Off, "is not referenced");
 function Env_Names_29
  (Node : Bare_Basic_Subp_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1556 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1556 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1556;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4479 '<FieldAccess for env_names at ast.py:9882>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9882
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicSubpDecl.env_names
Fld := Libadalang.Implementation.Basic_Subp_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 4479
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1556;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1556;
            raise;
      end;
   return Property_Result;
end Env_Names_29;
--# end
--# property-start BasicSubpDecl._env_do_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9884
pragma Warnings (Off, "is not referenced");
 function Env_Do_30
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1557 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1557 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1557;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4480 '<FieldAccess for populate_dependent_units at ast.py:9884>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9884
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4480
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1557;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1557;
            raise;
      end;
   return Property_Result;
end Env_Do_30;
--# end
--# property-start BasicSubpDecl._ref_env_nodes_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9887
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_31
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1558 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1558 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1558;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4481 '<FieldAccess for top_level_use_package_clauses at ast.py:9887>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9887
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4481
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1558;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1558;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_31;
--# end
--# property-start BasicSubpDecl._ref_cond_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9889
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_32
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4483 '<IsA LibraryItem, Subunit at ast.py:9889>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9889
--# expr-start 4482 '<parent at ast.py:9889>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9889
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 4482
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4483
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_32;
--# end
--# property-start BasicSubpDecl._ref_env_nodes_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9893
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_33
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1560 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1560 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1560;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4484 '<FieldAccess for top_level_use_type_clauses at ast.py:9893>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9893
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4484
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1560;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1560;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_33;
--# end
--# property-start BasicSubpDecl._ref_cond_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9895
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_34
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4486 '<IsA LibraryItem, Subunit at ast.py:9895>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9895
--# expr-start 4485 '<parent at ast.py:9895>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9895
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 4485
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4486
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_34;
--# end
--# property-start BasicSubpDecl._env_do_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9900
pragma Warnings (Off, "is not referenced");
 function Env_Do_35
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4487 '<FieldAccess for populate_body_unit at ast.py:9900>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9900
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_body_unit
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Body_Unit (Node => Self);
--# end
--# expr-done 4487
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_35;
--# end
   --
   --  Primitives for Bare_Classic_Subp_Decl
   --
      procedure Initialize_Fields_For_Classic_Subp_Decl
        (Self : Bare_Classic_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Self.Classic_Subp_Decl_F_Overriding := Classic_Subp_Decl_F_Overriding;
            Self.Classic_Subp_Decl_F_Subp_Spec := Classic_Subp_Decl_F_Subp_Spec;
      end Initialize_Fields_For_Classic_Subp_Decl;
   function Classic_Subp_Decl_F_Overriding
     (Node : Bare_Classic_Subp_Decl) return Bare_Overriding_Node
   is
   begin
         return Node.Classic_Subp_Decl_F_Overriding;
   end;
   function Classic_Subp_Decl_F_Subp_Spec
     (Node : Bare_Classic_Subp_Decl) return Bare_Subp_Spec
   is
   begin
         return Node.Classic_Subp_Decl_F_Subp_Spec;
   end;
--# property-start ClassicSubpDecl.subp_decl_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9914
pragma Warnings (Off, "is not referenced");
 function Classic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Classic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Classic_Subp_Decl  := Bare_Classic_Subp_Decl (Node);
     Ent : Internal_Entity_Classic_Subp_Decl :=
       Internal_Entity_Classic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4488 '<FieldAccess for subp_spec at ast.py:9914>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9914
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 4488
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start ClassicSubpDecl.body_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9916
pragma Warnings (Off, "is not referenced");
 function Classic_Subp_Decl_P_Body_Part
  (Node : Bare_Classic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Body
is
   Self : Bare_Classic_Subp_Decl  := Bare_Classic_Subp_Decl (Node);
     Ent : Internal_Entity_Classic_Subp_Decl :=
       Internal_Entity_Classic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Base_Subp_Body;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4491 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4490 '<Cast to BaseSubpBody at ast.py:9922>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9922
--# expr-start 4489 '<FieldAccess for body_part_for_decl at ast.py:9922>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9922
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4489
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4490
Let_Result := Cast_Result; 
--# end
--# expr-done 4491
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classic_Subp_Decl_P_Body_Part;
--# end
   --
   --  Primitives for Bare_Abstract_Subp_Decl
   --
      procedure Initialize_Fields_For_Abstract_Subp_Decl
        (Self : Bare_Abstract_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Abstract_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Abstract_Subp_Decl_F_Aspects := Abstract_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Abstract_Subp_Decl;
   --
   --  Primitives for Bare_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Formal_Subp_Decl
        (Self : Bare_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Formal_Subp_Decl_F_Default_Expr := Formal_Subp_Decl_F_Default_Expr;
            Self.Formal_Subp_Decl_F_Aspects := Formal_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Formal_Subp_Decl;
   function Formal_Subp_Decl_F_Default_Expr
     (Node : Bare_Formal_Subp_Decl) return Bare_Expr
   is
   begin
         return Node.Formal_Subp_Decl_F_Default_Expr;
   end;
--# property-start FormalSubpDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11854
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Defining_Names
  (Node : Bare_Formal_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_912 with Inline_Always;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_912 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_912;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4493 '<FieldAccess for name at ast.py:11854>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11854
--# expr-start 4492 '<FieldAccess for subp_spec at ast.py:11854>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11854
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 4492
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4493
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_1;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_912;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_912;
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Defining_Names;
--# end
--# property-start FormalSubpDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11856
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Xref_Equation
  (Node : Bare_Formal_Subp_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_913 with Inline_Always;
            procedure Finalizer_Scope_2487 with Inline_Always;
            procedure Finalizer_Scope_2488 with Inline_Always;
            procedure Finalizer_Scope_2489 with Inline_Always;
            procedure Finalizer_Scope_2490 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Match_0 : Internal_Entity_Null_Literal;
Match_1 : Internal_Entity_Box_Expr;
Match_2 : Internal_Entity_Name;
Match_3 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
True_Rel : Logic_Equation;
True_Rel_1 : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_3 : Logic_Equation;
True_Rel_2 : Logic_Equation;
Is_A : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Null_Literal;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Box_Expr;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Expr;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Scope_Result_4 : Logic_Equation;
True_Rel_3 : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_913 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (True_Rel_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_913;
            procedure Finalizer_Scope_2487 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2487;
            procedure Finalizer_Scope_2488 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_2488;
            procedure Finalizer_Scope_2489 is
            begin
                     Dec_Ref (True_Rel_1);
            end Finalizer_Scope_2489;
            procedure Finalizer_Scope_2490 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_3);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2490;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4494 '<FieldAccess for default_expr at ast.py:11857>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11857
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Formal_Subp_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 4494
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 4504 '<Match at ast.py:11857>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11857
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Null_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Null_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind _ Match_0
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11858") else null)); 
Let_Result := True_Rel; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2488;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Box_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Box_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11860") else null)); 
Let_Result_1 := True_Rel_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2489;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_String_Literal | Ada_Int_Literal .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind n Match_2
--# expr-start 4503 '<And at ast.py:11862>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11862
--# expr-start 4499 '<And at ast.py:11863>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11863
--# expr-start 4495 '<FieldAccess for xref_no_overloading at ast.py:11863>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11863
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Match_2.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 4495
--# expr-start 4498 '<Predicate on BasicDecl.subp_decl_match_signature at ast.py:11864>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11864
--# expr-start 4496 '<FieldAccess for ref_var at ast.py:11865>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11865
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 4496
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 4497 '<Cast to BasicDecl at ast.py:11865>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11865
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 4497
Pred := Solver.Create_Predicate (Fld_2, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result, Origin)); 
--# expr-done 4498
And_Pred := Create_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11863") else null)); 
--# expr-done 4499
--# expr-start 4502 '<If at ast.py:11867>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11867
--# expr-start 4500 '<IsA AttributeRef at ast.py:11867>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11867
Is_A := Match_2.Node /= null 
and then Match_2.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 4500
if Is_A then
   --# expr-start 4501 '<FieldAccess for sub_equation at ast.py:11867>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11867
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 4501
   If_Result := Fld_3;
else
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11867") else null)); 
   If_Result := True_Rel_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 4502
And_Pred_1 := Create_And (And_Pred, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11862") else null)); 
--# expr-done 4503
Let_Result_2 := And_Pred_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2490;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl_Expr .. Ada_Relation_Op | Ada_Case_Expr_Alternative .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 4504
Scope_Result_4 := Match_Result;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2487;
   Result_Var := Scope_Result_4;
else
   True_Rel_3 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11871") else null)); 
   Result_Var := True_Rel_3;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_913;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_913;
                     Finalizer_Scope_2487;
                     Finalizer_Scope_2488;
                     Finalizer_Scope_2489;
                     Finalizer_Scope_2490;
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Xref_Equation;
--# end
--# property-start FormalSubpDecl.designated_subprogram_from /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11874
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Designated_Subprogram_From
  (Node : Bare_Formal_Subp_Decl
      ; Inst : Internal_Entity_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind inst Inst
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_914 with Inline_Always;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Internal_Entity_Subp_Spec;
Block_Var_2 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Symbol_Type;
Fld_3 : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Fld_4 : Internal_Entity_Array_Access;
Fld_5 : Bare_Subp_Spec;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Fld_10 : Bare_Ada_Node;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Rebinding : Env_Rebindings;
Fld_13 : Internal_Entity_Info;
Fld_14 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Subp_Spec;
Fld_15 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_16 : Internal_Entity_Base_Subp_Spec;
Fld_17 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_18 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_914 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_914;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4531 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4509 '<FieldAccess for env_get at ast.py:11881>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11881
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4505 '<FieldAccess for node_env at ast.py:11882>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11882
      if Inst.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Inst.Node, E_Info => Inst.Info);
--# expr-done 4505
--# expr-start 4507 '<FieldAccess for name_symbol at ast.py:11883>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11883
--# expr-start 4506 '<FieldAccess for defining_name at ast.py:11883>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11883
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4506
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 4507
--# expr-start 4508 '<FieldAccess for node at ast.py:11884>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11884
Fld_3 := Inst.Node;
--# expr-done 4508
      Cast_Result := Fld_3;
--# property-call-start AdaNode.env_get
Fld_4 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld, Symbol => Fld_2, From_Node => Cast_Result);
--# end
--# expr-done 4509
Block_Var := Fld_4; Inc_Ref (Block_Var);
--# bind subps Block_Var
--# expr-start 4522 '<New SubpSpec.entity at ???>' New_Struct_1 None
--# expr-start 4520 '<New EntityInfo at ast.py:11902>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11902
--# expr-start 4511 '<FieldAccess for from_rebound at ast.py:11909>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11909
--# expr-start 4510 '<FieldAccess for info at ast.py:11909>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11909
Fld_13 := Ent.Info;
--# expr-done 4510
Fld_14 := Fld_13.From_Rebound;
--# expr-done 4511
--# expr-start 4513 '<FieldAccess for md at ast.py:11903>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11903
--# expr-start 4512 '<FieldAccess for info at ast.py:11903>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11903
Fld_6 := Ent.Info;
--# expr-done 4512
Fld_7 := Fld_6.Md;
--# expr-done 4513
--# expr-start 4519 '<append_rebinding at ast.py:11906>' Rebinding /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11906
--# expr-start 4515 '<FieldAccess for rebindings at ast.py:11904>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11904
--# expr-start 4514 '<FieldAccess for info at ast.py:11904>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11904
Fld_8 := Ent.Info;
--# expr-done 4514
Fld_9 := Fld_8.Rebindings;
--# expr-done 4515
--# expr-start 4517 '<FieldAccess for node_env at ast.py:11907>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11907
--# expr-start 4516 '<parent at ast.py:11907>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11907
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Parent; 
--# expr-done 4516
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Node_Env (Node => Fld_10);
--# expr-done 4517
--# expr-start 4518 '<FieldAccess for instantiation_env at ast.py:11907>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11907
      if Inst.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_12 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Inst.Node);
--# end
--# expr-done 4518
Rebinding := AST_Envs.Append_Rebinding (Fld_9, Fld_11, Fld_12); 
--# expr-done 4519
New_Struct := (From_Rebound => Fld_14, Md => Fld_7, Rebindings => Rebinding); 
--# expr-done 4520
--# expr-start 4521 '<FieldAccess for subp_spec at ast.py:11901>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11901
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Classic_Subp_Decl_F_Subp_Spec;
--# expr-done 4521
New_Struct_1 := (Info => New_Struct, Node => Fld_5); 
--# expr-done 4522
Block_Var_1 := New_Struct_1; 
--# bind actual_spec Block_Var_1
--# expr-start 4529 '<bind at ast.py:11914>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11914
--# expr-start 4523 '<FieldAccess for origin_node at ast.py:11914>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11914
      if Inst.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_15 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Inst.Node);
--# end
--# expr-done 4523
Bound_Origin := Fld_15; 
--# expr-start 4528 '<Cast to BasicDecl at ast.py:11923>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11923
--# expr-start 4527 '<Find at ast.py:11914>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11914
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_165 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_165
--# expr-start 4525 '<FieldAccess for subp_spec_or_null at ast.py:11915>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11915
--# expr-start 4524 '<Cast to BasicDecl at ast.py:11915>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11915
Cast_Expr := Item_165; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4524
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4525
Var_Expr := Fld_16;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 4526 '<FieldAccess for match_signature at ast.py:11916>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11916
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_17 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Block_Var_1.Node, Other => Var_Expr, Match_Name => False, Origin => Bound_Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4526
Scope_Result := Fld_17;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_165; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 4527
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4528
Dyn_Var_Bind_Result := Cast_Result_2; 
--# expr-done 4529
Block_Var_2 := Dyn_Var_Bind_Result; 
--# bind found Block_Var_2
Var_Expr_1 := Block_Var_2;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4530 '<FieldAccess for wrap_public_reference at ast.py:11927>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11927
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.wrap_public_reference
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Wrap_Public_Reference (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4530
Scope_Result_1 := Fld_18;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 4531
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_914;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_914;
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Designated_Subprogram_From;
--# end
   --
   --  Primitives for Bare_Abstract_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Abstract_Formal_Subp_Decl
        (Self : Bare_Abstract_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Formal_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec, Formal_Subp_Decl_F_Default_Expr, Formal_Subp_Decl_F_Aspects);
      end Initialize_Fields_For_Abstract_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Concrete_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Concrete_Formal_Subp_Decl
        (Self : Bare_Concrete_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Formal_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec, Formal_Subp_Decl_F_Default_Expr, Formal_Subp_Decl_F_Aspects);
      end Initialize_Fields_For_Concrete_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Subp_Decl
   --
      procedure Initialize_Fields_For_Subp_Decl
        (Self : Bare_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Subp_Decl_F_Aspects := Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Subp_Decl;
--# property-start SubpDecl.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9931
pragma Warnings (Off, "is not referenced");
 function Subp_Decl_P_Is_Constant_Object
  (Node : Bare_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subp_Decl  := Bare_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Decl_P_Is_Constant_Object;
--# end
   --
   --  Primitives for Bare_Entry_Decl
   --
      procedure Entry_Decl_Pre_Env_Actions
        (Self            : Bare_Entry_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_124 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:18697");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_125 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Decl
        (Self : Bare_Entry_Decl
         ; Entry_Decl_F_Overriding : Bare_Overriding_Node
         ; Entry_Decl_F_Spec : Bare_Entry_Spec
         ; Entry_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Entry_Decl_F_Overriding := Entry_Decl_F_Overriding;
            Self.Entry_Decl_F_Spec := Entry_Decl_F_Spec;
            Self.Entry_Decl_F_Aspects := Entry_Decl_F_Aspects;
      end Initialize_Fields_For_Entry_Decl;
   function Entry_Decl_F_Overriding
     (Node : Bare_Entry_Decl) return Bare_Overriding_Node
   is
   begin
         return Node.Entry_Decl_F_Overriding;
   end;
   function Entry_Decl_F_Spec
     (Node : Bare_Entry_Decl) return Bare_Entry_Spec
   is
   begin
         return Node.Entry_Decl_F_Spec;
   end;
--# property-start EntryDecl.subp_decl_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18615
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Subp_Decl_Spec
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Entry_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4532 '<FieldAccess for spec at ast.py:18615>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18615
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 4532
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Subp_Decl_Spec;
--# end
--# property-start EntryDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18617
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Defining_Names
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1300 with Inline_Always;
      Fld : Internal_Entity_Entry_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1300 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1300;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4534 '<FieldAccess for name at ast.py:18617>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18617
--# expr-start 4533 '<FieldAccess for spec at ast.py:18617>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18617
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 4533
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4534
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_1;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1300;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1300;
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Defining_Names;
--# end
--# property-start EntryDecl.body_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18619
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Body_Part
  (Node : Bare_Entry_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
      Fld : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Body_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4536 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4535 '<FieldAccess for body_part_for_decl at ast.py:18625>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18625
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4535
Let_Result := Fld; 
--# end
--# expr-done 4536
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Body_Part;
--# end
--# property-start EntryDecl.has_family /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18627
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Has_Family
  (Node : Bare_Entry_Decl
  )
   return Boolean
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Entry_Spec;
Fld_1 : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4541 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4540 '<Not at ast.py:18632>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18632
--# expr-start 4539 '<IsNull at ast.py:18632>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18632
--# expr-start 4538 '<FieldAccess for family_type at ast.py:18632>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18632
--# expr-start 4537 '<FieldAccess for spec at ast.py:18632>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18632
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Entry_Decl_F_Spec;
--# expr-done 4537
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Entry_Spec_F_Family_Type;
--# expr-done 4538
Is_Null := Fld_1 = null; 
--# expr-done 4539
Not_Val := not (Is_Null); 
--# expr-done 4540
Let_Result := Not_Val; 
--# end
--# expr-done 4541
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Has_Family;
--# end
--# property-start EntryDecl.family_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18634
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Family_Type
  (Node : Bare_Entry_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Entry_Spec;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4546 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4544 '<Cast to TypeExpr at ast.py:18641>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18641
--# expr-start 4543 '<FieldAccess for family_type at ast.py:18641>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18641
--# expr-start 4542 '<FieldAccess for spec at ast.py:18641>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18641
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 4542
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity (Node => Fld.Node.Entry_Spec_F_Family_Type, Info => Fld.Info);
--# expr-done 4543
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Type_Expr
   then
      Cast_Result := Create_Internal_Entity_Type_Expr
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Type_Expr;
   end if;
--# expr-done 4544
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 4545 '<FieldAccess for designated_type at ast.py:18641>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18641
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4545
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4546
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Family_Type;
--# end
--# property-start EntryDecl.accept_stmts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18643
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Accept_Stmts
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Accept_Stmt_Array_Access
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Accept_Stmt_Array_Access;
            procedure Finalizer_Scope_1304 with Inline_Always;
      Match_0 : Internal_Entity_Single_Task_Type_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Single_Task_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Cast_Expr : Internal_Entity_Task_Type_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Body_Node;
Cast_Result_4 : Internal_Entity;
Fld_3 : Internal_Entity_Accept_Stmt_Array_Access;
Let_Result_3 : Internal_Entity_Accept_Stmt_Array_Access;
            procedure Finalizer_Scope_1304 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1304;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4552 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 4551 '<FieldAccess for find_accept_stmts at ast.py:18648>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18648
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4550 '<FieldAccess for body_part_for_decl at ast.py:18657>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18657
--# expr-start 4549 '<Match at ast.py:18650>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18650
--# expr-start 4547 '<FieldAccess for parent_basic_decl at ast.py:18650>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18650
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4547
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Single_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 4548 '<FieldAccess for parent_basic_decl at ast.py:18651>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18651
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 4548
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tt Match_1
Cast_Expr := Match_1; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Protected_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Can only be called on EntryDecl in Tasks"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4549
      if Match_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Match_Result.Node, Imprecise_Fallback => False, E_Info => Match_Result.Info);
--# end
--# expr-done 4550
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# property-call-start EntryDecl.find_accept_stmts
Fld_3 := Libadalang.Implementation.Entry_Decl_P_Find_Accept_Stmts (Node => Ent.Node, Root => Cast_Result_4, E_Info => Ent.Info);
--# end
--# expr-done 4551
Let_Result_3 := Fld_3; Inc_Ref (Let_Result_3);
--# end
--# expr-done 4552
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1304;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1304;
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Accept_Stmts;
--# end
--# property-start EntryDecl.find_accept_stmts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18660
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Find_Accept_Stmts
  (Node : Bare_Entry_Decl
      ; Root : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Accept_Stmt_Array_Access
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   Property_Result : Internal_Entity_Accept_Stmt_Array_Access;
            procedure Finalizer_Scope_1305 with Inline_Always;
            procedure Finalizer_Scope_2499 with Inline_Always;
            procedure Finalizer_Scope_2501 with Inline_Always;
            procedure Finalizer_Scope_2502 with Inline_Always;
      Var_Expr : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Accept_Stmt;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Accept_Stmt_Array_Access;
Map_Result_1 : Internal_Entity_Accept_Stmt_Array_Access;
Scope_Result : Internal_Entity_Accept_Stmt_Array_Access;
Result_Var : Internal_Entity_Accept_Stmt_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt;
Singleton : Internal_Entity_Accept_Stmt_Array_Access;
Fld_2 : Internal_Entity_Entry_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Entry_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Accept_Stmt_Array_Access;
Scope_Result_1 : Internal_Entity_Accept_Stmt_Array_Access;
Result_Var_1 : Internal_Entity_Accept_Stmt_Array_Access;
Concat_Result : Internal_Entity_Accept_Stmt_Array_Access;
Let_Result : Internal_Entity_Accept_Stmt_Array_Access;
            procedure Finalizer_Scope_1305 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1305;
            procedure Finalizer_Scope_2499 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2499;
            procedure Finalizer_Scope_2501 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2501;
            procedure Finalizer_Scope_2502 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2502;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4564 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4563 '<Concat at ast.py:18669>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18669
--# expr-start 4553 '<children at ast.py:18666>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18666
      if Root.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 4553
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind child Var_Expr
--# expr-start 4558 '<Mapcat at ast.py:18668>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18668
   --# expr-start 4556 '<Filter at ast.py:18667>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18667
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr;
      begin
         for Item_167 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_167
         --# expr-start 4555 '<Not at ast.py:18667>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18667
--# expr-start 4554 '<IsNull at ast.py:18667>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18667
Is_Null := Item_167.Node = null; 
--# expr-done 4554
Not_Val := not (Is_Null); 
--# expr-done 4555
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_167;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4556
      declare
         Map_Result_1_Vec : Internal_Entity_Accept_Stmt_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_166 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_166
      --# expr-start 4557 '<FieldAccess for find_accept_stmts at ast.py:18668>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18668
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.find_accept_stmts
Fld_1 := Libadalang.Implementation.Entry_Decl_P_Find_Accept_Stmts (Node => Ent.Node, Root => Item_166, E_Info => Ent.Info);
--# end
--# expr-done 4557
         for Item_To_Append of
               Fld_1.Items
         loop
            Internal_Entity_Accept_Stmt_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2501;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Accept_Stmt_Array
           (Items_Count => Natural (Internal_Entity_Accept_Stmt_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Accept_Stmt_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Accept_Stmt_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Accept_Stmt_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 4558
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2499;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Accept_Stmt_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 4559 '<Cast to AcceptStmt at ast.py:18670>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18670
Cast_Expr := Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Accept_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Accept_Stmt
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Accept_Stmt;
   end if;
--# expr-done 4559
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Accept_Stmt then
   --# scope-start
   --# bind stmt Var_Expr_1
--# expr-start 4562 '<If at ast.py:18671>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18671
--# expr-start 4561 '<Eq at ast.py:18672>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18672
--# expr-start 4560 '<FieldAccess for corresponding_entry at ast.py:18672>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18672
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.corresponding_entry
Fld_2 := Libadalang.Implementation.Accept_Stmt_P_Corresponding_Entry (Node => Var_Expr_1.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4560
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 4561
if Is_Equiv then
                Singleton := Create_Internal_Entity_Accept_Stmt_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr_1;
   If_Result := Singleton;
else
   If_Result := No_Internal_Entity_Accept_Stmt_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 4562
Scope_Result_1 := If_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2502;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Accept_Stmt_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Concat_Result := Concat (Result_Var, Result_Var_1); 
--# expr-done 4563
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4564
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1305;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1305;
                     Finalizer_Scope_2499;
                     Finalizer_Scope_2501;
                     Finalizer_Scope_2502;
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Find_Accept_Stmts;
--# end
--# property-start EntryDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18679
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Next_Part_For_Decl
  (Node : Bare_Entry_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1306 with Inline_Always;
      Var_Expr : Internal_Entity_Accept_Stmt;
Fld : Internal_Entity_Accept_Stmt_Array_Access;
Get_Result : Internal_Entity_Accept_Stmt;
Fld_1 : Internal_Entity_Accept_Stmt_Body;
Scope_Result : Internal_Entity_Accept_Stmt_Body;
Result_Var : Internal_Entity_Accept_Stmt_Body;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1306 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1306;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4572 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4571 '<If at ast.py:18686>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18686
--# expr-start 4566 '<IsA SingleTaskTypeDecl, TaskTypeDecl at ast.py:18688>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18688
--# expr-start 4565 '<FieldAccess for parent_basic_decl at ast.py:18688>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18688
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_3 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4565
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Single_Task_Type_Decl_Range | Ada_Task_Type_Decl_Range; 
--# expr-done 4566
if Is_A then
--# expr-start 4568 '<at at ast.py:18690>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18690
--# expr-start 4567 '<FieldAccess for accept_stmts at ast.py:18690>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18690
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.accept_stmts
Fld := Libadalang.Implementation.Entry_Decl_P_Accept_Stmts (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4567
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 4568
Var_Expr := Get_Result;
if Var_Expr /= No_Entity_Accept_Stmt then
   --# scope-start
--# expr-start 4569 '<FieldAccess for body_decl at ast.py:18690>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18690
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Accept_Stmt_Body (Node => Var_Expr.Node.Accept_Stmt_F_Body_Decl, Info => Var_Expr.Info);
--# expr-done 4569
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Accept_Stmt_Body;
end if;
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result := Cast_Result;
else
   --# expr-start 4570 '<Super at ast.py:18693>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18693
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicSubpDecl.next_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Subp_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4570
   If_Result := Fld_2;
end if;
--# expr-done 4571
Let_Result := If_Result; 
--# end
--# expr-done 4572
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1306;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1306;
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Next_Part_For_Decl;
--# end
--# property-start EntryDecl._env_mappings_124 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18697
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_124
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1651 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Entry_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1651 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1651;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4575 '<New EnvAssoc at ast.py:18697>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18697
--# expr-start 4573 '<New DesignatedEnv at ast.py:18697>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18697
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 4573
--# expr-start 4574 '<FieldAccess for name_symbol at ast.py:18697>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18697
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4574
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4575
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1651;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1651;
            raise;
      end;
   return Property_Result;
end Env_Mappings_124;
--# end
--# property-start EntryDecl._env_trans_parent_125 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18696
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_125
  (Node : Bare_Entry_Decl
  )
   return Boolean
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_125;
--# end
   --
   --  Primitives for Bare_Enum_Literal_Decl
   --
      procedure Enum_Literal_Decl_Pre_Env_Actions
        (Self            : Bare_Enum_Literal_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_122 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:17897");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_123 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Enum_Literal_Decl
        (Self : Bare_Enum_Literal_Decl
         ; Enum_Literal_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Enum_Literal_Decl_F_Name := Enum_Literal_Decl_F_Name;
      end Initialize_Fields_For_Enum_Literal_Decl;
   function Enum_Literal_Decl_F_Name
     (Node : Bare_Enum_Literal_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Enum_Literal_Decl_F_Name;
   end;
--# property-start EnumLiteralDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17844
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Is_Static_Decl
  (Node : Bare_Enum_Literal_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Is_Static_Decl;
--# end
--# property-start EnumLiteralDecl.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17846
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Is_Constant_Object
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Is_Constant_Object;
--# end
--# property-start '[dispatcher]EnumLiteralDecl.enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Enum_Literal_Decl_P_Enum_Type
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Enum_Literal_Decl_Range (Self.Kind) is
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.enum_type
                  Property_Result := Enum_Literal_Decl_P_Enum_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start SyntheticCharEnumLit.enum_type
                  Property_Result := Synthetic_Char_Enum_Lit_P_Enum_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Enum_Literal_Decl_P_Enum_Type;
--# end
--# property-start EnumLiteralDecl.enum_rep /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17857
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Enum_Rep
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Big_Integer_Type
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Big_Integer_Type;
            procedure Finalizer_Scope_1233 with Inline_Always;
      Block_Var : Internal_Entity_Type_Decl;
Block_Var_1 : Internal_Entity_Enum_Rep_Clause;
Block_Var_2 : Integer;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Enum_Rep_Clause;
Fld_2 : Integer;
Big_Int : Big_Integer_Type;
Fld_3 : Internal_Entity_Base_Aggregate;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Big_Integer_Type;
Is_Null_1 : Boolean;
If_Result_1 : Big_Integer_Type;
Let_Result : Big_Integer_Type;
            procedure Finalizer_Scope_1233 is
            begin
                     Dec_Ref (Big_Int);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1233;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4586 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4576 '<FieldAccess for enum_type at ast.py:17865>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17865
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4576
Block_Var := Fld; 
--# bind enum_type Block_Var
--# expr-start 4577 '<FieldAccess for get_enum_representation_clause at ast.py:17866>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17866
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.get_enum_representation_clause
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Get_Enum_Representation_Clause (Node => Block_Var.Node, Imprecise_Fallback => False, E_Info => Block_Var.Info);
--# end
--# expr-done 4577
Block_Var_1 := Fld_1; 
--# bind rep_clause Block_Var_1
--# expr-start 4578 '<FieldAccess for child_index at ast.py:17870>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17870
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Child_Index (Node => Self);
--# expr-done 4578
Block_Var_2 := Fld_2; 
--# bind pos Block_Var_2
--# expr-start 4585 '<If at ast.py:17871>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17871
--# expr-start 4579 '<IsNull at ast.py:17872>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17872
Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 4579
if Is_Null_1 then
   --# expr-start 4580 '<BigInteger <Var pos at ast.py:17870> at ast.py:17876>' Big_Int /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17876
Big_Int := Create_Big_Integer (Block_Var_2); 
--# expr-done 4580
   If_Result_1 := Big_Int;
else
   --# expr-start 4584 '<FieldAccess for eval_as_int at ast.py:17882>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17882
--# expr-start 4583 '<FieldAccess for expr at ast.py:17882>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17882
--# expr-start 4582 '<FieldAccess for assocs at ast.py:17882>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17882
--# expr-start 4581 '<FieldAccess for aggregate at ast.py:17882>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17882
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Base_Aggregate (Node => Block_Var_1.Node.Enum_Rep_Clause_F_Aggregate, Info => Block_Var_1.Info);
--# expr-done 4581
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Fld_3.Node.Base_Aggregate_F_Assocs, Info => Fld_3.Info);
--# expr-done 4582
Fld_5 := Fld_4.Node;
Get_Result := Get (Self, Fld_5, Block_Var_2, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 4583
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_8 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 4584
   If_Result_1 := Fld_8;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4585
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4586
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1233;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1233;
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Enum_Rep;
--# end
--# property-start EnumLiteralDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17885
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Defining_Names
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1234 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1234 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1234;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4587 '<FieldAccess for name at ast.py:17885>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17885
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Enum_Literal_Decl_F_Name, Info => Ent.Info);
--# expr-done 4587
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1234;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1234;
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Defining_Names;
--# end
--# property-start EnumLiteralDecl.synth_type_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17887
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Synth_Type_Expr
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Lit_Synth_Type_Expr
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
      New_Node : Bare_Enum_Lit_Synth_Type_Expr;
As_Entity : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Let_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Literal_Decl_Enum_Literal_Decl_P_Synth_Type_Expr,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Literal_Decl,
                                As_Bare_Enum_Literal_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Enum_Lit_Synth_Type_Expr;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4590 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4589 '<as_entity at ast.py:17889>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17889
--# expr-start 4588 '<New EnumLitSynthTypeExpr at ast.py:17889>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17889
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Enum_Lit_Synth_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Enum_Lit_Synth_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 4588
Is_Null := New_Node = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Lit_Synth_Type_Expr;
else
As_Entity := (Info => E_Info, Node => New_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 4589
Let_Result := If_Result; 
--# end
--# expr-done 4590
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Enum_Lit_Synth_Type_Expr,
                        As_Internal_Entity_Enum_Lit_Synth_Type_Expr => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Enum_Literal_Decl_P_Synth_Type_Expr;
--# end
--# property-start EnumLiteralDecl.subp_decl_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17891
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Subp_Decl_Spec
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Subp_Spec;
      New_Node : Bare_Enum_Subp_Spec;
As_Entity : Internal_Entity_Enum_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Subp_Spec;
Let_Result : Internal_Entity_Enum_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Literal_Decl_Enum_Literal_Decl_P_Subp_Decl_Spec,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Literal_Decl,
                                As_Bare_Enum_Literal_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Subp_Spec;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 4593 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4592 '<as_entity at ???>' If_Result None
--# expr-start 4591 '<New EnumSubpSpec at ???>' New_Node None
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Enum_Subp_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Enum_Subp_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 4591
Is_Null := New_Node = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Subp_Spec;
else
As_Entity := (Info => E_Info, Node => New_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 4592
Let_Result := If_Result; 
--# end
--# expr-done 4593
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Let_Result.Node,
         Info => Let_Result.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Subp_Spec,
                        As_Internal_Entity_Base_Subp_Spec => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Enum_Literal_Decl_P_Subp_Decl_Spec;
--# end
--# property-start '[dispatcher]EnumLiteralDecl.enum_decl_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Enum_Literal_Decl_P_Enum_Decl_Name
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Enum_Literal_Decl_Range (Self.Kind) is
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.enum_decl_name
                  Property_Result := Enum_Literal_Decl_P_Enum_Decl_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start SyntheticCharEnumLit.enum_decl_name
                  Property_Result := Synthetic_Char_Enum_Lit_P_Enum_Decl_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Enum_Literal_Decl_P_Enum_Decl_Name;
--# end
--# property-start EnumLiteralDecl._env_mappings_122 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17897
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_122
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1649 with Inline_Always;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Enum_Literal_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1649 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1649;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4598 '<New EnvAssoc at ast.py:17897>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17897
--# expr-start 4596 '<New DesignatedEnv at ast.py:17898>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17898
--# expr-start 4595 '<FieldAccess for node_env at ast.py:17898>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17898
--# expr-start 4594 '<FieldAccess for enum_type at ast.py:17898>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17898
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4594
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Node_Env (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 4595
Inc_Ref (Fld_2);
New_Struct := (Direct_Env => Fld_2, Env_Name => null, Kind => Direct_Env); 
--# expr-done 4596
--# expr-start 4597 '<FieldAccess for name_symbol at ast.py:17897>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17897
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4597
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4598
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1649;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1649;
            raise;
      end;
   return Property_Result;
end Env_Mappings_122;
--# end
--# property-start EnumLiteralDecl._env_trans_parent_123 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17895
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_123
  (Node : Bare_Enum_Literal_Decl
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_123;
--# end
--# property-start EnumLiteralDecl.enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17848
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Enum_Type
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Decl;
            procedure Finalizer_Scope_1232 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
            procedure Finalizer_Scope_1232 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1232;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4604 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4603 '<Cast to TypeDecl at ast.py:17855>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17855
--# expr-start 4602 '<as_entity at ast.py:17855>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17855
--# expr-start 4601 '<Find at ast.py:17853>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17853
   --# expr-start 4599 '<FieldAccess for parents at ast.py:17853>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17853
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 4599
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_168 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_168
      --# expr-start 4600 '<IsA TypeDecl at ast.py:17854>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17854
Is_A := Item_168 /= null 
and then Item_168.Kind in Ada_Type_Decl; 
--# expr-done 4600
      if Is_A then
         Find_Result := Item_168; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 4601
Is_Null := Find_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Find_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 4602
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 4603
Let_Result := Cast_Result; 
--# end
--# expr-done 4604
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1232;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1232;
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Enum_Type;
--# end
--# property-start EnumLiteralDecl.enum_decl_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17906
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Enum_Decl_Name
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4605 '<FieldAccess for name at ast.py:17906>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17906
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Enum_Literal_Decl_F_Name, Info => Ent.Info);
--# expr-done 4605
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Enum_Decl_Name;
--# end
   --
   --  Primitives for Bare_Synthetic_Char_Enum_Lit
   --
      procedure Initialize_Fields_For_Synthetic_Char_Enum_Lit
        (Self : Bare_Synthetic_Char_Enum_Lit
         ; Enum_Literal_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Enum_Literal_Decl
              (Self, Enum_Literal_Decl_F_Name);
      Self.Synthetic_Char_Enum_Lit_Char_Symbol := null;
      Self.Synthetic_Char_Enum_Lit_Enum_Type_Decl := No_Entity_Type_Decl;
      end Initialize_Fields_For_Synthetic_Char_Enum_Lit;
--# property-start SyntheticCharEnumLit.enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17917
pragma Warnings (Off, "is not referenced");
 function Synthetic_Char_Enum_Lit_P_Enum_Type
  (Node : Bare_Synthetic_Char_Enum_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Synthetic_Char_Enum_Lit  := Bare_Synthetic_Char_Enum_Lit (Node);
     Ent : Internal_Entity_Synthetic_Char_Enum_Lit :=
       Internal_Entity_Synthetic_Char_Enum_Lit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4606 '<FieldAccess for enum_type_decl at ast.py:17917>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17917
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Ent.Node.Synthetic_Char_Enum_Lit_Enum_Type_Decl;
--# expr-done 4606
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Char_Enum_Lit_P_Enum_Type;
--# end
--# property-start SyntheticCharEnumLit.expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17919
pragma Warnings (Off, "is not referenced");
 function Synthetic_Char_Enum_Lit_P_Expr
  (Node : Bare_Synthetic_Char_Enum_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Synthetic_Char_Enum_Lit  := Bare_Synthetic_Char_Enum_Lit (Node);
     Ent : Internal_Entity_Synthetic_Char_Enum_Lit :=
       Internal_Entity_Synthetic_Char_Enum_Lit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1239 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1239 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1239;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4608 '<at at ast.py:17919>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17919
--# expr-start 4607 '<FieldAccess for defining_names at ast.py:17919>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17919
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4607
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 4608
         Property_Result := Get_Result;
   --# end
      Finalizer_Scope_1239;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1239;
            raise;
      end;
   return Property_Result;
end Synthetic_Char_Enum_Lit_P_Expr;
--# end
--# property-start SyntheticCharEnumLit.enum_decl_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17922
pragma Warnings (Off, "is not referenced");
 function Synthetic_Char_Enum_Lit_P_Enum_Decl_Name
  (Node : Bare_Synthetic_Char_Enum_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Synthetic_Char_Enum_Lit  := Bare_Synthetic_Char_Enum_Lit (Node);
     Ent : Internal_Entity_Synthetic_Char_Enum_Lit :=
       Internal_Entity_Synthetic_Char_Enum_Lit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4609 '<FieldAccess for expr at ast.py:17922>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17922
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SyntheticCharEnumLit.expr
Fld := Libadalang.Implementation.Synthetic_Char_Enum_Lit_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4609
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Char_Enum_Lit_P_Enum_Decl_Name;
--# end
--# property-start SyntheticCharEnumLit.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17924
pragma Warnings (Off, "is not referenced");
 function Synthetic_Char_Enum_Lit_P_Defining_Names
  (Node : Bare_Synthetic_Char_Enum_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Synthetic_Char_Enum_Lit  := Bare_Synthetic_Char_Enum_Lit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1241 with Inline_Always;
      Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1241 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1241;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4613 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 4612 '<as_entity at ast.py:17925>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17925
--# expr-start 4611 '<FieldAccess for synthesize_defining_name at ast.py:17925>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17925
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4610 '<FieldAccess for char_symbol at ast.py:17925>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17925
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Char_Enum_Lit_Char_Symbol;
--# expr-done 4610
--# property-call-start AdaNode.synthesize_defining_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Synthesize_Defining_Name (Node => Self, Sym => Fld);
--# end
--# expr-done 4611
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 4612
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => If_Result)); 
--# expr-done 4613
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1241;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1241;
            raise;
      end;
   return Property_Result;
end Synthetic_Char_Enum_Lit_P_Defining_Names;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Internal
   --
      procedure Generic_Subp_Internal_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Internal;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_98 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_99 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Internal
        (Self : Bare_Generic_Subp_Internal
         ; Generic_Subp_Internal_F_Subp_Spec : Bare_Subp_Spec
         ; Generic_Subp_Internal_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Subp_Internal_F_Subp_Spec := Generic_Subp_Internal_F_Subp_Spec;
            Self.Generic_Subp_Internal_F_Aspects := Generic_Subp_Internal_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Internal;
   function Generic_Subp_Internal_F_Subp_Spec
     (Node : Bare_Generic_Subp_Internal) return Bare_Subp_Spec
   is
   begin
         return Node.Generic_Subp_Internal_F_Subp_Spec;
   end;
--# property-start GenericSubpInternal.subp_decl_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12025
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Internal_P_Subp_Decl_Spec
  (Node : Bare_Generic_Subp_Internal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Generic_Subp_Internal  := Bare_Generic_Subp_Internal (Node);
     Ent : Internal_Entity_Generic_Subp_Internal :=
       Internal_Entity_Generic_Subp_Internal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4614 '<FieldAccess for subp_spec at ast.py:12025>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12025
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Ent.Info);
--# expr-done 4614
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Internal_P_Subp_Decl_Spec;
--# end
--# property-start GenericSubpInternal._env_trans_parent_98 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12027
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_98
  (Node : Bare_Generic_Subp_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Internal  := Bare_Generic_Subp_Internal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_98;
--# end
--# property-start GenericSubpInternal._env_names_99 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12027
pragma Warnings (Off, "is not referenced");
 function Env_Names_99
  (Node : Bare_Generic_Subp_Internal
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Subp_Internal  := Bare_Generic_Subp_Internal (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1626 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1626 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1626;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4615 '<FieldAccess for env_names at ast.py:12027>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12027
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicSubpDecl.env_names
Fld := Libadalang.Implementation.Basic_Subp_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 4615
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1626;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1626;
            raise;
      end;
   return Property_Result;
end Env_Names_99;
--# end
   --
   --  Primitives for Bare_Synthetic_Subp_Decl
   --
      procedure Initialize_Fields_For_Synthetic_Subp_Decl
        (Self : Bare_Synthetic_Subp_Decl
         ; Synthetic_Subp_Decl_F_Spec : Bare_Base_Subp_Spec
        ) is
      begin
            Self.Synthetic_Subp_Decl_F_Spec := Synthetic_Subp_Decl_F_Spec;
      end Initialize_Fields_For_Synthetic_Subp_Decl;
   function Synthetic_Subp_Decl_F_Spec
     (Node : Bare_Synthetic_Subp_Decl) return Bare_Base_Subp_Spec
   is
   begin
         return Node.Synthetic_Subp_Decl_F_Spec;
   end;
--# property-start SyntheticSubpDecl.subp_decl_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18581
pragma Warnings (Off, "is not referenced");
 function Synthetic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Synthetic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Synthetic_Subp_Decl  := Bare_Synthetic_Subp_Decl (Node);
     Ent : Internal_Entity_Synthetic_Subp_Decl :=
       Internal_Entity_Synthetic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4616 '<FieldAccess for spec at ast.py:18581>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18581
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Subp_Spec (Node => Ent.Node.Synthetic_Subp_Decl_F_Spec, Info => Ent.Info);
--# expr-done 4616
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start SyntheticSubpDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18583
pragma Warnings (Off, "is not referenced");
 function Synthetic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Synthetic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Synthetic_Subp_Decl  := Bare_Synthetic_Subp_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4617 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Entity_Basic_Decl; 
--# end
--# expr-done 4617
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start SyntheticSubpDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18587
pragma Warnings (Off, "is not referenced");
 function Synthetic_Subp_Decl_P_Is_Static_Decl
  (Node : Bare_Synthetic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Synthetic_Subp_Decl  := Bare_Synthetic_Subp_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4618 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := True; 
--# end
--# expr-done 4618
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Subp_Decl_P_Is_Static_Decl;
--# end
   --
   --  Primitives for Bare_Body_Node
   --
--# property-start Body.env_hook_body /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3598
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Env_Hook_Body
  (Node : Bare_Body_Node
  )
   return Boolean
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_224 with Inline_Always;
      Unused_1 : Internal_Unit;
Var_Expr : Bare_Subp_Body;
As_Entity : Internal_Entity_Body_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Fld_2 : Internal_Unit;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Subp_Body;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_224 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_224;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4630 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 4629 '<If at ast.py:3603>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3603
--# expr-start 4619 '<IsA PackageBody, SubpBody at ast.py:3606>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3606
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Package_Body_Range | Ada_Subp_Body_Range; 
--# expr-done 4619
if Is_A_1 then
   --# expr-start 4628 '<Let at ast.py:3608>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3608
--# scope-start
--# expr-start 4625 '<FieldAccess for get_unit at ast.py:3609>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3609
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4622 '<FieldAccess for as_symbol_array at ast.py:3610>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3610
--# expr-start 4621 '<FieldAccess for defining_name at ast.py:3610>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3610
--# expr-start 4620 '<as_bare_entity at ast.py:3610>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3610
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4620
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 4621
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld.Node);
--# end
--# expr-done 4622
--# expr-start 4624 '<Not at ast.py:3613>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3613
--# expr-start 4623 '<IsA SubpBody at ast.py:3613>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3613
Is_A := Self /= null 
and then Self.Kind in Ada_Subp_Body_Range; 
--# expr-done 4623
Not_Val := not (Is_A); 
--# expr-done 4624
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_1, Kind => Unit_Specification, Load_If_Needed => True, Not_Found_Is_Error => Not_Val);
--# expr-done 4625
Unused_1 := Fld_2; 
--# bind _ Unused_1
--# expr-start 4626 '<Cast to SubpBody at ast.py:3618>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3618
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Subp_Body_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4626
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind subp_body Var_Expr
--# expr-start 4627 '<FieldAccess for env_hook_basic_decl at ast.py:3619>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3619
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.env_hook_basic_decl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Env_Hook_Basic_Decl (Node => Var_Expr);
--# end
--# expr-done 4627
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4628
   If_Result_1 := Let_Result;
else
   If_Result_1 := False;
end if;
--# expr-done 4629
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 4630
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_224;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_224;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Env_Hook_Body;
--# end
--# property-start Body.subunit_decl_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3626
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Decl_Env
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_225 with Inline_Always;
            procedure Finalizer_Scope_2508 with Inline_Always;
            procedure Finalizer_Scope_2509 with Inline_Always;
            procedure Finalizer_Scope_2511 with Inline_Always;
            procedure Finalizer_Scope_2512 with Inline_Always;
      Var_Expr : Lexical_Env := Empty_Env;
Match_0 : Internal_Entity_Generic_Package_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Match_2 : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Generic_Package_Internal;
Fld_5 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Decl;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_7 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_225 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_225;
            procedure Finalizer_Scope_2508 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2508;
            procedure Finalizer_Scope_2509 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2509;
            procedure Finalizer_Scope_2511 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2511;
            procedure Finalizer_Scope_2512 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
            end Finalizer_Scope_2512;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4650 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 4649 '<bind at ast.py:3628>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3628
--# expr-start 4631 '<FieldAccess for default_initial_env at ast.py:3629>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3629
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4631
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4642 '<Match at ast.py:3633>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3633
--# expr-start 4635 '<at at ast.py:3633>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3633
--# expr-start 4634 '<EnvGet at ast.py:3630>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3630
--# expr-start 4632 '<FieldAccess for body_scope at ast.py:3630>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3630
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 4632
--# expr-start 4633 '<FieldAccess for name_symbol at ast.py:3631>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3631
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4633
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Fld_2, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 4634
Get_Result := Get (Self, Env_Get_Result, 0, True); 
--# expr-done 4635
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Get_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind gpd Match_0
--# expr-start 4640 '<env_group at ast.py:3639>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3639
--# expr-start 4639 '<ArrayLiteral at ast.py:3638>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3638
--# expr-start 4636 '<FieldAccess for children_env at ast.py:3638>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3638
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Match_0.Node, E_Info => Match_0.Info);
--# expr-done 4636
--# expr-start 4638 '<FieldAccess for children_env at ast.py:3638>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3638
--# expr-start 4637 '<FieldAccess for package_decl at ast.py:3638>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3638
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_0.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_0.Info);
--# expr-done 4637
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 4638
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_3, 2 => Fld_5)); 
--# expr-done 4639
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 4640
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2508;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind pd Match_1
--# expr-start 4641 '<FieldAccess for children_env at ast.py:3640>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3640
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Match_1.Node, E_Info => Match_1.Info);
--# expr-done 4641
Let_Result_1 := Fld_6; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2509;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 4642
Var_Expr := Match_Result;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind public_part Var_Expr
--# expr-start 4645 '<at at ast.py:3644>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3644
--# expr-start 4644 '<EnvGet at ast.py:3642>' Env_Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3642
--# expr-start 4643 '<Symbol __privatepart at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Privatepart); 
--# expr-done 4643
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Var_Expr, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 4644
Get_Result_1 := Get (Self, Env_Get_Result_1, 0, True); 
--# expr-done 4645
Var_Expr_1 := Get_Result_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 4648 '<env_group at ast.py:3646>' Group_Env_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3646
--# expr-start 4647 '<ArrayLiteral at ast.py:3646>' Array_Lit_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3646
--# expr-start 4646 '<FieldAccess for children_env at ast.py:3646>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3646
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 4646
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_7, 2 => Var_Expr)); 
--# expr-done 4647
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 4648
Scope_Result_3 := Group_Env_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2512;
   Result_Var := Scope_Result_3;
else
   Result_Var := Var_Expr;
end if;
      Inc_Ref (Result_Var);
Scope_Result_4 := Result_Var;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2511;
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Dyn_Var_Bind_Result := Result_Var_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4649
Let_Result_3 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 4650
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_225;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_225;
                     Finalizer_Scope_2508;
                     Finalizer_Scope_2509;
                     Finalizer_Scope_2511;
                     Finalizer_Scope_2512;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Subunit_Decl_Env;
--# end
--# property-start Body.in_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3651
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_In_Scope
  (Node : Bare_Body_Node
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_226 with Inline_Always;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Unit;
Fld_1 : Internal_Unit;
Is_Equal : Boolean;
If_Result : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Find_Result : Bare_Ada_Node;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_226 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_226;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4661 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4652 '<Not at ast.py:3657>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3657
--# expr-start 4651 '<IsNull at ast.py:3657>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3657
Is_Null := Origin = null; 
--# expr-done 4651
Not_Val := not (Is_Null); 
--# expr-done 4652
if Not_Val then
   --# expr-start 4655 '<Eq at ast.py:3658>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3658
--# expr-start 4653 '<FieldAccess for unit at ast.py:3658>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3658
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Origin);
--# expr-done 4653
--# expr-start 4654 '<FieldAccess for unit at ast.py:3658>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3658
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 4654
Is_Equal := Fld = Fld_1; 
--# expr-done 4655
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4660 '<Not at ast.py:3659>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3659
--# expr-start 4659 '<IsNull at ast.py:3659>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3659
--# expr-start 4658 '<Find at ast.py:3659>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3659
   --# expr-start 4656 '<FieldAccess for parents at ast.py:3659>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3659
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Origin, True); 
--# expr-done 4656
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_169 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_169
      --# expr-start 4657 '<Eq at ast.py:3659>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3659
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Cast_Result = Item_169; 
--# expr-done 4657
      if Is_Equal_1 then
         Find_Result := Item_169; 
      end if;
   --# end
               exit when Is_Equal_1;
         end loop;
      end;
--# expr-done 4658
Is_Null_1 := Find_Result = null; 
--# expr-done 4659
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4660
   If_Result_1 := Not_Val_1;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 4661
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_226;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_226;
            raise;
      end;
   return Property_Result;
end Body_Node_P_In_Scope;
--# end
--# property-start Body.body_decl_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3662
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Decl_Scope
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_227 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_227 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_227;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4665 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4664 '<bind at ast.py:3667>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3667
--# expr-start 4662 '<FieldAccess for default_initial_env at ast.py:3668>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3668
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4662
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4663 '<FieldAccess for body_scope at ast.py:3669>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3669
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 4663
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4664
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4665
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_227;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_227;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Body_Decl_Scope;
--# end
--# property-start Body.body_initial_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3672
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Initial_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_228 with Inline_Always;
      Fld : Boolean;
Fld_1 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Fld_2 : Boolean;
Fld_3 : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Sym_1 : Symbol_Type;
If_Result : Symbol_Type;
If_Result_1 : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_228 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_228;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4677 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4676 '<Cond at ast.py:3679>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3679
--# expr-start 4666 '<FieldAccess for is_library_item at ast.py:3680>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3680
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 4666
if Fld then
   --# expr-start 4670 '<to_symbol at ast.py:3681>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3681
--# expr-start 4669 '<Concat at ast.py:3681>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3681
--# expr-start 4667 '<FieldAccess for top_level_env_name at ast.py:3681>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3681
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4667
--# expr-start 4668 '<String at ast.py:3681>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3681
Str := Create_String (".__privatepart"); 
--# expr-done 4668
Concat_Result := Concat_String (Fld_1, Str); 
--# expr-done 4669
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 4670
   If_Result_1 := Sym;
else
--# expr-start 4671 '<FieldAccess for is_subunit at ast.py:3683>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3683
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld_2 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 4671
if Fld_2 then
   --# expr-start 4675 '<to_symbol at ast.py:3684>' Sym_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3684
--# expr-start 4674 '<Concat at ast.py:3684>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3684
--# expr-start 4672 '<FieldAccess for top_level_env_name at ast.py:3684>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3684
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4672
--# expr-start 4673 '<String at ast.py:3684>' Str_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3684
Str_1 := Create_String ("__stub"); 
--# expr-done 4673
Concat_Result_1 := Concat_String (Fld_3, Str_1); 
--# expr-done 4674
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Concat_Result_1); 
--# expr-done 4675
   If_Result := Sym_1;
else
   If_Result := null;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 4676
Let_Result := If_Result_1; 
--# end
--# expr-done 4677
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_228;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_228;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Body_Initial_Env_Name;
--# end
--# property-start Body.body_initial_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3689
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Initial_Env
  (Node : Bare_Body_Node
  )
   return Internal_Designated_Env
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_229 with Inline_Always;
            procedure Finalizer_Scope_2514 with Inline_Always;
      Var_Expr : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Let_Result : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_229 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_229;
            procedure Finalizer_Scope_2514 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2514;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4681 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4678 '<FieldAccess for body_initial_env_name at ast.py:3697>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3697
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env_name
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4678
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind non_null_name Var_Expr
--# expr-start 4679 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr, Kind => Named_Env); 
--# expr-done 4679
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2514;
   Result_Var := Scope_Result;
else
   --# expr-start 4680 '<New DesignatedEnv at ast.py:3696>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3696
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 4680
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 4681
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_229;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_229;
                     Finalizer_Scope_2514;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Body_Initial_Env;
--# end
--# property-start '[dispatcher]Body.previous_part_env_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Body_Node_P_Previous_Part_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Body_Node (Self.Kind) is
               when Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Task_Body =>
                  --# property-call-start Body.previous_part_env_name
                  Property_Result := Body_Node_P_Previous_Part_Env_Name
                    (Self
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.previous_part_env_name
                  Property_Result := Base_Subp_Body_P_Previous_Part_Env_Name
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Body_Node_P_Previous_Part_Env_Name;
--# end
--# property-start '[dispatcher]Body.previous_part_link_env_assoc' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Body_Node (Self.Kind) is
               when Ada_Accept_Stmt_Body .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Task_Body =>
                  --# property-call-start Body.previous_part_link_env_assoc
                  Property_Result := Body_Node_P_Previous_Part_Link_Env_Assoc
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
                  --# property-call-start BodyStub.previous_part_link_env_assoc
                  Property_Result := Body_Stub_P_Previous_Part_Link_Env_Assoc
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc;
--# end
--# property-start Body.subp_previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3744
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subp_Previous_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_232 with Inline_Always;
            procedure Finalizer_Scope_2517 with Inline_Always;
      Block_Var : Internal_Entity;
Block_Var_1 : Internal_Entity_Array_Access;
Block_Var_2 : Boolean;
Block_Var_3 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Compilation_Unit;
Match_0 : Internal_Entity_Generic_Subp_Decl;
Match_1 : Internal_Entity_Formal_Subp_Decl;
Match_2 : Internal_Entity_Basic_Subp_Decl;
Match_3 : Internal_Entity_Subp_Body_Stub;
Match_4 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Fld_3 : Boolean;
If_Result : Internal_Entity;
Is_Null : Boolean;
Is_A : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_4 : Internal_Entity_Compilation_Unit;
Fld_5 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Singleton : Internal_Entity_Array_Access;
Fld_6 : Bare_Basic_Decl;
Is_A_1 : Boolean;
If_Result_2 : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Singleton_1 : Internal_Entity_Array_Access;
Fld_7 : Lexical_Env := Empty_Env;
Fld_8 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
If_Result_3 : Internal_Entity_Array_Access;
If_Result_4 : Internal_Entity_Array_Access;
If_Result_5 : Internal_Entity_Array_Access;
Is_A_3 : Boolean;
Get_Result : Internal_Entity;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Fld_9 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Body_Node;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
If_Result_6 : Boolean;
Fld_10 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Internal_Entity_Base_Subp_Spec;
Cast_Result_3 : Internal_Entity_Subp_Spec;
Cast_Result_4 : Internal_Entity_Base_Subp_Spec;
Fld_13 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_14 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_15 : Internal_Entity_Subp_Spec;
Fld_16 : Internal_Entity_Base_Subp_Spec;
Cast_Result_5 : Internal_Entity_Subp_Spec;
Cast_Result_6 : Internal_Entity_Base_Subp_Spec;
Fld_17 : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_7 : Internal_Entity_Generic_Subp_Decl;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_8 : Internal_Entity_Formal_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_9 : Internal_Entity_Basic_Subp_Decl;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_10 : Internal_Entity_Subp_Body_Stub;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_11 : Internal_Entity;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Match_Result : Boolean;
If_Result_7 : Boolean;
Find_Result : Internal_Entity;
If_Result_8 : Internal_Entity;
Cast_Result_12 : Internal_Entity_Basic_Decl;
Is_Null_3 : Boolean;
Not_Val_2 : Boolean;
Fld_18 : Bare_Ada_Node;
Cast_Expr_2 : Bare_Body_Node;
Cast_Result_13 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Not_Val_3 : Boolean;
If_Result_9 : Boolean;
Is_A_4 : Boolean;
If_Result_10 : Boolean;
Find_Result_1 : Internal_Entity;
Cast_Result_14 : Internal_Entity_Basic_Decl;
Is_Null_4 : Boolean;
If_Result_11 : Boolean;
If_Result_12 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_232 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Block_Var_1);
            end Finalizer_Scope_232;
            procedure Finalizer_Scope_2517 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_2517;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4735 '<Block at ???>' Let_Result_5 None
--# scope-start
--# expr-start 4686 '<If at ast.py:3756>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3756
--# expr-start 4682 '<FieldAccess for is_library_item at ast.py:3757>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3757
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Ent.Node);
--# end
--# expr-done 4682
if Fld_3 then
   --# expr-start 4684 '<parent at ast.py:3758>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3758
--# expr-start 4683 '<parent at ast.py:3758>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3758
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4683
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4684
   If_Result := Fld_1;
else
   --# expr-start 4685 '<FieldAccess for semantic_parent at ast.py:3759>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3759
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4685
   If_Result := Fld_2;
end if;
--# expr-done 4686
Block_Var := If_Result; 
--# bind parent Block_Var
--# expr-start 4701 '<Cond at ast.py:3762>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3762
--# expr-start 4687 '<IsNull at ast.py:3763>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3763
Is_Null := Block_Var.Node = null; 
--# expr-done 4687
if Is_Null then
   If_Result_5 := No_Internal_Entity_Array_Type;
else
--# expr-start 4688 '<IsA CompilationUnit at ast.py:3768>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3768
Is_A := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 4688
if Is_A then
--# expr-start 4690 '<FieldAccess for other_part at ast.py:3769>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3769
--# expr-start 4689 '<Cast to CompilationUnit at ast.py:3769>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3769
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Compilation_Unit;
   end if;
--# expr-done 4689
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.other_part
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Other_Part (Node => Cast_Result.Node);
--# end
--# expr-done 4690
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Compilation_Unit then
   --# scope-start
   --# bind cu Var_Expr
--# expr-start 4696 '<If at ast.py:3772>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3772
--# expr-start 4692 '<IsA BasicSubpDecl, GenericSubpDecl at ast.py:3773>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3773
--# expr-start 4691 '<FieldAccess for decl at ast.py:3773>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3773
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_6 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr.Node);
--# end
--# expr-done 4691
Is_A_1 := Fld_6 /= null 
and then Fld_6.Kind in Ada_Basic_Subp_Decl | Ada_Generic_Subp_Decl_Range; 
--# expr-done 4692
if Is_A_1 then
   --# expr-start 4695 '<as_entity at ast.py:3774>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3774
--# expr-start 4694 '<Cast to AdaNode at ast.py:3774>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3774
--# expr-start 4693 '<FieldAccess for decl at ast.py:3774>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3774
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_5 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr.Node);
--# end
--# expr-done 4693
      Cast_Result_1 := Fld_5;
--# expr-done 4694
Is_Null_1 := Cast_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Cast_Result_1); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 4695
                Singleton := Create_Internal_Entity_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result_1;
   If_Result_2 := Singleton;
else
   If_Result_2 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4696
Scope_Result := If_Result_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2517;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
   If_Result_4 := Result_Var;
else
--# expr-start 4697 '<IsA BodyStub at ast.py:3781>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3781
Is_A_2 := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Body_Stub; 
--# expr-done 4697
if Is_A_2 then
                Singleton_1 := Create_Internal_Entity_Array (Items_Count => 1);
                Singleton_1.Items (1) := Block_Var;
   If_Result_3 := Singleton_1;
else
   --# expr-start 4700 '<EnvGet at ast.py:3785>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3785
--# expr-start 4698 '<FieldAccess for immediate_declarative_region at ast.py:3785>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3785
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.immediate_declarative_region'
Fld_7 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Immediate_Declarative_Region (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 4698
--# expr-start 4699 '<FieldAccess for name_symbol at ast.py:3786>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3786
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4699
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_7, Key => Fld_8, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 4700
   If_Result_3 := Env_Get_Result;
end if;
      Inc_Ref (If_Result_3);
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 4701
Block_Var_1 := If_Result_5; Inc_Ref (Block_Var_1);
--# bind elements Block_Var_1
--# expr-start 4702 '<IsA CompilationUnit, BodyStub at ast.py:3794>' Is_A_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3794
Is_A_3 := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Compilation_Unit_Range | Ada_Body_Stub; 
--# expr-done 4702
Block_Var_2 := Is_A_3; 
--# bind already_precise Block_Var_2
--# expr-start 4724 '<Cast to BasicDecl.entity at ast.py:3834>' Cast_Result_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3834
--# expr-start 4723 '<If at ast.py:3796>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3796
if Block_Var_2 then
   --# expr-start 4703 '<at at ast.py:3798>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3798
Get_Result := Get (Self, Block_Var_1, 0, True); 
--# expr-done 4703
   If_Result_8 := Get_Result;
else
   --# expr-start 4722 '<Find at ast.py:3799>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3799
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_170 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_170
--# expr-start 4705 '<Not at ast.py:3800>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3800
--# expr-start 4704 '<IsNull at ast.py:3800>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3800
Is_Null_2 := Item_170.Node = null; 
--# expr-done 4704
Not_Val := not (Is_Null_2); 
--# expr-done 4705
if Not_Val then
   --# expr-start 4708 '<Not at ast.py:3801>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3801
--# expr-start 4707 '<Eq at ast.py:3801>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3801
--# expr-start 4706 '<FieldAccess for node at ast.py:3801>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3801
Fld_9 := Item_170.Node;
--# expr-done 4706
Cast_Expr_1 := Self; 
      Cast_Result_2 := Cast_Expr_1;
Is_Equal := Fld_9 = Cast_Result_2; 
--# expr-done 4707
Not_Val_1 := not (Is_Equal); 
--# expr-done 4708
   If_Result_6 := Not_Val_1;
else
   If_Result_6 := False;
end if;
if If_Result_6 then
   --# expr-start 4721 '<Match at ast.py:3802>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3802
      if Item_170.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_170; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_7; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_8; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_9; 
--# bind subp_decl Match_2
--# expr-start 4714 '<bind at ast.py:3811>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3811
--# expr-start 4709 '<FieldAccess for origin_node at ast.py:3812>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3812
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_10 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4709
Bound_Origin := Fld_10; 
--# expr-start 4713 '<FieldAccess for match_signature at ast.py:3813>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3813
--# expr-start 4710 '<FieldAccess for subp_decl_spec at ast.py:3813>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3813
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 4710
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4712 '<Cast to SubpSpec at ast.py:3814>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3814
--# expr-start 4711 '<FieldAccess for subp_spec_or_null at ast.py:3814>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3814
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4711
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Cast_Result_3 := No_Entity_Subp_Spec;
   end if;
--# expr-done 4712
      Cast_Result_4 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_3.Node,
         Info => Cast_Result_3.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_13 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_11.Node, Other => Cast_Result_4, Match_Name => True, Use_Entity_Info => False, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 4713
Dyn_Var_Bind_Result := Fld_13; 
--# expr-done 4714
Let_Result_2 := Dyn_Var_Bind_Result; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_10; 
--# bind subp_stub Match_3
--# expr-start 4720 '<bind at ast.py:3821>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3821
--# expr-start 4715 '<FieldAccess for origin_node at ast.py:3822>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3822
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_14 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4715
Bound_Origin_1 := Fld_14; 
--# expr-start 4719 '<FieldAccess for match_signature at ast.py:3823>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3823
--# expr-start 4716 '<FieldAccess for subp_spec at ast.py:3823>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3823
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Subp_Spec (Node => Match_3.Node.Subp_Body_Stub_F_Subp_Spec, Info => Match_3.Info);
--# expr-done 4716
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4718 '<Cast to SubpSpec at ast.py:3824>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3824
--# expr-start 4717 '<FieldAccess for subp_spec_or_null at ast.py:3824>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3824
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4717
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result_5 := No_Entity_Subp_Spec;
   end if;
--# expr-done 4718
      Cast_Result_6 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_5.Node,
         Info => Cast_Result_5.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_17 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_15.Node, Other => Cast_Result_6, Match_Name => True, Use_Entity_Info => False, Origin => Bound_Origin_1, E_Info => Fld_15.Info);
--# end
--# expr-done 4719
Dyn_Var_Bind_Result_1 := Fld_17; 
--# expr-done 4720
Let_Result_3 := Dyn_Var_Bind_Result_1; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Formal_Type_Decl | Ada_Accept_Stmt_Body .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_11; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 4721
   If_Result_7 := Match_Result;
else
   If_Result_7 := False;
end if;
      if If_Result_7 then
         Find_Result := Item_170; 
      end if;
   --# end
               exit when If_Result_7;
         end loop;
      end;
--# expr-done 4722
   If_Result_8 := Find_Result;
end if;
--# expr-done 4723
   if If_Result_8.Node = null
      or else If_Result_8.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_12 := Create_Internal_Entity_Basic_Decl
        (Node => If_Result_8.Node,
         Info => If_Result_8.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4724
Block_Var_3 := Cast_Result_12; 
--# bind precise Block_Var_3
--# expr-start 4734 '<If at ast.py:3836>' If_Result_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3836
--# expr-start 4725 '<IsNull at ast.py:3837>' Is_Null_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3837
Is_Null_4 := Block_Var_3.Node = null; 
--# expr-done 4725
if Is_Null_4 then
   If_Result_11 := Imprecise_Fallback;
else
   If_Result_11 := False;
end if;
if If_Result_11 then
   --# expr-start 4733 '<Cast to BasicDecl.entity at ast.py:3844>' Cast_Result_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3844
--# expr-start 4732 '<Find at ast.py:3838>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3838
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_171 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_171
--# expr-start 4727 '<Not at ast.py:3840>' Not_Val_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3840
--# expr-start 4726 '<IsNull at ast.py:3840>' Is_Null_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3840
Is_Null_3 := Item_171.Node = null; 
--# expr-done 4726
Not_Val_2 := not (Is_Null_3); 
--# expr-done 4727
if Not_Val_2 then
   --# expr-start 4730 '<Not at ast.py:3841>' Not_Val_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3841
--# expr-start 4729 '<Eq at ast.py:3841>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3841
--# expr-start 4728 '<FieldAccess for node at ast.py:3841>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3841
Fld_18 := Item_171.Node;
--# expr-done 4728
Cast_Expr_2 := Self; 
      Cast_Result_13 := Cast_Expr_2;
Is_Equal_1 := Fld_18 = Cast_Result_13; 
--# expr-done 4729
Not_Val_3 := not (Is_Equal_1); 
--# expr-done 4730
   If_Result_9 := Not_Val_3;
else
   If_Result_9 := False;
end if;
if If_Result_9 then
   --# expr-start 4731 '<IsA BasicSubpDecl, SubpBodyStub at ast.py:3842>' Is_A_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3842
Is_A_4 := Item_171.Node /= null 
and then Item_171.Node.Kind in Ada_Basic_Subp_Decl | Ada_Subp_Body_Stub_Range; 
--# expr-done 4731
   If_Result_10 := Is_A_4;
else
   If_Result_10 := False;
end if;
      if If_Result_10 then
         Find_Result_1 := Item_171; 
      end if;
   --# end
               exit when If_Result_10;
         end loop;
      end;
--# expr-done 4732
   if Find_Result_1.Node = null
      or else Find_Result_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_14 := Create_Internal_Entity_Basic_Decl
        (Node => Find_Result_1.Node,
         Info => Find_Result_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4733
   If_Result_12 := Cast_Result_14;
else
   If_Result_12 := Block_Var_3;
end if;
--# expr-done 4734
Let_Result_5 := If_Result_12; 
--# end
--# expr-done 4735
         Property_Result := Let_Result_5;
   --# end
      Finalizer_Scope_232;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_232;
                     Finalizer_Scope_2517;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Subp_Previous_Part;
--# end
--# property-start Body.entry_previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3848
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Entry_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_233 with Inline_Always;
      Block_Var : Internal_Entity_Entry_Completion_Formal_Params;
Match_0 : Internal_Entity_Entry_Decl;
Match_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Entry_Body;
Fld : Internal_Entity_Entry_Completion_Formal_Params;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Body_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Internal_Entity_Entry_Spec;
Cast_Expr_2 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Entry_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_4 : Internal_Entity;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Find_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Entry_Decl;
Let_Result_2 : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_233 is
            begin
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_233;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4752 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 4737 '<FieldAccess for params at ast.py:3853>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3853
--# expr-start 4736 '<Cast to EntryBody at ast.py:3853>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3853
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4736
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Cast_Result.Node.Entry_Body_F_Params, Info => Cast_Result.Info);
--# expr-done 4737
Block_Var := Fld; 
--# bind spec Block_Var
--# expr-start 4751 '<Cast to EntryDecl.entity at ast.py:3866>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3866
--# expr-start 4750 '<Find at ast.py:3854>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3854
   --# expr-start 4739 '<EnvGet at ast.py:3854>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3854
--# expr-start 4738 '<FieldAccess for name_symbol at ast.py:3854>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3854
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4738
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Fld_1, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 4739
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_172 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_172
--# expr-start 4741 '<Not at ast.py:3856>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3856
--# expr-start 4740 '<IsNull at ast.py:3856>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3856
Is_Null := Item_172.Node = null; 
--# expr-done 4740
Not_Val := not (Is_Null); 
--# expr-done 4741
if Not_Val then
   --# expr-start 4744 '<Not at ast.py:3857>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3857
--# expr-start 4743 '<Eq at ast.py:3857>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3857
--# expr-start 4742 '<FieldAccess for node at ast.py:3857>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3857
Fld_2 := Item_172.Node;
--# expr-done 4742
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Fld_2 = Cast_Result_1; 
--# expr-done 4743
Not_Val_1 := not (Is_Equal); 
--# expr-done 4744
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4749 '<Match at ast.py:3858>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3858
      if Item_172.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_172; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Entry_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Entry_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind entry_decl Match_0
--# expr-start 4748 '<bind at ast.py:3859>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3859
--# expr-start 4745 '<FieldAccess for origin_node at ast.py:3860>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3860
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4745
Bound_Origin := Fld_3; 
--# expr-start 4747 '<FieldAccess for match_formal_params at ast.py:3861>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3861
--# expr-start 4746 '<FieldAccess for spec at ast.py:3861>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3861
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Entry_Spec (Node => Match_0.Node.Entry_Decl_F_Spec, Info => Match_0.Info);
--# expr-done 4746
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_4.Node, Other => Cast_Result_2, Origin => Bound_Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4747
Dyn_Var_Bind_Result := Fld_5; 
--# expr-done 4748
Let_Result := Dyn_Var_Bind_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Subp_Decl | Ada_Enum_Literal_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4749
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
      if If_Result_1 then
         Find_Result := Item_172; 
      end if;
   --# end
               exit when If_Result_1;
         end loop;
      end;
--# expr-done 4750
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Entry_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4751
Let_Result_2 := Cast_Result_5; 
--# end
--# expr-done 4752
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_233;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_233;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Entry_Previous_Part;
--# end
--# property-start Body.package_previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3868
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Package_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_234 with Inline_Always;
            procedure Finalizer_Scope_2528 with Inline_Always;
            procedure Finalizer_Scope_2529 with Inline_Always;
      Var_Expr : Internal_Entity_Compilation_Unit;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Package_Decl;
Match_1 : Internal_Entity_Generic_Package_Decl;
Match_2 : Internal_Entity;
Fld : Bare_Compilation_Unit;
Fld_1 : Internal_Entity_Compilation_Unit;
Fld_2 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Singleton : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Scope_Result_1 : Internal_Entity_Array_Access;
Result_Var_1 : Internal_Entity_Array_Access;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Fld_6 : Boolean;
If_Result_1 : Internal_Entity_Array_Access;
Fld_7 : Internal_Entity_Generic_Package_Internal;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Package_Decl;
Cast_Expr : Internal_Entity_Package_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Generic_Package_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Find_Result : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_234 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_234;
            procedure Finalizer_Scope_2528 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2528;
            procedure Finalizer_Scope_2529 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2529;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4770 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 4769 '<Find at ast.py:3891>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3891
   --# expr-start 4766 '<Map at ast.py:3884>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3884
   --# expr-start 4763 '<If at ast.py:3876>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3876
--# expr-start 4753 '<FieldAccess for is_library_item at ast.py:3877>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3877
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 4753
if Fld_6 then
--# expr-start 4755 '<FieldAccess for other_part at ast.py:3878>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3878
--# expr-start 4754 '<FieldAccess for enclosing_compilation_unit at ast.py:3878>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3878
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 4754
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.other_part
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Other_Part (Node => Fld);
--# end
--# expr-done 4755
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Compilation_Unit then
   --# scope-start
   --# bind part Var_Expr
--# expr-start 4758 '<Cast to AdaNode at ast.py:3879>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3879
--# expr-start 4757 '<as_entity at ast.py:3879>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3879
--# expr-start 4756 '<FieldAccess for decl at ast.py:3879>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3879
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr.Node);
--# end
--# expr-done 4756
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 4757
      Cast_Result := Create_Internal_Entity
        (Node => If_Result.Node,
         Info => If_Result.Info);
--# expr-done 4758
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
                Singleton := Create_Internal_Entity_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr_1;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2529;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2528;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_1 := Result_Var_1;
else
   --# expr-start 4762 '<bind at ast.py:3881>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3881
--# expr-start 4759 '<FieldAccess for origin_node at ast.py:3882>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3882
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4759
Bound_Origin := Fld_3; 
--# expr-start 4761 '<FieldAccess for all_env_els_impl at ast.py:3882>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3882
--# expr-start 4760 '<FieldAccess for defining_name at ast.py:3882>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3882
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4760
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_4.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4761
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4762
   If_Result_1 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4763
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := If_Result_1;
      begin
         for Item_174 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_174
      --# expr-start 4765 '<Match at ast.py:3885>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3885
      if Item_174.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_174; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind pkg_decl Match_0
Cast_Expr := Match_0; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_2; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind gen_pkg_decl Match_1
--# expr-start 4764 '<FieldAccess for package_decl at ast.py:3888>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3888
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_1.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_1.Info);
--# expr-done 4764
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Let_Result_1 := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Generic_Package_Internal | Ada_Discrete_Base_Subtype_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 4765
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Match_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4766
      Find_Result := No_Entity_Basic_Decl;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result;
      begin
         for Item_173 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_173
      --# expr-start 4768 '<Not at ast.py:3892>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3892
--# expr-start 4767 '<IsNull at ast.py:3892>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3892
Is_Null_1 := Item_173.Node = null; 
--# expr-done 4767
Not_Val := not (Is_Null_1); 
--# expr-done 4768
      if Not_Val then
         Find_Result := Item_173; 
      end if;
   --# end
               exit when Not_Val;
         end loop;
      end;
--# expr-done 4769
Let_Result_3 := Find_Result; 
--# end
--# expr-done 4770
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_234;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_234;
                     Finalizer_Scope_2528;
                     Finalizer_Scope_2529;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Package_Previous_Part;
--# end
--# property-start Body.task_previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3895
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Task_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_235 with Inline_Always;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_235 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_235;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4777 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4776 '<Cast to BasicDecl at ast.py:3902>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3902
--# expr-start 4775 '<at at ast.py:3902>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3902
--# expr-start 4774 '<bind at ast.py:3900>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3900
--# expr-start 4771 '<FieldAccess for origin_node at ast.py:3901>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3901
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4771
Bound_Origin := Fld; 
--# expr-start 4773 '<FieldAccess for all_env_els_impl at ast.py:3901>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3901
--# expr-start 4772 '<FieldAccess for defining_name at ast.py:3901>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3901
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4772
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4773
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4774
Get_Result := Get (Self, Dyn_Var_Bind_Result, 0, True); 
--# expr-done 4775
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4776
Let_Result := Cast_Result; 
--# end
--# expr-done 4777
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_235;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_235;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Task_Previous_Part;
--# end
--# property-start Body.protected_previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3904
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Protected_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_236 with Inline_Always;
      Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Protected_Type_Decl;
Match_1 : Internal_Entity_Single_Protected_Decl;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Protected_Type_Decl;
Cast_Expr : Internal_Entity_Protected_Type_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Single_Protected_Decl;
Cast_Expr_1 : Internal_Entity_Single_Protected_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_236 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_236;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4782 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 4780 '<at at ast.py:3909>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3909
--# expr-start 4779 '<FieldAccess for env_elements at ast.py:3909>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3909
--# expr-start 4778 '<FieldAccess for defining_name at ast.py:3909>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3909
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4778
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld.Node, Env => Env, No_Visibility => False, E_Info => Fld.Info);
--# end
--# expr-done 4779
Get_Result := Get (Self, Fld_1, 0, True); 
--# expr-done 4780
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 4781 '<Match at ast.py:3909>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3909
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind prot_type Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Single_Protected_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind prot_decl Match_1
Cast_Expr_1 := Match_1; 
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Incomplete_Tagged_Type_Decl | Ada_Task_Type_Decl .. Ada_Package_Renaming_Decl | Ada_Single_Task_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4781
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 4782
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_236;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_236;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Protected_Previous_Part;
--# end
--# property-start Body.unbound_previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3915
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Unbound_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Base_Subp_Body;
Match_1 : Internal_Entity_Subp_Body_Stub;
Match_2 : Internal_Entity_Entry_Body;
Match_3 : Internal_Entity_Package_Body;
Match_4 : Internal_Entity_Package_Body_Stub;
Match_5 : Internal_Entity_Protected_Body;
Match_6 : Internal_Entity_Protected_Body_Stub;
Match_7 : Internal_Entity_Task_Body;
Match_8 : Internal_Entity_Task_Body_Stub;
Match_9 : Internal_Entity_Accept_Stmt_Body;
Pp_Next_Part : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Entry_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Entry_Decl;
Match_Prefix : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Subp_Body_Stub;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Entry_Body;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Package_Body;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Package_Body_Stub;
Let_Result_4 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity_Protected_Body;
Let_Result_5 : Internal_Entity_Basic_Decl;
Scope_Result_5 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Protected_Body_Stub;
Let_Result_6 : Internal_Entity_Basic_Decl;
Scope_Result_6 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Task_Body;
Let_Result_7 : Internal_Entity_Basic_Decl;
Scope_Result_7 : Internal_Entity_Basic_Decl;
Cast_Result_9 : Internal_Entity_Task_Body_Stub;
Let_Result_8 : Internal_Entity_Basic_Decl;
Scope_Result_8 : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity_Accept_Stmt_Body;
Cast_Result_11 : Internal_Entity_Basic_Decl;
Let_Result_9 : Internal_Entity_Basic_Decl;
Scope_Result_9 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_10 : Internal_Entity_Basic_Decl;
Scope_Result_10 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result_10 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_11 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4800 '<Block at ???>' Let_Result_11 None
--# scope-start
--# expr-start 4793 '<Match at ast.py:3922>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3922
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Body_Node (Match_Prefix.Node.Kind) is
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 4783 '<FieldAccess for subp_previous_part at ast.py:3923>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3923
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subp_previous_part
Fld := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4783
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 4784 '<FieldAccess for subp_previous_part at ast.py:3924>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3924
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subp_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4784
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 4785 '<FieldAccess for entry_previous_part at ast.py:3925>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3925
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.entry_previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Entry_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4785
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_3; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
--# expr-start 4786 '<FieldAccess for package_previous_part at ast.py:3926>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3926
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.package_previous_part
Fld_3 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4786
Let_Result_3 := Fld_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Package_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Package_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_5; 
--# bind _ Match_4
--# expr-start 4787 '<FieldAccess for package_previous_part at ast.py:3927>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3927
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.package_previous_part
Fld_4 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4787
Let_Result_4 := Fld_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Protected_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_6; 
--# bind _ Match_5
--# expr-start 4788 '<FieldAccess for protected_previous_part at ast.py:3928>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3928
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.protected_previous_part
Fld_5 := Libadalang.Implementation.Body_Node_P_Protected_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4788
Let_Result_5 := Fld_5; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Protected_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Protected_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_7; 
--# bind _ Match_6
--# expr-start 4789 '<FieldAccess for protected_previous_part at ast.py:3929>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3929
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.protected_previous_part
Fld_6 := Libadalang.Implementation.Body_Node_P_Protected_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4789
Let_Result_6 := Fld_6; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_8; 
--# bind _ Match_7
--# expr-start 4790 '<FieldAccess for task_previous_part at ast.py:3930>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3930
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.task_previous_part
Fld_7 := Libadalang.Implementation.Body_Node_P_Task_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4790
Let_Result_7 := Fld_7; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Task_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_9; 
--# bind _ Match_8
--# expr-start 4791 '<FieldAccess for task_previous_part at ast.py:3931>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3931
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.task_previous_part
Fld_8 := Libadalang.Implementation.Body_Node_P_Task_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4791
Let_Result_8 := Fld_8; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Accept_Stmt_Body =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Accept_Stmt_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_9 := Cast_Result_10; 
--# bind a Match_9
--# expr-start 4792 '<FieldAccess for accept_stmt_previous_part at ast.py:3932>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3932
      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmtBody.accept_stmt_previous_part
Fld_9 := Libadalang.Implementation.Accept_Stmt_Body_P_Accept_Stmt_Previous_Part (Node => Match_9.Node, E_Info => Match_9.Info);
--# end
--# expr-done 4792
      Cast_Result_11 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Let_Result_9 := Cast_Result_11; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_9; 
end case;
--# expr-done 4793
Block_Var := Match_Result; 
--# bind pp Block_Var
--# expr-start 4799 '<If at ast.py:3939>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3939
--# expr-start 4794 '<IsA BodyStub at ast.py:3940>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3940
Is_A_1 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Body_Stub; 
--# expr-done 4794
if Is_A_1 then
   If_Result_1 := Block_Var;
else
   --# expr-start 4798 '<Let at ast.py:3942>' Let_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3942
--# scope-start
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4795 '<FieldAccess for next_part_for_decl at ast.py:3942>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3942
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 4795
Scope_Result_10 := Fld_10;
   --# end
   Result_Var := Scope_Result_10;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Pp_Next_Part := Result_Var; 
--# bind pp_next_part Pp_Next_Part
--# expr-start 4797 '<If at ast.py:3942>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3942
--# expr-start 4796 '<IsA BodyStub at ast.py:3943>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3943
Is_A := Pp_Next_Part.Node /= null 
and then Pp_Next_Part.Node.Kind in Ada_Body_Stub; 
--# expr-done 4796
if Is_A then
   If_Result := Pp_Next_Part;
else
   If_Result := Block_Var;
end if;
--# expr-done 4797
Let_Result_10 := If_Result; 
--# end
--# expr-done 4798
   If_Result_1 := Let_Result_10;
end if;
--# expr-done 4799
Let_Result_11 := If_Result_1; 
--# end
--# expr-done 4800
         Property_Result := Let_Result_11;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Unbound_Previous_Part;
--# end
--# property-start Body.previous_part_internal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3949
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Internal
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_238 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity_Body_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Node;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_238 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_238;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4807 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4806 '<bind at ast.py:3965>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3965
--# expr-start 4801 '<FieldAccess for node_env at ast.py:3966>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3966
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 4801
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4805 '<as_entity at ast.py:3967>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3967
--# expr-start 4804 '<FieldAccess for node at ast.py:3967>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3967
--# expr-start 4803 '<FieldAccess for unbound_previous_part at ast.py:3967>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3967
--# expr-start 4802 '<as_bare_entity at ast.py:3967>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3967
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4802
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.unbound_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Unbound_Previous_Part (Node => If_Result.Node, Env => Bound_Env, Imprecise_Fallback => Imprecise_Fallback, E_Info => If_Result.Info);
--# end
--# expr-done 4803
Fld_2 := Fld_1.Node;
--# expr-done 4804
Is_Null_1 := Fld_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Fld_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 4805
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 4806
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4807
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_238;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_238;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Previous_Part_Internal;
--# end
--# property-start Body.previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3970
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Body_Node_Body_Node_P_Previous_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Body_Node,
                                As_Bare_Body_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4809 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4808 '<FieldAccess for previous_part_internal at ast.py:3978>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3978
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part_internal
Fld := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4808
Let_Result := Fld; 
--# end
--# expr-done 4809
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Body_Node_P_Previous_Part;
--# end
--# property-start Body.decl_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3980
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Decl_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Body_Stub;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Stub;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4813 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 4810 '<FieldAccess for previous_part_internal at ast.py:3985>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3985
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part_internal
Fld := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4810
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind prev_part Var_Expr
--# expr-start 4812 '<Match at ast.py:3986>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3986
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind stub Match_0
--# expr-start 4811 '<FieldAccess for previous_part_internal at ast.py:3989>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3989
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part_internal
Fld_1 := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 4811
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind other Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4812
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result_2 := Result_Var; 
--# end
--# expr-done 4813
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Decl_Part;
--# end
--# property-start Body.safe_generic_decl_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3994
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Safe_Generic_Decl_Part
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_241 with Inline_Always;
            procedure Finalizer_Scope_2552 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Array_Access;
Var_Expr_2 : Internal_Entity_Generic_Subp_Decl;
Bound_Imprecise_Fallback : Boolean;
Is_A : Boolean;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Decl;
Scope_Result : Internal_Entity_Generic_Package_Decl;
Result_Var : Internal_Entity_Generic_Package_Decl;
Is_A_1 : Boolean;
Fld_2 : Lexical_Env := Empty_Env;
Env_Parent : Lexical_Env := Empty_Env;
Fld_3 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Generic_Subp_Decl;
Scope_Result_1 : Internal_Entity_Generic_Subp_Decl;
Get_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Generic_Subp_Decl;
Result_Var_1 : Internal_Entity_Generic_Subp_Decl;
Scope_Result_2 : Internal_Entity_Generic_Subp_Decl;
Result_Var_2 : Internal_Entity_Generic_Subp_Decl;
Cast_Result_3 : Internal_Entity_Generic_Decl;
If_Result : Internal_Entity_Generic_Decl;
Cast_Result_4 : Internal_Entity_Generic_Decl;
If_Result_1 : Internal_Entity_Generic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Generic_Decl;
Let_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_241 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_241;
            procedure Finalizer_Scope_2552 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_2552;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4829 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4828 '<bind at ast.py:4000>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4000
Bound_Imprecise_Fallback := False; 
--# expr-start 4827 '<Cond at ast.py:4002>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4002
--# expr-start 4814 '<IsA PackageBody at ast.py:4003>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4003
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Body_Range; 
--# expr-done 4814
if Is_A then
--# expr-start 4815 '<FieldAccess for decl_part at ast.py:4004>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4004
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4815
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 4817 '<Cast to GenericPackageDecl at ast.py:4005>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4005
--# expr-start 4816 '<parent at ast.py:4005>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4005
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 4816
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Package_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Generic_Package_Decl;
   end if;
--# expr-done 4817
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Package_Decl;
end if;
      Cast_Result_4 := Create_Internal_Entity_Generic_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_1 := Cast_Result_4;
else
--# expr-start 4818 '<IsA BaseSubpBody, SubpBodyStub at ast.py:4008>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4008
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Base_Subp_Body | Ada_Subp_Body_Stub_Range; 
--# expr-done 4818
if Is_A_1 then
--# expr-start 4822 '<EnvGet at ast.py:4014>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4014
--# expr-start 4820 '<env_parent at ast.py:4014>' Env_Parent /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4014
--# expr-start 4819 '<FieldAccess for children_env at ast.py:4014>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4014
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4819
Env_Parent := AST_Envs.Parent (Fld_2); 
--# expr-done 4820
--# expr-start 4821 '<FieldAccess for name_symbol at ast.py:4015>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4015
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4821
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env_Parent, Key => Fld_3, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 4822
Var_Expr_1 := Env_Get_Result;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind results Var_Expr_1
--# expr-start 4824 '<Cast to GenericSubpDecl at ast.py:4018>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4018
--# expr-start 4823 '<at at ast.py:4018>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4018
Get_Result := Get (Self, Var_Expr_1, 1, True); 
--# expr-done 4823
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Subp_Decl;
   end if;
--# expr-done 4824
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Generic_Subp_Decl then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_1 := Var_Expr_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 4826 '<Cast to GenericSubpDecl at ast.py:4019>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4019
--# expr-start 4825 '<at at ast.py:4019>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4019
Get_Result_1 := Get (Self, Var_Expr_1, 2, True); 
--# expr-done 4825
   if Get_Result_1.Node = null
      or else Get_Result_1.Node.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Get_Result_1.Node,
         Info => Get_Result_1.Info);
   else
         Cast_Result_2 := No_Entity_Generic_Subp_Decl;
   end if;
--# expr-done 4826
   Result_Var_1 := Cast_Result_2;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
      Finalizer_Scope_2552;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Generic_Subp_Decl;
end if;
      Cast_Result_3 := Create_Internal_Entity_Generic_Decl
        (Node => Result_Var_2.Node,
         Info => Result_Var_2.Info);
   If_Result := Cast_Result_3;
else
   If_Result := No_Entity_Generic_Decl;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 4827
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 4828
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4829
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_241;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_241;
                     Finalizer_Scope_2552;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Safe_Generic_Decl_Part;
--# end
--# property-start Body.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4027
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Next_Part_For_Decl
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4830 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Entity_Basic_Decl; 
--# end
--# expr-done 4830
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Next_Part_For_Decl;
--# end
--# property-start Body.is_subunit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4035
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Is_Subunit
  (Node : Bare_Body_Node
  )
   return Boolean
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4833 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4832 '<IsA Subunit at ast.py:4037>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4037
--# expr-start 4831 '<parent at ast.py:4037>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4037
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 4831
Is_A := Fld /= null 
and then Fld.Kind in Ada_Subunit_Range; 
--# expr-done 4832
Let_Result := Is_A; 
--# end
--# expr-done 4833
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Is_Subunit;
--# end
--# property-start Body.subunit_root /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4039
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Root
  (Node : Bare_Body_Node
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Bare_Subunit;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4837 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4835 '<Cast to Subunit at ast.py:4044>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4044
--# expr-start 4834 '<parent at ast.py:4044>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4044
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 4834
   if Fld = null
      or else Fld.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4835
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind su Var_Expr
--# expr-start 4836 '<FieldAccess for body_root at ast.py:4044>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4044
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.body_root
Fld_1 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Var_Expr);
--# end
--# expr-done 4836
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4837
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Subunit_Root;
--# end
--# property-start Body.body_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4046
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Scope
  (Node : Bare_Body_Node
      ; Follow_Private : Boolean
      ; Force_Decl : Boolean
         := False
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind follow_private Follow_Private
   --# bind force_decl Force_Decl
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_245 with Inline_Always;
            procedure Finalizer_Scope_2505 with Inline_Always;
            procedure Finalizer_Scope_2506 with Inline_Always;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Lexical_Env := Empty_Env;
Dns : Lexical_Env := Empty_Env;
Var_Expr : Bare_Generic_Package_Decl;
Var_Expr_1 : Bare_Ada_Node;
Var_Expr_2 : Internal_Entity;
Fld : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Boolean;
Fld_4 : Boolean;
If_Result : Boolean;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Fld_9 : Boolean;
If_Result_3 : Boolean;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Bare_Ada_Node;
Fld_13 : Lexical_Env := Empty_Env;
If_Result_4 : Lexical_Env := Empty_Env;
If_Result_5 : Lexical_Env := Empty_Env;
If_Result_6 : Lexical_Env := Empty_Env;
Env_Node_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Generic_Package_Decl;
Fld_14 : Bare_Generic_Package_Internal;
Fld_15 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_16 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Env_Node_2 : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_245 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Dns);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_245;
            procedure Finalizer_Scope_2505 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_2505;
            procedure Finalizer_Scope_2506 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_2506;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4871 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 4859 '<Cond at ast.py:4056>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4056
--# expr-start 4840 '<Not at ast.py:4061>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4061
--# expr-start 4839 '<IsNull at ast.py:4061>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4061
--# expr-start 4838 '<FieldAccess for subunit_root at ast.py:4061>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4061
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subunit_root
Fld := Libadalang.Implementation.Body_Node_P_Subunit_Root (Node => Self);
--# end
--# expr-done 4838
Is_Null := Fld.Node = null; 
--# expr-done 4839
Not_Val := not (Is_Null); 
--# expr-done 4840
if Not_Val then
   --# expr-start 4842 '<FieldAccess for children_env at ast.py:4061>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4061
--# expr-start 4841 '<FieldAccess for subunit_root at ast.py:4061>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4061
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subunit_root
Fld_1 := Libadalang.Implementation.Body_Node_P_Subunit_Root (Node => Self);
--# end
--# expr-done 4841
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 4842
   If_Result_6 := Fld_2;
else
--# expr-start 4843 '<FieldAccess for is_subprogram at ast.py:4066>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4066
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 4843
if Fld_3 then
   --# expr-start 4844 '<FieldAccess for is_compilation_unit_root at ast.py:4066>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4066
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 4844
   If_Result := Fld_4;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4853 '<Let at ast.py:4067>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4067
--# scope-start
--# expr-start 4846 '<FieldAccess for scope at ast.py:4067>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4067
--# expr-start 4845 '<FieldAccess for defining_name at ast.py:4067>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4067
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4845
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld_5.Node, Env => Env);
--# end
--# expr-done 4846
Dns := Fld_6; Inc_Ref (Dns);
--# bind dns Dns
--# expr-start 4852 '<If at ast.py:4070>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4070
--# expr-start 4847 '<IsNull at ast.py:4070>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4070
Is_Equal := Equivalent (Dns, Empty_Env); 
--# expr-done 4847
if Is_Equal then
   If_Result_1 := True;
else
   --# expr-start 4849 '<Eq at ast.py:4070>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4070
--# expr-start 4848 '<env_node at ast.py:4070>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4070
Env_Node := AST_Envs.Env_Node (Dns); 
--# expr-done 4848
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Env_Node = Cast_Result; 
--# expr-done 4849
   If_Result_1 := Is_Equal_1;
end if;
if If_Result_1 then
   --# expr-start 4851 '<FieldAccess for parent_scope at ast.py:4071>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4071
--# expr-start 4850 '<FieldAccess for defining_name at ast.py:4071>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4071
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4850
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_7.Node, Env => Env);
--# end
--# expr-done 4851
   If_Result_2 := Fld_8;
else
   If_Result_2 := Dns;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4852
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4853
   If_Result_5 := Let_Result;
else
--# expr-start 4854 '<FieldAccess for is_compilation_unit_root at ast.py:4076>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4076
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 4854
if Fld_9 then
   If_Result_3 := True;
else
   If_Result_3 := Force_Decl;
end if;
if If_Result_3 then
   --# expr-start 4856 '<FieldAccess for scope at ast.py:4077>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4077
--# expr-start 4855 '<FieldAccess for defining_name at ast.py:4077>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4077
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4855
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld_10.Node, Env => Env);
--# end
--# expr-done 4856
   If_Result_4 := Fld_11;
else
   --# expr-start 4858 '<FieldAccess for children_env at ast.py:4081>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4081
--# expr-start 4857 '<parent at ast.py:4081>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4081
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Parent; 
--# expr-done 4857
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Fld_12);
--# expr-done 4858
   If_Result_4 := Fld_13;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 4859
Block_Var := If_Result_6; Inc_Ref (Block_Var);
--# bind scope Block_Var
--# expr-start 4861 '<Cast to GenericPackageDecl at ast.py:4086>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4086
--# expr-start 4860 '<env_node at ast.py:4086>' Env_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4086
Env_Node_1 := AST_Envs.Env_Node (Block_Var); 
--# expr-done 4860
   if Env_Node_1 = null
      or else Env_Node_1.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result_1 := Env_Node_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 4861
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr
--# expr-start 4863 '<FieldAccess for children_env at ast.py:4087>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4087
--# expr-start 4862 '<FieldAccess for package_decl at ast.py:4087>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4087
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Var_Expr.Generic_Package_Decl_F_Package_Decl;
--# expr-done 4862
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Children_Env (Node => Fld_14);
--# expr-done 4863
Scope_Result := Fld_15;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2505;
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var;
end if;
      Inc_Ref (Result_Var);
Block_Var_1 := Result_Var; Inc_Ref (Block_Var_1);
--# bind public_scope Block_Var_1
--# expr-start 4870 '<If at ast.py:4093>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4093
if Follow_Private then
--# expr-start 4864 '<env_node at ast.py:4096>' Env_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4096
Env_Node_2 := AST_Envs.Env_Node (Block_Var_1); 
--# expr-done 4864
Var_Expr_1 := Env_Node_2;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4865 '<IsA BasePackageDecl, SingleProtectedDecl, ProtectedTypeDecl at ast.py:4096>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4096
Is_A := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Base_Package_Decl | Ada_Single_Protected_Decl_Range | Ada_Protected_Type_Decl_Range; 
--# expr-done 4865
Scope_Result_2 := Is_A;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_7 := Result_Var_2;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
--# expr-start 4868 '<at at ast.py:4103>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4103
--# expr-start 4867 '<EnvGet at ast.py:4101>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4101
--# expr-start 4866 '<Symbol __privatepart at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Privatepart); 
--# expr-done 4866
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_1, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 4867
Get_Result := Get (Self, Env_Get_Result, 0, True); 
--# expr-done 4868
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_2
--# expr-start 4869 '<FieldAccess for children_env at ast.py:4104>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4104
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 4869
Scope_Result_1 := Fld_16;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2506;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var_1;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_8 := Result_Var_1;
else
   If_Result_8 := Block_Var_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 4870
Let_Result_1 := If_Result_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 4871
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_245;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_245;
                     Finalizer_Scope_2505;
                     Finalizer_Scope_2506;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Body_Scope;
--# end
--# property-start Body.previous_part_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3701
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_230 with Inline_Always;
      Fld : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Fld_1 : String_Type;
Sym_1 : Symbol_Type;
Fld_2 : Boolean;
If_Result : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_230 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_230;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4880 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4879 '<If at ast.py:3709>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3709
--# expr-start 4872 '<FieldAccess for is_subunit at ast.py:3710>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3710
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld_2 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 4872
if Fld_2 then
   --# expr-start 4876 '<to_symbol at ast.py:3711>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3711
--# expr-start 4875 '<Concat at ast.py:3711>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3711
--# expr-start 4873 '<FieldAccess for top_level_env_name at ast.py:3711>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3711
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4873
--# expr-start 4874 '<String at ast.py:3711>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3711
Str := Create_String ("__stub"); 
--# expr-done 4874
Concat_Result := Concat_String (Fld, Str); 
--# expr-done 4875
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 4876
   If_Result := Sym;
else
   --# expr-start 4878 '<to_symbol at ast.py:3712>' Sym_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3712
--# expr-start 4877 '<FieldAccess for top_level_env_name at ast.py:3712>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3712
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4877
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Fld_1); 
--# expr-done 4878
   If_Result := Sym_1;
end if;
--# expr-done 4879
Let_Result := If_Result; 
--# end
--# expr-done 4880
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_230;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_230;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Previous_Part_Env_Name;
--# end
--# property-start Body.previous_part_link_env_assoc /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3715
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Link_Env_Assoc
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_231 with Inline_Always;
            procedure Finalizer_Scope_2515 with Inline_Always;
            procedure Finalizer_Scope_2516 with Inline_Always;
      Var_Expr : Symbol_Type;
Var_Expr_1 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Scope_Result_1 : Internal_Designated_Env := No_Designated_Env;
New_Struct_2 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Result_Var_1 : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
New_Struct_3 : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_231 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (New_Struct_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_231;
            procedure Finalizer_Scope_2515 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2515;
            procedure Finalizer_Scope_2516 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2516;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4890 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4889 '<New EnvAssoc at ast.py:3726>' New_Struct_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3726
--# expr-start 4881 '<FieldAccess for previous_part_env_name at ast.py:3729>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3729
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_env_name'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Env_Name (Node => Self);
--# end
--# expr-done 4881
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 4882 '<New DesignatedEnv at ast.py:3730>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3730
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr, Kind => Named_Env); 
--# expr-done 4882
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2515;
   Result_Var_1 := Scope_Result;
else
--# expr-start 4885 '<bind at ast.py:3732>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3732
--# expr-start 4883 '<FieldAccess for default_initial_env at ast.py:3733>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3733
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4883
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 4884 '<FieldAccess for body_scope at ast.py:3734>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3734
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_2 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 4884
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4885
Var_Expr_1 := Dyn_Var_Bind_Result;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= Empty_Env then
   --# scope-start
   --# bind non_null_env Var_Expr_1
--# expr-start 4886 '<New DesignatedEnv at ???>' New_Struct_1 None
Inc_Ref (Var_Expr_1);
New_Struct_1 := (Direct_Env => Var_Expr_1, Env_Name => null, Kind => Direct_Env); 
--# expr-done 4886
Scope_Result_1 := New_Struct_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2516;
   Result_Var := Scope_Result_1;
else
   --# expr-start 4887 '<New DesignatedEnv at ast.py:3731>' New_Struct_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3731
Inc_Ref (Empty_Env);
New_Struct_2 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 4887
   Result_Var := New_Struct_2;
end if;
      Inc_Ref (Result_Var);
   Result_Var_1 := Result_Var;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 4888 '<Symbol __nextpart at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 4888
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Result_Var_1);
New_Struct_3 := (Dest_Env => Result_Var_1, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4889
Let_Result := New_Struct_3; Inc_Ref (Let_Result);
--# end
--# expr-done 4890
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_231;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_231;
                     Finalizer_Scope_2515;
                     Finalizer_Scope_2516;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Previous_Part_Link_Env_Assoc;
--# end
function Body_Node_P_Subunit_Decl_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Body_Node
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Body_Node_P_Subunit_Decl_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
function Body_Node_P_Body_Decl_Scope
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Body_Node
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Body_Node_P_Body_Decl_Scope (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Accept_Stmt_Body
   --
      procedure Initialize_Fields_For_Accept_Stmt_Body
        (Self : Bare_Accept_Stmt_Body
         ; Accept_Stmt_Body_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Accept_Stmt_Body_F_Name := Accept_Stmt_Body_F_Name;
      end Initialize_Fields_For_Accept_Stmt_Body;
   function Accept_Stmt_Body_F_Name
     (Node : Bare_Accept_Stmt_Body) return Bare_Defining_Name
   is
   begin
         return Node.Accept_Stmt_Body_F_Name;
   end;
--# property-start AcceptStmtBody.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21530
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_Body_P_Defining_Names
  (Node : Bare_Accept_Stmt_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Accept_Stmt_Body  := Bare_Accept_Stmt_Body (Node);
     Ent : Internal_Entity_Accept_Stmt_Body :=
       Internal_Entity_Accept_Stmt_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1469 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1469 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1469;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4891 '<FieldAccess for name at ast.py:21531>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21531
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Accept_Stmt_Body_F_Name, Info => Ent.Info);
--# expr-done 4891
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1469;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1469;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_Body_P_Defining_Names;
--# end
--# property-start AcceptStmtBody.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21533
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_Body_P_Defining_Env
  (Node : Bare_Accept_Stmt_Body
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Accept_Stmt_Body  := Bare_Accept_Stmt_Body (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1470 with Inline_Always;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Accept_Stmt;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1470 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1470;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4894 '<FieldAccess for children_env at ast.py:21533>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21533
--# expr-start 4893 '<Cast to AcceptStmt at ast.py:21533>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21533
--# expr-start 4892 '<parent at ast.py:21533>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21533
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 4892
   if Fld = null
      or else Fld.Kind in Ada_Accept_Stmt_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4893
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Cast_Result);
--# expr-done 4894
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1470;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1470;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_Body_P_Defining_Env;
--# end
--# property-start AcceptStmtBody.accept_stmt_previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21535
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_Body_P_Accept_Stmt_Previous_Part
  (Node : Bare_Accept_Stmt_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Accept_Stmt_Body  := Bare_Accept_Stmt_Body (Node);
     Ent : Internal_Entity_Accept_Stmt_Body :=
       Internal_Entity_Accept_Stmt_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Entry_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt;
Fld_1 : Internal_Entity_Entry_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4897 '<FieldAccess for corresponding_entry at ast.py:21536>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21536
--# expr-start 4896 '<Cast to AcceptStmt at ast.py:21536>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21536
--# expr-start 4895 '<parent at ast.py:21536>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21536
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4895
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Accept_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Accept_Stmt
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4896
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.corresponding_entry
Fld_1 := Libadalang.Implementation.Accept_Stmt_P_Corresponding_Entry (Node => Cast_Result.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 4897
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Accept_Stmt_Body_P_Accept_Stmt_Previous_Part;
--# end
   --
   --  Primitives for Bare_Base_Subp_Body
   --
      procedure Base_Subp_Body_Pre_Env_Actions
        (Self            : Bare_Base_Subp_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_127 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_128 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:20636");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_129 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:20643");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_130 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_131 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:20651");
         Dec_Ref (Mapping.Dest_Env);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_132 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_134 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_133 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_136 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_135 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_138 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_137 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_140 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_139 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_142 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_141 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Base_Subp_Body
        (Self : Bare_Base_Subp_Body
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Self.Base_Subp_Body_F_Overriding := Base_Subp_Body_F_Overriding;
            Self.Base_Subp_Body_F_Subp_Spec := Base_Subp_Body_F_Subp_Spec;
      end Initialize_Fields_For_Base_Subp_Body;
   function Base_Subp_Body_F_Overriding
     (Node : Bare_Base_Subp_Body) return Bare_Overriding_Node
   is
   begin
         return Node.Base_Subp_Body_F_Overriding;
   end;
   function Base_Subp_Body_F_Subp_Spec
     (Node : Bare_Base_Subp_Body) return Bare_Subp_Spec
   is
   begin
         return Node.Base_Subp_Body_F_Subp_Spec;
   end;
--# property-start BaseSubpBody.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20555
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Defining_Names
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1416 with Inline_Always;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1416 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1416;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4899 '<FieldAccess for name at ast.py:20555>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20555
--# expr-start 4898 '<FieldAccess for subp_spec at ast.py:20555>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20555
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 4898
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4899
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_1;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1416;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1416;
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Defining_Names;
--# end
--# property-start BaseSubpBody.constrain_prefix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20557
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Constrain_Prefix
  (Node : Bare_Base_Subp_Body
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1417 with Inline_Always;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1417 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1417;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4901 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4900 '<FieldAccess for subp_constrain_prefix at ast.py:20559>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20559
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_constrain_prefix
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Constrain_Prefix (Node => Ent.Node, Prefix => Prefix, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4900
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4901
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1417;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1417;
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Constrain_Prefix;
--# end
--# property-start BaseSubpBody.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20561
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Defining_Env
  (Node : Bare_Base_Subp_Body
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1418 with Inline_Always;
            procedure Finalizer_Scope_2555 with Inline_Always;
            procedure Finalizer_Scope_2557 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Generic_Formal_Part;
Var_Expr_3 : Internal_Entity_Generic_Subp_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Boolean;
Fld_8 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Lexical_Env := Empty_Env;
Fld_9 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Subp_Decl;
Fld_10 : Internal_Entity_Generic_Formal_Part;
Scope_Result_1 : Internal_Entity_Generic_Formal_Part;
Result_Var_1 : Internal_Entity_Generic_Formal_Part;
Fld_11 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var_3 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Fld_12 : Internal_Entity_Subp_Spec;
Fld_13 : Lexical_Env := Empty_Env;
Fld_14 : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1418 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1418;
            procedure Finalizer_Scope_2555 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_2555;
            procedure Finalizer_Scope_2557 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2557;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4924 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4923 '<If at ast.py:20563>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20563
--# expr-start 4902 '<FieldAccess for in_scope at ast.py:20564>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20564
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.in_scope
Fld_14 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 4902
if Fld_14 then
   --# expr-start 4920 '<env_group at ast.py:20597>' Group_Env_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20597
--# expr-start 4919 '<ArrayLiteral at ast.py:20566>' Array_Lit_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20566
--# expr-start 4914 '<If at ast.py:20567>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20567
--# expr-start 4903 '<FieldAccess for subp_spec_or_null at ast.py:20568>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20568
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4903
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 4907 '<FieldAccess for paramless at ast.py:20569>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20569
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4906 '<FieldAccess for dottable_subp at ast.py:20569>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20569
--# expr-start 4905 '<FieldAccess for md at ast.py:20569>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20569
--# expr-start 4904 '<FieldAccess for info at ast.py:20569>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20569
Fld_5 := Ent.Info;
--# expr-done 4904
Fld_6 := Fld_5.Md;
--# expr-done 4905
Fld_7 := Fld_6.Dottable_Subp;
--# expr-done 4906
--# property-call-start BaseFormalParamHolder.paramless
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_7, Can_Be => True);
--# end
--# expr-done 4907
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 4912 '<env_group at ast.py:20572>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20572
--# expr-start 4911 '<ArrayLiteral at ast.py:20570>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20570
--# expr-start 4908 '<FieldAccess for children_env at ast.py:20571>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20571
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4908
--# expr-start 4910 '<FieldAccess for defining_env at ast.py:20571>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20571
--# expr-start 4909 '<FieldAccess for subp_spec at ast.py:20571>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20571
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 4909
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4910
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_2)); 
--# expr-done 4911
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 4912
   If_Result := Group_Env;
else
   --# expr-start 4913 '<FieldAccess for children_env at ast.py:20573>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20573
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4913
   If_Result := Fld_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 4914
--# expr-start 4915 '<FieldAccess for decl_part at ast.py:20593>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20593
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_9 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4915
Var_Expr_1 := Fld_9;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr_1
--# expr-start 4916 '<Cast to GenericSubpDecl at ast.py:20594>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20594
Cast_Expr := Var_Expr_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Subp_Decl;
   end if;
--# expr-done 4916
Var_Expr_3 := Cast_Result;
if Var_Expr_3 /= No_Entity_Generic_Subp_Decl then
   --# scope-start
--# expr-start 4917 '<FieldAccess for formal_part at ast.py:20595>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20595
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_3.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_3.Info);
--# expr-done 4917
Scope_Result_1 := Fld_10;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Generic_Formal_Part;
end if;
Var_Expr_2 := Result_Var_1;
if Var_Expr_2 /= No_Entity_Generic_Formal_Part then
   --# scope-start
--# expr-start 4918 '<FieldAccess for children_env at ast.py:20595>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20595
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 4918
Scope_Result_2 := Fld_11;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2557;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
Scope_Result_3 := Result_Var_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2555;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Empty_Env;
end if;
      Inc_Ref (Result_Var_3);
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => If_Result, 2 => Result_Var_3)); 
--# expr-done 4919
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 4920
   If_Result_1 := Group_Env_1;
else
   --# expr-start 4922 '<FieldAccess for defining_env at ast.py:20599>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20599
--# expr-start 4921 '<FieldAccess for subp_spec at ast.py:20599>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20599
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 4921
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_13 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld_12.Node, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 4922
   If_Result_1 := Fld_13;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4923
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4924
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1418;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1418;
                     Finalizer_Scope_2555;
                     Finalizer_Scope_2557;
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Defining_Env;
--# end
--# property-start BaseSubpBody.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20602
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Type_Expression
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4926 '<FieldAccess for returns at ast.py:20602>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20602
--# expr-start 4925 '<FieldAccess for subp_spec at ast.py:20602>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20602
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 4925
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4926
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Type_Expression;
--# end
--# property-start BaseSubpBody.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20604
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Expr_Type
  (Node : Bare_Base_Subp_Body
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4929 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4927 '<FieldAccess for subp_spec_or_null at ast.py:20606>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20606
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4927
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 4928 '<FieldAccess for return_type at ast.py:20606>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20606
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4928
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4929
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Expr_Type;
--# end
--# property-start BaseSubpBody.initial_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20608
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Initial_Env_Name
  (Node : Bare_Base_Subp_Body
      ; Follow_Private : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   --# bind follow_private Follow_Private
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Fld_2 : Boolean;
If_Result : Symbol_Type;
Let_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4934 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4933 '<If at ast.py:20610>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20610
--# expr-start 4930 '<FieldAccess for is_library_item at ast.py:20611>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20611
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 4930
if Fld_2 then
   --# expr-start 4931 '<FieldAccess for child_decl_initial_env_name at ast.py:20612>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20612
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => Follow_Private);
--# end
--# expr-done 4931
   If_Result := Fld;
else
   --# expr-start 4932 '<FieldAccess for body_initial_env_name at ast.py:20613>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20613
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env_name
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4932
   If_Result := Fld_1;
end if;
--# expr-done 4933
Let_Result := If_Result; 
--# end
--# expr-done 4934
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Initial_Env_Name;
--# end
--# property-start BaseSubpBody.previous_part_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20616
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Previous_Part_Env_Name
  (Node : Bare_Base_Subp_Body
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_1422 with Inline_Always;
      Fld : Boolean;
Fld_1 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Fld_2 : Boolean;
Fld_3 : String_Type;
Sym_1 : Symbol_Type;
If_Result : Symbol_Type;
If_Result_1 : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_1422 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1422;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4944 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4943 '<Cond at ast.py:20623>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20623
--# expr-start 4935 '<FieldAccess for is_subunit at ast.py:20624>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20624
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 4935
if Fld then
   --# expr-start 4939 '<to_symbol at ast.py:20625>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20625
--# expr-start 4938 '<Concat at ast.py:20625>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20625
--# expr-start 4936 '<FieldAccess for top_level_env_name at ast.py:20625>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20625
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4936
--# expr-start 4937 '<String at ast.py:20625>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20625
Str := Create_String ("__stub"); 
--# expr-done 4937
Concat_Result := Concat_String (Fld_1, Str); 
--# expr-done 4938
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 4939
   If_Result_1 := Sym;
else
--# expr-start 4940 '<FieldAccess for is_library_item at ast.py:20627>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20627
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 4940
if Fld_2 then
   --# expr-start 4942 '<to_symbol at ast.py:20628>' Sym_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20628
--# expr-start 4941 '<FieldAccess for top_level_env_name at ast.py:20628>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20628
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4941
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Fld_3); 
--# expr-done 4942
   If_Result := Sym_1;
else
   If_Result := null;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 4943
Let_Result := If_Result_1; 
--# end
--# expr-done 4944
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1422;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1422;
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Previous_Part_Env_Name;
--# end
--# property-start BaseSubpBody.gnatprove_annotations /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20719
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Gnatprove_Annotations
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1423 with Inline_Always;
            procedure Finalizer_Scope_2567 with Inline_Always;
            procedure Finalizer_Scope_2575 with Inline_Always;
            procedure Finalizer_Scope_2577 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Internal_Entity_Expr_Array_Access;
Block_Var_2 : Internal_Entity_Basic_Decl;
Block_Var_3 : Bare_Pragma_Node_Array_Access;
Block_Var_4 : Internal_Entity_Expr_Array_Access;
Block_Var_5 : Internal_Entity_Expr_Array_Access;
Var_Expr : Internal_Entity_Aspect_Spec;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Name;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Bare_Pragma_Node_Array_Access;
Var_Expr_5 : Bare_Declarative_Part;
Var_Expr_6 : Internal_Entity_Basic_Decl;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Var_Expr_8 : Bare_Pragma_Node;
Var_Expr_9 : Internal_Entity_Name;
Var_Expr_10 : Bare_Base_Assoc;
Var_Expr_11 : Internal_Entity_Name;
Var_Expr_12 : Bare_Base_Assoc;
Var_Expr_13 : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Aspect_Spec;
Scope_Result : Internal_Entity_Aspect_Spec;
Fld_2 : Internal_Entity_Aspect_Spec;
Result_Var : Internal_Entity_Aspect_Spec;
Fld_3 : Internal_Entity_Aspect_Assoc_List;
Scope_Result_1 : Internal_Entity_Aspect_Assoc_List;
Result_Var_1 : Internal_Entity_Aspect_Assoc_List;
Fld_4 : Bare_Aspect_Assoc_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity_Aspect_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Aspect_Assoc;
Fld_6 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Bare_Assoc_List;
Fld_9 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Item_175 : Internal_Entity_Aspect_Assoc;
Bare_Item_175 : Bare_Aspect_Assoc;
Fld_11 : Internal_Entity_Name;
Sym : Symbol_Type;
Fld_12 : Boolean;
Fld_13 : Internal_Entity_Expr;
Is_A : Boolean;
If_Result_2 : Boolean;
Fld_14 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_15 : Internal_Entity_Assoc_List;
Fld_16 : Bare_Assoc_List;
Fld_17 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity_Basic_Assoc;
Fld_18 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_19 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_4 : Boolean;
Map_Result : Internal_Entity_Expr_Array_Access;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Base_Subp_Body;
Cast_Result_3 : Internal_Entity_Expr_Function;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Result_Var_3 : Internal_Entity_Basic_Decl;
Fld_20 : Bare_Declarative_Part;
Scope_Result_4 : Bare_Declarative_Part;
Result_Var_4 : Bare_Declarative_Part;
Fld_21 : Bare_Ada_Node_List;
Scope_Result_5 : Bare_Ada_Node_List;
Result_Var_5 : Bare_Ada_Node_List;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_5 : Bare_Pragma_Node;
Item_176 : Bare_Ada_Node;
Is_A_1 : Boolean;
Map_Result_1 : Bare_Pragma_Node_Array_Access;
Scope_Result_6 : Bare_Pragma_Node_Array_Access;
Fld_22 : Internal_Entity_Pragma_Node_List;
Scope_Result_7 : Internal_Entity_Pragma_Node_List;
Result_Var_6 : Internal_Entity_Pragma_Node_List;
Fld_23 : Bare_Pragma_Node_List;
Fld_24 : Internal_Entity_Info;
As_Entity_3 : Internal_Entity_Pragma_Node;
Is_Null_3 : Boolean;
If_Result_5 : Internal_Entity_Pragma_Node;
Fld_25 : Bare_Pragma_Node;
Item_177 : Internal_Entity_Pragma_Node;
Bare_Item_177 : Bare_Pragma_Node;
Map_Result_2 : Bare_Pragma_Node_Array_Access;
Result_Var_7 : Bare_Pragma_Node_Array_Access;
Fld_26 : Bare_Base_Assoc_List;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_4 : Internal_Entity_Base_Assoc;
Is_Null_4 : Boolean;
If_Result_6 : Internal_Entity_Base_Assoc;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Bare_Basic_Decl;
Node_Comp : Boolean;
Fld_29 : Bare_Identifier;
Sym_2 : Symbol_Type;
Fld_30 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_8 : Boolean;
If_Result_7 : Boolean;
Fld_31 : Bare_Base_Assoc_List;
Get_Result_3 : Bare_Base_Assoc;
As_Entity_5 : Internal_Entity_Base_Assoc;
Is_Null_5 : Boolean;
If_Result_8 : Internal_Entity_Base_Assoc;
Fld_32 : Internal_Entity_Expr;
Scope_Result_9 : Internal_Entity_Expr;
Result_Var_9 : Internal_Entity_Expr;
Cast_Result_6 : Internal_Entity_Name;
Sym_3 : Symbol_Type;
Fld_33 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_10 : Boolean;
If_Result_9 : Boolean;
Fld_34 : Bare_Base_Assoc_List;
Get_Result_4 : Bare_Base_Assoc;
As_Entity_6 : Internal_Entity_Base_Assoc;
Is_Null_6 : Boolean;
If_Result_10 : Internal_Entity_Base_Assoc;
Fld_35 : Internal_Entity_Expr;
Scope_Result_11 : Internal_Entity_Expr;
Result_Var_11 : Internal_Entity_Expr;
Cast_Result_7 : Internal_Entity_Name;
Fld_36 : Internal_Entity_Defining_Name_Array_Access;
Get_Result_5 : Internal_Entity_Defining_Name;
Fld_37 : Symbol_Type;
Fld_38 : Boolean;
Scope_Result_12 : Boolean;
Result_Var_12 : Boolean;
If_Result_11 : Boolean;
Map_Result_3 : Internal_Entity_Expr_Array_Access;
Fld_39 : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Find_Result : Internal_Entity;
Cast_Result_8 : Internal_Entity_Base_Subp_Body;
Fld_40 : Internal_Entity_Expr_Array_Access;
Scope_Result_13 : Internal_Entity_Expr_Array_Access;
Result_Var_13 : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
Concat_Result_1 : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1423 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Fld_39);
                     Dec_Ref (Scope_Result_13);
                     Dec_Ref (Result_Var_13);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1423;
            procedure Finalizer_Scope_2567 is
            begin
                     Dec_Ref (Var_Expr_4);
            end Finalizer_Scope_2567;
            procedure Finalizer_Scope_2575 is
            begin
                     Dec_Ref (Fld_36);
            end Finalizer_Scope_2575;
            procedure Finalizer_Scope_2577 is
            begin
                     Dec_Ref (Fld_40);
            end Finalizer_Scope_2577;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Body_Base_Subp_Body_P_Gnatprove_Annotations,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Body,
                                As_Bare_Base_Subp_Body => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Expr_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5008 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 4945 '<FieldAccess for decl_part at ast.py:20724>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20724
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4945
Block_Var := Fld; 
--# bind subp_decl_part Block_Var
--# expr-start 4965 '<FilterMap at ast.py:20732>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20732
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind part Var_Expr_1
--# expr-start 4946 '<FieldAccess for aspects at ast.py:20730>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20730
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Aspect_Spec (Node => Implementation.Basic_Decl_F_Aspects (Var_Expr_1.Node), Info => Var_Expr_1.Info);
--# expr-done 4946
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 4947 '<FieldAccess for aspects at ast.py:20731>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20731
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Aspect_Spec (Node => Implementation.Basic_Decl_F_Aspects (Ent.Node), Info => Ent.Info);
--# expr-done 4947
   Result_Var := Fld_2;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Aspect_Spec then
   --# scope-start
--# expr-start 4948 '<FieldAccess for aspect_assocs at ast.py:20732>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20732
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Aspect_Assoc_List (Node => Var_Expr.Node.Aspect_Spec_F_Aspect_Assocs, Info => Var_Expr.Info);
--# expr-done 4948
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Aspect_Assoc_List;
end if;
Fld_4 := Result_Var_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_4 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Aspect_Assoc_List := Fld_4;
      begin
         for Untyped_Item_175 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_175 := Untyped_Item_175; 
Is_Null := Bare_Item_175 = null; 
if Is_Null then
   If_Result := No_Entity_Aspect_Assoc;
else
Fld_5 := Result_Var_1.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_175); 
   If_Result := As_Entity;
end if;
                  Item_175 := If_Result; 
   --# scope-start
               --# bind a Item_175
--# expr-start 4951 '<FieldAccess for name_is at ast.py:20734>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20734
--# expr-start 4949 '<FieldAccess for id at ast.py:20734>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20734
      if Item_175.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Item_175.Node.Aspect_Assoc_F_Id, Info => Item_175.Info);
--# expr-done 4949
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4950 '<Symbol Annotate at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 4950
--# property-call-start Name.name_is
Fld_12 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_11.Node, Sym => Sym);
--# end
--# expr-done 4951
if Fld_12 then
   --# expr-start 4953 '<IsA Aggregate at ast.py:20735>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20735
--# expr-start 4952 '<FieldAccess for expr at ast.py:20735>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20735
      if Item_175.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Item_175.Node.Aspect_Assoc_F_Expr, Info => Item_175.Info);
--# expr-done 4952
Is_A := Fld_13.Node /= null 
and then Fld_13.Node.Kind in Ada_Aggregate_Range; 
--# expr-done 4953
   If_Result_2 := Is_A;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
--# expr-start 4958 '<Cast to Name at ast.py:20737>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20737
--# expr-start 4957 '<FieldAccess for expr at ast.py:20737>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20737
--# expr-start 4956 '<FieldAccess for assocs at ast.py:20736>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20736
--# expr-start 4955 '<Cast to Aggregate at ast.py:20736>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20736
--# expr-start 4954 '<FieldAccess for expr at ast.py:20736>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20736
      if Item_175.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Item_175.Node.Aspect_Assoc_F_Expr, Info => Item_175.Info);
--# expr-done 4954
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 4955
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_1.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_1.Info);
--# expr-done 4956
Fld_16 := Fld_15.Node;
Get_Result_1 := Get (Self, Fld_16, 0, True); 
Is_Null_2 := Get_Result_1 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity_Basic_Assoc;
else
Fld_17 := Fld_15.Info;
As_Entity_2 := (Info => Fld_17, Node => Get_Result_1); 
   If_Result_3 := As_Entity_2;
end if;
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 4957
   if Fld_18.Node = null
      or else Fld_18.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 4958
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
--# expr-start 4960 '<FieldAccess for name_is at ast.py:20737>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20737
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4959 '<Symbol GNATprove at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Gnatprove); 
--# expr-done 4959
--# property-call-start Name.name_is
Fld_19 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_2.Node, Sym => Sym_1);
--# end
--# expr-done 4960
Scope_Result_2 := Fld_19;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_4 := Result_Var_2;
else
   If_Result_4 := False;
end if;
         if If_Result_4 then
      --# expr-start 4964 '<FieldAccess for expr at ast.py:20733>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20733
--# expr-start 4963 '<FieldAccess for assocs at ast.py:20733>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20733
--# expr-start 4962 '<Cast to Aggregate at ast.py:20733>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20733
--# expr-start 4961 '<FieldAccess for expr at ast.py:20733>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20733
      if Item_175.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Item_175.Node.Aspect_Assoc_F_Expr, Info => Item_175.Info);
--# expr-done 4961
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 4962
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Base_Aggregate_F_Assocs, Info => Cast_Result.Info);
--# expr-done 4963
Fld_8 := Fld_7.Node;
Get_Result := Get (Self, Fld_8, 1, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_9 := Fld_7.Info;
As_Entity_1 := (Info => Fld_9, Node => Get_Result); 
   If_Result_1 := As_Entity_1;
end if;
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 4964
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_10;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4965
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind aspects Block_Var_1
Var_Expr_3 := Block_Var;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_3 := Var_Expr_3;
   --# end
   Result_Var_3 := Scope_Result_3;
else
--# expr-start 4966 '<Cast to ExprFunction at ast.py:20744>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20744
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr_Function_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Expr_Function
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Expr_Function;
   end if;
--# expr-done 4966
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Result_3.Node,
         Info => Cast_Result_3.Info);
   Result_Var_3 := Cast_Result_4;
end if;
Block_Var_2 := Result_Var_3; 
--# bind pragma_scope Block_Var_2
--# expr-start 4971 '<FilterMap at ast.py:20751>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20751
Var_Expr_6 := Block_Var_2;
if Var_Expr_6 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4967 '<FieldAccess for declarative_scope at ast.py:20751>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20751
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_20 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Var_Expr_6.Node);
--# end
--# expr-done 4967
Scope_Result_4 := Fld_20;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Bare_Ada_Node;
end if;
Var_Expr_5 := Result_Var_4;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4968 '<FieldAccess for decls at ast.py:20751>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20751
      if Var_Expr_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Var_Expr_5.Declarative_Part_F_Decls;
--# expr-done 4968
Scope_Result_5 := Fld_21;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Bare_Ada_Node;
end if;
      declare
         Map_Result_1_Vec : Bare_Pragma_Node_Vectors.Vector;
      begin
      if Result_Var_5 = null then
      Map_Result_1 := Create_Bare_Pragma_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Result_Var_5;
      begin
         for Untyped_Item_176 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_176 := Untyped_Item_176; 
   --# scope-start
               --# bind p Item_176
         --# expr-start 4969 '<IsA Pragma at ast.py:20753>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20753
Is_A_1 := Item_176 /= null 
and then Item_176.Kind in Ada_Pragma_Node_Range; 
--# expr-done 4969
         if Is_A_1 then
      --# expr-start 4970 '<Cast to Pragma at ast.py:20752>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20752
Cast_Expr_1 := Item_176; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_5 := Cast_Expr_1;
   else
         Cast_Result_5 := No_Bare_Ada_Node;
   end if;
--# expr-done 4970
         declare
            Item_To_Append : constant Bare_Pragma_Node := Cast_Result_5;
         begin
            Bare_Pragma_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Bare_Pragma_Node_Array
           (Items_Count => Natural (Bare_Pragma_Node_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Pragma_Node_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Pragma_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Pragma_Node_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 4971
Var_Expr_4 := Map_Result_1;
      Inc_Ref (Var_Expr_4);
if Var_Expr_4 /= No_Bare_Pragma_Node_Array_Type then
   --# scope-start
   --# bind e Var_Expr_4
Scope_Result_6 := Var_Expr_4;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2567;
   Result_Var_7 := Scope_Result_6;
else
   --# expr-start 4974 '<Map at ast.py:20754>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20754
Var_Expr_7 := Block_Var_2;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4972 '<FieldAccess for library_item_pragmas at ast.py:20754>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20754
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.library_item_pragmas
Fld_22 := Libadalang.Implementation.Basic_Decl_P_Library_Item_Pragmas (Node => Var_Expr_7.Node, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 4972
Scope_Result_7 := Fld_22;
   --# end
   Result_Var_6 := Scope_Result_7;
else
   Result_Var_6 := No_Entity_Pragma_Node_List;
end if;
Fld_23 := Result_Var_6.Node;
      declare
         Map_Result_2_Vec : Bare_Pragma_Node_Vectors.Vector;
      begin
      if Fld_23 = null then
      Map_Result_2 := Create_Bare_Pragma_Node_Array (0);
      else
      declare
         Collection : constant Bare_Pragma_Node_List := Fld_23;
      begin
         for Untyped_Item_177 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_177 := Untyped_Item_177; 
Is_Null_3 := Bare_Item_177 = null; 
if Is_Null_3 then
   If_Result_5 := No_Entity_Pragma_Node;
else
Fld_24 := Result_Var_6.Info;
As_Entity_3 := (Info => Fld_24, Node => Bare_Item_177); 
   If_Result_5 := As_Entity_3;
end if;
                  Item_177 := If_Result_5; 
   --# scope-start
               --# bind p Item_177
      --# expr-start 4973 '<FieldAccess for node at ast.py:20755>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20755
Fld_25 := Item_177.Node;
--# expr-done 4973
         declare
            Item_To_Append : constant Bare_Pragma_Node := Fld_25;
         begin
            Bare_Pragma_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Bare_Pragma_Node_Array
           (Items_Count => Natural (Bare_Pragma_Node_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Bare_Pragma_Node_Vectors.Get
              (Map_Result_2_Vec,
               I + Bare_Pragma_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Bare_Pragma_Node_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 4974
   Result_Var_7 := Map_Result_2;
end if;
      Inc_Ref (Result_Var_7);
Block_Var_3 := Result_Var_7; Inc_Ref (Block_Var_3);
--# bind scope_decls Block_Var_3
--# expr-start 5000 '<FilterMap at ast.py:20760>' Map_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20760
      declare
         Map_Result_3_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Pragma_Node_Array_Access := Block_Var_3;
      begin
         for Item_178 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_178
--# expr-start 4976 '<OrderingTest '"'"'gt'"'"' at ast.py:20762>' Node_Comp /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20762
--# expr-start 4975 '<FieldAccess for node at ast.py:20762>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20762
Fld_28 := Block_Var_2.Node;
--# expr-done 4975
Node_Comp := Compare (Self, Item_178, Fld_28, Greater_Than); 
--# expr-done 4976
if Node_Comp then
Var_Expr_8 := Item_178;
if Var_Expr_8 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4979 '<FieldAccess for name_is at ast.py:20763>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20763
--# expr-start 4977 '<FieldAccess for id at ast.py:20763>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20763
      if Var_Expr_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Var_Expr_8.Pragma_Node_F_Id;
--# expr-done 4977
      if Fld_29 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4978 '<Symbol Annotate at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 4978
--# property-call-start Name.name_is
Fld_30 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_29, Sym => Sym_2);
--# end
--# expr-done 4979
Scope_Result_8 := Fld_30;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := False;
end if;
   If_Result_7 := Result_Var_8;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
--# expr-start 4984 '<Cast to Name at ast.py:20765>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20765
--# expr-start 4981 '<at at ast.py:20764>' Get_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20764
--# expr-start 4980 '<FieldAccess for args at ast.py:20764>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20764
      if Item_178 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Item_178.Pragma_Node_F_Args;
--# expr-done 4980
Get_Result_3 := Get (Self, Fld_31, 0, True); 
--# expr-done 4981
Var_Expr_10 := Get_Result_3;
if Var_Expr_10 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4983 '<FieldAccess for assoc_expr at ast.py:20765>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20765
--# expr-start 4982 '<as_entity at ast.py:20764>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20764
Is_Null_5 := Var_Expr_10 = null; 
if Is_Null_5 then
   If_Result_8 := No_Entity_Base_Assoc;
else
As_Entity_5 := (Info => E_Info, Node => Var_Expr_10); 
   If_Result_8 := As_Entity_5;
end if;
--# expr-done 4982
      if If_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_32 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_8.Node, E_Info => If_Result_8.Info);
--# end
--# expr-done 4983
Scope_Result_9 := Fld_32;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := No_Entity_Expr;
end if;
   if Result_Var_9.Node = null
      or else Result_Var_9.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Result_Var_9.Node,
         Info => Result_Var_9.Info);
   else
         Cast_Result_6 := No_Entity_Name;
   end if;
--# expr-done 4984
Var_Expr_9 := Cast_Result_6;
if Var_Expr_9 /= No_Entity_Name then
   --# scope-start
--# expr-start 4986 '<FieldAccess for name_is at ast.py:20765>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20765
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4985 '<Symbol GNATprove at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Gnatprove); 
--# expr-done 4985
--# property-call-start Name.name_is
Fld_33 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_9.Node, Sym => Sym_3);
--# end
--# expr-done 4986
Scope_Result_10 := Fld_33;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := False;
end if;
   If_Result_9 := Result_Var_10;
else
   If_Result_9 := False;
end if;
if If_Result_9 then
--# expr-start 4991 '<Cast to Name at ast.py:20767>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20767
--# expr-start 4988 '<at at ast.py:20766>' Get_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20766
--# expr-start 4987 '<FieldAccess for args at ast.py:20766>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20766
      if Item_178 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Item_178.Pragma_Node_F_Args;
--# expr-done 4987
Get_Result_4 := Get (Self, Fld_34, 2, True); 
--# expr-done 4988
Var_Expr_12 := Get_Result_4;
if Var_Expr_12 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4990 '<FieldAccess for assoc_expr at ast.py:20767>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20767
--# expr-start 4989 '<as_entity at ast.py:20766>' If_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20766
Is_Null_6 := Var_Expr_12 = null; 
if Is_Null_6 then
   If_Result_10 := No_Entity_Base_Assoc;
else
As_Entity_6 := (Info => E_Info, Node => Var_Expr_12); 
   If_Result_10 := As_Entity_6;
end if;
--# expr-done 4989
      if If_Result_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_35 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_10.Node, E_Info => If_Result_10.Info);
--# end
--# expr-done 4990
Scope_Result_11 := Fld_35;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   Result_Var_11 := No_Entity_Expr;
end if;
   if Result_Var_11.Node = null
      or else Result_Var_11.Node.Kind in Ada_Name
   then
      Cast_Result_7 := Create_Internal_Entity_Name
        (Node => Result_Var_11.Node,
         Info => Result_Var_11.Info);
   else
         Cast_Result_7 := No_Entity_Name;
   end if;
--# expr-done 4991
Var_Expr_11 := Cast_Result_7;
if Var_Expr_11 /= No_Entity_Name then
   --# scope-start
--# expr-start 4995 '<FieldAccess for name_is at ast.py:20768>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20768
      if Var_Expr_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4994 '<FieldAccess for name_symbol at ast.py:20768>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20768
--# expr-start 4993 '<at at ast.py:20768>' Get_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20768
--# expr-start 4992 '<FieldAccess for defining_names at ast.py:20768>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20768
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_36 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4992
Get_Result_5 := Get (Self, Fld_36, 0, True); 
--# expr-done 4993
      if Get_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_37 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Get_Result_5.Node);
--# end
--# expr-done 4994
--# property-call-start Name.name_is
Fld_38 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_11.Node, Sym => Fld_37);
--# end
--# expr-done 4995
Scope_Result_12 := Fld_38;
   --# end
      Finalizer_Scope_2575;
   Result_Var_12 := Scope_Result_12;
else
   Result_Var_12 := False;
end if;
   If_Result_11 := Result_Var_12;
else
   If_Result_11 := False;
end if;
         if If_Result_11 then
      --# expr-start 4999 '<FieldAccess for assoc_expr at ast.py:20761>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20761
--# expr-start 4998 '<as_entity at ast.py:20761>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20761
--# expr-start 4997 '<at at ast.py:20761>' Get_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20761
--# expr-start 4996 '<FieldAccess for args at ast.py:20761>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20761
      if Item_178 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Item_178.Pragma_Node_F_Args;
--# expr-done 4996
Get_Result_2 := Get (Self, Fld_26, 1, True); 
--# expr-done 4997
Is_Null_4 := Get_Result_2 = null; 
if Is_Null_4 then
   If_Result_6 := No_Entity_Base_Assoc;
else
As_Entity_4 := (Info => E_Info, Node => Get_Result_2); 
   If_Result_6 := As_Entity_4;
end if;
--# expr-done 4998
      if If_Result_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_27 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_6.Node, E_Info => If_Result_6.Info);
--# end
--# expr-done 4999
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_27;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 5000
Block_Var_4 := Map_Result_3; Inc_Ref (Block_Var_4);
--# bind pragmas Block_Var_4
--# expr-start 5004 '<Cast to BaseSubpBody at ast.py:20775>' Cast_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20775
--# expr-start 5003 '<Find at ast.py:20773>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20773
   --# expr-start 5001 '<FieldAccess for parents at ast.py:20773>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20773
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => False, E_Info => Ent.Info);
--# expr-done 5001
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_39;
      begin
         for Item_179 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_179
      --# expr-start 5002 '<IsA BaseSubpBody at ast.py:20774>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20774
Is_A_2 := Item_179.Node /= null 
and then Item_179.Node.Kind in Ada_Base_Subp_Body; 
--# expr-done 5002
      if Is_A_2 then
         Find_Result := Item_179; 
      end if;
   --# end
               exit when Is_A_2;
         end loop;
      end;
--# expr-done 5003
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_8 := Create_Internal_Entity_Base_Subp_Body
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_8 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 5004
Var_Expr_13 := Cast_Result_8;
if Var_Expr_13 /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 5005 '<FieldAccess for gnatprove_annotations at ast.py:20775>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20775
      if Var_Expr_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpBody.gnatprove_annotations
Fld_40 := Libadalang.Implementation.Base_Subp_Body_P_Gnatprove_Annotations (Node => Var_Expr_13.Node, E_Info => Var_Expr_13.Info);
--# end
--# expr-done 5005
Scope_Result_13 := Fld_40;
   Inc_Ref (Scope_Result_13);
   --# end
      Finalizer_Scope_2577;
   Result_Var_13 := Scope_Result_13;
else
   Result_Var_13 := No_Internal_Entity_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_13);
Block_Var_5 := Result_Var_13; Inc_Ref (Block_Var_5);
--# bind enclosing_subp_annotations Block_Var_5
--# expr-start 5007 '<Concat at ast.py:20778>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20778
--# expr-start 5006 '<Concat at ast.py:20778>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20778
Concat_Result := Concat (Block_Var_1, Block_Var_4); 
--# expr-done 5006
Concat_Result_1 := Concat (Concat_Result, Block_Var_5); 
--# expr-done 5007
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5008
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1423;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1423;
                     Finalizer_Scope_2567;
                     Finalizer_Scope_2575;
                     Finalizer_Scope_2577;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Expr_Array_Access,
                        As_Internal_Entity_Expr_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Subp_Body_P_Gnatprove_Annotations;
--# end
--# property-start BaseSubpBody.is_subject_to_proof /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20780
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Is_Subject_To_Proof
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5010 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5009 '<FieldAccess for is_spark_impl at ast.py:20786>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20786
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_spark_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Spark_Impl (Node => Ent.Node, Include_Skip_Proof_Annotations => True, E_Info => Ent.Info);
--# end
--# expr-done 5009
Let_Result := Fld; 
--# end
--# expr-done 5010
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Is_Subject_To_Proof;
--# end
--# property-start BaseSubpBody.has_spark_mode_on /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20788
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Has_Spark_Mode_On
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5012 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5011 '<FieldAccess for is_spark_impl at ast.py:20797>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20797
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_spark_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Spark_Impl (Node => Ent.Node, Include_Skip_Proof_Annotations => False, E_Info => Ent.Info);
--# end
--# expr-done 5011
Let_Result := Fld; 
--# end
--# expr-done 5012
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Has_Spark_Mode_On;
--# end
--# property-start BaseSubpBody._env_do_127 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20634
pragma Warnings (Off, "is not referenced");
 function Env_Do_127
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5013 '<FieldAccess for env_hook at ast.py:20634>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20634
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5013
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_127;
--# end
--# property-start BaseSubpBody._initial_env_128 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20637
pragma Warnings (Off, "is not referenced");
 function Initial_Env_128
  (Node : Bare_Base_Subp_Body
  )
   return Internal_Designated_Env
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1655 with Inline_Always;
            procedure Finalizer_Scope_2578 with Inline_Always;
      Var_Expr : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1655 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1655;
            procedure Finalizer_Scope_2578 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2578;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5014 '<FieldAccess for initial_env_name at ast.py:20638>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20638
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpBody.initial_env_name
Fld := Libadalang.Implementation.Base_Subp_Body_P_Initial_Env_Name (Node => Self, Follow_Private => True);
--# end
--# expr-done 5014
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind non_null_name Var_Expr
--# expr-start 5015 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr, Kind => Named_Env); 
--# expr-done 5015
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2578;
   Result_Var := Scope_Result;
else
   --# expr-start 5016 '<New DesignatedEnv at ast.py:20637>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20637
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5016
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1655;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1655;
                     Finalizer_Scope_2578;
            raise;
      end;
   return Property_Result;
end Initial_Env_128;
--# end
--# property-start BaseSubpBody._env_mappings_129 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20644
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_129
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1656 with Inline_Always;
            procedure Finalizer_Scope_2579 with Inline_Always;
      Var_Expr : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1656 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1656;
            procedure Finalizer_Scope_2579 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2579;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5020 '<FieldAccess for basic_decl_env_assocs at ast.py:20644>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20644
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5017 '<FieldAccess for initial_env_name at ast.py:20645>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20645
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpBody.initial_env_name
Fld := Libadalang.Implementation.Base_Subp_Body_P_Initial_Env_Name (Node => Self, Follow_Private => False);
--# end
--# expr-done 5017
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind non_null_name Var_Expr
--# expr-start 5018 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr, Kind => Named_Env); 
--# expr-done 5018
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2579;
   Result_Var := Scope_Result;
else
   --# expr-start 5019 '<New DesignatedEnv at ast.py:20645>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20645
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5019
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
--# property-call-start BasicDecl.basic_decl_env_assocs
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Env_Assocs (Node => Ent.Node, Dest_Env => Result_Var, E_Info => Ent.Info);
--# end
--# expr-done 5020
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1656;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1656;
                     Finalizer_Scope_2579;
            raise;
      end;
   return Property_Result;
end Env_Mappings_129;
--# end
--# property-start BaseSubpBody._env_trans_parent_130 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20633
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_130
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_130;
--# end
--# property-start BaseSubpBody._env_mappings_131 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20651
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_131
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1658 with Inline_Always;
            procedure Finalizer_Scope_2580 with Inline_Always;
            procedure Finalizer_Scope_2581 with Inline_Always;
      Var_Expr : Symbol_Type;
Var_Expr_1 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
New_Struct_2 : Internal_Designated_Env := No_Designated_Env;
Scope_Result_1 : Internal_Designated_Env := No_Designated_Env;
New_Struct_3 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Fld_3 : Boolean;
If_Result : Internal_Designated_Env := No_Designated_Env;
Result_Var_1 : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
New_Struct_4 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1658 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (New_Struct_3);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (New_Struct_4);
            end Finalizer_Scope_1658;
            procedure Finalizer_Scope_2580 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2580;
            procedure Finalizer_Scope_2581 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (New_Struct_2);
            end Finalizer_Scope_2581;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5032 '<New EnvAssoc at ast.py:20651>' New_Struct_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20651
--# expr-start 5021 '<FieldAccess for previous_part_env_name at ast.py:20654>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20654
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_env_name'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Env_Name (Node => Self);
--# end
--# expr-done 5021
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 5022 '<New DesignatedEnv at ast.py:20655>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20655
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr, Kind => Named_Env); 
--# expr-done 5022
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2580;
   Result_Var_1 := Scope_Result;
else
   --# expr-start 5030 '<If at ast.py:20661>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20661
--# expr-start 5023 '<FieldAccess for has_top_level_env_name at ast.py:20662>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20662
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5023
if Fld_3 then
   --# expr-start 5024 '<New DesignatedEnv at ast.py:20663>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20663
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => null, Kind => None); 
--# expr-done 5024
   If_Result := New_Struct_1;
else
--# expr-start 5027 '<bind at ast.py:20665>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20665
--# expr-start 5025 '<FieldAccess for default_initial_env at ast.py:20666>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20666
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 5025
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 5026 '<FieldAccess for body_scope at ast.py:20667>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20667
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_2 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 5026
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5027
Var_Expr_1 := Dyn_Var_Bind_Result;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= Empty_Env then
   --# scope-start
   --# bind non_null_env Var_Expr_1
--# expr-start 5028 '<New DesignatedEnv at ???>' New_Struct_2 None
Inc_Ref (Var_Expr_1);
New_Struct_2 := (Direct_Env => Var_Expr_1, Env_Name => null, Kind => Direct_Env); 
--# expr-done 5028
Scope_Result_1 := New_Struct_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2581;
   Result_Var := Scope_Result_1;
else
   --# expr-start 5029 '<New DesignatedEnv at ast.py:20664>' New_Struct_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20664
Inc_Ref (Empty_Env);
New_Struct_3 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5029
   Result_Var := New_Struct_3;
end if;
      Inc_Ref (Result_Var);
   If_Result := Result_Var;
end if;
      Inc_Ref (If_Result);
--# expr-done 5030
   Result_Var_1 := If_Result;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 5031 '<Symbol __nextpart at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 5031
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Result_Var_1);
New_Struct_4 := (Dest_Env => Result_Var_1, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5032
         Property_Result := New_Struct_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1658;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1658;
                     Finalizer_Scope_2580;
                     Finalizer_Scope_2581;
            raise;
      end;
   return Property_Result;
end Env_Mappings_131;
--# end
--# property-start BaseSubpBody._env_do_132 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20678
pragma Warnings (Off, "is not referenced");
 function Env_Do_132
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1659 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1659 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1659;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5033 '<FieldAccess for populate_dependent_units at ast.py:20678>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20678
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5033
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1659;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1659;
            raise;
      end;
   return Property_Result;
end Env_Do_132;
--# end
--# property-start BaseSubpBody._ref_env_nodes_133 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20681
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_133
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1660 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1660 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1660;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5034 '<FieldAccess for top_level_use_package_clauses at ast.py:20681>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20681
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5034
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1660;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1660;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_133;
--# end
--# property-start BaseSubpBody._ref_cond_134 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20683
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_134
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5036 '<IsA LibraryItem, Subunit at ast.py:20683>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20683
--# expr-start 5035 '<parent at ast.py:20683>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20683
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5035
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5036
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_134;
--# end
--# property-start BaseSubpBody._ref_env_nodes_135 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20687
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_135
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1662 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1662 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1662;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5037 '<FieldAccess for top_level_use_type_clauses at ast.py:20687>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20687
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5037
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1662;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1662;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_135;
--# end
--# property-start BaseSubpBody._ref_cond_136 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20689
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_136
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5039 '<IsA LibraryItem, Subunit at ast.py:20689>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20689
--# expr-start 5038 '<parent at ast.py:20689>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20689
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5038
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5039
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_136;
--# end
--# property-start BaseSubpBody._ref_env_nodes_137 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20696
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_137
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1664 with Inline_Always;
            procedure Finalizer_Scope_2582 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1664 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1664;
            procedure Finalizer_Scope_2582 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2582;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5040 '<Cast to AdaNode at ast.py:20696>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20696
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5040
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2582;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1664;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1664;
                     Finalizer_Scope_2582;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_137;
--# end
--# property-start BaseSubpBody._ref_cond_138 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20698
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_138
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5042 '<IsA LibraryItem at ast.py:20698>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20698
--# expr-start 5041 '<parent at ast.py:20698>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20698
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5041
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range; 
--# expr-done 5042
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_138;
--# end
--# property-start BaseSubpBody._ref_env_nodes_139 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20702
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_139
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1666 with Inline_Always;
            procedure Finalizer_Scope_2583 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1666 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1666;
            procedure Finalizer_Scope_2583 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2583;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5043 '<Cast to AdaNode at ast.py:20702>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20702
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5043
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2583;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1666;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1666;
                     Finalizer_Scope_2583;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_139;
--# end
--# property-start BaseSubpBody._ref_cond_140 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20704
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_140
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5044 '<FieldAccess for should_ref_generic_formals at ast.py:20704>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20704
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5044
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_140;
--# end
--# property-start BaseSubpBody._ref_env_nodes_141 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20712
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_141
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1668 with Inline_Always;
            procedure Finalizer_Scope_2584 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1668 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1668;
            procedure Finalizer_Scope_2584 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2584;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5045 '<Cast to AdaNode at ast.py:20712>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20712
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5045
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2584;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1668;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1668;
                     Finalizer_Scope_2584;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_141;
--# end
--# property-start BaseSubpBody._ref_cond_142 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20714
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_142
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5046 '<FieldAccess for should_ref_generic_formals at ast.py:20714>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20714
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5046
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_142;
--# end
   --
   --  Primitives for Bare_Expr_Function
   --
      procedure Initialize_Fields_For_Expr_Function
        (Self : Bare_Expr_Function
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Expr_Function_F_Expr : Bare_Expr
         ; Expr_Function_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Expr_Function_F_Expr := Expr_Function_F_Expr;
            Self.Expr_Function_F_Aspects := Expr_Function_F_Aspects;
      end Initialize_Fields_For_Expr_Function;
   function Expr_Function_F_Expr
     (Node : Bare_Expr_Function) return Bare_Expr
   is
   begin
         return Node.Expr_Function_F_Expr;
   end;
--# property-start ExprFunction.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20808
pragma Warnings (Off, "is not referenced");
 function Expr_Function_P_Xref_Equation
  (Node : Bare_Expr_Function
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Expr_Function  := Bare_Expr_Function (Node);
     Ent : Internal_Entity_Expr_Function :=
       Internal_Entity_Expr_Function'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1426 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1426 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1426;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5057 '<And at ast.py:20809>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20809
--# expr-start 5054 '<And at ast.py:20809>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20809
--# expr-start 5051 '<Bind at ast.py:20809>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20809
--# expr-start 5048 '<FieldAccess for expected_type_var at ast.py:20809>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20809
--# expr-start 5047 '<FieldAccess for expr at ast.py:20809>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20809
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Expr_Function_F_Expr, Info => Ent.Info);
--# expr-done 5047
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 5048
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 5050 '<FieldAccess for return_type at ast.py:20809>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20809
--# expr-start 5049 '<FieldAccess for subp_spec at ast.py:20809>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20809
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5049
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5050
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20809") else null)); 
--# expr-done 5051
--# expr-start 5053 '<FieldAccess for sub_equation at ast.py:20810>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20810
--# expr-start 5052 '<FieldAccess for expr at ast.py:20810>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20810
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Expr_Function_F_Expr, Info => Ent.Info);
--# expr-done 5052
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5053
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20809") else null)); 
--# expr-done 5054
--# expr-start 5056 '<FieldAccess for matches_expected_assign_type at ast.py:20811>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20811
--# expr-start 5055 '<FieldAccess for expr at ast.py:20811>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20811
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Expr_Function_F_Expr;
--# expr-done 5055
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_7 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_6, Origin => Origin);
--# end
--# expr-done 5056
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20809") else null)); 
--# expr-done 5057
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1426;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1426;
            raise;
      end;
   return Property_Result;
end Expr_Function_P_Xref_Equation;
--# end
--# property-start ExprFunction.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20814
pragma Warnings (Off, "is not referenced");
 function Expr_Function_P_Xref_Entry_Point
  (Node : Bare_Expr_Function
  )
   return Boolean
is
   Self : Bare_Expr_Function  := Bare_Expr_Function (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_Function_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Null_Subp_Decl
   --
      procedure Initialize_Fields_For_Null_Subp_Decl
        (Self : Bare_Null_Subp_Decl
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Null_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Null_Subp_Decl_F_Aspects := Null_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Null_Subp_Decl;
   --
   --  Primitives for Bare_Subp_Body
   --
      procedure Initialize_Fields_For_Subp_Body
        (Self : Bare_Subp_Body
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Body_F_Aspects : Bare_Aspect_Spec
         ; Subp_Body_F_Decls : Bare_Declarative_Part
         ; Subp_Body_F_Stmts : Bare_Handled_Stmts
         ; Subp_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Subp_Body_F_Aspects := Subp_Body_F_Aspects;
            Self.Subp_Body_F_Decls := Subp_Body_F_Decls;
            Self.Subp_Body_F_Stmts := Subp_Body_F_Stmts;
            Self.Subp_Body_F_End_Name := Subp_Body_F_End_Name;
      end Initialize_Fields_For_Subp_Body;
   function Subp_Body_F_Decls
     (Node : Bare_Subp_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Subp_Body_F_Decls;
   end;
   function Subp_Body_F_Stmts
     (Node : Bare_Subp_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Subp_Body_F_Stmts;
   end;
   function Subp_Body_F_End_Name
     (Node : Bare_Subp_Body) return Bare_End_Name
   is
   begin
         return Node.Subp_Body_F_End_Name;
   end;
--# property-start SubpBody.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20871
pragma Warnings (Off, "is not referenced");
 function Subp_Body_P_Declarative_Parts
  (Node : Bare_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Subp_Body  := Bare_Subp_Body (Node);
     Ent : Internal_Entity_Subp_Body :=
       Internal_Entity_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1430 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1430 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1430;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5058 '<FieldAccess for decls at ast.py:20871>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20871
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Subp_Body_F_Decls, Info => Ent.Info);
--# expr-done 5058
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1430;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1430;
            raise;
      end;
   return Property_Result;
end Subp_Body_P_Declarative_Parts;
--# end
   --
   --  Primitives for Bare_Subp_Renaming_Decl
   --
      procedure Initialize_Fields_For_Subp_Renaming_Decl
        (Self : Bare_Subp_Renaming_Decl
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Subp_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Subp_Renaming_Decl_F_Renames := Subp_Renaming_Decl_F_Renames;
            Self.Subp_Renaming_Decl_F_Aspects := Subp_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Subp_Renaming_Decl;
   function Subp_Renaming_Decl_F_Renames
     (Node : Bare_Subp_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Subp_Renaming_Decl_F_Renames;
   end;
--# property-start SubpRenamingDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20833
pragma Warnings (Off, "is not referenced");
 function Subp_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Subp_Renaming_Decl  := Bare_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start SubpRenamingDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20834
pragma Warnings (Off, "is not referenced");
 function Subp_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Subp_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subp_Renaming_Decl  := Bare_Subp_Renaming_Decl (Node);
     Ent : Internal_Entity_Subp_Renaming_Decl :=
       Internal_Entity_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1429 with Inline_Always;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Is_A : Boolean;
Fld_2 : Internal_Entity_Renaming_Clause;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Subp_Spec;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_7 : Internal_Entity_Renaming_Clause;
Fld_8 : Internal_Entity_Name;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Internal_Entity_Renaming_Clause;
Fld_11 : Internal_Entity_Name;
Is_A_1 : Boolean;
Fld_12 : Internal_Entity_Renaming_Clause;
Fld_13 : Internal_Entity_Name;
Fld_14 : Logic_Equation;
Fld_15 : Internal_Entity_Renaming_Clause;
Fld_16 : Internal_Entity_Name;
Fld_17 : Logic_Equation;
Fld_18 : Internal_Entity_Renaming_Clause;
Fld_19 : Internal_Entity_Name;
Fld_20 : Logic_Var;
Cast_Expr : Internal_Entity_Subp_Renaming_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
If_Result : Logic_Equation;
If_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
False_Rel : Logic_Equation;
Fld_21 : Internal_Entity_Renaming_Clause;
Fld_22 : Internal_Entity_Name;
Fld_23 : Boolean;
If_Result_2 : Logic_Equation;
Or_Pred : Logic_Equation;
            procedure Finalizer_Scope_1429 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_1429;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5092 '<Or at ast.py:20834>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20834
--# expr-start 5087 '<Cond at ast.py:20835>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20835
--# expr-start 5061 '<IsA CharLiteral at ast.py:20836>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20836
--# expr-start 5060 '<FieldAccess for renamed_object at ast.py:20836>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20836
--# expr-start 5059 '<FieldAccess for renames at ast.py:20836>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20836
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5059
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 5060
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Char_Literal_Range; 
--# expr-done 5061
if Is_A then
   --# expr-start 5071 '<And at ast.py:20840>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20840
--# expr-start 5067 '<Bind at ast.py:20840>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20840
--# expr-start 5064 '<FieldAccess for expected_type_var at ast.py:20840>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20840
--# expr-start 5063 '<FieldAccess for renamed_object at ast.py:20840>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20840
--# expr-start 5062 '<FieldAccess for renames at ast.py:20840>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20840
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5062
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Fld_2.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_2.Info);
--# expr-done 5063
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_3.Node);
--# expr-done 5064
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 5066 '<FieldAccess for return_type at ast.py:20841>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20841
--# expr-start 5065 '<FieldAccess for subp_spec at ast.py:20841>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20841
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5065
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_6 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_5.Node, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 5066
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20840") else null)); 
--# expr-done 5067
--# expr-start 5070 '<FieldAccess for sub_equation at ast.py:20842>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20842
--# expr-start 5069 '<FieldAccess for renamed_object at ast.py:20842>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20842
--# expr-start 5068 '<FieldAccess for renames at ast.py:20842>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20842
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5068
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Fld_7.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_7.Info);
--# expr-done 5069
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 5070
And_Pred := Create_And (Bind_Result, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20840") else null)); 
--# expr-done 5071
   If_Result_1 := And_Pred;
else
--# expr-start 5074 '<IsA AttributeRef at ast.py:20844>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20844
--# expr-start 5073 '<FieldAccess for renamed_object at ast.py:20844>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20844
--# expr-start 5072 '<FieldAccess for renames at ast.py:20844>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20844
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5072
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Fld_10.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_10.Info);
--# expr-done 5073
Is_A_1 := Fld_11.Node /= null 
and then Fld_11.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 5074
if Is_A_1 then
   --# expr-start 5077 '<FieldAccess for sub_equation at ast.py:20847>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20847
--# expr-start 5076 '<FieldAccess for renamed_object at ast.py:20847>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20847
--# expr-start 5075 '<FieldAccess for renames at ast.py:20847>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20847
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5075
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Fld_12.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_12.Info);
--# expr-done 5076
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_13.Node, Env => Env, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 5077
   If_Result := Fld_14;
else
   --# expr-start 5086 '<And at ast.py:20849>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20849
--# expr-start 5080 '<FieldAccess for xref_no_overloading at ast.py:20849>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20849
--# expr-start 5079 '<FieldAccess for renamed_object at ast.py:20849>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20849
--# expr-start 5078 '<FieldAccess for renames at ast.py:20849>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20849
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5078
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Name (Node => Fld_15.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_15.Info);
--# expr-done 5079
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_17 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_16.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Fld_16.Info);
--# end
--# expr-done 5080
--# expr-start 5085 '<Predicate on BasicDecl.subp_decl_match_signature at ast.py:20850>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20850
--# expr-start 5083 '<FieldAccess for ref_var at ast.py:20851>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20851
--# expr-start 5082 '<FieldAccess for renamed_object at ast.py:20851>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20851
--# expr-start 5081 '<FieldAccess for renames at ast.py:20851>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20851
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5081
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Name (Node => Fld_18.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_18.Info);
--# expr-done 5082
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_19.Node);
--# end
--# expr-done 5083
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
--# expr-start 5084 '<Cast to BasicDecl at ast.py:20852>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20852
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 5084
Pred := Solver.Create_Predicate (Fld_20, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result_1, Origin)); 
--# expr-done 5085
And_Pred_1 := Create_And (Fld_17, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20849") else null)); 
--# expr-done 5086
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5087
--# expr-start 5091 '<If at ast.py:20856>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20856
--# expr-start 5090 '<FieldAccess for is_operator_name at ast.py:20856>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20856
--# expr-start 5089 '<FieldAccess for renamed_object at ast.py:20856>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20856
--# expr-start 5088 '<FieldAccess for renames at ast.py:20856>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20856
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5088
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Name (Node => Fld_21.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_21.Info);
--# expr-done 5089
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_operator_name
Fld_23 := Libadalang.Implementation.Name_P_Is_Operator_Name (Node => Fld_22.Node, E_Info => Fld_22.Info);
--# end
--# expr-done 5090
if Fld_23 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20857") else null)); 
   If_Result_2 := True_Rel;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20857") else null)); 
   If_Result_2 := False_Rel;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 5091
Or_Pred := Create_Or (If_Result_1, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20834") else null)); 
--# expr-done 5092
         Property_Result := Or_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1429;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1429;
            raise;
      end;
   return Property_Result;
end Subp_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Body_Stub
   --
--# property-start BodyStub.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4117
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Next_Part_For_Decl
  (Node : Bare_Body_Stub
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Stub  := Bare_Body_Stub (Node);
     Ent : Internal_Entity_Body_Stub :=
       Internal_Entity_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_246 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_246 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_246;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5096 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5095 '<as_entity at ast.py:4124>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4124
--# expr-start 5094 '<FieldAccess for get_unit_root_decl at ast.py:4119>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4119
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5093 '<FieldAccess for fully_qualified_name_array at ast.py:4120>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4120
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fully_qualified_name_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Array (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5093
--# property-call-start AdaNode.get_unit_root_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Fld, Kind => Unit_Body, Process_Parents => False);
--# end
--# expr-done 5094
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 5095
Let_Result := If_Result; 
--# end
--# expr-done 5096
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_246;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_246;
            raise;
      end;
   return Property_Result;
end Body_Stub_P_Next_Part_For_Decl;
--# end
--# property-start BodyStub.syntactic_fully_qualified_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4126
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Syntactic_Fully_Qualified_Name
  (Node : Bare_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Body_Stub  := Bare_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_247 with Inline_Always;
      Block_Var : Symbol_Type_Array_Access;
Block_Var_1 : Bare_Body_Node;
Block_Var_2 : Bare_Compilation_Unit;
Match_0 : Bare_Subp_Body;
Match_1 : Bare_Protected_Body;
Match_2 : Bare_Package_Body;
Match_3 : Bare_Task_Body;
Match_4 : Bare_Ada_Node;
As_Entity : Internal_Entity_Body_Stub;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Stub;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Ada_Node;
Fld_3 : Bare_Ada_Node;
Fld_4 : Bare_Ada_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Subp_Body;
Cast_Expr : Bare_Subp_Body;
Cast_Result_1 : Bare_Body_Node;
Let_Result : Bare_Body_Node;
Scope_Result : Bare_Body_Node;
Cast_Result_2 : Bare_Protected_Body;
Cast_Expr_1 : Bare_Protected_Body;
Cast_Result_3 : Bare_Body_Node;
Let_Result_1 : Bare_Body_Node;
Scope_Result_1 : Bare_Body_Node;
Cast_Result_4 : Bare_Package_Body;
Cast_Expr_2 : Bare_Package_Body;
Cast_Result_5 : Bare_Body_Node;
Let_Result_2 : Bare_Body_Node;
Scope_Result_2 : Bare_Body_Node;
Cast_Result_6 : Bare_Task_Body;
Cast_Expr_3 : Bare_Task_Body;
Cast_Result_7 : Bare_Body_Node;
Let_Result_3 : Bare_Body_Node;
Scope_Result_3 : Bare_Body_Node;
Cast_Result_8 : Bare_Ada_Node;
Let_Result_4 : Bare_Body_Node;
Scope_Result_4 : Bare_Body_Node;
Match_Result : Bare_Body_Node;
Fld_5 : Bare_Ada_Node;
Fld_6 : Bare_Ada_Node;
Cast_Result_9 : Bare_Compilation_Unit;
Fld_7 : Bare_Ada_Node;
Is_A : Boolean;
Fld_8 : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Bare_Compilation_Unit;
Fld_9 : Symbol_Type_Array_Access;
Concat_Result : Symbol_Type_Array_Access;
Let_Result_5 : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_247 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_247;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5114 '<Block at ???>' Let_Result_5 None
--# scope-start
--# expr-start 5099 '<FieldAccess for as_symbol_array at ast.py:4135>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4135
--# expr-start 5098 '<FieldAccess for defining_name at ast.py:4135>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4135
--# expr-start 5097 '<as_bare_entity at ast.py:4135>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4135
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Stub;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 5097
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5098
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld.Node);
--# end
--# expr-done 5099
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind rel_name Block_Var
--# expr-start 5103 '<Match at ast.py:4146>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4146
--# expr-start 5102 '<parent at ast.py:4146>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4146
--# expr-start 5101 '<parent at ast.py:4146>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4146
--# expr-start 5100 '<parent at ast.py:4146>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4146
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Parent; 
--# expr-done 5100
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Fld_2.Parent; 
--# expr-done 5101
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 5102
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_4; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind b Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Cast_Expr;
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind b Match_1
Cast_Expr_1 := Match_1; 
      Cast_Result_3 := Cast_Expr_1;
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind b Match_2
Cast_Expr_2 := Match_2; 
      Cast_Result_5 := Cast_Expr_2;
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_3 := Cast_Result_6; 
--# bind b Match_3
Cast_Expr_3 := Match_3; 
      Cast_Result_7 := Cast_Expr_3;
Let_Result_3 := Cast_Result_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Entry_Index_Spec .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_4 := Cast_Result_8; 
--# bind _ Match_4
Let_Result_4 := raise Property_Error with "invalid body stub"; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 5103
Block_Var_1 := Match_Result; 
--# bind top_body Block_Var_1
--# expr-start 5111 '<If at ast.py:4153>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4153
--# expr-start 5105 '<IsA LibraryItem at ast.py:4154>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4154
--# expr-start 5104 '<parent at ast.py:4154>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4154
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Block_Var_1.Parent; 
--# expr-done 5104
Is_A := Fld_7 /= null 
and then Fld_7.Kind in Ada_Library_Item_Range; 
--# expr-done 5105
if Is_A then
   If_Result_1 := True;
else
   --# expr-start 5107 '<IsA Subunit at ast.py:4155>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4155
--# expr-start 5106 '<parent at ast.py:4155>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4155
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Block_Var_1.Parent; 
--# expr-done 5106
Is_A_1 := Fld_8 /= null 
and then Fld_8.Kind in Ada_Subunit_Range; 
--# expr-done 5107
   If_Result_1 := Is_A_1;
end if;
if If_Result_1 then
   --# expr-start 5110 '<Cast to CompilationUnit at ast.py:4157>' Cast_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4157
--# expr-start 5109 '<parent at ast.py:4157>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4157
--# expr-start 5108 '<parent at ast.py:4157>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4157
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Block_Var_1.Parent; 
--# expr-done 5108
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Fld_5.Parent; 
--# expr-done 5109
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_9 := Fld_6;
   else
         Cast_Result_9 := No_Bare_Ada_Node;
   end if;
--# expr-done 5110
   If_Result_2 := Cast_Result_9;
else
   If_Result_2 := raise Property_Error with "invalid body stub";
end if;
--# expr-done 5111
Block_Var_2 := If_Result_2; 
--# bind cu Block_Var_2
--# expr-start 5113 '<Concat at ast.py:4161>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4161
--# expr-start 5112 '<FieldAccess for syntactic_fully_qualified_name at ast.py:4161>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4161
      if Block_Var_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_9 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Block_Var_2);
--# end
--# expr-done 5112
Concat_Result := Concat (Fld_9, Block_Var); 
--# expr-done 5113
Let_Result_5 := Concat_Result; Inc_Ref (Let_Result_5);
--# end
--# expr-done 5114
         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_247;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_247;
            raise;
      end;
   return Property_Result;
end Body_Stub_P_Syntactic_Fully_Qualified_Name;
--# end
--# property-start BodyStub.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4163
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Env_Names
  (Node : Bare_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Body_Stub  := Bare_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_248 with Inline_Always;
      Fld : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_248 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_248;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5120 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5119 '<ArrayLiteral at ast.py:4169>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4169
--# expr-start 5118 '<to_symbol at ast.py:4170>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4170
--# expr-start 5117 '<Concat at ast.py:4170>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4170
--# expr-start 5115 '<FieldAccess for top_level_env_name at ast.py:4170>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4170
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5115
--# expr-start 5116 '<String at ast.py:4170>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4170
Str := Create_String ("__stub"); 
--# expr-done 5116
Concat_Result := Concat_String (Fld, Str); 
--# expr-done 5117
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 5118
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 5119
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 5120
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_248;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_248;
            raise;
      end;
   return Property_Result;
end Body_Stub_P_Env_Names;
--# end
--# property-start BodyStub.previous_part_link_env_assoc /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4173
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Previous_Part_Link_Env_Assoc
  (Node : Bare_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Body_Stub  := Bare_Body_Stub (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_249 with Inline_Always;
      Sym : Symbol_Type;
Fld : String_Type;
Sym_1 : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_249 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_249;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5126 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5125 '<New EnvAssoc at ast.py:4175>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4175
--# expr-start 5123 '<New DesignatedEnv at ast.py:4178>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4178
--# expr-start 5122 '<to_symbol at ast.py:4178>' Sym_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4178
--# expr-start 5121 '<FieldAccess for top_level_env_name at ast.py:4178>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4178
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5121
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Fld); 
--# expr-done 5122
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Sym_1, Kind => Named_Env); 
--# expr-done 5123
--# expr-start 5124 '<Symbol __nextpart at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 5124
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5125
Let_Result := New_Struct_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5126
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_249;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_249;
            raise;
      end;
   return Property_Result;
end Body_Stub_P_Previous_Part_Link_Env_Assoc;
--# end
   --
   --  Primitives for Bare_Package_Body_Stub
   --
      procedure Package_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Package_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_219 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:22143");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_220 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_221 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Package_Body_Stub
        (Self : Bare_Package_Body_Stub
         ; Package_Body_Stub_F_Name : Bare_Defining_Name
         ; Package_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Package_Body_Stub_F_Name := Package_Body_Stub_F_Name;
            Self.Package_Body_Stub_F_Aspects := Package_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Package_Body_Stub;
   function Package_Body_Stub_F_Name
     (Node : Bare_Package_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Package_Body_Stub_F_Name;
   end;
--# property-start PackageBodyStub.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22140
pragma Warnings (Off, "is not referenced");
 function Package_Body_Stub_P_Defining_Names
  (Node : Bare_Package_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Body_Stub  := Bare_Package_Body_Stub (Node);
     Ent : Internal_Entity_Package_Body_Stub :=
       Internal_Entity_Package_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1507 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1507 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1507;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5127 '<FieldAccess for name at ast.py:22140>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22140
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 5127
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1507;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1507;
            raise;
      end;
   return Property_Result;
end Package_Body_Stub_P_Defining_Names;
--# end
--# property-start PackageBodyStub._env_mappings_219 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22143
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_219
  (Node : Bare_Package_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Body_Stub  := Bare_Package_Body_Stub (Node);
     Ent : Internal_Entity_Package_Body_Stub :=
       Internal_Entity_Package_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1746 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1746 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1746;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5128 '<FieldAccess for previous_part_link_env_assoc at ast.py:22143>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22143
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5128
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1746;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1746;
            raise;
      end;
   return Property_Result;
end Env_Mappings_219;
--# end
--# property-start PackageBodyStub._env_trans_parent_220 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22142
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_220
  (Node : Bare_Package_Body_Stub
  )
   return Boolean
is
   Self : Bare_Package_Body_Stub  := Bare_Package_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_220;
--# end
--# property-start PackageBodyStub._env_names_221 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22144
pragma Warnings (Off, "is not referenced");
 function Env_Names_221
  (Node : Bare_Package_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Package_Body_Stub  := Bare_Package_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1748 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1748 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1748;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5129 '<FieldAccess for env_names at ast.py:22144>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22144
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 5129
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1748;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1748;
            raise;
      end;
   return Property_Result;
end Env_Names_221;
--# end
   --
   --  Primitives for Bare_Protected_Body_Stub
   --
      procedure Protected_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Protected_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_209 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:22086");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_210 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_211 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Protected_Body_Stub
        (Self : Bare_Protected_Body_Stub
         ; Protected_Body_Stub_F_Name : Bare_Defining_Name
         ; Protected_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Protected_Body_Stub_F_Name := Protected_Body_Stub_F_Name;
            Self.Protected_Body_Stub_F_Aspects := Protected_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Protected_Body_Stub;
   function Protected_Body_Stub_F_Name
     (Node : Bare_Protected_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Protected_Body_Stub_F_Name;
   end;
--# property-start ProtectedBodyStub.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22083
pragma Warnings (Off, "is not referenced");
 function Protected_Body_Stub_P_Defining_Names
  (Node : Bare_Protected_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Protected_Body_Stub  := Bare_Protected_Body_Stub (Node);
     Ent : Internal_Entity_Protected_Body_Stub :=
       Internal_Entity_Protected_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1504 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1504 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1504;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5130 '<FieldAccess for name at ast.py:22083>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22083
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Protected_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 5130
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1504;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1504;
            raise;
      end;
   return Property_Result;
end Protected_Body_Stub_P_Defining_Names;
--# end
--# property-start ProtectedBodyStub._env_mappings_209 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22086
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_209
  (Node : Bare_Protected_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Body_Stub  := Bare_Protected_Body_Stub (Node);
     Ent : Internal_Entity_Protected_Body_Stub :=
       Internal_Entity_Protected_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1736 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1736 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1736;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5131 '<FieldAccess for previous_part_link_env_assoc at ast.py:22086>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22086
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5131
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1736;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1736;
            raise;
      end;
   return Property_Result;
end Env_Mappings_209;
--# end
--# property-start ProtectedBodyStub._env_trans_parent_210 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22085
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_210
  (Node : Bare_Protected_Body_Stub
  )
   return Boolean
is
   Self : Bare_Protected_Body_Stub  := Bare_Protected_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_210;
--# end
--# property-start ProtectedBodyStub._env_names_211 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22087
pragma Warnings (Off, "is not referenced");
 function Env_Names_211
  (Node : Bare_Protected_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Body_Stub  := Bare_Protected_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1738 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1738 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1738;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5132 '<FieldAccess for env_names at ast.py:22087>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22087
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 5132
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1738;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1738;
            raise;
      end;
   return Property_Result;
end Env_Names_211;
--# end
   --
   --  Primitives for Bare_Subp_Body_Stub
   --
      procedure Subp_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Subp_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_212 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:22105");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_213 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_214 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      if Libadalang.Implementation.Ref_Cond_216 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_215 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_218 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_217 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Subp_Body_Stub
        (Self : Bare_Subp_Body_Stub
         ; Subp_Body_Stub_F_Overriding : Bare_Overriding_Node
         ; Subp_Body_Stub_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Subp_Body_Stub_F_Overriding := Subp_Body_Stub_F_Overriding;
            Self.Subp_Body_Stub_F_Subp_Spec := Subp_Body_Stub_F_Subp_Spec;
            Self.Subp_Body_Stub_F_Aspects := Subp_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Subp_Body_Stub;
   function Subp_Body_Stub_F_Overriding
     (Node : Bare_Subp_Body_Stub) return Bare_Overriding_Node
   is
   begin
         return Node.Subp_Body_Stub_F_Overriding;
   end;
   function Subp_Body_Stub_F_Subp_Spec
     (Node : Bare_Subp_Body_Stub) return Bare_Subp_Spec
   is
   begin
         return Node.Subp_Body_Stub_F_Subp_Spec;
   end;
--# property-start SubpBodyStub.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22100
pragma Warnings (Off, "is not referenced");
 function Subp_Body_Stub_P_Defining_Names
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
     Ent : Internal_Entity_Subp_Body_Stub :=
       Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1505 with Inline_Always;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1505 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1505;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5134 '<FieldAccess for name at ast.py:22100>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22100
--# expr-start 5133 '<FieldAccess for subp_spec at ast.py:22100>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22100
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Subp_Body_Stub_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5133
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5134
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_1;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1505;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1505;
            raise;
      end;
   return Property_Result;
end Subp_Body_Stub_P_Defining_Names;
--# end
--# property-start SubpBodyStub.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22129
pragma Warnings (Off, "is not referenced");
 function Subp_Body_Stub_P_Type_Expression
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
     Ent : Internal_Entity_Subp_Body_Stub :=
       Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5136 '<FieldAccess for returns at ast.py:22129>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22129
--# expr-start 5135 '<FieldAccess for subp_spec at ast.py:22129>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22129
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Subp_Body_Stub_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5135
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5136
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Body_Stub_P_Type_Expression;
--# end
--# property-start SubpBodyStub._env_mappings_212 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22105
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_212
  (Node : Bare_Subp_Body_Stub
  )
   return Internal_Env_Assoc
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1739 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1739 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1739;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5139 '<New EnvAssoc at ast.py:22105>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22105
--# expr-start 5137 '<New DesignatedEnv at ast.py:22105>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22105
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5137
--# expr-start 5138 '<FieldAccess for name_symbol at ast.py:22106>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22106
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 5138
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5139
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1739;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1739;
            raise;
      end;
   return Property_Result;
end Env_Mappings_212;
--# end
--# property-start SubpBodyStub._env_trans_parent_213 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22104
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_213
  (Node : Bare_Subp_Body_Stub
  )
   return Boolean
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_213;
--# end
--# property-start SubpBodyStub._env_names_214 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22109
pragma Warnings (Off, "is not referenced");
 function Env_Names_214
  (Node : Bare_Subp_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1741 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1741 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1741;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5140 '<FieldAccess for env_names at ast.py:22109>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22109
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 5140
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1741;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1741;
            raise;
      end;
   return Property_Result;
end Env_Names_214;
--# end
--# property-start SubpBodyStub._ref_env_nodes_215 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22112
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_215
  (Node : Bare_Subp_Body_Stub
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1742 with Inline_Always;
            procedure Finalizer_Scope_2590 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1742 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1742;
            procedure Finalizer_Scope_2590 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2590;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5141 '<Cast to AdaNode at ast.py:22112>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22112
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5141
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2590;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1742;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1742;
                     Finalizer_Scope_2590;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_215;
--# end
--# property-start SubpBodyStub._ref_cond_216 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22114
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_216
  (Node : Bare_Subp_Body_Stub
  )
   return Boolean
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5142 '<FieldAccess for should_ref_generic_formals at ast.py:22114>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22114
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5142
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_216;
--# end
--# property-start SubpBodyStub._ref_env_nodes_217 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22122
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_217
  (Node : Bare_Subp_Body_Stub
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1744 with Inline_Always;
            procedure Finalizer_Scope_2591 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1744 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1744;
            procedure Finalizer_Scope_2591 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2591;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5143 '<Cast to AdaNode at ast.py:22122>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22122
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5143
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2591;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1744;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1744;
                     Finalizer_Scope_2591;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_217;
--# end
--# property-start SubpBodyStub._ref_cond_218 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22124
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_218
  (Node : Bare_Subp_Body_Stub
  )
   return Boolean
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5144 '<FieldAccess for should_ref_generic_formals at ast.py:22124>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22124
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5144
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_218;
--# end
   --
   --  Primitives for Bare_Task_Body_Stub
   --
      procedure Task_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Task_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_222 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:22159");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_223 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_224 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Task_Body_Stub
        (Self : Bare_Task_Body_Stub
         ; Task_Body_Stub_F_Name : Bare_Defining_Name
         ; Task_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Task_Body_Stub_F_Name := Task_Body_Stub_F_Name;
            Self.Task_Body_Stub_F_Aspects := Task_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Task_Body_Stub;
   function Task_Body_Stub_F_Name
     (Node : Bare_Task_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Task_Body_Stub_F_Name;
   end;
--# property-start TaskBodyStub.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22156
pragma Warnings (Off, "is not referenced");
 function Task_Body_Stub_P_Defining_Names
  (Node : Bare_Task_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Task_Body_Stub  := Bare_Task_Body_Stub (Node);
     Ent : Internal_Entity_Task_Body_Stub :=
       Internal_Entity_Task_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1508 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1508 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1508;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5145 '<FieldAccess for name at ast.py:22156>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22156
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Task_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 5145
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1508;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1508;
            raise;
      end;
   return Property_Result;
end Task_Body_Stub_P_Defining_Names;
--# end
--# property-start TaskBodyStub._env_mappings_222 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22159
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_222
  (Node : Bare_Task_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Body_Stub  := Bare_Task_Body_Stub (Node);
     Ent : Internal_Entity_Task_Body_Stub :=
       Internal_Entity_Task_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1749 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1749 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1749;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5146 '<FieldAccess for previous_part_link_env_assoc at ast.py:22159>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22159
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5146
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1749;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1749;
            raise;
      end;
   return Property_Result;
end Env_Mappings_222;
--# end
--# property-start TaskBodyStub._env_trans_parent_223 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22158
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_223
  (Node : Bare_Task_Body_Stub
  )
   return Boolean
is
   Self : Bare_Task_Body_Stub  := Bare_Task_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_223;
--# end
--# property-start TaskBodyStub._env_names_224 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22160
pragma Warnings (Off, "is not referenced");
 function Env_Names_224
  (Node : Bare_Task_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Body_Stub  := Bare_Task_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1751 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1751 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1751;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5147 '<FieldAccess for env_names at ast.py:22160>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22160
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 5147
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1751;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1751;
            raise;
      end;
   return Property_Result;
end Env_Names_224;
--# end
   --
   --  Primitives for Bare_Entry_Body
   --
      procedure Entry_Body_Pre_Env_Actions
        (Self            : Bare_Entry_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_204 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_205 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:21909");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_206 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:21919");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_207 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Body
        (Self : Bare_Entry_Body
         ; Entry_Body_F_Entry_Name : Bare_Defining_Name
         ; Entry_Body_F_Index_Spec : Bare_Entry_Index_Spec
         ; Entry_Body_F_Params : Bare_Entry_Completion_Formal_Params
         ; Entry_Body_F_Aspects : Bare_Aspect_Spec
         ; Entry_Body_F_Barrier : Bare_Expr
         ; Entry_Body_F_Decls : Bare_Declarative_Part
         ; Entry_Body_F_Stmts : Bare_Handled_Stmts
         ; Entry_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Entry_Body_F_Entry_Name := Entry_Body_F_Entry_Name;
            Self.Entry_Body_F_Index_Spec := Entry_Body_F_Index_Spec;
            Self.Entry_Body_F_Params := Entry_Body_F_Params;
            Self.Entry_Body_F_Aspects := Entry_Body_F_Aspects;
            Self.Entry_Body_F_Barrier := Entry_Body_F_Barrier;
            Self.Entry_Body_F_Decls := Entry_Body_F_Decls;
            Self.Entry_Body_F_Stmts := Entry_Body_F_Stmts;
            Self.Entry_Body_F_End_Name := Entry_Body_F_End_Name;
      end Initialize_Fields_For_Entry_Body;
   function Entry_Body_F_Entry_Name
     (Node : Bare_Entry_Body) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Body_F_Entry_Name;
   end;
   function Entry_Body_F_Index_Spec
     (Node : Bare_Entry_Body) return Bare_Entry_Index_Spec
   is
   begin
         return Node.Entry_Body_F_Index_Spec;
   end;
   function Entry_Body_F_Params
     (Node : Bare_Entry_Body) return Bare_Entry_Completion_Formal_Params
   is
   begin
         return Node.Entry_Body_F_Params;
   end;
   function Entry_Body_F_Barrier
     (Node : Bare_Entry_Body) return Bare_Expr
   is
   begin
         return Node.Entry_Body_F_Barrier;
   end;
   function Entry_Body_F_Decls
     (Node : Bare_Entry_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Entry_Body_F_Decls;
   end;
   function Entry_Body_F_Stmts
     (Node : Bare_Entry_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Entry_Body_F_Stmts;
   end;
   function Entry_Body_F_End_Name
     (Node : Bare_Entry_Body) return Bare_End_Name
   is
   begin
         return Node.Entry_Body_F_End_Name;
   end;
--# property-start EntryBody.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21902
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Defining_Names
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1489 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1489 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1489;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5148 '<FieldAccess for entry_name at ast.py:21902>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21902
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Body_F_Entry_Name, Info => Ent.Info);
--# expr-done 5148
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1489;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1489;
            raise;
      end;
   return Property_Result;
end Entry_Body_P_Defining_Names;
--# end
--# property-start EntryBody.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21904
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Declarative_Parts
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1490 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1490 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1490;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5149 '<FieldAccess for decls at ast.py:21904>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21904
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Entry_Body_F_Decls, Info => Ent.Info);
--# expr-done 5149
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1490;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1490;
            raise;
      end;
   return Property_Result;
end Entry_Body_P_Declarative_Parts;
--# end
--# property-start EntryBody.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21926
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Xref_Entry_Point
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Body_P_Xref_Entry_Point;
--# end
--# property-start EntryBody.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21928
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Xref_Equation
  (Node : Bare_Entry_Body
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1492 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1492 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1492;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5160 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5159 '<And at ast.py:21930>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21930
--# expr-start 5156 '<And at ast.py:21930>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21930
--# expr-start 5153 '<Bind at ast.py:21931>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21931
--# expr-start 5151 '<FieldAccess for expected_type_var at ast.py:21931>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21931
--# expr-start 5150 '<FieldAccess for barrier at ast.py:21931>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21931
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Entry_Body_F_Barrier;
--# expr-done 5150
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 5151
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 5152 '<FieldAccess for bool_type at ast.py:21931>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21931
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 5152
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21931") else null)); 
--# expr-done 5153
--# expr-start 5155 '<FieldAccess for sub_equation at ast.py:21932>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21932
--# expr-start 5154 '<FieldAccess for barrier at ast.py:21932>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21932
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Entry_Body_F_Barrier, Info => Ent.Info);
--# expr-done 5154
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 5155
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21930") else null)); 
--# expr-done 5156
--# expr-start 5158 '<FieldAccess for matches_expected_formal_prim_type at ast.py:21933>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21933
--# expr-start 5157 '<FieldAccess for barrier at ast.py:21933>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21933
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Entry_Body_F_Barrier, Info => Ent.Info);
--# expr-done 5157
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 5158
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21930") else null)); 
--# expr-done 5159
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5160
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1492;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1492;
            raise;
      end;
   return Property_Result;
end Entry_Body_P_Xref_Equation;
--# end
--# property-start EntryBody._env_do_204 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21907
pragma Warnings (Off, "is not referenced");
 function Env_Do_204
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5161 '<FieldAccess for env_hook at ast.py:21907>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21907
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5161
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_204;
--# end
--# property-start EntryBody._initial_env_205 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21910
pragma Warnings (Off, "is not referenced");
 function Initial_Env_205
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Designated_Env
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1732 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
New_Struct : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1732 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1732;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5165 '<New DesignatedEnv at ast.py:21910>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21910
--# expr-start 5164 '<bind at ast.py:21911>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21911
--# expr-start 5162 '<FieldAccess for default_initial_env at ast.py:21912>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21912
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 5162
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5163 '<FieldAccess for body_scope at ast.py:21913>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21913
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 5163
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5164
Inc_Ref (Dyn_Var_Bind_Result);
New_Struct := (Direct_Env => Dyn_Var_Bind_Result, Env_Name => null, Kind => Direct_Env); 
--# expr-done 5165
         Property_Result := New_Struct;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1732;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1732;
            raise;
      end;
   return Property_Result;
end Initial_Env_205;
--# end
--# property-start EntryBody._env_mappings_206 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21920
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_206
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1733 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Entry_Body;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Singleton : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1733 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1733;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5168 '<New EnvAssoc at ast.py:21920>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21920
--# expr-start 5166 '<New DesignatedEnv at ast.py:21920>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21920
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5166
--# expr-start 5167 '<FieldAccess for name_symbol at ast.py:21920>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21920
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5167
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5168
                Singleton := Create_Internal_Env_Assoc_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct_1;
                Inc_Ref (Singleton.Items (1));
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1733;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1733;
            raise;
      end;
   return Property_Result;
end Env_Mappings_206;
--# end
--# property-start EntryBody._env_trans_parent_207 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21906
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_207
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_207;
--# end
   --
   --  Primitives for Bare_Package_Body
   --
      procedure Package_Body_Pre_Env_Actions
        (Self            : Bare_Package_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_153 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_154 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:21643");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_155 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:21645");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_156 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_157 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_159 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_158 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_161 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_160 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_163 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_162 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_165 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_164 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_167 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_166 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_169 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_168 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Body_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_171 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_170 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Package_Body_P_Package_Decl_Uses_Clauses_Envs'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Body
        (Self : Bare_Package_Body
         ; Package_Body_F_Package_Name : Bare_Defining_Name
         ; Package_Body_F_Aspects : Bare_Aspect_Spec
         ; Package_Body_F_Decls : Bare_Declarative_Part
         ; Package_Body_F_Stmts : Bare_Handled_Stmts
         ; Package_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Package_Body_F_Package_Name := Package_Body_F_Package_Name;
            Self.Package_Body_F_Aspects := Package_Body_F_Aspects;
            Self.Package_Body_F_Decls := Package_Body_F_Decls;
            Self.Package_Body_F_Stmts := Package_Body_F_Stmts;
            Self.Package_Body_F_End_Name := Package_Body_F_End_Name;
      end Initialize_Fields_For_Package_Body;
   function Package_Body_F_Package_Name
     (Node : Bare_Package_Body) return Bare_Defining_Name
   is
   begin
         return Node.Package_Body_F_Package_Name;
   end;
   function Package_Body_F_Decls
     (Node : Bare_Package_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Package_Body_F_Decls;
   end;
   function Package_Body_F_Stmts
     (Node : Bare_Package_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Package_Body_F_Stmts;
   end;
   function Package_Body_F_End_Name
     (Node : Bare_Package_Body) return Bare_End_Name
   is
   begin
         return Node.Package_Body_F_End_Name;
   end;
--# property-start PackageBody.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21721
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Defining_Names
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1478 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1478 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1478;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5169 '<FieldAccess for package_name at ast.py:21721>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21721
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Body_F_Package_Name, Info => Ent.Info);
--# expr-done 5169
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1478;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1478;
            raise;
      end;
   return Property_Result;
end Package_Body_P_Defining_Names;
--# end
--# property-start PackageBody.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21722
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Defining_Env
  (Node : Bare_Package_Body
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1479 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1479 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1479;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5170 '<FieldAccess for children_env at ast.py:21722>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21722
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5170
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1479;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1479;
            raise;
      end;
   return Property_Result;
end Package_Body_P_Defining_Env;
--# end
--# property-start PackageBody.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21724
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Declarative_Parts
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1480 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1480 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1480;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5171 '<FieldAccess for decls at ast.py:21724>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21724
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Package_Body_F_Decls, Info => Ent.Info);
--# expr-done 5171
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1480;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1480;
            raise;
      end;
   return Property_Result;
end Package_Body_P_Declarative_Parts;
--# end
--# property-start PackageBody.package_decl_uses_clauses_envs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21726
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1481 with Inline_Always;
            procedure Finalizer_Scope_2593 with Inline_Always;
      Block_Var : Internal_Entity_Base_Package_Decl;
Var_Expr : Internal_Entity_Private_Part;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Package_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Package_Decl;
Fld_1 : Internal_Entity_Public_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Private_Part;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1481 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1481;
            procedure Finalizer_Scope_2593 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2593;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5181 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5174 '<bind at ast.py:21732>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21732
Bound_Imprecise_Fallback := False; 
--# expr-start 5173 '<Cast to BasePackageDecl at ast.py:21733>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21733
--# expr-start 5172 '<FieldAccess for decl_part at ast.py:21733>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21733
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5172
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5173
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 5174
Block_Var := Dyn_Var_Bind_Result; 
--# bind pd Block_Var
--# expr-start 5180 '<env_group at ast.py:21737>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21737
--# expr-start 5179 '<ArrayLiteral at ast.py:21736>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21736
--# expr-start 5176 '<FieldAccess for use_clauses_envs at ast.py:21736>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21736
--# expr-start 5175 '<FieldAccess for public_part at ast.py:21736>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21736
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Block_Var.Node.Base_Package_Decl_F_Public_Part, Info => Block_Var.Info);
--# expr-done 5175
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.use_clauses_envs
Fld_2 := Libadalang.Implementation.Declarative_Part_P_Use_Clauses_Envs (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 5176
--# expr-start 5177 '<FieldAccess for private_part at ast.py:21737>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21737
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Private_Part (Node => Block_Var.Node.Base_Package_Decl_F_Private_Part, Info => Block_Var.Info);
--# expr-done 5177
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 5178 '<FieldAccess for use_clauses_envs at ast.py:21737>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21737
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.use_clauses_envs
Fld_4 := Libadalang.Implementation.Declarative_Part_P_Use_Clauses_Envs (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5178
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2593;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Result_Var)); 
--# expr-done 5179
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 5180
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 5181
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1481;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1481;
                     Finalizer_Scope_2593;
            raise;
      end;
   return Property_Result;
end Package_Body_P_Package_Decl_Uses_Clauses_Envs;
--# end
--# property-start PackageBody._env_do_153 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21640
pragma Warnings (Off, "is not referenced");
 function Env_Do_153
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5182 '<FieldAccess for env_hook at ast.py:21640>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21640
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5182
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_153;
--# end
--# property-start PackageBody._initial_env_154 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21643
pragma Warnings (Off, "is not referenced");
 function Initial_Env_154
  (Node : Bare_Package_Body
  )
   return Internal_Designated_Env
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1681 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1681 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1681;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5183 '<FieldAccess for body_initial_env at ast.py:21643>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21643
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env (Node => Self);
--# end
--# expr-done 5183
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1681;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1681;
            raise;
      end;
   return Property_Result;
end Initial_Env_154;
--# end
--# property-start PackageBody._env_mappings_155 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21645
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_155
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1682 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1682 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1682;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5184 '<FieldAccess for previous_part_link_env_assoc at ast.py:21645>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21645
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5184
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1682;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1682;
            raise;
      end;
   return Property_Result;
end Env_Mappings_155;
--# end
--# property-start PackageBody._env_trans_parent_156 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21649
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_156
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5185 '<FieldAccess for is_library_item at ast.py:21649>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21649
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 5185
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_156;
--# end
--# property-start PackageBody._env_do_157 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21651
pragma Warnings (Off, "is not referenced");
 function Env_Do_157
  (Node : Bare_Package_Body
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1684 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1684 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1684;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5186 '<FieldAccess for populate_dependent_units at ast.py:21651>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21651
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5186
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1684;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1684;
            raise;
      end;
   return Property_Result;
end Env_Do_157;
--# end
--# property-start PackageBody._ref_env_nodes_158 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21654
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_158
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1685 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1685 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1685;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5187 '<FieldAccess for top_level_use_package_clauses at ast.py:21654>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21654
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5187
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1685;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1685;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_158;
--# end
--# property-start PackageBody._ref_cond_159 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21656
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_159
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5189 '<IsA LibraryItem, Subunit at ast.py:21656>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21656
--# expr-start 5188 '<parent at ast.py:21656>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21656
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5188
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5189
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_159;
--# end
--# property-start PackageBody._ref_env_nodes_160 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21660
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_160
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1687 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1687 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1687;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5190 '<FieldAccess for top_level_use_type_clauses at ast.py:21660>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21660
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5190
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1687;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1687;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_160;
--# end
--# property-start PackageBody._ref_cond_161 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21662
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_161
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5192 '<IsA LibraryItem, Subunit at ast.py:21662>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21662
--# expr-start 5191 '<parent at ast.py:21662>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21662
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5191
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5192
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_161;
--# end
--# property-start PackageBody._ref_env_nodes_162 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21666
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_162
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1689 with Inline_Always;
            procedure Finalizer_Scope_2594 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1689 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1689;
            procedure Finalizer_Scope_2594 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2594;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5193 '<Cast to AdaNode at ast.py:21666>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21666
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5193
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2594;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1689;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1689;
                     Finalizer_Scope_2594;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_162;
--# end
--# property-start PackageBody._ref_cond_163 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21668
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_163
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5194 '<FieldAccess for should_ref_generic_formals at ast.py:21668>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21668
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5194
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_163;
--# end
--# property-start PackageBody._ref_env_nodes_164 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21676
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_164
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1691 with Inline_Always;
            procedure Finalizer_Scope_2595 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1691 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1691;
            procedure Finalizer_Scope_2595 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2595;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5195 '<Cast to AdaNode at ast.py:21676>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21676
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5195
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2595;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1691;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1691;
                     Finalizer_Scope_2595;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_164;
--# end
--# property-start PackageBody._ref_cond_165 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21678
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_165
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5196 '<FieldAccess for should_ref_generic_formals at ast.py:21678>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21678
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5196
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_165;
--# end
--# property-start PackageBody._ref_env_nodes_166 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21689
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_166
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1693 with Inline_Always;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1693 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1693;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5197 '<Cast to AdaNode at ast.py:21689>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21689
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5197
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1693;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1693;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_166;
--# end
--# property-start PackageBody._ref_cond_167 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21691
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_167
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5198 '<FieldAccess for is_subunit at ast.py:21691>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21691
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5198
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_167;
--# end
--# property-start PackageBody._ref_env_nodes_168 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21703
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_168
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1695 with Inline_Always;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1695 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1695;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5199 '<Cast to AdaNode at ast.py:21703>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21703
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5199
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1695;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1695;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_168;
--# end
--# property-start PackageBody._ref_cond_169 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21705
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_169
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5201 '<Not at ast.py:21705>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21705
--# expr-start 5200 '<FieldAccess for is_compilation_unit_root at ast.py:21705>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21705
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 5200
Not_Val := not (Fld); 
--# expr-done 5201
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_169;
--# end
--# property-start PackageBody._ref_env_nodes_170 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21710
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_170
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1697 with Inline_Always;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1697 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1697;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5202 '<Cast to AdaNode at ast.py:21710>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21710
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5202
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1697;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1697;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_170;
--# end
--# property-start PackageBody._ref_cond_171 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21712
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_171
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5204 '<Not at ast.py:21712>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21712
--# expr-start 5203 '<FieldAccess for is_compilation_unit_root at ast.py:21712>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21712
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 5203
Not_Val := not (Fld); 
--# expr-done 5204
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_171;
--# end
function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Package_Body_Range
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Package_Body_P_Package_Decl_Uses_Clauses_Envs (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Protected_Body
   --
      procedure Protected_Body_Pre_Env_Actions
        (Self            : Bare_Protected_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_188 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_189 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:21831");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_190 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:21833");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_191 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_192 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_194 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_193 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_196 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_195 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_198 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_197 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_200 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_199 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_201 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Body_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      if Libadalang.Implementation.Ref_Cond_203 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_202 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Protected_Body
        (Self : Bare_Protected_Body
         ; Protected_Body_F_Name : Bare_Defining_Name
         ; Protected_Body_F_Aspects : Bare_Aspect_Spec
         ; Protected_Body_F_Decls : Bare_Declarative_Part
         ; Protected_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Protected_Body_F_Name := Protected_Body_F_Name;
            Self.Protected_Body_F_Aspects := Protected_Body_F_Aspects;
            Self.Protected_Body_F_Decls := Protected_Body_F_Decls;
            Self.Protected_Body_F_End_Name := Protected_Body_F_End_Name;
      end Initialize_Fields_For_Protected_Body;
   function Protected_Body_F_Name
     (Node : Bare_Protected_Body) return Bare_Defining_Name
   is
   begin
         return Node.Protected_Body_F_Name;
   end;
   function Protected_Body_F_Decls
     (Node : Bare_Protected_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Protected_Body_F_Decls;
   end;
   function Protected_Body_F_End_Name
     (Node : Bare_Protected_Body) return Bare_End_Name
   is
   begin
         return Node.Protected_Body_F_End_Name;
   end;
--# property-start ProtectedBody.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21882
pragma Warnings (Off, "is not referenced");
 function Protected_Body_P_Defining_Names
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
     Ent : Internal_Entity_Protected_Body :=
       Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1487 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1487 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1487;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5205 '<FieldAccess for name at ast.py:21882>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21882
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Protected_Body_F_Name, Info => Ent.Info);
--# expr-done 5205
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1487;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1487;
            raise;
      end;
   return Property_Result;
end Protected_Body_P_Defining_Names;
--# end
--# property-start ProtectedBody.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21884
pragma Warnings (Off, "is not referenced");
 function Protected_Body_P_Declarative_Parts
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
     Ent : Internal_Entity_Protected_Body :=
       Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1488 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1488 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1488;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5206 '<FieldAccess for decls at ast.py:21884>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21884
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Protected_Body_F_Decls, Info => Ent.Info);
--# expr-done 5206
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1488;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1488;
            raise;
      end;
   return Property_Result;
end Protected_Body_P_Declarative_Parts;
--# end
--# property-start ProtectedBody._env_do_188 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21829
pragma Warnings (Off, "is not referenced");
 function Env_Do_188
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5207 '<FieldAccess for env_hook at ast.py:21829>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21829
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5207
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_188;
--# end
--# property-start ProtectedBody._initial_env_189 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21831
pragma Warnings (Off, "is not referenced");
 function Initial_Env_189
  (Node : Bare_Protected_Body
  )
   return Internal_Designated_Env
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1716 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1716 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1716;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5208 '<FieldAccess for body_initial_env at ast.py:21831>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21831
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env (Node => Self);
--# end
--# expr-done 5208
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1716;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1716;
            raise;
      end;
   return Property_Result;
end Initial_Env_189;
--# end
--# property-start ProtectedBody._env_mappings_190 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21833
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_190
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
     Ent : Internal_Entity_Protected_Body :=
       Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1717 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1717 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1717;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5209 '<FieldAccess for previous_part_link_env_assoc at ast.py:21833>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21833
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5209
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1717;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1717;
            raise;
      end;
   return Property_Result;
end Env_Mappings_190;
--# end
--# property-start ProtectedBody._env_trans_parent_191 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21828
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_191
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_191;
--# end
--# property-start ProtectedBody._env_do_192 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21837
pragma Warnings (Off, "is not referenced");
 function Env_Do_192
  (Node : Bare_Protected_Body
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1719 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1719 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1719;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5210 '<FieldAccess for populate_dependent_units at ast.py:21837>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21837
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5210
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1719;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1719;
            raise;
      end;
   return Property_Result;
end Env_Do_192;
--# end
--# property-start ProtectedBody._ref_env_nodes_193 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21840
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_193
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1720 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1720 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1720;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5211 '<FieldAccess for top_level_use_package_clauses at ast.py:21840>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21840
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5211
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1720;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1720;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_193;
--# end
--# property-start ProtectedBody._ref_cond_194 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21842
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_194
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5213 '<IsA LibraryItem, Subunit at ast.py:21842>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21842
--# expr-start 5212 '<parent at ast.py:21842>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21842
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5212
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5213
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_194;
--# end
--# property-start ProtectedBody._ref_env_nodes_195 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21845
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_195
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1722 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1722 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1722;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5214 '<FieldAccess for top_level_use_type_clauses at ast.py:21845>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21845
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5214
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1722;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1722;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_195;
--# end
--# property-start ProtectedBody._ref_cond_196 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21847
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_196
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5216 '<IsA LibraryItem, Subunit at ast.py:21847>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21847
--# expr-start 5215 '<parent at ast.py:21847>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21847
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5215
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5216
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_196;
--# end
--# property-start ProtectedBody._ref_env_nodes_197 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21850
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_197
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1724 with Inline_Always;
            procedure Finalizer_Scope_2596 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1724 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1724;
            procedure Finalizer_Scope_2596 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2596;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5217 '<Cast to AdaNode at ast.py:21850>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21850
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5217
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2596;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1724;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1724;
                     Finalizer_Scope_2596;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_197;
--# end
--# property-start ProtectedBody._ref_cond_198 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21852
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_198
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5218 '<FieldAccess for should_ref_generic_formals at ast.py:21852>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21852
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5218
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_198;
--# end
--# property-start ProtectedBody._ref_env_nodes_199 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21860
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_199
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1726 with Inline_Always;
            procedure Finalizer_Scope_2597 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1726 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1726;
            procedure Finalizer_Scope_2597 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2597;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5219 '<Cast to AdaNode at ast.py:21860>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21860
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5219
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2597;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1726;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1726;
                     Finalizer_Scope_2597;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_199;
--# end
--# property-start ProtectedBody._ref_cond_200 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21862
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_200
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5220 '<FieldAccess for should_ref_generic_formals at ast.py:21862>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21862
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5220
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_200;
--# end
--# property-start ProtectedBody._ref_env_nodes_201 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21866
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_201
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1728 with Inline_Always;
      Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1728 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1728;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5221 '<Cast to AdaNode at ast.py:21866>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21866
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5221
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1728;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1728;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_201;
--# end
--# property-start ProtectedBody._ref_env_nodes_202 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21871
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_202
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1729 with Inline_Always;
      Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1729 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1729;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5222 '<Cast to AdaNode at ast.py:21871>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21871
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5222
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1729;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1729;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_202;
--# end
--# property-start ProtectedBody._ref_cond_203 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21873
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_203
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5223 '<FieldAccess for is_subunit at ast.py:21873>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21873
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5223
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_203;
--# end
   --
   --  Primitives for Bare_Task_Body
   --
      procedure Task_Body_Pre_Env_Actions
        (Self            : Bare_Task_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_172 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_173 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:21759");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_174 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:21761");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_175 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_176 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_178 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_177 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_180 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_179 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_182 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_181 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_184 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_183 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_185 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Task_Body_P_Task_Type_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      if Libadalang.Implementation.Ref_Cond_187 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_186 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Task_Body
        (Self : Bare_Task_Body
         ; Task_Body_F_Name : Bare_Defining_Name
         ; Task_Body_F_Aspects : Bare_Aspect_Spec
         ; Task_Body_F_Decls : Bare_Declarative_Part
         ; Task_Body_F_Stmts : Bare_Handled_Stmts
         ; Task_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Task_Body_F_Name := Task_Body_F_Name;
            Self.Task_Body_F_Aspects := Task_Body_F_Aspects;
            Self.Task_Body_F_Decls := Task_Body_F_Decls;
            Self.Task_Body_F_Stmts := Task_Body_F_Stmts;
            Self.Task_Body_F_End_Name := Task_Body_F_End_Name;
      end Initialize_Fields_For_Task_Body;
   function Task_Body_F_Name
     (Node : Bare_Task_Body) return Bare_Defining_Name
   is
   begin
         return Node.Task_Body_F_Name;
   end;
   function Task_Body_F_Decls
     (Node : Bare_Task_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Task_Body_F_Decls;
   end;
   function Task_Body_F_Stmts
     (Node : Bare_Task_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Task_Body_F_Stmts;
   end;
   function Task_Body_F_End_Name
     (Node : Bare_Task_Body) return Bare_End_Name
   is
   begin
         return Node.Task_Body_F_End_Name;
   end;
--# property-start TaskBody.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21751
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Defining_Names
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1482 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1482 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1482;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5224 '<FieldAccess for name at ast.py:21751>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21751
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Task_Body_F_Name, Info => Ent.Info);
--# expr-done 5224
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1482;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1482;
            raise;
      end;
   return Property_Result;
end Task_Body_P_Defining_Names;
--# end
--# property-start TaskBody.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21752
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Defining_Env
  (Node : Bare_Task_Body
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1483 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1483 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1483;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5225 '<FieldAccess for children_env at ast.py:21752>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21752
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5225
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1483;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1483;
            raise;
      end;
   return Property_Result;
end Task_Body_P_Defining_Env;
--# end
--# property-start TaskBody.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21754
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Declarative_Parts
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1484 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1484 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1484;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5226 '<FieldAccess for decls at ast.py:21754>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21754
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Task_Body_F_Decls, Info => Ent.Info);
--# expr-done 5226
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1484;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1484;
            raise;
      end;
   return Property_Result;
end Task_Body_P_Declarative_Parts;
--# end
--# property-start TaskBody.task_type_decl_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21807
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Task_Type_Decl_Scope
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1485 with Inline_Always;
            procedure Finalizer_Scope_2603 with Inline_Always;
      Var_Expr : Internal_Entity_Private_Part;
Var_Expr_1 : Internal_Entity_Task_Def;
Var_Expr_2 : Internal_Entity_Task_Type_Decl;
Fld : Internal_Entity_Task_Type_Decl;
Fld_1 : Internal_Entity_Task_Def;
Scope_Result : Internal_Entity_Task_Def;
Result_Var : Internal_Entity_Task_Def;
Fld_2 : Internal_Entity_Private_Part;
Scope_Result_1 : Internal_Entity_Private_Part;
Result_Var_1 : Internal_Entity_Private_Part;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Task_Type_Decl;
Fld_5 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1485 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_1485;
            procedure Finalizer_Scope_2603 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2603;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5227 '<FieldAccess for task_type at ast.py:21808>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21808
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskBody.task_type
Fld := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5227
Var_Expr_2 := Fld;
if Var_Expr_2 /= No_Entity_Task_Type_Decl then
   --# scope-start
--# expr-start 5228 '<FieldAccess for definition at ast.py:21808>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21808
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Task_Def (Node => Var_Expr_2.Node.Task_Type_Decl_F_Definition, Info => Var_Expr_2.Info);
--# expr-done 5228
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Task_Def;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Task_Def then
   --# scope-start
--# expr-start 5229 '<FieldAccess for private_part at ast.py:21808>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21808
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Var_Expr_1.Node.Task_Def_F_Private_Part, Info => Var_Expr_1.Info);
--# expr-done 5229
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Private_Part;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Entity_Private_Part then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 5230 '<FieldAccess for children_env at ast.py:21809>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21809
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 5230
Scope_Result_2 := Fld_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2603;
   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 5232 '<FieldAccess for children_env at ast.py:21810>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21810
--# expr-start 5231 '<FieldAccess for task_type at ast.py:21810>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21810
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskBody.task_type
Fld_4 := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5231
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 5232
   Result_Var_2 := Fld_5;
end if;
      Inc_Ref (Result_Var_2);
         Property_Result := Result_Var_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1485;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1485;
                     Finalizer_Scope_2603;
            raise;
      end;
   return Property_Result;
end Task_Body_P_Task_Type_Decl_Scope;
--# end
--# property-start TaskBody.task_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21814
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Task_Type
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Task_Type_Decl
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Task_Type_Decl;
      Match_0 : Internal_Entity_Task_Type_Decl;
Match_1 : Internal_Entity_Single_Task_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Single_Task_Type_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Task_Type_Decl;
Let_Result : Internal_Entity_Task_Type_Decl;
Scope_Result : Internal_Entity_Task_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Decl;
Cast_Result_2 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Task_Type_Decl;
Scope_Result_1 : Internal_Entity_Task_Type_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Task_Type_Decl;
Scope_Result_2 : Internal_Entity_Task_Type_Decl;
Match_Result : Internal_Entity_Task_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Task_Type_Decl;
Let_Result_3 : Internal_Entity_Task_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5237 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 5236 '<bind at ast.py:21816>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21816
Bound_Imprecise_Fallback := False; 
--# expr-start 5235 '<Match at ast.py:21816>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21816
--# expr-start 5233 '<FieldAccess for decl_part at ast.py:21816>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21816
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5233
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
--# expr-start 5234 '<FieldAccess for task_type at ast.py:21818>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21818
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Single_Task_Type_Decl (Node => Match_1.Node.Single_Task_Decl_F_Task_Type, Info => Match_1.Info);
--# expr-done 5234
      Cast_Result_2 := Create_Internal_Entity_Task_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Protected_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Should not happen"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5235
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 5236
Let_Result_3 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 5237
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Task_Body_P_Task_Type;
--# end
--# property-start TaskBody._env_do_172 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21757
pragma Warnings (Off, "is not referenced");
 function Env_Do_172
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5238 '<FieldAccess for env_hook at ast.py:21757>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21757
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5238
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_172;
--# end
--# property-start TaskBody._initial_env_173 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21759
pragma Warnings (Off, "is not referenced");
 function Initial_Env_173
  (Node : Bare_Task_Body
  )
   return Internal_Designated_Env
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1700 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1700 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1700;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5239 '<FieldAccess for body_initial_env at ast.py:21759>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21759
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env (Node => Self);
--# end
--# expr-done 5239
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1700;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1700;
            raise;
      end;
   return Property_Result;
end Initial_Env_173;
--# end
--# property-start TaskBody._env_mappings_174 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21761
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_174
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1701 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1701 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1701;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5240 '<FieldAccess for previous_part_link_env_assoc at ast.py:21761>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21761
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5240
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1701;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1701;
            raise;
      end;
   return Property_Result;
end Env_Mappings_174;
--# end
--# property-start TaskBody._env_trans_parent_175 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21756
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_175
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_175;
--# end
--# property-start TaskBody._env_do_176 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21765
pragma Warnings (Off, "is not referenced");
 function Env_Do_176
  (Node : Bare_Task_Body
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1703 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1703 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1703;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5241 '<FieldAccess for populate_dependent_units at ast.py:21765>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21765
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5241
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1703;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1703;
            raise;
      end;
   return Property_Result;
end Env_Do_176;
--# end
--# property-start TaskBody._ref_env_nodes_177 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21768
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_177
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1704 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1704 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1704;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5242 '<FieldAccess for top_level_use_package_clauses at ast.py:21768>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21768
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5242
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1704;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1704;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_177;
--# end
--# property-start TaskBody._ref_cond_178 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21770
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_178
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5244 '<IsA LibraryItem, Subunit at ast.py:21770>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21770
--# expr-start 5243 '<parent at ast.py:21770>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21770
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5243
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5244
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_178;
--# end
--# property-start TaskBody._ref_env_nodes_179 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21774
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_179
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1706 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1706 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1706;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5245 '<FieldAccess for top_level_use_type_clauses at ast.py:21774>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21774
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5245
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1706;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1706;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_179;
--# end
--# property-start TaskBody._ref_cond_180 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21776
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_180
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5247 '<IsA LibraryItem, Subunit at ast.py:21776>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21776
--# expr-start 5246 '<parent at ast.py:21776>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21776
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5246
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5247
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_180;
--# end
--# property-start TaskBody._ref_env_nodes_181 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21780
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_181
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1708 with Inline_Always;
            procedure Finalizer_Scope_2604 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1708 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1708;
            procedure Finalizer_Scope_2604 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2604;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5248 '<Cast to AdaNode at ast.py:21780>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21780
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5248
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2604;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1708;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1708;
                     Finalizer_Scope_2604;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_181;
--# end
--# property-start TaskBody._ref_cond_182 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21782
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_182
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5249 '<FieldAccess for should_ref_generic_formals at ast.py:21782>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21782
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5249
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_182;
--# end
--# property-start TaskBody._ref_env_nodes_183 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21790
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_183
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1710 with Inline_Always;
            procedure Finalizer_Scope_2605 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1710 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1710;
            procedure Finalizer_Scope_2605 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2605;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5250 '<Cast to AdaNode at ast.py:21790>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21790
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5250
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2605;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1710;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1710;
                     Finalizer_Scope_2605;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_183;
--# end
--# property-start TaskBody._ref_cond_184 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21792
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_184
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5251 '<FieldAccess for should_ref_generic_formals at ast.py:21792>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21792
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5251
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_184;
--# end
--# property-start TaskBody._ref_env_nodes_185 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21796
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_185
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1712 with Inline_Always;
      Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1712 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1712;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5252 '<Cast to AdaNode at ast.py:21796>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21796
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5252
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1712;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1712;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_185;
--# end
--# property-start TaskBody._ref_env_nodes_186 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21801
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_186
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1713 with Inline_Always;
      Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1713 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1713;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5253 '<Cast to AdaNode at ast.py:21801>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21801
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5253
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1713;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1713;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_186;
--# end
--# property-start TaskBody._ref_cond_187 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21803
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_187
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5254 '<FieldAccess for is_subunit at ast.py:21803>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21803
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5254
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_187;
--# end
function Task_Body_P_Task_Type_Decl_Scope
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Task_Body_Range
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Task_Body_P_Task_Type_Decl_Scope (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Entry_Index_Spec
   --
      procedure Entry_Index_Spec_Pre_Env_Actions
        (Self            : Bare_Entry_Index_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_208 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:21946");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Index_Spec
        (Self : Bare_Entry_Index_Spec
         ; Entry_Index_Spec_F_Id : Bare_Defining_Name
         ; Entry_Index_Spec_F_Subtype : Bare_Ada_Node
         ; Entry_Index_Spec_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Entry_Index_Spec_F_Id := Entry_Index_Spec_F_Id;
            Self.Entry_Index_Spec_F_Subtype := Entry_Index_Spec_F_Subtype;
            Self.Entry_Index_Spec_F_Aspects := Entry_Index_Spec_F_Aspects;
      end Initialize_Fields_For_Entry_Index_Spec;
   function Entry_Index_Spec_F_Id
     (Node : Bare_Entry_Index_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Index_Spec_F_Id;
   end;
   function Entry_Index_Spec_F_Subtype
     (Node : Bare_Entry_Index_Spec) return Bare_Ada_Node
   is
   begin
         return Node.Entry_Index_Spec_F_Subtype;
   end;
--# property-start EntryIndexSpec.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21948
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Defining_Names
  (Node : Bare_Entry_Index_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1493 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1493 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1493;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5255 '<FieldAccess for id at ast.py:21948>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21948
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Index_Spec_F_Id, Info => Ent.Info);
--# expr-done 5255
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1493;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1493;
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Defining_Names;
--# end
--# property-start EntryIndexSpec.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21949
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Defining_Env
  (Node : Bare_Entry_Index_Spec
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1494 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1494 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1494;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5257 '<FieldAccess for defining_env at ast.py:21949>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21949
--# expr-start 5256 '<FieldAccess for expr_type at ast.py:21949>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21949
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5256
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5257
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1494;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1494;
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Defining_Env;
--# end
--# property-start EntryIndexSpec.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21950
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Expr_Type
  (Node : Bare_Entry_Index_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5262 '<Match at ast.py:21950>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21950
--# expr-start 5258 '<FieldAccess for subtype at ast.py:21950>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21950
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Index_Spec_F_Subtype, Info => Ent.Info);
--# expr-done 5258
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind subt Match_0
--# expr-start 5259 '<FieldAccess for designated_type at ast.py:21951>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21951
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 5259
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 5261 '<FieldAccess for expression_type at ast.py:21952>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21952
--# expr-start 5260 '<Cast to Expr at ast.py:21952>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21952
Cast_Expr := Match_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5260
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5261
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 5262
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Expr_Type;
--# end
--# property-start EntryIndexSpec.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21955
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Xref_Equation
  (Node : Bare_Entry_Index_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1496 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1496 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1496;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5265 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5264 '<FieldAccess for sub_equation at ast.py:21957>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21957
--# expr-start 5263 '<FieldAccess for subtype at ast.py:21957>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21957
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Index_Spec_F_Subtype, Info => Ent.Info);
--# expr-done 5263
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5264
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5265
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1496;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1496;
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Xref_Equation;
--# end
--# property-start EntryIndexSpec.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21959
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Xref_Entry_Point
  (Node : Bare_Entry_Index_Spec
  )
   return Boolean
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Xref_Entry_Point;
--# end
--# property-start EntryIndexSpec._env_mappings_208 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21946
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_208
  (Node : Bare_Entry_Index_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1735 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Entry_Index_Spec;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1735 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1735;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5268 '<New EnvAssoc at ast.py:21946>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21946
--# expr-start 5266 '<New DesignatedEnv at ast.py:21946>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21946
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5266
--# expr-start 5267 '<FieldAccess for name_symbol at ast.py:21946>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21946
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5267
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5268
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1735;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1735;
            raise;
      end;
   return Property_Result;
end Env_Mappings_208;
--# end
   --
   --  Primitives for Bare_Error_Decl
   --
--# property-start ErrorDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3579
pragma Warnings (Off, "is not referenced");
 function Error_Decl_P_Defining_Names
  (Node : Bare_Error_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Error_Decl  := Bare_Error_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Defining_Name_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Error_Decl_P_Defining_Names;
--# end
--# property-start ErrorDecl.child_decl_initial_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:3581
pragma Warnings (Off, "is not referenced");
 function Error_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Error_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Error_Decl  := Bare_Error_Decl (Node);
      --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
      Let_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5269 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := null; 
--# end
--# expr-done 5269
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Error_Decl_P_Child_Decl_Initial_Env_Name;
--# end
   --
   --  Primitives for Bare_Exception_Decl
   --
      procedure Exception_Decl_Pre_Env_Actions
        (Self            : Bare_Exception_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_58 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:11152");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Exception_Decl
        (Self : Bare_Exception_Decl
         ; Exception_Decl_F_Ids : Bare_Defining_Name_List
         ; Exception_Decl_F_Renames : Bare_Renaming_Clause
         ; Exception_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Exception_Decl_F_Ids := Exception_Decl_F_Ids;
            Self.Exception_Decl_F_Renames := Exception_Decl_F_Renames;
            Self.Exception_Decl_F_Aspects := Exception_Decl_F_Aspects;
      end Initialize_Fields_For_Exception_Decl;
   function Exception_Decl_F_Ids
     (Node : Bare_Exception_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Exception_Decl_F_Ids;
   end;
   function Exception_Decl_F_Renames
     (Node : Bare_Exception_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Exception_Decl_F_Renames;
   end;
--# property-start ExceptionDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11143
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Defining_Names
  (Node : Bare_Exception_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
     Ent : Internal_Entity_Exception_Decl :=
       Internal_Entity_Exception_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_863 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_180 : Internal_Entity_Defining_Name;
Bare_Item_180 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_863 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_863;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5271 '<Map at ast.py:11143>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11143
   --# expr-start 5270 '<FieldAccess for ids at ast.py:11143>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11143
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Exception_Decl_F_Ids, Info => Ent.Info);
--# expr-done 5270
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_180 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_180 := Untyped_Item_180; 
Is_Null := Bare_Item_180 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_180); 
   If_Result := As_Entity;
end if;
                  Item_180 := If_Result; 
   --# scope-start
               --# bind id Item_180
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_180;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5271
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_863;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_863;
            raise;
      end;
   return Property_Result;
end Exception_Decl_P_Defining_Names;
--# end
--# property-start ExceptionDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11145
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Next_Part_For_Decl
  (Node : Bare_Exception_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5272 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Entity_Basic_Decl; 
--# end
--# expr-done 5272
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Exception_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ExceptionDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11154
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Xref_Entry_Point
  (Node : Bare_Exception_Decl
  )
   return Boolean
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Exception_Decl_P_Xref_Entry_Point;
--# end
--# property-start ExceptionDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11156
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Xref_Equation
  (Node : Bare_Exception_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
     Ent : Internal_Entity_Exception_Decl :=
       Internal_Entity_Exception_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_866 with Inline_Always;
            procedure Finalizer_Scope_2609 with Inline_Always;
      Var_Expr : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_866 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_866;
            procedure Finalizer_Scope_2609 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2609;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5276 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5273 '<FieldAccess for renames at ast.py:11158>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11158
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Exception_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5273
Var_Expr := Fld;
if Var_Expr /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 5275 '<FieldAccess for xref_no_overloading at ast.py:11159>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11159
--# expr-start 5274 '<FieldAccess for renamed_object at ast.py:11159>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11159
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr.Info);
--# expr-done 5274
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 5275
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2609;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11160") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 5276
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_866;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_866;
                     Finalizer_Scope_2609;
            raise;
      end;
   return Property_Result;
end Exception_Decl_P_Xref_Equation;
--# end
--# property-start ExceptionDecl._env_mappings_58 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11152
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_58
  (Node : Bare_Exception_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1585 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Exception_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1585 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1585;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5278 '<FieldAccess for env_mappings at ast.py:11152>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11152
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5277 '<FieldAccess for ids at ast.py:11152>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11152
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Exception_Decl_F_Ids;
--# expr-done 5277
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 5278
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1585;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1585;
            raise;
      end;
   return Property_Result;
end Env_Mappings_58;
--# end
   --
   --  Primitives for Bare_Exception_Handler
   --
      procedure Exception_Handler_Pre_Env_Actions
        (Self            : Bare_Exception_Handler;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_143 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_144 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:20914");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Exception_Handler
        (Self : Bare_Exception_Handler
         ; Exception_Handler_F_Exception_Name : Bare_Defining_Name
         ; Exception_Handler_F_Handled_Exceptions : Bare_Alternatives_List
         ; Exception_Handler_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Exception_Handler_F_Exception_Name := Exception_Handler_F_Exception_Name;
            Self.Exception_Handler_F_Handled_Exceptions := Exception_Handler_F_Handled_Exceptions;
            Self.Exception_Handler_F_Stmts := Exception_Handler_F_Stmts;
      end Initialize_Fields_For_Exception_Handler;
   function Exception_Handler_F_Exception_Name
     (Node : Bare_Exception_Handler) return Bare_Defining_Name
   is
   begin
         return Node.Exception_Handler_F_Exception_Name;
   end;
   function Exception_Handler_F_Handled_Exceptions
     (Node : Bare_Exception_Handler) return Bare_Alternatives_List
   is
   begin
         return Node.Exception_Handler_F_Handled_Exceptions;
   end;
   function Exception_Handler_F_Stmts
     (Node : Bare_Exception_Handler) return Bare_Stmt_List
   is
   begin
         return Node.Exception_Handler_F_Stmts;
   end;
--# property-start ExceptionHandler.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20924
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Defining_Names
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
     Ent : Internal_Entity_Exception_Handler :=
       Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1432 with Inline_Always;
            procedure Finalizer_Scope_2610 with Inline_Always;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1432 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1432;
            procedure Finalizer_Scope_2610 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2610;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5279 '<FieldAccess for exception_name at ast.py:20925>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20925
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Exception_Handler_F_Exception_Name, Info => Ent.Info);
--# expr-done 5279
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind n Var_Expr
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2610;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1432;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1432;
                     Finalizer_Scope_2610;
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Defining_Names;
--# end
--# property-start ExceptionHandler.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20928
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Expr_Type
  (Node : Bare_Exception_Handler
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
     Ent : Internal_Entity_Exception_Handler :=
       Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1433 with Inline_Always;
            procedure Finalizer_Scope_2611 with Inline_Always;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1433 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1433;
            procedure Finalizer_Scope_2611 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2611;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5288 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5287 '<Cast to BaseTypeDecl at ast.py:20934>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20934
--# expr-start 5286 '<EnvGet at ast.py:20933>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20933
--# expr-start 5283 '<FieldAccess for get_unit_root_decl at ast.py:20931>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20931
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5282 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 5280 '<Symbol Ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 5280
--# expr-start 5281 '<Symbol Exceptions at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exceptions); 
--# expr-done 5281
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 5282
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 5283
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5284 '<FieldAccess for children_env at ast.py:20933>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20933
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 5284
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2611;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 5285 '<Symbol Exception_Occurrence at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exception_Occurrence); 
--# expr-done 5285
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 5286
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5287
Let_Result := Cast_Result; 
--# end
--# expr-done 5288
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1433;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1433;
                     Finalizer_Scope_2611;
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Expr_Type;
--# end
--# property-start ExceptionHandler.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20937
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Xref_Equation
  (Node : Bare_Exception_Handler
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1434 with Inline_Always;
            procedure Finalizer_Scope_2612 with Inline_Always;
      Fld : Bare_Alternatives_List;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Logic_Equation;
Item_181 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1434 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1434;
            procedure Finalizer_Scope_2612 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2612;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5293 '<LogicAll at ast.py:20938>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20938
--# expr-start 5292 '<Map at ast.py:20938>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20938
   --# expr-start 5289 '<FieldAccess for handled_exceptions at ast.py:20938>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20938
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Exception_Handler_F_Handled_Exceptions;
--# expr-done 5289
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld;
      begin
         for Untyped_Item_181 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_181 := Untyped_Item_181; 
   --# scope-start
               --# bind he Item_181
      --# expr-start 5291 '<FieldAccess for sub_equation at ast.py:20938>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20938
--# expr-start 5290 '<as_entity at ast.py:20938>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20938
Is_Null := Item_181 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Item_181); 
   If_Result := As_Entity;
end if;
--# expr-done 5290
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result.Node, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 5291
         declare
            Item_To_Append : constant Logic_Equation := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2612;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5292
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20938") else null)); 
--# expr-done 5293
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1434;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1434;
                     Finalizer_Scope_2612;
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Xref_Equation;
--# end
--# property-start ExceptionHandler.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20941
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Xref_Entry_Point
  (Node : Bare_Exception_Handler
  )
   return Boolean
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Xref_Entry_Point;
--# end
--# property-start ExceptionHandler.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20943
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Is_Constant_Object
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Is_Constant_Object;
--# end
--# property-start ExceptionHandler._env_trans_parent_143 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20912
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_143
  (Node : Bare_Exception_Handler
  )
   return Boolean
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_143;
--# end
--# property-start ExceptionHandler._env_mappings_144 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20915
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_144
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
     Ent : Internal_Entity_Exception_Handler :=
       Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1671 with Inline_Always;
            procedure Finalizer_Scope_2613 with Inline_Always;
            procedure Finalizer_Scope_2614 with Inline_Always;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Exception_Handler;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
Scope_Result : Internal_Env_Assoc_Array_Access;
Result_Var : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1671 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1671;
            procedure Finalizer_Scope_2613 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2613;
            procedure Finalizer_Scope_2614 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2614;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5294 '<FieldAccess for exception_name at ast.py:20915>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20915
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Exception_Handler_F_Exception_Name, Info => Ent.Info);
--# expr-done 5294
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 5298 '<Map at ast.py:20915>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20915
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
      declare
         Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Singleton;
      begin
         for Item_182 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_182
      --# expr-start 5297 '<New EnvAssoc at ast.py:20917>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20917
--# expr-start 5295 '<New DesignatedEnv at ast.py:20919>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20919
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5295
--# expr-start 5296 '<FieldAccess for name_symbol at ast.py:20917>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20917
      if Item_182.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_182.Node);
--# end
--# expr-done 5296
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5297
         declare
            Item_To_Append : constant Internal_Env_Assoc := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2614;
         end loop;
      end;
         Map_Result := Create_Internal_Env_Assoc_Array
           (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5298
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2613;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1671;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1671;
                     Finalizer_Scope_2613;
                     Finalizer_Scope_2614;
            raise;
      end;
   return Property_Result;
end Env_Mappings_144;
--# end
   --
   --  Primitives for Bare_For_Loop_Var_Decl
   --
      procedure For_Loop_Var_Decl_Pre_Env_Actions
        (Self            : Bare_For_Loop_Var_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_126 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:18792");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_For_Loop_Var_Decl
        (Self : Bare_For_Loop_Var_Decl
         ; For_Loop_Var_Decl_F_Id : Bare_Defining_Name
         ; For_Loop_Var_Decl_F_Id_Type : Bare_Type_Expr
        ) is
      begin
            Self.For_Loop_Var_Decl_F_Id := For_Loop_Var_Decl_F_Id;
            Self.For_Loop_Var_Decl_F_Id_Type := For_Loop_Var_Decl_F_Id_Type;
      end Initialize_Fields_For_For_Loop_Var_Decl;
   function For_Loop_Var_Decl_F_Id
     (Node : Bare_For_Loop_Var_Decl) return Bare_Defining_Name
   is
   begin
         return Node.For_Loop_Var_Decl_F_Id;
   end;
   function For_Loop_Var_Decl_F_Id_Type
     (Node : Bare_For_Loop_Var_Decl) return Bare_Type_Expr
   is
   begin
         return Node.For_Loop_Var_Decl_F_Id_Type;
   end;
--# property-start ForLoopVarDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18764
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Defining_Names
  (Node : Bare_For_Loop_Var_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1311 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1311 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1311;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5299 '<FieldAccess for id at ast.py:18764>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18764
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 5299
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1311;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1311;
            raise;
      end;
   return Property_Result;
end For_Loop_Var_Decl_P_Defining_Names;
--# end
--# property-start ForLoopVarDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18766
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Defining_Env
  (Node : Bare_For_Loop_Var_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1312 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1312 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1312;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5301 '<FieldAccess for defining_env at ast.py:18766>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18766
--# expr-start 5300 '<FieldAccess for expr_type at ast.py:18766>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18766
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5300
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5301
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1312;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1312;
            raise;
      end;
   return Property_Result;
end For_Loop_Var_Decl_P_Defining_Env;
--# end
--# property-start ForLoopVarDecl.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18768
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Expr_Type
  (Node : Bare_For_Loop_Var_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Defining_Name;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Ada_Node;
Cast_Result_1 : Bare_For_Loop_Spec;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Type_Expr;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Bare_Type_Expr;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_For_Loop_Var_Decl_For_Loop_Var_Decl_P_Expr_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_For_Loop_Var_Decl,
                                As_Bare_For_Loop_Var_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5316 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5315 '<If at ast.py:18770>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18770
--# expr-start 5303 '<IsNull at ast.py:18771>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18771
--# expr-start 5302 '<FieldAccess for id_type at ast.py:18771>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18771
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.For_Loop_Var_Decl_F_Id_Type;
--# expr-done 5302
Is_Null := Fld_8 = null; 
--# expr-done 5303
if Is_Null then
   --# expr-start 5312 '<If at ast.py:18782>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18782
--# expr-start 5306 '<FieldAccess for is_iterated_assoc_spec at ast.py:18783>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18783
--# expr-start 5305 '<Cast to ForLoopSpec at ast.py:18783>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18783
--# expr-start 5304 '<parent at ast.py:18783>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18783
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Parent; 
--# expr-done 5304
   if Fld_4 = null
      or else Fld_4.Kind in Ada_For_Loop_Spec_Range
   then
      Cast_Result_1 := Fld_4;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 5305
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ForLoopSpec.is_iterated_assoc_spec
Fld_5 := Libadalang.Implementation.For_Loop_Spec_P_Is_Iterated_Assoc_Spec (Node => Cast_Result_1);
--# end
--# expr-done 5306
if Fld_5 then
   --# expr-start 5309 '<Cast to BaseTypeDecl at ast.py:18784>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18784
--# expr-start 5308 '<FieldAccess for type_val at ast.py:18784>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18784
--# expr-start 5307 '<FieldAccess for id at ast.py:18784>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18784
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.For_Loop_Var_Decl_F_Id;
--# expr-done 5307
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_1 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld);
--# end
--# expr-done 5308
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5309
   If_Result := Cast_Result;
else
   --# expr-start 5311 '<FieldAccess for expression_type at ast.py:18785>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18785
--# expr-start 5310 '<FieldAccess for id at ast.py:18785>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18785
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 5310
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 5311
   If_Result := Fld_3;
end if;
--# expr-done 5312
   If_Result_1 := If_Result;
else
   --# expr-start 5314 '<FieldAccess for designated_type at ast.py:18789>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18789
--# expr-start 5313 '<FieldAccess for id_type at ast.py:18789>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18789
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.For_Loop_Var_Decl_F_Id_Type, Info => Ent.Info);
--# expr-done 5313
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 5314
   If_Result_1 := Fld_7;
end if;
--# expr-done 5315
Let_Result := If_Result_1; 
--# end
--# expr-done 5316
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end For_Loop_Var_Decl_P_Expr_Type;
--# end
--# property-start ForLoopVarDecl.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18794
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Is_Constant_Object
  (Node : Bare_For_Loop_Var_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Block_Var : Internal_Entity_For_Loop_Spec;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_For_Loop_Spec;
Fld_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Name;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Iter_Type;
Is_A : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5325 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5318 '<Cast to ForLoopSpec at ast.py:18797>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18797
--# expr-start 5317 '<parent at ast.py:18797>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18797
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5317
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_For_Loop_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_For_Loop_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_For_Loop_Spec;
   end if;
--# expr-done 5318
Block_Var := Cast_Result; 
--# bind loop_spec Block_Var
--# expr-start 5324 '<If at ast.py:18798>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18798
--# expr-start 5320 '<IsA IterType.Of at ast.py:18801>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18801
--# expr-start 5319 '<FieldAccess for loop_type at ast.py:18801>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18801
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Iter_Type (Node => Block_Var.Node.For_Loop_Spec_F_Loop_Type, Info => Block_Var.Info);
--# expr-done 5319
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Iter_Type_Of_Range; 
--# expr-done 5320
if Is_A then
   --# expr-start 5323 '<FieldAccess for is_constant at ast.py:18802>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18802
--# expr-start 5322 '<Cast to Name at ast.py:18802>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18802
--# expr-start 5321 '<FieldAccess for iter_expr at ast.py:18802>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18802
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity (Node => Block_Var.Node.For_Loop_Spec_F_Iter_Expr, Info => Block_Var.Info);
--# expr-done 5321
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5322
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 5323
   If_Result := Fld_2;
else
   If_Result := True;
end if;
--# expr-done 5324
Let_Result := If_Result; 
--# end
--# expr-done 5325
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end For_Loop_Var_Decl_P_Is_Constant_Object;
--# end
--# property-start ForLoopVarDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18807
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Xref_Equation
  (Node : Bare_For_Loop_Var_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1315 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1315 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1315;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5334 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5333 '<And at ast.py:18809>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18809
--# expr-start 5327 '<FieldAccess for sub_equation at ast.py:18809>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18809
--# expr-start 5326 '<FieldAccess for id at ast.py:18809>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18809
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 5326
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5327
--# expr-start 5332 '<If at ast.py:18809>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18809
--# expr-start 5329 '<IsNull at ast.py:18810>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18810
--# expr-start 5328 '<FieldAccess for id_type at ast.py:18810>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18810
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.For_Loop_Var_Decl_F_Id_Type, Info => Ent.Info);
--# expr-done 5328
Is_Null := Fld_4.Node = null; 
--# expr-done 5329
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18811") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 5331 '<FieldAccess for sub_equation at ast.py:18812>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18812
--# expr-start 5330 '<FieldAccess for id_type at ast.py:18812>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18812
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.For_Loop_Var_Decl_F_Id_Type, Info => Ent.Info);
--# expr-done 5330
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5331
   If_Result := Fld_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 5332
And_Pred := Create_And (Fld_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18809") else null)); 
--# expr-done 5333
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5334
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1315;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1315;
            raise;
      end;
   return Property_Result;
end For_Loop_Var_Decl_P_Xref_Equation;
--# end
--# property-start ForLoopVarDecl._env_mappings_126 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18792
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_126
  (Node : Bare_For_Loop_Var_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1653 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_For_Loop_Var_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1653 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1653;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5337 '<New EnvAssoc at ast.py:18792>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18792
--# expr-start 5335 '<New DesignatedEnv at ast.py:18792>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18792
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5335
--# expr-start 5336 '<FieldAccess for name_symbol at ast.py:18792>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18792
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 5336
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5337
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1653;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1653;
            raise;
      end;
   return Property_Result;
end Env_Mappings_126;
--# end
   --
   --  Primitives for Bare_Generic_Decl
   --
      procedure Initialize_Fields_For_Generic_Decl
        (Self : Bare_Generic_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
        ) is
      begin
            Self.Generic_Decl_F_Formal_Part := Generic_Decl_F_Formal_Part;
      end Initialize_Fields_For_Generic_Decl;
   function Generic_Decl_F_Formal_Part
     (Node : Bare_Generic_Decl) return Bare_Generic_Formal_Part
   is
   begin
         return Node.Generic_Decl_F_Formal_Part;
   end;
--# property-start '[dispatcher]GenericDecl.decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Decl_P_Decl
  (Node : Bare_Generic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Decl (Self.Kind) is
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.decl
                  Property_Result := Generic_Package_Decl_P_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.decl
                  Property_Result := Generic_Subp_Decl_P_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Decl_P_Decl;
--# end
--# property-start GenericDecl.get_aspect_assoc /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12040
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Generic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Aspect_Assoc;
Let_Result : Internal_Entity_Aspect_Assoc;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5340 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5339 '<FieldAccess for get_aspect_assoc at ast.py:12044>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12044
--# expr-start 5338 '<FieldAccess for decl at ast.py:12044>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12044
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5338
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Fld.Node, Name => Name, E_Info => Fld.Info);
--# end
--# expr-done 5339
Let_Result := Fld_1; 
--# end
--# expr-done 5340
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start GenericDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12046
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Generic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5343 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5342 '<FieldAccess for next_part_for_decl at ast.py:12048>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12048
--# expr-start 5341 '<FieldAccess for decl at ast.py:12048>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12048
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5341
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 5342
Let_Result := Fld_1; 
--# end
--# expr-done 5343
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Decl_P_Next_Part_For_Decl;
--# end
   --
   --  Primitives for Bare_Generic_Package_Decl
   --
      procedure Generic_Package_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_112 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_113 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:12135");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_114 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:12142");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_115 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_116 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_118 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_117 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_120 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_119 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Decl
        (Self : Bare_Generic_Package_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
         ; Generic_Package_Decl_F_Package_Decl : Bare_Generic_Package_Internal
        ) is
      begin
            Initialize_Fields_For_Generic_Decl
              (Self, Generic_Decl_F_Formal_Part);
            Self.Generic_Package_Decl_F_Package_Decl := Generic_Package_Decl_F_Package_Decl;
      end Initialize_Fields_For_Generic_Package_Decl;
   function Generic_Package_Decl_F_Package_Decl
     (Node : Bare_Generic_Package_Decl) return Bare_Generic_Package_Internal
   is
   begin
         return Node.Generic_Package_Decl_F_Package_Decl;
   end;
--# property-start GenericPackageDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12162
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Defining_Env
  (Node : Bare_Generic_Package_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_932 with Inline_Always;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_932 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_932;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5345 '<FieldAccess for defining_env at ast.py:12162>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12162
--# expr-start 5344 '<FieldAccess for package_decl at ast.py:12162>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12162
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 5344
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5345
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_932;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_932;
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Defining_Env;
--# end
--# property-start GenericPackageDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12164
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Defining_Names
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_933 with Inline_Always;
      Fld : Bare_Generic_Package_Internal;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_933 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_933;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5348 '<as_entity at ast.py:12165>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12165
--# expr-start 5347 '<FieldAccess for package_name at ast.py:12165>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12165
--# expr-start 5346 '<FieldAccess for package_decl at ast.py:12165>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12165
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Generic_Package_Decl_F_Package_Decl;
--# expr-done 5346
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Base_Package_Decl_F_Package_Name;
--# expr-done 5347
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 5348
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_933;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_933;
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Defining_Names;
--# end
--# property-start GenericPackageDecl.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12168
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Declarative_Parts
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_934 with Inline_Always;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_934 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_934;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5350 '<FieldAccess for declarative_parts at ast.py:12168>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12168
--# expr-start 5349 '<FieldAccess for package_decl at ast.py:12168>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12168
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 5349
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.declarative_parts'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Declarative_Parts (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5350
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_934;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_934;
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Declarative_Parts;
--# end
--# property-start GenericPackageDecl.body_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12170
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Body_Part
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Package_Body
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Package_Body;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Internal_Entity_Package_Body;
Let_Result : Internal_Entity_Package_Body;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5353 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5352 '<FieldAccess for body_part at ast.py:12176>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12176
--# expr-start 5351 '<FieldAccess for package_decl at ast.py:12176>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12176
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 5351
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_1 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5352
Let_Result := Fld_1; 
--# end
--# expr-done 5353
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Body_Part;
--# end
--# property-start GenericPackageDecl.decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12178
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Decl
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Package_Internal;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5354 '<FieldAccess for package_decl at ast.py:12178>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12178
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 5354
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Decl;
--# end
--# property-start GenericPackageDecl._env_do_112 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12133
pragma Warnings (Off, "is not referenced");
 function Env_Do_112
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5355 '<FieldAccess for env_hook at ast.py:12133>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12133
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5355
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_112;
--# end
--# property-start GenericPackageDecl._initial_env_113 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12139
pragma Warnings (Off, "is not referenced");
 function Initial_Env_113
  (Node : Bare_Generic_Package_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1640 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1640 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1640;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5356 '<FieldAccess for child_decl_initial_env at ast.py:12139>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12139
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self, Private_Part => True);
--# end
--# expr-done 5356
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1640;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1640;
            raise;
      end;
   return Property_Result;
end Initial_Env_113;
--# end
--# property-start GenericPackageDecl._env_mappings_114 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12142
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_114
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1641 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1641 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1641;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5357 '<FieldAccess for child_decl_env_assocs at ast.py:12142>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12142
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_env_assocs
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Env_Assocs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5357
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1641;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1641;
            raise;
      end;
   return Property_Result;
end Env_Mappings_114;
--# end
--# property-start GenericPackageDecl._env_trans_parent_115 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12132
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_115
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_115;
--# end
--# property-start GenericPackageDecl._env_do_116 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12146
pragma Warnings (Off, "is not referenced");
 function Env_Do_116
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1643 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1643 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1643;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5358 '<FieldAccess for populate_dependent_units at ast.py:12146>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12146
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5358
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1643;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1643;
            raise;
      end;
   return Property_Result;
end Env_Do_116;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_117 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12148
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_117
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1644 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1644 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1644;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5359 '<FieldAccess for top_level_use_package_clauses at ast.py:12148>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12148
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5359
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1644;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1644;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_117;
--# end
--# property-start GenericPackageDecl._ref_cond_118 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12150
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_118
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5361 '<IsA LibraryItem, Subunit at ast.py:12150>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12150
--# expr-start 5360 '<parent at ast.py:12150>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12150
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5360
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5361
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_118;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_119 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12153
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_119
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1646 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1646 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1646;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5362 '<FieldAccess for top_level_use_type_clauses at ast.py:12153>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12153
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5362
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1646;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1646;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_119;
--# end
--# property-start GenericPackageDecl._ref_cond_120 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12155
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_120
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5364 '<IsA LibraryItem, Subunit at ast.py:12155>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12155
--# expr-start 5363 '<parent at ast.py:12155>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12155
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5363
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5364
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_120;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Decl
   --
      procedure Generic_Subp_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_100 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_101 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:12072");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_102 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:12079");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_103 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_104 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_106 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_105 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_108 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_107 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Generic_Subp_Decl_Post_Env_Actions
        (Self : Bare_Generic_Subp_Decl; State : in out PLE_Node_State) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_109 (Node => 
Self);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Decl
        (Self : Bare_Generic_Subp_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
         ; Generic_Subp_Decl_F_Subp_Decl : Bare_Generic_Subp_Internal
        ) is
      begin
            Initialize_Fields_For_Generic_Decl
              (Self, Generic_Decl_F_Formal_Part);
            Self.Generic_Subp_Decl_F_Subp_Decl := Generic_Subp_Decl_F_Subp_Decl;
      end Initialize_Fields_For_Generic_Subp_Decl;
   function Generic_Subp_Decl_F_Subp_Decl
     (Node : Bare_Generic_Subp_Decl) return Bare_Generic_Subp_Internal
   is
   begin
         return Node.Generic_Subp_Decl_F_Subp_Decl;
   end;
--# property-start GenericSubpDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12059
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Defining_Names
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_926 with Inline_Always;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_926 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_926;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5367 '<FieldAccess for name at ast.py:12059>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12059
--# expr-start 5366 '<FieldAccess for subp_spec at ast.py:12059>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12059
--# expr-start 5365 '<FieldAccess for subp_decl at ast.py:12059>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12059
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 5365
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Fld.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Fld.Info);
--# expr-done 5366
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 5367
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_2;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_926;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_926;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Defining_Names;
--# end
--# property-start GenericSubpDecl.body_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12061
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Body_Part
  (Node : Bare_Generic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Body
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Base_Subp_Body;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5370 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5369 '<Cast to BaseSubpBody at ast.py:12066>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12066
--# expr-start 5368 '<FieldAccess for body_part_for_decl at ast.py:12066>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12066
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5368
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 5369
Let_Result := Cast_Result; 
--# end
--# expr-done 5370
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Body_Part;
--# end
--# property-start GenericSubpDecl.decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12101
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Decl
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Subp_Internal;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5371 '<FieldAccess for subp_decl at ast.py:12101>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12101
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 5371
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Decl;
--# end
--# property-start GenericSubpDecl.is_imported /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12104
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Is_Imported
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5373 '<FieldAccess for is_imported at ast.py:12104>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12104
--# expr-start 5372 '<FieldAccess for subp_decl at ast.py:12104>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12104
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 5372
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_imported'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Imported (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5373
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Is_Imported;
--# end
--# property-start GenericSubpDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12106
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Generic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5376 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5375 '<FieldAccess for next_part_for_decl at ast.py:12108>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12108
--# expr-start 5374 '<FieldAccess for subp_decl at ast.py:12108>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12108
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 5374
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 5375
Let_Result := Fld_1; 
--# end
--# expr-done 5376
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start GenericSubpDecl._env_do_100 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12070
pragma Warnings (Off, "is not referenced");
 function Env_Do_100
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5377 '<FieldAccess for env_hook at ast.py:12070>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12070
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5377
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_100;
--# end
--# property-start GenericSubpDecl._initial_env_101 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12076
pragma Warnings (Off, "is not referenced");
 function Initial_Env_101
  (Node : Bare_Generic_Subp_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1628 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1628 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1628;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5378 '<FieldAccess for child_decl_initial_env at ast.py:12076>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12076
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self, Private_Part => True);
--# end
--# expr-done 5378
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1628;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1628;
            raise;
      end;
   return Property_Result;
end Initial_Env_101;
--# end
--# property-start GenericSubpDecl._env_mappings_102 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12079
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_102
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1629 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1629 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1629;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5379 '<FieldAccess for child_decl_env_assocs at ast.py:12079>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12079
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_env_assocs
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Env_Assocs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5379
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1629;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1629;
            raise;
      end;
   return Property_Result;
end Env_Mappings_102;
--# end
--# property-start GenericSubpDecl._env_trans_parent_103 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12068
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_103
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_103;
--# end
--# property-start GenericSubpDecl._env_do_104 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12083
pragma Warnings (Off, "is not referenced");
 function Env_Do_104
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1631 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1631 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1631;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5380 '<FieldAccess for populate_dependent_units at ast.py:12083>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12083
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5380
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1631;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1631;
            raise;
      end;
   return Property_Result;
end Env_Do_104;
--# end
--# property-start GenericSubpDecl._ref_env_nodes_105 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12086
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_105
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1632 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1632 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1632;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5381 '<FieldAccess for top_level_use_package_clauses at ast.py:12086>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12086
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5381
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1632;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1632;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_105;
--# end
--# property-start GenericSubpDecl._ref_cond_106 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12088
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_106
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5383 '<IsA LibraryItem, Subunit at ast.py:12088>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12088
--# expr-start 5382 '<parent at ast.py:12088>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12088
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5382
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5383
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_106;
--# end
--# property-start GenericSubpDecl._ref_env_nodes_107 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12091
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_107
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1634 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1634 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1634;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5384 '<FieldAccess for top_level_use_type_clauses at ast.py:12091>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12091
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5384
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1634;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1634;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_107;
--# end
--# property-start GenericSubpDecl._ref_cond_108 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12093
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_108
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5386 '<IsA LibraryItem, Subunit at ast.py:12093>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12093
--# expr-start 5385 '<parent at ast.py:12093>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12093
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5385
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5386
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_108;
--# end
--# property-start GenericSubpDecl._env_do_109 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12098
pragma Warnings (Off, "is not referenced");
 function Env_Do_109
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5387 '<FieldAccess for populate_body_unit at ast.py:12098>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12098
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_body_unit
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Body_Unit (Node => Self);
--# end
--# expr-done 5387
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_109;
--# end
   --
   --  Primitives for Bare_Generic_Instantiation
   --
      procedure Generic_Instantiation_Pre_Env_Actions
        (Self            : Bare_Generic_Instantiation;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_59 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_60 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:11394");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_61 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:11401");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_62 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_63 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_64 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_66 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_65 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_68 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_67 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_70 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_69 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Generic_Instantiation_P_Parent_Instantiation_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Instantiation
        (Self : Bare_Generic_Instantiation
        ) is
      begin
      Self.Lf_Present_Instantiation_Env := False;
      Self.Lf_Stg_Instantiation_Env := Empty_Env;
      Self.Lf_Present_Actual_Expr_Decls := False;
      Self.Lf_Stg_Actual_Expr_Decls := No_Bare_Anonymous_Expr_Decl_Array_Type;
      end Initialize_Fields_For_Generic_Instantiation;
--# property-start GenericInstantiation.instantiation_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11169
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_F_Instantiation_Env
  (Node : Bare_Generic_Instantiation
  )
   return Lexical_Env
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_867 with Inline_Always;
      Dyn_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_867 is
            begin
                     Dec_Ref (Dyn_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_867;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Instantiation_Env then
         Property_Result := Self.Lf_Stg_Instantiation_Env;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5389 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5388 '<DynamicLexicalEnv at ast.py:11171>' Dyn_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11171
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Generic_Instantiation_P_Instantiation_Bindings'Access, Ada_Node_P_Resolve_Generic_Actual'Access, False); 
--# expr-done 5388
Let_Result := Dyn_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 5389
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_867;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_867;
            raise;
      end;
      Self.Lf_Present_Instantiation_Env := True;
      Self.Lf_Stg_Instantiation_Env := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Generic_Instantiation_F_Instantiation_Env;
--# end
--# property-start GenericInstantiation.instantiation_bindings /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11177
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Instantiation_Bindings
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_868 with Inline_Always;
      Block_Var : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_868 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_868;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5392 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5390 '<FieldAccess for nonbound_generic_decl at ast.py:11183>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11183
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 5390
Block_Var := Fld; 
--# expr-start 5391 '<FieldAccess for instantiation_bindings_internal at ast.py:11184>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11184
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.instantiation_bindings_internal
Fld_1 := Libadalang.Implementation.Generic_Instantiation_P_Instantiation_Bindings_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5391
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5392
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_868;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_868;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Instantiation_Bindings;
--# end
--# property-start GenericInstantiation.instantiation_bindings_internal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11186
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Instantiation_Bindings_Internal
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_869 with Inline_Always;
            procedure Finalizer_Scope_2615 with Inline_Always;
            procedure Finalizer_Scope_2617 with Inline_Always;
      Var_Expr : Internal_Entity_Generic_Decl;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Generic_Formal_Part;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Internal_Param_Match_Array_Access;
Scope_Result : Internal_Param_Match_Array_Access;
Result_Var : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Single_Actual;
Fld_6 : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Is_A : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_8 : Internal_Entity_Defining_Name;
Fld_9 : Internal_Entity_Name;
Fld_10 : Symbol_Type;
Fld_11 : Bare_Anonymous_Expr_Decl_Array_Access;
Get_Result : Bare_Anonymous_Expr_Decl;
Fld_12 : Internal_Single_Actual;
Fld_13 : Internal_Entity_Basic_Assoc;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Bare_Expr;
Cast_Result : Bare_Ada_Node;
Cast_Result_1 : Bare_Ada_Node;
Fld_16 : Internal_Entity_Defining_Name;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl;
Is_A_1 : Boolean;
If_Result : Bare_Ada_Node;
New_Struct : Internal_Metadata;
New_Struct_1 : Internal_Inner_Env_Assoc;
Index_184 : Integer;
Map_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
Dyn_Var_Bind_Result : Internal_Inner_Env_Assoc_Array_Access;
Fld_18 : Boolean;
If_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_869 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_869;
            procedure Finalizer_Scope_2615 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2615;
            procedure Finalizer_Scope_2617 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2617;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Generic_Instantiation_Generic_Instantiation_P_Instantiation_Bindings_Internal,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Generic_Instantiation,
                                As_Bare_Generic_Instantiation => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Inner_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5423 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5422 '<If at ast.py:11189>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11189
--# expr-start 5393 '<FieldAccess for is_any_formal at ast.py:11190>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11190
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.is_any_formal
Fld_18 := Libadalang.Implementation.Generic_Instantiation_P_Is_Any_Formal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5393
if Fld_18 then
   If_Result_1 := No_Internal_Inner_Env_Assoc_Array_Type;
else
   --# expr-start 5421 '<bind at ast.py:11192>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11192
--# expr-start 5394 '<FieldAccess for default_initial_env at ast.py:11193>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11193
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 5394
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5420 '<Map at ast.py:11198>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11198
   --# expr-start 5404 '<Filter at ast.py:11196>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11196
--# expr-start 5395 '<FieldAccess for nonbound_generic_decl at ast.py:11194>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11194
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld_1 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 5395
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 5398 '<FieldAccess for match_param_list at ast.py:11194>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11194
--# expr-start 5396 '<FieldAccess for formal_part at ast.py:11194>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11194
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr.Info);
--# expr-done 5396
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5397 '<FieldAccess for generic_inst_params at ast.py:11195>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11195
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld_3 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5397
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_2.Node, Params => Fld_3, Is_Dottable_Subp => False, Env => Bound_Env, E_Info => Fld_2.Info);
--# end
--# expr-done 5398
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2615;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var);
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Result_Var;
      begin
         for Item_185 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_185
         --# expr-start 5403 '<Not at ast.py:11197>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11197
--# expr-start 5402 '<IsA BoxExpr at ast.py:11197>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11197
--# expr-start 5401 '<FieldAccess for expr at ast.py:11197>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11197
--# expr-start 5400 '<FieldAccess for assoc at ast.py:11197>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11197
--# expr-start 5399 '<FieldAccess for actual at ast.py:11197>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11197
Fld_5 := Item_185.Actual;
--# expr-done 5399
Fld_6 := Fld_5.Assoc;
--# expr-done 5400
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 5401
Is_A := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 5402
Not_Val := not (Is_A); 
--# expr-done 5403
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_185;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5404
      declare
         Map_Result_1_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
         Index_184 := 0;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Map_Result;
      begin
         for Item_183 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_183
               --# bind i Index_184
      --# expr-start 5419 '<New InnerEnvAssoc at ast.py:11199>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11199
--# expr-start 5407 '<FieldAccess for name_symbol at ast.py:11200>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11200
--# expr-start 5406 '<FieldAccess for name at ast.py:11200>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11200
--# expr-start 5405 '<FieldAccess for formal at ast.py:11200>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11200
Fld_8 := Item_183.Formal;
--# expr-done 5405
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Fld_8.Node.Defining_Name_F_Name, Info => Fld_8.Info);
--# expr-done 5406
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_9.Node);
--# end
--# expr-done 5407
--# expr-start 5408 '<New Metadata at ast.py:11206>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11206
New_Struct := (Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 5408
--# expr-start 5418 '<If at ast.py:11201>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11201
--# expr-start 5411 '<IsA GenericFormalObjDecl at ast.py:11202>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11202
--# expr-start 5410 '<FieldAccess for formal_decl at ast.py:11202>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11202
--# expr-start 5409 '<FieldAccess for formal at ast.py:11202>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11202
Fld_16 := Item_183.Formal;
--# expr-done 5409
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_17 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 5410
Is_A_1 := Fld_17.Node /= null 
and then Fld_17.Node.Kind in Ada_Generic_Formal_Obj_Decl_Range; 
--# expr-done 5411
if Is_A_1 then
--# expr-start 5413 '<at at ast.py:11203>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11203
--# expr-start 5412 '<FieldAccess for actual_expr_decls at ast.py:11203>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11203
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.actual_expr_decls
Fld_11 := Libadalang.Implementation.Generic_Instantiation_F_Actual_Expr_Decls (Node => Ent.Node);
--# end
--# expr-done 5412
Get_Result := Get (Self, Fld_11, Index_184, True); 
--# expr-done 5413
      Cast_Result := Get_Result;
   If_Result := Cast_Result;
else
--# expr-start 5417 '<FieldAccess for node at ast.py:11204>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11204
--# expr-start 5416 '<FieldAccess for expr at ast.py:11204>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11204
--# expr-start 5415 '<FieldAccess for assoc at ast.py:11204>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11204
--# expr-start 5414 '<FieldAccess for actual at ast.py:11204>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11204
Fld_12 := Item_183.Actual;
--# expr-done 5414
Fld_13 := Fld_12.Assoc;
--# expr-done 5415
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 5416
Fld_15 := Fld_14.Node;
--# expr-done 5417
      Cast_Result_1 := Fld_15;
   If_Result := Cast_Result_1;
end if;
--# expr-done 5418
New_Struct_1 := (Key => Fld_10, Metadata => New_Struct, Value => If_Result); 
--# expr-done 5419
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := New_Struct_1;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2617;
               Index_184 := Index_184 + 1;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 5420
Dyn_Var_Bind_Result := Map_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5421
   If_Result_1 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5422
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5423
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_869;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_869;
                     Finalizer_Scope_2615;
                     Finalizer_Scope_2617;
               if Self /= null then
                     Add_Memoized_Error
                       (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
               end if;
            raise;
      end;
      if Self /= null then
            Mmz_Val := (Kind => Mmz_Internal_Inner_Env_Assoc_Array_Access,
                        As_Internal_Inner_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
      end if;
   return Property_Result;
end Generic_Instantiation_P_Instantiation_Bindings_Internal;
--# end
--# property-start '[dispatcher]GenericInstantiation.generic_entity_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.generic_entity_name
                  Property_Result := Generic_Package_Instantiation_P_Generic_Entity_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.generic_entity_name
                  Property_Result := Generic_Subp_Instantiation_P_Generic_Entity_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start '[dispatcher]GenericInstantiation.generic_inst_params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.generic_inst_params
                  Property_Result := Generic_Package_Instantiation_P_Generic_Inst_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.generic_inst_params
                  Property_Result := Generic_Subp_Instantiation_P_Generic_Inst_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericInstantiation.is_any_formal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11225
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Is_Any_Formal
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5426 '<IsA BoxExpr at ast.py:11226>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11226
--# expr-start 5424 '<FieldAccess for generic_inst_params at ast.py:11226>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11226
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5424
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 5425 '<FieldAccess for expr at ast.py:11226>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11226
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5425
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Is_A := Result_Var.Node /= null 
and then Result_Var.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 5426
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Is_Any_Formal;
--# end
--# property-start GenericInstantiation.nonbound_generic_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11229
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Nonbound_Generic_Decl
  (Node : Bare_Generic_Instantiation
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_873 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Body_Node;
Match_1 : Internal_Entity_Generic_Renaming_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Match_3 : Internal_Entity;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Name;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity_Array_Access;
Fld_2 : Boolean;
Find_Result : Internal_Entity;
Bound_Imprecise_Fallback : Boolean;
Fld_3 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Generic_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Renaming_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Generic_Decl;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Generic_Decl;
Scope_Result_5 : Internal_Entity_Generic_Decl;
Result_Var_1 : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_873 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_873;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5431 '<Find at ast.py:11233>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11233
   --# expr-start 5429 '<FieldAccess for all_env_elements_internal at ast.py:11231>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11231
--# expr-start 5428 '<FieldAccess for generic_entity_name at ast.py:11230>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11230
--# expr-start 5427 '<as_bare_entity at ast.py:11230>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11230
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 5427
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5428
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.all_env_elements_internal
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Fld.Node, Seq => True, Seq_From => Cast_Result, Categories => (Inherited_Primitives => False, Nocat => True), E_Info => Fld.Info);
--# end
--# expr-done 5429
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_186 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_186
      --# expr-start 5430 '<FieldAccess for has_visibility at ast.py:11234>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11234
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_2 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_186);
--# end
--# expr-done 5430
      if Fld_2 then
         Find_Result := Item_186; 
      end if;
   --# end
               exit when Fld_2;
         end loop;
      end;
--# expr-done 5431
Var_Expr_1 := Find_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
--# expr-start 5435 '<Match at ast.py:11235>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11235
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind b Match_0
--# expr-start 5433 '<bind at ast.py:11236>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11236
Bound_Imprecise_Fallback := False; 
--# expr-start 5432 '<FieldAccess for decl_part at ast.py:11236>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11236
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_3 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_0.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 5432
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 5433
Let_Result := Dyn_Var_Bind_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind rd Match_1
--# expr-start 5434 '<FieldAccess for resolve at ast.py:11237>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11237
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_4 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 5434
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind d Match_2
Let_Result_2 := Match_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Cast_Expr_1 := No_Entity_Generic_Decl; 
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result_3 := Cast_Result_6; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 5435
Scope_Result_4 := Match_Result;
   --# end
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5436 '<Cast to GenericDecl at ast.py:11240>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11240
Cast_Expr_2 := Var_Expr; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_7 := No_Entity_Generic_Decl;
   end if;
--# expr-done 5436
Scope_Result_5 := Cast_Result_7;
   --# end
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := No_Entity_Generic_Decl;
end if;
         Property_Result := Result_Var_1;
   --# end
      Finalizer_Scope_873;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_873;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Nonbound_Generic_Decl;
--# end
--# property-start '[dispatcher]GenericInstantiation.designated_generic_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.designated_generic_decl
                  Property_Result := Generic_Package_Instantiation_P_Designated_Generic_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.designated_generic_decl
                  Property_Result := Generic_Subp_Instantiation_P_Designated_Generic_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericInstantiation.designated_bare_generic_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11255
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Designated_Bare_Generic_Decl
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Basic_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Internal_Metadata;
Fld_4 : Internal_Entity_Info;
Fld_5 : Env_Rebindings;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Info;
Fld_8 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5449 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5437 '<FieldAccess for designated_generic_decl at ast.py:11261>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11261
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5437
Block_Var := Fld; 
--# bind decl Block_Var
--# expr-start 5448 '<Cast to BasicDecl.entity at ast.py:11269>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11269
--# expr-start 5447 '<New AdaNode.entity at ast.py:11262>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11262
--# expr-start 5445 '<New EntityInfo at ast.py:11264>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11264
--# expr-start 5439 '<FieldAccess for from_rebound at ast.py:11267>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11267
--# expr-start 5438 '<FieldAccess for info at ast.py:11267>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11267
Fld_7 := Block_Var.Info;
--# expr-done 5438
Fld_8 := Fld_7.From_Rebound;
--# expr-done 5439
--# expr-start 5441 '<FieldAccess for md at ast.py:11265>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11265
--# expr-start 5440 '<FieldAccess for info at ast.py:11265>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11265
Fld_2 := Block_Var.Info;
--# expr-done 5440
Fld_3 := Fld_2.Md;
--# expr-done 5441
--# expr-start 5444 '<FieldAccess for get_parent at ast.py:11266>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11266
--# expr-start 5443 '<FieldAccess for rebindings at ast.py:11266>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11266
--# expr-start 5442 '<FieldAccess for info at ast.py:11266>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11266
Fld_4 := Block_Var.Info;
--# expr-done 5442
Fld_5 := Fld_4.Rebindings;
--# expr-done 5443
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Fld_5.Parent;
--# expr-done 5444
New_Struct := (From_Rebound => Fld_8, Md => Fld_3, Rebindings => Fld_6); 
--# expr-done 5445
--# expr-start 5446 '<FieldAccess for node at ast.py:11263>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11263
Fld_1 := Block_Var.Node;
--# expr-done 5446
      Cast_Result := Fld_1;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result); 
--# expr-done 5447
   if New_Struct_1.Node = null
      or else New_Struct_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => New_Struct_1.Node,
         Info => New_Struct_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5448
Let_Result := Cast_Result_1; 
--# end
--# expr-done 5449
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Designated_Bare_Generic_Decl;
--# end
--# property-start GenericInstantiation.actual_expr_decls /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11271
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_F_Actual_Expr_Decls
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Anonymous_Expr_Decl_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Anonymous_Expr_Decl_Array_Access;
            procedure Finalizer_Scope_876 with Inline_Always;
      Match_0 : Bare_Generic_Subp_Instantiation;
Match_1 : Bare_Generic_Package_Instantiation;
Fld : Bare_Assoc_List;
Fld_1 : Bare_Assoc_List;
Match_Prefix : Bare_Generic_Instantiation;
Cast_Result : Bare_Generic_Subp_Instantiation;
Let_Result : Bare_Assoc_List;
Scope_Result : Bare_Assoc_List;
Cast_Result_1 : Bare_Generic_Package_Instantiation;
Let_Result_1 : Bare_Assoc_List;
Scope_Result_1 : Bare_Assoc_List;
Match_Result : Bare_Assoc_List;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Bare_Expr;
New_Node : Bare_Anonymous_Expr_Decl;
Item_187 : Bare_Basic_Assoc;
Map_Result : Bare_Anonymous_Expr_Decl_Array_Access;
Let_Result_2 : Bare_Anonymous_Expr_Decl_Array_Access;
            procedure Finalizer_Scope_876 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_876;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Actual_Expr_Decls then
         Property_Result := Self.Lf_Stg_Actual_Expr_Decls;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5458 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 5457 '<Map at ast.py:11277>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11277
   --# expr-start 5452 '<Match at ast.py:11274>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11274
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Generic_Instantiation (Match_Prefix.Kind) is
         when Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind subp Match_0
--# expr-start 5450 '<FieldAccess for params at ast.py:11275>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11275
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Match_0.Generic_Subp_Instantiation_F_Params;
--# expr-done 5450
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind pkg Match_1
--# expr-start 5451 '<FieldAccess for params at ast.py:11276>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11276
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_1.Generic_Package_Instantiation_F_Params;
--# expr-done 5451
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 5452
      declare
         Map_Result_Vec : Bare_Anonymous_Expr_Decl_Vectors.Vector;
      begin
      if Match_Result = null then
      Map_Result := Create_Bare_Anonymous_Expr_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Match_Result;
      begin
         for Untyped_Item_187 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_187 := Untyped_Item_187; 
   --# scope-start
               --# bind assoc Item_187
      --# expr-start 5456 '<New AnonymousExprDecl at ast.py:11278>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11278
--# expr-start 5455 '<FieldAccess for node at ast.py:11279>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11279
--# expr-start 5454 '<FieldAccess for expr at ast.py:11279>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11279
--# expr-start 5453 '<as_bare_entity at ast.py:11279>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11279
Is_Null := Item_187 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
As_Entity := (Info => No_Entity_Info, Node => Item_187); 
   If_Result := As_Entity;
end if;
--# expr-done 5453
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5454
Fld_3 := Fld_2.Node;
--# expr-done 5455
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_3 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "AnonymousExprDecl.expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Anonymous_Expr_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Anonymous_Expr_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Anonymous_Expr_Decl
     (Self => New_Node,
      Anonymous_Expr_Decl_F_Expr => Fld_3);
--# expr-done 5456
         declare
            Item_To_Append : constant Bare_Anonymous_Expr_Decl := New_Node;
         begin
            Bare_Anonymous_Expr_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Anonymous_Expr_Decl_Array
           (Items_Count => Natural (Bare_Anonymous_Expr_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Anonymous_Expr_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Anonymous_Expr_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Anonymous_Expr_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5457
Let_Result_2 := Map_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 5458
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_876;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_876;
            raise;
      end;
      Self.Lf_Present_Actual_Expr_Decls := True;
      Self.Lf_Stg_Actual_Expr_Decls := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Generic_Instantiation_F_Actual_Expr_Decls;
--# end
--# property-start GenericInstantiation.parent_instantiation_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11283
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Parent_Instantiation_Env
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_877 with Inline_Always;
            procedure Finalizer_Scope_2623 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Dotted_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Fld_5 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_877 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_877;
            procedure Finalizer_Scope_2623 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
            end Finalizer_Scope_2623;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5468 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5461 '<Cast to DottedName at ast.py:11313>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11313
--# expr-start 5460 '<FieldAccess for name at ast.py:11313>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11313
--# expr-start 5459 '<FieldAccess for defining_name at ast.py:11313>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11313
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5459
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 5460
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 5461
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Dotted_Name then
   --# scope-start
--# expr-start 5462 '<FieldAccess for prefix at ast.py:11313>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11313
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Dotted_Name_F_Prefix, Info => Var_Expr_1.Info);
--# expr-done 5462
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind parent_name Var_Expr
--# expr-start 5466 '<bind at ast.py:11314>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11314
Cast_Expr := Self; 
      Cast_Result_1 := Cast_Expr;
Bound_Origin := Cast_Result_1; 
--# expr-start 5465 '<bind at ast.py:11316>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11316
--# expr-start 5463 '<FieldAccess for children_env at ast.py:11317>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11317
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5463
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 5464 '<FieldAccess for designated_env_no_overloading at ast.py:11318>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11318
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_4 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Var_Expr.Node, Env => Bound_Env, Origin => Bound_Origin, No_Visibility => False, E_Info => Var_Expr.Info);
--# end
--# expr-done 5464
Dyn_Var_Bind_Result := Fld_4; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5465
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 5466
Scope_Result_1 := Dyn_Var_Bind_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2623;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 5467 '<FieldAccess for empty_env at ast.py:11321>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11321
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_5 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 5467
   Result_Var_1 := Fld_5;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5468
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_877;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_877;
                     Finalizer_Scope_2623;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Parent_Instantiation_Env;
--# end
--# property-start GenericInstantiation.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11324
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Xref_Entry_Point
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Xref_Entry_Point;
--# end
--# property-start GenericInstantiation.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11326
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Xref_Equation
  (Node : Bare_Generic_Instantiation
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_879 with Inline_Always;
            procedure Finalizer_Scope_2624 with Inline_Always;
            procedure Finalizer_Scope_2625 with Inline_Always;
            procedure Finalizer_Scope_2626 with Inline_Always;
            procedure Finalizer_Scope_2628 with Inline_Always;
            procedure Finalizer_Scope_2629 with Inline_Always;
            procedure Finalizer_Scope_2630 with Inline_Always;
            procedure Finalizer_Scope_2631 with Inline_Always;
            procedure Finalizer_Scope_2632 with Inline_Always;
            procedure Finalizer_Scope_2633 with Inline_Always;
            procedure Finalizer_Scope_2634 with Inline_Always;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Name;
Var_Expr : Internal_Entity_Generic_Decl;
Actual_Name : Internal_Entity_Name;
Match_0_1 : Internal_Entity_Base_Type_Decl;
Match_1_1 : Internal_Entity_Generic_Package_Instantiation;
Match_2 : Internal_Entity_Formal_Subp_Decl;
Match_3 : Internal_Entity_Object_Decl;
Match_4 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Bare_Base_Id;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Generic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Match_Result : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel_1 : Logic_Equation;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Generic_Decl;
Fld_7 : Internal_Entity_Generic_Formal_Part;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Internal_Param_Match_Array_Access;
Scope_Result_2 : Internal_Param_Match_Array_Access;
Result_Var : Internal_Param_Match_Array_Access;
Fld_10 : Internal_Single_Actual;
Fld_11 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Is_A : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_13 : Internal_Single_Actual;
Fld_14 : Internal_Entity_Basic_Assoc;
Fld_15 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Name;
Fld_16 : Internal_Entity_Defining_Name;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_5 : Internal_Entity_Generic_Formal;
Fld_18 : Internal_Entity_Basic_Decl;
Fld_19 : Logic_Equation;
Fld_20 : Logic_Equation;
Fld_21 : Logic_Equation;
Fld_22 : Logic_Equation;
Fld_23 : Logic_Var;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel_2 : Logic_Equation;
Or_Pred : Logic_Equation;
Is_A_1 : Boolean;
If_Result : Logic_Equation;
Fld_24 : Internal_Single_Actual;
Fld_25 : Internal_Entity_Basic_Assoc;
Fld_26 : Internal_Entity_Expr;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_29 : Internal_Single_Actual;
Fld_30 : Internal_Entity_Basic_Assoc;
Fld_31 : Internal_Entity_Expr;
Fld_32 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_33 : Internal_Single_Actual;
Fld_34 : Internal_Entity_Basic_Assoc;
Fld_35 : Internal_Entity_Expr;
Fld_36 : Logic_Equation;
And_Pred_3 : Logic_Equation;
True_Rel_3 : Logic_Equation;
Match_Prefix_1 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_9 : Internal_Entity_Generic_Package_Instantiation;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Result_10 : Internal_Entity_Formal_Subp_Decl;
Let_Result_4 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Cast_Result_11 : Internal_Entity_Object_Decl;
Let_Result_5 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_12 : Internal_Entity_Basic_Decl;
Let_Result_6 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result_1 : Logic_Equation;
Fld_37 : Internal_Single_Actual;
Fld_38 : Bare_Base_Id;
Fld_39 : Logic_Var;
Fld_40 : Internal_Entity_Defining_Name;
Fld_41 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Defining_Name;
Fld_42 : Internal_Entity_Basic_Decl;
Cast_Result_13 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
True_Rel_4 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result_7 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_43 : Boolean;
If_Result_2 : Logic_Equation;
And_Pred_5 : Logic_Equation;
            procedure Finalizer_Scope_879 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (And_Pred_5);
            end Finalizer_Scope_879;
            procedure Finalizer_Scope_2624 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2624;
            procedure Finalizer_Scope_2625 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_2625;
            procedure Finalizer_Scope_2626 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2626;
            procedure Finalizer_Scope_2628 is
            begin
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (True_Rel_4);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result_7);
            end Finalizer_Scope_2628;
            procedure Finalizer_Scope_2629 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_2629;
            procedure Finalizer_Scope_2630 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_2630;
            procedure Finalizer_Scope_2631 is
            begin
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2631;
            procedure Finalizer_Scope_2632 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_32);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_36);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2632;
            procedure Finalizer_Scope_2633 is
            begin
                     Dec_Ref (True_Rel_3);
            end Finalizer_Scope_2633;
            procedure Finalizer_Scope_2634 is
            begin
                     Dec_Ref (Bind_Result_2);
            end Finalizer_Scope_2634;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5539 '<And at ast.py:11327>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11327
--# expr-start 5477 '<And at ast.py:11327>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11327
--# expr-start 5472 '<Bind at ast.py:11327>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11327
--# expr-start 5470 '<FieldAccess for ref_var at ast.py:11327>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11327
--# expr-start 5469 '<FieldAccess for generic_entity_name at ast.py:11327>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11327
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5469
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld.Node);
--# end
--# expr-done 5470
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 5471 '<FieldAccess for nonbound_generic_decl at ast.py:11328>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11328
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld_2 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Ent.Node);
--# end
--# expr-done 5471
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11327") else null)); 
--# expr-done 5472
--# expr-start 5476 '<Match at ast.py:11329>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11329
--# expr-start 5473 '<FieldAccess for generic_entity_name at ast.py:11329>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11329
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld_3 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5473
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_3; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind dn Match_0
--# expr-start 5475 '<FieldAccess for xref_no_overloading at ast.py:11330>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11330
--# expr-start 5474 '<FieldAccess for prefix at ast.py:11330>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11330
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 5474
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5475
Let_Result := Fld_5; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2624;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11331") else null)); 
Let_Result_1 := True_Rel; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2625;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 5476
And_Pred := Create_And (Bind_Result, Match_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11327") else null)); 
--# expr-done 5477
--# expr-start 5538 '<If at ast.py:11332>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11332
--# expr-start 5478 '<FieldAccess for is_any_formal at ast.py:11333>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11333
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.is_any_formal
Fld_43 := Libadalang.Implementation.Generic_Instantiation_P_Is_Any_Formal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5478
if Fld_43 then
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11334") else null)); 
   If_Result_2 := True_Rel_1;
else
   --# expr-start 5537 '<LogicAll at ast.py:11341>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11341
--# expr-start 5536 '<Map at ast.py:11341>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11341
   --# expr-start 5489 '<Filter at ast.py:11339>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11339
--# expr-start 5480 '<Cast to GenericDecl at ast.py:11336>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11336
--# expr-start 5479 '<FieldAccess for designated_generic_decl at ast.py:11336>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11336
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_6 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5479
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Generic_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5480
Var_Expr := Cast_Result_3;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 5483 '<FieldAccess for match_param_list at ast.py:11337>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11337
--# expr-start 5481 '<FieldAccess for formal_part at ast.py:11337>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11337
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr.Info);
--# expr-done 5481
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5482 '<FieldAccess for generic_inst_params at ast.py:11338>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11338
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld_8 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5482
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_7.Node, Params => Fld_8, Is_Dottable_Subp => False, Env => Env, E_Info => Fld_7.Info);
--# end
--# expr-done 5483
Scope_Result_2 := Fld_9;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2626;
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var);
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Result_Var;
      begin
         for Item_189 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_189
         --# expr-start 5488 '<Not at ast.py:11340>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11340
--# expr-start 5487 '<IsA BoxExpr at ast.py:11340>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11340
--# expr-start 5486 '<FieldAccess for expr at ast.py:11340>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11340
--# expr-start 5485 '<FieldAccess for assoc at ast.py:11340>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11340
--# expr-start 5484 '<FieldAccess for actual at ast.py:11340>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11340
Fld_10 := Item_189.Actual;
--# expr-done 5484
Fld_11 := Fld_10.Assoc;
--# expr-done 5485
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 5486
Is_A := Fld_12.Node /= null 
and then Fld_12.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 5487
Not_Val := not (Is_A); 
--# expr-done 5488
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_189;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5489
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Map_Result;
      begin
         for Item_188 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_188
      --# expr-start 5535 '<Let at ast.py:11341>' Let_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11341
--# scope-start
--# expr-start 5493 '<Cast to Name at ast.py:11342>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11342
--# expr-start 5492 '<FieldAccess for expr at ast.py:11342>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11342
--# expr-start 5491 '<FieldAccess for assoc at ast.py:11342>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11342
--# expr-start 5490 '<FieldAccess for actual at ast.py:11342>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11342
Fld_13 := Item_188.Actual;
--# expr-done 5490
Fld_14 := Fld_13.Assoc;
--# expr-done 5491
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 5492
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Name
   then
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_4 := No_Entity_Name;
   end if;
--# expr-done 5493
Actual_Name := Cast_Result_4; 
--# bind actual_name Actual_Name
--# expr-start 5534 '<And at ast.py:11343>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11343
--# expr-start 5525 '<Match at ast.py:11343>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11343
--# expr-start 5497 '<FieldAccess for decl at ast.py:11343>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11343
--# expr-start 5496 '<Cast to GenericFormal at ast.py:11343>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11343
--# expr-start 5495 '<FieldAccess for formal_decl at ast.py:11343>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11343
--# expr-start 5494 '<FieldAccess for formal at ast.py:11343>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11343
Fld_16 := Item_188.Formal;
--# expr-done 5494
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_17 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 5495
   if Fld_17.Node = null
      or else Fld_17.Node.Kind in Ada_Generic_Formal
   then
      Cast_Result_5 := Create_Internal_Entity_Generic_Formal
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
   else
         Cast_Result_5 := No_Entity_Generic_Formal;
   end if;
--# expr-done 5496
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Basic_Decl (Node => Cast_Result_5.Node.Generic_Formal_F_Decl, Info => Cast_Result_5.Info);
--# expr-done 5497
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld_18; 
case Ada_Basic_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0_1 := Cast_Result_8; 
--# bind _ Match_0_1
--# expr-start 5498 '<FieldAccess for xref_no_overloading at ast.py:11344>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11344
      if Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_19 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Actual_Name.Node, Env => Env, Origin => Origin, E_Info => Actual_Name.Info);
--# end
--# expr-done 5498
Let_Result_2 := Fld_19; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2629;
            Match_Result_1 := Scope_Result_3; Inc_Ref (Match_Result_1);
         when Ada_Generic_Package_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1_1 := Cast_Result_9; 
--# bind _ Match_1_1
--# expr-start 5499 '<FieldAccess for xref_no_overloading at ast.py:11347>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11347
      if Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_20 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Actual_Name.Node, Env => Env, Origin => Origin, E_Info => Actual_Name.Info);
--# end
--# expr-done 5499
Let_Result_3 := Fld_20; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2630;
            Match_Result_1 := Scope_Result_4; Inc_Ref (Match_Result_1);
         when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_10; 
--# bind subp_decl Match_2
--# expr-start 5508 '<If at ast.py:11350>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11350
--# expr-start 5500 '<IsA AttributeRef at ast.py:11351>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11351
Is_A_1 := Actual_Name.Node /= null 
and then Actual_Name.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 5500
if Is_A_1 then
   --# expr-start 5501 '<FieldAccess for sub_equation at ast.py:11352>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11352
      if Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_21 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Actual_Name.Node, Env => Env, Origin => Origin, E_Info => Actual_Name.Info);
--# end
--# expr-done 5501
   If_Result := Fld_21;
else
   --# expr-start 5507 '<Or at ast.py:11353>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11353
--# expr-start 5506 '<And at ast.py:11353>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11353
--# expr-start 5502 '<FieldAccess for xref_no_overloading at ast.py:11353>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11353
      if Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_22 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Actual_Name.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Actual_Name.Info);
--# end
--# expr-done 5502
--# expr-start 5505 '<Predicate on BasicDecl.subp_decl_match_signature at ast.py:11354>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11354
--# expr-start 5503 '<FieldAccess for ref_var at ast.py:11355>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11355
      if Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Actual_Name.Node);
--# end
--# expr-done 5503
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
--# expr-start 5504 '<Cast to BasicDecl at ast.py:11356>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11356
Cast_Expr := Match_2; 
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 5504
Pred := Solver.Create_Predicate (Fld_23, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result_6, Origin)); 
--# expr-done 5505
And_Pred_1 := Create_And (Fld_22, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11353") else null)); 
--# expr-done 5506
True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11357") else null)); 
Or_Pred := Create_Or (And_Pred_1, True_Rel_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11353") else null)); 
--# expr-done 5507
   If_Result := Or_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 5508
Let_Result_4 := If_Result; Inc_Ref (Let_Result_4);
--# end
Scope_Result_5 := Let_Result_4;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2631;
            Match_Result_1 := Scope_Result_5; Inc_Ref (Match_Result_1);
         when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Object_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_11; 
--# bind obj_decl Match_3
--# expr-start 5524 '<And at ast.py:11361>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11361
--# expr-start 5519 '<And at ast.py:11361>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11361
--# expr-start 5514 '<Bind at ast.py:11361>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11361
--# expr-start 5512 '<FieldAccess for expected_type_var at ast.py:11361>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11361
--# expr-start 5511 '<FieldAccess for expr at ast.py:11361>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11361
--# expr-start 5510 '<FieldAccess for assoc at ast.py:11361>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11361
--# expr-start 5509 '<FieldAccess for actual at ast.py:11361>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11361
Fld_24 := Item_188.Actual;
--# expr-done 5509
Fld_25 := Fld_24.Assoc;
--# expr-done 5510
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_26 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_25.Node, E_Info => Fld_25.Info);
--# end
--# expr-done 5511
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_26.Node);
--# expr-done 5512
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 5513 '<FieldAccess for expr_type at ast.py:11362>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11362
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_28 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_3.Node, Origin => Origin, E_Info => Match_3.Info);
--# end
--# expr-done 5513
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_27, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11361") else null)); 
--# expr-done 5514
--# expr-start 5518 '<FieldAccess for sub_equation at ast.py:11363>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11363
--# expr-start 5517 '<FieldAccess for expr at ast.py:11363>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11363
--# expr-start 5516 '<FieldAccess for assoc at ast.py:11363>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11363
--# expr-start 5515 '<FieldAccess for actual at ast.py:11363>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11363
Fld_29 := Item_188.Actual;
--# expr-done 5515
Fld_30 := Fld_29.Assoc;
--# expr-done 5516
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_31 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_30.Node, E_Info => Fld_30.Info);
--# end
--# expr-done 5517
      if Fld_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_32 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_31.Node, Env => Env, Origin => Origin, E_Info => Fld_31.Info);
--# end
--# expr-done 5518
And_Pred_2 := Create_And (Bind_Result_1, Fld_32, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11361") else null)); 
--# expr-done 5519
--# expr-start 5523 '<FieldAccess for matches_expected_assign_type at ast.py:11364>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11364
--# expr-start 5522 '<FieldAccess for expr at ast.py:11364>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11364
--# expr-start 5521 '<FieldAccess for assoc at ast.py:11364>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11364
--# expr-start 5520 '<FieldAccess for actual at ast.py:11364>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11364
Fld_33 := Item_188.Actual;
--# expr-done 5520
Fld_34 := Fld_33.Assoc;
--# expr-done 5521
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_35 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_34.Node, E_Info => Fld_34.Info);
--# end
--# expr-done 5522
      if Fld_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_36 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_35.Node, Origin => Origin);
--# end
--# expr-done 5523
And_Pred_3 := Create_And (And_Pred_2, Fld_36, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11361") else null)); 
--# expr-done 5524
Let_Result_5 := And_Pred_3; Inc_Ref (Let_Result_5);
--# end
Scope_Result_6 := Let_Result_5;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2632;
            Match_Result_1 := Scope_Result_6; Inc_Ref (Match_Result_1);
         when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Generic_Subp_Decl | Ada_Generic_Subp_Instantiation .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_4 := Cast_Result_12; 
--# bind _ Match_4
True_Rel_3 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11366") else null)); 
Let_Result_6 := True_Rel_3; Inc_Ref (Let_Result_6);
--# end
Scope_Result_7 := Let_Result_6;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2633;
            Match_Result_1 := Scope_Result_7; Inc_Ref (Match_Result_1);
end case;
--# expr-done 5525
--# expr-start 5527 '<FieldAccess for name at ast.py:11367>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11367
--# expr-start 5526 '<FieldAccess for actual at ast.py:11367>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11367
Fld_37 := Item_188.Actual;
--# expr-done 5526
Fld_38 := Fld_37.Name;
--# expr-done 5527
Var_Expr_1 := Fld_38;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr_1
--# expr-start 5533 '<Bind at ast.py:11369>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11369
--# expr-start 5528 '<FieldAccess for ref_var at ast.py:11369>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11369
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_39 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr_1);
--# end
--# expr-done 5528
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
--# expr-start 5532 '<FieldAccess for basic_decl at ast.py:11370>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11370
--# expr-start 5531 '<as_bare_entity at ast.py:11370>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11370
--# expr-start 5530 '<FieldAccess for node at ast.py:11370>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11370
--# expr-start 5529 '<FieldAccess for formal at ast.py:11370>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11370
Fld_40 := Item_188.Formal;
--# expr-done 5529
Fld_41 := Fld_40.Node;
--# expr-done 5530
Is_Null := Fld_41 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Defining_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_41); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 5531
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_42 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 5532
      Cast_Result_13 := Create_Internal_Entity
        (Node => Fld_42.Node,
         Info => Fld_42.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_39, Cast_Result_13, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11369") else null)); 
--# expr-done 5533
Scope_Result_8 := Bind_Result_2;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2634;
   Result_Var_1 := Scope_Result_8;
else
   True_Rel_4 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11371") else null)); 
   Result_Var_1 := True_Rel_4;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_4 := Create_And (Match_Result_1, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11343") else null)); 
--# expr-done 5534
Let_Result_7 := And_Pred_4; Inc_Ref (Let_Result_7);
--# end
--# expr-done 5535
         declare
            Item_To_Append : constant Logic_Equation := Let_Result_7;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2628;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 5536
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11341") else null)); 
--# expr-done 5537
   If_Result_2 := Logic_Boolean_Op;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 5538
And_Pred_5 := Create_And (And_Pred, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:11327") else null)); 
--# expr-done 5539
         Property_Result := And_Pred_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_879;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_879;
                     Finalizer_Scope_2624;
                     Finalizer_Scope_2625;
                     Finalizer_Scope_2626;
                     Finalizer_Scope_2628;
                     Finalizer_Scope_2629;
                     Finalizer_Scope_2630;
                     Finalizer_Scope_2631;
                     Finalizer_Scope_2632;
                     Finalizer_Scope_2633;
                     Finalizer_Scope_2634;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Xref_Equation;
--# end
--# property-start GenericInstantiation.initial_env_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11377
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Initial_Env_Name
  (Node : Bare_Generic_Instantiation
  )
   return Symbol_Type
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Fld_1 : Boolean;
If_Result : Symbol_Type;
Let_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5543 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5542 '<If at ast.py:11379>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11379
--# expr-start 5540 '<FieldAccess for is_library_item at ast.py:11380>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11380
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 5540
if Fld_1 then
   --# expr-start 5541 '<FieldAccess for child_decl_initial_env_name at ast.py:11381>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11381
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self);
--# end
--# expr-done 5541
   If_Result := Fld;
else
   If_Result := null;
end if;
--# expr-done 5542
Let_Result := If_Result; 
--# end
--# expr-done 5543
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Initial_Env_Name;
--# end
--# property-start GenericInstantiation.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11385
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Env_Names
  (Node : Bare_Generic_Instantiation
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_881 with Inline_Always;
            procedure Finalizer_Scope_2635 with Inline_Always;
      Var_Expr : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_881 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_881;
            procedure Finalizer_Scope_2635 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2635;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5546 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5544 '<FieldAccess for top_level_env_name at ast.py:11387>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11387
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5544
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 5545 '<to_symbol at ast.py:11388>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11388
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr); 
--# expr-done 5545
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2635;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 5546
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_881;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_881;
                     Finalizer_Scope_2635;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Env_Names;
--# end
--# property-start GenericInstantiation.inst_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11423
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Inst_Params
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_882 with Inline_Always;
            procedure Finalizer_Scope_2637 with Inline_Always;
            procedure Finalizer_Scope_2638 with Inline_Always;
            procedure Finalizer_Scope_2639 with Inline_Always;
            procedure Finalizer_Scope_2640 with Inline_Always;
            procedure Finalizer_Scope_2642 with Inline_Always;
      Block_Var : Internal_Entity_Assoc_List;
Var_Expr : Internal_Entity_Generic_Decl;
Match_0 : Internal_Entity_Generic_Formal_Package;
Match_1 : Internal_Entity_Generic_Formal_Type_Decl;
Match_2 : Internal_Entity_Generic_Formal_Obj_Decl;
Match_3 : Internal_Entity_Generic_Formal_Subp_Decl;
Match_4 : Internal_Entity_Pragma_Node;
Match_5 : Internal_Entity_Use_Clause;
Match_6 : Internal_Entity;
Subp : Internal_Entity_Formal_Subp_Decl;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Generic_Formal_Part;
Fld_3 : Internal_Entity_Ada_Node_List;
Scope_Result : Internal_Entity_Ada_Node_List;
Result_Var : Internal_Entity_Ada_Node_List;
Fld_4 : Bare_Ada_Node_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Package_Instantiation;
Fld_7 : Internal_Entity_Defining_Name;
New_Struct : Internal_Param_Actual;
Singleton : Internal_Param_Actual_Array_Access;
Fld_8 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Defining_Name;
New_Struct_1 : Internal_Param_Actual;
Singleton_1 : Internal_Param_Actual_Array_Access;
Fld_10 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Object_Decl;
Fld_11 : Internal_Entity_Defining_Name_List;
Fld_12 : Bare_Defining_Name_List;
Fld_13 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Defining_Name;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Defining_Name;
Fld_14 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Object_Decl;
Fld_15 : Internal_Entity_Expr;
New_Struct_2 : Internal_Param_Actual;
Item_193 : Internal_Entity_Defining_Name;
Bare_Item_193 : Bare_Defining_Name;
Map_Result : Internal_Param_Actual_Array_Access;
Fld_16 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Formal_Subp_Decl;
Fld_17 : Internal_Entity_Defining_Name;
Fld_18 : Internal_Entity_Basic_Decl;
Fld_19 : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Fld_20 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Expr;
Fld_21 : Internal_Entity_Expr;
Is_A : Boolean;
If_Result_2 : Internal_Entity_Expr;
New_Struct_3 : Internal_Param_Actual;
Let_Result : Internal_Param_Actual;
Singleton_2 : Internal_Param_Actual_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_6 : Internal_Entity_Generic_Formal_Package;
Let_Result_1 : Internal_Param_Actual_Array_Access;
Scope_Result_2 : Internal_Param_Actual_Array_Access;
Cast_Result_7 : Internal_Entity_Generic_Formal_Type_Decl;
Let_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_3 : Internal_Param_Actual_Array_Access;
Cast_Result_8 : Internal_Entity_Generic_Formal_Obj_Decl;
Let_Result_3 : Internal_Param_Actual_Array_Access;
Scope_Result_4 : Internal_Param_Actual_Array_Access;
Cast_Result_9 : Internal_Entity_Generic_Formal_Subp_Decl;
Let_Result_4 : Internal_Param_Actual_Array_Access;
Scope_Result_5 : Internal_Param_Actual_Array_Access;
Cast_Result_10 : Internal_Entity_Pragma_Node;
Let_Result_5 : Internal_Param_Actual_Array_Access;
Scope_Result_6 : Internal_Param_Actual_Array_Access;
Cast_Result_11 : Internal_Entity_Use_Clause;
Let_Result_6 : Internal_Param_Actual_Array_Access;
Scope_Result_7 : Internal_Param_Actual_Array_Access;
Cast_Result_12 : Internal_Entity;
Let_Result_7 : Internal_Param_Actual_Array_Access;
Scope_Result_8 : Internal_Param_Actual_Array_Access;
Match_Result : Internal_Param_Actual_Array_Access;
Item_192 : Internal_Entity;
Bare_Item_192 : Bare_Ada_Node;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Fld_22 : Internal_Entity_Defining_Name;
Fld_23 : Internal_Entity_Defining_Name;
Fld_24 : Internal_Entity_Expr;
Fld_25 : Internal_Entity_Expr;
New_Struct_4 : Internal_Param_Actual;
Index_191 : Integer;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Let_Result_8 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_882 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result_8);
            end Finalizer_Scope_882;
            procedure Finalizer_Scope_2637 is
            begin
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_7);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2637;
            procedure Finalizer_Scope_2638 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2638;
            procedure Finalizer_Scope_2639 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_2639;
            procedure Finalizer_Scope_2640 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2640;
            procedure Finalizer_Scope_2642 is
            begin
                     Dec_Ref (Singleton_2);
            end Finalizer_Scope_2642;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5586 '<Block at ???>' Let_Result_8 None
--# scope-start
--# expr-start 5547 '<FieldAccess for generic_inst_params at ast.py:11429>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11429
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5547
Block_Var := Fld; 
--# bind ap Block_Var
--# expr-start 5585 '<Map at ast.py:11470>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11470
   --# expr-start 5579 '<Mapcat at ast.py:11431>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11431
--# expr-start 5548 '<FieldAccess for nonbound_generic_decl at ast.py:11431>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11431
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld_1 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Ent.Node);
--# end
--# expr-done 5548
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 5550 '<FieldAccess for decls at ast.py:11431>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11431
--# expr-start 5549 '<FieldAccess for formal_part at ast.py:11431>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11431
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr.Info);
--# expr-done 5549
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Ada_Node_List (Node => Fld_2.Node.Generic_Formal_Part_F_Decls, Info => Fld_2.Info);
--# expr-done 5550
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Ada_Node_List;
end if;
Fld_4 := Result_Var.Node;
      declare
         Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      if Fld_4 = null then
      Map_Result_1 := Create_Internal_Param_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_4;
      begin
         for Untyped_Item_192 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_192 := Untyped_Item_192; 
Is_Null := Bare_Item_192 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_5 := Result_Var.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_192); 
   If_Result := As_Entity;
end if;
                  Item_192 := If_Result; 
   --# scope-start
               --# bind d Item_192
      --# expr-start 5578 '<Match at ast.py:11433>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11433
      if Item_192.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_192; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Formal_Package =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Generic_Formal_Package
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_6; 
--# bind t Match_0
--# expr-start 5554 '<New ParamActual at ast.py:11434>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11434
--# expr-start 5553 '<FieldAccess for name at ast.py:11435>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11435
--# expr-start 5552 '<Cast to GenericPackageInstantiation at ast.py:11435>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11435
--# expr-start 5551 '<FieldAccess for decl at ast.py:11435>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11435
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Basic_Decl (Node => Match_0.Node.Generic_Formal_F_Decl, Info => Match_0.Info);
--# expr-done 5551
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 5552
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Defining_Name (Node => Cast_Result.Node.Generic_Package_Instantiation_F_Name, Info => Cast_Result.Info);
--# expr-done 5553
New_Struct := (Actual => No_Entity_Expr, Param => Fld_7); 
--# expr-done 5554
                Singleton := Create_Internal_Param_Actual_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
Let_Result_1 := Singleton; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2638;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Generic_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_7; 
--# bind t Match_1
--# expr-start 5558 '<New ParamActual at ast.py:11438>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11438
--# expr-start 5557 '<FieldAccess for name at ast.py:11439>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11439
--# expr-start 5556 '<Cast to BaseTypeDecl at ast.py:11439>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11439
--# expr-start 5555 '<FieldAccess for decl at ast.py:11439>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11439
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Basic_Decl (Node => Match_1.Node.Generic_Formal_F_Decl, Info => Match_1.Info);
--# expr-done 5555
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5556
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Defining_Name (Node => Cast_Result_1.Node.Base_Type_Decl_F_Name, Info => Cast_Result_1.Info);
--# expr-done 5557
New_Struct_1 := (Actual => No_Entity_Expr, Param => Fld_9); 
--# expr-done 5558
                Singleton_1 := Create_Internal_Param_Actual_Array (Items_Count => 1);
                Singleton_1.Items (1) := New_Struct_1;
Let_Result_2 := Singleton_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2639;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Generic_Formal_Obj_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Generic_Formal_Obj_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_8; 
--# bind o Match_2
--# expr-start 5566 '<Map at ast.py:11443>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11443
   --# expr-start 5561 '<FieldAccess for ids at ast.py:11443>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11443
--# expr-start 5560 '<Cast to ObjectDecl at ast.py:11443>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11443
--# expr-start 5559 '<FieldAccess for decl at ast.py:11443>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11443
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Basic_Decl (Node => Match_2.Node.Generic_Formal_F_Decl, Info => Match_2.Info);
--# expr-done 5559
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Object_Decl
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_2 := No_Entity_Object_Decl;
   end if;
--# expr-done 5560
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Defining_Name_List (Node => Cast_Result_2.Node.Object_Decl_F_Ids, Info => Cast_Result_2.Info);
--# expr-done 5561
Fld_12 := Fld_11.Node;
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      if Fld_12 = null then
      Map_Result := Create_Internal_Param_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_12;
      begin
         for Untyped_Item_193 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_193 := Untyped_Item_193; 
Is_Null_1 := Bare_Item_193 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Defining_Name;
else
Fld_13 := Fld_11.Info;
As_Entity_1 := (Info => Fld_13, Node => Bare_Item_193); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_193 := If_Result_1; 
   --# scope-start
               --# bind i Item_193
      --# expr-start 5565 '<New ParamActual at ast.py:11444>' New_Struct_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11444
--# expr-start 5564 '<FieldAccess for default_expr at ast.py:11446>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11446
--# expr-start 5563 '<Cast to ObjectDecl at ast.py:11446>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11446
--# expr-start 5562 '<FieldAccess for decl at ast.py:11446>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11446
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Basic_Decl (Node => Match_2.Node.Generic_Formal_F_Decl, Info => Match_2.Info);
--# expr-done 5562
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Object_Decl
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         Cast_Result_3 := No_Entity_Object_Decl;
   end if;
--# expr-done 5563
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Expr (Node => Cast_Result_3.Node.Object_Decl_F_Default_Expr, Info => Cast_Result_3.Info);
--# expr-done 5564
New_Struct_2 := (Actual => Fld_15, Param => Item_193); 
--# expr-done 5565
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_2;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5566
Let_Result_3 := Map_Result; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2640;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Ada_Generic_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_9; 
--# bind sp Match_3
--# expr-start 5577 '<Let at ast.py:11449>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11449
--# scope-start
--# expr-start 5568 '<Cast to FormalSubpDecl at ast.py:11450>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11450
--# expr-start 5567 '<FieldAccess for decl at ast.py:11450>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11450
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Basic_Decl (Node => Match_3.Node.Generic_Formal_F_Decl, Info => Match_3.Info);
--# expr-done 5567
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Formal_Subp_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result_4 := No_Entity_Formal_Subp_Decl;
   end if;
--# expr-done 5568
Subp := Cast_Result_4; 
--# bind subp Subp
--# expr-start 5576 '<New ParamActual at ast.py:11452>' New_Struct_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11452
--# expr-start 5574 '<If at ast.py:11454>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11454
--# expr-start 5570 '<IsA BoxExpr at ast.py:11455>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11455
--# expr-start 5569 '<FieldAccess for default_expr at ast.py:11455>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11455
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Expr (Node => Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Subp.Info);
--# expr-done 5569
Is_A := Fld_21.Node /= null 
and then Fld_21.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 5570
if Is_A then
--# expr-start 5571 '<FieldAccess for designated_subprogram_from at ast.py:11456>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11456
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start FormalSubpDecl.designated_subprogram_from
Fld_18 := Libadalang.Implementation.Formal_Subp_Decl_P_Designated_Subprogram_From (Node => Subp.Node, Inst => Ent, E_Info => Subp.Info);
--# end
--# expr-done 5571
Var_Expr_1 := Fld_18;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5572 '<FieldAccess for defining_name at ast.py:11458>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11458
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_19 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 5572
Scope_Result_1 := Fld_19;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Defining_Name;
end if;
      Cast_Result_5 := Create_Internal_Entity_Expr
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   If_Result_2 := Cast_Result_5;
else
   --# expr-start 5573 '<FieldAccess for default_expr at ast.py:11459>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11459
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Expr (Node => Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Subp.Info);
--# expr-done 5573
   If_Result_2 := Fld_20;
end if;
--# expr-done 5574
--# expr-start 5575 '<FieldAccess for defining_name at ast.py:11453>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11453
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 5575
New_Struct_3 := (Actual => If_Result_2, Param => Fld_17); 
--# expr-done 5576
Let_Result := New_Struct_3; 
--# end
--# expr-done 5577
                Singleton_2 := Create_Internal_Param_Actual_Array (Items_Count => 1);
                Singleton_2.Items (1) := Let_Result;
Let_Result_4 := Singleton_2; Inc_Ref (Let_Result_4);
--# end
Scope_Result_5 := Let_Result_4;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2642;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
         when Ada_Pragma_Node =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Pragma_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_10; 
--# bind _ Match_4
Let_Result_5 := No_Internal_Param_Actual_Array_Type; Inc_Ref (Let_Result_5);
--# end
Scope_Result_6 := Let_Result_5;
   Inc_Ref (Scope_Result_6);
   --# end
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Use_Package_Clause .. Ada_Use_Type_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Use_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_11; 
--# bind _ Match_5
Let_Result_6 := No_Internal_Param_Actual_Array_Type; Inc_Ref (Let_Result_6);
--# end
Scope_Result_7 := Let_Result_6;
   Inc_Ref (Scope_Result_7);
   --# end
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Discriminant_Spec | Ada_Param_Spec .. Ada_Pp_Then_Kw | Ada_Private_Absent .. Ada_Until_Present | Ada_Value_Sequence .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_12; 
--# bind _ Match_6
Let_Result_7 := raise Property_Error with "unexpected declaration in generic formal part"; Inc_Ref (Let_Result_7);
--# end
Scope_Result_8 := Let_Result_7;
   Inc_Ref (Scope_Result_8);
   --# end
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
end case;
--# expr-done 5578
         for Item_To_Append of
               Match_Result.Items
         loop
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2637;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 5579
      declare
         Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
         Index_191 := 0;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Map_Result_1;
      begin
         for Item_190 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_190
               --# bind i Index_191
      --# expr-start 5584 '<New ParamActual at ast.py:11472>' New_Struct_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11472
--# expr-start 5582 '<FieldAccess for actual_for_param_at at ast.py:11474>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11474
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5580 '<FieldAccess for param at ast.py:11474>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11474
Fld_23 := Item_190.Param;
--# expr-done 5580
--# expr-start 5581 '<FieldAccess for actual at ast.py:11474>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11474
Fld_24 := Item_190.Actual;
--# expr-done 5581
--# property-call-start AssocList.actual_for_param_at
Fld_25 := Libadalang.Implementation.Assoc_List_P_Actual_For_Param_At (Node => Block_Var.Node, Param => Fld_23, Pos => Index_191, Default_Expr => Fld_24, E_Info => Block_Var.Info);
--# end
--# expr-done 5582
--# expr-start 5583 '<FieldAccess for param at ast.py:11473>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11473
Fld_22 := Item_190.Param;
--# expr-done 5583
New_Struct_4 := (Actual => Fld_25, Param => Fld_22); 
--# expr-done 5584
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_4;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
               Index_191 := Index_191 + 1;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 5585
Let_Result_8 := Map_Result_2; Inc_Ref (Let_Result_8);
--# end
--# expr-done 5586
         Property_Result := Let_Result_8;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_882;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_882;
                     Finalizer_Scope_2637;
                     Finalizer_Scope_2638;
                     Finalizer_Scope_2639;
                     Finalizer_Scope_2640;
                     Finalizer_Scope_2642;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Inst_Params;
--# end
--# property-start GenericInstantiation._env_do_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11392
pragma Warnings (Off, "is not referenced");
 function Env_Do_59
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5587 '<FieldAccess for env_hook at ast.py:11392>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11392
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5587
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_59;
--# end
--# property-start GenericInstantiation._initial_env_60 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11395
pragma Warnings (Off, "is not referenced");
 function Initial_Env_60
  (Node : Bare_Generic_Instantiation
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1587 with Inline_Always;
            procedure Finalizer_Scope_2648 with Inline_Always;
      Var_Expr : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1587 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1587;
            procedure Finalizer_Scope_2648 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2648;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5588 '<FieldAccess for initial_env_name at ast.py:11396>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11396
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.initial_env_name
Fld := Libadalang.Implementation.Generic_Instantiation_P_Initial_Env_Name (Node => Self);
--# end
--# expr-done 5588
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind non_null_name Var_Expr
--# expr-start 5589 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr, Kind => Named_Env); 
--# expr-done 5589
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2648;
   Result_Var := Scope_Result;
else
   --# expr-start 5590 '<New DesignatedEnv at ast.py:11395>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11395
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5590
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1587;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1587;
                     Finalizer_Scope_2648;
            raise;
      end;
   return Property_Result;
end Initial_Env_60;
--# end
--# property-start GenericInstantiation._env_mappings_61 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11401
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_61
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1588 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1588 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1588;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5593 '<New EnvAssoc at ast.py:11401>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11401
--# expr-start 5591 '<New DesignatedEnv at ast.py:11401>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11401
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5591
--# expr-start 5592 '<FieldAccess for name_symbol at ast.py:11401>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11401
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5592
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5593
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1588;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1588;
            raise;
      end;
   return Property_Result;
end Env_Mappings_61;
--# end
--# property-start GenericInstantiation._env_trans_parent_62 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11391
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_62
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_62;
--# end
--# property-start GenericInstantiation._env_names_63 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11403
pragma Warnings (Off, "is not referenced");
 function Env_Names_63
  (Node : Bare_Generic_Instantiation
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1590 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1590 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1590;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5594 '<FieldAccess for env_names at ast.py:11403>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11403
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.env_names
Fld := Libadalang.Implementation.Generic_Instantiation_P_Env_Names (Node => Self);
--# end
--# expr-done 5594
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1590;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1590;
            raise;
      end;
   return Property_Result;
end Env_Names_63;
--# end
--# property-start GenericInstantiation._env_do_64 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11405
pragma Warnings (Off, "is not referenced");
 function Env_Do_64
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1591 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1591 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1591;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5595 '<FieldAccess for populate_dependent_units at ast.py:11405>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11405
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5595
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1591;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1591;
            raise;
      end;
   return Property_Result;
end Env_Do_64;
--# end
--# property-start GenericInstantiation._ref_env_nodes_65 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11407
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_65
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1592 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1592 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1592;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5596 '<FieldAccess for top_level_use_package_clauses at ast.py:11407>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11407
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5596
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1592;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1592;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_65;
--# end
--# property-start GenericInstantiation._ref_cond_66 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11409
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_66
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5598 '<IsA LibraryItem, Subunit at ast.py:11409>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11409
--# expr-start 5597 '<parent at ast.py:11409>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11409
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5597
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5598
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_66;
--# end
--# property-start GenericInstantiation._ref_env_nodes_67 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11412
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_67
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1594 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1594 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1594;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5599 '<FieldAccess for top_level_use_type_clauses at ast.py:11412>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11412
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5599
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1594;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1594;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_67;
--# end
--# property-start GenericInstantiation._ref_cond_68 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11414
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_68
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5601 '<IsA LibraryItem, Subunit at ast.py:11414>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11414
--# expr-start 5600 '<parent at ast.py:11414>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11414
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5600
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5601
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_68;
--# end
--# property-start GenericInstantiation._ref_env_nodes_69 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11417
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_69
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1596 with Inline_Always;
      Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1596 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1596;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5602 '<Cast to AdaNode at ast.py:11417>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11417
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5602
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1596;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1596;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_69;
--# end
--# property-start GenericInstantiation._ref_cond_70 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11419
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_70
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5604 '<IsA LibraryItem, Subunit at ast.py:11419>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11419
--# expr-start 5603 '<parent at ast.py:11419>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11419
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5603
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5604
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_70;
--# end
function Generic_Instantiation_P_Instantiation_Bindings
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Generic_Instantiation
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Internal_Inner_Env_Assoc_Array_Access :=
         Generic_Instantiation_P_Instantiation_Bindings (E.Node, E_Info);
   begin
         return Result;
   end;
end;
function Generic_Instantiation_P_Parent_Instantiation_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Generic_Instantiation
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Generic_Instantiation_P_Parent_Instantiation_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Generic_Package_Instantiation
   --
      procedure Initialize_Fields_For_Generic_Package_Instantiation
        (Self : Bare_Generic_Package_Instantiation
         ; Generic_Package_Instantiation_F_Name : Bare_Defining_Name
         ; Generic_Package_Instantiation_F_Generic_Pkg_Name : Bare_Name
         ; Generic_Package_Instantiation_F_Params : Bare_Assoc_List
         ; Generic_Package_Instantiation_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Generic_Instantiation
              (Self);
            Self.Generic_Package_Instantiation_F_Name := Generic_Package_Instantiation_F_Name;
            Self.Generic_Package_Instantiation_F_Generic_Pkg_Name := Generic_Package_Instantiation_F_Generic_Pkg_Name;
            Self.Generic_Package_Instantiation_F_Params := Generic_Package_Instantiation_F_Params;
            Self.Generic_Package_Instantiation_F_Aspects := Generic_Package_Instantiation_F_Aspects;
      end Initialize_Fields_For_Generic_Package_Instantiation;
   function Generic_Package_Instantiation_F_Name
     (Node : Bare_Generic_Package_Instantiation) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Package_Instantiation_F_Name;
   end;
   function Generic_Package_Instantiation_F_Generic_Pkg_Name
     (Node : Bare_Generic_Package_Instantiation) return Bare_Name
   is
   begin
         return Node.Generic_Package_Instantiation_F_Generic_Pkg_Name;
   end;
   function Generic_Package_Instantiation_F_Params
     (Node : Bare_Generic_Package_Instantiation) return Bare_Assoc_List
   is
   begin
         return Node.Generic_Package_Instantiation_F_Params;
   end;
--# property-start GenericPackageInstantiation.generic_entity_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11538
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5605 '<FieldAccess for generic_pkg_name at ast.py:11538>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11538
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Package_Instantiation_F_Generic_Pkg_Name, Info => Ent.Info);
--# expr-done 5605
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start GenericPackageInstantiation.generic_inst_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11539
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5606 '<FieldAccess for params at ast.py:11539>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11539
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Package_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 5606
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericPackageInstantiation.designated_package /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11541
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Designated_Package
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Package_Decl
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Package_Decl;
            procedure Finalizer_Scope_2649 with Inline_Always;
      Var_Expr : Internal_Entity_Generic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Cast_Result : Bare_Generic_Package_Decl;
Fld_2 : Bare_Generic_Package_Internal;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Info;
Scope_Result : Internal_Entity_Info;
Result_Var : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Rebinding : Env_Rebindings;
Fld_10 : Bare_Generic_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Rebinding_1 : Env_Rebindings;
Fld_13 : Internal_Entity_Info;
Fld_14 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result_1 : Bare_Base_Package_Decl;
New_Struct_1 : Internal_Entity_Base_Package_Decl;
Scope_Result_1 : Internal_Entity_Base_Package_Decl;
Result_Var_1 : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Base_Package_Decl;
            procedure Finalizer_Scope_2649 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2649;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5627 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5607 '<FieldAccess for nonbound_generic_decl at ast.py:11543>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11543
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 5607
Var_Expr := Fld;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 5626 '<New BasePackageDecl.entity at ???>' New_Struct_1 None
--# expr-start 5622 '<New EntityInfo at ast.py:11546>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11546
--# expr-start 5609 '<FieldAccess for from_rebound at ast.py:11563>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11563
--# expr-start 5608 '<FieldAccess for info at ast.py:11563>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11563
Fld_13 := Var_Expr.Info;
--# expr-done 5608
Fld_14 := Fld_13.From_Rebound;
--# expr-done 5609
--# expr-start 5611 '<FieldAccess for md at ast.py:11547>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11547
--# expr-start 5610 '<FieldAccess for info at ast.py:11547>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11547
Fld_3 := Var_Expr.Info;
--# expr-done 5610
Fld_4 := Fld_3.Md;
--# expr-done 5611
--# expr-start 5621 '<append_rebinding at ast.py:11561>' Rebinding_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11561
--# expr-start 5617 '<concat_rebindings at ast.py:11553>' Rebinding /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11553
--# expr-start 5613 '<FieldAccess for rebindings at ast.py:11550>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11550
--# expr-start 5612 '<FieldAccess for info at ast.py:11550>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11550
Fld_5 := Ent.Info;
--# expr-done 5612
Fld_6 := Fld_5.Rebindings;
--# expr-done 5613
--# expr-start 5616 '<FieldAccess for rebindings at ast.py:11553>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11553
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 5615 '<FieldAccess for info at ast.py:11553>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11553
--# expr-start 5614 '<FieldAccess for decl at ast.py:11553>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11553
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 5614
Fld_8 := Fld_7.Info;
--# expr-done 5615
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Info;
end if;
Fld_9 := Result_Var.Rebindings;
--# expr-done 5616
Rebinding := AST_Envs.Combine (Fld_6, Fld_9); 
--# expr-done 5617
--# expr-start 5619 '<FieldAccess for children_env at ast.py:11561>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11561
--# expr-start 5618 '<FieldAccess for node at ast.py:11561>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11561
Fld_10 := Var_Expr.Node;
--# expr-done 5618
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Fld_10);
--# expr-done 5619
--# expr-start 5620 '<FieldAccess for instantiation_env at ast.py:11562>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11562
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_12 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Self);
--# end
--# expr-done 5620
Rebinding_1 := AST_Envs.Append_Rebinding (Rebinding, Fld_11, Fld_12); 
--# expr-done 5621
New_Struct := (From_Rebound => Fld_14, Md => Fld_4, Rebindings => Rebinding_1); 
--# expr-done 5622
--# expr-start 5625 '<FieldAccess for package_decl at ast.py:11545>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11545
--# expr-start 5624 '<Cast to GenericPackageDecl at ast.py:11545>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11545
--# expr-start 5623 '<FieldAccess for node at ast.py:11545>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11545
Fld_1 := Var_Expr.Node;
--# expr-done 5623
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5624
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Generic_Package_Decl_F_Package_Decl;
--# expr-done 5625
      Cast_Result_1 := Fld_2;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_1); 
--# expr-done 5626
Scope_Result_1 := New_Struct_1;
   --# end
      Finalizer_Scope_2649;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Package_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 5627
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2649;
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Designated_Package;
--# end
--# property-start GenericPackageInstantiation.designated_generic_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11568
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Base_Package_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5630 '<Cast to BasicDecl at ast.py:11569>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11569
--# expr-start 5629 '<parent at ast.py:11569>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11569
--# expr-start 5628 '<FieldAccess for designated_package at ast.py:11569>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11569
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5628
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 5629
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 5630
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericPackageInstantiation.defining_env_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11572
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Env_Impl
  (Node : Bare_Generic_Package_Instantiation
      ; Inst_From_Formal : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind inst_from_formal Inst_From_Formal
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_894 with Inline_Always;
      Block_Var : Internal_Entity_Base_Package_Decl;
Fld : Internal_Entity_Base_Package_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
If_Result : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_894 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_894;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5645 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5631 '<FieldAccess for designated_package at ast.py:11582>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11582
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5631
Block_Var := Fld; 
--# bind dp Block_Var
--# expr-start 5644 '<If at ast.py:11583>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11583
--# expr-start 5632 '<IsNull at ast.py:11603>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11603
Is_Null := Block_Var.Node = null; 
--# expr-done 5632
if Is_Null then
   If_Result_2 := Empty_Env;
else
   --# expr-start 5643 '<env_group at ast.py:11625>' Group_Env_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11625
--# expr-start 5642 '<ArrayLiteral at ast.py:11605>' Array_Lit_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11605
--# expr-start 5640 '<If at ast.py:11606>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11606
--# expr-start 5633 '<FieldAccess for is_formal at ast.py:11607>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11607
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_formal
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Self);
--# end
--# expr-done 5633
if Fld_5 then
   If_Result := True;
else
   If_Result := Inst_From_Formal;
end if;
if If_Result then
   --# expr-start 5638 '<env_group at ast.py:11611>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11611
--# expr-start 5637 '<ArrayLiteral at ast.py:11608>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11608
--# expr-start 5634 '<FieldAccess for children_env at ast.py:11609>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11609
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 5634
--# expr-start 5636 '<FieldAccess for children_env at ast.py:11610>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11610
--# expr-start 5635 '<parent at ast.py:11610>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11610
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 5635
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 5636
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_1, 2 => Fld_3)); 
--# expr-done 5637
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 5638
   If_Result_1 := Group_Env;
else
   --# expr-start 5639 '<FieldAccess for children_env at ast.py:11612>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11612
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 5639
   If_Result_1 := Fld_4;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5640
--# expr-start 5641 '<FieldAccess for children_env at ast.py:11624>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11624
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5641
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => If_Result_1, 2 => Fld_6)); 
--# expr-done 5642
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 5643
   If_Result_2 := Group_Env_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 5644
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 5645
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_894;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_894;
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Defining_Env_Impl;
--# end
--# property-start GenericPackageInstantiation.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11628
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Env
  (Node : Bare_Generic_Package_Instantiation
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_895 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_895 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_895;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5647 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5646 '<FieldAccess for defining_env_impl at ast.py:11630>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11630
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericPackageInstantiation.defining_env_impl
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Defining_Env_Impl (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5646
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 5647
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_895;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_895;
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Defining_Env;
--# end
--# property-start GenericPackageInstantiation.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11632
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Names
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_896 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_896 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_896;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5648 '<FieldAccess for name at ast.py:11632>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11632
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Package_Instantiation_F_Name, Info => Ent.Info);
--# expr-done 5648
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_896;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_896;
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Defining_Names;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Instantiation
   --
      procedure Initialize_Fields_For_Generic_Subp_Instantiation
        (Self : Bare_Generic_Subp_Instantiation
         ; Generic_Subp_Instantiation_F_Overriding : Bare_Overriding_Node
         ; Generic_Subp_Instantiation_F_Kind : Bare_Subp_Kind
         ; Generic_Subp_Instantiation_F_Subp_Name : Bare_Defining_Name
         ; Generic_Subp_Instantiation_F_Generic_Subp_Name : Bare_Name
         ; Generic_Subp_Instantiation_F_Params : Bare_Assoc_List
         ; Generic_Subp_Instantiation_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Generic_Instantiation
              (Self);
            Self.Generic_Subp_Instantiation_F_Overriding := Generic_Subp_Instantiation_F_Overriding;
            Self.Generic_Subp_Instantiation_F_Kind := Generic_Subp_Instantiation_F_Kind;
            Self.Generic_Subp_Instantiation_F_Subp_Name := Generic_Subp_Instantiation_F_Subp_Name;
            Self.Generic_Subp_Instantiation_F_Generic_Subp_Name := Generic_Subp_Instantiation_F_Generic_Subp_Name;
            Self.Generic_Subp_Instantiation_F_Params := Generic_Subp_Instantiation_F_Params;
            Self.Generic_Subp_Instantiation_F_Aspects := Generic_Subp_Instantiation_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Instantiation;
   function Generic_Subp_Instantiation_F_Overriding
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Overriding_Node
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Overriding;
   end;
   function Generic_Subp_Instantiation_F_Kind
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Subp_Kind
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Kind;
   end;
   function Generic_Subp_Instantiation_F_Subp_Name
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Subp_Name;
   end;
   function Generic_Subp_Instantiation_F_Generic_Subp_Name
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Name
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Generic_Subp_Name;
   end;
   function Generic_Subp_Instantiation_F_Params
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Assoc_List
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Params;
   end;
--# property-start GenericSubpInstantiation.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11491
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Defining_Names
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_883 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_883 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_883;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5649 '<FieldAccess for subp_name at ast.py:11491>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11491
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Subp_Instantiation_F_Subp_Name, Info => Ent.Info);
--# expr-done 5649
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_883;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_883;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Defining_Names;
--# end
--# property-start GenericSubpInstantiation.generic_entity_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11493
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5650 '<FieldAccess for generic_subp_name at ast.py:11493>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11493
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Subp_Instantiation_F_Generic_Subp_Name, Info => Ent.Info);
--# expr-done 5650
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start GenericSubpInstantiation.generic_inst_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11494
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5651 '<FieldAccess for params at ast.py:11494>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11494
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Subp_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 5651
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericSubpInstantiation.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11496
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Expr_Type
  (Node : Bare_Generic_Subp_Instantiation
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5654 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5652 '<FieldAccess for subp_spec_or_null at ast.py:11498>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11498
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5652
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 5653 '<FieldAccess for return_type at ast.py:11498>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11498
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 5653
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 5654
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Expr_Type;
--# end
--# property-start GenericSubpInstantiation.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11500
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Defining_Env
  (Node : Bare_Generic_Subp_Instantiation
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_887 with Inline_Always;
            procedure Finalizer_Scope_2652 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_887 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_887;
            procedure Finalizer_Scope_2652 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2652;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5655 '<FieldAccess for subp_spec_or_null at ast.py:11500>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11500
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5655
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 5656 '<FieldAccess for defining_env at ast.py:11500>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11500
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 5656
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2652;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_887;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_887;
                     Finalizer_Scope_2652;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Defining_Env;
--# end
--# property-start GenericSubpInstantiation.designated_subp /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11502
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Designated_Subp
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Subp_Decl
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Subp_Decl;
            procedure Finalizer_Scope_2212 with Inline_Always;
      Var_Expr : Internal_Entity_Generic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Cast_Result : Bare_Generic_Subp_Decl;
Fld_2 : Bare_Generic_Subp_Internal;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Info;
Scope_Result : Internal_Entity_Info;
Result_Var : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Rebinding : Env_Rebindings;
Fld_10 : Bare_Generic_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Rebinding_1 : Env_Rebindings;
Fld_13 : Internal_Entity_Info;
Fld_14 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result_1 : Bare_Basic_Subp_Decl;
New_Struct_1 : Internal_Entity_Basic_Subp_Decl;
Scope_Result_1 : Internal_Entity_Basic_Subp_Decl;
Result_Var_1 : Internal_Entity_Basic_Subp_Decl;
Let_Result : Internal_Entity_Basic_Subp_Decl;
            procedure Finalizer_Scope_2212 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2212;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5677 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5657 '<FieldAccess for nonbound_generic_decl at ast.py:11507>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11507
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 5657
Var_Expr := Fld;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 5676 '<New BasicSubpDecl.entity at ???>' New_Struct_1 None
--# expr-start 5672 '<New EntityInfo at ast.py:11510>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11510
--# expr-start 5659 '<FieldAccess for from_rebound at ast.py:11518>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11518
--# expr-start 5658 '<FieldAccess for info at ast.py:11518>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11518
Fld_13 := Var_Expr.Info;
--# expr-done 5658
Fld_14 := Fld_13.From_Rebound;
--# expr-done 5659
--# expr-start 5661 '<FieldAccess for md at ast.py:11511>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11511
--# expr-start 5660 '<FieldAccess for info at ast.py:11511>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11511
Fld_3 := Ent.Info;
--# expr-done 5660
Fld_4 := Fld_3.Md;
--# expr-done 5661
--# expr-start 5671 '<append_rebinding at ast.py:11515>' Rebinding_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11515
--# expr-start 5667 '<concat_rebindings at ast.py:11514>' Rebinding /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11514
--# expr-start 5663 '<FieldAccess for rebindings at ast.py:11512>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11512
--# expr-start 5662 '<FieldAccess for info at ast.py:11512>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11512
Fld_5 := Ent.Info;
--# expr-done 5662
Fld_6 := Fld_5.Rebindings;
--# expr-done 5663
--# expr-start 5666 '<FieldAccess for rebindings at ast.py:11514>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11514
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 5665 '<FieldAccess for info at ast.py:11514>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11514
--# expr-start 5664 '<FieldAccess for decl at ast.py:11514>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11514
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 5664
Fld_8 := Fld_7.Info;
--# expr-done 5665
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Info;
end if;
Fld_9 := Result_Var.Rebindings;
--# expr-done 5666
Rebinding := AST_Envs.Combine (Fld_6, Fld_9); 
--# expr-done 5667
--# expr-start 5669 '<FieldAccess for children_env at ast.py:11516>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11516
--# expr-start 5668 '<FieldAccess for node at ast.py:11516>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11516
Fld_10 := Var_Expr.Node;
--# expr-done 5668
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Fld_10);
--# expr-done 5669
--# expr-start 5670 '<FieldAccess for instantiation_env at ast.py:11516>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11516
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_12 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Self);
--# end
--# expr-done 5670
Rebinding_1 := AST_Envs.Append_Rebinding (Rebinding, Fld_11, Fld_12); 
--# expr-done 5671
New_Struct := (From_Rebound => Fld_14, Md => Fld_4, Rebindings => Rebinding_1); 
--# expr-done 5672
--# expr-start 5675 '<FieldAccess for subp_decl at ast.py:11509>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11509
--# expr-start 5674 '<Cast to GenericSubpDecl at ast.py:11509>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11509
--# expr-start 5673 '<FieldAccess for node at ast.py:11509>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11509
Fld_1 := Var_Expr.Node;
--# expr-done 5673
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5674
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Generic_Subp_Decl_F_Subp_Decl;
--# expr-done 5675
      Cast_Result_1 := Fld_2;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_1); 
--# expr-done 5676
Scope_Result_1 := New_Struct_1;
   --# end
      Finalizer_Scope_2212;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Subp_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 5677
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2212;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Designated_Subp;
--# end
--# property-start GenericSubpInstantiation.designated_generic_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11523
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Subp_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5680 '<Cast to BasicDecl at ast.py:11524>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11524
--# expr-start 5679 '<parent at ast.py:11524>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11524
--# expr-start 5678 '<FieldAccess for designated_subp at ast.py:11524>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11524
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericSubpInstantiation.designated_subp
Fld := Libadalang.Implementation.Generic_Subp_Instantiation_P_Designated_Subp (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5678
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 5679
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 5680
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Designated_Generic_Decl;
--# end
   --
   --  Primitives for Bare_Generic_Renaming_Decl
   --
--# property-start '[dispatcher]GenericRenamingDecl.renaming_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Renaming_Decl  := Bare_Generic_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Renaming_Decl (Self.Kind) is
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.renaming_name
                  Property_Result := Generic_Package_Renaming_Decl_P_Renaming_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericSubpRenamingDecl.renaming_name
                  Property_Result := Generic_Subp_Renaming_Decl_P_Renaming_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericRenamingDecl.resolve /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11740
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Resolve
  (Node : Bare_Generic_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Renaming_Decl  := Bare_Generic_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Renaming_Decl :=
       Internal_Entity_Generic_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_904 with Inline_Always;
      Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Generic_Decl;
Match_1 : Internal_Entity_Generic_Renaming_Decl;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Name;
Cast_Expr : Bare_Generic_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_2 : Internal_Entity_Generic_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Generic_Decl;
Let_Result : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Renaming_Decl;
Let_Result_1 : Internal_Entity_Generic_Decl;
Scope_Result_1 : Internal_Entity_Generic_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Generic_Decl;
Scope_Result_2 : Internal_Entity_Generic_Decl;
Match_Result : Internal_Entity_Generic_Decl;
Scope_Result_3 : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_904 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_904;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5683 '<at at ast.py:11746>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11746
--# expr-start 5682 '<FieldAccess for all_env_elements_internal at ast.py:11744>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11744
--# expr-start 5681 '<FieldAccess for renaming_name at ast.py:11744>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11744
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5681
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.all_env_elements_internal
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Fld.Node, Seq => True, Seq_From => Cast_Result, Categories => (Inherited_Primitives => False, Nocat => True), E_Info => Fld.Info);
--# end
--# expr-done 5682
Get_Result := Get (Self, Fld_1, 0, True); 
--# expr-done 5683
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 5685 '<Match at ast.py:11746>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11746
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Generic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind gd Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind grd Match_1
--# expr-start 5684 '<FieldAccess for resolve at ast.py:11748>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11748
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_2 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 5684
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Generic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5685
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      Finalizer_Scope_904;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_904;
            raise;
      end;
   return Property_Result;
end Generic_Renaming_Decl_P_Resolve;
--# end
--# property-start GenericRenamingDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11755
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Generic_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Renaming_Decl  := Bare_Generic_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start GenericRenamingDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11756
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Generic_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Renaming_Decl  := Bare_Generic_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Renaming_Decl :=
       Internal_Entity_Generic_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_906 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_906 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_906;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5687 '<FieldAccess for xref_no_overloading at ast.py:11756>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11756
--# expr-start 5686 '<FieldAccess for renaming_name at ast.py:11756>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11756
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5686
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5687
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_906;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_906;
            raise;
      end;
   return Property_Result;
end Generic_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Generic_Package_Renaming_Decl
   --
      procedure Generic_Package_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_80 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_81 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:11774");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_82 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:11776");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_83 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_84 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_86 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_85 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_88 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_87 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Renaming_Decl
        (Self : Bare_Generic_Package_Renaming_Decl
         ; Generic_Package_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Generic_Package_Renaming_Decl_F_Renames : Bare_Name
         ; Generic_Package_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Package_Renaming_Decl_F_Name := Generic_Package_Renaming_Decl_F_Name;
            Self.Generic_Package_Renaming_Decl_F_Renames := Generic_Package_Renaming_Decl_F_Renames;
            Self.Generic_Package_Renaming_Decl_F_Aspects := Generic_Package_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Generic_Package_Renaming_Decl;
   function Generic_Package_Renaming_Decl_F_Name
     (Node : Bare_Generic_Package_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Package_Renaming_Decl_F_Name;
   end;
   function Generic_Package_Renaming_Decl_F_Renames
     (Node : Bare_Generic_Package_Renaming_Decl) return Bare_Name
   is
   begin
         return Node.Generic_Package_Renaming_Decl_F_Renames;
   end;
--# property-start GenericPackageRenamingDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11767
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Defining_Names
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
       Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_907 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_907 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_907;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5688 '<FieldAccess for name at ast.py:11767>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11767
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 5688
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_907;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_907;
            raise;
      end;
   return Property_Result;
end Generic_Package_Renaming_Decl_P_Defining_Names;
--# end
--# property-start GenericPackageRenamingDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11768
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Defining_Env
  (Node : Bare_Generic_Package_Renaming_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
       Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_908 with Inline_Always;
      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_908 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_908;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5690 '<FieldAccess for defining_env at ast.py:11768>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11768
--# expr-start 5689 '<FieldAccess for resolve at ast.py:11768>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11768
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5689
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5690
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_908;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_908;
            raise;
      end;
   return Property_Result;
end Generic_Package_Renaming_Decl_P_Defining_Env;
--# end
--# property-start GenericPackageRenamingDecl.renaming_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11769
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
       Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5691 '<FieldAccess for renames at ast.py:11769>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11769
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5691
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericPackageRenamingDecl._env_do_80 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11772
pragma Warnings (Off, "is not referenced");
 function Env_Do_80
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5692 '<FieldAccess for env_hook at ast.py:11772>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11772
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5692
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_80;
--# end
--# property-start GenericPackageRenamingDecl._initial_env_81 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11774
pragma Warnings (Off, "is not referenced");
 function Initial_Env_81
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1608 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1608 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1608;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5693 '<FieldAccess for child_decl_initial_env at ast.py:11774>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11774
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self);
--# end
--# expr-done 5693
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1608;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1608;
            raise;
      end;
   return Property_Result;
end Initial_Env_81;
--# end
--# property-start GenericPackageRenamingDecl._env_mappings_82 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11776
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_82
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
       Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1609 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Generic_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1609 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1609;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5696 '<New EnvAssoc at ast.py:11776>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11776
--# expr-start 5694 '<New DesignatedEnv at ast.py:11776>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11776
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5694
--# expr-start 5695 '<FieldAccess for name_symbol at ast.py:11777>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11777
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5695
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5696
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1609;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1609;
            raise;
      end;
   return Property_Result;
end Env_Mappings_82;
--# end
--# property-start GenericPackageRenamingDecl._env_trans_parent_83 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11771
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_83
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_83;
--# end
--# property-start GenericPackageRenamingDecl._env_do_84 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11783
pragma Warnings (Off, "is not referenced");
 function Env_Do_84
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1611 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1611 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1611;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5697 '<FieldAccess for populate_dependent_units at ast.py:11783>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11783
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5697
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1611;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1611;
            raise;
      end;
   return Property_Result;
end Env_Do_84;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_85 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11786
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_85
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1612 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1612 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1612;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5698 '<FieldAccess for top_level_use_package_clauses at ast.py:11786>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11786
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5698
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1612;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1612;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_85;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_86 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11788
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_86
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5700 '<IsA LibraryItem, Subunit at ast.py:11788>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11788
--# expr-start 5699 '<parent at ast.py:11788>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11788
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5699
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5700
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_86;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_87 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11791
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_87
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1614 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1614 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1614;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5701 '<FieldAccess for top_level_use_type_clauses at ast.py:11791>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11791
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5701
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1614;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1614;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_87;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_88 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11793
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_88
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5703 '<IsA LibraryItem, Subunit at ast.py:11793>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11793
--# expr-start 5702 '<parent at ast.py:11793>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11793
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5702
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5703
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_88;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Renaming_Decl
   --
      procedure Generic_Subp_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_89 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_90 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:11813");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_91 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:11815");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_92 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_93 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_95 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_94 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_97 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_96 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Renaming_Decl
        (Self : Bare_Generic_Subp_Renaming_Decl
         ; Generic_Subp_Renaming_Decl_F_Kind : Bare_Subp_Kind
         ; Generic_Subp_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Generic_Subp_Renaming_Decl_F_Renames : Bare_Name
         ; Generic_Subp_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Subp_Renaming_Decl_F_Kind := Generic_Subp_Renaming_Decl_F_Kind;
            Self.Generic_Subp_Renaming_Decl_F_Name := Generic_Subp_Renaming_Decl_F_Name;
            Self.Generic_Subp_Renaming_Decl_F_Renames := Generic_Subp_Renaming_Decl_F_Renames;
            Self.Generic_Subp_Renaming_Decl_F_Aspects := Generic_Subp_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Renaming_Decl;
   function Generic_Subp_Renaming_Decl_F_Kind
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Subp_Kind
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Kind;
   end;
   function Generic_Subp_Renaming_Decl_F_Name
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Name;
   end;
   function Generic_Subp_Renaming_Decl_F_Renames
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Name
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Renames;
   end;
--# property-start GenericSubpRenamingDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11841
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Renaming_Decl_P_Defining_Names
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
       Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_910 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_910 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_910;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5704 '<FieldAccess for name at ast.py:11841>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11841
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Subp_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 5704
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_910;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_910;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Renaming_Decl_P_Defining_Names;
--# end
--# property-start GenericSubpRenamingDecl.renaming_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11842
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
       Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5705 '<FieldAccess for renames at ast.py:11842>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11842
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5705
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericSubpRenamingDecl._env_do_89 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11811
pragma Warnings (Off, "is not referenced");
 function Env_Do_89
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5706 '<FieldAccess for env_hook at ast.py:11811>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11811
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5706
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_89;
--# end
--# property-start GenericSubpRenamingDecl._initial_env_90 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11813
pragma Warnings (Off, "is not referenced");
 function Initial_Env_90
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1617 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1617 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1617;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5707 '<FieldAccess for child_decl_initial_env at ast.py:11813>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11813
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self);
--# end
--# expr-done 5707
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1617;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1617;
            raise;
      end;
   return Property_Result;
end Initial_Env_90;
--# end
--# property-start GenericSubpRenamingDecl._env_mappings_91 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11815
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_91
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
       Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1618 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Generic_Subp_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1618 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1618;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5710 '<New EnvAssoc at ast.py:11815>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11815
--# expr-start 5708 '<New DesignatedEnv at ast.py:11815>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11815
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5708
--# expr-start 5709 '<FieldAccess for name_symbol at ast.py:11816>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11816
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5709
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5710
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1618;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1618;
            raise;
      end;
   return Property_Result;
end Env_Mappings_91;
--# end
--# property-start GenericSubpRenamingDecl._env_trans_parent_92 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11810
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_92
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_92;
--# end
--# property-start GenericSubpRenamingDecl._env_do_93 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11822
pragma Warnings (Off, "is not referenced");
 function Env_Do_93
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1620 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1620 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1620;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5711 '<FieldAccess for populate_dependent_units at ast.py:11822>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11822
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5711
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1620;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1620;
            raise;
      end;
   return Property_Result;
end Env_Do_93;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_94 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11825
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_94
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1621 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1621 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1621;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5712 '<FieldAccess for top_level_use_package_clauses at ast.py:11825>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11825
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5712
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1621;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1621;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_94;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_95 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11827
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_95
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5714 '<IsA LibraryItem, Subunit at ast.py:11827>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11827
--# expr-start 5713 '<parent at ast.py:11827>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11827
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5713
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5714
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_95;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_96 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11830
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_96
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1623 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1623 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1623;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5715 '<FieldAccess for top_level_use_type_clauses at ast.py:11830>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11830
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5715
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1623;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1623;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_96;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_97 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11832
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_97
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5717 '<IsA LibraryItem, Subunit at ast.py:11832>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11832
--# expr-start 5716 '<parent at ast.py:11832>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11832
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5716
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5717
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_97;
--# end
   --
   --  Primitives for Bare_Label_Decl
   --
      procedure Label_Decl_Pre_Env_Actions
        (Self            : Bare_Label_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_145 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:21300");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Label_Decl
        (Self : Bare_Label_Decl
         ; Label_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Label_Decl_F_Name := Label_Decl_F_Name;
      end Initialize_Fields_For_Label_Decl;
   function Label_Decl_F_Name
     (Node : Bare_Label_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Label_Decl_F_Name;
   end;
--# property-start LabelDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21298
pragma Warnings (Off, "is not referenced");
 function Label_Decl_P_Defining_Names
  (Node : Bare_Label_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Label_Decl  := Bare_Label_Decl (Node);
     Ent : Internal_Entity_Label_Decl :=
       Internal_Entity_Label_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1455 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1455 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1455;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5718 '<FieldAccess for name at ast.py:21298>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21298
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Label_Decl_F_Name, Info => Ent.Info);
--# expr-done 5718
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1455;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1455;
            raise;
      end;
   return Property_Result;
end Label_Decl_P_Defining_Names;
--# end
--# property-start LabelDecl._env_mappings_145 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21300
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_145
  (Node : Bare_Label_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Label_Decl  := Bare_Label_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1672 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Label_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1672 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1672;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5721 '<New EnvAssoc at ast.py:21300>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21300
--# expr-start 5719 '<New DesignatedEnv at ast.py:21300>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21300
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5719
--# expr-start 5720 '<FieldAccess for name_symbol at ast.py:21300>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21300
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 5720
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5721
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1672;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1672;
            raise;
      end;
   return Property_Result;
end Env_Mappings_145;
--# end
   --
   --  Primitives for Bare_Named_Stmt_Decl
   --
      procedure Initialize_Fields_For_Named_Stmt_Decl
        (Self : Bare_Named_Stmt_Decl
         ; Named_Stmt_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Named_Stmt_Decl_F_Name := Named_Stmt_Decl_F_Name;
      end Initialize_Fields_For_Named_Stmt_Decl;
   function Named_Stmt_Decl_F_Name
     (Node : Bare_Named_Stmt_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Named_Stmt_Decl_F_Name;
   end;
--# property-start NamedStmtDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21338
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_Decl_P_Defining_Names
  (Node : Bare_Named_Stmt_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Named_Stmt_Decl  := Bare_Named_Stmt_Decl (Node);
     Ent : Internal_Entity_Named_Stmt_Decl :=
       Internal_Entity_Named_Stmt_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1458 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1458 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1458;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5722 '<FieldAccess for name at ast.py:21338>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21338
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Named_Stmt_Decl_F_Name, Info => Ent.Info);
--# expr-done 5722
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1458;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1458;
            raise;
      end;
   return Property_Result;
end Named_Stmt_Decl_P_Defining_Names;
--# end
--# property-start NamedStmtDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21339
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_Decl_P_Defining_Env
  (Node : Bare_Named_Stmt_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Named_Stmt_Decl  := Bare_Named_Stmt_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1459 with Inline_Always;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Named_Stmt;
Fld_1 : Bare_Composite_Stmt;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1459 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1459;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5726 '<FieldAccess for children_env at ast.py:21339>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21339
--# expr-start 5725 '<FieldAccess for stmt at ast.py:21339>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21339
--# expr-start 5724 '<Cast to NamedStmt at ast.py:21339>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21339
--# expr-start 5723 '<parent at ast.py:21339>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21339
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5723
   if Fld = null
      or else Fld.Kind in Ada_Named_Stmt_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5724
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Cast_Result.Named_Stmt_F_Stmt;
--# expr-done 5725
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 5726
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1459;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1459;
            raise;
      end;
   return Property_Result;
end Named_Stmt_Decl_P_Defining_Env;
--# end
   --
   --  Primitives for Bare_Number_Decl
   --
      procedure Number_Decl_Pre_Env_Actions
        (Self            : Bare_Number_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_43 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:10701");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Number_Decl
        (Self : Bare_Number_Decl
         ; Number_Decl_F_Ids : Bare_Defining_Name_List
         ; Number_Decl_F_Expr : Bare_Expr
        ) is
      begin
            Self.Number_Decl_F_Ids := Number_Decl_F_Ids;
            Self.Number_Decl_F_Expr := Number_Decl_F_Expr;
      end Initialize_Fields_For_Number_Decl;
   function Number_Decl_F_Ids
     (Node : Bare_Number_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Number_Decl_F_Ids;
   end;
   function Number_Decl_F_Expr
     (Node : Bare_Number_Decl) return Bare_Expr
   is
   begin
         return Node.Number_Decl_F_Expr;
   end;
--# property-start NumberDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10699
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Defining_Names
  (Node : Bare_Number_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
     Ent : Internal_Entity_Number_Decl :=
       Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_826 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_194 : Internal_Entity_Defining_Name;
Bare_Item_194 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_826 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_826;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5728 '<Map at ast.py:10699>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10699
   --# expr-start 5727 '<FieldAccess for ids at ast.py:10699>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10699
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Number_Decl_F_Ids, Info => Ent.Info);
--# expr-done 5727
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_194 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_194 := Untyped_Item_194; 
Is_Null := Bare_Item_194 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_194); 
   If_Result := As_Entity;
end if;
                  Item_194 := If_Result; 
   --# scope-start
               --# bind id Item_194
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_194;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5728
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_826;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_826;
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Defining_Names;
--# end
--# property-start NumberDecl.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10703
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Expr_Type
  (Node : Bare_Number_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
     Ent : Internal_Entity_Number_Decl :=
       Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5736 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5735 '<Cast to BaseTypeDecl.entity at ast.py:10707>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10707
--# expr-start 5734 '<If at ast.py:10705>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10705
--# expr-start 5731 '<FieldAccess for is_int_type at ast.py:10705>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10705
--# expr-start 5730 '<FieldAccess for expression_type at ast.py:10705>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10705
--# expr-start 5729 '<FieldAccess for expr at ast.py:10705>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10705
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 5729
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 5730
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 5731
if Fld_4 then
   --# expr-start 5732 '<FieldAccess for universal_int_type at ast.py:10706>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10706
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 5732
   If_Result := Fld;
else
   --# expr-start 5733 '<FieldAccess for universal_real_type at ast.py:10707>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10707
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 5733
   If_Result := Fld_1;
end if;
--# expr-done 5734
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5735
Let_Result := Cast_Result; 
--# end
--# expr-done 5736
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Expr_Type;
--# end
--# property-start NumberDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10709
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Xref_Entry_Point
  (Node : Bare_Number_Decl
  )
   return Boolean
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Xref_Entry_Point;
--# end
--# property-start NumberDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10711
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Is_Static_Decl
  (Node : Bare_Number_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Is_Static_Decl;
--# end
--# property-start NumberDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10713
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Xref_Equation
  (Node : Bare_Number_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
     Ent : Internal_Entity_Number_Decl :=
       Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_830 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_830 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_830;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5751 '<And at ast.py:10714>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10714
--# expr-start 5748 '<And at ast.py:10714>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10714
--# expr-start 5745 '<Or at ast.py:10714>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10714
--# expr-start 5740 '<Bind at ast.py:10715>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10715
--# expr-start 5738 '<FieldAccess for expected_type_var at ast.py:10715>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10715
--# expr-start 5737 '<FieldAccess for expr at ast.py:10715>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10715
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Number_Decl_F_Expr;
--# expr-done 5737
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 5738
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 5739 '<FieldAccess for universal_int_type at ast.py:10715>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10715
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 5739
Bind_Result := Solver.Create_Assign (Fld_1, Fld_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10715") else null)); 
--# expr-done 5740
--# expr-start 5744 '<Bind at ast.py:10716>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10716
--# expr-start 5742 '<FieldAccess for expected_type_var at ast.py:10716>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10716
--# expr-start 5741 '<FieldAccess for expr at ast.py:10716>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10716
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Number_Decl_F_Expr;
--# expr-done 5741
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_3);
--# expr-done 5742
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 5743 '<FieldAccess for universal_real_type at ast.py:10716>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10716
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 5743
Bind_Result_1 := Solver.Create_Assign (Fld_4, Fld_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10716") else null)); 
--# expr-done 5744
Or_Pred := Create_Or (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10714") else null)); 
--# expr-done 5745
--# expr-start 5747 '<FieldAccess for sub_equation at ast.py:10718>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10718
--# expr-start 5746 '<FieldAccess for expr at ast.py:10718>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10718
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 5746
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 5747
And_Pred := Create_And (Or_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10714") else null)); 
--# expr-done 5748
--# expr-start 5750 '<FieldAccess for matches_expected_type at ast.py:10719>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10719
--# expr-start 5749 '<FieldAccess for expr at ast.py:10719>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10719
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 5749
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 5750
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10714") else null)); 
--# expr-done 5751
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_830;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_830;
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Xref_Equation;
--# end
--# property-start NumberDecl.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10722
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Is_Constant_Object
  (Node : Bare_Number_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Is_Constant_Object;
--# end
--# property-start NumberDecl._env_mappings_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10701
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_43
  (Node : Bare_Number_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1570 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Number_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1570 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1570;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5753 '<FieldAccess for env_mappings at ast.py:10701>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10701
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5752 '<FieldAccess for ids at ast.py:10701>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10701
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Number_Decl_F_Ids;
--# expr-done 5752
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 5753
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1570;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1570;
            raise;
      end;
   return Property_Result;
end Env_Mappings_43;
--# end
   --
   --  Primitives for Bare_Object_Decl
   --
      procedure Object_Decl_Pre_Env_Actions
        (Self            : Bare_Object_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_44 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:10741");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Object_Decl
        (Self : Bare_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Object_Decl_F_Ids := Object_Decl_F_Ids;
            Self.Object_Decl_F_Has_Aliased := Object_Decl_F_Has_Aliased;
            Self.Object_Decl_F_Has_Constant := Object_Decl_F_Has_Constant;
            Self.Object_Decl_F_Mode := Object_Decl_F_Mode;
            Self.Object_Decl_F_Type_Expr := Object_Decl_F_Type_Expr;
            Self.Object_Decl_F_Default_Expr := Object_Decl_F_Default_Expr;
            Self.Object_Decl_F_Renaming_Clause := Object_Decl_F_Renaming_Clause;
            Self.Object_Decl_F_Aspects := Object_Decl_F_Aspects;
      end Initialize_Fields_For_Object_Decl;
   function Object_Decl_F_Ids
     (Node : Bare_Object_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Object_Decl_F_Ids;
   end;
   function Object_Decl_F_Has_Aliased
     (Node : Bare_Object_Decl) return Bare_Aliased_Node
   is
   begin
         return Node.Object_Decl_F_Has_Aliased;
   end;
   function Object_Decl_F_Has_Constant
     (Node : Bare_Object_Decl) return Bare_Constant_Node
   is
   begin
         return Node.Object_Decl_F_Has_Constant;
   end;
   function Object_Decl_F_Mode
     (Node : Bare_Object_Decl) return Bare_Mode
   is
   begin
         return Node.Object_Decl_F_Mode;
   end;
   function Object_Decl_F_Type_Expr
     (Node : Bare_Object_Decl) return Bare_Type_Expr
   is
   begin
         return Node.Object_Decl_F_Type_Expr;
   end;
   function Object_Decl_F_Default_Expr
     (Node : Bare_Object_Decl) return Bare_Expr
   is
   begin
         return Node.Object_Decl_F_Default_Expr;
   end;
   function Object_Decl_F_Renaming_Clause
     (Node : Bare_Object_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Object_Decl_F_Renaming_Clause;
   end;
--# property-start ObjectDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10743
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Defining_Names
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_832 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_195 : Internal_Entity_Defining_Name;
Bare_Item_195 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_832 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_832;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5755 '<Map at ast.py:10743>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10743
   --# expr-start 5754 '<FieldAccess for ids at ast.py:10743>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10743
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Object_Decl_F_Ids, Info => Ent.Info);
--# expr-done 5754
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_195 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_195 := Untyped_Item_195; 
Is_Null := Bare_Item_195 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_195); 
   If_Result := As_Entity;
end if;
                  Item_195 := If_Result; 
   --# scope-start
               --# bind id Item_195
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_195;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5755
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_832;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_832;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Defining_Names;
--# end
--# property-start ObjectDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10744
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Defining_Env
  (Node : Bare_Object_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_833 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_833 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_833;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5757 '<FieldAccess for defining_env at ast.py:10744>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10744
--# expr-start 5756 '<FieldAccess for type_expr at ast.py:10744>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10744
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 5756
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5757
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_833;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_833;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Defining_Env;
--# end
--# property-start ObjectDecl.type_expression /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10745
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Type_Expression
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5758 '<FieldAccess for type_expr at ast.py:10745>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10745
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 5758
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Type_Expression;
--# end
--# property-start ObjectDecl.is_constant_object /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10747
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Is_Constant_Object
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Constant_Node;
Var_Expr_1 : Internal_Entity_Mode;
Var_Expr_2 : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Constant_Node;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_2 : Internal_Entity_Mode;
Fld_3 : Bare_Ada_Node;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Fld_4 : Internal_Entity_Renaming_Clause;
Fld_5 : Internal_Entity_Name;
Fld_6 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
Fld_7 : Internal_Entity_Type_Expr;
Fld_8 : Internal_Entity_Base_Type_Decl;
Is_A_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5771 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5759 '<FieldAccess for has_constant at ast.py:10750>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10750
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constant_Node (Node => Ent.Node.Object_Decl_F_Has_Constant, Info => Ent.Info);
--# expr-done 5759
Var_Expr := Fld;
if Var_Expr /= No_Entity_Constant_Node then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 5760 '<FieldAccess for as_bool at ast.py:10750>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10750
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Var_Expr.Node);
--# end
--# expr-done 5760
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result_1 := True;
else
--# expr-start 5761 '<FieldAccess for mode at ast.py:10753>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10753
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Mode (Node => Ent.Node.Object_Decl_F_Mode, Info => Ent.Info);
--# expr-done 5761
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Mode then
   --# scope-start
   --# bind m Var_Expr_1
--# expr-start 5763 '<IsA GenericFormalObjDecl at ast.py:10755>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10755
--# expr-start 5762 '<parent at ast.py:10755>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10755
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 5762
Is_A := Fld_3 /= null 
and then Fld_3.Kind in Ada_Generic_Formal_Obj_Decl_Range; 
--# expr-done 5763
if Is_A then
   --# expr-start 5764 '<IsA Mode.In, Mode.Default at ast.py:10756>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10756
Is_A_1 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Mode_In_Range | Ada_Mode_Default_Range; 
--# expr-done 5764
   If_Result := Is_A_1;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 5765 '<FieldAccess for renaming_clause at ast.py:10760>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10760
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 5765
Var_Expr_2 := Fld_4;
if Var_Expr_2 /= No_Entity_Renaming_Clause then
   --# scope-start
--# expr-start 5767 '<FieldAccess for is_constant at ast.py:10760>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10760
--# expr-start 5766 '<FieldAccess for renamed_object at ast.py:10760>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10760
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Var_Expr_2.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_2.Info);
--# expr-done 5766
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 5767
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 5770 '<IsA ProtectedTypeDecl at ast.py:10763>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10763
--# expr-start 5769 '<FieldAccess for designated_type_decl at ast.py:10763>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10763
--# expr-start 5768 '<FieldAccess for type_expr at ast.py:10763>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10763
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 5768
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_8 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 5769
Is_A_2 := Fld_8.Node /= null 
and then Fld_8.Node.Kind in Ada_Protected_Type_Decl_Range; 
--# expr-done 5770
   If_Result_3 := Is_A_2;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 5771
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Is_Constant_Object;
--# end
--# property-start ObjectDecl.is_static_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10766
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Is_Static_Decl
  (Node : Bare_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Renaming_Clause;
Fld : Bare_Constant_Node;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Renaming_Clause;
Fld_5 : Internal_Entity_Name;
Fld_6 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5779 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5773 '<FieldAccess for as_bool at ast.py:10769>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10769
--# expr-start 5772 '<FieldAccess for has_constant at ast.py:10769>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10769
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Object_Decl_F_Has_Constant;
--# expr-done 5772
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 5773
if Fld_1 then
--# expr-start 5774 '<FieldAccess for default_expr at ast.py:10769>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10769
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 5774
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr
--# expr-start 5775 '<FieldAccess for is_static_expr at ast.py:10770>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10770
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_3 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 5775
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 5776 '<FieldAccess for renaming_clause at ast.py:10772>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10772
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 5776
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Renaming_Clause then
   --# scope-start
--# expr-start 5778 '<FieldAccess for is_static_expr at ast.py:10772>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10772
--# expr-start 5777 '<FieldAccess for renamed_object at ast.py:10772>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10772
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 5777
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_6 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 5778
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 5779
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Is_Static_Decl;
--# end
--# property-start ObjectDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10775
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Xref_Equation
  (Node : Bare_Object_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_837 with Inline_Always;
            procedure Finalizer_Scope_2664 with Inline_Always;
            procedure Finalizer_Scope_2665 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_7 : Internal_Entity_Renaming_Clause;
Fld_8 : Internal_Entity_Name;
Fld_9 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_10 : Internal_Entity_Name;
Fld_11 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_12 : Internal_Entity_Name;
Fld_13 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_837 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_837;
            procedure Finalizer_Scope_2664 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2664;
            procedure Finalizer_Scope_2665 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2665;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5802 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5780 '<FieldAccess for expr_type at ast.py:10777>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10777
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5780
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 5801 '<And at ast.py:10779>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10779
--# expr-start 5790 '<And at ast.py:10779>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10779
--# expr-start 5782 '<FieldAccess for sub_equation at ast.py:10779>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10779
--# expr-start 5781 '<FieldAccess for type_expr at ast.py:10779>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10779
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 5781
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 5782
--# expr-start 5783 '<FieldAccess for default_expr at ast.py:10780>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10780
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 5783
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 5789 '<And at ast.py:10782>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10782
--# expr-start 5787 '<And at ast.py:10782>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10782
--# expr-start 5785 '<Bind at ast.py:10782>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10782
--# expr-start 5784 '<FieldAccess for expected_type_var at ast.py:10782>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10782
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 5784
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10782") else null)); 
--# expr-done 5785
--# expr-start 5786 '<FieldAccess for sub_equation at ast.py:10783>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10783
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 5786
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10782") else null)); 
--# expr-done 5787
--# expr-start 5788 '<FieldAccess for matches_expected_assign_type at ast.py:10784>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10784
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 5788
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10782") else null)); 
--# expr-done 5789
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2664;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10785") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10779") else null)); 
--# expr-done 5790
--# expr-start 5791 '<FieldAccess for renaming_clause at ast.py:10787>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10787
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 5791
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind rc Var_Expr_1
--# expr-start 5800 '<And at ast.py:10789>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10789
--# expr-start 5797 '<And at ast.py:10789>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10789
--# expr-start 5794 '<Bind at ast.py:10789>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10789
--# expr-start 5793 '<FieldAccess for expected_type_var at ast.py:10789>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10789
--# expr-start 5792 '<FieldAccess for renamed_object at ast.py:10789>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10789
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 5792
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_8.Node);
--# expr-done 5793
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_9, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10789") else null)); 
--# expr-done 5794
--# expr-start 5796 '<FieldAccess for sub_equation at ast.py:10790>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10790
--# expr-start 5795 '<FieldAccess for renamed_object at ast.py:10790>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10790
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 5795
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 5796
And_Pred_3 := Create_And (Bind_Result_1, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10789") else null)); 
--# expr-done 5797
--# expr-start 5799 '<FieldAccess for matches_expected_assign_type at ast.py:10791>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10791
--# expr-start 5798 '<FieldAccess for renamed_object at ast.py:10791>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10791
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 5798
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_12.Node, Origin => Origin);
--# end
--# expr-done 5799
And_Pred_4 := Create_And (And_Pred_3, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10789") else null)); 
--# expr-done 5800
Scope_Result_1 := And_Pred_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2665;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10792") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_5 := Create_And (And_Pred_2, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10779") else null)); 
--# expr-done 5801
Let_Result := And_Pred_5; Inc_Ref (Let_Result);
--# end
--# expr-done 5802
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_837;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_837;
                     Finalizer_Scope_2664;
                     Finalizer_Scope_2665;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Xref_Equation;
--# end
--# property-start ObjectDecl.next_part_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10796
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Next_Part_For_Name
  (Node : Bare_Object_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_838 with Inline_Always;
            procedure Finalizer_Scope_2666 with Inline_Always;
      Var_Expr : Internal_Entity_Private_Part;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Package_Decl;
Fld_2 : Internal_Entity_Private_Part;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Fld_5 : Bare_Constant_Node;
Fld_6 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_838 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_838;
            procedure Finalizer_Scope_2666 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2666;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5815 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5814 '<If at ast.py:10798>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10798
--# expr-start 5803 '<FieldAccess for is_in_public_part at ast.py:10799>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10799
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_in_public_part
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_In_Public_Part (Node => Ent.Node);
--# end
--# expr-done 5803
if Fld_4 then
   --# expr-start 5805 '<FieldAccess for as_bool at ast.py:10799>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10799
--# expr-start 5804 '<FieldAccess for has_constant at ast.py:10799>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10799
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Object_Decl_F_Has_Constant;
--# expr-done 5804
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_6 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_5);
--# end
--# expr-done 5805
   If_Result_1 := Fld_6;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 5813 '<Cast to BasicDecl at ast.py:10803>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10803
--# expr-start 5812 '<EnvGet at ast.py:10802>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10802
--# expr-start 5810 '<FieldAccess for private_part at ast.py:10801>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10801
--# expr-start 5809 '<Cast to BasePackageDecl at ast.py:10801>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10801
--# expr-start 5808 '<as_entity at ast.py:10800>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10800
--# expr-start 5807 '<parent at ast.py:10800>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10800
--# expr-start 5806 '<FieldAccess for declarative_scope at ast.py:10800>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10800
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 5806
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 5807
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 5808
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 5809
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Cast_Result.Node.Base_Package_Decl_F_Private_Part, Info => Cast_Result.Info);
--# expr-done 5810
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 5811 '<FieldAccess for children_env at ast.py:10801>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10801
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 5811
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2666;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 5812
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5813
   If_Result_2 := Cast_Result_1;
else
   If_Result_2 := No_Entity_Basic_Decl;
end if;
--# expr-done 5814
Let_Result := If_Result_2; 
--# end
--# expr-done 5815
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_838;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_838;
                     Finalizer_Scope_2666;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Next_Part_For_Name;
--# end
--# property-start ObjectDecl.previous_part_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10807
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Previous_Part_For_Name
  (Node : Bare_Object_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_839 with Inline_Always;
      Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Package_Decl;
Fld_2 : Internal_Entity_Public_Part;
Fld_3 : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Fld_5 : Bare_Constant_Node;
Fld_6 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_839 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_839;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5828 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5827 '<If at ast.py:10809>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10809
--# expr-start 5816 '<FieldAccess for is_in_private_part at ast.py:10810>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10810
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5816
if Fld_4 then
   --# expr-start 5818 '<FieldAccess for as_bool at ast.py:10810>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10810
--# expr-start 5817 '<FieldAccess for has_constant at ast.py:10810>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10810
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Object_Decl_F_Has_Constant;
--# expr-done 5817
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_6 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_5);
--# end
--# expr-done 5818
   If_Result_1 := Fld_6;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 5826 '<Cast to BasicDecl at ast.py:10814>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10814
--# expr-start 5825 '<EnvGet at ast.py:10813>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10813
--# expr-start 5824 '<FieldAccess for children_env at ast.py:10812>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10812
--# expr-start 5823 '<FieldAccess for public_part at ast.py:10812>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10812
--# expr-start 5822 '<Cast to BasePackageDecl at ast.py:10812>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10812
--# expr-start 5821 '<as_entity at ast.py:10811>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10811
--# expr-start 5820 '<parent at ast.py:10811>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10811
--# expr-start 5819 '<FieldAccess for declarative_scope at ast.py:10811>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10811
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 5819
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 5820
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 5821
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 5822
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Public_Part (Node => Cast_Result.Node.Base_Package_Decl_F_Public_Part, Info => Cast_Result.Info);
--# expr-done 5823
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 5824
Env_Get_Result := AST_Envs.Get_First (Self => Fld_3, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 5825
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5826
   If_Result_2 := Cast_Result_1;
else
   If_Result_2 := No_Entity_Basic_Decl;
end if;
--# expr-done 5827
Let_Result := If_Result_2; 
--# end
--# expr-done 5828
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_839;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_839;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Previous_Part_For_Name;
--# end
--# property-start ObjectDecl.private_part_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10818
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Private_Part_Decl
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5832 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5831 '<FieldAccess for next_part_for_name at ast.py:10824>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10824
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5830 '<FieldAccess for name_symbol at ast.py:10825>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10825
--# expr-start 5829 '<FieldAccess for defining_name_or_raise at ast.py:10825>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10825
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5829
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 5830
--# property-call-start '[dispatcher]BasicDecl.next_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Name (Node => Ent.Node, Sym => Fld_1, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 5831
Let_Result := Fld_2; 
--# end
--# expr-done 5832
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Private_Part_Decl;
--# end
--# property-start ObjectDecl.public_part_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10828
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Public_Part_Decl
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5836 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5835 '<FieldAccess for previous_part_for_name at ast.py:10834>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10834
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5834 '<FieldAccess for name_symbol at ast.py:10835>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10835
--# expr-start 5833 '<FieldAccess for defining_name_or_raise at ast.py:10835>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10835
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5833
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 5834
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Ent.Node, Sym => Fld_1, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 5835
Let_Result := Fld_2; 
--# end
--# expr-done 5836
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Public_Part_Decl;
--# end
--# property-start ObjectDecl.next_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10838
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Next_Part_For_Decl
  (Node : Bare_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5838 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5837 '<FieldAccess for private_part_decl at ast.py:10840>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10840
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ObjectDecl.private_part_decl
Fld := Libadalang.Implementation.Object_Decl_P_Private_Part_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5837
Let_Result := Fld; 
--# end
--# expr-done 5838
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ObjectDecl.previous_part_for_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10842
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5840 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5839 '<FieldAccess for public_part_decl at ast.py:10844>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10844
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ObjectDecl.public_part_decl
Fld := Libadalang.Implementation.Object_Decl_P_Public_Part_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5839
Let_Result := Fld; 
--# end
--# expr-done 5840
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start ObjectDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10846
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Xref_Entry_Point
  (Node : Bare_Object_Decl
  )
   return Boolean
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Xref_Entry_Point;
--# end
--# property-start ObjectDecl._env_mappings_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10741
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_44
  (Node : Bare_Object_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1571 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Object_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1571 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1571;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5842 '<FieldAccess for env_mappings at ast.py:10741>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10741
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5841 '<FieldAccess for ids at ast.py:10741>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10741
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Object_Decl_F_Ids;
--# expr-done 5841
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 5842
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1571;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1571;
            raise;
      end;
   return Property_Result;
end Env_Mappings_44;
--# end
   --
   --  Primitives for Bare_Extended_Return_Stmt_Object_Decl
   --
      procedure Initialize_Fields_For_Extended_Return_Stmt_Object_Decl
        (Self : Bare_Extended_Return_Stmt_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Object_Decl
              (Self, Object_Decl_F_Ids, Object_Decl_F_Has_Aliased, Object_Decl_F_Has_Constant, Object_Decl_F_Mode, Object_Decl_F_Type_Expr, Object_Decl_F_Default_Expr, Object_Decl_F_Renaming_Clause, Object_Decl_F_Aspects);
      end Initialize_Fields_For_Extended_Return_Stmt_Object_Decl;
   --
   --  Primitives for Bare_No_Type_Object_Renaming_Decl
   --
      procedure Initialize_Fields_For_No_Type_Object_Renaming_Decl
        (Self : Bare_No_Type_Object_Renaming_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Object_Decl
              (Self, Object_Decl_F_Ids, Object_Decl_F_Has_Aliased, Object_Decl_F_Has_Constant, Object_Decl_F_Mode, Object_Decl_F_Type_Expr, Object_Decl_F_Default_Expr, Object_Decl_F_Renaming_Clause, Object_Decl_F_Aspects);
      end Initialize_Fields_For_No_Type_Object_Renaming_Decl;
--# property-start NoTypeObjectRenamingDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10912
pragma Warnings (Off, "is not referenced");
 function No_Type_Object_Renaming_Decl_P_Xref_Equation
  (Node : Bare_No_Type_Object_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_No_Type_Object_Renaming_Decl  := Bare_No_Type_Object_Renaming_Decl (Node);
     Ent : Internal_Entity_No_Type_Object_Renaming_Decl :=
       Internal_Entity_No_Type_Object_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_850 with Inline_Always;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_850 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_850;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5846 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5845 '<FieldAccess for sub_equation at ast.py:10914>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10914
--# expr-start 5844 '<FieldAccess for renamed_object at ast.py:10914>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10914
--# expr-start 5843 '<FieldAccess for renaming_clause at ast.py:10914>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10914
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 5843
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 5844
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 5845
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 5846
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_850;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_850;
            raise;
      end;
   return Property_Result;
end No_Type_Object_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Package_Renaming_Decl
   --
      procedure Package_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Package_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_71 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_72 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:11701");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_73 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:11703");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_74 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_75 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_77 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_76 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_79 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_78 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Renaming_Decl
        (Self : Bare_Package_Renaming_Decl
         ; Package_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Package_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Package_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Package_Renaming_Decl_F_Name := Package_Renaming_Decl_F_Name;
            Self.Package_Renaming_Decl_F_Renames := Package_Renaming_Decl_F_Renames;
            Self.Package_Renaming_Decl_F_Aspects := Package_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Package_Renaming_Decl;
   function Package_Renaming_Decl_F_Name
     (Node : Bare_Package_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Package_Renaming_Decl_F_Name;
   end;
   function Package_Renaming_Decl_F_Renames
     (Node : Bare_Package_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Package_Renaming_Decl_F_Renames;
   end;
--# property-start PackageRenamingDecl.renamed_package /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11660
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Renamed_Package
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_897 with Inline_Always;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Internal_Entity_Array_Access;
Var_Expr : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Renaming_Clause;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_897 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_897;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5856 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5847 '<FieldAccess for node_env at ast.py:11674>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11674
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5847
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind node_env Block_Var
--# expr-start 5849 '<EnvGet at ast.py:11675>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11675
--# expr-start 5848 '<Symbol __dummy at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Dummy); 
--# expr-done 5848
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 5849
Block_Var_1 := Env_Get_Result; Inc_Ref (Block_Var_1);
--# expr-start 5855 '<bind at ast.py:11681>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11681
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 5853 '<at at ast.py:11683>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11683
--# expr-start 5852 '<FieldAccess for env_elements at ast.py:11683>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11683
--# expr-start 5851 '<FieldAccess for renamed_object at ast.py:11683>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11683
--# expr-start 5850 '<FieldAccess for renames at ast.py:11683>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11683
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5850
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Fld_1.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_1.Info);
--# expr-done 5851
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld_2.Node, Env => Bound_Env, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 5852
Get_Result := Get (Self, Fld_3, 0, True); 
--# expr-done 5853
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 5854 '<Cast to BasicDecl at ast.py:11683>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11683
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 5854
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 5855
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 5856
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_897;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_897;
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Renamed_Package;
--# end
--# property-start PackageRenamingDecl.final_renamed_package /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11686
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Final_Renamed_Package
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Package_Renaming_Decl;
Fld : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Package_Renaming_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5860 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5857 '<FieldAccess for renamed_package at ast.py:11692>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11692
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PackageRenamingDecl.renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Renamed_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5857
Block_Var := Fld; 
--# bind pkg Block_Var
--# expr-start 5858 '<Cast to PackageRenamingDecl at ast.py:11693>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11693
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 5858
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Package_Renaming_Decl then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 5859 '<FieldAccess for final_renamed_package at ast.py:11694>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11694
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PackageRenamingDecl.final_renamed_package
Fld_1 := Libadalang.Implementation.Package_Renaming_Decl_P_Final_Renamed_Package (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5859
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 5860
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Final_Renamed_Package;
--# end
--# property-start PackageRenamingDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11724
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Defining_Names
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_899 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_899 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_899;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5861 '<FieldAccess for name at ast.py:11724>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11724
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 5861
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_899;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_899;
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Defining_Names;
--# end
--# property-start PackageRenamingDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11725
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Defining_Env
  (Node : Bare_Package_Renaming_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_900 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_900 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_900;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5863 '<FieldAccess for defining_env at ast.py:11725>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11725
--# expr-start 5862 '<FieldAccess for renamed_package at ast.py:11725>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11725
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PackageRenamingDecl.renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Renamed_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5862
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5863
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_900;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_900;
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Defining_Env;
--# end
--# property-start PackageRenamingDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11727
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start PackageRenamingDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11728
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Package_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_902 with Inline_Always;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
            procedure Finalizer_Scope_902 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_902;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5866 '<FieldAccess for xref_no_overloading at ast.py:11729>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11729
--# expr-start 5865 '<FieldAccess for renamed_object at ast.py:11729>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11729
--# expr-start 5864 '<FieldAccess for renames at ast.py:11729>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11729
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5864
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 5865
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 5866
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_902;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_902;
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Xref_Equation;
--# end
--# property-start PackageRenamingDecl._env_do_71 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11699
pragma Warnings (Off, "is not referenced");
 function Env_Do_71
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5867 '<FieldAccess for env_hook at ast.py:11699>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11699
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5867
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_71;
--# end
--# property-start PackageRenamingDecl._initial_env_72 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11701
pragma Warnings (Off, "is not referenced");
 function Initial_Env_72
  (Node : Bare_Package_Renaming_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1599 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1599 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1599;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5868 '<FieldAccess for child_decl_initial_env at ast.py:11701>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11701
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self);
--# end
--# expr-done 5868
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1599;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1599;
            raise;
      end;
   return Property_Result;
end Initial_Env_72;
--# end
--# property-start PackageRenamingDecl._env_mappings_73 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11703
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_73
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1600 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1600 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1600;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5871 '<New EnvAssoc at ast.py:11703>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11703
--# expr-start 5869 '<New DesignatedEnv at ast.py:11703>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11703
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5869
--# expr-start 5870 '<FieldAccess for name_symbol at ast.py:11704>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11704
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5870
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5871
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1600;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1600;
            raise;
      end;
   return Property_Result;
end Env_Mappings_73;
--# end
--# property-start PackageRenamingDecl._env_trans_parent_74 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11698
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_74
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_74;
--# end
--# property-start PackageRenamingDecl._env_do_75 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11710
pragma Warnings (Off, "is not referenced");
 function Env_Do_75
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1602 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1602 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1602;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5872 '<FieldAccess for populate_dependent_units at ast.py:11710>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11710
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5872
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1602;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1602;
            raise;
      end;
   return Property_Result;
end Env_Do_75;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_76 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11713
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_76
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1603 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1603 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1603;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5873 '<FieldAccess for top_level_use_package_clauses at ast.py:11713>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11713
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5873
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1603;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1603;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_76;
--# end
--# property-start PackageRenamingDecl._ref_cond_77 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11715
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_77
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5875 '<IsA LibraryItem, Subunit at ast.py:11715>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11715
--# expr-start 5874 '<parent at ast.py:11715>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11715
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5874
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5875
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_77;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_78 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11718
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_78
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1605 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1605 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1605;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5876 '<FieldAccess for top_level_use_type_clauses at ast.py:11718>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11718
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5876
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1605;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1605;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_78;
--# end
--# property-start PackageRenamingDecl._ref_cond_79 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11720
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_79
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5878 '<IsA LibraryItem, Subunit at ast.py:11720>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11720
--# expr-start 5877 '<parent at ast.py:11720>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11720
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5877
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5878
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_79;
--# end
   --
   --  Primitives for Bare_Single_Protected_Decl
   --
      procedure Single_Protected_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Protected_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_40 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:10521");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_41 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_42 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Protected_Decl
        (Self : Bare_Single_Protected_Decl
         ; Single_Protected_Decl_F_Name : Bare_Defining_Name
         ; Single_Protected_Decl_F_Aspects : Bare_Aspect_Spec
         ; Single_Protected_Decl_F_Interfaces : Bare_Parent_List
         ; Single_Protected_Decl_F_Definition : Bare_Protected_Def
        ) is
      begin
            Self.Single_Protected_Decl_F_Name := Single_Protected_Decl_F_Name;
            Self.Single_Protected_Decl_F_Aspects := Single_Protected_Decl_F_Aspects;
            Self.Single_Protected_Decl_F_Interfaces := Single_Protected_Decl_F_Interfaces;
            Self.Single_Protected_Decl_F_Definition := Single_Protected_Decl_F_Definition;
      end Initialize_Fields_For_Single_Protected_Decl;
   function Single_Protected_Decl_F_Name
     (Node : Bare_Single_Protected_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Single_Protected_Decl_F_Name;
   end;
   function Single_Protected_Decl_F_Interfaces
     (Node : Bare_Single_Protected_Decl) return Bare_Parent_List
   is
   begin
         return Node.Single_Protected_Decl_F_Interfaces;
   end;
   function Single_Protected_Decl_F_Definition
     (Node : Bare_Single_Protected_Decl) return Bare_Protected_Def
   is
   begin
         return Node.Single_Protected_Decl_F_Definition;
   end;
--# property-start SingleProtectedDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10488
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Defining_Names
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_815 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_815 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_815;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5879 '<FieldAccess for name at ast.py:10488>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10488
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Single_Protected_Decl_F_Name, Info => Ent.Info);
--# expr-done 5879
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_815;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_815;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Defining_Names;
--# end
--# property-start SingleProtectedDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10490
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Defining_Env
  (Node : Bare_Single_Protected_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_816 with Inline_Always;
            procedure Finalizer_Scope_2669 with Inline_Always;
      Var_Expr : Internal_Entity_Private_Part;
Fld : Internal_Entity_Protected_Def;
Fld_1 : Internal_Entity_Private_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_816 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_816;
            procedure Finalizer_Scope_2669 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2669;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5881 '<FieldAccess for private_part at ast.py:10491>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10491
--# expr-start 5880 '<FieldAccess for definition at ast.py:10491>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10491
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Protected_Def (Node => Ent.Node.Single_Protected_Decl_F_Definition, Info => Ent.Info);
--# expr-done 5880
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Private_Part (Node => Fld.Node.Protected_Def_F_Private_Part, Info => Fld.Info);
--# expr-done 5881
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Private_Part then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 5885 '<env_group at ast.py:10496>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10496
--# expr-start 5884 '<ArrayLiteral at ast.py:10496>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10496
--# expr-start 5882 '<FieldAccess for children_env at ast.py:10496>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10496
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5882
--# expr-start 5883 '<FieldAccess for children_env at ast.py:10496>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10496
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 5883
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Fld_3)); 
--# expr-done 5884
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 5885
Scope_Result := Group_Env;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2669;
   Result_Var := Scope_Result;
else
   --# expr-start 5886 '<FieldAccess for children_env at ast.py:10497>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10497
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5886
   Result_Var := Fld_4;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_816;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_816;
                     Finalizer_Scope_2669;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Defining_Env;
--# end
--# property-start SingleProtectedDecl.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10501
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Xref_Entry_Point
  (Node : Bare_Single_Protected_Decl
  )
   return Boolean
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Xref_Entry_Point;
--# end
--# property-start SingleProtectedDecl.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10503
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Xref_Equation
  (Node : Bare_Single_Protected_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_818 with Inline_Always;
            procedure Finalizer_Scope_2670 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_196 : Internal_Entity_Name;
Bare_Item_196 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_818 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_818;
            procedure Finalizer_Scope_2670 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2670;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5891 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5890 '<LogicAll at ast.py:10505>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10505
--# expr-start 5889 '<Map at ast.py:10505>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10505
   --# expr-start 5887 '<FieldAccess for interfaces at ast.py:10505>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10505
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Single_Protected_Decl_F_Interfaces, Info => Ent.Info);
--# expr-done 5887
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_196 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_196 := Untyped_Item_196; 
Is_Null := Bare_Item_196 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_196); 
   If_Result := As_Entity;
end if;
                  Item_196 := If_Result; 
   --# scope-start
               --# bind ifc Item_196
      --# expr-start 5888 '<FieldAccess for xref_equation at ast.py:10505>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10505
      if Item_196.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_196.Node, Env => Env, Origin => Origin, E_Info => Item_196.Info);
--# end
--# expr-done 5888
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2670;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5889
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10505") else null)); 
--# expr-done 5890
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 5891
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_818;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_818;
                     Finalizer_Scope_2670;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Xref_Equation;
--# end
--# property-start SingleProtectedDecl.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10507
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Env_Names
  (Node : Bare_Single_Protected_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_819 with Inline_Always;
            procedure Finalizer_Scope_2671 with Inline_Always;
      Var_Expr : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_819 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_819;
            procedure Finalizer_Scope_2671 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2671;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5894 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5892 '<FieldAccess for top_level_env_name at ast.py:10509>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10509
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5892
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 5893 '<to_symbol at ast.py:10510>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10510
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr); 
--# expr-done 5893
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2671;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 5894
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_819;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_819;
                     Finalizer_Scope_2671;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Env_Names;
--# end
--# property-start SingleProtectedDecl.declarative_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10513
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Declarative_Parts
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_820 with Inline_Always;
            procedure Finalizer_Scope_2672 with Inline_Always;
      Block_Var : Internal_Entity_Protected_Def;
Var_Expr : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Protected_Def;
Fld_1 : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
Singleton : Internal_Entity_Declarative_Part_Array_Access;
Fld_2 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Declarative_Part;
Singleton_1 : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result : Internal_Entity_Declarative_Part_Array_Access;
Result_Var : Internal_Entity_Declarative_Part_Array_Access;
Concat_Result : Internal_Entity_Declarative_Part_Array_Access;
Let_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_820 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_820;
            procedure Finalizer_Scope_2672 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_2672;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5901 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5895 '<FieldAccess for definition at ast.py:10515>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10515
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Protected_Def (Node => Ent.Node.Single_Protected_Decl_F_Definition, Info => Ent.Info);
--# expr-done 5895
Block_Var := Fld; 
--# bind pdef Block_Var
--# expr-start 5900 '<Concat at ast.py:10516>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10516
--# expr-start 5897 '<Cast to DeclarativePart at ast.py:10516>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10516
--# expr-start 5896 '<FieldAccess for public_part at ast.py:10516>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10516
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Block_Var.Node.Protected_Def_F_Public_Part, Info => Block_Var.Info);
--# expr-done 5896
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 5897
                Singleton := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
--# expr-start 5899 '<Cast to DeclarativePart at ast.py:10517>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10517
--# expr-start 5898 '<FieldAccess for private_part at ast.py:10517>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10517
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Block_Var.Node.Protected_Def_F_Private_Part, Info => Block_Var.Info);
--# expr-done 5898
      Cast_Result_1 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 5899
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Declarative_Part then
   --# scope-start
                Singleton_1 := Create_Internal_Entity_Declarative_Part_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr;
Scope_Result := Singleton_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2672;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Singleton, Result_Var); 
--# expr-done 5900
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5901
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_820;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_820;
                     Finalizer_Scope_2672;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Declarative_Parts;
--# end
--# property-start SingleProtectedDecl._env_mappings_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10521
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_40
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1567 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Single_Protected_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1567 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1567;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5904 '<New EnvAssoc at ast.py:10521>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10521
--# expr-start 5902 '<New DesignatedEnv at ast.py:10521>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10521
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5902
--# expr-start 5903 '<FieldAccess for name_symbol at ast.py:10521>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10521
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5903
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5904
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1567;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1567;
            raise;
      end;
   return Property_Result;
end Env_Mappings_40;
--# end
--# property-start SingleProtectedDecl._env_trans_parent_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10520
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_41
  (Node : Bare_Single_Protected_Decl
  )
   return Boolean
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_41;
--# end
--# property-start SingleProtectedDecl._env_names_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10522
pragma Warnings (Off, "is not referenced");
 function Env_Names_42
  (Node : Bare_Single_Protected_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1569 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1569 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1569;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5905 '<FieldAccess for env_names at ast.py:10522>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10522
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleProtectedDecl.env_names
Fld := Libadalang.Implementation.Single_Protected_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 5905
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1569;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1569;
            raise;
      end;
   return Property_Result;
end Env_Names_42;
--# end
   --
   --  Primitives for Bare_Single_Task_Decl
   --
      procedure Single_Task_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Task_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_37 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:10474");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_38 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_39 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Task_Decl
        (Self : Bare_Single_Task_Decl
         ; Single_Task_Decl_F_Task_Type : Bare_Single_Task_Type_Decl
        ) is
      begin
            Self.Single_Task_Decl_F_Task_Type := Single_Task_Decl_F_Task_Type;
      end Initialize_Fields_For_Single_Task_Decl;
   function Single_Task_Decl_F_Task_Type
     (Node : Bare_Single_Task_Decl) return Bare_Single_Task_Type_Decl
   is
   begin
         return Node.Single_Task_Decl_F_Task_Type;
   end;
--# property-start SingleTaskDecl.defining_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10462
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Defining_Names
  (Node : Bare_Single_Task_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
     Ent : Internal_Entity_Single_Task_Decl :=
       Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_811 with Inline_Always;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_811 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_811;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5907 '<FieldAccess for defining_names at ast.py:10462>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10462
--# expr-start 5906 '<FieldAccess for task_type at ast.py:10462>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10462
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 5906
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5907
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_811;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_811;
            raise;
      end;
   return Property_Result;
end Single_Task_Decl_P_Defining_Names;
--# end
--# property-start SingleTaskDecl.expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10463
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Expr_Type
  (Node : Bare_Single_Task_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
     Ent : Internal_Entity_Single_Task_Decl :=
       Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5908 '<FieldAccess for task_type at ast.py:10463>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10463
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 5908
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Task_Decl_P_Expr_Type;
--# end
--# property-start SingleTaskDecl.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10465
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Defining_Env
  (Node : Bare_Single_Task_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
     Ent : Internal_Entity_Single_Task_Decl :=
       Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_813 with Inline_Always;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_813 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_813;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5910 '<FieldAccess for defining_env at ast.py:10465>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10465
--# expr-start 5909 '<FieldAccess for task_type at ast.py:10465>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10465
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 5909
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5910
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_813;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_813;
            raise;
      end;
   return Property_Result;
end Single_Task_Decl_P_Defining_Env;
--# end
--# property-start SingleTaskDecl.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10467
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Env_Names
  (Node : Bare_Single_Task_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_814 with Inline_Always;
            procedure Finalizer_Scope_2673 with Inline_Always;
      Var_Expr : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_814 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_814;
            procedure Finalizer_Scope_2673 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2673;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5913 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5911 '<FieldAccess for top_level_env_name at ast.py:10469>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10469
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5911
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 5912 '<to_symbol at ast.py:10470>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10470
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr); 
--# expr-done 5912
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2673;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 5913
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_814;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_814;
                     Finalizer_Scope_2673;
            raise;
      end;
   return Property_Result;
end Single_Task_Decl_P_Env_Names;
--# end
--# property-start SingleTaskDecl._env_mappings_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10474
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_37
  (Node : Bare_Single_Task_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1564 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Single_Task_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1564 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1564;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5916 '<New EnvAssoc at ast.py:10474>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10474
--# expr-start 5914 '<New DesignatedEnv at ast.py:10474>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10474
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 5914
--# expr-start 5915 '<FieldAccess for name_symbol at ast.py:10474>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10474
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 5915
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5916
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1564;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1564;
            raise;
      end;
   return Property_Result;
end Env_Mappings_37;
--# end
--# property-start SingleTaskDecl._env_trans_parent_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10473
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_38
  (Node : Bare_Single_Task_Decl
  )
   return Boolean
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_38;
--# end
--# property-start SingleTaskDecl._env_names_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10475
pragma Warnings (Off, "is not referenced");
 function Env_Names_39
  (Node : Bare_Single_Task_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1566 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1566 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1566;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5917 '<FieldAccess for env_names at ast.py:10475>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10475
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTaskDecl.env_names
Fld := Libadalang.Implementation.Single_Task_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 5917
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1566;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1566;
            raise;
      end;
   return Property_Result;
end Env_Names_39;
--# end
   --
   --  Primitives for Bare_Case_Stmt_Alternative
   --
      procedure Initialize_Fields_For_Case_Stmt_Alternative
        (Self : Bare_Case_Stmt_Alternative
         ; Case_Stmt_Alternative_F_Choices : Bare_Alternatives_List
         ; Case_Stmt_Alternative_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Case_Stmt_Alternative_F_Choices := Case_Stmt_Alternative_F_Choices;
            Self.Case_Stmt_Alternative_F_Stmts := Case_Stmt_Alternative_F_Stmts;
      end Initialize_Fields_For_Case_Stmt_Alternative;
   function Case_Stmt_Alternative_F_Choices
     (Node : Bare_Case_Stmt_Alternative) return Bare_Alternatives_List
   is
   begin
         return Node.Case_Stmt_Alternative_F_Choices;
   end;
   function Case_Stmt_Alternative_F_Stmts
     (Node : Bare_Case_Stmt_Alternative) return Bare_Stmt_List
   is
   begin
         return Node.Case_Stmt_Alternative_F_Stmts;
   end;
--# property-start CaseStmtAlternative.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21479
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_Alternative_P_Xref_Entry_Point
  (Node : Bare_Case_Stmt_Alternative
  )
   return Boolean
is
   Self : Bare_Case_Stmt_Alternative  := Bare_Case_Stmt_Alternative (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Case_Stmt_Alternative_P_Xref_Entry_Point;
--# end
--# property-start CaseStmtAlternative.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21481
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_Alternative_P_Xref_Equation
  (Node : Bare_Case_Stmt_Alternative
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Stmt_Alternative  := Bare_Case_Stmt_Alternative (Node);
     Ent : Internal_Entity_Case_Stmt_Alternative :=
       Internal_Entity_Case_Stmt_Alternative'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1467 with Inline_Always;
            procedure Finalizer_Scope_2674 with Inline_Always;
            procedure Finalizer_Scope_2675 with Inline_Always;
            procedure Finalizer_Scope_2677 with Inline_Always;
            procedure Finalizer_Scope_2678 with Inline_Always;
      Block_Var : Internal_Entity_Case_Stmt;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Var_Expr : Internal_Entity_Name;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Case_Stmt;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Alternatives_List;
Fld_5 : Bare_Alternatives_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Cast_Expr_2 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Fld_11 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result_1 : Logic_Equation;
Fld_12 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_4 : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_5 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_6 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_7 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Match_Result : Logic_Equation;
Item_197 : Internal_Entity;
Bare_Item_197 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1467 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1467;
            procedure Finalizer_Scope_2674 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2674;
            procedure Finalizer_Scope_2675 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_2675;
            procedure Finalizer_Scope_2677 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2677;
            procedure Finalizer_Scope_2678 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_2678;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5941 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 5920 '<Cast to CaseStmt at ast.py:21483>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21483
--# expr-start 5919 '<parent at ast.py:21483>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21483
--# expr-start 5918 '<parent at ast.py:21483>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21483
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5918
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 5919
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Case_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Case_Stmt
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5920
Block_Var := Cast_Result; 
--# bind case_stmt Block_Var
--# expr-start 5922 '<FieldAccess for expression_type at ast.py:21486>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21486
--# expr-start 5921 '<FieldAccess for expr at ast.py:21486>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21486
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Block_Var.Node.Case_Stmt_F_Expr, Info => Block_Var.Info);
--# expr-done 5921
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 5922
Block_Var_1 := Fld_3; 
--# bind selected_type Block_Var_1
--# expr-start 5940 '<LogicAll at ast.py:21488>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21488
--# expr-start 5939 '<Map at ast.py:21488>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21488
   --# expr-start 5923 '<FieldAccess for choices at ast.py:21488>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21488
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Case_Stmt_Alternative_F_Choices, Info => Ent.Info);
--# expr-done 5923
Fld_5 := Fld_4.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_5;
      begin
         for Untyped_Item_197 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_197 := Untyped_Item_197; 
Is_Null := Bare_Item_197 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_197); 
   If_Result := As_Entity;
end if;
                  Item_197 := If_Result; 
   --# scope-start
               --# bind c Item_197
      --# expr-start 5938 '<Match at ast.py:21488>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21488
      if Item_197.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_197; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_4; 
--# bind e Match_0
--# expr-start 5936 '<If at ast.py:21491>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21491
--# expr-start 5927 '<Not at ast.py:21492>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21492
--# expr-start 5926 '<IsNull at ast.py:21492>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21492
--# expr-start 5924 '<Cast to Name at ast.py:21492>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21492
Cast_Expr_2 := Match_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Name;
   end if;
--# expr-done 5924
Var_Expr := Cast_Result_3;
if Var_Expr /= No_Entity_Name then
   --# scope-start
--# expr-start 5925 '<FieldAccess for name_designated_type at ast.py:21492>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21492
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_11 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5925
Scope_Result := Fld_11;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Is_Null_1 := Result_Var.Node = null; 
--# expr-done 5926
Not_Val := not (Is_Null_1); 
--# expr-done 5927
if Not_Val then
   --# expr-start 5929 '<FieldAccess for xref_no_overloading at ast.py:21494>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21494
--# expr-start 5928 '<Cast to Name at ast.py:21494>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21494
Cast_Expr := Match_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 5928
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_7 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 5929
   If_Result_1 := Fld_7;
else
   --# expr-start 5935 '<And at ast.py:21496>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21496
--# expr-start 5933 '<And at ast.py:21496>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21496
--# expr-start 5931 '<Bind at ast.py:21496>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21496
--# expr-start 5930 '<FieldAccess for expected_type_var at ast.py:21496>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21496
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Match_0.Node);
--# expr-done 5930
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Cast_Expr_1 := Block_Var_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Solver.Create_Assign (Fld_8, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21496") else null)); 
--# expr-done 5931
--# expr-start 5932 '<FieldAccess for sub_equation at ast.py:21497>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21497
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 5932
And_Pred := Create_And (Bind_Result, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21496") else null)); 
--# expr-done 5933
--# expr-start 5934 '<FieldAccess for matches_expected_type at ast.py:21498>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21498
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_10 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Match_0.Node, Origin => Origin);
--# end
--# expr-done 5934
And_Pred_1 := Create_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21496") else null)); 
--# expr-done 5935
   If_Result_1 := And_Pred_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5936
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2675;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind t Match_1
--# expr-start 5937 '<FieldAccess for xref_equation at ast.py:21502>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21502
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_12 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 5937
Let_Result_1 := Fld_12; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2677;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind _ Match_2
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21504") else null)); 
Let_Result_2 := True_Rel; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2678;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_7; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;
--# expr-done 5938
         declare
            Item_To_Append : constant Logic_Equation := Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2674;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5939
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21488") else null)); 
--# expr-done 5940
Let_Result_4 := Logic_Boolean_Op; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5941
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1467;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1467;
                     Finalizer_Scope_2674;
                     Finalizer_Scope_2675;
                     Finalizer_Scope_2677;
                     Finalizer_Scope_2678;
            raise;
      end;
   return Property_Result;
end Case_Stmt_Alternative_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Compilation_Unit
   --
      procedure Initialize_Fields_For_Compilation_Unit
        (Self : Bare_Compilation_Unit
         ; Compilation_Unit_F_Prelude : Bare_Ada_Node_List
         ; Compilation_Unit_F_Body : Bare_Ada_Node
         ; Compilation_Unit_F_Pragmas : Bare_Pragma_Node_List
        ) is
      begin
            Self.Compilation_Unit_F_Prelude := Compilation_Unit_F_Prelude;
            Self.Compilation_Unit_F_Body := Compilation_Unit_F_Body;
            Self.Compilation_Unit_F_Pragmas := Compilation_Unit_F_Pragmas;
      Self.Compilation_Unit_No_Env := Empty_Env;
      end Initialize_Fields_For_Compilation_Unit;
   function Compilation_Unit_F_Prelude
     (Node : Bare_Compilation_Unit) return Bare_Ada_Node_List
   is
   begin
         return Node.Compilation_Unit_F_Prelude;
   end;
   function Compilation_Unit_F_Body
     (Node : Bare_Compilation_Unit) return Bare_Ada_Node
   is
   begin
         return Node.Compilation_Unit_F_Body;
   end;
   function Compilation_Unit_F_Pragmas
     (Node : Bare_Compilation_Unit) return Bare_Pragma_Node_List
   is
   begin
         return Node.Compilation_Unit_F_Pragmas;
   end;
--# property-start CompilationUnit.syntactic_fully_qualified_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20071
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Syntactic_Fully_Qualified_Name
  (Node : Bare_Compilation_Unit
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1393 with Inline_Always;
            procedure Finalizer_Scope_1836 with Inline_Always;
            procedure Finalizer_Scope_1837 with Inline_Always;
            procedure Finalizer_Scope_1838 with Inline_Always;
      Match_0 : Internal_Entity_Library_Item;
Match_1 : Internal_Entity_Subunit;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity_Defining_Name;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Fld_4 : Internal_Entity_Name;
Fld_5 : Symbol_Type_Array_Access;
Fld_6 : Internal_Entity_Body_Node;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Symbol_Type_Array_Access;
Concat_Result : Symbol_Type_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Library_Item;
Let_Result : Symbol_Type_Array_Access;
Scope_Result_1 : Symbol_Type_Array_Access;
Cast_Result_1 : Internal_Entity_Subunit;
Let_Result_1 : Symbol_Type_Array_Access;
Scope_Result_2 : Symbol_Type_Array_Access;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Symbol_Type_Array_Access;
Scope_Result_3 : Symbol_Type_Array_Access;
Match_Result : Symbol_Type_Array_Access;
Let_Result_3 : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1393 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1393;
            procedure Finalizer_Scope_1836 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1836;
            procedure Finalizer_Scope_1837 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1837;
            procedure Finalizer_Scope_1838 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1838;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5954 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 5953 '<Match at ast.py:20076>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20076
--# expr-start 5943 '<FieldAccess for body at ast.py:20076>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20076
--# expr-start 5942 '<as_bare_entity at ast.py:20076>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20076
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 5942
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => If_Result.Node.Compilation_Unit_F_Body, Info => If_Result.Info);
--# expr-done 5943
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Library_Item
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind li Match_0
--# expr-start 5945 '<FieldAccess for defining_name at ast.py:20077>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20077
--# expr-start 5944 '<FieldAccess for item at ast.py:20077>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20077
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Basic_Decl (Node => Match_0.Node.Library_Item_F_Item, Info => Match_0.Info);
--# expr-done 5944
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 5945
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 5946 '<FieldAccess for as_symbol_array at ast.py:20077>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20077
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Var_Expr.Node);
--# end
--# expr-done 5946
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1837;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1836;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subunit
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 5952 '<Concat at ast.py:20078>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20078
--# expr-start 5948 '<FieldAccess for as_symbol_array at ast.py:20078>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20078
--# expr-start 5947 '<FieldAccess for name at ast.py:20078>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20078
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_1.Node.Subunit_F_Name, Info => Match_1.Info);
--# expr-done 5947
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_4.Node);
--# end
--# expr-done 5948
--# expr-start 5951 '<FieldAccess for as_symbol_array at ast.py:20079>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20079
--# expr-start 5950 '<FieldAccess for defining_name at ast.py:20079>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20079
--# expr-start 5949 '<FieldAccess for body at ast.py:20079>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20079
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Body_Node (Node => Match_1.Node.Subunit_F_Body, Info => Match_1.Info);
--# expr-done 5949
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 5950
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_7.Node);
--# end
--# expr-done 5951
Concat_Result := Concat (Fld_5, Fld_8); 
--# expr-done 5952
Let_Result_1 := Concat_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1838;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5953
Let_Result_3 := Match_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 5954
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1393;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1393;
                     Finalizer_Scope_1836;
                     Finalizer_Scope_1837;
                     Finalizer_Scope_1838;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Syntactic_Fully_Qualified_Name;
--# end
--# property-start CompilationUnit.unit_kind /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20086
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Kind
  (Node : Bare_Compilation_Unit
  )
   return Analysis_Unit_Kind
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Analysis_Unit_Kind;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Match_0_1 : Bare_Body_Node;
Match_1_1 : Bare_Basic_Decl;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Basic_Decl;
Match_Prefix : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Let_Result : Analysis_Unit_Kind;
Scope_Result : Analysis_Unit_Kind;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Analysis_Unit_Kind;
Scope_Result_1 : Analysis_Unit_Kind;
Match_Result : Analysis_Unit_Kind;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Library_Item;
Let_Result_2 : Analysis_Unit_Kind;
Scope_Result_2 : Analysis_Unit_Kind;
Cast_Result_3 : Bare_Subunit;
Let_Result_3 : Analysis_Unit_Kind;
Scope_Result_3 : Analysis_Unit_Kind;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_4 : Analysis_Unit_Kind;
Scope_Result_4 : Analysis_Unit_Kind;
Match_Result_1 : Analysis_Unit_Kind;
Let_Result_5 : Analysis_Unit_Kind;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5959 '<Block at ???>' Let_Result_5 None
--# scope-start
--# expr-start 5958 '<Match at ast.py:20091>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20091
--# expr-start 5955 '<FieldAccess for body at ast.py:20091>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20091
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 5955
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix_1;
Match_0 := Cast_Result_2; 
--# bind li Match_0
--# expr-start 5957 '<Match at ast.py:20092>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20092
--# expr-start 5956 '<FieldAccess for item at ast.py:20092>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20092
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_0.Library_Item_F_Item;
--# expr-done 5956
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Basic_Decl (Match_Prefix.Kind) is
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0_1 := Cast_Result; 
--# bind _ Match_0_1
Let_Result := Unit_Body; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1_1 := Cast_Result_1; 
--# bind _ Match_1_1
Let_Result_1 := Unit_Specification; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 5957
Let_Result_2 := Match_Result; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix_1;
Match_1 := Cast_Result_3; 
--# bind _ Match_1
Let_Result_3 := Unit_Body; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_4 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
end case;
--# expr-done 5958
Let_Result_5 := Match_Result_1; 
--# end
--# expr-done 5959
         Property_Result := Let_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Unit_Kind;
--# end
--# property-start CompilationUnit.referenced_units_in /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20102
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Referenced_Units_In
  (Node : Bare_Compilation_Unit
      ; Name : Internal_Entity_Name
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1395 with Inline_Always;
            procedure Finalizer_Scope_2681 with Inline_Always;
            procedure Finalizer_Scope_2682 with Inline_Always;
      Block_Var : Bare_Compilation_Unit;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Dotted_Name;
Var_Expr_2 : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Compilation_Unit;
Scope_Result : Bare_Compilation_Unit;
Result_Var : Bare_Compilation_Unit;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_1 : Internal_Entity_Compilation_Unit_Array_Access;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Singleton : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_2 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_2 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1395 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1395;
            procedure Finalizer_Scope_2681 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2681;
            procedure Finalizer_Scope_2682 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2682;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5967 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5960 '<FieldAccess for referenced_decl at ast.py:20108>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20108
      if Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Name.Node, Imprecise_Fallback => False, E_Info => Name.Info);
--# end
--# expr-done 5960
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5961 '<FieldAccess for enclosing_compilation_unit at ast.py:20108>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20108
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_1 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Var_Expr.Node);
--# end
--# expr-done 5961
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Block_Var := Result_Var; 
--# bind self_refd Block_Var
--# expr-start 5966 '<Concat at ast.py:20112>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20112
--# expr-start 5962 '<Cast to DottedName at ast.py:20109>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20109
Cast_Expr := Name; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 5962
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr_1
--# expr-start 5964 '<FieldAccess for referenced_units_in at ast.py:20110>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20110
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5963 '<FieldAccess for prefix at ast.py:20110>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20110
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Dotted_Name_F_Prefix, Info => Var_Expr_1.Info);
--# expr-done 5963
--# property-call-start CompilationUnit.referenced_units_in
Fld_3 := Libadalang.Implementation.Compilation_Unit_P_Referenced_Units_In (Node => Self, Name => Fld_2);
--# end
--# expr-done 5964
Scope_Result_1 := Fld_3;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2681;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 5965 '<as_bare_entity at ast.py:20112>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20112
Is_Null := Block_Var = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Block_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 5965
Var_Expr_2 := If_Result;
if Var_Expr_2 /= No_Entity_Compilation_Unit then
   --# scope-start
                Singleton := Create_Internal_Entity_Compilation_Unit_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr_2;
Scope_Result_2 := Singleton;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2682;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var_1, Result_Var_2); 
--# expr-done 5966
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5967
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1395;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1395;
                     Finalizer_Scope_2681;
                     Finalizer_Scope_2682;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Referenced_Units_In;
--# end
--# property-start CompilationUnit.withed_units /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20114
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Withed_Units
  (Node : Bare_Compilation_Unit
      ; Include_Privates : Boolean
         := True
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind include_privates Include_Privates
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1396 with Inline_Always;
            procedure Finalizer_Scope_2683 with Inline_Always;
      Fld : Bare_Name_Array_Access;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Compilation_Unit_Array_Access;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1396 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1396;
            procedure Finalizer_Scope_2683 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2683;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Withed_Units,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Privates);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5973 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5972 '<unique at ast.py:20129>' Unique_Array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20129
--# expr-start 5971 '<Mapcat at ast.py:20124>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20124
   --# expr-start 5968 '<FieldAccess for top_level_with_package_clauses at ast.py:20124>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20124
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_with_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_With_Package_Clauses (Node => Self, Include_Privates => Include_Privates);
--# end
--# expr-done 5968
      declare
         Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Name_Array_Access := Fld;
      begin
         for Item_198 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_198
      --# expr-start 5970 '<FieldAccess for referenced_units_in at ast.py:20128>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20128
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5969 '<as_bare_entity at ast.py:20128>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20128
Is_Null := Item_198 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Item_198); 
   If_Result := As_Entity;
end if;
--# expr-done 5969
--# property-call-start CompilationUnit.referenced_units_in
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Referenced_Units_In (Node => Self, Name => If_Result);
--# end
--# expr-done 5970
         for Item_To_Append of
               Fld_1.Items
         loop
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2683;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Compilation_Unit_Array
           (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5971
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 5972
Let_Result := Unique_Array; Inc_Ref (Let_Result);
--# end
--# expr-done 5973
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1396;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1396;
                     Finalizer_Scope_2683;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                        As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Withed_Units;
--# end
--# property-start CompilationUnit.imported_units /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20131
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Imported_Units
  (Node : Bare_Compilation_Unit
      ; Include_Privates : Boolean
         := True
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind include_privates Include_Privates
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1397 with Inline_Always;
            procedure Finalizer_Scope_2684 with Inline_Always;
            procedure Finalizer_Scope_2685 with Inline_Always;
            procedure Finalizer_Scope_2686 with Inline_Always;
            procedure Finalizer_Scope_2687 with Inline_Always;
            procedure Finalizer_Scope_2688 with Inline_Always;
            procedure Finalizer_Scope_2689 with Inline_Always;
      Var_Expr : Internal_Entity_Compilation_Unit_Array_Access;
Var_Expr_1 : Bare_Base_Subp_Body;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Bare_Ada_Node;
Var_Expr_4 : Lexical_Env := Empty_Env;
Var_Expr_5 : Bare_Basic_Decl;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Bare_Basic_Decl;
Cast_Result : Bare_Base_Subp_Body;
As_Entity : Internal_Entity_Base_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Body;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Compilation_Unit;
As_Entity_1 : Internal_Entity_Compilation_Unit;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Compilation_Unit;
Singleton : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_1 : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_2 : Internal_Entity_Compilation_Unit_Array_Access;
Fld_4 : Bare_Basic_Decl;
Fld_5 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Scope_Result_4 : Bare_Ada_Node;
Result_Var_3 : Bare_Ada_Node;
Fld_6 : Bare_Compilation_Unit;
As_Entity_2 : Internal_Entity_Compilation_Unit;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Compilation_Unit;
Singleton_1 : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_5 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_4 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_5 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1397 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1397;
            procedure Finalizer_Scope_2684 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2684;
            procedure Finalizer_Scope_2685 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2685;
            procedure Finalizer_Scope_2686 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_2686;
            procedure Finalizer_Scope_2687 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2687;
            procedure Finalizer_Scope_2688 is
            begin
                     Dec_Ref (Var_Expr_4);
            end Finalizer_Scope_2688;
            procedure Finalizer_Scope_2689 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_2689;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Imported_Units,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Privates);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5987 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 5986 '<Concat at ast.py:20140>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20140
--# expr-start 5974 '<FieldAccess for withed_units at ast.py:20140>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20140
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.withed_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Withed_Units (Node => Self, Include_Privates => Include_Privates);
--# end
--# expr-done 5974
--# expr-start 5976 '<Cast to BaseSubpBody at ast.py:20145>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20145
--# expr-start 5975 '<FieldAccess for decl at ast.py:20145>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20145
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self);
--# end
--# expr-done 5975
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5976
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind subp Var_Expr_1
--# expr-start 5978 '<FieldAccess for subp_previous_part at ast.py:20149>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20149
--# expr-start 5977 '<as_bare_entity at ast.py:20149>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20149
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Body;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 5977
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subp_previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => If_Result.Node, Imprecise_Fallback => False, E_Info => If_Result.Info);
--# end
--# expr-done 5978
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr_2
--# expr-start 5980 '<as_bare_entity at ast.py:20151>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20151
--# expr-start 5979 '<FieldAccess for enclosing_compilation_unit at ast.py:20151>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20151
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_3 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Var_Expr_2.Node);
--# end
--# expr-done 5979
Is_Null_1 := Fld_3 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Compilation_Unit;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 5980
                Singleton := Create_Internal_Entity_Compilation_Unit_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result_1;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2685;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2684;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Var_Expr := Result_Var_1;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Compilation_Unit_Array_Type then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_2 := Var_Expr;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2686;
   Result_Var_5 := Scope_Result_2;
else
--# expr-start 5981 '<FieldAccess for decl at ast.py:20153>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20153
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self);
--# end
--# expr-done 5981
Var_Expr_5 := Fld_4;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5982 '<FieldAccess for node_env at ast.py:20153>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20153
      if Var_Expr_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Node_Env (Node => Var_Expr_5);
--# expr-done 5982
Scope_Result_3 := Fld_5;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2687;
   Result_Var_2 := Scope_Result_3;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
Var_Expr_4 := Result_Var_2;
      Inc_Ref (Var_Expr_4);
if Var_Expr_4 /= Empty_Env then
   --# scope-start
--# expr-start 5983 '<env_node at ast.py:20153>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20153
Env_Node := AST_Envs.Env_Node (Var_Expr_4); 
--# expr-done 5983
Scope_Result_4 := Env_Node;
   --# end
      Finalizer_Scope_2688;
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Bare_Ada_Node;
end if;
Var_Expr_3 := Result_Var_3;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr_3
--# expr-start 5985 '<as_bare_entity at ast.py:20155>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20155
--# expr-start 5984 '<FieldAccess for enclosing_compilation_unit at ast.py:20155>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20155
      if Var_Expr_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_6 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Var_Expr_3);
--# end
--# expr-done 5984
Is_Null_2 := Fld_6 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Compilation_Unit;
else
As_Entity_2 := (Info => No_Entity_Info, Node => Fld_6); 
   If_Result_2 := As_Entity_2;
end if;
--# expr-done 5985
                Singleton_1 := Create_Internal_Entity_Compilation_Unit_Array (Items_Count => 1);
                Singleton_1.Items (1) := If_Result_2;
Scope_Result_5 := Singleton_1;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2689;
   Result_Var_4 := Scope_Result_5;
else
   Result_Var_4 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
   Result_Var_5 := Result_Var_4;
end if;
      Inc_Ref (Result_Var_5);
Concat_Result := Concat (Fld, Result_Var_5); 
--# expr-done 5986
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5987
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1397;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1397;
                     Finalizer_Scope_2684;
                     Finalizer_Scope_2685;
                     Finalizer_Scope_2686;
                     Finalizer_Scope_2687;
                     Finalizer_Scope_2688;
                     Finalizer_Scope_2689;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                        As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Imported_Units;
--# end
--# property-start CompilationUnit.privately_imported_units /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20159
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Privately_Imported_Units
  (Node : Bare_Compilation_Unit
  )
   return Internal_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Internal_Unit_Array_Access;
            procedure Finalizer_Scope_1398 with Inline_Always;
      Var_Expr : Bare_With_Clause;
All_Imports : Internal_Entity_Compilation_Unit_Array_Access;
Public_Imports : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Internal_Entity_Compilation_Unit_Array_Access;
Fld_2 : Internal_Unit;
Cast_Expr : Internal_Entity_Compilation_Unit;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Compilation_Unit;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Unit_Array_Access;
Let_Result : Internal_Unit_Array_Access;
Fld_3 : Bare_Ada_Node_List;
Cast_Expr_2 : Bare_Ada_Node;
Cast_Result_2 : Bare_With_Clause;
Fld_4 : Bare_Private_Node;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Item_199 : Bare_Ada_Node;
Quantifier_Result_1 : Boolean;
If_Result : Internal_Unit_Array_Access;
Let_Result_1 : Internal_Unit_Array_Access;
            procedure Finalizer_Scope_1398 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (All_Imports);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Public_Imports);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1398;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Privately_Imported_Units,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Unit_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6002 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 6001 '<If at ast.py:20171>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20171
--# expr-start 5992 '<AnyQuantifier at ast.py:20174>' Quantifier_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20174
   --# expr-start 5988 '<FieldAccess for prelude at ast.py:20174>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20174
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Compilation_Unit_F_Prelude;
--# expr-done 5988
      Quantifier_Result_1 := False;
      if Fld_3 = null then
      null;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_3;
      begin
         for Untyped_Item_199 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_199 := Untyped_Item_199; 
   --# scope-start
               --# bind clause Item_199
--# expr-start 5989 '<Cast to WithClause at ast.py:20175>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20175
Cast_Expr_2 := Item_199; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_With_Clause_Range
   then
      Cast_Result_2 := Cast_Expr_2;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 5989
Var_Expr := Cast_Result_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5991 '<FieldAccess for as_bool at ast.py:20175>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20175
--# expr-start 5990 '<FieldAccess for has_private at ast.py:20175>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20175
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Var_Expr.With_Clause_F_Has_Private;
--# expr-done 5990
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Private.as_bool'
Fld_5 := Libadalang.Implementation.Dispatcher_Private_Node_P_As_Bool (Node => Fld_4);
--# end
--# expr-done 5991
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      Quantifier_Result_1 := Result_Var;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
      end if;
--# expr-done 5992
if Quantifier_Result_1 then
   --# expr-start 6000 '<Let at ast.py:20177>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20177
--# scope-start
--# expr-start 5993 '<FieldAccess for imported_units at ast.py:20179>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20179
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.imported_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Imported_Units (Node => Self, Include_Privates => True);
--# end
--# expr-done 5993
All_Imports := Fld; Inc_Ref (All_Imports);
--# bind all_imports All_Imports
--# expr-start 5994 '<FieldAccess for imported_units at ast.py:20180>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20180
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.imported_units
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Imported_Units (Node => Self, Include_Privates => False);
--# end
--# expr-done 5994
Public_Imports := Fld_1; Inc_Ref (Public_Imports);
--# bind public_imports Public_Imports
--# expr-start 5999 '<FilterMap at ast.py:20181>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20181
      declare
         Map_Result_Vec : Internal_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := All_Imports;
      begin
         for Item_200 of
               Collection.Items
         loop
   --# scope-start
               --# bind cu Item_200
         --# expr-start 5997 '<Not at ast.py:20183>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20183
--# expr-start 5996 '<Contains at ast.py:20183>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20183
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Public_Imports;
      begin
         for Item_201 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_201
      --# expr-start 5995 '<Eq at ???>' Is_Equiv None
Cast_Expr := Item_201; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_200; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 5995
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 5996
Not_Val := not (Quantifier_Result); 
--# expr-done 5997
         if Not_Val then
      --# expr-start 5998 '<FieldAccess for unit at ast.py:20182>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20182
      if Item_200.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Unit (Node => Item_200.Node);
--# expr-done 5998
         declare
            Item_To_Append : constant Internal_Unit := Fld_2;
         begin
            Internal_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Unit_Array
           (Items_Count => Natural (Internal_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5999
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6000
   If_Result := Let_Result;
else
   If_Result := No_Internal_Unit_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 6001
Let_Result_1 := If_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6002
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1398;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1398;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Unit_Array_Access,
                        As_Internal_Unit_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Privately_Imported_Units;
--# end
--# property-start CompilationUnit.unit_dependencies_helper /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20189
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Dependencies_Helper
  (Node : Bare_Compilation_Unit
      ; Visited : Internal_Entity_Compilation_Unit_Array_Access
      ; To_Visit : Internal_Entity_Compilation_Unit_Array_Access
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind visited Visited
   --# bind to_visit To_Visit
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1399 with Inline_Always;
            procedure Finalizer_Scope_2694 with Inline_Always;
      Block_Var : Internal_Entity_Compilation_Unit_Array_Access;
Block_Var_1 : Internal_Entity_Compilation_Unit_Array_Access;
Block_Var_2 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Cast_Expr : Internal_Entity_Compilation_Unit;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Compilation_Unit;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Internal_Entity_Compilation_Unit_Array_Access;
Len : Integer;
Comp_Result : Boolean;
If_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1399 is
            begin
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1399;
            procedure Finalizer_Scope_2694 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_2694;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6014 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6003 '<Concat at ast.py:20199>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20199
Concat_Result := Concat (Visited, To_Visit); 
--# expr-done 6003
Block_Var := Concat_Result; Inc_Ref (Block_Var);
--# bind now_visited Block_Var
--# expr-start 6006 '<unique at ast.py:20200>' Unique_Array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20200
--# expr-start 6005 '<Mapcat at ast.py:20200>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20200
      declare
         Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := To_Visit;
      begin
         for Item_202 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_202
      --# expr-start 6004 '<FieldAccess for imported_units at ast.py:20200>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20200
      if Item_202.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.imported_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Imported_Units (Node => Item_202.Node);
--# end
--# expr-done 6004
         for Item_To_Append of
               Fld.Items
         loop
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2694;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Compilation_Unit_Array
           (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6005
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 6006
Block_Var_1 := Unique_Array; Inc_Ref (Block_Var_1);
--# bind new_imports Block_Var_1
--# expr-start 6010 '<Filter at ast.py:20202>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20202
      declare
         Map_Result_1_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Block_Var_1;
      begin
         for Item_203 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_203
         --# expr-start 6009 '<Not at ast.py:20202>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20202
--# expr-start 6008 '<Contains at ast.py:20202>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20202
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Block_Var;
      begin
         for Item_204 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_204
      --# expr-start 6007 '<Eq at ???>' Is_Equiv None
Cast_Expr := Item_204; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_203; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 6007
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6008
Not_Val := not (Quantifier_Result); 
--# expr-done 6009
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit := Item_203;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Compilation_Unit_Array
           (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 6010
Block_Var_2 := Map_Result_1; Inc_Ref (Block_Var_2);
--# bind to_visit_next Block_Var_2
--# expr-start 6013 '<If at ast.py:20204>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20204
--# expr-start 6011 '<length at ast.py:20205>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20205
Len := Length (Block_Var_2); 
--# expr-done 6011
Comp_Result := Len > 0; 
if Comp_Result then
   --# expr-start 6012 '<FieldAccess for unit_dependencies_helper at ast.py:20206>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20206
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.unit_dependencies_helper
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Unit_Dependencies_Helper (Node => Self, Visited => Block_Var, To_Visit => Block_Var_2);
--# end
--# expr-done 6012
   If_Result := Fld_1;
else
   If_Result := Block_Var;
end if;
      Inc_Ref (If_Result);
--# expr-done 6013
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6014
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1399;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1399;
                     Finalizer_Scope_2694;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Unit_Dependencies_Helper;
--# end
--# property-start CompilationUnit.unit_dependencies /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20210
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Dependencies
  (Node : Bare_Compilation_Unit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
     Ent : Internal_Entity_Compilation_Unit :=
       Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1400 with Inline_Always;
      Singleton : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Cast_Result : Bare_Compilation_Unit;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1400 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1400;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Unit_Dependencies,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6021 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6020 '<Filter at ast.py:20222>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20222
   --# expr-start 6016 '<unique at ast.py:20222>' Unique_Array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20222
--# expr-start 6015 '<FieldAccess for unit_dependencies_helper at ast.py:20219>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20219
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
                Singleton := Create_Internal_Entity_Compilation_Unit_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
--# property-call-start CompilationUnit.unit_dependencies_helper
Fld := Libadalang.Implementation.Compilation_Unit_P_Unit_Dependencies_Helper (Node => Self, Visited => No_Internal_Entity_Compilation_Unit_Array_Type, To_Visit => Singleton);
--# end
--# expr-done 6015
Unique_Array := Make_Unique (Fld); 
--# expr-done 6016
      declare
         Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Unique_Array;
      begin
         for Item_205 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_205
         --# expr-start 6019 '<Not at ast.py:20224>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20224
--# expr-start 6018 '<Eq at ast.py:20224>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20224
--# expr-start 6017 '<FieldAccess for node at ast.py:20224>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20224
Fld_1 := Item_205.Node;
--# expr-done 6017
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Self; 
--# expr-done 6018
Not_Val := not (Is_Equal); 
--# expr-done 6019
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit := Item_205;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Compilation_Unit_Array
           (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6020
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6021
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1400;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1400;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                        As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Unit_Dependencies;
--# end
--# property-start CompilationUnit.decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20227
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Decl
  (Node : Bare_Compilation_Unit
  )
   return Bare_Basic_Decl
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Decl;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Basic_Decl;
Fld_2 : Bare_Body_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Cast_Result_1 : Bare_Subunit;
Cast_Result_2 : Bare_Basic_Decl;
Let_Result_1 : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_2 : Bare_Basic_Decl;
Scope_Result_2 : Bare_Basic_Decl;
Match_Result : Bare_Basic_Decl;
Let_Result_3 : Bare_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6026 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 6025 '<Match at ast.py:20233>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20233
--# expr-start 6022 '<FieldAccess for body at ast.py:20233>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20233
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 6022
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind li Match_0
--# expr-start 6023 '<FieldAccess for item at ast.py:20234>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20234
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_0.Library_Item_F_Item;
--# expr-done 6023
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 6024 '<FieldAccess for body at ast.py:20235>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20235
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Match_1.Subunit_F_Body;
--# expr-done 6024
      Cast_Result_2 := Fld_2;
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 6025
Let_Result_3 := Match_Result; 
--# end
--# expr-done 6026
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Decl;
--# end
--# property-start CompilationUnit.is_preelaborable_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20239
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Preelaborable_Impl
  (Node : Bare_Compilation_Unit
      ; From_Body : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
     Ent : Internal_Entity_Compilation_Unit :=
       Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind from_body From_Body
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Subunit;
Match_1 : Internal_Entity_Library_Item;
Match_2 : Internal_Entity;
Match_0_1 : Internal_Entity_Subp_Decl;
Match_1_1 : Internal_Entity_Generic_Subp_Decl;
Match_2_1 : Internal_Entity_Subp_Body;
Match_3 : Internal_Entity_Package_Decl;
Match_4 : Internal_Entity_Generic_Package_Decl;
Match_5 : Internal_Entity_Package_Body;
Match_6 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
Fld_9 : Boolean;
Fld_10 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity;
Fld_12 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Compilation_Unit;
Fld_13 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_14 : Boolean;
Fld_15 : Internal_Entity_Generic_Package_Internal;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Basic_Decl;
Fld_18 : Internal_Unit;
Fld_19 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Compilation_Unit;
Fld_20 : Boolean;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Subp_Decl;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_4 : Internal_Entity_Generic_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_5 : Internal_Entity_Subp_Body;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_6 : Internal_Entity_Package_Decl;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_7 : Internal_Entity_Generic_Package_Decl;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_8 : Internal_Entity_Package_Body;
Let_Result_5 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_9 : Internal_Entity_Basic_Decl;
Let_Result_6 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
Match_Prefix_1 : Internal_Entity;
Cast_Result_10 : Internal_Entity_Subunit;
Let_Result_7 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_11 : Internal_Entity_Library_Item;
Let_Result_8 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_12 : Internal_Entity;
Let_Result_9 : Boolean;
Scope_Result_10 : Boolean;
Match_Result_1 : Boolean;
Let_Result_10 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6054 '<Block at ???>' Let_Result_10 None
--# scope-start
--# expr-start 6053 '<Match at ast.py:20249>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20249
--# expr-start 6027 '<FieldAccess for body at ast.py:20249>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20249
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Compilation_Unit_F_Body, Info => Ent.Info);
--# expr-done 6027
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Node.Kind) is
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Subunit
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_10; 
--# bind su Match_0
--# expr-start 6032 '<FieldAccess for is_preelaborable_impl at ast.py:20254>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20254
--# expr-start 6031 '<Cast to CompilationUnit at ast.py:20253>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20253
--# expr-start 6030 '<parent at ast.py:20253>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20253
--# expr-start 6029 '<parent at ast.py:20253>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20253
--# expr-start 6028 '<FieldAccess for body_root at ast.py:20253>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20253
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.body_root
Fld_1 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Match_0.Node);
--# end
--# expr-done 6028
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 6029
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 6030
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6031
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 6032
Let_Result_7 := Fld_4; 
--# end
Scope_Result_8 := Let_Result_7;
   --# end
            Match_Result_1 := Scope_Result_8; 
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Library_Item
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_11; 
--# bind li Match_1
--# expr-start 6052 '<Match at ast.py:20257>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20257
--# expr-start 6033 '<FieldAccess for item at ast.py:20257>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20257
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Basic_Decl (Node => Match_1.Node.Library_Item_F_Item, Info => Match_1.Info);
--# expr-done 6033
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_5; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_3; 
--# bind subp_decl Match_0_1
--# expr-start 6034 '<FieldAccess for does_aspects_make_preelaborable at ast.py:20259>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20259
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_0_1.Node, From_Body => From_Body, E_Info => Match_0_1.Info);
--# end
--# expr-done 6034
Let_Result := Fld_6; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_4; 
--# bind gen_subp_decl Match_1_1
--# expr-start 6036 '<FieldAccess for does_aspects_make_preelaborable at ast.py:20263>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20263
--# expr-start 6035 '<FieldAccess for decl at ast.py:20262>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20262
      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Match_1_1.Node, E_Info => Match_1_1.Info);
--# end
--# expr-done 6035
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Fld_7.Node, From_Body => From_Body, E_Info => Fld_7.Info);
--# end
--# expr-done 6036
Let_Result_1 := Fld_8; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_5; 
--# bind subp_body Match_2_1
--# expr-start 6037 '<FieldAccess for does_aspects_make_preelaborable at ast.py:20268>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20268
      if Match_2_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_2_1.Node, From_Body => False, E_Info => Match_2_1.Info);
--# end
--# expr-done 6037
if Fld_9 then
   If_Result := True;
else
--# expr-start 6038 '<FieldAccess for decl_part at ast.py:20272>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20272
      if Match_2_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_10 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_2_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_2_1.Info);
--# end
--# expr-done 6038
Var_Expr := Fld_10;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr
--# expr-start 6042 '<FieldAccess for is_preelaborable_impl at ast.py:20275>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20275
--# expr-start 6041 '<Cast to CompilationUnit at ast.py:20274>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20274
--# expr-start 6040 '<parent at ast.py:20273>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20273
--# expr-start 6039 '<parent at ast.py:20273>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20273
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 6039
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Parent (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# expr-done 6040
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6041
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_13 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result_1.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 6042
Scope_Result := Fld_13;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result_2 := If_Result; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind pkg_decl Match_3
--# expr-start 6043 '<FieldAccess for does_aspects_make_preelaborable at ast.py:20279>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20279
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_3.Node, From_Body => From_Body, E_Info => Match_3.Info);
--# end
--# expr-done 6043
Let_Result_3 := Fld_14; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind gen_pkg_decl Match_4
--# expr-start 6045 '<FieldAccess for does_aspects_make_preelaborable at ast.py:20283>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20283
--# expr-start 6044 '<FieldAccess for package_decl at ast.py:20282>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20282
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_4.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_4.Info);
--# expr-done 6044
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Fld_15.Node, From_Body => From_Body, E_Info => Fld_15.Info);
--# end
--# expr-done 6045
Let_Result_4 := Fld_16; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_8; 
--# bind pkg_body Match_5
--# expr-start 6051 '<FieldAccess for is_preelaborable_impl at ast.py:20290>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20290
--# expr-start 6050 '<Cast to CompilationUnit at ast.py:20289>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20289
--# expr-start 6049 '<as_bare_entity at ast.py:20288>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20288
--# expr-start 6048 '<FieldAccess for root at ast.py:20288>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20288
--# expr-start 6047 '<FieldAccess for unit at ast.py:20288>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20288
--# expr-start 6046 '<FieldAccess for decl_part at ast.py:20288>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20288
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_17 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_5.Info);
--# end
--# expr-done 6046
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Unit (Node => Fld_17.Node);
--# expr-done 6047
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Fld_18.Ast_Root;
--# expr-done 6048
Is_Null := Fld_19 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_19); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 6049
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Compilation_Unit
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6050
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_20 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result_2.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 6051
Let_Result_5 := Fld_20; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Abstract_State_Decl .. Ada_Generic_Package_Internal | Ada_Discrete_Base_Subtype_Decl .. Ada_Concrete_Formal_Subp_Decl | Ada_Entry_Decl .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_9; 
--# bind _ Match_6
Let_Result_6 := False; 
--# end
Scope_Result_7 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 6052
Let_Result_8 := Match_Result; 
--# end
Scope_Result_9 := Let_Result_8;
   --# end
            Match_Result_1 := Scope_Result_9; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_12; 
--# bind _ Match_2
Let_Result_9 := False; 
--# end
Scope_Result_10 := Let_Result_9;
   --# end
            Match_Result_1 := Scope_Result_10; 
end case;
--# expr-done 6053
Let_Result_10 := Match_Result_1; 
--# end
--# expr-done 6054
         Property_Result := Let_Result_10;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Is_Preelaborable_Impl;
--# end
--# property-start CompilationUnit.is_preelaborable /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20299
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Preelaborable
  (Node : Bare_Compilation_Unit
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
     Ent : Internal_Entity_Compilation_Unit :=
       Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6056 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6055 '<FieldAccess for is_preelaborable_impl at ast.py:20305>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20305
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Ent.Node, From_Body => False, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6055
Let_Result := Fld; 
--# end
--# expr-done 6056
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Is_Preelaborable;
--# end
--# property-start CompilationUnit.self_restrictions /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20307
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Self_Restrictions
  (Node : Bare_Compilation_Unit
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1404 with Inline_Always;
      Var_Expr : Internal_Entity_Pragma_Node;
Var_Expr_1 : Internal_Entity_Base_Assoc;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld_3 : Internal_Entity_Base_Assoc_List;
Fld_4 : Bare_Base_Assoc_List;
Fld_5 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Assoc;
Fld_6 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Base_Id;
Fld_7 : Symbol_Type;
Item_206 : Internal_Entity;
Bare_Item_206 : Bare_Ada_Node;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Pragma_Node;
Fld_8 : Internal_Entity_Identifier;
Fld_9 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1404 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1404;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6070 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6069 '<FilterMap at ast.py:20314>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20314
   --# expr-start 6058 '<FieldAccess for prelude at ast.py:20314>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20314
--# expr-start 6057 '<as_bare_entity at ast.py:20314>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20314
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 6057
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => If_Result.Node.Compilation_Unit_F_Prelude, Info => If_Result.Info);
--# expr-done 6058
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Symbol_Type_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_206 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_206 := Untyped_Item_206; 
Is_Null_1 := Bare_Item_206 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity_1 := (Info => Fld_2, Node => Bare_Item_206); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_206 := If_Result_1; 
   --# scope-start
               --# bind n Item_206
--# expr-start 6059 '<Cast to Pragma at ast.py:20318>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20318
Cast_Expr_1 := Item_206; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Pragma_Node;
   end if;
--# expr-done 6059
Var_Expr := Cast_Result_2;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 6063 '<Eq at ast.py:20319>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20319
--# expr-start 6061 '<FieldAccess for name_symbol at ast.py:20319>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20319
--# expr-start 6060 '<FieldAccess for id at ast.py:20319>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20319
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Identifier (Node => Var_Expr.Node.Pragma_Node_F_Id, Info => Var_Expr.Info);
--# expr-done 6060
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_8.Node);
--# end
--# expr-done 6061
--# expr-start 6062 '<Symbol Restrictions at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Restrictions); 
--# expr-done 6062
Is_Equal := Fld_9 = Sym; 
--# expr-done 6063
Scope_Result_1 := Is_Equal;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
         if Result_Var_1 then
      --# expr-start 6068 '<FieldAccess for name_symbol at ast.py:20316>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20316
--# expr-start 6067 '<Cast to BaseId at ast.py:20316>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20316
--# expr-start 6065 '<FieldAccess for args at ast.py:20316>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20316
--# expr-start 6064 '<Cast to Pragma at ast.py:20316>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20316
Cast_Expr := Item_206; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 6064
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Base_Assoc_List (Node => Cast_Result.Node.Pragma_Node_F_Args, Info => Cast_Result.Info);
--# expr-done 6065
Fld_4 := Fld_3.Node;
Get_Result := Get (Self, Fld_4, 0, True); 
Is_Null_2 := Get_Result = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Base_Assoc;
else
Fld_5 := Fld_3.Info;
As_Entity_2 := (Info => Fld_5, Node => Get_Result); 
   If_Result_2 := As_Entity_2;
end if;
Var_Expr_1 := If_Result_2;
if Var_Expr_1 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 6066 '<FieldAccess for assoc_expr at ast.py:20316>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20316
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6066
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 6067
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result_1.Node);
--# end
--# expr-done 6068
         declare
            Item_To_Append : constant Symbol_Type := Fld_7;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6069
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6070
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1404;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1404;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Self_Restrictions;
--# end
--# property-start CompilationUnit.other_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20323
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Other_Part
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Compilation_Unit
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Compilation_Unit;
            procedure Finalizer_Scope_1405 with Inline_Always;
      Block_Var : Analysis_Unit_Kind;
Var_Expr : Bare_Compilation_Unit;
Fld : Analysis_Unit_Kind;
Is_Equal : Boolean;
If_Result : Analysis_Unit_Kind;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Compilation_Unit;
Scope_Result : Internal_Entity_Compilation_Unit;
Result_Var : Internal_Entity_Compilation_Unit;
Let_Result : Internal_Entity_Compilation_Unit;
            procedure Finalizer_Scope_1405 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1405;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6077 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6073 '<If at ast.py:20329>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20329
--# expr-start 6072 '<Eq at ast.py:20330>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20330
--# expr-start 6071 '<FieldAccess for unit_kind at ast.py:20330>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20330
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.unit_kind
Fld := Libadalang.Implementation.Compilation_Unit_P_Unit_Kind (Node => Self);
--# end
--# expr-done 6071
Is_Equal := Fld = Unit_Specification; 
--# expr-done 6072
if Is_Equal then
   If_Result := Unit_Body;
else
   If_Result := Unit_Specification;
end if;
--# expr-done 6073
Block_Var := If_Result; 
--# bind other_kind Block_Var
--# expr-start 6075 '<FieldAccess for designated_compilation_unit at ast.py:20334>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20334
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6074 '<FieldAccess for syntactic_fully_qualified_name at ast.py:20335>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20335
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 6074
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld_1, Kind => Block_Var, Not_Found_Is_Error => False);
--# end
--# expr-done 6075
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6076 '<as_bare_entity at ast.py:20338>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20338
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result_1 := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 6076
Scope_Result := If_Result_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Compilation_Unit;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 6077
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1405;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1405;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Other_Part;
--# end
--# property-start CompilationUnit.has_restriction /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20340
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Has_Restriction
  (Node : Bare_Compilation_Unit
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
            procedure Finalizer_Scope_2716 with Inline_Always;
            procedure Finalizer_Scope_2718 with Inline_Always;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Var_Expr : Internal_Entity_Compilation_Unit;
Var_Expr_1 : Internal_Entity_Compilation_Unit;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld : Bare_Ada_Node;
Fld_1 : Symbol_Type_Array_Access;
Is_Equal_1 : Boolean;
Quantifier_Result : Boolean;
Fld_2 : Internal_Entity_Compilation_Unit;
Fld_3 : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Is_Equal_2 : Boolean;
Quantifier_Result_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Compilation_Unit;
Fld_5 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_1 : Bare_Subunit;
Let_Result_1 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Boolean;
Scope_Result_4 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Let_Result_3 : Boolean;
            procedure Finalizer_Scope_2716 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2716;
            procedure Finalizer_Scope_2718 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2718;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6092 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 6091 '<Cond at ast.py:20350>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20350
--# expr-start 6079 '<Eq at ast.py:20351>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20351
--# expr-start 6078 '<Symbol No_Elaboration_Code at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_No_Elaboration_Code); 
--# expr-done 6078
Is_Equal := Name = Sym; 
--# expr-done 6079
if Is_Equal then
   --# expr-start 6090 '<Match at ast.py:20353>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20353
--# expr-start 6080 '<FieldAccess for body at ast.py:20353>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20353
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 6080
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 6083 '<Contains at ast.py:20357>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20357
   --# expr-start 6081 '<FieldAccess for self_restrictions at ast.py:20357>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20357
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.self_restrictions
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Self_Restrictions (Node => Self);
--# end
--# expr-done 6081
      Quantifier_Result := False;
      declare
         Collection : constant Symbol_Type_Array_Access := Fld_1;
      begin
         for Item_207 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_207
      --# expr-start 6082 '<Eq at ???>' Is_Equal_1 None
Is_Equal_1 := Item_207 = Name; 
--# expr-done 6082
      Quantifier_Result := Is_Equal_1;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6083
if Quantifier_Result then
   If_Result := True;
else
   --# expr-start 6087 '<Contains at ast.py:20358>' Quantifier_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20358
--# expr-start 6084 '<FieldAccess for other_part at ast.py:20358>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20358
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.other_part
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Other_Part (Node => Self);
--# end
--# expr-done 6084
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Compilation_Unit then
   --# scope-start
--# expr-start 6085 '<FieldAccess for self_restrictions at ast.py:20358>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20358
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.self_restrictions
Fld_3 := Libadalang.Implementation.Compilation_Unit_P_Self_Restrictions (Node => Var_Expr.Node);
--# end
--# expr-done 6085
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2718;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Quantifier_Result_1 := False;
      declare
         Collection : constant Symbol_Type_Array_Access := Result_Var;
      begin
         for Item_208 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_208
      --# expr-start 6086 '<Eq at ???>' Is_Equal_2 None
Is_Equal_2 := Item_208 = Name; 
--# expr-done 6086
      Quantifier_Result_1 := Is_Equal_2;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 6087
   If_Result := Quantifier_Result_1;
end if;
Let_Result := If_Result; 
--# end
Scope_Result_2 := Let_Result;
   --# end
      Finalizer_Scope_2716;
            Match_Result := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 6088 '<FieldAccess for root_unit at ast.py:20362>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20362
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld_4 := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Match_1);
--# end
--# expr-done 6088
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Compilation_Unit then
   --# scope-start
--# expr-start 6089 '<FieldAccess for has_restriction at ast.py:20362>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20362
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.has_restriction
Fld_5 := Libadalang.Implementation.Compilation_Unit_P_Has_Restriction (Node => Var_Expr_1.Node, Name => Name);
--# end
--# expr-done 6089
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Let_Result_1 := Result_Var_1; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 6090
   If_Result_1 := Match_Result;
else
   If_Result_1 := raise Property_Error with "Unsupported restriction";
end if;
--# expr-done 6091
Let_Result_3 := If_Result_1; 
--# end
--# expr-done 6092
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2716;
                     Finalizer_Scope_2718;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Has_Restriction;
--# end
--# property-start CompilationUnit.is_text_io_child /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20372
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Text_Io_Child
  (Node : Bare_Compilation_Unit
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1407 with Inline_Always;
      Block_Var : Symbol_Type_Array_Access;
Fld : Symbol_Type_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Get_Result : Symbol_Type;
Sym : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Get_Result_1 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_2 : Boolean;
Get_Result_2 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Get_Result_3 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1407 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_1407;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6108 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6093 '<FieldAccess for syntactic_fully_qualified_name at ast.py:20378>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20378
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 6093
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind name_parts Block_Var
--# expr-start 6095 '<Eq at ast.py:20380>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20380
--# expr-start 6094 '<length at ast.py:20380>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20380
Len := Length (Block_Var); 
--# expr-done 6094
Is_Equal := Len = 3; 
--# expr-done 6095
if Is_Equal then
   --# expr-start 6098 '<Eq at ast.py:20381>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20381
--# expr-start 6096 '<at at ast.py:20381>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20381
Get_Result := Get (Self, Block_Var, 0, True); 
--# expr-done 6096
--# expr-start 6097 '<Symbol ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada_291); 
--# expr-done 6097
Is_Equal_1 := Get_Result = Sym; 
--# expr-done 6098
   If_Result := Is_Equal_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 6101 '<Eq at ast.py:20382>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20382
--# expr-start 6099 '<at at ast.py:20382>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20382
Get_Result_1 := Get (Self, Block_Var, 1, True); 
--# expr-done 6099
--# expr-start 6100 '<Symbol text_io at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Text_Io); 
--# expr-done 6100
Is_Equal_2 := Get_Result_1 = Sym_1; 
--# expr-done 6101
if Is_Equal_2 then
   If_Result_1 := True;
else
   --# expr-start 6104 '<Eq at ast.py:20382>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20382
--# expr-start 6102 '<at at ast.py:20382>' Get_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20382
Get_Result_2 := Get (Self, Block_Var, 1, True); 
--# expr-done 6102
--# expr-start 6103 '<Symbol wide_text_io at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Text_Io); 
--# expr-done 6103
Is_Equal_3 := Get_Result_2 = Sym_2; 
--# expr-done 6104
   If_Result_1 := Is_Equal_3;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 6107 '<Eq at ast.py:20382>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20382
--# expr-start 6105 '<at at ast.py:20382>' Get_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20382
Get_Result_3 := Get (Self, Block_Var, 1, True); 
--# expr-done 6105
--# expr-start 6106 '<Symbol wide_wide_text_io at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Text_Io); 
--# expr-done 6106
Is_Equal_4 := Get_Result_3 = Sym_3; 
--# expr-done 6107
   If_Result_2 := Is_Equal_4;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 6108
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1407;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1407;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Is_Text_Io_Child;
--# end
--# property-start CompilationUnit.local_config_pragmas /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20400
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Local_Config_Pragmas
  (Node : Bare_Compilation_Unit
  )
   return Bare_Pragma_Node_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Bare_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1409 with Inline_Always;
      Fld : Bare_Ada_Node_List;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Pragma_Node;
Item_209 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Pragma_Node_Array_Access;
Let_Result : Bare_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1409 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1409;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6113 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6112 '<FilterMap at ast.py:20406>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20406
   --# expr-start 6109 '<FieldAccess for prelude at ast.py:20406>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20406
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Compilation_Unit_F_Prelude;
--# expr-done 6109
      declare
         Map_Result_Vec : Bare_Pragma_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Pragma_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld;
      begin
         for Untyped_Item_209 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_209 := Untyped_Item_209; 
   --# scope-start
               --# bind n Item_209
         --# expr-start 6110 '<IsA Pragma at ast.py:20408>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20408
Is_A := Item_209 /= null 
and then Item_209.Kind in Ada_Pragma_Node_Range; 
--# expr-done 6110
         if Is_A then
      --# expr-start 6111 '<Cast to Pragma at ast.py:20407>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20407
Cast_Expr := Item_209; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6111
         declare
            Item_To_Append : constant Bare_Pragma_Node := Cast_Result;
         begin
            Bare_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Pragma_Node_Array
           (Items_Count => Natural (Bare_Pragma_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Pragma_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6112
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6113
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1409;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1409;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Local_Config_Pragmas;
--# end
--# property-start CompilationUnit.sources_config_pragmas /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20411
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Sources_Config_Pragmas
  (Node : Bare_Compilation_Unit
      ; Include_Other_Part : Boolean
         := True
  )
   return Bare_Pragma_Node_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind include_other_part Include_Other_Part
   Property_Result : Bare_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1410 with Inline_Always;
            procedure Finalizer_Scope_2724 with Inline_Always;
            procedure Finalizer_Scope_2725 with Inline_Always;
      Block_Var : Bare_Pragma_Node_Array_Access;
Block_Var_1 : Bare_Pragma_Node_Array_Access;
Var_Expr : Internal_Entity_Compilation_Unit;
Var_Expr_1 : Internal_Entity_Compilation_Unit;
Fld : Bare_Pragma_Node_Array_Access;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Boolean;
Fld_2 : Internal_Entity_Compilation_Unit;
Fld_3 : Bare_Pragma_Node_Array_Access;
Scope_Result : Bare_Pragma_Node_Array_Access;
Result_Var : Bare_Pragma_Node_Array_Access;
Fld_4 : Bare_Ada_Node;
Is_A_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Fld_6 : Internal_Entity_Compilation_Unit;
Fld_7 : Bare_Pragma_Node_Array_Access;
Scope_Result_1 : Bare_Pragma_Node_Array_Access;
Result_Var_1 : Bare_Pragma_Node_Array_Access;
If_Result_1 : Bare_Pragma_Node_Array_Access;
If_Result_2 : Bare_Pragma_Node_Array_Access;
Concat_Result : Bare_Pragma_Node_Array_Access;
Let_Result : Bare_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1410 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1410;
            procedure Finalizer_Scope_2724 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2724;
            procedure Finalizer_Scope_2725 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2725;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6127 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6114 '<FieldAccess for local_config_pragmas at ast.py:20418>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20418
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.local_config_pragmas
Fld := Libadalang.Implementation.Compilation_Unit_P_Local_Config_Pragmas (Node => Self);
--# end
--# expr-done 6114
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind current_unit Block_Var
--# expr-start 6125 '<Cond at ast.py:20422>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20422
if Include_Other_Part then
   --# expr-start 6116 '<IsA LibraryItem at ast.py:20425>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20425
--# expr-start 6115 '<FieldAccess for body at ast.py:20425>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20425
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Compilation_Unit_F_Body;
--# expr-done 6115
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Library_Item_Range; 
--# expr-done 6116
   If_Result := Is_A;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 6117 '<FieldAccess for other_part at ast.py:20426>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20426
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.other_part
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Other_Part (Node => Self);
--# end
--# expr-done 6117
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Compilation_Unit then
   --# scope-start
--# expr-start 6118 '<FieldAccess for local_config_pragmas at ast.py:20426>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20426
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.local_config_pragmas
Fld_3 := Libadalang.Implementation.Compilation_Unit_P_Local_Config_Pragmas (Node => Var_Expr.Node);
--# end
--# expr-done 6118
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2724;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
   If_Result_2 := Result_Var;
else
--# expr-start 6120 '<IsA Subunit at ast.py:20430>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20430
--# expr-start 6119 '<FieldAccess for body at ast.py:20430>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20430
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Compilation_Unit_F_Body;
--# expr-done 6119
Is_A_1 := Fld_4 /= null 
and then Fld_4.Kind in Ada_Subunit_Range; 
--# expr-done 6120
if Is_A_1 then
--# expr-start 6123 '<FieldAccess for root_unit at ast.py:20431>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20431
--# expr-start 6122 '<Cast to Subunit at ast.py:20431>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20431
--# expr-start 6121 '<FieldAccess for body at ast.py:20431>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20431
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Compilation_Unit_F_Body;
--# expr-done 6121
   if Fld_5 = null
      or else Fld_5.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld_5;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6122
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld_6 := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Cast_Result);
--# end
--# expr-done 6123
Var_Expr_1 := Fld_6;
if Var_Expr_1 /= No_Entity_Compilation_Unit then
   --# scope-start
--# expr-start 6124 '<FieldAccess for sources_config_pragmas at ast.py:20432>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20432
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.sources_config_pragmas
Fld_7 := Libadalang.Implementation.Compilation_Unit_P_Sources_Config_Pragmas (Node => Var_Expr_1.Node, Include_Other_Part => Include_Other_Part);
--# end
--# expr-done 6124
Scope_Result_1 := Fld_7;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2725;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 6125
Block_Var_1 := If_Result_2; Inc_Ref (Block_Var_1);
--# bind related_units Block_Var_1
--# expr-start 6126 '<Concat at ast.py:20438>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20438
Concat_Result := Concat (Block_Var, Block_Var_1); 
--# expr-done 6126
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6127
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1410;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1410;
                     Finalizer_Scope_2724;
                     Finalizer_Scope_2725;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Sources_Config_Pragmas;
--# end
--# property-start CompilationUnit.all_config_pragmas /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20440
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_All_Config_Pragmas
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Pragma_Node_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1411 with Inline_Always;
      Fld : Bare_Pragma_Node_Array_Access;
Fld_1 : Bare_Pragma_Node_Array_Access;
Concat_Result : Bare_Pragma_Node_Array_Access;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Map_Result : Internal_Entity_Pragma_Node_Array_Access;
Let_Result : Internal_Entity_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1411 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1411;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6133 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6132 '<Map at ast.py:20448>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20448
   --# expr-start 6130 '<Concat at ast.py:20447>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20447
--# expr-start 6128 '<FieldAccess for sources_config_pragmas at ast.py:20447>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20447
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.sources_config_pragmas
Fld := Libadalang.Implementation.Compilation_Unit_P_Sources_Config_Pragmas (Node => Self);
--# end
--# expr-done 6128
--# expr-start 6129 '<FieldAccess for external_config_pragmas at ast.py:20447>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20447
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Compilation_Unit_P_External_Config_Pragmas (Node => Self);
--# expr-done 6129
Concat_Result := Concat (Fld, Fld_1); 
--# expr-done 6130
      declare
         Map_Result_Vec : Internal_Entity_Pragma_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Pragma_Node_Array_Access := Concat_Result;
      begin
         for Item_210 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_210
      --# expr-start 6131 '<as_bare_entity at ast.py:20448>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20448
Is_Null := Item_210 = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Item_210); 
   If_Result := As_Entity;
end if;
--# expr-done 6131
         declare
            Item_To_Append : constant Internal_Entity_Pragma_Node := If_Result;
         begin
            Internal_Entity_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Pragma_Node_Array
           (Items_Count => Natural (Internal_Entity_Pragma_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Pragma_Node_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6132
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6133
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1411;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1411;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_All_Config_Pragmas;
--# end
--# property-start CompilationUnit.spark_config_pragma /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20451
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Spark_Config_Pragma
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1412 with Inline_Always;
      Fld : Bare_Pragma_Node_Array_Access;
Fld_1 : Bare_Pragma_Node_Array_Access;
Concat_Result : Bare_Pragma_Node_Array_Access;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Fld_2 : Bare_Identifier;
Sym : Symbol_Type;
Fld_3 : Boolean;
Map_Result : Internal_Entity_Pragma_Node_Array_Access;
Get_Result : Internal_Entity_Pragma_Node;
Let_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1412 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1412;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6143 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6142 '<at at ast.py:20463>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20463
--# expr-start 6141 '<FilterMap at ast.py:20460>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20460
   --# expr-start 6136 '<Concat at ast.py:20458>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20458
--# expr-start 6134 '<FieldAccess for external_config_pragmas at ast.py:20458>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20458
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Compilation_Unit_P_External_Config_Pragmas (Node => Self);
--# expr-done 6134
--# expr-start 6135 '<FieldAccess for sources_config_pragmas at ast.py:20459>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20459
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.sources_config_pragmas
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Sources_Config_Pragmas (Node => Self, Include_Other_Part => False);
--# end
--# expr-done 6135
Concat_Result := Concat (Fld, Fld_1); 
--# expr-done 6136
      declare
         Map_Result_Vec : Internal_Entity_Pragma_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Pragma_Node_Array_Access := Concat_Result;
      begin
         for Item_211 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_211
         --# expr-start 6139 '<FieldAccess for name_is at ast.py:20462>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20462
--# expr-start 6137 '<FieldAccess for id at ast.py:20462>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20462
      if Item_211 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Item_211.Pragma_Node_F_Id;
--# expr-done 6137
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6138 '<Symbol SPARK_Mode at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 6138
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_2, Sym => Sym);
--# end
--# expr-done 6139
         if Fld_3 then
      --# expr-start 6140 '<as_bare_entity at ast.py:20461>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20461
Is_Null := Item_211 = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Item_211); 
   If_Result := As_Entity;
end if;
--# expr-done 6140
         declare
            Item_To_Append : constant Internal_Entity_Pragma_Node := If_Result;
         begin
            Internal_Entity_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Pragma_Node_Array
           (Items_Count => Natural (Internal_Entity_Pragma_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Pragma_Node_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6141
Get_Result := Get (Self, Map_Result, -1, True); 
--# expr-done 6142
Let_Result := Get_Result; 
--# end
--# expr-done 6143
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1412;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1412;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Spark_Config_Pragma;
--# end
--# property-start CompilationUnit.config_pragmas /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20466
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Config_Pragmas
  (Node : Bare_Compilation_Unit
      ; Name : Symbol_Type
  )
   return Internal_Entity_Pragma_Node_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Internal_Entity_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1413 with Inline_Always;
      Fld : Internal_Entity_Pragma_Node_Array_Access;
Fld_1 : Internal_Entity_Identifier;
Fld_2 : Symbol_Type;
Is_Equal : Boolean;
Map_Result : Internal_Entity_Pragma_Node_Array_Access;
Let_Result : Internal_Entity_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1413 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1413;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6149 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6148 '<Filter at ast.py:20472>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20472
   --# expr-start 6144 '<FieldAccess for all_config_pragmas at ast.py:20472>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20472
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.all_config_pragmas
Fld := Libadalang.Implementation.Compilation_Unit_P_All_Config_Pragmas (Node => Self);
--# end
--# expr-done 6144
      declare
         Map_Result_Vec : Internal_Entity_Pragma_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Pragma_Node_Array_Access := Fld;
      begin
         for Item_212 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_212
         --# expr-start 6147 '<Eq at ast.py:20473>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20473
--# expr-start 6146 '<FieldAccess for name_symbol at ast.py:20473>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20473
--# expr-start 6145 '<FieldAccess for id at ast.py:20473>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20473
      if Item_212.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Identifier (Node => Item_212.Node.Pragma_Node_F_Id, Info => Item_212.Info);
--# expr-done 6145
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 6146
Is_Equal := Fld_2 = Name; 
--# expr-done 6147
         if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Pragma_Node := Item_212;
         begin
            Internal_Entity_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Pragma_Node_Array
           (Items_Count => Natural (Internal_Entity_Pragma_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Pragma_Node_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6148
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6149
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1413;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1413;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Config_Pragmas;
--# end
--# property-start CompilationUnit.stub_for /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20476
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Stub_For
  (Node : Bare_Compilation_Unit
      ; Su : Bare_Subunit
  )
   return Bare_Body_Stub
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind su Su
   Property_Result : Bare_Body_Stub;
      Fld : Bare_Body_Stub;
Let_Result : Bare_Body_Stub;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Stub_For,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Subunit,
                                      As_Bare_Subunit => Su);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Bare_Body_Stub;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6151 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6150 '<FieldAccess for stub_for_impl at ast.py:20528>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20528
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Compilation_Unit_P_Stub_For_Impl (Node => Self, Su => Su);
--# expr-done 6150
Let_Result := Fld; 
--# end
--# expr-done 6151
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     Add_Memoized_Error
                       (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
               end if;
            raise;
      end;
      if Self /= null then
            Mmz_Val := (Kind => Mmz_Bare_Body_Stub,
                        As_Bare_Body_Stub => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   return Property_Result;
end Compilation_Unit_P_Stub_For;
--# end
   --
   --  Primitives for Bare_Component_Clause
   --
      procedure Initialize_Fields_For_Component_Clause
        (Self : Bare_Component_Clause
         ; Component_Clause_F_Id : Bare_Identifier
         ; Component_Clause_F_Position : Bare_Expr
         ; Component_Clause_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Component_Clause_F_Id := Component_Clause_F_Id;
            Self.Component_Clause_F_Position := Component_Clause_F_Position;
            Self.Component_Clause_F_Range := Component_Clause_F_Range;
      end Initialize_Fields_For_Component_Clause;
   function Component_Clause_F_Id
     (Node : Bare_Component_Clause) return Bare_Identifier
   is
   begin
         return Node.Component_Clause_F_Id;
   end;
   function Component_Clause_F_Position
     (Node : Bare_Component_Clause) return Bare_Expr
   is
   begin
         return Node.Component_Clause_F_Position;
   end;
   function Component_Clause_F_Range
     (Node : Bare_Component_Clause) return Bare_Range_Spec
   is
   begin
         return Node.Component_Clause_F_Range;
   end;
--# property-start ComponentClause.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10404
pragma Warnings (Off, "is not referenced");
 function Component_Clause_P_Xref_Entry_Point
  (Node : Bare_Component_Clause
  )
   return Boolean
is
   Self : Bare_Component_Clause  := Bare_Component_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Clause_P_Xref_Entry_Point;
--# end
--# property-start ComponentClause.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10406
pragma Warnings (Off, "is not referenced");
 function Component_Clause_P_Xref_Equation
  (Node : Bare_Component_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Clause  := Bare_Component_Clause (Node);
     Ent : Internal_Entity_Component_Clause :=
       Internal_Entity_Component_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_808 with Inline_Always;
      Block_Var : Internal_Entity_Record_Rep_Clause;
Block_Var_1 : Lexical_Env := Empty_Env;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Record_Rep_Clause;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Range_Spec;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_808 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_808;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6166 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6154 '<Cast to RecordRepClause at ast.py:10410>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10410
--# expr-start 6153 '<parent at ast.py:10410>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10410
--# expr-start 6152 '<parent at ast.py:10410>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10410
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6152
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 6153
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Record_Rep_Clause
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6154
Block_Var := Cast_Result; 
--# bind rep_clause Block_Var
--# expr-start 6156 '<FieldAccess for designated_env_no_overloading at ast.py:10414>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10414
--# expr-start 6155 '<FieldAccess for name at ast.py:10414>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10414
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Block_Var.Node.Record_Rep_Clause_F_Name, Info => Block_Var.Info);
--# expr-done 6155
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 6156
Block_Var_1 := Fld_3; Inc_Ref (Block_Var_1);
--# bind record_env Block_Var_1
--# expr-start 6165 '<And at ast.py:10416>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10416
--# expr-start 6162 '<And at ast.py:10416>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10416
--# expr-start 6159 '<bind at ast.py:10418>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10418
Bound_Env := Block_Var_1; Inc_Ref (Bound_Env);
--# expr-start 6158 '<FieldAccess for xref_equation at ast.py:10418>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10418
--# expr-start 6157 '<FieldAccess for id at ast.py:10418>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10418
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Component_Clause_F_Id, Info => Ent.Info);
--# expr-done 6157
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 6158
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6159
--# expr-start 6161 '<FieldAccess for sub_equation at ast.py:10419>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10419
--# expr-start 6160 '<FieldAccess for position at ast.py:10419>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10419
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Component_Clause_F_Position, Info => Ent.Info);
--# expr-done 6160
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6161
And_Pred := Create_And (Dyn_Var_Bind_Result, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10416") else null)); 
--# expr-done 6162
--# expr-start 6164 '<FieldAccess for sub_equation at ast.py:10420>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10420
--# expr-start 6163 '<FieldAccess for range at ast.py:10420>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10420
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Component_Clause_F_Range, Info => Ent.Info);
--# expr-done 6163
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 6164
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10416") else null)); 
--# expr-done 6165
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6166
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_808;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_808;
            raise;
      end;
   return Property_Result;
end Component_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Component_Def
   --
      procedure Initialize_Fields_For_Component_Def
        (Self : Bare_Component_Def
         ; Component_Def_F_Has_Aliased : Bare_Aliased_Node
         ; Component_Def_F_Has_Constant : Bare_Constant_Node
         ; Component_Def_F_Type_Expr : Bare_Type_Expr
        ) is
      begin
            Self.Component_Def_F_Has_Aliased := Component_Def_F_Has_Aliased;
            Self.Component_Def_F_Has_Constant := Component_Def_F_Has_Constant;
            Self.Component_Def_F_Type_Expr := Component_Def_F_Type_Expr;
      end Initialize_Fields_For_Component_Def;
   function Component_Def_F_Has_Aliased
     (Node : Bare_Component_Def) return Bare_Aliased_Node
   is
   begin
         return Node.Component_Def_F_Has_Aliased;
   end;
   function Component_Def_F_Has_Constant
     (Node : Bare_Component_Def) return Bare_Constant_Node
   is
   begin
         return Node.Component_Def_F_Has_Constant;
   end;
   function Component_Def_F_Type_Expr
     (Node : Bare_Component_Def) return Bare_Type_Expr
   is
   begin
         return Node.Component_Def_F_Type_Expr;
   end;
--# property-start ComponentDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8805
pragma Warnings (Off, "is not referenced");
 function Component_Def_P_Xref_Equation
  (Node : Bare_Component_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Def  := Bare_Component_Def (Node);
     Ent : Internal_Entity_Component_Def :=
       Internal_Entity_Component_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_663 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_663 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_663;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6169 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6168 '<FieldAccess for sub_equation at ast.py:8807>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8807
--# expr-start 6167 '<FieldAccess for type_expr at ast.py:8807>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8807
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Component_Def_F_Type_Expr, Info => Ent.Info);
--# expr-done 6167
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6168
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6169
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_663;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_663;
            raise;
      end;
   return Property_Result;
end Component_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Constant_Node
   --
--# property-start '[dispatcher]Constant.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Constant_Node_P_As_Bool
  (Node : Bare_Constant_Node
  )
   return Boolean
is
   Self : Bare_Constant_Node  := Bare_Constant_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Constant_Node (Self.Kind) is
               when Ada_Constant_Absent =>
                  --# property-call-start Constant.Absent.as_bool
                  Property_Result := Constant_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Constant_Present =>
                  --# property-call-start Constant.Present.as_bool
                  Property_Result := Constant_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Constant_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constant_Absent
   --
--# property-start Constant.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5300
pragma Warnings (Off, "is not referenced");
 function Constant_Absent_P_As_Bool
  (Node : Bare_Constant_Absent
  )
   return Boolean
is
   Self : Bare_Constant_Absent  := Bare_Constant_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constant_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constant_Present
   --
--# property-start Constant.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5300
pragma Warnings (Off, "is not referenced");
 function Constant_Present_P_As_Bool
  (Node : Bare_Constant_Present
  )
   return Boolean
is
   Self : Bare_Constant_Present  := Bare_Constant_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constant_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constraint
   --
--# property-start Constraint.subtype /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8304
pragma Warnings (Off, "is not referenced");
 function Constraint_P_Subtype
  (Node : Bare_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Constraint  := Bare_Constraint (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Subtype_Indication;
As_Entity : Internal_Entity_Subtype_Indication;
Is_Null : Boolean;
If_Result : Internal_Entity_Subtype_Indication;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6175 '<bind at ast.py:8304>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8304
--# expr-start 6170 '<FieldAccess for origin_node at ast.py:8305>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8305
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6170
Bound_Origin := Fld; 
--# expr-start 6174 '<FieldAccess for designated_type at ast.py:8307>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8307
--# expr-start 6173 '<as_entity at ast.py:8307>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8307
--# expr-start 6172 '<Cast to SubtypeIndication at ast.py:8306>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8306
--# expr-start 6171 '<parent at ast.py:8306>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8306
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 6171
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6172
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Subtype_Indication;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 6173
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => If_Result.Node, Origin => Bound_Origin, E_Info => If_Result.Info);
--# end
--# expr-done 6174
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 6175
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constraint_P_Subtype;
--# end
--# property-start Constraint.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8310
pragma Warnings (Off, "is not referenced");
 function Constraint_P_Is_Static
  (Node : Bare_Constraint
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Constraint  := Bare_Constraint (Node);
     Ent : Internal_Entity_Constraint :=
       Internal_Entity_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Range_Constraint;
Match_1 : Internal_Entity_Composite_Constraint;
Match_2 : Internal_Entity_Digits_Constraint;
Match_3 : Internal_Entity_Delta_Constraint;
Match_0_1 : Internal_Entity_Subtype_Indication;
Match_1_1 : Internal_Entity_Expr;
Match_2_1 : Internal_Entity;
Var_Expr : Internal_Entity_Range_Spec;
Var_Expr_1 : Internal_Entity_Range_Spec;
Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Bare_Assoc_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Cast_Expr : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity_Composite_Constraint_Assoc;
Fld_6 : Internal_Entity;
Fld_7 : Boolean;
Fld_8 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Item_213 : Internal_Entity_Basic_Assoc;
Bare_Item_213 : Bare_Basic_Assoc;
Quantifier_Result : Boolean;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Bare_Assoc_List;
Fld_11 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Boolean;
Item_214 : Internal_Entity_Basic_Assoc;
Bare_Item_214 : Bare_Basic_Assoc;
Quantifier_Result_1 : Boolean;
Fld_14 : Boolean;
If_Result_2 : Boolean;
Fld_15 : Internal_Entity_Range_Spec;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Boolean;
Scope_Result_3 : Boolean;
Result_Var : Boolean;
Fld_18 : Internal_Entity_Range_Spec;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_1 : Boolean;
Match_Prefix_1 : Internal_Entity_Constraint;
Cast_Result_4 : Internal_Entity_Range_Constraint;
Let_Result_3 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_5 : Internal_Entity_Composite_Constraint;
Let_Result_4 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_6 : Internal_Entity_Digits_Constraint;
Let_Result_5 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_7 : Internal_Entity_Delta_Constraint;
Let_Result_6 : Boolean;
Scope_Result_8 : Boolean;
Match_Result_1 : Boolean;
Let_Result_7 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6199 '<Block at ???>' Let_Result_7 None
--# scope-start
--# expr-start 6198 '<Match at ast.py:8312>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8312
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Ent; 
case Ada_Constraint (Match_Prefix_1.Node.Kind) is
         when Ada_Range_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Range_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_4; 
--# bind rc Match_0
--# expr-start 6178 '<FieldAccess for is_static_expr at ast.py:8313>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8313
--# expr-start 6177 '<FieldAccess for range at ast.py:8313>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8313
--# expr-start 6176 '<FieldAccess for range at ast.py:8313>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8313
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Match_0.Node.Range_Constraint_F_Range, Info => Match_0.Info);
--# expr-done 6176
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 6177
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 6178
Let_Result_3 := Fld_2; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_5; 
         when Ada_Composite_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Composite_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_5; 
--# bind cc Match_1
--# expr-start 6191 '<If at ast.py:8314>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8314
--# expr-start 6179 '<FieldAccess for is_index_constraint at ast.py:8315>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8315
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompositeConstraint.is_index_constraint
Fld_14 := Libadalang.Implementation.Composite_Constraint_P_Is_Index_Constraint (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6179
if Fld_14 then
   --# expr-start 6186 '<AllQuantifier at ast.py:8316>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8316
   --# expr-start 6180 '<FieldAccess for constraints at ast.py:8316>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8316
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Assoc_List (Node => Match_1.Node.Composite_Constraint_F_Constraints, Info => Match_1.Info);
--# expr-done 6180
Fld_4 := Fld_3.Node;
      Quantifier_Result := True;
      if Fld_4 = null then
      null;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_4;
      begin
         for Untyped_Item_213 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_213 := Untyped_Item_213; 
Is_Null := Bare_Item_213 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_213); 
   If_Result := As_Entity;
end if;
                  Item_213 := If_Result; 
   --# scope-start
               --# bind c Item_213
      --# expr-start 6185 '<Match at ast.py:8318>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8318
--# expr-start 6182 '<FieldAccess for constraint_expr at ast.py:8317>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8317
--# expr-start 6181 '<Cast to CompositeConstraintAssoc at ast.py:8317>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8317
Cast_Expr := Item_213; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Composite_Constraint_Assoc_Range
   then
      Cast_Result := Create_Internal_Entity_Composite_Constraint_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Composite_Constraint_Assoc;
   end if;
--# expr-done 6181
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity (Node => Cast_Result.Node.Composite_Constraint_Assoc_F_Constraint_Expr, Info => Cast_Result.Info);
--# expr-done 6182
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_6; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_1; 
--# bind st Match_0_1
--# expr-start 6183 '<FieldAccess for is_static_subtype at ast.py:8319>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8319
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_7 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Match_0_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0_1.Info);
--# end
--# expr-done 6183
Let_Result := Fld_7; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_2; 
--# bind e Match_1_1
--# expr-start 6184 '<FieldAccess for is_static_expr at ast.py:8320>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8320
      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_8 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Match_1_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1_1.Info);
--# end
--# expr-done 6184
Let_Result_1 := Fld_8; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_3; 
--# bind _ Match_2_1
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 6185
      Quantifier_Result := Match_Result;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 6186
   If_Result_2 := Quantifier_Result;
else
   --# expr-start 6190 '<AllQuantifier at ast.py:8324>' Quantifier_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8324
   --# expr-start 6187 '<FieldAccess for constraints at ast.py:8324>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8324
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Assoc_List (Node => Match_1.Node.Composite_Constraint_F_Constraints, Info => Match_1.Info);
--# expr-done 6187
Fld_10 := Fld_9.Node;
      Quantifier_Result_1 := True;
      if Fld_10 = null then
      null;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_10;
      begin
         for Untyped_Item_214 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_214 := Untyped_Item_214; 
Is_Null_1 := Bare_Item_214 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_11 := Fld_9.Info;
As_Entity_1 := (Info => Fld_11, Node => Bare_Item_214); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_214 := If_Result_1; 
   --# scope-start
               --# bind c Item_214
      --# expr-start 6189 '<FieldAccess for is_static_expr at ast.py:8325>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8325
--# expr-start 6188 '<FieldAccess for expr at ast.py:8325>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8325
      if Item_214.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_214.Node, E_Info => Item_214.Info);
--# end
--# expr-done 6188
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_13 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_12.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_12.Info);
--# end
--# expr-done 6189
      Quantifier_Result_1 := Fld_13;
   --# end
               exit when not Quantifier_Result_1;
         end loop;
      end;
      end if;
--# expr-done 6190
   If_Result_2 := Quantifier_Result_1;
end if;
--# expr-done 6191
Let_Result_4 := If_Result_2; 
--# end
Scope_Result_6 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_6; 
         when Ada_Digits_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Digits_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_6; 
--# bind dc Match_2
--# expr-start 6192 '<FieldAccess for range at ast.py:8328>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8328
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Range_Spec (Node => Match_2.Node.Digits_Constraint_F_Range, Info => Match_2.Info);
--# expr-done 6192
Var_Expr := Fld_15;
if Var_Expr /= No_Entity_Range_Spec then
   --# scope-start
   --# bind range Var_Expr
--# expr-start 6194 '<FieldAccess for is_static_expr at ast.py:8329>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8329
--# expr-start 6193 '<FieldAccess for range at ast.py:8329>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8329
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 6193
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_17 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_16.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_16.Info);
--# end
--# expr-done 6194
Scope_Result_3 := Fld_17;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := True;
end if;
Let_Result_5 := Result_Var; 
--# end
Scope_Result_7 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_7; 
         when Ada_Delta_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Delta_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_7; 
--# bind dc Match_3
--# expr-start 6195 '<FieldAccess for range at ast.py:8332>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8332
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Range_Spec (Node => Match_3.Node.Delta_Constraint_F_Range, Info => Match_3.Info);
--# expr-done 6195
Var_Expr_1 := Fld_18;
if Var_Expr_1 /= No_Entity_Range_Spec then
   --# scope-start
   --# bind range Var_Expr_1
--# expr-start 6197 '<FieldAccess for is_static_expr at ast.py:8333>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8333
--# expr-start 6196 '<FieldAccess for range at ast.py:8333>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8333
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Var_Expr_1.Node.Range_Spec_F_Range, Info => Var_Expr_1.Info);
--# expr-done 6196
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_20 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_19.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_19.Info);
--# end
--# expr-done 6197
Scope_Result_4 := Fld_20;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := True;
end if;
Let_Result_6 := Result_Var_1; 
--# end
Scope_Result_8 := Let_Result_6;
   --# end
            Match_Result_1 := Scope_Result_8; 
end case;
--# expr-done 6198
Let_Result_7 := Match_Result_1; 
--# end
--# expr-done 6199
         Property_Result := Let_Result_7;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constraint_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Composite_Constraint
   --
      procedure Initialize_Fields_For_Composite_Constraint
        (Self : Bare_Composite_Constraint
         ; Composite_Constraint_F_Constraints : Bare_Assoc_List
        ) is
      begin
            Self.Composite_Constraint_F_Constraints := Composite_Constraint_F_Constraints;
      end Initialize_Fields_For_Composite_Constraint;
   function Composite_Constraint_F_Constraints
     (Node : Bare_Composite_Constraint) return Bare_Assoc_List
   is
   begin
         return Node.Composite_Constraint_F_Constraints;
   end;
--# property-start CompositeConstraint.is_index_constraint /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8395
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Is_Index_Constraint
  (Node : Bare_Composite_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6207 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6201 '<FieldAccess for is_array_type at ast.py:8401>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8401
--# expr-start 6200 '<FieldAccess for subtype at ast.py:8401>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8401
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6200
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_array_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Array_Type (Node => Fld.Node, Origin => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 6201
if Fld_1 then
   If_Result_1 := True;
else
--# expr-start 6203 '<FieldAccess for is_access_type at ast.py:8402>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8402
--# expr-start 6202 '<FieldAccess for subtype at ast.py:8402>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8402
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_2 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6202
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld_2.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6203
if Fld_3 then
   --# expr-start 6206 '<FieldAccess for is_array_type at ast.py:8403>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8403
--# expr-start 6205 '<FieldAccess for accessed_type at ast.py:8403>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8403
--# expr-start 6204 '<FieldAccess for subtype at ast.py:8403>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8403
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_4 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6204
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_4.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_4.Info);
--# end
--# expr-done 6205
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_array_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_Array_Type (Node => Fld_5.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_5.Info);
--# end
--# expr-done 6206
   If_Result := Fld_6;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 6207
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Is_Index_Constraint;
--# end
--# property-start CompositeConstraint.complete_item_weight /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8406
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Complete_Item_Weight
  (Node : Bare_Composite_Constraint
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
            procedure Finalizer_Scope_614 with Inline_Always;
      Var_Expr : Internal_Entity_Enum_Literal_Decl;
Fld : Integer;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Enum_Literal_Decl;
Fld_4 : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
If_Result : Integer;
Let_Result : Integer;
            procedure Finalizer_Scope_614 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_614;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6217 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6216 '<If at ast.py:8410>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8410
--# expr-start 6214 '<AnyQuantifier at ast.py:8411>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8411
   --# expr-start 6209 '<FieldAccess for discriminants_list at ast.py:8411>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8411
--# expr-start 6208 '<FieldAccess for subtype at ast.py:8411>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8411
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_1 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6208
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 6209
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Fld_2;
      begin
         for Item_215 of
               Collection.Items
         loop
   --# scope-start
               --# bind td Item_215
      --# expr-start 6213 '<Eq at ast.py:8413>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8413
--# expr-start 6210 '<FieldAccess for formal_type at ast.py:8413>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8413
      if Item_215.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Item_215.Node, Origin => Origin, E_Info => Item_215.Info);
--# end
--# expr-done 6210
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 6211 '<Cast to EnumLiteralDecl at ast.py:8413>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8413
Cast_Expr := Item; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 6211
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Enum_Literal_Decl then
   --# scope-start
--# expr-start 6212 '<FieldAccess for enum_type at ast.py:8413>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8413
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6212
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Decl;
end if;
      Cast_Result_2 := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 6213
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6214
if Quantifier_Result then
   If_Result := 100;
else
   --# expr-start 6215 '<Super at ast.py:8416>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8416
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.complete_item_weight
Fld := Libadalang.Implementation.Ada_Node_P_Complete_Item_Weight (Node => Ent.Node, Item => Item, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6215
   If_Result := Fld;
end if;
--# expr-done 6216
Let_Result := If_Result; 
--# end
--# expr-done 6217
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_614;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_614;
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Complete_Item_Weight;
--# end
--# property-start CompositeConstraint.is_discriminant_constraint /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8419
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Is_Discriminant_Constraint
  (Node : Bare_Composite_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6220 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6219 '<Not at ast.py:8424>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8424
--# expr-start 6218 '<FieldAccess for is_index_constraint at ast.py:8424>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8424
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompositeConstraint.is_index_constraint
Fld := Libadalang.Implementation.Composite_Constraint_P_Is_Index_Constraint (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6218
Not_Val := not (Fld); 
--# expr-done 6219
Let_Result := Not_Val; 
--# end
--# expr-done 6220
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Is_Discriminant_Constraint;
--# end
--# property-start CompositeConstraint.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8426
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Xref_Equation
  (Node : Bare_Composite_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_616 with Inline_Always;
            procedure Finalizer_Scope_2742 with Inline_Always;
            procedure Finalizer_Scope_2743 with Inline_Always;
            procedure Finalizer_Scope_2744 with Inline_Always;
            procedure Finalizer_Scope_2745 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Ex : Internal_Entity;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Bare_Base_Id;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Cast_Expr : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity_Composite_Constraint_Assoc;
Fld_4 : Internal_Entity;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_8 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
Item_216 : Internal_Entity_Basic_Assoc;
Bare_Item_216 : Bare_Basic_Assoc;
Index_217 : Integer;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_10 : Internal_Entity_Assoc_List;
Fld_11 : Internal_Param_Match_Array_Access;
Fld_12 : Internal_Single_Actual;
Fld_13 : Internal_Entity_Basic_Assoc;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Logic_Var;
Fld_16 : Internal_Entity_Defining_Name;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl;
Fld_18 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_19 : Internal_Single_Actual;
Fld_20 : Internal_Entity_Basic_Assoc;
Fld_21 : Internal_Entity_Expr;
Fld_22 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_23 : Internal_Single_Actual;
Fld_24 : Internal_Entity_Basic_Assoc;
Fld_25 : Internal_Entity_Expr;
Fld_26 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_27 : Internal_Single_Actual;
Fld_28 : Bare_Base_Id;
Fld_29 : Logic_Var;
Fld_30 : Internal_Entity_Defining_Name;
Fld_31 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_32 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_616 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_616;
            procedure Finalizer_Scope_2742 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2742;
            procedure Finalizer_Scope_2743 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_2743;
            procedure Finalizer_Scope_2744 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_26);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2744;
            procedure Finalizer_Scope_2745 is
            begin
                     Dec_Ref (Bind_Result_2);
            end Finalizer_Scope_2745;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6267 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 6221 '<FieldAccess for subtype at ast.py:8428>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8428
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6221
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 6266 '<If at ast.py:8429>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8429
--# expr-start 6222 '<FieldAccess for is_index_constraint at ast.py:8430>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8430
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompositeConstraint.is_index_constraint
Fld_32 := Libadalang.Implementation.Composite_Constraint_P_Is_Index_Constraint (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6222
if Fld_32 then
   --# expr-start 6235 '<LogicAll at ast.py:8431>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8431
--# expr-start 6234 '<Map at ast.py:8431>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8431
   --# expr-start 6223 '<FieldAccess for constraints at ast.py:8431>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8431
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Composite_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 6223
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
         Index_217 := 0;
      declare
         Collection : constant Bare_Assoc_List := Fld_2;
      begin
         for Untyped_Item_216 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_216 := Untyped_Item_216; 
Is_Null := Bare_Item_216 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_216); 
   If_Result := As_Entity;
end if;
                  Item_216 := If_Result; 
   --# scope-start
               --# bind c Item_216
               --# bind i Index_217
      --# expr-start 6233 '<Let at ast.py:8431>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8431
--# scope-start
--# expr-start 6225 '<FieldAccess for constraint_expr at ast.py:8432>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8432
--# expr-start 6224 '<Cast to CompositeConstraintAssoc at ast.py:8432>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8432
Cast_Expr := Item_216; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Composite_Constraint_Assoc_Range
   then
      Cast_Result := Create_Internal_Entity_Composite_Constraint_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Composite_Constraint_Assoc;
   end if;
--# expr-done 6224
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity (Node => Cast_Result.Node.Composite_Constraint_Assoc_F_Constraint_Expr, Info => Cast_Result.Info);
--# expr-done 6225
Ex := Fld_4; 
--# bind ex Ex
--# expr-start 6232 '<And at ast.py:8452>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8452
--# expr-start 6226 '<Cast to Expr at ast.py:8452>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8452
Cast_Expr_1 := Ex; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Expr
   then
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Expr;
   end if;
--# expr-done 6226
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 6230 '<Bind at ast.py:8454>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8454
--# expr-start 6227 '<FieldAccess for expected_type_var at ast.py:8454>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8454
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 6227
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 6229 '<FieldAccess for base_subtype at ast.py:8454>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8454
--# expr-start 6228 '<FieldAccess for index_type at ast.py:8454>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8454
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Block_Var.Node, Dim => Index_217, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6228
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6229
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8454") else null)); 
--# expr-done 6230
Scope_Result := Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2743;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8455") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 6231 '<FieldAccess for sub_equation at ast.py:8456>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8456
      if Ex.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Ex.Node, Env => Env, Origin => Origin, E_Info => Ex.Info);
--# end
--# expr-done 6231
And_Pred := Create_And (Result_Var, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8452") else null)); 
--# expr-done 6232
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6233
         declare
            Item_To_Append : constant Logic_Equation := Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2742;
               Index_217 := Index_217 + 1;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6234
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8431") else null)); 
--# expr-done 6235
   If_Result_1 := Logic_Boolean_Op;
else
   --# expr-start 6265 '<LogicAll at ast.py:8462>' Logic_Boolean_Op_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8462
--# expr-start 6264 '<Map at ast.py:8462>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8462
   --# expr-start 6238 '<FieldAccess for match_formals at ast.py:8460>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8460
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6236 '<FieldAccess for discriminants_list at ast.py:8461>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8461
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6236
--# expr-start 6237 '<FieldAccess for constraints at ast.py:8461>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8461
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Composite_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 6237
--# property-call-start AdaNode.match_formals
Fld_11 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld_9, Params => Fld_10, Is_Dottable_Subp => False);
--# end
--# expr-done 6238
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_11;
      begin
         for Item_218 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_218
      --# expr-start 6263 '<And at ast.py:8464>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8464
--# expr-start 6256 '<And at ast.py:8464>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8464
--# expr-start 6251 '<And at ast.py:8464>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8464
--# expr-start 6246 '<Bind at ast.py:8464>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8464
--# expr-start 6242 '<FieldAccess for expected_type_var at ast.py:8464>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8464
--# expr-start 6241 '<FieldAccess for expr at ast.py:8464>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8464
--# expr-start 6240 '<FieldAccess for assoc at ast.py:8464>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8464
--# expr-start 6239 '<FieldAccess for actual at ast.py:8464>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8464
Fld_12 := Item_218.Actual;
--# expr-done 6239
Fld_13 := Fld_12.Assoc;
--# expr-done 6240
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 6241
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_14.Node);
--# expr-done 6242
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
--# expr-start 6245 '<FieldAccess for formal_type at ast.py:8465>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8465
--# expr-start 6244 '<FieldAccess for formal_decl at ast.py:8465>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8465
--# expr-start 6243 '<FieldAccess for formal at ast.py:8465>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8465
Fld_16 := Item_218.Formal;
--# expr-done 6243
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_17 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 6244
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_18 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_17.Node, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 6245
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_15, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8464") else null)); 
--# expr-done 6246
--# expr-start 6250 '<FieldAccess for sub_equation at ast.py:8466>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8466
--# expr-start 6249 '<FieldAccess for expr at ast.py:8466>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8466
--# expr-start 6248 '<FieldAccess for assoc at ast.py:8466>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8466
--# expr-start 6247 '<FieldAccess for actual at ast.py:8466>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8466
Fld_19 := Item_218.Actual;
--# expr-done 6247
Fld_20 := Fld_19.Assoc;
--# expr-done 6248
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_21 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_20.Node, E_Info => Fld_20.Info);
--# end
--# expr-done 6249
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_21.Node, Env => Env, Origin => Origin, E_Info => Fld_21.Info);
--# end
--# expr-done 6250
And_Pred_1 := Create_And (Bind_Result_1, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8464") else null)); 
--# expr-done 6251
--# expr-start 6255 '<FieldAccess for matches_expected_formal_type at ast.py:8467>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8467
--# expr-start 6254 '<FieldAccess for expr at ast.py:8467>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8467
--# expr-start 6253 '<FieldAccess for assoc at ast.py:8467>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8467
--# expr-start 6252 '<FieldAccess for actual at ast.py:8467>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8467
Fld_23 := Item_218.Actual;
--# expr-done 6252
Fld_24 := Fld_23.Assoc;
--# expr-done 6253
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_25 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_24.Node, E_Info => Fld_24.Info);
--# end
--# expr-done 6254
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_26 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_25.Node, Origin => Origin);
--# end
--# expr-done 6255
And_Pred_2 := Create_And (And_Pred_1, Fld_26, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8464") else null)); 
--# expr-done 6256
--# expr-start 6258 '<FieldAccess for name at ast.py:8468>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8468
--# expr-start 6257 '<FieldAccess for actual at ast.py:8468>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8468
Fld_27 := Item_218.Actual;
--# expr-done 6257
Fld_28 := Fld_27.Name;
--# expr-done 6258
Var_Expr_1 := Fld_28;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 6262 '<Bind at ast.py:8469>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8469
--# expr-start 6259 '<FieldAccess for ref_var at ast.py:8469>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8469
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr_1);
--# end
--# expr-done 6259
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 6261 '<FieldAccess for formal_decl at ast.py:8469>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8469
--# expr-start 6260 '<FieldAccess for formal at ast.py:8469>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8469
Fld_30 := Item_218.Formal;
--# expr-done 6260
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_31 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_30.Node, E_Info => Fld_30.Info);
--# end
--# expr-done 6261
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_29, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8469") else null)); 
--# expr-done 6262
Scope_Result_1 := Bind_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2745;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8470") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_3 := Create_And (And_Pred_2, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8464") else null)); 
--# expr-done 6263
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2744;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 6264
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8462") else null)); 
--# expr-done 6265
   If_Result_1 := Logic_Boolean_Op_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6266
Let_Result_1 := If_Result_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6267
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_616;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_616;
                     Finalizer_Scope_2742;
                     Finalizer_Scope_2743;
                     Finalizer_Scope_2744;
                     Finalizer_Scope_2745;
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Delta_Constraint
   --
      procedure Initialize_Fields_For_Delta_Constraint
        (Self : Bare_Delta_Constraint
         ; Delta_Constraint_F_Digits : Bare_Expr
         ; Delta_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Delta_Constraint_F_Digits := Delta_Constraint_F_Digits;
            Self.Delta_Constraint_F_Range := Delta_Constraint_F_Range;
      end Initialize_Fields_For_Delta_Constraint;
   function Delta_Constraint_F_Digits
     (Node : Bare_Delta_Constraint) return Bare_Expr
   is
   begin
         return Node.Delta_Constraint_F_Digits;
   end;
   function Delta_Constraint_F_Range
     (Node : Bare_Delta_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Delta_Constraint_F_Range;
   end;
--# property-start DeltaConstraint.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8377
pragma Warnings (Off, "is not referenced");
 function Delta_Constraint_P_Xref_Equation
  (Node : Bare_Delta_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Delta_Constraint  := Bare_Delta_Constraint (Node);
     Ent : Internal_Entity_Delta_Constraint :=
       Internal_Entity_Delta_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_612 with Inline_Always;
            procedure Finalizer_Scope_2746 with Inline_Always;
      Var_Expr : Internal_Entity_Range_Spec;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Range_Spec;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_612 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_612;
            procedure Finalizer_Scope_2746 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2746;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6272 '<And at ast.py:8378>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8378
--# expr-start 6269 '<FieldAccess for sub_equation at ast.py:8378>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8378
--# expr-start 6268 '<FieldAccess for digits at ast.py:8378>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8378
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Delta_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 6268
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6269
--# expr-start 6270 '<FieldAccess for range at ast.py:8378>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8378
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Delta_Constraint_F_Range, Info => Ent.Info);
--# expr-done 6270
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Range_Spec then
   --# scope-start
   --# bind range Var_Expr
--# expr-start 6271 '<FieldAccess for sub_equation at ast.py:8379>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8379
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6271
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2746;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8380") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8378") else null)); 
--# expr-done 6272
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_612;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_612;
                     Finalizer_Scope_2746;
            raise;
      end;
   return Property_Result;
end Delta_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Digits_Constraint
   --
      procedure Initialize_Fields_For_Digits_Constraint
        (Self : Bare_Digits_Constraint
         ; Digits_Constraint_F_Digits : Bare_Expr
         ; Digits_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Digits_Constraint_F_Digits := Digits_Constraint_F_Digits;
            Self.Digits_Constraint_F_Range := Digits_Constraint_F_Range;
      end Initialize_Fields_For_Digits_Constraint;
   function Digits_Constraint_F_Digits
     (Node : Bare_Digits_Constraint) return Bare_Expr
   is
   begin
         return Node.Digits_Constraint_F_Digits;
   end;
   function Digits_Constraint_F_Range
     (Node : Bare_Digits_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Digits_Constraint_F_Range;
   end;
--# property-start DigitsConstraint.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8362
pragma Warnings (Off, "is not referenced");
 function Digits_Constraint_P_Xref_Equation
  (Node : Bare_Digits_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Digits_Constraint  := Bare_Digits_Constraint (Node);
     Ent : Internal_Entity_Digits_Constraint :=
       Internal_Entity_Digits_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_611 with Inline_Always;
            procedure Finalizer_Scope_2747 with Inline_Always;
      Var_Expr : Internal_Entity_Range_Spec;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Range_Spec;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_611 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_611;
            procedure Finalizer_Scope_2747 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2747;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6277 '<And at ast.py:8363>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8363
--# expr-start 6274 '<FieldAccess for sub_equation at ast.py:8363>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8363
--# expr-start 6273 '<FieldAccess for digits at ast.py:8363>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8363
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Digits_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 6273
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6274
--# expr-start 6275 '<FieldAccess for range at ast.py:8363>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8363
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Digits_Constraint_F_Range, Info => Ent.Info);
--# expr-done 6275
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Range_Spec then
   --# scope-start
   --# bind range Var_Expr
--# expr-start 6276 '<FieldAccess for sub_equation at ast.py:8364>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8364
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6276
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2747;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8365") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8363") else null)); 
--# expr-done 6277
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_611;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_611;
                     Finalizer_Scope_2747;
            raise;
      end;
   return Property_Result;
end Digits_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Range_Constraint
   --
      procedure Initialize_Fields_For_Range_Constraint
        (Self : Bare_Range_Constraint
         ; Range_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Range_Constraint_F_Range := Range_Constraint_F_Range;
      end Initialize_Fields_For_Range_Constraint;
   function Range_Constraint_F_Range
     (Node : Bare_Range_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Range_Constraint_F_Range;
   end;
--# property-start RangeConstraint.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8345
pragma Warnings (Off, "is not referenced");
 function Range_Constraint_P_Xref_Equation
  (Node : Bare_Range_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Range_Constraint  := Bare_Range_Constraint (Node);
     Ent : Internal_Entity_Range_Constraint :=
       Internal_Entity_Range_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_610 with Inline_Always;
      Fld : Bare_Range_Spec;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Internal_Entity_Range_Spec;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Internal_Entity_Range_Spec;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_610 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_610;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6291 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6290 '<And at ast.py:8347>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8347
--# expr-start 6286 '<And at ast.py:8347>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8347
--# expr-start 6283 '<Bind at ast.py:8348>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8348
--# expr-start 6280 '<FieldAccess for expected_type_var at ast.py:8348>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8348
--# expr-start 6279 '<FieldAccess for range at ast.py:8348>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8348
--# expr-start 6278 '<FieldAccess for range at ast.py:8348>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8348
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Range_Constraint_F_Range;
--# expr-done 6278
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Range_Spec_F_Range;
--# expr-done 6279
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_1);
--# expr-done 6280
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 6282 '<FieldAccess for base_subtype at ast.py:8349>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8349
--# expr-start 6281 '<FieldAccess for subtype at ast.py:8349>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8349
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_3 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6281
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 6282
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8348") else null)); 
--# expr-done 6283
--# expr-start 6285 '<FieldAccess for sub_equation at ast.py:8350>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8350
--# expr-start 6284 '<FieldAccess for range at ast.py:8350>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8350
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Range_Constraint_F_Range, Info => Ent.Info);
--# expr-done 6284
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 6285
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8347") else null)); 
--# expr-done 6286
--# expr-start 6289 '<FieldAccess for matches_expected_type at ast.py:8351>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8351
--# expr-start 6288 '<FieldAccess for range at ast.py:8351>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8351
--# expr-start 6287 '<FieldAccess for range at ast.py:8351>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8351
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Range_Constraint_F_Range, Info => Ent.Info);
--# expr-done 6287
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Fld_7.Node.Range_Spec_F_Range, Info => Fld_7.Info);
--# expr-done 6288
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 6289
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8347") else null)); 
--# expr-done 6290
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6291
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_610;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_610;
            raise;
      end;
   return Property_Result;
end Range_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Declarative_Part
   --
      procedure Initialize_Fields_For_Declarative_Part
        (Self : Bare_Declarative_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Self.Declarative_Part_F_Decls := Declarative_Part_F_Decls;
      end Initialize_Fields_For_Declarative_Part;
   function Declarative_Part_F_Decls
     (Node : Bare_Declarative_Part) return Bare_Ada_Node_List
   is
   begin
         return Node.Declarative_Part_F_Decls;
   end;
--# property-start DeclarativePart.use_clauses_envs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10925
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Use_Clauses_Envs
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Declarative_Part  := Bare_Declarative_Part (Node);
     Ent : Internal_Entity_Declarative_Part :=
       Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_851 with Inline_Always;
            procedure Finalizer_Scope_2592 with Inline_Always;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Use_Clause;
Fld_2 : Lexical_Env := Empty_Env;
Is_A : Boolean;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_851 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_851;
            procedure Finalizer_Scope_2592 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2592;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6299 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6298 '<env_group at ast.py:10934>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10934
--# expr-start 6297 '<FilterMap at ast.py:10931>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10931
   --# expr-start 6293 '<children at ast.py:10931>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10931
--# expr-start 6292 '<FieldAccess for decls at ast.py:10931>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10931
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Declarative_Part_F_Decls, Info => Ent.Info);
--# expr-done 6292
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 6293
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_219 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_219
         --# expr-start 6294 '<IsA UseClause at ast.py:10933>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10933
Is_A := Item_219.Node /= null 
and then Item_219.Node.Kind in Ada_Use_Clause; 
--# expr-done 6294
         if Is_A then
      --# expr-start 6296 '<FieldAccess for used_envs at ast.py:10932>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10932
--# expr-start 6295 '<Cast to UseClause at ast.py:10932>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10932
Cast_Expr := Item_219; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Use_Clause
   then
      Cast_Result := Create_Internal_Entity_Use_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Use_Clause;
   end if;
--# expr-done 6295
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.used_envs
Fld_2 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 6296
         declare
            Item_To_Append : constant Lexical_Env := Fld_2;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
      Finalizer_Scope_2592;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6297
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 6298
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 6299
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_851;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_851;
                     Finalizer_Scope_2592;
            raise;
      end;
   return Property_Result;
end Declarative_Part_P_Use_Clauses_Envs;
--# end
--# property-start DeclarativePart.spark_mode_pragma /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10936
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Spark_Mode_Pragma
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Declarative_Part  := Bare_Declarative_Part (Node);
     Ent : Internal_Entity_Declarative_Part :=
       Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_852 with Inline_Always;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Item_221 : Internal_Entity;
Bare_Item_221 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld_3 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_4 : Boolean;
Find_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Pragma_Node;
Let_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_852 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_852;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6309 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6308 '<Cast to Pragma at ast.py:10948>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10948
--# expr-start 6307 '<Find at ast.py:10946>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10946
   --# expr-start 6302 '<TakeWhile at ast.py:10944>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10944
   --# expr-start 6300 '<FieldAccess for decls at ast.py:10944>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10944
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Declarative_Part_F_Decls, Info => Ent.Info);
--# expr-done 6300
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
            Take_While_Ok : Boolean := False;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_221 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_221 := Untyped_Item_221; 
Is_Null := Bare_Item_221 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_221); 
   If_Result := As_Entity;
end if;
                  Item_221 := If_Result; 
   --# scope-start
               --# bind decl Item_221
         --# expr-start 6301 '<IsA Pragma at ast.py:10945>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10945
Is_A := Item_221.Node /= null 
and then Item_221.Node.Kind in Ada_Pragma_Node_Range; 
--# expr-done 6301
         Take_While_Ok := Is_A;
         if Take_While_Ok then
         declare
            Item_To_Append : constant Internal_Entity := Item_221;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
               exit when not Take_While_Ok;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6302
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_220 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_220
      --# expr-start 6306 '<FieldAccess for name_is at ast.py:10947>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10947
--# expr-start 6304 '<FieldAccess for id at ast.py:10947>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10947
--# expr-start 6303 '<Cast to Pragma at ast.py:10947>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10947
Cast_Expr := Item_220; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 6303
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Identifier (Node => Cast_Result.Node.Pragma_Node_F_Id, Info => Cast_Result.Info);
--# expr-done 6304
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6305 '<Symbol SPARK_Mode at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 6305
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Sym);
--# end
--# expr-done 6306
      if Fld_4 then
         Find_Result := Item_220; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 6307
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Pragma_Node
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_1 := No_Entity_Pragma_Node;
   end if;
--# expr-done 6308
Let_Result := Cast_Result_1; 
--# end
--# expr-done 6309
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_852;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_852;
            raise;
      end;
   return Property_Result;
end Declarative_Part_P_Spark_Mode_Pragma;
--# end
--# property-start '[dispatcher]DeclarativePart.is_spark' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Declarative_Part_P_Is_Spark
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Declarative_Part  := Bare_Declarative_Part (Node);
     Ent : Internal_Entity_Declarative_Part :=
       Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Declarative_Part_Range (Self.Kind) is
               when Ada_Declarative_Part | Ada_Public_Part =>
                  --# property-call-start DeclarativePart.is_spark
                  Property_Result := Declarative_Part_P_Is_Spark
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Private_Part =>
                  --# property-call-start PrivatePart.is_spark
                  Property_Result := Private_Part_P_Is_Spark
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Declarative_Part_P_Is_Spark;
--# end
--# property-start DeclarativePart.parent_declarative_part_is_spark /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10975
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Parent_Declarative_Part_Is_Spark
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Declarative_Part  := Bare_Declarative_Part (Node);
     Ent : Internal_Entity_Declarative_Part :=
       Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_854 with Inline_Always;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Handled_Stmts;
Fld : Internal_Entity;
Fld_1 : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_2 : Boolean;
Scope_Result : Boolean;
Fld_3 : Internal_Entity;
Fld_4 : Internal_Entity_Array_Access;
Is_A : Boolean;
Fld_5 : Internal_Entity;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity;
Is_A_2 : Boolean;
If_Result_2 : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Handled_Stmts;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
Result_Var_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_854 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_854;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6325 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6311 '<FieldAccess for declarative_scope at ast.py:10982>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10982
--# expr-start 6310 '<parent at ast.py:10982>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10982
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6310
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_1 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Fld.Node);
--# end
--# expr-done 6311
Var_Expr := Fld_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind parent_scope Var_Expr
--# expr-start 6313 '<FieldAccess for is_spark at ast.py:10983>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10983
--# expr-start 6312 '<as_entity at ast.py:10983>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10983
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 6312
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]DeclarativePart.is_spark'
Fld_2 := Libadalang.Implementation.Dispatcher_Declarative_Part_P_Is_Spark (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6313
Scope_Result := Fld_2;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 6323 '<Cast to HandledStmts at ast.py:10992>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10992
--# expr-start 6322 '<Find at ast.py:10985>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10985
   --# expr-start 6315 '<FieldAccess for parents at ast.py:10985>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10985
--# expr-start 6314 '<parent at ast.py:10985>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10985
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6314
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parents (Node => Fld_3.Node, With_Self => True, E_Info => Fld_3.Info);
--# expr-done 6315
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_4;
      begin
         for Item_222 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_222
--# expr-start 6316 '<IsA HandledStmts at ast.py:10989>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10989
Is_A := Item_222.Node /= null 
and then Item_222.Node.Kind in Ada_Handled_Stmts_Range; 
--# expr-done 6316
if Is_A then
   --# expr-start 6318 '<IsA PackageBody at ast.py:10990>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10990
--# expr-start 6317 '<parent at ast.py:10990>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10990
      if Item_222.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Item_222.Node, E_Info => Item_222.Info);
--# expr-done 6317
Is_A_1 := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Package_Body_Range; 
--# expr-done 6318
   If_Result_1 := Is_A_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 6321 '<IsA LibraryItem at ast.py:10991>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10991
--# expr-start 6320 '<parent at ast.py:10991>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10991
--# expr-start 6319 '<parent at ast.py:10991>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10991
      if Item_222.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Parent (Node => Item_222.Node, E_Info => Item_222.Info);
--# expr-done 6319
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# expr-done 6320
Is_A_2 := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 6321
   If_Result_2 := Is_A_2;
else
   If_Result_2 := False;
end if;
      if If_Result_2 then
         Find_Result := Item_222; 
      end if;
   --# end
               exit when If_Result_2;
         end loop;
      end;
--# expr-done 6322
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Handled_Stmts_Range
   then
      Cast_Result := Create_Internal_Entity_Handled_Stmts
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result := No_Entity_Handled_Stmts;
   end if;
--# expr-done 6323
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Handled_Stmts then
   --# scope-start
--# expr-start 6324 '<FieldAccess for is_spark at ast.py:10992>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10992
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start HandledStmts.is_spark
Fld_8 := Libadalang.Implementation.Handled_Stmts_P_Is_Spark (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6324
Scope_Result_1 := Fld_8;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   Result_Var_1 := Result_Var;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 6325
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_854;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_854;
            raise;
      end;
   return Property_Result;
end Declarative_Part_P_Parent_Declarative_Part_Is_Spark;
--# end
--# property-start DeclarativePart.is_spark /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10950
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Is_Spark
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Declarative_Part  := Bare_Declarative_Part (Node);
     Ent : Internal_Entity_Declarative_Part :=
       Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Pragma_Node;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Aspect_Assoc;
Var_Expr_3 : Internal_Entity_Expr;
Fld : Internal_Entity_Pragma_Node;
Fld_1 : Boolean;
Scope_Result : Boolean;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Sym : Symbol_Type;
Fld_3 : Internal_Entity_Aspect_Assoc;
Fld_4 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_5 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
Scope_Result_2 : Boolean;
Fld_6 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_3 : Boolean;
Fld_7 : Boolean;
Result_Var_2 : Boolean;
Result_Var_3 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6338 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6326 '<FieldAccess for spark_mode_pragma at ast.py:10956>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10956
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.spark_mode_pragma
Fld := Libadalang.Implementation.Declarative_Part_P_Spark_Mode_Pragma (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6326
Var_Expr := Fld;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind pragma Var_Expr
--# expr-start 6327 '<FieldAccess for spark_mode_is_on at ast.py:10958>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10958
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.spark_mode_is_on
Fld_1 := Libadalang.Implementation.Pragma_Node_P_Spark_Mode_Is_On (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6327
Scope_Result := Fld_1;
   --# end
   Result_Var_3 := Scope_Result;
else
--# expr-start 6329 '<Cast to BasicDecl at ast.py:10960>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10960
--# expr-start 6328 '<parent at ast.py:10960>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10960
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6328
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 6329
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 6331 '<FieldAccess for get_aspect_assoc at ast.py:10961>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10961
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6330 '<Symbol SPARK_Mode at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 6330
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Var_Expr_1.Node, Name => Sym, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6331
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Aspect_Assoc then
   --# scope-start
   --# bind aa Var_Expr_2
--# expr-start 6332 '<FieldAccess for expr at ast.py:10962>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10962
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Var_Expr_2.Node.Aspect_Assoc_F_Expr, Info => Var_Expr_2.Info);
--# expr-done 6332
Var_Expr_3 := Fld_4;
if Var_Expr_3 /= No_Entity_Expr then
   --# scope-start
   --# bind mode Var_Expr_3
--# expr-start 6335 '<FieldAccess for name_is at ast.py:10963>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10963
--# expr-start 6333 '<Cast to Name at ast.py:10963>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10963
Cast_Expr := Var_Expr_3; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 6333
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6334 '<Symbol On at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_On); 
--# expr-done 6334
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Cast_Result_1.Node, Sym => Sym_1);
--# end
--# expr-done 6335
Scope_Result_1 := Fld_5;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := True;
end if;
Scope_Result_2 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   --# expr-start 6336 '<FieldAccess for parent_declarative_part_is_spark at ast.py:10969>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10969
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.parent_declarative_part_is_spark
Fld_6 := Libadalang.Implementation.Declarative_Part_P_Parent_Declarative_Part_Is_Spark (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6336
   Result_Var_1 := Fld_6;
end if;
Scope_Result_3 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_3;
else
   --# expr-start 6337 '<FieldAccess for parent_declarative_part_is_spark at ast.py:10971>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10971
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.parent_declarative_part_is_spark
Fld_7 := Libadalang.Implementation.Declarative_Part_P_Parent_Declarative_Part_Is_Spark (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6337
   Result_Var_2 := Fld_7;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Let_Result := Result_Var_3; 
--# end
--# expr-done 6338
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Declarative_Part_P_Is_Spark;
--# end
   --
   --  Primitives for Bare_Private_Part
   --
      procedure Private_Part_Pre_Env_Actions
        (Self            : Bare_Private_Part;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_45 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:11020");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_46 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_47 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Private_Part
        (Self : Bare_Private_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Initialize_Fields_For_Declarative_Part
              (Self, Declarative_Part_F_Decls);
      end Initialize_Fields_For_Private_Part;
--# property-start PrivatePart.env_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11000
pragma Warnings (Off, "is not referenced");
 function Private_Part_P_Env_Names
  (Node : Bare_Private_Part
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_855 with Inline_Always;
            procedure Finalizer_Scope_2757 with Inline_Always;
            procedure Finalizer_Scope_2758 with Inline_Always;
      Var_Expr : Bare_Base_Package_Decl;
Var_Expr_1 : String_Type;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
Fld_1 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Scope_Result_1 : Symbol_Type_Array_Access;
Result_Var_1 : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_855 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_855;
            procedure Finalizer_Scope_2757 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2757;
            procedure Finalizer_Scope_2758 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2758;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6346 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6340 '<Cast to BasePackageDecl at ast.py:11007>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11007
--# expr-start 6339 '<parent at ast.py:11007>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11007
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6339
   if Fld = null
      or else Fld.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6340
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pkg Var_Expr
--# expr-start 6341 '<FieldAccess for top_level_env_name at ast.py:11008>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11008
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Var_Expr);
--# end
--# expr-done 6341
Var_Expr_1 := Fld_1;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= Empty_String then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 6345 '<ArrayLiteral at ast.py:11009>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11009
--# expr-start 6344 '<to_symbol at ast.py:11010>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11010
--# expr-start 6343 '<Concat at ast.py:11010>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11010
--# expr-start 6342 '<String at ast.py:11010>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11010
Str := Create_String (".__privatepart"); 
--# expr-done 6342
Concat_Result := Concat_String (Var_Expr_1, Str); 
--# expr-done 6343
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 6344
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 6345
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2758;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2757;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6346
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_855;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_855;
                     Finalizer_Scope_2757;
                     Finalizer_Scope_2758;
            raise;
      end;
   return Property_Result;
end Private_Part_P_Env_Names;
--# end
--# property-start PrivatePart.immediate_declarative_region /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11015
pragma Warnings (Off, "is not referenced");
 function Private_Part_P_Immediate_Declarative_Region
  (Node : Bare_Private_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
     Ent : Internal_Entity_Private_Part :=
       Internal_Entity_Private_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_856 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_856 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_856;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6349 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6348 '<FieldAccess for immediate_declarative_region at ast.py:11017>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11017
--# expr-start 6347 '<FieldAccess for semantic_parent at ast.py:11017>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11017
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6347
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.immediate_declarative_region'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Immediate_Declarative_Region (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6348
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6349
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_856;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_856;
            raise;
      end;
   return Property_Result;
end Private_Part_P_Immediate_Declarative_Region;
--# end
--# property-start PrivatePart.is_spark /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11024
pragma Warnings (Off, "is not referenced");
 function Private_Part_P_Is_Spark
  (Node : Bare_Private_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
     Ent : Internal_Entity_Private_Part :=
       Internal_Entity_Private_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Pragma_Node;
Var_Expr_1 : Internal_Entity_Package_Decl;
Fld : Internal_Entity_Pragma_Node;
Fld_1 : Boolean;
Scope_Result : Boolean;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Package_Decl;
Fld_3 : Internal_Entity_Public_Part;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
Result_Var_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6356 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6350 '<FieldAccess for spark_mode_pragma at ast.py:11030>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11030
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.spark_mode_pragma
Fld := Libadalang.Implementation.Declarative_Part_P_Spark_Mode_Pragma (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6350
Var_Expr := Fld;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind pragma Var_Expr
--# expr-start 6351 '<FieldAccess for spark_mode_is_on at ast.py:11031>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11031
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.spark_mode_is_on
Fld_1 := Libadalang.Implementation.Pragma_Node_P_Spark_Mode_Is_On (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6351
Scope_Result := Fld_1;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 6353 '<Cast to PackageDecl at ast.py:11034>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11034
--# expr-start 6352 '<parent at ast.py:11034>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11034
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6352
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Package_Decl;
   end if;
--# expr-done 6353
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Package_Decl then
   --# scope-start
--# expr-start 6355 '<FieldAccess for is_spark at ast.py:11035>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11035
--# expr-start 6354 '<FieldAccess for public_part at ast.py:11035>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11035
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Public_Part (Node => Var_Expr_1.Node.Base_Package_Decl_F_Public_Part, Info => Var_Expr_1.Info);
--# expr-done 6354
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]DeclarativePart.is_spark'
Fld_4 := Libadalang.Implementation.Dispatcher_Declarative_Part_P_Is_Spark (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 6355
Scope_Result_1 := Fld_4;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   Result_Var_1 := Result_Var;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 6356
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Part_P_Is_Spark;
--# end
--# property-start PrivatePart._env_mappings_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11020
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_45
  (Node : Bare_Private_Part
  )
   return Internal_Env_Assoc
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1572 with Inline_Always;
      Sym : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Private_Part;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1572 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1572;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6359 '<New EnvAssoc at ast.py:11020>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11020
--# expr-start 6357 '<New DesignatedEnv at ast.py:11020>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11020
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 6357
--# expr-start 6358 '<Symbol __privatepart at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Privatepart); 
--# expr-done 6358
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6359
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1572;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1572;
            raise;
      end;
   return Property_Result;
end Env_Mappings_45;
--# end
--# property-start PrivatePart._env_trans_parent_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11019
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_46
  (Node : Bare_Private_Part
  )
   return Boolean
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_46;
--# end
--# property-start PrivatePart._env_names_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11021
pragma Warnings (Off, "is not referenced");
 function Env_Names_47
  (Node : Bare_Private_Part
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1574 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1574 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1574;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6360 '<FieldAccess for env_names at ast.py:11021>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:11021
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PrivatePart.env_names
Fld := Libadalang.Implementation.Private_Part_P_Env_Names (Node => Self);
--# end
--# expr-done 6360
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1574;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1574;
            raise;
      end;
   return Property_Result;
end Env_Names_47;
--# end
   --
   --  Primitives for Bare_Public_Part
   --
      procedure Initialize_Fields_For_Public_Part
        (Self : Bare_Public_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Initialize_Fields_For_Declarative_Part
              (Self, Declarative_Part_F_Decls);
      end Initialize_Fields_For_Public_Part;
   --
   --  Primitives for Bare_Elsif_Expr_Part
   --
      procedure Initialize_Fields_For_Elsif_Expr_Part
        (Self : Bare_Elsif_Expr_Part
         ; Elsif_Expr_Part_F_Cond_Expr : Bare_Expr
         ; Elsif_Expr_Part_F_Then_Expr : Bare_Expr
        ) is
      begin
            Self.Elsif_Expr_Part_F_Cond_Expr := Elsif_Expr_Part_F_Cond_Expr;
            Self.Elsif_Expr_Part_F_Then_Expr := Elsif_Expr_Part_F_Then_Expr;
      end Initialize_Fields_For_Elsif_Expr_Part;
   function Elsif_Expr_Part_F_Cond_Expr
     (Node : Bare_Elsif_Expr_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Expr_Part_F_Cond_Expr;
   end;
   function Elsif_Expr_Part_F_Then_Expr
     (Node : Bare_Elsif_Expr_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Expr_Part_F_Then_Expr;
   end;
   --
   --  Primitives for Bare_Elsif_Stmt_Part
   --
      procedure Initialize_Fields_For_Elsif_Stmt_Part
        (Self : Bare_Elsif_Stmt_Part
         ; Elsif_Stmt_Part_F_Cond_Expr : Bare_Expr
         ; Elsif_Stmt_Part_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Elsif_Stmt_Part_F_Cond_Expr := Elsif_Stmt_Part_F_Cond_Expr;
            Self.Elsif_Stmt_Part_F_Stmts := Elsif_Stmt_Part_F_Stmts;
      end Initialize_Fields_For_Elsif_Stmt_Part;
   function Elsif_Stmt_Part_F_Cond_Expr
     (Node : Bare_Elsif_Stmt_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Stmt_Part_F_Cond_Expr;
   end;
   function Elsif_Stmt_Part_F_Stmts
     (Node : Bare_Elsif_Stmt_Part) return Bare_Stmt_List
   is
   begin
         return Node.Elsif_Stmt_Part_F_Stmts;
   end;
--# property-start ElsifStmtPart.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21279
pragma Warnings (Off, "is not referenced");
 function Elsif_Stmt_Part_P_Xref_Entry_Point
  (Node : Bare_Elsif_Stmt_Part
  )
   return Boolean
is
   Self : Bare_Elsif_Stmt_Part  := Bare_Elsif_Stmt_Part (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Elsif_Stmt_Part_P_Xref_Entry_Point;
--# end
--# property-start ElsifStmtPart.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21281
pragma Warnings (Off, "is not referenced");
 function Elsif_Stmt_Part_P_Xref_Equation
  (Node : Bare_Elsif_Stmt_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Elsif_Stmt_Part  := Bare_Elsif_Stmt_Part (Node);
     Ent : Internal_Entity_Elsif_Stmt_Part :=
       Internal_Entity_Elsif_Stmt_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1454 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1454 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1454;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6371 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6370 '<And at ast.py:21284>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21284
--# expr-start 6367 '<And at ast.py:21284>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21284
--# expr-start 6364 '<Bind at ast.py:21284>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21284
--# expr-start 6362 '<FieldAccess for expected_type_var at ast.py:21284>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21284
--# expr-start 6361 '<FieldAccess for cond_expr at ast.py:21284>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21284
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Elsif_Stmt_Part_F_Cond_Expr;
--# expr-done 6361
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 6362
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 6363 '<FieldAccess for bool_type at ast.py:21284>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21284
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 6363
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21284") else null)); 
--# expr-done 6364
--# expr-start 6366 '<FieldAccess for sub_equation at ast.py:21285>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21285
--# expr-start 6365 '<FieldAccess for cond_expr at ast.py:21285>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21285
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Elsif_Stmt_Part_F_Cond_Expr, Info => Ent.Info);
--# expr-done 6365
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 6366
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21284") else null)); 
--# expr-done 6367
--# expr-start 6369 '<FieldAccess for matches_expected_formal_prim_type at ast.py:21286>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21286
--# expr-start 6368 '<FieldAccess for cond_expr at ast.py:21286>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21286
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Elsif_Stmt_Part_F_Cond_Expr;
--# expr-done 6368
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_5, Origin => Origin);
--# end
--# expr-done 6369
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21284") else null)); 
--# expr-done 6370
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6371
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1454;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1454;
            raise;
      end;
   return Property_Result;
end Elsif_Stmt_Part_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Expr
   --
      procedure Initialize_Fields_For_Expr
        (Self : Bare_Expr
        ) is
      begin
      Self.Expr_Logic_Vars := System.Null_Address;
      end Initialize_Fields_For_Expr;
--# property-start Expr.type_val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12221
pragma Warnings (Off, "is not referenced");
 function Expr_P_Type_Val
  (Node : Bare_Expr
  )
   return Internal_Entity
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Internal_Entity;
      Fld : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6373 '<get_value at ast.py:12221>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12221
--# expr-start 6372 '<FieldAccess for type_var at ast.py:12221>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12221
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6372
Is_Logic_Var_Defined := Entity_Vars.Is_Defined (Fld); 
if Is_Logic_Var_Defined then
Eq_Solution := Entity_Vars.Get_Value (Fld); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 6373
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Type_Val;
--# end
--# property-start Expr.expression_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12223
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expression_Type
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6377 '<Cast to BaseTypeDecl at ast.py:12225>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12225
--# expr-start 6376 '<FieldAccess for value at ast.py:12225>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12225
--# expr-start 6375 '<FieldAccess for logic_val at ast.py:12224>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12224
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 6374 '<FieldAccess for type_var at ast.py:12224>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12224
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6374
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 6375
Fld_2 := Fld_1.Value;
--# expr-done 6376
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6377
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Expression_Type;
--# end
--# property-start Expr.expected_expression_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12233
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expected_Expression_Type
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6381 '<Cast to BaseTypeDecl at ast.py:12235>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12235
--# expr-start 6380 '<FieldAccess for value at ast.py:12235>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12235
--# expr-start 6379 '<FieldAccess for logic_val at ast.py:12234>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12234
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 6378 '<FieldAccess for expected_type_var at ast.py:12234>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12234
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6378
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 6379
Fld_2 := Fld_1.Value;
--# expr-done 6380
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6381
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Expected_Expression_Type;
--# end
--# property-start Expr.xref_stop_resolution /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12243
pragma Warnings (Off, "is not referenced");
 function Expr_P_Xref_Stop_Resolution
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_For_Loop_Spec;
Fld : Bare_Ada_Node;
Cast_Result : Bare_For_Loop_Spec;
Fld_1 : Bare_Expr;
Cast_Result_1 : Bare_Expr;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 6383 '<Cast to ForLoopSpec at ast.py:12248>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12248
--# expr-start 6382 '<parent at ast.py:12248>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12248
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6382
   if Fld = null
      or else Fld.Kind in Ada_For_Loop_Spec_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6383
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 6385 '<Eq at ast.py:12249>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12249
--# expr-start 6384 '<FieldAccess for iter_filter at ast.py:12249>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12249
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr.For_Loop_Spec_F_Iter_Filter;
--# expr-done 6384
      Cast_Result_1 := Fld_1;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 6385
Scope_Result := Is_Equal;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Xref_Stop_Resolution;
--# end
--# property-start '[dispatcher]Expr.has_context_free_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Has_Context_Free_Type
  (Node : Bare_Expr
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr | Ada_Box_Expr .. Ada_Case_Expr_Alternative | Ada_Contract_Cases | Ada_Membership_Expr | Ada_Call_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Identifier .. Ada_Op_Xor | Ada_Int_Literal .. Ada_Update_Attribute_Ref | Ada_Quantified_Expr =>
                  --# property-call-start Expr.has_context_free_type
                  Property_Result := Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Allocator =>
                  --# property-call-start Allocator.has_context_free_type
                  Property_Result := Allocator_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
                  --# property-call-start BaseAggregate.has_context_free_type
                  Property_Result := Base_Aggregate_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Bin_Op =>
                  --# property-call-start BinOp.has_context_free_type
                  Property_Result := Bin_Op_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Relation_Op =>
                  --# property-call-start RelationOp.has_context_free_type
                  Property_Result := Relation_Op_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Concat_Op =>
                  --# property-call-start ConcatOp.has_context_free_type
                  Property_Result := Concat_Op_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Concat_Operand =>
                  --# property-call-start ConcatOperand.has_context_free_type
                  Property_Result := Concat_Operand_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.has_context_free_type
                  Property_Result := Case_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.has_context_free_type
                  Property_Result := If_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Decl_Expr =>
                  --# property-call-start DeclExpr.has_context_free_type
                  Property_Result := Decl_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.has_context_free_type
                  Property_Result := Attribute_Ref_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.has_context_free_type
                  Property_Result := Dotted_Name_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Char_Literal =>
                  --# property-call-start CharLiteral.has_context_free_type
                  Property_Result := Char_Literal_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_String_Literal =>
                  --# property-call-start StringLiteral.has_context_free_type
                  Property_Result := String_Literal_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Null_Literal =>
                  --# property-call-start NullLiteral.has_context_free_type
                  Property_Result := Null_Literal_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Paren_Expr =>
                  --# property-call-start ParenExpr.has_context_free_type
                  Property_Result := Paren_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Raise_Expr =>
                  --# property-call-start RaiseExpr.has_context_free_type
                  Property_Result := Raise_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.has_context_free_type
                  Property_Result := Un_Op_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Has_Context_Free_Type;
--# end
--# property-start Expr.matches_expected_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12268
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_944 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_944 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_944;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6389 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6388 '<Predicate on BaseTypeDecl.matching_type at ast.py:12270>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12270
--# expr-start 6386 '<FieldAccess for type_var at ast.py:12272>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12272
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6386
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 6387 '<FieldAccess for expected_type_var at ast.py:12273>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12273
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6387
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Type_0_Predicate (Origin)); 
--# expr-done 6388
Let_Result := Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6389
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_944;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_944;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Type;
--# end
--# property-start Expr.matches_expected_assign_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12276
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Assign_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_945 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_945 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_945;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6393 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6392 '<Predicate on BaseTypeDecl.matching_assign_type at ast.py:12278>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12278
--# expr-start 6390 '<FieldAccess for type_var at ast.py:12280>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12280
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6390
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 6391 '<FieldAccess for expected_type_var at ast.py:12281>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12281
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6391
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Assign_Type_0_Predicate (Origin)); 
--# expr-done 6392
Let_Result := Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6393
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_945;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_945;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Assign_Type;
--# end
--# property-start Expr.matches_expected_formal_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12284
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Formal_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_946 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_946 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_946;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6397 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6396 '<Predicate on BaseTypeDecl.matching_formal_type at ast.py:12286>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12286
--# expr-start 6394 '<FieldAccess for type_var at ast.py:12288>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12288
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6394
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 6395 '<FieldAccess for expected_type_var at ast.py:12289>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12289
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6395
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Formal_Type_0_Predicate (Origin)); 
--# expr-done 6396
Let_Result := Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6397
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_946;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_946;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Formal_Type;
--# end
--# property-start Expr.matches_expected_formal_prim_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12292
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Formal_Prim_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_947 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_947 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_947;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6401 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6400 '<Predicate on BaseTypeDecl.matching_formal_prim_type at ast.py:12294>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12294
--# expr-start 6398 '<FieldAccess for type_var at ast.py:12296>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12296
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6398
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 6399 '<FieldAccess for expected_type_var at ast.py:12297>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12297
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6399
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Formal_Prim_Type_0_Predicate (Origin)); 
--# expr-done 6400
Let_Result := Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6401
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_947;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_947;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Formal_Prim_Type;
--# end
--# property-start Expr.matches_expected_prefix_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12300
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Prefix_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_948 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_948 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_948;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6405 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6404 '<Predicate on BaseTypeDecl.matching_prefix_type at ast.py:12302>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12302
--# expr-start 6402 '<FieldAccess for type_var at ast.py:12304>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12304
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6402
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 6403 '<FieldAccess for expected_type_var at ast.py:12305>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12305
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6403
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate (Origin)); 
--# expr-done 6404
Let_Result := Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6405
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_948;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_948;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Prefix_Type;
--# end
--# property-start Expr.is_dynamically_tagged /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12308
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dynamically_Tagged
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_2766 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Qual_Expr;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Cond_Expr;
Match_3 : Internal_Entity_Decl_Expr;
Match_4 : Internal_Entity_Paren_Expr;
Match_5 : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Boolean;
Fld_8 : Boolean;
Fld_9 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld_10 : Boolean;
Fld_11 : Boolean;
If_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_2 : Boolean;
Fld_12 : Internal_Entity_Expr_Array_Access;
Fld_13 : Boolean;
Quantifier_Result : Boolean;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Boolean;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Qual_Expr;
Let_Result : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_1 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Internal_Entity_Cond_Expr;
Let_Result_2 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_4 : Internal_Entity_Decl_Expr;
Let_Result_3 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_5 : Internal_Entity_Paren_Expr;
Let_Result_4 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_6 : Internal_Entity_Expr;
Let_Result_5 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
If_Result_3 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_6 : Boolean;
            procedure Finalizer_Scope_2766 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2766;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6428 '<Block at ???>' Let_Result_6 None
--# scope-start
--# expr-start 6427 '<bind at ast.py:12316>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12316
--# expr-start 6406 '<FieldAccess for origin_node at ast.py:12316>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12316
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6406
Bound_Origin := Fld; 
--# expr-start 6408 '<FieldAccess for is_classwide at ast.py:12317>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12317
--# expr-start 6407 '<FieldAccess for expression_type at ast.py:12317>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12317
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6407
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6408
if Fld_2 then
   If_Result := True;
else
--# expr-start 6410 '<FieldAccess for accessed_type at ast.py:12318>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12318
--# expr-start 6409 '<FieldAccess for expression_type at ast.py:12318>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12318
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6409
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_3.Node, Origin => Bound_Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 6410
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 6411 '<FieldAccess for is_classwide at ast.py:12318>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12318
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6411
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_3 := True;
else
   --# expr-start 6426 '<Match at ast.py:12320>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12320
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind qual_expr Match_0
--# expr-start 6413 '<FieldAccess for is_dynamically_tagged at ast.py:12322>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12322
--# expr-start 6412 '<FieldAccess for suffix at ast.py:12322>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12322
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Match_0.Node.Qual_Expr_F_Suffix, Info => Match_0.Info);
--# expr-done 6412
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_7 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_6.Info);
--# end
--# expr-done 6413
Let_Result := Fld_7; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Attribute_Ref .. Ada_Explicit_Deref | Ada_Reduce_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind n Match_1
--# expr-start 6414 '<FieldAccess for is_direct_call at ast.py:12328>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12328
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_direct_call
Fld_8 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6414
if Fld_8 then
--# expr-start 6416 '<Cast to BaseSubpSpec at ast.py:12329>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12329
--# expr-start 6415 '<FieldAccess for called_subp_spec at ast.py:12329>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12329
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_subp_spec'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Called_Subp_Spec (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6415
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Spec;
   end if;
--# expr-done 6416
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_1
--# expr-start 6417 '<FieldAccess for has_controlling_result at ast.py:12331>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12331
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.has_controlling_result
Fld_10 := Libadalang.Implementation.Base_Subp_Spec_P_Has_Controlling_Result (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6417
if Fld_10 then
   --# expr-start 6418 '<FieldAccess for has_dynamic_controlling_operand at ast.py:12332>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12332
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.has_dynamic_controlling_operand
Fld_11 := Libadalang.Implementation.Expr_P_Has_Dynamic_Controlling_Operand (Node => Match_1.Node, Spec => Var_Expr_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 6418
   If_Result_1 := Fld_11;
else
   If_Result_1 := False;
end if;
Scope_Result_1 := If_Result_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_2 := Result_Var_1;
else
   If_Result_2 := False;
end if;
Let_Result_1 := If_Result_2; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Case_Expr .. Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Cond_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind cond_expr Match_2
--# expr-start 6421 '<AllQuantifier at ast.py:12336>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12336
   --# expr-start 6419 '<FieldAccess for dependent_exprs at ast.py:12336>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12336
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]CondExpr.dependent_exprs'
Fld_12 := Libadalang.Implementation.Dispatcher_Cond_Expr_P_Dependent_Exprs (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 6419
      Quantifier_Result := True;
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Fld_12;
      begin
         for Item_223 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_223
      --# expr-start 6420 '<FieldAccess for is_dynamically_tagged at ast.py:12337>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12337
      if Item_223.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_13 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Item_223.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_223.Info);
--# end
--# expr-done 6420
      Quantifier_Result := Fld_13;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
--# expr-done 6421
Let_Result_2 := Quantifier_Result; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
      Finalizer_Scope_2766;
            Match_Result := Scope_Result_4; 
         when Ada_Decl_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Decl_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind decl_expr Match_3
--# expr-start 6423 '<FieldAccess for is_dynamically_tagged at ast.py:12341>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12341
--# expr-start 6422 '<FieldAccess for expr at ast.py:12341>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12341
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Match_3.Node.Decl_Expr_F_Expr, Info => Match_3.Info);
--# expr-done 6422
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_15 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_14.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_14.Info);
--# end
--# expr-done 6423
Let_Result_3 := Fld_15; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_5; 
--# bind paren_expr Match_4
--# expr-start 6425 '<FieldAccess for is_dynamically_tagged at ast.py:12344>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12344
--# expr-start 6424 '<FieldAccess for expr at ast.py:12344>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12344
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Match_4.Node.Paren_Expr_F_Expr, Info => Match_4.Info);
--# expr-done 6424
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_17 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_16.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_16.Info);
--# end
--# expr-done 6425
Let_Result_4 := Fld_17; 
--# end
Scope_Result_6 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Concat_Operand | Ada_Contract_Cases | Ada_Membership_Expr | Ada_Quantified_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_6; 
--# bind _ Match_5
Let_Result_5 := False; 
--# end
Scope_Result_7 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 6426
   If_Result_3 := Match_Result;
end if;
Dyn_Var_Bind_Result := If_Result_3; 
--# expr-done 6427
Let_Result_6 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6428
         Property_Result := Let_Result_6;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2766;
            raise;
      end;
   return Property_Result;
end Expr_P_Is_Dynamically_Tagged;
--# end
--# property-start '[dispatcher]Expr.potential_actuals_for_dispatch' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Expr
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Raise_Expr =>
                  --# property-call-start Expr.potential_actuals_for_dispatch
                  Property_Result := Expr_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.potential_actuals_for_dispatch
                  Property_Result := Bin_Op_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.potential_actuals_for_dispatch
                  Property_Result := Name_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.potential_actuals_for_dispatch
                  Property_Result := Un_Op_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start Expr.has_dynamic_controlling_operand /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12367
pragma Warnings (Off, "is not referenced");
 function Expr_P_Has_Dynamic_Controlling_Operand
  (Node : Bare_Expr
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_951 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Expected_Type_For_Expr_Array_Access;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_951 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_951;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6437 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6436 '<AnyQuantifier at ast.py:12380>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12380
   --# expr-start 6429 '<FieldAccess for potential_actuals_for_dispatch at ast.py:12380>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12380
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.potential_actuals_for_dispatch'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Potential_Actuals_For_Dispatch (Node => Ent.Node, Spec => Spec, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6429
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Expected_Type_For_Expr_Array_Access := Fld;
      begin
         for Item_224 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_224
--# expr-start 6431 '<FieldAccess for get_candidate_type_for_primitive at ast.py:12382>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12382
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6430 '<FieldAccess for expected_type at ast.py:12382>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12382
Fld_1 := Item_224.Expected_Type;
--# expr-done 6430
--# property-call-start BaseSubpSpec.get_candidate_type_for_primitive
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Candidate_Type_For_Primitive (Node => Spec.Node, Type_Expr => Fld_1, E_Info => Spec.Info);
--# end
--# expr-done 6431
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 6433 '<FieldAccess for is_tagged_type at ast.py:12388>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12388
--# expr-start 6432 '<FieldAccess for full_view at ast.py:12388>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12388
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6432
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_3.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_3.Info);
--# end
--# expr-done 6433
if Fld_4 then
   --# expr-start 6435 '<FieldAccess for is_dynamically_tagged at ast.py:12389>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12389
--# expr-start 6434 '<FieldAccess for expr at ast.py:12389>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12389
Fld_5 := Item_224.Expr;
--# expr-done 6434
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_6 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 6435
   If_Result := Fld_6;
else
   If_Result := False;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      Quantifier_Result := Result_Var;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6436
Let_Result := Quantifier_Result; 
--# end
--# expr-done 6437
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_951;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_951;
            raise;
      end;
   return Property_Result;
end Expr_P_Has_Dynamic_Controlling_Operand;
--# end
--# property-start Expr.parent_candidate_dispatching_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12394
pragma Warnings (Off, "is not referenced");
 function Expr_P_Parent_Candidate_Dispatching_Call
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_2777 with Inline_Always;
      Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Dotted_Name;
Match_2 : Internal_Entity_Un_Op;
Match_3 : Internal_Entity_Bin_Op;
Match_4 : Internal_Entity_Cond_Expr;
Match_5 : Internal_Entity_Paren_Expr;
Match_6 : Internal_Entity_Qual_Expr;
Match_7 : Internal_Entity_Decl_Expr;
Match_8 : Internal_Entity_Param_Assoc;
Match_9 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Call_Expr;
Cast_Result : Internal_Entity_Expr;
Fld_2 : Internal_Entity_Name;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Expr;
Cast_Expr_2 : Internal_Entity_Dotted_Name;
Cast_Result_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Base_Id;
Cast_Expr_3 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity;
Cast_Result_5 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result_1 : Internal_Entity_Expr;
Fld_5 : Internal_Entity_Expr;
Cast_Expr_4 : Internal_Entity_Un_Op;
Cast_Result_6 : Internal_Entity_Expr;
Fld_6 : Internal_Entity_Op;
Cast_Expr_5 : Internal_Entity_Expr;
Cast_Result_7 : Internal_Entity;
Cast_Result_8 : Internal_Entity;
Is_Equiv_2 : Boolean;
If_Result_2 : Internal_Entity_Expr;
Fld_7 : Internal_Entity_Expr;
Cast_Expr_6 : Internal_Entity_Bin_Op;
Cast_Result_9 : Internal_Entity_Expr;
Fld_8 : Internal_Entity_Op;
Cast_Expr_7 : Internal_Entity_Expr;
Cast_Result_10 : Internal_Entity;
Cast_Result_11 : Internal_Entity;
Is_Equiv_3 : Boolean;
If_Result_3 : Internal_Entity_Expr;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Internal_Entity_Expr_Array_Access;
Cast_Expr_8 : Internal_Entity_Expr;
Cast_Result_12 : Internal_Entity;
Cast_Expr_9 : Internal_Entity_Expr;
Cast_Result_13 : Internal_Entity;
Is_Equiv_4 : Boolean;
Quantifier_Result : Boolean;
If_Result_4 : Internal_Entity_Expr;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Internal_Entity;
Fld_15 : Internal_Entity;
Cast_Result_14 : Internal_Entity_Call_Expr;
Match_Prefix : Internal_Entity;
Cast_Result_15 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Cast_Result_16 : Internal_Entity_Dotted_Name;
Let_Result_1 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Cast_Result_17 : Internal_Entity_Un_Op;
Let_Result_2 : Internal_Entity_Expr;
Scope_Result_2 : Internal_Entity_Expr;
Cast_Result_18 : Internal_Entity_Bin_Op;
Let_Result_3 : Internal_Entity_Expr;
Scope_Result_3 : Internal_Entity_Expr;
Cast_Result_19 : Internal_Entity_Cond_Expr;
Let_Result_4 : Internal_Entity_Expr;
Scope_Result_4 : Internal_Entity_Expr;
Cast_Result_20 : Internal_Entity_Paren_Expr;
Let_Result_5 : Internal_Entity_Expr;
Scope_Result_5 : Internal_Entity_Expr;
Cast_Result_21 : Internal_Entity_Qual_Expr;
Let_Result_6 : Internal_Entity_Expr;
Scope_Result_6 : Internal_Entity_Expr;
Cast_Result_22 : Internal_Entity_Decl_Expr;
Let_Result_7 : Internal_Entity_Expr;
Scope_Result_7 : Internal_Entity_Expr;
Cast_Result_23 : Internal_Entity_Param_Assoc;
Cast_Result_24 : Internal_Entity_Expr;
Let_Result_8 : Internal_Entity_Expr;
Scope_Result_8 : Internal_Entity_Expr;
Cast_Result_25 : Internal_Entity;
Let_Result_9 : Internal_Entity_Expr;
Scope_Result_9 : Internal_Entity_Expr;
Match_Result : Internal_Entity_Expr;
Let_Result_10 : Internal_Entity_Expr;
            procedure Finalizer_Scope_2777 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2777;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6467 '<Block at ???>' Let_Result_10 None
--# scope-start
--# expr-start 6466 '<Match at ast.py:12401>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12401
--# expr-start 6438 '<parent at ast.py:12401>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12401
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6438
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_15; 
--# bind ce Match_0
--# expr-start 6442 '<If at ast.py:12402>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12402
--# expr-start 6440 '<Eq at ast.py:12403>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12403
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 6439 '<FieldAccess for name at ast.py:12403>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12403
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Call_Expr_F_Name, Info => Match_0.Info);
--# expr-done 6439
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 6440
if Is_Equiv then
   --# expr-start 6441 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12404>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12404
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_1 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 6441
   If_Result := Fld_1;
else
Cast_Expr := Match_0; 
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result := Cast_Result;
end if;
--# expr-done 6442
Let_Result := If_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_16 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_16; 
--# bind dn Match_1
--# expr-start 6446 '<If at ast.py:12407>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12407
--# expr-start 6444 '<Eq at ast.py:12408>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12408
Cast_Expr_3 := Ent; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-start 6443 '<FieldAccess for suffix at ast.py:12408>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12408
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Match_1.Node.Dotted_Name_F_Suffix, Info => Match_1.Info);
--# expr-done 6443
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv_1 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 6444
if Is_Equiv_1 then
   --# expr-start 6445 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12409>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12409
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_3 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6445
   If_Result_1 := Fld_3;
else
Cast_Expr_2 := Match_1; 
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   If_Result_1 := Cast_Result_3;
end if;
--# expr-done 6446
Let_Result_1 := If_Result_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_17 := Create_Internal_Entity_Un_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_17; 
--# bind uo Match_2
--# expr-start 6450 '<If at ast.py:12412>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12412
--# expr-start 6448 '<Eq at ast.py:12413>' Is_Equiv_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12413
Cast_Expr_5 := Ent; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
--# expr-start 6447 '<FieldAccess for op at ast.py:12413>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12413
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Op (Node => Match_2.Node.Un_Op_F_Op, Info => Match_2.Info);
--# expr-done 6447
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Is_Equiv_2 := Equivalent (Cast_Result_7, Cast_Result_8); 
--# expr-done 6448
if Is_Equiv_2 then
   --# expr-start 6449 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12414>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12414
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_5 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 6449
   If_Result_2 := Fld_5;
else
Cast_Expr_4 := Match_2; 
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   If_Result_2 := Cast_Result_6;
end if;
--# expr-done 6450
Let_Result_2 := If_Result_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_18 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_18; 
--# bind bo Match_3
--# expr-start 6454 '<If at ast.py:12417>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12417
--# expr-start 6452 '<Eq at ast.py:12418>' Is_Equiv_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12418
Cast_Expr_7 := Ent; 
      Cast_Result_10 := Create_Internal_Entity
        (Node => Cast_Expr_7.Node,
         Info => Cast_Expr_7.Info);
--# expr-start 6451 '<FieldAccess for op at ast.py:12418>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12418
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Op (Node => Match_3.Node.Bin_Op_F_Op, Info => Match_3.Info);
--# expr-done 6451
      Cast_Result_11 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Is_Equiv_3 := Equivalent (Cast_Result_10, Cast_Result_11); 
--# expr-done 6452
if Is_Equiv_3 then
   --# expr-start 6453 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12419>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12419
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_7 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 6453
   If_Result_3 := Fld_7;
else
Cast_Expr_6 := Match_3; 
      Cast_Result_9 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
   If_Result_3 := Cast_Result_9;
end if;
--# expr-done 6454
Let_Result_3 := If_Result_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Case_Expr .. Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_19 := Create_Internal_Entity_Cond_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_19; 
--# bind ce Match_4
--# expr-start 6459 '<If at ast.py:12422>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12422
--# expr-start 6457 '<Contains at ast.py:12423>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12423
   --# expr-start 6455 '<FieldAccess for dependent_exprs at ast.py:12423>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12423
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]CondExpr.dependent_exprs'
Fld_10 := Libadalang.Implementation.Dispatcher_Cond_Expr_P_Dependent_Exprs (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 6455
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Fld_10;
      begin
         for Item_225 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_225
      --# expr-start 6456 '<Eq at ???>' Is_Equiv_4 None
Cast_Expr_8 := Item_225; 
      Cast_Result_12 := Create_Internal_Entity
        (Node => Cast_Expr_8.Node,
         Info => Cast_Expr_8.Info);
Cast_Expr_9 := Ent; 
      Cast_Result_13 := Create_Internal_Entity
        (Node => Cast_Expr_9.Node,
         Info => Cast_Expr_9.Info);
Is_Equiv_4 := Equivalent (Cast_Result_12, Cast_Result_13); 
--# expr-done 6456
      Quantifier_Result := Is_Equiv_4;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6457
if Quantifier_Result then
   --# expr-start 6458 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12424>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12424
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_9 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 6458
   If_Result_4 := Fld_9;
else
   If_Result_4 := No_Entity_Expr;
end if;
--# expr-done 6459
Let_Result_4 := If_Result_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
      Finalizer_Scope_2777;
            Match_Result := Scope_Result_4; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_20 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_20; 
--# bind pe Match_5
--# expr-start 6460 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12427>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12427
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_11 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_5.Node, E_Info => Match_5.Info);
--# end
--# expr-done 6460
Let_Result_5 := Fld_11; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_21 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_21; 
--# bind qe Match_6
--# expr-start 6461 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12428>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12428
      if Match_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_12 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_6.Node, E_Info => Match_6.Info);
--# end
--# expr-done 6461
Let_Result_6 := Fld_12; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Decl_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_22 := Create_Internal_Entity_Decl_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_22; 
--# bind de Match_7
--# expr-start 6462 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12429>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12429
      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_13 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_7.Node, E_Info => Match_7.Info);
--# end
--# expr-done 6462
Let_Result_7 := Fld_13; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Param_Assoc =>
   --# scope-start
--# scope-start
      Cast_Result_23 := Create_Internal_Entity_Param_Assoc
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_23; 
--# bind pa Match_8
--# expr-start 6465 '<Cast to CallExpr at ast.py:12430>' Cast_Result_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12430
--# expr-start 6464 '<parent at ast.py:12430>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12430
--# expr-start 6463 '<parent at ast.py:12430>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12430
      if Match_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Parent (Node => Match_8.Node, E_Info => Match_8.Info);
--# expr-done 6463
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Parent (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# expr-done 6464
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_14 := Create_Internal_Entity_Call_Expr
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_14 := No_Entity_Call_Expr;
   end if;
--# expr-done 6465
      Cast_Result_24 := Create_Internal_Entity_Expr
        (Node => Cast_Result_14.Node,
         Info => Cast_Result_14.Info);
Let_Result_8 := Cast_Result_24; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Abort_Absent .. Ada_Iterated_Assoc | Ada_Abstract_State_Decl .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Concat_Operand | Ada_Contract_Cases | Ada_Membership_Expr .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Explicit_Deref | Ada_Reduce_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Quantified_Expr .. Ada_Raise_Expr | Ada_Handled_Stmts .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_25 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_9 := Cast_Result_25; 
--# bind _ Match_9
Let_Result_9 := No_Entity_Expr; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_9; 
end case;
--# expr-done 6466
Let_Result_10 := Match_Result; 
--# end
--# expr-done 6467
         Property_Result := Let_Result_10;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2777;
            raise;
      end;
   return Property_Result;
end Expr_P_Parent_Candidate_Dispatching_Call;
--# end
--# property-start Expr.has_dynamic_context /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12434
pragma Warnings (Off, "is not referenced");
 function Expr_P_Has_Dynamic_Context
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Assign_Stmt;
Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assign_Stmt;
Fld_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
Result_Var_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6476 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6469 '<Cast to AssignStmt at ast.py:12443>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12443
--# expr-start 6468 '<parent at ast.py:12443>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12443
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6468
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assign_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Assign_Stmt
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Assign_Stmt;
   end if;
--# expr-done 6469
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Assign_Stmt then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 6471 '<Eq at ast.py:12448>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12448
--# expr-start 6470 '<FieldAccess for expr at ast.py:12448>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12448
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Assign_Stmt_F_Expr, Info => Var_Expr.Info);
--# expr-done 6470
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Cast_Expr := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 6471
if Is_Equiv then
   --# expr-start 6473 '<FieldAccess for is_classwide at ast.py:12449>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12449
--# expr-start 6472 '<FieldAccess for expected_expression_type at ast.py:12449>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12449
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expected_expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expected_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6472
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6473
   If_Result := Fld_3;
else
   If_Result := False;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 6474 '<FieldAccess for parent_candidate_dispatching_call at ast.py:12455>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12455
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_4 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6474
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
--# expr-start 6475 '<FieldAccess for is_dispatching_call at ast.py:12456>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12456
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_5 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6475
Scope_Result_1 := Fld_5;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   Result_Var_1 := Result_Var;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 6476
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Has_Dynamic_Context;
--# end
--# property-start Expr.is_dispatching_call_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12459
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dispatching_Call_Impl
  (Node : Bare_Expr
      ; Decl : Internal_Entity_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind decl Decl
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Is_A : Boolean;
Fld_2 : Boolean;
If_Result : Boolean;
Fld_3 : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6483 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6478 '<FieldAccess for subp_spec_or_null at ast.py:12467>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12467
--# expr-start 6477 '<FieldAccess for canonical_part at ast.py:12467>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12467
      if Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Decl.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Decl.Info);
--# end
--# expr-done 6477
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld.Node, Follow_Generic => True, E_Info => Fld.Info);
--# end
--# expr-done 6478
Block_Var := Fld_1; 
--# bind spec Block_Var
--# expr-start 6479 '<IsA AbstractFormalSubpDecl at ast.py:12471>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12471
Is_A := Decl.Node /= null 
and then Decl.Node.Kind in Ada_Abstract_Formal_Subp_Decl_Range; 
--# expr-done 6479
if Is_A then
   If_Result := True;
else
   --# expr-start 6480 '<FieldAccess for has_dynamic_controlling_operand at ast.py:12475>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12475
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.has_dynamic_controlling_operand
Fld_2 := Libadalang.Implementation.Expr_P_Has_Dynamic_Controlling_Operand (Node => Ent.Node, Spec => Block_Var, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6480
   If_Result := Fld_2;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 6481 '<FieldAccess for has_controlling_result at ast.py:12481>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12481
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.has_controlling_result
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Has_Controlling_Result (Node => Block_Var.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 6481
if Fld_3 then
   --# expr-start 6482 '<FieldAccess for has_dynamic_context at ast.py:12481>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12481
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.has_dynamic_context
Fld_4 := Libadalang.Implementation.Expr_P_Has_Dynamic_Context (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6482
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 6483
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Is_Dispatching_Call_Impl;
--# end
--# property-start '[dispatcher]Expr.is_dispatching_call' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Is_Dispatching_Call
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Raise_Expr =>
                  --# property-call-start Expr.is_dispatching_call
                  Property_Result := Expr_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.is_dispatching_call
                  Property_Result := Bin_Op_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Identifier | Ada_String_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.is_dispatching_call
                  Property_Result := Name_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.is_dispatching_call
                  Property_Result := Op_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.is_dispatching_call
                  Property_Result := Un_Op_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Is_Dispatching_Call;
--# end
--# property-start Expr.is_static_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12510
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Static_Expr
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Num_Literal;
Match_1 : Internal_Entity_String_Literal;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Call_Expr;
Match_4 : Internal_Entity_Qual_Expr;
Match_5 : Internal_Entity_Name;
Match_6 : Internal_Entity_Membership_Expr;
Match_7 : Internal_Entity_Bin_Op;
Match_8 : Internal_Entity_Un_Op;
Match_9 : Internal_Entity_If_Expr;
Match_10 : Internal_Entity_Paren_Expr;
Match_11 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Assoc_List;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Var_Expr_6 : Internal_Entity_Basic_Decl;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Var_Expr_8 : Internal_Entity_Basic_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Not_Val : Boolean;
Fld_7 : Internal_Entity_Identifier;
Fld_8 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_12 : Internal_Entity_Identifier;
Fld_13 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
Fld_14 : Internal_Entity_Identifier;
Fld_15 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_16 : Internal_Entity_Identifier;
Fld_17 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_18 : Internal_Entity_Identifier;
Fld_19 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Fld_20 : Internal_Entity_Name;
Fld_21 : Internal_Entity_Basic_Decl;
Fld_22 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_23 : Internal_Entity_Identifier;
Fld_24 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
Fld_25 : Internal_Entity_Identifier;
Fld_26 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_6 : Boolean;
Fld_27 : Internal_Entity_Identifier;
Fld_28 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_7 : Boolean;
Fld_29 : Internal_Entity_Identifier;
Fld_30 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_8 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
If_Result_11 : Boolean;
Fld_31 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_32 : Bare_Assoc_List;
Fld_33 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_12 : Internal_Entity_Basic_Assoc;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Boolean;
Item_226 : Internal_Entity_Basic_Assoc;
Bare_Item_226 : Bare_Basic_Assoc;
Quantifier_Result : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_13 : Boolean;
Fld_36 : Internal_Entity_Name;
Fld_37 : Boolean;
Fld_38 : Internal_Entity_Assoc_List;
Fld_39 : Bare_Assoc_List;
Fld_40 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_14 : Internal_Entity_Basic_Assoc;
Fld_41 : Internal_Entity_Expr;
Fld_42 : Boolean;
Item_227 : Internal_Entity_Basic_Assoc;
Bare_Item_227 : Bare_Basic_Assoc;
Quantifier_Result_1 : Boolean;
If_Result_15 : Boolean;
Fld_43 : Internal_Entity_Name;
Fld_44 : Internal_Entity_Base_Type_Decl;
Fld_45 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Fld_46 : Internal_Entity_Expr;
Fld_47 : Boolean;
If_Result_16 : Boolean;
Fld_48 : Internal_Entity_Basic_Decl;
Fld_49 : Boolean;
Scope_Result_6 : Boolean;
Result_Var_6 : Boolean;
Fld_50 : Internal_Entity_Expr;
Fld_51 : Boolean;
Fld_52 : Internal_Entity_Expr_Alternatives_List;
Fld_53 : Bare_Expr_Alternatives_List;
Fld_54 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity_Expr;
Is_Null_2 : Boolean;
If_Result_17 : Internal_Entity_Expr;
Fld_55 : Boolean;
Item_228 : Internal_Entity_Expr;
Bare_Item_228 : Bare_Expr;
Quantifier_Result_2 : Boolean;
If_Result_18 : Boolean;
Fld_56 : Internal_Entity_Expr;
Fld_57 : Boolean;
Fld_58 : Internal_Entity_Expr;
Fld_59 : Boolean;
If_Result_19 : Boolean;
Fld_60 : Internal_Entity_Op;
Fld_61 : Internal_Entity_Basic_Decl;
Fld_62 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_7 : Boolean;
If_Result_20 : Boolean;
Fld_63 : Internal_Entity_Expr;
Fld_64 : Boolean;
Fld_65 : Internal_Entity_Op;
Fld_66 : Internal_Entity_Basic_Decl;
Fld_67 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_8 : Boolean;
If_Result_21 : Boolean;
Fld_68 : Internal_Entity_Expr;
Fld_69 : Boolean;
Fld_70 : Internal_Entity_Expr;
Fld_71 : Boolean;
If_Result_22 : Boolean;
Fld_72 : Internal_Entity_Elsif_Expr_Part_List;
Fld_73 : Bare_Elsif_Expr_Part_List;
Fld_74 : Internal_Entity_Info;
As_Entity_3 : Internal_Entity_Elsif_Expr_Part;
Is_Null_3 : Boolean;
If_Result_23 : Internal_Entity_Elsif_Expr_Part;
Fld_75 : Internal_Entity_Expr;
Fld_76 : Boolean;
Fld_77 : Internal_Entity_Expr;
Fld_78 : Boolean;
If_Result_24 : Boolean;
Item_229 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_229 : Bare_Elsif_Expr_Part;
Quantifier_Result_3 : Boolean;
If_Result_25 : Boolean;
Fld_79 : Internal_Entity_Expr;
Fld_80 : Boolean;
If_Result_26 : Boolean;
Fld_81 : Internal_Entity_Expr;
Fld_82 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Num_Literal;
Let_Result : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_1 : Internal_Entity_String_Literal;
Let_Result_1 : Boolean;
Scope_Result_10 : Boolean;
Cast_Result_2 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Boolean;
Scope_Result_11 : Boolean;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result_3 : Boolean;
Scope_Result_12 : Boolean;
Cast_Result_4 : Internal_Entity_Qual_Expr;
Let_Result_4 : Boolean;
Scope_Result_13 : Boolean;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_5 : Boolean;
Scope_Result_14 : Boolean;
Cast_Result_6 : Internal_Entity_Membership_Expr;
Let_Result_6 : Boolean;
Scope_Result_15 : Boolean;
Cast_Result_7 : Internal_Entity_Bin_Op;
Let_Result_7 : Boolean;
Scope_Result_16 : Boolean;
Cast_Result_8 : Internal_Entity_Un_Op;
Let_Result_8 : Boolean;
Scope_Result_17 : Boolean;
Cast_Result_9 : Internal_Entity_If_Expr;
Let_Result_9 : Boolean;
Scope_Result_18 : Boolean;
Cast_Result_10 : Internal_Entity_Paren_Expr;
Let_Result_10 : Boolean;
Scope_Result_19 : Boolean;
Cast_Result_11 : Internal_Entity_Expr;
Let_Result_11 : Boolean;
Scope_Result_20 : Boolean;
Match_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_12 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6584 '<Block at ???>' Let_Result_12 None
--# scope-start
--# expr-start 6583 '<bind at ast.py:12517>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12517
--# expr-start 6484 '<FieldAccess for origin_node at ast.py:12517>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12517
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6484
Bound_Origin := Fld; 
--# expr-start 6582 '<Match at ast.py:12517>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12517
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Int_Literal .. Ada_Real_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Num_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_9 := Let_Result;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_String_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := True; 
--# end
Scope_Result_10 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind ar Match_2
--# expr-start 6486 '<FieldAccess for is_static_expr at ast.py:12521>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12521
--# expr-start 6485 '<FieldAccess for prefix at ast.py:12521>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12521
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 6485
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 6486
if Fld_2 then
--# expr-start 6491 '<Not at ast.py:12522>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12522
--# expr-start 6488 '<FieldAccess for name_designated_type at ast.py:12522>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12522
--# expr-start 6487 '<FieldAccess for prefix at ast.py:12522>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12522
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 6487
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_4 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 6488
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 6489 '<FieldAccess for root_type at ast.py:12523>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12523
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6489
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 6490 '<FieldAccess for is_formal at ast.py:12523>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12523
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_formal
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Var_Expr.Node);
--# end
--# expr-done 6490
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Not_Val := not (Result_Var_1); 
--# expr-done 6491
if Not_Val then
   --# expr-start 6495 '<Eq at ast.py:12524>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12524
--# expr-start 6493 '<FieldAccess for name_symbol at ast.py:12524>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12524
--# expr-start 6492 '<FieldAccess for attribute at ast.py:12524>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12524
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6492
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_7.Node);
--# end
--# expr-done 6493
--# expr-start 6494 '<Symbol Base at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Base); 
--# expr-done 6494
Is_Equal := Fld_8 = Sym; 
--# expr-done 6495
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_5 := True;
else
--# expr-start 6497 '<FieldAccess for name_designated_type at ast.py:12526>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12526
--# expr-start 6496 '<FieldAccess for prefix at ast.py:12526>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12526
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 6496
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_10 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 6497
Var_Expr_2 := Fld_10;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 6498 '<FieldAccess for is_static_decl at ast.py:12527>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 6498
Scope_Result_2 := Fld_11;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
--# expr-start 6502 '<Eq at ast.py:12527>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
--# expr-start 6500 '<FieldAccess for name_symbol at ast.py:12527>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
--# expr-start 6499 '<FieldAccess for attribute at ast.py:12527>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6499
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_12.Node);
--# end
--# expr-done 6500
--# expr-start 6501 '<Symbol First at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 6501
Is_Equal_1 := Fld_13 = Sym_1; 
--# expr-done 6502
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 6506 '<Eq at ast.py:12527>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
--# expr-start 6504 '<FieldAccess for name_symbol at ast.py:12527>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
--# expr-start 6503 '<FieldAccess for attribute at ast.py:12527>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6503
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_14.Node);
--# end
--# expr-done 6504
--# expr-start 6505 '<Symbol Last at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 6505
Is_Equal_2 := Fld_15 = Sym_2; 
--# expr-done 6506
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 6510 '<Eq at ast.py:12527>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
--# expr-start 6508 '<FieldAccess for name_symbol at ast.py:12527>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
--# expr-start 6507 '<FieldAccess for attribute at ast.py:12527>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6507
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_17 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_16.Node);
--# end
--# expr-done 6508
--# expr-start 6509 '<Symbol Range at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 6509
Is_Equal_3 := Fld_17 = Sym_3; 
--# expr-done 6510
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 6514 '<Eq at ast.py:12527>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
--# expr-start 6512 '<FieldAccess for name_symbol at ast.py:12527>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
--# expr-start 6511 '<FieldAccess for attribute at ast.py:12527>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12527
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6511
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_18.Node);
--# end
--# expr-done 6512
--# expr-start 6513 '<Symbol Val at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 6513
Is_Equal_4 := Fld_19 = Sym_4; 
--# expr-done 6514
   If_Result_3 := Is_Equal_4;
end if;
   If_Result_4 := If_Result_3;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
if If_Result_5 then
   If_Result_10 := True;
else
--# expr-start 6516 '<FieldAccess for referenced_decl at ast.py:12531>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12531
--# expr-start 6515 '<FieldAccess for prefix at ast.py:12531>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12531
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 6515
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_21 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_20.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_20.Info);
--# end
--# expr-done 6516
Var_Expr_3 := Fld_21;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 6517 '<FieldAccess for is_array at ast.py:12531>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12531
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld_22 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Var_Expr_3.Node, Origin => Bound_Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 6517
Scope_Result_3 := Fld_22;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
--# expr-start 6521 '<Eq at ast.py:12532>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
--# expr-start 6519 '<FieldAccess for name_symbol at ast.py:12532>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
--# expr-start 6518 '<FieldAccess for attribute at ast.py:12532>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6518
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_24 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_23.Node);
--# end
--# expr-done 6519
--# expr-start 6520 '<Symbol First at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 6520
Is_Equal_5 := Fld_24 = Sym_5; 
--# expr-done 6521
if Is_Equal_5 then
   If_Result_6 := True;
else
   --# expr-start 6525 '<Eq at ast.py:12532>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
--# expr-start 6523 '<FieldAccess for name_symbol at ast.py:12532>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
--# expr-start 6522 '<FieldAccess for attribute at ast.py:12532>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6522
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_26 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_25.Node);
--# end
--# expr-done 6523
--# expr-start 6524 '<Symbol Last at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 6524
Is_Equal_6 := Fld_26 = Sym_6; 
--# expr-done 6525
   If_Result_6 := Is_Equal_6;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 6529 '<Eq at ast.py:12532>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
--# expr-start 6527 '<FieldAccess for name_symbol at ast.py:12532>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
--# expr-start 6526 '<FieldAccess for attribute at ast.py:12532>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6526
      if Fld_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_28 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_27.Node);
--# end
--# expr-done 6527
--# expr-start 6528 '<Symbol Length at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 6528
Is_Equal_7 := Fld_28 = Sym_7; 
--# expr-done 6529
   If_Result_7 := Is_Equal_7;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 6533 '<Eq at ast.py:12532>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
--# expr-start 6531 '<FieldAccess for name_symbol at ast.py:12532>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
--# expr-start 6530 '<FieldAccess for attribute at ast.py:12532>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12532
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 6530
      if Fld_29.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_30 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_29.Node);
--# end
--# expr-done 6531
--# expr-start 6532 '<Symbol Range at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 6532
Is_Equal_8 := Fld_30 = Sym_8; 
--# expr-done 6533
   If_Result_8 := Is_Equal_8;
end if;
   If_Result_9 := If_Result_8;
else
   If_Result_9 := False;
end if;
   If_Result_10 := If_Result_9;
end if;
   If_Result_11 := If_Result_10;
else
   If_Result_11 := False;
end if;
if If_Result_11 then
--# expr-start 6534 '<FieldAccess for args at ast.py:12535>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12535
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Assoc_List (Node => Match_2.Node.Attribute_Ref_F_Args, Info => Match_2.Info);
--# expr-done 6534
Var_Expr_4 := Fld_31;
if Var_Expr_4 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind args Var_Expr_4
--# expr-start 6537 '<AllQuantifier at ast.py:12538>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12538
Saved := Var_Expr_4; 
Fld_32 := Saved.Node;
      Quantifier_Result := True;
      if Fld_32 = null then
      null;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_32;
      begin
         for Untyped_Item_226 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_226 := Untyped_Item_226; 
Is_Null := Bare_Item_226 = null; 
if Is_Null then
   If_Result_12 := No_Entity_Basic_Assoc;
else
Fld_33 := Saved.Info;
As_Entity := (Info => Fld_33, Node => Bare_Item_226); 
   If_Result_12 := As_Entity;
end if;
                  Item_226 := If_Result_12; 
   --# scope-start
               --# bind arg Item_226
      --# expr-start 6536 '<FieldAccess for is_static_expr at ast.py:12538>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12538
--# expr-start 6535 '<FieldAccess for expr at ast.py:12538>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12538
      if Item_226.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_34 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_226.Node, E_Info => Item_226.Info);
--# end
--# expr-done 6535
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_35 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_34.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_34.Info);
--# end
--# expr-done 6536
      Quantifier_Result := Fld_35;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 6537
Scope_Result_4 := Quantifier_Result;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := True;
end if;
   If_Result_13 := Result_Var_4;
else
   If_Result_13 := False;
end if;
Let_Result_2 := If_Result_13; 
--# end
Scope_Result_11 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind ce Match_3
--# expr-start 6539 '<FieldAccess for is_static_expr at ast.py:12543>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12543
--# expr-start 6538 '<FieldAccess for name at ast.py:12543>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12543
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Create_Internal_Entity_Name (Node => Match_3.Node.Call_Expr_F_Name, Info => Match_3.Info);
--# expr-done 6538
      if Fld_36.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_37 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_36.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_36.Info);
--# end
--# expr-done 6539
if Fld_37 then
   --# expr-start 6543 '<AllQuantifier at ast.py:12544>' Quantifier_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12544
   --# expr-start 6540 '<FieldAccess for params at ast.py:12544>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12544
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_38 := Libadalang.Implementation.Call_Expr_P_Params (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 6540
Fld_39 := Fld_38.Node;
      Quantifier_Result_1 := True;
      if Fld_39 = null then
      null;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_39;
      begin
         for Untyped_Item_227 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_227 := Untyped_Item_227; 
Is_Null_1 := Bare_Item_227 = null; 
if Is_Null_1 then
   If_Result_14 := No_Entity_Basic_Assoc;
else
Fld_40 := Fld_38.Info;
As_Entity_1 := (Info => Fld_40, Node => Bare_Item_227); 
   If_Result_14 := As_Entity_1;
end if;
                  Item_227 := If_Result_14; 
   --# scope-start
               --# bind pa Item_227
      --# expr-start 6542 '<FieldAccess for is_static_expr at ast.py:12544>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12544
--# expr-start 6541 '<FieldAccess for expr at ast.py:12544>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12544
      if Item_227.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_41 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_227.Node, E_Info => Item_227.Info);
--# end
--# expr-done 6541
      if Fld_41.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_42 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_41.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_41.Info);
--# end
--# expr-done 6542
      Quantifier_Result_1 := Fld_42;
   --# end
               exit when not Quantifier_Result_1;
         end loop;
      end;
      end if;
--# expr-done 6543
   If_Result_15 := Quantifier_Result_1;
else
   If_Result_15 := False;
end if;
Let_Result_3 := If_Result_15; 
--# end
Scope_Result_12 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_12; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind qe Match_4
--# expr-start 6545 '<FieldAccess for name_designated_type at ast.py:12547>' Fld_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12547
--# expr-start 6544 '<FieldAccess for prefix at ast.py:12547>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12547
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_43 := Create_Internal_Entity_Name (Node => Match_4.Node.Qual_Expr_F_Prefix, Info => Match_4.Info);
--# expr-done 6544
      if Fld_43.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_44 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_43.Node, E_Info => Fld_43.Info);
--# end
--# expr-done 6545
Var_Expr_5 := Fld_44;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 6546 '<FieldAccess for is_static_decl at ast.py:12547>' Fld_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12547
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_45 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 6546
Scope_Result_5 := Fld_45;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
   --# expr-start 6548 '<FieldAccess for is_static_expr at ast.py:12548>' Fld_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12548
--# expr-start 6547 '<FieldAccess for suffix at ast.py:12548>' Fld_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12548
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_46 := Create_Internal_Entity_Expr (Node => Match_4.Node.Qual_Expr_F_Suffix, Info => Match_4.Info);
--# expr-done 6547
      if Fld_46.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_47 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_46.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_46.Info);
--# end
--# expr-done 6548
   If_Result_16 := Fld_47;
else
   If_Result_16 := False;
end if;
Let_Result_4 := If_Result_16; 
--# end
Scope_Result_13 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_13; 
         when Ada_Defining_Name .. Ada_Explicit_Deref | Ada_Reduce_Attribute_Ref .. Ada_Op_Xor | Ada_Null_Literal | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind n Match_5
--# expr-start 6549 '<FieldAccess for referenced_decl at ast.py:12551>' Fld_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12551
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_48 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Match_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_5.Info);
--# end
--# expr-done 6549
Var_Expr_6 := Fld_48;
if Var_Expr_6 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 6550 '<FieldAccess for is_static_decl at ast.py:12551>' Fld_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12551
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_49 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 6550
Scope_Result_6 := Fld_49;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := False;
end if;
Let_Result_5 := Result_Var_6; 
--# end
Scope_Result_14 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_14; 
         when Ada_Membership_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Membership_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_6; 
--# bind me Match_6
--# expr-start 6552 '<FieldAccess for is_static_expr at ast.py:12554>' Fld_51 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12554
--# expr-start 6551 '<FieldAccess for expr at ast.py:12554>' Fld_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12554
      if Match_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Create_Internal_Entity_Expr (Node => Match_6.Node.Membership_Expr_F_Expr, Info => Match_6.Info);
--# expr-done 6551
      if Fld_50.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_51 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_50.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_50.Info);
--# end
--# expr-done 6552
if Fld_51 then
   --# expr-start 6555 '<AllQuantifier at ast.py:12554>' Quantifier_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12554
   --# expr-start 6553 '<FieldAccess for membership_exprs at ast.py:12554>' Fld_52 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12554
      if Match_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_52 := Create_Internal_Entity_Expr_Alternatives_List (Node => Match_6.Node.Membership_Expr_F_Membership_Exprs, Info => Match_6.Info);
--# expr-done 6553
Fld_53 := Fld_52.Node;
      Quantifier_Result_2 := True;
      if Fld_53 = null then
      null;
      else
      declare
         Collection : constant Bare_Expr_Alternatives_List := Fld_53;
      begin
         for Untyped_Item_228 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_228 := Untyped_Item_228; 
Is_Null_2 := Bare_Item_228 = null; 
if Is_Null_2 then
   If_Result_17 := No_Entity_Expr;
else
Fld_54 := Fld_52.Info;
As_Entity_2 := (Info => Fld_54, Node => Bare_Item_228); 
   If_Result_17 := As_Entity_2;
end if;
                  Item_228 := If_Result_17; 
   --# scope-start
               --# bind e Item_228
      --# expr-start 6554 '<FieldAccess for is_static_expr at ast.py:12555>' Fld_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12555
      if Item_228.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_55 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Item_228.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_228.Info);
--# end
--# expr-done 6554
      Quantifier_Result_2 := Fld_55;
   --# end
               exit when not Quantifier_Result_2;
         end loop;
      end;
      end if;
--# expr-done 6555
   If_Result_18 := Quantifier_Result_2;
else
   If_Result_18 := False;
end if;
Let_Result_6 := If_Result_18; 
--# end
Scope_Result_15 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_15; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_7; 
--# bind bo Match_7
--# expr-start 6557 '<FieldAccess for is_static_expr at ast.py:12559>' Fld_57 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12559
--# expr-start 6556 '<FieldAccess for left at ast.py:12559>' Fld_56 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12559
      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_56 := Create_Internal_Entity_Expr (Node => Match_7.Node.Bin_Op_F_Left, Info => Match_7.Info);
--# expr-done 6556
      if Fld_56.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_57 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_56.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_56.Info);
--# end
--# expr-done 6557
if Fld_57 then
   --# expr-start 6559 '<FieldAccess for is_static_expr at ast.py:12560>' Fld_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12560
--# expr-start 6558 '<FieldAccess for right at ast.py:12560>' Fld_58 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12560
      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_58 := Create_Internal_Entity_Expr (Node => Match_7.Node.Bin_Op_F_Right, Info => Match_7.Info);
--# expr-done 6558
      if Fld_58.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_59 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_58.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_58.Info);
--# end
--# expr-done 6559
   If_Result_19 := Fld_59;
else
   If_Result_19 := False;
end if;
if If_Result_19 then
--# expr-start 6561 '<FieldAccess for referenced_decl at ast.py:12561>' Fld_61 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12561
--# expr-start 6560 '<FieldAccess for op at ast.py:12561>' Fld_60 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12561
      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_60 := Create_Internal_Entity_Op (Node => Match_7.Node.Bin_Op_F_Op, Info => Match_7.Info);
--# expr-done 6560
      if Fld_60.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_61 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_60.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_60.Info);
--# end
--# expr-done 6561
Var_Expr_7 := Fld_61;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr_7
--# expr-start 6562 '<FieldAccess for is_static_decl at ast.py:12562>' Fld_62 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12562
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_62 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 6562
Scope_Result_7 := Fld_62;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := True;
end if;
   If_Result_20 := Result_Var_7;
else
   If_Result_20 := False;
end if;
Let_Result_7 := If_Result_20; 
--# end
Scope_Result_16 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_16; 
         when Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Un_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_8; 
--# bind uo Match_8
--# expr-start 6564 '<FieldAccess for is_static_expr at ast.py:12567>' Fld_64 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12567
--# expr-start 6563 '<FieldAccess for expr at ast.py:12567>' Fld_63 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12567
      if Match_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_63 := Create_Internal_Entity_Expr (Node => Match_8.Node.Un_Op_F_Expr, Info => Match_8.Info);
--# expr-done 6563
      if Fld_63.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_64 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_63.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_63.Info);
--# end
--# expr-done 6564
if Fld_64 then
--# expr-start 6566 '<FieldAccess for referenced_decl at ast.py:12568>' Fld_66 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12568
--# expr-start 6565 '<FieldAccess for op at ast.py:12568>' Fld_65 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12568
      if Match_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_65 := Create_Internal_Entity_Op (Node => Match_8.Node.Un_Op_F_Op, Info => Match_8.Info);
--# expr-done 6565
      if Fld_65.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_66 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_65.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_65.Info);
--# end
--# expr-done 6566
Var_Expr_8 := Fld_66;
if Var_Expr_8 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr_8
--# expr-start 6567 '<FieldAccess for is_static_decl at ast.py:12569>' Fld_67 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12569
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_67 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_8.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 6567
Scope_Result_8 := Fld_67;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := True;
end if;
   If_Result_21 := Result_Var_8;
else
   If_Result_21 := False;
end if;
Let_Result_8 := If_Result_21; 
--# end
Scope_Result_17 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_17; 
         when Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_If_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_9 := Cast_Result_9; 
--# bind i Match_9
--# expr-start 6569 '<FieldAccess for is_static_expr at ast.py:12574>' Fld_69 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12574
--# expr-start 6568 '<FieldAccess for cond_expr at ast.py:12574>' Fld_68 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12574
      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_68 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Cond_Expr, Info => Match_9.Info);
--# expr-done 6568
      if Fld_68.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_69 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_68.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_68.Info);
--# end
--# expr-done 6569
if Fld_69 then
   --# expr-start 6571 '<FieldAccess for is_static_expr at ast.py:12574>' Fld_71 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12574
--# expr-start 6570 '<FieldAccess for then_expr at ast.py:12574>' Fld_70 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12574
      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_70 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Then_Expr, Info => Match_9.Info);
--# expr-done 6570
      if Fld_70.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_71 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_70.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_70.Info);
--# end
--# expr-done 6571
   If_Result_22 := Fld_71;
else
   If_Result_22 := False;
end if;
if If_Result_22 then
   --# expr-start 6577 '<AllQuantifier at ast.py:12575>' Quantifier_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12575
   --# expr-start 6572 '<FieldAccess for alternatives at ast.py:12575>' Fld_72 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12575
      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_72 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Match_9.Node.If_Expr_F_Alternatives, Info => Match_9.Info);
--# expr-done 6572
Fld_73 := Fld_72.Node;
      Quantifier_Result_3 := True;
      if Fld_73 = null then
      null;
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_73;
      begin
         for Untyped_Item_229 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_229 := Untyped_Item_229; 
Is_Null_3 := Bare_Item_229 = null; 
if Is_Null_3 then
   If_Result_23 := No_Entity_Elsif_Expr_Part;
else
Fld_74 := Fld_72.Info;
As_Entity_3 := (Info => Fld_74, Node => Bare_Item_229); 
   If_Result_23 := As_Entity_3;
end if;
                  Item_229 := If_Result_23; 
   --# scope-start
               --# bind a Item_229
--# expr-start 6574 '<FieldAccess for is_static_expr at ast.py:12577>' Fld_76 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12577
--# expr-start 6573 '<FieldAccess for cond_expr at ast.py:12577>' Fld_75 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12577
      if Item_229.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_75 := Create_Internal_Entity_Expr (Node => Item_229.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_229.Info);
--# expr-done 6573
      if Fld_75.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_76 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_75.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_75.Info);
--# end
--# expr-done 6574
if Fld_76 then
   --# expr-start 6576 '<FieldAccess for is_static_expr at ast.py:12578>' Fld_78 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12578
--# expr-start 6575 '<FieldAccess for then_expr at ast.py:12578>' Fld_77 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12578
      if Item_229.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_77 := Create_Internal_Entity_Expr (Node => Item_229.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_229.Info);
--# expr-done 6575
      if Fld_77.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_78 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_77.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_77.Info);
--# end
--# expr-done 6576
   If_Result_24 := Fld_78;
else
   If_Result_24 := False;
end if;
      Quantifier_Result_3 := If_Result_24;
   --# end
               exit when not Quantifier_Result_3;
         end loop;
      end;
      end if;
--# expr-done 6577
   If_Result_25 := Quantifier_Result_3;
else
   If_Result_25 := False;
end if;
if If_Result_25 then
   --# expr-start 6579 '<FieldAccess for is_static_expr at ast.py:12580>' Fld_80 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12580
--# expr-start 6578 '<FieldAccess for else_expr at ast.py:12580>' Fld_79 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12580
      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_79 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Else_Expr, Info => Match_9.Info);
--# expr-done 6578
      if Fld_79.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_80 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_79.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_79.Info);
--# end
--# expr-done 6579
   If_Result_26 := Fld_80;
else
   If_Result_26 := False;
end if;
Let_Result_9 := If_Result_26; 
--# end
Scope_Result_18 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_18; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_10 := Cast_Result_10; 
--# bind pe Match_10
--# expr-start 6581 '<FieldAccess for is_static_expr at ast.py:12582>' Fld_82 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12582
--# expr-start 6580 '<FieldAccess for expr at ast.py:12582>' Fld_81 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12582
      if Match_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_81 := Create_Internal_Entity_Expr (Node => Match_10.Node.Paren_Expr_F_Expr, Info => Match_10.Info);
--# expr-done 6580
      if Fld_81.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_82 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_81.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_81.Info);
--# end
--# expr-done 6581
Let_Result_10 := Fld_82; 
--# end
Scope_Result_19 := Let_Result_10;
   --# end
            Match_Result := Scope_Result_19; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Case_Expr | Ada_Contract_Cases .. Ada_Decl_Expr | Ada_Quantified_Expr .. Ada_Raise_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_11 := Cast_Result_11; 
--# bind _ Match_11
Let_Result_11 := False; 
--# end
Scope_Result_20 := Let_Result_11;
   --# end
            Match_Result := Scope_Result_20; 
end case;
--# expr-done 6582
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 6583
Let_Result_12 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6584
         Property_Result := Let_Result_12;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Is_Static_Expr;
--# end
--# property-start '[dispatcher]Expr.first_corresponding_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_First_Corresponding_Decl
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
                  --# property-call-start Expr.first_corresponding_decl
                  Property_Result := Expr_P_First_Corresponding_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.first_corresponding_decl
                  Property_Result := Name_P_First_Corresponding_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_First_Corresponding_Decl;
--# end
--# property-start Expr.eval_as_int /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12595
pragma Warnings (Off, "is not referenced");
 function Expr_P_Eval_As_Int
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Big_Integer_Type
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Big_Integer_Type;
            procedure Finalizer_Scope_958 with Inline_Always;
      Fld : Big_Integer_Type;
Let_Result : Big_Integer_Type;
            procedure Finalizer_Scope_958 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_958;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6586 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6585 '<FieldAccess for eval_as_int_in_env at ast.py:12612>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12612
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Eval_As_Int_In_Env (Node => Ent.Node, Env => No_Internal_Substitution_Array_Type, E_Info => Ent.Info);
--# expr-done 6585
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 6586
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_958;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_958;
            raise;
      end;
   return Property_Result;
end Expr_P_Eval_As_Int;
--# end
--# property-start Expr.eval_as_string /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12635
pragma Warnings (Off, "is not referenced");
 function Expr_P_Eval_As_String
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_961 with Inline_Always;
      Fld : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_961 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_961;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6588 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6587 '<FieldAccess for eval_as_string_in_env at ast.py:12652>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12652
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Eval_As_String_In_Env (Node => Ent.Node, Env => No_Internal_Substitution_Array_Type, E_Info => Ent.Info);
--# expr-done 6587
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 6588
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_961;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_961;
            raise;
      end;
   return Property_Result;
end Expr_P_Eval_As_String;
--# end
--# property-start Expr.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12675
pragma Warnings (Off, "is not referenced");
 function Expr_P_Discrete_Range
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Match_0 : Internal_Entity_Attribute_Ref;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Bin_Op;
Match_3 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Discrete_Range;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Discrete_Range;
Scope_Result : Internal_Discrete_Range;
Result_Var : Internal_Discrete_Range;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Internal_Entity_Expr;
New_Struct : Internal_Discrete_Range;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Attribute_Ref;
Let_Result : Internal_Discrete_Range;
Scope_Result_1 : Internal_Discrete_Range;
Cast_Result_1 : Internal_Entity_Name;
Let_Result_1 : Internal_Discrete_Range;
Scope_Result_2 : Internal_Discrete_Range;
Cast_Result_2 : Internal_Entity_Bin_Op;
Let_Result_2 : Internal_Discrete_Range;
Scope_Result_3 : Internal_Discrete_Range;
Cast_Result_3 : Internal_Entity_Expr;
Let_Result_3 : Internal_Discrete_Range;
Scope_Result_4 : Internal_Discrete_Range;
Match_Result : Internal_Discrete_Range;
Let_Result_4 : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6597 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 6596 '<Match at ast.py:12680>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12680
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ar Match_0
--# expr-start 6590 '<FieldAccess for discrete_range at ast.py:12682>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12682
--# expr-start 6589 '<FieldAccess for prefix at ast.py:12682>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12682
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Attribute_Ref_F_Prefix, Info => Match_0.Info);
--# expr-done 6589
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.discrete_range
Fld_1 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6590
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Call_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 6591 '<FieldAccess for name_designated_type at ast.py:12684>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12684
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6591
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr
--# expr-start 6592 '<FieldAccess for discrete_range at ast.py:12685>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12685
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6592
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Discrete_Range;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind bo Match_2
--# expr-start 6595 '<New DiscreteRange at ast.py:12688>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12688
--# expr-start 6593 '<FieldAccess for right at ast.py:12689>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12689
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Match_2.Node.Bin_Op_F_Right, Info => Match_2.Info);
--# expr-done 6593
--# expr-start 6594 '<FieldAccess for left at ast.py:12689>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12689
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Match_2.Node.Bin_Op_F_Left, Info => Match_2.Info);
--# expr-done 6594
New_Struct := (High_Bound => Fld_5, Low_Bound => Fld_4); 
--# expr-done 6595
Let_Result_2 := New_Struct; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := No_Discrete_Range; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 6596
Let_Result_4 := Match_Result; 
--# end
--# expr-done 6597
         Property_Result := Let_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Discrete_Range;
--# end
--# property-start '[dispatcher]Expr.designated_env_no_overloading' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.designated_env_no_overloading
                  Property_Result := Expr_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_env_no_overloading
                  Property_Result := Call_Expr_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_env_no_overloading
                  Property_Result := Dotted_Name_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_env_no_overloading
                  Property_Result := Base_Id_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Designated_Env_No_Overloading;
--# end
--# property-start '[dispatcher]Expr.designated_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Designated_Env
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.designated_env
                  Property_Result := Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.designated_env
                  Property_Result := Attribute_Ref_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_env
                  Property_Result := Call_Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_env
                  Property_Result := Dotted_Name_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.designated_env
                  Property_Result := Explicit_Deref_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.designated_env
                  Property_Result := Qual_Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_env
                  Property_Result := Base_Id_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Designated_Env;
--# end
--# property-start Expr.env_elements /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12719
pragma Warnings (Off, "is not referenced");
 function Expr_P_Env_Elements
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind no_visibility No_Visibility
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_966 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_966 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_966;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6600 '<Filter at ast.py:12720>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12720
   --# expr-start 6598 '<FieldAccess for env_elements_impl at ast.py:12720>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12720
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6598
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_230 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_230
         --# expr-start 6599 '<FieldAccess for has_visibility at ast.py:12720>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12720
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_230);
--# end
--# expr-done 6599
         if Fld_1 then
         declare
            Item_To_Append : constant Internal_Entity := Item_230;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6600
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_966;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_966;
            raise;
      end;
   return Property_Result;
end Expr_P_Env_Elements;
--# end
--# property-start '[dispatcher]Expr.env_elements_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Env_Elements_Impl
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Discrete_Subtype_Name | Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.env_elements_impl
                  Property_Result := Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.env_elements_impl
                  Property_Result := Attribute_Ref_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.env_elements_impl
                  Property_Result := Call_Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.env_elements_impl
                  Property_Result := Defining_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.env_elements_impl
                  Property_Result := Dotted_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.env_elements_impl
                  Property_Result := End_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.env_elements_impl
                  Property_Result := Explicit_Deref_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.env_elements_impl
                  Property_Result := Qual_Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Op_Xor =>
                  --# property-call-start BaseId.env_elements_impl
                  Property_Result := Base_Id_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_String_Literal =>
                  --# property-call-start StringLiteral.env_elements_impl
                  Property_Result := String_Literal_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Env_Elements_Impl;
--# end
--# property-start Expr.matching_nodes /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12734
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matching_Nodes
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_968 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_968 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_968;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6604 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6603 '<bind at ast.py:12740>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12740
--# expr-start 6601 '<FieldAccess for node_env at ast.py:12740>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12740
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 6601
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 6602 '<FieldAccess for env_elements at ast.py:12740>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12740
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Bound_Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 6602
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6603
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6604
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_968;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_968;
            raise;
      end;
   return Property_Result;
end Expr_P_Matching_Nodes;
--# end
--# property-start Expr.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12253
pragma Warnings (Off, "is not referenced");
 function Expr_P_Has_Context_Free_Type
  (Node : Bare_Expr
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6605 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := True; 
--# end
--# expr-done 6605
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Has_Context_Free_Type;
--# end
 function Expr_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Expr
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is (raise Property_Error
    with "Property Expr.potential_actuals_for_dispatch not implemented on type "
    & Kind_Name (Node));
--# property-start Expr.is_dispatching_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12484
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dispatching_Call
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6606 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := raise Property_Error with "Invalid node type: expected Name, UnOp or BinOp"; 
--# end
--# expr-done 6606
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Is_Dispatching_Call;
--# end
--# property-start Expr.first_corresponding_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12587
pragma Warnings (Off, "is not referenced");
 function Expr_P_First_Corresponding_Decl
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6607 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Entity_Basic_Decl; 
--# end
--# expr-done 6607
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_First_Corresponding_Decl;
--# end
--# property-start Expr.designated_env_no_overloading /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12694
pragma Warnings (Off, "is not referenced");
 function Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_964 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_964 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_964;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6609 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6608 '<FieldAccess for designated_env at ast.py:12703>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12703
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Ent.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Ent.Info);
--# end
--# expr-done 6608
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 6609
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_964;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_964;
            raise;
      end;
   return Property_Result;
end Expr_P_Designated_Env_No_Overloading;
--# end
 function Expr_P_Designated_Env
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is (raise Property_Error
    with "Property Expr.designated_env not implemented on type "
    & Kind_Name (Node));
 function Expr_P_Env_Elements_Impl
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is (raise Property_Error
    with "Property Expr.env_elements_impl not implemented on type "
    & Kind_Name (Node));
   --
   --  Primitives for Bare_Abstract_State_Decl_Expr
   --
      procedure Initialize_Fields_For_Abstract_State_Decl_Expr
        (Self : Bare_Abstract_State_Decl_Expr
         ; Abstract_State_Decl_Expr_F_State_Decl : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Abstract_State_Decl_Expr_F_State_Decl := Abstract_State_Decl_Expr_F_State_Decl;
      end Initialize_Fields_For_Abstract_State_Decl_Expr;
   function Abstract_State_Decl_Expr_F_State_Decl
     (Node : Bare_Abstract_State_Decl_Expr) return Bare_Ada_Node
   is
   begin
         return Node.Abstract_State_Decl_Expr_F_State_Decl;
   end;
--# property-start AbstractStateDeclExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22412
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_Expr_P_Xref_Equation
  (Node : Bare_Abstract_State_Decl_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Abstract_State_Decl_Expr  := Bare_Abstract_State_Decl_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1525 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1525 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1525;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:22412") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1525;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1525;
            raise;
      end;
   return Property_Result;
end Abstract_State_Decl_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Allocator
   --
      procedure Initialize_Fields_For_Allocator
        (Self : Bare_Allocator
         ; Allocator_F_Subpool : Bare_Name
         ; Allocator_F_Type_Or_Expr : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Allocator_F_Subpool := Allocator_F_Subpool;
            Self.Allocator_F_Type_Or_Expr := Allocator_F_Type_Or_Expr;
      end Initialize_Fields_For_Allocator;
   function Allocator_F_Subpool
     (Node : Bare_Allocator) return Bare_Name
   is
   begin
         return Node.Allocator_F_Subpool;
   end;
   function Allocator_F_Type_Or_Expr
     (Node : Bare_Allocator) return Bare_Ada_Node
   is
   begin
         return Node.Allocator_F_Type_Or_Expr;
   end;
--# property-start Allocator.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18991
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Has_Context_Free_Type
  (Node : Bare_Allocator
  )
   return Boolean
is
   Self : Bare_Allocator  := Bare_Allocator (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Allocator_P_Has_Context_Free_Type;
--# end
--# property-start Allocator.get_allocated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18993
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Get_Allocated_Type
  (Node : Bare_Allocator
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Allocator  := Bare_Allocator (Node);
     Ent : Internal_Entity_Allocator :=
       Internal_Entity_Allocator'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Qual_Expr;
Match_2 : Internal_Entity;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Qual_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6616 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 6615 '<bind at ast.py:18998>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18998
--# expr-start 6610 '<FieldAccess for origin_node at ast.py:18998>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18998
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6610
Bound_Origin := Fld; 
--# expr-start 6614 '<Match at ast.py:18998>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18998
--# expr-start 6611 '<FieldAccess for type_or_expr at ast.py:18998>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18998
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.Allocator_F_Type_Or_Expr, Info => Ent.Info);
--# expr-done 6611
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
--# expr-start 6612 '<FieldAccess for designated_type at ast.py:18999>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18999
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Bound_Origin, E_Info => Match_0.Info);
--# end
--# expr-done 6612
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind q Match_1
--# expr-start 6613 '<FieldAccess for designated_type at ast.py:19000>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19000
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start QualExpr.designated_type
Fld_3 := Libadalang.Implementation.Qual_Expr_P_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6613
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Explicit_Deref | Ada_Reduce_Attribute_Ref .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 6614
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 6615
Let_Result_3 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6616
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Allocator_P_Get_Allocated_Type;
--# end
--# property-start Allocator.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19004
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Xref_Equation
  (Node : Bare_Allocator
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Allocator  := Bare_Allocator (Node);
     Ent : Internal_Entity_Allocator :=
       Internal_Entity_Allocator'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1324 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1324 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1324;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6627 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6626 '<And at ast.py:19007>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19007
--# expr-start 6622 '<And at ast.py:19007>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19007
--# expr-start 6618 '<FieldAccess for sub_equation at ast.py:19007>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19007
--# expr-start 6617 '<FieldAccess for type_or_expr at ast.py:19007>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19007
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Allocator_F_Type_Or_Expr, Info => Ent.Info);
--# expr-done 6617
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6618
--# expr-start 6621 '<Bind at ast.py:19008>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19008
--# expr-start 6619 '<FieldAccess for expected_type_var at ast.py:19008>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19008
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6619
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 6620 '<FieldAccess for type_var at ast.py:19008>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19008
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6620
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19008") else null)); 
--# expr-done 6621
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19007") else null)); 
--# expr-done 6622
--# expr-start 6625 '<Predicate on BaseTypeDecl.matching_allocator_type at ast.py:19009>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19009
--# expr-start 6623 '<FieldAccess for type_var at ast.py:19010>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19010
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6623
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 6624 '<FieldAccess for get_allocated_type at ast.py:19010>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Allocator.get_allocated_type
Fld_5 := Libadalang.Implementation.Allocator_P_Get_Allocated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6624
Pred := Solver.Create_Predicate (Fld_4, Create_Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate (Fld_5, Origin)); 
--# expr-done 6625
And_Pred_1 := Create_And (And_Pred, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19007") else null)); 
--# expr-done 6626
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6627
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1324;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1324;
            raise;
      end;
   return Property_Result;
end Allocator_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Base_Aggregate
   --
      procedure Initialize_Fields_For_Base_Aggregate
        (Self : Bare_Base_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Base_Aggregate_F_Ancestor_Expr := Base_Aggregate_F_Ancestor_Expr;
            Self.Base_Aggregate_F_Assocs := Base_Aggregate_F_Assocs;
      end Initialize_Fields_For_Base_Aggregate;
   function Base_Aggregate_F_Ancestor_Expr
     (Node : Bare_Base_Aggregate) return Bare_Expr
   is
   begin
         return Node.Base_Aggregate_F_Ancestor_Expr;
   end;
   function Base_Aggregate_F_Assocs
     (Node : Bare_Base_Aggregate) return Bare_Assoc_List
   is
   begin
         return Node.Base_Aggregate_F_Assocs;
   end;
--# property-start BaseAggregate.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13484
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Has_Context_Free_Type
  (Node : Bare_Base_Aggregate
  )
   return Boolean
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Has_Context_Free_Type;
--# end
--# property-start BaseAggregate.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13486
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Xref_Equation
  (Node : Bare_Base_Aggregate
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1002 with Inline_Always;
            procedure Finalizer_Scope_2819 with Inline_Always;
            procedure Finalizer_Scope_2821 with Inline_Always;
      Block_Var : Logic_Equation;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Name;
True_Rel : Logic_Equation;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Sym : Symbol_Type;
Fld_4 : Boolean;
Sym_1 : Symbol_Type;
Fld_5 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_6 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Fld_7 : Boolean;
If_Result_2 : Boolean;
Sym_4 : Symbol_Type;
Fld_8 : Boolean;
If_Result_3 : Boolean;
Sym_5 : Symbol_Type;
Fld_9 : Boolean;
If_Result_4 : Boolean;
Fld_10 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_12 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_13 : Logic_Equation;
Is_A : Boolean;
Fld_14 : Logic_Equation;
Fld_15 : Logic_Equation;
Fld_16 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
If_Result_7 : Logic_Equation;
If_Result_8 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_17 : Internal_Entity_Assoc_List;
Fld_18 : Bare_Assoc_List;
Fld_19 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_9 : Internal_Entity_Basic_Assoc;
Fld_20 : Logic_Equation;
Item_231 : Internal_Entity_Basic_Assoc;
Bare_Item_231 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1002 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1002;
            procedure Finalizer_Scope_2819 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
            end Finalizer_Scope_2819;
            procedure Finalizer_Scope_2821 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_2821;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6670 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6649 '<If at ast.py:13493>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13493
--# expr-start 6629 '<FieldAccess for in_aspect at ast.py:13497>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13497
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6628 '<Symbol Global at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Global); 
--# expr-done 6628
--# property-call-start AdaNode.in_aspect
Fld_4 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym);
--# end
--# expr-done 6629
if Fld_4 then
   If_Result := True;
else
   --# expr-start 6631 '<FieldAccess for in_aspect at ast.py:13497>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13497
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6630 '<Symbol Refined_Global at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Global); 
--# expr-done 6630
--# property-call-start AdaNode.in_aspect
Fld_5 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_1);
--# end
--# expr-done 6631
   If_Result := Fld_5;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 6633 '<FieldAccess for in_aspect at ast.py:13498>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13498
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6632 '<Symbol Depends at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Depends); 
--# expr-done 6632
--# property-call-start AdaNode.in_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_2);
--# end
--# expr-done 6633
   If_Result_1 := Fld_6;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 6635 '<FieldAccess for in_aspect at ast.py:13498>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13498
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6634 '<Symbol Refined_Depends at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Depends); 
--# expr-done 6634
--# property-call-start AdaNode.in_aspect
Fld_7 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_3);
--# end
--# expr-done 6635
   If_Result_2 := Fld_7;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 6637 '<FieldAccess for in_aspect at ast.py:13499>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13499
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6636 '<Symbol Test_Case at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 6636
--# property-call-start AdaNode.in_aspect
Fld_8 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_4);
--# end
--# expr-done 6637
   If_Result_3 := Fld_8;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 6639 '<FieldAccess for in_aspect at ast.py:13499>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13499
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6638 '<Symbol Refined_State at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_State); 
--# expr-done 6638
--# property-call-start AdaNode.in_aspect
Fld_9 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_5);
--# end
--# expr-done 6639
   If_Result_4 := Fld_9;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 6640 '<FieldAccess for is_contract_cases_base_aggregate at ast.py:13503>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13503
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.is_contract_cases_base_aggregate
Fld_10 := Libadalang.Implementation.Base_Aggregate_P_Is_Contract_Cases_Base_Aggregate (Node => Self);
--# end
--# expr-done 6640
   If_Result_5 := Fld_10;
end if;
if If_Result_5 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13504") else null)); 
   If_Result_6 := True_Rel;
else
   --# expr-start 6648 '<bind at ast.py:13505>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13505
--# expr-start 6641 '<FieldAccess for origin_node at ast.py:13506>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13506
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6641
Bound_Origin := Fld; 
--# expr-start 6647 '<And at ast.py:13507>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13507
--# expr-start 6643 '<Predicate on BaseTypeDecl.is_array_or_rec at ast.py:13507>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13507
--# expr-start 6642 '<FieldAccess for expected_type_var at ast.py:13507>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13507
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6642
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_Predicate (Fld_1, Create_Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate (Bound_Origin)); 
--# expr-done 6643
--# expr-start 6646 '<Bind at ast.py:13508>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13508
--# expr-start 6644 '<FieldAccess for expected_type_var at ast.py:13508>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13508
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6644
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 6645 '<FieldAccess for type_var at ast.py:13508>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13508
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6645
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13508") else null)); 
--# expr-done 6646
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13507") else null)); 
--# expr-done 6647
Dyn_Var_Bind_Result := And_Pred; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6648
   If_Result_6 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 6649
Block_Var := If_Result_6; Inc_Ref (Block_Var);
--# bind type_constraint Block_Var
--# expr-start 6669 '<And at ast.py:13512>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13512
--# expr-start 6664 '<And at ast.py:13512>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13512
--# expr-start 6650 '<FieldAccess for ancestor_expr at ast.py:13512>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13512
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Expr (Node => Ent.Node.Base_Aggregate_F_Ancestor_Expr, Info => Ent.Info);
--# expr-done 6650
Var_Expr := Fld_11;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind ae Var_Expr
--# expr-start 6663 '<Cond at ast.py:13516>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13516
--# expr-start 6654 '<Not at ast.py:13517>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13517
--# expr-start 6653 '<IsNull at ast.py:13517>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13517
--# expr-start 6651 '<Cast to Name at ast.py:13517>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13517
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 6651
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
--# expr-start 6652 '<FieldAccess for name_designated_type at ast.py:13517>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13517
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_12 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6652
Scope_Result := Fld_12;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Is_Null := Result_Var.Node = null; 
--# expr-done 6653
Not_Val := not (Is_Null); 
--# expr-done 6654
if Not_Val then
   --# expr-start 6656 '<FieldAccess for xref_no_overloading at ast.py:13518>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13518
--# expr-start 6655 '<Cast to Name at ast.py:13518>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13518
Cast_Expr_1 := Var_Expr; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 6655
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_13 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 6656
   If_Result_8 := Fld_13;
else
--# expr-start 6657 '<IsA DeltaAggregate at ast.py:13520>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13520
Is_A := Self /= null 
and then Self.Kind in Ada_Delta_Aggregate_Range; 
--# expr-done 6657
if Is_A then
   --# expr-start 6658 '<FieldAccess for sub_equation at ast.py:13521>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13521
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6658
   If_Result_7 := Fld_14;
else
   --# expr-start 6662 '<And at ast.py:13528>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13528
--# expr-start 6659 '<FieldAccess for sub_equation at ast.py:13528>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13528
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6659
--# expr-start 6661 '<Predicate on BaseTypeDecl.is_tagged_type_with_deref at ast.py:13529>' Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13529
--# expr-start 6660 '<FieldAccess for type_var at ast.py:13530>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13530
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 6660
Fld_16.Value := No_Entity;
Entity_Vars.Reset (Fld_16);
Pred_1 := Solver.Create_Predicate (Fld_16, Create_Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate (Origin)); 
--# expr-done 6661
And_Pred_1 := Create_And (Fld_15, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13528") else null)); 
--# expr-done 6662
   If_Result_7 := And_Pred_1;
end if;
      Inc_Ref (If_Result_7);
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 6663
Scope_Result_1 := If_Result_8;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2819;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13532") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_2 := Create_And (Block_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13512") else null)); 
--# expr-done 6664
--# expr-start 6668 '<LogicAll at ast.py:13533>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13533
--# expr-start 6667 '<Map at ast.py:13533>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13533
   --# expr-start 6665 '<FieldAccess for assocs at ast.py:13533>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13533
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 6665
Fld_18 := Fld_17.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_18 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_18;
      begin
         for Untyped_Item_231 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_231 := Untyped_Item_231; 
Is_Null_1 := Bare_Item_231 = null; 
if Is_Null_1 then
   If_Result_9 := No_Entity_Basic_Assoc;
else
Fld_19 := Fld_17.Info;
As_Entity := (Info => Fld_19, Node => Bare_Item_231); 
   If_Result_9 := As_Entity;
end if;
                  Item_231 := If_Result_9; 
   --# scope-start
               --# bind assoc Item_231
      --# expr-start 6666 '<FieldAccess for sub_equation at ast.py:13534>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13534
      if Item_231.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_231.Node, Env => Env, Origin => Origin, E_Info => Item_231.Info);
--# end
--# expr-done 6666
         declare
            Item_To_Append : constant Logic_Equation := Fld_20;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2821;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6667
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13533") else null)); 
--# expr-done 6668
And_Pred_3 := Create_And (And_Pred_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13512") else null)); 
--# expr-done 6669
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 6670
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1002;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1002;
                     Finalizer_Scope_2819;
                     Finalizer_Scope_2821;
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Xref_Equation;
--# end
--# property-start BaseAggregate.is_contract_cases_base_aggregate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13537
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Is_Contract_Cases_Base_Aggregate
  (Node : Bare_Base_Aggregate
  )
   return Boolean
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Aspect_Assoc;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld_1 : Bare_Name;
Sym : Symbol_Type;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6676 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6672 '<Cast to AspectAssoc at ast.py:13543>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13543
--# expr-start 6671 '<parent at ast.py:13543>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13543
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6671
   if Fld = null
      or else Fld.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6672
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind aspect Var_Expr
--# expr-start 6675 '<FieldAccess for name_is at ast.py:13544>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13544
--# expr-start 6673 '<FieldAccess for id at ast.py:13544>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13544
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 6673
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6674 '<Symbol Contract_Cases at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 6674
--# property-call-start Name.name_is
Fld_2 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1, Sym => Sym);
--# end
--# expr-done 6675
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 6676
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Is_Contract_Cases_Base_Aggregate;
--# end
--# property-start BaseAggregate.multidim_root_aggregate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13547
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Multidim_Root_Aggregate
  (Node : Bare_Base_Aggregate
      ; R : Integer
         := 0
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Multidim_Aggregate_Info
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind r R
   --# bind origin Origin
   Property_Result : Internal_Multidim_Aggregate_Info;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Aggregate;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
New_Struct : Internal_Multidim_Aggregate_Info;
Fld_1 : Integer;
Comp_Result : Boolean;
If_Result : Internal_Multidim_Aggregate_Info;
Scope_Result : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Aggregate;
Arith_Result : Integer;
Fld_5 : Internal_Multidim_Aggregate_Info;
Scope_Result_1 : Internal_Multidim_Aggregate_Info;
Result_Var : Internal_Multidim_Aggregate_Info;
Result_Var_1 : Internal_Multidim_Aggregate_Info;
Let_Result : Internal_Multidim_Aggregate_Info;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Multidim_Root_Aggregate,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Integer,
                                      As_Integer => R);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Multidim_Aggregate_Info;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6688 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6678 '<Cast to BaseTypeDecl at ast.py:13556>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13556
--# expr-start 6677 '<FieldAccess for type_val at ast.py:13556>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13556
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Ent.Node);
--# end
--# expr-done 6677
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6678
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind tv Var_Expr
--# expr-start 6681 '<If at ast.py:13557>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13557
--# expr-start 6679 '<FieldAccess for array_ndims at ast.py:13560>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13560
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6679
Comp_Result := Fld_1 > 1; 
if Comp_Result then
   --# expr-start 6680 '<New MultidimAggregateInfo at ast.py:13561>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13561
New_Struct := (Agg => Ent, Rank => R, Typ => Var_Expr); 
--# expr-done 6680
   If_Result := New_Struct;
else
   If_Result := No_Multidim_Aggregate_Info;
end if;
--# expr-done 6681
Scope_Result := If_Result;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 6685 '<Cast to Aggregate at ast.py:13569>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13569
--# expr-start 6684 '<parent at ast.py:13569>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13569
--# expr-start 6683 '<parent at ast.py:13569>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13569
--# expr-start 6682 '<parent at ast.py:13569>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13569
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6682
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 6683
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# expr-done 6684
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 6685
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Aggregate then
   --# scope-start
--# expr-start 6687 '<FieldAccess for multidim_root_aggregate at ast.py:13570>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13570
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6686 '<Arithmetic + at ast.py:13570>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13570
Arith_Result := (R + 1); 
--# expr-done 6686
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_5 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Var_Expr_1.Node, R => Arith_Result, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6687
Scope_Result_1 := Fld_5;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Multidim_Aggregate_Info;
end if;
   Result_Var_1 := Result_Var;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 6688
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Multidim_Aggregate_Info,
                        As_Internal_Multidim_Aggregate_Info => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_Multidim_Root_Aggregate;
--# end
--# property-start BaseAggregate.all_discriminants /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13573
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_All_Discriminants
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1005 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Type_Decl;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Fld_4 : Internal_Entity_Type_Decl;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1005 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1005;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_All_Discriminants,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6696 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6690 '<Cast to BaseTypeDecl at ast.py:13592>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13592
--# expr-start 6689 '<FieldAccess for type_val at ast.py:13592>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13592
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Self);
--# end
--# expr-done 6689
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6690
Block_Var := Cast_Result; 
--# bind td Block_Var
--# expr-start 6691 '<FieldAccess for ancestor_expr_type at ast.py:13593>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13593
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.ancestor_expr_type
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Ancestor_Expr_Type (Node => Ent.Node, Resolve_Type => False, E_Info => Ent.Info);
--# end
--# expr-done 6691
Block_Var_1 := Fld_1; 
--# bind stop_recurse_at Block_Var_1
--# expr-start 6694 '<FieldAccess for type_decl at ast.py:13594>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13594
--# expr-start 6693 '<FieldAccess for comps at ast.py:13594>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13594
--# expr-start 6692 '<FieldAccess for record_def at ast.py:13594>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13594
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6692
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6693
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_4 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 6694
Block_Var_2 := Fld_4; 
--# bind record_decl Block_Var_2
--# expr-start 6695 '<FieldAccess for discriminants_list at ast.py:13595>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13595
      if Block_Var_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var_2.Node, Stop_Recurse_At => Block_Var_1, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 6695
Let_Result := Fld_5; Inc_Ref (Let_Result);
--# end
--# expr-done 6696
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1005;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1005;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access,
                        As_Internal_Entity_Base_Formal_Param_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_All_Discriminants;
--# end
--# property-start BaseAggregate.all_components /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13597
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_All_Components
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1006 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Component_List;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Assoc_List;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Is_A : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1006 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1006;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_All_Components,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (3).As_Lexical_Env);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6707 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6698 '<Cast to BaseTypeDecl at ast.py:13609>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13609
--# expr-start 6697 '<FieldAccess for type_val at ast.py:13609>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13609
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Self);
--# end
--# expr-done 6697
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6698
Block_Var := Cast_Result; 
--# bind td Block_Var
--# expr-start 6699 '<FieldAccess for ancestor_expr_type at ast.py:13610>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13610
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.ancestor_expr_type
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Ancestor_Expr_Type (Node => Ent.Node, Resolve_Type => False, E_Info => Ent.Info);
--# end
--# expr-done 6699
Block_Var_1 := Fld_1; 
--# bind stop_recurse_at Block_Var_1
--# expr-start 6701 '<FieldAccess for comps at ast.py:13611>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13611
--# expr-start 6700 '<FieldAccess for record_def at ast.py:13611>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13611
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6700
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6701
Block_Var_2 := Fld_3; 
--# bind comp_list Block_Var_2
--# expr-start 6706 '<If at ast.py:13612>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13612
--# expr-start 6702 '<IsA DeltaAggregate at ast.py:13613>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13613
Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Delta_Aggregate_Range; 
--# expr-done 6702
if Is_A then
   --# expr-start 6703 '<FieldAccess for abstract_formal_params_for_delta_assocs at ast.py:13616>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13616
      if Block_Var_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_for_delta_assocs
Fld_4 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Delta_Assocs (Node => Block_Var_2.Node, E_Info => Block_Var_2.Info);
--# end
--# expr-done 6703
   If_Result := Fld_4;
else
   --# expr-start 6705 '<FieldAccess for abstract_formal_params_for_assocs at ast.py:13617>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13617
      if Block_Var_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6704 '<FieldAccess for assocs at ast.py:13617>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13617
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 6704
--# property-call-start ComponentList.abstract_formal_params_for_assocs
Fld_6 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Assocs (Node => Block_Var_2.Node, Assocs => Fld_5, Stop_Recurse_At => Block_Var_1, Env => Env, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 6705
   If_Result := Fld_6;
end if;
      Inc_Ref (If_Result);
--# expr-done 6706
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6707
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1006;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1006;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access,
                        As_Internal_Entity_Base_Formal_Param_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_All_Components;
--# end
--# property-start BaseAggregate.matched_discriminants /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13621
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Matched_Discriminants
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_1007 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_1007 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1007;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Matched_Discriminants,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6711 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6710 '<FieldAccess for match_formals at ast.py:13632>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13632
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6708 '<FieldAccess for all_discriminants at ast.py:13633>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13633
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_discriminants
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Discriminants (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6708
--# expr-start 6709 '<FieldAccess for assocs at ast.py:13633>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13633
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 6709
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Fld_1, Is_Dottable_Subp => False);
--# end
--# expr-done 6710
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6711
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1007;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1007;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                        As_Internal_Param_Match_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_Matched_Discriminants;
--# end
--# property-start BaseAggregate.matched_components /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13636
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Matched_Components
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_1008 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_1008 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1008;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Matched_Components,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (3).As_Lexical_Env);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6715 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6714 '<FieldAccess for match_formals at ast.py:13647>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13647
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6712 '<FieldAccess for all_components at ast.py:13648>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13648
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_components
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6712
--# expr-start 6713 '<FieldAccess for assocs at ast.py:13648>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13648
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 6713
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Fld_1, Is_Dottable_Subp => False);
--# end
--# expr-done 6714
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6715
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1008;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1008;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                        As_Internal_Param_Match_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_Matched_Components;
--# end
--# property-start BaseAggregate.first_unmatched_formal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13651
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_First_Unmatched_Formal
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1009 with Inline_Always;
            procedure Finalizer_Scope_2825 with Inline_Always;
            procedure Finalizer_Scope_2827 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_2 : Internal_Param_Match_Array_Access;
Fld_3 : Internal_Entity_Defining_Name;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Defining_Name_Array_Access;
Fld_6 : Internal_Param_Match_Array_Access;
Fld_7 : Internal_Entity_Defining_Name;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Quantifier_Result_1 : Boolean;
Not_Val_1 : Boolean;
Find_Result_1 : Internal_Entity_Defining_Name;
Is_Null : Boolean;
Not_Val_2 : Boolean;
If_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1009 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1009;
            procedure Finalizer_Scope_2825 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2825;
            procedure Finalizer_Scope_2827 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2827;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6735 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6723 '<Find at ast.py:13664>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13664
   --# expr-start 6717 '<FieldAccess for unpack_formals at ast.py:13664>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13664
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6716 '<FieldAccess for all_discriminants at ast.py:13664>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13664
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_discriminants
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Discriminants (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6716
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 6717
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_1;
      begin
         for Item_232 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_232
      --# expr-start 6722 '<Not at ast.py:13665>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13665
--# expr-start 6721 '<AnyQuantifier at ast.py:13665>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13665
   --# expr-start 6718 '<FieldAccess for matched_discriminants at ast.py:13665>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13665
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.matched_discriminants
Fld_2 := Libadalang.Implementation.Base_Aggregate_P_Matched_Discriminants (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6718
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_2;
      begin
         for Item_233 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_233
      --# expr-start 6720 '<Eq at ast.py:13666>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13666
--# expr-start 6719 '<FieldAccess for formal at ast.py:13666>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13666
Fld_3 := Item_233.Formal;
--# expr-done 6719
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr := Item_232; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 6720
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6721
Not_Val := not (Quantifier_Result); 
--# expr-done 6722
      if Not_Val then
         Find_Result := Item_232; 
      end if;
   --# end
      Finalizer_Scope_2825;
               exit when Not_Val;
         end loop;
      end;
--# expr-done 6723
Block_Var := Find_Result; 
--# bind unmatched_discr Block_Var
--# expr-start 6734 '<If at ast.py:13670>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13670
--# expr-start 6725 '<Not at ast.py:13671>' Not_Val_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13671
--# expr-start 6724 '<IsNull at ast.py:13671>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13671
Is_Null := Block_Var.Node = null; 
--# expr-done 6724
Not_Val_2 := not (Is_Null); 
--# expr-done 6725
if Not_Val_2 then
   If_Result := Block_Var;
else
   --# expr-start 6733 '<Find at ast.py:13680>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13680
   --# expr-start 6727 '<FieldAccess for unpack_formals at ast.py:13680>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13680
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6726 '<FieldAccess for all_components at ast.py:13680>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13680
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_components
Fld_4 := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6726
--# property-call-start AdaNode.unpack_formals
Fld_5 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld_4);
--# end
--# expr-done 6727
      Find_Result_1 := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_5;
      begin
         for Item_234 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_234
      --# expr-start 6732 '<Not at ast.py:13681>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13681
--# expr-start 6731 '<AnyQuantifier at ast.py:13681>' Quantifier_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13681
   --# expr-start 6728 '<FieldAccess for matched_components at ast.py:13681>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13681
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.matched_components
Fld_6 := Libadalang.Implementation.Base_Aggregate_P_Matched_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6728
      Quantifier_Result_1 := False;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_6;
      begin
         for Item_235 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_235
      --# expr-start 6730 '<Eq at ast.py:13682>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13682
--# expr-start 6729 '<FieldAccess for formal at ast.py:13682>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13682
Fld_7 := Item_235.Formal;
--# expr-done 6729
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Cast_Expr_1 := Item_234; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 6730
      Quantifier_Result_1 := Is_Equiv_1;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 6731
Not_Val_1 := not (Quantifier_Result_1); 
--# expr-done 6732
      if Not_Val_1 then
         Find_Result_1 := Item_234; 
      end if;
   --# end
      Finalizer_Scope_2827;
               exit when Not_Val_1;
         end loop;
      end;
--# expr-done 6733
   If_Result := Find_Result_1;
end if;
--# expr-done 6734
Let_Result := If_Result; 
--# end
--# expr-done 6735
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1009;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1009;
                     Finalizer_Scope_2825;
                     Finalizer_Scope_2827;
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_First_Unmatched_Formal;
--# end
--# property-start BaseAggregate.aggregate_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13687
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Aggregate_Params
  (Node : Bare_Base_Aggregate
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1011 with Inline_Always;
      Fld : Internal_Entity_Assoc_List;
Fld_1 : Internal_Param_Actual_Array_Access;
Let_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1011 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1011;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6738 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6737 '<FieldAccess for zip_with_params at ast.py:13693>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13693
--# expr-start 6736 '<FieldAccess for assocs at ast.py:13693>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13693
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 6736
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.zip_with_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Fld.Node, Imprecise_Fallback => False, E_Info => Fld.Info);
--# end
--# expr-done 6737
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6738
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1011;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1011;
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Aggregate_Params;
--# end
--# property-start BaseAggregate.is_subaggregate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13695
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Is_Subaggregate
  (Node : Bare_Base_Aggregate
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Block_Var : Boolean;
Fld : Boolean;
Cast_Expr : Bare_Base_Aggregate;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Integer;
Comp_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6743 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6739 '<FieldAccess for resolve_names_from_closest_entry_point at ast.py:13703>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13703
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6739
Block_Var := Fld; 
--# expr-start 6742 '<bind at ast.py:13704>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13704
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 6741 '<FieldAccess for rank at ast.py:13704>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13704
--# expr-start 6740 '<FieldAccess for multidim_root_aggregate at ast.py:13704>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13704
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 6740
Fld_2 := Fld_1.Rank;
--# expr-done 6741
Comp_Result := Fld_2 > 0; 
Dyn_Var_Bind_Result := Comp_Result; 
--# expr-done 6742
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6743
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Is_Subaggregate;
--# end
--# property-start BaseAggregate.ancestor_expr_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13706
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Ancestor_Expr_Type
  (Node : Bare_Base_Aggregate
      ; Resolve_Type : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind resolve_type Resolve_Type
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Name;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6751 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6744 '<FieldAccess for ancestor_expr at ast.py:13716>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13716
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Base_Aggregate_F_Ancestor_Expr, Info => Ent.Info);
--# expr-done 6744
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind ae Var_Expr
--# expr-start 6745 '<Cast to Name at ast.py:13717>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13717
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 6745
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
--# expr-start 6746 '<FieldAccess for name_designated_type at ast.py:13717>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13717
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 6746
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 6750 '<If at ast.py:13718>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13718
if Resolve_Type then
   --# expr-start 6747 '<FieldAccess for expression_type at ast.py:13719>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13719
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6747
   If_Result := Fld_2;
else
   --# expr-start 6749 '<Cast to BaseTypeDecl at ast.py:13720>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13720
--# expr-start 6748 '<FieldAccess for type_val at ast.py:13720>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13720
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Var_Expr.Node);
--# end
--# expr-done 6748
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6749
   If_Result := Cast_Result_1;
end if;
--# expr-done 6750
   Result_Var_1 := If_Result;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 6751
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Ancestor_Expr_Type;
--# end
   --
   --  Primitives for Bare_Aggregate
   --
      procedure Initialize_Fields_For_Aggregate
        (Self : Bare_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Aggregate;
   --
   --  Primitives for Bare_Bracket_Aggregate
   --
      procedure Initialize_Fields_For_Bracket_Aggregate
        (Self : Bare_Bracket_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Bracket_Aggregate;
   --
   --  Primitives for Bare_Delta_Aggregate
   --
      procedure Initialize_Fields_For_Delta_Aggregate
        (Self : Bare_Delta_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Delta_Aggregate;
   --
   --  Primitives for Bare_Bracket_Delta_Aggregate
   --
      procedure Initialize_Fields_For_Bracket_Delta_Aggregate
        (Self : Bare_Bracket_Delta_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Delta_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Bracket_Delta_Aggregate;
   --
   --  Primitives for Bare_Null_Record_Aggregate
   --
      procedure Initialize_Fields_For_Null_Record_Aggregate
        (Self : Bare_Null_Record_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Null_Record_Aggregate;
   --
   --  Primitives for Bare_Bin_Op
   --
      procedure Initialize_Fields_For_Bin_Op
        (Self : Bare_Bin_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Bin_Op_F_Left := Bin_Op_F_Left;
            Self.Bin_Op_F_Op := Bin_Op_F_Op;
            Self.Bin_Op_F_Right := Bin_Op_F_Right;
      end Initialize_Fields_For_Bin_Op;
   function Bin_Op_F_Left
     (Node : Bare_Bin_Op) return Bare_Expr
   is
   begin
         return Node.Bin_Op_F_Left;
   end;
   function Bin_Op_F_Op
     (Node : Bare_Bin_Op) return Bare_Op
   is
   begin
         return Node.Bin_Op_F_Op;
   end;
   function Bin_Op_F_Right
     (Node : Bare_Bin_Op) return Bare_Expr
   is
   begin
         return Node.Bin_Op_F_Right;
   end;
--# property-start BinOp.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12877
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Has_Context_Free_Type
  (Node : Bare_Bin_Op
  )
   return Boolean
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Op;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6754 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6753 '<IsA Op.AndThen, Op.OrElse at ast.py:12879>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12879
--# expr-start 6752 '<FieldAccess for op at ast.py:12879>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12879
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Bin_Op_F_Op;
--# expr-done 6752
Is_A := Fld /= null 
and then Fld.Kind in Ada_Op_And_Then_Range | Ada_Op_Or_Else_Range; 
--# expr-done 6753
Let_Result := Is_A; 
--# end
--# expr-done 6754
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Has_Context_Free_Type;
--# end
--# property-start BinOp.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12881
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Xref_Equation
  (Node : Bare_Bin_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_981 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Bare_Op;
Is_A : Boolean;
Fld_5 : Logic_Equation;
Fld_6 : Bare_Op;
Is_A_1 : Boolean;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Equation;
Or_Pred : Logic_Equation;
If_Result : Logic_Equation;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_981 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_981;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6771 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6770 '<And at ast.py:12883>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12883
--# expr-start 6759 '<And at ast.py:12883>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12883
--# expr-start 6756 '<FieldAccess for sub_equation at ast.py:12884>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12884
--# expr-start 6755 '<FieldAccess for left at ast.py:12884>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12884
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 6755
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6756
--# expr-start 6758 '<FieldAccess for sub_equation at ast.py:12885>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12885
--# expr-start 6757 '<FieldAccess for right at ast.py:12885>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12885
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 6757
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6758
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12883") else null)); 
--# expr-done 6759
--# expr-start 6769 '<Cond at ast.py:12886>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12886
--# expr-start 6761 '<IsA Op.DoubleDot at ast.py:12887>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12887
--# expr-start 6760 '<FieldAccess for op at ast.py:12887>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12887
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Bin_Op_F_Op;
--# expr-done 6760
Is_A := Fld_4 /= null 
and then Fld_4.Kind in Ada_Op_Double_Dot_Range; 
--# expr-done 6761
if Is_A then
   --# expr-start 6762 '<FieldAccess for double_dot_equation at ast.py:12888>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12888
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.double_dot_equation
Fld_5 := Libadalang.Implementation.Bin_Op_P_Double_Dot_Equation (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6762
   If_Result_1 := Fld_5;
else
--# expr-start 6764 '<IsA Op.AndThen, Op.OrElse at ast.py:12890>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12890
--# expr-start 6763 '<FieldAccess for op at ast.py:12890>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12890
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Bin_Op_F_Op;
--# expr-done 6763
Is_A_1 := Fld_6 /= null 
and then Fld_6.Kind in Ada_Op_And_Then_Range | Ada_Op_Or_Else_Range; 
--# expr-done 6764
if Is_A_1 then
   --# expr-start 6765 '<FieldAccess for test_eq at ast.py:12891>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12891
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.test_eq
Fld_7 := Libadalang.Implementation.Bin_Op_P_Test_Eq (Node => Self, Origin => Origin);
--# end
--# expr-done 6765
   If_Result := Fld_7;
else
   --# expr-start 6768 '<Or at ast.py:12893>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12893
--# expr-start 6766 '<FieldAccess for overload_equation at ast.py:12893>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12893
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.overload_equation
Fld_8 := Libadalang.Implementation.Bin_Op_P_Overload_Equation (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6766
--# expr-start 6767 '<FieldAccess for no_overload_equation at ast.py:12893>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12893
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BinOp.no_overload_equation'
Fld_9 := Libadalang.Implementation.Dispatcher_Bin_Op_P_No_Overload_Equation (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6767
Or_Pred := Create_Or (Fld_8, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12893") else null)); 
--# expr-done 6768
   If_Result := Or_Pred;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6769
And_Pred_1 := Create_And (And_Pred, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12883") else null)); 
--# expr-done 6770
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6771
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_981;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_981;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Xref_Equation;
--# end
--# property-start BinOp.double_dot_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12896
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Double_Dot_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_982 with Inline_Always;
      True_Rel : Logic_Equation;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Bind_Result_4 : Logic_Equation;
And_Pred_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_13 : Bare_Expr;
Fld_14 : Logic_Equation;
Fld_15 : Bare_Expr;
Fld_16 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_17 : Logic_Var;
Fld_18 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_982 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Or_Pred_2);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_982;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6806 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6805 '<And at ast.py:12898>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12898
--# expr-start 6796 '<And at ast.py:12898>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12898
--# expr-start 6779 '<Or at ast.py:12898>' Or_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12898
--# expr-start 6775 '<Or at ast.py:12898>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12898
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12899") else null)); 
--# expr-start 6774 '<Bind at ast.py:12900>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12900
--# expr-start 6772 '<FieldAccess for expected_type_var at ast.py:12900>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12900
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6772
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 6773 '<FieldAccess for root_int_type at ast.py:12900>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12900
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_int_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Root_Int_Type (Node => Self);
--# end
--# expr-done 6773
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12900") else null)); 
--# expr-done 6774
Or_Pred := Create_Or (True_Rel, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12898") else null)); 
--# expr-done 6775
--# expr-start 6778 '<Bind at ast.py:12901>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12901
--# expr-start 6776 '<FieldAccess for expected_type_var at ast.py:12901>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12901
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6776
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 6777 '<FieldAccess for int_type at ast.py:12901>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12901
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 6777
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_2, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12901") else null)); 
--# expr-done 6778
Or_Pred_1 := Create_Or (Or_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12898") else null)); 
--# expr-done 6779
--# expr-start 6795 '<And at ast.py:12902>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12902
--# expr-start 6790 '<And at ast.py:12902>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12902
--# expr-start 6785 '<And at ast.py:12902>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12902
--# expr-start 6781 '<Predicate on AdaNode.is_not_null at ast.py:12903>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12903
--# expr-start 6780 '<FieldAccess for expected_type_var at ast.py:12903>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12903
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6780
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_Predicate (Fld_4, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 6781
--# expr-start 6784 '<Bind at ast.py:12904>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12904
--# expr-start 6782 '<FieldAccess for expected_type_var at ast.py:12904>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12904
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6782
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 6783 '<FieldAccess for type_var at ast.py:12904>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12904
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6783
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Bind_Result_2 := Solver.Create_Unify (Fld_5, Fld_6, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12904") else null)); 
--# expr-done 6784
And_Pred := Create_And (Pred, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12902") else null)); 
--# expr-done 6785
--# expr-start 6789 '<Bind at ast.py:12905>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12905
--# expr-start 6786 '<FieldAccess for type_var at ast.py:12905>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12905
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6786
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 6788 '<FieldAccess for expected_type_var at ast.py:12905>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12905
--# expr-start 6787 '<FieldAccess for left at ast.py:12905>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12905
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Bin_Op_F_Left;
--# expr-done 6787
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_8);
--# expr-done 6788
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
Bind_Result_3 := Solver.Create_Unify (Fld_7, Fld_9, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12905") else null)); 
--# expr-done 6789
And_Pred_1 := Create_And (And_Pred, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12902") else null)); 
--# expr-done 6790
--# expr-start 6794 '<Bind at ast.py:12906>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12906
--# expr-start 6791 '<FieldAccess for type_var at ast.py:12906>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12906
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6791
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 6793 '<FieldAccess for expected_type_var at ast.py:12906>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12906
--# expr-start 6792 '<FieldAccess for right at ast.py:12906>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12906
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Bin_Op_F_Right;
--# expr-done 6792
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_11);
--# expr-done 6793
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_4 := Solver.Create_Unify (Fld_10, Fld_12, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12906") else null)); 
--# expr-done 6794
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12902") else null)); 
--# expr-done 6795
And_Pred_3 := Create_And (Or_Pred_1, And_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12898") else null)); 
--# expr-done 6796
--# expr-start 6804 '<Or at ast.py:12907>' Or_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12907
--# expr-start 6801 '<And at ast.py:12912>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12912
--# expr-start 6798 '<FieldAccess for matches_expected_formal_type at ast.py:12912>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12912
--# expr-start 6797 '<FieldAccess for left at ast.py:12912>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12912
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Self.Bin_Op_F_Left;
--# expr-done 6797
      if Fld_13 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_14 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_13, Origin => Origin);
--# end
--# expr-done 6798
--# expr-start 6800 '<FieldAccess for matches_expected_formal_type at ast.py:12913>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12913
--# expr-start 6799 '<FieldAccess for right at ast.py:12913>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12913
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Self.Bin_Op_F_Right;
--# expr-done 6799
      if Fld_15 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_16 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_15, Origin => Origin);
--# end
--# expr-done 6800
And_Pred_4 := Create_And (Fld_14, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12912") else null)); 
--# expr-done 6801
--# expr-start 6803 '<FieldAccess for numeric_type_from_operands_equation at ast.py:12917>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12917
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6802 '<FieldAccess for type_var at ast.py:12917>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12917
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6802
--# property-call-start BinOp.numeric_type_from_operands_equation
Fld_18 := Libadalang.Implementation.Bin_Op_P_Numeric_Type_From_Operands_Equation (Node => Ent.Node, Dest_Var => Fld_17, Origin => Origin);
--# end
--# expr-done 6803
Or_Pred_2 := Create_Or (And_Pred_4, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12907") else null)); 
--# expr-done 6804
And_Pred_5 := Create_And (And_Pred_3, Or_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12898") else null)); 
--# expr-done 6805
Let_Result := And_Pred_5; Inc_Ref (Let_Result);
--# end
--# expr-done 6806
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_982;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_982;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Double_Dot_Equation;
--# end
--# property-start BinOp.test_eq /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12920
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Test_Eq
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_983 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_983 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_983;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6827 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6826 '<And at ast.py:12922>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12922
--# expr-start 6821 '<And at ast.py:12922>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12922
--# expr-start 6818 '<And at ast.py:12922>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12922
--# expr-start 6815 '<And at ast.py:12922>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12922
--# expr-start 6810 '<Bind at ast.py:12923>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12923
--# expr-start 6808 '<FieldAccess for expected_type_var at ast.py:12923>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12923
--# expr-start 6807 '<FieldAccess for left at ast.py:12923>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12923
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Bin_Op_F_Left;
--# expr-done 6807
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 6808
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 6809 '<FieldAccess for bool_type at ast.py:12923>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12923
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 6809
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12923") else null)); 
--# expr-done 6810
--# expr-start 6814 '<Bind at ast.py:12924>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12924
--# expr-start 6812 '<FieldAccess for expected_type_var at ast.py:12924>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12924
--# expr-start 6811 '<FieldAccess for right at ast.py:12924>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12924
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Bin_Op_F_Right;
--# expr-done 6811
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_3);
--# expr-done 6812
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 6813 '<FieldAccess for bool_type at ast.py:12924>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12924
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 6813
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12924") else null)); 
--# expr-done 6814
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12922") else null)); 
--# expr-done 6815
--# expr-start 6817 '<FieldAccess for matches_expected_formal_prim_type at ast.py:12926>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12926
--# expr-start 6816 '<FieldAccess for left at ast.py:12926>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12926
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Bin_Op_F_Left;
--# expr-done 6816
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_7 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_6, Origin => Origin);
--# end
--# expr-done 6817
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12922") else null)); 
--# expr-done 6818
--# expr-start 6820 '<FieldAccess for matches_expected_formal_prim_type at ast.py:12927>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12927
--# expr-start 6819 '<FieldAccess for right at ast.py:12927>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12927
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Bin_Op_F_Right;
--# expr-done 6819
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_8, Origin => Origin);
--# end
--# expr-done 6820
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12922") else null)); 
--# expr-done 6821
--# expr-start 6825 '<Bind at ast.py:12929>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12929
--# expr-start 6822 '<FieldAccess for type_var at ast.py:12929>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12929
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6822
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 6824 '<FieldAccess for type_var at ast.py:12929>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12929
--# expr-start 6823 '<FieldAccess for left at ast.py:12929>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12929
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Bin_Op_F_Left;
--# expr-done 6823
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 6824
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_2 := Solver.Create_Unify (Fld_10, Fld_12, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12929") else null)); 
--# expr-done 6825
And_Pred_3 := Create_And (And_Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12922") else null)); 
--# expr-done 6826
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 6827
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_983;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_983;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Test_Eq;
--# end
--# property-start BinOp.arguments_eq /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12932
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Arguments_Eq
  (Node : Bare_Bin_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_984 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Len : Integer;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_984 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_984;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6845 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6828 '<FieldAccess for unpacked_formal_params at ast.py:12934>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12934
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 6828
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind ps Block_Var
--# expr-start 6844 '<If at ast.py:12935>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12935
--# expr-start 6830 '<Eq at ast.py:12936>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12936
--# expr-start 6829 '<length at ast.py:12936>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12936
Len := Length (Block_Var); 
--# expr-done 6829
Is_Equal := Len = 2; 
--# expr-done 6830
if Is_Equal then
   --# expr-start 6843 '<And at ast.py:12940>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12940
--# expr-start 6839 '<And at ast.py:12940>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12940
--# expr-start 6834 '<FieldAccess for call_argument_equation at ast.py:12940>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12940
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6832 '<FieldAccess for formal_decl at ast.py:12940>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12940
--# expr-start 6831 '<at at ast.py:12940>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12940
Get_Result := Get (Self, Block_Var, 0, True); 
--# expr-done 6831
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 6832
--# expr-start 6833 '<FieldAccess for left at ast.py:12940>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12940
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 6833
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Spec.Node, Param => Fld_1, Arg => Fld_2, Origin => Origin, E_Info => Spec.Info);
--# end
--# expr-done 6834
--# expr-start 6838 '<FieldAccess for call_argument_equation at ast.py:12944>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12944
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6836 '<FieldAccess for formal_decl at ast.py:12944>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12944
--# expr-start 6835 '<at at ast.py:12944>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12944
Get_Result_1 := Get (Self, Block_Var, 1, True); 
--# expr-done 6835
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 6836
--# expr-start 6837 '<FieldAccess for right at ast.py:12944>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12944
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 6837
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Spec.Node, Param => Fld_4, Arg => Fld_5, Origin => Origin, E_Info => Spec.Info);
--# end
--# expr-done 6838
And_Pred := Create_And (Fld_3, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12940") else null)); 
--# expr-done 6839
--# expr-start 6842 '<Bind at ast.py:12947>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12947
--# expr-start 6840 '<FieldAccess for type_var at ast.py:12947>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12947
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6840
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 6841 '<FieldAccess for return_type at ast.py:12947>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12947
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Spec.Node, Origin => Origin, E_Info => Spec.Info);
--# end
--# expr-done 6841
      Cast_Result := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Bind_Result := Solver.Create_Assign (Fld_7, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12947") else null)); 
--# expr-done 6842
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12940") else null)); 
--# expr-done 6843
   If_Result := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12949") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 6844
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6845
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_984;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_984;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Arguments_Eq;
--# end
--# property-start BinOp.entity_eq /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12952
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Entity_Eq
  (Node : Bare_Bin_Op
      ; Subp : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp Subp
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_985 with Inline_Always;
      Block_Var : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Op;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Bare_Op;
Fld_5 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_985 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_985;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6856 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6846 '<FieldAccess for subp_spec_or_null at ast.py:12954>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12954
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Subp.Node, E_Info => Subp.Info);
--# end
--# expr-done 6846
Block_Var := Fld; 
--# bind spec Block_Var
--# expr-start 6855 '<And at ast.py:12955>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12955
--# expr-start 6851 '<And at ast.py:12955>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12955
--# expr-start 6847 '<FieldAccess for arguments_eq at ast.py:12956>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12956
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.arguments_eq
Fld_1 := Libadalang.Implementation.Bin_Op_P_Arguments_Eq (Node => Ent.Node, Spec => Block_Var, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6847
--# expr-start 6850 '<Bind at ast.py:12959>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12959
--# expr-start 6849 '<FieldAccess for ref_var at ast.py:12959>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12959
--# expr-start 6848 '<FieldAccess for op at ast.py:12959>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12959
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Op;
--# expr-done 6848
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 6849
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Cast_Expr := Subp; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12959") else null)); 
--# expr-done 6850
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12955") else null)); 
--# expr-done 6851
--# expr-start 6854 '<Bind at ast.py:12960>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12960
--# expr-start 6853 '<FieldAccess for subp_spec_var at ast.py:12960>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12960
--# expr-start 6852 '<FieldAccess for op at ast.py:12960>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12960
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Bin_Op_F_Op;
--# expr-done 6852
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld_4);
--# end
--# expr-done 6853
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12960") else null)); 
--# expr-done 6854
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12955") else null)); 
--# expr-done 6855
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6856
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_985;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_985;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Entity_Eq;
--# end
--# property-start BinOp.overload_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12963
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_986 with Inline_Always;
            procedure Finalizer_Scope_2832 with Inline_Always;
      Fld : Internal_Entity_Op;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_2 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_986 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_986;
            procedure Finalizer_Scope_2832 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2832;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6862 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6861 '<LogicAny at ast.py:12965>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12965
--# expr-start 6860 '<Map at ast.py:12965>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12965
   --# expr-start 6858 '<FieldAccess for subprograms at ast.py:12965>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12965
--# expr-start 6857 '<FieldAccess for op at ast.py:12965>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12965
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 6857
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprograms
Fld_1 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6858
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_1;
      begin
         for Item_236 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_236
      --# expr-start 6859 '<FieldAccess for entity_eq at ast.py:12966>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12966
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.entity_eq
Fld_2 := Libadalang.Implementation.Bin_Op_P_Entity_Eq (Node => Ent.Node, Subp => Item_236, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6859
         declare
            Item_To_Append : constant Logic_Equation := Fld_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2832;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6860
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12965") else null)); 
--# expr-done 6861
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 6862
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_986;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_986;
                     Finalizer_Scope_2832;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Overload_Equation;
--# end
--# property-start '[dispatcher]BinOp.no_overload_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Bin_Op_P_No_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Bin_Op_Range (Self.Kind) is
               when Ada_Bin_Op =>
                  --# property-call-start BinOp.no_overload_equation
                  Property_Result := Bin_Op_P_No_Overload_Equation
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Relation_Op =>
                  --# property-call-start RelationOp.no_overload_equation
                  Property_Result := Relation_Op_P_No_Overload_Equation
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Bin_Op_P_No_Overload_Equation;
--# end
--# property-start BinOp.numeric_type_from_operands_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13031
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Numeric_Type_From_Operands_Equation
  (Node : Bare_Bin_Op
      ; Dest_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   --# bind dest_var Dest_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_988 with Inline_Always;
      Block_Var : Boolean;
Block_Var_1 : Boolean;
Infer_Left : Logic_Equation;
Infer_Right : Logic_Equation;
Fld : Bare_Expr;
Fld_1 : Boolean;
Fld_2 : Bare_Expr;
Fld_3 : Boolean;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Pred : Logic_Equation;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Var;
Fld_10 : Bare_Expr;
Fld_11 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_12 : Bare_Expr;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_14 : Bare_Expr;
Fld_15 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_16 : Bare_Expr;
Fld_17 : Logic_Var;
Pred_1 : Logic_Equation;
Fld_18 : Bare_Expr;
Fld_19 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_20 : Bare_Expr;
Fld_21 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_22 : Bare_Expr;
Fld_23 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_24 : Bare_Expr;
Fld_25 : Logic_Var;
Pred_2 : Logic_Equation;
Fld_26 : Bare_Expr;
Fld_27 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_28 : Bare_Expr;
Fld_29 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_30 : Bare_Expr;
Fld_31 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Or_Pred : Logic_Equation;
Or_Pred_1 : Logic_Equation;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
If_Result_1 : Boolean;
If_Result_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_988 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Pred_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Infer_Left);
                     Dec_Ref (Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_31);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Infer_Right);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_988;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6915 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 6864 '<FieldAccess for has_context_free_type at ast.py:13038>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13038
--# expr-start 6863 '<FieldAccess for left at ast.py:13038>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13038
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Bin_Op_F_Left;
--# expr-done 6863
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Fld);
--# end
--# expr-done 6864
Block_Var := Fld_1; 
--# bind left_ctx_free Block_Var
--# expr-start 6866 '<FieldAccess for has_context_free_type at ast.py:13039>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13039
--# expr-start 6865 '<FieldAccess for right at ast.py:13039>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13039
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Right;
--# expr-done 6865
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Fld_2);
--# end
--# expr-done 6866
Block_Var_1 := Fld_3; 
--# bind right_ctx_free Block_Var_1
--# expr-start 6914 '<If at ast.py:13040>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13040
if Block_Var then
   If_Result_1 := Block_Var_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 6883 '<And at ast.py:13047>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13047
--# expr-start 6880 '<And at ast.py:13047>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13047
--# expr-start 6877 '<And at ast.py:13047>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13047
--# expr-start 6871 '<Predicate on BaseTypeDecl.one_non_universal at ast.py:13047>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13047
--# expr-start 6868 '<FieldAccess for type_var at ast.py:13048>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13048
--# expr-start 6867 '<FieldAccess for left at ast.py:13048>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13048
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Bin_Op_F_Left;
--# expr-done 6867
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 6868
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 6870 '<FieldAccess for type_var at ast.py:13048>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13048
--# expr-start 6869 '<FieldAccess for right at ast.py:13048>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13048
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Bin_Op_F_Right;
--# expr-done 6869
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 6870
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Pred := Solver.Create_N_Predicate ((Fld_5, Fld_7), Create_Base_Type_Decl_P_One_Non_Universal_0_Predicate); 
--# expr-done 6871
--# expr-start 6876 '<NPropagate at ast.py:13049>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13049
Dest_Var.Value := No_Entity;
Entity_Vars.Reset (Dest_Var);
--# expr-start 6873 '<FieldAccess for type_var at ast.py:13051>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13051
--# expr-start 6872 '<FieldAccess for left at ast.py:13051>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13051
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Bin_Op_F_Left;
--# expr-done 6872
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8);
--# expr-done 6873
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 6875 '<FieldAccess for type_var at ast.py:13051>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13051
--# expr-start 6874 '<FieldAccess for right at ast.py:13051>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13051
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Bin_Op_F_Right;
--# expr-done 6874
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10);
--# expr-done 6875
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Bind_Result := Solver.Create_N_Propagate (Dest_Var, Logic_Functor_645'(N => 2, Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), (1 => Fld_9, 2 => Fld_11), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13049") else null)); 
--# expr-done 6876
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13047") else null)); 
--# expr-done 6877
--# expr-start 6879 '<FieldAccess for matches_expected_formal_type at ast.py:13052>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13052
--# expr-start 6878 '<FieldAccess for left at ast.py:13052>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13052
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Bin_Op_F_Left;
--# expr-done 6878
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_12, Origin => Origin);
--# end
--# expr-done 6879
And_Pred_1 := Create_And (And_Pred, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13047") else null)); 
--# expr-done 6880
--# expr-start 6882 '<FieldAccess for matches_expected_formal_type at ast.py:13053>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13053
--# expr-start 6881 '<FieldAccess for right at ast.py:13053>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13053
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Self.Bin_Op_F_Right;
--# expr-done 6881
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_15 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_14, Origin => Origin);
--# end
--# expr-done 6882
And_Pred_2 := Create_And (And_Pred_1, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13047") else null)); 
--# expr-done 6883
   If_Result_2 := And_Pred_2;
else
   --# expr-start 6913 '<Let at ast.py:13055>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13055
--# scope-start
--# expr-start 6896 '<And at ast.py:13057>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13057
--# expr-start 6893 '<And at ast.py:13057>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13057
--# expr-start 6890 '<And at ast.py:13057>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13057
--# expr-start 6886 '<Predicate on BaseTypeDecl.is_not_universal_type at ast.py:13057>' Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13057
--# expr-start 6885 '<FieldAccess for type_var at ast.py:13058>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13058
--# expr-start 6884 '<FieldAccess for left at ast.py:13058>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13058
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.Bin_Op_F_Left;
--# expr-done 6884
      if Fld_16 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_16);
--# expr-done 6885
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
Pred_1 := Solver.Create_Predicate (Fld_17, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate); 
--# expr-done 6886
--# expr-start 6889 '<Bind at ast.py:13059>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13059
--# expr-start 6888 '<FieldAccess for type_var at ast.py:13059>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13059
--# expr-start 6887 '<FieldAccess for left at ast.py:13059>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13059
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Self.Bin_Op_F_Left;
--# expr-done 6887
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_18);
--# expr-done 6888
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
Dest_Var.Value := No_Entity;
Entity_Vars.Reset (Dest_Var);
Bind_Result_1 := Solver.Create_Propagate (Fld_19, Dest_Var, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13059") else null)); 
--# expr-done 6889
And_Pred_3 := Create_And (Pred_1, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13057") else null)); 
--# expr-done 6890
--# expr-start 6892 '<FieldAccess for matches_expected_formal_type at ast.py:13061>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13061
--# expr-start 6891 '<FieldAccess for left at ast.py:13061>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13061
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Self.Bin_Op_F_Left;
--# expr-done 6891
      if Fld_20 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_21 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_20, Origin => Origin);
--# end
--# expr-done 6892
And_Pred_4 := Create_And (And_Pred_3, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13057") else null)); 
--# expr-done 6893
--# expr-start 6895 '<FieldAccess for matches_expected_formal_type at ast.py:13062>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13062
--# expr-start 6894 '<FieldAccess for right at ast.py:13062>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13062
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Self.Bin_Op_F_Right;
--# expr-done 6894
      if Fld_22 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_23 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_22, Origin => Origin);
--# end
--# expr-done 6895
And_Pred_5 := Create_And (And_Pred_4, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13057") else null)); 
--# expr-done 6896
Infer_Left := And_Pred_5; Inc_Ref (Infer_Left);
--# bind infer_left Infer_Left
--# expr-start 6909 '<And at ast.py:13064>' And_Pred_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13064
--# expr-start 6906 '<And at ast.py:13064>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13064
--# expr-start 6903 '<And at ast.py:13064>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13064
--# expr-start 6899 '<Predicate on BaseTypeDecl.is_not_universal_type at ast.py:13064>' Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13064
--# expr-start 6898 '<FieldAccess for type_var at ast.py:13065>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13065
--# expr-start 6897 '<FieldAccess for right at ast.py:13065>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13065
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Self.Bin_Op_F_Right;
--# expr-done 6897
      if Fld_24 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24);
--# expr-done 6898
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
Pred_2 := Solver.Create_Predicate (Fld_25, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate); 
--# expr-done 6899
--# expr-start 6902 '<Bind at ast.py:13066>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13066
--# expr-start 6901 '<FieldAccess for type_var at ast.py:13066>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13066
--# expr-start 6900 '<FieldAccess for right at ast.py:13066>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13066
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Self.Bin_Op_F_Right;
--# expr-done 6900
      if Fld_26 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_26);
--# expr-done 6901
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
Dest_Var.Value := No_Entity;
Entity_Vars.Reset (Dest_Var);
Bind_Result_2 := Solver.Create_Propagate (Fld_27, Dest_Var, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13066") else null)); 
--# expr-done 6902
And_Pred_6 := Create_And (Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13064") else null)); 
--# expr-done 6903
--# expr-start 6905 '<FieldAccess for matches_expected_formal_type at ast.py:13068>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13068
--# expr-start 6904 '<FieldAccess for left at ast.py:13068>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13068
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Self.Bin_Op_F_Left;
--# expr-done 6904
      if Fld_28 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_29 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_28, Origin => Origin);
--# end
--# expr-done 6905
And_Pred_7 := Create_And (And_Pred_6, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13064") else null)); 
--# expr-done 6906
--# expr-start 6908 '<FieldAccess for matches_expected_formal_type at ast.py:13069>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13069
--# expr-start 6907 '<FieldAccess for right at ast.py:13069>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13069
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Self.Bin_Op_F_Right;
--# expr-done 6907
      if Fld_30 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_31 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_30, Origin => Origin);
--# end
--# expr-done 6908
And_Pred_8 := Create_And (And_Pred_7, Fld_31, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13064") else null)); 
--# expr-done 6909
Infer_Right := And_Pred_8; Inc_Ref (Infer_Right);
--# bind infer_right Infer_Right
--# expr-start 6912 '<If at ast.py:13071>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13071
if Block_Var then
   --# expr-start 6910 '<Or at ast.py:13072>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13072
Or_Pred := Create_Or (Infer_Left, Infer_Right, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13072") else null)); 
--# expr-done 6910
   If_Result := Or_Pred;
else
   --# expr-start 6911 '<Or at ast.py:13073>' Or_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13073
Or_Pred_1 := Create_Or (Infer_Right, Infer_Left, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13073") else null)); 
--# expr-done 6911
   If_Result := Or_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 6912
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6913
   If_Result_2 := Let_Result;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 6914
Let_Result_1 := If_Result_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6915
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_988;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_988;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Numeric_Type_From_Operands_Equation;
--# end
--# property-start BinOp.potential_actuals_for_dispatch /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13077
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Bin_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_989 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Expr;
New_Struct_1 : Internal_Expected_Type_For_Expr;
Array_Lit : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_989 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_989;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6928 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6916 '<FieldAccess for unpacked_formal_params at ast.py:13079>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13079
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 6916
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind params Block_Var
--# expr-start 6927 '<ArrayLiteral at ast.py:13080>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13080
--# expr-start 6921 '<New ExpectedTypeForExpr at ast.py:13081>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13081
--# expr-start 6919 '<FieldAccess for type_expression at ast.py:13082>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13082
--# expr-start 6918 '<FieldAccess for formal_decl at ast.py:13082>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13082
--# expr-start 6917 '<at at ast.py:13082>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13082
Get_Result := Get (Self, Block_Var, 0, True); 
--# expr-done 6917
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 6918
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6919
--# expr-start 6920 '<FieldAccess for left at ast.py:13083>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13083
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 6920
New_Struct := (Expected_Type => Fld_2, Expr => Fld_3); 
--# expr-done 6921
--# expr-start 6926 '<New ExpectedTypeForExpr at ast.py:13085>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13085
--# expr-start 6924 '<FieldAccess for type_expression at ast.py:13086>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13086
--# expr-start 6923 '<FieldAccess for formal_decl at ast.py:13086>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13086
--# expr-start 6922 '<at at ast.py:13086>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13086
Get_Result_1 := Get (Self, Block_Var, 1, True); 
--# expr-done 6922
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 6923
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 6924
--# expr-start 6925 '<FieldAccess for right at ast.py:13087>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13087
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 6925
New_Struct_1 := (Expected_Type => Fld_5, Expr => Fld_6); 
--# expr-done 6926
Array_Lit := Create_Internal_Expected_Type_For_Expr_Array (Internal_Internal_Expected_Type_For_Expr_Array'(1 => New_Struct, 2 => New_Struct_1)); 
--# expr-done 6927
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 6928
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_989;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_989;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start BinOp.is_dispatching_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13091
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Is_Dispatching_Call
  (Node : Bare_Bin_Op
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Op;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6931 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 6930 '<FieldAccess for is_dispatching_call at ast.py:13093>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13093
--# expr-start 6929 '<FieldAccess for op at ast.py:13093>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13093
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 6929
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 6930
Let_Result := Fld_1; 
--# end
--# expr-done 6931
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Is_Dispatching_Call;
--# end
--# property-start BinOp.no_overload_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12969
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_No_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_987 with Inline_Always;
      Fld : Bare_Op;
Is_A : Boolean;
Fld_1 : Logic_Var;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Bare_Expr;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_10 : Bare_Expr;
Fld_11 : Logic_Var;
Fld_12 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_13 : Bare_Expr;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
Fld_16 : Bare_Expr;
Fld_17 : Logic_Var;
Fld_18 : Logic_Var;
Bind_Result_5 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_19 : Bare_Op;
Is_A_1 : Boolean;
Fld_20 : Bare_Expr;
Fld_21 : Logic_Var;
Fld_22 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_6 : Logic_Equation;
Fld_23 : Bare_Expr;
Fld_24 : Logic_Var;
Fld_25 : Logic_Var;
Bind_Result_7 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_26 : Logic_Var;
Fld_27 : Bare_Expr;
Fld_28 : Logic_Var;
Bind_Result_8 : Logic_Equation;
Fld_29 : Logic_Var;
Fld_30 : Bare_Expr;
Fld_31 : Logic_Var;
Bind_Result_9 : Logic_Equation;
And_Pred_4 : Logic_Equation;
If_Result : Logic_Equation;
If_Result_1 : Logic_Equation;
Fld_32 : Logic_Var;
Pred : Logic_Equation;
Fld_33 : Logic_Var;
Fld_34 : Logic_Var;
Bind_Result_10 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_35 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_11 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_36 : Logic_Var;
Pred_1 : Logic_Equation;
Fld_37 : Bare_Expr;
Fld_38 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_39 : Bare_Expr;
Fld_40 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_41 : Logic_Var;
Fld_42 : Logic_Equation;
Or_Pred_3 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_987 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_8);
                     Dec_Ref (Bind_Result_9);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result_10);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Bind_Result_11);
                     Dec_Ref (Or_Pred_2);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Pred_1);
                     Dec_Ref (Fld_38);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_40);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Fld_42);
                     Dec_Ref (Or_Pred_3);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_987;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7006 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7005 '<And at ast.py:12975>' And_Pred_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12975
--# expr-start 6993 '<And at ast.py:12975>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12975
--# expr-start 6983 '<Cond at ast.py:12975>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12975
--# expr-start 6933 '<IsA Op.Mult, Op.Div at ast.py:12980>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12980
--# expr-start 6932 '<FieldAccess for op at ast.py:12980>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12980
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Bin_Op_F_Op;
--# expr-done 6932
Is_A := Fld /= null 
and then Fld.Kind in Ada_Op_Mult_Range | Ada_Op_Div_Range; 
--# expr-done 6933
if Is_A then
   --# expr-start 6962 '<Or at ast.py:12981>' Or_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12981
--# expr-start 6952 '<Or at ast.py:12981>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12981
--# expr-start 6942 '<And at ast.py:12982>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12982
--# expr-start 6937 '<Bind at ast.py:12982>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12982
--# expr-start 6934 '<FieldAccess for type_var at ast.py:12982>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12982
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6934
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 6936 '<FieldAccess for expected_type_var at ast.py:12982>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12982
--# expr-start 6935 '<FieldAccess for left at ast.py:12982>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12982
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Left;
--# expr-done 6935
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 6936
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12982") else null)); 
--# expr-done 6937
--# expr-start 6941 '<Bind at ast.py:12983>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12983
--# expr-start 6938 '<FieldAccess for type_var at ast.py:12983>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12983
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6938
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 6940 '<FieldAccess for expected_type_var at ast.py:12983>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12983
--# expr-start 6939 '<FieldAccess for right at ast.py:12983>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12983
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Bin_Op_F_Right;
--# expr-done 6939
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_5);
--# expr-done 6940
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Bind_Result_1 := Solver.Create_Unify (Fld_4, Fld_6, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12983") else null)); 
--# expr-done 6941
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12982") else null)); 
--# expr-done 6942
--# expr-start 6951 '<And at ast.py:12985>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12985
--# expr-start 6946 '<Bind at ast.py:12985>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12985
--# expr-start 6944 '<FieldAccess for expected_type_var at ast.py:12985>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12985
--# expr-start 6943 '<FieldAccess for left at ast.py:12985>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12985
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Bin_Op_F_Left;
--# expr-done 6943
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_7);
--# expr-done 6944
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 6945 '<FieldAccess for int_type at ast.py:12985>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12985
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_9 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 6945
      Cast_Result := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_8, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12985") else null)); 
--# expr-done 6946
--# expr-start 6950 '<Bind at ast.py:12986>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12986
--# expr-start 6948 '<FieldAccess for expected_type_var at ast.py:12986>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12986
--# expr-start 6947 '<FieldAccess for right at ast.py:12986>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12986
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Bin_Op_F_Right;
--# expr-done 6947
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_10);
--# expr-done 6948
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 6949 '<FieldAccess for type_var at ast.py:12986>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12986
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6949
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_3 := Solver.Create_Unify (Fld_11, Fld_12, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12986") else null)); 
--# expr-done 6950
And_Pred_1 := Create_And (Bind_Result_2, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12985") else null)); 
--# expr-done 6951
Or_Pred := Create_Or (And_Pred, And_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12981") else null)); 
--# expr-done 6952
--# expr-start 6961 '<And at ast.py:12988>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12988
--# expr-start 6956 '<Bind at ast.py:12988>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12988
--# expr-start 6954 '<FieldAccess for expected_type_var at ast.py:12988>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12988
--# expr-start 6953 '<FieldAccess for right at ast.py:12988>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12988
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Self.Bin_Op_F_Right;
--# expr-done 6953
      if Fld_13 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_13);
--# expr-done 6954
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 6955 '<FieldAccess for int_type at ast.py:12988>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12988
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_15 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 6955
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_14, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12988") else null)); 
--# expr-done 6956
--# expr-start 6960 '<Bind at ast.py:12989>' Bind_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12989
--# expr-start 6958 '<FieldAccess for expected_type_var at ast.py:12989>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12989
--# expr-start 6957 '<FieldAccess for left at ast.py:12989>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12989
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.Bin_Op_F_Left;
--# expr-done 6957
      if Fld_16 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_16);
--# expr-done 6958
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
--# expr-start 6959 '<FieldAccess for type_var at ast.py:12989>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12989
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6959
Fld_18.Value := No_Entity;
Entity_Vars.Reset (Fld_18);
Bind_Result_5 := Solver.Create_Unify (Fld_17, Fld_18, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12989") else null)); 
--# expr-done 6960
And_Pred_2 := Create_And (Bind_Result_4, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12988") else null)); 
--# expr-done 6961
Or_Pred_1 := Create_Or (Or_Pred, And_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12981") else null)); 
--# expr-done 6962
   If_Result_1 := Or_Pred_1;
else
--# expr-start 6964 '<IsA Op.Pow at ast.py:12994>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12994
--# expr-start 6963 '<FieldAccess for op at ast.py:12994>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12994
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Self.Bin_Op_F_Op;
--# expr-done 6963
Is_A_1 := Fld_19 /= null 
and then Fld_19.Kind in Ada_Op_Pow_Range; 
--# expr-done 6964
if Is_A_1 then
   --# expr-start 6973 '<And at ast.py:12995>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12995
--# expr-start 6968 '<Bind at ast.py:12995>' Bind_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12995
--# expr-start 6966 '<FieldAccess for expected_type_var at ast.py:12995>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12995
--# expr-start 6965 '<FieldAccess for right at ast.py:12995>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12995
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Self.Bin_Op_F_Right;
--# expr-done 6965
      if Fld_20 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_20);
--# expr-done 6966
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
--# expr-start 6967 '<FieldAccess for int_type at ast.py:12995>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12995
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_22 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 6967
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_22.Node,
         Info => Fld_22.Info);
Bind_Result_6 := Solver.Create_Assign (Fld_21, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12995") else null)); 
--# expr-done 6968
--# expr-start 6972 '<Bind at ast.py:12996>' Bind_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12996
--# expr-start 6970 '<FieldAccess for expected_type_var at ast.py:12996>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12996
--# expr-start 6969 '<FieldAccess for left at ast.py:12996>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12996
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Self.Bin_Op_F_Left;
--# expr-done 6969
      if Fld_23 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_23);
--# expr-done 6970
Fld_24.Value := No_Entity;
Entity_Vars.Reset (Fld_24);
--# expr-start 6971 '<FieldAccess for type_var at ast.py:12996>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12996
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6971
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
Bind_Result_7 := Solver.Create_Unify (Fld_24, Fld_25, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12996") else null)); 
--# expr-done 6972
And_Pred_3 := Create_And (Bind_Result_6, Bind_Result_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12995") else null)); 
--# expr-done 6973
   If_Result := And_Pred_3;
else
   --# expr-start 6982 '<And at ast.py:13000>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13000
--# expr-start 6977 '<Bind at ast.py:13001>' Bind_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13001
--# expr-start 6974 '<FieldAccess for type_var at ast.py:13001>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13001
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6974
Fld_26.Value := No_Entity;
Entity_Vars.Reset (Fld_26);
--# expr-start 6976 '<FieldAccess for expected_type_var at ast.py:13001>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13001
--# expr-start 6975 '<FieldAccess for left at ast.py:13001>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13001
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Self.Bin_Op_F_Left;
--# expr-done 6975
      if Fld_27 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_27);
--# expr-done 6976
Fld_28.Value := No_Entity;
Entity_Vars.Reset (Fld_28);
Bind_Result_8 := Solver.Create_Unify (Fld_26, Fld_28, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13001") else null)); 
--# expr-done 6977
--# expr-start 6981 '<Bind at ast.py:13002>' Bind_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13002
--# expr-start 6978 '<FieldAccess for type_var at ast.py:13002>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13002
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6978
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 6980 '<FieldAccess for expected_type_var at ast.py:13002>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13002
--# expr-start 6979 '<FieldAccess for right at ast.py:13002>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13002
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Self.Bin_Op_F_Right;
--# expr-done 6979
      if Fld_30 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_30);
--# expr-done 6980
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
Bind_Result_9 := Solver.Create_Unify (Fld_29, Fld_31, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13002") else null)); 
--# expr-done 6981
And_Pred_4 := Create_And (Bind_Result_8, Bind_Result_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13000") else null)); 
--# expr-done 6982
   If_Result := And_Pred_4;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6983
--# expr-start 6992 '<Or at ast.py:13004>' Or_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13004
--# expr-start 6989 '<And at ast.py:13008>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13008
--# expr-start 6985 '<Predicate on AdaNode.is_not_null at ast.py:13008>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13008
--# expr-start 6984 '<FieldAccess for expected_type_var at ast.py:13008>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13008
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6984
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
Pred := Solver.Create_Predicate (Fld_32, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 6985
--# expr-start 6988 '<Bind at ast.py:13009>' Bind_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13009
--# expr-start 6986 '<FieldAccess for expected_type_var at ast.py:13009>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13009
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6986
Fld_33.Value := No_Entity;
Entity_Vars.Reset (Fld_33);
--# expr-start 6987 '<FieldAccess for type_var at ast.py:13009>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13009
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6987
Fld_34.Value := No_Entity;
Entity_Vars.Reset (Fld_34);
Bind_Result_10 := Solver.Create_Propagate (Fld_33, Fld_34, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13009") else null)); 
--# expr-done 6988
And_Pred_5 := Create_And (Pred, Bind_Result_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13008") else null)); 
--# expr-done 6989
--# expr-start 6991 '<Bind at ast.py:13016>' Bind_Result_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13016
--# expr-start 6990 '<FieldAccess for expected_type_var at ast.py:13016>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13016
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6990
Fld_35.Value := No_Entity;
Entity_Vars.Reset (Fld_35);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_11 := Solver.Create_Assign (Fld_35, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13016") else null)); 
--# expr-done 6991
Or_Pred_2 := Create_Or (And_Pred_5, Bind_Result_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13004") else null)); 
--# expr-done 6992
And_Pred_6 := Create_And (If_Result_1, Or_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12975") else null)); 
--# expr-done 6993
--# expr-start 7004 '<Or at ast.py:13017>' Or_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13017
--# expr-start 7001 '<And at ast.py:13020>' And_Pred_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13020
--# expr-start 6998 '<And at ast.py:13020>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13020
--# expr-start 6995 '<Predicate on AdaNode.is_not_null at ast.py:13020>' Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13020
--# expr-start 6994 '<FieldAccess for expected_type_var at ast.py:13020>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13020
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 6994
Fld_36.Value := No_Entity;
Entity_Vars.Reset (Fld_36);
Pred_1 := Solver.Create_Predicate (Fld_36, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 6995
--# expr-start 6997 '<FieldAccess for matches_expected_formal_type at ast.py:13021>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13021
--# expr-start 6996 '<FieldAccess for left at ast.py:13021>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13021
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Self.Bin_Op_F_Left;
--# expr-done 6996
      if Fld_37 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_38 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_37, Origin => Origin);
--# end
--# expr-done 6997
And_Pred_7 := Create_And (Pred_1, Fld_38, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13020") else null)); 
--# expr-done 6998
--# expr-start 7000 '<FieldAccess for matches_expected_formal_type at ast.py:13022>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13022
--# expr-start 6999 '<FieldAccess for right at ast.py:13022>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13022
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Self.Bin_Op_F_Right;
--# expr-done 6999
      if Fld_39 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_40 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_39, Origin => Origin);
--# end
--# expr-done 7000
And_Pred_8 := Create_And (And_Pred_7, Fld_40, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13020") else null)); 
--# expr-done 7001
--# expr-start 7003 '<FieldAccess for numeric_type_from_operands_equation at ast.py:13028>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13028
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7002 '<FieldAccess for type_var at ast.py:13028>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13028
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7002
--# property-call-start BinOp.numeric_type_from_operands_equation
Fld_42 := Libadalang.Implementation.Bin_Op_P_Numeric_Type_From_Operands_Equation (Node => Ent.Node, Dest_Var => Fld_41, Origin => Origin);
--# end
--# expr-done 7003
Or_Pred_3 := Create_Or (And_Pred_8, Fld_42, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13017") else null)); 
--# expr-done 7004
And_Pred_9 := Create_And (And_Pred_6, Or_Pred_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12975") else null)); 
--# expr-done 7005
Let_Result := And_Pred_9; Inc_Ref (Let_Result);
--# end
--# expr-done 7006
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_987;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_987;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_No_Overload_Equation;
--# end
   --
   --  Primitives for Bare_Relation_Op
   --
      procedure Initialize_Fields_For_Relation_Op
        (Self : Bare_Relation_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Bin_Op
              (Self, Bin_Op_F_Left, Bin_Op_F_Op, Bin_Op_F_Right);
      end Initialize_Fields_For_Relation_Op;
--# property-start RelationOp.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13404
pragma Warnings (Off, "is not referenced");
 function Relation_Op_P_Has_Context_Free_Type
  (Node : Bare_Relation_Op
  )
   return Boolean
is
   Self : Bare_Relation_Op  := Bare_Relation_Op (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Relation_Op_P_Has_Context_Free_Type;
--# end
--# property-start RelationOp.no_overload_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13406
pragma Warnings (Off, "is not referenced");
 function Relation_Op_P_No_Overload_Equation
  (Node : Bare_Relation_Op
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Relation_Op  := Bare_Relation_Op (Node);
     Ent : Internal_Entity_Relation_Op :=
       Internal_Entity_Relation_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_999 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Fld_8 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_999 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_999;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7019 '<And at ast.py:13407>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13407
--# expr-start 7015 '<And at ast.py:13407>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13407
--# expr-start 7009 '<Bind at ast.py:13407>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13407
--# expr-start 7007 '<FieldAccess for type_var at ast.py:13407>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13407
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7007
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7008 '<FieldAccess for bool_type at ast.py:13407>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13407
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7008
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13407") else null)); 
--# expr-done 7009
--# expr-start 7014 '<Bind at ast.py:13408>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13408
--# expr-start 7011 '<FieldAccess for expected_type_var at ast.py:13408>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13408
--# expr-start 7010 '<FieldAccess for left at ast.py:13408>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13408
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Left;
--# expr-done 7010
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 7011
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 7013 '<FieldAccess for expected_type_var at ast.py:13408>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13408
--# expr-start 7012 '<FieldAccess for right at ast.py:13408>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13408
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Bin_Op_F_Right;
--# expr-done 7012
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4);
--# expr-done 7013
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result_1 := Solver.Create_Unify (Fld_3, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13408") else null)); 
--# expr-done 7014
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13407") else null)); 
--# expr-done 7015
--# expr-start 7018 '<FieldAccess for numeric_type_from_operands_equation at ast.py:13409>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13409
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7017 '<FieldAccess for expected_type_var at ast.py:13410>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13410
--# expr-start 7016 '<FieldAccess for left at ast.py:13410>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13410
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Bin_Op_F_Left;
--# expr-done 7016
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_6);
--# expr-done 7017
--# property-call-start BinOp.numeric_type_from_operands_equation
Fld_8 := Libadalang.Implementation.Bin_Op_P_Numeric_Type_From_Operands_Equation (Node => Ent.Node, Dest_Var => Fld_7, Origin => Origin);
--# end
--# expr-done 7018
And_Pred_1 := Create_And (And_Pred, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13407") else null)); 
--# expr-done 7019
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_999;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_999;
            raise;
      end;
   return Property_Result;
end Relation_Op_P_No_Overload_Equation;
--# end
   --
   --  Primitives for Bare_Box_Expr
   --
      procedure Initialize_Fields_For_Box_Expr
        (Self : Bare_Box_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Box_Expr;
--# property-start BoxExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16136
pragma Warnings (Off, "is not referenced");
 function Box_Expr_P_Xref_Equation
  (Node : Bare_Box_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Box_Expr  := Bare_Box_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1134 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_1134 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_1134;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7022 '<Bind at ast.py:16136>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16136
--# expr-start 7020 '<FieldAccess for type_var at ast.py:16136>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16136
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7020
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7021 '<FieldAccess for expected_type_var at ast.py:16136>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16136
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7021
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Bind_Result := Solver.Create_Unify (Fld, Fld_1, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16136") else null)); 
--# expr-done 7022
         Property_Result := Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1134;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1134;
            raise;
      end;
   return Property_Result;
end Box_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Case_Expr_Alternative
   --
      procedure Initialize_Fields_For_Case_Expr_Alternative
        (Self : Bare_Case_Expr_Alternative
         ; Case_Expr_Alternative_F_Choices : Bare_Alternatives_List
         ; Case_Expr_Alternative_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Case_Expr_Alternative_F_Choices := Case_Expr_Alternative_F_Choices;
            Self.Case_Expr_Alternative_F_Expr := Case_Expr_Alternative_F_Expr;
      end Initialize_Fields_For_Case_Expr_Alternative;
   function Case_Expr_Alternative_F_Choices
     (Node : Bare_Case_Expr_Alternative) return Bare_Alternatives_List
   is
   begin
         return Node.Case_Expr_Alternative_F_Choices;
   end;
   function Case_Expr_Alternative_F_Expr
     (Node : Bare_Case_Expr_Alternative) return Bare_Expr
   is
   begin
         return Node.Case_Expr_Alternative_F_Expr;
   end;
   --
   --  Primitives for Bare_Concat_Op
   --
      procedure Initialize_Fields_For_Concat_Op
        (Self : Bare_Concat_Op
         ; Concat_Op_F_First_Operand : Bare_Expr
         ; Concat_Op_F_Other_Operands : Bare_Concat_Operand_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Concat_Op_F_First_Operand := Concat_Op_F_First_Operand;
            Self.Concat_Op_F_Other_Operands := Concat_Op_F_Other_Operands;
      end Initialize_Fields_For_Concat_Op;
   function Concat_Op_F_First_Operand
     (Node : Bare_Concat_Op) return Bare_Expr
   is
   begin
         return Node.Concat_Op_F_First_Operand;
   end;
   function Concat_Op_F_Other_Operands
     (Node : Bare_Concat_Op) return Bare_Concat_Operand_List
   is
   begin
         return Node.Concat_Op_F_Other_Operands;
   end;
--# property-start ConcatOp.operands /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13131
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Operands
  (Node : Bare_Concat_Op
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
     Ent : Internal_Entity_Concat_Op :=
       Internal_Entity_Concat_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_991 with Inline_Always;
      Fld : Internal_Entity_Expr;
Singleton : Internal_Entity_Expr_Array_Access;
Fld_1 : Internal_Entity_Concat_Operand_List;
Fld_2 : Bare_Concat_Operand_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Concat_Operand;
Is_Null : Boolean;
If_Result : Internal_Entity_Concat_Operand;
Fld_4 : Internal_Entity_Expr;
Item_237 : Internal_Entity_Concat_Operand;
Bare_Item_237 : Bare_Concat_Operand;
Map_Result : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_991 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_991;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7027 '<Concat at ast.py:13132>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13132
--# expr-start 7023 '<FieldAccess for first_operand at ast.py:13132>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13132
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Concat_Op_F_First_Operand, Info => Ent.Info);
--# expr-done 7023
                Singleton := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
--# expr-start 7026 '<Map at ast.py:13133>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13133
   --# expr-start 7024 '<FieldAccess for other_operands at ast.py:13133>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13133
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7024
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Concat_Operand_List := Fld_2;
      begin
         for Untyped_Item_237 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_237 := Untyped_Item_237; 
Is_Null := Bare_Item_237 = null; 
if Is_Null then
   If_Result := No_Entity_Concat_Operand;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_237); 
   If_Result := As_Entity;
end if;
                  Item_237 := If_Result; 
   --# scope-start
               --# bind oo Item_237
      --# expr-start 7025 '<FieldAccess for operand at ast.py:13133>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13133
      if Item_237.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Item_237.Node.Concat_Operand_F_Operand, Info => Item_237.Info);
--# expr-done 7025
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7026
Concat_Result := Concat (Singleton, Map_Result); 
--# expr-done 7027
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_991;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_991;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Operands;
--# end
--# property-start ConcatOp.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13139
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Has_Context_Free_Type
  (Node : Bare_Concat_Op
  )
   return Boolean
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
      --# bind self Self
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7028 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 7028
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Has_Context_Free_Type;
--# end
--# property-start ConcatOp.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13143
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Xref_Equation
  (Node : Bare_Concat_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
     Ent : Internal_Entity_Concat_Op :=
       Internal_Entity_Concat_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_993 with Inline_Always;
            procedure Finalizer_Scope_2834 with Inline_Always;
            procedure Finalizer_Scope_2835 with Inline_Always;
      Block_Var : Bare_Concat_Operand;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Left : Internal_Entity_Expr;
Right : Internal_Entity_Expr;
Spec : Internal_Entity_Base_Subp_Spec;
Fld : Bare_Concat_Operand_List;
Len : Integer;
Arith_Result : Integer;
Fld_1 : Bare_Concat_Operand_List;
Get_Result : Bare_Concat_Operand;
Fld_2 : Internal_Entity_Concat_Operand_List;
Fld_3 : Bare_Concat_Operand_List;
Fld_4 : Internal_Entity_Info;
Get_Result_1 : Bare_Concat_Operand;
As_Entity : Internal_Entity_Concat_Operand;
Is_Null : Boolean;
If_Result : Internal_Entity_Concat_Operand;
Fld_5 : Internal_Entity_Op_Concat;
Fld_6 : Internal_Entity_Basic_Decl_Array_Access;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Equation;
Fld_9 : Internal_Entity_Concat_Operand_List;
Fld_10 : Bare_Concat_Operand_List;
Fld_11 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Concat_Operand;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Concat_Operand;
Arith_Result_1 : Integer;
Fld_12 : Internal_Entity_Concat_Operand_List;
Fld_13 : Bare_Concat_Operand_List;
Fld_14 : Internal_Entity_Info;
Get_Result_2 : Bare_Concat_Operand;
As_Entity_2 : Internal_Entity_Concat_Operand;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Concat_Operand;
Fld_15 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Concat_Operand;
Cast_Result : Internal_Entity_Expr;
Comp_Result : Boolean;
If_Result_3 : Internal_Entity_Expr;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Logic_Equation;
Fld_18 : Internal_Entity_Base_Subp_Spec;
Fld_19 : Logic_Equation;
Fld_20 : Internal_Entity_Op_Concat;
Fld_21 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_22 : Internal_Entity_Op_Concat;
Fld_23 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Subp_Spec;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_24 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
Item_238 : Internal_Entity_Concat_Operand;
Bare_Item_238 : Bare_Concat_Operand;
Index_239 : Integer;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_25 : Logic_Var;
Fld_26 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_27 : Logic_Var;
Fld_28 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Let_Result_2 : Logic_Equation;
            procedure Finalizer_Scope_993 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_993;
            procedure Finalizer_Scope_2834 is
            begin
                     Dec_Ref (Fld_17);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_24);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_2834;
            procedure Finalizer_Scope_2835 is
            begin
                     Dec_Ref (Fld_19);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2835;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7074 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 7033 '<at at ast.py:13146>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13146
--# expr-start 7029 '<FieldAccess for other_operands at ast.py:13146>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13146
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Concat_Op_F_Other_Operands;
--# expr-done 7029
--# expr-start 7032 '<Arithmetic - at ast.py:13146>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13146
--# expr-start 7031 '<length at ast.py:13146>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13146
--# expr-start 7030 '<FieldAccess for other_operands at ast.py:13146>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13146
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Concat_Op_F_Other_Operands;
--# expr-done 7030
Len := Length (Fld); 
--# expr-done 7031
Arith_Result := (Len - 1); 
--# expr-done 7032
Get_Result := Get (Self, Fld_1, Arith_Result, True); 
--# expr-done 7033
Block_Var := Get_Result; 
--# bind last_operand Block_Var
--# expr-start 7036 '<FieldAccess for subprograms at ast.py:13148>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13148
--# expr-start 7035 '<FieldAccess for operator at ast.py:13148>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13148
--# expr-start 7034 '<FieldAccess for other_operands at ast.py:13148>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13148
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7034
Fld_3 := Fld_2.Node;
Get_Result_1 := Get (Self, Fld_3, 0, True); 
Is_Null := Get_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Concat_Operand;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result_1); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Op_Concat (Node => If_Result.Node.Concat_Operand_F_Operator, Info => If_Result.Info);
--# expr-done 7035
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprograms
Fld_6 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 7036
Block_Var_1 := Fld_6; Inc_Ref (Block_Var_1);
--# bind concat_subprograms Block_Var_1
--# expr-start 7073 '<And at ast.py:13152>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13152
--# expr-start 7069 '<And at ast.py:13152>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13152
--# expr-start 7065 '<And at ast.py:13152>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13152
--# expr-start 7038 '<FieldAccess for sub_equation at ast.py:13152>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13152
--# expr-start 7037 '<FieldAccess for first_operand at ast.py:13152>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13152
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Ent.Node.Concat_Op_F_First_Operand, Info => Ent.Info);
--# expr-done 7037
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 7038
--# expr-start 7064 '<LogicAll at ast.py:13153>' Logic_Boolean_Op_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13153
--# expr-start 7063 '<Map at ast.py:13153>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13153
   --# expr-start 7039 '<FieldAccess for other_operands at ast.py:13153>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13153
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7039
Fld_10 := Fld_9.Node;
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_10 = null then
      Map_Result_1 := Create_Logic_Equation_Array (0);
      else
         Index_239 := 0;
      declare
         Collection : constant Bare_Concat_Operand_List := Fld_10;
      begin
         for Untyped_Item_238 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_238 := Untyped_Item_238; 
Is_Null_1 := Bare_Item_238 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Concat_Operand;
else
Fld_11 := Fld_9.Info;
As_Entity_1 := (Info => Fld_11, Node => Bare_Item_238); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_238 := If_Result_1; 
   --# scope-start
               --# bind concat_operand Item_238
               --# bind pos Index_239
      --# expr-start 7062 '<Let at ast.py:13154>' Let_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13154
--# scope-start
--# expr-start 7043 '<If at ast.py:13158>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13158
Comp_Result := Index_239 > 0; 
if Comp_Result then
--# expr-start 7040 '<FieldAccess for other_operands at ast.py:13160>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13160
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7040
Fld_13 := Fld_12.Node;
--# expr-start 7041 '<Arithmetic - at ast.py:13160>' Arith_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13160
Arith_Result_1 := (Index_239 - 1); 
--# expr-done 7041
Get_Result_2 := Get (Self, Fld_13, Arith_Result_1, True); 
Is_Null_2 := Get_Result_2 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Concat_Operand;
else
Fld_14 := Fld_12.Info;
As_Entity_2 := (Info => Fld_14, Node => Get_Result_2); 
   If_Result_2 := As_Entity_2;
end if;
Cast_Expr := If_Result_2; 
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_3 := Cast_Result;
else
   --# expr-start 7042 '<FieldAccess for first_operand at ast.py:13161>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13161
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Expr (Node => Ent.Node.Concat_Op_F_First_Operand, Info => Ent.Info);
--# expr-done 7042
   If_Result_3 := Fld_15;
end if;
--# expr-done 7043
Left := If_Result_3; 
--# bind left Left
--# expr-start 7044 '<FieldAccess for operand at ast.py:13163>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13163
      if Item_238.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Item_238.Node.Concat_Operand_F_Operand, Info => Item_238.Info);
--# expr-done 7044
Right := Fld_16; 
--# bind right Right
--# expr-start 7061 '<And at ast.py:13165>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13165
--# expr-start 7045 '<FieldAccess for sub_equation at ast.py:13165>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13165
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_17 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Right.Node, Env => Env, Origin => Origin, E_Info => Right.Info);
--# end
--# expr-done 7045
--# expr-start 7060 '<Or at ast.py:13166>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13166
--# expr-start 7058 '<LogicAny at ast.py:13169>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13169
--# expr-start 7057 '<Map at ast.py:13169>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13169
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
      begin
         for Item_240 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_240
      --# expr-start 7056 '<Let at ast.py:13170>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13170
--# scope-start
--# expr-start 7046 '<FieldAccess for subp_spec_or_null at ast.py:13171>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13171
      if Item_240.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_240.Node, E_Info => Item_240.Info);
--# end
--# expr-done 7046
Spec := Fld_18; 
--# bind spec Spec
--# expr-start 7055 '<And at ast.py:13173>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13173
--# expr-start 7051 '<And at ast.py:13173>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13173
--# expr-start 7047 '<FieldAccess for arguments_eq at ast.py:13174>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13174
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ConcatOp.arguments_eq
Fld_19 := Libadalang.Implementation.Concat_Op_P_Arguments_Eq (Node => Ent.Node, Spec => Spec, Left => Left, Op => Item_238, Right => Right, Origin => Origin, Env => Env);
--# end
--# expr-done 7047
--# expr-start 7050 '<Bind at ast.py:13176>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13176
--# expr-start 7049 '<FieldAccess for ref_var at ast.py:13176>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13176
--# expr-start 7048 '<FieldAccess for operator at ast.py:13176>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13176
      if Item_238.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Op_Concat (Node => Item_238.Node.Concat_Operand_F_Operator, Info => Item_238.Info);
--# expr-done 7048
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_20.Node);
--# end
--# expr-done 7049
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
Cast_Expr_1 := Item_240; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Solver.Create_Assign (Fld_21, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13176") else null)); 
--# expr-done 7050
And_Pred := Create_And (Fld_19, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13173") else null)); 
--# expr-done 7051
--# expr-start 7054 '<Bind at ast.py:13178>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13178
--# expr-start 7053 '<FieldAccess for subp_spec_var at ast.py:13178>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13178
--# expr-start 7052 '<FieldAccess for operator at ast.py:13178>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13178
      if Item_238.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Op_Concat (Node => Item_238.Node.Concat_Operand_F_Operator, Info => Item_238.Info);
--# expr-done 7052
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld_22.Node);
--# end
--# expr-done 7053
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
Cast_Expr_2 := Spec; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_23, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13178") else null)); 
--# expr-done 7054
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13173") else null)); 
--# expr-done 7055
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7056
         declare
            Item_To_Append : constant Logic_Equation := Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2835;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7057
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13169") else null)); 
--# expr-done 7058
--# expr-start 7059 '<FieldAccess for operator_no_subprogram_equation at ast.py:13186>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13186
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ConcatOp.operator_no_subprogram_equation
Fld_24 := Libadalang.Implementation.Concat_Op_P_Operator_No_Subprogram_Equation (Node => Ent.Node, Left => Left, Op => Item_238, Right => Right, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7059
Or_Pred := Create_Or (Logic_Boolean_Op, Fld_24, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13166") else null)); 
--# expr-done 7060
And_Pred_2 := Create_And (Fld_17, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13165") else null)); 
--# expr-done 7061
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7062
         declare
            Item_To_Append : constant Logic_Equation := Let_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2834;
               Index_239 := Index_239 + 1;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 7063
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13153") else null)); 
--# expr-done 7064
And_Pred_3 := Create_And (Fld_8, Logic_Boolean_Op_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13152") else null)); 
--# expr-done 7065
--# expr-start 7068 '<Bind at ast.py:13193>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13193
--# expr-start 7066 '<FieldAccess for type_var at ast.py:13193>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13193
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7066
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
--# expr-start 7067 '<FieldAccess for type_var at ast.py:13193>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13193
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var);
--# expr-done 7067
Fld_26.Value := No_Entity;
Entity_Vars.Reset (Fld_26);
Bind_Result_2 := Solver.Create_Unify (Fld_25, Fld_26, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13193") else null)); 
--# expr-done 7068
And_Pred_4 := Create_And (And_Pred_3, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13152") else null)); 
--# expr-done 7069
--# expr-start 7072 '<Bind at ast.py:13194>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13194
--# expr-start 7070 '<FieldAccess for expected_type_var at ast.py:13194>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13194
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7070
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 7071 '<FieldAccess for expected_type_var at ast.py:13194>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13194
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Block_Var);
--# expr-done 7071
Fld_28.Value := No_Entity;
Entity_Vars.Reset (Fld_28);
Bind_Result_3 := Solver.Create_Unify (Fld_27, Fld_28, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13194") else null)); 
--# expr-done 7072
And_Pred_5 := Create_And (And_Pred_4, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13152") else null)); 
--# expr-done 7073
Let_Result_2 := And_Pred_5; Inc_Ref (Let_Result_2);
--# end
--# expr-done 7074
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_993;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_993;
                     Finalizer_Scope_2834;
                     Finalizer_Scope_2835;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Xref_Equation;
--# end
--# property-start ConcatOp.arguments_eq /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13197
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Arguments_Eq
  (Node : Bare_Concat_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Left : Internal_Entity_Expr
      ; Op : Internal_Entity_Concat_Operand
      ; Right : Internal_Entity_Expr
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
  )
   return Logic_Equation
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
      --# bind self Self
   --# bind spec Spec
   --# bind left Left
   --# bind op Op
   --# bind right Right
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_994 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Logic_Equation;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Len : Integer;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_994 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_994;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7090 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7075 '<FieldAccess for unpacked_formal_params at ast.py:13203>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13203
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 7075
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind ps Block_Var
--# expr-start 7089 '<If at ast.py:13204>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13204
--# expr-start 7077 '<Eq at ast.py:13205>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13205
--# expr-start 7076 '<length at ast.py:13205>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13205
Len := Length (Block_Var); 
--# expr-done 7076
Is_Equal := Len = 2; 
--# expr-done 7077
if Is_Equal then
   --# expr-start 7088 '<And at ast.py:13208>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13208
--# expr-start 7084 '<And at ast.py:13208>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13208
--# expr-start 7080 '<FieldAccess for call_argument_equation at ast.py:13208>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13208
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7079 '<FieldAccess for formal_decl at ast.py:13208>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13208
--# expr-start 7078 '<at at ast.py:13208>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13208
Get_Result := Get (Self, Block_Var, 0, True); 
--# expr-done 7078
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 7079
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Spec.Node, Param => Fld_1, Arg => Left, Origin => Origin, E_Info => Spec.Info);
--# end
--# expr-done 7080
--# expr-start 7083 '<FieldAccess for call_argument_equation at ast.py:13211>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13211
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7082 '<FieldAccess for formal_decl at ast.py:13211>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13211
--# expr-start 7081 '<at at ast.py:13211>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13211
Get_Result_1 := Get (Self, Block_Var, 1, True); 
--# expr-done 7081
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 7082
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Spec.Node, Param => Fld_3, Arg => Right, Origin => Origin, E_Info => Spec.Info);
--# end
--# expr-done 7083
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13208") else null)); 
--# expr-done 7084
--# expr-start 7087 '<Bind at ast.py:13214>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13214
--# expr-start 7085 '<FieldAccess for type_var at ast.py:13214>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13214
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7085
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 7086 '<FieldAccess for return_type at ast.py:13214>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13214
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_6 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Spec.Node, Origin => Origin, E_Info => Spec.Info);
--# end
--# expr-done 7086
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13214") else null)); 
--# expr-done 7087
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13208") else null)); 
--# expr-done 7088
   If_Result := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13216") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 7089
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7090
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_994;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_994;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Arguments_Eq;
--# end
--# property-start ConcatOp.operator_no_subprogram_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13219
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Operator_No_Subprogram_Equation
  (Node : Bare_Concat_Op
      ; Left : Internal_Entity_Expr
      ; Op : Internal_Entity_Concat_Operand
      ; Right : Internal_Entity_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
     Ent : Internal_Entity_Concat_Op :=
       Internal_Entity_Concat_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind left Left
   --# bind op Op
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_995 with Inline_Always;
      Fld : Logic_Var;
Pred : Logic_Equation;
Fld_1 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_2 : Logic_Var;
Pred_2 : Logic_Equation;
Fld_3 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_4 : Logic_Var;
Node_For_Entity : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Logic_Var;
Bind_Result_3 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_14 : Logic_Var;
Fld_15 : Logic_Var;
Bind_Result_4 : Logic_Equation;
Fld_16 : Logic_Var;
Fld_17 : Logic_Var;
Bind_Result_5 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_18 : Logic_Var;
Fld_19 : Logic_Var;
Bind_Result_6 : Logic_Equation;
Fld_20 : Logic_Var;
Fld_21 : Logic_Var;
Fld_22 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Or_Pred_3 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_995 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Or_Pred_2);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Or_Pred_3);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_995;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7137 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7136 '<And at ast.py:13228>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13228
--# expr-start 7095 '<And at ast.py:13228>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13228
--# expr-start 7092 '<Predicate on BaseTypeDecl.is_array_def_with_deref_or_null at ast.py:13228>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13228
--# expr-start 7091 '<FieldAccess for expected_type_var at ast.py:13229>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13229
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7091
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Pred := Solver.Create_Predicate (Fld, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate (Origin)); 
--# expr-done 7092
--# expr-start 7094 '<Predicate on BaseTypeDecl.is_array_def_with_deref at ast.py:13230>' Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13230
--# expr-start 7093 '<FieldAccess for type_var at ast.py:13230>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13230
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7093
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred_1 := Solver.Create_Predicate (Fld_1, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin)); 
--# expr-done 7094
And_Pred := Create_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13228") else null)); 
--# expr-done 7095
--# expr-start 7135 '<Or at ast.py:13231>' Or_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13231
--# expr-start 7099 '<And at ast.py:13234>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13234
--# expr-start 7097 '<Predicate on AdaNode.is_not_null at ast.py:13234>' Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13234
--# expr-start 7096 '<FieldAccess for expected_type_var at ast.py:13234>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13234
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7096
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Pred_2 := Solver.Create_Predicate (Fld_2, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 7097
--# expr-start 7098 '<FieldAccess for array_concat_expected_type_equation at ast.py:13235>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13235
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ConcatOp.array_concat_expected_type_equation
Fld_3 := Libadalang.Implementation.Concat_Op_P_Array_Concat_Expected_Type_Equation (Node => Ent.Node, Left => Left, Op => Op, Right => Right, Origin => Origin);
--# end
--# expr-done 7098
And_Pred_1 := Create_And (Pred_2, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13234") else null)); 
--# expr-done 7099
--# expr-start 7134 '<And at ast.py:13244>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13244
--# expr-start 7102 '<Bind at ast.py:13244>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13244
--# expr-start 7100 '<FieldAccess for expected_type_var at ast.py:13244>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13244
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7100
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 7101 '<No BaseTypeDecl at ast.py:13244>' Node_For_Entity /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13244
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 7101
Is_Null := Node_For_Entity = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Node_For_Entity); 
   If_Result := As_Entity;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => If_Result.Node,
         Info => If_Result.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13244") else null)); 
--# expr-done 7102
--# expr-start 7133 '<Or at ast.py:13245>' Or_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13245
--# expr-start 7117 '<And at ast.py:13247>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13247
--# expr-start 7109 '<And at ast.py:13247>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13247
--# expr-start 7105 '<Bind at ast.py:13247>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13247
--# expr-start 7103 '<FieldAccess for type_var at ast.py:13247>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13247
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Left.Node);
--# expr-done 7103
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 7104 '<FieldAccess for type_var at ast.py:13247>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13247
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7104
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Bind_Result_1 := Solver.Create_Propagate (Fld_5, Fld_6, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13247") else null)); 
--# expr-done 7105
--# expr-start 7108 '<Bind at ast.py:13249>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13249
--# expr-start 7106 '<FieldAccess for type_var at ast.py:13249>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13249
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7106
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 7107 '<FieldAccess for expected_type_var at ast.py:13249>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13249
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7107
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result_2 := Solver.Create_Unify (Fld_7, Fld_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13249") else null)); 
--# expr-done 7108
And_Pred_2 := Create_And (Bind_Result_1, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13247") else null)); 
--# expr-done 7109
--# expr-start 7116 '<Or at ast.py:13250>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13250
--# expr-start 7112 '<Bind at ast.py:13251>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13251
--# expr-start 7110 '<FieldAccess for type_var at ast.py:13251>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13251
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7110
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 7111 '<FieldAccess for expected_type_var at ast.py:13251>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13251
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7111
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Bind_Result_3 := Solver.Create_Unify (Fld_9, Fld_10, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13251") else null)); 
--# expr-done 7112
--# expr-start 7115 '<FieldAccess for comp_bind at ast.py:13252>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13252
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7113 '<FieldAccess for type_var at ast.py:13252>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13252
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7113
--# expr-start 7114 '<FieldAccess for expected_type_var at ast.py:13252>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13252
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7114
--# property-call-start AdaNode.comp_bind
Fld_13 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_11, Right => Fld_12, Origin => Origin);
--# end
--# expr-done 7115
Or_Pred := Create_Or (Bind_Result_3, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13250") else null)); 
--# expr-done 7116
And_Pred_3 := Create_And (And_Pred_2, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13247") else null)); 
--# expr-done 7117
--# expr-start 7132 '<And at ast.py:13256>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13256
--# expr-start 7124 '<And at ast.py:13256>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13256
--# expr-start 7120 '<Bind at ast.py:13256>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13256
--# expr-start 7118 '<FieldAccess for type_var at ast.py:13256>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13256
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Right.Node);
--# expr-done 7118
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 7119 '<FieldAccess for type_var at ast.py:13256>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13256
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7119
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
Bind_Result_4 := Solver.Create_Propagate (Fld_14, Fld_15, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13256") else null)); 
--# expr-done 7120
--# expr-start 7123 '<Bind at ast.py:13258>' Bind_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13258
--# expr-start 7121 '<FieldAccess for type_var at ast.py:13258>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13258
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7121
Fld_16.Value := No_Entity;
Entity_Vars.Reset (Fld_16);
--# expr-start 7122 '<FieldAccess for expected_type_var at ast.py:13258>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13258
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7122
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
Bind_Result_5 := Solver.Create_Unify (Fld_16, Fld_17, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13258") else null)); 
--# expr-done 7123
And_Pred_4 := Create_And (Bind_Result_4, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13256") else null)); 
--# expr-done 7124
--# expr-start 7131 '<Or at ast.py:13259>' Or_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13259
--# expr-start 7127 '<Bind at ast.py:13260>' Bind_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13260
--# expr-start 7125 '<FieldAccess for type_var at ast.py:13260>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13260
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7125
Fld_18.Value := No_Entity;
Entity_Vars.Reset (Fld_18);
--# expr-start 7126 '<FieldAccess for expected_type_var at ast.py:13260>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13260
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7126
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
Bind_Result_6 := Solver.Create_Unify (Fld_18, Fld_19, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13260") else null)); 
--# expr-done 7127
--# expr-start 7130 '<FieldAccess for comp_bind at ast.py:13261>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13261
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7128 '<FieldAccess for type_var at ast.py:13261>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13261
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7128
--# expr-start 7129 '<FieldAccess for expected_type_var at ast.py:13261>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13261
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7129
--# property-call-start AdaNode.comp_bind
Fld_22 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_20, Right => Fld_21, Origin => Origin);
--# end
--# expr-done 7130
Or_Pred_1 := Create_Or (Bind_Result_6, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13259") else null)); 
--# expr-done 7131
And_Pred_5 := Create_And (And_Pred_4, Or_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13256") else null)); 
--# expr-done 7132
Or_Pred_2 := Create_Or (And_Pred_3, And_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13245") else null)); 
--# expr-done 7133
And_Pred_6 := Create_And (Bind_Result, Or_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13244") else null)); 
--# expr-done 7134
Or_Pred_3 := Create_Or (And_Pred_1, And_Pred_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13231") else null)); 
--# expr-done 7135
And_Pred_7 := Create_And (And_Pred, Or_Pred_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13228") else null)); 
--# expr-done 7136
Let_Result := And_Pred_7; Inc_Ref (Let_Result);
--# end
--# expr-done 7137
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_995;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_995;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Operator_No_Subprogram_Equation;
--# end
--# property-start ConcatOp.array_concat_expected_type_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13267
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Array_Concat_Expected_Type_Equation
  (Node : Bare_Concat_Op
      ; Left : Internal_Entity_Expr
      ; Op : Internal_Entity_Concat_Operand
      ; Right : Internal_Entity_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
      --# bind self Self
   --# bind left Left
   --# bind op Op
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_996 with Inline_Always;
      Block_Var : Boolean;
Block_Var_1 : Boolean;
Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Fld_7 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Var;
Fld_10 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Logic_Var;
Bind_Result_3 : Logic_Equation;
Fld_13 : Logic_Var;
Fld_14 : Logic_Var;
Fld_15 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_16 : Logic_Var;
Fld_17 : Logic_Var;
Fld_18 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_19 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_20 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_21 : Logic_Var;
Fld_22 : Logic_Var;
Bind_Result_4 : Logic_Equation;
Fld_23 : Logic_Var;
Fld_24 : Logic_Var;
Bind_Result_5 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_25 : Logic_Var;
Fld_26 : Logic_Var;
Bind_Result_6 : Logic_Equation;
Fld_27 : Logic_Var;
Fld_28 : Logic_Var;
Fld_29 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_30 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_31 : Logic_Var;
Fld_32 : Logic_Var;
Bind_Result_7 : Logic_Equation;
Fld_33 : Logic_Var;
Fld_34 : Logic_Var;
Fld_35 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_36 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_37 : Logic_Var;
Fld_38 : Logic_Var;
Bind_Result_8 : Logic_Equation;
Fld_39 : Logic_Var;
Fld_40 : Logic_Var;
Bind_Result_9 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Fld_41 : Logic_Var;
Fld_42 : Logic_Var;
Bind_Result_10 : Logic_Equation;
Fld_43 : Logic_Var;
Fld_44 : Logic_Var;
Fld_45 : Logic_Equation;
And_Pred_12 : Logic_Equation;
Fld_46 : Logic_Equation;
And_Pred_13 : Logic_Equation;
Or_Pred_3 : Logic_Equation;
Fld_47 : Logic_Var;
Fld_48 : Logic_Var;
Bind_Result_11 : Logic_Equation;
Fld_49 : Logic_Var;
Fld_50 : Logic_Var;
Fld_51 : Logic_Equation;
Or_Pred_4 : Logic_Equation;
And_Pred_14 : Logic_Equation;
Fld_52 : Logic_Equation;
And_Pred_15 : Logic_Equation;
Fld_53 : Logic_Var;
Fld_54 : Logic_Var;
Bind_Result_12 : Logic_Equation;
Fld_55 : Logic_Var;
Fld_56 : Logic_Var;
Bind_Result_13 : Logic_Equation;
Fld_57 : Logic_Var;
Fld_58 : Logic_Var;
Fld_59 : Logic_Equation;
Or_Pred_5 : Logic_Equation;
And_Pred_16 : Logic_Equation;
Fld_60 : Logic_Var;
Fld_61 : Logic_Var;
Bind_Result_14 : Logic_Equation;
Fld_62 : Logic_Var;
Fld_63 : Logic_Var;
Fld_64 : Logic_Equation;
Or_Pred_6 : Logic_Equation;
And_Pred_17 : Logic_Equation;
Fld_65 : Logic_Equation;
And_Pred_18 : Logic_Equation;
Fld_66 : Logic_Equation;
And_Pred_19 : Logic_Equation;
If_Result_1 : Logic_Equation;
If_Result_2 : Logic_Equation;
If_Result_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_996 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_30);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (Fld_35);
                     Dec_Ref (Or_Pred_2);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (Fld_36);
                     Dec_Ref (And_Pred_10);
                     Dec_Ref (Bind_Result_8);
                     Dec_Ref (Bind_Result_9);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Bind_Result_10);
                     Dec_Ref (Fld_45);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (Fld_46);
                     Dec_Ref (And_Pred_13);
                     Dec_Ref (Or_Pred_3);
                     Dec_Ref (Bind_Result_11);
                     Dec_Ref (Fld_51);
                     Dec_Ref (Or_Pred_4);
                     Dec_Ref (And_Pred_14);
                     Dec_Ref (Fld_52);
                     Dec_Ref (And_Pred_15);
                     Dec_Ref (Bind_Result_12);
                     Dec_Ref (Bind_Result_13);
                     Dec_Ref (Fld_59);
                     Dec_Ref (Or_Pred_5);
                     Dec_Ref (And_Pred_16);
                     Dec_Ref (Bind_Result_14);
                     Dec_Ref (Fld_64);
                     Dec_Ref (Or_Pred_6);
                     Dec_Ref (And_Pred_17);
                     Dec_Ref (Fld_65);
                     Dec_Ref (And_Pred_18);
                     Dec_Ref (Fld_66);
                     Dec_Ref (And_Pred_19);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_996;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7248 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7138 '<FieldAccess for has_context_free_type at ast.py:13277>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13277
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Left.Node);
--# end
--# expr-done 7138
Block_Var := Fld; 
--# bind left_ctx_free Block_Var
--# expr-start 7139 '<FieldAccess for has_context_free_type at ast.py:13278>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13278
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Right.Node);
--# end
--# expr-done 7139
Block_Var_1 := Fld_1; 
--# bind right_ctx_free Block_Var_1
--# expr-start 7247 '<Cond at ast.py:13283>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13283
if Block_Var then
   If_Result := Block_Var_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 7169 '<Or at ast.py:13285>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13285
--# expr-start 7153 '<And at ast.py:13292>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13292
--# expr-start 7148 '<And at ast.py:13292>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13292
--# expr-start 7143 '<NPropagate at ast.py:13292>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13292
--# expr-start 7140 '<FieldAccess for type_var at ast.py:13292>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13292
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7140
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 7141 '<FieldAccess for type_var at ast.py:13294>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13294
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Left.Node);
--# expr-done 7141
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 7142 '<FieldAccess for type_var at ast.py:13294>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13294
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Right.Node);
--# expr-done 7142
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_N_Propagate (Fld_2, Logic_Functor_646'(N => 2, Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), (1 => Fld_3, 2 => Fld_4), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13292") else null)); 
--# expr-done 7143
--# expr-start 7147 '<NPropagate at ast.py:13295>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13295
--# expr-start 7144 '<FieldAccess for expected_type_var at ast.py:13295>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13295
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7144
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 7145 '<FieldAccess for type_var at ast.py:13297>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13297
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7145
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 7146 '<FieldAccess for type_var at ast.py:13297>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13297
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Left.Node);
--# expr-done 7146
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_1 := Solver.Create_N_Propagate (Fld_5, Logic_Functor_647'(N => 2, Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), (1 => Fld_6, 2 => Fld_7), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13295") else null)); 
--# expr-done 7147
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13292") else null)); 
--# expr-done 7148
--# expr-start 7152 '<NPropagate at ast.py:13298>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13298
--# expr-start 7149 '<FieldAccess for expected_type_var at ast.py:13298>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13298
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7149
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 7150 '<FieldAccess for type_var at ast.py:13300>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13300
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7150
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 7151 '<FieldAccess for type_var at ast.py:13300>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13300
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Right.Node);
--# expr-done 7151
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Bind_Result_2 := Solver.Create_N_Propagate (Fld_8, Logic_Functor_647'(N => 2, Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), (1 => Fld_9, 2 => Fld_10), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13298") else null)); 
--# expr-done 7152
And_Pred_1 := Create_And (And_Pred, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13292") else null)); 
--# expr-done 7153
--# expr-start 7168 '<And at ast.py:13306>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13306
--# expr-start 7166 '<And at ast.py:13306>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13306
--# expr-start 7164 '<And at ast.py:13306>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13306
--# expr-start 7160 '<And at ast.py:13306>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13306
--# expr-start 7156 '<Bind at ast.py:13306>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13306
--# expr-start 7154 '<FieldAccess for expected_type_var at ast.py:13306>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13306
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7154
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 7155 '<FieldAccess for type_var at ast.py:13306>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13306
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7155
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_3 := Solver.Create_Propagate (Fld_11, Fld_12, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13306") else null)); 
--# expr-done 7156
--# expr-start 7159 '<FieldAccess for comp_bind at ast.py:13308>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13308
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7157 '<FieldAccess for type_var at ast.py:13308>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13308
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7157
--# expr-start 7158 '<FieldAccess for expected_type_var at ast.py:13308>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13308
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7158
--# property-call-start AdaNode.comp_bind
Fld_15 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_13, Right => Fld_14, Origin => Origin);
--# end
--# expr-done 7159
And_Pred_2 := Create_And (Bind_Result_3, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13306") else null)); 
--# expr-done 7160
--# expr-start 7163 '<FieldAccess for comp_bind at ast.py:13309>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13309
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7161 '<FieldAccess for type_var at ast.py:13309>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13309
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7161
--# expr-start 7162 '<FieldAccess for expected_type_var at ast.py:13309>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13309
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7162
--# property-call-start AdaNode.comp_bind
Fld_18 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_16, Right => Fld_17, Origin => Origin);
--# end
--# expr-done 7163
And_Pred_3 := Create_And (And_Pred_2, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13306") else null)); 
--# expr-done 7164
--# expr-start 7165 '<FieldAccess for matches_expected_formal_type at ast.py:13310>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13310
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_19 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Left.Node, Origin => Origin);
--# end
--# expr-done 7165
And_Pred_4 := Create_And (And_Pred_3, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13306") else null)); 
--# expr-done 7166
--# expr-start 7167 '<FieldAccess for matches_expected_formal_type at ast.py:13311>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13311
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_20 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Right.Node, Origin => Origin);
--# end
--# expr-done 7167
And_Pred_5 := Create_And (And_Pred_4, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13306") else null)); 
--# expr-done 7168
Or_Pred := Create_Or (And_Pred_1, And_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13285") else null)); 
--# expr-done 7169
   If_Result_3 := Or_Pred;
else
if Block_Var then
   --# expr-start 7196 '<And at ast.py:13315>' And_Pred_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13315
--# expr-start 7194 '<And at ast.py:13315>' And_Pred_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13315
--# expr-start 7186 '<Or at ast.py:13315>' Or_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13315
--# expr-start 7176 '<And at ast.py:13319>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13319
--# expr-start 7172 '<Bind at ast.py:13319>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13319
--# expr-start 7170 '<FieldAccess for type_var at ast.py:13319>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13319
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Left.Node);
--# expr-done 7170
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
--# expr-start 7171 '<FieldAccess for type_var at ast.py:13319>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13319
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7171
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
Bind_Result_4 := Solver.Create_Propagate (Fld_21, Fld_22, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13319") else null)); 
--# expr-done 7172
--# expr-start 7175 '<Bind at ast.py:13321>' Bind_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13321
--# expr-start 7173 '<FieldAccess for type_var at ast.py:13321>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13321
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7173
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
--# expr-start 7174 '<FieldAccess for expected_type_var at ast.py:13321>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13321
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7174
Fld_24.Value := No_Entity;
Entity_Vars.Reset (Fld_24);
Bind_Result_5 := Solver.Create_Unify (Fld_23, Fld_24, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13321") else null)); 
--# expr-done 7175
And_Pred_6 := Create_And (Bind_Result_4, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13319") else null)); 
--# expr-done 7176
--# expr-start 7185 '<And at ast.py:13327>' And_Pred_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13327
--# expr-start 7183 '<And at ast.py:13327>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13327
--# expr-start 7179 '<Bind at ast.py:13327>' Bind_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13327
--# expr-start 7177 '<FieldAccess for expected_type_var at ast.py:13327>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13327
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7177
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
--# expr-start 7178 '<FieldAccess for type_var at ast.py:13327>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13327
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7178
Fld_26.Value := No_Entity;
Entity_Vars.Reset (Fld_26);
Bind_Result_6 := Solver.Create_Propagate (Fld_25, Fld_26, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13327") else null)); 
--# expr-done 7179
--# expr-start 7182 '<FieldAccess for comp_bind at ast.py:13329>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13329
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7180 '<FieldAccess for type_var at ast.py:13329>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13329
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7180
--# expr-start 7181 '<FieldAccess for expected_type_var at ast.py:13329>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13329
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7181
--# property-call-start AdaNode.comp_bind
Fld_29 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_27, Right => Fld_28, Origin => Origin);
--# end
--# expr-done 7182
And_Pred_7 := Create_And (Bind_Result_6, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13327") else null)); 
--# expr-done 7183
--# expr-start 7184 '<FieldAccess for matches_expected_formal_type at ast.py:13330>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13330
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_30 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Left.Node, Origin => Origin);
--# end
--# expr-done 7184
And_Pred_8 := Create_And (And_Pred_7, Fld_30, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13327") else null)); 
--# expr-done 7185
Or_Pred_1 := Create_Or (And_Pred_6, And_Pred_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13315") else null)); 
--# expr-done 7186
--# expr-start 7193 '<Or at ast.py:13334>' Or_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13334
--# expr-start 7189 '<Bind at ast.py:13335>' Bind_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13335
--# expr-start 7187 '<FieldAccess for type_var at ast.py:13335>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13335
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7187
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
--# expr-start 7188 '<FieldAccess for expected_type_var at ast.py:13335>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13335
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7188
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
Bind_Result_7 := Solver.Create_Unify (Fld_31, Fld_32, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13335") else null)); 
--# expr-done 7189
--# expr-start 7192 '<FieldAccess for comp_bind at ast.py:13336>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13336
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7190 '<FieldAccess for type_var at ast.py:13336>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13336
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7190
--# expr-start 7191 '<FieldAccess for expected_type_var at ast.py:13336>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13336
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7191
--# property-call-start AdaNode.comp_bind
Fld_35 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_33, Right => Fld_34, Origin => Origin);
--# end
--# expr-done 7192
Or_Pred_2 := Create_Or (Bind_Result_7, Fld_35, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13334") else null)); 
--# expr-done 7193
And_Pred_9 := Create_And (Or_Pred_1, Or_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13315") else null)); 
--# expr-done 7194
--# expr-start 7195 '<FieldAccess for matches_expected_formal_type at ast.py:13338>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13338
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_36 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Right.Node, Origin => Origin);
--# end
--# expr-done 7195
And_Pred_10 := Create_And (And_Pred_9, Fld_36, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13315") else null)); 
--# expr-done 7196
   If_Result_2 := And_Pred_10;
else
if Block_Var_1 then
   --# expr-start 7223 '<And at ast.py:13341>' And_Pred_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13341
--# expr-start 7221 '<And at ast.py:13341>' And_Pred_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13341
--# expr-start 7213 '<Or at ast.py:13341>' Or_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13341
--# expr-start 7203 '<And at ast.py:13345>' And_Pred_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13345
--# expr-start 7199 '<Bind at ast.py:13345>' Bind_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13345
--# expr-start 7197 '<FieldAccess for type_var at ast.py:13345>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13345
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Right.Node);
--# expr-done 7197
Fld_37.Value := No_Entity;
Entity_Vars.Reset (Fld_37);
--# expr-start 7198 '<FieldAccess for type_var at ast.py:13345>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13345
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7198
Fld_38.Value := No_Entity;
Entity_Vars.Reset (Fld_38);
Bind_Result_8 := Solver.Create_Propagate (Fld_37, Fld_38, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13345") else null)); 
--# expr-done 7199
--# expr-start 7202 '<Bind at ast.py:13347>' Bind_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13347
--# expr-start 7200 '<FieldAccess for type_var at ast.py:13347>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13347
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7200
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
--# expr-start 7201 '<FieldAccess for expected_type_var at ast.py:13347>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13347
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7201
Fld_40.Value := No_Entity;
Entity_Vars.Reset (Fld_40);
Bind_Result_9 := Solver.Create_Unify (Fld_39, Fld_40, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13347") else null)); 
--# expr-done 7202
And_Pred_11 := Create_And (Bind_Result_8, Bind_Result_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13345") else null)); 
--# expr-done 7203
--# expr-start 7212 '<And at ast.py:13353>' And_Pred_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13353
--# expr-start 7210 '<And at ast.py:13353>' And_Pred_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13353
--# expr-start 7206 '<Bind at ast.py:13353>' Bind_Result_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13353
--# expr-start 7204 '<FieldAccess for expected_type_var at ast.py:13353>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13353
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7204
Fld_41.Value := No_Entity;
Entity_Vars.Reset (Fld_41);
--# expr-start 7205 '<FieldAccess for type_var at ast.py:13353>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13353
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7205
Fld_42.Value := No_Entity;
Entity_Vars.Reset (Fld_42);
Bind_Result_10 := Solver.Create_Propagate (Fld_41, Fld_42, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13353") else null)); 
--# expr-done 7206
--# expr-start 7209 '<FieldAccess for comp_bind at ast.py:13355>' Fld_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13355
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7207 '<FieldAccess for type_var at ast.py:13355>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13355
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_43 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7207
--# expr-start 7208 '<FieldAccess for expected_type_var at ast.py:13355>' Fld_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13355
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_44 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7208
--# property-call-start AdaNode.comp_bind
Fld_45 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_43, Right => Fld_44, Origin => Origin);
--# end
--# expr-done 7209
And_Pred_12 := Create_And (Bind_Result_10, Fld_45, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13353") else null)); 
--# expr-done 7210
--# expr-start 7211 '<FieldAccess for matches_expected_formal_type at ast.py:13356>' Fld_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13356
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_46 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Right.Node, Origin => Origin);
--# end
--# expr-done 7211
And_Pred_13 := Create_And (And_Pred_12, Fld_46, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13353") else null)); 
--# expr-done 7212
Or_Pred_3 := Create_Or (And_Pred_11, And_Pred_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13341") else null)); 
--# expr-done 7213
--# expr-start 7220 '<Or at ast.py:13360>' Or_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13360
--# expr-start 7216 '<Bind at ast.py:13361>' Bind_Result_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13361
--# expr-start 7214 '<FieldAccess for type_var at ast.py:13361>' Fld_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13361
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_47 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7214
Fld_47.Value := No_Entity;
Entity_Vars.Reset (Fld_47);
--# expr-start 7215 '<FieldAccess for expected_type_var at ast.py:13361>' Fld_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13361
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7215
Fld_48.Value := No_Entity;
Entity_Vars.Reset (Fld_48);
Bind_Result_11 := Solver.Create_Unify (Fld_47, Fld_48, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13361") else null)); 
--# expr-done 7216
--# expr-start 7219 '<FieldAccess for comp_bind at ast.py:13362>' Fld_51 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13362
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7217 '<FieldAccess for type_var at ast.py:13362>' Fld_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13362
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_49 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7217
--# expr-start 7218 '<FieldAccess for expected_type_var at ast.py:13362>' Fld_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13362
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7218
--# property-call-start AdaNode.comp_bind
Fld_51 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_49, Right => Fld_50, Origin => Origin);
--# end
--# expr-done 7219
Or_Pred_4 := Create_Or (Bind_Result_11, Fld_51, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13360") else null)); 
--# expr-done 7220
And_Pred_14 := Create_And (Or_Pred_3, Or_Pred_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13341") else null)); 
--# expr-done 7221
--# expr-start 7222 '<FieldAccess for matches_expected_formal_type at ast.py:13364>' Fld_52 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13364
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_52 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Left.Node, Origin => Origin);
--# end
--# expr-done 7222
And_Pred_15 := Create_And (And_Pred_14, Fld_52, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13341") else null)); 
--# expr-done 7223
   If_Result_1 := And_Pred_15;
else
   --# expr-start 7246 '<And at ast.py:13369>' And_Pred_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13369
--# expr-start 7244 '<And at ast.py:13369>' And_Pred_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13369
--# expr-start 7242 '<And at ast.py:13369>' And_Pred_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13369
--# expr-start 7234 '<And at ast.py:13369>' And_Pred_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13369
--# expr-start 7226 '<Bind at ast.py:13369>' Bind_Result_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13369
--# expr-start 7224 '<FieldAccess for expected_type_var at ast.py:13369>' Fld_53 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13369
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_53 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7224
Fld_53.Value := No_Entity;
Entity_Vars.Reset (Fld_53);
--# expr-start 7225 '<FieldAccess for type_var at ast.py:13369>' Fld_54 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13369
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_54 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7225
Fld_54.Value := No_Entity;
Entity_Vars.Reset (Fld_54);
Bind_Result_12 := Solver.Create_Propagate (Fld_53, Fld_54, Logic_Functor_643'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13369") else null)); 
--# expr-done 7226
--# expr-start 7233 '<Or at ast.py:13371>' Or_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13371
--# expr-start 7229 '<Bind at ast.py:13372>' Bind_Result_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13372
--# expr-start 7227 '<FieldAccess for type_var at ast.py:13372>' Fld_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13372
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_55 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7227
Fld_55.Value := No_Entity;
Entity_Vars.Reset (Fld_55);
--# expr-start 7228 '<FieldAccess for expected_type_var at ast.py:13372>' Fld_56 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13372
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_56 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7228
Fld_56.Value := No_Entity;
Entity_Vars.Reset (Fld_56);
Bind_Result_13 := Solver.Create_Unify (Fld_55, Fld_56, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13372") else null)); 
--# expr-done 7229
--# expr-start 7232 '<FieldAccess for comp_bind at ast.py:13373>' Fld_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13373
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7230 '<FieldAccess for type_var at ast.py:13373>' Fld_57 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13373
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_57 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7230
--# expr-start 7231 '<FieldAccess for expected_type_var at ast.py:13373>' Fld_58 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13373
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_58 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7231
--# property-call-start AdaNode.comp_bind
Fld_59 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_57, Right => Fld_58, Origin => Origin);
--# end
--# expr-done 7232
Or_Pred_5 := Create_Or (Bind_Result_13, Fld_59, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13371") else null)); 
--# expr-done 7233
And_Pred_16 := Create_And (Bind_Result_12, Or_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13369") else null)); 
--# expr-done 7234
--# expr-start 7241 '<Or at ast.py:13375>' Or_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13375
--# expr-start 7237 '<Bind at ast.py:13376>' Bind_Result_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13376
--# expr-start 7235 '<FieldAccess for type_var at ast.py:13376>' Fld_60 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13376
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_60 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7235
Fld_60.Value := No_Entity;
Entity_Vars.Reset (Fld_60);
--# expr-start 7236 '<FieldAccess for expected_type_var at ast.py:13376>' Fld_61 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13376
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_61 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7236
Fld_61.Value := No_Entity;
Entity_Vars.Reset (Fld_61);
Bind_Result_14 := Solver.Create_Unify (Fld_60, Fld_61, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13376") else null)); 
--# expr-done 7237
--# expr-start 7240 '<FieldAccess for comp_bind at ast.py:13377>' Fld_64 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13377
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7238 '<FieldAccess for type_var at ast.py:13377>' Fld_62 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13377
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_62 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7238
--# expr-start 7239 '<FieldAccess for expected_type_var at ast.py:13377>' Fld_63 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13377
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_63 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7239
--# property-call-start AdaNode.comp_bind
Fld_64 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_62, Right => Fld_63, Origin => Origin);
--# end
--# expr-done 7240
Or_Pred_6 := Create_Or (Bind_Result_14, Fld_64, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13375") else null)); 
--# expr-done 7241
And_Pred_17 := Create_And (And_Pred_16, Or_Pred_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13369") else null)); 
--# expr-done 7242
--# expr-start 7243 '<FieldAccess for matches_expected_formal_type at ast.py:13379>' Fld_65 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13379
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_65 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Left.Node, Origin => Origin);
--# end
--# expr-done 7243
And_Pred_18 := Create_And (And_Pred_17, Fld_65, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13369") else null)); 
--# expr-done 7244
--# expr-start 7245 '<FieldAccess for matches_expected_formal_type at ast.py:13380>' Fld_66 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13380
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_66 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Right.Node, Origin => Origin);
--# end
--# expr-done 7245
And_Pred_19 := Create_And (And_Pred_18, Fld_66, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13369") else null)); 
--# expr-done 7246
   If_Result_1 := And_Pred_19;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 7247
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 7248
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_996;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_996;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Array_Concat_Expected_Type_Equation;
--# end
   --
   --  Primitives for Bare_Concat_Operand
   --
      procedure Initialize_Fields_For_Concat_Operand
        (Self : Bare_Concat_Operand
         ; Concat_Operand_F_Operator : Bare_Op_Concat
         ; Concat_Operand_F_Operand : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Concat_Operand_F_Operator := Concat_Operand_F_Operator;
            Self.Concat_Operand_F_Operand := Concat_Operand_F_Operand;
      end Initialize_Fields_For_Concat_Operand;
   function Concat_Operand_F_Operator
     (Node : Bare_Concat_Operand) return Bare_Op_Concat
   is
   begin
         return Node.Concat_Operand_F_Operator;
   end;
   function Concat_Operand_F_Operand
     (Node : Bare_Concat_Operand) return Bare_Expr
   is
   begin
         return Node.Concat_Operand_F_Operand;
   end;
--# property-start ConcatOperand.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13394
pragma Warnings (Off, "is not referenced");
 function Concat_Operand_P_Has_Context_Free_Type
  (Node : Bare_Concat_Operand
  )
   return Boolean
is
   Self : Bare_Concat_Operand  := Bare_Concat_Operand (Node);
      --# bind self Self
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7249 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 7249
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Concat_Operand_P_Has_Context_Free_Type;
--# end
   --
   --  Primitives for Bare_Cond_Expr
   --
      procedure Initialize_Fields_For_Cond_Expr
        (Self : Bare_Cond_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Cond_Expr;
--# property-start '[dispatcher]CondExpr.dependent_exprs' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Cond_Expr_P_Dependent_Exprs
  (Node : Bare_Cond_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Cond_Expr  := Bare_Cond_Expr (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Cond_Expr (Self.Kind) is
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.dependent_exprs
                  Property_Result := Case_Expr_P_Dependent_Exprs
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.dependent_exprs
                  Property_Result := If_Expr_P_Dependent_Exprs
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Cond_Expr_P_Dependent_Exprs;
--# end
   --
   --  Primitives for Bare_Case_Expr
   --
      procedure Initialize_Fields_For_Case_Expr
        (Self : Bare_Case_Expr
         ; Case_Expr_F_Expr : Bare_Expr
         ; Case_Expr_F_Cases : Bare_Case_Expr_Alternative_List
        ) is
      begin
            Initialize_Fields_For_Cond_Expr
              (Self);
            Self.Case_Expr_F_Expr := Case_Expr_F_Expr;
            Self.Case_Expr_F_Cases := Case_Expr_F_Cases;
      end Initialize_Fields_For_Case_Expr;
   function Case_Expr_F_Expr
     (Node : Bare_Case_Expr) return Bare_Expr
   is
   begin
         return Node.Case_Expr_F_Expr;
   end;
   function Case_Expr_F_Cases
     (Node : Bare_Case_Expr) return Bare_Case_Expr_Alternative_List
   is
   begin
         return Node.Case_Expr_F_Cases;
   end;
--# property-start CaseExpr.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16280
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Has_Context_Free_Type
  (Node : Bare_Case_Expr
  )
   return Boolean
is
   Self : Bare_Case_Expr  := Bare_Case_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Case_Expr_P_Has_Context_Free_Type;
--# end
--# property-start CaseExpr.dependent_exprs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16282
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Dependent_Exprs
  (Node : Bare_Case_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Case_Expr  := Bare_Case_Expr (Node);
     Ent : Internal_Entity_Case_Expr :=
       Internal_Entity_Case_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1143 with Inline_Always;
      Fld : Internal_Entity_Case_Expr_Alternative_List;
Fld_1 : Bare_Case_Expr_Alternative_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Case_Expr_Alternative;
Is_Null : Boolean;
If_Result : Internal_Entity_Case_Expr_Alternative;
Fld_3 : Internal_Entity_Expr;
Item_241 : Internal_Entity_Case_Expr_Alternative;
Bare_Item_241 : Bare_Case_Expr_Alternative;
Map_Result : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1143 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1143;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7253 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7252 '<Map at ast.py:16284>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16284
   --# expr-start 7250 '<FieldAccess for cases at ast.py:16284>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16284
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Case_Expr_Alternative_List (Node => Ent.Node.Case_Expr_F_Cases, Info => Ent.Info);
--# expr-done 7250
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Case_Expr_Alternative_List := Fld_1;
      begin
         for Untyped_Item_241 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_241 := Untyped_Item_241; 
Is_Null := Bare_Item_241 = null; 
if Is_Null then
   If_Result := No_Entity_Case_Expr_Alternative;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_241); 
   If_Result := As_Entity;
end if;
                  Item_241 := If_Result; 
   --# scope-start
               --# bind c Item_241
      --# expr-start 7251 '<FieldAccess for expr at ast.py:16284>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16284
      if Item_241.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Item_241.Node.Case_Expr_Alternative_F_Expr, Info => Item_241.Info);
--# expr-done 7251
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_3;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7252
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7253
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1143;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1143;
            raise;
      end;
   return Property_Result;
end Case_Expr_P_Dependent_Exprs;
--# end
--# property-start CaseExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16286
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Xref_Equation
  (Node : Bare_Case_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Expr  := Bare_Case_Expr (Node);
     Ent : Internal_Entity_Case_Expr :=
       Internal_Entity_Case_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1144 with Inline_Always;
            procedure Finalizer_Scope_2837 with Inline_Always;
            procedure Finalizer_Scope_2838 with Inline_Always;
            procedure Finalizer_Scope_2839 with Inline_Always;
            procedure Finalizer_Scope_2841 with Inline_Always;
            procedure Finalizer_Scope_2842 with Inline_Always;
      Block_Var : Boolean;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Var_Expr : Internal_Entity_Name;
Fld : Internal_Entity_Expr;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Pred : Logic_Equation;
Fld_3 : Boolean;
Fld_4 : Logic_Var;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Case_Expr_Alternative_List;
Fld_7 : Bare_Case_Expr_Alternative_List;
Fld_8 : Internal_Entity_Info;
As_Entity : Internal_Entity_Case_Expr_Alternative;
Is_Null : Boolean;
If_Result : Internal_Entity_Case_Expr_Alternative;
Fld_9 : Internal_Entity_Alternatives_List;
Fld_10 : Bare_Alternatives_List;
Fld_11 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_12 : Logic_Equation;
Fld_13 : Logic_Var;
Fld_14 : Bare_Expr;
Fld_15 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_16 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_17 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_18 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
Not_Val : Boolean;
If_Result_2 : Logic_Equation;
Fld_19 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_5 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Match_Result : Logic_Equation;
Item_243 : Internal_Entity;
Bare_Item_243 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_20 : Logic_Var;
Fld_21 : Internal_Entity_Expr;
Fld_22 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_23 : Internal_Entity_Expr;
Fld_24 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_25 : Internal_Entity_Expr;
Fld_26 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Item_242 : Internal_Entity_Case_Expr_Alternative;
Bare_Item_242 : Bare_Case_Expr_Alternative;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1144 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1144;
            procedure Finalizer_Scope_2837 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_24);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_26);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2837;
            procedure Finalizer_Scope_2838 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2838;
            procedure Finalizer_Scope_2839 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_2839;
            procedure Finalizer_Scope_2841 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_2841;
            procedure Finalizer_Scope_2842 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_2842;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7297 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 7258 '<FieldAccess for resolve_names_internal_with_eq at ast.py:16290>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16290
--# expr-start 7254 '<FieldAccess for expr at ast.py:16290>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16290
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Case_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7254
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7257 '<Predicate on BaseTypeDecl.is_discrete_type at ast.py:16291>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16291
--# expr-start 7256 '<FieldAccess for type_var at ast.py:16291>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16291
--# expr-start 7255 '<FieldAccess for expr at ast.py:16291>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16291
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Case_Expr_F_Expr;
--# expr-done 7255
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1);
--# expr-done 7256
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Pred := Solver.Create_Predicate (Fld_2, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin)); 
--# expr-done 7257
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_3 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7258
Block_Var := Fld_3; 
--# expr-start 7296 '<And at ast.py:16294>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16294
--# expr-start 7261 '<Bind at ast.py:16295>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16295
--# expr-start 7259 '<FieldAccess for type_var at ast.py:16295>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16295
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7259
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 7260 '<FieldAccess for expected_type_var at ast.py:16295>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16295
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7260
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result := Solver.Create_Unify (Fld_4, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16295") else null)); 
--# expr-done 7261
--# expr-start 7295 '<LogicAll at ast.py:16296>' Logic_Boolean_Op_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16296
--# expr-start 7294 '<Map at ast.py:16296>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16296
   --# expr-start 7262 '<FieldAccess for cases at ast.py:16296>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16296
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Case_Expr_Alternative_List (Node => Ent.Node.Case_Expr_F_Cases, Info => Ent.Info);
--# expr-done 7262
Fld_7 := Fld_6.Node;
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_7 = null then
      Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Case_Expr_Alternative_List := Fld_7;
      begin
         for Untyped_Item_242 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_242 := Untyped_Item_242; 
Is_Null := Bare_Item_242 = null; 
if Is_Null then
   If_Result := No_Entity_Case_Expr_Alternative;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_242); 
   If_Result := As_Entity;
end if;
                  Item_242 := If_Result; 
   --# scope-start
               --# bind alt Item_242
      --# expr-start 7293 '<And at ast.py:16299>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16299
--# expr-start 7290 '<And at ast.py:16299>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16299
--# expr-start 7287 '<And at ast.py:16299>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16299
--# expr-start 7282 '<LogicAll at ast.py:16299>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16299
--# expr-start 7281 '<Map at ast.py:16299>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16299
   --# expr-start 7263 '<FieldAccess for choices at ast.py:16299>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16299
      if Item_242.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Alternatives_List (Node => Item_242.Node.Case_Expr_Alternative_F_Choices, Info => Item_242.Info);
--# expr-done 7263
Fld_10 := Fld_9.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_10 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_10;
      begin
         for Untyped_Item_243 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_243 := Untyped_Item_243; 
Is_Null_1 := Bare_Item_243 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_11 := Fld_9.Info;
As_Entity_1 := (Info => Fld_11, Node => Bare_Item_243); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_243 := If_Result_1; 
   --# scope-start
               --# bind c Item_243
      --# expr-start 7280 '<Match at ast.py:16299>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16299
      if Item_243.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_243; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind e Match_0
--# expr-start 7278 '<If at ast.py:16301>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16301
--# expr-start 7267 '<Not at ast.py:16302>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16302
--# expr-start 7266 '<IsNull at ast.py:16302>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16302
--# expr-start 7264 '<Cast to Name at ast.py:16302>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16302
Cast_Expr_1 := Match_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 7264
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Name then
   --# scope-start
--# expr-start 7265 '<FieldAccess for name_designated_type at ast.py:16302>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16302
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_18 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7265
Scope_Result := Fld_18;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Is_Null_2 := Result_Var.Node = null; 
--# expr-done 7266
Not_Val := not (Is_Null_2); 
--# expr-done 7267
if Not_Val then
   --# expr-start 7269 '<FieldAccess for xref_no_overloading at ast.py:16304>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16304
--# expr-start 7268 '<Cast to Name at ast.py:16304>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16304
Cast_Expr := Match_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 7268
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_12 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result.Node, Env => Env, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 7269
   If_Result_2 := Fld_12;
else
   --# expr-start 7277 '<And at ast.py:16306>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16306
--# expr-start 7275 '<And at ast.py:16306>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16306
--# expr-start 7273 '<Bind at ast.py:16306>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16306
--# expr-start 7270 '<FieldAccess for expected_type_var at ast.py:16306>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16306
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Match_0.Node);
--# expr-done 7270
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 7272 '<FieldAccess for type_val at ast.py:16306>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16306
--# expr-start 7271 '<FieldAccess for expr at ast.py:16306>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16306
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Self.Case_Expr_F_Expr;
--# expr-done 7271
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_15 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_14);
--# end
--# expr-done 7272
Bind_Result_1 := Solver.Create_Assign (Fld_13, Fld_15, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16306") else null)); 
--# expr-done 7273
--# expr-start 7274 '<FieldAccess for sub_equation at ast.py:16307>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16307
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_16 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 7274
And_Pred := Create_And (Bind_Result_1, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16306") else null)); 
--# expr-done 7275
--# expr-start 7276 '<FieldAccess for matches_expected_type at ast.py:16308>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16308
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_17 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Match_0.Node, Origin => Origin);
--# end
--# expr-done 7276
And_Pred_1 := Create_And (And_Pred, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16306") else null)); 
--# expr-done 7277
   If_Result_2 := And_Pred_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 7278
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2839;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind t Match_1
--# expr-start 7279 '<FieldAccess for xref_equation at ast.py:16312>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16312
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_19 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 7279
Let_Result_1 := Fld_19; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2841;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16314") else null)); 
Let_Result_2 := True_Rel; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2842;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;
--# expr-done 7280
         declare
            Item_To_Append : constant Logic_Equation := Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2838;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7281
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16299") else null)); 
--# expr-done 7282
--# expr-start 7286 '<Bind at ast.py:16320>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16320
--# expr-start 7283 '<FieldAccess for expected_type_var at ast.py:16320>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16320
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7283
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
--# expr-start 7285 '<FieldAccess for expected_type_var at ast.py:16320>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16320
--# expr-start 7284 '<FieldAccess for expr at ast.py:16320>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16320
      if Item_242.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Expr (Node => Item_242.Node.Case_Expr_Alternative_F_Expr, Info => Item_242.Info);
--# expr-done 7284
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_21.Node);
--# expr-done 7285
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
Bind_Result_2 := Solver.Create_Unify (Fld_20, Fld_22, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16320") else null)); 
--# expr-done 7286
And_Pred_2 := Create_And (Logic_Boolean_Op, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16299") else null)); 
--# expr-done 7287
--# expr-start 7289 '<FieldAccess for sub_equation at ast.py:16321>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16321
--# expr-start 7288 '<FieldAccess for expr at ast.py:16321>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16321
      if Item_242.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Expr (Node => Item_242.Node.Case_Expr_Alternative_F_Expr, Info => Item_242.Info);
--# expr-done 7288
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_24 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_23.Node, Env => Env, Origin => Origin, E_Info => Fld_23.Info);
--# end
--# expr-done 7289
And_Pred_3 := Create_And (And_Pred_2, Fld_24, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16299") else null)); 
--# expr-done 7290
--# expr-start 7292 '<FieldAccess for matches_expected_type at ast.py:16322>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16322
--# expr-start 7291 '<FieldAccess for expr at ast.py:16322>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16322
      if Item_242.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Expr (Node => Item_242.Node.Case_Expr_Alternative_F_Expr, Info => Item_242.Info);
--# expr-done 7291
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_26 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_25.Node, Origin => Origin);
--# end
--# expr-done 7292
And_Pred_4 := Create_And (And_Pred_3, Fld_26, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16299") else null)); 
--# expr-done 7293
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_4;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2837;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 7294
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16296") else null)); 
--# expr-done 7295
And_Pred_5 := Create_And (Bind_Result, Logic_Boolean_Op_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16294") else null)); 
--# expr-done 7296
Let_Result_4 := And_Pred_5; Inc_Ref (Let_Result_4);
--# end
--# expr-done 7297
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1144;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1144;
                     Finalizer_Scope_2837;
                     Finalizer_Scope_2838;
                     Finalizer_Scope_2839;
                     Finalizer_Scope_2841;
                     Finalizer_Scope_2842;
            raise;
      end;
   return Property_Result;
end Case_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_If_Expr
   --
      procedure Initialize_Fields_For_If_Expr
        (Self : Bare_If_Expr
         ; If_Expr_F_Cond_Expr : Bare_Expr
         ; If_Expr_F_Then_Expr : Bare_Expr
         ; If_Expr_F_Alternatives : Bare_Elsif_Expr_Part_List
         ; If_Expr_F_Else_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Cond_Expr
              (Self);
            Self.If_Expr_F_Cond_Expr := If_Expr_F_Cond_Expr;
            Self.If_Expr_F_Then_Expr := If_Expr_F_Then_Expr;
            Self.If_Expr_F_Alternatives := If_Expr_F_Alternatives;
            Self.If_Expr_F_Else_Expr := If_Expr_F_Else_Expr;
      end Initialize_Fields_For_If_Expr;
   function If_Expr_F_Cond_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Cond_Expr;
   end;
   function If_Expr_F_Then_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Then_Expr;
   end;
   function If_Expr_F_Alternatives
     (Node : Bare_If_Expr) return Bare_Elsif_Expr_Part_List
   is
   begin
         return Node.If_Expr_F_Alternatives;
   end;
   function If_Expr_F_Else_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Else_Expr;
   end;
--# property-start IfExpr.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16172
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Has_Context_Free_Type
  (Node : Bare_If_Expr
  )
   return Boolean
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end If_Expr_P_Has_Context_Free_Type;
--# end
--# property-start IfExpr.dependent_exprs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16174
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Dependent_Exprs
  (Node : Bare_If_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1138 with Inline_Always;
      Fld : Internal_Entity_Expr;
Singleton : Internal_Entity_Expr_Array_Access;
Fld_1 : Internal_Entity_Elsif_Expr_Part_List;
Fld_2 : Bare_Elsif_Expr_Part_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Elsif_Expr_Part;
Fld_4 : Internal_Entity_Expr;
Item_244 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_244 : Bare_Elsif_Expr_Part;
Map_Result : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
Fld_5 : Internal_Entity_Expr;
Singleton_1 : Internal_Entity_Expr_Array_Access;
Concat_Result_1 : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1138 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1138;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7305 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7304 '<Concat at ast.py:16178>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16178
--# expr-start 7302 '<Concat at ast.py:16176>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16176
--# expr-start 7298 '<FieldAccess for then_expr at ast.py:16176>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16176
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 7298
                Singleton := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
--# expr-start 7301 '<Map at ast.py:16177>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16177
   --# expr-start 7299 '<FieldAccess for alternatives at ast.py:16177>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16177
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 7299
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_2;
      begin
         for Untyped_Item_244 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_244 := Untyped_Item_244; 
Is_Null := Bare_Item_244 = null; 
if Is_Null then
   If_Result := No_Entity_Elsif_Expr_Part;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_244); 
   If_Result := As_Entity;
end if;
                  Item_244 := If_Result; 
   --# scope-start
               --# bind a Item_244
      --# expr-start 7300 '<FieldAccess for then_expr at ast.py:16177>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16177
      if Item_244.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Item_244.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_244.Info);
--# expr-done 7300
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7301
Concat_Result := Concat (Singleton, Map_Result); 
--# expr-done 7302
--# expr-start 7303 '<FieldAccess for else_expr at ast.py:16178>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16178
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 7303
                Singleton_1 := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton_1.Items (1) := Fld_5;
Concat_Result_1 := Concat (Concat_Result, Singleton_1); 
--# expr-done 7304
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7305
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1138;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1138;
            raise;
      end;
   return Property_Result;
end If_Expr_P_Dependent_Exprs;
--# end
--# property-start IfExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16180
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Xref_Equation
  (Node : Bare_If_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1139 with Inline_Always;
            procedure Finalizer_Scope_2845 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_9 : Internal_Entity_Elsif_Expr_Part_List;
Fld_10 : Bare_Elsif_Expr_Part_List;
Fld_11 : Internal_Entity_Info;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Elsif_Expr_Part;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Var;
Fld_14 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_17 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Item_245 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_245 : Bare_Elsif_Expr_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_21 : Internal_Entity_Expr;
Fld_22 : Logic_Equation;
Fld_23 : Bare_Expr;
Fld_24 : Logic_Var;
Fld_25 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_26 : Internal_Entity_Expr;
Fld_27 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_28 : Bare_Expr;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_29 : Logic_Var;
Pred : Logic_Equation;
Fld_30 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_31 : Logic_Var;
Node_For_Entity : Bare_Base_Type_Decl;
As_Entity_1 : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
Fld_32 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_11 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1139 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_27);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_30);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_32);
                     Dec_Ref (And_Pred_10);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1139;
            procedure Finalizer_Scope_2845 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_5);
            end Finalizer_Scope_2845;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7361 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7360 '<And at ast.py:16184>' And_Pred_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
--# expr-start 7349 '<And at ast.py:16184>' And_Pred_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
--# expr-start 7335 '<And at ast.py:16184>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
--# expr-start 7318 '<And at ast.py:16184>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
--# expr-start 7315 '<And at ast.py:16184>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
--# expr-start 7312 '<And at ast.py:16184>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
--# expr-start 7309 '<Bind at ast.py:16184>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
--# expr-start 7307 '<FieldAccess for expected_type_var at ast.py:16184>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
--# expr-start 7306 '<FieldAccess for cond_expr at ast.py:16184>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.If_Expr_F_Cond_Expr;
--# expr-done 7306
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 7307
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 7308 '<FieldAccess for bool_type at ast.py:16184>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16184
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7308
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16184") else null)); 
--# expr-done 7309
--# expr-start 7311 '<FieldAccess for sub_equation at ast.py:16185>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16185
--# expr-start 7310 '<FieldAccess for cond_expr at ast.py:16185>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16185
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Cond_Expr, Info => Ent.Info);
--# expr-done 7310
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 7311
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16184") else null)); 
--# expr-done 7312
--# expr-start 7314 '<FieldAccess for matches_expected_formal_prim_type at ast.py:16186>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16186
--# expr-start 7313 '<FieldAccess for cond_expr at ast.py:16186>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16186
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Cond_Expr, Info => Ent.Info);
--# expr-done 7313
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 7314
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16184") else null)); 
--# expr-done 7315
--# expr-start 7317 '<FieldAccess for sub_equation at ast.py:16189>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16189
--# expr-start 7316 '<FieldAccess for then_expr at ast.py:16189>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16189
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 7316
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 7317
And_Pred_2 := Create_And (And_Pred_1, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16184") else null)); 
--# expr-done 7318
--# expr-start 7334 '<LogicAll at ast.py:16191>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16191
--# expr-start 7333 '<Map at ast.py:16191>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16191
   --# expr-start 7319 '<FieldAccess for alternatives at ast.py:16191>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16191
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 7319
Fld_10 := Fld_9.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_10 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_10;
      begin
         for Untyped_Item_245 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_245 := Untyped_Item_245; 
Is_Null := Bare_Item_245 = null; 
if Is_Null then
   If_Result := No_Entity_Elsif_Expr_Part;
else
Fld_11 := Fld_9.Info;
As_Entity := (Info => Fld_11, Node => Bare_Item_245); 
   If_Result := As_Entity;
end if;
                  Item_245 := If_Result; 
   --# scope-start
               --# bind elsif Item_245
      --# expr-start 7332 '<And at ast.py:16194>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16194
--# expr-start 7329 '<And at ast.py:16194>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16194
--# expr-start 7326 '<And at ast.py:16194>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16194
--# expr-start 7323 '<Bind at ast.py:16194>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16194
--# expr-start 7321 '<FieldAccess for expected_type_var at ast.py:16194>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16194
--# expr-start 7320 '<FieldAccess for cond_expr at ast.py:16194>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16194
      if Item_245.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Expr (Node => Item_245.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_245.Info);
--# expr-done 7320
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_12.Node);
--# expr-done 7321
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 7322 '<FieldAccess for bool_type at ast.py:16194>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16194
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_14 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7322
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_13, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16194") else null)); 
--# expr-done 7323
--# expr-start 7325 '<FieldAccess for sub_equation at ast.py:16195>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16195
--# expr-start 7324 '<FieldAccess for cond_expr at ast.py:16195>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16195
      if Item_245.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Expr (Node => Item_245.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_245.Info);
--# expr-done 7324
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_16 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_15.Node, Env => Env, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 7325
And_Pred_3 := Create_And (Bind_Result_1, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16194") else null)); 
--# expr-done 7326
--# expr-start 7328 '<FieldAccess for matches_expected_formal_prim_type at ast.py:16196>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16196
--# expr-start 7327 '<FieldAccess for cond_expr at ast.py:16196>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16196
      if Item_245.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Expr (Node => Item_245.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_245.Info);
--# expr-done 7327
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_18 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_17.Node, Origin => Origin);
--# end
--# expr-done 7328
And_Pred_4 := Create_And (And_Pred_3, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16194") else null)); 
--# expr-done 7329
--# expr-start 7331 '<FieldAccess for sub_equation at ast.py:16197>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16197
--# expr-start 7330 '<FieldAccess for then_expr at ast.py:16197>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16197
      if Item_245.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Item_245.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_245.Info);
--# expr-done 7330
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_19.Node, Env => Env, Origin => Origin, E_Info => Fld_19.Info);
--# end
--# expr-done 7331
And_Pred_5 := Create_And (And_Pred_4, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16194") else null)); 
--# expr-done 7332
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_5;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2845;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7333
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16191") else null)); 
--# expr-done 7334
And_Pred_6 := Create_And (And_Pred_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16184") else null)); 
--# expr-done 7335
--# expr-start 7348 '<If at ast.py:16200>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16200
--# expr-start 7338 '<Not at ast.py:16201>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16201
--# expr-start 7337 '<IsNull at ast.py:16201>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16201
--# expr-start 7336 '<FieldAccess for else_expr at ast.py:16201>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16201
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Self.If_Expr_F_Else_Expr;
--# expr-done 7336
Is_Null_1 := Fld_28 = null; 
--# expr-done 7337
Not_Val := not (Is_Null_1); 
--# expr-done 7338
if Not_Val then
   --# expr-start 7340 '<FieldAccess for sub_equation at ast.py:16204>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16204
--# expr-start 7339 '<FieldAccess for else_expr at ast.py:16204>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16204
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 7339
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_21.Node, Env => Env, Origin => Origin, E_Info => Fld_21.Info);
--# end
--# expr-done 7340
   If_Result_1 := Fld_22;
else
   --# expr-start 7347 '<And at ast.py:16207>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16207
--# expr-start 7344 '<Bind at ast.py:16207>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16207
--# expr-start 7342 '<FieldAccess for expected_type_var at ast.py:16207>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16207
--# expr-start 7341 '<FieldAccess for then_expr at ast.py:16207>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16207
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Self.If_Expr_F_Then_Expr;
--# expr-done 7341
      if Fld_23 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_23);
--# expr-done 7342
Fld_24.Value := No_Entity;
Entity_Vars.Reset (Fld_24);
--# expr-start 7343 '<FieldAccess for bool_type at ast.py:16207>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16207
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_25 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7343
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_25.Node,
         Info => Fld_25.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_24, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16207") else null)); 
--# expr-done 7344
--# expr-start 7346 '<FieldAccess for matches_expected_formal_prim_type at ast.py:16208>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16208
--# expr-start 7345 '<FieldAccess for then_expr at ast.py:16208>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16208
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 7345
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_27 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_26.Node, Origin => Origin);
--# end
--# expr-done 7346
And_Pred_7 := Create_And (Bind_Result_2, Fld_27, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16207") else null)); 
--# expr-done 7347
   If_Result_1 := And_Pred_7;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7348
And_Pred_8 := Create_And (And_Pred_6, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16184") else null)); 
--# expr-done 7349
--# expr-start 7359 '<Or at ast.py:16211>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16211
--# expr-start 7353 '<And at ast.py:16212>' And_Pred_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16212
--# expr-start 7351 '<Predicate on AdaNode.is_not_null at ast.py:16212>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16212
--# expr-start 7350 '<FieldAccess for expected_type_var at ast.py:16212>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16212
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7350
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
Pred := Solver.Create_Predicate (Fld_29, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 7351
--# expr-start 7352 '<FieldAccess for expected_type_equation at ast.py:16213>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16213
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IfExpr.expected_type_equation
Fld_30 := Libadalang.Implementation.If_Expr_P_Expected_Type_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7352
And_Pred_9 := Create_And (Pred, Fld_30, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16212") else null)); 
--# expr-done 7353
--# expr-start 7358 '<And at ast.py:16215>' And_Pred_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16215
--# expr-start 7356 '<Bind at ast.py:16215>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16215
--# expr-start 7354 '<FieldAccess for expected_type_var at ast.py:16215>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16215
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7354
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
--# expr-start 7355 '<No BaseTypeDecl at ast.py:16215>' Node_For_Entity /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16215
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 7355
Is_Null_2 := Node_For_Entity = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Base_Type_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Node_For_Entity); 
   If_Result_2 := As_Entity_1;
end if;
      Cast_Result_3 := Create_Internal_Entity
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_31, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16215") else null)); 
--# expr-done 7356
--# expr-start 7357 '<FieldAccess for no_expected_type_equation at ast.py:16216>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16216
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IfExpr.no_expected_type_equation
Fld_32 := Libadalang.Implementation.If_Expr_P_No_Expected_Type_Equation (Node => Ent.Node, Env => Env, Origin => Origin);
--# end
--# expr-done 7357
And_Pred_10 := Create_And (Bind_Result_3, Fld_32, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16215") else null)); 
--# expr-done 7358
Or_Pred := Create_Or (And_Pred_9, And_Pred_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16211") else null)); 
--# expr-done 7359
And_Pred_11 := Create_And (And_Pred_8, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16184") else null)); 
--# expr-done 7360
Let_Result := And_Pred_11; Inc_Ref (Let_Result);
--# end
--# expr-done 7361
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1139;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1139;
                     Finalizer_Scope_2845;
            raise;
      end;
   return Property_Result;
end If_Expr_P_Xref_Equation;
--# end
--# property-start IfExpr.expected_type_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16220
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Expected_Type_Equation
  (Node : Bare_If_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1140 with Inline_Always;
            procedure Finalizer_Scope_2846 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Var;
Fld_7 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
And_Pred_2 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_12 : Bare_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_13 : Internal_Entity_Elsif_Expr_Part_List;
Fld_14 : Bare_Elsif_Expr_Part_List;
Fld_15 : Internal_Entity_Info;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Elsif_Expr_Part;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Logic_Var;
Fld_18 : Logic_Var;
Bind_Result_3 : Logic_Equation;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Item_246 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_246 : Bare_Elsif_Expr_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_5 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1140 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1140;
            procedure Finalizer_Scope_2846 is
            begin
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2846;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7396 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7395 '<And at ast.py:16227>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16227
--# expr-start 7384 '<And at ast.py:16227>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16227
--# expr-start 7364 '<Bind at ast.py:16227>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16227
--# expr-start 7362 '<FieldAccess for type_var at ast.py:16227>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16227
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7362
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7363 '<FieldAccess for expected_type_var at ast.py:16227>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16227
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7363
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Bind_Result := Solver.Create_Unify (Fld, Fld_1, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16227") else null)); 
--# expr-done 7364
--# expr-start 7383 '<If at ast.py:16227>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16227
--# expr-start 7367 '<Not at ast.py:16228>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16228
--# expr-start 7366 '<IsNull at ast.py:16228>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16228
--# expr-start 7365 '<FieldAccess for else_expr at ast.py:16228>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16228
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.If_Expr_F_Else_Expr;
--# expr-done 7365
Is_Null := Fld_12 = null; 
--# expr-done 7366
Not_Val := not (Is_Null); 
--# expr-done 7367
if Not_Val then
   --# expr-start 7382 '<And at ast.py:16230>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16230
--# expr-start 7379 '<And at ast.py:16230>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16230
--# expr-start 7376 '<And at ast.py:16230>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16230
--# expr-start 7371 '<Bind at ast.py:16230>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16230
--# expr-start 7369 '<FieldAccess for expected_type_var at ast.py:16230>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16230
--# expr-start 7368 '<FieldAccess for else_expr at ast.py:16230>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16230
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.If_Expr_F_Else_Expr;
--# expr-done 7368
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 7369
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 7370 '<FieldAccess for expected_type_var at ast.py:16230>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16230
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7370
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result_1 := Solver.Create_Unify (Fld_3, Fld_4, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16230") else null)); 
--# expr-done 7371
--# expr-start 7375 '<Bind at ast.py:16231>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16231
--# expr-start 7373 '<FieldAccess for expected_type_var at ast.py:16231>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16231
--# expr-start 7372 '<FieldAccess for then_expr at ast.py:16231>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16231
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.If_Expr_F_Then_Expr;
--# expr-done 7372
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_5);
--# expr-done 7373
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 7374 '<FieldAccess for expected_type_var at ast.py:16231>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16231
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7374
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_2 := Solver.Create_Unify (Fld_6, Fld_7, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16231") else null)); 
--# expr-done 7375
And_Pred := Create_And (Bind_Result_1, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16230") else null)); 
--# expr-done 7376
--# expr-start 7378 '<FieldAccess for matches_expected_formal_type at ast.py:16232>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16232
--# expr-start 7377 '<FieldAccess for else_expr at ast.py:16232>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16232
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 7377
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 7378
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16230") else null)); 
--# expr-done 7379
--# expr-start 7381 '<FieldAccess for matches_expected_formal_type at ast.py:16233>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16233
--# expr-start 7380 '<FieldAccess for then_expr at ast.py:16233>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16233
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 7380
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_11 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_10.Node, Origin => Origin);
--# end
--# expr-done 7381
And_Pred_2 := Create_And (And_Pred_1, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16230") else null)); 
--# expr-done 7382
   If_Result := And_Pred_2;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16235") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 7383
And_Pred_3 := Create_And (Bind_Result, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16227") else null)); 
--# expr-done 7384
--# expr-start 7394 '<LogicAll at ast.py:16236>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16236
--# expr-start 7393 '<Map at ast.py:16236>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16236
   --# expr-start 7385 '<FieldAccess for alternatives at ast.py:16236>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16236
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 7385
Fld_14 := Fld_13.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_14 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_14;
      begin
         for Untyped_Item_246 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_246 := Untyped_Item_246; 
Is_Null_1 := Bare_Item_246 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Elsif_Expr_Part;
else
Fld_15 := Fld_13.Info;
As_Entity := (Info => Fld_15, Node => Bare_Item_246); 
   If_Result_1 := As_Entity;
end if;
                  Item_246 := If_Result_1; 
   --# scope-start
               --# bind elsif Item_246
      --# expr-start 7392 '<And at ast.py:16238>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16238
--# expr-start 7389 '<Bind at ast.py:16238>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16238
--# expr-start 7387 '<FieldAccess for expected_type_var at ast.py:16238>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16238
--# expr-start 7386 '<FieldAccess for then_expr at ast.py:16238>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16238
      if Item_246.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Item_246.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_246.Info);
--# expr-done 7386
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_16.Node);
--# expr-done 7387
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
--# expr-start 7388 '<FieldAccess for expected_type_var at ast.py:16238>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16238
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7388
Fld_18.Value := No_Entity;
Entity_Vars.Reset (Fld_18);
Bind_Result_3 := Solver.Create_Unify (Fld_17, Fld_18, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16238") else null)); 
--# expr-done 7389
--# expr-start 7391 '<FieldAccess for matches_expected_formal_type at ast.py:16239>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16239
--# expr-start 7390 '<FieldAccess for then_expr at ast.py:16239>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16239
      if Item_246.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Item_246.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_246.Info);
--# expr-done 7390
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_20 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_19.Node, Origin => Origin);
--# end
--# expr-done 7391
And_Pred_4 := Create_And (Bind_Result_3, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16238") else null)); 
--# expr-done 7392
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_4;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2846;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7393
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16236") else null)); 
--# expr-done 7394
And_Pred_5 := Create_And (And_Pred_3, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16227") else null)); 
--# expr-done 7395
Let_Result := And_Pred_5; Inc_Ref (Let_Result);
--# end
--# expr-done 7396
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1140;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1140;
                     Finalizer_Scope_2846;
            raise;
      end;
   return Property_Result;
end If_Expr_P_Expected_Type_Equation;
--# end
--# property-start IfExpr.no_expected_type_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16242
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_No_Expected_Type_Equation
  (Node : Bare_If_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1141 with Inline_Always;
            procedure Finalizer_Scope_2849 with Inline_Always;
            procedure Finalizer_Scope_2851 with Inline_Always;
      Var_Expr : Bare_Expr;
Fld : Bare_Expr;
Singleton : Bare_Expr_Array_Access;
Fld_1 : Bare_Elsif_Expr_Part_List;
Fld_2 : Bare_Expr;
Item_249 : Bare_Elsif_Expr_Part;
Map_Result : Bare_Expr_Array_Access;
Concat_Result : Bare_Expr_Array_Access;
Fld_3 : Bare_Expr;
Singleton_1 : Bare_Expr_Array_Access;
Scope_Result : Bare_Expr_Array_Access;
Result_Var : Bare_Expr_Array_Access;
Concat_Result_1 : Bare_Expr_Array_Access;
Fld_4 : Boolean;
Map_Result_1 : Bare_Expr_Array_Access;
Fld_5 : Logic_Var;
Pred : Logic_Equation;
Fld_6 : Logic_Var;
Pred_1 : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1141 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1141;
            procedure Finalizer_Scope_2849 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_2849;
            procedure Finalizer_Scope_2851 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_2851;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7417 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7416 '<LogicAll at ast.py:16256>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16256
--# expr-start 7415 '<Map at ast.py:16256>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16256
   --# expr-start 7405 '<Filter at ast.py:16254>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16254
   --# expr-start 7403 '<Concat at ast.py:16252>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16252
--# expr-start 7401 '<Concat at ast.py:16250>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16250
--# expr-start 7397 '<FieldAccess for then_expr at ast.py:16250>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16250
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.If_Expr_F_Then_Expr;
--# expr-done 7397
                Singleton := Create_Bare_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
--# expr-start 7400 '<Map at ast.py:16251>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16251
   --# expr-start 7398 '<FieldAccess for alternatives at ast.py:16251>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16251
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.If_Expr_F_Alternatives;
--# expr-done 7398
      declare
         Map_Result_Vec : Bare_Expr_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Bare_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_1;
      begin
         for Untyped_Item_249 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_249 := Untyped_Item_249; 
   --# scope-start
               --# bind a Item_249
      --# expr-start 7399 '<FieldAccess for then_expr at ast.py:16251>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16251
      if Item_249 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Item_249.Elsif_Expr_Part_F_Then_Expr;
--# expr-done 7399
         declare
            Item_To_Append : constant Bare_Expr := Fld_2;
         begin
            Bare_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Expr_Array
           (Items_Count => Natural (Bare_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7400
Concat_Result := Concat (Singleton, Map_Result); 
--# expr-done 7401
--# expr-start 7402 '<FieldAccess for else_expr at ast.py:16253>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16253
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.If_Expr_F_Else_Expr;
--# expr-done 7402
Var_Expr := Fld_3;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton_1 := Create_Bare_Expr_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr;
Scope_Result := Singleton_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2849;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result_1 := Concat (Concat_Result, Result_Var); 
--# expr-done 7403
      declare
         Map_Result_1_Vec : Bare_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Expr_Array_Access := Concat_Result_1;
      begin
         for Item_248 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_248
         --# expr-start 7404 '<FieldAccess for has_context_free_type at ast.py:16255>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16255
      if Item_248 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Item_248);
--# end
--# expr-done 7404
         if Fld_4 then
         declare
            Item_To_Append : constant Bare_Expr := Item_248;
         begin
            Bare_Expr_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Bare_Expr_Array
           (Items_Count => Natural (Bare_Expr_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Expr_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Expr_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Expr_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 7405
      declare
         Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Expr_Array_Access := Map_Result_1;
      begin
         for Item_247 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_247
      --# expr-start 7414 '<Or at ast.py:16258>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16258
--# expr-start 7407 '<Predicate on BaseTypeDecl.is_universal_type at ast.py:16258>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16258
--# expr-start 7406 '<FieldAccess for type_var at ast.py:16258>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16258
      if Item_247 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_247);
--# expr-done 7406
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Pred := Solver.Create_Predicate (Fld_5, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate); 
--# expr-done 7407
--# expr-start 7413 '<And at ast.py:16259>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16259
--# expr-start 7409 '<Predicate on AdaNode.is_not_null at ast.py:16259>' Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16259
--# expr-start 7408 '<FieldAccess for type_var at ast.py:16259>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16259
      if Item_247 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_247);
--# expr-done 7408
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Pred_1 := Solver.Create_Predicate (Fld_6, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 7409
--# expr-start 7412 '<Bind at ast.py:16260>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16260
--# expr-start 7410 '<FieldAccess for type_var at ast.py:16260>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16260
      if Item_247 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_247);
--# expr-done 7410
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 7411 '<FieldAccess for type_var at ast.py:16260>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16260
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7411
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result := Solver.Create_Propagate (Fld_7, Fld_8, Logic_Functor_614'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16260") else null)); 
--# expr-done 7412
And_Pred := Create_And (Pred_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16259") else null)); 
--# expr-done 7413
Or_Pred := Create_Or (Pred, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16258") else null)); 
--# expr-done 7414
         declare
            Item_To_Append : constant Logic_Equation := Or_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2851;
         end loop;
      end;
         Map_Result_2 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_2_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 7415
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16256") else null)); 
--# expr-done 7416
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 7417
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1141;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1141;
                     Finalizer_Scope_2849;
                     Finalizer_Scope_2851;
            raise;
      end;
   return Property_Result;
end If_Expr_P_No_Expected_Type_Equation;
--# end
   --
   --  Primitives for Bare_Contract_Cases
   --
      procedure Initialize_Fields_For_Contract_Cases
        (Self : Bare_Contract_Cases
         ; Contract_Cases_F_Contract_Cases : Bare_Contract_Case_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Contract_Cases_F_Contract_Cases := Contract_Cases_F_Contract_Cases;
      end Initialize_Fields_For_Contract_Cases;
   function Contract_Cases_F_Contract_Cases
     (Node : Bare_Contract_Cases) return Bare_Contract_Case_Assoc_List
   is
   begin
         return Node.Contract_Cases_F_Contract_Cases;
   end;
   --
   --  Primitives for Bare_Decl_Expr
   --
      procedure Decl_Expr_Pre_Env_Actions
        (Self            : Bare_Decl_Expr;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_121 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Decl_Expr
        (Self : Bare_Decl_Expr
         ; Decl_Expr_F_Decls : Bare_Basic_Decl_List
         ; Decl_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Decl_Expr_F_Decls := Decl_Expr_F_Decls;
            Self.Decl_Expr_F_Expr := Decl_Expr_F_Expr;
      end Initialize_Fields_For_Decl_Expr;
   function Decl_Expr_F_Decls
     (Node : Bare_Decl_Expr) return Bare_Basic_Decl_List
   is
   begin
         return Node.Decl_Expr_F_Decls;
   end;
   function Decl_Expr_F_Expr
     (Node : Bare_Decl_Expr) return Bare_Expr
   is
   begin
         return Node.Decl_Expr_F_Expr;
   end;
--# property-start DeclExpr.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12765
pragma Warnings (Off, "is not referenced");
 function Decl_Expr_P_Has_Context_Free_Type
  (Node : Bare_Decl_Expr
  )
   return Boolean
is
   Self : Bare_Decl_Expr  := Bare_Decl_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7419 '<FieldAccess for has_context_free_type at ast.py:12765>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12765
--# expr-start 7418 '<FieldAccess for expr at ast.py:12765>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12765
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Decl_Expr_F_Expr;
--# expr-done 7418
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Fld);
--# end
--# expr-done 7419
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Decl_Expr_P_Has_Context_Free_Type;
--# end
--# property-start DeclExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12767
pragma Warnings (Off, "is not referenced");
 function Decl_Expr_P_Xref_Equation
  (Node : Bare_Decl_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Decl_Expr  := Bare_Decl_Expr (Node);
     Ent : Internal_Entity_Decl_Expr :=
       Internal_Entity_Decl_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_971 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Fld_8 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_971 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_971;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7434 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7433 '<And at ast.py:12769>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12769
--# expr-start 7428 '<And at ast.py:12769>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12769
--# expr-start 7423 '<bind at ast.py:12770>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12770
--# expr-start 7420 '<FieldAccess for children_env at ast.py:12770>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12770
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7420
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 7422 '<FieldAccess for sub_equation at ast.py:12770>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12770
--# expr-start 7421 '<FieldAccess for expr at ast.py:12770>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12770
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Ent.Node.Decl_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7421
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 7422
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7423
--# expr-start 7427 '<Bind at ast.py:12771>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12771
--# expr-start 7425 '<FieldAccess for expected_type_var at ast.py:12771>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12771
--# expr-start 7424 '<FieldAccess for expr at ast.py:12771>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12771
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Decl_Expr_F_Expr;
--# expr-done 7424
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_3);
--# expr-done 7425
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 7426 '<FieldAccess for expected_type_var at ast.py:12771>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12771
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7426
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result := Solver.Create_Unify (Fld_4, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12771") else null)); 
--# expr-done 7427
And_Pred := Create_And (Dyn_Var_Bind_Result, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12769") else null)); 
--# expr-done 7428
--# expr-start 7432 '<Bind at ast.py:12772>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12772
--# expr-start 7430 '<FieldAccess for type_var at ast.py:12772>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12772
--# expr-start 7429 '<FieldAccess for expr at ast.py:12772>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12772
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Decl_Expr_F_Expr;
--# expr-done 7429
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 7430
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 7431 '<FieldAccess for type_var at ast.py:12772>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12772
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7431
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result_1 := Solver.Create_Unify (Fld_7, Fld_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12772") else null)); 
--# expr-done 7432
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12769") else null)); 
--# expr-done 7433
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7434
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_971;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_971;
            raise;
      end;
   return Property_Result;
end Decl_Expr_P_Xref_Equation;
--# end
--# property-start DeclExpr._env_trans_parent_121 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12761
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_121
  (Node : Bare_Decl_Expr
  )
   return Boolean
is
   Self : Bare_Decl_Expr  := Bare_Decl_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_121;
--# end
   --
   --  Primitives for Bare_Membership_Expr
   --
      procedure Initialize_Fields_For_Membership_Expr
        (Self : Bare_Membership_Expr
         ; Membership_Expr_F_Expr : Bare_Expr
         ; Membership_Expr_F_Op : Bare_Op
         ; Membership_Expr_F_Membership_Exprs : Bare_Expr_Alternatives_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Membership_Expr_F_Expr := Membership_Expr_F_Expr;
            Self.Membership_Expr_F_Op := Membership_Expr_F_Op;
            Self.Membership_Expr_F_Membership_Exprs := Membership_Expr_F_Membership_Exprs;
      end Initialize_Fields_For_Membership_Expr;
   function Membership_Expr_F_Expr
     (Node : Bare_Membership_Expr) return Bare_Expr
   is
   begin
         return Node.Membership_Expr_F_Expr;
   end;
   function Membership_Expr_F_Op
     (Node : Bare_Membership_Expr) return Bare_Op
   is
   begin
         return Node.Membership_Expr_F_Op;
   end;
   function Membership_Expr_F_Membership_Exprs
     (Node : Bare_Membership_Expr) return Bare_Expr_Alternatives_List
   is
   begin
         return Node.Membership_Expr_F_Membership_Exprs;
   end;
--# property-start MembershipExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13426
pragma Warnings (Off, "is not referenced");
 function Membership_Expr_P_Xref_Equation
  (Node : Bare_Membership_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Membership_Expr  := Bare_Membership_Expr (Node);
     Ent : Internal_Entity_Membership_Expr :=
       Internal_Entity_Membership_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1000 with Inline_Always;
            procedure Finalizer_Scope_2852 with Inline_Always;
            procedure Finalizer_Scope_2854 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Name;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr_Alternatives_List;
Fld_5 : Bare_Expr_Alternatives_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Fld_9 : Boolean;
Fld_10 : Bare_Expr;
Fld_11 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_12 : Bare_Expr;
Fld_13 : Logic_Var;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_14 : Bare_Expr;
Fld_15 : Logic_Var;
Fld_16 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_17 : Bare_Expr;
Fld_18 : Logic_Equation;
And_Pred_2 : Logic_Equation;
If_Result_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Fld_19 : Logic_Var;
Fld_20 : Bare_Expr;
Fld_21 : Logic_Var;
Bind_Result_3 : Logic_Equation;
Fld_22 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_23 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Item_250 : Internal_Entity_Expr;
Bare_Item_250 : Bare_Expr;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_6 : Logic_Equation;
            procedure Finalizer_Scope_1000 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_6);
            end Finalizer_Scope_1000;
            procedure Finalizer_Scope_2852 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2852;
            procedure Finalizer_Scope_2854 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2854;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7473 '<And at ast.py:13427>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13427
--# expr-start 7440 '<And at ast.py:13427>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13427
--# expr-start 7437 '<Bind at ast.py:13427>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13427
--# expr-start 7435 '<FieldAccess for type_var at ast.py:13427>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13427
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7435
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7436 '<FieldAccess for bool_type at ast.py:13427>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13427
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7436
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13427") else null)); 
--# expr-done 7437
--# expr-start 7439 '<FieldAccess for sub_equation at ast.py:13428>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13428
--# expr-start 7438 '<FieldAccess for expr at ast.py:13428>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13428
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Membership_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7438
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7439
And_Pred := Create_And (Bind_Result, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13427") else null)); 
--# expr-done 7440
--# expr-start 7472 '<LogicAll at ast.py:13429>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13429
--# expr-start 7471 '<Map at ast.py:13429>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13429
   --# expr-start 7441 '<FieldAccess for membership_exprs at ast.py:13429>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13429
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr_Alternatives_List (Node => Ent.Node.Membership_Expr_F_Membership_Exprs, Info => Ent.Info);
--# expr-done 7441
Fld_5 := Fld_4.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Expr_Alternatives_List := Fld_5;
      begin
         for Untyped_Item_250 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_250 := Untyped_Item_250; 
Is_Null := Bare_Item_250 = null; 
if Is_Null then
   If_Result := No_Entity_Expr;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_250); 
   If_Result := As_Entity;
end if;
                  Item_250 := If_Result; 
   --# scope-start
               --# bind m Item_250
--# expr-start 7442 '<Cast to Name at ast.py:13430>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13430
Cast_Expr := Item_250; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 7442
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
--# expr-start 7443 '<FieldAccess for name_designated_type at ast.py:13430>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13430
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_7 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7443
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 7462 '<And at ast.py:13433>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13433
--# expr-start 7445 '<FieldAccess for xref_no_overloading at ast.py:13433>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13433
--# expr-start 7444 '<Cast to Name at ast.py:13433>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13433
Cast_Expr_1 := Item_250; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 7444
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_2.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 7445
--# expr-start 7461 '<Cond at ast.py:13434>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13434
--# expr-start 7446 '<FieldAccess for is_tagged_type at ast.py:13442>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13442
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7446
if Fld_9 then
   --# expr-start 7453 '<And at ast.py:13443>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13443
--# expr-start 7449 '<Bind at ast.py:13443>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13443
--# expr-start 7448 '<FieldAccess for expected_type_var at ast.py:13443>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13443
--# expr-start 7447 '<FieldAccess for expr at ast.py:13443>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13443
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Membership_Expr_F_Expr;
--# expr-done 7447
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_10);
--# expr-done 7448
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Cast_Expr_2 := No_Entity_Base_Type_Decl; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_11, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13443") else null)); 
--# expr-done 7449
--# expr-start 7452 '<Predicate on BaseTypeDecl.is_tagged_type_with_deref at ast.py:13444>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13444
--# expr-start 7451 '<FieldAccess for type_var at ast.py:13445>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13445
--# expr-start 7450 '<FieldAccess for expr at ast.py:13445>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13445
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Membership_Expr_F_Expr;
--# expr-done 7450
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_12);
--# expr-done 7451
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
Pred := Solver.Create_Predicate (Fld_13, Create_Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate (Origin)); 
--# expr-done 7452
And_Pred_1 := Create_And (Bind_Result_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13443") else null)); 
--# expr-done 7453
   If_Result_1 := And_Pred_1;
else
   --# expr-start 7460 '<And at ast.py:13449>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13449
--# expr-start 7457 '<Bind at ast.py:13449>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13449
--# expr-start 7455 '<FieldAccess for expected_type_var at ast.py:13449>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13449
--# expr-start 7454 '<FieldAccess for expr at ast.py:13449>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13449
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Self.Membership_Expr_F_Expr;
--# expr-done 7454
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_14);
--# expr-done 7455
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
--# expr-start 7456 '<FieldAccess for base_subtype at ast.py:13450>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13450
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_16 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7456
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_15, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13449") else null)); 
--# expr-done 7457
--# expr-start 7459 '<FieldAccess for matches_expected_formal_type at ast.py:13451>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13451
--# expr-start 7458 '<FieldAccess for expr at ast.py:13451>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13451
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Self.Membership_Expr_F_Expr;
--# expr-done 7458
      if Fld_17 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_18 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_17, Origin => Origin);
--# end
--# expr-done 7459
And_Pred_2 := Create_And (Bind_Result_2, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13449") else null)); 
--# expr-done 7460
   If_Result_1 := And_Pred_2;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7461
And_Pred_3 := Create_And (Fld_8, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13433") else null)); 
--# expr-done 7462
Scope_Result_1 := And_Pred_3;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2854;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 7470 '<And at ast.py:13455>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13455
--# expr-start 7468 '<And at ast.py:13455>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13455
--# expr-start 7466 '<Bind at ast.py:13456>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13456
--# expr-start 7463 '<FieldAccess for expected_type_var at ast.py:13456>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13456
      if Item_250.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Item_250.Node);
--# expr-done 7463
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
--# expr-start 7465 '<FieldAccess for type_var at ast.py:13456>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13456
--# expr-start 7464 '<FieldAccess for expr at ast.py:13456>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13456
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Self.Membership_Expr_F_Expr;
--# expr-done 7464
      if Fld_20 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_20);
--# expr-done 7465
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
Bind_Result_3 := Solver.Create_Unify (Fld_19, Fld_21, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13456") else null)); 
--# expr-done 7466
--# expr-start 7467 '<FieldAccess for sub_equation at ast.py:13457>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13457
      if Item_250.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_250.Node, Env => Env, Origin => Origin, E_Info => Item_250.Info);
--# end
--# expr-done 7467
And_Pred_4 := Create_And (Bind_Result_3, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13455") else null)); 
--# expr-done 7468
--# expr-start 7469 '<FieldAccess for matches_expected_type at ast.py:13458>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13458
      if Item_250.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_23 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Item_250.Node, Origin => Origin);
--# end
--# expr-done 7469
And_Pred_5 := Create_And (And_Pred_4, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13455") else null)); 
--# expr-done 7470
   Result_Var_1 := And_Pred_5;
end if;
      Inc_Ref (Result_Var_1);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2852;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7471
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13429") else null)); 
--# expr-done 7472
And_Pred_6 := Create_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:13427") else null)); 
--# expr-done 7473
         Property_Result := And_Pred_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1000;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1000;
                     Finalizer_Scope_2852;
                     Finalizer_Scope_2854;
            raise;
      end;
   return Property_Result;
end Membership_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Name
   --
      procedure Initialize_Fields_For_Name
        (Self : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Name;
--# property-start Name.enclosing_defining_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13815
pragma Warnings (Off, "is not referenced");
 function Name_P_Enclosing_Defining_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1014 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1014 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1014;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7477 '<Cast to DefiningName at ast.py:13817>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13817
--# expr-start 7476 '<Find at ast.py:13816>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13816
   --# expr-start 7474 '<FieldAccess for parents at ast.py:13816>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13816
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 7474
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_251 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_251
      --# expr-start 7475 '<IsA DefiningName at ast.py:13816>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13816
Is_A := Item_251.Node /= null 
and then Item_251.Node.Kind in Ada_Defining_Name_Range; 
--# expr-done 7475
      if Is_A then
         Find_Result := Item_251; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 7476
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Defining_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Defining_Name
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result := No_Entity_Defining_Name;
   end if;
--# expr-done 7477
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_1014;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1014;
            raise;
      end;
   return Property_Result;
end Name_P_Enclosing_Defining_Name;
--# end
--# property-start Name.is_defining /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13824
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Defining
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Dotted_Name;
Is_A : Boolean;
Fld : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result : Boolean;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Is_A_2 : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_4 : Bare_Base_Id;
Scope_Result : Bare_Base_Id;
Result_Var : Bare_Base_Id;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7488 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7478 '<IsA DefiningName at ast.py:13831>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13831
Is_A := Self /= null 
and then Self.Kind in Ada_Defining_Name_Range; 
--# expr-done 7478
if Is_A then
   If_Result := True;
else
   --# expr-start 7480 '<IsA DefiningName at ast.py:13835>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13835
--# expr-start 7479 '<parent at ast.py:13835>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13835
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 7479
Is_A_1 := Fld /= null 
and then Fld.Kind in Ada_Defining_Name_Range; 
--# expr-done 7480
   If_Result := Is_A_1;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 7483 '<IsA DefiningName at ast.py:13839>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13839
--# expr-start 7482 '<parent at ast.py:13839>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13839
--# expr-start 7481 '<parent at ast.py:13839>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13839
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 7481
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 7482
Is_A_2 := Fld_2 /= null 
and then Fld_2.Kind in Ada_Defining_Name_Range; 
--# expr-done 7483
if Is_A_2 then
   --# expr-start 7487 '<Eq at ast.py:13840>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13840
--# expr-start 7485 '<Cast to DottedName at ast.py:13840>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13840
--# expr-start 7484 '<parent at ast.py:13840>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13840
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 7484
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7485
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 7486 '<FieldAccess for suffix at ast.py:13840>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13840
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Var_Expr.Dotted_Name_F_Suffix;
--# expr-done 7486
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
      Cast_Result_1 := Result_Var;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 7487
   If_Result_1 := Is_Equal;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 7488
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Defining;
--# end
--# property-start '[dispatcher]Name.parent_scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Parent_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_Explicit_Deref .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.parent_scope
                  Property_Result := Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.parent_scope
                  Property_Result := Defining_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.parent_scope
                  Property_Result := Dotted_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.parent_scope
                  Property_Result := End_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.parent_scope
                  Property_Result := Base_Id_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Parent_Scope;
--# end
--# property-start Name.name_is /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13852
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Is
  (Node : Bare_Name
      ; Sym : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Boolean;
      Var_Expr : Symbol_Type;
Fld : Symbol_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7491 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7489 '<FieldAccess for name_symbol at ast.py:13857>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13857
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 7489
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind ns Var_Expr
--# expr-start 7490 '<Eq at ast.py:13857>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13857
Is_Equal := Var_Expr = Sym; 
--# expr-done 7490
Scope_Result := Is_Equal;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 7491
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Name_Is;
--# end
--# property-start Name.is_direct_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13859
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Direct_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Entity;
Is_A : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7497 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7492 '<FieldAccess for is_call at ast.py:13866>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13866
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7492
if Fld then
   --# expr-start 7496 '<Not at ast.py:13867>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13867
--# expr-start 7495 '<IsA AccessToSubpDef at ast.py:13867>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13867
--# expr-start 7494 '<parent at ast.py:13867>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13867
--# expr-start 7493 '<FieldAccess for called_subp_spec at ast.py:13867>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13867
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_subp_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7493
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 7494
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 7495
Not_Val := not (Is_A); 
--# expr-done 7496
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 7497
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Direct_Call;
--# end
--# property-start Name.is_access_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13870
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Access_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Entity;
Is_A : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7502 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7498 '<FieldAccess for is_call at ast.py:13877>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13877
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7498
if Fld then
   --# expr-start 7501 '<IsA AccessToSubpDef at ast.py:13878>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13878
--# expr-start 7500 '<parent at ast.py:13878>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13878
--# expr-start 7499 '<FieldAccess for called_subp_spec at ast.py:13878>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13878
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_subp_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7499
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 7500
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 7501
   If_Result := Is_A;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 7502
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Access_Call;
--# end
--# property-start Name.is_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13881
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7508 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7504 '<Not at ast.py:13887>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13887
--# expr-start 7503 '<FieldAccess for is_defining at ast.py:13887>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13887
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 7503
Not_Val := not (Fld); 
--# expr-done 7504
if Not_Val then
   --# expr-start 7507 '<Not at ast.py:13888>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13888
--# expr-start 7506 '<IsNull at ast.py:13888>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13888
--# expr-start 7505 '<FieldAccess for called_subp_spec at ast.py:13888>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13888
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_subp_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7505
Is_Null := Fld_1.Node = null; 
--# expr-done 7506
Not_Val_1 := not (Is_Null); 
--# expr-done 7507
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 7508
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Call;
--# end
--# property-start Name.is_dot_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13891
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Dot_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Internal_Metadata;
Fld_4 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7515 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7510 '<Not at ast.py:13898>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13898
--# expr-start 7509 '<FieldAccess for is_defining at ast.py:13898>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13898
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 7509
Not_Val := not (Fld); 
--# expr-done 7510
if Not_Val then
   --# expr-start 7514 '<FieldAccess for dottable_subp at ast.py:13899>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13899
--# expr-start 7513 '<FieldAccess for md at ast.py:13899>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13899
--# expr-start 7512 '<FieldAccess for info at ast.py:13899>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13899
--# expr-start 7511 '<FieldAccess for referenced_decl at ast.py:13899>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13899
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7511
Fld_2 := Fld_1.Info;
--# expr-done 7512
Fld_3 := Fld_2.Md;
--# expr-done 7513
Fld_4 := Fld_3.Dottable_Subp;
--# expr-done 7514
   If_Result := Fld_4;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 7515
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Dot_Call;
--# end
--# property-start Name.failsafe_referenced_def_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13902
pragma Warnings (Off, "is not referenced");
 function Name_P_Failsafe_Referenced_Def_Name
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Def
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Def;
            procedure Finalizer_Scope_2858 with Inline_Always;
      Block_Var : Internal_Refd_Decl;
Block_Var_1 : Internal_Entity_Defining_Name;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Symbol_Type;
Fld : Internal_Refd_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Symbol_Type;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Fld_4 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Fld_6 : Ref_Result_Kind;
New_Struct : Internal_Refd_Def;
Let_Result : Internal_Refd_Def;
            procedure Finalizer_Scope_2858 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2858;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7525 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7516 '<FieldAccess for failsafe_referenced_decl at ast.py:13909>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13909
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.failsafe_referenced_decl
Fld := Libadalang.Implementation.Name_P_Failsafe_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7516
Block_Var := Fld; 
--# bind ref_decl Block_Var
--# expr-start 7517 '<FieldAccess for decl at ast.py:13911>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13911
Fld_1 := Block_Var.Decl;
--# expr-done 7517
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ref_decl Var_Expr
--# expr-start 7518 '<FieldAccess for name_symbol at ast.py:13912>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13912
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7518
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= null then
   --# scope-start
   --# bind rel_name Var_Expr_2
--# expr-start 7521 '<Find at ast.py:13913>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13913
   --# expr-start 7519 '<FieldAccess for defining_names at ast.py:13913>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13913
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7519
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_3;
      begin
         for Item_252 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_252
      --# expr-start 7520 '<FieldAccess for name_is at ast.py:13914>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13914
      if Item_252.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_252.Node, Sym => Var_Expr_2);
--# end
--# expr-done 7520
      if Fld_4 then
         Find_Result := Item_252; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 7521
Scope_Result := Find_Result;
   --# end
      Finalizer_Scope_2858;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 7522 '<FieldAccess for defining_name at ast.py:13916>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13916
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7522
   Result_Var_1 := Fld_5;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Block_Var_1 := Result_Var_2; 
--# bind def_name Block_Var_1
--# expr-start 7524 '<New RefdDef at ast.py:13920>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13920
--# expr-start 7523 '<FieldAccess for kind at ast.py:13920>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13920
Fld_6 := Block_Var.Kind;
--# expr-done 7523
New_Struct := (Def_Name => Block_Var_1, Kind => Fld_6); 
--# expr-done 7524
Let_Result := New_Struct; 
--# end
--# expr-done 7525
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2858;
            raise;
      end;
   return Property_Result;
end Name_P_Failsafe_Referenced_Def_Name;
--# end
--# property-start Name.referenced_defining_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13922
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Defining_Name
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2862 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Symbol_Type;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Fld_3 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2862 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2862;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7532 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7526 '<FieldAccess for referenced_decl at ast.py:13929>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13929
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7526
Block_Var := Fld; 
--# bind ref_decl Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ref_decl Var_Expr
--# expr-start 7527 '<FieldAccess for name_symbol at ast.py:13930>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13930
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7527
Var_Expr_2 := Fld_1;
if Var_Expr_2 /= null then
   --# scope-start
   --# bind rel_name Var_Expr_2
--# expr-start 7530 '<Find at ast.py:13931>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13931
   --# expr-start 7528 '<FieldAccess for defining_names at ast.py:13931>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13931
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7528
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_2;
      begin
         for Item_253 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_253
      --# expr-start 7529 '<FieldAccess for name_is at ast.py:13932>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13932
      if Item_253.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_253.Node, Sym => Var_Expr_2);
--# end
--# expr-done 7529
      if Fld_3 then
         Find_Result := Item_253; 
      end if;
   --# end
               exit when Fld_3;
         end loop;
      end;
--# expr-done 7530
Scope_Result := Find_Result;
   --# end
      Finalizer_Scope_2862;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 7531 '<FieldAccess for defining_name at ast.py:13934>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13934
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7531
   Result_Var_1 := Fld_4;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 7532
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2862;
            raise;
      end;
   return Property_Result;
end Name_P_Referenced_Defining_Name;
--# end
--# property-start Name.gnat_xref_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13936
pragma Warnings (Off, "is not referenced");
 function Name_P_Gnat_Xref_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Block_Var : Internal_Entity_Defining_Name;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Var_Expr_8 : Internal_Entity_Basic_Decl;
Var_Expr_9 : Internal_Entity_Basic_Decl;
Var_Expr_10 : Internal_Entity_Defining_Name;
Var_Expr_11 : Internal_Entity_Body_Node;
Fld : Boolean;
Fld_1 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Param_Spec;
Fld_3 : Internal_Entity_Defining_Name;
Is_A_1 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Body_Node;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Defining_Name;
Scope_Result_4 : Internal_Entity_Defining_Name;
Result_Var_4 : Internal_Entity_Defining_Name;
Is_A_2 : Boolean;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Cast_Expr_3 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Defining_Name;
Scope_Result_6 : Internal_Entity_Defining_Name;
Result_Var_6 : Internal_Entity_Defining_Name;
Is_A_3 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_7 : Boolean;
Is_A_4 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_8 : Boolean;
Is_A_5 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_9 : Boolean;
Fld_9 : Internal_Entity_Body_Node;
Fld_10 : Internal_Entity_Defining_Name;
Scope_Result_10 : Internal_Entity_Defining_Name;
Result_Var_10 : Internal_Entity_Defining_Name;
Scope_Result_11 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Defining_Name;
Result_Var_11 : Internal_Entity_Defining_Name;
Fld_12 : Internal_Entity_Defining_Name;
If_Result_1 : Internal_Entity_Defining_Name;
If_Result_2 : Internal_Entity_Defining_Name;
If_Result_3 : Internal_Entity_Defining_Name;
If_Result_4 : Internal_Entity_Defining_Name;
If_Result_5 : Internal_Entity_Defining_Name;
If_Result_6 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7557 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7533 '<FieldAccess for is_defining at ast.py:13943>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13943
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 7533
Var_Expr := Fld;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 7534 '<FieldAccess for enclosing_defining_name at ast.py:13944>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13944
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld_1 := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7534
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Block_Var := Result_Var; 
--# bind dn Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind dn Var_Expr_1
--# expr-start 7535 '<FieldAccess for basic_decl at ast.py:13946>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13946
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7535
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind bd Block_Var_1
--# expr-start 7556 '<Cond at ast.py:13948>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13948
Var_Expr_2 := Block_Var_1;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 7536 '<IsA ParamSpec at ast.py:13949>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13949
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 7536
Scope_Result_2 := Is_A;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 7538 '<FieldAccess for decl_param at ast.py:13950>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13950
--# expr-start 7537 '<Cast to ParamSpec at ast.py:13950>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13950
Cast_Expr := Block_Var_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Param_Spec;
   end if;
--# expr-done 7537
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.decl_param
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Decl_Param (Node => Cast_Result.Node, Param => Block_Var, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 7538
   If_Result_6 := Fld_3;
else
Var_Expr_3 := Block_Var_1;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 7539 '<IsA Body at ast.py:13952>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13952
Is_A_1 := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_Body_Node; 
--# expr-done 7539
Scope_Result_3 := Is_A_1;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
--# expr-start 7541 '<FieldAccess for decl_part at ast.py:13953>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13953
--# expr-start 7540 '<Cast to Body at ast.py:13953>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13953
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 7540
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_4 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 7541
Var_Expr_4 := Fld_4;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7542 '<FieldAccess for defining_name at ast.py:13953>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13953
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 7542
Scope_Result_4 := Fld_5;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Defining_Name;
end if;
   If_Result_5 := Result_Var_4;
else
Var_Expr_5 := Block_Var_1;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_5
--# expr-start 7543 '<IsA BaseTypeDecl at ast.py:13955>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13955
Is_A_2 := Var_Expr_5.Node /= null 
and then Var_Expr_5.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 7543
if Is_A_2 then
   --# expr-start 7545 '<FieldAccess for is_in_private_part at ast.py:13956>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13956
--# expr-start 7544 '<Cast to BaseTypeDecl at ast.py:13956>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13956
Cast_Expr_2 := Var_Expr_5; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7544
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 7545
   If_Result := Fld_6;
else
   If_Result := False;
end if;
Scope_Result_5 := If_Result;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
--# expr-start 7547 '<FieldAccess for previous_part at ast.py:13957>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13957
--# expr-start 7546 '<Cast to BaseTypeDecl at ast.py:13957>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13957
Cast_Expr_3 := Block_Var_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7546
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Cast_Result_3.Node, Go_To_Incomplete => True, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 7547
Var_Expr_6 := Fld_7;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7548 '<FieldAccess for defining_name at ast.py:13957>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13957
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 7548
Scope_Result_6 := Fld_8;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Defining_Name;
end if;
   If_Result_4 := Result_Var_6;
else
Var_Expr_7 := Block_Var_1;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_7
--# expr-start 7549 '<IsA ObjectDecl at ast.py:13959>' Is_A_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13959
Is_A_3 := Var_Expr_7.Node /= null 
and then Var_Expr_7.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 7549
Scope_Result_7 := Is_A_3;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := False;
end if;
if Result_Var_7 then
   If_Result_3 := No_Entity_Defining_Name;
else
Var_Expr_8 := Block_Var_1;
if Var_Expr_8 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_8
--# expr-start 7550 '<IsA NumberDecl at ast.py:13964>' Is_A_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13964
Is_A_4 := Var_Expr_8.Node /= null 
and then Var_Expr_8.Node.Kind in Ada_Number_Decl_Range; 
--# expr-done 7550
Scope_Result_8 := Is_A_4;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := False;
end if;
if Result_Var_8 then
   If_Result_2 := No_Entity_Defining_Name;
else
Var_Expr_9 := Block_Var_1;
if Var_Expr_9 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_9
--# expr-start 7551 '<IsA BasicDecl at ast.py:13968>' Is_A_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13968
Is_A_5 := Var_Expr_9.Node /= null 
and then Var_Expr_9.Node.Kind in Ada_Basic_Decl; 
--# expr-done 7551
Scope_Result_9 := Is_A_5;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := False;
end if;
if Result_Var_9 then
--# expr-start 7552 '<FieldAccess for body_part_for_decl at ast.py:13969>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13969
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7552
Var_Expr_11 := Fld_9;
if Var_Expr_11 /= No_Entity_Body_Node then
   --# scope-start
   --# bind bpe Var_Expr_11
--# expr-start 7553 '<FieldAccess for defining_name at ast.py:13969>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13969
      if Var_Expr_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_11.Node, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 7553
Scope_Result_10 := Fld_10;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := No_Entity_Defining_Name;
end if;
Var_Expr_10 := Result_Var_10;
if Var_Expr_10 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_10
Scope_Result_11 := Var_Expr_10;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   --# expr-start 7554 '<FieldAccess for defining_name at ast.py:13970>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13970
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7554
   Result_Var_11 := Fld_11;
end if;
   If_Result_1 := Result_Var_11;
else
   --# expr-start 7555 '<FieldAccess for referenced_defining_name at ast.py:13972>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_12 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7555
   If_Result_1 := Fld_12;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
--# expr-done 7556
Let_Result := If_Result_6; 
--# end
--# expr-done 7557
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Gnat_Xref_Decl;
--# end
--# property-start '[dispatcher]Name.all_env_els_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_All_Env_Els_Impl
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.all_env_els_impl
                  Property_Result := Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.all_env_els_impl
                  Property_Result := Defining_Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.all_env_els_impl
                  Property_Result := Dotted_Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.all_env_els_impl
                  Property_Result := Base_Id_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_All_Env_Els_Impl;
--# end
--# property-start Name.all_env_elements_internal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13985
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Env_Elements_Internal
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1026 with Inline_Always;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1026 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1026;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7563 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7562 '<bind at ast.py:13995>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13995
--# expr-start 7558 '<FieldAccess for origin_node at ast.py:13996>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13996
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7558
Bound_Origin := Fld; 
--# expr-start 7561 '<bind at ast.py:13997>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13997
--# expr-start 7559 '<FieldAccess for node_env at ast.py:13998>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13998
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7559
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 7560 '<FieldAccess for all_env_els_impl at ast.py:13999>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:13999
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Ent.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 7560
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7561
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 7562
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7563
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1026;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1026;
            raise;
      end;
   return Property_Result;
end Name_P_All_Env_Elements_Internal;
--# end
--# property-start Name.all_env_elements /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14007
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Env_Elements
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1027 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1027 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1027;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7565 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7564 '<FieldAccess for all_env_elements_internal at ast.py:14015>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14015
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Ent.Node, Seq => Seq, Seq_From => Seq_From, E_Info => Ent.Info);
--# end
--# expr-done 7564
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 7565
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1027;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1027;
            raise;
      end;
   return Property_Result;
end Name_P_All_Env_Elements;
--# end
--# property-start Name.first_corresponding_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14017
pragma Warnings (Off, "is not referenced");
 function Name_P_First_Corresponding_Decl
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1028 with Inline_Always;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Ada_Node;
Is_A : Boolean;
Fld_4 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1028 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1028;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7576 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7575 '<If at ast.py:14019>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14019
--# expr-start 7567 '<IsA DottedName at ast.py:14020>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14020
--# expr-start 7566 '<parent at ast.py:14020>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14020
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 7566
Is_A := Fld_3 /= null 
and then Fld_3.Kind in Ada_Dotted_Name_Range; 
--# expr-done 7567
if Is_A then
   --# expr-start 7568 '<FieldAccess for is_suffix at ast.py:14020>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14020
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_4 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 7568
   If_Result := Fld_4;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 7571 '<FieldAccess for first_corresponding_decl at ast.py:14021>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14021
--# expr-start 7570 '<Cast to DottedName at ast.py:14021>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14021
--# expr-start 7569 '<parent at ast.py:14021>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14021
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7569
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 7570
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.first_corresponding_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_First_Corresponding_Decl (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 7571
   If_Result_1 := Fld_1;
else
   --# expr-start 7574 '<Cast to BasicDecl at ast.py:14022>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14022
--# expr-start 7573 '<at at ast.py:14022>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14022
--# expr-start 7572 '<FieldAccess for all_env_elements_internal at ast.py:14022>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14022
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7572
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 7573
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7574
   If_Result_1 := Cast_Result_1;
end if;
--# expr-done 7575
Let_Result := If_Result_1; 
--# end
--# expr-done 7576
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1028;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1028;
            raise;
      end;
   return Property_Result;
end Name_P_First_Corresponding_Decl;
--# end
--# property-start Name.all_args_xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14025
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Args_Xref_Equation
  (Node : Bare_Name
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1029 with Inline_Always;
            procedure Finalizer_Scope_2877 with Inline_Always;
            procedure Finalizer_Scope_2878 with Inline_Always;
            procedure Finalizer_Scope_2879 with Inline_Always;
            procedure Finalizer_Scope_2880 with Inline_Always;
      Var_Expr : Internal_Entity_Call_Expr;
Var_Expr_1 : Internal_Entity_Assoc_List;
Var_Expr_2 : Internal_Entity_Name;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Item_254 : Internal_Entity_Basic_Assoc;
Bare_Item_254 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Scope_Result : Logic_Equation_Array_Access;
Result_Var : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_2 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1029 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1029;
            procedure Finalizer_Scope_2877 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_2877;
            procedure Finalizer_Scope_2878 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2878;
            procedure Finalizer_Scope_2879 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2879;
            procedure Finalizer_Scope_2880 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2880;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7586 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7585 '<And at ast.py:14035>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14035
--# expr-start 7577 '<Cast to CallExpr at ast.py:14035>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14035
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Call_Expr;
   end if;
--# expr-done 7577
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr
--# expr-start 7582 '<LogicAll at ast.py:14036>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14036
--# expr-start 7578 '<FieldAccess for params at ast.py:14036>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14036
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7578
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 7581 '<Map at ast.py:14036>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14036
Saved := Var_Expr_1; 
Fld_1 := Saved.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_1;
      begin
         for Untyped_Item_254 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_254 := Untyped_Item_254; 
Is_Null := Bare_Item_254 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Saved.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_254); 
   If_Result := As_Entity;
end if;
                  Item_254 := If_Result; 
   --# scope-start
               --# bind pa Item_254
      --# expr-start 7580 '<FieldAccess for sub_equation at ast.py:14037>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14037
--# expr-start 7579 '<FieldAccess for expr at ast.py:14037>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14037
      if Item_254.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_254.Node, E_Info => Item_254.Info);
--# end
--# expr-done 7579
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 7580
         declare
            Item_To_Append : constant Logic_Equation := Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2879;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7581
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2878;
   Result_Var := Scope_Result;
else
   Result_Var := No_Logic_Equation_Array_Type;
end if;
      Inc_Ref (Result_Var);
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Result_Var.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14036") else null)); 
--# expr-done 7582
Scope_Result_1 := Logic_Boolean_Op;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2877;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14039") else null)); 
   Result_Var_1 := True_Rel;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 7583 '<FieldAccess for parent_name at ast.py:14040>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14040
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_5 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 7583
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_2
--# expr-start 7584 '<FieldAccess for all_args_xref_equation at ast.py:14041>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14041
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_6 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Var_Expr_2.Node, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7584
Scope_Result_2 := Fld_6;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2880;
   Result_Var_2 := Scope_Result_2;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14042") else null)); 
   Result_Var_2 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_2);
And_Pred := Create_And (Result_Var_1, Result_Var_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14035") else null)); 
--# expr-done 7585
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7586
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1029;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1029;
                     Finalizer_Scope_2877;
                     Finalizer_Scope_2878;
                     Finalizer_Scope_2879;
                     Finalizer_Scope_2880;
            raise;
      end;
   return Property_Result;
end Name_P_All_Args_Xref_Equation;
--# end
--# property-start Name.bottom_up_name_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14045
pragma Warnings (Off, "is not referenced");
 function Name_P_Bottom_Up_Name_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1030 with Inline_Always;
            procedure Finalizer_Scope_2881 with Inline_Always;
            procedure Finalizer_Scope_2882 with Inline_Always;
            procedure Finalizer_Scope_2883 with Inline_Always;
            procedure Finalizer_Scope_2884 with Inline_Always;
      Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Explicit_Deref;
Match_2 : Internal_Entity_Qual_Expr;
Match_3 : Internal_Entity_Name;
Fld : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Equation;
False_Rel : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Internal_Entity_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Qual_Expr;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1030 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1030;
            procedure Finalizer_Scope_2881 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2881;
            procedure Finalizer_Scope_2882 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2882;
            procedure Finalizer_Scope_2883 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2883;
            procedure Finalizer_Scope_2884 is
            begin
                     Dec_Ref (False_Rel);
            end Finalizer_Scope_2884;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7593 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 7592 '<Match at ast.py:14047>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14047
--# expr-start 7588 '<as_entity at ast.py:14047>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14047
--# expr-start 7587 '<FieldAccess for innermost_name at ast.py:14047>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14047
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.innermost_name
Fld := Libadalang.Implementation.Name_P_Innermost_Name (Node => Self);
--# end
--# expr-done 7587
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 7588
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := If_Result; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 7589 '<FieldAccess for general_xref_equation at ast.py:14048>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14048
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.general_xref_equation
Fld_1 := Libadalang.Implementation.Call_Expr_P_General_Xref_Equation (Node => Match_0.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 7589
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2881;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Explicit_Deref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ed Match_1
--# expr-start 7590 '<FieldAccess for general_xref_equation at ast.py:14049>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14049
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ExplicitDeref.general_xref_equation
Fld_2 := Libadalang.Implementation.Explicit_Deref_P_General_Xref_Equation (Node => Match_1.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 7590
Let_Result_1 := Fld_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2882;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind qe Match_2
--# expr-start 7591 '<FieldAccess for general_xref_equation at ast.py:14050>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14050
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start QualExpr.general_xref_equation
Fld_3 := Libadalang.Implementation.Qual_Expr_P_General_Xref_Equation (Node => Match_2.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 7591
Let_Result_2 := Fld_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2883;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Reduce_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14051") else null)); 
Let_Result_3 := False_Rel; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2884;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 7592
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 7593
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1030;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1030;
                     Finalizer_Scope_2881;
                     Finalizer_Scope_2882;
                     Finalizer_Scope_2883;
                     Finalizer_Scope_2884;
            raise;
      end;
   return Property_Result;
end Name_P_Bottom_Up_Name_Equation;
--# end
--# property-start Name.innermost_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14054
pragma Warnings (Off, "is not referenced");
 function Name_P_Innermost_Name
  (Node : Bare_Name
  )
   return Bare_Name
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Bare_Name;
      Block_Var : Bare_Name;
Match_0 : Bare_Call_Expr;
Match_1 : Bare_Explicit_Deref;
Match_2 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Name;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Call_Expr;
Let_Result : Bare_Name;
Scope_Result : Bare_Name;
Cast_Result_1 : Bare_Explicit_Deref;
Let_Result_1 : Bare_Name;
Scope_Result_1 : Bare_Name;
Cast_Result_2 : Bare_Name;
Let_Result_2 : Bare_Name;
Scope_Result_2 : Bare_Name;
Match_Result : Bare_Name;
Is_A : Boolean;
Fld_2 : Bare_Name;
Is_A_1 : Boolean;
If_Result : Bare_Name;
If_Result_1 : Bare_Name;
Let_Result_3 : Bare_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7601 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 7596 '<Match at ast.py:14068>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14068
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 7594 '<FieldAccess for name at ast.py:14069>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14069
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Match_0.Call_Expr_F_Name;
--# expr-done 7594
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind ed Match_1
--# expr-start 7595 '<FieldAccess for prefix at ast.py:14070>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14070
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_1.Explicit_Deref_F_Prefix;
--# expr-done 7595
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 7596
Block_Var := Match_Result; 
--# bind name Block_Var
--# expr-start 7600 '<Cond at ast.py:14075>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14075
--# expr-start 7597 '<IsA CallExpr, ExplicitDeref at ast.py:14076>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14076
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Call_Expr_Range | Ada_Explicit_Deref_Range; 
--# expr-done 7597
if Is_A then
   --# expr-start 7598 '<FieldAccess for innermost_name at ast.py:14076>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14076
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.innermost_name
Fld_2 := Libadalang.Implementation.Name_P_Innermost_Name (Node => Block_Var);
--# end
--# expr-done 7598
   If_Result_1 := Fld_2;
else
--# expr-start 7599 '<IsA QualExpr at ast.py:14077>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14077
Is_A_1 := Block_Var /= null 
and then Block_Var.Kind in Ada_Qual_Expr_Range; 
--# expr-done 7599
if Is_A_1 then
   If_Result := Block_Var;
else
   If_Result := Self;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 7600
Let_Result_3 := If_Result_1; 
--# end
--# expr-done 7601
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Innermost_Name;
--# end
--# property-start Name.parent_name_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14081
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Name_Equation
  (Node : Bare_Name
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1032 with Inline_Always;
            procedure Finalizer_Scope_2894 with Inline_Always;
            procedure Finalizer_Scope_2895 with Inline_Always;
            procedure Finalizer_Scope_2897 with Inline_Always;
            procedure Finalizer_Scope_2898 with Inline_Always;
      Block_Var : Internal_Entity_Access_To_Subp_Def;
Block_Var_1 : Boolean;
Block_Var_2 : Boolean;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_2 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Match_0 : Bare_Call_Expr;
Match_1 : Bare_Explicit_Deref;
Match_2 : Bare_Name;
Var_Expr_6 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_7 : Internal_Entity_Name;
Var_Expr_8 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Result_Var : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Not_Val : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
False_Rel : Logic_Equation;
As_Entity : Internal_Entity_Call_Expr;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Call_Expr;
Fld_8 : Logic_Equation;
As_Entity_1 : Internal_Entity_Explicit_Deref;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Explicit_Deref;
Fld_9 : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Internal_Entity_Subp_Spec;
Scope_Result_6 : Internal_Entity_Subp_Spec;
Result_Var_6 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_12 : Logic_Var;
Fld_13 : Bare_Ada_Node;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Match_Prefix : Bare_Name;
Cast_Result_2 : Bare_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Cast_Result_3 : Bare_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Cast_Result_4 : Bare_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_14 : Internal_Entity_Name;
Is_A_1 : Boolean;
Is_Null_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_4 : Boolean;
Fld_15 : Logic_Equation;
Fld_16 : Logic_Equation;
Fld_17 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_18 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
Cast_Expr : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity_Call_Expr;
Fld_19 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_7 : Boolean;
Fld_20 : Logic_Equation;
Fld_21 : Logic_Equation;
If_Result_7 : Logic_Equation;
If_Result_8 : Logic_Equation;
Scope_Result_11 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_8 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null_4 : Boolean;
If_Result_9 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_1032 is
            begin
                     Dec_Ref (False_Rel);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_8);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1032;
            procedure Finalizer_Scope_2894 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2894;
            procedure Finalizer_Scope_2895 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2895;
            procedure Finalizer_Scope_2897 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2897;
            procedure Finalizer_Scope_2898 is
            begin
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_18);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
            end Finalizer_Scope_2898;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7644 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 7603 '<Cast to AccessToSubpDef at ast.py:14086>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14086
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7602 '<FieldAccess for access_def at ast.py:14086>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14086
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7602
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Access_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 7603
Block_Var := Cast_Result; 
--# bind as_subp_access Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 7605 '<FieldAccess for paramless at ast.py:14087>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14087
--# expr-start 7604 '<FieldAccess for subp_spec at ast.py:14087>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14087
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_1.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_1.Info);
--# expr-done 7604
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.paramless
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Fld_1.Node, Dottable_Subp => False, Can_Be => False);
--# end
--# expr-done 7605
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Block_Var_1 := Result_Var_1; 
--# bind is_paramless Block_Var_1
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 7607 '<FieldAccess for paramless at ast.py:14091>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14091
--# expr-start 7606 '<FieldAccess for subp_spec at ast.py:14091>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14091
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_2.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_2.Info);
--# expr-done 7606
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Fld_3.Node, Dottable_Subp => False, Can_Be => True);
--# end
--# expr-done 7607
Scope_Result_2 := Fld_4;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Block_Var_2 := Result_Var_2; 
--# bind can_be_paramless Block_Var_2
--# expr-start 7613 '<If at ast.py:14095>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14095
Var_Expr_3 := Typ;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7608 '<FieldAccess for is_iterable_type at ast.py:14099>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14099
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr_3.Node, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 7608
Scope_Result_5 := Fld_7;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
Var_Expr_4 := Typ;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7609 '<FieldAccess for iterable_comp_type at ast.py:14100>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14100
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_4.Node, Origin => Origin, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 7609
Scope_Result_3 := Fld_5;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
   If_Result := Result_Var_3;
else
Var_Expr_5 := Typ;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7612 '<FieldAccess for comp_type at ast.py:14101>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14101
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7611 '<Not at ast.py:14101>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14101
--# expr-start 7610 '<IsA ExplicitDeref at ast.py:14101>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14101
Is_A := Self /= null 
and then Self.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 7610
Not_Val := not (Is_A); 
--# expr-done 7611
--# property-call-start BaseTypeDecl.comp_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Var_Expr_5.Node, Is_Subscript => Not_Val, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 7612
Scope_Result_4 := Fld_6;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
   If_Result := Result_Var_4;
end if;
--# expr-done 7613
Block_Var_3 := If_Result; 
--# bind comp_type Block_Var_3
--# expr-start 7643 '<If at ast.py:14103>' If_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14103
--# expr-start 7614 '<IsNull at ast.py:14104>' Is_Null_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14104
Is_Null_4 := Typ.Node = null; 
--# expr-done 7614
if Is_Null_4 then
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14105") else null)); 
   If_Result_9 := False_Rel;
else
   --# expr-start 7642 '<And at ast.py:14107>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14107
--# expr-start 7626 '<Match at ast.py:14107>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14107
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_0 := Cast_Result_2; 
--# bind ce Match_0
--# expr-start 7616 '<FieldAccess for subscriptable_type_equation at ast.py:14109>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14109
--# expr-start 7615 '<as_entity at ast.py:14109>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14109
Is_Null := Match_0 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Call_Expr;
else
As_Entity := (Info => E_Info, Node => Match_0); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 7615
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.subscriptable_type_equation
Fld_8 := Libadalang.Implementation.Call_Expr_P_Subscriptable_Type_Equation (Node => If_Result_1.Node, Typ => Typ, Env => Env, Origin => Origin, E_Info => If_Result_1.Info);
--# end
--# expr-done 7616
Let_Result := Fld_8; Inc_Ref (Let_Result);
--# end
Scope_Result_7 := Let_Result;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2894;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_1 := Cast_Result_3; 
--# bind ed Match_1
--# expr-start 7622 '<And at ast.py:14112>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14112
--# expr-start 7618 '<FieldAccess for eq_for_type at ast.py:14112>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14112
--# expr-start 7617 '<as_entity at ast.py:14112>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14112
Is_Null_1 := Match_1 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Explicit_Deref;
else
As_Entity_1 := (Info => E_Info, Node => Match_1); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 7617
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ExplicitDeref.eq_for_type
Fld_9 := Libadalang.Implementation.Explicit_Deref_P_Eq_For_Type (Node => If_Result_2.Node, Typ => Typ, Env => Env, Origin => Origin, E_Info => If_Result_2.Info);
--# end
--# expr-done 7618
--# expr-start 7621 '<Bind at ast.py:14117>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14117
--# expr-start 7619 '<FieldAccess for subp_spec_var at ast.py:14117>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14117
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 7619
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Var_Expr_6 := Block_Var;
if Var_Expr_6 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 7620 '<FieldAccess for subp_spec at ast.py:14117>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14117
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_6.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_6.Info);
--# expr-done 7620
Scope_Result_6 := Fld_11;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Subp_Spec;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => Result_Var_6.Node,
         Info => Result_Var_6.Info);
Bind_Result := Solver.Create_Assign (Fld_10, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14117") else null)); 
--# expr-done 7621
And_Pred := Create_And (Fld_9, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14112") else null)); 
--# expr-done 7622
Let_Result_1 := And_Pred; Inc_Ref (Let_Result_1);
--# end
Scope_Result_8 := Let_Result_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2895;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
--# expr-start 7625 '<Bind at ast.py:14119>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14119
--# expr-start 7623 '<FieldAccess for type_var at ast.py:14119>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14119
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7623
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
--# expr-start 7624 '<FieldAccess for node at ast.py:14119>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14119
Fld_13 := No_Entity.Node;
--# expr-done 7624
Is_Null_2 := Fld_13 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity;
else
As_Entity_2 := (Info => E_Info, Node => Fld_13); 
   If_Result_3 := As_Entity_2;
end if;
Bind_Result_1 := Solver.Create_Assign (Fld_12, If_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14119") else null)); 
--# expr-done 7625
Let_Result_2 := Bind_Result_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_9 := Let_Result_2;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_2897;
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
end case;
--# expr-done 7626
--# expr-start 7627 '<FieldAccess for parent_name at ast.py:14120>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14120
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_14 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 7627
Var_Expr_7 := Fld_14;
if Var_Expr_7 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_7
--# expr-start 7641 '<Cond at ast.py:14121>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14121
--# expr-start 7628 '<IsA ExplicitDeref at ast.py:14122>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14122
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 7628
if Is_A_1 then
   --# expr-start 7630 '<Not at ast.py:14122>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14122
--# expr-start 7629 '<IsNull at ast.py:14122>' Is_Null_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14122
Is_Null_3 := Block_Var.Node = null; 
--# expr-done 7629
Not_Val_1 := not (Is_Null_3); 
--# expr-done 7630
   If_Result_4 := Not_Val_1;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 7636 '<Cond at ast.py:14126>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14126
if Block_Var_1 then
   --# expr-start 7631 '<FieldAccess for parent_name_equation at ast.py:14132>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14132
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_15 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_7.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 7631
   If_Result_6 := Fld_15;
else
if Block_Var_2 then
   --# expr-start 7634 '<Or at ast.py:14139>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14139
--# expr-start 7632 '<FieldAccess for parent_name_equation at ast.py:14140>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14140
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_16 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_7.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 7632
--# expr-start 7633 '<FieldAccess for parent_name_equation at ast.py:14141>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14141
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_17 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_7.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 7633
Or_Pred := Create_Or (Fld_16, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14139") else null)); 
--# expr-done 7634
   If_Result_5 := Or_Pred;
else
   --# expr-start 7635 '<FieldAccess for parent_name_equation at ast.py:14147>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14147
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_18 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_7.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 7635
   If_Result_5 := Fld_18;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 7636
   If_Result_8 := If_Result_6;
else
--# expr-start 7637 '<Cast to CallExpr at ast.py:14151>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14151
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 7637
Var_Expr_8 := Cast_Result_5;
if Var_Expr_8 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 7638 '<FieldAccess for check_array_slice at ast.py:14151>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14151
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.check_array_slice
Fld_19 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Var_Expr_8.Node, Typ => Typ, Origin => Origin, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 7638
Scope_Result_10 := Fld_19;
   --# end
   Result_Var_7 := Scope_Result_10;
else
   Result_Var_7 := False;
end if;
if Result_Var_7 then
   --# expr-start 7639 '<FieldAccess for parent_name_equation at ast.py:14152>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14152
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_20 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_7.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 7639
   If_Result_7 := Fld_20;
else
   --# expr-start 7640 '<FieldAccess for parent_name_equation at ast.py:14156>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14156
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_21 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_7.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 7640
   If_Result_7 := Fld_21;
end if;
      Inc_Ref (If_Result_7);
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 7641
Scope_Result_11 := If_Result_8;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_2898;
   Result_Var_8 := Scope_Result_11;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14158") else null)); 
   Result_Var_8 := True_Rel;
end if;
      Inc_Ref (Result_Var_8);
And_Pred_1 := Create_And (Match_Result, Result_Var_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14107") else null)); 
--# expr-done 7642
   If_Result_9 := And_Pred_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 7643
Let_Result_3 := If_Result_9; Inc_Ref (Let_Result_3);
--# end
--# expr-done 7644
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1032;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1032;
                     Finalizer_Scope_2894;
                     Finalizer_Scope_2895;
                     Finalizer_Scope_2897;
                     Finalizer_Scope_2898;
            raise;
      end;
   return Property_Result;
end Name_P_Parent_Name_Equation;
--# end
--# property-start Name.subtype_indication_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14162
pragma Warnings (Off, "is not referenced");
 function Name_P_Subtype_Indication_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1033 with Inline_Always;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1033 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1033;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7646 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7645 '<FieldAccess for xref_no_overloading at ast.py:14164>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14164
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7645
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 7646
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1033;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1033;
            raise;
      end;
   return Property_Result;
end Name_P_Subtype_Indication_Equation;
--# end
--# property-start Name.can_designate_primitive /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14166
pragma Warnings (Off, "is not referenced");
 function Name_P_Can_Designate_Primitive
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Name;
Match_1 : Bare_Renaming_Clause;
Match_2 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Boolean;
Fld_2 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Renaming_Clause;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7653 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 7652 '<Match at ast.py:14176>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14176
--# expr-start 7647 '<parent at ast.py:14176>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14176
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 7647
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind n Match_0
--# expr-start 7648 '<FieldAccess for can_designate_primitive at ast.py:14177>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14177
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.can_designate_primitive
Fld_1 := Libadalang.Implementation.Name_P_Can_Designate_Primitive (Node => Match_0);
--# end
--# expr-done 7648
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Renaming_Clause .. Ada_Synthetic_Renaming_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind r Match_1
--# expr-start 7651 '<Not at ast.py:14178>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14178
--# expr-start 7650 '<IsA PackageRenamingDecl at ast.py:14178>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14178
--# expr-start 7649 '<parent at ast.py:14178>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14178
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Match_1.Parent; 
--# expr-done 7649
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Package_Renaming_Decl_Range; 
--# expr-done 7650
Not_Val := not (Is_A); 
--# expr-done 7651
Let_Result_1 := Not_Val; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Range_Spec | Ada_Reverse_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := True; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 7652
Let_Result_3 := Match_Result; 
--# end
--# expr-done 7653
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Can_Designate_Primitive;
--# end
--# property-start Name.parent_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14182
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Name
  (Node : Bare_Name
      ; Stop_At : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_at Stop_At
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Is_Null : Boolean;
Cast_Expr : Bare_Name;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Name;
Let_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7659 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7658 '<If at ast.py:14187>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14187
--# expr-start 7654 '<IsNull at ast.py:14187>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14187
Is_Null := Stop_At = null; 
--# expr-done 7654
if Is_Null then
   If_Result := True;
else
   --# expr-start 7655 '<Eq at ast.py:14187>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14187
Cast_Expr := Self; 
      Cast_Result_1 := Cast_Expr;
Is_Equal := Cast_Result_1 = Stop_At; 
--# expr-done 7655
   If_Result := Is_Equal;
end if;
if If_Result then
   If_Result_1 := No_Entity_Name;
else
   --# expr-start 7657 '<Cast to Name at ast.py:14189>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14189
--# expr-start 7656 '<parent at ast.py:14189>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14189
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7656
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 7657
   If_Result_1 := Cast_Result;
end if;
--# expr-done 7658
Let_Result := If_Result_1; 
--# end
--# expr-done 7659
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Parent_Name;
--# end
--# property-start Name.parent_callexpr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14191
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Callexpr
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Call_Expr
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Call_Expr;
            procedure Finalizer_Scope_1036 with Inline_Always;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Call_Expr;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Is_A_1 : Boolean;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Id;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_3 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Find_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Call_Expr;
Let_Result_3 : Internal_Entity_Call_Expr;
            procedure Finalizer_Scope_1036 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1036;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7673 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 7672 '<Cast to CallExpr at ast.py:14216>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14216
--# expr-start 7671 '<Find at ast.py:14216>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14216
   --# expr-start 7669 '<TakeWhile at ast.py:14209>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14209
   --# expr-start 7660 '<FieldAccess for parents at ast.py:14209>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14209
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 7660
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
            Take_While_Ok : Boolean := False;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_256 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_256
--# expr-start 7661 '<IsA CallExpr at ast.py:14210>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14210
Is_A := Item_256.Node /= null 
and then Item_256.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 7661
if Is_A then
   If_Result_1 := True;
else
--# expr-start 7662 '<IsA DottedName, BaseId at ast.py:14211>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14211
Is_A_1 := Item_256.Node /= null 
and then Item_256.Node.Kind in Ada_Dotted_Name_Range | Ada_Base_Id; 
--# expr-done 7662
if Is_A_1 then
   --# expr-start 7668 '<Match at ast.py:14211>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14211
--# expr-start 7663 '<parent at ast.py:14211>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14211
      if Item_256.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Item_256.Node, E_Info => Item_256.Info);
--# expr-done 7663
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind pfx Match_0
--# expr-start 7665 '<Eq at ast.py:14212>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14212
--# expr-start 7664 '<FieldAccess for suffix at ast.py:14212>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14212
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 7664
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Is_Equiv := Equivalent (Cast_Result, Item_256); 
--# expr-done 7665
Let_Result := Is_Equiv; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind ce Match_1
--# expr-start 7667 '<Eq at ast.py:14213>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14213
--# expr-start 7666 '<FieldAccess for name at ast.py:14213>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14213
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Expr_F_Name, Info => Match_1.Info);
--# expr-done 7666
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Is_Equiv_1 := Equivalent (Cast_Result_1, Item_256); 
--# expr-done 7667
Let_Result_1 := Is_Equiv_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 7668
   If_Result := Match_Result;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
         Take_While_Ok := If_Result_1;
         if Take_While_Ok then
         declare
            Item_To_Append : constant Internal_Entity := Item_256;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
               exit when not Take_While_Ok;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7669
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_255 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_255
      --# expr-start 7670 '<IsA CallExpr at ast.py:14216>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14216
Is_A_2 := Item_255.Node /= null 
and then Item_255.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 7670
      if Is_A_2 then
         Find_Result := Item_255; 
      end if;
   --# end
               exit when Is_A_2;
         end loop;
      end;
--# expr-done 7671
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 7672
Let_Result_3 := Cast_Result_5; 
--# end
--# expr-done 7673
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_1036;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1036;
            raise;
      end;
   return Property_Result;
end Name_P_Parent_Callexpr;
--# end
--# property-start Name.is_range_attribute /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14218
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Range_Attribute
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Attribute_Ref;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Attribute_Ref;
As_Entity : Internal_Entity_Attribute_Ref;
Is_Null : Boolean;
If_Result : Internal_Entity_Attribute_Ref;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7679 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7674 '<Cast to AttributeRef at ast.py:14223>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14223
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7674
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind attr_ref Var_Expr
--# expr-start 7678 '<FieldAccess for name_is at ast.py:14225>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14225
--# expr-start 7676 '<FieldAccess for attribute at ast.py:14225>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14225
--# expr-start 7675 '<as_bare_entity at ast.py:14225>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14225
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity_Attribute_Ref;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 7675
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => If_Result.Node.Attribute_Ref_F_Attribute, Info => If_Result.Info);
--# expr-done 7676
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7677 '<Symbol Range at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 7677
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 7678
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 7679
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Range_Attribute;
--# end
--# property-start '[dispatcher]Name.scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_Explicit_Deref .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.scope
                  Property_Result := Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.scope
                  Property_Result := Defining_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.scope
                  Property_Result := Dotted_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.scope
                  Property_Result := End_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.scope
                  Property_Result := Base_Id_P_Scope
                    (Self
                        , Env
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Scope;
--# end
--# property-start '[dispatcher]Name.ref_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Ref_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Discrete_Subtype_Name | Ada_Synthetic_Identifier =>
                  --# property-call-start Name.ref_var
                  Property_Result := Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.ref_var
                  Property_Result := Attribute_Ref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.ref_var
                  Property_Result := Call_Expr_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.ref_var
                  Property_Result := Defining_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.ref_var
                  Property_Result := Dotted_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.ref_var
                  Property_Result := End_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.ref_var
                  Property_Result := Explicit_Deref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.ref_var
                  Property_Result := Qual_Expr_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Reduce_Attribute_Ref =>
                  --# property-call-start ReduceAttributeRef.ref_var
                  Property_Result := Reduce_Attribute_Ref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.ref_var
                  Property_Result := Single_Tok_Node_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.ref_var
                  Property_Result := Target_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Update_Attribute_Ref =>
                  --# property-call-start UpdateAttributeRef.ref_var
                  Property_Result := Update_Attribute_Ref_P_Ref_Var
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Ref_Var;
--# end
--# property-start '[dispatcher]Name.subp_spec_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Qual_Expr .. Ada_Reduce_Attribute_Ref | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.subp_spec_var
                  Property_Result := Name_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.subp_spec_var
                  Property_Result := Call_Expr_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.subp_spec_var
                  Property_Result := Dotted_Name_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.subp_spec_var
                  Property_Result := Explicit_Deref_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.subp_spec_var
                  Property_Result := Single_Tok_Node_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Subp_Spec_Var;
--# end
--# property-start '[dispatcher]Name.defines_subp_spec_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Qual_Expr .. Ada_Reduce_Attribute_Ref | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.defines_subp_spec_var
                  Property_Result := Name_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.defines_subp_spec_var
                  Property_Result := Call_Expr_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.defines_subp_spec_var
                  Property_Result := Dotted_Name_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.defines_subp_spec_var
                  Property_Result := Explicit_Deref_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.defines_subp_spec_var
                  Property_Result := Single_Tok_Node_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start '[dispatcher]Name.called_subp_spec' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Called_Subp_Spec
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Call_Expr .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.called_subp_spec
                  Property_Result := Name_P_Called_Subp_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.called_subp_spec
                  Property_Result := Attribute_Ref_P_Called_Subp_Spec
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Called_Subp_Spec;
--# end
--# property-start Name.referenced_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14285
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Refd_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7682 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7681 '<FieldAccess for decl at ast.py:14294>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14294
--# expr-start 7680 '<FieldAccess for referenced_decl_internal at ast.py:14294>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14294
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl_internal
Fld := Libadalang.Implementation.Name_P_Referenced_Decl_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7680
Fld_1 := Fld.Decl;
--# expr-done 7681
Let_Result := Fld_1; 
--# end
--# expr-done 7682
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Referenced_Decl;
--# end
--# property-start Name.failsafe_referenced_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14296
pragma Warnings (Off, "is not referenced");
 function Name_P_Failsafe_Referenced_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Decl;
      Fld : Internal_Refd_Decl;
New_Struct : Internal_Refd_Decl;
Try_Result : Internal_Refd_Decl;
Let_Result : Internal_Refd_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7686 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7685 '<Try at ast.py:14303>' Try_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14303
begin
   --# expr-start 7683 '<FieldAccess for referenced_decl_internal at ast.py:14304>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14304
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl_internal
Fld := Libadalang.Implementation.Name_P_Referenced_Decl_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7683
   Try_Result := Fld;
exception
   when Property_Error =>
      --# expr-start 7684 '<New RefdDecl at ast.py:14305>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14305
New_Struct := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 7684
      Try_Result := New_Struct;
end;
--# expr-done 7685
Let_Result := Try_Result; 
--# end
--# expr-done 7686
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Failsafe_Referenced_Decl;
--# end
--# property-start Name.get_referenced_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14308
pragma Warnings (Off, "is not referenced");
 function Name_P_Get_Referenced_Decl
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Logic_Val_Result
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Logic_Val_Result;
            procedure Finalizer_Scope_1045 with Inline_Always;
            procedure Finalizer_Scope_2909 with Inline_Always;
      Block_Var : Internal_Logic_Val_Result;
Block_Var_1 : Internal_Entity_Basic_Decl;
Block_Var_2 : Bare_Ada_Node;
Block_Var_3 : Internal_Entity_Basic_Decl;
Var_Expr : Env_Rebindings;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Body_Node;
Var_Expr_3 : Boolean;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_6 : Internal_Entity_Info;
Fld_7 : Env_Rebindings;
Fld_8 : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_9 : Bare_Basic_Decl;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Body_Node;
Fld_10 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Fld_11 : Bare_Basic_Decl;
Scope_Result_2 : Bare_Basic_Decl;
Result_Var_2 : Bare_Basic_Decl;
Cast_Result_4 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Fld_12 : Boolean;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity;
New_Struct : Internal_Logic_Val_Result;
Scope_Result_3 : Internal_Logic_Val_Result;
Result_Var_3 : Internal_Logic_Val_Result;
Let_Result : Internal_Logic_Val_Result;
            procedure Finalizer_Scope_1045 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1045;
            procedure Finalizer_Scope_2909 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2909;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7709 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7688 '<FieldAccess for logic_val at ast.py:14335>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14335
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 7687 '<FieldAccess for ref_var at ast.py:14335>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14335
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7687
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 7688
Block_Var := Fld_1; 
--# bind val_result Block_Var
--# expr-start 7690 '<Cast to BasicDecl.entity at ast.py:14336>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14336
--# expr-start 7689 '<FieldAccess for value at ast.py:14336>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14336
Fld_2 := Block_Var.Value;
--# expr-done 7689
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7690
Block_Var_1 := Cast_Result_1; 
--# bind entity Block_Var_1
--# expr-start 7694 '<env_node at ast.py:14338>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14338
--# expr-start 7692 '<FieldAccess for rebindings at ast.py:14338>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14338
--# expr-start 7691 '<FieldAccess for info at ast.py:14338>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14338
Fld_3 := Ent.Info;
--# expr-done 7691
Fld_4 := Fld_3.Rebindings;
--# expr-done 7692
Var_Expr := Fld_4;
if Var_Expr /= null then
   --# scope-start
--# expr-start 7693 '<FieldAccess for old_env at ast.py:14338>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14338
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Var_Expr.Old_Env;
Inc_Ref (Fld_5);
--# expr-done 7693
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2909;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Env_Node := AST_Envs.Env_Node (Result_Var); 
--# expr-done 7694
Block_Var_2 := Env_Node; 
--# bind generic_context_node Block_Var_2
--# expr-start 7706 '<If at ast.py:14342>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14342
--# expr-start 7696 '<Not at ast.py:14343>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14343
--# expr-start 7695 '<IsNull at ast.py:14343>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14343
Is_Null := Block_Var_2 = null; 
--# expr-done 7695
Not_Val := not (Is_Null); 
--# expr-done 7696
if Not_Val then
--# expr-start 7698 '<Eq at ast.py:14348>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14348
--# expr-start 7697 '<FieldAccess for node at ast.py:14348>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14348
Fld_9 := Block_Var_1.Node;
--# expr-done 7697
      Cast_Result_2 := Fld_9;
Is_Equal := Block_Var_2 = Cast_Result_2; 
--# expr-done 7698
if Is_Equal then
   If_Result := True;
else
   --# expr-start 7702 '<Eq at ast.py:14352>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14352
--# expr-start 7699 '<Cast to Body at ast.py:14353>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14353
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_3 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_3 := No_Entity_Body_Node;
   end if;
--# expr-done 7699
Var_Expr_2 := Cast_Result_3;
if Var_Expr_2 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 7700 '<FieldAccess for decl_part at ast.py:14353>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14353
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_10 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Var_Expr_2.Node, Imprecise_Fallback => False, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7700
Scope_Result_1 := Fld_10;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Var_Expr_1 := Result_Var_1;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7701 '<FieldAccess for node at ast.py:14353>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14353
Fld_11 := Var_Expr_1.Node;
--# expr-done 7701
Scope_Result_2 := Fld_11;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
      Cast_Result_4 := Result_Var_2;
Is_Equal_1 := Block_Var_2 = Cast_Result_4; 
--# expr-done 7702
   If_Result := Is_Equal_1;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 7705 '<FieldAccess for unshed_rebindings_helper at ast.py:14357>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14357
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7704 '<FieldAccess for rebindings at ast.py:14357>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14357
--# expr-start 7703 '<FieldAccess for info at ast.py:14357>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14357
Fld_6 := Ent.Info;
--# expr-done 7703
Fld_7 := Fld_6.Rebindings;
--# expr-done 7704
--# property-call-start BasicDecl.unshed_rebindings_helper
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings_Helper (Node => Block_Var_1.Node, Rebindings => Fld_7, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7705
   If_Result_2 := Fld_8;
else
   If_Result_2 := Block_Var_1;
end if;
--# expr-done 7706
Block_Var_3 := If_Result_2; 
--# bind entity_with_generic_context Block_Var_3
--# expr-start 7707 '<FieldAccess for success at ast.py:14361>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14361
Fld_12 := Block_Var.Success;
--# expr-done 7707
Var_Expr_3 := Fld_12;
if Var_Expr_3 /= False then
   --# scope-start
   --# bind s Var_Expr_3
--# expr-start 7708 '<New LogicValResult at ast.py:14361>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14361
Cast_Expr_2 := Block_Var_3; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
New_Struct := (Success => Var_Expr_3, Value => Cast_Result_5); 
--# expr-done 7708
Scope_Result_3 := New_Struct;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Logic_Val_Result;
end if;
Let_Result := Result_Var_3; 
--# end
--# expr-done 7709
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1045;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1045;
                     Finalizer_Scope_2909;
            raise;
      end;
   return Property_Result;
end Name_P_Get_Referenced_Decl;
--# end
--# property-start Name.referenced_decl_internal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14366
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Decl_Internal
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Decl;
      Var_Expr : Internal_Refd_Decl;
V : Internal_Logic_Val_Result;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Name;
V_1 : Internal_Logic_Val_Result;
Var_Expr_3 : Internal_Refd_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Fld : Internal_Logic_Val_Result;
Try_Result : Internal_Logic_Val_Result;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
New_Struct : Internal_Refd_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
New_Struct_1 : Internal_Refd_Decl;
Scope_Result_1 : Internal_Refd_Decl;
New_Struct_2 : Internal_Refd_Decl;
Result_Var_1 : Internal_Refd_Decl;
Fld_3 : Boolean;
If_Result : Internal_Refd_Decl;
Let_Result : Internal_Refd_Decl;
Fld_4 : Internal_Logic_Val_Result;
Fld_5 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
New_Struct_3 : Internal_Refd_Decl;
New_Struct_4 : Internal_Refd_Decl;
Fld_6 : Boolean;
If_Result_1 : Internal_Refd_Decl;
Let_Result_1 : Internal_Refd_Decl;
If_Result_2 : Internal_Refd_Decl;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var_2 : Internal_Entity_Basic_Decl;
Fld_9 : Ref_Result_Kind;
New_Struct_5 : Internal_Refd_Decl;
Fld_10 : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Fld_11 : Ref_Result_Kind;
Is_Equal : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Internal_Refd_Decl;
Scope_Result_3 : Internal_Refd_Decl;
Result_Var_3 : Internal_Refd_Decl;
Scope_Result_4 : Internal_Refd_Decl;
Result_Var_4 : Internal_Refd_Decl;
Fld_12 : Boolean;
If_Result_5 : Internal_Refd_Decl;
Let_Result_2 : Internal_Refd_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7741 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 7740 '<If at ast.py:14373>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14373
--# expr-start 7710 '<FieldAccess for is_defining at ast.py:14376>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14376
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld_12 := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 7710
if Fld_12 then
   If_Result_5 := No_Refd_Decl;
else
--# expr-start 7730 '<If at ast.py:14379>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14379
if Imprecise_Fallback then
   --# expr-start 7721 '<Let at ast.py:14383>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14383
--# scope-start
--# expr-start 7712 '<Try at ast.py:14383>' Try_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14383
begin
   --# expr-start 7711 '<FieldAccess for get_referenced_decl at ast.py:14384>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14384
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.get_referenced_decl
Fld := Libadalang.Implementation.Name_P_Get_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7711
   Try_Result := Fld;
exception
   when Property_Error =>
      Try_Result := No_Logic_Val_Result;
end;
--# expr-done 7712
V := Try_Result; 
--# bind v V
--# expr-start 7720 '<If at ast.py:14386>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14386
--# expr-start 7713 '<FieldAccess for success at ast.py:14387>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14387
Fld_3 := V.Success;
--# expr-done 7713
if Fld_3 then
   --# expr-start 7716 '<New RefdDecl at ast.py:14390>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14390
--# expr-start 7715 '<Cast to BasicDecl.entity at ast.py:14391>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14391
--# expr-start 7714 '<FieldAccess for value at ast.py:14391>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14391
Fld_1 := V.Value;
--# expr-done 7714
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7715
New_Struct := (Decl => Cast_Result, Kind => Precise); 
--# expr-done 7716
   If_Result := New_Struct;
else
Var_Expr_2 := Ent;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
--# expr-start 7717 '<FieldAccess for first_corresponding_decl at ast.py:14397>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14397
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.first_corresponding_decl'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_First_Corresponding_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7717
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind fcd Var_Expr_1
--# expr-start 7718 '<New RefdDecl at ast.py:14399>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14399
New_Struct_1 := (Decl => Var_Expr_1, Kind => Imprecise); 
--# expr-done 7718
Scope_Result_1 := New_Struct_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 7719 '<New RefdDecl at ast.py:14400>' New_Struct_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14400
New_Struct_2 := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 7719
   Result_Var_1 := New_Struct_2;
end if;
   If_Result := Result_Var_1;
end if;
--# expr-done 7720
Let_Result := If_Result; 
--# end
--# expr-done 7721
   If_Result_2 := Let_Result;
else
   --# expr-start 7729 '<Let at ast.py:14405>' Let_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14405
--# scope-start
--# expr-start 7722 '<FieldAccess for get_referenced_decl at ast.py:14405>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14405
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.get_referenced_decl
Fld_4 := Libadalang.Implementation.Name_P_Get_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7722
V_1 := Fld_4; 
--# bind v V_1
--# expr-start 7728 '<If at ast.py:14405>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14405
--# expr-start 7723 '<FieldAccess for success at ast.py:14406>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14406
Fld_6 := V_1.Success;
--# expr-done 7723
if Fld_6 then
   --# expr-start 7726 '<New RefdDecl at ast.py:14407>' New_Struct_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14407
--# expr-start 7725 '<Cast to BasicDecl.entity at ast.py:14408>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14408
--# expr-start 7724 '<FieldAccess for value at ast.py:14408>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14408
Fld_5 := V_1.Value;
--# expr-done 7724
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7725
New_Struct_3 := (Decl => Cast_Result_1, Kind => Precise); 
--# expr-done 7726
   If_Result_1 := New_Struct_3;
else
   --# expr-start 7727 '<New RefdDecl at ast.py:14411>' New_Struct_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14411
New_Struct_4 := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 7727
   If_Result_1 := New_Struct_4;
end if;
--# expr-done 7728
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 7729
   If_Result_2 := Let_Result_1;
end if;
--# expr-done 7730
Var_Expr := If_Result_2;
if Var_Expr /= No_Refd_Decl then
   --# scope-start
   --# bind res Var_Expr
--# expr-start 7734 '<New RefdDecl at ast.py:14416>' New_Struct_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14416
--# expr-start 7731 '<FieldAccess for decl at ast.py:14417>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14417
Fld_7 := Var_Expr.Decl;
--# expr-done 7731
Var_Expr_4 := Fld_7;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7732 '<FieldAccess for wrap_public_reference at ast.py:14417>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14417
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.wrap_public_reference
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Wrap_Public_Reference (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 7732
Scope_Result_2 := Fld_8;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Basic_Decl;
end if;
--# expr-start 7733 '<FieldAccess for kind at ast.py:14418>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14418
Fld_9 := Var_Expr.Kind;
--# expr-done 7733
New_Struct_5 := (Decl => Result_Var_2, Kind => Fld_9); 
--# expr-done 7734
Var_Expr_3 := New_Struct_5;
if Var_Expr_3 /= No_Refd_Decl then
   --# scope-start
   --# bind res Var_Expr_3
--# expr-start 7739 '<If at ast.py:14419>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14419
--# expr-start 7736 '<IsNull at ast.py:14422>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14422
--# expr-start 7735 '<FieldAccess for decl at ast.py:14422>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14422
Fld_10 := Var_Expr_3.Decl;
--# expr-done 7735
Is_Null := Fld_10.Node = null; 
--# expr-done 7736
if Is_Null then
   --# expr-start 7738 '<Eq at ast.py:14422>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14422
--# expr-start 7737 '<FieldAccess for kind at ast.py:14422>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14422
Fld_11 := Var_Expr_3.Kind;
--# expr-done 7737
Is_Equal := Fld_11 = Precise; 
--# expr-done 7738
   If_Result_3 := Is_Equal;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   If_Result_4 := No_Refd_Decl;
else
   If_Result_4 := Var_Expr_3;
end if;
--# expr-done 7739
Scope_Result_3 := If_Result_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Refd_Decl;
end if;
Scope_Result_4 := Result_Var_3;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Refd_Decl;
end if;
   If_Result_5 := Result_Var_4;
end if;
--# expr-done 7740
Let_Result_2 := If_Result_5; 
--# end
--# expr-done 7741
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Referenced_Decl_Internal;
--# end
--# property-start '[dispatcher]Name.designated_type_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Designated_Type_Impl
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.designated_type_impl
                  Property_Result := Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.designated_type_impl
                  Property_Result := Attribute_Ref_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_type_impl
                  Property_Result := Call_Expr_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_type_impl
                  Property_Result := Dotted_Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_type_impl
                  Property_Result := Base_Id_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Designated_Type_Impl;
--# end
--# property-start Name.name_designated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14439
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Designated_Type
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1048 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1048 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1048;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7746 '<bind at ast.py:14440>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14440
--# expr-start 7742 '<FieldAccess for node_env at ast.py:14440>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14440
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7742
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 7745 '<bind at ast.py:14441>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14441
--# expr-start 7743 '<FieldAccess for origin_node at ast.py:14441>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14441
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7743
Bound_Origin := Fld_1; 
--# expr-start 7744 '<FieldAccess for designated_type_impl at ast.py:14441>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14441
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 7744
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 7745
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 7746
         Property_Result := Dyn_Var_Bind_Result_1;
   --# end
      Finalizer_Scope_1048;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1048;
            raise;
      end;
   return Property_Result;
end Name_P_Name_Designated_Type;
--# end
--# property-start Name.is_static_subtype /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14450
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Static_Subtype
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7749 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7748 '<FieldAccess for is_static_decl at ast.py:14457>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14457
--# expr-start 7747 '<FieldAccess for name_designated_type at ast.py:14457>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14457
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7747
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 7748
Let_Result := Fld_1; 
--# end
--# expr-done 7749
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Static_Subtype;
--# end
--# property-start Name.name_designated_type_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14459
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Designated_Type_Env
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1050 with Inline_Always;
            procedure Finalizer_Scope_2918 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1050 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1050;
            procedure Finalizer_Scope_2918 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2918;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Name_Designated_Type_Env,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 7752 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7750 '<FieldAccess for name_designated_type at ast.py:14461>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14461
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7750
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7751 '<FieldAccess for primitives_env at ast.py:14461>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14461
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7751
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2918;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 7752
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1050;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1050;
                     Finalizer_Scope_2918;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Name_P_Name_Designated_Type_Env;
--# end
--# property-start Name.referenced_unit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14463
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Unit
  (Node : Bare_Name
      ; Kind : Analysis_Unit_Kind
      ; Not_Found_Is_Error : Boolean
         := True
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind kind Kind
   --# bind not_found_is_error Not_Found_Is_Error
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1051 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1051 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1051;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7755 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7754 '<FieldAccess for designated_compilation_unit at ast.py:14470>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14470
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7753 '<FieldAccess for as_symbol_array at ast.py:14471>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14471
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Self);
--# end
--# expr-done 7753
--# property-call-start AdaNode.designated_compilation_unit
Fld_1 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld, Kind => Kind, Load_If_Needed => True, Not_Found_Is_Error => Not_Found_Is_Error);
--# end
--# expr-done 7754
Let_Result := Fld_1; 
--# end
--# expr-done 7755
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1051;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1051;
            raise;
      end;
   return Property_Result;
end Name_P_Referenced_Unit;
--# end
--# property-start Name.matches /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14474
pragma Warnings (Off, "is not referenced");
 function Name_P_Matches
  (Node : Bare_Name
      ; N : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind n N
   Property_Result : Boolean;
      Match_0 : Bare_Synthetic_Identifier;
Match_1 : Bare_Identifier;
Match_2 : Bare_String_Literal;
Match_3 : Bare_Dotted_Name;
Match_4 : Bare_Defining_Name;
Match_5 : Bare_Name;
Var_Expr : Bare_Identifier;
Var_Expr_1 : Bare_Identifier;
Var_Expr_2 : Bare_String_Literal;
Var_Expr_3 : Bare_Dotted_Name;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Identifier;
Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Bare_Name;
Cast_Result_1 : Bare_Identifier;
Fld_2 : Symbol_Type;
Fld_3 : Symbol_Type;
Is_Equal_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Cast_Expr_2 : Bare_Name;
Cast_Result_2 : Bare_String_Literal;
Fld_4 : Symbol_Type;
Fld_5 : Symbol_Type;
Is_Equal_2 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Cast_Expr_3 : Bare_Name;
Cast_Result_3 : Bare_Dotted_Name;
Fld_6 : Bare_Name;
Fld_7 : Bare_Name;
Fld_8 : Boolean;
Fld_9 : Bare_Base_Id;
Fld_10 : Bare_Base_Id;
Cast_Result_4 : Bare_Name;
Fld_11 : Boolean;
If_Result : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_12 : Bare_Name;
Fld_13 : Boolean;
Match_Prefix : Bare_Name;
Cast_Result_5 : Bare_Synthetic_Identifier;
Let_Result : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_6 : Bare_Identifier;
Let_Result_1 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_7 : Bare_String_Literal;
Let_Result_2 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_8 : Bare_Dotted_Name;
Let_Result_3 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_9 : Bare_Defining_Name;
Let_Result_4 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_10 : Bare_Name;
Let_Result_5 : Boolean;
Scope_Result_9 : Boolean;
Match_Result : Boolean;
Let_Result_6 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7778 '<Block at ???>' Let_Result_6 None
--# scope-start
--# expr-start 7777 '<Match at ast.py:14482>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14482
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Synthetic_Identifier =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix;
Match_0 := Cast_Result_5; 
--# bind id Match_0
--# expr-start 7756 '<Cast to Identifier at ast.py:14483>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14483
Cast_Expr := N; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Identifier_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7756
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind other_id Var_Expr
--# expr-start 7759 '<Eq at ast.py:14490>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14490
--# expr-start 7757 '<FieldAccess for sym at ast.py:14490>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14490
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Match_0.Synthetic_Identifier_Sym;
--# expr-done 7757
--# expr-start 7758 '<FieldAccess for sym at ast.py:14490>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14490
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Var_Expr);
--# end
--# expr-done 7758
Is_Equal := Fld = Fld_1; 
--# expr-done 7759
Scope_Result := Is_Equal;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Identifier =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_1 := Cast_Result_6; 
--# bind id Match_1
--# expr-start 7760 '<Cast to Identifier at ast.py:14492>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14492
Cast_Expr_1 := N; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Identifier_Range
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 7760
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind other_id Var_Expr_1
--# expr-start 7763 '<Eq at ast.py:14493>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14493
--# expr-start 7761 '<FieldAccess for sym at ast.py:14493>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14493
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_1);
--# end
--# expr-done 7761
--# expr-start 7762 '<FieldAccess for sym at ast.py:14493>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14493
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_3 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Var_Expr_1);
--# end
--# expr-done 7762
Is_Equal_1 := Fld_2 = Fld_3; 
--# expr-done 7763
Scope_Result_1 := Is_Equal_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Let_Result_1 := Result_Var_1; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Match_Prefix;
Match_2 := Cast_Result_7; 
--# bind sl Match_2
--# expr-start 7764 '<Cast to StringLiteral at ast.py:14495>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14495
Cast_Expr_2 := N; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_String_Literal_Range
   then
      Cast_Result_2 := Cast_Expr_2;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 7764
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind other_sl Var_Expr_2
--# expr-start 7767 '<Eq at ast.py:14496>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14496
--# expr-start 7765 '<FieldAccess for sym at ast.py:14496>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14496
      if Match_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_4 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_2);
--# end
--# expr-done 7765
--# expr-start 7766 '<FieldAccess for sym at ast.py:14496>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14496
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_5 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Var_Expr_2);
--# end
--# expr-done 7766
Is_Equal_2 := Fld_4 = Fld_5; 
--# expr-done 7767
Scope_Result_2 := Is_Equal_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Let_Result_2 := Result_Var_2; 
--# end
Scope_Result_6 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_3 := Cast_Result_8; 
--# bind dn Match_3
--# expr-start 7768 '<Cast to DottedName at ast.py:14498>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14498
Cast_Expr_3 := N; 
   if Cast_Expr_3 = null
      or else Cast_Expr_3.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_3 := Cast_Expr_3;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 7768
Var_Expr_3 := Cast_Result_3;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind sn Var_Expr_3
--# expr-start 7771 '<FieldAccess for matches at ast.py:14500>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14500
--# expr-start 7769 '<FieldAccess for prefix at ast.py:14500>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14500
      if Var_Expr_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Var_Expr_3.Dotted_Name_F_Prefix;
--# expr-done 7769
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7770 '<FieldAccess for prefix at ast.py:14500>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14500
      if Match_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Match_3.Dotted_Name_F_Prefix;
--# expr-done 7770
--# property-call-start Name.matches
Fld_8 := Libadalang.Implementation.Name_P_Matches (Node => Fld_6, N => Fld_7);
--# end
--# expr-done 7771
if Fld_8 then
   --# expr-start 7774 '<FieldAccess for matches at ast.py:14501>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14501
--# expr-start 7772 '<FieldAccess for suffix at ast.py:14501>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14501
      if Var_Expr_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Var_Expr_3.Dotted_Name_F_Suffix;
--# expr-done 7772
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7773 '<FieldAccess for suffix at ast.py:14501>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14501
      if Match_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Match_3.Dotted_Name_F_Suffix;
--# expr-done 7773
      Cast_Result_4 := Fld_10;
--# property-call-start Name.matches
Fld_11 := Libadalang.Implementation.Name_P_Matches (Node => Fld_9, N => Cast_Result_4);
--# end
--# expr-done 7774
   If_Result := Fld_11;
else
   If_Result := False;
end if;
Scope_Result_3 := If_Result;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
Let_Result_3 := Result_Var_3; 
--# end
Scope_Result_7 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix;
Match_4 := Cast_Result_9; 
--# bind di Match_4
--# expr-start 7776 '<FieldAccess for matches at ast.py:14504>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14504
      if N = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7775 '<FieldAccess for name at ast.py:14504>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14504
      if Match_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Match_4.Defining_Name_F_Name;
--# expr-done 7775
--# property-call-start Name.matches
Fld_13 := Libadalang.Implementation.Name_P_Matches (Node => N, N => Fld_12);
--# end
--# expr-done 7776
Let_Result_4 := Fld_13; 
--# end
Scope_Result_8 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Op_Xor | Ada_Null_Literal .. Ada_Real_Literal | Ada_Target_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Match_Prefix;
Match_5 := Cast_Result_10; 
--# bind _ Match_5
Let_Result_5 := False; 
--# end
Scope_Result_9 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_9; 
end case;
--# expr-done 7777
Let_Result_6 := Match_Result; 
--# end
--# expr-done 7778
         Property_Result := Let_Result_6;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Matches;
--# end
--# property-start Name.name_matches /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14508
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Matches
  (Node : Bare_Name
      ; N : Internal_Entity_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind n N
   Property_Result : Boolean;
      Fld : Bare_Name;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7781 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7780 '<FieldAccess for matches at ast.py:14516>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14516
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7779 '<FieldAccess for node at ast.py:14516>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14516
Fld := N.Node;
--# expr-done 7779
--# property-call-start Name.matches
Fld_1 := Libadalang.Implementation.Name_P_Matches (Node => Self, N => Fld);
--# end
--# expr-done 7780
Let_Result := Fld_1; 
--# end
--# expr-done 7781
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Name_Matches;
--# end
--# property-start Name.use_package_name_designated_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14521
pragma Warnings (Off, "is not referenced");
 function Name_P_Use_Package_Name_Designated_Env
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1054 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Use_Package_Clause;
Fld_2 : Integer;
Fld_3 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1054 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1054;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7787 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7786 '<FieldAccess for designated_env at ast.py:14529>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14529
--# expr-start 7784 '<Cast to UsePackageClause at ast.py:14528>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14528
--# expr-start 7783 '<parent at ast.py:14528>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14528
--# expr-start 7782 '<parent at ast.py:14528>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14528
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7782
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 7783
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Use_Package_Clause
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7784
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7785 '<FieldAccess for child_index at ast.py:14529>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14529
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Child_Index (Node => Self);
--# expr-done 7785
--# property-call-start UsePackageClause.designated_env
Fld_3 := Libadalang.Implementation.Use_Package_Clause_P_Designated_Env (Node => Cast_Result.Node, Index => Fld_2, E_Info => Cast_Result.Info);
--# end
--# expr-done 7786
Let_Result := Fld_3; Inc_Ref (Let_Result);
--# end
--# expr-done 7787
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1054;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1054;
            raise;
      end;
   return Property_Result;
end Name_P_Use_Package_Name_Designated_Env;
--# end
--# property-start '[dispatcher]Name.relative_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Relative_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Discrete_Subtype_Name | Ada_Reduce_Attribute_Ref | Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.relative_name
                  Property_Result := Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.relative_name
                  Property_Result := Attribute_Ref_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.relative_name
                  Property_Result := Call_Expr_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.relative_name
                  Property_Result := Defining_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.relative_name
                  Property_Result := Dotted_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.relative_name
                  Property_Result := End_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.relative_name
                  Property_Result := Explicit_Deref_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.relative_name
                  Property_Result := Qual_Expr_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.relative_name
                  Property_Result := Single_Tok_Node_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Identifier =>
                  --# property-call-start SyntheticIdentifier.relative_name
                  Property_Result := Synthetic_Identifier_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.relative_name
                  Property_Result := Target_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Relative_Name;
--# end
--# property-start '[dispatcher]Name.name_symbol' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Name_Symbol
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Defining_Name | Ada_Discrete_Subtype_Name .. Ada_Reduce_Attribute_Ref | Ada_Target_Name .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.name_symbol
                  Property_Result := Name_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Synthetic_Defining_Name =>
                  --# property-call-start SyntheticDefiningName.name_symbol
                  Property_Result := Synthetic_Defining_Name_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Identifier | Ada_String_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.name_symbol
                  Property_Result := Single_Tok_Node_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.name_symbol
                  Property_Result := Op_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Synthetic_Identifier =>
                  --# property-call-start SyntheticIdentifier.name_symbol
                  Property_Result := Synthetic_Identifier_P_Name_Symbol
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Name_Symbol;
--# end
--# property-start Name.xref_no_overloading /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14544
pragma Warnings (Off, "is not referenced");
 function Name_P_Xref_No_Overloading
  (Node : Bare_Name
      ; Sequential : Boolean
         := True
      ; All_Els : Boolean
         := False
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sequential Sequential
   --# bind all_els All_Els
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1057 with Inline_Always;
            procedure Finalizer_Scope_2929 with Inline_Always;
            procedure Finalizer_Scope_2930 with Inline_Always;
            procedure Finalizer_Scope_2933 with Inline_Always;
            procedure Finalizer_Scope_2934 with Inline_Always;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Base_Id;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Symbol_Type;
Fld_7 : Boolean;
If_Result : Lookup_Kind;
Fld_8 : Bare_Name;
If_Result_1 : Bare_Name;
Cast_Result : Bare_Ada_Node;
Fld_9 : Internal_Entity_Array_Access;
Fld_10 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_11 : Logic_Var;
Domain_Equation : Logic_Equation;
Fld_12 : Logic_Var;
Fld_13 : Boolean;
If_Result_2 : Lookup_Kind;
Fld_14 : Bare_Name;
If_Result_3 : Bare_Name;
Cast_Result_1 : Bare_Ada_Node;
Fld_15 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_16 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Logic_Equation;
Fld_17 : Internal_Entity_Name;
Fld_18 : Logic_Equation;
Fld_19 : Internal_Entity_Identifier;
Fld_20 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_21 : Internal_Entity_Name;
Fld_22 : Logic_Var;
Fld_23 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_24 : Internal_Entity_Identifier;
Fld_25 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
Fld_26 : Internal_Entity_Name;
Fld_27 : Logic_Var;
Fld_28 : Logic_Var;
Bind_Result_2 : Logic_Equation;
True_Rel : Logic_Equation;
If_Result_6 : Logic_Equation;
If_Result_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_3 : Internal_Entity_Base_Id;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1057 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1057;
            procedure Finalizer_Scope_2929 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2929;
            procedure Finalizer_Scope_2930 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Domain_Equation);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (If_Result_5);
            end Finalizer_Scope_2930;
            procedure Finalizer_Scope_2933 is
            begin
                     Dec_Ref (Fld_18);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2933;
            procedure Finalizer_Scope_2934 is
            begin
                     Dec_Ref (False_Rel);
            end Finalizer_Scope_2934;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7836 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 7835 '<Match at ast.py:14554>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14554
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind dn Match_0
--# expr-start 7795 '<And at ast.py:14557>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14557
--# expr-start 7789 '<FieldAccess for xref_no_overloading at ast.py:14557>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14557
--# expr-start 7788 '<FieldAccess for prefix at ast.py:14557>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14557
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 7788
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => Sequential, All_Els => All_Els, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7789
--# expr-start 7794 '<bind at ast.py:14558>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14558
--# expr-start 7791 '<FieldAccess for designated_env at ast.py:14559>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14559
--# expr-start 7790 '<FieldAccess for prefix at ast.py:14559>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14559
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 7790
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 7791
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 7793 '<FieldAccess for xref_no_overloading at ast.py:14560>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14560
--# expr-start 7792 '<FieldAccess for suffix at ast.py:14560>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14560
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 7792
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Sequential => Sequential, All_Els => All_Els, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 7793
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7794
And_Pred := Create_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14557") else null)); 
--# expr-done 7795
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2929;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind i Match_1
--# expr-start 7814 '<If at ast.py:14562>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14562
if All_Els then
   --# expr-start 7796 '<FieldAccess for is_suffix at ast.py:14563>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14563
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_16 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 7796
   If_Result_4 := Fld_16;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 7806 '<domain at ast.py:14564>' Domain_Equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14564
--# expr-start 7804 '<Filter at ast.py:14570>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14570
   --# expr-start 7802 '<FieldAccess for env_get at ast.py:14565>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14565
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7797 '<FieldAccess for name_symbol at ast.py:14567>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14567
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Match_1.Node);
--# end
--# expr-done 7797
--# expr-start 7799 '<If at ast.py:14569>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14569
--# expr-start 7798 '<FieldAccess for is_prefix at ast.py:14569>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14569
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_7 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7798
if Fld_7 then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7799
--# expr-start 7801 '<If at ast.py:14568>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14568
if Sequential then
   --# expr-start 7800 '<FieldAccess for node at ast.py:14568>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14568
Fld_8 := Ent.Node;
--# expr-done 7800
   If_Result_1 := Fld_8;
else
   If_Result_1 := No_Bare_Ada_Node;
end if;
--# expr-done 7801
      Cast_Result := If_Result_1;
--# property-call-start AdaNode.env_get
Fld_9 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld_6, Lookup => If_Result, From_Node => Cast_Result);
--# end
--# expr-done 7802
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_9;
      begin
         for Item_257 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_257
         --# expr-start 7803 '<FieldAccess for has_visibility at ast.py:14570>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14570
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_10 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_257);
--# end
--# expr-done 7803
         if Fld_10 then
         declare
            Item_To_Append : constant Internal_Entity := Item_257;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7804
--# expr-start 7805 '<FieldAccess for ref_var at ast.py:14564>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14564
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 7805
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
declare
   Dom : Internal_Entity_Array_Access := Map_Result;
   A   : Entity_Vars.Value_Array (1 .. Length (Dom));
begin
   for J in 0 .. Length (Dom) - 1 loop
      declare
         Item : constant Internal_Entity := Get (Self, Dom, J);
      begin
         A (J + 1) := (Node => Item.Node, Info => Item.Info);
      end;
   end loop;
   Domain_Equation := Solver.Create_Domain
     (Fld_11, A, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14564") else null));
end;
--# expr-done 7806
   If_Result_5 := Domain_Equation;
else
   --# expr-start 7813 '<Bind at ast.py:14572>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14572
--# expr-start 7807 '<FieldAccess for ref_var at ast.py:14573>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14573
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 7807
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
--# expr-start 7812 '<FieldAccess for env_get_first_visible at ast.py:14574>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14574
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7809 '<If at ast.py:14577>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14577
--# expr-start 7808 '<FieldAccess for is_prefix at ast.py:14577>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14577
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_13 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7808
if Fld_13 then
   If_Result_2 := Recursive;
else
   If_Result_2 := Flat;
end if;
--# expr-done 7809
--# expr-start 7811 '<If at ast.py:14576>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14576
if Sequential then
   --# expr-start 7810 '<FieldAccess for node at ast.py:14576>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14576
Fld_14 := Ent.Node;
--# expr-done 7810
   If_Result_3 := Fld_14;
else
   If_Result_3 := No_Bare_Ada_Node;
end if;
--# expr-done 7811
      Cast_Result_1 := If_Result_3;
--# property-call-start SingleTokNode.env_get_first_visible
Fld_15 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Match_1.Node, Lex_Env => Env, Lookup_Type => If_Result_2, From_Node => Cast_Result_1, No_Visibility => False);
--# end
--# expr-done 7812
Bind_Result := Solver.Create_Assign (Fld_12, Fld_15, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14572") else null)); 
--# expr-done 7813
   If_Result_5 := Bind_Result;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 7814
Let_Result_1 := If_Result_5; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2930;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind ar Match_2
--# expr-start 7834 '<And at ast.py:14586>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14586
--# expr-start 7816 '<FieldAccess for xref_no_overloading at ast.py:14586>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14586
--# expr-start 7815 '<FieldAccess for prefix at ast.py:14586>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14586
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 7815
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_18 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_17.Node, Sequential => Sequential, All_Els => All_Els, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 7816
--# expr-start 7833 '<Cond at ast.py:14587>' If_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14587
--# expr-start 7820 '<Eq at ast.py:14588>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14588
--# expr-start 7818 '<FieldAccess for sym at ast.py:14588>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14588
--# expr-start 7817 '<FieldAccess for attribute at ast.py:14588>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14588
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 7817
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_20 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_19.Node);
--# end
--# expr-done 7818
--# expr-start 7819 '<Symbol Class at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Class); 
--# expr-done 7819
Is_Equal := Fld_20 = Sym; 
--# expr-done 7820
if Is_Equal then
   --# expr-start 7824 '<Bind at ast.py:14589>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14589
--# expr-start 7822 '<FieldAccess for ref_var at ast.py:14589>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14589
--# expr-start 7821 '<FieldAccess for prefix at ast.py:14589>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14589
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 7821
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_22 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_21.Node);
--# end
--# expr-done 7822
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
--# expr-start 7823 '<FieldAccess for ref_var at ast.py:14589>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14589
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 7823
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
Bind_Result_1 := Solver.Create_Propagate (Fld_22, Fld_23, Logic_Functor_651'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14589") else null)); 
--# expr-done 7824
   If_Result_7 := Bind_Result_1;
else
--# expr-start 7828 '<Eq at ast.py:14592>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14592
--# expr-start 7826 '<FieldAccess for sym at ast.py:14592>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14592
--# expr-start 7825 '<FieldAccess for attribute at ast.py:14592>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14592
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 7825
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_25 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_24.Node);
--# end
--# expr-done 7826
--# expr-start 7827 '<Symbol Base at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Base); 
--# expr-done 7827
Is_Equal_1 := Fld_25 = Sym_1; 
--# expr-done 7828
if Is_Equal_1 then
   --# expr-start 7832 '<Bind at ast.py:14593>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14593
--# expr-start 7830 '<FieldAccess for ref_var at ast.py:14593>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14593
--# expr-start 7829 '<FieldAccess for prefix at ast.py:14593>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14593
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 7829
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_26.Node);
--# end
--# expr-done 7830
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 7831 '<FieldAccess for ref_var at ast.py:14593>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14593
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_28 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 7831
Fld_28.Value := No_Entity;
Entity_Vars.Reset (Fld_28);
Bind_Result_2 := Solver.Create_Propagate (Fld_27, Fld_28, Logic_Functor_652'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14593") else null)); 
--# expr-done 7832
   If_Result_6 := Bind_Result_2;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14596") else null)); 
   If_Result_6 := True_Rel;
end if;
      Inc_Ref (If_Result_6);
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 7833
And_Pred_1 := Create_And (Fld_18, If_Result_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14586") else null)); 
--# expr-done 7834
Let_Result_2 := And_Pred_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2933;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14599") else null)); 
Let_Result_3 := False_Rel; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2934;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 7835
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 7836
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1057;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1057;
                     Finalizer_Scope_2929;
                     Finalizer_Scope_2930;
                     Finalizer_Scope_2933;
                     Finalizer_Scope_2934;
            raise;
      end;
   return Property_Result;
end Name_P_Xref_No_Overloading;
--# end
--# property-start Name.is_prefix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14602
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Prefix
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_2 : Bare_Name;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Is_Prefix,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 7849 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7838 '<IsA DottedName at ast.py:14610>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14610
--# expr-start 7837 '<parent at ast.py:14610>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14610
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 7837
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 7838
if Is_A then
   --# expr-start 7842 '<Eq at ast.py:14611>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14611
--# expr-start 7841 '<FieldAccess for prefix at ast.py:14611>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14611
--# expr-start 7840 '<Cast to DottedName at ast.py:14611>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14611
--# expr-start 7839 '<parent at ast.py:14611>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14611
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 7839
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7840
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Dotted_Name_F_Prefix;
--# expr-done 7841
      Cast_Result_1 := Fld_2;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 7842
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 7845 '<FieldAccess for is_prefix at ast.py:14612>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14612
--# expr-start 7844 '<Cast to Name at ast.py:14612>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14612
--# expr-start 7843 '<parent at ast.py:14612>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14612
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 7843
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 7844
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_4 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Cast_Result_2);
--# end
--# expr-done 7845
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 7848 '<Not at ast.py:14613>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14613
--# expr-start 7847 '<IsA DottedName at ast.py:14613>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14613
--# expr-start 7846 '<parent at ast.py:14613>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14613
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Parent; 
--# expr-done 7846
Is_A_1 := Fld_5 /= null 
and then Fld_5.Kind in Ada_Dotted_Name_Range; 
--# expr-done 7847
Not_Val := not (Is_A_1); 
--# expr-done 7848
   If_Result_2 := Not_Val;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 7849
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Name_P_Is_Prefix;
--# end
--# property-start Name.is_suffix /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14616
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Suffix
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_2 : Bare_Base_Id;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Is_Suffix,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 7862 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7851 '<IsA DottedName at ast.py:14622>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14622
--# expr-start 7850 '<parent at ast.py:14622>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14622
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 7850
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 7851
if Is_A then
   --# expr-start 7855 '<Eq at ast.py:14623>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14623
--# expr-start 7854 '<FieldAccess for suffix at ast.py:14623>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14623
--# expr-start 7853 '<Cast to DottedName at ast.py:14623>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14623
--# expr-start 7852 '<parent at ast.py:14623>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14623
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 7852
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7853
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Dotted_Name_F_Suffix;
--# expr-done 7854
      Cast_Result_1 := Fld_2;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 7855
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 7858 '<FieldAccess for is_suffix at ast.py:14624>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14624
--# expr-start 7857 '<Cast to Name at ast.py:14624>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14624
--# expr-start 7856 '<parent at ast.py:14624>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14624
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 7856
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 7857
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_4 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Cast_Result_2);
--# end
--# expr-done 7858
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 7861 '<Not at ast.py:14626>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14626
--# expr-start 7860 '<IsA DottedName at ast.py:14626>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14626
--# expr-start 7859 '<parent at ast.py:14626>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14626
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Parent; 
--# expr-done 7859
Is_A_1 := Fld_5 /= null 
and then Fld_5.Kind in Ada_Dotted_Name_Range; 
--# expr-done 7860
Not_Val := not (Is_A_1); 
--# expr-done 7861
   If_Result_2 := Not_Val;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 7862
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Name_P_Is_Suffix;
--# end
--# property-start Name.is_operator_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14629
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Operator_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_1 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_2 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_4 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Fld_5 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Fld_6 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Fld_7 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Fld_8 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Fld_9 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Fld_10 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Fld_11 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Fld_12 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Fld_13 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Fld_14 : Symbol_Type;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Fld_15 : Symbol_Type;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Fld_16 : Symbol_Type;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Fld_17 : Symbol_Type;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Fld_18 : Symbol_Type;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Fld_19 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Fld_20 : Symbol_Type;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7926 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7865 '<Eq at ast.py:14634>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7863 '<FieldAccess for name_symbol at ast.py:14634>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7863
--# expr-start 7864 '<Symbol "=" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 7864
Is_Equal := Fld = Sym; 
--# expr-done 7865
if Is_Equal then
   If_Result := True;
else
   --# expr-start 7868 '<Eq at ast.py:14634>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7866 '<FieldAccess for name_symbol at ast.py:14634>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7866
--# expr-start 7867 '<Symbol "=" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 7867
Is_Equal_1 := Fld_1 = Sym_1; 
--# expr-done 7868
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 7871 '<Eq at ast.py:14634>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7869 '<FieldAccess for name_symbol at ast.py:14634>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7869
--# expr-start 7870 '<Symbol "/=" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 7870
Is_Equal_2 := Fld_2 = Sym_2; 
--# expr-done 7871
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 7874 '<Eq at ast.py:14634>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7872 '<FieldAccess for name_symbol at ast.py:14634>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7872
--# expr-start 7873 '<Symbol "<" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 7873
Is_Equal_3 := Fld_3 = Sym_3; 
--# expr-done 7874
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 7877 '<Eq at ast.py:14634>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7875 '<FieldAccess for name_symbol at ast.py:14634>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7875
--# expr-start 7876 '<Symbol "<=" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 7876
Is_Equal_4 := Fld_4 = Sym_4; 
--# expr-done 7877
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 7880 '<Eq at ast.py:14634>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7878 '<FieldAccess for name_symbol at ast.py:14634>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7878
--# expr-start 7879 '<Symbol ">" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 7879
Is_Equal_5 := Fld_5 = Sym_5; 
--# expr-done 7880
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 7883 '<Eq at ast.py:14634>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7881 '<FieldAccess for name_symbol at ast.py:14634>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7881
--# expr-start 7882 '<Symbol ">=" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 7882
Is_Equal_6 := Fld_6 = Sym_6; 
--# expr-done 7883
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 7886 '<Eq at ast.py:14634>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7884 '<FieldAccess for name_symbol at ast.py:14634>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7884
--# expr-start 7885 '<Symbol "and" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 7885
Is_Equal_7 := Fld_7 = Sym_7; 
--# expr-done 7886
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 7889 '<Eq at ast.py:14634>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7887 '<FieldAccess for name_symbol at ast.py:14634>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7887
--# expr-start 7888 '<Symbol "or" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 7888
Is_Equal_8 := Fld_8 = Sym_8; 
--# expr-done 7889
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 7892 '<Eq at ast.py:14634>' Is_Equal_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7890 '<FieldAccess for name_symbol at ast.py:14634>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7890
--# expr-start 7891 '<Symbol "xor" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor); 
--# expr-done 7891
Is_Equal_9 := Fld_9 = Sym_9; 
--# expr-done 7892
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 7895 '<Eq at ast.py:14634>' Is_Equal_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7893 '<FieldAccess for name_symbol at ast.py:14634>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7893
--# expr-start 7894 '<Symbol "abs" at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 7894
Is_Equal_10 := Fld_10 = Sym_10; 
--# expr-done 7895
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 7898 '<Eq at ast.py:14634>' Is_Equal_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7896 '<FieldAccess for name_symbol at ast.py:14634>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7896
--# expr-start 7897 '<Symbol "*" at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 7897
Is_Equal_11 := Fld_11 = Sym_11; 
--# expr-done 7898
   If_Result_10 := Is_Equal_11;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 7901 '<Eq at ast.py:14634>' Is_Equal_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7899 '<FieldAccess for name_symbol at ast.py:14634>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7899
--# expr-start 7900 '<Symbol "/" at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 7900
Is_Equal_12 := Fld_12 = Sym_12; 
--# expr-done 7901
   If_Result_11 := Is_Equal_12;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 7904 '<Eq at ast.py:14634>' Is_Equal_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7902 '<FieldAccess for name_symbol at ast.py:14634>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7902
--# expr-start 7903 '<Symbol "mod" at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod); 
--# expr-done 7903
Is_Equal_13 := Fld_13 = Sym_13; 
--# expr-done 7904
   If_Result_12 := Is_Equal_13;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 7907 '<Eq at ast.py:14634>' Is_Equal_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7905 '<FieldAccess for name_symbol at ast.py:14634>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7905
--# expr-start 7906 '<Symbol "rem" at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem); 
--# expr-done 7906
Is_Equal_14 := Fld_14 = Sym_14; 
--# expr-done 7907
   If_Result_13 := Is_Equal_14;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 7910 '<Eq at ast.py:14634>' Is_Equal_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7908 '<FieldAccess for name_symbol at ast.py:14634>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7908
--# expr-start 7909 '<Symbol "+" at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 7909
Is_Equal_15 := Fld_15 = Sym_15; 
--# expr-done 7910
   If_Result_14 := Is_Equal_15;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 7913 '<Eq at ast.py:14634>' Is_Equal_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7911 '<FieldAccess for name_symbol at ast.py:14634>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7911
--# expr-start 7912 '<Symbol "-" at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 7912
Is_Equal_16 := Fld_16 = Sym_16; 
--# expr-done 7913
   If_Result_15 := Is_Equal_16;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 7916 '<Eq at ast.py:14634>' Is_Equal_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7914 '<FieldAccess for name_symbol at ast.py:14634>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_17 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7914
--# expr-start 7915 '<Symbol "&""+" at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_1); 
--# expr-done 7915
Is_Equal_17 := Fld_17 = Sym_17; 
--# expr-done 7916
   If_Result_16 := Is_Equal_17;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 7919 '<Eq at ast.py:14634>' Is_Equal_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7917 '<FieldAccess for name_symbol at ast.py:14634>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7917
--# expr-start 7918 '<Symbol "-" at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 7918
Is_Equal_18 := Fld_18 = Sym_18; 
--# expr-done 7919
   If_Result_17 := Is_Equal_18;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 7922 '<Eq at ast.py:14634>' Is_Equal_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7920 '<FieldAccess for name_symbol at ast.py:14634>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7920
--# expr-start 7921 '<Symbol "not" at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 7921
Is_Equal_19 := Fld_19 = Sym_19; 
--# expr-done 7922
   If_Result_18 := Is_Equal_19;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 7925 '<Eq at ast.py:14634>' Is_Equal_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
--# expr-start 7923 '<FieldAccess for name_symbol at ast.py:14634>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14634
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 7923
--# expr-start 7924 '<Symbol "abs" at ???>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 7924
Is_Equal_20 := Fld_20 = Sym_20; 
--# expr-done 7925
   If_Result_19 := Is_Equal_20;
end if;
Let_Result := If_Result_19; 
--# end
--# expr-done 7926
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Operator_Name;
--# end
--# property-start Name.is_write_reference /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14640
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Write_Reference
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_2938 with Inline_Always;
            procedure Finalizer_Scope_2941 with Inline_Always;
      Match_0 : Internal_Entity_Assign_Stmt;
Match_1 : Internal_Entity_Call_Expr;
Match_2 : Internal_Entity_Dotted_Name;
Match_3 : Internal_Entity_Param_Assoc;
Match_4 : Internal_Entity_Attribute_Ref;
Match_5 : Internal_Entity_Alternatives_List;
Match_6 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_1 : Internal_Entity_Mode;
Var_Expr_2 : Internal_Entity_Param_Spec;
Var_Expr_3 : Internal_Entity_Call_Expr;
Var_Expr_4 : Internal_Entity_Mode;
Var_Expr_5 : Internal_Entity_Param_Spec;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Name;
Cast_Result_4 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity;
Is_Equiv_2 : Boolean;
Fld_6 : Internal_Entity_Base_Id;
Fld_7 : Internal_Entity_Base_Formal_Param_Holder;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Boolean;
Fld_11 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_6 : Internal_Entity_Param_Spec;
Fld_12 : Internal_Entity_Mode;
Scope_Result : Internal_Entity_Mode;
Result_Var : Internal_Entity_Mode;
Fld_13 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_14 : Internal_Entity;
Fld_15 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Defining_Name_Array_Access;
Fld_18 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Param_Spec;
Fld_19 : Internal_Entity_Mode;
Scope_Result_3 : Internal_Entity_Mode;
Result_Var_3 : Internal_Entity_Mode;
Fld_20 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Quantifier_Result : Boolean;
Fld_21 : Internal_Entity;
Fld_22 : Internal_Entity;
Cast_Result_9 : Internal_Entity_Call_Expr;
Fld_23 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
If_Result_4 : Boolean;
Fld_24 : Internal_Entity_Name;
Cast_Result_10 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Name;
Cast_Result_11 : Internal_Entity;
Is_Equiv_3 : Boolean;
Fld_25 : Boolean;
If_Result_5 : Boolean;
Fld_26 : Internal_Entity;
Is_A : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_12 : Internal_Entity_Assign_Stmt;
Let_Result : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_13 : Internal_Entity_Call_Expr;
Let_Result_1 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_14 : Internal_Entity_Dotted_Name;
Let_Result_2 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_15 : Internal_Entity_Param_Assoc;
Let_Result_3 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_16 : Internal_Entity_Attribute_Ref;
Let_Result_4 : Boolean;
Scope_Result_10 : Boolean;
Cast_Result_17 : Internal_Entity_Alternatives_List;
Let_Result_5 : Boolean;
Scope_Result_11 : Boolean;
Cast_Result_18 : Internal_Entity;
Let_Result_6 : Boolean;
Scope_Result_12 : Boolean;
Match_Result : Boolean;
Let_Result_7 : Boolean;
            procedure Finalizer_Scope_2938 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2938;
            procedure Finalizer_Scope_2941 is
            begin
                     Dec_Ref (Fld_17);
            end Finalizer_Scope_2941;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7967 '<Block at ???>' Let_Result_7 None
--# scope-start
--# expr-start 7966 '<Match at ast.py:14659>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14659
--# expr-start 7927 '<parent at ast.py:14659>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14659
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7927
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Assign_Stmt =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Assign_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_12; 
--# bind a Match_0
--# expr-start 7929 '<Eq at ast.py:14662>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14662
--# expr-start 7928 '<FieldAccess for dest at ast.py:14662>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14662
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Assign_Stmt_F_Dest, Info => Match_0.Info);
--# expr-done 7928
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 7929
Let_Result := Is_Equiv; 
--# end
Scope_Result_6 := Let_Result;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_13; 
--# bind c Match_1
--# expr-start 7931 '<Eq at ast.py:14667>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14667
--# expr-start 7930 '<FieldAccess for name at ast.py:14667>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14667
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Expr_F_Name, Info => Match_1.Info);
--# expr-done 7930
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 7931
if Is_Equiv_1 then
   --# expr-start 7932 '<FieldAccess for is_write_reference at ast.py:14668>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14668
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_write_reference
Fld_3 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 7932
   If_Result := Fld_3;
else
   If_Result := False;
end if;
Let_Result_1 := If_Result; 
--# end
Scope_Result_7 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_14; 
--# bind d Match_2
--# expr-start 7933 '<FieldAccess for is_write_reference at ast.py:14678>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14678
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_write_reference
Fld_4 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Match_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_2.Info);
--# end
--# expr-done 7933
if Fld_4 then
   If_Result_3 := True;
else
--# expr-start 7935 '<Eq at ast.py:14681>' Is_Equiv_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14681
--# expr-start 7934 '<FieldAccess for prefix at ast.py:14681>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14681
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Match_2.Node.Dotted_Name_F_Prefix, Info => Match_2.Info);
--# expr-done 7934
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_2 := Ent; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv_2 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 7935
if Is_Equiv_2 then
--# expr-start 7937 '<FieldAccess for called_subp_spec at ast.py:14681>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14681
--# expr-start 7936 '<FieldAccess for suffix at ast.py:14681>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14681
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Base_Id (Node => Match_2.Node.Dotted_Name_F_Suffix, Info => Match_2.Info);
--# expr-done 7936
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_subp_spec'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Called_Subp_Spec (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 7937
Var_Expr := Fld_7;
if Var_Expr /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 7940 '<FieldAccess for dottable_subp at ast.py:14683>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14683
--# expr-start 7939 '<FieldAccess for md at ast.py:14683>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14683
--# expr-start 7938 '<FieldAccess for info at ast.py:14683>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14683
Fld_8 := Var_Expr.Info;
--# expr-done 7938
Fld_9 := Fld_8.Md;
--# expr-done 7939
Fld_10 := Fld_9.Dottable_Subp;
--# expr-done 7940
if Fld_10 then
--# expr-start 7943 '<Cast to ParamSpec at ast.py:14685>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14685
--# expr-start 7942 '<at at ast.py:14684>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14684
--# expr-start 7941 '<FieldAccess for abstract_formal_params at ast.py:14684>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14684
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7941
Get_Result := Get (Self, Fld_11, 0, True); 
--# expr-done 7942
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Param_Spec
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_6 := No_Entity_Param_Spec;
   end if;
--# expr-done 7943
Var_Expr_2 := Cast_Result_6;
if Var_Expr_2 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 7944 '<FieldAccess for mode at ast.py:14685>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14685
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Mode (Node => Var_Expr_2.Node.Param_Spec_F_Mode, Info => Var_Expr_2.Info);
--# expr-done 7944
Scope_Result := Fld_12;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Mode;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Mode then
   --# scope-start
--# expr-start 7945 '<FieldAccess for is_writable at ast.py:14685>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14685
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Mode.is_writable
Fld_13 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Var_Expr_1.Node);
--# end
--# expr-done 7945
Scope_Result_1 := Fld_13;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
      Finalizer_Scope_2938;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result_2 := If_Result_3; 
--# end
Scope_Result_8 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Param_Assoc =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Create_Internal_Entity_Param_Assoc
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_15; 
--# bind p Match_3
--# expr-start 7960 '<If at ast.py:14690>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14690
--# expr-start 7948 '<Cast to CallExpr at ast.py:14691>' Cast_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14691
--# expr-start 7947 '<parent at ast.py:14691>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14691
--# expr-start 7946 '<parent at ast.py:14691>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14691
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Parent (Node => Match_3.Node, E_Info => Match_3.Info);
--# expr-done 7946
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Parent (Node => Fld_21.Node, E_Info => Fld_21.Info);
--# expr-done 7947
   if Fld_22.Node = null
      or else Fld_22.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Call_Expr
        (Node => Fld_22.Node,
         Info => Fld_22.Info);
   else
         Cast_Result_9 := No_Entity_Call_Expr;
   end if;
--# expr-done 7948
Var_Expr_3 := Cast_Result_9;
if Var_Expr_3 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 7949 '<FieldAccess for is_type_conversion at ast.py:14691>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14691
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_type_conversion
Fld_23 := Libadalang.Implementation.Call_Expr_P_Is_Type_Conversion (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 7949
Scope_Result_5 := Fld_23;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
   --# expr-start 7953 '<FieldAccess for is_write_reference at ast.py:14692>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14692
--# expr-start 7952 '<Cast to CallExpr at ast.py:14692>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14692
--# expr-start 7951 '<parent at ast.py:14692>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14692
--# expr-start 7950 '<parent at ast.py:14692>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14692
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Parent (Node => Match_3.Node, E_Info => Match_3.Info);
--# expr-done 7950
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Parent (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# expr-done 7951
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 7952
      if Cast_Result_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_write_reference
Fld_16 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Cast_Result_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_7.Info);
--# end
--# expr-done 7953
   If_Result_4 := Fld_16;
else
   --# expr-start 7959 '<AnyQuantifier at ast.py:14693>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14693
   --# expr-start 7954 '<FieldAccess for get_params at ast.py:14693>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14693
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicAssoc.get_params
Fld_17 := Libadalang.Implementation.Basic_Assoc_P_Get_Params (Node => Match_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_3.Info);
--# end
--# expr-done 7954
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_17;
      begin
         for Item_258 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_258
--# expr-start 7956 '<Cast to ParamSpec at ast.py:14695>' Cast_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14695
--# expr-start 7955 '<FieldAccess for basic_decl at ast.py:14695>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14695
      if Item_258.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_18 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Item_258.Node, E_Info => Item_258.Info);
--# end
--# expr-done 7955
   if Fld_18.Node = null
      or else Fld_18.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_8 := Create_Internal_Entity_Param_Spec
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
   else
         Cast_Result_8 := No_Entity_Param_Spec;
   end if;
--# expr-done 7956
Var_Expr_5 := Cast_Result_8;
if Var_Expr_5 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 7957 '<FieldAccess for mode at ast.py:14695>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14695
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Mode (Node => Var_Expr_5.Node.Param_Spec_F_Mode, Info => Var_Expr_5.Info);
--# expr-done 7957
Scope_Result_3 := Fld_19;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Mode;
end if;
Var_Expr_4 := Result_Var_3;
if Var_Expr_4 /= No_Entity_Mode then
   --# scope-start
--# expr-start 7958 '<FieldAccess for is_writable at ast.py:14695>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14695
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Mode.is_writable
Fld_20 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Var_Expr_4.Node);
--# end
--# expr-done 7958
Scope_Result_4 := Fld_20;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
      Quantifier_Result := Result_Var_4;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 7959
   If_Result_4 := Quantifier_Result;
end if;
--# expr-done 7960
Let_Result_3 := If_Result_4; 
--# end
Scope_Result_9 := Let_Result_3;
   --# end
      Finalizer_Scope_2941;
            Match_Result := Scope_Result_9; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_16 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_16; 
--# bind a Match_4
--# expr-start 7962 '<Eq at ast.py:14700>' Is_Equiv_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14700
--# expr-start 7961 '<FieldAccess for prefix at ast.py:14700>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14700
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Name (Node => Match_4.Node.Attribute_Ref_F_Prefix, Info => Match_4.Info);
--# expr-done 7961
      Cast_Result_10 := Create_Internal_Entity
        (Node => Fld_24.Node,
         Info => Fld_24.Info);
Cast_Expr_3 := Ent; 
      Cast_Result_11 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv_3 := Equivalent (Cast_Result_10, Cast_Result_11); 
--# expr-done 7962
if Is_Equiv_3 then
   --# expr-start 7963 '<FieldAccess for is_access_attr at ast.py:14700>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14700
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_25 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Match_4.Node);
--# end
--# expr-done 7963
   If_Result_5 := Fld_25;
else
   If_Result_5 := False;
end if;
Let_Result_4 := If_Result_5; 
--# end
Scope_Result_10 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Alternatives_List =>
   --# scope-start
--# scope-start
      Cast_Result_17 := Create_Internal_Entity_Alternatives_List
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_17; 
--# bind l Match_5
--# expr-start 7965 '<IsA AggregateAssoc at ast.py:14702>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14702
--# expr-start 7964 '<parent at ast.py:14702>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14702
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Libadalang.Implementation.Parent (Node => Match_5.Node, E_Info => Match_5.Info);
--# expr-done 7964
Is_A := Fld_26.Node /= null 
and then Fld_26.Node.Kind in Ada_Aggregate_Assoc_Range; 
--# expr-done 7965
Let_Result_5 := Is_A; 
--# end
Scope_Result_11 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Abort_Absent .. Ada_Abstract_State_Decl_List | Ada_Constraint_List .. Ada_Iterated_Assoc | Ada_Abstract_State_Decl .. Ada_Membership_Expr | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Abort_Stmt | Ada_Call_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_18 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_18; 
--# bind _ Match_6
Let_Result_6 := False; 
--# end
Scope_Result_12 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_12; 
end case;
--# expr-done 7966
Let_Result_7 := Match_Result; 
--# end
--# expr-done 7967
         Property_Result := Let_Result_7;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2938;
                     Finalizer_Scope_2941;
            raise;
      end;
   return Property_Result;
end Name_P_Is_Write_Reference;
--# end
--# property-start Name.potential_actuals_for_dispatch /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14707
pragma Warnings (Off, "is not referenced");
 function Name_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Name
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_1062 with Inline_Always;
            procedure Finalizer_Scope_2949 with Inline_Always;
            procedure Finalizer_Scope_2954 with Inline_Always;
            procedure Finalizer_Scope_2956 with Inline_Always;
      Block_Var : Internal_Expected_Type_For_Expr_Array_Access;
Block_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Dotted_Name;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Dotted_Name;
Match_2 : Internal_Entity_Identifier;
Match_3 : Internal_Entity_Name;
Var_Expr_2 : Internal_Entity_Assoc_List;
Var_Expr_3 : Internal_Entity_Call_Expr;
Fld : Boolean;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Match_Prefix : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Dotted_Name;
Scope_Result : Internal_Entity_Dotted_Name;
Cast_Result_3 : Internal_Entity_Dotted_Name;
Let_Result_1 : Internal_Entity_Dotted_Name;
Scope_Result_1 : Internal_Entity_Dotted_Name;
Cast_Result_4 : Internal_Entity_Identifier;
Let_Result_2 : Internal_Entity_Dotted_Name;
Scope_Result_2 : Internal_Entity_Dotted_Name;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Internal_Entity_Dotted_Name;
Scope_Result_3 : Internal_Entity_Dotted_Name;
Match_Result : Internal_Entity_Dotted_Name;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Name;
Cast_Result_6 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Singleton : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_4 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_5 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Cast_Expr : Internal_Entity_Name;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_7 : Internal_Entity_Assoc_List;
Scope_Result_6 : Internal_Entity_Assoc_List;
Result_Var_2 : Internal_Entity_Assoc_List;
Fld_8 : Internal_Param_Actual_Array_Access;
Scope_Result_7 : Internal_Param_Actual_Array_Access;
Result_Var_3 : Internal_Param_Actual_Array_Access;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity_Type_Expr;
Fld_12 : Internal_Entity_Expr;
New_Struct_1 : Internal_Expected_Type_For_Expr;
Map_Result : Internal_Expected_Type_For_Expr_Array_Access;
Concat_Result : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result_4 : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_1062 is
            begin
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1062;
            procedure Finalizer_Scope_2949 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2949;
            procedure Finalizer_Scope_2954 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2954;
            procedure Finalizer_Scope_2956 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2956;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7990 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 7968 '<FieldAccess for is_dot_call at ast.py:14718>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14718
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_dot_call
Fld := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7968
Var_Expr := Fld;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 7973 '<Match at ast.py:14718>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14718
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind c Match_0
--# expr-start 7970 '<Cast to DottedName at ast.py:14719>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14719
--# expr-start 7969 '<FieldAccess for name at ast.py:14719>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14719
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Call_Expr_F_Name, Info => Match_0.Info);
--# expr-done 7969
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7970
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind d Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Identifier =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Identifier
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind i Match_2
--# expr-start 7972 '<Cast to DottedName at ast.py:14721>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14721
--# expr-start 7971 '<parent at ast.py:14721>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14721
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Match_2.Node, E_Info => Match_2.Info);
--# expr-done 7971
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7972
Let_Result_2 := Cast_Result_1; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Dotted_Name; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 7973
Var_Expr_1 := Match_Result;
if Var_Expr_1 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind d Var_Expr_1
--# expr-start 7979 '<New ExpectedTypeForExpr at ast.py:14723>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14723
--# expr-start 7977 '<FieldAccess for type_expression at ast.py:14725>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14725
--# expr-start 7976 '<FieldAccess for formal_decl at ast.py:14725>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14725
--# expr-start 7975 '<at at ast.py:14724>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14724
--# expr-start 7974 '<FieldAccess for unpacked_formal_params at ast.py:14724>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14724
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 7974
Get_Result := Get (Self, Fld_3, 0, True); 
--# expr-done 7975
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 7976
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 7977
--# expr-start 7978 '<FieldAccess for prefix at ast.py:14726>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14726
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Dotted_Name_F_Prefix, Info => Var_Expr_1.Info);
--# expr-done 7978
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
New_Struct := (Expected_Type => Fld_5, Expr => Cast_Result_6); 
--# expr-done 7979
                Singleton := Create_Internal_Expected_Type_For_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
Scope_Result_4 := Singleton;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2954;
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_5 := Result_Var;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2949;
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var := Result_Var_1; Inc_Ref (Block_Var);
--# bind prefix Block_Var
--# expr-start 7988 '<Map at ast.py:14733>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14733
--# expr-start 7980 '<Cast to CallExpr at ast.py:14733>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14733
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 7980
Var_Expr_3 := Cast_Result_7;
if Var_Expr_3 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 7981 '<FieldAccess for params at ast.py:14733>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14733
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_7 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 7981
Scope_Result_6 := Fld_7;
   --# end
   Result_Var_2 := Scope_Result_6;
else
   Result_Var_2 := No_Entity_Assoc_List;
end if;
Var_Expr_2 := Result_Var_2;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 7982 '<FieldAccess for zip_with_params at ast.py:14733>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14733
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.zip_with_params
Fld_8 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7982
Scope_Result_7 := Fld_8;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2956;
   Result_Var_3 := Scope_Result_7;
else
   Result_Var_3 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
      declare
         Map_Result_Vec : Internal_Expected_Type_For_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Result_Var_3;
      begin
         for Item_259 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_259
      --# expr-start 7987 '<New ExpectedTypeForExpr at ast.py:14734>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14734
--# expr-start 7985 '<FieldAccess for type_expression at ast.py:14735>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14735
--# expr-start 7984 '<FieldAccess for basic_decl at ast.py:14735>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14735
--# expr-start 7983 '<FieldAccess for param at ast.py:14735>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14735
Fld_9 := Item_259.Param;
--# expr-done 7983
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_10 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 7984
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 7985
--# expr-start 7986 '<FieldAccess for actual at ast.py:14736>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14736
Fld_12 := Item_259.Actual;
--# expr-done 7986
New_Struct_1 := (Expected_Type => Fld_11, Expr => Fld_12); 
--# expr-done 7987
         declare
            Item_To_Append : constant Internal_Expected_Type_For_Expr := New_Struct_1;
         begin
            Internal_Expected_Type_For_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Expected_Type_For_Expr_Array
           (Items_Count => Natural (Internal_Expected_Type_For_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Expected_Type_For_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Expected_Type_For_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Expected_Type_For_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7988
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind args Block_Var_1
--# expr-start 7989 '<Concat at ast.py:14741>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14741
Concat_Result := Concat (Block_Var, Block_Var_1); 
--# expr-done 7989
Let_Result_4 := Concat_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 7990
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1062;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1062;
                     Finalizer_Scope_2949;
                     Finalizer_Scope_2954;
                     Finalizer_Scope_2956;
            raise;
      end;
   return Property_Result;
end Name_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start Name.is_dispatching_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14743
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Dispatching_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Name;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Call_Expr;
Cast_Result : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7997 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7991 '<FieldAccess for is_access_call at ast.py:14746>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14746
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_access_call
Fld := Libadalang.Implementation.Name_P_Is_Access_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7991
if Fld then
   If_Result_1 := True;
else
--# expr-start 7992 '<FieldAccess for is_direct_call at ast.py:14747>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14747
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_direct_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7992
if Fld_1 then
   --# expr-start 7996 '<FieldAccess for is_dispatching_call_impl at ast.py:14749>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14749
--# expr-start 7994 '<Cast to Name.entity at ast.py:14748>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14748
--# expr-start 7993 '<FieldAccess for parent_callexpr at ast.py:14748>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14748
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_callexpr
Fld_2 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7993
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 7994
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
      if Result_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7995 '<FieldAccess for referenced_decl at ast.py:14749>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14749
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_3 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7995
--# property-call-start Expr.is_dispatching_call_impl
Fld_4 := Libadalang.Implementation.Expr_P_Is_Dispatching_Call_Impl (Node => Result_Var.Node, Decl => Fld_3, Imprecise_Fallback => Imprecise_Fallback, E_Info => Result_Var.Info);
--# end
--# expr-done 7996
   If_Result := Fld_4;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 7997
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Dispatching_Call;
--# end
--# property-start Name.is_static_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14752
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Static_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8001 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 7998 '<FieldAccess for is_call at ast.py:14763>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14763
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7998
if Fld then
   --# expr-start 8000 '<Not at ast.py:14763>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14763
--# expr-start 7999 '<FieldAccess for is_dispatching_call at ast.py:14763>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14763
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7999
Not_Val := not (Fld_1); 
--# expr-done 8000
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 8001
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Static_Call;
--# end
--# property-start Name.as_single_tok_node_array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14765
pragma Warnings (Off, "is not referenced");
 function Name_P_As_Single_Tok_Node_Array
  (Node : Bare_Name
  )
   return Bare_Single_Tok_Node_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_1065 with Inline_Always;
            procedure Finalizer_Scope_2959 with Inline_Always;
            procedure Finalizer_Scope_2960 with Inline_Always;
            procedure Finalizer_Scope_2961 with Inline_Always;
      Match_0 : Bare_Defining_Name;
Match_1 : Bare_Single_Tok_Node;
Match_2 : Bare_Dotted_Name;
Match_3 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Single_Tok_Node_Array_Access;
Singleton : Bare_Single_Tok_Node_Array_Access;
Fld_2 : Bare_Name;
Fld_3 : Bare_Single_Tok_Node_Array_Access;
Fld_4 : Bare_Base_Id;
Fld_5 : Bare_Single_Tok_Node_Array_Access;
Concat_Result : Bare_Single_Tok_Node_Array_Access;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Defining_Name;
Let_Result : Bare_Single_Tok_Node_Array_Access;
Scope_Result : Bare_Single_Tok_Node_Array_Access;
Cast_Result_1 : Bare_Single_Tok_Node;
Let_Result_1 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_1 : Bare_Single_Tok_Node_Array_Access;
Cast_Result_2 : Bare_Dotted_Name;
Let_Result_2 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_2 : Bare_Single_Tok_Node_Array_Access;
Cast_Result_3 : Bare_Name;
Let_Result_3 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_3 : Bare_Single_Tok_Node_Array_Access;
Match_Result : Bare_Single_Tok_Node_Array_Access;
Let_Result_4 : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_1065 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1065;
            procedure Finalizer_Scope_2959 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2959;
            procedure Finalizer_Scope_2960 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2960;
            procedure Finalizer_Scope_2961 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2961;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8010 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 8009 '<Match at ast.py:14773>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14773
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind dname Match_0
--# expr-start 8003 '<FieldAccess for as_single_tok_node_array at ast.py:14774>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14774
--# expr-start 8002 '<FieldAccess for name at ast.py:14774>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14774
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Match_0.Defining_Name_F_Name;
--# expr-done 8002
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_1 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld);
--# end
--# expr-done 8003
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2959;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_Real_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind tok Match_1
                Singleton := Create_Bare_Single_Tok_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Match_1;
Let_Result_1 := Singleton; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2960;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind dot Match_2
--# expr-start 8008 '<Concat at ast.py:14777>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14777
--# expr-start 8005 '<FieldAccess for as_single_tok_node_array at ast.py:14777>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14777
--# expr-start 8004 '<FieldAccess for prefix at ast.py:14777>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14777
      if Match_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Match_2.Dotted_Name_F_Prefix;
--# expr-done 8004
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_3 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_2);
--# end
--# expr-done 8005
--# expr-start 8007 '<FieldAccess for as_single_tok_node_array at ast.py:14778>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14778
--# expr-start 8006 '<FieldAccess for suffix at ast.py:14778>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14778
      if Match_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Match_2.Dotted_Name_F_Suffix;
--# expr-done 8006
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_5 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_4);
--# end
--# expr-done 8007
Concat_Result := Concat (Fld_3, Fld_5); 
--# expr-done 8008
Let_Result_2 := Concat_Result; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2961;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 8009
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 8010
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1065;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1065;
                     Finalizer_Scope_2959;
                     Finalizer_Scope_2960;
                     Finalizer_Scope_2961;
            raise;
      end;
   return Property_Result;
end Name_P_As_Single_Tok_Node_Array;
--# end
--# property-start '[dispatcher]Name.as_symbol_array' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_As_Symbol_Array
  (Node : Bare_Name
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Defining_Name | Ada_Discrete_Subtype_Name .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.as_symbol_array
                  Property_Result := Name_P_As_Symbol_Array
                    (Self
                    );
                  --# end
               when Ada_Synthetic_Defining_Name =>
                  --# property-call-start SyntheticDefiningName.as_symbol_array
                  Property_Result := Synthetic_Defining_Name_P_As_Symbol_Array
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_As_Symbol_Array;
--# end
--# property-start '[dispatcher]Name.canonical_text' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Canonical_Text
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Reduce_Attribute_Ref | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.canonical_text
                  Property_Result := Name_P_Canonical_Text
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.canonical_text
                  Property_Result := Single_Tok_Node_P_Canonical_Text
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Canonical_Text;
--# end
--# property-start '[dispatcher]Name.is_constant' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Is_Constant
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Reduce_Attribute_Ref .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Null_Literal | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.is_constant
                  Property_Result := Name_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.is_constant
                  Property_Result := Attribute_Ref_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.is_constant
                  Property_Result := Call_Expr_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.is_constant
                  Property_Result := Dotted_Name_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.is_constant
                  Property_Result := Explicit_Deref_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.is_constant
                  Property_Result := Qual_Expr_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Identifier =>
                  --# property-call-start Identifier.is_constant
                  Property_Result := Identifier_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Int_Literal .. Ada_Real_Literal =>
                  --# property-call-start NumLiteral.is_constant
                  Property_Result := Num_Literal_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Is_Constant;
--# end
--# property-start Name.call_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14814
pragma Warnings (Off, "is not referenced");
 function Name_P_Call_Params
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1069 with Inline_Always;
            procedure Finalizer_Scope_2970 with Inline_Always;
            procedure Finalizer_Scope_2973 with Inline_Always;
      Block_Var : Boolean;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Expr;
Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Attribute_Ref;
Match_2 : Internal_Entity_Name;
Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Call_Expr;
Offset : Integer;
Aparams : Internal_Entity_Assoc_List;
Dparams : Internal_Param_Actual_Array_Access;
Var_Expr_2 : Internal_Entity_Call_Expr;
Var_Expr_3 : Internal_Entity_Param_Spec;
Var_Expr_4 : Internal_Entity_Assoc_List;
Fld : Boolean;
Fld_1 : Boolean;
Is_A : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Fld_2 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Scope_Result_1 : Internal_Entity_Name;
Result_Var_1 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Name;
Match_Prefix : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
Scope_Result_2 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Attribute_Ref;
Cast_Result_4 : Internal_Entity_Expr;
Let_Result_1 : Internal_Entity_Expr;
Scope_Result_3 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_2 : Internal_Entity_Expr;
Scope_Result_4 : Internal_Entity_Expr;
Match_Result : Internal_Entity_Expr;
If_Result_2 : Integer;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_6 : Internal_Entity_Call_Expr;
Fld_5 : Internal_Entity_Assoc_List;
Scope_Result_5 : Internal_Entity_Assoc_List;
Result_Var_2 : Internal_Entity_Assoc_List;
Fld_6 : Internal_Entity_Base_Formal_Param_Holder;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Defining_Name_Array_Access;
Cast_Expr_2 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_7 : Internal_Entity_Param_Spec;
Fld_9 : Internal_Entity_Expr;
Scope_Result_6 : Internal_Entity_Expr;
Result_Var_3 : Internal_Entity_Expr;
Is_Equal : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
Map_Result : Internal_Param_Actual_Array_Access;
Index_262 : Integer;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Internal_Entity_Defining_Name;
Arith_Result : Integer;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Internal_Entity_Expr;
Is_Equal_1 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Internal_Entity_Expr;
New_Struct_1 : Internal_Param_Actual;
Index_265 : Integer;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_7 : Internal_Param_Actual_Array_Access;
Result_Var_4 : Internal_Param_Actual_Array_Access;
Let_Result_3 : Internal_Param_Actual_Array_Access;
If_Result_7 : Internal_Param_Actual_Array_Access;
Let_Result_4 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1069 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Dparams);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1069;
            procedure Finalizer_Scope_2970 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2970;
            procedure Finalizer_Scope_2973 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2973;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8045 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 8011 '<FieldAccess for is_call at ast.py:14820>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14820
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8011
Block_Var := Fld; 
--# bind is_call Block_Var
--# expr-start 8014 '<If at ast.py:14821>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14821
if Block_Var then
--# expr-start 8012 '<FieldAccess for is_dot_call at ast.py:14823>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14823
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_dot_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 8012
if Fld_1 then
   If_Result := True;
else
   --# expr-start 8013 '<IsA AttributeRef at ast.py:14823>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14823
Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 8013
   If_Result := Is_A;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
--# expr-done 8014
Block_Var_1 := If_Result_1; 
--# bind is_prefix_call Block_Var_1
--# expr-start 8019 '<Match at ast.py:14827>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14827
--# expr-start 8015 '<Cast to CallExpr at ast.py:14827>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14827
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Call_Expr;
   end if;
--# expr-done 8015
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 8016 '<FieldAccess for name at ast.py:14827>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14827
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Call_Expr_F_Name, Info => Var_Expr_1.Info);
--# expr-done 8016
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Ent;
end if;
      if Result_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Result_Var_1; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind dn Match_0
--# expr-start 8017 '<FieldAccess for prefix at ast.py:14828>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14828
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 8017
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Let_Result := Cast_Result_2; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind ar Match_1
--# expr-start 8018 '<FieldAccess for prefix at ast.py:14829>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14829
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_1.Node.Attribute_Ref_F_Prefix, Info => Match_1.Info);
--# expr-done 8018
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Let_Result_1 := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Call_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Expr; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 8019
Block_Var_2 := Match_Result; 
--# bind call_prefix Block_Var_2
--# expr-start 8044 '<If at ast.py:14833>' If_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14833
if Block_Var then
   --# expr-start 8043 '<Let at ast.py:14836>' Let_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14836
--# scope-start
--# expr-start 8020 '<If at ast.py:14837>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14837
if Block_Var_1 then
   If_Result_2 := 1;
else
   If_Result_2 := 0;
end if;
--# expr-done 8020
Offset := If_Result_2; 
--# bind offset Offset
--# expr-start 8021 '<Cast to CallExpr at ast.py:14839>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14839
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_6 := No_Entity_Call_Expr;
   end if;
--# expr-done 8021
Var_Expr_2 := Cast_Result_6;
if Var_Expr_2 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 8022 '<FieldAccess for params at ast.py:14839>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14839
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_5 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 8022
Scope_Result_5 := Fld_5;
   --# end
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := No_Entity_Assoc_List;
end if;
Aparams := Result_Var_2; 
--# bind aparams Aparams
--# expr-start 8032 '<Mapcat at ast.py:14842>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14842
   --# expr-start 8024 '<FieldAccess for abstract_formal_params at ast.py:14842>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14842
--# expr-start 8023 '<FieldAccess for called_subp_spec at ast.py:14842>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14842
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_subp_spec'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8023
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 8024
      declare
         Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
         Index_262 := 0;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Fld_7;
      begin
         for Item_261 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_261
               --# bind i Index_262
      --# expr-start 8031 '<Map at ast.py:14843>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14843
   --# expr-start 8025 '<FieldAccess for defining_names at ast.py:14843>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14843
      if Item_261.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Item_261.Node, E_Info => Item_261.Info);
--# end
--# expr-done 8025
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_8;
      begin
         for Item_263 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_263
      --# expr-start 8030 '<New ParamActual at ast.py:14844>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14844
--# expr-start 8029 '<If at ast.py:14846>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14846
if Block_Var_1 then
   --# expr-start 8026 '<Eq at ast.py:14849>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14849
Is_Equal := Index_262 = 0; 
--# expr-done 8026
   If_Result_3 := Is_Equal;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   If_Result_4 := Block_Var_2;
else
--# expr-start 8027 '<Cast to ParamSpec at ast.py:14851>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14851
Cast_Expr_2 := Item_261; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_7 := No_Entity_Param_Spec;
   end if;
--# expr-done 8027
Var_Expr_3 := Cast_Result_7;
if Var_Expr_3 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 8028 '<FieldAccess for default_expr at ast.py:14851>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14851
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Expr (Node => Var_Expr_3.Node.Param_Spec_F_Default_Expr, Info => Var_Expr_3.Info);
--# expr-done 8028
Scope_Result_6 := Fld_9;
   --# end
   Result_Var_3 := Scope_Result_6;
else
   Result_Var_3 := No_Entity_Expr;
end if;
   If_Result_4 := Result_Var_3;
end if;
--# expr-done 8029
New_Struct := (Actual => If_Result_4, Param => Item_263); 
--# expr-done 8030
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8031
         for Item_To_Append of
               Map_Result.Items
         loop
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2970;
               Index_262 := Index_262 + 1;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 8032
Dparams := Map_Result_1; Inc_Ref (Dparams);
--# bind dparams Dparams
Var_Expr_4 := Aparams;
if Var_Expr_4 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind ap Var_Expr_4
--# expr-start 8042 '<Map at ast.py:14860>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14860
      declare
         Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
         Index_265 := 0;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Dparams;
      begin
         for Item_264 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_264
               --# bind i Index_265
      --# expr-start 8041 '<New ParamActual at ast.py:14861>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14861
--# expr-start 8039 '<If at ast.py:14865>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14865
if Block_Var_1 then
   --# expr-start 8033 '<Eq at ast.py:14868>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14868
Is_Equal_1 := Index_265 = 0; 
--# expr-done 8033
   If_Result_5 := Is_Equal_1;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   --# expr-start 8034 '<FieldAccess for actual at ast.py:14870>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14870
Fld_11 := Item_264.Actual;
--# expr-done 8034
   If_Result_6 := Fld_11;
else
   --# expr-start 8038 '<FieldAccess for actual_for_param_at at ast.py:14872>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14872
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8035 '<FieldAccess for param at ast.py:14873>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14873
Fld_12 := Item_264.Param;
--# expr-done 8035
--# expr-start 8036 '<Arithmetic - at ast.py:14873>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14873
Arith_Result := (Index_265 - Offset); 
--# expr-done 8036
--# expr-start 8037 '<FieldAccess for actual at ast.py:14873>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14873
Fld_13 := Item_264.Actual;
--# expr-done 8037
--# property-call-start AssocList.actual_for_param_at
Fld_14 := Libadalang.Implementation.Assoc_List_P_Actual_For_Param_At (Node => Var_Expr_4.Node, Param => Fld_12, Pos => Arith_Result, Default_Expr => Fld_13, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 8038
   If_Result_6 := Fld_14;
end if;
--# expr-done 8039
--# expr-start 8040 '<FieldAccess for param at ast.py:14862>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14862
Fld_10 := Item_264.Param;
--# expr-done 8040
New_Struct_1 := (Actual => If_Result_6, Param => Fld_10); 
--# expr-done 8041
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_1;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
               Index_265 := Index_265 + 1;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 8042
Scope_Result_7 := Map_Result_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2973;
   Result_Var_4 := Scope_Result_7;
else
   Result_Var_4 := Dparams;
end if;
      Inc_Ref (Result_Var_4);
Let_Result_3 := Result_Var_4; Inc_Ref (Let_Result_3);
--# end
--# expr-done 8043
   If_Result_7 := Let_Result_3;
else
   If_Result_7 := raise Property_Error with "this name doesn't reference a call expression";
end if;
      Inc_Ref (If_Result_7);
--# expr-done 8044
Let_Result_4 := If_Result_7; Inc_Ref (Let_Result_4);
--# end
--# expr-done 8045
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1069;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1069;
                     Finalizer_Scope_2970;
                     Finalizer_Scope_2973;
            raise;
      end;
   return Property_Result;
end Name_P_Call_Params;
--# end
 function Name_P_Parent_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is (raise Property_Error
    with "Property Name.parent_scope not implemented on type "
    & Kind_Name (Node));
 function Name_P_All_Env_Els_Impl
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is (raise Property_Error
    with "Property Name.all_env_els_impl not implemented on type "
    & Kind_Name (Node));
--# property-start Name.scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14228
pragma Warnings (Off, "is not referenced");
 function Name_P_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Scope;
--# end
 function Name_P_Ref_Var
  (Node : Bare_Name
  )
   return Logic_Var
is (raise Property_Error
    with "Property Name.ref_var not implemented on type "
    & Kind_Name (Node));
 function Name_P_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Logic_Var
is (raise Property_Error
    with "Property Name.subp_spec_var not implemented on type "
    & Kind_Name (Node));
--# property-start Name.defines_subp_spec_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14260
pragma Warnings (Off, "is not referenced");
 function Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8046 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 8046
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start Name.called_subp_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14270
pragma Warnings (Off, "is not referenced");
 function Name_P_Called_Subp_Spec
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_3 : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8053 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8052 '<If at ast.py:14276>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14276
--# expr-start 8047 '<FieldAccess for defines_subp_spec_var at ast.py:14277>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14277
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.defines_subp_spec_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Defines_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 8047
if Fld_3 then
   --# expr-start 8051 '<Cast to BaseFormalParamHolder at ast.py:14281>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14281
--# expr-start 8050 '<FieldAccess for value at ast.py:14281>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14281
--# expr-start 8049 '<FieldAccess for logic_val at ast.py:14278>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14278
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 8048 '<FieldAccess for subp_spec_var at ast.py:14280>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14280
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 8048
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 8049
Fld_2 := Fld_1.Value;
--# expr-done 8050
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Formal_Param_Holder
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Base_Formal_Param_Holder;
   end if;
--# expr-done 8051
   If_Result := Cast_Result_1;
else
   If_Result := No_Entity_Base_Formal_Param_Holder;
end if;
--# expr-done 8052
Let_Result := If_Result; 
--# end
--# expr-done 8053
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Called_Subp_Spec;
--# end
--# property-start Name.designated_type_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14428
pragma Warnings (Off, "is not referenced");
 function Name_P_Designated_Type_Impl
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Designated_Type_Impl;
--# end
 function Name_P_Relative_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is (raise Property_Error
    with "Property Name.relative_name not implemented on type "
    & Kind_Name (Node));
--# property-start Name.name_symbol /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14539
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Symbol
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8056 '<FieldAccess for name_symbol at ast.py:14540>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14540
--# expr-start 8055 '<FieldAccess for relative_name at ast.py:14540>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14540
--# expr-start 8054 '<as_bare_entity at ast.py:14540>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14540
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 8054
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 8055
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8056
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Name_Symbol;
--# end
--# property-start Name.as_symbol_array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14783
pragma Warnings (Off, "is not referenced");
 function Name_P_As_Symbol_Array
  (Node : Bare_Name
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1066 with Inline_Always;
      Fld : Bare_Single_Tok_Node_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1066 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1066;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8060 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8059 '<Map at ast.py:14794>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14794
   --# expr-start 8057 '<FieldAccess for as_single_tok_node_array at ast.py:14794>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14794
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Self);
--# end
--# expr-done 8057
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld;
      begin
         for Item_260 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_260
      --# expr-start 8058 '<GetSymbol at ast.py:14794>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14794
Sym := Get_Symbol (Item_260); 
--# expr-done 8058
         declare
            Item_To_Append : constant Symbol_Type := Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8059
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8060
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1066;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1066;
            raise;
      end;
   return Property_Result;
end Name_P_As_Symbol_Array;
--# end
--# property-start Name.canonical_text /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14796
pragma Warnings (Off, "is not referenced");
 function Name_P_Canonical_Text
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_1067 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
Str : String_Type;
Fld_1 : String_Type;
Sym : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_1067 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1067;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8065 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8064 '<to_symbol at ast.py:14804>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14804
--# expr-start 8063 '<FieldAccess for sym_join at ast.py:14804>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14804
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8061 '<FieldAccess for as_symbol_array at ast.py:14804>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14804
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Self);
--# end
--# expr-done 8061
--# expr-start 8062 '<String at ast.py:14804>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14804
Str := Create_String ("."); 
--# expr-done 8062
--# property-call-start AdaNode.sym_join
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Self, Syms => Fld, Sep => Str);
--# end
--# expr-done 8063
Sym := String_To_Symbol (Self, Self.Unit.Context, Fld_1); 
--# expr-done 8064
Let_Result := Sym; 
--# end
--# expr-done 8065
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1067;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1067;
            raise;
      end;
   return Property_Result;
end Name_P_Canonical_Text;
--# end
 function Name_P_Is_Constant
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is (raise Property_Error
    with "Property Name.is_constant not implemented on type "
    & Kind_Name (Node));
function Name_P_Name_Designated_Type_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Name
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Name_P_Name_Designated_Type_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
function Name_P_Use_Package_Name_Designated_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Name
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Name_P_Use_Package_Name_Designated_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Attribute_Ref
   --
      procedure Initialize_Fields_For_Attribute_Ref
        (Self : Bare_Attribute_Ref
         ; Attribute_Ref_F_Prefix : Bare_Name
         ; Attribute_Ref_F_Attribute : Bare_Identifier
         ; Attribute_Ref_F_Args : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Attribute_Ref_F_Prefix := Attribute_Ref_F_Prefix;
            Self.Attribute_Ref_F_Attribute := Attribute_Ref_F_Attribute;
            Self.Attribute_Ref_F_Args := Attribute_Ref_F_Args;
      Self.Attribute_Ref_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Attribute_Ref;
   function Attribute_Ref_F_Prefix
     (Node : Bare_Attribute_Ref) return Bare_Name
   is
   begin
         return Node.Attribute_Ref_F_Prefix;
   end;
   function Attribute_Ref_F_Attribute
     (Node : Bare_Attribute_Ref) return Bare_Identifier
   is
   begin
         return Node.Attribute_Ref_F_Attribute;
   end;
   function Attribute_Ref_F_Args
     (Node : Bare_Attribute_Ref) return Bare_Assoc_List
   is
   begin
         return Node.Attribute_Ref_F_Args;
   end;
--# property-start AttributeRef.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19082
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Ref_Var
  (Node : Bare_Attribute_Ref
  )
   return Logic_Var
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8066 '<FieldAccess for r_ref_var at ast.py:19082>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19082
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_R_Ref_Var'Unrestricted_Access;
--# expr-done 8066
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Ref_Var;
--# end
--# property-start AttributeRef.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19085
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Relative_Name
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8068 '<FieldAccess for relative_name at ast.py:19085>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19085
--# expr-start 8067 '<FieldAccess for prefix at ast.py:19085>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19085
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8067
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8068
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Relative_Name;
--# end
--# property-start AttributeRef.is_constant /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19087
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Is_Constant
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Is_Constant;
--# end
--# property-start AttributeRef.designated_type_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19089
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Type_Impl
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Bare_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Result_Var : Internal_Entity_Classwide_Type_Decl;
Fld_5 : Bare_Identifier;
Fld_6 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Discrete_Base_Subtype_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8083 '<Cond at ast.py:19089>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19089
--# expr-start 8072 '<Eq at ast.py:19090>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19090
--# expr-start 8070 '<FieldAccess for sym at ast.py:19090>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19090
--# expr-start 8069 '<FieldAccess for attribute at ast.py:19090>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19090
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 8069
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld);
--# end
--# expr-done 8070
--# expr-start 8071 '<Symbol Class at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Class); 
--# expr-done 8071
Is_Equal := Fld_1 = Sym; 
--# expr-done 8072
if Is_Equal then
--# expr-start 8074 '<FieldAccess for designated_type_impl at ast.py:19091>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19091
--# expr-start 8073 '<FieldAccess for prefix at ast.py:19091>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19091
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8073
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8074
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8075 '<FieldAccess for classwide_type at ast.py:19091>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19091
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 8075
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Classwide_Type_Decl;
end if;
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_1 := Cast_Result_1;
else
--# expr-start 8079 '<Eq at ast.py:19093>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19093
--# expr-start 8077 '<FieldAccess for sym at ast.py:19093>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19093
--# expr-start 8076 '<FieldAccess for attribute at ast.py:19093>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19093
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Attribute_Ref_F_Attribute;
--# expr-done 8076
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_6 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_5);
--# end
--# expr-done 8077
--# expr-start 8078 '<Symbol Base at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Base); 
--# expr-done 8078
Is_Equal_1 := Fld_6 = Sym_1; 
--# expr-done 8079
if Is_Equal_1 then
--# expr-start 8082 '<FieldAccess for scalar_base_subtype at ast.py:19094>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19094
--# expr-start 8081 '<FieldAccess for name_designated_type at ast.py:19094>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19094
--# expr-start 8080 '<FieldAccess for prefix at ast.py:19094>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19094
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8080
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 8081
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 8082
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   If_Result := Cast_Result;
else
   If_Result := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 8083
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Designated_Type_Impl;
--# end
--# property-start AttributeRef.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19099
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Has_Context_Free_Type
  (Node : Bare_Attribute_Ref
  )
   return Boolean
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8085 '<Not at ast.py:19099>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19099
--# expr-start 8084 '<FieldAccess for is_access_attr at ast.py:19099>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19099
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Self);
--# end
--# expr-done 8084
Not_Val := not (Fld); 
--# expr-done 8085
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Has_Context_Free_Type;
--# end
--# property-start AttributeRef.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19101
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Env_Elements_Impl
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1338 with Inline_Always;
      Fld : Bare_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
If_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1338 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1338;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8093 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8092 '<Cond at ast.py:19103>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19103
--# expr-start 8089 '<Eq at ast.py:19104>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19104
--# expr-start 8087 '<FieldAccess for sym at ast.py:19104>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19104
--# expr-start 8086 '<FieldAccess for attribute at ast.py:19104>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19104
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 8086
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld);
--# end
--# expr-done 8087
--# expr-start 8088 '<Symbol Unrestricted_Access at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unrestricted_Access); 
--# expr-done 8088
Is_Equal := Fld_1 = Sym; 
--# expr-done 8089
if Is_Equal then
   --# expr-start 8091 '<FieldAccess for env_elements_impl at ast.py:19109>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19109
--# expr-start 8090 '<FieldAccess for prefix at ast.py:19109>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19109
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8090
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld_2.Node, Env => Env, E_Info => Fld_2.Info);
--# end
--# expr-done 8091
   If_Result := Fld_3;
else
   If_Result := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 8092
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8093
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1338;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1338;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Env_Elements_Impl;
--# end
--# property-start AttributeRef.is_access_attr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19114
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Is_Access_Attr
  (Node : Bare_Attribute_Ref
  )
   return Boolean
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Bare_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Bare_Identifier;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 8097 '<Eq at ast.py:19115>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
--# expr-start 8095 '<FieldAccess for name_symbol at ast.py:19115>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
--# expr-start 8094 '<FieldAccess for attribute at ast.py:19115>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 8094
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 8095
--# expr-start 8096 '<Symbol Access at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Access); 
--# expr-done 8096
Is_Equal := Fld_1 = Sym; 
--# expr-done 8097
if Is_Equal then
   If_Result := True;
else
   --# expr-start 8101 '<Eq at ast.py:19115>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
--# expr-start 8099 '<FieldAccess for name_symbol at ast.py:19115>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
--# expr-start 8098 '<FieldAccess for attribute at ast.py:19115>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Attribute_Ref_F_Attribute;
--# expr-done 8098
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2);
--# end
--# expr-done 8099
--# expr-start 8100 '<Symbol Unchecked_Access at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unchecked_Access); 
--# expr-done 8100
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 8101
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 8105 '<Eq at ast.py:19115>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
--# expr-start 8103 '<FieldAccess for name_symbol at ast.py:19115>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
--# expr-start 8102 '<FieldAccess for attribute at ast.py:19115>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19115
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Attribute_Ref_F_Attribute;
--# expr-done 8102
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4);
--# end
--# expr-done 8103
--# expr-start 8104 '<Symbol Unrestricted_Access at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unrestricted_Access); 
--# expr-done 8104
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 8105
   If_Result_1 := Is_Equal_2;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Is_Access_Attr;
--# end
--# property-start AttributeRef.designated_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19120
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Env
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1340 with Inline_Always;
      Fld : Boolean;
Fld_1 : Internal_Entity_Name;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_4 : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Base_Subp_Spec;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1340 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld_7);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1340;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8121 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8120 '<Cond at ast.py:19122>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19122
--# expr-start 8106 '<FieldAccess for is_access_attr at ast.py:19123>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19123
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Ent.Node);
--# end
--# expr-done 8106
if Fld then
   --# expr-start 8108 '<FieldAccess for designated_env at ast.py:19124>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19124
--# expr-start 8107 '<FieldAccess for prefix at ast.py:19124>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19124
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8107
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_1.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld_1.Info);
--# end
--# expr-done 8108
   If_Result_2 := Fld_2;
else
--# expr-start 8111 '<FieldAccess for name_is at ast.py:19126>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19126
--# expr-start 8109 '<FieldAccess for attribute at ast.py:19126>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19126
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8109
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8110 '<Symbol Result at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Result); 
--# expr-done 8110
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Sym);
--# end
--# expr-done 8111
if Fld_4 then
   --# expr-start 8119 '<FieldAccess for defining_env at ast.py:19129>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19129
--# expr-start 8118 '<FieldAccess for return_type at ast.py:19129>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19129
--# expr-start 8117 '<FieldAccess for subp_spec_or_null at ast.py:19128>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19128
--# expr-start 8116 '<Cast to BasicDecl at ast.py:19128>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19128
--# expr-start 8115 '<as_entity at ast.py:19128>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19128
--# expr-start 8114 '<Find at ast.py:19127>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19127
   --# expr-start 8112 '<FieldAccess for parents at ast.py:19127>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19127
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 8112
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_266 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_266
      --# expr-start 8113 '<IsA BasicSubpDecl, SubpBody at ast.py:19127>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19127
Is_A := Item_266 /= null 
and then Item_266.Kind in Ada_Basic_Subp_Decl | Ada_Subp_Body_Range; 
--# expr-done 8113
      if Is_A then
         Find_Result := Item_266; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 8114
Is_Null := Find_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Find_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 8115
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 8116
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 8117
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_6 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_5.Node, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 8118
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_6.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld_6.Info);
--# end
--# expr-done 8119
   If_Result_1 := Fld_7;
else
   If_Result_1 := Empty_Env;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 8120
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 8121
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1340;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1340;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Designated_Env;
--# end
--# property-start AttributeRef.synthesize_attribute_subprogram /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19134
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Synthesize_Attribute_Subprogram
  (Node : Bare_Attribute_Ref
      ; Typ : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Subp_Decl
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   Property_Result : Internal_Entity_Basic_Subp_Decl;
      Block_Var : Symbol_Type;
Block_Var_1 : Bare_Type_Attributes_Repository;
Block_Var_2 : Bare_Basic_Subp_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Fld_2 : Bare_Type_Attributes_Repository;
Scope_Result : Bare_Type_Attributes_Repository;
Result_Var : Bare_Type_Attributes_Repository;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_3 : Bare_Basic_Subp_Decl;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
Fld_4 : Bare_Basic_Subp_Decl;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
Fld_5 : Bare_Basic_Subp_Decl;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
Fld_6 : Bare_Basic_Subp_Decl;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Fld_7 : Bare_Basic_Subp_Decl;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
Fld_8 : Bare_Basic_Subp_Decl;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
Fld_9 : Bare_Basic_Subp_Decl;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
Fld_10 : Bare_Basic_Subp_Decl;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
Fld_11 : Bare_Basic_Subp_Decl;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
Fld_12 : Bare_Basic_Subp_Decl;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
Fld_13 : Bare_Basic_Subp_Decl;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
Fld_14 : Bare_Basic_Subp_Decl;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
Fld_15 : Bare_Basic_Subp_Decl;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
Fld_16 : Bare_Basic_Subp_Decl;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
Fld_17 : Bare_Basic_Subp_Decl;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
Fld_18 : Bare_Basic_Subp_Decl;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
Fld_19 : Bare_Basic_Subp_Decl;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
Fld_20 : Bare_Basic_Subp_Decl;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
Fld_21 : Bare_Basic_Subp_Decl;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
Fld_22 : Bare_Basic_Subp_Decl;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
Fld_23 : Bare_Basic_Subp_Decl;
Sym_21 : Symbol_Type;
Is_Equal_21 : Boolean;
Fld_24 : Bare_Basic_Subp_Decl;
Sym_22 : Symbol_Type;
Is_Equal_22 : Boolean;
Fld_25 : Bare_Basic_Subp_Decl;
Sym_23 : Symbol_Type;
Is_Equal_23 : Boolean;
Fld_26 : Bare_Basic_Subp_Decl;
Sym_24 : Symbol_Type;
Is_Equal_24 : Boolean;
Fld_27 : Bare_Basic_Subp_Decl;
Sym_25 : Symbol_Type;
Is_Equal_25 : Boolean;
Fld_28 : Bare_Basic_Subp_Decl;
Sym_26 : Symbol_Type;
Is_Equal_26 : Boolean;
Fld_29 : Bare_Basic_Subp_Decl;
Sym_27 : Symbol_Type;
Is_Equal_27 : Boolean;
Fld_30 : Bare_Basic_Subp_Decl;
Sym_28 : Symbol_Type;
Is_Equal_28 : Boolean;
Fld_31 : Bare_Basic_Subp_Decl;
Sym_29 : Symbol_Type;
Is_Equal_29 : Boolean;
Fld_32 : Bare_Basic_Subp_Decl;
Sym_30 : Symbol_Type;
Is_Equal_30 : Boolean;
Fld_33 : Bare_Basic_Subp_Decl;
Sym_31 : Symbol_Type;
Is_Equal_31 : Boolean;
Fld_34 : Bare_Basic_Subp_Decl;
Sym_32 : Symbol_Type;
Is_Equal_32 : Boolean;
Fld_35 : Bare_Basic_Subp_Decl;
Sym_33 : Symbol_Type;
Is_Equal_33 : Boolean;
Fld_36 : Bare_Basic_Subp_Decl;
Sym_34 : Symbol_Type;
Is_Equal_34 : Boolean;
Fld_37 : Bare_Basic_Subp_Decl;
Sym_35 : Symbol_Type;
Is_Equal_35 : Boolean;
Fld_38 : Bare_Basic_Subp_Decl;
Sym_36 : Symbol_Type;
Is_Equal_36 : Boolean;
Fld_39 : Bare_Basic_Subp_Decl;
Sym_37 : Symbol_Type;
Is_Equal_37 : Boolean;
Fld_40 : Bare_Basic_Subp_Decl;
Sym_38 : Symbol_Type;
Is_Equal_38 : Boolean;
Fld_41 : Bare_Basic_Subp_Decl;
Sym_39 : Symbol_Type;
Is_Equal_39 : Boolean;
Fld_42 : Bare_Basic_Subp_Decl;
If_Result : Bare_Basic_Subp_Decl;
If_Result_1 : Bare_Basic_Subp_Decl;
If_Result_2 : Bare_Basic_Subp_Decl;
If_Result_3 : Bare_Basic_Subp_Decl;
If_Result_4 : Bare_Basic_Subp_Decl;
If_Result_5 : Bare_Basic_Subp_Decl;
If_Result_6 : Bare_Basic_Subp_Decl;
If_Result_7 : Bare_Basic_Subp_Decl;
If_Result_8 : Bare_Basic_Subp_Decl;
If_Result_9 : Bare_Basic_Subp_Decl;
If_Result_10 : Bare_Basic_Subp_Decl;
If_Result_11 : Bare_Basic_Subp_Decl;
If_Result_12 : Bare_Basic_Subp_Decl;
If_Result_13 : Bare_Basic_Subp_Decl;
If_Result_14 : Bare_Basic_Subp_Decl;
If_Result_15 : Bare_Basic_Subp_Decl;
If_Result_16 : Bare_Basic_Subp_Decl;
If_Result_17 : Bare_Basic_Subp_Decl;
If_Result_18 : Bare_Basic_Subp_Decl;
If_Result_19 : Bare_Basic_Subp_Decl;
If_Result_20 : Bare_Basic_Subp_Decl;
If_Result_21 : Bare_Basic_Subp_Decl;
If_Result_22 : Bare_Basic_Subp_Decl;
If_Result_23 : Bare_Basic_Subp_Decl;
If_Result_24 : Bare_Basic_Subp_Decl;
If_Result_25 : Bare_Basic_Subp_Decl;
If_Result_26 : Bare_Basic_Subp_Decl;
If_Result_27 : Bare_Basic_Subp_Decl;
If_Result_28 : Bare_Basic_Subp_Decl;
If_Result_29 : Bare_Basic_Subp_Decl;
If_Result_30 : Bare_Basic_Subp_Decl;
If_Result_31 : Bare_Basic_Subp_Decl;
If_Result_32 : Bare_Basic_Subp_Decl;
If_Result_33 : Bare_Basic_Subp_Decl;
If_Result_34 : Bare_Basic_Subp_Decl;
If_Result_35 : Bare_Basic_Subp_Decl;
If_Result_36 : Bare_Basic_Subp_Decl;
If_Result_37 : Bare_Basic_Subp_Decl;
If_Result_38 : Bare_Basic_Subp_Decl;
If_Result_39 : Bare_Basic_Subp_Decl;
Fld_43 : Internal_Entity_Info;
Fld_44 : Env_Rebindings;
Fld_45 : Internal_Entity_Info;
Fld_46 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Basic_Subp_Decl;
Let_Result : Internal_Entity_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8252 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8123 '<FieldAccess for name_symbol at ast.py:19136>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19136
--# expr-start 8122 '<FieldAccess for attribute at ast.py:19136>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19136
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8122
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8123
Block_Var := Fld_1; 
--# bind rel_name Block_Var
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8124 '<FieldAccess for attributes_repo at ast.py:19137>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19137
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.attributes_repo
Fld_2 := Libadalang.Implementation.Base_Type_Decl_F_Attributes_Repo (Node => Var_Expr.Node);
--# end
--# expr-done 8124
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Block_Var_1 := Result_Var; 
--# bind repo Block_Var_1
--# expr-start 8245 '<Cond at ast.py:19138>' If_Result_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19138
--# expr-start 8126 '<Eq at ast.py:19139>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19139
--# expr-start 8125 '<Symbol succ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Succ_383); 
--# expr-done 8125
Is_Equal := Block_Var = Sym; 
--# expr-done 8126
if Is_Equal then
   --# expr-start 8127 '<FieldAccess for succ at ast.py:19139>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19139
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.succ
Fld_3 := Libadalang.Implementation.Type_Attributes_Repository_F_Succ (Node => Block_Var_1);
--# end
--# expr-done 8127
   If_Result_39 := Fld_3;
else
--# expr-start 8129 '<Eq at ast.py:19140>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19140
--# expr-start 8128 '<Symbol pred at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pred_357); 
--# expr-done 8128
Is_Equal_1 := Block_Var = Sym_1; 
--# expr-done 8129
if Is_Equal_1 then
   --# expr-start 8130 '<FieldAccess for pred at ast.py:19140>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19140
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.pred
Fld_4 := Libadalang.Implementation.Type_Attributes_Repository_F_Pred (Node => Block_Var_1);
--# end
--# expr-done 8130
   If_Result_38 := Fld_4;
else
--# expr-start 8132 '<Eq at ast.py:19141>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19141
--# expr-start 8131 '<Symbol min at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Min_342); 
--# expr-done 8131
Is_Equal_2 := Block_Var = Sym_2; 
--# expr-done 8132
if Is_Equal_2 then
   --# expr-start 8133 '<FieldAccess for min at ast.py:19141>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19141
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.min
Fld_5 := Libadalang.Implementation.Type_Attributes_Repository_F_Min (Node => Block_Var_1);
--# end
--# expr-done 8133
   If_Result_37 := Fld_5;
else
--# expr-start 8135 '<Eq at ast.py:19142>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19142
--# expr-start 8134 '<Symbol max at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max_341); 
--# expr-done 8134
Is_Equal_3 := Block_Var = Sym_3; 
--# expr-done 8135
if Is_Equal_3 then
   --# expr-start 8136 '<FieldAccess for max at ast.py:19142>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19142
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.max
Fld_6 := Libadalang.Implementation.Type_Attributes_Repository_F_Max (Node => Block_Var_1);
--# end
--# expr-done 8136
   If_Result_36 := Fld_6;
else
--# expr-start 8138 '<Eq at ast.py:19144>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19144
--# expr-start 8137 '<Symbol rounding at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rounding_376); 
--# expr-done 8137
Is_Equal_4 := Block_Var = Sym_4; 
--# expr-done 8138
if Is_Equal_4 then
   --# expr-start 8139 '<FieldAccess for rounding at ast.py:19144>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19144
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.rounding
Fld_7 := Libadalang.Implementation.Type_Attributes_Repository_F_Rounding (Node => Block_Var_1);
--# end
--# expr-done 8139
   If_Result_35 := Fld_7;
else
--# expr-start 8141 '<Eq at ast.py:19145>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19145
--# expr-start 8140 '<Symbol unbiased_rounding at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unbiased_Rounding_392); 
--# expr-done 8140
Is_Equal_5 := Block_Var = Sym_5; 
--# expr-done 8141
if Is_Equal_5 then
   --# expr-start 8142 '<FieldAccess for unbiased_rounding at ast.py:19145>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19145
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.unbiased_rounding
Fld_8 := Libadalang.Implementation.Type_Attributes_Repository_F_Unbiased_Rounding (Node => Block_Var_1);
--# end
--# expr-done 8142
   If_Result_34 := Fld_8;
else
--# expr-start 8144 '<Eq at ast.py:19146>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19146
--# expr-start 8143 '<Symbol ceiling at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ceiling_303); 
--# expr-done 8143
Is_Equal_6 := Block_Var = Sym_6; 
--# expr-done 8144
if Is_Equal_6 then
   --# expr-start 8145 '<FieldAccess for ceiling at ast.py:19146>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19146
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.ceiling
Fld_9 := Libadalang.Implementation.Type_Attributes_Repository_F_Ceiling (Node => Block_Var_1);
--# end
--# expr-done 8145
   If_Result_33 := Fld_9;
else
--# expr-start 8147 '<Eq at ast.py:19147>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19147
--# expr-start 8146 '<Symbol floor at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Floor_322); 
--# expr-done 8146
Is_Equal_7 := Block_Var = Sym_7; 
--# expr-done 8147
if Is_Equal_7 then
   --# expr-start 8148 '<FieldAccess for floor at ast.py:19147>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19147
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.floor
Fld_10 := Libadalang.Implementation.Type_Attributes_Repository_F_Floor (Node => Block_Var_1);
--# end
--# expr-done 8148
   If_Result_32 := Fld_10;
else
--# expr-start 8150 '<Eq at ast.py:19148>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19148
--# expr-start 8149 '<Symbol truncation at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Truncation_390); 
--# expr-done 8149
Is_Equal_8 := Block_Var = Sym_8; 
--# expr-done 8150
if Is_Equal_8 then
   --# expr-start 8151 '<FieldAccess for truncation at ast.py:19148>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19148
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.truncation
Fld_11 := Libadalang.Implementation.Type_Attributes_Repository_F_Truncation (Node => Block_Var_1);
--# end
--# expr-done 8151
   If_Result_31 := Fld_11;
else
--# expr-start 8153 '<Eq at ast.py:19149>' Is_Equal_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19149
--# expr-start 8152 '<Symbol machine at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_339); 
--# expr-done 8152
Is_Equal_9 := Block_Var = Sym_9; 
--# expr-done 8153
if Is_Equal_9 then
   --# expr-start 8154 '<FieldAccess for machine at ast.py:19149>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19149
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.machine
Fld_12 := Libadalang.Implementation.Type_Attributes_Repository_F_Machine (Node => Block_Var_1);
--# end
--# expr-done 8154
   If_Result_30 := Fld_12;
else
--# expr-start 8156 '<Eq at ast.py:19150>' Is_Equal_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19150
--# expr-start 8155 '<Symbol machine_rounding at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Rounding_340); 
--# expr-done 8155
Is_Equal_10 := Block_Var = Sym_10; 
--# expr-done 8156
if Is_Equal_10 then
   --# expr-start 8157 '<FieldAccess for machine_rounding at ast.py:19150>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19150
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.machine_rounding
Fld_13 := Libadalang.Implementation.Type_Attributes_Repository_F_Machine_Rounding (Node => Block_Var_1);
--# end
--# expr-done 8157
   If_Result_29 := Fld_13;
else
--# expr-start 8159 '<Eq at ast.py:19151>' Is_Equal_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19151
--# expr-start 8158 '<Symbol fraction at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fraction_324); 
--# expr-done 8158
Is_Equal_11 := Block_Var = Sym_11; 
--# expr-done 8159
if Is_Equal_11 then
   --# expr-start 8160 '<FieldAccess for fraction at ast.py:19151>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19151
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.fraction
Fld_14 := Libadalang.Implementation.Type_Attributes_Repository_F_Fraction (Node => Block_Var_1);
--# end
--# expr-done 8160
   If_Result_28 := Fld_14;
else
--# expr-start 8162 '<Eq at ast.py:19152>' Is_Equal_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19152
--# expr-start 8161 '<Symbol exponent at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exponent_320); 
--# expr-done 8161
Is_Equal_12 := Block_Var = Sym_12; 
--# expr-done 8162
if Is_Equal_12 then
   --# expr-start 8163 '<FieldAccess for exponent at ast.py:19152>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19152
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.exponent
Fld_15 := Libadalang.Implementation.Type_Attributes_Repository_F_Exponent (Node => Block_Var_1);
--# end
--# expr-done 8163
   If_Result_27 := Fld_15;
else
--# expr-start 8165 '<Eq at ast.py:19154>' Is_Equal_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19154
--# expr-start 8164 '<Symbol copy_sign at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Copy_Sign_306); 
--# expr-done 8164
Is_Equal_13 := Block_Var = Sym_13; 
--# expr-done 8165
if Is_Equal_13 then
   --# expr-start 8166 '<FieldAccess for copy_sign at ast.py:19154>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19154
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.copy_sign
Fld_16 := Libadalang.Implementation.Type_Attributes_Repository_F_Copy_Sign (Node => Block_Var_1);
--# end
--# expr-done 8166
   If_Result_26 := Fld_16;
else
--# expr-start 8168 '<Eq at ast.py:19155>' Is_Equal_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19155
--# expr-start 8167 '<Symbol remainder at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remainder_367); 
--# expr-done 8167
Is_Equal_14 := Block_Var = Sym_14; 
--# expr-done 8168
if Is_Equal_14 then
   --# expr-start 8169 '<FieldAccess for remainder at ast.py:19155>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19155
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.remainder
Fld_17 := Libadalang.Implementation.Type_Attributes_Repository_F_Remainder (Node => Block_Var_1);
--# end
--# expr-done 8169
   If_Result_25 := Fld_17;
else
--# expr-start 8171 '<Eq at ast.py:19156>' Is_Equal_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19156
--# expr-start 8170 '<Symbol adjacent at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Adjacent_292); 
--# expr-done 8170
Is_Equal_15 := Block_Var = Sym_15; 
--# expr-done 8171
if Is_Equal_15 then
   --# expr-start 8172 '<FieldAccess for adjacent at ast.py:19156>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19156
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.adjacent
Fld_18 := Libadalang.Implementation.Type_Attributes_Repository_F_Adjacent (Node => Block_Var_1);
--# end
--# expr-done 8172
   If_Result_24 := Fld_18;
else
--# expr-start 8174 '<Eq at ast.py:19157>' Is_Equal_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19157
--# expr-start 8173 '<Symbol scaling at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scaling_377); 
--# expr-done 8173
Is_Equal_16 := Block_Var = Sym_16; 
--# expr-done 8174
if Is_Equal_16 then
   --# expr-start 8175 '<FieldAccess for scaling at ast.py:19157>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19157
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.scaling
Fld_19 := Libadalang.Implementation.Type_Attributes_Repository_F_Scaling (Node => Block_Var_1);
--# end
--# expr-done 8175
   If_Result_23 := Fld_19;
else
--# expr-start 8177 '<Eq at ast.py:19158>' Is_Equal_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19158
--# expr-start 8176 '<Symbol compose at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compose_304); 
--# expr-done 8176
Is_Equal_17 := Block_Var = Sym_17; 
--# expr-done 8177
if Is_Equal_17 then
   --# expr-start 8178 '<FieldAccess for compose at ast.py:19158>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19158
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.compose
Fld_20 := Libadalang.Implementation.Type_Attributes_Repository_F_Compose (Node => Block_Var_1);
--# end
--# expr-done 8178
   If_Result_22 := Fld_20;
else
--# expr-start 8180 '<Eq at ast.py:19159>' Is_Equal_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19159
--# expr-start 8179 '<Symbol leading_part at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Leading_Part_335); 
--# expr-done 8179
Is_Equal_18 := Block_Var = Sym_18; 
--# expr-done 8180
if Is_Equal_18 then
   --# expr-start 8181 '<FieldAccess for leading_part at ast.py:19159>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19159
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.leading_part
Fld_21 := Libadalang.Implementation.Type_Attributes_Repository_F_Leading_Part (Node => Block_Var_1);
--# end
--# expr-done 8181
   If_Result_21 := Fld_21;
else
--# expr-start 8183 '<Eq at ast.py:19161>' Is_Equal_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19161
--# expr-start 8182 '<Symbol mod at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod_343); 
--# expr-done 8182
Is_Equal_19 := Block_Var = Sym_19; 
--# expr-done 8183
if Is_Equal_19 then
   --# expr-start 8184 '<FieldAccess for mod at ast.py:19161>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19161
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.mod
Fld_22 := Libadalang.Implementation.Type_Attributes_Repository_F_Mod (Node => Block_Var_1);
--# end
--# expr-done 8184
   If_Result_20 := Fld_22;
else
--# expr-start 8186 '<Eq at ast.py:19163>' Is_Equal_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19163
--# expr-start 8185 '<Symbol image at ???>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Image_329); 
--# expr-done 8185
Is_Equal_20 := Block_Var = Sym_20; 
--# expr-done 8186
if Is_Equal_20 then
   --# expr-start 8187 '<FieldAccess for image at ast.py:19163>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19163
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.image
Fld_23 := Libadalang.Implementation.Type_Attributes_Repository_F_Image (Node => Block_Var_1);
--# end
--# expr-done 8187
   If_Result_19 := Fld_23;
else
--# expr-start 8189 '<Eq at ast.py:19164>' Is_Equal_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19164
--# expr-start 8188 '<Symbol wide_image at ???>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Image_399); 
--# expr-done 8188
Is_Equal_21 := Block_Var = Sym_21; 
--# expr-done 8189
if Is_Equal_21 then
   --# expr-start 8190 '<FieldAccess for wide_image at ast.py:19164>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19164
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.wide_image
Fld_24 := Libadalang.Implementation.Type_Attributes_Repository_F_Wide_Image (Node => Block_Var_1);
--# end
--# expr-done 8190
   If_Result_18 := Fld_24;
else
--# expr-start 8192 '<Eq at ast.py:19165>' Is_Equal_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19165
--# expr-start 8191 '<Symbol wide_wide_image at ???>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Image_402); 
--# expr-done 8191
Is_Equal_22 := Block_Var = Sym_22; 
--# expr-done 8192
if Is_Equal_22 then
   --# expr-start 8193 '<FieldAccess for wide_wide_image at ast.py:19165>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19165
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.wide_wide_image
Fld_25 := Libadalang.Implementation.Type_Attributes_Repository_F_Wide_Wide_Image (Node => Block_Var_1);
--# end
--# expr-done 8193
   If_Result_17 := Fld_25;
else
--# expr-start 8195 '<Eq at ast.py:19166>' Is_Equal_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19166
--# expr-start 8194 '<Symbol put_image at ???>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image_361); 
--# expr-done 8194
Is_Equal_23 := Block_Var = Sym_23; 
--# expr-done 8195
if Is_Equal_23 then
   --# expr-start 8196 '<FieldAccess for put_image at ast.py:19166>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19166
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.put_image
Fld_26 := Libadalang.Implementation.Type_Attributes_Repository_F_Put_Image (Node => Block_Var_1);
--# end
--# expr-done 8196
   If_Result_16 := Fld_26;
else
--# expr-start 8198 '<Eq at ast.py:19168>' Is_Equal_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19168
--# expr-start 8197 '<Symbol value at ???>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value_396); 
--# expr-done 8197
Is_Equal_24 := Block_Var = Sym_24; 
--# expr-done 8198
if Is_Equal_24 then
   --# expr-start 8199 '<FieldAccess for value at ast.py:19168>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19168
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.value
Fld_27 := Libadalang.Implementation.Type_Attributes_Repository_F_Value (Node => Block_Var_1);
--# end
--# expr-done 8199
   If_Result_15 := Fld_27;
else
--# expr-start 8201 '<Eq at ast.py:19169>' Is_Equal_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19169
--# expr-start 8200 '<Symbol wide_value at ???>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Value_401); 
--# expr-done 8200
Is_Equal_25 := Block_Var = Sym_25; 
--# expr-done 8201
if Is_Equal_25 then
   --# expr-start 8202 '<FieldAccess for wide_value at ast.py:19169>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19169
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.wide_value
Fld_28 := Libadalang.Implementation.Type_Attributes_Repository_F_Wide_Value (Node => Block_Var_1);
--# end
--# expr-done 8202
   If_Result_14 := Fld_28;
else
--# expr-start 8204 '<Eq at ast.py:19170>' Is_Equal_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19170
--# expr-start 8203 '<Symbol wide_wide_value at ???>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Value_404); 
--# expr-done 8203
Is_Equal_26 := Block_Var = Sym_26; 
--# expr-done 8204
if Is_Equal_26 then
   --# expr-start 8205 '<FieldAccess for wide_wide_value at ast.py:19170>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19170
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.wide_wide_value
Fld_29 := Libadalang.Implementation.Type_Attributes_Repository_F_Wide_Wide_Value (Node => Block_Var_1);
--# end
--# expr-done 8205
   If_Result_13 := Fld_29;
else
--# expr-start 8207 '<Eq at ast.py:19172>' Is_Equal_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19172
--# expr-start 8206 '<Symbol fixed_value at ???>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fixed_Value_321); 
--# expr-done 8206
Is_Equal_27 := Block_Var = Sym_27; 
--# expr-done 8207
if Is_Equal_27 then
   --# expr-start 8208 '<FieldAccess for fixed_value at ast.py:19172>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19172
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.fixed_value
Fld_30 := Libadalang.Implementation.Type_Attributes_Repository_F_Fixed_Value (Node => Block_Var_1);
--# end
--# expr-done 8208
   If_Result_12 := Fld_30;
else
--# expr-start 8210 '<Eq at ast.py:19173>' Is_Equal_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19173
--# expr-start 8209 '<Symbol integer_value at ???>' Sym_28 None
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Value_332); 
--# expr-done 8209
Is_Equal_28 := Block_Var = Sym_28; 
--# expr-done 8210
if Is_Equal_28 then
   --# expr-start 8211 '<FieldAccess for integer_value at ast.py:19173>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19173
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.integer_value
Fld_31 := Libadalang.Implementation.Type_Attributes_Repository_F_Integer_Value (Node => Block_Var_1);
--# end
--# expr-done 8211
   If_Result_11 := Fld_31;
else
--# expr-start 8213 '<Eq at ast.py:19175>' Is_Equal_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19175
--# expr-start 8212 '<Symbol pos at ???>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pos_355); 
--# expr-done 8212
Is_Equal_29 := Block_Var = Sym_29; 
--# expr-done 8213
if Is_Equal_29 then
   --# expr-start 8214 '<FieldAccess for pos at ast.py:19175>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19175
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.pos
Fld_32 := Libadalang.Implementation.Type_Attributes_Repository_F_Pos (Node => Block_Var_1);
--# end
--# expr-done 8214
   If_Result_10 := Fld_32;
else
--# expr-start 8216 '<Eq at ast.py:19176>' Is_Equal_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19176
--# expr-start 8215 '<Symbol val at ???>' Sym_30 None
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val_395); 
--# expr-done 8215
Is_Equal_30 := Block_Var = Sym_30; 
--# expr-done 8216
if Is_Equal_30 then
   --# expr-start 8217 '<FieldAccess for val_attr at ast.py:19176>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19176
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.val_attr
Fld_33 := Libadalang.Implementation.Type_Attributes_Repository_F_Val_Attr (Node => Block_Var_1);
--# end
--# expr-done 8217
   If_Result_9 := Fld_33;
else
--# expr-start 8219 '<Eq at ast.py:19177>' Is_Equal_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19177
--# expr-start 8218 '<Symbol enum_rep at ???>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Rep_316); 
--# expr-done 8218
Is_Equal_31 := Block_Var = Sym_31; 
--# expr-done 8219
if Is_Equal_31 then
   --# expr-start 8220 '<FieldAccess for enum_rep at ast.py:19177>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19177
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.enum_rep
Fld_34 := Libadalang.Implementation.Type_Attributes_Repository_F_Enum_Rep (Node => Block_Var_1);
--# end
--# expr-done 8220
   If_Result_8 := Fld_34;
else
--# expr-start 8222 '<Eq at ast.py:19178>' Is_Equal_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19178
--# expr-start 8221 '<Symbol enum_val at ???>' Sym_32 None
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Val_317); 
--# expr-done 8221
Is_Equal_32 := Block_Var = Sym_32; 
--# expr-done 8222
if Is_Equal_32 then
   --# expr-start 8223 '<FieldAccess for enum_val at ast.py:19178>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19178
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.enum_val
Fld_35 := Libadalang.Implementation.Type_Attributes_Repository_F_Enum_Val (Node => Block_Var_1);
--# end
--# expr-done 8223
   If_Result_7 := Fld_35;
else
--# expr-start 8225 '<Eq at ast.py:19180>' Is_Equal_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19180
--# expr-start 8224 '<Symbol read at ???>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read_364); 
--# expr-done 8224
Is_Equal_33 := Block_Var = Sym_33; 
--# expr-done 8225
if Is_Equal_33 then
   --# expr-start 8226 '<FieldAccess for read at ast.py:19180>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19180
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.read
Fld_36 := Libadalang.Implementation.Type_Attributes_Repository_F_Read (Node => Block_Var_1);
--# end
--# expr-done 8226
   If_Result_6 := Fld_36;
else
--# expr-start 8228 '<Eq at ast.py:19181>' Is_Equal_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19181
--# expr-start 8227 '<Symbol write at ???>' Sym_34 None
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write_406); 
--# expr-done 8227
Is_Equal_34 := Block_Var = Sym_34; 
--# expr-done 8228
if Is_Equal_34 then
   --# expr-start 8229 '<FieldAccess for write at ast.py:19181>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19181
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.write
Fld_37 := Libadalang.Implementation.Type_Attributes_Repository_F_Write (Node => Block_Var_1);
--# end
--# expr-done 8229
   If_Result_5 := Fld_37;
else
--# expr-start 8231 '<Eq at ast.py:19182>' Is_Equal_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19182
--# expr-start 8230 '<Symbol input at ???>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input_331); 
--# expr-done 8230
Is_Equal_35 := Block_Var = Sym_35; 
--# expr-done 8231
if Is_Equal_35 then
   --# expr-start 8232 '<FieldAccess for input at ast.py:19182>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19182
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.input
Fld_38 := Libadalang.Implementation.Type_Attributes_Repository_F_Input (Node => Block_Var_1);
--# end
--# expr-done 8232
   If_Result_4 := Fld_38;
else
--# expr-start 8234 '<Eq at ast.py:19183>' Is_Equal_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19183
--# expr-start 8233 '<Symbol output at ???>' Sym_36 None
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output_352); 
--# expr-done 8233
Is_Equal_36 := Block_Var = Sym_36; 
--# expr-done 8234
if Is_Equal_36 then
   --# expr-start 8235 '<FieldAccess for output at ast.py:19183>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19183
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.output
Fld_39 := Libadalang.Implementation.Type_Attributes_Repository_F_Output (Node => Block_Var_1);
--# end
--# expr-done 8235
   If_Result_3 := Fld_39;
else
--# expr-start 8237 '<Eq at ast.py:19185>' Is_Equal_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19185
--# expr-start 8236 '<Symbol asm_input at ???>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Input_297); 
--# expr-done 8236
Is_Equal_37 := Block_Var = Sym_37; 
--# expr-done 8237
if Is_Equal_37 then
   --# expr-start 8238 '<FieldAccess for asm_input at ast.py:19185>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19185
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.asm_input
Fld_40 := Libadalang.Implementation.Type_Attributes_Repository_F_Asm_Input (Node => Block_Var_1);
--# end
--# expr-done 8238
   If_Result_2 := Fld_40;
else
--# expr-start 8240 '<Eq at ast.py:19186>' Is_Equal_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19186
--# expr-start 8239 '<Symbol asm_output at ???>' Sym_38 None
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Output_298); 
--# expr-done 8239
Is_Equal_38 := Block_Var = Sym_38; 
--# expr-done 8240
if Is_Equal_38 then
   --# expr-start 8241 '<FieldAccess for asm_output at ast.py:19186>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19186
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.asm_output
Fld_41 := Libadalang.Implementation.Type_Attributes_Repository_F_Asm_Output (Node => Block_Var_1);
--# end
--# expr-done 8241
   If_Result_1 := Fld_41;
else
--# expr-start 8243 '<Eq at ast.py:19188>' Is_Equal_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19188
--# expr-start 8242 '<Symbol model at ???>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_344); 
--# expr-done 8242
Is_Equal_39 := Block_Var = Sym_39; 
--# expr-done 8243
if Is_Equal_39 then
   --# expr-start 8244 '<FieldAccess for model at ast.py:19188>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19188
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.model
Fld_42 := Libadalang.Implementation.Type_Attributes_Repository_F_Model (Node => Block_Var_1);
--# end
--# expr-done 8244
   If_Result := Fld_42;
else
   If_Result := No_Bare_Ada_Node;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
   If_Result_7 := If_Result_6;
end if;
   If_Result_8 := If_Result_7;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
end if;
   If_Result_11 := If_Result_10;
end if;
   If_Result_12 := If_Result_11;
end if;
   If_Result_13 := If_Result_12;
end if;
   If_Result_14 := If_Result_13;
end if;
   If_Result_15 := If_Result_14;
end if;
   If_Result_16 := If_Result_15;
end if;
   If_Result_17 := If_Result_16;
end if;
   If_Result_18 := If_Result_17;
end if;
   If_Result_19 := If_Result_18;
end if;
   If_Result_20 := If_Result_19;
end if;
   If_Result_21 := If_Result_20;
end if;
   If_Result_22 := If_Result_21;
end if;
   If_Result_23 := If_Result_22;
end if;
   If_Result_24 := If_Result_23;
end if;
   If_Result_25 := If_Result_24;
end if;
   If_Result_26 := If_Result_25;
end if;
   If_Result_27 := If_Result_26;
end if;
   If_Result_28 := If_Result_27;
end if;
   If_Result_29 := If_Result_28;
end if;
   If_Result_30 := If_Result_29;
end if;
   If_Result_31 := If_Result_30;
end if;
   If_Result_32 := If_Result_31;
end if;
   If_Result_33 := If_Result_32;
end if;
   If_Result_34 := If_Result_33;
end if;
   If_Result_35 := If_Result_34;
end if;
   If_Result_36 := If_Result_35;
end if;
   If_Result_37 := If_Result_36;
end if;
   If_Result_38 := If_Result_37;
end if;
   If_Result_39 := If_Result_38;
end if;
--# expr-done 8245
Block_Var_2 := If_Result_39; 
--# bind subp Block_Var_2
--# expr-start 8251 '<New BasicSubpDecl.entity at ???>' New_Struct_1 None
--# expr-start 8250 '<New EntityInfo at ast.py:19194>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19194
--# expr-start 8247 '<FieldAccess for from_rebound at ast.py:19197>' Fld_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19197
--# expr-start 8246 '<FieldAccess for info at ast.py:19197>' Fld_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19197
Fld_45 := Typ.Info;
--# expr-done 8246
Fld_46 := Fld_45.From_Rebound;
--# expr-done 8247
--# expr-start 8249 '<FieldAccess for rebindings at ast.py:19195>' Fld_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19195
--# expr-start 8248 '<FieldAccess for info at ast.py:19195>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19195
Fld_43 := Typ.Info;
--# expr-done 8248
Fld_44 := Fld_43.Rebindings;
--# expr-done 8249
New_Struct := (From_Rebound => Fld_46, Md => No_Metadata, Rebindings => Fld_44); 
--# expr-done 8250
New_Struct_1 := (Info => New_Struct, Node => Block_Var_2); 
--# expr-done 8251
Let_Result := New_Struct_1; 
--# end
--# expr-done 8252
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Synthesize_Attribute_Subprogram;
--# end
--# property-start AttributeRef.attribute_subprogram_for_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19201
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Attribute_Subprogram_For_Type
  (Node : Bare_Attribute_Ref
      ; Typ : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Symbol_Type;
Var_Expr : Internal_Entity_Attribute_Def_Clause;
Var_Expr_1 : Internal_Entity_Name;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Is_Null : Boolean;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Fld_4 : Internal_Entity_Attribute_Def_Clause;
Fld_5 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_6 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Basic_Subp_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Sym_5 : Symbol_Type;
Fld_8 : Internal_Aspect;
Fld_9 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity_Basic_Subp_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Result_Var_2 : Internal_Entity_Basic_Decl;
Fld_12 : Internal_Entity_Basic_Subp_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
If_Result_3 : Internal_Entity_Basic_Decl;
If_Result_4 : Internal_Entity_Basic_Decl;
If_Result_5 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8281 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8254 '<FieldAccess for name_symbol at ast.py:19207>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19207
--# expr-start 8253 '<FieldAccess for attribute at ast.py:19207>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19207
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8253
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8254
Block_Var := Fld_1; 
--# bind rel_name Block_Var
--# expr-start 8280 '<Cond at ast.py:19208>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19208
--# expr-start 8255 '<IsNull at ast.py:19209>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19209
Is_Null := Typ.Node = null; 
--# expr-done 8255
if Is_Null then
   If_Result_5 := No_Entity_Basic_Decl;
else
--# expr-start 8257 '<Eq at ast.py:19212>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19212
--# expr-start 8256 '<Symbol read at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read_364); 
--# expr-done 8256
Is_Equal := Block_Var = Sym; 
--# expr-done 8257
if Is_Equal then
   If_Result := True;
else
   --# expr-start 8259 '<Eq at ast.py:19212>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19212
--# expr-start 8258 '<Symbol write at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write_406); 
--# expr-done 8258
Is_Equal_1 := Block_Var = Sym_1; 
--# expr-done 8259
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 8261 '<Eq at ast.py:19212>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19212
--# expr-start 8260 '<Symbol input at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input_331); 
--# expr-done 8260
Is_Equal_2 := Block_Var = Sym_2; 
--# expr-done 8261
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 8263 '<Eq at ast.py:19212>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19212
--# expr-start 8262 '<Symbol output at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output_352); 
--# expr-done 8262
Is_Equal_3 := Block_Var = Sym_3; 
--# expr-done 8263
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
--# expr-start 8266 '<FieldAccess for get_representation_clause at ast.py:19213>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19213
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8265 '<FieldAccess for name_symbol at ast.py:19213>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19213
--# expr-start 8264 '<FieldAccess for attribute at ast.py:19213>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19213
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8264
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 8265
--# property-call-start BasicDecl.get_representation_clause
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Get_Representation_Clause (Node => Typ.Node, Name => Fld_3, Imprecise_Fallback => False, E_Info => Typ.Info);
--# end
--# expr-done 8266
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind x Var_Expr
--# expr-start 8269 '<FieldAccess for referenced_decl at ast.py:19214>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19214
--# expr-start 8268 '<Cast to Name at ast.py:19214>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19214
--# expr-start 8267 '<FieldAccess for expr at ast.py:19214>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19214
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Attribute_Def_Clause_F_Expr, Info => Var_Expr.Info);
--# expr-done 8267
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8268
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_6 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result.Node, Imprecise_Fallback => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 8269
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
--# expr-start 8270 '<FieldAccess for synthesize_attribute_subprogram at ast.py:19215>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19215
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.synthesize_attribute_subprogram
Fld_7 := Libadalang.Implementation.Attribute_Ref_P_Synthesize_Attribute_Subprogram (Node => Ent.Node, Typ => Typ, E_Info => Ent.Info);
--# end
--# expr-done 8270
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   Result_Var := Cast_Result_1;
end if;
   If_Result_4 := Result_Var;
else
--# expr-start 8272 '<Eq at ast.py:19218>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19218
--# expr-start 8271 '<Symbol put_image at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image_361); 
--# expr-done 8271
Is_Equal_4 := Block_Var = Sym_4; 
--# expr-done 8272
if Is_Equal_4 then
--# expr-start 8276 '<Cast to Name at ast.py:19219>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19219
Var_Expr_2 := Typ;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8275 '<FieldAccess for value at ast.py:19219>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19219
--# expr-start 8274 '<FieldAccess for get_aspect at ast.py:19219>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19219
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8273 '<Symbol put_image at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image_361); 
--# expr-done 8273
--# property-call-start BasicDecl.get_aspect
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Var_Expr_2.Node, Name => Sym_5, Imprecise_Fallback => False, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 8274
Fld_9 := Fld_8.Value;
--# expr-done 8275
Scope_Result_1 := Fld_9;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 8276
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_1
--# expr-start 8277 '<FieldAccess for referenced_decl at ast.py:19220>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19220
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_10 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Var_Expr_1.Node, Imprecise_Fallback => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 8277
Scope_Result_2 := Fld_10;
   --# end
   Result_Var_2 := Scope_Result_2;
else
--# expr-start 8278 '<FieldAccess for synthesize_attribute_subprogram at ast.py:19221>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19221
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.synthesize_attribute_subprogram
Fld_11 := Libadalang.Implementation.Attribute_Ref_P_Synthesize_Attribute_Subprogram (Node => Ent.Node, Typ => Typ, E_Info => Ent.Info);
--# end
--# expr-done 8278
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   Result_Var_2 := Cast_Result_3;
end if;
   If_Result_3 := Result_Var_2;
else
--# expr-start 8279 '<FieldAccess for synthesize_attribute_subprogram at ast.py:19224>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19224
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.synthesize_attribute_subprogram
Fld_12 := Libadalang.Implementation.Attribute_Ref_P_Synthesize_Attribute_Subprogram (Node => Ent.Node, Typ => Typ, E_Info => Ent.Info);
--# end
--# expr-done 8279
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   If_Result_3 := Cast_Result_4;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
--# expr-done 8280
Let_Result := If_Result_5; 
--# end
--# expr-done 8281
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Attribute_Subprogram_For_Type;
--# end
--# property-start AttributeRef.attribute_subprogram /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19227
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Attribute_Subprogram
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8285 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8284 '<FieldAccess for attribute_subprogram_for_type at ast.py:19233>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19233
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8283 '<FieldAccess for name_designated_type at ast.py:19234>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19234
--# expr-start 8282 '<FieldAccess for prefix at ast.py:19234>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19234
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8282
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8283
--# property-call-start AttributeRef.attribute_subprogram_for_type
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram_For_Type (Node => Ent.Node, Typ => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 8284
Let_Result := Fld_2; 
--# end
--# expr-done 8285
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Attribute_Subprogram;
--# end
--# property-start AttributeRef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19237
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Xref_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1344 with Inline_Always;
      Block_Var : Symbol_Type;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_2 : Logic_Equation;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_3 : Boolean;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_4 : Boolean;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_5 : Boolean;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_6 : Boolean;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_7 : Boolean;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_8 : Boolean;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_9 : Boolean;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_10 : Boolean;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_11 : Boolean;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_12 : Boolean;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_13 : Boolean;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_14 : Boolean;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_15 : Boolean;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_16 : Boolean;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_17 : Boolean;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_18 : Boolean;
Sym_21 : Symbol_Type;
Is_Equal_21 : Boolean;
If_Result_19 : Boolean;
Sym_22 : Symbol_Type;
Is_Equal_22 : Boolean;
If_Result_20 : Boolean;
Sym_23 : Symbol_Type;
Is_Equal_23 : Boolean;
If_Result_21 : Boolean;
Sym_24 : Symbol_Type;
Is_Equal_24 : Boolean;
If_Result_22 : Boolean;
Sym_25 : Symbol_Type;
Is_Equal_25 : Boolean;
If_Result_23 : Boolean;
Sym_26 : Symbol_Type;
Is_Equal_26 : Boolean;
If_Result_24 : Boolean;
Sym_27 : Symbol_Type;
Is_Equal_27 : Boolean;
If_Result_25 : Boolean;
Sym_28 : Symbol_Type;
Is_Equal_28 : Boolean;
If_Result_26 : Boolean;
Sym_29 : Symbol_Type;
Is_Equal_29 : Boolean;
If_Result_27 : Boolean;
Sym_30 : Symbol_Type;
Is_Equal_30 : Boolean;
If_Result_28 : Boolean;
Sym_31 : Symbol_Type;
Is_Equal_31 : Boolean;
If_Result_29 : Boolean;
Sym_32 : Symbol_Type;
Is_Equal_32 : Boolean;
If_Result_30 : Boolean;
Sym_33 : Symbol_Type;
Is_Equal_33 : Boolean;
If_Result_31 : Boolean;
Sym_34 : Symbol_Type;
Is_Equal_34 : Boolean;
If_Result_32 : Boolean;
Sym_35 : Symbol_Type;
Is_Equal_35 : Boolean;
If_Result_33 : Boolean;
Sym_36 : Symbol_Type;
Is_Equal_36 : Boolean;
If_Result_34 : Boolean;
Sym_37 : Symbol_Type;
Is_Equal_37 : Boolean;
If_Result_35 : Boolean;
Sym_38 : Symbol_Type;
Is_Equal_38 : Boolean;
If_Result_36 : Boolean;
Sym_39 : Symbol_Type;
Is_Equal_39 : Boolean;
If_Result_37 : Boolean;
Fld_3 : Logic_Equation;
Sym_40 : Symbol_Type;
Is_Equal_40 : Boolean;
Sym_41 : Symbol_Type;
Is_Equal_41 : Boolean;
If_Result_38 : Boolean;
Fld_4 : Logic_Equation;
Sym_42 : Symbol_Type;
Is_Equal_42 : Boolean;
Sym_43 : Symbol_Type;
Is_Equal_43 : Boolean;
If_Result_39 : Boolean;
Sym_44 : Symbol_Type;
Is_Equal_44 : Boolean;
If_Result_40 : Boolean;
Sym_45 : Symbol_Type;
Is_Equal_45 : Boolean;
If_Result_41 : Boolean;
Sym_46 : Symbol_Type;
Is_Equal_46 : Boolean;
If_Result_42 : Boolean;
Fld_5 : Logic_Equation;
Sym_47 : Symbol_Type;
Is_Equal_47 : Boolean;
Sym_48 : Symbol_Type;
Is_Equal_48 : Boolean;
If_Result_43 : Boolean;
Sym_49 : Symbol_Type;
Is_Equal_49 : Boolean;
If_Result_44 : Boolean;
Fld_6 : Logic_Equation;
Sym_50 : Symbol_Type;
Is_Equal_50 : Boolean;
Sym_51 : Symbol_Type;
Fld_7 : Internal_Entity;
Fld_8 : Logic_Equation;
Sym_52 : Symbol_Type;
Is_Equal_51 : Boolean;
Sym_53 : Symbol_Type;
Fld_9 : Internal_Entity;
Fld_10 : Logic_Equation;
Sym_54 : Symbol_Type;
Is_Equal_52 : Boolean;
Sym_55 : Symbol_Type;
Fld_11 : Internal_Entity;
Fld_12 : Logic_Equation;
Sym_56 : Symbol_Type;
Is_Equal_53 : Boolean;
Fld_13 : Logic_Equation;
Sym_57 : Symbol_Type;
Is_Equal_54 : Boolean;
Sym_58 : Symbol_Type;
Is_Equal_55 : Boolean;
If_Result_45 : Boolean;
Sym_59 : Symbol_Type;
Is_Equal_56 : Boolean;
If_Result_46 : Boolean;
Fld_14 : Logic_Equation;
Sym_60 : Symbol_Type;
Is_Equal_57 : Boolean;
Fld_15 : Logic_Equation;
Sym_61 : Symbol_Type;
Is_Equal_58 : Boolean;
Fld_16 : Logic_Equation;
Sym_62 : Symbol_Type;
Is_Equal_59 : Boolean;
Sym_63 : Symbol_Type;
Is_Equal_60 : Boolean;
If_Result_47 : Boolean;
Sym_64 : Symbol_Type;
Is_Equal_61 : Boolean;
If_Result_48 : Boolean;
Sym_65 : Symbol_Type;
Is_Equal_62 : Boolean;
If_Result_49 : Boolean;
Sym_66 : Symbol_Type;
Is_Equal_63 : Boolean;
If_Result_50 : Boolean;
Sym_67 : Symbol_Type;
Is_Equal_64 : Boolean;
If_Result_51 : Boolean;
Sym_68 : Symbol_Type;
Is_Equal_65 : Boolean;
If_Result_52 : Boolean;
Sym_69 : Symbol_Type;
Is_Equal_66 : Boolean;
If_Result_53 : Boolean;
Sym_70 : Symbol_Type;
Is_Equal_67 : Boolean;
If_Result_54 : Boolean;
Sym_71 : Symbol_Type;
Is_Equal_68 : Boolean;
If_Result_55 : Boolean;
Fld_17 : Logic_Equation;
Sym_72 : Symbol_Type;
Is_Equal_69 : Boolean;
Sym_73 : Symbol_Type;
Fld_18 : Internal_Entity;
Fld_19 : Logic_Equation;
Sym_74 : Symbol_Type;
Is_Equal_70 : Boolean;
Fld_20 : Logic_Equation;
Sym_75 : Symbol_Type;
Is_Equal_71 : Boolean;
Fld_21 : Logic_Equation;
Sym_76 : Symbol_Type;
Is_Equal_72 : Boolean;
Sym_77 : Symbol_Type;
Is_Equal_73 : Boolean;
If_Result_56 : Boolean;
Fld_22 : Logic_Equation;
Sym_78 : Symbol_Type;
Is_Equal_74 : Boolean;
Sym_79 : Symbol_Type;
Is_Equal_75 : Boolean;
If_Result_57 : Boolean;
Fld_23 : Internal_Entity_Name;
Fld_24 : Logic_Equation;
Sym_80 : Symbol_Type;
Is_Equal_76 : Boolean;
Sym_81 : Symbol_Type;
Is_Equal_77 : Boolean;
If_Result_58 : Boolean;
Sym_82 : Symbol_Type;
Is_Equal_78 : Boolean;
If_Result_59 : Boolean;
Sym_83 : Symbol_Type;
Is_Equal_79 : Boolean;
If_Result_60 : Boolean;
Sym_84 : Symbol_Type;
Is_Equal_80 : Boolean;
If_Result_61 : Boolean;
Sym_85 : Symbol_Type;
Is_Equal_81 : Boolean;
If_Result_62 : Boolean;
Sym_86 : Symbol_Type;
Is_Equal_82 : Boolean;
If_Result_63 : Boolean;
Sym_87 : Symbol_Type;
Is_Equal_83 : Boolean;
If_Result_64 : Boolean;
Sym_88 : Symbol_Type;
Is_Equal_84 : Boolean;
If_Result_65 : Boolean;
Sym_89 : Symbol_Type;
Is_Equal_85 : Boolean;
If_Result_66 : Boolean;
Sym_90 : Symbol_Type;
Is_Equal_86 : Boolean;
If_Result_67 : Boolean;
Sym_91 : Symbol_Type;
Is_Equal_87 : Boolean;
If_Result_68 : Boolean;
Sym_92 : Symbol_Type;
Is_Equal_88 : Boolean;
If_Result_69 : Boolean;
Sym_93 : Symbol_Type;
Is_Equal_89 : Boolean;
If_Result_70 : Boolean;
Fld_25 : Internal_Entity_Name;
Fld_26 : Logic_Equation;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Sym_94 : Symbol_Type;
Is_Equal_90 : Boolean;
Fld_29 : Logic_Var;
Fld_30 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Sym_95 : Symbol_Type;
Is_Equal_91 : Boolean;
Sym_96 : Symbol_Type;
Is_Equal_92 : Boolean;
If_Result_71 : Boolean;
Sym_97 : Symbol_Type;
Is_Equal_93 : Boolean;
If_Result_72 : Boolean;
Sym_98 : Symbol_Type;
Is_Equal_94 : Boolean;
If_Result_73 : Boolean;
Sym_99 : Symbol_Type;
Is_Equal_95 : Boolean;
If_Result_74 : Boolean;
Sym_100 : Symbol_Type;
Is_Equal_96 : Boolean;
If_Result_75 : Boolean;
Sym_101 : Symbol_Type;
Is_Equal_97 : Boolean;
If_Result_76 : Boolean;
Sym_102 : Symbol_Type;
Is_Equal_98 : Boolean;
If_Result_77 : Boolean;
Sym_103 : Symbol_Type;
Is_Equal_99 : Boolean;
If_Result_78 : Boolean;
Sym_104 : Symbol_Type;
Is_Equal_100 : Boolean;
If_Result_79 : Boolean;
Sym_105 : Symbol_Type;
Is_Equal_101 : Boolean;
If_Result_80 : Boolean;
Sym_106 : Symbol_Type;
Is_Equal_102 : Boolean;
If_Result_81 : Boolean;
Sym_107 : Symbol_Type;
Is_Equal_103 : Boolean;
If_Result_82 : Boolean;
Sym_108 : Symbol_Type;
Is_Equal_104 : Boolean;
If_Result_83 : Boolean;
Sym_109 : Symbol_Type;
Is_Equal_105 : Boolean;
If_Result_84 : Boolean;
Sym_110 : Symbol_Type;
Is_Equal_106 : Boolean;
If_Result_85 : Boolean;
Sym_111 : Symbol_Type;
Is_Equal_107 : Boolean;
If_Result_86 : Boolean;
Sym_112 : Symbol_Type;
Is_Equal_108 : Boolean;
If_Result_87 : Boolean;
Sym_113 : Symbol_Type;
Is_Equal_109 : Boolean;
If_Result_88 : Boolean;
Sym_114 : Symbol_Type;
Is_Equal_110 : Boolean;
If_Result_89 : Boolean;
Sym_115 : Symbol_Type;
Is_Equal_111 : Boolean;
If_Result_90 : Boolean;
Sym_116 : Symbol_Type;
Is_Equal_112 : Boolean;
If_Result_91 : Boolean;
Sym_117 : Symbol_Type;
Is_Equal_113 : Boolean;
If_Result_92 : Boolean;
Sym_118 : Symbol_Type;
Is_Equal_114 : Boolean;
If_Result_93 : Boolean;
Sym_119 : Symbol_Type;
Is_Equal_115 : Boolean;
If_Result_94 : Boolean;
Sym_120 : Symbol_Type;
Is_Equal_116 : Boolean;
If_Result_95 : Boolean;
Sym_121 : Symbol_Type;
Is_Equal_117 : Boolean;
If_Result_96 : Boolean;
Sym_122 : Symbol_Type;
Is_Equal_118 : Boolean;
If_Result_97 : Boolean;
Sym_123 : Symbol_Type;
Is_Equal_119 : Boolean;
If_Result_98 : Boolean;
Sym_124 : Symbol_Type;
Is_Equal_120 : Boolean;
If_Result_99 : Boolean;
Fld_31 : Internal_Entity_Name;
Fld_32 : Logic_Equation;
Fld_33 : Logic_Var;
Fld_34 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Sym_125 : Symbol_Type;
Is_Equal_121 : Boolean;
Sym_126 : Symbol_Type;
Is_Equal_122 : Boolean;
If_Result_100 : Boolean;
Fld_35 : Internal_Entity_Name;
Fld_36 : Logic_Equation;
Fld_37 : Logic_Var;
Sym_127 : Symbol_Type;
Fld_38 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Sym_128 : Symbol_Type;
Is_Equal_123 : Boolean;
Fld_39 : Logic_Var;
Sym_129 : Symbol_Type;
Fld_40 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
Sym_130 : Symbol_Type;
Is_Equal_124 : Boolean;
Fld_41 : Logic_Equation;
Sym_131 : Symbol_Type;
Is_Equal_125 : Boolean;
Sym_132 : Symbol_Type;
Is_Equal_126 : Boolean;
If_Result_101 : Boolean;
Sym_133 : Symbol_Type;
Is_Equal_127 : Boolean;
If_Result_102 : Boolean;
Fld_42 : Logic_Equation;
Sym_134 : Symbol_Type;
Is_Equal_128 : Boolean;
Fld_43 : Logic_Equation;
Sym_135 : Symbol_Type;
Is_Equal_129 : Boolean;
Sym_136 : Symbol_Type;
Is_Equal_130 : Boolean;
If_Result_103 : Boolean;
Fld_44 : Internal_Entity_Name;
Fld_45 : Logic_Equation;
Fld_46 : Logic_Var;
Fld_47 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Sym_137 : Symbol_Type;
Is_Equal_131 : Boolean;
Fld_48 : Internal_Entity_Name;
Fld_49 : Logic_Equation;
Fld_50 : Logic_Var;
Fld_51 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Sym_138 : Symbol_Type;
Is_Equal_132 : Boolean;
Fld_52 : Internal_Entity_Name;
Fld_53 : Logic_Equation;
Fld_54 : Logic_Var;
Fld_55 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Sym_139 : Symbol_Type;
Is_Equal_133 : Boolean;
Fld_56 : Logic_Equation;
Sym_140 : Symbol_Type;
Is_Equal_134 : Boolean;
Fld_57 : Logic_Equation;
Sym_141 : Symbol_Type;
Is_Equal_135 : Boolean;
Fld_58 : Logic_Equation;
Sym_142 : Symbol_Type;
Is_Equal_136 : Boolean;
Fld_59 : Logic_Var;
Sym_143 : Symbol_Type;
Fld_60 : Internal_Entity;
Bind_Result_6 : Logic_Equation;
Fld_61 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_7 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Sym_144 : Symbol_Type;
Is_Equal_137 : Boolean;
Sym_145 : Symbol_Type;
Is_Equal_138 : Boolean;
If_Result_104 : Boolean;
Fld_62 : Logic_Equation;
If_Result_105 : Logic_Equation;
If_Result_106 : Logic_Equation;
If_Result_107 : Logic_Equation;
If_Result_108 : Logic_Equation;
If_Result_109 : Logic_Equation;
If_Result_110 : Logic_Equation;
If_Result_111 : Logic_Equation;
If_Result_112 : Logic_Equation;
If_Result_113 : Logic_Equation;
If_Result_114 : Logic_Equation;
If_Result_115 : Logic_Equation;
If_Result_116 : Logic_Equation;
If_Result_117 : Logic_Equation;
If_Result_118 : Logic_Equation;
If_Result_119 : Logic_Equation;
If_Result_120 : Logic_Equation;
If_Result_121 : Logic_Equation;
If_Result_122 : Logic_Equation;
If_Result_123 : Logic_Equation;
If_Result_124 : Logic_Equation;
If_Result_125 : Logic_Equation;
If_Result_126 : Logic_Equation;
If_Result_127 : Logic_Equation;
If_Result_128 : Logic_Equation;
If_Result_129 : Logic_Equation;
If_Result_130 : Logic_Equation;
If_Result_131 : Logic_Equation;
If_Result_132 : Logic_Equation;
If_Result_133 : Logic_Equation;
If_Result_134 : Logic_Equation;
If_Result_135 : Logic_Equation;
If_Result_136 : Logic_Equation;
If_Result_137 : Logic_Equation;
If_Result_138 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1344 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_24);
                     Dec_Ref (Fld_26);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Fld_34);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_36);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_41);
                     Dec_Ref (Fld_42);
                     Dec_Ref (Fld_43);
                     Dec_Ref (Fld_45);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_49);
                     Dec_Ref (Fld_51);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_53);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_56);
                     Dec_Ref (Fld_57);
                     Dec_Ref (Fld_58);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_62);
                     Dec_Ref (If_Result_105);
                     Dec_Ref (If_Result_106);
                     Dec_Ref (If_Result_107);
                     Dec_Ref (If_Result_108);
                     Dec_Ref (If_Result_109);
                     Dec_Ref (If_Result_110);
                     Dec_Ref (If_Result_111);
                     Dec_Ref (If_Result_112);
                     Dec_Ref (If_Result_113);
                     Dec_Ref (If_Result_114);
                     Dec_Ref (If_Result_115);
                     Dec_Ref (If_Result_116);
                     Dec_Ref (If_Result_117);
                     Dec_Ref (If_Result_118);
                     Dec_Ref (If_Result_119);
                     Dec_Ref (If_Result_120);
                     Dec_Ref (If_Result_121);
                     Dec_Ref (If_Result_122);
                     Dec_Ref (If_Result_123);
                     Dec_Ref (If_Result_124);
                     Dec_Ref (If_Result_125);
                     Dec_Ref (If_Result_126);
                     Dec_Ref (If_Result_127);
                     Dec_Ref (If_Result_128);
                     Dec_Ref (If_Result_129);
                     Dec_Ref (If_Result_130);
                     Dec_Ref (If_Result_131);
                     Dec_Ref (If_Result_132);
                     Dec_Ref (If_Result_133);
                     Dec_Ref (If_Result_134);
                     Dec_Ref (If_Result_135);
                     Dec_Ref (If_Result_136);
                     Dec_Ref (If_Result_137);
                     Dec_Ref (If_Result_138);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1344;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8650 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8287 '<FieldAccess for name_symbol at ast.py:19239>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19239
--# expr-start 8286 '<FieldAccess for attribute at ast.py:19239>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19239
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8286
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8287
Block_Var := Fld_1; 
--# bind rel_name Block_Var
--# expr-start 8649 '<Cond at ast.py:19240>' If_Result_138 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19240
--# expr-start 8289 '<Eq at ast.py:19242>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19242
--# expr-start 8288 '<Symbol First at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 8288
Is_Equal := Block_Var = Sym; 
--# expr-done 8289
if Is_Equal then
   If_Result := True;
else
   --# expr-start 8291 '<Eq at ast.py:19242>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19242
--# expr-start 8290 '<Symbol Last at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 8290
Is_Equal_1 := Block_Var = Sym_1; 
--# expr-done 8291
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 8293 '<Eq at ast.py:19242>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19242
--# expr-start 8292 '<Symbol Range at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 8292
Is_Equal_2 := Block_Var = Sym_2; 
--# expr-done 8293
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 8295 '<Eq at ast.py:19242>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19242
--# expr-start 8294 '<Symbol Length at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 8294
Is_Equal_3 := Block_Var = Sym_3; 
--# expr-done 8295
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   --# expr-start 8296 '<FieldAccess for array_attr_equation at ast.py:19243>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19243
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.array_attr_equation
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Array_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8296
   If_Result_138 := Fld_2;
else
--# expr-start 8298 '<Eq at ast.py:19246>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8297 '<Symbol Succ at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Succ); 
--# expr-done 8297
Is_Equal_4 := Block_Var = Sym_4; 
--# expr-done 8298
if Is_Equal_4 then
   If_Result_3 := True;
else
   --# expr-start 8300 '<Eq at ast.py:19246>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8299 '<Symbol Pred at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pred); 
--# expr-done 8299
Is_Equal_5 := Block_Var = Sym_5; 
--# expr-done 8300
   If_Result_3 := Is_Equal_5;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 8302 '<Eq at ast.py:19246>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8301 '<Symbol Min at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Min); 
--# expr-done 8301
Is_Equal_6 := Block_Var = Sym_6; 
--# expr-done 8302
   If_Result_4 := Is_Equal_6;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 8304 '<Eq at ast.py:19246>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8303 '<Symbol Max at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max); 
--# expr-done 8303
Is_Equal_7 := Block_Var = Sym_7; 
--# expr-done 8304
   If_Result_5 := Is_Equal_7;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 8306 '<Eq at ast.py:19246>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8305 '<Symbol Ceiling at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ceiling); 
--# expr-done 8305
Is_Equal_8 := Block_Var = Sym_8; 
--# expr-done 8306
   If_Result_6 := Is_Equal_8;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 8308 '<Eq at ast.py:19246>' Is_Equal_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8307 '<Symbol Floor at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Floor); 
--# expr-done 8307
Is_Equal_9 := Block_Var = Sym_9; 
--# expr-done 8308
   If_Result_7 := Is_Equal_9;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 8310 '<Eq at ast.py:19246>' Is_Equal_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8309 '<Symbol Rounding at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rounding); 
--# expr-done 8309
Is_Equal_10 := Block_Var = Sym_10; 
--# expr-done 8310
   If_Result_8 := Is_Equal_10;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 8312 '<Eq at ast.py:19246>' Is_Equal_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8311 '<Symbol Unbiased_Rounding at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unbiased_Rounding); 
--# expr-done 8311
Is_Equal_11 := Block_Var = Sym_11; 
--# expr-done 8312
   If_Result_9 := Is_Equal_11;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 8314 '<Eq at ast.py:19246>' Is_Equal_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8313 '<Symbol Leading_Part at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Leading_Part); 
--# expr-done 8313
Is_Equal_12 := Block_Var = Sym_12; 
--# expr-done 8314
   If_Result_10 := Is_Equal_12;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 8316 '<Eq at ast.py:19246>' Is_Equal_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8315 '<Symbol Truncation at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Truncation); 
--# expr-done 8315
Is_Equal_13 := Block_Var = Sym_13; 
--# expr-done 8316
   If_Result_11 := Is_Equal_13;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 8318 '<Eq at ast.py:19246>' Is_Equal_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8317 '<Symbol Exponent at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exponent); 
--# expr-done 8317
Is_Equal_14 := Block_Var = Sym_14; 
--# expr-done 8318
   If_Result_12 := Is_Equal_14;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 8320 '<Eq at ast.py:19246>' Is_Equal_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8319 '<Symbol Fraction at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fraction); 
--# expr-done 8319
Is_Equal_15 := Block_Var = Sym_15; 
--# expr-done 8320
   If_Result_13 := Is_Equal_15;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 8322 '<Eq at ast.py:19246>' Is_Equal_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8321 '<Symbol Copy_Sign at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Copy_Sign); 
--# expr-done 8321
Is_Equal_16 := Block_Var = Sym_16; 
--# expr-done 8322
   If_Result_14 := Is_Equal_16;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 8324 '<Eq at ast.py:19246>' Is_Equal_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8323 '<Symbol Remainder at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remainder); 
--# expr-done 8323
Is_Equal_17 := Block_Var = Sym_17; 
--# expr-done 8324
   If_Result_15 := Is_Equal_17;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 8326 '<Eq at ast.py:19246>' Is_Equal_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8325 '<Symbol Adjacent at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Adjacent); 
--# expr-done 8325
Is_Equal_18 := Block_Var = Sym_18; 
--# expr-done 8326
   If_Result_16 := Is_Equal_18;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 8328 '<Eq at ast.py:19246>' Is_Equal_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8327 '<Symbol Machine at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine); 
--# expr-done 8327
Is_Equal_19 := Block_Var = Sym_19; 
--# expr-done 8328
   If_Result_17 := Is_Equal_19;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 8330 '<Eq at ast.py:19246>' Is_Equal_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8329 '<Symbol Machine_Rounding at ???>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Rounding); 
--# expr-done 8329
Is_Equal_20 := Block_Var = Sym_20; 
--# expr-done 8330
   If_Result_18 := Is_Equal_20;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 8332 '<Eq at ast.py:19246>' Is_Equal_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8331 '<Symbol Scaling at ???>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scaling); 
--# expr-done 8331
Is_Equal_21 := Block_Var = Sym_21; 
--# expr-done 8332
   If_Result_19 := Is_Equal_21;
end if;
if If_Result_19 then
   If_Result_20 := True;
else
   --# expr-start 8334 '<Eq at ast.py:19246>' Is_Equal_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8333 '<Symbol Compose at ???>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compose); 
--# expr-done 8333
Is_Equal_22 := Block_Var = Sym_22; 
--# expr-done 8334
   If_Result_20 := Is_Equal_22;
end if;
if If_Result_20 then
   If_Result_21 := True;
else
   --# expr-start 8336 '<Eq at ast.py:19246>' Is_Equal_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8335 '<Symbol Mod at ???>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod_153); 
--# expr-done 8335
Is_Equal_23 := Block_Var = Sym_23; 
--# expr-done 8336
   If_Result_21 := Is_Equal_23;
end if;
if If_Result_21 then
   If_Result_22 := True;
else
   --# expr-start 8338 '<Eq at ast.py:19246>' Is_Equal_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8337 '<Symbol Value at ???>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 8337
Is_Equal_24 := Block_Var = Sym_24; 
--# expr-done 8338
   If_Result_22 := Is_Equal_24;
end if;
if If_Result_22 then
   If_Result_23 := True;
else
   --# expr-start 8340 '<Eq at ast.py:19246>' Is_Equal_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8339 '<Symbol Wide_Value at ???>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Value); 
--# expr-done 8339
Is_Equal_25 := Block_Var = Sym_25; 
--# expr-done 8340
   If_Result_23 := Is_Equal_25;
end if;
if If_Result_23 then
   If_Result_24 := True;
else
   --# expr-start 8342 '<Eq at ast.py:19246>' Is_Equal_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8341 '<Symbol Wide_Wide_Value at ???>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Value); 
--# expr-done 8341
Is_Equal_26 := Block_Var = Sym_26; 
--# expr-done 8342
   If_Result_24 := Is_Equal_26;
end if;
if If_Result_24 then
   If_Result_25 := True;
else
   --# expr-start 8344 '<Eq at ast.py:19246>' Is_Equal_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8343 '<Symbol Fixed_Value at ???>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fixed_Value); 
--# expr-done 8343
Is_Equal_27 := Block_Var = Sym_27; 
--# expr-done 8344
   If_Result_25 := Is_Equal_27;
end if;
if If_Result_25 then
   If_Result_26 := True;
else
   --# expr-start 8346 '<Eq at ast.py:19246>' Is_Equal_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8345 '<Symbol Integer_Value at ???>' Sym_28 None
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Value); 
--# expr-done 8345
Is_Equal_28 := Block_Var = Sym_28; 
--# expr-done 8346
   If_Result_26 := Is_Equal_28;
end if;
if If_Result_26 then
   If_Result_27 := True;
else
   --# expr-start 8348 '<Eq at ast.py:19246>' Is_Equal_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8347 '<Symbol Pos at ???>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pos); 
--# expr-done 8347
Is_Equal_29 := Block_Var = Sym_29; 
--# expr-done 8348
   If_Result_27 := Is_Equal_29;
end if;
if If_Result_27 then
   If_Result_28 := True;
else
   --# expr-start 8350 '<Eq at ast.py:19246>' Is_Equal_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8349 '<Symbol Val at ???>' Sym_30 None
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 8349
Is_Equal_30 := Block_Var = Sym_30; 
--# expr-done 8350
   If_Result_28 := Is_Equal_30;
end if;
if If_Result_28 then
   If_Result_29 := True;
else
   --# expr-start 8352 '<Eq at ast.py:19246>' Is_Equal_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8351 '<Symbol Enum_Val at ???>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Val); 
--# expr-done 8351
Is_Equal_31 := Block_Var = Sym_31; 
--# expr-done 8352
   If_Result_29 := Is_Equal_31;
end if;
if If_Result_29 then
   If_Result_30 := True;
else
   --# expr-start 8354 '<Eq at ast.py:19246>' Is_Equal_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8353 '<Symbol Write at ???>' Sym_32 None
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write); 
--# expr-done 8353
Is_Equal_32 := Block_Var = Sym_32; 
--# expr-done 8354
   If_Result_30 := Is_Equal_32;
end if;
if If_Result_30 then
   If_Result_31 := True;
else
   --# expr-start 8356 '<Eq at ast.py:19246>' Is_Equal_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8355 '<Symbol Read at ???>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read); 
--# expr-done 8355
Is_Equal_33 := Block_Var = Sym_33; 
--# expr-done 8356
   If_Result_31 := Is_Equal_33;
end if;
if If_Result_31 then
   If_Result_32 := True;
else
   --# expr-start 8358 '<Eq at ast.py:19246>' Is_Equal_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8357 '<Symbol Output at ???>' Sym_34 None
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output); 
--# expr-done 8357
Is_Equal_34 := Block_Var = Sym_34; 
--# expr-done 8358
   If_Result_32 := Is_Equal_34;
end if;
if If_Result_32 then
   If_Result_33 := True;
else
   --# expr-start 8360 '<Eq at ast.py:19246>' Is_Equal_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8359 '<Symbol Input at ???>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input); 
--# expr-done 8359
Is_Equal_35 := Block_Var = Sym_35; 
--# expr-done 8360
   If_Result_33 := Is_Equal_35;
end if;
if If_Result_33 then
   If_Result_34 := True;
else
   --# expr-start 8362 '<Eq at ast.py:19246>' Is_Equal_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8361 '<Symbol Put_Image at ???>' Sym_36 None
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image); 
--# expr-done 8361
Is_Equal_36 := Block_Var = Sym_36; 
--# expr-done 8362
   If_Result_34 := Is_Equal_36;
end if;
if If_Result_34 then
   If_Result_35 := True;
else
   --# expr-start 8364 '<Eq at ast.py:19246>' Is_Equal_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8363 '<Symbol Asm_Input at ???>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Input); 
--# expr-done 8363
Is_Equal_37 := Block_Var = Sym_37; 
--# expr-done 8364
   If_Result_35 := Is_Equal_37;
end if;
if If_Result_35 then
   If_Result_36 := True;
else
   --# expr-start 8366 '<Eq at ast.py:19246>' Is_Equal_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8365 '<Symbol Asm_Output at ???>' Sym_38 None
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Output); 
--# expr-done 8365
Is_Equal_38 := Block_Var = Sym_38; 
--# expr-done 8366
   If_Result_36 := Is_Equal_38;
end if;
if If_Result_36 then
   If_Result_37 := True;
else
   --# expr-start 8368 '<Eq at ast.py:19246>' Is_Equal_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19246
--# expr-start 8367 '<Symbol Model at ???>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model); 
--# expr-done 8367
Is_Equal_39 := Block_Var = Sym_39; 
--# expr-done 8368
   If_Result_37 := Is_Equal_39;
end if;
if If_Result_37 then
   --# expr-start 8369 '<FieldAccess for attribute_subprogram_equation at ast.py:19256>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19256
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram_equation
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8369
   If_Result_137 := Fld_3;
else
--# expr-start 8371 '<Eq at ast.py:19258>' Is_Equal_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19258
--# expr-start 8370 '<Symbol Size at ???>' Sym_40 None
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Size); 
--# expr-done 8370
Is_Equal_40 := Block_Var = Sym_40; 
--# expr-done 8371
if Is_Equal_40 then
   If_Result_38 := True;
else
   --# expr-start 8373 '<Eq at ast.py:19258>' Is_Equal_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19258
--# expr-start 8372 '<Symbol VADS_Size at ???>' Sym_41 None
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Vads_Size); 
--# expr-done 8372
Is_Equal_41 := Block_Var = Sym_41; 
--# expr-done 8373
   If_Result_38 := Is_Equal_41;
end if;
if If_Result_38 then
   --# expr-start 8374 '<FieldAccess for size_equation at ast.py:19258>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19258
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.size_equation
Fld_4 := Libadalang.Implementation.Attribute_Ref_P_Size_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8374
   If_Result_136 := Fld_4;
else
--# expr-start 8376 '<Eq at ast.py:19260>' Is_Equal_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19260
--# expr-start 8375 '<Symbol Max_Size_In_Storage_Elements at ???>' Sym_42 None
Sym_42 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max_Size_In_Storage_Elements); 
--# expr-done 8375
Is_Equal_42 := Block_Var = Sym_42; 
--# expr-done 8376
if Is_Equal_42 then
   If_Result_39 := True;
else
   --# expr-start 8378 '<Eq at ast.py:19260>' Is_Equal_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19260
--# expr-start 8377 '<Symbol Aft at ???>' Sym_43 None
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aft); 
--# expr-done 8377
Is_Equal_43 := Block_Var = Sym_43; 
--# expr-done 8378
   If_Result_39 := Is_Equal_43;
end if;
if If_Result_39 then
   If_Result_40 := True;
else
   --# expr-start 8380 '<Eq at ast.py:19260>' Is_Equal_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19260
--# expr-start 8379 '<Symbol Object_Size at ???>' Sym_44 None
Sym_44 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Object_Size); 
--# expr-done 8379
Is_Equal_44 := Block_Var = Sym_44; 
--# expr-done 8380
   If_Result_40 := Is_Equal_44;
end if;
if If_Result_40 then
   If_Result_41 := True;
else
   --# expr-start 8382 '<Eq at ast.py:19260>' Is_Equal_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19260
--# expr-start 8381 '<Symbol Value_Size at ???>' Sym_45 None
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value_Size); 
--# expr-done 8381
Is_Equal_45 := Block_Var = Sym_45; 
--# expr-done 8382
   If_Result_41 := Is_Equal_45;
end if;
if If_Result_41 then
   If_Result_42 := True;
else
   --# expr-start 8384 '<Eq at ast.py:19260>' Is_Equal_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19260
--# expr-start 8383 '<Symbol Storage_Size at ???>' Sym_46 None
Sym_46 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Size); 
--# expr-done 8383
Is_Equal_46 := Block_Var = Sym_46; 
--# expr-done 8384
   If_Result_42 := Is_Equal_46;
end if;
if If_Result_42 then
   --# expr-start 8385 '<FieldAccess for subtype_attr_equation at ast.py:19262>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19262
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.subtype_attr_equation
Fld_5 := Libadalang.Implementation.Attribute_Ref_P_Subtype_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8385
   If_Result_135 := Fld_5;
else
--# expr-start 8387 '<Eq at ast.py:19264>' Is_Equal_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19264
--# expr-start 8386 '<Symbol Access at ???>' Sym_47 None
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Access); 
--# expr-done 8386
Is_Equal_47 := Block_Var = Sym_47; 
--# expr-done 8387
if Is_Equal_47 then
   If_Result_43 := True;
else
   --# expr-start 8389 '<Eq at ast.py:19264>' Is_Equal_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19264
--# expr-start 8388 '<Symbol Unchecked_Access at ???>' Sym_48 None
Sym_48 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unchecked_Access); 
--# expr-done 8388
Is_Equal_48 := Block_Var = Sym_48; 
--# expr-done 8389
   If_Result_43 := Is_Equal_48;
end if;
if If_Result_43 then
   If_Result_44 := True;
else
   --# expr-start 8391 '<Eq at ast.py:19264>' Is_Equal_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19264
--# expr-start 8390 '<Symbol Unrestricted_Access at ???>' Sym_49 None
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unrestricted_Access); 
--# expr-done 8390
Is_Equal_49 := Block_Var = Sym_49; 
--# expr-done 8391
   If_Result_44 := Is_Equal_49;
end if;
if If_Result_44 then
   --# expr-start 8392 '<FieldAccess for access_equation at ast.py:19266>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19266
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.access_equation
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Access_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8392
   If_Result_134 := Fld_6;
else
--# expr-start 8394 '<Eq at ast.py:19268>' Is_Equal_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19268
--# expr-start 8393 '<Symbol Image at ???>' Sym_50 None
Sym_50 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Image); 
--# expr-done 8393
Is_Equal_50 := Block_Var = Sym_50; 
--# expr-done 8394
if Is_Equal_50 then
   --# expr-start 8397 '<FieldAccess for image_equation at ast.py:19269>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19269
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8396 '<FieldAccess for std_entity at ast.py:19269>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19269
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8395 '<Symbol String at ???>' Sym_51 None
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 8395
--# property-call-start AdaNode.std_entity
Fld_7 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_51);
--# end
--# expr-done 8396
--# property-call-start AttributeRef.image_equation
Fld_8 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_7, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8397
   If_Result_133 := Fld_8;
else
--# expr-start 8399 '<Eq at ast.py:19271>' Is_Equal_51 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19271
--# expr-start 8398 '<Symbol Wide_Image at ???>' Sym_52 None
Sym_52 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Image); 
--# expr-done 8398
Is_Equal_51 := Block_Var = Sym_52; 
--# expr-done 8399
if Is_Equal_51 then
   --# expr-start 8402 '<FieldAccess for image_equation at ast.py:19272>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19272
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8401 '<FieldAccess for std_entity at ast.py:19272>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19272
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8400 '<Symbol Wide_String at ???>' Sym_53 None
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_String); 
--# expr-done 8400
--# property-call-start AdaNode.std_entity
Fld_9 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_53);
--# end
--# expr-done 8401
--# property-call-start AttributeRef.image_equation
Fld_10 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_9, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8402
   If_Result_132 := Fld_10;
else
--# expr-start 8404 '<Eq at ast.py:19274>' Is_Equal_52 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19274
--# expr-start 8403 '<Symbol Wide_Wide_Image at ???>' Sym_54 None
Sym_54 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Image); 
--# expr-done 8403
Is_Equal_52 := Block_Var = Sym_54; 
--# expr-done 8404
if Is_Equal_52 then
   --# expr-start 8407 '<FieldAccess for image_equation at ast.py:19275>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19275
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8406 '<FieldAccess for std_entity at ast.py:19275>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19275
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8405 '<Symbol Wide_Wide_String at ???>' Sym_55 None
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_String); 
--# expr-done 8405
--# property-call-start AdaNode.std_entity
Fld_11 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_55);
--# end
--# expr-done 8406
--# property-call-start AttributeRef.image_equation
Fld_12 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_11, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8407
   If_Result_131 := Fld_12;
else
--# expr-start 8409 '<Eq at ast.py:19277>' Is_Equal_53 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19277
--# expr-start 8408 '<Symbol Enum_Rep at ???>' Sym_56 None
Sym_56 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Rep); 
--# expr-done 8408
Is_Equal_53 := Block_Var = Sym_56; 
--# expr-done 8409
if Is_Equal_53 then
   --# expr-start 8410 '<FieldAccess for enum_rep_equation at ast.py:19278>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19278
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.enum_rep_equation
Fld_13 := Libadalang.Implementation.Attribute_Ref_P_Enum_Rep_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8410
   If_Result_130 := Fld_13;
else
--# expr-start 8412 '<Eq at ast.py:19280>' Is_Equal_54 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19280
--# expr-start 8411 '<Symbol Invalid_Value at ???>' Sym_57 None
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invalid_Value); 
--# expr-done 8411
Is_Equal_54 := Block_Var = Sym_57; 
--# expr-done 8412
if Is_Equal_54 then
   If_Result_45 := True;
else
   --# expr-start 8414 '<Eq at ast.py:19280>' Is_Equal_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19280
--# expr-start 8413 '<Symbol First_Valid at ???>' Sym_58 None
Sym_58 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First_Valid); 
--# expr-done 8413
Is_Equal_55 := Block_Var = Sym_58; 
--# expr-done 8414
   If_Result_45 := Is_Equal_55;
end if;
if If_Result_45 then
   If_Result_46 := True;
else
   --# expr-start 8416 '<Eq at ast.py:19280>' Is_Equal_56 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19280
--# expr-start 8415 '<Symbol Last_Valid at ???>' Sym_59 None
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last_Valid); 
--# expr-done 8415
Is_Equal_56 := Block_Var = Sym_59; 
--# expr-done 8416
   If_Result_46 := Is_Equal_56;
end if;
if If_Result_46 then
   --# expr-start 8417 '<FieldAccess for self_type_equation at ast.py:19281>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19281
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.self_type_equation
Fld_14 := Libadalang.Implementation.Attribute_Ref_P_Self_Type_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8417
   If_Result_129 := Fld_14;
else
--# expr-start 8419 '<Eq at ast.py:19283>' Is_Equal_57 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19283
--# expr-start 8418 '<Symbol Identity at ???>' Sym_60 None
Sym_60 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Identity); 
--# expr-done 8418
Is_Equal_57 := Block_Var = Sym_60; 
--# expr-done 8419
if Is_Equal_57 then
   --# expr-start 8420 '<FieldAccess for identity_equation at ast.py:19283>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19283
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.identity_equation
Fld_15 := Libadalang.Implementation.Attribute_Ref_P_Identity_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8420
   If_Result_128 := Fld_15;
else
--# expr-start 8422 '<Eq at ast.py:19284>' Is_Equal_58 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19284
--# expr-start 8421 '<Symbol Address at ???>' Sym_61 None
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address); 
--# expr-done 8421
Is_Equal_58 := Block_Var = Sym_61; 
--# expr-done 8422
if Is_Equal_58 then
   --# expr-start 8423 '<FieldAccess for address_equation at ast.py:19284>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19284
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.address_equation
Fld_16 := Libadalang.Implementation.Attribute_Ref_P_Address_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8423
   If_Result_127 := Fld_16;
else
--# expr-start 8425 '<Eq at ast.py:19286>' Is_Equal_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8424 '<Symbol Small at ???>' Sym_62 None
Sym_62 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Small); 
--# expr-done 8424
Is_Equal_59 := Block_Var = Sym_62; 
--# expr-done 8425
if Is_Equal_59 then
   If_Result_47 := True;
else
   --# expr-start 8427 '<Eq at ast.py:19286>' Is_Equal_60 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8426 '<Symbol Model_Small at ???>' Sym_63 None
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Small); 
--# expr-done 8426
Is_Equal_60 := Block_Var = Sym_63; 
--# expr-done 8427
   If_Result_47 := Is_Equal_60;
end if;
if If_Result_47 then
   If_Result_48 := True;
else
   --# expr-start 8429 '<Eq at ast.py:19286>' Is_Equal_61 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8428 '<Symbol Safe_Small at ???>' Sym_64 None
Sym_64 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Safe_Small); 
--# expr-done 8428
Is_Equal_61 := Block_Var = Sym_64; 
--# expr-done 8429
   If_Result_48 := Is_Equal_61;
end if;
if If_Result_48 then
   If_Result_49 := True;
else
   --# expr-start 8431 '<Eq at ast.py:19286>' Is_Equal_62 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8430 '<Symbol Epsilon at ???>' Sym_65 None
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Epsilon); 
--# expr-done 8430
Is_Equal_62 := Block_Var = Sym_65; 
--# expr-done 8431
   If_Result_49 := Is_Equal_62;
end if;
if If_Result_49 then
   If_Result_50 := True;
else
   --# expr-start 8433 '<Eq at ast.py:19286>' Is_Equal_63 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8432 '<Symbol Model_Epsilon at ???>' Sym_66 None
Sym_66 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Epsilon); 
--# expr-done 8432
Is_Equal_63 := Block_Var = Sym_66; 
--# expr-done 8433
   If_Result_50 := Is_Equal_63;
end if;
if If_Result_50 then
   If_Result_51 := True;
else
   --# expr-start 8435 '<Eq at ast.py:19286>' Is_Equal_64 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8434 '<Symbol Large at ???>' Sym_67 None
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Large); 
--# expr-done 8434
Is_Equal_64 := Block_Var = Sym_67; 
--# expr-done 8435
   If_Result_51 := Is_Equal_64;
end if;
if If_Result_51 then
   If_Result_52 := True;
else
   --# expr-start 8437 '<Eq at ast.py:19286>' Is_Equal_65 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8436 '<Symbol Safe_Large at ???>' Sym_68 None
Sym_68 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Safe_Large); 
--# expr-done 8436
Is_Equal_65 := Block_Var = Sym_68; 
--# expr-done 8437
   If_Result_52 := Is_Equal_65;
end if;
if If_Result_52 then
   If_Result_53 := True;
else
   --# expr-start 8439 '<Eq at ast.py:19286>' Is_Equal_66 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8438 '<Symbol Delta at ???>' Sym_69 None
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Delta); 
--# expr-done 8438
Is_Equal_66 := Block_Var = Sym_69; 
--# expr-done 8439
   If_Result_53 := Is_Equal_66;
end if;
if If_Result_53 then
   If_Result_54 := True;
else
   --# expr-start 8441 '<Eq at ast.py:19286>' Is_Equal_67 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8440 '<Symbol Safe_First at ???>' Sym_70 None
Sym_70 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Safe_First); 
--# expr-done 8440
Is_Equal_67 := Block_Var = Sym_70; 
--# expr-done 8441
   If_Result_54 := Is_Equal_67;
end if;
if If_Result_54 then
   If_Result_55 := True;
else
   --# expr-start 8443 '<Eq at ast.py:19286>' Is_Equal_68 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19286
--# expr-start 8442 '<Symbol Safe_Last at ???>' Sym_71 None
Sym_71 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Safe_Last); 
--# expr-done 8442
Is_Equal_68 := Block_Var = Sym_71; 
--# expr-done 8443
   If_Result_55 := Is_Equal_68;
end if;
if If_Result_55 then
   --# expr-start 8444 '<FieldAccess for universal_real_equation at ast.py:19291>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19291
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.universal_real_equation
Fld_17 := Libadalang.Implementation.Attribute_Ref_P_Universal_Real_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8444
   If_Result_126 := Fld_17;
else
--# expr-start 8446 '<Eq at ast.py:19293>' Is_Equal_69 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19293
--# expr-start 8445 '<Symbol Img at ???>' Sym_72 None
Sym_72 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Img); 
--# expr-done 8445
Is_Equal_69 := Block_Var = Sym_72; 
--# expr-done 8446
if Is_Equal_69 then
   --# expr-start 8449 '<FieldAccess for img_equation at ast.py:19294>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19294
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8448 '<FieldAccess for std_entity at ast.py:19294>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19294
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8447 '<Symbol String at ???>' Sym_73 None
Sym_73 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 8447
--# property-call-start AdaNode.std_entity
Fld_18 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_73);
--# end
--# expr-done 8448
--# property-call-start AttributeRef.img_equation
Fld_19 := Libadalang.Implementation.Attribute_Ref_P_Img_Equation (Node => Ent.Node, Str_Type => Fld_18, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8449
   If_Result_125 := Fld_19;
else
--# expr-start 8451 '<Eq at ast.py:19296>' Is_Equal_70 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19296
--# expr-start 8450 '<Symbol Tag at ???>' Sym_74 None
Sym_74 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tag); 
--# expr-done 8450
Is_Equal_70 := Block_Var = Sym_74; 
--# expr-done 8451
if Is_Equal_70 then
   --# expr-start 8452 '<FieldAccess for tag_attr_equation at ast.py:19296>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19296
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.tag_attr_equation
Fld_20 := Libadalang.Implementation.Attribute_Ref_P_Tag_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8452
   If_Result_124 := Fld_20;
else
--# expr-start 8454 '<Eq at ast.py:19298>' Is_Equal_71 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19298
--# expr-start 8453 '<Symbol Result at ???>' Sym_75 None
Sym_75 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Result); 
--# expr-done 8453
Is_Equal_71 := Block_Var = Sym_75; 
--# expr-done 8454
if Is_Equal_71 then
   --# expr-start 8455 '<FieldAccess for result_attr_equation at ast.py:19298>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19298
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.result_attr_equation
Fld_21 := Libadalang.Implementation.Attribute_Ref_P_Result_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8455
   If_Result_123 := Fld_21;
else
--# expr-start 8457 '<Eq at ast.py:19300>' Is_Equal_72 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19300
--# expr-start 8456 '<Symbol Old at ???>' Sym_76 None
Sym_76 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Old); 
--# expr-done 8456
Is_Equal_72 := Block_Var = Sym_76; 
--# expr-done 8457
if Is_Equal_72 then
   If_Result_56 := True;
else
   --# expr-start 8459 '<Eq at ast.py:19300>' Is_Equal_73 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19300
--# expr-start 8458 '<Symbol Loop_Entry at ???>' Sym_77 None
Sym_77 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Loop_Entry); 
--# expr-done 8458
Is_Equal_73 := Block_Var = Sym_77; 
--# expr-done 8459
   If_Result_56 := Is_Equal_73;
end if;
if If_Result_56 then
   --# expr-start 8460 '<FieldAccess for bind_to_prefix_eq at ast.py:19301>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19301
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.bind_to_prefix_eq
Fld_22 := Libadalang.Implementation.Attribute_Ref_P_Bind_To_Prefix_Eq (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8460
   If_Result_122 := Fld_22;
else
--# expr-start 8462 '<Eq at ast.py:19303>' Is_Equal_74 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19303
--# expr-start 8461 '<Symbol Class at ???>' Sym_78 None
Sym_78 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Class); 
--# expr-done 8461
Is_Equal_74 := Block_Var = Sym_78; 
--# expr-done 8462
if Is_Equal_74 then
   If_Result_57 := True;
else
   --# expr-start 8464 '<Eq at ast.py:19303>' Is_Equal_75 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19303
--# expr-start 8463 '<Symbol Base at ???>' Sym_79 None
Sym_79 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Base); 
--# expr-done 8463
Is_Equal_75 := Block_Var = Sym_79; 
--# expr-done 8464
   If_Result_57 := Is_Equal_75;
end if;
if If_Result_57 then
   --# expr-start 8466 '<FieldAccess for sub_equation at ast.py:19304>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19304
--# expr-start 8465 '<FieldAccess for prefix at ast.py:19304>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19304
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8465
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_24 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_23.Node, Env => Env, Origin => Origin, E_Info => Fld_23.Info);
--# end
--# expr-done 8466
   If_Result_121 := Fld_24;
else
--# expr-start 8468 '<Eq at ast.py:19306>' Is_Equal_76 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8467 '<Symbol Valid at ???>' Sym_80 None
Sym_80 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Valid); 
--# expr-done 8467
Is_Equal_76 := Block_Var = Sym_80; 
--# expr-done 8468
if Is_Equal_76 then
   If_Result_58 := True;
else
   --# expr-start 8470 '<Eq at ast.py:19306>' Is_Equal_77 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8469 '<Symbol Machine_Overflows at ???>' Sym_81 None
Sym_81 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Overflows); 
--# expr-done 8469
Is_Equal_77 := Block_Var = Sym_81; 
--# expr-done 8470
   If_Result_58 := Is_Equal_77;
end if;
if If_Result_58 then
   If_Result_59 := True;
else
   --# expr-start 8472 '<Eq at ast.py:19306>' Is_Equal_78 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8471 '<Symbol Machine_Rounds at ???>' Sym_82 None
Sym_82 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Rounds); 
--# expr-done 8471
Is_Equal_78 := Block_Var = Sym_82; 
--# expr-done 8472
   If_Result_59 := Is_Equal_78;
end if;
if If_Result_59 then
   If_Result_60 := True;
else
   --# expr-start 8474 '<Eq at ast.py:19306>' Is_Equal_79 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8473 '<Symbol Has_Access_Values at ???>' Sym_83 None
Sym_83 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Access_Values); 
--# expr-done 8473
Is_Equal_79 := Block_Var = Sym_83; 
--# expr-done 8474
   If_Result_60 := Is_Equal_79;
end if;
if If_Result_60 then
   If_Result_61 := True;
else
   --# expr-start 8476 '<Eq at ast.py:19306>' Is_Equal_80 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8475 '<Symbol Has_Discriminants at ???>' Sym_84 None
Sym_84 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Discriminants); 
--# expr-done 8475
Is_Equal_80 := Block_Var = Sym_84; 
--# expr-done 8476
   If_Result_61 := Is_Equal_80;
end if;
if If_Result_61 then
   If_Result_62 := True;
else
   --# expr-start 8478 '<Eq at ast.py:19306>' Is_Equal_81 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8477 '<Symbol Has_Tagged_Values at ???>' Sym_85 None
Sym_85 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Tagged_Values); 
--# expr-done 8477
Is_Equal_81 := Block_Var = Sym_85; 
--# expr-done 8478
   If_Result_62 := Is_Equal_81;
end if;
if If_Result_62 then
   If_Result_63 := True;
else
   --# expr-start 8480 '<Eq at ast.py:19306>' Is_Equal_82 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8479 '<Symbol Definite at ???>' Sym_86 None
Sym_86 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Definite); 
--# expr-done 8479
Is_Equal_82 := Block_Var = Sym_86; 
--# expr-done 8480
   If_Result_63 := Is_Equal_82;
end if;
if If_Result_63 then
   If_Result_64 := True;
else
   --# expr-start 8482 '<Eq at ast.py:19306>' Is_Equal_83 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8481 '<Symbol Constrained at ???>' Sym_87 None
Sym_87 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constrained); 
--# expr-done 8481
Is_Equal_83 := Block_Var = Sym_87; 
--# expr-done 8482
   If_Result_64 := Is_Equal_83;
end if;
if If_Result_64 then
   If_Result_65 := True;
else
   --# expr-start 8484 '<Eq at ast.py:19306>' Is_Equal_84 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8483 '<Symbol Initialized at ???>' Sym_88 None
Sym_88 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Initialized); 
--# expr-done 8483
Is_Equal_84 := Block_Var = Sym_88; 
--# expr-done 8484
   If_Result_65 := Is_Equal_84;
end if;
if If_Result_65 then
   If_Result_66 := True;
else
   --# expr-start 8486 '<Eq at ast.py:19306>' Is_Equal_85 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8485 '<Symbol Valid_Scalars at ???>' Sym_89 None
Sym_89 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Valid_Scalars); 
--# expr-done 8485
Is_Equal_85 := Block_Var = Sym_89; 
--# expr-done 8486
   If_Result_66 := Is_Equal_85;
end if;
if If_Result_66 then
   If_Result_67 := True;
else
   --# expr-start 8488 '<Eq at ast.py:19306>' Is_Equal_86 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8487 '<Symbol Unconstrained_Array at ???>' Sym_90 None
Sym_90 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unconstrained_Array); 
--# expr-done 8487
Is_Equal_86 := Block_Var = Sym_90; 
--# expr-done 8488
   If_Result_67 := Is_Equal_86;
end if;
if If_Result_67 then
   If_Result_68 := True;
else
   --# expr-start 8490 '<Eq at ast.py:19306>' Is_Equal_87 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8489 '<Symbol Library_Level at ???>' Sym_91 None
Sym_91 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Library_Level); 
--# expr-done 8489
Is_Equal_87 := Block_Var = Sym_91; 
--# expr-done 8490
   If_Result_68 := Is_Equal_87;
end if;
if If_Result_68 then
   If_Result_69 := True;
else
   --# expr-start 8492 '<Eq at ast.py:19306>' Is_Equal_88 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8491 '<Symbol Denorm at ???>' Sym_92 None
Sym_92 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Denorm); 
--# expr-done 8491
Is_Equal_88 := Block_Var = Sym_92; 
--# expr-done 8492
   If_Result_69 := Is_Equal_88;
end if;
if If_Result_69 then
   If_Result_70 := True;
else
   --# expr-start 8494 '<Eq at ast.py:19306>' Is_Equal_89 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19306
--# expr-start 8493 '<Symbol Signed_Zeros at ???>' Sym_93 None
Sym_93 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Signed_Zeros); 
--# expr-done 8493
Is_Equal_89 := Block_Var = Sym_93; 
--# expr-done 8494
   If_Result_70 := Is_Equal_89;
end if;
if If_Result_70 then
   --# expr-start 8500 '<And at ast.py:19312>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19312
--# expr-start 8496 '<FieldAccess for sub_equation at ast.py:19312>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19312
--# expr-start 8495 '<FieldAccess for prefix at ast.py:19312>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19312
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8495
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_26 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_25.Node, Env => Env, Origin => Origin, E_Info => Fld_25.Info);
--# end
--# expr-done 8496
--# expr-start 8499 '<Bind at ast.py:19313>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19313
--# expr-start 8497 '<FieldAccess for type_var at ast.py:19313>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19313
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8497
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 8498 '<FieldAccess for bool_type at ast.py:19313>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19313
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_28 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 8498
      Cast_Result := Create_Internal_Entity
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
Bind_Result := Solver.Create_Assign (Fld_27, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19313") else null)); 
--# expr-done 8499
And_Pred := Create_And (Fld_26, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19312") else null)); 
--# expr-done 8500
   If_Result_120 := And_Pred;
else
--# expr-start 8502 '<Eq at ast.py:19319>' Is_Equal_90 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19319
--# expr-start 8501 '<Symbol Enabled at ???>' Sym_94 None
Sym_94 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enabled); 
--# expr-done 8501
Is_Equal_90 := Block_Var = Sym_94; 
--# expr-done 8502
if Is_Equal_90 then
   --# expr-start 8505 '<Bind at ast.py:19320>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19320
--# expr-start 8503 '<FieldAccess for type_var at ast.py:19320>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19320
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8503
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 8504 '<FieldAccess for bool_type at ast.py:19320>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19320
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_30 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 8504
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_30.Node,
         Info => Fld_30.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_29, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19320") else null)); 
--# expr-done 8505
   If_Result_119 := Bind_Result_1;
else
--# expr-start 8507 '<Eq at ast.py:19322>' Is_Equal_91 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8506 '<Symbol Width at ???>' Sym_95 None
Sym_95 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Width); 
--# expr-done 8506
Is_Equal_91 := Block_Var = Sym_95; 
--# expr-done 8507
if Is_Equal_91 then
   If_Result_71 := True;
else
   --# expr-start 8509 '<Eq at ast.py:19322>' Is_Equal_92 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8508 '<Symbol Component_Size at ???>' Sym_96 None
Sym_96 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Component_Size); 
--# expr-done 8508
Is_Equal_92 := Block_Var = Sym_96; 
--# expr-done 8509
   If_Result_71 := Is_Equal_92;
end if;
if If_Result_71 then
   If_Result_72 := True;
else
   --# expr-start 8511 '<Eq at ast.py:19322>' Is_Equal_93 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8510 '<Symbol Position at ???>' Sym_97 None
Sym_97 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Position); 
--# expr-done 8510
Is_Equal_93 := Block_Var = Sym_97; 
--# expr-done 8511
   If_Result_72 := Is_Equal_93;
end if;
if If_Result_72 then
   If_Result_73 := True;
else
   --# expr-start 8513 '<Eq at ast.py:19322>' Is_Equal_94 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8512 '<Symbol Mantissa at ???>' Sym_98 None
Sym_98 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mantissa); 
--# expr-done 8512
Is_Equal_94 := Block_Var = Sym_98; 
--# expr-done 8513
   If_Result_73 := Is_Equal_94;
end if;
if If_Result_73 then
   If_Result_74 := True;
else
   --# expr-start 8515 '<Eq at ast.py:19322>' Is_Equal_95 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8514 '<Symbol Model_Mantissa at ???>' Sym_99 None
Sym_99 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Mantissa); 
--# expr-done 8514
Is_Equal_95 := Block_Var = Sym_99; 
--# expr-done 8515
   If_Result_74 := Is_Equal_95;
end if;
if If_Result_74 then
   If_Result_75 := True;
else
   --# expr-start 8517 '<Eq at ast.py:19322>' Is_Equal_96 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8516 '<Symbol Machine_Mantissa at ???>' Sym_100 None
Sym_100 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Mantissa); 
--# expr-done 8516
Is_Equal_96 := Block_Var = Sym_100; 
--# expr-done 8517
   If_Result_75 := Is_Equal_96;
end if;
if If_Result_75 then
   If_Result_76 := True;
else
   --# expr-start 8519 '<Eq at ast.py:19322>' Is_Equal_97 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8518 '<Symbol Fore at ???>' Sym_101 None
Sym_101 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fore); 
--# expr-done 8518
Is_Equal_97 := Block_Var = Sym_101; 
--# expr-done 8519
   If_Result_76 := Is_Equal_97;
end if;
if If_Result_76 then
   If_Result_77 := True;
else
   --# expr-start 8521 '<Eq at ast.py:19322>' Is_Equal_98 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8520 '<Symbol Aft at ???>' Sym_102 None
Sym_102 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aft); 
--# expr-done 8520
Is_Equal_98 := Block_Var = Sym_102; 
--# expr-done 8521
   If_Result_77 := Is_Equal_98;
end if;
if If_Result_77 then
   If_Result_78 := True;
else
   --# expr-start 8523 '<Eq at ast.py:19322>' Is_Equal_99 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8522 '<Symbol Digits at ???>' Sym_103 None
Sym_103 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Digits); 
--# expr-done 8522
Is_Equal_99 := Block_Var = Sym_103; 
--# expr-done 8523
   If_Result_78 := Is_Equal_99;
end if;
if If_Result_78 then
   If_Result_79 := True;
else
   --# expr-start 8525 '<Eq at ast.py:19322>' Is_Equal_100 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8524 '<Symbol Modulus at ???>' Sym_104 None
Sym_104 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Modulus); 
--# expr-done 8524
Is_Equal_100 := Block_Var = Sym_104; 
--# expr-done 8525
   If_Result_79 := Is_Equal_100;
end if;
if If_Result_79 then
   If_Result_80 := True;
else
   --# expr-start 8527 '<Eq at ast.py:19322>' Is_Equal_101 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8526 '<Symbol Word_Size at ???>' Sym_105 None
Sym_105 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Word_Size); 
--# expr-done 8526
Is_Equal_101 := Block_Var = Sym_105; 
--# expr-done 8527
   If_Result_80 := Is_Equal_101;
end if;
if If_Result_80 then
   If_Result_81 := True;
else
   --# expr-start 8529 '<Eq at ast.py:19322>' Is_Equal_102 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8528 '<Symbol Wchar_T_Size at ???>' Sym_106 None
Sym_106 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wchar_T_Size); 
--# expr-done 8528
Is_Equal_102 := Block_Var = Sym_106; 
--# expr-done 8529
   If_Result_81 := Is_Equal_102;
end if;
if If_Result_81 then
   If_Result_82 := True;
else
   --# expr-start 8531 '<Eq at ast.py:19322>' Is_Equal_103 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8530 '<Symbol Max_Integer_Size at ???>' Sym_107 None
Sym_107 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max_Integer_Size); 
--# expr-done 8530
Is_Equal_103 := Block_Var = Sym_107; 
--# expr-done 8531
   If_Result_82 := Is_Equal_103;
end if;
if If_Result_82 then
   If_Result_83 := True;
else
   --# expr-start 8533 '<Eq at ast.py:19322>' Is_Equal_104 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8532 '<Symbol Address_Size at ???>' Sym_108 None
Sym_108 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address_Size); 
--# expr-done 8532
Is_Equal_104 := Block_Var = Sym_108; 
--# expr-done 8533
   If_Result_83 := Is_Equal_104;
end if;
if If_Result_83 then
   If_Result_84 := True;
else
   --# expr-start 8535 '<Eq at ast.py:19322>' Is_Equal_105 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8534 '<Symbol Maximum_Alignment at ???>' Sym_109 None
Sym_109 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Maximum_Alignment); 
--# expr-done 8534
Is_Equal_105 := Block_Var = Sym_109; 
--# expr-done 8535
   If_Result_84 := Is_Equal_105;
end if;
if If_Result_84 then
   If_Result_85 := True;
else
   --# expr-start 8537 '<Eq at ast.py:19322>' Is_Equal_106 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8536 '<Symbol System_Allocator_Alignment at ???>' Sym_110 None
Sym_110 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System_Allocator_Alignment); 
--# expr-done 8536
Is_Equal_106 := Block_Var = Sym_110; 
--# expr-done 8537
   If_Result_85 := Is_Equal_106;
end if;
if If_Result_85 then
   If_Result_86 := True;
else
   --# expr-start 8539 '<Eq at ast.py:19322>' Is_Equal_107 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8538 '<Symbol Finalization_Size at ???>' Sym_111 None
Sym_111 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Finalization_Size); 
--# expr-done 8538
Is_Equal_107 := Block_Var = Sym_111; 
--# expr-done 8539
   If_Result_86 := Is_Equal_107;
end if;
if If_Result_86 then
   If_Result_87 := True;
else
   --# expr-start 8541 '<Eq at ast.py:19322>' Is_Equal_108 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8540 '<Symbol Descriptor_Size at ???>' Sym_112 None
Sym_112 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Descriptor_Size); 
--# expr-done 8540
Is_Equal_108 := Block_Var = Sym_112; 
--# expr-done 8541
   If_Result_87 := Is_Equal_108;
end if;
if If_Result_87 then
   If_Result_88 := True;
else
   --# expr-start 8543 '<Eq at ast.py:19322>' Is_Equal_109 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8542 '<Symbol Alignment at ???>' Sym_113 None
Sym_113 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Alignment); 
--# expr-done 8542
Is_Equal_109 := Block_Var = Sym_113; 
--# expr-done 8543
   If_Result_88 := Is_Equal_109;
end if;
if If_Result_88 then
   If_Result_89 := True;
else
   --# expr-start 8545 '<Eq at ast.py:19322>' Is_Equal_110 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8544 '<Symbol First_Bit at ???>' Sym_114 None
Sym_114 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First_Bit); 
--# expr-done 8544
Is_Equal_110 := Block_Var = Sym_114; 
--# expr-done 8545
   If_Result_89 := Is_Equal_110;
end if;
if If_Result_89 then
   If_Result_90 := True;
else
   --# expr-start 8547 '<Eq at ast.py:19322>' Is_Equal_111 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8546 '<Symbol Last_Bit at ???>' Sym_115 None
Sym_115 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last_Bit); 
--# expr-done 8546
Is_Equal_111 := Block_Var = Sym_115; 
--# expr-done 8547
   If_Result_90 := Is_Equal_111;
end if;
if If_Result_90 then
   If_Result_91 := True;
else
   --# expr-start 8549 '<Eq at ast.py:19322>' Is_Equal_112 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8548 '<Symbol Default_Bit_Order at ???>' Sym_116 None
Sym_116 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Default_Bit_Order); 
--# expr-done 8548
Is_Equal_112 := Block_Var = Sym_116; 
--# expr-done 8549
   If_Result_91 := Is_Equal_112;
end if;
if If_Result_91 then
   If_Result_92 := True;
else
   --# expr-start 8551 '<Eq at ast.py:19322>' Is_Equal_113 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8550 '<Symbol Range_Length at ???>' Sym_117 None
Sym_117 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range_Length); 
--# expr-done 8550
Is_Equal_113 := Block_Var = Sym_117; 
--# expr-done 8551
   If_Result_92 := Is_Equal_113;
end if;
if If_Result_92 then
   If_Result_93 := True;
else
   --# expr-start 8553 '<Eq at ast.py:19322>' Is_Equal_114 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8552 '<Symbol Storage_Unit at ???>' Sym_118 None
Sym_118 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Unit); 
--# expr-done 8552
Is_Equal_114 := Block_Var = Sym_118; 
--# expr-done 8553
   If_Result_93 := Is_Equal_114;
end if;
if If_Result_93 then
   If_Result_94 := True;
else
   --# expr-start 8555 '<Eq at ast.py:19322>' Is_Equal_115 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8554 '<Symbol Stream_Size at ???>' Sym_119 None
Sym_119 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Stream_Size); 
--# expr-done 8554
Is_Equal_115 := Block_Var = Sym_119; 
--# expr-done 8555
   If_Result_94 := Is_Equal_115;
end if;
if If_Result_94 then
   If_Result_95 := True;
else
   --# expr-start 8557 '<Eq at ast.py:19322>' Is_Equal_116 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8556 '<Symbol Small_Numerator at ???>' Sym_120 None
Sym_120 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Small_Numerator); 
--# expr-done 8556
Is_Equal_116 := Block_Var = Sym_120; 
--# expr-done 8557
   If_Result_95 := Is_Equal_116;
end if;
if If_Result_95 then
   If_Result_96 := True;
else
   --# expr-start 8559 '<Eq at ast.py:19322>' Is_Equal_117 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8558 '<Symbol Small_Denominator at ???>' Sym_121 None
Sym_121 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Small_Denominator); 
--# expr-done 8558
Is_Equal_117 := Block_Var = Sym_121; 
--# expr-done 8559
   If_Result_96 := Is_Equal_117;
end if;
if If_Result_96 then
   If_Result_97 := True;
else
   --# expr-start 8561 '<Eq at ast.py:19322>' Is_Equal_118 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8560 '<Symbol Machine_Emin at ???>' Sym_122 None
Sym_122 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Emin); 
--# expr-done 8560
Is_Equal_118 := Block_Var = Sym_122; 
--# expr-done 8561
   If_Result_97 := Is_Equal_118;
end if;
if If_Result_97 then
   If_Result_98 := True;
else
   --# expr-start 8563 '<Eq at ast.py:19322>' Is_Equal_119 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8562 '<Symbol Machine_Emax at ???>' Sym_123 None
Sym_123 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Emax); 
--# expr-done 8562
Is_Equal_119 := Block_Var = Sym_123; 
--# expr-done 8563
   If_Result_98 := Is_Equal_119;
end if;
if If_Result_98 then
   If_Result_99 := True;
else
   --# expr-start 8565 '<Eq at ast.py:19322>' Is_Equal_120 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19322
--# expr-start 8564 '<Symbol Model_Emin at ???>' Sym_124 None
Sym_124 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Emin); 
--# expr-done 8564
Is_Equal_120 := Block_Var = Sym_124; 
--# expr-done 8565
   If_Result_99 := Is_Equal_120;
end if;
if If_Result_99 then
   --# expr-start 8570 '<And at ast.py:19334>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19334
--# expr-start 8567 '<FieldAccess for sub_equation at ast.py:19334>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19334
--# expr-start 8566 '<FieldAccess for prefix at ast.py:19334>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19334
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8566
      if Fld_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_32 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_31.Node, Env => Env, Origin => Origin, E_Info => Fld_31.Info);
--# end
--# expr-done 8567
--# expr-start 8569 '<FieldAccess for universal_int_bind at ast.py:19335>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19335
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8568 '<FieldAccess for type_var at ast.py:19335>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19335
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8568
--# property-call-start AdaNode.universal_int_bind
Fld_34 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_33, Origin => Origin);
--# end
--# expr-done 8569
And_Pred_1 := Create_And (Fld_32, Fld_34, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19334") else null)); 
--# expr-done 8570
   If_Result_118 := And_Pred_1;
else
--# expr-start 8572 '<Eq at ast.py:19337>' Is_Equal_121 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19337
--# expr-start 8571 '<Symbol External_Tag at ???>' Sym_125 None
Sym_125 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_External_Tag); 
--# expr-done 8571
Is_Equal_121 := Block_Var = Sym_125; 
--# expr-done 8572
if Is_Equal_121 then
   If_Result_100 := True;
else
   --# expr-start 8574 '<Eq at ast.py:19337>' Is_Equal_122 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19337
--# expr-start 8573 '<Symbol Type_Key at ???>' Sym_126 None
Sym_126 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Key); 
--# expr-done 8573
Is_Equal_122 := Block_Var = Sym_126; 
--# expr-done 8574
   If_Result_100 := Is_Equal_122;
end if;
if If_Result_100 then
   --# expr-start 8581 '<And at ast.py:19338>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19338
--# expr-start 8576 '<FieldAccess for sub_equation at ast.py:19338>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19338
--# expr-start 8575 '<FieldAccess for prefix at ast.py:19338>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19338
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8575
      if Fld_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_36 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_35.Node, Env => Env, Origin => Origin, E_Info => Fld_35.Info);
--# end
--# expr-done 8576
--# expr-start 8580 '<Bind at ast.py:19339>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19339
--# expr-start 8577 '<FieldAccess for type_var at ast.py:19339>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19339
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8577
Fld_37.Value := No_Entity;
Entity_Vars.Reset (Fld_37);
--# expr-start 8579 '<FieldAccess for std_entity at ast.py:19339>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19339
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8578 '<Symbol String at ???>' Sym_127 None
Sym_127 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 8578
--# property-call-start AdaNode.std_entity
Fld_38 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_127);
--# end
--# expr-done 8579
Bind_Result_2 := Solver.Create_Assign (Fld_37, Fld_38, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19339") else null)); 
--# expr-done 8580
And_Pred_2 := Create_And (Fld_36, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19338") else null)); 
--# expr-done 8581
   If_Result_117 := And_Pred_2;
else
--# expr-start 8583 '<Eq at ast.py:19341>' Is_Equal_123 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19341
--# expr-start 8582 '<Symbol Target_Name at ???>' Sym_128 None
Sym_128 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Target_Name); 
--# expr-done 8582
Is_Equal_123 := Block_Var = Sym_128; 
--# expr-done 8583
if Is_Equal_123 then
   --# expr-start 8587 '<Bind at ast.py:19342>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19342
--# expr-start 8584 '<FieldAccess for type_var at ast.py:19342>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19342
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8584
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
--# expr-start 8586 '<FieldAccess for std_entity at ast.py:19342>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19342
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8585 '<Symbol String at ???>' Sym_129 None
Sym_129 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 8585
--# property-call-start AdaNode.std_entity
Fld_40 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_129);
--# end
--# expr-done 8586
Bind_Result_3 := Solver.Create_Assign (Fld_39, Fld_40, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19342") else null)); 
--# expr-done 8587
   If_Result_116 := Bind_Result_3;
else
--# expr-start 8589 '<Eq at ast.py:19344>' Is_Equal_124 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19344
--# expr-start 8588 '<Symbol Storage_Pool at ???>' Sym_130 None
Sym_130 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Pool); 
--# expr-done 8588
Is_Equal_124 := Block_Var = Sym_130; 
--# expr-done 8589
if Is_Equal_124 then
   --# expr-start 8590 '<FieldAccess for storage_pool_equation at ast.py:19344>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19344
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.storage_pool_equation
Fld_41 := Libadalang.Implementation.Attribute_Ref_P_Storage_Pool_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8590
   If_Result_115 := Fld_41;
else
--# expr-start 8592 '<Eq at ast.py:19346>' Is_Equal_125 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19346
--# expr-start 8591 '<Symbol Bit_Order at ???>' Sym_131 None
Sym_131 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Bit_Order); 
--# expr-done 8591
Is_Equal_125 := Block_Var = Sym_131; 
--# expr-done 8592
if Is_Equal_125 then
   If_Result_101 := True;
else
   --# expr-start 8594 '<Eq at ast.py:19346>' Is_Equal_126 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19346
--# expr-start 8593 '<Symbol Scalar_Storage_Order at ???>' Sym_132 None
Sym_132 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scalar_Storage_Order); 
--# expr-done 8593
Is_Equal_126 := Block_Var = Sym_132; 
--# expr-done 8594
   If_Result_101 := Is_Equal_126;
end if;
if If_Result_101 then
   If_Result_102 := True;
else
   --# expr-start 8596 '<Eq at ast.py:19346>' Is_Equal_127 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19346
--# expr-start 8595 '<Symbol Default_Scalar_Storage_Order at ???>' Sym_133 None
Sym_133 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Default_Scalar_Storage_Order); 
--# expr-done 8595
Is_Equal_127 := Block_Var = Sym_133; 
--# expr-done 8596
   If_Result_102 := Is_Equal_127;
end if;
if If_Result_102 then
   --# expr-start 8597 '<FieldAccess for order_equation at ast.py:19348>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19348
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.order_equation
Fld_42 := Libadalang.Implementation.Attribute_Ref_P_Order_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8597
   If_Result_114 := Fld_42;
else
--# expr-start 8599 '<Eq at ast.py:19350>' Is_Equal_128 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19350
--# expr-start 8598 '<Symbol Type_Class at ???>' Sym_134 None
Sym_134 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Class); 
--# expr-done 8598
Is_Equal_128 := Block_Var = Sym_134; 
--# expr-done 8599
if Is_Equal_128 then
   --# expr-start 8600 '<FieldAccess for type_class_equation at ast.py:19350>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19350
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.type_class_equation
Fld_43 := Libadalang.Implementation.Attribute_Ref_P_Type_Class_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8600
   If_Result_113 := Fld_43;
else
--# expr-start 8602 '<Eq at ast.py:19353>' Is_Equal_129 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19353
--# expr-start 8601 '<Symbol Callable at ???>' Sym_135 None
Sym_135 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Callable); 
--# expr-done 8601
Is_Equal_129 := Block_Var = Sym_135; 
--# expr-done 8602
if Is_Equal_129 then
   If_Result_103 := True;
else
   --# expr-start 8604 '<Eq at ast.py:19353>' Is_Equal_130 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19353
--# expr-start 8603 '<Symbol Terminated at ???>' Sym_136 None
Sym_136 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Terminated); 
--# expr-done 8603
Is_Equal_130 := Block_Var = Sym_136; 
--# expr-done 8604
   If_Result_103 := Is_Equal_130;
end if;
if If_Result_103 then
   --# expr-start 8610 '<And at ast.py:19354>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19354
--# expr-start 8606 '<FieldAccess for sub_equation at ast.py:19354>' Fld_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19354
--# expr-start 8605 '<FieldAccess for prefix at ast.py:19354>' Fld_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19354
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_44 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8605
      if Fld_44.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_45 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_44.Node, Env => Env, Origin => Origin, E_Info => Fld_44.Info);
--# end
--# expr-done 8606
--# expr-start 8609 '<Bind at ast.py:19354>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19354
--# expr-start 8607 '<FieldAccess for type_var at ast.py:19354>' Fld_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19354
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_46 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8607
Fld_46.Value := No_Entity;
Entity_Vars.Reset (Fld_46);
--# expr-start 8608 '<FieldAccess for bool_type at ast.py:19354>' Fld_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19354
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_47 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 8608
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_47.Node,
         Info => Fld_47.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_46, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19354") else null)); 
--# expr-done 8609
And_Pred_3 := Create_And (Fld_45, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19354") else null)); 
--# expr-done 8610
   If_Result_112 := And_Pred_3;
else
--# expr-start 8612 '<Eq at ast.py:19357>' Is_Equal_131 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19357
--# expr-start 8611 '<Symbol Count at ???>' Sym_137 None
Sym_137 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Count); 
--# expr-done 8611
Is_Equal_131 := Block_Var = Sym_137; 
--# expr-done 8612
if Is_Equal_131 then
   --# expr-start 8617 '<And at ast.py:19358>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19358
--# expr-start 8614 '<FieldAccess for xref_no_overloading at ast.py:19358>' Fld_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19358
--# expr-start 8613 '<FieldAccess for prefix at ast.py:19358>' Fld_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19358
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8613
      if Fld_48.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_49 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_48.Node, Env => Env, Origin => Origin, E_Info => Fld_48.Info);
--# end
--# expr-done 8614
--# expr-start 8616 '<FieldAccess for universal_int_bind at ast.py:19359>' Fld_51 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19359
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8615 '<FieldAccess for type_var at ast.py:19359>' Fld_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19359
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8615
--# property-call-start AdaNode.universal_int_bind
Fld_51 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_50, Origin => Origin);
--# end
--# expr-done 8616
And_Pred_4 := Create_And (Fld_49, Fld_51, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19358") else null)); 
--# expr-done 8617
   If_Result_111 := And_Pred_4;
else
--# expr-start 8619 '<Eq at ast.py:19361>' Is_Equal_132 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19361
--# expr-start 8618 '<Symbol Caller at ???>' Sym_138 None
Sym_138 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Caller); 
--# expr-done 8618
Is_Equal_132 := Block_Var = Sym_138; 
--# expr-done 8619
if Is_Equal_132 then
   --# expr-start 8625 '<And at ast.py:19362>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19362
--# expr-start 8621 '<FieldAccess for xref_no_overloading at ast.py:19362>' Fld_53 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19362
--# expr-start 8620 '<FieldAccess for prefix at ast.py:19362>' Fld_52 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19362
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_52 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8620
      if Fld_52.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_53 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_52.Node, Env => Env, Origin => Origin, E_Info => Fld_52.Info);
--# end
--# expr-done 8621
--# expr-start 8624 '<Bind at ast.py:19363>' Bind_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19363
--# expr-start 8622 '<FieldAccess for type_var at ast.py:19363>' Fld_54 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19363
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_54 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8622
Fld_54.Value := No_Entity;
Entity_Vars.Reset (Fld_54);
--# expr-start 8623 '<FieldAccess for task_id_type at ast.py:19363>' Fld_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19363
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_55 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 8623
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_55.Node,
         Info => Fld_55.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_54, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19363") else null)); 
--# expr-done 8624
And_Pred_5 := Create_And (Fld_53, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19362") else null)); 
--# expr-done 8625
   If_Result_110 := And_Pred_5;
else
--# expr-start 8627 '<Eq at ast.py:19365>' Is_Equal_133 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19365
--# expr-start 8626 '<Symbol Machine_Radix at ???>' Sym_139 None
Sym_139 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Radix); 
--# expr-done 8626
Is_Equal_133 := Block_Var = Sym_139; 
--# expr-done 8627
if Is_Equal_133 then
   --# expr-start 8628 '<FieldAccess for universal_int_equation at ast.py:19366>' Fld_56 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19366
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.universal_int_equation
Fld_56 := Libadalang.Implementation.Attribute_Ref_P_Universal_Int_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8628
   If_Result_109 := Fld_56;
else
--# expr-start 8630 '<Eq at ast.py:19368>' Is_Equal_134 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19368
--# expr-start 8629 '<Symbol To_Address at ???>' Sym_140 None
Sym_140 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_To_Address); 
--# expr-done 8629
Is_Equal_134 := Block_Var = Sym_140; 
--# expr-done 8630
if Is_Equal_134 then
   --# expr-start 8631 '<FieldAccess for to_address_equation at ast.py:19369>' Fld_57 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19369
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.to_address_equation
Fld_57 := Libadalang.Implementation.Attribute_Ref_P_To_Address_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8631
   If_Result_108 := Fld_57;
else
--# expr-start 8633 '<Eq at ast.py:19371>' Is_Equal_135 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19371
--# expr-start 8632 '<Symbol Index at ???>' Sym_141 None
Sym_141 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Index); 
--# expr-done 8632
Is_Equal_135 := Block_Var = Sym_141; 
--# expr-done 8633
if Is_Equal_135 then
   --# expr-start 8634 '<FieldAccess for index_equation at ast.py:19372>' Fld_58 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19372
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.index_equation
Fld_58 := Libadalang.Implementation.Attribute_Ref_P_Index_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8634
   If_Result_107 := Fld_58;
else
--# expr-start 8636 '<Eq at ast.py:19374>' Is_Equal_136 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19374
--# expr-start 8635 '<Symbol Abort_Signal at ???>' Sym_142 None
Sym_142 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abort_Signal); 
--# expr-done 8635
Is_Equal_136 := Block_Var = Sym_142; 
--# expr-done 8636
if Is_Equal_136 then
   --# expr-start 8643 '<And at ast.py:19375>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19375
--# expr-start 8640 '<Bind at ast.py:19375>' Bind_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19375
--# expr-start 8637 '<FieldAccess for ref_var at ast.py:19375>' Fld_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19375
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_59 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 8637
Fld_59.Value := No_Entity;
Entity_Vars.Reset (Fld_59);
--# expr-start 8639 '<FieldAccess for std_entity at ast.py:19375>' Fld_60 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19375
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8638 '<Symbol abort_signal_ at ???>' Sym_143 None
Sym_143 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abort_Signal_286); 
--# expr-done 8638
--# property-call-start AdaNode.std_entity
Fld_60 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_143);
--# end
--# expr-done 8639
Bind_Result_6 := Solver.Create_Assign (Fld_59, Fld_60, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19375") else null)); 
--# expr-done 8640
--# expr-start 8642 '<Bind at ast.py:19376>' Bind_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19376
--# expr-start 8641 '<FieldAccess for type_var at ast.py:19376>' Fld_61 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19376
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_61 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8641
Fld_61.Value := No_Entity;
Entity_Vars.Reset (Fld_61);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_7 := Solver.Create_Assign (Fld_61, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19376") else null)); 
--# expr-done 8642
And_Pred_6 := Create_And (Bind_Result_6, Bind_Result_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19375") else null)); 
--# expr-done 8643
   If_Result_106 := And_Pred_6;
else
--# expr-start 8645 '<Eq at ast.py:19378>' Is_Equal_137 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19378
--# expr-start 8644 '<Symbol Has_Same_Storage at ???>' Sym_144 None
Sym_144 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Same_Storage); 
--# expr-done 8644
Is_Equal_137 := Block_Var = Sym_144; 
--# expr-done 8645
if Is_Equal_137 then
   If_Result_104 := True;
else
   --# expr-start 8647 '<Eq at ast.py:19378>' Is_Equal_138 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19378
--# expr-start 8646 '<Symbol Overlaps_Storage at ???>' Sym_145 None
Sym_145 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Overlaps_Storage); 
--# expr-done 8646
Is_Equal_138 := Block_Var = Sym_145; 
--# expr-done 8647
   If_Result_104 := Is_Equal_138;
end if;
if If_Result_104 then
   --# expr-start 8648 '<FieldAccess for storage_equation at ast.py:19379>' Fld_62 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19379
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.storage_equation
Fld_62 := Libadalang.Implementation.Attribute_Ref_P_Storage_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8648
   If_Result_105 := Fld_62;
else
   If_Result_105 := raise Property_Error with "Unhandled attribute";
end if;
      Inc_Ref (If_Result_105);
   If_Result_106 := If_Result_105;
end if;
      Inc_Ref (If_Result_106);
   If_Result_107 := If_Result_106;
end if;
      Inc_Ref (If_Result_107);
   If_Result_108 := If_Result_107;
end if;
      Inc_Ref (If_Result_108);
   If_Result_109 := If_Result_108;
end if;
      Inc_Ref (If_Result_109);
   If_Result_110 := If_Result_109;
end if;
      Inc_Ref (If_Result_110);
   If_Result_111 := If_Result_110;
end if;
      Inc_Ref (If_Result_111);
   If_Result_112 := If_Result_111;
end if;
      Inc_Ref (If_Result_112);
   If_Result_113 := If_Result_112;
end if;
      Inc_Ref (If_Result_113);
   If_Result_114 := If_Result_113;
end if;
      Inc_Ref (If_Result_114);
   If_Result_115 := If_Result_114;
end if;
      Inc_Ref (If_Result_115);
   If_Result_116 := If_Result_115;
end if;
      Inc_Ref (If_Result_116);
   If_Result_117 := If_Result_116;
end if;
      Inc_Ref (If_Result_117);
   If_Result_118 := If_Result_117;
end if;
      Inc_Ref (If_Result_118);
   If_Result_119 := If_Result_118;
end if;
      Inc_Ref (If_Result_119);
   If_Result_120 := If_Result_119;
end if;
      Inc_Ref (If_Result_120);
   If_Result_121 := If_Result_120;
end if;
      Inc_Ref (If_Result_121);
   If_Result_122 := If_Result_121;
end if;
      Inc_Ref (If_Result_122);
   If_Result_123 := If_Result_122;
end if;
      Inc_Ref (If_Result_123);
   If_Result_124 := If_Result_123;
end if;
      Inc_Ref (If_Result_124);
   If_Result_125 := If_Result_124;
end if;
      Inc_Ref (If_Result_125);
   If_Result_126 := If_Result_125;
end if;
      Inc_Ref (If_Result_126);
   If_Result_127 := If_Result_126;
end if;
      Inc_Ref (If_Result_127);
   If_Result_128 := If_Result_127;
end if;
      Inc_Ref (If_Result_128);
   If_Result_129 := If_Result_128;
end if;
      Inc_Ref (If_Result_129);
   If_Result_130 := If_Result_129;
end if;
      Inc_Ref (If_Result_130);
   If_Result_131 := If_Result_130;
end if;
      Inc_Ref (If_Result_131);
   If_Result_132 := If_Result_131;
end if;
      Inc_Ref (If_Result_132);
   If_Result_133 := If_Result_132;
end if;
      Inc_Ref (If_Result_133);
   If_Result_134 := If_Result_133;
end if;
      Inc_Ref (If_Result_134);
   If_Result_135 := If_Result_134;
end if;
      Inc_Ref (If_Result_135);
   If_Result_136 := If_Result_135;
end if;
      Inc_Ref (If_Result_136);
   If_Result_137 := If_Result_136;
end if;
      Inc_Ref (If_Result_137);
   If_Result_138 := If_Result_137;
end if;
      Inc_Ref (If_Result_138);
--# expr-done 8649
Let_Result := If_Result_138; Inc_Ref (Let_Result);
--# end
--# expr-done 8650
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1344;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1344;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Xref_Equation;
--# end
--# property-start AttributeRef.attribute_subprogram_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19384
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Attribute_Subprogram_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1345 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1345 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1345;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8657 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8656 '<And at ast.py:19390>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19390
--# expr-start 8652 '<FieldAccess for xref_no_overloading at ast.py:19390>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19390
--# expr-start 8651 '<FieldAccess for prefix at ast.py:19390>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19390
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8651
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8652
--# expr-start 8655 '<Bind at ast.py:19391>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19391
--# expr-start 8653 '<FieldAccess for ref_var at ast.py:19391>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19391
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 8653
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 8654 '<FieldAccess for attribute_subprogram at ast.py:19391>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19391
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8654
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19391") else null)); 
--# expr-done 8655
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19390") else null)); 
--# expr-done 8656
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8657
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1345;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1345;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Attribute_Subprogram_Equation;
--# end
--# property-start AttributeRef.type_class_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19394
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Type_Class_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1346 with Inline_Always;
            procedure Finalizer_Scope_2981 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1346 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1346;
            procedure Finalizer_Scope_2981 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2981;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8671 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8665 '<Cast to BaseTypeDecl at ast.py:19404>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19404
--# expr-start 8664 '<EnvGet at ast.py:19403>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19403
--# expr-start 8661 '<FieldAccess for get_unit_root_decl at ast.py:19401>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19401
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8660 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 8658 '<Symbol System at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 8658
--# expr-start 8659 '<Symbol Aux_DEC at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aux_Dec); 
--# expr-done 8659
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 8660
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 8661
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 8662 '<FieldAccess for children_env at ast.py:19403>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19403
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 8662
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2981;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8663 '<Symbol Type_Class at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Class); 
--# expr-done 8663
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 8664
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 8665
Block_Var := Cast_Result; 
--# bind typ Block_Var
--# expr-start 8670 '<And at ast.py:19407>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19407
--# expr-start 8667 '<FieldAccess for xref_equation at ast.py:19407>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19407
--# expr-start 8666 '<FieldAccess for prefix at ast.py:19407>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19407
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8666
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8667
--# expr-start 8669 '<Bind at ast.py:19407>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19407
--# expr-start 8668 '<FieldAccess for type_var at ast.py:19407>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19407
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8668
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19407") else null)); 
--# expr-done 8669
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19407") else null)); 
--# expr-done 8670
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8671
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1346;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1346;
                     Finalizer_Scope_2981;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Type_Class_Equation;
--# end
--# property-start AttributeRef.storage_pool_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19409
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Storage_Pool_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1347 with Inline_Always;
            procedure Finalizer_Scope_2982 with Inline_Always;
      Block_Var : Internal_Entity_Classwide_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1347 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1347;
            procedure Finalizer_Scope_2982 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2982;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8686 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8680 '<FieldAccess for classwide_type at ast.py:19418>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19418
--# expr-start 8679 '<Cast to BaseTypeDecl at ast.py:19418>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19418
--# expr-start 8678 '<EnvGet at ast.py:19417>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19417
--# expr-start 8675 '<FieldAccess for get_unit_root_decl at ast.py:19415>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19415
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8674 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 8672 '<Symbol System at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 8672
--# expr-start 8673 '<Symbol Storage_Pools at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Pools); 
--# expr-done 8673
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 8674
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 8675
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 8676 '<FieldAccess for children_env at ast.py:19417>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19417
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 8676
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2982;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8677 '<Symbol Root_Storage_Pool at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Storage_Pool); 
--# expr-done 8677
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 8678
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 8679
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 8680
Block_Var := Fld_2; 
--# bind typ Block_Var
--# expr-start 8685 '<And at ast.py:19421>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19421
--# expr-start 8682 '<FieldAccess for xref_equation at ast.py:19421>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19421
--# expr-start 8681 '<FieldAccess for prefix at ast.py:19421>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19421
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8681
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 8682
--# expr-start 8684 '<Bind at ast.py:19421>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19421
--# expr-start 8683 '<FieldAccess for type_var at ast.py:19421>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19421
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8683
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19421") else null)); 
--# expr-done 8684
And_Pred := Create_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19421") else null)); 
--# expr-done 8685
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8686
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1347;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1347;
                     Finalizer_Scope_2982;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Storage_Pool_Equation;
--# end
--# property-start AttributeRef.order_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19423
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Order_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1348 with Inline_Always;
            procedure Finalizer_Scope_2983 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1348 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1348;
            procedure Finalizer_Scope_2983 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2983;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8699 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8693 '<Cast to BaseTypeDecl at ast.py:19432>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19432
--# expr-start 8692 '<EnvGet at ast.py:19431>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19431
--# expr-start 8689 '<FieldAccess for get_unit_root_decl at ast.py:19429>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19429
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8688 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 8687 '<Symbol System at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 8687
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 8688
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 8689
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 8690 '<FieldAccess for children_env at ast.py:19431>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19431
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 8690
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2983;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8691 '<Symbol Bit_Order at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Bit_Order); 
--# expr-done 8691
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_1, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 8692
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 8693
Block_Var := Cast_Result; 
--# bind typ Block_Var
--# expr-start 8698 '<And at ast.py:19435>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19435
--# expr-start 8695 '<FieldAccess for xref_equation at ast.py:19435>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19435
--# expr-start 8694 '<FieldAccess for prefix at ast.py:19435>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19435
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8694
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8695
--# expr-start 8697 '<Bind at ast.py:19435>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19435
--# expr-start 8696 '<FieldAccess for type_var at ast.py:19435>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19435
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8696
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19435") else null)); 
--# expr-done 8697
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19435") else null)); 
--# expr-done 8698
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8699
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1348;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1348;
                     Finalizer_Scope_2983;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Order_Equation;
--# end
--# property-start AttributeRef.bind_to_prefix_eq /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19437
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Bind_To_Prefix_Eq
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1349 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1349 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1349;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8712 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8711 '<And at ast.py:19439>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19439
--# expr-start 8706 '<And at ast.py:19439>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19439
--# expr-start 8703 '<Bind at ast.py:19440>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19440
--# expr-start 8701 '<FieldAccess for expected_type_var at ast.py:19440>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19440
--# expr-start 8700 '<FieldAccess for prefix at ast.py:19440>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19440
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_F_Prefix;
--# expr-done 8700
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 8701
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 8702 '<FieldAccess for expected_type_var at ast.py:19440>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19440
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8702
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19440") else null)); 
--# expr-done 8703
--# expr-start 8705 '<FieldAccess for sub_equation at ast.py:19441>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19441
--# expr-start 8704 '<FieldAccess for prefix at ast.py:19441>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19441
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8704
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 8705
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19439") else null)); 
--# expr-done 8706
--# expr-start 8710 '<Bind at ast.py:19442>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19442
--# expr-start 8707 '<FieldAccess for type_var at ast.py:19442>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19442
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8707
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 8709 '<FieldAccess for type_var at ast.py:19442>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19442
--# expr-start 8708 '<FieldAccess for prefix at ast.py:19442>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19442
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8708
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 8709
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_1 := Solver.Create_Unify (Fld_5, Fld_7, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19442") else null)); 
--# expr-done 8710
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19439") else null)); 
--# expr-done 8711
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8712
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1349;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1349;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Bind_To_Prefix_Eq;
--# end
--# property-start AttributeRef.result_attr_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19445
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Result_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1350 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Env_Node : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1350 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1350;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8727 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8718 '<Cast to BasicDecl at ast.py:19452>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19452
--# expr-start 8717 '<as_entity at ast.py:19452>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19452
--# expr-start 8716 '<Find at ast.py:19450>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19450
   --# expr-start 8714 '<FieldAccess for parents at ast.py:19450>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19450
--# expr-start 8713 '<env_node at ast.py:19450>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19450
Env_Node := AST_Envs.Env_Node (Env); 
--# expr-done 8713
      if Env_Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Env_Node, True); 
--# expr-done 8714
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_267 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_267
      --# expr-start 8715 '<IsA BasicSubpDecl, BaseSubpBody at ast.py:19451>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19451
Is_A := Item_267 /= null 
and then Item_267.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 8715
      if Is_A then
         Find_Result := Item_267; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 8716
Is_Null := Find_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Find_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 8717
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 8718
Block_Var := Cast_Result; 
--# bind containing_subp Block_Var
--# expr-start 8719 '<FieldAccess for subp_spec_or_null at ast.py:19454>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19454
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 8719
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 8720 '<FieldAccess for return_type at ast.py:19455>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19455
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8720
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind returns Block_Var_1
--# expr-start 8726 '<And at ast.py:19458>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19458
--# expr-start 8722 '<Bind at ast.py:19459>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19459
--# expr-start 8721 '<FieldAccess for type_var at ast.py:19459>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19459
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8721
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19459") else null)); 
--# expr-done 8722
--# expr-start 8725 '<Bind at ast.py:19460>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19460
--# expr-start 8724 '<FieldAccess for ref_var at ast.py:19460>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19460
--# expr-start 8723 '<FieldAccess for prefix at ast.py:19460>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19460
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8723
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_3.Node);
--# end
--# expr-done 8724
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_4, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19460") else null)); 
--# expr-done 8725
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19458") else null)); 
--# expr-done 8726
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8727
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1350;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1350;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Result_Attr_Equation;
--# end
--# property-start AttributeRef.tag_attr_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19463
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Tag_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1351 with Inline_Always;
            procedure Finalizer_Scope_2986 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1351 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1351;
            procedure Finalizer_Scope_2986 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2986;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8741 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8735 '<Cast to BaseTypeDecl at ast.py:19469>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19469
--# expr-start 8734 '<EnvGet at ast.py:19468>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19468
--# expr-start 8731 '<FieldAccess for get_unit_root_decl at ast.py:19466>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19466
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8730 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 8728 '<Symbol Ada at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 8728
--# expr-start 8729 '<Symbol Tags at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tags); 
--# expr-done 8729
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 8730
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 8731
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 8732 '<FieldAccess for children_env at ast.py:19468>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19468
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 8732
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2986;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8733 '<Symbol Tag at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tag); 
--# expr-done 8733
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 8734
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 8735
Block_Var := Cast_Result; 
--# bind tag_type Block_Var
--# expr-start 8740 '<And at ast.py:19474>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19474
--# expr-start 8737 '<FieldAccess for xref_equation at ast.py:19474>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19474
--# expr-start 8736 '<FieldAccess for prefix at ast.py:19474>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19474
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8736
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8737
--# expr-start 8739 '<Bind at ast.py:19477>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19477
--# expr-start 8738 '<FieldAccess for type_var at ast.py:19477>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19477
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8738
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19477") else null)); 
--# expr-done 8739
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19474") else null)); 
--# expr-done 8740
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8741
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1351;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1351;
                     Finalizer_Scope_2986;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Tag_Attr_Equation;
--# end
--# property-start AttributeRef.address_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19480
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Address_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1352 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1352 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1352;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8755 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8754 '<And at ast.py:19485>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19485
--# expr-start 8750 '<Or at ast.py:19485>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19485
--# expr-start 8747 '<And at ast.py:19486>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19486
--# expr-start 8743 '<FieldAccess for xref_no_overloading at ast.py:19486>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19486
--# expr-start 8742 '<FieldAccess for prefix at ast.py:19486>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19486
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8742
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8743
--# expr-start 8746 '<Predicate on BasicDecl.is_subprogram at ast.py:19487>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19487
--# expr-start 8745 '<FieldAccess for ref_var at ast.py:19487>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19487
--# expr-start 8744 '<FieldAccess for prefix at ast.py:19487>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19487
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8744
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 8745
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Pred := Solver.Create_Predicate (Fld_3, Create_Basic_Decl_P_Is_Subprogram_0_Predicate); 
--# expr-done 8746
And_Pred := Create_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19486") else null)); 
--# expr-done 8747
--# expr-start 8749 '<FieldAccess for sub_equation at ast.py:19489>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19489
--# expr-start 8748 '<FieldAccess for prefix at ast.py:19489>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19489
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8748
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 8749
Or_Pred := Create_Or (And_Pred, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19485") else null)); 
--# expr-done 8750
--# expr-start 8753 '<Bind at ast.py:19490>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19490
--# expr-start 8751 '<FieldAccess for type_var at ast.py:19490>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19490
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8751
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 8752 '<FieldAccess for system_address_type at ast.py:19490>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19490
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.system_address_type
Fld_7 := Libadalang.Implementation.Ada_Node_P_System_Address_Type (Node => Ent.Node);
--# end
--# expr-done 8752
      Cast_Result := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Bind_Result := Solver.Create_Assign (Fld_6, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19490") else null)); 
--# expr-done 8753
And_Pred_1 := Create_And (Or_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19485") else null)); 
--# expr-done 8754
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8755
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1352;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1352;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Address_Equation;
--# end
--# property-start AttributeRef.identity_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19492
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Identity_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1353 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1353 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1353;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8763 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8762 '<And at ast.py:19496>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19496
--# expr-start 8757 '<FieldAccess for sub_equation at ast.py:19497>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19497
--# expr-start 8756 '<FieldAccess for prefix at ast.py:19497>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19497
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8756
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8757
--# expr-start 8761 '<Bind at ast.py:19498>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19498
--# expr-start 8759 '<FieldAccess for ref_var at ast.py:19498>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19498
--# expr-start 8758 '<FieldAccess for prefix at ast.py:19498>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19498
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8758
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 8759
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 8760 '<FieldAccess for type_var at ast.py:19498>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19498
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8760
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_Propagate (Fld_3, Fld_4, Logic_Functor_218'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19498") else null)); 
--# expr-done 8761
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19496") else null)); 
--# expr-done 8762
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8763
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1353;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1353;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Identity_Equation;
--# end
--# property-start AttributeRef.universal_real_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19502
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Universal_Real_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1354 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1354 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1354;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8769 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8768 '<And at ast.py:19505>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19505
--# expr-start 8765 '<FieldAccess for universal_real_bind at ast.py:19505>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19505
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8764 '<FieldAccess for type_var at ast.py:19505>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19505
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8764
--# property-call-start AdaNode.universal_real_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 8765
--# expr-start 8767 '<FieldAccess for sub_equation at ast.py:19506>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19506
--# expr-start 8766 '<FieldAccess for prefix at ast.py:19506>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19506
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8766
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8767
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19505") else null)); 
--# expr-done 8768
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8769
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1354;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1354;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Universal_Real_Equation;
--# end
--# property-start AttributeRef.universal_int_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19509
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Universal_Int_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1355 with Inline_Always;
            procedure Finalizer_Scope_2987 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Item_268 : Internal_Entity_Basic_Assoc;
Bare_Item_268 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1355 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1355;
            procedure Finalizer_Scope_2987 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2987;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8790 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8771 '<FieldAccess for name_designated_type at ast.py:19511>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19511
--# expr-start 8770 '<FieldAccess for prefix at ast.py:19511>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19511
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8770
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8771
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 8789 '<And at ast.py:19514>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19514
--# expr-start 8776 '<And at ast.py:19514>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19514
--# expr-start 8773 '<FieldAccess for sub_equation at ast.py:19514>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19514
--# expr-start 8772 '<FieldAccess for prefix at ast.py:19514>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19514
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8772
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8773
--# expr-start 8775 '<FieldAccess for universal_int_bind at ast.py:19515>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19515
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8774 '<FieldAccess for type_var at ast.py:19515>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19515
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8774
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 8775
And_Pred := Create_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19514") else null)); 
--# expr-done 8776
--# expr-start 8788 '<LogicAll at ast.py:19516>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19516
--# expr-start 8787 '<Map at ast.py:19516>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19516
   --# expr-start 8777 '<FieldAccess for args at ast.py:19516>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19516
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 8777
Fld_7 := Fld_6.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_7 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_7;
      begin
         for Untyped_Item_268 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_268 := Untyped_Item_268; 
Is_Null := Bare_Item_268 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_268); 
   If_Result := As_Entity;
end if;
                  Item_268 := If_Result; 
   --# scope-start
               --# bind arg Item_268
      --# expr-start 8786 '<And at ast.py:19518>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19518
--# expr-start 8783 '<And at ast.py:19518>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19518
--# expr-start 8780 '<Bind at ast.py:19518>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19518
--# expr-start 8779 '<FieldAccess for expected_type_var at ast.py:19518>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19518
--# expr-start 8778 '<FieldAccess for expr at ast.py:19518>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19518
      if Item_268.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_268.Node, E_Info => Item_268.Info);
--# end
--# expr-done 8778
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_9.Node);
--# expr-done 8779
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_10, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19518") else null)); 
--# expr-done 8780
--# expr-start 8782 '<FieldAccess for sub_equation at ast.py:19519>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19519
--# expr-start 8781 '<FieldAccess for expr at ast.py:19519>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19519
      if Item_268.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_268.Node, E_Info => Item_268.Info);
--# end
--# expr-done 8781
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 8782
And_Pred_1 := Create_And (Bind_Result, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19518") else null)); 
--# expr-done 8783
--# expr-start 8785 '<FieldAccess for matches_expected_type at ast.py:19520>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19520
--# expr-start 8784 '<FieldAccess for expr at ast.py:19520>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19520
      if Item_268.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_268.Node, E_Info => Item_268.Info);
--# end
--# expr-done 8784
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_14 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_13.Node, Origin => Origin);
--# end
--# expr-done 8785
And_Pred_2 := Create_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19518") else null)); 
--# expr-done 8786
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2987;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8787
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19516") else null)); 
--# expr-done 8788
And_Pred_3 := Create_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19514") else null)); 
--# expr-done 8789
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 8790
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1355;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1355;
                     Finalizer_Scope_2987;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Universal_Int_Equation;
--# end
--# property-start AttributeRef.image_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19524
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Image_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1356 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1356 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1356;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8806 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8792 '<FieldAccess for name_designated_type at ast.py:19526>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19526
--# expr-start 8791 '<FieldAccess for prefix at ast.py:19526>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19526
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8791
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8792
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 8805 '<If at ast.py:19527>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19527
--# expr-start 8793 '<IsNull at ast.py:19528>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19528
Is_Null := Block_Var.Node = null; 
--# expr-done 8793
if Is_Null then
   --# expr-start 8798 '<And at ast.py:19531>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19531
--# expr-start 8795 '<FieldAccess for sub_equation at ast.py:19531>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19531
--# expr-start 8794 '<FieldAccess for prefix at ast.py:19531>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19531
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8794
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8795
--# expr-start 8797 '<Bind at ast.py:19532>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19532
--# expr-start 8796 '<FieldAccess for type_var at ast.py:19532>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19532
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8796
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_Assign (Fld_4, Str_Type, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19532") else null)); 
--# expr-done 8797
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19531") else null)); 
--# expr-done 8798
   If_Result := And_Pred;
else
   --# expr-start 8804 '<And at ast.py:19534>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19534
--# expr-start 8800 '<FieldAccess for xref_no_overloading at ast.py:19534>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19534
--# expr-start 8799 '<FieldAccess for prefix at ast.py:19534>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19534
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8799
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_6 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 8800
--# expr-start 8803 '<Bind at ast.py:19535>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19535
--# expr-start 8801 '<FieldAccess for ref_var at ast.py:19535>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19535
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 8801
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 8802 '<FieldAccess for attribute_subprogram at ast.py:19535>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19535
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_8 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8802
      Cast_Result := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_7, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19535") else null)); 
--# expr-done 8803
And_Pred_1 := Create_And (Fld_6, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19534") else null)); 
--# expr-done 8804
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 8805
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8806
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1356;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1356;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Image_Equation;
--# end
--# property-start AttributeRef.img_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19538
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Img_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1357 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1357 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1357;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8812 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8811 '<And at ast.py:19542>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19542
--# expr-start 8808 '<FieldAccess for xref_equation at ast.py:19542>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19542
--# expr-start 8807 '<FieldAccess for prefix at ast.py:19542>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19542
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8807
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8808
--# expr-start 8810 '<Bind at ast.py:19545>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19545
--# expr-start 8809 '<FieldAccess for type_var at ast.py:19545>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19545
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8809
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Assign (Fld_2, Str_Type, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19545") else null)); 
--# expr-done 8810
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19542") else null)); 
--# expr-done 8811
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8812
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1357;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1357;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Img_Equation;
--# end
--# property-start AttributeRef.enum_rep_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19548
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Enum_Rep_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1358 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1358 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1358;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8829 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8814 '<FieldAccess for name_designated_type at ast.py:19550>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19550
--# expr-start 8813 '<FieldAccess for prefix at ast.py:19550>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19550
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8813
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8814
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 8828 '<If at ast.py:19552>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19552
--# expr-start 8815 '<IsNull at ast.py:19553>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19553
Is_Null := Block_Var.Node = null; 
--# expr-done 8815
if Is_Null then
   --# expr-start 8821 '<And at ast.py:19556>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19556
--# expr-start 8817 '<FieldAccess for sub_equation at ast.py:19556>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19556
--# expr-start 8816 '<FieldAccess for prefix at ast.py:19556>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19556
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8816
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8817
--# expr-start 8820 '<Bind at ast.py:19557>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19557
--# expr-start 8818 '<FieldAccess for type_var at ast.py:19557>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19557
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8818
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 8819 '<FieldAccess for universal_int_type at ast.py:19557>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19557
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 8819
Bind_Result := Solver.Create_Assign (Fld_4, Fld_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19557") else null)); 
--# expr-done 8820
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19556") else null)); 
--# expr-done 8821
   If_Result := And_Pred;
else
   --# expr-start 8827 '<And at ast.py:19559>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19559
--# expr-start 8823 '<FieldAccess for xref_no_overloading at ast.py:19559>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19559
--# expr-start 8822 '<FieldAccess for prefix at ast.py:19559>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19559
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8822
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_7 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 8823
--# expr-start 8826 '<Bind at ast.py:19560>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19560
--# expr-start 8824 '<FieldAccess for ref_var at ast.py:19560>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19560
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 8824
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 8825 '<FieldAccess for attribute_subprogram at ast.py:19560>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19560
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_9 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8825
      Cast_Result := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_8, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19560") else null)); 
--# expr-done 8826
And_Pred_1 := Create_And (Fld_7, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19559") else null)); 
--# expr-done 8827
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 8828
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8829
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1358;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1358;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Enum_Rep_Equation;
--# end
--# property-start AttributeRef.self_type_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19563
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Self_Type_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1359 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1359 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1359;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8837 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8831 '<FieldAccess for name_designated_type at ast.py:19569>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19569
--# expr-start 8830 '<FieldAccess for prefix at ast.py:19569>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19569
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8830
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8831
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 8836 '<And at ast.py:19570>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19570
--# expr-start 8833 '<FieldAccess for xref_no_overloading at ast.py:19571>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19571
--# expr-start 8832 '<FieldAccess for prefix at ast.py:19571>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19571
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8832
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8833
--# expr-start 8835 '<Bind at ast.py:19572>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19572
--# expr-start 8834 '<FieldAccess for type_var at ast.py:19572>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19572
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8834
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19572") else null)); 
--# expr-done 8835
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19570") else null)); 
--# expr-done 8836
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8837
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1359;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1359;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Self_Type_Equation;
--# end
--# property-start AttributeRef.access_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19575
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Access_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1360 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Bare_Name;
Fld_13 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_14 : Bare_Name;
Fld_15 : Logic_Var;
Fld_16 : Bare_Name;
Fld_17 : Logic_Var;
Bind_Result_3 : Logic_Equation;
Fld_18 : Bare_Name;
Fld_19 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_4 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_20 : Internal_Entity_Name;
Fld_21 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_22 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
Fld_23 : Bare_Name;
Fld_24 : Logic_Var;
Fld_25 : Logic_Var;
Bind_Result_6 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1360 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Or_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1360;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8882 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8881 '<Or at ast.py:19577>' Or_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19577
--# expr-start 8848 '<And at ast.py:19579>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19579
--# expr-start 8844 '<And at ast.py:19579>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19579
--# expr-start 8839 '<FieldAccess for xref_no_overloading at ast.py:19579>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19579
--# expr-start 8838 '<FieldAccess for prefix at ast.py:19579>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19579
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8838
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8839
--# expr-start 8843 '<Predicate on BaseTypeDecl.is_subp_access_of at ast.py:19580>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19580
--# expr-start 8840 '<FieldAccess for type_var at ast.py:19581>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19581
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8840
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 8842 '<FieldAccess for ref_var at ast.py:19582>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19582
--# expr-start 8841 '<FieldAccess for prefix at ast.py:19582>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19582
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8841
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_3);
--# end
--# expr-done 8842
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_N_Predicate ((Fld_2, Fld_4), Create_Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate (Origin)); 
--# expr-done 8843
And_Pred := Create_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19579") else null)); 
--# expr-done 8844
--# expr-start 8847 '<Bind at ast.py:19583>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19583
--# expr-start 8845 '<FieldAccess for type_var at ast.py:19583>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19583
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8845
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 8846 '<FieldAccess for expected_type_var at ast.py:19583>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19583
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8846
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Bind_Result := Solver.Create_Unify (Fld_5, Fld_6, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19583") else null)); 
--# expr-done 8847
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19579") else null)); 
--# expr-done 8848
--# expr-start 8880 '<And at ast.py:19586>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19586
--# expr-start 8850 '<FieldAccess for sub_equation at ast.py:19586>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19586
--# expr-start 8849 '<FieldAccess for prefix at ast.py:19586>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19586
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8849
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 8850
--# expr-start 8879 '<Or at ast.py:19587>' Or_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19587
--# expr-start 8871 '<And at ast.py:19597>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19597
--# expr-start 8858 '<And at ast.py:19597>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19597
--# expr-start 8853 '<Bind at ast.py:19597>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19597
--# expr-start 8851 '<FieldAccess for expected_type_var at ast.py:19597>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19597
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8851
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 8852 '<FieldAccess for type_var at ast.py:19597>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19597
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8852
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Bind_Result_1 := Solver.Create_Unify (Fld_9, Fld_10, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19597") else null)); 
--# expr-done 8853
--# expr-start 8857 '<Bind at ast.py:19598>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19598
--# expr-start 8854 '<FieldAccess for expected_type_var at ast.py:19598>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19598
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8854
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 8856 '<FieldAccess for expected_type_var at ast.py:19598>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19598
--# expr-start 8855 '<FieldAccess for prefix at ast.py:19598>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19598
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8855
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_12);
--# expr-done 8856
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
Bind_Result_2 := Solver.Create_Propagate (Fld_11, Fld_13, Logic_Functor_669'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19598") else null)); 
--# expr-done 8857
And_Pred_2 := Create_And (Bind_Result_1, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19597") else null)); 
--# expr-done 8858
--# expr-start 8870 '<Or at ast.py:19600>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19600
--# expr-start 8867 '<And at ast.py:19605>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19605
--# expr-start 8863 '<Bind at ast.py:19605>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19605
--# expr-start 8860 '<FieldAccess for expected_type_var at ast.py:19605>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19605
--# expr-start 8859 '<FieldAccess for prefix at ast.py:19605>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19605
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8859
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_14);
--# expr-done 8860
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
--# expr-start 8862 '<FieldAccess for type_var at ast.py:19605>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19605
--# expr-start 8861 '<FieldAccess for prefix at ast.py:19605>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19605
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8861
      if Fld_16 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_16);
--# expr-done 8862
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
Bind_Result_3 := Solver.Create_Unify (Fld_15, Fld_17, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19605") else null)); 
--# expr-done 8863
--# expr-start 8866 '<Bind at ast.py:19606>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19606
--# expr-start 8865 '<FieldAccess for type_var at ast.py:19606>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19606
--# expr-start 8864 '<FieldAccess for prefix at ast.py:19606>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19606
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8864
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_18);
--# expr-done 8865
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_19, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19606") else null)); 
--# expr-done 8866
And_Pred_3 := Create_And (Bind_Result_3, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19605") else null)); 
--# expr-done 8867
--# expr-start 8869 '<FieldAccess for matches_expected_formal_type at ast.py:19610>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19610
--# expr-start 8868 '<FieldAccess for prefix at ast.py:19610>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19610
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8868
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_21 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_20.Node, Origin => Origin);
--# end
--# expr-done 8869
Or_Pred := Create_Or (And_Pred_3, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19600") else null)); 
--# expr-done 8870
And_Pred_4 := Create_And (And_Pred_2, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19597") else null)); 
--# expr-done 8871
--# expr-start 8878 '<And at ast.py:19615>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19615
--# expr-start 8873 '<Bind at ast.py:19615>' Bind_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19615
--# expr-start 8872 '<FieldAccess for expected_type_var at ast.py:19615>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19615
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8872
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
Cast_Expr_1 := No_Entity_Base_Type_Decl; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_22, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19615") else null)); 
--# expr-done 8873
--# expr-start 8877 '<Bind at ast.py:19616>' Bind_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19616
--# expr-start 8875 '<FieldAccess for type_var at ast.py:19616>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19616
--# expr-start 8874 '<FieldAccess for prefix at ast.py:19616>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19616
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8874
      if Fld_23 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_23);
--# expr-done 8875
Fld_24.Value := No_Entity;
Entity_Vars.Reset (Fld_24);
--# expr-start 8876 '<FieldAccess for type_var at ast.py:19617>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19617
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8876
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
Bind_Result_6 := Solver.Create_Propagate (Fld_24, Fld_25, Logic_Functor_616'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19616") else null)); 
--# expr-done 8877
And_Pred_5 := Create_And (Bind_Result_5, Bind_Result_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19615") else null)); 
--# expr-done 8878
Or_Pred_1 := Create_Or (And_Pred_4, And_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19587") else null)); 
--# expr-done 8879
And_Pred_6 := Create_And (Fld_8, Or_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19586") else null)); 
--# expr-done 8880
Or_Pred_2 := Create_Or (And_Pred_1, And_Pred_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19577") else null)); 
--# expr-done 8881
Let_Result := Or_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 8882
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1360;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1360;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Access_Equation;
--# end
--# property-start AttributeRef.size_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19622
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Size_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1361 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1361 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1361;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8899 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8884 '<FieldAccess for name_designated_type at ast.py:19624>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19624
--# expr-start 8883 '<FieldAccess for prefix at ast.py:19624>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19624
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8883
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8884
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 8898 '<If at ast.py:19625>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19625
--# expr-start 8886 '<Not at ast.py:19626>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19626
--# expr-start 8885 '<IsNull at ast.py:19626>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19626
Is_Null := Block_Var.Node = null; 
--# expr-done 8885
Not_Val := not (Is_Null); 
--# expr-done 8886
if Not_Val then
   --# expr-start 8892 '<And at ast.py:19628>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19628
--# expr-start 8889 '<Bind at ast.py:19628>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19628
--# expr-start 8888 '<FieldAccess for ref_var at ast.py:19628>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19628
--# expr-start 8887 '<FieldAccess for prefix at ast.py:19628>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19628
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 8887
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 8888
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19628") else null)); 
--# expr-done 8889
--# expr-start 8891 '<FieldAccess for universal_int_bind at ast.py:19629>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19629
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8890 '<FieldAccess for type_var at ast.py:19629>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19629
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8890
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 8891
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19628") else null)); 
--# expr-done 8892
   If_Result := And_Pred;
else
   --# expr-start 8897 '<And at ast.py:19631>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19631
--# expr-start 8894 '<FieldAccess for sub_equation at ast.py:19631>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19631
--# expr-start 8893 '<FieldAccess for prefix at ast.py:19631>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19631
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8893
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 8894
--# expr-start 8896 '<FieldAccess for universal_int_bind at ast.py:19632>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19632
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8895 '<FieldAccess for type_var at ast.py:19632>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19632
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8895
--# property-call-start AdaNode.universal_int_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 8896
And_Pred_1 := Create_And (Fld_7, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19631") else null)); 
--# expr-done 8897
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 8898
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8899
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1361;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1361;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Size_Equation;
--# end
--# property-start AttributeRef.array_attr_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19635
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Array_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1362 with Inline_Always;
            procedure Finalizer_Scope_2989 with Inline_Always;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Integer;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Basic_Assoc;
Unused_2 : Boolean;
Res : Boolean;
Pfx_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Fld_8 : Boolean;
Fld_9 : Big_Integer_Type;
Small_Int : Integer;
Let_Result : Integer;
Scope_Result_1 : Integer;
Result_Var_1 : Integer;
Arith_Result : Integer;
Fld_10 : Internal_Entity_Name;
Fld_11 : Logic_Equation;
Fld_12 : Boolean;
If_Result_1 : Boolean;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
Fld_15 : Boolean;
Fld_16 : Logic_Var;
Fld_17 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_18 : Boolean;
Fld_19 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_20 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
False_Rel : Logic_Equation;
If_Result_4 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_21 : Internal_Entity_Name;
Fld_22 : Internal_Entity_Name;
Fld_23 : Logic_Var;
Pred : Logic_Equation;
Fld_24 : Boolean;
Fld_25 : Internal_Entity_Name;
Fld_26 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_27 : Logic_Var;
Fld_28 : Logic_Equation;
Fld_29 : Logic_Var;
Fld_30 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
If_Result_7 : Logic_Equation;
False_Rel_1 : Logic_Equation;
If_Result_8 : Logic_Equation;
Let_Result_1 : Logic_Equation;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_9 : Logic_Equation;
Let_Result_2 : Logic_Equation;
            procedure Finalizer_Scope_1362 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_28);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1362;
            procedure Finalizer_Scope_2989 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2989;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8947 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 8902 '<FieldAccess for name_is at ast.py:19637>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19637
--# expr-start 8900 '<FieldAccess for attribute at ast.py:19637>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19637
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8900
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8901 '<Symbol Length at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 8901
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 8902
Block_Var := Fld_1; 
--# bind is_length Block_Var
--# expr-start 8904 '<FieldAccess for name_designated_type at ast.py:19638>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19638
--# expr-start 8903 '<FieldAccess for prefix at ast.py:19638>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19638
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8903
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 8904
Block_Var_1 := Fld_3; 
--# bind typ Block_Var_1
--# expr-start 8911 '<Arithmetic - at ast.py:19642>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19642
--# expr-start 8905 '<FieldAccess for args at ast.py:19642>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19642
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 8905
Fld_5 := Fld_4.Node;
Get_Result := Get (Self, Fld_5, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 8906 '<FieldAccess for expr at ast.py:19642>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19642
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 8906
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr
--# expr-start 8910 '<Let at ast.py:19643>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19643
--# scope-start
--# expr-start 8907 '<FieldAccess for resolve_names_internal at ast.py:19644>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19644
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_8 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8907
Unused_2 := Fld_8; 
--# bind _ Unused_2
--# expr-start 8909 '<as_int at ast.py:19645>' Small_Int /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19645
--# expr-start 8908 '<FieldAccess for eval_as_int at ast.py:19645>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19645
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_9 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 8908
Small_Int := To_Integer (Self, Fld_9); 
--# expr-done 8909
Let_Result := Small_Int; 
--# end
--# expr-done 8910
Scope_Result_1 := Let_Result;
   --# end
      Finalizer_Scope_2989;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := 1;
end if;
Arith_Result := (Result_Var_1 - 1); 
--# expr-done 8911
Block_Var_2 := Arith_Result; 
--# bind dim Block_Var_2
--# expr-start 8946 '<If at ast.py:19648>' If_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19648
--# expr-start 8913 '<Not at ast.py:19649>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19649
--# expr-start 8912 '<IsNull at ast.py:19649>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19649
Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 8912
Not_Val_1 := not (Is_Null_1); 
--# expr-done 8913
if Not_Val_1 then
   --# expr-start 8929 '<And at ast.py:19652>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19652
--# expr-start 8915 '<FieldAccess for xref_no_overloading at ast.py:19652>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19652
--# expr-start 8914 '<FieldAccess for prefix at ast.py:19652>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19652
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8914
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_11 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 8915
--# expr-start 8928 '<Cond at ast.py:19652>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19652
--# expr-start 8916 '<FieldAccess for is_array_def_with_deref at ast.py:19653>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19653
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_array_def_with_deref
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Is_Array_Def_With_Deref (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 8916
if Fld_12 then
   If_Result_1 := Block_Var;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 8918 '<FieldAccess for universal_int_bind at ast.py:19654>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19654
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8917 '<FieldAccess for type_var at ast.py:19654>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19654
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8917
--# property-call-start AdaNode.universal_int_bind
Fld_14 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_13, Origin => Origin);
--# end
--# expr-done 8918
   If_Result_6 := Fld_14;
else
--# expr-start 8919 '<FieldAccess for is_array_def_with_deref at ast.py:19657>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19657
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_array_def_with_deref
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Is_Array_Def_With_Deref (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 8919
if Fld_15 then
   --# expr-start 8922 '<Bind at ast.py:19658>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19658
--# expr-start 8920 '<FieldAccess for type_var at ast.py:19658>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19658
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8920
Fld_16.Value := No_Entity;
Entity_Vars.Reset (Fld_16);
--# expr-start 8921 '<FieldAccess for index_type at ast.py:19658>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19658
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_17 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Block_Var_1.Node, Dim => Block_Var_2, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 8921
      Cast_Result := Create_Internal_Entity
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
Bind_Result := Solver.Create_Assign (Fld_16, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19658") else null)); 
--# expr-done 8922
   If_Result_5 := Bind_Result;
else
--# expr-start 8923 '<FieldAccess for is_discrete_type at ast.py:19661>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19661
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_18 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 8923
if Fld_18 then
   If_Result_3 := True;
else
--# expr-start 8924 '<FieldAccess for is_real_type at ast.py:19661>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19661
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_19 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 8924
if Fld_19 then
   --# expr-start 8925 '<Not at ast.py:19661>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19661
Not_Val := not (Block_Var); 
--# expr-done 8925
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   --# expr-start 8927 '<Bind at ast.py:19662>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19662
--# expr-start 8926 '<FieldAccess for type_var at ast.py:19662>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19662
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8926
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_20, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19662") else null)); 
--# expr-done 8927
   If_Result_4 := Bind_Result_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19664") else null)); 
   If_Result_4 := False_Rel;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 8928
And_Pred := Create_And (Fld_11, If_Result_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19652") else null)); 
--# expr-done 8929
   If_Result_9 := And_Pred;
else
   --# expr-start 8945 '<Let at ast.py:19669>' Let_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19669
--# scope-start
--# expr-start 8934 '<FieldAccess for resolve_names_internal_with_eq at ast.py:19671>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19671
--# expr-start 8930 '<FieldAccess for prefix at ast.py:19671>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19671
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8930
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8933 '<Predicate on BaseTypeDecl.is_array_def_with_deref at ast.py:19672>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19672
--# expr-start 8932 '<FieldAccess for type_var at ast.py:19673>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19673
--# expr-start 8931 '<FieldAccess for prefix at ast.py:19673>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19673
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8931
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_22.Node);
--# expr-done 8932
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
Pred := Solver.Create_Predicate (Fld_23, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin)); 
--# expr-done 8933
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_24 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_21.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld_21.Info);
--# end
--# expr-done 8934
Res := Fld_24; 
--# bind res Res
--# expr-start 8937 '<Cast to BaseTypeDecl at ast.py:19675>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19675
--# expr-start 8936 '<FieldAccess for type_val at ast.py:19675>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19675
--# expr-start 8935 '<FieldAccess for prefix at ast.py:19675>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19675
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8935
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_26 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_25.Node);
--# end
--# expr-done 8936
   if Fld_26.Node = null
      or else Fld_26.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_26.Node,
         Info => Fld_26.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 8937
Pfx_Typ := Cast_Result_2; 
--# bind pfx_typ Pfx_Typ
--# expr-start 8944 '<If at ast.py:19677>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19677
if Res then
   --# expr-start 8943 '<If at ast.py:19679>' If_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19679
if Block_Var then
   --# expr-start 8939 '<FieldAccess for universal_int_bind at ast.py:19681>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19681
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8938 '<FieldAccess for type_var at ast.py:19681>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19681
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8938
--# property-call-start AdaNode.universal_int_bind
Fld_28 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_27, Origin => Origin);
--# end
--# expr-done 8939
   If_Result_7 := Fld_28;
else
   --# expr-start 8942 '<Bind at ast.py:19682>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19682
--# expr-start 8940 '<FieldAccess for type_var at ast.py:19682>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19682
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8940
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 8941 '<FieldAccess for index_type at ast.py:19682>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19682
      if Pfx_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_30 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Pfx_Typ.Node, Dim => Block_Var_2, Origin => Origin, E_Info => Pfx_Typ.Info);
--# end
--# expr-done 8941
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_30.Node,
         Info => Fld_30.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_29, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19682") else null)); 
--# expr-done 8942
   If_Result_7 := Bind_Result_2;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 8943
   If_Result_8 := If_Result_7;
else
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19684") else null)); 
   If_Result_8 := False_Rel_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 8944
Let_Result_1 := If_Result_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 8945
   If_Result_9 := Let_Result_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 8946
Let_Result_2 := If_Result_9; Inc_Ref (Let_Result_2);
--# end
--# expr-done 8947
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1362;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1362;
                     Finalizer_Scope_2989;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Array_Attr_Equation;
--# end
--# property-start AttributeRef.subtype_attr_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19689
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Subtype_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1363 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1363 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1363;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8956 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8955 '<And at ast.py:19696>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19696
--# expr-start 8952 '<Bind at ast.py:19696>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19696
--# expr-start 8949 '<FieldAccess for ref_var at ast.py:19696>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19696
--# expr-start 8948 '<FieldAccess for prefix at ast.py:19696>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19696
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_F_Prefix;
--# expr-done 8948
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 8949
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 8951 '<FieldAccess for name_designated_type at ast.py:19696>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19696
--# expr-start 8950 '<FieldAccess for prefix at ast.py:19696>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19696
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8950
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 8951
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19696") else null)); 
--# expr-done 8952
--# expr-start 8954 '<FieldAccess for universal_int_bind at ast.py:19697>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19697
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8953 '<FieldAccess for type_var at ast.py:19697>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19697
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8953
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 8954
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19696") else null)); 
--# expr-done 8955
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8956
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1363;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1363;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Subtype_Attr_Equation;
--# end
--# property-start AttributeRef.to_address_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19700
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_To_Address_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1364 with Inline_Always;
            procedure Finalizer_Scope_2990 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Subp_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Subp_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Subp_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1364 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1364;
            procedure Finalizer_Scope_2990 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2990;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8970 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8964 '<Cast to BasicSubpDecl at ast.py:19712>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19712
--# expr-start 8963 '<EnvGet at ast.py:19710>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19710
--# expr-start 8960 '<FieldAccess for get_unit_root_decl at ast.py:19708>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19708
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8959 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 8957 '<Symbol System at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 8957
--# expr-start 8958 '<Symbol Storage_Elements at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Elements); 
--# expr-done 8958
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 8959
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 8960
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 8961 '<FieldAccess for children_env at ast.py:19710>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19710
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 8961
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2990;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8962 '<Symbol To_Address at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_To_Address); 
--# expr-done 8962
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 8963
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Subp_Decl;
   end if;
--# expr-done 8964
Block_Var := Cast_Result; 
--# bind to_address_subp Block_Var
--# expr-start 8969 '<And at ast.py:19714>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19714
--# expr-start 8966 '<FieldAccess for sub_equation at ast.py:19715>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19715
--# expr-start 8965 '<FieldAccess for prefix at ast.py:19715>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19715
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8965
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8966
--# expr-start 8968 '<Bind at ast.py:19716>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19716
--# expr-start 8967 '<FieldAccess for ref_var at ast.py:19716>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19716
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 8967
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19716") else null)); 
--# expr-done 8968
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19714") else null)); 
--# expr-done 8969
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8970
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1364;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1364;
                     Finalizer_Scope_2990;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_To_Address_Equation;
--# end
--# property-start AttributeRef.index_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19719
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Index_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1365 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Entry_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1365 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1365;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8981 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8975 '<FieldAccess for family_type at ast.py:19726>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19726
--# expr-start 8974 '<Cast to EntryDecl at ast.py:19726>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19726
--# expr-start 8973 '<EnvGet at ast.py:19725>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19725
--# expr-start 8972 '<FieldAccess for name_symbol at ast.py:19725>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19725
--# expr-start 8971 '<FieldAccess for prefix at ast.py:19725>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19725
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8971
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8972
Env_Get_Result := AST_Envs.Get_First (Self => Env, Key => Fld_1, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 8973
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8974
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.family_type
Fld_2 := Libadalang.Implementation.Entry_Decl_P_Family_Type (Node => Cast_Result.Node, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 8975
Block_Var := Fld_2; 
--# bind typ Block_Var
--# expr-start 8980 '<And at ast.py:19728>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19728
--# expr-start 8977 '<FieldAccess for sub_equation at ast.py:19728>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19728
--# expr-start 8976 '<FieldAccess for prefix at ast.py:19728>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19728
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8976
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 8977
--# expr-start 8979 '<Bind at ast.py:19728>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19728
--# expr-start 8978 '<FieldAccess for type_var at ast.py:19728>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19728
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8978
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19728") else null)); 
--# expr-done 8979
And_Pred := Create_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19728") else null)); 
--# expr-done 8980
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8981
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1365;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1365;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Index_Equation;
--# end
--# property-start AttributeRef.storage_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19730
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Storage_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1366 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1366 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1366;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8991 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8990 '<And at ast.py:19738>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19738
--# expr-start 8987 '<And at ast.py:19738>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19738
--# expr-start 8983 '<FieldAccess for sub_equation at ast.py:19738>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19738
--# expr-start 8982 '<FieldAccess for prefix at ast.py:19738>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19738
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8982
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8983
--# expr-start 8986 '<Bind at ast.py:19740>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19740
--# expr-start 8984 '<FieldAccess for type_var at ast.py:19740>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19740
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8984
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 8985 '<FieldAccess for bool_type at ast.py:19740>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19740
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 8985
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19740") else null)); 
--# expr-done 8986
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19738") else null)); 
--# expr-done 8987
--# expr-start 8989 '<FieldAccess for sub_equation at ast.py:19742>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19742
--# expr-start 8988 '<FieldAccess for args at ast.py:19742>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19742
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 8988
Fld_5 := Fld_4.Node;
Get_Result := Get (Self, Fld_5, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result.Node, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 8989
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19738") else null)); 
--# expr-done 8990
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8991
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1366;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1366;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Storage_Equation;
--# end
--# property-start AttributeRef.called_subp_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19745
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Called_Subp_Spec
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Block_Var : Symbol_Type;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Result_Var : Internal_Entity_Base_Subp_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Holder;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9005 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 8993 '<FieldAccess for name_symbol at ast.py:19747>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19747
--# expr-start 8992 '<FieldAccess for attribute at ast.py:19747>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19747
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8992
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8993
Block_Var := Fld_1; 
--# bind rel_name Block_Var
--# expr-start 9004 '<If at ast.py:19748>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19748
--# expr-start 8995 '<Eq at ast.py:19749>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19749
--# expr-start 8994 '<Symbol Image at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Image); 
--# expr-done 8994
Is_Equal := Block_Var = Sym; 
--# expr-done 8995
if Is_Equal then
   If_Result := True;
else
   --# expr-start 8997 '<Eq at ast.py:19749>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19749
--# expr-start 8996 '<Symbol Wide_Image at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Image); 
--# expr-done 8996
Is_Equal_1 := Block_Var = Sym_1; 
--# expr-done 8997
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 8999 '<Eq at ast.py:19749>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19749
--# expr-start 8998 '<Symbol Wide_Wide_Image at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Image); 
--# expr-done 8998
Is_Equal_2 := Block_Var = Sym_2; 
--# expr-done 8999
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
--# expr-start 9001 '<FieldAccess for expression_type at ast.py:19750>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19750
--# expr-start 9000 '<FieldAccess for prefix at ast.py:19750>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19750
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9000
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 9001
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 9003 '<FieldAccess for subp_spec_or_null at ast.py:19752>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19752
--# expr-start 9002 '<FieldAccess for attribute_subprogram_for_type at ast.py:19752>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19752
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram_for_type
Fld_4 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram_For_Type (Node => Ent.Node, Typ => Var_Expr, E_Info => Ent.Info);
--# end
--# expr-done 9002
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 9003
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Subp_Spec;
end if;
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_2 := Cast_Result;
else
   If_Result_2 := No_Entity_Base_Formal_Param_Holder;
end if;
--# expr-done 9004
Let_Result := If_Result_2; 
--# end
--# expr-done 9005
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Called_Subp_Spec;
--# end
   --
   --  Primitives for Bare_Call_Expr
   --
      procedure Initialize_Fields_For_Call_Expr
        (Self : Bare_Call_Expr
         ; Call_Expr_F_Name : Bare_Name
         ; Call_Expr_F_Suffix : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Call_Expr_F_Name := Call_Expr_F_Name;
            Self.Call_Expr_F_Suffix := Call_Expr_F_Suffix;
      Self.Call_Expr_R_Called_Spec := Null_Var_Record;
      end Initialize_Fields_For_Call_Expr;
   function Call_Expr_F_Name
     (Node : Bare_Call_Expr) return Bare_Name
   is
   begin
         return Node.Call_Expr_F_Name;
   end;
   function Call_Expr_F_Suffix
     (Node : Bare_Call_Expr) return Bare_Ada_Node
   is
   begin
         return Node.Call_Expr_F_Suffix;
   end;
--# property-start CallExpr.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14950
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Ref_Var
  (Node : Bare_Call_Expr
  )
   return Logic_Var
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9007 '<FieldAccess for ref_var at ast.py:14950>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14950
--# expr-start 9006 '<FieldAccess for name at ast.py:14950>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14950
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Call_Expr_F_Name;
--# expr-done 9006
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 9007
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Ref_Var;
--# end
--# property-start CallExpr.subp_spec_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14954
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subp_Spec_Var
  (Node : Bare_Call_Expr
  )
   return Logic_Var
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9008 '<FieldAccess for r_called_spec at ast.py:14954>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14954
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Call_Expr_R_Called_Spec'Unrestricted_Access;
--# expr-done 9008
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Subp_Spec_Var;
--# end
--# property-start CallExpr.defines_subp_spec_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14955
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Defines_Subp_Spec_Var
  (Node : Bare_Call_Expr
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Defines_Subp_Spec_Var;
--# end
--# property-start CallExpr.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14957
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Relative_Name
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9010 '<FieldAccess for relative_name at ast.py:14957>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14957
--# expr-start 9009 '<FieldAccess for name at ast.py:14957>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14957
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9009
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9010
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Relative_Name;
--# end
--# property-start CallExpr.kind /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14959
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Kind
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Call_Expr_Kind
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Call_Expr_Kind;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Fld_8 : Internal_Entity_Name;
Fld_9 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Boolean;
If_Result_1 : Call_Expr_Kind;
If_Result_2 : Call_Expr_Kind;
If_Result_3 : Call_Expr_Kind;
If_Result_4 : Call_Expr_Kind;
Let_Result : Call_Expr_Kind;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9027 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9026 '<Cond at ast.py:14965>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14965
--# expr-start 9011 '<FieldAccess for is_call at ast.py:14966>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14966
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9011
if Fld then
   If_Result_4 := Call;
else
--# expr-start 9012 '<FieldAccess for is_array_slice at ast.py:14969>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14969
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_array_slice
Fld_1 := Libadalang.Implementation.Call_Expr_P_Is_Array_Slice (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9012
if Fld_1 then
   If_Result_3 := Array_Slice;
else
--# expr-start 9019 '<bind at ast.py:14972>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14972
--# expr-start 9013 '<FieldAccess for origin_node at ast.py:14973>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14973
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 9013
Bound_Origin := Fld_2; 
--# expr-start 9018 '<Not at ast.py:14974>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14974
--# expr-start 9017 '<IsNull at ast.py:14974>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14974
--# expr-start 9015 '<FieldAccess for expression_type at ast.py:14974>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14974
--# expr-start 9014 '<FieldAccess for name at ast.py:14974>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14974
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9014
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_4 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 9015
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 9016 '<FieldAccess for array_def_with_deref at ast.py:14974>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14974
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Bound_Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 9016
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Is_Null := Result_Var.Node = null; 
--# expr-done 9017
Not_Val := not (Is_Null); 
--# expr-done 9018
Dyn_Var_Bind_Result := Not_Val; 
--# expr-done 9019
if Dyn_Var_Bind_Result then
   If_Result_2 := Array_Index;
else
--# expr-start 9022 '<Eq at ast.py:14981>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14981
--# expr-start 9021 '<length at ast.py:14981>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14981
--# expr-start 9020 '<FieldAccess for params at ast.py:14981>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14981
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_6 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9020
Fld_7 := Fld_6.Node; 
Len := Length (Fld_7); 
--# expr-done 9021
Is_Equal := Len = 1; 
--# expr-done 9022
if Is_Equal then
   --# expr-start 9025 '<IsA BaseTypeDecl at ast.py:14982>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14982
--# expr-start 9024 '<FieldAccess for referenced_decl at ast.py:14982>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14982
--# expr-start 9023 '<FieldAccess for name at ast.py:14982>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14982
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9023
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_9 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_8.Node, Imprecise_Fallback => False, E_Info => Fld_8.Info);
--# end
--# expr-done 9024
Is_A := Fld_9.Node /= null 
and then Fld_9.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 9025
   If_Result := Is_A;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := Type_Conversion;
else
   If_Result_1 := raise Property_Error with "undetermined CallExpr kind";
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 9026
Let_Result := If_Result_4; 
--# end
--# expr-done 9027
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Kind;
--# end
--# property-start CallExpr.is_constant /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14990
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Is_Constant
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Name;
Match_1 : Internal_Entity_Expr;
Fld : Call_Expr_Kind;
Is_Equal : Boolean;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
Fld_6 : Call_Expr_Kind;
Is_Equal_1 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9039 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 9038 '<Cond at ast.py:14992>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14992
--# expr-start 9029 '<Eq at ast.py:14993>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14993
--# expr-start 9028 '<FieldAccess for kind at ast.py:14993>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14993
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.kind
Fld := Libadalang.Implementation.Call_Expr_P_Kind (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9028
Is_Equal := Fld = Type_Conversion; 
--# expr-done 9029
if Is_Equal then
   --# expr-start 9033 '<Match at ast.py:14994>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14994
--# expr-start 9031 '<FieldAccess for expr at ast.py:14994>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14994
--# expr-start 9030 '<FieldAccess for params at ast.py:14994>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14994
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_1 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9030
Fld_2 := Fld_1.Node;
Get_Result := Get (Self, Fld_2, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 9031
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_4; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind n Match_0
--# expr-start 9032 '<FieldAccess for is_constant at ast.py:14997>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14997
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 9032
Let_Result := Fld_5; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := True; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 9033
   If_Result_2 := Match_Result;
else
--# expr-start 9035 '<Eq at ast.py:15002>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15002
--# expr-start 9034 '<FieldAccess for kind at ast.py:15002>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15002
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.kind
Fld_6 := Libadalang.Implementation.Call_Expr_P_Kind (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9034
Is_Equal_1 := Fld_6 = Call; 
--# expr-done 9035
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 9037 '<FieldAccess for is_constant_object at ast.py:15007>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15007
--# expr-start 9036 '<FieldAccess for referenced_decl at ast.py:15007>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15007
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_7 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 9036
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_constant_object'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Constant_Object (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 9037
   If_Result_1 := Fld_8;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 9038
Let_Result_2 := If_Result_2; 
--# end
--# expr-done 9039
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Is_Constant;
--# end
--# property-start CallExpr.designated_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15010
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Env
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1080 with Inline_Always;
            procedure Finalizer_Scope_2995 with Inline_Always;
            procedure Finalizer_Scope_2996 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Basic_Decl;
Match_1 : Internal_Entity;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Bound_Include_Ud_Indexing : Boolean;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1080 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1080;
            procedure Finalizer_Scope_2995 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2995;
            procedure Finalizer_Scope_2996 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2996;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9052 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 9041 '<FieldAccess for name_designated_type at ast.py:15012>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15012
--# expr-start 9040 '<FieldAccess for name at ast.py:15012>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15012
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9040
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9041
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 9051 '<If at ast.py:15014>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15014
--# expr-start 9043 '<Not at ast.py:15015>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15015
--# expr-start 9042 '<IsNull at ast.py:15015>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15015
Is_Null := Block_Var.Node = null; 
--# expr-done 9042
Not_Val := not (Is_Null); 
--# expr-done 9043
if Not_Val then
   --# expr-start 9044 '<FieldAccess for defining_env at ast.py:15017>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15017
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Block_Var.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Block_Var.Info);
--# end
--# expr-done 9044
   If_Result := Fld_2;
else
   --# expr-start 9050 '<bind at ast.py:15022>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15022
Bound_Include_Ud_Indexing := True; 
--# expr-start 9049 '<env_group at ast.py:15027>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15027
--# expr-start 9048 '<Map at ast.py:15024>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15024
   --# expr-start 9045 '<FieldAccess for env_elements at ast.py:15024>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15024
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => No_Visibility, E_Info => Ent.Info);
--# end
--# expr-done 9045
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_269 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_269
      --# expr-start 9047 '<Match at ast.py:15024>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15024
      if Item_269.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_269; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bd Match_0
--# expr-start 9046 '<FieldAccess for defining_env at ast.py:15025>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15025
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Match_0.Node, Origin => Origin, Include_Ud_Indexing => Bound_Include_Ud_Indexing, Dottable_Type => No_Bare_Ada_Node, E_Info => Match_0.Info);
--# end
--# expr-done 9046
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2996;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := Empty_Env; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 9047
         declare
            Item_To_Append : constant Lexical_Env := Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2995;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9048
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 9049
Dyn_Var_Bind_Result := Group_Env; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9050
   If_Result := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 9051
Let_Result_2 := If_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 9052
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1080;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1080;
                     Finalizer_Scope_2995;
                     Finalizer_Scope_2996;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Designated_Env;
--# end
--# property-start CallExpr.designated_env_no_overloading /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15031
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1081 with Inline_Always;
      Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1081 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1081;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9053 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := Empty_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 9053
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1081;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1081;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Designated_Env_No_Overloading;
--# end
--# property-start CallExpr.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15041
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Env_Elements_Impl
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1082 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1082 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1082;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9056 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9055 '<FieldAccess for env_elements_impl at ast.py:15043>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15043
--# expr-start 9054 '<FieldAccess for name at ast.py:15043>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9054
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 9055
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 9056
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1082;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1082;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Env_Elements_Impl;
--# end
--# property-start CallExpr.designated_type_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15047
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Type_Impl
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2998 with Inline_Always;
            procedure Finalizer_Scope_2999 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Boolean;
Var_Expr : Internal_Entity_Assoc_List;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_12 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_13 : Boolean;
If_Result : Boolean;
Quantifier_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2998 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2998;
            procedure Finalizer_Scope_2999 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2999;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9074 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9058 '<FieldAccess for designated_type_impl at ast.py:15050>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15050
--# expr-start 9057 '<FieldAccess for name at ast.py:15050>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15050
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9057
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 9058
Block_Var := Fld_1; 
--# bind prefix_tpe Block_Var
--# expr-start 9059 '<FieldAccess for params at ast.py:15055>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15055
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_2 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9059
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind ps Var_Expr
--# expr-start 9072 '<AllQuantifier at ast.py:15058>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15058
   --# expr-start 9061 '<FieldAccess for match_formals at ast.py:15056>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15056
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 9060 '<FieldAccess for discriminants_list at ast.py:15057>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15057
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9060
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2999;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# property-call-start AdaNode.match_formals
Fld_4 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Result_Var, Params => Var_Expr, Is_Dottable_Subp => False);
--# end
--# expr-done 9061
      Quantifier_Result := True;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_4;
      begin
         for Item_270 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_270
--# expr-start 9062 '<FieldAccess for has_matched at ast.py:15060>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15060
Fld_5 := Item_270.Has_Matched;
--# expr-done 9062
if Fld_5 then
   --# expr-start 9071 '<FieldAccess for matching_type at ast.py:15061>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15061
--# expr-start 9065 '<FieldAccess for formal_type at ast.py:15061>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15061
--# expr-start 9064 '<FieldAccess for formal_decl at ast.py:15061>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15061
--# expr-start 9063 '<FieldAccess for formal at ast.py:15061>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15061
Fld_6 := Item_270.Formal;
--# expr-done 9063
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 9064
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_7.Node, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 9065
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9069 '<Cast to Name at ast.py:15062>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15062
--# expr-start 9068 '<FieldAccess for expr at ast.py:15062>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15062
--# expr-start 9067 '<FieldAccess for assoc at ast.py:15062>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15062
--# expr-start 9066 '<FieldAccess for actual at ast.py:15062>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15062
Fld_9 := Item_270.Actual;
--# expr-done 9066
Fld_10 := Fld_9.Assoc;
--# expr-done 9067
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 9068
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 9069
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
--# expr-start 9070 '<FieldAccess for name_designated_type at ast.py:15062>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15062
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_12 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 9070
Scope_Result_1 := Fld_12;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_8.Node, Expected_Type => Result_Var_1, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 9071
   If_Result := Fld_13;
else
   If_Result := False;
end if;
      Quantifier_Result := If_Result;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
--# expr-done 9072
Scope_Result_2 := Quantifier_Result;
   --# end
      Finalizer_Scope_2998;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
Block_Var_1 := Result_Var_2; 
--# bind matches_formals Block_Var_1
--# expr-start 9073 '<If at ast.py:15071>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15071
if Block_Var_1 then
   If_Result_1 := Block_Var;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 9073
Let_Result := If_Result_1; 
--# end
--# expr-done 9074
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2998;
                     Finalizer_Scope_2999;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Designated_Type_Impl;
--# end
--# property-start CallExpr.params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15073
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Params
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9076 '<Cast to AssocList at ast.py:15073>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15073
--# expr-start 9075 '<FieldAccess for suffix at ast.py:15073>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15073
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 9075
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Assoc_List;
   end if;
--# expr-done 9076
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Params;
--# end
--# property-start CallExpr.check_array_slice /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15075
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_Array_Slice
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Assoc_List;
Var_Expr_3 : Internal_Entity_Name;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result : Boolean;
Is_A_2 : Boolean;
If_Result_1 : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Saved : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_4 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9092 '<Block at ???>' Let_Result None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 9077 '<FieldAccess for array_def_with_deref at ast.py:15081>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15081
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 9077
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
--# expr-start 9079 '<Not at ast.py:15083>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15083
--# expr-start 9078 '<IsNull at ast.py:15083>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15083
Is_Null := Block_Var.Node = null; 
--# expr-done 9078
Not_Val := not (Is_Null); 
--# expr-done 9079
if Not_Val then
--# expr-start 9080 '<FieldAccess for suffix at ast.py:15084>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15084
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 9080
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind sfx Var_Expr_1
--# expr-start 9081 '<IsA BinOp at ast.py:15087>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15087
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Bin_Op_Range; 
--# expr-done 9081
if Is_A then
   If_Result := True;
else
   --# expr-start 9082 '<IsA AttributeRef at ast.py:15089>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15089
Is_A_1 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 9082
   If_Result := Is_A_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 9083 '<IsA SubtypeIndication at ast.py:15091>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15091
Is_A_2 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Subtype_Indication_Range; 
--# expr-done 9083
   If_Result_1 := Is_A_2;
end if;
if If_Result_1 then
   If_Result_4 := True;
else
--# expr-start 9084 '<Cast to AssocList at ast.py:15093>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15093
Cast_Expr := Var_Expr_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Assoc_List;
   end if;
--# expr-done 9084
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind al Var_Expr_2
--# expr-start 9086 '<Eq at ast.py:15095>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15095
--# expr-start 9085 '<length at ast.py:15095>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15095
Fld_2 := Var_Expr_2.Node; 
Len := Length (Fld_2); 
--# expr-done 9085
Is_Equal := Len = 1; 
--# expr-done 9086
if Is_Equal then
--# expr-start 9088 '<Cast to Name at ast.py:15096>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15096
--# expr-start 9087 '<FieldAccess for expr at ast.py:15096>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15096
Saved := Var_Expr_2; 
Fld_3 := Saved.Node;
Get_Result := Get (Self, Fld_3, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_4 := Saved.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result_2 := As_Entity;
end if;
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 9087
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 9088
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_3
--# expr-start 9091 '<Not at ast.py:15097>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15097
--# expr-start 9090 '<IsNull at ast.py:15097>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15097
--# expr-start 9089 '<FieldAccess for name_designated_type at ast.py:15097>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15097
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 9089
Is_Null_2 := Fld_6.Node = null; 
--# expr-done 9090
Not_Val_1 := not (Is_Null_2); 
--# expr-done 9091
Scope_Result_1 := Not_Val_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
else
   If_Result_3 := False;
end if;
Scope_Result_2 := If_Result_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_4 := Result_Var_2;
end if;
Scope_Result_3 := If_Result_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_5 := Result_Var_3;
else
   If_Result_5 := False;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 9092
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Check_Array_Slice;
--# end
--# property-start CallExpr.is_array_slice /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15105
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Is_Array_Slice
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9098 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9097 '<bind at ast.py:15111>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15111
--# expr-start 9093 '<FieldAccess for origin_node at ast.py:15112>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15112
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 9093
Bound_Origin := Fld; 
--# expr-start 9096 '<FieldAccess for check_array_slice at ast.py:15113>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15113
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9095 '<FieldAccess for expression_type at ast.py:15113>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15113
--# expr-start 9094 '<FieldAccess for name at ast.py:15113>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15113
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9094
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 9095
--# property-call-start CallExpr.check_array_slice
Fld_3 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Ent.Node, Typ => Fld_2, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 9096
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 9097
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 9098
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Is_Array_Slice;
--# end
--# property-start CallExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15116
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Xref_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1087 with Inline_Always;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1087 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1087;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9100 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9099 '<FieldAccess for bottom_up_name_equation at ast.py:15118>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15118
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.bottom_up_name_equation
Fld := Libadalang.Implementation.Name_P_Bottom_Up_Name_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9099
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 9100
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1087;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1087;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Xref_Equation;
--# end
--# property-start CallExpr.type_conv_self_xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15120
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Type_Conv_Self_Xref_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1088 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Bare_Name;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1088 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1088;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9114 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9113 '<And at ast.py:15127>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15127
--# expr-start 9108 '<And at ast.py:15127>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15127
--# expr-start 9102 '<FieldAccess for subtype_indication_equation at ast.py:15128>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15128
--# expr-start 9101 '<FieldAccess for name at ast.py:15128>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15128
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9101
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.subtype_indication_equation
Fld_1 := Libadalang.Implementation.Name_P_Subtype_Indication_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 9102
--# expr-start 9107 '<Bind at ast.py:15129>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15129
--# expr-start 9104 '<FieldAccess for ref_var at ast.py:15129>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15129
--# expr-start 9103 '<FieldAccess for name at ast.py:15129>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15129
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Call_Expr_F_Name;
--# expr-done 9103
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 9104
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 9106 '<FieldAccess for type_var at ast.py:15129>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15129
--# expr-start 9105 '<FieldAccess for name at ast.py:15129>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15129
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Call_Expr_F_Name;
--# expr-done 9105
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 9106
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result := Solver.Create_Unify (Fld_3, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15129") else null)); 
--# expr-done 9107
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15127") else null)); 
--# expr-done 9108
--# expr-start 9112 '<Bind at ast.py:15130>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15130
--# expr-start 9109 '<FieldAccess for type_var at ast.py:15130>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15130
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9109
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 9111 '<FieldAccess for ref_var at ast.py:15130>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15130
--# expr-start 9110 '<FieldAccess for name at ast.py:15130>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15130
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Call_Expr_F_Name;
--# expr-done 9110
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 9111
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result_1 := Solver.Create_Unify (Fld_6, Fld_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15130") else null)); 
--# expr-done 9112
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15127") else null)); 
--# expr-done 9113
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 9114
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1088;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1088;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Type_Conv_Self_Xref_Equation;
--# end
--# property-start CallExpr.entry_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15133
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Entry_Equation
  (Node : Bare_Call_Expr
      ; E : Internal_Entity_Entry_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind e E
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1089 with Inline_Always;
            procedure Finalizer_Scope_3006 with Inline_Always;
            procedure Finalizer_Scope_3007 with Inline_Always;
            procedure Finalizer_Scope_3008 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_11 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Call_Expr;
Fld_12 : Internal_Entity_Assoc_List;
Fld_13 : Bare_Assoc_List;
Fld_14 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Equation;
Item_271 : Internal_Entity_Basic_Assoc;
Bare_Item_271 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Cast_Expr_1 : Internal_Entity_Entry_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_17 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Cast_Expr_2 : Internal_Entity_Entry_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_18 : Logic_Equation;
Fld_19 : Boolean;
If_Result_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1089 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_18);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1089;
            procedure Finalizer_Scope_3006 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3006;
            procedure Finalizer_Scope_3007 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3007;
            procedure Finalizer_Scope_3008 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_3008;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9137 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9136 '<If at ast.py:15139>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15139
--# expr-start 9115 '<FieldAccess for has_family at ast.py:15140>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15140
      if E.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.has_family
Fld_19 := Libadalang.Implementation.Entry_Decl_P_Has_Family (Node => E.Node);
--# end
--# expr-done 9115
if Fld_19 then
   --# expr-start 9134 '<And at ast.py:15143>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15143
--# expr-start 9116 '<FieldAccess for family_type at ast.py:15143>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15143
      if E.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.family_type
Fld := Libadalang.Implementation.Entry_Decl_P_Family_Type (Node => E.Node, Origin => Origin, E_Info => E.Info);
--# end
--# expr-done 9116
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind ft Var_Expr
--# expr-start 9124 '<And at ast.py:15145>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15145
--# expr-start 9120 '<Bind at ast.py:15145>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15145
--# expr-start 9119 '<FieldAccess for expected_type_var at ast.py:15145>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15145
--# expr-start 9118 '<FieldAccess for expr at ast.py:15145>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15145
--# expr-start 9117 '<FieldAccess for params at ast.py:15145>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15145
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_1 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9117
Fld_2 := Fld_1.Node;
Get_Result := Get (Self, Fld_2, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 9118
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4.Node);
--# expr-done 9119
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Var_Expr; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15145") else null)); 
--# expr-done 9120
--# expr-start 9123 '<FieldAccess for matches_expected_type at ast.py:15146>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15146
--# expr-start 9122 '<FieldAccess for expr at ast.py:15146>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15146
--# expr-start 9121 '<FieldAccess for params at ast.py:15146>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15146
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_6 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9121
Fld_7 := Fld_6.Node;
Get_Result_1 := Get (Self, Fld_7, 0, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity_1 := (Info => Fld_8, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 9122
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_10 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_9.Node, Origin => Origin);
--# end
--# expr-done 9123
And_Pred := Create_And (Bind_Result, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15145") else null)); 
--# expr-done 9124
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3006;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15150") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 9126 '<Cast to CallExpr at ast.py:15152>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15152
--# expr-start 9125 '<FieldAccess for parent_name at ast.py:15152>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15152
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_11 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 9125
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Call_Expr
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Cast_Result_1 := No_Entity_Call_Expr;
   end if;
--# expr-done 9126
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind c Var_Expr_1
--# expr-start 9133 '<And at ast.py:15154>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15154
--# expr-start 9131 '<LogicAll at ast.py:15154>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15154
--# expr-start 9130 '<Map at ast.py:15154>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15154
   --# expr-start 9127 '<FieldAccess for params at ast.py:15154>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15154
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_12 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9127
Fld_13 := Fld_12.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_13 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_13;
      begin
         for Untyped_Item_271 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_271 := Untyped_Item_271; 
Is_Null_2 := Bare_Item_271 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_14 := Fld_12.Info;
As_Entity_2 := (Info => Fld_14, Node => Bare_Item_271); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_271 := If_Result_2; 
   --# scope-start
               --# bind pa Item_271
      --# expr-start 9129 '<FieldAccess for sub_equation at ast.py:15154>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15154
--# expr-start 9128 '<FieldAccess for expr at ast.py:15154>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15154
      if Item_271.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_271.Node, E_Info => Item_271.Info);
--# end
--# expr-done 9128
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_16 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_15.Node, Env => Env, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 9129
         declare
            Item_To_Append : constant Logic_Equation := Fld_16;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3008;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9130
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15154") else null)); 
--# expr-done 9131
--# expr-start 9132 '<FieldAccess for entity_equation at ast.py:15155>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15155
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_1 := E; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start CallExpr.entity_equation
Fld_17 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Var_Expr_1.Node, S => Cast_Result_2, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9132
And_Pred_1 := Create_And (Logic_Boolean_Op, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15154") else null)); 
--# expr-done 9133
Scope_Result_1 := And_Pred_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3007;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15159") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_2 := Create_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15143") else null)); 
--# expr-done 9134
   If_Result_3 := And_Pred_2;
else
   --# expr-start 9135 '<FieldAccess for entity_equation at ast.py:15164>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15164
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_2 := E; 
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start CallExpr.entity_equation
Fld_18 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Ent.Node, S => Cast_Result_3, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9135
   If_Result_3 := Fld_18;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 9136
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 9137
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1089;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1089;
                     Finalizer_Scope_3006;
                     Finalizer_Scope_3007;
                     Finalizer_Scope_3008;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Entry_Equation;
--# end
--# property-start CallExpr.entity_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15167
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Entity_Equation
  (Node : Bare_Call_Expr
      ; S : Internal_Entity_Basic_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind s S
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1090 with Inline_Always;
            procedure Finalizer_Scope_3009 with Inline_Always;
            procedure Finalizer_Scope_3010 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Name;
Fld : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Logic_Equation;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Logic_Equation;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_9 : Internal_Entity_Info;
Fld_10 : Internal_Metadata;
Fld_11 : Boolean;
Fld_12 : Logic_Equation;
Fld_13 : Internal_Entity_Name;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_16 : Internal_Entity_Assoc_List;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_17 : Internal_Entity_Base_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_18 : Internal_Entity_Info;
Fld_19 : Internal_Metadata;
Fld_20 : Boolean;
Fld_21 : Logic_Equation;
Fld_22 : Internal_Entity_Name;
Fld_23 : Internal_Entity_Base_Type_Decl;
Fld_24 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
If_Result : Logic_Equation;
If_Result_1 : Logic_Equation;
If_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1090 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1090;
            procedure Finalizer_Scope_3009 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_3009;
            procedure Finalizer_Scope_3010 is
            begin
                     Dec_Ref (Fld_24);
            end Finalizer_Scope_3010;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9171 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9170 '<And at ast.py:15170>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15170
--# expr-start 9140 '<Bind at ast.py:15170>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15170
--# expr-start 9139 '<FieldAccess for ref_var at ast.py:15170>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15170
--# expr-start 9138 '<FieldAccess for name at ast.py:15170>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15170
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Call_Expr_F_Name;
--# expr-done 9138
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 9139
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Cast_Expr := S; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15170") else null)); 
--# expr-done 9140
--# expr-start 9169 '<Cond at ast.py:15170>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15170
--# expr-start 9141 '<FieldAccess for is_paramless at ast.py:15175>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15175
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_paramless
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Paramless (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 9141
if Fld_2 then
   --# expr-start 9143 '<FieldAccess for parent_name_equation at ast.py:15175>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15175
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9142 '<FieldAccess for expr_type at ast.py:15176>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15176
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 9142
--# property-call-start Name.parent_name_equation
Fld_4 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Fld_3, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9143
   If_Result_2 := Fld_4;
else
--# expr-start 9144 '<FieldAccess for can_be_paramless at ast.py:15181>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15181
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.can_be_paramless
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Can_Be_Paramless (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 9144
if Fld_5 then
   --# expr-start 9156 '<Or at ast.py:15181>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15181
--# expr-start 9146 '<FieldAccess for parent_name_equation at ast.py:15182>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15182
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9145 '<FieldAccess for expr_type at ast.py:15183>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15183
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 9145
--# property-call-start Name.parent_name_equation
Fld_7 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Fld_6, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9146
--# expr-start 9155 '<And at ast.py:15186>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15186
--# expr-start 9151 '<FieldAccess for subprogram_equation at ast.py:15187>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15187
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9147 '<FieldAccess for subp_spec_or_null at ast.py:15188>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15188
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 9147
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
--# expr-start 9150 '<FieldAccess for dottable_subp at ast.py:15189>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15189
--# expr-start 9149 '<FieldAccess for md at ast.py:15189>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15189
--# expr-start 9148 '<FieldAccess for info at ast.py:15189>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15189
Fld_9 := S.Info;
--# expr-done 9148
Fld_10 := Fld_9.Md;
--# expr-done 9149
Fld_11 := Fld_10.Dottable_Subp;
--# expr-done 9150
--# property-call-start CallExpr.subprogram_equation
Fld_12 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_1, Dottable_Subp => Fld_11, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9151
--# expr-start 9152 '<FieldAccess for parent_name at ast.py:15191>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15191
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_13 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 9152
Var_Expr := Fld_13;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 9154 '<FieldAccess for parent_name_equation at ast.py:15193>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15193
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9153 '<FieldAccess for expr_type at ast.py:15193>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15193
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 9153
--# property-call-start Name.parent_name_equation
Fld_15 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_14, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 9154
Scope_Result := Fld_15;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3009;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15194") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_12, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15186") else null)); 
--# expr-done 9155
Or_Pred := Create_Or (Fld_7, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15181") else null)); 
--# expr-done 9156
   If_Result_1 := Or_Pred;
else
--# expr-start 9159 '<Not at ast.py:15199>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15199
--# expr-start 9158 '<IsNull at ast.py:15199>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15199
--# expr-start 9157 '<FieldAccess for params at ast.py:15199>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15199
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_16 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9157
Is_Null := Fld_16.Node = null; 
--# expr-done 9158
Not_Val := not (Is_Null); 
--# expr-done 9159
if Not_Val then
   --# expr-start 9168 '<And at ast.py:15199>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15199
--# expr-start 9164 '<FieldAccess for subprogram_equation at ast.py:15200>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15200
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9160 '<FieldAccess for subp_spec_or_null at ast.py:15201>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15201
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 9160
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
--# expr-start 9163 '<FieldAccess for dottable_subp at ast.py:15202>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15202
--# expr-start 9162 '<FieldAccess for md at ast.py:15202>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15202
--# expr-start 9161 '<FieldAccess for info at ast.py:15202>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15202
Fld_18 := S.Info;
--# expr-done 9161
Fld_19 := Fld_18.Md;
--# expr-done 9162
Fld_20 := Fld_19.Dottable_Subp;
--# expr-done 9163
--# property-call-start CallExpr.subprogram_equation
Fld_21 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_2, Dottable_Subp => Fld_20, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9164
--# expr-start 9165 '<FieldAccess for parent_name at ast.py:15204>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15204
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_22 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 9165
Var_Expr_1 := Fld_22;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_1
--# expr-start 9167 '<FieldAccess for parent_name_equation at ast.py:15206>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15206
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9166 '<FieldAccess for expr_type at ast.py:15206>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15206
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_23 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 9166
--# property-call-start Name.parent_name_equation
Fld_24 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_1.Node, Typ => Fld_23, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9167
Scope_Result_1 := Fld_24;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3010;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15207") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Create_And (Fld_21, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15199") else null)); 
--# expr-done 9168
   If_Result := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15211") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 9169
And_Pred_2 := Create_And (Bind_Result, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15170") else null)); 
--# expr-done 9170
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 9171
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1090;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1090;
                     Finalizer_Scope_3009;
                     Finalizer_Scope_3010;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Entity_Equation;
--# end
--# property-start CallExpr.is_type_conversion /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15214
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Is_Type_Conversion
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Name;
Is_A : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9179 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9174 '<Not at ast.py:15220>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15220
--# expr-start 9173 '<IsA QualExpr at ast.py:15220>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15220
--# expr-start 9172 '<FieldAccess for name at ast.py:15220>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15220
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9172
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Qual_Expr_Range; 
--# expr-done 9173
Not_Val := not (Is_A); 
--# expr-done 9174
if Not_Val then
   --# expr-start 9178 '<Not at ast.py:15221>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15221
--# expr-start 9177 '<IsNull at ast.py:15221>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15221
--# expr-start 9176 '<FieldAccess for name_designated_type at ast.py:15221>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15221
--# expr-start 9175 '<FieldAccess for name at ast.py:15221>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15221
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9175
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 9176
Is_Null := Fld_2.Node = null; 
--# expr-done 9177
Not_Val_1 := not (Is_Null); 
--# expr-done 9178
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 9179
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Is_Type_Conversion;
--# end
--# property-start CallExpr.xref_stop_resolution /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15224
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Xref_Stop_Resolution
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 9180 '<Super at ast.py:15225>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15225
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.xref_stop_resolution
Fld := Libadalang.Implementation.Expr_P_Xref_Stop_Resolution (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9180
if Fld then
   If_Result := True;
else
   --# expr-start 9181 '<FieldAccess for is_type_conversion at ast.py:15225>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15225
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_type_conversion
Fld_1 := Libadalang.Implementation.Call_Expr_P_Is_Type_Conversion (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9181
   If_Result := Fld_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Xref_Stop_Resolution;
--# end
--# property-start CallExpr.stop_resolution_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15228
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Stop_Resolution_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1093 with Inline_Always;
      Fld : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_1093 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1093;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9185 '<If at ast.py:15228>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15228
--# expr-start 9182 '<FieldAccess for is_type_conversion at ast.py:15229>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15229
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_type_conversion
Fld_2 := Libadalang.Implementation.Call_Expr_P_Is_Type_Conversion (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9182
if Fld_2 then
   --# expr-start 9183 '<FieldAccess for type_conv_self_xref_equation at ast.py:15230>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15230
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.type_conv_self_xref_equation
Fld := Libadalang.Implementation.Call_Expr_P_Type_Conv_Self_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9183
   If_Result := Fld;
else
   --# expr-start 9184 '<Super at ast.py:15231>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15231
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.stop_resolution_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Stop_Resolution_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9184
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 9185
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1093;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1093;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Stop_Resolution_Equation;
--# end
--# property-start CallExpr.general_xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15234
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_General_Xref_Equation
  (Node : Bare_Call_Expr
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1094 with Inline_Always;
            procedure Finalizer_Scope_3011 with Inline_Always;
            procedure Finalizer_Scope_3012 with Inline_Always;
            procedure Finalizer_Scope_3013 with Inline_Always;
            procedure Finalizer_Scope_3014 with Inline_Always;
            procedure Finalizer_Scope_3015 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Boolean;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Subps : Internal_Entity_Array_Access;
Var_Expr_3 : Internal_Entity_Entry_Decl;
Fld : Boolean;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_8 : Internal_Entity_Name;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_12 : Internal_Entity_Name;
Is_A : Boolean;
Fld_13 : Internal_Entity_Name;
Fld_14 : Boolean;
Fld_15 : Logic_Equation;
Fld_16 : Internal_Entity_Name;
Fld_17 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_18 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Fld_19 : Internal_Entity_Name;
Fld_20 : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_21 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_2 : Logic_Equation;
Fld_22 : Logic_Equation;
Fld_23 : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Entry_Decl;
Fld_24 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Expr_2 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Fld_25 : Logic_Equation;
Result_Var_3 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_26 : Internal_Entity_Name;
Fld_27 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
And_Pred_5 : Logic_Equation;
If_Result_2 : Logic_Equation;
If_Result_3 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1094 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Subps);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_27);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1094;
            procedure Finalizer_Scope_3011 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_3011;
            procedure Finalizer_Scope_3012 is
            begin
                     Dec_Ref (Fld_15);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3012;
            procedure Finalizer_Scope_3013 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_3013;
            procedure Finalizer_Scope_3014 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Fld_25);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_3014;
            procedure Finalizer_Scope_3015 is
            begin
                     Dec_Ref (Fld_24);
            end Finalizer_Scope_3015;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9229 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 9228 '<Cond at ast.py:15240>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15240
--# expr-start 9186 '<FieldAccess for is_type_conversion at ast.py:15241>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15241
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_type_conversion
Fld := Libadalang.Implementation.Call_Expr_P_Is_Type_Conversion (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9186
if Fld then
   --# expr-start 9199 '<And at ast.py:15244>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15244
--# expr-start 9194 '<And at ast.py:15244>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15244
--# expr-start 9192 '<And at ast.py:15244>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15244
--# expr-start 9187 '<FieldAccess for type_conv_self_xref_equation at ast.py:15244>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15244
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.type_conv_self_xref_equation
Fld_1 := Libadalang.Implementation.Call_Expr_P_Type_Conv_Self_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9187
--# expr-start 9191 '<Bind at ast.py:15245>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15245
--# expr-start 9190 '<FieldAccess for expected_type_var at ast.py:15245>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15245
--# expr-start 9189 '<FieldAccess for expr at ast.py:15245>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15245
--# expr-start 9188 '<FieldAccess for params at ast.py:15245>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15245
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_2 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9188
Fld_3 := Fld_2.Node;
Get_Result := Get (Self, Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 9189
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_5.Node);
--# expr-done 9190
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_6, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15245") else null)); 
--# expr-done 9191
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15244") else null)); 
--# expr-done 9192
--# expr-start 9193 '<FieldAccess for all_args_xref_equation at ast.py:15247>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15247
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_7 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9193
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15244") else null)); 
--# expr-done 9194
--# expr-start 9195 '<FieldAccess for parent_name at ast.py:15248>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15248
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_8 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 9195
Var_Expr := Fld_8;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 9198 '<FieldAccess for parent_name_equation at ast.py:15249>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15249
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9197 '<FieldAccess for name_designated_type at ast.py:15250>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15250
--# expr-start 9196 '<FieldAccess for name at ast.py:15250>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15250
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9196
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_10 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 9197
--# property-call-start Name.parent_name_equation
Fld_11 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_10, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 9198
Scope_Result := Fld_11;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3011;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15253") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (And_Pred_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15244") else null)); 
--# expr-done 9199
   If_Result_3 := And_Pred_2;
else
--# expr-start 9201 '<IsA AttributeRef at ast.py:15259>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15259
--# expr-start 9200 '<FieldAccess for name at ast.py:15259>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15259
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9200
Is_A := Fld_12.Node /= null 
and then Fld_12.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 9201
if Is_A then
--# expr-start 9203 '<FieldAccess for resolve_names_internal at ast.py:15260>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15260
--# expr-start 9202 '<FieldAccess for name at ast.py:15260>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15260
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9202
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_14 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld_13.Node, Env => Env, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 9203
Var_Expr_1 := Fld_14;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 9214 '<And at ast.py:15260>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15260
--# expr-start 9204 '<FieldAccess for all_args_xref_equation at ast.py:15261>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15261
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_15 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9204
--# expr-start 9207 '<Cast to BaseTypeDecl at ast.py:15262>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15262
--# expr-start 9206 '<FieldAccess for type_val at ast.py:15262>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15262
--# expr-start 9205 '<FieldAccess for name at ast.py:15262>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15262
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9205
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_17 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_16.Node);
--# end
--# expr-done 9206
   if Fld_17.Node = null
      or else Fld_17.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 9207
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_2
--# expr-start 9208 '<FieldAccess for parent_name_equation at ast.py:15263>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15263
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_18 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Var_Expr_2, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9208
Scope_Result_1 := Fld_18;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3013;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 9213 '<FieldAccess for entity_equation at ast.py:15268>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15268
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9212 '<Cast to BasicDecl at ast.py:15269>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15269
--# expr-start 9211 '<get_value at ast.py:15269>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15269
--# expr-start 9210 '<FieldAccess for ref_var at ast.py:15269>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15269
--# expr-start 9209 '<FieldAccess for name at ast.py:15269>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15269
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9209
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_19.Node);
--# end
--# expr-done 9210
Is_Logic_Var_Defined := Entity_Vars.Is_Defined (Fld_20); 
if Is_Logic_Var_Defined then
Eq_Solution := Entity_Vars.Get_Value (Fld_20); 
   If_Result_1 := Eq_Solution;
else
   If_Result_1 := No_Entity;
end if;
--# expr-done 9211
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9212
--# property-call-start CallExpr.entity_equation
Fld_21 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Ent.Node, S => Cast_Result_2, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9213
   Result_Var_1 := Fld_21;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_3 := Create_And (Fld_15, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15260") else null)); 
--# expr-done 9214
Scope_Result_2 := And_Pred_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3012;
   Result_Var_2 := Scope_Result_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15273") else null)); 
   Result_Var_2 := False_Rel;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_2 := Result_Var_2;
else
   --# expr-start 9227 '<And at ast.py:15276>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15276
--# expr-start 9215 '<FieldAccess for all_args_xref_equation at ast.py:15277>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15277
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_22 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9215
--# expr-start 9226 '<Let at ast.py:15281>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15281
--# scope-start
--# expr-start 9216 '<FieldAccess for env_elements at ast.py:15281>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15281
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_23 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 9216
Subps := Fld_23; Inc_Ref (Subps);
--# bind subps Subps
--# expr-start 9225 '<And at ast.py:15281>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15281
--# expr-start 9222 '<LogicAny at ast.py:15282>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15282
--# expr-start 9221 '<Map at ast.py:15282>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15282
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Subps;
      begin
         for Item_272 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_272
--# expr-start 9217 '<Cast to EntryDecl at ast.py:15283>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15283
Cast_Expr_1 := Item_272; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_3 := No_Entity_Entry_Decl;
   end if;
--# expr-done 9217
Var_Expr_3 := Cast_Result_3;
if Var_Expr_3 /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind e Var_Expr_3
--# expr-start 9218 '<FieldAccess for entry_equation at ast.py:15284>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15284
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.entry_equation
Fld_24 := Libadalang.Implementation.Call_Expr_P_Entry_Equation (Node => Ent.Node, E => Var_Expr_3, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9218
Scope_Result_3 := Fld_24;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3015;
   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 9220 '<FieldAccess for entity_equation at ast.py:15285>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15285
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9219 '<Cast to BasicDecl at ast.py:15286>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15286
Cast_Expr_2 := Item_272; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9219
--# property-call-start CallExpr.entity_equation
Fld_25 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Ent.Node, S => Cast_Result_4, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9220
   Result_Var_3 := Fld_25;
end if;
      Inc_Ref (Result_Var_3);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3014;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9221
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15282") else null)); 
--# expr-done 9222
--# expr-start 9224 '<FieldAccess for sub_equation at ast.py:15291>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15291
--# expr-start 9223 '<FieldAccess for name at ast.py:15291>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15291
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9223
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_27 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_26.Node, Env => Env, Origin => Origin, E_Info => Fld_26.Info);
--# end
--# expr-done 9224
And_Pred_4 := Create_And (Logic_Boolean_Op, Fld_27, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15281") else null)); 
--# expr-done 9225
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 9226
And_Pred_5 := Create_And (Fld_22, Let_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15276") else null)); 
--# expr-done 9227
   If_Result_2 := And_Pred_5;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 9228
Let_Result_1 := If_Result_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 9229
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1094;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1094;
                     Finalizer_Scope_3011;
                     Finalizer_Scope_3012;
                     Finalizer_Scope_3013;
                     Finalizer_Scope_3014;
                     Finalizer_Scope_3015;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_General_Xref_Equation;
--# end
--# property-start CallExpr.subscriptable_type_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15298
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subscriptable_Type_Equation
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1095 with Inline_Always;
            procedure Finalizer_Scope_3018 with Inline_Always;
            procedure Finalizer_Scope_3020 with Inline_Always;
            procedure Finalizer_Scope_3022 with Inline_Always;
            procedure Finalizer_Scope_3023 with Inline_Always;
            procedure Finalizer_Scope_3024 with Inline_Always;
            procedure Finalizer_Scope_3025 with Inline_Always;
            procedure Finalizer_Scope_3026 with Inline_Always;
            procedure Finalizer_Scope_3027 with Inline_Always;
            procedure Finalizer_Scope_3028 with Inline_Always;
            procedure Finalizer_Scope_3029 with Inline_Always;
      Block_Var : Internal_Entity_Array_Type_Def;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_3 : Internal_Entity_Array_Type_Def;
Match_0 : Internal_Entity_Assoc_List;
Match_1 : Internal_Entity_Bin_Op;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Subtype_Indication;
Match_4 : Internal_Entity;
Var_Expr_4 : Internal_Entity_Name;
Var_Expr_5 : Internal_Entity_Basic_Assoc;
Var_Expr_6 : Internal_Entity_Assoc_List;
Formal : Internal_Entity_Defining_Name;
Ret_Type : Internal_Entity_Base_Type_Decl;
Param : Internal_Entity_Expr;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Access_Def;
Is_A : Boolean;
Fld_4 : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_5 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_6 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_2 : Logic_Equation;
Fld_7 : Internal_Entity_Array_Indices;
Scope_Result_3 : Internal_Entity_Array_Indices;
Result_Var_3 : Internal_Entity_Array_Indices;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_8 : Internal_Entity;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Bare_Assoc_List;
Fld_11 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Scope_Result_4 : Internal_Entity_Expr;
Result_Var_4 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
False_Rel_1 : Logic_Equation;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_15 : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
If_Result_2 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
False_Rel_2 : Logic_Equation;
Result_Var_5 : Logic_Equation;
Fld_16 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_17 : Bare_Assoc_List;
Fld_18 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Basic_Assoc;
Fld_19 : Internal_Entity_Array_Indices;
Fld_20 : Internal_Entity_Expr;
Fld_21 : Logic_Equation;
Item_273 : Internal_Entity_Basic_Assoc;
Bare_Item_273 : Bare_Basic_Assoc;
Index_274 : Integer;
Map_Result : Logic_Equation_Array_Access;
Scope_Result_6 : Logic_Equation_Array_Access;
Result_Var_6 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_22 : Logic_Var;
Fld_23 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_24 : Internal_Entity_Array_Indices;
Fld_25 : Internal_Entity_Expr;
Fld_26 : Logic_Equation;
Fld_27 : Internal_Entity_Array_Indices;
Fld_28 : Internal_Entity_Expr;
Fld_29 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_30 : Logic_Var;
Fld_31 : Internal_Entity_Expr;
Fld_32 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_33 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_36 : Internal_Entity_Expr;
Fld_37 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_38 : Logic_Equation;
Fld_39 : Internal_Entity_Array_Indices;
Cast_Expr_2 : Internal_Entity_Attribute_Ref;
Cast_Result_6 : Internal_Entity_Expr;
Fld_40 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_41 : Logic_Var;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_42 : Logic_Equation;
Fld_43 : Logic_Var;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
And_Pred_9 : Logic_Equation;
False_Rel_3 : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_9 : Internal_Entity_Assoc_List;
Let_Result : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Cast_Result_10 : Internal_Entity_Bin_Op;
Let_Result_1 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Cast_Result_11 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Cast_Result_12 : Internal_Entity_Subtype_Indication;
Let_Result_3 : Logic_Equation;
Scope_Result_10 : Logic_Equation;
Cast_Result_13 : Internal_Entity;
Let_Result_4 : Logic_Equation;
Scope_Result_11 : Logic_Equation;
Match_Result : Logic_Equation;
Is_Null_4 : Boolean;
Not_Val_1 : Boolean;
Fld_44 : Boolean;
If_Result_4 : Boolean;
Fld_45 : Internal_Entity_Basic_Decl_Array_Access;
Fld_46 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_47 : Internal_Entity_Base_Subp_Spec;
Fld_48 : Internal_Entity_Defining_Name_Array_Access;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_49 : Internal_Entity_Base_Subp_Spec;
Fld_50 : Internal_Entity_Base_Type_Decl;
Fld_51 : Internal_Entity_Assoc_List;
Fld_52 : Bare_Assoc_List;
Fld_53 : Internal_Entity_Info;
Get_Result_2 : Bare_Basic_Assoc;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_5 : Boolean;
If_Result_5 : Internal_Entity_Basic_Assoc;
Fld_54 : Internal_Entity_Expr;
Fld_55 : Logic_Var;
Cast_Expr_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_14 : Internal_Entity;
Bind_Result_6 : Logic_Equation;
Fld_56 : Logic_Var;
Fld_57 : Internal_Entity_Base_Formal_Param_Decl;
Fld_58 : Internal_Entity_Base_Type_Decl;
Cast_Result_15 : Internal_Entity;
Bind_Result_7 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_59 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
False_Rel_4 : Logic_Equation;
If_Result_6 : Logic_Equation;
If_Result_7 : Logic_Equation;
If_Result_8 : Logic_Equation;
Let_Result_6 : Logic_Equation;
            procedure Finalizer_Scope_1095 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Fld_45);
                     Dec_Ref (Fld_46);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (False_Rel_4);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_1095;
            procedure Finalizer_Scope_3018 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3018;
            procedure Finalizer_Scope_3020 is
            begin
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (False_Rel_2);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_3020;
            procedure Finalizer_Scope_3022 is
            begin
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_3022;
            procedure Finalizer_Scope_3023 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3023;
            procedure Finalizer_Scope_3024 is
            begin
                     Dec_Ref (Fld_21);
            end Finalizer_Scope_3024;
            procedure Finalizer_Scope_3025 is
            begin
                     Dec_Ref (Fld_26);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_35);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_37);
                     Dec_Ref (And_Pred_6);
            end Finalizer_Scope_3025;
            procedure Finalizer_Scope_3026 is
            begin
                     Dec_Ref (Fld_38);
                     Dec_Ref (Fld_40);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_8);
            end Finalizer_Scope_3026;
            procedure Finalizer_Scope_3027 is
            begin
                     Dec_Ref (Fld_42);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_9);
            end Finalizer_Scope_3027;
            procedure Finalizer_Scope_3028 is
            begin
                     Dec_Ref (False_Rel_3);
            end Finalizer_Scope_3028;
            procedure Finalizer_Scope_3029 is
            begin
                     Dec_Ref (Fld_48);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (And_Pred_10);
                     Dec_Ref (Fld_59);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_3029;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9324 '<Block at ???>' Let_Result_6 None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 9230 '<FieldAccess for array_def_with_deref at ast.py:15304>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15304
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 9230
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_1
--# expr-start 9233 '<If at ast.py:15306>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15306
--# expr-start 9231 '<FieldAccess for is_implicit_deref at ast.py:15306>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15306
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9231
if Fld_2 then
   --# expr-start 9232 '<FieldAccess for accessed_type at ast.py:15306>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15306
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9232
   If_Result := Fld_1;
else
   If_Result := Var_Expr_1;
end if;
--# expr-done 9233
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind real_typ Block_Var_1
--# expr-start 9323 '<Cond at ast.py:15309>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15309
--# expr-start 9235 '<IsA AccessToSubpDef at ast.py:15311>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15311
--# expr-start 9234 '<FieldAccess for access_def at ast.py:15311>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15311
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 9234
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 9235
if Is_A then
--# expr-start 9237 '<Cast to AccessToSubpDef at ast.py:15312>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15312
--# expr-start 9236 '<FieldAccess for access_def at ast.py:15312>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15312
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 9236
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 9237
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind asd Var_Expr_2
--# expr-start 9239 '<FieldAccess for subprogram_equation at ast.py:15314>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15314
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9238 '<FieldAccess for subp_spec at ast.py:15314>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15314
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_2.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_2.Info);
--# expr-done 9238
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
--# property-call-start CallExpr.subprogram_equation
Fld_6 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_1, Dottable_Subp => False, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9239
Scope_Result_2 := Fld_6;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3018;
   Result_Var_2 := Scope_Result_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15315") else null)); 
   Result_Var_2 := False_Rel;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_8 := Result_Var_2;
else
--# expr-start 9242 '<Not at ast.py:15318>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15318
--# expr-start 9241 '<IsNull at ast.py:15318>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15318
Var_Expr_3 := Block_Var;
if Var_Expr_3 /= No_Entity_Array_Type_Def then
   --# scope-start
--# expr-start 9240 '<FieldAccess for indices at ast.py:15318>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15318
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Array_Indices (Node => Var_Expr_3.Node.Array_Type_Def_F_Indices, Info => Var_Expr_3.Info);
--# expr-done 9240
Scope_Result_3 := Fld_7;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Array_Indices;
end if;
Is_Null := Result_Var_3.Node = null; 
--# expr-done 9241
Not_Val := not (Is_Null); 
--# expr-done 9242
if Not_Val then
   --# expr-start 9297 '<Match at ast.py:15318>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15318
--# expr-start 9243 '<FieldAccess for suffix at ast.py:15318>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15318
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 9243
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_8; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Assoc_List =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Assoc_List
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_9; 
--# bind _ Match_0
--# expr-start 9264 '<Or at ast.py:15319>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15319
--# expr-start 9246 '<Cast to Name at ast.py:15321>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15321
--# expr-start 9244 '<FieldAccess for params at ast.py:15321>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15321
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_9 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9244
Fld_10 := Fld_9.Node;
Get_Result := Get (Self, Fld_10, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_11 := Fld_9.Info;
As_Entity := (Info => Fld_11, Node => Get_Result); 
   If_Result_1 := As_Entity;
end if;
Var_Expr_5 := If_Result_1;
if Var_Expr_5 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 9245 '<FieldAccess for expr at ast.py:15321>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15321
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 9245
Scope_Result_4 := Fld_12;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Expr;
end if;
   if Result_Var_4.Node = null
      or else Result_Var_4.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Result_Var_4.Node,
         Info => Result_Var_4.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 9246
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_4
--# expr-start 9253 '<If at ast.py:15322>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15322
--# expr-start 9248 '<IsNull at ast.py:15323>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15323
--# expr-start 9247 '<FieldAccess for name_designated_type at ast.py:15323>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15323
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_15 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 9247
Is_Null_2 := Fld_15.Node = null; 
--# expr-done 9248
if Is_Null_2 then
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15324") else null)); 
   If_Result_2 := False_Rel_1;
else
   --# expr-start 9252 '<And at ast.py:15325>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15325
--# expr-start 9249 '<FieldAccess for xref_no_overloading at ast.py:15325>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15325
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_13 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_4.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 9249
--# expr-start 9251 '<Bind at ast.py:15326>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15326
--# expr-start 9250 '<FieldAccess for type_var at ast.py:15326>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15326
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9250
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
Cast_Expr := Block_Var_1; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_14, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15326") else null)); 
--# expr-done 9251
And_Pred := Create_And (Fld_13, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15325") else null)); 
--# expr-done 9252
   If_Result_2 := And_Pred;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 9253
Scope_Result_5 := If_Result_2;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3022;
   Result_Var_5 := Scope_Result_5;
else
   False_Rel_2 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15328") else null)); 
   Result_Var_5 := False_Rel_2;
end if;
      Inc_Ref (Result_Var_5);
--# expr-start 9263 '<And at ast.py:15332>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15332
--# expr-start 9259 '<LogicAll at ast.py:15332>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15332
--# expr-start 9254 '<FieldAccess for params at ast.py:15332>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15332
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_16 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9254
Var_Expr_6 := Fld_16;
if Var_Expr_6 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 9258 '<Map at ast.py:15332>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15332
Saved := Var_Expr_6; 
Fld_17 := Saved.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_17 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
         Index_274 := 0;
      declare
         Collection : constant Bare_Assoc_List := Fld_17;
      begin
         for Untyped_Item_273 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_273 := Untyped_Item_273; 
Is_Null_3 := Bare_Item_273 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Basic_Assoc;
else
Fld_18 := Saved.Info;
As_Entity_1 := (Info => Fld_18, Node => Bare_Item_273); 
   If_Result_3 := As_Entity_1;
end if;
                  Item_273 := If_Result_3; 
   --# scope-start
               --# bind pa Item_273
               --# bind i Index_274
      --# expr-start 9257 '<FieldAccess for constrain_index_expr at ast.py:15334>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15334
--# expr-start 9255 '<FieldAccess for indices at ast.py:15334>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15334
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 9255
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9256 '<FieldAccess for expr at ast.py:15334>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15334
      if Item_273.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_20 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_273.Node, E_Info => Item_273.Info);
--# end
--# expr-done 9256
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_21 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_19.Node, Index_Expr => Fld_20, Dim => Index_274, Origin => Origin, E_Info => Fld_19.Info);
--# end
--# expr-done 9257
         declare
            Item_To_Append : constant Logic_Equation := Fld_21;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3024;
               Index_274 := Index_274 + 1;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9258
Scope_Result_6 := Map_Result;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3023;
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Logic_Equation_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Result_Var_6.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15332") else null)); 
--# expr-done 9259
--# expr-start 9262 '<Bind at ast.py:15336>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15336
--# expr-start 9260 '<FieldAccess for type_var at ast.py:15336>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15336
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9260
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
--# expr-start 9261 '<FieldAccess for comp_type at ast.py:15336>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15336
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_23 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 9261
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_23.Node,
         Info => Fld_23.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_22, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15336") else null)); 
--# expr-done 9262
And_Pred_1 := Create_And (Logic_Boolean_Op, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15332") else null)); 
--# expr-done 9263
Or_Pred := Create_Or (Result_Var_5, And_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15319") else null)); 
--# expr-done 9264
Let_Result := Or_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result_7 := Let_Result;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3020;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_10; 
--# bind bo Match_1
--# expr-start 9285 '<And at ast.py:15341>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15341
--# expr-start 9282 '<And at ast.py:15341>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15341
--# expr-start 9279 '<And at ast.py:15341>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15341
--# expr-start 9276 '<And at ast.py:15341>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15341
--# expr-start 9271 '<And at ast.py:15341>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15341
--# expr-start 9267 '<FieldAccess for constrain_index_expr at ast.py:15341>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15341
--# expr-start 9265 '<FieldAccess for indices at ast.py:15341>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15341
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 9265
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9266 '<FieldAccess for left at ast.py:15341>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15341
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 9266
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_26 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_24.Node, Index_Expr => Fld_25, Dim => 0, Origin => Origin, E_Info => Fld_24.Info);
--# end
--# expr-done 9267
--# expr-start 9270 '<FieldAccess for constrain_index_expr at ast.py:15342>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15342
--# expr-start 9268 '<FieldAccess for indices at ast.py:15342>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15342
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 9268
      if Fld_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9269 '<FieldAccess for right at ast.py:15342>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15342
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 9269
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_27.Node, Index_Expr => Fld_28, Dim => 0, Origin => Origin, E_Info => Fld_27.Info);
--# end
--# expr-done 9270
And_Pred_2 := Create_And (Fld_26, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15341") else null)); 
--# expr-done 9271
--# expr-start 9275 '<Bind at ast.py:15343>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15343
--# expr-start 9272 '<FieldAccess for expected_type_var at ast.py:15343>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15343
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Match_1.Node);
--# expr-done 9272
Fld_30.Value := No_Entity;
Entity_Vars.Reset (Fld_30);
--# expr-start 9274 '<FieldAccess for expected_type_var at ast.py:15343>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15343
--# expr-start 9273 '<FieldAccess for right at ast.py:15343>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15343
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 9273
      if Fld_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_31.Node);
--# expr-done 9274
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
Bind_Result_2 := Solver.Create_Unify (Fld_30, Fld_32, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15343") else null)); 
--# expr-done 9275
And_Pred_3 := Create_And (And_Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15341") else null)); 
--# expr-done 9276
--# expr-start 9278 '<Bind at ast.py:15344>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15344
--# expr-start 9277 '<FieldAccess for type_var at ast.py:15344>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15344
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9277
Fld_33.Value := No_Entity;
Entity_Vars.Reset (Fld_33);
Cast_Expr_1 := Block_Var_1; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_33, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15344") else null)); 
--# expr-done 9278
And_Pred_4 := Create_And (And_Pred_3, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15341") else null)); 
--# expr-done 9279
--# expr-start 9281 '<FieldAccess for sub_equation at ast.py:15345>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15345
--# expr-start 9280 '<FieldAccess for left at ast.py:15345>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15345
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 9280
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_35 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_34.Node, Env => Env, Origin => Origin, E_Info => Fld_34.Info);
--# end
--# expr-done 9281
And_Pred_5 := Create_And (And_Pred_4, Fld_35, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15341") else null)); 
--# expr-done 9282
--# expr-start 9284 '<FieldAccess for sub_equation at ast.py:15346>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15346
--# expr-start 9283 '<FieldAccess for right at ast.py:15346>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15346
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 9283
      if Fld_36.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_37 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_36.Node, Env => Env, Origin => Origin, E_Info => Fld_36.Info);
--# end
--# expr-done 9284
And_Pred_6 := Create_And (And_Pred_5, Fld_37, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15341") else null)); 
--# expr-done 9285
Let_Result_1 := And_Pred_6; Inc_Ref (Let_Result_1);
--# end
Scope_Result_8 := Let_Result_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_3025;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_11; 
--# bind ar Match_2
--# expr-start 9292 '<And at ast.py:15350>' And_Pred_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15350
--# expr-start 9289 '<And at ast.py:15350>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15350
--# expr-start 9286 '<FieldAccess for sub_equation at ast.py:15350>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15350
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_38 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 9286
--# expr-start 9288 '<FieldAccess for constrain_index_expr at ast.py:15351>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15351
--# expr-start 9287 '<FieldAccess for indices at ast.py:15351>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15351
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 9287
      if Fld_39.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_2 := Match_2; 
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_40 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_39.Node, Index_Expr => Cast_Result_6, Dim => 0, Origin => Origin, E_Info => Fld_39.Info);
--# end
--# expr-done 9288
And_Pred_7 := Create_And (Fld_38, Fld_40, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15350") else null)); 
--# expr-done 9289
--# expr-start 9291 '<Bind at ast.py:15352>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15352
--# expr-start 9290 '<FieldAccess for type_var at ast.py:15352>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15352
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9290
Fld_41.Value := No_Entity;
Entity_Vars.Reset (Fld_41);
Cast_Expr_3 := Block_Var_1; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_41, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15352") else null)); 
--# expr-done 9291
And_Pred_8 := Create_And (And_Pred_7, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15350") else null)); 
--# expr-done 9292
Let_Result_2 := And_Pred_8; Inc_Ref (Let_Result_2);
--# end
Scope_Result_9 := Let_Result_2;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_3026;
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_12; 
--# bind st Match_3
--# expr-start 9296 '<And at ast.py:15356>' And_Pred_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15356
--# expr-start 9293 '<FieldAccess for sub_equation at ast.py:15356>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15356
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_42 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_3.Node, Env => Env, Origin => Origin, E_Info => Match_3.Info);
--# end
--# expr-done 9293
--# expr-start 9295 '<Bind at ast.py:15357>' Bind_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15357
--# expr-start 9294 '<FieldAccess for type_var at ast.py:15357>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15357
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_43 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9294
Fld_43.Value := No_Entity;
Entity_Vars.Reset (Fld_43);
Cast_Expr_4 := Block_Var_1; 
      Cast_Result_8 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_43, Cast_Result_8, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15357") else null)); 
--# expr-done 9295
And_Pred_9 := Create_And (Fld_42, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15356") else null)); 
--# expr-done 9296
Let_Result_3 := And_Pred_9; Inc_Ref (Let_Result_3);
--# end
Scope_Result_10 := Let_Result_3;
   Inc_Ref (Scope_Result_10);
   --# end
      Finalizer_Scope_3027;
            Match_Result := Scope_Result_10; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Base_Assoc_List | Ada_Basic_Decl_List .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Call_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_13; 
--# bind _ Match_4
False_Rel_3 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15359") else null)); 
Let_Result_4 := False_Rel_3; Inc_Ref (Let_Result_4);
--# end
Scope_Result_11 := Let_Result_4;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_3028;
            Match_Result := Scope_Result_11; Inc_Ref (Match_Result);
end case;
--# expr-done 9297
   If_Result_7 := Match_Result;
else
--# expr-start 9299 '<Not at ast.py:15363>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15363
--# expr-start 9298 '<IsNull at ast.py:15363>' Is_Null_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15363
Is_Null_4 := Typ.Node = null; 
--# expr-done 9298
Not_Val_1 := not (Is_Null_4); 
--# expr-done 9299
if Not_Val_1 then
   --# expr-start 9300 '<FieldAccess for has_ud_indexing at ast.py:15363>' Fld_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15363
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_44 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 9300
   If_Result_4 := Fld_44;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 9322 '<LogicAny at ast.py:15365>' Logic_Boolean_Op_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15365
--# expr-start 9321 '<Map at ast.py:15365>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15365
   --# expr-start 9303 '<Concat at ast.py:15364>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15364
--# expr-start 9301 '<FieldAccess for constant_indexing_fns at ast.py:15364>' Fld_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15364
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_45 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 9301
--# expr-start 9302 '<FieldAccess for variable_indexing_fns at ast.py:15364>' Fld_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15364
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_46 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 9302
Concat_Result := Concat (Fld_45, Fld_46); 
--# expr-done 9303
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_275 of
               Collection.Items
         loop
   --# scope-start
               --# bind fn Item_275
      --# expr-start 9320 '<Let at ast.py:15365>' Let_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15365
--# scope-start
--# expr-start 9306 '<at at ast.py:15367>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15367
--# expr-start 9305 '<FieldAccess for unpacked_formal_params at ast.py:15367>' Fld_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15367
--# expr-start 9304 '<FieldAccess for subp_spec_or_null at ast.py:15367>' Fld_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15367
      if Item_275.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_47 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_275.Node, E_Info => Item_275.Info);
--# end
--# expr-done 9304
      if Fld_47.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_48 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_47.Node, E_Info => Fld_47.Info);
--# end
--# expr-done 9305
Get_Result_1 := Get (Self, Fld_48, 1, True); 
--# expr-done 9306
Formal := Get_Result_1; 
--# bind formal Formal
--# expr-start 9308 '<FieldAccess for return_type at ast.py:15368>' Fld_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15368
--# expr-start 9307 '<FieldAccess for subp_spec_or_null at ast.py:15368>' Fld_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15368
      if Item_275.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_49 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_275.Node, E_Info => Item_275.Info);
--# end
--# expr-done 9307
      if Fld_49.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_50 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_49.Node, Origin => Origin, E_Info => Fld_49.Info);
--# end
--# expr-done 9308
Ret_Type := Fld_50; 
--# bind ret_type Ret_Type
--# expr-start 9310 '<FieldAccess for expr at ast.py:15369>' Fld_54 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15369
--# expr-start 9309 '<FieldAccess for params at ast.py:15369>' Fld_51 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15369
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_51 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9309
Fld_52 := Fld_51.Node;
Get_Result_2 := Get (Self, Fld_52, 0, True); 
Is_Null_5 := Get_Result_2 = null; 
if Is_Null_5 then
   If_Result_5 := No_Entity_Basic_Assoc;
else
Fld_53 := Fld_51.Info;
As_Entity_2 := (Info => Fld_53, Node => Get_Result_2); 
   If_Result_5 := As_Entity_2;
end if;
      if If_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_54 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_5.Node, E_Info => If_Result_5.Info);
--# end
--# expr-done 9310
Param := Fld_54; 
--# bind param Param
--# expr-start 9319 '<And at ast.py:15371>' And_Pred_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15371
--# expr-start 9317 '<And at ast.py:15371>' And_Pred_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15371
--# expr-start 9312 '<Bind at ast.py:15371>' Bind_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15371
--# expr-start 9311 '<FieldAccess for type_var at ast.py:15371>' Fld_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15371
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_55 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9311
Fld_55.Value := No_Entity;
Entity_Vars.Reset (Fld_55);
Cast_Expr_5 := Ret_Type; 
      Cast_Result_14 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
Bind_Result_6 := Solver.Create_Assign (Fld_55, Cast_Result_14, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15371") else null)); 
--# expr-done 9312
--# expr-start 9316 '<Bind at ast.py:15372>' Bind_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15372
--# expr-start 9313 '<FieldAccess for expected_type_var at ast.py:15372>' Fld_56 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15372
      if Param.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_56 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Param.Node);
--# expr-done 9313
Fld_56.Value := No_Entity;
Entity_Vars.Reset (Fld_56);
--# expr-start 9315 '<FieldAccess for formal_type at ast.py:15372>' Fld_58 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15372
--# expr-start 9314 '<FieldAccess for formal_decl at ast.py:15372>' Fld_57 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15372
      if Formal.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_57 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Formal.Node, E_Info => Formal.Info);
--# end
--# expr-done 9314
      if Fld_57.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_58 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_57.Node, Origin => Origin, E_Info => Fld_57.Info);
--# end
--# expr-done 9315
      Cast_Result_15 := Create_Internal_Entity
        (Node => Fld_58.Node,
         Info => Fld_58.Info);
Bind_Result_7 := Solver.Create_Assign (Fld_56, Cast_Result_15, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15372") else null)); 
--# expr-done 9316
And_Pred_10 := Create_And (Bind_Result_6, Bind_Result_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15371") else null)); 
--# expr-done 9317
--# expr-start 9318 '<FieldAccess for matches_expected_type at ast.py:15373>' Fld_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15373
      if Param.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_59 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Param.Node, Origin => Origin);
--# end
--# expr-done 9318
And_Pred_11 := Create_And (And_Pred_10, Fld_59, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15371") else null)); 
--# expr-done 9319
Let_Result_5 := And_Pred_11; Inc_Ref (Let_Result_5);
--# end
--# expr-done 9320
         declare
            Item_To_Append : constant Logic_Equation := Let_Result_5;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3029;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 9321
Logic_Boolean_Op_1 := Solver.Create_Any (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15365") else null)); 
--# expr-done 9322
   If_Result_6 := Logic_Boolean_Op_1;
else
   False_Rel_4 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15376") else null)); 
   If_Result_6 := False_Rel_4;
end if;
      Inc_Ref (If_Result_6);
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 9323
Let_Result_6 := If_Result_8; Inc_Ref (Let_Result_6);
--# end
--# expr-done 9324
         Property_Result := Let_Result_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1095;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1095;
                     Finalizer_Scope_3018;
                     Finalizer_Scope_3020;
                     Finalizer_Scope_3022;
                     Finalizer_Scope_3023;
                     Finalizer_Scope_3024;
                     Finalizer_Scope_3025;
                     Finalizer_Scope_3026;
                     Finalizer_Scope_3027;
                     Finalizer_Scope_3028;
                     Finalizer_Scope_3029;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Subscriptable_Type_Equation;
--# end
--# property-start CallExpr.subprogram_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15379
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subprogram_Equation
  (Node : Bare_Call_Expr
      ; Subp_Spec : Internal_Entity_Base_Formal_Param_Holder
      ; Dottable_Subp : Boolean
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp_spec Subp_Spec
   --# bind dottable_subp Dottable_Subp
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1096 with Inline_Always;
            procedure Finalizer_Scope_3030 with Inline_Always;
            procedure Finalizer_Scope_3032 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
N : Internal_Entity_Base_Formal_Param_Decl;
Fld : Logic_Var;
Cast_Expr : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl;
Fld_7 : Internal_Single_Actual;
Fld_8 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_11 : Internal_Single_Actual;
Fld_12 : Bare_Base_Id;
Fld_13 : Logic_Var;
Fld_14 : Internal_Entity_Defining_Name;
Fld_15 : Internal_Entity_Base_Formal_Param_Decl;
Fld_16 : Bare_Base_Formal_Param_Decl;
Cast_Result_3 : Bare_Ada_Node;
Fld_17 : Internal_Entity_Info;
Fld_18 : Env_Rebindings;
Fld_19 : Internal_Entity_Info;
Fld_20 : Boolean;
Fld_21 : Internal_Entity;
Let_Result : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_22 : Internal_Single_Actual;
Fld_23 : Bare_Base_Id;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_24 : Boolean;
If_Result_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
False_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1096 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1096;
            procedure Finalizer_Scope_3030 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3030;
            procedure Finalizer_Scope_3032 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_3032;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9363 '<Block at ???>' Let_Result_1 None
--# scope-start
Var_Expr := Subp_Spec;
if Var_Expr /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
   --# bind subp_spec Var_Expr
--# expr-start 9362 '<And at ast.py:15386>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15386
--# expr-start 9331 '<And at ast.py:15386>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15386
--# expr-start 9328 '<Bind at ast.py:15386>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15386
--# expr-start 9325 '<FieldAccess for type_var at ast.py:15386>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15386
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9325
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 9326 '<Cast to BaseSubpSpec at ast.py:15386>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15386
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Spec;
   end if;
--# expr-done 9326
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 9327 '<FieldAccess for return_type at ast.py:15386>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15386
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9327
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15386") else null)); 
--# expr-done 9328
--# expr-start 9330 '<Bind at ast.py:15390>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15390
--# expr-start 9329 '<FieldAccess for subp_spec_var at ast.py:15390>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15390
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 9329
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Cast_Expr_1 := Var_Expr; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_2, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15390") else null)); 
--# expr-done 9330
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15386") else null)); 
--# expr-done 9331
--# expr-start 9361 '<LogicAll at ast.py:15396>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15396
--# expr-start 9360 '<Map at ast.py:15396>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15396
   --# expr-start 9333 '<FieldAccess for match_param_list at ast.py:15394>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15394
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9332 '<FieldAccess for params at ast.py:15395>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15395
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_3 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9332
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Var_Expr.Node, Params => Fld_3, Is_Dottable_Subp => Dottable_Subp, Env => Env, E_Info => Var_Expr.Info);
--# end
--# expr-done 9333
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_4;
      begin
         for Item_276 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_276
      --# expr-start 9359 '<If at ast.py:15397>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15397
--# expr-start 9334 '<FieldAccess for has_matched at ast.py:15398>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15398
Fld_24 := Item_276.Has_Matched;
--# expr-done 9334
if Fld_24 then
   --# expr-start 9358 '<And at ast.py:15399>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15399
--# expr-start 9340 '<FieldAccess for call_argument_equation at ast.py:15399>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15399
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9336 '<FieldAccess for formal_decl at ast.py:15400>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15400
--# expr-start 9335 '<FieldAccess for formal at ast.py:15400>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15400
Fld_5 := Item_276.Formal;
--# expr-done 9335
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 9336
--# expr-start 9339 '<FieldAccess for expr at ast.py:15401>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15401
--# expr-start 9338 '<FieldAccess for assoc at ast.py:15401>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15401
--# expr-start 9337 '<FieldAccess for actual at ast.py:15401>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15401
Fld_7 := Item_276.Actual;
--# expr-done 9337
Fld_8 := Fld_7.Assoc;
--# expr-done 9338
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 9339
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_10 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Var_Expr.Node, Param => Fld_6, Arg => Fld_9, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 9340
--# expr-start 9357 '<If at ast.py:15402>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15402
--# expr-start 9343 '<IsNull at ast.py:15405>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15405
--# expr-start 9342 '<FieldAccess for name at ast.py:15405>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15405
--# expr-start 9341 '<FieldAccess for actual at ast.py:15405>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15405
Fld_22 := Item_276.Actual;
--# expr-done 9341
Fld_23 := Fld_22.Name;
--# expr-done 9342
Is_Null := Fld_23 = null; 
--# expr-done 9343
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15406") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 9356 '<Bind at ast.py:15407>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15407
--# expr-start 9346 '<FieldAccess for ref_var at ast.py:15408>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15408
--# expr-start 9345 '<FieldAccess for name at ast.py:15408>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15408
--# expr-start 9344 '<FieldAccess for actual at ast.py:15408>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15408
Fld_11 := Item_276.Actual;
--# expr-done 9344
Fld_12 := Fld_11.Name;
--# expr-done 9345
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_12);
--# end
--# expr-done 9346
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 9355 '<Let at ast.py:15409>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15409
--# scope-start
--# expr-start 9348 '<FieldAccess for formal_decl at ast.py:15409>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15409
--# expr-start 9347 '<FieldAccess for formal at ast.py:15409>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15409
Fld_14 := Item_276.Formal;
--# expr-done 9347
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_15 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 9348
N := Fld_15; 
--# bind n N
--# expr-start 9354 '<FieldAccess for entity_no_md at ast.py:15410>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15410
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9349 '<FieldAccess for node at ast.py:15411>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15411
Fld_16 := N.Node;
--# expr-done 9349
      Cast_Result_3 := Fld_16;
--# expr-start 9351 '<FieldAccess for rebindings at ast.py:15412>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15412
--# expr-start 9350 '<FieldAccess for info at ast.py:15412>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15412
Fld_17 := N.Info;
--# expr-done 9350
Fld_18 := Fld_17.Rebindings;
--# expr-done 9351
--# expr-start 9353 '<FieldAccess for from_rebound at ast.py:15413>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15413
--# expr-start 9352 '<FieldAccess for info at ast.py:15413>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15413
Fld_19 := N.Info;
--# expr-done 9352
Fld_20 := Fld_19.From_Rebound;
--# expr-done 9353
--# property-call-start AdaNode.entity_no_md
Fld_21 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Cast_Result_3, Rebindings => Fld_18, From_Rebound => Fld_20);
--# end
--# expr-done 9354
Let_Result := Fld_21; 
--# end
--# expr-done 9355
Bind_Result_2 := Solver.Create_Assign (Fld_13, Let_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15407") else null)); 
--# expr-done 9356
   If_Result := Bind_Result_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 9357
And_Pred_1 := Create_And (Fld_10, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15399") else null)); 
--# expr-done 9358
   If_Result_1 := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15417") else null)); 
   If_Result_1 := False_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 9359
         declare
            Item_To_Append : constant Logic_Equation := If_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3032;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9360
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15396") else null)); 
--# expr-done 9361
And_Pred_2 := Create_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15386") else null)); 
--# expr-done 9362
Scope_Result_1 := And_Pred_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3030;
   Result_Var_1 := Scope_Result_1;
else
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:15420") else null)); 
   Result_Var_1 := False_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
Let_Result_1 := Result_Var_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 9363
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1096;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1096;
                     Finalizer_Scope_3030;
                     Finalizer_Scope_3032;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Subprogram_Equation;
--# end
--# property-start CallExpr.check_for_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15423
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_For_Type
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Array_Type_Def;
Match_0 : Bare_Assoc_List;
Match_1 : Bare_Bin_Op;
Match_2 : Bare_Subtype_Indication;
Match_3 : Bare_Attribute_Ref;
Match_4 : Bare_Ada_Node;
Var_Expr_3 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_4 : Bare_Assoc_List;
Var_Expr_5 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Integer;
Len : Integer;
Is_Equal : Boolean;
Fld_4 : Integer;
Is_Equal_1 : Boolean;
Fld_5 : Integer;
Is_Equal_2 : Boolean;
Fld_6 : Integer;
Is_Equal_3 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Assoc_List;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Bare_Bin_Op;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Bare_Subtype_Indication;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Bare_Attribute_Ref;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Match_Result : Boolean;
Scope_Result_6 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Access_Def;
Cast_Result_5 : Internal_Entity_Access_To_Subp_Def;
Fld_8 : Internal_Entity_Subp_Spec;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_2 : Boolean;
If_Result : Boolean;
Fld_11 : Boolean;
Fld_12 : Bare_Ada_Node;
Cast_Result_6 : Bare_Assoc_List;
Len_1 : Integer;
Is_Equal_4 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_3 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_13 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_17 : Boolean;
If_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_18 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_4 : Boolean;
If_Result_5 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_5 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_5 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9397 '<Block at ???>' Let_Result_5 None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 9364 '<FieldAccess for array_def_with_deref at ast.py:15435>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15435
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 9364
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
--# expr-start 9396 '<bind at ast.py:15437>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15437
--# expr-start 9365 '<FieldAccess for origin_node at ast.py:15437>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15437
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 9365
Bound_Origin := Fld_1; 
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_1
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 9376 '<Match at ast.py:15440>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15440
--# expr-start 9366 '<FieldAccess for suffix at ast.py:15440>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15440
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Call_Expr_F_Suffix;
--# expr-done 9366
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_2; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Assoc_List =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind al Match_0
--# expr-start 9369 '<Eq at ast.py:15442>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15442
--# expr-start 9367 '<FieldAccess for array_ndims at ast.py:15442>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15442
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 9367
--# expr-start 9368 '<length at ast.py:15442>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15442
Len := Length (Match_0); 
--# expr-done 9368
Is_Equal := Fld_3 = Len; 
--# expr-done 9369
Let_Result := Is_Equal; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 9371 '<Eq at ast.py:15445>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15445
--# expr-start 9370 '<FieldAccess for array_ndims at ast.py:15445>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15445
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 9370
Is_Equal_1 := Fld_4 = 1; 
--# expr-done 9371
Let_Result_1 := Is_Equal_1; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 9373 '<Eq at ast.py:15446>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15446
--# expr-start 9372 '<FieldAccess for array_ndims at ast.py:15446>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15446
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 9372
Is_Equal_2 := Fld_5 = 1; 
--# expr-done 9373
Let_Result_2 := Is_Equal_2; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 9375 '<Eq at ast.py:15447>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15447
--# expr-start 9374 '<FieldAccess for array_ndims at ast.py:15447>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15447
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 9374
Is_Equal_3 := Fld_6 = 1; 
--# expr-done 9375
Let_Result_3 := Is_Equal_3; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Base_Assoc_List | Ada_Basic_Decl_List .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Call_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 9376
Scope_Result_6 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_6;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result := True;
else
--# expr-start 9378 '<Cast to AccessToSubpDef at ast.py:15453>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15453
--# expr-start 9377 '<FieldAccess for access_def at ast.py:15453>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15453
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9377
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_5 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 9378
Var_Expr_3 := Cast_Result_5;
if Var_Expr_3 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr_3
--# expr-start 9381 '<FieldAccess for is_matching_param_list at ast.py:15455>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15455
--# expr-start 9379 '<FieldAccess for subp_spec at ast.py:15455>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15455
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_3.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_3.Info);
--# expr-done 9379
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9380 '<FieldAccess for params at ast.py:15455>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15455
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_9 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9380
--# property-call-start BaseFormalParamHolder.is_matching_param_list
Fld_10 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Is_Matching_Param_List (Node => Fld_8.Node, Params => Fld_9, Is_Dottable_Subp => False, Env => Env);
--# end
--# expr-done 9381
Scope_Result_7 := Fld_10;
   --# end
   Result_Var_2 := Scope_Result_7;
else
   Result_Var_2 := False;
end if;
   If_Result := Result_Var_2;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 9382 '<FieldAccess for has_ud_indexing at ast.py:15459>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15459
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9382
if Fld_11 then
--# expr-start 9384 '<Cast to AssocList at ast.py:15460>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15460
--# expr-start 9383 '<FieldAccess for suffix at ast.py:15460>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15460
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Call_Expr_F_Suffix;
--# expr-done 9383
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_6 := Fld_12;
   else
         Cast_Result_6 := No_Bare_Ada_Node;
   end if;
--# expr-done 9384
Var_Expr_4 := Cast_Result_6;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind al Var_Expr_4
--# expr-start 9386 '<Eq at ast.py:15460>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15460
--# expr-start 9385 '<length at ast.py:15460>' Len_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15460
Len_1 := Length (Var_Expr_4); 
--# expr-done 9385
Is_Equal_4 := Len_1 = 1; 
--# expr-done 9386
Scope_Result_8 := Is_Equal_4;
   --# end
   Result_Var_3 := Scope_Result_8;
else
   Result_Var_3 := False;
end if;
   If_Result_1 := Result_Var_3;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
--# expr-start 9388 '<Cast to CallExpr at ast.py:15463>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15463
--# expr-start 9387 '<parent at ast.py:15463>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15463
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 9387
   if Fld_13.Node = null
      or else Fld_13.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 9388
Var_Expr_5 := Cast_Result_7;
if Var_Expr_5 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_5
--# expr-start 9395 '<FieldAccess for check_for_type at ast.py:15469>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15469
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9394 '<If at ast.py:15469>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15469
--# expr-start 9389 '<FieldAccess for check_array_slice at ast.py:15470>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15470
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.check_array_slice
Fld_17 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Ent.Node, Typ => Var_Expr_1, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 9389
if Fld_17 then
   If_Result_4 := Var_Expr_1;
else
   --# expr-start 9393 '<If at ast.py:15472>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15472
--# expr-start 9390 '<FieldAccess for is_iterable_type at ast.py:15474>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15474
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9390
if Fld_16 then
   --# expr-start 9391 '<FieldAccess for iterable_comp_type at ast.py:15475>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15475
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_14 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9391
   If_Result_3 := Fld_14;
else
   --# expr-start 9392 '<FieldAccess for comp_type at ast.py:15476>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:15476
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Var_Expr_1.Node, Is_Subscript => True, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9392
   If_Result_3 := Fld_15;
end if;
--# expr-done 9393
   If_Result_4 := If_Result_3;
end if;
--# expr-done 9394
--# property-call-start CallExpr.check_for_type
Fld_18 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Var_Expr_5.Node, Typ => If_Result_4, Env => Env, Origin => Bound_Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 9395
Scope_Result_9 := Fld_18;
   --# end
   Result_Var_4 := Scope_Result_9;
else
   Result_Var_4 := True;
end if;
   If_Result_5 := Result_Var_4;
else
   If_Result_5 := False;
end if;
Scope_Result_10 := If_Result_5;
   --# end
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := False;
end if;
Dyn_Var_Bind_Result := Result_Var_5; 
--# expr-done 9396
Let_Result_5 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 9397
         Property_Result := Let_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Check_For_Type;
--# end
   --
   --  Primitives for Bare_Defining_Name
   --
      procedure Initialize_Fields_For_Defining_Name
        (Self : Bare_Defining_Name
         ; Defining_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Defining_Name_F_Name := Defining_Name_F_Name;
      end Initialize_Fields_For_Defining_Name;
   function Defining_Name_F_Name
     (Node : Bare_Defining_Name) return Bare_Name
   is
   begin
         return Node.Defining_Name_F_Name;
   end;
--# property-start DefiningName.parent_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16401
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Parent_Scope
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1153 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1153 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1153;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9399 '<FieldAccess for parent_scope at ast.py:16401>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16401
--# expr-start 9398 '<FieldAccess for name at ast.py:16401>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16401
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 9398
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 9399
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1153;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1153;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Parent_Scope;
--# end
--# property-start DefiningName.scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16402
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Scope
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1154 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1154 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1154;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9401 '<FieldAccess for scope at ast.py:16402>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16402
--# expr-start 9400 '<FieldAccess for name at ast.py:16402>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16402
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 9400
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 9401
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1154;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1154;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Scope;
--# end
--# property-start DefiningName.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16403
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Relative_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9403 '<FieldAccess for relative_name at ast.py:16403>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16403
--# expr-start 9402 '<FieldAccess for name at ast.py:16403>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16403
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 9402
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9403
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Relative_Name;
--# end
--# property-start DefiningName.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16404
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Ref_Var
  (Node : Bare_Defining_Name
  )
   return Logic_Var
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9405 '<FieldAccess for ref_var at ast.py:16404>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16404
--# expr-start 9404 '<FieldAccess for name at ast.py:16404>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16404
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 9404
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 9405
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Ref_Var;
--# end
--# property-start DefiningName.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16405
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Env_Elements_Impl
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1157 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1157 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1157;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9407 '<FieldAccess for env_elements_impl at ast.py:16405>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16405
--# expr-start 9406 '<FieldAccess for name at ast.py:16405>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16405
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 9406
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 9407
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1157;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1157;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Env_Elements_Impl;
--# end
--# property-start DefiningName.formal_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16407
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Formal_Decl
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl;
            procedure Finalizer_Scope_1158 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Let_Result : Internal_Entity_Base_Formal_Param_Decl;
            procedure Finalizer_Scope_1158 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1158;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9412 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9411 '<Cast to BaseFormalParamDecl at ast.py:16415>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16415
--# expr-start 9410 '<Find at ast.py:16413>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16413
   --# expr-start 9408 '<FieldAccess for parents at ast.py:16413>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16413
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 9408
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_277 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_277
      --# expr-start 9409 '<IsA BaseFormalParamDecl at ast.py:16414>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16414
Is_A := Item_277.Node /= null 
and then Item_277.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 9409
      if Is_A then
         Find_Result := Item_277; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 9410
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9411
Let_Result := Cast_Result; 
--# end
--# expr-done 9412
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1158;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1158;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Formal_Decl;
--# end
--# property-start DefiningName.fully_qualified_name_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16417
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Fully_Qualified_Name_Impl
  (Node : Bare_Defining_Name
      ; Include_Profile : Boolean
      ; Suffix : String_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_profile Include_Profile
   --# bind suffix Suffix
   Property_Result : String_Type_Array_Access;
            procedure Finalizer_Scope_1159 with Inline_Always;
            procedure Finalizer_Scope_3045 with Inline_Always;
            procedure Finalizer_Scope_3046 with Inline_Always;
            procedure Finalizer_Scope_3047 with Inline_Always;
            procedure Finalizer_Scope_3049 with Inline_Always;
            procedure Finalizer_Scope_3050 with Inline_Always;
            procedure Finalizer_Scope_3051 with Inline_Always;
            procedure Finalizer_Scope_3052 with Inline_Always;
            procedure Finalizer_Scope_3053 with Inline_Always;
            procedure Finalizer_Scope_3054 with Inline_Always;
      Block_Var : String_Type_Array_Access;
Block_Var_1 : Internal_Entity_Basic_Decl;
Block_Var_2 : String_Type_Array_Access;
Block_Var_3 : Internal_Entity_Basic_Decl;
Block_Var_4 : Boolean;
Block_Var_5 : Boolean;
Block_Var_6 : String_Type_Array_Access;
Match_0 : Bare_Synthetic_Defining_Name;
Match_1 : Bare_Defining_Name;
Var_Expr : String_Type_Array_Access;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : String_Type_Array_Access;
Var_Expr_3 : Internal_Entity_Subunit;
Fld : Symbol_Type;
Fld_1 : String_Type;
Array_Lit : String_Type_Array_Access;
Fld_2 : Bare_Single_Tok_Node_Array_Access;
Fld_3 : String_Type;
Map_Result : String_Type_Array_Access;
Match_Prefix : Bare_Defining_Name;
Cast_Result : Bare_Synthetic_Defining_Name;
Let_Result : String_Type_Array_Access;
Scope_Result : String_Type_Array_Access;
Cast_Result_1 : Bare_Defining_Name;
Let_Result_1 : String_Type_Array_Access;
Scope_Result_1 : String_Type_Array_Access;
Match_Result : String_Type_Array_Access;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : String_Type;
Str : String_Type;
If_Result : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Len : Integer;
Arith_Result : Integer;
Is_Equal : Boolean;
If_Result_1 : String_Type;
Concat_Result_1 : String_Type;
Index_280 : Integer;
Map_Result_1 : String_Type_Array_Access;
Fld_6 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result_2 : Boolean;
Fld_7 : String_Type_Array_Access;
Scope_Result_2 : String_Type_Array_Access;
Result_Var : String_Type_Array_Access;
Concat_Result_2 : String_Type_Array_Access;
If_Result_3 : String_Type_Array_Access;
Scope_Result_3 : String_Type_Array_Access;
Result_Var_1 : String_Type_Array_Access;
Not_Val : Boolean;
Fld_8 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : String_Type_Array_Access;
Fld_9 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Subunit;
Fld_10 : Internal_Entity_Name;
Fld_11 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_4 : Bare_Single_Tok_Node_Array_Access;
Result_Var_2 : Bare_Single_Tok_Node_Array_Access;
Fld_12 : String_Type;
Map_Result_2 : String_Type_Array_Access;
Concat_Result_3 : String_Type_Array_Access;
Scope_Result_5 : String_Type_Array_Access;
Result_Var_3 : String_Type_Array_Access;
Let_Result_2 : String_Type_Array_Access;
            procedure Finalizer_Scope_1159 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Concat_Result_3);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1159;
            procedure Finalizer_Scope_3045 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_3045;
            procedure Finalizer_Scope_3046 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3046;
            procedure Finalizer_Scope_3047 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3047;
            procedure Finalizer_Scope_3049 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Str);
                     Dec_Ref (If_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_3049;
            procedure Finalizer_Scope_3050 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_3050;
            procedure Finalizer_Scope_3051 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_3051;
            procedure Finalizer_Scope_3052 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_3052;
            procedure Finalizer_Scope_3053 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_3053;
            procedure Finalizer_Scope_3054 is
            begin
                     Dec_Ref (Var_Expr_2);
            end Finalizer_Scope_3054;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9448 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 9419 '<Match at ast.py:16426>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16426
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Defining_Name_Range (Match_Prefix.Kind) is
         when Ada_Synthetic_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind scel Match_0
--# expr-start 9415 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 9414 '<FieldAccess for image at ast.py:16427>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16427
--# expr-start 9413 '<FieldAccess for name_symbol at ast.py:16427>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16427
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Match_0);
--# end
--# expr-done 9413
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Image (Self => Fld);
--# expr-done 9414
Array_Lit := Create_String_Type_Array (Internal_String_Type_Array'(1 => Fld_1)); 
--# expr-done 9415
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3045;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 9418 '<Map at ast.py:16428>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16428
   --# expr-start 9416 '<FieldAccess for as_single_tok_node_array at ast.py:16428>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16428
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_2 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Match_1);
--# end
--# expr-done 9416
      declare
         Map_Result_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld_2;
      begin
         for Item_278 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_278
      --# expr-start 9417 '<FieldAccess for text at ast.py:16428>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16428
      if Item_278 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Text (Node => Item_278);
--# expr-done 9417
         declare
            Item_To_Append : constant String_Type := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3047;
         end loop;
      end;
         Map_Result := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := String_Type_Vectors.Get
              (Map_Result_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9418
Let_Result_1 := Map_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3046;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 9419
Block_Var := Match_Result; Inc_Ref (Block_Var);
--# bind def_name_array Block_Var
--# expr-start 9420 '<FieldAccess for basic_decl_no_internal at ast.py:16430>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16430
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl_no_internal
Fld_4 := Libadalang.Implementation.Defining_Name_P_Basic_Decl_No_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9420
Block_Var_1 := Fld_4; 
--# bind bd Block_Var_1
--# expr-start 9431 '<Map at ast.py:16432>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16432
      declare
         Map_Result_1_Vec : String_Type_Vectors.Vector;
      begin
         Index_280 := 0;
      declare
         Collection : constant String_Type_Array_Access := Block_Var;
      begin
         for Item_279 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_279
               --# bind i Index_280
      --# expr-start 9430 '<Concat at ast.py:16435>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16435
--# expr-start 9424 '<Concat at ast.py:16433>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16433
--# expr-start 9423 '<If at ast.py:16434>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16434
if Include_Profile then
   --# expr-start 9421 '<FieldAccess for custom_id_text at ast.py:16434>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16434
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 9421
   If_Result := Fld_5;
else
   --# expr-start 9422 '<String at ast.py:16434>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16434
Str := Create_String (""); 
--# expr-done 9422
   If_Result := Str;
end if;
      Inc_Ref (If_Result);
--# expr-done 9423
Concat_Result := Concat_String (Item_279, If_Result); 
--# expr-done 9424
--# expr-start 9429 '<If at ast.py:16435>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16435
--# expr-start 9427 '<Eq at ast.py:16436>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16436
--# expr-start 9426 '<Arithmetic - at ast.py:16436>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16436
--# expr-start 9425 '<length at ast.py:16436>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16436
Len := Length (Block_Var); 
--# expr-done 9425
Arith_Result := (Len - 1); 
--# expr-done 9426
Is_Equal := Index_280 = Arith_Result; 
--# expr-done 9427
if Is_Equal then
   If_Result_1 := Suffix;
else
   --# expr-start 9428 '<String at ast.py:16436>' Str_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16436
Str_1 := Create_String (""); 
--# expr-done 9428
   If_Result_1 := Str_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 9429
Concat_Result_1 := Concat_String (Concat_Result, If_Result_1); 
--# expr-done 9430
         declare
            Item_To_Append : constant String_Type := Concat_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3049;
               Index_280 := Index_280 + 1;
         end loop;
      end;
         Map_Result_1 := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := String_Type_Vectors.Get
              (Map_Result_1_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 9431
Block_Var_2 := Map_Result_1; Inc_Ref (Block_Var_2);
--# bind self_name Block_Var_2
--# expr-start 9432 '<FieldAccess for parent_basic_decl at ast.py:16440>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16440
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_6 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 9432
Block_Var_3 := Fld_6; 
--# bind parent_decl Block_Var_3
--# expr-start 9433 '<IsA GenericDecl at ast.py:16441>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16441
Is_A := Block_Var_1.Node /= null 
and then Block_Var_1.Node.Kind in Ada_Generic_Decl; 
--# expr-done 9433
Block_Var_4 := Is_A; 
--# bind is_generic Block_Var_4
if Block_Var_4 then
   --# expr-start 9434 '<IsA GenericInstantiation at ast.py:16443>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16443
Is_A_1 := Block_Var_3.Node /= null 
and then Block_Var_3.Node.Kind in Ada_Generic_Instantiation; 
--# expr-done 9434
   If_Result_2 := Is_A_1;
else
   If_Result_2 := False;
end if;
Block_Var_5 := If_Result_2; 
--# bind is_instantiated Block_Var_5
--# expr-start 9440 '<If at ast.py:16446>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16446
--# expr-start 9435 '<Not at ast.py:16447>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16447
Not_Val := not (Block_Var_5); 
--# expr-done 9435
if Not_Val then
   --# expr-start 9436 '<FieldAccess for is_compilation_unit_root at ast.py:16447>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16447
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Block_Var_1.Node);
--# end
--# expr-done 9436
   If_Result_4 := Fld_8;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   If_Result_5 := Block_Var_2;
else
Var_Expr_1 := Block_Var_3;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 9437 '<FieldAccess for fully_qualified_name_string_array at ast.py:16449>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16449
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fully_qualified_name_string_array
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_String_Array (Node => Var_Expr_1.Node, Include_Profile => Include_Profile, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9437
Scope_Result_2 := Fld_7;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3050;
   Result_Var := Scope_Result_2;
else
   Result_Var := No_String_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Var_Expr := Result_Var;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_String_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 9439 '<If at ast.py:16455>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16455
if Block_Var_5 then
   If_Result_3 := Var_Expr;
else
   --# expr-start 9438 '<Concat at ast.py:16455>' Concat_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16455
Concat_Result_2 := Concat (Var_Expr, Block_Var_2); 
--# expr-done 9438
   If_Result_3 := Concat_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 9439
Scope_Result_3 := If_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3051;
   Result_Var_1 := Scope_Result_3;
else
   Result_Var_1 := No_String_Type_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_5 := Result_Var_1;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 9440
Block_Var_6 := If_Result_5; Inc_Ref (Block_Var_6);
--# bind fqn Block_Var_6
--# expr-start 9447 '<Concat at ast.py:16461>' Concat_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16461
--# expr-start 9446 '<Map at ast.py:16459>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16459
--# expr-start 9442 '<Cast to Subunit at ast.py:16459>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16459
--# expr-start 9441 '<parent at ast.py:16459>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16459
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Parent (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# expr-done 9441
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Subunit_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Subunit
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_2 := No_Entity_Subunit;
   end if;
--# expr-done 9442
Var_Expr_3 := Cast_Result_2;
if Var_Expr_3 /= No_Entity_Subunit then
   --# scope-start
--# expr-start 9444 '<FieldAccess for as_single_tok_node_array at ast.py:16459>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16459
--# expr-start 9443 '<FieldAccess for name at ast.py:16459>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16459
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Var_Expr_3.Node.Subunit_F_Name, Info => Var_Expr_3.Info);
--# expr-done 9443
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_11 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_10.Node);
--# end
--# expr-done 9444
Scope_Result_4 := Fld_11;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3052;
   Result_Var_2 := Scope_Result_4;
else
   Result_Var_2 := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
      declare
         Map_Result_2_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Result_Var_2;
      begin
         for Item_281 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_281
      --# expr-start 9445 '<FieldAccess for text at ast.py:16460>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16460
      if Item_281 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Text (Node => Item_281);
--# expr-done 9445
         declare
            Item_To_Append : constant String_Type := Fld_12;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3053;
         end loop;
      end;
         Map_Result_2 := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := String_Type_Vectors.Get
              (Map_Result_2_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 9446
Concat_Result_3 := Concat (Map_Result_2, Block_Var_6); 
--# expr-done 9447
Var_Expr_2 := Concat_Result_3;
      Inc_Ref (Var_Expr_2);
if Var_Expr_2 /= No_String_Type_Array_Type then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_5 := Var_Expr_2;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3054;
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := Block_Var_6;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_2 := Result_Var_3; Inc_Ref (Let_Result_2);
--# end
--# expr-done 9448
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1159;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1159;
                     Finalizer_Scope_3045;
                     Finalizer_Scope_3046;
                     Finalizer_Scope_3047;
                     Finalizer_Scope_3049;
                     Finalizer_Scope_3050;
                     Finalizer_Scope_3051;
                     Finalizer_Scope_3052;
                     Finalizer_Scope_3053;
                     Finalizer_Scope_3054;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Fully_Qualified_Name_Impl;
--# end
--# property-start DefiningName.canonical_fully_qualified_name_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16463
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Canonical_Fully_Qualified_Name_Impl
  (Node : Bare_Defining_Name
      ; Include_Profile : Boolean
      ; Suffix : String_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_profile Include_Profile
   --# bind suffix Suffix
   Property_Result : String_Type;
            procedure Finalizer_Scope_1160 with Inline_Always;
            procedure Finalizer_Scope_3056 with Inline_Always;
      Str : String_Type;
Fld : String_Type_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Fld_1 : String_Type;
Map_Result_1 : String_Type_Array_Access;
Join_Result : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_1160 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Join_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1160;
            procedure Finalizer_Scope_3056 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3056;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9456 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9455 '<Join at ast.py:16471>' Join_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16471
--# expr-start 9449 '<String at ast.py:16471>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16471
Str := Create_String ("."); 
--# expr-done 9449
--# expr-start 9454 '<Map at ast.py:16477>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16477
   --# expr-start 9452 '<Map at ast.py:16477>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16477
   --# expr-start 9450 '<FieldAccess for fully_qualified_name_impl at ast.py:16472>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16472
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.fully_qualified_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => Include_Profile, Suffix => Suffix, E_Info => Ent.Info);
--# end
--# expr-done 9450
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant String_Type_Array_Access := Fld;
      begin
         for Item_283 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_283
      --# expr-start 9451 '<to_symbol at ast.py:16477>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16477
Sym := String_To_Symbol (Self, Self.Unit.Context, Item_283); 
--# expr-done 9451
         declare
            Item_To_Append : constant Symbol_Type := Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9452
      declare
         Map_Result_1_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Symbol_Type_Array_Access := Map_Result;
      begin
         for Item_282 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_282
      --# expr-start 9453 '<FieldAccess for image at ast.py:16477>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16477
      if Item_282 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Image (Self => Item_282);
--# expr-done 9453
         declare
            Item_To_Append : constant String_Type := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3056;
         end loop;
      end;
         Map_Result_1 := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := String_Type_Vectors.Get
              (Map_Result_1_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 9454
Join_Result := Join_Strings (Str, Map_Result_1); 
--# expr-done 9455
Let_Result := Join_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9456
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1160;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1160;
                     Finalizer_Scope_3056;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Canonical_Fully_Qualified_Name_Impl;
--# end
--# property-start DefiningName.unique_identifying_name_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16480
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Unique_Identifying_Name_Impl
  (Node : Bare_Defining_Name
      ; Suffix : String_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind suffix Suffix
   Property_Result : String_Type;
            procedure Finalizer_Scope_1161 with Inline_Always;
            procedure Finalizer_Scope_3057 with Inline_Always;
            procedure Finalizer_Scope_3058 with Inline_Always;
      Match_0 : Internal_Entity_Anonymous_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : String_Type;
Fld_2 : String_Type;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Let_Result : String_Type;
Scope_Result : String_Type;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : String_Type;
Scope_Result_1 : String_Type;
Match_Result : String_Type;
Let_Result_2 : String_Type;
            procedure Finalizer_Scope_1161 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1161;
            procedure Finalizer_Scope_3057 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3057;
            procedure Finalizer_Scope_3058 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3058;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9461 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 9460 '<Match at ast.py:16485>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16485
--# expr-start 9457 '<FieldAccess for basic_decl at ast.py:16485>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16485
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9457
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind atd Match_0
--# expr-start 9458 '<FieldAccess for custom_id_text at ast.py:16486>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16486
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 9458
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3057;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Type_Decl | Ada_Concrete_Type_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 9459 '<FieldAccess for canonical_fully_qualified_name_impl at ast.py:16487>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16487
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_fully_qualified_name_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Canonical_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => True, Suffix => Suffix, E_Info => Ent.Info);
--# end
--# expr-done 9459
Let_Result_1 := Fld_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3058;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 9460
Let_Result_2 := Match_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 9461
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1161;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1161;
                     Finalizer_Scope_3057;
                     Finalizer_Scope_3058;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Unique_Identifying_Name_Impl;
--# end
--# property-start DefiningName.canonical_fully_qualified_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16493
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Canonical_Fully_Qualified_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1162 with Inline_Always;
      Str : String_Type;
Fld : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_1162 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1162;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9464 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9463 '<FieldAccess for canonical_fully_qualified_name_impl at ast.py:16499>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16499
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9462 '<String at ast.py:16501>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16501
Str := Create_String (""); 
--# expr-done 9462
--# property-call-start DefiningName.canonical_fully_qualified_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Canonical_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => False, Suffix => Str, E_Info => Ent.Info);
--# end
--# expr-done 9463
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 9464
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1162;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1162;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Canonical_Fully_Qualified_Name;
--# end
--# property-start DefiningName.unique_identifying_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16504
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Unique_Identifying_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1163 with Inline_Always;
      Str : String_Type;
Fld : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_1163 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1163;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9467 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9466 '<FieldAccess for unique_identifying_name_impl at ast.py:16516>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16516
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9465 '<String at ast.py:16516>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16516
Str := Create_String (""); 
--# expr-done 9465
--# property-call-start DefiningName.unique_identifying_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Unique_Identifying_Name_Impl (Node => Ent.Node, Suffix => Str, E_Info => Ent.Info);
--# end
--# expr-done 9466
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 9467
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1163;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1163;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Unique_Identifying_Name;
--# end
--# property-start DefiningName.fully_qualified_name_array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16518
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Fully_Qualified_Name_Array
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1164 with Inline_Always;
      Str : String_Type;
Fld : String_Type_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1164 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1164;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9472 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9471 '<Map at ast.py:16527>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16527
   --# expr-start 9469 '<FieldAccess for fully_qualified_name_impl at ast.py:16524>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16524
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9468 '<String at ast.py:16526>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16526
Str := Create_String (""); 
--# expr-done 9468
--# property-call-start DefiningName.fully_qualified_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => False, Suffix => Str, E_Info => Ent.Info);
--# end
--# expr-done 9469
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant String_Type_Array_Access := Fld;
      begin
         for Item_284 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_284
      --# expr-start 9470 '<to_symbol at ast.py:16527>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16527
Sym := String_To_Symbol (Self, Self.Unit.Context, Item_284); 
--# expr-done 9470
         declare
            Item_To_Append : constant Symbol_Type := Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9471
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9472
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1164;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1164;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Fully_Qualified_Name_Array;
--# end
--# property-start DefiningName.fully_qualified_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16529
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Fully_Qualified_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1165 with Inline_Always;
      Str : String_Type;
Str_1 : String_Type;
Fld : String_Type_Array_Access;
Join_Result : String_Type;
Let_Result : String_Type;
            procedure Finalizer_Scope_1165 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld);
                     Dec_Ref (Join_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1165;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9477 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9476 '<Join at ast.py:16534>' Join_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16534
--# expr-start 9473 '<String at ast.py:16534>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16534
Str := Create_String ("."); 
--# expr-done 9473
--# expr-start 9475 '<FieldAccess for fully_qualified_name_impl at ast.py:16534>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16534
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9474 '<String at ast.py:16536>' Str_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16536
Str_1 := Create_String (""); 
--# expr-done 9474
--# property-call-start DefiningName.fully_qualified_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => False, Suffix => Str_1, E_Info => Ent.Info);
--# end
--# expr-done 9475
Join_Result := Join_Strings (Str, Fld); 
--# expr-done 9476
Let_Result := Join_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9477
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1165;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1165;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Fully_Qualified_Name;
--# end
--# property-start DefiningName.all_env_els_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16539
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Env_Els_Impl
  (Node : Bare_Defining_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1166 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1166 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1166;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9480 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9479 '<FieldAccess for all_env_els_impl at ast.py:16545>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16545
--# expr-start 9478 '<FieldAccess for name at ast.py:16545>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16545
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 9478
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 9479
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 9480
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1166;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1166;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_All_Env_Els_Impl;
--# end
--# property-start DefiningName.basic_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16547
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Basic_Decl
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1167 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1167 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1167;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Basic_Decl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 9485 '<as_entity at ast.py:16549>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16549
--# expr-start 9484 '<Cast to BasicDecl at ast.py:16549>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16549
--# expr-start 9483 '<Find at ast.py:16548>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16548
   --# expr-start 9481 '<FieldAccess for parents at ast.py:16548>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16548
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 9481
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_285 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_285
      --# expr-start 9482 '<IsA BasicDecl at ast.py:16548>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16548
Is_A := Item_285 /= null 
and then Item_285.Kind in Ada_Basic_Decl; 
--# expr-done 9482
      if Is_A then
         Find_Result := Item_285; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 9483
   if Find_Result = null
      or else Find_Result.Kind in Ada_Basic_Decl
   then
      Cast_Result := Find_Result;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9484
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 9485
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_1167;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1167;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Basic_Decl;
--# end
--# property-start DefiningName.basic_decl_no_internal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16554
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Basic_Decl_No_Internal
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 9486 '<FieldAccess for basic_decl at ast.py:16555>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16555
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9486
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr
--# expr-start 9490 '<If at ast.py:16556>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16556
--# expr-start 9487 '<IsA GenericPackageInternal, GenericSubpInternal, SingleTaskTypeDecl at ast.py:16557>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16557
Is_A := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Generic_Package_Internal_Range | Ada_Generic_Subp_Internal_Range | Ada_Single_Task_Type_Decl_Range; 
--# expr-done 9487
if Is_A then
   --# expr-start 9489 '<Cast to BasicDecl at ast.py:16559>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16559
--# expr-start 9488 '<parent at ast.py:16559>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16559
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 9488
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9489
   If_Result := Cast_Result;
else
   If_Result := Var_Expr;
end if;
--# expr-done 9490
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Basic_Decl_No_Internal;
--# end
--# property-start DefiningName.find_refs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16567
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Refs
  (Node : Bare_Defining_Name
      ; Root : Internal_Entity
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1170 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1170 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1170;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9493 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9492 '<FieldAccess for find_refs_impl at ast.py:16574>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16574
--# expr-start 9491 '<FieldAccess for canonical_part at ast.py:16574>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16574
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_part
Fld := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9491
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.find_refs_impl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Find_Refs_Impl (Node => Fld.Node, Root => Root, Skip_Name => Self, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 9492
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 9493
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1170;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1170;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_Refs;
--# end
--# property-start DefiningName.find_refs_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16576
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Refs_Impl
  (Node : Bare_Defining_Name
      ; Root : Internal_Entity
      ; Skip_Name : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind skip_name Skip_Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1171 with Inline_Always;
            procedure Finalizer_Scope_3062 with Inline_Always;
            procedure Finalizer_Scope_3064 with Inline_Always;
            procedure Finalizer_Scope_3065 with Inline_Always;
            procedure Finalizer_Scope_3066 with Inline_Always;
      Var_Expr : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Base_Id;
Var_Expr_2 : Ref_Result_Kind;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_2 : Internal_Ref_Result_Array_Access;
Map_Result_1 : Internal_Ref_Result_Array_Access;
Scope_Result : Internal_Ref_Result_Array_Access;
Result_Var : Internal_Ref_Result_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Id;
Fld_3 : Ref_Result_Kind;
New_Struct : Internal_Ref_Result;
Singleton : Internal_Ref_Result_Array_Access;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Ref_Result_Array_Access;
Scope_Result_1 : Internal_Ref_Result_Array_Access;
Result_Var_1 : Internal_Ref_Result_Array_Access;
Scope_Result_2 : Internal_Ref_Result_Array_Access;
Result_Var_2 : Internal_Ref_Result_Array_Access;
Concat_Result : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1171 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1171;
            procedure Finalizer_Scope_3062 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_3062;
            procedure Finalizer_Scope_3064 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3064;
            procedure Finalizer_Scope_3065 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_3065;
            procedure Finalizer_Scope_3066 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_3066;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9509 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9508 '<Concat at ast.py:16590>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16590
--# expr-start 9494 '<children at ast.py:16586>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16586
      if Root.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 9494
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 9501 '<Mapcat at ast.py:16589>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16589
   --# expr-start 9499 '<Filter at ast.py:16587>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16587
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr;
      begin
         for Item_287 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_287
         --# expr-start 9498 '<Not at ast.py:16588>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16588
--# expr-start 9495 '<IsNull at ast.py:16588>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16588
Is_Null := Item_287.Node = null; 
--# expr-done 9495
if Is_Null then
   If_Result := True;
else
   --# expr-start 9497 '<Eq at ast.py:16588>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16588
--# expr-start 9496 '<FieldAccess for node at ast.py:16588>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16588
Fld_1 := Item_287.Node;
--# expr-done 9496
Cast_Expr := Skip_Name; 
      Cast_Result := Cast_Expr;
Is_Equal := Fld_1 = Cast_Result; 
--# expr-done 9497
   If_Result := Is_Equal;
end if;
Not_Val := not (If_Result); 
--# expr-done 9498
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_287;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9499
      declare
         Map_Result_1_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_286 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_286
      --# expr-start 9500 '<FieldAccess for find_refs_impl at ast.py:16589>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16589
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.find_refs_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Refs_Impl (Node => Ent.Node, Root => Item_286, Skip_Name => Skip_Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9500
         for Item_To_Append of
               Fld_2.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3064;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 9501
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3062;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 9502 '<Cast to BaseId at ast.py:16591>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16591
Cast_Expr_1 := Root; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 9502
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Base_Id then
   --# scope-start
   --# bind id Var_Expr_1
--# expr-start 9503 '<FieldAccess for is_referenced_by at ast.py:16592>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16592
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_referenced_by
Fld_3 := Libadalang.Implementation.Defining_Name_P_Is_Referenced_By (Node => Ent.Node, Id => Var_Expr_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9503
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Ref then
   --# scope-start
   --# bind ref_kind Var_Expr_2
--# expr-start 9507 '<If at ast.py:16593>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16593
--# expr-start 9504 '<Eq at ast.py:16594>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16594
Is_Equal_1 := Var_Expr_2 = Precise; 
--# expr-done 9504
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 9505 '<Eq at ast.py:16594>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16594
Is_Equal_2 := Var_Expr_2 = Imprecise; 
--# expr-done 9505
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   --# expr-start 9506 '<New RefResult at ast.py:16597>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16597
New_Struct := (Kind => Var_Expr_2, Ref => Var_Expr_1); 
--# expr-done 9506
                Singleton := Create_Internal_Ref_Result_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
   If_Result_2 := Singleton;
else
   If_Result_2 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 9507
Scope_Result_1 := If_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3066;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3065;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var, Result_Var_2); 
--# expr-done 9508
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9509
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1171;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1171;
                     Finalizer_Scope_3062;
                     Finalizer_Scope_3064;
                     Finalizer_Scope_3065;
                     Finalizer_Scope_3066;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_Refs_Impl;
--# end
--# property-start DefiningName.is_derivable_equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16603
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Derivable_Equal
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Type_Expr;
Sym : Symbol_Type;
Fld : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Internal_Entity_Type_Expr;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Derivable_Equal,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 9518 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9511 '<FieldAccess for name_is at ast.py:16612>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16612
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9510 '<Symbol "=" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 9510
--# property-call-start Name.name_is
Fld := Libadalang.Implementation.Name_P_Name_Is (Node => Self, Sym => Sym);
--# end
--# expr-done 9511
if Fld then
--# expr-start 9513 '<FieldAccess for subp_spec_or_null at ast.py:16613>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16613
--# expr-start 9512 '<FieldAccess for basic_decl at ast.py:16613>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16613
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9512
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 9513
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind s Var_Expr
--# expr-start 9517 '<Eq at ast.py:16614>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16614
--# expr-start 9514 '<FieldAccess for returns at ast.py:16614>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16614
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 9514
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 9515 '<FieldAccess for designated_type_decl at ast.py:16614>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16614
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_4 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9515
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
--# expr-start 9516 '<FieldAccess for bool_type at ast.py:16614>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16614
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Ent.Node);
--# end
--# expr-done 9516
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 9517
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 9518
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Is_Derivable_Equal;
--# end
--# property-start DefiningName.is_potential_reference /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16618
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Potential_Reference
  (Node : Bare_Defining_Name
      ; Symbol : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind symbol Symbol
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Potential_Reference,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Symbol);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 9523 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9519 '<FieldAccess for name_is at ast.py:16625>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16625
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld := Libadalang.Implementation.Name_P_Name_Is (Node => Self, Sym => Symbol);
--# end
--# expr-done 9519
if Fld then
   If_Result_1 := True;
else
--# expr-start 9520 '<FieldAccess for is_derivable_equal at ast.py:16626>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16626
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_derivable_equal
Fld_1 := Libadalang.Implementation.Defining_Name_P_Is_Derivable_Equal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9520
if Fld_1 then
   --# expr-start 9522 '<Eq at ast.py:16626>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16626
--# expr-start 9521 '<Symbol "/=" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 9521
Is_Equal := Symbol = Sym; 
--# expr-done 9522
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 9523
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Is_Potential_Reference;
--# end
--# property-start DefiningName.is_referenced_by /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16629
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Referenced_By
  (Node : Bare_Defining_Name
      ; Id : Internal_Entity_Base_Id
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Ref_Result_Kind
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind id Id
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Ref_Result_Kind;
            procedure Finalizer_Scope_3069 with Inline_Always;
            procedure Finalizer_Scope_3072 with Inline_Always;
      Var_Expr : Internal_Refd_Def;
Canon : Bare_Defining_Name;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Internal_Entity_Defining_Name;
Var_Expr_3 : Internal_Entity_Basic_Decl_Array_Access;
Fld : Internal_Entity_Defining_Name;
New_Struct : Internal_Refd_Def;
Fld_1 : Internal_Refd_Def;
Fld_2 : Boolean;
If_Result : Internal_Refd_Def;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_5 : Bare_Defining_Name;
Scope_Result_1 : Bare_Defining_Name;
Result_Var_1 : Bare_Defining_Name;
Fld_6 : Ref_Result_Kind;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Defining_Name;
Is_Equal : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl_Array_Access;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Bare_Defining_Name;
Cast_Result_1 : Bare_Defining_Name;
Is_Equal_1 : Boolean;
Quantifier_Result : Boolean;
Fld_11 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Ref_Result_Kind;
Let_Result : Ref_Result_Kind;
Scope_Result_3 : Ref_Result_Kind;
Result_Var_3 : Ref_Result_Kind;
Fld_12 : Symbol_Type;
Fld_13 : Boolean;
If_Result_4 : Ref_Result_Kind;
Let_Result_1 : Ref_Result_Kind;
            procedure Finalizer_Scope_3069 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3069;
            procedure Finalizer_Scope_3072 is
            begin
                     Dec_Ref (Var_Expr_3);
            end Finalizer_Scope_3072;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9546 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 9545 '<If at ast.py:16643>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16643
--# expr-start 9525 '<FieldAccess for is_potential_reference at ast.py:16644>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16644
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9524 '<FieldAccess for name_symbol at ast.py:16644>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16644
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Id.Node);
--# end
--# expr-done 9524
--# property-call-start DefiningName.is_potential_reference
Fld_13 := Libadalang.Implementation.Defining_Name_P_Is_Potential_Reference (Node => Ent.Node, Symbol => Fld_12, E_Info => Ent.Info);
--# end
--# expr-done 9525
if Fld_13 then
--# expr-start 9530 '<If at ast.py:16646>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16646
--# expr-start 9526 '<FieldAccess for is_defining at ast.py:16646>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16646
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld_2 := Libadalang.Implementation.Name_P_Is_Defining (Node => Id.Node);
--# end
--# expr-done 9526
if Fld_2 then
   --# expr-start 9528 '<New RefdDef at ast.py:16647>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16647
--# expr-start 9527 '<FieldAccess for enclosing_defining_name at ast.py:16647>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16647
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Id.Node, E_Info => Id.Info);
--# end
--# expr-done 9527
New_Struct := (Def_Name => Fld, Kind => Precise); 
--# expr-done 9528
   If_Result := New_Struct;
else
   --# expr-start 9529 '<FieldAccess for failsafe_referenced_def_name at ast.py:16649>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16649
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.failsafe_referenced_def_name
Fld_1 := Libadalang.Implementation.Name_P_Failsafe_Referenced_Def_Name (Node => Id.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Id.Info);
--# end
--# expr-done 9529
   If_Result := Fld_1;
end if;
--# expr-done 9530
Var_Expr := If_Result;
if Var_Expr /= No_Refd_Def then
   --# scope-start
   --# bind def_res Var_Expr
--# expr-start 9544 '<Let at ast.py:16652>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16652
--# scope-start
--# expr-start 9531 '<FieldAccess for def_name at ast.py:16653>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16653
Fld_3 := Var_Expr.Def_Name;
--# expr-done 9531
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 9532 '<FieldAccess for canonical_part at ast.py:16653>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16653
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_part
Fld_4 := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 9532
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 9533 '<FieldAccess for node at ast.py:16653>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16653
Fld_5 := Var_Expr_1.Node;
--# expr-done 9533
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Canon := Result_Var_1; 
--# bind canon Canon
--# expr-start 9543 '<If at ast.py:16655>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16655
--# expr-start 9534 '<Eq at ast.py:16658>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16658
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Canon; 
--# expr-done 9534
if Is_Equal then
   If_Result_2 := True;
else
--# expr-start 9536 '<FieldAccess for base_subp_declarations at ast.py:16663>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16663
--# expr-start 9535 '<FieldAccess for basic_decl at ast.py:16663>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16663
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9535
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.base_subp_declarations
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations (Node => Fld_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_7.Info);
--# end
--# expr-done 9536
Var_Expr_3 := Fld_8;
      Inc_Ref (Var_Expr_3);
if Var_Expr_3 /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind decls Var_Expr_3
--# expr-start 9540 '<AnyQuantifier at ast.py:16664>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16664
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Var_Expr_3;
      begin
         for Item_288 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_288
      --# expr-start 9539 '<Eq at ast.py:16665>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16665
--# expr-start 9538 '<FieldAccess for node at ast.py:16665>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16665
--# expr-start 9537 '<FieldAccess for defining_name at ast.py:16665>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16665
      if Item_288.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_288.Node, E_Info => Item_288.Info);
--# end
--# expr-done 9537
Fld_10 := Fld_9.Node;
--# expr-done 9538
      Cast_Result_1 := Fld_10;
Is_Equal_1 := Cast_Result_1 = Canon; 
--# expr-done 9539
      Quantifier_Result := Is_Equal_1;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 9540
if Quantifier_Result then
   --# expr-start 9541 '<FieldAccess for is_dispatching_call at ast.py:16667>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16667
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_11 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Id.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Id.Info);
--# end
--# expr-done 9541
   If_Result_1 := Fld_11;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
      Finalizer_Scope_3072;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
end if;
if If_Result_2 then
   --# expr-start 9542 '<FieldAccess for kind at ast.py:16670>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16670
Fld_6 := Var_Expr.Kind;
--# expr-done 9542
   If_Result_3 := Fld_6;
else
   If_Result_3 := No_Ref;
end if;
--# expr-done 9543
Let_Result := If_Result_3; 
--# end
--# expr-done 9544
Scope_Result_3 := Let_Result;
   --# end
      Finalizer_Scope_3069;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Ref;
end if;
   If_Result_4 := Result_Var_3;
else
   If_Result_4 := No_Ref;
end if;
--# expr-done 9545
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 9546
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3069;
                     Finalizer_Scope_3072;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Is_Referenced_By;
--# end
--# property-start DefiningName.find_all_references /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16679
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_All_References
  (Node : Bare_Defining_Name
      ; Units : Internal_Unit_Array_Access
      ; Follow_Renamings : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind units Units
   --# bind follow_renamings Follow_Renamings
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1175 with Inline_Always;
            procedure Finalizer_Scope_3074 with Inline_Always;
            procedure Finalizer_Scope_3075 with Inline_Always;
            procedure Finalizer_Scope_3076 with Inline_Always;
            procedure Finalizer_Scope_3077 with Inline_Always;
            procedure Finalizer_Scope_3078 with Inline_Always;
            procedure Finalizer_Scope_3081 with Inline_Always;
            procedure Finalizer_Scope_3083 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_2 : Internal_Unit_Array_Access;
Block_Var_3 : Internal_Ref_Result_Array_Access;
Var_Expr : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_1 : Bare_Ada_Node;
Var_Expr_2 : Internal_Entity_Renaming_Clause;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_3 : Internal_Entity_Basic_Decl;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_4 : Internal_Unit_Array_Access;
Map_Result : Internal_Unit_Array_Access;
Unique_Array : Internal_Unit_Array_Access;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Internal_Ref_Result_Array_Access;
Scope_Result_1 : Internal_Ref_Result_Array_Access;
Result_Var_1 : Internal_Ref_Result_Array_Access;
Map_Result_1 : Internal_Ref_Result_Array_Access;
Fld_7 : Internal_Entity_Base_Id;
Fld_8 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Renaming_Clause;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Defining_Name;
Fld_12 : Bare_Defining_Name;
Fld_13 : Bare_Defining_Name;
Cast_Result_2 : Bare_Defining_Name;
Is_Equal : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result_2 : Internal_Ref_Result_Array_Access;
Fld_14 : Internal_Entity_Base_Id;
Fld_15 : Internal_Entity_Array_Access;
Is_A_1 : Boolean;
Find_Result_1 : Internal_Entity;
Fld_16 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_17 : Internal_Entity_Defining_Name;
Fld_18 : Internal_Ref_Result_Array_Access;
Scope_Result_3 : Internal_Ref_Result_Array_Access;
Result_Var_3 : Internal_Ref_Result_Array_Access;
Map_Result_3 : Internal_Ref_Result_Array_Access;
Concat_Result : Internal_Ref_Result_Array_Access;
If_Result_1 : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1175 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Singleton);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1175;
            procedure Finalizer_Scope_3074 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_3074;
            procedure Finalizer_Scope_3075 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3075;
            procedure Finalizer_Scope_3076 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_3076;
            procedure Finalizer_Scope_3077 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3077;
            procedure Finalizer_Scope_3078 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3078;
            procedure Finalizer_Scope_3081 is
            begin
                     Dec_Ref (Fld_15);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_3081;
            procedure Finalizer_Scope_3083 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_3083;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9582 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9547 '<FieldAccess for canonical_part at ast.py:16691>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16691
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_part
Fld := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9547
Block_Var := Fld; 
--# bind dn Block_Var
--# expr-start 9551 '<bind at ast.py:16701>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16701
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 9549 '<FieldAccess for root_subp_declarations at ast.py:16703>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16703
--# expr-start 9548 '<FieldAccess for basic_decl at ast.py:16703>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16703
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 9548
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.root_subp_declarations
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Root_Subp_Declarations (Node => Fld_1.Node, Origin => Bound_Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 9549
Var_Expr := Fld_2;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3074;
   Result_Var := Scope_Result;
else
   --# expr-start 9550 '<FieldAccess for basic_decl at ast.py:16703>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16703
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 9550
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_3;
   Result_Var := Singleton;
end if;
      Inc_Ref (Result_Var);
Dyn_Var_Bind_Result := Result_Var; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9551
Block_Var_1 := Dyn_Var_Bind_Result; Inc_Ref (Block_Var_1);
--# bind bases Block_Var_1
--# expr-start 9554 '<unique at ast.py:16708>' Unique_Array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16708
--# expr-start 9553 '<Mapcat at ast.py:16706>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16706
      declare
         Map_Result_Vec : Internal_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
      begin
         for Item_289 of
               Collection.Items
         loop
   --# scope-start
               --# bind base Item_289
      --# expr-start 9552 '<FieldAccess for filter_is_imported_by at ast.py:16707>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16707
      if Item_289.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Ada_Node_P_Filter_Is_Imported_By (Node => Item_289.Node, Units => Units, Transitive => True);
--# expr-done 9552
         for Item_To_Append of
               Fld_4.Items
         loop
            Internal_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3075;
         end loop;
      end;
         Map_Result := Create_Internal_Unit_Array
           (Items_Count => Natural (Internal_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9553
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 9554
Block_Var_2 := Unique_Array; Inc_Ref (Block_Var_2);
--# bind all_units Block_Var_2
--# expr-start 9558 '<Mapcat at ast.py:16710>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16710
      declare
         Map_Result_1_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Unit_Array_Access := Block_Var_2;
      begin
         for Item_290 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_290
--# expr-start 9555 '<FieldAccess for root at ast.py:16710>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16710
      if Item_290 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Item_290.Ast_Root;
--# expr-done 9555
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind r Var_Expr_1
--# expr-start 9557 '<FieldAccess for find_refs_impl at ast.py:16711>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16711
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9556 '<as_bare_entity at ast.py:16711>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16711
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 9556
--# property-call-start DefiningName.find_refs_impl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Find_Refs_Impl (Node => Block_Var.Node, Root => If_Result, Skip_Name => Self, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 9557
Scope_Result_1 := Fld_6;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3077;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
         for Item_To_Append of
               Result_Var_1.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3076;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 9558
Block_Var_3 := Map_Result_1; Inc_Ref (Block_Var_3);
--# bind refs Block_Var_3
--# expr-start 9581 '<If at ast.py:16714>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16714
if Follow_Renamings then
   --# expr-start 9580 '<Concat at ast.py:16716>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16716
--# expr-start 9579 '<Mapcat at ast.py:16730>' Map_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16730
   --# expr-start 9570 '<Filter at ast.py:16717>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16717
      declare
         Map_Result_2_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Block_Var_3;
      begin
         for Item_292 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_292
--# expr-start 9563 '<Cast to RenamingClause at ast.py:16725>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16725
--# expr-start 9562 '<Find at ast.py:16723>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16723
   --# expr-start 9560 '<FieldAccess for parents at ast.py:16723>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16723
--# expr-start 9559 '<FieldAccess for ref at ast.py:16723>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16723
Fld_7 := Item_292.Ref;
--# expr-done 9559
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Parents (Node => Fld_7.Node, With_Self => True, E_Info => Fld_7.Info);
--# expr-done 9560
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_8;
      begin
         for Item_293 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_293
      --# expr-start 9561 '<IsA RenamingClause at ast.py:16724>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16724
Is_A := Item_293.Node /= null 
and then Item_293.Node.Kind in Ada_Renaming_Clause_Range; 
--# expr-done 9561
      if Is_A then
         Find_Result := Item_293; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 9562
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Renaming_Clause_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Renaming_Clause
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_1 := No_Entity_Renaming_Clause;
   end if;
--# expr-done 9563
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind r Var_Expr_2
--# expr-start 9569 '<Eq at ast.py:16727>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16727
--# expr-start 9567 '<FieldAccess for node at ast.py:16728>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16728
--# expr-start 9566 '<FieldAccess for canonical_part at ast.py:16728>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16728
--# expr-start 9565 '<FieldAccess for referenced_defining_name at ast.py:16727>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16727
--# expr-start 9564 '<FieldAccess for renamed_object at ast.py:16727>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16727
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Var_Expr_2.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_2.Info);
--# expr-done 9564
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_10 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Fld_9.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_9.Info);
--# end
--# expr-done 9565
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_part
Fld_11 := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Fld_10.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_10.Info);
--# end
--# expr-done 9566
Fld_12 := Fld_11.Node;
--# expr-done 9567
      Cast_Result_2 := Fld_12;
--# expr-start 9568 '<FieldAccess for node at ast.py:16728>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16728
Fld_13 := Block_Var.Node;
--# expr-done 9568
Is_Equal := Cast_Result_2 = Fld_13; 
--# expr-done 9569
Scope_Result_2 := Is_Equal;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
         if Result_Var_2 then
         declare
            Item_To_Append : constant Internal_Ref_Result := Item_292;
         begin
            Internal_Ref_Result_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
      Finalizer_Scope_3078;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 9570
      declare
         Map_Result_3_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Map_Result_2;
      begin
         for Item_291 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_291
--# expr-start 9576 '<Cast to BasicDecl at ast.py:16736>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16736
--# expr-start 9575 '<parent at ast.py:16736>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16736
--# expr-start 9574 '<Find at ast.py:16734>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16734
   --# expr-start 9572 '<FieldAccess for parents at ast.py:16734>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16734
--# expr-start 9571 '<FieldAccess for ref at ast.py:16734>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16734
Fld_14 := Item_291.Ref;
--# expr-done 9571
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Parents (Node => Fld_14.Node, With_Self => True, E_Info => Fld_14.Info);
--# expr-done 9572
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_15;
      begin
         for Item_294 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_294
      --# expr-start 9573 '<IsA RenamingClause at ast.py:16735>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16735
Is_A_1 := Item_294.Node /= null 
and then Item_294.Node.Kind in Ada_Renaming_Clause_Range; 
--# expr-done 9573
      if Is_A_1 then
         Find_Result_1 := Item_294; 
      end if;
   --# end
               exit when Is_A_1;
         end loop;
      end;
--# expr-done 9574
      if Find_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Parent (Node => Find_Result_1.Node, E_Info => Find_Result_1.Info);
--# expr-done 9575
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9576
Var_Expr_3 := Cast_Result_3;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 9578 '<FieldAccess for find_all_references at ast.py:16737>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16737
--# expr-start 9577 '<FieldAccess for defining_name at ast.py:16737>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16737
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 9577
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.find_all_references
Fld_18 := Libadalang.Implementation.Defining_Name_P_Find_All_References (Node => Fld_17.Node, Units => Units, Follow_Renamings => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_17.Info);
--# end
--# expr-done 9578
Scope_Result_3 := Fld_18;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3083;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
         for Item_To_Append of
               Result_Var_3.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3081;
         end loop;
      end;
         Map_Result_3 := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 9579
Concat_Result := Concat (Block_Var_3, Map_Result_3); 
--# expr-done 9580
   If_Result_1 := Concat_Result;
else
   If_Result_1 := Block_Var_3;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 9581
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 9582
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1175;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1175;
                     Finalizer_Scope_3074;
                     Finalizer_Scope_3075;
                     Finalizer_Scope_3076;
                     Finalizer_Scope_3077;
                     Finalizer_Scope_3078;
                     Finalizer_Scope_3081;
                     Finalizer_Scope_3083;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_All_References;
--# end
--# property-start DefiningName.find_matching_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16747
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Matching_Name
  (Node : Bare_Defining_Name
      ; Bd : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind bd Bd
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1176 with Inline_Always;
            procedure Finalizer_Scope_3060 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Name;
Fld_2 : Symbol_Type;
Fld_3 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1176 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1176;
            procedure Finalizer_Scope_3060 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_3060;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9588 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9587 '<Find at ast.py:16753>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16753
Var_Expr := Bd;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 9583 '<FieldAccess for defining_names at ast.py:16753>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16753
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 9583
Scope_Result := Fld;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3060;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Result_Var;
      begin
         for Item_295 of
               Collection.Items
         loop
   --# scope-start
               --# bind di Item_295
      --# expr-start 9586 '<FieldAccess for name_is at ast.py:16754>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16754
--# expr-start 9584 '<FieldAccess for name at ast.py:16754>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16754
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 9584
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9585 '<FieldAccess for name_symbol at ast.py:16754>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16754
      if Item_295.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_295.Node);
--# end
--# expr-done 9585
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1.Node, Sym => Fld_2);
--# end
--# expr-done 9586
      if Fld_3 then
         Find_Result := Item_295; 
      end if;
   --# end
               exit when Fld_3;
         end loop;
      end;
--# expr-done 9587
Let_Result := Find_Result; 
--# end
--# expr-done 9588
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1176;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1176;
                     Finalizer_Scope_3060;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_Matching_Name;
--# end
--# property-start DefiningName.find_all_calls /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16757
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_All_Calls
  (Node : Bare_Defining_Name
      ; Units : Internal_Unit_Array_Access
      ; Follow_Renamings : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind units Units
   --# bind follow_renamings Follow_Renamings
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1177 with Inline_Always;
      Fld : Internal_Ref_Result_Array_Access;
Fld_1 : Internal_Entity_Base_Id;
Fld_2 : Boolean;
Map_Result : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1177 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1177;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9593 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9592 '<Filter at ast.py:16771>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16771
   --# expr-start 9589 '<FieldAccess for find_all_references at ast.py:16771>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16771
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.find_all_references
Fld := Libadalang.Implementation.Defining_Name_P_Find_All_References (Node => Ent.Node, Units => Units, Follow_Renamings => Follow_Renamings, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9589
      declare
         Map_Result_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Fld;
      begin
         for Item_296 of
               Collection.Items
         loop
   --# scope-start
               --# bind r Item_296
         --# expr-start 9591 '<FieldAccess for is_direct_call at ast.py:16772>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16772
--# expr-start 9590 '<FieldAccess for ref at ast.py:16772>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16772
Fld_1 := Item_296.Ref;
--# expr-done 9590
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_direct_call
Fld_2 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 9591
         if Fld_2 then
         declare
            Item_To_Append : constant Internal_Ref_Result := Item_296;
         begin
            Internal_Ref_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9592
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9593
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1177;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1177;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_All_Calls;
--# end
--# property-start DefiningName.next_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16775
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Next_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9597 '<FieldAccess for find_matching_name at ast.py:16776>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16776
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9596 '<FieldAccess for next_part_for_name at ast.py:16777>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16777
--# expr-start 9594 '<FieldAccess for basic_decl at ast.py:16777>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16777
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9594
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9595 '<FieldAccess for name_symbol at ast.py:16777>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16777
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 9595
--# property-call-start '[dispatcher]BasicDecl.next_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Name (Node => Fld.Node, Sym => Fld_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 9596
--# property-call-start DefiningName.find_matching_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 9597
         Property_Result := Fld_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Next_Part;
--# end
--# property-start DefiningName.previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16784
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Previous_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9601 '<FieldAccess for find_matching_name at ast.py:16785>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16785
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9600 '<FieldAccess for previous_part_for_name at ast.py:16786>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16786
--# expr-start 9598 '<FieldAccess for basic_decl at ast.py:16786>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16786
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9598
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9599 '<FieldAccess for name_symbol at ast.py:16786>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16786
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 9599
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Fld.Node, Sym => Fld_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 9600
--# property-call-start DefiningName.find_matching_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 9601
         Property_Result := Fld_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Previous_Part;
--# end
--# property-start DefiningName.canonical_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16793
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Canonical_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9605 '<FieldAccess for find_matching_name at ast.py:16794>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16794
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9604 '<FieldAccess for canonical_part_for_name at ast.py:16795>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16795
--# expr-start 9602 '<FieldAccess for basic_decl at ast.py:16795>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16795
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9602
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9603 '<FieldAccess for name_symbol at ast.py:16795>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16795
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 9603
--# property-call-start BasicDecl.canonical_part_for_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part_For_Name (Node => Fld.Node, Sym => Fld_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 9604
--# property-call-start DefiningName.find_matching_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 9605
         Property_Result := Fld_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Canonical_Part;
--# end
--# property-start DefiningName.most_visible_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16802
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Most_Visible_Part
  (Node : Bare_Defining_Name
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9610 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9609 '<FieldAccess for find_matching_name at ast.py:16810>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16810
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9608 '<FieldAccess for most_visible_part_for_name at ast.py:16811>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16811
--# expr-start 9606 '<FieldAccess for basic_decl at ast.py:16811>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16811
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9606
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9607 '<FieldAccess for name_symbol at ast.py:16811>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16811
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 9607
--# property-call-start BasicDecl.most_visible_part_for_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Part_For_Name (Node => Fld.Node, Sym => Fld_1, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 9608
--# property-call-start DefiningName.find_matching_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 9609
Let_Result := Fld_3; 
--# end
--# expr-done 9610
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Most_Visible_Part;
--# end
--# property-start DefiningName.all_previous_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16814
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Previous_Parts
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1182 with Inline_Always;
            procedure Finalizer_Scope_3085 with Inline_Always;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Concat_Result : Internal_Entity_Defining_Name_Array_Access;
Cast_Expr : Internal_Entity_Defining_Name;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1182 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1182;
            procedure Finalizer_Scope_3085 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_3085;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9616 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9611 '<FieldAccess for previous_part at ast.py:16821>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16821
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.previous_part
Fld := Libadalang.Implementation.Defining_Name_P_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9611
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 9615 '<If at ast.py:16822>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16822
--# expr-start 9612 '<Eq at ast.py:16823>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16823
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 9612
if Is_Equiv then
   If_Result := No_Internal_Entity_Defining_Name_Array_Type;
else
   --# expr-start 9614 '<Concat at ast.py:16825>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16825
--# expr-start 9613 '<FieldAccess for all_previous_parts at ast.py:16825>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16825
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_previous_parts
Fld_1 := Libadalang.Implementation.Defining_Name_P_All_Previous_Parts (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 9613
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Concat_Result := Concat (Fld_1, Singleton); 
--# expr-done 9614
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 9615
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3085;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 9616
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1182;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1182;
                     Finalizer_Scope_3085;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_All_Previous_Parts;
--# end
--# property-start DefiningName.all_next_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16829
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Next_Parts
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1183 with Inline_Always;
            procedure Finalizer_Scope_3086 with Inline_Always;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Concat_Result : Internal_Entity_Defining_Name_Array_Access;
Cast_Expr : Internal_Entity_Defining_Name;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1183 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1183;
            procedure Finalizer_Scope_3086 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_3086;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9622 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9617 '<FieldAccess for next_part at ast.py:16836>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16836
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.next_part
Fld := Libadalang.Implementation.Defining_Name_P_Next_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9617
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 9621 '<If at ast.py:16837>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16837
--# expr-start 9618 '<Eq at ast.py:16838>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16838
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 9618
if Is_Equiv then
   If_Result := No_Internal_Entity_Defining_Name_Array_Type;
else
   --# expr-start 9620 '<Concat at ast.py:16840>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16840
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
--# expr-start 9619 '<FieldAccess for all_next_parts at ast.py:16840>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16840
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_next_parts
Fld_1 := Libadalang.Implementation.Defining_Name_P_All_Next_Parts (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 9619
Concat_Result := Concat (Singleton, Fld_1); 
--# expr-done 9620
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 9621
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3086;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 9622
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1183;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1183;
                     Finalizer_Scope_3086;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_All_Next_Parts;
--# end
--# property-start DefiningName.all_parts /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16844
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Parts
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1184 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Block_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Concat_Result : Internal_Entity_Defining_Name_Array_Access;
Concat_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1184 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1184;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9627 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9623 '<FieldAccess for all_previous_parts at ast.py:16851>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16851
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_previous_parts
Fld := Libadalang.Implementation.Defining_Name_P_All_Previous_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9623
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind prevs Block_Var
--# expr-start 9624 '<FieldAccess for all_next_parts at ast.py:16852>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16852
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_next_parts
Fld_1 := Libadalang.Implementation.Defining_Name_P_All_Next_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9624
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind nexts Block_Var_1
--# expr-start 9626 '<Concat at ast.py:16853>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16853
--# expr-start 9625 '<Concat at ast.py:16853>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16853
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
Concat_Result := Concat (Block_Var, Singleton); 
--# expr-done 9625
Concat_Result_1 := Concat (Concat_Result, Block_Var_1); 
--# expr-done 9626
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 9627
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1184;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1184;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_All_Parts;
--# end
--# property-start DefiningName.get_aspect_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16855
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Aspect_Impl
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Inherited : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind inherited Inherited
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
      Var_Expr : Internal_Aspect;
Var_Expr_1 : Internal_Aspect;
Var_Expr_2 : Internal_Aspect;
Var_Expr_3 : Internal_Aspect;
Var_Expr_4 : Internal_Entity_Pragma_Node;
Var_Expr_5 : Internal_Entity_Aspect_Assoc;
Var_Expr_6 : Internal_Entity_Attribute_Def_Clause;
Var_Expr_7 : Internal_Entity_At_Clause;
Var_Expr_8 : Internal_Entity_Base_Type_Decl;
Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Pragma_Node;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Pragma_Node;
Cast_Result : Internal_Entity;
New_Struct : Internal_Aspect;
Scope_Result : Internal_Aspect;
Result_Var : Internal_Aspect;
Scope_Result_1 : Internal_Aspect;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Aspect_Assoc;
Fld_4 : Internal_Entity_Expr;
Cast_Expr_1 : Internal_Entity_Aspect_Assoc;
Cast_Result_1 : Internal_Entity;
New_Struct_1 : Internal_Aspect;
Scope_Result_2 : Internal_Aspect;
Result_Var_1 : Internal_Aspect;
Result_Var_2 : Internal_Aspect;
Scope_Result_3 : Internal_Aspect;
Fld_5 : Internal_Entity_Attribute_Def_Clause;
Fld_6 : Internal_Entity_Expr;
Cast_Expr_2 : Internal_Entity_Attribute_Def_Clause;
Cast_Result_2 : Internal_Entity;
New_Struct_2 : Internal_Aspect;
Scope_Result_4 : Internal_Aspect;
Result_Var_3 : Internal_Aspect;
Result_Var_4 : Internal_Aspect;
Scope_Result_5 : Internal_Aspect;
Fld_7 : Internal_Entity_At_Clause;
Fld_8 : Internal_Entity_Expr;
Cast_Expr_3 : Internal_Entity_At_Clause;
Cast_Result_3 : Internal_Entity;
New_Struct_3 : Internal_Aspect;
Scope_Result_6 : Internal_Aspect;
Result_Var_5 : Internal_Aspect;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Internal_Aspect;
Result_Var_6 : Internal_Aspect;
Scope_Result_7 : Internal_Aspect;
Fld_9 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity_Base_Subtype_Decl;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_12 : Internal_Entity_Defining_Name;
Fld_13 : Internal_Aspect;
Is_Null : Boolean;
Cast_Expr_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Cast_Expr_6 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Aspect;
Let_Result : Internal_Aspect;
Scope_Result_8 : Internal_Aspect;
Result_Var_7 : Internal_Aspect;
Result_Var_8 : Internal_Aspect;
Let_Result_1 : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9657 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 9628 '<FieldAccess for get_pragma at ast.py:16862>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16862
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_pragma
Fld := Libadalang.Implementation.Defining_Name_P_Get_Pragma (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 9628
Var_Expr_4 := Fld;
if Var_Expr_4 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr_4
--# expr-start 9630 '<New Aspect at ast.py:16863>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16863
Cast_Expr := Var_Expr_4; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 9629 '<FieldAccess for value_expr at ast.py:16864>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16864
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.value_expr
Fld_1 := Libadalang.Implementation.Pragma_Node_P_Value_Expr (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 9629
New_Struct := (Exists => True, Inherited => Inherited, Node => Cast_Result, Value => Fld_1); 
--# expr-done 9630
Scope_Result := New_Struct;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Aspect;
end if;
Var_Expr_3 := Result_Var;
if Var_Expr_3 /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_1 := Var_Expr_3;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 9632 '<FieldAccess for get_aspect_assoc at ast.py:16866>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16866
--# expr-start 9631 '<FieldAccess for basic_decl at ast.py:16866>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16866
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9631
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Fld_2.Node, Name => Name, E_Info => Fld_2.Info);
--# end
--# expr-done 9632
Var_Expr_5 := Fld_3;
if Var_Expr_5 /= No_Entity_Aspect_Assoc then
   --# scope-start
   --# bind aa Var_Expr_5
--# expr-start 9634 '<New Aspect at ast.py:16867>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16867
Cast_Expr_1 := Var_Expr_5; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 9633 '<FieldAccess for expr at ast.py:16868>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16868
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Var_Expr_5.Node.Aspect_Assoc_F_Expr, Info => Var_Expr_5.Info);
--# expr-done 9633
New_Struct_1 := (Exists => True, Inherited => Inherited, Node => Cast_Result_1, Value => Fld_4); 
--# expr-done 9634
Scope_Result_2 := New_Struct_1;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Aspect;
end if;
   Result_Var_2 := Result_Var_1;
end if;
Var_Expr_2 := Result_Var_2;
if Var_Expr_2 /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_3 := Var_Expr_2;
   --# end
   Result_Var_4 := Scope_Result_3;
else
--# expr-start 9635 '<FieldAccess for get_representation_clause at ast.py:16869>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16869
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_representation_clause
Fld_5 := Libadalang.Implementation.Defining_Name_P_Get_Representation_Clause (Node => Ent.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9635
Var_Expr_6 := Fld_5;
if Var_Expr_6 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind rc Var_Expr_6
--# expr-start 9637 '<New Aspect at ast.py:16870>' New_Struct_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16870
Cast_Expr_2 := Var_Expr_6; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 9636 '<FieldAccess for expr at ast.py:16871>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16871
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Var_Expr_6.Node.Attribute_Def_Clause_F_Expr, Info => Var_Expr_6.Info);
--# expr-done 9636
New_Struct_2 := (Exists => True, Inherited => Inherited, Node => Cast_Result_2, Value => Fld_6); 
--# expr-done 9637
Scope_Result_4 := New_Struct_2;
   --# end
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Aspect;
end if;
   Result_Var_4 := Result_Var_3;
end if;
Var_Expr_1 := Result_Var_4;
if Var_Expr_1 /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_5 := Var_Expr_1;
   --# end
   Result_Var_6 := Scope_Result_5;
else
   --# expr-start 9643 '<If at ast.py:16872>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16872
--# expr-start 9639 '<Eq at ast.py:16873>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16873
--# expr-start 9638 '<Symbol Address at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address); 
--# expr-done 9638
Is_Equal := Name = Sym; 
--# expr-done 9639
if Is_Equal then
--# expr-start 9640 '<FieldAccess for get_at_clause at ast.py:16874>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16874
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_at_clause
Fld_7 := Libadalang.Implementation.Defining_Name_P_Get_At_Clause (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9640
Var_Expr_7 := Fld_7;
if Var_Expr_7 /= No_Entity_At_Clause then
   --# scope-start
   --# bind atc Var_Expr_7
--# expr-start 9642 '<New Aspect at ast.py:16875>' New_Struct_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16875
Cast_Expr_3 := Var_Expr_7; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-start 9641 '<FieldAccess for expr at ast.py:16876>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16876
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Var_Expr_7.Node.At_Clause_F_Expr, Info => Var_Expr_7.Info);
--# expr-done 9641
New_Struct_3 := (Exists => True, Inherited => Inherited, Node => Cast_Result_3, Value => Fld_8); 
--# expr-done 9642
Scope_Result_6 := New_Struct_3;
   --# end
   Result_Var_5 := Scope_Result_6;
else
   Result_Var_5 := No_Aspect;
end if;
   If_Result := Result_Var_5;
else
   If_Result := No_Aspect;
end if;
--# expr-done 9643
   Result_Var_6 := If_Result;
end if;
Var_Expr := Result_Var_6;
if Var_Expr /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_7 := Var_Expr;
   --# end
   Result_Var_8 := Scope_Result_7;
else
--# expr-start 9645 '<Cast to BaseTypeDecl at ast.py:16882>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16882
--# expr-start 9644 '<FieldAccess for basic_decl_no_internal at ast.py:16882>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16882
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl_no_internal
Fld_9 := Libadalang.Implementation.Defining_Name_P_Basic_Decl_No_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9644
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_4 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 9645
Var_Expr_8 := Cast_Result_4;
if Var_Expr_8 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bd Var_Expr_8
--# expr-start 9656 '<Let at ast.py:16883>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16883
--# scope-start
--# expr-start 9650 '<If at ast.py:16884>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16884
--# expr-start 9646 '<IsA BaseSubtypeDecl at ast.py:16884>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16884
Is_A := Var_Expr_8.Node /= null 
and then Var_Expr_8.Node.Kind in Ada_Base_Subtype_Decl; 
--# expr-done 9646
if Is_A then
   --# expr-start 9648 '<FieldAccess for get_type at ast.py:16885>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16885
--# expr-start 9647 '<Cast to BaseSubtypeDecl at ast.py:16885>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16885
Cast_Expr_4 := Var_Expr_8; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Base_Subtype_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_5 := No_Entity_Base_Subtype_Decl;
   end if;
--# expr-done 9647
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Cast_Result_5.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 9648
   If_Result_1 := Fld_10;
else
   --# expr-start 9649 '<FieldAccess for base_type at ast.py:16886>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16886
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Var_Expr_8.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 9649
   If_Result_1 := Fld_11;
end if;
--# expr-done 9650
Typ := If_Result_1; 
--# bind typ Typ
--# expr-start 9655 '<If at ast.py:16887>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16887
--# expr-start 9651 '<IsNull at ast.py:16888>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16888
Is_Null := Typ.Node = null; 
--# expr-done 9651
if Is_Null then
   If_Result_2 := True;
else
   --# expr-start 9652 '<Eq at ast.py:16888>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16888
Cast_Expr_5 := Typ; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
Cast_Expr_6 := Var_Expr_8; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
Is_Equiv := Equivalent (Cast_Result_6, Cast_Result_7); 
--# expr-done 9652
   If_Result_2 := Is_Equiv;
end if;
if If_Result_2 then
   If_Result_3 := No_Aspect;
else
   --# expr-start 9654 '<FieldAccess for get_aspect_impl at ast.py:16890>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16890
--# expr-start 9653 '<FieldAccess for name at ast.py:16890>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16890
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Defining_Name (Node => Typ.Node.Base_Type_Decl_F_Name, Info => Typ.Info);
--# expr-done 9653
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect_impl
Fld_13 := Libadalang.Implementation.Defining_Name_P_Get_Aspect_Impl (Node => Fld_12.Node, Name => Name, Inherited => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_12.Info);
--# end
--# expr-done 9654
   If_Result_3 := Fld_13;
end if;
--# expr-done 9655
Let_Result := If_Result_3; 
--# end
--# expr-done 9656
Scope_Result_8 := Let_Result;
   --# end
   Result_Var_7 := Scope_Result_8;
else
   Result_Var_7 := No_Aspect;
end if;
   Result_Var_8 := Result_Var_7;
end if;
Let_Result_1 := Result_Var_8; 
--# end
--# expr-done 9657
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Aspect_Impl;
--# end
--# property-start DefiningName.get_aspect /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16896
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Aspect
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
            procedure Finalizer_Scope_1186 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Aspect;
Map_Result : Internal_Aspect_Array_Access;
Fld_2 : Boolean;
Find_Result : Internal_Aspect;
Let_Result : Internal_Aspect;
            procedure Finalizer_Scope_1186 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1186;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9674 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9669 '<If at ast.py:16912>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16912
--# expr-start 9659 '<Eq at ast.py:16913>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16913
--# expr-start 9658 '<Symbol Annotate at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 9658
Is_Equal := Name = Sym; 
--# expr-done 9659
if Is_Equal then
   If_Result := True;
else
   --# expr-start 9661 '<Eq at ast.py:16913>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16913
--# expr-start 9660 '<Symbol Inline at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Inline); 
--# expr-done 9660
Is_Equal_1 := Name = Sym_1; 
--# expr-done 9661
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 9663 '<Eq at ast.py:16913>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16913
--# expr-start 9662 '<Symbol Obsolescent at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Obsolescent); 
--# expr-done 9662
Is_Equal_2 := Name = Sym_2; 
--# expr-done 9663
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 9665 '<Eq at ast.py:16913>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16913
--# expr-start 9664 '<Symbol Ghost at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ghost); 
--# expr-done 9664
Is_Equal_3 := Name = Sym_3; 
--# expr-done 9665
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 9667 '<Eq at ast.py:16913>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16913
--# expr-start 9666 '<Symbol Default_Initial_Condition at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Default_Initial_Condition); 
--# expr-done 9666
Is_Equal_4 := Name = Sym_4; 
--# expr-done 9667
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   --# expr-start 9668 '<FieldAccess for all_parts at ast.py:16920>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16920
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_parts
Fld := Libadalang.Implementation.Defining_Name_P_All_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9668
   If_Result_4 := Fld;
else
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
   If_Result_4 := Singleton;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 9669
Block_Var := If_Result_4; Inc_Ref (Block_Var);
--# bind parts_to_check Block_Var
--# expr-start 9673 '<Find at ast.py:16925>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16925
   --# expr-start 9671 '<Map at ast.py:16923>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16923
      declare
         Map_Result_Vec : Internal_Aspect_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Block_Var;
      begin
         for Item_298 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_298
      --# expr-start 9670 '<FieldAccess for get_aspect_impl at ast.py:16924>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16924
      if Item_298.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect_impl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Aspect_Impl (Node => Item_298.Node, Name => Name, Inherited => False, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_298.Info);
--# end
--# expr-done 9670
         declare
            Item_To_Append : constant Internal_Aspect := Fld_1;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9671
      Find_Result := No_Aspect;
      declare
         Collection : constant Internal_Aspect_Array_Access := Map_Result;
      begin
         for Item_297 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_297
      --# expr-start 9672 '<FieldAccess for exists at ast.py:16926>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16926
Fld_2 := Item_297.Exists;
--# expr-done 9672
      if Fld_2 then
         Find_Result := Item_297; 
      end if;
   --# end
               exit when Fld_2;
         end loop;
      end;
--# expr-done 9673
Let_Result := Find_Result; 
--# end
--# expr-done 9674
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1186;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1186;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Aspect;
--# end
--# property-start DefiningName.has_aspect /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16929
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Has_Aspect
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_3099 with Inline_Always;
      Block_Var : Internal_Aspect;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Aspect;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
Fld_5 : Big_Integer_Type;
Big_Int : Big_Integer_Type;
Is_Equal : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_6 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_3099 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Big_Int);
            end Finalizer_Scope_3099;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9687 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9675 '<FieldAccess for get_aspect at ast.py:16941>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16941
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect
Fld := Libadalang.Implementation.Defining_Name_P_Get_Aspect (Node => Ent.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 9675
Block_Var := Fld; 
--# bind a Block_Var
--# expr-start 9676 '<FieldAccess for exists at ast.py:16943>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16943
Fld_1 := Block_Var.Exists;
--# expr-done 9676
if Fld_1 then
   --# expr-start 9686 '<If at ast.py:16943>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16943
--# expr-start 9677 '<FieldAccess for is_contract_aspect at ast.py:16944>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16944
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_contract_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Name);
--# end
--# expr-done 9677
if Fld_6 then
   If_Result_1 := True;
else
--# expr-start 9678 '<FieldAccess for value at ast.py:16950>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16950
Fld_2 := Block_Var.Value;
--# expr-done 9678
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind val Var_Expr
--# expr-start 9682 '<Not at ast.py:16954>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16954
--# expr-start 9681 '<Eq at ast.py:16954>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16954
--# expr-start 9679 '<FieldAccess for expression_type at ast.py:16954>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16954
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 9679
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 9680 '<FieldAccess for bool_type at ast.py:16954>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16954
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 9680
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 9681
Not_Val := not (Is_Equiv); 
--# expr-done 9682
if Not_Val then
   If_Result := True;
else
   --# expr-start 9685 '<Eq at ast.py:16955>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16955
--# expr-start 9683 '<FieldAccess for eval_as_int at ast.py:16955>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16955
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_5 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 9683
--# expr-start 9684 '<BigInteger 1 at ast.py:16955>' Big_Int /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16955
Big_Int := Create_Big_Integer ("1"); 
--# expr-done 9684
Is_Equal := Equivalent (Fld_5, Big_Int); 
--# expr-done 9685
   If_Result := Is_Equal;
end if;
Scope_Result := If_Result;
   --# end
      Finalizer_Scope_3099;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
   If_Result_1 := Result_Var;
end if;
--# expr-done 9686
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 9687
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3099;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Has_Aspect;
--# end
--# property-start DefiningName.is_valid_pragma_for_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16959
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Valid_Pragma_For_Name
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Decl : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind decl Decl
   Property_Result : Boolean;
            procedure Finalizer_Scope_3100 with Inline_Always;
      Var_Expr : Internal_Entity_Pragma_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld : Internal_Entity_Identifier;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Defining_Name;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Pragma_Node;
Node_Comp : Boolean;
If_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_3100 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3100;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9698 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9688 '<Cast to Pragma at ast.py:16966>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16966
Cast_Expr := Decl; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 9688
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 9690 '<FieldAccess for name_is at ast.py:16969>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16969
--# expr-start 9689 '<FieldAccess for id at ast.py:16969>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16969
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Var_Expr.Node.Pragma_Node_F_Id, Info => Var_Expr.Info);
--# expr-done 9689
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Name);
--# end
--# expr-done 9690
if Fld_1 then
   --# expr-start 9695 '<Not at ast.py:16971>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16971
--# expr-start 9694 '<IsNull at ast.py:16972>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16972
--# expr-start 9693 '<Find at ast.py:16971>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16971
   --# expr-start 9691 '<FieldAccess for associated_entities at ast.py:16971>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16971
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entities
Fld_2 := Libadalang.Implementation.Pragma_Node_P_Associated_Entities (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 9691
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_2;
      begin
         for Item_299 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_299
      --# expr-start 9692 '<Eq at ast.py:16971>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16971
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Item_299; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 9692
      if Is_Equiv then
         Find_Result := Item_299; 
      end if;
   --# end
               exit when Is_Equiv;
         end loop;
      end;
--# expr-done 9693
Is_Null := Find_Result.Node = null; 
--# expr-done 9694
Not_Val := not (Is_Null); 
--# expr-done 9695
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 9697 '<OrderingTest '"'"'lt'"'"' at ast.py:16974>' Node_Comp /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16974
--# expr-start 9696 '<FieldAccess for node at ast.py:16974>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16974
Fld_3 := Var_Expr.Node;
--# expr-done 9696
Node_Comp := Compare (Self, Self, Fld_3, Less_Than); 
--# expr-done 9697
   If_Result_1 := Node_Comp;
else
   If_Result_1 := False;
end if;
Scope_Result := If_Result_1;
   --# end
      Finalizer_Scope_3100;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 9698
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3100;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Is_Valid_Pragma_For_Name;
--# end
--# property-start DefiningName.get_pragma /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16978
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Pragma
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1189 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Enum_Literal_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Pragma_Node;
Var_Expr_1 : Internal_Entity_Pragma_Node_List;
Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity;
Var_Expr_4 : Internal_Entity_Ada_Node_List;
Var_Expr_5 : Bare_Public_Part;
Var_Expr_6 : Bare_Private_Part;
Match_0_1 : Bare_Base_Package_Decl;
Match_1_1 : Bare_Protected_Def;
Match_2 : Bare_Ada_Node;
Var_Expr_7 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Enum_Literal_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Pragma_Node_List;
Saved : Internal_Entity_Pragma_Node_List;
Fld_5 : Bare_Pragma_Node_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Fld_7 : Internal_Entity_Identifier;
Fld_8 : Boolean;
Item_300 : Internal_Entity_Pragma_Node;
Bare_Item_300 : Bare_Pragma_Node;
Find_Result : Internal_Entity_Pragma_Node;
Scope_Result_2 : Internal_Entity_Pragma_Node;
Result_Var : Internal_Entity_Pragma_Node;
Scope_Result_3 : Internal_Entity_Pragma_Node;
Fld_9 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Ada_Node_List;
Saved_1 : Internal_Entity_Ada_Node_List;
Fld_10 : Bare_Ada_Node_List;
Fld_11 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_12 : Boolean;
Item_301 : Internal_Entity;
Bare_Item_301 : Bare_Ada_Node;
Find_Result_1 : Internal_Entity;
Scope_Result_4 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Scope_Result_5 : Internal_Entity;
Fld_13 : Bare_Declarative_Part;
Cast_Result_4 : Bare_Public_Part;
Fld_14 : Bare_Ada_Node;
Fld_15 : Bare_Private_Part;
Fld_16 : Bare_Private_Part;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_5 : Bare_Base_Package_Decl;
Let_Result_2 : Bare_Private_Part;
Scope_Result_6 : Bare_Private_Part;
Cast_Result_6 : Bare_Protected_Def;
Let_Result_3 : Bare_Private_Part;
Scope_Result_7 : Bare_Private_Part;
Cast_Result_7 : Bare_Ada_Node;
Let_Result_4 : Bare_Private_Part;
Scope_Result_8 : Bare_Private_Part;
Match_Result_1 : Bare_Private_Part;
Fld_17 : Bare_Ada_Node_List;
Scope_Result_9 : Bare_Ada_Node_List;
Result_Var_2 : Bare_Ada_Node_List;
As_Entity_2 : Internal_Entity_Ada_Node_List;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Ada_Node_List;
Fld_18 : Bare_Ada_Node_List;
Fld_19 : Internal_Entity_Info;
As_Entity_3 : Internal_Entity;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity;
Fld_20 : Boolean;
Item_302 : Internal_Entity;
Bare_Item_302 : Bare_Ada_Node;
Find_Result_2 : Internal_Entity;
Scope_Result_10 : Internal_Entity;
Result_Var_3 : Internal_Entity;
Result_Var_4 : Internal_Entity;
Scope_Result_11 : Internal_Entity;
Fld_21 : Internal_Entity_Declarative_Part_Array_Access;
Get_Result : Internal_Entity_Declarative_Part;
Fld_22 : Internal_Entity_Ada_Node_List;
Scope_Result_12 : Internal_Entity_Ada_Node_List;
Result_Var_5 : Internal_Entity_Ada_Node_List;
Fld_23 : Bare_Ada_Node_List;
Fld_24 : Internal_Entity_Info;
As_Entity_4 : Internal_Entity;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity;
Fld_25 : Boolean;
Item_303 : Internal_Entity;
Bare_Item_303 : Bare_Ada_Node;
Find_Result_3 : Internal_Entity;
Result_Var_6 : Internal_Entity;
Cast_Result_8 : Internal_Entity_Pragma_Node;
Result_Var_7 : Internal_Entity_Pragma_Node;
Let_Result_5 : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1189 is
            begin
                     Dec_Ref (Fld_21);
            end Finalizer_Scope_1189;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9729 '<Block at ???>' Let_Result_5 None
--# scope-start
--# expr-start 9704 '<Match at ast.py:16987>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16987
--# expr-start 9699 '<FieldAccess for basic_decl_no_internal at ast.py:16987>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16987
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl_no_internal
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl_No_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9699
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind eld Match_0
--# expr-start 9703 '<Cast to BasicDecl at ast.py:16991>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16991
--# expr-start 9702 '<parent at ast.py:16991>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16991
--# expr-start 9701 '<parent at ast.py:16991>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16991
--# expr-start 9700 '<parent at ast.py:16991>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16991
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Match_0.Node, E_Info => Match_0.Info);
--# expr-done 9700
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 9701
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 9702
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9703
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind o Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 9704
Block_Var := Match_Result; 
--# bind bd Block_Var
--# expr-start 9705 '<FieldAccess for library_item_pragmas at ast.py:16996>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16996
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.library_item_pragmas
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Library_Item_Pragmas (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 9705
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Pragma_Node_List then
   --# scope-start
   --# bind plist Var_Expr_1
--# expr-start 9708 '<Find at ast.py:16998>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16998
Saved := Var_Expr_1; 
Fld_5 := Saved.Node;
      Find_Result := No_Entity_Pragma_Node;
      if Fld_5 = null then
      Find_Result := No_Entity_Pragma_Node;
      else
      declare
         Collection : constant Bare_Pragma_Node_List := Fld_5;
      begin
         for Untyped_Item_300 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_300 := Untyped_Item_300; 
Is_Null := Bare_Item_300 = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
Fld_6 := Saved.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_300); 
   If_Result := As_Entity;
end if;
                  Item_300 := If_Result; 
   --# scope-start
               --# bind p Item_300
      --# expr-start 9707 '<FieldAccess for name_is at ast.py:16998>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16998
--# expr-start 9706 '<FieldAccess for id at ast.py:16998>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16998
      if Item_300.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Identifier (Node => Item_300.Node.Pragma_Node_F_Id, Info => Item_300.Info);
--# expr-done 9706
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_8 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_7.Node, Sym => Name);
--# end
--# expr-done 9707
      if Fld_8 then
         Find_Result := Item_300; 
      end if;
   --# end
               exit when Fld_8;
         end loop;
      end;
      end if;
--# expr-done 9708
Scope_Result_2 := Find_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Pragma_Node;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_3 := Var_Expr;
   --# end
   Result_Var_7 := Scope_Result_3;
else
   --# expr-start 9728 '<Cast to Pragma at ast.py:17025>' Cast_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17025
--# expr-start 9710 '<Cast to AdaNode.list at ast.py:17004>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17004
--# expr-start 9709 '<parent at ast.py:17004>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17004
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 9709
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Ada_Node_List_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Ada_Node_List
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_3 := No_Entity_Ada_Node_List;
   end if;
--# expr-done 9710
Var_Expr_4 := Cast_Result_3;
if Var_Expr_4 /= No_Entity_Ada_Node_List then
   --# scope-start
--# expr-start 9712 '<Find at ast.py:17004>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17004
Saved_1 := Var_Expr_4; 
Fld_10 := Saved_1.Node;
      Find_Result_1 := No_Entity;
      if Fld_10 = null then
      Find_Result_1 := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_10;
      begin
         for Untyped_Item_301 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_301 := Untyped_Item_301; 
Is_Null_1 := Bare_Item_301 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_11 := Saved_1.Info;
As_Entity_1 := (Info => Fld_11, Node => Bare_Item_301); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_301 := If_Result_1; 
   --# scope-start
               --# bind d Item_301
      --# expr-start 9711 '<FieldAccess for is_valid_pragma_for_name at ast.py:17005>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17005
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_valid_pragma_for_name
Fld_12 := Libadalang.Implementation.Defining_Name_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_301, E_Info => Ent.Info);
--# end
--# expr-done 9711
      if Fld_12 then
         Find_Result_1 := Item_301; 
      end if;
   --# end
               exit when Fld_12;
         end loop;
      end;
      end if;
--# expr-done 9712
Scope_Result_4 := Find_Result_1;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := No_Entity;
end if;
Var_Expr_3 := Result_Var_1;
if Var_Expr_3 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_5 := Var_Expr_3;
   --# end
   Result_Var_4 := Scope_Result_5;
else
--# expr-start 9714 '<Cast to PublicPart at ast.py:17010>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17010
--# expr-start 9713 '<FieldAccess for declarative_scope at ast.py:17010>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17010
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_13 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Block_Var.Node);
--# end
--# expr-done 9713
   if Fld_13 = null
      or else Fld_13.Kind in Ada_Public_Part_Range
   then
      Cast_Result_4 := Fld_13;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 9714
Var_Expr_5 := Cast_Result_4;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pp Var_Expr_5
--# expr-start 9722 '<Find at ast.py:17015>' Find_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17015
   --# expr-start 9720 '<as_entity at ast.py:17015>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17015
--# expr-start 9718 '<Match at ast.py:17011>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17011
--# expr-start 9715 '<parent at ast.py:17011>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17011
      if Var_Expr_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Var_Expr_5.Parent; 
--# expr-done 9715
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld_14; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix_1;
Match_0_1 := Cast_Result_5; 
--# bind pkg Match_0_1
--# expr-start 9716 '<FieldAccess for private_part at ast.py:17012>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17012
      if Match_0_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Match_0_1.Base_Package_Decl_F_Private_Part;
--# expr-done 9716
Let_Result_2 := Fld_15; 
--# end
Scope_Result_6 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_6; 
         when Ada_Protected_Def =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix_1;
Match_1_1 := Cast_Result_6; 
--# bind ptd Match_1_1
--# expr-start 9717 '<FieldAccess for private_part at ast.py:17013>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17013
      if Match_1_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Match_1_1.Protected_Def_F_Private_Part;
--# expr-done 9717
Let_Result_3 := Fld_16; 
--# end
Scope_Result_7 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_7; 
         when Ada_Abort_Absent .. Ada_Synthetic_Formal_Param_Decl | Ada_Discrete_Base_Subtype_Decl .. Ada_Private_Present | Ada_Protected_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Match_Prefix_1;
Match_2 := Cast_Result_7; 
--# bind _ Match_2
Let_Result_4 := No_Bare_Ada_Node; 
--# end
Scope_Result_8 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_8; 
end case;
--# expr-done 9718
Var_Expr_6 := Match_Result_1;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9719 '<FieldAccess for decls at ast.py:17015>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17015
      if Var_Expr_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Var_Expr_6.Declarative_Part_F_Decls;
--# expr-done 9719
Scope_Result_9 := Fld_17;
   --# end
   Result_Var_2 := Scope_Result_9;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
Is_Null_2 := Result_Var_2 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Ada_Node_List;
else
As_Entity_2 := (Info => E_Info, Node => Result_Var_2); 
   If_Result_2 := As_Entity_2;
end if;
--# expr-done 9720
Fld_18 := If_Result_2.Node;
      Find_Result_2 := No_Entity;
      if Fld_18 = null then
      Find_Result_2 := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_18;
      begin
         for Untyped_Item_302 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_302 := Untyped_Item_302; 
Is_Null_3 := Bare_Item_302 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity;
else
Fld_19 := If_Result_2.Info;
As_Entity_3 := (Info => Fld_19, Node => Bare_Item_302); 
   If_Result_3 := As_Entity_3;
end if;
                  Item_302 := If_Result_3; 
   --# scope-start
               --# bind d Item_302
      --# expr-start 9721 '<FieldAccess for is_valid_pragma_for_name at ast.py:17016>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17016
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_valid_pragma_for_name
Fld_20 := Libadalang.Implementation.Defining_Name_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_302, E_Info => Ent.Info);
--# end
--# expr-done 9721
      if Fld_20 then
         Find_Result_2 := Item_302; 
      end if;
   --# end
               exit when Fld_20;
         end loop;
      end;
      end if;
--# expr-done 9722
Scope_Result_10 := Find_Result_2;
   --# end
   Result_Var_3 := Scope_Result_10;
else
   Result_Var_3 := No_Entity;
end if;
   Result_Var_4 := Result_Var_3;
end if;
Var_Expr_2 := Result_Var_4;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_11 := Var_Expr_2;
   --# end
   Result_Var_6 := Scope_Result_11;
else
   --# expr-start 9727 '<Find at ast.py:17021>' Find_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17021
--# expr-start 9724 '<at at ast.py:17021>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17021
--# expr-start 9723 '<FieldAccess for declarative_parts at ast.py:17021>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17021
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.declarative_parts'
Fld_21 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Declarative_Parts (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 9723
Get_Result := Get (Self, Fld_21, 0, True); 
--# expr-done 9724
Var_Expr_7 := Get_Result;
if Var_Expr_7 /= No_Entity_Declarative_Part then
   --# scope-start
--# expr-start 9725 '<FieldAccess for decls at ast.py:17021>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17021
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_7.Node.Declarative_Part_F_Decls, Info => Var_Expr_7.Info);
--# expr-done 9725
Scope_Result_12 := Fld_22;
   --# end
   Result_Var_5 := Scope_Result_12;
else
   Result_Var_5 := No_Entity_Ada_Node_List;
end if;
Fld_23 := Result_Var_5.Node;
      Find_Result_3 := No_Entity;
      if Fld_23 = null then
      Find_Result_3 := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_23;
      begin
         for Untyped_Item_303 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_303 := Untyped_Item_303; 
Is_Null_4 := Bare_Item_303 = null; 
if Is_Null_4 then
   If_Result_4 := No_Entity;
else
Fld_24 := Result_Var_5.Info;
As_Entity_4 := (Info => Fld_24, Node => Bare_Item_303); 
   If_Result_4 := As_Entity_4;
end if;
                  Item_303 := If_Result_4; 
   --# scope-start
               --# bind d Item_303
      --# expr-start 9726 '<FieldAccess for is_valid_pragma_for_name at ast.py:17022>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17022
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_valid_pragma_for_name
Fld_25 := Libadalang.Implementation.Defining_Name_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_303, E_Info => Ent.Info);
--# end
--# expr-done 9726
      if Fld_25 then
         Find_Result_3 := Item_303; 
      end if;
   --# end
               exit when Fld_25;
         end loop;
      end;
      end if;
--# expr-done 9727
   Result_Var_6 := Find_Result_3;
end if;
   if Result_Var_6.Node = null
      or else Result_Var_6.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_8 := Create_Internal_Entity_Pragma_Node
        (Node => Result_Var_6.Node,
         Info => Result_Var_6.Info);
   else
         Cast_Result_8 := No_Entity_Pragma_Node;
   end if;
--# expr-done 9728
   Result_Var_7 := Cast_Result_8;
end if;
Let_Result_5 := Result_Var_7; 
--# end
--# expr-done 9729
         Property_Result := Let_Result_5;
   --# end
      Finalizer_Scope_1189;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1189;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Pragma;
--# end
--# property-start DefiningName.get_representation_clause /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17028
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Representation_Clause
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Attribute_Def_Clause
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Attribute_Def_Clause;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Attribute_Def_Clause;
Attr : Internal_Entity_Attribute_Ref;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Attribute_Def_Clause;
Fld_4 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_5 : Internal_Entity_Identifier;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Defining_Name;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Item_304 : Internal_Entity;
Bare_Item_304 : Bare_Ada_Node;
Find_Result : Internal_Entity;
Cast_Result_4 : Internal_Entity_Attribute_Def_Clause;
Let_Result_1 : Internal_Entity_Attribute_Def_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9744 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 9743 '<Cast to AttributeDefClause.entity at ast.py:17043>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17043
--# expr-start 9742 '<Find at ast.py:17035>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17035
   --# expr-start 9732 '<as_entity at ast.py:17035>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17035
--# expr-start 9730 '<FieldAccess for declarative_scope at ast.py:17035>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17035
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 9730
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9731 '<FieldAccess for decls at ast.py:17035>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17035
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 9731
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 9732
Fld_2 := If_Result.Node;
      Find_Result := No_Entity;
      if Fld_2 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_304 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_304 := Untyped_Item_304; 
Is_Null_1 := Bare_Item_304 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_304); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_304 := If_Result_1; 
   --# scope-start
               --# bind d Item_304
--# expr-start 9733 '<Cast to AttributeDefClause at ast.py:17036>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17036
Cast_Expr := Item_304; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Attribute_Def_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Attribute_Def_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Attribute_Def_Clause;
   end if;
--# expr-done 9733
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 9741 '<Let at ast.py:17037>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17037
--# scope-start
--# expr-start 9735 '<Cast to AttributeRef at ast.py:17038>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17038
--# expr-start 9734 '<FieldAccess for attribute_expr at ast.py:17038>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17038
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Attribute_Def_Clause_F_Attribute_Expr, Info => Var_Expr_1.Info);
--# expr-done 9734
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9735
Attr := Cast_Result_1; 
--# bind attr Attr
--# expr-start 9737 '<FieldAccess for name_is at ast.py:17039>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17039
--# expr-start 9736 '<FieldAccess for attribute at ast.py:17039>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17039
      if Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Attr.Node.Attribute_Ref_F_Attribute, Info => Attr.Info);
--# expr-done 9736
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_6 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_5.Node, Sym => Name);
--# end
--# expr-done 9737
if Fld_6 then
   --# expr-start 9740 '<Eq at ast.py:17040>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17040
--# expr-start 9739 '<FieldAccess for referenced_defining_name at ast.py:17040>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17040
--# expr-start 9738 '<FieldAccess for prefix at ast.py:17040>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17040
      if Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Attr.Node.Attribute_Ref_F_Prefix, Info => Attr.Info);
--# expr-done 9738
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Fld_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_7.Info);
--# end
--# expr-done 9739
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 9740
   If_Result_2 := Is_Equiv;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 9741
Scope_Result_1 := Let_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
      if Result_Var_1 then
         Find_Result := Item_304; 
      end if;
   --# end
               exit when Result_Var_1;
         end loop;
      end;
      end if;
--# expr-done 9742
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Attribute_Def_Clause_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Attribute_Def_Clause
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_4 := No_Entity_Attribute_Def_Clause;
   end if;
--# expr-done 9743
Let_Result_1 := Cast_Result_4; 
--# end
--# expr-done 9744
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Representation_Clause;
--# end
--# property-start DefiningName.get_at_clause /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17045
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_At_Clause
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_At_Clause
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_At_Clause;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_At_Clause;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_At_Clause;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Item_305 : Internal_Entity;
Bare_Item_305 : Bare_Ada_Node;
Find_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_At_Clause;
Let_Result : Internal_Entity_At_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9754 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9753 '<Cast to AtClause.entity at ast.py:17055>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17055
--# expr-start 9752 '<Find at ast.py:17051>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17051
   --# expr-start 9747 '<as_entity at ast.py:17051>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17051
--# expr-start 9745 '<FieldAccess for declarative_scope at ast.py:17051>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17051
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 9745
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9746 '<FieldAccess for decls at ast.py:17051>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17051
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 9746
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 9747
Fld_2 := If_Result.Node;
      Find_Result := No_Entity;
      if Fld_2 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_305 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_305 := Untyped_Item_305; 
Is_Null_1 := Bare_Item_305 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_305); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_305 := If_Result_1; 
   --# scope-start
               --# bind d Item_305
--# expr-start 9748 '<Cast to AtClause at ast.py:17052>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17052
Cast_Expr := Item_305; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_At_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_At_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_At_Clause;
   end if;
--# expr-done 9748
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_At_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 9751 '<Eq at ast.py:17053>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17053
--# expr-start 9750 '<FieldAccess for referenced_defining_name at ast.py:17053>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17053
--# expr-start 9749 '<FieldAccess for name at ast.py:17053>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17053
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Var_Expr_1.Node.At_Clause_F_Name, Info => Var_Expr_1.Info);
--# expr-done 9749
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 9750
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 9751
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
      if Result_Var_1 then
         Find_Result := Item_305; 
      end if;
   --# end
               exit when Result_Var_1;
         end loop;
      end;
      end if;
--# expr-done 9752
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_At_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_At_Clause
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_3 := No_Entity_At_Clause;
   end if;
--# expr-done 9753
Let_Result := Cast_Result_3; 
--# end
--# expr-done 9754
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_At_Clause;
--# end
--# property-start DefiningName.is_imported /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17057
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Imported
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Sym : Symbol_Type;
Fld : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9759 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9756 '<FieldAccess for has_aspect at ast.py:17063>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17063
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9755 '<Symbol Import at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 9755
--# property-call-start DefiningName.has_aspect
Fld := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 9756
if Fld then
   If_Result := True;
else
   --# expr-start 9758 '<FieldAccess for has_aspect at ast.py:17063>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17063
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9757 '<Symbol Interface at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface); 
--# expr-done 9757
--# property-call-start DefiningName.has_aspect
Fld_1 := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 9758
   If_Result := Fld_1;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 9759
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Is_Imported;
--# end
--# property-start DefiningName.is_ghost_code /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17065
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Ghost_Code
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Instantiation;
Var_Expr_2 : Internal_Entity_Name;
Match_0 : Internal_Entity_Subp_Renaming_Decl;
Match_1 : Internal_Entity_Package_Renaming_Decl;
Match_2 : Internal_Entity_Generic_Renaming_Decl;
Match_3 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Basic_Decl;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Instantiation;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Renaming_Clause;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Renaming_Clause;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Name;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Subp_Renaming_Decl;
Let_Result : Internal_Entity_Name;
Scope_Result_2 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Package_Renaming_Decl;
Let_Result_1 : Internal_Entity_Name;
Scope_Result_3 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity_Generic_Renaming_Decl;
Let_Result_2 : Internal_Entity_Name;
Scope_Result_4 : Internal_Entity_Name;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Name;
Scope_Result_5 : Internal_Entity_Name;
Match_Result : Internal_Entity_Name;
Fld_11 : Internal_Entity_Defining_Name;
Fld_12 : Boolean;
Scope_Result_6 : Boolean;
Result_Var_2 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_3 : Boolean;
If_Result_2 : Boolean;
Let_Result_4 : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Ghost_Code,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 9776 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 9760 '<FieldAccess for basic_decl_no_internal at ast.py:17071>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17071
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl_no_internal
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl_No_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9760
Block_Var := Fld; 
--# bind bd Block_Var
--# expr-start 9762 '<FieldAccess for has_aspect at ast.py:17073>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17073
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9761 '<Symbol Ghost at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ghost); 
--# expr-done 9761
--# property-call-start DefiningName.has_aspect
Fld_1 := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 9762
if Fld_1 then
   If_Result := True;
else
--# expr-start 9763 '<FieldAccess for parent_basic_decl at ast.py:17074>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17074
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 9763
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 9764 '<FieldAccess for is_ghost_code at ast.py:17074>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17074
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_ghost_code
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Ghost_Code (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 9764
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 9765 '<Cast to GenericInstantiation at ast.py:17077>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17077
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Create_Internal_Entity_Generic_Instantiation
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Instantiation;
   end if;
--# expr-done 9765
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind gi Var_Expr_1
--# expr-start 9767 '<FieldAccess for is_ghost_code at ast.py:17078>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17078
--# expr-start 9766 '<FieldAccess for designated_bare_generic_decl at ast.py:17078>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17078
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.designated_bare_generic_decl
Fld_4 := Libadalang.Implementation.Generic_Instantiation_P_Designated_Bare_Generic_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9766
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_ghost_code
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Ghost_Code (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 9767
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 9773 '<Match at ast.py:17082>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17082
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Block_Var; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subp_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind sr Match_0
--# expr-start 9769 '<FieldAccess for renamed_object at ast.py:17083>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17083
--# expr-start 9768 '<FieldAccess for renames at ast.py:17083>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17083
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Renaming_Clause (Node => Match_0.Node.Subp_Renaming_Decl_F_Renames, Info => Match_0.Info);
--# expr-done 9768
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Fld_6.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_6.Info);
--# expr-done 9769
Let_Result := Fld_7; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Package_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind pr Match_1
--# expr-start 9771 '<FieldAccess for renamed_object at ast.py:17084>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17084
--# expr-start 9770 '<FieldAccess for renames at ast.py:17084>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17084
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Renaming_Clause (Node => Match_1.Node.Package_Renaming_Decl_F_Renames, Info => Match_1.Info);
--# expr-done 9770
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Fld_8.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_8.Info);
--# expr-done 9771
Let_Result_1 := Fld_9; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind gr Match_2
--# expr-start 9772 '<FieldAccess for renaming_name at ast.py:17085>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17085
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld_10 := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 9772
Let_Result_2 := Fld_10; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_No_Type_Object_Renaming_Decl | Ada_Single_Protected_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Name; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 9773
Var_Expr_2 := Match_Result;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
   --# bind c Var_Expr_2
--# expr-start 9774 '<FieldAccess for referenced_defining_name at ast.py:17087>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17087
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_11 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Var_Expr_2.Node, Imprecise_Fallback => False, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 9774
Var_Expr_3 := Fld_11;
if Var_Expr_3 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 9775 '<FieldAccess for is_ghost_code at ast.py:17087>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17087
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_ghost_code
Fld_12 := Libadalang.Implementation.Defining_Name_P_Is_Ghost_Code (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 9775
Scope_Result_6 := Fld_12;
   --# end
   Result_Var_2 := Scope_Result_6;
else
   Result_Var_2 := False;
end if;
Scope_Result_7 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_7;
else
   Result_Var_3 := False;
end if;
   If_Result_2 := Result_Var_3;
end if;
Let_Result_4 := If_Result_2; 
--# end
--# expr-done 9776
         Property_Result := Let_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Is_Ghost_Code;
--# end
--# property-start DefiningName.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17090
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Xref_Equation
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1194 with Inline_Always;
            procedure Finalizer_Scope_3150 with Inline_Always;
      Var_Expr : Internal_Entity_Dotted_Name;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1194 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1194;
            procedure Finalizer_Scope_3150 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3150;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9783 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9778 '<Cast to DottedName at ast.py:17095>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17095
--# expr-start 9777 '<FieldAccess for name at ast.py:17095>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17095
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 9777
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 9778
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 9782 '<bind at ast.py:17100>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17100
--# expr-start 9779 '<FieldAccess for std_env at ast.py:17100>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17100
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Ent.Node);
--# end
--# expr-done 9779
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 9781 '<FieldAccess for xref_equation at ast.py:17100>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17100
--# expr-start 9780 '<FieldAccess for prefix at ast.py:17100>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17100
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 9780
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 9781
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9782
Scope_Result := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3150;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17101") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 9783
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1194;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1194;
                     Finalizer_Scope_3150;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Xref_Equation;
--# end
--# property-start DefiningName.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17106
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Xref_Entry_Point
  (Node : Bare_Defining_Name
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Name;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9785 '<IsA DottedName at ast.py:17106>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17106
--# expr-start 9784 '<FieldAccess for name at ast.py:17106>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17106
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 9784
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 9785
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Synthetic_Defining_Name
   --
      procedure Initialize_Fields_For_Synthetic_Defining_Name
        (Self : Bare_Synthetic_Defining_Name
         ; Defining_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Defining_Name
              (Self, Defining_Name_F_Name);
      end Initialize_Fields_For_Synthetic_Defining_Name;
--# property-start SyntheticDefiningName.name_symbol /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18465
pragma Warnings (Off, "is not referenced");
 function Synthetic_Defining_Name_P_Name_Symbol
  (Node : Bare_Synthetic_Defining_Name
  )
   return Symbol_Type
is
   Self : Bare_Synthetic_Defining_Name  := Bare_Synthetic_Defining_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Bare_Name;
Fld_1 : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9787 '<FieldAccess for name_symbol at ast.py:18465>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18465
--# expr-start 9786 '<FieldAccess for name at ast.py:18465>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18465
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 9786
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 9787
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Defining_Name_P_Name_Symbol;
--# end
--# property-start SyntheticDefiningName.as_symbol_array /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18467
pragma Warnings (Off, "is not referenced");
 function Synthetic_Defining_Name_P_As_Symbol_Array
  (Node : Bare_Synthetic_Defining_Name
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Synthetic_Defining_Name  := Bare_Synthetic_Defining_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1282 with Inline_Always;
      Fld : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1282 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1282;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9788 '<FieldAccess for name_symbol at ast.py:18467>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18467
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 9788
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1282;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1282;
            raise;
      end;
   return Property_Result;
end Synthetic_Defining_Name_P_As_Symbol_Array;
--# end
   --
   --  Primitives for Bare_Discrete_Subtype_Name
   --
      procedure Initialize_Fields_For_Discrete_Subtype_Name
        (Self : Bare_Discrete_Subtype_Name
         ; Discrete_Subtype_Name_F_Subtype : Bare_Discrete_Subtype_Indication
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Discrete_Subtype_Name_F_Subtype := Discrete_Subtype_Name_F_Subtype;
      end Initialize_Fields_For_Discrete_Subtype_Name;
   function Discrete_Subtype_Name_F_Subtype
     (Node : Bare_Discrete_Subtype_Name) return Bare_Discrete_Subtype_Indication
   is
   begin
         return Node.Discrete_Subtype_Name_F_Subtype;
   end;
   --
   --  Primitives for Bare_Dotted_Name
   --
      procedure Initialize_Fields_For_Dotted_Name
        (Self : Bare_Dotted_Name
         ; Dotted_Name_F_Prefix : Bare_Name
         ; Dotted_Name_F_Suffix : Bare_Base_Id
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Dotted_Name_F_Prefix := Dotted_Name_F_Prefix;
            Self.Dotted_Name_F_Suffix := Dotted_Name_F_Suffix;
      end Initialize_Fields_For_Dotted_Name;
   function Dotted_Name_F_Prefix
     (Node : Bare_Dotted_Name) return Bare_Name
   is
   begin
         return Node.Dotted_Name_F_Prefix;
   end;
   function Dotted_Name_F_Suffix
     (Node : Bare_Dotted_Name) return Bare_Base_Id
   is
   begin
         return Node.Dotted_Name_F_Suffix;
   end;
--# property-start DottedName.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19907
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Ref_Var
  (Node : Bare_Dotted_Name
  )
   return Logic_Var
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Base_Id;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9790 '<FieldAccess for ref_var at ast.py:19907>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19907
--# expr-start 9789 '<FieldAccess for suffix at ast.py:19907>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19907
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 9789
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 9790
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Ref_Var;
--# end
--# property-start DottedName.subp_spec_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19909
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Subp_Spec_Var
  (Node : Bare_Dotted_Name
  )
   return Logic_Var
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Base_Id;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9792 '<FieldAccess for subp_spec_var at ast.py:19909>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19909
--# expr-start 9791 '<FieldAccess for suffix at ast.py:19909>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19909
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 9791
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld);
--# end
--# expr-done 9792
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Subp_Spec_Var;
--# end
--# property-start DottedName.defines_subp_spec_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19910
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Dotted_Name
  )
   return Boolean
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start DottedName.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19912
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Has_Context_Free_Type
  (Node : Bare_Dotted_Name
  )
   return Boolean
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Base_Id;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9795 '<Not at ast.py:19912>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19912
--# expr-start 9794 '<IsA CharLiteral at ast.py:19912>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19912
--# expr-start 9793 '<FieldAccess for suffix at ast.py:19912>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19912
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 9793
Is_A := Fld /= null 
and then Fld.Kind in Ada_Char_Literal_Range; 
--# expr-done 9794
Not_Val := not (Is_A); 
--# expr-done 9795
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Has_Context_Free_Type;
--# end
--# property-start DottedName.complete_items /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19914
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Complete_Items
  (Node : Bare_Dotted_Name
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1380 with Inline_Always;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Bound_No_Visibility : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Boolean;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Boolean;
Fld_11 : Internal_Unit;
Fld_12 : Boolean;
If_Result : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_13 : Integer;
New_Struct : Internal_Completion_Item;
Is_A : Boolean;
Not_Val : Boolean;
Fld_14 : Analysis_Unit_Kind;
Is_Equal : Boolean;
Fld_15 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Map_Result : Internal_Completion_Item_Array_Access;
Dyn_Var_Bind_Result : Internal_Completion_Item_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Completion_Item_Array_Access;
Dyn_Var_Bind_Result_2 : Internal_Completion_Item_Array_Access;
Let_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1380 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1380;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9822 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9821 '<bind at ast.py:19916>' Dyn_Var_Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19916
--# expr-start 9796 '<FieldAccess for origin_node at ast.py:19916>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19916
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 9796
Bound_Origin := Fld; 
--# expr-start 9820 '<bind at ast.py:19916>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19916
--# expr-start 9797 '<FieldAccess for node_env at ast.py:19917>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19917
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 9797
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 9819 '<bind at ast.py:19921>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19921
Bound_No_Visibility := True; 
--# expr-start 9818 '<FilterMap at ast.py:19927>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19927
   --# expr-start 9800 '<FieldAccess for env_get_public at ast.py:19923>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19923
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9799 '<FieldAccess for designated_env at ast.py:19924>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19924
--# expr-start 9798 '<FieldAccess for prefix at ast.py:19924>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19924
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9798
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Bound_Env, Origin => Bound_Origin, No_Visibility => Bound_No_Visibility, E_Info => Fld_2.Info);
--# end
--# expr-done 9799
--# property-call-start AdaNode.env_get_public
Fld_4 := Libadalang.Implementation.Ada_Node_P_Env_Get_Public (Node => Self, Env => Fld_3, Symbol => null, Lookup => Flat);
--# end
--# expr-done 9800
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_4;
      begin
         for Item_306 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_306
--# expr-start 9802 '<Not at ast.py:19950>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19950
--# expr-start 9801 '<IsA PrivatePart at ast.py:19950>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19950
Is_A := Item_306.Node /= null 
and then Item_306.Node.Kind in Ada_Private_Part_Range; 
--# expr-done 9801
Not_Val := not (Is_A); 
--# expr-done 9802
if Not_Val then
--# expr-start 9804 '<Eq at ast.py:19952>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19952
--# expr-start 9803 '<FieldAccess for owning_unit_kind at ast.py:19952>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19952
      if Item_306.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.owning_unit_kind
Fld_14 := Libadalang.Implementation.Ada_Node_P_Owning_Unit_Kind (Node => Item_306.Node);
--# end
--# expr-done 9803
Is_Equal := Fld_14 = Unit_Specification; 
--# expr-done 9804
if Is_Equal then
   If_Result_1 := True;
else
   --# expr-start 9805 '<FieldAccess for has_visibility at ast.py:19953>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19953
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_15 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_306);
--# end
--# expr-done 9805
   If_Result_1 := Fld_15;
end if;
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
         if If_Result_2 then
      --# expr-start 9817 '<New CompletionItem at ast.py:19928>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19928
--# expr-start 9806 '<Cast to BasicDecl at ast.py:19929>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19929
Cast_Expr := Item_306; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 9806
--# expr-start 9809 '<FieldAccess for dottable_subp at ast.py:19930>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19930
--# expr-start 9808 '<FieldAccess for md at ast.py:19930>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19930
--# expr-start 9807 '<FieldAccess for info at ast.py:19930>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19930
Fld_5 := Item_306.Info;
--# expr-done 9807
Fld_6 := Fld_5.Md;
--# expr-done 9808
Fld_7 := Fld_6.Dottable_Subp;
--# expr-done 9809
--# expr-start 9812 '<FieldAccess for dottable_subp at ast.py:19934>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19934
--# expr-start 9811 '<FieldAccess for md at ast.py:19934>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19934
--# expr-start 9810 '<FieldAccess for info at ast.py:19934>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19934
Fld_8 := Item_306.Info;
--# expr-done 9810
Fld_9 := Fld_8.Md;
--# expr-done 9811
Fld_10 := Fld_9.Dottable_Subp;
--# expr-done 9812
if Fld_10 then
   If_Result := True;
else
   --# expr-start 9814 '<FieldAccess for has_with_visibility at ast.py:19937>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19937
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9813 '<FieldAccess for unit at ast.py:19937>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19937
      if Item_306.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Unit (Node => Item_306.Node);
--# expr-done 9813
--# property-call-start AdaNode.has_with_visibility
Fld_12 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_11);
--# end
--# expr-done 9814
   If_Result := Fld_12;
end if;
--# expr-start 9816 '<FieldAccess for complete_item_weight at ast.py:19940>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19940
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9815 '<Cast to BasicDecl at ast.py:19940>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19940
Cast_Expr_1 := Item_306; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 9815
--# property-call-start '[dispatcher]AdaNode.complete_item_weight'
Fld_13 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Complete_Item_Weight (Node => Ent.Node, Item => Cast_Result_1, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 9816
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_7, Is_Visible => If_Result, Weight => Fld_13); 
--# expr-done 9817
         declare
            Item_To_Append : constant Internal_Completion_Item := New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9818
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9819
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 9820
Dyn_Var_Bind_Result_2 := Dyn_Var_Bind_Result_1; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 9821
Let_Result := Dyn_Var_Bind_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 9822
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1380;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1380;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Complete_Items;
--# end
--# property-start DottedName.complete_item_weight /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19960
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Complete_Item_Weight
  (Node : Bare_Dotted_Name
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
      Is_A : Boolean;
Is_A_1 : Boolean;
Fld : Integer;
If_Result : Integer;
If_Result_1 : Integer;
Let_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9827 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9826 '<Cond at ast.py:19962>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19962
--# expr-start 9823 '<IsA ComponentDecl, DiscriminantSpec at ast.py:19964>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19964
Is_A := Item.Node /= null 
and then Item.Node.Kind in Ada_Component_Decl_Range | Ada_Discriminant_Spec_Range; 
--# expr-done 9823
if Is_A then
   If_Result_1 := 100;
else
--# expr-start 9824 '<IsA BasicSubpDecl, BaseSubpBody at ast.py:19968>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19968
Is_A_1 := Item.Node /= null 
and then Item.Node.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 9824
if Is_A_1 then
   If_Result := 75;
else
   --# expr-start 9825 '<Super at ast.py:19972>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.complete_item_weight
Fld := Libadalang.Implementation.Ada_Node_P_Complete_Item_Weight (Node => Ent.Node, Item => Item, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9825
   If_Result := Fld;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 9826
Let_Result := If_Result_1; 
--# end
--# expr-done 9827
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Complete_Item_Weight;
--# end
--# property-start DottedName.designated_env_no_overloading /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19975
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Env_No_Overloading
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1382 with Inline_Always;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1382 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1382;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9833 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9829 '<FieldAccess for designated_env_no_overloading at ast.py:19977>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19977
--# expr-start 9828 '<FieldAccess for prefix at ast.py:19977>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19977
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9828
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld.Info);
--# end
--# expr-done 9829
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 9832 '<bind at ast.py:19978>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19978
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 9831 '<FieldAccess for designated_env_no_overloading at ast.py:19979>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19979
--# expr-start 9830 '<FieldAccess for suffix at ast.py:19979>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19979
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 9830
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld_2.Info);
--# end
--# expr-done 9831
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9832
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9833
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1382;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1382;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Designated_Env_No_Overloading;
--# end
--# property-start DottedName.designated_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19981
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Env
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1383 with Inline_Always;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1383 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1383;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9839 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9835 '<FieldAccess for designated_env at ast.py:19983>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19983
--# expr-start 9834 '<FieldAccess for prefix at ast.py:19983>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19983
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9834
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld.Info);
--# end
--# expr-done 9835
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 9838 '<bind at ast.py:19984>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19984
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 9837 '<FieldAccess for designated_env at ast.py:19984>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19984
--# expr-start 9836 '<FieldAccess for suffix at ast.py:19984>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19984
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 9836
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld_2.Info);
--# end
--# expr-done 9837
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9838
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9839
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1383;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1383;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Designated_Env;
--# end
--# property-start DottedName.all_env_els_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19986
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_All_Env_Els_Impl
  (Node : Bare_Dotted_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1384 with Inline_Always;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1384 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1384;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9845 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9841 '<FieldAccess for designated_env at ast.py:19992>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19992
--# expr-start 9840 '<FieldAccess for prefix at ast.py:19992>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19992
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9840
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld.Info);
--# end
--# expr-done 9841
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 9844 '<bind at ast.py:19993>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19993
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 9843 '<FieldAccess for all_env_els_impl at ast.py:19995>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19995
--# expr-start 9842 '<FieldAccess for suffix at ast.py:19995>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19995
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 9842
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_2.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 9843
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9844
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9845
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1384;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1384;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_All_Env_Els_Impl;
--# end
--# property-start DottedName.scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19998
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Scope
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1385 with Inline_Always;
            procedure Finalizer_Scope_3152 with Inline_Always;
      Var_Expr : Bare_Base_Id;
Fld : Bare_Base_Id;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1385 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1385;
            procedure Finalizer_Scope_3152 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3152;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 9846 '<FieldAccess for suffix at ast.py:19998>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19998
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 9846
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind sfx Var_Expr
--# expr-start 9849 '<bind at ast.py:19999>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19999
--# expr-start 9847 '<FieldAccess for parent_scope at ast.py:19999>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19999
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Self, Env => Env);
--# end
--# expr-done 9847
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 9848 '<FieldAccess for scope at ast.py:19999>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19999
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Var_Expr, Env => Bound_Env);
--# end
--# expr-done 9848
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9849
Scope_Result := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3152;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1385;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1385;
                     Finalizer_Scope_3152;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Scope;
--# end
--# property-start DottedName.parent_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20003
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Parent_Scope
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1386 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1386 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1386;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9851 '<FieldAccess for scope at ast.py:20003>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20003
--# expr-start 9850 '<FieldAccess for prefix at ast.py:20003>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20003
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Prefix;
--# expr-done 9850
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 9851
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1386;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1386;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Parent_Scope;
--# end
--# property-start DottedName.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20005
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Relative_Name
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Base_Id;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9853 '<FieldAccess for relative_name at ast.py:20005>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20005
--# expr-start 9852 '<FieldAccess for suffix at ast.py:20005>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20005
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 9852
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9853
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Relative_Name;
--# end
--# property-start DottedName.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20007
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Env_Elements_Impl
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1388 with Inline_Always;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Base_Id;
Fld_4 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1388 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1388;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9861 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9857 '<bind at ast.py:20009>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20009
--# expr-start 9854 '<FieldAccess for origin_node at ast.py:20009>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20009
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 9854
Bound_Origin := Fld; 
--# expr-start 9856 '<FieldAccess for designated_env at ast.py:20010>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20010
--# expr-start 9855 '<FieldAccess for prefix at ast.py:20010>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9855
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, No_Visibility => False, E_Info => Fld_1.Info);
--# end
--# expr-done 9856
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9857
Block_Var := Dyn_Var_Bind_Result; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 9860 '<bind at ast.py:20011>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20011
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 9859 '<FieldAccess for env_elements_baseid at ast.py:20011>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20011
--# expr-start 9858 '<FieldAccess for suffix at ast.py:20011>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20011
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 9858
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld_4 := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Fld_3.Node, Env => Bound_Env, E_Info => Fld_3.Info);
--# end
--# expr-done 9859
Dyn_Var_Bind_Result_1 := Fld_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 9860
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 9861
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1388;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1388;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Env_Elements_Impl;
--# end
--# property-start DottedName.designated_type_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20013
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Type_Impl
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1389 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1389 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1389;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9867 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9866 '<bind at ast.py:20015>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20015
--# expr-start 9863 '<FieldAccess for designated_env_no_overloading at ast.py:20015>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20015
--# expr-start 9862 '<FieldAccess for prefix at ast.py:20015>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20015
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9862
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld.Info);
--# end
--# expr-done 9863
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 9865 '<FieldAccess for designated_type_impl at ast.py:20016>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20016
--# expr-start 9864 '<FieldAccess for suffix at ast.py:20016>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20016
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 9864
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 9865
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 9866
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 9867
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1389;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1389;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Designated_Type_Impl;
--# end
--# property-start DottedName.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20018
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Xref_Equation
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1390 with Inline_Always;
            procedure Finalizer_Scope_3166 with Inline_Always;
      Block_Var : Logic_Equation;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Bare_Base_Id;
Fld_8 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Bare_Base_Id;
Fld_11 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_12 : Internal_Entity_Array_Access;
Fld_13 : Bare_Base_Id;
Fld_14 : Logic_Var;
Bind_Result_2 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_15 : Bare_Name;
Cast_Result_1 : Bare_Expr;
Fld_16 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_17 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1390 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1390;
            procedure Finalizer_Scope_3166 is
            begin
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3166;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9901 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9875 '<And at ast.py:20020>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20020
--# expr-start 9869 '<FieldAccess for sub_equation at ast.py:20020>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20020
--# expr-start 9868 '<FieldAccess for prefix at ast.py:20020>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20020
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9868
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 9869
--# expr-start 9874 '<bind at ast.py:20021>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20021
--# expr-start 9871 '<FieldAccess for designated_env at ast.py:20021>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20021
--# expr-start 9870 '<FieldAccess for prefix at ast.py:20021>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20021
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9870
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 9871
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 9873 '<FieldAccess for sub_equation at ast.py:20022>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20022
--# expr-start 9872 '<FieldAccess for suffix at ast.py:20022>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20022
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 9872
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 9873
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9874
And_Pred := Create_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20020") else null)); 
--# expr-done 9875
Block_Var := And_Pred; Inc_Ref (Block_Var);
--# bind base Block_Var
--# expr-start 9900 '<If at ast.py:20024>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20024
--# expr-start 9878 '<Not at ast.py:20025>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20025
--# expr-start 9877 '<IsNull at ast.py:20025>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20025
--# expr-start 9876 '<FieldAccess for designated_type_impl at ast.py:20025>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20025
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_17 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9876
Is_Null := Fld_17.Node = null; 
--# expr-done 9877
Not_Val := not (Is_Null); 
--# expr-done 9878
if Not_Val then
   If_Result := Block_Var;
else
   --# expr-start 9899 '<And at ast.py:20027>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20027
--# expr-start 9888 '<And at ast.py:20027>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20027
--# expr-start 9883 '<And at ast.py:20027>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20027
--# expr-start 9882 '<Bind at ast.py:20028>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20028
--# expr-start 9879 '<FieldAccess for expected_type_var at ast.py:20028>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20028
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 9879
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 9881 '<FieldAccess for expected_type_var at ast.py:20028>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20028
--# expr-start 9880 '<FieldAccess for suffix at ast.py:20028>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20028
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Dotted_Name_F_Suffix;
--# expr-done 9880
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_7);
--# expr-done 9881
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result := Solver.Create_Unify (Fld_6, Fld_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20028") else null)); 
--# expr-done 9882
And_Pred_1 := Create_And (Block_Var, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20027") else null)); 
--# expr-done 9883
--# expr-start 9887 '<Bind at ast.py:20029>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20029
--# expr-start 9884 '<FieldAccess for type_var at ast.py:20029>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20029
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9884
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 9886 '<FieldAccess for type_var at ast.py:20029>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20029
--# expr-start 9885 '<FieldAccess for suffix at ast.py:20029>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20029
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Dotted_Name_F_Suffix;
--# expr-done 9885
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10);
--# expr-done 9886
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Bind_Result_1 := Solver.Create_Unify (Fld_9, Fld_11, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20029") else null)); 
--# expr-done 9887
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20027") else null)); 
--# expr-done 9888
--# expr-start 9898 '<LogicAny at ast.py:20030>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20030
--# expr-start 9897 '<Map at ast.py:20030>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20030
   --# expr-start 9889 '<FieldAccess for env_elements at ast.py:20030>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20030
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_12 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 9889
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_12;
      begin
         for Item_307 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_307
      --# expr-start 9896 '<And at ast.py:20032>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20032
--# expr-start 9892 '<Bind at ast.py:20032>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20032
--# expr-start 9891 '<FieldAccess for ref_var at ast.py:20032>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20032
--# expr-start 9890 '<FieldAccess for suffix at ast.py:20032>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20032
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Self.Dotted_Name_F_Suffix;
--# expr-done 9890
      if Fld_13 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_13);
--# end
--# expr-done 9891
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
Bind_Result_2 := Solver.Create_Assign (Fld_14, Item_307, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20032") else null)); 
--# expr-done 9892
--# expr-start 9895 '<FieldAccess for constrain_prefix at ast.py:20033>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20033
--# expr-start 9893 '<Cast to BasicDecl.entity at ast.py:20033>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20033
Cast_Expr := Item_307; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 9893
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9894 '<FieldAccess for prefix at ast.py:20033>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20033
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Self.Dotted_Name_F_Prefix;
--# expr-done 9894
      Cast_Result_1 := Fld_15;
--# property-call-start '[dispatcher]BasicDecl.constrain_prefix'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Constrain_Prefix (Node => Cast_Result.Node, Prefix => Cast_Result_1, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 9895
And_Pred_3 := Create_And (Bind_Result_2, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20032") else null)); 
--# expr-done 9896
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3166;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9897
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20030") else null)); 
--# expr-done 9898
And_Pred_4 := Create_And (And_Pred_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:20027") else null)); 
--# expr-done 9899
   If_Result := And_Pred_4;
end if;
      Inc_Ref (If_Result);
--# expr-done 9900
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9901
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1390;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1390;
                     Finalizer_Scope_3166;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Xref_Equation;
--# end
--# property-start DottedName.is_constant /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20037
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Is_Constant
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Name;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9906 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9903 '<FieldAccess for is_constant at ast.py:20042>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20042
--# expr-start 9902 '<FieldAccess for prefix at ast.py:20042>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20042
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 9902
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9903
if Fld_1 then
   If_Result := True;
else
   --# expr-start 9905 '<FieldAccess for is_constant at ast.py:20043>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20043
--# expr-start 9904 '<FieldAccess for suffix at ast.py:20043>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 9904
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 9905
   If_Result := Fld_3;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 9906
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Is_Constant;
--# end
   --
   --  Primitives for Bare_End_Name
   --
      procedure Initialize_Fields_For_End_Name
        (Self : Bare_End_Name
         ; End_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.End_Name_F_Name := End_Name_F_Name;
      end Initialize_Fields_For_End_Name;
   function End_Name_F_Name
     (Node : Bare_End_Name) return Bare_Name
   is
   begin
         return Node.End_Name_F_Name;
   end;
--# property-start EndName.parent_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17116
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Parent_Scope
  (Node : Bare_End_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1196 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1196 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1196;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9908 '<FieldAccess for parent_scope at ast.py:17116>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17116
--# expr-start 9907 '<FieldAccess for name at ast.py:17116>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17116
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 9907
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 9908
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1196;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1196;
            raise;
      end;
   return Property_Result;
end End_Name_P_Parent_Scope;
--# end
--# property-start EndName.scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17117
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Scope
  (Node : Bare_End_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1197 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1197 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1197;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9910 '<FieldAccess for scope at ast.py:17117>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17117
--# expr-start 9909 '<FieldAccess for name at ast.py:17117>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17117
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 9909
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 9910
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1197;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1197;
            raise;
      end;
   return Property_Result;
end End_Name_P_Scope;
--# end
--# property-start EndName.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17118
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Relative_Name
  (Node : Bare_End_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
     Ent : Internal_Entity_End_Name :=
       Internal_Entity_End_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9912 '<FieldAccess for relative_name at ast.py:17118>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17118
--# expr-start 9911 '<FieldAccess for name at ast.py:17118>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17118
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 9911
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9912
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end End_Name_P_Relative_Name;
--# end
--# property-start EndName.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17119
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Ref_Var
  (Node : Bare_End_Name
  )
   return Logic_Var
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9914 '<FieldAccess for ref_var at ast.py:17119>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17119
--# expr-start 9913 '<FieldAccess for name at ast.py:17119>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17119
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 9913
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 9914
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end End_Name_P_Ref_Var;
--# end
--# property-start EndName.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17120
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Env_Elements_Impl
  (Node : Bare_End_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
     Ent : Internal_Entity_End_Name :=
       Internal_Entity_End_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1200 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1200 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1200;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9916 '<FieldAccess for env_elements_impl at ast.py:17120>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17120
--# expr-start 9915 '<FieldAccess for name at ast.py:17120>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17120
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 9915
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 9916
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1200;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1200;
            raise;
      end;
   return Property_Result;
end End_Name_P_Env_Elements_Impl;
--# end
--# property-start EndName.basic_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17122
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Basic_Decl
  (Node : Bare_End_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1201 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Named_Stmt;
Fld : Bare_Named_Stmt_Decl;
Cast_Result_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Node_Parents_1 : Bare_Ada_Node_Array_Access;
Is_A_1 : Boolean;
Find_Result_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1201 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Node_Parents_1);
            end Finalizer_Scope_1201;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_End_Name_End_Name_P_Basic_Decl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_End_Name,
                                As_Bare_End_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 9919 '<Find at ast.py:17123>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17123
   --# expr-start 9917 '<FieldAccess for parents at ast.py:17123>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17123
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 9917
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_308 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_308
      --# expr-start 9918 '<IsA NamedStmt at ast.py:17123>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17123
Is_A := Item_308 /= null 
and then Item_308.Kind in Ada_Named_Stmt_Range; 
--# expr-done 9918
      if Is_A then
         Find_Result := Item_308; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 9919
Var_Expr_1 := Find_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ns Var_Expr_1
--# expr-start 9923 '<as_entity at ast.py:17125>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17125
--# expr-start 9922 '<Cast to BasicDecl at ast.py:17125>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17125
--# expr-start 9921 '<FieldAccess for decl at ast.py:17125>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17125
--# expr-start 9920 '<Cast to NamedStmt at ast.py:17125>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17125
Cast_Expr := Var_Expr_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Named_Stmt_Range
   then
      Cast_Result := Cast_Expr;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9920
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Cast_Result.Named_Stmt_F_Decl;
--# expr-done 9921
      Cast_Result_1 := Fld;
--# expr-done 9922
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 9923
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 9928 '<as_entity at ast.py:17128>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17128
--# expr-start 9927 '<Cast to BasicDecl at ast.py:17128>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17128
--# expr-start 9926 '<Find at ast.py:17127>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17127
   --# expr-start 9924 '<FieldAccess for parents at ast.py:17127>' Node_Parents_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17127
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents_1 := Parents (Self, True); 
--# expr-done 9924
      Find_Result_1 := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents_1;
      begin
         for Item_309 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_309
      --# expr-start 9925 '<IsA BasicDecl at ast.py:17127>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17127
Is_A_1 := Item_309 /= null 
and then Item_309.Kind in Ada_Basic_Decl; 
--# expr-done 9925
      if Is_A_1 then
         Find_Result_1 := Item_309; 
      end if;
   --# end
               exit when Is_A_1;
         end loop;
      end;
--# expr-done 9926
   if Find_Result_1 = null
      or else Find_Result_1.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Find_Result_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9927
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 9928
   Result_Var_1 := If_Result_1;
end if;
         Property_Result := Result_Var_1;
   --# end
      Finalizer_Scope_1201;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1201;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end End_Name_P_Basic_Decl;
--# end
--# property-start EndName.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17134
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Xref_Equation
  (Node : Bare_End_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
     Ent : Internal_Entity_End_Name :=
       Internal_Entity_End_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1202 with Inline_Always;
            procedure Finalizer_Scope_3171 with Inline_Always;
            procedure Finalizer_Scope_3172 with Inline_Always;
      Var_Expr : Internal_Entity_Accept_Stmt_With_Stmts;
Var_Expr_1 : Internal_Entity_Dotted_Name;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt_With_Stmts;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Entry_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Result_Var : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity_Dotted_Name;
Fld_6 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_7 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1202 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1202;
            procedure Finalizer_Scope_3171 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_3171;
            procedure Finalizer_Scope_3172 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3172;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9944 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9943 '<And at ast.py:17136>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17136
--# expr-start 9930 '<Cast to AcceptStmtWithStmts at ast.py:17136>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17136
--# expr-start 9929 '<parent at ast.py:17136>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17136
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 9929
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Accept_Stmt_With_Stmts_Range
   then
      Cast_Result := Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Accept_Stmt_With_Stmts;
   end if;
--# expr-done 9930
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Accept_Stmt_With_Stmts then
   --# scope-start
   --# bind stmt Var_Expr
--# expr-start 9933 '<Bind at ast.py:17137>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17137
--# expr-start 9931 '<FieldAccess for ref_var at ast.py:17137>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17137
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9931
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 9932 '<FieldAccess for designated_entry at ast.py:17137>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17137
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_2 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Var_Expr.Node, Origin => Origin, Env => Env, E_Info => Var_Expr.Info);
--# end
--# expr-done 9932
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17137") else null)); 
--# expr-done 9933
Scope_Result := Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3171;
   Result_Var := Scope_Result;
else
   --# expr-start 9936 '<Bind at ast.py:17138>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17138
--# expr-start 9934 '<FieldAccess for ref_var at ast.py:17138>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17138
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9934
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 9935 '<FieldAccess for basic_decl at ast.py:17138>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17138
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EndName.basic_decl
Fld_4 := Libadalang.Implementation.End_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9935
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_3, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17138") else null)); 
--# expr-done 9936
   Result_Var := Bind_Result_1;
end if;
      Inc_Ref (Result_Var);
--# expr-start 9938 '<Cast to DottedName at ast.py:17139>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17139
--# expr-start 9937 '<FieldAccess for name at ast.py:17139>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17139
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 9937
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_3 := No_Entity_Dotted_Name;
   end if;
--# expr-done 9938
Var_Expr_1 := Cast_Result_3;
if Var_Expr_1 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr_1
--# expr-start 9942 '<bind at ast.py:17143>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17143
--# expr-start 9939 '<FieldAccess for std_env at ast.py:17144>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17144
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_env
Fld_6 := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Ent.Node);
--# end
--# expr-done 9939
Bound_Env := Fld_6; Inc_Ref (Bound_Env);
--# expr-start 9941 '<FieldAccess for xref_no_overloading at ast.py:17145>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17145
--# expr-start 9940 '<FieldAccess for prefix at ast.py:17145>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17145
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Dotted_Name_F_Prefix, Info => Var_Expr_1.Info);
--# expr-done 9940
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_7.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 9941
Dyn_Var_Bind_Result := Fld_8; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9942
Scope_Result_1 := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3172;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17147") else null)); 
   Result_Var_1 := True_Rel;
end if;
      Inc_Ref (Result_Var_1);
And_Pred := Create_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17136") else null)); 
--# expr-done 9943
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 9944
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1202;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1202;
                     Finalizer_Scope_3171;
                     Finalizer_Scope_3172;
            raise;
      end;
   return Property_Result;
end End_Name_P_Xref_Equation;
--# end
--# property-start EndName.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17150
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Xref_Entry_Point
  (Node : Bare_End_Name
  )
   return Boolean
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end End_Name_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Explicit_Deref
   --
      procedure Initialize_Fields_For_Explicit_Deref
        (Self : Bare_Explicit_Deref
         ; Explicit_Deref_F_Prefix : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Explicit_Deref_F_Prefix := Explicit_Deref_F_Prefix;
      Self.Explicit_Deref_R_Called_Spec := Null_Var_Record;
      end Initialize_Fields_For_Explicit_Deref;
   function Explicit_Deref_F_Prefix
     (Node : Bare_Explicit_Deref) return Bare_Name
   is
   begin
         return Node.Explicit_Deref_F_Prefix;
   end;
--# property-start ExplicitDeref.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16051
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Ref_Var
  (Node : Bare_Explicit_Deref
  )
   return Logic_Var
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9946 '<FieldAccess for ref_var at ast.py:16051>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16051
--# expr-start 9945 '<FieldAccess for prefix at ast.py:16051>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16051
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Explicit_Deref_F_Prefix;
--# expr-done 9945
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 9946
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Ref_Var;
--# end
--# property-start ExplicitDeref.subp_spec_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16055
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Subp_Spec_Var
  (Node : Bare_Explicit_Deref
  )
   return Logic_Var
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9947 '<FieldAccess for r_called_spec at ast.py:16055>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16055
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Explicit_Deref_R_Called_Spec'Unrestricted_Access;
--# expr-done 9947
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Subp_Spec_Var;
--# end
--# property-start ExplicitDeref.defines_subp_spec_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16056
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Defines_Subp_Spec_Var
  (Node : Bare_Explicit_Deref
  )
   return Boolean
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Defines_Subp_Spec_Var;
--# end
--# property-start ExplicitDeref.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16058
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Relative_Name
  (Node : Bare_Explicit_Deref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9949 '<FieldAccess for relative_name at ast.py:16058>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16058
--# expr-start 9948 '<FieldAccess for prefix at ast.py:16058>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16058
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 9948
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9949
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Relative_Name;
--# end
--# property-start ExplicitDeref.designated_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16060
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Designated_Env
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1128 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1128 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1128;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9952 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9951 '<FieldAccess for designated_env at ast.py:16064>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16064
--# expr-start 9950 '<FieldAccess for prefix at ast.py:16064>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16064
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 9950
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld.Info);
--# end
--# expr-done 9951
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 9952
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1128;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1128;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Designated_Env;
--# end
--# property-start ExplicitDeref.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16066
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Env_Elements_Impl
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1129 with Inline_Always;
      Block_Var : Internal_Entity_Name;
Block_Var_1 : Internal_Entity_Array_Access;
Var_Expr : Internal_Entity_Attribute_Ref;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_5 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1129 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1129;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9964 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9953 '<FieldAccess for prefix at ast.py:16068>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16068
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 9953
Block_Var := Fld; 
--# bind prefix Block_Var
--# expr-start 9954 '<FieldAccess for env_elements_impl at ast.py:16069>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16069
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Block_Var.Node, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 9954
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind env_els Block_Var_1
--# expr-start 9963 '<If at ast.py:16070>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16070
--# expr-start 9955 '<Cast to AttributeRef at ast.py:16071>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16071
Cast_Expr_1 := Block_Var; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Attribute_Ref;
   end if;
--# expr-done 9955
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Attribute_Ref then
   --# scope-start
--# expr-start 9956 '<FieldAccess for is_access_attr at ast.py:16071>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16071
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_5 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Var_Expr.Node);
--# end
--# expr-done 9956
Scope_Result_2 := Fld_5;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   If_Result := Block_Var_1;
else
   --# expr-start 9962 '<bind at ast.py:16073>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16073
--# expr-start 9957 '<FieldAccess for origin_node at ast.py:16073>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16073
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 9957
Bound_Origin := Fld_2; 
--# expr-start 9961 '<Filter at ast.py:16073>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16073
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_310 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_310
--# expr-start 9958 '<Cast to BasicDecl at ast.py:16075>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16075
Cast_Expr := Item_310; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 9958
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 9959 '<FieldAccess for expr_type at ast.py:16075>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16075
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Var_Expr_2.Node, Origin => Bound_Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 9959
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_1
--# expr-start 9960 '<FieldAccess for is_access_type at ast.py:16076>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16076
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9960
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
         if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity := Item_310;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9961
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9962
   If_Result := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 9963
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9964
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1129;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1129;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Env_Elements_Impl;
--# end
--# property-start ExplicitDeref.eq_for_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16081
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Eq_For_Type
  (Node : Bare_Explicit_Deref
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1130 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_6 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1130 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1130;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9977 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9976 '<If at ast.py:16083>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16083
--# expr-start 9965 '<FieldAccess for is_access_type at ast.py:16084>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16084
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 9965
if Fld_6 then
   --# expr-start 9975 '<And at ast.py:16086>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16086
--# expr-start 9971 '<And at ast.py:16086>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16086
--# expr-start 9968 '<Bind at ast.py:16086>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16086
--# expr-start 9967 '<FieldAccess for expected_type_var at ast.py:16086>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16086
--# expr-start 9966 '<FieldAccess for prefix at ast.py:16086>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16086
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Explicit_Deref_F_Prefix;
--# expr-done 9966
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 9967
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Cast_Expr := Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16086") else null)); 
--# expr-done 9968
--# expr-start 9970 '<FieldAccess for matches_expected_type at ast.py:16087>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16087
--# expr-start 9969 '<FieldAccess for prefix at ast.py:16087>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16087
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 9969
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_3 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_2.Node, Origin => Origin);
--# end
--# expr-done 9970
And_Pred := Create_And (Bind_Result, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16086") else null)); 
--# expr-done 9971
--# expr-start 9974 '<Bind at ast.py:16088>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16088
--# expr-start 9972 '<FieldAccess for type_var at ast.py:16088>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16088
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9972
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 9973 '<FieldAccess for accessed_type at ast.py:16088>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16088
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 9973
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16088") else null)); 
--# expr-done 9974
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16086") else null)); 
--# expr-done 9975
   If_Result := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16090") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 9976
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9977
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1130;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1130;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Eq_For_Type;
--# end
--# property-start ExplicitDeref.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16093
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Xref_Equation
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1131 with Inline_Always;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1131 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1131;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9979 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 9978 '<FieldAccess for bottom_up_name_equation at ast.py:16095>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16095
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.bottom_up_name_equation
Fld := Libadalang.Implementation.Name_P_Bottom_Up_Name_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9978
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 9979
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1131;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1131;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Xref_Equation;
--# end
--# property-start ExplicitDeref.general_xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16097
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_General_Xref_Equation
  (Node : Bare_Explicit_Deref
      ; Root : Bare_Name
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1132 with Inline_Always;
            procedure Finalizer_Scope_3177 with Inline_Always;
            procedure Finalizer_Scope_3178 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Attribute_Ref;
Fld : Logic_Equation;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_11 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1132 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1132;
            procedure Finalizer_Scope_3177 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_3177;
            procedure Finalizer_Scope_3178 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3178;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10004 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10003 '<And at ast.py:16099>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16099
--# expr-start 9983 '<And at ast.py:16099>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16099
--# expr-start 9980 '<FieldAccess for all_args_xref_equation at ast.py:16100>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16100
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9980
--# expr-start 9982 '<FieldAccess for sub_equation at ast.py:16101>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16101
--# expr-start 9981 '<FieldAccess for prefix at ast.py:16101>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16101
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 9981
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 9982
And_Pred := Create_And (Fld, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16099") else null)); 
--# expr-done 9983
--# expr-start 10002 '<LogicAny at ast.py:16102>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16102
--# expr-start 10001 '<Map at ast.py:16102>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16102
   --# expr-start 9984 '<FieldAccess for env_elements at ast.py:16102>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16102
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 9984
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_311 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_311
--# expr-start 9986 '<FieldAccess for expr_type at ast.py:16103>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16103
--# expr-start 9985 '<Cast to BasicDecl at ast.py:16103>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16103
Cast_Expr := Item_311; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 9985
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Cast_Result.Node, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 9986
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 10000 '<And at ast.py:16105>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16105
--# expr-start 9993 '<And at ast.py:16105>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16105
--# expr-start 9988 '<Bind at ast.py:16105>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16105
--# expr-start 9987 '<FieldAccess for ref_var at ast.py:16105>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16105
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9987
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result := Solver.Create_Assign (Fld_5, Item_311, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16105") else null)); 
--# expr-done 9988
--# expr-start 9992 '<Bind at ast.py:16106>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16106
--# expr-start 9989 '<FieldAccess for ref_var at ast.py:16106>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16106
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9989
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 9991 '<FieldAccess for ref_var at ast.py:16106>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16106
--# expr-start 9990 '<FieldAccess for prefix at ast.py:16106>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16106
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Explicit_Deref_F_Prefix;
--# expr-done 9990
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 9991
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result_1 := Solver.Create_Unify (Fld_6, Fld_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16106") else null)); 
--# expr-done 9992
And_Pred_1 := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16105") else null)); 
--# expr-done 9993
--# expr-start 9999 '<FieldAccess for parent_name_equation at ast.py:16107>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16107
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9998 '<If at ast.py:16108>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16108
--# expr-start 9995 '<Cast to AttributeRef at ast.py:16108>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16108
--# expr-start 9994 '<FieldAccess for prefix at ast.py:16108>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16108
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 9994
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_1 := No_Entity_Attribute_Ref;
   end if;
--# expr-done 9995
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Attribute_Ref then
   --# scope-start
--# expr-start 9996 '<FieldAccess for is_access_attr at ast.py:16108>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16108
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_11 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Var_Expr_1.Node);
--# end
--# expr-done 9996
Scope_Result := Fld_11;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 9997 '<FieldAccess for anonymous_access_type at ast.py:16109>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16109
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 9997
   If_Result := Fld_9;
else
   If_Result := Var_Expr;
end if;
--# expr-done 9998
--# property-call-start Name.parent_name_equation
Fld_12 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => If_Result, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9999
And_Pred_2 := Create_And (And_Pred_1, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16105") else null)); 
--# expr-done 10000
Scope_Result_1 := And_Pred_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3178;
   Result_Var_1 := Scope_Result_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16113") else null)); 
   Result_Var_1 := False_Rel;
end if;
      Inc_Ref (Result_Var_1);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3177;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10001
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16102") else null)); 
--# expr-done 10002
And_Pred_3 := Create_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16099") else null)); 
--# expr-done 10003
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 10004
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1132;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1132;
                     Finalizer_Scope_3177;
                     Finalizer_Scope_3178;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_General_Xref_Equation;
--# end
--# property-start ExplicitDeref.is_constant /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16118
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Is_Constant
  (Node : Bare_Explicit_Deref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Type_Access_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Bare_Explicit_Deref;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Result_Var : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Type_Access_Def;
Fld_3 : Internal_Entity_Constant_Node;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10012 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10011 '<bind at ast.py:16122>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16122
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 10008 '<Cast to TypeAccessDef at ast.py:16125>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16125
--# expr-start 10006 '<FieldAccess for expression_type at ast.py:16124>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16124
--# expr-start 10005 '<FieldAccess for prefix at ast.py:16124>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16124
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10005
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10006
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 10007 '<FieldAccess for access_def at ast.py:16124>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16124
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10007
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Access_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Type_Access_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Access_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Type_Access_Def;
   end if;
--# expr-done 10008
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Type_Access_Def then
   --# scope-start
--# expr-start 10010 '<FieldAccess for as_bool at ast.py:16125>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16125
--# expr-start 10009 '<FieldAccess for has_constant at ast.py:16125>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16125
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Constant_Node (Node => Var_Expr.Node.Type_Access_Def_F_Has_Constant, Info => Var_Expr.Info);
--# expr-done 10009
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_4 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_3.Node);
--# end
--# expr-done 10010
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Dyn_Var_Bind_Result := Result_Var_1; 
--# expr-done 10011
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 10012
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Is_Constant;
--# end
   --
   --  Primitives for Bare_Qual_Expr
   --
      procedure Initialize_Fields_For_Qual_Expr
        (Self : Bare_Qual_Expr
         ; Qual_Expr_F_Prefix : Bare_Name
         ; Qual_Expr_F_Suffix : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Qual_Expr_F_Prefix := Qual_Expr_F_Prefix;
            Self.Qual_Expr_F_Suffix := Qual_Expr_F_Suffix;
      end Initialize_Fields_For_Qual_Expr;
   function Qual_Expr_F_Prefix
     (Node : Bare_Qual_Expr) return Bare_Name
   is
   begin
         return Node.Qual_Expr_F_Prefix;
   end;
   function Qual_Expr_F_Suffix
     (Node : Bare_Qual_Expr) return Bare_Expr
   is
   begin
         return Node.Qual_Expr_F_Suffix;
   end;
--# property-start QualExpr.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19022
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Ref_Var
  (Node : Bare_Qual_Expr
  )
   return Logic_Var
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10014 '<FieldAccess for ref_var at ast.py:19022>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19022
--# expr-start 10013 '<FieldAccess for prefix at ast.py:19022>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19022
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Qual_Expr_F_Prefix;
--# expr-done 10013
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 10014
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Ref_Var;
--# end
--# property-start QualExpr.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19024
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Relative_Name
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10016 '<FieldAccess for relative_name at ast.py:19024>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19024
--# expr-start 10015 '<FieldAccess for prefix at ast.py:19024>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19024
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10015
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10016
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Relative_Name;
--# end
--# property-start QualExpr.is_constant /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19026
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Is_Constant
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Is_Constant;
--# end
--# property-start QualExpr.general_xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19028
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_General_Xref_Equation
  (Node : Bare_Qual_Expr
      ; Root : Bare_Name
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1328 with Inline_Always;
            procedure Finalizer_Scope_3182 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Fld : Logic_Equation;
Fld_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1328 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1328;
            procedure Finalizer_Scope_3182 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_3182;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10025 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10024 '<And at ast.py:19030>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19030
--# expr-start 10019 '<And at ast.py:19030>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19030
--# expr-start 10017 '<FieldAccess for xref_equation at ast.py:19031>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19031
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10017
--# expr-start 10018 '<FieldAccess for all_args_xref_equation at ast.py:19032>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19032
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_1 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10018
And_Pred := Create_And (Fld, Fld_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19030") else null)); 
--# expr-done 10019
--# expr-start 10020 '<FieldAccess for parent_name at ast.py:19033>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19033
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_2 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 10020
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 10023 '<FieldAccess for parent_name_equation at ast.py:19035>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19035
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10022 '<FieldAccess for designated_type_impl at ast.py:19036>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19036
--# expr-start 10021 '<FieldAccess for prefix at ast.py:19036>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19036
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10021
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 10022
--# property-call-start Name.parent_name_equation
Fld_5 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_4, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 10023
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3182;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19039") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Create_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19030") else null)); 
--# expr-done 10024
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10025
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1328;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1328;
                     Finalizer_Scope_3182;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_General_Xref_Equation;
--# end
--# property-start QualExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19043
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Xref_Equation
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1329 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Name;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1329 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1329;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10044 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10027 '<FieldAccess for designated_type_impl at ast.py:19045>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19045
--# expr-start 10026 '<FieldAccess for prefix at ast.py:19045>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19045
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10026
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 10027
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 10043 '<And at ast.py:19048>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19048
--# expr-start 10040 '<And at ast.py:19048>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19048
--# expr-start 10037 '<And at ast.py:19048>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19048
--# expr-start 10033 '<And at ast.py:19048>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19048
--# expr-start 10029 '<FieldAccess for sub_equation at ast.py:19048>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19048
--# expr-start 10028 '<FieldAccess for suffix at ast.py:19048>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19048
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Qual_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 10028
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 10029
--# expr-start 10032 '<Bind at ast.py:19049>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19049
--# expr-start 10031 '<FieldAccess for ref_var at ast.py:19049>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19049
--# expr-start 10030 '<FieldAccess for prefix at ast.py:19049>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19049
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Qual_Expr_F_Prefix;
--# expr-done 10030
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_4);
--# end
--# expr-done 10031
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19049") else null)); 
--# expr-done 10032
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19048") else null)); 
--# expr-done 10033
--# expr-start 10036 '<Bind at ast.py:19050>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19050
--# expr-start 10035 '<FieldAccess for expected_type_var at ast.py:19050>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19050
--# expr-start 10034 '<FieldAccess for suffix at ast.py:19050>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19050
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Qual_Expr_F_Suffix;
--# expr-done 10034
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_6);
--# expr-done 10035
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_7, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19050") else null)); 
--# expr-done 10036
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19048") else null)); 
--# expr-done 10037
--# expr-start 10039 '<FieldAccess for matches_expected_type at ast.py:19051>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19051
--# expr-start 10038 '<FieldAccess for suffix at ast.py:19051>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19051
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Ent.Node.Qual_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 10038
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 10039
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19048") else null)); 
--# expr-done 10040
--# expr-start 10042 '<Bind at ast.py:19052>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19052
--# expr-start 10041 '<FieldAccess for type_var at ast.py:19052>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19052
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10041
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_10, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19052") else null)); 
--# expr-done 10042
And_Pred_3 := Create_And (And_Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19048") else null)); 
--# expr-done 10043
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 10044
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1329;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1329;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Xref_Equation;
--# end
--# property-start QualExpr.designated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19058
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Designated_Type
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1330 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1330 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1330;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10050 '<bind at ast.py:19059>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19059
--# expr-start 10045 '<FieldAccess for node_env at ast.py:19059>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19059
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 10045
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 10049 '<bind at ast.py:19060>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19060
--# expr-start 10046 '<FieldAccess for origin_node at ast.py:19060>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19060
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10046
Bound_Origin := Fld_1; 
--# expr-start 10048 '<FieldAccess for designated_type_impl at ast.py:19061>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19061
--# expr-start 10047 '<FieldAccess for prefix at ast.py:19061>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19061
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10047
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 10048
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 10049
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 10050
         Property_Result := Dyn_Var_Bind_Result_1;
   --# end
      Finalizer_Scope_1330;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1330;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Designated_Type;
--# end
--# property-start QualExpr.designated_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19064
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Designated_Env
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1331 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1331 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1331;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10053 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10052 '<FieldAccess for defining_env at ast.py:19066>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19066
--# expr-start 10051 '<FieldAccess for designated_type at ast.py:19066>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19066
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start QualExpr.designated_type
Fld := Libadalang.Implementation.Qual_Expr_P_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10051
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 10052
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10053
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1331;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1331;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Designated_Env;
--# end
--# property-start QualExpr.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19068
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Env_Elements_Impl
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1332 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1332 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1332;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10056 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10055 '<FieldAccess for env_elements_impl at ast.py:19070>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19070
--# expr-start 10054 '<FieldAccess for prefix at ast.py:19070>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19070
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10054
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 10055
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10056
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1332;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1332;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Env_Elements_Impl;
--# end
   --
   --  Primitives for Bare_Reduce_Attribute_Ref
   --
      procedure Initialize_Fields_For_Reduce_Attribute_Ref
        (Self : Bare_Reduce_Attribute_Ref
         ; Reduce_Attribute_Ref_F_Prefix : Bare_Ada_Node
         ; Reduce_Attribute_Ref_F_Attribute : Bare_Identifier
         ; Reduce_Attribute_Ref_F_Args : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Reduce_Attribute_Ref_F_Prefix := Reduce_Attribute_Ref_F_Prefix;
            Self.Reduce_Attribute_Ref_F_Attribute := Reduce_Attribute_Ref_F_Attribute;
            Self.Reduce_Attribute_Ref_F_Args := Reduce_Attribute_Ref_F_Args;
      Self.Reduce_Attribute_Ref_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Reduce_Attribute_Ref;
   function Reduce_Attribute_Ref_F_Prefix
     (Node : Bare_Reduce_Attribute_Ref) return Bare_Ada_Node
   is
   begin
         return Node.Reduce_Attribute_Ref_F_Prefix;
   end;
   function Reduce_Attribute_Ref_F_Attribute
     (Node : Bare_Reduce_Attribute_Ref) return Bare_Identifier
   is
   begin
         return Node.Reduce_Attribute_Ref_F_Attribute;
   end;
   function Reduce_Attribute_Ref_F_Args
     (Node : Bare_Reduce_Attribute_Ref) return Bare_Assoc_List
   is
   begin
         return Node.Reduce_Attribute_Ref_F_Args;
   end;
--# property-start ReduceAttributeRef.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19783
pragma Warnings (Off, "is not referenced");
 function Reduce_Attribute_Ref_P_Ref_Var
  (Node : Bare_Reduce_Attribute_Ref
  )
   return Logic_Var
is
   Self : Bare_Reduce_Attribute_Ref  := Bare_Reduce_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10057 '<FieldAccess for r_ref_var at ast.py:19783>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19783
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Reduce_Attribute_Ref_R_Ref_Var'Unrestricted_Access;
--# expr-done 10057
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Reduce_Attribute_Ref_P_Ref_Var;
--# end
--# property-start ReduceAttributeRef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19786
pragma Warnings (Off, "is not referenced");
 function Reduce_Attribute_Ref_P_Xref_Equation
  (Node : Bare_Reduce_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Reduce_Attribute_Ref  := Bare_Reduce_Attribute_Ref (Node);
     Ent : Internal_Entity_Reduce_Attribute_Ref :=
       Internal_Entity_Reduce_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1370 with Inline_Always;
            procedure Finalizer_Scope_3183 with Inline_Always;
            procedure Finalizer_Scope_3185 with Inline_Always;
      Block_Var : Internal_Entity_Expr;
Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Base_Id;
Fld_4 : Symbol_Type;
Fld_5 : Bare_Ada_Node;
Fld_6 : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_8 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1370 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1370;
            procedure Finalizer_Scope_3183 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_3183;
            procedure Finalizer_Scope_3185 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3185;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10070 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10059 '<FieldAccess for expr at ast.py:19796>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19796
--# expr-start 10058 '<FieldAccess for args at ast.py:19796>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19796
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Reduce_Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 10058
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 10059
Block_Var := Fld_3; 
--# bind reducer Block_Var
--# expr-start 10069 '<LogicAny at ast.py:19802>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19802
--# expr-start 10068 '<Map at ast.py:19802>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19802
   --# expr-start 10063 '<FieldAccess for env_get at ast.py:19798>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19798
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10061 '<FieldAccess for sym at ast.py:19800>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19800
--# expr-start 10060 '<Cast to BaseId at ast.py:19800>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19800
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Id
   then
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Id;
   end if;
--# expr-done 10060
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_4 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Cast_Result.Node);
--# end
--# expr-done 10061
--# expr-start 10062 '<FieldAccess for origin_node at ast.py:19801>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19801
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10062
--# property-call-start AdaNode.env_get
Fld_6 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld_4, From_Node => Fld_5);
--# end
--# expr-done 10063
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_6;
      begin
         for Item_312 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_312
--# expr-start 10064 '<Cast to BasicDecl at ast.py:19804>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19804
Cast_Expr_1 := Item_312; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10064
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 10065 '<FieldAccess for is_valid_reducer_candidate at ast.py:19804>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19804
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_valid_reducer_candidate
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Is_Valid_Reducer_Candidate (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10065
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Var_Expr := Result_Var;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 10067 '<FieldAccess for xref_equation_for_reducer_candidate at ast.py:19806>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19806
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10066 '<Cast to BasicDecl at ast.py:19807>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19807
Cast_Expr_2 := Item_312; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10066
--# property-call-start ReduceAttributeRef.xref_equation_for_reducer_candidate
Fld_8 := Libadalang.Implementation.Reduce_Attribute_Ref_P_Xref_Equation_For_Reducer_Candidate (Node => Ent.Node, Subp => Cast_Result_2, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10067
Scope_Result_1 := Fld_8;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3185;
   Result_Var_1 := Scope_Result_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19808") else null)); 
   Result_Var_1 := False_Rel;
end if;
      Inc_Ref (Result_Var_1);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3183;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10068
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19802") else null)); 
--# expr-done 10069
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 10070
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1370;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1370;
                     Finalizer_Scope_3183;
                     Finalizer_Scope_3185;
            raise;
      end;
   return Property_Result;
end Reduce_Attribute_Ref_P_Xref_Equation;
--# end
--# property-start ReduceAttributeRef.xref_equation_for_reducer_candidate /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19812
pragma Warnings (Off, "is not referenced");
 function Reduce_Attribute_Ref_P_Xref_Equation_For_Reducer_Candidate
  (Node : Bare_Reduce_Attribute_Ref
      ; Subp : Internal_Entity_Basic_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Reduce_Attribute_Ref  := Bare_Reduce_Attribute_Ref (Node);
     Ent : Internal_Entity_Reduce_Attribute_Ref :=
       Internal_Entity_Reduce_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp Subp
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1371 with Inline_Always;
            procedure Finalizer_Scope_3186 with Inline_Always;
      Block_Var : Internal_Entity_Base_Id;
Block_Var_1 : Internal_Entity_Expr;
Block_Var_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Block_Var_4 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Value_Sequence;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Base_Id;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Fld_9 : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result_2 : Internal_Entity_Base_Type_Decl;
Get_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Value_Sequence;
Fld_11 : Internal_Entity_Iterated_Assoc;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_14 : Internal_Entity_Iterated_Assoc;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_17 : Internal_Entity;
Fld_18 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_20 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_21 : Logic_Var;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_22 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_23 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1371 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1371;
            procedure Finalizer_Scope_3186 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3186;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10106 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10073 '<Cast to BaseId at ast.py:19817>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19817
--# expr-start 10072 '<FieldAccess for expr at ast.py:19817>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19817
--# expr-start 10071 '<FieldAccess for args at ast.py:19817>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19817
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Reduce_Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 10071
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 10072
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Id
   then
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Base_Id;
   end if;
--# expr-done 10073
Block_Var := Cast_Result; 
--# bind reducer Block_Var
--# expr-start 10075 '<FieldAccess for expr at ast.py:19818>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19818
--# expr-start 10074 '<FieldAccess for args at ast.py:19818>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19818
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Reduce_Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 10074
Fld_5 := Fld_4.Node;
Get_Result_1 := Get (Self, Fld_5, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 10075
Block_Var_1 := Fld_7; 
--# bind initial_value_expression Block_Var_1
--# expr-start 10077 '<FieldAccess for param_types at ast.py:19823>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19823
--# expr-start 10076 '<FieldAccess for subp_spec_or_null at ast.py:19823>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19823
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Subp.Node, E_Info => Subp.Info);
--# end
--# expr-done 10076
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Fld_8.Node, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 10077
Block_Var_2 := Fld_9; Inc_Ref (Block_Var_2);
--# bind param_types Block_Var_2
--# expr-start 10078 '<at at ast.py:19824>' Get_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19824
Get_Result_2 := Get (Self, Block_Var_2, 0, True); 
--# expr-done 10078
Block_Var_3 := Get_Result_2; 
--# bind accum_type Block_Var_3
--# expr-start 10079 '<at at ast.py:19825>' Get_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19825
Get_Result_3 := Get (Self, Block_Var_2, 1, True); 
--# expr-done 10079
Block_Var_4 := Get_Result_3; 
--# bind value_type Block_Var_4
--# expr-start 10105 '<And at ast.py:19827>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19827
--# expr-start 10103 '<And at ast.py:19827>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19827
--# expr-start 10101 '<And at ast.py:19827>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19827
--# expr-start 10098 '<And at ast.py:19827>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19827
--# expr-start 10095 '<And at ast.py:19827>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19827
--# expr-start 10092 '<And at ast.py:19827>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19827
--# expr-start 10081 '<Cast to ValueSequence at ast.py:19828>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19828
--# expr-start 10080 '<FieldAccess for prefix at ast.py:19828>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19828
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity (Node => Ent.Node.Reduce_Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 10080
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Value_Sequence_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Value_Sequence
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_1 := No_Entity_Value_Sequence;
   end if;
--# expr-done 10081
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Value_Sequence then
   --# scope-start
   --# bind vs Var_Expr
--# expr-start 10089 '<And at ast.py:19830>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19830
--# expr-start 10085 '<Bind at ast.py:19830>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19830
--# expr-start 10084 '<FieldAccess for expected_type_var at ast.py:19830>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19830
--# expr-start 10083 '<FieldAccess for expr at ast.py:19830>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19830
--# expr-start 10082 '<FieldAccess for iter_assoc at ast.py:19830>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19830
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Iterated_Assoc (Node => Var_Expr.Node.Value_Sequence_F_Iter_Assoc, Info => Var_Expr.Info);
--# expr-done 10082
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 10083
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_12.Node);
--# expr-done 10084
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
Cast_Expr := Block_Var_4; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_13, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19830") else null)); 
--# expr-done 10085
--# expr-start 10088 '<FieldAccess for matches_expected_type at ast.py:19831>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19831
--# expr-start 10087 '<FieldAccess for expr at ast.py:19831>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19831
--# expr-start 10086 '<FieldAccess for iter_assoc at ast.py:19831>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19831
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Iterated_Assoc (Node => Var_Expr.Node.Value_Sequence_F_Iter_Assoc, Info => Var_Expr.Info);
--# expr-done 10086
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 10087
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_16 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_15.Node, Origin => Origin);
--# end
--# expr-done 10088
And_Pred := Create_And (Bind_Result, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19830") else null)); 
--# expr-done 10089
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3186;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19832") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 10091 '<FieldAccess for sub_equation at ast.py:19834>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19834
--# expr-start 10090 '<FieldAccess for prefix at ast.py:19834>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19834
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity (Node => Ent.Node.Reduce_Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 10090
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_17.Node, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 10091
And_Pred_1 := Create_And (Result_Var, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19827") else null)); 
--# expr-done 10092
--# expr-start 10094 '<Bind at ast.py:19836>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19836
--# expr-start 10093 '<FieldAccess for type_var at ast.py:19836>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19836
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10093
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
Cast_Expr_1 := Block_Var_3; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_19, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19836") else null)); 
--# expr-done 10094
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19827") else null)); 
--# expr-done 10095
--# expr-start 10097 '<Bind at ast.py:19838>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19838
--# expr-start 10096 '<FieldAccess for ref_var at ast.py:19838>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19838
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Block_Var.Node);
--# end
--# expr-done 10096
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
Cast_Expr_2 := Subp; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_20, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19838") else null)); 
--# expr-done 10097
And_Pred_3 := Create_And (And_Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19827") else null)); 
--# expr-done 10098
--# expr-start 10100 '<Bind at ast.py:19840>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19840
--# expr-start 10099 '<FieldAccess for expected_type_var at ast.py:19840>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19840
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Block_Var_1.Node);
--# expr-done 10099
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
Cast_Expr_3 := Block_Var_3; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_21, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19840") else null)); 
--# expr-done 10100
And_Pred_4 := Create_And (And_Pred_3, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19827") else null)); 
--# expr-done 10101
--# expr-start 10102 '<FieldAccess for sub_equation at ast.py:19841>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19841
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 10102
And_Pred_5 := Create_And (And_Pred_4, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19827") else null)); 
--# expr-done 10103
--# expr-start 10104 '<FieldAccess for matches_expected_formal_type at ast.py:19842>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19842
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_23 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Block_Var_1.Node, Origin => Origin);
--# end
--# expr-done 10104
And_Pred_6 := Create_And (And_Pred_5, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19827") else null)); 
--# expr-done 10105
Let_Result := And_Pred_6; Inc_Ref (Let_Result);
--# end
--# expr-done 10106
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1371;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1371;
                     Finalizer_Scope_3186;
            raise;
      end;
   return Property_Result;
end Reduce_Attribute_Ref_P_Xref_Equation_For_Reducer_Candidate;
--# end
   --
   --  Primitives for Bare_Single_Tok_Node
   --
      procedure Initialize_Fields_For_Single_Tok_Node
        (Self : Bare_Single_Tok_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      end Initialize_Fields_For_Single_Tok_Node;
--# property-start SingleTokNode.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16343
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Relative_Name
  (Node : Bare_Single_Tok_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
     Ent : Internal_Entity_Single_Tok_Node :=
       Internal_Entity_Single_Tok_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Cast_Expr : Internal_Entity_Single_Tok_Node;
Cast_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Relative_Name;
--# end
--# property-start SingleTokNode.defines_subp_spec_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16355
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Defines_Subp_Spec_Var
  (Node : Bare_Single_Tok_Node
  )
   return Boolean
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Defines_Subp_Spec_Var;
--# end
--# property-start SingleTokNode.sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16357
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Sym
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Sym : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10107 '<GetSymbol at ast.py:16358>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16358
Sym := Get_Symbol (Self); 
--# expr-done 10107
         Property_Result := Sym;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Sym;
--# end
--# property-start SingleTokNode.name_symbol /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16366
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Name_Symbol
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Sym : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10108 '<GetSymbol at ast.py:16366>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16366
Sym := Get_Symbol (Self); 
--# expr-done 10108
         Property_Result := Sym;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Name_Symbol;
--# end
--# property-start SingleTokNode.canonical_text /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16368
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Canonical_Text
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Let_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10110 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10109 '<FieldAccess for sym at ast.py:16370>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16370
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Self);
--# end
--# expr-done 10109
Let_Result := Fld; 
--# end
--# expr-done 10110
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Canonical_Text;
--# end
--# property-start SingleTokNode.env_get_first_visible /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16372
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Env_Get_First_Visible
  (Node : Bare_Single_Tok_Node
      ; Lex_Env : Lexical_Env
      ; Lookup_Type : Lookup_Kind
      ; From_Node : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
  )
   return Internal_Entity
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   --# bind lex_env Lex_Env
   --# bind lookup_type Lookup_Type
   --# bind from_node From_Node
   --# bind no_visibility No_Visibility
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1152 with Inline_Always;
      Sym : Symbol_Type;
Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
If_Result : Boolean;
Find_Result : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_1152 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1152;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10115 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10114 '<Find at ast.py:16390>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16390
   --# expr-start 10112 '<FieldAccess for env_get at ast.py:16382>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16382
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10111 '<GetSymbol at ast.py:16384>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16384
Sym := Get_Symbol (Self); 
--# expr-done 10111
--# property-call-start AdaNode.env_get
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Lex_Env, Symbol => Sym, Lookup => Lookup_Type, From_Node => From_Node, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 10112
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_313 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_313
if No_Visibility then
   If_Result := True;
else
   --# expr-start 10113 '<FieldAccess for has_visibility at ast.py:16390>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16390
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_313);
--# end
--# expr-done 10113
   If_Result := Fld_1;
end if;
      if If_Result then
         Find_Result := Item_313; 
      end if;
   --# end
               exit when If_Result;
         end loop;
      end;
--# expr-done 10114
Let_Result := Find_Result; 
--# end
--# expr-done 10115
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1152;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1152;
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Env_Get_First_Visible;
--# end
   --
   --  Primitives for Bare_Base_Id
   --
      procedure Initialize_Fields_For_Base_Id
        (Self : Bare_Base_Id
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Base_Id;
--# property-start BaseId.scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17161
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Scope
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1204 with Inline_Always;
            procedure Finalizer_Scope_3187 with Inline_Always;
      Block_Var : Internal_Entity;
Block_Var_1 : Lexical_Env := Empty_Env;
Var_Expr : Bare_Generic_Package_Decl;
Sym : Symbol_Type;
Fld : Boolean;
If_Result : Lookup_Kind;
Env_Get_Result : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Package_Decl;
Fld_3 : Bare_Generic_Package_Internal;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1204 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1204;
            procedure Finalizer_Scope_3187 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3187;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Id_Base_Id_P_Scope,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Id,
                                As_Bare_Base_Id => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 10129 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10118 '<EnvGet at ast.py:17163>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17163
Sym := Get_Symbol (Self); 
--# expr-start 10117 '<If at ast.py:17165>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17165
--# expr-start 10116 '<FieldAccess for is_prefix at ast.py:17165>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17165
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10116
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 10117
Env_Get_Result := AST_Envs.Get_First (Self => Env, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (If_Result), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 10118
Block_Var := Env_Get_Result; 
--# bind elt Block_Var
--# expr-start 10124 '<If at ast.py:17168>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17168
--# expr-start 10120 '<Not at ast.py:17169>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17169
--# expr-start 10119 '<IsNull at ast.py:17169>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17169
Is_Null := Block_Var.Node = null; 
--# expr-done 10119
Not_Val := not (Is_Null); 
--# expr-done 10120
if Not_Val then
   --# expr-start 10122 '<IsA BasicDecl at ast.py:17169>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17169
--# expr-start 10121 '<FieldAccess for node at ast.py:17169>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17169
Fld_2 := Block_Var.Node;
--# expr-done 10121
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Basic_Decl; 
--# expr-done 10122
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 10123 '<FieldAccess for children_env at ast.py:17172>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17172
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 10123
   If_Result_2 := Fld_1;
else
   If_Result_2 := Empty_Env;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 10124
Block_Var_1 := If_Result_2; Inc_Ref (Block_Var_1);
--# bind ret Block_Var_1
--# expr-start 10126 '<Cast to GenericPackageDecl at ast.py:17178>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17178
--# expr-start 10125 '<env_node at ast.py:17178>' Env_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17178
Env_Node := AST_Envs.Env_Node (Block_Var_1); 
--# expr-done 10125
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Env_Node;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 10126
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr
--# expr-start 10128 '<FieldAccess for children_env at ast.py:17179>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17179
--# expr-start 10127 '<FieldAccess for package_decl at ast.py:17179>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17179
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr.Generic_Package_Decl_F_Package_Decl;
--# expr-done 10127
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Fld_3);
--# expr-done 10128
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3187;
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 10129
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1204;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1204;
                     Finalizer_Scope_3187;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Id_P_Scope;
--# end
--# property-start BaseId.designated_env_no_overloading /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17183
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Env_No_Overloading
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1205 with Inline_Always;
            procedure Finalizer_Scope_3188 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
If_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1205 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1205;
            procedure Finalizer_Scope_3188 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_3188;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10141 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10134 '<Cast to BasicDecl at ast.py:17189>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17189
--# expr-start 10133 '<FieldAccess for env_get_first_visible at ast.py:17185>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17185
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10131 '<If at ast.py:17187>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17187
--# expr-start 10130 '<FieldAccess for is_prefix at ast.py:17187>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17187
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10130
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 10131
--# expr-start 10132 '<FieldAccess for origin_node at ast.py:17188>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17188
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10132
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Fld_1, No_Visibility => No_Visibility);
--# end
--# expr-done 10133
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 10134
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr
--# expr-start 10140 '<Cond at ast.py:17189>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17189
--# expr-start 10135 '<IsA ObjectDecl at ast.py:17196>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17196
Is_A := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 10135
if Is_A then
   --# expr-start 10136 '<FieldAccess for empty_env at ast.py:17197>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17197
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 10136
   If_Result_2 := Fld_3;
else
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 10137 '<FieldAccess for is_package at ast.py:17199>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17199
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_package
Fld_4 := Libadalang.Implementation.Ada_Node_P_Is_Package (Node => Var_Expr_1.Node);
--# end
--# expr-done 10137
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 10138 '<FieldAccess for pkg_env at ast.py:17200>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17200
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.pkg_env
Fld_5 := Libadalang.Implementation.Base_Id_P_Pkg_Env (Node => Ent.Node, From_Pkg => Var_Expr, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10138
   If_Result_1 := Fld_5;
else
   --# expr-start 10139 '<FieldAccess for defining_env at ast.py:17202>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17202
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 10139
   If_Result_1 := Fld_6;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 10140
Scope_Result_1 := If_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3188;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10141
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1205;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1205;
                     Finalizer_Scope_3188;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Designated_Env_No_Overloading;
--# end
--# property-start BaseId.designated_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17205
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Env
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1206 with Inline_Always;
            procedure Finalizer_Scope_3198 with Inline_Always;
            procedure Finalizer_Scope_3200 with Inline_Always;
      Block_Var : Bare_Ada_Node;
Block_Var_1 : Internal_Entity;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Array_Access;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Array_Access;
Fld_5 : Bare_Ada_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Fld_6 : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_7 : Lexical_Env := Empty_Env;
Map_Result_1 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Cast_Expr_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Cast_Expr_3 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Fld_9 : Bare_Basic_Decl;
Cast_Result_5 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1206 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Group_Env);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1206;
            procedure Finalizer_Scope_3198 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3198;
            procedure Finalizer_Scope_3200 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_3200;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10168 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10144 '<Find at ast.py:17211>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17211
   --# expr-start 10142 '<FieldAccess for parents at ast.py:17211>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17211
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 10142
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_314 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_314
      --# expr-start 10143 '<IsA GenericPackageInstantiation at ast.py:17212>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17212
Is_A := Item_314 /= null 
and then Item_314.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 10143
      if Is_A then
         Find_Result := Item_314; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 10144
Block_Var := Find_Result; 
--# bind bd Block_Var
--# expr-start 10148 '<FieldAccess for env_get_first_visible at ast.py:17215>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17215
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10146 '<If at ast.py:17217>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17217
--# expr-start 10145 '<FieldAccess for is_prefix at ast.py:17217>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17217
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10145
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 10146
--# expr-start 10147 '<FieldAccess for origin_node at ast.py:17218>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17218
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10147
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Fld_1, No_Visibility => No_Visibility);
--# end
--# expr-done 10148
Block_Var_1 := Fld_2; 
--# expr-start 10167 '<If at ast.py:17221>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17221
--# expr-start 10149 '<Cast to BasicDecl at ast.py:17219>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17219
Cast_Expr_2 := Block_Var_1; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10149
Var_Expr := Cast_Result_3;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 10150 '<FieldAccess for is_package at ast.py:17223>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17223
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_package
Fld_8 := Libadalang.Implementation.Ada_Node_P_Is_Package (Node => Var_Expr.Node);
--# end
--# expr-done 10150
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   --# expr-start 10154 '<Not at ast.py:17223>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17223
--# expr-start 10153 '<Eq at ast.py:17223>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17223
--# expr-start 10152 '<FieldAccess for node at ast.py:17223>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17223
--# expr-start 10151 '<Cast to BasicDecl at ast.py:17219>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17219
Cast_Expr_3 := Block_Var_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_4 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10151
Fld_9 := Cast_Result_4.Node;
--# expr-done 10152
      Cast_Result_5 := Fld_9;
Is_Equal_1 := Cast_Result_5 = Block_Var; 
--# expr-done 10153
Not_Val_1 := not (Is_Equal_1); 
--# expr-done 10154
   If_Result_2 := Not_Val_1;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 10156 '<FieldAccess for pkg_env at ast.py:17224>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17224
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10155 '<Cast to BasicDecl at ast.py:17219>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17219
Cast_Expr := Block_Var_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 10155
--# property-call-start BaseId.pkg_env
Fld_3 := Libadalang.Implementation.Base_Id_P_Pkg_Env (Node => Ent.Node, From_Pkg => Cast_Result, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10156
   If_Result_3 := Fld_3;
else
   --# expr-start 10166 '<env_group at ast.py:17234>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17234
--# expr-start 10165 '<Map at ast.py:17234>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17234
--# expr-start 10157 '<FieldAccess for env_elements_baseid at ast.py:17226>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17226
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld_4 := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 10157
Var_Expr_1 := Fld_4;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind all_env_els Var_Expr_1
--# expr-start 10162 '<Filter at ast.py:17228>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17228
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_1;
      begin
         for Item_316 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_316
--# expr-start 10160 '<Not at ast.py:17230>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17230
--# expr-start 10159 '<Eq at ast.py:17230>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17230
--# expr-start 10158 '<FieldAccess for node at ast.py:17230>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17230
Fld_5 := Item_316.Node;
--# expr-done 10158
      Cast_Result_1 := Fld_5;
Is_Equal := Cast_Result_1 = Block_Var; 
--# expr-done 10159
Not_Val := not (Is_Equal); 
--# expr-done 10160
if Not_Val then
   --# expr-start 10161 '<FieldAccess for has_visibility at ast.py:17232>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17232
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_316);
--# end
--# expr-done 10161
   If_Result_1 := Fld_6;
else
   If_Result_1 := False;
end if;
         if If_Result_1 then
         declare
            Item_To_Append : constant Internal_Entity := Item_316;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10162
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3198;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
      declare
         Map_Result_1_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_315 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_315
      --# expr-start 10164 '<FieldAccess for defining_env at ast.py:17234>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17234
--# expr-start 10163 '<Cast to BasicDecl at ast.py:17234>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17234
Cast_Expr_1 := Item_315; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10163
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Cast_Result_2.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 10164
         declare
            Item_To_Append : constant Lexical_Env := Fld_7;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3200;
         end loop;
      end;
         Map_Result_1 := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_1_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10165
Group_Env := Group (Map_Result_1, No_Metadata); 
--# expr-done 10166
   If_Result_3 := Group_Env;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 10167
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 10168
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1206;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1206;
                     Finalizer_Scope_3198;
                     Finalizer_Scope_3200;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Designated_Env;
--# end
--# property-start BaseId.pkg_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17237
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Pkg_Env
  (Node : Bare_Base_Id
      ; From_Pkg : Internal_Entity_Basic_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind from_pkg From_Pkg
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1207 with Inline_Always;
            procedure Finalizer_Scope_3191 with Inline_Always;
            procedure Finalizer_Scope_3192 with Inline_Always;
            procedure Finalizer_Scope_3193 with Inline_Always;
            procedure Finalizer_Scope_3194 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Internal_Entity_Basic_Decl;
Block_Var_2 : Boolean;
Block_Var_3 : Lexical_Env := Empty_Env;
Block_Var_4 : Lexical_Env := Empty_Env;
Block_Var_5 : Lexical_Env := Empty_Env;
Block_Var_6 : Lexical_Env := Empty_Env;
Var_Expr : Internal_Entity_Package_Renaming_Decl;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity;
Var_Expr_4 : Internal_Entity_Generic_Package_Decl;
Var_Expr_5 : Bare_Ada_Node;
Var_Expr_6 : Bare_Ada_Node;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Package_Renaming_Decl;
Fld : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_4 : Internal_Entity_Info;
Fld_5 : Boolean;
If_Result : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Package_Instantiation;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Lexical_Env := Empty_Env;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_8 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_9 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result_2 : Internal_Entity_Array_Access;
Get_Result_2 : Internal_Entity;
Fld_10 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Fld_11 : Lexical_Env := Empty_Env;
Is_A_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var_3 : Lexical_Env := Empty_Env;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Package_Decl;
Fld_12 : Internal_Entity_Generic_Formal_Part;
Fld_13 : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Result_Var_4 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Not_Val : Boolean;
Scope_Result_5 : Bare_Ada_Node;
Cast_Expr_3 : Bare_Base_Id;
Cast_Result_3 : Bare_Ada_Node;
Result_Var_5 : Bare_Ada_Node;
Fld_14 : Lexical_Env := Empty_Env;
Fld_15 : Boolean;
If_Result_4 : Boolean;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result_6 : Bare_Ada_Node;
Cast_Expr_4 : Bare_Base_Id;
Cast_Result_4 : Bare_Ada_Node;
Result_Var_6 : Bare_Ada_Node;
Fld_16 : Lexical_Env := Empty_Env;
Fld_17 : Boolean;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Is_A_3 : Boolean;
Array_Lit_2 : Lexical_Env_Array_Access;
Group_Env_2 : Lexical_Env := Empty_Env;
If_Result_5 : Lexical_Env := Empty_Env;
If_Result_6 : Lexical_Env := Empty_Env;
If_Result_7 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1207 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Array_Lit_2);
                     Dec_Ref (Group_Env_2);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1207;
            procedure Finalizer_Scope_3191 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3191;
            procedure Finalizer_Scope_3192 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Env_Get_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_3192;
            procedure Finalizer_Scope_3193 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_3193;
            procedure Finalizer_Scope_3194 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_3194;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10214 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10169 '<Cast to PackageRenamingDecl at ast.py:17253>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17253
Cast_Expr := From_Pkg; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 10169
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Package_Renaming_Decl then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 10170 '<FieldAccess for final_renamed_package at ast.py:17254>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17254
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PackageRenamingDecl.final_renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Final_Renamed_Package (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 10170
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := From_Pkg;
end if;
Block_Var := Result_Var; 
--# bind pkg Block_Var
--# expr-start 10173 '<FieldAccess for unshed_rebindings at ast.py:17265>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17265
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10172 '<FieldAccess for rebindings at ast.py:17265>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17265
--# expr-start 10171 '<FieldAccess for info at ast.py:17265>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17265
Fld_1 := Ent.Info;
--# expr-done 10171
Fld_2 := Fld_1.Rebindings;
--# expr-done 10172
--# property-call-start BasicDecl.unshed_rebindings
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Block_Var.Node, Rebindings => Fld_2, E_Info => Block_Var.Info);
--# end
--# expr-done 10173
Block_Var_1 := Fld_3; 
--# bind bd Block_Var_1
--# expr-start 10174 '<IsA GenericPackageInstantiation at ast.py:17267>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17267
Is_A := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 10174
if Is_A then
   --# expr-start 10176 '<FieldAccess for from_rebound at ast.py:17268>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17268
--# expr-start 10175 '<FieldAccess for info at ast.py:17268>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17268
Fld_4 := From_Pkg.Info;
--# expr-done 10175
Fld_5 := Fld_4.From_Rebound;
--# expr-done 10176
   If_Result := Fld_5;
else
   If_Result := False;
end if;
Block_Var_2 := If_Result; 
--# bind is_inst_from_formal Block_Var_2
--# expr-start 10181 '<If at ast.py:17270>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17270
--# expr-start 10177 '<IsA GenericPackageInstantiation at ast.py:17271>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17271
Is_A_1 := Block_Var_1.Node /= null 
and then Block_Var_1.Node.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 10177
if Is_A_1 then
   If_Result_1 := Block_Var_2;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 10179 '<FieldAccess for defining_env_impl at ast.py:17272>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17272
--# expr-start 10178 '<Cast to GenericPackageInstantiation at ast.py:17272>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17272
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 10178
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericPackageInstantiation.defining_env_impl
Fld_6 := Libadalang.Implementation.Generic_Package_Instantiation_P_Defining_Env_Impl (Node => Cast_Result_1.Node, Inst_From_Formal => True, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 10179
   If_Result_2 := Fld_6;
else
   --# expr-start 10180 '<FieldAccess for defining_env at ast.py:17273>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17273
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Block_Var_1.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 10180
   If_Result_2 := Fld_7;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 10181
Block_Var_3 := If_Result_2; Inc_Ref (Block_Var_3);
--# bind env Block_Var_3
--# expr-start 10184 '<at at ast.py:17279>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17279
--# expr-start 10183 '<EnvGet at ast.py:17279>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17279
--# expr-start 10182 '<Symbol __privatepart at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Privatepart); 
--# expr-done 10182
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_3, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 10183
Get_Result := Get (Self, Env_Get_Result, 0, True); 
--# expr-done 10184
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 10185 '<FieldAccess for children_env at ast.py:17280>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17280
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 10185
Scope_Result_1 := Fld_8;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3191;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var_3;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_4 := Result_Var_1; Inc_Ref (Block_Var_4);
--# bind private_part_env Block_Var_4
--# expr-start 10188 '<at at ast.py:17286>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17286
--# expr-start 10187 '<EnvGet at ast.py:17285>' Env_Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17285
--# expr-start 10186 '<Symbol __nextpart at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 10186
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_4, Key => Sym_1, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 10187
Get_Result_1 := Get (Self, Env_Get_Result_1, 0, True); 
--# expr-done 10188
Var_Expr_2 := Get_Result_1;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pb Var_Expr_2
--# expr-start 10196 '<If at ast.py:17287>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17287
--# expr-start 10189 '<IsA PackageBodyStub at ast.py:17290>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17290
Is_A_2 := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Package_Body_Stub_Range; 
--# expr-done 10189
if Is_A_2 then
--# expr-start 10193 '<at at ast.py:17294>' Get_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17294
--# expr-start 10192 '<EnvGet at ast.py:17293>' Env_Get_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17293
--# expr-start 10190 '<FieldAccess for children_env at ast.py:17292>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17292
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 10190
--# expr-start 10191 '<Symbol __nextpart at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 10191
Env_Get_Result_2 := Construct_Entity_Array (AST_Envs.Get (Self => Fld_9, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 10192
Get_Result_2 := Get (Self, Env_Get_Result_2, 0, True); 
--# expr-done 10193
Var_Expr_3 := Get_Result_2;
if Var_Expr_3 /= No_Entity then
   --# scope-start
   --# bind pb Var_Expr_3
--# expr-start 10194 '<FieldAccess for children_env at ast.py:17294>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17294
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# expr-done 10194
Scope_Result_2 := Fld_10;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3193;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_3 := Result_Var_2;
else
   --# expr-start 10195 '<FieldAccess for children_env at ast.py:17296>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17296
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 10195
   If_Result_3 := Fld_11;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 10196
Scope_Result_3 := If_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3192;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Empty_Env;
end if;
      Inc_Ref (Result_Var_3);
Block_Var_5 := Result_Var_3; Inc_Ref (Block_Var_5);
--# bind package_body_env Block_Var_5
--# expr-start 10197 '<Cast to GenericPackageDecl at ast.py:17301>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17301
Cast_Expr_2 := Block_Var_1; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Generic_Package_Decl;
   end if;
--# expr-done 10197
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Generic_Package_Decl then
   --# scope-start
   --# bind pkg_g Var_Expr_4
--# expr-start 10199 '<FieldAccess for children_env at ast.py:17302>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17302
--# expr-start 10198 '<FieldAccess for formal_part at ast.py:17302>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17302
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_4.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_4.Info);
--# expr-done 10198
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Fld_12.Node, E_Info => Fld_12.Info);
--# expr-done 10199
Scope_Result_4 := Fld_13;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3194;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := Empty_Env;
end if;
      Inc_Ref (Result_Var_4);
Block_Var_6 := Result_Var_4; Inc_Ref (Block_Var_6);
--# bind formals_env Block_Var_6
--# expr-start 10213 '<Cond at ast.py:17306>' If_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17306
--# expr-start 10201 '<Not at ast.py:17310>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17310
--# expr-start 10200 '<Eq at ast.py:17310>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17310
Is_Equal := Equivalent (Block_Var_5, Empty_Env); 
--# expr-done 10200
Not_Val := not (Is_Equal); 
--# expr-done 10201
if Not_Val then
   --# expr-start 10203 '<FieldAccess for is_children_env at ast.py:17311>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17311
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10202 '<FieldAccess for node_env at ast.py:17312>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17312
Var_Expr_5 := Origin;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_5
Scope_Result_5 := Var_Expr_5;
   --# end
   Result_Var_5 := Scope_Result_5;
else
Cast_Expr_3 := Self; 
      Cast_Result_3 := Cast_Expr_3;
   Result_Var_5 := Cast_Result_3;
end if;
      if Result_Var_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Node_Env (Node => Result_Var_5);
--# expr-done 10202
--# property-call-start AdaNode.is_children_env
Fld_15 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Block_Var_5, Current_Env => Fld_14);
--# end
--# expr-done 10203
   If_Result_4 := Fld_15;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 10205 '<env_group at ast.py:17315>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17315
--# expr-start 10204 '<ArrayLiteral at ast.py:17313>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17313
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_5, 2 => Block_Var_4, 3 => Block_Var_3, 4 => Block_Var_6)); 
--# expr-done 10204
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 10205
   If_Result_7 := Group_Env;
else
--# expr-start 10207 '<FieldAccess for is_children_env at ast.py:17319>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17319
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10206 '<FieldAccess for node_env at ast.py:17320>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17320
Var_Expr_6 := Origin;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_6
Scope_Result_6 := Var_Expr_6;
   --# end
   Result_Var_6 := Scope_Result_6;
else
Cast_Expr_4 := Self; 
      Cast_Result_4 := Cast_Expr_4;
   Result_Var_6 := Cast_Result_4;
end if;
      if Result_Var_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Node_Env (Node => Result_Var_6);
--# expr-done 10206
--# property-call-start AdaNode.is_children_env
Fld_17 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Block_Var_4, Current_Env => Fld_16);
--# end
--# expr-done 10207
if Fld_17 then
   --# expr-start 10209 '<env_group at ast.py:17321>' Group_Env_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17321
--# expr-start 10208 '<ArrayLiteral at ast.py:17321>' Array_Lit_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17321
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_4, 2 => Block_Var_3, 3 => Block_Var_6)); 
--# expr-done 10208
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 10209
   If_Result_6 := Group_Env_1;
else
--# expr-start 10210 '<IsA GenericPackageDecl at ast.py:17327>' Is_A_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17327
Is_A_3 := Block_Var_1.Node /= null 
and then Block_Var_1.Node.Kind in Ada_Generic_Package_Decl_Range; 
--# expr-done 10210
if Is_A_3 then
   --# expr-start 10212 '<env_group at ast.py:17328>' Group_Env_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17328
--# expr-start 10211 '<ArrayLiteral at ast.py:17328>' Array_Lit_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17328
Array_Lit_2 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_3, 2 => Block_Var_6)); 
--# expr-done 10211
Group_Env_2 := Group (Array_Lit_2, No_Metadata); 
--# expr-done 10212
   If_Result_5 := Group_Env_2;
else
   If_Result_5 := Block_Var_3;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 10213
Let_Result := If_Result_7; Inc_Ref (Let_Result);
--# end
--# expr-done 10214
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1207;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1207;
                     Finalizer_Scope_3191;
                     Finalizer_Scope_3192;
                     Finalizer_Scope_3193;
                     Finalizer_Scope_3194;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Pkg_Env;
--# end
--# property-start BaseId.parent_scope /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17337
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Parent_Scope
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Id_P_Parent_Scope;
--# end
--# property-start BaseId.designated_type_impl_get_real_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17339
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Type_Impl_Get_Real_Type
  (Node : Bare_Base_Id
      ; N : Internal_Entity
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind n N
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Task_Body;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Task_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Task_Body;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10217 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 10216 '<Match at ast.py:17345>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17345
      if N.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := N; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tb Match_1
--# expr-start 10215 '<FieldAccess for task_type at ast.py:17347>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17347
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskBody.task_type
Fld := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 10215
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Protected_Body | Ada_Entry_Index_Spec .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 10216
Let_Result_3 := Match_Result; 
--# end
--# expr-done 10217
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Id_P_Designated_Type_Impl_Get_Real_Type;
--# end
--# property-start BaseId.designated_type_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17351
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Type_Impl
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1210 with Inline_Always;
            procedure Finalizer_Scope_3208 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Bare_Ada_Node;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Internal_Entity_Base_Type_Decl;
Var_Expr_8 : Internal_Entity_Base_Type_Decl;
Var_Expr_9 : Internal_Entity_Generic_Formal_Type_Decl;
Fld : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
If_Result_1 : Lookup_Kind;
Fld_4 : Internal_Entity;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Fld_6 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
If_Result_3 : Internal_Entity_Base_Type_Decl;
If_Result_4 : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_5 : Internal_Entity;
Fld_7 : Lexical_Env := Empty_Env;
Fld_8 : Symbol_Type;
Scope_Result_3 : Symbol_Type;
Sym : Symbol_Type;
Result_Var_3 : Symbol_Type;
Fld_9 : Internal_Entity_Array_Access;
Scope_Result_4 : Internal_Entity_Array_Access;
Result_Var_4 : Internal_Entity_Array_Access;
Cast_Expr_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Cast_Expr_4 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
If_Result_6 : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_6 : Internal_Entity_Base_Type_Decl;
Result_Var_6 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Generic_Formal_Type_Decl;
Fld_12 : Internal_Entity_Base_Type_Decl;
Scope_Result_7 : Internal_Entity_Base_Type_Decl;
Result_Var_7 : Internal_Entity_Base_Type_Decl;
Scope_Result_8 : Internal_Entity_Base_Type_Decl;
Result_Var_8 : Internal_Entity_Base_Type_Decl;
Scope_Result_9 : Internal_Entity_Base_Type_Decl;
Result_Var_9 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1210 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1210;
            procedure Finalizer_Scope_3208 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_3208;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10244 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10220 '<FieldAccess for env_get_first_visible at ast.py:17354>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17354
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10219 '<If at ast.py:17357>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17357
--# expr-start 10218 '<FieldAccess for is_prefix at ast.py:17357>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17357
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10218
if Fld then
   If_Result := Recursive;
else
   If_Result := Minimal;
end if;
--# expr-done 10219
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start SingleTokNode.env_get_first_visible
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Cast_Result, No_Visibility => False);
--# end
--# expr-done 10220
Var_Expr := Fld_1;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind env_el Var_Expr
--# expr-start 10221 '<FieldAccess for designated_type_impl_get_real_type at ast.py:17359>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17359
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.designated_type_impl_get_real_type
Fld_2 := Libadalang.Implementation.Base_Id_P_Designated_Type_Impl_Get_Real_Type (Node => Self, N => Var_Expr);
--# end
--# expr-done 10221
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var := Result_Var; 
--# bind des_type_1 Block_Var
--# expr-start 10224 '<FieldAccess for env_get_first_visible at ast.py:17363>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17363
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10223 '<If at ast.py:17366>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17366
--# expr-start 10222 '<FieldAccess for is_prefix at ast.py:17366>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17366
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_3 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10222
if Fld_3 then
   If_Result_1 := Recursive;
else
   If_Result_1 := Minimal;
end if;
--# expr-done 10223
--# property-call-start SingleTokNode.env_get_first_visible
Fld_4 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result_1, From_Node => Origin, No_Visibility => False);
--# end
--# expr-done 10224
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind env_el Var_Expr_1
--# expr-start 10225 '<FieldAccess for designated_type_impl_get_real_type at ast.py:17368>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17368
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.designated_type_impl_get_real_type
Fld_5 := Libadalang.Implementation.Base_Id_P_Designated_Type_Impl_Get_Real_Type (Node => Self, N => Var_Expr_1);
--# end
--# expr-done 10225
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind des_type_2 Block_Var_1
--# expr-start 10229 '<Cond at ast.py:17371>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17371
--# expr-start 10226 '<IsNull at ast.py:17373>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17373
Is_Null := Block_Var.Node = null; 
--# expr-done 10226
if Is_Null then
   If_Result_4 := Block_Var_1;
else
--# expr-start 10227 '<Eq at ast.py:17377>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17377
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Block_Var_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 10227
if Is_Equiv then
   If_Result_3 := Block_Var;
else
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind d Var_Expr_2
--# expr-start 10228 '<FieldAccess for is_view_of_type at ast.py:17381>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17381
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Var_Expr_2.Node, Comp_View => Block_Var_1, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 10228
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   If_Result_2 := Block_Var_1;
else
   If_Result_2 := Block_Var;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 10229
Block_Var_2 := If_Result_4; 
--# bind des_type Block_Var_2
--# expr-start 10240 '<at at ast.py:17407>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17407
--# expr-start 10239 '<FilterMap at ast.py:17402>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17402
Var_Expr_3 := Origin;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind o Var_Expr_3
--# expr-start 10234 '<FieldAccess for env_get at ast.py:17391>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17391
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10231 '<FieldAccess for children_env at ast.py:17392>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17392
--# expr-start 10230 '<as_entity at ast.py:17392>' If_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17392
Is_Null_1 := Var_Expr_3 = null; 
if Is_Null_1 then
   If_Result_5 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Var_Expr_3); 
   If_Result_5 := As_Entity;
end if;
--# expr-done 10230
      if If_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => If_Result_5.Node, E_Info => If_Result_5.Info);
--# expr-done 10231
Var_Expr_4 := Block_Var_2;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr_4
--# expr-start 10232 '<FieldAccess for name_symbol at ast.py:17400>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17400
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr_4.Node);
--# end
--# expr-done 10232
Scope_Result_3 := Fld_8;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 10233 '<GetSymbol at ast.py:17400>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17400
Sym := Get_Symbol (Self); 
--# expr-done 10233
   Result_Var_3 := Sym;
end if;
--# property-call-start AdaNode.env_get
Fld_9 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld_7, Symbol => Result_Var_3, From_Node => Origin, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 10234
Scope_Result_4 := Fld_9;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3208;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var_4;
      begin
         for Item_317 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_317
--# expr-start 10235 '<IsA BaseTypeDecl at ast.py:17405>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17405
Is_A := Item_317.Node /= null 
and then Item_317.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 10235
if Is_A then
Var_Expr_5 := Block_Var_2;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind d Var_Expr_5
--# expr-start 10237 '<FieldAccess for is_view_of_type at ast.py:17406>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17406
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10236 '<Cast to BaseTypeDecl at ast.py:17406>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17406
Cast_Expr_4 := Item_317; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_4 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 10236
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Var_Expr_5.Node, Comp_View => Cast_Result_4, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 10237
Scope_Result_5 := Fld_10;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
   If_Result_6 := Result_Var_5;
else
   If_Result_6 := False;
end if;
         if If_Result_6 then
      --# expr-start 10238 '<Cast to BaseTypeDecl at ast.py:17403>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17403
Cast_Expr_3 := Item_317; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 10238
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Cast_Result_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10239
Get_Result := Get (Self, Map_Result, 0, True); 
--# expr-done 10240
Block_Var_3 := Get_Result; 
--# bind completer_view Block_Var_3
Var_Expr_7 := Block_Var_3;
if Var_Expr_7 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_7
Scope_Result_6 := Var_Expr_7;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := Block_Var_2;
end if;
Var_Expr_6 := Result_Var_6;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind precise Var_Expr_6
--# expr-start 10242 '<Cast to GenericFormalTypeDecl at ast.py:17416>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17416
--# expr-start 10241 '<parent at ast.py:17416>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17416
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Parent (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# expr-done 10241
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Generic_Formal_Type_Decl_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Cast_Result_5 := No_Entity_Generic_Formal_Type_Decl;
   end if;
--# expr-done 10242
Var_Expr_9 := Cast_Result_5;
if Var_Expr_9 /= No_Entity_Generic_Formal_Type_Decl then
   --# scope-start
--# expr-start 10243 '<FieldAccess for default_type at ast.py:17416>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17416
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericFormalTypeDecl.default_type
Fld_12 := Libadalang.Implementation.Generic_Formal_Type_Decl_P_Default_Type (Node => Var_Expr_9.Node, E_Info => Var_Expr_9.Info);
--# end
--# expr-done 10243
Scope_Result_7 := Fld_12;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := No_Entity_Base_Type_Decl;
end if;
Var_Expr_8 := Result_Var_7;
if Var_Expr_8 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_8
Scope_Result_8 := Var_Expr_8;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := Var_Expr_6;
end if;
Scope_Result_9 := Result_Var_8;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_9; 
--# end
--# expr-done 10244
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1210;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1210;
                     Finalizer_Scope_3208;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Designated_Type_Impl;
--# end
--# property-start BaseId.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17420
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Env_Elements_Impl
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1211 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1211 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1211;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10246 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10245 '<FieldAccess for env_elements_baseid at ast.py:17422>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17422
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 10245
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 10246
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1211;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1211;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Env_Elements_Impl;
--# end
--# property-start BaseId.all_env_els_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17424
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_All_Env_Els_Impl
  (Node : Bare_Base_Id
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1212 with Inline_Always;
      Fld : Symbol_Type;
Fld_1 : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_1 : Bare_Ada_Node;
If_Result_2 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1212 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1212;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10255 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10254 '<FieldAccess for env_get at ast.py:17430>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17430
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10247 '<FieldAccess for name_symbol at ast.py:17432>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17432
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 10247
--# expr-start 10249 '<If at ast.py:17433>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17433
--# expr-start 10248 '<FieldAccess for is_prefix at ast.py:17433>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17433
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_1 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10248
if Fld_1 then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 10249
--# expr-start 10253 '<If at ast.py:17434>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17434
if Seq then
   --# expr-start 10252 '<If at ast.py:17434>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17434
--# expr-start 10251 '<Not at ast.py:17434>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17434
--# expr-start 10250 '<IsNull at ast.py:17434>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17434
Is_Null := Seq_From = null; 
--# expr-done 10250
Not_Val := not (Is_Null); 
--# expr-done 10251
if Not_Val then
   If_Result_1 := Seq_From;
else
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
   If_Result_1 := Cast_Result;
end if;
--# expr-done 10252
   If_Result_2 := If_Result_1;
else
   If_Result_2 := No_Bare_Ada_Node;
end if;
--# expr-done 10253
--# property-call-start AdaNode.env_get
Fld_2 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld, Lookup => If_Result, From_Node => If_Result_2, Categories => Categories);
--# end
--# expr-done 10254
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 10255
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1212;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1212;
            raise;
      end;
   return Property_Result;
end Base_Id_P_All_Env_Els_Impl;
--# end
--# property-start BaseId.env_elements_baseid /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17439
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Env_Elements_Baseid
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1213 with Inline_Always;
            procedure Finalizer_Scope_3156 with Inline_Always;
            procedure Finalizer_Scope_3157 with Inline_Always;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Internal_Entity_Call_Expr;
Block_Var_2 : Boolean;
Var_Expr : Internal_Entity;
Var_Expr_1 : Boolean;
Var_Expr_2 : Internal_Entity_Assoc_List;
Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Match_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity_Base_Subp_Spec;
Real_Pc : Internal_Entity_Call_Expr;
Var_Expr_4 : Internal_Entity_Entry_Spec;
Var_Expr_5 : Internal_Entity_Call_Expr;
Sym : Symbol_Type;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Boolean;
If_Result_1 : Ref_Categories;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Internal_Entity_Call_Expr;
Fld_5 : Boolean;
Not_Val : Boolean;
Fld_6 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Is_Null : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_7 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_8 : Internal_Entity_Array_Access;
Is_A : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_11 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Array_Lit : Internal_Entity_Array_Access;
Scope_Result_1 : Internal_Entity_Array_Access;
Result_Var_1 : Internal_Entity_Array_Access;
If_Result_2 : Internal_Entity_Array_Access;
Concat_Result : Internal_Entity_Array_Access;
Fld_12 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Assoc_List;
Fld_13 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Fld_14 : Internal_Entity_Base_Subp_Spec;
Fld_15 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Call_Expr;
Cast_Expr_2 : Internal_Entity_Base_Subp_Spec;
Cast_Result_4 : Internal_Entity_Entry_Spec;
Fld_16 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Result_Var_2 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_3 : Internal_Entity_Call_Expr;
Is_Null_2 : Boolean;
Fld_17 : Internal_Entity_Info;
Fld_18 : Internal_Metadata;
Fld_19 : Boolean;
Fld_20 : Boolean;
Fld_21 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Call_Expr;
Fld_22 : Internal_Entity_Base_Type_Decl;
Fld_23 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Fld_24 : Internal_Entity_Base_Type_Decl;
Fld_25 : Boolean;
Fld_26 : Internal_Entity_Info;
Fld_27 : Internal_Metadata;
Fld_28 : Boolean;
Fld_29 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
Scope_Result_4 : Boolean;
Fld_30 : Internal_Entity_Base_Type_Decl;
Fld_31 : Boolean;
Result_Var_4 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_7 : Internal_Entity_Basic_Decl;
Let_Result_2 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_8 : Internal_Entity;
Let_Result_3 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Scope_Result_8 : Internal_Entity_Array_Access;
Is_A_1 : Boolean;
Not_Val_1 : Boolean;
Map_Result_2 : Internal_Entity_Array_Access;
Result_Var_5 : Internal_Entity_Array_Access;
If_Result_8 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result_4 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1213 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1213;
            procedure Finalizer_Scope_3156 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_3156;
            procedure Finalizer_Scope_3157 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_3157;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Id_Base_Id_P_Env_Elements_Baseid,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Id,
                                As_Bare_Base_Id => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 10318 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 10262 '<FieldAccess for env_get at ast.py:17445>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17445
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10256 '<GetSymbol at ast.py:17447>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17447
Sym := Get_Symbol (Self); 
--# expr-done 10256
--# expr-start 10258 '<If at ast.py:17448>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17448
--# expr-start 10257 '<FieldAccess for is_prefix at ast.py:17448>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17448
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10257
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 10258
--# expr-start 10259 '<FieldAccess for origin_node at ast.py:17450>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17450
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10259
--# expr-start 10261 '<If at ast.py:17451>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17451
--# expr-start 10260 '<FieldAccess for can_designate_primitive at ast.py:17452>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17452
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.can_designate_primitive
Fld_2 := Libadalang.Implementation.Name_P_Can_Designate_Primitive (Node => Self);
--# end
--# expr-done 10260
if Fld_2 then
   If_Result_1 := (Inherited_Primitives => True, Nocat => True);
else
   If_Result_1 := (Inherited_Primitives => False, Nocat => True);
end if;
--# expr-done 10261
--# property-call-start AdaNode.env_get
Fld_3 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Sym, Lookup => If_Result, From_Node => Fld_1, Categories => If_Result_1);
--# end
--# expr-done 10262
Block_Var := Fld_3; Inc_Ref (Block_Var);
--# bind items Block_Var
--# expr-start 10263 '<FieldAccess for parent_callexpr at ast.py:17461>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17461
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_callexpr
Fld_4 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10263
Block_Var_1 := Fld_4; 
--# bind pc Block_Var_1
--# expr-start 10265 '<Not at ast.py:17462>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17462
--# expr-start 10264 '<FieldAccess for is_suffix at ast.py:17462>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17462
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_5 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 10264
Not_Val := not (Fld_5); 
--# expr-done 10265
Block_Var_2 := Not_Val; 
--# bind is_prefix Block_Var_2
--# expr-start 10317 '<bind at ast.py:17464>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17464
--# expr-start 10266 '<FieldAccess for origin_node at ast.py:17464>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17464
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10266
Bound_Origin := Fld_6; 
--# expr-start 10316 '<Cond at ast.py:17464>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17464
--# expr-start 10267 '<IsNull at ast.py:17465>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17465
Is_Null := Block_Var_1.Node = null; 
--# expr-done 10267
if Is_Null then
   --# expr-start 10281 '<Concat at ast.py:17475>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17475
--# expr-start 10270 '<Filter at ast.py:17471>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17471
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_318 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_318
         --# expr-start 10269 '<FieldAccess for can_be_paramless at ast.py:17474>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17474
--# expr-start 10268 '<Cast to BasicDecl at ast.py:17474>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17474
Cast_Expr := Item_318; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10268
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.can_be_paramless
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Can_Be_Paramless (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 10269
         if Fld_7 then
         declare
            Item_To_Append : constant Internal_Entity := Item_318;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10270
--# expr-start 10280 '<If at ast.py:17479>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17479
if Block_Var_2 then
--# expr-start 10278 '<Find at ast.py:17481>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17481
   --# expr-start 10271 '<FieldAccess for parents at ast.py:17481>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17481
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 10271
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_8;
      begin
         for Item_319 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_319
--# expr-start 10272 '<IsA TaskBody, BaseSubpBody at ast.py:17482>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17482
Is_A := Item_319.Node /= null 
and then Item_319.Node.Kind in Ada_Task_Body_Range | Ada_Base_Subp_Body; 
--# expr-done 10272
Var_Expr_1 := Is_A;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 10277 '<FieldAccess for name_is at ast.py:17485>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17485
--# expr-start 10275 '<FieldAccess for name at ast.py:17485>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17485
--# expr-start 10274 '<FieldAccess for defining_name at ast.py:17484>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17484
--# expr-start 10273 '<Cast to BasicDecl at ast.py:17484>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17484
Cast_Expr_1 := Item_319; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10273
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 10274
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Fld_9.Node.Defining_Name_F_Name, Info => Fld_9.Info);
--# expr-done 10275
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10276 '<GetSymbol at ast.py:17485>' Sym_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17485
Sym_1 := Get_Symbol (Self); 
--# expr-done 10276
--# property-call-start Name.name_is
Fld_11 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_10.Node, Sym => Sym_1);
--# end
--# expr-done 10277
Scope_Result := Fld_11;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_319; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 10278
Var_Expr := Find_Result;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind b Var_Expr
--# expr-start 10279 '<ArrayLiteral at ???>' Array_Lit None
Array_Lit := Create_Internal_Entity_Array (Internal_Internal_Entity_Array'(1 => Var_Expr)); 
--# expr-done 10279
Scope_Result_1 := Array_Lit;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3156;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_2 := Result_Var_1;
else
   If_Result_2 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 10280
Concat_Result := Concat (Map_Result, If_Result_2); 
--# expr-done 10281
   If_Result_8 := Concat_Result;
else
--# expr-start 10283 '<Cast to AssocList at ast.py:17501>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17501
--# expr-start 10282 '<FieldAccess for suffix at ast.py:17501>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17501
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity (Node => Block_Var_1.Node.Call_Expr_F_Suffix, Info => Block_Var_1.Info);
--# expr-done 10282
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Assoc_List
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Cast_Result_2 := No_Entity_Assoc_List;
   end if;
--# expr-done 10283
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind params Var_Expr_2
--# expr-start 10312 '<Filter at ast.py:17502>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17502
      declare
         Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_321 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_321
         --# expr-start 10311 '<Match at ast.py:17502>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17502
      if Item_321.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_321; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_6; 
--# bind _ Match_0
--# expr-start 10285 '<Eq at ast.py:17504>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17504
--# expr-start 10284 '<length at ast.py:17504>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17504
Fld_13 := Var_Expr_2.Node; 
Len := Length (Fld_13); 
--# expr-done 10284
Is_Equal := Len = 1; 
--# expr-done 10285
Let_Result_1 := Is_Equal; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_7; 
--# bind b Match_1
--# expr-start 10286 '<FieldAccess for subp_spec_or_null at ast.py:17507>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17507
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 10286
Var_Expr_3 := Fld_14;
if Var_Expr_3 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_3
--# expr-start 10308 '<Let at ast.py:17508>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17508
--# scope-start
--# expr-start 10292 '<If at ast.py:17509>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17509
--# expr-start 10289 '<IsNull at ast.py:17510>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17510
--# expr-start 10287 '<Cast to EntrySpec at ast.py:17510>' Cast_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17510
Cast_Expr_2 := Var_Expr_3; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Entry_Spec_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Entry_Spec
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_4 := No_Entity_Entry_Spec;
   end if;
--# expr-done 10287
Var_Expr_4 := Cast_Result_4;
if Var_Expr_4 /= No_Entity_Entry_Spec then
   --# scope-start
--# expr-start 10288 '<FieldAccess for family_type at ast.py:17510>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17510
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity (Node => Var_Expr_4.Node.Entry_Spec_F_Family_Type, Info => Var_Expr_4.Info);
--# expr-done 10288
Scope_Result_2 := Fld_16;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity;
end if;
Is_Null_1 := Result_Var_2.Node = null; 
--# expr-done 10289
if Is_Null_1 then
   If_Result_3 := Block_Var_1;
else
   --# expr-start 10291 '<Cast to CallExpr at ast.py:17511>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17511
--# expr-start 10290 '<parent at ast.py:17511>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17511
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Parent (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# expr-done 10290
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_3 := No_Entity_Call_Expr;
   end if;
--# expr-done 10291
   If_Result_3 := Cast_Result_3;
end if;
--# expr-done 10292
Real_Pc := If_Result_3; 
--# bind real_pc Real_Pc
--# expr-start 10293 '<IsNull at ast.py:17517>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17517
Is_Null_2 := Real_Pc.Node = null; 
--# expr-done 10293
if Is_Null_2 then
   If_Result_5 := True;
else
--# expr-start 10297 '<FieldAccess for is_matching_param_list at ast.py:17522>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17522
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10296 '<FieldAccess for dottable_subp at ast.py:17523>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17523
--# expr-start 10295 '<FieldAccess for md at ast.py:17523>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17523
--# expr-start 10294 '<FieldAccess for info at ast.py:17523>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17523
Fld_17 := Match_1.Info;
--# expr-done 10294
Fld_18 := Fld_17.Md;
--# expr-done 10295
Fld_19 := Fld_18.Dottable_Subp;
--# expr-done 10296
--# property-call-start BaseFormalParamHolder.is_matching_param_list
Fld_20 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Is_Matching_Param_List (Node => Var_Expr_3.Node, Params => Var_Expr_2, Is_Dottable_Subp => Fld_19, Env => Env);
--# end
--# expr-done 10297
if Fld_20 then
--# expr-start 10299 '<Cast to CallExpr at ast.py:17525>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17525
--# expr-start 10298 '<parent at ast.py:17525>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17525
      if Real_Pc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Parent (Node => Real_Pc.Node, E_Info => Real_Pc.Info);
--# expr-done 10298
   if Fld_21.Node = null
      or else Fld_21.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Fld_21.Node,
         Info => Fld_21.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 10299
Var_Expr_5 := Cast_Result_5;
if Var_Expr_5 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_5
--# expr-start 10301 '<FieldAccess for check_for_type at ast.py:17526>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17526
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10300 '<FieldAccess for expr_type at ast.py:17526>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17526
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_22 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 10300
--# property-call-start CallExpr.check_for_type
Fld_23 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Var_Expr_5.Node, Typ => Fld_22, Env => Env, Origin => Bound_Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 10301
Scope_Result_3 := Fld_23;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := True;
end if;
   If_Result_4 := Result_Var_3;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
if If_Result_5 then
   If_Result_7 := True;
else
--# expr-start 10303 '<FieldAccess for check_for_type at ast.py:17533>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17533
      if Real_Pc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10302 '<FieldAccess for expr_type at ast.py:17533>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17533
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_24 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 10302
--# property-call-start CallExpr.check_for_type
Fld_25 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Real_Pc.Node, Typ => Fld_24, Env => Env, Origin => Bound_Origin, E_Info => Real_Pc.Info);
--# end
--# expr-done 10303
if Fld_25 then
   --# expr-start 10307 '<FieldAccess for paramless at ast.py:17534>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17534
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10306 '<FieldAccess for dottable_subp at ast.py:17534>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17534
--# expr-start 10305 '<FieldAccess for md at ast.py:17534>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17534
--# expr-start 10304 '<FieldAccess for info at ast.py:17534>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17534
Fld_26 := Match_1.Info;
--# expr-done 10304
Fld_27 := Fld_26.Md;
--# expr-done 10305
Fld_28 := Fld_27.Dottable_Subp;
--# expr-done 10306
--# property-call-start BaseFormalParamHolder.paramless
Fld_29 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr_3.Node, Dottable_Subp => Fld_28);
--# end
--# expr-done 10307
   If_Result_6 := Fld_29;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
Let_Result := If_Result_7; 
--# end
--# expr-done 10308
Scope_Result_4 := Let_Result;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   --# expr-start 10310 '<FieldAccess for check_for_type at ast.py:17540>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17540
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10309 '<FieldAccess for expr_type at ast.py:17540>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17540
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 10309
--# property-call-start CallExpr.check_for_type
Fld_31 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Block_Var_1.Node, Typ => Fld_30, Env => Env, Origin => Bound_Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 10310
   Result_Var_4 := Fld_31;
end if;
Let_Result_2 := Result_Var_4; 
--# end
Scope_Result_6 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_8; 
--# bind _ Match_2
Let_Result_3 := False; 
--# end
Scope_Result_7 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 10311
         if Match_Result then
         declare
            Item_To_Append : constant Internal_Entity := Item_321;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10312
Scope_Result_8 := Map_Result_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_3157;
   Result_Var_5 := Scope_Result_8;
else
   --# expr-start 10315 '<Filter at ast.py:17548>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17548
      declare
         Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_320 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_320
         --# expr-start 10314 '<Not at ast.py:17548>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17548
--# expr-start 10313 '<IsA BaseTypeDecl at ast.py:17548>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17548
Is_A_1 := Item_320.Node /= null 
and then Item_320.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 10313
Not_Val_1 := not (Is_A_1); 
--# expr-done 10314
         if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Entity := Item_320;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 10315
   Result_Var_5 := Map_Result_2;
end if;
      Inc_Ref (Result_Var_5);
   If_Result_8 := Result_Var_5;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 10316
Dyn_Var_Bind_Result := If_Result_8; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10317
Let_Result_4 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 10318
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1213;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1213;
                     Finalizer_Scope_3156;
                     Finalizer_Scope_3157;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Array_Access,
                        As_Internal_Entity_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Id_P_Env_Elements_Baseid;
--# end
--# property-start BaseId.denotes_the_property_function /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17552
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Denotes_The_Property_Function
  (Node : Bare_Base_Id
      ; Subp_Spec : Internal_Entity_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp_spec Subp_Spec
   Property_Result : Boolean;
            procedure Finalizer_Scope_1214 with Inline_Always;
            procedure Finalizer_Scope_3218 with Inline_Always;
      Block_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subp_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Param_Spec_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Find_Result : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Find_Result_1 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Is_Equal_1 : Boolean;
Not_Val_2 : Boolean;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_5 : Internal_Entity_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result : Boolean;
Let_Result_3 : Boolean;
            procedure Finalizer_Scope_1214 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1214;
            procedure Finalizer_Scope_3218 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3218;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10338 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 10319 '<FieldAccess for primitive_subp_types at ast.py:17559>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17559
      if Subp_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Subp_Types (Node => Subp_Spec.Node, Imprecise_Fallback => False, E_Info => Subp_Spec.Info);
--# end
--# expr-done 10319
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind primitive_types Block_Var
--# expr-start 10322 '<Eq at ast.py:17565>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17565
--# expr-start 10321 '<length at ast.py:17565>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17565
--# expr-start 10320 '<FieldAccess for params at ast.py:17565>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17565
      if Subp_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Subp_Spec.Node, E_Info => Subp_Spec.Info);
--# end
--# expr-done 10320
Len := Length (Fld_1); 
--# expr-done 10321
Is_Equal := Len = 1; 
--# expr-done 10322
if Is_Equal then
   --# expr-start 10337 '<Match at ast.py:17569>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17569
--# expr-start 10323 '<FieldAccess for parent_basic_decl at ast.py:17569>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17569
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10323
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_2; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_4; 
--# bind td Match_0
--# expr-start 10327 '<Not at ast.py:17572>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17572
--# expr-start 10326 '<IsNull at ast.py:17575>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17575
--# expr-start 10325 '<Find at ast.py:17573>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17573
      Find_Result := No_Entity_Base_Type_Decl;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var;
      begin
         for Item_322 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_322
      --# expr-start 10324 '<Eq at ast.py:17574>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17574
Cast_Expr := Item_322; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Match_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 10324
      if Is_Equiv then
         Find_Result := Item_322; 
      end if;
   --# end
               exit when Is_Equiv;
         end loop;
      end;
--# expr-done 10325
Is_Null := Find_Result.Node = null; 
--# expr-done 10326
Not_Val := not (Is_Null); 
--# expr-done 10327
Let_Result := Not_Val; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind sd Match_1
--# expr-start 10336 '<Not at ast.py:17578>' Not_Val_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17578
--# expr-start 10335 '<IsNull at ast.py:17585>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17585
--# expr-start 10334 '<Filter at ast.py:17579>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17579
   --# expr-start 10329 '<FieldAccess for primitive_subp_types at ast.py:17579>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17579
--# expr-start 10328 '<FieldAccess for subp_spec at ast.py:17579>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17579
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Match_1.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Match_1.Info);
--# expr-done 10328
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_subp_types
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Subp_Types (Node => Fld_3.Node, Imprecise_Fallback => False, E_Info => Fld_3.Info);
--# end
--# expr-done 10329
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_323 of
               Collection.Items
         loop
   --# scope-start
               --# bind t1 Item_323
         --# expr-start 10333 '<Not at ast.py:17580>' Not_Val_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17580
--# expr-start 10332 '<IsNull at ast.py:17583>' Is_Null_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17583
--# expr-start 10331 '<Find at ast.py:17581>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17581
      Find_Result_1 := No_Entity_Base_Type_Decl;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var;
      begin
         for Item_324 of
               Collection.Items
         loop
   --# scope-start
               --# bind t2 Item_324
      --# expr-start 10330 '<Eq at ast.py:17582>' Is_Equiv_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17582
Cast_Expr_2 := Item_323; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Cast_Expr_3 := Item_324; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 10330
      if Is_Equiv_1 then
         Find_Result_1 := Item_324; 
      end if;
   --# end
               exit when Is_Equiv_1;
         end loop;
      end;
--# expr-done 10331
Is_Null_1 := Find_Result_1.Node = null; 
--# expr-done 10332
Not_Val_1 := not (Is_Null_1); 
--# expr-done 10333
         if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_323;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10334
Is_Equal_1 := Equivalent (Map_Result, No_Internal_Entity_Base_Type_Decl_Array_Type); 
--# expr-done 10335
Not_Val_2 := not (Is_Equal_1); 
--# expr-done 10336
Let_Result_1 := Not_Val_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
      Finalizer_Scope_3218;
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl | Ada_Entry_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 10337
   If_Result := Match_Result;
else
   If_Result := False;
end if;
Let_Result_3 := If_Result; 
--# end
--# expr-done 10338
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_1214;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1214;
                     Finalizer_Scope_3218;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Denotes_The_Property_Function;
--# end
--# property-start BaseId.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17591
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Xref_Equation
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1215 with Inline_Always;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1215 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1215;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10340 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10339 '<FieldAccess for base_id_xref_equation at ast.py:17593>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17593
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.base_id_xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Base_Id_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10339
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 10340
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1215;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1215;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Xref_Equation;
--# end
--# property-start BaseId.base_id_xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17595
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Base_Id_Xref_Equation
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1216 with Inline_Always;
            procedure Finalizer_Scope_3222 with Inline_Always;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Boolean;
Var_Expr : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
Not_Val : Boolean;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Var;
Bind_Result_2 : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred : Logic_Equation;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Subp_Body;
Fld_8 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1216 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1216;
            procedure Finalizer_Scope_3222 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3222;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10361 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10341 '<FieldAccess for env_elements at ast.py:17597>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17597
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 10341
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind env_els Block_Var
--# expr-start 10343 '<Not at ast.py:17598>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17598
--# expr-start 10342 '<FieldAccess for is_suffix at ast.py:17598>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17598
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_1 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 10342
Not_Val := not (Fld_1); 
--# expr-done 10343
Block_Var_1 := Not_Val; 
--# bind is_prefix Block_Var_1
--# expr-start 10360 '<LogicAny at ast.py:17600>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17600
--# expr-start 10359 '<Map at ast.py:17600>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17600
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_325 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_325
      --# expr-start 10358 '<And at ast.py:17602>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17602
--# expr-start 10345 '<Bind at ast.py:17602>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17602
--# expr-start 10344 '<FieldAccess for ref_var at ast.py:17602>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17602
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 10344
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Assign (Fld_2, Item_325, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17602") else null)); 
--# expr-done 10345
--# expr-start 10357 '<If at ast.py:17603>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17603
if Block_Var_1 then
--# expr-start 10346 '<Cast to BaseSubpBody at ast.py:17606>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17606
Cast_Expr_1 := Item_325; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 10346
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 10347 '<FieldAccess for in_scope at ast.py:17606>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17606
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.in_scope
Fld_8 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 10347
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 10349 '<Bind at ast.py:17608>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17608
--# expr-start 10348 '<FieldAccess for type_var at ast.py:17608>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17608
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10348
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17608") else null)); 
--# expr-done 10349
   If_Result_1 := Bind_Result_1;
else
   --# expr-start 10356 '<And at ast.py:17615>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17615
--# expr-start 10352 '<Bind at ast.py:17615>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17615
--# expr-start 10350 '<FieldAccess for ref_var at ast.py:17615>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17615
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 10350
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 10351 '<FieldAccess for type_var at ast.py:17615>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17615
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10351
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result_2 := Solver.Create_Propagate (Fld_4, Fld_5, Logic_Functor_221'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17615") else null)); 
--# expr-done 10352
--# expr-start 10355 '<Bind at ast.py:17617>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17617
--# expr-start 10353 '<FieldAccess for ref_var at ast.py:17617>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17617
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 10353
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 10354 '<FieldAccess for subp_spec_var at ast.py:17617>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17617
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 10354
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_3 := Solver.Create_Propagate (Fld_6, Fld_7, Logic_Functor_223'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17617") else null)); 
--# expr-done 10355
And_Pred := Create_And (Bind_Result_2, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17615") else null)); 
--# expr-done 10356
   If_Result_1 := And_Pred;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10357
And_Pred_1 := Create_And (Bind_Result, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17602") else null)); 
--# expr-done 10358
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3222;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10359
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17600") else null)); 
--# expr-done 10360
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 10361
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1216;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1216;
                     Finalizer_Scope_3222;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Base_Id_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Char_Literal
   --
      procedure Initialize_Fields_For_Char_Literal
        (Self : Bare_Char_Literal
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Char_Literal;
--# property-start CharLiteral.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17944
pragma Warnings (Off, "is not referenced");
 function Char_Literal_P_Has_Context_Free_Type
  (Node : Bare_Char_Literal
  )
   return Boolean
is
   Self : Bare_Char_Literal  := Bare_Char_Literal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Char_Literal_P_Has_Context_Free_Type;
--# end
--# property-start CharLiteral.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17946
pragma Warnings (Off, "is not referenced");
 function Char_Literal_P_Xref_Equation
  (Node : Bare_Char_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Char_Literal  := Bare_Char_Literal (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1244 with Inline_Always;
      Fld : Logic_Var;
Pred : Logic_Equation;
Fld_1 : Logic_Var;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Cast_Expr : Bare_Char_Literal;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1244 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1244;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10373 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10372 '<And at ast.py:17948>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17948
--# expr-start 10367 '<And at ast.py:17948>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17948
--# expr-start 10363 '<Predicate on BaseTypeDecl.is_non_null_char_type at ast.py:17949>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17949
--# expr-start 10362 '<FieldAccess for expected_type_var at ast.py:17950>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17950
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10362
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Pred := Solver.Create_Predicate (Fld, Create_Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate (Origin)); 
--# expr-done 10363
--# expr-start 10366 '<Bind at ast.py:17951>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17951
--# expr-start 10364 '<FieldAccess for expected_type_var at ast.py:17951>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17951
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10364
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 10365 '<FieldAccess for type_var at ast.py:17951>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17951
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10365
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17951") else null)); 
--# expr-done 10366
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17948") else null)); 
--# expr-done 10367
--# expr-start 10371 '<bind at ast.py:17960>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17960
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 10370 '<Bind at ast.py:17962>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17962
--# expr-start 10368 '<FieldAccess for type_var at ast.py:17962>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17962
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10368
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 10369 '<FieldAccess for ref_var at ast.py:17962>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17962
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 10369
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result_1 := Solver.Create_Propagate (Fld_3, Fld_4, Logic_Functor_710'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Bound_Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17962") else null)); 
--# expr-done 10370
Dyn_Var_Bind_Result := Bind_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10371
And_Pred_1 := Create_And (And_Pred, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17948") else null)); 
--# expr-done 10372
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10373
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1244;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1244;
            raise;
      end;
   return Property_Result;
end Char_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Identifier
   --
      procedure Initialize_Fields_For_Identifier
        (Self : Bare_Identifier
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Identifier;
--# property-start Identifier.is_attr_with_args /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17743
pragma Warnings (Off, "is not referenced");
 function Identifier_P_Is_Attr_With_Args
  (Node : Bare_Identifier
  )
   return Boolean
is
   Self : Bare_Identifier  := Bare_Identifier (Node);
      --# bind self Self
   Property_Result : Boolean;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 10376 '<Eq at ast.py:17744>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
--# expr-start 10374 '<GetSymbol at ast.py:17744>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
Sym := Get_Symbol (Self); 
--# expr-done 10374
--# expr-start 10375 '<Symbol First at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 10375
Is_Equal := Sym = Sym_1; 
--# expr-done 10376
if Is_Equal then
   If_Result := True;
else
   --# expr-start 10379 '<Eq at ast.py:17744>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
--# expr-start 10377 '<GetSymbol at ast.py:17744>' Sym_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
Sym_2 := Get_Symbol (Self); 
--# expr-done 10377
--# expr-start 10378 '<Symbol Last at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 10378
Is_Equal_1 := Sym_2 = Sym_3; 
--# expr-done 10379
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 10382 '<Eq at ast.py:17744>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
--# expr-start 10380 '<GetSymbol at ast.py:17744>' Sym_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
Sym_4 := Get_Symbol (Self); 
--# expr-done 10380
--# expr-start 10381 '<Symbol Range at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 10381
Is_Equal_2 := Sym_4 = Sym_5; 
--# expr-done 10382
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 10385 '<Eq at ast.py:17744>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
--# expr-start 10383 '<GetSymbol at ast.py:17744>' Sym_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
Sym_6 := Get_Symbol (Self); 
--# expr-done 10383
--# expr-start 10384 '<Symbol Length at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 10384
Is_Equal_3 := Sym_6 = Sym_7; 
--# expr-done 10385
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 10388 '<Eq at ast.py:17744>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
--# expr-start 10386 '<GetSymbol at ast.py:17744>' Sym_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
Sym_8 := Get_Symbol (Self); 
--# expr-done 10386
--# expr-start 10387 '<Symbol Has_Same_Storage at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Same_Storage); 
--# expr-done 10387
Is_Equal_4 := Sym_8 = Sym_9; 
--# expr-done 10388
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 10391 '<Eq at ast.py:17744>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
--# expr-start 10389 '<GetSymbol at ast.py:17744>' Sym_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17744
Sym_10 := Get_Symbol (Self); 
--# expr-done 10389
--# expr-start 10390 '<Symbol Overlaps_Storage at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Overlaps_Storage); 
--# expr-done 10390
Is_Equal_5 := Sym_10 = Sym_11; 
--# expr-done 10391
   If_Result_4 := Is_Equal_5;
end if;
         Property_Result := If_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Identifier_P_Is_Attr_With_Args;
--# end
--# property-start Identifier.complete_items /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17748
pragma Warnings (Off, "is not referenced");
 function Identifier_P_Complete_Items
  (Node : Bare_Identifier
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Identifier  := Bare_Identifier (Node);
     Ent : Internal_Entity_Identifier :=
       Internal_Entity_Identifier'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1224 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Internal_Completion_Item_Array_Access;
Let_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1224 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1224;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10394 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10393 '<FieldAccess for complete_items at ast.py:17750>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17750
--# expr-start 10392 '<parent at ast.py:17750>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17750
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 10392
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.complete_items'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Complete_Items (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 10393
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10394
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1224;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1224;
            raise;
      end;
   return Property_Result;
end Identifier_P_Complete_Items;
--# end
--# property-start Identifier.is_constant /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17752
pragma Warnings (Off, "is not referenced");
 function Identifier_P_Is_Constant
  (Node : Bare_Identifier
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Identifier  := Bare_Identifier (Node);
     Ent : Internal_Entity_Identifier :=
       Internal_Entity_Identifier'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_1225 with Inline_Always;
            procedure Finalizer_Scope_3226 with Inline_Always;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity;
Var_Expr_1 : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Is_A : Boolean;
Fld_5 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_7 : Internal_Entity_Subp_Spec;
Fld_8 : Internal_Entity_Subp_Kind;
Scope_Result : Internal_Entity_Subp_Kind;
Result_Var : Internal_Entity_Subp_Kind;
Is_A_1 : Boolean;
Find_Result : Internal_Entity;
Fld_9 : Boolean;
Fld_10 : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Find_Result_1 : Internal_Entity;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1225 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_1225;
            procedure Finalizer_Scope_3226 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_3226;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10415 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10395 '<FieldAccess for referenced_decl at ast.py:17754>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17754
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 10395
Block_Var := Fld; 
--# bind rd Block_Var
--# expr-start 10402 '<Cond at ast.py:17756>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17756
--# expr-start 10396 '<FieldAccess for is_defining at ast.py:17759>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17759
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld_1 := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 10396
if Fld_1 then
   --# expr-start 10399 '<FieldAccess for is_constant_object at ast.py:17760>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17760
--# expr-start 10398 '<FieldAccess for basic_decl at ast.py:17760>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17760
--# expr-start 10397 '<FieldAccess for enclosing_defining_name at ast.py:17760>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17760
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld_2 := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10397
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 10398
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_constant_object'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Constant_Object (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 10399
   If_Result_1 := Fld_4;
else
--# expr-start 10400 '<IsA ObjectDecl, ComponentDecl, EnumLiteralDecl, ParamSpec, NumberDecl at ast.py:17765>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17765
Is_A := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Object_Decl_Range | Ada_Component_Decl_Range | Ada_Enum_Literal_Decl_Range | Ada_Param_Spec_Range | Ada_Number_Decl_Range; 
--# expr-done 10400
if Is_A then
   --# expr-start 10401 '<FieldAccess for is_constant_object at ast.py:17767>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17767
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_constant_object'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Constant_Object (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 10401
   If_Result := Fld_5;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 10402
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 10408 '<Find at ast.py:17773>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17773
   --# expr-start 10403 '<FieldAccess for parents at ast.py:17773>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17773
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 10403
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_6;
      begin
         for Item_326 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_326
      --# expr-start 10407 '<IsA SubpKind.Function at ast.py:17779>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17779
--# expr-start 10404 '<Cast to BaseSubpBody at ast.py:17778>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17778
Cast_Expr := Item_326; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 10404
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 10406 '<FieldAccess for subp_kind at ast.py:17779>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17779
--# expr-start 10405 '<FieldAccess for subp_spec at ast.py:17778>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17778
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_1.Node.Base_Subp_Body_F_Subp_Spec, Info => Var_Expr_1.Info);
--# expr-done 10405
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Subp_Kind (Node => Fld_7.Node.Subp_Spec_F_Subp_Kind, Info => Fld_7.Info);
--# expr-done 10406
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Subp_Kind;
end if;
Is_A_1 := Result_Var.Node /= null 
and then Result_Var.Node.Kind in Ada_Subp_Kind_Function_Range; 
--# expr-done 10407
      if Is_A_1 then
         Find_Result := Item_326; 
      end if;
   --# end
               exit when Is_A_1;
         end loop;
      end;
--# expr-done 10408
Var_Expr := Find_Result;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 10409 '<FieldAccess for is_in_private_part at ast.py:17782>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17782
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 10409
if Fld_9 then
   --# expr-start 10414 '<Not at ast.py:17783>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17783
--# expr-start 10413 '<IsNull at ast.py:17786>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17786
--# expr-start 10412 '<Find at ast.py:17784>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17784
   --# expr-start 10410 '<FieldAccess for parents at ast.py:17784>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17784
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Parents (Node => Block_Var.Node, With_Self => True, E_Info => Block_Var.Info);
--# expr-done 10410
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_10;
      begin
         for Item_327 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_327
      --# expr-start 10411 '<IsA ProtectedTypeDecl at ast.py:17785>' Is_A_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17785
Is_A_2 := Item_327.Node /= null 
and then Item_327.Node.Kind in Ada_Protected_Type_Decl_Range; 
--# expr-done 10411
      if Is_A_2 then
         Find_Result_1 := Item_327; 
      end if;
   --# end
               exit when Is_A_2;
         end loop;
      end;
--# expr-done 10412
Is_Null := Find_Result_1.Node = null; 
--# expr-done 10413
Not_Val := not (Is_Null); 
--# expr-done 10414
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
Scope_Result_1 := If_Result_2;
   --# end
      Finalizer_Scope_3226;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 10415
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1225;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1225;
                     Finalizer_Scope_3226;
            raise;
      end;
   return Property_Result;
end Identifier_P_Is_Constant;
--# end
   --
   --  Primitives for Bare_Op
   --
      procedure Initialize_Fields_For_Op
        (Self : Bare_Op
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Op;
--# property-start Op.subprogram_symbol /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17636
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprogram_Symbol
  (Node : Bare_Op
  )
   return Symbol_Type
is
   Self : Bare_Op  := Bare_Op (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Match_0 : Bare_Op_And;
Match_1 : Bare_Op_Or;
Match_2 : Bare_Op_Xor;
Match_3 : Bare_Op_Abs;
Match_4 : Bare_Op_Not;
Match_5 : Bare_Op_Pow;
Match_6 : Bare_Op_Mult;
Match_7 : Bare_Op_Div;
Match_8 : Bare_Op_Mod;
Match_9 : Bare_Op_Rem;
Match_10 : Bare_Op_Plus;
Match_11 : Bare_Op_Minus;
Match_12 : Bare_Op_Concat;
Match_13 : Bare_Op_Eq;
Match_14 : Bare_Op_Neq;
Match_15 : Bare_Op_Lt;
Match_16 : Bare_Op_Lte;
Match_17 : Bare_Op_Gt;
Match_18 : Bare_Op_Gte;
Match_19 : Bare_Op;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Match_Prefix : Bare_Op;
Cast_Result : Bare_Op_And;
Let_Result : Symbol_Type;
Scope_Result : Symbol_Type;
Cast_Result_1 : Bare_Op_Or;
Let_Result_1 : Symbol_Type;
Scope_Result_1 : Symbol_Type;
Cast_Result_2 : Bare_Op_Xor;
Let_Result_2 : Symbol_Type;
Scope_Result_2 : Symbol_Type;
Cast_Result_3 : Bare_Op_Abs;
Let_Result_3 : Symbol_Type;
Scope_Result_3 : Symbol_Type;
Cast_Result_4 : Bare_Op_Not;
Let_Result_4 : Symbol_Type;
Scope_Result_4 : Symbol_Type;
Cast_Result_5 : Bare_Op_Pow;
Let_Result_5 : Symbol_Type;
Scope_Result_5 : Symbol_Type;
Cast_Result_6 : Bare_Op_Mult;
Let_Result_6 : Symbol_Type;
Scope_Result_6 : Symbol_Type;
Cast_Result_7 : Bare_Op_Div;
Let_Result_7 : Symbol_Type;
Scope_Result_7 : Symbol_Type;
Cast_Result_8 : Bare_Op_Mod;
Let_Result_8 : Symbol_Type;
Scope_Result_8 : Symbol_Type;
Cast_Result_9 : Bare_Op_Rem;
Let_Result_9 : Symbol_Type;
Scope_Result_9 : Symbol_Type;
Cast_Result_10 : Bare_Op_Plus;
Let_Result_10 : Symbol_Type;
Scope_Result_10 : Symbol_Type;
Cast_Result_11 : Bare_Op_Minus;
Let_Result_11 : Symbol_Type;
Scope_Result_11 : Symbol_Type;
Cast_Result_12 : Bare_Op_Concat;
Let_Result_12 : Symbol_Type;
Scope_Result_12 : Symbol_Type;
Cast_Result_13 : Bare_Op_Eq;
Let_Result_13 : Symbol_Type;
Scope_Result_13 : Symbol_Type;
Cast_Result_14 : Bare_Op_Neq;
Let_Result_14 : Symbol_Type;
Scope_Result_14 : Symbol_Type;
Cast_Result_15 : Bare_Op_Lt;
Let_Result_15 : Symbol_Type;
Scope_Result_15 : Symbol_Type;
Cast_Result_16 : Bare_Op_Lte;
Let_Result_16 : Symbol_Type;
Scope_Result_16 : Symbol_Type;
Cast_Result_17 : Bare_Op_Gt;
Let_Result_17 : Symbol_Type;
Scope_Result_17 : Symbol_Type;
Cast_Result_18 : Bare_Op_Gte;
Let_Result_18 : Symbol_Type;
Scope_Result_18 : Symbol_Type;
Cast_Result_19 : Bare_Op;
Let_Result_19 : Symbol_Type;
Scope_Result_19 : Symbol_Type;
Match_Result : Symbol_Type;
Let_Result_20 : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10437 '<Block at ???>' Let_Result_20 None
--# scope-start
--# expr-start 10436 '<Match at ast.py:17642>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17642
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Op (Match_Prefix.Kind) is
         when Ada_Op_And =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 10416 '<Symbol "and" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 10416
Let_Result := Sym; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Op_Or =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 10417 '<Symbol "or" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 10417
Let_Result_1 := Sym_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Op_Xor =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 10418 '<Symbol "xor" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor); 
--# expr-done 10418
Let_Result_2 := Sym_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Op_Abs =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 10419 '<Symbol "abs" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 10419
Let_Result_3 := Sym_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Op_Not =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
--# expr-start 10420 '<Symbol "not" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 10420
Let_Result_4 := Sym_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Op_Pow =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix;
Match_5 := Cast_Result_5; 
--# bind _ Match_5
--# expr-start 10421 '<Symbol "**" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 10421
Let_Result_5 := Sym_5; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Op_Mult =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_6 := Cast_Result_6; 
--# bind _ Match_6
--# expr-start 10422 '<Symbol "*" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 10422
Let_Result_6 := Sym_6; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Op_Div =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Match_Prefix;
Match_7 := Cast_Result_7; 
--# bind _ Match_7
--# expr-start 10423 '<Symbol "/" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 10423
Let_Result_7 := Sym_7; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Op_Mod =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_8 := Cast_Result_8; 
--# bind _ Match_8
--# expr-start 10424 '<Symbol "mod" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod); 
--# expr-done 10424
Let_Result_8 := Sym_8; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Op_Rem =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix;
Match_9 := Cast_Result_9; 
--# bind _ Match_9
--# expr-start 10425 '<Symbol "rem" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem); 
--# expr-done 10425
Let_Result_9 := Sym_9; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_Op_Plus =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Match_Prefix;
Match_10 := Cast_Result_10; 
--# bind _ Match_10
--# expr-start 10426 '<Symbol "+" at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 10426
Let_Result_10 := Sym_10; 
--# end
Scope_Result_10 := Let_Result_10;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Op_Minus =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Match_Prefix;
Match_11 := Cast_Result_11; 
--# bind _ Match_11
--# expr-start 10427 '<Symbol "-" at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 10427
Let_Result_11 := Sym_11; 
--# end
Scope_Result_11 := Let_Result_11;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Op_Concat =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Match_Prefix;
Match_12 := Cast_Result_12; 
--# bind _ Match_12
--# expr-start 10428 '<Symbol "&" at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 10428
Let_Result_12 := Sym_12; 
--# end
Scope_Result_12 := Let_Result_12;
   --# end
            Match_Result := Scope_Result_12; 
         when Ada_Op_Eq =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Match_Prefix;
Match_13 := Cast_Result_13; 
--# bind _ Match_13
--# expr-start 10429 '<Symbol "=" at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 10429
Let_Result_13 := Sym_13; 
--# end
Scope_Result_13 := Let_Result_13;
   --# end
            Match_Result := Scope_Result_13; 
         when Ada_Op_Neq =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Match_Prefix;
Match_14 := Cast_Result_14; 
--# bind _ Match_14
--# expr-start 10430 '<Symbol "/=" at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 10430
Let_Result_14 := Sym_14; 
--# end
Scope_Result_14 := Let_Result_14;
   --# end
            Match_Result := Scope_Result_14; 
         when Ada_Op_Lt =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Match_Prefix;
Match_15 := Cast_Result_15; 
--# bind _ Match_15
--# expr-start 10431 '<Symbol "<" at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 10431
Let_Result_15 := Sym_15; 
--# end
Scope_Result_15 := Let_Result_15;
   --# end
            Match_Result := Scope_Result_15; 
         when Ada_Op_Lte =>
   --# scope-start
--# scope-start
      Cast_Result_16 := Match_Prefix;
Match_16 := Cast_Result_16; 
--# bind _ Match_16
--# expr-start 10432 '<Symbol "<=" at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 10432
Let_Result_16 := Sym_16; 
--# end
Scope_Result_16 := Let_Result_16;
   --# end
            Match_Result := Scope_Result_16; 
         when Ada_Op_Gt =>
   --# scope-start
--# scope-start
      Cast_Result_17 := Match_Prefix;
Match_17 := Cast_Result_17; 
--# bind _ Match_17
--# expr-start 10433 '<Symbol ">" at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 10433
Let_Result_17 := Sym_17; 
--# end
Scope_Result_17 := Let_Result_17;
   --# end
            Match_Result := Scope_Result_17; 
         when Ada_Op_Gte =>
   --# scope-start
--# scope-start
      Cast_Result_18 := Match_Prefix;
Match_18 := Cast_Result_18; 
--# bind _ Match_18
--# expr-start 10434 '<Symbol ">=" at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 10434
Let_Result_18 := Sym_18; 
--# end
Scope_Result_18 := Let_Result_18;
   --# end
            Match_Result := Scope_Result_18; 
         when Ada_Op_And_Then | Ada_Op_Double_Dot | Ada_Op_In | Ada_Op_Not_In | Ada_Op_Or_Else =>
   --# scope-start
--# scope-start
      Cast_Result_19 := Match_Prefix;
Match_19 := Cast_Result_19; 
--# bind _ Match_19
--# expr-start 10435 '<Symbol <<>> at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_20); 
--# expr-done 10435
Let_Result_19 := Sym_19; 
--# end
Scope_Result_19 := Let_Result_19;
   --# end
            Match_Result := Scope_Result_19; 
end case;
--# expr-done 10436
Let_Result_20 := Match_Result; 
--# end
--# expr-done 10437
         Property_Result := Let_Result_20;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Op_P_Subprogram_Symbol;
--# end
--# property-start Op.subprograms_for_symbol /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17665
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprograms_For_Symbol
  (Node : Bare_Op
      ; Sym : Symbol_Type
      ; From_Node : Internal_Entity
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Op  := Bare_Op (Node);
      --# bind self Self
   --# bind sym Sym
   --# bind from_node From_Node
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1218 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Is_A : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1218 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1218;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10449 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10448 '<Concat at ast.py:17673>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17673
--# expr-start 10438 '<FieldAccess for root_type_ops at ast.py:17673>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17673
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_type_ops
Fld := Libadalang.Implementation.Ada_Node_P_Root_Type_Ops (Node => Self, Sym => Sym);
--# end
--# expr-done 10438
--# expr-start 10447 '<FilterMap at ast.py:17675>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17675
   --# expr-start 10441 '<FieldAccess for env_get at ast.py:17673>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17673
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10439 '<FieldAccess for node_env at ast.py:17674>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17674
      if From_Node.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => From_Node.Node, E_Info => From_Node.Info);
--# expr-done 10439
--# expr-start 10440 '<FieldAccess for node at ast.py:17674>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17674
Fld_2 := From_Node.Node;
--# expr-done 10440
--# property-call-start AdaNode.env_get
Fld_3 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld_1, Symbol => Sym, From_Node => Fld_2);
--# end
--# expr-done 10441
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_328 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_328
--# expr-start 10443 '<FieldAccess for is_subprogram at ast.py:17678>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17678
--# expr-start 10442 '<Cast to BasicDecl at ast.py:17678>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17678
Cast_Expr_1 := Item_328; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10442
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Cast_Result_1.Node);
--# end
--# expr-done 10443
if Fld_4 then
   --# expr-start 10445 '<Not at ast.py:17685>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17685
--# expr-start 10444 '<IsA SyntheticSubpDecl at ast.py:17685>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17685
Is_A := Item_328.Node /= null 
and then Item_328.Node.Kind in Ada_Synthetic_Subp_Decl_Range; 
--# expr-done 10444
Not_Val := not (Is_A); 
--# expr-done 10445
   If_Result := Not_Val;
else
   If_Result := False;
end if;
         if If_Result then
      --# expr-start 10446 '<Cast to BasicDecl at ast.py:17676>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17676
Cast_Expr := Item_328; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10446
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10447
Concat_Result := Concat (Fld, Map_Result); 
--# expr-done 10448
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10449
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1218;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1218;
            raise;
      end;
   return Property_Result;
end Op_P_Subprograms_For_Symbol;
--# end
--# property-start Op.subprograms /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17689
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprograms
  (Node : Bare_Op
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Op  := Bare_Op (Node);
     Ent : Internal_Entity_Op :=
       Internal_Entity_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1219 with Inline_Always;
      Fld : Symbol_Type;
Cast_Expr : Internal_Entity_Op;
Cast_Result : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1219 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1219;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10451 '<FieldAccess for subprograms_for_symbol at ast.py:17690>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17690
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10450 '<FieldAccess for subprogram_symbol at ast.py:17690>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17690
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprogram_symbol
Fld := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Self);
--# end
--# expr-done 10450
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start Op.subprograms_for_symbol
Fld_1 := Libadalang.Implementation.Op_P_Subprograms_For_Symbol (Node => Self, Sym => Fld, From_Node => Cast_Result);
--# end
--# expr-done 10451
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1219;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1219;
            raise;
      end;
   return Property_Result;
end Op_P_Subprograms;
--# end
--# property-start Op.name_symbol /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17697
pragma Warnings (Off, "is not referenced");
 function Op_P_Name_Symbol
  (Node : Bare_Op
  )
   return Symbol_Type
is
   Self : Bare_Op  := Bare_Op (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10452 '<FieldAccess for subprogram_symbol at ast.py:17697>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17697
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprogram_symbol
Fld := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Self);
--# end
--# expr-done 10452
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Op_P_Name_Symbol;
--# end
--# property-start Op.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17699
pragma Warnings (Off, "is not referenced");
 function Op_P_Xref_Equation
  (Node : Bare_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Op  := Bare_Op (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1221 with Inline_Always;
      False_Rel : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1221 is
            begin
                     Dec_Ref (False_Rel);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1221;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10453 '<Block at ???>' Let_Result None
--# scope-start
False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17703") else null)); 
Let_Result := False_Rel; Inc_Ref (Let_Result);
--# end
--# expr-done 10453
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1221;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1221;
            raise;
      end;
   return Property_Result;
end Op_P_Xref_Equation;
--# end
--# property-start Op.is_dispatching_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17705
pragma Warnings (Off, "is not referenced");
 function Op_P_Is_Dispatching_Call
  (Node : Bare_Op
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Op  := Bare_Op (Node);
     Ent : Internal_Entity_Op :=
       Internal_Entity_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_3249 with Inline_Always;
      Var_Expr : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Concat_Op;
Match_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_2 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Concat_Op;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var : Boolean;
Let_Result_2 : Boolean;
            procedure Finalizer_Scope_3249 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3249;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10461 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 10454 '<FieldAccess for referenced_decl at ast.py:17707>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17707
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10454
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr
--# expr-start 10460 '<Match at ast.py:17712>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17712
--# expr-start 10458 '<Cast to Expr at ast.py:17712>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17712
--# expr-start 10457 '<Find at ast.py:17711>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17711
   --# expr-start 10455 '<FieldAccess for parents at ast.py:17711>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17711
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 10455
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_329 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_329
      --# expr-start 10456 '<IsA BinOp, ConcatOp, UnOp at ast.py:17711>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17711
Is_A := Item_329.Node /= null 
and then Item_329.Node.Kind in Ada_Bin_Op_Range | Ada_Concat_Op_Range | Ada_Un_Op_Range; 
--# expr-done 10456
      if Is_A then
         Find_Result := Item_329; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 10457
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10458
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Cast_Result; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Concat_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Concat_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind _ Match_0
Let_Result := False; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Case_Expr_Alternative | Ada_Concat_Operand .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 10459 '<FieldAccess for is_dispatching_call_impl at ast.py:17718>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17718
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dispatching_call_impl
Fld_2 := Libadalang.Implementation.Expr_P_Is_Dispatching_Call_Impl (Node => Match_1.Node, Decl => Var_Expr, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 10459
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 10460
Scope_Result_2 := Match_Result;
   --# end
      Finalizer_Scope_3249;
   Result_Var := Scope_Result_2;
else
   Result_Var := False;
end if;
Let_Result_2 := Result_Var; 
--# end
--# expr-done 10461
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3249;
            raise;
      end;
   return Property_Result;
end Op_P_Is_Dispatching_Call;
--# end
   --
   --  Primitives for Bare_Op_Abs
   --
      procedure Initialize_Fields_For_Op_Abs
        (Self : Bare_Op_Abs
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Abs;
   --
   --  Primitives for Bare_Op_And
   --
      procedure Initialize_Fields_For_Op_And
        (Self : Bare_Op_And
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_And;
   --
   --  Primitives for Bare_Op_And_Then
   --
      procedure Initialize_Fields_For_Op_And_Then
        (Self : Bare_Op_And_Then
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_And_Then;
   --
   --  Primitives for Bare_Op_Concat
   --
      procedure Initialize_Fields_For_Op_Concat
        (Self : Bare_Op_Concat
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Concat;
   --
   --  Primitives for Bare_Op_Div
   --
      procedure Initialize_Fields_For_Op_Div
        (Self : Bare_Op_Div
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Div;
   --
   --  Primitives for Bare_Op_Double_Dot
   --
      procedure Initialize_Fields_For_Op_Double_Dot
        (Self : Bare_Op_Double_Dot
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Double_Dot;
   --
   --  Primitives for Bare_Op_Eq
   --
      procedure Initialize_Fields_For_Op_Eq
        (Self : Bare_Op_Eq
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Eq;
   --
   --  Primitives for Bare_Op_Gt
   --
      procedure Initialize_Fields_For_Op_Gt
        (Self : Bare_Op_Gt
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Gt;
   --
   --  Primitives for Bare_Op_Gte
   --
      procedure Initialize_Fields_For_Op_Gte
        (Self : Bare_Op_Gte
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Gte;
   --
   --  Primitives for Bare_Op_In
   --
      procedure Initialize_Fields_For_Op_In
        (Self : Bare_Op_In
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_In;
   --
   --  Primitives for Bare_Op_Lt
   --
      procedure Initialize_Fields_For_Op_Lt
        (Self : Bare_Op_Lt
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Lt;
   --
   --  Primitives for Bare_Op_Lte
   --
      procedure Initialize_Fields_For_Op_Lte
        (Self : Bare_Op_Lte
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Lte;
   --
   --  Primitives for Bare_Op_Minus
   --
      procedure Initialize_Fields_For_Op_Minus
        (Self : Bare_Op_Minus
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Minus;
   --
   --  Primitives for Bare_Op_Mod
   --
      procedure Initialize_Fields_For_Op_Mod
        (Self : Bare_Op_Mod
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Mod;
   --
   --  Primitives for Bare_Op_Mult
   --
      procedure Initialize_Fields_For_Op_Mult
        (Self : Bare_Op_Mult
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Mult;
   --
   --  Primitives for Bare_Op_Neq
   --
      procedure Initialize_Fields_For_Op_Neq
        (Self : Bare_Op_Neq
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Neq;
   --
   --  Primitives for Bare_Op_Not
   --
      procedure Initialize_Fields_For_Op_Not
        (Self : Bare_Op_Not
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Not;
   --
   --  Primitives for Bare_Op_Not_In
   --
      procedure Initialize_Fields_For_Op_Not_In
        (Self : Bare_Op_Not_In
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Not_In;
   --
   --  Primitives for Bare_Op_Or
   --
      procedure Initialize_Fields_For_Op_Or
        (Self : Bare_Op_Or
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Or;
   --
   --  Primitives for Bare_Op_Or_Else
   --
      procedure Initialize_Fields_For_Op_Or_Else
        (Self : Bare_Op_Or_Else
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Or_Else;
   --
   --  Primitives for Bare_Op_Plus
   --
      procedure Initialize_Fields_For_Op_Plus
        (Self : Bare_Op_Plus
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Plus;
   --
   --  Primitives for Bare_Op_Pow
   --
      procedure Initialize_Fields_For_Op_Pow
        (Self : Bare_Op_Pow
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Pow;
   --
   --  Primitives for Bare_Op_Rem
   --
      procedure Initialize_Fields_For_Op_Rem
        (Self : Bare_Op_Rem
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Rem;
   --
   --  Primitives for Bare_Op_Xor
   --
      procedure Initialize_Fields_For_Op_Xor
        (Self : Bare_Op_Xor
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Xor;
   --
   --  Primitives for Bare_String_Literal
   --
      procedure Initialize_Fields_For_String_Literal
        (Self : Bare_String_Literal
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_String_Literal;
--# property-start StringLiteral.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17808
pragma Warnings (Off, "is not referenced");
 function String_Literal_P_Has_Context_Free_Type
  (Node : Bare_String_Literal
  )
   return Boolean
is
   Self : Bare_String_Literal  := Bare_String_Literal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end String_Literal_P_Has_Context_Free_Type;
--# end
--# property-start StringLiteral.env_elements_impl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17810
pragma Warnings (Off, "is not referenced");
 function String_Literal_P_Env_Elements_Impl
  (Node : Bare_String_Literal
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_String_Literal  := Bare_String_Literal (Node);
     Ent : Internal_Entity_String_Literal :=
       Internal_Entity_String_Literal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1228 with Inline_Always;
      Sym : Symbol_Type;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Map_Result : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Array_Access;
Concat_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1228 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1228;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10468 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10467 '<Concat at ast.py:17819>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17819
--# expr-start 10465 '<Map at ast.py:17817>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17817
   --# expr-start 10463 '<FieldAccess for root_type_ops at ast.py:17817>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17817
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10462 '<GetSymbol at ast.py:17817>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17817
Sym := Get_Symbol (Self); 
--# expr-done 10462
--# property-call-start AdaNode.root_type_ops
Fld := Libadalang.Implementation.Ada_Node_P_Root_Type_Ops (Node => Self, Sym => Sym);
--# end
--# expr-done 10463
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld;
      begin
         for Item_330 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_330
      --# expr-start 10464 '<Cast to AdaNode.entity at ast.py:17818>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17818
Cast_Expr := Item_330; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 10464
         declare
            Item_To_Append : constant Internal_Entity := Cast_Result;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10465
--# expr-start 10466 '<FieldAccess for env_elements_baseid at ast.py:17820>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17820
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld_1 := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 10466
Concat_Result := Concat (Map_Result, Fld_1); 
--# expr-done 10467
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10468
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1228;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1228;
            raise;
      end;
   return Property_Result;
end String_Literal_P_Env_Elements_Impl;
--# end
--# property-start StringLiteral.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17823
pragma Warnings (Off, "is not referenced");
 function String_Literal_P_Xref_Equation
  (Node : Bare_String_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_String_Literal  := Bare_String_Literal (Node);
     Ent : Internal_Entity_String_Literal :=
       Internal_Entity_String_Literal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1229 with Inline_Always;
      Fld : Logic_Equation;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1229 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1229;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10479 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10478 '<If at ast.py:17825>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17825
--# expr-start 10470 '<IsA Name at ast.py:17828>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17828
--# expr-start 10469 '<parent at ast.py:17828>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17828
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Parent; 
--# expr-done 10469
Is_A := Fld_4 /= null 
and then Fld_4.Kind in Ada_Name; 
--# expr-done 10470
if Is_A then
   --# expr-start 10471 '<FieldAccess for base_id_xref_equation at ast.py:17829>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17829
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.base_id_xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Base_Id_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10471
   If_Result := Fld;
else
   --# expr-start 10477 '<And at ast.py:17830>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17830
--# expr-start 10473 '<Predicate on BaseTypeDecl.allows_string_literal at ast.py:17830>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17830
--# expr-start 10472 '<FieldAccess for expected_type_var at ast.py:17831>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17831
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10472
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_Predicate (Fld_1, Create_Base_Type_Decl_P_Allows_String_Literal_0_Predicate (Origin)); 
--# expr-done 10473
--# expr-start 10476 '<Bind at ast.py:17832>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17832
--# expr-start 10474 '<FieldAccess for expected_type_var at ast.py:17832>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17832
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10474
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 10475 '<FieldAccess for type_var at ast.py:17832>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17832
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10475
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17832") else null)); 
--# expr-done 10476
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:17830") else null)); 
--# expr-done 10477
   If_Result := And_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 10478
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10479
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1229;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1229;
            raise;
      end;
   return Property_Result;
end String_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Null_Literal
   --
      procedure Initialize_Fields_For_Null_Literal
        (Self : Bare_Null_Literal
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Null_Literal;
--# property-start NullLiteral.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18018
pragma Warnings (Off, "is not referenced");
 function Null_Literal_P_Has_Context_Free_Type
  (Node : Bare_Null_Literal
  )
   return Boolean
is
   Self : Bare_Null_Literal  := Bare_Null_Literal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Null_Literal_P_Has_Context_Free_Type;
--# end
--# property-start NullLiteral.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18020
pragma Warnings (Off, "is not referenced");
 function Null_Literal_P_Xref_Equation
  (Node : Bare_Null_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Null_Literal  := Bare_Null_Literal (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1250 with Inline_Always;
      Fld : Logic_Var;
Pred : Logic_Equation;
Fld_1 : Logic_Var;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1250 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1250;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10486 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10485 '<And at ast.py:18022>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18022
--# expr-start 10481 '<Predicate on BaseTypeDecl.is_access_type_predicate at ast.py:18023>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18023
--# expr-start 10480 '<FieldAccess for expected_type_var at ast.py:18024>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18024
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10480
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Pred := Solver.Create_Predicate (Fld, Create_Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate (Origin)); 
--# expr-done 10481
--# expr-start 10484 '<Bind at ast.py:18025>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18025
--# expr-start 10482 '<FieldAccess for expected_type_var at ast.py:18025>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18025
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10482
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 10483 '<FieldAccess for type_var at ast.py:18025>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18025
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10483
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18025") else null)); 
--# expr-done 10484
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18022") else null)); 
--# expr-done 10485
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 10486
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1250;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1250;
            raise;
      end;
   return Property_Result;
end Null_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Num_Literal
   --
      procedure Initialize_Fields_For_Num_Literal
        (Self : Bare_Num_Literal
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Num_Literal;
--# property-start NumLiteral.is_constant /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17976
pragma Warnings (Off, "is not referenced");
 function Num_Literal_P_Is_Constant
  (Node : Bare_Num_Literal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Num_Literal  := Bare_Num_Literal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Num_Literal_P_Is_Constant;
--# end
   --
   --  Primitives for Bare_Int_Literal
   --
      procedure Initialize_Fields_For_Int_Literal
        (Self : Bare_Int_Literal
        ) is
      begin
            Initialize_Fields_For_Num_Literal
              (Self);
      end Initialize_Fields_For_Int_Literal;
--# property-start IntLiteral.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17998
pragma Warnings (Off, "is not referenced");
 function Int_Literal_P_Xref_Equation
  (Node : Bare_Int_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Int_Literal  := Bare_Int_Literal (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1247 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1247 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1247;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10489 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10488 '<FieldAccess for universal_int_bind at ast.py:18000>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18000
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10487 '<FieldAccess for type_var at ast.py:18000>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18000
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10487
--# property-call-start AdaNode.universal_int_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 10488
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10489
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1247;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1247;
            raise;
      end;
   return Property_Result;
end Int_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Real_Literal
   --
      procedure Initialize_Fields_For_Real_Literal
        (Self : Bare_Real_Literal
        ) is
      begin
            Initialize_Fields_For_Num_Literal
              (Self);
      end Initialize_Fields_For_Real_Literal;
--# property-start RealLiteral.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17986
pragma Warnings (Off, "is not referenced");
 function Real_Literal_P_Xref_Equation
  (Node : Bare_Real_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Real_Literal  := Bare_Real_Literal (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1246 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1246 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1246;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10492 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10491 '<FieldAccess for universal_real_bind at ast.py:17988>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17988
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10490 '<FieldAccess for type_var at ast.py:17988>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:17988
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10490
--# property-call-start AdaNode.universal_real_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 10491
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10492
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1246;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1246;
            raise;
      end;
   return Property_Result;
end Real_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Synthetic_Identifier
   --
      procedure Initialize_Fields_For_Synthetic_Identifier
        (Self : Bare_Synthetic_Identifier
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      Self.Synthetic_Identifier_Sym := null;
      end Initialize_Fields_For_Synthetic_Identifier;
--# property-start SyntheticIdentifier.name_symbol /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18453
pragma Warnings (Off, "is not referenced");
 function Synthetic_Identifier_P_Name_Symbol
  (Node : Bare_Synthetic_Identifier
  )
   return Symbol_Type
is
   Self : Bare_Synthetic_Identifier  := Bare_Synthetic_Identifier (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10493 '<FieldAccess for sym at ast.py:18453>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18453
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Identifier_Sym;
--# expr-done 10493
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Identifier_P_Name_Symbol;
--# end
--# property-start SyntheticIdentifier.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18455
pragma Warnings (Off, "is not referenced");
 function Synthetic_Identifier_P_Relative_Name
  (Node : Bare_Synthetic_Identifier
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Synthetic_Identifier  := Bare_Synthetic_Identifier (Node);
     Ent : Internal_Entity_Synthetic_Identifier :=
       Internal_Entity_Synthetic_Identifier'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Cast_Expr : Internal_Entity_Synthetic_Identifier;
Cast_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Identifier_P_Relative_Name;
--# end
   --
   --  Primitives for Bare_Target_Name
   --
      procedure Initialize_Fields_For_Target_Name
        (Self : Bare_Target_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      Self.Target_Name_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Target_Name;
--# property-start TargetName.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14903
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Ref_Var
  (Node : Bare_Target_Name
  )
   return Logic_Var
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10494 '<FieldAccess for r_ref_var at ast.py:14903>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14903
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Target_Name_R_Ref_Var'Unrestricted_Access;
--# expr-done 10494
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Target_Name_P_Ref_Var;
--# end
--# property-start TargetName.assign_statement /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14905
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Assign_Statement
  (Node : Bare_Target_Name
  )
   return Bare_Assign_Stmt
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
      --# bind self Self
   Property_Result : Bare_Assign_Stmt;
            procedure Finalizer_Scope_1071 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Assign_Stmt;
            procedure Finalizer_Scope_1071 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1071;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10498 '<Cast to AssignStmt at ast.py:14907>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14907
--# expr-start 10497 '<Find at ast.py:14906>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14906
   --# expr-start 10495 '<FieldAccess for parents at ast.py:14906>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14906
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 10495
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_331 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_331
      --# expr-start 10496 '<IsA AssignStmt at ast.py:14906>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14906
Is_A := Item_331 /= null 
and then Item_331.Kind in Ada_Assign_Stmt_Range; 
--# expr-done 10496
      if Is_A then
         Find_Result := Item_331; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 10497
   if Find_Result = null
      or else Find_Result.Kind in Ada_Assign_Stmt_Range
   then
      Cast_Result := Find_Result;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10498
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_1071;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1071;
            raise;
      end;
   return Property_Result;
end Target_Name_P_Assign_Statement;
--# end
--# property-start TargetName.relative_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14911
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Relative_Name
  (Node : Bare_Target_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
      Fld : Bare_Assign_Stmt;
Fld_1 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10502 '<FieldAccess for relative_name at ast.py:14912>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14912
--# expr-start 10501 '<as_entity at ast.py:14912>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14912
--# expr-start 10500 '<FieldAccess for dest at ast.py:14912>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14912
--# expr-start 10499 '<FieldAccess for assign_statement at ast.py:14912>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14912
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TargetName.assign_statement
Fld := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 10499
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Assign_Stmt_F_Dest;
--# expr-done 10500
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 10501
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 10502
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Target_Name_P_Relative_Name;
--# end
--# property-start TargetName.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14915
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Xref_Equation
  (Node : Bare_Target_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1073 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Bare_Assign_Stmt;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Bare_Assign_Stmt;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1073 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1073;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10514 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10513 '<And at ast.py:14917>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14917
--# expr-start 10507 '<Bind at ast.py:14918>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14918
--# expr-start 10503 '<FieldAccess for type_var at ast.py:14918>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14918
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10503
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 10506 '<FieldAccess for type_var at ast.py:14918>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14918
--# expr-start 10505 '<FieldAccess for dest at ast.py:14918>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14918
--# expr-start 10504 '<FieldAccess for assign_statement at ast.py:14918>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14918
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TargetName.assign_statement
Fld_1 := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 10504
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Fld_1.Assign_Stmt_F_Dest;
--# expr-done 10505
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 10506
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14918") else null)); 
--# expr-done 10507
--# expr-start 10512 '<Bind at ast.py:14919>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14919
--# expr-start 10508 '<FieldAccess for ref_var at ast.py:14919>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14919
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 10508
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 10511 '<FieldAccess for ref_var at ast.py:14919>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14919
--# expr-start 10510 '<FieldAccess for dest at ast.py:14919>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14919
--# expr-start 10509 '<FieldAccess for assign_statement at ast.py:14919>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:14919
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TargetName.assign_statement
Fld_5 := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 10509
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Fld_5.Assign_Stmt_F_Dest;
--# expr-done 10510
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6);
--# end
--# expr-done 10511
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_1 := Solver.Create_Unify (Fld_4, Fld_7, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14919") else null)); 
--# expr-done 10512
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:14917") else null)); 
--# expr-done 10513
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 10514
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1073;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1073;
            raise;
      end;
   return Property_Result;
end Target_Name_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Update_Attribute_Ref
   --
      procedure Initialize_Fields_For_Update_Attribute_Ref
        (Self : Bare_Update_Attribute_Ref
         ; Update_Attribute_Ref_F_Prefix : Bare_Name
         ; Update_Attribute_Ref_F_Attribute : Bare_Identifier
         ; Update_Attribute_Ref_F_Values : Bare_Base_Aggregate
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Update_Attribute_Ref_F_Prefix := Update_Attribute_Ref_F_Prefix;
            Self.Update_Attribute_Ref_F_Attribute := Update_Attribute_Ref_F_Attribute;
            Self.Update_Attribute_Ref_F_Values := Update_Attribute_Ref_F_Values;
      Self.Update_Attribute_Ref_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Update_Attribute_Ref;
   function Update_Attribute_Ref_F_Prefix
     (Node : Bare_Update_Attribute_Ref) return Bare_Name
   is
   begin
         return Node.Update_Attribute_Ref_F_Prefix;
   end;
   function Update_Attribute_Ref_F_Attribute
     (Node : Bare_Update_Attribute_Ref) return Bare_Identifier
   is
   begin
         return Node.Update_Attribute_Ref_F_Attribute;
   end;
   function Update_Attribute_Ref_F_Values
     (Node : Bare_Update_Attribute_Ref) return Bare_Base_Aggregate
   is
   begin
         return Node.Update_Attribute_Ref_F_Values;
   end;
--# property-start UpdateAttributeRef.ref_var /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19856
pragma Warnings (Off, "is not referenced");
 function Update_Attribute_Ref_P_Ref_Var
  (Node : Bare_Update_Attribute_Ref
  )
   return Logic_Var
is
   Self : Bare_Update_Attribute_Ref  := Bare_Update_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10515 '<FieldAccess for r_ref_var at ast.py:19856>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19856
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Update_Attribute_Ref_R_Ref_Var'Unrestricted_Access;
--# expr-done 10515
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Update_Attribute_Ref_P_Ref_Var;
--# end
--# property-start UpdateAttributeRef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19859
pragma Warnings (Off, "is not referenced");
 function Update_Attribute_Ref_P_Xref_Equation
  (Node : Bare_Update_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Update_Attribute_Ref  := Bare_Update_Attribute_Ref (Node);
     Ent : Internal_Entity_Update_Attribute_Ref :=
       Internal_Entity_Update_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1373 with Inline_Always;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Aggregate;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1373 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1373;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10528 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10517 '<FieldAccess for resolve_names_internal at ast.py:19865>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19865
--# expr-start 10516 '<FieldAccess for prefix at ast.py:19865>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19865
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Update_Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 10516
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 10517
Block_Var := Fld_1; 
--# expr-start 10520 '<Cast to BaseTypeDecl at ast.py:19866>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19866
--# expr-start 10519 '<FieldAccess for type_val at ast.py:19866>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19866
--# expr-start 10518 '<FieldAccess for prefix at ast.py:19866>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19866
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Update_Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 10518
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_2.Node);
--# end
--# expr-done 10519
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 10520
Block_Var_1 := Cast_Result; 
--# bind prefix_type Block_Var_1
--# expr-start 10527 '<And at ast.py:19867>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19867
--# expr-start 10524 '<Bind at ast.py:19868>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19868
--# expr-start 10523 '<FieldAccess for type_var at ast.py:19868>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19868
--# expr-start 10522 '<Cast to Aggregate at ast.py:19868>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19868
--# expr-start 10521 '<FieldAccess for values at ast.py:19868>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19868
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Aggregate (Node => Ent.Node.Update_Attribute_Ref_F_Values, Info => Ent.Info);
--# expr-done 10521
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10522
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result_1.Node);
--# expr-done 10523
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Block_Var_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19868") else null)); 
--# expr-done 10524
--# expr-start 10526 '<Bind at ast.py:19869>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19869
--# expr-start 10525 '<FieldAccess for type_var at ast.py:19869>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19869
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10525
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Cast_Expr_1 := Block_Var_1; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_6, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19869") else null)); 
--# expr-done 10526
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19867") else null)); 
--# expr-done 10527
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 10528
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1373;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1373;
            raise;
      end;
   return Property_Result;
end Update_Attribute_Ref_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Paren_Expr
   --
      procedure Initialize_Fields_For_Paren_Expr
        (Self : Bare_Paren_Expr
         ; Paren_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Paren_Expr_F_Expr := Paren_Expr_F_Expr;
      end Initialize_Fields_For_Paren_Expr;
   function Paren_Expr_F_Expr
     (Node : Bare_Paren_Expr) return Bare_Expr
   is
   begin
         return Node.Paren_Expr_F_Expr;
   end;
--# property-start ParenExpr.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12793
pragma Warnings (Off, "is not referenced");
 function Paren_Expr_P_Has_Context_Free_Type
  (Node : Bare_Paren_Expr
  )
   return Boolean
is
   Self : Bare_Paren_Expr  := Bare_Paren_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10530 '<FieldAccess for has_context_free_type at ast.py:12793>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12793
--# expr-start 10529 '<FieldAccess for expr at ast.py:12793>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12793
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Paren_Expr_F_Expr;
--# expr-done 10529
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Fld);
--# end
--# expr-done 10530
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Paren_Expr_P_Has_Context_Free_Type;
--# end
--# property-start ParenExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12795
pragma Warnings (Off, "is not referenced");
 function Paren_Expr_P_Xref_Equation
  (Node : Bare_Paren_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Paren_Expr  := Bare_Paren_Expr (Node);
     Ent : Internal_Entity_Paren_Expr :=
       Internal_Entity_Paren_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_973 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Var;
Fld_7 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_973 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_973;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10543 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10542 '<And at ast.py:12798>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12798
--# expr-start 10537 '<And at ast.py:12798>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12798
--# expr-start 10532 '<FieldAccess for sub_equation at ast.py:12798>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12798
--# expr-start 10531 '<FieldAccess for expr at ast.py:12798>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12798
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Paren_Expr_F_Expr, Info => Ent.Info);
--# expr-done 10531
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 10532
--# expr-start 10536 '<Bind at ast.py:12799>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12799
--# expr-start 10534 '<FieldAccess for expected_type_var at ast.py:12799>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12799
--# expr-start 10533 '<FieldAccess for expr at ast.py:12799>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12799
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Paren_Expr_F_Expr;
--# expr-done 10533
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 10534
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 10535 '<FieldAccess for expected_type_var at ast.py:12799>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12799
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10535
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_Unify (Fld_3, Fld_4, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12799") else null)); 
--# expr-done 10536
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12798") else null)); 
--# expr-done 10537
--# expr-start 10541 '<Bind at ast.py:12800>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12800
--# expr-start 10539 '<FieldAccess for type_var at ast.py:12800>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12800
--# expr-start 10538 '<FieldAccess for expr at ast.py:12800>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12800
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Paren_Expr_F_Expr;
--# expr-done 10538
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_5);
--# expr-done 10539
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 10540 '<FieldAccess for type_var at ast.py:12800>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12800
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10540
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_1 := Solver.Create_Unify (Fld_6, Fld_7, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12800") else null)); 
--# expr-done 10541
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12798") else null)); 
--# expr-done 10542
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10543
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_973;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_973;
            raise;
      end;
   return Property_Result;
end Paren_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Quantified_Expr
   --
      procedure Initialize_Fields_For_Quantified_Expr
        (Self : Bare_Quantified_Expr
         ; Quantified_Expr_F_Quantifier : Bare_Quantifier
         ; Quantified_Expr_F_Loop_Spec : Bare_For_Loop_Spec
         ; Quantified_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Quantified_Expr_F_Quantifier := Quantified_Expr_F_Quantifier;
            Self.Quantified_Expr_F_Loop_Spec := Quantified_Expr_F_Loop_Spec;
            Self.Quantified_Expr_F_Expr := Quantified_Expr_F_Expr;
      end Initialize_Fields_For_Quantified_Expr;
   function Quantified_Expr_F_Quantifier
     (Node : Bare_Quantified_Expr) return Bare_Quantifier
   is
   begin
         return Node.Quantified_Expr_F_Quantifier;
   end;
   function Quantified_Expr_F_Loop_Spec
     (Node : Bare_Quantified_Expr) return Bare_For_Loop_Spec
   is
   begin
         return Node.Quantified_Expr_F_Loop_Spec;
   end;
   function Quantified_Expr_F_Expr
     (Node : Bare_Quantified_Expr) return Bare_Expr
   is
   begin
         return Node.Quantified_Expr_F_Expr;
   end;
--# property-start QuantifiedExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18967
pragma Warnings (Off, "is not referenced");
 function Quantified_Expr_P_Xref_Equation
  (Node : Bare_Quantified_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Quantified_Expr  := Bare_Quantified_Expr (Node);
     Ent : Internal_Entity_Quantified_Expr :=
       Internal_Entity_Quantified_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1321 with Inline_Always;
      Block_Var : Boolean;
Fld : Internal_Entity_For_Loop_Spec;
Fld_1 : Boolean;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Bare_Expr;
Fld_11 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
False_Rel : Logic_Equation;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1321 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1321;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10562 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10545 '<FieldAccess for resolve_names at ast.py:18971>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18971
--# expr-start 10544 '<FieldAccess for loop_spec at ast.py:18971>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18971
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Quantified_Expr_F_Loop_Spec, Info => Ent.Info);
--# expr-done 10544
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10545
Block_Var := Fld_1; 
--# bind spec_success Block_Var
--# expr-start 10561 '<If at ast.py:18973>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18973
if Block_Var then
   --# expr-start 10560 '<And at ast.py:18975>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18975
--# expr-start 10555 '<And at ast.py:18975>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18975
--# expr-start 10552 '<And at ast.py:18975>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18975
--# expr-start 10549 '<Bind at ast.py:18975>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18975
--# expr-start 10547 '<FieldAccess for expected_type_var at ast.py:18975>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18975
--# expr-start 10546 '<FieldAccess for expr at ast.py:18975>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18975
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Quantified_Expr_F_Expr;
--# expr-done 10546
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 10547
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 10548 '<FieldAccess for bool_type at ast.py:18975>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18975
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 10548
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18975") else null)); 
--# expr-done 10549
--# expr-start 10551 '<FieldAccess for sub_equation at ast.py:18976>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18976
--# expr-start 10550 '<FieldAccess for expr at ast.py:18976>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18976
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Quantified_Expr_F_Expr, Info => Ent.Info);
--# expr-done 10550
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 10551
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18975") else null)); 
--# expr-done 10552
--# expr-start 10554 '<FieldAccess for matches_expected_formal_prim_type at ast.py:18977>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18977
--# expr-start 10553 '<FieldAccess for expr at ast.py:18977>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18977
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Ent.Node.Quantified_Expr_F_Expr, Info => Ent.Info);
--# expr-done 10553
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_8 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_7.Node, Origin => Origin);
--# end
--# expr-done 10554
And_Pred_1 := Create_And (And_Pred, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18975") else null)); 
--# expr-done 10555
--# expr-start 10559 '<Bind at ast.py:18978>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18978
--# expr-start 10556 '<FieldAccess for type_var at ast.py:18978>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18978
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10556
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 10558 '<FieldAccess for type_var at ast.py:18978>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18978
--# expr-start 10557 '<FieldAccess for expr at ast.py:18978>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18978
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Quantified_Expr_F_Expr;
--# expr-done 10557
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10);
--# expr-done 10558
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Bind_Result_1 := Solver.Create_Unify (Fld_9, Fld_11, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18978") else null)); 
--# expr-done 10559
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18975") else null)); 
--# expr-done 10560
   If_Result := And_Pred_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18979") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 10561
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10562
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1321;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1321;
            raise;
      end;
   return Property_Result;
end Quantified_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Raise_Expr
   --
      procedure Initialize_Fields_For_Raise_Expr
        (Self : Bare_Raise_Expr
         ; Raise_Expr_F_Exception_Name : Bare_Name
         ; Raise_Expr_F_Error_Message : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Raise_Expr_F_Exception_Name := Raise_Expr_F_Exception_Name;
            Self.Raise_Expr_F_Error_Message := Raise_Expr_F_Error_Message;
      end Initialize_Fields_For_Raise_Expr;
   function Raise_Expr_F_Exception_Name
     (Node : Bare_Raise_Expr) return Bare_Name
   is
   begin
         return Node.Raise_Expr_F_Exception_Name;
   end;
   function Raise_Expr_F_Error_Message
     (Node : Bare_Raise_Expr) return Bare_Expr
   is
   begin
         return Node.Raise_Expr_F_Error_Message;
   end;
--# property-start RaiseExpr.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19881
pragma Warnings (Off, "is not referenced");
 function Raise_Expr_P_Has_Context_Free_Type
  (Node : Bare_Raise_Expr
  )
   return Boolean
is
   Self : Bare_Raise_Expr  := Bare_Raise_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Raise_Expr_P_Has_Context_Free_Type;
--# end
--# property-start RaiseExpr.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19883
pragma Warnings (Off, "is not referenced");
 function Raise_Expr_P_Xref_Equation
  (Node : Bare_Raise_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Raise_Expr  := Bare_Raise_Expr (Node);
     Ent : Internal_Entity_Raise_Expr :=
       Internal_Entity_Raise_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1375 with Inline_Always;
            procedure Finalizer_Scope_3255 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Sym : Symbol_Type;
Fld_6 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1375 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1375;
            procedure Finalizer_Scope_3255 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3255;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10577 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10576 '<And at ast.py:19885>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19885
--# expr-start 10568 '<And at ast.py:19885>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19885
--# expr-start 10564 '<FieldAccess for sub_equation at ast.py:19886>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19886
--# expr-start 10563 '<FieldAccess for exception_name at ast.py:19886>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19886
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Raise_Expr_F_Exception_Name, Info => Ent.Info);
--# expr-done 10563
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 10564
--# expr-start 10567 '<Bind at ast.py:19887>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19887
--# expr-start 10565 '<FieldAccess for expected_type_var at ast.py:19887>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19887
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10565
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 10566 '<FieldAccess for type_var at ast.py:19887>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19887
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10566
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19887") else null)); 
--# expr-done 10567
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19885") else null)); 
--# expr-done 10568
--# expr-start 10569 '<FieldAccess for error_message at ast.py:19888>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19888
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Expr_F_Error_Message, Info => Ent.Info);
--# expr-done 10569
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind er Var_Expr
--# expr-start 10575 '<And at ast.py:19889>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19889
--# expr-start 10573 '<Bind at ast.py:19892>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19892
--# expr-start 10570 '<FieldAccess for expected_type_var at ast.py:19892>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19892
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 10570
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 10572 '<FieldAccess for std_entity at ast.py:19892>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19892
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10571 '<Symbol String at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 10571
--# property-call-start AdaNode.std_entity
Fld_6 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 10572
Bind_Result_1 := Solver.Create_Assign (Fld_5, Fld_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19892") else null)); 
--# expr-done 10573
--# expr-start 10574 '<FieldAccess for sub_equation at ast.py:19893>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19893
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 10574
And_Pred_1 := Create_And (Bind_Result_1, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19889") else null)); 
--# expr-done 10575
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3255;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19895") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:19885") else null)); 
--# expr-done 10576
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 10577
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1375;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1375;
                     Finalizer_Scope_3255;
            raise;
      end;
   return Property_Result;
end Raise_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Un_Op
   --
      procedure Initialize_Fields_For_Un_Op
        (Self : Bare_Un_Op
         ; Un_Op_F_Op : Bare_Op
         ; Un_Op_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Un_Op_F_Op := Un_Op_F_Op;
            Self.Un_Op_F_Expr := Un_Op_F_Expr;
      end Initialize_Fields_For_Un_Op;
   function Un_Op_F_Op
     (Node : Bare_Un_Op) return Bare_Op
   is
   begin
         return Node.Un_Op_F_Op;
   end;
   function Un_Op_F_Expr
     (Node : Bare_Un_Op) return Bare_Expr
   is
   begin
         return Node.Un_Op_F_Expr;
   end;
--# property-start UnOp.has_context_free_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12816
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Has_Context_Free_Type
  (Node : Bare_Un_Op
  )
   return Boolean
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Un_Op_P_Has_Context_Free_Type;
--# end
--# property-start UnOp.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12818
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Xref_Equation
  (Node : Bare_Un_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_975 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Bare_Expr;
Fld_8 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Sym : Symbol_Type;
Fld_9 : Boolean;
Sym_1 : Symbol_Type;
Fld_10 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_975 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_975;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10597 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10596 '<And at ast.py:12820>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12820
--# expr-start 10579 '<FieldAccess for sub_equation at ast.py:12820>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12820
--# expr-start 10578 '<FieldAccess for expr at ast.py:12820>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12820
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 10578
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 10579
--# expr-start 10595 '<If at ast.py:12820>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12820
--# expr-start 10581 '<FieldAccess for in_aspect at ast.py:12821>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12821
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10580 '<Symbol Depends at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Depends); 
--# expr-done 10580
--# property-call-start AdaNode.in_aspect
Fld_9 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym);
--# end
--# expr-done 10581
if Fld_9 then
   If_Result := True;
else
   --# expr-start 10583 '<FieldAccess for in_aspect at ast.py:12821>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12821
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10582 '<Symbol Refined_Depends at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Depends); 
--# expr-done 10582
--# property-call-start AdaNode.in_aspect
Fld_10 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_1);
--# end
--# expr-done 10583
   If_Result := Fld_10;
end if;
if If_Result then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12822") else null)); 
   If_Result_1 := True_Rel;
else
   --# expr-start 10594 '<Or at ast.py:12823>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12823
--# expr-start 10584 '<FieldAccess for overload_equation at ast.py:12823>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12823
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UnOp.overload_equation
Fld_2 := Libadalang.Implementation.Un_Op_P_Overload_Equation (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 10584
--# expr-start 10593 '<And at ast.py:12824>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12824
--# expr-start 10588 '<Bind at ast.py:12824>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12824
--# expr-start 10585 '<FieldAccess for expected_type_var at ast.py:12824>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12824
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10585
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 10587 '<FieldAccess for expected_type_var at ast.py:12824>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12824
--# expr-start 10586 '<FieldAccess for expr at ast.py:12824>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12824
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Un_Op_F_Expr;
--# expr-done 10586
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4);
--# expr-done 10587
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result := Solver.Create_Unify (Fld_3, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12824") else null)); 
--# expr-done 10588
--# expr-start 10592 '<Bind at ast.py:12825>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12825
--# expr-start 10589 '<FieldAccess for type_var at ast.py:12825>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12825
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10589
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 10591 '<FieldAccess for type_var at ast.py:12825>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12825
--# expr-start 10590 '<FieldAccess for expr at ast.py:12825>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12825
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Un_Op_F_Expr;
--# expr-done 10590
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_7);
--# expr-done 10591
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result_1 := Solver.Create_Unify (Fld_6, Fld_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12825") else null)); 
--# expr-done 10592
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12824") else null)); 
--# expr-done 10593
Or_Pred := Create_Or (Fld_2, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12823") else null)); 
--# expr-done 10594
   If_Result_1 := Or_Pred;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10595
And_Pred_1 := Create_And (Fld_1, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12820") else null)); 
--# expr-done 10596
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10597
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_975;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_975;
            raise;
      end;
   return Property_Result;
end Un_Op_P_Xref_Equation;
--# end
--# property-start UnOp.overload_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12828
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Overload_Equation
  (Node : Bare_Un_Op
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_976 with Inline_Always;
            procedure Finalizer_Scope_3256 with Inline_Always;
      Fld : Internal_Entity_Op;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_2 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_976 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_976;
            procedure Finalizer_Scope_3256 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3256;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10603 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10602 '<LogicAny at ast.py:12830>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12830
--# expr-start 10601 '<Map at ast.py:12830>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12830
   --# expr-start 10599 '<FieldAccess for subprograms at ast.py:12830>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12830
--# expr-start 10598 '<FieldAccess for op at ast.py:12830>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12830
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 10598
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprograms
Fld_1 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10599
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_1;
      begin
         for Item_332 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_332
      --# expr-start 10600 '<FieldAccess for entity_eq at ast.py:12831>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12831
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UnOp.entity_eq
Fld_2 := Libadalang.Implementation.Un_Op_P_Entity_Eq (Node => Ent.Node, Subp => Item_332, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 10600
         declare
            Item_To_Append : constant Logic_Equation := Fld_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3256;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10601
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12830") else null)); 
--# expr-done 10602
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 10603
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_976;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_976;
                     Finalizer_Scope_3256;
            raise;
      end;
   return Property_Result;
end Un_Op_P_Overload_Equation;
--# end
--# property-start UnOp.entity_eq /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12834
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Entity_Eq
  (Node : Bare_Un_Op
      ; Subp : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp Subp
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_977 with Inline_Always;
      Block_Var : Internal_Entity_Base_Subp_Spec;
Block_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Bare_Op;
Fld_8 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_9 : Bare_Op;
Fld_10 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_2 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
False_Rel : Logic_Equation;
Len : Integer;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_977 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_977;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10625 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10604 '<FieldAccess for subp_spec_or_null at ast.py:12836>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12836
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Subp.Node, E_Info => Subp.Info);
--# end
--# expr-done 10604
Block_Var := Fld; 
--# bind spec Block_Var
--# expr-start 10605 '<FieldAccess for unpacked_formal_params at ast.py:12837>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12837
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 10605
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind ps Block_Var_1
--# expr-start 10624 '<If at ast.py:12838>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12838
--# expr-start 10607 '<Eq at ast.py:12839>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12839
--# expr-start 10606 '<length at ast.py:12839>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12839
Len := Length (Block_Var_1); 
--# expr-done 10606
Is_Equal := Len = 1; 
--# expr-done 10607
if Is_Equal then
   --# expr-start 10623 '<And at ast.py:12842>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12842
--# expr-start 10619 '<And at ast.py:12842>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12842
--# expr-start 10615 '<And at ast.py:12842>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12842
--# expr-start 10611 '<FieldAccess for call_argument_equation at ast.py:12842>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12842
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10609 '<FieldAccess for formal_decl at ast.py:12842>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12842
--# expr-start 10608 '<at at ast.py:12842>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12842
Get_Result := Get (Self, Block_Var_1, 0, True); 
--# expr-done 10608
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 10609
--# expr-start 10610 '<FieldAccess for expr at ast.py:12842>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12842
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 10610
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Block_Var.Node, Param => Fld_2, Arg => Fld_3, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 10611
--# expr-start 10614 '<Bind at ast.py:12845>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12845
--# expr-start 10612 '<FieldAccess for type_var at ast.py:12845>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12845
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10612
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 10613 '<FieldAccess for return_type at ast.py:12845>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12845
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_6 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 10613
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12845") else null)); 
--# expr-done 10614
And_Pred := Create_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12842") else null)); 
--# expr-done 10615
--# expr-start 10618 '<Bind at ast.py:12848>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12848
--# expr-start 10617 '<FieldAccess for ref_var at ast.py:12848>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12848
--# expr-start 10616 '<FieldAccess for op at ast.py:12848>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12848
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Un_Op_F_Op;
--# expr-done 10616
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 10617
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Cast_Expr := Subp; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_8, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12848") else null)); 
--# expr-done 10618
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12842") else null)); 
--# expr-done 10619
--# expr-start 10622 '<Bind at ast.py:12849>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12849
--# expr-start 10621 '<FieldAccess for subp_spec_var at ast.py:12849>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12849
--# expr-start 10620 '<FieldAccess for op at ast.py:12849>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12849
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Un_Op_F_Op;
--# expr-done 10620
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld_9);
--# end
--# expr-done 10621
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_10, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12849") else null)); 
--# expr-done 10622
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12842") else null)); 
--# expr-done 10623
   If_Result := And_Pred_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:12850") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 10624
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10625
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_977;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_977;
            raise;
      end;
   return Property_Result;
end Un_Op_P_Entity_Eq;
--# end
--# property-start UnOp.potential_actuals_for_dispatch /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12853
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Un_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_978 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Singleton : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_978 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Singleton);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_978;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10631 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10630 '<New ExpectedTypeForExpr at ast.py:12855>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12855
--# expr-start 10628 '<FieldAccess for type_expression at ast.py:12856>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12856
--# expr-start 10627 '<at at ast.py:12856>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12856
--# expr-start 10626 '<FieldAccess for abstract_formal_params at ast.py:12856>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12856
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 10626
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 10627
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 10628
--# expr-start 10629 '<FieldAccess for expr at ast.py:12857>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12857
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 10629
New_Struct := (Expected_Type => Fld_1, Expr => Fld_2); 
--# expr-done 10630
                Singleton := Create_Internal_Expected_Type_For_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
Let_Result := Singleton; Inc_Ref (Let_Result);
--# end
--# expr-done 10631
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_978;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_978;
            raise;
      end;
   return Property_Result;
end Un_Op_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start UnOp.is_dispatching_call /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12860
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Is_Dispatching_Call
  (Node : Bare_Un_Op
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Op;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10634 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10633 '<FieldAccess for is_dispatching_call at ast.py:12862>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12862
--# expr-start 10632 '<FieldAccess for op at ast.py:12862>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:12862
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 10632
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 10633
Let_Result := Fld_1; 
--# end
--# expr-done 10634
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Un_Op_P_Is_Dispatching_Call;
--# end
   --
   --  Primitives for Bare_Handled_Stmts
   --
      procedure Initialize_Fields_For_Handled_Stmts
        (Self : Bare_Handled_Stmts
         ; Handled_Stmts_F_Stmts : Bare_Stmt_List
         ; Handled_Stmts_F_Exceptions : Bare_Ada_Node_List
        ) is
      begin
            Self.Handled_Stmts_F_Stmts := Handled_Stmts_F_Stmts;
            Self.Handled_Stmts_F_Exceptions := Handled_Stmts_F_Exceptions;
      end Initialize_Fields_For_Handled_Stmts;
   function Handled_Stmts_F_Stmts
     (Node : Bare_Handled_Stmts) return Bare_Stmt_List
   is
   begin
         return Node.Handled_Stmts_F_Stmts;
   end;
   function Handled_Stmts_F_Exceptions
     (Node : Bare_Handled_Stmts) return Bare_Ada_Node_List
   is
   begin
         return Node.Handled_Stmts_F_Exceptions;
   end;
--# property-start HandledStmts.is_spark /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20884
pragma Warnings (Off, "is not referenced");
 function Handled_Stmts_P_Is_Spark
  (Node : Bare_Handled_Stmts
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Handled_Stmts  := Bare_Handled_Stmts (Node);
     Ent : Internal_Entity_Handled_Stmts :=
       Internal_Entity_Handled_Stmts'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_1431 with Inline_Always;
      Var_Expr : Internal_Entity;
Var_Expr_1 : Internal_Entity_Package_Body;
Fld : Internal_Entity_Stmt_List;
Fld_1 : Bare_Stmt_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Item_334 : Internal_Entity;
Bare_Item_334 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld_3 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_4 : Boolean;
Find_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Pragma_Node;
Fld_5 : Boolean;
Scope_Result : Boolean;
Fld_6 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Package_Body;
Fld_7 : Internal_Entity_Declarative_Part;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
Result_Var_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1431 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1431;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10649 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10642 '<Find at ast.py:20893>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20893
   --# expr-start 10637 '<TakeWhile at ast.py:20891>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20891
   --# expr-start 10635 '<FieldAccess for stmts at ast.py:20891>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20891
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Stmt_List (Node => Ent.Node.Handled_Stmts_F_Stmts, Info => Ent.Info);
--# expr-done 10635
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
            Take_While_Ok : Boolean := False;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Stmt_List := Fld_1;
      begin
         for Untyped_Item_334 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_334 := Untyped_Item_334; 
Is_Null := Bare_Item_334 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_334); 
   If_Result := As_Entity;
end if;
                  Item_334 := If_Result; 
   --# scope-start
               --# bind stmt Item_334
         --# expr-start 10636 '<IsA Pragma at ast.py:20892>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20892
Is_A := Item_334.Node /= null 
and then Item_334.Node.Kind in Ada_Pragma_Node_Range; 
--# expr-done 10636
         Take_While_Ok := Is_A;
         if Take_While_Ok then
         declare
            Item_To_Append : constant Internal_Entity := Item_334;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
               exit when not Take_While_Ok;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10637
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_333 of
               Collection.Items
         loop
   --# scope-start
               --# bind stmt Item_333
      --# expr-start 10641 '<FieldAccess for name_is at ast.py:20894>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20894
--# expr-start 10639 '<FieldAccess for id at ast.py:20894>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20894
--# expr-start 10638 '<Cast to Pragma at ast.py:20894>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20894
Cast_Expr := Item_333; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 10638
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Identifier (Node => Cast_Result.Node.Pragma_Node_F_Id, Info => Cast_Result.Info);
--# expr-done 10639
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10640 '<Symbol SPARK_Mode at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 10640
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Sym);
--# end
--# expr-done 10641
      if Fld_4 then
         Find_Result := Item_333; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 10642
Var_Expr := Find_Result;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind spark_mode Var_Expr
--# expr-start 10644 '<FieldAccess for spark_mode_is_on at ast.py:20896>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20896
--# expr-start 10643 '<Cast to Pragma at ast.py:20896>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20896
Cast_Expr_1 := Var_Expr; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Pragma_Node;
   end if;
--# expr-done 10643
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.spark_mode_is_on
Fld_5 := Libadalang.Implementation.Pragma_Node_P_Spark_Mode_Is_On (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 10644
Scope_Result := Fld_5;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 10646 '<Cast to PackageBody at ast.py:20898>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20898
--# expr-start 10645 '<parent at ast.py:20898>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20898
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 10645
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Package_Body_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Package_Body
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result_2 := No_Entity_Package_Body;
   end if;
--# expr-done 10646
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Entity_Package_Body then
   --# scope-start
--# expr-start 10648 '<FieldAccess for is_spark at ast.py:20898>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20898
--# expr-start 10647 '<FieldAccess for decls at ast.py:20898>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20898
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_1.Node.Package_Body_F_Decls, Info => Var_Expr_1.Info);
--# expr-done 10647
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]DeclarativePart.is_spark'
Fld_8 := Libadalang.Implementation.Dispatcher_Declarative_Part_P_Is_Spark (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 10648
Scope_Result_1 := Fld_8;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   Result_Var_1 := Result_Var;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 10649
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1431;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1431;
            raise;
      end;
   return Property_Result;
end Handled_Stmts_P_Is_Spark;
--# end
   --
   --  Primitives for Bare_Interface_Kind
   --
   --
   --  Primitives for Bare_Interface_Kind_Limited
   --
   --
   --  Primitives for Bare_Interface_Kind_Protected
   --
   --
   --  Primitives for Bare_Interface_Kind_Synchronized
   --
   --
   --  Primitives for Bare_Interface_Kind_Task
   --
   --
   --  Primitives for Bare_Iter_Type
   --
   --
   --  Primitives for Bare_Iter_Type_In
   --
   --
   --  Primitives for Bare_Iter_Type_Of
   --
   --
   --  Primitives for Bare_Library_Item
   --
      procedure Initialize_Fields_For_Library_Item
        (Self : Bare_Library_Item
         ; Library_Item_F_Has_Private : Bare_Private_Node
         ; Library_Item_F_Item : Bare_Basic_Decl
        ) is
      begin
            Self.Library_Item_F_Has_Private := Library_Item_F_Has_Private;
            Self.Library_Item_F_Item := Library_Item_F_Item;
      end Initialize_Fields_For_Library_Item;
   function Library_Item_F_Has_Private
     (Node : Bare_Library_Item) return Bare_Private_Node
   is
   begin
         return Node.Library_Item_F_Has_Private;
   end;
   function Library_Item_F_Item
     (Node : Bare_Library_Item) return Bare_Basic_Decl
   is
   begin
         return Node.Library_Item_F_Item;
   end;
   --
   --  Primitives for Bare_Limited_Node
   --
--# property-start '[dispatcher]Limited.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Limited_Node_P_As_Bool
  (Node : Bare_Limited_Node
  )
   return Boolean
is
   Self : Bare_Limited_Node  := Bare_Limited_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Limited_Node (Self.Kind) is
               when Ada_Limited_Absent =>
                  --# property-call-start Limited.Absent.as_bool
                  Property_Result := Limited_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Limited_Present =>
                  --# property-call-start Limited.Present.as_bool
                  Property_Result := Limited_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Limited_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Limited_Absent
   --
--# property-start Limited.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5268
pragma Warnings (Off, "is not referenced");
 function Limited_Absent_P_As_Bool
  (Node : Bare_Limited_Absent
  )
   return Boolean
is
   Self : Bare_Limited_Absent  := Bare_Limited_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Limited_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Limited_Present
   --
--# property-start Limited.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5268
pragma Warnings (Off, "is not referenced");
 function Limited_Present_P_As_Bool
  (Node : Bare_Limited_Present
  )
   return Boolean
is
   Self : Bare_Limited_Present  := Bare_Limited_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Limited_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Loop_Spec
   --
   --
   --  Primitives for Bare_For_Loop_Spec
   --
      procedure Initialize_Fields_For_For_Loop_Spec
        (Self : Bare_For_Loop_Spec
         ; For_Loop_Spec_F_Var_Decl : Bare_For_Loop_Var_Decl
         ; For_Loop_Spec_F_Loop_Type : Bare_Iter_Type
         ; For_Loop_Spec_F_Has_Reverse : Bare_Reverse_Node
         ; For_Loop_Spec_F_Iter_Expr : Bare_Ada_Node
         ; For_Loop_Spec_F_Iter_Filter : Bare_Expr
        ) is
      begin
            Self.For_Loop_Spec_F_Var_Decl := For_Loop_Spec_F_Var_Decl;
            Self.For_Loop_Spec_F_Loop_Type := For_Loop_Spec_F_Loop_Type;
            Self.For_Loop_Spec_F_Has_Reverse := For_Loop_Spec_F_Has_Reverse;
            Self.For_Loop_Spec_F_Iter_Expr := For_Loop_Spec_F_Iter_Expr;
            Self.For_Loop_Spec_F_Iter_Filter := For_Loop_Spec_F_Iter_Filter;
      end Initialize_Fields_For_For_Loop_Spec;
   function For_Loop_Spec_F_Var_Decl
     (Node : Bare_For_Loop_Spec) return Bare_For_Loop_Var_Decl
   is
   begin
         return Node.For_Loop_Spec_F_Var_Decl;
   end;
   function For_Loop_Spec_F_Loop_Type
     (Node : Bare_For_Loop_Spec) return Bare_Iter_Type
   is
   begin
         return Node.For_Loop_Spec_F_Loop_Type;
   end;
   function For_Loop_Spec_F_Has_Reverse
     (Node : Bare_For_Loop_Spec) return Bare_Reverse_Node
   is
   begin
         return Node.For_Loop_Spec_F_Has_Reverse;
   end;
   function For_Loop_Spec_F_Iter_Expr
     (Node : Bare_For_Loop_Spec) return Bare_Ada_Node
   is
   begin
         return Node.For_Loop_Spec_F_Iter_Expr;
   end;
   function For_Loop_Spec_F_Iter_Filter
     (Node : Bare_For_Loop_Spec) return Bare_Expr
   is
   begin
         return Node.For_Loop_Spec_F_Iter_Filter;
   end;
--# property-start ForLoopSpec.is_iterated_assoc_spec /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18827
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Is_Iterated_Assoc_Spec
  (Node : Bare_For_Loop_Spec
  )
   return Boolean
is
   Self : Bare_For_Loop_Spec  := Bare_For_Loop_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10652 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10651 '<IsA IteratedAssoc at ast.py:18833>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18833
--# expr-start 10650 '<parent at ast.py:18833>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18833
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 10650
Is_A := Fld /= null 
and then Fld.Kind in Ada_Iterated_Assoc_Range; 
--# expr-done 10651
Let_Result := Is_A; 
--# end
--# expr-done 10652
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end For_Loop_Spec_P_Is_Iterated_Assoc_Spec;
--# end
--# property-start ForLoopSpec.iter_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18835
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Iter_Type
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_For_Loop_Spec  := Bare_For_Loop_Spec (Node);
     Ent : Internal_Entity_For_Loop_Spec :=
       Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1317 with Inline_Always;
      Block_Var : Logic_Var;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity;
Pred : Logic_Equation;
True_Rel : Logic_Equation;
Fld_3 : Bare_Ada_Node;
Cast_Result_1 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_For_Loop_Var_Decl;
Fld_6 : Internal_Entity_Type_Expr;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_8 : Bare_For_Loop_Var_Decl;
Fld_9 : Bare_Type_Expr;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Boolean;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_11 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1317 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1317;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_For_Loop_Spec_For_Loop_Spec_P_Iter_Type,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_For_Loop_Spec,
                                As_Bare_For_Loop_Spec => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 10679 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10655 '<FieldAccess for type_var at ast.py:18838>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18838
--# expr-start 10654 '<Cast to Expr at ast.py:18838>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18838
--# expr-start 10653 '<FieldAccess for iter_expr at ast.py:18838>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18838
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 10653
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10654
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result.Node);
--# expr-done 10655
Block_Var := Fld_1; 
--# bind type_var Block_Var
--# expr-start 10670 '<FieldAccess for resolve_names_internal_with_eq at ast.py:18840>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18840
--# expr-start 10656 '<FieldAccess for iter_expr at ast.py:18840>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18840
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 10656
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10669 '<And at ast.py:18842>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18842
--# expr-start 10657 '<Predicate on AdaNode.is_not_null at ast.py:18842>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18842
Block_Var.Value := No_Entity;
Entity_Vars.Reset (Block_Var);
Pred := Solver.Create_Predicate (Block_Var, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 10657
--# expr-start 10668 '<If at ast.py:18846>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18846
--# expr-start 10660 '<IsNull at ast.py:18847>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18847
--# expr-start 10659 '<FieldAccess for id_type at ast.py:18847>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18847
--# expr-start 10658 '<FieldAccess for var_decl at ast.py:18847>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18847
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 10658
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Fld_8.For_Loop_Var_Decl_F_Id_Type;
--# expr-done 10659
Is_Null := Fld_9 = null; 
--# expr-done 10660
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18848") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 10667 '<Bind at ast.py:18849>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18849
--# expr-start 10663 '<FieldAccess for expected_type_var at ast.py:18849>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18849
--# expr-start 10662 '<Cast to Expr at ast.py:18849>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18849
--# expr-start 10661 '<FieldAccess for iter_expr at ast.py:18849>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18849
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.For_Loop_Spec_F_Iter_Expr;
--# expr-done 10661
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Expr
   then
      Cast_Result_1 := Fld_3;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 10662
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Cast_Result_1);
--# expr-done 10663
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 10666 '<FieldAccess for designated_type at ast.py:18850>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18850
--# expr-start 10665 '<FieldAccess for id_type at ast.py:18850>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18850
--# expr-start 10664 '<FieldAccess for var_decl at ast.py:18850>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18850
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_For_Loop_Var_Decl (Node => Ent.Node.For_Loop_Spec_F_Var_Decl, Info => Ent.Info);
--# expr-done 10664
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Type_Expr (Node => Fld_5.Node.For_Loop_Var_Decl_F_Id_Type, Info => Fld_5.Info);
--# expr-done 10665
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 10666
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18849") else null)); 
--# expr-done 10667
   If_Result := Bind_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 10668
And_Pred := Create_And (Pred, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18842") else null)); 
--# expr-done 10669
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_10 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_2.Node, Additional_Equation => And_Pred, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 10670
Block_Var_1 := Fld_10; 
--# bind p Block_Var_1
--# expr-start 10673 '<If at ast.py:18854>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18854
if Block_Var_1 then
   --# expr-start 10672 '<Cast to BaseTypeDecl at ast.py:18856>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18856
--# expr-start 10671 '<get_value at ast.py:18856>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18856
Is_Logic_Var_Defined := Entity_Vars.Is_Defined (Block_Var); 
if Is_Logic_Var_Defined then
Eq_Solution := Entity_Vars.Get_Value (Block_Var); 
   If_Result_1 := Eq_Solution;
else
   If_Result_1 := No_Entity;
end if;
--# expr-done 10671
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 10672
   If_Result_2 := Cast_Result_3;
else
   If_Result_2 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 10673
Block_Var_2 := If_Result_2; 
--# bind typ Block_Var_2
--# expr-start 10678 '<bind at ast.py:18860>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18860
--# expr-start 10674 '<FieldAccess for origin_node at ast.py:18860>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18860
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_11 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10674
Bound_Origin := Fld_11; 
--# expr-start 10677 '<If at ast.py:18860>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18860
--# expr-start 10675 '<FieldAccess for is_implicit_deref at ast.py:18861>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18861
      if Block_Var_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Block_Var_2.Node, Origin => Bound_Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 10675
if Fld_13 then
   --# expr-start 10676 '<FieldAccess for accessed_type at ast.py:18862>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18862
      if Block_Var_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var_2.Node, Origin => Bound_Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 10676
   If_Result_3 := Fld_12;
else
   If_Result_3 := Block_Var_2;
end if;
--# expr-done 10677
Dyn_Var_Bind_Result := If_Result_3; 
--# expr-done 10678
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 10679
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1317;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1317;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end For_Loop_Spec_P_Iter_Type;
--# end
--# property-start ForLoopSpec.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18866
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Xref_Equation
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Spec  := Bare_For_Loop_Spec (Node);
     Ent : Internal_Entity_For_Loop_Spec :=
       Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1318 with Inline_Always;
            procedure Finalizer_Scope_3261 with Inline_Always;
            procedure Finalizer_Scope_3262 with Inline_Always;
            procedure Finalizer_Scope_3263 with Inline_Always;
            procedure Finalizer_Scope_3264 with Inline_Always;
            procedure Finalizer_Scope_3265 with Inline_Always;
            procedure Finalizer_Scope_3266 with Inline_Always;
            procedure Finalizer_Scope_3267 with Inline_Always;
            procedure Finalizer_Scope_3268 with Inline_Always;
      Match_0 : Bare_Iter_Type_In;
Match_1 : Bare_Iter_Type_Of;
Match_0_1 : Internal_Entity_Bin_Op;
Match_1_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Name;
Match_4 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
It_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_For_Loop_Var_Decl;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Iter_Type;
Fld_3 : Internal_Entity;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_For_Loop_Var_Decl;
Fld_7 : Bare_Defining_Name;
Fld_8 : Logic_Var;
Fld_9 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_10 : Logic_Equation;
Fld_11 : Bare_For_Loop_Var_Decl;
Fld_12 : Bare_Defining_Name;
Fld_13 : Logic_Var;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_16 : Logic_Equation;
Fld_17 : Bare_For_Loop_Var_Decl;
Fld_18 : Bare_Defining_Name;
Fld_19 : Logic_Var;
Fld_20 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_21 : Internal_Entity_Base_Type_Decl;
Fld_22 : Logic_Equation;
Fld_23 : Bare_For_Loop_Var_Decl;
Fld_24 : Bare_Defining_Name;
Fld_25 : Logic_Var;
Fld_26 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Scope_Result : Logic_Equation;
Fld_27 : Logic_Equation;
Result_Var : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Bin_Op;
Let_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Result_6 : Internal_Entity;
Let_Result_4 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_28 : Internal_Entity_Base_Type_Decl;
Fld_29 : Bare_For_Loop_Var_Decl;
Fld_30 : Bare_Defining_Name;
Fld_31 : Logic_Var;
Fld_32 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_33 : Boolean;
If_Result : Logic_Equation;
Let_Result_5 : Logic_Equation;
Match_Prefix_1 : Bare_Iter_Type;
Cast_Result_8 : Bare_Iter_Type_In;
Let_Result_6 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_9 : Bare_Iter_Type_Of;
Let_Result_7 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result_1 : Logic_Equation;
And_Pred_5 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Fld_34 : Bare_Expr;
Fld_35 : Logic_Var;
Fld_36 : Internal_Entity_Base_Type_Decl;
Cast_Result_10 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
Fld_37 : Bare_Expr;
Fld_38 : Logic_Var;
Fld_39 : Internal_Entity_Base_Type_Decl;
Cast_Result_11 : Internal_Entity;
Bind_Result_6 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_40 : Internal_Entity_Expr;
Fld_41 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_42 : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Let_Result_8 : Logic_Equation;
            procedure Finalizer_Scope_1318 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_7);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_41);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Let_Result_8);
            end Finalizer_Scope_1318;
            procedure Finalizer_Scope_3261 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_3261;
            procedure Finalizer_Scope_3262 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3262;
            procedure Finalizer_Scope_3263 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3263;
            procedure Finalizer_Scope_3264 is
            begin
                     Dec_Ref (Fld_16);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3264;
            procedure Finalizer_Scope_3265 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_27);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_3265;
            procedure Finalizer_Scope_3266 is
            begin
                     Dec_Ref (Fld_22);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_3266;
            procedure Finalizer_Scope_3267 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_3267;
            procedure Finalizer_Scope_3268 is
            begin
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_3268;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10746 '<Block at ???>' Let_Result_8 None
--# scope-start
--# expr-start 10745 '<And at ast.py:18868>' And_Pred_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18868
--# expr-start 10729 '<And at ast.py:18868>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18868
--# expr-start 10681 '<FieldAccess for sub_equation at ast.py:18868>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18868
--# expr-start 10680 '<FieldAccess for var_decl at ast.py:18868>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18868
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_For_Loop_Var_Decl (Node => Ent.Node.For_Loop_Spec_F_Var_Decl, Info => Ent.Info);
--# expr-done 10680
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 10681
--# expr-start 10728 '<Match at ast.py:18868>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18868
--# expr-start 10682 '<FieldAccess for loop_type at ast.py:18868>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18868
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.For_Loop_Spec_F_Loop_Type;
--# expr-done 10682
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld_2; 
case Ada_Iter_Type (Match_Prefix_1.Kind) is
         when Ada_Iter_Type_In =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix_1;
Match_0 := Cast_Result_8; 
--# bind _ Match_0
--# expr-start 10718 '<Match at ast.py:18874>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18874
--# expr-start 10683 '<FieldAccess for iter_expr at ast.py:18874>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18874
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 10683
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_3; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_2; 
--# bind binop Match_0_1
--# expr-start 10693 '<And at ast.py:18877>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18877
--# expr-start 10687 '<And at ast.py:18877>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18877
--# expr-start 10684 '<FieldAccess for sub_equation at ast.py:18877>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18877
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0_1.Node, Env => Env, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 10684
--# expr-start 10686 '<Predicate on BaseTypeDecl.is_not_root_int_type at ast.py:18880>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18880
--# expr-start 10685 '<FieldAccess for type_var at ast.py:18880>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18880
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0_1.Node);
--# expr-done 10685
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Pred := Solver.Create_Predicate (Fld_5, Create_Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate); 
--# expr-done 10686
And_Pred := Create_And (Fld_4, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18877") else null)); 
--# expr-done 10687
--# expr-start 10692 '<Bind at ast.py:18881>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18881
--# expr-start 10690 '<FieldAccess for type_var at ast.py:18881>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18881
--# expr-start 10689 '<FieldAccess for id at ast.py:18881>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18881
--# expr-start 10688 '<FieldAccess for var_decl at ast.py:18881>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18881
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 10688
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Fld_6.For_Loop_Var_Decl_F_Id;
--# expr-done 10689
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_7);
--# expr-done 10690
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 10691 '<FieldAccess for type_var at ast.py:18881>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18881
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0_1.Node);
--# expr-done 10691
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
Bind_Result := Solver.Create_Unify (Fld_8, Fld_9, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18881") else null)); 
--# expr-done 10692
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18877") else null)); 
--# expr-done 10693
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3262;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_3; 
--# bind t Match_1_1
--# expr-start 10701 '<And at ast.py:18886>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18886
--# expr-start 10694 '<FieldAccess for sub_equation at ast.py:18886>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18886
      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_1_1.Node, Env => Env, Origin => Origin, E_Info => Match_1_1.Info);
--# end
--# expr-done 10694
--# expr-start 10700 '<Bind at ast.py:18887>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18887
--# expr-start 10697 '<FieldAccess for type_var at ast.py:18887>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18887
--# expr-start 10696 '<FieldAccess for id at ast.py:18887>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18887
--# expr-start 10695 '<FieldAccess for var_decl at ast.py:18887>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18887
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 10695
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Fld_11.For_Loop_Var_Decl_F_Id;
--# expr-done 10696
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_12);
--# expr-done 10697
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 10699 '<FieldAccess for canonical_type at ast.py:18888>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18888
--# expr-start 10698 '<FieldAccess for designated_type at ast.py:18888>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18888
      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_14 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_1_1.Node, Origin => Origin, E_Info => Match_1_1.Info);
--# end
--# expr-done 10698
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld_14.Node, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 10699
      Cast_Result := Create_Internal_Entity
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_13, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18887") else null)); 
--# expr-done 10700
And_Pred_2 := Create_And (Fld_10, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18886") else null)); 
--# expr-done 10701
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3263;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind r Match_2
--# expr-start 10708 '<And at ast.py:18891>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18891
--# expr-start 10702 '<FieldAccess for sub_equation at ast.py:18891>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18891
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_16 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 10702
--# expr-start 10707 '<Bind at ast.py:18892>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18892
--# expr-start 10705 '<FieldAccess for type_var at ast.py:18892>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18892
--# expr-start 10704 '<FieldAccess for id at ast.py:18892>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18892
--# expr-start 10703 '<FieldAccess for var_decl at ast.py:18892>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18892
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 10703
      if Fld_17 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Fld_17.For_Loop_Var_Decl_F_Id;
--# expr-done 10704
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_18);
--# expr-done 10705
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
--# expr-start 10706 '<FieldAccess for type_var at ast.py:18892>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18892
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_2.Node);
--# expr-done 10706
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
Bind_Result_2 := Solver.Create_Unify (Fld_19, Fld_20, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18892") else null)); 
--# expr-done 10707
And_Pred_3 := Create_And (Fld_16, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18891") else null)); 
--# expr-done 10708
Let_Result_2 := And_Pred_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3264;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind t Match_3
--# expr-start 10709 '<FieldAccess for name_designated_type at ast.py:18898>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18898
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_21 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 10709
Var_Expr := Fld_21;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 10716 '<And at ast.py:18900>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18900
--# expr-start 10710 '<FieldAccess for sub_equation at ast.py:18900>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18900
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_3.Node, Env => Env, Origin => Origin, E_Info => Match_3.Info);
--# end
--# expr-done 10710
--# expr-start 10715 '<Bind at ast.py:18901>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18901
--# expr-start 10713 '<FieldAccess for type_var at ast.py:18901>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18901
--# expr-start 10712 '<FieldAccess for id at ast.py:18901>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18901
--# expr-start 10711 '<FieldAccess for var_decl at ast.py:18901>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18901
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 10711
      if Fld_23 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Fld_23.For_Loop_Var_Decl_F_Id;
--# expr-done 10712
      if Fld_24 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24);
--# expr-done 10713
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
--# expr-start 10714 '<FieldAccess for canonical_type at ast.py:18901>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18901
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_26 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 10714
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_26.Node,
         Info => Fld_26.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_25, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18901") else null)); 
--# expr-done 10715
And_Pred_4 := Create_And (Fld_22, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18900") else null)); 
--# expr-done 10716
Scope_Result := And_Pred_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3266;
   Result_Var := Scope_Result;
else
   --# expr-start 10717 '<FieldAccess for iterator_xref_equation at ast.py:18902>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18902
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ForLoopSpec.iterator_xref_equation
Fld_27 := Libadalang.Implementation.For_Loop_Spec_P_Iterator_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10717
   Result_Var := Fld_27;
end if;
      Inc_Ref (Result_Var);
Let_Result_3 := Result_Var; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3265;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_6; 
--# bind _ Match_4
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18905") else null)); 
Let_Result_4 := True_Rel; Inc_Ref (Let_Result_4);
--# end
Scope_Result_5 := Let_Result_4;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3267;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
end case;
--# expr-done 10718
Let_Result_6 := Match_Result; Inc_Ref (Let_Result_6);
--# end
Scope_Result_6 := Let_Result_6;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3261;
            Match_Result_1 := Scope_Result_6; Inc_Ref (Match_Result_1);
         when Ada_Iter_Type_Of =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix_1;
Match_1 := Cast_Result_9; 
--# bind _ Match_1
--# expr-start 10727 '<Let at ast.py:18909>' Let_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18909
--# scope-start
--# expr-start 10719 '<FieldAccess for iter_type at ast.py:18909>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18909
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ForLoopSpec.iter_type
Fld_28 := Libadalang.Implementation.For_Loop_Spec_P_Iter_Type (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10719
It_Typ := Fld_28; 
--# bind it_typ It_Typ
--# expr-start 10726 '<If at ast.py:18909>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18909
--# expr-start 10720 '<FieldAccess for is_iterable_type at ast.py:18910>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18910
      if It_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_33 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => It_Typ.Node, Origin => Origin, E_Info => It_Typ.Info);
--# end
--# expr-done 10720
if Fld_33 then
   --# expr-start 10725 '<Bind at ast.py:18914>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18914
--# expr-start 10723 '<FieldAccess for type_var at ast.py:18914>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18914
--# expr-start 10722 '<FieldAccess for id at ast.py:18914>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18914
--# expr-start 10721 '<FieldAccess for var_decl at ast.py:18914>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18914
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 10721
      if Fld_29 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Fld_29.For_Loop_Var_Decl_F_Id;
--# expr-done 10722
      if Fld_30 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_30);
--# expr-done 10723
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
--# expr-start 10724 '<FieldAccess for iterable_comp_type at ast.py:18914>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18914
      if It_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_32 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => It_Typ.Node, Origin => Origin, E_Info => It_Typ.Info);
--# end
--# expr-done 10724
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_32.Node,
         Info => Fld_32.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_31, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18914") else null)); 
--# expr-done 10725
   If_Result := Bind_Result_4;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18916") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 10726
Let_Result_5 := If_Result; Inc_Ref (Let_Result_5);
--# end
--# expr-done 10727
Let_Result_7 := Let_Result_5; Inc_Ref (Let_Result_7);
--# end
Scope_Result_7 := Let_Result_7;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3268;
            Match_Result_1 := Scope_Result_7; Inc_Ref (Match_Result_1);
end case;
--# expr-done 10728
And_Pred_5 := Create_And (Fld_1, Match_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18868") else null)); 
--# expr-done 10729
--# expr-start 10744 '<If at ast.py:18918>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18918
--# expr-start 10731 '<IsNull at ast.py:18919>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18919
--# expr-start 10730 '<FieldAccess for iter_filter at ast.py:18919>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18919
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Create_Internal_Entity_Expr (Node => Ent.Node.For_Loop_Spec_F_Iter_Filter, Info => Ent.Info);
--# expr-done 10730
Is_Null := Fld_42.Node = null; 
--# expr-done 10731
if Is_Null then
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18920") else null)); 
   If_Result_1 := True_Rel_1;
else
   --# expr-start 10743 '<And at ast.py:18921>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18921
--# expr-start 10740 '<And at ast.py:18921>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18921
--# expr-start 10735 '<Bind at ast.py:18921>' Bind_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18921
--# expr-start 10733 '<FieldAccess for expected_type_var at ast.py:18921>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18921
--# expr-start 10732 '<FieldAccess for iter_filter at ast.py:18921>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18921
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Self.For_Loop_Spec_F_Iter_Filter;
--# expr-done 10732
      if Fld_34 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_34);
--# expr-done 10733
Fld_35.Value := No_Entity;
Entity_Vars.Reset (Fld_35);
--# expr-start 10734 '<FieldAccess for bool_type at ast.py:18921>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18921
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_36 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 10734
      Cast_Result_10 := Create_Internal_Entity
        (Node => Fld_36.Node,
         Info => Fld_36.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_35, Cast_Result_10, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18921") else null)); 
--# expr-done 10735
--# expr-start 10739 '<Bind at ast.py:18922>' Bind_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18922
--# expr-start 10737 '<FieldAccess for type_var at ast.py:18922>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18922
--# expr-start 10736 '<FieldAccess for iter_filter at ast.py:18922>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18922
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Self.For_Loop_Spec_F_Iter_Filter;
--# expr-done 10736
      if Fld_37 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_37);
--# expr-done 10737
Fld_38.Value := No_Entity;
Entity_Vars.Reset (Fld_38);
--# expr-start 10738 '<FieldAccess for bool_type at ast.py:18922>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18922
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_39 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 10738
      Cast_Result_11 := Create_Internal_Entity
        (Node => Fld_39.Node,
         Info => Fld_39.Info);
Bind_Result_6 := Solver.Create_Assign (Fld_38, Cast_Result_11, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18922") else null)); 
--# expr-done 10739
And_Pred_6 := Create_And (Bind_Result_5, Bind_Result_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18921") else null)); 
--# expr-done 10740
--# expr-start 10742 '<FieldAccess for sub_equation at ast.py:18923>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18923
--# expr-start 10741 '<FieldAccess for iter_filter at ast.py:18923>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18923
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Create_Internal_Entity_Expr (Node => Ent.Node.For_Loop_Spec_F_Iter_Filter, Info => Ent.Info);
--# expr-done 10741
      if Fld_40.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_41 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_40.Node, Env => Env, Origin => Origin, E_Info => Fld_40.Info);
--# end
--# expr-done 10742
And_Pred_7 := Create_And (And_Pred_6, Fld_41, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18921") else null)); 
--# expr-done 10743
   If_Result_1 := And_Pred_7;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10744
And_Pred_8 := Create_And (And_Pred_5, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18868") else null)); 
--# expr-done 10745
Let_Result_8 := And_Pred_8; Inc_Ref (Let_Result_8);
--# end
--# expr-done 10746
         Property_Result := Let_Result_8;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1318;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1318;
                     Finalizer_Scope_3261;
                     Finalizer_Scope_3262;
                     Finalizer_Scope_3263;
                     Finalizer_Scope_3264;
                     Finalizer_Scope_3265;
                     Finalizer_Scope_3266;
                     Finalizer_Scope_3267;
                     Finalizer_Scope_3268;
            raise;
      end;
   return Property_Result;
end For_Loop_Spec_P_Xref_Equation;
--# end
--# property-start ForLoopSpec.iterator_xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18926
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Iterator_Xref_Equation
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Spec  := Bare_For_Loop_Spec (Node);
     Ent : Internal_Entity_For_Loop_Spec :=
       Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1319 with Inline_Always;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Tv : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Boolean;
Fld_3 : Internal_Entity;
Fld_4 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity;
Cast_Result_2 : Internal_Entity_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Fld_6 : Bare_For_Loop_Var_Decl;
Fld_7 : Bare_Defining_Name;
Fld_8 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result : Logic_Equation;
False_Rel : Logic_Equation;
If_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1319 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1319;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10765 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 10748 '<Cast to Expr at ast.py:18928>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18928
--# expr-start 10747 '<FieldAccess for iter_expr at ast.py:18928>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18928
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 10747
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10748
Block_Var := Cast_Result; 
--# bind iter_expr Block_Var
--# expr-start 10751 '<FieldAccess for resolve_names_internal_with_eq at ast.py:18930>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18930
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10750 '<Predicate on BaseTypeDecl.is_iterator_type at ast.py:18931>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18931
--# expr-start 10749 '<FieldAccess for type_var at ast.py:18932>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18932
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var.Node);
--# expr-done 10749
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_Predicate (Fld_1, Create_Base_Type_Decl_P_Is_Iterator_Type_0_Predicate (Origin)); 
--# expr-done 10750
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Block_Var.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 10751
Block_Var_1 := Fld_2; 
--# bind p Block_Var_1
--# expr-start 10759 '<Let at ast.py:18936>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18936
--# scope-start
--# expr-start 10752 '<FieldAccess for type_val at ast.py:18937>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18937
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Block_Var.Node);
--# end
--# expr-done 10752
Tv := Fld_3; 
--# bind tv Tv
--# expr-start 10756 '<Cast to BaseTypeDecl at ast.py:18941>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18941
--# expr-start 10755 '<EnvGet at ast.py:18940>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18940
--# expr-start 10753 '<FieldAccess for children_env at ast.py:18940>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18940
      if Tv.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Tv.Node, E_Info => Tv.Info);
--# expr-done 10753
--# expr-start 10754 '<Symbol Cursor at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Cursor); 
--# expr-done 10754
Env_Get_Result := AST_Envs.Get_First (Self => Fld_4, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 10755
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10756
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 10758 '<FieldAccess for iterable_cursor_type at ast.py:18943>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18943
--# expr-start 10757 '<Cast to TypeDecl at ast.py:18943>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18943
Cast_Expr := Tv; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_2 := No_Entity_Type_Decl;
   end if;
--# expr-done 10757
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.iterable_cursor_type
Fld_5 := Libadalang.Implementation.Type_Decl_P_Iterable_Cursor_Type (Node => Cast_Result_2.Node, Origin => Origin, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 10758
   Result_Var := Fld_5;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 10759
Block_Var_2 := Let_Result; 
--# bind cursor_type Block_Var_2
--# expr-start 10764 '<If at ast.py:18947>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18947
if Block_Var_1 then
   --# expr-start 10763 '<Bind at ast.py:18949>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18949
--# expr-start 10762 '<FieldAccess for type_var at ast.py:18949>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18949
--# expr-start 10761 '<FieldAccess for id at ast.py:18949>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18949
--# expr-start 10760 '<FieldAccess for var_decl at ast.py:18949>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18949
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 10760
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Fld_6.For_Loop_Var_Decl_F_Id;
--# expr-done 10761
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_7);
--# expr-done 10762
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Cast_Expr_1 := Block_Var_2; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Solver.Create_Assign (Fld_8, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18949") else null)); 
--# expr-done 10763
   If_Result := Bind_Result;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:18950") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 10764
Let_Result_1 := If_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10765
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1319;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1319;
            raise;
      end;
   return Property_Result;
end For_Loop_Spec_P_Iterator_Xref_Equation;
--# end
--# property-start ForLoopSpec.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18956
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Xref_Entry_Point
  (Node : Bare_For_Loop_Spec
  )
   return Boolean
is
   Self : Bare_For_Loop_Spec  := Bare_For_Loop_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10767 '<Not at ast.py:18956>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18956
--# expr-start 10766 '<FieldAccess for is_iterated_assoc_spec at ast.py:18956>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18956
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ForLoopSpec.is_iterated_assoc_spec
Fld := Libadalang.Implementation.For_Loop_Spec_P_Is_Iterated_Assoc_Spec (Node => Self);
--# end
--# expr-done 10766
Not_Val := not (Fld); 
--# expr-done 10767
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end For_Loop_Spec_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_While_Loop_Spec
   --
      procedure Initialize_Fields_For_While_Loop_Spec
        (Self : Bare_While_Loop_Spec
         ; While_Loop_Spec_F_Expr : Bare_Expr
        ) is
      begin
            Self.While_Loop_Spec_F_Expr := While_Loop_Spec_F_Expr;
      end Initialize_Fields_For_While_Loop_Spec;
   function While_Loop_Spec_F_Expr
     (Node : Bare_While_Loop_Spec) return Bare_Expr
   is
   begin
         return Node.While_Loop_Spec_F_Expr;
   end;
--# property-start WhileLoopSpec.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21322
pragma Warnings (Off, "is not referenced");
 function While_Loop_Spec_P_Xref_Equation
  (Node : Bare_While_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_While_Loop_Spec  := Bare_While_Loop_Spec (Node);
     Ent : Internal_Entity_While_Loop_Spec :=
       Internal_Entity_While_Loop_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1457 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1457 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1457;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10778 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10777 '<And at ast.py:21324>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21324
--# expr-start 10774 '<And at ast.py:21324>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21324
--# expr-start 10771 '<Bind at ast.py:21325>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21325
--# expr-start 10769 '<FieldAccess for expected_type_var at ast.py:21325>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21325
--# expr-start 10768 '<FieldAccess for expr at ast.py:21325>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21325
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.While_Loop_Spec_F_Expr;
--# expr-done 10768
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 10769
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 10770 '<FieldAccess for bool_type at ast.py:21325>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21325
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 10770
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21325") else null)); 
--# expr-done 10771
--# expr-start 10773 '<FieldAccess for sub_equation at ast.py:21326>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21326
--# expr-start 10772 '<FieldAccess for expr at ast.py:21326>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21326
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.While_Loop_Spec_F_Expr, Info => Ent.Info);
--# expr-done 10772
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 10773
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21324") else null)); 
--# expr-done 10774
--# expr-start 10776 '<FieldAccess for matches_expected_formal_prim_type at ast.py:21327>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21327
--# expr-start 10775 '<FieldAccess for expr at ast.py:21327>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21327
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.While_Loop_Spec_F_Expr, Info => Ent.Info);
--# expr-done 10775
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 10776
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21324") else null)); 
--# expr-done 10777
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10778
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1457;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1457;
            raise;
      end;
   return Property_Result;
end While_Loop_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Mode
   --
--# property-start Mode.is_writable /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9672
pragma Warnings (Off, "is not referenced");
 function Mode_P_Is_Writable
  (Node : Bare_Mode
  )
   return Boolean
is
   Self : Bare_Mode  := Bare_Mode (Node);
      --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10780 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10779 '<IsA Mode.Out, Mode.InOut at ast.py:9678>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9678
Is_A := Self /= null 
and then Self.Kind in Ada_Mode_Out_Range | Ada_Mode_In_Out_Range; 
--# expr-done 10779
Let_Result := Is_A; 
--# end
--# expr-done 10780
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Mode_P_Is_Writable;
--# end
   --
   --  Primitives for Bare_Mode_Default
   --
   --
   --  Primitives for Bare_Mode_In
   --
   --
   --  Primitives for Bare_Mode_In_Out
   --
   --
   --  Primitives for Bare_Mode_Out
   --
   --
   --  Primitives for Bare_Multi_Abstract_State_Decl
   --
      procedure Initialize_Fields_For_Multi_Abstract_State_Decl
        (Self : Bare_Multi_Abstract_State_Decl
         ; Multi_Abstract_State_Decl_F_Decls : Bare_Abstract_State_Decl_List
        ) is
      begin
            Self.Multi_Abstract_State_Decl_F_Decls := Multi_Abstract_State_Decl_F_Decls;
      end Initialize_Fields_For_Multi_Abstract_State_Decl;
   function Multi_Abstract_State_Decl_F_Decls
     (Node : Bare_Multi_Abstract_State_Decl) return Bare_Abstract_State_Decl_List
   is
   begin
         return Node.Multi_Abstract_State_Decl_F_Decls;
   end;
   --
   --  Primitives for Bare_Not_Null
   --
--# property-start '[dispatcher]NotNull.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Not_Null_P_As_Bool
  (Node : Bare_Not_Null
  )
   return Boolean
is
   Self : Bare_Not_Null  := Bare_Not_Null (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Not_Null (Self.Kind) is
               when Ada_Not_Null_Absent =>
                  --# property-call-start NotNull.Absent.as_bool
                  Property_Result := Not_Null_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Not_Null_Present =>
                  --# property-call-start NotNull.Present.as_bool
                  Property_Result := Not_Null_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Not_Null_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Not_Null_Absent
   --
--# property-start NotNull.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5292
pragma Warnings (Off, "is not referenced");
 function Not_Null_Absent_P_As_Bool
  (Node : Bare_Not_Null_Absent
  )
   return Boolean
is
   Self : Bare_Not_Null_Absent  := Bare_Not_Null_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Not_Null_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Not_Null_Present
   --
--# property-start NotNull.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5292
pragma Warnings (Off, "is not referenced");
 function Not_Null_Present_P_As_Bool
  (Node : Bare_Not_Null_Present
  )
   return Boolean
is
   Self : Bare_Not_Null_Present  := Bare_Not_Null_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Not_Null_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Null_Component_Decl
   --
   --
   --  Primitives for Bare_Others_Designator
   --
--# property-start OthersDesignator.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:16143
pragma Warnings (Off, "is not referenced");
 function Others_Designator_P_Xref_Equation
  (Node : Bare_Others_Designator
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Others_Designator  := Bare_Others_Designator (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1135 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1135 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1135;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:16143") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1135;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1135;
            raise;
      end;
   return Property_Result;
end Others_Designator_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Overriding_Node
   --
   --
   --  Primitives for Bare_Overriding_Not_Overriding
   --
   --
   --  Primitives for Bare_Overriding_Overriding
   --
   --
   --  Primitives for Bare_Overriding_Unspecified
   --
   --
   --  Primitives for Bare_Params
   --
      procedure Initialize_Fields_For_Params
        (Self : Bare_Params
         ; Params_F_Params : Bare_Param_Spec_List
        ) is
      begin
            Self.Params_F_Params := Params_F_Params;
      end Initialize_Fields_For_Params;
   function Params_F_Params
     (Node : Bare_Params) return Bare_Param_Spec_List
   is
   begin
         return Node.Params_F_Params;
   end;
   --
   --  Primitives for Bare_Paren_Abstract_State_Decl
   --
      procedure Initialize_Fields_For_Paren_Abstract_State_Decl
        (Self : Bare_Paren_Abstract_State_Decl
         ; Paren_Abstract_State_Decl_F_Decl : Bare_Ada_Node
        ) is
      begin
            Self.Paren_Abstract_State_Decl_F_Decl := Paren_Abstract_State_Decl_F_Decl;
      end Initialize_Fields_For_Paren_Abstract_State_Decl;
   function Paren_Abstract_State_Decl_F_Decl
     (Node : Bare_Paren_Abstract_State_Decl) return Bare_Ada_Node
   is
   begin
         return Node.Paren_Abstract_State_Decl_F_Decl;
   end;
   --
   --  Primitives for Bare_Pp_Directive
   --
   --
   --  Primitives for Bare_Pp_Else_Directive
   --
   --
   --  Primitives for Bare_Pp_Elsif_Directive
   --
      procedure Initialize_Fields_For_Pp_Elsif_Directive
        (Self : Bare_Pp_Elsif_Directive
         ; Pp_Elsif_Directive_F_Expr : Bare_Expr
         ; Pp_Elsif_Directive_F_Then_Kw : Bare_Pp_Then_Kw
        ) is
      begin
            Self.Pp_Elsif_Directive_F_Expr := Pp_Elsif_Directive_F_Expr;
            Self.Pp_Elsif_Directive_F_Then_Kw := Pp_Elsif_Directive_F_Then_Kw;
      end Initialize_Fields_For_Pp_Elsif_Directive;
   function Pp_Elsif_Directive_F_Expr
     (Node : Bare_Pp_Elsif_Directive) return Bare_Expr
   is
   begin
         return Node.Pp_Elsif_Directive_F_Expr;
   end;
   function Pp_Elsif_Directive_F_Then_Kw
     (Node : Bare_Pp_Elsif_Directive) return Bare_Pp_Then_Kw
   is
   begin
         return Node.Pp_Elsif_Directive_F_Then_Kw;
   end;
   --
   --  Primitives for Bare_Pp_End_If_Directive
   --
   --
   --  Primitives for Bare_Pp_If_Directive
   --
      procedure Initialize_Fields_For_Pp_If_Directive
        (Self : Bare_Pp_If_Directive
         ; Pp_If_Directive_F_Expr : Bare_Expr
         ; Pp_If_Directive_F_Then_Kw : Bare_Pp_Then_Kw
        ) is
      begin
            Self.Pp_If_Directive_F_Expr := Pp_If_Directive_F_Expr;
            Self.Pp_If_Directive_F_Then_Kw := Pp_If_Directive_F_Then_Kw;
      end Initialize_Fields_For_Pp_If_Directive;
   function Pp_If_Directive_F_Expr
     (Node : Bare_Pp_If_Directive) return Bare_Expr
   is
   begin
         return Node.Pp_If_Directive_F_Expr;
   end;
   function Pp_If_Directive_F_Then_Kw
     (Node : Bare_Pp_If_Directive) return Bare_Pp_Then_Kw
   is
   begin
         return Node.Pp_If_Directive_F_Then_Kw;
   end;
   --
   --  Primitives for Bare_Pp_Then_Kw
   --
   --
   --  Primitives for Bare_Pragma_Node
   --
      procedure Pragma_Pre_Env_Actions
        (Self            : Bare_Pragma_Node;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_36 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:10295");
      end;
      end;
      procedure Initialize_Fields_For_Pragma_Node
        (Self : Bare_Pragma_Node
         ; Pragma_Node_F_Id : Bare_Identifier
         ; Pragma_Node_F_Args : Bare_Base_Assoc_List
        ) is
      begin
            Self.Pragma_Node_F_Id := Pragma_Node_F_Id;
            Self.Pragma_Node_F_Args := Pragma_Node_F_Args;
      end Initialize_Fields_For_Pragma_Node;
   function Pragma_Node_F_Id
     (Node : Bare_Pragma_Node) return Bare_Identifier
   is
   begin
         return Node.Pragma_Node_F_Id;
   end;
   function Pragma_Node_F_Args
     (Node : Bare_Pragma_Node) return Bare_Base_Assoc_List
   is
   begin
         return Node.Pragma_Node_F_Args;
   end;
--# property-start Pragma.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9949
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Entry_Point
  (Node : Bare_Pragma_Node
  )
   return Boolean
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Xref_Entry_Point;
--# end
--# property-start Pragma.is_ghost_code /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9951
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Is_Ghost_Code
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Identifier;
Fld_7 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10797 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10784 '<Eq at ast.py:9958>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
--# expr-start 10782 '<FieldAccess for sym at ast.py:9958>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
--# expr-start 10781 '<FieldAccess for id at ast.py:9958>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10781
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld.Node);
--# end
--# expr-done 10782
--# expr-start 10783 '<Symbol Assert at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Assert); 
--# expr-done 10783
Is_Equal := Fld_1 = Sym; 
--# expr-done 10784
if Is_Equal then
   If_Result := True;
else
   --# expr-start 10788 '<Eq at ast.py:9958>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
--# expr-start 10786 '<FieldAccess for sym at ast.py:9958>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
--# expr-start 10785 '<FieldAccess for id at ast.py:9958>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10785
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_3 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_2.Node);
--# end
--# expr-done 10786
--# expr-start 10787 '<Symbol Assert_And_Cut at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Assert_And_Cut); 
--# expr-done 10787
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 10788
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 10792 '<Eq at ast.py:9958>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
--# expr-start 10790 '<FieldAccess for sym at ast.py:9958>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
--# expr-start 10789 '<FieldAccess for id at ast.py:9958>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10789
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_5 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_4.Node);
--# end
--# expr-done 10790
--# expr-start 10791 '<Symbol Assume at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Assume); 
--# expr-done 10791
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 10792
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 10796 '<Eq at ast.py:9958>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
--# expr-start 10794 '<FieldAccess for sym at ast.py:9958>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
--# expr-start 10793 '<FieldAccess for id at ast.py:9958>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9958
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10793
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_7 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_6.Node);
--# end
--# expr-done 10794
--# expr-start 10795 '<Symbol Loop_Invariant at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Loop_Invariant); 
--# expr-done 10795
Is_Equal_3 := Fld_7 = Sym_3; 
--# expr-done 10796
   If_Result_2 := Is_Equal_3;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 10797
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Is_Ghost_Code;
--# end
--# property-start Pragma.xref_initial_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9962
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Initial_Env
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_794 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Identifier;
Fld_4 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_5 : Internal_Entity_Identifier;
Fld_6 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_7 : Internal_Entity_Identifier;
Fld_8 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_9 : Internal_Entity_Identifier;
Fld_10 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_11 : Internal_Entity_Identifier;
Fld_12 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Fld_13 : Internal_Entity_Identifier;
Fld_14 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Fld_15 : Internal_Entity_Identifier;
Fld_16 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Fld_17 : Internal_Entity_Identifier;
Fld_18 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Fld_19 : Internal_Entity_Identifier;
Fld_20 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Fld_21 : Internal_Entity_Identifier;
Fld_22 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
If_Result_9 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_794 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_794;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10843 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 10842 '<If at ast.py:9971>' If_Result_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9971
--# expr-start 10801 '<Eq at ast.py:9972>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10799 '<FieldAccess for name_symbol at ast.py:9972>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10798 '<FieldAccess for id at ast.py:9972>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10798
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_3.Node);
--# end
--# expr-done 10799
--# expr-start 10800 '<Symbol Pre at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 10800
Is_Equal := Fld_4 = Sym; 
--# expr-done 10801
if Is_Equal then
   If_Result := True;
else
   --# expr-start 10805 '<Eq at ast.py:9972>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10803 '<FieldAccess for name_symbol at ast.py:9972>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10802 '<FieldAccess for id at ast.py:9972>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10802
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_5.Node);
--# end
--# expr-done 10803
--# expr-start 10804 '<Symbol Post at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 10804
Is_Equal_1 := Fld_6 = Sym_1; 
--# expr-done 10805
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 10809 '<Eq at ast.py:9972>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10807 '<FieldAccess for name_symbol at ast.py:9972>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10806 '<FieldAccess for id at ast.py:9972>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10806
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_7.Node);
--# end
--# expr-done 10807
--# expr-start 10808 '<Symbol Pre'"'"'Class at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre_Class); 
--# expr-done 10808
Is_Equal_2 := Fld_8 = Sym_2; 
--# expr-done 10809
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 10813 '<Eq at ast.py:9972>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10811 '<FieldAccess for name_symbol at ast.py:9972>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10810 '<FieldAccess for id at ast.py:9972>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10810
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_9.Node);
--# end
--# expr-done 10811
--# expr-start 10812 '<Symbol Post'"'"'Class at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post_Class); 
--# expr-done 10812
Is_Equal_3 := Fld_10 = Sym_3; 
--# expr-done 10813
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 10817 '<Eq at ast.py:9972>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10815 '<FieldAccess for name_symbol at ast.py:9972>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10814 '<FieldAccess for id at ast.py:9972>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10814
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_11.Node);
--# end
--# expr-done 10815
--# expr-start 10816 '<Symbol Precondition at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition); 
--# expr-done 10816
Is_Equal_4 := Fld_12 = Sym_4; 
--# expr-done 10817
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 10821 '<Eq at ast.py:9972>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10819 '<FieldAccess for name_symbol at ast.py:9972>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10818 '<FieldAccess for id at ast.py:9972>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10818
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_13.Node);
--# end
--# expr-done 10819
--# expr-start 10820 '<Symbol Postcondition at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition); 
--# expr-done 10820
Is_Equal_5 := Fld_14 = Sym_5; 
--# expr-done 10821
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 10825 '<Eq at ast.py:9972>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10823 '<FieldAccess for name_symbol at ast.py:9972>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10822 '<FieldAccess for id at ast.py:9972>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10822
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_15.Node);
--# end
--# expr-done 10823
--# expr-start 10824 '<Symbol Precondition'"'"'Class at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition_Class); 
--# expr-done 10824
Is_Equal_6 := Fld_16 = Sym_6; 
--# expr-done 10825
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 10829 '<Eq at ast.py:9972>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10827 '<FieldAccess for name_symbol at ast.py:9972>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10826 '<FieldAccess for id at ast.py:9972>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10826
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_17.Node);
--# end
--# expr-done 10827
--# expr-start 10828 '<Symbol Postcondition'"'"'Class at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition_Class); 
--# expr-done 10828
Is_Equal_7 := Fld_18 = Sym_7; 
--# expr-done 10829
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 10833 '<Eq at ast.py:9972>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10831 '<FieldAccess for name_symbol at ast.py:9972>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10830 '<FieldAccess for id at ast.py:9972>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10830
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_19.Node);
--# end
--# expr-done 10831
--# expr-start 10832 '<Symbol Test_Case at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 10832
Is_Equal_8 := Fld_20 = Sym_8; 
--# expr-done 10833
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 10837 '<Eq at ast.py:9972>' Is_Equal_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10835 '<FieldAccess for name_symbol at ast.py:9972>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
--# expr-start 10834 '<FieldAccess for id at ast.py:9972>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9972
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10834
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_22 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_21.Node);
--# end
--# expr-done 10835
--# expr-start 10836 '<Symbol Contract_Cases at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 10836
Is_Equal_9 := Fld_22 = Sym_9; 
--# expr-done 10837
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   --# expr-start 10840 '<FieldAccess for children_env at ast.py:9978>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9978
--# expr-start 10839 '<at at ast.py:9978>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9978
--# expr-start 10838 '<FieldAccess for associated_entities at ast.py:9978>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9978
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entities
Fld := Libadalang.Implementation.Pragma_Node_P_Associated_Entities (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10838
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 10839
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# expr-done 10840
   If_Result_9 := Fld_1;
else
   --# expr-start 10841 '<FieldAccess for children_env at ast.py:9979>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9979
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 10841
   If_Result_9 := Fld_2;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 10842
Let_Result := If_Result_9; Inc_Ref (Let_Result);
--# end
--# expr-done 10843
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_794;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_794;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Xref_Initial_Env;
--# end
--# property-start Pragma.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9982
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Equation
  (Node : Bare_Pragma_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_795 with Inline_Always;
            procedure Finalizer_Scope_3271 with Inline_Always;
            procedure Finalizer_Scope_3272 with Inline_Always;
            procedure Finalizer_Scope_3273 with Inline_Always;
            procedure Finalizer_Scope_3275 with Inline_Always;
            procedure Finalizer_Scope_3276 with Inline_Always;
            procedure Finalizer_Scope_3280 with Inline_Always;
            procedure Finalizer_Scope_3281 with Inline_Always;
            procedure Finalizer_Scope_3283 with Inline_Always;
      Expr : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Assoc;
Var_Expr_2 : Internal_Entity_Expr;
Var_Expr_3 : Internal_Entity_Base_Assoc;
Var_Expr_4 : Internal_Entity_Expr;
Var_Expr_5 : Internal_Entity_Pragma_Argument_Assoc;
Expr_1 : Internal_Entity_Expr;
Proc : Internal_Entity_Expr;
Var_Expr_6 : Internal_Entity_Base_Assoc;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Identifier;
Sym_1 : Symbol_Type;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Sym_2 : Symbol_Type;
Fld_5 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Identifier;
Sym_3 : Symbol_Type;
Fld_7 : Boolean;
If_Result_2 : Boolean;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Fld_13 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_14 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_15 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
Fld_16 : Internal_Entity_Base_Assoc_List;
Fld_17 : Bare_Base_Assoc_List;
Fld_18 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity_Base_Assoc;
Fld_19 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Fld_20 : Logic_Var;
Sym_4 : Symbol_Type;
Fld_21 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_22 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_23 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_24 : Internal_Entity_Identifier;
Sym_5 : Symbol_Type;
Fld_25 : Boolean;
Fld_26 : Internal_Entity_Base_Assoc_List;
Fld_27 : Bare_Base_Assoc_List;
Fld_28 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_5 : Internal_Entity_Base_Assoc;
Fld_29 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_30 : Logic_Equation;
Item_335 : Internal_Entity_Base_Assoc;
Bare_Item_335 : Bare_Base_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_31 : Internal_Entity_Identifier;
Fld_32 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal : Boolean;
Fld_33 : Internal_Entity_Identifier;
Fld_34 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_6 : Boolean;
Fld_35 : Internal_Entity_Identifier;
Fld_36 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_7 : Boolean;
Fld_37 : Internal_Entity_Identifier;
Fld_38 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_8 : Boolean;
Fld_39 : Internal_Entity_Identifier;
Fld_40 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_9 : Boolean;
Fld_41 : Internal_Entity_Identifier;
Fld_42 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_10 : Boolean;
Fld_43 : Internal_Entity_Identifier;
Fld_44 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_11 : Boolean;
Fld_45 : Internal_Entity_Identifier;
Fld_46 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_12 : Boolean;
Fld_47 : Internal_Entity_Identifier;
Fld_48 : Symbol_Type;
Sym_14 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_13 : Boolean;
Fld_49 : Internal_Entity_Identifier;
Fld_50 : Symbol_Type;
Sym_15 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_14 : Boolean;
Fld_51 : Internal_Entity_Name_Array_Access;
Fld_52 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_53 : Internal_Entity_Identifier;
Sym_16 : Symbol_Type;
Fld_54 : Boolean;
Fld_55 : Internal_Entity_Base_Assoc_List;
Fld_56 : Bare_Base_Assoc_List;
Fld_57 : Internal_Entity_Info;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_3 : Internal_Entity_Base_Assoc;
Is_Null_3 : Boolean;
If_Result_15 : Internal_Entity_Base_Assoc;
Fld_58 : Internal_Entity_Expr;
Scope_Result_2 : Internal_Entity_Expr;
Result_Var_2 : Internal_Entity_Expr;
Fld_59 : Logic_Equation;
Fld_60 : Logic_Var;
Sym_17 : Symbol_Type;
Fld_61 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_62 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_63 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Is_A : Boolean;
If_Result_16 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_3 : Logic_Equation;
Fld_64 : Internal_Entity_Identifier;
Fld_65 : Symbol_Type;
Sym_18 : Symbol_Type;
Is_Equal_10 : Boolean;
Fld_66 : Internal_Entity_Identifier;
Fld_67 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_17 : Boolean;
Fld_68 : Internal_Entity_Identifier;
Fld_69 : Symbol_Type;
Sym_20 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_18 : Boolean;
True_Rel_2 : Logic_Equation;
Fld_70 : Internal_Entity_Identifier;
Fld_71 : Symbol_Type;
Sym_21 : Symbol_Type;
Is_Equal_13 : Boolean;
Fld_72 : Internal_Entity_Identifier;
Fld_73 : Symbol_Type;
Sym_22 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_19 : Boolean;
Fld_74 : Internal_Entity_Identifier;
Fld_75 : Symbol_Type;
Sym_23 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_20 : Boolean;
Fld_76 : Internal_Entity_Identifier;
Fld_77 : Symbol_Type;
Sym_24 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_21 : Boolean;
Fld_78 : Internal_Entity_Identifier;
Fld_79 : Symbol_Type;
Sym_25 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_22 : Boolean;
Fld_80 : Internal_Entity_Identifier;
Fld_81 : Symbol_Type;
Sym_26 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_23 : Boolean;
Fld_82 : Internal_Entity_Identifier;
Fld_83 : Symbol_Type;
Sym_27 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_24 : Boolean;
Fld_84 : Internal_Entity_Identifier;
Fld_85 : Symbol_Type;
Sym_28 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_25 : Boolean;
Fld_86 : Internal_Entity_Identifier;
Fld_87 : Symbol_Type;
Sym_29 : Symbol_Type;
Is_Equal_21 : Boolean;
If_Result_26 : Boolean;
Fld_88 : Internal_Entity_Base_Assoc_List;
Fld_89 : Bare_Base_Assoc_List;
Fld_90 : Internal_Entity_Info;
Get_Result_3 : Bare_Base_Assoc;
As_Entity_4 : Internal_Entity_Base_Assoc;
Is_Null_4 : Boolean;
If_Result_27 : Internal_Entity_Base_Assoc;
Fld_91 : Internal_Entity_Expr;
Fld_92 : Logic_Var;
Fld_93 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
Fld_94 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_95 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_4 : Logic_Equation;
Fld_96 : Internal_Entity_Identifier;
Sym_30 : Symbol_Type;
Fld_97 : Boolean;
Fld_98 : Internal_Entity_Base_Assoc_List;
Fld_99 : Bare_Base_Assoc_List;
Fld_100 : Internal_Entity_Info;
As_Entity_5 : Internal_Entity_Base_Assoc;
Is_Null_5 : Boolean;
If_Result_28 : Internal_Entity_Base_Assoc;
Item_338 : Internal_Entity_Base_Assoc;
Bare_Item_338 : Bare_Base_Assoc;
Cast_Expr : Internal_Entity_Base_Assoc;
Cast_Result_3 : Internal_Entity_Pragma_Argument_Assoc;
Fld_101 : Internal_Entity_Name;
Fld_102 : Symbol_Type;
Sym_31 : Symbol_Type;
Is_Equal_22 : Boolean;
Fld_103 : Internal_Entity_Name;
Fld_104 : Symbol_Type;
Sym_32 : Symbol_Type;
Is_Equal_23 : Boolean;
If_Result_29 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Map_Result_2 : Internal_Entity_Base_Assoc_Array_Access;
Fld_105 : Internal_Entity_Expr;
Fld_106 : Logic_Var;
Fld_107 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
Fld_108 : Internal_Entity_Expr;
Fld_109 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_110 : Internal_Entity_Expr;
Fld_111 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Map_Result_3 : Logic_Equation_Array_Access;
Logic_Boolean_Op_2 : Logic_Equation;
Fld_112 : Internal_Entity_Identifier;
Sym_33 : Symbol_Type;
Fld_113 : Boolean;
Fld_114 : Internal_Entity_Base_Assoc_List;
Fld_115 : Bare_Base_Assoc_List;
Fld_116 : Internal_Entity_Info;
Get_Result_4 : Bare_Base_Assoc;
As_Entity_6 : Internal_Entity_Base_Assoc;
Is_Null_6 : Boolean;
If_Result_30 : Internal_Entity_Base_Assoc;
Fld_117 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Base_Aggregate;
Fld_118 : Internal_Entity_Assoc_List;
Fld_119 : Bare_Assoc_List;
Fld_120 : Internal_Entity_Info;
As_Entity_7 : Internal_Entity_Basic_Assoc;
Is_Null_7 : Boolean;
If_Result_31 : Internal_Entity_Basic_Assoc;
Cast_Expr_1 : Internal_Entity_Basic_Assoc;
Cast_Result_6 : Internal_Entity_Aggregate_Assoc;
Fld_121 : Logic_Equation;
Item_339 : Internal_Entity_Basic_Assoc;
Bare_Item_339 : Bare_Basic_Assoc;
Map_Result_4 : Logic_Equation_Array_Access;
Logic_Boolean_Op_3 : Logic_Equation;
Fld_122 : Internal_Entity_Identifier;
Sym_34 : Symbol_Type;
Fld_123 : Boolean;
Fld_124 : Internal_Entity_Base_Assoc_List;
Fld_125 : Bare_Base_Assoc_List;
Fld_126 : Internal_Entity_Info;
Get_Result_5 : Bare_Base_Assoc;
As_Entity_8 : Internal_Entity_Base_Assoc;
Is_Null_8 : Boolean;
If_Result_32 : Internal_Entity_Base_Assoc;
Fld_127 : Internal_Entity_Expr;
Fld_128 : Logic_Var;
Fld_129 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
Fld_130 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Fld_131 : Logic_Equation;
And_Pred_12 : Logic_Equation;
Let_Result_1 : Logic_Equation;
True_Rel_3 : Logic_Equation;
Fld_132 : Internal_Entity_Base_Assoc_List;
Fld_133 : Bare_Base_Assoc_List;
Len : Integer;
Is_Equal_24 : Boolean;
If_Result_33 : Logic_Equation;
Fld_134 : Internal_Entity_Base_Assoc_List;
Fld_135 : Bare_Base_Assoc_List;
Len_1 : Integer;
Arith_Result : Integer;
Fld_136 : Internal_Entity_Base_Assoc_List;
Fld_137 : Bare_Base_Assoc_List;
Fld_138 : Internal_Entity_Info;
Get_Result_6 : Bare_Base_Assoc;
As_Entity_9 : Internal_Entity_Base_Assoc;
Is_Null_9 : Boolean;
If_Result_34 : Internal_Entity_Base_Assoc;
Fld_139 : Internal_Entity_Expr;
Scope_Result_6 : Internal_Entity_Expr;
Result_Var_6 : Internal_Entity_Expr;
Fld_140 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Bind_Result_6 : Logic_Equation;
Fld_141 : Logic_Equation;
And_Pred_13 : Logic_Equation;
Let_Result_2 : Logic_Equation;
And_Pred_14 : Logic_Equation;
Fld_142 : Internal_Entity_Base_Assoc_List;
Fld_143 : Bare_Base_Assoc_List;
Fld_144 : Internal_Entity_Info;
As_Entity_10 : Internal_Entity_Base_Assoc;
Is_Null_10 : Boolean;
If_Result_35 : Internal_Entity_Base_Assoc;
Fld_145 : Internal_Entity_Expr;
Fld_146 : Logic_Equation;
True_Rel_4 : Logic_Equation;
Or_Pred : Logic_Equation;
Item_340 : Internal_Entity_Base_Assoc;
Bare_Item_340 : Bare_Base_Assoc;
Map_Result_5 : Logic_Equation_Array_Access;
Logic_Boolean_Op_4 : Logic_Equation;
If_Result_36 : Logic_Equation;
If_Result_37 : Logic_Equation;
If_Result_38 : Logic_Equation;
If_Result_39 : Logic_Equation;
If_Result_40 : Logic_Equation;
If_Result_41 : Logic_Equation;
If_Result_42 : Logic_Equation;
If_Result_43 : Logic_Equation;
If_Result_44 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_795 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_51);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Logic_Boolean_Op_2);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Logic_Boolean_Op_3);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (Fld_130);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Fld_131);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (True_Rel_3);
                     Dec_Ref (If_Result_33);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Fld_141);
                     Dec_Ref (And_Pred_13);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (And_Pred_14);
                     Dec_Ref (Map_Result_5);
                     Dec_Ref (Logic_Boolean_Op_4);
                     Dec_Ref (If_Result_36);
                     Dec_Ref (If_Result_37);
                     Dec_Ref (If_Result_38);
                     Dec_Ref (If_Result_39);
                     Dec_Ref (If_Result_40);
                     Dec_Ref (If_Result_41);
                     Dec_Ref (If_Result_42);
                     Dec_Ref (If_Result_43);
                     Dec_Ref (If_Result_44);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_795;
            procedure Finalizer_Scope_3271 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3271;
            procedure Finalizer_Scope_3272 is
            begin
                     Dec_Ref (Fld_30);
            end Finalizer_Scope_3272;
            procedure Finalizer_Scope_3273 is
            begin
                     Dec_Ref (Fld_52);
            end Finalizer_Scope_3273;
            procedure Finalizer_Scope_3275 is
            begin
                     Dec_Ref (Fld_59);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_62);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_63);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (If_Result_16);
            end Finalizer_Scope_3275;
            procedure Finalizer_Scope_3276 is
            begin
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_94);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_95);
                     Dec_Ref (And_Pred_8);
            end Finalizer_Scope_3276;
            procedure Finalizer_Scope_3280 is
            begin
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Fld_109);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (Fld_111);
                     Dec_Ref (And_Pred_10);
            end Finalizer_Scope_3280;
            procedure Finalizer_Scope_3281 is
            begin
                     Dec_Ref (Fld_121);
            end Finalizer_Scope_3281;
            procedure Finalizer_Scope_3283 is
            begin
                     Dec_Ref (Fld_146);
                     Dec_Ref (True_Rel_4);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_3283;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11075 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 11074 '<Cond at ast.py:9984>' If_Result_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9984
--# expr-start 10846 '<FieldAccess for name_is at ast.py:9986>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9986
--# expr-start 10844 '<FieldAccess for id at ast.py:9986>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9986
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10844
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10845 '<Symbol Assert at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Assert); 
--# expr-done 10845
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 10846
if Fld_1 then
   If_Result := True;
else
   --# expr-start 10849 '<FieldAccess for name_is at ast.py:9987>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9987
--# expr-start 10847 '<FieldAccess for id at ast.py:9987>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9987
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10847
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10848 '<Symbol Loop_Invariant at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Loop_Invariant); 
--# expr-done 10848
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_2.Node, Sym => Sym_1);
--# end
--# expr-done 10849
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 10852 '<FieldAccess for name_is at ast.py:9988>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9988
--# expr-start 10850 '<FieldAccess for id at ast.py:9988>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9988
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10850
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10851 '<Symbol Compile_Time_Warning at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compile_Time_Warning); 
--# expr-done 10851
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_4.Node, Sym => Sym_2);
--# end
--# expr-done 10852
   If_Result_1 := Fld_5;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 10855 '<FieldAccess for name_is at ast.py:9989>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9989
--# expr-start 10853 '<FieldAccess for id at ast.py:9989>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9989
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10853
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10854 '<Symbol Compile_Time_Error at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compile_Time_Error); 
--# expr-done 10854
--# property-call-start Name.name_is
Fld_7 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_6.Node, Sym => Sym_3);
--# end
--# expr-done 10855
   If_Result_2 := Fld_7;
end if;
if If_Result_2 then
   --# expr-start 10876 '<And at ast.py:9991>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9991
--# expr-start 10865 '<Let at ast.py:9991>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9991
--# scope-start
--# expr-start 10857 '<FieldAccess for assoc_expr at ast.py:9992>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9992
--# expr-start 10856 '<FieldAccess for args at ast.py:9992>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9992
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 10856
Fld_9 := Fld_8.Node;
Get_Result := Get (Self, Fld_9, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_3 := As_Entity;
end if;
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 10857
Expr := Fld_11; 
--# bind expr Expr
--# expr-start 10864 '<And at ast.py:9993>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9993
--# expr-start 10862 '<And at ast.py:9993>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9993
--# expr-start 10860 '<Bind at ast.py:9993>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9993
--# expr-start 10858 '<FieldAccess for expected_type_var at ast.py:9993>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9993
      if Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Expr.Node);
--# expr-done 10858
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
--# expr-start 10859 '<FieldAccess for bool_type at ast.py:9993>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9993
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_13 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 10859
      Cast_Result := Create_Internal_Entity
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
Bind_Result := Solver.Create_Assign (Fld_12, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9993") else null)); 
--# expr-done 10860
--# expr-start 10861 '<FieldAccess for sub_equation at ast.py:9994>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9994
      if Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Expr.Node, Env => Env, Origin => Origin, E_Info => Expr.Info);
--# end
--# expr-done 10861
And_Pred := Create_And (Bind_Result, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9993") else null)); 
--# expr-done 10862
--# expr-start 10863 '<FieldAccess for matches_expected_formal_prim_type at ast.py:9995>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9995
      if Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_15 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Expr.Node, Origin => Origin);
--# end
--# expr-done 10863
And_Pred_1 := Create_And (And_Pred, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9993") else null)); 
--# expr-done 10864
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10865
--# expr-start 10866 '<FieldAccess for args at ast.py:9996>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9996
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 10866
Fld_17 := Fld_16.Node;
Get_Result_1 := Get (Self, Fld_17, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_4 := No_Entity_Base_Assoc;
else
Fld_18 := Fld_16.Info;
As_Entity_1 := (Info => Fld_18, Node => Get_Result_1); 
   If_Result_4 := As_Entity_1;
end if;
Var_Expr_1 := If_Result_4;
if Var_Expr_1 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 10867 '<FieldAccess for assoc_expr at ast.py:9996>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9996
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_19 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10867
Scope_Result := Fld_19;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind msg Var_Expr
--# expr-start 10875 '<And at ast.py:9998>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9998
--# expr-start 10873 '<And at ast.py:9998>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9998
--# expr-start 10871 '<Bind at ast.py:9998>' Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9998
--# expr-start 10868 '<FieldAccess for expected_type_var at ast.py:9998>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9998
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 10868
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
--# expr-start 10870 '<FieldAccess for std_entity at ast.py:9998>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9998
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10869 '<Symbol String at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 10869
--# property-call-start AdaNode.std_entity
Fld_21 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_4);
--# end
--# expr-done 10870
Bind_Result_1 := Solver.Create_Assign (Fld_20, Fld_21, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9998") else null)); 
--# expr-done 10871
--# expr-start 10872 '<FieldAccess for sub_equation at ast.py:9999>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9999
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 10872
And_Pred_2 := Create_And (Bind_Result_1, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9998") else null)); 
--# expr-done 10873
--# expr-start 10874 '<FieldAccess for matches_expected_type at ast.py:10000>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10000
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_23 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 10874
And_Pred_3 := Create_And (And_Pred_2, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9998") else null)); 
--# expr-done 10875
Scope_Result_1 := And_Pred_3;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3271;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10001") else null)); 
   Result_Var_1 := True_Rel;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_4 := Create_And (Let_Result, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9991") else null)); 
--# expr-done 10876
   If_Result_44 := And_Pred_4;
else
--# expr-start 10879 '<FieldAccess for name_is at ast.py:10004>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10004
--# expr-start 10877 '<FieldAccess for id at ast.py:10004>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10004
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10877
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10878 '<Symbol Unreferenced at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unreferenced); 
--# expr-done 10878
--# property-call-start Name.name_is
Fld_25 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_24.Node, Sym => Sym_5);
--# end
--# expr-done 10879
if Fld_25 then
   --# expr-start 10885 '<LogicAll at ast.py:10005>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10005
--# expr-start 10884 '<Map at ast.py:10005>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10005
   --# expr-start 10880 '<FieldAccess for args at ast.py:10005>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10005
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 10880
Fld_27 := Fld_26.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_27 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_27;
      begin
         for Untyped_Item_335 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_335 := Untyped_Item_335; 
Is_Null_2 := Bare_Item_335 = null; 
if Is_Null_2 then
   If_Result_5 := No_Entity_Base_Assoc;
else
Fld_28 := Fld_26.Info;
As_Entity_2 := (Info => Fld_28, Node => Bare_Item_335); 
   If_Result_5 := As_Entity_2;
end if;
                  Item_335 := If_Result_5; 
   --# scope-start
               --# bind assoc Item_335
      --# expr-start 10883 '<FieldAccess for xref_no_overloading at ast.py:10007>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10007
--# expr-start 10882 '<Cast to Name at ast.py:10007>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10007
--# expr-start 10881 '<FieldAccess for assoc_expr at ast.py:10007>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10007
      if Item_335.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_335.Node, E_Info => Item_335.Info);
--# end
--# expr-done 10881
   if Fld_29.Node = null
      or else Fld_29.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10882
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_30 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 10883
         declare
            Item_To_Append : constant Logic_Equation := Fld_30;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3272;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10884
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10005") else null)); 
--# expr-done 10885
   If_Result_43 := Logic_Boolean_Op;
else
--# expr-start 10889 '<Eq at ast.py:10010>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10887 '<FieldAccess for name_symbol at ast.py:10010>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10886 '<FieldAccess for id at ast.py:10010>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10886
      if Fld_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_32 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_31.Node);
--# end
--# expr-done 10887
--# expr-start 10888 '<Symbol Import at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 10888
Is_Equal := Fld_32 = Sym_6; 
--# expr-done 10889
if Is_Equal then
   If_Result_6 := True;
else
   --# expr-start 10893 '<Eq at ast.py:10010>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10891 '<FieldAccess for name_symbol at ast.py:10010>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10890 '<FieldAccess for id at ast.py:10010>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10890
      if Fld_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_34 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_33.Node);
--# end
--# expr-done 10891
--# expr-start 10892 '<Symbol Export at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Export); 
--# expr-done 10892
Is_Equal_1 := Fld_34 = Sym_7; 
--# expr-done 10893
   If_Result_6 := Is_Equal_1;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 10897 '<Eq at ast.py:10010>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10895 '<FieldAccess for name_symbol at ast.py:10010>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10894 '<FieldAccess for id at ast.py:10010>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10894
      if Fld_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_36 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_35.Node);
--# end
--# expr-done 10895
--# expr-start 10896 '<Symbol Interface at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface); 
--# expr-done 10896
Is_Equal_2 := Fld_36 = Sym_8; 
--# expr-done 10897
   If_Result_7 := Is_Equal_2;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 10901 '<Eq at ast.py:10010>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10899 '<FieldAccess for name_symbol at ast.py:10010>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10898 '<FieldAccess for id at ast.py:10010>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10898
      if Fld_37.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_38 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_37.Node);
--# end
--# expr-done 10899
--# expr-start 10900 '<Symbol Convention at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Convention); 
--# expr-done 10900
Is_Equal_3 := Fld_38 = Sym_9; 
--# expr-done 10901
   If_Result_8 := Is_Equal_3;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 10905 '<Eq at ast.py:10010>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10903 '<FieldAccess for name_symbol at ast.py:10010>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10902 '<FieldAccess for id at ast.py:10010>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10902
      if Fld_39.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_40 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_39.Node);
--# end
--# expr-done 10903
--# expr-start 10904 '<Symbol Pack at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pack); 
--# expr-done 10904
Is_Equal_4 := Fld_40 = Sym_10; 
--# expr-done 10905
   If_Result_9 := Is_Equal_4;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 10909 '<Eq at ast.py:10010>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10907 '<FieldAccess for name_symbol at ast.py:10010>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10906 '<FieldAccess for id at ast.py:10010>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_41 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10906
      if Fld_41.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_42 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_41.Node);
--# end
--# expr-done 10907
--# expr-start 10908 '<Symbol Pure at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pure); 
--# expr-done 10908
Is_Equal_5 := Fld_42 = Sym_11; 
--# expr-done 10909
   If_Result_10 := Is_Equal_5;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 10913 '<Eq at ast.py:10010>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10911 '<FieldAccess for name_symbol at ast.py:10010>' Fld_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10910 '<FieldAccess for id at ast.py:10010>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_43 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10910
      if Fld_43.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_44 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_43.Node);
--# end
--# expr-done 10911
--# expr-start 10912 '<Symbol Preelaborate at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Preelaborate); 
--# expr-done 10912
Is_Equal_6 := Fld_44 = Sym_12; 
--# expr-done 10913
   If_Result_11 := Is_Equal_6;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 10917 '<Eq at ast.py:10010>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10915 '<FieldAccess for name_symbol at ast.py:10010>' Fld_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10914 '<FieldAccess for id at ast.py:10010>' Fld_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_45 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10914
      if Fld_45.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_46 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_45.Node);
--# end
--# expr-done 10915
--# expr-start 10916 '<Symbol Elaborate_Body at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Elaborate_Body); 
--# expr-done 10916
Is_Equal_7 := Fld_46 = Sym_13; 
--# expr-done 10917
   If_Result_12 := Is_Equal_7;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 10921 '<Eq at ast.py:10010>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10919 '<FieldAccess for name_symbol at ast.py:10010>' Fld_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10918 '<FieldAccess for id at ast.py:10010>' Fld_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_47 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10918
      if Fld_47.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_48 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_47.Node);
--# end
--# expr-done 10919
--# expr-start 10920 '<Symbol Inline at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Inline); 
--# expr-done 10920
Is_Equal_8 := Fld_48 = Sym_14; 
--# expr-done 10921
   If_Result_13 := Is_Equal_8;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 10925 '<Eq at ast.py:10010>' Is_Equal_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10923 '<FieldAccess for name_symbol at ast.py:10010>' Fld_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
--# expr-start 10922 '<FieldAccess for id at ast.py:10010>' Fld_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10010
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_49 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10922
      if Fld_49.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_50 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_49.Node);
--# end
--# expr-done 10923
--# expr-start 10924 '<Symbol Volatile at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Volatile); 
--# expr-done 10924
Is_Equal_9 := Fld_50 = Sym_15; 
--# expr-done 10925
   If_Result_14 := Is_Equal_9;
end if;
if If_Result_14 then
   --# expr-start 10929 '<LogicAll at ast.py:10015>' Logic_Boolean_Op_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10015
--# expr-start 10928 '<Map at ast.py:10015>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10015
   --# expr-start 10926 '<FieldAccess for associated_entity_names at ast.py:10015>' Fld_51 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10015
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entity_names
Fld_51 := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10926
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Name_Array_Access := Fld_51;
      begin
         for Item_336 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_336
      --# expr-start 10927 '<FieldAccess for xref_no_overloading at ast.py:10016>' Fld_52 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10016
      if Item_336.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_52 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_336.Node, Env => Env, Origin => Origin, E_Info => Item_336.Info);
--# end
--# expr-done 10927
         declare
            Item_To_Append : constant Logic_Equation := Fld_52;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3273;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10928
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10015") else null)); 
--# expr-done 10929
   If_Result_42 := Logic_Boolean_Op_1;
else
--# expr-start 10932 '<FieldAccess for name_is at ast.py:10019>' Fld_54 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10019
--# expr-start 10930 '<FieldAccess for id at ast.py:10019>' Fld_53 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10019
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_53 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10930
      if Fld_53.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10931 '<Symbol Warnings at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Warnings); 
--# expr-done 10931
--# property-call-start Name.name_is
Fld_54 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_53.Node, Sym => Sym_16);
--# end
--# expr-done 10932
if Fld_54 then
--# expr-start 10933 '<FieldAccess for args at ast.py:10020>' Fld_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10020
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_55 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 10933
Fld_56 := Fld_55.Node;
Get_Result_2 := Get (Self, Fld_56, 1, True); 
Is_Null_3 := Get_Result_2 = null; 
if Is_Null_3 then
   If_Result_15 := No_Entity_Base_Assoc;
else
Fld_57 := Fld_55.Info;
As_Entity_3 := (Info => Fld_57, Node => Get_Result_2); 
   If_Result_15 := As_Entity_3;
end if;
Var_Expr_3 := If_Result_15;
if Var_Expr_3 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 10934 '<FieldAccess for assoc_expr at ast.py:10020>' Fld_58 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10020
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_58 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 10934
Scope_Result_2 := Fld_58;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Expr;
end if;
Var_Expr_2 := Result_Var_2;
if Var_Expr_2 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_2
--# expr-start 10945 '<If at ast.py:10021>' If_Result_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10021
--# expr-start 10935 '<IsA Identifier at ast.py:10022>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10022
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Identifier_Range; 
--# expr-done 10935
if Is_A then
   --# expr-start 10936 '<FieldAccess for sub_equation at ast.py:10024>' Fld_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10024
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_59 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_2.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 10936
   If_Result_16 := Fld_59;
else
   --# expr-start 10944 '<And at ast.py:10026>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10026
--# expr-start 10942 '<And at ast.py:10026>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10026
--# expr-start 10940 '<Bind at ast.py:10026>' Bind_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10026
--# expr-start 10937 '<FieldAccess for expected_type_var at ast.py:10026>' Fld_60 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10026
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_60 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_2.Node);
--# expr-done 10937
Fld_60.Value := No_Entity;
Entity_Vars.Reset (Fld_60);
--# expr-start 10939 '<FieldAccess for std_entity at ast.py:10026>' Fld_61 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10026
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10938 '<Symbol String at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 10938
--# property-call-start AdaNode.std_entity
Fld_61 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_17);
--# end
--# expr-done 10939
Bind_Result_2 := Solver.Create_Assign (Fld_60, Fld_61, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10026") else null)); 
--# expr-done 10940
--# expr-start 10941 '<FieldAccess for sub_equation at ast.py:10027>' Fld_62 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10027
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_62 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_2.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 10941
And_Pred_5 := Create_And (Bind_Result_2, Fld_62, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10026") else null)); 
--# expr-done 10942
--# expr-start 10943 '<FieldAccess for matches_expected_type at ast.py:10028>' Fld_63 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10028
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_63 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr_2.Node, Origin => Origin);
--# end
--# expr-done 10943
And_Pred_6 := Create_And (And_Pred_5, Fld_63, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10026") else null)); 
--# expr-done 10944
   If_Result_16 := And_Pred_6;
end if;
      Inc_Ref (If_Result_16);
--# expr-done 10945
Scope_Result_3 := If_Result_16;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3275;
   Result_Var_3 := Scope_Result_3;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10030") else null)); 
   Result_Var_3 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_3);
   If_Result_41 := Result_Var_3;
else
--# expr-start 10949 '<Eq at ast.py:10038>' Is_Equal_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
--# expr-start 10947 '<FieldAccess for name_symbol at ast.py:10038>' Fld_65 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
--# expr-start 10946 '<FieldAccess for id at ast.py:10038>' Fld_64 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_64 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10946
      if Fld_64.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_65 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_64.Node);
--# end
--# expr-done 10947
--# expr-start 10948 '<Symbol Style_Checks at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Style_Checks); 
--# expr-done 10948
Is_Equal_10 := Fld_65 = Sym_18; 
--# expr-done 10949
if Is_Equal_10 then
   If_Result_17 := True;
else
   --# expr-start 10953 '<Eq at ast.py:10038>' Is_Equal_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
--# expr-start 10951 '<FieldAccess for name_symbol at ast.py:10038>' Fld_67 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
--# expr-start 10950 '<FieldAccess for id at ast.py:10038>' Fld_66 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_66 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10950
      if Fld_66.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_67 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_66.Node);
--# end
--# expr-done 10951
--# expr-start 10952 '<Symbol Import_Function at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import_Function); 
--# expr-done 10952
Is_Equal_11 := Fld_67 = Sym_19; 
--# expr-done 10953
   If_Result_17 := Is_Equal_11;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 10957 '<Eq at ast.py:10038>' Is_Equal_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
--# expr-start 10955 '<FieldAccess for name_symbol at ast.py:10038>' Fld_69 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
--# expr-start 10954 '<FieldAccess for id at ast.py:10038>' Fld_68 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10038
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_68 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10954
      if Fld_68.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_69 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_68.Node);
--# end
--# expr-done 10955
--# expr-start 10956 '<Symbol Import_Procedure at ???>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import_Procedure); 
--# expr-done 10956
Is_Equal_12 := Fld_69 = Sym_20; 
--# expr-done 10957
   If_Result_18 := Is_Equal_12;
end if;
if If_Result_18 then
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10041") else null)); 
   If_Result_40 := True_Rel_2;
else
--# expr-start 10961 '<Eq at ast.py:10043>' Is_Equal_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10959 '<FieldAccess for name_symbol at ast.py:10043>' Fld_71 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10958 '<FieldAccess for id at ast.py:10043>' Fld_70 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_70 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10958
      if Fld_70.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_71 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_70.Node);
--# end
--# expr-done 10959
--# expr-start 10960 '<Symbol Pre at ???>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 10960
Is_Equal_13 := Fld_71 = Sym_21; 
--# expr-done 10961
if Is_Equal_13 then
   If_Result_19 := True;
else
   --# expr-start 10965 '<Eq at ast.py:10043>' Is_Equal_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10963 '<FieldAccess for name_symbol at ast.py:10043>' Fld_73 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10962 '<FieldAccess for id at ast.py:10043>' Fld_72 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_72 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10962
      if Fld_72.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_73 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_72.Node);
--# end
--# expr-done 10963
--# expr-start 10964 '<Symbol Post at ???>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 10964
Is_Equal_14 := Fld_73 = Sym_22; 
--# expr-done 10965
   If_Result_19 := Is_Equal_14;
end if;
if If_Result_19 then
   If_Result_20 := True;
else
   --# expr-start 10969 '<Eq at ast.py:10043>' Is_Equal_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10967 '<FieldAccess for name_symbol at ast.py:10043>' Fld_75 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10966 '<FieldAccess for id at ast.py:10043>' Fld_74 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_74 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10966
      if Fld_74.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_75 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_74.Node);
--# end
--# expr-done 10967
--# expr-start 10968 '<Symbol Pre'"'"'Class at ???>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre_Class); 
--# expr-done 10968
Is_Equal_15 := Fld_75 = Sym_23; 
--# expr-done 10969
   If_Result_20 := Is_Equal_15;
end if;
if If_Result_20 then
   If_Result_21 := True;
else
   --# expr-start 10973 '<Eq at ast.py:10043>' Is_Equal_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10971 '<FieldAccess for name_symbol at ast.py:10043>' Fld_77 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10970 '<FieldAccess for id at ast.py:10043>' Fld_76 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_76 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10970
      if Fld_76.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_77 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_76.Node);
--# end
--# expr-done 10971
--# expr-start 10972 '<Symbol Post'"'"'Class at ???>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post_Class); 
--# expr-done 10972
Is_Equal_16 := Fld_77 = Sym_24; 
--# expr-done 10973
   If_Result_21 := Is_Equal_16;
end if;
if If_Result_21 then
   If_Result_22 := True;
else
   --# expr-start 10977 '<Eq at ast.py:10043>' Is_Equal_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10975 '<FieldAccess for name_symbol at ast.py:10043>' Fld_79 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10974 '<FieldAccess for id at ast.py:10043>' Fld_78 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_78 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10974
      if Fld_78.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_79 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_78.Node);
--# end
--# expr-done 10975
--# expr-start 10976 '<Symbol Precondition at ???>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition); 
--# expr-done 10976
Is_Equal_17 := Fld_79 = Sym_25; 
--# expr-done 10977
   If_Result_22 := Is_Equal_17;
end if;
if If_Result_22 then
   If_Result_23 := True;
else
   --# expr-start 10981 '<Eq at ast.py:10043>' Is_Equal_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10979 '<FieldAccess for name_symbol at ast.py:10043>' Fld_81 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10978 '<FieldAccess for id at ast.py:10043>' Fld_80 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_80 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10978
      if Fld_80.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_81 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_80.Node);
--# end
--# expr-done 10979
--# expr-start 10980 '<Symbol Postcondition at ???>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition); 
--# expr-done 10980
Is_Equal_18 := Fld_81 = Sym_26; 
--# expr-done 10981
   If_Result_23 := Is_Equal_18;
end if;
if If_Result_23 then
   If_Result_24 := True;
else
   --# expr-start 10985 '<Eq at ast.py:10043>' Is_Equal_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10983 '<FieldAccess for name_symbol at ast.py:10043>' Fld_83 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10982 '<FieldAccess for id at ast.py:10043>' Fld_82 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_82 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10982
      if Fld_82.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_83 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_82.Node);
--# end
--# expr-done 10983
--# expr-start 10984 '<Symbol Precondition'"'"'Class at ???>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition_Class); 
--# expr-done 10984
Is_Equal_19 := Fld_83 = Sym_27; 
--# expr-done 10985
   If_Result_24 := Is_Equal_19;
end if;
if If_Result_24 then
   If_Result_25 := True;
else
   --# expr-start 10989 '<Eq at ast.py:10043>' Is_Equal_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10987 '<FieldAccess for name_symbol at ast.py:10043>' Fld_85 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10986 '<FieldAccess for id at ast.py:10043>' Fld_84 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_84 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10986
      if Fld_84.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_85 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_84.Node);
--# end
--# expr-done 10987
--# expr-start 10988 '<Symbol Postcondition'"'"'Class at ???>' Sym_28 None
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition_Class); 
--# expr-done 10988
Is_Equal_20 := Fld_85 = Sym_28; 
--# expr-done 10989
   If_Result_25 := Is_Equal_20;
end if;
if If_Result_25 then
   If_Result_26 := True;
else
   --# expr-start 10993 '<Eq at ast.py:10043>' Is_Equal_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10991 '<FieldAccess for name_symbol at ast.py:10043>' Fld_87 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
--# expr-start 10990 '<FieldAccess for id at ast.py:10043>' Fld_86 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10043
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_86 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 10990
      if Fld_86.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_87 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_86.Node);
--# end
--# expr-done 10991
--# expr-start 10992 '<Symbol Initial_Condition at ???>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Initial_Condition); 
--# expr-done 10992
Is_Equal_21 := Fld_87 = Sym_29; 
--# expr-done 10993
   If_Result_26 := Is_Equal_21;
end if;
if If_Result_26 then
--# expr-start 10995 '<FieldAccess for assoc_expr at ast.py:10049>' Fld_91 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10049
--# expr-start 10994 '<FieldAccess for args at ast.py:10049>' Fld_88 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10049
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_88 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 10994
Fld_89 := Fld_88.Node;
Get_Result_3 := Get (Self, Fld_89, 0, True); 
Is_Null_4 := Get_Result_3 = null; 
if Is_Null_4 then
   If_Result_27 := No_Entity_Base_Assoc;
else
Fld_90 := Fld_88.Info;
As_Entity_4 := (Info => Fld_90, Node => Get_Result_3); 
   If_Result_27 := As_Entity_4;
end if;
      if If_Result_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_91 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_27.Node, E_Info => If_Result_27.Info);
--# end
--# expr-done 10995
Var_Expr_4 := Fld_91;
if Var_Expr_4 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_4
--# expr-start 11002 '<And at ast.py:10051>' And_Pred_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10051
--# expr-start 11000 '<And at ast.py:10051>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10051
--# expr-start 10998 '<Bind at ast.py:10051>' Bind_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10051
--# expr-start 10996 '<FieldAccess for expected_type_var at ast.py:10051>' Fld_92 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10051
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_92 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_4.Node);
--# expr-done 10996
Fld_92.Value := No_Entity;
Entity_Vars.Reset (Fld_92);
--# expr-start 10997 '<FieldAccess for bool_type at ast.py:10051>' Fld_93 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10051
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_93 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 10997
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_93.Node,
         Info => Fld_93.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_92, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10051") else null)); 
--# expr-done 10998
--# expr-start 10999 '<FieldAccess for sub_equation at ast.py:10052>' Fld_94 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10052
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_94 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_4.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 10999
And_Pred_7 := Create_And (Bind_Result_3, Fld_94, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10051") else null)); 
--# expr-done 11000
--# expr-start 11001 '<FieldAccess for matches_expected_formal_prim_type at ast.py:10053>' Fld_95 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10053
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_95 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Var_Expr_4.Node, Origin => Origin);
--# end
--# expr-done 11001
And_Pred_8 := Create_And (And_Pred_7, Fld_95, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10051") else null)); 
--# expr-done 11002
Scope_Result_4 := And_Pred_8;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3276;
   Result_Var_4 := Scope_Result_4;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10054") else null)); 
   Result_Var_4 := False_Rel;
end if;
      Inc_Ref (Result_Var_4);
   If_Result_39 := Result_Var_4;
else
--# expr-start 11005 '<FieldAccess for name_is at ast.py:10057>' Fld_97 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10057
--# expr-start 11003 '<FieldAccess for id at ast.py:10057>' Fld_96 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10057
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_96 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11003
      if Fld_96.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11004 '<Symbol Test_Case at ???>' Sym_30 None
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 11004
--# property-call-start Name.name_is
Fld_97 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_96.Node, Sym => Sym_30);
--# end
--# expr-done 11005
if Fld_97 then
   --# expr-start 11028 '<LogicAll at ast.py:10064>' Logic_Boolean_Op_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10064
--# expr-start 11027 '<Map at ast.py:10064>' Map_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10064
   --# expr-start 11016 '<Filter at ast.py:10058>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10058
   --# expr-start 11006 '<FieldAccess for args at ast.py:10058>' Fld_98 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10058
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_98 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11006
Fld_99 := Fld_98.Node;
      declare
         Map_Result_2_Vec : Internal_Entity_Base_Assoc_Vectors.Vector;
      begin
      if Fld_99 = null then
      Map_Result_2 := Create_Internal_Entity_Base_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_99;
      begin
         for Untyped_Item_338 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_338 := Untyped_Item_338; 
Is_Null_5 := Bare_Item_338 = null; 
if Is_Null_5 then
   If_Result_28 := No_Entity_Base_Assoc;
else
Fld_100 := Fld_98.Info;
As_Entity_5 := (Info => Fld_100, Node => Bare_Item_338); 
   If_Result_28 := As_Entity_5;
end if;
                  Item_338 := If_Result_28; 
   --# scope-start
               --# bind arg Item_338
--# expr-start 11007 '<Cast to PragmaArgumentAssoc at ast.py:10059>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10059
Cast_Expr := Item_338; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Argument_Assoc_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Pragma_Argument_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Pragma_Argument_Assoc;
   end if;
--# expr-done 11007
Var_Expr_5 := Cast_Result_3;
if Var_Expr_5 /= No_Entity_Pragma_Argument_Assoc then
   --# scope-start
   --# bind parg Var_Expr_5
--# expr-start 11011 '<Eq at ast.py:10060>' Is_Equal_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10060
--# expr-start 11009 '<FieldAccess for name_symbol at ast.py:10060>' Fld_102 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10060
--# expr-start 11008 '<FieldAccess for name at ast.py:10060>' Fld_101 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10060
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_101 := Create_Internal_Entity_Name (Node => Var_Expr_5.Node.Pragma_Argument_Assoc_F_Name, Info => Var_Expr_5.Info);
--# expr-done 11008
      if Fld_101.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_102 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_101.Node);
--# end
--# expr-done 11009
--# expr-start 11010 '<Symbol Requires at ???>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Requires); 
--# expr-done 11010
Is_Equal_22 := Fld_102 = Sym_31; 
--# expr-done 11011
if Is_Equal_22 then
   If_Result_29 := True;
else
   --# expr-start 11015 '<Eq at ast.py:10060>' Is_Equal_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10060
--# expr-start 11013 '<FieldAccess for name_symbol at ast.py:10060>' Fld_104 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10060
--# expr-start 11012 '<FieldAccess for name at ast.py:10060>' Fld_103 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10060
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_103 := Create_Internal_Entity_Name (Node => Var_Expr_5.Node.Pragma_Argument_Assoc_F_Name, Info => Var_Expr_5.Info);
--# expr-done 11012
      if Fld_103.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_104 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_103.Node);
--# end
--# expr-done 11013
--# expr-start 11014 '<Symbol Ensures at ???>' Sym_32 None
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ensures); 
--# expr-done 11014
Is_Equal_23 := Fld_104 = Sym_32; 
--# expr-done 11015
   If_Result_29 := Is_Equal_23;
end if;
Scope_Result_5 := If_Result_29;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
         if Result_Var_5 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Assoc := Item_338;
         begin
            Internal_Entity_Base_Assoc_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Internal_Entity_Base_Assoc_Array
           (Items_Count => Natural (Internal_Entity_Base_Assoc_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Base_Assoc_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Base_Assoc_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Base_Assoc_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 11016
      declare
         Map_Result_3_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Assoc_Array_Access := Map_Result_2;
      begin
         for Item_337 of
               Collection.Items
         loop
   --# scope-start
               --# bind arg Item_337
      --# expr-start 11026 '<And at ast.py:10066>' And_Pred_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10066
--# expr-start 11023 '<And at ast.py:10066>' And_Pred_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10066
--# expr-start 11020 '<Bind at ast.py:10066>' Bind_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10066
--# expr-start 11018 '<FieldAccess for expected_type_var at ast.py:10066>' Fld_106 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10066
--# expr-start 11017 '<FieldAccess for assoc_expr at ast.py:10066>' Fld_105 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10066
      if Item_337.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_105 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_337.Node, E_Info => Item_337.Info);
--# end
--# expr-done 11017
      if Fld_105.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_106 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_105.Node);
--# expr-done 11018
Fld_106.Value := No_Entity;
Entity_Vars.Reset (Fld_106);
--# expr-start 11019 '<FieldAccess for bool_type at ast.py:10066>' Fld_107 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10066
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_107 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11019
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_107.Node,
         Info => Fld_107.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_106, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10066") else null)); 
--# expr-done 11020
--# expr-start 11022 '<FieldAccess for sub_equation at ast.py:10067>' Fld_109 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10067
--# expr-start 11021 '<FieldAccess for assoc_expr at ast.py:10067>' Fld_108 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10067
      if Item_337.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_108 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_337.Node, E_Info => Item_337.Info);
--# end
--# expr-done 11021
      if Fld_108.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_109 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_108.Node, Env => Env, Origin => Origin, E_Info => Fld_108.Info);
--# end
--# expr-done 11022
And_Pred_9 := Create_And (Bind_Result_4, Fld_109, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10066") else null)); 
--# expr-done 11023
--# expr-start 11025 '<FieldAccess for matches_expected_formal_prim_type at ast.py:10068>' Fld_111 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10068
--# expr-start 11024 '<FieldAccess for assoc_expr at ast.py:10068>' Fld_110 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10068
      if Item_337.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_110 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_337.Node, E_Info => Item_337.Info);
--# end
--# expr-done 11024
      if Fld_110.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_111 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_110.Node, Origin => Origin);
--# end
--# expr-done 11025
And_Pred_10 := Create_And (And_Pred_9, Fld_111, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10066") else null)); 
--# expr-done 11026
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_10;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3280;
         end loop;
      end;
         Map_Result_3 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_3_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 11027
Logic_Boolean_Op_2 := Solver.Create_All (Relation_Array (Map_Result_3.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10064") else null)); 
--# expr-done 11028
   If_Result_38 := Logic_Boolean_Op_2;
else
--# expr-start 11031 '<FieldAccess for name_is at ast.py:10071>' Fld_113 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10071
--# expr-start 11029 '<FieldAccess for id at ast.py:10071>' Fld_112 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10071
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_112 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11029
      if Fld_112.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11030 '<Symbol Contract_Cases at ???>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 11030
--# property-call-start Name.name_is
Fld_113 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_112.Node, Sym => Sym_33);
--# end
--# expr-done 11031
if Fld_113 then
   --# expr-start 11039 '<LogicAll at ast.py:10072>' Logic_Boolean_Op_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10072
--# expr-start 11038 '<Map at ast.py:10072>' Map_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10072
   --# expr-start 11035 '<FieldAccess for assocs at ast.py:10072>' Fld_118 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10072
--# expr-start 11034 '<Cast to BaseAggregate at ast.py:10072>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10072
--# expr-start 11033 '<FieldAccess for assoc_expr at ast.py:10072>' Fld_117 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10072
--# expr-start 11032 '<FieldAccess for args at ast.py:10072>' Fld_114 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10072
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_114 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11032
Fld_115 := Fld_114.Node;
Get_Result_4 := Get (Self, Fld_115, 0, True); 
Is_Null_6 := Get_Result_4 = null; 
if Is_Null_6 then
   If_Result_30 := No_Entity_Base_Assoc;
else
Fld_116 := Fld_114.Info;
As_Entity_6 := (Info => Fld_116, Node => Get_Result_4); 
   If_Result_30 := As_Entity_6;
end if;
      if If_Result_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_117 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_30.Node, E_Info => If_Result_30.Info);
--# end
--# expr-done 11033
   if Fld_117.Node = null
      or else Fld_117.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result_5 := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_117.Node,
         Info => Fld_117.Info);
   else
         Cast_Result_5 := No_Entity_Base_Aggregate;
   end if;
--# expr-done 11034
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_118 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_5.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_5.Info);
--# expr-done 11035
Fld_119 := Fld_118.Node;
      declare
         Map_Result_4_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_119 = null then
      Map_Result_4 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_119;
      begin
         for Untyped_Item_339 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_339 := Untyped_Item_339; 
Is_Null_7 := Bare_Item_339 = null; 
if Is_Null_7 then
   If_Result_31 := No_Entity_Basic_Assoc;
else
Fld_120 := Fld_118.Info;
As_Entity_7 := (Info => Fld_120, Node => Bare_Item_339); 
   If_Result_31 := As_Entity_7;
end if;
                  Item_339 := If_Result_31; 
   --# scope-start
               --# bind assoc Item_339
      --# expr-start 11037 '<FieldAccess for contract_cases_assoc_equation at ast.py:10074>' Fld_121 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10074
--# expr-start 11036 '<Cast to AggregateAssoc at ast.py:10074>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10074
Cast_Expr_1 := Item_339; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Aggregate_Assoc_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Aggregate_Assoc
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_6 := No_Entity_Aggregate_Assoc;
   end if;
--# expr-done 11036
      if Cast_Result_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.contract_cases_assoc_equation
Fld_121 := Libadalang.Implementation.Aggregate_Assoc_P_Contract_Cases_Assoc_Equation (Node => Cast_Result_6.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_6.Info);
--# end
--# expr-done 11037
         declare
            Item_To_Append : constant Logic_Equation := Fld_121;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3281;
         end loop;
      end;
      end if;
         Map_Result_4 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_4_Vec)));
         for I in Map_Result_4.Items'Range loop
            Map_Result_4.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_4_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_4.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_4_Vec);
      end;
--# expr-done 11038
Logic_Boolean_Op_3 := Solver.Create_All (Relation_Array (Map_Result_4.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10072") else null)); 
--# expr-done 11039
   If_Result_37 := Logic_Boolean_Op_3;
else
--# expr-start 11042 '<FieldAccess for name_is at ast.py:10077>' Fld_123 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10077
--# expr-start 11040 '<FieldAccess for id at ast.py:10077>' Fld_122 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10077
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_122 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11040
      if Fld_122.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11041 '<Symbol Debug at ???>' Sym_34 None
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Debug); 
--# expr-done 11041
--# property-call-start Name.name_is
Fld_123 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_122.Node, Sym => Sym_34);
--# end
--# expr-done 11042
if Fld_123 then
   --# expr-start 11067 '<And at ast.py:10078>' And_Pred_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10078
--# expr-start 11056 '<If at ast.py:10078>' If_Result_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10078
--# expr-start 11045 '<Eq at ast.py:10081>' Is_Equal_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10081
--# expr-start 11044 '<length at ast.py:10081>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10081
--# expr-start 11043 '<FieldAccess for args at ast.py:10081>' Fld_132 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10081
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_132 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11043
Fld_133 := Fld_132.Node; 
Len := Length (Fld_133); 
--# expr-done 11044
Is_Equal_24 := Len = 2; 
--# expr-done 11045
if Is_Equal_24 then
   --# expr-start 11055 '<Let at ast.py:10082>' Let_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10082
--# scope-start
--# expr-start 11047 '<FieldAccess for assoc_expr at ast.py:10083>' Fld_127 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10083
--# expr-start 11046 '<FieldAccess for args at ast.py:10083>' Fld_124 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10083
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_124 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11046
Fld_125 := Fld_124.Node;
Get_Result_5 := Get (Self, Fld_125, 0, True); 
Is_Null_8 := Get_Result_5 = null; 
if Is_Null_8 then
   If_Result_32 := No_Entity_Base_Assoc;
else
Fld_126 := Fld_124.Info;
As_Entity_8 := (Info => Fld_126, Node => Get_Result_5); 
   If_Result_32 := As_Entity_8;
end if;
      if If_Result_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_127 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_32.Node, E_Info => If_Result_32.Info);
--# end
--# expr-done 11047
Expr_1 := Fld_127; 
--# bind expr Expr_1
--# expr-start 11054 '<And at ast.py:10084>' And_Pred_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10084
--# expr-start 11052 '<And at ast.py:10084>' And_Pred_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10084
--# expr-start 11050 '<Bind at ast.py:10084>' Bind_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10084
--# expr-start 11048 '<FieldAccess for expected_type_var at ast.py:10084>' Fld_128 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10084
      if Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_128 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Expr_1.Node);
--# expr-done 11048
Fld_128.Value := No_Entity;
Entity_Vars.Reset (Fld_128);
--# expr-start 11049 '<FieldAccess for bool_type at ast.py:10084>' Fld_129 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10084
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_129 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11049
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_129.Node,
         Info => Fld_129.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_128, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10084") else null)); 
--# expr-done 11050
--# expr-start 11051 '<FieldAccess for sub_equation at ast.py:10085>' Fld_130 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10085
      if Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_130 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Expr_1.Node, Env => Env, Origin => Origin, E_Info => Expr_1.Info);
--# end
--# expr-done 11051
And_Pred_11 := Create_And (Bind_Result_5, Fld_130, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10084") else null)); 
--# expr-done 11052
--# expr-start 11053 '<FieldAccess for matches_expected_formal_prim_type at ast.py:10086>' Fld_131 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10086
      if Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_131 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Expr_1.Node, Origin => Origin);
--# end
--# expr-done 11053
And_Pred_12 := Create_And (And_Pred_11, Fld_131, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10084") else null)); 
--# expr-done 11054
Let_Result_1 := And_Pred_12; Inc_Ref (Let_Result_1);
--# end
--# expr-done 11055
   If_Result_33 := Let_Result_1;
else
   True_Rel_3 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10088") else null)); 
   If_Result_33 := True_Rel_3;
end if;
      Inc_Ref (If_Result_33);
--# expr-done 11056
--# expr-start 11066 '<Let at ast.py:10089>' Let_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10089
--# scope-start
--# expr-start 11057 '<FieldAccess for args at ast.py:10091>' Fld_136 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10091
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_136 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11057
Fld_137 := Fld_136.Node;
--# expr-start 11060 '<Arithmetic - at ast.py:10091>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10091
--# expr-start 11059 '<length at ast.py:10091>' Len_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10091
--# expr-start 11058 '<FieldAccess for args at ast.py:10091>' Fld_134 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10091
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_134 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11058
Fld_135 := Fld_134.Node; 
Len_1 := Length (Fld_135); 
--# expr-done 11059
Arith_Result := (Len_1 - 1); 
--# expr-done 11060
Get_Result_6 := Get (Self, Fld_137, Arith_Result, True); 
Is_Null_9 := Get_Result_6 = null; 
if Is_Null_9 then
   If_Result_34 := No_Entity_Base_Assoc;
else
Fld_138 := Fld_136.Info;
As_Entity_9 := (Info => Fld_138, Node => Get_Result_6); 
   If_Result_34 := As_Entity_9;
end if;
Var_Expr_6 := If_Result_34;
if Var_Expr_6 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11061 '<FieldAccess for assoc_expr at ast.py:10091>' Fld_139 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10091
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_139 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 11061
Scope_Result_6 := Fld_139;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Expr;
end if;
Proc := Result_Var_6; 
--# bind proc Proc
--# expr-start 11065 '<And at ast.py:10092>' And_Pred_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10092
--# expr-start 11063 '<Bind at ast.py:10092>' Bind_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10092
--# expr-start 11062 '<FieldAccess for type_var at ast.py:10092>' Fld_140 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10092
      if Proc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_140 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Proc.Node);
--# expr-done 11062
Fld_140.Value := No_Entity;
Entity_Vars.Reset (Fld_140);
Cast_Expr_2 := No_Entity_Base_Type_Decl; 
      Cast_Result_8 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_6 := Solver.Create_Assign (Fld_140, Cast_Result_8, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10092") else null)); 
--# expr-done 11063
--# expr-start 11064 '<FieldAccess for sub_equation at ast.py:10093>' Fld_141 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10093
      if Proc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_141 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Proc.Node, Env => Env, Origin => Origin, E_Info => Proc.Info);
--# end
--# expr-done 11064
And_Pred_13 := Create_And (Bind_Result_6, Fld_141, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10092") else null)); 
--# expr-done 11065
Let_Result_2 := And_Pred_13; Inc_Ref (Let_Result_2);
--# end
--# expr-done 11066
And_Pred_14 := Create_And (If_Result_33, Let_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10078") else null)); 
--# expr-done 11067
   If_Result_36 := And_Pred_14;
else
   --# expr-start 11073 '<LogicAll at ast.py:10096>' Logic_Boolean_Op_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10096
--# expr-start 11072 '<Map at ast.py:10096>' Map_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10096
   --# expr-start 11068 '<FieldAccess for args at ast.py:10096>' Fld_142 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10096
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_142 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11068
Fld_143 := Fld_142.Node;
      declare
         Map_Result_5_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_143 = null then
      Map_Result_5 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_143;
      begin
         for Untyped_Item_340 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_340 := Untyped_Item_340; 
Is_Null_10 := Bare_Item_340 = null; 
if Is_Null_10 then
   If_Result_35 := No_Entity_Base_Assoc;
else
Fld_144 := Fld_142.Info;
As_Entity_10 := (Info => Fld_144, Node => Bare_Item_340); 
   If_Result_35 := As_Entity_10;
end if;
                  Item_340 := If_Result_35; 
   --# scope-start
               --# bind a Item_340
      --# expr-start 11071 '<Or at ast.py:10100>' Or_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10100
--# expr-start 11070 '<FieldAccess for sub_equation at ast.py:10100>' Fld_146 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10100
--# expr-start 11069 '<FieldAccess for assoc_expr at ast.py:10100>' Fld_145 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10100
      if Item_340.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_145 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_340.Node, E_Info => Item_340.Info);
--# end
--# expr-done 11069
      if Fld_145.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_146 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_145.Node, Env => Env, Origin => Origin, E_Info => Fld_145.Info);
--# end
--# expr-done 11070
True_Rel_4 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10100") else null)); 
Or_Pred := Create_Or (Fld_146, True_Rel_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10100") else null)); 
--# expr-done 11071
         declare
            Item_To_Append : constant Logic_Equation := Or_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_5_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3283;
         end loop;
      end;
      end if;
         Map_Result_5 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_5_Vec)));
         for I in Map_Result_5.Items'Range loop
            Map_Result_5.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_5_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_5.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_5_Vec);
      end;
--# expr-done 11072
Logic_Boolean_Op_4 := Solver.Create_All (Relation_Array (Map_Result_5.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:10096") else null)); 
--# expr-done 11073
   If_Result_36 := Logic_Boolean_Op_4;
end if;
      Inc_Ref (If_Result_36);
   If_Result_37 := If_Result_36;
end if;
      Inc_Ref (If_Result_37);
   If_Result_38 := If_Result_37;
end if;
      Inc_Ref (If_Result_38);
   If_Result_39 := If_Result_38;
end if;
      Inc_Ref (If_Result_39);
   If_Result_40 := If_Result_39;
end if;
      Inc_Ref (If_Result_40);
   If_Result_41 := If_Result_40;
end if;
      Inc_Ref (If_Result_41);
   If_Result_42 := If_Result_41;
end if;
      Inc_Ref (If_Result_42);
   If_Result_43 := If_Result_42;
end if;
      Inc_Ref (If_Result_43);
   If_Result_44 := If_Result_43;
end if;
      Inc_Ref (If_Result_44);
--# expr-done 11074
Let_Result_3 := If_Result_44; Inc_Ref (Let_Result_3);
--# end
--# expr-done 11075
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_795;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_795;
                     Finalizer_Scope_3271;
                     Finalizer_Scope_3272;
                     Finalizer_Scope_3273;
                     Finalizer_Scope_3275;
                     Finalizer_Scope_3276;
                     Finalizer_Scope_3280;
                     Finalizer_Scope_3281;
                     Finalizer_Scope_3283;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Xref_Equation;
--# end
--# property-start Pragma.value_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10104
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Value_Expr
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Var_Expr : Internal_Entity_Base_Assoc;
Var_Expr_1 : Internal_Entity_Base_Assoc_List;
Var_Expr_2 : Internal_Entity_Base_Assoc;
Var_Expr_3 : Internal_Entity_Base_Assoc_List;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Identifier;
Fld_7 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Internal_Entity_Identifier;
Sym_4 : Symbol_Type;
Fld_13 : Boolean;
Fld_14 : Internal_Entity_Base_Assoc_List;
Saved : Internal_Entity_Base_Assoc_List;
Fld_15 : Bare_Base_Assoc_List;
Fld_16 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity_Base_Assoc;
Scope_Result : Internal_Entity_Base_Assoc;
Result_Var : Internal_Entity_Base_Assoc;
Fld_17 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Fld_18 : Internal_Entity_Base_Assoc_List;
Saved_1 : Internal_Entity_Base_Assoc_List;
Fld_19 : Bare_Base_Assoc_List;
Fld_20 : Internal_Entity_Info;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_5 : Internal_Entity_Base_Assoc;
Scope_Result_2 : Internal_Entity_Base_Assoc;
Result_Var_2 : Internal_Entity_Base_Assoc;
Fld_21 : Internal_Entity_Expr;
Scope_Result_3 : Internal_Entity_Expr;
Result_Var_3 : Internal_Entity_Expr;
If_Result_6 : Internal_Entity_Expr;
If_Result_7 : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11102 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11101 '<Cond at ast.py:10116>' If_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10116
--# expr-start 11079 '<Eq at ast.py:10117>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
--# expr-start 11077 '<FieldAccess for name_symbol at ast.py:10117>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
--# expr-start 11076 '<FieldAccess for id at ast.py:10117>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11076
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 11077
--# expr-start 11078 '<Symbol Import at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 11078
Is_Equal := Fld_1 = Sym; 
--# expr-done 11079
if Is_Equal then
   If_Result := True;
else
   --# expr-start 11083 '<Eq at ast.py:10117>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
--# expr-start 11081 '<FieldAccess for name_symbol at ast.py:10117>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
--# expr-start 11080 '<FieldAccess for id at ast.py:10117>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11080
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 11081
--# expr-start 11082 '<Symbol Export at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Export); 
--# expr-done 11082
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 11083
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 11087 '<Eq at ast.py:10117>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
--# expr-start 11085 '<FieldAccess for name_symbol at ast.py:10117>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
--# expr-start 11084 '<FieldAccess for id at ast.py:10117>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11084
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4.Node);
--# end
--# expr-done 11085
--# expr-start 11086 '<Symbol Interface at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface); 
--# expr-done 11086
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 11087
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 11091 '<Eq at ast.py:10117>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
--# expr-start 11089 '<FieldAccess for name_symbol at ast.py:10117>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
--# expr-start 11088 '<FieldAccess for id at ast.py:10117>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10117
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11088
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6.Node);
--# end
--# expr-done 11089
--# expr-start 11090 '<Symbol Convention at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Convention); 
--# expr-done 11090
Is_Equal_3 := Fld_7 = Sym_3; 
--# expr-done 11091
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   --# expr-start 11093 '<FieldAccess for assoc_expr at ast.py:10120>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10120
--# expr-start 11092 '<FieldAccess for args at ast.py:10120>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10120
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11092
Fld_9 := Fld_8.Node;
Get_Result := Get (Self, Fld_9, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_3 := As_Entity;
end if;
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 11093
   If_Result_7 := Fld_11;
else
--# expr-start 11096 '<FieldAccess for name_is at ast.py:10125>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10125
--# expr-start 11094 '<FieldAccess for id at ast.py:10125>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10125
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11094
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11095 '<Symbol SPARK_Mode at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 11095
--# property-call-start Name.name_is
Fld_13 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_12.Node, Sym => Sym_4);
--# end
--# expr-done 11096
if Fld_13 then
--# expr-start 11097 '<FieldAccess for args at ast.py:10126>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10126
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11097
Var_Expr_1 := Fld_14;
if Var_Expr_1 /= No_Entity_Base_Assoc_List then
   --# scope-start
Saved := Var_Expr_1; 
Fld_15 := Saved.Node;
Get_Result_1 := Get (Self, Fld_15, 0, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_4 := No_Entity_Base_Assoc;
else
Fld_16 := Saved.Info;
As_Entity_1 := (Info => Fld_16, Node => Get_Result_1); 
   If_Result_4 := As_Entity_1;
end if;
Scope_Result := If_Result_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Assoc;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11098 '<FieldAccess for assoc_expr at ast.py:10126>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10126
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 11098
Scope_Result_1 := Fld_17;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
   If_Result_6 := Result_Var_1;
else
--# expr-start 11099 '<FieldAccess for args at ast.py:10128>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10128
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11099
Var_Expr_3 := Fld_18;
if Var_Expr_3 /= No_Entity_Base_Assoc_List then
   --# scope-start
Saved_1 := Var_Expr_3; 
Fld_19 := Saved_1.Node;
Get_Result_2 := Get (Self, Fld_19, 1, True); 
Is_Null_2 := Get_Result_2 = null; 
if Is_Null_2 then
   If_Result_5 := No_Entity_Base_Assoc;
else
Fld_20 := Saved_1.Info;
As_Entity_2 := (Info => Fld_20, Node => Get_Result_2); 
   If_Result_5 := As_Entity_2;
end if;
Scope_Result_2 := If_Result_5;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Assoc;
end if;
Var_Expr_2 := Result_Var_2;
if Var_Expr_2 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11100 '<FieldAccess for assoc_expr at ast.py:10128>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10128
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_21 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 11100
Scope_Result_3 := Fld_21;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Expr;
end if;
   If_Result_6 := Result_Var_3;
end if;
   If_Result_7 := If_Result_6;
end if;
--# expr-done 11101
Let_Result := If_Result_7; 
--# end
--# expr-done 11102
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Value_Expr;
--# end
--# property-start Pragma.spark_mode_is_on /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10131
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Spark_Mode_Is_On
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11110 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11105 '<FieldAccess for name_is at ast.py:10137>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10137
--# expr-start 11103 '<FieldAccess for id at ast.py:10137>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10137
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11103
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11104 '<Symbol SPARK_Mode at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 11104
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 11105
if Fld_1 then
--# expr-start 11106 '<FieldAccess for value_expr at ast.py:10138>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10138
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.value_expr
Fld_2 := Libadalang.Implementation.Pragma_Node_P_Value_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11106
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind mode Var_Expr
--# expr-start 11109 '<FieldAccess for name_is at ast.py:10140>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10140
--# expr-start 11107 '<Cast to Name at ast.py:10140>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10140
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 11107
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11108 '<Symbol On at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_On); 
--# expr-done 11108
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Cast_Result.Node, Sym => Sym_1);
--# end
--# expr-done 11109
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 11110
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Spark_Mode_Is_On;
--# end
--# property-start Pragma.associated_entity_names /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10146
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Entity_Names
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name_Array_Access
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name_Array_Access;
            procedure Finalizer_Scope_798 with Inline_Always;
            procedure Finalizer_Scope_3291 with Inline_Always;
            procedure Finalizer_Scope_3293 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Base_Assoc;
Var_Expr_2 : Internal_Entity_Name;
Var_Expr_3 : Internal_Entity_Base_Assoc;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Identifier;
Fld_7 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Singleton : Internal_Entity_Name_Array_Access;
Fld_12 : Internal_Entity_Identifier;
Sym_4 : Symbol_Type;
Fld_13 : Boolean;
Fld_14 : Internal_Entity_Base_Assoc_List;
Fld_15 : Bare_Base_Assoc_List;
Fld_16 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity_Base_Assoc;
Fld_17 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Item_341 : Internal_Entity_Base_Assoc;
Bare_Item_341 : Bare_Base_Assoc;
Map_Result : Internal_Entity_Name_Array_Access;
Fld_18 : Internal_Entity_Identifier;
Fld_19 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_4 : Boolean;
Fld_20 : Internal_Entity_Identifier;
Fld_21 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_5 : Boolean;
Fld_22 : Internal_Entity_Identifier;
Fld_23 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_6 : Boolean;
Fld_24 : Internal_Entity_Identifier;
Fld_25 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_7 : Boolean;
Fld_26 : Internal_Entity_Identifier;
Fld_27 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_8 : Boolean;
Fld_28 : Internal_Entity_Identifier;
Fld_29 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_9 : Boolean;
Fld_30 : Internal_Entity_Identifier;
Fld_31 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_10 : Boolean;
Fld_32 : Internal_Entity_Identifier;
Fld_33 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_11 : Boolean;
Fld_34 : Internal_Entity_Identifier;
Fld_35 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_12 : Boolean;
Fld_36 : Internal_Entity_Identifier;
Fld_37 : Symbol_Type;
Sym_14 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_13 : Boolean;
Fld_38 : Internal_Entity_Identifier;
Fld_39 : Symbol_Type;
Sym_15 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_14 : Boolean;
Fld_40 : Internal_Entity_Identifier;
Fld_41 : Symbol_Type;
Sym_16 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_15 : Boolean;
Fld_42 : Internal_Entity_Identifier;
Fld_43 : Symbol_Type;
Sym_17 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_16 : Boolean;
Fld_44 : Internal_Entity_Identifier;
Fld_45 : Symbol_Type;
Sym_18 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_17 : Boolean;
Fld_46 : Internal_Entity_Identifier;
Fld_47 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_18 : Boolean;
Fld_48 : Internal_Entity_Identifier;
Fld_49 : Symbol_Type;
Sym_20 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_19 : Boolean;
Fld_50 : Internal_Entity_Base_Assoc_List;
Fld_51 : Bare_Base_Assoc_List;
Fld_52 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_20 : Internal_Entity_Base_Assoc;
Fld_53 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Singleton_1 : Internal_Entity_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Name_Array_Access;
Result_Var_1 : Internal_Entity_Name_Array_Access;
Fld_54 : Internal_Entity_Identifier;
Sym_21 : Symbol_Type;
Fld_55 : Boolean;
Fld_56 : Internal_Entity_Base_Assoc_List;
Fld_57 : Bare_Base_Assoc_List;
Fld_58 : Internal_Entity_Info;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_3 : Internal_Entity_Base_Assoc;
Is_Null_3 : Boolean;
If_Result_21 : Internal_Entity_Base_Assoc;
Fld_59 : Internal_Entity_Expr;
Scope_Result_2 : Internal_Entity_Expr;
Result_Var_2 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Singleton_2 : Internal_Entity_Name_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
If_Result_22 : Internal_Entity_Name_Array_Access;
Scope_Result_3 : Internal_Entity_Name_Array_Access;
Result_Var_3 : Internal_Entity_Name_Array_Access;
If_Result_23 : Internal_Entity_Name_Array_Access;
If_Result_24 : Internal_Entity_Name_Array_Access;
If_Result_25 : Internal_Entity_Name_Array_Access;
If_Result_26 : Internal_Entity_Name_Array_Access;
Let_Result : Internal_Entity_Name_Array_Access;
            procedure Finalizer_Scope_798 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (If_Result_23);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (If_Result_25);
                     Dec_Ref (If_Result_26);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_798;
            procedure Finalizer_Scope_3291 is
            begin
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_3291;
            procedure Finalizer_Scope_3293 is
            begin
                     Dec_Ref (Singleton_2);
                     Dec_Ref (If_Result_22);
            end Finalizer_Scope_3293;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11214 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11213 '<Cond at ast.py:10148>' If_Result_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10148
--# expr-start 11114 '<Eq at ast.py:10149>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
--# expr-start 11112 '<FieldAccess for name_symbol at ast.py:10149>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
--# expr-start 11111 '<FieldAccess for id at ast.py:10149>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11111
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 11112
--# expr-start 11113 '<Symbol Import at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 11113
Is_Equal := Fld_1 = Sym; 
--# expr-done 11114
if Is_Equal then
   If_Result := True;
else
   --# expr-start 11118 '<Eq at ast.py:10149>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
--# expr-start 11116 '<FieldAccess for name_symbol at ast.py:10149>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
--# expr-start 11115 '<FieldAccess for id at ast.py:10149>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11115
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 11116
--# expr-start 11117 '<Symbol Export at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Export); 
--# expr-done 11117
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 11118
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 11122 '<Eq at ast.py:10149>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
--# expr-start 11120 '<FieldAccess for name_symbol at ast.py:10149>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
--# expr-start 11119 '<FieldAccess for id at ast.py:10149>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11119
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4.Node);
--# end
--# expr-done 11120
--# expr-start 11121 '<Symbol Interface at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface); 
--# expr-done 11121
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 11122
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 11126 '<Eq at ast.py:10149>' Is_Equal_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
--# expr-start 11124 '<FieldAccess for name_symbol at ast.py:10149>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
--# expr-start 11123 '<FieldAccess for id at ast.py:10149>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10149
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11123
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6.Node);
--# end
--# expr-done 11124
--# expr-start 11125 '<Symbol Convention at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Convention); 
--# expr-done 11125
Is_Equal_3 := Fld_7 = Sym_3; 
--# expr-done 11126
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   --# expr-start 11129 '<Cast to Name at ast.py:10152>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10152
--# expr-start 11128 '<FieldAccess for assoc_expr at ast.py:10152>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10152
--# expr-start 11127 '<FieldAccess for args at ast.py:10152>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10152
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11127
Fld_9 := Fld_8.Node;
Get_Result := Get (Self, Fld_9, 1, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_3 := As_Entity;
end if;
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 11128
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11129
                Singleton := Create_Internal_Entity_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
   If_Result_26 := Singleton;
else
--# expr-start 11132 '<FieldAccess for name_is at ast.py:10154>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10154
--# expr-start 11130 '<FieldAccess for id at ast.py:10154>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10154
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11130
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11131 '<Symbol Inline at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Inline); 
--# expr-done 11131
--# property-call-start Name.name_is
Fld_13 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_12.Node, Sym => Sym_4);
--# end
--# expr-done 11132
if Fld_13 then
   --# expr-start 11136 '<Map at ast.py:10155>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10155
   --# expr-start 11133 '<FieldAccess for args at ast.py:10155>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10155
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11133
Fld_15 := Fld_14.Node;
      declare
         Map_Result_Vec : Internal_Entity_Name_Vectors.Vector;
      begin
      if Fld_15 = null then
      Map_Result := Create_Internal_Entity_Name_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_15;
      begin
         for Untyped_Item_341 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_341 := Untyped_Item_341; 
Is_Null_1 := Bare_Item_341 = null; 
if Is_Null_1 then
   If_Result_4 := No_Entity_Base_Assoc;
else
Fld_16 := Fld_14.Info;
As_Entity_1 := (Info => Fld_16, Node => Bare_Item_341); 
   If_Result_4 := As_Entity_1;
end if;
                  Item_341 := If_Result_4; 
   --# scope-start
               --# bind a Item_341
      --# expr-start 11135 '<Cast to Name at ast.py:10155>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10155
--# expr-start 11134 '<FieldAccess for assoc_expr at ast.py:10155>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10155
      if Item_341.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_341.Node, E_Info => Item_341.Info);
--# end
--# expr-done 11134
   if Fld_17.Node = null
      or else Fld_17.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 11135
         declare
            Item_To_Append : constant Internal_Entity_Name := Cast_Result_1;
         begin
            Internal_Entity_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Name_Array
           (Items_Count => Natural (Internal_Entity_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11136
   If_Result_25 := Map_Result;
else
--# expr-start 11140 '<Eq at ast.py:10157>' Is_Equal_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11138 '<FieldAccess for name_symbol at ast.py:10157>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11137 '<FieldAccess for id at ast.py:10157>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11137
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_18.Node);
--# end
--# expr-done 11138
--# expr-start 11139 '<Symbol Pack at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pack); 
--# expr-done 11139
Is_Equal_4 := Fld_19 = Sym_5; 
--# expr-done 11140
if Is_Equal_4 then
   If_Result_5 := True;
else
   --# expr-start 11144 '<Eq at ast.py:10157>' Is_Equal_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11142 '<FieldAccess for name_symbol at ast.py:10157>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11141 '<FieldAccess for id at ast.py:10157>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11141
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_20.Node);
--# end
--# expr-done 11142
--# expr-start 11143 '<Symbol Pure at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pure); 
--# expr-done 11143
Is_Equal_5 := Fld_21 = Sym_6; 
--# expr-done 11144
   If_Result_5 := Is_Equal_5;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 11148 '<Eq at ast.py:10157>' Is_Equal_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11146 '<FieldAccess for name_symbol at ast.py:10157>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11145 '<FieldAccess for id at ast.py:10157>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11145
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_22.Node);
--# end
--# expr-done 11146
--# expr-start 11147 '<Symbol Preelaborate at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Preelaborate); 
--# expr-done 11147
Is_Equal_6 := Fld_23 = Sym_7; 
--# expr-done 11148
   If_Result_6 := Is_Equal_6;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 11152 '<Eq at ast.py:10157>' Is_Equal_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11150 '<FieldAccess for name_symbol at ast.py:10157>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11149 '<FieldAccess for id at ast.py:10157>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11149
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 11150
--# expr-start 11151 '<Symbol Elaborate_Body at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Elaborate_Body); 
--# expr-done 11151
Is_Equal_7 := Fld_25 = Sym_8; 
--# expr-done 11152
   If_Result_7 := Is_Equal_7;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 11156 '<Eq at ast.py:10157>' Is_Equal_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11154 '<FieldAccess for name_symbol at ast.py:10157>' Fld_27 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11153 '<FieldAccess for id at ast.py:10157>' Fld_26 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11153
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_26.Node);
--# end
--# expr-done 11154
--# expr-start 11155 '<Symbol Volatile at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Volatile); 
--# expr-done 11155
Is_Equal_8 := Fld_27 = Sym_9; 
--# expr-done 11156
   If_Result_8 := Is_Equal_8;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 11160 '<Eq at ast.py:10157>' Is_Equal_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11158 '<FieldAccess for name_symbol at ast.py:10157>' Fld_29 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11157 '<FieldAccess for id at ast.py:10157>' Fld_28 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11157
      if Fld_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_28.Node);
--# end
--# expr-done 11158
--# expr-start 11159 '<Symbol Volatile_Components at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Volatile_Components); 
--# expr-done 11159
Is_Equal_9 := Fld_29 = Sym_10; 
--# expr-done 11160
   If_Result_9 := Is_Equal_9;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 11164 '<Eq at ast.py:10157>' Is_Equal_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11162 '<FieldAccess for name_symbol at ast.py:10157>' Fld_31 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11161 '<FieldAccess for id at ast.py:10157>' Fld_30 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11161
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_31 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_30.Node);
--# end
--# expr-done 11162
--# expr-start 11163 '<Symbol Unchecked_Union at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unchecked_Union); 
--# expr-done 11163
Is_Equal_10 := Fld_31 = Sym_11; 
--# expr-done 11164
   If_Result_10 := Is_Equal_10;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 11168 '<Eq at ast.py:10157>' Is_Equal_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11166 '<FieldAccess for name_symbol at ast.py:10157>' Fld_33 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11165 '<FieldAccess for id at ast.py:10157>' Fld_32 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11165
      if Fld_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_33 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_32.Node);
--# end
--# expr-done 11166
--# expr-start 11167 '<Symbol Atomic at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Atomic); 
--# expr-done 11167
Is_Equal_11 := Fld_33 = Sym_12; 
--# expr-done 11168
   If_Result_11 := Is_Equal_11;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 11172 '<Eq at ast.py:10157>' Is_Equal_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11170 '<FieldAccess for name_symbol at ast.py:10157>' Fld_35 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11169 '<FieldAccess for id at ast.py:10157>' Fld_34 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11169
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_35 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_34.Node);
--# end
--# expr-done 11170
--# expr-start 11171 '<Symbol Atomic_Components at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Atomic_Components); 
--# expr-done 11171
Is_Equal_12 := Fld_35 = Sym_13; 
--# expr-done 11172
   If_Result_12 := Is_Equal_12;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 11176 '<Eq at ast.py:10157>' Is_Equal_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11174 '<FieldAccess for name_symbol at ast.py:10157>' Fld_37 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11173 '<FieldAccess for id at ast.py:10157>' Fld_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11173
      if Fld_36.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_37 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_36.Node);
--# end
--# expr-done 11174
--# expr-start 11175 '<Symbol No_Return at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_No_Return); 
--# expr-done 11175
Is_Equal_13 := Fld_37 = Sym_14; 
--# expr-done 11176
   If_Result_13 := Is_Equal_13;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 11180 '<Eq at ast.py:10157>' Is_Equal_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11178 '<FieldAccess for name_symbol at ast.py:10157>' Fld_39 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11177 '<FieldAccess for id at ast.py:10157>' Fld_38 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11177
      if Fld_38.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_39 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_38.Node);
--# end
--# expr-done 11178
--# expr-start 11179 '<Symbol Discard_Names at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Discard_Names); 
--# expr-done 11179
Is_Equal_14 := Fld_39 = Sym_15; 
--# expr-done 11180
   If_Result_14 := Is_Equal_14;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 11184 '<Eq at ast.py:10157>' Is_Equal_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11182 '<FieldAccess for name_symbol at ast.py:10157>' Fld_41 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11181 '<FieldAccess for id at ast.py:10157>' Fld_40 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11181
      if Fld_40.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_41 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_40.Node);
--# end
--# expr-done 11182
--# expr-start 11183 '<Symbol Independent at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Independent); 
--# expr-done 11183
Is_Equal_15 := Fld_41 = Sym_16; 
--# expr-done 11184
   If_Result_15 := Is_Equal_15;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 11188 '<Eq at ast.py:10157>' Is_Equal_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11186 '<FieldAccess for name_symbol at ast.py:10157>' Fld_43 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11185 '<FieldAccess for id at ast.py:10157>' Fld_42 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11185
      if Fld_42.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_43 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_42.Node);
--# end
--# expr-done 11186
--# expr-start 11187 '<Symbol Independent_Components at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Independent_Components); 
--# expr-done 11187
Is_Equal_16 := Fld_43 = Sym_17; 
--# expr-done 11188
   If_Result_16 := Is_Equal_16;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 11192 '<Eq at ast.py:10157>' Is_Equal_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11190 '<FieldAccess for name_symbol at ast.py:10157>' Fld_45 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11189 '<FieldAccess for id at ast.py:10157>' Fld_44 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_44 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11189
      if Fld_44.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_45 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_44.Node);
--# end
--# expr-done 11190
--# expr-start 11191 '<Symbol Asynchronous at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asynchronous); 
--# expr-done 11191
Is_Equal_17 := Fld_45 = Sym_18; 
--# expr-done 11192
   If_Result_17 := Is_Equal_17;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 11196 '<Eq at ast.py:10157>' Is_Equal_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11194 '<FieldAccess for name_symbol at ast.py:10157>' Fld_47 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11193 '<FieldAccess for id at ast.py:10157>' Fld_46 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_46 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11193
      if Fld_46.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_47 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_46.Node);
--# end
--# expr-done 11194
--# expr-start 11195 '<Symbol Interrupt_Handler at ???>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interrupt_Handler); 
--# expr-done 11195
Is_Equal_18 := Fld_47 = Sym_19; 
--# expr-done 11196
   If_Result_18 := Is_Equal_18;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 11200 '<Eq at ast.py:10157>' Is_Equal_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11198 '<FieldAccess for name_symbol at ast.py:10157>' Fld_49 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
--# expr-start 11197 '<FieldAccess for id at ast.py:10157>' Fld_48 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10157
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11197
      if Fld_48.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_49 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_48.Node);
--# end
--# expr-done 11198
--# expr-start 11199 '<Symbol Attach_Handler at ???>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Attach_Handler); 
--# expr-done 11199
Is_Equal_19 := Fld_49 = Sym_20; 
--# expr-done 11200
   If_Result_19 := Is_Equal_19;
end if;
if If_Result_19 then
--# expr-start 11203 '<Cast to Name at ast.py:10164>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10164
--# expr-start 11201 '<FieldAccess for args at ast.py:10164>' Fld_50 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10164
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11201
Fld_51 := Fld_50.Node;
Get_Result_1 := Get (Self, Fld_51, 0, True); 
Is_Null_2 := Get_Result_1 = null; 
if Is_Null_2 then
   If_Result_20 := No_Entity_Base_Assoc;
else
Fld_52 := Fld_50.Info;
As_Entity_2 := (Info => Fld_52, Node => Get_Result_1); 
   If_Result_20 := As_Entity_2;
end if;
Var_Expr_1 := If_Result_20;
if Var_Expr_1 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11202 '<FieldAccess for assoc_expr at ast.py:10164>' Fld_53 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10164
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_53 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11202
Scope_Result := Fld_53;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 11203
Var_Expr := Cast_Result_2;
if Var_Expr /= No_Entity_Name then
   --# scope-start
                Singleton_1 := Create_Internal_Entity_Name_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr;
Scope_Result_1 := Singleton_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3291;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_24 := Result_Var_1;
else
--# expr-start 11206 '<FieldAccess for name_is at ast.py:10166>' Fld_55 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10166
--# expr-start 11204 '<FieldAccess for id at ast.py:10166>' Fld_54 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10166
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_54 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11204
      if Fld_54.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11205 '<Symbol Obsolescent at ???>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Obsolescent); 
--# expr-done 11205
--# property-call-start Name.name_is
Fld_55 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_54.Node, Sym => Sym_21);
--# end
--# expr-done 11206
if Fld_55 then
--# expr-start 11209 '<Cast to Name at ast.py:10167>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10167
--# expr-start 11207 '<FieldAccess for args at ast.py:10167>' Fld_56 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10167
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_56 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11207
Fld_57 := Fld_56.Node;
Get_Result_2 := Get (Self, Fld_57, 0, True); 
Is_Null_3 := Get_Result_2 = null; 
if Is_Null_3 then
   If_Result_21 := No_Entity_Base_Assoc;
else
Fld_58 := Fld_56.Info;
As_Entity_3 := (Info => Fld_58, Node => Get_Result_2); 
   If_Result_21 := As_Entity_3;
end if;
Var_Expr_3 := If_Result_21;
if Var_Expr_3 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11208 '<FieldAccess for assoc_expr at ast.py:10167>' Fld_59 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10167
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_59 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 11208
Scope_Result_2 := Fld_59;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Expr;
end if;
   if Result_Var_2.Node = null
      or else Result_Var_2.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Result_Var_2.Node,
         Info => Result_Var_2.Info);
   else
         Cast_Result_3 := No_Entity_Name;
   end if;
--# expr-done 11209
Var_Expr_2 := Cast_Result_3;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_2
--# expr-start 11212 '<If at ast.py:10171>' If_Result_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10171
--# expr-start 11211 '<Not at ast.py:10172>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10172
--# expr-start 11210 '<IsA StringLiteral at ast.py:10172>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10172
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_String_Literal_Range; 
--# expr-done 11210
Not_Val := not (Is_A); 
--# expr-done 11211
if Not_Val then
                Singleton_2 := Create_Internal_Entity_Name_Array (Items_Count => 1);
                Singleton_2.Items (1) := Var_Expr_2;
   If_Result_22 := Singleton_2;
else
   If_Result_22 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (If_Result_22);
--# expr-done 11212
Scope_Result_3 := If_Result_22;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3293;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
   If_Result_23 := Result_Var_3;
else
   If_Result_23 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (If_Result_23);
   If_Result_24 := If_Result_23;
end if;
      Inc_Ref (If_Result_24);
   If_Result_25 := If_Result_24;
end if;
      Inc_Ref (If_Result_25);
   If_Result_26 := If_Result_25;
end if;
      Inc_Ref (If_Result_26);
--# expr-done 11213
Let_Result := If_Result_26; Inc_Ref (Let_Result);
--# end
--# expr-done 11214
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_798;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_798;
                     Finalizer_Scope_3291;
                     Finalizer_Scope_3293;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Associated_Entity_Names;
--# end
--# property-start Pragma.associated_entities_helper /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10181
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Entities_Helper
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_799 with Inline_Always;
            procedure Finalizer_Scope_3104 with Inline_Always;
            procedure Finalizer_Scope_3105 with Inline_Always;
            procedure Finalizer_Scope_3106 with Inline_Always;
      Var_Expr : Internal_Entity;
Fld : Internal_Entity_Name_Array_Access;
Fld_1 : Internal_Entity;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Defining_Name_Array_Access;
Fld_6 : Symbol_Type;
Fld_7 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_8 : Internal_Unit;
Fld_9 : Internal_Unit;
Is_Equal : Boolean;
Fld_10 : Bare_Defining_Name;
Node_Comp : Boolean;
If_Result : Boolean;
Map_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Map_Result_2 : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_799 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_799;
            procedure Finalizer_Scope_3104 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_3104;
            procedure Finalizer_Scope_3105 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3105;
            procedure Finalizer_Scope_3106 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_3106;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11234 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11233 '<Mapcat at ast.py:10183>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10183
   --# expr-start 11215 '<FieldAccess for associated_entity_names at ast.py:10183>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10183
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entity_names
Fld := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11215
      declare
         Map_Result_2_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Name_Array_Access := Fld;
      begin
         for Item_342 of
               Collection.Items
         loop
   --# scope-start
               --# bind name Item_342
      --# expr-start 11232 '<Filter at ast.py:10204>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10204
--# expr-start 11216 '<FieldAccess for semantic_parent at ast.py:10187>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10187
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11216
Var_Expr := Fld_1;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind parent Var_Expr
--# expr-start 11226 '<Map at ast.py:10194>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10194
   --# expr-start 11219 '<EnvGet at ast.py:10189>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10189
--# expr-start 11217 '<FieldAccess for children_env at ast.py:10189>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10189
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 11217
--# expr-start 11218 '<FieldAccess for name_symbol at ast.py:10190>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10190
      if Item_342.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_342.Node);
--# end
--# expr-done 11218
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_2, Key => Fld_3, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 11219
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_344 of
               Collection.Items
         loop
   --# scope-start
               --# bind node Item_344
      --# expr-start 11225 '<Find at ast.py:10197>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10197
   --# expr-start 11222 '<FieldAccess for defining_names at ast.py:10197>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10197
--# expr-start 11221 '<FieldAccess for wrap_public_reference at ast.py:10196>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10196
--# expr-start 11220 '<Cast to BasicDecl at ast.py:10196>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10196
Cast_Expr := Item_344; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 11220
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.wrap_public_reference
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Wrap_Public_Reference (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 11221
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 11222
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_5;
      begin
         for Item_345 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_345
      --# expr-start 11224 '<FieldAccess for name_is at ast.py:10198>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10198
      if Item_345.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11223 '<FieldAccess for name_symbol at ast.py:10198>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10198
      if Item_342.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_342.Node);
--# end
--# expr-done 11223
--# property-call-start Name.name_is
Fld_7 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_345.Node, Sym => Fld_6);
--# end
--# expr-done 11224
      if Fld_7 then
         Find_Result := Item_345; 
      end if;
   --# end
               exit when Fld_7;
         end loop;
      end;
--# expr-done 11225
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Find_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3106;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11226
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3105;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
      declare
         Map_Result_1_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Result_Var;
      begin
         for Item_343 of
               Collection.Items
         loop
   --# scope-start
               --# bind ent Item_343
--# expr-start 11229 '<Eq at ast.py:10204>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10204
--# expr-start 11227 '<FieldAccess for unit at ast.py:10204>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10204
      if Item_343.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Unit (Node => Item_343.Node);
--# expr-done 11227
--# expr-start 11228 '<FieldAccess for unit at ast.py:10204>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10204
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 11228
Is_Equal := Fld_8 = Fld_9; 
--# expr-done 11229
if Is_Equal then
   --# expr-start 11231 '<OrderingTest '"'"'lt'"'"' at ast.py:10204>' Node_Comp /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10204
--# expr-start 11230 '<FieldAccess for node at ast.py:10204>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10204
Fld_10 := Item_343.Node;
--# expr-done 11230
Node_Comp := Compare (Self, Fld_10, Self, Less_Than); 
--# expr-done 11231
   If_Result := Node_Comp;
else
   If_Result := False;
end if;
         if If_Result then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_343;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 11232
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3104;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 11233
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 11234
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_799;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_799;
                     Finalizer_Scope_3104;
                     Finalizer_Scope_3105;
                     Finalizer_Scope_3106;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Associated_Entities_Helper;
--# end
--# property-start Pragma.associated_entities /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10207
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Entities
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_800 with Inline_Always;
            procedure Finalizer_Scope_3101 with Inline_Always;
            procedure Finalizer_Scope_3103 with Inline_Always;
            procedure Finalizer_Scope_3109 with Inline_Always;
            procedure Finalizer_Scope_3110 with Inline_Always;
            procedure Finalizer_Scope_3111 with Inline_Always;
            procedure Finalizer_Scope_3112 with Inline_Always;
            procedure Finalizer_Scope_3114 with Inline_Always;
            procedure Finalizer_Scope_3115 with Inline_Always;
            procedure Finalizer_Scope_3116 with Inline_Always;
      Block_Var : Internal_Entity_Defining_Name_Array_Access;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Bare_Compilation_Unit;
Var_Expr_1 : Internal_Entity_Name_Array_Access;
Var_Expr_2 : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_3 : Bare_Ada_Node_List;
Var_Expr_4 : Bare_Ada_Node_List;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Bare_Ada_Node_Array_Access;
P : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_7 : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_8 : Internal_Entity_Basic_Decl;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Fld_3 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Name_Array_Access;
Fld_6 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_7 : Internal_Entity_Defining_Name;
Singleton_1 : Internal_Entity_Defining_Name_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Get_Result : Internal_Entity_Name;
Fld_8 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_2 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_2 : Internal_Entity_Defining_Name_Array_Access;
Is_Equal_1 : Boolean;
Not_Val : Boolean;
If_Result_4 : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_3 : Internal_Entity_Defining_Name_Array_Access;
Fld_9 : Bare_Ada_Node;
Cast_Result_5 : Bare_Ada_Node_List;
Item_348 : Bare_Ada_Node;
Is_A_1 : Boolean;
Node_Comp : Boolean;
If_Result_5 : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Len_1 : Integer;
Arith_Result : Integer;
Get_Result_1 : Bare_Ada_Node;
Scope_Result_4 : Bare_Ada_Node;
Result_Var_3 : Bare_Ada_Node;
Cast_Result_6 : Bare_Basic_Decl;
As_Entity_2 : Internal_Entity_Basic_Decl;
Is_Null_2 : Boolean;
If_Result_6 : Internal_Entity_Basic_Decl;
Singleton_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_5 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_6 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_5 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_7 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_6 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_8 : Internal_Entity_Basic_Decl_Array_Access;
Singleton_3 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_7 : Internal_Entity_Basic_Decl_Array_Access;
Singleton_4 : Internal_Entity_Basic_Decl_Array_Access;
Fld_10 : Internal_Entity_Identifier;
Fld_11 : Symbol_Type;
Fld_12 : Boolean;
Fld_13 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_14 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Internal_Entity_Basic_Decl_Array_Access;
Fld_15 : Internal_Entity_Defining_Name;
Map_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_8 : Internal_Entity_Defining_Name_Array_Access;
Let_Result_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_800 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Singleton_3);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (Singleton_4);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Result_Var_8);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_800;
            procedure Finalizer_Scope_3101 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_3101;
            procedure Finalizer_Scope_3103 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (P);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3103;
            procedure Finalizer_Scope_3109 is
            begin
                     Dec_Ref (Var_Expr_7);
            end Finalizer_Scope_3109;
            procedure Finalizer_Scope_3110 is
            begin
                     Dec_Ref (Singleton_1);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_3110;
            procedure Finalizer_Scope_3111 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_5);
            end Finalizer_Scope_3111;
            procedure Finalizer_Scope_3112 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_4);
            end Finalizer_Scope_3112;
            procedure Finalizer_Scope_3114 is
            begin
                     Dec_Ref (Var_Expr_6);
            end Finalizer_Scope_3114;
            procedure Finalizer_Scope_3115 is
            begin
                     Dec_Ref (Singleton_2);
            end Finalizer_Scope_3115;
            procedure Finalizer_Scope_3116 is
            begin
                     Dec_Ref (Var_Expr_2);
            end Finalizer_Scope_3116;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 11280 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 11237 '<Cast to CompilationUnit at ast.py:10213>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10213
--# expr-start 11236 '<parent at ast.py:10213>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10213
--# expr-start 11235 '<parent at ast.py:10213>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10213
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 11235
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 11236
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 11237
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind cu Var_Expr
--# expr-start 11242 '<FieldAccess for defining_name at ast.py:10215>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10215
--# expr-start 11241 '<as_entity at ast.py:10215>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10215
--# expr-start 11240 '<FieldAccess for item at ast.py:10215>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10215
--# expr-start 11239 '<Cast to LibraryItem at ast.py:10214>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10214
--# expr-start 11238 '<FieldAccess for body at ast.py:10214>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10214
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr.Compilation_Unit_F_Body;
--# expr-done 11238
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Library_Item_Range
   then
      Cast_Result_1 := Fld_2;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11239
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Cast_Result_1.Library_Item_F_Item;
--# expr-done 11240
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 11241
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 11242
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_4;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3101;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
Block_Var := Result_Var; Inc_Ref (Block_Var);
--# bind top_level_decl Block_Var
--# expr-start 11247 '<as_entity at ast.py:10221>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10221
--# expr-start 11246 '<Cast to BasicDecl at ast.py:10221>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10221
--# expr-start 11245 '<Find at ast.py:10219>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10219
   --# expr-start 11243 '<FieldAccess for parents at ast.py:10219>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10219
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, False); 
--# expr-done 11243
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_346 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_346
      --# expr-start 11244 '<IsA BasicDecl at ast.py:10220>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10220
Is_A := Item_346 /= null 
and then Item_346.Kind in Ada_Basic_Decl; 
--# expr-done 11244
      if Is_A then
         Find_Result := Item_346; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 11245
   if Find_Result = null
      or else Find_Result.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Find_Result;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 11246
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 11247
Block_Var_1 := If_Result_1; 
--# bind enclosing_program_unit Block_Var_1
--# expr-start 11248 '<FieldAccess for associated_entity_names at ast.py:10234>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10234
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entity_names
Fld_5 := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11248
Var_Expr_1 := Fld_5;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Name_Array_Type then
   --# scope-start
   --# bind names Var_Expr_1
--# expr-start 11260 '<Let at ast.py:10235>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10235
--# scope-start
--# expr-start 11249 '<FieldAccess for associated_entities_helper at ast.py:10236>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10236
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entities_helper
Fld_6 := Libadalang.Implementation.Pragma_Node_P_Associated_Entities_Helper (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11249
Var_Expr_7 := Fld_6;
      Inc_Ref (Var_Expr_7);
if Var_Expr_7 /= No_Internal_Entity_Defining_Name_Array_Type then
   --# scope-start
   --# bind e Var_Expr_7
Scope_Result_1 := Var_Expr_7;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3109;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var;
end if;
      Inc_Ref (Result_Var_1);
P := Result_Var_1; Inc_Ref (P);
--# bind p P
--# expr-start 11259 '<If at ast.py:10237>' If_Result_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10237
--# expr-start 11251 '<Not at ast.py:10238>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10238
--# expr-start 11250 '<Eq at ast.py:10238>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10238
Is_Equal_1 := Equivalent (P, No_Internal_Entity_Defining_Name_Array_Type); 
--# expr-done 11250
Not_Val := not (Is_Equal_1); 
--# expr-done 11251
if Not_Val then
   If_Result_4 := P;
else
Var_Expr_8 := Block_Var_1;
if Var_Expr_8 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind epu Var_Expr_8
--# expr-start 11258 '<If at ast.py:10240>' If_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10240
--# expr-start 11253 '<Eq at ast.py:10242>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10242
--# expr-start 11252 '<length at ast.py:10242>' Len /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10242
Len := Length (Var_Expr_1); 
--# expr-done 11252
Is_Equal := Len = 1; 
--# expr-done 11253
if Is_Equal then
   --# expr-start 11256 '<Eq at ast.py:10243>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10243
--# expr-start 11255 '<FieldAccess for referenced_decl at ast.py:10243>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10243
--# expr-start 11254 '<at at ast.py:10243>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10243
Get_Result := Get (Self, Var_Expr_1, 0, True); 
--# expr-done 11254
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Get_Result.Node, Imprecise_Fallback => False, E_Info => Get_Result.Info);
--# end
--# expr-done 11255
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Cast_Expr := Var_Expr_8; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result_3, Cast_Result_4); 
--# expr-done 11256
   If_Result_2 := Is_Equiv;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 11257 '<FieldAccess for defining_name at ast.py:10245>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10245
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_8.Node, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 11257
                Singleton_1 := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton_1.Items (1) := Fld_7;
   If_Result_3 := Singleton_1;
else
   If_Result_3 := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 11258
Scope_Result_2 := If_Result_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3110;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_4 := Result_Var_2;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 11259
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 11260
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3103;
   Result_Var_8 := Scope_Result_3;
else
   --# expr-start 11279 '<Map at ast.py:10279>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10279
   --# expr-start 11277 '<If at ast.py:10250>' If_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10250
--# expr-start 11263 '<FieldAccess for is_contract_aspect at ast.py:10254>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10254
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11262 '<FieldAccess for name_symbol at ast.py:10254>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10254
--# expr-start 11261 '<FieldAccess for id at ast.py:10254>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10254
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11261
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_10.Node);
--# end
--# expr-done 11262
--# property-call-start AdaNode.is_contract_aspect
Fld_12 := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Fld_11);
--# end
--# expr-done 11263
if Fld_12 then
   If_Result_7 := True;
else
   --# expr-start 11266 '<FieldAccess for name_is at ast.py:10256>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10256
--# expr-start 11264 '<FieldAccess for id at ast.py:10256>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10256
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11264
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11265 '<Symbol Obsolescent at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Obsolescent); 
--# expr-done 11265
--# property-call-start Name.name_is
Fld_14 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_13.Node, Sym => Sym);
--# end
--# expr-done 11266
   If_Result_7 := Fld_14;
end if;
if If_Result_7 then
--# expr-start 11268 '<Cast to AdaNode.list at ast.py:10266>' Cast_Result_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10266
--# expr-start 11267 '<parent at ast.py:10266>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10266
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Parent; 
--# expr-done 11267
   if Fld_9 = null
      or else Fld_9.Kind in Ada_Ada_Node_List_Range
   then
      Cast_Result_5 := Fld_9;
   else
         Cast_Result_5 := No_Bare_Ada_Node;
   end if;
--# expr-done 11268
Var_Expr_3 := Cast_Result_5;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
Var_Expr_4 := Var_Expr_3;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind decls Var_Expr_4
--# expr-start 11276 '<as_entity at ast.py:10271>' If_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10271
--# expr-start 11275 '<Cast to BasicDecl at ast.py:10271>' Cast_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10271
--# expr-start 11271 '<Filter at ast.py:10267>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10267
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Var_Expr_4 = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Var_Expr_4;
      begin
         for Untyped_Item_348 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_348 := Untyped_Item_348; 
   --# scope-start
               --# bind decl Item_348
--# expr-start 11269 '<IsA BasicDecl at ast.py:10268>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10268
Is_A_1 := Item_348 /= null 
and then Item_348.Kind in Ada_Basic_Decl; 
--# expr-done 11269
if Is_A_1 then
   --# expr-start 11270 '<OrderingTest '"'"'gt'"'"' at ast.py:10268>' Node_Comp /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10268
Node_Comp := Compare (Self, Self, Item_348, Greater_Than); 
--# expr-done 11270
   If_Result_5 := Node_Comp;
else
   If_Result_5 := False;
end if;
         if If_Result_5 then
         declare
            Item_To_Append : constant Bare_Ada_Node := Item_348;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11271
Var_Expr_6 := Map_Result;
      Inc_Ref (Var_Expr_6);
if Var_Expr_6 /= No_Bare_Ada_Node_Array_Type then
   --# scope-start
   --# bind decls Var_Expr_6
--# expr-start 11274 '<at at ast.py:10270>' Get_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10270
--# expr-start 11273 '<Arithmetic - at ast.py:10270>' Arith_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10270
--# expr-start 11272 '<length at ast.py:10270>' Len_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10270
Len_1 := Length (Var_Expr_6); 
--# expr-done 11272
Arith_Result := (Len_1 - 1); 
--# expr-done 11273
Get_Result_1 := Get (Self, Var_Expr_6, Arith_Result, True); 
--# expr-done 11274
Scope_Result_4 := Get_Result_1;
   --# end
      Finalizer_Scope_3114;
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Bare_Ada_Node;
end if;
   if Result_Var_3 = null
      or else Result_Var_3.Kind in Ada_Basic_Decl
   then
      Cast_Result_6 := Result_Var_3;
   else
         Cast_Result_6 := No_Bare_Ada_Node;
   end if;
--# expr-done 11275
Is_Null_2 := Cast_Result_6 = null; 
if Is_Null_2 then
   If_Result_6 := No_Entity_Basic_Decl;
else
As_Entity_2 := (Info => E_Info, Node => Cast_Result_6); 
   If_Result_6 := As_Entity_2;
end if;
--# expr-done 11276
Var_Expr_5 := If_Result_6;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
                Singleton_2 := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton_2.Items (1) := Var_Expr_5;
Scope_Result_5 := Singleton_2;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3115;
   Result_Var_4 := Scope_Result_5;
else
   Result_Var_4 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Scope_Result_6 := Result_Var_4;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3112;
   Result_Var_5 := Scope_Result_6;
else
   Result_Var_5 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_5);
Scope_Result_7 := Result_Var_5;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3111;
   Result_Var_6 := Scope_Result_7;
else
   Result_Var_6 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Var_Expr_2 := Result_Var_6;
      Inc_Ref (Var_Expr_2);
if Var_Expr_2 /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_8 := Var_Expr_2;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_3116;
   Result_Var_7 := Scope_Result_8;
else
                Singleton_3 := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton_3.Items (1) := Block_Var_1;
   Result_Var_7 := Singleton_3;
end if;
      Inc_Ref (Result_Var_7);
   If_Result_8 := Result_Var_7;
else
                Singleton_4 := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton_4.Items (1) := Block_Var_1;
   If_Result_8 := Singleton_4;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 11277
      declare
         Map_Result_1_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := If_Result_8;
      begin
         for Item_347 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_347
      --# expr-start 11278 '<FieldAccess for defining_name at ast.py:10279>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10279
      if Item_347.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_347.Node, E_Info => Item_347.Info);
--# end
--# expr-done 11278
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Fld_15;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 11279
   Result_Var_8 := Map_Result_1;
end if;
      Inc_Ref (Result_Var_8);
Let_Result_1 := Result_Var_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 11280
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_800;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_800;
                     Finalizer_Scope_3101;
                     Finalizer_Scope_3103;
                     Finalizer_Scope_3109;
                     Finalizer_Scope_3110;
                     Finalizer_Scope_3111;
                     Finalizer_Scope_3112;
                     Finalizer_Scope_3114;
                     Finalizer_Scope_3115;
                     Finalizer_Scope_3116;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Associated_Entities;
--# end
--# property-start Pragma.initial_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10282
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Initial_Env
  (Node : Bare_Pragma_Node
  )
   return Internal_Designated_Env
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_801 with Inline_Always;
      Sym : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Internal_Designated_Env := No_Designated_Env;
Let_Result : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_801 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_801;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11288 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11287 '<If at ast.py:10288>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10288
--# expr-start 11283 '<IsA CompilationUnit at ast.py:10289>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10289
--# expr-start 11282 '<parent at ast.py:10289>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10289
--# expr-start 11281 '<parent at ast.py:10289>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10289
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 11281
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 11282
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 11283
if Is_A then
   --# expr-start 11285 '<New DesignatedEnv at ast.py:10290>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10290
--# expr-start 11284 '<Symbol Standard at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard); 
--# expr-done 11284
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Sym, Kind => Named_Env); 
--# expr-done 11285
   If_Result := New_Struct;
else
   --# expr-start 11286 '<New DesignatedEnv at ast.py:10291>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10291
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 11286
   If_Result := New_Struct_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 11287
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 11288
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_801;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_801;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Initial_Env;
--# end
--# property-start Pragma._initial_env_36 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10295
pragma Warnings (Off, "is not referenced");
 function Initial_Env_36
  (Node : Bare_Pragma_Node
  )
   return Internal_Designated_Env
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1563 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1563 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1563;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11289 '<FieldAccess for initial_env at ast.py:10295>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:10295
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.initial_env
Fld := Libadalang.Implementation.Pragma_Node_P_Initial_Env (Node => Self);
--# end
--# expr-done 11289
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1563;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1563;
            raise;
      end;
   return Property_Result;
end Initial_Env_36;
--# end
   --
   --  Primitives for Bare_Private_Node
   --
--# property-start '[dispatcher]Private.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Private_Node_P_As_Bool
  (Node : Bare_Private_Node
  )
   return Boolean
is
   Self : Bare_Private_Node  := Bare_Private_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Private_Node (Self.Kind) is
               when Ada_Private_Absent =>
                  --# property-call-start Private.Absent.as_bool
                  Property_Result := Private_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Private_Present =>
                  --# property-call-start Private.Present.as_bool
                  Property_Result := Private_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Private_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Private_Absent
   --
--# property-start Private.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5276
pragma Warnings (Off, "is not referenced");
 function Private_Absent_P_As_Bool
  (Node : Bare_Private_Absent
  )
   return Boolean
is
   Self : Bare_Private_Absent  := Bare_Private_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Private_Present
   --
--# property-start Private.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5276
pragma Warnings (Off, "is not referenced");
 function Private_Present_P_As_Bool
  (Node : Bare_Private_Present
  )
   return Boolean
is
   Self : Bare_Private_Present  := Bare_Private_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Def
   --
      procedure Initialize_Fields_For_Protected_Def
        (Self : Bare_Protected_Def
         ; Protected_Def_F_Public_Part : Bare_Public_Part
         ; Protected_Def_F_Private_Part : Bare_Private_Part
         ; Protected_Def_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Protected_Def_F_Public_Part := Protected_Def_F_Public_Part;
            Self.Protected_Def_F_Private_Part := Protected_Def_F_Private_Part;
            Self.Protected_Def_F_End_Name := Protected_Def_F_End_Name;
      end Initialize_Fields_For_Protected_Def;
   function Protected_Def_F_Public_Part
     (Node : Bare_Protected_Def) return Bare_Public_Part
   is
   begin
         return Node.Protected_Def_F_Public_Part;
   end;
   function Protected_Def_F_Private_Part
     (Node : Bare_Protected_Def) return Bare_Private_Part
   is
   begin
         return Node.Protected_Def_F_Private_Part;
   end;
   function Protected_Def_F_End_Name
     (Node : Bare_Protected_Def) return Bare_End_Name
   is
   begin
         return Node.Protected_Def_F_End_Name;
   end;
   --
   --  Primitives for Bare_Protected_Node
   --
--# property-start '[dispatcher]Protected.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Protected_Node_P_As_Bool
  (Node : Bare_Protected_Node
  )
   return Boolean
is
   Self : Bare_Protected_Node  := Bare_Protected_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Protected_Node (Self.Kind) is
               when Ada_Protected_Absent =>
                  --# property-call-start Protected.Absent.as_bool
                  Property_Result := Protected_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Protected_Present =>
                  --# property-call-start Protected.Present.as_bool
                  Property_Result := Protected_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Protected_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Absent
   --
--# property-start Protected.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5356
pragma Warnings (Off, "is not referenced");
 function Protected_Absent_P_As_Bool
  (Node : Bare_Protected_Absent
  )
   return Boolean
is
   Self : Bare_Protected_Absent  := Bare_Protected_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Protected_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Present
   --
--# property-start Protected.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5356
pragma Warnings (Off, "is not referenced");
 function Protected_Present_P_As_Bool
  (Node : Bare_Protected_Present
  )
   return Boolean
is
   Self : Bare_Protected_Present  := Bare_Protected_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Protected_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Quantifier
   --
   --
   --  Primitives for Bare_Quantifier_All
   --
   --
   --  Primitives for Bare_Quantifier_Some
   --
   --
   --  Primitives for Bare_Range_Spec
   --
      procedure Initialize_Fields_For_Range_Spec
        (Self : Bare_Range_Spec
         ; Range_Spec_F_Range : Bare_Expr
        ) is
      begin
            Self.Range_Spec_F_Range := Range_Spec_F_Range;
      end Initialize_Fields_For_Range_Spec;
   function Range_Spec_F_Range
     (Node : Bare_Range_Spec) return Bare_Expr
   is
   begin
         return Node.Range_Spec_F_Range;
   end;
--# property-start RangeSpec.xref_stop_resolution /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22180
pragma Warnings (Off, "is not referenced");
 function Range_Spec_P_Xref_Stop_Resolution
  (Node : Bare_Range_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Range_Spec  := Bare_Range_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11291 '<IsA ComponentClause at ast.py:22180>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22180
--# expr-start 11290 '<parent at ast.py:22180>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22180
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 11290
Is_A := Fld /= null 
and then Fld.Kind in Ada_Component_Clause_Range; 
--# expr-done 11291
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Range_Spec_P_Xref_Stop_Resolution;
--# end
--# property-start RangeSpec.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22181
pragma Warnings (Off, "is not referenced");
 function Range_Spec_P_Xref_Equation
  (Node : Bare_Range_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Range_Spec  := Bare_Range_Spec (Node);
     Ent : Internal_Entity_Range_Spec :=
       Internal_Entity_Range_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1510 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel : Logic_Equation;
Fld_7 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1510 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1510;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11303 '<And at ast.py:22181>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22181
--# expr-start 11293 '<FieldAccess for xref_equation at ast.py:22181>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22181
--# expr-start 11292 '<FieldAccess for range at ast.py:22181>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22181
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Range_Spec_F_Range, Info => Ent.Info);
--# expr-done 11292
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11293
--# expr-start 11302 '<If at ast.py:22181>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22181
--# expr-start 11295 '<IsA ComponentClause, SignedIntTypeDef at ast.py:22184>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22184
--# expr-start 11294 '<parent at ast.py:22184>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22184
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Parent; 
--# expr-done 11294
Is_A := Fld_7 /= null 
and then Fld_7.Kind in Ada_Component_Clause_Range | Ada_Signed_Int_Type_Def_Range; 
--# expr-done 11295
if Is_A then
   --# expr-start 11301 '<And at ast.py:22186>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22186
--# expr-start 11298 '<FieldAccess for universal_int_bind at ast.py:22186>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22186
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11297 '<FieldAccess for expected_type_var at ast.py:22186>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22186
--# expr-start 11296 '<FieldAccess for range at ast.py:22186>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22186
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Range_Spec_F_Range;
--# expr-done 11296
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 11297
--# property-call-start AdaNode.universal_int_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 11298
--# expr-start 11300 '<FieldAccess for matches_expected_type at ast.py:22187>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22187
--# expr-start 11299 '<FieldAccess for range at ast.py:22187>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22187
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Range_Spec_F_Range, Info => Ent.Info);
--# expr-done 11299
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 11300
And_Pred := Create_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:22186") else null)); 
--# expr-done 11301
   If_Result := And_Pred;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:22189") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 11302
And_Pred_1 := Create_And (Fld_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:22181") else null)); 
--# expr-done 11303
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1510;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1510;
            raise;
      end;
   return Property_Result;
end Range_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Renaming_Clause
   --
      procedure Initialize_Fields_For_Renaming_Clause
        (Self : Bare_Renaming_Clause
         ; Renaming_Clause_F_Renamed_Object : Bare_Name
        ) is
      begin
            Self.Renaming_Clause_F_Renamed_Object := Renaming_Clause_F_Renamed_Object;
      end Initialize_Fields_For_Renaming_Clause;
   function Renaming_Clause_F_Renamed_Object
     (Node : Bare_Renaming_Clause) return Bare_Name
   is
   begin
         return Node.Renaming_Clause_F_Renamed_Object;
   end;
   --
   --  Primitives for Bare_Synthetic_Renaming_Clause
   --
      procedure Initialize_Fields_For_Synthetic_Renaming_Clause
        (Self : Bare_Synthetic_Renaming_Clause
         ; Renaming_Clause_F_Renamed_Object : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Renaming_Clause
              (Self, Renaming_Clause_F_Renamed_Object);
      end Initialize_Fields_For_Synthetic_Renaming_Clause;
   --
   --  Primitives for Bare_Reverse_Node
   --
--# property-start '[dispatcher]Reverse.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Reverse_Node_P_As_Bool
  (Node : Bare_Reverse_Node
  )
   return Boolean
is
   Self : Bare_Reverse_Node  := Bare_Reverse_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Reverse_Node (Self.Kind) is
               when Ada_Reverse_Absent =>
                  --# property-call-start Reverse.Absent.as_bool
                  Property_Result := Reverse_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Reverse_Present =>
                  --# property-call-start Reverse.Present.as_bool
                  Property_Result := Reverse_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Reverse_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Reverse_Absent
   --
--# property-start Reverse.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5324
pragma Warnings (Off, "is not referenced");
 function Reverse_Absent_P_As_Bool
  (Node : Bare_Reverse_Absent
  )
   return Boolean
is
   Self : Bare_Reverse_Absent  := Bare_Reverse_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Reverse_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Reverse_Present
   --
--# property-start Reverse.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5324
pragma Warnings (Off, "is not referenced");
 function Reverse_Present_P_As_Bool
  (Node : Bare_Reverse_Present
  )
   return Boolean
is
   Self : Bare_Reverse_Present  := Bare_Reverse_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Reverse_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Select_When_Part
   --
      procedure Initialize_Fields_For_Select_When_Part
        (Self : Bare_Select_When_Part
         ; Select_When_Part_F_Cond_Expr : Bare_Expr
         ; Select_When_Part_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Select_When_Part_F_Cond_Expr := Select_When_Part_F_Cond_Expr;
            Self.Select_When_Part_F_Stmts := Select_When_Part_F_Stmts;
      end Initialize_Fields_For_Select_When_Part;
   function Select_When_Part_F_Cond_Expr
     (Node : Bare_Select_When_Part) return Bare_Expr
   is
   begin
         return Node.Select_When_Part_F_Cond_Expr;
   end;
   function Select_When_Part_F_Stmts
     (Node : Bare_Select_When_Part) return Bare_Stmt_List
   is
   begin
         return Node.Select_When_Part_F_Stmts;
   end;
--# property-start SelectWhenPart.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21615
pragma Warnings (Off, "is not referenced");
 function Select_When_Part_P_Xref_Equation
  (Node : Bare_Select_When_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Select_When_Part  := Bare_Select_When_Part (Node);
     Ent : Internal_Entity_Select_When_Part :=
       Internal_Entity_Select_When_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1476 with Inline_Always;
            procedure Finalizer_Scope_3294 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1476 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1476;
            procedure Finalizer_Scope_3294 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3294;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11312 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11304 '<FieldAccess for cond_expr at ast.py:21617>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21617
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Select_When_Part_F_Cond_Expr, Info => Ent.Info);
--# expr-done 11304
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 11311 '<And at ast.py:21618>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21618
--# expr-start 11309 '<And at ast.py:21618>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21618
--# expr-start 11307 '<Bind at ast.py:21619>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21619
--# expr-start 11305 '<FieldAccess for expected_type_var at ast.py:21619>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21619
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 11305
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 11306 '<FieldAccess for bool_type at ast.py:21619>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21619
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11306
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21619") else null)); 
--# expr-done 11307
--# expr-start 11308 '<FieldAccess for sub_equation at ast.py:21620>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21620
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 11308
And_Pred := Create_And (Bind_Result, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21618") else null)); 
--# expr-done 11309
--# expr-start 11310 '<FieldAccess for matches_expected_formal_prim_type at ast.py:21621>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21621
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_4 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 11310
And_Pred_1 := Create_And (And_Pred, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21618") else null)); 
--# expr-done 11311
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3294;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21623") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 11312
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1476;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1476;
                     Finalizer_Scope_3294;
            raise;
      end;
   return Property_Result;
end Select_When_Part_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Stmt
   --
--# property-start Stmt.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20952
pragma Warnings (Off, "is not referenced");
 function Stmt_P_Xref_Entry_Point
  (Node : Bare_Stmt
  )
   return Boolean
is
   Self : Bare_Stmt  := Bare_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Stmt_P_Xref_Entry_Point;
--# end
--# property-start Stmt.is_ghost_code /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20954
pragma Warnings (Off, "is not referenced");
 function Stmt_P_Is_Ghost_Code
  (Node : Bare_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Stmt  := Bare_Stmt (Node);
     Ent : Internal_Entity_Stmt :=
       Internal_Entity_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Refd_Def;
Match_0 : Internal_Entity_Assign_Stmt;
Match_1 : Internal_Entity_Call_Stmt;
Match_2 : Internal_Entity_Stmt;
Var_Expr_2 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Refd_Def;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Refd_Def;
Match_Prefix : Internal_Entity_Stmt;
Cast_Result : Internal_Entity_Assign_Stmt;
Let_Result : Internal_Refd_Def;
Scope_Result_1 : Internal_Refd_Def;
Cast_Result_1 : Internal_Entity_Call_Stmt;
Let_Result_1 : Internal_Refd_Def;
Scope_Result_2 : Internal_Refd_Def;
Cast_Result_2 : Internal_Entity_Stmt;
Let_Result_2 : Internal_Refd_Def;
Scope_Result_3 : Internal_Refd_Def;
Match_Result : Internal_Refd_Def;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_1 : Boolean;
Fld_8 : Ref_Result_Kind;
Is_Equal : Boolean;
If_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_2 : Boolean;
If_Result_1 : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 11325 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 11313 '<FieldAccess for parent_basic_decl at ast.py:20962>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20962
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11313
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr
--# expr-start 11314 '<FieldAccess for is_ghost_code at ast.py:20963>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20963
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_ghost_code
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Ghost_Code (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 11314
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result_1 := True;
else
--# expr-start 11319 '<Match at ast.py:20968>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20968
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Stmt (Match_Prefix.Node.Kind) is
         when Ada_Assign_Stmt =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Assign_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ass Match_0
--# expr-start 11316 '<FieldAccess for failsafe_referenced_def_name at ast.py:20969>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20969
--# expr-start 11315 '<FieldAccess for dest at ast.py:20969>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20969
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Assign_Stmt_F_Dest, Info => Match_0.Info);
--# expr-done 11315
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.failsafe_referenced_def_name
Fld_3 := Libadalang.Implementation.Name_P_Failsafe_Referenced_Def_Name (Node => Fld_2.Node, Imprecise_Fallback => False, E_Info => Fld_2.Info);
--# end
--# expr-done 11316
Let_Result := Fld_3; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Call_Stmt =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Call_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind call Match_1
--# expr-start 11318 '<FieldAccess for failsafe_referenced_def_name at ast.py:20970>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20970
--# expr-start 11317 '<FieldAccess for call at ast.py:20970>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20970
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Stmt_F_Call, Info => Match_1.Info);
--# expr-done 11317
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.failsafe_referenced_def_name
Fld_5 := Libadalang.Implementation.Name_P_Failsafe_Referenced_Def_Name (Node => Fld_4.Node, Imprecise_Fallback => False, E_Info => Fld_4.Info);
--# end
--# expr-done 11318
Let_Result_1 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Accept_Stmt .. Ada_Abort_Stmt | Ada_Delay_Stmt .. Ada_Terminate_Alternative =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Refd_Def; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 11319
Var_Expr_1 := Match_Result;
if Var_Expr_1 /= No_Refd_Def then
   --# scope-start
   --# bind res Var_Expr_1
--# expr-start 11324 '<If at ast.py:20972>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20972
--# expr-start 11321 '<Eq at ast.py:20979>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20979
--# expr-start 11320 '<FieldAccess for kind at ast.py:20979>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20979
Fld_8 := Var_Expr_1.Kind;
--# expr-done 11320
Is_Equal := Fld_8 = Error; 
--# expr-done 11321
if Is_Equal then
   If_Result := raise Property_Error with "Name resolution error";
else
--# expr-start 11322 '<FieldAccess for def_name at ast.py:20981>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20981
Fld_6 := Var_Expr_1.Def_Name;
--# expr-done 11322
Var_Expr_2 := Fld_6;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 11323 '<FieldAccess for is_ghost_code at ast.py:20981>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:20981
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_ghost_code
Fld_7 := Libadalang.Implementation.Defining_Name_P_Is_Ghost_Code (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 11323
Scope_Result_4 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
end if;
--# expr-done 11324
Scope_Result_5 := If_Result;
   --# end
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := False;
end if;
   If_Result_1 := Result_Var_2;
end if;
Let_Result_3 := If_Result_1; 
--# end
--# expr-done 11325
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Stmt_P_Is_Ghost_Code;
--# end
   --
   --  Primitives for Bare_Composite_Stmt
   --
   --
   --  Primitives for Bare_Accept_Stmt
   --
      procedure Accept_Stmt_Pre_Env_Actions
        (Self            : Bare_Accept_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_151 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:21550");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_152 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Accept_Stmt
        (Self : Bare_Accept_Stmt
         ; Accept_Stmt_F_Body_Decl : Bare_Accept_Stmt_Body
         ; Accept_Stmt_F_Entry_Index_Expr : Bare_Expr
         ; Accept_Stmt_F_Params : Bare_Entry_Completion_Formal_Params
        ) is
      begin
            Self.Accept_Stmt_F_Body_Decl := Accept_Stmt_F_Body_Decl;
            Self.Accept_Stmt_F_Entry_Index_Expr := Accept_Stmt_F_Entry_Index_Expr;
            Self.Accept_Stmt_F_Params := Accept_Stmt_F_Params;
      end Initialize_Fields_For_Accept_Stmt;
   function Accept_Stmt_F_Body_Decl
     (Node : Bare_Accept_Stmt) return Bare_Accept_Stmt_Body
   is
   begin
         return Node.Accept_Stmt_F_Body_Decl;
   end;
   function Accept_Stmt_F_Entry_Index_Expr
     (Node : Bare_Accept_Stmt) return Bare_Expr
   is
   begin
         return Node.Accept_Stmt_F_Entry_Index_Expr;
   end;
   function Accept_Stmt_F_Params
     (Node : Bare_Accept_Stmt) return Bare_Entry_Completion_Formal_Params
   is
   begin
         return Node.Accept_Stmt_F_Params;
   end;
--# property-start AcceptStmt.designated_entry /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21556
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Designated_Entry
  (Node : Bare_Accept_Stmt
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
     Ent : Internal_Entity_Accept_Stmt :=
       Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1472 with Inline_Always;
      Var_Expr : Internal_Entity_Entry_Decl;
Fld : Internal_Entity_Accept_Stmt_Body;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Entry_Decl;
Fld_3 : Internal_Entity_Entry_Spec;
Fld_4 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Entry_Decl;
Let_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1472 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1472;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11335 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11334 '<Cast to EntryDecl at ast.py:21563>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21563
--# expr-start 11333 '<Find at ast.py:21559>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21559
   --# expr-start 11328 '<FieldAccess for all_env_els_impl at ast.py:21559>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21559
--# expr-start 11327 '<FieldAccess for name at ast.py:21559>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21559
--# expr-start 11326 '<FieldAccess for body_decl at ast.py:21559>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21559
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Accept_Stmt_Body (Node => Ent.Node.Accept_Stmt_F_Body_Decl, Info => Ent.Info);
--# expr-done 11326
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Defining_Name (Node => Fld.Node.Accept_Stmt_Body_F_Name, Info => Fld.Info);
--# expr-done 11327
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 11328
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_349 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_349
--# expr-start 11329 '<Cast to EntryDecl at ast.py:21560>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21560
Cast_Expr := Item_349; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Entry_Decl;
   end if;
--# expr-done 11329
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 11332 '<FieldAccess for match_formal_params at ast.py:21561>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21561
--# expr-start 11330 '<FieldAccess for spec at ast.py:21561>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21561
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr.Node.Entry_Decl_F_Spec, Info => Var_Expr.Info);
--# expr-done 11330
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11331 '<FieldAccess for params at ast.py:21561>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21561
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Ent.Node.Accept_Stmt_F_Params, Info => Ent.Info);
--# expr-done 11331
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_3.Node, Other => Cast_Result_1, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 11332
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_349; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 11333
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Entry_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_2 := No_Entity_Entry_Decl;
   end if;
--# expr-done 11334
Let_Result := Cast_Result_2; 
--# end
--# expr-done 11335
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1472;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1472;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_P_Designated_Entry;
--# end
--# property-start AcceptStmt.corresponding_entry /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21565
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Corresponding_Entry
  (Node : Bare_Accept_Stmt
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
     Ent : Internal_Entity_Accept_Stmt :=
       Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1473 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Entry_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Entry_Decl;
Let_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1473 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1473;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 11339 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11338 '<bind at ast.py:21571>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21571
--# expr-start 11336 '<FieldAccess for node_env at ast.py:21571>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21571
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11336
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 11337 '<FieldAccess for designated_entry at ast.py:21571>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21571
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_1 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Ent.Node, Origin => Origin, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 11337
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 11338
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 11339
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1473;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1473;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_P_Corresponding_Entry;
--# end
--# property-start AcceptStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21573
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Xref_Equation
  (Node : Bare_Accept_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
     Ent : Internal_Entity_Accept_Stmt :=
       Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1474 with Inline_Always;
            procedure Finalizer_Scope_3303 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Bare_Accept_Stmt_Body;
Fld_1 : Bare_Defining_Name;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Entry_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1474 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1474;
            procedure Finalizer_Scope_3303 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_3303;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11348 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11347 '<And at ast.py:21575>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21575
--# expr-start 11344 '<Bind at ast.py:21576>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21576
--# expr-start 11342 '<FieldAccess for ref_var at ast.py:21576>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21576
--# expr-start 11341 '<FieldAccess for name at ast.py:21576>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21576
--# expr-start 11340 '<FieldAccess for body_decl at ast.py:21576>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21576
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Accept_Stmt_F_Body_Decl;
--# expr-done 11340
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Accept_Stmt_Body_F_Name;
--# expr-done 11341
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_1);
--# end
--# expr-done 11342
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 11343 '<FieldAccess for designated_entry at ast.py:21576>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21576
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_3 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 11343
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21576") else null)); 
--# expr-done 11344
--# expr-start 11345 '<FieldAccess for entry_index_expr at ast.py:21577>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21577
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Accept_Stmt_F_Entry_Index_Expr, Info => Ent.Info);
--# expr-done 11345
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 11346 '<FieldAccess for sub_equation at ast.py:21578>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21578
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 11346
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3303;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21579") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Bind_Result, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21575") else null)); 
--# expr-done 11347
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 11348
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1474;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1474;
                     Finalizer_Scope_3303;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_P_Xref_Equation;
--# end
--# property-start AcceptStmt._env_mappings_151 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21552
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_151
  (Node : Bare_Accept_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
     Ent : Internal_Entity_Accept_Stmt :=
       Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1678 with Inline_Always;
      Fld : Internal_Entity_Accept_Stmt_Body;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Symbol_Type;
Fld_4 : Bare_Accept_Stmt_Body;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Singleton : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1678 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1678;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11355 '<New EnvAssoc at ast.py:21550>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21550
--# expr-start 11349 '<New DesignatedEnv at ast.py:21550>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21550
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 11349
--# expr-start 11353 '<FieldAccess for name_symbol at ast.py:21551>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21551
--# expr-start 11352 '<FieldAccess for relative_name at ast.py:21551>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21551
--# expr-start 11351 '<FieldAccess for name at ast.py:21551>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21551
--# expr-start 11350 '<FieldAccess for body_decl at ast.py:21551>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21551
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Accept_Stmt_Body (Node => Ent.Node.Accept_Stmt_F_Body_Decl, Info => Ent.Info);
--# expr-done 11350
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Defining_Name (Node => Fld.Node.Accept_Stmt_Body_F_Name, Info => Fld.Info);
--# expr-done 11351
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 11352
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 11353
--# expr-start 11354 '<FieldAccess for body_decl at ast.py:21551>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21551
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Accept_Stmt_F_Body_Decl;
--# expr-done 11354
      Cast_Result := Fld_4;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_3, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 11355
                Singleton := Create_Internal_Env_Assoc_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct_1;
                Inc_Ref (Singleton.Items (1));
         Property_Result := Singleton;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1678;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1678;
            raise;
      end;
   return Property_Result;
end Env_Mappings_151;
--# end
--# property-start AcceptStmt._env_trans_parent_152 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21549
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_152
  (Node : Bare_Accept_Stmt
  )
   return Boolean
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_152;
--# end
   --
   --  Primitives for Bare_Accept_Stmt_With_Stmts
   --
      procedure Initialize_Fields_For_Accept_Stmt_With_Stmts
        (Self : Bare_Accept_Stmt_With_Stmts
         ; Accept_Stmt_F_Body_Decl : Bare_Accept_Stmt_Body
         ; Accept_Stmt_F_Entry_Index_Expr : Bare_Expr
         ; Accept_Stmt_F_Params : Bare_Entry_Completion_Formal_Params
         ; Accept_Stmt_With_Stmts_F_Stmts : Bare_Handled_Stmts
         ; Accept_Stmt_With_Stmts_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Accept_Stmt
              (Self, Accept_Stmt_F_Body_Decl, Accept_Stmt_F_Entry_Index_Expr, Accept_Stmt_F_Params);
            Self.Accept_Stmt_With_Stmts_F_Stmts := Accept_Stmt_With_Stmts_F_Stmts;
            Self.Accept_Stmt_With_Stmts_F_End_Name := Accept_Stmt_With_Stmts_F_End_Name;
      end Initialize_Fields_For_Accept_Stmt_With_Stmts;
   function Accept_Stmt_With_Stmts_F_Stmts
     (Node : Bare_Accept_Stmt_With_Stmts) return Bare_Handled_Stmts
   is
   begin
         return Node.Accept_Stmt_With_Stmts_F_Stmts;
   end;
   function Accept_Stmt_With_Stmts_F_End_Name
     (Node : Bare_Accept_Stmt_With_Stmts) return Bare_End_Name
   is
   begin
         return Node.Accept_Stmt_With_Stmts_F_End_Name;
   end;
   --
   --  Primitives for Bare_Base_Loop_Stmt
   --
      procedure Initialize_Fields_For_Base_Loop_Stmt
        (Self : Bare_Base_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Base_Loop_Stmt_F_Spec := Base_Loop_Stmt_F_Spec;
            Self.Base_Loop_Stmt_F_Stmts := Base_Loop_Stmt_F_Stmts;
            Self.Base_Loop_Stmt_F_End_Name := Base_Loop_Stmt_F_End_Name;
      end Initialize_Fields_For_Base_Loop_Stmt;
   function Base_Loop_Stmt_F_Spec
     (Node : Bare_Base_Loop_Stmt) return Bare_Loop_Spec
   is
   begin
         return Node.Base_Loop_Stmt_F_Spec;
   end;
   function Base_Loop_Stmt_F_Stmts
     (Node : Bare_Base_Loop_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Base_Loop_Stmt_F_Stmts;
   end;
   function Base_Loop_Stmt_F_End_Name
     (Node : Bare_Base_Loop_Stmt) return Bare_End_Name
   is
   begin
         return Node.Base_Loop_Stmt_F_End_Name;
   end;
--# property-start BaseLoopStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21369
pragma Warnings (Off, "is not referenced");
 function Base_Loop_Stmt_P_Xref_Equation
  (Node : Bare_Base_Loop_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Loop_Stmt  := Bare_Base_Loop_Stmt (Node);
     Ent : Internal_Entity_Base_Loop_Stmt :=
       Internal_Entity_Base_Loop_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1461 with Inline_Always;
            procedure Finalizer_Scope_3304 with Inline_Always;
      Var_Expr : Internal_Entity_Loop_Spec;
Fld : Internal_Entity_Loop_Spec;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1461 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1461;
            procedure Finalizer_Scope_3304 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3304;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11358 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11356 '<FieldAccess for spec at ast.py:21371>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21371
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Loop_Spec (Node => Ent.Node.Base_Loop_Stmt_F_Spec, Info => Ent.Info);
--# expr-done 11356
Var_Expr := Fld;
if Var_Expr /= No_Entity_Loop_Spec then
   --# scope-start
   --# bind s Var_Expr
--# expr-start 11357 '<FieldAccess for xref_equation at ast.py:21371>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21371
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 11357
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3304;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21372") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 11358
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1461;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1461;
                     Finalizer_Scope_3304;
            raise;
      end;
   return Property_Result;
end Base_Loop_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_For_Loop_Stmt
   --
      procedure For_Loop_Stmt_Pre_Env_Actions
        (Self            : Bare_For_Loop_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_148 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_For_Loop_Stmt
        (Self : Bare_For_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_For_Loop_Stmt;
--# property-start ForLoopStmt._env_trans_parent_148 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21389
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_148
  (Node : Bare_For_Loop_Stmt
  )
   return Boolean
is
   Self : Bare_For_Loop_Stmt  := Bare_For_Loop_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_148;
--# end
   --
   --  Primitives for Bare_Loop_Stmt
   --
      procedure Initialize_Fields_For_Loop_Stmt
        (Self : Bare_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_Loop_Stmt;
   --
   --  Primitives for Bare_While_Loop_Stmt
   --
      procedure Initialize_Fields_For_While_Loop_Stmt
        (Self : Bare_While_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_While_Loop_Stmt;
   --
   --  Primitives for Bare_Block_Stmt
   --
      procedure Block_Stmt_Pre_Env_Actions
        (Self            : Bare_Block_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_149 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
--# property-start BlockStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21409
pragma Warnings (Off, "is not referenced");
 function Block_Stmt_P_Xref_Equation
  (Node : Bare_Block_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Block_Stmt  := Bare_Block_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1462 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1462 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1462;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21409") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1462;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1462;
            raise;
      end;
   return Property_Result;
end Block_Stmt_P_Xref_Equation;
--# end
--# property-start BlockStmt._env_trans_parent_149 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21407
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_149
  (Node : Bare_Block_Stmt
  )
   return Boolean
is
   Self : Bare_Block_Stmt  := Bare_Block_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_149;
--# end
   --
   --  Primitives for Bare_Begin_Block
   --
      procedure Initialize_Fields_For_Begin_Block
        (Self : Bare_Begin_Block
         ; Begin_Block_F_Stmts : Bare_Handled_Stmts
         ; Begin_Block_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Begin_Block_F_Stmts := Begin_Block_F_Stmts;
            Self.Begin_Block_F_End_Name := Begin_Block_F_End_Name;
      end Initialize_Fields_For_Begin_Block;
   function Begin_Block_F_Stmts
     (Node : Bare_Begin_Block) return Bare_Handled_Stmts
   is
   begin
         return Node.Begin_Block_F_Stmts;
   end;
   function Begin_Block_F_End_Name
     (Node : Bare_Begin_Block) return Bare_End_Name
   is
   begin
         return Node.Begin_Block_F_End_Name;
   end;
   --
   --  Primitives for Bare_Decl_Block
   --
      procedure Initialize_Fields_For_Decl_Block
        (Self : Bare_Decl_Block
         ; Decl_Block_F_Decls : Bare_Declarative_Part
         ; Decl_Block_F_Stmts : Bare_Handled_Stmts
         ; Decl_Block_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Decl_Block_F_Decls := Decl_Block_F_Decls;
            Self.Decl_Block_F_Stmts := Decl_Block_F_Stmts;
            Self.Decl_Block_F_End_Name := Decl_Block_F_End_Name;
      end Initialize_Fields_For_Decl_Block;
   function Decl_Block_F_Decls
     (Node : Bare_Decl_Block) return Bare_Declarative_Part
   is
   begin
         return Node.Decl_Block_F_Decls;
   end;
   function Decl_Block_F_Stmts
     (Node : Bare_Decl_Block) return Bare_Handled_Stmts
   is
   begin
         return Node.Decl_Block_F_Stmts;
   end;
   function Decl_Block_F_End_Name
     (Node : Bare_Decl_Block) return Bare_End_Name
   is
   begin
         return Node.Decl_Block_F_End_Name;
   end;
--# property-start DeclBlock.immediate_declarative_region /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21421
pragma Warnings (Off, "is not referenced");
 function Decl_Block_P_Immediate_Declarative_Region
  (Node : Bare_Decl_Block
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Decl_Block  := Bare_Decl_Block (Node);
     Ent : Internal_Entity_Decl_Block :=
       Internal_Entity_Decl_Block'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1463 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1463 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1463;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11360 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11359 '<FieldAccess for children_env at ast.py:21423>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21423
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11359
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 11360
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1463;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1463;
            raise;
      end;
   return Property_Result;
end Decl_Block_P_Immediate_Declarative_Region;
--# end
   --
   --  Primitives for Bare_Case_Stmt
   --
      procedure Initialize_Fields_For_Case_Stmt
        (Self : Bare_Case_Stmt
         ; Case_Stmt_F_Expr : Bare_Expr
         ; Case_Stmt_F_Pragmas : Bare_Pragma_Node_List
         ; Case_Stmt_F_Alternatives : Bare_Case_Stmt_Alternative_List
        ) is
      begin
            Self.Case_Stmt_F_Expr := Case_Stmt_F_Expr;
            Self.Case_Stmt_F_Pragmas := Case_Stmt_F_Pragmas;
            Self.Case_Stmt_F_Alternatives := Case_Stmt_F_Alternatives;
      end Initialize_Fields_For_Case_Stmt;
   function Case_Stmt_F_Expr
     (Node : Bare_Case_Stmt) return Bare_Expr
   is
   begin
         return Node.Case_Stmt_F_Expr;
   end;
   function Case_Stmt_F_Pragmas
     (Node : Bare_Case_Stmt) return Bare_Pragma_Node_List
   is
   begin
         return Node.Case_Stmt_F_Pragmas;
   end;
   function Case_Stmt_F_Alternatives
     (Node : Bare_Case_Stmt) return Bare_Case_Stmt_Alternative_List
   is
   begin
         return Node.Case_Stmt_F_Alternatives;
   end;
--# property-start CaseStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21459
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_P_Xref_Equation
  (Node : Bare_Case_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Stmt  := Bare_Case_Stmt (Node);
     Ent : Internal_Entity_Case_Stmt :=
       Internal_Entity_Case_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1465 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Pred : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1465 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1465;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11371 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11370 '<And at ast.py:21461>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21461
--# expr-start 11362 '<FieldAccess for sub_equation at ast.py:21461>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21461
--# expr-start 11361 '<FieldAccess for expr at ast.py:21461>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21461
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Case_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 11361
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11362
--# expr-start 11369 '<And at ast.py:21465>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21465
--# expr-start 11365 '<Predicate on AdaNode.is_not_null at ast.py:21465>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21465
--# expr-start 11364 '<FieldAccess for type_var at ast.py:21465>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21465
--# expr-start 11363 '<FieldAccess for expr at ast.py:21465>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21465
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Case_Stmt_F_Expr;
--# expr-done 11363
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 11364
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Pred := Solver.Create_Predicate (Fld_3, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 11365
--# expr-start 11368 '<Predicate on BaseTypeDecl.is_discrete_type at ast.py:21467>' Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21467
--# expr-start 11367 '<FieldAccess for type_var at ast.py:21467>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21467
--# expr-start 11366 '<FieldAccess for expr at ast.py:21467>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21467
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Case_Stmt_F_Expr;
--# expr-done 11366
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 11367
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Pred_1 := Solver.Create_Predicate (Fld_5, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin)); 
--# expr-done 11368
And_Pred := Create_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21465") else null)); 
--# expr-done 11369
And_Pred_1 := Create_And (Fld_1, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21461") else null)); 
--# expr-done 11370
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 11371
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1465;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1465;
            raise;
      end;
   return Property_Result;
end Case_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Extended_Return_Stmt
   --
      procedure Extended_Return_Stmt_Pre_Env_Actions
        (Self            : Bare_Extended_Return_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_150 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Extended_Return_Stmt
        (Self : Bare_Extended_Return_Stmt
         ; Extended_Return_Stmt_F_Decl : Bare_Extended_Return_Stmt_Object_Decl
         ; Extended_Return_Stmt_F_Stmts : Bare_Handled_Stmts
        ) is
      begin
            Self.Extended_Return_Stmt_F_Decl := Extended_Return_Stmt_F_Decl;
            Self.Extended_Return_Stmt_F_Stmts := Extended_Return_Stmt_F_Stmts;
      end Initialize_Fields_For_Extended_Return_Stmt;
   function Extended_Return_Stmt_F_Decl
     (Node : Bare_Extended_Return_Stmt) return Bare_Extended_Return_Stmt_Object_Decl
   is
   begin
         return Node.Extended_Return_Stmt_F_Decl;
   end;
   function Extended_Return_Stmt_F_Stmts
     (Node : Bare_Extended_Return_Stmt) return Bare_Handled_Stmts
   is
   begin
         return Node.Extended_Return_Stmt_F_Stmts;
   end;
--# property-start ExtendedReturnStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21443
pragma Warnings (Off, "is not referenced");
 function Extended_Return_Stmt_P_Xref_Equation
  (Node : Bare_Extended_Return_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Extended_Return_Stmt  := Bare_Extended_Return_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1464 with Inline_Always;
      True_Rel : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1464 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1464;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11372 '<Block at ???>' Let_Result None
--# scope-start
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21445") else null)); 
Let_Result := True_Rel; Inc_Ref (Let_Result);
--# end
--# expr-done 11372
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1464;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1464;
            raise;
      end;
   return Property_Result;
end Extended_Return_Stmt_P_Xref_Equation;
--# end
--# property-start ExtendedReturnStmt._env_trans_parent_150 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21447
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_150
  (Node : Bare_Extended_Return_Stmt
  )
   return Boolean
is
   Self : Bare_Extended_Return_Stmt  := Bare_Extended_Return_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_150;
--# end
   --
   --  Primitives for Bare_If_Stmt
   --
      procedure Initialize_Fields_For_If_Stmt
        (Self : Bare_If_Stmt
         ; If_Stmt_F_Cond_Expr : Bare_Expr
         ; If_Stmt_F_Then_Stmts : Bare_Stmt_List
         ; If_Stmt_F_Alternatives : Bare_Elsif_Stmt_Part_List
         ; If_Stmt_F_Else_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.If_Stmt_F_Cond_Expr := If_Stmt_F_Cond_Expr;
            Self.If_Stmt_F_Then_Stmts := If_Stmt_F_Then_Stmts;
            Self.If_Stmt_F_Alternatives := If_Stmt_F_Alternatives;
            Self.If_Stmt_F_Else_Stmts := If_Stmt_F_Else_Stmts;
      end Initialize_Fields_For_If_Stmt;
   function If_Stmt_F_Cond_Expr
     (Node : Bare_If_Stmt) return Bare_Expr
   is
   begin
         return Node.If_Stmt_F_Cond_Expr;
   end;
   function If_Stmt_F_Then_Stmts
     (Node : Bare_If_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.If_Stmt_F_Then_Stmts;
   end;
   function If_Stmt_F_Alternatives
     (Node : Bare_If_Stmt) return Bare_Elsif_Stmt_Part_List
   is
   begin
         return Node.If_Stmt_F_Alternatives;
   end;
   function If_Stmt_F_Else_Stmts
     (Node : Bare_If_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.If_Stmt_F_Else_Stmts;
   end;
--# property-start IfStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21262
pragma Warnings (Off, "is not referenced");
 function If_Stmt_P_Xref_Equation
  (Node : Bare_If_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Stmt  := Bare_If_Stmt (Node);
     Ent : Internal_Entity_If_Stmt :=
       Internal_Entity_If_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1452 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1452 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1452;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11383 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11382 '<And at ast.py:21265>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21265
--# expr-start 11379 '<And at ast.py:21265>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21265
--# expr-start 11376 '<Bind at ast.py:21265>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21265
--# expr-start 11374 '<FieldAccess for expected_type_var at ast.py:21265>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21265
--# expr-start 11373 '<FieldAccess for cond_expr at ast.py:21265>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21265
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.If_Stmt_F_Cond_Expr;
--# expr-done 11373
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 11374
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 11375 '<FieldAccess for bool_type at ast.py:21265>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21265
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11375
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21265") else null)); 
--# expr-done 11376
--# expr-start 11378 '<FieldAccess for sub_equation at ast.py:21266>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21266
--# expr-start 11377 '<FieldAccess for cond_expr at ast.py:21266>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21266
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Stmt_F_Cond_Expr, Info => Ent.Info);
--# expr-done 11377
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 11378
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21265") else null)); 
--# expr-done 11379
--# expr-start 11381 '<FieldAccess for matches_expected_formal_prim_type at ast.py:21267>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21267
--# expr-start 11380 '<FieldAccess for cond_expr at ast.py:21267>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21267
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.If_Stmt_F_Cond_Expr;
--# expr-done 11380
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Fld_5, Origin => Origin);
--# end
--# expr-done 11381
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21265") else null)); 
--# expr-done 11382
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 11383
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1452;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1452;
            raise;
      end;
   return Property_Result;
end If_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Named_Stmt
   --
      procedure Named_Stmt_Pre_Env_Actions
        (Self            : Bare_Named_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_146 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:21352");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_147 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Named_Stmt
        (Self : Bare_Named_Stmt
         ; Named_Stmt_F_Decl : Bare_Named_Stmt_Decl
         ; Named_Stmt_F_Stmt : Bare_Composite_Stmt
        ) is
      begin
            Self.Named_Stmt_F_Decl := Named_Stmt_F_Decl;
            Self.Named_Stmt_F_Stmt := Named_Stmt_F_Stmt;
      end Initialize_Fields_For_Named_Stmt;
   function Named_Stmt_F_Decl
     (Node : Bare_Named_Stmt) return Bare_Named_Stmt_Decl
   is
   begin
         return Node.Named_Stmt_F_Decl;
   end;
   function Named_Stmt_F_Stmt
     (Node : Bare_Named_Stmt) return Bare_Composite_Stmt
   is
   begin
         return Node.Named_Stmt_F_Stmt;
   end;
--# property-start NamedStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21356
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_P_Xref_Equation
  (Node : Bare_Named_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Named_Stmt  := Bare_Named_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1460 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1460 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1460;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21356") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1460;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1460;
            raise;
      end;
   return Property_Result;
end Named_Stmt_P_Xref_Equation;
--# end
--# property-start NamedStmt._env_mappings_146 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21352
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_146
  (Node : Bare_Named_Stmt
  )
   return Internal_Env_Assoc
is
   Self : Bare_Named_Stmt  := Bare_Named_Stmt (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1673 with Inline_Always;
      Fld : Bare_Named_Stmt_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Bare_Named_Stmt_Decl;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1673 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1673;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11388 '<New EnvAssoc at ast.py:21352>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21352
--# expr-start 11384 '<New DesignatedEnv at ast.py:21352>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21352
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 11384
--# expr-start 11386 '<FieldAccess for name_symbol at ast.py:21352>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21352
--# expr-start 11385 '<FieldAccess for decl at ast.py:21352>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21352
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Named_Stmt_F_Decl;
--# expr-done 11385
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 11386
--# expr-start 11387 '<FieldAccess for decl at ast.py:21352>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21352
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Named_Stmt_F_Decl;
--# expr-done 11387
      Cast_Result := Fld_2;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 11388
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1673;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1673;
            raise;
      end;
   return Property_Result;
end Env_Mappings_146;
--# end
--# property-start NamedStmt._env_trans_parent_147 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21351
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_147
  (Node : Bare_Named_Stmt
  )
   return Boolean
is
   Self : Bare_Named_Stmt  := Bare_Named_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_147;
--# end
   --
   --  Primitives for Bare_Select_Stmt
   --
      procedure Initialize_Fields_For_Select_Stmt
        (Self : Bare_Select_Stmt
         ; Select_Stmt_F_Guards : Bare_Select_When_Part_List
         ; Select_Stmt_F_Else_Stmts : Bare_Stmt_List
         ; Select_Stmt_F_Abort_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Select_Stmt_F_Guards := Select_Stmt_F_Guards;
            Self.Select_Stmt_F_Else_Stmts := Select_Stmt_F_Else_Stmts;
            Self.Select_Stmt_F_Abort_Stmts := Select_Stmt_F_Abort_Stmts;
      end Initialize_Fields_For_Select_Stmt;
   function Select_Stmt_F_Guards
     (Node : Bare_Select_Stmt) return Bare_Select_When_Part_List
   is
   begin
         return Node.Select_Stmt_F_Guards;
   end;
   function Select_Stmt_F_Else_Stmts
     (Node : Bare_Select_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Select_Stmt_F_Else_Stmts;
   end;
   function Select_Stmt_F_Abort_Stmts
     (Node : Bare_Select_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Select_Stmt_F_Abort_Stmts;
   end;
--# property-start SelectStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21602
pragma Warnings (Off, "is not referenced");
 function Select_Stmt_P_Xref_Equation
  (Node : Bare_Select_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Select_Stmt  := Bare_Select_Stmt (Node);
     Ent : Internal_Entity_Select_Stmt :=
       Internal_Entity_Select_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1475 with Inline_Always;
            procedure Finalizer_Scope_3305 with Inline_Always;
      Fld : Internal_Entity_Select_When_Part_List;
Fld_1 : Bare_Select_When_Part_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Select_When_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Select_When_Part;
Fld_3 : Logic_Equation;
Item_350 : Internal_Entity_Select_When_Part;
Bare_Item_350 : Bare_Select_When_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1475 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1475;
            procedure Finalizer_Scope_3305 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3305;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11393 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11392 '<LogicAll at ast.py:21604>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21604
--# expr-start 11391 '<Map at ast.py:21604>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21604
   --# expr-start 11389 '<FieldAccess for guards at ast.py:21604>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21604
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Select_When_Part_List (Node => Ent.Node.Select_Stmt_F_Guards, Info => Ent.Info);
--# expr-done 11389
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Select_When_Part_List := Fld_1;
      begin
         for Untyped_Item_350 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_350 := Untyped_Item_350; 
Is_Null := Bare_Item_350 = null; 
if Is_Null then
   If_Result := No_Entity_Select_When_Part;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_350); 
   If_Result := As_Entity;
end if;
                  Item_350 := If_Result; 
   --# scope-start
               --# bind wp Item_350
      --# expr-start 11390 '<FieldAccess for sub_equation at ast.py:21604>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21604
      if Item_350.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_350.Node, Env => Env, Origin => Origin, E_Info => Item_350.Info);
--# end
--# expr-done 11390
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3305;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11391
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21604") else null)); 
--# expr-done 11392
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 11393
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1475;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1475;
                     Finalizer_Scope_3305;
            raise;
      end;
   return Property_Result;
end Select_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Error_Stmt
   --
   --
   --  Primitives for Bare_Simple_Stmt
   --
   --
   --  Primitives for Bare_Abort_Stmt
   --
      procedure Initialize_Fields_For_Abort_Stmt
        (Self : Bare_Abort_Stmt
         ; Abort_Stmt_F_Names : Bare_Name_List
        ) is
      begin
            Self.Abort_Stmt_F_Names := Abort_Stmt_F_Names;
      end Initialize_Fields_For_Abort_Stmt;
   function Abort_Stmt_F_Names
     (Node : Bare_Abort_Stmt) return Bare_Name_List
   is
   begin
         return Node.Abort_Stmt_F_Names;
   end;
--# property-start AbortStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21198
pragma Warnings (Off, "is not referenced");
 function Abort_Stmt_P_Xref_Equation
  (Node : Bare_Abort_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Abort_Stmt  := Bare_Abort_Stmt (Node);
     Ent : Internal_Entity_Abort_Stmt :=
       Internal_Entity_Abort_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1449 with Inline_Always;
            procedure Finalizer_Scope_3306 with Inline_Always;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Item_351 : Internal_Entity_Name;
Bare_Item_351 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1449 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1449;
            procedure Finalizer_Scope_3306 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3306;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11401 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11400 '<LogicAll at ast.py:21200>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21200
--# expr-start 11399 '<Map at ast.py:21200>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21200
   --# expr-start 11394 '<FieldAccess for names at ast.py:21200>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21200
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Abort_Stmt_F_Names, Info => Ent.Info);
--# expr-done 11394
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_351 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_351 := Untyped_Item_351; 
Is_Null := Bare_Item_351 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_351); 
   If_Result := As_Entity;
end if;
                  Item_351 := If_Result; 
   --# scope-start
               --# bind name Item_351
      --# expr-start 11398 '<And at ast.py:21202>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21202
--# expr-start 11395 '<FieldAccess for sub_equation at ast.py:21202>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21202
      if Item_351.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_351.Node, Env => Env, Origin => Origin, E_Info => Item_351.Info);
--# end
--# expr-done 11395
--# expr-start 11397 '<Predicate on BaseTypeDecl.is_task_type at ast.py:21202>' Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21202
--# expr-start 11396 '<FieldAccess for type_var at ast.py:21203>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21203
      if Item_351.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_351.Node);
--# expr-done 11396
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_Predicate (Fld_4, Create_Base_Type_Decl_P_Is_Task_Type_0_Predicate); 
--# expr-done 11397
And_Pred := Create_And (Fld_3, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21202") else null)); 
--# expr-done 11398
         declare
            Item_To_Append : constant Logic_Equation := And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3306;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11399
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21200") else null)); 
--# expr-done 11400
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 11401
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1449;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1449;
                     Finalizer_Scope_3306;
            raise;
      end;
   return Property_Result;
end Abort_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Assign_Stmt
   --
      procedure Initialize_Fields_For_Assign_Stmt
        (Self : Bare_Assign_Stmt
         ; Assign_Stmt_F_Dest : Bare_Name
         ; Assign_Stmt_F_Expr : Bare_Expr
        ) is
      begin
            Self.Assign_Stmt_F_Dest := Assign_Stmt_F_Dest;
            Self.Assign_Stmt_F_Expr := Assign_Stmt_F_Expr;
      end Initialize_Fields_For_Assign_Stmt;
   function Assign_Stmt_F_Dest
     (Node : Bare_Assign_Stmt) return Bare_Name
   is
   begin
         return Node.Assign_Stmt_F_Dest;
   end;
   function Assign_Stmt_F_Expr
     (Node : Bare_Assign_Stmt) return Bare_Expr
   is
   begin
         return Node.Assign_Stmt_F_Expr;
   end;
--# property-start AssignStmt.complete_item_weight /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21046
pragma Warnings (Off, "is not referenced");
 function Assign_Stmt_P_Complete_Item_Weight
  (Node : Bare_Assign_Stmt
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Assign_Stmt  := Bare_Assign_Stmt (Node);
     Ent : Internal_Entity_Assign_Stmt :=
       Internal_Entity_Assign_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Name;
Fld_1 : Integer;
Let_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11404 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11403 '<FieldAccess for complete_item_weight_matching_type at ast.py:21048>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21048
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11402 '<FieldAccess for dest at ast.py:21048>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21048
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Assign_Stmt_F_Dest, Info => Ent.Info);
--# expr-done 11402
--# property-call-start AdaNode.complete_item_weight_matching_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Complete_Item_Weight_Matching_Type (Node => Self, Item => Item, Name => Fld, Origin => Origin);
--# end
--# expr-done 11403
Let_Result := Fld_1; 
--# end
--# expr-done 11404
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Assign_Stmt_P_Complete_Item_Weight;
--# end
--# property-start AssignStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21050
pragma Warnings (Off, "is not referenced");
 function Assign_Stmt_P_Xref_Equation
  (Node : Bare_Assign_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Assign_Stmt  := Bare_Assign_Stmt (Node);
     Ent : Internal_Entity_Assign_Stmt :=
       Internal_Entity_Assign_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1442 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1442 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1442;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11419 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11418 '<And at ast.py:21053>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21053
--# expr-start 11415 '<And at ast.py:21053>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21053
--# expr-start 11412 '<And at ast.py:21053>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21053
--# expr-start 11406 '<FieldAccess for sub_equation at ast.py:21053>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21053
--# expr-start 11405 '<FieldAccess for dest at ast.py:21053>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21053
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Assign_Stmt_F_Dest, Info => Ent.Info);
--# expr-done 11405
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11406
--# expr-start 11411 '<Bind at ast.py:21054>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21054
--# expr-start 11408 '<FieldAccess for type_var at ast.py:21054>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21054
--# expr-start 11407 '<FieldAccess for dest at ast.py:21054>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21054
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Assign_Stmt_F_Dest;
--# expr-done 11407
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 11408
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 11410 '<FieldAccess for expected_type_var at ast.py:21054>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21054
--# expr-start 11409 '<FieldAccess for expr at ast.py:21054>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21054
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Assign_Stmt_F_Expr;
--# expr-done 11409
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4);
--# expr-done 11410
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result := Solver.Create_Propagate (Fld_3, Fld_5, Logic_Functor_642'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21054") else null)); 
--# expr-done 11411
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21053") else null)); 
--# expr-done 11412
--# expr-start 11414 '<FieldAccess for sub_equation at ast.py:21056>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21056
--# expr-start 11413 '<FieldAccess for expr at ast.py:21056>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21056
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Assign_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 11413
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 11414
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21053") else null)); 
--# expr-done 11415
--# expr-start 11417 '<FieldAccess for matches_expected_assign_type at ast.py:21057>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21057
--# expr-start 11416 '<FieldAccess for expr at ast.py:21057>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21057
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Assign_Stmt_F_Expr;
--# expr-done 11416
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_8, Origin => Origin);
--# end
--# expr-done 11417
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21053") else null)); 
--# expr-done 11418
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 11419
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1442;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1442;
            raise;
      end;
   return Property_Result;
end Assign_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Call_Stmt
   --
      procedure Initialize_Fields_For_Call_Stmt
        (Self : Bare_Call_Stmt
         ; Call_Stmt_F_Call : Bare_Name
        ) is
      begin
            Self.Call_Stmt_F_Call := Call_Stmt_F_Call;
      end Initialize_Fields_For_Call_Stmt;
   function Call_Stmt_F_Call
     (Node : Bare_Call_Stmt) return Bare_Name
   is
   begin
         return Node.Call_Stmt_F_Call;
   end;
--# property-start CallStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21018
pragma Warnings (Off, "is not referenced");
 function Call_Stmt_P_Xref_Equation
  (Node : Bare_Call_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Stmt  := Bare_Call_Stmt (Node);
     Ent : Internal_Entity_Call_Stmt :=
       Internal_Entity_Call_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1439 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1439 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1439;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11426 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11425 '<And at ast.py:21021>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21021
--# expr-start 11421 '<FieldAccess for sub_equation at ast.py:21021>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21021
--# expr-start 11420 '<FieldAccess for call at ast.py:21021>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21021
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Stmt_F_Call, Info => Ent.Info);
--# expr-done 11420
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11421
--# expr-start 11424 '<Bind at ast.py:21024>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21024
--# expr-start 11423 '<FieldAccess for type_var at ast.py:21024>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21024
--# expr-start 11422 '<FieldAccess for call at ast.py:21024>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21024
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Call_Stmt_F_Call;
--# expr-done 11422
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 11423
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Assign (Fld_3, No_Entity, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21024") else null)); 
--# expr-done 11424
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21021") else null)); 
--# expr-done 11425
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 11426
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1439;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1439;
            raise;
      end;
   return Property_Result;
end Call_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Delay_Stmt
   --
      procedure Initialize_Fields_For_Delay_Stmt
        (Self : Bare_Delay_Stmt
         ; Delay_Stmt_F_Has_Until : Bare_Until_Node
         ; Delay_Stmt_F_Expr : Bare_Expr
        ) is
      begin
            Self.Delay_Stmt_F_Has_Until := Delay_Stmt_F_Has_Until;
            Self.Delay_Stmt_F_Expr := Delay_Stmt_F_Expr;
      end Initialize_Fields_For_Delay_Stmt;
   function Delay_Stmt_F_Has_Until
     (Node : Bare_Delay_Stmt) return Bare_Until_Node
   is
   begin
         return Node.Delay_Stmt_F_Has_Until;
   end;
   function Delay_Stmt_F_Expr
     (Node : Bare_Delay_Stmt) return Bare_Expr
   is
   begin
         return Node.Delay_Stmt_F_Expr;
   end;
--# property-start DelayStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21215
pragma Warnings (Off, "is not referenced");
 function Delay_Stmt_P_Xref_Equation
  (Node : Bare_Delay_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Delay_Stmt  := Bare_Delay_Stmt (Node);
     Ent : Internal_Entity_Delay_Stmt :=
       Internal_Entity_Delay_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1450 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Bare_Until_Node;
Fld_8 : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1450 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1450;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11441 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11440 '<And at ast.py:21217>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21217
--# expr-start 11428 '<FieldAccess for sub_equation at ast.py:21217>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21217
--# expr-start 11427 '<FieldAccess for expr at ast.py:21217>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21217
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Delay_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 11427
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11428
--# expr-start 11439 '<If at ast.py:21217>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21217
--# expr-start 11430 '<FieldAccess for as_bool at ast.py:21218>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21218
--# expr-start 11429 '<FieldAccess for has_until at ast.py:21218>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21218
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Delay_Stmt_F_Has_Until;
--# expr-done 11429
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Until.as_bool'
Fld_8 := Libadalang.Implementation.Dispatcher_Until_Node_P_As_Bool (Node => Fld_7);
--# end
--# expr-done 11430
if Fld_8 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21219") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 11438 '<And at ast.py:21220>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21220
--# expr-start 11435 '<Bind at ast.py:21220>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21220
--# expr-start 11432 '<FieldAccess for expected_type_var at ast.py:21220>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21220
--# expr-start 11431 '<FieldAccess for expr at ast.py:21220>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21220
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Delay_Stmt_F_Expr;
--# expr-done 11431
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 11432
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 11434 '<FieldAccess for std_entity at ast.py:21220>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21220
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11433 '<Symbol Duration at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Duration); 
--# expr-done 11433
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 11434
Bind_Result := Solver.Create_Assign (Fld_3, Fld_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21220") else null)); 
--# expr-done 11435
--# expr-start 11437 '<FieldAccess for matches_expected_type at ast.py:21221>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21221
--# expr-start 11436 '<FieldAccess for expr at ast.py:21221>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21221
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Delay_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 11436
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 11437
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21220") else null)); 
--# expr-done 11438
   If_Result := And_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 11439
And_Pred_1 := Create_And (Fld_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21217") else null)); 
--# expr-done 11440
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 11441
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1450;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1450;
            raise;
      end;
   return Property_Result;
end Delay_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Exit_Stmt
   --
      procedure Initialize_Fields_For_Exit_Stmt
        (Self : Bare_Exit_Stmt
         ; Exit_Stmt_F_Loop_Name : Bare_Name
         ; Exit_Stmt_F_Cond_Expr : Bare_Expr
        ) is
      begin
            Self.Exit_Stmt_F_Loop_Name := Exit_Stmt_F_Loop_Name;
            Self.Exit_Stmt_F_Cond_Expr := Exit_Stmt_F_Cond_Expr;
      end Initialize_Fields_For_Exit_Stmt;
   function Exit_Stmt_F_Loop_Name
     (Node : Bare_Exit_Stmt) return Bare_Name
   is
   begin
         return Node.Exit_Stmt_F_Loop_Name;
   end;
   function Exit_Stmt_F_Cond_Expr
     (Node : Bare_Exit_Stmt) return Bare_Expr
   is
   begin
         return Node.Exit_Stmt_F_Cond_Expr;
   end;
--# property-start ExitStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21081
pragma Warnings (Off, "is not referenced");
 function Exit_Stmt_P_Xref_Equation
  (Node : Bare_Exit_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exit_Stmt  := Bare_Exit_Stmt (Node);
     Ent : Internal_Entity_Exit_Stmt :=
       Internal_Entity_Exit_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1444 with Inline_Always;
            procedure Finalizer_Scope_3307 with Inline_Always;
            procedure Finalizer_Scope_3308 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Name;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1444 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1444;
            procedure Finalizer_Scope_3307 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3307;
            procedure Finalizer_Scope_3308 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3308;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11453 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11452 '<And at ast.py:21083>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21083
--# expr-start 11442 '<FieldAccess for cond_expr at ast.py:21084>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21084
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Exit_Stmt_F_Cond_Expr, Info => Ent.Info);
--# expr-done 11442
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind cond Var_Expr
--# expr-start 11449 '<And at ast.py:21086>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21086
--# expr-start 11447 '<And at ast.py:21086>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21086
--# expr-start 11445 '<Bind at ast.py:21086>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21086
--# expr-start 11443 '<FieldAccess for expected_type_var at ast.py:21086>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21086
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 11443
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 11444 '<FieldAccess for bool_type at ast.py:21086>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21086
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11444
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21086") else null)); 
--# expr-done 11445
--# expr-start 11446 '<FieldAccess for sub_equation at ast.py:21087>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21087
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 11446
And_Pred := Create_And (Bind_Result, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21086") else null)); 
--# expr-done 11447
--# expr-start 11448 '<FieldAccess for matches_expected_formal_prim_type at ast.py:21088>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21088
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_prim_type
Fld_4 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Prim_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 11448
And_Pred_1 := Create_And (And_Pred, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21086") else null)); 
--# expr-done 11449
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3307;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21089") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 11450 '<FieldAccess for loop_name at ast.py:21092>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21092
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Exit_Stmt_F_Loop_Name, Info => Ent.Info);
--# expr-done 11450
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind ln Var_Expr_1
--# expr-start 11451 '<FieldAccess for xref_no_overloading at ast.py:21093>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21093
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_6 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11451
Scope_Result_1 := Fld_6;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3308;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21094") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_2 := Create_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21083") else null)); 
--# expr-done 11452
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 11453
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1444;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1444;
                     Finalizer_Scope_3307;
                     Finalizer_Scope_3308;
            raise;
      end;
   return Property_Result;
end Exit_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Goto_Stmt
   --
      procedure Initialize_Fields_For_Goto_Stmt
        (Self : Bare_Goto_Stmt
         ; Goto_Stmt_F_Label_Name : Bare_Name
        ) is
      begin
            Self.Goto_Stmt_F_Label_Name := Goto_Stmt_F_Label_Name;
      end Initialize_Fields_For_Goto_Stmt;
   function Goto_Stmt_F_Label_Name
     (Node : Bare_Goto_Stmt) return Bare_Name
   is
   begin
         return Node.Goto_Stmt_F_Label_Name;
   end;
--# property-start GotoStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21068
pragma Warnings (Off, "is not referenced");
 function Goto_Stmt_P_Xref_Equation
  (Node : Bare_Goto_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Goto_Stmt  := Bare_Goto_Stmt (Node);
     Ent : Internal_Entity_Goto_Stmt :=
       Internal_Entity_Goto_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1443 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1443 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1443;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11456 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11455 '<FieldAccess for xref_no_overloading at ast.py:21070>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21070
--# expr-start 11454 '<FieldAccess for label_name at ast.py:21070>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21070
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Goto_Stmt_F_Label_Name, Info => Ent.Info);
--# expr-done 11454
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => False, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11455
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 11456
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1443;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1443;
            raise;
      end;
   return Property_Result;
end Goto_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Label
   --
      procedure Initialize_Fields_For_Label
        (Self : Bare_Label
         ; Label_F_Decl : Bare_Label_Decl
        ) is
      begin
            Self.Label_F_Decl := Label_F_Decl;
      end Initialize_Fields_For_Label;
   function Label_F_Decl
     (Node : Bare_Label) return Bare_Label_Decl
   is
   begin
         return Node.Label_F_Decl;
   end;
--# property-start Label.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21310
pragma Warnings (Off, "is not referenced");
 function Label_P_Xref_Equation
  (Node : Bare_Label
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Label  := Bare_Label (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1456 with Inline_Always;
      True_Rel : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1456 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1456;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11457 '<Block at ???>' Let_Result None
--# scope-start
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21312") else null)); 
Let_Result := True_Rel; Inc_Ref (Let_Result);
--# end
--# expr-done 11457
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1456;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1456;
            raise;
      end;
   return Property_Result;
end Label_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Null_Stmt
   --
--# property-start NullStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21033
pragma Warnings (Off, "is not referenced");
 function Null_Stmt_P_Xref_Equation
  (Node : Bare_Null_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Null_Stmt  := Bare_Null_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1440 with Inline_Always;
      True_Rel : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1440 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1440;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11458 '<Block at ???>' Let_Result None
--# scope-start
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21035") else null)); 
Let_Result := True_Rel; Inc_Ref (Let_Result);
--# end
--# expr-done 11458
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1440;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1440;
            raise;
      end;
   return Property_Result;
end Null_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Raise_Stmt
   --
      procedure Initialize_Fields_For_Raise_Stmt
        (Self : Bare_Raise_Stmt
         ; Raise_Stmt_F_Exception_Name : Bare_Name
         ; Raise_Stmt_F_Error_Message : Bare_Expr
        ) is
      begin
            Self.Raise_Stmt_F_Exception_Name := Raise_Stmt_F_Exception_Name;
            Self.Raise_Stmt_F_Error_Message := Raise_Stmt_F_Error_Message;
      end Initialize_Fields_For_Raise_Stmt;
   function Raise_Stmt_F_Exception_Name
     (Node : Bare_Raise_Stmt) return Bare_Name
   is
   begin
         return Node.Raise_Stmt_F_Exception_Name;
   end;
   function Raise_Stmt_F_Error_Message
     (Node : Bare_Raise_Stmt) return Bare_Expr
   is
   begin
         return Node.Raise_Stmt_F_Error_Message;
   end;
--# property-start RaiseStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21233
pragma Warnings (Off, "is not referenced");
 function Raise_Stmt_P_Xref_Equation
  (Node : Bare_Raise_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Raise_Stmt  := Bare_Raise_Stmt (Node);
     Ent : Internal_Entity_Raise_Stmt :=
       Internal_Entity_Raise_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1451 with Inline_Always;
            procedure Finalizer_Scope_3309 with Inline_Always;
            procedure Finalizer_Scope_3310 with Inline_Always;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1451 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1451;
            procedure Finalizer_Scope_3309 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3309;
            procedure Finalizer_Scope_3310 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3310;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11469 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11468 '<And at ast.py:21235>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21235
--# expr-start 11459 '<FieldAccess for exception_name at ast.py:21236>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21236
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Raise_Stmt_F_Exception_Name, Info => Ent.Info);
--# expr-done 11459
Var_Expr := Fld;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind en Var_Expr
--# expr-start 11460 '<FieldAccess for sub_equation at ast.py:21237>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21237
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 11460
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3309;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21238") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 11461 '<FieldAccess for error_message at ast.py:21240>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21240
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Stmt_F_Error_Message, Info => Ent.Info);
--# expr-done 11461
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind er Var_Expr_1
--# expr-start 11467 '<And at ast.py:21241>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21241
--# expr-start 11465 '<Bind at ast.py:21244>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21244
--# expr-start 11462 '<FieldAccess for expected_type_var at ast.py:21244>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21244
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 11462
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 11464 '<FieldAccess for std_entity at ast.py:21244>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21244
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11463 '<Symbol String at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 11463
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 11464
Bind_Result := Solver.Create_Assign (Fld_3, Fld_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21244") else null)); 
--# expr-done 11465
--# expr-start 11466 '<FieldAccess for sub_equation at ast.py:21245>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21245
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11466
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21241") else null)); 
--# expr-done 11467
Scope_Result_1 := And_Pred;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3310;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21247") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Create_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21235") else null)); 
--# expr-done 11468
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 11469
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1451;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1451;
                     Finalizer_Scope_3309;
                     Finalizer_Scope_3310;
            raise;
      end;
   return Property_Result;
end Raise_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Requeue_Stmt
   --
      procedure Initialize_Fields_For_Requeue_Stmt
        (Self : Bare_Requeue_Stmt
         ; Requeue_Stmt_F_Call_Name : Bare_Name
         ; Requeue_Stmt_F_Has_Abort : Bare_Abort_Node
        ) is
      begin
            Self.Requeue_Stmt_F_Call_Name := Requeue_Stmt_F_Call_Name;
            Self.Requeue_Stmt_F_Has_Abort := Requeue_Stmt_F_Has_Abort;
      end Initialize_Fields_For_Requeue_Stmt;
   function Requeue_Stmt_F_Call_Name
     (Node : Bare_Requeue_Stmt) return Bare_Name
   is
   begin
         return Node.Requeue_Stmt_F_Call_Name;
   end;
   function Requeue_Stmt_F_Has_Abort
     (Node : Bare_Requeue_Stmt) return Bare_Abort_Node
   is
   begin
         return Node.Requeue_Stmt_F_Has_Abort;
   end;
--# property-start RequeueStmt.innermost_entry_or_accept_stmt_params /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21131
pragma Warnings (Off, "is not referenced");
 function Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params
  (Node : Bare_Requeue_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Requeue_Stmt  := Bare_Requeue_Stmt (Node);
     Ent : Internal_Entity_Requeue_Stmt :=
       Internal_Entity_Requeue_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
            procedure Finalizer_Scope_1447 with Inline_Always;
      Match_0 : Internal_Entity_Accept_Stmt_With_Stmts;
Match_1 : Internal_Entity_Entry_Body;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Fld_1 : Internal_Entity_Entry_Completion_Formal_Params;
Fld_2 : Internal_Entity_Entry_Completion_Formal_Params;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt_With_Stmts;
Let_Result : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_1 : Internal_Entity_Entry_Body;
Let_Result_1 : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result_1 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result_2 : Internal_Entity_Entry_Completion_Formal_Params;
Match_Result : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
            procedure Finalizer_Scope_1447 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1447;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11476 '<Cast to BaseFormalParamHolder.entity at ast.py:21138>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21138
--# expr-start 11475 '<Match at ast.py:21134>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21134
--# expr-start 11472 '<Find at ast.py:21132>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21132
   --# expr-start 11470 '<FieldAccess for parents at ast.py:21132>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21132
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 11470
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_352 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_352
      --# expr-start 11471 '<IsA AcceptStmtWithStmts, EntryBody at ast.py:21133>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21133
Is_A := Item_352.Node /= null 
and then Item_352.Node.Kind in Ada_Accept_Stmt_With_Stmts_Range | Ada_Entry_Body_Range; 
--# expr-done 11471
      if Is_A then
         Find_Result := Item_352; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 11472
      if Find_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Find_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Accept_Stmt_With_Stmts =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind a Match_0
--# expr-start 11473 '<FieldAccess for params at ast.py:21135>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21135
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_0.Node.Accept_Stmt_F_Params, Info => Match_0.Info);
--# expr-done 11473
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind b Match_1
--# expr-start 11474 '<FieldAccess for params at ast.py:21136>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21136
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_1.Node.Entry_Body_F_Params, Info => Match_1.Info);
--# expr-done 11474
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Accept_Stmt | Ada_For_Loop_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Entry_Completion_Formal_Params; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 11475
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Match_Result.Node,
         Info => Match_Result.Info);
--# expr-done 11476
         Property_Result := Cast_Result_3;
   --# end
      Finalizer_Scope_1447;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1447;
            raise;
      end;
   return Property_Result;
end Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params;
--# end
--# property-start RequeueStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21141
pragma Warnings (Off, "is not referenced");
 function Requeue_Stmt_P_Xref_Equation
  (Node : Bare_Requeue_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Requeue_Stmt  := Bare_Requeue_Stmt (Node);
     Ent : Internal_Entity_Requeue_Stmt :=
       Internal_Entity_Requeue_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1448 with Inline_Always;
            procedure Finalizer_Scope_3319 with Inline_Always;
            procedure Finalizer_Scope_3324 with Inline_Always;
            procedure Finalizer_Scope_3325 with Inline_Always;
      Block_Var : Internal_Entity_Call_Expr;
Block_Var_1 : Internal_Entity_Name;
Block_Var_2 : Internal_Entity_Array_Access;
Var_Expr : Internal_Entity_Call_Expr;
Var_Expr_1 : Internal_Entity_Entry_Decl;
Var_Expr_2 : Internal_Entity_Base_Subp_Spec;
Fam_Type : Internal_Entity_Base_Type_Decl;
First_Param : Internal_Entity_Expr;
Var_Expr_3 : Internal_Entity_Subtype_Indication;
Var_Expr_4 : Internal_Entity_Entry_Decl;
Var_Expr_5 : Internal_Entity_Basic_Assoc;
Var_Expr_6 : Internal_Entity_Call_Expr;
Var_Expr_7 : Internal_Entity_Expr;
Var_Expr_8 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Fld_1 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Entry_Decl;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Integer;
Is_Equal : Boolean;
Fld_6 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Entry_Spec;
Fld_8 : Internal_Entity_Base_Formal_Param_Holder;
Fld_9 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_10 : Logic_Equation;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Entry_Decl;
Fld_11 : Internal_Entity_Entry_Spec;
Fld_12 : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Result_Var_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Fld_13 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Fld_14 : Internal_Entity_Assoc_List;
Scope_Result_5 : Internal_Entity_Assoc_List;
Result_Var_5 : Internal_Entity_Assoc_List;
Fld_15 : Bare_Assoc_List;
Fld_16 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Scope_Result_6 : Internal_Entity_Expr;
Result_Var_6 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_20 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_7 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_8 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1448 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1448;
            procedure Finalizer_Scope_3319 is
            begin
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_8);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3319;
            procedure Finalizer_Scope_3324 is
            begin
                     Dec_Ref (Fld_18);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3324;
            procedure Finalizer_Scope_3325 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3325;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11510 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 11478 '<Cast to CallExpr at ast.py:21143>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21143
--# expr-start 11477 '<FieldAccess for call_name at ast.py:21143>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21143
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Requeue_Stmt_F_Call_Name, Info => Ent.Info);
--# expr-done 11477
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Call_Expr;
   end if;
--# expr-done 11478
Block_Var := Cast_Result; 
--# bind ce Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr
--# expr-start 11479 '<FieldAccess for name at ast.py:21144>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21144
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Call_Expr_F_Name, Info => Var_Expr.Info);
--# expr-done 11479
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 11480 '<FieldAccess for call_name at ast.py:21145>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21145
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Requeue_Stmt_F_Call_Name, Info => Ent.Info);
--# expr-done 11480
   Result_Var := Fld_2;
end if;
Block_Var_1 := Result_Var; 
--# bind name Block_Var_1
--# expr-start 11491 '<Filter at ast.py:21147>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21147
   --# expr-start 11481 '<FieldAccess for all_env_elements_internal at ast.py:21147>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21147
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_3 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 11481
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_353 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_353
--# expr-start 11482 '<Cast to EntryDecl at ast.py:21150>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21150
Cast_Expr := Item_353; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Entry_Decl;
   end if;
--# expr-done 11482
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind e Var_Expr_1
--# expr-start 11483 '<FieldAccess for subp_spec_or_null at ast.py:21152>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21152
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11483
Var_Expr_2 := Fld_4;
if Var_Expr_2 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_2
--# expr-start 11485 '<Eq at ast.py:21153>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21153
--# expr-start 11484 '<FieldAccess for nb_max_params at ast.py:21153>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21153
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Var_Expr_2.Node);
--# end
--# expr-done 11484
Is_Equal := Fld_5 = 0; 
--# expr-done 11485
if Is_Equal then
   --# expr-start 11487 '<IsNull at ast.py:21154>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21154
--# expr-start 11486 '<FieldAccess for returns at ast.py:21154>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21154
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 11486
Is_Null := Fld_6.Node = null; 
--# expr-done 11487
   If_Result := Is_Null;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result_1 := True;
else
   --# expr-start 11490 '<FieldAccess for match_formal_params at ast.py:21157>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21157
--# expr-start 11488 '<FieldAccess for spec at ast.py:21157>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21157
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_1.Node.Entry_Decl_F_Spec, Info => Var_Expr_1.Info);
--# expr-done 11488
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11489 '<FieldAccess for innermost_entry_or_accept_stmt_params at ast.py:21158>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21158
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start RequeueStmt.innermost_entry_or_accept_stmt_params
Fld_8 := Libadalang.Implementation.Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11489
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_7.Node, Other => Fld_8, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 11490
   If_Result_1 := Fld_9;
end if;
Scope_Result_2 := If_Result_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
         if Result_Var_2 then
         declare
            Item_To_Append : constant Internal_Entity := Item_353;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11491
Block_Var_2 := Map_Result; Inc_Ref (Block_Var_2);
--# bind entries Block_Var_2
--# expr-start 11509 '<And at ast.py:21163>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21163
--# expr-start 11492 '<FieldAccess for xref_no_overloading at ast.py:21166>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21166
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_10 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 11492
--# expr-start 11508 '<LogicAny at ast.py:21169>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21169
--# expr-start 11507 '<Map at ast.py:21169>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21169
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_2;
      begin
         for Item_354 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_354
      --# expr-start 11506 '<Let at ast.py:21169>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21169
--# scope-start
--# expr-start 11496 '<Cast to SubtypeIndication at ast.py:21174>' Cast_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21174
--# expr-start 11493 '<Cast to EntryDecl at ast.py:21173>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21173
Cast_Expr_1 := Item_354; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Entry_Decl;
   end if;
--# expr-done 11493
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Entry_Decl then
   --# scope-start
--# expr-start 11495 '<FieldAccess for family_type at ast.py:21173>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21173
--# expr-start 11494 '<FieldAccess for spec at ast.py:21173>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21173
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_4.Node.Entry_Decl_F_Spec, Info => Var_Expr_4.Info);
--# expr-done 11494
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity (Node => Fld_11.Node.Entry_Spec_F_Family_Type, Info => Fld_11.Info);
--# expr-done 11495
Scope_Result_3 := Fld_12;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity;
end if;
   if Result_Var_3.Node = null
      or else Result_Var_3.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Result_Var_3.Node,
         Info => Result_Var_3.Info);
   else
         Cast_Result_3 := No_Entity_Subtype_Indication;
   end if;
--# expr-done 11496
Var_Expr_3 := Cast_Result_3;
if Var_Expr_3 /= No_Entity_Subtype_Indication then
   --# scope-start
--# expr-start 11497 '<FieldAccess for designated_type at ast.py:21174>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21174
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_3.Node, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 11497
Scope_Result_4 := Fld_13;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
Fam_Type := Result_Var_4; 
--# bind fam_type Fam_Type
Var_Expr_6 := Block_Var;
if Var_Expr_6 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 11498 '<FieldAccess for params at ast.py:21176>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21176
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_14 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 11498
Scope_Result_5 := Fld_14;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Assoc_List;
end if;
Fld_15 := Result_Var_5.Node;
Get_Result := Get (Self, Fld_15, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_16 := Result_Var_5.Info;
As_Entity := (Info => Fld_16, Node => Get_Result); 
   If_Result_2 := As_Entity;
end if;
Var_Expr_5 := If_Result_2;
if Var_Expr_5 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 11499 '<FieldAccess for expr at ast.py:21176>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21176
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 11499
Scope_Result_6 := Fld_17;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Expr;
end if;
First_Param := Result_Var_6; 
--# bind first_param First_Param
Var_Expr_7 := First_Param;
if Var_Expr_7 /= No_Entity_Expr then
   --# scope-start
   --# bind p Var_Expr_7
--# expr-start 11505 '<And at ast.py:21179>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21179
--# expr-start 11500 '<FieldAccess for sub_equation at ast.py:21179>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21179
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_7.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 11500
Var_Expr_8 := Fam_Type;
if Var_Expr_8 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind eft Var_Expr_8
--# expr-start 11504 '<And at ast.py:21181>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21181
--# expr-start 11502 '<Bind at ast.py:21181>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21181
--# expr-start 11501 '<FieldAccess for expected_type_var at ast.py:21181>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21181
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_7.Node);
--# expr-done 11501
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
Cast_Expr_2 := Var_Expr_8; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result := Solver.Create_Assign (Fld_19, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21181") else null)); 
--# expr-done 11502
--# expr-start 11503 '<FieldAccess for matches_expected_type at ast.py:21182>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21182
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_20 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr_7.Node, Origin => Origin);
--# end
--# expr-done 11503
And_Pred := Create_And (Bind_Result, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21181") else null)); 
--# expr-done 11504
Scope_Result_7 := And_Pred;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3325;
   Result_Var_7 := Scope_Result_7;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21183") else null)); 
   Result_Var_7 := True_Rel;
end if;
      Inc_Ref (Result_Var_7);
And_Pred_1 := Create_And (Fld_18, Result_Var_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21179") else null)); 
--# expr-done 11505
Scope_Result_8 := And_Pred_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_3324;
   Result_Var_8 := Scope_Result_8;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21185") else null)); 
   Result_Var_8 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_8);
Let_Result := Result_Var_8; Inc_Ref (Let_Result);
--# end
--# expr-done 11506
         declare
            Item_To_Append : constant Logic_Equation := Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3319;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 11507
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21169") else null)); 
--# expr-done 11508
And_Pred_2 := Create_And (Fld_10, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21163") else null)); 
--# expr-done 11509
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 11510
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1448;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1448;
                     Finalizer_Scope_3319;
                     Finalizer_Scope_3324;
                     Finalizer_Scope_3325;
            raise;
      end;
   return Property_Result;
end Requeue_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Return_Stmt
   --
      procedure Initialize_Fields_For_Return_Stmt
        (Self : Bare_Return_Stmt
         ; Return_Stmt_F_Return_Expr : Bare_Expr
        ) is
      begin
            Self.Return_Stmt_F_Return_Expr := Return_Stmt_F_Return_Expr;
      end Initialize_Fields_For_Return_Stmt;
   function Return_Stmt_F_Return_Expr
     (Node : Bare_Return_Stmt) return Bare_Expr
   is
   begin
         return Node.Return_Stmt_F_Return_Expr;
   end;
--# property-start ReturnStmt.subp /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21106
pragma Warnings (Off, "is not referenced");
 function Return_Stmt_P_Subp
  (Node : Bare_Return_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Subp_Body
is
   Self : Bare_Return_Stmt  := Bare_Return_Stmt (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Subp_Body;
            procedure Finalizer_Scope_1445 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Subp_Body;
As_Entity : Internal_Entity_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Subp_Body;
            procedure Finalizer_Scope_1445 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1445;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11515 '<as_entity at ast.py:21107>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21107
--# expr-start 11514 '<Cast to SubpBody at ast.py:21107>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21107
--# expr-start 11513 '<Find at ast.py:21107>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21107
   --# expr-start 11511 '<FieldAccess for parents at ast.py:21107>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21107
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 11511
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_355 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_355
      --# expr-start 11512 '<IsA SubpBody at ast.py:21107>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21107
Is_A := Item_355 /= null 
and then Item_355.Kind in Ada_Subp_Body_Range; 
--# expr-done 11512
      if Is_A then
         Find_Result := Item_355; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 11513
   if Find_Result = null
      or else Find_Result.Kind in Ada_Subp_Body_Range
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 11514
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Subp_Body;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 11515
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_1445;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1445;
            raise;
      end;
   return Property_Result;
end Return_Stmt_P_Subp;
--# end
--# property-start ReturnStmt.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21111
pragma Warnings (Off, "is not referenced");
 function Return_Stmt_P_Xref_Equation
  (Node : Bare_Return_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Return_Stmt  := Bare_Return_Stmt (Node);
     Ent : Internal_Entity_Return_Stmt :=
       Internal_Entity_Return_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1446 with Inline_Always;
            procedure Finalizer_Scope_3327 with Inline_Always;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Subp_Body;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Internal_Entity_Type_Expr;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1446 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1446;
            procedure Finalizer_Scope_3327 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3327;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11527 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11516 '<FieldAccess for return_expr at ast.py:21113>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21113
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Return_Stmt_F_Return_Expr, Info => Ent.Info);
--# expr-done 11516
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind rexpr Var_Expr
--# expr-start 11526 '<And at ast.py:21115>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21115
--# expr-start 11524 '<And at ast.py:21115>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21115
--# expr-start 11522 '<Bind at ast.py:21115>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21115
--# expr-start 11517 '<FieldAccess for expected_type_var at ast.py:21115>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21115
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr.Node);
--# expr-done 11517
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 11521 '<FieldAccess for designated_type at ast.py:21116>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21116
--# expr-start 11520 '<FieldAccess for returns at ast.py:21116>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21116
--# expr-start 11519 '<FieldAccess for subp_spec at ast.py:21116>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21116
--# expr-start 11518 '<FieldAccess for subp at ast.py:21116>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21116
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ReturnStmt.subp
Fld_2 := Libadalang.Implementation.Return_Stmt_P_Subp (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11518
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Fld_2.Node.Base_Subp_Body_F_Subp_Spec, Info => Fld_2.Info);
--# expr-done 11519
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 11520
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 11521
      Cast_Result := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21115") else null)); 
--# expr-done 11522
--# expr-start 11523 '<FieldAccess for sub_equation at ast.py:21117>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21117
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 11523
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21115") else null)); 
--# expr-done 11524
--# expr-start 11525 '<FieldAccess for matches_expected_assign_type at ast.py:21118>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21118
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_7 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 11525
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21115") else null)); 
--# expr-done 11526
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3327;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21119") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 11527
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1446;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1446;
                     Finalizer_Scope_3327;
            raise;
      end;
   return Property_Result;
end Return_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Terminate_Alternative
   --
--# property-start TerminateAlternative.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21632
pragma Warnings (Off, "is not referenced");
 function Terminate_Alternative_P_Xref_Equation
  (Node : Bare_Terminate_Alternative
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Terminate_Alternative  := Bare_Terminate_Alternative (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1477 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1477 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1477;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:21632") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1477;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1477;
            raise;
      end;
   return Property_Result;
end Terminate_Alternative_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Subp_Kind
   --
   --
   --  Primitives for Bare_Subp_Kind_Function
   --
   --
   --  Primitives for Bare_Subp_Kind_Procedure
   --
   --
   --  Primitives for Bare_Subunit
   --
      procedure Initialize_Fields_For_Subunit
        (Self : Bare_Subunit
         ; Subunit_F_Name : Bare_Name
         ; Subunit_F_Body : Bare_Body_Node
        ) is
      begin
            Self.Subunit_F_Name := Subunit_F_Name;
            Self.Subunit_F_Body := Subunit_F_Body;
      end Initialize_Fields_For_Subunit;
   function Subunit_F_Name
     (Node : Bare_Subunit) return Bare_Name
   is
   begin
         return Node.Subunit_F_Name;
   end;
   function Subunit_F_Body
     (Node : Bare_Subunit) return Bare_Body_Node
   is
   begin
         return Node.Subunit_F_Body;
   end;
--# property-start Subunit.root_unit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21970
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Root_Unit
  (Node : Bare_Subunit
  )
   return Internal_Entity_Compilation_Unit
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Compilation_Unit;
            procedure Finalizer_Scope_1498 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Let_Result : Internal_Entity_Compilation_Unit;
            procedure Finalizer_Scope_1498 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1498;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11532 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11531 '<as_bare_entity at ast.py:21980>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21980
--# expr-start 11530 '<FieldAccess for designated_compilation_unit at ast.py:21975>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21975
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11529 '<FieldAccess for as_symbol_array at ast.py:21976>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21976
--# expr-start 11528 '<FieldAccess for name at ast.py:21976>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21976
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 11528
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld);
--# end
--# expr-done 11529
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld_1, Kind => Unit_Body, Load_If_Needed => True, Not_Found_Is_Error => True);
--# end
--# expr-done 11530
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 11531
Let_Result := If_Result; 
--# end
--# expr-done 11532
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1498;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1498;
            raise;
      end;
   return Property_Result;
end Subunit_P_Root_Unit;
--# end
--# property-start Subunit.env_hook_subunit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21982
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Env_Hook_Subunit
  (Node : Bare_Subunit
  )
   return Boolean
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity_Compilation_Unit;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11534 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11533 '<FieldAccess for root_unit at ast.py:21989>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21989
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Self);
--# end
--# expr-done 11533
Block_Var := Fld; 
Let_Result := False; 
--# end
--# expr-done 11534
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subunit_P_Env_Hook_Subunit;
--# end
--# property-start Subunit.body_root /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21992
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Body_Root
  (Node : Bare_Subunit
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 11538 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11537 '<as_bare_entity at ast.py:21997>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21997
--# expr-start 11536 '<FieldAccess for decl at ast.py:21997>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21997
--# expr-start 11535 '<FieldAccess for root_unit at ast.py:21997>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21997
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Self);
--# end
--# expr-done 11535
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Fld.Node);
--# end
--# expr-done 11536
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 11537
Let_Result := If_Result; 
--# end
--# expr-done 11538
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subunit_P_Body_Root;
--# end
--# property-start Subunit.stub /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:21999
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Stub
  (Node : Bare_Subunit
  )
   return Bare_Body_Stub
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Bare_Body_Stub;
      Block_Var : Bare_Compilation_Unit;
Block_Var_1 : Bare_Body_Node;
Block_Var_2 : Bare_Ada_Node_List;
Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Match_0_1 : Bare_Subp_Body;
Match_1_1 : Bare_Package_Body;
Match_2_1 : Bare_Body_Node;
Stub_Name : Internal_Entity_Name;
Subunit_Name : Internal_Entity_Name;
Var_Expr : Internal_Entity_Body_Stub;
Var_Expr_1 : Internal_Entity_Name;
Fld : Internal_Entity_Compilation_Unit;
Fld_1 : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Fld_3 : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Fld_4 : Bare_Body_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Let_Result : Bare_Body_Node;
Scope_Result : Bare_Body_Node;
Cast_Result_2 : Bare_Subunit;
Let_Result_1 : Bare_Body_Node;
Scope_Result_1 : Bare_Body_Node;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_2 : Bare_Body_Node;
Scope_Result_2 : Bare_Body_Node;
Match_Result : Bare_Body_Node;
Fld_5 : Bare_Declarative_Part;
Fld_6 : Bare_Ada_Node_List;
Fld_7 : Bare_Declarative_Part;
Fld_8 : Bare_Ada_Node_List;
Match_Prefix_1 : Bare_Body_Node;
Cast_Result_4 : Bare_Subp_Body;
Let_Result_3 : Bare_Ada_Node_List;
Scope_Result_3 : Bare_Ada_Node_List;
Cast_Result_5 : Bare_Package_Body;
Let_Result_4 : Bare_Ada_Node_List;
Scope_Result_4 : Bare_Ada_Node_List;
Cast_Result_6 : Bare_Body_Node;
Let_Result_5 : Bare_Ada_Node_List;
Scope_Result_5 : Bare_Ada_Node_List;
Match_Result_1 : Bare_Ada_Node_List;
Cast_Expr : Bare_Ada_Node;
Cast_Result_7 : Bare_Body_Stub;
As_Entity : Internal_Entity_Body_Stub;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Stub;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Name;
Scope_Result_6 : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_11 : Bare_Body_Node;
As_Entity_1 : Internal_Entity_Body_Node;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Body_Node;
Fld_12 : Internal_Entity_Defining_Name;
Fld_13 : Internal_Entity_Name;
Fld_14 : Symbol_Type;
Fld_15 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_1 : Boolean;
Let_Result_6 : Boolean;
Item_356 : Bare_Ada_Node;
Find_Result : Bare_Ada_Node;
Cast_Result_8 : Bare_Body_Stub;
Let_Result_7 : Bare_Body_Stub;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11564 '<Block at ???>' Let_Result_7 None
--# scope-start
--# expr-start 11540 '<FieldAccess for node at ast.py:22021>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22021
--# expr-start 11539 '<FieldAccess for root_unit at ast.py:22021>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22021
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Self);
--# end
--# expr-done 11539
Fld_1 := Fld.Node;
--# expr-done 11540
Block_Var := Fld_1; 
--# bind root Block_Var
--# expr-start 11545 '<Match at ast.py:22022>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22022
--# expr-start 11541 '<FieldAccess for body at ast.py:22022>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22022
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Block_Var.Compilation_Unit_F_Body;
--# expr-done 11541
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_2; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_0 := Cast_Result_1; 
--# bind li Match_0
--# expr-start 11543 '<Cast to Body at ast.py:22023>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22023
--# expr-start 11542 '<FieldAccess for item at ast.py:22023>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22023
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Match_0.Library_Item_F_Item;
--# expr-done 11542
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Body_Node
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 11543
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind su Match_1
--# expr-start 11544 '<FieldAccess for body at ast.py:22024>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22024
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Match_1.Subunit_F_Body;
--# expr-done 11544
Let_Result_1 := Fld_4; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 11545
Block_Var_1 := Match_Result; 
--# bind root_body Block_Var_1
--# expr-start 11550 '<Match at ast.py:22027>' Match_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22027
      if Block_Var_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Block_Var_1; 
case Ada_Body_Node (Match_Prefix_1.Kind) is
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_0_1 := Cast_Result_4; 
--# bind subp Match_0_1
--# expr-start 11547 '<FieldAccess for decls at ast.py:22028>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22028
--# expr-start 11546 '<FieldAccess for decls at ast.py:22028>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22028
      if Match_0_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Match_0_1.Subp_Body_F_Decls;
--# expr-done 11546
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Fld_5.Declarative_Part_F_Decls;
--# expr-done 11547
Let_Result_3 := Fld_6; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix_1;
Match_1_1 := Cast_Result_5; 
--# bind pkg Match_1_1
--# expr-start 11549 '<FieldAccess for decls at ast.py:22029>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22029
--# expr-start 11548 '<FieldAccess for decls at ast.py:22029>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22029
      if Match_1_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Match_1_1.Package_Body_F_Decls;
--# expr-done 11548
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Fld_7.Declarative_Part_F_Decls;
--# expr-done 11549
Let_Result_4 := Fld_8; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Accept_Stmt_Body .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix_1;
Match_2_1 := Cast_Result_6; 
--# bind _ Match_2_1
Let_Result_5 := No_Bare_Ada_Node; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
end case;
--# expr-done 11550
Block_Var_2 := Match_Result_1; 
--# bind decls Block_Var_2
--# expr-start 11563 '<Cast to BodyStub at ast.py:22055>' Cast_Result_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22055
--# expr-start 11562 '<Find at ast.py:22035>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22035
      Find_Result := No_Bare_Ada_Node;
      if Block_Var_2 = null then
      Find_Result := No_Bare_Ada_Node;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Block_Var_2;
      begin
         for Untyped_Item_356 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_356 := Untyped_Item_356; 
   --# scope-start
               --# bind d Item_356
      --# expr-start 11561 '<Let at ast.py:22036>' Let_Result_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22036
--# scope-start
--# expr-start 11552 '<as_bare_entity at ast.py:22041>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22041
--# expr-start 11551 '<Cast to BodyStub at ast.py:22041>' Cast_Result_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22041
Cast_Expr := Item_356; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Body_Stub
   then
      Cast_Result_7 := Cast_Expr;
   else
         Cast_Result_7 := No_Bare_Ada_Node;
   end if;
--# expr-done 11551
Is_Null := Cast_Result_7 = null; 
if Is_Null then
   If_Result := No_Entity_Body_Stub;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result_7); 
   If_Result := As_Entity;
end if;
--# expr-done 11552
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Body_Stub then
   --# scope-start
--# expr-start 11554 '<FieldAccess for name at ast.py:22043>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22043
--# expr-start 11553 '<FieldAccess for defining_name_or_raise at ast.py:22042>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22042
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 11553
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Fld_9.Node.Defining_Name_F_Name, Info => Fld_9.Info);
--# expr-done 11554
Scope_Result_6 := Fld_10;
   --# end
   Result_Var := Scope_Result_6;
else
   Result_Var := No_Entity_Name;
end if;
Stub_Name := Result_Var; 
--# bind stub_name Stub_Name
--# expr-start 11558 '<FieldAccess for name at ast.py:22050>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22050
--# expr-start 11557 '<FieldAccess for defining_name_or_raise at ast.py:22049>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22049
--# expr-start 11556 '<as_bare_entity at ast.py:22048>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22048
--# expr-start 11555 '<FieldAccess for body at ast.py:22048>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22048
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Subunit_F_Body;
--# expr-done 11555
Is_Null_1 := Fld_11 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Body_Node;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_11); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 11556
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 11557
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Fld_12.Node.Defining_Name_F_Name, Info => Fld_12.Info);
--# expr-done 11558
Subunit_Name := Fld_13; 
--# bind subunit_name Subunit_Name
Var_Expr_1 := Stub_Name;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
--# expr-start 11560 '<FieldAccess for name_is at ast.py:22053>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22053
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11559 '<FieldAccess for name_symbol at ast.py:22053>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22053
      if Subunit_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Subunit_Name.Node);
--# end
--# expr-done 11559
--# property-call-start Name.name_is
Fld_15 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_1.Node, Sym => Fld_14);
--# end
--# expr-done 11560
Scope_Result_7 := Fld_15;
   --# end
   Result_Var_1 := Scope_Result_7;
else
   Result_Var_1 := False;
end if;
Let_Result_6 := Result_Var_1; 
--# end
--# expr-done 11561
      if Let_Result_6 then
         Find_Result := Item_356; 
      end if;
   --# end
               exit when Let_Result_6;
         end loop;
      end;
      end if;
--# expr-done 11562
   if Find_Result = null
      or else Find_Result.Kind in Ada_Body_Stub
   then
      Cast_Result_8 := Find_Result;
   else
         Cast_Result_8 := No_Bare_Ada_Node;
   end if;
--# expr-done 11563
Let_Result_7 := Cast_Result_8; 
--# end
--# expr-done 11564
         Property_Result := Let_Result_7;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subunit_P_Stub;
--# end
--# property-start Subunit.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22057
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Xref_Entry_Point
  (Node : Bare_Subunit
  )
   return Boolean
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subunit_P_Xref_Entry_Point;
--# end
--# property-start Subunit.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22059
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Xref_Equation
  (Node : Bare_Subunit
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
     Ent : Internal_Entity_Subunit :=
       Internal_Entity_Subunit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1503 with Inline_Always;
            procedure Finalizer_Scope_3337 with Inline_Always;
      Var_Expr : Internal_Entity_Dotted_Name;
Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Fld_4 : Bare_Body_Node;
Fld_5 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1503 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1503;
            procedure Finalizer_Scope_3337 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3337;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11577 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11576 '<And at ast.py:22061>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22061
--# expr-start 11568 '<Bind at ast.py:22062>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22062
--# expr-start 11566 '<FieldAccess for ref_var at ast.py:22062>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22062
--# expr-start 11565 '<FieldAccess for name at ast.py:22062>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22062
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 11565
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 11566
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 11567 '<FieldAccess for body_root at ast.py:22062>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22062
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.body_root
Fld_2 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Ent.Node);
--# end
--# expr-done 11567
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:22062") else null)); 
--# expr-done 11568
--# expr-start 11570 '<Cast to DottedName at ast.py:22065>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22065
--# expr-start 11569 '<FieldAccess for name at ast.py:22065>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22065
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Subunit_F_Name, Info => Ent.Info);
--# expr-done 11569
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Dotted_Name;
   end if;
--# expr-done 11570
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 11575 '<bind at ast.py:22066>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22066
--# expr-start 11572 '<FieldAccess for node_env at ast.py:22067>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22067
--# expr-start 11571 '<FieldAccess for body at ast.py:22067>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22067
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Subunit_F_Body;
--# expr-done 11571
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Node_Env (Node => Fld_4);
--# expr-done 11572
Bound_Env := Fld_5; Inc_Ref (Bound_Env);
--# expr-start 11574 '<FieldAccess for xref_no_overloading at ast.py:22068>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22068
--# expr-start 11573 '<FieldAccess for prefix at ast.py:22068>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22068
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 11573
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_7 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_6.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 11574
Dyn_Var_Bind_Result := Fld_7; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 11575
Scope_Result := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3337;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:22070") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Bind_Result, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:22061") else null)); 
--# expr-done 11576
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 11577
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1503;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1503;
                     Finalizer_Scope_3337;
            raise;
      end;
   return Property_Result;
end Subunit_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Synchronized_Node
   --
--# property-start '[dispatcher]Synchronized.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Synchronized_Node_P_As_Bool
  (Node : Bare_Synchronized_Node
  )
   return Boolean
is
   Self : Bare_Synchronized_Node  := Bare_Synchronized_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Synchronized_Node (Self.Kind) is
               when Ada_Synchronized_Absent =>
                  --# property-call-start Synchronized.Absent.as_bool
                  Property_Result := Synchronized_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Synchronized_Present =>
                  --# property-call-start Synchronized.Present.as_bool
                  Property_Result := Synchronized_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Synchronized_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Synchronized_Absent
   --
--# property-start Synchronized.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5348
pragma Warnings (Off, "is not referenced");
 function Synchronized_Absent_P_As_Bool
  (Node : Bare_Synchronized_Absent
  )
   return Boolean
is
   Self : Bare_Synchronized_Absent  := Bare_Synchronized_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synchronized_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Synchronized_Present
   --
--# property-start Synchronized.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5348
pragma Warnings (Off, "is not referenced");
 function Synchronized_Present_P_As_Bool
  (Node : Bare_Synchronized_Present
  )
   return Boolean
is
   Self : Bare_Synchronized_Present  := Bare_Synchronized_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synchronized_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Node
   --
--# property-start '[dispatcher]Tagged.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Tagged_Node_P_As_Bool
  (Node : Bare_Tagged_Node
  )
   return Boolean
is
   Self : Bare_Tagged_Node  := Bare_Tagged_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Tagged_Node (Self.Kind) is
               when Ada_Tagged_Absent =>
                  --# property-call-start Tagged.Absent.as_bool
                  Property_Result := Tagged_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Tagged_Present =>
                  --# property-call-start Tagged.Present.as_bool
                  Property_Result := Tagged_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Tagged_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Absent
   --
--# property-start Tagged.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5252
pragma Warnings (Off, "is not referenced");
 function Tagged_Absent_P_As_Bool
  (Node : Bare_Tagged_Absent
  )
   return Boolean
is
   Self : Bare_Tagged_Absent  := Bare_Tagged_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Tagged_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Present
   --
--# property-start Tagged.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5252
pragma Warnings (Off, "is not referenced");
 function Tagged_Present_P_As_Bool
  (Node : Bare_Tagged_Present
  )
   return Boolean
is
   Self : Bare_Tagged_Present  := Bare_Tagged_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Tagged_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Task_Def
   --
      procedure Initialize_Fields_For_Task_Def
        (Self : Bare_Task_Def
         ; Task_Def_F_Interfaces : Bare_Parent_List
         ; Task_Def_F_Public_Part : Bare_Public_Part
         ; Task_Def_F_Private_Part : Bare_Private_Part
         ; Task_Def_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Task_Def_F_Interfaces := Task_Def_F_Interfaces;
            Self.Task_Def_F_Public_Part := Task_Def_F_Public_Part;
            Self.Task_Def_F_Private_Part := Task_Def_F_Private_Part;
            Self.Task_Def_F_End_Name := Task_Def_F_End_Name;
      end Initialize_Fields_For_Task_Def;
   function Task_Def_F_Interfaces
     (Node : Bare_Task_Def) return Bare_Parent_List
   is
   begin
         return Node.Task_Def_F_Interfaces;
   end;
   function Task_Def_F_Public_Part
     (Node : Bare_Task_Def) return Bare_Public_Part
   is
   begin
         return Node.Task_Def_F_Public_Part;
   end;
   function Task_Def_F_Private_Part
     (Node : Bare_Task_Def) return Bare_Private_Part
   is
   begin
         return Node.Task_Def_F_Private_Part;
   end;
   function Task_Def_F_End_Name
     (Node : Bare_Task_Def) return Bare_End_Name
   is
   begin
         return Node.Task_Def_F_End_Name;
   end;
--# property-start TaskDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9030
pragma Warnings (Off, "is not referenced");
 function Task_Def_P_Xref_Equation
  (Node : Bare_Task_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Task_Def  := Bare_Task_Def (Node);
     Ent : Internal_Entity_Task_Def :=
       Internal_Entity_Task_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_713 with Inline_Always;
            procedure Finalizer_Scope_3338 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_357 : Internal_Entity_Name;
Bare_Item_357 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_713 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_713;
            procedure Finalizer_Scope_3338 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3338;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11582 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11581 '<LogicAll at ast.py:9032>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9032
--# expr-start 11580 '<Map at ast.py:9032>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9032
   --# expr-start 11578 '<FieldAccess for interfaces at ast.py:9032>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9032
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Task_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 11578
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_357 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_357 := Untyped_Item_357; 
Is_Null := Bare_Item_357 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_357); 
   If_Result := As_Entity;
end if;
                  Item_357 := If_Result; 
   --# scope-start
               --# bind ifc Item_357
      --# expr-start 11579 '<FieldAccess for xref_equation at ast.py:9032>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9032
      if Item_357.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_357.Node, Env => Env, Origin => Origin, E_Info => Item_357.Info);
--# end
--# expr-done 11579
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3338;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11580
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9032") else null)); 
--# expr-done 11581
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 11582
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_713;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_713;
                     Finalizer_Scope_3338;
            raise;
      end;
   return Property_Result;
end Task_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Type_Attributes_Repository
   --
      procedure Initialize_Fields_For_Type_Attributes_Repository
        (Self : Bare_Type_Attributes_Repository
        ) is
      begin
      Self.Type_Attributes_Repository_Base_Type := No_Bare_Ada_Node;
      Self.Lf_Present_Base_Type_Expr := False;
      Self.Lf_Stg_Base_Type_Expr := No_Bare_Ada_Node;
      Self.Lf_Present_Universal_Int_Type_Expr := False;
      Self.Lf_Stg_Universal_Int_Type_Expr := No_Bare_Ada_Node;
      Self.Lf_Present_Base_Type_Param := False;
      Self.Lf_Stg_Base_Type_Param := No_Bare_Ada_Node;
      Self.Lf_Present_Universal_Int_Param := False;
      Self.Lf_Stg_Universal_Int_Param := No_Bare_Ada_Node;
      Self.Lf_Present_Universal_Real_Param := False;
      Self.Lf_Stg_Universal_Real_Param := No_Bare_Ada_Node;
      Self.Lf_Present_Root_Stream_Param := False;
      Self.Lf_Stg_Root_Stream_Param := No_Bare_Ada_Node;
      Self.Lf_Present_Succ := False;
      Self.Lf_Stg_Succ := No_Bare_Ada_Node;
      Self.Lf_Present_Pred := False;
      Self.Lf_Stg_Pred := No_Bare_Ada_Node;
      Self.Lf_Present_Min := False;
      Self.Lf_Stg_Min := No_Bare_Ada_Node;
      Self.Lf_Present_Max := False;
      Self.Lf_Stg_Max := No_Bare_Ada_Node;
      Self.Lf_Present_Rounding := False;
      Self.Lf_Stg_Rounding := No_Bare_Ada_Node;
      Self.Lf_Present_Unbiased_Rounding := False;
      Self.Lf_Stg_Unbiased_Rounding := No_Bare_Ada_Node;
      Self.Lf_Present_Ceiling := False;
      Self.Lf_Stg_Ceiling := No_Bare_Ada_Node;
      Self.Lf_Present_Floor := False;
      Self.Lf_Stg_Floor := No_Bare_Ada_Node;
      Self.Lf_Present_Truncation := False;
      Self.Lf_Stg_Truncation := No_Bare_Ada_Node;
      Self.Lf_Present_Machine := False;
      Self.Lf_Stg_Machine := No_Bare_Ada_Node;
      Self.Lf_Present_Machine_Rounding := False;
      Self.Lf_Stg_Machine_Rounding := No_Bare_Ada_Node;
      Self.Lf_Present_Fraction := False;
      Self.Lf_Stg_Fraction := No_Bare_Ada_Node;
      Self.Lf_Present_Exponent := False;
      Self.Lf_Stg_Exponent := No_Bare_Ada_Node;
      Self.Lf_Present_Copy_Sign := False;
      Self.Lf_Stg_Copy_Sign := No_Bare_Ada_Node;
      Self.Lf_Present_Remainder := False;
      Self.Lf_Stg_Remainder := No_Bare_Ada_Node;
      Self.Lf_Present_Adjacent := False;
      Self.Lf_Stg_Adjacent := No_Bare_Ada_Node;
      Self.Lf_Present_Scaling := False;
      Self.Lf_Stg_Scaling := No_Bare_Ada_Node;
      Self.Lf_Present_Compose := False;
      Self.Lf_Stg_Compose := No_Bare_Ada_Node;
      Self.Lf_Present_Leading_Part := False;
      Self.Lf_Stg_Leading_Part := No_Bare_Ada_Node;
      Self.Lf_Present_Mod := False;
      Self.Lf_Stg_Mod := No_Bare_Ada_Node;
      Self.Lf_Present_Value := False;
      Self.Lf_Stg_Value := No_Bare_Ada_Node;
      Self.Lf_Present_Wide_Value := False;
      Self.Lf_Stg_Wide_Value := No_Bare_Ada_Node;
      Self.Lf_Present_Wide_Wide_Value := False;
      Self.Lf_Stg_Wide_Wide_Value := No_Bare_Ada_Node;
      Self.Lf_Present_Fixed_Value := False;
      Self.Lf_Stg_Fixed_Value := No_Bare_Ada_Node;
      Self.Lf_Present_Integer_Value := False;
      Self.Lf_Stg_Integer_Value := No_Bare_Ada_Node;
      Self.Lf_Present_Pos := False;
      Self.Lf_Stg_Pos := No_Bare_Ada_Node;
      Self.Lf_Present_Val_Attr := False;
      Self.Lf_Stg_Val_Attr := No_Bare_Ada_Node;
      Self.Lf_Present_Enum_Rep := False;
      Self.Lf_Stg_Enum_Rep := No_Bare_Ada_Node;
      Self.Lf_Present_Enum_Val := False;
      Self.Lf_Stg_Enum_Val := No_Bare_Ada_Node;
      Self.Lf_Present_Read := False;
      Self.Lf_Stg_Read := No_Bare_Ada_Node;
      Self.Lf_Present_Write := False;
      Self.Lf_Stg_Write := No_Bare_Ada_Node;
      Self.Lf_Present_Input := False;
      Self.Lf_Stg_Input := No_Bare_Ada_Node;
      Self.Lf_Present_Output := False;
      Self.Lf_Stg_Output := No_Bare_Ada_Node;
      Self.Lf_Present_Image := False;
      Self.Lf_Stg_Image := No_Bare_Ada_Node;
      Self.Lf_Present_Wide_Image := False;
      Self.Lf_Stg_Wide_Image := No_Bare_Ada_Node;
      Self.Lf_Present_Wide_Wide_Image := False;
      Self.Lf_Stg_Wide_Wide_Image := No_Bare_Ada_Node;
      Self.Lf_Present_Put_Image := False;
      Self.Lf_Stg_Put_Image := No_Bare_Ada_Node;
      Self.Lf_Present_Asm_Input := False;
      Self.Lf_Stg_Asm_Input := No_Bare_Ada_Node;
      Self.Lf_Present_Asm_Output := False;
      Self.Lf_Stg_Asm_Output := No_Bare_Ada_Node;
      Self.Lf_Present_Model := False;
      Self.Lf_Stg_Model := No_Bare_Ada_Node;
      end Initialize_Fields_For_Type_Attributes_Repository;
--# property-start TypeAttributesRepository.base_type_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5479
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Base_Type_Expr
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Type_Expr
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Type_Expr;
      Fld : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Let_Result : Bare_Synthetic_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Base_Type_Expr then
         Property_Result := Self.Lf_Stg_Base_Type_Expr;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11585 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11584 '<New SyntheticTypeExpr at ast.py:5481>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5481
--# expr-start 11583 '<FieldAccess for base_type at ast.py:5481>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5481
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 11583
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld);
--# expr-done 11584
Let_Result := New_Node; 
--# end
--# expr-done 11585
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Base_Type_Expr := True;
      Self.Lf_Stg_Base_Type_Expr := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Base_Type_Expr;
--# end
--# property-start TypeAttributesRepository.universal_int_type_expr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5483
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Universal_Int_Type_Expr
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Type_Expr
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Type_Expr;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Let_Result : Bare_Synthetic_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Universal_Int_Type_Expr then
         Property_Result := Self.Lf_Stg_Universal_Int_Type_Expr;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11590 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11589 '<New SyntheticTypeExpr at ast.py:5485>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5485
--# expr-start 11588 '<FieldAccess for node at ast.py:5486>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5486
--# expr-start 11587 '<Cast to BaseTypeDecl at ast.py:5486>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5486
--# expr-start 11586 '<FieldAccess for universal_int_type at ast.py:5486>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5486
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 11586
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11587
Fld_1 := Cast_Result.Node;
--# expr-done 11588
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_1);
--# expr-done 11589
Let_Result := New_Node; 
--# end
--# expr-done 11590
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Universal_Int_Type_Expr := True;
      Self.Lf_Stg_Universal_Int_Type_Expr := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Universal_Int_Type_Expr;
--# end
--# property-start TypeAttributesRepository.base_type_param /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5489
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Base_Type_Param
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Formal_Param_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Formal_Param_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Formal_Param_Decl;
Let_Result : Bare_Synthetic_Formal_Param_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Base_Type_Param then
         Property_Result := Self.Lf_Stg_Base_Type_Param;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11594 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11593 '<New SyntheticFormalParamDecl at ast.py:5492>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5492
--# expr-start 11591 '<FieldAccess for base_type_expr at ast.py:5494>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5494
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11591
      Cast_Result := Fld;
--# expr-start 11592 '<Symbol Value at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 11592
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result);
      New_Node.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym;
--# expr-done 11593
Let_Result := New_Node; 
--# end
--# expr-done 11594
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Base_Type_Param := True;
      Self.Lf_Stg_Base_Type_Param := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Base_Type_Param;
--# end
--# property-start TypeAttributesRepository.universal_int_param /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5497
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Universal_Int_Param
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Formal_Param_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Formal_Param_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Formal_Param_Decl;
Let_Result : Bare_Synthetic_Formal_Param_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Universal_Int_Param then
         Property_Result := Self.Lf_Stg_Universal_Int_Param;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11598 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11597 '<New SyntheticFormalParamDecl at ast.py:5500>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5500
--# expr-start 11595 '<FieldAccess for universal_int_type_expr at ast.py:5502>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5502
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_type_expr
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Type_Expr (Node => Self);
--# end
--# expr-done 11595
      Cast_Result := Fld;
--# expr-start 11596 '<Symbol Value at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 11596
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result);
      New_Node.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym;
--# expr-done 11597
Let_Result := New_Node; 
--# end
--# expr-done 11598
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Universal_Int_Param := True;
      Self.Lf_Stg_Universal_Int_Param := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Universal_Int_Param;
--# end
--# property-start TypeAttributesRepository.universal_real_param /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5505
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Universal_Real_Param
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Formal_Param_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Formal_Param_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
Let_Result : Bare_Synthetic_Formal_Param_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Universal_Real_Param then
         Property_Result := Self.Lf_Stg_Universal_Real_Param;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11605 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11604 '<New SyntheticFormalParamDecl at ast.py:5508>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5508
--# expr-start 11602 '<New SyntheticTypeExpr at ast.py:5510>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5510
--# expr-start 11601 '<FieldAccess for node at ast.py:5511>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5511
--# expr-start 11600 '<Cast to BaseTypeDecl at ast.py:5511>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5511
--# expr-start 11599 '<FieldAccess for universal_real_type at ast.py:5511>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5511
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 11599
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11600
Fld_1 := Cast_Result.Node;
--# expr-done 11601
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_1);
--# expr-done 11602
      Cast_Result_1 := New_Node;
--# expr-start 11603 '<Symbol Value at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 11603
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result_1);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym;
--# expr-done 11604
Let_Result := New_Node_1; 
--# end
--# expr-done 11605
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Universal_Real_Param := True;
      Self.Lf_Stg_Universal_Real_Param := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Universal_Real_Param;
--# end
--# property-start TypeAttributesRepository.root_stream_param /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5515
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Root_Stream_Param
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Formal_Param_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Formal_Param_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
Let_Result : Bare_Synthetic_Formal_Param_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Root_Stream_Param then
         Property_Result := Self.Lf_Stg_Root_Stream_Param;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11612 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11611 '<New SyntheticFormalParamDecl at ast.py:5518>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5518
--# expr-start 11609 '<New SyntheticTypeExpr at ast.py:5520>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5520
--# expr-start 11608 '<FieldAccess for node at ast.py:5521>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5521
--# expr-start 11607 '<FieldAccess for anonymous_access_type at ast.py:5521>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5521
--# expr-start 11606 '<FieldAccess for root_stream_type at ast.py:5521>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5521
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_stream_type
Fld := Libadalang.Implementation.Ada_Node_P_Root_Stream_Type (Node => Self);
--# end
--# expr-done 11606
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 11607
Fld_2 := Fld_1.Node;
--# expr-done 11608
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_2);
--# expr-done 11609
      Cast_Result := New_Node;
--# expr-start 11610 '<Symbol S at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_S); 
--# expr-done 11610
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym;
--# expr-done 11611
Let_Result := New_Node_1; 
--# end
--# expr-done 11612
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Root_Stream_Param := True;
      Self.Lf_Stg_Root_Stream_Param := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Root_Stream_Param;
--# end
--# property-start TypeAttributesRepository.succ /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5525
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Succ
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Succ then
         Property_Result := Self.Lf_Stg_Succ;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11618 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11617 '<New SyntheticSubpDecl at ast.py:5527>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5527
--# expr-start 11616 '<New SyntheticUnarySpec at ast.py:5527>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5527
--# expr-start 11613 '<FieldAccess for base_type_expr at ast.py:5530>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5530
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11613
--# expr-start 11614 '<FieldAccess for base_type_param at ast.py:5529>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5529
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11614
--# expr-start 11615 '<Symbol Succ at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Succ); 
--# expr-done 11615
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11616
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11617
Let_Result := New_Node_1; 
--# end
--# expr-done 11618
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Succ := True;
      Self.Lf_Stg_Succ := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Succ;
--# end
--# property-start TypeAttributesRepository.pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5533
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Pred
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Pred then
         Property_Result := Self.Lf_Stg_Pred;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11624 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11623 '<New SyntheticSubpDecl at ast.py:5535>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5535
--# expr-start 11622 '<New SyntheticUnarySpec at ast.py:5535>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5535
--# expr-start 11619 '<FieldAccess for base_type_expr at ast.py:5538>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5538
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11619
--# expr-start 11620 '<FieldAccess for base_type_param at ast.py:5537>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5537
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11620
--# expr-start 11621 '<Symbol Pred at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pred); 
--# expr-done 11621
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11622
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11623
Let_Result := New_Node_1; 
--# end
--# expr-done 11624
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Pred := True;
      Self.Lf_Stg_Pred := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Pred;
--# end
--# property-start TypeAttributesRepository.min /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5541
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Min
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Min then
         Property_Result := Self.Lf_Stg_Min;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11631 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11630 '<New SyntheticSubpDecl at ast.py:5543>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5543
--# expr-start 11629 '<New SyntheticBinarySpec at ast.py:5543>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5543
--# expr-start 11625 '<FieldAccess for base_type_param at ast.py:5545>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5545
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11625
--# expr-start 11626 '<FieldAccess for base_type_expr at ast.py:5547>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5547
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11626
      Cast_Result := Fld_2;
--# expr-start 11627 '<FieldAccess for base_type_param at ast.py:5546>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5546
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11627
--# expr-start 11628 '<Symbol Min at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Min); 
--# expr-done 11628
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11629
      Cast_Result_1 := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11630
Let_Result := New_Node_1; 
--# end
--# expr-done 11631
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Min := True;
      Self.Lf_Stg_Min := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Min;
--# end
--# property-start TypeAttributesRepository.max /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5550
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Max
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Max then
         Property_Result := Self.Lf_Stg_Max;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11638 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11637 '<New SyntheticSubpDecl at ast.py:5552>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5552
--# expr-start 11636 '<New SyntheticBinarySpec at ast.py:5552>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5552
--# expr-start 11632 '<FieldAccess for base_type_param at ast.py:5554>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5554
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11632
--# expr-start 11633 '<FieldAccess for base_type_expr at ast.py:5556>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5556
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11633
      Cast_Result := Fld_2;
--# expr-start 11634 '<FieldAccess for base_type_param at ast.py:5555>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5555
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11634
--# expr-start 11635 '<Symbol Max at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max); 
--# expr-done 11635
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11636
      Cast_Result_1 := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11637
Let_Result := New_Node_1; 
--# end
--# expr-done 11638
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Max := True;
      Self.Lf_Stg_Max := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Max;
--# end
--# property-start TypeAttributesRepository.rounding /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5559
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Rounding
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Rounding then
         Property_Result := Self.Lf_Stg_Rounding;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11644 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11643 '<New SyntheticSubpDecl at ast.py:5561>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5561
--# expr-start 11642 '<New SyntheticUnarySpec at ast.py:5561>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5561
--# expr-start 11639 '<FieldAccess for base_type_expr at ast.py:5564>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5564
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11639
--# expr-start 11640 '<FieldAccess for base_type_param at ast.py:5563>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5563
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11640
--# expr-start 11641 '<Symbol Rounding at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rounding); 
--# expr-done 11641
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11642
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11643
Let_Result := New_Node_1; 
--# end
--# expr-done 11644
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Rounding := True;
      Self.Lf_Stg_Rounding := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Rounding;
--# end
--# property-start TypeAttributesRepository.unbiased_rounding /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5567
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Unbiased_Rounding
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Unbiased_Rounding then
         Property_Result := Self.Lf_Stg_Unbiased_Rounding;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11650 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11649 '<New SyntheticSubpDecl at ast.py:5569>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5569
--# expr-start 11648 '<New SyntheticUnarySpec at ast.py:5569>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5569
--# expr-start 11645 '<FieldAccess for base_type_expr at ast.py:5572>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5572
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11645
--# expr-start 11646 '<FieldAccess for base_type_param at ast.py:5571>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5571
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11646
--# expr-start 11647 '<Symbol Unbiased_Rounding at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unbiased_Rounding); 
--# expr-done 11647
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11648
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11649
Let_Result := New_Node_1; 
--# end
--# expr-done 11650
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Unbiased_Rounding := True;
      Self.Lf_Stg_Unbiased_Rounding := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Unbiased_Rounding;
--# end
--# property-start TypeAttributesRepository.ceiling /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5575
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Ceiling
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Ceiling then
         Property_Result := Self.Lf_Stg_Ceiling;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11656 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11655 '<New SyntheticSubpDecl at ast.py:5577>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5577
--# expr-start 11654 '<New SyntheticUnarySpec at ast.py:5577>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5577
--# expr-start 11651 '<FieldAccess for base_type_expr at ast.py:5580>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5580
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11651
--# expr-start 11652 '<FieldAccess for base_type_param at ast.py:5579>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5579
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11652
--# expr-start 11653 '<Symbol Ceiling at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ceiling); 
--# expr-done 11653
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11654
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11655
Let_Result := New_Node_1; 
--# end
--# expr-done 11656
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Ceiling := True;
      Self.Lf_Stg_Ceiling := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Ceiling;
--# end
--# property-start TypeAttributesRepository.floor /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5583
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Floor
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Floor then
         Property_Result := Self.Lf_Stg_Floor;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11662 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11661 '<New SyntheticSubpDecl at ast.py:5585>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5585
--# expr-start 11660 '<New SyntheticUnarySpec at ast.py:5585>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5585
--# expr-start 11657 '<FieldAccess for base_type_expr at ast.py:5588>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5588
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11657
--# expr-start 11658 '<FieldAccess for base_type_param at ast.py:5587>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5587
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11658
--# expr-start 11659 '<Symbol Floor at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Floor); 
--# expr-done 11659
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11660
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11661
Let_Result := New_Node_1; 
--# end
--# expr-done 11662
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Floor := True;
      Self.Lf_Stg_Floor := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Floor;
--# end
--# property-start TypeAttributesRepository.truncation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5591
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Truncation
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Truncation then
         Property_Result := Self.Lf_Stg_Truncation;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11668 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11667 '<New SyntheticSubpDecl at ast.py:5593>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5593
--# expr-start 11666 '<New SyntheticUnarySpec at ast.py:5593>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5593
--# expr-start 11663 '<FieldAccess for base_type_expr at ast.py:5596>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5596
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11663
--# expr-start 11664 '<FieldAccess for base_type_param at ast.py:5595>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5595
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11664
--# expr-start 11665 '<Symbol Truncation at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Truncation); 
--# expr-done 11665
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11666
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11667
Let_Result := New_Node_1; 
--# end
--# expr-done 11668
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Truncation := True;
      Self.Lf_Stg_Truncation := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Truncation;
--# end
--# property-start TypeAttributesRepository.machine /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5599
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Machine
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Machine then
         Property_Result := Self.Lf_Stg_Machine;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11674 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11673 '<New SyntheticSubpDecl at ast.py:5601>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5601
--# expr-start 11672 '<New SyntheticUnarySpec at ast.py:5601>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5601
--# expr-start 11669 '<FieldAccess for base_type_expr at ast.py:5604>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5604
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11669
--# expr-start 11670 '<FieldAccess for base_type_param at ast.py:5603>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5603
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11670
--# expr-start 11671 '<Symbol Machine at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine); 
--# expr-done 11671
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11672
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11673
Let_Result := New_Node_1; 
--# end
--# expr-done 11674
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Machine := True;
      Self.Lf_Stg_Machine := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Machine;
--# end
--# property-start TypeAttributesRepository.machine_rounding /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5607
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Machine_Rounding
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Machine_Rounding then
         Property_Result := Self.Lf_Stg_Machine_Rounding;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11680 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11679 '<New SyntheticSubpDecl at ast.py:5609>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5609
--# expr-start 11678 '<New SyntheticUnarySpec at ast.py:5609>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5609
--# expr-start 11675 '<FieldAccess for base_type_expr at ast.py:5612>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5612
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11675
--# expr-start 11676 '<FieldAccess for base_type_param at ast.py:5611>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5611
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11676
--# expr-start 11677 '<Symbol Machine_Rounding at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Rounding); 
--# expr-done 11677
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11678
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11679
Let_Result := New_Node_1; 
--# end
--# expr-done 11680
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Machine_Rounding := True;
      Self.Lf_Stg_Machine_Rounding := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Machine_Rounding;
--# end
--# property-start TypeAttributesRepository.fraction /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5615
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Fraction
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Fraction then
         Property_Result := Self.Lf_Stg_Fraction;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11686 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11685 '<New SyntheticSubpDecl at ast.py:5617>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5617
--# expr-start 11684 '<New SyntheticUnarySpec at ast.py:5617>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5617
--# expr-start 11681 '<FieldAccess for base_type_expr at ast.py:5620>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5620
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11681
--# expr-start 11682 '<FieldAccess for base_type_param at ast.py:5619>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5619
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11682
--# expr-start 11683 '<Symbol Fraction at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fraction); 
--# expr-done 11683
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11684
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11685
Let_Result := New_Node_1; 
--# end
--# expr-done 11686
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Fraction := True;
      Self.Lf_Stg_Fraction := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Fraction;
--# end
--# property-start TypeAttributesRepository.exponent /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5623
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Exponent
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Exponent then
         Property_Result := Self.Lf_Stg_Exponent;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11692 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11691 '<New SyntheticSubpDecl at ast.py:5625>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5625
--# expr-start 11690 '<New SyntheticUnarySpec at ast.py:5625>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5625
--# expr-start 11687 '<FieldAccess for universal_int_type_expr at ast.py:5628>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5628
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Type_Expr (Node => Self);
--# end
--# expr-done 11687
--# expr-start 11688 '<FieldAccess for base_type_param at ast.py:5627>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5627
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11688
--# expr-start 11689 '<Symbol Exponent at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exponent); 
--# expr-done 11689
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11690
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11691
Let_Result := New_Node_1; 
--# end
--# expr-done 11692
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Exponent := True;
      Self.Lf_Stg_Exponent := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Exponent;
--# end
--# property-start TypeAttributesRepository.copy_sign /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5631
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Copy_Sign
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Copy_Sign then
         Property_Result := Self.Lf_Stg_Copy_Sign;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11699 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11698 '<New SyntheticSubpDecl at ast.py:5633>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5633
--# expr-start 11697 '<New SyntheticBinarySpec at ast.py:5633>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5633
--# expr-start 11693 '<FieldAccess for base_type_param at ast.py:5635>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5635
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11693
--# expr-start 11694 '<FieldAccess for base_type_expr at ast.py:5637>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5637
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11694
      Cast_Result := Fld_2;
--# expr-start 11695 '<FieldAccess for base_type_param at ast.py:5636>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5636
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11695
--# expr-start 11696 '<Symbol Copy_Sign at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Copy_Sign); 
--# expr-done 11696
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11697
      Cast_Result_1 := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11698
Let_Result := New_Node_1; 
--# end
--# expr-done 11699
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Copy_Sign := True;
      Self.Lf_Stg_Copy_Sign := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Copy_Sign;
--# end
--# property-start TypeAttributesRepository.remainder /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5640
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Remainder
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Remainder then
         Property_Result := Self.Lf_Stg_Remainder;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11706 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11705 '<New SyntheticSubpDecl at ast.py:5642>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5642
--# expr-start 11704 '<New SyntheticBinarySpec at ast.py:5642>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5642
--# expr-start 11700 '<FieldAccess for base_type_param at ast.py:5644>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5644
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11700
--# expr-start 11701 '<FieldAccess for base_type_expr at ast.py:5646>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5646
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11701
      Cast_Result := Fld_2;
--# expr-start 11702 '<FieldAccess for base_type_param at ast.py:5645>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5645
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11702
--# expr-start 11703 '<Symbol Remainder at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remainder); 
--# expr-done 11703
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11704
      Cast_Result_1 := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11705
Let_Result := New_Node_1; 
--# end
--# expr-done 11706
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Remainder := True;
      Self.Lf_Stg_Remainder := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Remainder;
--# end
--# property-start TypeAttributesRepository.adjacent /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5649
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Adjacent
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Adjacent then
         Property_Result := Self.Lf_Stg_Adjacent;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11713 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11712 '<New SyntheticSubpDecl at ast.py:5651>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5651
--# expr-start 11711 '<New SyntheticBinarySpec at ast.py:5651>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5651
--# expr-start 11707 '<FieldAccess for base_type_param at ast.py:5653>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5653
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11707
--# expr-start 11708 '<FieldAccess for base_type_expr at ast.py:5655>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5655
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11708
      Cast_Result := Fld_2;
--# expr-start 11709 '<FieldAccess for base_type_param at ast.py:5654>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5654
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11709
--# expr-start 11710 '<Symbol Adjacent at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Adjacent); 
--# expr-done 11710
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11711
      Cast_Result_1 := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11712
Let_Result := New_Node_1; 
--# end
--# expr-done 11713
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Adjacent := True;
      Self.Lf_Stg_Adjacent := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Adjacent;
--# end
--# property-start TypeAttributesRepository.scaling /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5658
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Scaling
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Scaling then
         Property_Result := Self.Lf_Stg_Scaling;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11720 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11719 '<New SyntheticSubpDecl at ast.py:5660>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5660
--# expr-start 11718 '<New SyntheticBinarySpec at ast.py:5660>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5660
--# expr-start 11714 '<FieldAccess for base_type_param at ast.py:5662>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5662
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11714
--# expr-start 11715 '<FieldAccess for base_type_expr at ast.py:5664>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5664
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11715
      Cast_Result := Fld_2;
--# expr-start 11716 '<FieldAccess for universal_int_param at ast.py:5663>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5663
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 11716
--# expr-start 11717 '<Symbol Scaling at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scaling); 
--# expr-done 11717
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11718
      Cast_Result_1 := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11719
Let_Result := New_Node_1; 
--# end
--# expr-done 11720
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Scaling := True;
      Self.Lf_Stg_Scaling := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Scaling;
--# end
--# property-start TypeAttributesRepository.compose /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5667
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Compose
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Compose then
         Property_Result := Self.Lf_Stg_Compose;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11727 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11726 '<New SyntheticSubpDecl at ast.py:5669>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5669
--# expr-start 11725 '<New SyntheticBinarySpec at ast.py:5669>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5669
--# expr-start 11721 '<FieldAccess for base_type_param at ast.py:5671>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5671
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11721
--# expr-start 11722 '<FieldAccess for base_type_expr at ast.py:5673>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5673
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11722
      Cast_Result := Fld_2;
--# expr-start 11723 '<FieldAccess for universal_int_param at ast.py:5672>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5672
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 11723
--# expr-start 11724 '<Symbol Compose at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compose); 
--# expr-done 11724
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11725
      Cast_Result_1 := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11726
Let_Result := New_Node_1; 
--# end
--# expr-done 11727
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Compose := True;
      Self.Lf_Stg_Compose := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Compose;
--# end
--# property-start TypeAttributesRepository.leading_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5676
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Leading_Part
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Leading_Part then
         Property_Result := Self.Lf_Stg_Leading_Part;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11734 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11733 '<New SyntheticSubpDecl at ast.py:5678>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5678
--# expr-start 11732 '<New SyntheticBinarySpec at ast.py:5678>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5678
--# expr-start 11728 '<FieldAccess for base_type_param at ast.py:5680>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5680
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11728
--# expr-start 11729 '<FieldAccess for base_type_expr at ast.py:5682>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5682
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11729
      Cast_Result := Fld_2;
--# expr-start 11730 '<FieldAccess for universal_int_param at ast.py:5681>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5681
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 11730
--# expr-start 11731 '<Symbol Leading_Part at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Leading_Part); 
--# expr-done 11731
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11732
      Cast_Result_1 := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11733
Let_Result := New_Node_1; 
--# end
--# expr-done 11734
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Leading_Part := True;
      Self.Lf_Stg_Leading_Part := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Leading_Part;
--# end
--# property-start TypeAttributesRepository.mod /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5685
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Mod
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Mod then
         Property_Result := Self.Lf_Stg_Mod;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11740 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11739 '<New SyntheticSubpDecl at ast.py:5687>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5687
--# expr-start 11738 '<New SyntheticUnarySpec at ast.py:5687>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5687
--# expr-start 11735 '<FieldAccess for base_type_expr at ast.py:5690>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5690
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11735
--# expr-start 11736 '<FieldAccess for universal_int_param at ast.py:5689>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5689
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 11736
--# expr-start 11737 '<Symbol Mod at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod_153); 
--# expr-done 11737
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11738
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11739
Let_Result := New_Node_1; 
--# end
--# expr-done 11740
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Mod := True;
      Self.Lf_Stg_Mod := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Mod;
--# end
--# property-start TypeAttributesRepository.value /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5693
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
New_Node_2 : Bare_Synthetic_Unary_Spec;
Cast_Result_2 : Bare_Base_Subp_Spec;
New_Node_3 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_3 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Value then
         Property_Result := Self.Lf_Stg_Value;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11752 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11751 '<New SyntheticSubpDecl at ast.py:5695>' New_Node_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5695
--# expr-start 11750 '<New SyntheticUnarySpec at ast.py:5695>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5695
--# expr-start 11741 '<FieldAccess for base_type_expr at ast.py:5704>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5704
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11741
--# expr-start 11748 '<New SyntheticFormalParamDecl at ast.py:5697>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5697
--# expr-start 11746 '<New SyntheticTypeExpr at ast.py:5699>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5699
--# expr-start 11745 '<FieldAccess for node at ast.py:5701>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5701
--# expr-start 11744 '<Cast to BaseTypeDecl at ast.py:5701>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5701
--# expr-start 11743 '<FieldAccess for std_entity at ast.py:5700>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5700
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11742 '<Symbol String at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 11742
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_2);
--# end
--# expr-done 11743
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11744
Fld_1 := Cast_Result.Node;
--# expr-done 11745
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_1);
--# expr-done 11746
      Cast_Result_1 := New_Node;
--# expr-start 11747 '<Symbol Val at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 11747
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result_1);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym_1;
--# expr-done 11748
--# expr-start 11749 '<Symbol Value at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 11749
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node_2,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_2, Synthetic_Unary_Spec_F_Right_Param => New_Node_1);
      New_Node_2.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11750
      Cast_Result_2 := New_Node_2;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_3,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_2);
--# expr-done 11751
Let_Result := New_Node_3; 
--# end
--# expr-done 11752
      Cast_Result_3 := Let_Result;
         Property_Result := Cast_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Value := True;
      Self.Lf_Stg_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Value;
--# end
--# property-start TypeAttributesRepository.wide_value /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5707
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Wide_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
New_Node_2 : Bare_Synthetic_Unary_Spec;
Cast_Result_2 : Bare_Base_Subp_Spec;
New_Node_3 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_3 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Wide_Value then
         Property_Result := Self.Lf_Stg_Wide_Value;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11764 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11763 '<New SyntheticSubpDecl at ast.py:5709>' New_Node_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5709
--# expr-start 11762 '<New SyntheticUnarySpec at ast.py:5709>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5709
--# expr-start 11753 '<FieldAccess for base_type_expr at ast.py:5718>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5718
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11753
--# expr-start 11760 '<New SyntheticFormalParamDecl at ast.py:5711>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5711
--# expr-start 11758 '<New SyntheticTypeExpr at ast.py:5713>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5713
--# expr-start 11757 '<FieldAccess for node at ast.py:5715>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5715
--# expr-start 11756 '<Cast to BaseTypeDecl at ast.py:5715>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5715
--# expr-start 11755 '<FieldAccess for std_entity at ast.py:5714>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5714
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11754 '<Symbol Wide_String at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_String); 
--# expr-done 11754
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_2);
--# end
--# expr-done 11755
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11756
Fld_1 := Cast_Result.Node;
--# expr-done 11757
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_1);
--# expr-done 11758
      Cast_Result_1 := New_Node;
--# expr-start 11759 '<Symbol Val at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 11759
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result_1);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym_1;
--# expr-done 11760
--# expr-start 11761 '<Symbol Wide_Value at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Value); 
--# expr-done 11761
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node_2,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_2, Synthetic_Unary_Spec_F_Right_Param => New_Node_1);
      New_Node_2.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11762
      Cast_Result_2 := New_Node_2;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_3,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_2);
--# expr-done 11763
Let_Result := New_Node_3; 
--# end
--# expr-done 11764
      Cast_Result_3 := Let_Result;
         Property_Result := Cast_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Wide_Value := True;
      Self.Lf_Stg_Wide_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Wide_Value;
--# end
--# property-start TypeAttributesRepository.wide_wide_value /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5721
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Wide_Wide_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_2 : Bare_Synthetic_Type_Expr;
New_Node_2 : Bare_Synthetic_Unary_Spec;
Cast_Result_2 : Bare_Base_Subp_Spec;
New_Node_3 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_3 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Wide_Wide_Value then
         Property_Result := Self.Lf_Stg_Wide_Wide_Value;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11776 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11775 '<New SyntheticSubpDecl at ast.py:5723>' New_Node_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5723
--# expr-start 11774 '<New SyntheticUnarySpec at ast.py:5723>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5723
--# expr-start 11765 '<FieldAccess for base_type_expr at ast.py:5732>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5732
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11765
--# expr-start 11772 '<New SyntheticFormalParamDecl at ast.py:5725>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5725
--# expr-start 11770 '<New SyntheticTypeExpr at ast.py:5727>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5727
--# expr-start 11769 '<FieldAccess for node at ast.py:5729>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5729
--# expr-start 11768 '<Cast to BaseTypeDecl at ast.py:5729>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5729
--# expr-start 11767 '<FieldAccess for std_entity at ast.py:5728>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5728
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11766 '<Symbol Wide_Wide_String at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_String); 
--# expr-done 11766
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_2);
--# end
--# expr-done 11767
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11768
Fld_1 := Cast_Result.Node;
--# expr-done 11769
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_1);
--# expr-done 11770
      Cast_Result_1 := New_Node;
--# expr-start 11771 '<Symbol Val at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 11771
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result_1);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym_1;
--# expr-done 11772
--# expr-start 11773 '<Symbol Wide_Wide_Value at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Value); 
--# expr-done 11773
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node_2,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_2, Synthetic_Unary_Spec_F_Right_Param => New_Node_1);
      New_Node_2.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11774
      Cast_Result_2 := New_Node_2;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_3,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_2);
--# expr-done 11775
Let_Result := New_Node_3; 
--# end
--# expr-done 11776
      Cast_Result_3 := Let_Result;
         Property_Result := Cast_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Wide_Wide_Value := True;
      Self.Lf_Stg_Wide_Wide_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Wide_Wide_Value;
--# end
--# property-start TypeAttributesRepository.fixed_value /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5735
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Fixed_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Fixed_Value then
         Property_Result := Self.Lf_Stg_Fixed_Value;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11782 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11781 '<New SyntheticSubpDecl at ast.py:5737>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5737
--# expr-start 11780 '<New SyntheticUnarySpec at ast.py:5737>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5737
--# expr-start 11777 '<FieldAccess for base_type_expr at ast.py:5740>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5740
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11777
--# expr-start 11778 '<FieldAccess for universal_int_param at ast.py:5739>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5739
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 11778
--# expr-start 11779 '<Symbol Fixed_Value at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fixed_Value); 
--# expr-done 11779
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11780
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11781
Let_Result := New_Node_1; 
--# end
--# expr-done 11782
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Fixed_Value := True;
      Self.Lf_Stg_Fixed_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Fixed_Value;
--# end
--# property-start TypeAttributesRepository.integer_value /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5743
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Integer_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Integer_Value then
         Property_Result := Self.Lf_Stg_Integer_Value;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11788 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11787 '<New SyntheticSubpDecl at ast.py:5745>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5745
--# expr-start 11786 '<New SyntheticUnarySpec at ast.py:5745>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5745
--# expr-start 11783 '<FieldAccess for base_type_expr at ast.py:5748>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5748
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11783
--# expr-start 11784 '<FieldAccess for universal_real_param at ast.py:5747>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5747
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_real_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Real_Param (Node => Self);
--# end
--# expr-done 11784
--# expr-start 11785 '<Symbol Integer_Value at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Value); 
--# expr-done 11785
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11786
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11787
Let_Result := New_Node_1; 
--# end
--# expr-done 11788
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Integer_Value := True;
      Self.Lf_Stg_Integer_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Integer_Value;
--# end
--# property-start TypeAttributesRepository.pos /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5751
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Pos
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Pos then
         Property_Result := Self.Lf_Stg_Pos;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11794 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11793 '<New SyntheticSubpDecl at ast.py:5753>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5753
--# expr-start 11792 '<New SyntheticUnarySpec at ast.py:5753>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5753
--# expr-start 11789 '<FieldAccess for universal_int_type_expr at ast.py:5756>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5756
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Type_Expr (Node => Self);
--# end
--# expr-done 11789
--# expr-start 11790 '<FieldAccess for base_type_param at ast.py:5755>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5755
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11790
--# expr-start 11791 '<Symbol Pos at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pos); 
--# expr-done 11791
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11792
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11793
Let_Result := New_Node_1; 
--# end
--# expr-done 11794
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Pos := True;
      Self.Lf_Stg_Pos := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Pos;
--# end
--# property-start TypeAttributesRepository.val_attr /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5760
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Val_Attr
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Val_Attr then
         Property_Result := Self.Lf_Stg_Val_Attr;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11800 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11799 '<New SyntheticSubpDecl at ast.py:5762>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5762
--# expr-start 11798 '<New SyntheticUnarySpec at ast.py:5762>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5762
--# expr-start 11795 '<FieldAccess for base_type_expr at ast.py:5765>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5765
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11795
--# expr-start 11796 '<FieldAccess for universal_int_param at ast.py:5764>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5764
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 11796
--# expr-start 11797 '<Symbol Val at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 11797
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11798
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11799
Let_Result := New_Node_1; 
--# end
--# expr-done 11800
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Val_Attr := True;
      Self.Lf_Stg_Val_Attr := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Val_Attr;
--# end
--# property-start TypeAttributesRepository.enum_rep /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5768
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Enum_Rep
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Enum_Rep then
         Property_Result := Self.Lf_Stg_Enum_Rep;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11806 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11805 '<New SyntheticSubpDecl at ast.py:5770>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5770
--# expr-start 11804 '<New SyntheticUnarySpec at ast.py:5770>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5770
--# expr-start 11801 '<FieldAccess for universal_int_type_expr at ast.py:5773>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5773
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Type_Expr (Node => Self);
--# end
--# expr-done 11801
--# expr-start 11802 '<FieldAccess for base_type_param at ast.py:5772>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5772
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11802
--# expr-start 11803 '<Symbol Enum_Rep at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Rep); 
--# expr-done 11803
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11804
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11805
Let_Result := New_Node_1; 
--# end
--# expr-done 11806
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Enum_Rep := True;
      Self.Lf_Stg_Enum_Rep := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Enum_Rep;
--# end
--# property-start TypeAttributesRepository.enum_val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5776
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Enum_Val
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Enum_Val then
         Property_Result := Self.Lf_Stg_Enum_Val;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11812 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11811 '<New SyntheticSubpDecl at ast.py:5778>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5778
--# expr-start 11810 '<New SyntheticUnarySpec at ast.py:5778>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5778
--# expr-start 11807 '<FieldAccess for base_type_expr at ast.py:5781>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5781
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11807
--# expr-start 11808 '<FieldAccess for universal_int_param at ast.py:5780>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5780
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 11808
--# expr-start 11809 '<Symbol Enum_Val at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Val); 
--# expr-done 11809
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11810
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11811
Let_Result := New_Node_1; 
--# end
--# expr-done 11812
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Enum_Val := True;
      Self.Lf_Stg_Enum_Val := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Enum_Val;
--# end
--# property-start TypeAttributesRepository.read /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5784
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Read
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Read then
         Property_Result := Self.Lf_Stg_Read;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11818 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11817 '<New SyntheticSubpDecl at ast.py:5786>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5786
--# expr-start 11816 '<New SyntheticBinarySpec at ast.py:5786>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5786
--# expr-start 11813 '<FieldAccess for root_stream_param at ast.py:5788>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5788
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.root_stream_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Root_Stream_Param (Node => Self);
--# end
--# expr-done 11813
--# expr-start 11814 '<FieldAccess for base_type_param at ast.py:5789>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5789
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11814
--# expr-start 11815 '<Symbol Read at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read); 
--# expr-done 11815
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => No_Bare_Ada_Node, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11816
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11817
Let_Result := New_Node_1; 
--# end
--# expr-done 11818
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Read := True;
      Self.Lf_Stg_Read := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Read;
--# end
--# property-start TypeAttributesRepository.write /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5793
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Write
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Write then
         Property_Result := Self.Lf_Stg_Write;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11824 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11823 '<New SyntheticSubpDecl at ast.py:5795>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5795
--# expr-start 11822 '<New SyntheticBinarySpec at ast.py:5795>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5795
--# expr-start 11819 '<FieldAccess for root_stream_param at ast.py:5797>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5797
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.root_stream_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Root_Stream_Param (Node => Self);
--# end
--# expr-done 11819
--# expr-start 11820 '<FieldAccess for base_type_param at ast.py:5798>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5798
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11820
--# expr-start 11821 '<Symbol Write at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write); 
--# expr-done 11821
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => No_Bare_Ada_Node, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11822
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11823
Let_Result := New_Node_1; 
--# end
--# expr-done 11824
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Write := True;
      Self.Lf_Stg_Write := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Write;
--# end
--# property-start TypeAttributesRepository.input /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5802
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Input
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Input then
         Property_Result := Self.Lf_Stg_Input;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11830 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11829 '<New SyntheticSubpDecl at ast.py:5804>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5804
--# expr-start 11828 '<New SyntheticUnarySpec at ast.py:5804>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5804
--# expr-start 11825 '<FieldAccess for base_type_expr at ast.py:5807>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5807
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11825
--# expr-start 11826 '<FieldAccess for root_stream_param at ast.py:5806>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5806
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.root_stream_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Root_Stream_Param (Node => Self);
--# end
--# expr-done 11826
--# expr-start 11827 '<Symbol Input at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input); 
--# expr-done 11827
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11828
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11829
Let_Result := New_Node_1; 
--# end
--# expr-done 11830
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Input := True;
      Self.Lf_Stg_Input := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Input;
--# end
--# property-start TypeAttributesRepository.output /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5810
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Output
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Formal_Param_Decl;
New_Node : Bare_Synthetic_Binary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Output then
         Property_Result := Self.Lf_Stg_Output;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11836 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11835 '<New SyntheticSubpDecl at ast.py:5812>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5812
--# expr-start 11834 '<New SyntheticBinarySpec at ast.py:5812>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5812
--# expr-start 11831 '<FieldAccess for root_stream_param at ast.py:5814>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5814
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.root_stream_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Root_Stream_Param (Node => Self);
--# end
--# expr-done 11831
--# expr-start 11832 '<FieldAccess for base_type_param at ast.py:5815>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5815
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11832
--# expr-start 11833 '<Symbol Output at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output); 
--# expr-done 11833
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node,
      Synthetic_Binary_Spec_F_Left_Param => Fld, Synthetic_Binary_Spec_F_Return_Type_Expr => No_Bare_Ada_Node, Synthetic_Binary_Spec_F_Right_Param => Fld_1);
      New_Node.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11834
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11835
Let_Result := New_Node_1; 
--# end
--# expr-done 11836
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Output := True;
      Self.Lf_Stg_Output := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Output;
--# end
--# property-start TypeAttributesRepository.image /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5819
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Image
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
New_Node_1 : Bare_Synthetic_Unary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_2 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Image then
         Property_Result := Self.Lf_Stg_Image;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11846 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11845 '<New SyntheticSubpDecl at ast.py:5821>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5821
--# expr-start 11844 '<New SyntheticUnarySpec at ast.py:5821>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5821
--# expr-start 11841 '<New SyntheticTypeExpr at ast.py:5824>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5824
--# expr-start 11840 '<FieldAccess for node at ast.py:5825>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5825
--# expr-start 11839 '<Cast to BaseTypeDecl at ast.py:5825>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5825
--# expr-start 11838 '<FieldAccess for std_entity at ast.py:5825>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5825
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11837 '<Symbol String at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 11837
--# property-call-start AdaNode.std_entity
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_1);
--# end
--# expr-done 11838
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11839
Fld_2 := Cast_Result.Node;
--# expr-done 11840
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_2);
--# expr-done 11841
--# expr-start 11842 '<FieldAccess for base_type_param at ast.py:5823>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5823
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11842
--# expr-start 11843 '<Symbol Image at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Image); 
--# expr-done 11843
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node_1,
      Synthetic_Unary_Spec_F_Return_Type_Expr => New_Node, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node_1.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11844
      Cast_Result_1 := New_Node_1;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_2,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11845
Let_Result := New_Node_2; 
--# end
--# expr-done 11846
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Image := True;
      Self.Lf_Stg_Image := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Image;
--# end
--# property-start TypeAttributesRepository.wide_image /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5829
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Wide_Image
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
New_Node_1 : Bare_Synthetic_Unary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_2 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Wide_Image then
         Property_Result := Self.Lf_Stg_Wide_Image;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11856 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11855 '<New SyntheticSubpDecl at ast.py:5831>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5831
--# expr-start 11854 '<New SyntheticUnarySpec at ast.py:5831>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5831
--# expr-start 11851 '<New SyntheticTypeExpr at ast.py:5834>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5834
--# expr-start 11850 '<FieldAccess for node at ast.py:5836>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5836
--# expr-start 11849 '<Cast to BaseTypeDecl at ast.py:5836>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5836
--# expr-start 11848 '<FieldAccess for std_entity at ast.py:5835>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5835
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11847 '<Symbol Wide_String at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_String); 
--# expr-done 11847
--# property-call-start AdaNode.std_entity
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_1);
--# end
--# expr-done 11848
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11849
Fld_2 := Cast_Result.Node;
--# expr-done 11850
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_2);
--# expr-done 11851
--# expr-start 11852 '<FieldAccess for base_type_param at ast.py:5833>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5833
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11852
--# expr-start 11853 '<Symbol Wide_Image at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Image); 
--# expr-done 11853
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node_1,
      Synthetic_Unary_Spec_F_Return_Type_Expr => New_Node, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node_1.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11854
      Cast_Result_1 := New_Node_1;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_2,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11855
Let_Result := New_Node_2; 
--# end
--# expr-done 11856
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Wide_Image := True;
      Self.Lf_Stg_Wide_Image := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Wide_Image;
--# end
--# property-start TypeAttributesRepository.wide_wide_image /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5840
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Wide_Wide_Image
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
New_Node_1 : Bare_Synthetic_Unary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_2 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Wide_Wide_Image then
         Property_Result := Self.Lf_Stg_Wide_Wide_Image;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11866 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11865 '<New SyntheticSubpDecl at ast.py:5842>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5842
--# expr-start 11864 '<New SyntheticUnarySpec at ast.py:5842>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5842
--# expr-start 11861 '<New SyntheticTypeExpr at ast.py:5845>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5845
--# expr-start 11860 '<FieldAccess for node at ast.py:5847>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5847
--# expr-start 11859 '<Cast to BaseTypeDecl at ast.py:5847>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5847
--# expr-start 11858 '<FieldAccess for std_entity at ast.py:5846>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5846
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11857 '<Symbol Wide_Wide_String at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_String); 
--# expr-done 11857
--# property-call-start AdaNode.std_entity
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_1);
--# end
--# expr-done 11858
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11859
Fld_2 := Cast_Result.Node;
--# expr-done 11860
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_2);
--# expr-done 11861
--# expr-start 11862 '<FieldAccess for base_type_param at ast.py:5844>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5844
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11862
--# expr-start 11863 '<Symbol Wide_Wide_Image at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Image); 
--# expr-done 11863
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node_1,
      Synthetic_Unary_Spec_F_Return_Type_Expr => New_Node, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node_1.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11864
      Cast_Result_1 := New_Node_1;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_2,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 11865
Let_Result := New_Node_2; 
--# end
--# expr-done 11866
      Cast_Result_2 := Let_Result;
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Wide_Wide_Image := True;
      Self.Lf_Stg_Wide_Wide_Image := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Wide_Wide_Image;
--# end
--# property-start TypeAttributesRepository.put_image /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5851
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Put_Image
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Classwide_Type_Decl;
Fld_2 : Bare_Classwide_Type_Decl;
Cast_Result : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_3 : Bare_Synthetic_Formal_Param_Decl;
New_Node_2 : Bare_Synthetic_Binary_Spec;
Cast_Result_2 : Bare_Base_Subp_Spec;
New_Node_3 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_3 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Put_Image then
         Property_Result := Self.Lf_Stg_Put_Image;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11877 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11876 '<New SyntheticSubpDecl at ast.py:5853>' New_Node_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5853
--# expr-start 11875 '<New SyntheticBinarySpec at ast.py:5853>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5853
--# expr-start 11872 '<New SyntheticFormalParamDecl at ast.py:5855>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5855
--# expr-start 11870 '<New SyntheticTypeExpr at ast.py:5857>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5857
--# expr-start 11869 '<FieldAccess for node at ast.py:5858>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5858
--# expr-start 11868 '<FieldAccess for classwide_type at ast.py:5858>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5858
--# expr-start 11867 '<FieldAccess for root_buffer_type at ast.py:5858>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5858
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_buffer_type
Fld := Libadalang.Implementation.Ada_Node_P_Root_Buffer_Type (Node => Self);
--# end
--# expr-done 11867
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 11868
Fld_2 := Fld_1.Node;
--# expr-done 11869
      Cast_Result := Fld_2;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Cast_Result);
--# expr-done 11870
      Cast_Result_1 := New_Node;
--# expr-start 11871 '<Symbol Buffer at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Buffer); 
--# expr-done 11871
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result_1);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym_1;
--# expr-done 11872
--# expr-start 11873 '<FieldAccess for base_type_param at ast.py:5861>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5861
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_3 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11873
--# expr-start 11874 '<Symbol Put_Image at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image); 
--# expr-done 11874
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_3 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node_2,
      Synthetic_Binary_Spec_F_Left_Param => New_Node_1, Synthetic_Binary_Spec_F_Return_Type_Expr => No_Bare_Ada_Node, Synthetic_Binary_Spec_F_Right_Param => Fld_3);
      New_Node_2.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11875
      Cast_Result_2 := New_Node_2;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_3,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_2);
--# expr-done 11876
Let_Result := New_Node_3; 
--# end
--# expr-done 11877
      Cast_Result_3 := Let_Result;
         Property_Result := Cast_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Put_Image := True;
      Self.Lf_Stg_Put_Image := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Put_Image;
--# end
--# property-start TypeAttributesRepository.asm_input /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5865
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Asm_Input
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_419 with Inline_Always;
            procedure Finalizer_Scope_3339 with Inline_Always;
      Block_Var : Bare_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Fld_2 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result_2 : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_5 : Bare_Synthetic_Formal_Param_Decl;
New_Node_2 : Bare_Synthetic_Type_Expr;
Cast_Result_3 : Bare_Type_Expr;
New_Node_3 : Bare_Synthetic_Binary_Spec;
Cast_Result_4 : Bare_Base_Subp_Spec;
New_Node_4 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_5 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_419 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_419;
            procedure Finalizer_Scope_3339 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3339;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Asm_Input then
         Property_Result := Self.Lf_Stg_Asm_Input;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11899 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11886 '<Cast to BaseTypeDecl at ast.py:5871>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5871
--# expr-start 11885 '<FieldAccess for node at ast.py:5871>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5871
--# expr-start 11884 '<EnvGet at ast.py:5870>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5870
--# expr-start 11881 '<FieldAccess for get_unit_root_decl at ast.py:5868>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5868
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11880 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 11878 '<Symbol System at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 11878
--# expr-start 11879 '<Symbol Machine_Code at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Code); 
--# expr-done 11879
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 11880
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 11881
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 11882 '<FieldAccess for children_env at ast.py:5870>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5870
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 11882
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3339;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 11883 '<Symbol Asm_Input_Operand at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Input_Operand); 
--# expr-done 11883
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 11884
Fld_2 := Env_Get_Result.Node;
--# expr-done 11885
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_2;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 11886
Block_Var := Cast_Result; 
--# bind input_type Block_Var
--# expr-start 11898 '<New SyntheticSubpDecl at ast.py:5873>' New_Node_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5873
--# expr-start 11897 '<New SyntheticBinarySpec at ast.py:5873>' New_Node_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5873
--# expr-start 11893 '<New SyntheticFormalParamDecl at ast.py:5875>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5875
--# expr-start 11891 '<New SyntheticTypeExpr at ast.py:5877>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5877
--# expr-start 11890 '<FieldAccess for node at ast.py:5879>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5879
--# expr-start 11889 '<Cast to BaseTypeDecl at ast.py:5879>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5879
--# expr-start 11888 '<FieldAccess for std_entity at ast.py:5878>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5878
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11887 '<Symbol String at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 11887
--# property-call-start AdaNode.std_entity
Fld_3 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_5);
--# end
--# expr-done 11888
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11889
Fld_4 := Cast_Result_1.Node;
--# expr-done 11890
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_4 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_4);
--# expr-done 11891
      Cast_Result_2 := New_Node;
--# expr-start 11892 '<Symbol S at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_S); 
--# expr-done 11892
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result_2);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym_4;
--# expr-done 11893
--# expr-start 11894 '<New SyntheticTypeExpr at ast.py:5883>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5883
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Block_Var = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_2,
      Synthetic_Type_Expr_F_Target_Type => Block_Var);
--# expr-done 11894
      Cast_Result_3 := New_Node_2;
--# expr-start 11895 '<FieldAccess for base_type_param at ast.py:5882>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5882
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_5 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11895
--# expr-start 11896 '<Symbol Asm_Input at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Input); 
--# expr-done 11896
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_5 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node_3,
      Synthetic_Binary_Spec_F_Left_Param => New_Node_1, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result_3, Synthetic_Binary_Spec_F_Right_Param => Fld_5);
      New_Node_3.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym_3;
--# expr-done 11897
      Cast_Result_4 := New_Node_3;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_4 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_4 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_4,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_4);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_4,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_4);
--# expr-done 11898
Let_Result := New_Node_4; 
--# end
--# expr-done 11899
      Cast_Result_5 := Let_Result;
         Property_Result := Cast_Result_5;
   --# end
      Finalizer_Scope_419;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_419;
                     Finalizer_Scope_3339;
            raise;
      end;
      Self.Lf_Present_Asm_Input := True;
      Self.Lf_Stg_Asm_Input := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Asm_Input;
--# end
--# property-start TypeAttributesRepository.asm_output /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5886
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Asm_Output
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_420 with Inline_Always;
            procedure Finalizer_Scope_3340 with Inline_Always;
      Block_Var : Bare_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Fld_2 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result_2 : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
Fld_5 : Bare_Synthetic_Formal_Param_Decl;
New_Node_2 : Bare_Synthetic_Type_Expr;
Cast_Result_3 : Bare_Type_Expr;
New_Node_3 : Bare_Synthetic_Binary_Spec;
Cast_Result_4 : Bare_Base_Subp_Spec;
New_Node_4 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_5 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_420 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_420;
            procedure Finalizer_Scope_3340 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3340;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Asm_Output then
         Property_Result := Self.Lf_Stg_Asm_Output;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11921 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11908 '<Cast to BaseTypeDecl at ast.py:5892>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5892
--# expr-start 11907 '<FieldAccess for node at ast.py:5892>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5892
--# expr-start 11906 '<EnvGet at ast.py:5891>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5891
--# expr-start 11903 '<FieldAccess for get_unit_root_decl at ast.py:5889>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5889
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11902 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 11900 '<Symbol System at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 11900
--# expr-start 11901 '<Symbol Machine_Code at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Code); 
--# expr-done 11901
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 11902
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 11903
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 11904 '<FieldAccess for children_env at ast.py:5891>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5891
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 11904
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3340;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 11905 '<Symbol Asm_Output_Operand at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Output_Operand); 
--# expr-done 11905
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 11906
Fld_2 := Env_Get_Result.Node;
--# expr-done 11907
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_2;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 11908
Block_Var := Cast_Result; 
--# bind output_type Block_Var
--# expr-start 11920 '<New SyntheticSubpDecl at ast.py:5894>' New_Node_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5894
--# expr-start 11919 '<New SyntheticBinarySpec at ast.py:5894>' New_Node_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5894
--# expr-start 11915 '<New SyntheticFormalParamDecl at ast.py:5896>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5896
--# expr-start 11913 '<New SyntheticTypeExpr at ast.py:5898>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5898
--# expr-start 11912 '<FieldAccess for node at ast.py:5900>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5900
--# expr-start 11911 '<Cast to BaseTypeDecl at ast.py:5900>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5900
--# expr-start 11910 '<FieldAccess for std_entity at ast.py:5899>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5899
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11909 '<Symbol String at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 11909
--# property-call-start AdaNode.std_entity
Fld_3 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_5);
--# end
--# expr-done 11910
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11911
Fld_4 := Cast_Result_1.Node;
--# expr-done 11912
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_4 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Fld_4);
--# expr-done 11913
      Cast_Result_2 := New_Node;
--# expr-start 11914 '<Symbol S at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_S); 
--# expr-done 11914
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result_2);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym_4;
--# expr-done 11915
--# expr-start 11916 '<New SyntheticTypeExpr at ast.py:5904>' New_Node_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5904
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Block_Var = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_2,
      Synthetic_Type_Expr_F_Target_Type => Block_Var);
--# expr-done 11916
      Cast_Result_3 := New_Node_2;
--# expr-start 11917 '<FieldAccess for base_type_param at ast.py:5903>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5903
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_5 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 11917
--# expr-start 11918 '<Symbol Asm_Output at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Output); 
--# expr-done 11918
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld_5 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node_3,
      Synthetic_Binary_Spec_F_Left_Param => New_Node_1, Synthetic_Binary_Spec_F_Return_Type_Expr => Cast_Result_3, Synthetic_Binary_Spec_F_Right_Param => Fld_5);
      New_Node_3.Synthetic_Binary_Spec_Subp_Symbol :=
         Sym_3;
--# expr-done 11919
      Cast_Result_4 := New_Node_3;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_4 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_4 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_4,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_4);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_4,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_4);
--# expr-done 11920
Let_Result := New_Node_4; 
--# end
--# expr-done 11921
      Cast_Result_5 := Let_Result;
         Property_Result := Cast_Result_5;
   --# end
      Finalizer_Scope_420;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_420;
                     Finalizer_Scope_3340;
            raise;
      end;
      Self.Lf_Present_Asm_Output := True;
      Self.Lf_Stg_Asm_Output := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Asm_Output;
--# end
--# property-start TypeAttributesRepository.model /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5907
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Model
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl;
Fld_1 : Bare_Synthetic_Type_Expr;
New_Node : Bare_Synthetic_Unary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_1 : Bare_Synthetic_Subp_Decl;
Let_Result : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self.Lf_Present_Model then
         Property_Result := Self.Lf_Stg_Model;
         return Property_Result;
      end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11927 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11926 '<New SyntheticSubpDecl at ast.py:5909>' New_Node_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5909
--# expr-start 11925 '<New SyntheticUnarySpec at ast.py:5909>' New_Node /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5909
--# expr-start 11922 '<FieldAccess for base_type_expr at ast.py:5912>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5912
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 11922
--# expr-start 11923 '<FieldAccess for universal_real_param at ast.py:5911>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5911
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_real_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Real_Param (Node => Self);
--# end
--# expr-done 11923
--# expr-start 11924 '<Symbol Model at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model); 
--# expr-done 11924
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if Fld = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node,
      Synthetic_Unary_Spec_F_Return_Type_Expr => Fld_1, Synthetic_Unary_Spec_F_Right_Param => Fld);
      New_Node.Synthetic_Unary_Spec_Subp_Symbol :=
         Sym;
--# expr-done 11925
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_1,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 11926
Let_Result := New_Node_1; 
--# end
--# expr-done 11927
      Cast_Result_1 := Let_Result;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
      Self.Lf_Present_Model := True;
      Self.Lf_Stg_Model := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Model;
--# end
   --
   --  Primitives for Bare_Type_Def
   --
--# property-start '[dispatcher]TypeDef.discrete_range' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Discrete_Range
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def .. Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.discrete_range
                  Property_Result := Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.discrete_range
                  Property_Result := Derived_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.discrete_range
                  Property_Result := Mod_Int_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.discrete_range
                  Property_Result := Signed_Int_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Discrete_Range;
--# end
--# property-start '[dispatcher]TypeDef.array_ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Array_Ndims
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.array_ndims
                  Property_Result := Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.array_ndims
                  Property_Result := Array_Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.array_ndims
                  Property_Result := Derived_Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Array_Ndims;
--# end
--# property-start TypeDef.is_real_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4702
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Real_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11928 '<FieldAccess for is_float_type at ast.py:4703>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4703
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_float_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Float_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 11928
if Fld then
   If_Result := True;
else
   --# expr-start 11929 '<FieldAccess for is_fixed_point at ast.py:4703>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4703
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Fixed_Point (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 11929
   If_Result := Fld_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Real_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_float_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Float_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Decimal_Fixed_Point_Def | Ada_Ordinary_Fixed_Point_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_float_type
                  Property_Result := Type_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_float_type
                  Property_Result := Derived_Type_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Floating_Point_Def =>
                  --# property-call-start FloatingPointDef.is_float_type
                  Property_Result := Floating_Point_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Float_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_fixed_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Fixed_Point
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Private_Type_Def | Ada_Floating_Point_Def | Ada_Record_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_fixed_point
                  Property_Result := Type_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_fixed_point
                  Property_Result := Derived_Type_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def =>
                  --# property-call-start DecimalFixedPointDef.is_fixed_point
                  Property_Result := Decimal_Fixed_Point_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start OrdinaryFixedPointDef.is_fixed_point
                  Property_Result := Ordinary_Fixed_Point_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Fixed_Point;
--# end
--# property-start '[dispatcher]TypeDef.predefined_operators' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Predefined_Operators
  (Node : Bare_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Derived_Type_Def | Ada_Interface_Type_Def =>
                  --# property-call-start TypeDef.predefined_operators
                  Property_Result := Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
                  --# property-call-start AccessDef.predefined_operators
                  Property_Result := Access_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.predefined_operators
                  Property_Result := Array_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.predefined_operators
                  Property_Result := Enum_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.predefined_operators
                  Property_Result := Formal_Discrete_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.predefined_operators
                  Property_Result := Mod_Int_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.predefined_operators
                  Property_Result := Private_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Floating_Point_Def =>
                  --# property-call-start RealTypeDef.predefined_operators
                  Property_Result := Real_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def =>
                  --# property-call-start DecimalFixedPointDef.predefined_operators
                  Property_Result := Decimal_Fixed_Point_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start OrdinaryFixedPointDef.predefined_operators
                  Property_Result := Ordinary_Fixed_Point_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.predefined_operators
                  Property_Result := Record_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.predefined_operators
                  Property_Result := Signed_Int_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Predefined_Operators;
--# end
--# property-start TypeDef.predefined_equality_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4728
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Predefined_Equality_Operators
  (Node : Bare_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_289 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_289 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_289;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11939 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11931 '<Cast to TypeDecl at ast.py:4730>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4730
--# expr-start 11930 '<parent at ast.py:4730>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4730
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 11930
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 11931
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 11933 '<FieldAccess for node at ast.py:4731>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4731
--# expr-start 11932 '<FieldAccess for bool_type at ast.py:4731>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4731
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11932
Fld_2 := Fld_1.Node;
--# expr-done 11933
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 11938 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 11935 '<FieldAccess for create_binop_assoc at ast.py:4733>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4733
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11934 '<Symbol "=" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 11934
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
--# property-call-start AdaNode.create_binop_assoc
Fld_3 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Block_Var_1);
--# end
--# expr-done 11935
--# expr-start 11937 '<FieldAccess for create_binop_assoc at ast.py:4734>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4734
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11936 '<Symbol "/=" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 11936
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
--# property-call-start AdaNode.create_binop_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_3, Rhs => Cast_Result_4, Ret => Block_Var_1);
--# end
--# expr-done 11937
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_3, 2 => Fld_4)); 
--# expr-done 11938
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 11939
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_289;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_289;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Predefined_Equality_Operators;
--# end
--# property-start '[dispatcher]TypeDef.is_discrete_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Discrete_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Enum_Type_Def | Ada_Interface_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_discrete_type
                  Property_Result := Type_Def_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.is_discrete_type
                  Property_Result := Formal_Discrete_Type_Def_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Discrete_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_int_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Int_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def .. Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.is_int_type
                  Property_Result := Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_int_type
                  Property_Result := Derived_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.is_int_type
                  Property_Result := Mod_Int_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.is_int_type
                  Property_Result := Signed_Int_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Int_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_access_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Access_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_access_type
                  Property_Result := Type_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
                  --# property-call-start AccessDef.is_access_type
                  Property_Result := Access_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_access_type
                  Property_Result := Derived_Type_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Access_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_char_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Char_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Formal_Discrete_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_char_type
                  Property_Result := Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_char_type
                  Property_Result := Derived_Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_char_type
                  Property_Result := Enum_Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Char_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Enum_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Formal_Discrete_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_enum_type
                  Property_Result := Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_enum_type
                  Property_Result := Derived_Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_enum_type
                  Property_Result := Enum_Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Enum_Type;
--# end
--# property-start '[dispatcher]TypeDef.accessed_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Accessed_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.accessed_type
                  Property_Result := Type_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Access_To_Subp_Def =>
                  --# property-call-start AccessToSubpDef.accessed_type
                  Property_Result := Access_To_Subp_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Access_Def =>
                  --# property-call-start AnonymousTypeAccessDef.accessed_type
                  Property_Result := Anonymous_Type_Access_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Access_Def =>
                  --# property-call-start TypeAccessDef.accessed_type
                  Property_Result := Type_Access_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.accessed_type
                  Property_Result := Derived_Type_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Accessed_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_tagged_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def | Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_tagged_type
                  Property_Result := Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_tagged_type
                  Property_Result := Derived_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.is_tagged_type
                  Property_Result := Interface_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.is_tagged_type
                  Property_Result := Private_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.is_tagged_type
                  Property_Result := Record_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start '[dispatcher]TypeDef.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Base_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.base_type
                  Property_Result := Type_Def_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.base_type
                  Property_Result := Derived_Type_Def_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Base_Type;
--# end
--# property-start TypeDef.base_types /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4778
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Types
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_298 with Inline_Always;
            procedure Finalizer_Scope_2430 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Singleton : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_298 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_298;
            procedure Finalizer_Scope_2430 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2430;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11942 '<Concat at ast.py:4780>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4780
--# expr-start 11940 '<FieldAccess for base_type at ast.py:4779>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4779
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 11940
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
                Singleton := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2430;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 11941 '<FieldAccess for base_interfaces at ast.py:4780>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4780
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Interfaces (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11941
Concat_Result := Concat (Result_Var, Fld_1); 
--# expr-done 11942
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_298;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_298;
                     Finalizer_Scope_2430;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Base_Types;
--# end
--# property-start '[dispatcher]TypeDef.base_interfaces' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Base_Interfaces
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.base_interfaces
                  Property_Result := Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.base_interfaces
                  Property_Result := Derived_Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.base_interfaces
                  Property_Result := Interface_Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Base_Interfaces;
--# end
--# property-start TypeDef.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4793
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Defining_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_300 with Inline_Always;
            procedure Finalizer_Scope_2431 with Inline_Always;
            procedure Finalizer_Scope_2432 with Inline_Always;
            procedure Finalizer_Scope_2433 with Inline_Always;
            procedure Finalizer_Scope_2434 with Inline_Always;
            procedure Finalizer_Scope_2435 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Array_Type_Def;
Match_1 : Internal_Entity_Access_Def;
Match_2 : Internal_Entity_Type_Def;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Is_A_1 : Boolean;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Scope_Result : Bare_Ada_Node;
Fld_5 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Bound_Dottable_Type : Bare_Ada_Node;
Fld_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_7 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Dyn_Var_Bind_Result : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Fld_8 : Lexical_Env := Empty_Env;
Singleton : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Array_Type_Def;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Access_Def;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_5 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_300 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Match_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_300;
            procedure Finalizer_Scope_2431 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2431;
            procedure Finalizer_Scope_2432 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2432;
            procedure Finalizer_Scope_2433 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2433;
            procedure Finalizer_Scope_2434 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_2434;
            procedure Finalizer_Scope_2435 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2435;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11968 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 11967 '<Cond at ast.py:4798>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4798
--# expr-start 11943 '<IsA RecordTypeDef, PrivateTypeDef at ast.py:4802>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4802
Is_A := Self /= null 
and then Self.Kind in Ada_Record_Type_Def_Range | Ada_Private_Type_Def_Range; 
--# expr-done 11943
if Is_A then
   --# expr-start 11948 '<env_group at ast.py:4807>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4807
--# expr-start 11947 '<ArrayLiteral at ast.py:4803>' Array_Lit /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4803
--# expr-start 11944 '<FieldAccess for children_env at ast.py:4804>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4804
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11944
--# expr-start 11945 '<FieldAccess for dottable_subps_env at ast.py:4805>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4805
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_1 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 11945
--# expr-start 11946 '<FieldAccess for previous_part_env at ast.py:4806>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4806
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.previous_part_env
Fld_2 := Libadalang.Implementation.Type_Def_P_Previous_Part_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 11946
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_1, 3 => Fld_2)); 
--# expr-done 11947
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 11948
   If_Result_1 := Group_Env;
else
--# expr-start 11949 '<IsA DerivedTypeDef, InterfaceTypeDef at ast.py:4811>' Is_A_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4811
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 11949
if Is_A_1 then
   --# expr-start 11961 '<env_group at ast.py:4823>' Group_Env_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4823
--# expr-start 11960 '<Concat at ast.py:4823>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4823
--# expr-start 11958 '<Concat at ast.py:4818>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4818
--# expr-start 11952 '<ArrayLiteral at ast.py:4815>' Array_Lit_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4815
--# expr-start 11950 '<FieldAccess for children_env at ast.py:4816>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4816
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11950
--# expr-start 11951 '<FieldAccess for dottable_subps_env at ast.py:4817>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4817
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_4 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 11951
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_3, 2 => Fld_4)); 
--# expr-done 11952
--# expr-start 11957 '<bind at ast.py:4818>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4818
Var_Expr := Dottable_Type;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 11953 '<parent at ast.py:4821>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4821
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Parent; 
--# expr-done 11953
   Result_Var := Fld_5;
end if;
Bound_Dottable_Type := Result_Var; 
--# expr-start 11956 '<Map at ast.py:4822>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4822
   --# expr-start 11954 '<FieldAccess for base_types at ast.py:4822>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4822
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.base_types
Fld_6 := Libadalang.Implementation.Type_Def_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 11954
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_6;
      begin
         for Item_358 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_358
Var_Expr_1 := Item_358;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 11955 '<FieldAccess for defining_env at ast.py:4822>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4822
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr_1.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Bound_Dottable_Type, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11955
Scope_Result_1 := Fld_7;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2432;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
         declare
            Item_To_Append : constant Lexical_Env := Result_Var_1;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2431;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11956
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 11957
Concat_Result := Concat (Array_Lit_1, Dyn_Var_Bind_Result); 
--# expr-done 11958
--# expr-start 11959 '<FieldAccess for previous_part_env at ast.py:4823>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4823
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.previous_part_env
Fld_8 := Libadalang.Implementation.Type_Def_P_Previous_Part_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 11959
                Singleton := Create_Lexical_Env_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_8;
                Inc_Ref (Singleton.Items (1));
Concat_Result_1 := Concat (Concat_Result, Singleton); 
--# expr-done 11960
Group_Env_1 := Group (Concat_Result_1, No_Metadata); 
--# expr-done 11961
   If_Result := Group_Env_1;
else
   --# expr-start 11966 '<Match at ast.py:4825>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4825
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ar Match_0
--# expr-start 11963 '<FieldAccess for defining_env at ast.py:4826>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4826
--# expr-start 11962 '<FieldAccess for comp_type at ast.py:4826>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4826
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_9 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 11962
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_9.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld_9.Info);
--# end
--# expr-done 11963
Let_Result := Fld_10; Inc_Ref (Let_Result);
--# end
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2433;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ac Match_1
--# expr-start 11964 '<FieldAccess for accessed_type at ast.py:4830>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4830
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_11 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 11964
Var_Expr_2 := Fld_11;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 11965 '<FieldAccess for defining_env at ast.py:4830>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4830
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr_2.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 11965
Scope_Result_2 := Fld_12;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2435;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_4 := Let_Result_1;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2434;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Ada_Derived_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := Empty_Env; Inc_Ref (Let_Result_2);
--# end
Scope_Result_5 := Let_Result_2;
   Inc_Ref (Scope_Result_5);
   --# end
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
end case;
--# expr-done 11966
   If_Result := Match_Result;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 11967
Let_Result_3 := If_Result_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 11968
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_300;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_300;
                     Finalizer_Scope_2431;
                     Finalizer_Scope_2432;
                     Finalizer_Scope_2433;
                     Finalizer_Scope_2434;
                     Finalizer_Scope_2435;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Defining_Env;
--# end
--# property-start TypeDef.containing_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4836
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Containing_Type
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11970 '<Cast to TypeDecl at ast.py:4837>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4837
--# expr-start 11969 '<parent at ast.py:4837>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4837
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11969
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11970
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Containing_Type;
--# end
--# property-start TypeDef.previous_part /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4842
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Previous_Part
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11972 '<FieldAccess for previous_part at ast.py:4842>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4842
--# expr-start 11971 '<FieldAccess for containing_type at ast.py:4842>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4842
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.containing_type
Fld := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11971
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Fld.Node, Go_To_Incomplete => True, E_Info => Fld.Info);
--# end
--# expr-done 11972
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Previous_Part;
--# end
--# property-start TypeDef.previous_part_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4844
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Previous_Part_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
      ; Dottable_Type : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_303 with Inline_Always;
            procedure Finalizer_Scope_2428 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_303 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_303;
            procedure Finalizer_Scope_2428 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2428;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11973 '<FieldAccess for previous_part at ast.py:4845>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4845
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.previous_part
Fld := Libadalang.Implementation.Type_Def_P_Previous_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11973
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 11974 '<FieldAccess for defining_env at ast.py:4845>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4845
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => Dottable_Type, E_Info => Var_Expr.Info);
--# end
--# expr-done 11974
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2428;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_303;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_303;
                     Finalizer_Scope_2428;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Previous_Part_Env;
--# end
--# property-start TypeDef.dottable_subps_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4849
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Dottable_Subps_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
      ; Dottable_Type : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_304 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Info;
Fld_3 : Env_Rebindings;
Rebound_Env : Lexical_Env := Empty_Env;
Singleton : Lexical_Env_Array_Access;
Fld_4 : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Scope_Result : Internal_Metadata;
Result_Var : Internal_Metadata;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_304 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Rebound_Env);
                     Dec_Ref (Singleton);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_304;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11982 '<env_group at ast.py:4862>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4862
--# expr-start 11979 '<rebind_env at ast.py:4860>' Rebound_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4860
--# expr-start 11976 '<FieldAccess for dottable_subps_env at ast.py:4860>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4860
--# expr-start 11975 '<FieldAccess for containing_type at ast.py:4860>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4860
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.containing_type
Fld := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11975
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.dottable_subps_env
Fld_1 := Libadalang.Implementation.Base_Type_Decl_F_Dottable_Subps_Env (Node => Fld.Node);
--# end
--# expr-done 11976
--# expr-start 11978 '<FieldAccess for rebindings at ast.py:4861>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4861
--# expr-start 11977 '<FieldAccess for info at ast.py:4861>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4861
Fld_2 := Ent.Info;
--# expr-done 11977
Fld_3 := Fld_2.Rebindings;
--# expr-done 11978
Rebound_Env := Rebind_Env (Fld_1, Fld_3); 
--# expr-done 11979
                Singleton := Create_Lexical_Env_Array (Items_Count => 1);
                Singleton.Items (1) := Rebound_Env;
                Inc_Ref (Singleton.Items (1));
Var_Expr := Dottable_Type;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 11981 '<New Metadata at ast.py:4863>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4863
--# expr-start 11980 '<parent at ast.py:4864>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4864
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Parent; 
--# expr-done 11980
New_Struct := (Dottable_Subp => False, Primitive => Fld_4, Primitive_Real_Type => Var_Expr); 
--# expr-done 11981
Scope_Result := New_Struct;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Metadata;
end if;
Group_Env := Group (Singleton, Result_Var); 
--# expr-done 11982
         Property_Result := Group_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_304;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_304;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Dottable_Subps_Env;
--# end
--# property-start '[dispatcher]TypeDef.is_static' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Static
  (Node : Bare_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Formal_Discrete_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def | Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.is_static
                  Property_Result := Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.is_static
                  Property_Result := Array_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_static
                  Property_Result := Derived_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_static
                  Property_Result := Enum_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.is_static
                  Property_Result := Mod_Int_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start RealTypeDef.is_static
                  Property_Result := Real_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.is_static
                  Property_Result := Signed_Int_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Static;
--# end
--# property-start TypeDef.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4686
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Discrete_Range
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Discrete_Range;
      Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11983 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Discrete_Range; 
--# end
--# expr-done 11983
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Discrete_Range;
--# end
--# property-start TypeDef.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4693
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Array_Ndims
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := 0;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Array_Ndims;
--# end
--# property-start TypeDef.is_float_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4708
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Float_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Float_Type;
--# end
--# property-start TypeDef.is_fixed_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4712
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Fixed_Point
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Fixed_Point;
--# end
--# property-start TypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4717
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Predefined_Operators
  (Node : Bare_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_288 with Inline_Always;
      Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_288 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_288;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11984 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Internal_Env_Assoc_Array_Type; Inc_Ref (Let_Result);
--# end
--# expr-done 11984
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_288;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_288;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Predefined_Operators;
--# end
--# property-start TypeDef.is_discrete_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4737
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Discrete_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11990 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11985 '<FieldAccess for base_type at ast.py:4739>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4739
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 11985
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 11986 '<FieldAccess for is_discrete_type at ast.py:4740>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4740
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 11986
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
--# expr-start 11987 '<FieldAccess for is_int_type at ast.py:4742>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4742
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_int_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 11987
if Fld_2 then
   If_Result := True;
else
   --# expr-start 11988 '<FieldAccess for is_enum_type at ast.py:4742>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4742
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_enum_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 11988
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 11989 '<FieldAccess for is_char_type at ast.py:4742>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4742
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_char_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 11989
   If_Result_1 := Fld_4;
end if;
   Result_Var := If_Result_1;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 11990
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Discrete_Type;
--# end
--# property-start TypeDef.is_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4746
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Int_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11991 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 11991
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Int_Type;
--# end
--# property-start TypeDef.is_access_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4751
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Access_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Access_Type;
--# end
--# property-start TypeDef.is_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4757
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Char_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Char_Type;
--# end
--# property-start TypeDef.is_enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4758
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Enum_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Enum_Type;
--# end
--# property-start TypeDef.accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4760
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Accessed_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11992 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 11992
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Accessed_Type;
--# end
--# property-start TypeDef.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4764
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Tagged_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11993 '<Block at ???>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 11993
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Tagged_Type;
--# end
--# property-start TypeDef.base_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4771
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Base_Type;
--# end
--# property-start TypeDef.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4787
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Interfaces
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Base_Interfaces;
--# end
--# property-start TypeDef.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4871
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Static
  (Node : Bare_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Access_Def
   --
      procedure Initialize_Fields_For_Access_Def
        (Self : Bare_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
        ) is
      begin
            Self.Access_Def_F_Has_Not_Null := Access_Def_F_Has_Not_Null;
      end Initialize_Fields_For_Access_Def;
   function Access_Def_F_Has_Not_Null
     (Node : Bare_Access_Def) return Bare_Not_Null
   is
   begin
         return Node.Access_Def_F_Has_Not_Null;
   end;
--# property-start AccessDef.is_access_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9175
pragma Warnings (Off, "is not referenced");
 function Access_Def_P_Is_Access_Type
  (Node : Bare_Access_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Access_Def  := Bare_Access_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Access_Def_P_Is_Access_Type;
--# end
--# property-start AccessDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9177
pragma Warnings (Off, "is not referenced");
 function Access_Def_P_Predefined_Operators
  (Node : Bare_Access_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Access_Def  := Bare_Access_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_731 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_731 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_731;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Access_Def_Access_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Access_Def,
                                As_Bare_Access_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 11995 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 11994 '<FieldAccess for predefined_equality_operators at ast.py:9179>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9179
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.predefined_equality_operators
Fld := Libadalang.Implementation.Type_Def_P_Predefined_Equality_Operators (Node => Self);
--# end
--# expr-done 11994
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 11995
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_731;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_731;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Access_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Access_To_Subp_Def
   --
      procedure Access_To_Subp_Def_Pre_Env_Actions
        (Self            : Bare_Access_To_Subp_Def;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_16 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Access_To_Subp_Def
        (Self : Bare_Access_To_Subp_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Access_To_Subp_Def_F_Has_Protected : Bare_Protected_Node
         ; Access_To_Subp_Def_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Initialize_Fields_For_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Access_To_Subp_Def_F_Has_Protected := Access_To_Subp_Def_F_Has_Protected;
            Self.Access_To_Subp_Def_F_Subp_Spec := Access_To_Subp_Def_F_Subp_Spec;
      end Initialize_Fields_For_Access_To_Subp_Def;
   function Access_To_Subp_Def_F_Has_Protected
     (Node : Bare_Access_To_Subp_Def) return Bare_Protected_Node
   is
   begin
         return Node.Access_To_Subp_Def_F_Has_Protected;
   end;
   function Access_To_Subp_Def_F_Subp_Spec
     (Node : Bare_Access_To_Subp_Def) return Bare_Subp_Spec
   is
   begin
         return Node.Access_To_Subp_Def_F_Subp_Spec;
   end;
--# property-start AccessToSubpDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9189
pragma Warnings (Off, "is not referenced");
 function Access_To_Subp_Def_P_Xref_Equation
  (Node : Bare_Access_To_Subp_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Access_To_Subp_Def  := Bare_Access_To_Subp_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_732 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_732 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_732;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9189") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_732;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_732;
            raise;
      end;
   return Property_Result;
end Access_To_Subp_Def_P_Xref_Equation;
--# end
--# property-start AccessToSubpDef.accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9191
pragma Warnings (Off, "is not referenced");
 function Access_To_Subp_Def_P_Accessed_Type
  (Node : Bare_Access_To_Subp_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Access_To_Subp_Def  := Bare_Access_To_Subp_Def (Node);
     Ent : Internal_Entity_Access_To_Subp_Def :=
       Internal_Entity_Access_To_Subp_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11997 '<FieldAccess for return_type at ast.py:9191>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9191
--# expr-start 11996 '<FieldAccess for subp_spec at ast.py:9191>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9191
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Ent.Info);
--# expr-done 11996
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11997
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Access_To_Subp_Def_P_Accessed_Type;
--# end
--# property-start AccessToSubpDef._env_trans_parent_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9195
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_16
  (Node : Bare_Access_To_Subp_Def
  )
   return Boolean
is
   Self : Bare_Access_To_Subp_Def  := Bare_Access_To_Subp_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_16;
--# end
   --
   --  Primitives for Bare_Base_Type_Access_Def
   --
      procedure Initialize_Fields_For_Base_Type_Access_Def
        (Self : Bare_Base_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
        ) is
      begin
            Initialize_Fields_For_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
      end Initialize_Fields_For_Base_Type_Access_Def;
   --
   --  Primitives for Bare_Anonymous_Type_Access_Def
   --
      procedure Initialize_Fields_For_Anonymous_Type_Access_Def
        (Self : Bare_Anonymous_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Anonymous_Type_Access_Def_F_Type_Decl : Bare_Base_Type_Decl
        ) is
      begin
            Initialize_Fields_For_Base_Type_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Anonymous_Type_Access_Def_F_Type_Decl := Anonymous_Type_Access_Def_F_Type_Decl;
      end Initialize_Fields_For_Anonymous_Type_Access_Def;
   function Anonymous_Type_Access_Def_F_Type_Decl
     (Node : Bare_Anonymous_Type_Access_Def) return Bare_Base_Type_Decl
   is
   begin
         return Node.Anonymous_Type_Access_Def_F_Type_Decl;
   end;
--# property-start AnonymousTypeAccessDef.accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9226
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Access_Def_P_Accessed_Type
  (Node : Bare_Anonymous_Type_Access_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Anonymous_Type_Access_Def  := Bare_Anonymous_Type_Access_Def (Node);
     Ent : Internal_Entity_Anonymous_Type_Access_Def :=
       Internal_Entity_Anonymous_Type_Access_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11998 '<FieldAccess for type_decl at ast.py:9226>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9226
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Type_Decl (Node => Ent.Node.Anonymous_Type_Access_Def_F_Type_Decl, Info => Ent.Info);
--# expr-done 11998
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Type_Access_Def_P_Accessed_Type;
--# end
   --
   --  Primitives for Bare_Type_Access_Def
   --
      procedure Initialize_Fields_For_Type_Access_Def
        (Self : Bare_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Type_Access_Def_F_Has_All : Bare_All_Node
         ; Type_Access_Def_F_Has_Constant : Bare_Constant_Node
         ; Type_Access_Def_F_Subtype_Indication : Bare_Subtype_Indication
        ) is
      begin
            Initialize_Fields_For_Base_Type_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Type_Access_Def_F_Has_All := Type_Access_Def_F_Has_All;
            Self.Type_Access_Def_F_Has_Constant := Type_Access_Def_F_Has_Constant;
            Self.Type_Access_Def_F_Subtype_Indication := Type_Access_Def_F_Subtype_Indication;
      end Initialize_Fields_For_Type_Access_Def;
   function Type_Access_Def_F_Has_All
     (Node : Bare_Type_Access_Def) return Bare_All_Node
   is
   begin
         return Node.Type_Access_Def_F_Has_All;
   end;
   function Type_Access_Def_F_Has_Constant
     (Node : Bare_Type_Access_Def) return Bare_Constant_Node
   is
   begin
         return Node.Type_Access_Def_F_Has_Constant;
   end;
   function Type_Access_Def_F_Subtype_Indication
     (Node : Bare_Type_Access_Def) return Bare_Subtype_Indication
   is
   begin
         return Node.Type_Access_Def_F_Subtype_Indication;
   end;
--# property-start TypeAccessDef.accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9214
pragma Warnings (Off, "is not referenced");
 function Type_Access_Def_P_Accessed_Type
  (Node : Bare_Type_Access_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Access_Def  := Bare_Type_Access_Def (Node);
     Ent : Internal_Entity_Type_Access_Def :=
       Internal_Entity_Type_Access_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12000 '<FieldAccess for designated_type at ast.py:9214>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9214
--# expr-start 11999 '<FieldAccess for subtype_indication at ast.py:9214>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9214
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Type_Access_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 11999
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12000
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Access_Def_P_Accessed_Type;
--# end
--# property-start TypeAccessDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9215
pragma Warnings (Off, "is not referenced");
 function Type_Access_Def_P_Xref_Equation
  (Node : Bare_Type_Access_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Type_Access_Def  := Bare_Type_Access_Def (Node);
     Ent : Internal_Entity_Type_Access_Def :=
       Internal_Entity_Type_Access_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_735 with Inline_Always;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_735 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_735;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12002 '<FieldAccess for xref_equation at ast.py:9215>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9215
--# expr-start 12001 '<FieldAccess for subtype_indication at ast.py:9215>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9215
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Type_Access_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12001
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12002
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_735;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_735;
            raise;
      end;
   return Property_Result;
end Type_Access_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Array_Type_Def
   --
      procedure Initialize_Fields_For_Array_Type_Def
        (Self : Bare_Array_Type_Def
         ; Array_Type_Def_F_Indices : Bare_Array_Indices
         ; Array_Type_Def_F_Component_Type : Bare_Component_Def
        ) is
      begin
            Self.Array_Type_Def_F_Indices := Array_Type_Def_F_Indices;
            Self.Array_Type_Def_F_Component_Type := Array_Type_Def_F_Component_Type;
      end Initialize_Fields_For_Array_Type_Def;
   function Array_Type_Def_F_Indices
     (Node : Bare_Array_Type_Def) return Bare_Array_Indices
   is
   begin
         return Node.Array_Type_Def_F_Indices;
   end;
   function Array_Type_Def_F_Component_Type
     (Node : Bare_Array_Type_Def) return Bare_Component_Def
   is
   begin
         return Node.Array_Type_Def_F_Component_Type;
   end;
--# property-start ArrayTypeDef.comp_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8817
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Comp_Type
  (Node : Bare_Array_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12006 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12005 '<FieldAccess for designated_type at ast.py:8820>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8820
--# expr-start 12004 '<FieldAccess for type_expr at ast.py:8820>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8820
--# expr-start 12003 '<FieldAccess for component_type at ast.py:8820>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8820
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Array_Type_Def_F_Component_Type, Info => Ent.Info);
--# expr-done 12003
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 12004
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 12005
Let_Result := Fld_2; 
--# end
--# expr-done 12006
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Comp_Type;
--# end
--# property-start ArrayTypeDef.index_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8822
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Index_Type
  (Node : Bare_Array_Type_Def
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12009 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12008 '<FieldAccess for index_type at ast.py:8824>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8824
--# expr-start 12007 '<FieldAccess for indices at ast.py:8824>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8824
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 12007
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Fld.Node, Dim => Dim, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12008
Let_Result := Fld_1; 
--# end
--# expr-done 12009
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Index_Type;
--# end
--# property-start ArrayTypeDef.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8826
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Array_Ndims
  (Node : Bare_Array_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Bare_Array_Indices;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12011 '<FieldAccess for ndims at ast.py:8826>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8826
--# expr-start 12010 '<FieldAccess for indices at ast.py:8826>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8826
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Array_Type_Def_F_Indices;
--# expr-done 12010
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Ndims (Node => Fld);
--# end
--# expr-done 12011
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Array_Ndims;
--# end
--# property-start ArrayTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8828
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Xref_Equation
  (Node : Bare_Array_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_667 with Inline_Always;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Component_Def;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_667 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_667;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12017 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12016 '<And at ast.py:8830>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8830
--# expr-start 12013 '<FieldAccess for sub_equation at ast.py:8831>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8831
--# expr-start 12012 '<FieldAccess for indices at ast.py:8831>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8831
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 12012
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12013
--# expr-start 12015 '<FieldAccess for sub_equation at ast.py:8832>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8832
--# expr-start 12014 '<FieldAccess for component_type at ast.py:8832>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8832
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Component_Def (Node => Ent.Node.Array_Type_Def_F_Component_Type, Info => Ent.Info);
--# expr-done 12014
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 12015
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8830") else null)); 
--# expr-done 12016
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 12017
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_667;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_667;
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Xref_Equation;
--# end
--# property-start ArrayTypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8835
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Predefined_Operators
  (Node : Bare_Array_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_668 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Block_Var_2 : Bare_Type_Expr;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Bare_Component_Def;
Fld_4 : Bare_Type_Expr;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_668 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_668;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Array_Type_Def_Array_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Array_Type_Def,
                                As_Bare_Array_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12053 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12019 '<Cast to TypeDecl at ast.py:8837>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8837
--# expr-start 12018 '<parent at ast.py:8837>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8837
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12018
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12019
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12021 '<FieldAccess for node at ast.py:8838>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8838
--# expr-start 12020 '<FieldAccess for bool_type at ast.py:8838>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8838
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12020
Fld_2 := Fld_1.Node;
--# expr-done 12021
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 12023 '<FieldAccess for type_expr at ast.py:8839>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8839
--# expr-start 12022 '<FieldAccess for component_type at ast.py:8839>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8839
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Array_Type_Def_F_Component_Type;
--# expr-done 12022
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Fld_3.Component_Def_F_Type_Expr;
--# expr-done 12023
Block_Var_2 := Fld_4; 
--# bind comp_type_expr Block_Var_2
--# expr-start 12052 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 12025 '<FieldAccess for create_binop_assoc at ast.py:8850>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8850
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12024 '<Symbol "<" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12024
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Block_Var_1);
--# end
--# expr-done 12025
--# expr-start 12027 '<FieldAccess for create_binop_assoc at ast.py:8851>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8851
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12026 '<Symbol "<=" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12026
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_3, Rhs => Cast_Result_4, Ret => Block_Var_1);
--# end
--# expr-done 12027
--# expr-start 12029 '<FieldAccess for create_binop_assoc at ast.py:8852>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8852
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12028 '<Symbol "=" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12028
Cast_Expr_4 := Block_Var; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Block_Var; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_5, Rhs => Cast_Result_6, Ret => Block_Var_1);
--# end
--# expr-done 12029
--# expr-start 12031 '<FieldAccess for create_binop_assoc at ast.py:8853>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8853
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12030 '<Symbol "/=" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12030
Cast_Expr_6 := Block_Var; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Block_Var; 
      Cast_Result_8 := Cast_Expr_7;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Block_Var_1);
--# end
--# expr-done 12031
--# expr-start 12033 '<FieldAccess for create_binop_assoc at ast.py:8854>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8854
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12032 '<Symbol ">" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12032
Cast_Expr_8 := Block_Var; 
      Cast_Result_9 := Cast_Expr_8;
Cast_Expr_9 := Block_Var; 
      Cast_Result_10 := Cast_Expr_9;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_9, Rhs => Cast_Result_10, Ret => Block_Var_1);
--# end
--# expr-done 12033
--# expr-start 12035 '<FieldAccess for create_binop_assoc at ast.py:8855>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8855
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12034 '<Symbol ">=" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12034
Cast_Expr_10 := Block_Var; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Block_Var; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_11, Rhs => Cast_Result_12, Ret => Block_Var_1);
--# end
--# expr-done 12035
--# expr-start 12037 '<FieldAccess for create_binop_assoc at ast.py:8856>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8856
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12036 '<Symbol "and" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 12036
Cast_Expr_12 := Block_Var; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Block_Var; 
      Cast_Result_14 := Cast_Expr_13;
Cast_Expr_14 := Block_Var; 
      Cast_Result_15 := Cast_Expr_14;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_13, Rhs => Cast_Result_14, Ret => Cast_Result_15);
--# end
--# expr-done 12037
--# expr-start 12039 '<FieldAccess for create_binop_assoc at ast.py:8857>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8857
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12038 '<Symbol "or" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 12038
Cast_Expr_15 := Block_Var; 
      Cast_Result_16 := Cast_Expr_15;
Cast_Expr_16 := Block_Var; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Block_Var; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_16, Rhs => Cast_Result_17, Ret => Cast_Result_18);
--# end
--# expr-done 12039
--# expr-start 12041 '<FieldAccess for create_binop_assoc at ast.py:8858>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8858
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12040 '<Symbol "xor" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor); 
--# expr-done 12040
Cast_Expr_18 := Block_Var; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Block_Var; 
      Cast_Result_20 := Cast_Expr_19;
Cast_Expr_20 := Block_Var; 
      Cast_Result_21 := Cast_Expr_20;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_19, Rhs => Cast_Result_20, Ret => Cast_Result_21);
--# end
--# expr-done 12041
--# expr-start 12043 '<FieldAccess for create_unop_assoc at ast.py:8859>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8859
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12042 '<Symbol "not" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 12042
Cast_Expr_21 := Block_Var; 
      Cast_Result_22 := Cast_Expr_21;
Cast_Expr_22 := Block_Var; 
      Cast_Result_23 := Cast_Expr_22;
--# property-call-start AdaNode.create_unop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_9, Rhs => Cast_Result_22, Ret => Cast_Result_23);
--# end
--# expr-done 12043
--# expr-start 12045 '<FieldAccess for create_binop_assoc_l_r_expr at ast.py:8862>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8862
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12044 '<Symbol "&" at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 12044
Cast_Expr_23 := Block_Var; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc_l_r_expr
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_L_R_Expr (Node => Self, Op => Sym_10, Lhs => Block_Var_2, Rhs => Block_Var_2, Ret => Cast_Result_24);
--# end
--# expr-done 12045
--# expr-start 12047 '<FieldAccess for create_binop_assoc_l_expr at ast.py:8865>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8865
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12046 '<Symbol "&" at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 12046
Cast_Expr_24 := Block_Var; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Block_Var; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc_l_expr
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_L_Expr (Node => Self, Op => Sym_11, Lhs => Block_Var_2, Rhs => Cast_Result_25, Ret => Cast_Result_26);
--# end
--# expr-done 12047
--# expr-start 12049 '<FieldAccess for create_binop_assoc_r_expr at ast.py:8868>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8868
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12048 '<Symbol "&" at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 12048
Cast_Expr_26 := Block_Var; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Block_Var; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc_r_expr
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_R_Expr (Node => Self, Op => Sym_12, Lhs => Cast_Result_27, Rhs => Block_Var_2, Ret => Cast_Result_28);
--# end
--# expr-done 12049
--# expr-start 12051 '<FieldAccess for create_binop_assoc at ast.py:8871>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8871
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12050 '<Symbol "&" at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 12050
Cast_Expr_28 := Block_Var; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Block_Var; 
      Cast_Result_30 := Cast_Expr_29;
Cast_Expr_30 := Block_Var; 
      Cast_Result_31 := Cast_Expr_30;
--# property-call-start AdaNode.create_binop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_13, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Cast_Result_31);
--# end
--# expr-done 12051
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18)); 
--# expr-done 12052
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12053
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_668;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_668;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Array_Type_Def_P_Predefined_Operators;
--# end
--# property-start ArrayTypeDef.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8876
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Is_Static
  (Node : Bare_Array_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12055 '<FieldAccess for is_static at ast.py:8876>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8876
--# expr-start 12054 '<FieldAccess for indices at ast.py:8876>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8876
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 12054
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.is_static'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Is_Static (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 12055
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Is_Static;
--# end
--# property-start ArrayTypeDef.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8878
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Xref_Entry_Point
  (Node : Bare_Array_Type_Def
  )
   return Boolean
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Derived_Type_Def
   --
      procedure Initialize_Fields_For_Derived_Type_Def
        (Self : Bare_Derived_Type_Def
         ; Derived_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Derived_Type_Def_F_Has_Limited : Bare_Limited_Node
         ; Derived_Type_Def_F_Has_Synchronized : Bare_Synchronized_Node
         ; Derived_Type_Def_F_Subtype_Indication : Bare_Subtype_Indication
         ; Derived_Type_Def_F_Interfaces : Bare_Parent_List
         ; Derived_Type_Def_F_Record_Extension : Bare_Base_Record_Def
         ; Derived_Type_Def_F_Has_With_Private : Bare_With_Private
        ) is
      begin
            Self.Derived_Type_Def_F_Has_Abstract := Derived_Type_Def_F_Has_Abstract;
            Self.Derived_Type_Def_F_Has_Limited := Derived_Type_Def_F_Has_Limited;
            Self.Derived_Type_Def_F_Has_Synchronized := Derived_Type_Def_F_Has_Synchronized;
            Self.Derived_Type_Def_F_Subtype_Indication := Derived_Type_Def_F_Subtype_Indication;
            Self.Derived_Type_Def_F_Interfaces := Derived_Type_Def_F_Interfaces;
            Self.Derived_Type_Def_F_Record_Extension := Derived_Type_Def_F_Record_Extension;
            Self.Derived_Type_Def_F_Has_With_Private := Derived_Type_Def_F_Has_With_Private;
      end Initialize_Fields_For_Derived_Type_Def;
   function Derived_Type_Def_F_Has_Abstract
     (Node : Bare_Derived_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Abstract;
   end;
   function Derived_Type_Def_F_Has_Limited
     (Node : Bare_Derived_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Limited;
   end;
   function Derived_Type_Def_F_Has_Synchronized
     (Node : Bare_Derived_Type_Def) return Bare_Synchronized_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Synchronized;
   end;
   function Derived_Type_Def_F_Subtype_Indication
     (Node : Bare_Derived_Type_Def) return Bare_Subtype_Indication
   is
   begin
         return Node.Derived_Type_Def_F_Subtype_Indication;
   end;
   function Derived_Type_Def_F_Interfaces
     (Node : Bare_Derived_Type_Def) return Bare_Parent_List
   is
   begin
         return Node.Derived_Type_Def_F_Interfaces;
   end;
   function Derived_Type_Def_F_Record_Extension
     (Node : Bare_Derived_Type_Def) return Bare_Base_Record_Def
   is
   begin
         return Node.Derived_Type_Def_F_Record_Extension;
   end;
   function Derived_Type_Def_F_Has_With_Private
     (Node : Bare_Derived_Type_Def) return Bare_With_Private
   is
   begin
         return Node.Derived_Type_Def_F_Has_With_Private;
   end;
--# property-start DerivedTypeDef.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8523
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Array_Ndims
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12057 '<FieldAccess for array_ndims at ast.py:8523>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8523
--# expr-start 12056 '<FieldAccess for base_type at ast.py:8523>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8523
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12056
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12057
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Array_Ndims;
--# end
--# property-start DerivedTypeDef.base_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8525
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Base_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12059 '<FieldAccess for designated_type at ast.py:8525>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8525
--# expr-start 12058 '<FieldAccess for subtype_indication at ast.py:8525>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8525
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12058
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12059
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Base_Type;
--# end
--# property-start DerivedTypeDef.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8527
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Base_Interfaces
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_624 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Item_359 : Internal_Entity_Name;
Bare_Item_359 : Bare_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_624 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_624;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12062 '<Map at ast.py:8528>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8528
   --# expr-start 12060 '<FieldAccess for interfaces at ast.py:8528>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8528
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Derived_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12060
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_359 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_359 := Untyped_Item_359; 
Is_Null := Bare_Item_359 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_359); 
   If_Result := As_Entity;
end if;
                  Item_359 := If_Result; 
   --# scope-start
               --# bind i Item_359
      --# expr-start 12061 '<FieldAccess for name_designated_type at ast.py:8528>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8528
      if Item_359.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_359.Node, E_Info => Item_359.Info);
--# end
--# expr-done 12061
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12062
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_624;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_624;
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Base_Interfaces;
--# end
--# property-start DerivedTypeDef.is_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8531
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Int_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12064 '<FieldAccess for is_int_type at ast.py:8531>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8531
--# expr-start 12063 '<FieldAccess for base_type at ast.py:8531>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8531
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12063
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12064
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Int_Type;
--# end
--# property-start DerivedTypeDef.is_access_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8532
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Access_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Derived_Type_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Derived_Type_Def;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12067 '<FieldAccess for is_access_type at ast.py:8532>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8532
--# expr-start 12066 '<FieldAccess for base_type at ast.py:8532>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8532
--# expr-start 12065 '<as_bare_entity at ast.py:8532>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8532
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Derived_Type_Def;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 12065
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 12066
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12067
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Access_Type;
--# end
--# property-start DerivedTypeDef.is_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8533
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Char_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12069 '<FieldAccess for is_char_type at ast.py:8533>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8533
--# expr-start 12068 '<FieldAccess for base_type at ast.py:8533>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8533
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12068
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12069
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Char_Type;
--# end
--# property-start DerivedTypeDef.is_float_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8534
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Float_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12071 '<FieldAccess for is_float_type at ast.py:8534>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8534
--# expr-start 12070 '<FieldAccess for base_type at ast.py:8534>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8534
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12070
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12071
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Float_Type;
--# end
--# property-start DerivedTypeDef.is_fixed_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8535
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Fixed_Point
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12073 '<FieldAccess for is_fixed_point at ast.py:8535>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8535
--# expr-start 12072 '<FieldAccess for base_type at ast.py:8535>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8535
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12072
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12073
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Fixed_Point;
--# end
--# property-start DerivedTypeDef.accessed_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8536
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Accessed_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12074 '<FieldAccess for base_type at ast.py:8536>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8536
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12074
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 12075 '<FieldAccess for accessed_type at ast.py:8536>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8536
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 12075
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Accessed_Type;
--# end
--# property-start DerivedTypeDef.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8537
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Record_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_With_Private;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 12078 '<Not at ast.py:8538>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8538
--# expr-start 12077 '<IsNull at ast.py:8538>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8538
--# expr-start 12076 '<FieldAccess for record_extension at ast.py:8538>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8538
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Record_Def (Node => Ent.Node.Derived_Type_Def_F_Record_Extension, Info => Ent.Info);
--# expr-done 12076
Is_Null := Fld.Node = null; 
--# expr-done 12077
Not_Val := not (Is_Null); 
--# expr-done 12078
if Not_Val then
   If_Result := True;
else
   --# expr-start 12080 '<FieldAccess for as_bool at ast.py:8538>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8538
--# expr-start 12079 '<FieldAccess for has_with_private at ast.py:8538>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8538
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_With_Private (Node => Ent.Node.Derived_Type_Def_F_Has_With_Private, Info => Ent.Info);
--# expr-done 12079
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]WithPrivate.as_bool'
Fld_2 := Libadalang.Implementation.Dispatcher_With_Private_P_As_Bool (Node => Fld_1.Node);
--# end
--# expr-done 12080
   If_Result := Fld_2;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start DerivedTypeDef.is_enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8541
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Enum_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12082 '<FieldAccess for is_enum_type at ast.py:8541>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8541
--# expr-start 12081 '<FieldAccess for base_type at ast.py:8541>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8541
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12081
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12082
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Enum_Type;
--# end
--# property-start DerivedTypeDef.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8542
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Static
  (Node : Bare_Derived_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12084 '<FieldAccess for is_static_subtype at ast.py:8542>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8542
--# expr-start 12083 '<FieldAccess for subtype_indication at ast.py:8542>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8542
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12083
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 12084
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Static;
--# end
--# property-start DerivedTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8544
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Xref_Equation
  (Node : Bare_Derived_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_634 with Inline_Always;
            procedure Finalizer_Scope_3343 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Subtype_Indication;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Parent_List;
Fld_4 : Bare_Parent_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Item_360 : Internal_Entity_Name;
Bare_Item_360 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_634 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_634;
            procedure Finalizer_Scope_3343 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3343;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12094 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12093 '<bind at ast.py:8548>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8548
--# expr-start 12085 '<FieldAccess for children_env at ast.py:8548>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8548
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12085
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 12092 '<And at ast.py:8549>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8549
--# expr-start 12087 '<FieldAccess for xref_equation at ast.py:8549>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8549
--# expr-start 12086 '<FieldAccess for subtype_indication at ast.py:8549>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8549
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12086
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 12087
--# expr-start 12091 '<LogicAll at ast.py:8550>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8550
--# expr-start 12090 '<Map at ast.py:8550>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8550
   --# expr-start 12088 '<FieldAccess for interfaces at ast.py:8550>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8550
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Parent_List (Node => Ent.Node.Derived_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12088
Fld_4 := Fld_3.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_4 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_4;
      begin
         for Untyped_Item_360 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_360 := Untyped_Item_360; 
Is_Null := Bare_Item_360 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_360); 
   If_Result := As_Entity;
end if;
                  Item_360 := If_Result; 
   --# scope-start
               --# bind ifc Item_360
      --# expr-start 12089 '<FieldAccess for xref_equation at ast.py:8550>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8550
      if Item_360.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_360.Node, Env => Bound_Env, Origin => Origin, E_Info => Item_360.Info);
--# end
--# expr-done 12089
         declare
            Item_To_Append : constant Logic_Equation := Fld_6;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3343;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12090
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8550") else null)); 
--# expr-done 12091
And_Pred := Create_And (Fld_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8549") else null)); 
--# expr-done 12092
Dyn_Var_Bind_Result := And_Pred; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 12093
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12094
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_634;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_634;
                     Finalizer_Scope_3343;
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Xref_Equation;
--# end
--# property-start DerivedTypeDef.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8553
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Discrete_Range
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12097 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12096 '<FieldAccess for discrete_range at ast.py:8555>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8555
--# expr-start 12095 '<FieldAccess for subtype_indication at ast.py:8555>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8555
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12095
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 12096
Let_Result := Fld_1; 
--# end
--# expr-done 12097
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Discrete_Range;
--# end
   --
   --  Primitives for Bare_Enum_Type_Def
   --
      procedure Initialize_Fields_For_Enum_Type_Def
        (Self : Bare_Enum_Type_Def
         ; Enum_Type_Def_F_Enum_Literals : Bare_Enum_Literal_Decl_List
        ) is
      begin
            Self.Enum_Type_Def_F_Enum_Literals := Enum_Type_Def_F_Enum_Literals;
      end Initialize_Fields_For_Enum_Type_Def;
   function Enum_Type_Def_F_Enum_Literals
     (Node : Bare_Enum_Type_Def) return Bare_Enum_Literal_Decl_List
   is
   begin
         return Node.Enum_Type_Def_F_Enum_Literals;
   end;
--# property-start EnumTypeDef.is_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8079
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Char_Type
  (Node : Bare_Enum_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Enum_Literal_Decl_List;
Fld_1 : Bare_Defining_Name;
Fld_2 : Bare_Name;
Is_A : Boolean;
Item_361 : Bare_Enum_Literal_Decl;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12102 '<AnyQuantifier at ast.py:8079>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8079
   --# expr-start 12098 '<FieldAccess for enum_literals at ast.py:8079>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8079
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Enum_Type_Def_F_Enum_Literals;
--# expr-done 12098
      Quantifier_Result := False;
      if Fld = null then
      null;
      else
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld;
      begin
         for Untyped_Item_361 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_361 := Untyped_Item_361; 
   --# scope-start
               --# bind lit Item_361
      --# expr-start 12101 '<IsA CharLiteral at ast.py:8080>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8080
--# expr-start 12100 '<FieldAccess for name at ast.py:8080>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8080
--# expr-start 12099 '<FieldAccess for name at ast.py:8080>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8080
      if Item_361 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Item_361.Enum_Literal_Decl_F_Name;
--# expr-done 12099
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Fld_1.Defining_Name_F_Name;
--# expr-done 12100
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Char_Literal_Range; 
--# expr-done 12101
      Quantifier_Result := Is_A;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 12102
         Property_Result := Quantifier_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Is_Char_Type;
--# end
--# property-start EnumTypeDef.is_enum_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8083
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Enum_Type
  (Node : Bare_Enum_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Is_Enum_Type;
--# end
--# property-start EnumTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8085
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Xref_Equation
  (Node : Bare_Enum_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_595 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_595 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_595;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8085") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_595;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_595;
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Xref_Equation;
--# end
--# property-start EnumTypeDef.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8087
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Static
  (Node : Bare_Enum_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Is_Static;
--# end
--# property-start EnumTypeDef.is_std_char_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8089
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Std_Char_Type
  (Node : Bare_Enum_Type_Def
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
      Block_Var : Bare_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Is_Equal : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12114 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12104 '<Cast to TypeDecl at ast.py:8091>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8091
--# expr-start 12103 '<parent at ast.py:8091>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8091
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12103
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12104
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12107 '<Eq at ast.py:8092>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8092
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
--# expr-start 12106 '<FieldAccess for node at ast.py:8093>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8093
--# expr-start 12105 '<FieldAccess for std_char_type at ast.py:8093>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8093
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_char_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Char_Type (Node => Self);
--# end
--# expr-done 12105
Fld_2 := Fld_1.Node;
--# expr-done 12106
Is_Equal := Cast_Result_1 = Fld_2; 
--# expr-done 12107
if Is_Equal then
   If_Result := True;
else
   --# expr-start 12110 '<Eq at ast.py:8092>' Is_Equal_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8092
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-start 12109 '<FieldAccess for node at ast.py:8094>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8094
--# expr-start 12108 '<FieldAccess for std_wide_char_type at ast.py:8094>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8094
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_char_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Std_Wide_Char_Type (Node => Self);
--# end
--# expr-done 12108
Fld_4 := Fld_3.Node;
--# expr-done 12109
Is_Equal_1 := Cast_Result_2 = Fld_4; 
--# expr-done 12110
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 12113 '<Eq at ast.py:8092>' Is_Equal_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8092
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
--# expr-start 12112 '<FieldAccess for node at ast.py:8095>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8095
--# expr-start 12111 '<FieldAccess for std_wide_wide_char_type at ast.py:8095>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8095
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_wide_char_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Std_Wide_Wide_Char_Type (Node => Self);
--# end
--# expr-done 12111
Fld_6 := Fld_5.Node;
--# expr-done 12112
Is_Equal_2 := Cast_Result_3 = Fld_6; 
--# expr-done 12113
   If_Result_1 := Is_Equal_2;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 12114
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Is_Std_Char_Type;
--# end
--# property-start EnumTypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8098
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Predefined_Operators
  (Node : Bare_Enum_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_598 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Block_Var_2 : Internal_Env_Assoc_Array_Access;
Block_Var_3 : Internal_Env_Assoc_Array_Access;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Sym_6 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit_1 : Internal_Env_Assoc_Array_Access;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Is_Equal : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Concat_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_598 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_598;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Type_Def_Enum_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Type_Def,
                                As_Bare_Enum_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12144 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12116 '<Cast to TypeDecl at ast.py:8100>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8100
--# expr-start 12115 '<parent at ast.py:8100>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8100
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12115
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12116
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12118 '<FieldAccess for node at ast.py:8101>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8101
--# expr-start 12117 '<FieldAccess for bool_type at ast.py:8101>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8101
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12117
Fld_2 := Fld_1.Node;
--# expr-done 12118
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 12131 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 12120 '<FieldAccess for create_binop_assoc at ast.py:8104>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8104
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12119 '<Symbol "<" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12119
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
--# property-call-start AdaNode.create_binop_assoc
Fld_3 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Block_Var_1);
--# end
--# expr-done 12120
--# expr-start 12122 '<FieldAccess for create_binop_assoc at ast.py:8105>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8105
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12121 '<Symbol "<=" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12121
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
--# property-call-start AdaNode.create_binop_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_3, Rhs => Cast_Result_4, Ret => Block_Var_1);
--# end
--# expr-done 12122
--# expr-start 12124 '<FieldAccess for create_binop_assoc at ast.py:8106>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8106
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12123 '<Symbol "=" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12123
Cast_Expr_4 := Block_Var; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Block_Var; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_5, Rhs => Cast_Result_6, Ret => Block_Var_1);
--# end
--# expr-done 12124
--# expr-start 12126 '<FieldAccess for create_binop_assoc at ast.py:8107>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8107
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12125 '<Symbol "/=" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12125
Cast_Expr_6 := Block_Var; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Block_Var; 
      Cast_Result_8 := Cast_Expr_7;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Block_Var_1);
--# end
--# expr-done 12126
--# expr-start 12128 '<FieldAccess for create_binop_assoc at ast.py:8108>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8108
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12127 '<Symbol ">" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12127
Cast_Expr_8 := Block_Var; 
      Cast_Result_9 := Cast_Expr_8;
Cast_Expr_9 := Block_Var; 
      Cast_Result_10 := Cast_Expr_9;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_9, Rhs => Cast_Result_10, Ret => Block_Var_1);
--# end
--# expr-done 12128
--# expr-start 12130 '<FieldAccess for create_binop_assoc at ast.py:8109>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8109
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12129 '<Symbol ">=" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12129
Cast_Expr_10 := Block_Var; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Block_Var; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_11, Rhs => Cast_Result_12, Ret => Block_Var_1);
--# end
--# expr-done 12130
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_3, 2 => Fld_4, 3 => Fld_5, 4 => Fld_6, 5 => Fld_7, 6 => Fld_8)); 
--# expr-done 12131
Block_Var_2 := Array_Lit; Inc_Ref (Block_Var_2);
--# bind defaults Block_Var_2
--# expr-start 12142 '<If at ast.py:8113>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8113
--# expr-start 12132 '<Eq at ast.py:8114>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8114
Cast_Expr_23 := Block_Var; 
      Cast_Result_24 := Cast_Expr_23;
Is_Equal := Cast_Result_24 = Block_Var_1; 
--# expr-done 12132
if Is_Equal then
   --# expr-start 12141 '<ArrayLiteral at ???>' Array_Lit_1 None
--# expr-start 12134 '<FieldAccess for create_binop_assoc at ast.py:8115>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8115
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12133 '<Symbol "and" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 12133
Cast_Expr_12 := Block_Var; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Block_Var; 
      Cast_Result_14 := Cast_Expr_13;
Cast_Expr_14 := Block_Var; 
      Cast_Result_15 := Cast_Expr_14;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_13, Rhs => Cast_Result_14, Ret => Cast_Result_15);
--# end
--# expr-done 12134
--# expr-start 12136 '<FieldAccess for create_binop_assoc at ast.py:8116>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8116
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12135 '<Symbol "or" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 12135
Cast_Expr_15 := Block_Var; 
      Cast_Result_16 := Cast_Expr_15;
Cast_Expr_16 := Block_Var; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Block_Var; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_16, Rhs => Cast_Result_17, Ret => Cast_Result_18);
--# end
--# expr-done 12136
--# expr-start 12138 '<FieldAccess for create_binop_assoc at ast.py:8117>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8117
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12137 '<Symbol "xor" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor); 
--# expr-done 12137
Cast_Expr_18 := Block_Var; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Block_Var; 
      Cast_Result_20 := Cast_Expr_19;
Cast_Expr_20 := Block_Var; 
      Cast_Result_21 := Cast_Expr_20;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_19, Rhs => Cast_Result_20, Ret => Cast_Result_21);
--# end
--# expr-done 12138
--# expr-start 12140 '<FieldAccess for create_unop_assoc at ast.py:8118>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8118
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12139 '<Symbol "not" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 12139
Cast_Expr_21 := Block_Var; 
      Cast_Result_22 := Cast_Expr_21;
Cast_Expr_22 := Block_Var; 
      Cast_Result_23 := Cast_Expr_22;
--# property-call-start AdaNode.create_unop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_9, Rhs => Cast_Result_22, Ret => Cast_Result_23);
--# end
--# expr-done 12140
Array_Lit_1 := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_9, 2 => Fld_10, 3 => Fld_11, 4 => Fld_12)); 
--# expr-done 12141
   If_Result := Array_Lit_1;
else
   If_Result := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 12142
Block_Var_3 := If_Result; Inc_Ref (Block_Var_3);
--# bind specials Block_Var_3
--# expr-start 12143 '<Concat at ast.py:8122>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8122
Concat_Result := Concat (Block_Var_2, Block_Var_3); 
--# expr-done 12143
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12144
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_598;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_598;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Enum_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Formal_Discrete_Type_Def
   --
--# property-start FormalDiscreteTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9234
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Xref_Equation
  (Node : Bare_Formal_Discrete_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Discrete_Type_Def  := Bare_Formal_Discrete_Type_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_737 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_737 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_737;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9234") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_737;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_737;
            raise;
      end;
   return Property_Result;
end Formal_Discrete_Type_Def_P_Xref_Equation;
--# end
--# property-start FormalDiscreteTypeDef.is_discrete_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9236
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Is_Discrete_Type
  (Node : Bare_Formal_Discrete_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Formal_Discrete_Type_Def  := Bare_Formal_Discrete_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Formal_Discrete_Type_Def_P_Is_Discrete_Type;
--# end
--# property-start FormalDiscreteTypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9238
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Predefined_Operators
  (Node : Bare_Formal_Discrete_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Formal_Discrete_Type_Def  := Bare_Formal_Discrete_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_739 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_739 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_739;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Formal_Discrete_Type_Def_Formal_Discrete_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Formal_Discrete_Type_Def,
                                As_Bare_Formal_Discrete_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12162 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12146 '<Cast to TypeDecl at ast.py:9240>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9240
--# expr-start 12145 '<parent at ast.py:9240>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9240
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12145
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12146
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12148 '<FieldAccess for node at ast.py:9241>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9241
--# expr-start 12147 '<FieldAccess for bool_type at ast.py:9241>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9241
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12147
Fld_2 := Fld_1.Node;
--# expr-done 12148
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 12161 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 12150 '<FieldAccess for create_binop_assoc at ast.py:9243>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9243
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12149 '<Symbol "<" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12149
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
--# property-call-start AdaNode.create_binop_assoc
Fld_3 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Block_Var_1);
--# end
--# expr-done 12150
--# expr-start 12152 '<FieldAccess for create_binop_assoc at ast.py:9244>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9244
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12151 '<Symbol "<=" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12151
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
--# property-call-start AdaNode.create_binop_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_3, Rhs => Cast_Result_4, Ret => Block_Var_1);
--# end
--# expr-done 12152
--# expr-start 12154 '<FieldAccess for create_binop_assoc at ast.py:9245>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9245
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12153 '<Symbol "=" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12153
Cast_Expr_4 := Block_Var; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Block_Var; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_5, Rhs => Cast_Result_6, Ret => Block_Var_1);
--# end
--# expr-done 12154
--# expr-start 12156 '<FieldAccess for create_binop_assoc at ast.py:9246>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9246
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12155 '<Symbol "/=" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12155
Cast_Expr_6 := Block_Var; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Block_Var; 
      Cast_Result_8 := Cast_Expr_7;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Block_Var_1);
--# end
--# expr-done 12156
--# expr-start 12158 '<FieldAccess for create_binop_assoc at ast.py:9247>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9247
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12157 '<Symbol ">" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12157
Cast_Expr_8 := Block_Var; 
      Cast_Result_9 := Cast_Expr_8;
Cast_Expr_9 := Block_Var; 
      Cast_Result_10 := Cast_Expr_9;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_9, Rhs => Cast_Result_10, Ret => Block_Var_1);
--# end
--# expr-done 12158
--# expr-start 12160 '<FieldAccess for create_binop_assoc at ast.py:9248>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9248
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12159 '<Symbol ">=" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12159
Cast_Expr_10 := Block_Var; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Block_Var; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_11, Rhs => Cast_Result_12, Ret => Block_Var_1);
--# end
--# expr-done 12160
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_3, 2 => Fld_4, 3 => Fld_5, 4 => Fld_6, 5 => Fld_7, 6 => Fld_8)); 
--# expr-done 12161
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12162
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_739;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_739;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Formal_Discrete_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Interface_Type_Def
   --
      procedure Initialize_Fields_For_Interface_Type_Def
        (Self : Bare_Interface_Type_Def
         ; Interface_Type_Def_F_Interface_Kind : Bare_Interface_Kind
         ; Interface_Type_Def_F_Interfaces : Bare_Parent_List
        ) is
      begin
            Self.Interface_Type_Def_F_Interface_Kind := Interface_Type_Def_F_Interface_Kind;
            Self.Interface_Type_Def_F_Interfaces := Interface_Type_Def_F_Interfaces;
      end Initialize_Fields_For_Interface_Type_Def;
   function Interface_Type_Def_F_Interface_Kind
     (Node : Bare_Interface_Type_Def) return Bare_Interface_Kind
   is
   begin
         return Node.Interface_Type_Def_F_Interface_Kind;
   end;
   function Interface_Type_Def_F_Interfaces
     (Node : Bare_Interface_Type_Def) return Bare_Parent_List
   is
   begin
         return Node.Interface_Type_Def_F_Interfaces;
   end;
--# property-start InterfaceTypeDef.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8896
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Interface_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Interface_Type_Def  := Bare_Interface_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Interface_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start InterfaceTypeDef.base_interfaces /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8898
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Base_Interfaces
  (Node : Bare_Interface_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Interface_Type_Def  := Bare_Interface_Type_Def (Node);
     Ent : Internal_Entity_Interface_Type_Def :=
       Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_672 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Item_362 : Internal_Entity_Name;
Bare_Item_362 : Bare_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_672 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_672;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12165 '<Map at ast.py:8899>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8899
   --# expr-start 12163 '<FieldAccess for interfaces at ast.py:8899>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8899
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Interface_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12163
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_362 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_362 := Untyped_Item_362; 
Is_Null := Bare_Item_362 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_362); 
   If_Result := As_Entity;
end if;
                  Item_362 := If_Result; 
   --# scope-start
               --# bind i Item_362
      --# expr-start 12164 '<FieldAccess for name_designated_type at ast.py:8899>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8899
      if Item_362.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_362.Node, E_Info => Item_362.Info);
--# end
--# expr-done 12164
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12165
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_672;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_672;
            raise;
      end;
   return Property_Result;
end Interface_Type_Def_P_Base_Interfaces;
--# end
--# property-start InterfaceTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8902
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Xref_Equation
  (Node : Bare_Interface_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Interface_Type_Def  := Bare_Interface_Type_Def (Node);
     Ent : Internal_Entity_Interface_Type_Def :=
       Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_673 with Inline_Always;
            procedure Finalizer_Scope_3346 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_363 : Internal_Entity_Name;
Bare_Item_363 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_673 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_673;
            procedure Finalizer_Scope_3346 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3346;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12170 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12169 '<LogicAll at ast.py:8904>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8904
--# expr-start 12168 '<Map at ast.py:8904>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8904
   --# expr-start 12166 '<FieldAccess for interfaces at ast.py:8904>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8904
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Interface_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12166
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_363 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_363 := Untyped_Item_363; 
Is_Null := Bare_Item_363 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_363); 
   If_Result := As_Entity;
end if;
                  Item_363 := If_Result; 
   --# scope-start
               --# bind ifc Item_363
      --# expr-start 12167 '<FieldAccess for xref_equation at ast.py:8904>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8904
      if Item_363.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_363.Node, Env => Env, Origin => Origin, E_Info => Item_363.Info);
--# end
--# expr-done 12167
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3346;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12168
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8904") else null)); 
--# expr-done 12169
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 12170
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_673;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_673;
                     Finalizer_Scope_3346;
            raise;
      end;
   return Property_Result;
end Interface_Type_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Mod_Int_Type_Def
   --
      procedure Initialize_Fields_For_Mod_Int_Type_Def
        (Self : Bare_Mod_Int_Type_Def
         ; Mod_Int_Type_Def_F_Expr : Bare_Expr
        ) is
      begin
            Self.Mod_Int_Type_Def_F_Expr := Mod_Int_Type_Def_F_Expr;
      end Initialize_Fields_For_Mod_Int_Type_Def;
   function Mod_Int_Type_Def_F_Expr
     (Node : Bare_Mod_Int_Type_Def) return Bare_Expr
   is
   begin
         return Node.Mod_Int_Type_Def_F_Expr;
   end;
--# property-start ModIntTypeDef.is_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8634
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Is_Int_Type
  (Node : Bare_Mod_Int_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Mod_Int_Type_Def_P_Is_Int_Type;
--# end
--# property-start ModIntTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8636
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Xref_Equation
  (Node : Bare_Mod_Int_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
     Ent : Internal_Entity_Mod_Int_Type_Def :=
       Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_645 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_645 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_645;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12172 '<FieldAccess for sub_equation at ast.py:8636>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8636
--# expr-start 12171 '<FieldAccess for expr at ast.py:8636>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8636
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 12171
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12172
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_645;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_645;
            raise;
      end;
   return Property_Result;
end Mod_Int_Type_Def_P_Xref_Equation;
--# end
--# property-start ModIntTypeDef.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8638
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Is_Static
  (Node : Bare_Mod_Int_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
     Ent : Internal_Entity_Mod_Int_Type_Def :=
       Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12174 '<FieldAccess for is_static_expr at ast.py:8638>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8638
--# expr-start 12173 '<FieldAccess for expr at ast.py:8638>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8638
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 12173
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_1 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 12174
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Mod_Int_Type_Def_P_Is_Static;
--# end
--# property-start ModIntTypeDef.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8640
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Discrete_Range
  (Node : Bare_Mod_Int_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
     Ent : Internal_Entity_Mod_Int_Type_Def :=
       Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Expr;
New_Struct : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12177 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12176 '<New DiscreteRange at ast.py:8642>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8642
--# expr-start 12175 '<FieldAccess for expr at ast.py:8643>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8643
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 12175
New_Struct := (High_Bound => Fld, Low_Bound => No_Entity_Expr); 
--# expr-done 12176
Let_Result := New_Struct; 
--# end
--# expr-done 12177
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Mod_Int_Type_Def_P_Discrete_Range;
--# end
--# property-start ModIntTypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8645
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Predefined_Operators
  (Node : Bare_Mod_Int_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_648 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Block_Var_2 : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_14 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_38 : Bare_Type_Decl;
Cast_Result_39 : Bare_Base_Type_Decl;
Cast_Expr_39 : Bare_Type_Decl;
Cast_Result_40 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Sym_16 : Symbol_Type;
Cast_Expr_40 : Bare_Type_Decl;
Cast_Result_41 : Bare_Base_Type_Decl;
Cast_Expr_41 : Bare_Type_Decl;
Cast_Result_42 : Bare_Base_Type_Decl;
Fld_21 : Internal_Env_Assoc := No_Env_Assoc;
Sym_17 : Symbol_Type;
Cast_Expr_42 : Bare_Type_Decl;
Cast_Result_43 : Bare_Base_Type_Decl;
Cast_Expr_43 : Bare_Type_Decl;
Cast_Result_44 : Bare_Base_Type_Decl;
Fld_22 : Internal_Env_Assoc := No_Env_Assoc;
Sym_18 : Symbol_Type;
Cast_Expr_44 : Bare_Type_Decl;
Cast_Result_45 : Bare_Base_Type_Decl;
Cast_Expr_45 : Bare_Type_Decl;
Cast_Result_46 : Bare_Base_Type_Decl;
Fld_23 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_648 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_648;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Mod_Int_Type_Def_Mod_Int_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Mod_Int_Type_Def,
                                As_Bare_Mod_Int_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12223 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12179 '<Cast to TypeDecl at ast.py:8647>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8647
--# expr-start 12178 '<parent at ast.py:8647>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8647
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12178
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12179
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12181 '<FieldAccess for node at ast.py:8648>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8648
--# expr-start 12180 '<FieldAccess for bool_type at ast.py:8648>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8648
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12180
Fld_2 := Fld_1.Node;
--# expr-done 12181
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 12183 '<FieldAccess for node at ast.py:8649>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8649
--# expr-start 12182 '<FieldAccess for int_type at ast.py:8649>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8649
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 12182
Fld_4 := Fld_3.Node;
--# expr-done 12183
Block_Var_2 := Fld_4; 
--# bind int_type Block_Var_2
--# expr-start 12222 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 12185 '<FieldAccess for create_binop_assoc at ast.py:8652>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8652
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12184 '<Symbol "+" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12184
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 12185
--# expr-start 12187 '<FieldAccess for create_binop_assoc at ast.py:8653>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8653
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12186 '<Symbol "-" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12186
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Block_Var; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Block_Var; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 12187
--# expr-start 12189 '<FieldAccess for create_binop_assoc at ast.py:8654>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8654
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12188 '<Symbol "*" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12188
Cast_Expr_6 := Block_Var; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Block_Var; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Block_Var; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 12189
--# expr-start 12191 '<FieldAccess for create_binop_assoc at ast.py:8655>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8655
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12190 '<Symbol "/" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12190
Cast_Expr_9 := Block_Var; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Block_Var; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Block_Var; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 12191
--# expr-start 12193 '<FieldAccess for create_binop_assoc at ast.py:8656>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8656
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12192 '<Symbol "mod" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod); 
--# expr-done 12192
Cast_Expr_12 := Block_Var; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Block_Var; 
      Cast_Result_14 := Cast_Expr_13;
Cast_Expr_14 := Block_Var; 
      Cast_Result_15 := Cast_Expr_14;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_13, Rhs => Cast_Result_14, Ret => Cast_Result_15);
--# end
--# expr-done 12193
--# expr-start 12195 '<FieldAccess for create_binop_assoc at ast.py:8657>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8657
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12194 '<Symbol "rem" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem); 
--# expr-done 12194
Cast_Expr_15 := Block_Var; 
      Cast_Result_16 := Cast_Expr_15;
Cast_Expr_16 := Block_Var; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Block_Var; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_16, Rhs => Cast_Result_17, Ret => Cast_Result_18);
--# end
--# expr-done 12195
--# expr-start 12197 '<FieldAccess for create_binop_assoc at ast.py:8658>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8658
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12196 '<Symbol "and" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 12196
Cast_Expr_18 := Block_Var; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Block_Var; 
      Cast_Result_20 := Cast_Expr_19;
Cast_Expr_20 := Block_Var; 
      Cast_Result_21 := Cast_Expr_20;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_19, Rhs => Cast_Result_20, Ret => Cast_Result_21);
--# end
--# expr-done 12197
--# expr-start 12199 '<FieldAccess for create_binop_assoc at ast.py:8659>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8659
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12198 '<Symbol "or" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 12198
Cast_Expr_21 := Block_Var; 
      Cast_Result_22 := Cast_Expr_21;
Cast_Expr_22 := Block_Var; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Block_Var; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_22, Rhs => Cast_Result_23, Ret => Cast_Result_24);
--# end
--# expr-done 12199
--# expr-start 12201 '<FieldAccess for create_binop_assoc at ast.py:8661>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8661
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12200 '<Symbol "**" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 12200
Cast_Expr_24 := Block_Var; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Block_Var; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_25, Rhs => Block_Var_2, Ret => Cast_Result_26);
--# end
--# expr-done 12201
--# expr-start 12203 '<FieldAccess for create_binop_assoc at ast.py:8663>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8663
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12202 '<Symbol "<" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12202
Cast_Expr_26 := Block_Var; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Block_Var; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_27, Rhs => Cast_Result_28, Ret => Block_Var_1);
--# end
--# expr-done 12203
--# expr-start 12205 '<FieldAccess for create_binop_assoc at ast.py:8664>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8664
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12204 '<Symbol "<=" at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12204
Cast_Expr_28 := Block_Var; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Block_Var; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Block_Var_1);
--# end
--# expr-done 12205
--# expr-start 12207 '<FieldAccess for create_binop_assoc at ast.py:8665>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8665
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12206 '<Symbol "=" at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12206
Cast_Expr_30 := Block_Var; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Block_Var; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_11, Lhs => Cast_Result_31, Rhs => Cast_Result_32, Ret => Block_Var_1);
--# end
--# expr-done 12207
--# expr-start 12209 '<FieldAccess for create_binop_assoc at ast.py:8666>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8666
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12208 '<Symbol "/=" at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12208
Cast_Expr_32 := Block_Var; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Block_Var; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_12, Lhs => Cast_Result_33, Rhs => Cast_Result_34, Ret => Block_Var_1);
--# end
--# expr-done 12209
--# expr-start 12211 '<FieldAccess for create_binop_assoc at ast.py:8667>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8667
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12210 '<Symbol ">" at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12210
Cast_Expr_34 := Block_Var; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Block_Var; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_binop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_13, Lhs => Cast_Result_35, Rhs => Cast_Result_36, Ret => Block_Var_1);
--# end
--# expr-done 12211
--# expr-start 12213 '<FieldAccess for create_binop_assoc at ast.py:8668>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8668
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12212 '<Symbol ">=" at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12212
Cast_Expr_36 := Block_Var; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Block_Var; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_binop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_14, Lhs => Cast_Result_37, Rhs => Cast_Result_38, Ret => Block_Var_1);
--# end
--# expr-done 12213
--# expr-start 12215 '<FieldAccess for create_unop_assoc at ast.py:8670>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8670
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12214 '<Symbol "+" at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12214
Cast_Expr_38 := Block_Var; 
      Cast_Result_39 := Cast_Expr_38;
Cast_Expr_39 := Block_Var; 
      Cast_Result_40 := Cast_Expr_39;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_15, Rhs => Cast_Result_39, Ret => Cast_Result_40);
--# end
--# expr-done 12215
--# expr-start 12217 '<FieldAccess for create_unop_assoc at ast.py:8671>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8671
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12216 '<Symbol "-" at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12216
Cast_Expr_40 := Block_Var; 
      Cast_Result_41 := Cast_Expr_40;
Cast_Expr_41 := Block_Var; 
      Cast_Result_42 := Cast_Expr_41;
--# property-call-start AdaNode.create_unop_assoc
Fld_21 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_16, Rhs => Cast_Result_41, Ret => Cast_Result_42);
--# end
--# expr-done 12217
--# expr-start 12219 '<FieldAccess for create_unop_assoc at ast.py:8672>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8672
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12218 '<Symbol "abs" at ???>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 12218
Cast_Expr_42 := Block_Var; 
      Cast_Result_43 := Cast_Expr_42;
Cast_Expr_43 := Block_Var; 
      Cast_Result_44 := Cast_Expr_43;
--# property-call-start AdaNode.create_unop_assoc
Fld_22 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_17, Rhs => Cast_Result_43, Ret => Cast_Result_44);
--# end
--# expr-done 12219
--# expr-start 12221 '<FieldAccess for create_unop_assoc at ast.py:8673>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8673
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12220 '<Symbol "not" at ???>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 12220
Cast_Expr_44 := Block_Var; 
      Cast_Result_45 := Cast_Expr_44;
Cast_Expr_45 := Block_Var; 
      Cast_Result_46 := Cast_Expr_45;
--# property-call-start AdaNode.create_unop_assoc
Fld_23 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_18, Rhs => Cast_Result_45, Ret => Cast_Result_46);
--# end
--# expr-done 12221
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18, 15 => Fld_19, 16 => Fld_20, 17 => Fld_21, 18 => Fld_22, 19 => Fld_23)); 
--# expr-done 12222
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12223
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_648;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_648;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Mod_Int_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Private_Type_Def
   --
      procedure Initialize_Fields_For_Private_Type_Def
        (Self : Bare_Private_Type_Def
         ; Private_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Private_Type_Def_F_Has_Tagged : Bare_Tagged_Node
         ; Private_Type_Def_F_Has_Limited : Bare_Limited_Node
        ) is
      begin
            Self.Private_Type_Def_F_Has_Abstract := Private_Type_Def_F_Has_Abstract;
            Self.Private_Type_Def_F_Has_Tagged := Private_Type_Def_F_Has_Tagged;
            Self.Private_Type_Def_F_Has_Limited := Private_Type_Def_F_Has_Limited;
      end Initialize_Fields_For_Private_Type_Def;
   function Private_Type_Def_F_Has_Abstract
     (Node : Bare_Private_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Abstract;
   end;
   function Private_Type_Def_F_Has_Tagged
     (Node : Bare_Private_Type_Def) return Bare_Tagged_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Tagged;
   end;
   function Private_Type_Def_F_Has_Limited
     (Node : Bare_Private_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Limited;
   end;
--# property-start PrivateTypeDef.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8570
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Private_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Private_Type_Def  := Bare_Private_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12225 '<FieldAccess for as_bool at ast.py:8570>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8570
--# expr-start 12224 '<FieldAccess for has_tagged at ast.py:8570>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8570
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Private_Type_Def_F_Has_Tagged;
--# expr-done 12224
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Tagged.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Tagged_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 12225
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start PrivateTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8572
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Xref_Equation
  (Node : Bare_Private_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Private_Type_Def  := Bare_Private_Type_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_637 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_637 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_637;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8572") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_637;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_637;
            raise;
      end;
   return Property_Result;
end Private_Type_Def_P_Xref_Equation;
--# end
--# property-start PrivateTypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8574
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Predefined_Operators
  (Node : Bare_Private_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Private_Type_Def  := Bare_Private_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_638 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
Fld_1 : Bare_Limited_Node;
Fld_2 : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_638 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_638;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Private_Type_Def_Private_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Private_Type_Def,
                                As_Bare_Private_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12230 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12229 '<If at ast.py:8576>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8576
--# expr-start 12227 '<FieldAccess for as_bool at ast.py:8577>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8577
--# expr-start 12226 '<FieldAccess for has_limited at ast.py:8577>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8577
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Private_Type_Def_F_Has_Limited;
--# expr-done 12226
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Limited.as_bool'
Fld_2 := Libadalang.Implementation.Dispatcher_Limited_Node_P_As_Bool (Node => Fld_1);
--# end
--# expr-done 12227
if Fld_2 then
   If_Result := No_Internal_Env_Assoc_Array_Type;
else
   --# expr-start 12228 '<FieldAccess for predefined_equality_operators at ast.py:8579>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8579
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.predefined_equality_operators
Fld := Libadalang.Implementation.Type_Def_P_Predefined_Equality_Operators (Node => Self);
--# end
--# expr-done 12228
   If_Result := Fld;
end if;
      Inc_Ref (If_Result);
--# expr-done 12229
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12230
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_638;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_638;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Private_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Real_Type_Def
   --
--# property-start RealTypeDef.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5391
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Is_Static
  (Node : Bare_Real_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Real_Type_Def  := Bare_Real_Type_Def (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Real_Type_Def_P_Is_Static;
--# end
--# property-start RealTypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5393
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Predefined_Operators
  (Node : Bare_Real_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Real_Type_Def  := Bare_Real_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_375 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Block_Var_2 : Bare_Base_Type_Decl;
Block_Var_3 : Bare_Base_Type_Decl;
Block_Var_4 : Internal_Env_Assoc_Array_Access;
Block_Var_5 : Internal_Env_Assoc_Array_Access;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Sym_14 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_21 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_22 : Internal_Env_Assoc := No_Env_Assoc;
Sym_16 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_23 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit_1 : Internal_Env_Assoc_Array_Access;
Fld_24 : Internal_Entity_Base_Type_Decl;
Fld_25 : Bare_Base_Type_Decl;
Cast_Expr_38 : Bare_Type_Decl;
Cast_Result_39 : Bare_Base_Type_Decl;
Is_Equal : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Concat_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_375 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_375;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Real_Type_Def_Real_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Real_Type_Def,
                                As_Bare_Real_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12280 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12232 '<Cast to TypeDecl at ast.py:5395>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5395
--# expr-start 12231 '<parent at ast.py:5395>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5395
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12231
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12232
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12234 '<FieldAccess for node at ast.py:5396>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5396
--# expr-start 12233 '<FieldAccess for bool_type at ast.py:5396>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5396
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12233
Fld_2 := Fld_1.Node;
--# expr-done 12234
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 12236 '<FieldAccess for node at ast.py:5397>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5397
--# expr-start 12235 '<FieldAccess for int_type at ast.py:5397>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5397
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 12235
Fld_4 := Fld_3.Node;
--# expr-done 12236
Block_Var_2 := Fld_4; 
--# bind int_type Block_Var_2
--# expr-start 12238 '<FieldAccess for node at ast.py:5398>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5398
--# expr-start 12237 '<FieldAccess for root_int_type at ast.py:5398>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5398
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_int_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Root_Int_Type (Node => Self);
--# end
--# expr-done 12237
Fld_6 := Fld_5.Node;
--# expr-done 12238
Block_Var_3 := Fld_6; 
--# bind root_int_type Block_Var_3
--# expr-start 12267 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 12240 '<FieldAccess for create_binop_assoc at ast.py:5401>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5401
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12239 '<Symbol "+" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12239
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 12240
--# expr-start 12242 '<FieldAccess for create_binop_assoc at ast.py:5402>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5402
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12241 '<Symbol "-" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12241
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Block_Var; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Block_Var; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 12242
--# expr-start 12244 '<FieldAccess for create_binop_assoc at ast.py:5403>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5403
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12243 '<Symbol "*" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12243
Cast_Expr_6 := Block_Var; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Block_Var; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Block_Var; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 12244
--# expr-start 12246 '<FieldAccess for create_binop_assoc at ast.py:5404>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5404
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12245 '<Symbol "/" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12245
Cast_Expr_9 := Block_Var; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Block_Var; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Block_Var; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 12246
--# expr-start 12248 '<FieldAccess for create_binop_assoc at ast.py:5406>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5406
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12247 '<Symbol "**" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 12247
Cast_Expr_12 := Block_Var; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Block_Var; 
      Cast_Result_14 := Cast_Expr_13;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_13, Rhs => Block_Var_2, Ret => Cast_Result_14);
--# end
--# expr-done 12248
--# expr-start 12250 '<FieldAccess for create_binop_assoc at ast.py:5408>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5408
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12249 '<Symbol "<" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12249
Cast_Expr_14 := Block_Var; 
      Cast_Result_15 := Cast_Expr_14;
Cast_Expr_15 := Block_Var; 
      Cast_Result_16 := Cast_Expr_15;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_15, Rhs => Cast_Result_16, Ret => Block_Var_1);
--# end
--# expr-done 12250
--# expr-start 12252 '<FieldAccess for create_binop_assoc at ast.py:5409>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5409
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12251 '<Symbol "<=" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12251
Cast_Expr_16 := Block_Var; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Block_Var; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_17, Rhs => Cast_Result_18, Ret => Block_Var_1);
--# end
--# expr-done 12252
--# expr-start 12254 '<FieldAccess for create_binop_assoc at ast.py:5410>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5410
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12253 '<Symbol "=" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12253
Cast_Expr_18 := Block_Var; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Block_Var; 
      Cast_Result_20 := Cast_Expr_19;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_19, Rhs => Cast_Result_20, Ret => Block_Var_1);
--# end
--# expr-done 12254
--# expr-start 12256 '<FieldAccess for create_binop_assoc at ast.py:5411>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5411
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12255 '<Symbol "/=" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12255
Cast_Expr_20 := Block_Var; 
      Cast_Result_21 := Cast_Expr_20;
Cast_Expr_21 := Block_Var; 
      Cast_Result_22 := Cast_Expr_21;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_21, Rhs => Cast_Result_22, Ret => Block_Var_1);
--# end
--# expr-done 12256
--# expr-start 12258 '<FieldAccess for create_binop_assoc at ast.py:5412>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5412
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12257 '<Symbol ">" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12257
Cast_Expr_22 := Block_Var; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Block_Var; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_23, Rhs => Cast_Result_24, Ret => Block_Var_1);
--# end
--# expr-done 12258
--# expr-start 12260 '<FieldAccess for create_binop_assoc at ast.py:5413>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5413
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12259 '<Symbol ">=" at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12259
Cast_Expr_24 := Block_Var; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Block_Var; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_25, Rhs => Cast_Result_26, Ret => Block_Var_1);
--# end
--# expr-done 12260
--# expr-start 12262 '<FieldAccess for create_unop_assoc at ast.py:5415>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5415
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12261 '<Symbol "+" at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12261
Cast_Expr_26 := Block_Var; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Block_Var; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_unop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_11, Rhs => Cast_Result_27, Ret => Cast_Result_28);
--# end
--# expr-done 12262
--# expr-start 12264 '<FieldAccess for create_unop_assoc at ast.py:5416>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5416
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12263 '<Symbol "-" at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12263
Cast_Expr_28 := Block_Var; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Block_Var; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_unop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_12, Rhs => Cast_Result_29, Ret => Cast_Result_30);
--# end
--# expr-done 12264
--# expr-start 12266 '<FieldAccess for create_unop_assoc at ast.py:5417>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5417
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12265 '<Symbol "abs" at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 12265
Cast_Expr_30 := Block_Var; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Block_Var; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_13, Rhs => Cast_Result_31, Ret => Cast_Result_32);
--# end
--# expr-done 12266
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_7, 2 => Fld_8, 3 => Fld_9, 4 => Fld_10, 5 => Fld_11, 6 => Fld_12, 7 => Fld_13, 8 => Fld_14, 9 => Fld_15, 10 => Fld_16, 11 => Fld_17, 12 => Fld_18, 13 => Fld_19, 14 => Fld_20)); 
--# expr-done 12267
Block_Var_4 := Array_Lit; Inc_Ref (Block_Var_4);
--# bind defaults Block_Var_4
--# expr-start 12278 '<If at ast.py:5421>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5421
--# expr-start 12270 '<Eq at ast.py:5422>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5422
Cast_Expr_38 := Block_Var; 
      Cast_Result_39 := Cast_Expr_38;
--# expr-start 12269 '<FieldAccess for node at ast.py:5422>' Fld_25 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5422
--# expr-start 12268 '<FieldAccess for root_real_type at ast.py:5422>' Fld_24 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5422
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_real_type
Fld_24 := Libadalang.Implementation.Ada_Node_P_Root_Real_Type (Node => Self);
--# end
--# expr-done 12268
Fld_25 := Fld_24.Node;
--# expr-done 12269
Is_Equal := Cast_Result_39 = Fld_25; 
--# expr-done 12270
if Is_Equal then
   --# expr-start 12277 '<ArrayLiteral at ???>' Array_Lit_1 None
--# expr-start 12272 '<FieldAccess for create_binop_assoc at ast.py:5424>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5424
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12271 '<Symbol "*" at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12271
Cast_Expr_32 := Block_Var; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Block_Var; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_binop_assoc
Fld_21 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_14, Lhs => Block_Var_3, Rhs => Cast_Result_33, Ret => Cast_Result_34);
--# end
--# expr-done 12272
--# expr-start 12274 '<FieldAccess for create_binop_assoc at ast.py:5427>' Fld_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5427
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12273 '<Symbol "*" at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12273
Cast_Expr_34 := Block_Var; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Block_Var; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_binop_assoc
Fld_22 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_15, Lhs => Cast_Result_35, Rhs => Block_Var_3, Ret => Cast_Result_36);
--# end
--# expr-done 12274
--# expr-start 12276 '<FieldAccess for create_binop_assoc at ast.py:5430>' Fld_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5430
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12275 '<Symbol "/" at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12275
Cast_Expr_36 := Block_Var; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Block_Var; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_binop_assoc
Fld_23 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_16, Lhs => Cast_Result_37, Rhs => Block_Var_3, Ret => Cast_Result_38);
--# end
--# expr-done 12276
Array_Lit_1 := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_21, 2 => Fld_22, 3 => Fld_23)); 
--# expr-done 12277
   If_Result := Array_Lit_1;
else
   If_Result := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 12278
Block_Var_5 := If_Result; Inc_Ref (Block_Var_5);
--# bind specials Block_Var_5
--# expr-start 12279 '<Concat at ast.py:5437>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5437
Concat_Result := Concat (Block_Var_4, Block_Var_5); 
--# expr-done 12279
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12280
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_375;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_375;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Real_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Decimal_Fixed_Point_Def
   --
      procedure Initialize_Fields_For_Decimal_Fixed_Point_Def
        (Self : Bare_Decimal_Fixed_Point_Def
         ; Decimal_Fixed_Point_Def_F_Delta : Bare_Expr
         ; Decimal_Fixed_Point_Def_F_Digits : Bare_Expr
         ; Decimal_Fixed_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Decimal_Fixed_Point_Def_F_Delta := Decimal_Fixed_Point_Def_F_Delta;
            Self.Decimal_Fixed_Point_Def_F_Digits := Decimal_Fixed_Point_Def_F_Digits;
            Self.Decimal_Fixed_Point_Def_F_Range := Decimal_Fixed_Point_Def_F_Range;
      end Initialize_Fields_For_Decimal_Fixed_Point_Def;
   function Decimal_Fixed_Point_Def_F_Delta
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Delta;
   end;
   function Decimal_Fixed_Point_Def_F_Digits
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Digits;
   end;
   function Decimal_Fixed_Point_Def_F_Range
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Range;
   end;
--# property-start DecimalFixedPointDef.is_fixed_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8227
pragma Warnings (Off, "is not referenced");
 function Decimal_Fixed_Point_Def_P_Is_Fixed_Point
  (Node : Bare_Decimal_Fixed_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Decimal_Fixed_Point_Def  := Bare_Decimal_Fixed_Point_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Decimal_Fixed_Point_Def_P_Is_Fixed_Point;
--# end
--# property-start DecimalFixedPointDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8229
pragma Warnings (Off, "is not referenced");
 function Decimal_Fixed_Point_Def_P_Xref_Equation
  (Node : Bare_Decimal_Fixed_Point_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Decimal_Fixed_Point_Def  := Bare_Decimal_Fixed_Point_Def (Node);
     Ent : Internal_Entity_Decimal_Fixed_Point_Def :=
       Internal_Entity_Decimal_Fixed_Point_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_605 with Inline_Always;
            procedure Finalizer_Scope_3347 with Inline_Always;
      Var_Expr : Internal_Entity_Range_Spec;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_14 : Internal_Entity_Range_Spec;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Var;
Fld_17 : Logic_Equation;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_20 : Internal_Entity_Expr;
Fld_21 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_7 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_605 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_605;
            procedure Finalizer_Scope_3347 is
            begin
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_6);
            end Finalizer_Scope_3347;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12311 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12310 '<And at ast.py:8234>' And_Pred_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8234
--# expr-start 12299 '<And at ast.py:8234>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8234
--# expr-start 12296 '<And at ast.py:8234>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8234
--# expr-start 12293 '<And at ast.py:8234>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8234
--# expr-start 12289 '<And at ast.py:8234>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8234
--# expr-start 12286 '<And at ast.py:8234>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8234
--# expr-start 12283 '<FieldAccess for universal_real_bind at ast.py:8237>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8237
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12282 '<FieldAccess for expected_type_var at ast.py:8237>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8237
--# expr-start 12281 '<FieldAccess for delta at ast.py:8237>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8237
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12281
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 12282
--# property-call-start AdaNode.universal_real_bind
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_1, Origin => Origin);
--# end
--# expr-done 12283
--# expr-start 12285 '<FieldAccess for sub_equation at ast.py:8238>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8238
--# expr-start 12284 '<FieldAccess for delta at ast.py:8238>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8238
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12284
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 12285
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8234") else null)); 
--# expr-done 12286
--# expr-start 12288 '<FieldAccess for matches_expected_type at ast.py:8239>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8239
--# expr-start 12287 '<FieldAccess for delta at ast.py:8239>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8239
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12287
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 12288
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8234") else null)); 
--# expr-done 12289
--# expr-start 12292 '<FieldAccess for universal_int_bind at ast.py:8242>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8242
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12291 '<FieldAccess for expected_type_var at ast.py:8242>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8242
--# expr-start 12290 '<FieldAccess for digits at ast.py:8242>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8242
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Digits, Info => Ent.Info);
--# expr-done 12290
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_7.Node);
--# expr-done 12291
--# property-call-start AdaNode.universal_int_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Ent.Node, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 12292
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8234") else null)); 
--# expr-done 12293
--# expr-start 12295 '<FieldAccess for sub_equation at ast.py:8243>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8243
--# expr-start 12294 '<FieldAccess for digits at ast.py:8243>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8243
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Digits, Info => Ent.Info);
--# expr-done 12294
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 12295
And_Pred_3 := Create_And (And_Pred_2, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8234") else null)); 
--# expr-done 12296
--# expr-start 12298 '<FieldAccess for matches_expected_type at ast.py:8244>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8244
--# expr-start 12297 '<FieldAccess for digits at ast.py:8244>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8244
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Digits, Info => Ent.Info);
--# expr-done 12297
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_12.Node, Origin => Origin);
--# end
--# expr-done 12298
And_Pred_4 := Create_And (And_Pred_3, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8234") else null)); 
--# expr-done 12299
--# expr-start 12300 '<FieldAccess for range at ast.py:8248>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8248
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Range, Info => Ent.Info);
--# expr-done 12300
Var_Expr := Fld_14;
if Var_Expr /= No_Entity_Range_Spec then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 12309 '<And at ast.py:8250>' And_Pred_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8250
--# expr-start 12306 '<And at ast.py:8250>' And_Pred_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8250
--# expr-start 12303 '<FieldAccess for universal_real_bind at ast.py:8250>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8250
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12302 '<FieldAccess for expected_type_var at ast.py:8250>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8250
--# expr-start 12301 '<FieldAccess for range at ast.py:8250>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8250
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12301
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_15.Node);
--# expr-done 12302
--# property-call-start AdaNode.universal_real_bind
Fld_17 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_16, Origin => Origin);
--# end
--# expr-done 12303
--# expr-start 12305 '<FieldAccess for sub_equation at ast.py:8251>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8251
--# expr-start 12304 '<FieldAccess for range at ast.py:8251>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8251
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12304
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_19 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_18.Node, Env => Env, Origin => Origin, E_Info => Fld_18.Info);
--# end
--# expr-done 12305
And_Pred_5 := Create_And (Fld_17, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8250") else null)); 
--# expr-done 12306
--# expr-start 12308 '<FieldAccess for matches_expected_type at ast.py:8252>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8252
--# expr-start 12307 '<FieldAccess for range at ast.py:8252>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8252
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12307
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_21 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_20.Node, Origin => Origin);
--# end
--# expr-done 12308
And_Pred_6 := Create_And (And_Pred_5, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8250") else null)); 
--# expr-done 12309
Scope_Result := And_Pred_6;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3347;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8253") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_7 := Create_And (And_Pred_4, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8234") else null)); 
--# expr-done 12310
Let_Result := And_Pred_7; Inc_Ref (Let_Result);
--# end
--# expr-done 12311
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_605;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_605;
                     Finalizer_Scope_3347;
            raise;
      end;
   return Property_Result;
end Decimal_Fixed_Point_Def_P_Xref_Equation;
--# end
--# property-start DecimalFixedPointDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8257
pragma Warnings (Off, "is not referenced");
 function Decimal_Fixed_Point_Def_P_Predefined_Operators
  (Node : Bare_Decimal_Fixed_Point_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Decimal_Fixed_Point_Def  := Bare_Decimal_Fixed_Point_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_606 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Block_Var_2 : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_14 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Sym_16 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_21 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_606 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_606;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Decimal_Fixed_Point_Def_Decimal_Fixed_Point_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Decimal_Fixed_Point_Def,
                                As_Bare_Decimal_Fixed_Point_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12353 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12313 '<Cast to TypeDecl at ast.py:8259>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8259
--# expr-start 12312 '<parent at ast.py:8259>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8259
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12312
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12313
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12315 '<FieldAccess for node at ast.py:8260>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8260
--# expr-start 12314 '<FieldAccess for bool_type at ast.py:8260>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8260
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12314
Fld_2 := Fld_1.Node;
--# expr-done 12315
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 12317 '<FieldAccess for node at ast.py:8261>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8261
--# expr-start 12316 '<FieldAccess for int_type at ast.py:8261>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8261
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 12316
Fld_4 := Fld_3.Node;
--# expr-done 12317
Block_Var_2 := Fld_4; 
--# bind int_type Block_Var_2
--# expr-start 12352 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 12319 '<FieldAccess for create_binop_assoc at ast.py:8264>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8264
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12318 '<Symbol "+" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12318
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 12319
--# expr-start 12321 '<FieldAccess for create_binop_assoc at ast.py:8265>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8265
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12320 '<Symbol "-" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12320
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Block_Var; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Block_Var; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 12321
--# expr-start 12323 '<FieldAccess for create_binop_assoc at ast.py:8266>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8266
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12322 '<Symbol "*" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12322
Cast_Expr_6 := Block_Var; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Block_Var; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Block_Var; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 12323
--# expr-start 12325 '<FieldAccess for create_binop_assoc at ast.py:8267>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8267
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12324 '<Symbol "/" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12324
Cast_Expr_9 := Block_Var; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Block_Var; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Block_Var; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 12325
--# expr-start 12327 '<FieldAccess for create_binop_assoc at ast.py:8269>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8269
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12326 '<Symbol "*" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12326
Cast_Expr_12 := Block_Var; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Block_Var; 
      Cast_Result_14 := Cast_Expr_13;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Block_Var_2, Rhs => Cast_Result_13, Ret => Cast_Result_14);
--# end
--# expr-done 12327
--# expr-start 12329 '<FieldAccess for create_binop_assoc at ast.py:8270>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8270
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12328 '<Symbol "*" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12328
Cast_Expr_14 := Block_Var; 
      Cast_Result_15 := Cast_Expr_14;
Cast_Expr_15 := Block_Var; 
      Cast_Result_16 := Cast_Expr_15;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_15, Rhs => Block_Var_2, Ret => Cast_Result_16);
--# end
--# expr-done 12329
--# expr-start 12331 '<FieldAccess for create_binop_assoc at ast.py:8271>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8271
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12330 '<Symbol "/" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12330
Cast_Expr_16 := Block_Var; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Block_Var; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_17, Rhs => Block_Var_2, Ret => Cast_Result_18);
--# end
--# expr-done 12331
--# expr-start 12333 '<FieldAccess for create_binop_assoc at ast.py:8272>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8272
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12332 '<Symbol "**" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 12332
Cast_Expr_18 := Block_Var; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Block_Var; 
      Cast_Result_20 := Cast_Expr_19;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_19, Rhs => Block_Var_2, Ret => Cast_Result_20);
--# end
--# expr-done 12333
--# expr-start 12335 '<FieldAccess for create_binop_assoc at ast.py:8274>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8274
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12334 '<Symbol "<" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12334
Cast_Expr_20 := Block_Var; 
      Cast_Result_21 := Cast_Expr_20;
Cast_Expr_21 := Block_Var; 
      Cast_Result_22 := Cast_Expr_21;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_21, Rhs => Cast_Result_22, Ret => Block_Var_1);
--# end
--# expr-done 12335
--# expr-start 12337 '<FieldAccess for create_binop_assoc at ast.py:8275>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8275
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12336 '<Symbol "<=" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12336
Cast_Expr_22 := Block_Var; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Block_Var; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_23, Rhs => Cast_Result_24, Ret => Block_Var_1);
--# end
--# expr-done 12337
--# expr-start 12339 '<FieldAccess for create_binop_assoc at ast.py:8276>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8276
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12338 '<Symbol "=" at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12338
Cast_Expr_24 := Block_Var; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Block_Var; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_25, Rhs => Cast_Result_26, Ret => Block_Var_1);
--# end
--# expr-done 12339
--# expr-start 12341 '<FieldAccess for create_binop_assoc at ast.py:8277>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8277
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12340 '<Symbol "/=" at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12340
Cast_Expr_26 := Block_Var; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Block_Var; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_11, Lhs => Cast_Result_27, Rhs => Cast_Result_28, Ret => Block_Var_1);
--# end
--# expr-done 12341
--# expr-start 12343 '<FieldAccess for create_binop_assoc at ast.py:8278>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8278
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12342 '<Symbol ">" at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12342
Cast_Expr_28 := Block_Var; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Block_Var; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_12, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Block_Var_1);
--# end
--# expr-done 12343
--# expr-start 12345 '<FieldAccess for create_binop_assoc at ast.py:8279>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8279
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12344 '<Symbol ">=" at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12344
Cast_Expr_30 := Block_Var; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Block_Var; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_binop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_13, Lhs => Cast_Result_31, Rhs => Cast_Result_32, Ret => Block_Var_1);
--# end
--# expr-done 12345
--# expr-start 12347 '<FieldAccess for create_unop_assoc at ast.py:8281>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8281
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12346 '<Symbol "+" at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12346
Cast_Expr_32 := Block_Var; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Block_Var; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_unop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_14, Rhs => Cast_Result_33, Ret => Cast_Result_34);
--# end
--# expr-done 12347
--# expr-start 12349 '<FieldAccess for create_unop_assoc at ast.py:8282>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8282
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12348 '<Symbol "-" at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12348
Cast_Expr_34 := Block_Var; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Block_Var; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_15, Rhs => Cast_Result_35, Ret => Cast_Result_36);
--# end
--# expr-done 12349
--# expr-start 12351 '<FieldAccess for create_unop_assoc at ast.py:8283>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8283
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12350 '<Symbol "abs" at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 12350
Cast_Expr_36 := Block_Var; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Block_Var; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_unop_assoc
Fld_21 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_16, Rhs => Cast_Result_37, Ret => Cast_Result_38);
--# end
--# expr-done 12351
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18, 15 => Fld_19, 16 => Fld_20, 17 => Fld_21)); 
--# expr-done 12352
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12353
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_606;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_606;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Decimal_Fixed_Point_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Floating_Point_Def
   --
      procedure Initialize_Fields_For_Floating_Point_Def
        (Self : Bare_Floating_Point_Def
         ; Floating_Point_Def_F_Num_Digits : Bare_Expr
         ; Floating_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Floating_Point_Def_F_Num_Digits := Floating_Point_Def_F_Num_Digits;
            Self.Floating_Point_Def_F_Range := Floating_Point_Def_F_Range;
      end Initialize_Fields_For_Floating_Point_Def;
   function Floating_Point_Def_F_Num_Digits
     (Node : Bare_Floating_Point_Def) return Bare_Expr
   is
   begin
         return Node.Floating_Point_Def_F_Num_Digits;
   end;
   function Floating_Point_Def_F_Range
     (Node : Bare_Floating_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Floating_Point_Def_F_Range;
   end;
--# property-start FloatingPointDef.is_float_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8131
pragma Warnings (Off, "is not referenced");
 function Floating_Point_Def_P_Is_Float_Type
  (Node : Bare_Floating_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Floating_Point_Def  := Bare_Floating_Point_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Floating_Point_Def_P_Is_Float_Type;
--# end
--# property-start FloatingPointDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8133
pragma Warnings (Off, "is not referenced");
 function Floating_Point_Def_P_Xref_Equation
  (Node : Bare_Floating_Point_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Floating_Point_Def  := Bare_Floating_Point_Def (Node);
     Ent : Internal_Entity_Floating_Point_Def :=
       Internal_Entity_Floating_Point_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_600 with Inline_Always;
            procedure Finalizer_Scope_3348 with Inline_Always;
      Var_Expr : Internal_Entity_Range_Spec;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Range_Spec;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_600 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_600;
            procedure Finalizer_Scope_3348 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3348;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12374 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12373 '<And at ast.py:8138>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8138
--# expr-start 12362 '<And at ast.py:8138>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8138
--# expr-start 12359 '<And at ast.py:8138>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8138
--# expr-start 12356 '<FieldAccess for universal_int_bind at ast.py:8141>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8141
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12355 '<FieldAccess for expected_type_var at ast.py:8141>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8141
--# expr-start 12354 '<FieldAccess for num_digits at ast.py:8141>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8141
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Floating_Point_Def_F_Num_Digits, Info => Ent.Info);
--# expr-done 12354
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 12355
--# property-call-start AdaNode.universal_int_bind
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Ent.Node, Type_Var => Fld_1, Origin => Origin);
--# end
--# expr-done 12356
--# expr-start 12358 '<FieldAccess for sub_equation at ast.py:8142>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8142
--# expr-start 12357 '<FieldAccess for num_digits at ast.py:8142>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8142
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Floating_Point_Def_F_Num_Digits, Info => Ent.Info);
--# expr-done 12357
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 12358
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8138") else null)); 
--# expr-done 12359
--# expr-start 12361 '<FieldAccess for matches_expected_type at ast.py:8143>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8143
--# expr-start 12360 '<FieldAccess for num_digits at ast.py:8143>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8143
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Floating_Point_Def_F_Num_Digits, Info => Ent.Info);
--# expr-done 12360
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 12361
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8138") else null)); 
--# expr-done 12362
--# expr-start 12363 '<FieldAccess for range at ast.py:8147>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8147
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Floating_Point_Def_F_Range, Info => Ent.Info);
--# expr-done 12363
Var_Expr := Fld_7;
if Var_Expr /= No_Entity_Range_Spec then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 12372 '<And at ast.py:8149>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8149
--# expr-start 12369 '<And at ast.py:8149>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8149
--# expr-start 12366 '<FieldAccess for universal_real_bind at ast.py:8149>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8149
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12365 '<FieldAccess for expected_type_var at ast.py:8149>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8149
--# expr-start 12364 '<FieldAccess for range at ast.py:8149>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8149
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12364
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_8.Node);
--# expr-done 12365
--# property-call-start AdaNode.universal_real_bind
Fld_10 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_9, Origin => Origin);
--# end
--# expr-done 12366
--# expr-start 12368 '<FieldAccess for sub_equation at ast.py:8150>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8150
--# expr-start 12367 '<FieldAccess for range at ast.py:8150>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8150
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12367
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 12368
And_Pred_2 := Create_And (Fld_10, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8149") else null)); 
--# expr-done 12369
--# expr-start 12371 '<FieldAccess for matches_expected_type at ast.py:8151>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8151
--# expr-start 12370 '<FieldAccess for range at ast.py:8151>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8151
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12370
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_14 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_13.Node, Origin => Origin);
--# end
--# expr-done 12371
And_Pred_3 := Create_And (And_Pred_2, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8149") else null)); 
--# expr-done 12372
Scope_Result := And_Pred_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3348;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8152") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_4 := Create_And (And_Pred_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8138") else null)); 
--# expr-done 12373
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 12374
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_600;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_600;
                     Finalizer_Scope_3348;
            raise;
      end;
   return Property_Result;
end Floating_Point_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Ordinary_Fixed_Point_Def
   --
      procedure Initialize_Fields_For_Ordinary_Fixed_Point_Def
        (Self : Bare_Ordinary_Fixed_Point_Def
         ; Ordinary_Fixed_Point_Def_F_Delta : Bare_Expr
         ; Ordinary_Fixed_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Ordinary_Fixed_Point_Def_F_Delta := Ordinary_Fixed_Point_Def_F_Delta;
            Self.Ordinary_Fixed_Point_Def_F_Range := Ordinary_Fixed_Point_Def_F_Range;
      end Initialize_Fields_For_Ordinary_Fixed_Point_Def;
   function Ordinary_Fixed_Point_Def_F_Delta
     (Node : Bare_Ordinary_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Ordinary_Fixed_Point_Def_F_Delta;
   end;
   function Ordinary_Fixed_Point_Def_F_Range
     (Node : Bare_Ordinary_Fixed_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Ordinary_Fixed_Point_Def_F_Range;
   end;
--# property-start OrdinaryFixedPointDef.is_fixed_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8164
pragma Warnings (Off, "is not referenced");
 function Ordinary_Fixed_Point_Def_P_Is_Fixed_Point
  (Node : Bare_Ordinary_Fixed_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ordinary_Fixed_Point_Def  := Bare_Ordinary_Fixed_Point_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ordinary_Fixed_Point_Def_P_Is_Fixed_Point;
--# end
--# property-start OrdinaryFixedPointDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8166
pragma Warnings (Off, "is not referenced");
 function Ordinary_Fixed_Point_Def_P_Xref_Equation
  (Node : Bare_Ordinary_Fixed_Point_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ordinary_Fixed_Point_Def  := Bare_Ordinary_Fixed_Point_Def (Node);
     Ent : Internal_Entity_Ordinary_Fixed_Point_Def :=
       Internal_Entity_Ordinary_Fixed_Point_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_602 with Inline_Always;
            procedure Finalizer_Scope_3349 with Inline_Always;
      Var_Expr : Internal_Entity_Range_Spec;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Range_Spec;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_602 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_602;
            procedure Finalizer_Scope_3349 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3349;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12395 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12394 '<And at ast.py:8171>' And_Pred_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8171
--# expr-start 12383 '<And at ast.py:8171>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8171
--# expr-start 12380 '<And at ast.py:8171>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8171
--# expr-start 12377 '<FieldAccess for universal_real_bind at ast.py:8174>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8174
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12376 '<FieldAccess for expected_type_var at ast.py:8174>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8174
--# expr-start 12375 '<FieldAccess for delta at ast.py:8174>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8174
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Ordinary_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12375
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 12376
--# property-call-start AdaNode.universal_real_bind
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_1, Origin => Origin);
--# end
--# expr-done 12377
--# expr-start 12379 '<FieldAccess for sub_equation at ast.py:8175>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8175
--# expr-start 12378 '<FieldAccess for delta at ast.py:8175>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8175
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Ordinary_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12378
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 12379
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8171") else null)); 
--# expr-done 12380
--# expr-start 12382 '<FieldAccess for matches_expected_type at ast.py:8176>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8176
--# expr-start 12381 '<FieldAccess for delta at ast.py:8176>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8176
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Ordinary_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12381
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 12382
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8171") else null)); 
--# expr-done 12383
--# expr-start 12384 '<FieldAccess for range at ast.py:8180>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8180
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Ordinary_Fixed_Point_Def_F_Range, Info => Ent.Info);
--# expr-done 12384
Var_Expr := Fld_7;
if Var_Expr /= No_Entity_Range_Spec then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 12393 '<And at ast.py:8182>' And_Pred_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8182
--# expr-start 12390 '<And at ast.py:8182>' And_Pred_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8182
--# expr-start 12387 '<FieldAccess for universal_real_bind at ast.py:8182>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8182
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12386 '<FieldAccess for expected_type_var at ast.py:8182>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8182
--# expr-start 12385 '<FieldAccess for range at ast.py:8182>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8182
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12385
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_8.Node);
--# expr-done 12386
--# property-call-start AdaNode.universal_real_bind
Fld_10 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_9, Origin => Origin);
--# end
--# expr-done 12387
--# expr-start 12389 '<FieldAccess for sub_equation at ast.py:8183>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8183
--# expr-start 12388 '<FieldAccess for range at ast.py:8183>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8183
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12388
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 12389
And_Pred_2 := Create_And (Fld_10, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8182") else null)); 
--# expr-done 12390
--# expr-start 12392 '<FieldAccess for matches_expected_type at ast.py:8184>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8184
--# expr-start 12391 '<FieldAccess for range at ast.py:8184>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8184
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Range_Spec_F_Range, Info => Var_Expr.Info);
--# expr-done 12391
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_14 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_13.Node, Origin => Origin);
--# end
--# expr-done 12392
And_Pred_3 := Create_And (And_Pred_2, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8182") else null)); 
--# expr-done 12393
Scope_Result := And_Pred_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3349;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8185") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_4 := Create_And (And_Pred_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:8171") else null)); 
--# expr-done 12394
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 12395
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_602;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_602;
                     Finalizer_Scope_3349;
            raise;
      end;
   return Property_Result;
end Ordinary_Fixed_Point_Def_P_Xref_Equation;
--# end
--# property-start OrdinaryFixedPointDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8189
pragma Warnings (Off, "is not referenced");
 function Ordinary_Fixed_Point_Def_P_Predefined_Operators
  (Node : Bare_Ordinary_Fixed_Point_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Ordinary_Fixed_Point_Def  := Bare_Ordinary_Fixed_Point_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_603 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Block_Var_2 : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_14 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Sym_16 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_21 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_603 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_603;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ordinary_Fixed_Point_Def_Ordinary_Fixed_Point_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ordinary_Fixed_Point_Def,
                                As_Bare_Ordinary_Fixed_Point_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12437 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12397 '<Cast to TypeDecl at ast.py:8191>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8191
--# expr-start 12396 '<parent at ast.py:8191>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8191
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12396
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12397
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12399 '<FieldAccess for node at ast.py:8192>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8192
--# expr-start 12398 '<FieldAccess for bool_type at ast.py:8192>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8192
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12398
Fld_2 := Fld_1.Node;
--# expr-done 12399
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 12401 '<FieldAccess for node at ast.py:8193>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8193
--# expr-start 12400 '<FieldAccess for int_type at ast.py:8193>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8193
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 12400
Fld_4 := Fld_3.Node;
--# expr-done 12401
Block_Var_2 := Fld_4; 
--# bind int_type Block_Var_2
--# expr-start 12436 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 12403 '<FieldAccess for create_binop_assoc at ast.py:8196>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8196
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12402 '<Symbol "+" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12402
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 12403
--# expr-start 12405 '<FieldAccess for create_binop_assoc at ast.py:8197>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8197
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12404 '<Symbol "-" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12404
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Block_Var; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Block_Var; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 12405
--# expr-start 12407 '<FieldAccess for create_binop_assoc at ast.py:8198>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8198
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12406 '<Symbol "*" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12406
Cast_Expr_6 := Block_Var; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Block_Var; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Block_Var; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 12407
--# expr-start 12409 '<FieldAccess for create_binop_assoc at ast.py:8199>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8199
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12408 '<Symbol "/" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12408
Cast_Expr_9 := Block_Var; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Block_Var; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Block_Var; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 12409
--# expr-start 12411 '<FieldAccess for create_binop_assoc at ast.py:8201>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8201
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12410 '<Symbol "*" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12410
Cast_Expr_12 := Block_Var; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Block_Var; 
      Cast_Result_14 := Cast_Expr_13;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Block_Var_2, Rhs => Cast_Result_13, Ret => Cast_Result_14);
--# end
--# expr-done 12411
--# expr-start 12413 '<FieldAccess for create_binop_assoc at ast.py:8202>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8202
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12412 '<Symbol "*" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12412
Cast_Expr_14 := Block_Var; 
      Cast_Result_15 := Cast_Expr_14;
Cast_Expr_15 := Block_Var; 
      Cast_Result_16 := Cast_Expr_15;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_15, Rhs => Block_Var_2, Ret => Cast_Result_16);
--# end
--# expr-done 12413
--# expr-start 12415 '<FieldAccess for create_binop_assoc at ast.py:8203>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8203
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12414 '<Symbol "/" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12414
Cast_Expr_16 := Block_Var; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Block_Var; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_17, Rhs => Block_Var_2, Ret => Cast_Result_18);
--# end
--# expr-done 12415
--# expr-start 12417 '<FieldAccess for create_binop_assoc at ast.py:8204>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8204
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12416 '<Symbol "**" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 12416
Cast_Expr_18 := Block_Var; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Block_Var; 
      Cast_Result_20 := Cast_Expr_19;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_19, Rhs => Block_Var_2, Ret => Cast_Result_20);
--# end
--# expr-done 12417
--# expr-start 12419 '<FieldAccess for create_binop_assoc at ast.py:8206>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8206
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12418 '<Symbol "<" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12418
Cast_Expr_20 := Block_Var; 
      Cast_Result_21 := Cast_Expr_20;
Cast_Expr_21 := Block_Var; 
      Cast_Result_22 := Cast_Expr_21;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_21, Rhs => Cast_Result_22, Ret => Block_Var_1);
--# end
--# expr-done 12419
--# expr-start 12421 '<FieldAccess for create_binop_assoc at ast.py:8207>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8207
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12420 '<Symbol "<=" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12420
Cast_Expr_22 := Block_Var; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Block_Var; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_23, Rhs => Cast_Result_24, Ret => Block_Var_1);
--# end
--# expr-done 12421
--# expr-start 12423 '<FieldAccess for create_binop_assoc at ast.py:8208>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8208
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12422 '<Symbol "=" at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12422
Cast_Expr_24 := Block_Var; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Block_Var; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_25, Rhs => Cast_Result_26, Ret => Block_Var_1);
--# end
--# expr-done 12423
--# expr-start 12425 '<FieldAccess for create_binop_assoc at ast.py:8209>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8209
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12424 '<Symbol "/=" at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12424
Cast_Expr_26 := Block_Var; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Block_Var; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_11, Lhs => Cast_Result_27, Rhs => Cast_Result_28, Ret => Block_Var_1);
--# end
--# expr-done 12425
--# expr-start 12427 '<FieldAccess for create_binop_assoc at ast.py:8210>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8210
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12426 '<Symbol ">" at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12426
Cast_Expr_28 := Block_Var; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Block_Var; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_12, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Block_Var_1);
--# end
--# expr-done 12427
--# expr-start 12429 '<FieldAccess for create_binop_assoc at ast.py:8211>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8211
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12428 '<Symbol ">=" at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12428
Cast_Expr_30 := Block_Var; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Block_Var; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_binop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_13, Lhs => Cast_Result_31, Rhs => Cast_Result_32, Ret => Block_Var_1);
--# end
--# expr-done 12429
--# expr-start 12431 '<FieldAccess for create_unop_assoc at ast.py:8213>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8213
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12430 '<Symbol "+" at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12430
Cast_Expr_32 := Block_Var; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Block_Var; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_unop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_14, Rhs => Cast_Result_33, Ret => Cast_Result_34);
--# end
--# expr-done 12431
--# expr-start 12433 '<FieldAccess for create_unop_assoc at ast.py:8214>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8214
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12432 '<Symbol "-" at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12432
Cast_Expr_34 := Block_Var; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Block_Var; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_15, Rhs => Cast_Result_35, Ret => Cast_Result_36);
--# end
--# expr-done 12433
--# expr-start 12435 '<FieldAccess for create_unop_assoc at ast.py:8215>' Fld_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8215
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12434 '<Symbol "abs" at ???>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 12434
Cast_Expr_36 := Block_Var; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Block_Var; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_unop_assoc
Fld_21 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_16, Rhs => Cast_Result_37, Ret => Cast_Result_38);
--# end
--# expr-done 12435
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18, 15 => Fld_19, 16 => Fld_20, 17 => Fld_21)); 
--# expr-done 12436
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12437
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_603;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_603;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ordinary_Fixed_Point_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Record_Type_Def
   --
      procedure Initialize_Fields_For_Record_Type_Def
        (Self : Bare_Record_Type_Def
         ; Record_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Record_Type_Def_F_Has_Tagged : Bare_Tagged_Node
         ; Record_Type_Def_F_Has_Limited : Bare_Limited_Node
         ; Record_Type_Def_F_Record_Def : Bare_Base_Record_Def
        ) is
      begin
            Self.Record_Type_Def_F_Has_Abstract := Record_Type_Def_F_Has_Abstract;
            Self.Record_Type_Def_F_Has_Tagged := Record_Type_Def_F_Has_Tagged;
            Self.Record_Type_Def_F_Has_Limited := Record_Type_Def_F_Has_Limited;
            Self.Record_Type_Def_F_Record_Def := Record_Type_Def_F_Record_Def;
      end Initialize_Fields_For_Record_Type_Def;
   function Record_Type_Def_F_Has_Abstract
     (Node : Bare_Record_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Abstract;
   end;
   function Record_Type_Def_F_Has_Tagged
     (Node : Bare_Record_Type_Def) return Bare_Tagged_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Tagged;
   end;
   function Record_Type_Def_F_Has_Limited
     (Node : Bare_Record_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Limited;
   end;
   function Record_Type_Def_F_Record_Def
     (Node : Bare_Record_Type_Def) return Bare_Base_Record_Def
   is
   begin
         return Node.Record_Type_Def_F_Record_Def;
   end;
--# property-start RecordTypeDef.is_tagged_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5373
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Record_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Record_Type_Def  := Bare_Record_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12439 '<FieldAccess for as_bool at ast.py:5373>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5373
--# expr-start 12438 '<FieldAccess for has_tagged at ast.py:5373>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5373
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Record_Type_Def_F_Has_Tagged;
--# expr-done 12438
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Tagged.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Tagged_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 12439
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Record_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start RecordTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5375
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Xref_Equation
  (Node : Bare_Record_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Record_Type_Def  := Bare_Record_Type_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_372 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_372 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_372;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:5375") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_372;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_372;
            raise;
      end;
   return Property_Result;
end Record_Type_Def_P_Xref_Equation;
--# end
--# property-start RecordTypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5377
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Predefined_Operators
  (Node : Bare_Record_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Record_Type_Def  := Bare_Record_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_373 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
Fld_1 : Bare_Limited_Node;
Fld_2 : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_373 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_373;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Record_Type_Def_Record_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Record_Type_Def,
                                As_Bare_Record_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12444 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12443 '<If at ast.py:5379>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5379
--# expr-start 12441 '<FieldAccess for as_bool at ast.py:5380>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5380
--# expr-start 12440 '<FieldAccess for has_limited at ast.py:5380>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5380
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Record_Type_Def_F_Has_Limited;
--# expr-done 12440
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Limited.as_bool'
Fld_2 := Libadalang.Implementation.Dispatcher_Limited_Node_P_As_Bool (Node => Fld_1);
--# end
--# expr-done 12441
if Fld_2 then
   If_Result := No_Internal_Env_Assoc_Array_Type;
else
   --# expr-start 12442 '<FieldAccess for predefined_equality_operators at ast.py:5382>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5382
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.predefined_equality_operators
Fld := Libadalang.Implementation.Type_Def_P_Predefined_Equality_Operators (Node => Self);
--# end
--# expr-done 12442
   If_Result := Fld;
end if;
      Inc_Ref (If_Result);
--# expr-done 12443
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12444
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_373;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_373;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Record_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Signed_Int_Type_Def
   --
      procedure Initialize_Fields_For_Signed_Int_Type_Def
        (Self : Bare_Signed_Int_Type_Def
         ; Signed_Int_Type_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Signed_Int_Type_Def_F_Range := Signed_Int_Type_Def_F_Range;
      end Initialize_Fields_For_Signed_Int_Type_Def;
   function Signed_Int_Type_Def_F_Range
     (Node : Bare_Signed_Int_Type_Def) return Bare_Range_Spec
   is
   begin
         return Node.Signed_Int_Type_Def_F_Range;
   end;
--# property-start SignedIntTypeDef.is_int_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8588
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Is_Int_Type
  (Node : Bare_Signed_Int_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Signed_Int_Type_Def_P_Is_Int_Type;
--# end
--# property-start SignedIntTypeDef.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8590
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Xref_Equation
  (Node : Bare_Signed_Int_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
     Ent : Internal_Entity_Signed_Int_Type_Def :=
       Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_640 with Inline_Always;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_640 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_640;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12446 '<FieldAccess for xref_equation at ast.py:8590>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8590
--# expr-start 12445 '<FieldAccess for range at ast.py:8590>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8590
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 12445
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12446
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_640;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_640;
            raise;
      end;
   return Property_Result;
end Signed_Int_Type_Def_P_Xref_Equation;
--# end
--# property-start SignedIntTypeDef.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8592
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Discrete_Range
  (Node : Bare_Signed_Int_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
     Ent : Internal_Entity_Signed_Int_Type_Def :=
       Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12450 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12449 '<FieldAccess for discrete_range at ast.py:8594>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8594
--# expr-start 12448 '<FieldAccess for range at ast.py:8594>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8594
--# expr-start 12447 '<FieldAccess for range at ast.py:8594>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8594
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 12447
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 12448
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.discrete_range
Fld_2 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 12449
Let_Result := Fld_2; 
--# end
--# expr-done 12450
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Signed_Int_Type_Def_P_Discrete_Range;
--# end
--# property-start SignedIntTypeDef.predefined_operators /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8596
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Predefined_Operators
  (Node : Bare_Signed_Int_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_642 with Inline_Always;
      Block_Var : Bare_Type_Decl;
Block_Var_1 : Bare_Base_Type_Decl;
Block_Var_2 : Bare_Base_Type_Decl;
Block_Var_3 : Internal_Env_Assoc_Array_Access;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_14 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_642 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_642;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Signed_Int_Type_Def_Signed_Int_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Signed_Int_Type_Def,
                                As_Bare_Signed_Int_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12490 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12452 '<Cast to TypeDecl at ast.py:8598>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8598
--# expr-start 12451 '<parent at ast.py:8598>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8598
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12451
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12452
Block_Var := Cast_Result; 
--# bind self_type Block_Var
--# expr-start 12454 '<FieldAccess for node at ast.py:8599>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8599
--# expr-start 12453 '<FieldAccess for bool_type at ast.py:8599>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8599
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12453
Fld_2 := Fld_1.Node;
--# expr-done 12454
Block_Var_1 := Fld_2; 
--# bind bool_type Block_Var_1
--# expr-start 12456 '<FieldAccess for node at ast.py:8600>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8600
--# expr-start 12455 '<FieldAccess for int_type at ast.py:8600>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8600
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 12455
Fld_4 := Fld_3.Node;
--# expr-done 12456
Block_Var_2 := Fld_4; 
--# bind int_type Block_Var_2
--# expr-start 12489 '<ArrayLiteral at ???>' Array_Lit None
--# expr-start 12458 '<FieldAccess for create_binop_assoc at ast.py:8603>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8603
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12457 '<Symbol "+" at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12457
Cast_Expr := Block_Var; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Block_Var; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 12458
--# expr-start 12460 '<FieldAccess for create_binop_assoc at ast.py:8604>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8604
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12459 '<Symbol "-" at ???>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12459
Cast_Expr_3 := Block_Var; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Block_Var; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Block_Var; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 12460
--# expr-start 12462 '<FieldAccess for create_binop_assoc at ast.py:8605>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8605
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12461 '<Symbol "*" at ???>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12461
Cast_Expr_6 := Block_Var; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Block_Var; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Block_Var; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 12462
--# expr-start 12464 '<FieldAccess for create_binop_assoc at ast.py:8606>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8606
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12463 '<Symbol "/" at ???>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12463
Cast_Expr_9 := Block_Var; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Block_Var; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Block_Var; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 12464
--# expr-start 12466 '<FieldAccess for create_binop_assoc at ast.py:8607>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8607
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12465 '<Symbol "mod" at ???>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod); 
--# expr-done 12465
Cast_Expr_12 := Block_Var; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Block_Var; 
      Cast_Result_14 := Cast_Expr_13;
Cast_Expr_14 := Block_Var; 
      Cast_Result_15 := Cast_Expr_14;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_13, Rhs => Cast_Result_14, Ret => Cast_Result_15);
--# end
--# expr-done 12466
--# expr-start 12468 '<FieldAccess for create_binop_assoc at ast.py:8608>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8608
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12467 '<Symbol "rem" at ???>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem); 
--# expr-done 12467
Cast_Expr_15 := Block_Var; 
      Cast_Result_16 := Cast_Expr_15;
Cast_Expr_16 := Block_Var; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Block_Var; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_16, Rhs => Cast_Result_17, Ret => Cast_Result_18);
--# end
--# expr-done 12468
--# expr-start 12470 '<FieldAccess for create_binop_assoc at ast.py:8610>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8610
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12469 '<Symbol "**" at ???>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 12469
Cast_Expr_18 := Block_Var; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Block_Var; 
      Cast_Result_20 := Cast_Expr_19;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_19, Rhs => Block_Var_2, Ret => Cast_Result_20);
--# end
--# expr-done 12470
--# expr-start 12472 '<FieldAccess for create_binop_assoc at ast.py:8612>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8612
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12471 '<Symbol "<" at ???>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12471
Cast_Expr_20 := Block_Var; 
      Cast_Result_21 := Cast_Expr_20;
Cast_Expr_21 := Block_Var; 
      Cast_Result_22 := Cast_Expr_21;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_21, Rhs => Cast_Result_22, Ret => Block_Var_1);
--# end
--# expr-done 12472
--# expr-start 12474 '<FieldAccess for create_binop_assoc at ast.py:8613>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8613
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12473 '<Symbol "<=" at ???>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12473
Cast_Expr_22 := Block_Var; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Block_Var; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_23, Rhs => Cast_Result_24, Ret => Block_Var_1);
--# end
--# expr-done 12474
--# expr-start 12476 '<FieldAccess for create_binop_assoc at ast.py:8614>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8614
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12475 '<Symbol "=" at ???>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12475
Cast_Expr_24 := Block_Var; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Block_Var; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_25, Rhs => Cast_Result_26, Ret => Block_Var_1);
--# end
--# expr-done 12476
--# expr-start 12478 '<FieldAccess for create_binop_assoc at ast.py:8615>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8615
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12477 '<Symbol "/=" at ???>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12477
Cast_Expr_26 := Block_Var; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Block_Var; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_27, Rhs => Cast_Result_28, Ret => Block_Var_1);
--# end
--# expr-done 12478
--# expr-start 12480 '<FieldAccess for create_binop_assoc at ast.py:8616>' Fld_16 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8616
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12479 '<Symbol ">" at ???>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12479
Cast_Expr_28 := Block_Var; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Block_Var; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_11, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Block_Var_1);
--# end
--# expr-done 12480
--# expr-start 12482 '<FieldAccess for create_binop_assoc at ast.py:8617>' Fld_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8617
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12481 '<Symbol ">=" at ???>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12481
Cast_Expr_30 := Block_Var; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Block_Var; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_12, Lhs => Cast_Result_31, Rhs => Cast_Result_32, Ret => Block_Var_1);
--# end
--# expr-done 12482
--# expr-start 12484 '<FieldAccess for create_unop_assoc at ast.py:8619>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8619
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12483 '<Symbol "+" at ???>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12483
Cast_Expr_32 := Block_Var; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Block_Var; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_unop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_13, Rhs => Cast_Result_33, Ret => Cast_Result_34);
--# end
--# expr-done 12484
--# expr-start 12486 '<FieldAccess for create_unop_assoc at ast.py:8620>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8620
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12485 '<Symbol "-" at ???>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12485
Cast_Expr_34 := Block_Var; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Block_Var; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_unop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_14, Rhs => Cast_Result_35, Ret => Cast_Result_36);
--# end
--# expr-done 12486
--# expr-start 12488 '<FieldAccess for create_unop_assoc at ast.py:8621>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8621
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12487 '<Symbol "abs" at ???>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 12487
Cast_Expr_36 := Block_Var; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Block_Var; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_15, Rhs => Cast_Result_37, Ret => Cast_Result_38);
--# end
--# expr-done 12488
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18, 15 => Fld_19, 16 => Fld_20)); 
--# expr-done 12489
Block_Var_3 := Array_Lit; Inc_Ref (Block_Var_3);
--# bind defaults Block_Var_3
Let_Result := Block_Var_3; Inc_Ref (Let_Result);
--# end
--# expr-done 12490
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_642;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_642;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Signed_Int_Type_Def_P_Predefined_Operators;
--# end
--# property-start SignedIntTypeDef.is_static /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8626
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Is_Static
  (Node : Bare_Signed_Int_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
     Ent : Internal_Entity_Signed_Int_Type_Def :=
       Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12493 '<FieldAccess for is_static_expr at ast.py:8626>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8626
--# expr-start 12492 '<FieldAccess for range at ast.py:8626>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8626
--# expr-start 12491 '<FieldAccess for range at ast.py:8626>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:8626
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 12491
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 12492
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 12493
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Signed_Int_Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Type_Expr
   --
--# property-start TypeExpr.array_ndims /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9412
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Array_Ndims
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Integer;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Integer;
Dyn_Var_Bind_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12497 '<bind at ast.py:9413>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9413
--# expr-start 12494 '<FieldAccess for origin_node at ast.py:9413>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9413
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 12494
Bound_Origin := Fld; 
--# expr-start 12496 '<FieldAccess for array_ndims at ast.py:9413>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9413
--# expr-start 12495 '<FieldAccess for designated_type at ast.py:9413>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9413
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 12495
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 12496
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 12497
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Array_Ndims;
--# end
--# property-start TypeExpr.type_name /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9416
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Type_Name
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Var_Expr : Internal_Entity_Subtype_Indication;
Cast_Expr : Internal_Entity_Type_Expr;
Cast_Result : Internal_Entity_Subtype_Indication;
Fld : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 12498 '<Cast to SubtypeIndication at ast.py:9417>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9417
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Subtype_Indication;
   end if;
--# expr-done 12498
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Subtype_Indication then
   --# scope-start
   --# bind sti Var_Expr
--# expr-start 12499 '<FieldAccess for name at ast.py:9417>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9417
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Var_Expr.Node.Subtype_Indication_F_Name, Info => Var_Expr.Info);
--# expr-done 12499
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Type_Name;
--# end
--# property-start TypeExpr.defining_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9423
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Defining_Env
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_752 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_752 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_752;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12502 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12501 '<FieldAccess for defining_env at ast.py:9426>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9426
--# expr-start 12500 '<FieldAccess for designated_type at ast.py:9426>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9426
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12500
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 12501
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 12502
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_752;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_752;
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Defining_Env;
--# end
--# property-start '[dispatcher]TypeExpr.designated_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Expr_P_Designated_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Expr (Self.Kind) is
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.designated_type
                  Property_Result := Anonymous_Type_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Lit_Synth_Type_Expr =>
                  --# property-call-start EnumLitSynthTypeExpr.designated_type
                  Property_Result := Enum_Lit_Synth_Type_Expr_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.designated_type
                  Property_Result := Subtype_Indication_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Type_Expr =>
                  --# property-call-start SyntheticTypeExpr.designated_type
                  Property_Result := Synthetic_Type_Expr_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Expr_P_Designated_Type;
--# end
--# property-start TypeExpr.designated_type_decl /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9436
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Designated_Type_Decl
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 12505 '<bind at ast.py:9437>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9437
--# expr-start 12503 '<FieldAccess for origin_node at ast.py:9437>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9437
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 12503
Bound_Origin := Fld; 
--# expr-start 12504 '<FieldAccess for designated_type at ast.py:9437>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9437
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 12504
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 12505
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Designated_Type_Decl;
--# end
--# property-start TypeExpr.designated_type_decl_from /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9444
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Designated_Type_Decl_From
  (Node : Bare_Type_Expr
      ; Origin_Node : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin_node Origin_Node
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 12510 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12509 '<bind at ast.py:9450>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9450
--# expr-start 12507 '<FieldAccess for origin_node at ast.py:9450>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9450
--# expr-start 12506 '<FieldAccess for node at ast.py:9450>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9450
Fld := Origin_Node.Node;
--# expr-done 12506
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld);
--# end
--# expr-done 12507
Bound_Origin := Fld_1; 
--# expr-start 12508 '<FieldAccess for designated_type at ast.py:9451>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9451
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 12508
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 12509
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 12510
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Designated_Type_Decl_From;
--# end
--# property-start TypeExpr.element_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9453
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Element_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Anonymous_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Fld_2 : Internal_Entity_Type_Def;
Scope_Result : Internal_Entity_Type_Def;
Result_Var : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Access_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12518 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12511 '<FieldAccess for designated_type at ast.py:9459>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9459
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12511
Block_Var := Fld; 
--# bind d Block_Var
--# expr-start 12517 '<If at ast.py:9460>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9460
--# expr-start 12515 '<IsNull at ast.py:9461>' Is_Null /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9461
--# expr-start 12514 '<Cast to AccessDef at ast.py:9461>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9461
--# expr-start 12512 '<Cast to AnonymousTypeDecl at ast.py:9461>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9461
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Anonymous_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Anonymous_Type_Decl;
   end if;
--# expr-done 12512
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Anonymous_Type_Decl then
   --# scope-start
--# expr-start 12513 '<FieldAccess for type_def at ast.py:9461>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9461
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Type_Def (Node => Var_Expr.Node.Type_Decl_F_Type_Def, Info => Var_Expr.Info);
--# expr-done 12513
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Access_Def
   then
      Cast_Result_1 := Create_Internal_Entity_Access_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Access_Def;
   end if;
--# expr-done 12514
Is_Null := Cast_Result_1.Node = null; 
--# expr-done 12515
if Is_Null then
   If_Result := Block_Var;
else
   --# expr-start 12516 '<FieldAccess for accessed_type at ast.py:9463>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9463
      if Block_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 12516
   If_Result := Fld_1;
end if;
--# expr-done 12517
Let_Result := If_Result; 
--# end
--# expr-done 12518
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Element_Type;
--# end
--# property-start TypeExpr.canonical_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9466
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Canonical_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12521 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12519 '<FieldAccess for designated_type at ast.py:9469>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9469
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12519
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 12520 '<FieldAccess for canonical_type at ast.py:9469>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9469
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 12520
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 12521
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Canonical_Type;
--# end
   --
   --  Primitives for Bare_Anonymous_Type
   --
      procedure Initialize_Fields_For_Anonymous_Type
        (Self : Bare_Anonymous_Type
         ; Anonymous_Type_F_Type_Decl : Bare_Anonymous_Type_Decl
        ) is
      begin
            Self.Anonymous_Type_F_Type_Decl := Anonymous_Type_F_Type_Decl;
      end Initialize_Fields_For_Anonymous_Type;
   function Anonymous_Type_F_Type_Decl
     (Node : Bare_Anonymous_Type) return Bare_Anonymous_Type_Decl
   is
   begin
         return Node.Anonymous_Type_F_Type_Decl;
   end;
--# property-start AnonymousType.designated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9505
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Designated_Type
  (Node : Bare_Anonymous_Type
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Anonymous_Type  := Bare_Anonymous_Type (Node);
     Ent : Internal_Entity_Anonymous_Type :=
       Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 12522 '<FieldAccess for type_decl at ast.py:9505>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9505
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 12522
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Type_P_Designated_Type;
--# end
--# property-start AnonymousType.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9506
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Xref_Equation
  (Node : Bare_Anonymous_Type
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Anonymous_Type  := Bare_Anonymous_Type (Node);
     Ent : Internal_Entity_Anonymous_Type :=
       Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_761 with Inline_Always;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_761 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_761;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12524 '<FieldAccess for sub_equation at ast.py:9506>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9506
--# expr-start 12523 '<FieldAccess for type_decl at ast.py:9506>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9506
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 12523
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12524
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_761;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_761;
            raise;
      end;
   return Property_Result;
end Anonymous_Type_P_Xref_Equation;
--# end
--# property-start AnonymousType.custom_id_text /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9513
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Custom_Id_Text
  (Node : Bare_Anonymous_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Anonymous_Type  := Bare_Anonymous_Type (Node);
     Ent : Internal_Entity_Anonymous_Type :=
       Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_762 with Inline_Always;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Fld_1 : String_Type;
            procedure Finalizer_Scope_762 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_762;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12526 '<FieldAccess for text at ast.py:9513>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9513
--# expr-start 12525 '<FieldAccess for type_decl at ast.py:9513>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9513
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 12525
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Text (Node => Fld.Node);
--# expr-done 12526
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_762;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_762;
            raise;
      end;
   return Property_Result;
end Anonymous_Type_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Enum_Lit_Synth_Type_Expr
   --
--# property-start EnumLitSynthTypeExpr.designated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9477
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Synth_Type_Expr_P_Designated_Type
  (Node : Bare_Enum_Lit_Synth_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Enum_Lit_Synth_Type_Expr  := Bare_Enum_Lit_Synth_Type_Expr (Node);
     Ent : Internal_Entity_Enum_Lit_Synth_Type_Expr :=
       Internal_Entity_Enum_Lit_Synth_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 12529 '<FieldAccess for enum_type at ast.py:9478>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9478
--# expr-start 12528 '<Cast to EnumLiteralDecl at ast.py:9478>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9478
--# expr-start 12527 '<parent at ast.py:9478>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9478
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12527
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 12528
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 12529
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Lit_Synth_Type_Expr_P_Designated_Type;
--# end
--# property-start EnumLitSynthTypeExpr.custom_id_text /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9481
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text
  (Node : Bare_Enum_Lit_Synth_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Enum_Lit_Synth_Type_Expr  := Bare_Enum_Lit_Synth_Type_Expr (Node);
     Ent : Internal_Entity_Enum_Lit_Synth_Type_Expr :=
       Internal_Entity_Enum_Lit_Synth_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_759 with Inline_Always;
      Cast_Expr : Bare_Enum_Lit_Synth_Type_Expr;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Enum_Literal_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type_Array_Access;
Str_1 : String_Type;
Fld_5 : String_Type;
Concat_Result_1 : String_Type;
Dyn_Var_Bind_Result : String_Type;
            procedure Finalizer_Scope_759 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_759;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12541 '<bind at ast.py:9484>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9484
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 12540 '<Concat at ast.py:9488>' Concat_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9488
--# expr-start 12533 '<Concat at ast.py:9487>' Concat_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9487
--# expr-start 12531 '<FieldAccess for canonical_fully_qualified_name at ast.py:9486>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9486
--# expr-start 12530 '<FieldAccess for designated_type at ast.py:9486>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9486
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 12530
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 12531
--# expr-start 12532 '<String at ast.py:9487>' Str /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9487
Str := Create_String ("."); 
--# expr-done 12532
Concat_Result := Concat_String (Fld_1, Str); 
--# expr-done 12533
--# expr-start 12539 '<FieldAccess for sym_join at ast.py:9489>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9489
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12537 '<FieldAccess for as_symbol_array at ast.py:9491>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9491
--# expr-start 12536 '<FieldAccess for defining_name at ast.py:9491>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9491
--# expr-start 12535 '<Cast to EnumLiteralDecl at ast.py:9490>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9490
--# expr-start 12534 '<parent at ast.py:9490>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9490
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12534
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 12535
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 12536
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_3.Node);
--# end
--# expr-done 12537
--# expr-start 12538 '<String at ast.py:9492>' Str_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9492
Str_1 := Create_String (""); 
--# expr-done 12538
--# property-call-start AdaNode.sym_join
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Ent.Node, Syms => Fld_4, Sep => Str_1);
--# end
--# expr-done 12539
Concat_Result_1 := Concat_String (Concat_Result, Fld_5); 
--# expr-done 12540
Dyn_Var_Bind_Result := Concat_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 12541
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_759;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_759;
            raise;
      end;
   return Property_Result;
end Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Subtype_Indication
   --
      procedure Initialize_Fields_For_Subtype_Indication
        (Self : Bare_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Self.Subtype_Indication_F_Has_Not_Null := Subtype_Indication_F_Has_Not_Null;
            Self.Subtype_Indication_F_Name := Subtype_Indication_F_Name;
            Self.Subtype_Indication_F_Constraint := Subtype_Indication_F_Constraint;
      end Initialize_Fields_For_Subtype_Indication;
   function Subtype_Indication_F_Has_Not_Null
     (Node : Bare_Subtype_Indication) return Bare_Not_Null
   is
   begin
         return Node.Subtype_Indication_F_Has_Not_Null;
   end;
   function Subtype_Indication_F_Name
     (Node : Bare_Subtype_Indication) return Bare_Name
   is
   begin
         return Node.Subtype_Indication_F_Name;
   end;
   function Subtype_Indication_F_Constraint
     (Node : Bare_Subtype_Indication) return Bare_Constraint
   is
   begin
         return Node.Subtype_Indication_F_Constraint;
   end;
--# property-start SubtypeIndication.designated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9540
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Designated_Type
  (Node : Bare_Subtype_Indication
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_763 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_763 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_763;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12545 '<bind at ast.py:9541>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9541
--# expr-start 12542 '<FieldAccess for node_env at ast.py:9541>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9541
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12542
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 12544 '<FieldAccess for designated_type_impl at ast.py:9541>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9541
--# expr-start 12543 '<FieldAccess for name at ast.py:9541>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9541
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Subtype_Indication_F_Name, Info => Ent.Info);
--# expr-done 12543
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 12544
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 12545
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_763;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_763;
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Designated_Type;
--# end
--# property-start SubtypeIndication.complete_items /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9544
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Complete_Items
  (Node : Bare_Subtype_Indication
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_764 with Inline_Always;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Package_Decl;
Match_2 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Internal_Unit;
Fld_5 : Boolean;
Fld_6 : Internal_Entity;
Is_A : Boolean;
Fld_7 : Internal_Entity;
Fld_8 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Boolean;
If_Result_1 : Integer;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Integer;
Scope_Result : Integer;
Cast_Result_3 : Internal_Entity_Package_Decl;
Let_Result_1 : Integer;
Scope_Result_1 : Integer;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Integer;
Scope_Result_2 : Integer;
Match_Result : Integer;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
Let_Result_3 : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_764 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_764;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12563 '<Block at ???>' Let_Result_3 None
--# scope-start
--# expr-start 12562 '<Map at ast.py:9552>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9552
   --# expr-start 12547 '<EnvGet at ast.py:9552>' Env_Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9552
--# expr-start 12546 '<FieldAccess for children_env at ast.py:9552>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9552
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12546
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 12547
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_364 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_364
      --# expr-start 12561 '<New CompletionItem at ast.py:9553>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9553
--# expr-start 12548 '<Cast to BasicDecl at ast.py:9554>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9554
Cast_Expr := Item_364; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 12548
--# expr-start 12551 '<FieldAccess for dottable_subp at ast.py:9555>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9555
--# expr-start 12550 '<FieldAccess for md at ast.py:9555>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9555
--# expr-start 12549 '<FieldAccess for info at ast.py:9555>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9555
Fld_1 := Item_364.Info;
--# expr-done 12549
Fld_2 := Fld_1.Md;
--# expr-done 12550
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 12551
--# expr-start 12553 '<FieldAccess for has_with_visibility at ast.py:9556>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9556
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12552 '<FieldAccess for unit at ast.py:9556>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9556
      if Item_364.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Unit (Node => Item_364.Node);
--# expr-done 12552
--# property-call-start AdaNode.has_with_visibility
Fld_5 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_4);
--# end
--# expr-done 12553
--# expr-start 12560 '<Match at ast.py:9557>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9557
      if Item_364.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_364; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind btd Match_0
--# expr-start 12559 '<If at ast.py:9558>' If_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9558
--# expr-start 12555 '<IsA DerivedTypeDef at ast.py:9565>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9565
--# expr-start 12554 '<parent at ast.py:9565>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9565
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12554
Is_A := Fld_6.Node /= null 
and then Fld_6.Node.Kind in Ada_Derived_Type_Def_Range; 
--# expr-done 12555
if Is_A then
   --# expr-start 12558 '<Eq at ast.py:9566>' Is_Equiv /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9566
--# expr-start 12557 '<parent at ast.py:9566>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9566
--# expr-start 12556 '<parent at ast.py:9566>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9566
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12556
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Parent (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# expr-done 12557
Cast_Expr_1 := Match_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Fld_8, Cast_Result_1); 
--# expr-done 12558
   If_Result := Is_Equiv;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := 0;
else
   If_Result_1 := 100;
end if;
--# expr-done 12559
Let_Result := If_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind _ Match_1
Let_Result_1 := 50; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Generic_Package_Internal | Ada_Abstract_Subp_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := 0; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 12560
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_3, Is_Visible => Fld_5, Weight => Match_Result); 
--# expr-done 12561
         declare
            Item_To_Append : constant Internal_Completion_Item := New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12562
Let_Result_3 := Map_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 12563
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_764;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_764;
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Complete_Items;
--# end
--# property-start SubtypeIndication.subtype_constraints /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9577
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Subtype_Constraints
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_765 with Inline_Always;
            procedure Finalizer_Scope_3358 with Inline_Always;
            procedure Finalizer_Scope_3359 with Inline_Always;
            procedure Finalizer_Scope_3361 with Inline_Always;
      Block_Var : Internal_Entity_Assoc_List;
Block_Var_1 : Internal_Param_Actual_Array_Access;
Var_Expr : Internal_Entity_Constraint;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Ds : Internal_Entity_Discriminant_Spec;
Var_Expr_2 : Internal_Entity_Assoc_List;
Fld : Internal_Entity_Constraint;
Cast_Expr : Internal_Entity_Constraint;
Cast_Result : Internal_Entity_Composite_Constraint;
Fld_1 : Internal_Entity_Assoc_List;
Scope_Result : Internal_Entity_Assoc_List;
Result_Var : Internal_Entity_Assoc_List;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_1 : Internal_Entity_Discriminant_Spec;
Fld_4 : Internal_Entity_Defining_Name_List;
Fld_5 : Bare_Defining_Name_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Fld_7 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
Item_366 : Internal_Entity_Defining_Name;
Bare_Item_366 : Bare_Defining_Name;
Map_Result : Internal_Param_Actual_Array_Access;
Let_Result : Internal_Param_Actual_Array_Access;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Fld_8 : Internal_Entity_Defining_Name;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Internal_Entity_Expr;
New_Struct_1 : Internal_Param_Actual;
Index_368 : Integer;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_2 : Internal_Param_Actual_Array_Access;
Result_Var_2 : Internal_Param_Actual_Array_Access;
Let_Result_1 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_765 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_765;
            procedure Finalizer_Scope_3358 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3358;
            procedure Finalizer_Scope_3359 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3359;
            procedure Finalizer_Scope_3361 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_3361;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 12582 '<Block at ???>' Let_Result_1 None
--# scope-start
--# expr-start 12564 '<FieldAccess for constraint at ast.py:9584>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9584
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 12564
Var_Expr := Fld;
if Var_Expr /= No_Entity_Constraint then
   --# scope-start
--# expr-start 12566 '<FieldAccess for constraints at ast.py:9584>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9584
--# expr-start 12565 '<Cast to CompositeConstraint at ast.py:9584>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9584
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Composite_Constraint_Range
   then
      Cast_Result := Create_Internal_Entity_Composite_Constraint
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Composite_Constraint;
   end if;
--# expr-done 12565
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Composite_Constraint_F_Constraints, Info => Cast_Result.Info);
--# expr-done 12566
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Assoc_List;
end if;
Block_Var := Result_Var; 
--# bind constraints Block_Var
--# expr-start 12575 '<Mapcat at ast.py:9588>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9588
--# expr-start 12567 '<FieldAccess for designated_type_decl at ast.py:9588>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9588
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_2 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12567
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 12568 '<FieldAccess for discriminants_list at ast.py:9588>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9588
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_1.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 12568
Scope_Result_1 := Fld_3;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3358;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
      declare
         Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Result_Var_1;
      begin
         for Item_365 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_365
      --# expr-start 12574 '<Let at ast.py:9589>' Let_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9589
--# scope-start
--# expr-start 12569 '<Cast to DiscriminantSpec at ast.py:9590>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9590
Cast_Expr_1 := Item_365; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Discriminant_Spec_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Discriminant_Spec
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Discriminant_Spec;
   end if;
--# expr-done 12569
Ds := Cast_Result_1; 
--# bind ds Ds
--# expr-start 12573 '<Map at ast.py:9591>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9591
   --# expr-start 12570 '<FieldAccess for ids at ast.py:9591>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9591
      if Ds.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Defining_Name_List (Node => Ds.Node.Discriminant_Spec_F_Ids, Info => Ds.Info);
--# expr-done 12570
Fld_5 := Fld_4.Node;
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Internal_Param_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_5;
      begin
         for Untyped_Item_366 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_366 := Untyped_Item_366; 
Is_Null := Bare_Item_366 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_366); 
   If_Result := As_Entity;
end if;
                  Item_366 := If_Result; 
   --# scope-start
               --# bind i Item_366
      --# expr-start 12572 '<New ParamActual at ast.py:9592>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9592
--# expr-start 12571 '<FieldAccess for default_expr at ast.py:9594>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9594
      if Ds.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Ds.Node.Discriminant_Spec_F_Default_Expr, Info => Ds.Info);
--# expr-done 12571
New_Struct := (Actual => Fld_7, Param => Item_366); 
--# expr-done 12572
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12573
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12574
         for Item_To_Append of
               Let_Result.Items
         loop
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3359;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 12575
Block_Var_1 := Map_Result_1; Inc_Ref (Block_Var_1);
--# bind discrs Block_Var_1
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind c Var_Expr_2
--# expr-start 12581 '<Map at ast.py:9603>' Map_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9603
      declare
         Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
         Index_368 := 0;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Block_Var_1;
      begin
         for Item_367 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_367
               --# bind i Index_368
      --# expr-start 12580 '<New ParamActual at ast.py:9604>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9604
--# expr-start 12578 '<FieldAccess for actual_for_param_at at ast.py:9606>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9606
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12576 '<FieldAccess for param at ast.py:9606>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9606
Fld_9 := Item_367.Param;
--# expr-done 12576
--# expr-start 12577 '<FieldAccess for actual at ast.py:9606>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9606
Fld_10 := Item_367.Actual;
--# expr-done 12577
--# property-call-start AssocList.actual_for_param_at
Fld_11 := Libadalang.Implementation.Assoc_List_P_Actual_For_Param_At (Node => Var_Expr_2.Node, Param => Fld_9, Pos => Index_368, Default_Expr => Fld_10, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 12578
--# expr-start 12579 '<FieldAccess for param at ast.py:9605>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9605
Fld_8 := Item_367.Param;
--# expr-done 12579
New_Struct_1 := (Actual => Fld_11, Param => Fld_8); 
--# expr-done 12580
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_1;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
               Index_368 := Index_368 + 1;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 12581
Scope_Result_2 := Map_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3361;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var_1;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 12582
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_765;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_765;
                     Finalizer_Scope_3358;
                     Finalizer_Scope_3359;
                     Finalizer_Scope_3361;
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Subtype_Constraints;
--# end
--# property-start SubtypeIndication.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9612
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Xref_Equation
  (Node : Bare_Subtype_Indication
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_766 with Inline_Always;
            procedure Finalizer_Scope_3363 with Inline_Always;
      Var_Expr : Internal_Entity_Constraint;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Constraint;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_766 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_766;
            procedure Finalizer_Scope_3363 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3363;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12588 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12587 '<And at ast.py:9616>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9616
--# expr-start 12584 '<FieldAccess for subtype_indication_equation at ast.py:9617>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9617
--# expr-start 12583 '<FieldAccess for name at ast.py:9617>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9617
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Subtype_Indication_F_Name, Info => Ent.Info);
--# expr-done 12583
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.subtype_indication_equation
Fld_1 := Libadalang.Implementation.Name_P_Subtype_Indication_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12584
--# expr-start 12585 '<FieldAccess for constraint at ast.py:9618>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9618
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 12585
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Constraint then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 12586 '<FieldAccess for sub_equation at ast.py:9619>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9619
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 12586
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3363;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9619") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9616") else null)); 
--# expr-done 12587
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 12588
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_766;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_766;
                     Finalizer_Scope_3363;
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Xref_Equation;
--# end
--# property-start SubtypeIndication.discrete_range /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9623
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Discrete_Range
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Block_Var : Internal_Entity_Range_Constraint;
Var_Expr : Internal_Entity_Range_Constraint;
Fld : Internal_Entity_Constraint;
Cast_Result : Internal_Entity_Range_Constraint;
Fld_1 : Internal_Entity_Range_Spec;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Discrete_Range;
Scope_Result : Internal_Discrete_Range;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Discrete_Range;
Result_Var : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12596 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12590 '<Cast to RangeConstraint at ast.py:9625>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9625
--# expr-start 12589 '<FieldAccess for constraint at ast.py:9625>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9625
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 12589
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Range_Constraint_Range
   then
      Cast_Result := Create_Internal_Entity_Range_Constraint
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 12590
Block_Var := Cast_Result; 
--# bind rc Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Range_Constraint then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 12593 '<FieldAccess for discrete_range at ast.py:9627>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9627
--# expr-start 12592 '<FieldAccess for range at ast.py:9627>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9627
--# expr-start 12591 '<FieldAccess for range at ast.py:9627>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9627
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Range_Spec (Node => Var_Expr.Node.Range_Constraint_F_Range, Info => Var_Expr.Info);
--# expr-done 12591
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Fld_1.Node.Range_Spec_F_Range, Info => Fld_1.Info);
--# expr-done 12592
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.discrete_range
Fld_3 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 12593
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 12595 '<FieldAccess for discrete_range at ast.py:9630>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9630
--# expr-start 12594 '<FieldAccess for designated_type_decl at ast.py:9630>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9630
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_4 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12594
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 12595
   Result_Var := Fld_5;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 12596
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Discrete_Range;
--# end
--# property-start SubtypeIndication.is_static_subtype /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9633
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Is_Static_Subtype
  (Node : Bare_Subtype_Indication
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Constraint;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Constraint;
Fld_2 : Boolean;
Scope_Result : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 12603 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12602 '<bind at ast.py:9640>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9640
--# expr-start 12597 '<FieldAccess for origin_node at ast.py:9640>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9640
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 12597
Bound_Origin := Fld; 
--# expr-start 12598 '<FieldAccess for constraint at ast.py:9640>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9640
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 12598
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Constraint then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 12599 '<FieldAccess for is_static at ast.py:9641>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9641
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.is_static
Fld_2 := Libadalang.Implementation.Constraint_P_Is_Static (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 12599
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 12601 '<FieldAccess for is_static_decl at ast.py:9642>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9642
--# expr-start 12600 '<FieldAccess for designated_type at ast.py:9642>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9642
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 12600
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_3.Info);
--# end
--# expr-done 12601
   Result_Var := Fld_4;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 12602
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 12603
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Is_Static_Subtype;
--# end
--# property-start SubtypeIndication.custom_id_text /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9645
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Custom_Id_Text
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_769 with Inline_Always;
      Cast_Expr : Bare_Subtype_Indication;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : String_Type;
Dyn_Var_Bind_Result : String_Type;
            procedure Finalizer_Scope_769 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_769;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12606 '<bind at ast.py:9645>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9645
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 12605 '<FieldAccess for canonical_fully_qualified_name at ast.py:9647>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9647
--# expr-start 12604 '<FieldAccess for designated_type at ast.py:9647>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9647
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 12604
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 12605
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 12606
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_769;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_769;
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Constrained_Subtype_Indication
   --
      procedure Initialize_Fields_For_Constrained_Subtype_Indication
        (Self : Bare_Constrained_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Initialize_Fields_For_Subtype_Indication
              (Self, Subtype_Indication_F_Has_Not_Null, Subtype_Indication_F_Name, Subtype_Indication_F_Constraint);
      end Initialize_Fields_For_Constrained_Subtype_Indication;
   --
   --  Primitives for Bare_Discrete_Subtype_Indication
   --
      procedure Initialize_Fields_For_Discrete_Subtype_Indication
        (Self : Bare_Discrete_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Initialize_Fields_For_Subtype_Indication
              (Self, Subtype_Indication_F_Has_Not_Null, Subtype_Indication_F_Name, Subtype_Indication_F_Constraint);
      end Initialize_Fields_For_Discrete_Subtype_Indication;
   --
   --  Primitives for Bare_Synthetic_Type_Expr
   --
      procedure Initialize_Fields_For_Synthetic_Type_Expr
        (Self : Bare_Synthetic_Type_Expr
         ; Synthetic_Type_Expr_F_Target_Type : Bare_Base_Type_Decl
        ) is
      begin
            Self.Synthetic_Type_Expr_F_Target_Type := Synthetic_Type_Expr_F_Target_Type;
      end Initialize_Fields_For_Synthetic_Type_Expr;
   function Synthetic_Type_Expr_F_Target_Type
     (Node : Bare_Synthetic_Type_Expr) return Bare_Base_Type_Decl
   is
   begin
         return Node.Synthetic_Type_Expr_F_Target_Type;
   end;
--# property-start SyntheticTypeExpr.designated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18478
pragma Warnings (Off, "is not referenced");
 function Synthetic_Type_Expr_P_Designated_Type
  (Node : Bare_Synthetic_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Synthetic_Type_Expr  := Bare_Synthetic_Type_Expr (Node);
     Ent : Internal_Entity_Synthetic_Type_Expr :=
       Internal_Entity_Synthetic_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1283 with Inline_Always;
      Block_Var : Internal_Entity_Generic_Decl;
Block_Var_1 : Env_Rebindings;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Fld : Bare_Base_Type_Decl;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Generic_Decl;
As_Entity : Internal_Entity_Generic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Info;
Fld_5 : Env_Rebindings;
Scope_Result_1 : Env_Rebindings;
Result_Var_1 : Env_Rebindings;
Fld_6 : Bare_Base_Type_Decl;
Fld_7 : Internal_Entity_Info;
Fld_8 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1283 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1283;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12623 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12612 '<as_bare_entity at ast.py:18495>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18495
--# expr-start 12611 '<Cast to GenericDecl at ast.py:18495>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18495
--# expr-start 12610 '<Find at ast.py:18493>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18493
   --# expr-start 12608 '<FieldAccess for parents at ast.py:18493>' Node_Parents /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18493
--# expr-start 12607 '<FieldAccess for target_type at ast.py:18493>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18493
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Type_Expr_F_Target_Type;
--# expr-done 12607
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Fld, True); 
--# expr-done 12608
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_369 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_369
      --# expr-start 12609 '<IsA GenericDecl at ast.py:18494>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18494
Is_A := Item_369 /= null 
and then Item_369.Kind in Ada_Generic_Decl; 
--# expr-done 12609
      if Is_A then
         Find_Result := Item_369; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 12610
   if Find_Result = null
      or else Find_Result.Kind in Ada_Generic_Decl
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12611
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 12612
Block_Var := If_Result; 
--# bind gd Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 12615 '<FieldAccess for unshed_rebindings at ast.py:18498>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18498
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12614 '<FieldAccess for rebindings at ast.py:18499>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18499
--# expr-start 12613 '<FieldAccess for info at ast.py:18499>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18499
Fld_1 := Ent.Info;
--# expr-done 12613
Fld_2 := Fld_1.Rebindings;
--# expr-done 12614
--# property-call-start BasicDecl.unshed_rebindings
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Var_Expr_1.Node, Rebindings => Fld_2, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 12615
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind fixed Var_Expr
--# expr-start 12617 '<FieldAccess for rebindings at ast.py:18501>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18501
--# expr-start 12616 '<FieldAccess for info at ast.py:18501>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18501
Fld_4 := Var_Expr.Info;
--# expr-done 12616
Fld_5 := Fld_4.Rebindings;
--# expr-done 12617
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := null;
end if;
Block_Var_1 := Result_Var_1; 
--# bind relevant_rebindings Block_Var_1
--# expr-start 12622 '<New BaseTypeDecl.entity at ???>' New_Struct_1 None
--# expr-start 12620 '<New EntityInfo at ast.py:18508>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18508
--# expr-start 12619 '<FieldAccess for from_rebound at ast.py:18511>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18511
--# expr-start 12618 '<FieldAccess for info at ast.py:18511>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18511
Fld_7 := Ent.Info;
--# expr-done 12618
Fld_8 := Fld_7.From_Rebound;
--# expr-done 12619
New_Struct := (From_Rebound => Fld_8, Md => No_Metadata, Rebindings => Block_Var_1); 
--# expr-done 12620
--# expr-start 12621 '<FieldAccess for target_type at ast.py:18507>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:18507
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Synthetic_Type_Expr_F_Target_Type;
--# expr-done 12621
New_Struct_1 := (Info => New_Struct, Node => Fld_6); 
--# expr-done 12622
Let_Result := New_Struct_1; 
--# end
--# expr-done 12623
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1283;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1283;
            raise;
      end;
   return Property_Result;
end Synthetic_Type_Expr_P_Designated_Type;
--# end
   --
   --  Primitives for Bare_Unconstrained_Array_Index
   --
      procedure Initialize_Fields_For_Unconstrained_Array_Index
        (Self : Bare_Unconstrained_Array_Index
         ; Unconstrained_Array_Index_F_Subtype_Indication : Bare_Subtype_Indication
        ) is
      begin
            Self.Unconstrained_Array_Index_F_Subtype_Indication := Unconstrained_Array_Index_F_Subtype_Indication;
      end Initialize_Fields_For_Unconstrained_Array_Index;
   function Unconstrained_Array_Index_F_Subtype_Indication
     (Node : Bare_Unconstrained_Array_Index) return Bare_Subtype_Indication
   is
   begin
         return Node.Unconstrained_Array_Index_F_Subtype_Indication;
   end;
--# property-start UnconstrainedArrayIndex.designated_type /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22347
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Index_P_Designated_Type
  (Node : Bare_Unconstrained_Array_Index
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Unconstrained_Array_Index  := Bare_Unconstrained_Array_Index (Node);
     Ent : Internal_Entity_Unconstrained_Array_Index :=
       Internal_Entity_Unconstrained_Array_Index'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12626 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12625 '<FieldAccess for designated_type at ast.py:22349>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22349
--# expr-start 12624 '<FieldAccess for subtype_indication at ast.py:22349>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:22349
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12624
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12625
Let_Result := Fld_1; 
--# end
--# expr-done 12626
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Index_P_Designated_Type;
--# end
   --
   --  Primitives for Bare_Until_Node
   --
--# property-start '[dispatcher]Until.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Until_Node_P_As_Bool
  (Node : Bare_Until_Node
  )
   return Boolean
is
   Self : Bare_Until_Node  := Bare_Until_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Until_Node (Self.Kind) is
               when Ada_Until_Absent =>
                  --# property-call-start Until.Absent.as_bool
                  Property_Result := Until_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Until_Present =>
                  --# property-call-start Until.Present.as_bool
                  Property_Result := Until_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Until_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Until_Absent
   --
--# property-start Until.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5340
pragma Warnings (Off, "is not referenced");
 function Until_Absent_P_As_Bool
  (Node : Bare_Until_Absent
  )
   return Boolean
is
   Self : Bare_Until_Absent  := Bare_Until_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Until_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Until_Present
   --
--# property-start Until.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5340
pragma Warnings (Off, "is not referenced");
 function Until_Present_P_As_Bool
  (Node : Bare_Until_Present
  )
   return Boolean
is
   Self : Bare_Until_Present  := Bare_Until_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Until_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Use_Clause
   --
--# property-start UseClause.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9297
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Xref_Entry_Point
  (Node : Bare_Use_Clause
  )
   return Boolean
is
   Self : Bare_Use_Clause  := Bare_Use_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Use_Clause_P_Xref_Entry_Point;
--# end
--# property-start UseClause.used_envs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9299
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Used_Envs
  (Node : Bare_Use_Clause
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Use_Clause  := Bare_Use_Clause (Node);
     Ent : Internal_Entity_Use_Clause :=
       Internal_Entity_Use_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_744 with Inline_Always;
            procedure Finalizer_Scope_3368 with Inline_Always;
            procedure Finalizer_Scope_3369 with Inline_Always;
            procedure Finalizer_Scope_3370 with Inline_Always;
      Match_0 : Internal_Entity_Use_Package_Clause;
Match_1 : Internal_Entity_Use_Type_Clause;
Fld : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name_List;
Fld_2 : Bare_Name_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_4 : Lexical_Env := Empty_Env;
Item_370 : Internal_Entity_Name;
Bare_Item_370 : Bare_Name;
Map_Result : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity_Use_Clause;
Cast_Result : Internal_Entity_Use_Package_Clause;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Use_Type_Clause;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_744 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_744;
            procedure Finalizer_Scope_3368 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_3368;
            procedure Finalizer_Scope_3369 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env_1);
            end Finalizer_Scope_3369;
            procedure Finalizer_Scope_3370 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3370;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12634 '<Block at ???>' Let_Result_2 None
--# scope-start
--# expr-start 12633 '<Match at ast.py:9305>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9305
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Use_Clause (Match_Prefix.Node.Kind) is
         when Ada_Use_Package_Clause =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Use_Package_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind upc Match_0
--# expr-start 12628 '<env_group at ast.py:9306>' Group_Env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9306
--# expr-start 12627 '<FieldAccess for designated_envs at ast.py:9306>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9306
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UsePackageClause.designated_envs
Fld := Libadalang.Implementation.Use_Package_Clause_P_Designated_Envs (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 12627
Group_Env := Group (Fld, No_Metadata); 
--# expr-done 12628
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3368;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Use_Type_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Use_Type_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind utc Match_1
--# expr-start 12632 '<env_group at ast.py:9308>' Group_Env_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9308
--# expr-start 12631 '<Map at ast.py:9308>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9308
   --# expr-start 12629 '<FieldAccess for types at ast.py:9308>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9308
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name_List (Node => Match_1.Node.Use_Type_Clause_F_Types, Info => Match_1.Info);
--# expr-done 12629
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_2;
      begin
         for Untyped_Item_370 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_370 := Untyped_Item_370; 
Is_Null := Bare_Item_370 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_370); 
   If_Result := As_Entity;
end if;
                  Item_370 := If_Result; 
   --# scope-start
               --# bind n Item_370
      --# expr-start 12630 '<FieldAccess for name_designated_type_env at ast.py:9308>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9308
      if Item_370.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type_env
Fld_4 := Libadalang.Implementation.Name_P_Name_Designated_Type_Env (Node => Item_370.Node, E_Info => Item_370.Info);
--# end
--# expr-done 12630
         declare
            Item_To_Append : constant Lexical_Env := Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3370;
         end loop;
      end;
      end if;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12631
Group_Env_1 := Group (Map_Result, No_Metadata); 
--# expr-done 12632
Let_Result_1 := Group_Env_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3369;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 12633
Let_Result_2 := Match_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 12634
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_744;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_744;
                     Finalizer_Scope_3368;
                     Finalizer_Scope_3369;
                     Finalizer_Scope_3370;
            raise;
      end;
   return Property_Result;
end Use_Clause_P_Used_Envs;
--# end
--# property-start UseClause.initial_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9311
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Initial_Env
  (Node : Bare_Use_Clause
  )
   return Internal_Designated_Env
is
   Self : Bare_Use_Clause  := Bare_Use_Clause (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_745 with Inline_Always;
      Sym : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Internal_Designated_Env := No_Designated_Env;
Let_Result : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_745 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_745;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12642 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12641 '<If at ast.py:9317>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9317
--# expr-start 12637 '<IsA CompilationUnit at ast.py:9318>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9318
--# expr-start 12636 '<parent at ast.py:9318>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9318
--# expr-start 12635 '<parent at ast.py:9318>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9318
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12635
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 12636
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 12637
if Is_A then
   --# expr-start 12639 '<New DesignatedEnv at ast.py:9319>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9319
--# expr-start 12638 '<Symbol Standard at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard); 
--# expr-done 12638
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Sym, Kind => Named_Env); 
--# expr-done 12639
   If_Result := New_Struct;
else
   --# expr-start 12640 '<New DesignatedEnv at ast.py:9320>' New_Struct_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9320
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => null, Kind => Current_Env); 
--# expr-done 12640
   If_Result := New_Struct_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 12641
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12642
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_745;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_745;
            raise;
      end;
   return Property_Result;
end Use_Clause_P_Initial_Env;
--# end
   --
   --  Primitives for Bare_Use_Package_Clause
   --
      procedure Use_Package_Clause_Pre_Env_Actions
        (Self            : Bare_Use_Package_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_18 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:9331");
      end;
      end;
      procedure Use_Package_Clause_Post_Env_Actions
        (Self : Bare_Use_Package_Clause; State : in out PLE_Node_State) is
      begin
      if Libadalang.Implementation.Ref_Cond_20 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_19 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Use_Package_Clause
        (Self : Bare_Use_Package_Clause
         ; Use_Package_Clause_F_Packages : Bare_Name_List
        ) is
      begin
            Self.Use_Package_Clause_F_Packages := Use_Package_Clause_F_Packages;
      end Initialize_Fields_For_Use_Package_Clause;
   function Use_Package_Clause_F_Packages
     (Node : Bare_Use_Package_Clause) return Bare_Name_List
   is
   begin
         return Node.Use_Package_Clause_F_Packages;
   end;
--# property-start UsePackageClause.designated_env /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9343
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Designated_Env
  (Node : Bare_Use_Package_Clause
      ; Index : Integer
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
     Ent : Internal_Entity_Use_Package_Clause :=
       Internal_Entity_Use_Package_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind index Index
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_746 with Inline_Always;
      Block_Var : Bare_Name;
Fld : Bare_Name_List;
Get_Result : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_746 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_746;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12651 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12644 '<at at ast.py:9349>' Get_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9349
--# expr-start 12643 '<FieldAccess for packages at ast.py:9349>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9349
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 12643
Get_Result := Get (Self, Fld, Index, True); 
--# expr-done 12644
Block_Var := Get_Result; 
--# bind pkg Block_Var
--# expr-start 12650 '<bind at ast.py:9350>' Dyn_Var_Bind_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9350
--# expr-start 12645 '<FieldAccess for node_env at ast.py:9351>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9351
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12645
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 12649 '<bind at ast.py:9352>' Dyn_Var_Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9352
--# expr-start 12646 '<FieldAccess for origin_node at ast.py:9353>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9353
      if Block_Var = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Block_Var);
--# end
--# expr-done 12646
Bound_Origin := Fld_2; 
--# expr-start 12648 '<FieldAccess for designated_env at ast.py:9354>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9354
--# expr-start 12647 '<as_bare_entity at ast.py:9354>' If_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9354
Is_Null := Block_Var = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Block_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 12647
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => If_Result.Node, Env => Bound_Env, Origin => Bound_Origin, No_Visibility => False, E_Info => If_Result.Info);
--# end
--# expr-done 12648
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 12649
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 12650
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 12651
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_746;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_746;
            raise;
      end;
   return Property_Result;
end Use_Package_Clause_P_Designated_Env;
--# end
--# property-start UsePackageClause.designated_envs /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9358
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Designated_Envs
  (Node : Bare_Use_Package_Clause
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
     Ent : Internal_Entity_Use_Package_Clause :=
       Internal_Entity_Use_Package_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_747 with Inline_Always;
            procedure Finalizer_Scope_3371 with Inline_Always;
      Fld : Bare_Name_List;
Fld_1 : Lexical_Env := Empty_Env;
Item_371 : Bare_Name;
Index_372 : Integer;
Map_Result : Lexical_Env_Array_Access;
Let_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_747 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_747;
            procedure Finalizer_Scope_3371 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3371;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12655 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12654 '<Map at ast.py:9366>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9366
   --# expr-start 12652 '<FieldAccess for packages at ast.py:9366>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9366
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 12652
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Lexical_Env_Array (0);
      else
         Index_372 := 0;
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_371 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_371 := Untyped_Item_371; 
   --# scope-start
               --# bind _ Item_371
               --# bind i Index_372
      --# expr-start 12653 '<FieldAccess for designated_env at ast.py:9367>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9367
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UsePackageClause.designated_env
Fld_1 := Libadalang.Implementation.Use_Package_Clause_P_Designated_Env (Node => Ent.Node, Index => Index_372, E_Info => Ent.Info);
--# end
--# expr-done 12653
         declare
            Item_To_Append : constant Lexical_Env := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3371;
               Index_372 := Index_372 + 1;
         end loop;
      end;
      end if;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12654
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12655
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_747;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_747;
                     Finalizer_Scope_3371;
            raise;
      end;
   return Property_Result;
end Use_Package_Clause_P_Designated_Envs;
--# end
--# property-start UsePackageClause.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9370
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Xref_Equation
  (Node : Bare_Use_Package_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
     Ent : Internal_Entity_Use_Package_Clause :=
       Internal_Entity_Use_Package_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_748 with Inline_Always;
            procedure Finalizer_Scope_3372 with Inline_Always;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_373 : Internal_Entity_Name;
Bare_Item_373 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_748 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_748;
            procedure Finalizer_Scope_3372 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3372;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12659 '<LogicAll at ast.py:9371>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9371
--# expr-start 12658 '<Map at ast.py:9371>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9371
   --# expr-start 12656 '<FieldAccess for packages at ast.py:9371>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9371
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Use_Package_Clause_F_Packages, Info => Ent.Info);
--# expr-done 12656
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_373 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_373 := Untyped_Item_373; 
Is_Null := Bare_Item_373 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_373); 
   If_Result := As_Entity;
end if;
                  Item_373 := If_Result; 
   --# scope-start
               --# bind p Item_373
      --# expr-start 12657 '<FieldAccess for xref_no_overloading at ast.py:9371>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9371
      if Item_373.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_373.Node, Env => Env, Origin => Origin, E_Info => Item_373.Info);
--# end
--# expr-done 12657
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3372;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12658
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9371") else null)); 
--# expr-done 12659
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_748;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_748;
                     Finalizer_Scope_3372;
            raise;
      end;
   return Property_Result;
end Use_Package_Clause_P_Xref_Equation;
--# end
--# property-start UsePackageClause._initial_env_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9331
pragma Warnings (Off, "is not referenced");
 function Initial_Env_18
  (Node : Bare_Use_Package_Clause
  )
   return Internal_Designated_Env
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1545 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1545 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1545;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12660 '<FieldAccess for initial_env at ast.py:9331>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9331
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.initial_env
Fld := Libadalang.Implementation.Use_Clause_P_Initial_Env (Node => Self);
--# end
--# expr-done 12660
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1545;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1545;
            raise;
      end;
   return Property_Result;
end Initial_Env_18;
--# end
--# property-start UsePackageClause._ref_env_nodes_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9337
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_19
  (Node : Bare_Use_Package_Clause
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1546 with Inline_Always;
      Fld : Bare_Name_List;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Item_374 : Bare_Name;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1546 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1546;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12663 '<Map at ast.py:9337>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9337
   --# expr-start 12661 '<FieldAccess for packages at ast.py:9337>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9337
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 12661
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_374 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_374 := Untyped_Item_374; 
   --# scope-start
               --# bind n Item_374
      --# expr-start 12662 '<Cast to AdaNode at ast.py:9337>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9337
Cast_Expr := Item_374; 
      Cast_Result := Cast_Expr;
--# expr-done 12662
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12663
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1546;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1546;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_19;
--# end
--# property-start UsePackageClause._ref_cond_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9339
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_20
  (Node : Bare_Use_Package_Clause
  )
   return Boolean
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12667 '<Not at ast.py:9339>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9339
--# expr-start 12666 '<IsA CompilationUnit at ast.py:9339>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9339
--# expr-start 12665 '<parent at ast.py:9339>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9339
--# expr-start 12664 '<parent at ast.py:9339>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9339
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12664
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 12665
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 12666
Not_Val := not (Is_A); 
--# expr-done 12667
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_20;
--# end
   --
   --  Primitives for Bare_Use_Type_Clause
   --
      procedure Use_Type_Clause_Pre_Env_Actions
        (Self            : Bare_Use_Type_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_21 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:9383");
      end;
      end;
      procedure Use_Type_Clause_Post_Env_Actions
        (Self : Bare_Use_Type_Clause; State : in out PLE_Node_State) is
      begin
      if Libadalang.Implementation.Ref_Cond_23 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_22 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Use_Type_Clause
        (Self : Bare_Use_Type_Clause
         ; Use_Type_Clause_F_Has_All : Bare_All_Node
         ; Use_Type_Clause_F_Types : Bare_Name_List
        ) is
      begin
            Self.Use_Type_Clause_F_Has_All := Use_Type_Clause_F_Has_All;
            Self.Use_Type_Clause_F_Types := Use_Type_Clause_F_Types;
      end Initialize_Fields_For_Use_Type_Clause;
   function Use_Type_Clause_F_Has_All
     (Node : Bare_Use_Type_Clause) return Bare_All_Node
   is
   begin
         return Node.Use_Type_Clause_F_Has_All;
   end;
   function Use_Type_Clause_F_Types
     (Node : Bare_Use_Type_Clause) return Bare_Name_List
   is
   begin
         return Node.Use_Type_Clause_F_Types;
   end;
--# property-start UseTypeClause.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9395
pragma Warnings (Off, "is not referenced");
 function Use_Type_Clause_P_Xref_Equation
  (Node : Bare_Use_Type_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Use_Type_Clause  := Bare_Use_Type_Clause (Node);
     Ent : Internal_Entity_Use_Type_Clause :=
       Internal_Entity_Use_Type_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_749 with Inline_Always;
            procedure Finalizer_Scope_3374 with Inline_Always;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_375 : Internal_Entity_Name;
Bare_Item_375 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_749 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_749;
            procedure Finalizer_Scope_3374 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3374;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12671 '<LogicAll at ast.py:9396>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9396
--# expr-start 12670 '<Map at ast.py:9396>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9396
   --# expr-start 12668 '<FieldAccess for types at ast.py:9396>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9396
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Use_Type_Clause_F_Types, Info => Ent.Info);
--# expr-done 12668
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_375 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_375 := Untyped_Item_375; 
Is_Null := Bare_Item_375 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_375); 
   If_Result := As_Entity;
end if;
                  Item_375 := If_Result; 
   --# scope-start
               --# bind p Item_375
      --# expr-start 12669 '<FieldAccess for xref_no_overloading at ast.py:9396>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9396
      if Item_375.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_375.Node, Env => Env, Origin => Origin, E_Info => Item_375.Info);
--# end
--# expr-done 12669
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3374;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12670
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9396") else null)); 
--# expr-done 12671
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_749;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_749;
                     Finalizer_Scope_3374;
            raise;
      end;
   return Property_Result;
end Use_Type_Clause_P_Xref_Equation;
--# end
--# property-start UseTypeClause._initial_env_21 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9383
pragma Warnings (Off, "is not referenced");
 function Initial_Env_21
  (Node : Bare_Use_Type_Clause
  )
   return Internal_Designated_Env
is
   Self : Bare_Use_Type_Clause  := Bare_Use_Type_Clause (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1548 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1548 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1548;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12672 '<FieldAccess for initial_env at ast.py:9383>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9383
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.initial_env
Fld := Libadalang.Implementation.Use_Clause_P_Initial_Env (Node => Self);
--# end
--# expr-done 12672
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1548;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1548;
            raise;
      end;
   return Property_Result;
end Initial_Env_21;
--# end
--# property-start UseTypeClause._ref_env_nodes_22 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9389
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_22
  (Node : Bare_Use_Type_Clause
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Use_Type_Clause  := Bare_Use_Type_Clause (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1549 with Inline_Always;
      Fld : Bare_Name_List;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Item_376 : Bare_Name;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1549 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1549;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12675 '<Map at ast.py:9389>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9389
   --# expr-start 12673 '<FieldAccess for types at ast.py:9389>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9389
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Use_Type_Clause_F_Types;
--# expr-done 12673
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_376 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_376 := Untyped_Item_376; 
   --# scope-start
               --# bind n Item_376
      --# expr-start 12674 '<Cast to AdaNode at ast.py:9389>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9389
Cast_Expr := Item_376; 
      Cast_Result := Cast_Expr;
--# expr-done 12674
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12675
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1549;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1549;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_22;
--# end
--# property-start UseTypeClause._ref_cond_23 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9391
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_23
  (Node : Bare_Use_Type_Clause
  )
   return Boolean
is
   Self : Bare_Use_Type_Clause  := Bare_Use_Type_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12679 '<Not at ast.py:9391>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9391
--# expr-start 12678 '<IsA CompilationUnit at ast.py:9391>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9391
--# expr-start 12677 '<parent at ast.py:9391>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9391
--# expr-start 12676 '<parent at ast.py:9391>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9391
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12676
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 12677
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 12678
Not_Val := not (Is_A); 
--# expr-done 12679
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_23;
--# end
   --
   --  Primitives for Bare_Value_Sequence
   --
      procedure Initialize_Fields_For_Value_Sequence
        (Self : Bare_Value_Sequence
         ; Value_Sequence_F_Iter_Assoc : Bare_Iterated_Assoc
        ) is
      begin
            Self.Value_Sequence_F_Iter_Assoc := Value_Sequence_F_Iter_Assoc;
      end Initialize_Fields_For_Value_Sequence;
   function Value_Sequence_F_Iter_Assoc
     (Node : Bare_Value_Sequence) return Bare_Iterated_Assoc
   is
   begin
         return Node.Value_Sequence_F_Iter_Assoc;
   end;
--# property-start ValueSequence.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19767
pragma Warnings (Off, "is not referenced");
 function Value_Sequence_P_Xref_Equation
  (Node : Bare_Value_Sequence
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Value_Sequence  := Bare_Value_Sequence (Node);
     Ent : Internal_Entity_Value_Sequence :=
       Internal_Entity_Value_Sequence'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1368 with Inline_Always;
      Fld : Internal_Entity_Iterated_Assoc;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1368 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1368;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12682 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12681 '<FieldAccess for xref_equation_for_reduce at ast.py:19772>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19772
--# expr-start 12680 '<FieldAccess for iter_assoc at ast.py:19772>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:19772
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Iterated_Assoc (Node => Ent.Node.Value_Sequence_F_Iter_Assoc, Info => Ent.Info);
--# expr-done 12680
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IteratedAssoc.xref_equation_for_reduce
Fld_1 := Libadalang.Implementation.Iterated_Assoc_P_Xref_Equation_For_Reduce (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12681
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 12682
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1368;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1368;
            raise;
      end;
   return Property_Result;
end Value_Sequence_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Variant
   --
      procedure Initialize_Fields_For_Variant
        (Self : Bare_Variant
         ; Variant_F_Choices : Bare_Alternatives_List
         ; Variant_F_Components : Bare_Component_List
        ) is
      begin
            Self.Variant_F_Choices := Variant_F_Choices;
            Self.Variant_F_Components := Variant_F_Components;
      end Initialize_Fields_For_Variant;
   function Variant_F_Choices
     (Node : Bare_Variant) return Bare_Alternatives_List
   is
   begin
         return Node.Variant_F_Choices;
   end;
   function Variant_F_Components
     (Node : Bare_Variant) return Bare_Component_List
   is
   begin
         return Node.Variant_F_Components;
   end;
--# property-start Variant.matches /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4883
pragma Warnings (Off, "is not referenced");
 function Variant_P_Matches
  (Node : Bare_Variant
      ; Expr : Internal_Entity_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Variant  := Bare_Variant (Node);
     Ent : Internal_Entity_Variant :=
       Internal_Entity_Variant'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind expr Expr
   Property_Result : Boolean;
            procedure Finalizer_Scope_306 with Inline_Always;
      Block_Var : Big_Integer_Type;
Fld : Big_Integer_Type;
Fld_1 : Internal_Entity_Alternatives_List;
Fld_2 : Bare_Alternatives_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_4 : Boolean;
Item_377 : Internal_Entity;
Bare_Item_377 : Bare_Ada_Node;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_306 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_306;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12687 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12683 '<FieldAccess for eval_as_int at ast.py:4889>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4889
      if Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Expr.Node, E_Info => Expr.Info);
--# end
--# expr-done 12683
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind expr_val Block_Var
--# expr-start 12686 '<AnyQuantifier at ast.py:4891>' Quantifier_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4891
   --# expr-start 12684 '<FieldAccess for choices at ast.py:4891>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4891
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Variant_F_Choices, Info => Ent.Info);
--# expr-done 12684
Fld_2 := Fld_1.Node;
      Quantifier_Result := False;
      if Fld_2 = null then
      null;
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_2;
      begin
         for Untyped_Item_377 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_377 := Untyped_Item_377; 
Is_Null := Bare_Item_377 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_377); 
   If_Result := As_Entity;
end if;
                  Item_377 := If_Result; 
   --# scope-start
               --# bind c Item_377
      --# expr-start 12685 '<FieldAccess for choice_match at ast.py:4892>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4892
      if Item_377.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.choice_match
Fld_4 := Libadalang.Implementation.Ada_Node_P_Choice_Match (Node => Item_377.Node, Value => Block_Var, E_Info => Item_377.Info);
--# end
--# expr-done 12685
      Quantifier_Result := Fld_4;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 12686
Let_Result := Quantifier_Result; 
--# end
--# expr-done 12687
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_306;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_306;
            raise;
      end;
   return Property_Result;
end Variant_P_Matches;
--# end
   --
   --  Primitives for Bare_Variant_Part
   --
      procedure Initialize_Fields_For_Variant_Part
        (Self : Bare_Variant_Part
         ; Variant_Part_F_Discr_Name : Bare_Identifier
         ; Variant_Part_F_Variant : Bare_Variant_List
        ) is
      begin
            Self.Variant_Part_F_Discr_Name := Variant_Part_F_Discr_Name;
            Self.Variant_Part_F_Variant := Variant_Part_F_Variant;
      end Initialize_Fields_For_Variant_Part;
   function Variant_Part_F_Discr_Name
     (Node : Bare_Variant_Part) return Bare_Identifier
   is
   begin
         return Node.Variant_Part_F_Discr_Name;
   end;
   function Variant_Part_F_Variant
     (Node : Bare_Variant_Part) return Bare_Variant_List
   is
   begin
         return Node.Variant_Part_F_Variant;
   end;
--# property-start VariantPart.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4906
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Xref_Entry_Point
  (Node : Bare_Variant_Part
  )
   return Boolean
is
   Self : Bare_Variant_Part  := Bare_Variant_Part (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Variant_Part_P_Xref_Entry_Point;
--# end
--# property-start VariantPart.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4908
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Xref_Equation
  (Node : Bare_Variant_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Variant_Part  := Bare_Variant_Part (Node);
     Ent : Internal_Entity_Variant_Part :=
       Internal_Entity_Variant_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_308 with Inline_Always;
            procedure Finalizer_Scope_3377 with Inline_Always;
            procedure Finalizer_Scope_3378 with Inline_Always;
            procedure Finalizer_Scope_3379 with Inline_Always;
            procedure Finalizer_Scope_3381 with Inline_Always;
            procedure Finalizer_Scope_3382 with Inline_Always;
      Block_Var : Boolean;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Var_Expr : Internal_Entity_Name;
Fld : Internal_Entity_Identifier;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Variant_List;
Fld_3 : Bare_Variant_List;
Fld_4 : Internal_Entity_Info;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Fld_5 : Internal_Entity_Alternatives_List;
Fld_6 : Bare_Alternatives_List;
Fld_7 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Bare_Identifier;
Fld_11 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_12 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_14 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
Not_Val : Boolean;
If_Result_2 : Logic_Equation;
Fld_15 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_5 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Match_Result : Logic_Equation;
Item_379 : Internal_Entity;
Bare_Item_379 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Item_378 : Internal_Entity_Variant;
Bare_Item_378 : Bare_Variant;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_308 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_308;
            procedure Finalizer_Scope_3377 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_3377;
            procedure Finalizer_Scope_3378 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_3378;
            procedure Finalizer_Scope_3379 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_3379;
            procedure Finalizer_Scope_3381 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_3381;
            procedure Finalizer_Scope_3382 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_3382;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12713 '<Block at ???>' Let_Result_4 None
--# scope-start
--# expr-start 12689 '<FieldAccess for resolve_names_internal at ast.py:4910>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4910
--# expr-start 12688 '<FieldAccess for discr_name at ast.py:4910>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4910
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Variant_Part_F_Discr_Name, Info => Ent.Info);
--# expr-done 12688
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12689
Block_Var := Fld_1; 
--# expr-start 12712 '<LogicAll at ast.py:4912>' Logic_Boolean_Op_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4912
--# expr-start 12711 '<Map at ast.py:4912>' Map_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4912
   --# expr-start 12690 '<FieldAccess for variant at ast.py:4912>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4912
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Variant_List (Node => Ent.Node.Variant_Part_F_Variant, Info => Ent.Info);
--# expr-done 12690
Fld_3 := Fld_2.Node;
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_3 = null then
      Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_3;
      begin
         for Untyped_Item_378 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_378 := Untyped_Item_378; 
Is_Null := Bare_Item_378 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Bare_Item_378); 
   If_Result := As_Entity;
end if;
                  Item_378 := If_Result; 
   --# scope-start
               --# bind var Item_378
      --# expr-start 12710 '<LogicAll at ast.py:4913>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4913
--# expr-start 12709 '<Map at ast.py:4913>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4913
   --# expr-start 12691 '<FieldAccess for choices at ast.py:4913>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4913
      if Item_378.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Alternatives_List (Node => Item_378.Node.Variant_F_Choices, Info => Item_378.Info);
--# expr-done 12691
Fld_6 := Fld_5.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_6 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_6;
      begin
         for Untyped_Item_379 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_379 := Untyped_Item_379; 
Is_Null_1 := Bare_Item_379 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_7 := Fld_5.Info;
As_Entity_1 := (Info => Fld_7, Node => Bare_Item_379); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_379 := If_Result_1; 
   --# scope-start
               --# bind c Item_379
      --# expr-start 12708 '<Match at ast.py:4913>' Match_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4913
      if Item_379.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_379; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind e Match_0
--# expr-start 12706 '<If at ast.py:4915>' If_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4915
--# expr-start 12695 '<Not at ast.py:4916>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4916
--# expr-start 12694 '<IsNull at ast.py:4916>' Is_Null_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4916
--# expr-start 12692 '<Cast to Name at ast.py:4916>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4916
Cast_Expr_1 := Match_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 12692
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Name then
   --# scope-start
--# expr-start 12693 '<FieldAccess for name_designated_type at ast.py:4916>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4916
      if Var_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_14 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 12693
Scope_Result := Fld_14;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Is_Null_2 := Result_Var.Node = null; 
--# expr-done 12694
Not_Val := not (Is_Null_2); 
--# expr-done 12695
if Not_Val then
   --# expr-start 12697 '<FieldAccess for xref_no_overloading at ast.py:4918>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4918
--# expr-start 12696 '<Cast to Name at ast.py:4918>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4918
Cast_Expr := Match_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 12696
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result.Node, Env => Env, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 12697
   If_Result_2 := Fld_8;
else
   --# expr-start 12705 '<And at ast.py:4920>' And_Pred_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4920
--# expr-start 12703 '<And at ast.py:4920>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4920
--# expr-start 12701 '<Bind at ast.py:4920>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4920
--# expr-start 12698 '<FieldAccess for expected_type_var at ast.py:4920>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4920
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Match_0.Node);
--# expr-done 12698
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 12700 '<FieldAccess for type_val at ast.py:4920>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4920
--# expr-start 12699 '<FieldAccess for discr_name at ast.py:4920>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4920
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Variant_Part_F_Discr_Name;
--# expr-done 12699
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_11 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_10);
--# end
--# expr-done 12700
Bind_Result := Solver.Create_Assign (Fld_9, Fld_11, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4920") else null)); 
--# expr-done 12701
--# expr-start 12702 '<FieldAccess for sub_equation at ast.py:4921>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4921
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 12702
And_Pred := Create_And (Bind_Result, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4920") else null)); 
--# expr-done 12703
--# expr-start 12704 '<FieldAccess for matches_expected_type at ast.py:4922>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4922
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Match_0.Node, Origin => Origin);
--# end
--# expr-done 12704
And_Pred_1 := Create_And (And_Pred, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4920") else null)); 
--# expr-done 12705
   If_Result_2 := And_Pred_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 12706
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3379;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind t Match_1
--# expr-start 12707 '<FieldAccess for xref_equation at ast.py:4927>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4927
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_15 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 12707
Let_Result_1 := Fld_15; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3381;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4929") else null)); 
Let_Result_2 := True_Rel; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3382;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;
--# expr-done 12708
         declare
            Item_To_Append : constant Logic_Equation := Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3378;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12709
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4913") else null)); 
--# expr-done 12710
         declare
            Item_To_Append : constant Logic_Equation := Logic_Boolean_Op;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3377;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 12711
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:4912") else null)); 
--# expr-done 12712
Let_Result_4 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 12713
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_308;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_308;
                     Finalizer_Scope_3377;
                     Finalizer_Scope_3378;
                     Finalizer_Scope_3379;
                     Finalizer_Scope_3381;
                     Finalizer_Scope_3382;
            raise;
      end;
   return Property_Result;
end Variant_Part_P_Xref_Equation;
--# end
--# property-start VariantPart.get_components /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4935
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Get_Components
  (Node : Bare_Variant_Part
      ; Discriminants : Internal_Param_Match_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Variant_Part  := Bare_Variant_Part (Node);
     Ent : Internal_Entity_Variant_Part :=
       Internal_Entity_Variant_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind discriminants Discriminants
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_309 with Inline_Always;
            procedure Finalizer_Scope_3387 with Inline_Always;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Internal_Entity_Variant;
Var_Expr : Internal_Param_Match;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Fld_2 : Bare_Identifier;
Sym : Symbol_Type;
Fld_3 : Boolean;
Find_Result : Internal_Param_Match;
Fld_4 : Internal_Single_Actual;
Fld_5 : Internal_Entity_Basic_Assoc;
Fld_6 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Fld_7 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result_1 : Internal_Entity;
Cast_Result : Internal_Entity_Concrete_Type_Decl;
Fld_8 : Internal_Entity_Discriminant_Part;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_1 : Internal_Entity_Discriminant_Spec;
Fld_10 : Internal_Entity_Defining_Name_Array_Access;
Fld_11 : Internal_Entity_Name;
Fld_12 : Bare_Identifier;
Sym_1 : Symbol_Type;
Fld_13 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Is_Equal : Boolean;
Not_Val : Boolean;
Find_Result_2 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_2 : Internal_Entity_Discriminant_Spec;
Fld_14 : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Fld_15 : Internal_Entity_Variant_List;
Fld_16 : Bare_Variant_List;
Fld_17 : Internal_Entity_Info;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Fld_18 : Boolean;
Item_384 : Internal_Entity_Variant;
Bare_Item_384 : Bare_Variant;
Find_Result_3 : Internal_Entity_Variant;
Fld_19 : Internal_Entity_Component_List;
Fld_20 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_309 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_309;
            procedure Finalizer_Scope_3387 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3387;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12746 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12719 '<Find at ast.py:4942>' Find_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4942
      Find_Result := No_Param_Match;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Discriminants;
      begin
         for Item_380 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_380
      --# expr-start 12718 '<FieldAccess for name_is at ast.py:4943>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4943
--# expr-start 12715 '<FieldAccess for name at ast.py:4943>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4943
--# expr-start 12714 '<FieldAccess for formal at ast.py:4943>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4943
Fld := Item_380.Formal;
--# expr-done 12714
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 12715
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12717 '<GetSymbol at ast.py:4943>' Sym /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4943
--# expr-start 12716 '<FieldAccess for discr_name at ast.py:4943>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4943
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Variant_Part_F_Discr_Name;
--# expr-done 12716
Sym := Get_Symbol (Fld_2); 
--# expr-done 12717
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1.Node, Sym => Sym);
--# end
--# expr-done 12718
      if Fld_3 then
         Find_Result := Item_380; 
      end if;
   --# end
               exit when Fld_3;
         end loop;
      end;
--# expr-done 12719
Var_Expr := Find_Result;
if Var_Expr /= No_Param_Match then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 12722 '<FieldAccess for expr at ast.py:4947>' Fld_6 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4947
--# expr-start 12721 '<FieldAccess for assoc at ast.py:4947>' Fld_5 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4947
--# expr-start 12720 '<FieldAccess for actual at ast.py:4947>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4947
Fld_4 := Var_Expr.Actual;
--# expr-done 12720
Fld_5 := Fld_4.Assoc;
--# expr-done 12721
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 12722
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 12740 '<FieldAccess for default_expr at ast.py:4957>' Fld_14 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4957
--# expr-start 12739 '<Cast to DiscriminantSpec at ast.py:4957>' Cast_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4957
--# expr-start 12738 '<Find at ast.py:4953>' Find_Result_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4953
   --# expr-start 12728 '<FieldAccess for abstract_formal_params at ast.py:4952>' Fld_9 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4952
--# expr-start 12727 '<FieldAccess for discriminants at ast.py:4952>' Fld_8 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4952
--# expr-start 12726 '<Cast to ConcreteTypeDecl at ast.py:4952>' Cast_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4952
--# expr-start 12725 '<Find at ast.py:4950>' Find_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4950
   --# expr-start 12723 '<FieldAccess for parents at ast.py:4950>' Fld_7 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4950
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 12723
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_7;
      begin
         for Item_382 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_382
      --# expr-start 12724 '<IsA ConcreteTypeDecl at ast.py:4951>' Is_A /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4951
Is_A := Item_382.Node /= null 
and then Item_382.Node.Kind in Ada_Concrete_Type_Decl_Range; 
--# expr-done 12724
      if Is_A then
         Find_Result_1 := Item_382; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 12725
   if Find_Result_1.Node = null
      or else Find_Result_1.Node.Kind in Ada_Concrete_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Concrete_Type_Decl
        (Node => Find_Result_1.Node,
         Info => Find_Result_1.Info);
   else
         Cast_Result := No_Entity_Concrete_Type_Decl;
   end if;
--# expr-done 12726
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Discriminant_Part (Node => Cast_Result.Node.Type_Decl_F_Discriminants, Info => Cast_Result.Info);
--# expr-done 12727
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 12728
      Find_Result_2 := No_Entity_Base_Formal_Param_Decl;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Fld_9;
      begin
         for Item_381 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_381
      --# expr-start 12737 '<Not at ast.py:4954>' Not_Val /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4954
--# expr-start 12736 '<IsNull at ast.py:4956>' Is_Equal /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4956
--# expr-start 12735 '<Filter at ast.py:4954>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4954
   --# expr-start 12730 '<FieldAccess for defining_names at ast.py:4954>' Fld_10 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4954
--# expr-start 12729 '<Cast to DiscriminantSpec at ast.py:4954>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4954
Cast_Expr := Item_381; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Discriminant_Spec_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Discriminant_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Discriminant_Spec;
   end if;
--# expr-done 12729
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 12730
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_10;
      begin
         for Item_383 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_383
         --# expr-start 12734 '<FieldAccess for name_is at ast.py:4955>' Fld_13 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4955
--# expr-start 12731 '<FieldAccess for name at ast.py:4955>' Fld_11 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4955
      if Item_383.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Item_383.Node.Defining_Name_F_Name, Info => Item_383.Info);
--# expr-done 12731
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12733 '<GetSymbol at ast.py:4955>' Sym_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4955
--# expr-start 12732 '<FieldAccess for discr_name at ast.py:4955>' Fld_12 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4955
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Variant_Part_F_Discr_Name;
--# expr-done 12732
Sym_1 := Get_Symbol (Fld_12); 
--# expr-done 12733
--# property-call-start Name.name_is
Fld_13 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_11.Node, Sym => Sym_1);
--# end
--# expr-done 12734
         if Fld_13 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_383;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12735
Is_Equal := Equivalent (Map_Result, No_Internal_Entity_Defining_Name_Array_Type); 
--# expr-done 12736
Not_Val := not (Is_Equal); 
--# expr-done 12737
      if Not_Val then
         Find_Result_2 := Item_381; 
      end if;
   --# end
      Finalizer_Scope_3387;
               exit when Not_Val;
         end loop;
      end;
--# expr-done 12738
   if Find_Result_2.Node = null
      or else Find_Result_2.Node.Kind in Ada_Discriminant_Spec_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Discriminant_Spec
        (Node => Find_Result_2.Node,
         Info => Find_Result_2.Info);
   else
         Cast_Result_2 := No_Entity_Discriminant_Spec;
   end if;
--# expr-done 12739
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Cast_Result_2.Node.Discriminant_Spec_F_Default_Expr, Info => Cast_Result_2.Info);
--# expr-done 12740
   Result_Var := Fld_14;
end if;
Block_Var := Result_Var; 
--# bind discr Block_Var
--# expr-start 12743 '<Find at ast.py:4962>' Find_Result_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4962
   --# expr-start 12741 '<FieldAccess for variant at ast.py:4962>' Fld_15 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4962
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Variant_List (Node => Ent.Node.Variant_Part_F_Variant, Info => Ent.Info);
--# expr-done 12741
Fld_16 := Fld_15.Node;
      Find_Result_3 := No_Entity_Variant;
      if Fld_16 = null then
      Find_Result_3 := No_Entity_Variant;
      else
      declare
         Collection : constant Bare_Variant_List := Fld_16;
      begin
         for Untyped_Item_384 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_384 := Untyped_Item_384; 
Is_Null := Bare_Item_384 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_17 := Fld_15.Info;
As_Entity := (Info => Fld_17, Node => Bare_Item_384); 
   If_Result := As_Entity;
end if;
                  Item_384 := If_Result; 
   --# scope-start
               --# bind v Item_384
      --# expr-start 12742 '<FieldAccess for matches at ast.py:4962>' Fld_18 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4962
      if Item_384.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Variant.matches
Fld_18 := Libadalang.Implementation.Variant_P_Matches (Node => Item_384.Node, Expr => Block_Var, E_Info => Item_384.Info);
--# end
--# expr-done 12742
      if Fld_18 then
         Find_Result_3 := Item_384; 
      end if;
   --# end
               exit when Fld_18;
         end loop;
      end;
      end if;
--# expr-done 12743
Block_Var_1 := Find_Result_3; 
--# bind variant Block_Var_1
--# expr-start 12745 '<FieldAccess for abstract_formal_params_impl at ast.py:4967>' Fld_20 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4967
--# expr-start 12744 '<FieldAccess for components at ast.py:4967>' Fld_19 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:4967
      if Block_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Component_List (Node => Block_Var_1.Node.Variant_F_Components, Info => Block_Var_1.Info);
--# expr-done 12744
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_20 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Fld_19.Node, Discriminants => Discriminants, Include_Discriminants => False, Recurse => False, E_Info => Fld_19.Info);
--# end
--# expr-done 12745
Let_Result := Fld_20; Inc_Ref (Let_Result);
--# end
--# expr-done 12746
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_309;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_309;
                     Finalizer_Scope_3387;
            raise;
      end;
   return Property_Result;
end Variant_Part_P_Get_Components;
--# end
   --
   --  Primitives for Bare_With_Clause
   --
      procedure With_Clause_Pre_Env_Actions
        (Self            : Bare_With_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Initial_Env_17 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "ast.py:9288");
      end;
      end;
      procedure Initialize_Fields_For_With_Clause
        (Self : Bare_With_Clause
         ; With_Clause_F_Has_Limited : Bare_Limited_Node
         ; With_Clause_F_Has_Private : Bare_Private_Node
         ; With_Clause_F_Packages : Bare_Name_List
        ) is
      begin
            Self.With_Clause_F_Has_Limited := With_Clause_F_Has_Limited;
            Self.With_Clause_F_Has_Private := With_Clause_F_Has_Private;
            Self.With_Clause_F_Packages := With_Clause_F_Packages;
      end Initialize_Fields_For_With_Clause;
   function With_Clause_F_Has_Limited
     (Node : Bare_With_Clause) return Bare_Limited_Node
   is
   begin
         return Node.With_Clause_F_Has_Limited;
   end;
   function With_Clause_F_Has_Private
     (Node : Bare_With_Clause) return Bare_Private_Node
   is
   begin
         return Node.With_Clause_F_Has_Private;
   end;
   function With_Clause_F_Packages
     (Node : Bare_With_Clause) return Bare_Name_List
   is
   begin
         return Node.With_Clause_F_Packages;
   end;
--# property-start WithClause.child_unit_xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9267
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Child_Unit_Xref_Equation
  (Node : Bare_With_Clause
      ; Name : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_With_Clause  := Bare_With_Clause (Node);
     Ent : Internal_Entity_With_Clause :=
       Internal_Entity_With_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_740 with Inline_Always;
            procedure Finalizer_Scope_3391 with Inline_Always;
      Block_Var : Logic_Equation;
Var_Expr : Bare_Compilation_Unit;
Var_Expr_1 : Bare_Dotted_Name;
Fld : Logic_Var;
Fld_1 : Bare_Compilation_Unit;
Fld_2 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Result_Var : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Cast_Expr : Bare_Name;
Cast_Result_1 : Bare_Dotted_Name;
Fld_3 : Bare_Name;
Fld_4 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_740 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_740;
            procedure Finalizer_Scope_3391 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3391;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12755 '<Block at ???>' Let_Result None
--# scope-start
--# expr-start 12750 '<Bind at ast.py:9274>' Bind_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9274
--# expr-start 12747 '<FieldAccess for ref_var at ast.py:9274>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9274
      if Name = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Name);
--# end
--# expr-done 12747
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 12748 '<FieldAccess for withed_unit_helper at ast.py:9274>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9274
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.withed_unit_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Withed_Unit_Helper (Node => Self, Unit_Name => Name);
--# end
--# expr-done 12748
Var_Expr := Fld_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 12749 '<FieldAccess for decl at ast.py:9274>' Fld_2 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9274
      if Var_Expr = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr);
--# end
--# expr-done 12749
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => If_Result.Node,
         Info => If_Result.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9274") else null)); 
--# expr-done 12750
Block_Var := Bind_Result; Inc_Ref (Block_Var);
--# bind self_eq Block_Var
--# expr-start 12754 '<And at ast.py:9275>' And_Pred /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9275
--# expr-start 12751 '<Cast to DottedName at ast.py:9275>' Cast_Result_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9275
Cast_Expr := Name; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 12751
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind dn Var_Expr_1
--# expr-start 12753 '<FieldAccess for child_unit_xref_equation at ast.py:9276>' Fld_4 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9276
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12752 '<FieldAccess for prefix at ast.py:9276>' Fld_3 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9276
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_1.Dotted_Name_F_Prefix;
--# expr-done 12752
--# property-call-start WithClause.child_unit_xref_equation
Fld_4 := Libadalang.Implementation.With_Clause_P_Child_Unit_Xref_Equation (Node => Ent.Node, Name => Fld_3, E_Info => Ent.Info);
--# end
--# expr-done 12753
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3391;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9277") else null)); 
   Result_Var_1 := True_Rel;
end if;
      Inc_Ref (Result_Var_1);
And_Pred := Create_And (Block_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9275") else null)); 
--# expr-done 12754
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 12755
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_740;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_740;
                     Finalizer_Scope_3391;
            raise;
      end;
   return Property_Result;
end With_Clause_P_Child_Unit_Xref_Equation;
--# end
--# property-start WithClause.xref_entry_point /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9280
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Xref_Entry_Point
  (Node : Bare_With_Clause
  )
   return Boolean
is
   Self : Bare_With_Clause  := Bare_With_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end With_Clause_P_Xref_Entry_Point;
--# end
--# property-start WithClause.xref_equation /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9281
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Xref_Equation
  (Node : Bare_With_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_With_Clause  := Bare_With_Clause (Node);
     Ent : Internal_Entity_With_Clause :=
       Internal_Entity_With_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_742 with Inline_Always;
            procedure Finalizer_Scope_3392 with Inline_Always;
      Fld : Bare_Name_List;
Fld_1 : Logic_Equation;
Item_385 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_742 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_742;
            procedure Finalizer_Scope_3392 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3392;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12759 '<LogicAll at ast.py:9282>' Logic_Boolean_Op /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9282
--# expr-start 12758 '<Map at ast.py:9282>' Map_Result /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9282
   --# expr-start 12756 '<FieldAccess for packages at ast.py:9282>' Fld /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9282
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.With_Clause_F_Packages;
--# expr-done 12756
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_385 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_385 := Untyped_Item_385; 
   --# scope-start
               --# bind p Item_385
      --# expr-start 12757 '<FieldAccess for child_unit_xref_equation at ast.py:9283>' Fld_1 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9283
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start WithClause.child_unit_xref_equation
Fld_1 := Libadalang.Implementation.With_Clause_P_Child_Unit_Xref_Equation (Node => Ent.Node, Name => Item_385, E_Info => Ent.Info);
--# end
--# expr-done 12757
         declare
            Item_To_Append : constant Logic_Equation := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3392;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12758
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "ast.py:9282") else null)); 
--# expr-done 12759
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_742;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_742;
                     Finalizer_Scope_3392;
            raise;
      end;
   return Property_Result;
end With_Clause_P_Xref_Equation;
--# end
--# property-start WithClause._initial_env_17 /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9288
pragma Warnings (Off, "is not referenced");
 function Initial_Env_17
  (Node : Bare_With_Clause
  )
   return Internal_Designated_Env
is
   Self : Bare_With_Clause  := Bare_With_Clause (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1544 with Inline_Always;
      Sym : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1544 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1544;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12761 '<New DesignatedEnv at ast.py:9288>' New_Struct /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:9288
--# expr-start 12760 '<Symbol Standard at ???>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard); 
--# expr-done 12760
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Sym, Kind => Named_Env); 
--# expr-done 12761
         Property_Result := New_Struct;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1544;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1544;
            raise;
      end;
   return Property_Result;
end Initial_Env_17;
--# end
   --
   --  Primitives for Bare_With_Private
   --
--# property-start '[dispatcher]WithPrivate.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_With_Private_P_As_Bool
  (Node : Bare_With_Private
  )
   return Boolean
is
   Self : Bare_With_Private  := Bare_With_Private (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_With_Private (Self.Kind) is
               when Ada_With_Private_Absent =>
                  --# property-call-start WithPrivate.Absent.as_bool
                  Property_Result := With_Private_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_With_Private_Present =>
                  --# property-call-start WithPrivate.Present.as_bool
                  Property_Result := With_Private_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_With_Private_P_As_Bool;
--# end
   --
   --  Primitives for Bare_With_Private_Absent
   --
--# property-start WithPrivate.Absent.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5332
pragma Warnings (Off, "is not referenced");
 function With_Private_Absent_P_As_Bool
  (Node : Bare_With_Private_Absent
  )
   return Boolean
is
   Self : Bare_With_Private_Absent  := Bare_With_Private_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end With_Private_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_With_Private_Present
   --
--# property-start WithPrivate.Present.as_bool /builddir/build/BUILD/libadalang-24.0.0-build/libadalang-24.0.0/ada/ast.py:5332
pragma Warnings (Off, "is not referenced");
 function With_Private_Present_P_As_Bool
  (Node : Bare_With_Private_Present
  )
   return Boolean
is
   Self : Bare_With_Private_Present  := Bare_With_Private_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end With_Private_Present_P_As_Bool;
--# end
   ----------------------------
   -- Destroy_Synthetic_Node --
   ----------------------------
   procedure Destroy_Synthetic_Node (Node : in out Bare_Ada_Node) is
      procedure Free is new Ada.Unchecked_Deallocation
        (Root_Node_Record, Bare_Ada_Node);
   begin
      --  Don't call Node.Destroy, as Node's children may be gone already: they
      --  have their own destructor and there is no specified order for the
      --  call of these destructors.
      Free_User_Fields (Node);
      Free (Node);
   end Destroy_Synthetic_Node;
   -----------
   -- Image --
   -----------
   function Image (Value : Boolean) return String
   is (if Value then "True" else "False");
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image
        (Node       : Bare_Ada_Node;
         Decoration : Boolean := True) return String is
      begin
         if Node = null then
            return "None";
         else
            declare
               Result : constant String :=
                 (Kind_Name (Node) & " "
                  & Basename (Node.Unit) & ":"
                  & Image (Sloc_Range (Node)));
            begin
               return (if Decoration then "<" & Result & ">" else Result);
            end;
         end if;
      end Trace_Image;
   Kind_Names : array (Ada_Node_Kind_Type) of Unbounded_String :=
     (Ada_Abort_Absent => To_Unbounded_String ("AbortAbsent"), 
Ada_Abort_Present => To_Unbounded_String ("AbortPresent"), 
Ada_Abstract_Absent => To_Unbounded_String ("AbstractAbsent"), 
Ada_Abstract_Present => To_Unbounded_String ("AbstractPresent"), 
Ada_Ada_Node_List => To_Unbounded_String ("AdaNodeList"), 
Ada_Abstract_State_Decl_List => To_Unbounded_String ("AbstractStateDeclList"), 
Ada_Alternatives_List => To_Unbounded_String ("AlternativesList"), 
Ada_Constraint_List => To_Unbounded_String ("ConstraintList"), 
Ada_Decl_List => To_Unbounded_String ("DeclList"), 
Ada_Stmt_List => To_Unbounded_String ("StmtList"), 
Ada_Aspect_Assoc_List => To_Unbounded_String ("AspectAssocList"), 
Ada_Base_Assoc_List => To_Unbounded_String ("BaseAssocList"), 
Ada_Assoc_List => To_Unbounded_String ("AssocList"), 
Ada_Basic_Decl_List => To_Unbounded_String ("BasicDeclList"), 
Ada_Case_Expr_Alternative_List => To_Unbounded_String ("CaseExprAlternativeList"), 
Ada_Case_Stmt_Alternative_List => To_Unbounded_String ("CaseStmtAlternativeList"), 
Ada_Compilation_Unit_List => To_Unbounded_String ("CompilationUnitList"), 
Ada_Concat_Operand_List => To_Unbounded_String ("ConcatOperandList"), 
Ada_Contract_Case_Assoc_List => To_Unbounded_String ("ContractCaseAssocList"), 
Ada_Defining_Name_List => To_Unbounded_String ("DefiningNameList"), 
Ada_Discriminant_Spec_List => To_Unbounded_String ("DiscriminantSpecList"), 
Ada_Elsif_Expr_Part_List => To_Unbounded_String ("ElsifExprPartList"), 
Ada_Elsif_Stmt_Part_List => To_Unbounded_String ("ElsifStmtPartList"), 
Ada_Enum_Literal_Decl_List => To_Unbounded_String ("EnumLiteralDeclList"), 
Ada_Expr_Alternatives_List => To_Unbounded_String ("ExprAlternativesList"), 
Ada_Discriminant_Choice_List => To_Unbounded_String ("DiscriminantChoiceList"), 
Ada_Name_List => To_Unbounded_String ("NameList"), 
Ada_Parent_List => To_Unbounded_String ("ParentList"), 
Ada_Param_Spec_List => To_Unbounded_String ("ParamSpecList"), 
Ada_Pragma_Node_List => To_Unbounded_String ("PragmaNodeList"), 
Ada_Select_When_Part_List => To_Unbounded_String ("SelectWhenPartList"), 
Ada_Unconstrained_Array_Index_List => To_Unbounded_String ("UnconstrainedArrayIndexList"), 
Ada_Variant_List => To_Unbounded_String ("VariantList"), 
Ada_Aliased_Absent => To_Unbounded_String ("AliasedAbsent"), 
Ada_Aliased_Present => To_Unbounded_String ("AliasedPresent"), 
Ada_All_Absent => To_Unbounded_String ("AllAbsent"), 
Ada_All_Present => To_Unbounded_String ("AllPresent"), 
Ada_Constrained_Array_Indices => To_Unbounded_String ("ConstrainedArrayIndices"), 
Ada_Unconstrained_Array_Indices => To_Unbounded_String ("UnconstrainedArrayIndices"), 
Ada_Aspect_Assoc => To_Unbounded_String ("AspectAssoc"), 
Ada_At_Clause => To_Unbounded_String ("AtClause"), 
Ada_Attribute_Def_Clause => To_Unbounded_String ("AttributeDefClause"), 
Ada_Enum_Rep_Clause => To_Unbounded_String ("EnumRepClause"), 
Ada_Record_Rep_Clause => To_Unbounded_String ("RecordRepClause"), 
Ada_Aspect_Spec => To_Unbounded_String ("AspectSpec"), 
Ada_Contract_Case_Assoc => To_Unbounded_String ("ContractCaseAssoc"), 
Ada_Pragma_Argument_Assoc => To_Unbounded_String ("PragmaArgumentAssoc"), 
Ada_Entry_Spec => To_Unbounded_String ("EntrySpec"), 
Ada_Enum_Subp_Spec => To_Unbounded_String ("EnumSubpSpec"), 
Ada_Subp_Spec => To_Unbounded_String ("SubpSpec"), 
Ada_Synthetic_Binary_Spec => To_Unbounded_String ("SyntheticBinarySpec"), 
Ada_Synthetic_Unary_Spec => To_Unbounded_String ("SyntheticUnarySpec"), 
Ada_Component_List => To_Unbounded_String ("ComponentList"), 
Ada_Known_Discriminant_Part => To_Unbounded_String ("KnownDiscriminantPart"), 
Ada_Unknown_Discriminant_Part => To_Unbounded_String ("UnknownDiscriminantPart"), 
Ada_Entry_Completion_Formal_Params => To_Unbounded_String ("EntryCompletionFormalParams"), 
Ada_Generic_Formal_Part => To_Unbounded_String ("GenericFormalPart"), 
Ada_Null_Record_Def => To_Unbounded_String ("NullRecordDef"), 
Ada_Record_Def => To_Unbounded_String ("RecordDef"), 
Ada_Aggregate_Assoc => To_Unbounded_String ("AggregateAssoc"), 
Ada_Multi_Dim_Array_Assoc => To_Unbounded_String ("MultiDimArrayAssoc"), 
Ada_Composite_Constraint_Assoc => To_Unbounded_String ("CompositeConstraintAssoc"), 
Ada_Iterated_Assoc => To_Unbounded_String ("IteratedAssoc"), 
Ada_Param_Assoc => To_Unbounded_String ("ParamAssoc"), 
Ada_Abstract_State_Decl => To_Unbounded_String ("AbstractStateDecl"), 
Ada_Anonymous_Expr_Decl => To_Unbounded_String ("AnonymousExprDecl"), 
Ada_Component_Decl => To_Unbounded_String ("ComponentDecl"), 
Ada_Discriminant_Spec => To_Unbounded_String ("DiscriminantSpec"), 
Ada_Generic_Formal_Obj_Decl => To_Unbounded_String ("GenericFormalObjDecl"), 
Ada_Generic_Formal_Package => To_Unbounded_String ("GenericFormalPackage"), 
Ada_Generic_Formal_Subp_Decl => To_Unbounded_String ("GenericFormalSubpDecl"), 
Ada_Generic_Formal_Type_Decl => To_Unbounded_String ("GenericFormalTypeDecl"), 
Ada_Param_Spec => To_Unbounded_String ("ParamSpec"), 
Ada_Synthetic_Formal_Param_Decl => To_Unbounded_String ("SyntheticFormalParamDecl"), 
Ada_Generic_Package_Internal => To_Unbounded_String ("GenericPackageInternal"), 
Ada_Package_Decl => To_Unbounded_String ("PackageDecl"), 
Ada_Discrete_Base_Subtype_Decl => To_Unbounded_String ("DiscreteBaseSubtypeDecl"), 
Ada_Subtype_Decl => To_Unbounded_String ("SubtypeDecl"), 
Ada_Classwide_Type_Decl => To_Unbounded_String ("ClasswideTypeDecl"), 
Ada_Incomplete_Type_Decl => To_Unbounded_String ("IncompleteTypeDecl"), 
Ada_Incomplete_Formal_Type_Decl => To_Unbounded_String ("IncompleteFormalTypeDecl"), 
Ada_Incomplete_Tagged_Type_Decl => To_Unbounded_String ("IncompleteTaggedTypeDecl"), 
Ada_Protected_Type_Decl => To_Unbounded_String ("ProtectedTypeDecl"), 
Ada_Task_Type_Decl => To_Unbounded_String ("TaskTypeDecl"), 
Ada_Single_Task_Type_Decl => To_Unbounded_String ("SingleTaskTypeDecl"), 
Ada_Anonymous_Type_Decl => To_Unbounded_String ("AnonymousTypeDecl"), 
Ada_Synth_Anonymous_Type_Decl => To_Unbounded_String ("SynthAnonymousTypeDecl"), 
Ada_Concrete_Type_Decl => To_Unbounded_String ("ConcreteTypeDecl"), 
Ada_Formal_Type_Decl => To_Unbounded_String ("FormalTypeDecl"), 
Ada_Abstract_Subp_Decl => To_Unbounded_String ("AbstractSubpDecl"), 
Ada_Abstract_Formal_Subp_Decl => To_Unbounded_String ("AbstractFormalSubpDecl"), 
Ada_Concrete_Formal_Subp_Decl => To_Unbounded_String ("ConcreteFormalSubpDecl"), 
Ada_Subp_Decl => To_Unbounded_String ("SubpDecl"), 
Ada_Entry_Decl => To_Unbounded_String ("EntryDecl"), 
Ada_Enum_Literal_Decl => To_Unbounded_String ("EnumLiteralDecl"), 
Ada_Synthetic_Char_Enum_Lit => To_Unbounded_String ("SyntheticCharEnumLit"), 
Ada_Generic_Subp_Internal => To_Unbounded_String ("GenericSubpInternal"), 
Ada_Synthetic_Subp_Decl => To_Unbounded_String ("SyntheticSubpDecl"), 
Ada_Accept_Stmt_Body => To_Unbounded_String ("AcceptStmtBody"), 
Ada_Expr_Function => To_Unbounded_String ("ExprFunction"), 
Ada_Null_Subp_Decl => To_Unbounded_String ("NullSubpDecl"), 
Ada_Subp_Body => To_Unbounded_String ("SubpBody"), 
Ada_Subp_Renaming_Decl => To_Unbounded_String ("SubpRenamingDecl"), 
Ada_Package_Body_Stub => To_Unbounded_String ("PackageBodyStub"), 
Ada_Protected_Body_Stub => To_Unbounded_String ("ProtectedBodyStub"), 
Ada_Subp_Body_Stub => To_Unbounded_String ("SubpBodyStub"), 
Ada_Task_Body_Stub => To_Unbounded_String ("TaskBodyStub"), 
Ada_Entry_Body => To_Unbounded_String ("EntryBody"), 
Ada_Package_Body => To_Unbounded_String ("PackageBody"), 
Ada_Protected_Body => To_Unbounded_String ("ProtectedBody"), 
Ada_Task_Body => To_Unbounded_String ("TaskBody"), 
Ada_Entry_Index_Spec => To_Unbounded_String ("EntryIndexSpec"), 
Ada_Error_Decl => To_Unbounded_String ("ErrorDecl"), 
Ada_Exception_Decl => To_Unbounded_String ("ExceptionDecl"), 
Ada_Exception_Handler => To_Unbounded_String ("ExceptionHandler"), 
Ada_For_Loop_Var_Decl => To_Unbounded_String ("ForLoopVarDecl"), 
Ada_Generic_Package_Decl => To_Unbounded_String ("GenericPackageDecl"), 
Ada_Generic_Subp_Decl => To_Unbounded_String ("GenericSubpDecl"), 
Ada_Generic_Package_Instantiation => To_Unbounded_String ("GenericPackageInstantiation"), 
Ada_Generic_Subp_Instantiation => To_Unbounded_String ("GenericSubpInstantiation"), 
Ada_Generic_Package_Renaming_Decl => To_Unbounded_String ("GenericPackageRenamingDecl"), 
Ada_Generic_Subp_Renaming_Decl => To_Unbounded_String ("GenericSubpRenamingDecl"), 
Ada_Label_Decl => To_Unbounded_String ("LabelDecl"), 
Ada_Named_Stmt_Decl => To_Unbounded_String ("NamedStmtDecl"), 
Ada_Number_Decl => To_Unbounded_String ("NumberDecl"), 
Ada_Object_Decl => To_Unbounded_String ("ObjectDecl"), 
Ada_Extended_Return_Stmt_Object_Decl => To_Unbounded_String ("ExtendedReturnStmtObjectDecl"), 
Ada_No_Type_Object_Renaming_Decl => To_Unbounded_String ("NoTypeObjectRenamingDecl"), 
Ada_Package_Renaming_Decl => To_Unbounded_String ("PackageRenamingDecl"), 
Ada_Single_Protected_Decl => To_Unbounded_String ("SingleProtectedDecl"), 
Ada_Single_Task_Decl => To_Unbounded_String ("SingleTaskDecl"), 
Ada_Case_Stmt_Alternative => To_Unbounded_String ("CaseStmtAlternative"), 
Ada_Compilation_Unit => To_Unbounded_String ("CompilationUnit"), 
Ada_Component_Clause => To_Unbounded_String ("ComponentClause"), 
Ada_Component_Def => To_Unbounded_String ("ComponentDef"), 
Ada_Constant_Absent => To_Unbounded_String ("ConstantAbsent"), 
Ada_Constant_Present => To_Unbounded_String ("ConstantPresent"), 
Ada_Composite_Constraint => To_Unbounded_String ("CompositeConstraint"), 
Ada_Delta_Constraint => To_Unbounded_String ("DeltaConstraint"), 
Ada_Digits_Constraint => To_Unbounded_String ("DigitsConstraint"), 
Ada_Range_Constraint => To_Unbounded_String ("RangeConstraint"), 
Ada_Declarative_Part => To_Unbounded_String ("DeclarativePart"), 
Ada_Private_Part => To_Unbounded_String ("PrivatePart"), 
Ada_Public_Part => To_Unbounded_String ("PublicPart"), 
Ada_Elsif_Expr_Part => To_Unbounded_String ("ElsifExprPart"), 
Ada_Elsif_Stmt_Part => To_Unbounded_String ("ElsifStmtPart"), 
Ada_Abstract_State_Decl_Expr => To_Unbounded_String ("AbstractStateDeclExpr"), 
Ada_Allocator => To_Unbounded_String ("Allocator"), 
Ada_Aggregate => To_Unbounded_String ("Aggregate"), 
Ada_Bracket_Aggregate => To_Unbounded_String ("BracketAggregate"), 
Ada_Delta_Aggregate => To_Unbounded_String ("DeltaAggregate"), 
Ada_Bracket_Delta_Aggregate => To_Unbounded_String ("BracketDeltaAggregate"), 
Ada_Null_Record_Aggregate => To_Unbounded_String ("NullRecordAggregate"), 
Ada_Bin_Op => To_Unbounded_String ("BinOp"), 
Ada_Relation_Op => To_Unbounded_String ("RelationOp"), 
Ada_Box_Expr => To_Unbounded_String ("BoxExpr"), 
Ada_Case_Expr_Alternative => To_Unbounded_String ("CaseExprAlternative"), 
Ada_Concat_Op => To_Unbounded_String ("ConcatOp"), 
Ada_Concat_Operand => To_Unbounded_String ("ConcatOperand"), 
Ada_Case_Expr => To_Unbounded_String ("CaseExpr"), 
Ada_If_Expr => To_Unbounded_String ("IfExpr"), 
Ada_Contract_Cases => To_Unbounded_String ("ContractCases"), 
Ada_Decl_Expr => To_Unbounded_String ("DeclExpr"), 
Ada_Membership_Expr => To_Unbounded_String ("MembershipExpr"), 
Ada_Attribute_Ref => To_Unbounded_String ("AttributeRef"), 
Ada_Call_Expr => To_Unbounded_String ("CallExpr"), 
Ada_Defining_Name => To_Unbounded_String ("DefiningName"), 
Ada_Synthetic_Defining_Name => To_Unbounded_String ("SyntheticDefiningName"), 
Ada_Discrete_Subtype_Name => To_Unbounded_String ("DiscreteSubtypeName"), 
Ada_Dotted_Name => To_Unbounded_String ("DottedName"), 
Ada_End_Name => To_Unbounded_String ("EndName"), 
Ada_Explicit_Deref => To_Unbounded_String ("ExplicitDeref"), 
Ada_Qual_Expr => To_Unbounded_String ("QualExpr"), 
Ada_Reduce_Attribute_Ref => To_Unbounded_String ("ReduceAttributeRef"), 
Ada_Char_Literal => To_Unbounded_String ("Chr"), 
Ada_Identifier => To_Unbounded_String ("Id"), 
Ada_Op_Abs => To_Unbounded_String ("OpAbs"), 
Ada_Op_And => To_Unbounded_String ("OpAnd"), 
Ada_Op_And_Then => To_Unbounded_String ("OpAndThen"), 
Ada_Op_Concat => To_Unbounded_String ("OpConcat"), 
Ada_Op_Div => To_Unbounded_String ("OpDiv"), 
Ada_Op_Double_Dot => To_Unbounded_String ("OpDoubleDot"), 
Ada_Op_Eq => To_Unbounded_String ("OpEq"), 
Ada_Op_Gt => To_Unbounded_String ("OpGt"), 
Ada_Op_Gte => To_Unbounded_String ("OpGte"), 
Ada_Op_In => To_Unbounded_String ("OpIn"), 
Ada_Op_Lt => To_Unbounded_String ("OpLt"), 
Ada_Op_Lte => To_Unbounded_String ("OpLte"), 
Ada_Op_Minus => To_Unbounded_String ("OpMinus"), 
Ada_Op_Mod => To_Unbounded_String ("OpMod"), 
Ada_Op_Mult => To_Unbounded_String ("OpMult"), 
Ada_Op_Neq => To_Unbounded_String ("OpNeq"), 
Ada_Op_Not => To_Unbounded_String ("OpNot"), 
Ada_Op_Not_In => To_Unbounded_String ("OpNotIn"), 
Ada_Op_Or => To_Unbounded_String ("OpOr"), 
Ada_Op_Or_Else => To_Unbounded_String ("OpOrElse"), 
Ada_Op_Plus => To_Unbounded_String ("OpPlus"), 
Ada_Op_Pow => To_Unbounded_String ("OpPow"), 
Ada_Op_Rem => To_Unbounded_String ("OpRem"), 
Ada_Op_Xor => To_Unbounded_String ("OpXor"), 
Ada_String_Literal => To_Unbounded_String ("Str"), 
Ada_Null_Literal => To_Unbounded_String ("Null"), 
Ada_Int_Literal => To_Unbounded_String ("Int"), 
Ada_Real_Literal => To_Unbounded_String ("Real"), 
Ada_Synthetic_Identifier => To_Unbounded_String ("SyntheticIdentifier"), 
Ada_Target_Name => To_Unbounded_String ("TargetName"), 
Ada_Update_Attribute_Ref => To_Unbounded_String ("UpdateAttributeRef"), 
Ada_Paren_Expr => To_Unbounded_String ("ParenExpr"), 
Ada_Quantified_Expr => To_Unbounded_String ("QuantifiedExpr"), 
Ada_Raise_Expr => To_Unbounded_String ("RaiseExpr"), 
Ada_Un_Op => To_Unbounded_String ("UnOp"), 
Ada_Handled_Stmts => To_Unbounded_String ("HandledStmts"), 
Ada_Interface_Kind_Limited => To_Unbounded_String ("InterfaceKindLimited"), 
Ada_Interface_Kind_Protected => To_Unbounded_String ("InterfaceKindProtected"), 
Ada_Interface_Kind_Synchronized => To_Unbounded_String ("InterfaceKindSynchronized"), 
Ada_Interface_Kind_Task => To_Unbounded_String ("InterfaceKindTask"), 
Ada_Iter_Type_In => To_Unbounded_String ("IterTypeIn"), 
Ada_Iter_Type_Of => To_Unbounded_String ("IterTypeOf"), 
Ada_Library_Item => To_Unbounded_String ("LibraryItem"), 
Ada_Limited_Absent => To_Unbounded_String ("LimitedAbsent"), 
Ada_Limited_Present => To_Unbounded_String ("LimitedPresent"), 
Ada_For_Loop_Spec => To_Unbounded_String ("ForLoopSpec"), 
Ada_While_Loop_Spec => To_Unbounded_String ("WhileLoopSpec"), 
Ada_Mode_Default => To_Unbounded_String ("ModeDefault"), 
Ada_Mode_In => To_Unbounded_String ("ModeIn"), 
Ada_Mode_In_Out => To_Unbounded_String ("ModeInOut"), 
Ada_Mode_Out => To_Unbounded_String ("ModeOut"), 
Ada_Multi_Abstract_State_Decl => To_Unbounded_String ("MultiAbstractStateDecl"), 
Ada_Not_Null_Absent => To_Unbounded_String ("NotNullAbsent"), 
Ada_Not_Null_Present => To_Unbounded_String ("NotNullPresent"), 
Ada_Null_Component_Decl => To_Unbounded_String ("NullComponentDecl"), 
Ada_Others_Designator => To_Unbounded_String ("OthersDesignator"), 
Ada_Overriding_Not_Overriding => To_Unbounded_String ("OverridingNotOverriding"), 
Ada_Overriding_Overriding => To_Unbounded_String ("OverridingOverriding"), 
Ada_Overriding_Unspecified => To_Unbounded_String ("OverridingUnspecified"), 
Ada_Params => To_Unbounded_String ("Params"), 
Ada_Paren_Abstract_State_Decl => To_Unbounded_String ("ParenAbstractStateDecl"), 
Ada_Pp_Else_Directive => To_Unbounded_String ("PpElseDirective"), 
Ada_Pp_Elsif_Directive => To_Unbounded_String ("PpElsifDirective"), 
Ada_Pp_End_If_Directive => To_Unbounded_String ("PpEndIfDirective"), 
Ada_Pp_If_Directive => To_Unbounded_String ("PpIfDirective"), 
Ada_Pp_Then_Kw => To_Unbounded_String ("PpThenKw"), 
Ada_Pragma_Node => To_Unbounded_String ("PragmaNode"), 
Ada_Private_Absent => To_Unbounded_String ("PrivateAbsent"), 
Ada_Private_Present => To_Unbounded_String ("PrivatePresent"), 
Ada_Protected_Def => To_Unbounded_String ("ProtectedDef"), 
Ada_Protected_Absent => To_Unbounded_String ("ProtectedAbsent"), 
Ada_Protected_Present => To_Unbounded_String ("ProtectedPresent"), 
Ada_Quantifier_All => To_Unbounded_String ("QuantifierAll"), 
Ada_Quantifier_Some => To_Unbounded_String ("QuantifierSome"), 
Ada_Range_Spec => To_Unbounded_String ("RangeSpec"), 
Ada_Renaming_Clause => To_Unbounded_String ("RenamingClause"), 
Ada_Synthetic_Renaming_Clause => To_Unbounded_String ("SyntheticRenamingClause"), 
Ada_Reverse_Absent => To_Unbounded_String ("ReverseAbsent"), 
Ada_Reverse_Present => To_Unbounded_String ("ReversePresent"), 
Ada_Select_When_Part => To_Unbounded_String ("SelectWhenPart"), 
Ada_Accept_Stmt => To_Unbounded_String ("AcceptStmt"), 
Ada_Accept_Stmt_With_Stmts => To_Unbounded_String ("AcceptStmtWithStmts"), 
Ada_For_Loop_Stmt => To_Unbounded_String ("ForLoopStmt"), 
Ada_Loop_Stmt => To_Unbounded_String ("LoopStmt"), 
Ada_While_Loop_Stmt => To_Unbounded_String ("WhileLoopStmt"), 
Ada_Begin_Block => To_Unbounded_String ("BeginBlock"), 
Ada_Decl_Block => To_Unbounded_String ("DeclBlock"), 
Ada_Case_Stmt => To_Unbounded_String ("CaseStmt"), 
Ada_Extended_Return_Stmt => To_Unbounded_String ("ExtendedReturnStmt"), 
Ada_If_Stmt => To_Unbounded_String ("IfStmt"), 
Ada_Named_Stmt => To_Unbounded_String ("NamedStmt"), 
Ada_Select_Stmt => To_Unbounded_String ("SelectStmt"), 
Ada_Error_Stmt => To_Unbounded_String ("ErrorStmt"), 
Ada_Abort_Stmt => To_Unbounded_String ("AbortStmt"), 
Ada_Assign_Stmt => To_Unbounded_String ("AssignStmt"), 
Ada_Call_Stmt => To_Unbounded_String ("CallStmt"), 
Ada_Delay_Stmt => To_Unbounded_String ("DelayStmt"), 
Ada_Exit_Stmt => To_Unbounded_String ("ExitStmt"), 
Ada_Goto_Stmt => To_Unbounded_String ("GotoStmt"), 
Ada_Label => To_Unbounded_String ("Label"), 
Ada_Null_Stmt => To_Unbounded_String ("NullStmt"), 
Ada_Raise_Stmt => To_Unbounded_String ("RaiseStmt"), 
Ada_Requeue_Stmt => To_Unbounded_String ("RequeueStmt"), 
Ada_Return_Stmt => To_Unbounded_String ("ReturnStmt"), 
Ada_Terminate_Alternative => To_Unbounded_String ("TerminateAlternative"), 
Ada_Subp_Kind_Function => To_Unbounded_String ("SubpKindFunction"), 
Ada_Subp_Kind_Procedure => To_Unbounded_String ("SubpKindProcedure"), 
Ada_Subunit => To_Unbounded_String ("Subunit"), 
Ada_Synchronized_Absent => To_Unbounded_String ("SynchronizedAbsent"), 
Ada_Synchronized_Present => To_Unbounded_String ("SynchronizedPresent"), 
Ada_Tagged_Absent => To_Unbounded_String ("TaggedAbsent"), 
Ada_Tagged_Present => To_Unbounded_String ("TaggedPresent"), 
Ada_Task_Def => To_Unbounded_String ("TaskDef"), 
Ada_Type_Attributes_Repository => To_Unbounded_String ("TypeAttributesRepository"), 
Ada_Access_To_Subp_Def => To_Unbounded_String ("AccessToSubpDef"), 
Ada_Anonymous_Type_Access_Def => To_Unbounded_String ("AnonymousTypeAccessDef"), 
Ada_Type_Access_Def => To_Unbounded_String ("TypeAccessDef"), 
Ada_Array_Type_Def => To_Unbounded_String ("ArrayTypeDef"), 
Ada_Derived_Type_Def => To_Unbounded_String ("DerivedTypeDef"), 
Ada_Enum_Type_Def => To_Unbounded_String ("EnumTypeDef"), 
Ada_Formal_Discrete_Type_Def => To_Unbounded_String ("FormalDiscreteTypeDef"), 
Ada_Interface_Type_Def => To_Unbounded_String ("InterfaceTypeDef"), 
Ada_Mod_Int_Type_Def => To_Unbounded_String ("ModIntTypeDef"), 
Ada_Private_Type_Def => To_Unbounded_String ("PrivateTypeDef"), 
Ada_Decimal_Fixed_Point_Def => To_Unbounded_String ("DecimalFixedPointDef"), 
Ada_Floating_Point_Def => To_Unbounded_String ("FloatingPointDef"), 
Ada_Ordinary_Fixed_Point_Def => To_Unbounded_String ("OrdinaryFixedPointDef"), 
Ada_Record_Type_Def => To_Unbounded_String ("RecordTypeDef"), 
Ada_Signed_Int_Type_Def => To_Unbounded_String ("SignedIntTypeDef"), 
Ada_Anonymous_Type => To_Unbounded_String ("AnonymousType"), 
Ada_Enum_Lit_Synth_Type_Expr => To_Unbounded_String ("EnumLitSynthTypeExpr"), 
Ada_Subtype_Indication => To_Unbounded_String ("SubtypeIndication"), 
Ada_Constrained_Subtype_Indication => To_Unbounded_String ("ConstrainedSubtypeIndication"), 
Ada_Discrete_Subtype_Indication => To_Unbounded_String ("DiscreteSubtypeIndication"), 
Ada_Synthetic_Type_Expr => To_Unbounded_String ("SyntheticTypeExpr"), 
Ada_Unconstrained_Array_Index => To_Unbounded_String ("UnconstrainedArrayIndex"), 
Ada_Until_Absent => To_Unbounded_String ("UntilAbsent"), 
Ada_Until_Present => To_Unbounded_String ("UntilPresent"), 
Ada_Use_Package_Clause => To_Unbounded_String ("UsePackageClause"), 
Ada_Use_Type_Clause => To_Unbounded_String ("UseTypeClause"), 
Ada_Value_Sequence => To_Unbounded_String ("ValueSequence"), 
Ada_Variant => To_Unbounded_String ("Variant"), 
Ada_Variant_Part => To_Unbounded_String ("VariantPart"), 
Ada_With_Clause => To_Unbounded_String ("WithClause"), 
Ada_With_Private_Absent => To_Unbounded_String ("WithPrivateAbsent"), 
Ada_With_Private_Present => To_Unbounded_String ("WithPrivatePresent"));
   ---------------
   -- Kind_Name --
   ---------------
   function Kind_Name (Node : Bare_Ada_Node) return String is
   begin
      return To_String (Kind_Names (Node.Kind));
   end Kind_Name;
   --------------------
   -- Children_Count --
   --------------------
   function Children_Count (Node : Bare_Ada_Node) return Natural is
      C : Integer := Kind_To_Node_Children_Count (Node.Kind);
   begin
      if C = -1 then
         return Node.Count;
      else
         return C;
      end if;
   end Children_Count;
   ----------------------
   -- Free_User_Fields --
   ----------------------
   procedure Free_User_Fields (Node : Bare_Ada_Node) is
      procedure Reset_Logic_Var (LV : in out Logic_Var_Record);
      --  Reset the LV logic variable, clearing the value it stores
      ---------------------
      -- Reset_Logic_Var --
      ---------------------
      procedure Reset_Logic_Var (LV : in out Logic_Var_Record) is
      begin
         LV.Value := No_Entity;
         Entity_Vars.Reset (LV'Unrestricted_Access);
         Entity_Vars.Destroy (LV);
      end Reset_Logic_Var;
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      case Ada_Ada_Node (K) is
when Ada_Base_Type_Decl =>
declare
N_Bare_Base_Type_Decl : constant Bare_Base_Type_Decl := Node;
begin
Dec_Ref (N_Bare_Base_Type_Decl.Lf_Stg_Dottable_Subps_Env);
case Ada_Base_Type_Decl (K) is
when Ada_Type_Decl =>
declare
N_Bare_Type_Decl : constant Bare_Type_Decl := N_Bare_Base_Type_Decl;
begin
Dec_Ref (N_Bare_Type_Decl.Lf_Stg_Direct_Primitives_Env);
Dec_Ref (N_Bare_Type_Decl.Lf_Stg_Direct_Primitive_Subps);
end;
when others => null;
end case;
end;
when Ada_Generic_Instantiation =>
declare
N_Bare_Generic_Instantiation : constant Bare_Generic_Instantiation := Node;
begin
Dec_Ref (N_Bare_Generic_Instantiation.Lf_Stg_Instantiation_Env);
Dec_Ref (N_Bare_Generic_Instantiation.Lf_Stg_Actual_Expr_Decls);
end;
when Ada_Compilation_Unit_Range =>
declare
N_Bare_Compilation_Unit : constant Bare_Compilation_Unit := Node;
begin
Dec_Ref (N_Bare_Compilation_Unit.Compilation_Unit_No_Env);
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
Reset_Logic_Var (N_Bare_Attribute_Ref.Attribute_Ref_R_Ref_Var);
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
Reset_Logic_Var (N_Bare_Call_Expr.Call_Expr_R_Called_Spec);
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
Reset_Logic_Var (N_Bare_Explicit_Deref.Explicit_Deref_R_Called_Spec);
end;
when Ada_Reduce_Attribute_Ref_Range =>
declare
N_Bare_Reduce_Attribute_Ref : constant Bare_Reduce_Attribute_Ref := Node;
begin
Reset_Logic_Var (N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_R_Ref_Var);
end;
when Ada_Target_Name_Range =>
declare
N_Bare_Target_Name : constant Bare_Target_Name := Node;
begin
Reset_Logic_Var (N_Bare_Target_Name.Target_Name_R_Ref_Var);
end;
when Ada_Update_Attribute_Ref_Range =>
declare
N_Bare_Update_Attribute_Ref : constant Bare_Update_Attribute_Ref := Node;
begin
Reset_Logic_Var (N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_R_Ref_Var);
end;
when others => null;
end case;
   end Free_User_Fields;
   ----------------
   -- Token_Data --
   ----------------
   function Token_Data (Unit : Internal_Unit) return Token_Data_Handler_Access
   is (Unit.TDH'Access);
   -------------------
   -- Lookup_Symbol --
   -------------------
   function Lookup_Symbol
     (Context : Internal_Context; Symbol : Text_Type) return Symbol_Type
   is
      Canon_Symbol : constant Symbolization_Result :=
            Libadalang.Sources.Canonicalize (Symbol)
      ;
   begin
      if Canon_Symbol.Success then
         return Get_Symbol
           (Context.Symbols, Find (Context.Symbols, Canon_Symbol.Symbol));
      else
         raise Invalid_Symbol_Error with Image (Canon_Symbol.Error_Message);
      end if;
   end Lookup_Symbol;
   -------------------------
   -- Create_Special_Unit --
   -------------------------
   function Create_Special_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      Unit : Internal_Unit := new Analysis_Unit_Type'
        (Context                      => Context,
         Is_Internal                  => False,
         Ast_Root                     => null,
         Filename                     => Normalized_Filename,
         Charset                      => To_Unbounded_String (Charset),
         TDH                          => <>,
         Diagnostics                  => <>,
         Rule                         => Rule,
         Ast_Mem_Pool                 => No_Pool,
         Destroyables                 => Destroyable_Vectors.Empty_Vector,
         Referenced_Units             => <>,
         Exiled_Entries               => Exiled_Entry_Vectors.Empty_Vector,
         Foreign_Nodes                =>
            Foreign_Node_Entry_Vectors.Empty_Vector,
         Exiled_Entries_In_NED        =>
            Exiled_Entry_In_NED_Vectors.Empty_Vector,
         Exiled_Envs                  => Exiled_Env_Vectors.Empty_Vector,
         Named_Envs                   => Named_Env_Vectors.Empty_Vector,
         Nodes_With_Foreign_Env       => <>,
         Rebindings                   => Env_Rebindings_Vectors.Empty_Vector,
         Cache_Version                => <>,
         Unit_Version                 => <>,
         Memoization_Map            => <>,
         others => <>
      );
   begin
      Initialize
        (Unit.TDH, Context.Symbols, Unit.all'Address, Context.Tab_Stop);
      return Unit;
   end Create_Special_Unit;
   --------------------
   -- Templates_Unit --
   --------------------
   function Templates_Unit (Context : Internal_Context) return Internal_Unit is
   begin
      if Context.Templates_Unit = No_Analysis_Unit then
         Context.Templates_Unit := Create_Special_Unit
           (Context             => Context,
            Normalized_Filename => No_File,
            Charset             => Default_Charset,
            Rule                => Compilation_Rule);
      end if;
      return Context.Templates_Unit;
   end Templates_Unit;
   --------------
   -- Set_Rule --
   --------------
   procedure Set_Rule (Unit : Internal_Unit; Rule : Grammar_Rule) is
   begin
      Unit.Rule := Rule;
   end Set_Rule;
   ------------------------------
   -- Normalized_Unit_Filename --
   ------------------------------
   function Normalized_Unit_Filename
     (Context : Internal_Context; Filename : String) return Virtual_File
   is
      use Virtual_File_Maps;
      Key : constant Unbounded_String := To_Unbounded_String (Filename);
      Cur : Cursor := Context.Filenames.Find (Key);
   begin
      if Cur = No_Element then
         declare
            F : constant Virtual_File := Create
              (Create_From_Base (+Filename).Full_Name,
               Normalize => True);
         begin
            Context.Filenames.Insert (Key, F);
            return F;
         end;
      else
         return Element (Cur);
      end if;
   end Normalized_Unit_Filename;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure)
   is
   begin
      Destroyable_Vectors.Append (Unit.Destroyables, (Object, Destroy));
   end Register_Destroyable_Helper;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable
     (Unit : Internal_Unit; Node : Bare_Ada_Node)
   is
      procedure Helper is new Register_Destroyable_Gen
        (Root_Node_Record,
         Bare_Ada_Node,
         Destroy_Synthetic_Node);
   begin
      Helper (Unit, Node);
   end Register_Destroyable;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable
     (Unit : Internal_Unit; Env : AST_Envs.Lexical_Env_Access)
   is
      procedure Helper is new Register_Destroyable_Gen
        (AST_Envs.Lexical_Env_Record, AST_Envs.Lexical_Env_Access, Destroy);
   begin
      Helper (Unit, Env);
   end Register_Destroyable;
   -----------------------
   -- Invalidate_Caches --
   -----------------------
   procedure Invalidate_Caches
     (Context : Internal_Context; Invalidate_Envs : Boolean) is
   begin
      --  Increase Context's version number. If we are about to overflow, reset
      --  all version numbers from analysis units.
      if Context.Cache_Version = Version_Number'Last then
         Context.Cache_Version := 1;
         for Unit of Context.Units loop
            Unit.Cache_Version := 0;
         end loop;
      else
         Context.Cache_Version := Context.Cache_Version + 1;
      end if;
      if Invalidate_Envs then
         Context.Reparse_Cache_Version := Context.Cache_Version;
      end if;
   end Invalidate_Caches;
   ------------------
   --  Reset_Envs  --
   ------------------
   procedure Reset_Envs (Unit : Internal_Unit) is
      procedure Deactivate_Refd_Envs (Node : Bare_Ada_Node);
      procedure Recompute_Refd_Envs (Node : Bare_Ada_Node);
      --------------------------
      -- Deactivate_Refd_Envs --
      --------------------------
      procedure Deactivate_Refd_Envs (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         Deactivate_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Deactivate_Refd_Envs (Child (Node, I));
         end loop;
      end Deactivate_Refd_Envs;
      -------------------------
      -- Recompute_Refd_Envs --
      -------------------------
      procedure Recompute_Refd_Envs (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         Recompute_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Recompute_Refd_Envs (Child (Node, I));
         end loop;
      end Recompute_Refd_Envs;
   begin
      --  First pass will deactivate every referenced envs that Unit possesses
      Deactivate_Refd_Envs (Unit.Ast_Root);
      --  Second pass will recompute the env they are pointing to
      Recompute_Refd_Envs (Unit.Ast_Root);
   end Reset_Envs;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Reparsed : in out Reparsed_Unit) is
   begin
      Free (Reparsed.TDH);
      Reparsed.Diagnostics := Diagnostics_Vectors.Empty_Vector;
      Free (Reparsed.Ast_Mem_Pool);
      Reparsed.Ast_Root := null;
   end Destroy;
   --------------
   -- Basename --
   --------------
   function Basename (Filename : String) return String is
   begin
      return +Create (+Filename).Base_Name;
   end Basename;
   --------------
   -- Basename --
   --------------
   function Basename (Unit : Internal_Unit) return String is
   begin
      return +Unit.Filename.Base_Name;
   end Basename;
   ------------------
   -- Reset_Caches --
   ------------------
   procedure Reset_Caches (Unit : Internal_Unit) is
      Cache_Version : constant Version_Number := Unit.Cache_Version;
   begin
      if Cache_Version < Unit.Context.Reparse_Cache_Version then
         Unit.Cache_Version := Unit.Context.Reparse_Cache_Version;
         Reset_Envs (Unit);
      end if;
      if Cache_Version < Unit.Context.Cache_Version then
         Unit.Cache_Version := Unit.Context.Cache_Version;
            Destroy (Unit.Memoization_Map);
      end if;
   end Reset_Caches;
   --------------------
   -- Reference_Unit --
   --------------------
   procedure Reference_Unit (From, Referenced : Internal_Unit) is
      Dummy : Boolean;
   begin
      Dummy := Analysis_Unit_Sets.Add (From.Referenced_Units, Referenced);
   end Reference_Unit;
   ------------------------
   -- Is_Referenced_From --
   ------------------------
   function Is_Referenced_From
     (Self, Unit : Internal_Unit) return Boolean is
   begin
      if Unit = null or else Self = null then
         return False;
      elsif Unit = Self then
         return True;
      else
         return Analysis_Unit_Sets.Has (Unit.Referenced_Units, Self);
      end if;
   end Is_Referenced_From;
   ----------------
   -- Do_Parsing --
   ----------------
   procedure Do_Parsing
     (Unit   : Internal_Unit;
      Input  : Internal_Lexer_Input;
      Result : out Reparsed_Unit)
   is
      Context  : constant Internal_Context := Unit.Context;
      Unit_TDH : constant Token_Data_Handler_Access := Token_Data (Unit);
      Saved_TDH : Token_Data_Handler;
      --  Holder to save tokens data in Unit.
      --
      --  By design, parsing is required to bind the nodes it creates to an
      --  analysis unit. However, this procedure is supposed to preserve the
      --  Unit itself and return its parsing result in Result.
      --
      --  In order to implement this, we first move "old" token data in this
      --  variable, then we do parsing. Only then, we can move "new" token data
      --  from the unit to Result, and restore the "old" token data to Unit.
      --  This last step is what Rotate_TDH (see below) is above.
      procedure Rotate_TDH;
      --  Move token data from Unit to Result and restore data in Saved_TDH to
      --  Unit.
      ----------------
      -- Rotate_TDH --
      ----------------
      procedure Rotate_TDH is
      begin
         Move (Result.TDH, Unit_TDH.all);
         Move (Unit_TDH.all, Saved_TDH);
      end Rotate_TDH;
   begin
      GNATCOLL.Traces.Trace (Main_Trace, "Parsing unit " & Basename (Unit));
      Result.Ast_Root := null;
      Move (Saved_TDH, Unit_TDH.all);
      Initialize (Unit_TDH.all,
                  Saved_TDH.Symbols,
                  Unit.all'Address,
                  Unit.Context.Tab_Stop);
      --  This is where lexing occurs, so this is where we get most "setup"
      --  issues: missing input file, bad charset, etc. If we have such an
      --  error, catch it, turn it into diagnostics and abort parsing.
      --
      --  As it is quite common, first check if the file is readable: if not,
      --  don't bother opening it and directly emit a diagnostic. This avoid
      --  pointless exceptions which harm debugging. Note that this
      --  optimization is valid only when there is no file reader, which can
      --  work even when there is no real source file.
      if Context.File_Reader = null
         and then Input.Kind = File
         and then (Input.Filename.Is_Directory
                   or else (not Input.Filename.Is_Readable))
      then
         declare
            Name : constant String := Basename (Unit);
         begin
            GNATCOLL.Traces.Trace
              (Main_Trace, "WARNING: File is not readable: " & Name);
            Append
              (Result.Diagnostics,
               No_Source_Location_Range,
               "Cannot read " & To_Text (Name));
            Rotate_TDH;
            return;
         end;
      end if;
      --  Initialize the parser, which fetches the source buffer and extract
      --  all tokens.
      Init_Parser
        (Input, Context.With_Trivia, Unit, Unit_TDH, Unit.Context.Parser);
      --  If we could run the lexer, run the parser and get the root node
      if Unit_TDH.Source_Buffer /= null then
         Result.Ast_Mem_Pool := Create;
         Unit.Context.Parser.Mem_Pool := Result.Ast_Mem_Pool;
         Result.Ast_Root := Bare_Ada_Node
           (Parse (Unit.Context.Parser, Rule => Unit.Rule));
      end if;
      --  Forward token data and diagnostics to the returned unit
      Rotate_TDH;
      Result.Diagnostics.Append_Vector (Unit.Context.Parser.Diagnostics);
   end Do_Parsing;
   --------------------------
   -- Update_After_Reparse --
   --------------------------
   procedure Update_After_Reparse
     (Unit : Internal_Unit; Reparsed : in out Reparsed_Unit) is
   begin
      --  Remove the `symbol -> AST node` associations for Unit's nodes in
      --  foreign lexical environments.
      Remove_Exiled_Entries (Unit);
      --  Remove the named envs that Unit created
      declare
         Named_Envs_Needing_Update : NED_Maps.Map;
      begin
         Remove_Named_Envs (Unit, Named_Envs_Needing_Update);
         Update_Named_Envs (Unit.Context, Named_Envs_Needing_Update);
      end;
      --  At this point, envs and nodes that don't belong to this unit no
      --  longer reference this unit's envs and nodes. It is thus now safe to
      --  deallocate this unit's obsolete data.
      --  Replace Unit's diagnostics by Reparsed's
      Unit.Diagnostics := Reparsed.Diagnostics;
      Reparsed.Diagnostics.Clear;
      --  As (re-)loading a unit can change how any AST node property in the
      --  whole analysis context behaves, we have to invalidate caches. This
      --  is likely overkill, but kill all caches here as it's easy to do.
      --
      --  As an optimization, invalidate referenced envs cache only if this is
      --  not the first time we parse Unit.
      Invalidate_Caches
        (Unit.Context, Invalidate_Envs => Unit.Ast_Root /= null);
      --  Likewise for token data
      Free (Unit.TDH);
      Move (Unit.TDH, Reparsed.TDH);
      --  Reparsing will invalidate all lexical environments related to this
      --  unit, so destroy all related rebindings as well. This browses AST
      --  nodes, so we have to do this before destroying the old AST nodes
      --  pool.
      Destroy_Rebindings (Unit.Rebindings'Access);
      --  Destroy the old AST node and replace it by the new one
      if Unit.Ast_Root /= null then
         Destroy (Unit.Ast_Root);
      end if;
      Unit.Ast_Root := Reparsed.Ast_Root;
      --  Likewise for memory pools
      Free (Unit.Ast_Mem_Pool);
      Unit.Ast_Mem_Pool := Reparsed.Ast_Mem_Pool;
      Reparsed.Ast_Mem_Pool := No_Pool;
      --  Increment unit version number to invalidate caches and stale node
      --  reference. Also propagate it to the TDH.
      Unit.Unit_Version := Unit.Unit_Version + 1;
      Unit.TDH.Version := Unit.Unit_Version;
      --  Compute the PLE_Roots_Starting_Token table
      Unit.PLE_Roots_Starting_Token.Clear;
         if Unit.Ast_Root /= null
            and then Unit.Ast_Root.Kind
                     = Ada_Compilation_Unit_List
         then
            for I in 1 .. Children_Count (Unit.Ast_Root) loop
               Unit.PLE_Roots_Starting_Token.Append
                 (Child (Unit.Ast_Root, I).Token_Start_Index);
            end loop;
         end if;
      --  Update all the lexical envs entries affected by the reparse
      declare
         Unit_Name     : constant String := +Unit.Filename.Base_Name;
         Context       : constant Internal_Context := Unit.Context;
         Foreign_Nodes : Bare_Ada_Node_Vectors.Vector :=
           Bare_Ada_Node_Vectors.Empty_Vector;
         Saved_In_Populate_Lexical_Env : constant Boolean :=
           Context.In_Populate_Lexical_Env;
         Saved_Env_Populated_Roots     : constant Boolean_Vectors.Vector :=
           Unit.Env_Populated_Roots;
      begin
         Context.In_Populate_Lexical_Env := True;
         if Main_Trace.Active then
            Main_Trace.Trace
              ("Updating lexical envs for " & Unit_Name & " after reparse");
            Main_Trace.Increase_Indent;
         end if;
         --  Collect all nodes that are foreign in this Unit's lexical envs.
         --  Exclude them from the corresponding lists of exiled entries.
         Extract_Foreign_Nodes (Unit, Foreign_Nodes);
         --  Temporarily reset Env_Populated_Roots so that Populate_Lexical_Env
         --  accepts to do its work on reparsed trees.
         Unit.Env_Populated_Roots := Boolean_Vectors.Empty_Vector;
         --  Now that Unit has been reparsed, we can destroy all its
         --  destroyables, which refer to the old tree (i.e. dangling
         --  pointers).
         Destroy_Unit_Destroyables (Unit);
         for FN of Foreign_Nodes loop
            declare
               Node_Image : constant String := Image (Short_Text_Image (FN));
               Unit_Name  : constant String := +FN.Unit.Filename.Base_Name;
            begin
               GNATCOLL.Traces.Trace
                 (Main_Trace, "Rerooting: " & Node_Image
                              & " (from " & Unit_Name & ")");
            end;
            Reroot_Foreign_Node (FN);
         end loop;
         Foreign_Nodes.Destroy;
         --  Re-populate all PLE roots that were requested so far for this
         --  unit. In the case where the unit has no PLE root, run PLE on the
         --  whole unit iff it was requested on at least one PLE root.
         declare
            function At_Least_One_Root_Populated return Boolean
            is (for some B of Saved_Env_Populated_Roots => B);
         begin
               if not Unit.PLE_Roots_Starting_Token.Is_Empty then
                  for I in 1 .. Children_Count (Unit.Ast_Root) loop
                     if Saved_Env_Populated_Roots.Last_Index >= I
                        and then Saved_Env_Populated_Roots.Get (I)
                     then
                        Populate_Lexical_Env (Unit, I);
                     end if;
                  end loop;
               elsif At_Least_One_Root_Populated then
                  Populate_Lexical_Env (Unit, 1);
               end if;
         end;
         --  Restore the unit's original Env_Populated_Roots flags
         Unit.Env_Populated_Roots.Destroy;
         Unit.Env_Populated_Roots := Saved_Env_Populated_Roots;
         Context.In_Populate_Lexical_Env := Saved_In_Populate_Lexical_Env;
         if Main_Trace.Is_Active then
            Main_Trace.Decrease_Indent;
         end if;
      end;
   end Update_After_Reparse;
   -------------------------------
   -- Destroy_Unit_Destroyables --
   -------------------------------
   procedure Destroy_Unit_Destroyables (Unit : Internal_Unit) is
   begin
      for D of Unit.Destroyables loop
         D.Destroy (D.Object);
      end loop;
      Destroyable_Vectors.Clear (Unit.Destroyables);
   end Destroy_Unit_Destroyables;
   ---------------------------
   -- Remove_Exiled_Entries --
   ---------------------------
   procedure Remove_Exiled_Entries (Unit : Internal_Unit) is
   begin
      for EE of Unit.Exiled_Entries loop
         AST_Envs.Remove (EE.Env, EE.Key, EE.Node);
         --  Also strip foreign nodes information from "outer" units so that it
         --  does not contain stale information (i.e. dangling pointers to
         --  nodes that belong to the units in the queue).
         if EE.Env.Owner /= No_Generic_Unit then
            declare
               Foreign_Nodes : Foreign_Node_Entry_Vectors.Vector renames
                  Convert_Unit (EE.Env.Owner).Foreign_Nodes;
               Current       : Positive := Foreign_Nodes.First_Index;
            begin
               while Current <= Foreign_Nodes.Last_Index loop
                  if Foreign_Nodes.Get (Current).Node = EE.Node then
                     Foreign_Nodes.Pop (Current);
                  else
                     Current := Current + 1;
                  end if;
               end loop;
            end;
         end if;
      end loop;
      Unit.Exiled_Entries.Clear;
   end Remove_Exiled_Entries;
   -----------------------
   -- Remove_Named_Envs --
   -----------------------
   procedure Remove_Named_Envs
     (Unit                      : Internal_Unit;
      Named_Envs_Needing_Update : in out NED_Maps.Map) is
   begin
      --  Remove nodes in this unit from the Named_Env_Descriptor.Foreign_Nodes
      --  components in which they are registered and from the foreign
      --  environments themselves.
      for EE of Unit.Exiled_Entries_In_NED loop
         Remove (EE.Named_Env.Foreign_Nodes, EE.Key, EE.Node);
         Remove (EE.Named_Env.Env_With_Precedence, EE.Key, EE.Node);
      end loop;
      Unit.Exiled_Entries_In_NED.Clear;
      --  Remove nodes in this unit from the
      --  Named_Env_Descriptor.Nodes_With_Foreign_Env components in which they
      --  are registered.
      for Cur in Unit.Nodes_With_Foreign_Env.Iterate loop
         declare
            use Node_To_Named_Env_Maps;
            Node : constant Bare_Ada_Node := Key (Cur);
            NE   : constant Named_Env_Descriptor_Access := Element (Cur);
         begin
            NE.Nodes_With_Foreign_Env.Delete (Node);
         end;
      end loop;
      Unit.Nodes_With_Foreign_Env.Clear;
      --  Remove ends in this unit from the Named_Env_Descriptor.Foreign_Envs
      --  components in which they are registered.
      for EE of Unit.Exiled_Envs loop
         EE.Named_Env.Foreign_Envs.Delete (Env_Node (EE.Env));
      end loop;
      Unit.Exiled_Envs.Clear;
      --  Remove named envs that this unit created
      for NE of Unit.Named_Envs loop
         declare
            NED_Access : constant Named_Env_Descriptor_Access :=
               Unit.Context.Named_Envs.Element (NE.Name);
            NED        : Named_Env_Descriptor renames NED_Access.all;
         begin
            NED.Envs.Delete (Env_Node (NE.Env));
            --  If this named environment had precedence, we must schedule an
            --  update for this name environment entry.
            if NE.Env = NED.Env_With_Precedence then
               Named_Envs_Needing_Update.Include (NE.Name, NED_Access);
               NED.Env_With_Precedence := Empty_Env;
            end if;
         end;
      end loop;
      Unit.Named_Envs.Clear;
   end Remove_Named_Envs;
   ---------------------------
   -- Extract_Foreign_Nodes --
   ---------------------------
   procedure Extract_Foreign_Nodes
     (Unit          : Internal_Unit;
      Foreign_Nodes : in out Bare_Ada_Node_Vectors.Vector) is
   begin
      --  Go through all foreign nodes registered in Unit's lexical
      --  environments.
      for FN of Unit.Foreign_Nodes loop
         --  Collect them
         Foreign_Nodes.Append (FN.Node);
         --  For each foreign node, remove the corresponding exiled entry in
         --  that foreign unit (each foreign node in unit A has a corresponding
         --  exiled entry in unit B).
         declare
            Exiled_Entries : Exiled_Entry_Vectors.Vector renames
               FN.Unit.Exiled_Entries;
            Current        : Positive := Exiled_Entries.First_Index;
         begin
            while Current <= Exiled_Entries.Last_Index loop
               if Exiled_Entries.Get (Current).Node = FN.Node then
                  Exiled_Entries.Pop (Current);
               else
                  Current := Current + 1;
               end if;
            end loop;
         end;
      end loop;
      Unit.Foreign_Nodes.Clear;
   end Extract_Foreign_Nodes;
   --------------------------
   -- Reroot_Foreign_Nodes --
   --------------------------
   procedure Reroot_Foreign_Node (Node : Bare_Ada_Node) is
      Unit : constant Internal_Unit := Node.Unit;
   begin
      --  First, filter the exiled entries in foreign units so that they don't
      --  contain references to this unit's lexical environments.  We need to
      --  do that before running the partial Populate_Lexical_Env pass so that
      --  we don't remove exiled entries that this pass will produce.
      declare
         Exiled_Entries : Exiled_Entry_Vectors.Vector renames
            Unit.Exiled_Entries;
         Current        : Positive := Exiled_Entries.First_Index;
      begin
         while Current <= Exiled_Entries.Last_Index loop
            if Exiled_Entries.Get (Current).Node = Node then
               Exiled_Entries.Pop (Current);
            else
               Current := Current + 1;
            end if;
         end loop;
      end;
      --  Re-do a partial Populate_Lexical_Env pass for each foreign node that
      --  this unit contains so that they are relocated in our new lexical
      --  environments.
      declare
         Unit_State : aliased PLE_Unit_State :=
           (Named_Envs_Needing_Update => <>);
         State      : PLE_Node_State :=
           (Unit_State  => Unit_State'Unchecked_Access,
            Current_Env => Node.Self_Env,
            Current_NED => null);
      begin
         Pre_Env_Actions (Node, State, Add_To_Env_Only => True);
         Post_Env_Actions (Node, State);
      end;
   end Reroot_Foreign_Node;
   ----------
   -- Text --
   ----------
   function Text (Node : Bare_Ada_Node) return String_Type is
   begin
      return Create_String (Text (Node));
   end Text;
   ------------------------
   -- Destroy_Rebindings --
   ------------------------
   procedure Destroy_Rebindings
     (Rebindings : access Env_Rebindings_Vectors.Vector)
   is
      procedure Recurse (R : in out Env_Rebindings);
      --  Destroy R's children and then destroy R. It is up to the caller to
      --  remove R from its parent's Children vector.
      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector);
      --  Remove R from Rebindings
      -------------
      -- Recurse --
      -------------
      procedure Recurse (R : in out Env_Rebindings) is
      begin
         for C of R.Children loop
            declare
               C_Var : Env_Rebindings := C;
            begin
               Recurse (C_Var);
            end;
         end loop;
         R.Children.Destroy;
         Unregister (R, Convert_Unit (R.Old_Env.Owner).Rebindings);
         Unregister (R, Convert_Unit (R.New_Env.Owner).Rebindings);
         Release_Rebinding (R);
      end Recurse;
      ----------------
      -- Unregister --
      ----------------
      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector) is
      begin
         for I in 1 .. Rebindings.Length loop
            if Rebindings.Get (I) = R then
               Rebindings.Pop (I);
               return;
            end if;
         end loop;
         --  We are always supposed to find R in Rebindings, so this should be
         --  unreachable.
         raise Program_Error;
      end Unregister;
   begin
      while Rebindings.Length > 0 loop
         declare
            R : Env_Rebindings := Rebindings.Get (1);
         begin
            --  Here, we basically undo what has been done in AST_Envs.Append
            --  If this rebinding has no parent, then during its creation we
            --  registered it in its Old_Env. Otherwise, it is registered
            --  in its Parent's Children list.
            if R.Parent = null then
               Unwrap (R.Old_Env).Rebindings_Pool.Delete (R.New_Env);
            else
               Unregister (R, R.Parent.Children);
            end if;
            --  In all cases it's registered in Old_Env's and New_Env's units
            Recurse (R);
         end;
      end loop;
   end Destroy_Rebindings;
   --------------------------
   -- Get_Rewriting_Handle --
   --------------------------
   function Get_Rewriting_Handle
     (Context : Internal_Context) return Rewriting_Handle_Pointer is
   begin
      return Context.Rewriting_Handle;
   end Get_Rewriting_Handle;
   --------------------------
   -- Set_Rewriting_Handle --
   --------------------------
   procedure Set_Rewriting_Handle
     (Context : Internal_Context; Handle : Rewriting_Handle_Pointer) is
   begin
      Context.Rewriting_Handle := Handle;
   end Set_Rewriting_Handle;
   -----------------------
   -- Create_Safety_Net --
   -----------------------
   function Create_Safety_Net
     (Context : Internal_Context) return Iterator_Safety_Net
   is
   begin
      return (Context         => Context,
              Context_Serial  => Context.Serial_Number,
              Context_Version => Context.Cache_Version);
   end Create_Safety_Net;
   ----------------------
   -- Check_Safety_Net --
   ----------------------
   procedure Check_Safety_Net (Self : Iterator_Safety_Net) is
   begin
      if Self.Context = null then
         return;
      end if;
      --  Check that the context is still the same (not released nor reused)
      if Self.Context.Serial_Number /= Self.Context_Serial
         or else Self.Context.Cache_Version /= Self.Context_Version
      then
         raise Stale_Reference_Error;
      end if;
   end Check_Safety_Net;
   ----------------------
   -- String_To_Symbol --
   ----------------------
   function String_To_Symbol
     (Self    : Bare_Ada_Node;
      Context : Internal_Context;
      S       : String_Type) return Symbol_Type is
   begin
      return (if S.Length > 0
              then Lookup_Symbol (Context, S.Content)
              else null);
   exception
      when Exc : Invalid_Symbol_Error =>
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            Ada.Exceptions.Exception_Message (Exc));
   end String_To_Symbol;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Self : String_Type) is
   begin
      if Self.Ref_Count >= 0 then
         Self.Ref_Count := Self.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Self : in out String_Type) is
   begin
      if Self = null or else Self.Ref_Count < 0 then
         return;
      end if;
      if Self.Ref_Count = 1 then
         Free (Self);
      else
         Self.Ref_Count := Self.Ref_Count - 1;
         Self := null;
      end if;
   end Dec_Ref;
   -------------------
   -- Create_String --
   -------------------
   function Create_String (Content : Text_Type) return String_Type is
   begin
      return Result : constant String_Type := new String_Record'
        (Length    => Content'Length,
         Ref_Count => 1,
         Content   => Content);
   end Create_String;
   -------------------
   -- Create_String --
   -------------------
   function Create_String (Content : Unbounded_Text_Type) return String_Type is
      S : Big_Wide_Wide_String_Access;
      L : Natural;
   begin
      Get_Wide_Wide_String (Content, S, L);
      return Create_String (S.all (1 .. L));
   end Create_String;
   -------------------
   -- Concat_String --
   -------------------
   function Concat_String (Left, Right : String_Type) return String_Type is
   begin
      return Result : constant String_Type :=
        new String_Record (Length => Left.Length + Right.Length)
      do
         Result.Ref_Count := 1;
         Result.Content (1 .. Left.Length) := Left.Content;
         Result.Content (Left.Length + 1 .. Result.Length) := Right.Content;
      end return;
   end Concat_String;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (Left, Right : String_Type) return Boolean is
   begin
      return Left.Content = Right.Content;
   end Equivalent;
begin
   No_Big_Integer.Value.Set (0);
end Libadalang.Implementation;